(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     10684,        261]*)
(*NotebookOutlinePosition[     14918,        376]*)
(*  CellTagsIndexPosition[     14430,        360]*)
(*WindowFrame->Normal*)



Notebook[{
Cell["QPrincipalQuantumNumber", "Section",
  CellTags->"QPrincipalQuantumNumber"],

Cell["\<\
QPrincipalQuantumNumber[nrad,ell] gives the principal quantum number in terms of the radial quantum number and the angular momentum. Package: VQM`Coulomb`.\
\>", "Usage",
  CellTags->"QPrincipalQuantumNumber"],

Cell[TextData[{
  "This command is contained in the package ",
  ButtonBox["VQM`Coulomb`",
    ButtonStyle->"AddOnsLink"]
}], "Text",
  CellTags->"QPrincipalQuantumNumber"],

Cell["QRadialQuantumNumber", "Section",
  CellTags->"QRadialQuantumNumber"],

Cell["\<\
QRadialQuantumNumber[n,ell] is the radial quantum number n - ell - 1. It counts the number of radial zeros of the wave function. Package: VQM`Coulomb`.\
\>", "Usage",
  CellTags->"QRadialQuantumNumber"],

Cell[TextData[{
  "This command is contained in the package ",
  ButtonBox["VQM`Coulomb`",
    ButtonStyle->"AddOnsLink"]
}], "Text",
  CellTags->"QRadialQuantumNumber"],

Cell["QCoulombEnergy", "Section",
  CellTags->"QCoulombEnergy"],

Cell["\<\
QCoulombEnergy[n, options] gives the energy of a particle in the Coulomb field g/r (here n=nrad+ell+1 is the principal quantum number). QCoulombEnergy is an eigenfunction of the differential operator -1/2 Delta + g/r. The Coulomb coupling constant g can be specified by giving the option QCoulombCoupling->g. Default is g=$QCoulombCoupling. The constant $QCoulombCoupling is initially set to 1, but can be redefined by the user. The energy depends on the space dimension. Default space dimension is 3; you can change this by the option QCoulombSpaceDimension->dim, or by redefining the constant $QCoulombSpaceDimension. Package: VQM`Coulomb`.\
\>", "Usage",
  CellTags->"QCoulombEnergy"],

Cell[TextData[{
  "This command is contained in the package ",
  ButtonBox["VQM`Coulomb`",
    ButtonStyle->"AddOnsLink"]
}], "Text",
  CellTags->"QCoulombEnergy"],

Cell["QCoulombTimePeriod", "Section",
  CellTags->"QCoulombTimePeriod"],

Cell["\<\
QCoulombTimePeriod[n1,n2,..] gives the time period of a superposition of Coulomb eigenstates with principal quantum numbers n1,n2, etc. Package: VQM`Coulomb`.\
\>", "Usage",
  CellTags->"QCoulombTimePeriod"],

Cell[TextData[{
  "This command is contained in the package ",
  ButtonBox["VQM`Coulomb`",
    ButtonStyle->"AddOnsLink"]
}], "Text",
  CellTags->"QCoulombTimePeriod"],

Cell["QCoulombFunction", "Section",
  CellTags->"QCoulombFunction"],

Cell["\<\
QCoulombFunction[n,ell,m,{x,y,z},options] gives the energy eigenfunction of a particle in the three-dimensional attractive Coulomb field -g/r. n is the principal quantum number, ell orbital angular momentum, m is the magnetic quantum number. The function is an eigenfunction of (-1/2 Delta - g/r) Psi = QCoulombEnergy Psi. The Coulomb coupling constant g can be specified by giving the option QCoulombCoupling->g. Default is g=$QCoulombCoupling (attractive for positive values of g). The constant $QCoulombCoupling is initially set to 1, but can be redefined by the user. The energy depends on the space dimension. Default space dimension is 3; you can change this by the option QCoulombSpaceDimension->dim, or by redefining the constant $QCoulombSpaceDimension. The default coordinate system is Cartesian, as set by the package Calculus`VectorAnalysis`. The coordinatesystem can be changed by the command SetCoordinates[Spherical]. In this case, the function has to be used in the form QCoulombFunction[n,ell,m,{r,theta,phi}]. QCoulombFunction[n,m,{x,y},options] resp. QCoulombFunction[n,m,{r,phi},options] give the eigenfunction for the two-dimensional Coulomb problem in Cartesian resp. Spherical coordinates. Package: VQM`Coulomb`.\
\>", "Usage",
  CellTags->"QCoulombFunction"],

Cell[TextData[{
  "This command is contained in the package ",
  ButtonBox["VQM`Coulomb`",
    ButtonStyle->"AddOnsLink"]
}], "Text",
  CellTags->"QCoulombFunction"],

Cell["QEffectiveCoulombPotential", "Section",
  CellTags->"QEffectiveCoulombPotential"],

Cell["\<\
QEffectiveCoulombPotential[ell, r] is the effective potential for the radial Coulomb equation. Consists of the Coulomb potential and the repulsive angular momentum barrier, which depends on the space dimension. Package: VQM`Coulomb`.\
\>", "Usage",
  CellTags->"QEffectiveCoulombPotential"],

Cell[TextData[{
  "This command is contained in the package ",
  ButtonBox["VQM`Coulomb`",
    ButtonStyle->"AddOnsLink"]
}], "Text",
  CellTags->"QEffectiveCoulombPotential"],

Cell["QCoulombHamiltonian", "Section",
  CellTags->"QCoulombHamiltonian"],

Cell["\<\
QCoulombHamiltonian[psi[x,y,z],{x,y,z}] or QCoulombHamiltonian[psi[x,y],{x,y}] evaluates the action of the Hamiltonian operator -1/2 Delta - g/r on the wave function psi. Package: VQM`Coulomb`.\
\>", "Usage",
  CellTags->"QCoulombHamiltonian"],

Cell[TextData[{
  "This command is contained in the package ",
  ButtonBox["VQM`Coulomb`",
    ButtonStyle->"AddOnsLink"]
}], "Text",
  CellTags->"QCoulombHamiltonian"],

Cell["QRadialCoulombFunction", "Section",
  CellTags->"QRadialCoulombFunction"],

Cell["\<\
QRadialCoulombFunction[n,ell,r] is the radial part of the eigenfunction of the Schroedinger equation in the angular momentum subspace described by ell. Here n is the principal quantum number. n - ell - 1 (=the radial quantum number) is the number of zeros of the radial Coulomb eigenfunction. Package: VQM`Coulomb`.\
\>", "Usage",
  CellTags->"QRadialCoulombFunction"],

Cell[TextData[{
  "This command is contained in the package ",
  ButtonBox["VQM`Coulomb`",
    ButtonStyle->"AddOnsLink"]
}], "Text",
  CellTags->"QRadialCoulombFunction"],

Cell["QRadialPositionAmplitude", "Section",
  CellTags->"QRadialPositionAmplitude"],

Cell["\<\
QRadialPositionAmplitude[n,ell,r] describes the amplitude of the radial position density. The square of the radial position amplitude gives the probability density for having a position at the distance r from the coordinate origin. Here n is the principal quantum number, ell is the orbital angular momentum quantum number. Package: VQM`Coulomb`.\
\>", "Usage",
  CellTags->"QRadialPositionAmplitude"],

Cell[TextData[{
  "This command is contained in the package ",
  ButtonBox["VQM`Coulomb`",
    ButtonStyle->"AddOnsLink"]
}], "Text",
  CellTags->"QRadialPositionAmplitude"],

Cell["$QCoulombSpaceDimension", "Section",
  CellTags->"$QCoulombSpaceDimension"],

Cell["\<\
Constant describing the default value of the space dimension in the package VQM`Coulomb`. Initially set to 3. Package: VQM`Coulomb`.\
\>", "Usage",
  CellTags->"$QCoulombSpaceDimension"],

Cell[BoxData[
    FormBox[
      StyleBox[
        FrameBox[
          StyleBox[GridBox[{
                {
                  FormBox["QCoulombSpaceDimension",
                    
                    "InputForm"], \(QCoulombSpaceDimension -> 
                      dim\ is\ an\ option\ used\ in\ the\ package\ \
VQM`Coulomb` . \ 
                          The\ formulas\ for\ eigenfunctions\ and\ \
energies\ depend\ on\ the\ space\ dimension . \ 
                          Default\ value\ is\ $QCoulombSpaceDimension\
\  = \ 3, \ 
                  but\ this\ constant\ can\ be\ redefined\ by\ the\ \
user . \ \(Package : \ \(\(VQM`Coulomb`\)\(.\)\)\)\)}
                },
              ColumnAlignments->{Right, Left}],
            GridBoxOptions->{RowLines->False}]],
        "2ColumnBox"], TraditionalForm]], "Text",
  CellTags->"$QCoulombSpaceDimension"],

Cell[TextData[{
  "This command is contained in the package ",
  ButtonBox["VQM`Coulomb`",
    ButtonStyle->"AddOnsLink"]
}], "Text",
  CellTags->"$QCoulombSpaceDimension"],

Cell["$QCoulombCoupling", "Section",
  CellTags->"$QCoulombCoupling"],

Cell["\<\
Constant describing the default value of the strength of the Coulomb potential g/r for the package VQM`Coulomb`. Initial value is g=1, which describes an attractive Coulomb potential. Package: VQM`Coulomb`.\
\>", "Usage",
  CellTags->"$QCoulombCoupling"],

Cell[BoxData[
    FormBox[
      StyleBox[
        FrameBox[
          StyleBox[GridBox[{
                {
                  FormBox["QCoulombCoupling",
                    
                    "InputForm"], \(QCoulombCoupling -> 
                      g\ is\ an\ option\ used\ in\ the\ package\ \
VQM`Coulomb` . \ 
                            It\ describes\ the\ strength\ of\ the\ \
Coulomb\ potential\  - g/r\ in\ the\ Schroedinger\ equation . \ 
                            Default\ value\ is\ $QCoulombCoupling\  = \
\ 1, \ but\ this\ constant\ can\ be\ redefined\ by\ the\ user . \ \
\(Package : \ \(\(VQM`Coulomb`\)\(.\)\)\)\)}
                },
              ColumnAlignments->{Right, Left}],
            GridBoxOptions->{RowLines->False}]],
        "2ColumnBox"], TraditionalForm]], "Text",
  CellTags->"$QCoulombCoupling"],

Cell[TextData[{
  "This command is contained in the package ",
  ButtonBox["VQM`Coulomb`",
    ButtonStyle->"AddOnsLink"]
}], "Text",
  CellTags->"$QCoulombCoupling"]
},
FrontEndVersion->"5.0 for Macintosh",
ScreenRectangle->{{0, 1280}, {0, 961}},
StyleDefinitions -> "HelpBrowser.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "QPrincipalQuantumNumber"->{
    Cell[1754, 51, 81, 1, 70, "Section",
      CellTags->"QPrincipalQuantumNumber"],
    Cell[1838, 54, 219, 3, 70, "Usage",
      CellTags->"QPrincipalQuantumNumber"],
    Cell[2060, 59, 172, 5, 70, "Text",
      CellTags->"QPrincipalQuantumNumber"]},
  "QRadialQuantumNumber"->{
    Cell[2235, 66, 75, 1, 70, "Section",
      CellTags->"QRadialQuantumNumber"],
    Cell[2313, 69, 212, 3, 70, "Usage",
      CellTags->"QRadialQuantumNumber"],
    Cell[2528, 74, 169, 5, 70, "Text",
      CellTags->"QRadialQuantumNumber"]},
  "QCoulombEnergy"->{
    Cell[2700, 81, 63, 1, 70, "Section",
      CellTags->"QCoulombEnergy"],
    Cell[2766, 84, 697, 3, 70, "Usage",
      CellTags->"QCoulombEnergy"],
    Cell[3466, 89, 163, 5, 70, "Text",
      CellTags->"QCoulombEnergy"]},
  "QCoulombTimePeriod"->{
    Cell[3632, 96, 71, 1, 70, "Section",
      CellTags->"QCoulombTimePeriod"],
    Cell[3706, 99, 217, 3, 70, "Usage",
      CellTags->"QCoulombTimePeriod"],
    Cell[3926, 104, 167, 5, 70, "Text",
      CellTags->"QCoulombTimePeriod"]},
  "QCoulombFunction"->{
    Cell[4096, 111, 67, 1, 70, "Section",
      CellTags->"QCoulombFunction"],
    Cell[4166, 114, 1292, 3, 70, "Usage",
      CellTags->"QCoulombFunction"],
    Cell[5461, 119, 165, 5, 70, "Text",
      CellTags->"QCoulombFunction"]},
  "QEffectiveCoulombPotential"->{
    Cell[5629, 126, 87, 1, 70, "Section",
      CellTags->"QEffectiveCoulombPotential"],
    Cell[5719, 129, 300, 3, 70, "Usage",
      CellTags->"QEffectiveCoulombPotential"],
    Cell[6022, 134, 175, 5, 70, "Text",
      CellTags->"QEffectiveCoulombPotential"]},
  "QCoulombHamiltonian"->{
    Cell[6200, 141, 73, 1, 70, "Section",
      CellTags->"QCoulombHamiltonian"],
    Cell[6276, 144, 253, 3, 70, "Usage",
      CellTags->"QCoulombHamiltonian"],
    Cell[6532, 149, 168, 5, 70, "Text",
      CellTags->"QCoulombHamiltonian"]},
  "QRadialCoulombFunction"->{
    Cell[6703, 156, 79, 1, 70, "Section",
      CellTags->"QRadialCoulombFunction"],
    Cell[6785, 159, 378, 3, 70, "Usage",
      CellTags->"QRadialCoulombFunction"],
    Cell[7166, 164, 171, 5, 70, "Text",
      CellTags->"QRadialCoulombFunction"]},
  "QRadialPositionAmplitude"->{
    Cell[7340, 171, 83, 1, 70, "Section",
      CellTags->"QRadialPositionAmplitude"],
    Cell[7426, 174, 411, 3, 70, "Usage",
      CellTags->"QRadialPositionAmplitude"],
    Cell[7840, 179, 173, 5, 70, "Text",
      CellTags->"QRadialPositionAmplitude"]},
  "$QCoulombSpaceDimension"->{
    Cell[8016, 186, 81, 1, 70, "Section",
      CellTags->"$QCoulombSpaceDimension"],
    Cell[8100, 189, 196, 3, 70, "Usage",
      CellTags->"$QCoulombSpaceDimension"],
    Cell[8299, 194, 861, 21, 70, "Text",
      CellTags->"$QCoulombSpaceDimension"],
    Cell[9163, 217, 172, 5, 70, "Text",
      CellTags->"$QCoulombSpaceDimension"]},
  "$QCoulombCoupling"->{
    Cell[9338, 224, 69, 1, 70, "Section",
      CellTags->"$QCoulombCoupling"],
    Cell[9410, 227, 264, 3, 70, "Usage",
      CellTags->"$QCoulombCoupling"],
    Cell[9677, 232, 834, 20, 70, "Text",
      CellTags->"$QCoulombCoupling"],
    Cell[10514, 254, 166, 5, 70, "Text",
      CellTags->"$QCoulombCoupling"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"QPrincipalQuantumNumber", 11248, 275},
  {"QRadialQuantumNumber", 11529, 282},
  {"QCoulombEnergy", 11795, 289},
  {"QCoulombTimePeriod", 12047, 296},
  {"QCoulombFunction", 12310, 303},
  {"QEffectiveCoulombPotential", 12580, 310},
  {"QCoulombHamiltonian", 12872, 317},
  {"QRadialCoulombFunction", 13146, 324},
  {"QRadialPositionAmplitude", 13431, 331},
  {"$QCoulombSpaceDimension", 13721, 338},
  {"$QCoulombCoupling", 14087, 347}
  }
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 81, 1, 70, "Section",
  CellTags->"QPrincipalQuantumNumber"],
Cell[1838, 54, 219, 3, 70, "Usage",
  CellTags->"QPrincipalQuantumNumber"],
Cell[2060, 59, 172, 5, 70, "Text",
  CellTags->"QPrincipalQuantumNumber"],
Cell[2235, 66, 75, 1, 70, "Section",
  CellTags->"QRadialQuantumNumber"],
Cell[2313, 69, 212, 3, 70, "Usage",
  CellTags->"QRadialQuantumNumber"],
Cell[2528, 74, 169, 5, 70, "Text",
  CellTags->"QRadialQuantumNumber"],
Cell[2700, 81, 63, 1, 70, "Section",
  CellTags->"QCoulombEnergy"],
Cell[2766, 84, 697, 3, 70, "Usage",
  CellTags->"QCoulombEnergy"],
Cell[3466, 89, 163, 5, 70, "Text",
  CellTags->"QCoulombEnergy"],
Cell[3632, 96, 71, 1, 70, "Section",
  CellTags->"QCoulombTimePeriod"],
Cell[3706, 99, 217, 3, 70, "Usage",
  CellTags->"QCoulombTimePeriod"],
Cell[3926, 104, 167, 5, 70, "Text",
  CellTags->"QCoulombTimePeriod"],
Cell[4096, 111, 67, 1, 70, "Section",
  CellTags->"QCoulombFunction"],
Cell[4166, 114, 1292, 3, 70, "Usage",
  CellTags->"QCoulombFunction"],
Cell[5461, 119, 165, 5, 70, "Text",
  CellTags->"QCoulombFunction"],
Cell[5629, 126, 87, 1, 70, "Section",
  CellTags->"QEffectiveCoulombPotential"],
Cell[5719, 129, 300, 3, 70, "Usage",
  CellTags->"QEffectiveCoulombPotential"],
Cell[6022, 134, 175, 5, 70, "Text",
  CellTags->"QEffectiveCoulombPotential"],
Cell[6200, 141, 73, 1, 70, "Section",
  CellTags->"QCoulombHamiltonian"],
Cell[6276, 144, 253, 3, 70, "Usage",
  CellTags->"QCoulombHamiltonian"],
Cell[6532, 149, 168, 5, 70, "Text",
  CellTags->"QCoulombHamiltonian"],
Cell[6703, 156, 79, 1, 70, "Section",
  CellTags->"QRadialCoulombFunction"],
Cell[6785, 159, 378, 3, 70, "Usage",
  CellTags->"QRadialCoulombFunction"],
Cell[7166, 164, 171, 5, 70, "Text",
  CellTags->"QRadialCoulombFunction"],
Cell[7340, 171, 83, 1, 70, "Section",
  CellTags->"QRadialPositionAmplitude"],
Cell[7426, 174, 411, 3, 70, "Usage",
  CellTags->"QRadialPositionAmplitude"],
Cell[7840, 179, 173, 5, 70, "Text",
  CellTags->"QRadialPositionAmplitude"],
Cell[8016, 186, 81, 1, 70, "Section",
  CellTags->"$QCoulombSpaceDimension"],
Cell[8100, 189, 196, 3, 70, "Usage",
  CellTags->"$QCoulombSpaceDimension"],
Cell[8299, 194, 861, 21, 70, "Text",
  CellTags->"$QCoulombSpaceDimension"],
Cell[9163, 217, 172, 5, 70, "Text",
  CellTags->"$QCoulombSpaceDimension"],
Cell[9338, 224, 69, 1, 70, "Section",
  CellTags->"$QCoulombCoupling"],
Cell[9410, 227, 264, 3, 70, "Usage",
  CellTags->"$QCoulombCoupling"],
Cell[9677, 232, 834, 20, 70, "Text",
  CellTags->"$QCoulombCoupling"],
Cell[10514, 254, 166, 5, 70, "Text",
  CellTags->"$QCoulombCoupling"]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)
