(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    255243,       4620]
NotebookOptionsPosition[    254073,       4577]
NotebookOutlinePosition[    254684,       4602]
CellTagsIndexPosition[    254607,       4597]
WindowFrame->Normal
ContainsDynamic->False*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Spinors ", "Subsection",
 CellTags->"VQM`Spinors`"],

Cell["\<\
VQM`Spinors` is a package for 'Visual Quantum Mechanics'. It defines basic \
operations with spinors and provides tools for the visualization of spinors. \
\
\>", "Text",
 CellTags->"VQM`Spinors`"],

Cell[BoxData[
 FormBox[
  StyleBox[
   FrameBox[GridBox[{
      {
       StyleBox[
        RowBox[{"QNorm", "[", "spinor", "]"}],
        FontFamily->"Courier"], 
       StyleBox[
        RowBox[{" ", 
         RowBox[{
         "determines", " ", "the", " ", "norm", " ", "of", " ", "a", " ", 
          RowBox[{"(", "complex", ")"}], " ", 
          RowBox[{"vector", ".", " ", 
           RowBox[{"Package", ":", " ", 
            RowBox[{"VQM`Spinors`", "."}]}]}]}]}],
        FontFamily->"Times"]},
      {
       StyleBox[
        RowBox[{"QNormalize", "[", "spinor", "]"}],
        FontFamily->"Courier"], 
       StyleBox[
        RowBox[{" ", 
         RowBox[{"returns", " ", "a", " ", 
          RowBox[{"(", "complex", ")"}], " ", "vector", " ", "with", " ", 
          "norm", " ", "1.", " ", 
          RowBox[{"Package", ":", " ", 
           RowBox[{"VQM`Spinors`", "."}]}]}]}],
        FontFamily->"Times"]},
      {
       StyleBox["Gives",
        FontFamily->"Courier"], 
       StyleBox[
        RowBox[{" ", 
         RowBox[{
          RowBox[{
          "the", " ", "standard", " ", "reference", " ", "basis", " ", "in", 
           " ", "the", " ", "two"}], "-", 
          RowBox[{"dimensional", " ", "complex", " ", "linear", " ", 
           RowBox[{"space", ".", " ", 
            RowBox[{"Package", ":", " ", 
             RowBox[{"VQM`Spinors`", "."}]}]}]}]}]}],
        FontFamily->"Times"]},
      {
       StyleBox[
        RowBox[{"QSetSpinBasis", "[", 
         RowBox[{"{", 
          RowBox[{"vec1", ",", "vec2"}], "}"}], "]"}],
        FontFamily->"Courier"], 
       StyleBox[
        RowBox[{" ", 
         RowBox[{
          RowBox[{
          "defines", " ", "QSpinBasis", " ", "to", " ", "consist", " ", "of", 
           " ", "two", " ", "complex", " ", "unit", " ", "vectors", " ", "in",
            " ", "the", " ", "direction", " ", "of", " ", "vec1", " ", "and", 
           " ", 
           RowBox[{"vec2", ".", " ", "The"}], " ", "complex", " ", "2"}], "-",
           
          RowBox[{
          "vectors", " ", "vec1", " ", "and", " ", "vec2", " ", "must", " ", 
           "be", " ", 
           RowBox[{"orthogonal", ".", " ", 
            RowBox[{"Package", ":", " ", 
             RowBox[{"VQM`Spinors`", "."}]}]}]}]}]}],
        FontFamily->"Times"]},
      {
       StyleBox[
        RowBox[{"QSpinorToComponents", "[", "spinor", "]"}],
        FontFamily->"Courier"], 
       StyleBox[
        RowBox[{" ", 
         RowBox[{
         "returns", " ", "the", " ", "components", " ", "of", " ", "the", " ",
           "complex", " ", 
          RowBox[{"vector", " ", "'"}], 
          RowBox[{"spinor", "'"}], " ", 
          RowBox[{"(", 
           RowBox[{
           "which", " ", "is", " ", "given", " ", "with", " ", "respect", " ",
             "to", " ", "the", " ", "reference", " ", "basis", " ", 
            "$QSpinBasis"}], ")"}], " ", "with", " ", "respect", " ", "to", 
          " ", "the", " ", "basis", " ", "QSpinBasis", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
             "or", " ", "the", " ", "basis", " ", "specified", " ", "by", " ",
               "the", " ", "QUseBasis"}], "-", 
             RowBox[{"option", ".", " ", "Hence", " ", ".", "QSpinBasis"}]}], 
            " ", "==", " ", 
            RowBox[{"spinor", ".", " ", 
             RowBox[{"Package", ":", " ", 
              RowBox[{"VQM`Spinors`", "."}]}]}]}]}]}]}],
        FontFamily->"Times"]},
      {
       StyleBox[
        RowBox[{"QComponentsToSpinor", "[", 
         RowBox[{"{", 
          RowBox[{"c1", ",", "c2"}], "}"}], "]"}],
        FontFamily->"Courier"], 
       StyleBox[
        RowBox[{" ", 
         RowBox[{
          RowBox[{
          "gives", " ", "the", " ", "spinor", " ", "in", " ", "$QSpinBasis", 
           " ", "that", " ", "has", " ", "the", " ", "components", " ", "c1", 
           " ", "and", " ", "c2", " ", "with", " ", "respect", " ", "to", " ",
            "QSpinBasis", " ", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{
              "or", " ", "the", " ", "basis", " ", "specified", " ", "by", 
               " ", "the", " ", "QUseBasis"}], "-", "option"}], ")"}], 
            "."}]}], "  ", "==", " ", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"c1", ",", "c2"}], "}"}], ".", "QSpinBasis", ".", " ", 
           RowBox[{"Package", ":", " ", 
            RowBox[{"VQM`Spinors`", "."}]}]}]}]}],
        FontFamily->"Times"]},
      {
       StyleBox[
        RowBox[{"QProjectUp", "[", "spinor", "]"}],
        FontFamily->"Courier"], 
       StyleBox[
        RowBox[{" ", 
         RowBox[{
         "projects", " ", "a", " ", "spinor", " ", "into", " ", "the", " ", 
          "direction", " ", "of", " ", "the", " ", "first", " ", "basis", " ",
           "vector", " ", "of", " ", "QSpinBasis", " ", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"i", ".", "e", "."}], ",", " ", 
             RowBox[{
              RowBox[{"into", " ", "the", " ", 
               RowBox[{"current", " ", "'"}], "up"}], "-", 
              RowBox[{"direction", "'"}]}]}], ")"}], ".", " ", 
           RowBox[{"Package", ":", " ", 
            RowBox[{"VQM`Spinors`", "."}]}]}]}]}],
        FontFamily->"Times"]},
      {
       StyleBox[
        RowBox[{"QProjectDown", "[", "spinor", "]"}],
        FontFamily->"Courier"], 
       StyleBox[
        RowBox[{" ", 
         RowBox[{
         "projects", " ", "a", " ", "spinor", " ", "into", " ", "the", " ", 
          "direction", " ", "of", " ", "the", " ", "second", " ", "basis", 
          " ", "vector", " ", "of", " ", "QSpinBasis", " ", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"i", ".", "e", "."}], ",", " ", 
             RowBox[{
              RowBox[{"into", " ", "the", " ", 
               RowBox[{"current", " ", "'"}], "down"}], "-", 
              RowBox[{"direction", "'"}]}]}], ")"}], ".", " ", 
           RowBox[{"Package", ":", " ", 
            RowBox[{"VQM`Spinors`", "."}]}]}]}]}],
        FontFamily->"Times"]},
      {
       StyleBox[
        RowBox[{"QProbabilityUp", "[", "spinor", "]"}],
        FontFamily->"Courier"], 
       StyleBox[
        RowBox[{" ", 
         RowBox[{
          RowBox[{
          "gives", " ", "the", " ", "probability", " ", "that", " ", "the", 
           " ", "spinor", " ", "has", " ", "spin"}], "-", 
          RowBox[{
          "up", " ", "with", " ", "respect", " ", "to", " ", "the", " ", 
           RowBox[{"QSpinBasis", ".", " ", 
            RowBox[{"Package", ":", " ", 
             RowBox[{"VQM`Spinors`", "."}]}]}]}]}]}],
        FontFamily->"Times"]},
      {
       StyleBox[
        RowBox[{"QSpinorBarDiagram", "[", 
         RowBox[{"spinor", ",", "opts"}], "]"}],
        FontFamily->"Courier"], 
       StyleBox[
        RowBox[{" ", 
         RowBox[{
          RowBox[{"gives", " ", "a", " ", "bar"}], "-", 
          RowBox[{
          "diagram", " ", "showing", " ", "the", " ", "components", " ", "of",
            " ", "the", " ", "spinor", " ", "with", " ", "respect", " ", "to",
            " ", "the", " ", "current", " ", 
           RowBox[{"QSpinBasis", ".", " ", "The"}], " ", "options", " ", 
           "are", " ", 
           RowBox[{
            RowBox[{"Options", "[", "Graphics", "]"}], ".", " ", 
            RowBox[{"Package", ":", " ", 
             RowBox[{"VQM`Spinors`", "."}]}]}]}]}]}],
        FontFamily->"Times"]},
      {
       StyleBox[
        RowBox[{"QVectorToHermitianMatrix", "[", 
         RowBox[{"{", 
          RowBox[{"k0", ",", "k1", ",", "k2", ",", "k3"}], "}"}], "]"}],
        FontFamily->"Courier"], 
       StyleBox[
        RowBox[{" ", 
         RowBox[{"=", " ", 
          RowBox[{
           RowBox[{"QIdentity2", " ", "*", " ", "k0"}], " ", "+", " ", 
           RowBox[{
            RowBox[{
             RowBox[{"Sum", "[", 
              RowBox[{
               RowBox[{"QPauliSigma_i", " ", "*", " ", "k_i"}], ",", " ", 
               RowBox[{"{", 
                RowBox[{"i", ",", "1", ",", "3"}], "}"}]}], "]"}], ".", " ", 
             "Converts"}], " ", "a", " ", "four"}], "-", 
           RowBox[{
           "dimensional", " ", "real", " ", "vector", " ", "into", " ", "a", 
            " ", "Hermitian", " ", "2", "x2", " ", 
            RowBox[{"matrix", ".", " ", 
             RowBox[{"Package", ":", " ", 
              RowBox[{"VQM`Spinors`", "."}]}]}]}]}]}]}],
        FontFamily->"Times"]},
      {
       StyleBox[
        RowBox[{"QVectorToDensityMatrix", "[", 
         RowBox[{"{", 
          RowBox[{"k1", ",", "k2", ",", "k3"}], "}"}], "]"}],
        FontFamily->"Courier"], 
       StyleBox[
        RowBox[{" ", 
         RowBox[{"=", " ", 
          RowBox[{
           RowBox[{
            RowBox[{"1", "/", "2"}], " ", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"QIdentity2", "+", 
               RowBox[{"Sum", "[", 
                RowBox[{
                 RowBox[{"QPauliSigma_i", " ", "*", " ", "k_i"}], ",", " ", 
                 RowBox[{"{", 
                  RowBox[{"i", ",", "1", ",", "3"}], "}"}]}], "]"}]}], ")"}], 
             ".", " ", "Converts"}], " ", "a", " ", "three"}], "-", 
           RowBox[{
           "dimensional", " ", "real", " ", "vector", " ", "into", " ", "a", 
            " ", "Hermitian", " ", "2", "x2", " ", "matrix", " ", "with", " ",
             "trace", " ", "1", " ", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"a", " ", "density", " ", "matrix"}], ")"}], ".", " ", 
             RowBox[{"Package", ":", " ", 
              RowBox[{"VQM`Spinors`", "."}]}]}]}]}]}]}],
        FontFamily->"Times"]},
      {
       StyleBox[
        RowBox[{"QSpinHamiltonian", "[", 
         RowBox[{"{", 
          RowBox[{"k0", ",", "k1", ",", "k2", ",", "k3"}], "}"}], "]"}],
        FontFamily->"Courier"], 
       StyleBox[
        RowBox[{" ", 
         RowBox[{"=", " ", 
          RowBox[{
           RowBox[{"QIdentity2", " ", "*", " ", "k0"}], " ", "+", " ", 
           RowBox[{
            RowBox[{"Sum", "[", 
             RowBox[{
              RowBox[{"QPauliSigma_i", " ", "*", " ", "k_i"}], ",", " ", 
              RowBox[{"{", 
               RowBox[{"i", ",", "1", ",", "3"}], "}"}]}], "]"}], ".", " ", 
            RowBox[{"Package", ":", " ", 
             RowBox[{"VQM`Spinors`", "."}]}]}]}]}]}],
        FontFamily->"Times"]},
      {
       StyleBox[
        RowBox[{"QSpinorUp", "[", 
         RowBox[{"{", 
          RowBox[{"k1", ",", "k2", ",", "k3"}], "}"}], "]"}],
        FontFamily->"Courier"], 
       StyleBox[
        RowBox[{" ", 
         RowBox[{
          RowBox[{
           RowBox[{"is", " ", "the", " ", "two"}], "-", 
           RowBox[{"dimensional", " ", "complex", " ", "vector", " ", 
            RowBox[{"describing", " ", "'"}], "spin", " ", 
            RowBox[{"up", "'"}], " ", "in", " ", "the", " ", "direction", " ",
             "defined", " ", "by", " ", "k"}]}], " ", "=", " ", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"{", 
               RowBox[{"k1", ",", "k2", ",", "k3"}], "}"}], ".", " ", 
              "Normalized"}], " ", "eigenvector", " ", "of", " ", 
             RowBox[{"(", 
              RowBox[{"QPauliSigmaV", " ", ".", " ", "k"}], ")"}], " ", 
             "belonging", " ", "to", " ", "eigenvalue"}], " ", "+"}], "|", 
           "k", "|", 
           RowBox[{".", " ", 
            RowBox[{"Package", ":", " ", 
             RowBox[{"VQM`Spinors`", "."}]}]}]}]}]}],
        FontFamily->"Times"]},
      {
       StyleBox[
        RowBox[{"QSpinorDown", "[", 
         RowBox[{"{", 
          RowBox[{"k1", ",", "k2", ",", "k3"}], "}"}], "]"}],
        FontFamily->"Courier"], 
       StyleBox[
        RowBox[{" ", 
         RowBox[{
          RowBox[{
           RowBox[{"is", " ", "the", " ", "two"}], "-", 
           RowBox[{"dimensional", " ", "complex", " ", "vector", " ", 
            RowBox[{"describing", " ", "'"}], "spin", " ", 
            RowBox[{"down", "'"}], " ", "in", " ", "the", " ", "direction", 
            " ", "defined", " ", "by", " ", "k"}]}], " ", "=", " ", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"{", 
               RowBox[{"k1", ",", "k2", ",", "k3"}], "}"}], ".", " ", 
              "Normalized"}], " ", "eigenvector", " ", "of", " ", 
             RowBox[{"(", 
              RowBox[{"QPauliSigmaV", " ", ".", " ", "k"}], ")"}], " ", 
             "belonging", " ", "to", " ", "eigenvalue"}], " ", "-"}], "|", 
           "k", "|", 
           RowBox[{".", " ", 
            RowBox[{"Package", ":", " ", 
             RowBox[{"VQM`Spinors`", "."}]}]}]}]}]}],
        FontFamily->"Times"]},
      {
       StyleBox[
        RowBox[{"QConjSpinorUp", "[", 
         RowBox[{"{", 
          RowBox[{"k1", ",", "k2", ",", "k3"}], "}"}], "]"}],
        FontFamily->"Courier"], 
       StyleBox[
        RowBox[{".", " ", 
         RowBox[{"Package", ":", " ", 
          RowBox[{"VQM`Spinors`", "."}]}]}],
        FontFamily->"Times"]},
      {
       StyleBox[
        RowBox[{"QConjSpinorDown", "[", 
         RowBox[{"{", 
          RowBox[{"k1", ",", "k2", ",", "k3"}], "}"}], "]"}],
        FontFamily->"Courier"], 
       StyleBox[
        RowBox[{".", " ", 
         RowBox[{"Package", ":", " ", 
          RowBox[{"VQM`Spinors`", "."}]}]}],
        FontFamily->"Times"]},
      {
       StyleBox[
        RowBox[{"QSpinorHarmonicUp", "[", 
         RowBox[{"j", ",", "m", ",", 
          RowBox[{"{", 
           RowBox[{"theta", ",", "phi"}], "}"}]}], "]"}],
        FontFamily->"Courier"], 
       StyleBox[
        RowBox[{" ", 
         RowBox[{
          RowBox[{
          "gives", " ", "the", " ", "spherical", " ", "harmonic", " ", 
           "spinor", " ", "with", " ", "spin", " ", "parallel", " ", "to", 
           " ", "the", " ", "orbital", " ", "angular", " ", 
           RowBox[{"momentum", ".", " ", "It"}], " ", "is", " ", "an", " ", 
           "eigenfunction", " ", "of", " ", "the", " ", "angular", " ", 
           "momentum", " ", "operators", " ", 
           RowBox[{"J", "^", "2"}]}], ",", " ", 
          RowBox[{"L", "^", "2"}], ",", " ", 
          RowBox[{"J_", "3"}], ",", " ", 
          RowBox[{"and", " ", 
           RowBox[{"L", ".", "S", ".", " ", 
            RowBox[{"Package", ":", " ", 
             RowBox[{"VQM`Spinors`", "."}]}]}]}]}]}],
        FontFamily->"Times"]},
      {
       StyleBox[
        RowBox[{"QSpinorHarmonicDown", "[", 
         RowBox[{"j", ",", "m", ",", 
          RowBox[{"{", 
           RowBox[{"theta", ",", "phi"}], "}"}]}], "]"}],
        FontFamily->"Courier"], 
       StyleBox[
        RowBox[{" ", 
         RowBox[{
          RowBox[{
          "gives", " ", "the", " ", "spherical", " ", "harmonic", " ", 
           "spinor", " ", "with", " ", "spin", " ", "antiparallel", " ", "to",
            " ", "the", " ", "orbital", " ", "angular", " ", 
           RowBox[{"momentum", ".", " ", "It"}], " ", "is", " ", "an", " ", 
           "eigenfunction", " ", "of", " ", "the", " ", "angular", " ", 
           "momentum", " ", "operators", " ", 
           RowBox[{"J", "^", "2"}]}], ",", " ", 
          RowBox[{"L", "^", "2"}], ",", " ", 
          RowBox[{"J_", "3"}], ",", " ", 
          RowBox[{"and", " ", 
           RowBox[{"L", ".", "S", ".", " ", 
            RowBox[{"Package", ":", " ", 
             RowBox[{"VQM`Spinors`", "."}]}]}]}]}]}],
        FontFamily->"Times"]},
      {
       StyleBox[
        RowBox[{"QSpinorToVector", "[", 
         RowBox[{"{", 
          RowBox[{"z1", ",", "z2"}], "}"}], "]"}],
        FontFamily->"Courier"], 
       StyleBox[
        RowBox[{" ", 
         RowBox[{
          RowBox[{
           RowBox[{"defines", " ", "a", " ", "real", " ", "three"}], "-", 
           RowBox[{
           "dimensional", " ", "vector", " ", "in", " ", "the", " ", "spin"}],
            "-", 
           RowBox[{"up", " ", "direction", " ", "of", " ", "psi"}]}], " ", 
          "=", " ", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"{", 
              RowBox[{"z1", ",", "z2"}], "}"}], ".", " ", "The"}], " ", 
            "vector", " ", "is", " ", "obtained", " ", "from", " ", "the", 
            " ", "expectation", " ", "value", " ", "of", " ", "QPauliSigmaV", 
            " ", "in", " ", "the", " ", "state", " ", "defined", " ", "by", 
            " ", "psi"}], " ", "=", " ", 
           RowBox[{
            RowBox[{
             RowBox[{"{", 
              RowBox[{"z1", ",", "z2"}], "}"}], ".", " ", "The"}], " ", 
            "length", " ", "of", " ", "the", " ", "vector", " ", "is", " ", 
            RowBox[{
             RowBox[{"Abs", "[", "psi", "]"}], "^", "2."}], " ", "The", " ", 
            "spinor", " ", 
            RowBox[{"{", 
             RowBox[{"z1", ",", "z2"}], "}"}], " ", "must", " ", "be", " ", 
            RowBox[{"nonzero", ".", " ", 
             RowBox[{"Package", ":", " ", 
              RowBox[{"VQM`Spinors`", "."}]}]}]}]}]}]}],
        FontFamily->"Times"]},
      {
       StyleBox[
        RowBox[{"QVectorToSpinor", "[", 
         RowBox[{"{", 
          RowBox[{"x", ",", "y", ",", "z"}], "}"}], "]"}],
        FontFamily->"Courier"], 
       StyleBox[
        RowBox[{" ", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
            "defines", " ", "a", " ", "spinor", " ", "psi", " ", "which", " ",
              "is", " ", "spin"}], "-", 
            RowBox[{
            "up", " ", "in", " ", "the", " ", "direction", " ", "defined", 
             " ", "by", " ", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"x", ",", "y", ",", "z"}], "}"}], ".", " ", "The"}], 
             " ", "norm", " ", "of", " ", "the", " ", "spinor", " ", "is", 
             " ", "related", " ", "to", " ", "the", " ", "norm", " ", "of", 
             " ", "the", " ", "vector", " ", "by"}]}], " ", "|", 
           RowBox[{"{", 
            RowBox[{"x", ",", "y", ",", "z"}], "}"}], "|"}], "=", 
          RowBox[{
           RowBox[{
            RowBox[{"Abs", "[", "psi", "]"}], "^", "2."}], " ", 
           RowBox[{"Package", ":", " ", 
            RowBox[{"VQM`Spinors`", "."}]}]}]}]}],
        FontFamily->"Times"]},
      {
       StyleBox[
        RowBox[{"QExtractPhase", "[", "psi", "]"}],
        FontFamily->"Courier"], 
       StyleBox[
        RowBox[{" ", 
         RowBox[{
          RowBox[{
           RowBox[{
           "returns", " ", "a", " ", "real", " ", "number", " ", "arg", " ", 
            "for", " ", "any", " ", "nonzero", " ", "psi"}], "=", 
           RowBox[{
            RowBox[{
             RowBox[{"{", 
              RowBox[{"z1", ",", "z2"}], "}"}], ".", " ", "The"}], " ", 
            "number", " ", "arg", " ", "is", " ", "a", " ", "phase", " ", 
            "determined", " ", "from", " ", "comparison", " ", "with", " ", 
            "QSpinorUp", " ", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
              "with", " ", "respect", " ", "to", " ", "the", " ", "direction",
                " ", "defined", " ", "by", " ", "psi"}], ")"}], ".", " ", 
             "Since"}], " ", "QSpinorUp", " ", "is", " ", "defined", " ", 
            "with", " ", "a", " ", "real", " ", "first", " ", "component"}]}],
           ",", " ", 
          RowBox[{
          "arg", " ", "is", " ", "just", " ", "the", " ", "argument", " ", 
           "of", " ", 
           RowBox[{"z1", ".", " ", "For"}], " ", "a", " ", "normalized", " ", 
           "spinor", " ", "psi"}], ",", " ", 
          RowBox[{
           RowBox[{"we", " ", "have", " ", "psi"}], "=", 
           RowBox[{
            RowBox[{"Exp", "[", 
             RowBox[{"I", " ", "arg"}], "]"}], " ", 
            RowBox[{
             RowBox[{"QSpinorUp", "[", 
              RowBox[{"QSpinorToVector", "[", "psi", "]"}], "]"}], ".", " ", 
             RowBox[{"Package", ":", " ", 
              RowBox[{"VQM`Spinors`", "."}]}]}]}]}]}]}],
        FontFamily->"Times"]},
      {
       StyleBox[
        RowBox[{"QHermitianMatrixToVector", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"z1", ",", "z2"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"z3", ",", "z4"}], "}"}]}], "}"}], "]"}],
        FontFamily->"Courier"], 
       StyleBox[
        RowBox[{" ", 
         RowBox[{
          RowBox[{
           RowBox[{"converts", " ", "a", "  ", "Hermitian", " ", "two"}], "-",
            "by", "-", 
           RowBox[{
           "two", " ", "matrix", " ", "into", " ", "a", " ", "real"}]}], ",", 
          " ", 
          RowBox[{"four", "-", 
           RowBox[{"dimensional", " ", "vector"}]}], ",", " ", 
          RowBox[{
          "by", " ", "writing", " ", "the", " ", "matrix", " ", "as", " ", 
           "a", " ", "linear", " ", "combination", " ", "of", " ", "the", " ",
            "identity", " ", "and", " ", "the", " ", "Pauli", " ", 
           RowBox[{"matrices", ".", " ", 
            RowBox[{"Package", ":", " ", 
             RowBox[{"VQM`Spinors`", "."}]}]}]}]}]}],
        FontFamily->"Times"]},
      {
       StyleBox[
        RowBox[{"QDensityMatrixToVector", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"z1", ",", "z2"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"z3", ",", "z4"}], "}"}]}], "}"}], "]"}],
        FontFamily->"Courier"], 
       StyleBox[
        RowBox[{" ", 
         RowBox[{
          RowBox[{
           RowBox[{"converts", " ", "a", "  ", "Hermitian", " ", "two"}], "-",
            "by", "-", 
           RowBox[{
           "two", " ", "matrix", " ", "into", " ", "a", " ", "real"}]}], ",", 
          " ", 
          RowBox[{"three", "-", 
           RowBox[{"dimensional", " ", "vector"}]}], ",", " ", 
          RowBox[{
          "assuming", " ", "that", " ", "the", " ", "matrix", " ", "is", " ", 
           "a", " ", "density", " ", "matrix", " ", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"i", ".", "e", "."}], ",", " ", 
              RowBox[{"has", " ", "trace", " ", "1"}]}], ")"}], ".", " ", 
            RowBox[{"Package", ":", " ", 
             RowBox[{"VQM`Spinors`", "."}]}]}]}]}]}],
        FontFamily->"Times"]},
      {
       StyleBox[
        RowBox[{"QSpinorToArrow", "[", 
         RowBox[{"pt", ",", "spinor"}], "]"}],
        FontFamily->"Courier"], 
       StyleBox[
        RowBox[{" ", 
         RowBox[{
         "gives", " ", "a", " ", "list", " ", "of", " ", "lines", " ", 
          "representing", " ", "an", " ", "arrow", " ", "that", " ", 
          "corresponds", " ", "to", " ", "the", " ", "spinor", " ", "as", " ",
           "defined", " ", "in", " ", 
          RowBox[{"QSpinorToVector", ".", " ", "The"}], " ", "argument", " ", 
          "pt", " ", "is", " ", "optional", " ", "and", " ", "defaults", " ", 
          "to", " ", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"0", ",", "0", ",", "0"}], "}"}], ".", " ", 
           RowBox[{"Package", ":", " ", 
            RowBox[{"VQM`Spinors`", "."}]}]}]}]}],
        FontFamily->"Times"]},
      {
       StyleBox[
        RowBox[{"QRotationSO3", "[", 
         RowBox[{"3", "vector"}], "]"}],
        FontFamily->"Courier"], 
       StyleBox[
        RowBox[{" ", 
         RowBox[{
         "is", " ", "an", " ", "orthogonal", " ", "3", " ", "by", " ", "3", 
          " ", "matrix", " ", "that", " ", "rotates", " ", "a", " ", "vector",
           " ", "around", " ", "the", " ", "axis", " ", "defined", " ", "by", 
          " ", "the", " ", "direction", " ", "of", " ", "3", "vector", " ", 
          "through", " ", "an", " ", "angle", " ", "defined", " ", "by", " ", 
          "the", " ", "size", " ", "of", " ", "3", 
          RowBox[{"vector", ".", " ", 
           RowBox[{"Package", ":", " ", 
            RowBox[{"VQM`Spinors`", "."}]}]}]}]}],
        FontFamily->"Times"]},
      {
       StyleBox[
        RowBox[{"QRotationSU2", "[", 
         RowBox[{"3", "vector"}], "]"}],
        FontFamily->"Courier"], 
       StyleBox[
        RowBox[{" ", 
         RowBox[{
         "is", " ", "a", " ", "unitary", " ", "2", " ", "by", " ", "2", " ", 
          "matrix", " ", "that", " ", "rotates", " ", "a", " ", "spinor", " ",
           "around", " ", "the", " ", "axis", " ", "defined", " ", "by", " ", 
          "the", " ", "direction", " ", "of", " ", "3", "vector", " ", 
          "through", " ", "an", " ", "angle", " ", "defined", " ", "by", " ", 
          "the", " ", "size", " ", "of", " ", "3", 
          RowBox[{"vector", ".", " ", 
           RowBox[{"Package", ":", " ", 
            RowBox[{"VQM`Spinors`", "."}]}]}]}]}],
        FontFamily->"Times"]},
      {
       StyleBox[
        RowBox[{"QVisualizeSpinor", "[", "spinor", "]"}],
        FontFamily->"Courier"], 
       StyleBox[
        RowBox[{" ", 
         RowBox[{
          RowBox[{
          "converts", " ", "a", " ", "spinor", " ", "into", " ", "a", " ", 
           "magnetic", " ", "needle", " ", "and", " ", "displays", " ", "it", 
           " ", "together", " ", "with", " ", "other", " ", "graphics", " ", 
           "elements", " ", "whose", " ", "appearance", " ", "is", " ", 
           "controlled", " ", "by", " ", "the", " ", "options", " ", 
           "QDrawUnitSphere"}], ",", " ", "QDrawAxes", ",", " ", 
          "QCoordinateCube", ",", " ", 
          RowBox[{
           RowBox[{
            RowBox[{"QCoordinateCircles", ".", " ", "Behavior"}], " ", "for", 
            " ", "QNeedleStyle"}], "->", 
           RowBox[{
            RowBox[{
             RowBox[{"True", " ", 
              RowBox[{"(", "default", ")"}]}], ":", " ", 
             RowBox[{
             "The", " ", "needle", " ", "points", " ", "from", " ", "the", 
              " ", "coordinate", " ", "origin", " ", "to", " ", "the", " ", 
              "point", " ", 
              RowBox[{
               RowBox[{"QSpinorToVector", "[", "spinor", "]"}], ".", " ", 
               "The"}], " ", "upper", " ", 
              RowBox[{"(", "lower", ")"}], " ", "half", " ", "of", " ", "the",
               " ", "needle", " ", "has", " ", "a", " ", "color", " ", 
              "determined", " ", "from", " ", "the", " ", "upper", " ", 
              RowBox[{"(", "lower", ")"}], " ", "component", " ", "of", " ", 
              "the", " ", "spinor", " ", "via", " ", "QComplexToColor", " ", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                "from", " ", "the", " ", "package", " ", "VQM`ComplexPlot`"}],
                 ")"}], ".", " ", "Behavior"}], " ", "for", " ", 
              "QNeedleStyle"}]}], "->", 
            RowBox[{
             RowBox[{
              RowBox[{"False", ".", " ", "An"}], " ", "arrow", " ", "is", " ",
               "shown", " ", "instead", " ", "of", " ", "the", " ", 
              RowBox[{"needle", ".", " ", "Giving"}], " ", "the", " ", 
              "option", " ", "QHeadColor"}], "->", 
             RowBox[{
             "QExtractPhase", " ", "colors", " ", "the", " ", "head", " ", 
              "of", " ", "the", " ", "arrow", " ", "by", " ", "the", " ", 
              "phase", " ", "of", " ", "the", " ", "first", " ", "component", 
              " ", "of", " ", "the", " ", 
              RowBox[{"spinor", ".", " ", "You"}], " ", "can", " ", "give", 
              " ", "all", " ", "options", " ", "from", " ", 
              "QVisualizeVector", " ", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                "from", " ", "the", " ", "package", " ", 
                 "VQM`VisualizeVector`"}], ")"}], ".", " ", 
               RowBox[{"Package", ":", " ", 
                RowBox[{"VQM`Spinors`", "."}]}]}]}]}]}]}]}]}],
        FontFamily->"Times"]},
      {
       StyleBox[
        RowBox[{"QVisualizeDensityMatrix", "[", "matrix", "]"}],
        FontFamily->"Courier"], 
       StyleBox[
        RowBox[{" ", 
         RowBox[{
          RowBox[{
          "converts", " ", "a", " ", "Hermitian", " ", "matrix", " ", "with", 
           " ", "trace", " ", "1", " ", "into", " ", "an", " ", "arrow", " ", 
           "graphics", " ", 
           RowBox[{"(", 
            RowBox[{
            "with", " ", "the", " ", "help", " ", "of", " ", 
             "QDensityMatrixToVector"}], ")"}], " ", "and", " ", "displays", 
           " ", "it", " ", "together", " ", "with", " ", "other", " ", 
           "graphics", " ", "elements", " ", "whose", " ", "appearance", " ", 
           "is", " ", "controlled", " ", "by", " ", "the", " ", "options", 
           " ", "QDrawUnitSphere"}], ",", " ", "QDrawAxes", ",", " ", 
          "QCoordinateCube", ",", " ", 
          RowBox[{"QCoordinateCircles", ".", " ", 
           RowBox[{"Package", ":", " ", 
            RowBox[{"VQM`Spinors`", "."}]}]}]}]}],
        FontFamily->"Times"]}
     },
     GridBoxAlignment->{
      "Columns" -> {Right, {Left}}, "ColumnsIndexed" -> {}, 
       "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
     GridBoxDividers->{
      "Columns" -> {{False}}, "ColumnsIndexed" -> {}, "Rows" -> {{False}}, 
       "RowsIndexed" -> {}}]], "2ColumnBox"], TraditionalForm]], "Text",
 CellTags->"VQM`Spinors`"],

Cell["This loads the package", "MathCaption"],

Cell["Needs[\"VQM`Spinors`\"]", "Input",
 CellChangeTimes->{{3.391858495609375*^9, 3.391858499*^9}},
 CellLabel->"In[98]:="],

Cell["This is an example", "MathCaption"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QVisualizeSpinor", "[", 
  RowBox[{
   RowBox[{"QNormalize", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "1"}], "+", "\[ImaginaryI]"}], ",", 
      RowBox[{
       RowBox[{"-", 
        FractionBox["\[ImaginaryI]", "3"]}], "-", "1"}]}], "}"}], "]"}], ",", 
   
   RowBox[{"QDrawUnitSphere", "\[Rule]", "15"}], ",", 
   RowBox[{"QCoordinateCube", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellChangeTimes->{3.391858503703125*^9},
 CellLabel->"In[99]:="],

Cell[BoxData[
 Graphics3DBox[{
   StyleBox[{
     {Glow[
       Hue[
        Rational[3, 8], 1, 0.8604904689046003]], 
      StyleBox[{
        Polygon3DBox[
         NCache[{{Rational[3, 7], Rational[6, 7], Rational[2, 7]}, {
           Rational[3, 14] + Rational[1, 4] 5^Rational[-1, 2], Rational[3, 7] + 
            Rational[-1, 8] 5^Rational[-1, 2], Rational[1, 7]}, {
           Rational[3, 14] + Rational[1, 56] Rational[3, 5]^Rational[1, 2] + 
            Rational[1, 8] 5^Rational[-1, 2], Rational[3, 7] + 
            Rational[1, 28] Rational[3, 5]^Rational[1, 2] + 
            Rational[-1, 16] 5^Rational[-1, 2], Rational[1, 7] + 
            Rational[-3, 112] 15^Rational[1, 2]}}, {{0.42857142857142855`, 
          0.8571428571428571, 0.2857142857142857}, {0.32608911316070377`, 
          0.3726697291339338, 0.14285714285714285`}, {0.2840194971025212, 
          0.4282847456113056, 0.03911651751230133}}]], 
        Polygon3DBox[
         NCache[{{Rational[3, 7], Rational[6, 7], Rational[2, 7]}, {
           Rational[3, 14] + Rational[1, 56] Rational[3, 5]^Rational[1, 2] + 
            Rational[1, 8] 5^Rational[-1, 2], Rational[3, 7] + 
            Rational[1, 28] Rational[3, 5]^Rational[1, 2] + 
            Rational[-1, 16] 5^Rational[-1, 2], Rational[1, 7] + 
            Rational[-3, 112] 15^Rational[1, 2]}, {
           Rational[3, 14] + Rational[1, 56] Rational[3, 5]^Rational[1, 2] + 
            Rational[-1, 8] 5^Rational[-1, 2], Rational[3, 7] + 
            Rational[1, 28] Rational[3, 5]^Rational[1, 2] + 
            Rational[1, 16] 5^Rational[-1, 2], Rational[1, 7] + 
            Rational[-3, 112] 15^Rational[1, 2]}}, {{0.42857142857142855`, 
          0.8571428571428571, 0.2857142857142857}, {0.2840194971025212, 
          0.4282847456113056, 0.03911651751230133}, {0.17221609822753173`, 
          0.4841864450488003, 0.03911651751230133}}]], 
        Polygon3DBox[
         NCache[{{Rational[3, 7], Rational[6, 7], Rational[2, 7]}, {
           Rational[3, 14] + Rational[1, 56] Rational[3, 5]^Rational[1, 2] + 
            Rational[-1, 8] 5^Rational[-1, 2], Rational[3, 7] + 
            Rational[1, 28] Rational[3, 5]^Rational[1, 2] + 
            Rational[1, 16] 5^Rational[-1, 2], Rational[1, 7] + 
            Rational[-3, 112] 15^Rational[1, 2]}, {
           Rational[3, 14] + Rational[-1, 4] 5^Rational[-1, 2], 
            Rational[3, 7] + Rational[1, 8] 5^Rational[-1, 2], Rational[
            1, 7]}}, {{0.42857142857142855`, 0.8571428571428571, 
          0.2857142857142857}, {0.17221609822753173`, 0.4841864450488003, 
          0.03911651751230133}, {0.10248231541072479`, 0.4844731280089233, 
          0.14285714285714285`}}]], 
        Polygon3DBox[
         NCache[{{Rational[3, 7], Rational[6, 7], Rational[2, 7]}, {
           Rational[3, 14] + Rational[-1, 4] 5^Rational[-1, 2], 
            Rational[3, 7] + Rational[1, 8] 5^Rational[-1, 2], Rational[
            1, 7]}, {
           Rational[3, 14] + Rational[-1, 56] Rational[3, 5]^Rational[1, 2] + 
            Rational[-1, 8] 5^Rational[-1, 2], Rational[3, 7] + 
            Rational[-1, 28] Rational[3, 5]^Rational[1, 2] + 
            Rational[1, 16] 5^Rational[-1, 2], Rational[1, 7] + 
            Rational[3, 112] 15^Rational[1, 2]}}, {{0.42857142857142855`, 
          0.8571428571428571, 0.2857142857142857}, {0.10248231541072479`, 
          0.4844731280089233, 0.14285714285714285`}, {0.14455193146890732`, 
          0.4288581115315515, 0.24659776820198437`}}]], 
        Polygon3DBox[
         NCache[{{Rational[3, 7], Rational[6, 7], Rational[2, 7]}, {
           Rational[3, 14] + Rational[-1, 56] Rational[3, 5]^Rational[1, 2] + 
            Rational[-1, 8] 5^Rational[-1, 2], Rational[3, 7] + 
            Rational[-1, 28] Rational[3, 5]^Rational[1, 2] + 
            Rational[1, 16] 5^Rational[-1, 2], Rational[1, 7] + 
            Rational[3, 112] 15^Rational[1, 2]}, {
           Rational[3, 14] + Rational[-1, 56] Rational[3, 5]^Rational[1, 2] + 
            Rational[1, 8] 5^Rational[-1, 2], Rational[3, 7] + 
            Rational[-1, 28] Rational[3, 5]^Rational[1, 2] + 
            Rational[-1, 16] 5^Rational[-1, 2], Rational[1, 7] + 
            Rational[3, 112] 15^Rational[1, 2]}}, {{0.42857142857142855`, 
          0.8571428571428571, 0.2857142857142857}, {0.14455193146890732`, 
          0.4288581115315515, 0.24659776820198437`}, {0.2563553303438968, 
          0.3729564120940568, 0.24659776820198437`}}]], 
        Polygon3DBox[
         NCache[{{Rational[3, 7], Rational[6, 7], Rational[2, 7]}, {
           Rational[3, 14] + Rational[-1, 56] Rational[3, 5]^Rational[1, 2] + 
            Rational[1, 8] 5^Rational[-1, 2], Rational[3, 7] + 
            Rational[-1, 28] Rational[3, 5]^Rational[1, 2] + 
            Rational[-1, 16] 5^Rational[-1, 2], Rational[1, 7] + 
            Rational[3, 112] 15^Rational[1, 2]}, {
           Rational[3, 14] + Rational[1, 4] 5^Rational[-1, 2], Rational[3, 7] + 
            Rational[-1, 8] 5^Rational[-1, 2], Rational[1, 7]}}, {{
          0.42857142857142855`, 0.8571428571428571, 0.2857142857142857}, {
          0.2563553303438968, 0.3729564120940568, 0.24659776820198437`}, {
          0.32608911316070377`, 0.3726697291339338, 0.14285714285714285`}}]]},
       
       Lighting->None]}, 
     {Glow[
       Hue[0.5512081911747834, 1, 0.6858476263181663]], 
      StyleBox[{
        Polygon3DBox[
         NCache[{{0, 0, 0}, {
           Rational[3, 14] + Rational[1, 4] 5^Rational[-1, 2], Rational[3, 7] + 
            Rational[-1, 8] 5^Rational[-1, 2], Rational[1, 7]}, {
           Rational[3, 14] + Rational[1, 56] Rational[3, 5]^Rational[1, 2] + 
            Rational[1, 8] 5^Rational[-1, 2], Rational[3, 7] + 
            Rational[1, 28] Rational[3, 5]^Rational[1, 2] + 
            Rational[-1, 16] 5^Rational[-1, 2], Rational[1, 7] + 
            Rational[-3, 112] 15^Rational[1, 2]}}, {{0, 0, 0}, {
           0.32608911316070377`, 0.3726697291339338, 0.14285714285714285`}, {
           0.2840194971025212, 0.4282847456113056, 0.03911651751230133}}]], 
        Polygon3DBox[
         NCache[{{0, 0, 0}, {
           Rational[3, 14] + Rational[1, 56] Rational[3, 5]^Rational[1, 2] + 
            Rational[1, 8] 5^Rational[-1, 2], Rational[3, 7] + 
            Rational[1, 28] Rational[3, 5]^Rational[1, 2] + 
            Rational[-1, 16] 5^Rational[-1, 2], Rational[1, 7] + 
            Rational[-3, 112] 15^Rational[1, 2]}, {
           Rational[3, 14] + Rational[1, 56] Rational[3, 5]^Rational[1, 2] + 
            Rational[-1, 8] 5^Rational[-1, 2], Rational[3, 7] + 
            Rational[1, 28] Rational[3, 5]^Rational[1, 2] + 
            Rational[1, 16] 5^Rational[-1, 2], Rational[1, 7] + 
            Rational[-3, 112] 15^Rational[1, 2]}}, {{0, 0, 0}, {
           0.2840194971025212, 0.4282847456113056, 0.03911651751230133}, {
           0.17221609822753173`, 0.4841864450488003, 0.03911651751230133}}]], 
        Polygon3DBox[
         NCache[{{0, 0, 0}, {
           Rational[3, 14] + Rational[1, 56] Rational[3, 5]^Rational[1, 2] + 
            Rational[-1, 8] 5^Rational[-1, 2], Rational[3, 7] + 
            Rational[1, 28] Rational[3, 5]^Rational[1, 2] + 
            Rational[1, 16] 5^Rational[-1, 2], Rational[1, 7] + 
            Rational[-3, 112] 15^Rational[1, 2]}, {
           Rational[3, 14] + Rational[-1, 4] 5^Rational[-1, 2], 
            Rational[3, 7] + Rational[1, 8] 5^Rational[-1, 2], Rational[
            1, 7]}}, {{0, 0, 0}, {0.17221609822753173`, 0.4841864450488003, 
           0.03911651751230133}, {0.10248231541072479`, 0.4844731280089233, 
           0.14285714285714285`}}]], 
        Polygon3DBox[
         NCache[{{0, 0, 0}, {
           Rational[3, 14] + Rational[-1, 4] 5^Rational[-1, 2], 
            Rational[3, 7] + Rational[1, 8] 5^Rational[-1, 2], Rational[
            1, 7]}, {
           Rational[3, 14] + Rational[-1, 56] Rational[3, 5]^Rational[1, 2] + 
            Rational[-1, 8] 5^Rational[-1, 2], Rational[3, 7] + 
            Rational[-1, 28] Rational[3, 5]^Rational[1, 2] + 
            Rational[1, 16] 5^Rational[-1, 2], Rational[1, 7] + 
            Rational[3, 112] 15^Rational[1, 2]}}, {{0, 0, 0}, {
           0.10248231541072479`, 0.4844731280089233, 0.14285714285714285`}, {
           0.14455193146890732`, 0.4288581115315515, 0.24659776820198437`}}]],
         Polygon3DBox[
         NCache[{{0, 0, 0}, {
           Rational[3, 14] + Rational[-1, 56] Rational[3, 5]^Rational[1, 2] + 
            Rational[-1, 8] 5^Rational[-1, 2], Rational[3, 7] + 
            Rational[-1, 28] Rational[3, 5]^Rational[1, 2] + 
            Rational[1, 16] 5^Rational[-1, 2], Rational[1, 7] + 
            Rational[3, 112] 15^Rational[1, 2]}, {
           Rational[3, 14] + Rational[-1, 56] Rational[3, 5]^Rational[1, 2] + 
            Rational[1, 8] 5^Rational[-1, 2], Rational[3, 7] + 
            Rational[-1, 28] Rational[3, 5]^Rational[1, 2] + 
            Rational[-1, 16] 5^Rational[-1, 2], Rational[1, 7] + 
            Rational[3, 112] 15^Rational[1, 2]}}, {{0, 0, 0}, {
           0.14455193146890732`, 0.4288581115315515, 0.24659776820198437`}, {
           0.2563553303438968, 0.3729564120940568, 0.24659776820198437`}}]], 
        Polygon3DBox[
         NCache[{{0, 0, 0}, {
           Rational[3, 14] + Rational[-1, 56] Rational[3, 5]^Rational[1, 2] + 
            Rational[1, 8] 5^Rational[-1, 2], Rational[3, 7] + 
            Rational[-1, 28] Rational[3, 5]^Rational[1, 2] + 
            Rational[-1, 16] 5^Rational[-1, 2], Rational[1, 7] + 
            Rational[3, 112] 15^Rational[1, 2]}, {
           Rational[3, 14] + Rational[1, 4] 5^Rational[-1, 2], Rational[3, 7] + 
            Rational[-1, 8] 5^Rational[-1, 2], Rational[1, 7]}}, {{0, 0, 0}, {
           0.2563553303438968, 0.3729564120940568, 0.24659776820198437`}, {
           0.32608911316070377`, 0.3726697291339338, 
           0.14285714285714285`}}]]},
       Lighting->None]}},
    Lighting->None], {
    {GrayLevel[0.5], Line3DBox[CompressedData["
1:eJxN2HkgVFscB3BlSV6JEClJtiRCFMLJkiREiSxP5cn0hJCSSilZGktUyr7v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      "]], Line3DBox[CompressedData["
1:eJxN2Gk4FV8cB3CEyFpZQpJQipSI+lOHEkokZKmoEFmSJCRZIpFkX1JKtiwR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      "]], Line3DBox[CompressedData["
1:eJxN2Hk8lNsfB3BlTVJZKpEsNykqlxSiQzctdFWWkiw/lUtylT2uPWSPEFmy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      "]], Line3DBox[CompressedData["
1:eJxN2Hk8VN0fB/AYFHkQKUuJJxJJkajQwROVJ2EkiYpCVFpJSSiVirJvUZbs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      "]], Line3DBox[CompressedData["
1:eJxN2HlUzOsfB/BpJwmRog2hciNJtEw9qQglNRX5kRDZijbtN+RW2tw2adNN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      "]], Line3DBox[CompressedData["
1:eJxN2Gk4VdsfB3BSZqWJSoiQsZTpumJlLKUrUpRE4pilkAgpqYQGSblFEpIM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      "]], Line3DBox[CompressedData["
1:eJxN2Gk8lN0bB3BLKIXCU5KU9V+E9lJyJIVCHplIlKVQEVJNiyVkXyJFihYS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      "]], Line3DBox[CompressedData["
1:eJxN2Gk8lN0bB3BLKIXCU5KU9V+E9lJyJIVCHplIlKVQEVJNiyVkXyJFihYS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      "]], Line3DBox[CompressedData["
1:eJxN2Gk8VVsfB3BSZqWJSoiQsZTpumJlLKUrUpRE4pilkAgpqYQGSblFEpIM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      "]], Line3DBox[CompressedData["
1:eJxN2Hk01OsfB/AhVJJKiWKoVOgqScoyeWylIjEo/QpFaaNs2W+Lrp1uItly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      "]], Line3DBox[CompressedData["
1:eJxN2Hk8lNsfB/AYFLmIlKXEjURSJCp0cKNyE0aSqChEpZWUhFKpKPsWZcku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      "]], Line3DBox[CompressedData["
1:eJxN2Hk8lNsfB3BlTVJZKpEsNykqlxSiQzctdFWWkiw/lUtylT2uPWSPEFmy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      "]], Line3DBox[CompressedData["
1:eJxN2Hk4VF0cB3CEyFpZQl4JpUiJqJc6lFAiIUtFhciSJCHJEokk+5JSsmWJ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      "]], Line3DBox[CompressedData["
1:eJxN2HkgVFscB3BlSV6JEClJtiRCFMLJkiREiSxP5cn0hJCSSilZGktUyr7v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      "]]}, {
     {GrayLevel[0.4], Thickness[0.005], 
      Line3DBox[{{-1., 0, 0}, {1., 0, 0}}]}, 
     {GrayLevel[0.4], Thickness[0.005], 
      Line3DBox[{{0, -1., 0}, {0, 1., 0}}]}, 
     {GrayLevel[0.4], Thickness[0.005], 
      Line3DBox[{{0, 0, -1.}, {0, 0, 1.}}]}}, 
    {RGBColor[0, 0.6, 0.4], 
     Line3DBox[
      NCache[{{0, 0, 0}, {Rational[3, 7], 0, 0}, {
        Rational[3, 7], Rational[6, 7], 0}, {
        Rational[3, 7], Rational[6, 7], Rational[2, 7]}, {
        Rational[3, 7], 0, Rational[2, 7]}, {0, 0, Rational[2, 7]}}, {{0, 0, 
        0}, {0.42857142857142855`, 0, 0}, {
        0.42857142857142855`, 0.8571428571428571, 0}, {0.42857142857142855`, 
        0.8571428571428571, 0.2857142857142857}, {
        0.42857142857142855`, 0, 0.2857142857142857}, {
        0, 0, 0.2857142857142857}}]], 
     Line3DBox[
      NCache[{{0, 0, 0}, {0, Rational[6, 7], 0}, {
        Rational[3, 7], Rational[6, 7], 0}, {
        Rational[3, 7], Rational[6, 7], Rational[2, 7]}, {
        0, Rational[6, 7], Rational[2, 7]}, {0, 0, Rational[2, 7]}}, {{0, 0, 
        0}, {0, 0.8571428571428571, 0}, {
        0.42857142857142855`, 0.8571428571428571, 0}, {0.42857142857142855`, 
        0.8571428571428571, 0.2857142857142857}, {
        0, 0.8571428571428571, 0.2857142857142857}, {
        0, 0, 0.2857142857142857}}]], 
     Line3DBox[
      NCache[{{Rational[3, 7], 0, 0}, {Rational[3, 7], 0, Rational[2, 7]}}, {{
        0.42857142857142855`, 0, 0}, {
        0.42857142857142855`, 0, 0.2857142857142857}}]], 
     Line3DBox[
      NCache[{{0, Rational[6, 7], 0}, {0, Rational[6, 7], Rational[2, 7]}}, {{
        0, 0.8571428571428571, 0}, {
        0, 0.8571428571428571, 0.2857142857142857}}]]}, 
    {RGBColor[0, 0, 1], PointSize[0.02], Thickness[0.003], 
     Point3DBox[
      NCache[{Rational[3, 7], Rational[6, 7], Rational[2, 7]}, {
       0.42857142857142855`, 0.8571428571428571, 0.2857142857142857}]], 
     Line3DBox[CompressedData["
1:eJxN1ns41NkfB3Bk7ZNGopV+vxISbUqRZh5SDqJmIqwuREhtatjKSslG5JIl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      "]], Line3DBox[CompressedData["
1:eJw92HlYDf3/x3GVUOG23fYtCmlfRLYXyhKVNVS2EO5blki+aNN+p5RQpEWo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      "]]}}},
  Axes->False,
  Boxed->False,
  Lighting->None]], "Output",
 CellChangeTimes->{3.391858505515625*^9},
 CellLabel->"Out[99]=",
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QVisualizeSpinor", "[", 
  RowBox[{
   RowBox[{"QNormalize", "[", 
    RowBox[{"{", 
     RowBox[{"\[ImaginaryI]", ",", 
      RowBox[{"-", "\[ImaginaryI]"}]}], "}"}], "]"}], ",", 
   RowBox[{"QNeedleStyle", "\[Rule]", "False"}], ",", 
   RowBox[{"QArrowShaft", "\[Rule]", "True"}], ",", 
   RowBox[{"Lighting", "\[Rule]", "None"}], ",", 
   RowBox[{"QArrowShape", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"10", ",", 
      FractionBox["1", "2"], ",", 
      FractionBox["1", "2"], ",", 
      FractionBox["1", "2"]}], "}"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.391858505875*^9, 3.3918585149375*^9}},
 CellLabel->"In[101]:="],

Cell[BoxData[
 Graphics3DBox[{{
    {Glow[
      Hue[
       Rational[1, 4], 1, 0.7836531040612145]], 
     StyleBox[{
       Polygon3DBox[
        NCache[{{-1, 0, 0}, {Rational[-1, 2], Rational[1, 4], 0}, {
          Rational[-1, 2], Rational[1, 16] (1 + 5^Rational[1, 2]), 
           Rational[-1, 4] (Rational[5, 8] + 
             Rational[-1, 8] 5^Rational[1, 2])^Rational[1, 2]}}, {{-1, 0, 
          0}, {-0.5, 0.25, 0}, {-0.5, 
          0.20225424859373686`, -0.14694631307311828`}}]], 
       Polygon3DBox[
        NCache[{{-1, 0, 0}, {
          Rational[-1, 2], Rational[1, 16] (1 + 5^Rational[1, 2]), 
           Rational[-1, 4] (Rational[5, 8] + 
             Rational[-1, 8] 5^Rational[1, 2])^Rational[1, 2]}, {
          Rational[-1, 2], Rational[1, 16] (-1 + 5^Rational[1, 2]), 
           Rational[-1, 4] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}}, {{-1, 0, 0}, {-0.5, 
          0.20225424859373686`, -0.14694631307311828`}, {-0.5, 
          0.07725424859373686, -0.23776412907378838`}}]], 
       Polygon3DBox[
        NCache[{{-1, 0, 0}, {
          Rational[-1, 2], Rational[1, 16] (-1 + 5^Rational[1, 2]), 
           Rational[-1, 4] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}, {
          Rational[-1, 2], Rational[1, 16] (1 - 5^Rational[1, 2]), 
           Rational[-1, 4] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}}, {{-1, 0, 0}, {-0.5, 
          0.07725424859373686, -0.23776412907378838`}, {-0.5, \
-0.07725424859373686, -0.23776412907378838`}}]], 
       Polygon3DBox[
        NCache[{{-1, 0, 0}, {
          Rational[-1, 2], Rational[1, 16] (1 - 5^Rational[1, 2]), 
           Rational[-1, 4] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}, {
          Rational[-1, 2], Rational[1, 16] (-1 - 5^Rational[1, 2]), 
           Rational[-1, 4] (Rational[5, 8] + 
             Rational[-1, 8] 5^Rational[1, 2])^Rational[1, 2]}}, {{-1, 0, 
          0}, {-0.5, -0.07725424859373686, -0.23776412907378838`}, {-0.5, \
-0.20225424859373686`, -0.14694631307311828`}}]], 
       Polygon3DBox[
        NCache[{{-1, 0, 0}, {
          Rational[-1, 2], Rational[1, 16] (-1 - 5^Rational[1, 2]), 
           Rational[-1, 4] (Rational[5, 8] + 
             Rational[-1, 8] 5^Rational[1, 2])^Rational[1, 2]}, {
          Rational[-1, 2], Rational[-1, 4], 0}}, {{-1, 0, 
          0}, {-0.5, -0.20225424859373686`, -0.14694631307311828`}, {-0.5, \
-0.25, 0}}]], 
       Polygon3DBox[
        NCache[{{-1, 0, 0}, {Rational[-1, 2], Rational[-1, 4], 0}, {
          Rational[-1, 2], Rational[1, 16] (-1 - 5^Rational[1, 2]), 
           Rational[1, 4] (Rational[5, 8] + Rational[-1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}}, {{-1, 0, 
          0}, {-0.5, -0.25, 0}, {-0.5, -0.20225424859373686`, 
          0.14694631307311828`}}]], 
       Polygon3DBox[
        NCache[{{-1, 0, 0}, {
          Rational[-1, 2], Rational[1, 16] (-1 - 5^Rational[1, 2]), 
           Rational[1, 4] (Rational[5, 8] + Rational[-1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}, {
          Rational[-1, 2], Rational[1, 16] (1 - 5^Rational[1, 2]), 
           Rational[1, 4] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}}, {{-1, 0, 0}, {-0.5, -0.20225424859373686`, 
          0.14694631307311828`}, {-0.5, -0.07725424859373686, 
          0.23776412907378838`}}]], 
       Polygon3DBox[
        NCache[{{-1, 0, 0}, {
          Rational[-1, 2], Rational[1, 16] (1 - 5^Rational[1, 2]), 
           Rational[1, 4] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}, {
          Rational[-1, 2], Rational[1, 16] (-1 + 5^Rational[1, 2]), 
           Rational[1, 4] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}}, {{-1, 0, 0}, {-0.5, -0.07725424859373686, 
          0.23776412907378838`}, {-0.5, 0.07725424859373686, 
          0.23776412907378838`}}]], 
       Polygon3DBox[
        NCache[{{-1, 0, 0}, {
          Rational[-1, 2], Rational[1, 16] (-1 + 5^Rational[1, 2]), 
           Rational[1, 4] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}, {
          Rational[-1, 2], Rational[1, 16] (1 + 5^Rational[1, 2]), 
           Rational[1, 4] (Rational[5, 8] + Rational[-1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}}, {{-1, 0, 0}, {-0.5, 0.07725424859373686, 
          0.23776412907378838`}, {-0.5, 0.20225424859373686`, 
          0.14694631307311828`}}]], 
       Polygon3DBox[
        NCache[{{-1, 0, 0}, {
          Rational[-1, 2], Rational[1, 16] (1 + 5^Rational[1, 2]), 
           Rational[1, 4] (Rational[5, 8] + Rational[-1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}, {Rational[-1, 2], Rational[1, 4], 0}}, {{-1, 0, 
          0}, {-0.5, 0.20225424859373686`, 
          0.14694631307311828`}, {-0.5, 0.25, 0}}]]},
      Lighting->None]}, 
    {Glow[
      Hue[0.75, 1, 0.7836531040612145]], 
     StyleBox[{
       Polygon3DBox[
        NCache[{{0, Rational[1, 8], 0}, {
          Rational[-1, 2], Rational[1, 8], 0}, {
          Rational[-1, 2], Rational[1, 32] (1 + 5^Rational[1, 2]), 
           Rational[-1, 8] (Rational[5, 8] + 
             Rational[-1, 8] 5^Rational[1, 2])^Rational[1, 2]}, {
          0, Rational[1, 32] (1 + 5^Rational[1, 2]), 
           Rational[-1, 8] (Rational[5, 8] + 
             Rational[-1, 8] 5^Rational[1, 2])^Rational[1, 2]}}, {{
          0, 0.125, 0}, {-0.5, 0.125, 0}, {-0.5, 
          0.10112712429686843`, -0.07347315653655914}, {
          0, 0.10112712429686843`, -0.07347315653655914}}]], 
       Polygon3DBox[
        NCache[{{
          0, Rational[1, 32] (1 + 5^Rational[1, 2]), 
           Rational[-1, 8] (Rational[5, 8] + 
             Rational[-1, 8] 5^Rational[1, 2])^Rational[1, 2]}, {
          Rational[-1, 2], Rational[1, 32] (1 + 5^Rational[1, 2]), 
           Rational[-1, 8] (Rational[5, 8] + 
             Rational[-1, 8] 5^Rational[1, 2])^Rational[1, 2]}, {
          Rational[-1, 2], Rational[1, 32] (-1 + 5^Rational[1, 2]), 
           Rational[-1, 8] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}, {
          0, Rational[1, 32] (-1 + 5^Rational[1, 2]), 
           Rational[-1, 8] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}}, {{
          0, 0.10112712429686843`, -0.07347315653655914}, {-0.5, 
          0.10112712429686843`, -0.07347315653655914}, {-0.5, 
          0.03862712429686843, -0.11888206453689419`}, {
          0, 0.03862712429686843, -0.11888206453689419`}}]], 
       Polygon3DBox[
        NCache[{{
          0, Rational[1, 32] (-1 + 5^Rational[1, 2]), 
           Rational[-1, 8] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}, {
          Rational[-1, 2], Rational[1, 32] (-1 + 5^Rational[1, 2]), 
           Rational[-1, 8] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}, {
          Rational[-1, 2], Rational[1, 32] (1 - 5^Rational[1, 2]), 
           Rational[-1, 8] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}, {
          0, Rational[1, 32] (1 - 5^Rational[1, 2]), 
           Rational[-1, 8] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}}, {{
          0, 0.03862712429686843, -0.11888206453689419`}, {-0.5, 
          0.03862712429686843, -0.11888206453689419`}, {-0.5, \
-0.03862712429686843, -0.11888206453689419`}, {
          0, -0.03862712429686843, -0.11888206453689419`}}]], 
       Polygon3DBox[
        NCache[{{
          0, Rational[1, 32] (1 - 5^Rational[1, 2]), 
           Rational[-1, 8] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}, {
          Rational[-1, 2], Rational[1, 32] (1 - 5^Rational[1, 2]), 
           Rational[-1, 8] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}, {
          Rational[-1, 2], Rational[1, 32] (-1 - 5^Rational[1, 2]), 
           Rational[-1, 8] (Rational[5, 8] + 
             Rational[-1, 8] 5^Rational[1, 2])^Rational[1, 2]}, {
          0, Rational[1, 32] (-1 - 5^Rational[1, 2]), 
           Rational[-1, 8] (Rational[5, 8] + 
             Rational[-1, 8] 5^Rational[1, 2])^Rational[1, 2]}}, {{
          0, -0.03862712429686843, -0.11888206453689419`}, {-0.5, \
-0.03862712429686843, -0.11888206453689419`}, {-0.5, -0.10112712429686843`, \
-0.07347315653655914}, {0, -0.10112712429686843`, -0.07347315653655914}}]], 
       Polygon3DBox[
        NCache[{{
          0, Rational[1, 32] (-1 - 5^Rational[1, 2]), 
           Rational[-1, 8] (Rational[5, 8] + 
             Rational[-1, 8] 5^Rational[1, 2])^Rational[1, 2]}, {
          Rational[-1, 2], Rational[1, 32] (-1 - 5^Rational[1, 2]), 
           Rational[-1, 8] (Rational[5, 8] + 
             Rational[-1, 8] 5^Rational[1, 2])^Rational[1, 2]}, {
          Rational[-1, 2], Rational[-1, 8], 0}, {0, Rational[-1, 8], 0}}, {{
          0, -0.10112712429686843`, -0.07347315653655914}, {-0.5, \
-0.10112712429686843`, -0.07347315653655914}, {-0.5, -0.125, 0}, {
          0, -0.125, 0}}]], 
       Polygon3DBox[
        NCache[{{0, Rational[-1, 8], 0}, {
          Rational[-1, 2], Rational[-1, 8], 0}, {
          Rational[-1, 2], Rational[1, 32] (-1 - 5^Rational[1, 2]), 
           Rational[1, 8] (Rational[5, 8] + Rational[-1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}, {
          0, Rational[1, 32] (-1 - 5^Rational[1, 2]), 
           Rational[1, 8] (Rational[5, 8] + Rational[-1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}}, {{
          0, -0.125, 0}, {-0.5, -0.125, 0}, {-0.5, -0.10112712429686843`, 
          0.07347315653655914}, {
          0, -0.10112712429686843`, 0.07347315653655914}}]], 
       Polygon3DBox[
        NCache[{{
          0, Rational[1, 32] (-1 - 5^Rational[1, 2]), 
           Rational[1, 8] (Rational[5, 8] + Rational[-1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}, {
          Rational[-1, 2], Rational[1, 32] (-1 - 5^Rational[1, 2]), 
           Rational[1, 8] (Rational[5, 8] + Rational[-1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}, {
          Rational[-1, 2], Rational[1, 32] (1 - 5^Rational[1, 2]), 
           Rational[1, 8] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}, {
          0, Rational[1, 32] (1 - 5^Rational[1, 2]), 
           Rational[1, 8] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}}, {{
          0, -0.10112712429686843`, 
           0.07347315653655914}, {-0.5, -0.10112712429686843`, 
          0.07347315653655914}, {-0.5, -0.03862712429686843, 
          0.11888206453689419`}, {
          0, -0.03862712429686843, 0.11888206453689419`}}]], 
       Polygon3DBox[
        NCache[{{
          0, Rational[1, 32] (1 - 5^Rational[1, 2]), 
           Rational[1, 8] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}, {
          Rational[-1, 2], Rational[1, 32] (1 - 5^Rational[1, 2]), 
           Rational[1, 8] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}, {
          Rational[-1, 2], Rational[1, 32] (-1 + 5^Rational[1, 2]), 
           Rational[1, 8] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}, {
          0, Rational[1, 32] (-1 + 5^Rational[1, 2]), 
           Rational[1, 8] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}}, {{
          0, -0.03862712429686843, 
           0.11888206453689419`}, {-0.5, -0.03862712429686843, 
          0.11888206453689419`}, {-0.5, 0.03862712429686843, 
          0.11888206453689419`}, {
          0, 0.03862712429686843, 0.11888206453689419`}}]], 
       Polygon3DBox[
        NCache[{{
          0, Rational[1, 32] (-1 + 5^Rational[1, 2]), 
           Rational[1, 8] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}, {
          Rational[-1, 2], Rational[1, 32] (-1 + 5^Rational[1, 2]), 
           Rational[1, 8] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}, {
          Rational[-1, 2], Rational[1, 32] (1 + 5^Rational[1, 2]), 
           Rational[1, 8] (Rational[5, 8] + Rational[-1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}, {
          0, Rational[1, 32] (1 + 5^Rational[1, 2]), 
           Rational[1, 8] (Rational[5, 8] + Rational[-1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}}, {{
          0, 0.03862712429686843, 0.11888206453689419`}, {-0.5, 
          0.03862712429686843, 0.11888206453689419`}, {-0.5, 
          0.10112712429686843`, 0.07347315653655914}, {
          0, 0.10112712429686843`, 0.07347315653655914}}]], 
       Polygon3DBox[
        NCache[{{
          0, Rational[1, 32] (1 + 5^Rational[1, 2]), 
           Rational[1, 8] (Rational[5, 8] + Rational[-1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}, {
          Rational[-1, 2], Rational[1, 32] (1 + 5^Rational[1, 2]), 
           Rational[1, 8] (Rational[5, 8] + Rational[-1, 8] 5^Rational[1, 2])^
            Rational[1, 2]}, {Rational[-1, 2], Rational[1, 8], 0}, {
          0, Rational[1, 8], 0}}, {{
          0, 0.10112712429686843`, 0.07347315653655914}, {-0.5, 
          0.10112712429686843`, 0.07347315653655914}, {-0.5, 0.125, 0}, {
          0, 0.125, 0}}]]},
      Lighting->None]}, 
    {Glow[
      Hue[0.75, 1, 0.7836531040612145]], 
     StyleBox[
      Polygon3DBox[
       NCache[{{0, Rational[1, 8], 0}, {
         0, Rational[1, 32] (1 + 5^Rational[1, 2]), 
          Rational[-1, 8] (Rational[5, 8] + Rational[-1, 8] 5^Rational[1, 2])^
           Rational[1, 2]}, {
         0, Rational[1, 32] (-1 + 5^Rational[1, 2]), 
          Rational[-1, 8] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
           Rational[1, 2]}, {
         0, Rational[1, 32] (1 - 5^Rational[1, 2]), 
          Rational[-1, 8] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
           Rational[1, 2]}, {
         0, Rational[1, 32] (-1 - 5^Rational[1, 2]), 
          Rational[-1, 8] (Rational[5, 8] + Rational[-1, 8] 5^Rational[1, 2])^
           Rational[1, 2]}, {0, Rational[-1, 8], 0}, {
         0, Rational[1, 32] (-1 - 5^Rational[1, 2]), 
          Rational[1, 8] (Rational[5, 8] + Rational[-1, 8] 5^Rational[1, 2])^
           Rational[1, 2]}, {
         0, Rational[1, 32] (1 - 5^Rational[1, 2]), 
          Rational[1, 8] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
           Rational[1, 2]}, {
         0, Rational[1, 32] (-1 + 5^Rational[1, 2]), 
          Rational[1, 8] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
           Rational[1, 2]}, {
         0, Rational[1, 32] (1 + 5^Rational[1, 2]), 
          Rational[1, 8] (Rational[5, 8] + Rational[-1, 8] 5^Rational[1, 2])^
           Rational[1, 2]}}, {{0, 0.125, 0}, {
         0, 0.10112712429686843`, -0.07347315653655914}, {
         0, 0.03862712429686843, -0.11888206453689419`}, {
         0, -0.03862712429686843, -0.11888206453689419`}, {
         0, -0.10112712429686843`, -0.07347315653655914}, {0, -0.125, 0}, {
         0, -0.10112712429686843`, 0.07347315653655914}, {
         0, -0.03862712429686843, 0.11888206453689419`}, {
         0, 0.03862712429686843, 0.11888206453689419`}, {
         0, 0.10112712429686843`, 0.07347315653655914}}]],
      Lighting->None]}, 
    {Glow[
      Hue[
       Rational[1, 4], 1, 0.7836531040612145]], 
     StyleBox[
      Polygon3DBox[
       NCache[{{Rational[-1, 2], Rational[1, 4], 0}, {
         Rational[-1, 2], Rational[1, 16] (1 + 5^Rational[1, 2]), 
          Rational[-1, 4] (Rational[5, 8] + Rational[-1, 8] 5^Rational[1, 2])^
           Rational[1, 2]}, {
         Rational[-1, 2], Rational[1, 16] (-1 + 5^Rational[1, 2]), 
          Rational[-1, 4] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
           Rational[1, 2]}, {
         Rational[-1, 2], Rational[1, 16] (1 - 5^Rational[1, 2]), 
          Rational[-1, 4] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
           Rational[1, 2]}, {
         Rational[-1, 2], Rational[1, 16] (-1 - 5^Rational[1, 2]), 
          Rational[-1, 4] (Rational[5, 8] + Rational[-1, 8] 5^Rational[1, 2])^
           Rational[1, 2]}, {Rational[-1, 2], Rational[-1, 4], 0}, {
         Rational[-1, 2], Rational[1, 16] (-1 - 5^Rational[1, 2]), 
          Rational[1, 4] (Rational[5, 8] + Rational[-1, 8] 5^Rational[1, 2])^
           Rational[1, 2]}, {
         Rational[-1, 2], Rational[1, 16] (1 - 5^Rational[1, 2]), 
          Rational[1, 4] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
           Rational[1, 2]}, {
         Rational[-1, 2], Rational[1, 16] (-1 + 5^Rational[1, 2]), 
          Rational[1, 4] (Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
           Rational[1, 2]}, {
         Rational[-1, 2], Rational[1, 16] (1 + 5^Rational[1, 2]), 
          Rational[1, 4] (Rational[5, 8] + Rational[-1, 8] 5^Rational[1, 2])^
           Rational[1, 2]}}, {{-0.5, 0.25, 0}, {-0.5, 
         0.20225424859373686`, -0.14694631307311828`}, {-0.5, 
         0.07725424859373686, -0.23776412907378838`}, {-0.5, \
-0.07725424859373686, -0.23776412907378838`}, {-0.5, -0.20225424859373686`, \
-0.14694631307311828`}, {-0.5, -0.25, 0}, {-0.5, -0.20225424859373686`, 
         0.14694631307311828`}, {-0.5, -0.07725424859373686, 
         0.23776412907378838`}, {-0.5, 0.07725424859373686, 
         0.23776412907378838`}, {-0.5, 0.20225424859373686`, 
         0.14694631307311828`}}]],
      Lighting->None]}}, {
    {GrayLevel[0.5], Line3DBox[CompressedData["
1:eJxN2Hk8VOsfB3C7sqW6IWulblmiKCQ8ylIhkuxalFBJIq4lokHWUCJklxGy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      "]], Line3DBox[CompressedData["
1:eJxN2Gk4VVscBnBJ4lLGEkrclBIhJJIlZO64NxIqIlGSkjkkQykiMmc4Cklm
IvMyn2N2hm0eMsStK0lSibp9uWv1YX/5PedZz9lr/9f77nMk7K6dvMjKwsIy
/Ota/+ta5Apzfz5FAMGytst18noa22VPai3zzYP8RFOh9b9cX102SPu4B/jf
xSc7OQUmCHDwWpH1E507yKtDOHuDRwmg0RNvKHAxCvnCG32T6wMEKJi82lSy
Jx75XTvHo7PtBLB413FA1DUdees21s0hFQSIX7e0lbcoC3lR6EkwkkaAgJWy
TpEzuciXjrx8FBtIgOrTVvFGJkXIS7ZQrXjPEqA+Wi2Lc3cpcuUNjQlOCgSY
91A0N/+nDHm0n5akLxsBzj9+TW3zrMCfP894cKCbCTRCzWoXlKuRNzmuFpLi
mWCufYSRDGuRu92/XVZpxQSBlRf4KSL1yNP2tGo1CjLBso+mPsm5AXk2q8LF
8G4GyPGOo648bUTuvPydY5c/A3zRsVerm29CLuS3oXhBigG62/t8uORakJ+R
Pbx1iEEHeU62V7PMWpGvn8nyLnSjg0fWsn/LR1CQ5xVmplhtoQNSmQ7Lllwq
cvPvtilaL2hga8SRG4pDbciTlIYsDNVpINX9TEvtajvyehqjyp/WC8g56WbK
GzqRHxBxoala9gJlLbqo7/Yu5Nwrd7hL23pAjuas+GfhbuSP2Nlv8411g8W6
1gpHrh7kij/eGVr1dQGXIHuFRLZe5CpOfuEzA50g/AS3AZ8XdoUmgS6vug4g
Jt7GVT6P/XNQ8/tnT9pBnIWxwiMbGnKPVVl6aPSv+9zXzpnejN3MYZQ39wIV
HFBpYAmSpSPnvSc72KtNATf7+wVnwrCHPVXjPyTZCpYjBEslJrG7N5a+PTvT
DJJqeGaHjzCQz35tMS6oaAIu9/86PBaGPTs2n8vOuRH8zDqlZjyMXYz/W4en
QgPQCyoXc5Zg4rmajRiz+AhBeEe8w3Vb7AJf1fouaNWCGqkrM0FPsFsa3HvI
UlUF/tl1KebQAHbVq3EFckuvwHV1lpXxTQTy1WtH3rcolYHdrzXHaerYr37g
PHGVXALm/z6+yfUC9n9H1093PS4Eyiylphmh2AtyA/WrSS9Aa+a+YZ5M7Hmz
sf17hbJAZ/AzxbEq7H0vM1dP0NKA6xcZ7rku7H+nyNWxBCaBpxea6RQm9mOP
W4G8XCyokynYYTGEvSdUj3SP8QAM/Ssn/XYM+6fy9Ml83UDQxv6Qd2kCe0q5
iHVtlSPgTXdf5zGF/S2HEZX57CKMHeEi+//mASx7Us+tBsCr7xj+ByexH0q0
0aiMDoM7mm8KpY9jf1/rm6vIFgOrioCW/fBv++bUqKRelwjJ3mqL2QR20Yq2
zu1lT6CgDe+1D63YjSXq1M95Z8MzR396CRdjH40UnT0rkg+T8z5cqYrDft6N
O9xIuhiO6nO/eeeJXda3kGPn91Lo5cma2WGC/dvM0BeZhHJo08ybP78bu8OI
9jhdvBImho+Uvf+C58fiFU1pYaga7t477hpS/5vvXHJPsKmDabG7icJQ7FY8
7EIb79fDLE7akVgd7LZMd2Gv/gboanMt5MAann+KpyBzK1cTnCpwEKwqw35P
TTwkzbQZGj0sk9twATtVkdNHK7AFDt7tF17lxd4vsn9lMaMVRlrVZu6owefU
+E1h7bVBCsx061DfdQa7irm807klKuRmd/q6exXnw4NBFxZb8XZoMLrcYhOF
vcVESfu7Qgf0KPxJ/VsKu91612C6SieUvZrmrl+Dc2lR0MvJ1rgLSvZJ3KBo
Y/9QZf5VS68bMnxyv0Sm4Ty0lbtS8pHUA3muxDpGJuH8vFtmPZUh2Qudnj62
WYrCeetxv0vRJ64XitWVa808wvlc8KlM0oGLBl3mZycGvTrw/ny+nKHhQYNl
q0vXKi7hnH8ivK54boAGFxls7tUWuBdqdRgRdpp0KPVEf4FtP+4RcuKgsmsK
HZZlnVUO4cK9E2UzuFX1Ox0GSGXEVnfh/vq04vJazJQBRcMXZpkPm5Gb+Nw8
cyWdAZtdeiZO2eEeTGvnuLD5AwOWz2x7Abbi3tSMtBIMVmTCqvxocuMw7t8Z
qfmld65MeGD7Ws4WPYjPddtHYVIRE1ZnbS63TKpBvkLS9nd7w4S6IjnHI/mr
kOcLHTf2Fybg7PilXhfSK5wDdw5LdugQUHt+0t669CXyd1kRjppOBCw0GukO
ES9BHrKFnVUzgoDW++qs5J4WIF/HeJ+V8JyAOrEul2rO5CDfKMRyx76egGmK
Bg1eqRl4/XsC2QSNgB9HVQ+avE3Gvf/EZmGpn4D8y17mW8j4vYukdfB9xwgB
91p6+IeaRiOXMYsTTHxNwKeLf5iw7wtFntBk8mRikoA//UwrqSe9kOfkfFHK
niIgeUW++3v0UeSVgZuzRX+55J0b5VnpbvB/j7RMJhtNENA+72VbjHkI8m0L
jCqVMQLGcDjmZw9HIhepbmq+MUjAfNH2IYpEHPKwW0NCCgwCeoqdfm7EfIy8
qvcPu/AmAl5Um7sZUPEUr//IeKUhj4AHWKXHaW7P8ToyEyvBUQQ0If4oLt1U
gFyFVTrq83UC2vo+sxgjFyP/KMJ5S8CAgH8JNU16Or9EvjVOmC1G7NdzUUuJ
12d7hbynojWZ9o4JXdhCbpt1VSL/6Zfw1qCECXXs/yqzDKpBntM0syvAmwk5
GvtYFtZB5KHbhr9dUGLCp6wuXA/665Hb1FL14K/5X4g6Fqoo0IhceCGKjZLN
gKT4TyRnnSbkN1NVi8PNGNCcdccD77Bm5P/MROeKsjJg02Y2raKyFuSGZFLP
8ww6NOEJsNq11Iq874CS0NgxOqSeixy8v4eKnPXWxj8PT9LgFSGl4z4abcj3
7mGXb/qVJ80Xnk1ZOrYjn6Ws+clw02DV3j7P9x4dyDlOXysqSu+F9d1rKhTv
TuTE8va4SqleyP3m1fFDYV3Iv15kr+y82ANDVhJsRIK6ka9F/5D583w37G0V
+fHibg/yJZ+fm6Ttu6DJzm9kTtVe5Oy1qt4qDp2wyKLNNO059rsa7cpGRh1Q
onLdwLEdNORftgV/v6neDq8cG6mfuoNdhiUcfuJpgwJHb5g9WMTurdUhxjtJ
gUbUuIOnLOjIdTxWP6Y1tsLWFcpAbwn28/Gjgu9CWqDv/UHpJR4Gcrcemumc
eTM03x5vL2WHnSI6kCoq3QQj1iv5KhVg9+Iiq9aMNcBjnnH0yO/Yp47NU3Zk
1UPXas1FhgYT+TlnOV2fpDromrx+wiQIe52HluOPuWrYSa4XZ4HYd1Xf8qU7
VsJ/T/PJxS5i36TqZeqcVQ6n98wNrEgSyNmm+Twvc7+E1y/v7KKQsFse5jfx
OVQMmWc39pM8sfMlHxzaRM+DEk8/u2kkYO94zuOrG5UNP3p9aSkuwj52vkiO
s/8J/INqWXC7Fft+QTtX+lIitE76RLtMYF+eO/q5sSgGqvDR3TuGsDOfWYnt
6H8A263ZVmfHsOvNbxwWdAiExT2aA+8nsEep5Ild73GEIwPh2e5T2P2OMHbm
ztmBrdG+1aG/eZvL9L90xi3QprtP6MTkb/uwYhqcuBoGYOZn24Bx7C4tVdIl
ujFgeDqOVXoYe1AgmzF1MRHsEinY4/zbfenEkPvlUtNAD63eU7n7t/X7eHiT
YjOB4jRjhF6D/c+OS56sei9A2IZPtsqZ2J0SWZzXogsBaahpU38o9uTI6Cyf
cyVA/9A32XWO2A3ZGaqGu8uAeVzRCPUo9uG3hipGFa/ApW2Rvtv5f1uHyrys
61cFhFrUUv4cxfPT51JRLilTC6YNjo7dzMAuwOlWkfIMggkXp5yKS9hDnZ1f
KUo1gBPKX/RTdmH31+XMUD7fCHodOJZ3jOH59/Ungi8XNIHvgoXx7hHYY2Ri
JPd+bAZcDqW6c4ex//R7cERSuBUoRBeSbN7gcxr3eGOxtBkFRBobJDXewV5D
Ygp7u1MBn6xOwIgU9jr+fWu299vAZs9v5REUnA+S+c/9hArbwbLIT/JlK+z7
76qIp1Z1AKVUdQ7DBZxL2ikRgduJThBEb/pW6Y09a5xfiTzZBdSNT2TUreHc
qwlN3tfQ1w1q0/Je5PJjJzFXbE7394DxIIP4u1w4P0/doKy/adMLrjZ4XXHa
jPM2nd06WL6rF3Cw7f9xhwfns4DO5KchQAM5yuc+kD/hnDcdeeSa9owG5JP0
T0lO417YvuSRSeKjg3hJqYj5ctwjsVz1WQ7X6eBMrfZB7wQK8uOznJNGbXRg
pOh8e5s37qOZHvvqVkkGCCaH7fNRw/11r8FL+aoHA3BEb9rTxo37zub8aY04
CgPcK7ri5VmPe1PlK9w/xM8Et39sG9ni0YC8upDQOWDGBNEZihblR3D/doul
mqTFMYGXs3T+EUot8keUAcPYLia4fuxGC+loNX4/ifPgjv3JBCGumx+2V1Yg
PxRLFjKTJ8DgoJz4nm9lyHmSlaVTLQgQO5nAPGxXivzxRc3Gh7cIoL0z76R/
ZBFyaw51t7VUArwMHAKrd3Px93eT4OAvJwBH3DQL2fAZ8jPF1lLu7QTIL0+5
yZeQjlxU2u+1wAABuPsnsiQC45Hnnjp2S3eUALdMKlz4y6OQr4u8S/vwmgCy
PPuP5jPu4Pcu8TX9lV+/h2e2jtmeZXog////xnEFiUt18np1//t/kUPtWQ==

      "]], Line3DBox[CompressedData["
1:eJxN2Gk8lN0bB3CKhFDG2uNpFYk2IiqOFFpIhKciIa08ioSpJJQtylrZUqnI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      "]], Line3DBox[CompressedData["
1:eJxN2Gk4VdsfB3BJhtxouKVBM5IometgRVH0lzHSYEoyJJXxmjlCyFyIlCFj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      "]], Line3DBox[CompressedData["
1:eJxN2Gk4lU0DB3A7EU8J2VKJpKJ4UkhNkRKS4qRFiCRrIookkVJKWRJlX7Ir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      "]], Line3DBox[CompressedData["
1:eJxN2HlUzOsfB/BpJwmRog2hciNJtEw9qQglNRX5kRDZijbtN+RW2tw2adNN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      "]], Line3DBox[CompressedData["
1:eJxN2Hc81d8fB/BCRUZCQiGkIjPKaByFZDXMVEhmSspKQ7KTkRKl7FmyIink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      "]], Line3DBox[CompressedData["
1:eJxN2Gk0VW0bB3CpKBVKhHrIVAoZmyi3IUM4UTIlKokkFOI1ZJZQFBqoECoZ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      "]], Line3DBox[CompressedData["
1:eJxN2nc4l9//B3DKCKHIKp82IhUqI3JE0iAZSZGQVZSZUckqq8zskcreuxAd
e2RkZGTvzTvZWb/bdf2+97n947oef7zv9zjndZ6v13WO6Joq6+8gIyPrIicj
24n9X9/a/iMBmr3GJ23ntSXJ/v/v4oJX2bYPHl4kl+EA4H9+7I84w7abSamv
brIr4n5viaZ/A3OS7KHNVOU7uKvTrYiuY37niB1PN/sD3DXOU3H8w7yLPm3m
telD3B+/POe4jDnVrfOvwpUNcc/0nBv6g/n85jX3KnZT3AceO/ZPY+44K2Ex
a2+OO33YGb1xzF0FpXttTS1xH+cjkfVi/vP01+Z3yja4P9NZ9CnDvC++8EEG
uwPuLcvuFd8wP/Gog43zjSPuSUEXnbMx/55jXNJn74S7ngmt3yfMn19Ov25g
6oK7irIwhS3m1G1DDabKbrjb5KxJMWB+XKjF0JzdB3fXe1njm5skcPTds47g
Xb64WzW/YyNhTk5vXrz0GnnWiTTaBszdfR60ZNv74T5nmHjFFfNbGXlNfKbv
cVdthSz9GyRwz1MxkFY5GPcvPF+ZxNax353uz4PXbFG4h2xep2XH3J5R0l/h
A/LewD7LpTUSuBrOyvmW6SPuTpr0kZmYK+T9mFQORf7o3Du5I5gntXrR9sV9
wt38WdzGyCr2eZUKFcZLotHvmH1tnmKZBFhnp3K+LMfj3qkhfS6URAJcJ7sa
yAQzcN8U04v42UUClMJ/zinQ5OF+uj2Wb7aYBN79uJHonwNx9w5OTzEJJwFJ
lXzVmc0y3A8+Vf/2wYwEWk5G+ve1V+G+NFmiUClGAoFGCTZt67W4a/3xCnuy
Ngt0b5cEG+9sxL0xZyitMH8WtMv4Fv4YaMLdniFPW+PxLOA3SqUoG2rBnbLV
2y2FfRYM1W0IR39vxf0fVyaff8kM0Ot+Q87xth33lfoInWsaMyDfLXLEzfY3
7netywajF6aBxkb25xzJLrTO3aSUPtlPA/K3ExxfjvXg3sOlV7CPfBrE1owz
LC334k4WtUDidp0C43vsioKU+3GPeNw947s6CeKTv9fQigzgLn7HLyHRdBIc
CuU4RrFrEHc/C3Ou/S0TYB+takPACHLv88LX1YQmgPBNw/z5/CHcr+iJrp0L
GgcbvWe/yrwYxl3KrHSP3ugYcG/sO2kgN4L72bg7sdISY8DGXTEslWwU9/+Y
Od9S+Y4Chdd5h/gykRs2TXPL/BwBV8Lv0CY/HsN9cubYTTLWERBlclqUgmkc
7TulTXdvuWHQkdllO12MPEbR7oeWzRCIfL+4NKE/gbuQw3kb64xB4DF0jDt2
FXmvRxGpsWUAkK8Pn1z1n0T7fW/qhhH5ANDwSjB3PDCFe9jeusdnzvQDgZ4p
Fupw5FfDhFhyvXoBw9OEK2cPTuPOLNX4PPVXN6DnfcOf44c88+vajuHZTuB2
3KL4KvUM7tcKpr2cj/4Gr5LeJdY9QW6msy/54t12MFLXla9bi3xOXr/zwMNW
cN7C5D097yzuFs+1ZU96tACVSM8ASjvkXie6aVWdm4BMHYOSbRVyp09xPbfJ
foLK2EUFjt0k3L/eCGN0EasFnUKLej6XkTfTqlmvcVSBXfJy1XZ2yMNOuRuX
XigDF8NFNKijkee9r2Cka/0OVNTuWmiVI69PTuTOcv0CdlWr6Ij1I7/XoOUc
Y/gFkHn0T/kSPOA/ao+gq1/A+C3jjTGCv67fFWpH+wWI1z/LDRhA/sIjt1Hc
Oxd4ll6wGx9EbnxigDIzMAf8bTe5aTeC/Erb8RKdmCygnraXR2oSOc+30yHT
TqlANENaVfcvcslXZ8ifSaaCepew+x8Jvrq7fGz9Xwr4KmDT0UNwCl5NITqr
FCDZWHD0zjzyJg6h3iMGyUCiQoFGZgF51XRNsOj1REAdC/8jW0Ku0lJYz8wU
C5LdJO8zriIvY9US8G6IAU91bwaKE5x//KzwrrcxgPxjhaQhwa2qBAXWdsaA
U+JjI4UE39FssNm9+Bl0XBp8rfMPuWX3RTq/3x+BtNh+St815NnsCldcL4UC
Yw9eWucN5HWy5a/7C0NAqWH6fDjB9X6Nql8QDQFvurJVcgnuVMxaO306GCg3
CduMEjxy/BWLAmcg8A7lkJfeRM4goXV+Y8kXMMpv8fcT3Gep+beShS9IeyL1
ao7gKjP8pvJdPiDF6J/sji3kWtEjihHVXuB9XbvoUYJf7C0+JxztAVZWBVI0
CL5vitNCTd0JmMzy1uQQ/J2F09rKTUewPNTlVERwWc1Qe+G3r8Ci7M7MCoI7
uLXQfNd/DibLMy1+EXwFXjtCrmILLD6LUnYRHJgINcpIWYOAJ+cYBghuIkXG
Xr3fAggYerpMEfzHv6+lNLvMwBejYRKJ4CKfGJSCJo3BzEZW8zzBwzjSY9gr
DMFR1qWTywSPXD/mHxb1EJA+p66tEjyhxUFNdEELSIX8vrROcKeVpJ3iiWrg
5rTd5gbBqdV7PjwvlAefgr0Ftwgu1lj8bENeFDAFsnURPdPl7rPFoiOwemDf
GtG1X7/UtaOUhWL2rn5EJ+VHuxbUK0P+RyYfNgl+WUmYLo9dA+YmVh4hvh8f
/QSvzkwdKCobeGCN4KeX4mrU2/Xho8sdb1cIvup/ilX16FN4KuLbzzmC1591
dxo/Zg6z92pGzxD82x6FtDhhS1h33HxmnOCW8yepj2raQAqb1eEegnt4FhQy
xzvAtfdvdEoIzuFdenlXrSP8EX/aIZ/ggs+1TFxLneDUxXNcmQQ3HCn/45Tp
An+Y3qaNIvh/06yPnvm4QfXSedFnBDet4Ju9fsMHGn8q56EjeHx3xKsbOsHw
8LKwQg9hP5oNdV3e2RMMF9IarCsJLqE2mFmgHgJHhX6cSCd4sRyd3YlbodBs
Y4bLgeANR4U9tiTDYYf/39ccBHfLvcAZxhkFDU7ItomsI4/Laj1bYBwNXdZZ
a/YT6hLXuAVZTVo0PFn9uXqOUMeosxyt2+eioUOKt3g1wSVNlO/N28RA+wiP
F5YEF0zz6ud+HQvd0g7rla0gD76oO/IiIh5qnnLsuLqMvPvgr+LWumTItDSm
Mkmo5yFMo8coWLPgQpNOndsU4fu8IRu0PJgPpcSYw3a0Ib/P5kbdzFMCH/BE
fGD9Qjh33B53/JatgDtqnMiYvQnr37c1mJq9BrJCl/O7dJDbve4LbDhfDxO5
OYbaeAh1oyVnScSlERZ1fPoWPYnOfWi3qLVPqxmy3CVf0o1HrrpWQqal+Qtm
7TCz11FDbnV+tvL92TaYTO3YLr8TeYZeUKksfwdk6Cv7K/gJ5ZPcD1ybZ1g6
4e782S9NoshPpfkrjbV1web2rEWNRpSLdB9cTT/9tQc20SRLW95BPrDil8tt
1gdflIkVvOlDuQtEnT7t/a4fshY6V1g+RO51Qd698ukA/FFpvpnainJdaTQN
S+DFQdizxOTWeB15GikS9rMPwfMnwjWkU1E+HPFSFuntH4I1YcL/UpmRr2k8
qJb+OAzFo5smztqg/LkYZK/06PEIjH8VmUpThXIsTX4dQ+2RUZh1T2spnxN5
GRuTlPvgKJSPdFD+ex/lYde/92lfh45BNSvZox4RKFez7nv51kZyHEoLnrBq
HUQ5/I/DZnP35DjUm5189ocZ+dlpEgh9MwFvCg+/K1FAeV5u6yeXMtskdIlc
5PvrivL/Yr7lkF/aJNwxKBhslIT6iJ5hShru81OQ6pYSg3QH6juO51BY5cIp
GHIxz/VvXx8695987jknNQ0/XRneeZ4R9S8POE0yzmdOQ89YvzULtW50Ln9n
uXby+AykNv7hvGLWiTutD1uKpPsMpD0fzSca24G7v3l0u93QDMzPt/aq6W7D
XfTHFlua5CycG7od+G7sF+67Uw+3P3g3C2+liHBWMqD+7qLVO0rLjlno1zKU
cZIG9YMnK1rNSthIcJH9VPwJmQaUqw8bkL++SYLeR+oZ1x1+4E6vWbY3zYkE
Yc/ggpReJe6TMn1dcYkkyPvaQuK4RynuEVQvaC3qSHB18x3TSEoh7gYZ6x/n
Rkjwajo3X59BDu7yd4G91CgJZqWwvlmQQ+6bGTvgjXnlkeb3NLzIv7V3SPON
kWD18ZcNQlPZ6P0I5htojJOgS7RShZMp8sMfHp2NnyTBmKq0g0y2WbgPwiO/
h2exz2v/I3XLHfX1Y9Z2h1cWSZCNceKeomoi7i2hwmynlkhQMAK+OrIX+dvT
0FEHcyGX9rPz9Qm4v997JqUa8+LAp2qBcsh3UJjr+S6TYGRBRuSvC2jO8LgG
3qVcJcGoh9KiUkdicVeU26/8Zo0EM5x6v7XMEOYbYjkfEzDf5NwjF5aE3E2P
XLcW80yqj+7ahsiZd2RLMK6TYPbbDYOOEjRX+XLiobIv5rNORn5kCZG4Xw22
37TZIEGn4R8TolahqP5E3lTr2SRB/dvvx+7uRvMf599fm2cwL7z12araG82L
DmzMtGxgHj9KKhHeizyIqliPc4sElxsTr3Hxe+Pe/3sBqmC+wqHk5AM80frf
+K6rjXmNwn80jy57oLqhqOppgjnlwocz0tfccd9V0WfogvlmS6/hvLIr7iy3
nmkmY67fwkpSMnBGdUYggAvLu1DZSbXthDGas9VuvLQsxNxf3PH5limayz3r
SY6sxZzG9OVe9fmXuPflztk1Y+52t2hh58Zz3B0pybs6MI/nYzZLp7LDfWfS
i5PD2+9TS1iBcv8zdE61jO6fwLxR5PD39GNorljAecAHy2EQdN8ZuXsKzSFp
6nspFzFX3d0ycc/pMe7n2J7uwnIevLi7uCM1wADlPblKn3/brx9SYEaWoIvy
8O2xcCzvQke9706x1vdRPRFt4sdyJ0zMDsi4Fnkb9+YJF2ksp8KCK7rsXW43
UM402Ore9ufDQrWXk0Rw30ySWd/2E/IrrdnTB+H/vObrrY/bLnnovLyirwzu
817cNdvPndE+OeHySQn3rwIV1ljehTl/1cf7nt/F/VQsfyKWd+HWsdzS/5K0
cZf9q6mN5XuYxM2sVBukh3vdYbWIJczfU/D52roY4W4kzn4f6x8gj3SPGZeZ
Ce7fjJZNsD4EJhipzHw/boF7jPWxylbM61xUfzFv2uGeWqt/4ifm4pml3EYL
L3DPjTFnrMb8iqAXfeGkPe5TSqPGeZi3821JZE454j5KK6f1HnP5cR171Y7X
uD8Pp+YVwnyo59uWZ+Y73OX3ix45vv19tp32hfFe6Lls5NEsmKtoDWfNR3rj
7mlwX2YZ26fu+k9unZL1xb04UnY5B/P9ofH3/QT9cefT3M2/H3OusCADedog
3JWk3tW+xerJ76n5IqdvkbhngsIVuhUSLCpdnpP2jsP9hY5qVDdWD6M3Bw6u
NyK37U0sSMG8Od5wPIc5Hvc/p30Vb2AeEFsgzxWKvMNX568TVm8vG+VeII9J
wH1FrZOzdYEE9+XlpMbkJ+E+CfLomOdIMEhhYcNvOA29vrmX1wx2vuyWSHrw
MCIbrcP0lBD6Jmx/VWb85NX6hntkrV1BUTq2Thh2yEX1leCeVFx1ysqVBGVZ
z1zPW67A/UZ5vbKyOrYf8xw5JspqcM/51CBEcRD7XW4pTTt21aN90RtEPtiH
nftxXyJWmhtx96PekywaMAud3EY0+bubcX+7VW4dcGkWprh/Kyps+4V7J7Pg
AtvMDKxVY3aSTGrDPWFwysLNYwYe9Ot4yBjVgdZb6+LgKa4ZeD1cjPq1SSfu
Lo0xj5e/TUOm3n6NPL5u3Ed2lwlcl56GTvxNpVrUvbjTCgfyV9ZMwVH9frq6
wj703D5HumwwBa0tTYxPVvXj/jR08FthxiTU7vl76Gj8AO4blX4J48cnYZhU
VkCx5SDu+4XZqTRcJ6DLm/+USuSHcE9RnK/enBqHQyujru5Mw7jLXf2q3XNz
HN7eG+pV9BN526M9DlQfxqC/p6hZePAIqhuLD/dcnB+FkFouuV5+FPfs6Vf8
q2dH4YuarhcydGO4qy9VNda+GIHCF17k3v6K3HWy7BJ/6TBcKgp3tro3jntV
cGqR1vwQ3D2xenaFYgJ34xJ1IeNTQ5Al/EbgbCRyyXtWClBpEA4GBGRVSUzi
Tjk3Dfc9G4C8u85eD2hCHmhYFbj6uR/uOeYv13V7CtWBzmvPXz3rg8+ODQ/s
7EWueSbk1M7CHrhWeJzyjNo07u7xxquAohvKOd+e1qxA7m9TXtl4qBNWM5nv
sBCewZ0+s6/j9u0OSJLIDK8JQa4w8Kp/8H4bPEVPfdt8DbndokaY76NfkGEH
9U9e1Vncy8lNO1f9muEXu64O3yjkvLnk5J+9GuFo30s5MIF84Vl7EItiPUzJ
36DS4Cbhvip0xrqVtwbWaL1iOq6NXOuYMP/WngrY1spw+bov8o+MExryYiXw
1vg4lUMucv+oF4o++wpgtd7pTO1W5Jq636MmL2ZBRefBVf4p5GoH+hyPHcqC
T7Qk91sS/Hhc2EFNsiz4RXi4LI/g6XtT/OpKM6FAE0229DTy+qAi3SS5TChc
/pFMaQb5L4GaffeUMiDF1jH1OyTkfVmXcjhqUuBBSfq/tPPIjfKm1F++TYEB
+36JSRP8TKfJ+16FFPigu4rWjuAJjB71n5qToQjT2asjBN81L2Z6rDsJOors
EMxbQN47Sy24j5QA+dg68+WWkLM+/fM3c18sjJBzjKZfRU7mJfCnoSkGXj9z
wUaM4DurVPynvGOgLzk11CP4LvO2mmM0MTDZoi0pn+AhF5u4fDc+w3sGovOa
/5CvO1f63R39CM8Yml7xWEOeouVu/dQ0FMpQa/213UAuMqLMHjQRAnM+nZN7
T3CPw2wKRQ9D4D1Ouj2pBG/Jz7alVQ+GN+dyN3oJfotBkf2jVCDcd3f5qsQm
ctrHH+0K9vpBs4XmD90Ehx4R97vf+sKoCZuzswRXED2zukHpCysaWSW2CK6i
4lDsuOAFBRQuJR7aQu5JJq35usID0j+9NH+X4GvhzO9tDJ0gR3y+bA7Bb91x
jnv/xBFKj+q4FRL80X9xS1pjr2DKUybFcoKr5q+Q6raeQ7vRt2bNBC8ce3tz
5247uBml+qOD4AErcQpi7DaQXoQrqZfgXr9Eb8QIWEKquJ61cYKrxPip/5Yw
h+FqFU9mCH4emlzl9H8KGYJy9OcIXnZQwPhN0mModTt5aIHg1C8UT8yWGsB+
18SxZYKL/07xVuvShWaH0s3/EbzaQfWe/dh9KMNS6LROcCEvFS4FdjX4UruR
bZPgPGzComL+8pBhbersFsHFzhWap4+JweJ+hjaiN32p+K6oygK4GcU2iX54
Uf/D1FVpkPjmcQrRD0V+dIibvwUcZKP7ic99KGBOaitVB1lKQ2EbBL+pZOa4
85o2OJDO17VGcMGsimohdiNA+/znzBLBoySiJmu2jIH6P/7ieYLLrMNP2mOm
wGTOj/0PwUWMeK5f6bIET+0tNMYIrhS6YPU06QVQpfTlbSR4feYnT5EP9sCH
5yR/DcG77jw+Q+bvADp1G3JKCP7jhCxFdaETuBpySj2T4LspbgeoxrwBxV7K
Y54E1x6rvXnf0gu8ZLrLw0PwdVGqAIkn3iCTxc2TneBMO+T0Dxj6AJEWaElD
8JUT9g3Gqb6AWV+5Y4Kwv6jsXq0f/+QP6j/umIwheIbpHj8P9yDwVaX2DAXB
6yXdXkwyBgMunfC904Q6MM9bvHk9OBi4tJ3VayF4Hy9ZJW1cCLDMhRofCf5C
3IzlTVkYOL6sGHCW6DrNsbqbH4Dx3eKTYB25LZO/fUd+NOD0al6eJNRDAeuE
f7SnU4DaZRf+QELdFrTx8XpklgIeVxhp3SN4oU9xbFVWCojwN6c6RPBXAW46
ziKpYIh5Li/uL3Le03fG5qTSwN4r5fHpc4S6MV3A3s2bAfIS4lPjCefRb+XV
RROJLMB9V1JXgHDevbTnzArV/gKa7vHmxvQT1oNVexk/63fQkM19/0IVclnu
pWf/nS8D9qoGIpLRyLdKaNg9mavANdFBgwo75FdPyf9k//0DXGbZai2/Rqgn
aQnK2RMNoFdAWMeZnpAHksxL36k1ATtl3tLEepQrEhh9U3ZrtgC75YiPIo7I
9eHRrQ7lVgBDSSqzJ5HP1leSRR5tB9J6w2y72lDOMbLgpHRm/g1YZD/4bZkh
X5W2Pcze0wlKDo91GtIhZ1entUqq6AZ0c8v2LwNR7kr3iIp/GdkLVstKbMc5
kGv60FOy8/SD53JHh8uiUK5zuCIWrUk/ABbZrqhtMCOn4X9Yzd4/AHizvq19
9US5UZ6RrsUpaRDoPb+xf2YD5c9um6xkQachYEvjZfJAC3mw5pBy0eVhUG9t
zihehvLtQ8X9l8SZR0CwlwSz+T7kUUIs5OIdI6CUM9tDQRfl5BnZmoVUz1Ew
lckab/cF5W0453lf/MoYGGX+zXRkCeXzbncnfTfSGCi6Yip58CJyA08Jzimf
cSCYdnCi3hnl/MnLLE/fiUwAT4qx+mvZqF/obc0wqqqfAHPfnmx2TKP+ws7g
OD98NAkiLUJdUvcgN/ThKChbmATv/uPx9ZZEfcr5XO2APOspEF6xzu9ogvqa
P2kusVbrU2Dv0tHDybtQH/RjvbJSw2IasBb7231S7MG9UnbAy3pyGuxbPRzR
8KgL7Ucb03YH9Rkg75z6+f6b37i7DSu1keXNgKvWBdmS39pRfo47l57ENgvy
Voqmzpe34r6sLd5r92gWmFI6RHJNtOA+ePREt1/WLJA5oPO+nAn1lfocAdZt
q7PgEXWRZS4j6kO5dNP4vp0jgYRKFperanW4Xxicnz9nQgKPx2jbL52rRvto
3HGZO4wERPoWnpIdKsd9sctq7V4RCSyPKf2UvlGMzmv7qvs8XSRgRF9mWCeV
h/LkYx9mJhIJRIO1lf2WGbg/cGf3xXIwYCwUyJFUQl5aTmEfgfmxJQpu3TPI
HUrJE47/wd6/Y+uDW+3paJ14mEifniOBDNv4Tp1qNB+4cWunO/c8CVA1QB7j
pBRU522SbiYvkQAnq6vVoado/rCVYdrai3l26UC/Gh/yg6es6/Yuk8DkGb9+
r1E0x1DXlKGzxvxNdNqHf1rILY5X1oqskAB/rrjIT0U0J9FnyrrzcZUE5NJ0
h/UFY9D+yqsPpFwngf1L+bTR41G4T0j/OsGFOff6zW7aR8jrrodJXMZctHha
wGLiA1qfo7qjjpjbMgkag0k0z7l65bf0AuZUNx9q/JoKJ+yjlYNlGyRA8yR+
dGI2GO2XkJKiTszjJUrarpsij6KT6ZnDfHLgw6NkEpojHQh6+uHgJvbcpAAH
4z+BuJMnaPFaYD5rOx89Nvce94vRmjrkWyRg4BbCEj3hg/uuWA+6vZgLtCvy
aUyiedfy68fChzHXn6AuZ5pC87FO0hWGi5jzjaoHOU6/xT1fVa/SDPO1nACu
OyQ33Gs5+axfbt/btDJqovvjinss2+c4N8xDrv47UvznDe6Ri/EeEZifXBIu
4fnrgnLsHyr1YsxrFdQbRicccDfpu0tfjfkV+z23bkyhOeGRX/dlfmL+Wqj4
Xvo0miuOadkKdmPORlbOZ02ywd29xHti+16FG/XyVsefZyi/ld0+iuU5kG7d
Zy/+1xL3SyqLLNv3YGW7OXm2JkzR5y2qb9y+V1HZb2hdOm2McqOj2u7texU0
AaFqb0iGuDd4+VRv36toWfBtl/v7EHfPLlvK7XsVFVT3mPZNPsDdR5upbPse
g65z6FIw6Q7ucyya5Nv3ITpLJZaFJxVxL1q6XbztIolhFC8mAcqB/3/fuLL8
HJ/tvPb3//n/AdVdrBc=
      "]], Line3DBox[CompressedData["
1:eJxN2Gk0VW0bB3CpKBVKhHrIVAoZmyi3IUM4UTIlKokkFOI1ZJZQFBqoECoZ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      "]], Line3DBox[CompressedData["
1:eJxN2Hc81d8fB/BCRUZCQiGkIjPKaByFZDXMVEhmSspKQ7KTkRKl7FmyIink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      "]], Line3DBox[CompressedData["
1:eJxN2Hk01OsfB/AhVJJKiWKoVOgqScoyeWylIjEo/QpFaaNs2W+Lrp1uItly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      "]], Line3DBox[CompressedData["
1:eJxN2Gk4lU0DB3A7EU8J2VKJpKJ4UkhNkRKS5KRFKZKsiSiSREopZUlk37JX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      "]], Line3DBox[CompressedData["
1:eJxN2Gk4VdsfB3BJhtxouKVBM5IometgRVH0lzHSYEoyJJXxmjlCyFyIlCFj
XREnMqxkJtMxT8cZNhkqQ1SG8vfmrtWL/ebz7Gfvs5+1ft/vPnu/hYPeDXY2
NraB1WPt6rFjm6Le2ygCeEruHqY0XlQ+odOa6kAeg20znF93rfryWqfa4WUy
+M95eMcaNSIIEGUtbnt2OQa5QJ3a/ZlQAjhUGY1OtiYj778kdF08iABG9Y+e
UtrSkX/OuKHh5E6AN2sapINH/0XeOVm9KG5DAA8XnwCSeSHyrIt2qpt0CXA/
WJ63urAEeUjYsZSHsgTQF/3N9iujArn6mZan/9tMgPLrt/9y+1WJfEaTon5i
ggUYMhLfBR9XI592totagSxwWd86dPuvWuRV/iPnFsJYQD3d8X1PVAPy0rGZ
v9wuscBDvg1tBnNNyNPnHKfd97GA/i2nWI0zLcjn2dbkldKYIBZys+uatiHf
d6S+630SE2THtQiqkNuRG7rYaqnoMAH3yr1I9lIqcvqmqHNSbEwwX8Go+f2p
A7nlBT1CIp8BpLbZf2ItdSJPDh2dbzVkAOe6n6GP9ncj/yf3udOR33SwTvad
jeLRHuSOwR49wQl08MHh1o4t53uR235euSYlRQeS3MWDAlf7kPtpBh19lTgM
GlTqbyQZ9CP/MlXSlVFCAzFNVdHrrwwgbze4kDZFGwLTIWsASWMQuZT73c/k
NUNgQcBYZo/4EPKMOfrBQp5BcHrv5br4nTS8Xl5OVx/sGgD3Dysbls1g1wx+
vubg5n5wXImIiaMOI9+c6KHrw9kHfIzjAo316ci/F81ZN3D2AnHJ2JPaBdib
GxzW7p/sBmHuGjNDfzOQP5PaNZg62AUC1JaPPb+FXXJ4i55ZSydQu6rDm1CB
va48T68grQP4DbgtjG9mIlcZktCQDaYCNcNRb9Fr2NXINurftNtBaaxkbVkW
9ssaJ76vE20D8gos7tQJ7JKqtsbHOVpAp8rxpQYRFvItWQR3jncTcFad0lix
wM4dL8RhvrsBiH0PvGP1GPu7B51VJv61oO9BtmlHPfbUp1INZl+qgH/TVd/p
OezHD3i4T/hUgiZhDRvSDgLP+/Ibd1pBOfAfiR5UJmE3rFYaiCkuBly6PyLm
LmIf4QwfCWt/Awam/2qzsMM+tHTU9f78S/BulstL0wv77Iyqj8qPTHBEBJqV
kbGfE+ZIjfqRCgpG79V+D8Y+3lKw4rUtEVCLTPVUHmKX+faLPE+KBMOXH4Ld
kdivsCKNr//PBSh/dVkIjMI+ICNTFn7JCWaVrd8c8Yc/srFZCuULh2pW3ONG
f1xnPEE6LOhHPEzLJct/+eO+ctQva5/SUmDcGWU4+wB7GtfhSdKpTDilx7sc
HoA9SE/TszTtNZw+UVNe4oKdrdnu51o+Crz/0YGQscB+c3xl4wbjUuh7Y8XO
8xz2HfGy1zX2v4fbjY5FBx/BLvQt+Ex67QdYTArRruHEftxKUdf6ZA18x+mR
dXsYr7v7A+hHT6qDygvHvkoXYt9oPDvEEm+EeqSdQgd8sWu5T5BzQj7CQ5X0
iXea2I1W1lt4bWmFPxu4zXJ5sTf2+N5aW90GU1kp0zH1eJ9P7cnt2TrSDh9z
DOTevo89NpvLuXlHB9S9y8iiyGMP+cDjcUq2E041K21/PoHn8abnyB4b9S54
f93hT8NPsI9cF2sh2XTDWS5zSgXAPjjt6LDDsQf+41cy5zGK84H73YrUuoe9
8E0ZzJD1w94Htt7YG94H91wpjPLag/1lwSsx8YB+yPdoyKPdEefS+QFn0w3B
A/CYzm3tS1E4x3qbf2Y0Ow5Cjmdx8TcoOA8ZD+64PrIbgm3H0yMPVOL83P+7
8F+qIQ2S1TeledXjvL2U3azDEBuGzcC8ubIP5zMpLnXpFAcdPhWh6ta14jw/
dXHxytubdFjYTqldacP5H0w762lRQ4esC2unbNpxX8wU7uRLFGFAQ5/XbiNF
uF/OXOtzFfRiQFevcOvSzC7kS6+v8Ul0M6DY6Q8zgl64p9zf9LXXiDLh2AhN
2UIP95rU7/wx67tMaKXs+GWTPO5BrYJQllYpE867tOazr+A+rfJ9RElZYkLy
npfMhX9bkYv+HbFPXY4FEzYfrpWybEau8Ixn++gdFlzX5ZXVPtyI3NyMbbE7
lQWLPaIZXL71yCnq0Ly8gwUPh/iPJ43WIKdZtOTcXmbBXzuDttU5ViE/xOf1
PGUvATnX8J7L5sbvIdpk+9GAUwTsySf/T7mhDDk/2Ggjd5mACwM1ZMXOt3i+
Dkl2x9whoN6WtfTU2wV4vS65vP7hS0DXlOhly5M5yH9Rx52KAwlYkKr4pFvl
BXKysWXJmVACanGklXKfe47c12kFdIcTkNLTOe1c9Qi53aFEz+VIAira73nx
ZuI+nhe3KWFKFAEllZKOOjecx+eHRrBLr/rFmdKKZ3f94X+e8ELwh3cEAcMc
9fkfCcYg39Nn9vhJGAHXcEhdBStJyMvHLqvuDSZgK1ft16CBNORnQ+UklP0J
2OGeJrFNLhv5ta8660McV5/LVCD+n7h85Aen3tzeZ0LAv9gqnS8sUpDP2P50
enyagF2c/OveKpUhb6u1OzklTMCie5C9JuI9cudiq1EnNgJmO40AkeNVyOXT
K+MDellw9I6l09TzGuRMx4kTzEwW3HRTf2hRuR77MXi26h4LNpo3Vcg0NiIf
Hpkpd1Jkwb2372UqHGxGvu3FIq/zMhPeIG2h8mW2Iv/tprl7bnX/O7qCROHl
NuRaNcE/9dyY8GCQ1+HvO6nIczeapAqKrXqeS8YlrQ7kx1uEjV4MMqC00amu
AKtO5PGLjekEmQHtnNZ4VkV1Yb8w9WblEAPuU/OPyMvvRm6xr89+Twsd3gpl
CZcX9yCX8JOekLamQ4dJaxNKay/yrXKl15ZW80dClDfhZ3MfcvtNW2fnwTB8
erOdR66yH/nQmC81+SoN7o+dTDn0YQB52Wt6Cr/fEDydz/lse+4g8hdRujTx
0EGopJPtPPtsCLnvBUuqVfIArMgyNRT1oiH3FO/sr87rhy+nGbaDF4aR76qo
eTj2og9+6aItcG+lIxfbeM/APrMXes1a+lxxxW4vnDaw7mEPtBDwC3IZwn7C
vpHN6G43FAmrWtilxEA+u5gcTrreBcWWYvMORGNvLz6fk6fQCeUSeP7OGscu
5OenOibYAbPl55VG5ZjIRaibK0jsVOjmayp9zRu7yaK+dm9jG/TcHqmzoQ77
09QjLmfPtcJd57PopzlZeB+K/lPEV/kRctrnzLxSw06zKtEeUGqEw0e3KRd5
Yxekm/+OG66DiiHbVsTzsBfJ3qI0na2B9+JKgx/QsPvr5EFW9wdoG08Sal9H
ILeV4dhfKv4e/ja692VeDPvuHXdNdm4thWVm+hvGNbAz56r4PIUosGlK37HD
HDtIzTiYVPUa6l6dytF1wS6/wrwZ5JgJd6SNPzEIwH5IXkS22jcVEk5M0esP
sPflW9wf2JEI+dXs5U49xB54ifTK1TgSTgH1mwKR2GHmpbBuUxc4SVpMX31v
RF589EqTT/1d0GBmFh3/h08uefqxfoSBrbGUT3f+uE6twqe+uSsJ4FzxonvL
H/eNrz7wdfvuVLB3zRB72R+/84lUDUy6mwn+dshsMvzjufgrLb6FVr8ELodF
3SO8sOtbmLYPGr4BgUr/nj1hj92FI+eUXn4xeOVr2SFgiL1SM0ngcU45ICfx
ujmQsAsF+3DPqFQClQdfg2IFsS+ZHdjXSasCY0R8fO88XvfJXsr8Ns1awHd2
6J+Wj9gLjRj9pqx64Ny5yZMzETtZPNnkg00TKAEiQ72W2E1CEtl/lDeDgWq1
thQJ7JRIYx3zPW1Aet8X29ef8T63O83T6n+mHcjwqi9Z5GLviCfxRHhTQZS9
2ORjU+y3joyPdb/qAFVMxVeHNmL/5CFisVDVCVRfDSanV+J59KJxnmH73AUM
Yip5Pt7E/nEDf87EYjcwciAnRfBjL5R3NNm/0gMCK8rz+YtwPvyMzLHYt6UP
1EjuTfC9gF1LvLyLzNcP7Lo/UMO7cC5Ja2dQFwQGgCz5snf/HM4xueteKaSt
g+DWKcO3PfzYxxdcfNMXB4GG94GlGSmch+thh3Hl5BAI0ztDziXh/BRnhSpY
1dCAX5s+d4cWzlsJV83ouJhhoOQ736JlgvPZlhk/FSpLB1vLBGoD9XGea3Hv
JgQf00FvUaW/rQHO/45Ep+/XFumAqh3NYXoC90WbmUJviT4DKMieL1Q+gvsl
4sDzDNVMBqixlc+T3o376HBnxpETvxhAKpbKUTaLey3XXJLMrsEET/aa5yj1
4B4UruM4LPeYCSgKOk/nk9vxfHXd45oZZIKEkpPro27iPiXZWYQY7GSB5BoP
yWP+Lci1+eO1qUYswDV57Zv3UhPyZDsnA+swFqDPF62vjm/A/cKdJGZZygIR
VkoMPsU65O6FqtHS4ywgZKwlPf6iGveXa+BrPn4CZMS8f/5d7gPOvZmfdm+l
COAuFy1duRsizz7NNQp0CBDA84ybjVaCfG/MUBHdigAJqqIqbrOFyC8ICQlx
uxPgnPz4M7pEHvJMBYugrkACUNPfGeQLv0DeU3109G0oAVx8WuWFrZ8hl5Ew
LxCNIECnGqOrwD8W+RXv9Mwtq/+Tz9+NtzUiByD/73uj1NnwIUrjxYr//P+8
rlcZ
      "]], Line3DBox[CompressedData["
1:eJxN2Gk8lN0bB3CKhFDG2uNpFYk2IiqOFFpIhKciIa08ioSpJJQtylrZUqnI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      "]], Line3DBox[CompressedData["
1:eJxN2Gk4VVscBnBJ4lLGEkrclBIhJJIlZO64NxIqIlGSkjkkQykiMmc4Cklm
IvMyn2N2hm0eMsStK0lSibp9uWv1YX/5PedZz9lr/9f77nMk7K6dvMjKwsIy
/Ota/+ta5Apzfz5FAMGytst18noa22VPai3zzcP8RFOh9b9cX102SPu4B/jf
xSc7OQUmCHDwWpH1E507yKtDOHuDRwmg0RNvKHAxCvnCG32T6wMEKJi82lSy
Jx75XTvHo7PtBLB413FA1DUdees21s0hFQSIX7e0lbcoC3lR6EkwkkaAgJWy
TpEzuciXjrx8FBtIgOrTVvFGJkXIS7ZQrXjPEqA+Wi2Lc3cpcuUNjQlOCgSY
91A0N/+nDHm0n5akLxsBzj9+TW3zrMCfP894cKCbCTRCzWoXlKuRNzmuFpLi
mWCufYSRDGuRu92/XVZpxQSBlRf4KSL1yNP2tGo1CjLBso+mPsm5AXk2q8LF
8G4GyPGOo648bUTuvPydY5c/A3zRsVerm29CLuS3oXhBigG62/t8uORakJ+R
Pbx1iEEHeU62V7PMWpGvn8nyLnSjg0fWsn/LR1CQ5xVmplhtoQNSmQ7Lllwq
cvPvtilaL2hga8SRG4pDbciTlIYsDNVpINX9TEvtajvyehqjyp/WC8g56WbK
GzqRHxBxoala9gJlLbqo7/Yu5Nwrd7hL23pAjuas+GfhbuSP2Nlv8411g8W6
1gpHrh7kij/eGVr1dQGXIHuFRLZe5CpOfuEzA50g/AS3AZ8XdoUmgS6vug4g
Jt7GVT6P/XNQ8/tnT9pBnIWxwiMbGnKPVVl6aPSv+9zXzpnejN3MYZQ39wIV
HFBpYAmSpSPnvSc72KtNATf7+wVnwrCHPVXjPyTZCpYjBEslJrG7N5a+PTvT
DJJqeGaHjzCQz35tMS6oaAIu9/86PBaGPTs2n8vOuRH8zDqlZjyMXYz/W4en
QgPQCyoXc5Zg4rmajRiz+AhBeEe8w3Vb7AJf1fouaNWCGqkrM0FPsFsa3HvI
UlUF/tl1KebQAHbVq3EFckuvwHV1lpXxTQTy1WtH3rcolYHdrzXHaerYr37g
PHGVXALm/z6+yfUC9n9H1093PS4Eyiylphmh2AtyA/WrSS9Aa+a+YZ5M7Hmz
sf17hbJAZ/AzxbEq7H0vM1dP0NKA6xcZ7rku7H+nyNWxBCaBpxea6RQm9mOP
W4G8XCyokynYYTGEvSdUj3SP8QAM/Ssn/XYM+6fy9Ml83UDQxv6Qd2kCe0q5
iHVtlSPgTXdf5zGF/S2HEZX57CKMHeEi+//mASx7Us+tBsCr7xj+ByexH0q0
0aiMDoM7mm8KpY9jf1/rm6vIFgOrioCW/fBv++bUqKRelwjJ3mqL2QR20Yq2
zu1lT6CgDe+1D63YjSXq1M95Z8MzR396CRdjH40UnT0rkg+T8z5cqYrDft6N
O9xIuhiO6nO/eeeJXda3kGPn91Lo5cma2WGC/dvM0BeZhHJo08ybP78bu8OI
9jhdvBImho+Uvf+C58fiFU1pYaga7t477hpS/5vvXHJPsKmDabG7icJQ7FY8
7EIb79fDLE7akVgd7LZMd2Gv/gboanMt5MAann+KpyBzK1cTnCpwEKwqw35P
TTwkzbQZGj0sk9twATtVkdNHK7AFDt7tF17lxd4vsn9lMaMVRlrVZu6owefU
+E1h7bVBCsx061DfdQa7irm807klKuRmd/q6exXnw4NBFxZb8XZoMLrcYhOF
vcVESfu7Qgf0KPxJ/VsKu91612C6SieUvZrmrl+Dc2lR0MvJ1rgLSvZJ3KBo
Y/9QZf5VS68bMnxyv0Sm4Ty0lbtS8pHUA3muxDpGJuH8vFtmPZUh2Qudnj62
WYrCeetxv0vRJ64XitWVa808wvlc8KlM0oGLBl3mZycGvTrw/ny+nKHhQYNl
q0vXKi7hnH8ivK54boAGFxls7tUWuBdqdRgRdpp0KPVEf4FtP+4RcuKgsmsK
HZZlnVUO4cK9E2UzuFX1Ox0GSGXEVnfh/vq04vJazJQBRcMXZpkPm5Gb+Nw8
cyWdAZtdeiZO2eEeTGvnuLD5AwOWz2x7Abbi3tSMtBIMVmTCqvxocuMw7t8Z
qfmld65MeGD7Ws4WPYjPddtHYVIRE1ZnbS63TKpBvkLS9nd7w4S6IjnHI/mr
kOcLHTf2Fybg7PilXhfSK5wDdw5LdugQUHt+0t669CXyd1kRjppOBCw0GukO
ES9BHrKFnVUzgoDW++qs5J4WIF/HeJ+V8JyAOrEul2rO5CDfKMRyx76egGmK
Bg1eqRl4/XsC2QSNgB9HVQ+avE3Gvf/EZmGpn4D8y17mW8j4vYukdfB9xwgB
91p6+IeaRiOXMYsTTHxNwKeLf5iw7wtFntBk8mRikoA//UwrqSe9kOfkfFHK
niIgeUW++3v0UeSVgZuzRX+55J0b5VnpbvB/j7RMJhtNENA+72VbjHkI8m0L
jCqVMQLGcDjmZw9HIhepbmq+MUjAfNH2IYpEHPKwW0NCCgwCeoqdfm7EfIy8
qvcPu/AmAl5Um7sZUPEUr//IeKUhj4AHWKXHaW7P8ToyEyvBUQQ0If4oLt1U
gFyFVTrq83UC2vo+sxgjFyP/KMJ5S8CAgH8JNU16Or9EvjVOmC1G7NdzUUuJ
12d7hbynojWZ9o4JXdhCbpt1VSL/6Zfw1qCECXXs/yqzDKpBntM0syvAmwk5
GvtYFtZB5KHbhr9dUGLCp6wuXA/665Hb1FL14K/5X4g6Fqoo0IhceCGKjZLN
gKT4TyRnnSbkN1NVi8PNGNCcdccD77Bm5P/MROeKsjJg02Y2raKyFuSGZFLP
8ww6NOEJsNq11Iq874CS0NgxOqSeixy8v4eKnPXWxj8PT9LgFSGl4z4abcj3
7mGXb/qVJ80Xnk1ZOrYjn6Ws+clw02DV3j7P9x4dyDlOXysqSu+F9d1rKhTv
TuTE8va4SqleyP3m1fFDYV3Iv15kr+y82ANDVhJsRIK6ka9F/5D583w37G0V
+fHibg/yJZ+fm6Ttu6DJzm9kTtVe5Oy1qt4qDp2wyKLNNO059rsa7cpGRh1Q
onLdwLEdNORftgV/v6neDq8cG6mfuoNdhiUcfuJpgwJHb5g9WMTurdUhxjtJ
gUbUuIOnLOjIdTxWP6Y1tsLWFcpAbwn28/Gjgu9CWqDv/UHpJR4Gcrcemumc
eTM03x5vL2WHnSI6kCoq3QQj1iv5KhVg9+Iiq9aMNcBjnnH0yO/Yp47NU3Zk
1UPXas1FhgYT+TlnOV2fpDromrx+wiQIe52HluOPuWrYSa4XZ4HYd1Xf8qU7
VsJ/T/PJxS5i36TqZeqcVQ6n98wNrEgSyNmm+Twvc7+E1y/v7KKQsFse5jfx
OVQMmWc39pM8sfMlHxzaRM+DEk8/u2kkYO94zuOrG5UNP3p9aSkuwj52vkiO
s/8J/INqWXC7Fft+QTtX+lIitE76RLtMYF+eO/q5sSgGqvDR3TuGsDOfWYnt
6H8A263ZVmfHsOvNbxwWdAiExT2aA+8nsEep5Ild73GEIwPh2e5T2P2OMHbm
ztmBrdG+1aG/eZvL9L90xi3QprtP6MTkb/uwYhqcuBoGYOZn24Bx7C4tVdIl
ujFgeDqOVXoYe1AgmzF1MRHsEinY4/zbfenEkPvlUtNAD63eU7n7t/X7eHiT
YjOB4jRjhF6D/c+OS56sei9A2IZPtsqZ2J0SWZzXogsBaahpU38o9uTI6Cyf
cyVA/9A32XWO2A3ZGaqGu8uAeVzRCPUo9uG3hipGFa/ApW2Rvtv5f1uHyrys
61cFhFrUUv4cxfPT51JRLilTC6YNjo7dzMAuwOlWkfIMggkXp5yKS9hDnZ1f
KUo1gBPKX/RTdmH31+XMUD7fCHodOJZ3jOH59/Ungi8XNIHvgoXx7hHYY2Ri
JPd+bAZcDqW6c4ex//R7cERSuBUoRBeSbN7gcxr3eGOxtBkFRBobJDXewV5D
Ygp7u1MBn6xOwIgU9jr+fWu299vAZs9v5REUnA+S+c/9hArbwbLIT/JlK+z7
76qIp1Z1AKVUdQ7DBZxL2ikRgduJThBEb/pW6Y09a5xfiTzZBdSNT2TUreHc
qwlN3tfQ1w1q0/Je5PJjJzFXbE7394DxIIP4u1w4P0/doKy/adMLrjZ4XXHa
jPM2nd06WL6rF3Cw7f9xhwfns4DO5KchQAM5yuc+kD/hnDcdeeSa9owG5JP0
T0lO417YvuSRSeKjg3hJqYj5ctwjsVz1WQ7X6eBMrfZB7wQK8uOznJNGbXRg
pOh8e5s37qOZHvvqVkkGCCaH7fNRw/11r8FL+aoHA3BEb9rTxo37zub8aY04
CgPcK7ri5VmPe1PlK9w/xM8Et39sG9ni0YC8upDQOWDGBNEZihblR3D/doul
mqTFMYGXs3T+EUot8keUAcPYLia4fuxGC+loNX4/ifPgjv3JBCGumx+2V1Yg
PxRLFjKTJ8DgoJz4nm9lyHmSlaVTLQgQO5nAPGxXivzxRc3Gh7cIoL0z76R/
ZBFyaw51t7VUArwMHAKrd3Px93eT4OAvJwBH3DQL2fAZ8jPF1lLu7QTIL0+5
yZeQjlxU2u+1wAABuPsnsiQC45Hnnjp2S3eUALdMKlz4y6OQr4u8S/vwmgCy
PPuP5jPu4Pcu8TX9lV+/h2e2jtmeZXog////xnEFiUt18np1//t/NGNa6A==

      "]], Line3DBox[CompressedData["
1:eJxN2Hk8VOsfB3C7sqW6IWulblmiKCQ8ylIhkuxalFBJIq4lokHWUCJklxGy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      "]]}, {
     {GrayLevel[0.4], Thickness[0.005], 
      Line3DBox[{{-1., 0, 0}, {1., 0, 0}}]}, 
     {GrayLevel[0.4], Thickness[0.005], 
      Line3DBox[{{0, -1., 0}, {0, 1., 0}}]}, 
     {GrayLevel[0.4], Thickness[0.005], 
      Line3DBox[{{0, 0, -1.}, {0, 0, 1.}}]}}, 
    {RGBColor[0, 0, 1], PointSize[0.02], Thickness[0.003], 
     Point3DBox[{-1, 0, 0}], Line3DBox[CompressedData["
1:eJxN1nk41dsaB3BD3Yw50jFmdkydtjE5qb2uzA4O21RxOKaiSJq0qauuqwFF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      "]], Line3DBox[CompressedData["
1:eJxt2Gk8VVsfB/AToSjDrVtkSDdJmROlsBSpzI2UIUqhuHSRW0IplMxySCJk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      "]]}}},
  Axes->False,
  Boxed->False,
  Lighting->None]], "Output",
 CellChangeTimes->{{3.391858507796875*^9, 3.391858516296875*^9}},
 CellLabel->"Out[101]=",
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]]
}, Open  ]]
},
WindowSize->{615, 750},
WindowMargins->{{388, Automatic}, {Automatic, 91}},
FrontEndVersion->"6.0 for Microsoft Windows (32-bit) (April 20, 2007)",
StyleDefinitions->"HelpBrowser.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "VQM`Spinors`"->{
  Cell[590, 23, 57, 1, 37, "Subsection",
   CellTags->"VQM`Spinors`"],
  Cell[650, 26, 207, 5, 39, "Text",
   CellTags->"VQM`Spinors`"],
  Cell[860, 33, 29548, 733, 1989, "Text",
   CellTags->"VQM`Spinors`"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"VQM`Spinors`", 254374, 4587}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[590, 23, 57, 1, 37, "Subsection",
 CellTags->"VQM`Spinors`"],
Cell[650, 26, 207, 5, 39, "Text",
 CellTags->"VQM`Spinors`"],
Cell[860, 33, 29548, 733, 1989, "Text",
 CellTags->"VQM`Spinors`"],
Cell[30411, 768, 45, 0, 39, "MathCaption"],
Cell[30459, 770, 124, 2, 27, "Input"],
Cell[30586, 774, 41, 0, 39, "MathCaption"],
Cell[CellGroupData[{
Cell[30652, 778, 504, 15, 69, "Input"],
Cell[31159, 795, 98673, 1655, 401, 87430, 1468, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[129869, 2455, 658, 17, 68, "Input"],
Cell[130530, 2474, 123515, 2099, 401, 112198, 1911, "CachedBoxData", \
"BoxData", "Output"]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
