(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     11243,        376]
NotebookOptionsPosition[      6427,        218]
NotebookOutlinePosition[      9234,        307]
CellTagsIndexPosition[      8890,        294]
WindowFrame->Normal
ContainsDynamic->False*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["QArgColorPlot", "Section",
 CellTags->"QArgColorPlot"],

Cell["\<\
QArgColorPlot[f[x],{x,x0,x1},opts] is used like the usual Plot command. It \
gives a two-dimensional plot of a complex-valued function f of a single real \
variable x in the range {x0,x1}. The plot shows the curve Abs[f] with area \
between the curve and the x-axis colored by Hue[Arg[f[x]]/(2 Pi)]. The \
default options of Plot are changed to Axes->{True,None}, Fame->True. \
Package: VQM`ArgColorPlot`\
\>", "Usage",
 CellTags->"QArgColorPlot"],

Cell[TextData[{
 "This command is contained in the package ",
 ButtonBox["VQM`ArgColorPlot`",
  BaseStyle->"AddOnsLink"]
}], "Text",
 CellTags->"QArgColorPlot"]
}, Open  ]],

Cell[CellGroupData[{

Cell["QListArgColorPlot", "Section",
 CellTags->"QListArgColorPlot"],

Cell["\<\
QListArgColorPlot[f,{x,x0,x1},opts] plots a Abs[f], where f is a list of \
complex numbers. The  points of the list Abs[f] are joined by a line. The \
area between the curve and the x-axis is colored at each point by \
Hue[Arg[f]/(2 Pi)]. Package: VQM`ArgColorPlot`\
\>", "Usage",
 CellTags->"QListArgColorPlot"],

Cell[TextData[{
 "This command is contained in the package ",
 ButtonBox["VQM`ArgColorPlot`",
  BaseStyle->"AddOnsLink"]
}], "Text",
 CellTags->"QListArgColorPlot"]
}, Open  ]],

Cell[CellGroupData[{

Cell["QCombinedPlot", "Section",
 CellTags->"QCombinedPlot"],

Cell["\<\
QCombinedPlot[{f[x],g[x]},{x,x0,x1},opts] works like QArgColorPlot with \
respect to f. The curve g is drawn in front of the QArgColorPlot of f. \
Package: VQM`ArgColorPlot`\
\>", "Usage",
 CellTags->"QCombinedPlot"],

Cell[TextData[{
 "This command is contained in the package ",
 ButtonBox["VQM`ArgColorPlot`",
  BaseStyle->"AddOnsLink"]
}], "Text",
 CellTags->"QCombinedPlot"]
}, Open  ]],

Cell[CellGroupData[{

Cell["QListCombinedPlot", "Section",
 CellTags->"QListCombinedPlot"],

Cell["\<\
QListCombinedPlot[{list,f[x]},{x,x0,x1},opts] works like QListArgColorPlot \
with respect to list. It is assumed that list represents the discretized \
values of a function defined on the interval [x0,x1]. The color list plot is \
then combined with an ordinary plot of f on the same scale and with the Ticks \
automatically adjusted. Package: VQM`ArgColorPlot`\
\>", "Usage",
 CellTags->"QListCombinedPlot"],

Cell[TextData[{
 "This command is contained in the package ",
 ButtonBox["VQM`ArgColorPlot`",
  BaseStyle->"AddOnsLink"]
}], "Text",
 CellTags->"QListCombinedPlot"]
}, Open  ]],

Cell[CellGroupData[{

Cell["QSpinorPlot", "Section",
 CellTags->"QSpinorPlot"],

Cell["\<\
QSpinorPlot[{func1,func2},{x,x0,x1},opts] provides a method to visualize \
C^2-valued functions (for example, spinor wavefunctions in quantum \
mechanics). The QSpinorPlot combines a QArgColorPlot of func1 with a \
QArgColorPlot of func2 (upside down, with less saturation) Both curves are \
plotted with the option QSquared->True (that is, a plot of the curve \
Abs[func]^2 is filled with a color describing the phase). In the background, \
a filled plot of Abs[func1]^2 + Abs[func2]^2 displays the corresponding \
density. Package: VQM`ArgColorPlot`\
\>", "Usage",
 CellTags->"QSpinorPlot"],

Cell[TextData[{
 "This command is contained in the package ",
 ButtonBox["VQM`ArgColorPlot`",
  BaseStyle->"AddOnsLink"]
}], "Text",
 CellTags->"QSpinorPlot"]
}, Open  ]],

Cell[CellGroupData[{

Cell["QListSpinorPlot", "Section",
 CellTags->"QListSpinorPlot"],

Cell["\<\
QListSpinorPlot[list,opts] visualizes a spinor-valued list of complex \
numbers. Each element of list is a C^-vector, that is, list = \
{{z11,z12},{z21,z22},...}. Alternatively, list = {list1,list2} with two lists \
of complex numbers, list1 giving the upper component of the spinor-valued \
wave function, and list2 giving the lower component. The lower component is \
plotted upside down with less saturation. See also the description of \
QSpinorPlot. Package: VQM`ArgColorPlot`\
\>", "Usage",
 CellTags->"QListSpinorPlot"],

Cell[TextData[{
 "This command is contained in the package ",
 ButtonBox["VQM`ArgColorPlot`",
  BaseStyle->"AddOnsLink"]
}], "Text",
 CellTags->"QListSpinorPlot"]
}, Open  ]],

Cell[CellGroupData[{

Cell["QSpinorCombinedPlot", "Section",
 CellTags->"QSpinorCombinedPlot"],

Cell["\<\
QSpinorCombinedPlot[{func1,func2},{x,x0,x1},opts] combines a QSpinorPlot of \
func1 with an ordinary Plot of a real-valued function func2. See the \
description of QCombinedPlot and of QSpinorPlot. Package: VQM`ArgColorPlot`\
\>", "Usage",
 CellTags->"QSpinorCombinedPlot"],

Cell[TextData[{
 "This command is contained in the package ",
 ButtonBox["VQM`ArgColorPlot`",
  BaseStyle->"AddOnsLink"]
}], "Text",
 CellTags->"QSpinorCombinedPlot"]
}, Open  ]],

Cell[CellGroupData[{

Cell["QListSpinorCombinedPlot", "Section",
 CellTags->"QListSpinorCombinedPlot"],

Cell["\<\
QListSpinorCombinedPlot[{list,f[x]},{x,x0,x1},opts] combines a \
QListSpinorPlot of list1 with an ordinary Plot  of a real-valued function f. \
See the description of QListCombinedPlot and of QListSpinorPlot. Package: \
VQM`ArgColorPlot`\
\>", "Usage",
 CellTags->"QListSpinorCombinedPlot"],

Cell[TextData[{
 "This command is contained in the package ",
 ButtonBox["VQM`ArgColorPlot`",
  BaseStyle->"AddOnsLink"]
}], "Text",
 CellTags->"QListSpinorCombinedPlot"]
}, Open  ]],

Cell[CellGroupData[{

Cell["QNiceTicks", "Section",
 CellTags->"QNiceTicks"],

Cell["\<\
QNiceTicks[xmin,xmax,dx] provides a list of nice positions for use in the \
Ticks or FrameTicks option in a ListPlot, where it is assumed that the list \
of values ranges between xmin and xmax in steps dx. Package: \
VQM`ArgColorPlot`\
\>", "Usage",
 CellTags->"QNiceTicks"],

Cell[TextData[{
 "This command is contained in the package ",
 ButtonBox["VQM`ArgColorPlot`",
  BaseStyle->"AddOnsLink"]
}], "Text",
 CellTags->"QNiceTicks"]
}, Open  ]]
},
WindowSize->{615, 750},
WindowMargins->{{388, Automatic}, {Automatic, 91}},
FrontEndVersion->"6.0 for Microsoft Windows (32-bit) (April 20, 2007)",
StyleDefinitions->"HelpBrowser.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "QArgColorPlot"->{
  Cell[590, 23, 60, 1, 93, "Section",
   CellTags->"QArgColorPlot"],
  Cell[653, 26, 457, 8, 173, "Usage",
   CellTags->"QArgColorPlot"],
  Cell[1113, 36, 160, 5, 33, "Text",
   CellTags->"QArgColorPlot"]},
 "QListArgColorPlot"->{
  Cell[1310, 46, 68, 1, 93, "Section",
   CellTags->"QListArgColorPlot"],
  Cell[1381, 49, 322, 6, 109, "Usage",
   CellTags->"QListArgColorPlot"],
  Cell[1706, 57, 164, 5, 33, "Text",
   CellTags->"QListArgColorPlot"]},
 "QCombinedPlot"->{
  Cell[1907, 67, 60, 1, 93, "Section",
   CellTags->"QCombinedPlot"],
  Cell[1970, 70, 226, 5, 87, "Usage",
   CellTags->"QCombinedPlot"],
  Cell[2199, 77, 160, 5, 33, "Text",
   CellTags->"QCombinedPlot"]},
 "QListCombinedPlot"->{
  Cell[2396, 87, 68, 1, 93, "Section",
   CellTags->"QListCombinedPlot"],
  Cell[2467, 90, 418, 7, 152, "Usage",
   CellTags->"QListCombinedPlot"],
  Cell[2888, 99, 164, 5, 33, "Text",
   CellTags->"QListCombinedPlot"]},
 "QSpinorPlot"->{
  Cell[3089, 109, 56, 1, 93, "Section",
   CellTags->"QSpinorPlot"],
  Cell[3148, 112, 602, 10, 216, "Usage",
   CellTags->"QSpinorPlot"],
  Cell[3753, 124, 158, 5, 33, "Text",
   CellTags->"QSpinorPlot"]},
 "QListSpinorPlot"->{
  Cell[3948, 134, 64, 1, 93, "Section",
   CellTags->"QListSpinorPlot"],
  Cell[4015, 137, 536, 9, 195, "Usage",
   CellTags->"QListSpinorPlot"],
  Cell[4554, 148, 162, 5, 33, "Text",
   CellTags->"QListSpinorPlot"]},
 "QSpinorCombinedPlot"->{
  Cell[4753, 158, 72, 1, 93, "Section",
   CellTags->"QSpinorCombinedPlot"],
  Cell[4828, 161, 283, 5, 109, "Usage",
   CellTags->"QSpinorCombinedPlot"],
  Cell[5114, 168, 166, 5, 33, "Text",
   CellTags->"QSpinorCombinedPlot"]},
 "QListSpinorCombinedPlot"->{
  Cell[5317, 178, 80, 1, 93, "Section",
   CellTags->"QListSpinorCombinedPlot"],
  Cell[5400, 181, 300, 6, 109, "Usage",
   CellTags->"QListSpinorCombinedPlot"],
  Cell[5703, 189, 170, 5, 33, "Text",
   CellTags->"QListSpinorCombinedPlot"]},
 "QNiceTicks"->{
  Cell[5910, 199, 54, 1, 93, "Section",
   CellTags->"QNiceTicks"],
  Cell[5967, 202, 284, 6, 109, "Usage",
   CellTags->"QNiceTicks"],
  Cell[6254, 210, 157, 5, 33, "Text",
   CellTags->"QNiceTicks"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"QArgColorPlot", 6729, 228},
 {"QListArgColorPlot", 6960, 235},
 {"QCombinedPlot", 7201, 242},
 {"QListCombinedPlot", 7433, 249},
 {"QSpinorPlot", 7672, 256},
 {"QListSpinorPlot", 7901, 263},
 {"QSpinorCombinedPlot", 8145, 270},
 {"QListSpinorCombinedPlot", 8405, 277},
 {"QNiceTicks", 8664, 284}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[590, 23, 60, 1, 93, "Section",
 CellTags->"QArgColorPlot"],
Cell[653, 26, 457, 8, 173, "Usage",
 CellTags->"QArgColorPlot"],
Cell[1113, 36, 160, 5, 33, "Text",
 CellTags->"QArgColorPlot"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1310, 46, 68, 1, 93, "Section",
 CellTags->"QListArgColorPlot"],
Cell[1381, 49, 322, 6, 109, "Usage",
 CellTags->"QListArgColorPlot"],
Cell[1706, 57, 164, 5, 33, "Text",
 CellTags->"QListArgColorPlot"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1907, 67, 60, 1, 93, "Section",
 CellTags->"QCombinedPlot"],
Cell[1970, 70, 226, 5, 87, "Usage",
 CellTags->"QCombinedPlot"],
Cell[2199, 77, 160, 5, 33, "Text",
 CellTags->"QCombinedPlot"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2396, 87, 68, 1, 93, "Section",
 CellTags->"QListCombinedPlot"],
Cell[2467, 90, 418, 7, 152, "Usage",
 CellTags->"QListCombinedPlot"],
Cell[2888, 99, 164, 5, 33, "Text",
 CellTags->"QListCombinedPlot"]
}, Open  ]],
Cell[CellGroupData[{
Cell[3089, 109, 56, 1, 93, "Section",
 CellTags->"QSpinorPlot"],
Cell[3148, 112, 602, 10, 216, "Usage",
 CellTags->"QSpinorPlot"],
Cell[3753, 124, 158, 5, 33, "Text",
 CellTags->"QSpinorPlot"]
}, Open  ]],
Cell[CellGroupData[{
Cell[3948, 134, 64, 1, 93, "Section",
 CellTags->"QListSpinorPlot"],
Cell[4015, 137, 536, 9, 195, "Usage",
 CellTags->"QListSpinorPlot"],
Cell[4554, 148, 162, 5, 33, "Text",
 CellTags->"QListSpinorPlot"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4753, 158, 72, 1, 93, "Section",
 CellTags->"QSpinorCombinedPlot"],
Cell[4828, 161, 283, 5, 109, "Usage",
 CellTags->"QSpinorCombinedPlot"],
Cell[5114, 168, 166, 5, 33, "Text",
 CellTags->"QSpinorCombinedPlot"]
}, Open  ]],
Cell[CellGroupData[{
Cell[5317, 178, 80, 1, 93, "Section",
 CellTags->"QListSpinorCombinedPlot"],
Cell[5400, 181, 300, 6, 109, "Usage",
 CellTags->"QListSpinorCombinedPlot"],
Cell[5703, 189, 170, 5, 33, "Text",
 CellTags->"QListSpinorCombinedPlot"]
}, Open  ]],
Cell[CellGroupData[{
Cell[5910, 199, 54, 1, 93, "Section",
 CellTags->"QNiceTicks"],
Cell[5967, 202, 284, 6, 109, "Usage",
 CellTags->"QNiceTicks"],
Cell[6254, 210, 157, 5, 33, "Text",
 CellTags->"QNiceTicks"]
}, Open  ]]
}
]
*)

(* End of internal cache information *)

