(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[   3324578,      62108]
NotebookOptionsPosition[   3321684,      62014]
NotebookOutlinePosition[   3322117,      62033]
CellTagsIndexPosition[   3322074,      62030]
WindowFrame->Normal
ContainsDynamic->True *)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["\<\
Numerical solution of the one-dimensional Schr\[ODoubleDot]dinger equation\
\>", "Title"],

Cell["\<\
Bernd Thaller
18-May-1999\
\>", "Subsubtitle"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"$Version", ",", 
   RowBox[{"Date", "[", "]"}], ",", " ", 
   RowBox[{"$starttime", "=", 
    RowBox[{"AbsoluteTime", "[", "]"}]}]}], "}"}]], "Input",
 CellChangeTimes->{{3.391758665325775*^9, 3.391758672078021*^9}, {
  3.3917589195040874`*^9, 3.3917589253341517`*^9}},
 CellLabel->"In[34]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"6.0 for Linux x86 (64-bit) (April 20, 2007)\"\>", ",", 
   RowBox[{"{", 
    RowBox[{
    "2007", ",", "7", ",", "3", ",", "19", ",", "27", ",", 
     "48.397275`8.437395885526698"}], "}"}], ",", 
   "3.392479668397365`16.283092242712275*^9"}], "}"}]], "Output",
 CellChangeTimes->{{3.3917586739692755`*^9, 3.391758685988899*^9}, 
   3.3917587273464084`*^9, 3.3917589509364195`*^9, 3.391758987542348*^9, 
   3.391850151859375*^9, 3.3924724361076937`*^9, 3.392472468404931*^9},
 CellLabel->"Out[34]="]
}, Open  ]],

Cell[CellGroupData[{

Cell["Load packages", "Section"],

Cell[TextData[{
 "We load the package QuantumKernel.m, assuming that all files are properly \
installed.\n(The application \"QuantumKernel\" (\"quantumkernel.exe\"), the \
file \"QuantumKernel.m\", and the directory \"Kernel\" containing the file \
\"init.m\" have to be placed into the directory AddOns/Applications/VQM/ \
within the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " folder)."
}], "Text"],

Cell["Needs[\"VQM`QuantumKernel`\"]", "Input",
 CellChangeTimes->{{3.391758066220209*^9, 3.3917580754263964`*^9}},
 CellLabel->"In[35]:="],

Cell[TextData[{
 "This starts the application \"QuantumKernel\" (\"quantumkernel.exe\") which \
is able to exchange data with the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " kernel via the ",
 StyleBox["MathLink",
  FontSlant->"Italic"],
 " protocol."
}], "Text"],

Cell[TextData[{
 "For the graphical representation of the wave functions we use the package \
ArgColorPlot.m.\n(The proper place for this package is in the \
AddOns/Applications/VQM/ subdirectory of the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " folder)"
}], "Text"],

Cell["Needs[\"VQM`ArgColorPlot`\"]", "Input",
 CellChangeTimes->{{3.3917580788337803`*^9, 3.3917580833977985`*^9}},
 CellLabel->"In[36]:="],

Cell["\<\
We also take some precautions against unwanted error messages:\
\>", "Text"],

Cell["Off[General::spell1, Arg::indet];", "Input",
 CellLabel->"In[37]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell["Prepare the data", "Section"],

Cell["\<\
We are going to calculate the motion of a one-dimensional wave packet. The \
wave packet is a solution of the one-dimensional Schr\[ODoubleDot]dinger \
equation\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[ImaginaryI]", 
   FractionBox["\[PartialD]", 
    RowBox[{"\[PartialD]", "t"}]], "\[Psi]", 
   RowBox[{"(", 
    RowBox[{"x", ",", "t"}], ")"}]}], " ", "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     FractionBox["1", "2"]}], 
    FractionBox[
     SuperscriptBox["\[PartialD]", "2"], 
     RowBox[{"\[PartialD]", 
      SuperscriptBox["x", "2"]}]], "\[Psi]", 
    RowBox[{"(", 
     RowBox[{"x", ",", "t"}], ")"}]}], " ", "+", " ", 
   RowBox[{"V", 
    RowBox[{"(", "x", ")"}], " ", "\[Psi]", 
    RowBox[{"(", 
     RowBox[{"x", ",", "t"}], ")"}]}]}]}]], "DisplayFormula"],

Cell["\<\
The numerical calculations are done on a sufficiently large bounded interval. \
The application QuantumKernel automatically assumes Dirichlet boundary \
conditions, i.e., the wave function is set to zero at the borders of the \
interval. These boundary conditions may cause unwanted reflections. Therefore \
we have to place the initial wave packet far away from the border, and \
restrict the time interval, so that the reflections do not disturb the \
solution in the region of interest. \
\>", "Text"],

Cell["\<\
We start by defining the initial wave function. We take a Gaussian wave \
packet, initially localized at x0=-8, with average momentum p0=8. The Schr\
\[ODoubleDot]dinger equation above has been scaled such that the mass of the \
particle is 1, hence the momentum equals the velocity of the particle. For \
the numerical calculation we generate a one-dimensional list of complex \
numbers psi0 which represents the values of the initial wave function on the \
points of a space grid.\
\>", "Text"],

Cell["\<\
numleft = -20;      (* left border of the domain *)
numright = 20;      (* right border of the domain *)
dx = 0.02;          (* space step *)

p0 = 8.;            (* initial momentum *)
x0 = -7.;            (* initial position *)

psi0 = Table[
          Module[ {x=x1-x0},
             (1./Pi)^(1/4) Exp[I*p0*x-x*x/2]
          ],
       {x1,numleft,numright,dx}];\
\>", "Input",
 CellChangeTimes->{{3.391758137650223*^9, 3.391758151983116*^9}},
 CellLabel->"In[38]:="],

Cell["\<\
You have to make sure that psi0 is indeed a numerical list. Symbolic \
expressions cannot be handled by QuantumKernel. The table of complex values \
can be passed to QuantumKernel by the command QNewFunction. This command \
generates a \"function object\" for the QuantumKernel and returns a reference \
to this object.\
\>", "Text"],

Cell["psi = QNewFunction[Re[psi0],Im[psi0]];", "Input",
 CellChangeTimes->{{3.391758156578395*^9, 3.391758159673175*^9}},
 CellLabel->"In[44]:="],

Cell["\<\
QNewFunction only accepts real arguments. A complex list has to be split into \
the list of real parts and the list of imaginary parts. Note that all \
commands which are provided by the application QuantumKernel start with the \
letter \"Q\".\
\>", "Text"],

Cell["\<\
psi now contains the reference to the function object which contains the \
initial data. These data will later be modified by QuantumKernel. We can \
always read out the data contained in psi with the command QGetArray. Note \
that QGetArray[psi][[1]] contains the real part and QGetArray[psi][[2]] \
contains the imaginary part of the list of complex numbers representing the \
wave function.\
\>", "Text"],

Cell["\<\
psitemp = QGetArray[psi];
psidata = psitemp[[1]] + I*psitemp[[2]];\
\>", "Input",
 CellLabel->"In[45]:="],

Cell["\<\
We could plot this list with ListArgColorPlot[psidata,PlotRange\[Rule]All], \
but since the list contains many points, this takes a long time. It is \
sufficient to extract the interesting part from the list. This will be done \
later.\
\>", "Text"],

Cell["In the same way, we can define the potential function. ", "Text"],

Cell["potential[x_] := (p0^2/2) (Tanh[x]+1)/2;", "Input",
 CellLabel->"In[47]:="],

Cell[TextData[{
 "This is a smoothed potential step whose height is about the kinetic energy \
",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p0", "2"], TraditionalForm]]],
 "/2 of the wave packet. We generate a table of real numbers from the \
potential function and pass it to QuantumKernel:"
}], "Text"],

Cell["pot = Table[N[potential[x]],{x,numleft,numright,dx}];", "Input",
 CellLabel->"In[48]:="],

Cell["V = QNewFunction[Re[pot],Im[pot]];", "Input",
 CellLabel->"In[49]:="],

Cell["\<\
Now, we have enough information to build an operator object. Since we want to \
solve the one-dimensional Schr\[ODoubleDot]dinger equation, we create this \
object with the command QSchr\[ODoubleDot]dinger1D.\
\>", "Text"],

Cell["H = QSchroedinger1D[V, 1, dx];", "Input",
 CellLabel->"In[50]:="],

Cell["\<\
This defines the generator of the time evolution for QuantumKernel. Here the \
arguments describe the potential V (a function object), the mass of the \
particle (=1 in our case), and the spacial step size dx.\
\>", "Text"],

Cell["\<\
The basic functionality of the application QuantumKernel can now be accessed \
with the help of the command QTimeEvolution.\
\>", "Text"],

Cell["\<\
QTimeEvolution[H, psi, dt, 0, n] performs n time steps of size dt to \
calculate the solution of the Schr\[ODoubleDot]dinger equation (described by \
the operator H) with initial function psi. The solution is calculated with \
double precision using a Crank-Nicolson formula. After the execution of the \
command QTimeEvolution[H, psi, dt, 0, n], the function object psi will \
contain the data of the solution at time n*dt (the initial data are \
overwritten). \
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Graphics preparations", "Section"],

Cell["\<\
The numerical calculation is done on a large interval in order to avoid \
boundary effects. It is sufficient to look at a smaller interval for the \
graphical presentation of the result. The plotregion is defined by the \
following quantities:\
\>", "Text"],

Cell["\<\
plotleft = -10;     (* left border of plot interval *)
plotright = 10;     (* right border of plot interval *)
lower = -.1;        (* lower border of plot region *)
upper = 1.1;        (* upper border of plot region *)
\
\>", "Input",
 CellLabel->"In[51]:="],

Cell["\<\
The step width dx is very small to achieve high accuracy for the numerical \
calculation. It is not necessary to use that many plot points. Here we \
specify that we want use only every fourth value for the plot.\
\>", "Text"],

Cell["skipfac = 4;        (* plot skipping values *)", "Input",
 CellLabel->"In[55]:="],

Cell["\<\
We have to define the part specification in order to extract from the large \
list of values the ones we are interested in.\
\>", "Text"],

Cell["\<\
a = Floor[(plotleft-numleft)/dx]+1;
b = (plotright-numleft)/dx+1;
partlist  = Range[a,b,skipfac];\
\>", "Input",
 CellChangeTimes->{3.391758193324995*^9},
 CellLabel->"In[56]:="],

Cell["\<\
For the plot it is useful to have the positions of the tickmarks on the \
x-axes. The package ArgColorPlot provides the command NiceTicks for this \
purpose.\
\>", "Text"],

Cell["myticks = QNiceTicks[plotleft,plotright,dx*skipfac];", "Input",
 CellChangeTimes->{{3.3917588149380493`*^9, 3.3917588516377587`*^9}},
 CellLabel->"In[59]:="],

Cell["\<\
Now we define the plot command which gets the values referred to by psi from \
QuantumKernel, extracts the parts, and performs a ListPlot with the \
appropriate settings.\
\>", "Text"],

Cell["\<\
showIt :=
   Module[{psi1=QGetArray[psi], psi2},
      psi2 = psi1[[1]] + I psi1[[2]];
      QListArgColorPlot[psi2[[partlist]],
         PlotRange->{lower,upper},
         Axes->{True,None},
         Frame->True,
         FrameTicks->{myticks,Automatic,None,None}]
   ];
   \
\>", "Input",
 CellLabel->"In[60]:="],

Cell["\<\
We can test this command immediately. This shows the initial wave function:\
\>", "Text"],

Cell[CellGroupData[{

Cell["showIt     (* Initial state *)", "Input",
 CellLabel->"In[62]:="],

Cell[BoxData[
 GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF13tQFfcVwPEfCkYySlRUJMEUCBCiEHkJ8hAP8lael3t5P/ZibAw+oLaW
pFrzU8fSPCpNjMYgjZukGWNrQ6KMNU0Cv5iSxMQaMZJqCLhKrKR5FBUyWgnU
zj2Z6Z6Z7rl/wHxYvnvu3f39duYG1NbbVk4QQtx568d/f7teQ0ty1jy/+sPZ
W5bgH2DDcG/6q/u3oydDa0pIw/S0J9DTYN35K52ZXk+hZ8Le4L/8/k/Ru9Bz
4IGBKdFTIlrQfvDX4FW9/dN1tD9EpLecf2XPS+hAmB342LXHOvajg+DttYNz
o3cfRIdAVFDflCTf19GhMDnAdviyrR09D77zrfXqF0fRYbDxUs3O6fPfQt8P
N0fCvIPWKHQE+HwzuLZ30rvoSPjmyPY/5F7uQkdBaXCD859Tj6OjYVPxvKx3
HjmBjoENv/pp5PyZp9ALoeDz7saO20+jYyG5duy3z889g46DbW7vNV7M+hS9
CD7+86472pvOouPh6itNe0rOfIZOgKbXn/4sMbQPnQhJb9V7e286j06CU1e7
7vn3gIFeDA87o/KPuV1EJ0PiSN0H3t4D6CUQ7nVy7DnvL9AAXw9HPuQzjpYA
668dKN/afQmPp0Bz7+zuwKZ/4PEUWFN9Ov1Vv8t4fCn4DXzXrzWj5VL4e1fj
Tu++H46nQkjAheM57oN4nFikmf9fppnPJ9LN82S6+f2IDPP7lRnmzyMyzZ9X
ZpLrkWW+XjKLXM9s8/WW2eR+LDPfL7mM3M/l5vstl5P1kGNeLzKHrKdc83qT
uWQ95pnXq8wj6znfvN5lPtkPBeb9AgXm/SQLzPtNFZD9WGjer1Bo3s+y0Lzf
VSF5HtjMzwuwmZ8n0mZ+3igbeR4Vgel5BUUQEdvXc/DEL7Avgow7O59LuvAz
7IvAb2znwPX7GrC3Q9a938Zf7FuFvR0SZ0zautptBfZ2ODLYOvJRXyX2djj1
Qvs7oR84sHeAf89YzIf2Auwd0NRXUB/lyMLeAc2fjoZMnLgUewdMyw9/+8zm
BOyLYajr9ozrldHYF8NW35K88bvDsC+G2Qeui6hlQdgXQ1zj7/yf8ZmLfQls
STgStvCNmdiXwOHIzFi34qnYl8BISr3PSy+4Y18Cq93vjn6yZzTZ1ZdC5j7v
5HSPYZehFD6v7zs8uO8rl2UpdLm37fnkXxddVqXQebS9deKJc9iXwZXqc7lb
NnRjXwYLdid+Gb3xfezLoLen7ZCfsxP7MnDb5kzN72nHvhzKffM9mjf9Efty
aFv3+B1p7i9iXw4t87/tPNv4LPblkLFzju39mh3YV8BtHjd6jKZt2FfAoYAH
w74Yfhj7CnDET7inva0e+woAY7z/tdEV2FdC2LWfeJ2eWoZ9JVz12rd5c0w+
9pWw48KC48H5qdhXQriI/GR9Rxz2VXCj8MaZjyeEY18F0zc9+VpkawD2VVCT
UBcTnjwL+yrIDu7/0b0/98S+GkZi82a8axtb7OqrwffRjv0eV664LKthzcFd
dY1zLrmsquHc12nry14+67Koge+frnHMbz2BfQ00R3gOjT6qsK+B9/ae/n7S
l4ewr4HR7XedPPnrl7HXYFZQbuexqj0u+2vwou8bSR4rn8DzabAq0HOGo2Wz
y5oGPkMxe8dvNuD5NYjP6Shd+2aty7oGH9kvFYY22XGeBm+qXX9bVJ7hsqHB
7q2B8VMWLML5TsgpOrD7N+P34XwnrPul0lO77sL5TriZua9hwyNTcb4TOvr7
ni2aNZ7kmu+ErrMbPWdkD7msO8FzwUn70WHDZXWr35H2Vcvj3S4bTjharOJu
m3zMZVEL9vH4CW11h1z2rwXxvy8g1oglsU6siA1iscJsf2Ig1oglsU6siA1i
8QCZTwzEGrEk1okVsUEsVpL5xECsEUtinVgRG8Tix2Q+MRBrxJJYJ1bEBrF4
kMwnBmKNWBLrxIrYIBaryHxiINaIJbFOrIgNYvEQmU8MxBqxJNaJFbFBLOrI
fGIg1oglsU6siA1isZrMJwZijVgS68SK2CAWa8h8YiDWiCWxTqyIDWKxlswn
BmKNWBLrxIrYIBbryHxiINaIJbFOrIgNYlFP5hMDsUZ86/u62QKsPZnxNMYz
Gc9h7MfYn3Eg4yDGIYxDGc9jHMb4fsYRjCMZRzGOZhzDeCHjWMZxjBcxjmec
wDiRcRLjxYyTGS9hDNaWjEWKtSVjsdTakrFItbZkLNKsLRmLdGtLxiLD2pKx
yLS2ZCyyrC0Zi2xrS8ZimbUlY7Hc2pKxyLG2ZCxyrS0ZizxrS8Yi39qSsSiw
NjCWjBVjUWhtYCwZK8bCZm1gLBkrxqLI2sBYMlaMhd3awFgyVoyFw9rAWDJW
jEWxtYGxZKwYixJrA2PJWDEWpdYGxpKxYizKrA2MJWPFWJRbGxhLxoqxqLA2
MJaMFWNRaW1gLBkrxqLK2sBYMlaMRbW1gbFkrBiLGmsDY8lYMRaatf0ZA2ON
sWSsM1aMDcbCaW1/xsBYYywZ64wVY4OxqP2//g/GCPe5
   "], {
    {GrayLevel[0.5], EdgeForm[None], 
     GraphicsGroupBox[{PolygonBox[{{144, 538, 145}}], 
       PolygonBox[CompressedData["
1:eJwt1WdwVkUYhuF3v4QkJCGkQEhoEkVFQMBGExAUFQERAyIgIkZFBKRJUZDe
O/bee++9994FO1hQ7L2hAt7PPPlxzdyz58zuOefbTapqJlZPyETEWmQjF3kp
oi7yaF3bjh3IqhVcS8n3S4bOQp2wbDVy6BI0pZuhNDyWQ+em2rWQTxdoTbo+
KuhKFIfHCul6yKcLUEyXoDCslC7TPWEN6IYoCiunGyXPrTmK1Mlz7YKedC+0
DK/ZmG6S/Ow7owd9QPK9lWhL74nGWkv36JrWDGtJ74rysN3o3fUMYa3oPVBB
l6EFXZU8l75Rc3qn5Gtaox3dXs9EV6Eb3T352TTWgd4LTelm2JfeD83DOtKd
NGdYZ7oLWoR1pfdPnltz7E3vkzyXnrE13Sb53fWNDqQP0jvS3TGVnoYedBsM
pI9EW707DqEP1TcI60MfhlZhfel++iZh/enD0To8RzU9SN88PDaAPiJ5LY0N
po9CO707augT9A3o9hhKD0OHsOH0MfpmeleMpI/TO9eegwo2fSXWhcdGce34
5Lm1xhD66OS59Q160wcnv6vWPJE+CR3pThhDn4LO4W80nZ6hPUV3wXj6VHQN
m0BP1G8SNomejG5hU+jTkr+95hxLj0ueS2uOpk9OXltrnE6fgZ76LbCEXqrf
JDw2k56lM6DfFnPoufqNw+bR89E7bAG9UO8ctohenDy35jiTnp08l9ZYRi9H
n7AV9ErtgbBV9Gr0pfthHX0W+oedTZ+jPRF2Ln0eBoSdT1+gPRF2IX0RBoZd
TF+iPUkPw630bRgedjt9h/YEPQhX0ldhcNjV9DXaY2HX0tdhSNj19A3aE2E3
0jdhaNjN9C3Ja+sZLqUvQ3XY5fQVyWvrGe6k78II7SU8TT+jPUUfi3vp+zAy
7H76Ae3hsAfphzAq7GH6Ee3hsEfpx1AT9jj9hM4MPQ5v0m9hfPgZ7qbvSV5b
z/As/RxGhz1Pv6A9F/Yi/RLGhL1Mv6IzEPYq/RrGhr1Ov5G8tp7hSfqp5HfX
nlhDr03eG3qmt+l3dGZ0VrCBfk9nhp6Kz+kvMC089j79ASaFfUh/pDMV9jH9
CaaEbaQ36YzpLOBP+i+difDYp/RnyWtpjc30l5iuvY9f6d90BsJjX9FbMENn
Ed/S3+lMhn1P/4CZYT/SP+lMhv1M/5I8t+b4mv4meS6t8Tv9R/Kz6hn/prdi
XvgbvUuvT/5W8/EfvQ0LwrbTO3Smw/RPP2FRWIbOwuKwbLoOloTl0LlYGl7z
H+b6N3ktjeVxrS6W0ctRSNfDivBYPl2Q8TWNFdH1sTL8N3qE9l7y32qNFXOt
BKvo1WhAN8SasHK6UcZ/y3VPKV2W8b1J749tfs34HyMp2TY=
        "]]}]}, {{}, {}, {
      {Hue[0.18028136750030463`, 1, 1], LineBox[{1, 2}]}, 
      {Hue[0.2821405277234505, 1, 1], LineBox[{2, 3}]}, 
      {Hue[0.3839996895098306, 1, 1], LineBox[{3, 4}]}, 
      {Hue[0.48585885551422175`, 1, 1], LineBox[{4, 5}]}, 
      {Hue[0.5877180215108367, 1, 1], LineBox[{5, 6}]}, 
      {Hue[0.6895771830460102, 1, 1], LineBox[{6, 7}]}, 
      {Hue[0.7914363480013062, 1, 1], LineBox[{7, 8}]}, 
      {Hue[0.8932955106345002, 1, 1], LineBox[{8, 9}]}, 
      {Hue[0.995154674764976, 1, 1], LineBox[{9, 10}]}, 
      {Hue[0.09701383869155131, 1, 1], LineBox[{10, 11}]}, 
      {Hue[0.19887300185257092`, 1, 1], LineBox[{11, 12}]}, 
      {Hue[0.3007321646766159, 1, 1], LineBox[{12, 13}]}, 
      {Hue[0.4025913281533913, 1, 1], LineBox[{13, 14}]}, 
      {Hue[0.5044504921497359, 1, 1], LineBox[{14, 15}]}, 
      {Hue[0.6063096585468639, 1, 1], LineBox[{15, 16}]}, 
      {Hue[0.7081688161245494, 1, 1], LineBox[{16, 17}]}, 
      {Hue[0.8100279863161979, 1, 1], LineBox[{17, 18}]}, 
      {Hue[0.9118871469170837, 1, 1], LineBox[{18, 19}]}, 
      {Hue[0.013746311097204865`, 1, 1], LineBox[{19, 20}]}, 
      {Hue[0.11560547430984798`, 1, 1], LineBox[{20, 21}]}, 
      {Hue[0.21746463943072125`, 1, 1], LineBox[{21, 22}]}, 
      {Hue[0.31932379769118113`, 1, 1], LineBox[{22, 23}]}, 
      {Hue[0.42118296101349495`, 1, 1], LineBox[{23, 24}]}, 
      {Hue[0.5230421281566322, 1, 1], LineBox[{24, 25}]}, 
      {Hue[0.6249012884316127, 1, 1], LineBox[{25, 26}]}, 
      {Hue[0.7267604553979363, 1, 1], LineBox[{26, 27}]}, 
      {Hue[0.8286196194726919, 1, 1], LineBox[{27, 28}]}, 
      {Hue[0.930478782116229, 1, 1], LineBox[{28, 29}]}, 
      {Hue[0.03233794751233168, 1, 1], LineBox[{29, 30}]}, 
      {Hue[0.13419710785786682`, 1, 1], LineBox[{30, 31}]}, 
      {Hue[0.2360562725367862, 1, 1], LineBox[{31, 32}]}, 
      {Hue[0.337915437570838, 1, 1], LineBox[{32, 33}]}, 
      {Hue[0.4397746003827427, 1, 1], LineBox[{33, 34}]}, 
      {Hue[0.5416337626792227, 1, 1], LineBox[{34, 35}]}, 
      {Hue[0.6434929247365134, 1, 1], LineBox[{35, 36}]}, 
      {Hue[0.7453520909304452, 1, 1], LineBox[{36, 37}]}, 
      {Hue[0.8472112558675396, 1, 1], LineBox[{37, 38}]}, 
      {Hue[0.9490704184852241, 1, 1], LineBox[{38, 39}]}, 
      {Hue[0.05092958151477587, 1, 1], LineBox[{39, 40}]}, 
      {Hue[0.15278874413246035`, 1, 1], LineBox[{40, 41}]}, 
      {Hue[0.25464790906955476`, 1, 1], LineBox[{41, 42}]}, 
      {Hue[0.3565070752634866, 1, 1], LineBox[{42, 43}]}, 
      {Hue[0.4583662373207773, 1, 1], LineBox[{43, 44}]}, 
      {Hue[0.5602253996172573, 1, 1], LineBox[{44, 45}]}, 
      {Hue[0.662084562429162, 1, 1], LineBox[{45, 46}]}, 
      {Hue[0.7639437274632138, 1, 1], LineBox[{46, 47}]}, 
      {Hue[0.8658028921421332, 1, 1], LineBox[{47, 48}]}, 
      {Hue[0.9676620524876683, 1, 1], LineBox[{48, 49}]}, 
      {Hue[0.06952121788377091, 1, 1], LineBox[{49, 50}]}, 
      {Hue[0.1713803805273081, 1, 1], LineBox[{50, 51}]}, 
      {Hue[0.2732395446020636, 1, 1], LineBox[{51, 52}]}, 
      {Hue[0.3750987115683873, 1, 1], LineBox[{52, 53}]}, 
      {Hue[0.47695787184336774`, 1, 1], LineBox[{53, 54}]}, 
      {Hue[0.578817038986505, 1, 1], LineBox[{54, 55}]}, 
      {Hue[0.6806762023088189, 1, 1], LineBox[{55, 56}]}, 
      {Hue[0.7825353605692787, 1, 1], LineBox[{56, 57}]}, 
      {Hue[0.884394525690152, 1, 1], LineBox[{57, 58}]}, 
      {Hue[0.9862536889027952, 1, 1], LineBox[{58, 59}]}, 
      {Hue[0.0881128530829163, 1, 1], LineBox[{59, 60}]}, 
      {Hue[0.18997201368380215`, 1, 1], LineBox[{60, 61}]}, 
      {Hue[0.2918311838754506, 1, 1], LineBox[{61, 62}]}, 
      {Hue[0.39369034145313603`, 1, 1], LineBox[{62, 63}]}, 
      {Hue[0.4955495078502641, 1, 1], LineBox[{63, 64}]}, 
      {Hue[0.5974086718466087, 1, 1], LineBox[{64, 65}]}, 
      {Hue[0.6992678353233841, 1, 1], LineBox[{65, 66}]}, 
      {Hue[0.801126998147429, 1, 1], LineBox[{66, 67}]}, 
      {Hue[0.9029861613084487, 1, 1], LineBox[{67, 68}]}, 
      {Hue[0.004845325235023987, 1, 1], LineBox[{68, 69}]}, 
      {Hue[0.1067044893654998, 1, 1], LineBox[{69, 70}]}, 
      {Hue[0.2085636519986938, 1, 1], LineBox[{70, 71}]}, 
      {Hue[0.3104228169539898, 1, 1], LineBox[{71, 72}]}, 
      {Hue[0.41228197848916326`, 1, 1], LineBox[{72, 73}]}, 
      {Hue[0.5141411444857782, 1, 1], LineBox[{73, 74}]}, 
      {Hue[0.6160003104901695, 1, 1], LineBox[{74, 75}]}, 
      {Hue[0.7178594722765494, 1, 1], LineBox[{75, 76}]}, 
      {Hue[0.8197186324996953, 1, 1], LineBox[{76, 77}]}, 
      {Hue[0.9215777941909739, 1, 1], LineBox[{77, 78}]}, 
      {Hue[0.02343696238670696, 1, 1], LineBox[{78, 79}]}, 
      {Hue[0.1252961237555336, 1, 1], LineBox[{79, 80}]}, 
      {Hue[0.2271552890346943, 1, 1], LineBox[{80, 81}]}, 
      {Hue[0.32901445094794335`, 1, 1], LineBox[{81, 82}]}, 
      {Hue[0.43087361809934777`, 1, 1], LineBox[{82, 83}]}, 
      {Hue[0.5327327802452424, 1, 1], LineBox[{83, 84}]}, 
      {Hue[0.6345919419118717, 1, 1], LineBox[{84, 85}]}, 
      {Hue[0.736451107195464, 1, 1], LineBox[{85, 86}]}, 
      {Hue[0.8383102675915894, 1, 1], LineBox[{86, 87}]}, 
      {Hue[0.9401694331949104, 1, 1], LineBox[{87, 88}]}, 
      {Hue[0.042028596081378496`, 1, 1], LineBox[{88, 89}]}, 
      {Hue[0.143887761134637, 1, 1], LineBox[{89, 90}]}, 
      {Hue[0.24574692390835293`, 1, 1], LineBox[{90, 91}]}, 
      {Hue[0.3476060900256417, 1, 1], LineBox[{91, 92}]}, 
      {Hue[0.4494652500868171, 1, 1], LineBox[{92, 93}]}, 
      {Hue[0.5513244158951738, 1, 1], LineBox[{93, 94}]}, 
      {Hue[0.6531835736908052, 1, 1], LineBox[{94, 95}]}, 
      {Hue[0.7550427419614101, 1, 1], LineBox[{95, 96}]}, 
      {Hue[0.8569019034490182, 1, 1], LineBox[{96, 97}]}, 
      {Hue[0.9587610682684513, 1, 1], LineBox[{97, 98}]}, 
      {Hue[0.060620233206281154`, 1, 1], LineBox[{98, 99}]}, 
      {Hue[0.16247939410148296`, 1, 1], LineBox[{99, 100}]}, 
      {Hue[0.2643385598483031, 1, 1], LineBox[{100, 101}]}, 
      {Hue[0.3661977210045819, 1, 1], LineBox[{101, 102}]}, 
      {Hue[0.4680568867372577, 1, 1], LineBox[{102, 103}]}, 
      {Hue[0.5699160517664983, 1, 1], LineBox[{103, 104}]}, 
      {Hue[0.6717752158714863, 1, 1], LineBox[{104, 105}]}, 
      {Hue[0.7736343773628815, 1, 1], LineBox[{105, 106}]}, 
      {Hue[0.8754935383776598, 1, 1], LineBox[{106, 107}]}, 
      {Hue[0.9773527053475997, 1, 1], LineBox[{107, 108}]}, 
      {Hue[0.07921187154410292, 1, 1], LineBox[{108, 109}]}, 
      {Hue[0.18107103176047457`, 1, 1], LineBox[{109, 110}]}, 
      {Hue[0.2829301959870124, 1, 1], LineBox[{110, 111}]}, 
      {Hue[0.3847893572618031, 1, 1], LineBox[{111, 112}]}, 
      {Hue[0.48664852296885064`, 1, 1], LineBox[{112, 113}]}, 
      {Hue[0.5885076890195262, 1, 1], LineBox[{113, 114}]}, 
      {Hue[0.6903668523618101, 1, 1], LineBox[{114, 115}]}, 
      {Hue[0.7922260129636224, 1, 1], LineBox[{115, 116}]}, 
      {Hue[0.8940851796772397, 1, 1], LineBox[{116, 117}]}, 
      {Hue[0.99594434093879, 1, 1], LineBox[{117, 118}]}, 
      {Hue[0.0978035039647874, 1, 1], LineBox[{118, 119}]}, 
      {Hue[0.19966266872886562`, 1, 1], LineBox[{119, 120}]}, 
      {Hue[0.30152183135520416`, 1, 1], LineBox[{120, 121}]}, 
      {Hue[0.40338099665443905`, 1, 1], LineBox[{121, 122}]}, 
      {Hue[0.5052401592430923, 1, 1], LineBox[{122, 123}]}, 
      {Hue[0.607099323428438, 1, 1], LineBox[{123, 124}]}, 
      {Hue[0.7089584849626256, 1, 1], LineBox[{124, 125}]}, 
      {Hue[0.8108176506926813, 1, 1], LineBox[{125, 126}]}, 
      {Hue[0.9126768176019221, 1, 1], LineBox[{126, 127}]}, 
      {Hue[0.014535976785237121`, 1, 1], LineBox[{127, 128}]}, 
      {Hue[0.11639513558854607`, 1, 1], LineBox[{128, 129}]}, 
      {Hue[0.21825430575895977`, 1, 1], LineBox[{129, 130}]}, 
      {Hue[0.32011346404796626`, 1, 1], LineBox[{130, 131}]}, 
      {Hue[0.4219726297608182, 1, 1], LineBox[{131, 132}]}, 
      {Hue[0.5238317956302203, 1, 1], LineBox[{132, 133}]}, 
      {Hue[0.6256909569427747, 1, 1], LineBox[{133, 134}]}, 
      {Hue[0.727550121081068, 1, 1], LineBox[{134, 135}]}, 
      {Hue[0.8294092864077768, 1, 1], LineBox[{135, 136}]}, 
      {Hue[0.9312684497178493, 1, 1], LineBox[{136, 137}]}, 
      {Hue[0.033127611863875285`, 1, 1], LineBox[{137, 138}]}, 
      {Hue[0.13498677820638982`, 1, 1], LineBox[{138, 139}]}, 
      {Hue[0.2368459401250907, 1, 1], LineBox[{139, 140}]}, 
      {Hue[0.33870510239514673`, 1, 1], LineBox[{140, 141}]}, 
      {Hue[0.4405642648760048, 1, 1], LineBox[{141, 142}]}, 
      {Hue[0.5424234275736929, 1, 1], LineBox[{142, 143}]}, 
      {Hue[0.6442825916003974, 1, 1], LineBox[{143, 144}]}, 
      {Hue[0.746141757651104, 1, 1], LineBox[{144, 145}]}, 
      {Hue[0.8480009182266539, 1, 1], LineBox[{145, 146}]}, 
      {Hue[0.9498600826122677, 1, 1], LineBox[{146, 147}]}, 
      {Hue[0.051719249585379296`, 1, 1], LineBox[{147, 148}]}, 
      {Hue[0.15357841060037475`, 1, 1], LineBox[{148, 149}]}, 
      {Hue[0.25543757548193646`, 1, 1], LineBox[{149, 150}]}, 
      {Hue[0.3572967409946449, 1, 1], LineBox[{150, 151}]}, 
      {Hue[0.45915590232163533`, 1, 1], LineBox[{151, 152}]}, 
      {Hue[0.5610150635373692, 1, 1], LineBox[{152, 153}]}, 
      {Hue[0.6628742293835823, 1, 1], LineBox[{153, 154}]}, 
      {Hue[0.7647333935702304, 1, 1], LineBox[{154, 155}]}, 
      {Hue[0.8665925593341647, 1, 1], LineBox[{155, 156}]}, 
      {Hue[0.9684517188910791, 1, 1], LineBox[{156, 157}]}, 
      {Hue[0.07031088734743306, 1, 1], LineBox[{157, 158}]}, 
      {Hue[0.17217004562512694`, 1, 1], LineBox[{158, 159}]}, 
      {Hue[0.274029210966428, 1, 1], LineBox[{159, 160}]}, 
      {Hue[0.375888373572094, 1, 1], LineBox[{160, 161}]}, 
      {Hue[0.4777475389365056, 1, 1], LineBox[{161, 162}]}, 
      {Hue[0.5796067036319594, 1, 1], LineBox[{162, 163}]}, 
      {Hue[0.6814658687691331, 1, 1], LineBox[{163, 164}]}, 
      {Hue[0.7833250283541157, 1, 1], LineBox[{164, 165}]}, 
      {Hue[0.8851841969163478, 1, 1], LineBox[{165, 166}]}, 
      {Hue[0.987043356640097, 1, 1], LineBox[{166, 167}]}, 
      {Hue[0.08890252571682883, 1, 1], LineBox[{167, 168}]}, 
      {Hue[0.19076168586000714`, 1, 1], LineBox[{168, 169}]}, 
      {Hue[0.292620847373047, 1, 1], LineBox[{169, 170}]}, 
      {Hue[0.39448000715328024`, 1, 1], LineBox[{170, 171}]}, 
      {Hue[0.4963391743031589, 1, 1], LineBox[{171, 172}]}, 
      {Hue[0.5981983383709037, 1, 1], LineBox[{172, 173}]}, 
      {Hue[0.7000575032370594, 1, 1], LineBox[{173, 174}]}, 
      {Hue[0.8019166646042429, 1, 1], LineBox[{174, 175}]}, 
      {Hue[0.9037758261816837, 1, 1], LineBox[{175, 176}]}, 
      {Hue[0.005634991919216007, 1, 1], LineBox[{176, 177}]}, 
      {Hue[0.10749415064042574`, 1, 1], LineBox[{177, 178}]}, 
      {Hue[0.20935332017583547`, 1, 1], LineBox[{178, 179}]}, 
      {Hue[0.3112124825674883, 1, 1], LineBox[{179, 180}]}, 
      {Hue[0.4130716490685422, 1, 1], LineBox[{180, 181}]}, 
      {Hue[0.5149308102038845, 1, 1], LineBox[{181, 182}]}, 
      {Hue[0.616789974021388, 1, 1], LineBox[{182, 183}]}, 
      {Hue[0.7186491354964677, 1, 1], LineBox[{183, 184}]}, 
      {Hue[0.8205082995769946, 1, 1], LineBox[{184, 185}]}, 
      {Hue[0.9223674666126417, 1, 1], LineBox[{185, 186}]}, 
      {Hue[0.02422662897639987, 1, 1], LineBox[{186, 187}]}, 
      {Hue[0.12608578948147126`, 1, 1], LineBox[{187, 188}]}, 
      {Hue[0.22794495533336445`, 1, 1], LineBox[{188, 189}]}, 
      {Hue[0.32980411988616976`, 1, 1], LineBox[{189, 190}]}, 
      {Hue[0.43166328074234533`, 1, 1], LineBox[{190, 191}]}, 
      {Hue[0.5335224458588795, 1, 1], LineBox[{191, 192}]}, 
      {Hue[0.6353816097101509, 1, 1], LineBox[{192, 193}]}, 
      {Hue[0.7372407724843723, 1, 1], LineBox[{193, 194}]}, 
      {Hue[0.8390999401640373, 1, 1], LineBox[{194, 195}]}, 
      {Hue[0.9409591005084146, 1, 1], LineBox[{195, 196}]}, 
      {Hue[0.042818261174902755`, 1, 1], LineBox[{196, 197}]}, 
      {Hue[0.14467743035764916`, 1, 1], LineBox[{197, 198}]}, 
      {Hue[0.24653659073009973`, 1, 1], LineBox[{198, 199}]}, 
      {Hue[0.34839574974108056`, 1, 1], LineBox[{199, 200}]}, 
      {Hue[0.45025491685569347`, 1, 1], LineBox[{200, 201}]}, 
      {Hue[0.552114080374339, 1, 1], LineBox[{201, 202}]}, 
      {Hue[0.6539732460042567, 1, 1], LineBox[{202, 203}]}, 
      {Hue[0.7558324145141517, 1, 1], LineBox[{203, 204}]}, 
      {Hue[0.8576915934514485, 1, 1], LineBox[{204, 205}]}, 
      {Hue[0.9595507628714376, 1, 1], LineBox[{205, 206}]}, 
      {Hue[0.061409821696748426`, 1, 1], LineBox[{206, 207}]}, 
      {Hue[0.16326931675960346`, 1, 1], LineBox[{207, 208}]}, 
      {Hue[0.2651289529572075, 1, 1], LineBox[{208, 209}]}, 
      {Hue[0.3669882808535222, 1, 1], LineBox[{209, 210}]}, 
      {Hue[0.46884142783573884`, 1, 1], LineBox[{210, 211}]}, 
      {Hue[0.5707343955276603, 1, 1], LineBox[{211, 212}]}, 
      {Hue[0.6725813560157026, 1, 1], LineBox[{212, 213}]}, 
      {Hue[0.7741541746595132, 1, 1], LineBox[{213, 214}]}, 
      {Hue[0.8763374050162176, 1, 1], LineBox[{214, 215}]}, 
      {Hue[0.9765917527181093, 1, 1], LineBox[{215, 216}]}, 
      {Hue[0.07379180882521663, 1, 1], LineBox[{216, 217}]}, 
      {Hue[0.19879180882521666`, 1, 1], LineBox[{217, 218}]}, 
      {Hue[0.25, 1, 1], LineBox[{218, 219}]}, 
      {Hue[0., 1, 1], LineBox[{219, 220}]}, 
      {Hue[0., 1, 1], LineBox[{220, 221}]}, 
      {Hue[0., 1, 1], LineBox[{221, 222}]}, 
      {Hue[0., 1, 1], LineBox[{222, 223}]}, 
      {Hue[0., 1, 1], LineBox[{223, 224}]}, 
      {Hue[0., 1, 1], LineBox[{224, 225}]}, 
      {Hue[0., 1, 1], LineBox[{225, 226}]}, 
      {Hue[0., 1, 1], LineBox[{226, 227}]}, 
      {Hue[0., 1, 1], LineBox[{227, 228}]}, 
      {Hue[0., 1, 1], LineBox[{228, 229}]}, 
      {Hue[0., 1, 1], LineBox[{229, 230}]}, 
      {Hue[0., 1, 1], LineBox[{230, 231}]}, 
      {Hue[0., 1, 1], LineBox[{231, 232}]}, 
      {Hue[0., 1, 1], LineBox[{232, 233}]}, 
      {Hue[0., 1, 1], LineBox[{233, 234}]}, 
      {Hue[0., 1, 1], LineBox[{234, 235}]}, 
      {Hue[0., 1, 1], LineBox[{235, 236}]}, 
      {Hue[0., 1, 1], LineBox[{236, 237}]}, 
      {Hue[0., 1, 1], LineBox[{237, 238}]}, 
      {Hue[0., 1, 1], LineBox[{238, 239}]}, 
      {Hue[0., 1, 1], LineBox[{239, 240}]}, 
      {Hue[0., 1, 1], LineBox[{240, 241}]}, 
      {Hue[0., 1, 1], LineBox[{241, 242}]}, 
      {Hue[0., 1, 1], LineBox[{242, 243}]}, 
      {Hue[0., 1, 1], LineBox[{243, 244}]}, 
      {Hue[0., 1, 1], LineBox[{244, 245}]}, 
      {Hue[0., 1, 1], LineBox[{245, 246}]}, 
      {Hue[0., 1, 1], LineBox[{246, 247}]}, 
      {Hue[0., 1, 1], LineBox[{247, 248}]}, 
      {Hue[0., 1, 1], LineBox[{248, 249}]}, 
      {Hue[0., 1, 1], LineBox[{249, 250}]}, 
      {Hue[0., 1, 1], LineBox[{250, 251}]}}}},
   VertexColors->{
     Hue[0.18028136750030463`, 1, 1], 
     Hue[0.2821405277234505, 1, 1], 
     Hue[0.3839996895098306, 1, 1], 
     Hue[0.48585885551422175`, 1, 1], 
     Hue[0.5877180215108367, 1, 1], 
     Hue[0.6895771830460102, 1, 1], 
     Hue[0.7914363480013062, 1, 1], 
     Hue[0.8932955106345002, 1, 1], 
     Hue[0.995154674764976, 1, 1], 
     Hue[0.09701383869155131, 1, 1], 
     Hue[0.19887300185257092`, 1, 1], 
     Hue[0.3007321646766159, 1, 1], 
     Hue[0.4025913281533913, 1, 1], 
     Hue[0.5044504921497359, 1, 1], 
     Hue[0.6063096585468639, 1, 1], 
     Hue[0.7081688161245494, 1, 1], 
     Hue[0.8100279863161979, 1, 1], 
     Hue[0.9118871469170837, 1, 1], 
     Hue[0.013746311097204865`, 1, 1], 
     Hue[0.11560547430984798`, 1, 1], 
     Hue[0.21746463943072125`, 1, 1], 
     Hue[0.31932379769118113`, 1, 1], 
     Hue[0.42118296101349495`, 1, 1], 
     Hue[0.5230421281566322, 1, 1], 
     Hue[0.6249012884316127, 1, 1], 
     Hue[0.7267604553979363, 1, 1], 
     Hue[0.8286196194726919, 1, 1], 
     Hue[0.930478782116229, 1, 1], 
     Hue[0.03233794751233168, 1, 1], 
     Hue[0.13419710785786682`, 1, 1], 
     Hue[0.2360562725367862, 1, 1], 
     Hue[0.337915437570838, 1, 1], 
     Hue[0.4397746003827427, 1, 1], 
     Hue[0.5416337626792227, 1, 1], 
     Hue[0.6434929247365134, 1, 1], 
     Hue[0.7453520909304452, 1, 1], 
     Hue[0.8472112558675396, 1, 1], 
     Hue[0.9490704184852241, 1, 1], 
     Hue[0.05092958151477587, 1, 1], 
     Hue[0.15278874413246035`, 1, 1], 
     Hue[0.25464790906955476`, 1, 1], 
     Hue[0.3565070752634866, 1, 1], 
     Hue[0.4583662373207773, 1, 1], 
     Hue[0.5602253996172573, 1, 1], 
     Hue[0.662084562429162, 1, 1], 
     Hue[0.7639437274632138, 1, 1], 
     Hue[0.8658028921421332, 1, 1], 
     Hue[0.9676620524876683, 1, 1], 
     Hue[0.06952121788377091, 1, 1], 
     Hue[0.1713803805273081, 1, 1], 
     Hue[0.2732395446020636, 1, 1], 
     Hue[0.3750987115683873, 1, 1], 
     Hue[0.47695787184336774`, 1, 1], 
     Hue[0.578817038986505, 1, 1], 
     Hue[0.6806762023088189, 1, 1], 
     Hue[0.7825353605692787, 1, 1], 
     Hue[0.884394525690152, 1, 1], 
     Hue[0.9862536889027952, 1, 1], 
     Hue[0.0881128530829163, 1, 1], 
     Hue[0.18997201368380215`, 1, 1], 
     Hue[0.2918311838754506, 1, 1], 
     Hue[0.39369034145313603`, 1, 1], 
     Hue[0.4955495078502641, 1, 1], 
     Hue[0.5974086718466087, 1, 1], 
     Hue[0.6992678353233841, 1, 1], 
     Hue[0.801126998147429, 1, 1], 
     Hue[0.9029861613084487, 1, 1], 
     Hue[0.004845325235023987, 1, 1], 
     Hue[0.1067044893654998, 1, 1], 
     Hue[0.2085636519986938, 1, 1], 
     Hue[0.3104228169539898, 1, 1], 
     Hue[0.41228197848916326`, 1, 1], 
     Hue[0.5141411444857782, 1, 1], 
     Hue[0.6160003104901695, 1, 1], 
     Hue[0.7178594722765494, 1, 1], 
     Hue[0.8197186324996953, 1, 1], 
     Hue[0.9215777941909739, 1, 1], 
     Hue[0.02343696238670696, 1, 1], 
     Hue[0.1252961237555336, 1, 1], 
     Hue[0.2271552890346943, 1, 1], 
     Hue[0.32901445094794335`, 1, 1], 
     Hue[0.43087361809934777`, 1, 1], 
     Hue[0.5327327802452424, 1, 1], 
     Hue[0.6345919419118717, 1, 1], 
     Hue[0.736451107195464, 1, 1], 
     Hue[0.8383102675915894, 1, 1], 
     Hue[0.9401694331949104, 1, 1], 
     Hue[0.042028596081378496`, 1, 1], 
     Hue[0.143887761134637, 1, 1], 
     Hue[0.24574692390835293`, 1, 1], 
     Hue[0.3476060900256417, 1, 1], 
     Hue[0.4494652500868171, 1, 1], 
     Hue[0.5513244158951738, 1, 1], 
     Hue[0.6531835736908052, 1, 1], 
     Hue[0.7550427419614101, 1, 1], 
     Hue[0.8569019034490182, 1, 1], 
     Hue[0.9587610682684513, 1, 1], 
     Hue[0.060620233206281154`, 1, 1], 
     Hue[0.16247939410148296`, 1, 1], 
     Hue[0.2643385598483031, 1, 1], 
     Hue[0.3661977210045819, 1, 1], 
     Hue[0.4680568867372577, 1, 1], 
     Hue[0.5699160517664983, 1, 1], 
     Hue[0.6717752158714863, 1, 1], 
     Hue[0.7736343773628815, 1, 1], 
     Hue[0.8754935383776598, 1, 1], 
     Hue[0.9773527053475997, 1, 1], 
     Hue[0.07921187154410292, 1, 1], 
     Hue[0.18107103176047457`, 1, 1], 
     Hue[0.2829301959870124, 1, 1], 
     Hue[0.3847893572618031, 1, 1], 
     Hue[0.48664852296885064`, 1, 1], 
     Hue[0.5885076890195262, 1, 1], 
     Hue[0.6903668523618101, 1, 1], 
     Hue[0.7922260129636224, 1, 1], 
     Hue[0.8940851796772397, 1, 1], 
     Hue[0.99594434093879, 1, 1], 
     Hue[0.0978035039647874, 1, 1], 
     Hue[0.19966266872886562`, 1, 1], 
     Hue[0.30152183135520416`, 1, 1], 
     Hue[0.40338099665443905`, 1, 1], 
     Hue[0.5052401592430923, 1, 1], 
     Hue[0.607099323428438, 1, 1], 
     Hue[0.7089584849626256, 1, 1], 
     Hue[0.8108176506926813, 1, 1], 
     Hue[0.9126768176019221, 1, 1], 
     Hue[0.014535976785237121`, 1, 1], 
     Hue[0.11639513558854607`, 1, 1], 
     Hue[0.21825430575895977`, 1, 1], 
     Hue[0.32011346404796626`, 1, 1], 
     Hue[0.4219726297608182, 1, 1], 
     Hue[0.5238317956302203, 1, 1], 
     Hue[0.6256909569427747, 1, 1], 
     Hue[0.727550121081068, 1, 1], 
     Hue[0.8294092864077768, 1, 1], 
     Hue[0.9312684497178493, 1, 1], 
     Hue[0.033127611863875285`, 1, 1], 
     Hue[0.13498677820638982`, 1, 1], 
     Hue[0.2368459401250907, 1, 1], 
     Hue[0.33870510239514673`, 1, 1], 
     Hue[0.4405642648760048, 1, 1], 
     Hue[0.5424234275736929, 1, 1], 
     Hue[0.6442825916003974, 1, 1], 
     Hue[0.746141757651104, 1, 1], 
     Hue[0.8480009182266539, 1, 1], 
     Hue[0.9498600826122677, 1, 1], 
     Hue[0.051719249585379296`, 1, 1], 
     Hue[0.15357841060037475`, 1, 1], 
     Hue[0.25543757548193646`, 1, 1], 
     Hue[0.3572967409946449, 1, 1], 
     Hue[0.45915590232163533`, 1, 1], 
     Hue[0.5610150635373692, 1, 1], 
     Hue[0.6628742293835823, 1, 1], 
     Hue[0.7647333935702304, 1, 1], 
     Hue[0.8665925593341647, 1, 1], 
     Hue[0.9684517188910791, 1, 1], 
     Hue[0.07031088734743306, 1, 1], 
     Hue[0.17217004562512694`, 1, 1], 
     Hue[0.274029210966428, 1, 1], 
     Hue[0.375888373572094, 1, 1], 
     Hue[0.4777475389365056, 1, 1], 
     Hue[0.5796067036319594, 1, 1], 
     Hue[0.6814658687691331, 1, 1], 
     Hue[0.7833250283541157, 1, 1], 
     Hue[0.8851841969163478, 1, 1], 
     Hue[0.987043356640097, 1, 1], 
     Hue[0.08890252571682883, 1, 1], 
     Hue[0.19076168586000714`, 1, 1], 
     Hue[0.292620847373047, 1, 1], 
     Hue[0.39448000715328024`, 1, 1], 
     Hue[0.4963391743031589, 1, 1], 
     Hue[0.5981983383709037, 1, 1], 
     Hue[0.7000575032370594, 1, 1], 
     Hue[0.8019166646042429, 1, 1], 
     Hue[0.9037758261816837, 1, 1], 
     Hue[0.005634991919216007, 1, 1], 
     Hue[0.10749415064042574`, 1, 1], 
     Hue[0.20935332017583547`, 1, 1], 
     Hue[0.3112124825674883, 1, 1], 
     Hue[0.4130716490685422, 1, 1], 
     Hue[0.5149308102038845, 1, 1], 
     Hue[0.616789974021388, 1, 1], 
     Hue[0.7186491354964677, 1, 1], 
     Hue[0.8205082995769946, 1, 1], 
     Hue[0.9223674666126417, 1, 1], 
     Hue[0.02422662897639987, 1, 1], 
     Hue[0.12608578948147126`, 1, 1], 
     Hue[0.22794495533336445`, 1, 1], 
     Hue[0.32980411988616976`, 1, 1], 
     Hue[0.43166328074234533`, 1, 1], 
     Hue[0.5335224458588795, 1, 1], 
     Hue[0.6353816097101509, 1, 1], 
     Hue[0.7372407724843723, 1, 1], 
     Hue[0.8390999401640373, 1, 1], 
     Hue[0.9409591005084146, 1, 1], 
     Hue[0.042818261174902755`, 1, 1], 
     Hue[0.14467743035764916`, 1, 1], 
     Hue[0.24653659073009973`, 1, 1], 
     Hue[0.34839574974108056`, 1, 1], 
     Hue[0.45025491685569347`, 1, 1], 
     Hue[0.552114080374339, 1, 1], 
     Hue[0.6539732460042567, 1, 1], 
     Hue[0.7558324145141517, 1, 1], 
     Hue[0.8576915934514485, 1, 1], 
     Hue[0.9595507628714376, 1, 1], 
     Hue[0.061409821696748426`, 1, 1], 
     Hue[0.16326931675960346`, 1, 1], 
     Hue[0.2651289529572075, 1, 1], 
     Hue[0.3669882808535222, 1, 1], 
     Hue[0.46884142783573884`, 1, 1], 
     Hue[0.5707343955276603, 1, 1], 
     Hue[0.6725813560157026, 1, 1], 
     Hue[0.7741541746595132, 1, 1], 
     Hue[0.8763374050162176, 1, 1], 
     Hue[0.9765917527181093, 1, 1], 
     Hue[0.07379180882521663, 1, 1], 
     Hue[0.19879180882521666`, 1, 1], 
     Hue[0.25, 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0., 1, 1], 
     Hue[0.18028136750030463`, 1, 1], 
     Hue[0.2821405277234505, 1, 1], 
     Hue[0.2821405277234505, 1, 1], 
     Hue[0.3839996895098306, 1, 1], 
     Hue[0.3839996895098306, 1, 1], 
     Hue[0.48585885551422175`, 1, 1], 
     Hue[0.48585885551422175`, 1, 1], 
     Hue[0.5877180215108367, 1, 1], 
     Hue[0.5877180215108367, 1, 1], 
     Hue[0.6895771830460102, 1, 1], 
     Hue[0.6895771830460102, 1, 1], 
     Hue[0.7914363480013062, 1, 1], 
     Hue[0.7914363480013062, 1, 1], 
     Hue[0.8932955106345002, 1, 1], 
     Hue[0.8932955106345002, 1, 1], 
     Hue[0.995154674764976, 1, 1], 
     Hue[0.995154674764976, 1, 1], 
     Hue[0.09701383869155131, 1, 1], 
     Hue[0.09701383869155131, 1, 1], 
     Hue[0.19887300185257092`, 1, 1], 
     Hue[0.19887300185257092`, 1, 1], 
     Hue[0.3007321646766159, 1, 1], 
     Hue[0.3007321646766159, 1, 1], 
     Hue[0.4025913281533913, 1, 1], 
     Hue[0.4025913281533913, 1, 1], 
     Hue[0.5044504921497359, 1, 1], 
     Hue[0.5044504921497359, 1, 1], 
     Hue[0.6063096585468639, 1, 1], 
     Hue[0.6063096585468639, 1, 1], 
     Hue[0.7081688161245494, 1, 1], 
     Hue[0.7081688161245494, 1, 1], 
     Hue[0.8100279863161979, 1, 1], 
     Hue[0.8100279863161979, 1, 1], 
     Hue[0.9118871469170837, 1, 1], 
     Hue[0.9118871469170837, 1, 1], 
     Hue[0.013746311097204865`, 1, 1], 
     Hue[0.013746311097204865`, 1, 1], 
     Hue[0.11560547430984798`, 1, 1], 
     Hue[0.11560547430984798`, 1, 1], 
     Hue[0.21746463943072125`, 1, 1], 
     Hue[0.21746463943072125`, 1, 1], 
     Hue[0.31932379769118113`, 1, 1], 
     Hue[0.31932379769118113`, 1, 1], 
     Hue[0.42118296101349495`, 1, 1], 
     Hue[0.42118296101349495`, 1, 1], 
     Hue[0.5230421281566322, 1, 1], 
     Hue[0.5230421281566322, 1, 1], 
     Hue[0.6249012884316127, 1, 1], 
     Hue[0.6249012884316127, 1, 1], 
     Hue[0.7267604553979363, 1, 1], 
     Hue[0.7267604553979363, 1, 1], 
     Hue[0.8286196194726919, 1, 1], 
     Hue[0.8286196194726919, 1, 1], 
     Hue[0.930478782116229, 1, 1], 
     Hue[0.930478782116229, 1, 1], 
     Hue[0.03233794751233168, 1, 1], 
     Hue[0.03233794751233168, 1, 1], 
     Hue[0.13419710785786682`, 1, 1], 
     Hue[0.13419710785786682`, 1, 1], 
     Hue[0.2360562725367862, 1, 1], 
     Hue[0.2360562725367862, 1, 1], 
     Hue[0.337915437570838, 1, 1], 
     Hue[0.337915437570838, 1, 1], 
     Hue[0.4397746003827427, 1, 1], 
     Hue[0.4397746003827427, 1, 1], 
     Hue[0.5416337626792227, 1, 1], 
     Hue[0.5416337626792227, 1, 1], 
     Hue[0.6434929247365134, 1, 1], 
     Hue[0.6434929247365134, 1, 1], 
     Hue[0.7453520909304452, 1, 1], 
     Hue[0.7453520909304452, 1, 1], 
     Hue[0.8472112558675396, 1, 1], 
     Hue[0.8472112558675396, 1, 1], 
     Hue[0.9490704184852241, 1, 1], 
     Hue[0.9490704184852241, 1, 1], 
     Hue[0.05092958151477587, 1, 1], 
     Hue[0.05092958151477587, 1, 1], 
     Hue[0.15278874413246035`, 1, 1], 
     Hue[0.15278874413246035`, 1, 1], 
     Hue[0.25464790906955476`, 1, 1], 
     Hue[0.25464790906955476`, 1, 1], 
     Hue[0.3565070752634866, 1, 1], 
     Hue[0.3565070752634866, 1, 1], 
     Hue[0.4583662373207773, 1, 1], 
     Hue[0.4583662373207773, 1, 1], 
     Hue[0.5602253996172573, 1, 1], 
     Hue[0.5602253996172573, 1, 1], 
     Hue[0.662084562429162, 1, 1], 
     Hue[0.662084562429162, 1, 1], 
     Hue[0.7639437274632138, 1, 1], 
     Hue[0.7639437274632138, 1, 1], 
     Hue[0.8658028921421332, 1, 1], 
     Hue[0.8658028921421332, 1, 1], 
     Hue[0.9676620524876683, 1, 1], 
     Hue[0.9676620524876683, 1, 1], 
     Hue[0.06952121788377091, 1, 1], 
     Hue[0.06952121788377091, 1, 1], 
     Hue[0.1713803805273081, 1, 1], 
     Hue[0.1713803805273081, 1, 1], 
     Hue[0.2732395446020636, 1, 1], 
     Hue[0.2732395446020636, 1, 1], 
     Hue[0.3750987115683873, 1, 1], 
     Hue[0.3750987115683873, 1, 1], 
     Hue[0.47695787184336774`, 1, 1], 
     Hue[0.47695787184336774`, 1, 1], 
     Hue[0.578817038986505, 1, 1], 
     Hue[0.578817038986505, 1, 1], 
     Hue[0.6806762023088189, 1, 1], 
     Hue[0.6806762023088189, 1, 1], 
     Hue[0.7825353605692787, 1, 1], 
     Hue[0.7825353605692787, 1, 1], 
     Hue[0.884394525690152, 1, 1], 
     Hue[0.884394525690152, 1, 1], 
     Hue[0.9862536889027952, 1, 1], 
     Hue[0.9862536889027952, 1, 1], 
     Hue[0.0881128530829163, 1, 1], 
     Hue[0.0881128530829163, 1, 1], 
     Hue[0.18997201368380215`, 1, 1], 
     Hue[0.18997201368380215`, 1, 1], 
     Hue[0.2918311838754506, 1, 1], 
     Hue[0.2918311838754506, 1, 1], 
     Hue[0.39369034145313603`, 1, 1], 
     Hue[0.39369034145313603`, 1, 1], 
     Hue[0.4955495078502641, 1, 1], 
     Hue[0.4955495078502641, 1, 1], 
     Hue[0.5974086718466087, 1, 1], 
     Hue[0.5974086718466087, 1, 1], 
     Hue[0.6992678353233841, 1, 1], 
     Hue[0.6992678353233841, 1, 1], 
     Hue[0.801126998147429, 1, 1], 
     Hue[0.801126998147429, 1, 1], 
     Hue[0.9029861613084487, 1, 1], 
     Hue[0.9029861613084487, 1, 1], 
     Hue[0.004845325235023987, 1, 1], 
     Hue[0.004845325235023987, 1, 1], 
     Hue[0.1067044893654998, 1, 1], 
     Hue[0.1067044893654998, 1, 1], 
     Hue[0.2085636519986938, 1, 1], 
     Hue[0.2085636519986938, 1, 1], 
     Hue[0.3104228169539898, 1, 1], 
     Hue[0.3104228169539898, 1, 1], 
     Hue[0.41228197848916326`, 1, 1], 
     Hue[0.41228197848916326`, 1, 1], 
     Hue[0.5141411444857782, 1, 1], 
     Hue[0.5141411444857782, 1, 1], 
     Hue[0.6160003104901695, 1, 1], 
     Hue[0.6160003104901695, 1, 1], 
     Hue[0.7178594722765494, 1, 1], 
     Hue[0.7178594722765494, 1, 1], 
     Hue[0.8197186324996953, 1, 1], 
     Hue[0.8197186324996953, 1, 1], 
     Hue[0.9215777941909739, 1, 1], 
     Hue[0.9215777941909739, 1, 1], 
     Hue[0.02343696238670696, 1, 1], 
     Hue[0.02343696238670696, 1, 1], 
     Hue[0.1252961237555336, 1, 1], 
     Hue[0.1252961237555336, 1, 1], 
     Hue[0.2271552890346943, 1, 1], 
     Hue[0.2271552890346943, 1, 1], 
     Hue[0.32901445094794335`, 1, 1], 
     Hue[0.32901445094794335`, 1, 1], 
     Hue[0.43087361809934777`, 1, 1], 
     Hue[0.43087361809934777`, 1, 1], 
     Hue[0.5327327802452424, 1, 1], 
     Hue[0.5327327802452424, 1, 1], 
     Hue[0.6345919419118717, 1, 1], 
     Hue[0.6345919419118717, 1, 1], 
     Hue[0.736451107195464, 1, 1], 
     Hue[0.736451107195464, 1, 1], 
     Hue[0.8383102675915894, 1, 1], 
     Hue[0.8383102675915894, 1, 1], 
     Hue[0.9401694331949104, 1, 1], 
     Hue[0.9401694331949104, 1, 1], 
     Hue[0.042028596081378496`, 1, 1], 
     Hue[0.042028596081378496`, 1, 1], 
     Hue[0.143887761134637, 1, 1], 
     Hue[0.143887761134637, 1, 1], 
     Hue[0.24574692390835293`, 1, 1], 
     Hue[0.24574692390835293`, 1, 1], 
     Hue[0.3476060900256417, 1, 1], 
     Hue[0.3476060900256417, 1, 1], 
     Hue[0.4494652500868171, 1, 1], 
     Hue[0.4494652500868171, 1, 1], 
     Hue[0.5513244158951738, 1, 1], 
     Hue[0.5513244158951738, 1, 1], 
     Hue[0.6531835736908052, 1, 1], 
     Hue[0.6531835736908052, 1, 1], 
     Hue[0.7550427419614101, 1, 1], 
     Hue[0.7550427419614101, 1, 1], 
     Hue[0.8569019034490182, 1, 1], 
     Hue[0.8569019034490182, 1, 1], 
     Hue[0.9587610682684513, 1, 1], 
     Hue[0.9587610682684513, 1, 1], 
     Hue[0.060620233206281154`, 1, 1], 
     Hue[0.060620233206281154`, 1, 1], 
     Hue[0.16247939410148296`, 1, 1], 
     Hue[0.16247939410148296`, 1, 1], 
     Hue[0.2643385598483031, 1, 1], 
     Hue[0.2643385598483031, 1, 1], 
     Hue[0.3661977210045819, 1, 1], 
     Hue[0.3661977210045819, 1, 1], 
     Hue[0.4680568867372577, 1, 1], 
     Hue[0.4680568867372577, 1, 1], 
     Hue[0.5699160517664983, 1, 1], 
     Hue[0.5699160517664983, 1, 1], 
     Hue[0.6717752158714863, 1, 1], 
     Hue[0.6717752158714863, 1, 1], 
     Hue[0.7736343773628815, 1, 1], 
     Hue[0.7736343773628815, 1, 1], 
     Hue[0.8754935383776598, 1, 1], 
     Hue[0.8754935383776598, 1, 1], 
     Hue[0.9773527053475997, 1, 1], 
     Hue[0.9773527053475997, 1, 1], 
     Hue[0.07921187154410292, 1, 1], 
     Hue[0.07921187154410292, 1, 1], 
     Hue[0.18107103176047457`, 1, 1], 
     Hue[0.18107103176047457`, 1, 1], 
     Hue[0.2829301959870124, 1, 1], 
     Hue[0.2829301959870124, 1, 1], 
     Hue[0.3847893572618031, 1, 1], 
     Hue[0.3847893572618031, 1, 1], 
     Hue[0.48664852296885064`, 1, 1], 
     Hue[0.48664852296885064`, 1, 1], 
     Hue[0.5885076890195262, 1, 1], 
     Hue[0.5885076890195262, 1, 1], 
     Hue[0.6903668523618101, 1, 1], 
     Hue[0.6903668523618101, 1, 1], 
     Hue[0.7922260129636224, 1, 1], 
     Hue[0.7922260129636224, 1, 1], 
     Hue[0.8940851796772397, 1, 1], 
     Hue[0.8940851796772397, 1, 1], 
     Hue[0.99594434093879, 1, 1], 
     Hue[0.99594434093879, 1, 1], 
     Hue[0.0978035039647874, 1, 1], 
     Hue[0.0978035039647874, 1, 1], 
     Hue[0.19966266872886562`, 1, 1], 
     Hue[0.19966266872886562`, 1, 1], 
     Hue[0.30152183135520416`, 1, 1], 
     Hue[0.30152183135520416`, 1, 1], 
     Hue[0.40338099665443905`, 1, 1], 
     Hue[0.40338099665443905`, 1, 1], 
     Hue[0.5052401592430923, 1, 1], 
     Hue[0.5052401592430923, 1, 1], 
     Hue[0.607099323428438, 1, 1], 
     Hue[0.607099323428438, 1, 1], 
     Hue[0.7089584849626256, 1, 1], 
     Hue[0.7089584849626256, 1, 1], 
     Hue[0.8108176506926813, 1, 1], 
     Hue[0.8108176506926813, 1, 1], 
     Hue[0.9126768176019221, 1, 1], 
     Hue[0.9126768176019221, 1, 1], 
     Hue[0.014535976785237121`, 1, 1], 
     Hue[0.014535976785237121`, 1, 1], 
     Hue[0.11639513558854607`, 1, 1], 
     Hue[0.11639513558854607`, 1, 1], 
     Hue[0.21825430575895977`, 1, 1], 
     Hue[0.21825430575895977`, 1, 1], 
     Hue[0.32011346404796626`, 1, 1], 
     Hue[0.32011346404796626`, 1, 1], 
     Hue[0.4219726297608182, 1, 1], 
     Hue[0.4219726297608182, 1, 1], 
     Hue[0.5238317956302203, 1, 1], 
     Hue[0.5238317956302203, 1, 1], 
     Hue[0.6256909569427747, 1, 1], 
     Hue[0.6256909569427747, 1, 1], 
     Hue[0.727550121081068, 1, 1], 
     Hue[0.727550121081068, 1, 1], 
     Hue[0.8294092864077768, 1, 1], 
     Hue[0.8294092864077768, 1, 1], 
     Hue[0.9312684497178493, 1, 1], 
     Hue[0.9312684497178493, 1, 1], 
     Hue[0.033127611863875285`, 1, 1], 
     Hue[0.033127611863875285`, 1, 1], 
     Hue[0.13498677820638982`, 1, 1], 
     Hue[0.13498677820638982`, 1, 1], 
     Hue[0.2368459401250907, 1, 1], 
     Hue[0.2368459401250907, 1, 1], 
     Hue[0.33870510239514673`, 1, 1], 
     Hue[0.33870510239514673`, 1, 1], 
     Hue[0.4405642648760048, 1, 1], 
     Hue[0.4405642648760048, 1, 1], 
     Hue[0.5424234275736929, 1, 1], 
     Hue[0.5424234275736929, 1, 1], 
     Hue[0.6442825916003974, 1, 1], 
     Hue[0.6442825916003974, 1, 1], 
     Hue[0.746141757651104, 1, 1], 
     Hue[0.746141757651104, 1, 1]}],
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{True, None},
  AxesOrigin->{0, 0},
  Epilog->LineBox[{{1, 0.008344251069322926}, {2, 0.010573731905873832`}, {
     3, 0.013313423380879645`}, {4, 0.01665603741682225}, {
     5, 0.020704949549814704`}, {6, 0.02557391225420548}, {
     7, 0.03138634136491761}, {8, 0.038274075789681894`}, {
     9, 0.046375571230388754`}, {10, 0.055833432579869925`}, {
     11, 0.06679129900034367}, {12, 0.07939003112744925}, {
     13, 0.09376323557999186}, {14, 0.11003217552197057`}, {
     15, 0.12830019890127162`}, {16, 0.1486467634839321}, {
     17, 0.1711213190678146}, {18, 0.1957371688625169}, {
     19, 0.22246566795063305`}, {20, 0.25123100117898584`}, {
     21, 0.2819057626833841}, {22, 0.3143078247026415}, {
     23, 0.3481985253966046}, {24, 0.3832826759524978}, {
     25, 0.4192103375959412}, {26, 0.455580676508156}, {
     27, 0.49194790742962247`}, {28, 0.5278292368598958}, {
     29, 0.5627147242367263}, {30, 0.5960787691507571}, {
     31, 0.6273927666338232}, {32, 0.6561390928833624}, {
     33, 0.6818248530176858}, {34, 0.7039960941100264}, {
     35, 0.7222510684686774}, {36, 0.736252246358251}, {
     37, 0.7457368736488047}, {38, 0.750524896903731}, {
     39, 0.750524896903731}, {40, 0.7457368736488047}, {
     41, 0.736252246358251}, {42, 0.7222510684686774}, {
     43, 0.7039960941100264}, {44, 0.6818248530176858}, {
     45, 0.6561390928833624}, {46, 0.6273927666338232}, {
     47, 0.5960787691507571}, {48, 0.5627147242367263}, {
     49, 0.5278292368598958}, {50, 0.49194790742962247`}, {
     51, 0.455580676508156}, {52, 0.4192103375959412}, {
     53, 0.3832826759524978}, {54, 0.3481985253966046}, {
     55, 0.3143078247026415}, {56, 0.2819057626833841}, {
     57, 0.25123100117898584`}, {58, 0.22246566795063305`}, {
     59, 0.1957371688625169}, {60, 0.1711213190678146}, {
     61, 0.1486467634839321}, {62, 0.12830019890127162`}, {
     63, 0.11003217552197057`}, {64, 0.09376323557999186}, {
     65, 0.07939003112744925}, {66, 0.06679129900034367}, {
     67, 0.055833432579869925`}, {68, 0.046375571230388754`}, {
     69, 0.038274075789681894`}, {70, 0.03138634136491761}, {
     71, 0.02557391225420548}, {72, 0.020704949549814704`}, {
     73, 0.01665603741682225}, {74, 0.013313423380879645`}, {
     75, 0.010573731905873832`}, {76, 0.008344251069322926}, {
     77, 0.006542849672610582}, {78, 0.005097614975915271}, {
     79, 0.003946277830663838}, {80, 0.0030354898912649288`}, {
     81, 0.002320013023736215}, {82, 0.0017618648329987114`}, {
     83, 0.0013294599546701431`}, {84, 0.0009967779806008895}, {
     85, 0.0007425781501585884}, {86, 0.0005496755020206345}, {
     87, 0.0004042882330186218}, {88, 0.0002954584252093653}, {
     89, 0.00021454683807871014`}, {90, 0.00015479908368949057`}, {
     91, 0.00011097756956021392`}, {92, 0.0000790537553056834}, {
     93, 0.00005595389555743419}, {94, 0.00003935126272092477}, {
     95, 0.00002749840786675889}, {96, 0.000019093120390417035`}, {
     97, 0.000013172456984449016`}, {98, 9.029780604909846*^-6}, {
     99, 6.150467423622445*^-6}, {100, 4.162550991525008*^-6}, {
     101, 2.7991844236848377`*^-6}, {102, 1.8703546611464252`*^-6}, {
     103, 1.2417579120725904`*^-6}, {104, 8.191632426761341*^-7}, {
     105, 5.369384146916037*^-7}, {106, 3.497027022894919*^-7}, {
     107, 2.2630494193975768`*^-7}, {108, 1.4551560932784334`*^-7}, {
     109, 9.297060463772275*^-8}, {110, 5.902041200308431*^-8}, {
     111, 3.722882613294182*^-8}, {112, 2.333334317123441*^-8}, {
     113, 1.4530986153819031`*^-8}, {114, 8.991533106601375*^-9}, {
     115, 5.528316737604801*^-9}, {116, 3.3773230113519837`*^-9}, {
     117, 2.0500893243344816`*^-9}, {118, 1.236498196940082*^-9}, {
     119, 7.410281830304226*^-10}, {120, 4.412619317730651*^-10}, {
     121, 2.6108307854555324`*^-10}, {122, 1.534904888019036*^-10}, {
     123, 8.966124766886046*^-11}, {124, 5.204135271301691*^-11}, {
     125, 3.001324016094829*^-11}, {126, 1.7198783945555636`*^-11}, {
     127, 9.792713897473725*^-12}, {128, 5.5402444689948545`*^-12}, {
     129, 3.1144067137712433`*^-12}, {130, 1.7395712218376893`*^-12}, {
     131, 9.65449600713352*^-13}, {132, 5.323993618595665*^-13}, {
     133, 2.9171984270610186`*^-13}, {134, 1.5882356227341697`*^-13}, {
     135, 8.591804693824838*^-14}, {136, 4.618217669505971*^-14}, {
     137, 2.4665214910893163`*^-14}, {138, 1.3089284430168242`*^-14}, {
     139, 6.90188057405099*^-15}, {140, 3.616092197562592*^-15}, {
     141, 1.8824872407545214`*^-15}, {142, 9.73744772401734*^-16}, {
     143, 5.004707796674516*^-16}, {144, 2.555835115880366*^-16}, {
     145, 1.2969028752602597`*^-16}, {146, 6.538868185472437*^-17}, {
     147, 3.275806488180611*^-17}, {148, 1.6306262346309655`*^-17}, {
     149, 8.065125086332684*^-18}, {150, 3.963586216936936*^-18}, {
     151, 1.935468047847275*^-18}, {152, 9.39083615568533*^-19}, {
     153, 4.527338616399333*^-19}, {154, 2.168713857139742*^-19}, {
     155, 1.0322432159252343`*^-19}, {156, 4.8818261438306306`*^-20}, {
     157, 2.2940512560498628`*^-20}, {158, 1.0711355860916956`*^-20}, {
     159, 4.9694268860552594`*^-21}, {160, 2.2908077741352386`*^-21}, {
     161, 1.0492803234403497`*^-21}, {162, 4.775456422785383*^-22}, {
     163, 2.159527796515092*^-22}, {164, 9.70338388975237*^-23}, {
     165, 4.332196514791527*^-23}, {166, 1.92182389297254*^-23}, {
     167, 8.471094580540771*^-24}, {168, 3.710103589758693*^-24}, {
     169, 1.6145556665390757`*^-24}, {170, 6.981367744865415*^-25}, {
     171, 2.9994978003471004`*^-25}, {172, 1.2804926092179273`*^-25}, {
     173, 5.431579298213634*^-26}, {174, 2.289262975184917*^-26}, {
     175, 9.587067366269083*^-27}, {176, 3.989297246320632*^-27}, {
     177, 1.6494056905941098`*^-27}, {178, 6.776089200620111*^-28}, {
     179, 2.7659940867097817`*^-28}, {180, 1.121873580717982*^-28}, {
     181, 4.5212350378199956`*^-29}, {182, 1.810467701443014*^-29}, {
     183, 7.20352348469477*^-30}, {184, 2.8478670551429574`*^-30}, {
     185, 1.1187034171767422`*^-30}, {186, 4.366471998516355*^-31}, {
     187, 1.6934289259538306`*^-31}, {188, 6.525649813779407*^-32}, {
     189, 2.498624758017261*^-32}, {190, 9.506023014512616*^-33}, {
     191, 3.593496137939883*^-33}, {192, 1.349758346283483*^-33}, {
     193, 5.037503678261213*^-34}, {194, 1.868079091926022*^-34}, {
     195, 6.883283532194499*^-35}, {196, 2.520092973812606*^-35}, {
     197, 9.16764902366263*^-36}, {198, 3.31375118532769*^-36}, {
     199, 1.1901518389630339`*^-36}, {200, 4.247225381990478*^-37}, {
     201, 1.5060133255110523`*^-37}, {202, 5.306067920534574*^-38}, {
     203, 1.857536285561286*^-38}, {204, 6.46133616889376*^-39}, {
     205, 2.233201412465634*^-39}, {206, 7.669260615709758*^-40}, {
     207, 2.6169846844346205`*^-40}, {208, 8.873002905884784*^-41}, {
     209, 2.989270865400848*^-41}, {210, 1.0005833992080868`*^-41}, {
     211, 3.327656650579037*^-42}, {212, 1.1001085455535052`*^-42}, {
     213, 3.614861179861282*^-43}, {214, 1.1791741318019692`*^-43}, {
     215, 3.824800455786841*^-44}, {216, 1.253359449198542*^-44}, {
     217, 4.431294828962626*^-45}, {218, 1.401298464324817*^-45}, {219, 0.}, {
     220, 0.}, {221, 0.}, {222, 0.}, {223, 0.}, {224, 0.}, {225, 0.}, {
     226, 0.}, {227, 0.}, {228, 0.}, {229, 0.}, {230, 0.}, {231, 0.}, {
     232, 0.}, {233, 0.}, {234, 0.}, {235, 0.}, {236, 0.}, {237, 0.}, {
     238, 0.}, {239, 0.}, {240, 0.}, {241, 0.}, {242, 0.}, {243, 0.}, {
     244, 0.}, {245, 0.}, {246, 0.}, {247, 0.}, {248, 0.}, {249, 0.}, {
     250, 0.}, {251, 0.}}],
  Frame->True,
  FrameTicks->{{{1., 
      FormBox[
       RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
      FormBox[
       RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
      FormBox[
       RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
      FormBox[
       RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
      FormBox["0", TraditionalForm]}, {157.25, 
      FormBox["2.5`", TraditionalForm]}, {188.5, 
      FormBox["5", TraditionalForm]}, {219.75, 
      FormBox["7.5`", TraditionalForm]}, {251., 
      FormBox["10", TraditionalForm]}}, Automatic, None, None},
  PlotRange->{-0.1, 1.1},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], Automatic}]], "Output",
 CellChangeTimes->{3.391758202843787*^9, 3.391758237011404*^9, 
  3.391758687192425*^9, 3.3917587282060695`*^9, 3.391758836929741*^9, 
  3.391758868299552*^9, 3.3917589535935535`*^9, 3.391758988667722*^9, 
  3.39185015403125*^9, 3.3924724381043367`*^9, 3.392472469095957*^9},
 CellLabel->"Out[62]=",
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Animation", "Section"],

Cell["dt = 0.005;", "Input",
 CellLabel->"In[63]:="],

Cell[TextData[{
 "The time between two of the frames is  20*0.005=0.1",
 StyleBox[":",
  FontWeight->"Bold"]
}], "Text",
 CellChangeTimes->{{3.391850194375*^9, 3.39185019646875*^9}}],

Cell[CellGroupData[{

Cell["qtab=Table[QTimeEvolution[H,psi,dt,0,20]; showIt, {24}]", "Input",
 CellChangeTimes->{{3.391758255079915*^9, 3.391758256439743*^9}, {
  3.3917584523486695`*^9, 3.391758454771351*^9}},
 CellLabel->"In[64]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF2Xl0jXcex/EfghhLU8tUCb0Vu4QsJEHCRxZLgmyyb09uYokgjWliachv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     "], {
      {GrayLevel[0.5], EdgeForm[None], 
       GraphicsGroupBox[{PolygonBox[{{154, 558, 155}}], 
         PolygonBox[CompressedData["
1:eJw1lgV0FWQYhr//jo2xjW2MGtuIjTFghKiIiICFBYg6AZFQHFiohGKBEkqH
dNrd3d3d3d3dLfG859Vz7nPOc/74vj++/95bWT+pbmImIjZAA5Bvhi2Q9R+R
+CT3Z2scng05eC7k4fnQKNyWgzcU/8fEs5LndoAB+J5QE55TgDeGPDwfivEm
UBCmBG+qMWGa4c2hMJwjV7mT11IEpXgrKA5ThpcrZpgKvDWUhHMW4kXJuRWz
Bd4yOZbGtMHbag3KDVV4e60BL4ce+LZQoblQg3dUjDCd8M5QGqYW76I1humK
d4OycIzt8O21xjA98R2gTZhe+I5aE14Nu+N7JJ9tJfTF+0FVeExvfCdoF95D
O7wyeS9q64PvnDxXe6pWrOS9tIdd8d2Scylmf3yX5D7VTVeKphtsDN/pXvTt
rTPQ3mEgPkhnEGYwvh/UhhmC768zUSyoww/SmeDbwAj8EOgRbhuKD4Pu4TkH
4Acmz1XOffB9k3Nrzkh8lO5IZwuH4ofpjMOMxQ+HnmHq8XE683CO4fjByWvp
BUfiR+kOwhyNHwO9w0zAj9WZh3OOxsck51bbcfjx0CfMRHyS7iDMZHwK9NXd
wgx8pu4Y7wdT8ZOgf7htFj4bBoTbTsZP0R3p7mAaPl13GOZ0/Izk2BpzKn5a
8ljVYHftNbm2FfNM/Cy9WXwQLMdXwGCdNSzCF+vMdfcwD5+vGgjPmYPPTe5T
2wJ8YfJc7fEE/MTkvQ2Es/FlybkUcwm+NLlPOVfiq1RDYVbja2BImLX4OtUU
PgwuxS+D4aoV2Iifo5oJcy5+HtSFOR+/QDUY5kL8IhgajnE5foVqIsyV+FUw
IszV+DWqWdUa3I7foZrDR8GN+E0wOjzmWvw6GBnOcTF+SfLa1XY9fkPy3LFw
N36PahYfA7fityXnUsyb8VuS+5TzTvyu5Lmacy9+H9Tr7OF9/APdgWoZnsaf
UU3j4+BB/CEYH257Fn8OJoTbHsYfgSP0NuBx/Am9kTBP4k8lx9aYR/HHkscq
54f4RzA1HPN5/AW9IXwKvIu/l7zWifA6/obejN4SvIy/ojcVnvMi/lJyn9pe
xV9Lnqs9348/kLy3yfA2/k5yLsV8E38ruU/fAeO1t+S3rzV+jH+iN6hahRZ8
97WEZXpL8DV93+hN6S3C5/gXepNhvsS/Sh6rMd/i38E0vU34Gf9FbzRcw+vx
Dcm1q7Zf8d9gBj4T/sT/gln4XGjAWrJhnt4ubKZvi95weMzf+D8wO5zze/wH
mB5u+xfflDxXOX7H/0jONQcyxM7KOJdi6k9DyrhPMX5k7E/Je9EacuhrCPPD
5OKNYAG+HMrwclgRPsNSvFXGfWqrwFvDSnwRFOJFsBhfAiV4U1gaphnePOO7
0ZhivEnGY3Vnn7K2z5LvRjHb0NcWVuHroBPeGdaH15iH58NCfA1U4x1gbXhM
Ld4l499CxWiHV8Lq8JgavGPGsRWjAG+c8V40pgpvn3HspPOHTT7W2AqLWufS

          "]]}]}, {{}, {}, {
        {Hue[0.7749898951612308, 1, 1], LineBox[{1, 2}]}, 
        {Hue[0.8722290584923409, 1, 1], LineBox[{2, 3}]}, 
        {Hue[0.9695841775471524, 1, 1], LineBox[{3, 4}]}, 
        {Hue[0.06702656516515425, 1, 1], LineBox[{4, 5}]}, 
        {Hue[0.16458210327625292`, 1, 1], LineBox[{5, 6}]}, 
        {Hue[0.2622279195690579, 1, 1], LineBox[{6, 7}]}, 
        {Hue[0.35998704518367436`, 1, 1], LineBox[{7, 8}]}, 
        {Hue[0.45783566493262245`, 1, 1], LineBox[{8, 9}]}, 
        {Hue[0.5557890356018238, 1, 1], LineBox[{9, 10}]}, 
        {Hue[0.653844920101932, 1, 1], LineBox[{10, 11}]}, 
        {Hue[0.7519981602293165, 1, 1], LineBox[{11, 12}]}, 
        {Hue[0.8502551726866501, 1, 1], LineBox[{12, 13}]}, 
        {Hue[0.9486085695831875, 1, 1], LineBox[{13, 14}]}, 
        {Hue[0.0470609664999944, 1, 1], LineBox[{14, 15}]}, 
        {Hue[0.14561566828510647`, 1, 1], LineBox[{15, 16}]}, 
        {Hue[0.24427022386525324`, 1, 1], LineBox[{16, 17}]}, 
        {Hue[0.343021643237129, 1, 1], LineBox[{17, 18}]}, 
        {Hue[0.44187379272680516`, 1, 1], LineBox[{18, 19}]}, 
        {Hue[0.5408248371910815, 1, 1], LineBox[{19, 20}]}, 
        {Hue[0.6398744958829277, 1, 1], LineBox[{20, 21}]}, 
        {Hue[0.7390232792464664, 1, 1], LineBox[{21, 22}]}, 
        {Hue[0.838270524168504, 1, 1], LineBox[{22, 23}]}, 
        {Hue[0.9376160928746887, 1, 1], LineBox[{23, 24}]}, 
        {Hue[0.03706054230140248, 1, 1], LineBox[{24, 25}]}, 
        {Hue[0.13660362360586886`, 1, 1], LineBox[{25, 26}]}, 
        {Hue[0.23624319452712514`, 1, 1], LineBox[{26, 27}]}, 
        {Hue[0.33598260881557607`, 1, 1], LineBox[{27, 28}]}, 
        {Hue[0.435818781503373, 1, 1], LineBox[{28, 29}]}, 
        {Hue[0.5357533041230387, 1, 1], LineBox[{29, 30}]}, 
        {Hue[0.6357851273644934, 1, 1], LineBox[{30, 31}]}, 
        {Hue[0.735914881028622, 1, 1], LineBox[{31, 32}]}, 
        {Hue[0.8361419713192397, 1, 1], LineBox[{32, 33}]}, 
        {Hue[0.9364664851620053, 1, 1], LineBox[{33, 34}]}, 
        {Hue[0.03688834509492618, 1, 1], LineBox[{34, 35}]}, 
        {Hue[0.13740705788019347`, 1, 1], LineBox[{35, 36}]}, 
        {Hue[0.23802342482553834`, 1, 1], LineBox[{36, 37}]}, 
        {Hue[0.33873639122288024`, 1, 1], LineBox[{37, 38}]}, 
        {Hue[0.43954626861141466`, 1, 1], LineBox[{38, 39}]}, 
        {Hue[0.5404528403401376, 1, 1], LineBox[{39, 40}]}, 
        {Hue[0.6414564693287275, 1, 1], LineBox[{40, 41}]}, 
        {Hue[0.742557176478602, 1, 1], LineBox[{41, 42}]}, 
        {Hue[0.843754208203586, 1, 1], LineBox[{42, 43}]}, 
        {Hue[0.9450473394371686, 1, 1], LineBox[{43, 44}]}, 
        {Hue[0.04643704926151799, 1, 1], LineBox[{44, 45}]}, 
        {Hue[0.14792346076075114`, 1, 1], LineBox[{45, 46}]}, 
        {Hue[0.24950675009915688`, 1, 1], LineBox[{46, 47}]}, 
        {Hue[0.35118600578268955`, 1, 1], LineBox[{47, 48}]}, 
        {Hue[0.4529611660509637, 1, 1], LineBox[{48, 49}]}, 
        {Hue[0.5548322704627584, 1, 1], LineBox[{49, 50}]}, 
        {Hue[0.6567991697961184, 1, 1], LineBox[{50, 51}]}, 
        {Hue[0.7588625703273734, 1, 1], LineBox[{51, 52}]}, 
        {Hue[0.8610221377534003, 1, 1], LineBox[{52, 53}]}, 
        {Hue[0.9632780091731467, 1, 1], LineBox[{53, 54}]}, 
        {Hue[0.06562985710310025, 1, 1], LineBox[{54, 55}]}, 
        {Hue[0.168077395961652, 1, 1], LineBox[{55, 56}]}, 
        {Hue[0.2706209976066884, 1, 1], LineBox[{56, 57}]}, 
        {Hue[0.37326058880617385`, 1, 1], LineBox[{57, 58}]}, 
        {Hue[0.4759959473707332, 1, 1], LineBox[{58, 59}]}, 
        {Hue[0.5788270998124714, 1, 1], LineBox[{59, 60}]}, 
        {Hue[0.6817539353409436, 1, 1], LineBox[{60, 61}]}, 
        {Hue[0.7847767535259931, 1, 1], LineBox[{61, 62}]}, 
        {Hue[0.8878954180530303, 1, 1], LineBox[{62, 63}]}, 
        {Hue[0.9911097590231914, 1, 1], LineBox[{63, 64}]}, 
        {Hue[0.09441974610041529, 1, 1], LineBox[{64, 65}]}, 
        {Hue[0.1978253627188138, 1, 1], LineBox[{65, 66}]}, 
        {Hue[0.3013268292330902, 1, 1], LineBox[{66, 67}]}, 
        {Hue[0.40492419793388484`, 1, 1], LineBox[{67, 68}]}, 
        {Hue[0.508616503730313, 1, 1], LineBox[{68, 69}]}, 
        {Hue[0.6124040131257059, 1, 1], LineBox[{69, 70}]}, 
        {Hue[0.7162870697903432, 1, 1], LineBox[{70, 71}]}, 
        {Hue[0.820265753877946, 1, 1], LineBox[{71, 72}]}, 
        {Hue[0.9243396644570422, 1, 1], LineBox[{72, 73}]}, 
        {Hue[0.028508445063498762`, 1, 1], LineBox[{73, 74}]}, 
        {Hue[0.13277225503614024`, 1, 1], LineBox[{74, 75}]}, 
        {Hue[0.23713093589427062`, 1, 1], LineBox[{75, 76}]}, 
        {Hue[0.3415839747802137, 1, 1], LineBox[{76, 77}]}, 
        {Hue[0.4461313425988212, 1, 1], LineBox[{77, 78}]}, 
        {Hue[0.5507727312686279, 1, 1], LineBox[{78, 79}]}, 
        {Hue[0.6555078730306969, 1, 1], LineBox[{79, 80}]}, 
        {Hue[0.7603362356798644, 1, 1], LineBox[{80, 81}]}, 
        {Hue[0.8652575837223297, 1, 1], LineBox[{81, 82}]}, 
        {Hue[0.9702710585264879, 1, 1], LineBox[{82, 83}]}, 
        {Hue[0.07537595344508677, 1, 1], LineBox[{83, 84}]}, 
        {Hue[0.1805713877230246, 1, 1], LineBox[{84, 85}]}, 
        {Hue[0.28585628035949945`, 1, 1], LineBox[{85, 86}]}, 
        {Hue[0.3912295774894582, 1, 1], LineBox[{86, 87}]}, 
        {Hue[0.4966904112408541, 1, 1], LineBox[{87, 88}]}, 
        {Hue[0.6022370667295118, 1, 1], LineBox[{88, 89}]}, 
        {Hue[0.70786676887722, 1, 1], LineBox[{89, 90}]}, 
        {Hue[0.8135773679517109, 1, 1], LineBox[{90, 91}]}, 
        {Hue[0.9193669962723475, 1, 1], LineBox[{91, 92}]}, 
        {Hue[0.02523258448171614, 1, 1], LineBox[{92, 93}]}, 
        {Hue[0.1311700458947892, 1, 1], LineBox[{93, 94}]}, 
        {Hue[0.23717520772041145`, 1, 1], LineBox[{94, 95}]}, 
        {Hue[0.3432433458032074, 1, 1], LineBox[{95, 96}]}, 
        {Hue[0.4493686985356077, 1, 1], LineBox[{96, 97}]}, 
        {Hue[0.5555437495804476, 1, 1], LineBox[{97, 98}]}, 
        {Hue[0.6617608642013143, 1, 1], LineBox[{98, 99}]}, 
        {Hue[0.7680114358431309, 1, 1], LineBox[{99, 100}]}, 
        {Hue[0.8742849075770401, 1, 1], LineBox[{100, 101}]}, 
        {Hue[0.9805688072229441, 1, 1], LineBox[{101, 102}]}, 
        {Hue[0.08684850786555587, 1, 1], LineBox[{102, 103}]}, 
        {Hue[0.1931073492585604, 1, 1], LineBox[{103, 104}]}, 
        {Hue[0.29932613581724765`, 1, 1], LineBox[{104, 105}]}, 
        {Hue[0.4054832774981767, 1, 1], LineBox[{105, 106}]}, 
        {Hue[0.511554075856528, 1, 1], LineBox[{106, 107}]}, 
        {Hue[0.6175095361091129, 1, 1], LineBox[{107, 108}]}, 
        {Hue[0.7233171354656784, 1, 1], LineBox[{108, 109}]}, 
        {Hue[0.8289397327072737, 1, 1], LineBox[{109, 110}]}, 
        {Hue[0.9343363087113633, 1, 1], LineBox[{110, 111}]}, 
        {Hue[0.03946140402520984, 1, 1], LineBox[{111, 112}]}, 
        {Hue[0.14426430790282302`, 1, 1], LineBox[{112, 113}]}, 
        {Hue[0.2486887623010038, 1, 1], LineBox[{113, 114}]}, 
        {Hue[0.3526741834381149, 1, 1], LineBox[{114, 115}]}, 
        {Hue[0.45615598834782883`, 1, 1], LineBox[{115, 116}]}, 
        {Hue[0.5590664490818031, 1, 1], LineBox[{116, 117}]}, 
        {Hue[0.6613355896608559, 1, 1], LineBox[{117, 118}]}, 
        {Hue[0.7628925659823964, 1, 1], LineBox[{118, 119}]}, 
        {Hue[0.8636679499759634, 1, 1], LineBox[{119, 120}]}, 
        {Hue[0.9635960997077214, 1, 1], LineBox[{120, 121}]}, 
        {Hue[0.06261731240532682, 1, 1], LineBox[{121, 122}]}, 
        {Hue[0.16067962802358593`, 1, 1], LineBox[{122, 123}]}, 
        {Hue[0.257741847061097, 1, 1], LineBox[{123, 124}]}, 
        {Hue[0.3537773726657826, 1, 1], LineBox[{124, 125}]}, 
        {Hue[0.44877573855416353`, 1, 1], LineBox[{125, 126}]}, 
        {Hue[0.5427446454772279, 1, 1], LineBox[{126, 127}]}, 
        {Hue[0.6357130551987016, 1, 1], LineBox[{127, 128}]}, 
        {Hue[0.7277350307086694, 1, 1], LineBox[{128, 129}]}, 
        {Hue[0.818891906069626, 1, 1], LineBox[{129, 130}]}, 
        {Hue[0.9092929010507722, 1, 1], LineBox[{130, 131}]}, 
        {Hue[0.9990790665473557, 1, 1], LineBox[{131, 132}]}, 
        {Hue[0.0884304839703989, 1, 1], LineBox[{132, 133}]}, 
        {Hue[0.17756605313563623`, 1, 1], LineBox[{133, 134}]}, 
        {Hue[0.2667301868473516, 1, 1], LineBox[{134, 135}]}, 
        {Hue[0.3561835023519964, 1, 1], LineBox[{135, 136}]}, 
        {Hue[0.4461975135842855, 1, 1], LineBox[{136, 137}]}, 
        {Hue[0.5370394232765678, 1, 1], LineBox[{137, 138}]}, 
        {Hue[0.6289504058124807, 1, 1], LineBox[{138, 139}]}, 
        {Hue[0.722133989077812, 1, 1], LineBox[{139, 140}]}, 
        {Hue[0.8167482696906572, 1, 1], LineBox[{140, 141}]}, 
        {Hue[0.9128958389165969, 1, 1], LineBox[{141, 142}]}, 
        {Hue[0.01062320557995812, 1, 1], LineBox[{142, 143}]}, 
        {Hue[0.10993026899221506`, 1, 1], LineBox[{143, 144}]}, 
        {Hue[0.2107766388559164, 1, 1], LineBox[{144, 145}]}, 
        {Hue[0.31309037761082614`, 1, 1], LineBox[{145, 146}]}, 
        {Hue[0.4167761395354911, 1, 1], LineBox[{146, 147}]}, 
        {Hue[0.5217208944670482, 1, 1], LineBox[{147, 148}]}, 
        {Hue[0.6277983545596304, 1, 1], LineBox[{148, 149}]}, 
        {Hue[0.7348794471728705, 1, 1], LineBox[{149, 150}]}, 
        {Hue[0.8428413068614792, 1, 1], LineBox[{150, 151}]}, 
        {Hue[0.9515712395853337, 1, 1], LineBox[{151, 152}]}, 
        {Hue[0.06096695036510244, 1, 1], LineBox[{152, 153}]}, 
        {Hue[0.1709356742763382, 1, 1], LineBox[{153, 154}]}, 
        {Hue[0.28139292300740415`, 1, 1], LineBox[{154, 155}]}, 
        {Hue[0.39226373560099825`, 1, 1], LineBox[{155, 156}]}, 
        {Hue[0.503488967576359, 1, 1], LineBox[{156, 157}]}, 
        {Hue[0.6150271693063969, 1, 1], LineBox[{157, 158}]}, 
        {Hue[0.7268457566459958, 1, 1], LineBox[{158, 159}]}, 
        {Hue[0.8389106407489604, 1, 1], LineBox[{159, 160}]}, 
        {Hue[0.9511904422560358, 1, 1], LineBox[{160, 161}]}, 
        {Hue[0.06366457578475905, 1, 1], LineBox[{161, 162}]}, 
        {Hue[0.17632399770969037`, 1, 1], LineBox[{162, 163}]}, 
        {Hue[0.2891634352733916, 1, 1], LineBox[{163, 164}]}, 
        {Hue[0.40217446463777096`, 1, 1], LineBox[{164, 165}]}, 
        {Hue[0.5153428538195237, 1, 1], LineBox[{165, 166}]}, 
        {Hue[0.6286535390852653, 1, 1], LineBox[{166, 167}]}, 
        {Hue[0.7420989550247571, 1, 1], LineBox[{167, 168}]}, 
        {Hue[0.8556824988993403, 1, 1], LineBox[{168, 169}]}, 
        {Hue[0.9694108265582719, 1, 1], LineBox[{169, 170}]}, 
        {Hue[0.08328308408885278, 1, 1], LineBox[{170, 171}]}, 
        {Hue[0.197289107027446, 1, 1], LineBox[{171, 172}]}, 
        {Hue[0.3114172647845141, 1, 1], LineBox[{172, 173}]}, 
        {Hue[0.425663848694229, 1, 1], LineBox[{173, 174}]}, 
        {Hue[0.5400364449850104, 1, 1], LineBox[{174, 175}]}, 
        {Hue[0.654545337569995, 1, 1], LineBox[{175, 176}]}, 
        {Hue[0.7691919589545142, 1, 1], LineBox[{176, 177}]}, 
        {Hue[0.8839673278665687, 1, 1], LineBox[{177, 178}]}, 
        {Hue[0.9988648083453614, 1, 1], LineBox[{178, 179}]}, 
        {Hue[0.11389045041208445`, 1, 1], LineBox[{179, 180}]}, 
        {Hue[0.22905317250681512`, 1, 1], LineBox[{180, 181}]}, 
        {Hue[0.3443477222842044, 1, 1], LineBox[{181, 182}]}, 
        {Hue[0.4597564467478896, 1, 1], LineBox[{182, 183}]}, 
        {Hue[0.575272954437088, 1, 1], LineBox[{183, 184}]}, 
        {Hue[0.690916850151059, 1, 1], LineBox[{184, 185}]}, 
        {Hue[0.8067172895240435, 1, 1], LineBox[{185, 186}]}, 
        {Hue[0.9226830304114683, 1, 1], LineBox[{186, 187}]}, 
        {Hue[0.03879193317028482, 1, 1], LineBox[{187, 188}]}, 
        {Hue[0.15501007233955133`, 1, 1], LineBox[{188, 189}]}, 
        {Hue[0.271322652888202, 1, 1], LineBox[{189, 190}]}, 
        {Hue[0.38774699933720297`, 1, 1], LineBox[{190, 191}]}, 
        {Hue[0.5043184018206746, 1, 1], LineBox[{191, 192}]}, 
        {Hue[0.6210616490432017, 1, 1], LineBox[{192, 193}]}, 
        {Hue[0.7379741223799546, 1, 1], LineBox[{193, 194}]}, 
        {Hue[0.8550330753268756, 1, 1], LineBox[{194, 195}]}, 
        {Hue[0.9722146191777067, 1, 1], LineBox[{195, 196}]}, 
        {Hue[0.08950548352353373, 1, 1], LineBox[{196, 197}]}, 
        {Hue[0.20690376138675443`, 1, 1], LineBox[{197, 198}]}, 
        {Hue[0.3244227059977963, 1, 1], LineBox[{198, 199}]}, 
        {Hue[0.44209693973826225`, 1, 1], LineBox[{199, 200}]}, 
        {Hue[0.5599691644159934, 1, 1], LineBox[{200, 201}]}, 
        {Hue[0.6780539150001488, 1, 1], LineBox[{201, 202}]}, 
        {Hue[0.7963115534339857, 1, 1], LineBox[{202, 203}]}, 
        {Hue[0.9146706065985621, 1, 1], LineBox[{203, 204}]}, 
        {Hue[0.03308838080242426, 1, 1], LineBox[{204, 205}]}, 
        {Hue[0.1515920280512786, 1, 1], LineBox[{205, 206}]}, 
        {Hue[0.27025289101267647`, 1, 1], LineBox[{206, 207}]}, 
        {Hue[0.389119757508248, 1, 1], LineBox[{207, 208}]}, 
        {Hue[0.5081848430282914, 1, 1], LineBox[{208, 209}]}, 
        {Hue[0.627414082696829, 1, 1], LineBox[{209, 210}]}, 
        {Hue[0.7467971716276445, 1, 1], LineBox[{210, 211}]}, 
        {Hue[0.8663546614564116, 1, 1], LineBox[{211, 212}]}, 
        {Hue[0.9860997934088671, 1, 1], LineBox[{212, 213}]}, 
        {Hue[0.10600667585355196`, 1, 1], LineBox[{213, 214}]}, 
        {Hue[0.2260244737159512, 1, 1], LineBox[{214, 215}]}, 
        {Hue[0.3461272749252381, 1, 1], LineBox[{215, 216}]}, 
        {Hue[0.466347997981795, 1, 1], LineBox[{216, 217}]}, 
        {Hue[0.5867510597322951, 1, 1], LineBox[{217, 218}]}, 
        {Hue[0.7073591620063187, 1, 1], LineBox[{218, 219}]}, 
        {Hue[0.8281193177149418, 1, 1], LineBox[{219, 220}]}, 
        {Hue[0.9489791169151242, 1, 1], LineBox[{220, 221}]}, 
        {Hue[0.06995961206559426, 1, 1], LineBox[{221, 222}]}, 
        {Hue[0.19134767685266463`, 1, 1], LineBox[{222, 223}]}, 
        {Hue[0.3131695453007627, 1, 1], LineBox[{223, 224}]}, 
        {Hue[0.43461463065006734`, 1, 1], LineBox[{224, 225}]}, 
        {Hue[0.5587062218899088, 1, 1], LineBox[{225, 226}]}, 
        {Hue[0.7011321539939399, 1, 1], LineBox[{226, 227}]}, 
        {Hue[0.8279791303773694, 1, 1], LineBox[{227, 228}]}, 
        {Hue[0.8573883562613613, 1, 1], LineBox[{228, 229}]}, 
        {Hue[0.8656487203793617, 1, 1], LineBox[{229, 230}]}, 
        {Hue[0.8656487203793617, 1, 1], LineBox[{230, 231}]}, 
        {Hue[0.8656487203793617, 1, 1], LineBox[{231, 232}]}, 
        {Hue[0.8656487203793617, 1, 1], LineBox[{232, 233}]}, 
        {Hue[0.8656487203793617, 1, 1], LineBox[{233, 234}]}, 
        {Hue[0.8656487203793617, 1, 1], LineBox[{234, 235}]}, 
        {Hue[0.8656487203793617, 1, 1], LineBox[{235, 236}]}, 
        {Hue[0.8656487203793617, 1, 1], LineBox[{236, 237}]}, 
        {Hue[0.8656487203793617, 1, 1], LineBox[{237, 238}]}, 
        {Hue[0.8656487203793617, 1, 1], LineBox[{238, 239}]}, 
        {Hue[0.8656487203793617, 1, 1], LineBox[{239, 240}]}, 
        {Hue[0.8656487203793617, 1, 1], LineBox[{240, 241}]}, 
        {Hue[0.8656487203793617, 1, 1], LineBox[{241, 242}]}, 
        {Hue[0.8656487203793617, 1, 1], LineBox[{242, 243}]}, 
        {Hue[0.8656487203793617, 1, 1], LineBox[{243, 244}]}, 
        {Hue[0.8656487203793617, 1, 1], LineBox[{244, 245}]}, 
        {Hue[0.8656487203793617, 1, 1], LineBox[{245, 246}]}, 
        {Hue[0.8656487203793617, 1, 1], LineBox[{246, 247}]}, 
        {Hue[0.8656487203793617, 1, 1], LineBox[{247, 248}]}, 
        {Hue[0.8656487203793617, 1, 1], LineBox[{248, 249}]}, 
        {Hue[0.8656487203793617, 1, 1], LineBox[{249, 250}]}, 
        {Hue[0.8656487203793617, 1, 1], LineBox[{250, 251}]}}}},
     VertexColors->{
       Hue[0.7749898951612308, 1, 1], 
       Hue[0.8722290584923409, 1, 1], 
       Hue[0.9695841775471524, 1, 1], 
       Hue[0.06702656516515425, 1, 1], 
       Hue[0.16458210327625292`, 1, 1], 
       Hue[0.2622279195690579, 1, 1], 
       Hue[0.35998704518367436`, 1, 1], 
       Hue[0.45783566493262245`, 1, 1], 
       Hue[0.5557890356018238, 1, 1], 
       Hue[0.653844920101932, 1, 1], 
       Hue[0.7519981602293165, 1, 1], 
       Hue[0.8502551726866501, 1, 1], 
       Hue[0.9486085695831875, 1, 1], 
       Hue[0.0470609664999944, 1, 1], 
       Hue[0.14561566828510647`, 1, 1], 
       Hue[0.24427022386525324`, 1, 1], 
       Hue[0.343021643237129, 1, 1], 
       Hue[0.44187379272680516`, 1, 1], 
       Hue[0.5408248371910815, 1, 1], 
       Hue[0.6398744958829277, 1, 1], 
       Hue[0.7390232792464664, 1, 1], 
       Hue[0.838270524168504, 1, 1], 
       Hue[0.9376160928746887, 1, 1], 
       Hue[0.03706054230140248, 1, 1], 
       Hue[0.13660362360586886`, 1, 1], 
       Hue[0.23624319452712514`, 1, 1], 
       Hue[0.33598260881557607`, 1, 1], 
       Hue[0.435818781503373, 1, 1], 
       Hue[0.5357533041230387, 1, 1], 
       Hue[0.6357851273644934, 1, 1], 
       Hue[0.735914881028622, 1, 1], 
       Hue[0.8361419713192397, 1, 1], 
       Hue[0.9364664851620053, 1, 1], 
       Hue[0.03688834509492618, 1, 1], 
       Hue[0.13740705788019347`, 1, 1], 
       Hue[0.23802342482553834`, 1, 1], 
       Hue[0.33873639122288024`, 1, 1], 
       Hue[0.43954626861141466`, 1, 1], 
       Hue[0.5404528403401376, 1, 1], 
       Hue[0.6414564693287275, 1, 1], 
       Hue[0.742557176478602, 1, 1], 
       Hue[0.843754208203586, 1, 1], 
       Hue[0.9450473394371686, 1, 1], 
       Hue[0.04643704926151799, 1, 1], 
       Hue[0.14792346076075114`, 1, 1], 
       Hue[0.24950675009915688`, 1, 1], 
       Hue[0.35118600578268955`, 1, 1], 
       Hue[0.4529611660509637, 1, 1], 
       Hue[0.5548322704627584, 1, 1], 
       Hue[0.6567991697961184, 1, 1], 
       Hue[0.7588625703273734, 1, 1], 
       Hue[0.8610221377534003, 1, 1], 
       Hue[0.9632780091731467, 1, 1], 
       Hue[0.06562985710310025, 1, 1], 
       Hue[0.168077395961652, 1, 1], 
       Hue[0.2706209976066884, 1, 1], 
       Hue[0.37326058880617385`, 1, 1], 
       Hue[0.4759959473707332, 1, 1], 
       Hue[0.5788270998124714, 1, 1], 
       Hue[0.6817539353409436, 1, 1], 
       Hue[0.7847767535259931, 1, 1], 
       Hue[0.8878954180530303, 1, 1], 
       Hue[0.9911097590231914, 1, 1], 
       Hue[0.09441974610041529, 1, 1], 
       Hue[0.1978253627188138, 1, 1], 
       Hue[0.3013268292330902, 1, 1], 
       Hue[0.40492419793388484`, 1, 1], 
       Hue[0.508616503730313, 1, 1], 
       Hue[0.6124040131257059, 1, 1], 
       Hue[0.7162870697903432, 1, 1], 
       Hue[0.820265753877946, 1, 1], 
       Hue[0.9243396644570422, 1, 1], 
       Hue[0.028508445063498762`, 1, 1], 
       Hue[0.13277225503614024`, 1, 1], 
       Hue[0.23713093589427062`, 1, 1], 
       Hue[0.3415839747802137, 1, 1], 
       Hue[0.4461313425988212, 1, 1], 
       Hue[0.5507727312686279, 1, 1], 
       Hue[0.6555078730306969, 1, 1], 
       Hue[0.7603362356798644, 1, 1], 
       Hue[0.8652575837223297, 1, 1], 
       Hue[0.9702710585264879, 1, 1], 
       Hue[0.07537595344508677, 1, 1], 
       Hue[0.1805713877230246, 1, 1], 
       Hue[0.28585628035949945`, 1, 1], 
       Hue[0.3912295774894582, 1, 1], 
       Hue[0.4966904112408541, 1, 1], 
       Hue[0.6022370667295118, 1, 1], 
       Hue[0.70786676887722, 1, 1], 
       Hue[0.8135773679517109, 1, 1], 
       Hue[0.9193669962723475, 1, 1], 
       Hue[0.02523258448171614, 1, 1], 
       Hue[0.1311700458947892, 1, 1], 
       Hue[0.23717520772041145`, 1, 1], 
       Hue[0.3432433458032074, 1, 1], 
       Hue[0.4493686985356077, 1, 1], 
       Hue[0.5555437495804476, 1, 1], 
       Hue[0.6617608642013143, 1, 1], 
       Hue[0.7680114358431309, 1, 1], 
       Hue[0.8742849075770401, 1, 1], 
       Hue[0.9805688072229441, 1, 1], 
       Hue[0.08684850786555587, 1, 1], 
       Hue[0.1931073492585604, 1, 1], 
       Hue[0.29932613581724765`, 1, 1], 
       Hue[0.4054832774981767, 1, 1], 
       Hue[0.511554075856528, 1, 1], 
       Hue[0.6175095361091129, 1, 1], 
       Hue[0.7233171354656784, 1, 1], 
       Hue[0.8289397327072737, 1, 1], 
       Hue[0.9343363087113633, 1, 1], 
       Hue[0.03946140402520984, 1, 1], 
       Hue[0.14426430790282302`, 1, 1], 
       Hue[0.2486887623010038, 1, 1], 
       Hue[0.3526741834381149, 1, 1], 
       Hue[0.45615598834782883`, 1, 1], 
       Hue[0.5590664490818031, 1, 1], 
       Hue[0.6613355896608559, 1, 1], 
       Hue[0.7628925659823964, 1, 1], 
       Hue[0.8636679499759634, 1, 1], 
       Hue[0.9635960997077214, 1, 1], 
       Hue[0.06261731240532682, 1, 1], 
       Hue[0.16067962802358593`, 1, 1], 
       Hue[0.257741847061097, 1, 1], 
       Hue[0.3537773726657826, 1, 1], 
       Hue[0.44877573855416353`, 1, 1], 
       Hue[0.5427446454772279, 1, 1], 
       Hue[0.6357130551987016, 1, 1], 
       Hue[0.7277350307086694, 1, 1], 
       Hue[0.818891906069626, 1, 1], 
       Hue[0.9092929010507722, 1, 1], 
       Hue[0.9990790665473557, 1, 1], 
       Hue[0.0884304839703989, 1, 1], 
       Hue[0.17756605313563623`, 1, 1], 
       Hue[0.2667301868473516, 1, 1], 
       Hue[0.3561835023519964, 1, 1], 
       Hue[0.4461975135842855, 1, 1], 
       Hue[0.5370394232765678, 1, 1], 
       Hue[0.6289504058124807, 1, 1], 
       Hue[0.722133989077812, 1, 1], 
       Hue[0.8167482696906572, 1, 1], 
       Hue[0.9128958389165969, 1, 1], 
       Hue[0.01062320557995812, 1, 1], 
       Hue[0.10993026899221506`, 1, 1], 
       Hue[0.2107766388559164, 1, 1], 
       Hue[0.31309037761082614`, 1, 1], 
       Hue[0.4167761395354911, 1, 1], 
       Hue[0.5217208944670482, 1, 1], 
       Hue[0.6277983545596304, 1, 1], 
       Hue[0.7348794471728705, 1, 1], 
       Hue[0.8428413068614792, 1, 1], 
       Hue[0.9515712395853337, 1, 1], 
       Hue[0.06096695036510244, 1, 1], 
       Hue[0.1709356742763382, 1, 1], 
       Hue[0.28139292300740415`, 1, 1], 
       Hue[0.39226373560099825`, 1, 1], 
       Hue[0.503488967576359, 1, 1], 
       Hue[0.6150271693063969, 1, 1], 
       Hue[0.7268457566459958, 1, 1], 
       Hue[0.8389106407489604, 1, 1], 
       Hue[0.9511904422560358, 1, 1], 
       Hue[0.06366457578475905, 1, 1], 
       Hue[0.17632399770969037`, 1, 1], 
       Hue[0.2891634352733916, 1, 1], 
       Hue[0.40217446463777096`, 1, 1], 
       Hue[0.5153428538195237, 1, 1], 
       Hue[0.6286535390852653, 1, 1], 
       Hue[0.7420989550247571, 1, 1], 
       Hue[0.8556824988993403, 1, 1], 
       Hue[0.9694108265582719, 1, 1], 
       Hue[0.08328308408885278, 1, 1], 
       Hue[0.197289107027446, 1, 1], 
       Hue[0.3114172647845141, 1, 1], 
       Hue[0.425663848694229, 1, 1], 
       Hue[0.5400364449850104, 1, 1], 
       Hue[0.654545337569995, 1, 1], 
       Hue[0.7691919589545142, 1, 1], 
       Hue[0.8839673278665687, 1, 1], 
       Hue[0.9988648083453614, 1, 1], 
       Hue[0.11389045041208445`, 1, 1], 
       Hue[0.22905317250681512`, 1, 1], 
       Hue[0.3443477222842044, 1, 1], 
       Hue[0.4597564467478896, 1, 1], 
       Hue[0.575272954437088, 1, 1], 
       Hue[0.690916850151059, 1, 1], 
       Hue[0.8067172895240435, 1, 1], 
       Hue[0.9226830304114683, 1, 1], 
       Hue[0.03879193317028482, 1, 1], 
       Hue[0.15501007233955133`, 1, 1], 
       Hue[0.271322652888202, 1, 1], 
       Hue[0.38774699933720297`, 1, 1], 
       Hue[0.5043184018206746, 1, 1], 
       Hue[0.6210616490432017, 1, 1], 
       Hue[0.7379741223799546, 1, 1], 
       Hue[0.8550330753268756, 1, 1], 
       Hue[0.9722146191777067, 1, 1], 
       Hue[0.08950548352353373, 1, 1], 
       Hue[0.20690376138675443`, 1, 1], 
       Hue[0.3244227059977963, 1, 1], 
       Hue[0.44209693973826225`, 1, 1], 
       Hue[0.5599691644159934, 1, 1], 
       Hue[0.6780539150001488, 1, 1], 
       Hue[0.7963115534339857, 1, 1], 
       Hue[0.9146706065985621, 1, 1], 
       Hue[0.03308838080242426, 1, 1], 
       Hue[0.1515920280512786, 1, 1], 
       Hue[0.27025289101267647`, 1, 1], 
       Hue[0.389119757508248, 1, 1], 
       Hue[0.5081848430282914, 1, 1], 
       Hue[0.627414082696829, 1, 1], 
       Hue[0.7467971716276445, 1, 1], 
       Hue[0.8663546614564116, 1, 1], 
       Hue[0.9860997934088671, 1, 1], 
       Hue[0.10600667585355196`, 1, 1], 
       Hue[0.2260244737159512, 1, 1], 
       Hue[0.3461272749252381, 1, 1], 
       Hue[0.466347997981795, 1, 1], 
       Hue[0.5867510597322951, 1, 1], 
       Hue[0.7073591620063187, 1, 1], 
       Hue[0.8281193177149418, 1, 1], 
       Hue[0.9489791169151242, 1, 1], 
       Hue[0.06995961206559426, 1, 1], 
       Hue[0.19134767685266463`, 1, 1], 
       Hue[0.3131695453007627, 1, 1], 
       Hue[0.43461463065006734`, 1, 1], 
       Hue[0.5587062218899088, 1, 1], 
       Hue[0.7011321539939399, 1, 1], 
       Hue[0.8279791303773694, 1, 1], 
       Hue[0.8573883562613613, 1, 1], 
       Hue[0.8656487203793617, 1, 1], 
       Hue[0.8656487203793617, 1, 1], 
       Hue[0.8656487203793617, 1, 1], 
       Hue[0.8656487203793617, 1, 1], 
       Hue[0.8656487203793617, 1, 1], 
       Hue[0.8656487203793617, 1, 1], 
       Hue[0.8656487203793617, 1, 1], 
       Hue[0.8656487203793617, 1, 1], 
       Hue[0.8656487203793617, 1, 1], 
       Hue[0.8656487203793617, 1, 1], 
       Hue[0.8656487203793617, 1, 1], 
       Hue[0.8656487203793617, 1, 1], 
       Hue[0.8656487203793617, 1, 1], 
       Hue[0.8656487203793617, 1, 1], 
       Hue[0.8656487203793617, 1, 1], 
       Hue[0.8656487203793617, 1, 1], 
       Hue[0.8656487203793617, 1, 1], 
       Hue[0.8656487203793617, 1, 1], 
       Hue[0.8656487203793617, 1, 1], 
       Hue[0.8656487203793617, 1, 1], 
       Hue[0.8656487203793617, 1, 1], 
       Hue[0.8656487203793617, 1, 1], 
       Hue[0.8656487203793617, 1, 1], 
       Hue[0.7749898951612308, 1, 1], 
       Hue[0.8722290584923409, 1, 1], 
       Hue[0.8722290584923409, 1, 1], 
       Hue[0.9695841775471524, 1, 1], 
       Hue[0.9695841775471524, 1, 1], 
       Hue[0.06702656516515425, 1, 1], 
       Hue[0.06702656516515425, 1, 1], 
       Hue[0.16458210327625292`, 1, 1], 
       Hue[0.16458210327625292`, 1, 1], 
       Hue[0.2622279195690579, 1, 1], 
       Hue[0.2622279195690579, 1, 1], 
       Hue[0.35998704518367436`, 1, 1], 
       Hue[0.35998704518367436`, 1, 1], 
       Hue[0.45783566493262245`, 1, 1], 
       Hue[0.45783566493262245`, 1, 1], 
       Hue[0.5557890356018238, 1, 1], 
       Hue[0.5557890356018238, 1, 1], 
       Hue[0.653844920101932, 1, 1], 
       Hue[0.653844920101932, 1, 1], 
       Hue[0.7519981602293165, 1, 1], 
       Hue[0.7519981602293165, 1, 1], 
       Hue[0.8502551726866501, 1, 1], 
       Hue[0.8502551726866501, 1, 1], 
       Hue[0.9486085695831875, 1, 1], 
       Hue[0.9486085695831875, 1, 1], 
       Hue[0.0470609664999944, 1, 1], 
       Hue[0.0470609664999944, 1, 1], 
       Hue[0.14561566828510647`, 1, 1], 
       Hue[0.14561566828510647`, 1, 1], 
       Hue[0.24427022386525324`, 1, 1], 
       Hue[0.24427022386525324`, 1, 1], 
       Hue[0.343021643237129, 1, 1], 
       Hue[0.343021643237129, 1, 1], 
       Hue[0.44187379272680516`, 1, 1], 
       Hue[0.44187379272680516`, 1, 1], 
       Hue[0.5408248371910815, 1, 1], 
       Hue[0.5408248371910815, 1, 1], 
       Hue[0.6398744958829277, 1, 1], 
       Hue[0.6398744958829277, 1, 1], 
       Hue[0.7390232792464664, 1, 1], 
       Hue[0.7390232792464664, 1, 1], 
       Hue[0.838270524168504, 1, 1], 
       Hue[0.838270524168504, 1, 1], 
       Hue[0.9376160928746887, 1, 1], 
       Hue[0.9376160928746887, 1, 1], 
       Hue[0.03706054230140248, 1, 1], 
       Hue[0.03706054230140248, 1, 1], 
       Hue[0.13660362360586886`, 1, 1], 
       Hue[0.13660362360586886`, 1, 1], 
       Hue[0.23624319452712514`, 1, 1], 
       Hue[0.23624319452712514`, 1, 1], 
       Hue[0.33598260881557607`, 1, 1], 
       Hue[0.33598260881557607`, 1, 1], 
       Hue[0.435818781503373, 1, 1], 
       Hue[0.435818781503373, 1, 1], 
       Hue[0.5357533041230387, 1, 1], 
       Hue[0.5357533041230387, 1, 1], 
       Hue[0.6357851273644934, 1, 1], 
       Hue[0.6357851273644934, 1, 1], 
       Hue[0.735914881028622, 1, 1], 
       Hue[0.735914881028622, 1, 1], 
       Hue[0.8361419713192397, 1, 1], 
       Hue[0.8361419713192397, 1, 1], 
       Hue[0.9364664851620053, 1, 1], 
       Hue[0.9364664851620053, 1, 1], 
       Hue[0.03688834509492618, 1, 1], 
       Hue[0.03688834509492618, 1, 1], 
       Hue[0.13740705788019347`, 1, 1], 
       Hue[0.13740705788019347`, 1, 1], 
       Hue[0.23802342482553834`, 1, 1], 
       Hue[0.23802342482553834`, 1, 1], 
       Hue[0.33873639122288024`, 1, 1], 
       Hue[0.33873639122288024`, 1, 1], 
       Hue[0.43954626861141466`, 1, 1], 
       Hue[0.43954626861141466`, 1, 1], 
       Hue[0.5404528403401376, 1, 1], 
       Hue[0.5404528403401376, 1, 1], 
       Hue[0.6414564693287275, 1, 1], 
       Hue[0.6414564693287275, 1, 1], 
       Hue[0.742557176478602, 1, 1], 
       Hue[0.742557176478602, 1, 1], 
       Hue[0.843754208203586, 1, 1], 
       Hue[0.843754208203586, 1, 1], 
       Hue[0.9450473394371686, 1, 1], 
       Hue[0.9450473394371686, 1, 1], 
       Hue[0.04643704926151799, 1, 1], 
       Hue[0.04643704926151799, 1, 1], 
       Hue[0.14792346076075114`, 1, 1], 
       Hue[0.14792346076075114`, 1, 1], 
       Hue[0.24950675009915688`, 1, 1], 
       Hue[0.24950675009915688`, 1, 1], 
       Hue[0.35118600578268955`, 1, 1], 
       Hue[0.35118600578268955`, 1, 1], 
       Hue[0.4529611660509637, 1, 1], 
       Hue[0.4529611660509637, 1, 1], 
       Hue[0.5548322704627584, 1, 1], 
       Hue[0.5548322704627584, 1, 1], 
       Hue[0.6567991697961184, 1, 1], 
       Hue[0.6567991697961184, 1, 1], 
       Hue[0.7588625703273734, 1, 1], 
       Hue[0.7588625703273734, 1, 1], 
       Hue[0.8610221377534003, 1, 1], 
       Hue[0.8610221377534003, 1, 1], 
       Hue[0.9632780091731467, 1, 1], 
       Hue[0.9632780091731467, 1, 1], 
       Hue[0.06562985710310025, 1, 1], 
       Hue[0.06562985710310025, 1, 1], 
       Hue[0.168077395961652, 1, 1], 
       Hue[0.168077395961652, 1, 1], 
       Hue[0.2706209976066884, 1, 1], 
       Hue[0.2706209976066884, 1, 1], 
       Hue[0.37326058880617385`, 1, 1], 
       Hue[0.37326058880617385`, 1, 1], 
       Hue[0.4759959473707332, 1, 1], 
       Hue[0.4759959473707332, 1, 1], 
       Hue[0.5788270998124714, 1, 1], 
       Hue[0.5788270998124714, 1, 1], 
       Hue[0.6817539353409436, 1, 1], 
       Hue[0.6817539353409436, 1, 1], 
       Hue[0.7847767535259931, 1, 1], 
       Hue[0.7847767535259931, 1, 1], 
       Hue[0.8878954180530303, 1, 1], 
       Hue[0.8878954180530303, 1, 1], 
       Hue[0.9911097590231914, 1, 1], 
       Hue[0.9911097590231914, 1, 1], 
       Hue[0.09441974610041529, 1, 1], 
       Hue[0.09441974610041529, 1, 1], 
       Hue[0.1978253627188138, 1, 1], 
       Hue[0.1978253627188138, 1, 1], 
       Hue[0.3013268292330902, 1, 1], 
       Hue[0.3013268292330902, 1, 1], 
       Hue[0.40492419793388484`, 1, 1], 
       Hue[0.40492419793388484`, 1, 1], 
       Hue[0.508616503730313, 1, 1], 
       Hue[0.508616503730313, 1, 1], 
       Hue[0.6124040131257059, 1, 1], 
       Hue[0.6124040131257059, 1, 1], 
       Hue[0.7162870697903432, 1, 1], 
       Hue[0.7162870697903432, 1, 1], 
       Hue[0.820265753877946, 1, 1], 
       Hue[0.820265753877946, 1, 1], 
       Hue[0.9243396644570422, 1, 1], 
       Hue[0.9243396644570422, 1, 1], 
       Hue[0.028508445063498762`, 1, 1], 
       Hue[0.028508445063498762`, 1, 1], 
       Hue[0.13277225503614024`, 1, 1], 
       Hue[0.13277225503614024`, 1, 1], 
       Hue[0.23713093589427062`, 1, 1], 
       Hue[0.23713093589427062`, 1, 1], 
       Hue[0.3415839747802137, 1, 1], 
       Hue[0.3415839747802137, 1, 1], 
       Hue[0.4461313425988212, 1, 1], 
       Hue[0.4461313425988212, 1, 1], 
       Hue[0.5507727312686279, 1, 1], 
       Hue[0.5507727312686279, 1, 1], 
       Hue[0.6555078730306969, 1, 1], 
       Hue[0.6555078730306969, 1, 1], 
       Hue[0.7603362356798644, 1, 1], 
       Hue[0.7603362356798644, 1, 1], 
       Hue[0.8652575837223297, 1, 1], 
       Hue[0.8652575837223297, 1, 1], 
       Hue[0.9702710585264879, 1, 1], 
       Hue[0.9702710585264879, 1, 1], 
       Hue[0.07537595344508677, 1, 1], 
       Hue[0.07537595344508677, 1, 1], 
       Hue[0.1805713877230246, 1, 1], 
       Hue[0.1805713877230246, 1, 1], 
       Hue[0.28585628035949945`, 1, 1], 
       Hue[0.28585628035949945`, 1, 1], 
       Hue[0.3912295774894582, 1, 1], 
       Hue[0.3912295774894582, 1, 1], 
       Hue[0.4966904112408541, 1, 1], 
       Hue[0.4966904112408541, 1, 1], 
       Hue[0.6022370667295118, 1, 1], 
       Hue[0.6022370667295118, 1, 1], 
       Hue[0.70786676887722, 1, 1], 
       Hue[0.70786676887722, 1, 1], 
       Hue[0.8135773679517109, 1, 1], 
       Hue[0.8135773679517109, 1, 1], 
       Hue[0.9193669962723475, 1, 1], 
       Hue[0.9193669962723475, 1, 1], 
       Hue[0.02523258448171614, 1, 1], 
       Hue[0.02523258448171614, 1, 1], 
       Hue[0.1311700458947892, 1, 1], 
       Hue[0.1311700458947892, 1, 1], 
       Hue[0.23717520772041145`, 1, 1], 
       Hue[0.23717520772041145`, 1, 1], 
       Hue[0.3432433458032074, 1, 1], 
       Hue[0.3432433458032074, 1, 1], 
       Hue[0.4493686985356077, 1, 1], 
       Hue[0.4493686985356077, 1, 1], 
       Hue[0.5555437495804476, 1, 1], 
       Hue[0.5555437495804476, 1, 1], 
       Hue[0.6617608642013143, 1, 1], 
       Hue[0.6617608642013143, 1, 1], 
       Hue[0.7680114358431309, 1, 1], 
       Hue[0.7680114358431309, 1, 1], 
       Hue[0.8742849075770401, 1, 1], 
       Hue[0.8742849075770401, 1, 1], 
       Hue[0.9805688072229441, 1, 1], 
       Hue[0.9805688072229441, 1, 1], 
       Hue[0.08684850786555587, 1, 1], 
       Hue[0.08684850786555587, 1, 1], 
       Hue[0.1931073492585604, 1, 1], 
       Hue[0.1931073492585604, 1, 1], 
       Hue[0.29932613581724765`, 1, 1], 
       Hue[0.29932613581724765`, 1, 1], 
       Hue[0.4054832774981767, 1, 1], 
       Hue[0.4054832774981767, 1, 1], 
       Hue[0.511554075856528, 1, 1], 
       Hue[0.511554075856528, 1, 1], 
       Hue[0.6175095361091129, 1, 1], 
       Hue[0.6175095361091129, 1, 1], 
       Hue[0.7233171354656784, 1, 1], 
       Hue[0.7233171354656784, 1, 1], 
       Hue[0.8289397327072737, 1, 1], 
       Hue[0.8289397327072737, 1, 1], 
       Hue[0.9343363087113633, 1, 1], 
       Hue[0.9343363087113633, 1, 1], 
       Hue[0.03946140402520984, 1, 1], 
       Hue[0.03946140402520984, 1, 1], 
       Hue[0.14426430790282302`, 1, 1], 
       Hue[0.14426430790282302`, 1, 1], 
       Hue[0.2486887623010038, 1, 1], 
       Hue[0.2486887623010038, 1, 1], 
       Hue[0.3526741834381149, 1, 1], 
       Hue[0.3526741834381149, 1, 1], 
       Hue[0.45615598834782883`, 1, 1], 
       Hue[0.45615598834782883`, 1, 1], 
       Hue[0.5590664490818031, 1, 1], 
       Hue[0.5590664490818031, 1, 1], 
       Hue[0.6613355896608559, 1, 1], 
       Hue[0.6613355896608559, 1, 1], 
       Hue[0.7628925659823964, 1, 1], 
       Hue[0.7628925659823964, 1, 1], 
       Hue[0.8636679499759634, 1, 1], 
       Hue[0.8636679499759634, 1, 1], 
       Hue[0.9635960997077214, 1, 1], 
       Hue[0.9635960997077214, 1, 1], 
       Hue[0.06261731240532682, 1, 1], 
       Hue[0.06261731240532682, 1, 1], 
       Hue[0.16067962802358593`, 1, 1], 
       Hue[0.16067962802358593`, 1, 1], 
       Hue[0.257741847061097, 1, 1], 
       Hue[0.257741847061097, 1, 1], 
       Hue[0.3537773726657826, 1, 1], 
       Hue[0.3537773726657826, 1, 1], 
       Hue[0.44877573855416353`, 1, 1], 
       Hue[0.44877573855416353`, 1, 1], 
       Hue[0.5427446454772279, 1, 1], 
       Hue[0.5427446454772279, 1, 1], 
       Hue[0.6357130551987016, 1, 1], 
       Hue[0.6357130551987016, 1, 1], 
       Hue[0.7277350307086694, 1, 1], 
       Hue[0.7277350307086694, 1, 1], 
       Hue[0.818891906069626, 1, 1], 
       Hue[0.818891906069626, 1, 1], 
       Hue[0.9092929010507722, 1, 1], 
       Hue[0.9092929010507722, 1, 1], 
       Hue[0.9990790665473557, 1, 1], 
       Hue[0.9990790665473557, 1, 1], 
       Hue[0.0884304839703989, 1, 1], 
       Hue[0.0884304839703989, 1, 1], 
       Hue[0.17756605313563623`, 1, 1], 
       Hue[0.17756605313563623`, 1, 1], 
       Hue[0.2667301868473516, 1, 1], 
       Hue[0.2667301868473516, 1, 1], 
       Hue[0.3561835023519964, 1, 1], 
       Hue[0.3561835023519964, 1, 1], 
       Hue[0.4461975135842855, 1, 1], 
       Hue[0.4461975135842855, 1, 1], 
       Hue[0.5370394232765678, 1, 1], 
       Hue[0.5370394232765678, 1, 1], 
       Hue[0.6289504058124807, 1, 1], 
       Hue[0.6289504058124807, 1, 1], 
       Hue[0.722133989077812, 1, 1], 
       Hue[0.722133989077812, 1, 1], 
       Hue[0.8167482696906572, 1, 1], 
       Hue[0.8167482696906572, 1, 1], 
       Hue[0.9128958389165969, 1, 1], 
       Hue[0.9128958389165969, 1, 1], 
       Hue[0.01062320557995812, 1, 1], 
       Hue[0.01062320557995812, 1, 1], 
       Hue[0.10993026899221506`, 1, 1], 
       Hue[0.10993026899221506`, 1, 1], 
       Hue[0.2107766388559164, 1, 1], 
       Hue[0.2107766388559164, 1, 1], 
       Hue[0.31309037761082614`, 1, 1], 
       Hue[0.31309037761082614`, 1, 1], 
       Hue[0.4167761395354911, 1, 1], 
       Hue[0.4167761395354911, 1, 1], 
       Hue[0.5217208944670482, 1, 1], 
       Hue[0.5217208944670482, 1, 1], 
       Hue[0.6277983545596304, 1, 1], 
       Hue[0.6277983545596304, 1, 1], 
       Hue[0.7348794471728705, 1, 1], 
       Hue[0.7348794471728705, 1, 1], 
       Hue[0.8428413068614792, 1, 1], 
       Hue[0.8428413068614792, 1, 1], 
       Hue[0.9515712395853337, 1, 1], 
       Hue[0.9515712395853337, 1, 1], 
       Hue[0.06096695036510244, 1, 1], 
       Hue[0.06096695036510244, 1, 1], 
       Hue[0.1709356742763382, 1, 1], 
       Hue[0.1709356742763382, 1, 1], 
       Hue[0.28139292300740415`, 1, 1], 
       Hue[0.28139292300740415`, 1, 1]}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->{True, None},
    AxesOrigin->{0, 0},
    Epilog->
     LineBox[{{1, 0.0005978036838236523}, {2, 0.0008053826706155865}, {
       3, 0.0010783281061545478`}, {4, 0.0014344865472014994`}, {
       5, 0.001896207945665435}, {6, 0.002490598524366172}, {
       7, 0.0032505695525940704`}, {8, 0.004215593487139871}, {
       9, 0.005432165107508066}, {10, 0.0069555683735350775`}, {
       11, 0.008849580652412104}, {12, 0.011187903541496564`}, {
       13, 0.014054462469394725`}, {14, 0.017543191377605898`}, {
       15, 0.021758767049308975`}, {16, 0.02681647778534678}, {
       17, 0.03283999937702506}, {18, 0.039961194393297074`}, {
       19, 0.04831812268402073}, {20, 0.05805212604061957}, {
       21, 0.06930424471365276}, {22, 0.08221315192120926}, {
       23, 0.09690711648582484}, {24, 0.11350362619005733`}, {
       25, 0.13209941153498989`}, {26, 0.15276682324916818`}, {
       27, 0.17554686627183963`}, {28, 0.2004460270980389}, {
       29, 0.2274252707278486}, {30, 0.25640041172926176`}, {
       31, 0.28723533620292424`}, {32, 0.3197393342617633}, {
       33, 0.3536664144879694}, {34, 0.388714848345864}, {
       35, 0.42453053069338237`}, {36, 0.4607088517733009}, {
       37, 0.4968048013040667}, {38, 0.5323364841100944}, {
       39, 0.566796855209019}, {40, 0.5996671663438695}, {
       41, 0.6304280335187514}, {42, 0.6585737678217439}, {
       43, 0.6836221641113502}, {44, 0.7051321016931162}, {
       45, 0.7227158273629761}, {46, 0.7360535445923382}, {
       47, 0.7448993289637308}, {48, 0.7490843082657368}, {
       49, 0.7485282257909024}, {50, 0.7432430662614795}, {
       51, 0.733328891710943}, {52, 0.7189740119957582}, {
       53, 0.7004475353997589}, {54, 0.6780871567544573}, {
       55, 0.6522926770282905}, {56, 0.6235164860247326}, {
       57, 0.5922461540627697}, {58, 0.558992725196141}, {
       59, 0.5242751410811061}, {60, 0.48860999161873314`}, {
       61, 0.45249624849738507`}, {62, 0.4164082649414819}, {
       63, 0.38078040520181694`}, {64, 0.3460043305384273}, {
       65, 0.3124209307742743}, {66, 0.2803180865445572}, {
       67, 0.2499280460164679}, {68, 0.2214275591471378}, {
       69, 0.19494063287362245`}, {70, 0.1705404969474726}, {
       71, 0.1482541170912516}, {72, 0.1280679962072261}, {
       73, 0.10993357751924059`}, {74, 0.0937725318990729}, {
       75, 0.0794835218552028}, {76, 0.0669474933402219}, {
       77, 0.05603350931486707}, {78, 0.046603485356780036`}, {
       79, 0.03851640133275329}, {80, 0.031632254938806115`}, {
       81, 0.02581501719694651}, {82, 0.02093490744124618}, {
       83, 0.016870470504046636`}, {84, 0.013509519133381805`}, {
       85, 0.010750021626502956`}, {86, 0.008500339470996873}, {
       87, 0.006679120039506415}, {88, 0.005215018758582762}, {
       89, 0.004046182627650348}, {90, 0.003119522808065956}, {
       91, 0.0023899180121836425`}, {92, 0.0018193916773749262`}, {
       93, 0.0013763050395730976`}, {94, 0.001034537652863203}, {
       95, 0.0007727106562897323}, {96, 0.0005734831336549152}, {
       97, 0.00042291475674991456`}, {98, 0.00030989002993746955`}, {
       99, 0.00022561955752772904`}, {100, 0.00016321121780623593`}, {
       101, 0.00011730456036459204`}, {102, 0.000083764047120075}, {
       103, 0.000059424096198848745`}, {104, 0.00004188041764585625}, {
       105, 0.000029321150905136}, {106, 0.000020391297449909892`}, {
       107, 0.000014085479428168673`}, {108, 9.663304179834228*^-6}, {
       109, 6.583634863668514*^-6}, {110, 4.453958166502931*^-6}, {
       111, 2.9916622901711298`*^-6}, {112, 1.9948146181534026`*^-6}, {
       113, 1.3202243416442894`*^-6}, {114, 8.671105099514225*^-7}, {
       115, 5.650692673957306*^-7}, {116, 3.652965795644892*^-7}, {
       117, 2.3421768562283517`*^-7}, {118, 1.489147050614743*^-7}, {
       119, 9.386888440090016*^-8}, {120, 5.865502851490493*^-8}, {
       121, 3.632797659857819*^-8}, {122, 2.2300329433110763`*^-8}, {
       123, 1.3568690646364813`*^-8}, {124, 8.184536091772735*^-9}, {
       125, 4.895643838211951*^-9}, {126, 2.905235223785311*^-9}, {
       127, 1.7115079763678964`*^-9}, {128, 1.001718225125788*^-9}, {
       129, 5.830244280446148*^-10}, {130, 3.3780105496683146`*^-10}, {
       131, 1.9506109654349634`*^-10}, {132, 1.1239068223834314`*^-10}, {
       133, 6.468622379687457*^-11}, {134, 3.722222072882125*^-11}, {
       135, 2.1428379887313934`*^-11}, {136, 1.2346278825006124`*^-11}, {
       137, 7.119413969897597*^-12}, {138, 4.107335708559928*^-12}, {
       139, 2.3692065077203713`*^-12}, {140, 1.3651622175567317`*^-12}, {
       141, 7.849507907633935*^-13}, {142, 4.498774206551361*^-13}, {
       143, 2.5672636079354647`*^-13}, {144, 1.4572481897223778`*^-13}, {
       145, 8.220449138279655*^-14}, {146, 4.604962947625631*^-14}, {
       147, 2.5600720899091714`*^-14}, {148, 1.4117895812743714`*^-14}, {
       149, 7.720615155415827*^-15}, {150, 4.186322110517518*^-15}, {
       151, 2.2505628162462104`*^-15}, {152, 1.1996034260298354`*^-15}, {
       153, 6.340236047782132*^-16}, {154, 3.32313812207569*^-16}, {
       155, 1.7276135800859778`*^-16}, {156, 8.910471161341757*^-17}, {
       157, 4.560421538440369*^-17}, {158, 2.3164728544126162`*^-17}, {
       159, 1.1679503097307834`*^-17}, {160, 5.846132471219055*^-18}, {
       161, 2.905633862934292*^-18}, {162, 1.4341861814967735`*^-18}, {
       163, 7.030721417189377*^-19}, {164, 3.423274259667461*^-19}, {
       165, 1.6555775751413006`*^-19}, {166, 7.953522417306659*^-20}, {
       167, 3.7959688096682886`*^-20}, {168, 1.8000289902287722`*^-20}, {
       169, 8.480856435530723*^-21}, {170, 3.970002716637773*^-21}, {
       171, 1.8464101911452966`*^-21}, {172, 8.532433986826816*^-22}, {
       173, 3.91800403598377*^-22}, {174, 1.7878642617973032`*^-22}, {
       175, 8.107362796764301*^-23}, {176, 3.653226629432639*^-23}, {
       177, 1.635748211101474*^-23}, {178, 7.278170092615176*^-24}, {
       179, 3.2182283844083147`*^-24}, {180, 1.4141216048569485`*^-24}, {
       181, 6.174475400430278*^-25}, {182, 2.678945688733389*^-25}, {
       183, 1.155151110511368*^-25}, {184, 4.950815150661714*^-26}, {
       185, 2.1088987092434958`*^-26}, {186, 8.926723624621219*^-27}, {
       187, 3.7542069263221626`*^-27}, {188, 1.5687315282769104`*^-27}, {
       189, 6.514345816862464*^-28}, {190, 2.688871098587076*^-28}, {
       191, 1.103214026704079*^-28}, {192, 4.4985869886202567`*^-29}, {
       193, 1.822776961971838*^-29}, {194, 7.33816645881475*^-30}, {
       195, 2.9352948551187117`*^-30}, {196, 1.166737308083731*^-30}, {
       197, 4.609059873028647*^-31}, {198, 1.809820168594749*^-31}, {
       199, 7.064454269561081*^-32}, {200, 2.740858340767667*^-32}, {
       201, 1.0565945346847473`*^-32}, {202, 4.045783589742848*^-33}, {
       203, 1.53876838255855*^-33}, {204, 5.8155468969536534`*^-34}, {
       205, 2.1849935211684695`*^-34}, {206, 8.162098256350196*^-35}, {
       207, 3.030598622460437*^-35}, {208, 1.1181252666740965`*^-35}, {
       209, 4.0988108118497964`*^-36}, {210, 1.4930859785358631`*^-36}, {
       211, 5.404894314409645*^-37}, {212, 1.9439680428861522`*^-37}, {
       213, 6.94546997570754*^-38}, {214, 2.465139926227765*^-38}, {
       215, 8.694671700419666*^-39}, {216, 3.048500162067952*^-39}, {
       217, 1.0625106717845274`*^-39}, {218, 3.679906245531537*^-40}, {
       219, 1.2661667988279013`*^-40}, {220, 4.3300427408341196`*^-41}, {
       221, 1.4732497407239412`*^-41}, {222, 4.994657259048155*^-42}, {
       223, 1.6819930873413686`*^-42}, {224, 5.578963232753847*^-43}, {
       225, 1.7877954506559312`*^-43}, {226, 6.027212593164703*^-44}, {
       227, 2.382207389352189*^-44}, {228, 1.7945376321591332`*^-44}, {
       229, 1.6873867991283687`*^-44}, {230, 1.6873867991283687`*^-44}, {
       231, 1.6873867991283687`*^-44}, {232, 1.6873867991283687`*^-44}, {
       233, 1.6873867991283687`*^-44}, {234, 1.6873867991283687`*^-44}, {
       235, 1.6873867991283687`*^-44}, {236, 1.6873867991283687`*^-44}, {
       237, 1.6873867991283687`*^-44}, {238, 1.6873867991283687`*^-44}, {
       239, 1.6873867991283687`*^-44}, {240, 1.6873867991283687`*^-44}, {
       241, 1.6873867991283687`*^-44}, {242, 1.6873867991283687`*^-44}, {
       243, 1.6873867991283687`*^-44}, {244, 1.6873867991283687`*^-44}, {
       245, 1.6873867991283687`*^-44}, {246, 1.6873867991283687`*^-44}, {
       247, 1.6873867991283687`*^-44}, {248, 1.6873867991283687`*^-44}, {
       249, 1.6873867991283687`*^-44}, {250, 1.6873867991283687`*^-44}, {
       251, 1.6873867991283687`*^-44}}],
    Frame->True,
    FrameTicks->{{{1., 
        FormBox[
         RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
        FormBox[
         RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
        FormBox[
         RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
        FormBox[
         RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
        FormBox["0", TraditionalForm]}, {157.25, 
        FormBox["2.5`", TraditionalForm]}, {188.5, 
        FormBox["5", TraditionalForm]}, {219.75, 
        FormBox["7.5`", TraditionalForm]}, {251., 
        FormBox["10", TraditionalForm]}}, Automatic, None, None},
    PlotRange->{-0.1, 1.1},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], Automatic}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF2XtQ1PUax/FvejBEDUM0JTJE85h3RQRF4cNdEXG5CrLAj2UxDI2SFDWp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     "], {
      {GrayLevel[0.5], EdgeForm[None], 
       GraphicsGroupBox[{PolygonBox[{{166, 582, 167}}], 
         PolygonBox[CompressedData["
1:eJwtlmW0llUQRue8dHd3d126GxEVAbEQEBAUAQFRAQFppLu7BLu7u7u7u7sF
3M967o+91l5n5sQ7Z853b60xU4ZMziLiEOTNJUsReSCfHI7CMcgDhaAosWJQ
OHcs8JQ8tyAUxosk5+YXeAERXjOv1k6OaaygclPuXCiOl9AaeGmojteAMtpb
Y3KdAS8LtfE6UC48pyReKjm3OJTHK2hNxaAyXkU5Yari1ZL3Uk5FvFJyrvas
iddK3kt71MXrQXm8IQzAT4BGeCVoijeDyngF5SgGFfUt0A5vr28KjzXGmyTP
rQZt8LbJuVWgJd4KqobXbI63SI5pjQ54R6gZphPeWWfGG0B//Pjks9aGbnh3
1Syc0wXvmhzTN9XHGySfXTk98J5QN0wvvLdqELl3oNon17o+9MOPS95bOX3w
vskx9VkfmqovHA7X7ERiJ0FjvAkMwgdD0zBD8FNU0zBD8VOheZjT8NNVkzBn
4GdCyzDD8LNUwzDD8RGQE2Ykfja0DjMKHw1twozBz9GdhBmLj4N2Yc7Fz9Od
qrYwHZ+hGutuYAI+UXcUZhJ+AXQKMxmfojvDu8LF+CXJa3WBafhFyTHlTMUv
TI6pJ3J09uTe0Z4z8Uuhh2oLG/CNqjHeE+bgl0GvMHPxebrTMPPxBdAnzEJ8
ke4wzGJ8CfQLczm+VHceZhm+HPqHWYGvVA+GWYWvhgFh1uBr9YbCrMPXJ59d
3zALn518dn3DJnwzDAyzBd8KJ4fZhm+HQeGeGqhYcm8Nhl34bhgSztmB70yO
aWwPvlc9F2Yfvh+GhjmAH1QPqtfgGvxa9Zx6EQ7jV6on8RFwG347jAyPXYVf
nTx3ONyC35qcOwxuwG9Uz4bXvA6/PjmmNe7A71TPhrkLvxtGqRfhCfxJ9SQ+
Gu7D74cx4Zx78HuTY7PhJ/xnmBPOeQB/UD0f5iH8YRgbPtNN+M3JZx8Hj+GP
J++tnEfwR5NjWvMX/Ff1XPhMT+FPw3j8fHgOfx4m4BPhJfxlmBTmFfxVvZkw
r+Gvw+Qwb+Bv6g2FeQt/G6aGeQd/V28mzHv4+zAtvOcL+IvJe2vsA/xDvTm9
RfgE/1RvEp8BX+Ffw8zw2Gf45zA9zBf4l8m5yvkG/1ZvMrzmR/jHyWurBs/g
zybXYhb8gP+YfDea8x3+fXJMvzHjlZv826Ka/ob/DnPx7dCU39ZmsANfAP8R
OwIL8XnwF/43zNfbhfzkFoCl4bF/iP2bPHcJ5CWWL3PuItA/BQkWh9c8Su6x
5JjWKEisECwLUxgvAsvxtVARrwTr8BVQHC8BK8M5RfFimWProSpeDTaEc0ri
pWBVmNJ4GVgdPlOG58l89jVQHq+QeW/llMXLZY6pZn9w9j+Ta6MzVSZWJfPe
euNXEDuU/La3QH1iDWArvglq43VgM74TWuKtYFd4rC5eL/Nc3UlzvEXm3G3Q
GG+S+e60ZkO8UeaY1sjBW8PuMG3wtrBHZ4NeeO/Mf1v3Qge8I+wL57TD22eO
bYSaeK3MZ1dOJ7wz7A/TBe8KB8I1r47XyDz3IPTAe2beWznd8O6ZY0n9Bkfc
JvE/xHj8cg==
          "]]}]}, {{}, {}, {
        {Hue[0.4651544301308045, 1, 1], LineBox[{1, 2}]}, 
        {Hue[0.5554105816111728, 1, 1], LineBox[{2, 3}]}, 
        {Hue[0.6470845464116377, 1, 1], LineBox[{3, 4}]}, 
        {Hue[0.7379011536823059, 1, 1], LineBox[{4, 5}]}, 
        {Hue[0.8295664135425962, 1, 1], LineBox[{5, 6}]}, 
        {Hue[0.9214195643235872, 1, 1], LineBox[{6, 7}]}, 
        {Hue[0.013054762008105212`, 1, 1], LineBox[{7, 8}]}, 
        {Hue[0.10496038717414542`, 1, 1], LineBox[{8, 9}]}, 
        {Hue[0.19751458917685086`, 1, 1], LineBox[{9, 10}]}, 
        {Hue[0.2897640620074146, 1, 1], LineBox[{10, 11}]}, 
        {Hue[0.3824828729642137, 1, 1], LineBox[{11, 12}]}, 
        {Hue[0.4753040283446231, 1, 1], LineBox[{12, 13}]}, 
        {Hue[0.5684100351355065, 1, 1], LineBox[{13, 14}]}, 
        {Hue[0.6616080261793253, 1, 1], LineBox[{14, 15}]}, 
        {Hue[0.7550855940217207, 1, 1], LineBox[{15, 16}]}, 
        {Hue[0.8487100201155822, 1, 1], LineBox[{16, 17}]}, 
        {Hue[0.9425509991658001, 1, 1], LineBox[{17, 18}]}, 
        {Hue[0.036609959351899514`, 1, 1], LineBox[{18, 19}]}, 
        {Hue[0.1308358768599252, 1, 1], LineBox[{19, 20}]}, 
        {Hue[0.22527489516639876`, 1, 1], LineBox[{20, 21}]}, 
        {Hue[0.3199020157682274, 1, 1], LineBox[{21, 22}]}, 
        {Hue[0.41473227333091395`, 1, 1], LineBox[{22, 23}]}, 
        {Hue[0.509757605107152, 1, 1], LineBox[{23, 24}]}, 
        {Hue[0.6049705984441132, 1, 1], LineBox[{24, 25}]}, 
        {Hue[0.7003881175130392, 1, 1], LineBox[{25, 26}]}, 
        {Hue[0.7960010298713771, 1, 1], LineBox[{26, 27}]}, 
        {Hue[0.8918060581525241, 1, 1], LineBox[{27, 28}]}, 
        {Hue[0.987804312899045, 1, 1], LineBox[{28, 29}]}, 
        {Hue[0.08399480518255445, 1, 1], LineBox[{29, 30}]}, 
        {Hue[0.18038477209683, 1, 1], LineBox[{30, 31}]}, 
        {Hue[0.27696843286290557`, 1, 1], LineBox[{31, 32}]}, 
        {Hue[0.37374369528133805`, 1, 1], LineBox[{32, 33}]}, 
        {Hue[0.4707099465669432, 1, 1], LineBox[{33, 34}]}, 
        {Hue[0.567869481609075, 1, 1], LineBox[{34, 35}]}, 
        {Hue[0.6652240721329183, 1, 1], LineBox[{35, 36}]}, 
        {Hue[0.7627725097368594, 1, 1], LineBox[{36, 37}]}, 
        {Hue[0.8605108972765441, 1, 1], LineBox[{37, 38}]}, 
        {Hue[0.958440389766678, 1, 1], LineBox[{38, 39}]}, 
        {Hue[0.05656129026424089, 1, 1], LineBox[{39, 40}]}, 
        {Hue[0.15487414605873545`, 1, 1], LineBox[{40, 41}]}, 
        {Hue[0.2533794441638736, 1, 1], LineBox[{41, 42}]}, 
        {Hue[0.35207574889560544`, 1, 1], LineBox[{42, 43}]}, 
        {Hue[0.4509616563755431, 1, 1], LineBox[{43, 44}]}, 
        {Hue[0.5500368479038305, 1, 1], LineBox[{44, 45}]}, 
        {Hue[0.6493029200071799, 1, 1], LineBox[{45, 46}]}, 
        {Hue[0.7487594785778681, 1, 1], LineBox[{46, 47}]}, 
        {Hue[0.8484065019661553, 1, 1], LineBox[{47, 48}]}, 
        {Hue[0.9482424326521234, 1, 1], LineBox[{48, 49}]}, 
        {Hue[0.0482667009607877, 1, 1], LineBox[{49, 50}]}, 
        {Hue[0.14848009586554878`, 1, 1], LineBox[{50, 51}]}, 
        {Hue[0.24888253739037916`, 1, 1], LineBox[{51, 52}]}, 
        {Hue[0.3494738481964279, 1, 1], LineBox[{52, 53}]}, 
        {Hue[0.4502538400841612, 1, 1], LineBox[{53, 54}]}, 
        {Hue[0.5512215313210937, 1, 1], LineBox[{54, 55}]}, 
        {Hue[0.6523770888719254, 1, 1], LineBox[{55, 56}]}, 
        {Hue[0.7537211195726249, 1, 1], LineBox[{56, 57}]}, 
        {Hue[0.8552526263705517, 1, 1], LineBox[{57, 58}]}, 
        {Hue[0.956971653741671, 1, 1], LineBox[{58, 59}]}, 
        {Hue[0.05887788638436986, 1, 1], LineBox[{59, 60}]}, 
        {Hue[0.1609711068475429, 1, 1], LineBox[{60, 61}]}, 
        {Hue[0.263251461198306, 1, 1], LineBox[{61, 62}]}, 
        {Hue[0.36571881160627995`, 1, 1], LineBox[{62, 63}]}, 
        {Hue[0.46837235562530183`, 1, 1], LineBox[{63, 64}]}, 
        {Hue[0.571212188793546, 1, 1], LineBox[{64, 65}]}, 
        {Hue[0.6742384387634822, 1, 1], LineBox[{65, 66}]}, 
        {Hue[0.7774505905900797, 1, 1], LineBox[{66, 67}]}, 
        {Hue[0.88084887240729, 1, 1], LineBox[{67, 68}]}, 
        {Hue[0.9844326836374937, 1, 1], LineBox[{68, 69}]}, 
        {Hue[0.08820140979164921, 1, 1], LineBox[{69, 70}]}, 
        {Hue[0.1921553760160884, 1, 1], LineBox[{70, 71}]}, 
        {Hue[0.2962945397620034, 1, 1], LineBox[{71, 72}]}, 
        {Hue[0.40061857163953374`, 1, 1], LineBox[{72, 73}]}, 
        {Hue[0.5051269690985097, 1, 1], LineBox[{73, 74}]}, 
        {Hue[0.6098193140164784, 1, 1], LineBox[{74, 75}]}, 
        {Hue[0.7146956052782951, 1, 1], LineBox[{75, 76}]}, 
        {Hue[0.8197550348620458, 1, 1], LineBox[{76, 77}]}, 
        {Hue[0.92499766979028, 1, 1], LineBox[{77, 78}]}, 
        {Hue[0.030422802443574908`, 1, 1], LineBox[{78, 79}]}, 
        {Hue[0.13603006751000404`, 1, 1], LineBox[{79, 80}]}, 
        {Hue[0.24181881541715228`, 1, 1], LineBox[{80, 81}]}, 
        {Hue[0.34778816817848446`, 1, 1], LineBox[{81, 82}]}, 
        {Hue[0.4539371094543324, 1, 1], LineBox[{82, 83}]}, 
        {Hue[0.5602651505544005, 1, 1], LineBox[{83, 84}]}, 
        {Hue[0.6667712634439584, 1, 1], LineBox[{84, 85}]}, 
        {Hue[0.7734535461630561, 1, 1], LineBox[{85, 86}]}, 
        {Hue[0.8803101814100706, 1, 1], LineBox[{86, 87}]}, 
        {Hue[0.9873409380665711, 1, 1], LineBox[{87, 88}]}, 
        {Hue[0.09454433494227246, 1, 1], LineBox[{88, 89}]}, 
        {Hue[0.20191668511525024`, 1, 1], LineBox[{89, 90}]}, 
        {Hue[0.3094548457672087, 1, 1], LineBox[{90, 91}]}, 
        {Hue[0.4171562070936044, 1, 1], LineBox[{91, 92}]}, 
        {Hue[0.5250179871183952, 1, 1], LineBox[{92, 93}]}, 
        {Hue[0.6330355272821271, 1, 1], LineBox[{93, 94}]}, 
        {Hue[0.7412034759242306, 1, 1], LineBox[{94, 95}]}, 
        {Hue[0.849516723315727, 1, 1], LineBox[{95, 96}]}, 
        {Hue[0.9579680811132738, 1, 1], LineBox[{96, 97}]}, 
        {Hue[0.06654996696082718, 1, 1], LineBox[{97, 98}]}, 
        {Hue[0.1752542034971408, 1, 1], LineBox[{98, 99}]}, 
        {Hue[0.28406982820136284`, 1, 1], LineBox[{99, 100}]}, 
        {Hue[0.39298448987471163`, 1, 1], LineBox[{100, 101}]}, 
        {Hue[0.5019841493283912, 1, 1], LineBox[{101, 102}]}, 
        {Hue[0.6110525350493271, 1, 1], LineBox[{102, 103}]}, 
        {Hue[0.7201713819939524, 1, 1], LineBox[{103, 104}]}, 
        {Hue[0.8293217984945689, 1, 1], LineBox[{104, 105}]}, 
        {Hue[0.9384800401343466, 1, 1], LineBox[{105, 106}]}, 
        {Hue[0.04761399197175331, 1, 1], LineBox[{106, 107}]}, 
        {Hue[0.15668862421436977`, 1, 1], LineBox[{107, 108}]}, 
        {Hue[0.26567158533576374`, 1, 1], LineBox[{108, 109}]}, 
        {Hue[0.37452804323486194`, 1, 1], LineBox[{109, 110}]}, 
        {Hue[0.48321158195502617`, 1, 1], LineBox[{110, 111}]}, 
        {Hue[0.591665801751543, 1, 1], LineBox[{111, 112}]}, 
        {Hue[0.6998335322551498, 1, 1], LineBox[{112, 113}]}, 
        {Hue[0.8076561474609649, 1, 1], LineBox[{113, 114}]}, 
        {Hue[0.9150674728730183, 1, 1], LineBox[{114, 115}]}, 
        {Hue[0.021995142902907924`, 1, 1], LineBox[{115, 116}]}, 
        {Hue[0.12836837744267132`, 1, 1], LineBox[{116, 117}]}, 
        {Hue[0.2341169635451068, 1, 1], LineBox[{117, 118}]}, 
        {Hue[0.3391660604792021, 1, 1], LineBox[{118, 119}]}, 
        {Hue[0.4434364948665046, 1, 1], LineBox[{119, 120}]}, 
        {Hue[0.546852954234411, 1, 1], LineBox[{120, 121}]}, 
        {Hue[0.6493540844131561, 1, 1], LineBox[{121, 122}]}, 
        {Hue[0.7508968981911877, 1, 1], LineBox[{122, 123}]}, 
        {Hue[0.851451910761745, 1, 1], LineBox[{123, 124}]}, 
        {Hue[0.9509963989571802, 1, 1], LineBox[{124, 125}]}, 
        {Hue[0.04951252491739281, 1, 1], LineBox[{125, 126}]}, 
        {Hue[0.1469910489023467, 1, 1], LineBox[{126, 127}]}, 
        {Hue[0.2434442346796176, 1, 1], LineBox[{127, 128}]}, 
        {Hue[0.33891490119097384`, 1, 1], LineBox[{128, 129}]}, 
        {Hue[0.43345638334230663`, 1, 1], LineBox[{129, 130}]}, 
        {Hue[0.527095344156284, 1, 1], LineBox[{130, 131}]}, 
        {Hue[0.6198251547570155, 1, 1], LineBox[{131, 132}]}, 
        {Hue[0.7116325629947977, 1, 1], LineBox[{132, 133}]}, 
        {Hue[0.8025203294870583, 1, 1], LineBox[{133, 134}]}, 
        {Hue[0.892489925483982, 1, 1], LineBox[{134, 135}]}, 
        {Hue[0.9814950793701384, 1, 1], LineBox[{135, 136}]}, 
        {Hue[0.06940879585274738, 1, 1], LineBox[{136, 137}]}, 
        {Hue[0.1560379167494746, 1, 1], LineBox[{137, 138}]}, 
        {Hue[0.2411397908261848, 1, 1], LineBox[{138, 139}]}, 
        {Hue[0.3243570164916532, 1, 1], LineBox[{139, 140}]}, 
        {Hue[0.40512776526272315`, 1, 1], LineBox[{140, 141}]}, 
        {Hue[0.4827689796728388, 1, 1], LineBox[{141, 142}]}, 
        {Hue[0.5567366414205508, 1, 1], LineBox[{142, 143}]}, 
        {Hue[0.6268618216854998, 1, 1], LineBox[{143, 144}]}, 
        {Hue[0.6936551818635579, 1, 1], LineBox[{144, 145}]}, 
        {Hue[0.7588789911312471, 1, 1], LineBox[{145, 146}]}, 
        {Hue[0.8254234195078273, 1, 1], LineBox[{146, 147}]}, 
        {Hue[0.8957871458702045, 1, 1], LineBox[{147, 148}]}, 
        {Hue[0.9712803960295606, 1, 1], LineBox[{148, 149}]}, 
        {Hue[0.0526909438225915, 1, 1], LineBox[{149, 150}]}, 
        {Hue[0.14028738520395323`, 1, 1], LineBox[{150, 151}]}, 
        {Hue[0.23330222879300616`, 1, 1], LineBox[{151, 152}]}, 
        {Hue[0.3304408906001906, 1, 1], LineBox[{152, 153}]}, 
        {Hue[0.43089611044311255`, 1, 1], LineBox[{153, 154}]}, 
        {Hue[0.5345867051543077, 1, 1], LineBox[{154, 155}]}, 
        {Hue[0.6416275575907403, 1, 1], LineBox[{155, 156}]}, 
        {Hue[0.7519045675691594, 1, 1], LineBox[{156, 157}]}, 
        {Hue[0.865098064379245, 1, 1], LineBox[{157, 158}]}, 
        {Hue[0.9808402665833201, 1, 1], LineBox[{158, 159}]}, 
        {Hue[0.09878598465087692, 1, 1], LineBox[{159, 160}]}, 
        {Hue[0.2186316551202424, 1, 1], LineBox[{160, 161}]}, 
        {Hue[0.3401338285178514, 1, 1], LineBox[{161, 162}]}, 
        {Hue[0.4631214068126678, 1, 1], LineBox[{162, 163}]}, 
        {Hue[0.5874801753732921, 1, 1], LineBox[{163, 164}]}, 
        {Hue[0.7130934559046723, 1, 1], LineBox[{164, 165}]}, 
        {Hue[0.8397487315274839, 1, 1], LineBox[{165, 166}]}, 
        {Hue[0.9670748643982374, 1, 1], LineBox[{166, 167}]}, 
        {Hue[0.09457976930817419, 1, 1], LineBox[{167, 168}]}, 
        {Hue[0.22179504405250422`, 1, 1], LineBox[{168, 169}]}, 
        {Hue[0.3484568885698326, 1, 1], LineBox[{169, 170}]}, 
        {Hue[0.4746096680737803, 1, 1], LineBox[{170, 171}]}, 
        {Hue[0.6005198489903314, 1, 1], LineBox[{171, 172}]}, 
        {Hue[0.7263863088486961, 1, 1], LineBox[{172, 173}]}, 
        {Hue[0.8520468555923485, 1, 1], LineBox[{173, 174}]}, 
        {Hue[0.9770017580001946, 1, 1], LineBox[{174, 175}]}, 
        {Hue[0.10087798050703355`, 1, 1], LineBox[{175, 176}]}, 
        {Hue[0.22399620151989078`, 1, 1], LineBox[{176, 177}]}, 
        {Hue[0.34734765213365987`, 1, 1], LineBox[{177, 178}]}, 
        {Hue[0.47175243518309923`, 1, 1], LineBox[{178, 179}]}, 
        {Hue[0.5970599486235515, 1, 1], LineBox[{179, 180}]}, 
        {Hue[0.722304988072564, 1, 1], LineBox[{180, 181}]}, 
        {Hue[0.8467009602545368, 1, 1], LineBox[{181, 182}]}, 
        {Hue[0.9706106571787333, 1, 1], LineBox[{182, 183}]}, 
        {Hue[0.09543940162180947, 1, 1], LineBox[{183, 184}]}, 
        {Hue[0.2222604194299728, 1, 1], LineBox[{184, 185}]}, 
        {Hue[0.3505885939092142, 1, 1], LineBox[{185, 186}]}, 
        {Hue[0.47851986808596186`, 1, 1], LineBox[{186, 187}]}, 
        {Hue[0.6040683153079265, 1, 1], LineBox[{187, 188}]}, 
        {Hue[0.7270629798245565, 1, 1], LineBox[{188, 189}]}, 
        {Hue[0.8500757901129055, 1, 1], LineBox[{189, 190}]}, 
        {Hue[0.9763751814294643, 1, 1], LineBox[{190, 191}]}, 
        {Hue[0.10674965681726335`, 1, 1], LineBox[{191, 192}]}, 
        {Hue[0.23891176768458297`, 1, 1], LineBox[{192, 193}]}, 
        {Hue[0.3690996557259483, 1, 1], LineBox[{193, 194}]}, 
        {Hue[0.4944875289910466, 1, 1], LineBox[{194, 195}]}, 
        {Hue[0.6160926085787073, 1, 1], LineBox[{195, 196}]}, 
        {Hue[0.7389647173416496, 1, 1], LineBox[{196, 197}]}, 
        {Hue[0.8670570961233259, 1, 1], LineBox[{197, 198}]}, 
        {Hue[0.999696967846253, 1, 1], LineBox[{198, 199}]}, 
        {Hue[0.13360266491970105`, 1, 1], LineBox[{199, 200}]}, 
        {Hue[0.2661866503459161, 1, 1], LineBox[{200, 201}]}, 
        {Hue[0.3973874472708366, 1, 1], LineBox[{201, 202}]}, 
        {Hue[0.528600014946299, 1, 1], LineBox[{202, 203}]}, 
        {Hue[0.6591720785215255, 1, 1], LineBox[{203, 204}]}, 
        {Hue[0.7845489645344041, 1, 1], LineBox[{204, 205}]}, 
        {Hue[0.9003656193469529, 1, 1], LineBox[{205, 206}]}, 
        {Hue[0.013714912174225602`, 1, 1], LineBox[{206, 207}]}, 
        {Hue[0.1408471718351541, 1, 1], LineBox[{207, 208}]}, 
        {Hue[0.28661504066258015`, 1, 1], LineBox[{208, 209}]}, 
        {Hue[0.44534697210746116`, 1, 1], LineBox[{209, 210}]}, 
        {Hue[0.6095009105377235, 1, 1], LineBox[{210, 211}]}, 
        {Hue[0.7690038856007232, 1, 1], LineBox[{211, 212}]}, 
        {Hue[0.8949038714677751, 1, 1], LineBox[{212, 213}]}, 
        {Hue[0.9086342977794758, 1, 1], LineBox[{213, 214}]}, 
        {Hue[0.9845249830133344, 1, 1], LineBox[{214, 215}]}, 
        {Hue[0.14123330332232742`, 1, 1], LineBox[{215, 216}]}, 
        {Hue[0.31951791804120616`, 1, 1], LineBox[{216, 217}]}, 
        {Hue[0.5065938980359681, 1, 1], LineBox[{217, 218}]}, 
        {Hue[0.6998846256103239, 1, 1], LineBox[{218, 219}]}, 
        {Hue[0.9016850504011885, 1, 1], LineBox[{219, 220}]}, 
        {Hue[0.12781140313159747`, 1, 1], LineBox[{220, 221}]}, 
        {Hue[0.5089285539277778, 1, 1], LineBox[{221, 222}]}, 
        {Hue[0.8765034481206417, 1, 1], LineBox[{222, 223}]}, 
        {Hue[0.10736464903210238`, 1, 1], LineBox[{223, 224}]}, 
        {Hue[0.3213807049021133, 1, 1], LineBox[{224, 225}]}, 
        {Hue[0.5341230584416392, 1, 1], LineBox[{225, 226}]}, 
        {Hue[0.7506241615262762, 1, 1], LineBox[{226, 227}]}, 
        {Hue[0.9751545195339061, 1, 1], LineBox[{227, 228}]}, 
        {Hue[0.2140165315639541, 1, 1], LineBox[{228, 229}]}, 
        {Hue[0.4755204440520452, 1, 1], LineBox[{229, 230}]}, 
        {Hue[0.756792506749526, 1, 1], LineBox[{230, 231}]}, 
        {Hue[0.030535038964111475`, 1, 1], LineBox[{231, 232}]}, 
        {Hue[0.28164947178050603`, 1, 1], LineBox[{232, 233}]}, 
        {Hue[0.5162105627879208, 1, 1], LineBox[{233, 234}]}, 
        {Hue[0.7416446994261499, 1, 1], LineBox[{234, 235}]}, 
        {Hue[0.9624384494465095, 1, 1], LineBox[{235, 236}]}, 
        {Hue[0.1814804022546692, 1, 1], LineBox[{236, 237}]}, 
        {Hue[0.401511966546178, 1, 1], LineBox[{237, 238}]}, 
        {Hue[0.6248849205239658, 1, 1], LineBox[{238, 239}]}, 
        {Hue[0.8526414952731264, 1, 1], LineBox[{239, 240}]}, 
        {Hue[0.10869555432495916`, 1, 1], LineBox[{240, 241}]}, 
        {Hue[0.42386784600606003`, 1, 1], LineBox[{241, 242}]}, 
        {Hue[0.6301322507388738, 1, 1], LineBox[{242, 243}]}, 
        {Hue[0.65362457352435, 1, 1], LineBox[{243, 244}]}, 
        {Hue[0.6475836176504333, 1, 1], LineBox[{244, 245}]}, 
        {Hue[0.6475836176504333, 1, 1], LineBox[{245, 246}]}, 
        {Hue[0.6475836176504333, 1, 1], LineBox[{246, 247}]}, 
        {Hue[0.6475836176504333, 1, 1], LineBox[{247, 248}]}, 
        {Hue[0.6475836176504333, 1, 1], LineBox[{248, 249}]}, 
        {Hue[0.6475836176504333, 1, 1], LineBox[{249, 250}]}, 
        {Hue[0.6475836176504333, 1, 1], LineBox[{250, 251}]}}}},
     VertexColors->{
       Hue[0.4651544301308045, 1, 1], 
       Hue[0.5554105816111728, 1, 1], 
       Hue[0.6470845464116377, 1, 1], 
       Hue[0.7379011536823059, 1, 1], 
       Hue[0.8295664135425962, 1, 1], 
       Hue[0.9214195643235872, 1, 1], 
       Hue[0.013054762008105212`, 1, 1], 
       Hue[0.10496038717414542`, 1, 1], 
       Hue[0.19751458917685086`, 1, 1], 
       Hue[0.2897640620074146, 1, 1], 
       Hue[0.3824828729642137, 1, 1], 
       Hue[0.4753040283446231, 1, 1], 
       Hue[0.5684100351355065, 1, 1], 
       Hue[0.6616080261793253, 1, 1], 
       Hue[0.7550855940217207, 1, 1], 
       Hue[0.8487100201155822, 1, 1], 
       Hue[0.9425509991658001, 1, 1], 
       Hue[0.036609959351899514`, 1, 1], 
       Hue[0.1308358768599252, 1, 1], 
       Hue[0.22527489516639876`, 1, 1], 
       Hue[0.3199020157682274, 1, 1], 
       Hue[0.41473227333091395`, 1, 1], 
       Hue[0.509757605107152, 1, 1], 
       Hue[0.6049705984441132, 1, 1], 
       Hue[0.7003881175130392, 1, 1], 
       Hue[0.7960010298713771, 1, 1], 
       Hue[0.8918060581525241, 1, 1], 
       Hue[0.987804312899045, 1, 1], 
       Hue[0.08399480518255445, 1, 1], 
       Hue[0.18038477209683, 1, 1], 
       Hue[0.27696843286290557`, 1, 1], 
       Hue[0.37374369528133805`, 1, 1], 
       Hue[0.4707099465669432, 1, 1], 
       Hue[0.567869481609075, 1, 1], 
       Hue[0.6652240721329183, 1, 1], 
       Hue[0.7627725097368594, 1, 1], 
       Hue[0.8605108972765441, 1, 1], 
       Hue[0.958440389766678, 1, 1], 
       Hue[0.05656129026424089, 1, 1], 
       Hue[0.15487414605873545`, 1, 1], 
       Hue[0.2533794441638736, 1, 1], 
       Hue[0.35207574889560544`, 1, 1], 
       Hue[0.4509616563755431, 1, 1], 
       Hue[0.5500368479038305, 1, 1], 
       Hue[0.6493029200071799, 1, 1], 
       Hue[0.7487594785778681, 1, 1], 
       Hue[0.8484065019661553, 1, 1], 
       Hue[0.9482424326521234, 1, 1], 
       Hue[0.0482667009607877, 1, 1], 
       Hue[0.14848009586554878`, 1, 1], 
       Hue[0.24888253739037916`, 1, 1], 
       Hue[0.3494738481964279, 1, 1], 
       Hue[0.4502538400841612, 1, 1], 
       Hue[0.5512215313210937, 1, 1], 
       Hue[0.6523770888719254, 1, 1], 
       Hue[0.7537211195726249, 1, 1], 
       Hue[0.8552526263705517, 1, 1], 
       Hue[0.956971653741671, 1, 1], 
       Hue[0.05887788638436986, 1, 1], 
       Hue[0.1609711068475429, 1, 1], 
       Hue[0.263251461198306, 1, 1], 
       Hue[0.36571881160627995`, 1, 1], 
       Hue[0.46837235562530183`, 1, 1], 
       Hue[0.571212188793546, 1, 1], 
       Hue[0.6742384387634822, 1, 1], 
       Hue[0.7774505905900797, 1, 1], 
       Hue[0.88084887240729, 1, 1], 
       Hue[0.9844326836374937, 1, 1], 
       Hue[0.08820140979164921, 1, 1], 
       Hue[0.1921553760160884, 1, 1], 
       Hue[0.2962945397620034, 1, 1], 
       Hue[0.40061857163953374`, 1, 1], 
       Hue[0.5051269690985097, 1, 1], 
       Hue[0.6098193140164784, 1, 1], 
       Hue[0.7146956052782951, 1, 1], 
       Hue[0.8197550348620458, 1, 1], 
       Hue[0.92499766979028, 1, 1], 
       Hue[0.030422802443574908`, 1, 1], 
       Hue[0.13603006751000404`, 1, 1], 
       Hue[0.24181881541715228`, 1, 1], 
       Hue[0.34778816817848446`, 1, 1], 
       Hue[0.4539371094543324, 1, 1], 
       Hue[0.5602651505544005, 1, 1], 
       Hue[0.6667712634439584, 1, 1], 
       Hue[0.7734535461630561, 1, 1], 
       Hue[0.8803101814100706, 1, 1], 
       Hue[0.9873409380665711, 1, 1], 
       Hue[0.09454433494227246, 1, 1], 
       Hue[0.20191668511525024`, 1, 1], 
       Hue[0.3094548457672087, 1, 1], 
       Hue[0.4171562070936044, 1, 1], 
       Hue[0.5250179871183952, 1, 1], 
       Hue[0.6330355272821271, 1, 1], 
       Hue[0.7412034759242306, 1, 1], 
       Hue[0.849516723315727, 1, 1], 
       Hue[0.9579680811132738, 1, 1], 
       Hue[0.06654996696082718, 1, 1], 
       Hue[0.1752542034971408, 1, 1], 
       Hue[0.28406982820136284`, 1, 1], 
       Hue[0.39298448987471163`, 1, 1], 
       Hue[0.5019841493283912, 1, 1], 
       Hue[0.6110525350493271, 1, 1], 
       Hue[0.7201713819939524, 1, 1], 
       Hue[0.8293217984945689, 1, 1], 
       Hue[0.9384800401343466, 1, 1], 
       Hue[0.04761399197175331, 1, 1], 
       Hue[0.15668862421436977`, 1, 1], 
       Hue[0.26567158533576374`, 1, 1], 
       Hue[0.37452804323486194`, 1, 1], 
       Hue[0.48321158195502617`, 1, 1], 
       Hue[0.591665801751543, 1, 1], 
       Hue[0.6998335322551498, 1, 1], 
       Hue[0.8076561474609649, 1, 1], 
       Hue[0.9150674728730183, 1, 1], 
       Hue[0.021995142902907924`, 1, 1], 
       Hue[0.12836837744267132`, 1, 1], 
       Hue[0.2341169635451068, 1, 1], 
       Hue[0.3391660604792021, 1, 1], 
       Hue[0.4434364948665046, 1, 1], 
       Hue[0.546852954234411, 1, 1], 
       Hue[0.6493540844131561, 1, 1], 
       Hue[0.7508968981911877, 1, 1], 
       Hue[0.851451910761745, 1, 1], 
       Hue[0.9509963989571802, 1, 1], 
       Hue[0.04951252491739281, 1, 1], 
       Hue[0.1469910489023467, 1, 1], 
       Hue[0.2434442346796176, 1, 1], 
       Hue[0.33891490119097384`, 1, 1], 
       Hue[0.43345638334230663`, 1, 1], 
       Hue[0.527095344156284, 1, 1], 
       Hue[0.6198251547570155, 1, 1], 
       Hue[0.7116325629947977, 1, 1], 
       Hue[0.8025203294870583, 1, 1], 
       Hue[0.892489925483982, 1, 1], 
       Hue[0.9814950793701384, 1, 1], 
       Hue[0.06940879585274738, 1, 1], 
       Hue[0.1560379167494746, 1, 1], 
       Hue[0.2411397908261848, 1, 1], 
       Hue[0.3243570164916532, 1, 1], 
       Hue[0.40512776526272315`, 1, 1], 
       Hue[0.4827689796728388, 1, 1], 
       Hue[0.5567366414205508, 1, 1], 
       Hue[0.6268618216854998, 1, 1], 
       Hue[0.6936551818635579, 1, 1], 
       Hue[0.7588789911312471, 1, 1], 
       Hue[0.8254234195078273, 1, 1], 
       Hue[0.8957871458702045, 1, 1], 
       Hue[0.9712803960295606, 1, 1], 
       Hue[0.0526909438225915, 1, 1], 
       Hue[0.14028738520395323`, 1, 1], 
       Hue[0.23330222879300616`, 1, 1], 
       Hue[0.3304408906001906, 1, 1], 
       Hue[0.43089611044311255`, 1, 1], 
       Hue[0.5345867051543077, 1, 1], 
       Hue[0.6416275575907403, 1, 1], 
       Hue[0.7519045675691594, 1, 1], 
       Hue[0.865098064379245, 1, 1], 
       Hue[0.9808402665833201, 1, 1], 
       Hue[0.09878598465087692, 1, 1], 
       Hue[0.2186316551202424, 1, 1], 
       Hue[0.3401338285178514, 1, 1], 
       Hue[0.4631214068126678, 1, 1], 
       Hue[0.5874801753732921, 1, 1], 
       Hue[0.7130934559046723, 1, 1], 
       Hue[0.8397487315274839, 1, 1], 
       Hue[0.9670748643982374, 1, 1], 
       Hue[0.09457976930817419, 1, 1], 
       Hue[0.22179504405250422`, 1, 1], 
       Hue[0.3484568885698326, 1, 1], 
       Hue[0.4746096680737803, 1, 1], 
       Hue[0.6005198489903314, 1, 1], 
       Hue[0.7263863088486961, 1, 1], 
       Hue[0.8520468555923485, 1, 1], 
       Hue[0.9770017580001946, 1, 1], 
       Hue[0.10087798050703355`, 1, 1], 
       Hue[0.22399620151989078`, 1, 1], 
       Hue[0.34734765213365987`, 1, 1], 
       Hue[0.47175243518309923`, 1, 1], 
       Hue[0.5970599486235515, 1, 1], 
       Hue[0.722304988072564, 1, 1], 
       Hue[0.8467009602545368, 1, 1], 
       Hue[0.9706106571787333, 1, 1], 
       Hue[0.09543940162180947, 1, 1], 
       Hue[0.2222604194299728, 1, 1], 
       Hue[0.3505885939092142, 1, 1], 
       Hue[0.47851986808596186`, 1, 1], 
       Hue[0.6040683153079265, 1, 1], 
       Hue[0.7270629798245565, 1, 1], 
       Hue[0.8500757901129055, 1, 1], 
       Hue[0.9763751814294643, 1, 1], 
       Hue[0.10674965681726335`, 1, 1], 
       Hue[0.23891176768458297`, 1, 1], 
       Hue[0.3690996557259483, 1, 1], 
       Hue[0.4944875289910466, 1, 1], 
       Hue[0.6160926085787073, 1, 1], 
       Hue[0.7389647173416496, 1, 1], 
       Hue[0.8670570961233259, 1, 1], 
       Hue[0.999696967846253, 1, 1], 
       Hue[0.13360266491970105`, 1, 1], 
       Hue[0.2661866503459161, 1, 1], 
       Hue[0.3973874472708366, 1, 1], 
       Hue[0.528600014946299, 1, 1], 
       Hue[0.6591720785215255, 1, 1], 
       Hue[0.7845489645344041, 1, 1], 
       Hue[0.9003656193469529, 1, 1], 
       Hue[0.013714912174225602`, 1, 1], 
       Hue[0.1408471718351541, 1, 1], 
       Hue[0.28661504066258015`, 1, 1], 
       Hue[0.44534697210746116`, 1, 1], 
       Hue[0.6095009105377235, 1, 1], 
       Hue[0.7690038856007232, 1, 1], 
       Hue[0.8949038714677751, 1, 1], 
       Hue[0.9086342977794758, 1, 1], 
       Hue[0.9845249830133344, 1, 1], 
       Hue[0.14123330332232742`, 1, 1], 
       Hue[0.31951791804120616`, 1, 1], 
       Hue[0.5065938980359681, 1, 1], 
       Hue[0.6998846256103239, 1, 1], 
       Hue[0.9016850504011885, 1, 1], 
       Hue[0.12781140313159747`, 1, 1], 
       Hue[0.5089285539277778, 1, 1], 
       Hue[0.8765034481206417, 1, 1], 
       Hue[0.10736464903210238`, 1, 1], 
       Hue[0.3213807049021133, 1, 1], 
       Hue[0.5341230584416392, 1, 1], 
       Hue[0.7506241615262762, 1, 1], 
       Hue[0.9751545195339061, 1, 1], 
       Hue[0.2140165315639541, 1, 1], 
       Hue[0.4755204440520452, 1, 1], 
       Hue[0.756792506749526, 1, 1], 
       Hue[0.030535038964111475`, 1, 1], 
       Hue[0.28164947178050603`, 1, 1], 
       Hue[0.5162105627879208, 1, 1], 
       Hue[0.7416446994261499, 1, 1], 
       Hue[0.9624384494465095, 1, 1], 
       Hue[0.1814804022546692, 1, 1], 
       Hue[0.401511966546178, 1, 1], 
       Hue[0.6248849205239658, 1, 1], 
       Hue[0.8526414952731264, 1, 1], 
       Hue[0.10869555432495916`, 1, 1], 
       Hue[0.42386784600606003`, 1, 1], 
       Hue[0.6301322507388738, 1, 1], 
       Hue[0.65362457352435, 1, 1], 
       Hue[0.6475836176504333, 1, 1], 
       Hue[0.6475836176504333, 1, 1], 
       Hue[0.6475836176504333, 1, 1], 
       Hue[0.6475836176504333, 1, 1], 
       Hue[0.6475836176504333, 1, 1], 
       Hue[0.6475836176504333, 1, 1], 
       Hue[0.6475836176504333, 1, 1], 
       Hue[0.6475836176504333, 1, 1], 
       Hue[0.4651544301308045, 1, 1], 
       Hue[0.5554105816111728, 1, 1], 
       Hue[0.5554105816111728, 1, 1], 
       Hue[0.6470845464116377, 1, 1], 
       Hue[0.6470845464116377, 1, 1], 
       Hue[0.7379011536823059, 1, 1], 
       Hue[0.7379011536823059, 1, 1], 
       Hue[0.8295664135425962, 1, 1], 
       Hue[0.8295664135425962, 1, 1], 
       Hue[0.9214195643235872, 1, 1], 
       Hue[0.9214195643235872, 1, 1], 
       Hue[0.013054762008105212`, 1, 1], 
       Hue[0.013054762008105212`, 1, 1], 
       Hue[0.10496038717414542`, 1, 1], 
       Hue[0.10496038717414542`, 1, 1], 
       Hue[0.19751458917685086`, 1, 1], 
       Hue[0.19751458917685086`, 1, 1], 
       Hue[0.2897640620074146, 1, 1], 
       Hue[0.2897640620074146, 1, 1], 
       Hue[0.3824828729642137, 1, 1], 
       Hue[0.3824828729642137, 1, 1], 
       Hue[0.4753040283446231, 1, 1], 
       Hue[0.4753040283446231, 1, 1], 
       Hue[0.5684100351355065, 1, 1], 
       Hue[0.5684100351355065, 1, 1], 
       Hue[0.6616080261793253, 1, 1], 
       Hue[0.6616080261793253, 1, 1], 
       Hue[0.7550855940217207, 1, 1], 
       Hue[0.7550855940217207, 1, 1], 
       Hue[0.8487100201155822, 1, 1], 
       Hue[0.8487100201155822, 1, 1], 
       Hue[0.9425509991658001, 1, 1], 
       Hue[0.9425509991658001, 1, 1], 
       Hue[0.036609959351899514`, 1, 1], 
       Hue[0.036609959351899514`, 1, 1], 
       Hue[0.1308358768599252, 1, 1], 
       Hue[0.1308358768599252, 1, 1], 
       Hue[0.22527489516639876`, 1, 1], 
       Hue[0.22527489516639876`, 1, 1], 
       Hue[0.3199020157682274, 1, 1], 
       Hue[0.3199020157682274, 1, 1], 
       Hue[0.41473227333091395`, 1, 1], 
       Hue[0.41473227333091395`, 1, 1], 
       Hue[0.509757605107152, 1, 1], 
       Hue[0.509757605107152, 1, 1], 
       Hue[0.6049705984441132, 1, 1], 
       Hue[0.6049705984441132, 1, 1], 
       Hue[0.7003881175130392, 1, 1], 
       Hue[0.7003881175130392, 1, 1], 
       Hue[0.7960010298713771, 1, 1], 
       Hue[0.7960010298713771, 1, 1], 
       Hue[0.8918060581525241, 1, 1], 
       Hue[0.8918060581525241, 1, 1], 
       Hue[0.987804312899045, 1, 1], 
       Hue[0.987804312899045, 1, 1], 
       Hue[0.08399480518255445, 1, 1], 
       Hue[0.08399480518255445, 1, 1], 
       Hue[0.18038477209683, 1, 1], 
       Hue[0.18038477209683, 1, 1], 
       Hue[0.27696843286290557`, 1, 1], 
       Hue[0.27696843286290557`, 1, 1], 
       Hue[0.37374369528133805`, 1, 1], 
       Hue[0.37374369528133805`, 1, 1], 
       Hue[0.4707099465669432, 1, 1], 
       Hue[0.4707099465669432, 1, 1], 
       Hue[0.567869481609075, 1, 1], 
       Hue[0.567869481609075, 1, 1], 
       Hue[0.6652240721329183, 1, 1], 
       Hue[0.6652240721329183, 1, 1], 
       Hue[0.7627725097368594, 1, 1], 
       Hue[0.7627725097368594, 1, 1], 
       Hue[0.8605108972765441, 1, 1], 
       Hue[0.8605108972765441, 1, 1], 
       Hue[0.958440389766678, 1, 1], 
       Hue[0.958440389766678, 1, 1], 
       Hue[0.05656129026424089, 1, 1], 
       Hue[0.05656129026424089, 1, 1], 
       Hue[0.15487414605873545`, 1, 1], 
       Hue[0.15487414605873545`, 1, 1], 
       Hue[0.2533794441638736, 1, 1], 
       Hue[0.2533794441638736, 1, 1], 
       Hue[0.35207574889560544`, 1, 1], 
       Hue[0.35207574889560544`, 1, 1], 
       Hue[0.4509616563755431, 1, 1], 
       Hue[0.4509616563755431, 1, 1], 
       Hue[0.5500368479038305, 1, 1], 
       Hue[0.5500368479038305, 1, 1], 
       Hue[0.6493029200071799, 1, 1], 
       Hue[0.6493029200071799, 1, 1], 
       Hue[0.7487594785778681, 1, 1], 
       Hue[0.7487594785778681, 1, 1], 
       Hue[0.8484065019661553, 1, 1], 
       Hue[0.8484065019661553, 1, 1], 
       Hue[0.9482424326521234, 1, 1], 
       Hue[0.9482424326521234, 1, 1], 
       Hue[0.0482667009607877, 1, 1], 
       Hue[0.0482667009607877, 1, 1], 
       Hue[0.14848009586554878`, 1, 1], 
       Hue[0.14848009586554878`, 1, 1], 
       Hue[0.24888253739037916`, 1, 1], 
       Hue[0.24888253739037916`, 1, 1], 
       Hue[0.3494738481964279, 1, 1], 
       Hue[0.3494738481964279, 1, 1], 
       Hue[0.4502538400841612, 1, 1], 
       Hue[0.4502538400841612, 1, 1], 
       Hue[0.5512215313210937, 1, 1], 
       Hue[0.5512215313210937, 1, 1], 
       Hue[0.6523770888719254, 1, 1], 
       Hue[0.6523770888719254, 1, 1], 
       Hue[0.7537211195726249, 1, 1], 
       Hue[0.7537211195726249, 1, 1], 
       Hue[0.8552526263705517, 1, 1], 
       Hue[0.8552526263705517, 1, 1], 
       Hue[0.956971653741671, 1, 1], 
       Hue[0.956971653741671, 1, 1], 
       Hue[0.05887788638436986, 1, 1], 
       Hue[0.05887788638436986, 1, 1], 
       Hue[0.1609711068475429, 1, 1], 
       Hue[0.1609711068475429, 1, 1], 
       Hue[0.263251461198306, 1, 1], 
       Hue[0.263251461198306, 1, 1], 
       Hue[0.36571881160627995`, 1, 1], 
       Hue[0.36571881160627995`, 1, 1], 
       Hue[0.46837235562530183`, 1, 1], 
       Hue[0.46837235562530183`, 1, 1], 
       Hue[0.571212188793546, 1, 1], 
       Hue[0.571212188793546, 1, 1], 
       Hue[0.6742384387634822, 1, 1], 
       Hue[0.6742384387634822, 1, 1], 
       Hue[0.7774505905900797, 1, 1], 
       Hue[0.7774505905900797, 1, 1], 
       Hue[0.88084887240729, 1, 1], 
       Hue[0.88084887240729, 1, 1], 
       Hue[0.9844326836374937, 1, 1], 
       Hue[0.9844326836374937, 1, 1], 
       Hue[0.08820140979164921, 1, 1], 
       Hue[0.08820140979164921, 1, 1], 
       Hue[0.1921553760160884, 1, 1], 
       Hue[0.1921553760160884, 1, 1], 
       Hue[0.2962945397620034, 1, 1], 
       Hue[0.2962945397620034, 1, 1], 
       Hue[0.40061857163953374`, 1, 1], 
       Hue[0.40061857163953374`, 1, 1], 
       Hue[0.5051269690985097, 1, 1], 
       Hue[0.5051269690985097, 1, 1], 
       Hue[0.6098193140164784, 1, 1], 
       Hue[0.6098193140164784, 1, 1], 
       Hue[0.7146956052782951, 1, 1], 
       Hue[0.7146956052782951, 1, 1], 
       Hue[0.8197550348620458, 1, 1], 
       Hue[0.8197550348620458, 1, 1], 
       Hue[0.92499766979028, 1, 1], 
       Hue[0.92499766979028, 1, 1], 
       Hue[0.030422802443574908`, 1, 1], 
       Hue[0.030422802443574908`, 1, 1], 
       Hue[0.13603006751000404`, 1, 1], 
       Hue[0.13603006751000404`, 1, 1], 
       Hue[0.24181881541715228`, 1, 1], 
       Hue[0.24181881541715228`, 1, 1], 
       Hue[0.34778816817848446`, 1, 1], 
       Hue[0.34778816817848446`, 1, 1], 
       Hue[0.4539371094543324, 1, 1], 
       Hue[0.4539371094543324, 1, 1], 
       Hue[0.5602651505544005, 1, 1], 
       Hue[0.5602651505544005, 1, 1], 
       Hue[0.6667712634439584, 1, 1], 
       Hue[0.6667712634439584, 1, 1], 
       Hue[0.7734535461630561, 1, 1], 
       Hue[0.7734535461630561, 1, 1], 
       Hue[0.8803101814100706, 1, 1], 
       Hue[0.8803101814100706, 1, 1], 
       Hue[0.9873409380665711, 1, 1], 
       Hue[0.9873409380665711, 1, 1], 
       Hue[0.09454433494227246, 1, 1], 
       Hue[0.09454433494227246, 1, 1], 
       Hue[0.20191668511525024`, 1, 1], 
       Hue[0.20191668511525024`, 1, 1], 
       Hue[0.3094548457672087, 1, 1], 
       Hue[0.3094548457672087, 1, 1], 
       Hue[0.4171562070936044, 1, 1], 
       Hue[0.4171562070936044, 1, 1], 
       Hue[0.5250179871183952, 1, 1], 
       Hue[0.5250179871183952, 1, 1], 
       Hue[0.6330355272821271, 1, 1], 
       Hue[0.6330355272821271, 1, 1], 
       Hue[0.7412034759242306, 1, 1], 
       Hue[0.7412034759242306, 1, 1], 
       Hue[0.849516723315727, 1, 1], 
       Hue[0.849516723315727, 1, 1], 
       Hue[0.9579680811132738, 1, 1], 
       Hue[0.9579680811132738, 1, 1], 
       Hue[0.06654996696082718, 1, 1], 
       Hue[0.06654996696082718, 1, 1], 
       Hue[0.1752542034971408, 1, 1], 
       Hue[0.1752542034971408, 1, 1], 
       Hue[0.28406982820136284`, 1, 1], 
       Hue[0.28406982820136284`, 1, 1], 
       Hue[0.39298448987471163`, 1, 1], 
       Hue[0.39298448987471163`, 1, 1], 
       Hue[0.5019841493283912, 1, 1], 
       Hue[0.5019841493283912, 1, 1], 
       Hue[0.6110525350493271, 1, 1], 
       Hue[0.6110525350493271, 1, 1], 
       Hue[0.7201713819939524, 1, 1], 
       Hue[0.7201713819939524, 1, 1], 
       Hue[0.8293217984945689, 1, 1], 
       Hue[0.8293217984945689, 1, 1], 
       Hue[0.9384800401343466, 1, 1], 
       Hue[0.9384800401343466, 1, 1], 
       Hue[0.04761399197175331, 1, 1], 
       Hue[0.04761399197175331, 1, 1], 
       Hue[0.15668862421436977`, 1, 1], 
       Hue[0.15668862421436977`, 1, 1], 
       Hue[0.26567158533576374`, 1, 1], 
       Hue[0.26567158533576374`, 1, 1], 
       Hue[0.37452804323486194`, 1, 1], 
       Hue[0.37452804323486194`, 1, 1], 
       Hue[0.48321158195502617`, 1, 1], 
       Hue[0.48321158195502617`, 1, 1], 
       Hue[0.591665801751543, 1, 1], 
       Hue[0.591665801751543, 1, 1], 
       Hue[0.6998335322551498, 1, 1], 
       Hue[0.6998335322551498, 1, 1], 
       Hue[0.8076561474609649, 1, 1], 
       Hue[0.8076561474609649, 1, 1], 
       Hue[0.9150674728730183, 1, 1], 
       Hue[0.9150674728730183, 1, 1], 
       Hue[0.021995142902907924`, 1, 1], 
       Hue[0.021995142902907924`, 1, 1], 
       Hue[0.12836837744267132`, 1, 1], 
       Hue[0.12836837744267132`, 1, 1], 
       Hue[0.2341169635451068, 1, 1], 
       Hue[0.2341169635451068, 1, 1], 
       Hue[0.3391660604792021, 1, 1], 
       Hue[0.3391660604792021, 1, 1], 
       Hue[0.4434364948665046, 1, 1], 
       Hue[0.4434364948665046, 1, 1], 
       Hue[0.546852954234411, 1, 1], 
       Hue[0.546852954234411, 1, 1], 
       Hue[0.6493540844131561, 1, 1], 
       Hue[0.6493540844131561, 1, 1], 
       Hue[0.7508968981911877, 1, 1], 
       Hue[0.7508968981911877, 1, 1], 
       Hue[0.851451910761745, 1, 1], 
       Hue[0.851451910761745, 1, 1], 
       Hue[0.9509963989571802, 1, 1], 
       Hue[0.9509963989571802, 1, 1], 
       Hue[0.04951252491739281, 1, 1], 
       Hue[0.04951252491739281, 1, 1], 
       Hue[0.1469910489023467, 1, 1], 
       Hue[0.1469910489023467, 1, 1], 
       Hue[0.2434442346796176, 1, 1], 
       Hue[0.2434442346796176, 1, 1], 
       Hue[0.33891490119097384`, 1, 1], 
       Hue[0.33891490119097384`, 1, 1], 
       Hue[0.43345638334230663`, 1, 1], 
       Hue[0.43345638334230663`, 1, 1], 
       Hue[0.527095344156284, 1, 1], 
       Hue[0.527095344156284, 1, 1], 
       Hue[0.6198251547570155, 1, 1], 
       Hue[0.6198251547570155, 1, 1], 
       Hue[0.7116325629947977, 1, 1], 
       Hue[0.7116325629947977, 1, 1], 
       Hue[0.8025203294870583, 1, 1], 
       Hue[0.8025203294870583, 1, 1], 
       Hue[0.892489925483982, 1, 1], 
       Hue[0.892489925483982, 1, 1], 
       Hue[0.9814950793701384, 1, 1], 
       Hue[0.9814950793701384, 1, 1], 
       Hue[0.06940879585274738, 1, 1], 
       Hue[0.06940879585274738, 1, 1], 
       Hue[0.1560379167494746, 1, 1], 
       Hue[0.1560379167494746, 1, 1], 
       Hue[0.2411397908261848, 1, 1], 
       Hue[0.2411397908261848, 1, 1], 
       Hue[0.3243570164916532, 1, 1], 
       Hue[0.3243570164916532, 1, 1], 
       Hue[0.40512776526272315`, 1, 1], 
       Hue[0.40512776526272315`, 1, 1], 
       Hue[0.4827689796728388, 1, 1], 
       Hue[0.4827689796728388, 1, 1], 
       Hue[0.5567366414205508, 1, 1], 
       Hue[0.5567366414205508, 1, 1], 
       Hue[0.6268618216854998, 1, 1], 
       Hue[0.6268618216854998, 1, 1], 
       Hue[0.6936551818635579, 1, 1], 
       Hue[0.6936551818635579, 1, 1], 
       Hue[0.7588789911312471, 1, 1], 
       Hue[0.7588789911312471, 1, 1], 
       Hue[0.8254234195078273, 1, 1], 
       Hue[0.8254234195078273, 1, 1], 
       Hue[0.8957871458702045, 1, 1], 
       Hue[0.8957871458702045, 1, 1], 
       Hue[0.9712803960295606, 1, 1], 
       Hue[0.9712803960295606, 1, 1], 
       Hue[0.0526909438225915, 1, 1], 
       Hue[0.0526909438225915, 1, 1], 
       Hue[0.14028738520395323`, 1, 1], 
       Hue[0.14028738520395323`, 1, 1], 
       Hue[0.23330222879300616`, 1, 1], 
       Hue[0.23330222879300616`, 1, 1], 
       Hue[0.3304408906001906, 1, 1], 
       Hue[0.3304408906001906, 1, 1], 
       Hue[0.43089611044311255`, 1, 1], 
       Hue[0.43089611044311255`, 1, 1], 
       Hue[0.5345867051543077, 1, 1], 
       Hue[0.5345867051543077, 1, 1], 
       Hue[0.6416275575907403, 1, 1], 
       Hue[0.6416275575907403, 1, 1], 
       Hue[0.7519045675691594, 1, 1], 
       Hue[0.7519045675691594, 1, 1], 
       Hue[0.865098064379245, 1, 1], 
       Hue[0.865098064379245, 1, 1], 
       Hue[0.9808402665833201, 1, 1], 
       Hue[0.9808402665833201, 1, 1], 
       Hue[0.09878598465087692, 1, 1], 
       Hue[0.09878598465087692, 1, 1], 
       Hue[0.2186316551202424, 1, 1], 
       Hue[0.2186316551202424, 1, 1], 
       Hue[0.3401338285178514, 1, 1], 
       Hue[0.3401338285178514, 1, 1], 
       Hue[0.4631214068126678, 1, 1], 
       Hue[0.4631214068126678, 1, 1], 
       Hue[0.5874801753732921, 1, 1], 
       Hue[0.5874801753732921, 1, 1], 
       Hue[0.7130934559046723, 1, 1], 
       Hue[0.7130934559046723, 1, 1], 
       Hue[0.8397487315274839, 1, 1], 
       Hue[0.8397487315274839, 1, 1], 
       Hue[0.9670748643982374, 1, 1], 
       Hue[0.9670748643982374, 1, 1]}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->{True, None},
    AxesOrigin->{0, 0},
    Epilog->
     LineBox[{{1, 0.00002891020892332036}, {2, 0.00004092709715677925}, {
       3, 0.0000578442623831742}, {4, 0.00008132170006101293}, {
       5, 0.0001132123506546545}, {6, 0.00015732521161777313`}, {
       7, 0.00021706148730958605`}, {8, 0.00029679391155740205`}, {
       9, 0.0004044437869046104}, {10, 0.0005472342408465536}, {
       11, 0.0007357821766619172}, {12, 0.0009829499454222842}, {
       13, 0.0013059764142051215`}, {14, 0.0017232519495184514`}, {
       15, 0.002260713272967188}, {16, 0.002946800006478862}, {
       17, 0.0038173768386365785`}, {18, 0.0049149116761541415`}, {
       19, 0.006288876610526376}, {20, 0.007996683145507118}, {
       21, 0.010105666130880783`}, {22, 0.012691372899594027`}, {
       23, 0.015840722427496182`}, {24, 0.019649107316154976`}, {
       25, 0.02422033336326641}, {26, 0.029673292220142568`}, {
       27, 0.0361270537731897}, {28, 0.04371311709373623}, {
       29, 0.052564193855269684`}, {30, 0.06281535460071211}, {
       31, 0.07460291718896549}, {32, 0.0880555151108855}, {
       33, 0.10328974726463552`}, {34, 0.12040811749593244`}, {
       35, 0.1394954020401599}, {36, 0.16061091395511376`}, {
       37, 0.18378087874642873`}, {38, 0.2089922199749991}, {
       39, 0.2361917925602136}, {40, 0.2652790478412556}, {
       41, 0.29610895455565495`}, {42, 0.3284817261818962}, {
       43, 0.3621433275612805}, {44, 0.3967865039045263}, {
       45, 0.4320552259778682}, {46, 0.46755521182451004`}, {
       47, 0.502848612260845}, {48, 0.5374715173738412}, {
       49, 0.5709318093085728}, {50, 0.6027321885660719}, {
       51, 0.6323769993170906}, {52, 0.6593878623229199}, {
       53, 0.683314527710014}, {54, 0.7037438284627603}, {
       55, 0.720314922534282}, {56, 0.7327336595044417}, {
       57, 0.7407756379784874}, {58, 0.7442929847235761}, {
       59, 0.7432220041413445}, {60, 0.7375829745887565}, {
       61, 0.727482398832054}, {62, 0.713104883727343}, {
       63, 0.6947127448381861}, {64, 0.672633440982742}, {
       65, 0.6472501550561022}, {66, 0.6189974934494784}, {
       67, 0.5883416268738719}, {68, 0.5557685597855335}, {
       69, 0.5217745903424703}, {70, 0.48685238284288923`}, {
       71, 0.45147933044212796`}, {72, 0.41610763357236824`}, {
       73, 0.3811551145094326}, {74, 0.3469971943359706}, {
       75, 0.31396435207500445`}, {76, 0.28233459460566235`}, {
       77, 0.25233526155361025`}, {78, 0.22414177952445052`}, {
       79, 0.1978791948170029}, {80, 0.17362383911706927`}, {
       81, 0.15140880388809472`}, {82, 0.13122793688418682`}, {
       83, 0.11304092722139888`}, {84, 0.09677834565813032}, {
       85, 0.08234803303136913}, {86, 0.06964079160188397}, {
       87, 0.05853433492781175}, {88, 0.048897844664849976`}, {
       89, 0.04059741069384246}, {90, 0.03349942904664628}, {
       91, 0.027473050215111466`}, {92, 0.022392600022269244`}, {
       93, 0.018139521535058042`}, {94, 0.014603855708234886`}, {
       95, 0.011684918935038858`}, {96, 0.009291702582069678}, {
       97, 0.007342946920873598}, {98, 0.0057668916461557414`}, {
       99, 0.004500893449613142}, {100, 0.003490844303002224}, {
       101, 0.002690439568320477}, {102, 0.002060441646598003}, {
       103, 0.0015679265151644249`}, {104, 0.001185481301271069}, {
       105, 0.0008904909429065257}, {106, 0.0006644926244939608}, {
       107, 0.0004925486128969269}, {108, 0.00036263689520873944`}, {
       109, 0.0002651530607565262}, {110, 0.00019250490900931877`}, {
       111, 0.00013874917878154179`}, {112, 0.00009926295614784529}, {
       113, 0.00007047173943797045}, {114, 0.000049635798552467466`}, {
       115, 0.00003467388849944929}, {116, 0.000024016154323433876`}, {
       117, 0.000016486707346631874`}, {118, 0.000011212525533547908`}, {
       119, 7.551223400127575*^-6}, {120, 5.033782062115359*^-6}, {
       121, 3.3202087014511472`*^-6}, {122, 2.1659889767730572`*^-6}, {
       123, 1.3969490532916797`*^-6}, {124, 8.903416292425542*^-7}, {
       125, 5.605784753529043*^-7}, {126, 3.4860769672483537`*^-7}, {
       127, 2.1411714295504502`*^-7}, {128, 1.298982749564206*^-7}, {
       129, 7.783615867321636*^-8}, {130, 4.606425442606486*^-8}, {
       131, 2.6928475360203448`*^-8}, {132, 1.5556195071803505`*^-8}, {
       133, 8.885186758671052*^-9}, {134, 5.0196350717437815`*^-9}, {
       135, 2.8055407531168087`*^-9}, {136, 1.5517446726957088`*^-9}, {
       137, 8.498258748257083*^-10}, {138, 4.6119296558491886`*^-10}, {
       139, 2.4824138882945997`*^-10}, {140, 1.3277177808329952`*^-10}, {
       141, 7.086064999210945*^-11}, {142, 3.801026333431942*^-11}, {
       143, 2.0696721414753352`*^-11}, {144, 1.1582566254040135`*^-11}, {
       145, 6.7437210789930295`*^-12}, {146, 4.101660830478816*^-12}, {
       147, 2.588258987503109*^-12}, {148, 1.677398476264057*^-12}, {
       149, 1.1056530443190712`*^-12}, {150, 7.326545945600222*^-13}, {
       151, 4.829412105180564*^-13}, {152, 3.1537391770488354`*^-13}, {
       153, 2.043063784043066*^-13}, {154, 1.314820599904095*^-13}, {
       155, 8.394517153718718*^-14}, {156, 5.302690052076813*^-14}, {
       157, 3.306900582764145*^-14}, {158, 2.033386408969542*^-14}, {
       159, 1.2320213377579618`*^-14}, {160, 7.354143039946028*^-15}, {
       161, 4.325176770077754*^-15}, {162, 2.5066362010932763`*^-15}, {
       163, 1.4312908779257063`*^-15}, {164, 8.046569771216889*^-16}, {
       165, 4.44886744753384*^-16}, {166, 2.416796690954281*^-16}, {
       167, 1.290153750284501*^-16}, {168, 6.779355076761803*^-17}, {
       169, 3.5168030021614194`*^-17}, {170, 1.8063215544407602`*^-17}, {
       171, 9.199179135910566*^-18}, {172, 4.641987822757133*^-18}, {
       173, 2.3174779056761736`*^-18}, {174, 1.1452059894028952`*^-18}, {
       175, 5.624104126413987*^-19}, {176, 2.7599947295600498`*^-19}, {
       177, 1.3560619278634575`*^-19}, {178, 6.642886965129633*^-20}, {
       179, 3.2237954242053554`*^-20}, {180, 1.546624066782157*^-20}, {
       181, 7.370221724968152*^-21}, {182, 3.514752502332634*^-21}, {
       183, 1.6802961176895993`*^-21}, {184, 7.991471929231893*^-22}, {
       185, 3.7399007907499105`*^-22}, {186, 1.7145451406333492`*^-22}, {
       187, 7.761154651199172*^-23}, {188, 3.52984693233935*^-23}, {
       189, 1.6326750179855814`*^-23}, {190, 7.619470907545238*^-24}, {
       191, 3.509935742563039*^-24}, {192, 1.568446653381133*^-24}, {
       193, 6.7949655651238565`*^-25}, {194, 2.9111091180487185`*^-25}, {
       195, 1.2710382446206862`*^-25}, {196, 5.71941839854841*^-26}, {
       197, 2.596278265435298*^-26}, {198, 1.1558779943164204`*^-26}, {
       199, 4.9935204122097425`*^-27}, {200, 2.1103569849562185`*^-27}, {
       201, 8.838585429776227*^-28}, {202, 3.668692009429326*^-28}, {
       203, 1.4892269509956578`*^-28}, {204, 5.90475119302719*^-29}, {
       205, 2.390957975287992*^-29}, {206, 1.0603451847817696`*^-29}, {
       207, 5.060315596891892*^-30}, {208, 2.3681572225553716`*^-30}, {
       209, 1.0163040672217539`*^-30}, {210, 3.8457874062906504`*^-31}, {
       211, 1.2256088027958108`*^-31}, {212, 3.0687848060388567`*^-32}, {
       213, 9.531978210287135*^-33}, {214, 6.97059595648626*^-33}, {
       215, 4.267400202337467*^-33}, {216, 2.1179288063670586`*^-33}, {
       217, 8.987222616149098*^-34}, {218, 3.2995954930822204`*^-34}, {
       219, 1.01816079322637*^-34}, {220, 2.356542731308659*^-35}, {
       221, 4.2842353129339825`*^-36}, {222, 4.054623859961678*^-36}, {
       223, 2.8234781589665136`*^-36}, {224, 1.499188288927744*^-36}, {
       225, 6.793560946884133*^-37}, {226, 2.733625636757853*^-37}, {
       227, 9.934019281650187*^-38}, {228, 3.320485400670432*^-38}, {
       229, 1.0831024208197172`*^-38}, {230, 3.945846408887182*^-39}, {
       231, 1.688584083746897*^-39}, {232, 7.477084301291403*^-40}, {
       233, 3.1290048397775705`*^-40}, {234, 1.2107924998030126`*^-40}, {
       235, 4.3371552934346244`*^-41}, {236, 1.4474842703160065`*^-41}, {
       237, 4.5536452248293894`*^-42}, {238, 1.370370313465693*^-42}, {
       239, 3.962876021177806*^-43}, {240, 1.0658162261243229`*^-43}, {
       241, 4.261882896279717*^-44}, {242, 3.073287462511899*^-44}, {
       243, 2.2156474144813128`*^-44}, {244, 2.1019476964872256`*^-44}, {
       245, 2.1019476964872256`*^-44}, {246, 2.1019476964872256`*^-44}, {
       247, 2.1019476964872256`*^-44}, {248, 2.1019476964872256`*^-44}, {
       249, 2.1019476964872256`*^-44}, {250, 2.1019476964872256`*^-44}, {
       251, 2.1019476964872256`*^-44}}],
    Frame->True,
    FrameTicks->{{{1., 
        FormBox[
         RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
        FormBox[
         RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
        FormBox[
         RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
        FormBox[
         RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
        FormBox["0", TraditionalForm]}, {157.25, 
        FormBox["2.5`", TraditionalForm]}, {188.5, 
        FormBox["5", TraditionalForm]}, {219.75, 
        FormBox["7.5`", TraditionalForm]}, {251., 
        FormBox["10", TraditionalForm]}}, Automatic, None, None},
    PlotRange->{-0.1, 1.1},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], Automatic}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF2Xl0jPceBvDfEQ0uCWm1dRF36maxVZFELBFPFmSVyCr7m5lM1NYoEm1V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     "], {
      {GrayLevel[0.5], EdgeForm[None], 
       GraphicsGroupBox[{PolygonBox[{{179, 608, 180}}], 
         PolygonBox[CompressedData["
1:eJwtlgOQnVkQRrv/2LaNiW3bmWhibrBIstEmu0k2uxvbts21bdu27eB0fVM1
p+pMd1+82/e+euXGTEmdnJjZecgIGcCcP9f/kbsEl9Nz2SAnuVyQPX1MgmeA
TCYyhkNmU01uPA/kwLNAVjwbZDXVZMazeHouxkSta60YkxfPBznx/FAaLwMF
8FxQEC8EuaMWiuMlYgyeB4rixVy5iJXES7nmSoE+eF+oYelrxNyuuWONsng5
KGiiPF4h1jRREa8EhU1UxqtAERNV8WpQFC8GNfCaUDzmhmZ481gDLwF18LpQ
0kQ9vH7s2UQDvCGUNs1RC6/tGlsGmuBNXXNHTSO8sSsXe6iOp7j2Emu2wFtC
ebwCtMHbQkVTrBXe2pWLWDu8fXxmEx3wjlDZ1IPC8dldZx+xTnjnOJM4C+iG
d48zMcW64F1duerQC+/t6k3U9MB7unJpcBw/AYNNnMRPwZA4CxiID4ozibOE
IfjQOFNTLC3GunKNYDw+Ic7IVDMMHw71TIzAR0YPTDUT8UnQJHoBY/Cx0RNT
zSh8tCsXsXH4Va61YszV+DXRI7w1zMbnQJu4CzAZnxJ3Am8JM/CZ0Cp6BdPw
6a5cxGbhN7jmqgn98QGus4g1rsWvc80da9yI3xQ9NjEXnwftTMzHb44em1iA
3wIdTNyK3xY9j97CYnxJ9BjvCRvxTdAregvL8RXRYxMr8VXQzcRqfE3cCdMc
S/FlrrE9YD2+wTV31KzF17lysYeF+CLXXmLNzfiWuEPxtmE7vgP6mmJb8W2u
XMR24rsg1cRufA/0M30npIa7zjZie/F90B8fAAfxQzDQFNuPH3DlBsFR/Jjr
7kbNYfyIKxc9nopf7+pt3OHT+Jm4s/imeHN8CVeCzfhwuJ3cHTAi7iqcxy+4
cmPhYfwRGGequRO/K+6wibvxe2CUqeZR/LG4o/houB9/AMaYau7F73PlIvYg
/pBrrRjzOP4EjI+7Bq/ir8WdwyfA0/gzMDHuPryIvxRvAJ8Ez+MvuHIRexl/
xTXXIsjIZ88Ei01rPEnuKdfcscbr+Bsw2cSb+Fvxhky8jb8DU028i78XZ27i
ffwDmIZPh4/xT2BGvBX4Ef8p3gw+Ez7Hv4BZJr7Ev4o3aOJr/BuYbZrjU/wz
19g58D3+g2vuqPkW/86Viz18iH/k2kus+TP+C8zF58Hv+B8w3xT7Ff/NlYvY
n/hf8YZN/I3/AwtMPXgWf8519hH7F/8v3ni8bbiEX443bor9j1905RZCQi8y
JOpN1MSPBE+Ui98RI/CRcMF0h88y9pzrri6F7ORywDJ8OeTG88AKUywnnitR
bi0Ux0vAOlNNXjwfrDSRHy8Aq0w1JfFSsB5fDYXxIrDGVFMQL5QoF7GieLFE
a8WY0ngZ2IBvgxp4TdiOb4TyeIVEb3MLVMWrwVbTG62MV0mUi1h1PCXRXEsg
K54t0VnEGmXxconmjjVq4bVhh4k6eF3YaaIeXh92mWiAN4TdJhrhjWEPvhea
4c1hH34EuuBd4Si+H1rhreGAiTZ4Wzhooh3eHg6Z5miBt0w09jB0wjsnmjtq
OuAdE+ViD03wpon2Emt2w7vDMfw49MJ7wwlTrAfeM1EuYn3wvnDSRCreD06Z
vhMy41kSnW3E+uMD4DR+BtLwwXDWFBuID0qUOwfD8OGJ7m7UDMGHJsp53H+4
qGtuVwBRgRao
          "]]}]}, {{}, {}, {
        {Hue[0.2740759612382822, 1, 1], LineBox[{1, 2}]}, 
        {Hue[0.34061641190314307`, 1, 1], LineBox[{2, 3}]}, 
        {Hue[0.42976605858521966`, 1, 1], LineBox[{3, 4}]}, 
        {Hue[0.5138753870902316, 1, 1], LineBox[{4, 5}]}, 
        {Hue[0.5840491086657771, 1, 1], LineBox[{5, 6}]}, 
        {Hue[0.6852575862750414, 1, 1], LineBox[{6, 7}]}, 
        {Hue[0.7690215103953824, 1, 1], LineBox[{7, 8}]}, 
        {Hue[0.8448677927359287, 1, 1], LineBox[{8, 9}]}, 
        {Hue[0.9361130126135394, 1, 1], LineBox[{9, 10}]}, 
        {Hue[0.018348444993657845`, 1, 1], LineBox[{10, 11}]}, 
        {Hue[0.1067341007690407, 1, 1], LineBox[{11, 12}]}, 
        {Hue[0.1916021807806745, 1, 1], LineBox[{12, 13}]}, 
        {Hue[0.27772484219464055`, 1, 1], LineBox[{13, 14}]}, 
        {Hue[0.3643349903317177, 1, 1], LineBox[{14, 15}]}, 
        {Hue[0.45177282749741526`, 1, 1], LineBox[{15, 16}]}, 
        {Hue[0.538908701070393, 1, 1], LineBox[{16, 17}]}, 
        {Hue[0.6261843843359687, 1, 1], LineBox[{17, 18}]}, 
        {Hue[0.7137393391661621, 1, 1], LineBox[{18, 19}]}, 
        {Hue[0.8018583969042162, 1, 1], LineBox[{19, 20}]}, 
        {Hue[0.8904918701851595, 1, 1], LineBox[{20, 21}]}, 
        {Hue[0.9790975713397599, 1, 1], LineBox[{21, 22}]}, 
        {Hue[0.06789470579575665, 1, 1], LineBox[{22, 23}]}, 
        {Hue[0.15697474778197495`, 1, 1], LineBox[{23, 24}]}, 
        {Hue[0.24655060468658918`, 1, 1], LineBox[{24, 25}]}, 
        {Hue[0.3363665345986199, 1, 1], LineBox[{25, 26}]}, 
        {Hue[0.4264547735004385, 1, 1], LineBox[{26, 27}]}, 
        {Hue[0.5167730399489824, 1, 1], LineBox[{27, 28}]}, 
        {Hue[0.6073369670309214, 1, 1], LineBox[{28, 29}]}, 
        {Hue[0.6982587635747064, 1, 1], LineBox[{29, 30}]}, 
        {Hue[0.7894629494103195, 1, 1], LineBox[{30, 31}]}, 
        {Hue[0.8809698434134233, 1, 1], LineBox[{31, 32}]}, 
        {Hue[0.9727289977353122, 1, 1], LineBox[{32, 33}]}, 
        {Hue[0.06475513559485649, 1, 1], LineBox[{33, 34}]}, 
        {Hue[0.15706346690015022`, 1, 1], LineBox[{34, 35}]}, 
        {Hue[0.24967597355959564`, 1, 1], LineBox[{35, 36}]}, 
        {Hue[0.34256934792907295`, 1, 1], LineBox[{36, 37}]}, 
        {Hue[0.43574581435487925`, 1, 1], LineBox[{37, 38}]}, 
        {Hue[0.5291912090168738, 1, 1], LineBox[{38, 39}]}, 
        {Hue[0.6229120360801792, 1, 1], LineBox[{39, 40}]}, 
        {Hue[0.7169177088892706, 1, 1], LineBox[{40, 41}]}, 
        {Hue[0.8112095824461873, 1, 1], LineBox[{41, 42}]}, 
        {Hue[0.9057801263860004, 1, 1], LineBox[{42, 43}]}, 
        {Hue[0.0006257096597658727, 1, 1], LineBox[{43, 44}]}, 
        {Hue[0.09574830413526374, 1, 1], LineBox[{44, 45}]}, 
        {Hue[0.19114909128921104`, 1, 1], LineBox[{45, 46}]}, 
        {Hue[0.2868292849773195, 1, 1], LineBox[{46, 47}]}, 
        {Hue[0.3827869813995296, 1, 1], LineBox[{47, 48}]}, 
        {Hue[0.47902067137753845`, 1, 1], LineBox[{48, 49}]}, 
        {Hue[0.5755290682770862, 1, 1], LineBox[{49, 50}]}, 
        {Hue[0.6723137955010745, 1, 1], LineBox[{50, 51}]}, 
        {Hue[0.7693745554168671, 1, 1], LineBox[{51, 52}]}, 
        {Hue[0.8667116851118895, 1, 1], LineBox[{52, 53}]}, 
        {Hue[0.9643234476361405, 1, 1], LineBox[{53, 54}]}, 
        {Hue[0.062208530739026324`, 1, 1], LineBox[{54, 55}]}, 
        {Hue[0.16036799852267106`, 1, 1], LineBox[{55, 56}]}, 
        {Hue[0.258801810944857, 1, 1], LineBox[{56, 57}]}, 
        {Hue[0.3575091461923081, 1, 1], LineBox[{57, 58}]}, 
        {Hue[0.4564888992590564, 1, 1], LineBox[{58, 59}]}, 
        {Hue[0.5557409571993345, 1, 1], LineBox[{59, 60}]}, 
        {Hue[0.655265089711039, 1, 1], LineBox[{60, 61}]}, 
        {Hue[0.7550614056935411, 1, 1], LineBox[{61, 62}]}, 
        {Hue[0.8551296658837929, 1, 1], LineBox[{62, 63}]}, 
        {Hue[0.9554684222196816, 1, 1], LineBox[{63, 64}]}, 
        {Hue[0.056077780299213835`, 1, 1], LineBox[{64, 65}]}, 
        {Hue[0.1569576070864384, 1, 1], LineBox[{65, 66}]}, 
        {Hue[0.2581074846488935, 1, 1], LineBox[{66, 67}]}, 
        {Hue[0.35952720768489815`, 1, 1], LineBox[{67, 68}]}, 
        {Hue[0.4612154447314414, 1, 1], LineBox[{68, 69}]}, 
        {Hue[0.5631724067610944, 1, 1], LineBox[{69, 70}]}, 
        {Hue[0.6653976393554915, 1, 1], LineBox[{70, 71}]}, 
        {Hue[0.7678905877870311, 1, 1], LineBox[{71, 72}]}, 
        {Hue[0.8706510630208145, 1, 1], LineBox[{72, 73}]}, 
        {Hue[0.9736784294934824, 1, 1], LineBox[{73, 74}]}, 
        {Hue[0.07697211848436797, 1, 1], LineBox[{74, 75}]}, 
        {Hue[0.18053149069666805`, 1, 1], LineBox[{75, 76}]}, 
        {Hue[0.28435568477136225`, 1, 1], LineBox[{76, 77}]}, 
        {Hue[0.3884441911870997, 1, 1], LineBox[{77, 78}]}, 
        {Hue[0.4927967517078755, 1, 1], LineBox[{78, 79}]}, 
        {Hue[0.5974123653015715, 1, 1], LineBox[{79, 80}]}, 
        {Hue[0.7022899159669522, 1, 1], LineBox[{80, 81}]}, 
        {Hue[0.8074287288457369, 1, 1], LineBox[{81, 82}]}, 
        {Hue[0.9128275507242074, 1, 1], LineBox[{82, 83}]}, 
        {Hue[0.018485500919929655`, 1, 1], LineBox[{83, 84}]}, 
        {Hue[0.12440106059642532`, 1, 1], LineBox[{84, 85}]}, 
        {Hue[0.2305726939366271, 1, 1], LineBox[{85, 86}]}, 
        {Hue[0.3369984854894055, 1, 1], LineBox[{86, 87}]}, 
        {Hue[0.4436764093552376, 1, 1], LineBox[{87, 88}]}, 
        {Hue[0.5506045244741555, 1, 1], LineBox[{88, 89}]}, 
        {Hue[0.6577800886416443, 1, 1], LineBox[{89, 90}]}, 
        {Hue[0.7652001869956759, 1, 1], LineBox[{90, 91}]}, 
        {Hue[0.8728611074438166, 1, 1], LineBox[{91, 92}]}, 
        {Hue[0.9807589144955988, 1, 1], LineBox[{92, 93}]}, 
        {Hue[0.08888919496304422, 1, 1], LineBox[{93, 94}]}, 
        {Hue[0.1972463957493058, 1, 1], LineBox[{94, 95}]}, 
        {Hue[0.30582418937682976`, 1, 1], LineBox[{95, 96}]}, 
        {Hue[0.41461542530876894`, 1, 1], LineBox[{96, 97}]}, 
        {Hue[0.523611607046055, 1, 1], LineBox[{97, 98}]}, 
        {Hue[0.6328035280323666, 1, 1], LineBox[{98, 99}]}, 
        {Hue[0.7421791292100071, 1, 1], LineBox[{99, 100}]}, 
        {Hue[0.8517262210776776, 1, 1], LineBox[{100, 101}]}, 
        {Hue[0.9614313829679946, 1, 1], LineBox[{101, 102}]}, 
        {Hue[0.07127581809817282, 1, 1], LineBox[{102, 103}]}, 
        {Hue[0.18123831391879858`, 1, 1], LineBox[{103, 104}]}, 
        {Hue[0.2912992515505204, 1, 1], LineBox[{104, 105}]}, 
        {Hue[0.4014357821497693, 1, 1], LineBox[{105, 106}]}, 
        {Hue[0.5116154678989273, 1, 1], LineBox[{106, 107}]}, 
        {Hue[0.6217988884400221, 1, 1], LineBox[{107, 108}]}, 
        {Hue[0.7319483440755872, 1, 1], LineBox[{108, 109}]}, 
        {Hue[0.8420262140980949, 1, 1], LineBox[{109, 110}]}, 
        {Hue[0.9519858790730588, 1, 1], LineBox[{110, 111}]}, 
        {Hue[0.061770917376315, 1, 1], LineBox[{111, 112}]}, 
        {Hue[0.17131950476346894`, 1, 1], LineBox[{112, 113}]}, 
        {Hue[0.2805652433311565, 1, 1], LineBox[{113, 114}]}, 
        {Hue[0.3894387845260012, 1, 1], LineBox[{114, 115}]}, 
        {Hue[0.497868118280707, 1, 1], LineBox[{115, 116}]}, 
        {Hue[0.6057737050224179, 1, 1], LineBox[{116, 117}]}, 
        {Hue[0.7130713214880129, 1, 1], LineBox[{117, 118}]}, 
        {Hue[0.8196801378961807, 1, 1], LineBox[{118, 119}]}, 
        {Hue[0.9255229902660722, 1, 1], LineBox[{119, 120}]}, 
        {Hue[0.030529801027407266`, 1, 1], LineBox[{120, 121}]}, 
        {Hue[0.13465081125510253`, 1, 1], LineBox[{121, 122}]}, 
        {Hue[0.23784994238387336`, 1, 1], LineBox[{122, 123}]}, 
        {Hue[0.34007354031929565`, 1, 1], LineBox[{123, 124}]}, 
        {Hue[0.4412627880888048, 1, 1], LineBox[{124, 125}]}, 
        {Hue[0.5414319533204015, 1, 1], LineBox[{125, 126}]}, 
        {Hue[0.6406952567590228, 1, 1], LineBox[{126, 127}]}, 
        {Hue[0.7391561792610386, 1, 1], LineBox[{127, 128}]}, 
        {Hue[0.8368067224869767, 1, 1], LineBox[{128, 129}]}, 
        {Hue[0.933611299963676, 1, 1], LineBox[{129, 130}]}, 
        {Hue[0.02966183917474022, 1, 1], LineBox[{130, 131}]}, 
        {Hue[0.12515565615431143`, 1, 1], LineBox[{131, 132}]}, 
        {Hue[0.22025100378644757`, 1, 1], LineBox[{132, 133}]}, 
        {Hue[0.3150509908005036, 1, 1], LineBox[{133, 134}]}, 
        {Hue[0.409700986425785, 1, 1], LineBox[{134, 135}]}, 
        {Hue[0.5043456473843696, 1, 1], LineBox[{135, 136}]}, 
        {Hue[0.5989362670288179, 1, 1], LineBox[{136, 137}]}, 
        {Hue[0.6932230603805059, 1, 1], LineBox[{137, 138}]}, 
        {Hue[0.7871257717961646, 1, 1], LineBox[{138, 139}]}, 
        {Hue[0.8811210035608937, 1, 1], LineBox[{139, 140}]}, 
        {Hue[0.976043668502006, 1, 1], LineBox[{140, 141}]}, 
        {Hue[0.07233130360111649, 1, 1], LineBox[{141, 142}]}, 
        {Hue[0.1694413486033889, 1, 1], LineBox[{142, 143}]}, 
        {Hue[0.2660171617229767, 1, 1], LineBox[{143, 144}]}, 
        {Hue[0.36108283356206833`, 1, 1], LineBox[{144, 145}]}, 
        {Hue[0.45592188250768106`, 1, 1], LineBox[{145, 146}]}, 
        {Hue[0.5541536405087248, 1, 1], LineBox[{146, 147}]}, 
        {Hue[0.6582616045411482, 1, 1], LineBox[{147, 148}]}, 
        {Hue[0.766613722852846, 1, 1], LineBox[{148, 149}]}, 
        {Hue[0.874029838680562, 1, 1], LineBox[{149, 150}]}, 
        {Hue[0.9752100114454223, 1, 1], LineBox[{150, 151}]}, 
        {Hue[0.07288244035433329, 1, 1], LineBox[{151, 152}]}, 
        {Hue[0.18183762518900837`, 1, 1], LineBox[{152, 153}]}, 
        {Hue[0.3128118513040628, 1, 1], LineBox[{153, 154}]}, 
        {Hue[0.4627513612045987, 1, 1], LineBox[{154, 155}]}, 
        {Hue[0.6188156983257607, 1, 1], LineBox[{155, 156}]}, 
        {Hue[0.7659699254591066, 1, 1], LineBox[{156, 157}]}, 
        {Hue[0.900255093224109, 1, 1], LineBox[{157, 158}]}, 
        {Hue[0.027221273580469205`, 1, 1], LineBox[{158, 159}]}, 
        {Hue[0.14925197585231578`, 1, 1], LineBox[{159, 160}]}, 
        {Hue[0.2642182068786057, 1, 1], LineBox[{160, 161}]}, 
        {Hue[0.37219080794297776`, 1, 1], LineBox[{161, 162}]}, 
        {Hue[0.47834136069253086`, 1, 1], LineBox[{162, 163}]}, 
        {Hue[0.5866752936506326, 1, 1], LineBox[{163, 164}]}, 
        {Hue[0.6967194221160611, 1, 1], LineBox[{164, 165}]}, 
        {Hue[0.8091853189345573, 1, 1], LineBox[{165, 166}]}, 
        {Hue[0.9280958405079539, 1, 1], LineBox[{166, 167}]}, 
        {Hue[0.05495597451428205, 1, 1], LineBox[{167, 168}]}, 
        {Hue[0.1858218646652519, 1, 1], LineBox[{168, 169}]}, 
        {Hue[0.31408752473670615`, 1, 1], LineBox[{169, 170}]}, 
        {Hue[0.43511710210004056`, 1, 1], LineBox[{170, 171}]}, 
        {Hue[0.5521633524083553, 1, 1], LineBox[{171, 172}]}, 
        {Hue[0.6750664035148826, 1, 1], LineBox[{172, 173}]}, 
        {Hue[0.8088793717674257, 1, 1], LineBox[{173, 174}]}, 
        {Hue[0.9514016888140933, 1, 1], LineBox[{174, 175}]}, 
        {Hue[0.09849358523699003, 1, 1], LineBox[{175, 176}]}, 
        {Hue[0.2469004214429708, 1, 1], LineBox[{176, 177}]}, 
        {Hue[0.3946861926401874, 1, 1], LineBox[{177, 178}]}, 
        {Hue[0.5410819268855549, 1, 1], LineBox[{178, 179}]}, 
        {Hue[0.6862933635086946, 1, 1], LineBox[{179, 180}]}, 
        {Hue[0.8313368849143955, 1, 1], LineBox[{180, 181}]}, 
        {Hue[0.9778423483567932, 1, 1], LineBox[{181, 182}]}, 
        {Hue[0.1276628869951627, 1, 1], LineBox[{182, 183}]}, 
        {Hue[0.28227537150474946`, 1, 1], LineBox[{183, 184}]}, 
        {Hue[0.44229630855671426`, 1, 1], LineBox[{184, 185}]}, 
        {Hue[0.6073850295098555, 1, 1], LineBox[{185, 186}]}, 
        {Hue[0.7763711353670936, 1, 1], LineBox[{186, 187}]}, 
        {Hue[0.9471867145675702, 1, 1], LineBox[{187, 188}]}, 
        {Hue[0.11600160479303977`, 1, 1], LineBox[{188, 189}]}, 
        {Hue[0.27387095448388904`, 1, 1], LineBox[{189, 190}]}, 
        {Hue[0.3962254851792768, 1, 1], LineBox[{190, 191}]}, 
        {Hue[0.46305170608323365`, 1, 1], LineBox[{191, 192}]}, 
        {Hue[0.5694959575492422, 1, 1], LineBox[{192, 193}]}, 
        {Hue[0.7301243631668375, 1, 1], LineBox[{193, 194}]}, 
        {Hue[0.9137879918990288, 1, 1], LineBox[{194, 195}]}, 
        {Hue[0.1106167560401981, 1, 1], LineBox[{195, 196}]}, 
        {Hue[0.7913631345256851, 1, 1], LineBox[{196, 197}]}, 
        {Hue[0.0028615328745636315`, 1, 1], LineBox[{197, 198}]}, 
        {Hue[0.2096236893305576, 1, 1], LineBox[{198, 199}]}, 
        {Hue[0.4177109221353927, 1, 1], LineBox[{199, 200}]}, 
        {Hue[0.6269797211930057, 1, 1], LineBox[{200, 201}]}, 
        {Hue[0.8360059584538889, 1, 1], LineBox[{201, 202}]}, 
        {Hue[0.0348818559659671, 1, 1], LineBox[{202, 203}]}, 
        {Hue[0.847839796942091, 1, 1], LineBox[{203, 204}]}, 
        {Hue[0.020982790830345505`, 1, 1], LineBox[{204, 205}]}, 
        {Hue[0.24158613432752232`, 1, 1], LineBox[{205, 206}]}, 
        {Hue[0.4741393505207045, 1, 1], LineBox[{206, 207}]}, 
        {Hue[0.7208471264822096, 1, 1], LineBox[{207, 208}]}, 
        {Hue[0.9892888113983458, 1, 1], LineBox[{208, 209}]}, 
        {Hue[0.2828148033760653, 1, 1], LineBox[{209, 210}]}, 
        {Hue[0.5800212428125786, 1, 1], LineBox[{210, 211}]}, 
        {Hue[0.8581499613212836, 1, 1], LineBox[{211, 212}]}, 
        {Hue[0.12028273327694287`, 1, 1], LineBox[{212, 213}]}, 
        {Hue[0.37594161632222606`, 1, 1], LineBox[{213, 214}]}, 
        {Hue[0.6320919036749972, 1, 1], LineBox[{214, 215}]}, 
        {Hue[0.8934095514045742, 1, 1], LineBox[{215, 216}]}, 
        {Hue[0.16192256336513522`, 1, 1], LineBox[{216, 217}]}, 
        {Hue[0.43495904779326117`, 1, 1], LineBox[{217, 218}]}, 
        {Hue[0.7055894196988116, 1, 1], LineBox[{218, 219}]}, 
        {Hue[0.9681377653949481, 1, 1], LineBox[{219, 220}]}, 
        {Hue[0.2213746800354963, 1, 1], LineBox[{220, 221}]}, 
        {Hue[0.4665413997702319, 1, 1], LineBox[{221, 222}]}, 
        {Hue[0.7051081290350656, 1, 1], LineBox[{222, 223}]}, 
        {Hue[0.9382072452706954, 1, 1], LineBox[{223, 224}]}, 
        {Hue[0.1672102846488853, 1, 1], LineBox[{224, 225}]}, 
        {Hue[0.3945864673089715, 1, 1], LineBox[{225, 226}]}, 
        {Hue[0.6236533885898886, 1, 1], LineBox[{226, 227}]}, 
        {Hue[0.8568562407239165, 1, 1], LineBox[{227, 228}]}, 
        {Hue[0.09470133370263792, 1, 1], LineBox[{228, 229}]}, 
        {Hue[0.33644601534657576`, 1, 1], LineBox[{229, 230}]}, 
        {Hue[0.5811669538459834, 1, 1], LineBox[{230, 231}]}, 
        {Hue[0.8282212401023837, 1, 1], LineBox[{231, 232}]}, 
        {Hue[0.07723329757905519, 1, 1], LineBox[{232, 233}]}, 
        {Hue[0.3279418824207741, 1, 1], LineBox[{233, 234}]}, 
        {Hue[0.5800695997267706, 1, 1], LineBox[{234, 235}]}, 
        {Hue[0.8332640886537754, 1, 1], LineBox[{235, 236}]}, 
        {Hue[0.087111534514982, 1, 1], LineBox[{236, 237}]}, 
        {Hue[0.341199953193986, 1, 1], LineBox[{237, 238}]}, 
        {Hue[0.5951926565049659, 1, 1], LineBox[{238, 239}]}, 
        {Hue[0.8488741618835237, 1, 1], LineBox[{239, 240}]}, 
        {Hue[0.10215276053117331`, 1, 1], LineBox[{240, 241}]}, 
        {Hue[0.3550286828792829, 1, 1], LineBox[{241, 242}]}, 
        {Hue[0.607548195949124, 1, 1], LineBox[{242, 243}]}, 
        {Hue[0.8597629474201154, 1, 1], LineBox[{243, 244}]}, 
        {Hue[0.11170520728591252`, 1, 1], LineBox[{244, 245}]}, 
        {Hue[0.36338197592002286`, 1, 1], LineBox[{245, 246}]}, 
        {Hue[0.614783091609361, 1, 1], LineBox[{246, 247}]}, 
        {Hue[0.8658951219648834, 1, 1], LineBox[{247, 248}]}, 
        {Hue[0.11671322034353572`, 1, 1], LineBox[{248, 249}]}, 
        {Hue[0.3672457427656668, 1, 1], LineBox[{249, 250}]}, 
        {Hue[0.6175115430478646, 1, 1], LineBox[{250, 251}]}}}},
     VertexColors->{
       Hue[0.2740759612382822, 1, 1], 
       Hue[0.34061641190314307`, 1, 1], 
       Hue[0.42976605858521966`, 1, 1], 
       Hue[0.5138753870902316, 1, 1], 
       Hue[0.5840491086657771, 1, 1], 
       Hue[0.6852575862750414, 1, 1], 
       Hue[0.7690215103953824, 1, 1], 
       Hue[0.8448677927359287, 1, 1], 
       Hue[0.9361130126135394, 1, 1], 
       Hue[0.018348444993657845`, 1, 1], 
       Hue[0.1067341007690407, 1, 1], 
       Hue[0.1916021807806745, 1, 1], 
       Hue[0.27772484219464055`, 1, 1], 
       Hue[0.3643349903317177, 1, 1], 
       Hue[0.45177282749741526`, 1, 1], 
       Hue[0.538908701070393, 1, 1], 
       Hue[0.6261843843359687, 1, 1], 
       Hue[0.7137393391661621, 1, 1], 
       Hue[0.8018583969042162, 1, 1], 
       Hue[0.8904918701851595, 1, 1], 
       Hue[0.9790975713397599, 1, 1], 
       Hue[0.06789470579575665, 1, 1], 
       Hue[0.15697474778197495`, 1, 1], 
       Hue[0.24655060468658918`, 1, 1], 
       Hue[0.3363665345986199, 1, 1], 
       Hue[0.4264547735004385, 1, 1], 
       Hue[0.5167730399489824, 1, 1], 
       Hue[0.6073369670309214, 1, 1], 
       Hue[0.6982587635747064, 1, 1], 
       Hue[0.7894629494103195, 1, 1], 
       Hue[0.8809698434134233, 1, 1], 
       Hue[0.9727289977353122, 1, 1], 
       Hue[0.06475513559485649, 1, 1], 
       Hue[0.15706346690015022`, 1, 1], 
       Hue[0.24967597355959564`, 1, 1], 
       Hue[0.34256934792907295`, 1, 1], 
       Hue[0.43574581435487925`, 1, 1], 
       Hue[0.5291912090168738, 1, 1], 
       Hue[0.6229120360801792, 1, 1], 
       Hue[0.7169177088892706, 1, 1], 
       Hue[0.8112095824461873, 1, 1], 
       Hue[0.9057801263860004, 1, 1], 
       Hue[0.0006257096597658727, 1, 1], 
       Hue[0.09574830413526374, 1, 1], 
       Hue[0.19114909128921104`, 1, 1], 
       Hue[0.2868292849773195, 1, 1], 
       Hue[0.3827869813995296, 1, 1], 
       Hue[0.47902067137753845`, 1, 1], 
       Hue[0.5755290682770862, 1, 1], 
       Hue[0.6723137955010745, 1, 1], 
       Hue[0.7693745554168671, 1, 1], 
       Hue[0.8667116851118895, 1, 1], 
       Hue[0.9643234476361405, 1, 1], 
       Hue[0.062208530739026324`, 1, 1], 
       Hue[0.16036799852267106`, 1, 1], 
       Hue[0.258801810944857, 1, 1], 
       Hue[0.3575091461923081, 1, 1], 
       Hue[0.4564888992590564, 1, 1], 
       Hue[0.5557409571993345, 1, 1], 
       Hue[0.655265089711039, 1, 1], 
       Hue[0.7550614056935411, 1, 1], 
       Hue[0.8551296658837929, 1, 1], 
       Hue[0.9554684222196816, 1, 1], 
       Hue[0.056077780299213835`, 1, 1], 
       Hue[0.1569576070864384, 1, 1], 
       Hue[0.2581074846488935, 1, 1], 
       Hue[0.35952720768489815`, 1, 1], 
       Hue[0.4612154447314414, 1, 1], 
       Hue[0.5631724067610944, 1, 1], 
       Hue[0.6653976393554915, 1, 1], 
       Hue[0.7678905877870311, 1, 1], 
       Hue[0.8706510630208145, 1, 1], 
       Hue[0.9736784294934824, 1, 1], 
       Hue[0.07697211848436797, 1, 1], 
       Hue[0.18053149069666805`, 1, 1], 
       Hue[0.28435568477136225`, 1, 1], 
       Hue[0.3884441911870997, 1, 1], 
       Hue[0.4927967517078755, 1, 1], 
       Hue[0.5974123653015715, 1, 1], 
       Hue[0.7022899159669522, 1, 1], 
       Hue[0.8074287288457369, 1, 1], 
       Hue[0.9128275507242074, 1, 1], 
       Hue[0.018485500919929655`, 1, 1], 
       Hue[0.12440106059642532`, 1, 1], 
       Hue[0.2305726939366271, 1, 1], 
       Hue[0.3369984854894055, 1, 1], 
       Hue[0.4436764093552376, 1, 1], 
       Hue[0.5506045244741555, 1, 1], 
       Hue[0.6577800886416443, 1, 1], 
       Hue[0.7652001869956759, 1, 1], 
       Hue[0.8728611074438166, 1, 1], 
       Hue[0.9807589144955988, 1, 1], 
       Hue[0.08888919496304422, 1, 1], 
       Hue[0.1972463957493058, 1, 1], 
       Hue[0.30582418937682976`, 1, 1], 
       Hue[0.41461542530876894`, 1, 1], 
       Hue[0.523611607046055, 1, 1], 
       Hue[0.6328035280323666, 1, 1], 
       Hue[0.7421791292100071, 1, 1], 
       Hue[0.8517262210776776, 1, 1], 
       Hue[0.9614313829679946, 1, 1], 
       Hue[0.07127581809817282, 1, 1], 
       Hue[0.18123831391879858`, 1, 1], 
       Hue[0.2912992515505204, 1, 1], 
       Hue[0.4014357821497693, 1, 1], 
       Hue[0.5116154678989273, 1, 1], 
       Hue[0.6217988884400221, 1, 1], 
       Hue[0.7319483440755872, 1, 1], 
       Hue[0.8420262140980949, 1, 1], 
       Hue[0.9519858790730588, 1, 1], 
       Hue[0.061770917376315, 1, 1], 
       Hue[0.17131950476346894`, 1, 1], 
       Hue[0.2805652433311565, 1, 1], 
       Hue[0.3894387845260012, 1, 1], 
       Hue[0.497868118280707, 1, 1], 
       Hue[0.6057737050224179, 1, 1], 
       Hue[0.7130713214880129, 1, 1], 
       Hue[0.8196801378961807, 1, 1], 
       Hue[0.9255229902660722, 1, 1], 
       Hue[0.030529801027407266`, 1, 1], 
       Hue[0.13465081125510253`, 1, 1], 
       Hue[0.23784994238387336`, 1, 1], 
       Hue[0.34007354031929565`, 1, 1], 
       Hue[0.4412627880888048, 1, 1], 
       Hue[0.5414319533204015, 1, 1], 
       Hue[0.6406952567590228, 1, 1], 
       Hue[0.7391561792610386, 1, 1], 
       Hue[0.8368067224869767, 1, 1], 
       Hue[0.933611299963676, 1, 1], 
       Hue[0.02966183917474022, 1, 1], 
       Hue[0.12515565615431143`, 1, 1], 
       Hue[0.22025100378644757`, 1, 1], 
       Hue[0.3150509908005036, 1, 1], 
       Hue[0.409700986425785, 1, 1], 
       Hue[0.5043456473843696, 1, 1], 
       Hue[0.5989362670288179, 1, 1], 
       Hue[0.6932230603805059, 1, 1], 
       Hue[0.7871257717961646, 1, 1], 
       Hue[0.8811210035608937, 1, 1], 
       Hue[0.976043668502006, 1, 1], 
       Hue[0.07233130360111649, 1, 1], 
       Hue[0.1694413486033889, 1, 1], 
       Hue[0.2660171617229767, 1, 1], 
       Hue[0.36108283356206833`, 1, 1], 
       Hue[0.45592188250768106`, 1, 1], 
       Hue[0.5541536405087248, 1, 1], 
       Hue[0.6582616045411482, 1, 1], 
       Hue[0.766613722852846, 1, 1], 
       Hue[0.874029838680562, 1, 1], 
       Hue[0.9752100114454223, 1, 1], 
       Hue[0.07288244035433329, 1, 1], 
       Hue[0.18183762518900837`, 1, 1], 
       Hue[0.3128118513040628, 1, 1], 
       Hue[0.4627513612045987, 1, 1], 
       Hue[0.6188156983257607, 1, 1], 
       Hue[0.7659699254591066, 1, 1], 
       Hue[0.900255093224109, 1, 1], 
       Hue[0.027221273580469205`, 1, 1], 
       Hue[0.14925197585231578`, 1, 1], 
       Hue[0.2642182068786057, 1, 1], 
       Hue[0.37219080794297776`, 1, 1], 
       Hue[0.47834136069253086`, 1, 1], 
       Hue[0.5866752936506326, 1, 1], 
       Hue[0.6967194221160611, 1, 1], 
       Hue[0.8091853189345573, 1, 1], 
       Hue[0.9280958405079539, 1, 1], 
       Hue[0.05495597451428205, 1, 1], 
       Hue[0.1858218646652519, 1, 1], 
       Hue[0.31408752473670615`, 1, 1], 
       Hue[0.43511710210004056`, 1, 1], 
       Hue[0.5521633524083553, 1, 1], 
       Hue[0.6750664035148826, 1, 1], 
       Hue[0.8088793717674257, 1, 1], 
       Hue[0.9514016888140933, 1, 1], 
       Hue[0.09849358523699003, 1, 1], 
       Hue[0.2469004214429708, 1, 1], 
       Hue[0.3946861926401874, 1, 1], 
       Hue[0.5410819268855549, 1, 1], 
       Hue[0.6862933635086946, 1, 1], 
       Hue[0.8313368849143955, 1, 1], 
       Hue[0.9778423483567932, 1, 1], 
       Hue[0.1276628869951627, 1, 1], 
       Hue[0.28227537150474946`, 1, 1], 
       Hue[0.44229630855671426`, 1, 1], 
       Hue[0.6073850295098555, 1, 1], 
       Hue[0.7763711353670936, 1, 1], 
       Hue[0.9471867145675702, 1, 1], 
       Hue[0.11600160479303977`, 1, 1], 
       Hue[0.27387095448388904`, 1, 1], 
       Hue[0.3962254851792768, 1, 1], 
       Hue[0.46305170608323365`, 1, 1], 
       Hue[0.5694959575492422, 1, 1], 
       Hue[0.7301243631668375, 1, 1], 
       Hue[0.9137879918990288, 1, 1], 
       Hue[0.1106167560401981, 1, 1], 
       Hue[0.7913631345256851, 1, 1], 
       Hue[0.0028615328745636315`, 1, 1], 
       Hue[0.2096236893305576, 1, 1], 
       Hue[0.4177109221353927, 1, 1], 
       Hue[0.6269797211930057, 1, 1], 
       Hue[0.8360059584538889, 1, 1], 
       Hue[0.0348818559659671, 1, 1], 
       Hue[0.847839796942091, 1, 1], 
       Hue[0.020982790830345505`, 1, 1], 
       Hue[0.24158613432752232`, 1, 1], 
       Hue[0.4741393505207045, 1, 1], 
       Hue[0.7208471264822096, 1, 1], 
       Hue[0.9892888113983458, 1, 1], 
       Hue[0.2828148033760653, 1, 1], 
       Hue[0.5800212428125786, 1, 1], 
       Hue[0.8581499613212836, 1, 1], 
       Hue[0.12028273327694287`, 1, 1], 
       Hue[0.37594161632222606`, 1, 1], 
       Hue[0.6320919036749972, 1, 1], 
       Hue[0.8934095514045742, 1, 1], 
       Hue[0.16192256336513522`, 1, 1], 
       Hue[0.43495904779326117`, 1, 1], 
       Hue[0.7055894196988116, 1, 1], 
       Hue[0.9681377653949481, 1, 1], 
       Hue[0.2213746800354963, 1, 1], 
       Hue[0.4665413997702319, 1, 1], 
       Hue[0.7051081290350656, 1, 1], 
       Hue[0.9382072452706954, 1, 1], 
       Hue[0.1672102846488853, 1, 1], 
       Hue[0.3945864673089715, 1, 1], 
       Hue[0.6236533885898886, 1, 1], 
       Hue[0.8568562407239165, 1, 1], 
       Hue[0.09470133370263792, 1, 1], 
       Hue[0.33644601534657576`, 1, 1], 
       Hue[0.5811669538459834, 1, 1], 
       Hue[0.8282212401023837, 1, 1], 
       Hue[0.07723329757905519, 1, 1], 
       Hue[0.3279418824207741, 1, 1], 
       Hue[0.5800695997267706, 1, 1], 
       Hue[0.8332640886537754, 1, 1], 
       Hue[0.087111534514982, 1, 1], 
       Hue[0.341199953193986, 1, 1], 
       Hue[0.5951926565049659, 1, 1], 
       Hue[0.8488741618835237, 1, 1], 
       Hue[0.10215276053117331`, 1, 1], 
       Hue[0.3550286828792829, 1, 1], 
       Hue[0.607548195949124, 1, 1], 
       Hue[0.8597629474201154, 1, 1], 
       Hue[0.11170520728591252`, 1, 1], 
       Hue[0.36338197592002286`, 1, 1], 
       Hue[0.614783091609361, 1, 1], 
       Hue[0.8658951219648834, 1, 1], 
       Hue[0.11671322034353572`, 1, 1], 
       Hue[0.3672457427656668, 1, 1], 
       Hue[0.6175115430478646, 1, 1], 
       Hue[0.8675314868122538, 1, 1], 
       Hue[0.2740759612382822, 1, 1], 
       Hue[0.34061641190314307`, 1, 1], 
       Hue[0.34061641190314307`, 1, 1], 
       Hue[0.42976605858521966`, 1, 1], 
       Hue[0.42976605858521966`, 1, 1], 
       Hue[0.5138753870902316, 1, 1], 
       Hue[0.5138753870902316, 1, 1], 
       Hue[0.5840491086657771, 1, 1], 
       Hue[0.5840491086657771, 1, 1], 
       Hue[0.6852575862750414, 1, 1], 
       Hue[0.6852575862750414, 1, 1], 
       Hue[0.7690215103953824, 1, 1], 
       Hue[0.7690215103953824, 1, 1], 
       Hue[0.8448677927359287, 1, 1], 
       Hue[0.8448677927359287, 1, 1], 
       Hue[0.9361130126135394, 1, 1], 
       Hue[0.9361130126135394, 1, 1], 
       Hue[0.018348444993657845`, 1, 1], 
       Hue[0.018348444993657845`, 1, 1], 
       Hue[0.1067341007690407, 1, 1], 
       Hue[0.1067341007690407, 1, 1], 
       Hue[0.1916021807806745, 1, 1], 
       Hue[0.1916021807806745, 1, 1], 
       Hue[0.27772484219464055`, 1, 1], 
       Hue[0.27772484219464055`, 1, 1], 
       Hue[0.3643349903317177, 1, 1], 
       Hue[0.3643349903317177, 1, 1], 
       Hue[0.45177282749741526`, 1, 1], 
       Hue[0.45177282749741526`, 1, 1], 
       Hue[0.538908701070393, 1, 1], 
       Hue[0.538908701070393, 1, 1], 
       Hue[0.6261843843359687, 1, 1], 
       Hue[0.6261843843359687, 1, 1], 
       Hue[0.7137393391661621, 1, 1], 
       Hue[0.7137393391661621, 1, 1], 
       Hue[0.8018583969042162, 1, 1], 
       Hue[0.8018583969042162, 1, 1], 
       Hue[0.8904918701851595, 1, 1], 
       Hue[0.8904918701851595, 1, 1], 
       Hue[0.9790975713397599, 1, 1], 
       Hue[0.9790975713397599, 1, 1], 
       Hue[0.06789470579575665, 1, 1], 
       Hue[0.06789470579575665, 1, 1], 
       Hue[0.15697474778197495`, 1, 1], 
       Hue[0.15697474778197495`, 1, 1], 
       Hue[0.24655060468658918`, 1, 1], 
       Hue[0.24655060468658918`, 1, 1], 
       Hue[0.3363665345986199, 1, 1], 
       Hue[0.3363665345986199, 1, 1], 
       Hue[0.4264547735004385, 1, 1], 
       Hue[0.4264547735004385, 1, 1], 
       Hue[0.5167730399489824, 1, 1], 
       Hue[0.5167730399489824, 1, 1], 
       Hue[0.6073369670309214, 1, 1], 
       Hue[0.6073369670309214, 1, 1], 
       Hue[0.6982587635747064, 1, 1], 
       Hue[0.6982587635747064, 1, 1], 
       Hue[0.7894629494103195, 1, 1], 
       Hue[0.7894629494103195, 1, 1], 
       Hue[0.8809698434134233, 1, 1], 
       Hue[0.8809698434134233, 1, 1], 
       Hue[0.9727289977353122, 1, 1], 
       Hue[0.9727289977353122, 1, 1], 
       Hue[0.06475513559485649, 1, 1], 
       Hue[0.06475513559485649, 1, 1], 
       Hue[0.15706346690015022`, 1, 1], 
       Hue[0.15706346690015022`, 1, 1], 
       Hue[0.24967597355959564`, 1, 1], 
       Hue[0.24967597355959564`, 1, 1], 
       Hue[0.34256934792907295`, 1, 1], 
       Hue[0.34256934792907295`, 1, 1], 
       Hue[0.43574581435487925`, 1, 1], 
       Hue[0.43574581435487925`, 1, 1], 
       Hue[0.5291912090168738, 1, 1], 
       Hue[0.5291912090168738, 1, 1], 
       Hue[0.6229120360801792, 1, 1], 
       Hue[0.6229120360801792, 1, 1], 
       Hue[0.7169177088892706, 1, 1], 
       Hue[0.7169177088892706, 1, 1], 
       Hue[0.8112095824461873, 1, 1], 
       Hue[0.8112095824461873, 1, 1], 
       Hue[0.9057801263860004, 1, 1], 
       Hue[0.9057801263860004, 1, 1], 
       Hue[0.0006257096597658727, 1, 1], 
       Hue[0.0006257096597658727, 1, 1], 
       Hue[0.09574830413526374, 1, 1], 
       Hue[0.09574830413526374, 1, 1], 
       Hue[0.19114909128921104`, 1, 1], 
       Hue[0.19114909128921104`, 1, 1], 
       Hue[0.2868292849773195, 1, 1], 
       Hue[0.2868292849773195, 1, 1], 
       Hue[0.3827869813995296, 1, 1], 
       Hue[0.3827869813995296, 1, 1], 
       Hue[0.47902067137753845`, 1, 1], 
       Hue[0.47902067137753845`, 1, 1], 
       Hue[0.5755290682770862, 1, 1], 
       Hue[0.5755290682770862, 1, 1], 
       Hue[0.6723137955010745, 1, 1], 
       Hue[0.6723137955010745, 1, 1], 
       Hue[0.7693745554168671, 1, 1], 
       Hue[0.7693745554168671, 1, 1], 
       Hue[0.8667116851118895, 1, 1], 
       Hue[0.8667116851118895, 1, 1], 
       Hue[0.9643234476361405, 1, 1], 
       Hue[0.9643234476361405, 1, 1], 
       Hue[0.062208530739026324`, 1, 1], 
       Hue[0.062208530739026324`, 1, 1], 
       Hue[0.16036799852267106`, 1, 1], 
       Hue[0.16036799852267106`, 1, 1], 
       Hue[0.258801810944857, 1, 1], 
       Hue[0.258801810944857, 1, 1], 
       Hue[0.3575091461923081, 1, 1], 
       Hue[0.3575091461923081, 1, 1], 
       Hue[0.4564888992590564, 1, 1], 
       Hue[0.4564888992590564, 1, 1], 
       Hue[0.5557409571993345, 1, 1], 
       Hue[0.5557409571993345, 1, 1], 
       Hue[0.655265089711039, 1, 1], 
       Hue[0.655265089711039, 1, 1], 
       Hue[0.7550614056935411, 1, 1], 
       Hue[0.7550614056935411, 1, 1], 
       Hue[0.8551296658837929, 1, 1], 
       Hue[0.8551296658837929, 1, 1], 
       Hue[0.9554684222196816, 1, 1], 
       Hue[0.9554684222196816, 1, 1], 
       Hue[0.056077780299213835`, 1, 1], 
       Hue[0.056077780299213835`, 1, 1], 
       Hue[0.1569576070864384, 1, 1], 
       Hue[0.1569576070864384, 1, 1], 
       Hue[0.2581074846488935, 1, 1], 
       Hue[0.2581074846488935, 1, 1], 
       Hue[0.35952720768489815`, 1, 1], 
       Hue[0.35952720768489815`, 1, 1], 
       Hue[0.4612154447314414, 1, 1], 
       Hue[0.4612154447314414, 1, 1], 
       Hue[0.5631724067610944, 1, 1], 
       Hue[0.5631724067610944, 1, 1], 
       Hue[0.6653976393554915, 1, 1], 
       Hue[0.6653976393554915, 1, 1], 
       Hue[0.7678905877870311, 1, 1], 
       Hue[0.7678905877870311, 1, 1], 
       Hue[0.8706510630208145, 1, 1], 
       Hue[0.8706510630208145, 1, 1], 
       Hue[0.9736784294934824, 1, 1], 
       Hue[0.9736784294934824, 1, 1], 
       Hue[0.07697211848436797, 1, 1], 
       Hue[0.07697211848436797, 1, 1], 
       Hue[0.18053149069666805`, 1, 1], 
       Hue[0.18053149069666805`, 1, 1], 
       Hue[0.28435568477136225`, 1, 1], 
       Hue[0.28435568477136225`, 1, 1], 
       Hue[0.3884441911870997, 1, 1], 
       Hue[0.3884441911870997, 1, 1], 
       Hue[0.4927967517078755, 1, 1], 
       Hue[0.4927967517078755, 1, 1], 
       Hue[0.5974123653015715, 1, 1], 
       Hue[0.5974123653015715, 1, 1], 
       Hue[0.7022899159669522, 1, 1], 
       Hue[0.7022899159669522, 1, 1], 
       Hue[0.8074287288457369, 1, 1], 
       Hue[0.8074287288457369, 1, 1], 
       Hue[0.9128275507242074, 1, 1], 
       Hue[0.9128275507242074, 1, 1], 
       Hue[0.018485500919929655`, 1, 1], 
       Hue[0.018485500919929655`, 1, 1], 
       Hue[0.12440106059642532`, 1, 1], 
       Hue[0.12440106059642532`, 1, 1], 
       Hue[0.2305726939366271, 1, 1], 
       Hue[0.2305726939366271, 1, 1], 
       Hue[0.3369984854894055, 1, 1], 
       Hue[0.3369984854894055, 1, 1], 
       Hue[0.4436764093552376, 1, 1], 
       Hue[0.4436764093552376, 1, 1], 
       Hue[0.5506045244741555, 1, 1], 
       Hue[0.5506045244741555, 1, 1], 
       Hue[0.6577800886416443, 1, 1], 
       Hue[0.6577800886416443, 1, 1], 
       Hue[0.7652001869956759, 1, 1], 
       Hue[0.7652001869956759, 1, 1], 
       Hue[0.8728611074438166, 1, 1], 
       Hue[0.8728611074438166, 1, 1], 
       Hue[0.9807589144955988, 1, 1], 
       Hue[0.9807589144955988, 1, 1], 
       Hue[0.08888919496304422, 1, 1], 
       Hue[0.08888919496304422, 1, 1], 
       Hue[0.1972463957493058, 1, 1], 
       Hue[0.1972463957493058, 1, 1], 
       Hue[0.30582418937682976`, 1, 1], 
       Hue[0.30582418937682976`, 1, 1], 
       Hue[0.41461542530876894`, 1, 1], 
       Hue[0.41461542530876894`, 1, 1], 
       Hue[0.523611607046055, 1, 1], 
       Hue[0.523611607046055, 1, 1], 
       Hue[0.6328035280323666, 1, 1], 
       Hue[0.6328035280323666, 1, 1], 
       Hue[0.7421791292100071, 1, 1], 
       Hue[0.7421791292100071, 1, 1], 
       Hue[0.8517262210776776, 1, 1], 
       Hue[0.8517262210776776, 1, 1], 
       Hue[0.9614313829679946, 1, 1], 
       Hue[0.9614313829679946, 1, 1], 
       Hue[0.07127581809817282, 1, 1], 
       Hue[0.07127581809817282, 1, 1], 
       Hue[0.18123831391879858`, 1, 1], 
       Hue[0.18123831391879858`, 1, 1], 
       Hue[0.2912992515505204, 1, 1], 
       Hue[0.2912992515505204, 1, 1], 
       Hue[0.4014357821497693, 1, 1], 
       Hue[0.4014357821497693, 1, 1], 
       Hue[0.5116154678989273, 1, 1], 
       Hue[0.5116154678989273, 1, 1], 
       Hue[0.6217988884400221, 1, 1], 
       Hue[0.6217988884400221, 1, 1], 
       Hue[0.7319483440755872, 1, 1], 
       Hue[0.7319483440755872, 1, 1], 
       Hue[0.8420262140980949, 1, 1], 
       Hue[0.8420262140980949, 1, 1], 
       Hue[0.9519858790730588, 1, 1], 
       Hue[0.9519858790730588, 1, 1], 
       Hue[0.061770917376315, 1, 1], 
       Hue[0.061770917376315, 1, 1], 
       Hue[0.17131950476346894`, 1, 1], 
       Hue[0.17131950476346894`, 1, 1], 
       Hue[0.2805652433311565, 1, 1], 
       Hue[0.2805652433311565, 1, 1], 
       Hue[0.3894387845260012, 1, 1], 
       Hue[0.3894387845260012, 1, 1], 
       Hue[0.497868118280707, 1, 1], 
       Hue[0.497868118280707, 1, 1], 
       Hue[0.6057737050224179, 1, 1], 
       Hue[0.6057737050224179, 1, 1], 
       Hue[0.7130713214880129, 1, 1], 
       Hue[0.7130713214880129, 1, 1], 
       Hue[0.8196801378961807, 1, 1], 
       Hue[0.8196801378961807, 1, 1], 
       Hue[0.9255229902660722, 1, 1], 
       Hue[0.9255229902660722, 1, 1], 
       Hue[0.030529801027407266`, 1, 1], 
       Hue[0.030529801027407266`, 1, 1], 
       Hue[0.13465081125510253`, 1, 1], 
       Hue[0.13465081125510253`, 1, 1], 
       Hue[0.23784994238387336`, 1, 1], 
       Hue[0.23784994238387336`, 1, 1], 
       Hue[0.34007354031929565`, 1, 1], 
       Hue[0.34007354031929565`, 1, 1], 
       Hue[0.4412627880888048, 1, 1], 
       Hue[0.4412627880888048, 1, 1], 
       Hue[0.5414319533204015, 1, 1], 
       Hue[0.5414319533204015, 1, 1], 
       Hue[0.6406952567590228, 1, 1], 
       Hue[0.6406952567590228, 1, 1], 
       Hue[0.7391561792610386, 1, 1], 
       Hue[0.7391561792610386, 1, 1], 
       Hue[0.8368067224869767, 1, 1], 
       Hue[0.8368067224869767, 1, 1], 
       Hue[0.933611299963676, 1, 1], 
       Hue[0.933611299963676, 1, 1], 
       Hue[0.02966183917474022, 1, 1], 
       Hue[0.02966183917474022, 1, 1], 
       Hue[0.12515565615431143`, 1, 1], 
       Hue[0.12515565615431143`, 1, 1], 
       Hue[0.22025100378644757`, 1, 1], 
       Hue[0.22025100378644757`, 1, 1], 
       Hue[0.3150509908005036, 1, 1], 
       Hue[0.3150509908005036, 1, 1], 
       Hue[0.409700986425785, 1, 1], 
       Hue[0.409700986425785, 1, 1], 
       Hue[0.5043456473843696, 1, 1], 
       Hue[0.5043456473843696, 1, 1], 
       Hue[0.5989362670288179, 1, 1], 
       Hue[0.5989362670288179, 1, 1], 
       Hue[0.6932230603805059, 1, 1], 
       Hue[0.6932230603805059, 1, 1], 
       Hue[0.7871257717961646, 1, 1], 
       Hue[0.7871257717961646, 1, 1], 
       Hue[0.8811210035608937, 1, 1], 
       Hue[0.8811210035608937, 1, 1], 
       Hue[0.976043668502006, 1, 1], 
       Hue[0.976043668502006, 1, 1], 
       Hue[0.07233130360111649, 1, 1], 
       Hue[0.07233130360111649, 1, 1], 
       Hue[0.1694413486033889, 1, 1], 
       Hue[0.1694413486033889, 1, 1], 
       Hue[0.2660171617229767, 1, 1], 
       Hue[0.2660171617229767, 1, 1], 
       Hue[0.36108283356206833`, 1, 1], 
       Hue[0.36108283356206833`, 1, 1], 
       Hue[0.45592188250768106`, 1, 1], 
       Hue[0.45592188250768106`, 1, 1], 
       Hue[0.5541536405087248, 1, 1], 
       Hue[0.5541536405087248, 1, 1], 
       Hue[0.6582616045411482, 1, 1], 
       Hue[0.6582616045411482, 1, 1], 
       Hue[0.766613722852846, 1, 1], 
       Hue[0.766613722852846, 1, 1], 
       Hue[0.874029838680562, 1, 1], 
       Hue[0.874029838680562, 1, 1], 
       Hue[0.9752100114454223, 1, 1], 
       Hue[0.9752100114454223, 1, 1], 
       Hue[0.07288244035433329, 1, 1], 
       Hue[0.07288244035433329, 1, 1], 
       Hue[0.18183762518900837`, 1, 1], 
       Hue[0.18183762518900837`, 1, 1], 
       Hue[0.3128118513040628, 1, 1], 
       Hue[0.3128118513040628, 1, 1], 
       Hue[0.4627513612045987, 1, 1], 
       Hue[0.4627513612045987, 1, 1], 
       Hue[0.6188156983257607, 1, 1], 
       Hue[0.6188156983257607, 1, 1], 
       Hue[0.7659699254591066, 1, 1], 
       Hue[0.7659699254591066, 1, 1], 
       Hue[0.900255093224109, 1, 1], 
       Hue[0.900255093224109, 1, 1], 
       Hue[0.027221273580469205`, 1, 1], 
       Hue[0.027221273580469205`, 1, 1], 
       Hue[0.14925197585231578`, 1, 1], 
       Hue[0.14925197585231578`, 1, 1], 
       Hue[0.2642182068786057, 1, 1], 
       Hue[0.2642182068786057, 1, 1], 
       Hue[0.37219080794297776`, 1, 1], 
       Hue[0.37219080794297776`, 1, 1], 
       Hue[0.47834136069253086`, 1, 1], 
       Hue[0.47834136069253086`, 1, 1], 
       Hue[0.5866752936506326, 1, 1], 
       Hue[0.5866752936506326, 1, 1], 
       Hue[0.6967194221160611, 1, 1], 
       Hue[0.6967194221160611, 1, 1], 
       Hue[0.8091853189345573, 1, 1], 
       Hue[0.8091853189345573, 1, 1], 
       Hue[0.9280958405079539, 1, 1], 
       Hue[0.9280958405079539, 1, 1], 
       Hue[0.05495597451428205, 1, 1], 
       Hue[0.05495597451428205, 1, 1], 
       Hue[0.1858218646652519, 1, 1], 
       Hue[0.1858218646652519, 1, 1], 
       Hue[0.31408752473670615`, 1, 1], 
       Hue[0.31408752473670615`, 1, 1], 
       Hue[0.43511710210004056`, 1, 1], 
       Hue[0.43511710210004056`, 1, 1], 
       Hue[0.5521633524083553, 1, 1], 
       Hue[0.5521633524083553, 1, 1], 
       Hue[0.6750664035148826, 1, 1], 
       Hue[0.6750664035148826, 1, 1], 
       Hue[0.8088793717674257, 1, 1], 
       Hue[0.8088793717674257, 1, 1], 
       Hue[0.9514016888140933, 1, 1], 
       Hue[0.9514016888140933, 1, 1], 
       Hue[0.09849358523699003, 1, 1], 
       Hue[0.09849358523699003, 1, 1], 
       Hue[0.2469004214429708, 1, 1], 
       Hue[0.2469004214429708, 1, 1], 
       Hue[0.3946861926401874, 1, 1], 
       Hue[0.3946861926401874, 1, 1], 
       Hue[0.5410819268855549, 1, 1], 
       Hue[0.5410819268855549, 1, 1], 
       Hue[0.6862933635086946, 1, 1], 
       Hue[0.6862933635086946, 1, 1]}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->{True, None},
    AxesOrigin->{0, 0},
    Epilog->
     LineBox[{{1, 1.0060939673004288`*^-6}, {2, 1.9028034995904528`*^-6}, {
       3, 2.214491810956645*^-6}, {4, 4.087228850688351*^-6}, {
       5, 5.134365838566222*^-6}, {6, 7.371767357820723*^-6}, {
       7, 0.00001165372710711609}, {8, 0.000015942184345643765`}, {
       9, 0.000022865159820867053`}, {10, 0.000032216550384681804`}, {
       11, 0.00004531982940408456}, {12, 0.00006391139327604779}, {
       13, 0.00008879778874723313}, {14, 0.00012220914588380733`}, {
       15, 0.00016837634707687606`}, {16, 0.0002304191416659198}, {
       17, 0.000313330230771826}, {18, 0.00042327333137639}, {
       19, 0.0005665090879436251}, {20, 0.0007572810178161954}, {
       21, 0.001005831880287899}, {22, 0.0013268803199349435`}, {
       23, 0.0017391163480508696`}, {24, 0.0022660961357999173`}, {
       25, 0.0029364923325162057`}, {26, 0.003783476455054223}, {
       27, 0.004846710171243579}, {28, 0.006168486543946245}, {
       29, 0.007804057557584411}, {30, 0.009815973448589142}, {
       31, 0.012274732172491804`}, {32, 0.015261005919086659`}, {
       33, 0.018858882179092214`}, {34, 0.023163979488719316`}, {
       35, 0.028283904251113486`}, {36, 0.03433193553691799}, {
       37, 0.0414287305216337}, {38, 0.04969702664817599}, {
       39, 0.05925947235567039}, {40, 0.07024061686016954}, {
       41, 0.08276481303417263}, {42, 0.09694643359644554}, {
       43, 0.11288613149152181`}, {44, 0.130666962000904}, {
       45, 0.15035010155817907`}, {46, 0.17197400269955715`}, {
       47, 0.1955431220108868}, {48, 0.22102763981507995`}, {
       49, 0.24835302349513763`}, {50, 0.27740154811758816`}, {
       51, 0.3080119086144073}, {52, 0.3399762036602181}, {
       53, 0.3730382159591162}, {54, 0.40689263790584307`}, {
       55, 0.4411897581894739}, {56, 0.4755466371172123}, {
       57, 0.509548967173053}, {58, 0.542754466090125}, {
       59, 0.5747023054196585}, {60, 0.6049310886711522}, {
       61, 0.6329850962818948}, {62, 0.6584265248926926}, {
       63, 0.6808433345012275}, {64, 0.6998643430946516}, {
       65, 0.715165637206513}, {66, 0.7264858990053652}, {
       67, 0.7336304023963114}, {68, 0.7364734571070242}, {
       69, 0.7349657214266813}, {70, 0.729133566232684}, {
       71, 0.7190808722418666}, {72, 0.7049882415274422}, {
       73, 0.6870981021966865}, {74, 0.665717646082974}, {
       75, 0.6412035112765463}, {76, 0.6139555205838362}, {
       77, 0.5844051790844851}, {78, 0.5530048503076144}, {
       79, 0.5202132421740492}, {80, 0.4864885030547204}, {
       81, 0.45227595486284317`}, {82, 0.41799797463716465`}, {
       83, 0.384048462118922}, {84, 0.350783370503897}, {
       85, 0.3185176293754406}, {86, 0.28752103645030824`}, {
       87, 0.2580170663937899}, {88, 0.23018141936997993`}, {
       89, 0.20414282734699182`}, {90, 0.1799860714977934}, {
       91, 0.157755243147453}, {92, 0.13745759396913695`}, {
       93, 0.11906694362998292`}, {94, 0.1025294296099039}, {
       95, 0.08776820549138109}, {96, 0.07468858883400269}, {
       97, 0.0631822748244973}, {98, 0.053131568411669855`}, {
       99, 0.044413889468005625`}, {100, 0.036905342135417546`}, {
       101, 0.03048234076287701}, {102, 0.025025243195574372`}, {
       103, 0.020420649337431384`}, {104, 0.01656181108565891}, {
       105, 0.01334932748083014}, {106, 0.010692574790119742`}, {
       107, 0.008510368599458121}, {108, 0.00673027892552953}, {
       109, 0.005287974898104728}, {110, 0.00412713655811019}, {
       111, 0.003199170136874578}, {112, 0.00246250767529086}, {
       113, 0.0018818411486922785`}, {114, 0.0014274258169291948`}, {
       115, 0.0010743963007635695`}, {116, 0.0008021734836276542}, {
       117, 0.0005938981170839203}, {118, 0.0004358437750483404}, {
       119, 0.0003169143098870495}, {120, 0.00022822345065750627`}, {
       121, 0.0001627001272008966}, {122, 0.00011475425052318849`}, {
       123, 0.0000800260541898452}, {124, 0.00005516292207792344}, {
       125, 0.00003758789282828423}, {126, 0.000025314466690062543`}, {
       127, 0.000016838027811155756`}, {128, 0.000011054501119483348`}, {
       129, 7.1655136221218016`*^-6}, {130, 4.589964828963215*^-6}, {
       131, 2.906933090609105*^-6}, {132, 1.8201919219658998`*^-6}, {
       133, 1.1270270792791408`*^-6}, {134, 6.902640131868779*^-7}, {
       135, 4.180183801076222*^-7}, {136, 2.501184398881478*^-7}, {
       137, 1.479636453594186*^-7}, {138, 8.680633038614577*^-8}, {
       139, 5.06653280168099*^-8}, {140, 2.9399614941896473`*^-8}, {
       141, 1.6869931056056762`*^-8}, {142, 9.514779299252841*^-9}, {
       143, 5.272670355686785*^-9}, {144, 2.898929397941354*^-9}, {
       145, 1.6041878654532542`*^-9}, {146, 8.936820823976941*^-10}, {
       147, 4.896731263798972*^-10}, {148, 2.562848336534942*^-10}, {
       149, 1.2681716371188922`*^-10}, {150, 6.111460626146627*^-11}, {
       151, 3.070893633672525*^-11}, {152, 1.6575077692769548`*^-11}, {
       153, 9.195631750182398*^-12}, {154, 5.153797796699761*^-12}, {
       155, 3.0522209672263964`*^-12}, {156, 1.993857641881664*^-12}, {
       157, 1.4221696991339278`*^-12}, {158, 1.0490133273302367`*^-12}, {
       159, 7.588589589693029*^-13}, {160, 5.331956471141299*^-13}, {
       161, 3.7457829774578716`*^-13}, {162, 2.685778504148808*^-13}, {
       163, 1.940500524749891*^-13}, {164, 1.4001556424587258`*^-13}, {
       165, 1.0204355672203919`*^-13}, {166, 7.483665694650635*^-14}, {
       167, 5.347910219035631*^-14}, {168, 3.620484922047855*^-14}, {
       169, 2.326471685926075*^-14}, {170, 1.47617064363966*^-14}, {
       171, 9.769967251951077*^-15}, {172, 6.796912093454265*^-15}, {
       173, 4.7512022351669356`*^-15}, {174, 3.2089960637776472`*^-15}, {
       175, 2.0671743953597076`*^-15}, {176, 1.2740320892155748`*^-15}, {
       177, 7.586352984322208*^-16}, {178, 4.415716037259406*^-16}, {
       179, 2.5399341718929953`*^-16}, {180, 1.4557341737399026`*^-16}, {
       181, 8.343536015433051*^-17}, {182, 4.768197795701147*^-17}, {
       183, 2.6907693223305463`*^-17}, {184, 1.4782036064853723`*^-17}, {
       185, 7.780973176767766*^-18}, {186, 3.860579985642399*^-18}, {
       187, 1.7726870071704155`*^-18}, {188, 7.357054351639424*^-19}, {
       189, 2.677124945936663*^-19}, {190, 8.646722352541604*^-20}, {
       191, 3.5795685934324046`*^-20}, {192, 2.2260313008291673`*^-20}, {
       193, 1.2353234324457536`*^-20}, {194, 5.171882233315641*^-21}, {
       195, 1.2329921260399653`*^-21}, {196, 3.5697379682382633`*^-22}, {
       197, 6.984088193968665*^-22}, {198, 5.614517877255946*^-22}, {
       199, 3.3495412370170086`*^-22}, {200, 1.606895604155378*^-22}, {
       201, 6.012656079710269*^-23}, {202, 1.3702289045464478`*^-23}, {
       203, 3.176748949691128*^-24}, {204, 5.9906052281992155`*^-24}, {
       205, 4.634817176155701*^-24}, {206, 2.7094443837280076`*^-24}, {
       207, 1.3425486156671524`*^-24}, {208, 6.075732300979954*^-25}, {
       209, 2.8664838988972673`*^-25}, {210, 1.5814663506259344`*^-25}, {
       211, 9.342359763911288*^-26}, {212, 5.272603860863187*^-26}, {
       213, 2.751337247881723*^-26}, {214, 1.3341384879998547`*^-26}, {
       215, 6.139988063566283*^-27}, {216, 2.7772905130818968`*^-27}, {
       217, 1.2833952994966071`*^-27}, {218, 6.140892038997121*^-28}, {
       219, 2.9761548591698386`*^-28}, {220, 1.418088674611575*^-28}, {
       221, 6.5433295675369*^-29}, {222, 2.929250527447788*^-29}, {
       223, 1.2926786050298013`*^-29}, {224, 5.773637459621317*^-30}, {
       225, 2.683219739682183*^-30}, {226, 1.3154971342703058`*^-30}, {
       227, 6.749788967552416*^-31}, {228, 3.548869862050382*^-31}, {
       229, 1.8721015041609338`*^-31}, {230, 9.764670895508588*^-32}, {
       231, 4.997300003623216*^-32}, {232, 2.5031563237895134`*^-32}, {
       233, 1.2282006772383095`*^-32}, {234, 5.917888472830059*^-33}, {
       235, 2.8084316007143253`*^-33}, {236, 1.3160390749146207`*^-33}, {
       237, 6.100008144197109*^-34}, {238, 2.799268290862233*^-34}, {
       239, 1.2722999305358613`*^-34}, {240, 5.729239788761659*^-35}, {
       241, 2.557255695078455*^-35}, {242, 1.1322487175699076`*^-35}, {
       243, 4.9773435952795905`*^-36}, {244, 2.1743796476380444`*^-36}, {
       245, 9.44627126821288*^-37}, {246, 4.082537411854193*^-37}, {
       247, 1.7552364589517332`*^-37}, {248, 7.504517103744936*^-38}, {
       249, 3.1888794961033176`*^-38}, {250, 1.345796770018393*^-38}, {
       251, 5.6368908933762926`*^-39}}],
    Frame->True,
    FrameTicks->{{{1., 
        FormBox[
         RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
        FormBox[
         RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
        FormBox[
         RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
        FormBox[
         RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
        FormBox["0", TraditionalForm]}, {157.25, 
        FormBox["2.5`", TraditionalForm]}, {188.5, 
        FormBox["5", TraditionalForm]}, {219.75, 
        FormBox["7.5`", TraditionalForm]}, {251., 
        FormBox["10", TraditionalForm]}}, Automatic, None, None},
    PlotRange->{-0.1, 1.1},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], Automatic}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF2XlYVPUeBvBDZu4pGqClNgipuWW4C8iLgAIi+76eAWYGvBJ6zSWl6480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     "], {
      {GrayLevel[0.5], EdgeForm[None], 
       GraphicsGroupBox[{PolygonBox[{{194, 638, 195}}], 
         PolygonBox[CompressedData["
1:eJwtlmV0VUcURs/c4E5wJxDc3d2CQ3CH4BrcaSnuFHd3h+IudXd3d/cW6T79
+LFX9po5ct/cM3kvLik5cUxkZjchBaT0v4G/kAr3vbtwD2LuY+yFoPj0kAXP
Chnu14jwmKBa6SATnjko1mumwlM7eBqP8T1IayIDnjEo9/8Y3wuK9R6xeDaP
wbNDEbwo5PBekBPP5T1N5MbzQBYTefF8/sx2v4bnBuX6Wn68AMTi2aAwHhfU
y9cK4oWC9rxnPF4McpoojpfwZ8ArQHe8B1Q0rZXES0FuPA+UxctBXq8N1fEa
3gPPBxXxSpDfRGW8ij+jKac8XiEo1teq4tWCahWFhngjiDfVrInXgsJ4HNTF
60ERE/XxBkG5/oyl8TJBz+o1GuNN/DP7s0Mi3sk/g8dCO7y95+AlIAFvCSVN
tMJb+xn4WUFzvEVQrK+1wdsG1fIeTfFmQbFeswPeMai39+yMd4HyJrri3YLO
3ud6JkM8C26Z3kFP9nr5mfpZQl+8n5+pnz0MwYf6OzCt9ccHQFUTSfhAP2M/
OxiPT/AzNK0NwgcH1aoDY/FxQbFecxg+HGritWAUPhpqmxiDJwfleswIfGRQ
rPeYiE+C+ngzmIfPh+Z4A5iKT4OGJqbjM3wGTMzEZ0FjU43J+JSgXF97AH/Q
37GfPczB5wb18rXZ+ENBe95zAb7Q36GJRfhiSDCxBF/qM2BiGb4cWplYga/0
mfDZgfX4Bp8hExvxTdDBZwNW42t8RkysxdcF5XrMZnyLzwjeCXbiu6Czid34
Hp8ZU8xWfBskmtiO7wjK9Wd6GF8V1Ntz9uL7oCveDQ7ih6A73gtO4qegt2nt
MH4Eepg4ih+DniaO4yeCcj3nEfw09DH12I8fCOrla2fws9DXxDn8vM+wiQv4
RehvuuO1fZaC7rqvXcIv+0ybuIJfhSQT1/DrPuP4UHgKfxqG+WzDLfxRn3ET
j+GPwxB8GnyHfw/TTWtP4E8G1RoFr+Cv+sybaj6DP+t3wmcdXsBf9Jk38RL+
clCuxzyHPx8U6zVew1+HMfgk+Az/HCbjE+Aj/GOY6HcR3sHf9Ttp4j38fRiP
J8Nb+NtBsb72Af5hUC3v8Qb+ZlCs1/wE/zSot/f8Av/S75SJr/CvYaqJb/Bv
g87Kz+gH/Ee/oz5rkJf/Vflgld9F+IO9P/1OmvgL/xtm+12GX/Bf/U6b+A3/
PSjXY/7B//U7i88F/1IPMM9EhMfAfFPMbWLvwBwTd/F7Qbn+jD/hPwf19pwU
5KaEBfhCSIOnhUX4UsiCZ4VlprV0eHpYbCIDnhGWmMiEZ46U6zmxeDZYbuqR
Ck8dqZevZcdzwAoTOfFcsNJEbjxPpLP1M82PF4DVpu+A3n7Xgr4LfK0ge4Vg
jd9Fv2N4PziNr4UieFFYZyIeLwbr8W1QFa8G201Ux2vADnwDlMRLwUYTpfEy
sMlUozheIlLsFqiIV4KtJirjVSL12gzl8QqRYr1HTbwW7MQPQALeEg7iu6Au
Xg92m6iPN4A9pjt/g7O4GXTX90Nz9lpEquU1a+N1ItXaC43xJrDPVKMh3ijS
nvdshbeGQ6aYpnizSLX9M5fFy0X6LB7TBm8Lh020w9vDEfwk9MR7wSn8GHTG
u8BxE13xbnACPwqJeKdIsb7WHe8RqdZZGIQPhnOmmr3xPpHevb/z/vgAOGMi
CR8YKddzhuBD4Tx+AUbgI+Eifg0m41PgumltFD4aLpnwH+DJcNn0GTvgHSM9
+xUYj0+Aq6aYsfi4SHu+NhGfFKmX95iKT4MbJqbjMyL9FvJnHIYPj/SsPuOF
8bhIs81rt9twx/Tb/z9UaTnB
          "]]}]}, {{}, {}, {
        {Hue[0.9640532089338167, 1, 1], LineBox[{1, 2}]}, 
        {Hue[0.7638938367871528, 1, 1], LineBox[{2, 3}]}, 
        {Hue[0.5456953793797118, 1, 1], LineBox[{3, 4}]}, 
        {Hue[0.4880359025202404, 1, 1], LineBox[{4, 5}]}, 
        {Hue[0.418755936350388, 1, 1], LineBox[{5, 6}]}, 
        {Hue[0.22471186555443298`, 1, 1], LineBox[{6, 7}]}, 
        {Hue[0.7664365867469122, 1, 1], LineBox[{7, 8}]}, 
        {Hue[0.7569067121630682, 1, 1], LineBox[{8, 9}]}, 
        {Hue[0.8201765986296474, 1, 1], LineBox[{9, 10}]}, 
        {Hue[0.8383737951258301, 1, 1], LineBox[{10, 11}]}, 
        {Hue[0.8528114312520936, 1, 1], LineBox[{11, 12}]}, 
        {Hue[0.0005814275632171318, 1, 1], LineBox[{12, 13}]}, 
        {Hue[0.09900120701277339, 1, 1], LineBox[{13, 14}]}, 
        {Hue[0.1775661426240637, 1, 1], LineBox[{14, 15}]}, 
        {Hue[0.24423919184988493`, 1, 1], LineBox[{15, 16}]}, 
        {Hue[0.3156688798563375, 1, 1], LineBox[{16, 17}]}, 
        {Hue[0.38950220432683996`, 1, 1], LineBox[{17, 18}]}, 
        {Hue[0.4814958525831977, 1, 1], LineBox[{18, 19}]}, 
        {Hue[0.5657261147473843, 1, 1], LineBox[{19, 20}]}, 
        {Hue[0.6467337892415539, 1, 1], LineBox[{20, 21}]}, 
        {Hue[0.7248416611021938, 1, 1], LineBox[{21, 22}]}, 
        {Hue[0.8055142767348117, 1, 1], LineBox[{22, 23}]}, 
        {Hue[0.8872992703340177, 1, 1], LineBox[{23, 24}]}, 
        {Hue[0.9720085880616776, 1, 1], LineBox[{24, 25}]}, 
        {Hue[0.05610154862290952, 1, 1], LineBox[{25, 26}]}, 
        {Hue[0.13926945181155012`, 1, 1], LineBox[{26, 27}]}, 
        {Hue[0.22225896671723153`, 1, 1], LineBox[{27, 28}]}, 
        {Hue[0.30594469461240686`, 1, 1], LineBox[{28, 29}]}, 
        {Hue[0.3907297817738525, 1, 1], LineBox[{29, 30}]}, 
        {Hue[0.47584462841575326`, 1, 1], LineBox[{30, 31}]}, 
        {Hue[0.5613011947128719, 1, 1], LineBox[{31, 32}]}, 
        {Hue[0.6467734454202569, 1, 1], LineBox[{32, 33}]}, 
        {Hue[0.732620054471488, 1, 1], LineBox[{33, 34}]}, 
        {Hue[0.8188508419403508, 1, 1], LineBox[{34, 35}]}, 
        {Hue[0.9055713829157245, 1, 1], LineBox[{35, 36}]}, 
        {Hue[0.9926765120282037, 1, 1], LineBox[{36, 37}]}, 
        {Hue[0.08008348174035529, 1, 1], LineBox[{37, 38}]}, 
        {Hue[0.1678032957843583, 1, 1], LineBox[{38, 39}]}, 
        {Hue[0.2558772079638403, 1, 1], LineBox[{39, 40}]}, 
        {Hue[0.34433867572467236`, 1, 1], LineBox[{40, 41}]}, 
        {Hue[0.43317536470416307`, 1, 1], LineBox[{41, 42}]}, 
        {Hue[0.5223512697273097, 1, 1], LineBox[{42, 43}]}, 
        {Hue[0.611865760320544, 1, 1], LineBox[{43, 44}]}, 
        {Hue[0.7017376629319375, 1, 1], LineBox[{44, 45}]}, 
        {Hue[0.791973646004684, 1, 1], LineBox[{45, 46}]}, 
        {Hue[0.8825715119856232, 1, 1], LineBox[{46, 47}]}, 
        {Hue[0.9735201231658492, 1, 1], LineBox[{47, 48}]}, 
        {Hue[0.06481664673905892, 1, 1], LineBox[{48, 49}]}, 
        {Hue[0.1564624526544423, 1, 1], LineBox[{49, 50}]}, 
        {Hue[0.24846339708111093`, 1, 1], LineBox[{50, 51}]}, 
        {Hue[0.34082044254039434`, 1, 1], LineBox[{51, 52}]}, 
        {Hue[0.4335285188995974, 1, 1], LineBox[{52, 53}]}, 
        {Hue[0.5265858504718376, 1, 1], LineBox[{53, 54}]}, 
        {Hue[0.619991238019795, 1, 1], LineBox[{54, 55}]}, 
        {Hue[0.7137489787936226, 1, 1], LineBox[{55, 56}]}, 
        {Hue[0.8078581491198726, 1, 1], LineBox[{56, 57}]}, 
        {Hue[0.9023167908162214, 1, 1], LineBox[{57, 58}]}, 
        {Hue[0.997123518820222, 1, 1], LineBox[{58, 59}]}, 
        {Hue[0.09227809771084206, 1, 1], LineBox[{59, 60}]}, 
        {Hue[0.18778045984197525`, 1, 1], LineBox[{60, 61}]}, 
        {Hue[0.28363150538320664`, 1, 1], LineBox[{61, 62}]}, 
        {Hue[0.37982913467653123`, 1, 1], LineBox[{62, 63}]}, 
        {Hue[0.4763730534549754, 1, 1], LineBox[{63, 64}]}, 
        {Hue[0.5732620582331205, 1, 1], LineBox[{64, 65}]}, 
        {Hue[0.6704969545124291, 1, 1], LineBox[{65, 66}]}, 
        {Hue[0.7680769044926821, 1, 1], LineBox[{66, 67}]}, 
        {Hue[0.866001209191981, 1, 1], LineBox[{67, 68}]}, 
        {Hue[0.9642694798933042, 1, 1], LineBox[{68, 69}]}, 
        {Hue[0.06288033404720918, 1, 1], LineBox[{69, 70}]}, 
        {Hue[0.1618340190608663, 1, 1], LineBox[{70, 71}]}, 
        {Hue[0.26112965826933277`, 1, 1], LineBox[{71, 72}]}, 
        {Hue[0.36076682669457827`, 1, 1], LineBox[{72, 73}]}, 
        {Hue[0.4607446702920191, 1, 1], LineBox[{73, 74}]}, 
        {Hue[0.5610625757967369, 1, 1], LineBox[{74, 75}]}, 
        {Hue[0.6617196759615321, 1, 1], LineBox[{75, 76}]}, 
        {Hue[0.7627147629687548, 1, 1], LineBox[{76, 77}]}, 
        {Hue[0.8640473849020327, 1, 1], LineBox[{77, 78}]}, 
        {Hue[0.9657168711695278, 1, 1], LineBox[{78, 79}]}, 
        {Hue[0.06772203209084474, 1, 1], LineBox[{79, 80}]}, 
        {Hue[0.17006179329564264`, 1, 1], LineBox[{80, 81}]}, 
        {Hue[0.2727348799293318, 1, 1], LineBox[{81, 82}]}, 
        {Hue[0.3757400482244432, 1, 1], LineBox[{82, 83}]}, 
        {Hue[0.47907615599813785`, 1, 1], LineBox[{83, 84}]}, 
        {Hue[0.5827411225608394, 1, 1], LineBox[{84, 85}]}, 
        {Hue[0.686732977202875, 1, 1], LineBox[{85, 86}]}, 
        {Hue[0.7910505657857274, 1, 1], LineBox[{86, 87}]}, 
        {Hue[0.8956910816409434, 1, 1], LineBox[{87, 88}]}, 
        {Hue[0.0006518319749389761, 1, 1], LineBox[{88, 89}]}, 
        {Hue[0.10592980643142957`, 1, 1], LineBox[{89, 90}]}, 
        {Hue[0.21152177735264863`, 1, 1], LineBox[{90, 91}]}, 
        {Hue[0.3174239892158998, 1, 1], LineBox[{91, 92}]}, 
        {Hue[0.4236321033035602, 1, 1], LineBox[{92, 93}]}, 
        {Hue[0.5301403332116573, 1, 1], LineBox[{93, 94}]}, 
        {Hue[0.6369434399221541, 1, 1], LineBox[{94, 95}]}, 
        {Hue[0.7440350187497498, 1, 1], LineBox[{95, 96}]}, 
        {Hue[0.8514071856099105, 1, 1], LineBox[{96, 97}]}, 
        {Hue[0.9590505875381703, 1, 1], LineBox[{97, 98}]}, 
        {Hue[0.06695495506670603, 1, 1], LineBox[{98, 99}]}, 
        {Hue[0.17510840763985963`, 1, 1], LineBox[{99, 100}]}, 
        {Hue[0.2834976571366844, 1, 1], LineBox[{100, 101}]}, 
        {Hue[0.3921067036965846, 1, 1], LineBox[{101, 102}]}, 
        {Hue[0.5009171822537385, 1, 1], LineBox[{102, 103}]}, 
        {Hue[0.609907960840949, 1, 1], LineBox[{103, 104}]}, 
        {Hue[0.7190549385107554, 1, 1], LineBox[{104, 105}]}, 
        {Hue[0.8283307021079086, 1, 1], LineBox[{105, 106}]}, 
        {Hue[0.9377040687499487, 1, 1], LineBox[{106, 107}]}, 
        {Hue[0.04713886558145295, 1, 1], LineBox[{107, 108}]}, 
        {Hue[0.15659333346659132`, 1, 1], LineBox[{108, 109}]}, 
        {Hue[0.26602139648303896`, 1, 1], LineBox[{109, 110}]}, 
        {Hue[0.37537095342567955`, 1, 1], LineBox[{110, 111}]}, 
        {Hue[0.48458443441848675`, 1, 1], LineBox[{111, 112}]}, 
        {Hue[0.5935967404345542, 1, 1], LineBox[{112, 113}]}, 
        {Hue[0.7023349586497987, 1, 1], LineBox[{113, 114}]}, 
        {Hue[0.8107237607449748, 1, 1], LineBox[{114, 115}]}, 
        {Hue[0.9186813036861303, 1, 1], LineBox[{115, 116}]}, 
        {Hue[0.02611720447668775, 1, 1], LineBox[{116, 117}]}, 
        {Hue[0.1329418218181866, 1, 1], LineBox[{117, 118}]}, 
        {Hue[0.23906924624217446`, 1, 1], LineBox[{118, 119}]}, 
        {Hue[0.34440835836416867`, 1, 1], LineBox[{119, 120}]}, 
        {Hue[0.4488665860210912, 1, 1], LineBox[{120, 121}]}, 
        {Hue[0.5523753592326575, 1, 1], LineBox[{121, 122}]}, 
        {Hue[0.6548918734757004, 1, 1], LineBox[{122, 123}]}, 
        {Hue[0.7563690714498079, 1, 1], LineBox[{123, 124}]}, 
        {Hue[0.856754833013501, 1, 1], LineBox[{124, 125}]}, 
        {Hue[0.9560476299526799, 1, 1], LineBox[{125, 126}]}, 
        {Hue[0.05431861348849255, 1, 1], LineBox[{126, 127}]}, 
        {Hue[0.15165859477128055`, 1, 1], LineBox[{127, 128}]}, 
        {Hue[0.2481278362722127, 1, 1], LineBox[{128, 129}]}, 
        {Hue[0.34377181562590414`, 1, 1], LineBox[{129, 130}]}, 
        {Hue[0.43867250905938415`, 1, 1], LineBox[{130, 131}]}, 
        {Hue[0.5329818091731069, 1, 1], LineBox[{131, 132}]}, 
        {Hue[0.6268784974511232, 1, 1], LineBox[{132, 133}]}, 
        {Hue[0.7204800452917036, 1, 1], LineBox[{133, 134}]}, 
        {Hue[0.8138447115607892, 1, 1], LineBox[{134, 135}]}, 
        {Hue[0.9070647396475882, 1, 1], LineBox[{135, 136}]}, 
        {Hue[0.00027500323336431757`, 1, 1], LineBox[{136, 137}]}, 
        {Hue[0.0936287218150815, 1, 1], LineBox[{137, 138}]}, 
        {Hue[0.18738898166912799`, 1, 1], LineBox[{138, 139}]}, 
        {Hue[0.2817608025135326, 1, 1], LineBox[{139, 140}]}, 
        {Hue[0.3762560119614834, 1, 1], LineBox[{140, 141}]}, 
        {Hue[0.46983126202546877`, 1, 1], LineBox[{141, 142}]}, 
        {Hue[0.5628818631021897, 1, 1], LineBox[{142, 143}]}, 
        {Hue[0.6583764075712767, 1, 1], LineBox[{143, 144}]}, 
        {Hue[0.7583085727453893, 1, 1], LineBox[{144, 145}]}, 
        {Hue[0.8597558378859961, 1, 1], LineBox[{145, 146}]}, 
        {Hue[0.9565076209795028, 1, 1], LineBox[{146, 147}]}, 
        {Hue[0.04627851579084141, 1, 1], LineBox[{147, 148}]}, 
        {Hue[0.13696150887572137`, 1, 1], LineBox[{148, 149}]}, 
        {Hue[0.236959712104721, 1, 1], LineBox[{149, 150}]}, 
        {Hue[0.34512012689175703`, 1, 1], LineBox[{150, 151}]}, 
        {Hue[0.45688774460841647`, 1, 1], LineBox[{151, 152}]}, 
        {Hue[0.5703780558213525, 1, 1], LineBox[{152, 153}]}, 
        {Hue[0.6836540521049628, 1, 1], LineBox[{153, 154}]}, 
        {Hue[0.7838772463981823, 1, 1], LineBox[{154, 155}]}, 
        {Hue[0.8381977472273094, 1, 1], LineBox[{155, 156}]}, 
        {Hue[0.8870544606419305, 1, 1], LineBox[{156, 157}]}, 
        {Hue[0.003981834612291627, 1, 1], LineBox[{157, 158}]}, 
        {Hue[0.16018129844586784`, 1, 1], LineBox[{158, 159}]}, 
        {Hue[0.33780729398489723`, 1, 1], LineBox[{159, 160}]}, 
        {Hue[0.5369775037050852, 1, 1], LineBox[{160, 161}]}, 
        {Hue[0.7905257246587302, 1, 1], LineBox[{161, 162}]}, 
        {Hue[0.1945036657769734, 1, 1], LineBox[{162, 163}]}, 
        {Hue[0.48496065810113864`, 1, 1], LineBox[{163, 164}]}, 
        {Hue[0.7153646411488792, 1, 1], LineBox[{164, 165}]}, 
        {Hue[0.9394067463789025, 1, 1], LineBox[{165, 166}]}, 
        {Hue[0.17080211619971455`, 1, 1], LineBox[{166, 167}]}, 
        {Hue[0.4207562168980021, 1, 1], LineBox[{167, 168}]}, 
        {Hue[0.6977457893116823, 1, 1], LineBox[{168, 169}]}, 
        {Hue[0.9810396549863314, 1, 1], LineBox[{169, 170}]}, 
        {Hue[0.23870865443606856`, 1, 1], LineBox[{170, 171}]}, 
        {Hue[0.4717879390337842, 1, 1], LineBox[{171, 172}]}, 
        {Hue[0.6898493957007891, 1, 1], LineBox[{172, 173}]}, 
        {Hue[0.8977125550908651, 1, 1], LineBox[{173, 174}]}, 
        {Hue[0.09510862803082076, 1, 1], LineBox[{174, 175}]}, 
        {Hue[0.22425606418634192`, 1, 1], LineBox[{175, 176}]}, 
        {Hue[0.9819966346546343, 1, 1], LineBox[{176, 177}]}, 
        {Hue[0.12933728393861235`, 1, 1], LineBox[{177, 178}]}, 
        {Hue[0.24718059981823032`, 1, 1], LineBox[{178, 179}]}, 
        {Hue[0.3560042502224064, 1, 1], LineBox[{179, 180}]}, 
        {Hue[0.4832174243664257, 1, 1], LineBox[{180, 181}]}, 
        {Hue[0.6225466914902852, 1, 1], LineBox[{181, 182}]}, 
        {Hue[0.7669134155846555, 1, 1], LineBox[{182, 183}]}, 
        {Hue[0.919797571218377, 1, 1], LineBox[{183, 184}]}, 
        {Hue[0.08538170051501558, 1, 1], LineBox[{184, 185}]}, 
        {Hue[0.2620427531752457, 1, 1], LineBox[{185, 186}]}, 
        {Hue[0.44406556173858014`, 1, 1], LineBox[{186, 187}]}, 
        {Hue[0.6185988292105646, 1, 1], LineBox[{187, 188}]}, 
        {Hue[0.6838184160682961, 1, 1], LineBox[{188, 189}]}, 
        {Hue[0.6378535206406792, 1, 1], LineBox[{189, 190}]}, 
        {Hue[0.8073537712650531, 1, 1], LineBox[{190, 191}]}, 
        {Hue[0.0008353865620092372, 1, 1], LineBox[{191, 192}]}, 
        {Hue[0.1996041412947276, 1, 1], LineBox[{192, 193}]}, 
        {Hue[0.3994928084092794, 1, 1], LineBox[{193, 194}]}, 
        {Hue[0.598581266602507, 1, 1], LineBox[{194, 195}]}, 
        {Hue[0.795286513537276, 1, 1], LineBox[{195, 196}]}, 
        {Hue[0.9874661970418543, 1, 1], LineBox[{196, 197}]}, 
        {Hue[0.17098866130101048`, 1, 1], LineBox[{197, 198}]}, 
        {Hue[0.33608016823445647`, 1, 1], LineBox[{198, 199}]}, 
        {Hue[0.4626610597119734, 1, 1], LineBox[{199, 200}]}, 
        {Hue[0.5600193469710358, 1, 1], LineBox[{200, 201}]}, 
        {Hue[0.6951140435126271, 1, 1], LineBox[{201, 202}]}, 
        {Hue[0.8688175337237586, 1, 1], LineBox[{202, 203}]}, 
        {Hue[0.06217243610467935, 1, 1], LineBox[{203, 204}]}, 
        {Hue[0.26673052970331346`, 1, 1], LineBox[{204, 205}]}, 
        {Hue[0.47918613708383234`, 1, 1], LineBox[{205, 206}]}, 
        {Hue[0.6981752604591175, 1, 1], LineBox[{206, 207}]}, 
        {Hue[0.923139519550703, 1, 1], LineBox[{207, 208}]}, 
        {Hue[0.15391413553106462`, 1, 1], LineBox[{208, 209}]}, 
        {Hue[0.39058746840666203`, 1, 1], LineBox[{209, 210}]}, 
        {Hue[0.6335130531533011, 1, 1], LineBox[{210, 211}]}, 
        {Hue[0.8834878616778492, 1, 1], LineBox[{211, 212}]}, 
        {Hue[0.14218772425266332`, 1, 1], LineBox[{212, 213}]}, 
        {Hue[0.4129145318615668, 1, 1], LineBox[{213, 214}]}, 
        {Hue[0.7008647172568718, 1, 1], LineBox[{214, 215}]}, 
        {Hue[0.008276153030424088, 1, 1], LineBox[{215, 216}]}, 
        {Hue[0.32203152553002135`, 1, 1], LineBox[{216, 217}]}, 
        {Hue[0.6228272969220539, 1, 1], LineBox[{217, 218}]}, 
        {Hue[0.9077377802688978, 1, 1], LineBox[{218, 219}]}, 
        {Hue[0.18295813338288494`, 1, 1], LineBox[{219, 220}]}, 
        {Hue[0.45436977284719, 1, 1], LineBox[{220, 221}]}, 
        {Hue[0.7264230564635294, 1, 1], LineBox[{221, 222}]}, 
        {Hue[0.0029732826562475275`, 1, 1], LineBox[{222, 223}]}, 
        {Hue[0.28772899918756284`, 1, 1], LineBox[{223, 224}]}, 
        {Hue[0.5832291550635875, 1, 1], LineBox[{224, 225}]}, 
        {Hue[0.8874633543999638, 1, 1], LineBox[{225, 226}]}, 
        {Hue[0.1922430080867037, 1, 1], LineBox[{226, 227}]}, 
        {Hue[0.489719235032304, 1, 1], LineBox[{227, 228}]}, 
        {Hue[0.7780535457697576, 1, 1], LineBox[{228, 229}]}, 
        {Hue[0.0590819613523457, 1, 1], LineBox[{229, 230}]}, 
        {Hue[0.3350544804696013, 1, 1], LineBox[{230, 231}]}, 
        {Hue[0.6076027369926792, 1, 1], LineBox[{231, 232}]}, 
        {Hue[0.8777648644902436, 1, 1], LineBox[{232, 233}]}, 
        {Hue[0.1461932685057524, 1, 1], LineBox[{233, 234}]}, 
        {Hue[0.4133070810184004, 1, 1], LineBox[{234, 235}]}, 
        {Hue[0.6793688455124909, 1, 1], LineBox[{235, 236}]}, 
        {Hue[0.944513321626013, 1, 1], LineBox[{236, 237}]}, 
        {Hue[0.20875530569204304`, 1, 1], LineBox[{237, 238}]}, 
        {Hue[0.4719956959154934, 1, 1], LineBox[{238, 239}]}, 
        {Hue[0.7340353362877327, 1, 1], LineBox[{239, 240}]}, 
        {Hue[0.9945952634521681, 1, 1], LineBox[{240, 241}]}, 
        {Hue[0.2533389529365845, 1, 1], LineBox[{241, 242}]}, 
        {Hue[0.5098912530483598, 1, 1], LineBox[{242, 243}]}, 
        {Hue[0.7638545596346901, 1, 1], LineBox[{243, 244}]}, 
        {Hue[0.01483438802715526, 1, 1], LineBox[{244, 245}]}, 
        {Hue[0.2624959819063229, 1, 1], LineBox[{245, 246}]}, 
        {Hue[0.5066777947454011, 1, 1], LineBox[{246, 247}]}, 
        {Hue[0.7475644420744317, 1, 1], LineBox[{247, 248}]}, 
        {Hue[0.9858518958082757, 1, 1], LineBox[{248, 249}]}, 
        {Hue[0.2227535812882585, 1, 1], LineBox[{249, 250}]}, 
        {Hue[0.45973174016904605`, 1, 1], LineBox[{250, 251}]}}}},
     VertexColors->{
       Hue[0.9640532089338167, 1, 1], 
       Hue[0.7638938367871528, 1, 1], 
       Hue[0.5456953793797118, 1, 1], 
       Hue[0.4880359025202404, 1, 1], 
       Hue[0.418755936350388, 1, 1], 
       Hue[0.22471186555443298`, 1, 1], 
       Hue[0.7664365867469122, 1, 1], 
       Hue[0.7569067121630682, 1, 1], 
       Hue[0.8201765986296474, 1, 1], 
       Hue[0.8383737951258301, 1, 1], 
       Hue[0.8528114312520936, 1, 1], 
       Hue[0.0005814275632171318, 1, 1], 
       Hue[0.09900120701277339, 1, 1], 
       Hue[0.1775661426240637, 1, 1], 
       Hue[0.24423919184988493`, 1, 1], 
       Hue[0.3156688798563375, 1, 1], 
       Hue[0.38950220432683996`, 1, 1], 
       Hue[0.4814958525831977, 1, 1], 
       Hue[0.5657261147473843, 1, 1], 
       Hue[0.6467337892415539, 1, 1], 
       Hue[0.7248416611021938, 1, 1], 
       Hue[0.8055142767348117, 1, 1], 
       Hue[0.8872992703340177, 1, 1], 
       Hue[0.9720085880616776, 1, 1], 
       Hue[0.05610154862290952, 1, 1], 
       Hue[0.13926945181155012`, 1, 1], 
       Hue[0.22225896671723153`, 1, 1], 
       Hue[0.30594469461240686`, 1, 1], 
       Hue[0.3907297817738525, 1, 1], 
       Hue[0.47584462841575326`, 1, 1], 
       Hue[0.5613011947128719, 1, 1], 
       Hue[0.6467734454202569, 1, 1], 
       Hue[0.732620054471488, 1, 1], 
       Hue[0.8188508419403508, 1, 1], 
       Hue[0.9055713829157245, 1, 1], 
       Hue[0.9926765120282037, 1, 1], 
       Hue[0.08008348174035529, 1, 1], 
       Hue[0.1678032957843583, 1, 1], 
       Hue[0.2558772079638403, 1, 1], 
       Hue[0.34433867572467236`, 1, 1], 
       Hue[0.43317536470416307`, 1, 1], 
       Hue[0.5223512697273097, 1, 1], 
       Hue[0.611865760320544, 1, 1], 
       Hue[0.7017376629319375, 1, 1], 
       Hue[0.791973646004684, 1, 1], 
       Hue[0.8825715119856232, 1, 1], 
       Hue[0.9735201231658492, 1, 1], 
       Hue[0.06481664673905892, 1, 1], 
       Hue[0.1564624526544423, 1, 1], 
       Hue[0.24846339708111093`, 1, 1], 
       Hue[0.34082044254039434`, 1, 1], 
       Hue[0.4335285188995974, 1, 1], 
       Hue[0.5265858504718376, 1, 1], 
       Hue[0.619991238019795, 1, 1], 
       Hue[0.7137489787936226, 1, 1], 
       Hue[0.8078581491198726, 1, 1], 
       Hue[0.9023167908162214, 1, 1], 
       Hue[0.997123518820222, 1, 1], 
       Hue[0.09227809771084206, 1, 1], 
       Hue[0.18778045984197525`, 1, 1], 
       Hue[0.28363150538320664`, 1, 1], 
       Hue[0.37982913467653123`, 1, 1], 
       Hue[0.4763730534549754, 1, 1], 
       Hue[0.5732620582331205, 1, 1], 
       Hue[0.6704969545124291, 1, 1], 
       Hue[0.7680769044926821, 1, 1], 
       Hue[0.866001209191981, 1, 1], 
       Hue[0.9642694798933042, 1, 1], 
       Hue[0.06288033404720918, 1, 1], 
       Hue[0.1618340190608663, 1, 1], 
       Hue[0.26112965826933277`, 1, 1], 
       Hue[0.36076682669457827`, 1, 1], 
       Hue[0.4607446702920191, 1, 1], 
       Hue[0.5610625757967369, 1, 1], 
       Hue[0.6617196759615321, 1, 1], 
       Hue[0.7627147629687548, 1, 1], 
       Hue[0.8640473849020327, 1, 1], 
       Hue[0.9657168711695278, 1, 1], 
       Hue[0.06772203209084474, 1, 1], 
       Hue[0.17006179329564264`, 1, 1], 
       Hue[0.2727348799293318, 1, 1], 
       Hue[0.3757400482244432, 1, 1], 
       Hue[0.47907615599813785`, 1, 1], 
       Hue[0.5827411225608394, 1, 1], 
       Hue[0.686732977202875, 1, 1], 
       Hue[0.7910505657857274, 1, 1], 
       Hue[0.8956910816409434, 1, 1], 
       Hue[0.0006518319749389761, 1, 1], 
       Hue[0.10592980643142957`, 1, 1], 
       Hue[0.21152177735264863`, 1, 1], 
       Hue[0.3174239892158998, 1, 1], 
       Hue[0.4236321033035602, 1, 1], 
       Hue[0.5301403332116573, 1, 1], 
       Hue[0.6369434399221541, 1, 1], 
       Hue[0.7440350187497498, 1, 1], 
       Hue[0.8514071856099105, 1, 1], 
       Hue[0.9590505875381703, 1, 1], 
       Hue[0.06695495506670603, 1, 1], 
       Hue[0.17510840763985963`, 1, 1], 
       Hue[0.2834976571366844, 1, 1], 
       Hue[0.3921067036965846, 1, 1], 
       Hue[0.5009171822537385, 1, 1], 
       Hue[0.609907960840949, 1, 1], 
       Hue[0.7190549385107554, 1, 1], 
       Hue[0.8283307021079086, 1, 1], 
       Hue[0.9377040687499487, 1, 1], 
       Hue[0.04713886558145295, 1, 1], 
       Hue[0.15659333346659132`, 1, 1], 
       Hue[0.26602139648303896`, 1, 1], 
       Hue[0.37537095342567955`, 1, 1], 
       Hue[0.48458443441848675`, 1, 1], 
       Hue[0.5935967404345542, 1, 1], 
       Hue[0.7023349586497987, 1, 1], 
       Hue[0.8107237607449748, 1, 1], 
       Hue[0.9186813036861303, 1, 1], 
       Hue[0.02611720447668775, 1, 1], 
       Hue[0.1329418218181866, 1, 1], 
       Hue[0.23906924624217446`, 1, 1], 
       Hue[0.34440835836416867`, 1, 1], 
       Hue[0.4488665860210912, 1, 1], 
       Hue[0.5523753592326575, 1, 1], 
       Hue[0.6548918734757004, 1, 1], 
       Hue[0.7563690714498079, 1, 1], 
       Hue[0.856754833013501, 1, 1], 
       Hue[0.9560476299526799, 1, 1], 
       Hue[0.05431861348849255, 1, 1], 
       Hue[0.15165859477128055`, 1, 1], 
       Hue[0.2481278362722127, 1, 1], 
       Hue[0.34377181562590414`, 1, 1], 
       Hue[0.43867250905938415`, 1, 1], 
       Hue[0.5329818091731069, 1, 1], 
       Hue[0.6268784974511232, 1, 1], 
       Hue[0.7204800452917036, 1, 1], 
       Hue[0.8138447115607892, 1, 1], 
       Hue[0.9070647396475882, 1, 1], 
       Hue[0.00027500323336431757`, 1, 1], 
       Hue[0.0936287218150815, 1, 1], 
       Hue[0.18738898166912799`, 1, 1], 
       Hue[0.2817608025135326, 1, 1], 
       Hue[0.3762560119614834, 1, 1], 
       Hue[0.46983126202546877`, 1, 1], 
       Hue[0.5628818631021897, 1, 1], 
       Hue[0.6583764075712767, 1, 1], 
       Hue[0.7583085727453893, 1, 1], 
       Hue[0.8597558378859961, 1, 1], 
       Hue[0.9565076209795028, 1, 1], 
       Hue[0.04627851579084141, 1, 1], 
       Hue[0.13696150887572137`, 1, 1], 
       Hue[0.236959712104721, 1, 1], 
       Hue[0.34512012689175703`, 1, 1], 
       Hue[0.45688774460841647`, 1, 1], 
       Hue[0.5703780558213525, 1, 1], 
       Hue[0.6836540521049628, 1, 1], 
       Hue[0.7838772463981823, 1, 1], 
       Hue[0.8381977472273094, 1, 1], 
       Hue[0.8870544606419305, 1, 1], 
       Hue[0.003981834612291627, 1, 1], 
       Hue[0.16018129844586784`, 1, 1], 
       Hue[0.33780729398489723`, 1, 1], 
       Hue[0.5369775037050852, 1, 1], 
       Hue[0.7905257246587302, 1, 1], 
       Hue[0.1945036657769734, 1, 1], 
       Hue[0.48496065810113864`, 1, 1], 
       Hue[0.7153646411488792, 1, 1], 
       Hue[0.9394067463789025, 1, 1], 
       Hue[0.17080211619971455`, 1, 1], 
       Hue[0.4207562168980021, 1, 1], 
       Hue[0.6977457893116823, 1, 1], 
       Hue[0.9810396549863314, 1, 1], 
       Hue[0.23870865443606856`, 1, 1], 
       Hue[0.4717879390337842, 1, 1], 
       Hue[0.6898493957007891, 1, 1], 
       Hue[0.8977125550908651, 1, 1], 
       Hue[0.09510862803082076, 1, 1], 
       Hue[0.22425606418634192`, 1, 1], 
       Hue[0.9819966346546343, 1, 1], 
       Hue[0.12933728393861235`, 1, 1], 
       Hue[0.24718059981823032`, 1, 1], 
       Hue[0.3560042502224064, 1, 1], 
       Hue[0.4832174243664257, 1, 1], 
       Hue[0.6225466914902852, 1, 1], 
       Hue[0.7669134155846555, 1, 1], 
       Hue[0.919797571218377, 1, 1], 
       Hue[0.08538170051501558, 1, 1], 
       Hue[0.2620427531752457, 1, 1], 
       Hue[0.44406556173858014`, 1, 1], 
       Hue[0.6185988292105646, 1, 1], 
       Hue[0.6838184160682961, 1, 1], 
       Hue[0.6378535206406792, 1, 1], 
       Hue[0.8073537712650531, 1, 1], 
       Hue[0.0008353865620092372, 1, 1], 
       Hue[0.1996041412947276, 1, 1], 
       Hue[0.3994928084092794, 1, 1], 
       Hue[0.598581266602507, 1, 1], 
       Hue[0.795286513537276, 1, 1], 
       Hue[0.9874661970418543, 1, 1], 
       Hue[0.17098866130101048`, 1, 1], 
       Hue[0.33608016823445647`, 1, 1], 
       Hue[0.4626610597119734, 1, 1], 
       Hue[0.5600193469710358, 1, 1], 
       Hue[0.6951140435126271, 1, 1], 
       Hue[0.8688175337237586, 1, 1], 
       Hue[0.06217243610467935, 1, 1], 
       Hue[0.26673052970331346`, 1, 1], 
       Hue[0.47918613708383234`, 1, 1], 
       Hue[0.6981752604591175, 1, 1], 
       Hue[0.923139519550703, 1, 1], 
       Hue[0.15391413553106462`, 1, 1], 
       Hue[0.39058746840666203`, 1, 1], 
       Hue[0.6335130531533011, 1, 1], 
       Hue[0.8834878616778492, 1, 1], 
       Hue[0.14218772425266332`, 1, 1], 
       Hue[0.4129145318615668, 1, 1], 
       Hue[0.7008647172568718, 1, 1], 
       Hue[0.008276153030424088, 1, 1], 
       Hue[0.32203152553002135`, 1, 1], 
       Hue[0.6228272969220539, 1, 1], 
       Hue[0.9077377802688978, 1, 1], 
       Hue[0.18295813338288494`, 1, 1], 
       Hue[0.45436977284719, 1, 1], 
       Hue[0.7264230564635294, 1, 1], 
       Hue[0.0029732826562475275`, 1, 1], 
       Hue[0.28772899918756284`, 1, 1], 
       Hue[0.5832291550635875, 1, 1], 
       Hue[0.8874633543999638, 1, 1], 
       Hue[0.1922430080867037, 1, 1], 
       Hue[0.489719235032304, 1, 1], 
       Hue[0.7780535457697576, 1, 1], 
       Hue[0.0590819613523457, 1, 1], 
       Hue[0.3350544804696013, 1, 1], 
       Hue[0.6076027369926792, 1, 1], 
       Hue[0.8777648644902436, 1, 1], 
       Hue[0.1461932685057524, 1, 1], 
       Hue[0.4133070810184004, 1, 1], 
       Hue[0.6793688455124909, 1, 1], 
       Hue[0.944513321626013, 1, 1], 
       Hue[0.20875530569204304`, 1, 1], 
       Hue[0.4719956959154934, 1, 1], 
       Hue[0.7340353362877327, 1, 1], 
       Hue[0.9945952634521681, 1, 1], 
       Hue[0.2533389529365845, 1, 1], 
       Hue[0.5098912530483598, 1, 1], 
       Hue[0.7638545596346901, 1, 1], 
       Hue[0.01483438802715526, 1, 1], 
       Hue[0.2624959819063229, 1, 1], 
       Hue[0.5066777947454011, 1, 1], 
       Hue[0.7475644420744317, 1, 1], 
       Hue[0.9858518958082757, 1, 1], 
       Hue[0.2227535812882585, 1, 1], 
       Hue[0.45973174016904605`, 1, 1], 
       Hue[0.6980657527124632, 1, 1], 
       Hue[0.9640532089338167, 1, 1], 
       Hue[0.7638938367871528, 1, 1], 
       Hue[0.7638938367871528, 1, 1], 
       Hue[0.5456953793797118, 1, 1], 
       Hue[0.5456953793797118, 1, 1], 
       Hue[0.4880359025202404, 1, 1], 
       Hue[0.4880359025202404, 1, 1], 
       Hue[0.418755936350388, 1, 1], 
       Hue[0.418755936350388, 1, 1], 
       Hue[0.22471186555443298`, 1, 1], 
       Hue[0.22471186555443298`, 1, 1], 
       Hue[0.7664365867469122, 1, 1], 
       Hue[0.7664365867469122, 1, 1], 
       Hue[0.7569067121630682, 1, 1], 
       Hue[0.7569067121630682, 1, 1], 
       Hue[0.8201765986296474, 1, 1], 
       Hue[0.8201765986296474, 1, 1], 
       Hue[0.8383737951258301, 1, 1], 
       Hue[0.8383737951258301, 1, 1], 
       Hue[0.8528114312520936, 1, 1], 
       Hue[0.8528114312520936, 1, 1], 
       Hue[0.0005814275632171318, 1, 1], 
       Hue[0.0005814275632171318, 1, 1], 
       Hue[0.09900120701277339, 1, 1], 
       Hue[0.09900120701277339, 1, 1], 
       Hue[0.1775661426240637, 1, 1], 
       Hue[0.1775661426240637, 1, 1], 
       Hue[0.24423919184988493`, 1, 1], 
       Hue[0.24423919184988493`, 1, 1], 
       Hue[0.3156688798563375, 1, 1], 
       Hue[0.3156688798563375, 1, 1], 
       Hue[0.38950220432683996`, 1, 1], 
       Hue[0.38950220432683996`, 1, 1], 
       Hue[0.4814958525831977, 1, 1], 
       Hue[0.4814958525831977, 1, 1], 
       Hue[0.5657261147473843, 1, 1], 
       Hue[0.5657261147473843, 1, 1], 
       Hue[0.6467337892415539, 1, 1], 
       Hue[0.6467337892415539, 1, 1], 
       Hue[0.7248416611021938, 1, 1], 
       Hue[0.7248416611021938, 1, 1], 
       Hue[0.8055142767348117, 1, 1], 
       Hue[0.8055142767348117, 1, 1], 
       Hue[0.8872992703340177, 1, 1], 
       Hue[0.8872992703340177, 1, 1], 
       Hue[0.9720085880616776, 1, 1], 
       Hue[0.9720085880616776, 1, 1], 
       Hue[0.05610154862290952, 1, 1], 
       Hue[0.05610154862290952, 1, 1], 
       Hue[0.13926945181155012`, 1, 1], 
       Hue[0.13926945181155012`, 1, 1], 
       Hue[0.22225896671723153`, 1, 1], 
       Hue[0.22225896671723153`, 1, 1], 
       Hue[0.30594469461240686`, 1, 1], 
       Hue[0.30594469461240686`, 1, 1], 
       Hue[0.3907297817738525, 1, 1], 
       Hue[0.3907297817738525, 1, 1], 
       Hue[0.47584462841575326`, 1, 1], 
       Hue[0.47584462841575326`, 1, 1], 
       Hue[0.5613011947128719, 1, 1], 
       Hue[0.5613011947128719, 1, 1], 
       Hue[0.6467734454202569, 1, 1], 
       Hue[0.6467734454202569, 1, 1], 
       Hue[0.732620054471488, 1, 1], 
       Hue[0.732620054471488, 1, 1], 
       Hue[0.8188508419403508, 1, 1], 
       Hue[0.8188508419403508, 1, 1], 
       Hue[0.9055713829157245, 1, 1], 
       Hue[0.9055713829157245, 1, 1], 
       Hue[0.9926765120282037, 1, 1], 
       Hue[0.9926765120282037, 1, 1], 
       Hue[0.08008348174035529, 1, 1], 
       Hue[0.08008348174035529, 1, 1], 
       Hue[0.1678032957843583, 1, 1], 
       Hue[0.1678032957843583, 1, 1], 
       Hue[0.2558772079638403, 1, 1], 
       Hue[0.2558772079638403, 1, 1], 
       Hue[0.34433867572467236`, 1, 1], 
       Hue[0.34433867572467236`, 1, 1], 
       Hue[0.43317536470416307`, 1, 1], 
       Hue[0.43317536470416307`, 1, 1], 
       Hue[0.5223512697273097, 1, 1], 
       Hue[0.5223512697273097, 1, 1], 
       Hue[0.611865760320544, 1, 1], 
       Hue[0.611865760320544, 1, 1], 
       Hue[0.7017376629319375, 1, 1], 
       Hue[0.7017376629319375, 1, 1], 
       Hue[0.791973646004684, 1, 1], 
       Hue[0.791973646004684, 1, 1], 
       Hue[0.8825715119856232, 1, 1], 
       Hue[0.8825715119856232, 1, 1], 
       Hue[0.9735201231658492, 1, 1], 
       Hue[0.9735201231658492, 1, 1], 
       Hue[0.06481664673905892, 1, 1], 
       Hue[0.06481664673905892, 1, 1], 
       Hue[0.1564624526544423, 1, 1], 
       Hue[0.1564624526544423, 1, 1], 
       Hue[0.24846339708111093`, 1, 1], 
       Hue[0.24846339708111093`, 1, 1], 
       Hue[0.34082044254039434`, 1, 1], 
       Hue[0.34082044254039434`, 1, 1], 
       Hue[0.4335285188995974, 1, 1], 
       Hue[0.4335285188995974, 1, 1], 
       Hue[0.5265858504718376, 1, 1], 
       Hue[0.5265858504718376, 1, 1], 
       Hue[0.619991238019795, 1, 1], 
       Hue[0.619991238019795, 1, 1], 
       Hue[0.7137489787936226, 1, 1], 
       Hue[0.7137489787936226, 1, 1], 
       Hue[0.8078581491198726, 1, 1], 
       Hue[0.8078581491198726, 1, 1], 
       Hue[0.9023167908162214, 1, 1], 
       Hue[0.9023167908162214, 1, 1], 
       Hue[0.997123518820222, 1, 1], 
       Hue[0.997123518820222, 1, 1], 
       Hue[0.09227809771084206, 1, 1], 
       Hue[0.09227809771084206, 1, 1], 
       Hue[0.18778045984197525`, 1, 1], 
       Hue[0.18778045984197525`, 1, 1], 
       Hue[0.28363150538320664`, 1, 1], 
       Hue[0.28363150538320664`, 1, 1], 
       Hue[0.37982913467653123`, 1, 1], 
       Hue[0.37982913467653123`, 1, 1], 
       Hue[0.4763730534549754, 1, 1], 
       Hue[0.4763730534549754, 1, 1], 
       Hue[0.5732620582331205, 1, 1], 
       Hue[0.5732620582331205, 1, 1], 
       Hue[0.6704969545124291, 1, 1], 
       Hue[0.6704969545124291, 1, 1], 
       Hue[0.7680769044926821, 1, 1], 
       Hue[0.7680769044926821, 1, 1], 
       Hue[0.866001209191981, 1, 1], 
       Hue[0.866001209191981, 1, 1], 
       Hue[0.9642694798933042, 1, 1], 
       Hue[0.9642694798933042, 1, 1], 
       Hue[0.06288033404720918, 1, 1], 
       Hue[0.06288033404720918, 1, 1], 
       Hue[0.1618340190608663, 1, 1], 
       Hue[0.1618340190608663, 1, 1], 
       Hue[0.26112965826933277`, 1, 1], 
       Hue[0.26112965826933277`, 1, 1], 
       Hue[0.36076682669457827`, 1, 1], 
       Hue[0.36076682669457827`, 1, 1], 
       Hue[0.4607446702920191, 1, 1], 
       Hue[0.4607446702920191, 1, 1], 
       Hue[0.5610625757967369, 1, 1], 
       Hue[0.5610625757967369, 1, 1], 
       Hue[0.6617196759615321, 1, 1], 
       Hue[0.6617196759615321, 1, 1], 
       Hue[0.7627147629687548, 1, 1], 
       Hue[0.7627147629687548, 1, 1], 
       Hue[0.8640473849020327, 1, 1], 
       Hue[0.8640473849020327, 1, 1], 
       Hue[0.9657168711695278, 1, 1], 
       Hue[0.9657168711695278, 1, 1], 
       Hue[0.06772203209084474, 1, 1], 
       Hue[0.06772203209084474, 1, 1], 
       Hue[0.17006179329564264`, 1, 1], 
       Hue[0.17006179329564264`, 1, 1], 
       Hue[0.2727348799293318, 1, 1], 
       Hue[0.2727348799293318, 1, 1], 
       Hue[0.3757400482244432, 1, 1], 
       Hue[0.3757400482244432, 1, 1], 
       Hue[0.47907615599813785`, 1, 1], 
       Hue[0.47907615599813785`, 1, 1], 
       Hue[0.5827411225608394, 1, 1], 
       Hue[0.5827411225608394, 1, 1], 
       Hue[0.686732977202875, 1, 1], 
       Hue[0.686732977202875, 1, 1], 
       Hue[0.7910505657857274, 1, 1], 
       Hue[0.7910505657857274, 1, 1], 
       Hue[0.8956910816409434, 1, 1], 
       Hue[0.8956910816409434, 1, 1], 
       Hue[0.0006518319749389761, 1, 1], 
       Hue[0.0006518319749389761, 1, 1], 
       Hue[0.10592980643142957`, 1, 1], 
       Hue[0.10592980643142957`, 1, 1], 
       Hue[0.21152177735264863`, 1, 1], 
       Hue[0.21152177735264863`, 1, 1], 
       Hue[0.3174239892158998, 1, 1], 
       Hue[0.3174239892158998, 1, 1], 
       Hue[0.4236321033035602, 1, 1], 
       Hue[0.4236321033035602, 1, 1], 
       Hue[0.5301403332116573, 1, 1], 
       Hue[0.5301403332116573, 1, 1], 
       Hue[0.6369434399221541, 1, 1], 
       Hue[0.6369434399221541, 1, 1], 
       Hue[0.7440350187497498, 1, 1], 
       Hue[0.7440350187497498, 1, 1], 
       Hue[0.8514071856099105, 1, 1], 
       Hue[0.8514071856099105, 1, 1], 
       Hue[0.9590505875381703, 1, 1], 
       Hue[0.9590505875381703, 1, 1], 
       Hue[0.06695495506670603, 1, 1], 
       Hue[0.06695495506670603, 1, 1], 
       Hue[0.17510840763985963`, 1, 1], 
       Hue[0.17510840763985963`, 1, 1], 
       Hue[0.2834976571366844, 1, 1], 
       Hue[0.2834976571366844, 1, 1], 
       Hue[0.3921067036965846, 1, 1], 
       Hue[0.3921067036965846, 1, 1], 
       Hue[0.5009171822537385, 1, 1], 
       Hue[0.5009171822537385, 1, 1], 
       Hue[0.609907960840949, 1, 1], 
       Hue[0.609907960840949, 1, 1], 
       Hue[0.7190549385107554, 1, 1], 
       Hue[0.7190549385107554, 1, 1], 
       Hue[0.8283307021079086, 1, 1], 
       Hue[0.8283307021079086, 1, 1], 
       Hue[0.9377040687499487, 1, 1], 
       Hue[0.9377040687499487, 1, 1], 
       Hue[0.04713886558145295, 1, 1], 
       Hue[0.04713886558145295, 1, 1], 
       Hue[0.15659333346659132`, 1, 1], 
       Hue[0.15659333346659132`, 1, 1], 
       Hue[0.26602139648303896`, 1, 1], 
       Hue[0.26602139648303896`, 1, 1], 
       Hue[0.37537095342567955`, 1, 1], 
       Hue[0.37537095342567955`, 1, 1], 
       Hue[0.48458443441848675`, 1, 1], 
       Hue[0.48458443441848675`, 1, 1], 
       Hue[0.5935967404345542, 1, 1], 
       Hue[0.5935967404345542, 1, 1], 
       Hue[0.7023349586497987, 1, 1], 
       Hue[0.7023349586497987, 1, 1], 
       Hue[0.8107237607449748, 1, 1], 
       Hue[0.8107237607449748, 1, 1], 
       Hue[0.9186813036861303, 1, 1], 
       Hue[0.9186813036861303, 1, 1], 
       Hue[0.02611720447668775, 1, 1], 
       Hue[0.02611720447668775, 1, 1], 
       Hue[0.1329418218181866, 1, 1], 
       Hue[0.1329418218181866, 1, 1], 
       Hue[0.23906924624217446`, 1, 1], 
       Hue[0.23906924624217446`, 1, 1], 
       Hue[0.34440835836416867`, 1, 1], 
       Hue[0.34440835836416867`, 1, 1], 
       Hue[0.4488665860210912, 1, 1], 
       Hue[0.4488665860210912, 1, 1], 
       Hue[0.5523753592326575, 1, 1], 
       Hue[0.5523753592326575, 1, 1], 
       Hue[0.6548918734757004, 1, 1], 
       Hue[0.6548918734757004, 1, 1], 
       Hue[0.7563690714498079, 1, 1], 
       Hue[0.7563690714498079, 1, 1], 
       Hue[0.856754833013501, 1, 1], 
       Hue[0.856754833013501, 1, 1], 
       Hue[0.9560476299526799, 1, 1], 
       Hue[0.9560476299526799, 1, 1], 
       Hue[0.05431861348849255, 1, 1], 
       Hue[0.05431861348849255, 1, 1], 
       Hue[0.15165859477128055`, 1, 1], 
       Hue[0.15165859477128055`, 1, 1], 
       Hue[0.2481278362722127, 1, 1], 
       Hue[0.2481278362722127, 1, 1], 
       Hue[0.34377181562590414`, 1, 1], 
       Hue[0.34377181562590414`, 1, 1], 
       Hue[0.43867250905938415`, 1, 1], 
       Hue[0.43867250905938415`, 1, 1], 
       Hue[0.5329818091731069, 1, 1], 
       Hue[0.5329818091731069, 1, 1], 
       Hue[0.6268784974511232, 1, 1], 
       Hue[0.6268784974511232, 1, 1], 
       Hue[0.7204800452917036, 1, 1], 
       Hue[0.7204800452917036, 1, 1], 
       Hue[0.8138447115607892, 1, 1], 
       Hue[0.8138447115607892, 1, 1], 
       Hue[0.9070647396475882, 1, 1], 
       Hue[0.9070647396475882, 1, 1], 
       Hue[0.00027500323336431757`, 1, 1], 
       Hue[0.00027500323336431757`, 1, 1], 
       Hue[0.0936287218150815, 1, 1], 
       Hue[0.0936287218150815, 1, 1], 
       Hue[0.18738898166912799`, 1, 1], 
       Hue[0.18738898166912799`, 1, 1], 
       Hue[0.2817608025135326, 1, 1], 
       Hue[0.2817608025135326, 1, 1], 
       Hue[0.3762560119614834, 1, 1], 
       Hue[0.3762560119614834, 1, 1], 
       Hue[0.46983126202546877`, 1, 1], 
       Hue[0.46983126202546877`, 1, 1], 
       Hue[0.5628818631021897, 1, 1], 
       Hue[0.5628818631021897, 1, 1], 
       Hue[0.6583764075712767, 1, 1], 
       Hue[0.6583764075712767, 1, 1], 
       Hue[0.7583085727453893, 1, 1], 
       Hue[0.7583085727453893, 1, 1], 
       Hue[0.8597558378859961, 1, 1], 
       Hue[0.8597558378859961, 1, 1], 
       Hue[0.9565076209795028, 1, 1], 
       Hue[0.9565076209795028, 1, 1], 
       Hue[0.04627851579084141, 1, 1], 
       Hue[0.04627851579084141, 1, 1], 
       Hue[0.13696150887572137`, 1, 1], 
       Hue[0.13696150887572137`, 1, 1], 
       Hue[0.236959712104721, 1, 1], 
       Hue[0.236959712104721, 1, 1], 
       Hue[0.34512012689175703`, 1, 1], 
       Hue[0.34512012689175703`, 1, 1], 
       Hue[0.45688774460841647`, 1, 1], 
       Hue[0.45688774460841647`, 1, 1], 
       Hue[0.5703780558213525, 1, 1], 
       Hue[0.5703780558213525, 1, 1], 
       Hue[0.6836540521049628, 1, 1], 
       Hue[0.6836540521049628, 1, 1], 
       Hue[0.7838772463981823, 1, 1], 
       Hue[0.7838772463981823, 1, 1], 
       Hue[0.8381977472273094, 1, 1], 
       Hue[0.8381977472273094, 1, 1], 
       Hue[0.8870544606419305, 1, 1], 
       Hue[0.8870544606419305, 1, 1], 
       Hue[0.003981834612291627, 1, 1], 
       Hue[0.003981834612291627, 1, 1], 
       Hue[0.16018129844586784`, 1, 1], 
       Hue[0.16018129844586784`, 1, 1], 
       Hue[0.33780729398489723`, 1, 1], 
       Hue[0.33780729398489723`, 1, 1], 
       Hue[0.5369775037050852, 1, 1], 
       Hue[0.5369775037050852, 1, 1], 
       Hue[0.7905257246587302, 1, 1], 
       Hue[0.7905257246587302, 1, 1], 
       Hue[0.1945036657769734, 1, 1], 
       Hue[0.1945036657769734, 1, 1], 
       Hue[0.48496065810113864`, 1, 1], 
       Hue[0.48496065810113864`, 1, 1], 
       Hue[0.7153646411488792, 1, 1], 
       Hue[0.7153646411488792, 1, 1], 
       Hue[0.9394067463789025, 1, 1], 
       Hue[0.9394067463789025, 1, 1], 
       Hue[0.17080211619971455`, 1, 1], 
       Hue[0.17080211619971455`, 1, 1], 
       Hue[0.4207562168980021, 1, 1], 
       Hue[0.4207562168980021, 1, 1], 
       Hue[0.6977457893116823, 1, 1], 
       Hue[0.6977457893116823, 1, 1], 
       Hue[0.9810396549863314, 1, 1], 
       Hue[0.9810396549863314, 1, 1], 
       Hue[0.23870865443606856`, 1, 1], 
       Hue[0.23870865443606856`, 1, 1], 
       Hue[0.4717879390337842, 1, 1], 
       Hue[0.4717879390337842, 1, 1], 
       Hue[0.6898493957007891, 1, 1], 
       Hue[0.6898493957007891, 1, 1], 
       Hue[0.8977125550908651, 1, 1], 
       Hue[0.8977125550908651, 1, 1], 
       Hue[0.09510862803082076, 1, 1], 
       Hue[0.09510862803082076, 1, 1], 
       Hue[0.22425606418634192`, 1, 1], 
       Hue[0.22425606418634192`, 1, 1], 
       Hue[0.9819966346546343, 1, 1], 
       Hue[0.9819966346546343, 1, 1], 
       Hue[0.12933728393861235`, 1, 1], 
       Hue[0.12933728393861235`, 1, 1], 
       Hue[0.24718059981823032`, 1, 1], 
       Hue[0.24718059981823032`, 1, 1], 
       Hue[0.3560042502224064, 1, 1], 
       Hue[0.3560042502224064, 1, 1], 
       Hue[0.4832174243664257, 1, 1], 
       Hue[0.4832174243664257, 1, 1], 
       Hue[0.6225466914902852, 1, 1], 
       Hue[0.6225466914902852, 1, 1], 
       Hue[0.7669134155846555, 1, 1], 
       Hue[0.7669134155846555, 1, 1], 
       Hue[0.919797571218377, 1, 1], 
       Hue[0.919797571218377, 1, 1], 
       Hue[0.08538170051501558, 1, 1], 
       Hue[0.08538170051501558, 1, 1], 
       Hue[0.2620427531752457, 1, 1], 
       Hue[0.2620427531752457, 1, 1], 
       Hue[0.44406556173858014`, 1, 1], 
       Hue[0.44406556173858014`, 1, 1], 
       Hue[0.6185988292105646, 1, 1], 
       Hue[0.6185988292105646, 1, 1], 
       Hue[0.6838184160682961, 1, 1], 
       Hue[0.6838184160682961, 1, 1], 
       Hue[0.6378535206406792, 1, 1], 
       Hue[0.6378535206406792, 1, 1], 
       Hue[0.8073537712650531, 1, 1], 
       Hue[0.8073537712650531, 1, 1], 
       Hue[0.0008353865620092372, 1, 1], 
       Hue[0.0008353865620092372, 1, 1], 
       Hue[0.1996041412947276, 1, 1], 
       Hue[0.1996041412947276, 1, 1], 
       Hue[0.3994928084092794, 1, 1], 
       Hue[0.3994928084092794, 1, 1], 
       Hue[0.598581266602507, 1, 1], 
       Hue[0.598581266602507, 1, 1]}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->{True, None},
    AxesOrigin->{0, 0},
    Epilog->
     LineBox[{{1, 4.994783395296727*^-7}, {2, 3.986765672925796*^-7}, {
       3, 3.6484116999930404`*^-7}, {4, 4.244605140076791*^-7}, {
       5, 9.418473172586208*^-7}, {6, 6.977534201348251*^-7}, {
       7, 4.324780565545246*^-7}, {8, 7.739143884695598*^-7}, {
       9, 1.4925662927061932`*^-6}, {10, 2.7363528323711107`*^-6}, {
       11, 2.3694317865726362`*^-6}, {12, 2.9428984745531942`*^-6}, {
       13, 4.927896848144337*^-6}, {14, 7.473823446927416*^-6}, {
       15, 0.000011730229772105373`}, {16, 0.00001624049470443861}, {
       17, 0.000020944855409634024`}, {18, 0.000028871176479729104`}, {
       19, 0.00004168759055026988}, {20, 0.000058977781410749604`}, {
       21, 0.0000821512438370269}, {22, 0.00011153933646128924`}, {
       23, 0.0001501877725257336}, {24, 0.0002028127511679077}, {
       25, 0.0002752063422036464}, {26, 0.00037196451476663195`}, {
       27, 0.0004968254289896502}, {28, 0.0006569383380542082}, {
       29, 0.0008646777264225473}, {30, 0.0011341787163496409`}, {
       31, 0.0014812983035309155`}, {32, 0.00192451958873768}, {
       33, 0.002482691121822296}, {34, 0.003184244258854491}, {
       35, 0.004061454108004423}, {36, 0.005153334725356818}, {
       37, 0.006503897404234565}, {38, 0.00816267274936306}, {
       39, 0.010184664622306338`}, {40, 0.012635410541207903`}, {
       41, 0.015590589359775657`}, {42, 0.019132140781103642`}, {
       43, 0.02334596316779318}, {44, 0.028326687478641765`}, {
       45, 0.03417704305872553}, {46, 0.04100739099097344}, {
       47, 0.04893038528968952}, {48, 0.05805861289827473}, {
       49, 0.06850378714107186}, {50, 0.08037362824657279}, {
       51, 0.09377546516286478}, {52, 0.10880283771743832`}, {
       53, 0.12553376031808236`}, {54, 0.1440262124893879}, {
       55, 0.1643184702438419}, {56, 0.18642212994643712`}, {
       57, 0.21031955947502556`}, {58, 0.23595640277360674`}, {
       59, 0.2632369775285807}, {60, 0.2920304586291676}, {
       61, 0.3221626679815606}, {62, 0.35342105068282276`}, {
       63, 0.38554662093534187`}, {64, 0.4182415652473806}, {
       65, 0.4511732152331836}, {66, 0.4839799614117237}, {
       67, 0.516273069711347}, {68, 0.5476464942019678}, {
       69, 0.577682177984169}, {70, 0.6059621045677198}, {
       71, 0.6320778322936406}, {72, 0.6556404966435839}, {
       73, 0.676285525540514}, {74, 0.693689416263403}, {
       75, 0.7075731820501371}, {76, 0.7177089826948957}, {
       77, 0.7239293602809228}, {78, 0.7261337407825015}, {
       79, 0.7242859668634876}, {80, 0.7184165466300587}, {
       81, 0.7086250347294344}, {82, 0.695074795190095}, {
       83, 0.677987255579029}, {84, 0.6576373569956018}, {
       85, 0.6343483778240276}, {86, 0.6084786474910284}, {
       87, 0.5804163306991071}, {88, 0.5505677145457509}, {
       89, 0.519347793284551}, {90, 0.4871724957189612}, {
       91, 0.45444776977078666`}, {92, 0.42156145609977047`}, {
       93, 0.3888776271948038}, {94, 0.3567295637615142}, {
       95, 0.3254148972148669}, {96, 0.2951925286448773}, {
       97, 0.2662803790311598}, {98, 0.23885501015649402`}, {
       99, 0.2130524435276181}, {100, 0.18896874158666613`}, {
       101, 0.16666166593939843`}, {102, 0.14615450600623264`}, {
       103, 0.1274408271590448}, {104, 0.11048699731783652`}, {
       105, 0.09523581019732066}, {106, 0.08161186277289911}, {
       107, 0.0695251588752548}, {108, 0.058875386586847635`}, {
       109, 0.04955547836873383}, {110, 0.041454307948559935`}, {
       111, 0.034459699409057146`}, {112, 0.028460898980589244`}, {
       113, 0.02335110917215513}, {114, 0.0190283327245}, {
       115, 0.01539628957952017}, {116, 0.012366074596434767`}, {
       117, 0.00985640965709907}, {118, 0.007793159716799637}, {
       119, 0.006109706828236233}, {120, 0.004747389885704473}, {
       121, 0.003654630612157861}, {122, 0.00278583449591614}, {
       123, 0.002101416925266802}, {124, 0.0015679027167395609`}, {
       125, 0.0011569275591605903`}, {126, 0.0008440839473146381}, {
       127, 0.000608650078946116}, {128, 0.0004335854234812388}, {
       129, 0.000305138037729062}, {130, 0.0002122319275392624}, {
       131, 0.00014596258855150972`}, {132, 0.00009928326149451607}, {
       133, 0.00006679085306027818}, {134, 0.00004445655855107141}, {
       135, 0.000029301007706538234`}, {136, 0.000019137126774696508`}, {
       137, 0.000012394565342042676`}, {138, 7.962990185967206*^-6}, {
       139, 5.065137310831807*^-6}, {140, 3.181557334776959*^-6}, {
       141, 1.981775686439238*^-6}, {142, 1.2399440938483532`*^-6}, {
       143, 7.818038533064367*^-7}, {144, 4.862508096344292*^-7}, {
       145, 2.909588468730301*^-7}, {146, 1.6860679387934232`*^-7}, {
       147, 9.943081735171632*^-8}, {148, 6.198255773861349*^-8}, {
       149, 3.9474549989156254`*^-8}, {150, 2.44297731769855*^-8}, {
       151, 1.447854293784885*^-8}, {152, 8.20793397722054*^-9}, {
       153, 4.342580316686153*^-9}, {154, 2.062342025287604*^-9}, {
       155, 1.0076368918492748`*^-9}, {156, 7.896011603640955*^-10}, {
       157, 6.98709259034934*^-10}, {158, 5.239002823885162*^-10}, {
       159, 3.247461338186444*^-10}, {160, 1.6094234553971135`*^-10}, {
       161, 5.7890093799645316`*^-11}, {162, 3.277946154119626*^-11}, {
       163, 4.214313430738415*^-11}, {164, 3.9492506979803*^-11}, {
       165, 2.955361322243714*^-11}, {166, 1.884218596997743*^-11}, {
       167, 1.0776428920419602`*^-11}, {168, 6.264934023401239*^-12}, {
       169, 4.331228658996611*^-12}, {170, 3.241645366087931*^-12}, {
       171, 2.2306143417328605`*^-12}, {172, 1.31707993640091*^-12}, {
       173, 6.323571184904889*^-13}, {174, 2.1377788431473829`*^-13}, {
       175, 1.4334996999488572`*^-14}, {176, 5.101102014106216*^-14}, {
       177, 5.2624323987021615`*^-14}, {178, 4.124389086521208*^-14}, {
       179, 3.5078488374246167`*^-14}, {180, 3.0903911663198325`*^-14}, {
       181, 2.5791482943213817`*^-14}, {182, 2.079675721174681*^-14}, {
       183, 1.6507204738868056`*^-14}, {184, 1.2465274374698773`*^-14}, {
       185, 8.476546445570054*^-15}, {186, 4.885914903938889*^-15}, {
       187, 2.1220261024332918`*^-15}, {188, 5.151240139758338*^-16}, {
       189, 7.99574431085779*^-16}, {190, 1.056800930724421*^-15}, {
       191, 9.846417193976231*^-16}, {192, 7.662773364131214*^-16}, {
       193, 5.274523526772276*^-16}, {194, 3.290537145353*^-16}, {
       195, 1.8798858806434994`*^-16}, {196, 9.851637199785806*^-17}, {
       197, 4.713023954228207*^-17}, {198, 2.057607043224922*^-17}, {
       199, 8.950115268951903*^-18}, {200, 5.351109179036183*^-18}, {
       201, 4.237162057364687*^-18}, {202, 3.2896698581742215`*^-18}, {
       203, 2.354117431939352*^-18}, {204, 1.5660407574147986`*^-18}, {
       205, 9.800090437312581*^-19}, {206, 5.816804785448641*^-19}, {
       207, 3.2922684628414906`*^-19}, {208, 1.7831273345047157`*^-19}, {
       209, 9.261475510174089*^-20}, {210, 4.6173453453776934`*^-20}, {
       211, 2.2091537893628526`*^-20}, {212, 1.0139860177673312`*^-20}, {
       213, 4.4851396665016416`*^-21}, {214, 1.9624031100872154`*^-21}, {
       215, 9.160410256889905*^-22}, {216, 4.953388953388919*^-22}, {
       217, 2.9782906916741086`*^-22}, {218, 1.8038045164791116`*^-22}, {
       219, 1.04671288746965*^-22}, {220, 5.74468597913101*^-23}, {
       221, 2.984204839762825*^-23}, {222, 1.478437538719076*^-23}, {
       223, 7.107342540248776*^-24}, {224, 3.4315368410250665`*^-24}, {
       225, 1.745920460011025*^-24}, {226, 9.606622142992313*^-25}, {
       227, 5.585118706486809*^-25}, {228, 3.2956046235983315`*^-25}, {
       229, 1.9246520742020925`*^-25}, {230, 1.1018257114828082`*^-25}, {
       231, 6.170538318072313*^-26}, {232, 3.382433919856732*^-26}, {
       233, 1.817069358656203*^-26}, {234, 9.579390093348711*^-27}, {
       235, 4.9627919182631355`*^-27}, {236, 2.5301041513653094`*^-27}, {
       237, 1.2710356470657414`*^-27}, {238, 6.2994704638294175`*^-28}, {
       239, 3.0831482420624158`*^-28}, {240, 1.4912070596623323`*^-28}, {
       241, 7.131500414102161*^-29}, {242, 3.3745137194828654`*^-29}, {
       243, 1.5816624089791583`*^-29}, {244, 7.358064935844816*^-30}, {
       245, 3.408962563464699*^-30}, {246, 1.5807318097306387`*^-30}, {
       247, 7.3829435981384236`*^-31}, {248, 3.495756697896012*^-31}, {
       249, 1.6854287796342513`*^-31}, {250, 8.278063589009809*^-32}, {
       251, 4.1264891084231985`*^-32}}],
    Frame->True,
    FrameTicks->{{{1., 
        FormBox[
         RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
        FormBox[
         RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
        FormBox[
         RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
        FormBox[
         RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
        FormBox["0", TraditionalForm]}, {157.25, 
        FormBox["2.5`", TraditionalForm]}, {188.5, 
        FormBox["5", TraditionalForm]}, {219.75, 
        FormBox["7.5`", TraditionalForm]}, {251., 
        FormBox["10", TraditionalForm]}}, Automatic, None, None},
    PlotRange->{-0.1, 1.1},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], Automatic}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF2XdUFIcaBfDR6BNLwKgxIOpbFdHEhlQRkauASFHpvQy9xIYlMQTJJAbU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     "], {
      {GrayLevel[0.5], EdgeForm[None], 
       GraphicsGroupBox[{PolygonBox[{{212, 674, 213}}], 
         PolygonBox[CompressedData["
1:eJwtl3fYjmUYh6/7sRVlhJRZmSUhykhkE7Iqe5PssmUnIUWyEkmU1TZDqSia
aO+99550/o6fP87jOL/nua/rvu5xve/3lus7osPwLCKOQs4TZCkiB+TCT4JT
8UJwcpjCeBEogOeAwFM6EQvH4PiJd8qRU7kgN54fCuKnJOfOA3nxfJA3PCY3
nif5neYoip8GBfF8qkFzpxO5oBheXDnx4lAJrwwlVDuUxM/QGsKciZeCwngR
KIuXg6LhHCXw05NjNaY0XiZ5rMaUx89STeE5quBVFRN+djZ+DhTDa0JvvA/U
Cj+rgFdMrlVrzq+9SF6bcpyLnwclw1TDz9caVAvUxuuopjAX4RdDWeWGZnhz
zYGXglr4hcmxGlMXr6c1h6mPN4Dyqh0a45dpDWGa4E2Tc2vMJXhD7YH2Emrg
NZPn0pwt8JZQCa8MbfDLoQpeHbri3eACrRU64B215nBMK7x1cqzWXF1jk+eq
Cu3xK5Jjtad98X5aY3iOtni75LHVoAt+pfYwPGd3vAfUCNMT75V8VhpzFX51
cq2qqRPeOTmX1nwp3ih5rzRnf3wA1FYsbMQ3KYfOBobg1+qMwgzFh0Fd5YKJ
+CTlxOvAYPya5FiNGY6P0JmFGYmPgvo6CxiLj9OZhBmPT0jOrTGj8et0xuE5
JuM3QOPws+vxMcm59GwKPlV3AG8Nt+ELoY3uAszAZ+pOhJmF3wjNwszGb9Id
DDMHvxla4C1hPn4LtMLbwTJ8ObQPz7EIv113Jsxi/A5oG45ZgN+aXJtqnIZP
T65NY5bgS5NzK+cK/E7dmfDn3N18SK2BV/GO+pt3a6BTmHvwtTpzvBs8hD8M
3XVXYRW+OjlWY+7F10GXMOvx+3TnwndgM74FuoZ5AH8wObfG3I9vSL47muMR
/FHd0TCP4VuhZ5ht+HbdWXwgHMCfhUF4b9iFPw59wuzG90BfvB88ie+D/uEc
O/CdybG6wwOVK/kuKmYv/kRyrPZwJX5X8l5ozufwgzA4zCH8ed1hfAA8g+9P
rlVzPoU/nfxOY17AX4QhYV7CX1bPhHkFPwxD1QvwOv6GeiLMm/hbMFK9AJ/j
X6gn8GHwKv5acqzGvI2/ox4K8y7+HoxWL8DH+CfqiTCf4p8l59aY9/EP1FPh
Ob7Ev4Lx4Wcf4h8l59Kzr/Fv1JP4dPgX/w9m4BPhe/wHmBTmR/wnmBzmZ/wX
9WyYX/HfYAo+Ff7E/4Jpups6Q+52HVgdnuMY746rZ8PoSzrBrHDM37z7J7k2
1fgt/l1ybRqTMTYH3BgmJ54LZofJjeeBm/A5kB8/CW4OczJeAOaGKYifAvPC
nIoXgvlhCuNF4JYwRfHTYEGYYnhxuDVMCfx0uC1MSfwMWBjmTLwULArXmBfP
l7lWPSuNl4Hb8cVQHj8L7sCXQAW8IiwNUwmvDMvCMWXxcpljFXM2fk7mWJ3R
7+zlH8lnpZgqvKsKy/EVUA0/H+4MUx2/AFaGqYHXhLvC1MIvzHzWynEufl7m
XOqRI8x1NPnu6w5cxLuL4W58DdTHG8A9+Fq4FG8E94ZpjF8G6/BN0B6/AjaH
Yy7BG2aO1ZgmeFNYH6YZ3hzuwzdAG/xy2BimLd4uc26NaYG3hPvDc3TAO8KW
MJ3wzvBAmC74lfAgvg0G4oNgO/4QdMW7wcNhuuM94BH8UeiN94HHwjmuwq/O
HKsaWuGtM9eumJ54r8yx2sO6eL3Me6k5B+PXwI4wQ/BrYSe+FfrjAzLXqjn7
4v0yv9N34lzOal7yd6NihvJuGOzCH4eR+CjYje+B6/ExsDfMWHwcPBFmPD4B
ngwzEZ8E+8JMxm+Ap8JMwafC02Gm4dPhmTAz8JmwP8wsfRbAgTCz1fvwbLjG
0fh1mWvVszn6LIDn8IMwX70Nh/Dn1bPqVXghzCL1IrwYjpmLz8scq5gF6v3M
sdoj/YAZkXmvFLNYvQsv4S+r59Rr8EqYFeo1OBxmpXoLjoRZha/O/L+CcixR
72fOxTHFv/Bf+LfO/83MatU=
          "]]}]}, {{}, {}, {
        {Hue[0.1880703612269147, 1, 1], LineBox[{1, 2}]}, 
        {Hue[0.07884389201651706, 1, 1], LineBox[{2, 3}]}, 
        {Hue[0.997136729011347, 1, 1], LineBox[{3, 4}]}, 
        {Hue[0.8689395434525204, 1, 1], LineBox[{4, 5}]}, 
        {Hue[0.7062546158712821, 1, 1], LineBox[{5, 6}]}, 
        {Hue[0.7388461845627924, 1, 1], LineBox[{6, 7}]}, 
        {Hue[0.5855521737701119, 1, 1], LineBox[{7, 8}]}, 
        {Hue[0.5339300205386036, 1, 1], LineBox[{8, 9}]}, 
        {Hue[0.46682903323219344`, 1, 1], LineBox[{9, 10}]}, 
        {Hue[0.3723331935375797, 1, 1], LineBox[{10, 11}]}, 
        {Hue[0.21409818648276444`, 1, 1], LineBox[{11, 12}]}, 
        {Hue[0.12489905314846222`, 1, 1], LineBox[{12, 13}]}, 
        {Hue[0.03131991234497626, 1, 1], LineBox[{13, 14}]}, 
        {Hue[0.0021162108163126875`, 1, 1], LineBox[{14, 15}]}, 
        {Hue[0.9204190719936995, 1, 1], LineBox[{15, 16}]}, 
        {Hue[0.8532338736598175, 1, 1], LineBox[{16, 17}]}, 
        {Hue[0.5961847194378479, 1, 1], LineBox[{17, 18}]}, 
        {Hue[0.46862351167277533`, 1, 1], LineBox[{18, 19}]}, 
        {Hue[0.4458251915675756, 1, 1], LineBox[{19, 20}]}, 
        {Hue[0.46895428974704645`, 1, 1], LineBox[{20, 21}]}, 
        {Hue[0.5125418994642816, 1, 1], LineBox[{21, 22}]}, 
        {Hue[0.5989540200443739, 1, 1], LineBox[{22, 23}]}, 
        {Hue[0.7063579466621666, 1, 1], LineBox[{23, 24}]}, 
        {Hue[0.7990936586658762, 1, 1], LineBox[{24, 25}]}, 
        {Hue[0.8690664659236853, 1, 1], LineBox[{25, 26}]}, 
        {Hue[0.9327432316916345, 1, 1], LineBox[{26, 27}]}, 
        {Hue[0.004137892867792425, 1, 1], LineBox[{27, 28}]}, 
        {Hue[0.08342054374255543, 1, 1], LineBox[{28, 29}]}, 
        {Hue[0.16503786284242647`, 1, 1], LineBox[{29, 30}]}, 
        {Hue[0.2442067612292575, 1, 1], LineBox[{30, 31}]}, 
        {Hue[0.32216285879652434`, 1, 1], LineBox[{31, 32}]}, 
        {Hue[0.39923262219595296`, 1, 1], LineBox[{32, 33}]}, 
        {Hue[0.4780029083854207, 1, 1], LineBox[{33, 34}]}, 
        {Hue[0.5582453418924969, 1, 1], LineBox[{34, 35}]}, 
        {Hue[0.6387858577875118, 1, 1], LineBox[{35, 36}]}, 
        {Hue[0.7192316129758667, 1, 1], LineBox[{36, 37}]}, 
        {Hue[0.7996332928670785, 1, 1], LineBox[{37, 38}]}, 
        {Hue[0.8808137668476514, 1, 1], LineBox[{38, 39}]}, 
        {Hue[0.9626442596312538, 1, 1], LineBox[{39, 40}]}, 
        {Hue[0.044988617541437445`, 1, 1], LineBox[{40, 41}]}, 
        {Hue[0.12760340502421383`, 1, 1], LineBox[{41, 42}]}, 
        {Hue[0.21047848037237515`, 1, 1], LineBox[{42, 43}]}, 
        {Hue[0.29376911801795574`, 1, 1], LineBox[{43, 44}]}, 
        {Hue[0.3775536220592098, 1, 1], LineBox[{44, 45}]}, 
        {Hue[0.4618126702069244, 1, 1], LineBox[{45, 46}]}, 
        {Hue[0.5464592745186805, 1, 1], LineBox[{46, 47}]}, 
        {Hue[0.6315026619907433, 1, 1], LineBox[{47, 48}]}, 
        {Hue[0.7169267598757019, 1, 1], LineBox[{48, 49}]}, 
        {Hue[0.8027916833342827, 1, 1], LineBox[{49, 50}]}, 
        {Hue[0.889089585445414, 1, 1], LineBox[{50, 51}]}, 
        {Hue[0.9757950557057082, 1, 1], LineBox[{51, 52}]}, 
        {Hue[0.06291248955601675, 1, 1], LineBox[{52, 53}]}, 
        {Hue[0.15043625406158562`, 1, 1], LineBox[{53, 54}]}, 
        {Hue[0.23836997371235044`, 1, 1], LineBox[{54, 55}]}, 
        {Hue[0.32672315535883, 1, 1], LineBox[{55, 56}]}, 
        {Hue[0.4154914518436902, 1, 1], LineBox[{56, 57}]}, 
        {Hue[0.5046658300732096, 1, 1], LineBox[{57, 58}]}, 
        {Hue[0.5942503894406991, 1, 1], LineBox[{58, 59}]}, 
        {Hue[0.6842458102565003, 1, 1], LineBox[{59, 60}]}, 
        {Hue[0.7746521596919396, 1, 1], LineBox[{60, 61}]}, 
        {Hue[0.8654707714978663, 1, 1], LineBox[{61, 62}]}, 
        {Hue[0.9566988597906313, 1, 1], LineBox[{62, 63}]}, 
        {Hue[0.04833514536830779, 1, 1], LineBox[{63, 64}]}, 
        {Hue[0.14037917323177285`, 1, 1], LineBox[{64, 65}]}, 
        {Hue[0.23283177667960436`, 1, 1], LineBox[{65, 66}]}, 
        {Hue[0.32569164258084476`, 1, 1], LineBox[{66, 67}]}, 
        {Hue[0.41895898555335936`, 1, 1], LineBox[{67, 68}]}, 
        {Hue[0.5126322184725833, 1, 1], LineBox[{68, 69}]}, 
        {Hue[0.6067109627284879, 1, 1], LineBox[{69, 70}]}, 
        {Hue[0.7011944510053674, 1, 1], LineBox[{70, 71}]}, 
        {Hue[0.7960825734629512, 1, 1], LineBox[{71, 72}]}, 
        {Hue[0.8913750773674782, 1, 1], LineBox[{72, 73}]}, 
        {Hue[0.9870703085292256, 1, 1], LineBox[{73, 74}]}, 
        {Hue[0.0831682078717523, 1, 1], LineBox[{74, 75}]}, 
        {Hue[0.17966690754711842`, 1, 1], LineBox[{75, 76}]}, 
        {Hue[0.27656651526156967`, 1, 1], LineBox[{76, 77}]}, 
        {Hue[0.37386606936161204`, 1, 1], LineBox[{77, 78}]}, 
        {Hue[0.47156409420583095`, 1, 1], LineBox[{78, 79}]}, 
        {Hue[0.5696598302845104, 1, 1], LineBox[{79, 80}]}, 
        {Hue[0.6681520841585072, 1, 1], LineBox[{80, 81}]}, 
        {Hue[0.7670396100066637, 1, 1], LineBox[{81, 82}]}, 
        {Hue[0.8663210495276291, 1, 1], LineBox[{82, 83}]}, 
        {Hue[0.965994561204372, 1, 1], LineBox[{83, 84}]}, 
        {Hue[0.06605854132798218, 1, 1], LineBox[{84, 85}]}, 
        {Hue[0.1665109206955648, 1, 1], LineBox[{85, 86}]}, 
        {Hue[0.26735002732299895`, 1, 1], LineBox[{86, 87}]}, 
        {Hue[0.3685727399772961, 1, 1], LineBox[{87, 88}]}, 
        {Hue[0.4701766148566998, 1, 1], LineBox[{88, 89}]}, 
        {Hue[0.5721579648495672, 1, 1], LineBox[{89, 90}]}, 
        {Hue[0.6745138217897079, 1, 1], LineBox[{90, 91}]}, 
        {Hue[0.7772401123597215, 1, 1], LineBox[{91, 92}]}, 
        {Hue[0.8803315074975259, 1, 1], LineBox[{92, 93}]}, 
        {Hue[0.9837828931126193, 1, 1], LineBox[{93, 94}]}, 
        {Hue[0.08758813029091003, 1, 1], LineBox[{94, 95}]}, 
        {Hue[0.19173995962207716`, 1, 1], LineBox[{95, 96}]}, 
        {Hue[0.29623052831179936`, 1, 1], LineBox[{96, 97}]}, 
        {Hue[0.40104982353295887`, 1, 1], LineBox[{97, 98}]}, 
        {Hue[0.5061869922694149, 1, 1], LineBox[{98, 99}]}, 
        {Hue[0.6116299342344147, 1, 1], LineBox[{99, 100}]}, 
        {Hue[0.7173639634709316, 1, 1], LineBox[{100, 101}]}, 
        {Hue[0.8233718474500051, 1, 1], LineBox[{101, 102}]}, 
        {Hue[0.9296346319294032, 1, 1], LineBox[{102, 103}]}, 
        {Hue[0.03613037661478721, 1, 1], LineBox[{103, 104}]}, 
        {Hue[0.1428334271268505, 1, 1], LineBox[{104, 105}]}, 
        {Hue[0.24971412507695165`, 1, 1], LineBox[{105, 106}]}, 
        {Hue[0.356738931200211, 1, 1], LineBox[{106, 107}]}, 
        {Hue[0.4638696268399593, 1, 1], LineBox[{107, 108}]}, 
        {Hue[0.571063319566488, 1, 1], LineBox[{108, 109}]}, 
        {Hue[0.6782700343654368, 1, 1], LineBox[{109, 110}]}, 
        {Hue[0.785434006690692, 1, 1], LineBox[{110, 111}]}, 
        {Hue[0.8924930729167271, 1, 1], LineBox[{111, 112}]}, 
        {Hue[0.9993779783296385, 1, 1], LineBox[{112, 113}]}, 
        {Hue[0.10601149166352605`, 1, 1], LineBox[{113, 114}]}, 
        {Hue[0.2123087313751443, 1, 1], LineBox[{114, 115}]}, 
        {Hue[0.3181787875662695, 1, 1], LineBox[{115, 116}]}, 
        {Hue[0.4235247873915342, 1, 1], LineBox[{116, 117}]}, 
        {Hue[0.5282462551631992, 1, 1], LineBox[{117, 118}]}, 
        {Hue[0.6322395345155183, 1, 1], LineBox[{118, 119}]}, 
        {Hue[0.7354006500082138, 1, 1], LineBox[{119, 120}]}, 
        {Hue[0.8376300003692168, 1, 1], LineBox[{120, 121}]}, 
        {Hue[0.9388378605035128, 1, 1], LineBox[{121, 122}]}, 
        {Hue[0.038949607055956306`, 1, 1], LineBox[{122, 123}]}, 
        {Hue[0.13790895816925902`, 1, 1], LineBox[{123, 124}]}, 
        {Hue[0.23568267151472239`, 1, 1], LineBox[{124, 125}]}, 
        {Hue[0.3322654544287647, 1, 1], LineBox[{125, 126}]}, 
        {Hue[0.42768181401220956`, 1, 1], LineBox[{126, 127}]}, 
        {Hue[0.5219917613987968, 1, 1], LineBox[{127, 128}]}, 
        {Hue[0.6152841180730468, 1, 1], LineBox[{128, 129}]}, 
        {Hue[0.7076595650334099, 1, 1], LineBox[{129, 130}]}, 
        {Hue[0.7992366467292306, 1, 1], LineBox[{130, 131}]}, 
        {Hue[0.8901637845684595, 1, 1], LineBox[{131, 132}]}, 
        {Hue[0.980586996534017, 1, 1], LineBox[{132, 133}]}, 
        {Hue[0.0706304063174194, 1, 1], LineBox[{133, 134}]}, 
        {Hue[0.16044150686162284`, 1, 1], LineBox[{134, 135}]}, 
        {Hue[0.25018705154374354`, 1, 1], LineBox[{135, 136}]}, 
        {Hue[0.33995086535139046`, 1, 1], LineBox[{136, 137}]}, 
        {Hue[0.4297694192998456, 1, 1], LineBox[{137, 138}]}, 
        {Hue[0.5198289187276111, 1, 1], LineBox[{138, 139}]}, 
        {Hue[0.610354057984487, 1, 1], LineBox[{139, 140}]}, 
        {Hue[0.7012144543506329, 1, 1], LineBox[{140, 141}]}, 
        {Hue[0.7921081889537456, 1, 1], LineBox[{141, 142}]}, 
        {Hue[0.8833380798161025, 1, 1], LineBox[{142, 143}]}, 
        {Hue[0.975815306966425, 1, 1], LineBox[{143, 144}]}, 
        {Hue[0.0698586820590084, 1, 1], LineBox[{144, 145}]}, 
        {Hue[0.16445521240978872`, 1, 1], LineBox[{145, 146}]}, 
        {Hue[0.2582012082242161, 1, 1], LineBox[{146, 147}]}, 
        {Hue[0.35121116346775805`, 1, 1], LineBox[{147, 148}]}, 
        {Hue[0.44563606809176354`, 1, 1], LineBox[{148, 149}]}, 
        {Hue[0.5430933601796459, 1, 1], LineBox[{149, 150}]}, 
        {Hue[0.6423155915986974, 1, 1], LineBox[{150, 151}]}, 
        {Hue[0.7407113090471817, 1, 1], LineBox[{151, 152}]}, 
        {Hue[0.8376881189539714, 1, 1], LineBox[{152, 153}]}, 
        {Hue[0.9352232891503853, 1, 1], LineBox[{153, 154}]}, 
        {Hue[0.03614858908370498, 1, 1], LineBox[{154, 155}]}, 
        {Hue[0.14398074114436193`, 1, 1], LineBox[{155, 156}]}, 
        {Hue[0.25920378352449885`, 1, 1], LineBox[{156, 157}]}, 
        {Hue[0.3668844355750923, 1, 1], LineBox[{157, 158}]}, 
        {Hue[0.41640280589271683`, 1, 1], LineBox[{158, 159}]}, 
        {Hue[0.46331902891034815`, 1, 1], LineBox[{159, 160}]}, 
        {Hue[0.6033248517219278, 1, 1], LineBox[{160, 161}]}, 
        {Hue[0.7883325306388308, 1, 1], LineBox[{161, 162}]}, 
        {Hue[0.0056238243984809925`, 1, 1], LineBox[{162, 163}]}, 
        {Hue[0.2774077069850332, 1, 1], LineBox[{163, 164}]}, 
        {Hue[0.6155398081712723, 1, 1], LineBox[{164, 165}]}, 
        {Hue[0.9030578930956729, 1, 1], LineBox[{165, 166}]}, 
        {Hue[0.14414120650805495`, 1, 1], LineBox[{166, 167}]}, 
        {Hue[0.3701881994243456, 1, 1], LineBox[{167, 168}]}, 
        {Hue[0.5916882192261339, 1, 1], LineBox[{168, 169}]}, 
        {Hue[0.8128425253061538, 1, 1], LineBox[{169, 170}]}, 
        {Hue[0.03606014119192609, 1, 1], LineBox[{170, 171}]}, 
        {Hue[0.26223236189121907`, 1, 1], LineBox[{171, 172}]}, 
        {Hue[0.49054148919426266`, 1, 1], LineBox[{172, 173}]}, 
        {Hue[0.7203779998491203, 1, 1], LineBox[{173, 174}]}, 
        {Hue[0.9525369026809333, 1, 1], LineBox[{174, 175}]}, 
        {Hue[0.18843993116706134`, 1, 1], LineBox[{175, 176}]}, 
        {Hue[0.42932440021741863`, 1, 1], LineBox[{176, 177}]}, 
        {Hue[0.6757389699208721, 1, 1], LineBox[{177, 178}]}, 
        {Hue[0.9271433109474645, 1, 1], LineBox[{178, 179}]}, 
        {Hue[0.181969958793826, 1, 1], LineBox[{179, 180}]}, 
        {Hue[0.438446713394616, 1, 1], LineBox[{180, 181}]}, 
        {Hue[0.6955392300974138, 1, 1], LineBox[{181, 182}]}, 
        {Hue[0.9530958902650895, 1, 1], LineBox[{182, 183}]}, 
        {Hue[0.21123375064817898`, 1, 1], LineBox[{183, 184}]}, 
        {Hue[0.46967263406929427`, 1, 1], LineBox[{184, 185}]}, 
        {Hue[0.7276023629921933, 1, 1], LineBox[{185, 186}]}, 
        {Hue[0.984161950447084, 1, 1], LineBox[{186, 187}]}, 
        {Hue[0.2390623682607009, 1, 1], LineBox[{187, 188}]}, 
        {Hue[0.49283556399150985`, 1, 1], LineBox[{188, 189}]}, 
        {Hue[0.7466324493665115, 1, 1], LineBox[{189, 190}]}, 
        {Hue[0.0017163865090559496`, 1, 1], LineBox[{190, 191}]}, 
        {Hue[0.2586526882451303, 1, 1], LineBox[{191, 192}]}, 
        {Hue[0.5162546501565902, 1, 1], LineBox[{192, 193}]}, 
        {Hue[0.7710500352229288, 1, 1], LineBox[{193, 194}]}, 
        {Hue[0.018209373829782306`, 1, 1], LineBox[{194, 195}]}, 
        {Hue[0.25272791954391893`, 1, 1], LineBox[{195, 196}]}, 
        {Hue[0.46754293320048695`, 1, 1], LineBox[{196, 197}]}, 
        {Hue[0.6414161184083459, 1, 1], LineBox[{197, 198}]}, 
        {Hue[0.7116095092051098, 1, 1], LineBox[{198, 199}]}, 
        {Hue[0.7941755173099261, 1, 1], LineBox[{199, 200}]}, 
        {Hue[0.959677603338433, 1, 1], LineBox[{200, 201}]}, 
        {Hue[0.15416041543491266`, 1, 1], LineBox[{201, 202}]}, 
        {Hue[0.3659312105320584, 1, 1], LineBox[{202, 203}]}, 
        {Hue[0.5945187685231991, 1, 1], LineBox[{203, 204}]}, 
        {Hue[0.8435841217998448, 1, 1], LineBox[{204, 205}]}, 
        {Hue[0.11798106092260775`, 1, 1], LineBox[{205, 206}]}, 
        {Hue[0.40976957642296785`, 1, 1], LineBox[{206, 207}]}, 
        {Hue[0.6931950268848085, 1, 1], LineBox[{207, 208}]}, 
        {Hue[0.9572768482725862, 1, 1], LineBox[{208, 209}]}, 
        {Hue[0.20736297205040422`, 1, 1], LineBox[{209, 210}]}, 
        {Hue[0.4495073129009342, 1, 1], LineBox[{210, 211}]}, 
        {Hue[0.6873302371223096, 1, 1], LineBox[{211, 212}]}, 
        {Hue[0.922917256847574, 1, 1], LineBox[{212, 213}]}, 
        {Hue[0.15766446546505655`, 1, 1], LineBox[{213, 214}]}, 
        {Hue[0.39281267806319864`, 1, 1], LineBox[{214, 215}]}, 
        {Hue[0.6298954761469904, 1, 1], LineBox[{215, 216}]}, 
        {Hue[0.8713537747234714, 1, 1], LineBox[{216, 217}]}, 
        {Hue[0.12165943440260661`, 1, 1], LineBox[{217, 218}]}, 
        {Hue[0.3889776048863425, 1, 1], LineBox[{218, 219}]}, 
        {Hue[0.6821005554196526, 1, 1], LineBox[{219, 220}]}, 
        {Hue[0.9874793453943027, 1, 1], LineBox[{220, 221}]}, 
        {Hue[0.27333428148134026`, 1, 1], LineBox[{221, 222}]}, 
        {Hue[0.536210098334362, 1, 1], LineBox[{222, 223}]}, 
        {Hue[0.7860931382583406, 1, 1], LineBox[{223, 224}]}, 
        {Hue[0.029915025712982726`, 1, 1], LineBox[{224, 225}]}, 
        {Hue[0.27135597860730676`, 1, 1], LineBox[{225, 226}]}, 
        {Hue[0.5124025961869167, 1, 1], LineBox[{226, 227}]}, 
        {Hue[0.7542039770961328, 1, 1], LineBox[{227, 228}]}, 
        {Hue[0.9974861242682816, 1, 1], LineBox[{228, 229}]}, 
        {Hue[0.2427641987352073, 1, 1], LineBox[{229, 230}]}, 
        {Hue[0.49046522259802094`, 1, 1], LineBox[{230, 231}]}, 
        {Hue[0.7410080688609925, 1, 1], LineBox[{231, 232}]}, 
        {Hue[0.9948612689328367, 1, 1], LineBox[{232, 233}]}, 
        {Hue[0.25258201027232025`, 1, 1], LineBox[{233, 234}]}, 
        {Hue[0.5148223127241571, 1, 1], LineBox[{234, 235}]}, 
        {Hue[0.7822622604242208, 1, 1], LineBox[{235, 236}]}, 
        {Hue[0.05540122424524702, 1, 1], LineBox[{236, 237}]}, 
        {Hue[0.33416566373549533`, 1, 1], LineBox[{237, 238}]}, 
        {Hue[0.6174964318648639, 1, 1], LineBox[{238, 239}]}, 
        {Hue[0.9033933394447586, 1, 1], LineBox[{239, 240}]}, 
        {Hue[0.18972674509790277`, 1, 1], LineBox[{240, 241}]}, 
        {Hue[0.4752451532292422, 1, 1], LineBox[{241, 242}]}, 
        {Hue[0.7599383987575214, 1, 1], LineBox[{242, 243}]}, 
        {Hue[0.044725871382062, 1, 1], LineBox[{243, 244}]}, 
        {Hue[0.3309301529199232, 1, 1], LineBox[{244, 245}]}, 
        {Hue[0.619740435821263, 1, 1], LineBox[{245, 246}]}, 
        {Hue[0.9115976593850698, 1, 1], LineBox[{246, 247}]}, 
        {Hue[0.2055958773373284, 1, 1], LineBox[{247, 248}]}, 
        {Hue[0.49947597567191315`, 1, 1], LineBox[{248, 249}]}, 
        {Hue[0.79064522017179, 1, 1], LineBox[{249, 250}]}, 
        {Hue[0.0774344648203574, 1, 1], LineBox[{250, 251}]}}}},
     VertexColors->{
       Hue[0.1880703612269147, 1, 1], 
       Hue[0.07884389201651706, 1, 1], 
       Hue[0.997136729011347, 1, 1], 
       Hue[0.8689395434525204, 1, 1], 
       Hue[0.7062546158712821, 1, 1], 
       Hue[0.7388461845627924, 1, 1], 
       Hue[0.5855521737701119, 1, 1], 
       Hue[0.5339300205386036, 1, 1], 
       Hue[0.46682903323219344`, 1, 1], 
       Hue[0.3723331935375797, 1, 1], 
       Hue[0.21409818648276444`, 1, 1], 
       Hue[0.12489905314846222`, 1, 1], 
       Hue[0.03131991234497626, 1, 1], 
       Hue[0.0021162108163126875`, 1, 1], 
       Hue[0.9204190719936995, 1, 1], 
       Hue[0.8532338736598175, 1, 1], 
       Hue[0.5961847194378479, 1, 1], 
       Hue[0.46862351167277533`, 1, 1], 
       Hue[0.4458251915675756, 1, 1], 
       Hue[0.46895428974704645`, 1, 1], 
       Hue[0.5125418994642816, 1, 1], 
       Hue[0.5989540200443739, 1, 1], 
       Hue[0.7063579466621666, 1, 1], 
       Hue[0.7990936586658762, 1, 1], 
       Hue[0.8690664659236853, 1, 1], 
       Hue[0.9327432316916345, 1, 1], 
       Hue[0.004137892867792425, 1, 1], 
       Hue[0.08342054374255543, 1, 1], 
       Hue[0.16503786284242647`, 1, 1], 
       Hue[0.2442067612292575, 1, 1], 
       Hue[0.32216285879652434`, 1, 1], 
       Hue[0.39923262219595296`, 1, 1], 
       Hue[0.4780029083854207, 1, 1], 
       Hue[0.5582453418924969, 1, 1], 
       Hue[0.6387858577875118, 1, 1], 
       Hue[0.7192316129758667, 1, 1], 
       Hue[0.7996332928670785, 1, 1], 
       Hue[0.8808137668476514, 1, 1], 
       Hue[0.9626442596312538, 1, 1], 
       Hue[0.044988617541437445`, 1, 1], 
       Hue[0.12760340502421383`, 1, 1], 
       Hue[0.21047848037237515`, 1, 1], 
       Hue[0.29376911801795574`, 1, 1], 
       Hue[0.3775536220592098, 1, 1], 
       Hue[0.4618126702069244, 1, 1], 
       Hue[0.5464592745186805, 1, 1], 
       Hue[0.6315026619907433, 1, 1], 
       Hue[0.7169267598757019, 1, 1], 
       Hue[0.8027916833342827, 1, 1], 
       Hue[0.889089585445414, 1, 1], 
       Hue[0.9757950557057082, 1, 1], 
       Hue[0.06291248955601675, 1, 1], 
       Hue[0.15043625406158562`, 1, 1], 
       Hue[0.23836997371235044`, 1, 1], 
       Hue[0.32672315535883, 1, 1], 
       Hue[0.4154914518436902, 1, 1], 
       Hue[0.5046658300732096, 1, 1], 
       Hue[0.5942503894406991, 1, 1], 
       Hue[0.6842458102565003, 1, 1], 
       Hue[0.7746521596919396, 1, 1], 
       Hue[0.8654707714978663, 1, 1], 
       Hue[0.9566988597906313, 1, 1], 
       Hue[0.04833514536830779, 1, 1], 
       Hue[0.14037917323177285`, 1, 1], 
       Hue[0.23283177667960436`, 1, 1], 
       Hue[0.32569164258084476`, 1, 1], 
       Hue[0.41895898555335936`, 1, 1], 
       Hue[0.5126322184725833, 1, 1], 
       Hue[0.6067109627284879, 1, 1], 
       Hue[0.7011944510053674, 1, 1], 
       Hue[0.7960825734629512, 1, 1], 
       Hue[0.8913750773674782, 1, 1], 
       Hue[0.9870703085292256, 1, 1], 
       Hue[0.0831682078717523, 1, 1], 
       Hue[0.17966690754711842`, 1, 1], 
       Hue[0.27656651526156967`, 1, 1], 
       Hue[0.37386606936161204`, 1, 1], 
       Hue[0.47156409420583095`, 1, 1], 
       Hue[0.5696598302845104, 1, 1], 
       Hue[0.6681520841585072, 1, 1], 
       Hue[0.7670396100066637, 1, 1], 
       Hue[0.8663210495276291, 1, 1], 
       Hue[0.965994561204372, 1, 1], 
       Hue[0.06605854132798218, 1, 1], 
       Hue[0.1665109206955648, 1, 1], 
       Hue[0.26735002732299895`, 1, 1], 
       Hue[0.3685727399772961, 1, 1], 
       Hue[0.4701766148566998, 1, 1], 
       Hue[0.5721579648495672, 1, 1], 
       Hue[0.6745138217897079, 1, 1], 
       Hue[0.7772401123597215, 1, 1], 
       Hue[0.8803315074975259, 1, 1], 
       Hue[0.9837828931126193, 1, 1], 
       Hue[0.08758813029091003, 1, 1], 
       Hue[0.19173995962207716`, 1, 1], 
       Hue[0.29623052831179936`, 1, 1], 
       Hue[0.40104982353295887`, 1, 1], 
       Hue[0.5061869922694149, 1, 1], 
       Hue[0.6116299342344147, 1, 1], 
       Hue[0.7173639634709316, 1, 1], 
       Hue[0.8233718474500051, 1, 1], 
       Hue[0.9296346319294032, 1, 1], 
       Hue[0.03613037661478721, 1, 1], 
       Hue[0.1428334271268505, 1, 1], 
       Hue[0.24971412507695165`, 1, 1], 
       Hue[0.356738931200211, 1, 1], 
       Hue[0.4638696268399593, 1, 1], 
       Hue[0.571063319566488, 1, 1], 
       Hue[0.6782700343654368, 1, 1], 
       Hue[0.785434006690692, 1, 1], 
       Hue[0.8924930729167271, 1, 1], 
       Hue[0.9993779783296385, 1, 1], 
       Hue[0.10601149166352605`, 1, 1], 
       Hue[0.2123087313751443, 1, 1], 
       Hue[0.3181787875662695, 1, 1], 
       Hue[0.4235247873915342, 1, 1], 
       Hue[0.5282462551631992, 1, 1], 
       Hue[0.6322395345155183, 1, 1], 
       Hue[0.7354006500082138, 1, 1], 
       Hue[0.8376300003692168, 1, 1], 
       Hue[0.9388378605035128, 1, 1], 
       Hue[0.038949607055956306`, 1, 1], 
       Hue[0.13790895816925902`, 1, 1], 
       Hue[0.23568267151472239`, 1, 1], 
       Hue[0.3322654544287647, 1, 1], 
       Hue[0.42768181401220956`, 1, 1], 
       Hue[0.5219917613987968, 1, 1], 
       Hue[0.6152841180730468, 1, 1], 
       Hue[0.7076595650334099, 1, 1], 
       Hue[0.7992366467292306, 1, 1], 
       Hue[0.8901637845684595, 1, 1], 
       Hue[0.980586996534017, 1, 1], 
       Hue[0.0706304063174194, 1, 1], 
       Hue[0.16044150686162284`, 1, 1], 
       Hue[0.25018705154374354`, 1, 1], 
       Hue[0.33995086535139046`, 1, 1], 
       Hue[0.4297694192998456, 1, 1], 
       Hue[0.5198289187276111, 1, 1], 
       Hue[0.610354057984487, 1, 1], 
       Hue[0.7012144543506329, 1, 1], 
       Hue[0.7921081889537456, 1, 1], 
       Hue[0.8833380798161025, 1, 1], 
       Hue[0.975815306966425, 1, 1], 
       Hue[0.0698586820590084, 1, 1], 
       Hue[0.16445521240978872`, 1, 1], 
       Hue[0.2582012082242161, 1, 1], 
       Hue[0.35121116346775805`, 1, 1], 
       Hue[0.44563606809176354`, 1, 1], 
       Hue[0.5430933601796459, 1, 1], 
       Hue[0.6423155915986974, 1, 1], 
       Hue[0.7407113090471817, 1, 1], 
       Hue[0.8376881189539714, 1, 1], 
       Hue[0.9352232891503853, 1, 1], 
       Hue[0.03614858908370498, 1, 1], 
       Hue[0.14398074114436193`, 1, 1], 
       Hue[0.25920378352449885`, 1, 1], 
       Hue[0.3668844355750923, 1, 1], 
       Hue[0.41640280589271683`, 1, 1], 
       Hue[0.46331902891034815`, 1, 1], 
       Hue[0.6033248517219278, 1, 1], 
       Hue[0.7883325306388308, 1, 1], 
       Hue[0.0056238243984809925`, 1, 1], 
       Hue[0.2774077069850332, 1, 1], 
       Hue[0.6155398081712723, 1, 1], 
       Hue[0.9030578930956729, 1, 1], 
       Hue[0.14414120650805495`, 1, 1], 
       Hue[0.3701881994243456, 1, 1], 
       Hue[0.5916882192261339, 1, 1], 
       Hue[0.8128425253061538, 1, 1], 
       Hue[0.03606014119192609, 1, 1], 
       Hue[0.26223236189121907`, 1, 1], 
       Hue[0.49054148919426266`, 1, 1], 
       Hue[0.7203779998491203, 1, 1], 
       Hue[0.9525369026809333, 1, 1], 
       Hue[0.18843993116706134`, 1, 1], 
       Hue[0.42932440021741863`, 1, 1], 
       Hue[0.6757389699208721, 1, 1], 
       Hue[0.9271433109474645, 1, 1], 
       Hue[0.181969958793826, 1, 1], 
       Hue[0.438446713394616, 1, 1], 
       Hue[0.6955392300974138, 1, 1], 
       Hue[0.9530958902650895, 1, 1], 
       Hue[0.21123375064817898`, 1, 1], 
       Hue[0.46967263406929427`, 1, 1], 
       Hue[0.7276023629921933, 1, 1], 
       Hue[0.984161950447084, 1, 1], 
       Hue[0.2390623682607009, 1, 1], 
       Hue[0.49283556399150985`, 1, 1], 
       Hue[0.7466324493665115, 1, 1], 
       Hue[0.0017163865090559496`, 1, 1], 
       Hue[0.2586526882451303, 1, 1], 
       Hue[0.5162546501565902, 1, 1], 
       Hue[0.7710500352229288, 1, 1], 
       Hue[0.018209373829782306`, 1, 1], 
       Hue[0.25272791954391893`, 1, 1], 
       Hue[0.46754293320048695`, 1, 1], 
       Hue[0.6414161184083459, 1, 1], 
       Hue[0.7116095092051098, 1, 1], 
       Hue[0.7941755173099261, 1, 1], 
       Hue[0.959677603338433, 1, 1], 
       Hue[0.15416041543491266`, 1, 1], 
       Hue[0.3659312105320584, 1, 1], 
       Hue[0.5945187685231991, 1, 1], 
       Hue[0.8435841217998448, 1, 1], 
       Hue[0.11798106092260775`, 1, 1], 
       Hue[0.40976957642296785`, 1, 1], 
       Hue[0.6931950268848085, 1, 1], 
       Hue[0.9572768482725862, 1, 1], 
       Hue[0.20736297205040422`, 1, 1], 
       Hue[0.4495073129009342, 1, 1], 
       Hue[0.6873302371223096, 1, 1], 
       Hue[0.922917256847574, 1, 1], 
       Hue[0.15766446546505655`, 1, 1], 
       Hue[0.39281267806319864`, 1, 1], 
       Hue[0.6298954761469904, 1, 1], 
       Hue[0.8713537747234714, 1, 1], 
       Hue[0.12165943440260661`, 1, 1], 
       Hue[0.3889776048863425, 1, 1], 
       Hue[0.6821005554196526, 1, 1], 
       Hue[0.9874793453943027, 1, 1], 
       Hue[0.27333428148134026`, 1, 1], 
       Hue[0.536210098334362, 1, 1], 
       Hue[0.7860931382583406, 1, 1], 
       Hue[0.029915025712982726`, 1, 1], 
       Hue[0.27135597860730676`, 1, 1], 
       Hue[0.5124025961869167, 1, 1], 
       Hue[0.7542039770961328, 1, 1], 
       Hue[0.9974861242682816, 1, 1], 
       Hue[0.2427641987352073, 1, 1], 
       Hue[0.49046522259802094`, 1, 1], 
       Hue[0.7410080688609925, 1, 1], 
       Hue[0.9948612689328367, 1, 1], 
       Hue[0.25258201027232025`, 1, 1], 
       Hue[0.5148223127241571, 1, 1], 
       Hue[0.7822622604242208, 1, 1], 
       Hue[0.05540122424524702, 1, 1], 
       Hue[0.33416566373549533`, 1, 1], 
       Hue[0.6174964318648639, 1, 1], 
       Hue[0.9033933394447586, 1, 1], 
       Hue[0.18972674509790277`, 1, 1], 
       Hue[0.4752451532292422, 1, 1], 
       Hue[0.7599383987575214, 1, 1], 
       Hue[0.044725871382062, 1, 1], 
       Hue[0.3309301529199232, 1, 1], 
       Hue[0.619740435821263, 1, 1], 
       Hue[0.9115976593850698, 1, 1], 
       Hue[0.2055958773373284, 1, 1], 
       Hue[0.49947597567191315`, 1, 1], 
       Hue[0.79064522017179, 1, 1], 
       Hue[0.0774344648203574, 1, 1], 
       Hue[0.359464892025414, 1, 1], 
       Hue[0.1880703612269147, 1, 1], 
       Hue[0.07884389201651706, 1, 1], 
       Hue[0.07884389201651706, 1, 1], 
       Hue[0.997136729011347, 1, 1], 
       Hue[0.997136729011347, 1, 1], 
       Hue[0.8689395434525204, 1, 1], 
       Hue[0.8689395434525204, 1, 1], 
       Hue[0.7062546158712821, 1, 1], 
       Hue[0.7062546158712821, 1, 1], 
       Hue[0.7388461845627924, 1, 1], 
       Hue[0.7388461845627924, 1, 1], 
       Hue[0.5855521737701119, 1, 1], 
       Hue[0.5855521737701119, 1, 1], 
       Hue[0.5339300205386036, 1, 1], 
       Hue[0.5339300205386036, 1, 1], 
       Hue[0.46682903323219344`, 1, 1], 
       Hue[0.46682903323219344`, 1, 1], 
       Hue[0.3723331935375797, 1, 1], 
       Hue[0.3723331935375797, 1, 1], 
       Hue[0.21409818648276444`, 1, 1], 
       Hue[0.21409818648276444`, 1, 1], 
       Hue[0.12489905314846222`, 1, 1], 
       Hue[0.12489905314846222`, 1, 1], 
       Hue[0.03131991234497626, 1, 1], 
       Hue[0.03131991234497626, 1, 1], 
       Hue[0.0021162108163126875`, 1, 1], 
       Hue[0.0021162108163126875`, 1, 1], 
       Hue[0.9204190719936995, 1, 1], 
       Hue[0.9204190719936995, 1, 1], 
       Hue[0.8532338736598175, 1, 1], 
       Hue[0.8532338736598175, 1, 1], 
       Hue[0.5961847194378479, 1, 1], 
       Hue[0.5961847194378479, 1, 1], 
       Hue[0.46862351167277533`, 1, 1], 
       Hue[0.46862351167277533`, 1, 1], 
       Hue[0.4458251915675756, 1, 1], 
       Hue[0.4458251915675756, 1, 1], 
       Hue[0.46895428974704645`, 1, 1], 
       Hue[0.46895428974704645`, 1, 1], 
       Hue[0.5125418994642816, 1, 1], 
       Hue[0.5125418994642816, 1, 1], 
       Hue[0.5989540200443739, 1, 1], 
       Hue[0.5989540200443739, 1, 1], 
       Hue[0.7063579466621666, 1, 1], 
       Hue[0.7063579466621666, 1, 1], 
       Hue[0.7990936586658762, 1, 1], 
       Hue[0.7990936586658762, 1, 1], 
       Hue[0.8690664659236853, 1, 1], 
       Hue[0.8690664659236853, 1, 1], 
       Hue[0.9327432316916345, 1, 1], 
       Hue[0.9327432316916345, 1, 1], 
       Hue[0.004137892867792425, 1, 1], 
       Hue[0.004137892867792425, 1, 1], 
       Hue[0.08342054374255543, 1, 1], 
       Hue[0.08342054374255543, 1, 1], 
       Hue[0.16503786284242647`, 1, 1], 
       Hue[0.16503786284242647`, 1, 1], 
       Hue[0.2442067612292575, 1, 1], 
       Hue[0.2442067612292575, 1, 1], 
       Hue[0.32216285879652434`, 1, 1], 
       Hue[0.32216285879652434`, 1, 1], 
       Hue[0.39923262219595296`, 1, 1], 
       Hue[0.39923262219595296`, 1, 1], 
       Hue[0.4780029083854207, 1, 1], 
       Hue[0.4780029083854207, 1, 1], 
       Hue[0.5582453418924969, 1, 1], 
       Hue[0.5582453418924969, 1, 1], 
       Hue[0.6387858577875118, 1, 1], 
       Hue[0.6387858577875118, 1, 1], 
       Hue[0.7192316129758667, 1, 1], 
       Hue[0.7192316129758667, 1, 1], 
       Hue[0.7996332928670785, 1, 1], 
       Hue[0.7996332928670785, 1, 1], 
       Hue[0.8808137668476514, 1, 1], 
       Hue[0.8808137668476514, 1, 1], 
       Hue[0.9626442596312538, 1, 1], 
       Hue[0.9626442596312538, 1, 1], 
       Hue[0.044988617541437445`, 1, 1], 
       Hue[0.044988617541437445`, 1, 1], 
       Hue[0.12760340502421383`, 1, 1], 
       Hue[0.12760340502421383`, 1, 1], 
       Hue[0.21047848037237515`, 1, 1], 
       Hue[0.21047848037237515`, 1, 1], 
       Hue[0.29376911801795574`, 1, 1], 
       Hue[0.29376911801795574`, 1, 1], 
       Hue[0.3775536220592098, 1, 1], 
       Hue[0.3775536220592098, 1, 1], 
       Hue[0.4618126702069244, 1, 1], 
       Hue[0.4618126702069244, 1, 1], 
       Hue[0.5464592745186805, 1, 1], 
       Hue[0.5464592745186805, 1, 1], 
       Hue[0.6315026619907433, 1, 1], 
       Hue[0.6315026619907433, 1, 1], 
       Hue[0.7169267598757019, 1, 1], 
       Hue[0.7169267598757019, 1, 1], 
       Hue[0.8027916833342827, 1, 1], 
       Hue[0.8027916833342827, 1, 1], 
       Hue[0.889089585445414, 1, 1], 
       Hue[0.889089585445414, 1, 1], 
       Hue[0.9757950557057082, 1, 1], 
       Hue[0.9757950557057082, 1, 1], 
       Hue[0.06291248955601675, 1, 1], 
       Hue[0.06291248955601675, 1, 1], 
       Hue[0.15043625406158562`, 1, 1], 
       Hue[0.15043625406158562`, 1, 1], 
       Hue[0.23836997371235044`, 1, 1], 
       Hue[0.23836997371235044`, 1, 1], 
       Hue[0.32672315535883, 1, 1], 
       Hue[0.32672315535883, 1, 1], 
       Hue[0.4154914518436902, 1, 1], 
       Hue[0.4154914518436902, 1, 1], 
       Hue[0.5046658300732096, 1, 1], 
       Hue[0.5046658300732096, 1, 1], 
       Hue[0.5942503894406991, 1, 1], 
       Hue[0.5942503894406991, 1, 1], 
       Hue[0.6842458102565003, 1, 1], 
       Hue[0.6842458102565003, 1, 1], 
       Hue[0.7746521596919396, 1, 1], 
       Hue[0.7746521596919396, 1, 1], 
       Hue[0.8654707714978663, 1, 1], 
       Hue[0.8654707714978663, 1, 1], 
       Hue[0.9566988597906313, 1, 1], 
       Hue[0.9566988597906313, 1, 1], 
       Hue[0.04833514536830779, 1, 1], 
       Hue[0.04833514536830779, 1, 1], 
       Hue[0.14037917323177285`, 1, 1], 
       Hue[0.14037917323177285`, 1, 1], 
       Hue[0.23283177667960436`, 1, 1], 
       Hue[0.23283177667960436`, 1, 1], 
       Hue[0.32569164258084476`, 1, 1], 
       Hue[0.32569164258084476`, 1, 1], 
       Hue[0.41895898555335936`, 1, 1], 
       Hue[0.41895898555335936`, 1, 1], 
       Hue[0.5126322184725833, 1, 1], 
       Hue[0.5126322184725833, 1, 1], 
       Hue[0.6067109627284879, 1, 1], 
       Hue[0.6067109627284879, 1, 1], 
       Hue[0.7011944510053674, 1, 1], 
       Hue[0.7011944510053674, 1, 1], 
       Hue[0.7960825734629512, 1, 1], 
       Hue[0.7960825734629512, 1, 1], 
       Hue[0.8913750773674782, 1, 1], 
       Hue[0.8913750773674782, 1, 1], 
       Hue[0.9870703085292256, 1, 1], 
       Hue[0.9870703085292256, 1, 1], 
       Hue[0.0831682078717523, 1, 1], 
       Hue[0.0831682078717523, 1, 1], 
       Hue[0.17966690754711842`, 1, 1], 
       Hue[0.17966690754711842`, 1, 1], 
       Hue[0.27656651526156967`, 1, 1], 
       Hue[0.27656651526156967`, 1, 1], 
       Hue[0.37386606936161204`, 1, 1], 
       Hue[0.37386606936161204`, 1, 1], 
       Hue[0.47156409420583095`, 1, 1], 
       Hue[0.47156409420583095`, 1, 1], 
       Hue[0.5696598302845104, 1, 1], 
       Hue[0.5696598302845104, 1, 1], 
       Hue[0.6681520841585072, 1, 1], 
       Hue[0.6681520841585072, 1, 1], 
       Hue[0.7670396100066637, 1, 1], 
       Hue[0.7670396100066637, 1, 1], 
       Hue[0.8663210495276291, 1, 1], 
       Hue[0.8663210495276291, 1, 1], 
       Hue[0.965994561204372, 1, 1], 
       Hue[0.965994561204372, 1, 1], 
       Hue[0.06605854132798218, 1, 1], 
       Hue[0.06605854132798218, 1, 1], 
       Hue[0.1665109206955648, 1, 1], 
       Hue[0.1665109206955648, 1, 1], 
       Hue[0.26735002732299895`, 1, 1], 
       Hue[0.26735002732299895`, 1, 1], 
       Hue[0.3685727399772961, 1, 1], 
       Hue[0.3685727399772961, 1, 1], 
       Hue[0.4701766148566998, 1, 1], 
       Hue[0.4701766148566998, 1, 1], 
       Hue[0.5721579648495672, 1, 1], 
       Hue[0.5721579648495672, 1, 1], 
       Hue[0.6745138217897079, 1, 1], 
       Hue[0.6745138217897079, 1, 1], 
       Hue[0.7772401123597215, 1, 1], 
       Hue[0.7772401123597215, 1, 1], 
       Hue[0.8803315074975259, 1, 1], 
       Hue[0.8803315074975259, 1, 1], 
       Hue[0.9837828931126193, 1, 1], 
       Hue[0.9837828931126193, 1, 1], 
       Hue[0.08758813029091003, 1, 1], 
       Hue[0.08758813029091003, 1, 1], 
       Hue[0.19173995962207716`, 1, 1], 
       Hue[0.19173995962207716`, 1, 1], 
       Hue[0.29623052831179936`, 1, 1], 
       Hue[0.29623052831179936`, 1, 1], 
       Hue[0.40104982353295887`, 1, 1], 
       Hue[0.40104982353295887`, 1, 1], 
       Hue[0.5061869922694149, 1, 1], 
       Hue[0.5061869922694149, 1, 1], 
       Hue[0.6116299342344147, 1, 1], 
       Hue[0.6116299342344147, 1, 1], 
       Hue[0.7173639634709316, 1, 1], 
       Hue[0.7173639634709316, 1, 1], 
       Hue[0.8233718474500051, 1, 1], 
       Hue[0.8233718474500051, 1, 1], 
       Hue[0.9296346319294032, 1, 1], 
       Hue[0.9296346319294032, 1, 1], 
       Hue[0.03613037661478721, 1, 1], 
       Hue[0.03613037661478721, 1, 1], 
       Hue[0.1428334271268505, 1, 1], 
       Hue[0.1428334271268505, 1, 1], 
       Hue[0.24971412507695165`, 1, 1], 
       Hue[0.24971412507695165`, 1, 1], 
       Hue[0.356738931200211, 1, 1], 
       Hue[0.356738931200211, 1, 1], 
       Hue[0.4638696268399593, 1, 1], 
       Hue[0.4638696268399593, 1, 1], 
       Hue[0.571063319566488, 1, 1], 
       Hue[0.571063319566488, 1, 1], 
       Hue[0.6782700343654368, 1, 1], 
       Hue[0.6782700343654368, 1, 1], 
       Hue[0.785434006690692, 1, 1], 
       Hue[0.785434006690692, 1, 1], 
       Hue[0.8924930729167271, 1, 1], 
       Hue[0.8924930729167271, 1, 1], 
       Hue[0.9993779783296385, 1, 1], 
       Hue[0.9993779783296385, 1, 1], 
       Hue[0.10601149166352605`, 1, 1], 
       Hue[0.10601149166352605`, 1, 1], 
       Hue[0.2123087313751443, 1, 1], 
       Hue[0.2123087313751443, 1, 1], 
       Hue[0.3181787875662695, 1, 1], 
       Hue[0.3181787875662695, 1, 1], 
       Hue[0.4235247873915342, 1, 1], 
       Hue[0.4235247873915342, 1, 1], 
       Hue[0.5282462551631992, 1, 1], 
       Hue[0.5282462551631992, 1, 1], 
       Hue[0.6322395345155183, 1, 1], 
       Hue[0.6322395345155183, 1, 1], 
       Hue[0.7354006500082138, 1, 1], 
       Hue[0.7354006500082138, 1, 1], 
       Hue[0.8376300003692168, 1, 1], 
       Hue[0.8376300003692168, 1, 1], 
       Hue[0.9388378605035128, 1, 1], 
       Hue[0.9388378605035128, 1, 1], 
       Hue[0.038949607055956306`, 1, 1], 
       Hue[0.038949607055956306`, 1, 1], 
       Hue[0.13790895816925902`, 1, 1], 
       Hue[0.13790895816925902`, 1, 1], 
       Hue[0.23568267151472239`, 1, 1], 
       Hue[0.23568267151472239`, 1, 1], 
       Hue[0.3322654544287647, 1, 1], 
       Hue[0.3322654544287647, 1, 1], 
       Hue[0.42768181401220956`, 1, 1], 
       Hue[0.42768181401220956`, 1, 1], 
       Hue[0.5219917613987968, 1, 1], 
       Hue[0.5219917613987968, 1, 1], 
       Hue[0.6152841180730468, 1, 1], 
       Hue[0.6152841180730468, 1, 1], 
       Hue[0.7076595650334099, 1, 1], 
       Hue[0.7076595650334099, 1, 1], 
       Hue[0.7992366467292306, 1, 1], 
       Hue[0.7992366467292306, 1, 1], 
       Hue[0.8901637845684595, 1, 1], 
       Hue[0.8901637845684595, 1, 1], 
       Hue[0.980586996534017, 1, 1], 
       Hue[0.980586996534017, 1, 1], 
       Hue[0.0706304063174194, 1, 1], 
       Hue[0.0706304063174194, 1, 1], 
       Hue[0.16044150686162284`, 1, 1], 
       Hue[0.16044150686162284`, 1, 1], 
       Hue[0.25018705154374354`, 1, 1], 
       Hue[0.25018705154374354`, 1, 1], 
       Hue[0.33995086535139046`, 1, 1], 
       Hue[0.33995086535139046`, 1, 1], 
       Hue[0.4297694192998456, 1, 1], 
       Hue[0.4297694192998456, 1, 1], 
       Hue[0.5198289187276111, 1, 1], 
       Hue[0.5198289187276111, 1, 1], 
       Hue[0.610354057984487, 1, 1], 
       Hue[0.610354057984487, 1, 1], 
       Hue[0.7012144543506329, 1, 1], 
       Hue[0.7012144543506329, 1, 1], 
       Hue[0.7921081889537456, 1, 1], 
       Hue[0.7921081889537456, 1, 1], 
       Hue[0.8833380798161025, 1, 1], 
       Hue[0.8833380798161025, 1, 1], 
       Hue[0.975815306966425, 1, 1], 
       Hue[0.975815306966425, 1, 1], 
       Hue[0.0698586820590084, 1, 1], 
       Hue[0.0698586820590084, 1, 1], 
       Hue[0.16445521240978872`, 1, 1], 
       Hue[0.16445521240978872`, 1, 1], 
       Hue[0.2582012082242161, 1, 1], 
       Hue[0.2582012082242161, 1, 1], 
       Hue[0.35121116346775805`, 1, 1], 
       Hue[0.35121116346775805`, 1, 1], 
       Hue[0.44563606809176354`, 1, 1], 
       Hue[0.44563606809176354`, 1, 1], 
       Hue[0.5430933601796459, 1, 1], 
       Hue[0.5430933601796459, 1, 1], 
       Hue[0.6423155915986974, 1, 1], 
       Hue[0.6423155915986974, 1, 1], 
       Hue[0.7407113090471817, 1, 1], 
       Hue[0.7407113090471817, 1, 1], 
       Hue[0.8376881189539714, 1, 1], 
       Hue[0.8376881189539714, 1, 1], 
       Hue[0.9352232891503853, 1, 1], 
       Hue[0.9352232891503853, 1, 1], 
       Hue[0.03614858908370498, 1, 1], 
       Hue[0.03614858908370498, 1, 1], 
       Hue[0.14398074114436193`, 1, 1], 
       Hue[0.14398074114436193`, 1, 1], 
       Hue[0.25920378352449885`, 1, 1], 
       Hue[0.25920378352449885`, 1, 1], 
       Hue[0.3668844355750923, 1, 1], 
       Hue[0.3668844355750923, 1, 1], 
       Hue[0.41640280589271683`, 1, 1], 
       Hue[0.41640280589271683`, 1, 1], 
       Hue[0.46331902891034815`, 1, 1], 
       Hue[0.46331902891034815`, 1, 1], 
       Hue[0.6033248517219278, 1, 1], 
       Hue[0.6033248517219278, 1, 1], 
       Hue[0.7883325306388308, 1, 1], 
       Hue[0.7883325306388308, 1, 1], 
       Hue[0.0056238243984809925`, 1, 1], 
       Hue[0.0056238243984809925`, 1, 1], 
       Hue[0.2774077069850332, 1, 1], 
       Hue[0.2774077069850332, 1, 1], 
       Hue[0.6155398081712723, 1, 1], 
       Hue[0.6155398081712723, 1, 1], 
       Hue[0.9030578930956729, 1, 1], 
       Hue[0.9030578930956729, 1, 1], 
       Hue[0.14414120650805495`, 1, 1], 
       Hue[0.14414120650805495`, 1, 1], 
       Hue[0.3701881994243456, 1, 1], 
       Hue[0.3701881994243456, 1, 1], 
       Hue[0.5916882192261339, 1, 1], 
       Hue[0.5916882192261339, 1, 1], 
       Hue[0.8128425253061538, 1, 1], 
       Hue[0.8128425253061538, 1, 1], 
       Hue[0.03606014119192609, 1, 1], 
       Hue[0.03606014119192609, 1, 1], 
       Hue[0.26223236189121907`, 1, 1], 
       Hue[0.26223236189121907`, 1, 1], 
       Hue[0.49054148919426266`, 1, 1], 
       Hue[0.49054148919426266`, 1, 1], 
       Hue[0.7203779998491203, 1, 1], 
       Hue[0.7203779998491203, 1, 1], 
       Hue[0.9525369026809333, 1, 1], 
       Hue[0.9525369026809333, 1, 1], 
       Hue[0.18843993116706134`, 1, 1], 
       Hue[0.18843993116706134`, 1, 1], 
       Hue[0.42932440021741863`, 1, 1], 
       Hue[0.42932440021741863`, 1, 1], 
       Hue[0.6757389699208721, 1, 1], 
       Hue[0.6757389699208721, 1, 1], 
       Hue[0.9271433109474645, 1, 1], 
       Hue[0.9271433109474645, 1, 1], 
       Hue[0.181969958793826, 1, 1], 
       Hue[0.181969958793826, 1, 1], 
       Hue[0.438446713394616, 1, 1], 
       Hue[0.438446713394616, 1, 1], 
       Hue[0.6955392300974138, 1, 1], 
       Hue[0.6955392300974138, 1, 1], 
       Hue[0.9530958902650895, 1, 1], 
       Hue[0.9530958902650895, 1, 1], 
       Hue[0.21123375064817898`, 1, 1], 
       Hue[0.21123375064817898`, 1, 1], 
       Hue[0.46967263406929427`, 1, 1], 
       Hue[0.46967263406929427`, 1, 1], 
       Hue[0.7276023629921933, 1, 1], 
       Hue[0.7276023629921933, 1, 1], 
       Hue[0.984161950447084, 1, 1], 
       Hue[0.984161950447084, 1, 1], 
       Hue[0.2390623682607009, 1, 1], 
       Hue[0.2390623682607009, 1, 1], 
       Hue[0.49283556399150985`, 1, 1], 
       Hue[0.49283556399150985`, 1, 1], 
       Hue[0.7466324493665115, 1, 1], 
       Hue[0.7466324493665115, 1, 1], 
       Hue[0.0017163865090559496`, 1, 1], 
       Hue[0.0017163865090559496`, 1, 1], 
       Hue[0.2586526882451303, 1, 1], 
       Hue[0.2586526882451303, 1, 1], 
       Hue[0.5162546501565902, 1, 1], 
       Hue[0.5162546501565902, 1, 1], 
       Hue[0.7710500352229288, 1, 1], 
       Hue[0.7710500352229288, 1, 1], 
       Hue[0.018209373829782306`, 1, 1], 
       Hue[0.018209373829782306`, 1, 1], 
       Hue[0.25272791954391893`, 1, 1], 
       Hue[0.25272791954391893`, 1, 1], 
       Hue[0.46754293320048695`, 1, 1], 
       Hue[0.46754293320048695`, 1, 1], 
       Hue[0.6414161184083459, 1, 1], 
       Hue[0.6414161184083459, 1, 1], 
       Hue[0.7116095092051098, 1, 1], 
       Hue[0.7116095092051098, 1, 1], 
       Hue[0.7941755173099261, 1, 1], 
       Hue[0.7941755173099261, 1, 1], 
       Hue[0.959677603338433, 1, 1], 
       Hue[0.959677603338433, 1, 1], 
       Hue[0.15416041543491266`, 1, 1], 
       Hue[0.15416041543491266`, 1, 1], 
       Hue[0.3659312105320584, 1, 1], 
       Hue[0.3659312105320584, 1, 1], 
       Hue[0.5945187685231991, 1, 1], 
       Hue[0.5945187685231991, 1, 1], 
       Hue[0.8435841217998448, 1, 1], 
       Hue[0.8435841217998448, 1, 1], 
       Hue[0.11798106092260775`, 1, 1], 
       Hue[0.11798106092260775`, 1, 1], 
       Hue[0.40976957642296785`, 1, 1], 
       Hue[0.40976957642296785`, 1, 1], 
       Hue[0.6931950268848085, 1, 1], 
       Hue[0.6931950268848085, 1, 1], 
       Hue[0.9572768482725862, 1, 1], 
       Hue[0.9572768482725862, 1, 1], 
       Hue[0.20736297205040422`, 1, 1], 
       Hue[0.20736297205040422`, 1, 1], 
       Hue[0.4495073129009342, 1, 1], 
       Hue[0.4495073129009342, 1, 1], 
       Hue[0.6873302371223096, 1, 1], 
       Hue[0.6873302371223096, 1, 1], 
       Hue[0.922917256847574, 1, 1], 
       Hue[0.922917256847574, 1, 1]}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->{True, None},
    AxesOrigin->{0, 0},
    Epilog->
     LineBox[{{1, 7.414327510823197*^-7}, {2, 7.726648117048535*^-7}, {
       3, 8.227827566517121*^-7}, {4, 1.093774842111724*^-6}, {
       5, 6.92663342057923*^-7}, {6, 9.029940042057901*^-7}, {
       7, 1.0029934330924888`*^-6}, {8, 9.594195588106386*^-7}, {
       9, 1.0271893547518258`*^-6}, {10, 1.553112599900039*^-6}, {
       11, 1.4122340889127595`*^-6}, {12, 1.6304166533229733`*^-6}, {
       13, 1.6350909292463403`*^-6}, {14, 2.2090677784137976`*^-6}, {
       15, 2.2962723623266234`*^-6}, {16, 9.707863278121864*^-7}, {
       17, 8.020548870281221*^-7}, {18, 2.822056189897182*^-6}, {
       19, 4.6869507720681674`*^-6}, {20, 6.096744079244237*^-6}, {
       21, 7.063427491395416*^-6}, {22, 7.794429600735892*^-6}, {
       23, 0.000010000334409831495`}, {24, 0.000016079909290535753`}, {
       25, 0.000024694801992314865`}, {26, 0.00003455279454292162}, {
       27, 0.000045972828071435314`}, {28, 0.000060129516385812365`}, {
       29, 0.00008244508585235948}, {30, 0.00011348064676914458`}, {
       31, 0.00015421730440595075`}, {32, 0.00020666137672112157`}, {
       33, 0.0002738907954709313}, {34, 0.00036249393886701866`}, {
       35, 0.0004795055246468662}, {36, 0.0006319610183347425}, {
       37, 0.0008262340614665134}, {38, 0.001072238789977809}, {
       39, 0.0013856598500324628`}, {40, 0.0017828545468639974`}, {
       41, 0.0022847224707939176`}, {42, 0.0029112548477058732`}, {
       43, 0.0036885158297529684`}, {44, 0.0046471797210618495`}, {
       45, 0.005826297023032688}, {46, 0.007269174042799757}, {
       47, 0.00902337255522693}, {48, 0.011139495212259065`}, {
       49, 0.013680367241996328`}, {50, 0.01671405533605794}, {
       51, 0.0203167530574952}, {52, 0.02456848322042616}, {
       53, 0.02955560705012436}, {54, 0.03536918925939972}, {
       55, 0.0421061246410018}, {56, 0.04986818001732029}, {
       57, 0.05875546153974489}, {58, 0.06886598944801064}, {
       59, 0.08029625234723621}, {60, 0.09313682382931113}, {
       61, 0.10747049427939853`}, {62, 0.12336581515893144`}, {
       63, 0.140877328810592}, {64, 0.1600380256187275}, {
       65, 0.18085763736287935`}, {66, 0.20332458653902022`}, {
       67, 0.22739331558537393`}, {68, 0.2529884165191143}, {
       69, 0.280001793265108}, {70, 0.30828551653997743`}, {
       71, 0.33765916254082884`}, {72, 0.36790720370645474`}, {
       73, 0.39877914143774007`}, {74, 0.42999236379254935`}, {
       75, 0.4612356746723151}, {76, 0.49217361421945327`}, {
       77, 0.5224524023023419}, {78, 0.5517088725374619}, {
       79, 0.5795702050496168}, {80, 0.6056696818733874}, {
       81, 0.6296501632222378}, {82, 0.6511746336594946}, {
       83, 0.669930757354472}, {84, 0.6856399305512036}, {
       85, 0.6980665916937486}, {86, 0.7070203315403472}, {
       87, 0.7123637287383301}, {88, 0.7140158929677963}, {
       89, 0.7119476364974316}, {90, 0.7061922836568995}, {
       91, 0.6968398384595758}, {92, 0.6840349839321099}, {
       93, 0.6679709575650906}, {94, 0.6488889052524311}, {
       95, 0.6270693205699949}, {96, 0.6028275804551093}, {
       97, 0.5765001309514965}, {98, 0.548444547215035}, {
       99, 0.5190284406133917}, {100, 0.48861740532525}, {
       101, 0.4575741128314588}, {102, 0.42624811702523574`}, {
       103, 0.39496988820163076`}, {104, 0.3640457908144311}, {
       105, 0.33375305133422806`}, {106, 0.30433775400326585`}, {
       107, 0.2760132868026079}, {108, 0.2489571357334717}, {
       109, 0.22331019272275382`}, {110, 0.1991815252064953}, {
       111, 0.17664572645663257`}, {112, 0.15574708320501904`}, {
       113, 0.13650094059901738`}, {114, 0.11889856500892278`}, {
       115, 0.1029094474280986}, {116, 0.08848510494641651}, {
       117, 0.07556141916118712}, {118, 0.06406190154534439}, {
       119, 0.05390217484054021}, {120, 0.0449922563097948}, {
       121, 0.0372386216672715}, {122, 0.030546115015454664`}, {
       123, 0.02481945644097266}, {124, 0.019965172713635293`}, {
       125, 0.015892201526474237`}, {126, 0.012512529524703274`}, {
       127, 0.009741382974183893}, {128, 0.007497271014814265}, {
       129, 0.0057035103917303775`}, {130, 0.004289283991860947}, {
       131, 0.0031895139578431916`}, {132, 0.0023455531991862415`}, {
       133, 0.001706513453597146}, {134, 0.0012290554977053695`}, {
       135, 0.0008765476755997755}, {136, 0.0006191399800800749}, {
       137, 0.00043348917358537286`}, {138, 0.0003011735024753659}, {
       139, 0.00020754971025556095`}, {140, 0.00014173769256671488`}, {
       141, 0.00009611499901884002}, {142, 0.00006498672083488544}, {
       143, 0.00004380237613635753}, {144, 0.00002924481123072321}, {
       145, 0.000019246461737039464`}, {146, 0.000012545604184898961`}, {
       147, 8.200360770300677*^-6}, {148, 5.401433218187356*^-6}, {
       149, 3.5431454016851195`*^-6}, {150, 2.282352420502683*^-6}, {
       151, 1.4475910571191847`*^-6}, {152, 9.189024635576254*^-7}, {
       153, 5.901948711551626*^-7}, {154, 3.8311119386201366`*^-7}, {
       155, 2.4667724473902263`*^-7}, {156, 1.4865099651632655`*^-7}, {
       157, 7.696523884268475*^-8}, {158, 3.89326617851635*^-8}, {
       159, 3.726456412680317*^-8}, {160, 3.7605901478163517`*^-8}, {
       161, 2.9453433551011027`*^-8}, {162, 1.7993605160619925`*^-8}, {
       163, 9.103960578601715*^-9}, {164, 6.311695715418187*^-9}, {
       165, 6.401799679296129*^-9}, {166, 5.68501651573685*^-9}, {
       167, 4.269643070896361*^-9}, {168, 2.847684714471986*^-9}, {
       169, 1.7895282937427172`*^-9}, {170, 1.1437952757066059`*^-9}, {
       171, 7.999984398510176*^-10}, {172, 6.218463821877656*^-10}, {
       173, 5.115524015505435*^-10}, {174, 4.207630480745786*^-10}, {
       175, 3.357366216645681*^-10}, {176, 2.5843136097154293`*^-10}, {
       177, 1.935474922958874*^-10}, {178, 1.4293116852212076`*^-10}, {
       179, 1.0511453421243559`*^-10}, {180, 7.712115487772525*^-11}, {
       181, 5.621667430914929*^-11}, {182, 4.054390586868913*^-11}, {
       183, 2.8908104750926594`*^-11}, {184, 2.040498913143773*^-11}, {
       185, 1.4251912976180901`*^-11}, {186, 9.800630048217516*^-12}, {
       187, 6.57913131445202*^-12}, {188, 4.274770790377179*^-12}, {
       189, 2.6749152561166425`*^-12}, {190, 1.6133543412842098`*^-12}, {
       191, 9.448194700992246*^-13}, {192, 5.42731984055033*^-13}, {
       193, 3.067097325077313*^-13}, {194, 1.6801927044294415`*^-13}, {
       195, 8.6261312595424*^-14}, {196, 3.948342048738686*^-14}, {
       197, 1.5193867717940518`*^-14}, {198, 6.5694239209621976`*^-15}, {
       199, 6.708009929094076*^-15}, {200, 6.9279436136124596`*^-15}, {
       201, 6.194400127951439*^-15}, {202, 4.965903588682642*^-15}, {
       203, 3.618768463808099*^-15}, {204, 2.442251038285703*^-15}, {
       205, 1.6301870902702573`*^-15}, {206, 1.2158632258205339`*^-15}, {
       207, 1.0318398450867304`*^-15}, {208, 8.898102756987287*^-16}, {
       209, 7.263451976926728*^-16}, {210, 5.522281081260929*^-16}, {
       211, 3.9170248855427457`*^-16}, {212, 2.6044039812402194`*^-16}, {
       213, 1.6295044994113111`*^-16}, {214, 9.614260209824502*^-17}, {
       215, 5.350234815246673*^-17}, {216, 2.8047113721690084`*^-17}, {
       217, 1.3861995886414788`*^-17}, {218, 6.5904240015867805`*^-18}, {
       219, 3.3080978813061304`*^-18}, {220, 2.0503510579974192`*^-18}, {
       221, 1.508888521740555*^-18}, {222, 1.132026207373764*^-18}, {
       223, 8.160692801667178*^-19}, {224, 5.613574032985331*^-19}, {
       225, 3.7000252062908056`*^-19}, {226, 2.347957359001564*^-19}, {
       227, 1.4396579220806302`*^-19}, {228, 8.550562430544295*^-20}, {
       229, 4.927676895626111*^-20}, {230, 2.758946893870306*^-20}, {
       231, 1.5023139820600678`*^-20}, {232, 7.96560350376841*^-21}, {
       233, 4.119992897606218*^-21}, {234, 2.085043581322321*^-21}, {
       235, 1.0378245174651536`*^-21}, {236, 5.122350162534242*^-22}, {
       237, 2.5346963760975466`*^-22}, {238, 1.271573166528436*^-22}, {
       239, 6.507491161079296*^-23}, {240, 3.3893415267401256`*^-23}, {
       241, 1.7805307740970742`*^-23}, {242, 9.34133877014123*^-24}, {
       243, 4.863817014848145*^-24}, {244, 2.5120476447117986`*^-24}, {
       245, 1.2943882778761253`*^-24}, {246, 6.729119257503642*^-25}, {
       247, 3.575203807497975*^-25}, {248, 1.955977041851322*^-25}, {
       249, 1.0987051004439517`*^-25}, {250, 6.267042066341017*^-26}, {
       251, 3.5851017056091055`*^-26}}],
    Frame->True,
    FrameTicks->{{{1., 
        FormBox[
         RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
        FormBox[
         RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
        FormBox[
         RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
        FormBox[
         RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
        FormBox["0", TraditionalForm]}, {157.25, 
        FormBox["2.5`", TraditionalForm]}, {188.5, 
        FormBox["5", TraditionalForm]}, {219.75, 
        FormBox["7.5`", TraditionalForm]}, {251., 
        FormBox["10", TraditionalForm]}}, Automatic, None, None},
    PlotRange->{-0.1, 1.1},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], Automatic}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF2XdYlXUfx/EfrnBkuCd6UCTMrag4kA9LUUEEGQdZN/swJYRUQLlBsWFm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     "], {
      {GrayLevel[0.5], EdgeForm[None], 
       GraphicsGroupBox[{PolygonBox[{{230, 710, 231}}], 
         PolygonBox[CompressedData["
1:eJwtlwfYFlIYht9zCiHRIJJoD1kt2kmSJikqSZK0l1S0QxIKpW2UrFQISUJS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          
          "]]}]}, {{}, {}, {
        {Hue[0.7796801289652671, 1, 1], LineBox[{1, 2}]}, 
        {Hue[0.6407245824031771, 1, 1], LineBox[{2, 3}]}, 
        {Hue[0.5503816896589118, 1, 1], LineBox[{3, 4}]}, 
        {Hue[0.46693777518468293`, 1, 1], LineBox[{4, 5}]}, 
        {Hue[0.34337602294665853`, 1, 1], LineBox[{5, 6}]}, 
        {Hue[0.2800240470435314, 1, 1], LineBox[{6, 7}]}, 
        {Hue[0.19178230855508877`, 1, 1], LineBox[{7, 8}]}, 
        {Hue[0.09597816212744277, 1, 1], LineBox[{8, 9}]}, 
        {Hue[0.03441314240637002, 1, 1], LineBox[{9, 10}]}, 
        {Hue[0.9619307728034987, 1, 1], LineBox[{10, 11}]}, 
        {Hue[0.8556957513387586, 1, 1], LineBox[{11, 12}]}, 
        {Hue[0.7617493391570483, 1, 1], LineBox[{12, 13}]}, 
        {Hue[0.6649750661560606, 1, 1], LineBox[{13, 14}]}, 
        {Hue[0.6003782106730116, 1, 1], LineBox[{14, 15}]}, 
        {Hue[0.5016480547751627, 1, 1], LineBox[{15, 16}]}, 
        {Hue[0.40821669939596, 1, 1], LineBox[{16, 17}]}, 
        {Hue[0.3228877865793172, 1, 1], LineBox[{17, 18}]}, 
        {Hue[0.2545692858199149, 1, 1], LineBox[{18, 19}]}, 
        {Hue[0.16962359970764626`, 1, 1], LineBox[{19, 20}]}, 
        {Hue[0.056865225111496887`, 1, 1], LineBox[{20, 21}]}, 
        {Hue[0.9539397202584826, 1, 1], LineBox[{21, 22}]}, 
        {Hue[0.7837294353244291, 1, 1], LineBox[{22, 23}]}, 
        {Hue[0.6953650268548688, 1, 1], LineBox[{23, 24}]}, 
        {Hue[0.6305218286702476, 1, 1], LineBox[{24, 25}]}, 
        {Hue[0.6528261996783604, 1, 1], LineBox[{25, 26}]}, 
        {Hue[0.7458918504427492, 1, 1], LineBox[{26, 27}]}, 
        {Hue[0.92126851876369, 1, 1], LineBox[{27, 28}]}, 
        {Hue[0.9782053362672661, 1, 1], LineBox[{28, 29}]}, 
        {Hue[0.020789729978857467`, 1, 1], LineBox[{29, 30}]}, 
        {Hue[0.07110836451469632, 1, 1], LineBox[{30, 31}]}, 
        {Hue[0.13731102313054688`, 1, 1], LineBox[{31, 32}]}, 
        {Hue[0.2237270578911424, 1, 1], LineBox[{32, 33}]}, 
        {Hue[0.308233058166907, 1, 1], LineBox[{33, 34}]}, 
        {Hue[0.37858341969070897`, 1, 1], LineBox[{34, 35}]}, 
        {Hue[0.44582144082943587`, 1, 1], LineBox[{35, 36}]}, 
        {Hue[0.5171911046750017, 1, 1], LineBox[{36, 37}]}, 
        {Hue[0.5903988201832231, 1, 1], LineBox[{37, 38}]}, 
        {Hue[0.6664515169027294, 1, 1], LineBox[{38, 39}]}, 
        {Hue[0.7432363389081607, 1, 1], LineBox[{39, 40}]}, 
        {Hue[0.8184062390390154, 1, 1], LineBox[{40, 41}]}, 
        {Hue[0.8937470175078367, 1, 1], LineBox[{41, 42}]}, 
        {Hue[0.9698398309055655, 1, 1], LineBox[{42, 43}]}, 
        {Hue[0.046820964778370125`, 1, 1], LineBox[{43, 44}]}, 
        {Hue[0.12449143522202596`, 1, 1], LineBox[{44, 45}]}, 
        {Hue[0.20232629915042974`, 1, 1], LineBox[{45, 46}]}, 
        {Hue[0.2805663074213705, 1, 1], LineBox[{46, 47}]}, 
        {Hue[0.3590896587353866, 1, 1], LineBox[{47, 48}]}, 
        {Hue[0.4382390460728896, 1, 1], LineBox[{48, 49}]}, 
        {Hue[0.517902737197748, 1, 1], LineBox[{49, 50}]}, 
        {Hue[0.5980097653972472, 1, 1], LineBox[{50, 51}]}, 
        {Hue[0.6785327051929539, 1, 1], LineBox[{51, 52}]}, 
        {Hue[0.7595095736388215, 1, 1], LineBox[{52, 53}]}, 
        {Hue[0.8409610804245629, 1, 1], LineBox[{53, 54}]}, 
        {Hue[0.9228629740322556, 1, 1], LineBox[{54, 55}]}, 
        {Hue[0.005253870251030078, 1, 1], LineBox[{55, 56}]}, 
        {Hue[0.0881095112778214, 1, 1], LineBox[{56, 57}]}, 
        {Hue[0.17139434226227557`, 1, 1], LineBox[{57, 58}]}, 
        {Hue[0.2551463423565107, 1, 1], LineBox[{58, 59}]}, 
        {Hue[0.33933976584552206`, 1, 1], LineBox[{59, 60}]}, 
        {Hue[0.4239966318214204, 1, 1], LineBox[{60, 61}]}, 
        {Hue[0.5091224007223621, 1, 1], LineBox[{61, 62}]}, 
        {Hue[0.5946966107789967, 1, 1], LineBox[{62, 63}]}, 
        {Hue[0.6807271939580889, 1, 1], LineBox[{63, 64}]}, 
        {Hue[0.7672101480855866, 1, 1], LineBox[{64, 65}]}, 
        {Hue[0.8541512225764117, 1, 1], LineBox[{65, 66}]}, 
        {Hue[0.941550181896359, 1, 1], LineBox[{66, 67}]}, 
        {Hue[0.029403984208433615`, 1, 1], LineBox[{67, 68}]}, 
        {Hue[0.11770952561423882`, 1, 1], LineBox[{68, 69}]}, 
        {Hue[0.20646973711035155`, 1, 1], LineBox[{69, 70}]}, 
        {Hue[0.295683004779304, 1, 1], LineBox[{70, 71}]}, 
        {Hue[0.3853477154083707, 1, 1], LineBox[{71, 72}]}, 
        {Hue[0.4754668125881876, 1, 1], LineBox[{72, 73}]}, 
        {Hue[0.5660352585959618, 1, 1], LineBox[{73, 74}]}, 
        {Hue[0.6570548476475564, 1, 1], LineBox[{74, 75}]}, 
        {Hue[0.7485248677659424, 1, 1], LineBox[{75, 76}]}, 
        {Hue[0.8404436664280217, 1, 1], LineBox[{76, 77}]}, 
        {Hue[0.9328115568614501, 1, 1], LineBox[{77, 78}]}, 
        {Hue[0.025625635071901066`, 1, 1], LineBox[{78, 79}]}, 
        {Hue[0.11888645287894296`, 1, 1], LineBox[{79, 80}]}, 
        {Hue[0.21259272180999705`, 1, 1], LineBox[{80, 81}]}, 
        {Hue[0.3067437454522678, 1, 1], LineBox[{81, 82}]}, 
        {Hue[0.4013375289297376, 1, 1], LineBox[{82, 83}]}, 
        {Hue[0.49637135639775043`, 1, 1], LineBox[{83, 84}]}, 
        {Hue[0.5918450111035345, 1, 1], LineBox[{84, 85}]}, 
        {Hue[0.6877562589564038, 1, 1], LineBox[{85, 86}]}, 
        {Hue[0.78410256268569, 1, 1], LineBox[{86, 87}]}, 
        {Hue[0.8808823473932245, 1, 1], LineBox[{87, 88}]}, 
        {Hue[0.9780911430522481, 1, 1], LineBox[{88, 89}]}, 
        {Hue[0.07572664367055548, 1, 1], LineBox[{89, 90}]}, 
        {Hue[0.1737853758905005, 1, 1], LineBox[{90, 91}]}, 
        {Hue[0.272262768039806, 1, 1], LineBox[{91, 92}]}, 
        {Hue[0.371153556618099, 1, 1], LineBox[{92, 93}]}, 
        {Hue[0.4704529220778693, 1, 1], LineBox[{93, 94}]}, 
        {Hue[0.5701542763470691, 1, 1], LineBox[{94, 95}]}, 
        {Hue[0.6702500724172249, 1, 1], LineBox[{95, 96}]}, 
        {Hue[0.7707317610985269, 1, 1], LineBox[{96, 97}]}, 
        {Hue[0.8715892246887066, 1, 1], LineBox[{97, 98}]}, 
        {Hue[0.9728114871992333, 1, 1], LineBox[{98, 99}]}, 
        {Hue[0.0743857178097356, 1, 1], LineBox[{99, 100}]}, 
        {Hue[0.17629597248002574`, 1, 1], LineBox[{100, 101}]}, 
        {Hue[0.2785251813360693, 1, 1], LineBox[{101, 102}]}, 
        {Hue[0.38105343827909777`, 1, 1], LineBox[{102, 103}]}, 
        {Hue[0.4838574827290812, 1, 1], LineBox[{103, 104}]}, 
        {Hue[0.5869100103646415, 1, 1], LineBox[{104, 105}]}, 
        {Hue[0.6901806989907271, 1, 1], LineBox[{105, 106}]}, 
        {Hue[0.7936343981668283, 1, 1], LineBox[{106, 107}]}, 
        {Hue[0.89723070358846, 1, 1], LineBox[{107, 108}]}, 
        {Hue[0.0009237190342984455, 1, 1], LineBox[{108, 109}]}, 
        {Hue[0.10466113495925189`, 1, 1], LineBox[{109, 110}]}, 
        {Hue[0.20838404049157222`, 1, 1], LineBox[{110, 111}]}, 
        {Hue[0.3120259981560382, 1, 1], LineBox[{111, 112}]}, 
        {Hue[0.4155125941036146, 1, 1], LineBox[{112, 113}]}, 
        {Hue[0.5187618099741917, 1, 1], LineBox[{113, 114}]}, 
        {Hue[0.6216818288821319, 1, 1], LineBox[{114, 115}]}, 
        {Hue[0.7241743766947099, 1, 1], LineBox[{115, 116}]}, 
        {Hue[0.8261341567272162, 1, 1], LineBox[{116, 117}]}, 
        {Hue[0.9274483379521479, 1, 1], LineBox[{117, 118}]}, 
        {Hue[0.027999913112641616`, 1, 1], LineBox[{118, 119}]}, 
        {Hue[0.12767141780559033`, 1, 1], LineBox[{119, 120}]}, 
        {Hue[0.226347300174275, 1, 1], LineBox[{120, 121}]}, 
        {Hue[0.3239190041183996, 1, 1], LineBox[{121, 122}]}, 
        {Hue[0.42028976211005264`, 1, 1], LineBox[{122, 123}]}, 
        {Hue[0.5153815945444176, 1, 1], LineBox[{123, 124}]}, 
        {Hue[0.6091439972339776, 1, 1], LineBox[{124, 125}]}, 
        {Hue[0.7015583727446617, 1, 1], LineBox[{125, 126}]}, 
        {Hue[0.7926410197746079, 1, 1], LineBox[{126, 127}]}, 
        {Hue[0.8824458895514251, 1, 1], LineBox[{127, 128}]}, 
        {Hue[0.9710638880548629, 1, 1], LineBox[{128, 129}]}, 
        {Hue[0.05861534812403368, 1, 1], LineBox[{129, 130}]}, 
        {Hue[0.14524236947836713`, 1, 1], LineBox[{130, 131}]}, 
        {Hue[0.2311013783598076, 1, 1], LineBox[{131, 132}]}, 
        {Hue[0.3163582834028976, 1, 1], LineBox[{132, 133}]}, 
        {Hue[0.4011833062789287, 1, 1], LineBox[{133, 134}]}, 
        {Hue[0.48573932037648593`, 1, 1], LineBox[{134, 135}]}, 
        {Hue[0.5701759410951197, 1, 1], LineBox[{135, 136}]}, 
        {Hue[0.6546269333325413, 1, 1], LineBox[{136, 137}]}, 
        {Hue[0.7392210534650951, 1, 1], LineBox[{137, 138}]}, 
        {Hue[0.8240822234173037, 1, 1], LineBox[{138, 139}]}, 
        {Hue[0.9093113235791046, 1, 1], LineBox[{139, 140}]}, 
        {Hue[0.9949861214641248, 1, 1], LineBox[{140, 141}]}, 
        {Hue[0.08117017774335357, 1, 1], LineBox[{141, 142}]}, 
        {Hue[0.16791383557109385`, 1, 1], LineBox[{142, 143}]}, 
        {Hue[0.25530021949010917`, 1, 1], LineBox[{143, 144}]}, 
        {Hue[0.34343541671925093`, 1, 1], LineBox[{144, 145}]}, 
        {Hue[0.43231232279874066`, 1, 1], LineBox[{145, 146}]}, 
        {Hue[0.5218524601207256, 1, 1], LineBox[{146, 147}]}, 
        {Hue[0.612173062059559, 1, 1], LineBox[{147, 148}]}, 
        {Hue[0.7034698819664108, 1, 1], LineBox[{148, 149}]}, 
        {Hue[0.7955898813013543, 1, 1], LineBox[{149, 150}]}, 
        {Hue[0.888364574202874, 1, 1], LineBox[{150, 151}]}, 
        {Hue[0.9822000502315932, 1, 1], LineBox[{151, 152}]}, 
        {Hue[0.0771550576325835, 1, 1], LineBox[{152, 153}]}, 
        {Hue[0.1720959120088873, 1, 1], LineBox[{153, 154}]}, 
        {Hue[0.26688998366448946`, 1, 1], LineBox[{154, 155}]}, 
        {Hue[0.3639293449103173, 1, 1], LineBox[{155, 156}]}, 
        {Hue[0.4634940195293576, 1, 1], LineBox[{156, 157}]}, 
        {Hue[0.5603649207077858, 1, 1], LineBox[{157, 158}]}, 
        {Hue[0.6519246304843664, 1, 1], LineBox[{158, 159}]}, 
        {Hue[0.7495382230954826, 1, 1], LineBox[{159, 160}]}, 
        {Hue[0.8643080095757046, 1, 1], LineBox[{160, 161}]}, 
        {Hue[0.9906253074833521, 1, 1], LineBox[{161, 162}]}, 
        {Hue[0.10779399475058227`, 1, 1], LineBox[{162, 163}]}, 
        {Hue[0.1661984781166469, 1, 1], LineBox[{163, 164}]}, 
        {Hue[0.2013592736498602, 1, 1], LineBox[{164, 165}]}, 
        {Hue[0.32046871065631627`, 1, 1], LineBox[{165, 166}]}, 
        {Hue[0.4756075296246733, 1, 1], LineBox[{166, 167}]}, 
        {Hue[0.6385764348360206, 1, 1], LineBox[{167, 168}]}, 
        {Hue[0.7869345435130637, 1, 1], LineBox[{168, 169}]}, 
        {Hue[0.8313036715488447, 1, 1], LineBox[{169, 170}]}, 
        {Hue[0.8287660083714148, 1, 1], LineBox[{170, 171}]}, 
        {Hue[0.9760417001103902, 1, 1], LineBox[{171, 172}]}, 
        {Hue[0.17203733354249606`, 1, 1], LineBox[{172, 173}]}, 
        {Hue[0.41546928442185527`, 1, 1], LineBox[{173, 174}]}, 
        {Hue[0.7130492258316945, 1, 1], LineBox[{174, 175}]}, 
        {Hue[0.013352993052113508`, 1, 1], LineBox[{175, 176}]}, 
        {Hue[0.2865809602909467, 1, 1], LineBox[{176, 177}]}, 
        {Hue[0.5485481811412078, 1, 1], LineBox[{177, 178}]}, 
        {Hue[0.8100483978103337, 1, 1], LineBox[{178, 179}]}, 
        {Hue[0.07438821094553806, 1, 1], LineBox[{179, 180}]}, 
        {Hue[0.34079459211773067`, 1, 1], LineBox[{180, 181}]}, 
        {Hue[0.6067300720056761, 1, 1], LineBox[{181, 182}]}, 
        {Hue[0.8699265236196507, 1, 1], LineBox[{182, 183}]}, 
        {Hue[0.12928216875586565`, 1, 1], LineBox[{183, 184}]}, 
        {Hue[0.3844879401823965, 1, 1], LineBox[{184, 185}]}, 
        {Hue[0.6353011644887285, 1, 1], LineBox[{185, 186}]}, 
        {Hue[0.8810951367672261, 1, 1], LineBox[{186, 187}]}, 
        {Hue[0.12083724148470129`, 1, 1], LineBox[{187, 188}]}, 
        {Hue[0.35359810181285867`, 1, 1], LineBox[{188, 189}]}, 
        {Hue[0.5797037414262387, 1, 1], LineBox[{189, 190}]}, 
        {Hue[0.801927284818749, 1, 1], LineBox[{190, 191}]}, 
        {Hue[0.02490207809077299, 1, 1], LineBox[{191, 192}]}, 
        {Hue[0.2524952787854635, 1, 1], LineBox[{192, 193}]}, 
        {Hue[0.4861575156782732, 1, 1], LineBox[{193, 194}]}, 
        {Hue[0.7255279616649843, 1, 1], LineBox[{194, 195}]}, 
        {Hue[0.9696088981681895, 1, 1], LineBox[{195, 196}]}, 
        {Hue[0.21741244056167142`, 1, 1], LineBox[{196, 197}]}, 
        {Hue[0.46813755920083117`, 1, 1], LineBox[{197, 198}]}, 
        {Hue[0.7211432166767873, 1, 1], LineBox[{198, 199}]}, 
        {Hue[0.975875284824123, 1, 1], LineBox[{199, 200}]}, 
        {Hue[0.23181597238659138`, 1, 1], LineBox[{200, 201}]}, 
        {Hue[0.48848723203835615`, 1, 1], LineBox[{201, 202}]}, 
        {Hue[0.745497138351656, 1, 1], LineBox[{202, 203}]}, 
        {Hue[0.0025923563209303813`, 1, 1], LineBox[{203, 204}]}, 
        {Hue[0.25967466350231044`, 1, 1], LineBox[{204, 205}]}, 
        {Hue[0.5167645910330184, 1, 1], LineBox[{205, 206}]}, 
        {Hue[0.7739220318465841, 1, 1], LineBox[{206, 207}]}, 
        {Hue[0.031150257131776243`, 1, 1], LineBox[{207, 208}]}, 
        {Hue[0.2883194585058068, 1, 1], LineBox[{208, 209}]}, 
        {Hue[0.5451448055773551, 1, 1], LineBox[{209, 210}]}, 
        {Hue[0.801241512943925, 1, 1], LineBox[{210, 211}]}, 
        {Hue[0.05625312288155043, 1, 1], LineBox[{211, 212}]}, 
        {Hue[0.31002956423589995`, 1, 1], LineBox[{212, 213}]}, 
        {Hue[0.5628570448388254, 1, 1], LineBox[{213, 214}]}, 
        {Hue[0.8158451729427638, 1, 1], LineBox[{214, 215}]}, 
        {Hue[0.07184273217758473, 1, 1], LineBox[{215, 216}]}, 
        {Hue[0.33804700588730185`, 1, 1], LineBox[{216, 217}]}, 
        {Hue[0.6330401547699546, 1, 1], LineBox[{217, 218}]}, 
        {Hue[0.9793681228383752, 1, 1], LineBox[{218, 219}]}, 
        {Hue[0.31651445767840364`, 1, 1], LineBox[{219, 220}]}, 
        {Hue[0.6077541122814277, 1, 1], LineBox[{220, 221}]}, 
        {Hue[0.8802124139644494, 1, 1], LineBox[{221, 222}]}, 
        {Hue[0.14828103651834512`, 1, 1], LineBox[{222, 223}]}, 
        {Hue[0.4157813282383809, 1, 1], LineBox[{223, 224}]}, 
        {Hue[0.6821139026892058, 1, 1], LineBox[{224, 225}]}, 
        {Hue[0.9456040737426483, 1, 1], LineBox[{225, 226}]}, 
        {Hue[0.20528754302196955`, 1, 1], LineBox[{226, 227}]}, 
        {Hue[0.46120974183242636`, 1, 1], LineBox[{227, 228}]}, 
        {Hue[0.7139993425376341, 1, 1], LineBox[{228, 229}]}, 
        {Hue[0.9644555293237684, 1, 1], LineBox[{229, 230}]}, 
        {Hue[0.2133525195580862, 1, 1], LineBox[{230, 231}]}, 
        {Hue[0.4613964047250365, 1, 1], LineBox[{231, 232}]}, 
        {Hue[0.7092509138974926, 1, 1], LineBox[{232, 233}]}, 
        {Hue[0.9575856208274757, 1, 1], LineBox[{233, 234}]}, 
        {Hue[0.20712353313556747`, 1, 1], LineBox[{234, 235}]}, 
        {Hue[0.4586661154329677, 1, 1], LineBox[{235, 236}]}, 
        {Hue[0.7130546309764122, 1, 1], LineBox[{236, 237}]}, 
        {Hue[0.9709943653608248, 1, 1], LineBox[{237, 238}]}, 
        {Hue[0.23267334689881333`, 1, 1], LineBox[{238, 239}]}, 
        {Hue[0.49727980286706513`, 1, 1], LineBox[{239, 240}]}, 
        {Hue[0.7628869304352861, 1, 1], LineBox[{240, 241}]}, 
        {Hue[0.02716246416619776, 1, 1], LineBox[{241, 242}]}, 
        {Hue[0.28845986121524614`, 1, 1], LineBox[{242, 243}]}, 
        {Hue[0.5463100846568116, 1, 1], LineBox[{243, 244}]}, 
        {Hue[0.8011400052763996, 1, 1], LineBox[{244, 245}]}, 
        {Hue[0.053766015412985584`, 1, 1], LineBox[{245, 246}]}, 
        {Hue[0.3050462366515147, 1, 1], LineBox[{246, 247}]}, 
        {Hue[0.555729431609835, 1, 1], LineBox[{247, 248}]}, 
        {Hue[0.8064210755015563, 1, 1], LineBox[{248, 249}]}, 
        {Hue[0.05759929417499629, 1, 1], LineBox[{249, 250}]}, 
        {Hue[0.3096448548061538, 1, 1], LineBox[{250, 251}]}}}},
     VertexColors->{
       Hue[0.7796801289652671, 1, 1], 
       Hue[0.6407245824031771, 1, 1], 
       Hue[0.5503816896589118, 1, 1], 
       Hue[0.46693777518468293`, 1, 1], 
       Hue[0.34337602294665853`, 1, 1], 
       Hue[0.2800240470435314, 1, 1], 
       Hue[0.19178230855508877`, 1, 1], 
       Hue[0.09597816212744277, 1, 1], 
       Hue[0.03441314240637002, 1, 1], 
       Hue[0.9619307728034987, 1, 1], 
       Hue[0.8556957513387586, 1, 1], 
       Hue[0.7617493391570483, 1, 1], 
       Hue[0.6649750661560606, 1, 1], 
       Hue[0.6003782106730116, 1, 1], 
       Hue[0.5016480547751627, 1, 1], 
       Hue[0.40821669939596, 1, 1], 
       Hue[0.3228877865793172, 1, 1], 
       Hue[0.2545692858199149, 1, 1], 
       Hue[0.16962359970764626`, 1, 1], 
       Hue[0.056865225111496887`, 1, 1], 
       Hue[0.9539397202584826, 1, 1], 
       Hue[0.7837294353244291, 1, 1], 
       Hue[0.6953650268548688, 1, 1], 
       Hue[0.6305218286702476, 1, 1], 
       Hue[0.6528261996783604, 1, 1], 
       Hue[0.7458918504427492, 1, 1], 
       Hue[0.92126851876369, 1, 1], 
       Hue[0.9782053362672661, 1, 1], 
       Hue[0.020789729978857467`, 1, 1], 
       Hue[0.07110836451469632, 1, 1], 
       Hue[0.13731102313054688`, 1, 1], 
       Hue[0.2237270578911424, 1, 1], 
       Hue[0.308233058166907, 1, 1], 
       Hue[0.37858341969070897`, 1, 1], 
       Hue[0.44582144082943587`, 1, 1], 
       Hue[0.5171911046750017, 1, 1], 
       Hue[0.5903988201832231, 1, 1], 
       Hue[0.6664515169027294, 1, 1], 
       Hue[0.7432363389081607, 1, 1], 
       Hue[0.8184062390390154, 1, 1], 
       Hue[0.8937470175078367, 1, 1], 
       Hue[0.9698398309055655, 1, 1], 
       Hue[0.046820964778370125`, 1, 1], 
       Hue[0.12449143522202596`, 1, 1], 
       Hue[0.20232629915042974`, 1, 1], 
       Hue[0.2805663074213705, 1, 1], 
       Hue[0.3590896587353866, 1, 1], 
       Hue[0.4382390460728896, 1, 1], 
       Hue[0.517902737197748, 1, 1], 
       Hue[0.5980097653972472, 1, 1], 
       Hue[0.6785327051929539, 1, 1], 
       Hue[0.7595095736388215, 1, 1], 
       Hue[0.8409610804245629, 1, 1], 
       Hue[0.9228629740322556, 1, 1], 
       Hue[0.005253870251030078, 1, 1], 
       Hue[0.0881095112778214, 1, 1], 
       Hue[0.17139434226227557`, 1, 1], 
       Hue[0.2551463423565107, 1, 1], 
       Hue[0.33933976584552206`, 1, 1], 
       Hue[0.4239966318214204, 1, 1], 
       Hue[0.5091224007223621, 1, 1], 
       Hue[0.5946966107789967, 1, 1], 
       Hue[0.6807271939580889, 1, 1], 
       Hue[0.7672101480855866, 1, 1], 
       Hue[0.8541512225764117, 1, 1], 
       Hue[0.941550181896359, 1, 1], 
       Hue[0.029403984208433615`, 1, 1], 
       Hue[0.11770952561423882`, 1, 1], 
       Hue[0.20646973711035155`, 1, 1], 
       Hue[0.295683004779304, 1, 1], 
       Hue[0.3853477154083707, 1, 1], 
       Hue[0.4754668125881876, 1, 1], 
       Hue[0.5660352585959618, 1, 1], 
       Hue[0.6570548476475564, 1, 1], 
       Hue[0.7485248677659424, 1, 1], 
       Hue[0.8404436664280217, 1, 1], 
       Hue[0.9328115568614501, 1, 1], 
       Hue[0.025625635071901066`, 1, 1], 
       Hue[0.11888645287894296`, 1, 1], 
       Hue[0.21259272180999705`, 1, 1], 
       Hue[0.3067437454522678, 1, 1], 
       Hue[0.4013375289297376, 1, 1], 
       Hue[0.49637135639775043`, 1, 1], 
       Hue[0.5918450111035345, 1, 1], 
       Hue[0.6877562589564038, 1, 1], 
       Hue[0.78410256268569, 1, 1], 
       Hue[0.8808823473932245, 1, 1], 
       Hue[0.9780911430522481, 1, 1], 
       Hue[0.07572664367055548, 1, 1], 
       Hue[0.1737853758905005, 1, 1], 
       Hue[0.272262768039806, 1, 1], 
       Hue[0.371153556618099, 1, 1], 
       Hue[0.4704529220778693, 1, 1], 
       Hue[0.5701542763470691, 1, 1], 
       Hue[0.6702500724172249, 1, 1], 
       Hue[0.7707317610985269, 1, 1], 
       Hue[0.8715892246887066, 1, 1], 
       Hue[0.9728114871992333, 1, 1], 
       Hue[0.0743857178097356, 1, 1], 
       Hue[0.17629597248002574`, 1, 1], 
       Hue[0.2785251813360693, 1, 1], 
       Hue[0.38105343827909777`, 1, 1], 
       Hue[0.4838574827290812, 1, 1], 
       Hue[0.5869100103646415, 1, 1], 
       Hue[0.6901806989907271, 1, 1], 
       Hue[0.7936343981668283, 1, 1], 
       Hue[0.89723070358846, 1, 1], 
       Hue[0.0009237190342984455, 1, 1], 
       Hue[0.10466113495925189`, 1, 1], 
       Hue[0.20838404049157222`, 1, 1], 
       Hue[0.3120259981560382, 1, 1], 
       Hue[0.4155125941036146, 1, 1], 
       Hue[0.5187618099741917, 1, 1], 
       Hue[0.6216818288821319, 1, 1], 
       Hue[0.7241743766947099, 1, 1], 
       Hue[0.8261341567272162, 1, 1], 
       Hue[0.9274483379521479, 1, 1], 
       Hue[0.027999913112641616`, 1, 1], 
       Hue[0.12767141780559033`, 1, 1], 
       Hue[0.226347300174275, 1, 1], 
       Hue[0.3239190041183996, 1, 1], 
       Hue[0.42028976211005264`, 1, 1], 
       Hue[0.5153815945444176, 1, 1], 
       Hue[0.6091439972339776, 1, 1], 
       Hue[0.7015583727446617, 1, 1], 
       Hue[0.7926410197746079, 1, 1], 
       Hue[0.8824458895514251, 1, 1], 
       Hue[0.9710638880548629, 1, 1], 
       Hue[0.05861534812403368, 1, 1], 
       Hue[0.14524236947836713`, 1, 1], 
       Hue[0.2311013783598076, 1, 1], 
       Hue[0.3163582834028976, 1, 1], 
       Hue[0.4011833062789287, 1, 1], 
       Hue[0.48573932037648593`, 1, 1], 
       Hue[0.5701759410951197, 1, 1], 
       Hue[0.6546269333325413, 1, 1], 
       Hue[0.7392210534650951, 1, 1], 
       Hue[0.8240822234173037, 1, 1], 
       Hue[0.9093113235791046, 1, 1], 
       Hue[0.9949861214641248, 1, 1], 
       Hue[0.08117017774335357, 1, 1], 
       Hue[0.16791383557109385`, 1, 1], 
       Hue[0.25530021949010917`, 1, 1], 
       Hue[0.34343541671925093`, 1, 1], 
       Hue[0.43231232279874066`, 1, 1], 
       Hue[0.5218524601207256, 1, 1], 
       Hue[0.612173062059559, 1, 1], 
       Hue[0.7034698819664108, 1, 1], 
       Hue[0.7955898813013543, 1, 1], 
       Hue[0.888364574202874, 1, 1], 
       Hue[0.9822000502315932, 1, 1], 
       Hue[0.0771550576325835, 1, 1], 
       Hue[0.1720959120088873, 1, 1], 
       Hue[0.26688998366448946`, 1, 1], 
       Hue[0.3639293449103173, 1, 1], 
       Hue[0.4634940195293576, 1, 1], 
       Hue[0.5603649207077858, 1, 1], 
       Hue[0.6519246304843664, 1, 1], 
       Hue[0.7495382230954826, 1, 1], 
       Hue[0.8643080095757046, 1, 1], 
       Hue[0.9906253074833521, 1, 1], 
       Hue[0.10779399475058227`, 1, 1], 
       Hue[0.1661984781166469, 1, 1], 
       Hue[0.2013592736498602, 1, 1], 
       Hue[0.32046871065631627`, 1, 1], 
       Hue[0.4756075296246733, 1, 1], 
       Hue[0.6385764348360206, 1, 1], 
       Hue[0.7869345435130637, 1, 1], 
       Hue[0.8313036715488447, 1, 1], 
       Hue[0.8287660083714148, 1, 1], 
       Hue[0.9760417001103902, 1, 1], 
       Hue[0.17203733354249606`, 1, 1], 
       Hue[0.41546928442185527`, 1, 1], 
       Hue[0.7130492258316945, 1, 1], 
       Hue[0.013352993052113508`, 1, 1], 
       Hue[0.2865809602909467, 1, 1], 
       Hue[0.5485481811412078, 1, 1], 
       Hue[0.8100483978103337, 1, 1], 
       Hue[0.07438821094553806, 1, 1], 
       Hue[0.34079459211773067`, 1, 1], 
       Hue[0.6067300720056761, 1, 1], 
       Hue[0.8699265236196507, 1, 1], 
       Hue[0.12928216875586565`, 1, 1], 
       Hue[0.3844879401823965, 1, 1], 
       Hue[0.6353011644887285, 1, 1], 
       Hue[0.8810951367672261, 1, 1], 
       Hue[0.12083724148470129`, 1, 1], 
       Hue[0.35359810181285867`, 1, 1], 
       Hue[0.5797037414262387, 1, 1], 
       Hue[0.801927284818749, 1, 1], 
       Hue[0.02490207809077299, 1, 1], 
       Hue[0.2524952787854635, 1, 1], 
       Hue[0.4861575156782732, 1, 1], 
       Hue[0.7255279616649843, 1, 1], 
       Hue[0.9696088981681895, 1, 1], 
       Hue[0.21741244056167142`, 1, 1], 
       Hue[0.46813755920083117`, 1, 1], 
       Hue[0.7211432166767873, 1, 1], 
       Hue[0.975875284824123, 1, 1], 
       Hue[0.23181597238659138`, 1, 1], 
       Hue[0.48848723203835615`, 1, 1], 
       Hue[0.745497138351656, 1, 1], 
       Hue[0.0025923563209303813`, 1, 1], 
       Hue[0.25967466350231044`, 1, 1], 
       Hue[0.5167645910330184, 1, 1], 
       Hue[0.7739220318465841, 1, 1], 
       Hue[0.031150257131776243`, 1, 1], 
       Hue[0.2883194585058068, 1, 1], 
       Hue[0.5451448055773551, 1, 1], 
       Hue[0.801241512943925, 1, 1], 
       Hue[0.05625312288155043, 1, 1], 
       Hue[0.31002956423589995`, 1, 1], 
       Hue[0.5628570448388254, 1, 1], 
       Hue[0.8158451729427638, 1, 1], 
       Hue[0.07184273217758473, 1, 1], 
       Hue[0.33804700588730185`, 1, 1], 
       Hue[0.6330401547699546, 1, 1], 
       Hue[0.9793681228383752, 1, 1], 
       Hue[0.31651445767840364`, 1, 1], 
       Hue[0.6077541122814277, 1, 1], 
       Hue[0.8802124139644494, 1, 1], 
       Hue[0.14828103651834512`, 1, 1], 
       Hue[0.4157813282383809, 1, 1], 
       Hue[0.6821139026892058, 1, 1], 
       Hue[0.9456040737426483, 1, 1], 
       Hue[0.20528754302196955`, 1, 1], 
       Hue[0.46120974183242636`, 1, 1], 
       Hue[0.7139993425376341, 1, 1], 
       Hue[0.9644555293237684, 1, 1], 
       Hue[0.2133525195580862, 1, 1], 
       Hue[0.4613964047250365, 1, 1], 
       Hue[0.7092509138974926, 1, 1], 
       Hue[0.9575856208274757, 1, 1], 
       Hue[0.20712353313556747`, 1, 1], 
       Hue[0.4586661154329677, 1, 1], 
       Hue[0.7130546309764122, 1, 1], 
       Hue[0.9709943653608248, 1, 1], 
       Hue[0.23267334689881333`, 1, 1], 
       Hue[0.49727980286706513`, 1, 1], 
       Hue[0.7628869304352861, 1, 1], 
       Hue[0.02716246416619776, 1, 1], 
       Hue[0.28845986121524614`, 1, 1], 
       Hue[0.5463100846568116, 1, 1], 
       Hue[0.8011400052763996, 1, 1], 
       Hue[0.053766015412985584`, 1, 1], 
       Hue[0.3050462366515147, 1, 1], 
       Hue[0.555729431609835, 1, 1], 
       Hue[0.8064210755015563, 1, 1], 
       Hue[0.05759929417499629, 1, 1], 
       Hue[0.3096448548061538, 1, 1], 
       Hue[0.562871819288882, 1, 1], 
       Hue[0.7796801289652671, 1, 1], 
       Hue[0.6407245824031771, 1, 1], 
       Hue[0.6407245824031771, 1, 1], 
       Hue[0.5503816896589118, 1, 1], 
       Hue[0.5503816896589118, 1, 1], 
       Hue[0.46693777518468293`, 1, 1], 
       Hue[0.46693777518468293`, 1, 1], 
       Hue[0.34337602294665853`, 1, 1], 
       Hue[0.34337602294665853`, 1, 1], 
       Hue[0.2800240470435314, 1, 1], 
       Hue[0.2800240470435314, 1, 1], 
       Hue[0.19178230855508877`, 1, 1], 
       Hue[0.19178230855508877`, 1, 1], 
       Hue[0.09597816212744277, 1, 1], 
       Hue[0.09597816212744277, 1, 1], 
       Hue[0.03441314240637002, 1, 1], 
       Hue[0.03441314240637002, 1, 1], 
       Hue[0.9619307728034987, 1, 1], 
       Hue[0.9619307728034987, 1, 1], 
       Hue[0.8556957513387586, 1, 1], 
       Hue[0.8556957513387586, 1, 1], 
       Hue[0.7617493391570483, 1, 1], 
       Hue[0.7617493391570483, 1, 1], 
       Hue[0.6649750661560606, 1, 1], 
       Hue[0.6649750661560606, 1, 1], 
       Hue[0.6003782106730116, 1, 1], 
       Hue[0.6003782106730116, 1, 1], 
       Hue[0.5016480547751627, 1, 1], 
       Hue[0.5016480547751627, 1, 1], 
       Hue[0.40821669939596, 1, 1], 
       Hue[0.40821669939596, 1, 1], 
       Hue[0.3228877865793172, 1, 1], 
       Hue[0.3228877865793172, 1, 1], 
       Hue[0.2545692858199149, 1, 1], 
       Hue[0.2545692858199149, 1, 1], 
       Hue[0.16962359970764626`, 1, 1], 
       Hue[0.16962359970764626`, 1, 1], 
       Hue[0.056865225111496887`, 1, 1], 
       Hue[0.056865225111496887`, 1, 1], 
       Hue[0.9539397202584826, 1, 1], 
       Hue[0.9539397202584826, 1, 1], 
       Hue[0.7837294353244291, 1, 1], 
       Hue[0.7837294353244291, 1, 1], 
       Hue[0.6953650268548688, 1, 1], 
       Hue[0.6953650268548688, 1, 1], 
       Hue[0.6305218286702476, 1, 1], 
       Hue[0.6305218286702476, 1, 1], 
       Hue[0.6528261996783604, 1, 1], 
       Hue[0.6528261996783604, 1, 1], 
       Hue[0.7458918504427492, 1, 1], 
       Hue[0.7458918504427492, 1, 1], 
       Hue[0.92126851876369, 1, 1], 
       Hue[0.92126851876369, 1, 1], 
       Hue[0.9782053362672661, 1, 1], 
       Hue[0.9782053362672661, 1, 1], 
       Hue[0.020789729978857467`, 1, 1], 
       Hue[0.020789729978857467`, 1, 1], 
       Hue[0.07110836451469632, 1, 1], 
       Hue[0.07110836451469632, 1, 1], 
       Hue[0.13731102313054688`, 1, 1], 
       Hue[0.13731102313054688`, 1, 1], 
       Hue[0.2237270578911424, 1, 1], 
       Hue[0.2237270578911424, 1, 1], 
       Hue[0.308233058166907, 1, 1], 
       Hue[0.308233058166907, 1, 1], 
       Hue[0.37858341969070897`, 1, 1], 
       Hue[0.37858341969070897`, 1, 1], 
       Hue[0.44582144082943587`, 1, 1], 
       Hue[0.44582144082943587`, 1, 1], 
       Hue[0.5171911046750017, 1, 1], 
       Hue[0.5171911046750017, 1, 1], 
       Hue[0.5903988201832231, 1, 1], 
       Hue[0.5903988201832231, 1, 1], 
       Hue[0.6664515169027294, 1, 1], 
       Hue[0.6664515169027294, 1, 1], 
       Hue[0.7432363389081607, 1, 1], 
       Hue[0.7432363389081607, 1, 1], 
       Hue[0.8184062390390154, 1, 1], 
       Hue[0.8184062390390154, 1, 1], 
       Hue[0.8937470175078367, 1, 1], 
       Hue[0.8937470175078367, 1, 1], 
       Hue[0.9698398309055655, 1, 1], 
       Hue[0.9698398309055655, 1, 1], 
       Hue[0.046820964778370125`, 1, 1], 
       Hue[0.046820964778370125`, 1, 1], 
       Hue[0.12449143522202596`, 1, 1], 
       Hue[0.12449143522202596`, 1, 1], 
       Hue[0.20232629915042974`, 1, 1], 
       Hue[0.20232629915042974`, 1, 1], 
       Hue[0.2805663074213705, 1, 1], 
       Hue[0.2805663074213705, 1, 1], 
       Hue[0.3590896587353866, 1, 1], 
       Hue[0.3590896587353866, 1, 1], 
       Hue[0.4382390460728896, 1, 1], 
       Hue[0.4382390460728896, 1, 1], 
       Hue[0.517902737197748, 1, 1], 
       Hue[0.517902737197748, 1, 1], 
       Hue[0.5980097653972472, 1, 1], 
       Hue[0.5980097653972472, 1, 1], 
       Hue[0.6785327051929539, 1, 1], 
       Hue[0.6785327051929539, 1, 1], 
       Hue[0.7595095736388215, 1, 1], 
       Hue[0.7595095736388215, 1, 1], 
       Hue[0.8409610804245629, 1, 1], 
       Hue[0.8409610804245629, 1, 1], 
       Hue[0.9228629740322556, 1, 1], 
       Hue[0.9228629740322556, 1, 1], 
       Hue[0.005253870251030078, 1, 1], 
       Hue[0.005253870251030078, 1, 1], 
       Hue[0.0881095112778214, 1, 1], 
       Hue[0.0881095112778214, 1, 1], 
       Hue[0.17139434226227557`, 1, 1], 
       Hue[0.17139434226227557`, 1, 1], 
       Hue[0.2551463423565107, 1, 1], 
       Hue[0.2551463423565107, 1, 1], 
       Hue[0.33933976584552206`, 1, 1], 
       Hue[0.33933976584552206`, 1, 1], 
       Hue[0.4239966318214204, 1, 1], 
       Hue[0.4239966318214204, 1, 1], 
       Hue[0.5091224007223621, 1, 1], 
       Hue[0.5091224007223621, 1, 1], 
       Hue[0.5946966107789967, 1, 1], 
       Hue[0.5946966107789967, 1, 1], 
       Hue[0.6807271939580889, 1, 1], 
       Hue[0.6807271939580889, 1, 1], 
       Hue[0.7672101480855866, 1, 1], 
       Hue[0.7672101480855866, 1, 1], 
       Hue[0.8541512225764117, 1, 1], 
       Hue[0.8541512225764117, 1, 1], 
       Hue[0.941550181896359, 1, 1], 
       Hue[0.941550181896359, 1, 1], 
       Hue[0.029403984208433615`, 1, 1], 
       Hue[0.029403984208433615`, 1, 1], 
       Hue[0.11770952561423882`, 1, 1], 
       Hue[0.11770952561423882`, 1, 1], 
       Hue[0.20646973711035155`, 1, 1], 
       Hue[0.20646973711035155`, 1, 1], 
       Hue[0.295683004779304, 1, 1], 
       Hue[0.295683004779304, 1, 1], 
       Hue[0.3853477154083707, 1, 1], 
       Hue[0.3853477154083707, 1, 1], 
       Hue[0.4754668125881876, 1, 1], 
       Hue[0.4754668125881876, 1, 1], 
       Hue[0.5660352585959618, 1, 1], 
       Hue[0.5660352585959618, 1, 1], 
       Hue[0.6570548476475564, 1, 1], 
       Hue[0.6570548476475564, 1, 1], 
       Hue[0.7485248677659424, 1, 1], 
       Hue[0.7485248677659424, 1, 1], 
       Hue[0.8404436664280217, 1, 1], 
       Hue[0.8404436664280217, 1, 1], 
       Hue[0.9328115568614501, 1, 1], 
       Hue[0.9328115568614501, 1, 1], 
       Hue[0.025625635071901066`, 1, 1], 
       Hue[0.025625635071901066`, 1, 1], 
       Hue[0.11888645287894296`, 1, 1], 
       Hue[0.11888645287894296`, 1, 1], 
       Hue[0.21259272180999705`, 1, 1], 
       Hue[0.21259272180999705`, 1, 1], 
       Hue[0.3067437454522678, 1, 1], 
       Hue[0.3067437454522678, 1, 1], 
       Hue[0.4013375289297376, 1, 1], 
       Hue[0.4013375289297376, 1, 1], 
       Hue[0.49637135639775043`, 1, 1], 
       Hue[0.49637135639775043`, 1, 1], 
       Hue[0.5918450111035345, 1, 1], 
       Hue[0.5918450111035345, 1, 1], 
       Hue[0.6877562589564038, 1, 1], 
       Hue[0.6877562589564038, 1, 1], 
       Hue[0.78410256268569, 1, 1], 
       Hue[0.78410256268569, 1, 1], 
       Hue[0.8808823473932245, 1, 1], 
       Hue[0.8808823473932245, 1, 1], 
       Hue[0.9780911430522481, 1, 1], 
       Hue[0.9780911430522481, 1, 1], 
       Hue[0.07572664367055548, 1, 1], 
       Hue[0.07572664367055548, 1, 1], 
       Hue[0.1737853758905005, 1, 1], 
       Hue[0.1737853758905005, 1, 1], 
       Hue[0.272262768039806, 1, 1], 
       Hue[0.272262768039806, 1, 1], 
       Hue[0.371153556618099, 1, 1], 
       Hue[0.371153556618099, 1, 1], 
       Hue[0.4704529220778693, 1, 1], 
       Hue[0.4704529220778693, 1, 1], 
       Hue[0.5701542763470691, 1, 1], 
       Hue[0.5701542763470691, 1, 1], 
       Hue[0.6702500724172249, 1, 1], 
       Hue[0.6702500724172249, 1, 1], 
       Hue[0.7707317610985269, 1, 1], 
       Hue[0.7707317610985269, 1, 1], 
       Hue[0.8715892246887066, 1, 1], 
       Hue[0.8715892246887066, 1, 1], 
       Hue[0.9728114871992333, 1, 1], 
       Hue[0.9728114871992333, 1, 1], 
       Hue[0.0743857178097356, 1, 1], 
       Hue[0.0743857178097356, 1, 1], 
       Hue[0.17629597248002574`, 1, 1], 
       Hue[0.17629597248002574`, 1, 1], 
       Hue[0.2785251813360693, 1, 1], 
       Hue[0.2785251813360693, 1, 1], 
       Hue[0.38105343827909777`, 1, 1], 
       Hue[0.38105343827909777`, 1, 1], 
       Hue[0.4838574827290812, 1, 1], 
       Hue[0.4838574827290812, 1, 1], 
       Hue[0.5869100103646415, 1, 1], 
       Hue[0.5869100103646415, 1, 1], 
       Hue[0.6901806989907271, 1, 1], 
       Hue[0.6901806989907271, 1, 1], 
       Hue[0.7936343981668283, 1, 1], 
       Hue[0.7936343981668283, 1, 1], 
       Hue[0.89723070358846, 1, 1], 
       Hue[0.89723070358846, 1, 1], 
       Hue[0.0009237190342984455, 1, 1], 
       Hue[0.0009237190342984455, 1, 1], 
       Hue[0.10466113495925189`, 1, 1], 
       Hue[0.10466113495925189`, 1, 1], 
       Hue[0.20838404049157222`, 1, 1], 
       Hue[0.20838404049157222`, 1, 1], 
       Hue[0.3120259981560382, 1, 1], 
       Hue[0.3120259981560382, 1, 1], 
       Hue[0.4155125941036146, 1, 1], 
       Hue[0.4155125941036146, 1, 1], 
       Hue[0.5187618099741917, 1, 1], 
       Hue[0.5187618099741917, 1, 1], 
       Hue[0.6216818288821319, 1, 1], 
       Hue[0.6216818288821319, 1, 1], 
       Hue[0.7241743766947099, 1, 1], 
       Hue[0.7241743766947099, 1, 1], 
       Hue[0.8261341567272162, 1, 1], 
       Hue[0.8261341567272162, 1, 1], 
       Hue[0.9274483379521479, 1, 1], 
       Hue[0.9274483379521479, 1, 1], 
       Hue[0.027999913112641616`, 1, 1], 
       Hue[0.027999913112641616`, 1, 1], 
       Hue[0.12767141780559033`, 1, 1], 
       Hue[0.12767141780559033`, 1, 1], 
       Hue[0.226347300174275, 1, 1], 
       Hue[0.226347300174275, 1, 1], 
       Hue[0.3239190041183996, 1, 1], 
       Hue[0.3239190041183996, 1, 1], 
       Hue[0.42028976211005264`, 1, 1], 
       Hue[0.42028976211005264`, 1, 1], 
       Hue[0.5153815945444176, 1, 1], 
       Hue[0.5153815945444176, 1, 1], 
       Hue[0.6091439972339776, 1, 1], 
       Hue[0.6091439972339776, 1, 1], 
       Hue[0.7015583727446617, 1, 1], 
       Hue[0.7015583727446617, 1, 1], 
       Hue[0.7926410197746079, 1, 1], 
       Hue[0.7926410197746079, 1, 1], 
       Hue[0.8824458895514251, 1, 1], 
       Hue[0.8824458895514251, 1, 1], 
       Hue[0.9710638880548629, 1, 1], 
       Hue[0.9710638880548629, 1, 1], 
       Hue[0.05861534812403368, 1, 1], 
       Hue[0.05861534812403368, 1, 1], 
       Hue[0.14524236947836713`, 1, 1], 
       Hue[0.14524236947836713`, 1, 1], 
       Hue[0.2311013783598076, 1, 1], 
       Hue[0.2311013783598076, 1, 1], 
       Hue[0.3163582834028976, 1, 1], 
       Hue[0.3163582834028976, 1, 1], 
       Hue[0.4011833062789287, 1, 1], 
       Hue[0.4011833062789287, 1, 1], 
       Hue[0.48573932037648593`, 1, 1], 
       Hue[0.48573932037648593`, 1, 1], 
       Hue[0.5701759410951197, 1, 1], 
       Hue[0.5701759410951197, 1, 1], 
       Hue[0.6546269333325413, 1, 1], 
       Hue[0.6546269333325413, 1, 1], 
       Hue[0.7392210534650951, 1, 1], 
       Hue[0.7392210534650951, 1, 1], 
       Hue[0.8240822234173037, 1, 1], 
       Hue[0.8240822234173037, 1, 1], 
       Hue[0.9093113235791046, 1, 1], 
       Hue[0.9093113235791046, 1, 1], 
       Hue[0.9949861214641248, 1, 1], 
       Hue[0.9949861214641248, 1, 1], 
       Hue[0.08117017774335357, 1, 1], 
       Hue[0.08117017774335357, 1, 1], 
       Hue[0.16791383557109385`, 1, 1], 
       Hue[0.16791383557109385`, 1, 1], 
       Hue[0.25530021949010917`, 1, 1], 
       Hue[0.25530021949010917`, 1, 1], 
       Hue[0.34343541671925093`, 1, 1], 
       Hue[0.34343541671925093`, 1, 1], 
       Hue[0.43231232279874066`, 1, 1], 
       Hue[0.43231232279874066`, 1, 1], 
       Hue[0.5218524601207256, 1, 1], 
       Hue[0.5218524601207256, 1, 1], 
       Hue[0.612173062059559, 1, 1], 
       Hue[0.612173062059559, 1, 1], 
       Hue[0.7034698819664108, 1, 1], 
       Hue[0.7034698819664108, 1, 1], 
       Hue[0.7955898813013543, 1, 1], 
       Hue[0.7955898813013543, 1, 1], 
       Hue[0.888364574202874, 1, 1], 
       Hue[0.888364574202874, 1, 1], 
       Hue[0.9822000502315932, 1, 1], 
       Hue[0.9822000502315932, 1, 1], 
       Hue[0.0771550576325835, 1, 1], 
       Hue[0.0771550576325835, 1, 1], 
       Hue[0.1720959120088873, 1, 1], 
       Hue[0.1720959120088873, 1, 1], 
       Hue[0.26688998366448946`, 1, 1], 
       Hue[0.26688998366448946`, 1, 1], 
       Hue[0.3639293449103173, 1, 1], 
       Hue[0.3639293449103173, 1, 1], 
       Hue[0.4634940195293576, 1, 1], 
       Hue[0.4634940195293576, 1, 1], 
       Hue[0.5603649207077858, 1, 1], 
       Hue[0.5603649207077858, 1, 1], 
       Hue[0.6519246304843664, 1, 1], 
       Hue[0.6519246304843664, 1, 1], 
       Hue[0.7495382230954826, 1, 1], 
       Hue[0.7495382230954826, 1, 1], 
       Hue[0.8643080095757046, 1, 1], 
       Hue[0.8643080095757046, 1, 1], 
       Hue[0.9906253074833521, 1, 1], 
       Hue[0.9906253074833521, 1, 1], 
       Hue[0.10779399475058227`, 1, 1], 
       Hue[0.10779399475058227`, 1, 1], 
       Hue[0.1661984781166469, 1, 1], 
       Hue[0.1661984781166469, 1, 1], 
       Hue[0.2013592736498602, 1, 1], 
       Hue[0.2013592736498602, 1, 1], 
       Hue[0.32046871065631627`, 1, 1], 
       Hue[0.32046871065631627`, 1, 1], 
       Hue[0.4756075296246733, 1, 1], 
       Hue[0.4756075296246733, 1, 1], 
       Hue[0.6385764348360206, 1, 1], 
       Hue[0.6385764348360206, 1, 1], 
       Hue[0.7869345435130637, 1, 1], 
       Hue[0.7869345435130637, 1, 1], 
       Hue[0.8313036715488447, 1, 1], 
       Hue[0.8313036715488447, 1, 1], 
       Hue[0.8287660083714148, 1, 1], 
       Hue[0.8287660083714148, 1, 1], 
       Hue[0.9760417001103902, 1, 1], 
       Hue[0.9760417001103902, 1, 1], 
       Hue[0.17203733354249606`, 1, 1], 
       Hue[0.17203733354249606`, 1, 1], 
       Hue[0.41546928442185527`, 1, 1], 
       Hue[0.41546928442185527`, 1, 1], 
       Hue[0.7130492258316945, 1, 1], 
       Hue[0.7130492258316945, 1, 1], 
       Hue[0.013352993052113508`, 1, 1], 
       Hue[0.013352993052113508`, 1, 1], 
       Hue[0.2865809602909467, 1, 1], 
       Hue[0.2865809602909467, 1, 1], 
       Hue[0.5485481811412078, 1, 1], 
       Hue[0.5485481811412078, 1, 1], 
       Hue[0.8100483978103337, 1, 1], 
       Hue[0.8100483978103337, 1, 1], 
       Hue[0.07438821094553806, 1, 1], 
       Hue[0.07438821094553806, 1, 1], 
       Hue[0.34079459211773067`, 1, 1], 
       Hue[0.34079459211773067`, 1, 1], 
       Hue[0.6067300720056761, 1, 1], 
       Hue[0.6067300720056761, 1, 1], 
       Hue[0.8699265236196507, 1, 1], 
       Hue[0.8699265236196507, 1, 1], 
       Hue[0.12928216875586565`, 1, 1], 
       Hue[0.12928216875586565`, 1, 1], 
       Hue[0.3844879401823965, 1, 1], 
       Hue[0.3844879401823965, 1, 1], 
       Hue[0.6353011644887285, 1, 1], 
       Hue[0.6353011644887285, 1, 1], 
       Hue[0.8810951367672261, 1, 1], 
       Hue[0.8810951367672261, 1, 1], 
       Hue[0.12083724148470129`, 1, 1], 
       Hue[0.12083724148470129`, 1, 1], 
       Hue[0.35359810181285867`, 1, 1], 
       Hue[0.35359810181285867`, 1, 1], 
       Hue[0.5797037414262387, 1, 1], 
       Hue[0.5797037414262387, 1, 1], 
       Hue[0.801927284818749, 1, 1], 
       Hue[0.801927284818749, 1, 1], 
       Hue[0.02490207809077299, 1, 1], 
       Hue[0.02490207809077299, 1, 1], 
       Hue[0.2524952787854635, 1, 1], 
       Hue[0.2524952787854635, 1, 1], 
       Hue[0.4861575156782732, 1, 1], 
       Hue[0.4861575156782732, 1, 1], 
       Hue[0.7255279616649843, 1, 1], 
       Hue[0.7255279616649843, 1, 1], 
       Hue[0.9696088981681895, 1, 1], 
       Hue[0.9696088981681895, 1, 1], 
       Hue[0.21741244056167142`, 1, 1], 
       Hue[0.21741244056167142`, 1, 1], 
       Hue[0.46813755920083117`, 1, 1], 
       Hue[0.46813755920083117`, 1, 1], 
       Hue[0.7211432166767873, 1, 1], 
       Hue[0.7211432166767873, 1, 1], 
       Hue[0.975875284824123, 1, 1], 
       Hue[0.975875284824123, 1, 1], 
       Hue[0.23181597238659138`, 1, 1], 
       Hue[0.23181597238659138`, 1, 1], 
       Hue[0.48848723203835615`, 1, 1], 
       Hue[0.48848723203835615`, 1, 1], 
       Hue[0.745497138351656, 1, 1], 
       Hue[0.745497138351656, 1, 1], 
       Hue[0.0025923563209303813`, 1, 1], 
       Hue[0.0025923563209303813`, 1, 1], 
       Hue[0.25967466350231044`, 1, 1], 
       Hue[0.25967466350231044`, 1, 1], 
       Hue[0.5167645910330184, 1, 1], 
       Hue[0.5167645910330184, 1, 1], 
       Hue[0.7739220318465841, 1, 1], 
       Hue[0.7739220318465841, 1, 1], 
       Hue[0.031150257131776243`, 1, 1], 
       Hue[0.031150257131776243`, 1, 1], 
       Hue[0.2883194585058068, 1, 1], 
       Hue[0.2883194585058068, 1, 1], 
       Hue[0.5451448055773551, 1, 1], 
       Hue[0.5451448055773551, 1, 1], 
       Hue[0.801241512943925, 1, 1], 
       Hue[0.801241512943925, 1, 1], 
       Hue[0.05625312288155043, 1, 1], 
       Hue[0.05625312288155043, 1, 1], 
       Hue[0.31002956423589995`, 1, 1], 
       Hue[0.31002956423589995`, 1, 1], 
       Hue[0.5628570448388254, 1, 1], 
       Hue[0.5628570448388254, 1, 1], 
       Hue[0.8158451729427638, 1, 1], 
       Hue[0.8158451729427638, 1, 1], 
       Hue[0.07184273217758473, 1, 1], 
       Hue[0.07184273217758473, 1, 1], 
       Hue[0.33804700588730185`, 1, 1], 
       Hue[0.33804700588730185`, 1, 1], 
       Hue[0.6330401547699546, 1, 1], 
       Hue[0.6330401547699546, 1, 1], 
       Hue[0.9793681228383752, 1, 1], 
       Hue[0.9793681228383752, 1, 1], 
       Hue[0.31651445767840364`, 1, 1], 
       Hue[0.31651445767840364`, 1, 1], 
       Hue[0.6077541122814277, 1, 1], 
       Hue[0.6077541122814277, 1, 1], 
       Hue[0.8802124139644494, 1, 1], 
       Hue[0.8802124139644494, 1, 1], 
       Hue[0.14828103651834512`, 1, 1], 
       Hue[0.14828103651834512`, 1, 1], 
       Hue[0.4157813282383809, 1, 1], 
       Hue[0.4157813282383809, 1, 1], 
       Hue[0.6821139026892058, 1, 1], 
       Hue[0.6821139026892058, 1, 1], 
       Hue[0.9456040737426483, 1, 1], 
       Hue[0.9456040737426483, 1, 1], 
       Hue[0.20528754302196955`, 1, 1], 
       Hue[0.20528754302196955`, 1, 1], 
       Hue[0.46120974183242636`, 1, 1], 
       Hue[0.46120974183242636`, 1, 1], 
       Hue[0.7139993425376341, 1, 1], 
       Hue[0.7139993425376341, 1, 1], 
       Hue[0.9644555293237684, 1, 1], 
       Hue[0.9644555293237684, 1, 1], 
       Hue[0.2133525195580862, 1, 1], 
       Hue[0.2133525195580862, 1, 1]}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->{True, None},
    AxesOrigin->{0, 0},
    Epilog->
     LineBox[{{1, 1.3578616670247881`*^-6}, {2, 1.7875241658911324`*^-6}, {
       3, 1.3103993465932616`*^-6}, {4, 1.8216579940536379`*^-6}, {
       5, 1.5182456124162612`*^-6}, {6, 1.5915857528156287`*^-6}, {
       7, 1.6562529840457176`*^-6}, {8, 1.6155346667930268`*^-6}, {
       9, 1.6093333732317148`*^-6}, {10, 1.81716422545438*^-6}, {
       11, 2.241367317592486*^-6}, {12, 2.300923295728431*^-6}, {
       13, 2.0346675200493288`*^-6}, {14, 1.9750831096706663`*^-6}, {
       15, 2.225434568468253*^-6}, {16, 1.935483230855609*^-6}, {
       17, 2.1734845906928817`*^-6}, {18, 2.210747147632964*^-6}, {
       19, 2.3999074824891674`*^-6}, {20, 1.709320260604519*^-6}, {
       21, 1.2232913505398706`*^-6}, {22, 1.2807414685936205`*^-6}, {
       23, 2.4656133378253376`*^-6}, {24, 2.6317470425984405`*^-6}, {
       25, 2.73191883772434*^-6}, {26, 2.161439319185387*^-6}, {
       27, 2.9993781934814074`*^-6}, {28, 5.885392164480087*^-6}, {
       29, 9.140637439099608*^-6}, {30, 0.000012537549689778418`}, {
       31, 0.000015125531152692758`}, {32, 0.000019317912257021085`}, {
       33, 0.000027799915899672324`}, {34, 0.00004062888695791183}, {
       35, 0.00005493732462684299}, {36, 0.0000734364992144021}, {
       37, 0.00009658428414418722}, {38, 0.00012796459911777018`}, {
       39, 0.000170813089428751}, {40, 0.00022696971291477682`}, {
       41, 0.000299636428986054}, {42, 0.0003910582358853813}, {
       43, 0.0005084328649574035}, {44, 0.0006601813774843741}, {
       45, 0.0008533502212744484}, {46, 0.0010973566411053204`}, {
       47, 0.0014042731783203211`}, {48, 0.001786221219027729}, {
       49, 0.0022622360056601017`}, {50, 0.0028531421052515322`}, {
       51, 0.003581392976911822}, {52, 0.004473592560734086}, {
       53, 0.005560497943693344}, {54, 0.006878054778466725}, {
       55, 0.008468844177113395}, {56, 0.010378608142441953`}, {
       57, 0.01265858088407264}, {58, 0.01536651990372765}, {
       59, 0.018563495254488412`}, {60, 0.022318225943976984`}, {
       61, 0.02670515095385385}, {62, 0.03180287667558553}, {
       63, 0.037692974885989126`}, {64, 0.04446019492143582}, {
       65, 0.05219142104214091}, {66, 0.06097357503607669}, {
       67, 0.07089558283135813}, {68, 0.08203792902250882}, {
       69, 0.09447815368599492}, {70, 0.10828358624555064`}, {
       71, 0.123512893412521}, {72, 0.1402095530049863}, {
       73, 0.1584024435663279}, {74, 0.17809894639984414`}, {
       75, 0.19928566276647516`}, {76, 0.22192477190074908`}, {
       77, 0.24595194505811113`}, {78, 0.2712737623793337}, {
       79, 0.2977683872323064}, {80, 0.3252825318464745}, {
       81, 0.35363557733727796`}, {82, 0.3826182999584437}, {
       83, 0.4119903692221059}, {84, 0.4414879086848425}, {
       85, 0.470827754776116}, {86, 0.49970964595311035`}, {
       87, 0.5278201645933404}, {88, 0.5548366243419628}, {
       89, 0.580437763650691}, {90, 0.6043074113454318}, {
       91, 0.6261409988330071}, {92, 0.6456514554388889}, {
       93, 0.6625749076950869}, {94, 0.6766801964750884}, {
       95, 0.687771076988495}, {96, 0.6956898827986803}, {
       97, 0.7003218973983162}, {98, 0.7015987452623819}, {
       99, 0.6995035557496568}, {100, 0.6940614186036079}, {
       101, 0.6853459264951608}, {102, 0.6734791966567298}, {
       103, 0.658621962544615}, {104, 0.6409760879553257}, {
       105, 0.6207734769911902}, {106, 0.5982750040816684}, {
       107, 0.5737647870662654}, {108, 0.5475430917750653}, {
       109, 0.5199189419161926}, {110, 0.4912039516155621}, {
       111, 0.4617090541035805}, {112, 0.4317386345853279}, {
       113, 0.40158017517091327`}, {114, 0.3715076168379809}, {
       115, 0.3417749018112787}, {116, 0.31261349515941717`}, {
       117, 0.2842295081237318}, {118, 0.25680583426915427`}, {
       119, 0.23050129959149365`}, {120, 0.20545165508947386`}, {
       121, 0.18176941902956462`}, {122, 0.15954718950965407`}, {
       123, 0.13885984045994407`}, {124, 0.1197651230651645}, {
       125, 0.10230357089374113`}, {126, 0.08649928792914117}, {
       127, 0.07235827507074691}, {128, 0.05986317239481794}, {
       129, 0.048969742650728634`}, {130, 0.039606133182448336`}, {
       131, 0.03167408322519318}, {132, 0.025053002326810153`}, {
       133, 0.019605476376695116`}, {134, 0.01518553197705968}, {
       135, 0.011646988506975207`}, {136, 0.008850222590473791}, {
       137, 0.006666504517610634}, {138, 0.004980372895791658}, {
       139, 0.003691677467039283}, {140, 0.0027162303507356615`}, {
       141, 0.0019846048042396606`}, {142, 0.0014406680748210495`}, {
       143, 0.0010396299889217052`}, {144, 0.0007458456831020373}, {
       145, 0.0005318076067174114}, {146, 0.00037710631583470663`}, {
       147, 0.0002662201787648565}, {148, 0.00018698485377092197`}, {
       149, 0.0001305203043121926}, {150, 0.00009068959598857076}, {
       151, 0.0000627773097781486}, {152, 0.000043106948920869545`}, {
       153, 0.000029353360974000126`}, {154, 0.000020034389591908615`}, {
       155, 0.000013722976025379663`}, {156, 9.232758252075829*^-6}, {
       157, 6.069303133707478*^-6}, {158, 4.102082951343716*^-6}, {
       159, 2.960733311087227*^-6}, {160, 2.11683677966345*^-6}, {
       161, 1.3455872632310667`*^-6}, {162, 7.057570641144632*^-7}, {
       163, 3.4137138754889196`*^-7}, {164, 2.977709987555004*^-7}, {
       165, 2.9413439060320487`*^-7}, {166, 2.279128080971359*^-7}, {
       167, 1.3669536634937134`*^-7}, {168, 6.006314143393278*^-8}, {
       169, 1.971630225127637*^-8}, {170, 2.221893726538528*^-8}, {
       171, 2.539935454267735*^-8}, {172, 2.232795126757919*^-8}, {
       173, 1.7191890733676927`*^-8}, {174, 1.4736212748019013`*^-8}, {
       175, 1.6669375500447787`*^-8}, {176, 1.9242945537034262`*^-8}, {
       177, 2.0007141834995885`*^-8}, {178, 1.891303765524477*^-8}, {
       179, 1.6775224667260526`*^-8}, {180, 1.4328663110985998`*^-8}, {
       181, 1.1926660424116892`*^-8}, {182, 9.662128318034177*^-9}, {
       183, 7.572349252150447*^-9}, {184, 5.718835914263641*^-9}, {
       185, 4.1648231855897414`*^-9}, {186, 2.9401470573357896`*^-9}, {
       187, 2.0323836352054213`*^-9}, {188, 1.3982041579841351`*^-9}, {
       189, 9.78824556477881*^-10}, {190, 7.117852341948728*^-10}, {
       191, 5.405773407421047*^-10}, {192, 4.231994735814963*^-10}, {
       193, 3.3492866075375823`*^-10}, {194, 2.6408642276844017`*^-10}, {
       195, 2.0591097065050514`*^-10}, {196, 1.5835781028446087`*^-10}, {
       197, 1.2012255869962224`*^-10}, {198, 8.996850196999382*^-11}, {
       199, 6.661255499962208*^-11}, {200, 4.878971711098*^-11}, {
       201, 3.5344813557897254`*^-11}, {202, 2.5295063913191857`*^-11}, {
       203, 1.7849693821885105`*^-11}, {204, 1.2393415426388838`*^-11}, {
       205, 8.451361843885001*^-12}, {206, 5.653391888322103*^-12}, {
       207, 3.707196895984505*^-12}, {208, 2.3817498112488497`*^-12}, {
       209, 1.4975256882161408`*^-12}, {210, 9.19255175704502*^-13}, {
       211, 5.486164926513217*^-13}, {212, 3.163935523475936*^-13}, {
       213, 1.7491807169023416`*^-13}, {214, 9.176024729416635*^-14}, {
       215, 4.5074835322737055`*^-14}, {216, 2.0425273096470337`*^-14}, {
       217, 8.683533237441414*^-15}, {218, 4.458770092149619*^-15}, {
       219, 3.658535436267328*^-15}, {220, 3.3309519957799806`*^-15}, {
       221, 2.878677603272283*^-15}, {222, 2.3890902074055295`*^-15}, {
       223, 1.949497394214547*^-15}, {224, 1.5820279836881724`*^-15}, {
       225, 1.2738135802945798`*^-15}, {226, 1.0079058960944382`*^-15}, {
       227, 7.76197669192485*^-16}, {228, 5.781262065644672*^-16}, {
       229, 4.1523156990714817`*^-16}, {230, 2.8741102700943933`*^-16}, {
       231, 1.918665072443445*^-16}, {232, 1.23734807678485*^-16}, {
       233, 7.726864601608759*^-17}, {234, 4.687384324679332*^-17}, {
       235, 2.774965603840204*^-17}, {236, 1.613999563539118*^-17}, {
       237, 9.313541658879721*^-18}, {238, 5.401540449087361*^-18}, {
       239, 3.192373235958878*^-18}, {240, 1.9406722239128857`*^-18}, {
       241, 1.2134259763425004`*^-18}, {242, 7.736389261906779*^-19}, {
       243, 4.970535617599421*^-19}, {244, 3.1853882222610627`*^-19}, {
       245, 2.021750204960456*^-19}, {246, 1.2652925461690181`*^-19}, {
       247, 7.788315533966009*^-20}, {248, 4.708293883442487*^-20}, {
       249, 2.793320747395414*^-20}, {250, 1.625792464700552*^-20}, {
       251, 9.282592651594108*^-21}}],
    Frame->True,
    FrameTicks->{{{1., 
        FormBox[
         RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
        FormBox[
         RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
        FormBox[
         RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
        FormBox[
         RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
        FormBox["0", TraditionalForm]}, {157.25, 
        FormBox["2.5`", TraditionalForm]}, {188.5, 
        FormBox["5", TraditionalForm]}, {219.75, 
        FormBox["7.5`", TraditionalForm]}, {251., 
        FormBox["10", TraditionalForm]}}, Automatic, None, None},
    PlotRange->{-0.1, 1.1},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], Automatic}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF2XlUlIUax/HXBQVvKAaVWzYSoJQbyCaB/JBVRIEBZIcZBphh1euKqN2X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     "], {
      {GrayLevel[0.5], EdgeForm[None], 
       GraphicsGroupBox[{PolygonBox[{{249, 748, 250}}], 
         PolygonBox[CompressedData["
1:eJwtl3n0FkIUhu9MSqi0KO3KkgrJFpL2HZUiIUUKLSpFu7IUJVJEVEgokrSQ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          "]]}]}, {{}, {}, {
        {Hue[0.3607696028161082, 1, 1], LineBox[{1, 2}]}, 
        {Hue[0.267057753080544, 1, 1], LineBox[{2, 3}]}, 
        {Hue[0.1686832503326415, 1, 1], LineBox[{3, 4}]}, 
        {Hue[0.0839902357583648, 1, 1], LineBox[{4, 5}]}, 
        {Hue[0.9997109477118303, 1, 1], LineBox[{5, 6}]}, 
        {Hue[0.9203737430652636, 1, 1], LineBox[{6, 7}]}, 
        {Hue[0.8006315064242483, 1, 1], LineBox[{7, 8}]}, 
        {Hue[0.7290597125194981, 1, 1], LineBox[{8, 9}]}, 
        {Hue[0.6389748737056098, 1, 1], LineBox[{9, 10}]}, 
        {Hue[0.5422263215544533, 1, 1], LineBox[{10, 11}]}, 
        {Hue[0.43031212923343726`, 1, 1], LineBox[{11, 12}]}, 
        {Hue[0.362450423374641, 1, 1], LineBox[{12, 13}]}, 
        {Hue[0.2585024535212568, 1, 1], LineBox[{13, 14}]}, 
        {Hue[0.17167954532191643`, 1, 1], LineBox[{14, 15}]}, 
        {Hue[0.08752763848837851, 1, 1], LineBox[{15, 16}]}, 
        {Hue[0.9634484512814975, 1, 1], LineBox[{16, 17}]}, 
        {Hue[0.8125204942144689, 1, 1], LineBox[{17, 18}]}, 
        {Hue[0.6799926598771155, 1, 1], LineBox[{18, 19}]}, 
        {Hue[0.58093094876618, 1, 1], LineBox[{19, 20}]}, 
        {Hue[0.46481982190682386`, 1, 1], LineBox[{20, 21}]}, 
        {Hue[0.32700913340710674`, 1, 1], LineBox[{21, 22}]}, 
        {Hue[0.28037209140499164`, 1, 1], LineBox[{22, 23}]}, 
        {Hue[0.1675991623172561, 1, 1], LineBox[{23, 24}]}, 
        {Hue[0.07255009251719302, 1, 1], LineBox[{24, 25}]}, 
        {Hue[0.9911313410433275, 1, 1], LineBox[{25, 26}]}, 
        {Hue[0.8390801589542314, 1, 1], LineBox[{26, 27}]}, 
        {Hue[0.7723732432560815, 1, 1], LineBox[{27, 28}]}, 
        {Hue[0.7250092002080438, 1, 1], LineBox[{28, 29}]}, 
        {Hue[0.704030291923211, 1, 1], LineBox[{29, 30}]}, 
        {Hue[0.6606090358406091, 1, 1], LineBox[{30, 31}]}, 
        {Hue[0.18458027210925845`, 1, 1], LineBox[{31, 32}]}, 
        {Hue[0.20736414094701383`, 1, 1], LineBox[{32, 33}]}, 
        {Hue[0.19627293875574253`, 1, 1], LineBox[{33, 34}]}, 
        {Hue[0.22709226991981238`, 1, 1], LineBox[{34, 35}]}, 
        {Hue[0.309336981052953, 1, 1], LineBox[{35, 36}]}, 
        {Hue[0.38668987546377964`, 1, 1], LineBox[{36, 37}]}, 
        {Hue[0.4580886691838912, 1, 1], LineBox[{37, 38}]}, 
        {Hue[0.527587973755167, 1, 1], LineBox[{38, 39}]}, 
        {Hue[0.5843741913273628, 1, 1], LineBox[{39, 40}]}, 
        {Hue[0.6529738372754882, 1, 1], LineBox[{40, 41}]}, 
        {Hue[0.7231717946712797, 1, 1], LineBox[{41, 42}]}, 
        {Hue[0.7940198162849148, 1, 1], LineBox[{42, 43}]}, 
        {Hue[0.8670233124123604, 1, 1], LineBox[{43, 44}]}, 
        {Hue[0.9376088157986627, 1, 1], LineBox[{44, 45}]}, 
        {Hue[0.008213234289735359, 1, 1], LineBox[{45, 46}]}, 
        {Hue[0.07954226466320549, 1, 1], LineBox[{46, 47}]}, 
        {Hue[0.15182109711666714`, 1, 1], LineBox[{47, 48}]}, 
        {Hue[0.22507428783865907`, 1, 1], LineBox[{48, 49}]}, 
        {Hue[0.298601788645345, 1, 1], LineBox[{49, 50}]}, 
        {Hue[0.37261978757492364`, 1, 1], LineBox[{50, 51}]}, 
        {Hue[0.4464695067172625, 1, 1], LineBox[{51, 52}]}, 
        {Hue[0.520985703519601, 1, 1], LineBox[{52, 53}]}, 
        {Hue[0.5962503222699094, 1, 1], LineBox[{53, 54}]}, 
        {Hue[0.6720211162908327, 1, 1], LineBox[{54, 55}]}, 
        {Hue[0.7482495212329322, 1, 1], LineBox[{55, 56}]}, 
        {Hue[0.8250046849443391, 1, 1], LineBox[{56, 57}]}, 
        {Hue[0.9021656385273559, 1, 1], LineBox[{57, 58}]}, 
        {Hue[0.9798307401607916, 1, 1], LineBox[{58, 59}]}, 
        {Hue[0.05800235227659842, 1, 1], LineBox[{59, 60}]}, 
        {Hue[0.13669273281546193`, 1, 1], LineBox[{60, 61}]}, 
        {Hue[0.21585977065513692`, 1, 1], LineBox[{61, 62}]}, 
        {Hue[0.29548816845309706`, 1, 1], LineBox[{62, 63}]}, 
        {Hue[0.37560341603106856`, 1, 1], LineBox[{63, 64}]}, 
        {Hue[0.4562040007508113, 1, 1], LineBox[{64, 65}]}, 
        {Hue[0.5373038430435617, 1, 1], LineBox[{65, 66}]}, 
        {Hue[0.6188951496447265, 1, 1], LineBox[{66, 67}]}, 
        {Hue[0.7009511042349899, 1, 1], LineBox[{67, 68}]}, 
        {Hue[0.7835073631502996, 1, 1], LineBox[{68, 69}]}, 
        {Hue[0.8665536235626842, 1, 1], LineBox[{69, 70}]}, 
        {Hue[0.9500892860673488, 1, 1], LineBox[{70, 71}]}, 
        {Hue[0.034106981671226305`, 1, 1], LineBox[{71, 72}]}, 
        {Hue[0.11860815259491314`, 1, 1], LineBox[{72, 73}]}, 
        {Hue[0.2035904553821011, 1, 1], LineBox[{73, 74}]}, 
        {Hue[0.28906409794720567`, 1, 1], LineBox[{74, 75}]}, 
        {Hue[0.37502184323541427`, 1, 1], LineBox[{75, 76}]}, 
        {Hue[0.4614619759364616, 1, 1], LineBox[{76, 77}]}, 
        {Hue[0.5483815239488417, 1, 1], LineBox[{77, 78}]}, 
        {Hue[0.63578442230926, 1, 1], LineBox[{78, 79}]}, 
        {Hue[0.7236676785821858, 1, 1], LineBox[{79, 80}]}, 
        {Hue[0.8120314596811757, 1, 1], LineBox[{80, 81}]}, 
        {Hue[0.900873601632316, 1, 1], LineBox[{81, 82}]}, 
        {Hue[0.9901942213417271, 1, 1], LineBox[{82, 83}]}, 
        {Hue[0.0799895702292984, 1, 1], LineBox[{83, 84}]}, 
        {Hue[0.170260706962758, 1, 1], LineBox[{84, 85}]}, 
        {Hue[0.2610034850012405, 1, 1], LineBox[{85, 86}]}, 
        {Hue[0.35221637057648686`, 1, 1], LineBox[{86, 87}]}, 
        {Hue[0.44389826161878776`, 1, 1], LineBox[{87, 88}]}, 
        {Hue[0.5360449736481047, 1, 1], LineBox[{88, 89}]}, 
        {Hue[0.6286541129036205, 1, 1], LineBox[{89, 90}]}, 
        {Hue[0.7217217985068132, 1, 1], LineBox[{90, 91}]}, 
        {Hue[0.8152433603000737, 1, 1], LineBox[{91, 92}]}, 
        {Hue[0.9092139707072192, 1, 1], LineBox[{92, 93}]}, 
        {Hue[0.0036290315923241745`, 1, 1], LineBox[{93, 94}]}, 
        {Hue[0.09848073066420522, 1, 1], LineBox[{94, 95}]}, 
        {Hue[0.19376227185500616`, 1, 1], LineBox[{95, 96}]}, 
        {Hue[0.289464672794272, 1, 1], LineBox[{96, 97}]}, 
        {Hue[0.3855781333910314, 1, 1], LineBox[{97, 98}]}, 
        {Hue[0.4820908974496971, 1, 1], LineBox[{98, 99}]}, 
        {Hue[0.5789897695238124, 1, 1], LineBox[{99, 100}]}, 
        {Hue[0.6762580878511103, 1, 1], LineBox[{100, 101}]}, 
        {Hue[0.7738792026198127, 1, 1], LineBox[{101, 102}]}, 
        {Hue[0.8718316149733819, 1, 1], LineBox[{102, 103}]}, 
        {Hue[0.9700914448448266, 1, 1], LineBox[{103, 104}]}, 
        {Hue[0.06863078135729278, 1, 1], LineBox[{104, 105}]}, 
        {Hue[0.1674173699230757, 1, 1], LineBox[{105, 106}]}, 
        {Hue[0.2664150890214937, 1, 1], LineBox[{106, 107}]}, 
        {Hue[0.36558201499268755`, 1, 1], LineBox[{107, 108}]}, 
        {Hue[0.4648696110448278, 1, 1], LineBox[{108, 109}]}, 
        {Hue[0.5642232715551221, 1, 1], LineBox[{109, 110}]}, 
        {Hue[0.6635811232888242, 1, 1], LineBox[{110, 111}]}, 
        {Hue[0.7628731825198262, 1, 1], LineBox[{111, 112}]}, 
        {Hue[0.8620209439738609, 1, 1], LineBox[{112, 113}]}, 
        {Hue[0.960936178647772, 1, 1], LineBox[{113, 114}]}, 
        {Hue[0.05952243736322674, 1, 1], LineBox[{114, 115}]}, 
        {Hue[0.1576739410013476, 1, 1], LineBox[{115, 116}]}, 
        {Hue[0.25527613254844134`, 1, 1], LineBox[{116, 117}]}, 
        {Hue[0.3522059883891288, 1, 1], LineBox[{117, 118}]}, 
        {Hue[0.4483344792343556, 1, 1], LineBox[{118, 119}]}, 
        {Hue[0.5435302454469617, 1, 1], LineBox[{119, 120}]}, 
        {Hue[0.6376602769840898, 1, 1], LineBox[{120, 121}]}, 
        {Hue[0.7305964827538581, 1, 1], LineBox[{121, 122}]}, 
        {Hue[0.822219008183593, 1, 1], LineBox[{122, 123}]}, 
        {Hue[0.9124241755271605, 1, 1], LineBox[{123, 124}]}, 
        {Hue[0.0011313352912279788`, 1, 1], LineBox[{124, 125}]}, 
        {Hue[0.08829537377171129, 1, 1], LineBox[{125, 126}]}, 
        {Hue[0.173914091059342, 1, 1], LineBox[{126, 127}]}, 
        {Hue[0.2580327570304501, 1, 1], LineBox[{127, 128}]}, 
        {Hue[0.34074391743273624`, 1, 1], LineBox[{128, 129}]}, 
        {Hue[0.42218350829950074`, 1, 1], LineBox[{129, 130}]}, 
        {Hue[0.5025176745363464, 1, 1], LineBox[{130, 131}]}, 
        {Hue[0.5819340841751708, 1, 1], LineBox[{131, 132}]}, 
        {Hue[0.6606322089404484, 1, 1], LineBox[{132, 133}]}, 
        {Hue[0.7388112677112173, 1, 1], LineBox[{133, 134}]}, 
        {Hue[0.8166629771094296, 1, 1], LineBox[{134, 135}]}, 
        {Hue[0.894367291049327, 1, 1], LineBox[{135, 136}]}, 
        {Hue[0.9720925084202446, 1, 1], LineBox[{136, 137}]}, 
        {Hue[0.04998896335048285, 1, 1], LineBox[{137, 138}]}, 
        {Hue[0.1281877525899212, 1, 1], LineBox[{138, 139}]}, 
        {Hue[0.20680136452066197`, 1, 1], LineBox[{139, 140}]}, 
        {Hue[0.28593303904703327`, 1, 1], LineBox[{140, 141}]}, 
        {Hue[0.36567016562670085`, 1, 1], LineBox[{141, 142}]}, 
        {Hue[0.44607683475779675`, 1, 1], LineBox[{142, 143}]}, 
        {Hue[0.5272080444436378, 1, 1], LineBox[{143, 144}]}, 
        {Hue[0.6091162047792973, 1, 1], LineBox[{144, 145}]}, 
        {Hue[0.691847083043619, 1, 1], LineBox[{145, 146}]}, 
        {Hue[0.7754258035423244, 1, 1], LineBox[{146, 147}]}, 
        {Hue[0.8598681254734846, 1, 1], LineBox[{147, 148}]}, 
        {Hue[0.9452056790005225, 1, 1], LineBox[{148, 149}]}, 
        {Hue[0.03147188112713823, 1, 1], LineBox[{149, 150}]}, 
        {Hue[0.11867304215521769`, 1, 1], LineBox[{150, 151}]}, 
        {Hue[0.20676513826419657`, 1, 1], LineBox[{151, 152}]}, 
        {Hue[0.2956907086256168, 1, 1], LineBox[{152, 153}]}, 
        {Hue[0.3854998251506179, 1, 1], LineBox[{153, 154}]}, 
        {Hue[0.4763031405657942, 1, 1], LineBox[{154, 155}]}, 
        {Hue[0.5680698599260738, 1, 1], LineBox[{155, 156}]}, 
        {Hue[0.6606728731331002, 1, 1], LineBox[{156, 157}]}, 
        {Hue[0.754025841465475, 1, 1], LineBox[{157, 158}]}, 
        {Hue[0.8480804931757541, 1, 1], LineBox[{158, 159}]}, 
        {Hue[0.9431181536174291, 1, 1], LineBox[{159, 160}]}, 
        {Hue[0.0397869542393286, 1, 1], LineBox[{160, 161}]}, 
        {Hue[0.137840700355691, 1, 1], LineBox[{161, 162}]}, 
        {Hue[0.23574526072148794`, 1, 1], LineBox[{162, 163}]}, 
        {Hue[0.333496700037884, 1, 1], LineBox[{163, 164}]}, 
        {Hue[0.43380775123773196`, 1, 1], LineBox[{164, 165}]}, 
        {Hue[0.536904103339749, 1, 1], LineBox[{165, 166}]}, 
        {Hue[0.6388111314179277, 1, 1], LineBox[{166, 167}]}, 
        {Hue[0.7395416522567456, 1, 1], LineBox[{167, 168}]}, 
        {Hue[0.8447363652466423, 1, 1], LineBox[{168, 169}]}, 
        {Hue[0.9501147795477853, 1, 1], LineBox[{169, 170}]}, 
        {Hue[0.04045361918740696, 1, 1], LineBox[{170, 171}]}, 
        {Hue[0.1286078858347576, 1, 1], LineBox[{171, 172}]}, 
        {Hue[0.24672799717042326`, 1, 1], LineBox[{172, 173}]}, 
        {Hue[0.37861522354390637`, 1, 1], LineBox[{173, 174}]}, 
        {Hue[0.45736632935843075`, 1, 1], LineBox[{174, 175}]}, 
        {Hue[0.49308730974207005`, 1, 1], LineBox[{175, 176}]}, 
        {Hue[0.6470686768988838, 1, 1], LineBox[{176, 177}]}, 
        {Hue[0.8568677735491859, 1, 1], LineBox[{177, 178}]}, 
        {Hue[0.10431225644341774`, 1, 1], LineBox[{178, 179}]}, 
        {Hue[0.3894102905074262, 1, 1], LineBox[{179, 180}]}, 
        {Hue[0.6838263781448488, 1, 1], LineBox[{180, 181}]}, 
        {Hue[0.9569069071984381, 1, 1], LineBox[{181, 182}]}, 
        {Hue[0.21395671894896143`, 1, 1], LineBox[{182, 183}]}, 
        {Hue[0.4664485349712569, 1, 1], LineBox[{183, 184}]}, 
        {Hue[0.7219258772292536, 1, 1], LineBox[{184, 185}]}, 
        {Hue[0.9856372041087308, 1, 1], LineBox[{185, 186}]}, 
        {Hue[0.2608722601555482, 1, 1], LineBox[{186, 187}]}, 
        {Hue[0.5470837038038487, 1, 1], LineBox[{187, 188}]}, 
        {Hue[0.8401196088581602, 1, 1], LineBox[{188, 189}]}, 
        {Hue[0.13624437333826112`, 1, 1], LineBox[{189, 190}]}, 
        {Hue[0.4309682311873325, 1, 1], LineBox[{190, 191}]}, 
        {Hue[0.7184660058323185, 1, 1], LineBox[{191, 192}]}, 
        {Hue[0.9973525953613468, 1, 1], LineBox[{192, 193}]}, 
        {Hue[0.2702841777599323, 1, 1], LineBox[{193, 194}]}, 
        {Hue[0.5401052324973117, 1, 1], LineBox[{194, 195}]}, 
        {Hue[0.8084590354995888, 1, 1], LineBox[{195, 196}]}, 
        {Hue[0.07594959588080527, 1, 1], LineBox[{196, 197}]}, 
        {Hue[0.34257008552859086`, 1, 1], LineBox[{197, 198}]}, 
        {Hue[0.608041503369714, 1, 1], LineBox[{198, 199}]}, 
        {Hue[0.8720135963849045, 1, 1], LineBox[{199, 200}]}, 
        {Hue[0.13414354537958378`, 1, 1], LineBox[{200, 201}]}, 
        {Hue[0.3940928156514941, 1, 1], LineBox[{201, 202}]}, 
        {Hue[0.6514812971209296, 1, 1], LineBox[{202, 203}]}, 
        {Hue[0.9058301300769357, 1, 1], LineBox[{203, 204}]}, 
        {Hue[0.1565116040087216, 1, 1], LineBox[{204, 205}]}, 
        {Hue[0.4027360261695492, 1, 1], LineBox[{205, 206}]}, 
        {Hue[0.6436450448936091, 1, 1], LineBox[{206, 207}]}, 
        {Hue[0.8786518091264015, 1, 1], LineBox[{207, 208}]}, 
        {Hue[0.10814245304970796`, 1, 1], LineBox[{208, 209}]}, 
        {Hue[0.3342040087120603, 1, 1], LineBox[{209, 210}]}, 
        {Hue[0.5603417438283603, 1, 1], LineBox[{210, 211}]}, 
        {Hue[0.7898375376869549, 1, 1], LineBox[{211, 212}]}, 
        {Hue[0.02435059756972669, 1, 1], LineBox[{212, 213}]}, 
        {Hue[0.263972200373601, 1, 1], LineBox[{213, 214}]}, 
        {Hue[0.5080091112724794, 1, 1], LineBox[{214, 215}]}, 
        {Hue[0.7555873580958158, 1, 1], LineBox[{215, 216}]}, 
        {Hue[0.005910038993356071, 1, 1], LineBox[{216, 217}]}, 
        {Hue[0.2583208773049551, 1, 1], LineBox[{217, 218}]}, 
        {Hue[0.5122990607419959, 1, 1], LineBox[{218, 219}]}, 
        {Hue[0.7674399787929954, 1, 1], LineBox[{219, 220}]}, 
        {Hue[0.02343753155512082, 1, 1], LineBox[{220, 221}]}, 
        {Hue[0.2800651076356245, 1, 1], LineBox[{221, 222}]}, 
        {Hue[0.5371545059966867, 1, 1], LineBox[{222, 223}]}, 
        {Hue[0.7945733996717301, 1, 1], LineBox[{223, 224}]}, 
        {Hue[0.052204389580622186`, 1, 1], LineBox[{224, 225}]}, 
        {Hue[0.30993216728720485`, 1, 1], LineBox[{225, 226}]}, 
        {Hue[0.5676409258558874, 1, 1], LineBox[{226, 227}]}, 
        {Hue[0.8252245236518228, 1, 1], LineBox[{227, 228}]}, 
        {Hue[0.08260688618784486, 1, 1], LineBox[{228, 229}]}, 
        {Hue[0.3397704033682906, 1, 1], LineBox[{229, 230}]}, 
        {Hue[0.5967914666911367, 1, 1], LineBox[{230, 231}]}, 
        {Hue[0.8538866103988598, 1, 1], LineBox[{231, 232}]}, 
        {Hue[0.11148110441383434`, 1, 1], LineBox[{232, 233}]}, 
        {Hue[0.37032221044370817`, 1, 1], LineBox[{233, 234}]}, 
        {Hue[0.6316738642015586, 1, 1], LineBox[{234, 235}]}, 
        {Hue[0.8976395052931159, 1, 1], LineBox[{235, 236}]}, 
        {Hue[0.17158596189859066`, 1, 1], LineBox[{236, 237}]}, 
        {Hue[0.45806602624555387`, 1, 1], LineBox[{237, 238}]}, 
        {Hue[0.759613669603346, 1, 1], LineBox[{238, 239}]}, 
        {Hue[0.06876575145420652, 1, 1], LineBox[{239, 240}]}, 
        {Hue[0.3704520490070801, 1, 1], LineBox[{240, 241}]}, 
        {Hue[0.6581400003275106, 1, 1], LineBox[{241, 242}]}, 
        {Hue[0.9342542877440975, 1, 1], LineBox[{242, 243}]}, 
        {Hue[0.20258989419990772`, 1, 1], LineBox[{243, 244}]}, 
        {Hue[0.46580311190130813`, 1, 1], LineBox[{244, 245}]}, 
        {Hue[0.7255298424272546, 1, 1], LineBox[{245, 246}]}, 
        {Hue[0.9828031804668734, 1, 1], LineBox[{246, 247}]}, 
        {Hue[0.2383262257411595, 1, 1], LineBox[{247, 248}]}, 
        {Hue[0.4926179409024767, 1, 1], LineBox[{248, 249}]}, 
        {Hue[0.7460921021000613, 1, 1], LineBox[{249, 250}]}, 
        {Hue[0.9991008808619908, 1, 1], LineBox[{250, 251}]}}}},
     VertexColors->{
       Hue[0.3607696028161082, 1, 1], 
       Hue[0.267057753080544, 1, 1], 
       Hue[0.1686832503326415, 1, 1], 
       Hue[0.0839902357583648, 1, 1], 
       Hue[0.9997109477118303, 1, 1], 
       Hue[0.9203737430652636, 1, 1], 
       Hue[0.8006315064242483, 1, 1], 
       Hue[0.7290597125194981, 1, 1], 
       Hue[0.6389748737056098, 1, 1], 
       Hue[0.5422263215544533, 1, 1], 
       Hue[0.43031212923343726`, 1, 1], 
       Hue[0.362450423374641, 1, 1], 
       Hue[0.2585024535212568, 1, 1], 
       Hue[0.17167954532191643`, 1, 1], 
       Hue[0.08752763848837851, 1, 1], 
       Hue[0.9634484512814975, 1, 1], 
       Hue[0.8125204942144689, 1, 1], 
       Hue[0.6799926598771155, 1, 1], 
       Hue[0.58093094876618, 1, 1], 
       Hue[0.46481982190682386`, 1, 1], 
       Hue[0.32700913340710674`, 1, 1], 
       Hue[0.28037209140499164`, 1, 1], 
       Hue[0.1675991623172561, 1, 1], 
       Hue[0.07255009251719302, 1, 1], 
       Hue[0.9911313410433275, 1, 1], 
       Hue[0.8390801589542314, 1, 1], 
       Hue[0.7723732432560815, 1, 1], 
       Hue[0.7250092002080438, 1, 1], 
       Hue[0.704030291923211, 1, 1], 
       Hue[0.6606090358406091, 1, 1], 
       Hue[0.18458027210925845`, 1, 1], 
       Hue[0.20736414094701383`, 1, 1], 
       Hue[0.19627293875574253`, 1, 1], 
       Hue[0.22709226991981238`, 1, 1], 
       Hue[0.309336981052953, 1, 1], 
       Hue[0.38668987546377964`, 1, 1], 
       Hue[0.4580886691838912, 1, 1], 
       Hue[0.527587973755167, 1, 1], 
       Hue[0.5843741913273628, 1, 1], 
       Hue[0.6529738372754882, 1, 1], 
       Hue[0.7231717946712797, 1, 1], 
       Hue[0.7940198162849148, 1, 1], 
       Hue[0.8670233124123604, 1, 1], 
       Hue[0.9376088157986627, 1, 1], 
       Hue[0.008213234289735359, 1, 1], 
       Hue[0.07954226466320549, 1, 1], 
       Hue[0.15182109711666714`, 1, 1], 
       Hue[0.22507428783865907`, 1, 1], 
       Hue[0.298601788645345, 1, 1], 
       Hue[0.37261978757492364`, 1, 1], 
       Hue[0.4464695067172625, 1, 1], 
       Hue[0.520985703519601, 1, 1], 
       Hue[0.5962503222699094, 1, 1], 
       Hue[0.6720211162908327, 1, 1], 
       Hue[0.7482495212329322, 1, 1], 
       Hue[0.8250046849443391, 1, 1], 
       Hue[0.9021656385273559, 1, 1], 
       Hue[0.9798307401607916, 1, 1], 
       Hue[0.05800235227659842, 1, 1], 
       Hue[0.13669273281546193`, 1, 1], 
       Hue[0.21585977065513692`, 1, 1], 
       Hue[0.29548816845309706`, 1, 1], 
       Hue[0.37560341603106856`, 1, 1], 
       Hue[0.4562040007508113, 1, 1], 
       Hue[0.5373038430435617, 1, 1], 
       Hue[0.6188951496447265, 1, 1], 
       Hue[0.7009511042349899, 1, 1], 
       Hue[0.7835073631502996, 1, 1], 
       Hue[0.8665536235626842, 1, 1], 
       Hue[0.9500892860673488, 1, 1], 
       Hue[0.034106981671226305`, 1, 1], 
       Hue[0.11860815259491314`, 1, 1], 
       Hue[0.2035904553821011, 1, 1], 
       Hue[0.28906409794720567`, 1, 1], 
       Hue[0.37502184323541427`, 1, 1], 
       Hue[0.4614619759364616, 1, 1], 
       Hue[0.5483815239488417, 1, 1], 
       Hue[0.63578442230926, 1, 1], 
       Hue[0.7236676785821858, 1, 1], 
       Hue[0.8120314596811757, 1, 1], 
       Hue[0.900873601632316, 1, 1], 
       Hue[0.9901942213417271, 1, 1], 
       Hue[0.0799895702292984, 1, 1], 
       Hue[0.170260706962758, 1, 1], 
       Hue[0.2610034850012405, 1, 1], 
       Hue[0.35221637057648686`, 1, 1], 
       Hue[0.44389826161878776`, 1, 1], 
       Hue[0.5360449736481047, 1, 1], 
       Hue[0.6286541129036205, 1, 1], 
       Hue[0.7217217985068132, 1, 1], 
       Hue[0.8152433603000737, 1, 1], 
       Hue[0.9092139707072192, 1, 1], 
       Hue[0.0036290315923241745`, 1, 1], 
       Hue[0.09848073066420522, 1, 1], 
       Hue[0.19376227185500616`, 1, 1], 
       Hue[0.289464672794272, 1, 1], 
       Hue[0.3855781333910314, 1, 1], 
       Hue[0.4820908974496971, 1, 1], 
       Hue[0.5789897695238124, 1, 1], 
       Hue[0.6762580878511103, 1, 1], 
       Hue[0.7738792026198127, 1, 1], 
       Hue[0.8718316149733819, 1, 1], 
       Hue[0.9700914448448266, 1, 1], 
       Hue[0.06863078135729278, 1, 1], 
       Hue[0.1674173699230757, 1, 1], 
       Hue[0.2664150890214937, 1, 1], 
       Hue[0.36558201499268755`, 1, 1], 
       Hue[0.4648696110448278, 1, 1], 
       Hue[0.5642232715551221, 1, 1], 
       Hue[0.6635811232888242, 1, 1], 
       Hue[0.7628731825198262, 1, 1], 
       Hue[0.8620209439738609, 1, 1], 
       Hue[0.960936178647772, 1, 1], 
       Hue[0.05952243736322674, 1, 1], 
       Hue[0.1576739410013476, 1, 1], 
       Hue[0.25527613254844134`, 1, 1], 
       Hue[0.3522059883891288, 1, 1], 
       Hue[0.4483344792343556, 1, 1], 
       Hue[0.5435302454469617, 1, 1], 
       Hue[0.6376602769840898, 1, 1], 
       Hue[0.7305964827538581, 1, 1], 
       Hue[0.822219008183593, 1, 1], 
       Hue[0.9124241755271605, 1, 1], 
       Hue[0.0011313352912279788`, 1, 1], 
       Hue[0.08829537377171129, 1, 1], 
       Hue[0.173914091059342, 1, 1], 
       Hue[0.2580327570304501, 1, 1], 
       Hue[0.34074391743273624`, 1, 1], 
       Hue[0.42218350829950074`, 1, 1], 
       Hue[0.5025176745363464, 1, 1], 
       Hue[0.5819340841751708, 1, 1], 
       Hue[0.6606322089404484, 1, 1], 
       Hue[0.7388112677112173, 1, 1], 
       Hue[0.8166629771094296, 1, 1], 
       Hue[0.894367291049327, 1, 1], 
       Hue[0.9720925084202446, 1, 1], 
       Hue[0.04998896335048285, 1, 1], 
       Hue[0.1281877525899212, 1, 1], 
       Hue[0.20680136452066197`, 1, 1], 
       Hue[0.28593303904703327`, 1, 1], 
       Hue[0.36567016562670085`, 1, 1], 
       Hue[0.44607683475779675`, 1, 1], 
       Hue[0.5272080444436378, 1, 1], 
       Hue[0.6091162047792973, 1, 1], 
       Hue[0.691847083043619, 1, 1], 
       Hue[0.7754258035423244, 1, 1], 
       Hue[0.8598681254734846, 1, 1], 
       Hue[0.9452056790005225, 1, 1], 
       Hue[0.03147188112713823, 1, 1], 
       Hue[0.11867304215521769`, 1, 1], 
       Hue[0.20676513826419657`, 1, 1], 
       Hue[0.2956907086256168, 1, 1], 
       Hue[0.3854998251506179, 1, 1], 
       Hue[0.4763031405657942, 1, 1], 
       Hue[0.5680698599260738, 1, 1], 
       Hue[0.6606728731331002, 1, 1], 
       Hue[0.754025841465475, 1, 1], 
       Hue[0.8480804931757541, 1, 1], 
       Hue[0.9431181536174291, 1, 1], 
       Hue[0.0397869542393286, 1, 1], 
       Hue[0.137840700355691, 1, 1], 
       Hue[0.23574526072148794`, 1, 1], 
       Hue[0.333496700037884, 1, 1], 
       Hue[0.43380775123773196`, 1, 1], 
       Hue[0.536904103339749, 1, 1], 
       Hue[0.6388111314179277, 1, 1], 
       Hue[0.7395416522567456, 1, 1], 
       Hue[0.8447363652466423, 1, 1], 
       Hue[0.9501147795477853, 1, 1], 
       Hue[0.04045361918740696, 1, 1], 
       Hue[0.1286078858347576, 1, 1], 
       Hue[0.24672799717042326`, 1, 1], 
       Hue[0.37861522354390637`, 1, 1], 
       Hue[0.45736632935843075`, 1, 1], 
       Hue[0.49308730974207005`, 1, 1], 
       Hue[0.6470686768988838, 1, 1], 
       Hue[0.8568677735491859, 1, 1], 
       Hue[0.10431225644341774`, 1, 1], 
       Hue[0.3894102905074262, 1, 1], 
       Hue[0.6838263781448488, 1, 1], 
       Hue[0.9569069071984381, 1, 1], 
       Hue[0.21395671894896143`, 1, 1], 
       Hue[0.4664485349712569, 1, 1], 
       Hue[0.7219258772292536, 1, 1], 
       Hue[0.9856372041087308, 1, 1], 
       Hue[0.2608722601555482, 1, 1], 
       Hue[0.5470837038038487, 1, 1], 
       Hue[0.8401196088581602, 1, 1], 
       Hue[0.13624437333826112`, 1, 1], 
       Hue[0.4309682311873325, 1, 1], 
       Hue[0.7184660058323185, 1, 1], 
       Hue[0.9973525953613468, 1, 1], 
       Hue[0.2702841777599323, 1, 1], 
       Hue[0.5401052324973117, 1, 1], 
       Hue[0.8084590354995888, 1, 1], 
       Hue[0.07594959588080527, 1, 1], 
       Hue[0.34257008552859086`, 1, 1], 
       Hue[0.608041503369714, 1, 1], 
       Hue[0.8720135963849045, 1, 1], 
       Hue[0.13414354537958378`, 1, 1], 
       Hue[0.3940928156514941, 1, 1], 
       Hue[0.6514812971209296, 1, 1], 
       Hue[0.9058301300769357, 1, 1], 
       Hue[0.1565116040087216, 1, 1], 
       Hue[0.4027360261695492, 1, 1], 
       Hue[0.6436450448936091, 1, 1], 
       Hue[0.8786518091264015, 1, 1], 
       Hue[0.10814245304970796`, 1, 1], 
       Hue[0.3342040087120603, 1, 1], 
       Hue[0.5603417438283603, 1, 1], 
       Hue[0.7898375376869549, 1, 1], 
       Hue[0.02435059756972669, 1, 1], 
       Hue[0.263972200373601, 1, 1], 
       Hue[0.5080091112724794, 1, 1], 
       Hue[0.7555873580958158, 1, 1], 
       Hue[0.005910038993356071, 1, 1], 
       Hue[0.2583208773049551, 1, 1], 
       Hue[0.5122990607419959, 1, 1], 
       Hue[0.7674399787929954, 1, 1], 
       Hue[0.02343753155512082, 1, 1], 
       Hue[0.2800651076356245, 1, 1], 
       Hue[0.5371545059966867, 1, 1], 
       Hue[0.7945733996717301, 1, 1], 
       Hue[0.052204389580622186`, 1, 1], 
       Hue[0.30993216728720485`, 1, 1], 
       Hue[0.5676409258558874, 1, 1], 
       Hue[0.8252245236518228, 1, 1], 
       Hue[0.08260688618784486, 1, 1], 
       Hue[0.3397704033682906, 1, 1], 
       Hue[0.5967914666911367, 1, 1], 
       Hue[0.8538866103988598, 1, 1], 
       Hue[0.11148110441383434`, 1, 1], 
       Hue[0.37032221044370817`, 1, 1], 
       Hue[0.6316738642015586, 1, 1], 
       Hue[0.8976395052931159, 1, 1], 
       Hue[0.17158596189859066`, 1, 1], 
       Hue[0.45806602624555387`, 1, 1], 
       Hue[0.759613669603346, 1, 1], 
       Hue[0.06876575145420652, 1, 1], 
       Hue[0.3704520490070801, 1, 1], 
       Hue[0.6581400003275106, 1, 1], 
       Hue[0.9342542877440975, 1, 1], 
       Hue[0.20258989419990772`, 1, 1], 
       Hue[0.46580311190130813`, 1, 1], 
       Hue[0.7255298424272546, 1, 1], 
       Hue[0.9828031804668734, 1, 1], 
       Hue[0.2383262257411595, 1, 1], 
       Hue[0.4926179409024767, 1, 1], 
       Hue[0.7460921021000613, 1, 1], 
       Hue[0.9991008808619908, 1, 1], 
       Hue[0.25196147333959246`, 1, 1], 
       Hue[0.3607696028161082, 1, 1], 
       Hue[0.267057753080544, 1, 1], 
       Hue[0.267057753080544, 1, 1], 
       Hue[0.1686832503326415, 1, 1], 
       Hue[0.1686832503326415, 1, 1], 
       Hue[0.0839902357583648, 1, 1], 
       Hue[0.0839902357583648, 1, 1], 
       Hue[0.9997109477118303, 1, 1], 
       Hue[0.9997109477118303, 1, 1], 
       Hue[0.9203737430652636, 1, 1], 
       Hue[0.9203737430652636, 1, 1], 
       Hue[0.8006315064242483, 1, 1], 
       Hue[0.8006315064242483, 1, 1], 
       Hue[0.7290597125194981, 1, 1], 
       Hue[0.7290597125194981, 1, 1], 
       Hue[0.6389748737056098, 1, 1], 
       Hue[0.6389748737056098, 1, 1], 
       Hue[0.5422263215544533, 1, 1], 
       Hue[0.5422263215544533, 1, 1], 
       Hue[0.43031212923343726`, 1, 1], 
       Hue[0.43031212923343726`, 1, 1], 
       Hue[0.362450423374641, 1, 1], 
       Hue[0.362450423374641, 1, 1], 
       Hue[0.2585024535212568, 1, 1], 
       Hue[0.2585024535212568, 1, 1], 
       Hue[0.17167954532191643`, 1, 1], 
       Hue[0.17167954532191643`, 1, 1], 
       Hue[0.08752763848837851, 1, 1], 
       Hue[0.08752763848837851, 1, 1], 
       Hue[0.9634484512814975, 1, 1], 
       Hue[0.9634484512814975, 1, 1], 
       Hue[0.8125204942144689, 1, 1], 
       Hue[0.8125204942144689, 1, 1], 
       Hue[0.6799926598771155, 1, 1], 
       Hue[0.6799926598771155, 1, 1], 
       Hue[0.58093094876618, 1, 1], 
       Hue[0.58093094876618, 1, 1], 
       Hue[0.46481982190682386`, 1, 1], 
       Hue[0.46481982190682386`, 1, 1], 
       Hue[0.32700913340710674`, 1, 1], 
       Hue[0.32700913340710674`, 1, 1], 
       Hue[0.28037209140499164`, 1, 1], 
       Hue[0.28037209140499164`, 1, 1], 
       Hue[0.1675991623172561, 1, 1], 
       Hue[0.1675991623172561, 1, 1], 
       Hue[0.07255009251719302, 1, 1], 
       Hue[0.07255009251719302, 1, 1], 
       Hue[0.9911313410433275, 1, 1], 
       Hue[0.9911313410433275, 1, 1], 
       Hue[0.8390801589542314, 1, 1], 
       Hue[0.8390801589542314, 1, 1], 
       Hue[0.7723732432560815, 1, 1], 
       Hue[0.7723732432560815, 1, 1], 
       Hue[0.7250092002080438, 1, 1], 
       Hue[0.7250092002080438, 1, 1], 
       Hue[0.704030291923211, 1, 1], 
       Hue[0.704030291923211, 1, 1], 
       Hue[0.6606090358406091, 1, 1], 
       Hue[0.6606090358406091, 1, 1], 
       Hue[0.18458027210925845`, 1, 1], 
       Hue[0.18458027210925845`, 1, 1], 
       Hue[0.20736414094701383`, 1, 1], 
       Hue[0.20736414094701383`, 1, 1], 
       Hue[0.19627293875574253`, 1, 1], 
       Hue[0.19627293875574253`, 1, 1], 
       Hue[0.22709226991981238`, 1, 1], 
       Hue[0.22709226991981238`, 1, 1], 
       Hue[0.309336981052953, 1, 1], 
       Hue[0.309336981052953, 1, 1], 
       Hue[0.38668987546377964`, 1, 1], 
       Hue[0.38668987546377964`, 1, 1], 
       Hue[0.4580886691838912, 1, 1], 
       Hue[0.4580886691838912, 1, 1], 
       Hue[0.527587973755167, 1, 1], 
       Hue[0.527587973755167, 1, 1], 
       Hue[0.5843741913273628, 1, 1], 
       Hue[0.5843741913273628, 1, 1], 
       Hue[0.6529738372754882, 1, 1], 
       Hue[0.6529738372754882, 1, 1], 
       Hue[0.7231717946712797, 1, 1], 
       Hue[0.7231717946712797, 1, 1], 
       Hue[0.7940198162849148, 1, 1], 
       Hue[0.7940198162849148, 1, 1], 
       Hue[0.8670233124123604, 1, 1], 
       Hue[0.8670233124123604, 1, 1], 
       Hue[0.9376088157986627, 1, 1], 
       Hue[0.9376088157986627, 1, 1], 
       Hue[0.008213234289735359, 1, 1], 
       Hue[0.008213234289735359, 1, 1], 
       Hue[0.07954226466320549, 1, 1], 
       Hue[0.07954226466320549, 1, 1], 
       Hue[0.15182109711666714`, 1, 1], 
       Hue[0.15182109711666714`, 1, 1], 
       Hue[0.22507428783865907`, 1, 1], 
       Hue[0.22507428783865907`, 1, 1], 
       Hue[0.298601788645345, 1, 1], 
       Hue[0.298601788645345, 1, 1], 
       Hue[0.37261978757492364`, 1, 1], 
       Hue[0.37261978757492364`, 1, 1], 
       Hue[0.4464695067172625, 1, 1], 
       Hue[0.4464695067172625, 1, 1], 
       Hue[0.520985703519601, 1, 1], 
       Hue[0.520985703519601, 1, 1], 
       Hue[0.5962503222699094, 1, 1], 
       Hue[0.5962503222699094, 1, 1], 
       Hue[0.6720211162908327, 1, 1], 
       Hue[0.6720211162908327, 1, 1], 
       Hue[0.7482495212329322, 1, 1], 
       Hue[0.7482495212329322, 1, 1], 
       Hue[0.8250046849443391, 1, 1], 
       Hue[0.8250046849443391, 1, 1], 
       Hue[0.9021656385273559, 1, 1], 
       Hue[0.9021656385273559, 1, 1], 
       Hue[0.9798307401607916, 1, 1], 
       Hue[0.9798307401607916, 1, 1], 
       Hue[0.05800235227659842, 1, 1], 
       Hue[0.05800235227659842, 1, 1], 
       Hue[0.13669273281546193`, 1, 1], 
       Hue[0.13669273281546193`, 1, 1], 
       Hue[0.21585977065513692`, 1, 1], 
       Hue[0.21585977065513692`, 1, 1], 
       Hue[0.29548816845309706`, 1, 1], 
       Hue[0.29548816845309706`, 1, 1], 
       Hue[0.37560341603106856`, 1, 1], 
       Hue[0.37560341603106856`, 1, 1], 
       Hue[0.4562040007508113, 1, 1], 
       Hue[0.4562040007508113, 1, 1], 
       Hue[0.5373038430435617, 1, 1], 
       Hue[0.5373038430435617, 1, 1], 
       Hue[0.6188951496447265, 1, 1], 
       Hue[0.6188951496447265, 1, 1], 
       Hue[0.7009511042349899, 1, 1], 
       Hue[0.7009511042349899, 1, 1], 
       Hue[0.7835073631502996, 1, 1], 
       Hue[0.7835073631502996, 1, 1], 
       Hue[0.8665536235626842, 1, 1], 
       Hue[0.8665536235626842, 1, 1], 
       Hue[0.9500892860673488, 1, 1], 
       Hue[0.9500892860673488, 1, 1], 
       Hue[0.034106981671226305`, 1, 1], 
       Hue[0.034106981671226305`, 1, 1], 
       Hue[0.11860815259491314`, 1, 1], 
       Hue[0.11860815259491314`, 1, 1], 
       Hue[0.2035904553821011, 1, 1], 
       Hue[0.2035904553821011, 1, 1], 
       Hue[0.28906409794720567`, 1, 1], 
       Hue[0.28906409794720567`, 1, 1], 
       Hue[0.37502184323541427`, 1, 1], 
       Hue[0.37502184323541427`, 1, 1], 
       Hue[0.4614619759364616, 1, 1], 
       Hue[0.4614619759364616, 1, 1], 
       Hue[0.5483815239488417, 1, 1], 
       Hue[0.5483815239488417, 1, 1], 
       Hue[0.63578442230926, 1, 1], 
       Hue[0.63578442230926, 1, 1], 
       Hue[0.7236676785821858, 1, 1], 
       Hue[0.7236676785821858, 1, 1], 
       Hue[0.8120314596811757, 1, 1], 
       Hue[0.8120314596811757, 1, 1], 
       Hue[0.900873601632316, 1, 1], 
       Hue[0.900873601632316, 1, 1], 
       Hue[0.9901942213417271, 1, 1], 
       Hue[0.9901942213417271, 1, 1], 
       Hue[0.0799895702292984, 1, 1], 
       Hue[0.0799895702292984, 1, 1], 
       Hue[0.170260706962758, 1, 1], 
       Hue[0.170260706962758, 1, 1], 
       Hue[0.2610034850012405, 1, 1], 
       Hue[0.2610034850012405, 1, 1], 
       Hue[0.35221637057648686`, 1, 1], 
       Hue[0.35221637057648686`, 1, 1], 
       Hue[0.44389826161878776`, 1, 1], 
       Hue[0.44389826161878776`, 1, 1], 
       Hue[0.5360449736481047, 1, 1], 
       Hue[0.5360449736481047, 1, 1], 
       Hue[0.6286541129036205, 1, 1], 
       Hue[0.6286541129036205, 1, 1], 
       Hue[0.7217217985068132, 1, 1], 
       Hue[0.7217217985068132, 1, 1], 
       Hue[0.8152433603000737, 1, 1], 
       Hue[0.8152433603000737, 1, 1], 
       Hue[0.9092139707072192, 1, 1], 
       Hue[0.9092139707072192, 1, 1], 
       Hue[0.0036290315923241745`, 1, 1], 
       Hue[0.0036290315923241745`, 1, 1], 
       Hue[0.09848073066420522, 1, 1], 
       Hue[0.09848073066420522, 1, 1], 
       Hue[0.19376227185500616`, 1, 1], 
       Hue[0.19376227185500616`, 1, 1], 
       Hue[0.289464672794272, 1, 1], 
       Hue[0.289464672794272, 1, 1], 
       Hue[0.3855781333910314, 1, 1], 
       Hue[0.3855781333910314, 1, 1], 
       Hue[0.4820908974496971, 1, 1], 
       Hue[0.4820908974496971, 1, 1], 
       Hue[0.5789897695238124, 1, 1], 
       Hue[0.5789897695238124, 1, 1], 
       Hue[0.6762580878511103, 1, 1], 
       Hue[0.6762580878511103, 1, 1], 
       Hue[0.7738792026198127, 1, 1], 
       Hue[0.7738792026198127, 1, 1], 
       Hue[0.8718316149733819, 1, 1], 
       Hue[0.8718316149733819, 1, 1], 
       Hue[0.9700914448448266, 1, 1], 
       Hue[0.9700914448448266, 1, 1], 
       Hue[0.06863078135729278, 1, 1], 
       Hue[0.06863078135729278, 1, 1], 
       Hue[0.1674173699230757, 1, 1], 
       Hue[0.1674173699230757, 1, 1], 
       Hue[0.2664150890214937, 1, 1], 
       Hue[0.2664150890214937, 1, 1], 
       Hue[0.36558201499268755`, 1, 1], 
       Hue[0.36558201499268755`, 1, 1], 
       Hue[0.4648696110448278, 1, 1], 
       Hue[0.4648696110448278, 1, 1], 
       Hue[0.5642232715551221, 1, 1], 
       Hue[0.5642232715551221, 1, 1], 
       Hue[0.6635811232888242, 1, 1], 
       Hue[0.6635811232888242, 1, 1], 
       Hue[0.7628731825198262, 1, 1], 
       Hue[0.7628731825198262, 1, 1], 
       Hue[0.8620209439738609, 1, 1], 
       Hue[0.8620209439738609, 1, 1], 
       Hue[0.960936178647772, 1, 1], 
       Hue[0.960936178647772, 1, 1], 
       Hue[0.05952243736322674, 1, 1], 
       Hue[0.05952243736322674, 1, 1], 
       Hue[0.1576739410013476, 1, 1], 
       Hue[0.1576739410013476, 1, 1], 
       Hue[0.25527613254844134`, 1, 1], 
       Hue[0.25527613254844134`, 1, 1], 
       Hue[0.3522059883891288, 1, 1], 
       Hue[0.3522059883891288, 1, 1], 
       Hue[0.4483344792343556, 1, 1], 
       Hue[0.4483344792343556, 1, 1], 
       Hue[0.5435302454469617, 1, 1], 
       Hue[0.5435302454469617, 1, 1], 
       Hue[0.6376602769840898, 1, 1], 
       Hue[0.6376602769840898, 1, 1], 
       Hue[0.7305964827538581, 1, 1], 
       Hue[0.7305964827538581, 1, 1], 
       Hue[0.822219008183593, 1, 1], 
       Hue[0.822219008183593, 1, 1], 
       Hue[0.9124241755271605, 1, 1], 
       Hue[0.9124241755271605, 1, 1], 
       Hue[0.0011313352912279788`, 1, 1], 
       Hue[0.0011313352912279788`, 1, 1], 
       Hue[0.08829537377171129, 1, 1], 
       Hue[0.08829537377171129, 1, 1], 
       Hue[0.173914091059342, 1, 1], 
       Hue[0.173914091059342, 1, 1], 
       Hue[0.2580327570304501, 1, 1], 
       Hue[0.2580327570304501, 1, 1], 
       Hue[0.34074391743273624`, 1, 1], 
       Hue[0.34074391743273624`, 1, 1], 
       Hue[0.42218350829950074`, 1, 1], 
       Hue[0.42218350829950074`, 1, 1], 
       Hue[0.5025176745363464, 1, 1], 
       Hue[0.5025176745363464, 1, 1], 
       Hue[0.5819340841751708, 1, 1], 
       Hue[0.5819340841751708, 1, 1], 
       Hue[0.6606322089404484, 1, 1], 
       Hue[0.6606322089404484, 1, 1], 
       Hue[0.7388112677112173, 1, 1], 
       Hue[0.7388112677112173, 1, 1], 
       Hue[0.8166629771094296, 1, 1], 
       Hue[0.8166629771094296, 1, 1], 
       Hue[0.894367291049327, 1, 1], 
       Hue[0.894367291049327, 1, 1], 
       Hue[0.9720925084202446, 1, 1], 
       Hue[0.9720925084202446, 1, 1], 
       Hue[0.04998896335048285, 1, 1], 
       Hue[0.04998896335048285, 1, 1], 
       Hue[0.1281877525899212, 1, 1], 
       Hue[0.1281877525899212, 1, 1], 
       Hue[0.20680136452066197`, 1, 1], 
       Hue[0.20680136452066197`, 1, 1], 
       Hue[0.28593303904703327`, 1, 1], 
       Hue[0.28593303904703327`, 1, 1], 
       Hue[0.36567016562670085`, 1, 1], 
       Hue[0.36567016562670085`, 1, 1], 
       Hue[0.44607683475779675`, 1, 1], 
       Hue[0.44607683475779675`, 1, 1], 
       Hue[0.5272080444436378, 1, 1], 
       Hue[0.5272080444436378, 1, 1], 
       Hue[0.6091162047792973, 1, 1], 
       Hue[0.6091162047792973, 1, 1], 
       Hue[0.691847083043619, 1, 1], 
       Hue[0.691847083043619, 1, 1], 
       Hue[0.7754258035423244, 1, 1], 
       Hue[0.7754258035423244, 1, 1], 
       Hue[0.8598681254734846, 1, 1], 
       Hue[0.8598681254734846, 1, 1], 
       Hue[0.9452056790005225, 1, 1], 
       Hue[0.9452056790005225, 1, 1], 
       Hue[0.03147188112713823, 1, 1], 
       Hue[0.03147188112713823, 1, 1], 
       Hue[0.11867304215521769`, 1, 1], 
       Hue[0.11867304215521769`, 1, 1], 
       Hue[0.20676513826419657`, 1, 1], 
       Hue[0.20676513826419657`, 1, 1], 
       Hue[0.2956907086256168, 1, 1], 
       Hue[0.2956907086256168, 1, 1], 
       Hue[0.3854998251506179, 1, 1], 
       Hue[0.3854998251506179, 1, 1], 
       Hue[0.4763031405657942, 1, 1], 
       Hue[0.4763031405657942, 1, 1], 
       Hue[0.5680698599260738, 1, 1], 
       Hue[0.5680698599260738, 1, 1], 
       Hue[0.6606728731331002, 1, 1], 
       Hue[0.6606728731331002, 1, 1], 
       Hue[0.754025841465475, 1, 1], 
       Hue[0.754025841465475, 1, 1], 
       Hue[0.8480804931757541, 1, 1], 
       Hue[0.8480804931757541, 1, 1], 
       Hue[0.9431181536174291, 1, 1], 
       Hue[0.9431181536174291, 1, 1], 
       Hue[0.0397869542393286, 1, 1], 
       Hue[0.0397869542393286, 1, 1], 
       Hue[0.137840700355691, 1, 1], 
       Hue[0.137840700355691, 1, 1], 
       Hue[0.23574526072148794`, 1, 1], 
       Hue[0.23574526072148794`, 1, 1], 
       Hue[0.333496700037884, 1, 1], 
       Hue[0.333496700037884, 1, 1], 
       Hue[0.43380775123773196`, 1, 1], 
       Hue[0.43380775123773196`, 1, 1], 
       Hue[0.536904103339749, 1, 1], 
       Hue[0.536904103339749, 1, 1], 
       Hue[0.6388111314179277, 1, 1], 
       Hue[0.6388111314179277, 1, 1], 
       Hue[0.7395416522567456, 1, 1], 
       Hue[0.7395416522567456, 1, 1], 
       Hue[0.8447363652466423, 1, 1], 
       Hue[0.8447363652466423, 1, 1], 
       Hue[0.9501147795477853, 1, 1], 
       Hue[0.9501147795477853, 1, 1], 
       Hue[0.04045361918740696, 1, 1], 
       Hue[0.04045361918740696, 1, 1], 
       Hue[0.1286078858347576, 1, 1], 
       Hue[0.1286078858347576, 1, 1], 
       Hue[0.24672799717042326`, 1, 1], 
       Hue[0.24672799717042326`, 1, 1], 
       Hue[0.37861522354390637`, 1, 1], 
       Hue[0.37861522354390637`, 1, 1], 
       Hue[0.45736632935843075`, 1, 1], 
       Hue[0.45736632935843075`, 1, 1], 
       Hue[0.49308730974207005`, 1, 1], 
       Hue[0.49308730974207005`, 1, 1], 
       Hue[0.6470686768988838, 1, 1], 
       Hue[0.6470686768988838, 1, 1], 
       Hue[0.8568677735491859, 1, 1], 
       Hue[0.8568677735491859, 1, 1], 
       Hue[0.10431225644341774`, 1, 1], 
       Hue[0.10431225644341774`, 1, 1], 
       Hue[0.3894102905074262, 1, 1], 
       Hue[0.3894102905074262, 1, 1], 
       Hue[0.6838263781448488, 1, 1], 
       Hue[0.6838263781448488, 1, 1], 
       Hue[0.9569069071984381, 1, 1], 
       Hue[0.9569069071984381, 1, 1], 
       Hue[0.21395671894896143`, 1, 1], 
       Hue[0.21395671894896143`, 1, 1], 
       Hue[0.4664485349712569, 1, 1], 
       Hue[0.4664485349712569, 1, 1], 
       Hue[0.7219258772292536, 1, 1], 
       Hue[0.7219258772292536, 1, 1], 
       Hue[0.9856372041087308, 1, 1], 
       Hue[0.9856372041087308, 1, 1], 
       Hue[0.2608722601555482, 1, 1], 
       Hue[0.2608722601555482, 1, 1], 
       Hue[0.5470837038038487, 1, 1], 
       Hue[0.5470837038038487, 1, 1], 
       Hue[0.8401196088581602, 1, 1], 
       Hue[0.8401196088581602, 1, 1], 
       Hue[0.13624437333826112`, 1, 1], 
       Hue[0.13624437333826112`, 1, 1], 
       Hue[0.4309682311873325, 1, 1], 
       Hue[0.4309682311873325, 1, 1], 
       Hue[0.7184660058323185, 1, 1], 
       Hue[0.7184660058323185, 1, 1], 
       Hue[0.9973525953613468, 1, 1], 
       Hue[0.9973525953613468, 1, 1], 
       Hue[0.2702841777599323, 1, 1], 
       Hue[0.2702841777599323, 1, 1], 
       Hue[0.5401052324973117, 1, 1], 
       Hue[0.5401052324973117, 1, 1], 
       Hue[0.8084590354995888, 1, 1], 
       Hue[0.8084590354995888, 1, 1], 
       Hue[0.07594959588080527, 1, 1], 
       Hue[0.07594959588080527, 1, 1], 
       Hue[0.34257008552859086`, 1, 1], 
       Hue[0.34257008552859086`, 1, 1], 
       Hue[0.608041503369714, 1, 1], 
       Hue[0.608041503369714, 1, 1], 
       Hue[0.8720135963849045, 1, 1], 
       Hue[0.8720135963849045, 1, 1], 
       Hue[0.13414354537958378`, 1, 1], 
       Hue[0.13414354537958378`, 1, 1], 
       Hue[0.3940928156514941, 1, 1], 
       Hue[0.3940928156514941, 1, 1], 
       Hue[0.6514812971209296, 1, 1], 
       Hue[0.6514812971209296, 1, 1], 
       Hue[0.9058301300769357, 1, 1], 
       Hue[0.9058301300769357, 1, 1], 
       Hue[0.1565116040087216, 1, 1], 
       Hue[0.1565116040087216, 1, 1], 
       Hue[0.4027360261695492, 1, 1], 
       Hue[0.4027360261695492, 1, 1], 
       Hue[0.6436450448936091, 1, 1], 
       Hue[0.6436450448936091, 1, 1], 
       Hue[0.8786518091264015, 1, 1], 
       Hue[0.8786518091264015, 1, 1], 
       Hue[0.10814245304970796`, 1, 1], 
       Hue[0.10814245304970796`, 1, 1], 
       Hue[0.3342040087120603, 1, 1], 
       Hue[0.3342040087120603, 1, 1], 
       Hue[0.5603417438283603, 1, 1], 
       Hue[0.5603417438283603, 1, 1], 
       Hue[0.7898375376869549, 1, 1], 
       Hue[0.7898375376869549, 1, 1], 
       Hue[0.02435059756972669, 1, 1], 
       Hue[0.02435059756972669, 1, 1], 
       Hue[0.263972200373601, 1, 1], 
       Hue[0.263972200373601, 1, 1], 
       Hue[0.5080091112724794, 1, 1], 
       Hue[0.5080091112724794, 1, 1], 
       Hue[0.7555873580958158, 1, 1], 
       Hue[0.7555873580958158, 1, 1], 
       Hue[0.005910038993356071, 1, 1], 
       Hue[0.005910038993356071, 1, 1], 
       Hue[0.2583208773049551, 1, 1], 
       Hue[0.2583208773049551, 1, 1], 
       Hue[0.5122990607419959, 1, 1], 
       Hue[0.5122990607419959, 1, 1], 
       Hue[0.7674399787929954, 1, 1], 
       Hue[0.7674399787929954, 1, 1], 
       Hue[0.02343753155512082, 1, 1], 
       Hue[0.02343753155512082, 1, 1], 
       Hue[0.2800651076356245, 1, 1], 
       Hue[0.2800651076356245, 1, 1], 
       Hue[0.5371545059966867, 1, 1], 
       Hue[0.5371545059966867, 1, 1], 
       Hue[0.7945733996717301, 1, 1], 
       Hue[0.7945733996717301, 1, 1], 
       Hue[0.052204389580622186`, 1, 1], 
       Hue[0.052204389580622186`, 1, 1], 
       Hue[0.30993216728720485`, 1, 1], 
       Hue[0.30993216728720485`, 1, 1], 
       Hue[0.5676409258558874, 1, 1], 
       Hue[0.5676409258558874, 1, 1], 
       Hue[0.8252245236518228, 1, 1], 
       Hue[0.8252245236518228, 1, 1], 
       Hue[0.08260688618784486, 1, 1], 
       Hue[0.08260688618784486, 1, 1], 
       Hue[0.3397704033682906, 1, 1], 
       Hue[0.3397704033682906, 1, 1], 
       Hue[0.5967914666911367, 1, 1], 
       Hue[0.5967914666911367, 1, 1], 
       Hue[0.8538866103988598, 1, 1], 
       Hue[0.8538866103988598, 1, 1], 
       Hue[0.11148110441383434`, 1, 1], 
       Hue[0.11148110441383434`, 1, 1], 
       Hue[0.37032221044370817`, 1, 1], 
       Hue[0.37032221044370817`, 1, 1], 
       Hue[0.6316738642015586, 1, 1], 
       Hue[0.6316738642015586, 1, 1], 
       Hue[0.8976395052931159, 1, 1], 
       Hue[0.8976395052931159, 1, 1], 
       Hue[0.17158596189859066`, 1, 1], 
       Hue[0.17158596189859066`, 1, 1], 
       Hue[0.45806602624555387`, 1, 1], 
       Hue[0.45806602624555387`, 1, 1], 
       Hue[0.759613669603346, 1, 1], 
       Hue[0.759613669603346, 1, 1], 
       Hue[0.06876575145420652, 1, 1], 
       Hue[0.06876575145420652, 1, 1], 
       Hue[0.3704520490070801, 1, 1], 
       Hue[0.3704520490070801, 1, 1], 
       Hue[0.6581400003275106, 1, 1], 
       Hue[0.6581400003275106, 1, 1], 
       Hue[0.9342542877440975, 1, 1], 
       Hue[0.9342542877440975, 1, 1], 
       Hue[0.20258989419990772`, 1, 1], 
       Hue[0.20258989419990772`, 1, 1], 
       Hue[0.46580311190130813`, 1, 1], 
       Hue[0.46580311190130813`, 1, 1], 
       Hue[0.7255298424272546, 1, 1], 
       Hue[0.7255298424272546, 1, 1], 
       Hue[0.9828031804668734, 1, 1], 
       Hue[0.9828031804668734, 1, 1], 
       Hue[0.2383262257411595, 1, 1], 
       Hue[0.2383262257411595, 1, 1], 
       Hue[0.4926179409024767, 1, 1], 
       Hue[0.4926179409024767, 1, 1], 
       Hue[0.7460921021000613, 1, 1], 
       Hue[0.7460921021000613, 1, 1]}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->{True, None},
    AxesOrigin->{0, 0},
    Epilog->
     LineBox[{{1, 1.6982610874419622`*^-6}, {2, 1.9647701693871465`*^-6}, {
       3, 2.0948437179045684`*^-6}, {4, 2.0302758518194934`*^-6}, {
       5, 2.0415101989582863`*^-6}, {6, 2.4237144694841933`*^-6}, {
       7, 2.263507117589174*^-6}, {8, 2.15188375044499*^-6}, {
       9, 2.389458724731309*^-6}, {10, 2.2209139272981044`*^-6}, {
       11, 2.0436269132659483`*^-6}, {12, 1.7497485462304771`*^-6}, {
       13, 1.7704495747455398`*^-6}, {14, 1.4545976910234285`*^-6}, {
       15, 1.76508901076846*^-6}, {16, 1.6365180323201764`*^-6}, {
       17, 1.7531282690360914`*^-6}, {18, 1.1382589000422142`*^-6}, {
       19, 1.2741942634533994`*^-6}, {20, 1.2716663098030212`*^-6}, {
       21, 1.5070659336485792`*^-6}, {22, 1.358278950778587*^-6}, {
       23, 1.2723159736908188`*^-6}, {24, 1.428478902232972*^-6}, {
       25, 1.5594612339969622`*^-6}, {26, 2.0812536031578334`*^-6}, {
       27, 2.0599880018011587`*^-6}, {28, 1.911509335940612*^-6}, {
       29, 1.275693584243608*^-6}, {30, 5.592922461325838*^-7}, {
       31, 1.5033435767768933`*^-6}, {32, 2.6308798571919916`*^-6}, {
       33, 4.4215774115370905`*^-6}, {34, 5.12757909305354*^-6}, {
       35, 5.72760521533108*^-6}, {36, 8.227998861627223*^-6}, {
       37, 0.000011496751643690234`}, {38, 0.00001641178926049975}, {
       39, 0.000022328379297166416`}, {40, 0.000029130572054353337`}, {
       41, 0.000038573511936454706`}, {42, 0.00005142662480180197}, {
       43, 0.00006810732508637353}, {44, 0.0000911546459833953}, {
       45, 0.00012077470985243289`}, {46, 0.00015796610167937512`}, {
       47, 0.0002064050644225185}, {48, 0.00026808475547251704`}, {
       49, 0.0003470294687037318}, {50, 0.00044956544971822574`}, {
       51, 0.0005780051125679727}, {52, 0.0007399352695906527}, {
       53, 0.0009408092852317185}, {54, 0.0011929766658488746`}, {
       55, 0.0015061783531894648`}, {56, 0.001894756352925618}, {
       57, 0.0023719167714152003`}, {58, 0.002956496462836706}, {
       59, 0.003668471692467315}, {60, 0.004533488441720348}, {
       61, 0.005578673811343152}, {62, 0.006834720140660297}, {
       63, 0.008337922235369858}, {64, 0.010125773580268912`}, {
       65, 0.012244541904668123`}, {66, 0.014742906615013941`}, {
       67, 0.017672704373976108`}, {68, 0.02109220059254929}, {
       69, 0.02506509972756395}, {70, 0.029656494270082816`}, {
       71, 0.03493720991780949}, {72, 0.04097893986486279}, {
       73, 0.04785463741800247}, {74, 0.055640216363719255`}, {
       75, 0.0644125128869491}, {76, 0.07424265631887662}, {
       77, 0.08519935490738817}, {78, 0.09734634691951272}, {
       79, 0.11073877360399402`}, {80, 0.1254249289938402}, {
       81, 0.14143861107288347`}, {82, 0.15879884542870257`}, {
       83, 0.1775115044913922}, {84, 0.19756209466556782`}, {
       85, 0.2189171098734944}, {86, 0.24151841102316288`}, {
       87, 0.26528960762072673`}, {88, 0.29012622675921884`}, {
       89, 0.315901005078622}, {90, 0.3424631209840068}, {
       91, 0.36963656854100463`}, {92, 0.39722223248745586`}, {
       93, 0.4250025479830893}, {94, 0.4527414477605905}, {
       95, 0.4801855720565026}, {96, 0.5070706215499177}, {
       97, 0.5331265865110192}, {98, 0.5580784883728015}, {
       99, 0.5816552522714593}, {100, 0.6035882809440296}, {
       101, 0.6236209000280355}, {102, 0.6415165287012181}, {
       103, 0.6570557732577346}, {104, 0.6700446279125143}, {
       105, 0.6803152854825002}, {106, 0.687734108920405}, {
       107, 0.6921986813412877}, {108, 0.6936453454100678}, {
       109, 0.6920399413480028}, {110, 0.6873869749461042}, {
       111, 0.6797249679215843}, {112, 0.6691278665235665}, {
       113, 0.6556921635290287}, {114, 0.6395453447840154}, {
       115, 0.6208382616754691}, {116, 0.5997411293807583}, {
       117, 0.5764401704231145}, {118, 0.5511334776968638}, {
       119, 0.5240305068689864}, {120, 0.49534997192265817`}, {
       121, 0.4653249818133817}, {122, 0.43419970261802016`}, {
       123, 0.40223057501093107`}, {124, 0.36969224814526735`}, {
       125, 0.3368861729576676}, {126, 0.3041515786051314}, {
       127, 0.2718703623851708}, {128, 0.24046192898104318`}, {
       129, 0.21035706497599305`}, {130, 0.1819649617957836}, {
       131, 0.1556411215041802}, {132, 0.1316517549384237}, {
       133, 0.11015913659298437`}, {134, 0.09121860431563601}, {
       135, 0.07478808492258321}, {136, 0.06074390577387306}, {
       137, 0.048902053562975036`}, {138, 0.03904220194724473}, {
       139, 0.030928136282631313`}, {140, 0.024322179759817087`}, {
       141, 0.01899566338179031}, {142, 0.014739178891111785`}, {
       143, 0.01136650432361027}, {144, 0.008714954975170101}, {
       145, 0.006644985971218271}, {146, 0.005039590827510158}, {
       147, 0.003802605791960672}, {148, 0.0028553383581204775`}, {
       149, 0.00213379177038994}, {150, 0.0015868030531876213`}, {
       151, 0.001174223915727052}, {152, 0.0008650182387141355}, {
       153, 0.0006347903835024599}, {154, 0.00046396334584149164`}, {
       155, 0.00033749842409038503`}, {156, 0.0002443422560242348}, {
       157, 0.00017616107162067316`}, {158, 0.0001266309479384828}, {
       159, 0.00009092834283296823}, {160, 0.00006509498741269296}, {
       161, 0.00004614899305824531}, {162, 0.00003242317121705183}, {
       163, 0.000022869994455753856`}, {164, 0.000016203980099513416`}, {
       165, 0.000011289214982628973`}, {166, 7.715153041070073*^-6}, {
       167, 5.322149728176927*^-6}, {168, 3.6676550396962726`*^-6}, {
       169, 2.3935315225552227`*^-6}, {170, 1.549880981005627*^-6}, {
       171, 1.167067405975017*^-6}, {172, 9.062389395707861*^-7}, {
       173, 5.708653782239252*^-7}, {174, 2.8945899973879995`*^-7}, {
       175, 2.967207380330455*^-7}, {176, 3.653042334172365*^-7}, {
       177, 3.4204143315030274`*^-7}, {178, 2.667783534666879*^-7}, {
       179, 2.0697146870377512`*^-7}, {180, 1.896090490189446*^-7}, {
       181, 1.832458510126788*^-7}, {182, 1.6261564615772594`*^-7}, {
       183, 1.286417963815216*^-7}, {184, 9.169261386046591*^-8}, {
       185, 6.053939613475948*^-8}, {186, 3.885686033006999*^-8}, {
       187, 2.5900849836937016`*^-8}, {188, 1.899370168661009*^-8}, {
       189, 1.5862338164688535`*^-8}, {190, 1.5241901735972995`*^-8}, {
       191, 1.6187204930059355`*^-8}, {192, 1.7713463652247404`*^-8}, {
       193, 1.9005458583186304`*^-8}, {194, 1.9590719033333195`*^-8}, {
       195, 1.932363266290875*^-8}, {196, 1.827823135123405*^-8}, {
       197, 1.6635355337833728`*^-8}, {198, 1.4605923721175196`*^-8}, {
       199, 1.2392123394311898`*^-8}, {200, 1.0169865918038008`*^-8}, {
       201, 8.079218824983797*^-9}, {202, 6.218712054094677*^-9}, {
       203, 4.644218033499428*^-9}, {204, 3.3732348073758506`*^-9}, {
       205, 2.3930921865282627`*^-9}, {206, 1.6708413960370641`*^-9}, {
       207, 1.1623881569200185`*^-9}, {208, 8.195943176920906*^-10}, {
       209, 5.954823945304898*^-10}, {210, 4.4886240724376257`*^-10}, {
       211, 3.4852163178805556`*^-10}, {212, 2.746945337663734*^-10}, {
       213, 2.1685384809247283`*^-10}, {214, 1.7002102768423432`*^-10}, {
       215, 1.318386451064352*^-10}, {216, 1.0094732808474467`*^-10}, {
       217, 7.629426594331476*^-11}, {218, 5.6915407756874474`*^-11}, {
       219, 4.190743787146259*^-11}, {220, 3.0449648929368786`*^-11}, {
       221, 2.1824113674691968`*^-11}, {222, 1.5421888997366352`*^-11}, {
       223, 1.0738372429719186`*^-11}, {224, 7.36311575431131*^-12}, {
       225, 4.967952894865655*^-12}, {226, 3.295153981128586*^-12}, {
       227, 2.1460575514314102`*^-12}, {228, 1.3704134411195273`*^-12}, {
       229, 8.566435530940083*^-13}, {230, 5.232977769443833*^-13}, {
       231, 3.11890589369434*^-13}, {232, 1.8113341000573393`*^-13}, {
       233, 1.0243158000355152`*^-13}, {234, 5.6426822851840584`*^-14}, {
       235, 3.036609779468772*^-14}, {236, 1.6112304934858916`*^-14}, {
       237, 8.65417514361298*^-15}, {238, 4.993682472955497*^-15}, {
       239, 3.321981832551148*^-15}, {240, 2.5355995858752223`*^-15}, {
       241, 2.0660815329268104`*^-15}, {242, 1.7009075383757556`*^-15}, {
       243, 1.3815175723394891`*^-15}, {244, 1.0973438723643075`*^-15}, {
       245, 8.494648561693853*^-16}, {246, 6.399647522652412*^-16}, {
       247, 4.690292980897647*^-16}, {248, 3.344884025643928*^-16}, {
       249, 2.322805531343931*^-16}, {250, 1.5724757171557174`*^-16}, {
       251, 1.0393113940608272`*^-16}}],
    Frame->True,
    FrameTicks->{{{1., 
        FormBox[
         RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
        FormBox[
         RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
        FormBox[
         RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
        FormBox[
         RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
        FormBox["0", TraditionalForm]}, {157.25, 
        FormBox["2.5`", TraditionalForm]}, {188.5, 
        FormBox["5", TraditionalForm]}, {219.75, 
        FormBox["7.5`", TraditionalForm]}, {251., 
        FormBox["10", TraditionalForm]}}, Automatic, None, None},
    PlotRange->{-0.1, 1.1},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], Automatic}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF1mdYVAcWxvGLWQP2soqxRAY1FgQEVMT+ilIEUXpHLkMZOhFRs6jJlYDR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     "], {
      {GrayLevel[0.5], EdgeForm[None], 
       GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ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         "]]]}, {{}, {}, {
        {Hue[0.9407694159419471, 1, 1], LineBox[{1, 2}]}, 
        {Hue[0.8652990571156856, 1, 1], LineBox[{2, 3}]}, 
        {Hue[0.7534371156171316, 1, 1], LineBox[{3, 4}]}, 
        {Hue[0.6481847977203763, 1, 1], LineBox[{4, 5}]}, 
        {Hue[0.5354894789766873, 1, 1], LineBox[{5, 6}]}, 
        {Hue[0.46816222537833896`, 1, 1], LineBox[{6, 7}]}, 
        {Hue[0.35585854458977223`, 1, 1], LineBox[{7, 8}]}, 
        {Hue[0.2817377719638701, 1, 1], LineBox[{8, 9}]}, 
        {Hue[0.14460000879947793`, 1, 1], LineBox[{9, 10}]}, 
        {Hue[0.01469903437668414, 1, 1], LineBox[{10, 11}]}, 
        {Hue[0.8988936709144447, 1, 1], LineBox[{11, 12}]}, 
        {Hue[0.810522269771311, 1, 1], LineBox[{12, 13}]}, 
        {Hue[0.6480896528173896, 1, 1], LineBox[{13, 14}]}, 
        {Hue[0.5591795772773027, 1, 1], LineBox[{14, 15}]}, 
        {Hue[0.4540232922687597, 1, 1], LineBox[{15, 16}]}, 
        {Hue[0.40055182835091585`, 1, 1], LineBox[{16, 17}]}, 
        {Hue[0.27385460721166377`, 1, 1], LineBox[{17, 18}]}, 
        {Hue[0.14121327890295116`, 1, 1], LineBox[{18, 19}]}, 
        {Hue[0.11383263176537513`, 1, 1], LineBox[{19, 20}]}, 
        {Hue[0.9936820338959563, 1, 1], LineBox[{20, 21}]}, 
        {Hue[0.9282939753793533, 1, 1], LineBox[{21, 22}]}, 
        {Hue[0.812205984476198, 1, 1], LineBox[{22, 23}]}, 
        {Hue[0.6632661263018338, 1, 1], LineBox[{23, 24}]}, 
        {Hue[0.6142780797353196, 1, 1], LineBox[{24, 25}]}, 
        {Hue[0.5144926704498913, 1, 1], LineBox[{25, 26}]}, 
        {Hue[0.5040634907503884, 1, 1], LineBox[{26, 27}]}, 
        {Hue[0.43416845221881806`, 1, 1], LineBox[{27, 28}]}, 
        {Hue[0.26450211931177586`, 1, 1], LineBox[{28, 29}]}, 
        {Hue[0.11758076126626354`, 1, 1], LineBox[{29, 30}]}, 
        {Hue[0.9354851250970341, 1, 1], LineBox[{30, 31}]}, 
        {Hue[0.7503898389339675, 1, 1], LineBox[{31, 32}]}, 
        {Hue[0.9769323213302773, 1, 1], LineBox[{32, 33}]}, 
        {Hue[0.932109019870338, 1, 1], LineBox[{33, 34}]}, 
        {Hue[0.6669326654057793, 1, 1], LineBox[{34, 35}]}, 
        {Hue[0.43016993026155, 1, 1], LineBox[{35, 36}]}, 
        {Hue[0.39716477337106854`, 1, 1], LineBox[{36, 37}]}, 
        {Hue[0.36493756558008444`, 1, 1], LineBox[{37, 38}]}, 
        {Hue[0.4080060559607373, 1, 1], LineBox[{38, 39}]}, 
        {Hue[0.4309672884697319, 1, 1], LineBox[{39, 40}]}, 
        {Hue[0.5565706017286116, 1, 1], LineBox[{40, 41}]}, 
        {Hue[0.6190552894798738, 1, 1], LineBox[{41, 42}]}, 
        {Hue[0.6934773859825238, 1, 1], LineBox[{42, 43}]}, 
        {Hue[0.7431865445042352, 1, 1], LineBox[{43, 44}]}, 
        {Hue[0.7979053944788896, 1, 1], LineBox[{44, 45}]}, 
        {Hue[0.8639723377143136, 1, 1], LineBox[{45, 46}]}, 
        {Hue[0.931354074203931, 1, 1], LineBox[{46, 47}]}, 
        {Hue[0.0019813757552162815`, 1, 1], LineBox[{47, 48}]}, 
        {Hue[0.0721006651445593, 1, 1], LineBox[{48, 49}]}, 
        {Hue[0.13836744610171361`, 1, 1], LineBox[{49, 50}]}, 
        {Hue[0.20386232588584846`, 1, 1], LineBox[{50, 51}]}, 
        {Hue[0.27293800925391504`, 1, 1], LineBox[{51, 52}]}, 
        {Hue[0.3407829136148149, 1, 1], LineBox[{52, 53}]}, 
        {Hue[0.41103261850344186`, 1, 1], LineBox[{53, 54}]}, 
        {Hue[0.48015625333872314`, 1, 1], LineBox[{54, 55}]}, 
        {Hue[0.5500525672930505, 1, 1], LineBox[{55, 56}]}, 
        {Hue[0.620445851492982, 1, 1], LineBox[{56, 57}]}, 
        {Hue[0.6917554854759792, 1, 1], LineBox[{57, 58}]}, 
        {Hue[0.7634716246789225, 1, 1], LineBox[{58, 59}]}, 
        {Hue[0.8356895882785431, 1, 1], LineBox[{59, 60}]}, 
        {Hue[0.9082214829959486, 1, 1], LineBox[{60, 61}]}, 
        {Hue[0.9815323348687448, 1, 1], LineBox[{61, 62}]}, 
        {Hue[0.055317258485876146`, 1, 1], LineBox[{62, 63}]}, 
        {Hue[0.12961575956451213`, 1, 1], LineBox[{63, 64}]}, 
        {Hue[0.20433090079601726`, 1, 1], LineBox[{64, 65}]}, 
        {Hue[0.2795540383337438, 1, 1], LineBox[{65, 66}]}, 
        {Hue[0.35524739473256994`, 1, 1], LineBox[{66, 67}]}, 
        {Hue[0.4315296393312207, 1, 1], LineBox[{67, 68}]}, 
        {Hue[0.5082753399542235, 1, 1], LineBox[{68, 69}]}, 
        {Hue[0.5855122705463676, 1, 1], LineBox[{69, 70}]}, 
        {Hue[0.6632910732135462, 1, 1], LineBox[{70, 71}]}, 
        {Hue[0.7415493785590683, 1, 1], LineBox[{71, 72}]}, 
        {Hue[0.8203168163406356, 1, 1], LineBox[{72, 73}]}, 
        {Hue[0.899594466264572, 1, 1], LineBox[{73, 74}]}, 
        {Hue[0.9793808535751918, 1, 1], LineBox[{74, 75}]}, 
        {Hue[0.05966611579066608, 1, 1], LineBox[{75, 76}]}, 
        {Hue[0.14046103040636695`, 1, 1], LineBox[{76, 77}]}, 
        {Hue[0.22176099485644657`, 1, 1], LineBox[{77, 78}]}, 
        {Hue[0.30355640042518384`, 1, 1], LineBox[{78, 79}]}, 
        {Hue[0.3858558561080394, 1, 1], LineBox[{79, 80}]}, 
        {Hue[0.46865920579523773`, 1, 1], LineBox[{80, 81}]}, 
        {Hue[0.5519572630975732, 1, 1], LineBox[{81, 82}]}, 
        {Hue[0.6357616925070961, 1, 1], LineBox[{82, 83}]}, 
        {Hue[0.7200614226226909, 1, 1], LineBox[{83, 84}]}, 
        {Hue[0.8048540983332827, 1, 1], LineBox[{84, 85}]}, 
        {Hue[0.8901452092823243, 1, 1], LineBox[{85, 86}]}, 
        {Hue[0.9759256850504838, 1, 1], LineBox[{86, 87}]}, 
        {Hue[0.06219940245881586, 1, 1], LineBox[{87, 88}]}, 
        {Hue[0.14895952789651382`, 1, 1], LineBox[{88, 89}]}, 
        {Hue[0.23620276109148372`, 1, 1], LineBox[{89, 90}]}, 
        {Hue[0.32392749417342787`, 1, 1], LineBox[{90, 91}]}, 
        {Hue[0.41212937430084273`, 1, 1], LineBox[{91, 92}]}, 
        {Hue[0.5008039055547484, 1, 1], LineBox[{92, 93}]}, 
        {Hue[0.5899448498811499, 1, 1], LineBox[{93, 94}]}, 
        {Hue[0.6795444249013187, 1, 1], LineBox[{94, 95}]}, 
        {Hue[0.7695963834453866, 1, 1], LineBox[{95, 96}]}, 
        {Hue[0.8600924523736628, 1, 1], LineBox[{96, 97}]}, 
        {Hue[0.9510219118737493, 1, 1], LineBox[{97, 98}]}, 
        {Hue[0.04237273932485363, 1, 1], LineBox[{98, 99}]}, 
        {Hue[0.1341303083120591, 1, 1], LineBox[{99, 100}]}, 
        {Hue[0.2262799008469206, 1, 1], LineBox[{100, 101}]}, 
        {Hue[0.318803207291762, 1, 1], LineBox[{101, 102}]}, 
        {Hue[0.41167848966155357`, 1, 1], LineBox[{102, 103}]}, 
        {Hue[0.5048808051332369, 1, 1], LineBox[{103, 104}]}, 
        {Hue[0.5983810634330753, 1, 1], LineBox[{104, 105}]}, 
        {Hue[0.6921470473254048, 1, 1], LineBox[{105, 106}]}, 
        {Hue[0.7861407557274118, 1, 1], LineBox[{106, 107}]}, 
        {Hue[0.8803183444062389, 1, 1], LineBox[{107, 108}]}, 
        {Hue[0.9746296130822348, 1, 1], LineBox[{108, 109}]}, 
        {Hue[0.06901767356908702, 1, 1], LineBox[{109, 110}]}, 
        {Hue[0.1634186711999563, 1, 1], LineBox[{110, 111}]}, 
        {Hue[0.257759298319671, 1, 1], LineBox[{111, 112}]}, 
        {Hue[0.35195625424943705`, 1, 1], LineBox[{112, 113}]}, 
        {Hue[0.4459170200063353, 1, 1], LineBox[{113, 114}]}, 
        {Hue[0.5395389647253133, 1, 1], LineBox[{114, 115}]}, 
        {Hue[0.6327115014530438, 1, 1], LineBox[{115, 116}]}, 
        {Hue[0.7253120626933802, 1, 1], LineBox[{116, 117}]}, 
        {Hue[0.8172080503626297, 1, 1], LineBox[{117, 118}]}, 
        {Hue[0.9082571817465559, 1, 1], LineBox[{118, 119}]}, 
        {Hue[0.9983125469259199, 1, 1], LineBox[{119, 120}]}, 
        {Hue[0.0872270893257851, 1, 1], LineBox[{120, 121}]}, 
        {Hue[0.17485685640692983`, 1, 1], LineBox[{121, 122}]}, 
        {Hue[0.26106162246570447`, 1, 1], LineBox[{122, 123}]}, 
        {Hue[0.3457071509121968, 1, 1], LineBox[{123, 124}]}, 
        {Hue[0.4286721641297487, 1, 1], LineBox[{124, 125}]}, 
        {Hue[0.5098651650273955, 1, 1], LineBox[{125, 126}]}, 
        {Hue[0.5892417687680118, 1, 1], LineBox[{126, 127}]}, 
        {Hue[0.6668211853508332, 1, 1], LineBox[{127, 128}]}, 
        {Hue[0.7426906112473048, 1, 1], LineBox[{128, 129}]}, 
        {Hue[0.8170015469243537, 1, 1], LineBox[{129, 130}]}, 
        {Hue[0.8899533966442782, 1, 1], LineBox[{130, 131}]}, 
        {Hue[0.9617783185528035, 1, 1], LineBox[{131, 132}]}, 
        {Hue[0.03272320056978751, 1, 1], LineBox[{132, 133}]}, 
        {Hue[0.10303459335594814`, 1, 1], LineBox[{133, 134}]}, 
        {Hue[0.17294878870530267`, 1, 1], LineBox[{134, 135}]}, 
        {Hue[0.2426863090271892, 1, 1], LineBox[{135, 136}]}, 
        {Hue[0.31244541224626743`, 1, 1], LineBox[{136, 137}]}, 
        {Hue[0.3824019140435007, 1, 1], LineBox[{137, 138}]}, 
        {Hue[0.4527103697766939, 1, 1], LineBox[{138, 139}]}, 
        {Hue[0.5235041204448956, 1, 1], LineBox[{139, 140}]}, 
        {Hue[0.5948940264243402, 1, 1], LineBox[{140, 141}]}, 
        {Hue[0.6669759737360023, 1, 1], LineBox[{141, 142}]}, 
        {Hue[0.7398286321251204, 1, 1], LineBox[{142, 143}]}, 
        {Hue[0.8135130404584696, 1, 1], LineBox[{143, 144}]}, 
        {Hue[0.8880832031234254, 1, 1], LineBox[{144, 145}]}, 
        {Hue[0.9635811958810462, 1, 1], LineBox[{145, 146}]}, 
        {Hue[0.04003531910755127, 1, 1], LineBox[{146, 147}]}, 
        {Hue[0.11746914394991234`, 1, 1], LineBox[{147, 148}]}, 
        {Hue[0.195904600981633, 1, 1], LineBox[{148, 149}]}, 
        {Hue[0.27535047794179796`, 1, 1], LineBox[{149, 150}]}, 
        {Hue[0.3558085223791187, 1, 1], LineBox[{150, 151}]}, 
        {Hue[0.4372840484618632, 1, 1], LineBox[{151, 152}]}, 
        {Hue[0.5197882895126544, 1, 1], LineBox[{152, 153}]}, 
        {Hue[0.6033219242113339, 1, 1], LineBox[{153, 154}]}, 
        {Hue[0.6878536441947171, 1, 1], LineBox[{154, 155}]}, 
        {Hue[0.7733697728793487, 1, 1], LineBox[{155, 156}]}, 
        {Hue[0.8598942801945528, 1, 1], LineBox[{156, 157}]}, 
        {Hue[0.9474247816814833, 1, 1], LineBox[{157, 158}]}, 
        {Hue[0.035907408921143254`, 1, 1], LineBox[{158, 159}]}, 
        {Hue[0.1253083000319576, 1, 1], LineBox[{159, 160}]}, 
        {Hue[0.2156850527530413, 1, 1], LineBox[{160, 161}]}, 
        {Hue[0.30709160770072824`, 1, 1], LineBox[{161, 162}]}, 
        {Hue[0.3994547063543386, 1, 1], LineBox[{162, 163}]}, 
        {Hue[0.4926921458642045, 1, 1], LineBox[{163, 164}]}, 
        {Hue[0.5868410658775327, 1, 1], LineBox[{164, 165}]}, 
        {Hue[0.6819856197390084, 1, 1], LineBox[{165, 166}]}, 
        {Hue[0.7780905715872407, 1, 1], LineBox[{166, 167}]}, 
        {Hue[0.8747788347944276, 1, 1], LineBox[{167, 168}]}, 
        {Hue[0.9716418911349147, 1, 1], LineBox[{168, 169}]}, 
        {Hue[0.06920173882244322, 1, 1], LineBox[{169, 170}]}, 
        {Hue[0.16854962278205396`, 1, 1], LineBox[{170, 171}]}, 
        {Hue[0.2695185878541521, 1, 1], LineBox[{171, 172}]}, 
        {Hue[0.3707722030072139, 1, 1], LineBox[{172, 173}]}, 
        {Hue[0.47160607498768725`, 1, 1], LineBox[{173, 174}]}, 
        {Hue[0.5721384519808977, 1, 1], LineBox[{174, 175}]}, 
        {Hue[0.6738855996878312, 1, 1], LineBox[{175, 176}]}, 
        {Hue[0.78145634117785, 1, 1], LineBox[{176, 177}]}, 
        {Hue[0.8966100687768956, 1, 1], LineBox[{177, 178}]}, 
        {Hue[0.00987144155480758, 1, 1], LineBox[{178, 179}]}, 
        {Hue[0.10665989586565482`, 1, 1], LineBox[{179, 180}]}, 
        {Hue[0.2017755608777928, 1, 1], LineBox[{180, 181}]}, 
        {Hue[0.3243851462421015, 1, 1], LineBox[{181, 182}]}, 
        {Hue[0.46035977061619515`, 1, 1], LineBox[{182, 183}]}, 
        {Hue[0.5698711687553977, 1, 1], LineBox[{183, 184}]}, 
        {Hue[0.6446141696538195, 1, 1], LineBox[{184, 185}]}, 
        {Hue[0.7649959746908312, 1, 1], LineBox[{185, 186}]}, 
        {Hue[0.8499991304409061, 1, 1], LineBox[{186, 187}]}, 
        {Hue[0.7527311439527943, 1, 1], LineBox[{187, 188}]}, 
        {Hue[0.9407324783675022, 1, 1], LineBox[{188, 189}]}, 
        {Hue[0.16279964711927528`, 1, 1], LineBox[{189, 190}]}, 
        {Hue[0.40538122810309657`, 1, 1], LineBox[{190, 191}]}, 
        {Hue[0.7662466372646557, 1, 1], LineBox[{191, 192}]}, 
        {Hue[0.325746588547473, 1, 1], LineBox[{192, 193}]}, 
        {Hue[0.6014346559379302, 1, 1], LineBox[{193, 194}]}, 
        {Hue[0.8702034520468045, 1, 1], LineBox[{194, 195}]}, 
        {Hue[0.14000852383014625`, 1, 1], LineBox[{195, 196}]}, 
        {Hue[0.4094397952932074, 1, 1], LineBox[{196, 197}]}, 
        {Hue[0.6768951372489489, 1, 1], LineBox[{197, 198}]}, 
        {Hue[0.9421369682877887, 1, 1], LineBox[{198, 199}]}, 
        {Hue[0.2058797630336536, 1, 1], LineBox[{199, 200}]}, 
        {Hue[0.4690627681848623, 1, 1], LineBox[{200, 201}]}, 
        {Hue[0.7324642750692172, 1, 1], LineBox[{201, 202}]}, 
        {Hue[0.996675128822844, 1, 1], LineBox[{202, 203}]}, 
        {Hue[0.2623619251828773, 1, 1], LineBox[{203, 204}]}, 
        {Hue[0.5308036556013873, 1, 1], LineBox[{204, 205}]}, 
        {Hue[0.8045450818299433, 1, 1], LineBox[{205, 206}]}, 
        {Hue[0.08721915474396433, 1, 1], LineBox[{206, 207}]}, 
        {Hue[0.38020271952928, 1, 1], LineBox[{207, 208}]}, 
        {Hue[0.6773830849870384, 1, 1], LineBox[{208, 209}]}, 
        {Hue[0.969304645903326, 1, 1], LineBox[{209, 210}]}, 
        {Hue[0.25270882801694694`, 1, 1], LineBox[{210, 211}]}, 
        {Hue[0.5294894977396489, 1, 1], LineBox[{211, 212}]}, 
        {Hue[0.8021274837723045, 1, 1], LineBox[{212, 213}]}, 
        {Hue[0.07220163138290007, 1, 1], LineBox[{213, 214}]}, 
        {Hue[0.3404993776492227, 1, 1], LineBox[{214, 215}]}, 
        {Hue[0.607341117597648, 1, 1], LineBox[{215, 216}]}, 
        {Hue[0.8728024358639596, 1, 1], LineBox[{216, 217}]}, 
        {Hue[0.13683263563569506`, 1, 1], LineBox[{217, 218}]}, 
        {Hue[0.3993081326856766, 1, 1], LineBox[{218, 219}]}, 
        {Hue[0.6600489781726215, 1, 1], LineBox[{219, 220}]}, 
        {Hue[0.9188134565589243, 1, 1], LineBox[{220, 221}]}, 
        {Hue[0.17528128961485592`, 1, 1], LineBox[{221, 222}]}, 
        {Hue[0.42902982578944954`, 1, 1], LineBox[{222, 223}]}, 
        {Hue[0.6795145524088642, 1, 1], LineBox[{223, 224}]}, 
        {Hue[0.9260767691882732, 1, 1], LineBox[{224, 225}]}, 
        {Hue[0.16803549279117072`, 1, 1], LineBox[{225, 226}]}, 
        {Hue[0.40495850769743, 1, 1], LineBox[{226, 227}]}, 
        {Hue[0.6371737992290389, 1, 1], LineBox[{227, 228}]}, 
        {Hue[0.8662893283077272, 1, 1], LineBox[{228, 229}]}, 
        {Hue[0.09505552883252164, 1, 1], LineBox[{229, 230}]}, 
        {Hue[0.3262617090805855, 1, 1], LineBox[{230, 231}]}, 
        {Hue[0.5615905657572104, 1, 1], LineBox[{231, 232}]}, 
        {Hue[0.8014193438577213, 1, 1], LineBox[{232, 233}]}, 
        {Hue[0.04532016587981987, 1, 1], LineBox[{233, 234}]}, 
        {Hue[0.2925849492448882, 1, 1], LineBox[{234, 235}]}, 
        {Hue[0.5425124691739087, 1, 1], LineBox[{235, 236}]}, 
        {Hue[0.7945105484655892, 1, 1], LineBox[{236, 237}]}, 
        {Hue[0.048109368347345416`, 1, 1], LineBox[{237, 238}]}, 
        {Hue[0.3029441434058117, 1, 1], LineBox[{238, 239}]}, 
        {Hue[0.5587316333455382, 1, 1], LineBox[{239, 240}]}, 
        {Hue[0.8152486151476237, 1, 1], LineBox[{240, 241}]}, 
        {Hue[0.07231525149390143, 1, 1], LineBox[{241, 242}]}, 
        {Hue[0.3297838895830974, 1, 1], LineBox[{242, 243}]}, 
        {Hue[0.587531735292107, 1, 1], LineBox[{243, 244}]}, 
        {Hue[0.8454588286735683, 1, 1], LineBox[{244, 245}]}, 
        {Hue[0.10348856732185839`, 1, 1], LineBox[{245, 246}]}, 
        {Hue[0.36157113441264427`, 1, 1], LineBox[{246, 247}]}, 
        {Hue[0.6196890157024028, 1, 1], LineBox[{247, 248}]}, 
        {Hue[0.877865442493381, 1, 1], LineBox[{248, 249}]}, 
        {Hue[0.136176524325591, 1, 1], LineBox[{249, 250}]}, 
        {Hue[0.3947708920661652, 1, 1], LineBox[{250, 251}]}}}},
     VertexColors->{
       Hue[0.9407694159419471, 1, 1], 
       Hue[0.8652990571156856, 1, 1], 
       Hue[0.7534371156171316, 1, 1], 
       Hue[0.6481847977203763, 1, 1], 
       Hue[0.5354894789766873, 1, 1], 
       Hue[0.46816222537833896`, 1, 1], 
       Hue[0.35585854458977223`, 1, 1], 
       Hue[0.2817377719638701, 1, 1], 
       Hue[0.14460000879947793`, 1, 1], 
       Hue[0.01469903437668414, 1, 1], 
       Hue[0.8988936709144447, 1, 1], 
       Hue[0.810522269771311, 1, 1], 
       Hue[0.6480896528173896, 1, 1], 
       Hue[0.5591795772773027, 1, 1], 
       Hue[0.4540232922687597, 1, 1], 
       Hue[0.40055182835091585`, 1, 1], 
       Hue[0.27385460721166377`, 1, 1], 
       Hue[0.14121327890295116`, 1, 1], 
       Hue[0.11383263176537513`, 1, 1], 
       Hue[0.9936820338959563, 1, 1], 
       Hue[0.9282939753793533, 1, 1], 
       Hue[0.812205984476198, 1, 1], 
       Hue[0.6632661263018338, 1, 1], 
       Hue[0.6142780797353196, 1, 1], 
       Hue[0.5144926704498913, 1, 1], 
       Hue[0.5040634907503884, 1, 1], 
       Hue[0.43416845221881806`, 1, 1], 
       Hue[0.26450211931177586`, 1, 1], 
       Hue[0.11758076126626354`, 1, 1], 
       Hue[0.9354851250970341, 1, 1], 
       Hue[0.7503898389339675, 1, 1], 
       Hue[0.9769323213302773, 1, 1], 
       Hue[0.932109019870338, 1, 1], 
       Hue[0.6669326654057793, 1, 1], 
       Hue[0.43016993026155, 1, 1], 
       Hue[0.39716477337106854`, 1, 1], 
       Hue[0.36493756558008444`, 1, 1], 
       Hue[0.4080060559607373, 1, 1], 
       Hue[0.4309672884697319, 1, 1], 
       Hue[0.5565706017286116, 1, 1], 
       Hue[0.6190552894798738, 1, 1], 
       Hue[0.6934773859825238, 1, 1], 
       Hue[0.7431865445042352, 1, 1], 
       Hue[0.7979053944788896, 1, 1], 
       Hue[0.8639723377143136, 1, 1], 
       Hue[0.931354074203931, 1, 1], 
       Hue[0.0019813757552162815`, 1, 1], 
       Hue[0.0721006651445593, 1, 1], 
       Hue[0.13836744610171361`, 1, 1], 
       Hue[0.20386232588584846`, 1, 1], 
       Hue[0.27293800925391504`, 1, 1], 
       Hue[0.3407829136148149, 1, 1], 
       Hue[0.41103261850344186`, 1, 1], 
       Hue[0.48015625333872314`, 1, 1], 
       Hue[0.5500525672930505, 1, 1], 
       Hue[0.620445851492982, 1, 1], 
       Hue[0.6917554854759792, 1, 1], 
       Hue[0.7634716246789225, 1, 1], 
       Hue[0.8356895882785431, 1, 1], 
       Hue[0.9082214829959486, 1, 1], 
       Hue[0.9815323348687448, 1, 1], 
       Hue[0.055317258485876146`, 1, 1], 
       Hue[0.12961575956451213`, 1, 1], 
       Hue[0.20433090079601726`, 1, 1], 
       Hue[0.2795540383337438, 1, 1], 
       Hue[0.35524739473256994`, 1, 1], 
       Hue[0.4315296393312207, 1, 1], 
       Hue[0.5082753399542235, 1, 1], 
       Hue[0.5855122705463676, 1, 1], 
       Hue[0.6632910732135462, 1, 1], 
       Hue[0.7415493785590683, 1, 1], 
       Hue[0.8203168163406356, 1, 1], 
       Hue[0.899594466264572, 1, 1], 
       Hue[0.9793808535751918, 1, 1], 
       Hue[0.05966611579066608, 1, 1], 
       Hue[0.14046103040636695`, 1, 1], 
       Hue[0.22176099485644657`, 1, 1], 
       Hue[0.30355640042518384`, 1, 1], 
       Hue[0.3858558561080394, 1, 1], 
       Hue[0.46865920579523773`, 1, 1], 
       Hue[0.5519572630975732, 1, 1], 
       Hue[0.6357616925070961, 1, 1], 
       Hue[0.7200614226226909, 1, 1], 
       Hue[0.8048540983332827, 1, 1], 
       Hue[0.8901452092823243, 1, 1], 
       Hue[0.9759256850504838, 1, 1], 
       Hue[0.06219940245881586, 1, 1], 
       Hue[0.14895952789651382`, 1, 1], 
       Hue[0.23620276109148372`, 1, 1], 
       Hue[0.32392749417342787`, 1, 1], 
       Hue[0.41212937430084273`, 1, 1], 
       Hue[0.5008039055547484, 1, 1], 
       Hue[0.5899448498811499, 1, 1], 
       Hue[0.6795444249013187, 1, 1], 
       Hue[0.7695963834453866, 1, 1], 
       Hue[0.8600924523736628, 1, 1], 
       Hue[0.9510219118737493, 1, 1], 
       Hue[0.04237273932485363, 1, 1], 
       Hue[0.1341303083120591, 1, 1], 
       Hue[0.2262799008469206, 1, 1], 
       Hue[0.318803207291762, 1, 1], 
       Hue[0.41167848966155357`, 1, 1], 
       Hue[0.5048808051332369, 1, 1], 
       Hue[0.5983810634330753, 1, 1], 
       Hue[0.6921470473254048, 1, 1], 
       Hue[0.7861407557274118, 1, 1], 
       Hue[0.8803183444062389, 1, 1], 
       Hue[0.9746296130822348, 1, 1], 
       Hue[0.06901767356908702, 1, 1], 
       Hue[0.1634186711999563, 1, 1], 
       Hue[0.257759298319671, 1, 1], 
       Hue[0.35195625424943705`, 1, 1], 
       Hue[0.4459170200063353, 1, 1], 
       Hue[0.5395389647253133, 1, 1], 
       Hue[0.6327115014530438, 1, 1], 
       Hue[0.7253120626933802, 1, 1], 
       Hue[0.8172080503626297, 1, 1], 
       Hue[0.9082571817465559, 1, 1], 
       Hue[0.9983125469259199, 1, 1], 
       Hue[0.0872270893257851, 1, 1], 
       Hue[0.17485685640692983`, 1, 1], 
       Hue[0.26106162246570447`, 1, 1], 
       Hue[0.3457071509121968, 1, 1], 
       Hue[0.4286721641297487, 1, 1], 
       Hue[0.5098651650273955, 1, 1], 
       Hue[0.5892417687680118, 1, 1], 
       Hue[0.6668211853508332, 1, 1], 
       Hue[0.7426906112473048, 1, 1], 
       Hue[0.8170015469243537, 1, 1], 
       Hue[0.8899533966442782, 1, 1], 
       Hue[0.9617783185528035, 1, 1], 
       Hue[0.03272320056978751, 1, 1], 
       Hue[0.10303459335594814`, 1, 1], 
       Hue[0.17294878870530267`, 1, 1], 
       Hue[0.2426863090271892, 1, 1], 
       Hue[0.31244541224626743`, 1, 1], 
       Hue[0.3824019140435007, 1, 1], 
       Hue[0.4527103697766939, 1, 1], 
       Hue[0.5235041204448956, 1, 1], 
       Hue[0.5948940264243402, 1, 1], 
       Hue[0.6669759737360023, 1, 1], 
       Hue[0.7398286321251204, 1, 1], 
       Hue[0.8135130404584696, 1, 1], 
       Hue[0.8880832031234254, 1, 1], 
       Hue[0.9635811958810462, 1, 1], 
       Hue[0.04003531910755127, 1, 1], 
       Hue[0.11746914394991234`, 1, 1], 
       Hue[0.195904600981633, 1, 1], 
       Hue[0.27535047794179796`, 1, 1], 
       Hue[0.3558085223791187, 1, 1], 
       Hue[0.4372840484618632, 1, 1], 
       Hue[0.5197882895126544, 1, 1], 
       Hue[0.6033219242113339, 1, 1], 
       Hue[0.6878536441947171, 1, 1], 
       Hue[0.7733697728793487, 1, 1], 
       Hue[0.8598942801945528, 1, 1], 
       Hue[0.9474247816814833, 1, 1], 
       Hue[0.035907408921143254`, 1, 1], 
       Hue[0.1253083000319576, 1, 1], 
       Hue[0.2156850527530413, 1, 1], 
       Hue[0.30709160770072824`, 1, 1], 
       Hue[0.3994547063543386, 1, 1], 
       Hue[0.4926921458642045, 1, 1], 
       Hue[0.5868410658775327, 1, 1], 
       Hue[0.6819856197390084, 1, 1], 
       Hue[0.7780905715872407, 1, 1], 
       Hue[0.8747788347944276, 1, 1], 
       Hue[0.9716418911349147, 1, 1], 
       Hue[0.06920173882244322, 1, 1], 
       Hue[0.16854962278205396`, 1, 1], 
       Hue[0.2695185878541521, 1, 1], 
       Hue[0.3707722030072139, 1, 1], 
       Hue[0.47160607498768725`, 1, 1], 
       Hue[0.5721384519808977, 1, 1], 
       Hue[0.6738855996878312, 1, 1], 
       Hue[0.78145634117785, 1, 1], 
       Hue[0.8966100687768956, 1, 1], 
       Hue[0.00987144155480758, 1, 1], 
       Hue[0.10665989586565482`, 1, 1], 
       Hue[0.2017755608777928, 1, 1], 
       Hue[0.3243851462421015, 1, 1], 
       Hue[0.46035977061619515`, 1, 1], 
       Hue[0.5698711687553977, 1, 1], 
       Hue[0.6446141696538195, 1, 1], 
       Hue[0.7649959746908312, 1, 1], 
       Hue[0.8499991304409061, 1, 1], 
       Hue[0.7527311439527943, 1, 1], 
       Hue[0.9407324783675022, 1, 1], 
       Hue[0.16279964711927528`, 1, 1], 
       Hue[0.40538122810309657`, 1, 1], 
       Hue[0.7662466372646557, 1, 1], 
       Hue[0.325746588547473, 1, 1], 
       Hue[0.6014346559379302, 1, 1], 
       Hue[0.8702034520468045, 1, 1], 
       Hue[0.14000852383014625`, 1, 1], 
       Hue[0.4094397952932074, 1, 1], 
       Hue[0.6768951372489489, 1, 1], 
       Hue[0.9421369682877887, 1, 1], 
       Hue[0.2058797630336536, 1, 1], 
       Hue[0.4690627681848623, 1, 1], 
       Hue[0.7324642750692172, 1, 1], 
       Hue[0.996675128822844, 1, 1], 
       Hue[0.2623619251828773, 1, 1], 
       Hue[0.5308036556013873, 1, 1], 
       Hue[0.8045450818299433, 1, 1], 
       Hue[0.08721915474396433, 1, 1], 
       Hue[0.38020271952928, 1, 1], 
       Hue[0.6773830849870384, 1, 1], 
       Hue[0.969304645903326, 1, 1], 
       Hue[0.25270882801694694`, 1, 1], 
       Hue[0.5294894977396489, 1, 1], 
       Hue[0.8021274837723045, 1, 1], 
       Hue[0.07220163138290007, 1, 1], 
       Hue[0.3404993776492227, 1, 1], 
       Hue[0.607341117597648, 1, 1], 
       Hue[0.8728024358639596, 1, 1], 
       Hue[0.13683263563569506`, 1, 1], 
       Hue[0.3993081326856766, 1, 1], 
       Hue[0.6600489781726215, 1, 1], 
       Hue[0.9188134565589243, 1, 1], 
       Hue[0.17528128961485592`, 1, 1], 
       Hue[0.42902982578944954`, 1, 1], 
       Hue[0.6795145524088642, 1, 1], 
       Hue[0.9260767691882732, 1, 1], 
       Hue[0.16803549279117072`, 1, 1], 
       Hue[0.40495850769743, 1, 1], 
       Hue[0.6371737992290389, 1, 1], 
       Hue[0.8662893283077272, 1, 1], 
       Hue[0.09505552883252164, 1, 1], 
       Hue[0.3262617090805855, 1, 1], 
       Hue[0.5615905657572104, 1, 1], 
       Hue[0.8014193438577213, 1, 1], 
       Hue[0.04532016587981987, 1, 1], 
       Hue[0.2925849492448882, 1, 1], 
       Hue[0.5425124691739087, 1, 1], 
       Hue[0.7945105484655892, 1, 1], 
       Hue[0.048109368347345416`, 1, 1], 
       Hue[0.3029441434058117, 1, 1], 
       Hue[0.5587316333455382, 1, 1], 
       Hue[0.8152486151476237, 1, 1], 
       Hue[0.07231525149390143, 1, 1], 
       Hue[0.3297838895830974, 1, 1], 
       Hue[0.587531735292107, 1, 1], 
       Hue[0.8454588286735683, 1, 1], 
       Hue[0.10348856732185839`, 1, 1], 
       Hue[0.36157113441264427`, 1, 1], 
       Hue[0.6196890157024028, 1, 1], 
       Hue[0.877865442493381, 1, 1], 
       Hue[0.136176524325591, 1, 1], 
       Hue[0.3947708920661652, 1, 1], 
       Hue[0.6539029141821653, 1, 1], 
       Hue[0.9407694159419471, 1, 1], 
       Hue[0.8652990571156856, 1, 1], 
       Hue[0.8652990571156856, 1, 1], 
       Hue[0.7534371156171316, 1, 1], 
       Hue[0.7534371156171316, 1, 1], 
       Hue[0.6481847977203763, 1, 1], 
       Hue[0.6481847977203763, 1, 1], 
       Hue[0.5354894789766873, 1, 1], 
       Hue[0.5354894789766873, 1, 1], 
       Hue[0.46816222537833896`, 1, 1], 
       Hue[0.46816222537833896`, 1, 1], 
       Hue[0.35585854458977223`, 1, 1], 
       Hue[0.35585854458977223`, 1, 1], 
       Hue[0.2817377719638701, 1, 1], 
       Hue[0.2817377719638701, 1, 1], 
       Hue[0.14460000879947793`, 1, 1], 
       Hue[0.14460000879947793`, 1, 1], 
       Hue[0.01469903437668414, 1, 1], 
       Hue[0.01469903437668414, 1, 1], 
       Hue[0.8988936709144447, 1, 1], 
       Hue[0.8988936709144447, 1, 1], 
       Hue[0.810522269771311, 1, 1], 
       Hue[0.810522269771311, 1, 1], 
       Hue[0.6480896528173896, 1, 1], 
       Hue[0.6480896528173896, 1, 1], 
       Hue[0.5591795772773027, 1, 1], 
       Hue[0.5591795772773027, 1, 1], 
       Hue[0.4540232922687597, 1, 1], 
       Hue[0.4540232922687597, 1, 1], 
       Hue[0.40055182835091585`, 1, 1], 
       Hue[0.40055182835091585`, 1, 1], 
       Hue[0.27385460721166377`, 1, 1], 
       Hue[0.27385460721166377`, 1, 1], 
       Hue[0.14121327890295116`, 1, 1], 
       Hue[0.14121327890295116`, 1, 1], 
       Hue[0.11383263176537513`, 1, 1], 
       Hue[0.11383263176537513`, 1, 1], 
       Hue[0.9936820338959563, 1, 1], 
       Hue[0.9936820338959563, 1, 1], 
       Hue[0.9282939753793533, 1, 1], 
       Hue[0.9282939753793533, 1, 1], 
       Hue[0.812205984476198, 1, 1], 
       Hue[0.812205984476198, 1, 1], 
       Hue[0.6632661263018338, 1, 1], 
       Hue[0.6632661263018338, 1, 1], 
       Hue[0.6142780797353196, 1, 1], 
       Hue[0.6142780797353196, 1, 1], 
       Hue[0.5144926704498913, 1, 1], 
       Hue[0.5144926704498913, 1, 1], 
       Hue[0.5040634907503884, 1, 1], 
       Hue[0.5040634907503884, 1, 1], 
       Hue[0.43416845221881806`, 1, 1], 
       Hue[0.43416845221881806`, 1, 1], 
       Hue[0.26450211931177586`, 1, 1], 
       Hue[0.26450211931177586`, 1, 1], 
       Hue[0.11758076126626354`, 1, 1], 
       Hue[0.11758076126626354`, 1, 1], 
       Hue[0.9354851250970341, 1, 1], 
       Hue[0.9354851250970341, 1, 1], 
       Hue[0.7503898389339675, 1, 1], 
       Hue[0.7503898389339675, 1, 1], 
       Hue[0.9769323213302773, 1, 1], 
       Hue[0.9769323213302773, 1, 1], 
       Hue[0.932109019870338, 1, 1], 
       Hue[0.932109019870338, 1, 1], 
       Hue[0.6669326654057793, 1, 1], 
       Hue[0.6669326654057793, 1, 1], 
       Hue[0.43016993026155, 1, 1], 
       Hue[0.43016993026155, 1, 1], 
       Hue[0.39716477337106854`, 1, 1], 
       Hue[0.39716477337106854`, 1, 1], 
       Hue[0.36493756558008444`, 1, 1], 
       Hue[0.36493756558008444`, 1, 1], 
       Hue[0.4080060559607373, 1, 1], 
       Hue[0.4080060559607373, 1, 1], 
       Hue[0.4309672884697319, 1, 1], 
       Hue[0.4309672884697319, 1, 1], 
       Hue[0.5565706017286116, 1, 1], 
       Hue[0.5565706017286116, 1, 1], 
       Hue[0.6190552894798738, 1, 1], 
       Hue[0.6190552894798738, 1, 1], 
       Hue[0.6934773859825238, 1, 1], 
       Hue[0.6934773859825238, 1, 1], 
       Hue[0.7431865445042352, 1, 1], 
       Hue[0.7431865445042352, 1, 1], 
       Hue[0.7979053944788896, 1, 1], 
       Hue[0.7979053944788896, 1, 1], 
       Hue[0.8639723377143136, 1, 1], 
       Hue[0.8639723377143136, 1, 1], 
       Hue[0.931354074203931, 1, 1], 
       Hue[0.931354074203931, 1, 1], 
       Hue[0.0019813757552162815`, 1, 1], 
       Hue[0.0019813757552162815`, 1, 1], 
       Hue[0.0721006651445593, 1, 1], 
       Hue[0.0721006651445593, 1, 1], 
       Hue[0.13836744610171361`, 1, 1], 
       Hue[0.13836744610171361`, 1, 1], 
       Hue[0.20386232588584846`, 1, 1], 
       Hue[0.20386232588584846`, 1, 1], 
       Hue[0.27293800925391504`, 1, 1], 
       Hue[0.27293800925391504`, 1, 1], 
       Hue[0.3407829136148149, 1, 1], 
       Hue[0.3407829136148149, 1, 1], 
       Hue[0.41103261850344186`, 1, 1], 
       Hue[0.41103261850344186`, 1, 1], 
       Hue[0.48015625333872314`, 1, 1], 
       Hue[0.48015625333872314`, 1, 1], 
       Hue[0.5500525672930505, 1, 1], 
       Hue[0.5500525672930505, 1, 1], 
       Hue[0.620445851492982, 1, 1], 
       Hue[0.620445851492982, 1, 1], 
       Hue[0.6917554854759792, 1, 1], 
       Hue[0.6917554854759792, 1, 1], 
       Hue[0.7634716246789225, 1, 1], 
       Hue[0.7634716246789225, 1, 1], 
       Hue[0.8356895882785431, 1, 1], 
       Hue[0.8356895882785431, 1, 1], 
       Hue[0.9082214829959486, 1, 1], 
       Hue[0.9082214829959486, 1, 1], 
       Hue[0.9815323348687448, 1, 1], 
       Hue[0.9815323348687448, 1, 1], 
       Hue[0.055317258485876146`, 1, 1], 
       Hue[0.055317258485876146`, 1, 1], 
       Hue[0.12961575956451213`, 1, 1], 
       Hue[0.12961575956451213`, 1, 1], 
       Hue[0.20433090079601726`, 1, 1], 
       Hue[0.20433090079601726`, 1, 1], 
       Hue[0.2795540383337438, 1, 1], 
       Hue[0.2795540383337438, 1, 1], 
       Hue[0.35524739473256994`, 1, 1], 
       Hue[0.35524739473256994`, 1, 1], 
       Hue[0.4315296393312207, 1, 1], 
       Hue[0.4315296393312207, 1, 1], 
       Hue[0.5082753399542235, 1, 1], 
       Hue[0.5082753399542235, 1, 1], 
       Hue[0.5855122705463676, 1, 1], 
       Hue[0.5855122705463676, 1, 1], 
       Hue[0.6632910732135462, 1, 1], 
       Hue[0.6632910732135462, 1, 1], 
       Hue[0.7415493785590683, 1, 1], 
       Hue[0.7415493785590683, 1, 1], 
       Hue[0.8203168163406356, 1, 1], 
       Hue[0.8203168163406356, 1, 1], 
       Hue[0.899594466264572, 1, 1], 
       Hue[0.899594466264572, 1, 1], 
       Hue[0.9793808535751918, 1, 1], 
       Hue[0.9793808535751918, 1, 1], 
       Hue[0.05966611579066608, 1, 1], 
       Hue[0.05966611579066608, 1, 1], 
       Hue[0.14046103040636695`, 1, 1], 
       Hue[0.14046103040636695`, 1, 1], 
       Hue[0.22176099485644657`, 1, 1], 
       Hue[0.22176099485644657`, 1, 1], 
       Hue[0.30355640042518384`, 1, 1], 
       Hue[0.30355640042518384`, 1, 1], 
       Hue[0.3858558561080394, 1, 1], 
       Hue[0.3858558561080394, 1, 1], 
       Hue[0.46865920579523773`, 1, 1], 
       Hue[0.46865920579523773`, 1, 1], 
       Hue[0.5519572630975732, 1, 1], 
       Hue[0.5519572630975732, 1, 1], 
       Hue[0.6357616925070961, 1, 1], 
       Hue[0.6357616925070961, 1, 1], 
       Hue[0.7200614226226909, 1, 1], 
       Hue[0.7200614226226909, 1, 1], 
       Hue[0.8048540983332827, 1, 1], 
       Hue[0.8048540983332827, 1, 1], 
       Hue[0.8901452092823243, 1, 1], 
       Hue[0.8901452092823243, 1, 1], 
       Hue[0.9759256850504838, 1, 1], 
       Hue[0.9759256850504838, 1, 1], 
       Hue[0.06219940245881586, 1, 1], 
       Hue[0.06219940245881586, 1, 1], 
       Hue[0.14895952789651382`, 1, 1], 
       Hue[0.14895952789651382`, 1, 1], 
       Hue[0.23620276109148372`, 1, 1], 
       Hue[0.23620276109148372`, 1, 1], 
       Hue[0.32392749417342787`, 1, 1], 
       Hue[0.32392749417342787`, 1, 1], 
       Hue[0.41212937430084273`, 1, 1], 
       Hue[0.41212937430084273`, 1, 1], 
       Hue[0.5008039055547484, 1, 1], 
       Hue[0.5008039055547484, 1, 1], 
       Hue[0.5899448498811499, 1, 1], 
       Hue[0.5899448498811499, 1, 1], 
       Hue[0.6795444249013187, 1, 1], 
       Hue[0.6795444249013187, 1, 1], 
       Hue[0.7695963834453866, 1, 1], 
       Hue[0.7695963834453866, 1, 1], 
       Hue[0.8600924523736628, 1, 1], 
       Hue[0.8600924523736628, 1, 1], 
       Hue[0.9510219118737493, 1, 1], 
       Hue[0.9510219118737493, 1, 1], 
       Hue[0.04237273932485363, 1, 1], 
       Hue[0.04237273932485363, 1, 1], 
       Hue[0.1341303083120591, 1, 1], 
       Hue[0.1341303083120591, 1, 1], 
       Hue[0.2262799008469206, 1, 1], 
       Hue[0.2262799008469206, 1, 1], 
       Hue[0.318803207291762, 1, 1], 
       Hue[0.318803207291762, 1, 1], 
       Hue[0.41167848966155357`, 1, 1], 
       Hue[0.41167848966155357`, 1, 1], 
       Hue[0.5048808051332369, 1, 1], 
       Hue[0.5048808051332369, 1, 1], 
       Hue[0.5983810634330753, 1, 1], 
       Hue[0.5983810634330753, 1, 1], 
       Hue[0.6921470473254048, 1, 1], 
       Hue[0.6921470473254048, 1, 1], 
       Hue[0.7861407557274118, 1, 1], 
       Hue[0.7861407557274118, 1, 1], 
       Hue[0.8803183444062389, 1, 1], 
       Hue[0.8803183444062389, 1, 1], 
       Hue[0.9746296130822348, 1, 1], 
       Hue[0.9746296130822348, 1, 1], 
       Hue[0.06901767356908702, 1, 1], 
       Hue[0.06901767356908702, 1, 1], 
       Hue[0.1634186711999563, 1, 1], 
       Hue[0.1634186711999563, 1, 1], 
       Hue[0.257759298319671, 1, 1], 
       Hue[0.257759298319671, 1, 1], 
       Hue[0.35195625424943705`, 1, 1], 
       Hue[0.35195625424943705`, 1, 1], 
       Hue[0.4459170200063353, 1, 1], 
       Hue[0.4459170200063353, 1, 1], 
       Hue[0.5395389647253133, 1, 1], 
       Hue[0.5395389647253133, 1, 1], 
       Hue[0.6327115014530438, 1, 1], 
       Hue[0.6327115014530438, 1, 1], 
       Hue[0.7253120626933802, 1, 1], 
       Hue[0.7253120626933802, 1, 1], 
       Hue[0.8172080503626297, 1, 1], 
       Hue[0.8172080503626297, 1, 1], 
       Hue[0.9082571817465559, 1, 1], 
       Hue[0.9082571817465559, 1, 1], 
       Hue[0.9983125469259199, 1, 1], 
       Hue[0.9983125469259199, 1, 1], 
       Hue[0.0872270893257851, 1, 1], 
       Hue[0.0872270893257851, 1, 1], 
       Hue[0.17485685640692983`, 1, 1], 
       Hue[0.17485685640692983`, 1, 1], 
       Hue[0.26106162246570447`, 1, 1], 
       Hue[0.26106162246570447`, 1, 1], 
       Hue[0.3457071509121968, 1, 1], 
       Hue[0.3457071509121968, 1, 1], 
       Hue[0.4286721641297487, 1, 1], 
       Hue[0.4286721641297487, 1, 1], 
       Hue[0.5098651650273955, 1, 1], 
       Hue[0.5098651650273955, 1, 1], 
       Hue[0.5892417687680118, 1, 1], 
       Hue[0.5892417687680118, 1, 1], 
       Hue[0.6668211853508332, 1, 1], 
       Hue[0.6668211853508332, 1, 1], 
       Hue[0.7426906112473048, 1, 1], 
       Hue[0.7426906112473048, 1, 1], 
       Hue[0.8170015469243537, 1, 1], 
       Hue[0.8170015469243537, 1, 1], 
       Hue[0.8899533966442782, 1, 1], 
       Hue[0.8899533966442782, 1, 1], 
       Hue[0.9617783185528035, 1, 1], 
       Hue[0.9617783185528035, 1, 1], 
       Hue[0.03272320056978751, 1, 1], 
       Hue[0.03272320056978751, 1, 1], 
       Hue[0.10303459335594814`, 1, 1], 
       Hue[0.10303459335594814`, 1, 1], 
       Hue[0.17294878870530267`, 1, 1], 
       Hue[0.17294878870530267`, 1, 1], 
       Hue[0.2426863090271892, 1, 1], 
       Hue[0.2426863090271892, 1, 1], 
       Hue[0.31244541224626743`, 1, 1], 
       Hue[0.31244541224626743`, 1, 1], 
       Hue[0.3824019140435007, 1, 1], 
       Hue[0.3824019140435007, 1, 1], 
       Hue[0.4527103697766939, 1, 1], 
       Hue[0.4527103697766939, 1, 1], 
       Hue[0.5235041204448956, 1, 1], 
       Hue[0.5235041204448956, 1, 1], 
       Hue[0.5948940264243402, 1, 1], 
       Hue[0.5948940264243402, 1, 1], 
       Hue[0.6669759737360023, 1, 1], 
       Hue[0.6669759737360023, 1, 1], 
       Hue[0.7398286321251204, 1, 1], 
       Hue[0.7398286321251204, 1, 1], 
       Hue[0.8135130404584696, 1, 1], 
       Hue[0.8135130404584696, 1, 1], 
       Hue[0.8880832031234254, 1, 1], 
       Hue[0.8880832031234254, 1, 1], 
       Hue[0.9635811958810462, 1, 1], 
       Hue[0.9635811958810462, 1, 1], 
       Hue[0.04003531910755127, 1, 1], 
       Hue[0.04003531910755127, 1, 1], 
       Hue[0.11746914394991234`, 1, 1], 
       Hue[0.11746914394991234`, 1, 1], 
       Hue[0.195904600981633, 1, 1], 
       Hue[0.195904600981633, 1, 1], 
       Hue[0.27535047794179796`, 1, 1], 
       Hue[0.27535047794179796`, 1, 1], 
       Hue[0.3558085223791187, 1, 1], 
       Hue[0.3558085223791187, 1, 1], 
       Hue[0.4372840484618632, 1, 1], 
       Hue[0.4372840484618632, 1, 1], 
       Hue[0.5197882895126544, 1, 1], 
       Hue[0.5197882895126544, 1, 1], 
       Hue[0.6033219242113339, 1, 1], 
       Hue[0.6033219242113339, 1, 1], 
       Hue[0.6878536441947171, 1, 1], 
       Hue[0.6878536441947171, 1, 1], 
       Hue[0.7733697728793487, 1, 1], 
       Hue[0.7733697728793487, 1, 1], 
       Hue[0.8598942801945528, 1, 1], 
       Hue[0.8598942801945528, 1, 1], 
       Hue[0.9474247816814833, 1, 1], 
       Hue[0.9474247816814833, 1, 1], 
       Hue[0.035907408921143254`, 1, 1], 
       Hue[0.035907408921143254`, 1, 1], 
       Hue[0.1253083000319576, 1, 1], 
       Hue[0.1253083000319576, 1, 1], 
       Hue[0.2156850527530413, 1, 1], 
       Hue[0.2156850527530413, 1, 1], 
       Hue[0.30709160770072824`, 1, 1], 
       Hue[0.30709160770072824`, 1, 1], 
       Hue[0.3994547063543386, 1, 1], 
       Hue[0.3994547063543386, 1, 1], 
       Hue[0.4926921458642045, 1, 1], 
       Hue[0.4926921458642045, 1, 1], 
       Hue[0.5868410658775327, 1, 1], 
       Hue[0.5868410658775327, 1, 1], 
       Hue[0.6819856197390084, 1, 1], 
       Hue[0.6819856197390084, 1, 1], 
       Hue[0.7780905715872407, 1, 1], 
       Hue[0.7780905715872407, 1, 1], 
       Hue[0.8747788347944276, 1, 1], 
       Hue[0.8747788347944276, 1, 1], 
       Hue[0.9716418911349147, 1, 1], 
       Hue[0.9716418911349147, 1, 1], 
       Hue[0.06920173882244322, 1, 1], 
       Hue[0.06920173882244322, 1, 1], 
       Hue[0.16854962278205396`, 1, 1], 
       Hue[0.16854962278205396`, 1, 1], 
       Hue[0.2695185878541521, 1, 1], 
       Hue[0.2695185878541521, 1, 1], 
       Hue[0.3707722030072139, 1, 1], 
       Hue[0.3707722030072139, 1, 1], 
       Hue[0.47160607498768725`, 1, 1], 
       Hue[0.47160607498768725`, 1, 1], 
       Hue[0.5721384519808977, 1, 1], 
       Hue[0.5721384519808977, 1, 1], 
       Hue[0.6738855996878312, 1, 1], 
       Hue[0.6738855996878312, 1, 1], 
       Hue[0.78145634117785, 1, 1], 
       Hue[0.78145634117785, 1, 1], 
       Hue[0.8966100687768956, 1, 1], 
       Hue[0.8966100687768956, 1, 1], 
       Hue[0.00987144155480758, 1, 1], 
       Hue[0.00987144155480758, 1, 1], 
       Hue[0.10665989586565482`, 1, 1], 
       Hue[0.10665989586565482`, 1, 1], 
       Hue[0.2017755608777928, 1, 1], 
       Hue[0.2017755608777928, 1, 1], 
       Hue[0.3243851462421015, 1, 1], 
       Hue[0.3243851462421015, 1, 1], 
       Hue[0.46035977061619515`, 1, 1], 
       Hue[0.46035977061619515`, 1, 1], 
       Hue[0.5698711687553977, 1, 1], 
       Hue[0.5698711687553977, 1, 1], 
       Hue[0.6446141696538195, 1, 1], 
       Hue[0.6446141696538195, 1, 1], 
       Hue[0.7649959746908312, 1, 1], 
       Hue[0.7649959746908312, 1, 1], 
       Hue[0.8499991304409061, 1, 1], 
       Hue[0.8499991304409061, 1, 1], 
       Hue[0.7527311439527943, 1, 1], 
       Hue[0.7527311439527943, 1, 1], 
       Hue[0.9407324783675022, 1, 1], 
       Hue[0.9407324783675022, 1, 1], 
       Hue[0.16279964711927528`, 1, 1], 
       Hue[0.16279964711927528`, 1, 1], 
       Hue[0.40538122810309657`, 1, 1], 
       Hue[0.40538122810309657`, 1, 1], 
       Hue[0.7662466372646557, 1, 1], 
       Hue[0.7662466372646557, 1, 1], 
       Hue[0.325746588547473, 1, 1], 
       Hue[0.325746588547473, 1, 1], 
       Hue[0.6014346559379302, 1, 1], 
       Hue[0.6014346559379302, 1, 1], 
       Hue[0.8702034520468045, 1, 1], 
       Hue[0.8702034520468045, 1, 1], 
       Hue[0.14000852383014625`, 1, 1], 
       Hue[0.14000852383014625`, 1, 1], 
       Hue[0.4094397952932074, 1, 1], 
       Hue[0.4094397952932074, 1, 1], 
       Hue[0.6768951372489489, 1, 1], 
       Hue[0.6768951372489489, 1, 1], 
       Hue[0.9421369682877887, 1, 1], 
       Hue[0.9421369682877887, 1, 1], 
       Hue[0.2058797630336536, 1, 1], 
       Hue[0.2058797630336536, 1, 1], 
       Hue[0.4690627681848623, 1, 1], 
       Hue[0.4690627681848623, 1, 1], 
       Hue[0.7324642750692172, 1, 1], 
       Hue[0.7324642750692172, 1, 1], 
       Hue[0.996675128822844, 1, 1], 
       Hue[0.996675128822844, 1, 1], 
       Hue[0.2623619251828773, 1, 1], 
       Hue[0.2623619251828773, 1, 1], 
       Hue[0.5308036556013873, 1, 1], 
       Hue[0.5308036556013873, 1, 1], 
       Hue[0.8045450818299433, 1, 1], 
       Hue[0.8045450818299433, 1, 1], 
       Hue[0.08721915474396433, 1, 1], 
       Hue[0.08721915474396433, 1, 1], 
       Hue[0.38020271952928, 1, 1], 
       Hue[0.38020271952928, 1, 1], 
       Hue[0.6773830849870384, 1, 1], 
       Hue[0.6773830849870384, 1, 1], 
       Hue[0.969304645903326, 1, 1], 
       Hue[0.969304645903326, 1, 1], 
       Hue[0.25270882801694694`, 1, 1], 
       Hue[0.25270882801694694`, 1, 1], 
       Hue[0.5294894977396489, 1, 1], 
       Hue[0.5294894977396489, 1, 1], 
       Hue[0.8021274837723045, 1, 1], 
       Hue[0.8021274837723045, 1, 1], 
       Hue[0.07220163138290007, 1, 1], 
       Hue[0.07220163138290007, 1, 1], 
       Hue[0.3404993776492227, 1, 1], 
       Hue[0.3404993776492227, 1, 1], 
       Hue[0.607341117597648, 1, 1], 
       Hue[0.607341117597648, 1, 1], 
       Hue[0.8728024358639596, 1, 1], 
       Hue[0.8728024358639596, 1, 1], 
       Hue[0.13683263563569506`, 1, 1], 
       Hue[0.13683263563569506`, 1, 1], 
       Hue[0.3993081326856766, 1, 1], 
       Hue[0.3993081326856766, 1, 1], 
       Hue[0.6600489781726215, 1, 1], 
       Hue[0.6600489781726215, 1, 1], 
       Hue[0.9188134565589243, 1, 1], 
       Hue[0.9188134565589243, 1, 1], 
       Hue[0.17528128961485592`, 1, 1], 
       Hue[0.17528128961485592`, 1, 1], 
       Hue[0.42902982578944954`, 1, 1], 
       Hue[0.42902982578944954`, 1, 1], 
       Hue[0.6795145524088642, 1, 1], 
       Hue[0.6795145524088642, 1, 1], 
       Hue[0.9260767691882732, 1, 1], 
       Hue[0.9260767691882732, 1, 1], 
       Hue[0.16803549279117072`, 1, 1], 
       Hue[0.16803549279117072`, 1, 1], 
       Hue[0.40495850769743, 1, 1], 
       Hue[0.40495850769743, 1, 1], 
       Hue[0.6371737992290389, 1, 1], 
       Hue[0.6371737992290389, 1, 1], 
       Hue[0.8662893283077272, 1, 1], 
       Hue[0.8662893283077272, 1, 1], 
       Hue[0.09505552883252164, 1, 1], 
       Hue[0.09505552883252164, 1, 1], 
       Hue[0.3262617090805855, 1, 1], 
       Hue[0.3262617090805855, 1, 1], 
       Hue[0.5615905657572104, 1, 1], 
       Hue[0.5615905657572104, 1, 1], 
       Hue[0.8014193438577213, 1, 1], 
       Hue[0.8014193438577213, 1, 1], 
       Hue[0.04532016587981987, 1, 1], 
       Hue[0.04532016587981987, 1, 1], 
       Hue[0.2925849492448882, 1, 1], 
       Hue[0.2925849492448882, 1, 1], 
       Hue[0.5425124691739087, 1, 1], 
       Hue[0.5425124691739087, 1, 1], 
       Hue[0.7945105484655892, 1, 1], 
       Hue[0.7945105484655892, 1, 1], 
       Hue[0.048109368347345416`, 1, 1], 
       Hue[0.048109368347345416`, 1, 1], 
       Hue[0.3029441434058117, 1, 1], 
       Hue[0.3029441434058117, 1, 1], 
       Hue[0.5587316333455382, 1, 1], 
       Hue[0.5587316333455382, 1, 1], 
       Hue[0.8152486151476237, 1, 1], 
       Hue[0.8152486151476237, 1, 1], 
       Hue[0.07231525149390143, 1, 1], 
       Hue[0.07231525149390143, 1, 1], 
       Hue[0.3297838895830974, 1, 1], 
       Hue[0.3297838895830974, 1, 1], 
       Hue[0.587531735292107, 1, 1], 
       Hue[0.587531735292107, 1, 1], 
       Hue[0.8454588286735683, 1, 1], 
       Hue[0.8454588286735683, 1, 1], 
       Hue[0.10348856732185839`, 1, 1], 
       Hue[0.10348856732185839`, 1, 1], 
       Hue[0.36157113441264427`, 1, 1], 
       Hue[0.36157113441264427`, 1, 1], 
       Hue[0.6196890157024028, 1, 1], 
       Hue[0.6196890157024028, 1, 1], 
       Hue[0.877865442493381, 1, 1], 
       Hue[0.877865442493381, 1, 1], 
       Hue[0.136176524325591, 1, 1], 
       Hue[0.136176524325591, 1, 1], 
       Hue[0.3947708920661652, 1, 1], 
       Hue[0.3947708920661652, 1, 1], 
       Hue[0.6539029141821653, 1, 1]}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->{True, None},
    AxesOrigin->{0, 0},
    Epilog->
     LineBox[{{1, 2.1382858201739083`*^-6}, {2, 2.339060079404161*^-6}, {
       3, 2.376119221526492*^-6}, {4, 2.120555731073843*^-6}, {
       5, 1.8529336061359418`*^-6}, {6, 1.6153949960995193`*^-6}, {
       7, 1.3789852090552707`*^-6}, {8, 1.302415804345069*^-6}, {
       9, 1.501766439045054*^-6}, {10, 1.1276930061497953`*^-6}, {
       11, 1.1879861406833316`*^-6}, {12, 1.193354966904871*^-6}, {
       13, 1.3233768002965314`*^-6}, {14, 1.1680905784284983`*^-6}, {
       15, 1.1667147504799766`*^-6}, {16, 1.507444492921938*^-6}, {
       17, 1.8006348740448443`*^-6}, {18, 1.857867578875766*^-6}, {
       19, 1.3902697453345237`*^-6}, {20, 1.510050681483681*^-6}, {
       21, 1.567813569924143*^-6}, {22, 1.5903914531180477`*^-6}, {
       23, 1.1865065976286821`*^-6}, {24, 1.1054353594364884`*^-6}, {
       25, 7.210588452885815*^-7}, {26, 8.940815105079953*^-7}, {
       27, 5.273881262098304*^-7}, {28, 6.835336546419997*^-7}, {
       29, 4.340082885104023*^-7}, {30, 5.290014758706824*^-7}, {
       31, 2.5051757124893184`*^-7}, {32, 4.966136797615175*^-7}, {
       33, 2.592564196897756*^-7}, {34, 3.4915790359867765`*^-7}, {
       35, 9.30152011544545*^-7}, {36, 1.7623475378555014`*^-6}, {
       37, 2.461035211615893*^-6}, {38, 2.884707192602002*^-6}, {
       39, 3.0230334310874387`*^-6}, {40, 3.5256801542079176`*^-6}, {
       41, 4.505484980139137*^-6}, {42, 7.730591120875744*^-6}, {
       43, 0.000010995019951795655`}, {44, 0.000013647755202971156`}, {
       45, 0.000017440561292533342`}, {46, 0.00002324323038979623}, {
       47, 0.000030360057994099544`}, {48, 0.000040980497481915245`}, {
       49, 0.00005487871616545832}, {50, 0.00007133697413596286}, {
       51, 0.00009289810998376431}, {52, 0.00012106783215629224`}, {
       53, 0.00015717524395985}, {54, 0.00020302805126036442`}, {
       55, 0.00026130598508145616`}, {56, 0.0003337261550141855}, {
       57, 0.00042548660628561474`}, {58, 0.0005408066692216618}, {
       59, 0.000684315054768601}, {60, 0.0008628818638162705}, {
       61, 0.0010825869868716193`}, {62, 0.0013537209241357726`}, {
       63, 0.001687037849989684}, {64, 0.0020945832426354635`}, {
       65, 0.0025904209624385576`}, {66, 0.0031894699452079673`}, {
       67, 0.003911201350898368}, {68, 0.00477828352225269}, {
       69, 0.005815411304276308}, {70, 0.007049280497034606}, {
       71, 0.008510468802000865}, {72, 0.010235691605007152`}, {
       73, 0.012260152492054262`}, {74, 0.014628175286118844`}, {
       75, 0.01738573164227005}, {76, 0.020580502482031342`}, {
       77, 0.024266053760217246`}, {78, 0.028499869783183248`}, {
       79, 0.03334015248560203}, {80, 0.03884839975581055}, {
       81, 0.04508765485524873}, {82, 0.0521222361758455}, {
       83, 0.060018031402123126`}, {84, 0.06883609810206841}, {
       85, 0.07863668597706475}, {86, 0.08947909356075248}, {
       87, 0.10141422313530525`}, {88, 0.11448791636504813`}, {
       89, 0.12873690280441855`}, {90, 0.14418648199119524`}, {
       91, 0.16085331249797363`}, {92, 0.17874005462737072`}, {
       93, 0.19783466439917705`}, {94, 0.2181080970444795}, {
       95, 0.2395117687511355}, {96, 0.26198419912917414`}, {
       97, 0.28544343387091464`}, {98, 0.3097868490238605}, {
       99, 0.33489443146655284`}, {100, 0.36062502566214794`}, {
       101, 0.38682252494012076`}, {102, 0.4133155533673882}, {
       103, 0.43991919370791405`}, {104, 0.4664300189745512}, {
       105, 0.4926381883665389}, {106, 0.5183249534190161}, {
       107, 0.5432729644459915}, {108, 0.5672531321032863}, {
       109, 0.5900413228532021}, {110, 0.6114159746716843}, {
       111, 0.6311649709831127}, {112, 0.6490799492485502}, {
       113, 0.6649576898228602}, {114, 0.6786032629691555}, {
       115, 0.689831306803855}, {116, 0.6984709989259636}, {
       117, 0.7043562745743857}, {118, 0.7073213525534522}, {
       119, 0.7071865721206809}, {120, 0.7037745871626101}, {
       121, 0.6969171152803978}, {122, 0.6864704926454155}, {
       123, 0.672297028395317}, {124, 0.6542516273464094}, {
       125, 0.6321816205350365}, {126, 0.6059782408152176}, {
       127, 0.575654936274103}, {128, 0.5414445591938546}, {
       129, 0.5038454476027572}, {130, 0.4636228790127586}, {
       131, 0.4217439917114315}, {132, 0.3792768951817269}, {
       133, 0.33727978102526346`}, {134, 0.29670854278782666`}, {
       135, 0.25834692225007083`}, {136, 0.22277412642377586`}, {
       137, 0.19036272275752997`}, {138, 0.16129426094679222`}, {
       139, 0.13558927504375637`}, {140, 0.11314459446037002`}, {
       141, 0.09376949977285415}, {142, 0.07721311145120266}, {
       143, 0.06319658541171463}, {144, 0.05143069853887677}, {
       145, 0.041629078058473934`}, {146, 0.03352153723412999}, {
       147, 0.026860089085101572`}, {148, 0.021420227538107066`}, {
       149, 0.017003089258188206`}, {150, 0.013436445001017506`}, {
       151, 0.010572072002457059`}, {152, 0.008283157490318927}, {
       153, 0.0064620991332260385`}, {154, 0.005020018877321288}, {
       155, 0.0038841587806388612`}, {156, 0.0029935584296353424`}, {
       157, 0.00229772155355313}, {158, 0.0017563471040131194`}, {
       159, 0.0013375003958987185`}, {160, 0.001015028627598602}, {
       161, 0.0007673245357657564}, {162, 0.0005775587687211761}, {
       163, 0.00043298401051007575`}, {164, 0.00032343522327817867`}, {
       165, 0.00024065693104272095`}, {166, 0.00017814000375619864`}, {
       167, 0.000131089456116824}, {168, 0.00009620733453323634}, {
       169, 0.00007075891623862015}, {170, 0.00005202167837452654}, {
       171, 0.000037919710004153324`}, {172, 0.000027370459729067403`}, {
       173, 0.00001969208537012718}, {174, 0.000014234723241473395`}, {
       175, 0.000010455440115850874`}, {176, 7.765678371761643*^-6}, {
       177, 5.563917013442224*^-6}, {178, 3.686566958292898*^-6}, {
       179, 2.420571159035341*^-6}, {180, 1.8288925410163764`*^-6}, {
       181, 1.4119495814965288`*^-6}, {182, 9.117324648688405*^-7}, {
       183, 4.905996001761964*^-7}, {184, 3.208482041388912*^-7}, {
       185, 2.2374557380483393`*^-7}, {186, 7.239111397880053*^-8}, {
       187, 1.3621249555414346`*^-7}, {188, 2.2651663064551017`*^-7}, {
       189, 2.1760248581751497`*^-7}, {190, 1.3323360094581935`*^-7}, {
       191, 2.8605117859651873`*^-8}, {192, 8.93382664466351*^-8}, {
       193, 1.6206649975950964`*^-7}, {194, 2.0636853080860873`*^-7}, {
       195, 2.293018789119855*^-7}, {196, 2.3704008524187569`*^-7}, {
       197, 2.318941389157153*^-7}, {198, 2.1494341561434727`*^-7}, {
       199, 1.8859407010545268`*^-7}, {200, 1.568202962216502*^-7}, {
       201, 1.2400247214876664`*^-7}, {202, 9.373083593807168*^-8}, {
       203, 6.822403132828696*^-8}, {204, 4.836159429191074*^-8}, {
       205, 3.405879680945633*^-8}, {206, 2.4724329773627344`*^-8}, {
       207, 1.9583119490142026`*^-8}, {208, 1.7664377127784197`*^-8}, {
       209, 1.771435317475953*^-8}, {210, 1.8532803912708786`*^-8}, {
       211, 1.930548765974165*^-8}, {212, 1.9599580769626324`*^-8}, {
       213, 1.924833045316238*^-8}, {214, 1.8256067931996337`*^-8}, {
       215, 1.6730222546134842`*^-8}, {216, 1.4830750483790228`*^-8}, {
       217, 1.2732520571241701`*^-8}, {218, 1.0598463786916573`*^-8}, {
       219, 8.56250966648243*^-9}, {220, 6.721092959136898*^-9}, {
       221, 5.131840822242209*^-9}, {222, 3.81759590202339*^-9}, {
       223, 2.7736281729708353`*^-9}, {224, 1.976049425438888*^-9}, {
       225, 1.3897757185833489`*^-9}, {226, 9.75007799863787*^-10}, {
       227, 6.91834731105331*^-10}, {228, 5.033307615248293*^-10}, {
       229, 3.780161128633329*^-10}, {230, 2.9191362980048405`*^-10}, {
       231, 2.2918958748551362`*^-10}, {232, 1.807814771992523*^-10}, {
       233, 1.420172115229306*^-10}, {234, 1.105410705034356*^-10}, {
       235, 8.503124009844324*^-11}, {236, 6.456816906764726*^-11}, {
       237, 4.838052752373304*^-11}, {238, 3.5767522174338105`*^-11}, {
       239, 2.60889446130894*^-11}, {240, 1.877322159982756*^-11}, {
       241, 1.3324770622130949`*^-11}, {242, 9.32596023686214*^-12}, {
       243, 6.433724368330998*^-12}, {244, 4.372615605967666*^-12}, {
       245, 2.925946982945934*^-12}, {246, 1.926430237779039*^-12}, {
       247, 1.2471446420856075`*^-12}, {248, 7.934093691289404*^-13}, {
       249, 4.957783403383221*^-13}, {250, 3.0420069407013193`*^-13}, {
       251, 1.83271421292399*^-13}}],
    Frame->True,
    FrameTicks->{{{1., 
        FormBox[
         RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
        FormBox[
         RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
        FormBox[
         RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
        FormBox[
         RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
        FormBox["0", TraditionalForm]}, {157.25, 
        FormBox["2.5`", TraditionalForm]}, {188.5, 
        FormBox["5", TraditionalForm]}, {219.75, 
        FormBox["7.5`", TraditionalForm]}, {251., 
        FormBox["10", TraditionalForm]}}, Automatic, None, None},
    PlotRange->{-0.1, 1.1},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], Automatic}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF2Hlczfkex/GfLJPMDGaYGuvR2K4lFRKl3qWkfY+0/ep02k6LMbmjMvql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     "], {
      {GrayLevel[0.5], EdgeForm[None], 
       GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ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         "]]]}, {{}, {}, {
        {Hue[0.4168600680892608, 1, 1], LineBox[{1, 2}]}, 
        {Hue[0.3284984963233482, 1, 1], LineBox[{2, 3}]}, 
        {Hue[0.24932412379762658`, 1, 1], LineBox[{3, 4}]}, 
        {Hue[0.13504033356722836`, 1, 1], LineBox[{4, 5}]}, 
        {Hue[0.9913179497102926, 1, 1], LineBox[{5, 6}]}, 
        {Hue[0.9044701728619814, 1, 1], LineBox[{6, 7}]}, 
        {Hue[0.7391832250120645, 1, 1], LineBox[{7, 8}]}, 
        {Hue[0.6580534334747035, 1, 1], LineBox[{8, 9}]}, 
        {Hue[0.5388133016638648, 1, 1], LineBox[{9, 10}]}, 
        {Hue[0.4425053349418048, 1, 1], LineBox[{10, 11}]}, 
        {Hue[0.4131213114772012, 1, 1], LineBox[{11, 12}]}, 
        {Hue[0.2819609923741898, 1, 1], LineBox[{12, 13}]}, 
        {Hue[0.23054292097004633`, 1, 1], LineBox[{13, 14}]}, 
        {Hue[0.07316661394169041, 1, 1], LineBox[{14, 15}]}, 
        {Hue[0.02333481242210441, 1, 1], LineBox[{15, 16}]}, 
        {Hue[0.9691545115150134, 1, 1], LineBox[{16, 17}]}, 
        {Hue[0.8837532272468197, 1, 1], LineBox[{17, 18}]}, 
        {Hue[0.8375921797780413, 1, 1], LineBox[{18, 19}]}, 
        {Hue[0.7888660194019311, 1, 1], LineBox[{19, 20}]}, 
        {Hue[0.6068473004411263, 1, 1], LineBox[{20, 21}]}, 
        {Hue[0.586597492966769, 1, 1], LineBox[{21, 22}]}, 
        {Hue[0.3828866975293449, 1, 1], LineBox[{22, 23}]}, 
        {Hue[0.339288400593652, 1, 1], LineBox[{23, 24}]}, 
        {Hue[0.18584920775669142`, 1, 1], LineBox[{24, 25}]}, 
        {Hue[0.21568596776334784`, 1, 1], LineBox[{25, 26}]}, 
        {Hue[0.9680533520465127, 1, 1], LineBox[{26, 27}]}, 
        {Hue[0.7800790594740148, 1, 1], LineBox[{27, 28}]}, 
        {Hue[0.7392259322801114, 1, 1], LineBox[{28, 29}]}, 
        {Hue[0.5822206436209341, 1, 1], LineBox[{29, 30}]}, 
        {Hue[0.6048288102153387, 1, 1], LineBox[{30, 31}]}, 
        {Hue[0.5106564504880033, 1, 1], LineBox[{31, 32}]}, 
        {Hue[0.45122902163414735`, 1, 1], LineBox[{32, 33}]}, 
        {Hue[0.34778934937168415`, 1, 1], LineBox[{33, 34}]}, 
        {Hue[0.2321365582273194, 1, 1], LineBox[{34, 35}]}, 
        {Hue[0.18910973578281579`, 1, 1], LineBox[{35, 36}]}, 
        {Hue[0.09788173808824142, 1, 1], LineBox[{36, 37}]}, 
        {Hue[0.9791798487066989, 1, 1], LineBox[{37, 38}]}, 
        {Hue[0.900859846867653, 1, 1], LineBox[{38, 39}]}, 
        {Hue[0.6167238393623564, 1, 1], LineBox[{39, 40}]}, 
        {Hue[0.534487194829494, 1, 1], LineBox[{40, 41}]}, 
        {Hue[0.5813004010570966, 1, 1], LineBox[{41, 42}]}, 
        {Hue[0.515034083624454, 1, 1], LineBox[{42, 43}]}, 
        {Hue[0.6103811277352958, 1, 1], LineBox[{43, 44}]}, 
        {Hue[0.7170574961996892, 1, 1], LineBox[{44, 45}]}, 
        {Hue[0.789739331819158, 1, 1], LineBox[{45, 46}]}, 
        {Hue[0.8290180831382562, 1, 1], LineBox[{46, 47}]}, 
        {Hue[0.8874325399101823, 1, 1], LineBox[{47, 48}]}, 
        {Hue[0.951266500016831, 1, 1], LineBox[{48, 49}]}, 
        {Hue[0.025375996397833736`, 1, 1], LineBox[{49, 50}]}, 
        {Hue[0.09518071115517238, 1, 1], LineBox[{50, 51}]}, 
        {Hue[0.15224845113862764`, 1, 1], LineBox[{51, 52}]}, 
        {Hue[0.20884666010258823`, 1, 1], LineBox[{52, 53}]}, 
        {Hue[0.27363270321178623`, 1, 1], LineBox[{53, 54}]}, 
        {Hue[0.33988303732676983`, 1, 1], LineBox[{54, 55}]}, 
        {Hue[0.40458577082292346`, 1, 1], LineBox[{55, 56}]}, 
        {Hue[0.46924365834371023`, 1, 1], LineBox[{56, 57}]}, 
        {Hue[0.5337862177226609, 1, 1], LineBox[{57, 58}]}, 
        {Hue[0.5977805099182919, 1, 1], LineBox[{58, 59}]}, 
        {Hue[0.6644374554337116, 1, 1], LineBox[{59, 60}]}, 
        {Hue[0.7316138102104992, 1, 1], LineBox[{60, 61}]}, 
        {Hue[0.7988801075922869, 1, 1], LineBox[{61, 62}]}, 
        {Hue[0.8676110266452448, 1, 1], LineBox[{62, 63}]}, 
        {Hue[0.9359784086629239, 1, 1], LineBox[{63, 64}]}, 
        {Hue[0.004853955433337728, 1, 1], LineBox[{64, 65}]}, 
        {Hue[0.07487819872850442, 1, 1], LineBox[{65, 66}]}, 
        {Hue[0.1448715690079386, 1, 1], LineBox[{66, 67}]}, 
        {Hue[0.21537600544420657`, 1, 1], LineBox[{67, 68}]}, 
        {Hue[0.28661749291712185`, 1, 1], LineBox[{68, 69}]}, 
        {Hue[0.3582009007979773, 1, 1], LineBox[{69, 70}]}, 
        {Hue[0.4303446834356813, 1, 1], LineBox[{70, 71}]}, 
        {Hue[0.5030331518422426, 1, 1], LineBox[{71, 72}]}, 
        {Hue[0.5761971000286218, 1, 1], LineBox[{72, 73}]}, 
        {Hue[0.6498180906501252, 1, 1], LineBox[{73, 74}]}, 
        {Hue[0.7240252819100452, 1, 1], LineBox[{74, 75}]}, 
        {Hue[0.7987410737277055, 1, 1], LineBox[{75, 76}]}, 
        {Hue[0.8739893909809054, 1, 1], LineBox[{76, 77}]}, 
        {Hue[0.9497212641793275, 1, 1], LineBox[{77, 78}]}, 
        {Hue[0.025953380835229217`, 1, 1], LineBox[{78, 79}]}, 
        {Hue[0.1027263958323548, 1, 1], LineBox[{79, 80}]}, 
        {Hue[0.18001765715205367`, 1, 1], LineBox[{80, 81}]}, 
        {Hue[0.2578107175059787, 1, 1], LineBox[{81, 82}]}, 
        {Hue[0.3361008681236953, 1, 1], LineBox[{82, 83}]}, 
        {Hue[0.41490177682211926`, 1, 1], LineBox[{83, 84}]}, 
        {Hue[0.49421569243712077`, 1, 1], LineBox[{84, 85}]}, 
        {Hue[0.5740441176728341, 1, 1], LineBox[{85, 86}]}, 
        {Hue[0.654369993646139, 1, 1], LineBox[{86, 87}]}, 
        {Hue[0.7351953469422576, 1, 1], LineBox[{87, 88}]}, 
        {Hue[0.81651418750048, 1, 1], LineBox[{88, 89}]}, 
        {Hue[0.8983316598890928, 1, 1], LineBox[{89, 90}]}, 
        {Hue[0.9806458205478827, 1, 1], LineBox[{90, 91}]}, 
        {Hue[0.06344814211594595, 1, 1], LineBox[{91, 92}]}, 
        {Hue[0.14673048541690584`, 1, 1], LineBox[{92, 93}]}, 
        {Hue[0.2304863422174567, 1, 1], LineBox[{93, 94}]}, 
        {Hue[0.31471886861517806`, 1, 1], LineBox[{94, 95}]}, 
        {Hue[0.39941476841128004`, 1, 1], LineBox[{95, 96}]}, 
        {Hue[0.48456659088365756`, 1, 1], LineBox[{96, 97}]}, 
        {Hue[0.5701597559844244, 1, 1], LineBox[{97, 98}]}, 
        {Hue[0.6561853153134112, 1, 1], LineBox[{98, 99}]}, 
        {Hue[0.7426309139751042, 1, 1], LineBox[{99, 100}]}, 
        {Hue[0.8294782523205144, 1, 1], LineBox[{100, 101}]}, 
        {Hue[0.9167098749001431, 1, 1], LineBox[{101, 102}]}, 
        {Hue[0.004301163725584873, 1, 1], LineBox[{102, 103}]}, 
        {Hue[0.09222752982918228, 1, 1], LineBox[{103, 104}]}, 
        {Hue[0.18046028996418825`, 1, 1], LineBox[{104, 105}]}, 
        {Hue[0.26896728276140036`, 1, 1], LineBox[{105, 106}]}, 
        {Hue[0.3577101757639991, 1, 1], LineBox[{106, 107}]}, 
        {Hue[0.44663945537731664`, 1, 1], LineBox[{107, 108}]}, 
        {Hue[0.5357014982016148, 1, 1], LineBox[{108, 109}]}, 
        {Hue[0.62483975362368, 1, 1], LineBox[{109, 110}]}, 
        {Hue[0.7139918935953822, 1, 1], LineBox[{110, 111}]}, 
        {Hue[0.8030846259919716, 1, 1], LineBox[{111, 112}]}, 
        {Hue[0.892026133441005, 1, 1], LineBox[{112, 113}]}, 
        {Hue[0.9807129381766048, 1, 1], LineBox[{113, 114}]}, 
        {Hue[0.06903599557663395, 1, 1], LineBox[{114, 115}]}, 
        {Hue[0.15688514799288122`, 1, 1], LineBox[{115, 116}]}, 
        {Hue[0.24414326416059198`, 1, 1], LineBox[{116, 117}]}, 
        {Hue[0.3306707147725039, 1, 1], LineBox[{117, 118}]}, 
        {Hue[0.41630227337717696`, 1, 1], LineBox[{118, 119}]}, 
        {Hue[0.500860694705684, 1, 1], LineBox[{119, 120}]}, 
        {Hue[0.5841800911294179, 1, 1], LineBox[{120, 121}]}, 
        {Hue[0.6661179424522261, 1, 1], LineBox[{121, 122}]}, 
        {Hue[0.7465420638122583, 1, 1], LineBox[{122, 123}]}, 
        {Hue[0.8253073721807094, 1, 1], LineBox[{123, 124}]}, 
        {Hue[0.9022415102401787, 1, 1], LineBox[{124, 125}]}, 
        {Hue[0.9771627557972062, 1, 1], LineBox[{125, 126}]}, 
        {Hue[0.04992714740017764, 1, 1], LineBox[{126, 127}]}, 
        {Hue[0.12047663023156521`, 1, 1], LineBox[{127, 128}]}, 
        {Hue[0.18886801330590997`, 1, 1], LineBox[{128, 129}]}, 
        {Hue[0.2552696366393901, 1, 1], LineBox[{129, 130}]}, 
        {Hue[0.31993599055739874`, 1, 1], LineBox[{130, 131}]}, 
        {Hue[0.3831737264972136, 1, 1], LineBox[{131, 132}]}, 
        {Hue[0.4453093562026673, 1, 1], LineBox[{132, 133}]}, 
        {Hue[0.5066649663249492, 1, 1], LineBox[{133, 134}]}, 
        {Hue[0.5675424486716532, 1, 1], LineBox[{134, 135}]}, 
        {Hue[0.6282157733354248, 1, 1], LineBox[{135, 136}]}, 
        {Hue[0.6889261744265864, 1, 1], LineBox[{136, 137}]}, 
        {Hue[0.7498836141773992, 1, 1], LineBox[{137, 138}]}, 
        {Hue[0.8112670476816446, 1, 1], LineBox[{138, 139}]}, 
        {Hue[0.8732283191632844, 1, 1], LineBox[{139, 140}]}, 
        {Hue[0.9358926524849951, 1, 1], LineBox[{140, 141}]}, 
        {Hue[0.9993644809767988, 1, 1], LineBox[{141, 142}]}, 
        {Hue[0.06372770144449971, 1, 1], LineBox[{142, 143}]}, 
        {Hue[0.12904974272742295`, 1, 1], LineBox[{143, 144}]}, 
        {Hue[0.19538451094276366`, 1, 1], LineBox[{144, 145}]}, 
        {Hue[0.26277242101711923`, 1, 1], LineBox[{145, 146}]}, 
        {Hue[0.33124284884960614`, 1, 1], LineBox[{146, 147}]}, 
        {Hue[0.40081906563092584`, 1, 1], LineBox[{147, 148}]}, 
        {Hue[0.4715144182185708, 1, 1], LineBox[{148, 149}]}, 
        {Hue[0.5433368070487188, 1, 1], LineBox[{149, 150}]}, 
        {Hue[0.6162905659653262, 1, 1], LineBox[{150, 151}]}, 
        {Hue[0.6903767969105157, 1, 1], LineBox[{151, 152}]}, 
        {Hue[0.7655875006949686, 1, 1], LineBox[{152, 153}]}, 
        {Hue[0.841917362845512, 1, 1], LineBox[{153, 154}]}, 
        {Hue[0.9193636432215327, 1, 1], LineBox[{154, 155}]}, 
        {Hue[0.9979094392787704, 1, 1], LineBox[{155, 156}]}, 
        {Hue[0.07754188310626753, 1, 1], LineBox[{156, 157}]}, 
        {Hue[0.15825696090997052`, 1, 1], LineBox[{157, 158}]}, 
        {Hue[0.2400363503843214, 1, 1], LineBox[{158, 159}]}, 
        {Hue[0.32286155835865576`, 1, 1], LineBox[{159, 160}]}, 
        {Hue[0.4067358030853234, 1, 1], LineBox[{160, 161}]}, 
        {Hue[0.49164280861494974`, 1, 1], LineBox[{161, 162}]}, 
        {Hue[0.5775504068185704, 1, 1], LineBox[{162, 163}]}, 
        {Hue[0.664455646848052, 1, 1], LineBox[{163, 164}]}, 
        {Hue[0.7523608376696441, 1, 1], LineBox[{164, 165}]}, 
        {Hue[0.8412461268252658, 1, 1], LineBox[{165, 166}]}, 
        {Hue[0.9310707885385651, 1, 1], LineBox[{166, 167}]}, 
        {Hue[0.021832696308046587`, 1, 1], LineBox[{167, 168}]}, 
        {Hue[0.11358777873201778`, 1, 1], LineBox[{168, 169}]}, 
        {Hue[0.20629340625631443`, 1, 1], LineBox[{169, 170}]}, 
        {Hue[0.2997868713777356, 1, 1], LineBox[{170, 171}]}, 
        {Hue[0.3940467464102866, 1, 1], LineBox[{171, 172}]}, 
        {Hue[0.48928446488002764`, 1, 1], LineBox[{172, 173}]}, 
        {Hue[0.5856052024320231, 1, 1], LineBox[{173, 174}]}, 
        {Hue[0.6827196800638444, 1, 1], LineBox[{174, 175}]}, 
        {Hue[0.7803999870354645, 1, 1], LineBox[{175, 176}]}, 
        {Hue[0.8790687016307588, 1, 1], LineBox[{176, 177}]}, 
        {Hue[0.9791807877677713, 1, 1], LineBox[{177, 178}]}, 
        {Hue[0.08034201063185673, 1, 1], LineBox[{178, 179}]}, 
        {Hue[0.18179689340613547`, 1, 1], LineBox[{179, 180}]}, 
        {Hue[0.283346050031219, 1, 1], LineBox[{180, 181}]}, 
        {Hue[0.3855860147913132, 1, 1], LineBox[{181, 182}]}, 
        {Hue[0.4891772391858982, 1, 1], LineBox[{182, 183}]}, 
        {Hue[0.5928155783977882, 1, 1], LineBox[{183, 184}]}, 
        {Hue[0.6934891136564652, 1, 1], LineBox[{184, 185}]}, 
        {Hue[0.793080936386119, 1, 1], LineBox[{185, 186}]}, 
        {Hue[0.8998311366440165, 1, 1], LineBox[{186, 187}]}, 
        {Hue[0.015176503380478905`, 1, 1], LineBox[{187, 188}]}, 
        {Hue[0.13044450564833976`, 1, 1], LineBox[{188, 189}]}, 
        {Hue[0.23708488707843128`, 1, 1], LineBox[{189, 190}]}, 
        {Hue[0.33533235811624007`, 1, 1], LineBox[{190, 191}]}, 
        {Hue[0.4339269518627205, 1, 1], LineBox[{191, 192}]}, 
        {Hue[0.551635230054077, 1, 1], LineBox[{192, 193}]}, 
        {Hue[0.7025576980009608, 1, 1], LineBox[{193, 194}]}, 
        {Hue[0.8827000587356161, 1, 1], LineBox[{194, 195}]}, 
        {Hue[0.08418342314818303, 1, 1], LineBox[{195, 196}]}, 
        {Hue[0.7699995303057103, 1, 1], LineBox[{196, 197}]}, 
        {Hue[0.02429688523103022, 1, 1], LineBox[{197, 198}]}, 
        {Hue[0.27179627875331236`, 1, 1], LineBox[{198, 199}]}, 
        {Hue[0.5495531291764919, 1, 1], LineBox[{199, 200}]}, 
        {Hue[0.8756326603745737, 1, 1], LineBox[{200, 201}]}, 
        {Hue[0.1870945145097854, 1, 1], LineBox[{201, 202}]}, 
        {Hue[0.3864954222807409, 1, 1], LineBox[{202, 203}]}, 
        {Hue[0.44858201992730534`, 1, 1], LineBox[{203, 204}]}, 
        {Hue[0.6439880334050112, 1, 1], LineBox[{204, 205}]}, 
        {Hue[0.8858058217557241, 1, 1], LineBox[{205, 206}]}, 
        {Hue[0.14297565563919354`, 1, 1], LineBox[{206, 207}]}, 
        {Hue[0.4141276726633181, 1, 1], LineBox[{207, 208}]}, 
        {Hue[0.7168368232742479, 1, 1], LineBox[{208, 209}]}, 
        {Hue[0.14197056687390944`, 1, 1], LineBox[{209, 210}]}, 
        {Hue[0.5524620912908854, 1, 1], LineBox[{210, 211}]}, 
        {Hue[0.8553893891830866, 1, 1], LineBox[{211, 212}]}, 
        {Hue[0.13438705262114328`, 1, 1], LineBox[{212, 213}]}, 
        {Hue[0.4058233803609347, 1, 1], LineBox[{213, 214}]}, 
        {Hue[0.6738903749915348, 1, 1], LineBox[{214, 215}]}, 
        {Hue[0.9401789901094688, 1, 1], LineBox[{215, 216}]}, 
        {Hue[0.20547177438491626`, 1, 1], LineBox[{216, 217}]}, 
        {Hue[0.47020848142795374`, 1, 1], LineBox[{217, 218}]}, 
        {Hue[0.734657523019001, 1, 1], LineBox[{218, 219}]}, 
        {Hue[0.9990322339559546, 1, 1], LineBox[{219, 220}]}, 
        {Hue[0.2636330251999652, 1, 1], LineBox[{220, 221}]}, 
        {Hue[0.5290628537980339, 1, 1], LineBox[{221, 222}]}, 
        {Hue[0.7965350950639479, 1, 1], LineBox[{222, 223}]}, 
        {Hue[0.06817073638838694, 1, 1], LineBox[{223, 224}]}, 
        {Hue[0.3467897323971712, 1, 1], LineBox[{224, 225}]}, 
        {Hue[0.6341252035470084, 1, 1], LineBox[{225, 226}]}, 
        {Hue[0.9275574020329773, 1, 1], LineBox[{226, 227}]}, 
        {Hue[0.22028416339828585`, 1, 1], LineBox[{227, 228}]}, 
        {Hue[0.5071579118616438, 1, 1], LineBox[{228, 229}]}, 
        {Hue[0.7874886413105265, 1, 1], LineBox[{229, 230}]}, 
        {Hue[0.06275314718690028, 1, 1], LineBox[{230, 231}]}, 
        {Hue[0.3345026343916775, 1, 1], LineBox[{231, 232}]}, 
        {Hue[0.6037857107221525, 1, 1], LineBox[{232, 233}]}, 
        {Hue[0.8712023572484743, 1, 1], LineBox[{233, 234}]}, 
        {Hue[0.13705768198816723`, 1, 1], LineBox[{234, 235}]}, 
        {Hue[0.401476064729792, 1, 1], LineBox[{235, 236}]}, 
        {Hue[0.6644687447989364, 1, 1], LineBox[{236, 237}]}, 
        {Hue[0.9259678258280817, 1, 1], LineBox[{237, 238}]}, 
        {Hue[0.18583835754938358`, 1, 1], LineBox[{238, 239}]}, 
        {Hue[0.44387781820781336`, 1, 1], LineBox[{239, 240}]}, 
        {Hue[0.6998052797815966, 1, 1], LineBox[{240, 241}]}, 
        {Hue[0.9532478642107147, 1, 1], LineBox[{241, 242}]}, 
        {Hue[0.20373127791721357`, 1, 1], LineBox[{242, 243}]}, 
        {Hue[0.45069692166927705`, 1, 1], LineBox[{243, 244}]}, 
        {Hue[0.693588719734344, 1, 1], LineBox[{244, 245}]}, 
        {Hue[0.9320772375617843, 1, 1], LineBox[{245, 246}]}, 
        {Hue[0.16645123969962386`, 1, 1], LineBox[{246, 247}]}, 
        {Hue[0.398004171513236, 1, 1], LineBox[{247, 248}]}, 
        {Hue[0.6289620661548705, 1, 1], LineBox[{248, 249}]}, 
        {Hue[0.8617056861385596, 1, 1], LineBox[{249, 250}]}, 
        {Hue[0.09785397911062299, 1, 1], LineBox[{250, 251}]}}}},
     VertexColors->{
       Hue[0.4168600680892608, 1, 1], 
       Hue[0.3284984963233482, 1, 1], 
       Hue[0.24932412379762658`, 1, 1], 
       Hue[0.13504033356722836`, 1, 1], 
       Hue[0.9913179497102926, 1, 1], 
       Hue[0.9044701728619814, 1, 1], 
       Hue[0.7391832250120645, 1, 1], 
       Hue[0.6580534334747035, 1, 1], 
       Hue[0.5388133016638648, 1, 1], 
       Hue[0.4425053349418048, 1, 1], 
       Hue[0.4131213114772012, 1, 1], 
       Hue[0.2819609923741898, 1, 1], 
       Hue[0.23054292097004633`, 1, 1], 
       Hue[0.07316661394169041, 1, 1], 
       Hue[0.02333481242210441, 1, 1], 
       Hue[0.9691545115150134, 1, 1], 
       Hue[0.8837532272468197, 1, 1], 
       Hue[0.8375921797780413, 1, 1], 
       Hue[0.7888660194019311, 1, 1], 
       Hue[0.6068473004411263, 1, 1], 
       Hue[0.586597492966769, 1, 1], 
       Hue[0.3828866975293449, 1, 1], 
       Hue[0.339288400593652, 1, 1], 
       Hue[0.18584920775669142`, 1, 1], 
       Hue[0.21568596776334784`, 1, 1], 
       Hue[0.9680533520465127, 1, 1], 
       Hue[0.7800790594740148, 1, 1], 
       Hue[0.7392259322801114, 1, 1], 
       Hue[0.5822206436209341, 1, 1], 
       Hue[0.6048288102153387, 1, 1], 
       Hue[0.5106564504880033, 1, 1], 
       Hue[0.45122902163414735`, 1, 1], 
       Hue[0.34778934937168415`, 1, 1], 
       Hue[0.2321365582273194, 1, 1], 
       Hue[0.18910973578281579`, 1, 1], 
       Hue[0.09788173808824142, 1, 1], 
       Hue[0.9791798487066989, 1, 1], 
       Hue[0.900859846867653, 1, 1], 
       Hue[0.6167238393623564, 1, 1], 
       Hue[0.534487194829494, 1, 1], 
       Hue[0.5813004010570966, 1, 1], 
       Hue[0.515034083624454, 1, 1], 
       Hue[0.6103811277352958, 1, 1], 
       Hue[0.7170574961996892, 1, 1], 
       Hue[0.789739331819158, 1, 1], 
       Hue[0.8290180831382562, 1, 1], 
       Hue[0.8874325399101823, 1, 1], 
       Hue[0.951266500016831, 1, 1], 
       Hue[0.025375996397833736`, 1, 1], 
       Hue[0.09518071115517238, 1, 1], 
       Hue[0.15224845113862764`, 1, 1], 
       Hue[0.20884666010258823`, 1, 1], 
       Hue[0.27363270321178623`, 1, 1], 
       Hue[0.33988303732676983`, 1, 1], 
       Hue[0.40458577082292346`, 1, 1], 
       Hue[0.46924365834371023`, 1, 1], 
       Hue[0.5337862177226609, 1, 1], 
       Hue[0.5977805099182919, 1, 1], 
       Hue[0.6644374554337116, 1, 1], 
       Hue[0.7316138102104992, 1, 1], 
       Hue[0.7988801075922869, 1, 1], 
       Hue[0.8676110266452448, 1, 1], 
       Hue[0.9359784086629239, 1, 1], 
       Hue[0.004853955433337728, 1, 1], 
       Hue[0.07487819872850442, 1, 1], 
       Hue[0.1448715690079386, 1, 1], 
       Hue[0.21537600544420657`, 1, 1], 
       Hue[0.28661749291712185`, 1, 1], 
       Hue[0.3582009007979773, 1, 1], 
       Hue[0.4303446834356813, 1, 1], 
       Hue[0.5030331518422426, 1, 1], 
       Hue[0.5761971000286218, 1, 1], 
       Hue[0.6498180906501252, 1, 1], 
       Hue[0.7240252819100452, 1, 1], 
       Hue[0.7987410737277055, 1, 1], 
       Hue[0.8739893909809054, 1, 1], 
       Hue[0.9497212641793275, 1, 1], 
       Hue[0.025953380835229217`, 1, 1], 
       Hue[0.1027263958323548, 1, 1], 
       Hue[0.18001765715205367`, 1, 1], 
       Hue[0.2578107175059787, 1, 1], 
       Hue[0.3361008681236953, 1, 1], 
       Hue[0.41490177682211926`, 1, 1], 
       Hue[0.49421569243712077`, 1, 1], 
       Hue[0.5740441176728341, 1, 1], 
       Hue[0.654369993646139, 1, 1], 
       Hue[0.7351953469422576, 1, 1], 
       Hue[0.81651418750048, 1, 1], 
       Hue[0.8983316598890928, 1, 1], 
       Hue[0.9806458205478827, 1, 1], 
       Hue[0.06344814211594595, 1, 1], 
       Hue[0.14673048541690584`, 1, 1], 
       Hue[0.2304863422174567, 1, 1], 
       Hue[0.31471886861517806`, 1, 1], 
       Hue[0.39941476841128004`, 1, 1], 
       Hue[0.48456659088365756`, 1, 1], 
       Hue[0.5701597559844244, 1, 1], 
       Hue[0.6561853153134112, 1, 1], 
       Hue[0.7426309139751042, 1, 1], 
       Hue[0.8294782523205144, 1, 1], 
       Hue[0.9167098749001431, 1, 1], 
       Hue[0.004301163725584873, 1, 1], 
       Hue[0.09222752982918228, 1, 1], 
       Hue[0.18046028996418825`, 1, 1], 
       Hue[0.26896728276140036`, 1, 1], 
       Hue[0.3577101757639991, 1, 1], 
       Hue[0.44663945537731664`, 1, 1], 
       Hue[0.5357014982016148, 1, 1], 
       Hue[0.62483975362368, 1, 1], 
       Hue[0.7139918935953822, 1, 1], 
       Hue[0.8030846259919716, 1, 1], 
       Hue[0.892026133441005, 1, 1], 
       Hue[0.9807129381766048, 1, 1], 
       Hue[0.06903599557663395, 1, 1], 
       Hue[0.15688514799288122`, 1, 1], 
       Hue[0.24414326416059198`, 1, 1], 
       Hue[0.3306707147725039, 1, 1], 
       Hue[0.41630227337717696`, 1, 1], 
       Hue[0.500860694705684, 1, 1], 
       Hue[0.5841800911294179, 1, 1], 
       Hue[0.6661179424522261, 1, 1], 
       Hue[0.7465420638122583, 1, 1], 
       Hue[0.8253073721807094, 1, 1], 
       Hue[0.9022415102401787, 1, 1], 
       Hue[0.9771627557972062, 1, 1], 
       Hue[0.04992714740017764, 1, 1], 
       Hue[0.12047663023156521`, 1, 1], 
       Hue[0.18886801330590997`, 1, 1], 
       Hue[0.2552696366393901, 1, 1], 
       Hue[0.31993599055739874`, 1, 1], 
       Hue[0.3831737264972136, 1, 1], 
       Hue[0.4453093562026673, 1, 1], 
       Hue[0.5066649663249492, 1, 1], 
       Hue[0.5675424486716532, 1, 1], 
       Hue[0.6282157733354248, 1, 1], 
       Hue[0.6889261744265864, 1, 1], 
       Hue[0.7498836141773992, 1, 1], 
       Hue[0.8112670476816446, 1, 1], 
       Hue[0.8732283191632844, 1, 1], 
       Hue[0.9358926524849951, 1, 1], 
       Hue[0.9993644809767988, 1, 1], 
       Hue[0.06372770144449971, 1, 1], 
       Hue[0.12904974272742295`, 1, 1], 
       Hue[0.19538451094276366`, 1, 1], 
       Hue[0.26277242101711923`, 1, 1], 
       Hue[0.33124284884960614`, 1, 1], 
       Hue[0.40081906563092584`, 1, 1], 
       Hue[0.4715144182185708, 1, 1], 
       Hue[0.5433368070487188, 1, 1], 
       Hue[0.6162905659653262, 1, 1], 
       Hue[0.6903767969105157, 1, 1], 
       Hue[0.7655875006949686, 1, 1], 
       Hue[0.841917362845512, 1, 1], 
       Hue[0.9193636432215327, 1, 1], 
       Hue[0.9979094392787704, 1, 1], 
       Hue[0.07754188310626753, 1, 1], 
       Hue[0.15825696090997052`, 1, 1], 
       Hue[0.2400363503843214, 1, 1], 
       Hue[0.32286155835865576`, 1, 1], 
       Hue[0.4067358030853234, 1, 1], 
       Hue[0.49164280861494974`, 1, 1], 
       Hue[0.5775504068185704, 1, 1], 
       Hue[0.664455646848052, 1, 1], 
       Hue[0.7523608376696441, 1, 1], 
       Hue[0.8412461268252658, 1, 1], 
       Hue[0.9310707885385651, 1, 1], 
       Hue[0.021832696308046587`, 1, 1], 
       Hue[0.11358777873201778`, 1, 1], 
       Hue[0.20629340625631443`, 1, 1], 
       Hue[0.2997868713777356, 1, 1], 
       Hue[0.3940467464102866, 1, 1], 
       Hue[0.48928446488002764`, 1, 1], 
       Hue[0.5856052024320231, 1, 1], 
       Hue[0.6827196800638444, 1, 1], 
       Hue[0.7803999870354645, 1, 1], 
       Hue[0.8790687016307588, 1, 1], 
       Hue[0.9791807877677713, 1, 1], 
       Hue[0.08034201063185673, 1, 1], 
       Hue[0.18179689340613547`, 1, 1], 
       Hue[0.283346050031219, 1, 1], 
       Hue[0.3855860147913132, 1, 1], 
       Hue[0.4891772391858982, 1, 1], 
       Hue[0.5928155783977882, 1, 1], 
       Hue[0.6934891136564652, 1, 1], 
       Hue[0.793080936386119, 1, 1], 
       Hue[0.8998311366440165, 1, 1], 
       Hue[0.015176503380478905`, 1, 1], 
       Hue[0.13044450564833976`, 1, 1], 
       Hue[0.23708488707843128`, 1, 1], 
       Hue[0.33533235811624007`, 1, 1], 
       Hue[0.4339269518627205, 1, 1], 
       Hue[0.551635230054077, 1, 1], 
       Hue[0.7025576980009608, 1, 1], 
       Hue[0.8827000587356161, 1, 1], 
       Hue[0.08418342314818303, 1, 1], 
       Hue[0.7699995303057103, 1, 1], 
       Hue[0.02429688523103022, 1, 1], 
       Hue[0.27179627875331236`, 1, 1], 
       Hue[0.5495531291764919, 1, 1], 
       Hue[0.8756326603745737, 1, 1], 
       Hue[0.1870945145097854, 1, 1], 
       Hue[0.3864954222807409, 1, 1], 
       Hue[0.44858201992730534`, 1, 1], 
       Hue[0.6439880334050112, 1, 1], 
       Hue[0.8858058217557241, 1, 1], 
       Hue[0.14297565563919354`, 1, 1], 
       Hue[0.4141276726633181, 1, 1], 
       Hue[0.7168368232742479, 1, 1], 
       Hue[0.14197056687390944`, 1, 1], 
       Hue[0.5524620912908854, 1, 1], 
       Hue[0.8553893891830866, 1, 1], 
       Hue[0.13438705262114328`, 1, 1], 
       Hue[0.4058233803609347, 1, 1], 
       Hue[0.6738903749915348, 1, 1], 
       Hue[0.9401789901094688, 1, 1], 
       Hue[0.20547177438491626`, 1, 1], 
       Hue[0.47020848142795374`, 1, 1], 
       Hue[0.734657523019001, 1, 1], 
       Hue[0.9990322339559546, 1, 1], 
       Hue[0.2636330251999652, 1, 1], 
       Hue[0.5290628537980339, 1, 1], 
       Hue[0.7965350950639479, 1, 1], 
       Hue[0.06817073638838694, 1, 1], 
       Hue[0.3467897323971712, 1, 1], 
       Hue[0.6341252035470084, 1, 1], 
       Hue[0.9275574020329773, 1, 1], 
       Hue[0.22028416339828585`, 1, 1], 
       Hue[0.5071579118616438, 1, 1], 
       Hue[0.7874886413105265, 1, 1], 
       Hue[0.06275314718690028, 1, 1], 
       Hue[0.3345026343916775, 1, 1], 
       Hue[0.6037857107221525, 1, 1], 
       Hue[0.8712023572484743, 1, 1], 
       Hue[0.13705768198816723`, 1, 1], 
       Hue[0.401476064729792, 1, 1], 
       Hue[0.6644687447989364, 1, 1], 
       Hue[0.9259678258280817, 1, 1], 
       Hue[0.18583835754938358`, 1, 1], 
       Hue[0.44387781820781336`, 1, 1], 
       Hue[0.6998052797815966, 1, 1], 
       Hue[0.9532478642107147, 1, 1], 
       Hue[0.20373127791721357`, 1, 1], 
       Hue[0.45069692166927705`, 1, 1], 
       Hue[0.693588719734344, 1, 1], 
       Hue[0.9320772375617843, 1, 1], 
       Hue[0.16645123969962386`, 1, 1], 
       Hue[0.398004171513236, 1, 1], 
       Hue[0.6289620661548705, 1, 1], 
       Hue[0.8617056861385596, 1, 1], 
       Hue[0.09785397911062299, 1, 1], 
       Hue[0.33796398999826804`, 1, 1], 
       Hue[0.4168600680892608, 1, 1], 
       Hue[0.3284984963233482, 1, 1], 
       Hue[0.3284984963233482, 1, 1], 
       Hue[0.24932412379762658`, 1, 1], 
       Hue[0.24932412379762658`, 1, 1], 
       Hue[0.13504033356722836`, 1, 1], 
       Hue[0.13504033356722836`, 1, 1], 
       Hue[0.9913179497102926, 1, 1], 
       Hue[0.9913179497102926, 1, 1], 
       Hue[0.9044701728619814, 1, 1], 
       Hue[0.9044701728619814, 1, 1], 
       Hue[0.7391832250120645, 1, 1], 
       Hue[0.7391832250120645, 1, 1], 
       Hue[0.6580534334747035, 1, 1], 
       Hue[0.6580534334747035, 1, 1], 
       Hue[0.5388133016638648, 1, 1], 
       Hue[0.5388133016638648, 1, 1], 
       Hue[0.4425053349418048, 1, 1], 
       Hue[0.4425053349418048, 1, 1], 
       Hue[0.4131213114772012, 1, 1], 
       Hue[0.4131213114772012, 1, 1], 
       Hue[0.2819609923741898, 1, 1], 
       Hue[0.2819609923741898, 1, 1], 
       Hue[0.23054292097004633`, 1, 1], 
       Hue[0.23054292097004633`, 1, 1], 
       Hue[0.07316661394169041, 1, 1], 
       Hue[0.07316661394169041, 1, 1], 
       Hue[0.02333481242210441, 1, 1], 
       Hue[0.02333481242210441, 1, 1], 
       Hue[0.9691545115150134, 1, 1], 
       Hue[0.9691545115150134, 1, 1], 
       Hue[0.8837532272468197, 1, 1], 
       Hue[0.8837532272468197, 1, 1], 
       Hue[0.8375921797780413, 1, 1], 
       Hue[0.8375921797780413, 1, 1], 
       Hue[0.7888660194019311, 1, 1], 
       Hue[0.7888660194019311, 1, 1], 
       Hue[0.6068473004411263, 1, 1], 
       Hue[0.6068473004411263, 1, 1], 
       Hue[0.586597492966769, 1, 1], 
       Hue[0.586597492966769, 1, 1], 
       Hue[0.3828866975293449, 1, 1], 
       Hue[0.3828866975293449, 1, 1], 
       Hue[0.339288400593652, 1, 1], 
       Hue[0.339288400593652, 1, 1], 
       Hue[0.18584920775669142`, 1, 1], 
       Hue[0.18584920775669142`, 1, 1], 
       Hue[0.21568596776334784`, 1, 1], 
       Hue[0.21568596776334784`, 1, 1], 
       Hue[0.9680533520465127, 1, 1], 
       Hue[0.9680533520465127, 1, 1], 
       Hue[0.7800790594740148, 1, 1], 
       Hue[0.7800790594740148, 1, 1], 
       Hue[0.7392259322801114, 1, 1], 
       Hue[0.7392259322801114, 1, 1], 
       Hue[0.5822206436209341, 1, 1], 
       Hue[0.5822206436209341, 1, 1], 
       Hue[0.6048288102153387, 1, 1], 
       Hue[0.6048288102153387, 1, 1], 
       Hue[0.5106564504880033, 1, 1], 
       Hue[0.5106564504880033, 1, 1], 
       Hue[0.45122902163414735`, 1, 1], 
       Hue[0.45122902163414735`, 1, 1], 
       Hue[0.34778934937168415`, 1, 1], 
       Hue[0.34778934937168415`, 1, 1], 
       Hue[0.2321365582273194, 1, 1], 
       Hue[0.2321365582273194, 1, 1], 
       Hue[0.18910973578281579`, 1, 1], 
       Hue[0.18910973578281579`, 1, 1], 
       Hue[0.09788173808824142, 1, 1], 
       Hue[0.09788173808824142, 1, 1], 
       Hue[0.9791798487066989, 1, 1], 
       Hue[0.9791798487066989, 1, 1], 
       Hue[0.900859846867653, 1, 1], 
       Hue[0.900859846867653, 1, 1], 
       Hue[0.6167238393623564, 1, 1], 
       Hue[0.6167238393623564, 1, 1], 
       Hue[0.534487194829494, 1, 1], 
       Hue[0.534487194829494, 1, 1], 
       Hue[0.5813004010570966, 1, 1], 
       Hue[0.5813004010570966, 1, 1], 
       Hue[0.515034083624454, 1, 1], 
       Hue[0.515034083624454, 1, 1], 
       Hue[0.6103811277352958, 1, 1], 
       Hue[0.6103811277352958, 1, 1], 
       Hue[0.7170574961996892, 1, 1], 
       Hue[0.7170574961996892, 1, 1], 
       Hue[0.789739331819158, 1, 1], 
       Hue[0.789739331819158, 1, 1], 
       Hue[0.8290180831382562, 1, 1], 
       Hue[0.8290180831382562, 1, 1], 
       Hue[0.8874325399101823, 1, 1], 
       Hue[0.8874325399101823, 1, 1], 
       Hue[0.951266500016831, 1, 1], 
       Hue[0.951266500016831, 1, 1], 
       Hue[0.025375996397833736`, 1, 1], 
       Hue[0.025375996397833736`, 1, 1], 
       Hue[0.09518071115517238, 1, 1], 
       Hue[0.09518071115517238, 1, 1], 
       Hue[0.15224845113862764`, 1, 1], 
       Hue[0.15224845113862764`, 1, 1], 
       Hue[0.20884666010258823`, 1, 1], 
       Hue[0.20884666010258823`, 1, 1], 
       Hue[0.27363270321178623`, 1, 1], 
       Hue[0.27363270321178623`, 1, 1], 
       Hue[0.33988303732676983`, 1, 1], 
       Hue[0.33988303732676983`, 1, 1], 
       Hue[0.40458577082292346`, 1, 1], 
       Hue[0.40458577082292346`, 1, 1], 
       Hue[0.46924365834371023`, 1, 1], 
       Hue[0.46924365834371023`, 1, 1], 
       Hue[0.5337862177226609, 1, 1], 
       Hue[0.5337862177226609, 1, 1], 
       Hue[0.5977805099182919, 1, 1], 
       Hue[0.5977805099182919, 1, 1], 
       Hue[0.6644374554337116, 1, 1], 
       Hue[0.6644374554337116, 1, 1], 
       Hue[0.7316138102104992, 1, 1], 
       Hue[0.7316138102104992, 1, 1], 
       Hue[0.7988801075922869, 1, 1], 
       Hue[0.7988801075922869, 1, 1], 
       Hue[0.8676110266452448, 1, 1], 
       Hue[0.8676110266452448, 1, 1], 
       Hue[0.9359784086629239, 1, 1], 
       Hue[0.9359784086629239, 1, 1], 
       Hue[0.004853955433337728, 1, 1], 
       Hue[0.004853955433337728, 1, 1], 
       Hue[0.07487819872850442, 1, 1], 
       Hue[0.07487819872850442, 1, 1], 
       Hue[0.1448715690079386, 1, 1], 
       Hue[0.1448715690079386, 1, 1], 
       Hue[0.21537600544420657`, 1, 1], 
       Hue[0.21537600544420657`, 1, 1], 
       Hue[0.28661749291712185`, 1, 1], 
       Hue[0.28661749291712185`, 1, 1], 
       Hue[0.3582009007979773, 1, 1], 
       Hue[0.3582009007979773, 1, 1], 
       Hue[0.4303446834356813, 1, 1], 
       Hue[0.4303446834356813, 1, 1], 
       Hue[0.5030331518422426, 1, 1], 
       Hue[0.5030331518422426, 1, 1], 
       Hue[0.5761971000286218, 1, 1], 
       Hue[0.5761971000286218, 1, 1], 
       Hue[0.6498180906501252, 1, 1], 
       Hue[0.6498180906501252, 1, 1], 
       Hue[0.7240252819100452, 1, 1], 
       Hue[0.7240252819100452, 1, 1], 
       Hue[0.7987410737277055, 1, 1], 
       Hue[0.7987410737277055, 1, 1], 
       Hue[0.8739893909809054, 1, 1], 
       Hue[0.8739893909809054, 1, 1], 
       Hue[0.9497212641793275, 1, 1], 
       Hue[0.9497212641793275, 1, 1], 
       Hue[0.025953380835229217`, 1, 1], 
       Hue[0.025953380835229217`, 1, 1], 
       Hue[0.1027263958323548, 1, 1], 
       Hue[0.1027263958323548, 1, 1], 
       Hue[0.18001765715205367`, 1, 1], 
       Hue[0.18001765715205367`, 1, 1], 
       Hue[0.2578107175059787, 1, 1], 
       Hue[0.2578107175059787, 1, 1], 
       Hue[0.3361008681236953, 1, 1], 
       Hue[0.3361008681236953, 1, 1], 
       Hue[0.41490177682211926`, 1, 1], 
       Hue[0.41490177682211926`, 1, 1], 
       Hue[0.49421569243712077`, 1, 1], 
       Hue[0.49421569243712077`, 1, 1], 
       Hue[0.5740441176728341, 1, 1], 
       Hue[0.5740441176728341, 1, 1], 
       Hue[0.654369993646139, 1, 1], 
       Hue[0.654369993646139, 1, 1], 
       Hue[0.7351953469422576, 1, 1], 
       Hue[0.7351953469422576, 1, 1], 
       Hue[0.81651418750048, 1, 1], 
       Hue[0.81651418750048, 1, 1], 
       Hue[0.8983316598890928, 1, 1], 
       Hue[0.8983316598890928, 1, 1], 
       Hue[0.9806458205478827, 1, 1], 
       Hue[0.9806458205478827, 1, 1], 
       Hue[0.06344814211594595, 1, 1], 
       Hue[0.06344814211594595, 1, 1], 
       Hue[0.14673048541690584`, 1, 1], 
       Hue[0.14673048541690584`, 1, 1], 
       Hue[0.2304863422174567, 1, 1], 
       Hue[0.2304863422174567, 1, 1], 
       Hue[0.31471886861517806`, 1, 1], 
       Hue[0.31471886861517806`, 1, 1], 
       Hue[0.39941476841128004`, 1, 1], 
       Hue[0.39941476841128004`, 1, 1], 
       Hue[0.48456659088365756`, 1, 1], 
       Hue[0.48456659088365756`, 1, 1], 
       Hue[0.5701597559844244, 1, 1], 
       Hue[0.5701597559844244, 1, 1], 
       Hue[0.6561853153134112, 1, 1], 
       Hue[0.6561853153134112, 1, 1], 
       Hue[0.7426309139751042, 1, 1], 
       Hue[0.7426309139751042, 1, 1], 
       Hue[0.8294782523205144, 1, 1], 
       Hue[0.8294782523205144, 1, 1], 
       Hue[0.9167098749001431, 1, 1], 
       Hue[0.9167098749001431, 1, 1], 
       Hue[0.004301163725584873, 1, 1], 
       Hue[0.004301163725584873, 1, 1], 
       Hue[0.09222752982918228, 1, 1], 
       Hue[0.09222752982918228, 1, 1], 
       Hue[0.18046028996418825`, 1, 1], 
       Hue[0.18046028996418825`, 1, 1], 
       Hue[0.26896728276140036`, 1, 1], 
       Hue[0.26896728276140036`, 1, 1], 
       Hue[0.3577101757639991, 1, 1], 
       Hue[0.3577101757639991, 1, 1], 
       Hue[0.44663945537731664`, 1, 1], 
       Hue[0.44663945537731664`, 1, 1], 
       Hue[0.5357014982016148, 1, 1], 
       Hue[0.5357014982016148, 1, 1], 
       Hue[0.62483975362368, 1, 1], 
       Hue[0.62483975362368, 1, 1], 
       Hue[0.7139918935953822, 1, 1], 
       Hue[0.7139918935953822, 1, 1], 
       Hue[0.8030846259919716, 1, 1], 
       Hue[0.8030846259919716, 1, 1], 
       Hue[0.892026133441005, 1, 1], 
       Hue[0.892026133441005, 1, 1], 
       Hue[0.9807129381766048, 1, 1], 
       Hue[0.9807129381766048, 1, 1], 
       Hue[0.06903599557663395, 1, 1], 
       Hue[0.06903599557663395, 1, 1], 
       Hue[0.15688514799288122`, 1, 1], 
       Hue[0.15688514799288122`, 1, 1], 
       Hue[0.24414326416059198`, 1, 1], 
       Hue[0.24414326416059198`, 1, 1], 
       Hue[0.3306707147725039, 1, 1], 
       Hue[0.3306707147725039, 1, 1], 
       Hue[0.41630227337717696`, 1, 1], 
       Hue[0.41630227337717696`, 1, 1], 
       Hue[0.500860694705684, 1, 1], 
       Hue[0.500860694705684, 1, 1], 
       Hue[0.5841800911294179, 1, 1], 
       Hue[0.5841800911294179, 1, 1], 
       Hue[0.6661179424522261, 1, 1], 
       Hue[0.6661179424522261, 1, 1], 
       Hue[0.7465420638122583, 1, 1], 
       Hue[0.7465420638122583, 1, 1], 
       Hue[0.8253073721807094, 1, 1], 
       Hue[0.8253073721807094, 1, 1], 
       Hue[0.9022415102401787, 1, 1], 
       Hue[0.9022415102401787, 1, 1], 
       Hue[0.9771627557972062, 1, 1], 
       Hue[0.9771627557972062, 1, 1], 
       Hue[0.04992714740017764, 1, 1], 
       Hue[0.04992714740017764, 1, 1], 
       Hue[0.12047663023156521`, 1, 1], 
       Hue[0.12047663023156521`, 1, 1], 
       Hue[0.18886801330590997`, 1, 1], 
       Hue[0.18886801330590997`, 1, 1], 
       Hue[0.2552696366393901, 1, 1], 
       Hue[0.2552696366393901, 1, 1], 
       Hue[0.31993599055739874`, 1, 1], 
       Hue[0.31993599055739874`, 1, 1], 
       Hue[0.3831737264972136, 1, 1], 
       Hue[0.3831737264972136, 1, 1], 
       Hue[0.4453093562026673, 1, 1], 
       Hue[0.4453093562026673, 1, 1], 
       Hue[0.5066649663249492, 1, 1], 
       Hue[0.5066649663249492, 1, 1], 
       Hue[0.5675424486716532, 1, 1], 
       Hue[0.5675424486716532, 1, 1], 
       Hue[0.6282157733354248, 1, 1], 
       Hue[0.6282157733354248, 1, 1], 
       Hue[0.6889261744265864, 1, 1], 
       Hue[0.6889261744265864, 1, 1], 
       Hue[0.7498836141773992, 1, 1], 
       Hue[0.7498836141773992, 1, 1], 
       Hue[0.8112670476816446, 1, 1], 
       Hue[0.8112670476816446, 1, 1], 
       Hue[0.8732283191632844, 1, 1], 
       Hue[0.8732283191632844, 1, 1], 
       Hue[0.9358926524849951, 1, 1], 
       Hue[0.9358926524849951, 1, 1], 
       Hue[0.9993644809767988, 1, 1], 
       Hue[0.9993644809767988, 1, 1], 
       Hue[0.06372770144449971, 1, 1], 
       Hue[0.06372770144449971, 1, 1], 
       Hue[0.12904974272742295`, 1, 1], 
       Hue[0.12904974272742295`, 1, 1], 
       Hue[0.19538451094276366`, 1, 1], 
       Hue[0.19538451094276366`, 1, 1], 
       Hue[0.26277242101711923`, 1, 1], 
       Hue[0.26277242101711923`, 1, 1], 
       Hue[0.33124284884960614`, 1, 1], 
       Hue[0.33124284884960614`, 1, 1], 
       Hue[0.40081906563092584`, 1, 1], 
       Hue[0.40081906563092584`, 1, 1], 
       Hue[0.4715144182185708, 1, 1], 
       Hue[0.4715144182185708, 1, 1], 
       Hue[0.5433368070487188, 1, 1], 
       Hue[0.5433368070487188, 1, 1], 
       Hue[0.6162905659653262, 1, 1], 
       Hue[0.6162905659653262, 1, 1], 
       Hue[0.6903767969105157, 1, 1], 
       Hue[0.6903767969105157, 1, 1], 
       Hue[0.7655875006949686, 1, 1], 
       Hue[0.7655875006949686, 1, 1], 
       Hue[0.841917362845512, 1, 1], 
       Hue[0.841917362845512, 1, 1], 
       Hue[0.9193636432215327, 1, 1], 
       Hue[0.9193636432215327, 1, 1], 
       Hue[0.9979094392787704, 1, 1], 
       Hue[0.9979094392787704, 1, 1], 
       Hue[0.07754188310626753, 1, 1], 
       Hue[0.07754188310626753, 1, 1], 
       Hue[0.15825696090997052`, 1, 1], 
       Hue[0.15825696090997052`, 1, 1], 
       Hue[0.2400363503843214, 1, 1], 
       Hue[0.2400363503843214, 1, 1], 
       Hue[0.32286155835865576`, 1, 1], 
       Hue[0.32286155835865576`, 1, 1], 
       Hue[0.4067358030853234, 1, 1], 
       Hue[0.4067358030853234, 1, 1], 
       Hue[0.49164280861494974`, 1, 1], 
       Hue[0.49164280861494974`, 1, 1], 
       Hue[0.5775504068185704, 1, 1], 
       Hue[0.5775504068185704, 1, 1], 
       Hue[0.664455646848052, 1, 1], 
       Hue[0.664455646848052, 1, 1], 
       Hue[0.7523608376696441, 1, 1], 
       Hue[0.7523608376696441, 1, 1], 
       Hue[0.8412461268252658, 1, 1], 
       Hue[0.8412461268252658, 1, 1], 
       Hue[0.9310707885385651, 1, 1], 
       Hue[0.9310707885385651, 1, 1], 
       Hue[0.021832696308046587`, 1, 1], 
       Hue[0.021832696308046587`, 1, 1], 
       Hue[0.11358777873201778`, 1, 1], 
       Hue[0.11358777873201778`, 1, 1], 
       Hue[0.20629340625631443`, 1, 1], 
       Hue[0.20629340625631443`, 1, 1], 
       Hue[0.2997868713777356, 1, 1], 
       Hue[0.2997868713777356, 1, 1], 
       Hue[0.3940467464102866, 1, 1], 
       Hue[0.3940467464102866, 1, 1], 
       Hue[0.48928446488002764`, 1, 1], 
       Hue[0.48928446488002764`, 1, 1], 
       Hue[0.5856052024320231, 1, 1], 
       Hue[0.5856052024320231, 1, 1], 
       Hue[0.6827196800638444, 1, 1], 
       Hue[0.6827196800638444, 1, 1], 
       Hue[0.7803999870354645, 1, 1], 
       Hue[0.7803999870354645, 1, 1], 
       Hue[0.8790687016307588, 1, 1], 
       Hue[0.8790687016307588, 1, 1], 
       Hue[0.9791807877677713, 1, 1], 
       Hue[0.9791807877677713, 1, 1], 
       Hue[0.08034201063185673, 1, 1], 
       Hue[0.08034201063185673, 1, 1], 
       Hue[0.18179689340613547`, 1, 1], 
       Hue[0.18179689340613547`, 1, 1], 
       Hue[0.283346050031219, 1, 1], 
       Hue[0.283346050031219, 1, 1], 
       Hue[0.3855860147913132, 1, 1], 
       Hue[0.3855860147913132, 1, 1], 
       Hue[0.4891772391858982, 1, 1], 
       Hue[0.4891772391858982, 1, 1], 
       Hue[0.5928155783977882, 1, 1], 
       Hue[0.5928155783977882, 1, 1], 
       Hue[0.6934891136564652, 1, 1], 
       Hue[0.6934891136564652, 1, 1], 
       Hue[0.793080936386119, 1, 1], 
       Hue[0.793080936386119, 1, 1], 
       Hue[0.8998311366440165, 1, 1], 
       Hue[0.8998311366440165, 1, 1], 
       Hue[0.015176503380478905`, 1, 1], 
       Hue[0.015176503380478905`, 1, 1], 
       Hue[0.13044450564833976`, 1, 1], 
       Hue[0.13044450564833976`, 1, 1], 
       Hue[0.23708488707843128`, 1, 1], 
       Hue[0.23708488707843128`, 1, 1], 
       Hue[0.33533235811624007`, 1, 1], 
       Hue[0.33533235811624007`, 1, 1], 
       Hue[0.4339269518627205, 1, 1], 
       Hue[0.4339269518627205, 1, 1], 
       Hue[0.551635230054077, 1, 1], 
       Hue[0.551635230054077, 1, 1], 
       Hue[0.7025576980009608, 1, 1], 
       Hue[0.7025576980009608, 1, 1], 
       Hue[0.8827000587356161, 1, 1], 
       Hue[0.8827000587356161, 1, 1], 
       Hue[0.08418342314818303, 1, 1], 
       Hue[0.08418342314818303, 1, 1], 
       Hue[0.7699995303057103, 1, 1], 
       Hue[0.7699995303057103, 1, 1], 
       Hue[0.02429688523103022, 1, 1], 
       Hue[0.02429688523103022, 1, 1], 
       Hue[0.27179627875331236`, 1, 1], 
       Hue[0.27179627875331236`, 1, 1], 
       Hue[0.5495531291764919, 1, 1], 
       Hue[0.5495531291764919, 1, 1], 
       Hue[0.8756326603745737, 1, 1], 
       Hue[0.8756326603745737, 1, 1], 
       Hue[0.1870945145097854, 1, 1], 
       Hue[0.1870945145097854, 1, 1], 
       Hue[0.3864954222807409, 1, 1], 
       Hue[0.3864954222807409, 1, 1], 
       Hue[0.44858201992730534`, 1, 1], 
       Hue[0.44858201992730534`, 1, 1], 
       Hue[0.6439880334050112, 1, 1], 
       Hue[0.6439880334050112, 1, 1], 
       Hue[0.8858058217557241, 1, 1], 
       Hue[0.8858058217557241, 1, 1], 
       Hue[0.14297565563919354`, 1, 1], 
       Hue[0.14297565563919354`, 1, 1], 
       Hue[0.4141276726633181, 1, 1], 
       Hue[0.4141276726633181, 1, 1], 
       Hue[0.7168368232742479, 1, 1], 
       Hue[0.7168368232742479, 1, 1], 
       Hue[0.14197056687390944`, 1, 1], 
       Hue[0.14197056687390944`, 1, 1], 
       Hue[0.5524620912908854, 1, 1], 
       Hue[0.5524620912908854, 1, 1], 
       Hue[0.8553893891830866, 1, 1], 
       Hue[0.8553893891830866, 1, 1], 
       Hue[0.13438705262114328`, 1, 1], 
       Hue[0.13438705262114328`, 1, 1], 
       Hue[0.4058233803609347, 1, 1], 
       Hue[0.4058233803609347, 1, 1], 
       Hue[0.6738903749915348, 1, 1], 
       Hue[0.6738903749915348, 1, 1], 
       Hue[0.9401789901094688, 1, 1], 
       Hue[0.9401789901094688, 1, 1], 
       Hue[0.20547177438491626`, 1, 1], 
       Hue[0.20547177438491626`, 1, 1], 
       Hue[0.47020848142795374`, 1, 1], 
       Hue[0.47020848142795374`, 1, 1], 
       Hue[0.734657523019001, 1, 1], 
       Hue[0.734657523019001, 1, 1], 
       Hue[0.9990322339559546, 1, 1], 
       Hue[0.9990322339559546, 1, 1], 
       Hue[0.2636330251999652, 1, 1], 
       Hue[0.2636330251999652, 1, 1], 
       Hue[0.5290628537980339, 1, 1], 
       Hue[0.5290628537980339, 1, 1], 
       Hue[0.7965350950639479, 1, 1], 
       Hue[0.7965350950639479, 1, 1], 
       Hue[0.06817073638838694, 1, 1], 
       Hue[0.06817073638838694, 1, 1], 
       Hue[0.3467897323971712, 1, 1], 
       Hue[0.3467897323971712, 1, 1], 
       Hue[0.6341252035470084, 1, 1], 
       Hue[0.6341252035470084, 1, 1], 
       Hue[0.9275574020329773, 1, 1], 
       Hue[0.9275574020329773, 1, 1], 
       Hue[0.22028416339828585`, 1, 1], 
       Hue[0.22028416339828585`, 1, 1], 
       Hue[0.5071579118616438, 1, 1], 
       Hue[0.5071579118616438, 1, 1], 
       Hue[0.7874886413105265, 1, 1], 
       Hue[0.7874886413105265, 1, 1], 
       Hue[0.06275314718690028, 1, 1], 
       Hue[0.06275314718690028, 1, 1], 
       Hue[0.3345026343916775, 1, 1], 
       Hue[0.3345026343916775, 1, 1], 
       Hue[0.6037857107221525, 1, 1], 
       Hue[0.6037857107221525, 1, 1], 
       Hue[0.8712023572484743, 1, 1], 
       Hue[0.8712023572484743, 1, 1], 
       Hue[0.13705768198816723`, 1, 1], 
       Hue[0.13705768198816723`, 1, 1], 
       Hue[0.401476064729792, 1, 1], 
       Hue[0.401476064729792, 1, 1], 
       Hue[0.6644687447989364, 1, 1], 
       Hue[0.6644687447989364, 1, 1], 
       Hue[0.9259678258280817, 1, 1], 
       Hue[0.9259678258280817, 1, 1], 
       Hue[0.18583835754938358`, 1, 1], 
       Hue[0.18583835754938358`, 1, 1], 
       Hue[0.44387781820781336`, 1, 1], 
       Hue[0.44387781820781336`, 1, 1], 
       Hue[0.6998052797815966, 1, 1], 
       Hue[0.6998052797815966, 1, 1], 
       Hue[0.9532478642107147, 1, 1], 
       Hue[0.9532478642107147, 1, 1], 
       Hue[0.20373127791721357`, 1, 1], 
       Hue[0.20373127791721357`, 1, 1], 
       Hue[0.45069692166927705`, 1, 1], 
       Hue[0.45069692166927705`, 1, 1], 
       Hue[0.693588719734344, 1, 1], 
       Hue[0.693588719734344, 1, 1], 
       Hue[0.9320772375617843, 1, 1], 
       Hue[0.9320772375617843, 1, 1], 
       Hue[0.16645123969962386`, 1, 1], 
       Hue[0.16645123969962386`, 1, 1], 
       Hue[0.398004171513236, 1, 1], 
       Hue[0.398004171513236, 1, 1], 
       Hue[0.6289620661548705, 1, 1], 
       Hue[0.6289620661548705, 1, 1], 
       Hue[0.8617056861385596, 1, 1], 
       Hue[0.8617056861385596, 1, 1], 
       Hue[0.09785397911062299, 1, 1], 
       Hue[0.09785397911062299, 1, 1], 
       Hue[0.33796398999826804`, 1, 1]}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->{True, None},
    AxesOrigin->{0, 0},
    Epilog->
     LineBox[{{1, 1.304380383737595*^-6}, {2, 9.629156898030052*^-7}, {
       3, 9.032900884567711*^-7}, {4, 1.0373227579521636`*^-6}, {
       5, 1.1315018027794283`*^-6}, {6, 1.432676966517547*^-6}, {
       7, 1.5242952737204173`*^-6}, {8, 1.4674159318854584`*^-6}, {
       9, 1.4190676919269421`*^-6}, {10, 1.2139812463018507`*^-6}, {
       11, 1.2904452266002735`*^-6}, {12, 1.538037344047358*^-6}, {
       13, 1.5125623946061144`*^-6}, {14, 1.397844282342397*^-6}, {
       15, 1.0782540134931227`*^-6}, {16, 9.061955126660892*^-7}, {
       17, 5.362202811743981*^-7}, {18, 8.152745909536184*^-7}, {
       19, 6.332578572294198*^-7}, {20, 6.449360220860603*^-7}, {
       21, 6.33376259274902*^-7}, {22, 7.887259241188358*^-7}, {
       23, 3.6743487421042405`*^-7}, {24, 1.235500858470756*^-7}, {
       25, 5.064205796295638*^-7}, {26, 1.0841054261785144`*^-6}, {
       27, 7.044021040626456*^-7}, {28, 9.808584854079515*^-7}, {
       29, 8.956095248079672*^-7}, {30, 5.439898906183192*^-7}, {
       31, 1.103570430545134*^-6}, {32, 8.54911256353534*^-7}, {
       33, 1.07874097863397*^-6}, {34, 7.811829745586429*^-7}, {
       35, 9.5592624380291*^-7}, {36, 5.298554720683576*^-7}, {
       37, 3.8995572112254075`*^-7}, {38, 4.427745130584191*^-7}, {
       39, 9.928965231940917*^-7}, {40, 1.112065013186867*^-6}, {
       41, 1.7190219615230559`*^-6}, {42, 1.8282257305026606`*^-6}, {
       43, 1.4711942109645388`*^-6}, {44, 2.2477877159309747`*^-6}, {
       45, 3.172475454053349*^-6}, {46, 3.723124351694845*^-6}, {
       47, 5.458674755317944*^-6}, {48, 6.6410853868583535`*^-6}, {
       49, 8.03685681726843*^-6}, {50, 0.000012243606787475356`}, {
       51, 0.000016202644180892942`}, {52, 0.000020916086173850225`}, {
       53, 0.00002715864602049856}, {54, 0.00003578593120562926}, {
       55, 0.000046707612943799305`}, {56, 0.00006093738990640693}, {
       57, 0.00007931776292023355}, {58, 0.00010124319103865022`}, {
       59, 0.00012965715251245875`}, {60, 0.0001652184262445919}, {
       61, 0.00021070607757578402`}, {62, 0.00026706057946122885`}, {
       63, 0.0003375466976779362}, {64, 0.000424750641144717}, {
       65, 0.0005335389928184603}, {66, 0.0006685709324789817}, {
       67, 0.000833697099042285}, {68, 0.0010351930740482775`}, {
       69, 0.001281545042875129}, {70, 0.0015817836788480037`}, {
       71, 0.0019445059845353218`}, {72, 0.0023819412998479996`}, {
       73, 0.0029071103864555577`}, {74, 0.0035356522085702603`}, {
       75, 0.004284295821573266}, {76, 0.005174325557735281}, {
       77, 0.0062266481535458015`}, {78, 0.007465823368299143}, {
       79, 0.008919008605186053}, {80, 0.01061821172547095}, {
       81, 0.012597352819328065`}, {82, 0.014891544699183724`}, {
       83, 0.017540079728647955`}, {84, 0.020585731204115325`}, {
       85, 0.02407462916734266}, {86, 0.028056806327298605`}, {
       87, 0.03258154002325569}, {88, 0.03770128574982179}, {
       89, 0.04346872822828755}, {90, 0.049941338680835376`}, {
       91, 0.0571771557928496}, {92, 0.06523001180642563}, {
       93, 0.07415248764148753}, {94, 0.0839979716623994}, {
       95, 0.09481631665433748}, {96, 0.10665439417967083`}, {
       97, 0.11955198565177566`}, {98, 0.1335406133311027}, {
       99, 0.14864848607449443`}, {100, 0.16489410852233816`}, {
       101, 0.18228798800728993`}, {102, 0.20082834815434092`}, {
       103, 0.22050392699708302`}, {104, 0.24128891036818118`}, {
       105, 0.263147951520146}, {106, 0.28604035243811315`}, {
       107, 0.3099115410549996}, {108, 0.334683039515788}, {
       109, 0.3602663228412355}, {110, 0.38656268562356294`}, {
       111, 0.4134835469962792}, {112, 0.4409265278656139}, {
       113, 0.4687620701608355}, {114, 0.49682142648520244`}, {
       115, 0.5249285694578344}, {116, 0.552924946944084}, {
       117, 0.5806733894641674}, {118, 0.6079994045309144}, {
       119, 0.63462523765809}, {120, 0.6601575908398875}, {
       121, 0.6841621818362645}, {122, 0.7062554709286154}, {
       123, 0.7261075128035555}, {124, 0.7433148537999551}, {
       125, 0.7572078110597844}, {126, 0.7667599545390897}, {
       127, 0.7706756788784703}, {128, 0.7676502807652559}, {
       129, 0.7566859746593542}, {130, 0.7373488418893955}, {
       131, 0.7098626238028884}, {132, 0.6750710832816553}, {
       133, 0.6342832833534616}, {134, 0.5890813679254239}, {
       135, 0.5411240287219006}, {136, 0.4919916061629801}, {
       137, 0.4430774442912328}, {138, 0.3955303462601081}, {
       139, 0.3502314542429103}, {140, 0.3078085569932737}, {
       141, 0.26866006459640857`}, {142, 0.23299054352683432`}, {
       143, 0.2008530161252165}, {144, 0.17218190691170104`}, {
       145, 0.14682603144900405`}, {146, 0.12458004102748296`}, {
       147, 0.10520184293587102`}, {148, 0.08843238184821163}, {
       149, 0.0740093904889083}, {150, 0.0616756501235092}, {
       151, 0.05118426414991482}, {152, 0.042305326067561856`}, {
       153, 0.034828991055787774`}, {154, 0.02856219766143268}, {
       155, 0.023332515058396282`}, {156, 0.01898868316559714}, {
       157, 0.015395866592511739`}, {158, 0.012436135985384074`}, {
       159, 0.010008781861144097`}, {160, 0.008026172857656336}, {
       161, 0.006412440134820296}, {162, 0.00510450378905798}, {
       163, 0.004049021298917947}, {164, 0.003200306833496937}, {
       165, 0.0025201728799015273`}, {166, 0.0019773560552377393`}, {
       167, 0.001546238502737701}, {168, 0.001204890761178659}, {
       169, 0.0009350022339971854}, {170, 0.0007226613520117723}, {
       171, 0.000556982984964478}, {172, 0.0004282547875772621}, {
       173, 0.0003279812780222784}, {174, 0.00024993972448124314`}, {
       175, 0.00018993337322628718`}, {176, 0.000144209093655313}, {
       177, 0.00010907024451671457`}, {178, 0.00008184276173099311}, {
       179, 0.00006097626515968016}, {180, 0.00004530442269158052}, {
       181, 0.000033658099287237654`}, {182, 0.0000248843899827779}, {
       183, 0.000018153907330621393`}, {184, 0.000013223451202338003`}, {
       185, 9.963764949990425*^-6}, {186, 7.724307684131922*^-6}, {
       187, 5.80543629987835*^-6}, {188, 4.102828122612841*^-6}, {
       189, 2.827451607612426*^-6}, {190, 2.032892416954505*^-6}, {
       191, 1.6106628657117503`*^-6}, {192, 1.4041728160521102`*^-6}, {
       193, 1.1798805921599137`*^-6}, {194, 8.079934780963365*^-7}, {
       195, 3.556446757918429*^-7}, {196, 2.546949050174494*^-8}, {
       197, 2.2850477355468987`*^-7}, {198, 2.528800192503595*^-7}, {
       199, 1.806456782338626*^-7}, {200, 1.1667792907080793`*^-7}, {
       201, 9.006204444596412*^-8}, {202, 5.635361049712508*^-8}, {
       203, 7.32045438122571*^-8}, {204, 1.3707651137223234`*^-7}, {
       205, 1.778813899422182*^-7}, {206, 1.7806604029286668`*^-7}, {
       207, 1.4189267605809225`*^-7}, {208, 8.578004895793163*^-8}, {
       209, 4.9819202470450704`*^-8}, {210, 8.972369549354587*^-8}, {
       211, 1.4506298276404592`*^-7}, {212, 1.8975320201858187`*^-7}, {
       213, 2.1856689255803505`*^-7}, {214, 2.3055555397033795`*^-7}, {
       215, 2.2708109218557778`*^-7}, {216, 2.1110640057225708`*^-7}, {
       217, 1.8650892436038908`*^-7}, {218, 1.5736542112103807`*^-7}, {
       219, 1.2733903044577994`*^-7}, {220, 9.92704270549027*^-8}, {
       221, 7.500484520111254*^-8}, {222, 5.542957376388741*^-8}, {
       223, 4.066821604665421*^-8}, {224, 3.036283441282917*^-8}, {
       225, 2.3928998369784318`*^-8}, {226, 2.0627484727031363`*^-8}, {
       227, 1.9495985991114893`*^-8}, {228, 1.9482414369140438`*^-8}, {
       229, 1.9744327165034357`*^-8}, {230, 1.9768316718986537`*^-8}, {
       231, 1.9314466284698768`*^-8}, {232, 1.8328711162169572`*^-8}, {
       233, 1.6872297479929105`*^-8}, {234, 1.5069496006086792`*^-8}, {
       235, 1.3068841459865362`*^-8}, {236, 1.1015253781473726`*^-8}, {
       237, 9.032110441132644*^-9}, {238, 7.211733456886917*^-9}, {
       239, 5.612885300624509*^-9}, {240, 4.2632885328740104`*^-9}, {
       241, 3.1652340412508406`*^-9}, {242, 2.3025167109426824`*^-9}, {
       243, 1.6473203311378306`*^-9}, {244, 1.1661145086376094`*^-9}, {
       245, 8.241084823545091*^-10}, {246, 5.882170342111297*^-10}, {
       247, 4.2890653665416393`*^-10}, {248, 3.215122368663073*^-10}, {
       249, 2.472321589515314*^-10}, {250, 1.9331961245459556`*^-10}, {
       251, 1.521207633440912*^-10}}],
    Frame->True,
    FrameTicks->{{{1., 
        FormBox[
         RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
        FormBox[
         RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
        FormBox[
         RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
        FormBox[
         RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
        FormBox["0", TraditionalForm]}, {157.25, 
        FormBox["2.5`", TraditionalForm]}, {188.5, 
        FormBox["5", TraditionalForm]}, {219.75, 
        FormBox["7.5`", TraditionalForm]}, {251., 
        FormBox["10", TraditionalForm]}}, Automatic, None, None},
    PlotRange->{-0.1, 1.1},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], Automatic}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF2WlUFGcWBuBSiZJIEpXEGEBpUCPirhjQILyIQhQJW7MvFg00W9OAuERE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     "], {
      {GrayLevel[0.5], EdgeForm[None], 
       GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ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         "]]]}, {{}, {}, {
        {Hue[0.8135838915291268, 1, 1], LineBox[{1, 2}]}, 
        {Hue[0.7439114747630289, 1, 1], LineBox[{2, 3}]}, 
        {Hue[0.6845940903831713, 1, 1], LineBox[{3, 4}]}, 
        {Hue[0.5899282786123202, 1, 1], LineBox[{4, 5}]}, 
        {Hue[0.5537484979137711, 1, 1], LineBox[{5, 6}]}, 
        {Hue[0.38417448662969855`, 1, 1], LineBox[{6, 7}]}, 
        {Hue[0.34907528892008843`, 1, 1], LineBox[{7, 8}]}, 
        {Hue[0.33302783616838333`, 1, 1], LineBox[{8, 9}]}, 
        {Hue[0.23767149843348115`, 1, 1], LineBox[{9, 10}]}, 
        {Hue[0.14225313723343055`, 1, 1], LineBox[{10, 11}]}, 
        {Hue[0.043298420103147786`, 1, 1], LineBox[{11, 12}]}, 
        {Hue[0.907956457126252, 1, 1], LineBox[{12, 13}]}, 
        {Hue[0.8863378129830908, 1, 1], LineBox[{13, 14}]}, 
        {Hue[0.7324579169392005, 1, 1], LineBox[{14, 15}]}, 
        {Hue[0.6938488004081815, 1, 1], LineBox[{15, 16}]}, 
        {Hue[0.39015314796355327`, 1, 1], LineBox[{16, 17}]}, 
        {Hue[0.47524975557936294`, 1, 1], LineBox[{17, 18}]}, 
        {Hue[0.23061422533859277`, 1, 1], LineBox[{18, 19}]}, 
        {Hue[0.18759108425177645`, 1, 1], LineBox[{19, 20}]}, 
        {Hue[0.10579347430510228`, 1, 1], LineBox[{20, 21}]}, 
        {Hue[0.9460988040503769, 1, 1], LineBox[{21, 22}]}, 
        {Hue[0.883853234242489, 1, 1], LineBox[{22, 23}]}, 
        {Hue[0.7416327814295387, 1, 1], LineBox[{23, 24}]}, 
        {Hue[0.7749364531338783, 1, 1], LineBox[{24, 25}]}, 
        {Hue[0.6422587050581101, 1, 1], LineBox[{25, 26}]}, 
        {Hue[0.5160848082862535, 1, 1], LineBox[{26, 27}]}, 
        {Hue[0.41998315715370105`, 1, 1], LineBox[{27, 28}]}, 
        {Hue[0.3677902653730239, 1, 1], LineBox[{28, 29}]}, 
        {Hue[0.2596215547704807, 1, 1], LineBox[{29, 30}]}, 
        {Hue[0.3028772709352434, 1, 1], LineBox[{30, 31}]}, 
        {Hue[0.041896744764891025`, 1, 1], LineBox[{31, 32}]}, 
        {Hue[0.8588600627762908, 1, 1], LineBox[{32, 33}]}, 
        {Hue[0.7343447439619594, 1, 1], LineBox[{33, 34}]}, 
        {Hue[0.6365872375656859, 1, 1], LineBox[{34, 35}]}, 
        {Hue[0.47100754517141513`, 1, 1], LineBox[{35, 36}]}, 
        {Hue[0.2813280179514035, 1, 1], LineBox[{36, 37}]}, 
        {Hue[0.9735888980687284, 1, 1], LineBox[{37, 38}]}, 
        {Hue[0.5552827557441284, 1, 1], LineBox[{38, 39}]}, 
        {Hue[0.9381015599604834, 1, 1], LineBox[{39, 40}]}, 
        {Hue[0.6549908474106629, 1, 1], LineBox[{40, 41}]}, 
        {Hue[0.6518281404891956, 1, 1], LineBox[{41, 42}]}, 
        {Hue[0.5400340083574924, 1, 1], LineBox[{42, 43}]}, 
        {Hue[0.506450929345753, 1, 1], LineBox[{43, 44}]}, 
        {Hue[0.5285733574892757, 1, 1], LineBox[{44, 45}]}, 
        {Hue[0.5921813181707918, 1, 1], LineBox[{45, 46}]}, 
        {Hue[0.9333562836332183, 1, 1], LineBox[{46, 47}]}, 
        {Hue[0.8853458163957009, 1, 1], LineBox[{47, 48}]}, 
        {Hue[0.853662012170136, 1, 1], LineBox[{48, 49}]}, 
        {Hue[0.8968065708923977, 1, 1], LineBox[{49, 50}]}, 
        {Hue[0.9826277227569763, 1, 1], LineBox[{50, 51}]}, 
        {Hue[0.05395650772204826, 1, 1], LineBox[{51, 52}]}, 
        {Hue[0.1370908908529463, 1, 1], LineBox[{52, 53}]}, 
        {Hue[0.18163710723817444`, 1, 1], LineBox[{53, 54}]}, 
        {Hue[0.24445818798964217`, 1, 1], LineBox[{54, 55}]}, 
        {Hue[0.30857439043837853`, 1, 1], LineBox[{55, 56}]}, 
        {Hue[0.3606133290881668, 1, 1], LineBox[{56, 57}]}, 
        {Hue[0.420044573230797, 1, 1], LineBox[{57, 58}]}, 
        {Hue[0.48419173721395464`, 1, 1], LineBox[{58, 59}]}, 
        {Hue[0.5361081510744629, 1, 1], LineBox[{59, 60}]}, 
        {Hue[0.5980659943808304, 1, 1], LineBox[{60, 61}]}, 
        {Hue[0.6600463844653425, 1, 1], LineBox[{61, 62}]}, 
        {Hue[0.7235566724925602, 1, 1], LineBox[{62, 63}]}, 
        {Hue[0.7878680606789363, 1, 1], LineBox[{63, 64}]}, 
        {Hue[0.8513131084760756, 1, 1], LineBox[{64, 65}]}, 
        {Hue[0.9141237531356108, 1, 1], LineBox[{65, 66}]}, 
        {Hue[0.9796550132334485, 1, 1], LineBox[{66, 67}]}, 
        {Hue[0.04554279610193454, 1, 1], LineBox[{67, 68}]}, 
        {Hue[0.11187390374902148`, 1, 1], LineBox[{68, 69}]}, 
        {Hue[0.17762159787379167`, 1, 1], LineBox[{69, 70}]}, 
        {Hue[0.24427137933553897`, 1, 1], LineBox[{70, 71}]}, 
        {Hue[0.3117162635752856, 1, 1], LineBox[{71, 72}]}, 
        {Hue[0.37969314417642314`, 1, 1], LineBox[{72, 73}]}, 
        {Hue[0.44815967961866965`, 1, 1], LineBox[{73, 74}]}, 
        {Hue[0.5171221163629012, 1, 1], LineBox[{74, 75}]}, 
        {Hue[0.5862931145848354, 1, 1], LineBox[{75, 76}]}, 
        {Hue[0.6560422323667225, 1, 1], LineBox[{76, 77}]}, 
        {Hue[0.7265285874458343, 1, 1], LineBox[{77, 78}]}, 
        {Hue[0.797530226193851, 1, 1], LineBox[{78, 79}]}, 
        {Hue[0.8690849455760121, 1, 1], LineBox[{79, 80}]}, 
        {Hue[0.9409585144911263, 1, 1], LineBox[{80, 81}]}, 
        {Hue[0.013394114338599833`, 1, 1], LineBox[{81, 82}]}, 
        {Hue[0.08635321152175987, 1, 1], LineBox[{82, 83}]}, 
        {Hue[0.15990432497714052`, 1, 1], LineBox[{83, 84}]}, 
        {Hue[0.2339381258426344, 1, 1], LineBox[{84, 85}]}, 
        {Hue[0.30843325664141336`, 1, 1], LineBox[{85, 86}]}, 
        {Hue[0.38345074264343776`, 1, 1], LineBox[{86, 87}]}, 
        {Hue[0.4589748378028819, 1, 1], LineBox[{87, 88}]}, 
        {Hue[0.5350070831107332, 1, 1], LineBox[{88, 89}]}, 
        {Hue[0.6115514859638331, 1, 1], LineBox[{89, 90}]}, 
        {Hue[0.688588607439024, 1, 1], LineBox[{90, 91}]}, 
        {Hue[0.7660936824814056, 1, 1], LineBox[{91, 92}]}, 
        {Hue[0.8440844237695915, 1, 1], LineBox[{92, 93}]}, 
        {Hue[0.9225589053693233, 1, 1], LineBox[{93, 94}]}, 
        {Hue[0.0015271511254317436`, 1, 1], LineBox[{94, 95}]}, 
        {Hue[0.08096308242320557, 1, 1], LineBox[{95, 96}]}, 
        {Hue[0.16084335970662272`, 1, 1], LineBox[{96, 97}]}, 
        {Hue[0.241147292875211, 1, 1], LineBox[{97, 98}]}, 
        {Hue[0.32189121078452165`, 1, 1], LineBox[{98, 99}]}, 
        {Hue[0.40307511451608796`, 1, 1], LineBox[{99, 100}]}, 
        {Hue[0.48468106235468705`, 1, 1], LineBox[{100, 101}]}, 
        {Hue[0.5666666550014439, 1, 1], LineBox[{101, 102}]}, 
        {Hue[0.648984876342054, 1, 1], LineBox[{102, 103}]}, 
        {Hue[0.7316139984139767, 1, 1], LineBox[{103, 104}]}, 
        {Hue[0.8145654914194567, 1, 1], LineBox[{104, 105}]}, 
        {Hue[0.8978277043733119, 1, 1], LineBox[{105, 106}]}, 
        {Hue[0.9813450138645246, 1, 1], LineBox[{106, 107}]}, 
        {Hue[0.06501907948890219, 1, 1], LineBox[{107, 108}]}, 
        {Hue[0.14876757678712846`, 1, 1], LineBox[{108, 109}]}, 
        {Hue[0.23256003250733165`, 1, 1], LineBox[{109, 110}]}, 
        {Hue[0.31639830470966585`, 1, 1], LineBox[{110, 111}]}, 
        {Hue[0.40023530109883876`, 1, 1], LineBox[{111, 112}]}, 
        {Hue[0.48393024058144063`, 1, 1], LineBox[{112, 113}]}, 
        {Hue[0.5672876532384993, 1, 1], LineBox[{113, 114}]}, 
        {Hue[0.6501592758058539, 1, 1], LineBox[{114, 115}]}, 
        {Hue[0.7324987736444111, 1, 1], LineBox[{115, 116}]}, 
        {Hue[0.8142983145499002, 1, 1], LineBox[{116, 117}]}, 
        {Hue[0.8954590993731899, 1, 1], LineBox[{117, 118}]}, 
        {Hue[0.975722643047566, 1, 1], LineBox[{118, 119}]}, 
        {Hue[0.05474766844007228, 1, 1], LineBox[{119, 120}]}, 
        {Hue[0.13227308453669961`, 1, 1], LineBox[{120, 121}]}, 
        {Hue[0.20821636750529138`, 1, 1], LineBox[{121, 122}]}, 
        {Hue[0.2826194654284094, 1, 1], LineBox[{122, 123}]}, 
        {Hue[0.3554752741691703, 1, 1], LineBox[{123, 124}]}, 
        {Hue[0.4265889065828907, 1, 1], LineBox[{124, 125}]}, 
        {Hue[0.4955650664400778, 1, 1], LineBox[{125, 126}]}, 
        {Hue[0.5619584530314572, 1, 1], LineBox[{126, 127}]}, 
        {Hue[0.6254635887802759, 1, 1], LineBox[{127, 128}]}, 
        {Hue[0.6860331182124972, 1, 1], LineBox[{128, 129}]}, 
        {Hue[0.7438757636849702, 1, 1], LineBox[{129, 130}]}, 
        {Hue[0.799376342278605, 1, 1], LineBox[{130, 131}]}, 
        {Hue[0.8530037452326122, 1, 1], LineBox[{131, 132}]}, 
        {Hue[0.9052386573064211, 1, 1], LineBox[{132, 133}]}, 
        {Hue[0.9565330054338083, 1, 1], LineBox[{133, 134}]}, 
        {Hue[0.007289019921117269, 1, 1], LineBox[{134, 135}]}, 
        {Hue[0.05785436886123326, 1, 1], LineBox[{135, 136}]}, 
        {Hue[0.10852361694626046`, 1, 1], LineBox[{136, 137}]}, 
        {Hue[0.1595424939905707, 1, 1], LineBox[{137, 138}]}, 
        {Hue[0.21111490902177088`, 1, 1], LineBox[{138, 139}]}, 
        {Hue[0.2634076300324872, 1, 1], LineBox[{139, 140}]}, 
        {Hue[0.31655628975593003`, 1, 1], LineBox[{140, 141}]}, 
        {Hue[0.37066907391576337`, 1, 1], LineBox[{141, 142}]}, 
        {Hue[0.42583171930078956`, 1, 1], LineBox[{142, 143}]}, 
        {Hue[0.4821112905542982, 1, 1], LineBox[{143, 144}]}, 
        {Hue[0.5395581595296779, 1, 1], LineBox[{144, 145}]}, 
        {Hue[0.5982089184764874, 1, 1], LineBox[{145, 146}]}, 
        {Hue[0.6580902546204306, 1, 1], LineBox[{146, 147}]}, 
        {Hue[0.7192184186201758, 1, 1], LineBox[{147, 148}]}, 
        {Hue[0.7816029392773315, 1, 1], LineBox[{148, 149}]}, 
        {Hue[0.845246320651291, 1, 1], LineBox[{149, 150}]}, 
        {Hue[0.9101471280449227, 1, 1], LineBox[{150, 151}]}, 
        {Hue[0.9762994778057256, 1, 1], LineBox[{151, 152}]}, 
        {Hue[0.043693647064499526`, 1, 1], LineBox[{152, 153}]}, 
        {Hue[0.1123192138725254, 1, 1], LineBox[{153, 154}]}, 
        {Hue[0.18216349795160328`, 1, 1], LineBox[{154, 155}]}, 
        {Hue[0.2532094583491348, 1, 1], LineBox[{155, 156}]}, 
        {Hue[0.32544339799515104`, 1, 1], LineBox[{156, 157}]}, 
        {Hue[0.39884894807087073`, 1, 1], LineBox[{157, 158}]}, 
        {Hue[0.4734080173466877, 1, 1], LineBox[{158, 159}]}, 
        {Hue[0.5491064682724095, 1, 1], LineBox[{159, 160}]}, 
        {Hue[0.6259260435306573, 1, 1], LineBox[{160, 161}]}, 
        {Hue[0.7038469620065598, 1, 1], LineBox[{161, 162}]}, 
        {Hue[0.7828539447311839, 1, 1], LineBox[{162, 163}]}, 
        {Hue[0.86293510134423, 1, 1], LineBox[{163, 164}]}, 
        {Hue[0.9440694759822695, 1, 1], LineBox[{164, 165}]}, 
        {Hue[0.026237977530103335`, 1, 1], LineBox[{165, 166}]}, 
        {Hue[0.10943912867403406`, 1, 1], LineBox[{166, 167}]}, 
        {Hue[0.1936492364678344, 1, 1], LineBox[{167, 168}]}, 
        {Hue[0.2788390158538952, 1, 1], LineBox[{168, 169}]}, 
        {Hue[0.36501718255201326`, 1, 1], LineBox[{169, 170}]}, 
        {Hue[0.4521779601579034, 1, 1], LineBox[{170, 171}]}, 
        {Hue[0.5402709397872767, 1, 1], LineBox[{171, 172}]}, 
        {Hue[0.6292987165623611, 1, 1], LineBox[{172, 173}]}, 
        {Hue[0.7192852763459747, 1, 1], LineBox[{173, 174}]}, 
        {Hue[0.8101807715827263, 1, 1], LineBox[{174, 175}]}, 
        {Hue[0.9019724559267499, 1, 1], LineBox[{175, 176}]}, 
        {Hue[0.99470033906565, 1, 1], LineBox[{176, 177}]}, 
        {Hue[0.08832783678624961, 1, 1], LineBox[{177, 178}]}, 
        {Hue[0.18277987722455066`, 1, 1], LineBox[{178, 179}]}, 
        {Hue[0.2780640171066228, 1, 1], LineBox[{179, 180}]}, 
        {Hue[0.3742762888141277, 1, 1], LineBox[{180, 181}]}, 
        {Hue[0.4714059652914306, 1, 1], LineBox[{181, 182}]}, 
        {Hue[0.5692514338009139, 1, 1], LineBox[{182, 183}]}, 
        {Hue[0.6677981386855287, 1, 1], LineBox[{183, 184}]}, 
        {Hue[0.7673407367346702, 1, 1], LineBox[{184, 185}]}, 
        {Hue[0.8678504620247419, 1, 1], LineBox[{185, 186}]}, 
        {Hue[0.9686973038820387, 1, 1], LineBox[{186, 187}]}, 
        {Hue[0.06963060970896097, 1, 1], LineBox[{187, 188}]}, 
        {Hue[0.17180207357002875`, 1, 1], LineBox[{188, 189}]}, 
        {Hue[0.27636830103356114`, 1, 1], LineBox[{189, 190}]}, 
        {Hue[0.38209981443647867`, 1, 1], LineBox[{190, 191}]}, 
        {Hue[0.48684916138316015`, 1, 1], LineBox[{191, 192}]}, 
        {Hue[0.591638674225299, 1, 1], LineBox[{192, 193}]}, 
        {Hue[0.6996440399500621, 1, 1], LineBox[{193, 194}]}, 
        {Hue[0.8106408769056764, 1, 1], LineBox[{194, 195}]}, 
        {Hue[0.9203208846246161, 1, 1], LineBox[{195, 196}]}, 
        {Hue[0.02405236895185515, 1, 1], LineBox[{196, 197}]}, 
        {Hue[0.1228424398166391, 1, 1], LineBox[{197, 198}]}, 
        {Hue[0.22619536848471866`, 1, 1], LineBox[{198, 199}]}, 
        {Hue[0.3370375970414235, 1, 1], LineBox[{199, 200}]}, 
        {Hue[0.4399614845447659, 1, 1], LineBox[{200, 201}]}, 
        {Hue[0.5268701196885841, 1, 1], LineBox[{201, 202}]}, 
        {Hue[0.6371482431217275, 1, 1], LineBox[{202, 203}]}, 
        {Hue[0.7836147903328474, 1, 1], LineBox[{203, 204}]}, 
        {Hue[0.9442860013131829, 1, 1], LineBox[{204, 205}]}, 
        {Hue[0.09297398599726875, 1, 1], LineBox[{205, 206}]}, 
        {Hue[0.17887958096169843`, 1, 1], LineBox[{206, 207}]}, 
        {Hue[0.19638153085605062`, 1, 1], LineBox[{207, 208}]}, 
        {Hue[0.30819964433545805`, 1, 1], LineBox[{208, 209}]}, 
        {Hue[0.4887367267379165, 1, 1], LineBox[{209, 210}]}, 
        {Hue[0.7100622885787811, 1, 1], LineBox[{210, 211}]}, 
        {Hue[0.9616419546043693, 1, 1], LineBox[{211, 212}]}, 
        {Hue[0.23801211969541905`, 1, 1], LineBox[{212, 213}]}, 
        {Hue[0.5347134004266552, 1, 1], LineBox[{213, 214}]}, 
        {Hue[0.851720999705813, 1, 1], LineBox[{214, 215}]}, 
        {Hue[0.19772709565797011`, 1, 1], LineBox[{215, 216}]}, 
        {Hue[0.5542643360085939, 1, 1], LineBox[{216, 217}]}, 
        {Hue[0.8726282053207132, 1, 1], LineBox[{217, 218}]}, 
        {Hue[0.1490042529253488, 1, 1], LineBox[{218, 219}]}, 
        {Hue[0.37880560395471286`, 1, 1], LineBox[{219, 220}]}, 
        {Hue[0.5180102868178059, 1, 1], LineBox[{220, 221}]}, 
        {Hue[0.6661968199894867, 1, 1], LineBox[{221, 222}]}, 
        {Hue[0.8912713919398864, 1, 1], LineBox[{222, 223}]}, 
        {Hue[0.14372955017319863`, 1, 1], LineBox[{223, 224}]}, 
        {Hue[0.40895675323907044`, 1, 1], LineBox[{224, 225}]}, 
        {Hue[0.687498852931313, 1, 1], LineBox[{225, 226}]}, 
        {Hue[0.9983927521379041, 1, 1], LineBox[{226, 227}]}, 
        {Hue[0.41402290612424353`, 1, 1], LineBox[{227, 228}]}, 
        {Hue[0.8107856520326657, 1, 1], LineBox[{228, 229}]}, 
        {Hue[0.11575640376966859`, 1, 1], LineBox[{229, 230}]}, 
        {Hue[0.39601159431580035`, 1, 1], LineBox[{230, 231}]}, 
        {Hue[0.6681929871460615, 1, 1], LineBox[{231, 232}]}, 
        {Hue[0.9369112472018684, 1, 1], LineBox[{232, 233}]}, 
        {Hue[0.20387105287694343`, 1, 1], LineBox[{233, 234}]}, 
        {Hue[0.46982427789568754`, 1, 1], LineBox[{234, 235}]}, 
        {Hue[0.7351428404407656, 1, 1], LineBox[{235, 236}]}, 
        {Hue[0.000045043010442031465`, 1, 1], LineBox[{236, 237}]}, 
        {Hue[0.26472928293010783`, 1, 1], LineBox[{237, 238}]}, 
        {Hue[0.529502741806376, 1, 1], LineBox[{238, 239}]}, 
        {Hue[0.7949436009945829, 1, 1], LineBox[{239, 240}]}, 
        {Hue[0.06210321930714255, 1, 1], LineBox[{240, 241}]}, 
        {Hue[0.33266837165463214`, 1, 1], LineBox[{241, 242}]}, 
        {Hue[0.6088044722098869, 1, 1], LineBox[{242, 243}]}, 
        {Hue[0.892140756761359, 1, 1], LineBox[{243, 244}]}, 
        {Hue[0.18175714747005245`, 1, 1], LineBox[{244, 245}]}, 
        {Hue[0.47326275941150026`, 1, 1], LineBox[{245, 246}]}, 
        {Hue[0.7616040472763749, 1, 1], LineBox[{246, 247}]}, 
        {Hue[0.044437360962688716`, 1, 1], LineBox[{247, 248}]}, 
        {Hue[0.32202075940832503`, 1, 1], LineBox[{248, 249}]}, 
        {Hue[0.5955234481590492, 1, 1], LineBox[{249, 250}]}, 
        {Hue[0.8660326380659066, 1, 1], LineBox[{250, 251}]}}}},
     VertexColors->{
       Hue[0.8135838915291268, 1, 1], 
       Hue[0.7439114747630289, 1, 1], 
       Hue[0.6845940903831713, 1, 1], 
       Hue[0.5899282786123202, 1, 1], 
       Hue[0.5537484979137711, 1, 1], 
       Hue[0.38417448662969855`, 1, 1], 
       Hue[0.34907528892008843`, 1, 1], 
       Hue[0.33302783616838333`, 1, 1], 
       Hue[0.23767149843348115`, 1, 1], 
       Hue[0.14225313723343055`, 1, 1], 
       Hue[0.043298420103147786`, 1, 1], 
       Hue[0.907956457126252, 1, 1], 
       Hue[0.8863378129830908, 1, 1], 
       Hue[0.7324579169392005, 1, 1], 
       Hue[0.6938488004081815, 1, 1], 
       Hue[0.39015314796355327`, 1, 1], 
       Hue[0.47524975557936294`, 1, 1], 
       Hue[0.23061422533859277`, 1, 1], 
       Hue[0.18759108425177645`, 1, 1], 
       Hue[0.10579347430510228`, 1, 1], 
       Hue[0.9460988040503769, 1, 1], 
       Hue[0.883853234242489, 1, 1], 
       Hue[0.7416327814295387, 1, 1], 
       Hue[0.7749364531338783, 1, 1], 
       Hue[0.6422587050581101, 1, 1], 
       Hue[0.5160848082862535, 1, 1], 
       Hue[0.41998315715370105`, 1, 1], 
       Hue[0.3677902653730239, 1, 1], 
       Hue[0.2596215547704807, 1, 1], 
       Hue[0.3028772709352434, 1, 1], 
       Hue[0.041896744764891025`, 1, 1], 
       Hue[0.8588600627762908, 1, 1], 
       Hue[0.7343447439619594, 1, 1], 
       Hue[0.6365872375656859, 1, 1], 
       Hue[0.47100754517141513`, 1, 1], 
       Hue[0.2813280179514035, 1, 1], 
       Hue[0.9735888980687284, 1, 1], 
       Hue[0.5552827557441284, 1, 1], 
       Hue[0.9381015599604834, 1, 1], 
       Hue[0.6549908474106629, 1, 1], 
       Hue[0.6518281404891956, 1, 1], 
       Hue[0.5400340083574924, 1, 1], 
       Hue[0.506450929345753, 1, 1], 
       Hue[0.5285733574892757, 1, 1], 
       Hue[0.5921813181707918, 1, 1], 
       Hue[0.9333562836332183, 1, 1], 
       Hue[0.8853458163957009, 1, 1], 
       Hue[0.853662012170136, 1, 1], 
       Hue[0.8968065708923977, 1, 1], 
       Hue[0.9826277227569763, 1, 1], 
       Hue[0.05395650772204826, 1, 1], 
       Hue[0.1370908908529463, 1, 1], 
       Hue[0.18163710723817444`, 1, 1], 
       Hue[0.24445818798964217`, 1, 1], 
       Hue[0.30857439043837853`, 1, 1], 
       Hue[0.3606133290881668, 1, 1], 
       Hue[0.420044573230797, 1, 1], 
       Hue[0.48419173721395464`, 1, 1], 
       Hue[0.5361081510744629, 1, 1], 
       Hue[0.5980659943808304, 1, 1], 
       Hue[0.6600463844653425, 1, 1], 
       Hue[0.7235566724925602, 1, 1], 
       Hue[0.7878680606789363, 1, 1], 
       Hue[0.8513131084760756, 1, 1], 
       Hue[0.9141237531356108, 1, 1], 
       Hue[0.9796550132334485, 1, 1], 
       Hue[0.04554279610193454, 1, 1], 
       Hue[0.11187390374902148`, 1, 1], 
       Hue[0.17762159787379167`, 1, 1], 
       Hue[0.24427137933553897`, 1, 1], 
       Hue[0.3117162635752856, 1, 1], 
       Hue[0.37969314417642314`, 1, 1], 
       Hue[0.44815967961866965`, 1, 1], 
       Hue[0.5171221163629012, 1, 1], 
       Hue[0.5862931145848354, 1, 1], 
       Hue[0.6560422323667225, 1, 1], 
       Hue[0.7265285874458343, 1, 1], 
       Hue[0.797530226193851, 1, 1], 
       Hue[0.8690849455760121, 1, 1], 
       Hue[0.9409585144911263, 1, 1], 
       Hue[0.013394114338599833`, 1, 1], 
       Hue[0.08635321152175987, 1, 1], 
       Hue[0.15990432497714052`, 1, 1], 
       Hue[0.2339381258426344, 1, 1], 
       Hue[0.30843325664141336`, 1, 1], 
       Hue[0.38345074264343776`, 1, 1], 
       Hue[0.4589748378028819, 1, 1], 
       Hue[0.5350070831107332, 1, 1], 
       Hue[0.6115514859638331, 1, 1], 
       Hue[0.688588607439024, 1, 1], 
       Hue[0.7660936824814056, 1, 1], 
       Hue[0.8440844237695915, 1, 1], 
       Hue[0.9225589053693233, 1, 1], 
       Hue[0.0015271511254317436`, 1, 1], 
       Hue[0.08096308242320557, 1, 1], 
       Hue[0.16084335970662272`, 1, 1], 
       Hue[0.241147292875211, 1, 1], 
       Hue[0.32189121078452165`, 1, 1], 
       Hue[0.40307511451608796`, 1, 1], 
       Hue[0.48468106235468705`, 1, 1], 
       Hue[0.5666666550014439, 1, 1], 
       Hue[0.648984876342054, 1, 1], 
       Hue[0.7316139984139767, 1, 1], 
       Hue[0.8145654914194567, 1, 1], 
       Hue[0.8978277043733119, 1, 1], 
       Hue[0.9813450138645246, 1, 1], 
       Hue[0.06501907948890219, 1, 1], 
       Hue[0.14876757678712846`, 1, 1], 
       Hue[0.23256003250733165`, 1, 1], 
       Hue[0.31639830470966585`, 1, 1], 
       Hue[0.40023530109883876`, 1, 1], 
       Hue[0.48393024058144063`, 1, 1], 
       Hue[0.5672876532384993, 1, 1], 
       Hue[0.6501592758058539, 1, 1], 
       Hue[0.7324987736444111, 1, 1], 
       Hue[0.8142983145499002, 1, 1], 
       Hue[0.8954590993731899, 1, 1], 
       Hue[0.975722643047566, 1, 1], 
       Hue[0.05474766844007228, 1, 1], 
       Hue[0.13227308453669961`, 1, 1], 
       Hue[0.20821636750529138`, 1, 1], 
       Hue[0.2826194654284094, 1, 1], 
       Hue[0.3554752741691703, 1, 1], 
       Hue[0.4265889065828907, 1, 1], 
       Hue[0.4955650664400778, 1, 1], 
       Hue[0.5619584530314572, 1, 1], 
       Hue[0.6254635887802759, 1, 1], 
       Hue[0.6860331182124972, 1, 1], 
       Hue[0.7438757636849702, 1, 1], 
       Hue[0.799376342278605, 1, 1], 
       Hue[0.8530037452326122, 1, 1], 
       Hue[0.9052386573064211, 1, 1], 
       Hue[0.9565330054338083, 1, 1], 
       Hue[0.007289019921117269, 1, 1], 
       Hue[0.05785436886123326, 1, 1], 
       Hue[0.10852361694626046`, 1, 1], 
       Hue[0.1595424939905707, 1, 1], 
       Hue[0.21111490902177088`, 1, 1], 
       Hue[0.2634076300324872, 1, 1], 
       Hue[0.31655628975593003`, 1, 1], 
       Hue[0.37066907391576337`, 1, 1], 
       Hue[0.42583171930078956`, 1, 1], 
       Hue[0.4821112905542982, 1, 1], 
       Hue[0.5395581595296779, 1, 1], 
       Hue[0.5982089184764874, 1, 1], 
       Hue[0.6580902546204306, 1, 1], 
       Hue[0.7192184186201758, 1, 1], 
       Hue[0.7816029392773315, 1, 1], 
       Hue[0.845246320651291, 1, 1], 
       Hue[0.9101471280449227, 1, 1], 
       Hue[0.9762994778057256, 1, 1], 
       Hue[0.043693647064499526`, 1, 1], 
       Hue[0.1123192138725254, 1, 1], 
       Hue[0.18216349795160328`, 1, 1], 
       Hue[0.2532094583491348, 1, 1], 
       Hue[0.32544339799515104`, 1, 1], 
       Hue[0.39884894807087073`, 1, 1], 
       Hue[0.4734080173466877, 1, 1], 
       Hue[0.5491064682724095, 1, 1], 
       Hue[0.6259260435306573, 1, 1], 
       Hue[0.7038469620065598, 1, 1], 
       Hue[0.7828539447311839, 1, 1], 
       Hue[0.86293510134423, 1, 1], 
       Hue[0.9440694759822695, 1, 1], 
       Hue[0.026237977530103335`, 1, 1], 
       Hue[0.10943912867403406`, 1, 1], 
       Hue[0.1936492364678344, 1, 1], 
       Hue[0.2788390158538952, 1, 1], 
       Hue[0.36501718255201326`, 1, 1], 
       Hue[0.4521779601579034, 1, 1], 
       Hue[0.5402709397872767, 1, 1], 
       Hue[0.6292987165623611, 1, 1], 
       Hue[0.7192852763459747, 1, 1], 
       Hue[0.8101807715827263, 1, 1], 
       Hue[0.9019724559267499, 1, 1], 
       Hue[0.99470033906565, 1, 1], 
       Hue[0.08832783678624961, 1, 1], 
       Hue[0.18277987722455066`, 1, 1], 
       Hue[0.2780640171066228, 1, 1], 
       Hue[0.3742762888141277, 1, 1], 
       Hue[0.4714059652914306, 1, 1], 
       Hue[0.5692514338009139, 1, 1], 
       Hue[0.6677981386855287, 1, 1], 
       Hue[0.7673407367346702, 1, 1], 
       Hue[0.8678504620247419, 1, 1], 
       Hue[0.9686973038820387, 1, 1], 
       Hue[0.06963060970896097, 1, 1], 
       Hue[0.17180207357002875`, 1, 1], 
       Hue[0.27636830103356114`, 1, 1], 
       Hue[0.38209981443647867`, 1, 1], 
       Hue[0.48684916138316015`, 1, 1], 
       Hue[0.591638674225299, 1, 1], 
       Hue[0.6996440399500621, 1, 1], 
       Hue[0.8106408769056764, 1, 1], 
       Hue[0.9203208846246161, 1, 1], 
       Hue[0.02405236895185515, 1, 1], 
       Hue[0.1228424398166391, 1, 1], 
       Hue[0.22619536848471866`, 1, 1], 
       Hue[0.3370375970414235, 1, 1], 
       Hue[0.4399614845447659, 1, 1], 
       Hue[0.5268701196885841, 1, 1], 
       Hue[0.6371482431217275, 1, 1], 
       Hue[0.7836147903328474, 1, 1], 
       Hue[0.9442860013131829, 1, 1], 
       Hue[0.09297398599726875, 1, 1], 
       Hue[0.17887958096169843`, 1, 1], 
       Hue[0.19638153085605062`, 1, 1], 
       Hue[0.30819964433545805`, 1, 1], 
       Hue[0.4887367267379165, 1, 1], 
       Hue[0.7100622885787811, 1, 1], 
       Hue[0.9616419546043693, 1, 1], 
       Hue[0.23801211969541905`, 1, 1], 
       Hue[0.5347134004266552, 1, 1], 
       Hue[0.851720999705813, 1, 1], 
       Hue[0.19772709565797011`, 1, 1], 
       Hue[0.5542643360085939, 1, 1], 
       Hue[0.8726282053207132, 1, 1], 
       Hue[0.1490042529253488, 1, 1], 
       Hue[0.37880560395471286`, 1, 1], 
       Hue[0.5180102868178059, 1, 1], 
       Hue[0.6661968199894867, 1, 1], 
       Hue[0.8912713919398864, 1, 1], 
       Hue[0.14372955017319863`, 1, 1], 
       Hue[0.40895675323907044`, 1, 1], 
       Hue[0.687498852931313, 1, 1], 
       Hue[0.9983927521379041, 1, 1], 
       Hue[0.41402290612424353`, 1, 1], 
       Hue[0.8107856520326657, 1, 1], 
       Hue[0.11575640376966859`, 1, 1], 
       Hue[0.39601159431580035`, 1, 1], 
       Hue[0.6681929871460615, 1, 1], 
       Hue[0.9369112472018684, 1, 1], 
       Hue[0.20387105287694343`, 1, 1], 
       Hue[0.46982427789568754`, 1, 1], 
       Hue[0.7351428404407656, 1, 1], 
       Hue[0.000045043010442031465`, 1, 1], 
       Hue[0.26472928293010783`, 1, 1], 
       Hue[0.529502741806376, 1, 1], 
       Hue[0.7949436009945829, 1, 1], 
       Hue[0.06210321930714255, 1, 1], 
       Hue[0.33266837165463214`, 1, 1], 
       Hue[0.6088044722098869, 1, 1], 
       Hue[0.892140756761359, 1, 1], 
       Hue[0.18175714747005245`, 1, 1], 
       Hue[0.47326275941150026`, 1, 1], 
       Hue[0.7616040472763749, 1, 1], 
       Hue[0.044437360962688716`, 1, 1], 
       Hue[0.32202075940832503`, 1, 1], 
       Hue[0.5955234481590492, 1, 1], 
       Hue[0.8660326380659066, 1, 1], 
       Hue[0.1343067124487573, 1, 1], 
       Hue[0.8135838915291268, 1, 1], 
       Hue[0.7439114747630289, 1, 1], 
       Hue[0.7439114747630289, 1, 1], 
       Hue[0.6845940903831713, 1, 1], 
       Hue[0.6845940903831713, 1, 1], 
       Hue[0.5899282786123202, 1, 1], 
       Hue[0.5899282786123202, 1, 1], 
       Hue[0.5537484979137711, 1, 1], 
       Hue[0.5537484979137711, 1, 1], 
       Hue[0.38417448662969855`, 1, 1], 
       Hue[0.38417448662969855`, 1, 1], 
       Hue[0.34907528892008843`, 1, 1], 
       Hue[0.34907528892008843`, 1, 1], 
       Hue[0.33302783616838333`, 1, 1], 
       Hue[0.33302783616838333`, 1, 1], 
       Hue[0.23767149843348115`, 1, 1], 
       Hue[0.23767149843348115`, 1, 1], 
       Hue[0.14225313723343055`, 1, 1], 
       Hue[0.14225313723343055`, 1, 1], 
       Hue[0.043298420103147786`, 1, 1], 
       Hue[0.043298420103147786`, 1, 1], 
       Hue[0.907956457126252, 1, 1], 
       Hue[0.907956457126252, 1, 1], 
       Hue[0.8863378129830908, 1, 1], 
       Hue[0.8863378129830908, 1, 1], 
       Hue[0.7324579169392005, 1, 1], 
       Hue[0.7324579169392005, 1, 1], 
       Hue[0.6938488004081815, 1, 1], 
       Hue[0.6938488004081815, 1, 1], 
       Hue[0.39015314796355327`, 1, 1], 
       Hue[0.39015314796355327`, 1, 1], 
       Hue[0.47524975557936294`, 1, 1], 
       Hue[0.47524975557936294`, 1, 1], 
       Hue[0.23061422533859277`, 1, 1], 
       Hue[0.23061422533859277`, 1, 1], 
       Hue[0.18759108425177645`, 1, 1], 
       Hue[0.18759108425177645`, 1, 1], 
       Hue[0.10579347430510228`, 1, 1], 
       Hue[0.10579347430510228`, 1, 1], 
       Hue[0.9460988040503769, 1, 1], 
       Hue[0.9460988040503769, 1, 1], 
       Hue[0.883853234242489, 1, 1], 
       Hue[0.883853234242489, 1, 1], 
       Hue[0.7416327814295387, 1, 1], 
       Hue[0.7416327814295387, 1, 1], 
       Hue[0.7749364531338783, 1, 1], 
       Hue[0.7749364531338783, 1, 1], 
       Hue[0.6422587050581101, 1, 1], 
       Hue[0.6422587050581101, 1, 1], 
       Hue[0.5160848082862535, 1, 1], 
       Hue[0.5160848082862535, 1, 1], 
       Hue[0.41998315715370105`, 1, 1], 
       Hue[0.41998315715370105`, 1, 1], 
       Hue[0.3677902653730239, 1, 1], 
       Hue[0.3677902653730239, 1, 1], 
       Hue[0.2596215547704807, 1, 1], 
       Hue[0.2596215547704807, 1, 1], 
       Hue[0.3028772709352434, 1, 1], 
       Hue[0.3028772709352434, 1, 1], 
       Hue[0.041896744764891025`, 1, 1], 
       Hue[0.041896744764891025`, 1, 1], 
       Hue[0.8588600627762908, 1, 1], 
       Hue[0.8588600627762908, 1, 1], 
       Hue[0.7343447439619594, 1, 1], 
       Hue[0.7343447439619594, 1, 1], 
       Hue[0.6365872375656859, 1, 1], 
       Hue[0.6365872375656859, 1, 1], 
       Hue[0.47100754517141513`, 1, 1], 
       Hue[0.47100754517141513`, 1, 1], 
       Hue[0.2813280179514035, 1, 1], 
       Hue[0.2813280179514035, 1, 1], 
       Hue[0.9735888980687284, 1, 1], 
       Hue[0.9735888980687284, 1, 1], 
       Hue[0.5552827557441284, 1, 1], 
       Hue[0.5552827557441284, 1, 1], 
       Hue[0.9381015599604834, 1, 1], 
       Hue[0.9381015599604834, 1, 1], 
       Hue[0.6549908474106629, 1, 1], 
       Hue[0.6549908474106629, 1, 1], 
       Hue[0.6518281404891956, 1, 1], 
       Hue[0.6518281404891956, 1, 1], 
       Hue[0.5400340083574924, 1, 1], 
       Hue[0.5400340083574924, 1, 1], 
       Hue[0.506450929345753, 1, 1], 
       Hue[0.506450929345753, 1, 1], 
       Hue[0.5285733574892757, 1, 1], 
       Hue[0.5285733574892757, 1, 1], 
       Hue[0.5921813181707918, 1, 1], 
       Hue[0.5921813181707918, 1, 1], 
       Hue[0.9333562836332183, 1, 1], 
       Hue[0.9333562836332183, 1, 1], 
       Hue[0.8853458163957009, 1, 1], 
       Hue[0.8853458163957009, 1, 1], 
       Hue[0.853662012170136, 1, 1], 
       Hue[0.853662012170136, 1, 1], 
       Hue[0.8968065708923977, 1, 1], 
       Hue[0.8968065708923977, 1, 1], 
       Hue[0.9826277227569763, 1, 1], 
       Hue[0.9826277227569763, 1, 1], 
       Hue[0.05395650772204826, 1, 1], 
       Hue[0.05395650772204826, 1, 1], 
       Hue[0.1370908908529463, 1, 1], 
       Hue[0.1370908908529463, 1, 1], 
       Hue[0.18163710723817444`, 1, 1], 
       Hue[0.18163710723817444`, 1, 1], 
       Hue[0.24445818798964217`, 1, 1], 
       Hue[0.24445818798964217`, 1, 1], 
       Hue[0.30857439043837853`, 1, 1], 
       Hue[0.30857439043837853`, 1, 1], 
       Hue[0.3606133290881668, 1, 1], 
       Hue[0.3606133290881668, 1, 1], 
       Hue[0.420044573230797, 1, 1], 
       Hue[0.420044573230797, 1, 1], 
       Hue[0.48419173721395464`, 1, 1], 
       Hue[0.48419173721395464`, 1, 1], 
       Hue[0.5361081510744629, 1, 1], 
       Hue[0.5361081510744629, 1, 1], 
       Hue[0.5980659943808304, 1, 1], 
       Hue[0.5980659943808304, 1, 1], 
       Hue[0.6600463844653425, 1, 1], 
       Hue[0.6600463844653425, 1, 1], 
       Hue[0.7235566724925602, 1, 1], 
       Hue[0.7235566724925602, 1, 1], 
       Hue[0.7878680606789363, 1, 1], 
       Hue[0.7878680606789363, 1, 1], 
       Hue[0.8513131084760756, 1, 1], 
       Hue[0.8513131084760756, 1, 1], 
       Hue[0.9141237531356108, 1, 1], 
       Hue[0.9141237531356108, 1, 1], 
       Hue[0.9796550132334485, 1, 1], 
       Hue[0.9796550132334485, 1, 1], 
       Hue[0.04554279610193454, 1, 1], 
       Hue[0.04554279610193454, 1, 1], 
       Hue[0.11187390374902148`, 1, 1], 
       Hue[0.11187390374902148`, 1, 1], 
       Hue[0.17762159787379167`, 1, 1], 
       Hue[0.17762159787379167`, 1, 1], 
       Hue[0.24427137933553897`, 1, 1], 
       Hue[0.24427137933553897`, 1, 1], 
       Hue[0.3117162635752856, 1, 1], 
       Hue[0.3117162635752856, 1, 1], 
       Hue[0.37969314417642314`, 1, 1], 
       Hue[0.37969314417642314`, 1, 1], 
       Hue[0.44815967961866965`, 1, 1], 
       Hue[0.44815967961866965`, 1, 1], 
       Hue[0.5171221163629012, 1, 1], 
       Hue[0.5171221163629012, 1, 1], 
       Hue[0.5862931145848354, 1, 1], 
       Hue[0.5862931145848354, 1, 1], 
       Hue[0.6560422323667225, 1, 1], 
       Hue[0.6560422323667225, 1, 1], 
       Hue[0.7265285874458343, 1, 1], 
       Hue[0.7265285874458343, 1, 1], 
       Hue[0.797530226193851, 1, 1], 
       Hue[0.797530226193851, 1, 1], 
       Hue[0.8690849455760121, 1, 1], 
       Hue[0.8690849455760121, 1, 1], 
       Hue[0.9409585144911263, 1, 1], 
       Hue[0.9409585144911263, 1, 1], 
       Hue[0.013394114338599833`, 1, 1], 
       Hue[0.013394114338599833`, 1, 1], 
       Hue[0.08635321152175987, 1, 1], 
       Hue[0.08635321152175987, 1, 1], 
       Hue[0.15990432497714052`, 1, 1], 
       Hue[0.15990432497714052`, 1, 1], 
       Hue[0.2339381258426344, 1, 1], 
       Hue[0.2339381258426344, 1, 1], 
       Hue[0.30843325664141336`, 1, 1], 
       Hue[0.30843325664141336`, 1, 1], 
       Hue[0.38345074264343776`, 1, 1], 
       Hue[0.38345074264343776`, 1, 1], 
       Hue[0.4589748378028819, 1, 1], 
       Hue[0.4589748378028819, 1, 1], 
       Hue[0.5350070831107332, 1, 1], 
       Hue[0.5350070831107332, 1, 1], 
       Hue[0.6115514859638331, 1, 1], 
       Hue[0.6115514859638331, 1, 1], 
       Hue[0.688588607439024, 1, 1], 
       Hue[0.688588607439024, 1, 1], 
       Hue[0.7660936824814056, 1, 1], 
       Hue[0.7660936824814056, 1, 1], 
       Hue[0.8440844237695915, 1, 1], 
       Hue[0.8440844237695915, 1, 1], 
       Hue[0.9225589053693233, 1, 1], 
       Hue[0.9225589053693233, 1, 1], 
       Hue[0.0015271511254317436`, 1, 1], 
       Hue[0.0015271511254317436`, 1, 1], 
       Hue[0.08096308242320557, 1, 1], 
       Hue[0.08096308242320557, 1, 1], 
       Hue[0.16084335970662272`, 1, 1], 
       Hue[0.16084335970662272`, 1, 1], 
       Hue[0.241147292875211, 1, 1], 
       Hue[0.241147292875211, 1, 1], 
       Hue[0.32189121078452165`, 1, 1], 
       Hue[0.32189121078452165`, 1, 1], 
       Hue[0.40307511451608796`, 1, 1], 
       Hue[0.40307511451608796`, 1, 1], 
       Hue[0.48468106235468705`, 1, 1], 
       Hue[0.48468106235468705`, 1, 1], 
       Hue[0.5666666550014439, 1, 1], 
       Hue[0.5666666550014439, 1, 1], 
       Hue[0.648984876342054, 1, 1], 
       Hue[0.648984876342054, 1, 1], 
       Hue[0.7316139984139767, 1, 1], 
       Hue[0.7316139984139767, 1, 1], 
       Hue[0.8145654914194567, 1, 1], 
       Hue[0.8145654914194567, 1, 1], 
       Hue[0.8978277043733119, 1, 1], 
       Hue[0.8978277043733119, 1, 1], 
       Hue[0.9813450138645246, 1, 1], 
       Hue[0.9813450138645246, 1, 1], 
       Hue[0.06501907948890219, 1, 1], 
       Hue[0.06501907948890219, 1, 1], 
       Hue[0.14876757678712846`, 1, 1], 
       Hue[0.14876757678712846`, 1, 1], 
       Hue[0.23256003250733165`, 1, 1], 
       Hue[0.23256003250733165`, 1, 1], 
       Hue[0.31639830470966585`, 1, 1], 
       Hue[0.31639830470966585`, 1, 1], 
       Hue[0.40023530109883876`, 1, 1], 
       Hue[0.40023530109883876`, 1, 1], 
       Hue[0.48393024058144063`, 1, 1], 
       Hue[0.48393024058144063`, 1, 1], 
       Hue[0.5672876532384993, 1, 1], 
       Hue[0.5672876532384993, 1, 1], 
       Hue[0.6501592758058539, 1, 1], 
       Hue[0.6501592758058539, 1, 1], 
       Hue[0.7324987736444111, 1, 1], 
       Hue[0.7324987736444111, 1, 1], 
       Hue[0.8142983145499002, 1, 1], 
       Hue[0.8142983145499002, 1, 1], 
       Hue[0.8954590993731899, 1, 1], 
       Hue[0.8954590993731899, 1, 1], 
       Hue[0.975722643047566, 1, 1], 
       Hue[0.975722643047566, 1, 1], 
       Hue[0.05474766844007228, 1, 1], 
       Hue[0.05474766844007228, 1, 1], 
       Hue[0.13227308453669961`, 1, 1], 
       Hue[0.13227308453669961`, 1, 1], 
       Hue[0.20821636750529138`, 1, 1], 
       Hue[0.20821636750529138`, 1, 1], 
       Hue[0.2826194654284094, 1, 1], 
       Hue[0.2826194654284094, 1, 1], 
       Hue[0.3554752741691703, 1, 1], 
       Hue[0.3554752741691703, 1, 1], 
       Hue[0.4265889065828907, 1, 1], 
       Hue[0.4265889065828907, 1, 1], 
       Hue[0.4955650664400778, 1, 1], 
       Hue[0.4955650664400778, 1, 1], 
       Hue[0.5619584530314572, 1, 1], 
       Hue[0.5619584530314572, 1, 1], 
       Hue[0.6254635887802759, 1, 1], 
       Hue[0.6254635887802759, 1, 1], 
       Hue[0.6860331182124972, 1, 1], 
       Hue[0.6860331182124972, 1, 1], 
       Hue[0.7438757636849702, 1, 1], 
       Hue[0.7438757636849702, 1, 1], 
       Hue[0.799376342278605, 1, 1], 
       Hue[0.799376342278605, 1, 1], 
       Hue[0.8530037452326122, 1, 1], 
       Hue[0.8530037452326122, 1, 1], 
       Hue[0.9052386573064211, 1, 1], 
       Hue[0.9052386573064211, 1, 1], 
       Hue[0.9565330054338083, 1, 1], 
       Hue[0.9565330054338083, 1, 1], 
       Hue[0.007289019921117269, 1, 1], 
       Hue[0.007289019921117269, 1, 1], 
       Hue[0.05785436886123326, 1, 1], 
       Hue[0.05785436886123326, 1, 1], 
       Hue[0.10852361694626046`, 1, 1], 
       Hue[0.10852361694626046`, 1, 1], 
       Hue[0.1595424939905707, 1, 1], 
       Hue[0.1595424939905707, 1, 1], 
       Hue[0.21111490902177088`, 1, 1], 
       Hue[0.21111490902177088`, 1, 1], 
       Hue[0.2634076300324872, 1, 1], 
       Hue[0.2634076300324872, 1, 1], 
       Hue[0.31655628975593003`, 1, 1], 
       Hue[0.31655628975593003`, 1, 1], 
       Hue[0.37066907391576337`, 1, 1], 
       Hue[0.37066907391576337`, 1, 1], 
       Hue[0.42583171930078956`, 1, 1], 
       Hue[0.42583171930078956`, 1, 1], 
       Hue[0.4821112905542982, 1, 1], 
       Hue[0.4821112905542982, 1, 1], 
       Hue[0.5395581595296779, 1, 1], 
       Hue[0.5395581595296779, 1, 1], 
       Hue[0.5982089184764874, 1, 1], 
       Hue[0.5982089184764874, 1, 1], 
       Hue[0.6580902546204306, 1, 1], 
       Hue[0.6580902546204306, 1, 1], 
       Hue[0.7192184186201758, 1, 1], 
       Hue[0.7192184186201758, 1, 1], 
       Hue[0.7816029392773315, 1, 1], 
       Hue[0.7816029392773315, 1, 1], 
       Hue[0.845246320651291, 1, 1], 
       Hue[0.845246320651291, 1, 1], 
       Hue[0.9101471280449227, 1, 1], 
       Hue[0.9101471280449227, 1, 1], 
       Hue[0.9762994778057256, 1, 1], 
       Hue[0.9762994778057256, 1, 1], 
       Hue[0.043693647064499526`, 1, 1], 
       Hue[0.043693647064499526`, 1, 1], 
       Hue[0.1123192138725254, 1, 1], 
       Hue[0.1123192138725254, 1, 1], 
       Hue[0.18216349795160328`, 1, 1], 
       Hue[0.18216349795160328`, 1, 1], 
       Hue[0.2532094583491348, 1, 1], 
       Hue[0.2532094583491348, 1, 1], 
       Hue[0.32544339799515104`, 1, 1], 
       Hue[0.32544339799515104`, 1, 1], 
       Hue[0.39884894807087073`, 1, 1], 
       Hue[0.39884894807087073`, 1, 1], 
       Hue[0.4734080173466877, 1, 1], 
       Hue[0.4734080173466877, 1, 1], 
       Hue[0.5491064682724095, 1, 1], 
       Hue[0.5491064682724095, 1, 1], 
       Hue[0.6259260435306573, 1, 1], 
       Hue[0.6259260435306573, 1, 1], 
       Hue[0.7038469620065598, 1, 1], 
       Hue[0.7038469620065598, 1, 1], 
       Hue[0.7828539447311839, 1, 1], 
       Hue[0.7828539447311839, 1, 1], 
       Hue[0.86293510134423, 1, 1], 
       Hue[0.86293510134423, 1, 1], 
       Hue[0.9440694759822695, 1, 1], 
       Hue[0.9440694759822695, 1, 1], 
       Hue[0.026237977530103335`, 1, 1], 
       Hue[0.026237977530103335`, 1, 1], 
       Hue[0.10943912867403406`, 1, 1], 
       Hue[0.10943912867403406`, 1, 1], 
       Hue[0.1936492364678344, 1, 1], 
       Hue[0.1936492364678344, 1, 1], 
       Hue[0.2788390158538952, 1, 1], 
       Hue[0.2788390158538952, 1, 1], 
       Hue[0.36501718255201326`, 1, 1], 
       Hue[0.36501718255201326`, 1, 1], 
       Hue[0.4521779601579034, 1, 1], 
       Hue[0.4521779601579034, 1, 1], 
       Hue[0.5402709397872767, 1, 1], 
       Hue[0.5402709397872767, 1, 1], 
       Hue[0.6292987165623611, 1, 1], 
       Hue[0.6292987165623611, 1, 1], 
       Hue[0.7192852763459747, 1, 1], 
       Hue[0.7192852763459747, 1, 1], 
       Hue[0.8101807715827263, 1, 1], 
       Hue[0.8101807715827263, 1, 1], 
       Hue[0.9019724559267499, 1, 1], 
       Hue[0.9019724559267499, 1, 1], 
       Hue[0.99470033906565, 1, 1], 
       Hue[0.99470033906565, 1, 1], 
       Hue[0.08832783678624961, 1, 1], 
       Hue[0.08832783678624961, 1, 1], 
       Hue[0.18277987722455066`, 1, 1], 
       Hue[0.18277987722455066`, 1, 1], 
       Hue[0.2780640171066228, 1, 1], 
       Hue[0.2780640171066228, 1, 1], 
       Hue[0.3742762888141277, 1, 1], 
       Hue[0.3742762888141277, 1, 1], 
       Hue[0.4714059652914306, 1, 1], 
       Hue[0.4714059652914306, 1, 1], 
       Hue[0.5692514338009139, 1, 1], 
       Hue[0.5692514338009139, 1, 1], 
       Hue[0.6677981386855287, 1, 1], 
       Hue[0.6677981386855287, 1, 1], 
       Hue[0.7673407367346702, 1, 1], 
       Hue[0.7673407367346702, 1, 1], 
       Hue[0.8678504620247419, 1, 1], 
       Hue[0.8678504620247419, 1, 1], 
       Hue[0.9686973038820387, 1, 1], 
       Hue[0.9686973038820387, 1, 1], 
       Hue[0.06963060970896097, 1, 1], 
       Hue[0.06963060970896097, 1, 1], 
       Hue[0.17180207357002875`, 1, 1], 
       Hue[0.17180207357002875`, 1, 1], 
       Hue[0.27636830103356114`, 1, 1], 
       Hue[0.27636830103356114`, 1, 1], 
       Hue[0.38209981443647867`, 1, 1], 
       Hue[0.38209981443647867`, 1, 1], 
       Hue[0.48684916138316015`, 1, 1], 
       Hue[0.48684916138316015`, 1, 1], 
       Hue[0.591638674225299, 1, 1], 
       Hue[0.591638674225299, 1, 1], 
       Hue[0.6996440399500621, 1, 1], 
       Hue[0.6996440399500621, 1, 1], 
       Hue[0.8106408769056764, 1, 1], 
       Hue[0.8106408769056764, 1, 1], 
       Hue[0.9203208846246161, 1, 1], 
       Hue[0.9203208846246161, 1, 1], 
       Hue[0.02405236895185515, 1, 1], 
       Hue[0.02405236895185515, 1, 1], 
       Hue[0.1228424398166391, 1, 1], 
       Hue[0.1228424398166391, 1, 1], 
       Hue[0.22619536848471866`, 1, 1], 
       Hue[0.22619536848471866`, 1, 1], 
       Hue[0.3370375970414235, 1, 1], 
       Hue[0.3370375970414235, 1, 1], 
       Hue[0.4399614845447659, 1, 1], 
       Hue[0.4399614845447659, 1, 1], 
       Hue[0.5268701196885841, 1, 1], 
       Hue[0.5268701196885841, 1, 1], 
       Hue[0.6371482431217275, 1, 1], 
       Hue[0.6371482431217275, 1, 1], 
       Hue[0.7836147903328474, 1, 1], 
       Hue[0.7836147903328474, 1, 1], 
       Hue[0.9442860013131829, 1, 1], 
       Hue[0.9442860013131829, 1, 1], 
       Hue[0.09297398599726875, 1, 1], 
       Hue[0.09297398599726875, 1, 1], 
       Hue[0.17887958096169843`, 1, 1], 
       Hue[0.17887958096169843`, 1, 1], 
       Hue[0.19638153085605062`, 1, 1], 
       Hue[0.19638153085605062`, 1, 1], 
       Hue[0.30819964433545805`, 1, 1], 
       Hue[0.30819964433545805`, 1, 1], 
       Hue[0.4887367267379165, 1, 1], 
       Hue[0.4887367267379165, 1, 1], 
       Hue[0.7100622885787811, 1, 1], 
       Hue[0.7100622885787811, 1, 1], 
       Hue[0.9616419546043693, 1, 1], 
       Hue[0.9616419546043693, 1, 1], 
       Hue[0.23801211969541905`, 1, 1], 
       Hue[0.23801211969541905`, 1, 1], 
       Hue[0.5347134004266552, 1, 1], 
       Hue[0.5347134004266552, 1, 1], 
       Hue[0.851720999705813, 1, 1], 
       Hue[0.851720999705813, 1, 1], 
       Hue[0.19772709565797011`, 1, 1], 
       Hue[0.19772709565797011`, 1, 1], 
       Hue[0.5542643360085939, 1, 1], 
       Hue[0.5542643360085939, 1, 1], 
       Hue[0.8726282053207132, 1, 1], 
       Hue[0.8726282053207132, 1, 1], 
       Hue[0.1490042529253488, 1, 1], 
       Hue[0.1490042529253488, 1, 1], 
       Hue[0.37880560395471286`, 1, 1], 
       Hue[0.37880560395471286`, 1, 1], 
       Hue[0.5180102868178059, 1, 1], 
       Hue[0.5180102868178059, 1, 1], 
       Hue[0.6661968199894867, 1, 1], 
       Hue[0.6661968199894867, 1, 1], 
       Hue[0.8912713919398864, 1, 1], 
       Hue[0.8912713919398864, 1, 1], 
       Hue[0.14372955017319863`, 1, 1], 
       Hue[0.14372955017319863`, 1, 1], 
       Hue[0.40895675323907044`, 1, 1], 
       Hue[0.40895675323907044`, 1, 1], 
       Hue[0.687498852931313, 1, 1], 
       Hue[0.687498852931313, 1, 1], 
       Hue[0.9983927521379041, 1, 1], 
       Hue[0.9983927521379041, 1, 1], 
       Hue[0.41402290612424353`, 1, 1], 
       Hue[0.41402290612424353`, 1, 1], 
       Hue[0.8107856520326657, 1, 1], 
       Hue[0.8107856520326657, 1, 1], 
       Hue[0.11575640376966859`, 1, 1], 
       Hue[0.11575640376966859`, 1, 1], 
       Hue[0.39601159431580035`, 1, 1], 
       Hue[0.39601159431580035`, 1, 1], 
       Hue[0.6681929871460615, 1, 1], 
       Hue[0.6681929871460615, 1, 1], 
       Hue[0.9369112472018684, 1, 1], 
       Hue[0.9369112472018684, 1, 1], 
       Hue[0.20387105287694343`, 1, 1], 
       Hue[0.20387105287694343`, 1, 1], 
       Hue[0.46982427789568754`, 1, 1], 
       Hue[0.46982427789568754`, 1, 1], 
       Hue[0.7351428404407656, 1, 1], 
       Hue[0.7351428404407656, 1, 1], 
       Hue[0.000045043010442031465`, 1, 1], 
       Hue[0.000045043010442031465`, 1, 1], 
       Hue[0.26472928293010783`, 1, 1], 
       Hue[0.26472928293010783`, 1, 1], 
       Hue[0.529502741806376, 1, 1], 
       Hue[0.529502741806376, 1, 1], 
       Hue[0.7949436009945829, 1, 1], 
       Hue[0.7949436009945829, 1, 1], 
       Hue[0.06210321930714255, 1, 1], 
       Hue[0.06210321930714255, 1, 1], 
       Hue[0.33266837165463214`, 1, 1], 
       Hue[0.33266837165463214`, 1, 1], 
       Hue[0.6088044722098869, 1, 1], 
       Hue[0.6088044722098869, 1, 1], 
       Hue[0.892140756761359, 1, 1], 
       Hue[0.892140756761359, 1, 1], 
       Hue[0.18175714747005245`, 1, 1], 
       Hue[0.18175714747005245`, 1, 1], 
       Hue[0.47326275941150026`, 1, 1], 
       Hue[0.47326275941150026`, 1, 1], 
       Hue[0.7616040472763749, 1, 1], 
       Hue[0.7616040472763749, 1, 1], 
       Hue[0.044437360962688716`, 1, 1], 
       Hue[0.044437360962688716`, 1, 1], 
       Hue[0.32202075940832503`, 1, 1], 
       Hue[0.32202075940832503`, 1, 1], 
       Hue[0.5955234481590492, 1, 1], 
       Hue[0.5955234481590492, 1, 1], 
       Hue[0.8660326380659066, 1, 1], 
       Hue[0.8660326380659066, 1, 1], 
       Hue[0.1343067124487573, 1, 1]}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->{True, None},
    AxesOrigin->{0, 0},
    Epilog->
     LineBox[{{1, 1.263148251317129*^-6}, {2, 1.0988000515666537`*^-6}, {
       3, 1.0817263863807828`*^-6}, {4, 1.0119678170251821`*^-6}, {
       5, 1.4268495912945795`*^-6}, {6, 1.2158198379080188`*^-6}, {
       7, 8.503215924495397*^-7}, {8, 7.119171371667034*^-7}, {
       9, 1.2699117946955453`*^-6}, {10, 1.1097360510145725`*^-6}, {
       11, 9.686198262614006*^-7}, {12, 1.0440379619942911`*^-6}, {
       13, 5.361746432048057*^-7}, {14, 2.912173475928828*^-7}, {
       15, 7.050495895397025*^-7}, {16, 3.5814730863871657`*^-7}, {
       17, 5.396703710757984*^-7}, {18, 7.664627341804243*^-7}, {
       19, 4.0801767795511484`*^-7}, {20, 6.991707873749146*^-7}, {
       21, 5.263730152481841*^-7}, {22, 9.70780317961668*^-7}, {
       23, 8.644035522275964*^-7}, {24, 6.937235396962769*^-7}, {
       25, 1.3516433050748546`*^-6}, {26, 1.2768069719849307`*^-6}, {
       27, 1.0238978125815495`*^-6}, {28, 8.898613927937892*^-7}, {
       29, 5.341727732913914*^-7}, {30, 6.209342000461613*^-7}, {
       31, 9.009864392952196*^-7}, {32, 5.475783116943175*^-7}, {
       33, 4.523964194472669*^-7}, {34, 7.779450560138102*^-7}, {
       35, 5.653741555509746*^-7}, {36, 6.678906401350125*^-7}, {
       37, 3.048580799157426*^-7}, {38, 1.7862246494954367`*^-7}, {
       39, 3.1412611311898113`*^-7}, {40, 4.728133286029297*^-7}, {
       41, 1.1059742026166754`*^-6}, {42, 9.711285501908659*^-7}, {
       43, 6.804852580820673*^-7}, {44, 4.0418701019270225`*^-7}, {
       45, 8.061249137868119*^-7}, {46, 1.6011490721009806`*^-7}, {
       47, 1.1980869562401297`*^-6}, {48, 1.7954501904300843`*^-6}, {
       49, 1.764768036572435*^-6}, {50, 1.741554027841161*^-6}, {
       51, 2.547808912351531*^-6}, {52, 3.967349067635245*^-6}, {
       53, 4.675277820415082*^-6}, {54, 6.705248721334987*^-6}, {
       55, 9.1105241168038*^-6}, {56, 0.000012079194735455833`}, {
       57, 0.000016305790019528445`}, {58, 0.000020496125247843785`}, {
       59, 0.000026719599565117968`}, {60, 0.00003372651932261041}, {
       61, 0.00004303338915770956}, {62, 0.00005491463309140772}, {
       63, 0.00007083746466335644}, {64, 0.00008993613583068241}, {
       65, 0.00011330692805068455`}, {66, 0.00014343820280123958`}, {
       67, 0.00017999827480562385`}, {68, 0.00022746334371359815`}, {
       69, 0.000284740801932554}, {70, 0.0003560084751828962}, {
       71, 0.00044258016571382285`}, {72, 0.00054888069144532}, {
       73, 0.0006797480336058403}, {74, 0.0008386814466846738}, {
       75, 0.0010318864144614696`}, {76, 0.0012624659024528951`}, {
       77, 0.0015406394425161105`}, {78, 0.0018749729185660578`}, {
       79, 0.0022757260584992576`}, {80, 0.0027541709743578217`}, {
       81, 0.0033193790856586147`}, {82, 0.0039877325408871804`}, {
       83, 0.0047761189427326925`}, {84, 0.005703182499766771}, {
       85, 0.006788783044794935}, {86, 0.00805369122293874}, {
       87, 0.009524395031424866}, {88, 0.01122698434952529}, {
       89, 0.013192045776814772`}, {90, 0.015454561606664587`}, {
       91, 0.018046737583053998`}, {92, 0.021006611899095824`}, {
       93, 0.024372539765145262`}, {94, 0.028189047597036564`}, {
       95, 0.03250456613877092}, {96, 0.037367231421487866`}, {
       97, 0.0428209875868495}, {98, 0.048912629874264715`}, {
       99, 0.055696952208377455`}, {100, 0.06323502455006375}, {
       101, 0.07158860340452554}, {102, 0.08080870784750611}, {
       103, 0.0909318631190164}, {104, 0.10200305896600698`}, {
       105, 0.1140904859340883}, {106, 0.12728168839326717`}, {
       107, 0.141643848728605}, {108, 0.15719544874447805`}, {
       109, 0.1739270345635291}, {110, 0.1918782184002109}, {
       111, 0.21117046433785905`}, {112, 0.23196424641881064`}, {
       113, 0.25431892722897964`}, {114, 0.27814526654604743`}, {
       115, 0.30331112666387466`}, {116, 0.32986095364484397`}, {
       117, 0.35809505922371804`}, {118, 0.3883717365143623}, {
       119, 0.4207607713769854}, {120, 0.4548740029477722}, {
       121, 0.490110334868377}, {122, 0.5261493338945208}, {
       123, 0.5632673404284136}, {124, 0.6021412600336473}, {
       125, 0.6431759683372389}, {126, 0.6857803321079176}, {
       127, 0.728075313886562}, {128, 0.7672047976373915}, {
       129, 0.800014715616252}, {130, 0.8237640602600242}, {
       131, 0.8365869153357057}, {132, 0.8376782875817699}, {
       133, 0.8272295868527458}, {134, 0.8062211834197465}, {
       135, 0.7761660585836947}, {136, 0.7388520666179677}, {
       137, 0.6961367814201835}, {138, 0.6497960193904806}, {
       139, 0.6014253585615972}, {140, 0.5523908288486171}, {
       141, 0.5038091195575622}, {142, 0.4565611420950155}, {
       143, 0.4113057663216284}, {144, 0.36851019085678266`}, {
       145, 0.328482001850657}, {146, 0.29139632688865214`}, {
       147, 0.25732124094024783`}, {148, 0.2262453381993219}, {
       149, 0.1980946081755368}, {150, 0.17275083936779465`}, {
       151, 0.15006261914988958`}, {152, 0.12986009072911917`}, {
       153, 0.11196120849466382`}, {154, 0.09617775359400356}, {
       155, 0.08232318253314833}, {156, 0.07021616422173953}, {
       157, 0.0596802037105046}, {158, 0.05054994265191247}, {
       159, 0.042669791788711206`}, {160, 0.03589490528875932}, {
       161, 0.030093437210281387`}, {162, 0.025145102355836807`}, {
       163, 0.020940067802506627`}, {164, 0.017379509419725825`}, {
       165, 0.014376847488959774`}, {166, 0.011853757125176678`}, {
       167, 0.009740389731214268}, {168, 0.007977573433021856}, {
       169, 0.006512956797516867}, {170, 0.005299299322158557}, {
       171, 0.004297337987412937}, {172, 0.0034740358857803864`}, {
       173, 0.002799294019160139}, {174, 0.0022479382273943526`}, {
       175, 0.0017995581984829431`}, {176, 0.0014359787938274453`}, {
       177, 0.001141760038936783}, {178, 0.0009047212899598665}, {
       179, 0.0007147979075875034}, {180, 0.00056305882367613}, {
       181, 0.0004418018253960445}, {182, 0.0003452920105716575}, {
       183, 0.000269206229752395}, {184, 0.00020936042558596652`}, {
       185, 0.00016198302063449976`}, {186, 0.00012465461503090568`}, {
       187, 0.00009595256142123079}, {188, 0.00007417392975360292}, {
       189, 0.00005711911105166829}, {190, 0.00004335333802676895}, {
       191, 0.00003265278475943498}, {192, 0.00002480302838990986}, {
       193, 0.00001888902636019418}, {194, 0.000014069409021099529`}, {
       195, 0.000010140238061285121`}, {196, 7.223437696523649*^-6}, {
       197, 5.331976073950158*^-6}, {198, 4.116946903917992*^-6}, {
       199, 3.1000435757834277`*^-6}, {200, 2.212493793781606*^-6}, {
       201, 1.7640415257678798`*^-6}, {202, 1.6774412610150379`*^-6}, {
       203, 1.4863064073338804`*^-6}, {204, 1.073783501540085*^-6}, {
       205, 6.067001431692634*^-7}, {206, 2.8887998994694905`*^-7}, {
       207, 2.5692678868152457`*^-7}, {208, 3.6375127468225415`*^-7}, {
       209, 4.586372637019976*^-7}, {210, 4.920130033778911*^-7}, {
       211, 4.5768653173881383`*^-7}, {212, 3.833207615838428*^-7}, {
       213, 2.985179898232454*^-7}, {214, 2.187705880180826*^-7}, {
       215, 1.6257060200826846`*^-7}, {216, 1.5012810121569064`*^-7}, {
       217, 1.551910978414857*^-7}, {218, 1.393170641899645*^-7}, {
       219, 9.902569698696222*^-8}, {220, 7.324759957669555*^-8}, {
       221, 1.0613153206144688`*^-7}, {222, 1.4904763485985006`*^-7}, {
       223, 1.6954928874328623`*^-7}, {224, 1.6091045023207582`*^-7}, {
       225, 1.270671391642489*^-7}, {226, 7.982965316575254*^-8}, {
       227, 5.301977242089482*^-8}, {228, 8.8312965017922*^-8}, {
       229, 1.3920213858281637`*^-7}, {230, 1.8171512090898223`*^-7}, {
       231, 2.0987282765413233`*^-7}, {232, 2.2238103460634007`*^-7}, {
       233, 2.204307549653656*^-7}, {234, 2.0675662642762887`*^-7}, {
       235, 1.8488282323233655`*^-7}, {236, 1.5847104530444418`*^-7}, {
       237, 1.3080985986707924`*^-7}, {238, 1.0448319462058098`*^-7}, {
       239, 8.122465889264352*^-8}, {240, 6.194217610725269*^-8}, {
       241, 4.687140568053066*^-8}, {242, 3.580397842895375*^-8}, {
       243, 2.830297650785658*^-8}, {244, 2.3799015296220542`*^-8}, {
       245, 2.15533365868183*^-8}, {246, 2.0687898647831395`*^-8}, {
       247, 2.039239155408677*^-8}, {248, 2.009900121546737*^-8}, {
       249, 1.949959901940705*^-8}, {250, 1.8482672936234614`*^-8}, {
       251, 1.7064321965583427`*^-8}}],
    Frame->True,
    FrameTicks->{{{1., 
        FormBox[
         RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
        FormBox[
         RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
        FormBox[
         RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
        FormBox[
         RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
        FormBox["0", TraditionalForm]}, {157.25, 
        FormBox["2.5`", TraditionalForm]}, {188.5, 
        FormBox["5", TraditionalForm]}, {219.75, 
        FormBox["7.5`", TraditionalForm]}, {251., 
        FormBox["10", TraditionalForm]}}, Automatic, None, None},
    PlotRange->{-0.1, 1.1},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], Automatic}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF13dYFHcaB/BJsJ7GS0CiMZZVERsgEQU7XwXBBtI7y1B2gaWdothOGUEE
FY1BgxJE15LYQAVRRKNMjGKw9wNjWdRALCDBeEbU43LHu88998+8/AHPh3e/
853ZmZ39Tf+IJC/Nh4IgvPjz13/+tv00OXarcj256fCRSfQP/FHkUW5feIDc
CQsmfLXazauI/DE6uXh4d71SQu6ORvP1Gz9MNr6+J6IW6YLtexaQe6P7fTt/
E9edZBU8Szq5BjdvIA/A1TfRezO/NW7PAgdunvs6e8k3ZEv8YF2TqF/7HXkI
pozX9tgz0dg3DAtn5ieeWreFbIU/DhxZt9PZuL82UG3vW7LoViHZFj++93LZ
/6lx/gV2TbVuqO5vPP6RCMlbeu+BSznZDrPTTNvL5YfJo3D/0eCCvzbuI49G
sl2jy/tc4/7Zw9GuPOSnTsa5AypWnrn2/J1xPgbVeVZVedk55LHYmHHx9v7n
+eRxuP5+xotOc3eQx0NtE3pp3NpN5AmwSFVfchj6NXkidu8rNsFooyeh9LZP
QWofPdkRT6weH6rL3EgGtg12nlmzm95/CX+eL58K8d0emk/GxnEfdl7ahfLS
ZOR0Wzb62me7aT4Fqdpk12f9aX+lKehxu+zmnXbf0twJTqLJyXO+dP4kJ7y+
ZVVw63Pj8TpjeszDarmXMe8Mh5Q4r9616TSfiu8HbfuLx0l6vTQVb84dKj5Q
lURzF2Qk+vyleOhXNHfBrPP2fUKebKe5K5q7L8z/O3Jp7opNQYOXPO1inE+D
yxabXSkuq2k+DTnOXx9z327c/+mY88a8nf1mur6k6cjq+tbEsdF4PmagxiS4
YW0F7a80A0HbJ7bOVx2k+Uzs8DPfVhwk03wmvjm+5NymZ6doPgup5n10o3Or
aD4LF/qPP+JadJ7mbnjYX322/qdrNHfD1KJk81a7apq7Y6FHe8shYQaau+Nv
Tzt++rC8juazUe6Tld565BnNZ2NUmo396mVNNPdAr/qMsulzfm8zPNB7YevR
guUt9HoP+G9t77zozQeO/7XsAY30fdGRCR3aLHhim31qRp+ULm2GJzYs/Vtg
qtknbZY8YVU4JtK0xJzynqjLtnixJ/hzyntB/faFeuH4/pT3gnVNknnqukGU
90J9T9vRb0uGUd4LiZUNcVfW21LeG3u7/rN00LxRlPfG8qz5stXMsZT3xtaQ
OP98K0fKeyPaHc/zBzhR3gdOiy82S76ulPfB4vMLDrrHzqK8D8z3BFZ/6epJ
eR+s/vmP+Xfe+lDeFy/OfPlZx6hAyvui1+oi1+gtasr7Yv6y55OeBUdQ3hfW
sR2CLVs0lPeDZ/WO3zpO1VHeD1d0vkPdg5Io7wedfX7kie/mUt4PVn1DP93V
I4Xy/ihM6pwc9tFiyvtje9qaoRXfLaO8P6om93++0ySN8v7QrDS76zk4g/IB
iM/7l9vI01mUD8DrlmN7xZvZlA9A+uuIe4ml6ykfgErXD0ZP8NpA+UBEt6a3
zgvKpXwgDicOz4i5k0f5QGTZ296NOl9A+UDE/bhiRH237ZQPQtVnj6IvJe2k
fBAK+nTIHKn6jvJByP6hc4Y4cS/lg7BRV2vZfLmQ8sGoNGu++U59iPLB6Bt0
dFHlghLKB8Nq4qnk4mOllA+GKjH39RqnMsqHIEE7NNJ00nHKhyB+U3VjUOFJ
yodg5q+HazqWypQPweOc01ZLd56mfCgyLOZ1mLL3DOVDkdW7IsWksZLyoei/
Z3SPRd9WUT4UpW871m2uvUh5NQYGJ6ZMdrxKeTXO4meXjv2uU16NuX13RExv
vUF59Z/Hs8LGtPkW5cPQ/uj1sXnDqykfhkcZg7qX1t+hfBiynBfUL3W+T/kw
7Pp+iiHj01rKi1ha3CCX2z5qs0qERj/vZrjrL7Q9ER2tMn8/PLK+zaKIbWtO
zIy48SttX8SV8fNtTMyftlkv4tcBV9OeN5BlERPDq7+MD33WZoOIf5WNsf97
LFkIh2lRlOfy3mRVODLrpmY5pFMe4Zh+Xv0B9j+h/nAMLJRPtmw39ofjja9p
3u1k2j99OHKW3TvXNLyO+sPxwPtN3sMbj6k/HOl+7QKrkul4hQh89sL2VaDp
Q+qPQOpLM9/3pQbqj8Chwf36NFXQ+ydGYMktk0Jb67vUH4E4s3HavUdrqD8C
hdnNm5+F/YP6I2B/o7h89nA6X4YI7L2XaTepF51PIRLdvng339v6GvVHor1+
Ql6PqsvUHwkvz0mWh5ddoP5IOP4SHr5p0U/UH4m+ZbVZ48vOUn8kKnou2Fc4
6kfqj0T9vlu26xrp+jVE4n66w0drT9P1LUThyRxrL19Tuv5VUVhxaMvm12eO
Un8UnF1Ks59eP0z9UYi86Lf13Khi6o/CG2v/8p8qi6g/CldtL2VXHTR+XqOg
u3VMjR70eTZEoXLQoFdHmnZQvwZl808M3m+rp34N0jLW9xqwOZ/6NSj47W3Z
0TmbqF+DgH9mTa36me4/kgaLExoaJ5+l+5Neg1tjK48GTaT7l6xB+xNDC8JL
M6lfg+5XMr/KvZ1O/VqUfLPBedpaifq1aBxgObJ41RLq1+KOVcDwef+g+62o
hRzilu7+iO7HkhY5W7Z2qxtI92u9Fpri0K0WebHUr0Vk55VeD1uiqF8L7THv
m3EDw6k/GqannCNTngdTfzQe719Rqz3kR/3RaNWecJ8W5kX90Xi76oePf9zj
Rv3ROLguMyTkIX1/6aMRrPYu6WJN329yNK4tzzQdfGAS9UdjVc20glULx1B/
DCyG5r7vu8CO+mPQWrqrZkeNDfXHoNurNfkegUOoPwbqB0X3PEcMpP4YpFmo
e1mO6kv9MRC+vFNQubsH9cdgrZSgsosypf4YaOuiPzmyriv1x6Jpf/WrZCda
T6hikaY+XrF3Fq03EIuD/o/fvUuj9YgYi2bxTMwUd1qvSLGou7An4rhA6xl9
LLr/8uuq44uftFmOxXGzprCLbo/abIjFi40xPh+F/EzrHx2Kkoaf1S2+0WaV
DoNKbOvrGi/RekiHY/p+DnHPKqlfh0UP8r+IuE7rN0mHGcvC9HMeHKJ+HU43
5FS57KL1vazDOLO51S0j6PnCoINtRWTzgibj80ocbPaP2TKsH623VXEY3HCq
dN88Wm8jDm8q+tknDKT1phiHlqud7Xo30PpUikOS5kON7uQu6o/DI1Oz/dtC
t1J/HBwW/TJ85Y711B+H9Vtrq8f/vpD647HB9CtEey6l/nj0cOs3a/nFNdQf
j28up/1wvoieF8R41Po42Jd6G9fz8Vi+QbqwOoye7/TxUJ1w2nisiZ6/5HgM
e9D55UvLPOqPR+bXIzpfnmB83ktA7YSoHOuntP5WJWDK4eyYuy0rqD8BZvdT
Uhd+TxYTcEAbc/ecE1lKgG+XP4RtlyXqT8DGUa9QP4iOT07A59+uyqt8mUL9
CRhz6oVZ13b0eiER4ku7uXbxtD1VIjIM78bM+3wl9SfC971Fu9xUY38i1gde
6ONvRnkpESm/xey+nDOP+hPh+Mlj9cmABOpPxKxUfWiCZgH1J2Kl+crz5h3S
qD8JQR0P5cbFZFJ/Esrumpxu+avx/U9CnxG9G/wuZlN/EgK8NwxZeZL85/O6
8P8/UHYnxh8z7s64J+PejFWMBzC2YGzJeAjjYYytGNswtmX8BeORjO0Yj2I8
mrE9YwfGYxiPZTyO8XjGExhPZDyJsSNjKFtiLExWtsRYmKJsibHgpGyJseCs
bImxMFXZEmPBRdkSY8FV2RJjYZqyJcbCdGVLjIUZypYYCzOVLTEWZilbYiy4
KVtiLLgrW2IszFa2xFjwUDYYS4xlxoKnssFYYiwzFryUDcYSY5mx4K1sMJYY
y4wFH2WDscRYZiz4KhuMJcYyY8FP2WAsMZYZC/7KBmOJscxYCFA2GEuMZcZC
oLLBWGIsMxaClA3GEmOZsRCsbDCWGMuMhRBlg7HEWGYshCobjCXGMmNBrWww
lhjLjIUwZYOxxFhmLIjKVjEGY5GxxFjPWGZsYCyEK1vFGIxFxhJjPWOZsYGx
EKFsFWMwFhlLjPWMZcYGxkKkslWMwVhkLDHWM5YZGxgLUcpWMQZjkbHEWM9Y
ZmxgLGiUrWIMxiJjibGesczYwFjQKlvFGIxFxhJjPWOZsYGxEK1sFWMwFhlL
jPWMZcYGxkKMslWMwVhkLDHWM5YZGxgLscpWMQZjkbHEWM9YZmxgLOiUrWIM
xiJjibGesczYwFiIU7aKMRiLjCXGesYyYwNjIV7ZKsZgLDKWGOsZy4wNjIUE
ZasYg7HIWGKsZywzNjAWEpWtYgzGImOJsZ6xzNjAWEhStooxGIv/878BUhiC
Vw==
     "], {
      {GrayLevel[0.5], EdgeForm[None], 
       GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ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         "]]]}, {{}, {}, {
        {Hue[0.5134986713962597, 1, 1], LineBox[{1, 2}]}, 
        {Hue[0.429675798262789, 1, 1], LineBox[{2, 3}]}, 
        {Hue[0.36537759142896803`, 1, 1], LineBox[{3, 4}]}, 
        {Hue[0.287489306520422, 1, 1], LineBox[{4, 5}]}, 
        {Hue[0.11714250593167995`, 1, 1], LineBox[{5, 6}]}, 
        {Hue[0.030302165263602618`, 1, 1], LineBox[{6, 7}]}, 
        {Hue[0.0651831562564822, 1, 1], LineBox[{7, 8}]}, 
        {Hue[0.03005619215611696, 1, 1], LineBox[{8, 9}]}, 
        {Hue[0.8086350452323195, 1, 1], LineBox[{9, 10}]}, 
        {Hue[0.5538481087637268, 1, 1], LineBox[{10, 11}]}, 
        {Hue[0.5327463853830385, 1, 1], LineBox[{11, 12}]}, 
        {Hue[0.3282688019815088, 1, 1], LineBox[{12, 13}]}, 
        {Hue[0.2199518739415025, 1, 1], LineBox[{13, 14}]}, 
        {Hue[0.22297225551656205`, 1, 1], LineBox[{14, 15}]}, 
        {Hue[0.08319886681240779, 1, 1], LineBox[{15, 16}]}, 
        {Hue[0.059918982847335045`, 1, 1], LineBox[{16, 17}]}, 
        {Hue[0.9702160847775785, 1, 1], LineBox[{17, 18}]}, 
        {Hue[0.7991004302923844, 1, 1], LineBox[{18, 19}]}, 
        {Hue[0.644117418730759, 1, 1], LineBox[{19, 20}]}, 
        {Hue[0.6356139264194032, 1, 1], LineBox[{20, 21}]}, 
        {Hue[0.5268321040319055, 1, 1], LineBox[{21, 22}]}, 
        {Hue[0.4849149904511286, 1, 1], LineBox[{22, 23}]}, 
        {Hue[0.2951625421267708, 1, 1], LineBox[{23, 24}]}, 
        {Hue[0.17072137554429173`, 1, 1], LineBox[{24, 25}]}, 
        {Hue[0.8425020370839564, 1, 1], LineBox[{25, 26}]}, 
        {Hue[0.5623136607528382, 1, 1], LineBox[{26, 27}]}, 
        {Hue[0.3847582675161996, 1, 1], LineBox[{27, 28}]}, 
        {Hue[0.6570983483750655, 1, 1], LineBox[{28, 29}]}, 
        {Hue[0.41201678385957174`, 1, 1], LineBox[{29, 30}]}, 
        {Hue[0.35912242361652047`, 1, 1], LineBox[{30, 31}]}, 
        {Hue[0.18101548966175515`, 1, 1], LineBox[{31, 32}]}, 
        {Hue[0.16450131136982224`, 1, 1], LineBox[{32, 33}]}, 
        {Hue[0.9480130500106693, 1, 1], LineBox[{33, 34}]}, 
        {Hue[0.7136847614331843, 1, 1], LineBox[{34, 35}]}, 
        {Hue[0.6820379269327059, 1, 1], LineBox[{35, 36}]}, 
        {Hue[0.5438279527273757, 1, 1], LineBox[{36, 37}]}, 
        {Hue[0.4815860989652991, 1, 1], LineBox[{37, 38}]}, 
        {Hue[0.4540529949539039, 1, 1], LineBox[{38, 39}]}, 
        {Hue[0.588981116835043, 1, 1], LineBox[{39, 40}]}, 
        {Hue[0.44981566460748473`, 1, 1], LineBox[{40, 41}]}, 
        {Hue[0.1962791259326371, 1, 1], LineBox[{41, 42}]}, 
        {Hue[0.5211218777608584, 1, 1], LineBox[{42, 43}]}, 
        {Hue[0.47547988701683014`, 1, 1], LineBox[{43, 44}]}, 
        {Hue[0.7495477847379686, 1, 1], LineBox[{44, 45}]}, 
        {Hue[0.6681734265858652, 1, 1], LineBox[{45, 46}]}, 
        {Hue[0.5077961245052601, 1, 1], LineBox[{46, 47}]}, 
        {Hue[0.6866704623527522, 1, 1], LineBox[{47, 48}]}, 
        {Hue[0.6452365448802283, 1, 1], LineBox[{48, 49}]}, 
        {Hue[0.8005378726770057, 1, 1], LineBox[{49, 50}]}, 
        {Hue[0.8643750617374688, 1, 1], LineBox[{50, 51}]}, 
        {Hue[0.6693497106381865, 1, 1], LineBox[{51, 52}]}, 
        {Hue[0.9575525251427208, 1, 1], LineBox[{52, 53}]}, 
        {Hue[0.22592667680464848`, 1, 1], LineBox[{53, 54}]}, 
        {Hue[0.2661863141424179, 1, 1], LineBox[{54, 55}]}, 
        {Hue[0.22840163982044662`, 1, 1], LineBox[{55, 56}]}, 
        {Hue[0.31215738198099585`, 1, 1], LineBox[{56, 57}]}, 
        {Hue[0.31477941715247937`, 1, 1], LineBox[{57, 58}]}, 
        {Hue[0.39024845611634024`, 1, 1], LineBox[{58, 59}]}, 
        {Hue[0.47248760524899003`, 1, 1], LineBox[{59, 60}]}, 
        {Hue[0.5211974196257148, 1, 1], LineBox[{60, 61}]}, 
        {Hue[0.5636352335907608, 1, 1], LineBox[{61, 62}]}, 
        {Hue[0.6027404106171735, 1, 1], LineBox[{62, 63}]}, 
        {Hue[0.6707034118133188, 1, 1], LineBox[{63, 64}]}, 
        {Hue[0.7327249096922388, 1, 1], LineBox[{64, 65}]}, 
        {Hue[0.7986624195234826, 1, 1], LineBox[{65, 66}]}, 
        {Hue[0.855887994596807, 1, 1], LineBox[{66, 67}]}, 
        {Hue[0.9095456426708814, 1, 1], LineBox[{67, 68}]}, 
        {Hue[0.970077867929754, 1, 1], LineBox[{68, 69}]}, 
        {Hue[0.036075328277316174`, 1, 1], LineBox[{69, 70}]}, 
        {Hue[0.0986472384956598, 1, 1], LineBox[{70, 71}]}, 
        {Hue[0.16258323239669467`, 1, 1], LineBox[{71, 72}]}, 
        {Hue[0.22439962875667932`, 1, 1], LineBox[{72, 73}]}, 
        {Hue[0.2868228301934304, 1, 1], LineBox[{73, 74}]}, 
        {Hue[0.3506068304944729, 1, 1], LineBox[{74, 75}]}, 
        {Hue[0.4151345914868187, 1, 1], LineBox[{75, 76}]}, 
        {Hue[0.4806693457773988, 1, 1], LineBox[{76, 77}]}, 
        {Hue[0.5460311877138803, 1, 1], LineBox[{77, 78}]}, 
        {Hue[0.6120088180229494, 1, 1], LineBox[{78, 79}]}, 
        {Hue[0.6780807385652687, 1, 1], LineBox[{79, 80}]}, 
        {Hue[0.7447573890440767, 1, 1], LineBox[{80, 81}]}, 
        {Hue[0.8124943399697049, 1, 1], LineBox[{81, 82}]}, 
        {Hue[0.8807725622213715, 1, 1], LineBox[{82, 83}]}, 
        {Hue[0.9493408685771565, 1, 1], LineBox[{83, 84}]}, 
        {Hue[0.018181533898883856`, 1, 1], LineBox[{84, 85}]}, 
        {Hue[0.08740416222963696, 1, 1], LineBox[{85, 86}]}, 
        {Hue[0.15726713787309227`, 1, 1], LineBox[{86, 87}]}, 
        {Hue[0.22793403650425142`, 1, 1], LineBox[{87, 88}]}, 
        {Hue[0.2992108352550998, 1, 1], LineBox[{88, 89}]}, 
        {Hue[0.37080822391280877`, 1, 1], LineBox[{89, 90}]}, 
        {Hue[0.44261246362794693`, 1, 1], LineBox[{90, 91}]}, 
        {Hue[0.5147944799691418, 1, 1], LineBox[{91, 92}]}, 
        {Hue[0.5876513479851337, 1, 1], LineBox[{92, 93}]}, 
        {Hue[0.6613124979945698, 1, 1], LineBox[{93, 94}]}, 
        {Hue[0.7355536948539427, 1, 1], LineBox[{94, 95}]}, 
        {Hue[0.8100331941773403, 1, 1], LineBox[{95, 96}]}, 
        {Hue[0.8845760918967533, 1, 1], LineBox[{96, 97}]}, 
        {Hue[0.9594392237794754, 1, 1], LineBox[{97, 98}]}, 
        {Hue[0.035026619894552934`, 1, 1], LineBox[{98, 99}]}, 
        {Hue[0.11148126313743942`, 1, 1], LineBox[{99, 100}]}, 
        {Hue[0.18846222297838114`, 1, 1], LineBox[{100, 101}]}, 
        {Hue[0.2654320874095458, 1, 1], LineBox[{101, 102}]}, 
        {Hue[0.3422347714240084, 1, 1], LineBox[{102, 103}]}, 
        {Hue[0.4192481284631285, 1, 1], LineBox[{103, 104}]}, 
        {Hue[0.4970426128281789, 1, 1], LineBox[{104, 105}]}, 
        {Hue[0.5757258532795645, 1, 1], LineBox[{105, 106}]}, 
        {Hue[0.6547295590250144, 1, 1], LineBox[{106, 107}]}, 
        {Hue[0.7333012536351259, 1, 1], LineBox[{107, 108}]}, 
        {Hue[0.8112505621189473, 1, 1], LineBox[{108, 109}]}, 
        {Hue[0.8891688664450038, 1, 1], LineBox[{109, 110}]}, 
        {Hue[0.9678089829795671, 1, 1], LineBox[{110, 111}]}, 
        {Hue[0.04724536239282264, 1, 1], LineBox[{111, 112}]}, 
        {Hue[0.12659362795393675`, 1, 1], LineBox[{112, 113}]}, 
        {Hue[0.2047569751445727, 1, 1], LineBox[{113, 114}]}, 
        {Hue[0.2814529462598848, 1, 1], LineBox[{114, 115}]}, 
        {Hue[0.3574199080216351, 1, 1], LineBox[{115, 116}]}, 
        {Hue[0.4336652855048778, 1, 1], LineBox[{116, 117}]}, 
        {Hue[0.5104069744697515, 1, 1], LineBox[{117, 118}]}, 
        {Hue[0.5866138643940774, 1, 1], LineBox[{118, 119}]}, 
        {Hue[0.6607219582217274, 1, 1], LineBox[{119, 120}]}, 
        {Hue[0.7319085375689991, 1, 1], LineBox[{120, 121}]}, 
        {Hue[0.800597168484313, 1, 1], LineBox[{121, 122}]}, 
        {Hue[0.8679469213743725, 1, 1], LineBox[{122, 123}]}, 
        {Hue[0.9349223090369303, 1, 1], LineBox[{123, 124}]}, 
        {Hue[0.001510620429620076, 1, 1], LineBox[{124, 125}]}, 
        {Hue[0.06652459152823118, 1, 1], LineBox[{125, 126}]}, 
        {Hue[0.1282599143269595, 1, 1], LineBox[{126, 127}]}, 
        {Hue[0.18550179423443017`, 1, 1], LineBox[{127, 128}]}, 
        {Hue[0.23800735641251244`, 1, 1], LineBox[{128, 129}]}, 
        {Hue[0.2862968986993083, 1, 1], LineBox[{129, 130}]}, 
        {Hue[0.33123280398036214`, 1, 1], LineBox[{130, 131}]}, 
        {Hue[0.3737126292032006, 1, 1], LineBox[{131, 132}]}, 
        {Hue[0.41453560107924475`, 1, 1], LineBox[{132, 133}]}, 
        {Hue[0.4543646210431929, 1, 1], LineBox[{133, 134}]}, 
        {Hue[0.49373324008370056`, 1, 1], LineBox[{134, 135}]}, 
        {Hue[0.5330669882843551, 1, 1], LineBox[{135, 136}]}, 
        {Hue[0.5727040733701112, 1, 1], LineBox[{136, 137}]}, 
        {Hue[0.6129127180864538, 1, 1], LineBox[{137, 138}]}, 
        {Hue[0.6539059380017792, 1, 1], LineBox[{138, 139}]}, 
        {Hue[0.6958512749606287, 1, 1], LineBox[{139, 140}]}, 
        {Hue[0.7388802296152222, 1, 1], LineBox[{140, 141}]}, 
        {Hue[0.7830946197868673, 1, 1], LineBox[{141, 142}]}, 
        {Hue[0.8285721896317425, 1, 1], LineBox[{142, 143}]}, 
        {Hue[0.8753695061192841, 1, 1], LineBox[{143, 144}]}, 
        {Hue[0.923527316882723, 1, 1], LineBox[{144, 145}]}, 
        {Hue[0.9730728782416933, 1, 1], LineBox[{145, 146}]}, 
        {Hue[0.024022163083270768`, 1, 1], LineBox[{146, 147}]}, 
        {Hue[0.07638221788367007, 1, 1], LineBox[{147, 148}]}, 
        {Hue[0.13015318159727385`, 1, 1], LineBox[{148, 149}]}, 
        {Hue[0.1853285701027619, 1, 1], LineBox[{149, 150}]}, 
        {Hue[0.24189914564441517`, 1, 1], LineBox[{150, 151}]}, 
        {Hue[0.2998500303351519, 1, 1], LineBox[{151, 152}]}, 
        {Hue[0.35916544194696576`, 1, 1], LineBox[{152, 153}]}, 
        {Hue[0.4198266093930038, 1, 1], LineBox[{153, 154}]}, 
        {Hue[0.4818143337854014, 1, 1], LineBox[{154, 155}]}, 
        {Hue[0.5451075206194197, 1, 1], LineBox[{155, 156}]}, 
        {Hue[0.6096848949214557, 1, 1], LineBox[{156, 157}]}, 
        {Hue[0.6755247680557995, 1, 1], LineBox[{157, 158}]}, 
        {Hue[0.7426074280870318, 1, 1], LineBox[{158, 159}]}, 
        {Hue[0.8109106936417033, 1, 1], LineBox[{159, 160}]}, 
        {Hue[0.8804129310143901, 1, 1], LineBox[{160, 161}]}, 
        {Hue[0.951093924998524, 1, 1], LineBox[{161, 162}]}, 
        {Hue[0.022934143089617415`, 1, 1], LineBox[{162, 163}]}, 
        {Hue[0.09591393913800811, 1, 1], LineBox[{163, 164}]}, 
        {Hue[0.17001339104666313`, 1, 1], LineBox[{164, 165}]}, 
        {Hue[0.2452148794921656, 1, 1], LineBox[{165, 166}]}, 
        {Hue[0.3215035907254118, 1, 1], LineBox[{166, 167}]}, 
        {Hue[0.3988570522565961, 1, 1], LineBox[{167, 168}]}, 
        {Hue[0.4772609021301015, 1, 1], LineBox[{168, 169}]}, 
        {Hue[0.5567029416512079, 1, 1], LineBox[{169, 170}]}, 
        {Hue[0.6371604032186986, 1, 1], LineBox[{170, 171}]}, 
        {Hue[0.7186211088714941, 1, 1], LineBox[{171, 172}]}, 
        {Hue[0.8010744330456714, 1, 1], LineBox[{172, 173}]}, 
        {Hue[0.8845040486856267, 1, 1], LineBox[{173, 174}]}, 
        {Hue[0.9688939180842179, 1, 1], LineBox[{174, 175}]}, 
        {Hue[0.05423573609039471, 1, 1], LineBox[{175, 176}]}, 
        {Hue[0.14051347720991264`, 1, 1], LineBox[{176, 177}]}, 
        {Hue[0.22770374758516046`, 1, 1], LineBox[{177, 178}]}, 
        {Hue[0.3158165858776364, 1, 1], LineBox[{178, 179}]}, 
        {Hue[0.40485117363829604`, 1, 1], LineBox[{179, 180}]}, 
        {Hue[0.4947574943312316, 1, 1], LineBox[{180, 181}]}, 
        {Hue[0.5855326413478238, 1, 1], LineBox[{181, 182}]}, 
        {Hue[0.6772189855349321, 1, 1], LineBox[{182, 183}]}, 
        {Hue[0.76975196888853, 1, 1], LineBox[{183, 184}]}, 
        {Hue[0.863080250307886, 1, 1], LineBox[{184, 185}]}, 
        {Hue[0.95731350477976, 1, 1], LineBox[{185, 186}]}, 
        {Hue[0.052435140580638674`, 1, 1], LineBox[{186, 187}]}, 
        {Hue[0.14829772018432766`, 1, 1], LineBox[{187, 188}]}, 
        {Hue[0.24497580630907112`, 1, 1], LineBox[{188, 189}]}, 
        {Hue[0.34257966189593014`, 1, 1], LineBox[{189, 190}]}, 
        {Hue[0.44099189618055484`, 1, 1], LineBox[{190, 191}]}, 
        {Hue[0.5401359020862424, 1, 1], LineBox[{191, 192}]}, 
        {Hue[0.6401264403994193, 1, 1], LineBox[{192, 193}]}, 
        {Hue[0.7409793231976449, 1, 1], LineBox[{193, 194}]}, 
        {Hue[0.842431620268548, 1, 1], LineBox[{194, 195}]}, 
        {Hue[0.9444064645043521, 1, 1], LineBox[{195, 196}]}, 
        {Hue[0.04739874473607524, 1, 1], LineBox[{196, 197}]}, 
        {Hue[0.15160986904984824`, 1, 1], LineBox[{197, 198}]}, 
        {Hue[0.2562192662631057, 1, 1], LineBox[{198, 199}]}, 
        {Hue[0.36062671712047917`, 1, 1], LineBox[{199, 200}]}, 
        {Hue[0.46586838643294537`, 1, 1], LineBox[{200, 201}]}, 
        {Hue[0.5731461208010898, 1, 1], LineBox[{201, 202}]}, 
        {Hue[0.680934766329103, 1, 1], LineBox[{202, 203}]}, 
        {Hue[0.7865115346926551, 1, 1], LineBox[{203, 204}]}, 
        {Hue[0.8927582528892654, 1, 1], LineBox[{204, 205}]}, 
        {Hue[0.00683152188586909, 1, 1], LineBox[{205, 206}]}, 
        {Hue[0.12756989049417516`, 1, 1], LineBox[{206, 207}]}, 
        {Hue[0.24415429036566494`, 1, 1], LineBox[{207, 208}]}, 
        {Hue[0.3490163234031179, 1, 1], LineBox[{208, 209}]}, 
        {Hue[0.45444314884782067`, 1, 1], LineBox[{209, 210}]}, 
        {Hue[0.5747967080651628, 1, 1], LineBox[{210, 211}]}, 
        {Hue[0.7022712739746917, 1, 1], LineBox[{211, 212}]}, 
        {Hue[0.8051517666191494, 1, 1], LineBox[{212, 213}]}, 
        {Hue[0.8245393293497276, 1, 1], LineBox[{213, 214}]}, 
        {Hue[0.8981395757716686, 1, 1], LineBox[{214, 215}]}, 
        {Hue[0.043647678620711894`, 1, 1], LineBox[{215, 216}]}, 
        {Hue[0.1952359421759699, 1, 1], LineBox[{216, 217}]}, 
        {Hue[0.2979846970988019, 1, 1], LineBox[{217, 218}]}, 
        {Hue[0.3842789352023804, 1, 1], LineBox[{218, 219}]}, 
        {Hue[0.5490636286686705, 1, 1], LineBox[{219, 220}]}, 
        {Hue[0.7531878470074891, 1, 1], LineBox[{220, 221}]}, 
        {Hue[0.9741316560530584, 1, 1], LineBox[{221, 222}]}, 
        {Hue[0.21069204938267144`, 1, 1], LineBox[{222, 223}]}, 
        {Hue[0.5179625464986409, 1, 1], LineBox[{223, 224}]}, 
        {Hue[0.9515960101338363, 1, 1], LineBox[{224, 225}]}, 
        {Hue[0.21823819534412597`, 1, 1], LineBox[{225, 226}]}, 
        {Hue[0.46127061830899735`, 1, 1], LineBox[{226, 227}]}, 
        {Hue[0.7122031613185107, 1, 1], LineBox[{227, 228}]}, 
        {Hue[0.9728813343268621, 1, 1], LineBox[{228, 229}]}, 
        {Hue[0.2420268926362587, 1, 1], LineBox[{229, 230}]}, 
        {Hue[0.5202779494326837, 1, 1], LineBox[{230, 231}]}, 
        {Hue[0.8127800983269537, 1, 1], LineBox[{231, 232}]}, 
        {Hue[0.13233314843452446`, 1, 1], LineBox[{232, 233}]}, 
        {Hue[0.48454859416905616`, 1, 1], LineBox[{233, 234}]}, 
        {Hue[0.8231200751796759, 1, 1], LineBox[{234, 235}]}, 
        {Hue[0.11904642996065473`, 1, 1], LineBox[{235, 236}]}, 
        {Hue[0.3777046432339246, 1, 1], LineBox[{236, 237}]}, 
        {Hue[0.5843606419824321, 1, 1], LineBox[{237, 238}]}, 
        {Hue[0.7304586385055036, 1, 1], LineBox[{238, 239}]}, 
        {Hue[0.9181566980435721, 1, 1], LineBox[{239, 240}]}, 
        {Hue[0.15404779540900293`, 1, 1], LineBox[{240, 241}]}, 
        {Hue[0.409814915165141, 1, 1], LineBox[{241, 242}]}, 
        {Hue[0.6769779427689762, 1, 1], LineBox[{242, 243}]}, 
        {Hue[0.9577656899112758, 1, 1], LineBox[{243, 244}]}, 
        {Hue[0.2723615776834439, 1, 1], LineBox[{244, 245}]}, 
        {Hue[0.6820614538136163, 1, 1], LineBox[{245, 246}]}, 
        {Hue[0.06994172851893946, 1, 1], LineBox[{246, 247}]}, 
        {Hue[0.37591322907590524`, 1, 1], LineBox[{247, 248}]}, 
        {Hue[0.6574196870928151, 1, 1], LineBox[{248, 249}]}, 
        {Hue[0.9305902035811251, 1, 1], LineBox[{249, 250}]}, 
        {Hue[0.20010644558572172`, 1, 1], LineBox[{250, 251}]}}}},
     VertexColors->{
       Hue[0.5134986713962597, 1, 1], 
       Hue[0.429675798262789, 1, 1], 
       Hue[0.36537759142896803`, 1, 1], 
       Hue[0.287489306520422, 1, 1], 
       Hue[0.11714250593167995`, 1, 1], 
       Hue[0.030302165263602618`, 1, 1], 
       Hue[0.0651831562564822, 1, 1], 
       Hue[0.03005619215611696, 1, 1], 
       Hue[0.8086350452323195, 1, 1], 
       Hue[0.5538481087637268, 1, 1], 
       Hue[0.5327463853830385, 1, 1], 
       Hue[0.3282688019815088, 1, 1], 
       Hue[0.2199518739415025, 1, 1], 
       Hue[0.22297225551656205`, 1, 1], 
       Hue[0.08319886681240779, 1, 1], 
       Hue[0.059918982847335045`, 1, 1], 
       Hue[0.9702160847775785, 1, 1], 
       Hue[0.7991004302923844, 1, 1], 
       Hue[0.644117418730759, 1, 1], 
       Hue[0.6356139264194032, 1, 1], 
       Hue[0.5268321040319055, 1, 1], 
       Hue[0.4849149904511286, 1, 1], 
       Hue[0.2951625421267708, 1, 1], 
       Hue[0.17072137554429173`, 1, 1], 
       Hue[0.8425020370839564, 1, 1], 
       Hue[0.5623136607528382, 1, 1], 
       Hue[0.3847582675161996, 1, 1], 
       Hue[0.6570983483750655, 1, 1], 
       Hue[0.41201678385957174`, 1, 1], 
       Hue[0.35912242361652047`, 1, 1], 
       Hue[0.18101548966175515`, 1, 1], 
       Hue[0.16450131136982224`, 1, 1], 
       Hue[0.9480130500106693, 1, 1], 
       Hue[0.7136847614331843, 1, 1], 
       Hue[0.6820379269327059, 1, 1], 
       Hue[0.5438279527273757, 1, 1], 
       Hue[0.4815860989652991, 1, 1], 
       Hue[0.4540529949539039, 1, 1], 
       Hue[0.588981116835043, 1, 1], 
       Hue[0.44981566460748473`, 1, 1], 
       Hue[0.1962791259326371, 1, 1], 
       Hue[0.5211218777608584, 1, 1], 
       Hue[0.47547988701683014`, 1, 1], 
       Hue[0.7495477847379686, 1, 1], 
       Hue[0.6681734265858652, 1, 1], 
       Hue[0.5077961245052601, 1, 1], 
       Hue[0.6866704623527522, 1, 1], 
       Hue[0.6452365448802283, 1, 1], 
       Hue[0.8005378726770057, 1, 1], 
       Hue[0.8643750617374688, 1, 1], 
       Hue[0.6693497106381865, 1, 1], 
       Hue[0.9575525251427208, 1, 1], 
       Hue[0.22592667680464848`, 1, 1], 
       Hue[0.2661863141424179, 1, 1], 
       Hue[0.22840163982044662`, 1, 1], 
       Hue[0.31215738198099585`, 1, 1], 
       Hue[0.31477941715247937`, 1, 1], 
       Hue[0.39024845611634024`, 1, 1], 
       Hue[0.47248760524899003`, 1, 1], 
       Hue[0.5211974196257148, 1, 1], 
       Hue[0.5636352335907608, 1, 1], 
       Hue[0.6027404106171735, 1, 1], 
       Hue[0.6707034118133188, 1, 1], 
       Hue[0.7327249096922388, 1, 1], 
       Hue[0.7986624195234826, 1, 1], 
       Hue[0.855887994596807, 1, 1], 
       Hue[0.9095456426708814, 1, 1], 
       Hue[0.970077867929754, 1, 1], 
       Hue[0.036075328277316174`, 1, 1], 
       Hue[0.0986472384956598, 1, 1], 
       Hue[0.16258323239669467`, 1, 1], 
       Hue[0.22439962875667932`, 1, 1], 
       Hue[0.2868228301934304, 1, 1], 
       Hue[0.3506068304944729, 1, 1], 
       Hue[0.4151345914868187, 1, 1], 
       Hue[0.4806693457773988, 1, 1], 
       Hue[0.5460311877138803, 1, 1], 
       Hue[0.6120088180229494, 1, 1], 
       Hue[0.6780807385652687, 1, 1], 
       Hue[0.7447573890440767, 1, 1], 
       Hue[0.8124943399697049, 1, 1], 
       Hue[0.8807725622213715, 1, 1], 
       Hue[0.9493408685771565, 1, 1], 
       Hue[0.018181533898883856`, 1, 1], 
       Hue[0.08740416222963696, 1, 1], 
       Hue[0.15726713787309227`, 1, 1], 
       Hue[0.22793403650425142`, 1, 1], 
       Hue[0.2992108352550998, 1, 1], 
       Hue[0.37080822391280877`, 1, 1], 
       Hue[0.44261246362794693`, 1, 1], 
       Hue[0.5147944799691418, 1, 1], 
       Hue[0.5876513479851337, 1, 1], 
       Hue[0.6613124979945698, 1, 1], 
       Hue[0.7355536948539427, 1, 1], 
       Hue[0.8100331941773403, 1, 1], 
       Hue[0.8845760918967533, 1, 1], 
       Hue[0.9594392237794754, 1, 1], 
       Hue[0.035026619894552934`, 1, 1], 
       Hue[0.11148126313743942`, 1, 1], 
       Hue[0.18846222297838114`, 1, 1], 
       Hue[0.2654320874095458, 1, 1], 
       Hue[0.3422347714240084, 1, 1], 
       Hue[0.4192481284631285, 1, 1], 
       Hue[0.4970426128281789, 1, 1], 
       Hue[0.5757258532795645, 1, 1], 
       Hue[0.6547295590250144, 1, 1], 
       Hue[0.7333012536351259, 1, 1], 
       Hue[0.8112505621189473, 1, 1], 
       Hue[0.8891688664450038, 1, 1], 
       Hue[0.9678089829795671, 1, 1], 
       Hue[0.04724536239282264, 1, 1], 
       Hue[0.12659362795393675`, 1, 1], 
       Hue[0.2047569751445727, 1, 1], 
       Hue[0.2814529462598848, 1, 1], 
       Hue[0.3574199080216351, 1, 1], 
       Hue[0.4336652855048778, 1, 1], 
       Hue[0.5104069744697515, 1, 1], 
       Hue[0.5866138643940774, 1, 1], 
       Hue[0.6607219582217274, 1, 1], 
       Hue[0.7319085375689991, 1, 1], 
       Hue[0.800597168484313, 1, 1], 
       Hue[0.8679469213743725, 1, 1], 
       Hue[0.9349223090369303, 1, 1], 
       Hue[0.001510620429620076, 1, 1], 
       Hue[0.06652459152823118, 1, 1], 
       Hue[0.1282599143269595, 1, 1], 
       Hue[0.18550179423443017`, 1, 1], 
       Hue[0.23800735641251244`, 1, 1], 
       Hue[0.2862968986993083, 1, 1], 
       Hue[0.33123280398036214`, 1, 1], 
       Hue[0.3737126292032006, 1, 1], 
       Hue[0.41453560107924475`, 1, 1], 
       Hue[0.4543646210431929, 1, 1], 
       Hue[0.49373324008370056`, 1, 1], 
       Hue[0.5330669882843551, 1, 1], 
       Hue[0.5727040733701112, 1, 1], 
       Hue[0.6129127180864538, 1, 1], 
       Hue[0.6539059380017792, 1, 1], 
       Hue[0.6958512749606287, 1, 1], 
       Hue[0.7388802296152222, 1, 1], 
       Hue[0.7830946197868673, 1, 1], 
       Hue[0.8285721896317425, 1, 1], 
       Hue[0.8753695061192841, 1, 1], 
       Hue[0.923527316882723, 1, 1], 
       Hue[0.9730728782416933, 1, 1], 
       Hue[0.024022163083270768`, 1, 1], 
       Hue[0.07638221788367007, 1, 1], 
       Hue[0.13015318159727385`, 1, 1], 
       Hue[0.1853285701027619, 1, 1], 
       Hue[0.24189914564441517`, 1, 1], 
       Hue[0.2998500303351519, 1, 1], 
       Hue[0.35916544194696576`, 1, 1], 
       Hue[0.4198266093930038, 1, 1], 
       Hue[0.4818143337854014, 1, 1], 
       Hue[0.5451075206194197, 1, 1], 
       Hue[0.6096848949214557, 1, 1], 
       Hue[0.6755247680557995, 1, 1], 
       Hue[0.7426074280870318, 1, 1], 
       Hue[0.8109106936417033, 1, 1], 
       Hue[0.8804129310143901, 1, 1], 
       Hue[0.951093924998524, 1, 1], 
       Hue[0.022934143089617415`, 1, 1], 
       Hue[0.09591393913800811, 1, 1], 
       Hue[0.17001339104666313`, 1, 1], 
       Hue[0.2452148794921656, 1, 1], 
       Hue[0.3215035907254118, 1, 1], 
       Hue[0.3988570522565961, 1, 1], 
       Hue[0.4772609021301015, 1, 1], 
       Hue[0.5567029416512079, 1, 1], 
       Hue[0.6371604032186986, 1, 1], 
       Hue[0.7186211088714941, 1, 1], 
       Hue[0.8010744330456714, 1, 1], 
       Hue[0.8845040486856267, 1, 1], 
       Hue[0.9688939180842179, 1, 1], 
       Hue[0.05423573609039471, 1, 1], 
       Hue[0.14051347720991264`, 1, 1], 
       Hue[0.22770374758516046`, 1, 1], 
       Hue[0.3158165858776364, 1, 1], 
       Hue[0.40485117363829604`, 1, 1], 
       Hue[0.4947574943312316, 1, 1], 
       Hue[0.5855326413478238, 1, 1], 
       Hue[0.6772189855349321, 1, 1], 
       Hue[0.76975196888853, 1, 1], 
       Hue[0.863080250307886, 1, 1], 
       Hue[0.95731350477976, 1, 1], 
       Hue[0.052435140580638674`, 1, 1], 
       Hue[0.14829772018432766`, 1, 1], 
       Hue[0.24497580630907112`, 1, 1], 
       Hue[0.34257966189593014`, 1, 1], 
       Hue[0.44099189618055484`, 1, 1], 
       Hue[0.5401359020862424, 1, 1], 
       Hue[0.6401264403994193, 1, 1], 
       Hue[0.7409793231976449, 1, 1], 
       Hue[0.842431620268548, 1, 1], 
       Hue[0.9444064645043521, 1, 1], 
       Hue[0.04739874473607524, 1, 1], 
       Hue[0.15160986904984824`, 1, 1], 
       Hue[0.2562192662631057, 1, 1], 
       Hue[0.36062671712047917`, 1, 1], 
       Hue[0.46586838643294537`, 1, 1], 
       Hue[0.5731461208010898, 1, 1], 
       Hue[0.680934766329103, 1, 1], 
       Hue[0.7865115346926551, 1, 1], 
       Hue[0.8927582528892654, 1, 1], 
       Hue[0.00683152188586909, 1, 1], 
       Hue[0.12756989049417516`, 1, 1], 
       Hue[0.24415429036566494`, 1, 1], 
       Hue[0.3490163234031179, 1, 1], 
       Hue[0.45444314884782067`, 1, 1], 
       Hue[0.5747967080651628, 1, 1], 
       Hue[0.7022712739746917, 1, 1], 
       Hue[0.8051517666191494, 1, 1], 
       Hue[0.8245393293497276, 1, 1], 
       Hue[0.8981395757716686, 1, 1], 
       Hue[0.043647678620711894`, 1, 1], 
       Hue[0.1952359421759699, 1, 1], 
       Hue[0.2979846970988019, 1, 1], 
       Hue[0.3842789352023804, 1, 1], 
       Hue[0.5490636286686705, 1, 1], 
       Hue[0.7531878470074891, 1, 1], 
       Hue[0.9741316560530584, 1, 1], 
       Hue[0.21069204938267144`, 1, 1], 
       Hue[0.5179625464986409, 1, 1], 
       Hue[0.9515960101338363, 1, 1], 
       Hue[0.21823819534412597`, 1, 1], 
       Hue[0.46127061830899735`, 1, 1], 
       Hue[0.7122031613185107, 1, 1], 
       Hue[0.9728813343268621, 1, 1], 
       Hue[0.2420268926362587, 1, 1], 
       Hue[0.5202779494326837, 1, 1], 
       Hue[0.8127800983269537, 1, 1], 
       Hue[0.13233314843452446`, 1, 1], 
       Hue[0.48454859416905616`, 1, 1], 
       Hue[0.8231200751796759, 1, 1], 
       Hue[0.11904642996065473`, 1, 1], 
       Hue[0.3777046432339246, 1, 1], 
       Hue[0.5843606419824321, 1, 1], 
       Hue[0.7304586385055036, 1, 1], 
       Hue[0.9181566980435721, 1, 1], 
       Hue[0.15404779540900293`, 1, 1], 
       Hue[0.409814915165141, 1, 1], 
       Hue[0.6769779427689762, 1, 1], 
       Hue[0.9577656899112758, 1, 1], 
       Hue[0.2723615776834439, 1, 1], 
       Hue[0.6820614538136163, 1, 1], 
       Hue[0.06994172851893946, 1, 1], 
       Hue[0.37591322907590524`, 1, 1], 
       Hue[0.6574196870928151, 1, 1], 
       Hue[0.9305902035811251, 1, 1], 
       Hue[0.20010644558572172`, 1, 1], 
       Hue[0.4677155460079821, 1, 1], 
       Hue[0.5134986713962597, 1, 1], 
       Hue[0.429675798262789, 1, 1], 
       Hue[0.429675798262789, 1, 1], 
       Hue[0.36537759142896803`, 1, 1], 
       Hue[0.36537759142896803`, 1, 1], 
       Hue[0.287489306520422, 1, 1], 
       Hue[0.287489306520422, 1, 1], 
       Hue[0.11714250593167995`, 1, 1], 
       Hue[0.11714250593167995`, 1, 1], 
       Hue[0.030302165263602618`, 1, 1], 
       Hue[0.030302165263602618`, 1, 1], 
       Hue[0.0651831562564822, 1, 1], 
       Hue[0.0651831562564822, 1, 1], 
       Hue[0.03005619215611696, 1, 1], 
       Hue[0.03005619215611696, 1, 1], 
       Hue[0.8086350452323195, 1, 1], 
       Hue[0.8086350452323195, 1, 1], 
       Hue[0.5538481087637268, 1, 1], 
       Hue[0.5538481087637268, 1, 1], 
       Hue[0.5327463853830385, 1, 1], 
       Hue[0.5327463853830385, 1, 1], 
       Hue[0.3282688019815088, 1, 1], 
       Hue[0.3282688019815088, 1, 1], 
       Hue[0.2199518739415025, 1, 1], 
       Hue[0.2199518739415025, 1, 1], 
       Hue[0.22297225551656205`, 1, 1], 
       Hue[0.22297225551656205`, 1, 1], 
       Hue[0.08319886681240779, 1, 1], 
       Hue[0.08319886681240779, 1, 1], 
       Hue[0.059918982847335045`, 1, 1], 
       Hue[0.059918982847335045`, 1, 1], 
       Hue[0.9702160847775785, 1, 1], 
       Hue[0.9702160847775785, 1, 1], 
       Hue[0.7991004302923844, 1, 1], 
       Hue[0.7991004302923844, 1, 1], 
       Hue[0.644117418730759, 1, 1], 
       Hue[0.644117418730759, 1, 1], 
       Hue[0.6356139264194032, 1, 1], 
       Hue[0.6356139264194032, 1, 1], 
       Hue[0.5268321040319055, 1, 1], 
       Hue[0.5268321040319055, 1, 1], 
       Hue[0.4849149904511286, 1, 1], 
       Hue[0.4849149904511286, 1, 1], 
       Hue[0.2951625421267708, 1, 1], 
       Hue[0.2951625421267708, 1, 1], 
       Hue[0.17072137554429173`, 1, 1], 
       Hue[0.17072137554429173`, 1, 1], 
       Hue[0.8425020370839564, 1, 1], 
       Hue[0.8425020370839564, 1, 1], 
       Hue[0.5623136607528382, 1, 1], 
       Hue[0.5623136607528382, 1, 1], 
       Hue[0.3847582675161996, 1, 1], 
       Hue[0.3847582675161996, 1, 1], 
       Hue[0.6570983483750655, 1, 1], 
       Hue[0.6570983483750655, 1, 1], 
       Hue[0.41201678385957174`, 1, 1], 
       Hue[0.41201678385957174`, 1, 1], 
       Hue[0.35912242361652047`, 1, 1], 
       Hue[0.35912242361652047`, 1, 1], 
       Hue[0.18101548966175515`, 1, 1], 
       Hue[0.18101548966175515`, 1, 1], 
       Hue[0.16450131136982224`, 1, 1], 
       Hue[0.16450131136982224`, 1, 1], 
       Hue[0.9480130500106693, 1, 1], 
       Hue[0.9480130500106693, 1, 1], 
       Hue[0.7136847614331843, 1, 1], 
       Hue[0.7136847614331843, 1, 1], 
       Hue[0.6820379269327059, 1, 1], 
       Hue[0.6820379269327059, 1, 1], 
       Hue[0.5438279527273757, 1, 1], 
       Hue[0.5438279527273757, 1, 1], 
       Hue[0.4815860989652991, 1, 1], 
       Hue[0.4815860989652991, 1, 1], 
       Hue[0.4540529949539039, 1, 1], 
       Hue[0.4540529949539039, 1, 1], 
       Hue[0.588981116835043, 1, 1], 
       Hue[0.588981116835043, 1, 1], 
       Hue[0.44981566460748473`, 1, 1], 
       Hue[0.44981566460748473`, 1, 1], 
       Hue[0.1962791259326371, 1, 1], 
       Hue[0.1962791259326371, 1, 1], 
       Hue[0.5211218777608584, 1, 1], 
       Hue[0.5211218777608584, 1, 1], 
       Hue[0.47547988701683014`, 1, 1], 
       Hue[0.47547988701683014`, 1, 1], 
       Hue[0.7495477847379686, 1, 1], 
       Hue[0.7495477847379686, 1, 1], 
       Hue[0.6681734265858652, 1, 1], 
       Hue[0.6681734265858652, 1, 1], 
       Hue[0.5077961245052601, 1, 1], 
       Hue[0.5077961245052601, 1, 1], 
       Hue[0.6866704623527522, 1, 1], 
       Hue[0.6866704623527522, 1, 1], 
       Hue[0.6452365448802283, 1, 1], 
       Hue[0.6452365448802283, 1, 1], 
       Hue[0.8005378726770057, 1, 1], 
       Hue[0.8005378726770057, 1, 1], 
       Hue[0.8643750617374688, 1, 1], 
       Hue[0.8643750617374688, 1, 1], 
       Hue[0.6693497106381865, 1, 1], 
       Hue[0.6693497106381865, 1, 1], 
       Hue[0.9575525251427208, 1, 1], 
       Hue[0.9575525251427208, 1, 1], 
       Hue[0.22592667680464848`, 1, 1], 
       Hue[0.22592667680464848`, 1, 1], 
       Hue[0.2661863141424179, 1, 1], 
       Hue[0.2661863141424179, 1, 1], 
       Hue[0.22840163982044662`, 1, 1], 
       Hue[0.22840163982044662`, 1, 1], 
       Hue[0.31215738198099585`, 1, 1], 
       Hue[0.31215738198099585`, 1, 1], 
       Hue[0.31477941715247937`, 1, 1], 
       Hue[0.31477941715247937`, 1, 1], 
       Hue[0.39024845611634024`, 1, 1], 
       Hue[0.39024845611634024`, 1, 1], 
       Hue[0.47248760524899003`, 1, 1], 
       Hue[0.47248760524899003`, 1, 1], 
       Hue[0.5211974196257148, 1, 1], 
       Hue[0.5211974196257148, 1, 1], 
       Hue[0.5636352335907608, 1, 1], 
       Hue[0.5636352335907608, 1, 1], 
       Hue[0.6027404106171735, 1, 1], 
       Hue[0.6027404106171735, 1, 1], 
       Hue[0.6707034118133188, 1, 1], 
       Hue[0.6707034118133188, 1, 1], 
       Hue[0.7327249096922388, 1, 1], 
       Hue[0.7327249096922388, 1, 1], 
       Hue[0.7986624195234826, 1, 1], 
       Hue[0.7986624195234826, 1, 1], 
       Hue[0.855887994596807, 1, 1], 
       Hue[0.855887994596807, 1, 1], 
       Hue[0.9095456426708814, 1, 1], 
       Hue[0.9095456426708814, 1, 1], 
       Hue[0.970077867929754, 1, 1], 
       Hue[0.970077867929754, 1, 1], 
       Hue[0.036075328277316174`, 1, 1], 
       Hue[0.036075328277316174`, 1, 1], 
       Hue[0.0986472384956598, 1, 1], 
       Hue[0.0986472384956598, 1, 1], 
       Hue[0.16258323239669467`, 1, 1], 
       Hue[0.16258323239669467`, 1, 1], 
       Hue[0.22439962875667932`, 1, 1], 
       Hue[0.22439962875667932`, 1, 1], 
       Hue[0.2868228301934304, 1, 1], 
       Hue[0.2868228301934304, 1, 1], 
       Hue[0.3506068304944729, 1, 1], 
       Hue[0.3506068304944729, 1, 1], 
       Hue[0.4151345914868187, 1, 1], 
       Hue[0.4151345914868187, 1, 1], 
       Hue[0.4806693457773988, 1, 1], 
       Hue[0.4806693457773988, 1, 1], 
       Hue[0.5460311877138803, 1, 1], 
       Hue[0.5460311877138803, 1, 1], 
       Hue[0.6120088180229494, 1, 1], 
       Hue[0.6120088180229494, 1, 1], 
       Hue[0.6780807385652687, 1, 1], 
       Hue[0.6780807385652687, 1, 1], 
       Hue[0.7447573890440767, 1, 1], 
       Hue[0.7447573890440767, 1, 1], 
       Hue[0.8124943399697049, 1, 1], 
       Hue[0.8124943399697049, 1, 1], 
       Hue[0.8807725622213715, 1, 1], 
       Hue[0.8807725622213715, 1, 1], 
       Hue[0.9493408685771565, 1, 1], 
       Hue[0.9493408685771565, 1, 1], 
       Hue[0.018181533898883856`, 1, 1], 
       Hue[0.018181533898883856`, 1, 1], 
       Hue[0.08740416222963696, 1, 1], 
       Hue[0.08740416222963696, 1, 1], 
       Hue[0.15726713787309227`, 1, 1], 
       Hue[0.15726713787309227`, 1, 1], 
       Hue[0.22793403650425142`, 1, 1], 
       Hue[0.22793403650425142`, 1, 1], 
       Hue[0.2992108352550998, 1, 1], 
       Hue[0.2992108352550998, 1, 1], 
       Hue[0.37080822391280877`, 1, 1], 
       Hue[0.37080822391280877`, 1, 1], 
       Hue[0.44261246362794693`, 1, 1], 
       Hue[0.44261246362794693`, 1, 1], 
       Hue[0.5147944799691418, 1, 1], 
       Hue[0.5147944799691418, 1, 1], 
       Hue[0.5876513479851337, 1, 1], 
       Hue[0.5876513479851337, 1, 1], 
       Hue[0.6613124979945698, 1, 1], 
       Hue[0.6613124979945698, 1, 1], 
       Hue[0.7355536948539427, 1, 1], 
       Hue[0.7355536948539427, 1, 1], 
       Hue[0.8100331941773403, 1, 1], 
       Hue[0.8100331941773403, 1, 1], 
       Hue[0.8845760918967533, 1, 1], 
       Hue[0.8845760918967533, 1, 1], 
       Hue[0.9594392237794754, 1, 1], 
       Hue[0.9594392237794754, 1, 1], 
       Hue[0.035026619894552934`, 1, 1], 
       Hue[0.035026619894552934`, 1, 1], 
       Hue[0.11148126313743942`, 1, 1], 
       Hue[0.11148126313743942`, 1, 1], 
       Hue[0.18846222297838114`, 1, 1], 
       Hue[0.18846222297838114`, 1, 1], 
       Hue[0.2654320874095458, 1, 1], 
       Hue[0.2654320874095458, 1, 1], 
       Hue[0.3422347714240084, 1, 1], 
       Hue[0.3422347714240084, 1, 1], 
       Hue[0.4192481284631285, 1, 1], 
       Hue[0.4192481284631285, 1, 1], 
       Hue[0.4970426128281789, 1, 1], 
       Hue[0.4970426128281789, 1, 1], 
       Hue[0.5757258532795645, 1, 1], 
       Hue[0.5757258532795645, 1, 1], 
       Hue[0.6547295590250144, 1, 1], 
       Hue[0.6547295590250144, 1, 1], 
       Hue[0.7333012536351259, 1, 1], 
       Hue[0.7333012536351259, 1, 1], 
       Hue[0.8112505621189473, 1, 1], 
       Hue[0.8112505621189473, 1, 1], 
       Hue[0.8891688664450038, 1, 1], 
       Hue[0.8891688664450038, 1, 1], 
       Hue[0.9678089829795671, 1, 1], 
       Hue[0.9678089829795671, 1, 1], 
       Hue[0.04724536239282264, 1, 1], 
       Hue[0.04724536239282264, 1, 1], 
       Hue[0.12659362795393675`, 1, 1], 
       Hue[0.12659362795393675`, 1, 1], 
       Hue[0.2047569751445727, 1, 1], 
       Hue[0.2047569751445727, 1, 1], 
       Hue[0.2814529462598848, 1, 1], 
       Hue[0.2814529462598848, 1, 1], 
       Hue[0.3574199080216351, 1, 1], 
       Hue[0.3574199080216351, 1, 1], 
       Hue[0.4336652855048778, 1, 1], 
       Hue[0.4336652855048778, 1, 1], 
       Hue[0.5104069744697515, 1, 1], 
       Hue[0.5104069744697515, 1, 1], 
       Hue[0.5866138643940774, 1, 1], 
       Hue[0.5866138643940774, 1, 1], 
       Hue[0.6607219582217274, 1, 1], 
       Hue[0.6607219582217274, 1, 1], 
       Hue[0.7319085375689991, 1, 1], 
       Hue[0.7319085375689991, 1, 1], 
       Hue[0.800597168484313, 1, 1], 
       Hue[0.800597168484313, 1, 1], 
       Hue[0.8679469213743725, 1, 1], 
       Hue[0.8679469213743725, 1, 1], 
       Hue[0.9349223090369303, 1, 1], 
       Hue[0.9349223090369303, 1, 1], 
       Hue[0.001510620429620076, 1, 1], 
       Hue[0.001510620429620076, 1, 1], 
       Hue[0.06652459152823118, 1, 1], 
       Hue[0.06652459152823118, 1, 1], 
       Hue[0.1282599143269595, 1, 1], 
       Hue[0.1282599143269595, 1, 1], 
       Hue[0.18550179423443017`, 1, 1], 
       Hue[0.18550179423443017`, 1, 1], 
       Hue[0.23800735641251244`, 1, 1], 
       Hue[0.23800735641251244`, 1, 1], 
       Hue[0.2862968986993083, 1, 1], 
       Hue[0.2862968986993083, 1, 1], 
       Hue[0.33123280398036214`, 1, 1], 
       Hue[0.33123280398036214`, 1, 1], 
       Hue[0.3737126292032006, 1, 1], 
       Hue[0.3737126292032006, 1, 1], 
       Hue[0.41453560107924475`, 1, 1], 
       Hue[0.41453560107924475`, 1, 1], 
       Hue[0.4543646210431929, 1, 1], 
       Hue[0.4543646210431929, 1, 1], 
       Hue[0.49373324008370056`, 1, 1], 
       Hue[0.49373324008370056`, 1, 1], 
       Hue[0.5330669882843551, 1, 1], 
       Hue[0.5330669882843551, 1, 1], 
       Hue[0.5727040733701112, 1, 1], 
       Hue[0.5727040733701112, 1, 1], 
       Hue[0.6129127180864538, 1, 1], 
       Hue[0.6129127180864538, 1, 1], 
       Hue[0.6539059380017792, 1, 1], 
       Hue[0.6539059380017792, 1, 1], 
       Hue[0.6958512749606287, 1, 1], 
       Hue[0.6958512749606287, 1, 1], 
       Hue[0.7388802296152222, 1, 1], 
       Hue[0.7388802296152222, 1, 1], 
       Hue[0.7830946197868673, 1, 1], 
       Hue[0.7830946197868673, 1, 1], 
       Hue[0.8285721896317425, 1, 1], 
       Hue[0.8285721896317425, 1, 1], 
       Hue[0.8753695061192841, 1, 1], 
       Hue[0.8753695061192841, 1, 1], 
       Hue[0.923527316882723, 1, 1], 
       Hue[0.923527316882723, 1, 1], 
       Hue[0.9730728782416933, 1, 1], 
       Hue[0.9730728782416933, 1, 1], 
       Hue[0.024022163083270768`, 1, 1], 
       Hue[0.024022163083270768`, 1, 1], 
       Hue[0.07638221788367007, 1, 1], 
       Hue[0.07638221788367007, 1, 1], 
       Hue[0.13015318159727385`, 1, 1], 
       Hue[0.13015318159727385`, 1, 1], 
       Hue[0.1853285701027619, 1, 1], 
       Hue[0.1853285701027619, 1, 1], 
       Hue[0.24189914564441517`, 1, 1], 
       Hue[0.24189914564441517`, 1, 1], 
       Hue[0.2998500303351519, 1, 1], 
       Hue[0.2998500303351519, 1, 1], 
       Hue[0.35916544194696576`, 1, 1], 
       Hue[0.35916544194696576`, 1, 1], 
       Hue[0.4198266093930038, 1, 1], 
       Hue[0.4198266093930038, 1, 1], 
       Hue[0.4818143337854014, 1, 1], 
       Hue[0.4818143337854014, 1, 1], 
       Hue[0.5451075206194197, 1, 1], 
       Hue[0.5451075206194197, 1, 1], 
       Hue[0.6096848949214557, 1, 1], 
       Hue[0.6096848949214557, 1, 1], 
       Hue[0.6755247680557995, 1, 1], 
       Hue[0.6755247680557995, 1, 1], 
       Hue[0.7426074280870318, 1, 1], 
       Hue[0.7426074280870318, 1, 1], 
       Hue[0.8109106936417033, 1, 1], 
       Hue[0.8109106936417033, 1, 1], 
       Hue[0.8804129310143901, 1, 1], 
       Hue[0.8804129310143901, 1, 1], 
       Hue[0.951093924998524, 1, 1], 
       Hue[0.951093924998524, 1, 1], 
       Hue[0.022934143089617415`, 1, 1], 
       Hue[0.022934143089617415`, 1, 1], 
       Hue[0.09591393913800811, 1, 1], 
       Hue[0.09591393913800811, 1, 1], 
       Hue[0.17001339104666313`, 1, 1], 
       Hue[0.17001339104666313`, 1, 1], 
       Hue[0.2452148794921656, 1, 1], 
       Hue[0.2452148794921656, 1, 1], 
       Hue[0.3215035907254118, 1, 1], 
       Hue[0.3215035907254118, 1, 1], 
       Hue[0.3988570522565961, 1, 1], 
       Hue[0.3988570522565961, 1, 1], 
       Hue[0.4772609021301015, 1, 1], 
       Hue[0.4772609021301015, 1, 1], 
       Hue[0.5567029416512079, 1, 1], 
       Hue[0.5567029416512079, 1, 1], 
       Hue[0.6371604032186986, 1, 1], 
       Hue[0.6371604032186986, 1, 1], 
       Hue[0.7186211088714941, 1, 1], 
       Hue[0.7186211088714941, 1, 1], 
       Hue[0.8010744330456714, 1, 1], 
       Hue[0.8010744330456714, 1, 1], 
       Hue[0.8845040486856267, 1, 1], 
       Hue[0.8845040486856267, 1, 1], 
       Hue[0.9688939180842179, 1, 1], 
       Hue[0.9688939180842179, 1, 1], 
       Hue[0.05423573609039471, 1, 1], 
       Hue[0.05423573609039471, 1, 1], 
       Hue[0.14051347720991264`, 1, 1], 
       Hue[0.14051347720991264`, 1, 1], 
       Hue[0.22770374758516046`, 1, 1], 
       Hue[0.22770374758516046`, 1, 1], 
       Hue[0.3158165858776364, 1, 1], 
       Hue[0.3158165858776364, 1, 1], 
       Hue[0.40485117363829604`, 1, 1], 
       Hue[0.40485117363829604`, 1, 1], 
       Hue[0.4947574943312316, 1, 1], 
       Hue[0.4947574943312316, 1, 1], 
       Hue[0.5855326413478238, 1, 1], 
       Hue[0.5855326413478238, 1, 1], 
       Hue[0.6772189855349321, 1, 1], 
       Hue[0.6772189855349321, 1, 1], 
       Hue[0.76975196888853, 1, 1], 
       Hue[0.76975196888853, 1, 1], 
       Hue[0.863080250307886, 1, 1], 
       Hue[0.863080250307886, 1, 1], 
       Hue[0.95731350477976, 1, 1], 
       Hue[0.95731350477976, 1, 1], 
       Hue[0.052435140580638674`, 1, 1], 
       Hue[0.052435140580638674`, 1, 1], 
       Hue[0.14829772018432766`, 1, 1], 
       Hue[0.14829772018432766`, 1, 1], 
       Hue[0.24497580630907112`, 1, 1], 
       Hue[0.24497580630907112`, 1, 1], 
       Hue[0.34257966189593014`, 1, 1], 
       Hue[0.34257966189593014`, 1, 1], 
       Hue[0.44099189618055484`, 1, 1], 
       Hue[0.44099189618055484`, 1, 1], 
       Hue[0.5401359020862424, 1, 1], 
       Hue[0.5401359020862424, 1, 1], 
       Hue[0.6401264403994193, 1, 1], 
       Hue[0.6401264403994193, 1, 1], 
       Hue[0.7409793231976449, 1, 1], 
       Hue[0.7409793231976449, 1, 1], 
       Hue[0.842431620268548, 1, 1], 
       Hue[0.842431620268548, 1, 1], 
       Hue[0.9444064645043521, 1, 1], 
       Hue[0.9444064645043521, 1, 1], 
       Hue[0.04739874473607524, 1, 1], 
       Hue[0.04739874473607524, 1, 1], 
       Hue[0.15160986904984824`, 1, 1], 
       Hue[0.15160986904984824`, 1, 1], 
       Hue[0.2562192662631057, 1, 1], 
       Hue[0.2562192662631057, 1, 1], 
       Hue[0.36062671712047917`, 1, 1], 
       Hue[0.36062671712047917`, 1, 1], 
       Hue[0.46586838643294537`, 1, 1], 
       Hue[0.46586838643294537`, 1, 1], 
       Hue[0.5731461208010898, 1, 1], 
       Hue[0.5731461208010898, 1, 1], 
       Hue[0.680934766329103, 1, 1], 
       Hue[0.680934766329103, 1, 1], 
       Hue[0.7865115346926551, 1, 1], 
       Hue[0.7865115346926551, 1, 1], 
       Hue[0.8927582528892654, 1, 1], 
       Hue[0.8927582528892654, 1, 1], 
       Hue[0.00683152188586909, 1, 1], 
       Hue[0.00683152188586909, 1, 1], 
       Hue[0.12756989049417516`, 1, 1], 
       Hue[0.12756989049417516`, 1, 1], 
       Hue[0.24415429036566494`, 1, 1], 
       Hue[0.24415429036566494`, 1, 1], 
       Hue[0.3490163234031179, 1, 1], 
       Hue[0.3490163234031179, 1, 1], 
       Hue[0.45444314884782067`, 1, 1], 
       Hue[0.45444314884782067`, 1, 1], 
       Hue[0.5747967080651628, 1, 1], 
       Hue[0.5747967080651628, 1, 1], 
       Hue[0.7022712739746917, 1, 1], 
       Hue[0.7022712739746917, 1, 1], 
       Hue[0.8051517666191494, 1, 1], 
       Hue[0.8051517666191494, 1, 1], 
       Hue[0.8245393293497276, 1, 1], 
       Hue[0.8245393293497276, 1, 1], 
       Hue[0.8981395757716686, 1, 1], 
       Hue[0.8981395757716686, 1, 1], 
       Hue[0.043647678620711894`, 1, 1], 
       Hue[0.043647678620711894`, 1, 1], 
       Hue[0.1952359421759699, 1, 1], 
       Hue[0.1952359421759699, 1, 1], 
       Hue[0.2979846970988019, 1, 1], 
       Hue[0.2979846970988019, 1, 1], 
       Hue[0.3842789352023804, 1, 1], 
       Hue[0.3842789352023804, 1, 1], 
       Hue[0.5490636286686705, 1, 1], 
       Hue[0.5490636286686705, 1, 1], 
       Hue[0.7531878470074891, 1, 1], 
       Hue[0.7531878470074891, 1, 1], 
       Hue[0.9741316560530584, 1, 1], 
       Hue[0.9741316560530584, 1, 1], 
       Hue[0.21069204938267144`, 1, 1], 
       Hue[0.21069204938267144`, 1, 1], 
       Hue[0.5179625464986409, 1, 1], 
       Hue[0.5179625464986409, 1, 1], 
       Hue[0.9515960101338363, 1, 1], 
       Hue[0.9515960101338363, 1, 1], 
       Hue[0.21823819534412597`, 1, 1], 
       Hue[0.21823819534412597`, 1, 1], 
       Hue[0.46127061830899735`, 1, 1], 
       Hue[0.46127061830899735`, 1, 1], 
       Hue[0.7122031613185107, 1, 1], 
       Hue[0.7122031613185107, 1, 1], 
       Hue[0.9728813343268621, 1, 1], 
       Hue[0.9728813343268621, 1, 1], 
       Hue[0.2420268926362587, 1, 1], 
       Hue[0.2420268926362587, 1, 1], 
       Hue[0.5202779494326837, 1, 1], 
       Hue[0.5202779494326837, 1, 1], 
       Hue[0.8127800983269537, 1, 1], 
       Hue[0.8127800983269537, 1, 1], 
       Hue[0.13233314843452446`, 1, 1], 
       Hue[0.13233314843452446`, 1, 1], 
       Hue[0.48454859416905616`, 1, 1], 
       Hue[0.48454859416905616`, 1, 1], 
       Hue[0.8231200751796759, 1, 1], 
       Hue[0.8231200751796759, 1, 1], 
       Hue[0.11904642996065473`, 1, 1], 
       Hue[0.11904642996065473`, 1, 1], 
       Hue[0.3777046432339246, 1, 1], 
       Hue[0.3777046432339246, 1, 1], 
       Hue[0.5843606419824321, 1, 1], 
       Hue[0.5843606419824321, 1, 1], 
       Hue[0.7304586385055036, 1, 1], 
       Hue[0.7304586385055036, 1, 1], 
       Hue[0.9181566980435721, 1, 1], 
       Hue[0.9181566980435721, 1, 1], 
       Hue[0.15404779540900293`, 1, 1], 
       Hue[0.15404779540900293`, 1, 1], 
       Hue[0.409814915165141, 1, 1], 
       Hue[0.409814915165141, 1, 1], 
       Hue[0.6769779427689762, 1, 1], 
       Hue[0.6769779427689762, 1, 1], 
       Hue[0.9577656899112758, 1, 1], 
       Hue[0.9577656899112758, 1, 1], 
       Hue[0.2723615776834439, 1, 1], 
       Hue[0.2723615776834439, 1, 1], 
       Hue[0.6820614538136163, 1, 1], 
       Hue[0.6820614538136163, 1, 1], 
       Hue[0.06994172851893946, 1, 1], 
       Hue[0.06994172851893946, 1, 1], 
       Hue[0.37591322907590524`, 1, 1], 
       Hue[0.37591322907590524`, 1, 1], 
       Hue[0.6574196870928151, 1, 1], 
       Hue[0.6574196870928151, 1, 1], 
       Hue[0.9305902035811251, 1, 1], 
       Hue[0.9305902035811251, 1, 1], 
       Hue[0.20010644558572172`, 1, 1], 
       Hue[0.20010644558572172`, 1, 1], 
       Hue[0.4677155460079821, 1, 1]}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->{True, None},
    AxesOrigin->{0, 0},
    Epilog->
     LineBox[{{1, 1.293512231656321*^-6}, {2, 9.141179925278823*^-7}, {
       3, 8.738486673240133*^-7}, {4, 1.1210907591190583`*^-6}, {
       5, 9.07458607036266*^-7}, {6, 4.335430178556219*^-7}, {
       7, 5.74511132360924*^-7}, {8, 2.9722907071946993`*^-7}, {
       9, 1.1114240457962253`*^-6}, {10, 3.9462169867884443`*^-7}, {
       11, 6.72138609389269*^-7}, {12, 9.01247445382832*^-7}, {
       13, 4.255413005141942*^-7}, {14, 8.722579684489437*^-7}, {
       15, 8.593168312381147*^-7}, {16, 8.669063730412045*^-7}, {
       17, 1.2595782450007813`*^-6}, {18, 1.506617868651792*^-6}, {
       19, 1.1757365608115608`*^-6}, {20, 8.025743290499689*^-7}, {
       21, 6.732285678809414*^-7}, {22, 7.758828188347845*^-7}, {
       23, 6.850954466267486*^-7}, {24, 2.7640494086808906`*^-7}, {
       25, 4.1610743188772513`*^-7}, {26, 5.497172127152359*^-7}, {
       27, 3.509473577203337*^-7}, {28, 3.1532358712482945`*^-7}, {
       29, 3.1602394629538306`*^-7}, {30, 4.802712349263791*^-7}, {
       31, 3.059337111566676*^-7}, {32, 2.9289214899306654`*^-7}, {
       33, 7.446363940210739*^-7}, {34, 4.781745861647592*^-7}, {
       35, 6.884589351806089*^-7}, {36, 4.043057013188036*^-7}, {
       37, 6.26413944214673*^-7}, {38, 3.9242325745653125`*^-7}, {
       39, 2.6984107852635816`*^-7}, {40, 4.0388859979536897`*^-7}, {
       41, 1.481126130275057*^-7}, {42, 2.7924034872987627`*^-7}, {
       43, 5.920943221166556*^-8}, {44, 2.5574650870813133`*^-7}, {
       45, 5.338061392273096*^-7}, {46, 3.3157916898313786`*^-7}, {
       47, 5.080266787759876*^-7}, {48, 1.9576585161032869`*^-7}, {
       49, 6.446274351345195*^-7}, {50, 9.117893641634066*^-7}, {
       51, 5.641773057275045*^-7}, {52, 1.4713599820993626`*^-7}, {
       53, 9.276509431557103*^-7}, {54, 1.949581240149834*^-6}, {
       55, 1.8429925869274515`*^-6}, {56, 3.0501785922866223`*^-6}, {
       57, 3.1801313669957977`*^-6}, {58, 4.240223537517871*^-6}, {
       59, 5.770471339188001*^-6}, {60, 7.806637939346476*^-6}, {
       61, 0.000010836651825653717`}, {62, 0.000013213601151066506`}, {
       63, 0.00001573018688427932}, {64, 0.0000194955168537409}, {
       65, 0.000025278389727982726`}, {66, 0.000034283650623131885`}, {
       67, 0.000042413992302537525`}, {68, 0.000052378440784301256`}, {
       69, 0.00006514038828741289}, {70, 0.00008276211956885553}, {
       71, 0.00010436266510232029`}, {72, 0.00013147205706328773`}, {
       73, 0.0001644640094295517}, {74, 0.00020369831563033585`}, {
       75, 0.0002527719879515726}, {76, 0.0003120986993609501}, {
       77, 0.0003859504945095197}, {78, 0.0004756812335648483}, {
       79, 0.000583959707457682}, {80, 0.0007120971327799281}, {
       81, 0.0008670458036575534}, {82, 0.0010546593415718998`}, {
       83, 0.0012807496458557576`}, {84, 0.001549727376410733}, {
       85, 0.0018683441728276685`}, {86, 0.0022410959858374887`}, {
       87, 0.002682757294860802}, {88, 0.003206744685566178}, {
       89, 0.003827240618801887}, {90, 0.00455353926527167}, {
       91, 0.005393639532321564}, {92, 0.006360409592698273}, {
       93, 0.007483240821529209}, {94, 0.008796005862820604}, {
       95, 0.010328902748770472`}, {96, 0.012089150433735006`}, {
       97, 0.01407695310547073}, {98, 0.01631329752193934}, {
       99, 0.01886838983075003}, {100, 0.021829441161571064`}, {
       101, 0.0252472239761647}, {102, 0.029092384280423406`}, {
       103, 0.033316826391802966`}, {104, 0.03796235936310983}, {
       105, 0.04321439645996892}, {106, 0.049296394577029806`}, {
       107, 0.056266371961745026`}, {108, 0.06394939612262282}, {
       109, 0.0721401259564591}, {110, 0.08094319415817058}, {
       111, 0.09087443155879571}, {112, 0.10250964089336785`}, {
       113, 0.11591118174746179`}, {114, 0.13049409990327585`}, {
       115, 0.14561428723859315`}, {116, 0.16141534152146533`}, {
       117, 0.17913859651728026`}, {118, 0.20036233552247132`}, {
       119, 0.22563468200675418`}, {120, 0.2538608223489282}, {
       121, 0.2831438247640217}, {122, 0.31245646125942195`}, {
       123, 0.34289984266884477`}, {124, 0.3776610436133015}, {
       125, 0.42039205520094763`}, {126, 0.4729304085167958}, {
       127, 0.5339993839351291}, {128, 0.5996979492315827}, {
       129, 0.6649636411211871}, {130, 0.7249393249452404}, {
       131, 0.7758037070419624}, {132, 0.8150649936086384}, {
       133, 0.8415434412900966}, {134, 0.8551543054082128}, {
       135, 0.8566266116156382}, {136, 0.84722963235018}, {
       137, 0.8285176908095345}, {138, 0.8021545435679595}, {
       139, 0.7697728830438603}, {140, 0.732890414207474}, {
       141, 0.6928638366334374}, {142, 0.6508648879340589}, {
       143, 0.6078840960132798}, {144, 0.5647401608361832}, {
       145, 0.5220934545210268}, {146, 0.4804649624244304}, {
       147, 0.4402581863244774}, {148, 0.4017737863835495}, {
       149, 0.36522951065326015`}, {150, 0.33076851972809906`}, {
       151, 0.2984767440061327}, {152, 0.26839148065552443`}, {
       153, 0.240511846373238}, {154, 0.21480407111506367`}, {
       155, 0.1912106729030629}, {156, 0.1696543138854673}, {
       157, 0.15004499743935715`}, {158, 0.13228013405326708`}, {
       159, 0.11624986241032739`}, {160, 0.10184224373137564`}, {
       161, 0.08894298053627564}, {162, 0.07743707595025123}, {
       163, 0.06721160522174467}, {164, 0.05815722715814426}, {
       165, 0.05016927570122365}, {166, 0.043146144449944426`}, {
       167, 0.03699272744649877}, {168, 0.031621311760072705`}, {
       169, 0.026947529670115163`}, {170, 0.02289472101164024}, {
       171, 0.01939310538204206}, {172, 0.01637747784421382}, {
       173, 0.013788899138288105`}, {174, 0.011574518784269747`}, {
       175, 0.009686638180006514}, {176, 0.008081974594870676}, {
       177, 0.006723110362494051}, {178, 0.0055765664924465}, {
       179, 0.004611307803165932}, {180, 0.003801297502792852}, {
       181, 0.003124782105194667}, {182, 0.002561037097792583}, {
       183, 0.0020920854609505655`}, {184, 0.0017043262606185644`}, {
       185, 0.001384830922755092}, {186, 0.001121349919165515}, {
       187, 0.0009051277606995639}, {188, 0.000728888605462127}, {
       189, 0.0005852038490514287}, {190, 0.000468143977313343}, {
       191, 0.0003733955357207099}, {192, 0.00029701716047071864`}, {
       193, 0.00023536789815051942`}, {194, 0.00018576725940270443`}, {
       195, 0.00014636873569506386`}, {196, 0.00011521799623727578`}, {
       197, 0.0000902380920373945}, {198, 0.00007015764286455641}, {
       199, 0.00005446901393564296}, {200, 0.000042464233711868196`}, {
       201, 0.00003299611122747644}, {202, 0.000025283501423243874`}, {
       203, 0.000019365025353130492`}, {204, 0.000015261501326867872`}, {
       205, 0.000012149528911031253`}, {206, 9.204816704419756*^-6}, {
       207, 6.523031805848554*^-6}, {208, 4.643962784735902*^-6}, {
       209, 3.5683167936200154`*^-6}, {210, 2.7322297961770383`*^-6}, {
       211, 1.837123085155947*^-6}, {212, 1.0056988926017879`*^-6}, {
       213, 6.747753956907239*^-7}, {214, 8.69852940691384*^-7}, {
       215, 9.22063894374567*^-7}, {216, 7.188839891749446*^-7}, {
       217, 4.904189636537491*^-7}, {218, 5.405660028472834*^-7}, {
       219, 6.534520250027231*^-7}, {220, 6.180973262270588*^-7}, {
       221, 4.5248445263946285`*^-7}, {222, 2.478787272298718*^-7}, {
       223, 9.296103639784852*^-8}, {224, 1.0921799299772099`*^-7}, {
       225, 2.0711755693726188`*^-7}, {226, 3.081392505696856*^-7}, {
       227, 3.923024023110412*^-7}, {228, 4.3769496072188006`*^-7}, {
       229, 4.312443204886432*^-7}, {230, 3.747963886256413*^-7}, {
       231, 2.866613694737511*^-7}, {232, 2.005305727655022*^-7}, {
       233, 1.5631262708631255`*^-7}, {234, 1.5449683109040095`*^-7}, {
       235, 1.5098429014051752`*^-7}, {236, 1.2522275526235434`*^-7}, {
       237, 8.995853327982884*^-8}, {238, 8.55037910558022*^-8}, {
       239, 1.1932707796824371`*^-7}, {240, 1.521606743728176*^-7}, {
       241, 1.647117895354992*^-7}, {242, 1.5272485327266351`*^-7}, {
       243, 1.1977216631568382`*^-7}, {244, 7.664213250933791*^-8}, {
       245, 5.466237583583313*^-8}, {246, 8.715763702470234*^-8}, {
       247, 1.3428755108958065`*^-7}, {248, 1.744023835071272*^-7}, {
       249, 2.0163074144323127`*^-7}, {250, 2.1456332049311917`*^-7}, {
       251, 2.1413213272558206`*^-7}}],
    Frame->True,
    FrameTicks->{{{1., 
        FormBox[
         RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
        FormBox[
         RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
        FormBox[
         RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
        FormBox[
         RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
        FormBox["0", TraditionalForm]}, {157.25, 
        FormBox["2.5`", TraditionalForm]}, {188.5, 
        FormBox["5", TraditionalForm]}, {219.75, 
        FormBox["7.5`", TraditionalForm]}, {251., 
        FormBox["10", TraditionalForm]}}, Automatic, None, None},
    PlotRange->{-0.1, 1.1},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], Automatic}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF13tcTekeBvCV0SCXiUM1hD1FQxMquTd5KilF6a6LWrV3u/tlmjiMwQop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     "], {
      {GrayLevel[0.5], EdgeForm[None], 
       GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ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         "]]]}, {{}, {}, {
        {Hue[0.10248796621842285`, 1, 1], LineBox[{1, 2}]}, 
        {Hue[0.8290457167148891, 1, 1], LineBox[{2, 3}]}, 
        {Hue[0.8703632125845846, 1, 1], LineBox[{3, 4}]}, 
        {Hue[0.7393175820802007, 1, 1], LineBox[{4, 5}]}, 
        {Hue[0.6032998790616205, 1, 1], LineBox[{5, 6}]}, 
        {Hue[0.5291282883599169, 1, 1], LineBox[{6, 7}]}, 
        {Hue[0.37551039951532045`, 1, 1], LineBox[{7, 8}]}, 
        {Hue[0.22433259918415324`, 1, 1], LineBox[{8, 9}]}, 
        {Hue[0.11852503188071434`, 1, 1], LineBox[{9, 10}]}, 
        {Hue[0.07197136740119216, 1, 1], LineBox[{10, 11}]}, 
        {Hue[0.027235479998395193`, 1, 1], LineBox[{11, 12}]}, 
        {Hue[0.9072061553893599, 1, 1], LineBox[{12, 13}]}, 
        {Hue[0.8405170901747704, 1, 1], LineBox[{13, 14}]}, 
        {Hue[0.8075393950498422, 1, 1], LineBox[{14, 15}]}, 
        {Hue[0.5331074488604866, 1, 1], LineBox[{15, 16}]}, 
        {Hue[0.5358981991195568, 1, 1], LineBox[{16, 17}]}, 
        {Hue[0.25004222875796983`, 1, 1], LineBox[{17, 18}]}, 
        {Hue[0.7962568815322872, 1, 1], LineBox[{18, 19}]}, 
        {Hue[0.8602643648636271, 1, 1], LineBox[{19, 20}]}, 
        {Hue[0.7061829013619123, 1, 1], LineBox[{20, 21}]}, 
        {Hue[0.5353893983812265, 1, 1], LineBox[{21, 22}]}, 
        {Hue[0.5574883411356903, 1, 1], LineBox[{22, 23}]}, 
        {Hue[0.3982347746773217, 1, 1], LineBox[{23, 24}]}, 
        {Hue[0.376614336574381, 1, 1], LineBox[{24, 25}]}, 
        {Hue[0.1848960953069641, 1, 1], LineBox[{25, 26}]}, 
        {Hue[0.39778971853907324`, 1, 1], LineBox[{26, 27}]}, 
        {Hue[0.09352019075476536, 1, 1], LineBox[{27, 28}]}, 
        {Hue[0.783376412070276, 1, 1], LineBox[{28, 29}]}, 
        {Hue[0.7824547112898794, 1, 1], LineBox[{29, 30}]}, 
        {Hue[0.7761971219426127, 1, 1], LineBox[{30, 31}]}, 
        {Hue[0.6038072405559677, 1, 1], LineBox[{31, 32}]}, 
        {Hue[0.560930797116872, 1, 1], LineBox[{32, 33}]}, 
        {Hue[0.33776760760892577`, 1, 1], LineBox[{33, 34}]}, 
        {Hue[0.35334587499948134`, 1, 1], LineBox[{34, 35}]}, 
        {Hue[0.06870296802393036, 1, 1], LineBox[{35, 36}]}, 
        {Hue[0.6821357962793559, 1, 1], LineBox[{36, 37}]}, 
        {Hue[0.6351997075059883, 1, 1], LineBox[{37, 38}]}, 
        {Hue[0.40994418070536254`, 1, 1], LineBox[{38, 39}]}, 
        {Hue[0.6600956815350549, 1, 1], LineBox[{39, 40}]}, 
        {Hue[0.3763685961693272, 1, 1], LineBox[{40, 41}]}, 
        {Hue[0.0525203213503501, 1, 1], LineBox[{41, 42}]}, 
        {Hue[0.9908307255976239, 1, 1], LineBox[{42, 43}]}, 
        {Hue[0.009823911060358891, 1, 1], LineBox[{43, 44}]}, 
        {Hue[0.8095599146726241, 1, 1], LineBox[{44, 45}]}, 
        {Hue[0.6425002953713946, 1, 1], LineBox[{45, 46}]}, 
        {Hue[0.6328654713099459, 1, 1], LineBox[{46, 47}]}, 
        {Hue[0.6176065487895835, 1, 1], LineBox[{47, 48}]}, 
        {Hue[0.491800422525271, 1, 1], LineBox[{48, 49}]}, 
        {Hue[0.3386878267347081, 1, 1], LineBox[{49, 50}]}, 
        {Hue[0.19665319598299283`, 1, 1], LineBox[{50, 51}]}, 
        {Hue[0.0021306817817562038`, 1, 1], LineBox[{51, 52}]}, 
        {Hue[0.8729735920333851, 1, 1], LineBox[{52, 53}]}, 
        {Hue[0.7003776259936819, 1, 1], LineBox[{53, 54}]}, 
        {Hue[0.6587624394900979, 1, 1], LineBox[{54, 55}]}, 
        {Hue[0.46120340580633984`, 1, 1], LineBox[{55, 56}]}, 
        {Hue[0.36582744313624216`, 1, 1], LineBox[{56, 57}]}, 
        {Hue[0.36577014645724054`, 1, 1], LineBox[{57, 58}]}, 
        {Hue[0.2768708630533676, 1, 1], LineBox[{58, 59}]}, 
        {Hue[0.401572703318089, 1, 1], LineBox[{59, 60}]}, 
        {Hue[0.4543051107625647, 1, 1], LineBox[{60, 61}]}, 
        {Hue[0.4950883665628377, 1, 1], LineBox[{61, 62}]}, 
        {Hue[0.5727212045820049, 1, 1], LineBox[{62, 63}]}, 
        {Hue[0.6144593839253265, 1, 1], LineBox[{63, 64}]}, 
        {Hue[0.650777198099475, 1, 1], LineBox[{64, 65}]}, 
        {Hue[0.6812023476718669, 1, 1], LineBox[{65, 66}]}, 
        {Hue[0.7366775496335508, 1, 1], LineBox[{66, 67}]}, 
        {Hue[0.8040407063237689, 1, 1], LineBox[{67, 68}]}, 
        {Hue[0.8751337841577034, 1, 1], LineBox[{68, 69}]}, 
        {Hue[0.9349979895740793, 1, 1], LineBox[{69, 70}]}, 
        {Hue[0.9831611795516255, 1, 1], LineBox[{70, 71}]}, 
        {Hue[0.03690540732882161, 1, 1], LineBox[{71, 72}]}, 
        {Hue[0.09340805898442196, 1, 1], LineBox[{72, 73}]}, 
        {Hue[0.15555150481302726`, 1, 1], LineBox[{73, 74}]}, 
        {Hue[0.22260102211802169`, 1, 1], LineBox[{74, 75}]}, 
        {Hue[0.2874873707395083, 1, 1], LineBox[{75, 76}]}, 
        {Hue[0.34371795285997764`, 1, 1], LineBox[{76, 77}]}, 
        {Hue[0.3987686908705344, 1, 1], LineBox[{77, 78}]}, 
        {Hue[0.45584195734129307`, 1, 1], LineBox[{78, 79}]}, 
        {Hue[0.5194171517525992, 1, 1], LineBox[{79, 80}]}, 
        {Hue[0.5871948626381034, 1, 1], LineBox[{80, 81}]}, 
        {Hue[0.6550071558423163, 1, 1], LineBox[{81, 82}]}, 
        {Hue[0.7169971729064637, 1, 1], LineBox[{82, 83}]}, 
        {Hue[0.7758915686233734, 1, 1], LineBox[{83, 84}]}, 
        {Hue[0.8351048410044376, 1, 1], LineBox[{84, 85}]}, 
        {Hue[0.8997708000560913, 1, 1], LineBox[{85, 86}]}, 
        {Hue[0.9703622628350036, 1, 1], LineBox[{86, 87}]}, 
        {Hue[0.04204741694621801, 1, 1], LineBox[{87, 88}]}, 
        {Hue[0.1084845737470324, 1, 1], LineBox[{88, 89}]}, 
        {Hue[0.1701666187456011, 1, 1], LineBox[{89, 90}]}, 
        {Hue[0.2313775399901313, 1, 1], LineBox[{90, 91}]}, 
        {Hue[0.2976877502367062, 1, 1], LineBox[{91, 92}]}, 
        {Hue[0.3711740550744846, 1, 1], LineBox[{92, 93}]}, 
        {Hue[0.44687625426845595`, 1, 1], LineBox[{93, 94}]}, 
        {Hue[0.5176126759525406, 1, 1], LineBox[{94, 95}]}, 
        {Hue[0.5819664960418984, 1, 1], LineBox[{95, 96}]}, 
        {Hue[0.6448711364223789, 1, 1], LineBox[{96, 97}]}, 
        {Hue[0.7127443005598428, 1, 1], LineBox[{97, 98}]}, 
        {Hue[0.7888911244253934, 1, 1], LineBox[{98, 99}]}, 
        {Hue[0.8685972235020938, 1, 1], LineBox[{99, 100}]}, 
        {Hue[0.9429413869593951, 1, 1], LineBox[{100, 101}]}, 
        {Hue[0.009597528089488936, 1, 1], LineBox[{101, 102}]}, 
        {Hue[0.07369556643187233, 1, 1], LineBox[{102, 103}]}, 
        {Hue[0.14259002452799643`, 1, 1], LineBox[{103, 104}]}, 
        {Hue[0.2208055343087441, 1, 1], LineBox[{104, 105}]}, 
        {Hue[0.30397005118067305`, 1, 1], LineBox[{105, 106}]}, 
        {Hue[0.3814022118325686, 1, 1], LineBox[{106, 107}]}, 
        {Hue[0.44958930198492464`, 1, 1], LineBox[{107, 108}]}, 
        {Hue[0.5137878132576624, 1, 1], LineBox[{108, 109}]}, 
        {Hue[0.5819989623965363, 1, 1], LineBox[{109, 110}]}, 
        {Hue[0.6601775740658046, 1, 1], LineBox[{110, 111}]}, 
        {Hue[0.745202837419231, 1, 1], LineBox[{111, 112}]}, 
        {Hue[0.8250475018811899, 1, 1], LineBox[{112, 113}]}, 
        {Hue[0.8938729371910848, 1, 1], LineBox[{113, 114}]}, 
        {Hue[0.9561034162561497, 1, 1], LineBox[{114, 115}]}, 
        {Hue[0.019638189549080177`, 1, 1], LineBox[{115, 116}]}, 
        {Hue[0.09164212060033193, 1, 1], LineBox[{116, 117}]}, 
        {Hue[0.17317222581323283`, 1, 1], LineBox[{117, 118}]}, 
        {Hue[0.2541781134560693, 1, 1], LineBox[{118, 119}]}, 
        {Hue[0.3241269011239155, 1, 1], LineBox[{119, 120}]}, 
        {Hue[0.3830525677867494, 1, 1], LineBox[{120, 121}]}, 
        {Hue[0.43671758530245763`, 1, 1], LineBox[{121, 122}]}, 
        {Hue[0.491429567541259, 1, 1], LineBox[{122, 123}]}, 
        {Hue[0.5524263664053911, 1, 1], LineBox[{123, 124}]}, 
        {Hue[0.6213032633035116, 1, 1], LineBox[{124, 125}]}, 
        {Hue[0.6922386416207904, 1, 1], LineBox[{125, 126}]}, 
        {Hue[0.7556807223227779, 1, 1], LineBox[{126, 127}]}, 
        {Hue[0.8075261375163918, 1, 1], LineBox[{127, 128}]}, 
        {Hue[0.8494469370502941, 1, 1], LineBox[{128, 129}]}, 
        {Hue[0.8845456784920832, 1, 1], LineBox[{129, 130}]}, 
        {Hue[0.9153607707770892, 1, 1], LineBox[{130, 131}]}, 
        {Hue[0.9436483964312276, 1, 1], LineBox[{131, 132}]}, 
        {Hue[0.9705913152413616, 1, 1], LineBox[{132, 133}]}, 
        {Hue[0.9969991385198878, 1, 1], LineBox[{133, 134}]}, 
        {Hue[0.023443344689877202`, 1, 1], LineBox[{134, 135}]}, 
        {Hue[0.0503392189712252, 1, 1], LineBox[{135, 136}]}, 
        {Hue[0.07799583048219152, 1, 1], LineBox[{136, 137}]}, 
        {Hue[0.10664739890542389`, 1, 1], LineBox[{137, 138}]}, 
        {Hue[0.13647223751425389`, 1, 1], LineBox[{138, 139}]}, 
        {Hue[0.16760651012276648`, 1, 1], LineBox[{139, 140}]}, 
        {Hue[0.20015193386020552`, 1, 1], LineBox[{140, 141}]}, 
        {Hue[0.2341836723124326, 1, 1], LineBox[{141, 142}]}, 
        {Hue[0.2697549633000815, 1, 1], LineBox[{142, 143}]}, 
        {Hue[0.3069010347031737, 1, 1], LineBox[{143, 144}]}, 
        {Hue[0.34564242381955096`, 1, 1], LineBox[{144, 145}]}, 
        {Hue[0.385988114107708, 1, 1], LineBox[{145, 146}]}, 
        {Hue[0.4279373960933569, 1, 1], LineBox[{146, 147}]}, 
        {Hue[0.4714821138884691, 1, 1], LineBox[{147, 148}]}, 
        {Hue[0.516608142439269, 1, 1], LineBox[{148, 149}]}, 
        {Hue[0.5632967986431676, 1, 1], LineBox[{149, 150}]}, 
        {Hue[0.6115262291158869, 1, 1], LineBox[{150, 151}]}, 
        {Hue[0.6612723650240759, 1, 1], LineBox[{151, 152}]}, 
        {Hue[0.7125082110086003, 1, 1], LineBox[{152, 153}]}, 
        {Hue[0.7652067245285928, 1, 1], LineBox[{153, 154}]}, 
        {Hue[0.8193404899939781, 1, 1], LineBox[{154, 155}]}, 
        {Hue[0.8748814082258843, 1, 1], LineBox[{155, 156}]}, 
        {Hue[0.93180121767383, 1, 1], LineBox[{156, 157}]}, 
        {Hue[0.9900732339915105, 1, 1], LineBox[{157, 158}]}, 
        {Hue[0.04966897366675908, 1, 1], LineBox[{158, 159}]}, 
        {Hue[0.11056355278904713`, 1, 1], LineBox[{159, 160}]}, 
        {Hue[0.17273127731108673`, 1, 1], LineBox[{160, 161}]}, 
        {Hue[0.23614688428905453`, 1, 1], LineBox[{161, 162}]}, 
        {Hue[0.30078631795382943`, 1, 1], LineBox[{162, 163}]}, 
        {Hue[0.36662673011340297`, 1, 1], LineBox[{163, 164}]}, 
        {Hue[0.4336462625630184, 1, 1], LineBox[{164, 165}]}, 
        {Hue[0.5018243590466764, 1, 1], LineBox[{165, 166}]}, 
        {Hue[0.5711391738638567, 1, 1], LineBox[{166, 167}]}, 
        {Hue[0.6415712630612694, 1, 1], LineBox[{167, 168}]}, 
        {Hue[0.7131026687380215, 1, 1], LineBox[{168, 169}]}, 
        {Hue[0.785714952794411, 1, 1], LineBox[{169, 170}]}, 
        {Hue[0.85939125988999, 1, 1], LineBox[{170, 171}]}, 
        {Hue[0.9341142030507813, 1, 1], LineBox[{171, 172}]}, 
        {Hue[0.009867814283582065, 1, 1], LineBox[{172, 173}]}, 
        {Hue[0.08663975672705043, 1, 1], LineBox[{173, 174}]}, 
        {Hue[0.16441331632110967`, 1, 1], LineBox[{174, 175}]}, 
        {Hue[0.24317140299829, 1, 1], LineBox[{175, 176}]}, 
        {Hue[0.32290236585478704`, 1, 1], LineBox[{176, 177}]}, 
        {Hue[0.4035943587363709, 1, 1], LineBox[{177, 178}]}, 
        {Hue[0.48523999204304397`, 1, 1], LineBox[{178, 179}]}, 
        {Hue[0.5678193374677074, 1, 1], LineBox[{179, 180}]}, 
        {Hue[0.6513185536279276, 1, 1], LineBox[{180, 181}]}, 
        {Hue[0.7357401903243395, 1, 1], LineBox[{181, 182}]}, 
        {Hue[0.8210622956913834, 1, 1], LineBox[{182, 183}]}, 
        {Hue[0.9072682139525383, 1, 1], LineBox[{183, 184}]}, 
        {Hue[0.9943604247210516, 1, 1], LineBox[{184, 185}]}, 
        {Hue[0.08232920234124495, 1, 1], LineBox[{185, 186}]}, 
        {Hue[0.17115182211425745`, 1, 1], LineBox[{186, 187}]}, 
        {Hue[0.2608259833213761, 1, 1], LineBox[{187, 188}]}, 
        {Hue[0.3513592109082063, 1, 1], LineBox[{188, 189}]}, 
        {Hue[0.44272159494008767`, 1, 1], LineBox[{189, 190}]}, 
        {Hue[0.5348962505409033, 1, 1], LineBox[{190, 191}]}, 
        {Hue[0.6279072641081033, 1, 1], LineBox[{191, 192}]}, 
        {Hue[0.7217267880302329, 1, 1], LineBox[{192, 193}]}, 
        {Hue[0.816313267532884, 1, 1], LineBox[{193, 194}]}, 
        {Hue[0.9117512965679803, 1, 1], LineBox[{194, 195}]}, 
        {Hue[0.008047879736664992, 1, 1], LineBox[{195, 196}]}, 
        {Hue[0.10501187209075581`, 1, 1], LineBox[{196, 197}]}, 
        {Hue[0.20270535185021696`, 1, 1], LineBox[{197, 198}]}, 
        {Hue[0.30133780047475683`, 1, 1], LineBox[{198, 199}]}, 
        {Hue[0.40068737757147616`, 1, 1], LineBox[{199, 200}]}, 
        {Hue[0.500621695745771, 1, 1], LineBox[{200, 201}]}, 
        {Hue[0.6015194562805606, 1, 1], LineBox[{201, 202}]}, 
        {Hue[0.7033866071661006, 1, 1], LineBox[{202, 203}]}, 
        {Hue[0.8057600247529666, 1, 1], LineBox[{203, 204}]}, 
        {Hue[0.9087115831851035, 1, 1], LineBox[{204, 205}]}, 
        {Hue[0.012728494365962887`, 1, 1], LineBox[{205, 206}]}, 
        {Hue[0.11768230946122678`, 1, 1], LineBox[{206, 207}]}, 
        {Hue[0.22299007300419238`, 1, 1], LineBox[{207, 208}]}, 
        {Hue[0.32878942455954685`, 1, 1], LineBox[{208, 209}]}, 
        {Hue[0.43566326944187145`, 1, 1], LineBox[{209, 210}]}, 
        {Hue[0.5430660387149626, 1, 1], LineBox[{210, 211}]}, 
        {Hue[0.6501242831582152, 1, 1], LineBox[{211, 212}]}, 
        {Hue[0.758048676916268, 1, 1], LineBox[{212, 213}]}, 
        {Hue[0.8686784353266866, 1, 1], LineBox[{213, 214}]}, 
        {Hue[0.9804881391600404, 1, 1], LineBox[{214, 215}]}, 
        {Hue[0.08932909517187086, 1, 1], LineBox[{215, 216}]}, 
        {Hue[0.1955941259474388, 1, 1], LineBox[{216, 217}]}, 
        {Hue[0.30741713430835516`, 1, 1], LineBox[{217, 218}]}, 
        {Hue[0.4270155355366292, 1, 1], LineBox[{218, 219}]}, 
        {Hue[0.5429924302256508, 1, 1], LineBox[{219, 220}]}, 
        {Hue[0.6483558235535518, 1, 1], LineBox[{220, 221}]}, 
        {Hue[0.7667154238986771, 1, 1], LineBox[{221, 222}]}, 
        {Hue[0.9142402763382171, 1, 1], LineBox[{222, 223}]}, 
        {Hue[0.07850281472384095, 1, 1], LineBox[{223, 224}]}, 
        {Hue[0.2333380105758965, 1, 1], LineBox[{224, 225}]}, 
        {Hue[0.23584220459241986`, 1, 1], LineBox[{225, 226}]}, 
        {Hue[0.2548056612472624, 1, 1], LineBox[{226, 227}]}, 
        {Hue[0.41845090004777136`, 1, 1], LineBox[{227, 228}]}, 
        {Hue[0.5707092498069801, 1, 1], LineBox[{228, 229}]}, 
        {Hue[0.33232299147995975`, 1, 1], LineBox[{229, 230}]}, 
        {Hue[0.531991234171352, 1, 1], LineBox[{230, 231}]}, 
        {Hue[0.7431920582950459, 1, 1], LineBox[{231, 232}]}, 
        {Hue[0.9572424609776871, 1, 1], LineBox[{232, 233}]}, 
        {Hue[0.16180360112036146`, 1, 1], LineBox[{233, 234}]}, 
        {Hue[0.30347340649352716`, 1, 1], LineBox[{234, 235}]}, 
        {Hue[0.35228833642655205`, 1, 1], LineBox[{235, 236}]}, 
        {Hue[0.5320890788639869, 1, 1], LineBox[{236, 237}]}, 
        {Hue[0.7606664269256596, 1, 1], LineBox[{237, 238}]}, 
        {Hue[0.00496704327457589, 1, 1], LineBox[{238, 239}]}, 
        {Hue[0.26145708925610656`, 1, 1], LineBox[{239, 240}]}, 
        {Hue[0.543029142422131, 1, 1], LineBox[{240, 241}]}, 
        {Hue[0.8648449595129252, 1, 1], LineBox[{241, 242}]}, 
        {Hue[0.16854952740027157`, 1, 1], LineBox[{242, 243}]}, 
        {Hue[0.43839966896805005`, 1, 1], LineBox[{243, 244}]}, 
        {Hue[0.7007213800317906, 1, 1], LineBox[{244, 245}]}, 
        {Hue[0.9650895094938333, 1, 1], LineBox[{245, 246}]}, 
        {Hue[0.23385603595579127`, 1, 1], LineBox[{246, 247}]}, 
        {Hue[0.5085921548983394, 1, 1], LineBox[{247, 248}]}, 
        {Hue[0.7925154147969324, 1, 1], LineBox[{248, 249}]}, 
        {Hue[0.09250870720938896, 1, 1], LineBox[{249, 250}]}, 
        {Hue[0.417902691531048, 1, 1], LineBox[{250, 251}]}}}},
     VertexColors->{
       Hue[0.10248796621842285`, 1, 1], 
       Hue[0.8290457167148891, 1, 1], 
       Hue[0.8703632125845846, 1, 1], 
       Hue[0.7393175820802007, 1, 1], 
       Hue[0.6032998790616205, 1, 1], 
       Hue[0.5291282883599169, 1, 1], 
       Hue[0.37551039951532045`, 1, 1], 
       Hue[0.22433259918415324`, 1, 1], 
       Hue[0.11852503188071434`, 1, 1], 
       Hue[0.07197136740119216, 1, 1], 
       Hue[0.027235479998395193`, 1, 1], 
       Hue[0.9072061553893599, 1, 1], 
       Hue[0.8405170901747704, 1, 1], 
       Hue[0.8075393950498422, 1, 1], 
       Hue[0.5331074488604866, 1, 1], 
       Hue[0.5358981991195568, 1, 1], 
       Hue[0.25004222875796983`, 1, 1], 
       Hue[0.7962568815322872, 1, 1], 
       Hue[0.8602643648636271, 1, 1], 
       Hue[0.7061829013619123, 1, 1], 
       Hue[0.5353893983812265, 1, 1], 
       Hue[0.5574883411356903, 1, 1], 
       Hue[0.3982347746773217, 1, 1], 
       Hue[0.376614336574381, 1, 1], 
       Hue[0.1848960953069641, 1, 1], 
       Hue[0.39778971853907324`, 1, 1], 
       Hue[0.09352019075476536, 1, 1], 
       Hue[0.783376412070276, 1, 1], 
       Hue[0.7824547112898794, 1, 1], 
       Hue[0.7761971219426127, 1, 1], 
       Hue[0.6038072405559677, 1, 1], 
       Hue[0.560930797116872, 1, 1], 
       Hue[0.33776760760892577`, 1, 1], 
       Hue[0.35334587499948134`, 1, 1], 
       Hue[0.06870296802393036, 1, 1], 
       Hue[0.6821357962793559, 1, 1], 
       Hue[0.6351997075059883, 1, 1], 
       Hue[0.40994418070536254`, 1, 1], 
       Hue[0.6600956815350549, 1, 1], 
       Hue[0.3763685961693272, 1, 1], 
       Hue[0.0525203213503501, 1, 1], 
       Hue[0.9908307255976239, 1, 1], 
       Hue[0.009823911060358891, 1, 1], 
       Hue[0.8095599146726241, 1, 1], 
       Hue[0.6425002953713946, 1, 1], 
       Hue[0.6328654713099459, 1, 1], 
       Hue[0.6176065487895835, 1, 1], 
       Hue[0.491800422525271, 1, 1], 
       Hue[0.3386878267347081, 1, 1], 
       Hue[0.19665319598299283`, 1, 1], 
       Hue[0.0021306817817562038`, 1, 1], 
       Hue[0.8729735920333851, 1, 1], 
       Hue[0.7003776259936819, 1, 1], 
       Hue[0.6587624394900979, 1, 1], 
       Hue[0.46120340580633984`, 1, 1], 
       Hue[0.36582744313624216`, 1, 1], 
       Hue[0.36577014645724054`, 1, 1], 
       Hue[0.2768708630533676, 1, 1], 
       Hue[0.401572703318089, 1, 1], 
       Hue[0.4543051107625647, 1, 1], 
       Hue[0.4950883665628377, 1, 1], 
       Hue[0.5727212045820049, 1, 1], 
       Hue[0.6144593839253265, 1, 1], 
       Hue[0.650777198099475, 1, 1], 
       Hue[0.6812023476718669, 1, 1], 
       Hue[0.7366775496335508, 1, 1], 
       Hue[0.8040407063237689, 1, 1], 
       Hue[0.8751337841577034, 1, 1], 
       Hue[0.9349979895740793, 1, 1], 
       Hue[0.9831611795516255, 1, 1], 
       Hue[0.03690540732882161, 1, 1], 
       Hue[0.09340805898442196, 1, 1], 
       Hue[0.15555150481302726`, 1, 1], 
       Hue[0.22260102211802169`, 1, 1], 
       Hue[0.2874873707395083, 1, 1], 
       Hue[0.34371795285997764`, 1, 1], 
       Hue[0.3987686908705344, 1, 1], 
       Hue[0.45584195734129307`, 1, 1], 
       Hue[0.5194171517525992, 1, 1], 
       Hue[0.5871948626381034, 1, 1], 
       Hue[0.6550071558423163, 1, 1], 
       Hue[0.7169971729064637, 1, 1], 
       Hue[0.7758915686233734, 1, 1], 
       Hue[0.8351048410044376, 1, 1], 
       Hue[0.8997708000560913, 1, 1], 
       Hue[0.9703622628350036, 1, 1], 
       Hue[0.04204741694621801, 1, 1], 
       Hue[0.1084845737470324, 1, 1], 
       Hue[0.1701666187456011, 1, 1], 
       Hue[0.2313775399901313, 1, 1], 
       Hue[0.2976877502367062, 1, 1], 
       Hue[0.3711740550744846, 1, 1], 
       Hue[0.44687625426845595`, 1, 1], 
       Hue[0.5176126759525406, 1, 1], 
       Hue[0.5819664960418984, 1, 1], 
       Hue[0.6448711364223789, 1, 1], 
       Hue[0.7127443005598428, 1, 1], 
       Hue[0.7888911244253934, 1, 1], 
       Hue[0.8685972235020938, 1, 1], 
       Hue[0.9429413869593951, 1, 1], 
       Hue[0.009597528089488936, 1, 1], 
       Hue[0.07369556643187233, 1, 1], 
       Hue[0.14259002452799643`, 1, 1], 
       Hue[0.2208055343087441, 1, 1], 
       Hue[0.30397005118067305`, 1, 1], 
       Hue[0.3814022118325686, 1, 1], 
       Hue[0.44958930198492464`, 1, 1], 
       Hue[0.5137878132576624, 1, 1], 
       Hue[0.5819989623965363, 1, 1], 
       Hue[0.6601775740658046, 1, 1], 
       Hue[0.745202837419231, 1, 1], 
       Hue[0.8250475018811899, 1, 1], 
       Hue[0.8938729371910848, 1, 1], 
       Hue[0.9561034162561497, 1, 1], 
       Hue[0.019638189549080177`, 1, 1], 
       Hue[0.09164212060033193, 1, 1], 
       Hue[0.17317222581323283`, 1, 1], 
       Hue[0.2541781134560693, 1, 1], 
       Hue[0.3241269011239155, 1, 1], 
       Hue[0.3830525677867494, 1, 1], 
       Hue[0.43671758530245763`, 1, 1], 
       Hue[0.491429567541259, 1, 1], 
       Hue[0.5524263664053911, 1, 1], 
       Hue[0.6213032633035116, 1, 1], 
       Hue[0.6922386416207904, 1, 1], 
       Hue[0.7556807223227779, 1, 1], 
       Hue[0.8075261375163918, 1, 1], 
       Hue[0.8494469370502941, 1, 1], 
       Hue[0.8845456784920832, 1, 1], 
       Hue[0.9153607707770892, 1, 1], 
       Hue[0.9436483964312276, 1, 1], 
       Hue[0.9705913152413616, 1, 1], 
       Hue[0.9969991385198878, 1, 1], 
       Hue[0.023443344689877202`, 1, 1], 
       Hue[0.0503392189712252, 1, 1], 
       Hue[0.07799583048219152, 1, 1], 
       Hue[0.10664739890542389`, 1, 1], 
       Hue[0.13647223751425389`, 1, 1], 
       Hue[0.16760651012276648`, 1, 1], 
       Hue[0.20015193386020552`, 1, 1], 
       Hue[0.2341836723124326, 1, 1], 
       Hue[0.2697549633000815, 1, 1], 
       Hue[0.3069010347031737, 1, 1], 
       Hue[0.34564242381955096`, 1, 1], 
       Hue[0.385988114107708, 1, 1], 
       Hue[0.4279373960933569, 1, 1], 
       Hue[0.4714821138884691, 1, 1], 
       Hue[0.516608142439269, 1, 1], 
       Hue[0.5632967986431676, 1, 1], 
       Hue[0.6115262291158869, 1, 1], 
       Hue[0.6612723650240759, 1, 1], 
       Hue[0.7125082110086003, 1, 1], 
       Hue[0.7652067245285928, 1, 1], 
       Hue[0.8193404899939781, 1, 1], 
       Hue[0.8748814082258843, 1, 1], 
       Hue[0.93180121767383, 1, 1], 
       Hue[0.9900732339915105, 1, 1], 
       Hue[0.04966897366675908, 1, 1], 
       Hue[0.11056355278904713`, 1, 1], 
       Hue[0.17273127731108673`, 1, 1], 
       Hue[0.23614688428905453`, 1, 1], 
       Hue[0.30078631795382943`, 1, 1], 
       Hue[0.36662673011340297`, 1, 1], 
       Hue[0.4336462625630184, 1, 1], 
       Hue[0.5018243590466764, 1, 1], 
       Hue[0.5711391738638567, 1, 1], 
       Hue[0.6415712630612694, 1, 1], 
       Hue[0.7131026687380215, 1, 1], 
       Hue[0.785714952794411, 1, 1], 
       Hue[0.85939125988999, 1, 1], 
       Hue[0.9341142030507813, 1, 1], 
       Hue[0.009867814283582065, 1, 1], 
       Hue[0.08663975672705043, 1, 1], 
       Hue[0.16441331632110967`, 1, 1], 
       Hue[0.24317140299829, 1, 1], 
       Hue[0.32290236585478704`, 1, 1], 
       Hue[0.4035943587363709, 1, 1], 
       Hue[0.48523999204304397`, 1, 1], 
       Hue[0.5678193374677074, 1, 1], 
       Hue[0.6513185536279276, 1, 1], 
       Hue[0.7357401903243395, 1, 1], 
       Hue[0.8210622956913834, 1, 1], 
       Hue[0.9072682139525383, 1, 1], 
       Hue[0.9943604247210516, 1, 1], 
       Hue[0.08232920234124495, 1, 1], 
       Hue[0.17115182211425745`, 1, 1], 
       Hue[0.2608259833213761, 1, 1], 
       Hue[0.3513592109082063, 1, 1], 
       Hue[0.44272159494008767`, 1, 1], 
       Hue[0.5348962505409033, 1, 1], 
       Hue[0.6279072641081033, 1, 1], 
       Hue[0.7217267880302329, 1, 1], 
       Hue[0.816313267532884, 1, 1], 
       Hue[0.9117512965679803, 1, 1], 
       Hue[0.008047879736664992, 1, 1], 
       Hue[0.10501187209075581`, 1, 1], 
       Hue[0.20270535185021696`, 1, 1], 
       Hue[0.30133780047475683`, 1, 1], 
       Hue[0.40068737757147616`, 1, 1], 
       Hue[0.500621695745771, 1, 1], 
       Hue[0.6015194562805606, 1, 1], 
       Hue[0.7033866071661006, 1, 1], 
       Hue[0.8057600247529666, 1, 1], 
       Hue[0.9087115831851035, 1, 1], 
       Hue[0.012728494365962887`, 1, 1], 
       Hue[0.11768230946122678`, 1, 1], 
       Hue[0.22299007300419238`, 1, 1], 
       Hue[0.32878942455954685`, 1, 1], 
       Hue[0.43566326944187145`, 1, 1], 
       Hue[0.5430660387149626, 1, 1], 
       Hue[0.6501242831582152, 1, 1], 
       Hue[0.758048676916268, 1, 1], 
       Hue[0.8686784353266866, 1, 1], 
       Hue[0.9804881391600404, 1, 1], 
       Hue[0.08932909517187086, 1, 1], 
       Hue[0.1955941259474388, 1, 1], 
       Hue[0.30741713430835516`, 1, 1], 
       Hue[0.4270155355366292, 1, 1], 
       Hue[0.5429924302256508, 1, 1], 
       Hue[0.6483558235535518, 1, 1], 
       Hue[0.7667154238986771, 1, 1], 
       Hue[0.9142402763382171, 1, 1], 
       Hue[0.07850281472384095, 1, 1], 
       Hue[0.2333380105758965, 1, 1], 
       Hue[0.23584220459241986`, 1, 1], 
       Hue[0.2548056612472624, 1, 1], 
       Hue[0.41845090004777136`, 1, 1], 
       Hue[0.5707092498069801, 1, 1], 
       Hue[0.33232299147995975`, 1, 1], 
       Hue[0.531991234171352, 1, 1], 
       Hue[0.7431920582950459, 1, 1], 
       Hue[0.9572424609776871, 1, 1], 
       Hue[0.16180360112036146`, 1, 1], 
       Hue[0.30347340649352716`, 1, 1], 
       Hue[0.35228833642655205`, 1, 1], 
       Hue[0.5320890788639869, 1, 1], 
       Hue[0.7606664269256596, 1, 1], 
       Hue[0.00496704327457589, 1, 1], 
       Hue[0.26145708925610656`, 1, 1], 
       Hue[0.543029142422131, 1, 1], 
       Hue[0.8648449595129252, 1, 1], 
       Hue[0.16854952740027157`, 1, 1], 
       Hue[0.43839966896805005`, 1, 1], 
       Hue[0.7007213800317906, 1, 1], 
       Hue[0.9650895094938333, 1, 1], 
       Hue[0.23385603595579127`, 1, 1], 
       Hue[0.5085921548983394, 1, 1], 
       Hue[0.7925154147969324, 1, 1], 
       Hue[0.09250870720938896, 1, 1], 
       Hue[0.417902691531048, 1, 1], 
       Hue[0.7584781495977884, 1, 1], 
       Hue[0.10248796621842285`, 1, 1], 
       Hue[0.8290457167148891, 1, 1], 
       Hue[0.8290457167148891, 1, 1], 
       Hue[0.8703632125845846, 1, 1], 
       Hue[0.8703632125845846, 1, 1], 
       Hue[0.7393175820802007, 1, 1], 
       Hue[0.7393175820802007, 1, 1], 
       Hue[0.6032998790616205, 1, 1], 
       Hue[0.6032998790616205, 1, 1], 
       Hue[0.5291282883599169, 1, 1], 
       Hue[0.5291282883599169, 1, 1], 
       Hue[0.37551039951532045`, 1, 1], 
       Hue[0.37551039951532045`, 1, 1], 
       Hue[0.22433259918415324`, 1, 1], 
       Hue[0.22433259918415324`, 1, 1], 
       Hue[0.11852503188071434`, 1, 1], 
       Hue[0.11852503188071434`, 1, 1], 
       Hue[0.07197136740119216, 1, 1], 
       Hue[0.07197136740119216, 1, 1], 
       Hue[0.027235479998395193`, 1, 1], 
       Hue[0.027235479998395193`, 1, 1], 
       Hue[0.9072061553893599, 1, 1], 
       Hue[0.9072061553893599, 1, 1], 
       Hue[0.8405170901747704, 1, 1], 
       Hue[0.8405170901747704, 1, 1], 
       Hue[0.8075393950498422, 1, 1], 
       Hue[0.8075393950498422, 1, 1], 
       Hue[0.5331074488604866, 1, 1], 
       Hue[0.5331074488604866, 1, 1], 
       Hue[0.5358981991195568, 1, 1], 
       Hue[0.5358981991195568, 1, 1], 
       Hue[0.25004222875796983`, 1, 1], 
       Hue[0.25004222875796983`, 1, 1], 
       Hue[0.7962568815322872, 1, 1], 
       Hue[0.7962568815322872, 1, 1], 
       Hue[0.8602643648636271, 1, 1], 
       Hue[0.8602643648636271, 1, 1], 
       Hue[0.7061829013619123, 1, 1], 
       Hue[0.7061829013619123, 1, 1], 
       Hue[0.5353893983812265, 1, 1], 
       Hue[0.5353893983812265, 1, 1], 
       Hue[0.5574883411356903, 1, 1], 
       Hue[0.5574883411356903, 1, 1], 
       Hue[0.3982347746773217, 1, 1], 
       Hue[0.3982347746773217, 1, 1], 
       Hue[0.376614336574381, 1, 1], 
       Hue[0.376614336574381, 1, 1], 
       Hue[0.1848960953069641, 1, 1], 
       Hue[0.1848960953069641, 1, 1], 
       Hue[0.39778971853907324`, 1, 1], 
       Hue[0.39778971853907324`, 1, 1], 
       Hue[0.09352019075476536, 1, 1], 
       Hue[0.09352019075476536, 1, 1], 
       Hue[0.783376412070276, 1, 1], 
       Hue[0.783376412070276, 1, 1], 
       Hue[0.7824547112898794, 1, 1], 
       Hue[0.7824547112898794, 1, 1], 
       Hue[0.7761971219426127, 1, 1], 
       Hue[0.7761971219426127, 1, 1], 
       Hue[0.6038072405559677, 1, 1], 
       Hue[0.6038072405559677, 1, 1], 
       Hue[0.560930797116872, 1, 1], 
       Hue[0.560930797116872, 1, 1], 
       Hue[0.33776760760892577`, 1, 1], 
       Hue[0.33776760760892577`, 1, 1], 
       Hue[0.35334587499948134`, 1, 1], 
       Hue[0.35334587499948134`, 1, 1], 
       Hue[0.06870296802393036, 1, 1], 
       Hue[0.06870296802393036, 1, 1], 
       Hue[0.6821357962793559, 1, 1], 
       Hue[0.6821357962793559, 1, 1], 
       Hue[0.6351997075059883, 1, 1], 
       Hue[0.6351997075059883, 1, 1], 
       Hue[0.40994418070536254`, 1, 1], 
       Hue[0.40994418070536254`, 1, 1], 
       Hue[0.6600956815350549, 1, 1], 
       Hue[0.6600956815350549, 1, 1], 
       Hue[0.3763685961693272, 1, 1], 
       Hue[0.3763685961693272, 1, 1], 
       Hue[0.0525203213503501, 1, 1], 
       Hue[0.0525203213503501, 1, 1], 
       Hue[0.9908307255976239, 1, 1], 
       Hue[0.9908307255976239, 1, 1], 
       Hue[0.009823911060358891, 1, 1], 
       Hue[0.009823911060358891, 1, 1], 
       Hue[0.8095599146726241, 1, 1], 
       Hue[0.8095599146726241, 1, 1], 
       Hue[0.6425002953713946, 1, 1], 
       Hue[0.6425002953713946, 1, 1], 
       Hue[0.6328654713099459, 1, 1], 
       Hue[0.6328654713099459, 1, 1], 
       Hue[0.6176065487895835, 1, 1], 
       Hue[0.6176065487895835, 1, 1], 
       Hue[0.491800422525271, 1, 1], 
       Hue[0.491800422525271, 1, 1], 
       Hue[0.3386878267347081, 1, 1], 
       Hue[0.3386878267347081, 1, 1], 
       Hue[0.19665319598299283`, 1, 1], 
       Hue[0.19665319598299283`, 1, 1], 
       Hue[0.0021306817817562038`, 1, 1], 
       Hue[0.0021306817817562038`, 1, 1], 
       Hue[0.8729735920333851, 1, 1], 
       Hue[0.8729735920333851, 1, 1], 
       Hue[0.7003776259936819, 1, 1], 
       Hue[0.7003776259936819, 1, 1], 
       Hue[0.6587624394900979, 1, 1], 
       Hue[0.6587624394900979, 1, 1], 
       Hue[0.46120340580633984`, 1, 1], 
       Hue[0.46120340580633984`, 1, 1], 
       Hue[0.36582744313624216`, 1, 1], 
       Hue[0.36582744313624216`, 1, 1], 
       Hue[0.36577014645724054`, 1, 1], 
       Hue[0.36577014645724054`, 1, 1], 
       Hue[0.2768708630533676, 1, 1], 
       Hue[0.2768708630533676, 1, 1], 
       Hue[0.401572703318089, 1, 1], 
       Hue[0.401572703318089, 1, 1], 
       Hue[0.4543051107625647, 1, 1], 
       Hue[0.4543051107625647, 1, 1], 
       Hue[0.4950883665628377, 1, 1], 
       Hue[0.4950883665628377, 1, 1], 
       Hue[0.5727212045820049, 1, 1], 
       Hue[0.5727212045820049, 1, 1], 
       Hue[0.6144593839253265, 1, 1], 
       Hue[0.6144593839253265, 1, 1], 
       Hue[0.650777198099475, 1, 1], 
       Hue[0.650777198099475, 1, 1], 
       Hue[0.6812023476718669, 1, 1], 
       Hue[0.6812023476718669, 1, 1], 
       Hue[0.7366775496335508, 1, 1], 
       Hue[0.7366775496335508, 1, 1], 
       Hue[0.8040407063237689, 1, 1], 
       Hue[0.8040407063237689, 1, 1], 
       Hue[0.8751337841577034, 1, 1], 
       Hue[0.8751337841577034, 1, 1], 
       Hue[0.9349979895740793, 1, 1], 
       Hue[0.9349979895740793, 1, 1], 
       Hue[0.9831611795516255, 1, 1], 
       Hue[0.9831611795516255, 1, 1], 
       Hue[0.03690540732882161, 1, 1], 
       Hue[0.03690540732882161, 1, 1], 
       Hue[0.09340805898442196, 1, 1], 
       Hue[0.09340805898442196, 1, 1], 
       Hue[0.15555150481302726`, 1, 1], 
       Hue[0.15555150481302726`, 1, 1], 
       Hue[0.22260102211802169`, 1, 1], 
       Hue[0.22260102211802169`, 1, 1], 
       Hue[0.2874873707395083, 1, 1], 
       Hue[0.2874873707395083, 1, 1], 
       Hue[0.34371795285997764`, 1, 1], 
       Hue[0.34371795285997764`, 1, 1], 
       Hue[0.3987686908705344, 1, 1], 
       Hue[0.3987686908705344, 1, 1], 
       Hue[0.45584195734129307`, 1, 1], 
       Hue[0.45584195734129307`, 1, 1], 
       Hue[0.5194171517525992, 1, 1], 
       Hue[0.5194171517525992, 1, 1], 
       Hue[0.5871948626381034, 1, 1], 
       Hue[0.5871948626381034, 1, 1], 
       Hue[0.6550071558423163, 1, 1], 
       Hue[0.6550071558423163, 1, 1], 
       Hue[0.7169971729064637, 1, 1], 
       Hue[0.7169971729064637, 1, 1], 
       Hue[0.7758915686233734, 1, 1], 
       Hue[0.7758915686233734, 1, 1], 
       Hue[0.8351048410044376, 1, 1], 
       Hue[0.8351048410044376, 1, 1], 
       Hue[0.8997708000560913, 1, 1], 
       Hue[0.8997708000560913, 1, 1], 
       Hue[0.9703622628350036, 1, 1], 
       Hue[0.9703622628350036, 1, 1], 
       Hue[0.04204741694621801, 1, 1], 
       Hue[0.04204741694621801, 1, 1], 
       Hue[0.1084845737470324, 1, 1], 
       Hue[0.1084845737470324, 1, 1], 
       Hue[0.1701666187456011, 1, 1], 
       Hue[0.1701666187456011, 1, 1], 
       Hue[0.2313775399901313, 1, 1], 
       Hue[0.2313775399901313, 1, 1], 
       Hue[0.2976877502367062, 1, 1], 
       Hue[0.2976877502367062, 1, 1], 
       Hue[0.3711740550744846, 1, 1], 
       Hue[0.3711740550744846, 1, 1], 
       Hue[0.44687625426845595`, 1, 1], 
       Hue[0.44687625426845595`, 1, 1], 
       Hue[0.5176126759525406, 1, 1], 
       Hue[0.5176126759525406, 1, 1], 
       Hue[0.5819664960418984, 1, 1], 
       Hue[0.5819664960418984, 1, 1], 
       Hue[0.6448711364223789, 1, 1], 
       Hue[0.6448711364223789, 1, 1], 
       Hue[0.7127443005598428, 1, 1], 
       Hue[0.7127443005598428, 1, 1], 
       Hue[0.7888911244253934, 1, 1], 
       Hue[0.7888911244253934, 1, 1], 
       Hue[0.8685972235020938, 1, 1], 
       Hue[0.8685972235020938, 1, 1], 
       Hue[0.9429413869593951, 1, 1], 
       Hue[0.9429413869593951, 1, 1], 
       Hue[0.009597528089488936, 1, 1], 
       Hue[0.009597528089488936, 1, 1], 
       Hue[0.07369556643187233, 1, 1], 
       Hue[0.07369556643187233, 1, 1], 
       Hue[0.14259002452799643`, 1, 1], 
       Hue[0.14259002452799643`, 1, 1], 
       Hue[0.2208055343087441, 1, 1], 
       Hue[0.2208055343087441, 1, 1], 
       Hue[0.30397005118067305`, 1, 1], 
       Hue[0.30397005118067305`, 1, 1], 
       Hue[0.3814022118325686, 1, 1], 
       Hue[0.3814022118325686, 1, 1], 
       Hue[0.44958930198492464`, 1, 1], 
       Hue[0.44958930198492464`, 1, 1], 
       Hue[0.5137878132576624, 1, 1], 
       Hue[0.5137878132576624, 1, 1], 
       Hue[0.5819989623965363, 1, 1], 
       Hue[0.5819989623965363, 1, 1], 
       Hue[0.6601775740658046, 1, 1], 
       Hue[0.6601775740658046, 1, 1], 
       Hue[0.745202837419231, 1, 1], 
       Hue[0.745202837419231, 1, 1], 
       Hue[0.8250475018811899, 1, 1], 
       Hue[0.8250475018811899, 1, 1], 
       Hue[0.8938729371910848, 1, 1], 
       Hue[0.8938729371910848, 1, 1], 
       Hue[0.9561034162561497, 1, 1], 
       Hue[0.9561034162561497, 1, 1], 
       Hue[0.019638189549080177`, 1, 1], 
       Hue[0.019638189549080177`, 1, 1], 
       Hue[0.09164212060033193, 1, 1], 
       Hue[0.09164212060033193, 1, 1], 
       Hue[0.17317222581323283`, 1, 1], 
       Hue[0.17317222581323283`, 1, 1], 
       Hue[0.2541781134560693, 1, 1], 
       Hue[0.2541781134560693, 1, 1], 
       Hue[0.3241269011239155, 1, 1], 
       Hue[0.3241269011239155, 1, 1], 
       Hue[0.3830525677867494, 1, 1], 
       Hue[0.3830525677867494, 1, 1], 
       Hue[0.43671758530245763`, 1, 1], 
       Hue[0.43671758530245763`, 1, 1], 
       Hue[0.491429567541259, 1, 1], 
       Hue[0.491429567541259, 1, 1], 
       Hue[0.5524263664053911, 1, 1], 
       Hue[0.5524263664053911, 1, 1], 
       Hue[0.6213032633035116, 1, 1], 
       Hue[0.6213032633035116, 1, 1], 
       Hue[0.6922386416207904, 1, 1], 
       Hue[0.6922386416207904, 1, 1], 
       Hue[0.7556807223227779, 1, 1], 
       Hue[0.7556807223227779, 1, 1], 
       Hue[0.8075261375163918, 1, 1], 
       Hue[0.8075261375163918, 1, 1], 
       Hue[0.8494469370502941, 1, 1], 
       Hue[0.8494469370502941, 1, 1], 
       Hue[0.8845456784920832, 1, 1], 
       Hue[0.8845456784920832, 1, 1], 
       Hue[0.9153607707770892, 1, 1], 
       Hue[0.9153607707770892, 1, 1], 
       Hue[0.9436483964312276, 1, 1], 
       Hue[0.9436483964312276, 1, 1], 
       Hue[0.9705913152413616, 1, 1], 
       Hue[0.9705913152413616, 1, 1], 
       Hue[0.9969991385198878, 1, 1], 
       Hue[0.9969991385198878, 1, 1], 
       Hue[0.023443344689877202`, 1, 1], 
       Hue[0.023443344689877202`, 1, 1], 
       Hue[0.0503392189712252, 1, 1], 
       Hue[0.0503392189712252, 1, 1], 
       Hue[0.07799583048219152, 1, 1], 
       Hue[0.07799583048219152, 1, 1], 
       Hue[0.10664739890542389`, 1, 1], 
       Hue[0.10664739890542389`, 1, 1], 
       Hue[0.13647223751425389`, 1, 1], 
       Hue[0.13647223751425389`, 1, 1], 
       Hue[0.16760651012276648`, 1, 1], 
       Hue[0.16760651012276648`, 1, 1], 
       Hue[0.20015193386020552`, 1, 1], 
       Hue[0.20015193386020552`, 1, 1], 
       Hue[0.2341836723124326, 1, 1], 
       Hue[0.2341836723124326, 1, 1], 
       Hue[0.2697549633000815, 1, 1], 
       Hue[0.2697549633000815, 1, 1], 
       Hue[0.3069010347031737, 1, 1], 
       Hue[0.3069010347031737, 1, 1], 
       Hue[0.34564242381955096`, 1, 1], 
       Hue[0.34564242381955096`, 1, 1], 
       Hue[0.385988114107708, 1, 1], 
       Hue[0.385988114107708, 1, 1], 
       Hue[0.4279373960933569, 1, 1], 
       Hue[0.4279373960933569, 1, 1], 
       Hue[0.4714821138884691, 1, 1], 
       Hue[0.4714821138884691, 1, 1], 
       Hue[0.516608142439269, 1, 1], 
       Hue[0.516608142439269, 1, 1], 
       Hue[0.5632967986431676, 1, 1], 
       Hue[0.5632967986431676, 1, 1], 
       Hue[0.6115262291158869, 1, 1], 
       Hue[0.6115262291158869, 1, 1], 
       Hue[0.6612723650240759, 1, 1], 
       Hue[0.6612723650240759, 1, 1], 
       Hue[0.7125082110086003, 1, 1], 
       Hue[0.7125082110086003, 1, 1], 
       Hue[0.7652067245285928, 1, 1], 
       Hue[0.7652067245285928, 1, 1], 
       Hue[0.8193404899939781, 1, 1], 
       Hue[0.8193404899939781, 1, 1], 
       Hue[0.8748814082258843, 1, 1], 
       Hue[0.8748814082258843, 1, 1], 
       Hue[0.93180121767383, 1, 1], 
       Hue[0.93180121767383, 1, 1], 
       Hue[0.9900732339915105, 1, 1], 
       Hue[0.9900732339915105, 1, 1], 
       Hue[0.04966897366675908, 1, 1], 
       Hue[0.04966897366675908, 1, 1], 
       Hue[0.11056355278904713`, 1, 1], 
       Hue[0.11056355278904713`, 1, 1], 
       Hue[0.17273127731108673`, 1, 1], 
       Hue[0.17273127731108673`, 1, 1], 
       Hue[0.23614688428905453`, 1, 1], 
       Hue[0.23614688428905453`, 1, 1], 
       Hue[0.30078631795382943`, 1, 1], 
       Hue[0.30078631795382943`, 1, 1], 
       Hue[0.36662673011340297`, 1, 1], 
       Hue[0.36662673011340297`, 1, 1], 
       Hue[0.4336462625630184, 1, 1], 
       Hue[0.4336462625630184, 1, 1], 
       Hue[0.5018243590466764, 1, 1], 
       Hue[0.5018243590466764, 1, 1], 
       Hue[0.5711391738638567, 1, 1], 
       Hue[0.5711391738638567, 1, 1], 
       Hue[0.6415712630612694, 1, 1], 
       Hue[0.6415712630612694, 1, 1], 
       Hue[0.7131026687380215, 1, 1], 
       Hue[0.7131026687380215, 1, 1], 
       Hue[0.785714952794411, 1, 1], 
       Hue[0.785714952794411, 1, 1], 
       Hue[0.85939125988999, 1, 1], 
       Hue[0.85939125988999, 1, 1], 
       Hue[0.9341142030507813, 1, 1], 
       Hue[0.9341142030507813, 1, 1], 
       Hue[0.009867814283582065, 1, 1], 
       Hue[0.009867814283582065, 1, 1], 
       Hue[0.08663975672705043, 1, 1], 
       Hue[0.08663975672705043, 1, 1], 
       Hue[0.16441331632110967`, 1, 1], 
       Hue[0.16441331632110967`, 1, 1], 
       Hue[0.24317140299829, 1, 1], 
       Hue[0.24317140299829, 1, 1], 
       Hue[0.32290236585478704`, 1, 1], 
       Hue[0.32290236585478704`, 1, 1], 
       Hue[0.4035943587363709, 1, 1], 
       Hue[0.4035943587363709, 1, 1], 
       Hue[0.48523999204304397`, 1, 1], 
       Hue[0.48523999204304397`, 1, 1], 
       Hue[0.5678193374677074, 1, 1], 
       Hue[0.5678193374677074, 1, 1], 
       Hue[0.6513185536279276, 1, 1], 
       Hue[0.6513185536279276, 1, 1], 
       Hue[0.7357401903243395, 1, 1], 
       Hue[0.7357401903243395, 1, 1], 
       Hue[0.8210622956913834, 1, 1], 
       Hue[0.8210622956913834, 1, 1], 
       Hue[0.9072682139525383, 1, 1], 
       Hue[0.9072682139525383, 1, 1], 
       Hue[0.9943604247210516, 1, 1], 
       Hue[0.9943604247210516, 1, 1], 
       Hue[0.08232920234124495, 1, 1], 
       Hue[0.08232920234124495, 1, 1], 
       Hue[0.17115182211425745`, 1, 1], 
       Hue[0.17115182211425745`, 1, 1], 
       Hue[0.2608259833213761, 1, 1], 
       Hue[0.2608259833213761, 1, 1], 
       Hue[0.3513592109082063, 1, 1], 
       Hue[0.3513592109082063, 1, 1], 
       Hue[0.44272159494008767`, 1, 1], 
       Hue[0.44272159494008767`, 1, 1], 
       Hue[0.5348962505409033, 1, 1], 
       Hue[0.5348962505409033, 1, 1], 
       Hue[0.6279072641081033, 1, 1], 
       Hue[0.6279072641081033, 1, 1], 
       Hue[0.7217267880302329, 1, 1], 
       Hue[0.7217267880302329, 1, 1], 
       Hue[0.816313267532884, 1, 1], 
       Hue[0.816313267532884, 1, 1], 
       Hue[0.9117512965679803, 1, 1], 
       Hue[0.9117512965679803, 1, 1], 
       Hue[0.008047879736664992, 1, 1], 
       Hue[0.008047879736664992, 1, 1], 
       Hue[0.10501187209075581`, 1, 1], 
       Hue[0.10501187209075581`, 1, 1], 
       Hue[0.20270535185021696`, 1, 1], 
       Hue[0.20270535185021696`, 1, 1], 
       Hue[0.30133780047475683`, 1, 1], 
       Hue[0.30133780047475683`, 1, 1], 
       Hue[0.40068737757147616`, 1, 1], 
       Hue[0.40068737757147616`, 1, 1], 
       Hue[0.500621695745771, 1, 1], 
       Hue[0.500621695745771, 1, 1], 
       Hue[0.6015194562805606, 1, 1], 
       Hue[0.6015194562805606, 1, 1], 
       Hue[0.7033866071661006, 1, 1], 
       Hue[0.7033866071661006, 1, 1], 
       Hue[0.8057600247529666, 1, 1], 
       Hue[0.8057600247529666, 1, 1], 
       Hue[0.9087115831851035, 1, 1], 
       Hue[0.9087115831851035, 1, 1], 
       Hue[0.012728494365962887`, 1, 1], 
       Hue[0.012728494365962887`, 1, 1], 
       Hue[0.11768230946122678`, 1, 1], 
       Hue[0.11768230946122678`, 1, 1], 
       Hue[0.22299007300419238`, 1, 1], 
       Hue[0.22299007300419238`, 1, 1], 
       Hue[0.32878942455954685`, 1, 1], 
       Hue[0.32878942455954685`, 1, 1], 
       Hue[0.43566326944187145`, 1, 1], 
       Hue[0.43566326944187145`, 1, 1], 
       Hue[0.5430660387149626, 1, 1], 
       Hue[0.5430660387149626, 1, 1], 
       Hue[0.6501242831582152, 1, 1], 
       Hue[0.6501242831582152, 1, 1], 
       Hue[0.758048676916268, 1, 1], 
       Hue[0.758048676916268, 1, 1], 
       Hue[0.8686784353266866, 1, 1], 
       Hue[0.8686784353266866, 1, 1], 
       Hue[0.9804881391600404, 1, 1], 
       Hue[0.9804881391600404, 1, 1], 
       Hue[0.08932909517187086, 1, 1], 
       Hue[0.08932909517187086, 1, 1], 
       Hue[0.1955941259474388, 1, 1], 
       Hue[0.1955941259474388, 1, 1], 
       Hue[0.30741713430835516`, 1, 1], 
       Hue[0.30741713430835516`, 1, 1], 
       Hue[0.4270155355366292, 1, 1], 
       Hue[0.4270155355366292, 1, 1], 
       Hue[0.5429924302256508, 1, 1], 
       Hue[0.5429924302256508, 1, 1], 
       Hue[0.6483558235535518, 1, 1], 
       Hue[0.6483558235535518, 1, 1], 
       Hue[0.7667154238986771, 1, 1], 
       Hue[0.7667154238986771, 1, 1], 
       Hue[0.9142402763382171, 1, 1], 
       Hue[0.9142402763382171, 1, 1], 
       Hue[0.07850281472384095, 1, 1], 
       Hue[0.07850281472384095, 1, 1], 
       Hue[0.2333380105758965, 1, 1], 
       Hue[0.2333380105758965, 1, 1], 
       Hue[0.23584220459241986`, 1, 1], 
       Hue[0.23584220459241986`, 1, 1], 
       Hue[0.2548056612472624, 1, 1], 
       Hue[0.2548056612472624, 1, 1], 
       Hue[0.41845090004777136`, 1, 1], 
       Hue[0.41845090004777136`, 1, 1], 
       Hue[0.5707092498069801, 1, 1], 
       Hue[0.5707092498069801, 1, 1], 
       Hue[0.33232299147995975`, 1, 1], 
       Hue[0.33232299147995975`, 1, 1], 
       Hue[0.531991234171352, 1, 1], 
       Hue[0.531991234171352, 1, 1], 
       Hue[0.7431920582950459, 1, 1], 
       Hue[0.7431920582950459, 1, 1], 
       Hue[0.9572424609776871, 1, 1], 
       Hue[0.9572424609776871, 1, 1], 
       Hue[0.16180360112036146`, 1, 1], 
       Hue[0.16180360112036146`, 1, 1], 
       Hue[0.30347340649352716`, 1, 1], 
       Hue[0.30347340649352716`, 1, 1], 
       Hue[0.35228833642655205`, 1, 1], 
       Hue[0.35228833642655205`, 1, 1], 
       Hue[0.5320890788639869, 1, 1], 
       Hue[0.5320890788639869, 1, 1], 
       Hue[0.7606664269256596, 1, 1], 
       Hue[0.7606664269256596, 1, 1], 
       Hue[0.00496704327457589, 1, 1], 
       Hue[0.00496704327457589, 1, 1], 
       Hue[0.26145708925610656`, 1, 1], 
       Hue[0.26145708925610656`, 1, 1], 
       Hue[0.543029142422131, 1, 1], 
       Hue[0.543029142422131, 1, 1], 
       Hue[0.8648449595129252, 1, 1], 
       Hue[0.8648449595129252, 1, 1], 
       Hue[0.16854952740027157`, 1, 1], 
       Hue[0.16854952740027157`, 1, 1], 
       Hue[0.43839966896805005`, 1, 1], 
       Hue[0.43839966896805005`, 1, 1], 
       Hue[0.7007213800317906, 1, 1], 
       Hue[0.7007213800317906, 1, 1], 
       Hue[0.9650895094938333, 1, 1], 
       Hue[0.9650895094938333, 1, 1], 
       Hue[0.23385603595579127`, 1, 1], 
       Hue[0.23385603595579127`, 1, 1], 
       Hue[0.5085921548983394, 1, 1], 
       Hue[0.5085921548983394, 1, 1], 
       Hue[0.7925154147969324, 1, 1], 
       Hue[0.7925154147969324, 1, 1], 
       Hue[0.09250870720938896, 1, 1], 
       Hue[0.09250870720938896, 1, 1], 
       Hue[0.417902691531048, 1, 1], 
       Hue[0.417902691531048, 1, 1], 
       Hue[0.7584781495977884, 1, 1]}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->{True, None},
    AxesOrigin->{0, 0},
    Epilog->
     LineBox[{{1, 1.3438449168047713`*^-6}, {2, 5.050073771653918*^-7}, {
       3, 6.855638028766111*^-7}, {4, 3.8179203479447143`*^-7}, {
       5, 7.508603360329421*^-7}, {6, 9.863227720421532*^-7}, {
       7, 1.2155460876151615`*^-6}, {8, 1.3764484103715475`*^-6}, {
       9, 8.860643544638477*^-7}, {10, 7.970423102534875*^-7}, {
       11, 7.277582299714451*^-7}, {12, 8.800794860302492*^-7}, {
       13, 3.582363026860292*^-7}, {14, 6.570367993999595*^-7}, {
       15, 4.2600734910275776`*^-7}, {16, 3.117866987224697*^-7}, {
       17, 7.755335491452408*^-8}, {18, 1.7280895051173313`*^-7}, {
       19, 3.238175698959648*^-7}, {20, 6.444259440967693*^-7}, {
       21, 6.001646297483413*^-7}, {22, 5.308526312369224*^-7}, {
       23, 5.501795048657428*^-7}, {24, 5.880886923884123*^-7}, {
       25, 3.6339121006225795`*^-7}, {26, 7.828114432257981*^-8}, {
       27, 4.009478784132785*^-7}, {28, 1.5077047672234011`*^-7}, {
       29, 3.7956738003471573`*^-7}, {30, 3.675254922703899*^-7}, {
       31, 5.729003618224869*^-7}, {32, 4.896950804797941*^-7}, {
       33, 3.6930699272705243`*^-7}, {34, 2.2832008136322795`*^-7}, {
       35, 2.672340564393481*^-7}, {36, 5.297091315972509*^-7}, {
       37, 4.4744785302170775`*^-7}, {38, 4.707307285350399*^-7}, {
       39, 3.2214292679393745`*^-7}, {40, 6.049555476280254*^-7}, {
       41, 3.2998149166183465`*^-7}, {42, 1.3751834669131524`*^-7}, {
       43, 5.158705154512046*^-7}, {44, 8.862478157837731*^-7}, {
       45, 9.80485350079701*^-7}, {46, 1.0092223842856086`*^-6}, {
       47, 8.579114723324283*^-7}, {48, 1.7486290941597078`*^-6}, {
       49, 9.054521848466083*^-7}, {50, 8.259347454515807*^-7}, {
       51, 1.070798592799538*^-6}, {52, 8.190627097875383*^-7}, {
       53, 9.593272749811963*^-7}, {54, 7.478993037142122*^-7}, {
       55, 1.0712710648962929`*^-6}, {56, 1.6523331545411954`*^-6}, {
       57, 1.5654480130518108`*^-6}, {58, 1.9173987565508257`*^-6}, {
       59, 1.4079807461187133`*^-6}, {60, 1.6069212552473312`*^-6}, {
       61, 2.227450433275266*^-6}, {62, 2.487365436638597*^-6}, {
       63, 5.026716832323899*^-6}, {64, 6.0071582606822355`*^-6}, {
       65, 7.72925896247523*^-6}, {66, 7.968046530132231*^-6}, {
       67, 9.750304369810336*^-6}, {68, 0.000012010820018980802`}, {
       69, 0.00001695662743718514}, {70, 0.000021838288456610445`}, {
       71, 0.000027545364569211193`}, {72, 0.00003280517031879668}, {
       73, 0.0000397289539871845}, {74, 0.00004990496590362894}, {
       75, 0.00006507609589356898}, {76, 0.00008400469294037497}, {
       77, 0.00010488977155735191`}, {78, 0.00012595144863133373`}, {
       79, 0.0001504478447275558}, {80, 0.00018302024466741338`}, {
       81, 0.0002306537564104}, {82, 0.00029305570926458355`}, {
       83, 0.0003639439168739834}, {84, 0.0004355366064725881}, {
       85, 0.0005094748215040786}, {86, 0.0006043612252946217}, {
       87, 0.0007452875180149037}, {88, 0.0009368617929305138}, {
       89, 0.0011538333711350818`}, {90, 0.0013649377313992438`}, {
       91, 0.00156766787445566}, {92, 0.0018152004024004707`}, {
       93, 0.0021935312331993633`}, {94, 0.0027275459679547242`}, {
       95, 0.0033369844639424976`}, {96, 0.003904867095906774}, {
       97, 0.004401431136862781}, {98, 0.004978110175082453}, {
       99, 0.005904094500136208}, {100, 0.007275753779572891}, {
       101, 0.008847806987205883}, {102, 0.010248771318140405`}, {
       103, 0.011345142560833464`}, {104, 0.01253285738118848}, {
       105, 0.014599867811876072`}, {106, 0.017867544513828392`}, {
       107, 0.021667703537470377`}, {108, 0.024950541768689296`}, {
       109, 0.027238495986368776`}, {110, 0.02940311931934233}, {
       111, 0.03351230587360265}, {112, 0.040729395517057}, {
       113, 0.049659882684868796`}, {114, 0.057673069959102916`}, {
       115, 0.06308854116158401}, {116, 0.06690215058106808}, {
       117, 0.07339236308727035}, {118, 0.0870608107984569}, {
       119, 0.10753201867699332`}, {120, 0.12995958304616997`}, {
       121, 0.14911920520456104`}, {122, 0.16244801066148032`}, {
       123, 0.1721619288517853}, {124, 0.1862680126406907}, {
       125, 0.21539485625645893`}, {126, 0.264976114350546}, {
       127, 0.3320452268584973}, {128, 0.40947303322547146`}, {
       129, 0.4899994582841429}, {130, 0.5676920888365797}, {
       131, 0.6382087421275998}, {132, 0.6987166706429022}, {
       133, 0.7476863091312034}, {134, 0.7846285759743065}, {
       135, 0.8098267519905125}, {136, 0.8240797004065663}, {
       137, 0.8284917687953577}, {138, 0.8243133093150494}, {
       139, 0.8128235362431939}, {140, 0.7952521187231224}, {
       141, 0.7727324104388693}, {142, 0.7462865979566424}, {
       143, 0.7168179897770113}, {144, 0.6851062078648421}, {
       145, 0.6518299674536496}, {146, 0.6175629545140168}, {
       147, 0.5827943279857422}, {148, 0.5479388160825146}, {
       149, 0.5133441422392822}, {150, 0.4792999587345991}, {
       151, 0.4460434487292549}, {152, 0.4137704736498192}, {
       153, 0.38263950511929795`}, {154, 0.3527714554612603}, {
       155, 0.3242602513218213}, {156, 0.2971724532008716}, {
       157, 0.2715527340965929}, {158, 0.24742353029016012`}, {
       159, 0.2247939525022326}, {160, 0.2036523446691271}, {
       161, 0.18397777905830978`}, {162, 0.16573598265025766`}, {
       163, 0.14888508429895186`}, {164, 0.13337462413585321`}, {
       165, 0.11914782680738818`}, {166, 0.10614276729661636`}, {
       167, 0.09429614807521265}, {168, 0.08354068220910087}, {
       169, 0.07380815169384132}, {170, 0.06503013721849481}, {
       171, 0.05713849101411949}, {172, 0.0500671793562392}, {
       173, 0.04375092686238379}, {174, 0.03812614549723036}, {
       175, 0.033133619668249}, {176, 0.028716380125324337`}, {
       177, 0.024820460704744866`}, {178, 0.021394451298417515`}, {
       179, 0.018390240342439877`}, {180, 0.015765329077248633`}, {
       181, 0.013478310748997345`}, {182, 0.011490914705107165`}, {
       183, 0.00977006632718522}, {184, 0.008284577191326116}, {
       185, 0.007005526316663681}, {186, 0.005907672408679385}, {
       187, 0.004968658789847037}, {188, 0.0041674666728667275`}, {
       189, 0.003485596864708379}, {190, 0.0029075710061982753`}, {
       191, 0.002418950582181375}, {192, 0.0020066691709867504`}, {
       193, 0.001660402413691285}, {194, 0.001370625758092963}, {
       195, 0.0011277504204931037`}, {196, 0.0009250077954023635}, {
       197, 0.0007573544376017891}, {198, 0.0006184492461656033}, {
       199, 0.0005031056181638236}, {200, 0.0004084998824557169}, {
       201, 0.0003311899695683833}, {202, 0.00026737501939824715`}, {
       203, 0.00021498646877285307`}, {204, 0.00017267039016009137`}, {
       205, 0.00013842908104329103`}, {206, 0.00011039992670949446`}, {
       207, 0.00008764947102213703}, {208, 0.00006952916199485672}, {
       209, 0.00005501169866745292}, {210, 0.00004323365295301365}, {
       211, 0.00003393569435602977}, {212, 0.00002681571119707127}, {
       213, 0.00002113504143743491}, {214, 0.00001633126975793672}, {
       215, 0.000012465478675476022`}, {216, 9.765833054529133*^-6}, {
       217, 7.891181340785691*^-6}, {218, 6.189870811177567*^-6}, {
       219, 4.591315917875258*^-6}, {220, 3.560800344707003*^-6}, {
       221, 3.0834467563532345`*^-6}, {222, 2.532692419875134*^-6}, {
       223, 1.6778020217233635`*^-6}, {224, 7.648025130755926*^-7}, {
       225, 2.3711932750827905`*^-7}, {226, 3.7414362600598133`*^-7}, {
       227, 3.3167912302392594`*^-7}, {228, 7.902566195493602*^-8}, {
       229, 2.654603186421689*^-7}, {230, 5.60193173383047*^-7}, {
       231, 7.079174757880079*^-7}, {232, 6.63482538974332*^-7}, {
       233, 4.624047157019311*^-7}, {234, 2.3394632954248082`*^-7}, {
       235, 2.472913142786779*^-7}, {236, 3.69627078493883*^-7}, {
       237, 4.0112951083116867`*^-7}, {238, 3.426030704612548*^-7}, {
       239, 2.425202527781865*^-7}, {240, 1.5532513664155531`*^-7}, {
       241, 1.2833865924653207`*^-7}, {242, 1.7388447828129748`*^-7}, {
       243, 2.5838021233589175`*^-7}, {244, 3.493734919940622*^-7}, {
       245, 4.175911984705985*^-7}, {246, 4.414058280392368*^-7}, {
       247, 4.139526252759938*^-7}, {248, 3.453080914589958*^-7}, {
       249, 2.599588323658625*^-7}, {250, 1.9088605678547244`*^-7}, {
       251, 1.615070789048257*^-7}}],
    Frame->True,
    FrameTicks->{{{1., 
        FormBox[
         RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
        FormBox[
         RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
        FormBox[
         RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
        FormBox[
         RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
        FormBox["0", TraditionalForm]}, {157.25, 
        FormBox["2.5`", TraditionalForm]}, {188.5, 
        FormBox["5", TraditionalForm]}, {219.75, 
        FormBox["7.5`", TraditionalForm]}, {251., 
        FormBox["10", TraditionalForm]}}, Automatic, None, None},
    PlotRange->{-0.1, 1.1},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], Automatic}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF13tcTfkaBvAVEsOhmXEJyc4lI4QpoRl5VKgo3XRVrdpdd7VzHRmD37iO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     "], {
      {GrayLevel[0.5], EdgeForm[None], 
       GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ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         "]]]}, {{}, {}, {
        {Hue[0.46903121084379074`, 1, 1], LineBox[{1, 2}]}, 
        {Hue[0.3276739181280018, 1, 1], LineBox[{2, 3}]}, 
        {Hue[0.27302120989995315`, 1, 1], LineBox[{3, 4}]}, 
        {Hue[0.1503982909211788, 1, 1], LineBox[{4, 5}]}, 
        {Hue[0.1159540677729317, 1, 1], LineBox[{5, 6}]}, 
        {Hue[0.9885034259412886, 1, 1], LineBox[{6, 7}]}, 
        {Hue[0.555919303286334, 1, 1], LineBox[{7, 8}]}, 
        {Hue[0.014674981446927316`, 1, 1], LineBox[{8, 9}]}, 
        {Hue[0.2505963104170238, 1, 1], LineBox[{9, 10}]}, 
        {Hue[0.08114438222060529, 1, 1], LineBox[{10, 11}]}, 
        {Hue[0.01429524804121616, 1, 1], LineBox[{11, 12}]}, 
        {Hue[0.887197895954603, 1, 1], LineBox[{12, 13}]}, 
        {Hue[0.776711378633425, 1, 1], LineBox[{13, 14}]}, 
        {Hue[0.8220325704507755, 1, 1], LineBox[{14, 15}]}, 
        {Hue[0.6796760798270274, 1, 1], LineBox[{15, 16}]}, 
        {Hue[0.7081680584902381, 1, 1], LineBox[{16, 17}]}, 
        {Hue[0.5453095997956825, 1, 1], LineBox[{17, 18}]}, 
        {Hue[0.4431782547061368, 1, 1], LineBox[{18, 19}]}, 
        {Hue[0.24332838511522192`, 1, 1], LineBox[{19, 20}]}, 
        {Hue[0.11867547147282674`, 1, 1], LineBox[{20, 21}]}, 
        {Hue[0.20671099307287558`, 1, 1], LineBox[{21, 22}]}, 
        {Hue[0.874019357415786, 1, 1], LineBox[{22, 23}]}, 
        {Hue[0.6591515413554456, 1, 1], LineBox[{23, 24}]}, 
        {Hue[0.18066228033355544`, 1, 1], LineBox[{24, 25}]}, 
        {Hue[0.6678350196899061, 1, 1], LineBox[{25, 26}]}, 
        {Hue[0.5241258581961699, 1, 1], LineBox[{26, 27}]}, 
        {Hue[0.1431727845363231, 1, 1], LineBox[{27, 28}]}, 
        {Hue[0.7756794556885923, 1, 1], LineBox[{28, 29}]}, 
        {Hue[0.5095140846201118, 1, 1], LineBox[{29, 30}]}, 
        {Hue[0.6861711805072839, 1, 1], LineBox[{30, 31}]}, 
        {Hue[0.8636511930406199, 1, 1], LineBox[{31, 32}]}, 
        {Hue[0.5811421883741066, 1, 1], LineBox[{32, 33}]}, 
        {Hue[0.5145867440569433, 1, 1], LineBox[{33, 34}]}, 
        {Hue[0.3775090338495559, 1, 1], LineBox[{34, 35}]}, 
        {Hue[0.24014184273348146`, 1, 1], LineBox[{35, 36}]}, 
        {Hue[0.0808948788107402, 1, 1], LineBox[{36, 37}]}, 
        {Hue[0.00828126453665834, 1, 1], LineBox[{37, 38}]}, 
        {Hue[0.8486886497933208, 1, 1], LineBox[{38, 39}]}, 
        {Hue[0.7430682886052937, 1, 1], LineBox[{39, 40}]}, 
        {Hue[0.661458391422076, 1, 1], LineBox[{40, 41}]}, 
        {Hue[0.502941759363241, 1, 1], LineBox[{41, 42}]}, 
        {Hue[0.3928074615720261, 1, 1], LineBox[{42, 43}]}, 
        {Hue[0.17436336899653096`, 1, 1], LineBox[{43, 44}]}, 
        {Hue[0.19971063365582253`, 1, 1], LineBox[{44, 45}]}, 
        {Hue[0.9072160164407977, 1, 1], LineBox[{45, 46}]}, 
        {Hue[0.7435128384384582, 1, 1], LineBox[{46, 47}]}, 
        {Hue[0.7542167129026032, 1, 1], LineBox[{47, 48}]}, 
        {Hue[0.7220480636268023, 1, 1], LineBox[{48, 49}]}, 
        {Hue[0.5960170125858861, 1, 1], LineBox[{49, 50}]}, 
        {Hue[0.504394885168866, 1, 1], LineBox[{50, 51}]}, 
        {Hue[0.5980761131160199, 1, 1], LineBox[{51, 52}]}, 
        {Hue[0.39175536296490443`, 1, 1], LineBox[{52, 53}]}, 
        {Hue[0.3080870960109756, 1, 1], LineBox[{53, 54}]}, 
        {Hue[0.22637965615689903`, 1, 1], LineBox[{54, 55}]}, 
        {Hue[0.04078695456709334, 1, 1], LineBox[{55, 56}]}, 
        {Hue[0.9979339064190902, 1, 1], LineBox[{56, 57}]}, 
        {Hue[0.9379513734489464, 1, 1], LineBox[{57, 58}]}, 
        {Hue[0.7568969236098246, 1, 1], LineBox[{58, 59}]}, 
        {Hue[0.6044031428332308, 1, 1], LineBox[{59, 60}]}, 
        {Hue[0.546835631606627, 1, 1], LineBox[{60, 61}]}, 
        {Hue[0.48445950589899756`, 1, 1], LineBox[{61, 62}]}, 
        {Hue[0.45836739143999605`, 1, 1], LineBox[{62, 63}]}, 
        {Hue[0.4050876322941351, 1, 1], LineBox[{63, 64}]}, 
        {Hue[0.3310440859556128, 1, 1], LineBox[{64, 65}]}, 
        {Hue[0.8952884846740966, 1, 1], LineBox[{65, 66}]}, 
        {Hue[0.849914391383057, 1, 1], LineBox[{66, 67}]}, 
        {Hue[0.8210799392004464, 1, 1], LineBox[{67, 68}]}, 
        {Hue[0.797476270966503, 1, 1], LineBox[{68, 69}]}, 
        {Hue[0.7677111204344772, 1, 1], LineBox[{69, 70}]}, 
        {Hue[0.7373996974703723, 1, 1], LineBox[{70, 71}]}, 
        {Hue[0.6751855643100823, 1, 1], LineBox[{71, 72}]}, 
        {Hue[0.24133341544108203`, 1, 1], LineBox[{72, 73}]}, 
        {Hue[0.20414494146012305`, 1, 1], LineBox[{73, 74}]}, 
        {Hue[0.17127988511038536`, 1, 1], LineBox[{74, 75}]}, 
        {Hue[0.15097260410212796`, 1, 1], LineBox[{75, 76}]}, 
        {Hue[0.1323205065914355, 1, 1], LineBox[{76, 77}]}, 
        {Hue[0.11577470259370665`, 1, 1], LineBox[{77, 78}]}, 
        {Hue[0.40072236254851873`, 1, 1], LineBox[{78, 79}]}, 
        {Hue[0.5642972190009667, 1, 1], LineBox[{79, 80}]}, 
        {Hue[0.5486440775627401, 1, 1], LineBox[{80, 81}]}, 
        {Hue[0.5323579238759228, 1, 1], LineBox[{81, 82}]}, 
        {Hue[0.5186050976851513, 1, 1], LineBox[{82, 83}]}, 
        {Hue[0.5076758901256943, 1, 1], LineBox[{83, 84}]}, 
        {Hue[0.5231123375576997, 1, 1], LineBox[{84, 85}]}, 
        {Hue[0.9249542200277258, 1, 1], LineBox[{85, 86}]}, 
        {Hue[0.9326440595614945, 1, 1], LineBox[{86, 87}]}, 
        {Hue[0.92414635791773, 1, 1], LineBox[{87, 88}]}, 
        {Hue[0.914328727297987, 1, 1], LineBox[{88, 89}]}, 
        {Hue[0.9075469174696386, 1, 1], LineBox[{89, 90}]}, 
        {Hue[0.9162986729778786, 1, 1], LineBox[{90, 91}]}, 
        {Hue[0.22438990344755358`, 1, 1], LineBox[{91, 92}]}, 
        {Hue[0.3233701921895654, 1, 1], LineBox[{92, 93}]}, 
        {Hue[0.3255239702566573, 1, 1], LineBox[{93, 94}]}, 
        {Hue[0.32135596500062585`, 1, 1], LineBox[{94, 95}]}, 
        {Hue[0.3185714943521732, 1, 1], LineBox[{95, 96}]}, 
        {Hue[0.3260807234884014, 1, 1], LineBox[{96, 97}]}, 
        {Hue[0.44731094999721865`, 1, 1], LineBox[{97, 98}]}, 
        {Hue[0.718402716585326, 1, 1], LineBox[{98, 99}]}, 
        {Hue[0.7378073209369546, 1, 1], LineBox[{99, 100}]}, 
        {Hue[0.7400657932223309, 1, 1], LineBox[{100, 101}]}, 
        {Hue[0.7412978514163524, 1, 1], LineBox[{101, 102}]}, 
        {Hue[0.7498344855551069, 1, 1], LineBox[{102, 103}]}, 
        {Hue[0.8080132497531595, 1, 1], LineBox[{103, 104}]}, 
        {Hue[0.10678166006413865`, 1, 1], LineBox[{104, 105}]}, 
        {Hue[0.15908580322017676`, 1, 1], LineBox[{105, 106}]}, 
        {Hue[0.16950671056358974`, 1, 1], LineBox[{106, 107}]}, 
        {Hue[0.1750900190849448, 1, 1], LineBox[{107, 108}]}, 
        {Hue[0.18498588973514118`, 1, 1], LineBox[{108, 109}]}, 
        {Hue[0.2205776860799222, 1, 1], LineBox[{109, 110}]}, 
        {Hue[0.4486392775157134, 1, 1], LineBox[{110, 111}]}, 
        {Hue[0.5823447125514417, 1, 1], LineBox[{111, 112}]}, 
        {Hue[0.6062875308462049, 1, 1], LineBox[{112, 113}]}, 
        {Hue[0.6170867116073702, 1, 1], LineBox[{113, 114}]}, 
        {Hue[0.6280375327039411, 1, 1], LineBox[{114, 115}]}, 
        {Hue[0.6498635797223722, 1, 1], LineBox[{115, 116}]}, 
        {Hue[0.7294631065042633, 1, 1], LineBox[{116, 117}]}, 
        {Hue[0.9619442698698356, 1, 1], LineBox[{117, 118}]}, 
        {Hue[0.0358694738259074, 1, 1], LineBox[{118, 119}]}, 
        {Hue[0.058560492456003414`, 1, 1], LineBox[{119, 120}]}, 
        {Hue[0.07200337794989134, 1, 1], LineBox[{120, 121}]}, 
        {Hue[0.08546181681129635, 1, 1], LineBox[{121, 122}]}, 
        {Hue[0.10595949252681901`, 1, 1], LineBox[{122, 123}]}, 
        {Hue[0.15064552410101642`, 1, 1], LineBox[{123, 124}]}, 
        {Hue[0.2776938160586117, 1, 1], LineBox[{124, 125}]}, 
        {Hue[0.42749515722735437`, 1, 1], LineBox[{125, 126}]}, 
        {Hue[0.48338919600598795`, 1, 1], LineBox[{126, 127}]}, 
        {Hue[0.5077457331726007, 1, 1], LineBox[{127, 128}]}, 
        {Hue[0.522453270537981, 1, 1], LineBox[{128, 129}]}, 
        {Hue[0.533643024834802, 1, 1], LineBox[{129, 130}]}, 
        {Hue[0.5436190540030256, 1, 1], LineBox[{130, 131}]}, 
        {Hue[0.5534698627463082, 1, 1], LineBox[{131, 132}]}, 
        {Hue[0.5638043360423671, 1, 1], LineBox[{132, 133}]}, 
        {Hue[0.5750106310394598, 1, 1], LineBox[{133, 134}]}, 
        {Hue[0.5873592564502069, 1, 1], LineBox[{134, 135}]}, 
        {Hue[0.6010505587702185, 1, 1], LineBox[{135, 136}]}, 
        {Hue[0.6162377444445248, 1, 1], LineBox[{136, 137}]}, 
        {Hue[0.633039328584811, 1, 1], LineBox[{137, 138}]}, 
        {Hue[0.6515452342370982, 1, 1], LineBox[{138, 139}]}, 
        {Hue[0.671822399406282, 1, 1], LineBox[{139, 140}]}, 
        {Hue[0.6939174624732731, 1, 1], LineBox[{140, 141}]}, 
        {Hue[0.7178601571527807, 1, 1], LineBox[{141, 142}]}, 
        {Hue[0.743664904009497, 1, 1], LineBox[{142, 143}]}, 
        {Hue[0.7713339799014306, 1, 1], LineBox[{143, 144}]}, 
        {Hue[0.8008597716600634, 1, 1], LineBox[{144, 145}]}, 
        {Hue[0.8322257562878227, 1, 1], LineBox[{145, 146}]}, 
        {Hue[0.8654094415339608, 1, 1], LineBox[{146, 147}]}, 
        {Hue[0.900382675750429, 1, 1], LineBox[{147, 148}]}, 
        {Hue[0.9371137956725565, 1, 1], LineBox[{148, 149}]}, 
        {Hue[0.9755688441042102, 1, 1], LineBox[{149, 150}]}, 
        {Hue[0.015710822193555896`, 1, 1], LineBox[{150, 151}]}, 
        {Hue[0.05750341769511634, 1, 1], LineBox[{151, 152}]}, 
        {Hue[0.1009088276816796, 1, 1], LineBox[{152, 153}]}, 
        {Hue[0.14588984153644521`, 1, 1], LineBox[{153, 154}]}, 
        {Hue[0.19240917392454862`, 1, 1], LineBox[{154, 155}]}, 
        {Hue[0.2404306955446981, 1, 1], LineBox[{155, 156}]}, 
        {Hue[0.28991896844010645`, 1, 1], LineBox[{156, 157}]}, 
        {Hue[0.34084052389359315`, 1, 1], LineBox[{157, 158}]}, 
        {Hue[0.3931618756369694, 1, 1], LineBox[{158, 159}]}, 
        {Hue[0.44685127226500027`, 1, 1], LineBox[{159, 160}]}, 
        {Hue[0.5018784756128514, 1, 1], LineBox[{160, 161}]}, 
        {Hue[0.5582146923969578, 1, 1], LineBox[{161, 162}]}, 
        {Hue[0.6158309711166918, 1, 1], LineBox[{162, 163}]}, 
        {Hue[0.6747017479612505, 1, 1], LineBox[{163, 164}]}, 
        {Hue[0.7348012513906026, 1, 1], LineBox[{164, 165}]}, 
        {Hue[0.7961058202286575, 1, 1], LineBox[{165, 166}]}, 
        {Hue[0.8585911448709992, 1, 1], LineBox[{166, 167}]}, 
        {Hue[0.9222357148841215, 1, 1], LineBox[{167, 168}]}, 
        {Hue[0.9870184060201163, 1, 1], LineBox[{168, 169}]}, 
        {Hue[0.05292099295597615, 1, 1], LineBox[{169, 170}]}, 
        {Hue[0.11992120925220558`, 1, 1], LineBox[{170, 171}]}, 
        {Hue[0.1880026820180965, 1, 1], LineBox[{171, 172}]}, 
        {Hue[0.2571478502170692, 1, 1], LineBox[{172, 173}]}, 
        {Hue[0.3273391787139689, 1, 1], LineBox[{173, 174}]}, 
        {Hue[0.39856225931539346`, 1, 1], LineBox[{174, 175}]}, 
        {Hue[0.47080188987063876`, 1, 1], LineBox[{175, 176}]}, 
        {Hue[0.5440415604188835, 1, 1], LineBox[{176, 177}]}, 
        {Hue[0.6182681067487781, 1, 1], LineBox[{177, 178}]}, 
        {Hue[0.6934684017449322, 1, 1], LineBox[{178, 179}]}, 
        {Hue[0.7696304170032996, 1, 1], LineBox[{179, 180}]}, 
        {Hue[0.8467428738832437, 1, 1], LineBox[{180, 181}]}, 
        {Hue[0.924790561415135, 1, 1], LineBox[{181, 182}]}, 
        {Hue[0.003762021946077935, 1, 1], LineBox[{182, 183}]}, 
        {Hue[0.08364972930613161, 1, 1], LineBox[{183, 184}]}, 
        {Hue[0.16444234996376278`, 1, 1], LineBox[{184, 185}]}, 
        {Hue[0.24613038943135804`, 1, 1], LineBox[{185, 186}]}, 
        {Hue[0.32870200662097754`, 1, 1], LineBox[{186, 187}]}, 
        {Hue[0.4121436885489237, 1, 1], LineBox[{187, 188}]}, 
        {Hue[0.4964553658376828, 1, 1], LineBox[{188, 189}]}, 
        {Hue[0.5816237800416533, 1, 1], LineBox[{189, 190}]}, 
        {Hue[0.6676374651199203, 1, 1], LineBox[{190, 191}]}, 
        {Hue[0.7544888084455762, 1, 1], LineBox[{191, 192}]}, 
        {Hue[0.8421678489328173, 1, 1], LineBox[{192, 193}]}, 
        {Hue[0.9306872201453219, 1, 1], LineBox[{193, 194}]}, 
        {Hue[0.020028044921177234`, 1, 1], LineBox[{194, 195}]}, 
        {Hue[0.11015384531023154`, 1, 1], LineBox[{195, 196}]}, 
        {Hue[0.2010779247037523, 1, 1], LineBox[{196, 197}]}, 
        {Hue[0.2928164041304295, 1, 1], LineBox[{197, 198}]}, 
        {Hue[0.38533319026587, 1, 1], LineBox[{198, 199}]}, 
        {Hue[0.47860709073898694`, 1, 1], LineBox[{199, 200}]}, 
        {Hue[0.5726744637481624, 1, 1], LineBox[{200, 201}]}, 
        {Hue[0.6675282615363339, 1, 1], LineBox[{201, 202}]}, 
        {Hue[0.7631119700483477, 1, 1], LineBox[{202, 203}]}, 
        {Hue[0.8594478928195965, 1, 1], LineBox[{203, 204}]}, 
        {Hue[0.9565833397615986, 1, 1], LineBox[{204, 205}]}, 
        {Hue[0.05442891042184869, 1, 1], LineBox[{205, 206}]}, 
        {Hue[0.15295126944831824`, 1, 1], LineBox[{206, 207}]}, 
        {Hue[0.2523057656500986, 1, 1], LineBox[{207, 208}]}, 
        {Hue[0.35240163726979257`, 1, 1], LineBox[{208, 209}]}, 
        {Hue[0.4530685174367239, 1, 1], LineBox[{209, 210}]}, 
        {Hue[0.5546446182690019, 1, 1], LineBox[{210, 211}]}, 
        {Hue[0.6571564465191764, 1, 1], LineBox[{211, 212}]}, 
        {Hue[0.7599530157392963, 1, 1], LineBox[{212, 213}]}, 
        {Hue[0.86323881815049, 1, 1], LineBox[{213, 214}]}, 
        {Hue[0.9678183223563761, 1, 1], LineBox[{214, 215}]}, 
        {Hue[0.07322977077513246, 1, 1], LineBox[{215, 216}]}, 
        {Hue[0.17878726522152358`, 1, 1], LineBox[{216, 217}]}, 
        {Hue[0.2853080718697084, 1, 1], LineBox[{217, 218}]}, 
        {Hue[0.39338501586632146`, 1, 1], LineBox[{218, 219}]}, 
        {Hue[0.5017840227505358, 1, 1], LineBox[{219, 220}]}, 
        {Hue[0.6096976623540389, 1, 1], LineBox[{220, 221}]}, 
        {Hue[0.7186763960117638, 1, 1], LineBox[{221, 222}]}, 
        {Hue[0.8296894368998564, 1, 1], LineBox[{222, 223}]}, 
        {Hue[0.9407097079137103, 1, 1], LineBox[{223, 224}]}, 
        {Hue[0.05042165514197633, 1, 1], LineBox[{224, 225}]}, 
        {Hue[0.1611490802839811, 1, 1], LineBox[{225, 226}]}, 
        {Hue[0.2739566445765149, 1, 1], LineBox[{226, 227}]}, 
        {Hue[0.38568383428444303`, 1, 1], LineBox[{227, 228}]}, 
        {Hue[0.4956018360147952, 1, 1], LineBox[{228, 229}]}, 
        {Hue[0.6101557229342287, 1, 1], LineBox[{229, 230}]}, 
        {Hue[0.7326512288615409, 1, 1], LineBox[{230, 231}]}, 
        {Hue[0.8528047721115746, 1, 1], LineBox[{231, 232}]}, 
        {Hue[0.9542542608112418, 1, 1], LineBox[{232, 233}]}, 
        {Hue[0.0558345373254985, 1, 1], LineBox[{233, 234}]}, 
        {Hue[0.1919635521635686, 1, 1], LineBox[{234, 235}]}, 
        {Hue[0.34847173818108124`, 1, 1], LineBox[{235, 236}]}, 
        {Hue[0.48833365764146647`, 1, 1], LineBox[{236, 237}]}, 
        {Hue[0.5804563486032157, 1, 1], LineBox[{237, 238}]}, 
        {Hue[0.7220462748464456, 1, 1], LineBox[{238, 239}]}, 
        {Hue[0.939032460554266, 1, 1], LineBox[{239, 240}]}, 
        {Hue[0.20366116429204512`, 1, 1], LineBox[{240, 241}]}, 
        {Hue[0.4854402698286375, 1, 1], LineBox[{241, 242}]}, 
        {Hue[0.7471307691604958, 1, 1], LineBox[{242, 243}]}, 
        {Hue[0.982320906152381, 1, 1], LineBox[{243, 244}]}, 
        {Hue[0.18815268440253508`, 1, 1], LineBox[{244, 245}]}, 
        {Hue[0.2570783535583873, 1, 1], LineBox[{245, 246}]}, 
        {Hue[0.28697439108716605`, 1, 1], LineBox[{246, 247}]}, 
        {Hue[0.4877808018408883, 1, 1], LineBox[{247, 248}]}, 
        {Hue[0.7145537078305133, 1, 1], LineBox[{248, 249}]}, 
        {Hue[0.9491777610707849, 1, 1], LineBox[{249, 250}]}, 
        {Hue[0.18421774847407577`, 1, 1], LineBox[{250, 251}]}}}},
     VertexColors->{
       Hue[0.46903121084379074`, 1, 1], 
       Hue[0.3276739181280018, 1, 1], 
       Hue[0.27302120989995315`, 1, 1], 
       Hue[0.1503982909211788, 1, 1], 
       Hue[0.1159540677729317, 1, 1], 
       Hue[0.9885034259412886, 1, 1], 
       Hue[0.555919303286334, 1, 1], 
       Hue[0.014674981446927316`, 1, 1], 
       Hue[0.2505963104170238, 1, 1], 
       Hue[0.08114438222060529, 1, 1], 
       Hue[0.01429524804121616, 1, 1], 
       Hue[0.887197895954603, 1, 1], 
       Hue[0.776711378633425, 1, 1], 
       Hue[0.8220325704507755, 1, 1], 
       Hue[0.6796760798270274, 1, 1], 
       Hue[0.7081680584902381, 1, 1], 
       Hue[0.5453095997956825, 1, 1], 
       Hue[0.4431782547061368, 1, 1], 
       Hue[0.24332838511522192`, 1, 1], 
       Hue[0.11867547147282674`, 1, 1], 
       Hue[0.20671099307287558`, 1, 1], 
       Hue[0.874019357415786, 1, 1], 
       Hue[0.6591515413554456, 1, 1], 
       Hue[0.18066228033355544`, 1, 1], 
       Hue[0.6678350196899061, 1, 1], 
       Hue[0.5241258581961699, 1, 1], 
       Hue[0.1431727845363231, 1, 1], 
       Hue[0.7756794556885923, 1, 1], 
       Hue[0.5095140846201118, 1, 1], 
       Hue[0.6861711805072839, 1, 1], 
       Hue[0.8636511930406199, 1, 1], 
       Hue[0.5811421883741066, 1, 1], 
       Hue[0.5145867440569433, 1, 1], 
       Hue[0.3775090338495559, 1, 1], 
       Hue[0.24014184273348146`, 1, 1], 
       Hue[0.0808948788107402, 1, 1], 
       Hue[0.00828126453665834, 1, 1], 
       Hue[0.8486886497933208, 1, 1], 
       Hue[0.7430682886052937, 1, 1], 
       Hue[0.661458391422076, 1, 1], 
       Hue[0.502941759363241, 1, 1], 
       Hue[0.3928074615720261, 1, 1], 
       Hue[0.17436336899653096`, 1, 1], 
       Hue[0.19971063365582253`, 1, 1], 
       Hue[0.9072160164407977, 1, 1], 
       Hue[0.7435128384384582, 1, 1], 
       Hue[0.7542167129026032, 1, 1], 
       Hue[0.7220480636268023, 1, 1], 
       Hue[0.5960170125858861, 1, 1], 
       Hue[0.504394885168866, 1, 1], 
       Hue[0.5980761131160199, 1, 1], 
       Hue[0.39175536296490443`, 1, 1], 
       Hue[0.3080870960109756, 1, 1], 
       Hue[0.22637965615689903`, 1, 1], 
       Hue[0.04078695456709334, 1, 1], 
       Hue[0.9979339064190902, 1, 1], 
       Hue[0.9379513734489464, 1, 1], 
       Hue[0.7568969236098246, 1, 1], 
       Hue[0.6044031428332308, 1, 1], 
       Hue[0.546835631606627, 1, 1], 
       Hue[0.48445950589899756`, 1, 1], 
       Hue[0.45836739143999605`, 1, 1], 
       Hue[0.4050876322941351, 1, 1], 
       Hue[0.3310440859556128, 1, 1], 
       Hue[0.8952884846740966, 1, 1], 
       Hue[0.849914391383057, 1, 1], 
       Hue[0.8210799392004464, 1, 1], 
       Hue[0.797476270966503, 1, 1], 
       Hue[0.7677111204344772, 1, 1], 
       Hue[0.7373996974703723, 1, 1], 
       Hue[0.6751855643100823, 1, 1], 
       Hue[0.24133341544108203`, 1, 1], 
       Hue[0.20414494146012305`, 1, 1], 
       Hue[0.17127988511038536`, 1, 1], 
       Hue[0.15097260410212796`, 1, 1], 
       Hue[0.1323205065914355, 1, 1], 
       Hue[0.11577470259370665`, 1, 1], 
       Hue[0.40072236254851873`, 1, 1], 
       Hue[0.5642972190009667, 1, 1], 
       Hue[0.5486440775627401, 1, 1], 
       Hue[0.5323579238759228, 1, 1], 
       Hue[0.5186050976851513, 1, 1], 
       Hue[0.5076758901256943, 1, 1], 
       Hue[0.5231123375576997, 1, 1], 
       Hue[0.9249542200277258, 1, 1], 
       Hue[0.9326440595614945, 1, 1], 
       Hue[0.92414635791773, 1, 1], 
       Hue[0.914328727297987, 1, 1], 
       Hue[0.9075469174696386, 1, 1], 
       Hue[0.9162986729778786, 1, 1], 
       Hue[0.22438990344755358`, 1, 1], 
       Hue[0.3233701921895654, 1, 1], 
       Hue[0.3255239702566573, 1, 1], 
       Hue[0.32135596500062585`, 1, 1], 
       Hue[0.3185714943521732, 1, 1], 
       Hue[0.3260807234884014, 1, 1], 
       Hue[0.44731094999721865`, 1, 1], 
       Hue[0.718402716585326, 1, 1], 
       Hue[0.7378073209369546, 1, 1], 
       Hue[0.7400657932223309, 1, 1], 
       Hue[0.7412978514163524, 1, 1], 
       Hue[0.7498344855551069, 1, 1], 
       Hue[0.8080132497531595, 1, 1], 
       Hue[0.10678166006413865`, 1, 1], 
       Hue[0.15908580322017676`, 1, 1], 
       Hue[0.16950671056358974`, 1, 1], 
       Hue[0.1750900190849448, 1, 1], 
       Hue[0.18498588973514118`, 1, 1], 
       Hue[0.2205776860799222, 1, 1], 
       Hue[0.4486392775157134, 1, 1], 
       Hue[0.5823447125514417, 1, 1], 
       Hue[0.6062875308462049, 1, 1], 
       Hue[0.6170867116073702, 1, 1], 
       Hue[0.6280375327039411, 1, 1], 
       Hue[0.6498635797223722, 1, 1], 
       Hue[0.7294631065042633, 1, 1], 
       Hue[0.9619442698698356, 1, 1], 
       Hue[0.0358694738259074, 1, 1], 
       Hue[0.058560492456003414`, 1, 1], 
       Hue[0.07200337794989134, 1, 1], 
       Hue[0.08546181681129635, 1, 1], 
       Hue[0.10595949252681901`, 1, 1], 
       Hue[0.15064552410101642`, 1, 1], 
       Hue[0.2776938160586117, 1, 1], 
       Hue[0.42749515722735437`, 1, 1], 
       Hue[0.48338919600598795`, 1, 1], 
       Hue[0.5077457331726007, 1, 1], 
       Hue[0.522453270537981, 1, 1], 
       Hue[0.533643024834802, 1, 1], 
       Hue[0.5436190540030256, 1, 1], 
       Hue[0.5534698627463082, 1, 1], 
       Hue[0.5638043360423671, 1, 1], 
       Hue[0.5750106310394598, 1, 1], 
       Hue[0.5873592564502069, 1, 1], 
       Hue[0.6010505587702185, 1, 1], 
       Hue[0.6162377444445248, 1, 1], 
       Hue[0.633039328584811, 1, 1], 
       Hue[0.6515452342370982, 1, 1], 
       Hue[0.671822399406282, 1, 1], 
       Hue[0.6939174624732731, 1, 1], 
       Hue[0.7178601571527807, 1, 1], 
       Hue[0.743664904009497, 1, 1], 
       Hue[0.7713339799014306, 1, 1], 
       Hue[0.8008597716600634, 1, 1], 
       Hue[0.8322257562878227, 1, 1], 
       Hue[0.8654094415339608, 1, 1], 
       Hue[0.900382675750429, 1, 1], 
       Hue[0.9371137956725565, 1, 1], 
       Hue[0.9755688441042102, 1, 1], 
       Hue[0.015710822193555896`, 1, 1], 
       Hue[0.05750341769511634, 1, 1], 
       Hue[0.1009088276816796, 1, 1], 
       Hue[0.14588984153644521`, 1, 1], 
       Hue[0.19240917392454862`, 1, 1], 
       Hue[0.2404306955446981, 1, 1], 
       Hue[0.28991896844010645`, 1, 1], 
       Hue[0.34084052389359315`, 1, 1], 
       Hue[0.3931618756369694, 1, 1], 
       Hue[0.44685127226500027`, 1, 1], 
       Hue[0.5018784756128514, 1, 1], 
       Hue[0.5582146923969578, 1, 1], 
       Hue[0.6158309711166918, 1, 1], 
       Hue[0.6747017479612505, 1, 1], 
       Hue[0.7348012513906026, 1, 1], 
       Hue[0.7961058202286575, 1, 1], 
       Hue[0.8585911448709992, 1, 1], 
       Hue[0.9222357148841215, 1, 1], 
       Hue[0.9870184060201163, 1, 1], 
       Hue[0.05292099295597615, 1, 1], 
       Hue[0.11992120925220558`, 1, 1], 
       Hue[0.1880026820180965, 1, 1], 
       Hue[0.2571478502170692, 1, 1], 
       Hue[0.3273391787139689, 1, 1], 
       Hue[0.39856225931539346`, 1, 1], 
       Hue[0.47080188987063876`, 1, 1], 
       Hue[0.5440415604188835, 1, 1], 
       Hue[0.6182681067487781, 1, 1], 
       Hue[0.6934684017449322, 1, 1], 
       Hue[0.7696304170032996, 1, 1], 
       Hue[0.8467428738832437, 1, 1], 
       Hue[0.924790561415135, 1, 1], 
       Hue[0.003762021946077935, 1, 1], 
       Hue[0.08364972930613161, 1, 1], 
       Hue[0.16444234996376278`, 1, 1], 
       Hue[0.24613038943135804`, 1, 1], 
       Hue[0.32870200662097754`, 1, 1], 
       Hue[0.4121436885489237, 1, 1], 
       Hue[0.4964553658376828, 1, 1], 
       Hue[0.5816237800416533, 1, 1], 
       Hue[0.6676374651199203, 1, 1], 
       Hue[0.7544888084455762, 1, 1], 
       Hue[0.8421678489328173, 1, 1], 
       Hue[0.9306872201453219, 1, 1], 
       Hue[0.020028044921177234`, 1, 1], 
       Hue[0.11015384531023154`, 1, 1], 
       Hue[0.2010779247037523, 1, 1], 
       Hue[0.2928164041304295, 1, 1], 
       Hue[0.38533319026587, 1, 1], 
       Hue[0.47860709073898694`, 1, 1], 
       Hue[0.5726744637481624, 1, 1], 
       Hue[0.6675282615363339, 1, 1], 
       Hue[0.7631119700483477, 1, 1], 
       Hue[0.8594478928195965, 1, 1], 
       Hue[0.9565833397615986, 1, 1], 
       Hue[0.05442891042184869, 1, 1], 
       Hue[0.15295126944831824`, 1, 1], 
       Hue[0.2523057656500986, 1, 1], 
       Hue[0.35240163726979257`, 1, 1], 
       Hue[0.4530685174367239, 1, 1], 
       Hue[0.5546446182690019, 1, 1], 
       Hue[0.6571564465191764, 1, 1], 
       Hue[0.7599530157392963, 1, 1], 
       Hue[0.86323881815049, 1, 1], 
       Hue[0.9678183223563761, 1, 1], 
       Hue[0.07322977077513246, 1, 1], 
       Hue[0.17878726522152358`, 1, 1], 
       Hue[0.2853080718697084, 1, 1], 
       Hue[0.39338501586632146`, 1, 1], 
       Hue[0.5017840227505358, 1, 1], 
       Hue[0.6096976623540389, 1, 1], 
       Hue[0.7186763960117638, 1, 1], 
       Hue[0.8296894368998564, 1, 1], 
       Hue[0.9407097079137103, 1, 1], 
       Hue[0.05042165514197633, 1, 1], 
       Hue[0.1611490802839811, 1, 1], 
       Hue[0.2739566445765149, 1, 1], 
       Hue[0.38568383428444303`, 1, 1], 
       Hue[0.4956018360147952, 1, 1], 
       Hue[0.6101557229342287, 1, 1], 
       Hue[0.7326512288615409, 1, 1], 
       Hue[0.8528047721115746, 1, 1], 
       Hue[0.9542542608112418, 1, 1], 
       Hue[0.0558345373254985, 1, 1], 
       Hue[0.1919635521635686, 1, 1], 
       Hue[0.34847173818108124`, 1, 1], 
       Hue[0.48833365764146647`, 1, 1], 
       Hue[0.5804563486032157, 1, 1], 
       Hue[0.7220462748464456, 1, 1], 
       Hue[0.939032460554266, 1, 1], 
       Hue[0.20366116429204512`, 1, 1], 
       Hue[0.4854402698286375, 1, 1], 
       Hue[0.7471307691604958, 1, 1], 
       Hue[0.982320906152381, 1, 1], 
       Hue[0.18815268440253508`, 1, 1], 
       Hue[0.2570783535583873, 1, 1], 
       Hue[0.28697439108716605`, 1, 1], 
       Hue[0.4877808018408883, 1, 1], 
       Hue[0.7145537078305133, 1, 1], 
       Hue[0.9491777610707849, 1, 1], 
       Hue[0.18421774847407577`, 1, 1], 
       Hue[0.40213423036871093`, 1, 1], 
       Hue[0.46903121084379074`, 1, 1], 
       Hue[0.3276739181280018, 1, 1], 
       Hue[0.3276739181280018, 1, 1], 
       Hue[0.27302120989995315`, 1, 1], 
       Hue[0.27302120989995315`, 1, 1], 
       Hue[0.1503982909211788, 1, 1], 
       Hue[0.1503982909211788, 1, 1], 
       Hue[0.1159540677729317, 1, 1], 
       Hue[0.1159540677729317, 1, 1], 
       Hue[0.9885034259412886, 1, 1], 
       Hue[0.9885034259412886, 1, 1], 
       Hue[0.555919303286334, 1, 1], 
       Hue[0.555919303286334, 1, 1], 
       Hue[0.014674981446927316`, 1, 1], 
       Hue[0.014674981446927316`, 1, 1], 
       Hue[0.2505963104170238, 1, 1], 
       Hue[0.2505963104170238, 1, 1], 
       Hue[0.08114438222060529, 1, 1], 
       Hue[0.08114438222060529, 1, 1], 
       Hue[0.01429524804121616, 1, 1], 
       Hue[0.01429524804121616, 1, 1], 
       Hue[0.887197895954603, 1, 1], 
       Hue[0.887197895954603, 1, 1], 
       Hue[0.776711378633425, 1, 1], 
       Hue[0.776711378633425, 1, 1], 
       Hue[0.8220325704507755, 1, 1], 
       Hue[0.8220325704507755, 1, 1], 
       Hue[0.6796760798270274, 1, 1], 
       Hue[0.6796760798270274, 1, 1], 
       Hue[0.7081680584902381, 1, 1], 
       Hue[0.7081680584902381, 1, 1], 
       Hue[0.5453095997956825, 1, 1], 
       Hue[0.5453095997956825, 1, 1], 
       Hue[0.4431782547061368, 1, 1], 
       Hue[0.4431782547061368, 1, 1], 
       Hue[0.24332838511522192`, 1, 1], 
       Hue[0.24332838511522192`, 1, 1], 
       Hue[0.11867547147282674`, 1, 1], 
       Hue[0.11867547147282674`, 1, 1], 
       Hue[0.20671099307287558`, 1, 1], 
       Hue[0.20671099307287558`, 1, 1], 
       Hue[0.874019357415786, 1, 1], 
       Hue[0.874019357415786, 1, 1], 
       Hue[0.6591515413554456, 1, 1], 
       Hue[0.6591515413554456, 1, 1], 
       Hue[0.18066228033355544`, 1, 1], 
       Hue[0.18066228033355544`, 1, 1], 
       Hue[0.6678350196899061, 1, 1], 
       Hue[0.6678350196899061, 1, 1], 
       Hue[0.5241258581961699, 1, 1], 
       Hue[0.5241258581961699, 1, 1], 
       Hue[0.1431727845363231, 1, 1], 
       Hue[0.1431727845363231, 1, 1], 
       Hue[0.7756794556885923, 1, 1], 
       Hue[0.7756794556885923, 1, 1], 
       Hue[0.5095140846201118, 1, 1], 
       Hue[0.5095140846201118, 1, 1], 
       Hue[0.6861711805072839, 1, 1], 
       Hue[0.6861711805072839, 1, 1], 
       Hue[0.8636511930406199, 1, 1], 
       Hue[0.8636511930406199, 1, 1], 
       Hue[0.5811421883741066, 1, 1], 
       Hue[0.5811421883741066, 1, 1], 
       Hue[0.5145867440569433, 1, 1], 
       Hue[0.5145867440569433, 1, 1], 
       Hue[0.3775090338495559, 1, 1], 
       Hue[0.3775090338495559, 1, 1], 
       Hue[0.24014184273348146`, 1, 1], 
       Hue[0.24014184273348146`, 1, 1], 
       Hue[0.0808948788107402, 1, 1], 
       Hue[0.0808948788107402, 1, 1], 
       Hue[0.00828126453665834, 1, 1], 
       Hue[0.00828126453665834, 1, 1], 
       Hue[0.8486886497933208, 1, 1], 
       Hue[0.8486886497933208, 1, 1], 
       Hue[0.7430682886052937, 1, 1], 
       Hue[0.7430682886052937, 1, 1], 
       Hue[0.661458391422076, 1, 1], 
       Hue[0.661458391422076, 1, 1], 
       Hue[0.502941759363241, 1, 1], 
       Hue[0.502941759363241, 1, 1], 
       Hue[0.3928074615720261, 1, 1], 
       Hue[0.3928074615720261, 1, 1], 
       Hue[0.17436336899653096`, 1, 1], 
       Hue[0.17436336899653096`, 1, 1], 
       Hue[0.19971063365582253`, 1, 1], 
       Hue[0.19971063365582253`, 1, 1], 
       Hue[0.9072160164407977, 1, 1], 
       Hue[0.9072160164407977, 1, 1], 
       Hue[0.7435128384384582, 1, 1], 
       Hue[0.7435128384384582, 1, 1], 
       Hue[0.7542167129026032, 1, 1], 
       Hue[0.7542167129026032, 1, 1], 
       Hue[0.7220480636268023, 1, 1], 
       Hue[0.7220480636268023, 1, 1], 
       Hue[0.5960170125858861, 1, 1], 
       Hue[0.5960170125858861, 1, 1], 
       Hue[0.504394885168866, 1, 1], 
       Hue[0.504394885168866, 1, 1], 
       Hue[0.5980761131160199, 1, 1], 
       Hue[0.5980761131160199, 1, 1], 
       Hue[0.39175536296490443`, 1, 1], 
       Hue[0.39175536296490443`, 1, 1], 
       Hue[0.3080870960109756, 1, 1], 
       Hue[0.3080870960109756, 1, 1], 
       Hue[0.22637965615689903`, 1, 1], 
       Hue[0.22637965615689903`, 1, 1], 
       Hue[0.04078695456709334, 1, 1], 
       Hue[0.04078695456709334, 1, 1], 
       Hue[0.9979339064190902, 1, 1], 
       Hue[0.9979339064190902, 1, 1], 
       Hue[0.9379513734489464, 1, 1], 
       Hue[0.9379513734489464, 1, 1], 
       Hue[0.7568969236098246, 1, 1], 
       Hue[0.7568969236098246, 1, 1], 
       Hue[0.6044031428332308, 1, 1], 
       Hue[0.6044031428332308, 1, 1], 
       Hue[0.546835631606627, 1, 1], 
       Hue[0.546835631606627, 1, 1], 
       Hue[0.48445950589899756`, 1, 1], 
       Hue[0.48445950589899756`, 1, 1], 
       Hue[0.45836739143999605`, 1, 1], 
       Hue[0.45836739143999605`, 1, 1], 
       Hue[0.4050876322941351, 1, 1], 
       Hue[0.4050876322941351, 1, 1], 
       Hue[0.3310440859556128, 1, 1], 
       Hue[0.3310440859556128, 1, 1], 
       Hue[0.8952884846740966, 1, 1], 
       Hue[0.8952884846740966, 1, 1], 
       Hue[0.849914391383057, 1, 1], 
       Hue[0.849914391383057, 1, 1], 
       Hue[0.8210799392004464, 1, 1], 
       Hue[0.8210799392004464, 1, 1], 
       Hue[0.797476270966503, 1, 1], 
       Hue[0.797476270966503, 1, 1], 
       Hue[0.7677111204344772, 1, 1], 
       Hue[0.7677111204344772, 1, 1], 
       Hue[0.7373996974703723, 1, 1], 
       Hue[0.7373996974703723, 1, 1], 
       Hue[0.6751855643100823, 1, 1], 
       Hue[0.6751855643100823, 1, 1], 
       Hue[0.24133341544108203`, 1, 1], 
       Hue[0.24133341544108203`, 1, 1], 
       Hue[0.20414494146012305`, 1, 1], 
       Hue[0.20414494146012305`, 1, 1], 
       Hue[0.17127988511038536`, 1, 1], 
       Hue[0.17127988511038536`, 1, 1], 
       Hue[0.15097260410212796`, 1, 1], 
       Hue[0.15097260410212796`, 1, 1], 
       Hue[0.1323205065914355, 1, 1], 
       Hue[0.1323205065914355, 1, 1], 
       Hue[0.11577470259370665`, 1, 1], 
       Hue[0.11577470259370665`, 1, 1], 
       Hue[0.40072236254851873`, 1, 1], 
       Hue[0.40072236254851873`, 1, 1], 
       Hue[0.5642972190009667, 1, 1], 
       Hue[0.5642972190009667, 1, 1], 
       Hue[0.5486440775627401, 1, 1], 
       Hue[0.5486440775627401, 1, 1], 
       Hue[0.5323579238759228, 1, 1], 
       Hue[0.5323579238759228, 1, 1], 
       Hue[0.5186050976851513, 1, 1], 
       Hue[0.5186050976851513, 1, 1], 
       Hue[0.5076758901256943, 1, 1], 
       Hue[0.5076758901256943, 1, 1], 
       Hue[0.5231123375576997, 1, 1], 
       Hue[0.5231123375576997, 1, 1], 
       Hue[0.9249542200277258, 1, 1], 
       Hue[0.9249542200277258, 1, 1], 
       Hue[0.9326440595614945, 1, 1], 
       Hue[0.9326440595614945, 1, 1], 
       Hue[0.92414635791773, 1, 1], 
       Hue[0.92414635791773, 1, 1], 
       Hue[0.914328727297987, 1, 1], 
       Hue[0.914328727297987, 1, 1], 
       Hue[0.9075469174696386, 1, 1], 
       Hue[0.9075469174696386, 1, 1], 
       Hue[0.9162986729778786, 1, 1], 
       Hue[0.9162986729778786, 1, 1], 
       Hue[0.22438990344755358`, 1, 1], 
       Hue[0.22438990344755358`, 1, 1], 
       Hue[0.3233701921895654, 1, 1], 
       Hue[0.3233701921895654, 1, 1], 
       Hue[0.3255239702566573, 1, 1], 
       Hue[0.3255239702566573, 1, 1], 
       Hue[0.32135596500062585`, 1, 1], 
       Hue[0.32135596500062585`, 1, 1], 
       Hue[0.3185714943521732, 1, 1], 
       Hue[0.3185714943521732, 1, 1], 
       Hue[0.3260807234884014, 1, 1], 
       Hue[0.3260807234884014, 1, 1], 
       Hue[0.44731094999721865`, 1, 1], 
       Hue[0.44731094999721865`, 1, 1], 
       Hue[0.718402716585326, 1, 1], 
       Hue[0.718402716585326, 1, 1], 
       Hue[0.7378073209369546, 1, 1], 
       Hue[0.7378073209369546, 1, 1], 
       Hue[0.7400657932223309, 1, 1], 
       Hue[0.7400657932223309, 1, 1], 
       Hue[0.7412978514163524, 1, 1], 
       Hue[0.7412978514163524, 1, 1], 
       Hue[0.7498344855551069, 1, 1], 
       Hue[0.7498344855551069, 1, 1], 
       Hue[0.8080132497531595, 1, 1], 
       Hue[0.8080132497531595, 1, 1], 
       Hue[0.10678166006413865`, 1, 1], 
       Hue[0.10678166006413865`, 1, 1], 
       Hue[0.15908580322017676`, 1, 1], 
       Hue[0.15908580322017676`, 1, 1], 
       Hue[0.16950671056358974`, 1, 1], 
       Hue[0.16950671056358974`, 1, 1], 
       Hue[0.1750900190849448, 1, 1], 
       Hue[0.1750900190849448, 1, 1], 
       Hue[0.18498588973514118`, 1, 1], 
       Hue[0.18498588973514118`, 1, 1], 
       Hue[0.2205776860799222, 1, 1], 
       Hue[0.2205776860799222, 1, 1], 
       Hue[0.4486392775157134, 1, 1], 
       Hue[0.4486392775157134, 1, 1], 
       Hue[0.5823447125514417, 1, 1], 
       Hue[0.5823447125514417, 1, 1], 
       Hue[0.6062875308462049, 1, 1], 
       Hue[0.6062875308462049, 1, 1], 
       Hue[0.6170867116073702, 1, 1], 
       Hue[0.6170867116073702, 1, 1], 
       Hue[0.6280375327039411, 1, 1], 
       Hue[0.6280375327039411, 1, 1], 
       Hue[0.6498635797223722, 1, 1], 
       Hue[0.6498635797223722, 1, 1], 
       Hue[0.7294631065042633, 1, 1], 
       Hue[0.7294631065042633, 1, 1], 
       Hue[0.9619442698698356, 1, 1], 
       Hue[0.9619442698698356, 1, 1], 
       Hue[0.0358694738259074, 1, 1], 
       Hue[0.0358694738259074, 1, 1], 
       Hue[0.058560492456003414`, 1, 1], 
       Hue[0.058560492456003414`, 1, 1], 
       Hue[0.07200337794989134, 1, 1], 
       Hue[0.07200337794989134, 1, 1], 
       Hue[0.08546181681129635, 1, 1], 
       Hue[0.08546181681129635, 1, 1], 
       Hue[0.10595949252681901`, 1, 1], 
       Hue[0.10595949252681901`, 1, 1], 
       Hue[0.15064552410101642`, 1, 1], 
       Hue[0.15064552410101642`, 1, 1], 
       Hue[0.2776938160586117, 1, 1], 
       Hue[0.2776938160586117, 1, 1], 
       Hue[0.42749515722735437`, 1, 1], 
       Hue[0.42749515722735437`, 1, 1], 
       Hue[0.48338919600598795`, 1, 1], 
       Hue[0.48338919600598795`, 1, 1], 
       Hue[0.5077457331726007, 1, 1], 
       Hue[0.5077457331726007, 1, 1], 
       Hue[0.522453270537981, 1, 1], 
       Hue[0.522453270537981, 1, 1], 
       Hue[0.533643024834802, 1, 1], 
       Hue[0.533643024834802, 1, 1], 
       Hue[0.5436190540030256, 1, 1], 
       Hue[0.5436190540030256, 1, 1], 
       Hue[0.5534698627463082, 1, 1], 
       Hue[0.5534698627463082, 1, 1], 
       Hue[0.5638043360423671, 1, 1], 
       Hue[0.5638043360423671, 1, 1], 
       Hue[0.5750106310394598, 1, 1], 
       Hue[0.5750106310394598, 1, 1], 
       Hue[0.5873592564502069, 1, 1], 
       Hue[0.5873592564502069, 1, 1], 
       Hue[0.6010505587702185, 1, 1], 
       Hue[0.6010505587702185, 1, 1], 
       Hue[0.6162377444445248, 1, 1], 
       Hue[0.6162377444445248, 1, 1], 
       Hue[0.633039328584811, 1, 1], 
       Hue[0.633039328584811, 1, 1], 
       Hue[0.6515452342370982, 1, 1], 
       Hue[0.6515452342370982, 1, 1], 
       Hue[0.671822399406282, 1, 1], 
       Hue[0.671822399406282, 1, 1], 
       Hue[0.6939174624732731, 1, 1], 
       Hue[0.6939174624732731, 1, 1], 
       Hue[0.7178601571527807, 1, 1], 
       Hue[0.7178601571527807, 1, 1], 
       Hue[0.743664904009497, 1, 1], 
       Hue[0.743664904009497, 1, 1], 
       Hue[0.7713339799014306, 1, 1], 
       Hue[0.7713339799014306, 1, 1], 
       Hue[0.8008597716600634, 1, 1], 
       Hue[0.8008597716600634, 1, 1], 
       Hue[0.8322257562878227, 1, 1], 
       Hue[0.8322257562878227, 1, 1], 
       Hue[0.8654094415339608, 1, 1], 
       Hue[0.8654094415339608, 1, 1], 
       Hue[0.900382675750429, 1, 1], 
       Hue[0.900382675750429, 1, 1], 
       Hue[0.9371137956725565, 1, 1], 
       Hue[0.9371137956725565, 1, 1], 
       Hue[0.9755688441042102, 1, 1], 
       Hue[0.9755688441042102, 1, 1], 
       Hue[0.015710822193555896`, 1, 1], 
       Hue[0.015710822193555896`, 1, 1], 
       Hue[0.05750341769511634, 1, 1], 
       Hue[0.05750341769511634, 1, 1], 
       Hue[0.1009088276816796, 1, 1], 
       Hue[0.1009088276816796, 1, 1], 
       Hue[0.14588984153644521`, 1, 1], 
       Hue[0.14588984153644521`, 1, 1], 
       Hue[0.19240917392454862`, 1, 1], 
       Hue[0.19240917392454862`, 1, 1], 
       Hue[0.2404306955446981, 1, 1], 
       Hue[0.2404306955446981, 1, 1], 
       Hue[0.28991896844010645`, 1, 1], 
       Hue[0.28991896844010645`, 1, 1], 
       Hue[0.34084052389359315`, 1, 1], 
       Hue[0.34084052389359315`, 1, 1], 
       Hue[0.3931618756369694, 1, 1], 
       Hue[0.3931618756369694, 1, 1], 
       Hue[0.44685127226500027`, 1, 1], 
       Hue[0.44685127226500027`, 1, 1], 
       Hue[0.5018784756128514, 1, 1], 
       Hue[0.5018784756128514, 1, 1], 
       Hue[0.5582146923969578, 1, 1], 
       Hue[0.5582146923969578, 1, 1], 
       Hue[0.6158309711166918, 1, 1], 
       Hue[0.6158309711166918, 1, 1], 
       Hue[0.6747017479612505, 1, 1], 
       Hue[0.6747017479612505, 1, 1], 
       Hue[0.7348012513906026, 1, 1], 
       Hue[0.7348012513906026, 1, 1], 
       Hue[0.7961058202286575, 1, 1], 
       Hue[0.7961058202286575, 1, 1], 
       Hue[0.8585911448709992, 1, 1], 
       Hue[0.8585911448709992, 1, 1], 
       Hue[0.9222357148841215, 1, 1], 
       Hue[0.9222357148841215, 1, 1], 
       Hue[0.9870184060201163, 1, 1], 
       Hue[0.9870184060201163, 1, 1], 
       Hue[0.05292099295597615, 1, 1], 
       Hue[0.05292099295597615, 1, 1], 
       Hue[0.11992120925220558`, 1, 1], 
       Hue[0.11992120925220558`, 1, 1], 
       Hue[0.1880026820180965, 1, 1], 
       Hue[0.1880026820180965, 1, 1], 
       Hue[0.2571478502170692, 1, 1], 
       Hue[0.2571478502170692, 1, 1], 
       Hue[0.3273391787139689, 1, 1], 
       Hue[0.3273391787139689, 1, 1], 
       Hue[0.39856225931539346`, 1, 1], 
       Hue[0.39856225931539346`, 1, 1], 
       Hue[0.47080188987063876`, 1, 1], 
       Hue[0.47080188987063876`, 1, 1], 
       Hue[0.5440415604188835, 1, 1], 
       Hue[0.5440415604188835, 1, 1], 
       Hue[0.6182681067487781, 1, 1], 
       Hue[0.6182681067487781, 1, 1], 
       Hue[0.6934684017449322, 1, 1], 
       Hue[0.6934684017449322, 1, 1], 
       Hue[0.7696304170032996, 1, 1], 
       Hue[0.7696304170032996, 1, 1], 
       Hue[0.8467428738832437, 1, 1], 
       Hue[0.8467428738832437, 1, 1], 
       Hue[0.924790561415135, 1, 1], 
       Hue[0.924790561415135, 1, 1], 
       Hue[0.003762021946077935, 1, 1], 
       Hue[0.003762021946077935, 1, 1], 
       Hue[0.08364972930613161, 1, 1], 
       Hue[0.08364972930613161, 1, 1], 
       Hue[0.16444234996376278`, 1, 1], 
       Hue[0.16444234996376278`, 1, 1], 
       Hue[0.24613038943135804`, 1, 1], 
       Hue[0.24613038943135804`, 1, 1], 
       Hue[0.32870200662097754`, 1, 1], 
       Hue[0.32870200662097754`, 1, 1], 
       Hue[0.4121436885489237, 1, 1], 
       Hue[0.4121436885489237, 1, 1], 
       Hue[0.4964553658376828, 1, 1], 
       Hue[0.4964553658376828, 1, 1], 
       Hue[0.5816237800416533, 1, 1], 
       Hue[0.5816237800416533, 1, 1], 
       Hue[0.6676374651199203, 1, 1], 
       Hue[0.6676374651199203, 1, 1], 
       Hue[0.7544888084455762, 1, 1], 
       Hue[0.7544888084455762, 1, 1], 
       Hue[0.8421678489328173, 1, 1], 
       Hue[0.8421678489328173, 1, 1], 
       Hue[0.9306872201453219, 1, 1], 
       Hue[0.9306872201453219, 1, 1], 
       Hue[0.020028044921177234`, 1, 1], 
       Hue[0.020028044921177234`, 1, 1], 
       Hue[0.11015384531023154`, 1, 1], 
       Hue[0.11015384531023154`, 1, 1], 
       Hue[0.2010779247037523, 1, 1], 
       Hue[0.2010779247037523, 1, 1], 
       Hue[0.2928164041304295, 1, 1], 
       Hue[0.2928164041304295, 1, 1], 
       Hue[0.38533319026587, 1, 1], 
       Hue[0.38533319026587, 1, 1], 
       Hue[0.47860709073898694`, 1, 1], 
       Hue[0.47860709073898694`, 1, 1], 
       Hue[0.5726744637481624, 1, 1], 
       Hue[0.5726744637481624, 1, 1], 
       Hue[0.6675282615363339, 1, 1], 
       Hue[0.6675282615363339, 1, 1], 
       Hue[0.7631119700483477, 1, 1], 
       Hue[0.7631119700483477, 1, 1], 
       Hue[0.8594478928195965, 1, 1], 
       Hue[0.8594478928195965, 1, 1], 
       Hue[0.9565833397615986, 1, 1], 
       Hue[0.9565833397615986, 1, 1], 
       Hue[0.05442891042184869, 1, 1], 
       Hue[0.05442891042184869, 1, 1], 
       Hue[0.15295126944831824`, 1, 1], 
       Hue[0.15295126944831824`, 1, 1], 
       Hue[0.2523057656500986, 1, 1], 
       Hue[0.2523057656500986, 1, 1], 
       Hue[0.35240163726979257`, 1, 1], 
       Hue[0.35240163726979257`, 1, 1], 
       Hue[0.4530685174367239, 1, 1], 
       Hue[0.4530685174367239, 1, 1], 
       Hue[0.5546446182690019, 1, 1], 
       Hue[0.5546446182690019, 1, 1], 
       Hue[0.6571564465191764, 1, 1], 
       Hue[0.6571564465191764, 1, 1], 
       Hue[0.7599530157392963, 1, 1], 
       Hue[0.7599530157392963, 1, 1], 
       Hue[0.86323881815049, 1, 1], 
       Hue[0.86323881815049, 1, 1], 
       Hue[0.9678183223563761, 1, 1], 
       Hue[0.9678183223563761, 1, 1], 
       Hue[0.07322977077513246, 1, 1], 
       Hue[0.07322977077513246, 1, 1], 
       Hue[0.17878726522152358`, 1, 1], 
       Hue[0.17878726522152358`, 1, 1], 
       Hue[0.2853080718697084, 1, 1], 
       Hue[0.2853080718697084, 1, 1], 
       Hue[0.39338501586632146`, 1, 1], 
       Hue[0.39338501586632146`, 1, 1], 
       Hue[0.5017840227505358, 1, 1], 
       Hue[0.5017840227505358, 1, 1], 
       Hue[0.6096976623540389, 1, 1], 
       Hue[0.6096976623540389, 1, 1], 
       Hue[0.7186763960117638, 1, 1], 
       Hue[0.7186763960117638, 1, 1], 
       Hue[0.8296894368998564, 1, 1], 
       Hue[0.8296894368998564, 1, 1], 
       Hue[0.9407097079137103, 1, 1], 
       Hue[0.9407097079137103, 1, 1], 
       Hue[0.05042165514197633, 1, 1], 
       Hue[0.05042165514197633, 1, 1], 
       Hue[0.1611490802839811, 1, 1], 
       Hue[0.1611490802839811, 1, 1], 
       Hue[0.2739566445765149, 1, 1], 
       Hue[0.2739566445765149, 1, 1], 
       Hue[0.38568383428444303`, 1, 1], 
       Hue[0.38568383428444303`, 1, 1], 
       Hue[0.4956018360147952, 1, 1], 
       Hue[0.4956018360147952, 1, 1], 
       Hue[0.6101557229342287, 1, 1], 
       Hue[0.6101557229342287, 1, 1], 
       Hue[0.7326512288615409, 1, 1], 
       Hue[0.7326512288615409, 1, 1], 
       Hue[0.8528047721115746, 1, 1], 
       Hue[0.8528047721115746, 1, 1], 
       Hue[0.9542542608112418, 1, 1], 
       Hue[0.9542542608112418, 1, 1], 
       Hue[0.0558345373254985, 1, 1], 
       Hue[0.0558345373254985, 1, 1], 
       Hue[0.1919635521635686, 1, 1], 
       Hue[0.1919635521635686, 1, 1], 
       Hue[0.34847173818108124`, 1, 1], 
       Hue[0.34847173818108124`, 1, 1], 
       Hue[0.48833365764146647`, 1, 1], 
       Hue[0.48833365764146647`, 1, 1], 
       Hue[0.5804563486032157, 1, 1], 
       Hue[0.5804563486032157, 1, 1], 
       Hue[0.7220462748464456, 1, 1], 
       Hue[0.7220462748464456, 1, 1], 
       Hue[0.939032460554266, 1, 1], 
       Hue[0.939032460554266, 1, 1], 
       Hue[0.20366116429204512`, 1, 1], 
       Hue[0.20366116429204512`, 1, 1], 
       Hue[0.4854402698286375, 1, 1], 
       Hue[0.4854402698286375, 1, 1], 
       Hue[0.7471307691604958, 1, 1], 
       Hue[0.7471307691604958, 1, 1], 
       Hue[0.982320906152381, 1, 1], 
       Hue[0.982320906152381, 1, 1], 
       Hue[0.18815268440253508`, 1, 1], 
       Hue[0.18815268440253508`, 1, 1], 
       Hue[0.2570783535583873, 1, 1], 
       Hue[0.2570783535583873, 1, 1], 
       Hue[0.28697439108716605`, 1, 1], 
       Hue[0.28697439108716605`, 1, 1], 
       Hue[0.4877808018408883, 1, 1], 
       Hue[0.4877808018408883, 1, 1], 
       Hue[0.7145537078305133, 1, 1], 
       Hue[0.7145537078305133, 1, 1], 
       Hue[0.9491777610707849, 1, 1], 
       Hue[0.9491777610707849, 1, 1], 
       Hue[0.18421774847407577`, 1, 1], 
       Hue[0.18421774847407577`, 1, 1], 
       Hue[0.40213423036871093`, 1, 1]}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->{True, None},
    AxesOrigin->{0, 0},
    Epilog->
     LineBox[{{1, 8.282894589782796*^-7}, {2, 1.1202877807983794`*^-6}, {
       3, 6.401588066558767*^-7}, {4, 7.044868800422878*^-7}, {
       5, 5.063156044883944*^-7}, {6, 6.359801451412574*^-7}, {
       7, 2.8240946626931734`*^-8}, {8, 2.5856877142292223`*^-7}, {
       9, 1.8892115337134525`*^-7}, {10, 3.4125006734039956`*^-7}, {
       11, 5.949025187346364*^-7}, {12, 3.991365478167852*^-7}, {
       13, 9.308445072940111*^-8}, {14, 4.5679961770593727`*^-7}, {
       15, 4.261737112417841*^-7}, {16, 7.24800811334896*^-7}, {
       17, 6.665932762372686*^-7}, {18, 7.570937846972064*^-7}, {
       19, 4.5653925275661676`*^-7}, {20, 8.792856427796261*^-8}, {
       21, 2.0756682956855616`*^-7}, {22, 1.4595892735127165`*^-7}, {
       23, 4.128528806868028*^-7}, {24, 8.025050299090712*^-8}, {
       25, 3.0109430416895856`*^-7}, {26, 6.32527447117659*^-7}, {
       27, 7.220476592701592*^-7}, {28, 6.078546112048344*^-7}, {
       29, 2.836595495698056*^-7}, {30, 7.817803951595045*^-8}, {
       31, 4.130836281500989*^-7}, {32, 8.077501389718034*^-7}, {
       33, 7.695854456873939*^-7}, {34, 1.085921208119925*^-6}, {
       35, 8.271624497134395*^-7}, {36, 8.101373205921064*^-7}, {
       37, 6.602088573970645*^-7}, {38, 1.254642413330229*^-6}, {
       39, 7.852945997554673*^-7}, {40, 1.4316997818164336`*^-6}, {
       41, 1.155711798328248*^-6}, {42, 1.364144920999599*^-6}, {
       43, 1.6561765717457883`*^-7}, {44, 2.8662991973381915`*^-7}, {
       45, 2.4458657582565824`*^-7}, {46, 2.9177512669774036`*^-7}, {
       47, 5.36454314871079*^-7}, {48, 7.171172377656772*^-7}, {
       49, 9.139045972448797*^-7}, {50, 1.1798845935274491`*^-6}, {
       51, 1.087532731691727*^-6}, {52, 1.3128950449757542`*^-6}, {
       53, 8.650038653407684*^-7}, {54, 9.860043111151074*^-7}, {
       55, 8.160602897951036*^-7}, {56, 4.202865967114354*^-7}, {
       57, 7.461161585658179*^-7}, {58, 1.2222415058272401`*^-6}, {
       59, 1.3812463545144571`*^-6}, {60, 1.943249590853916*^-6}, {
       61, 2.7829321649514686`*^-6}, {62, 2.517934741324609*^-6}, {
       63, 2.3081514522406843`*^-6}, {64, 1.2016925075575837`*^-6}, {
       65, 6.561629017829647*^-7}, {66, 2.639759589456634*^-6}, {
       67, 5.830265056689041*^-6}, {68, 8.096040582559952*^-6}, {
       69, 9.49287197447064*^-6}, {70, 8.368529338993974*^-6}, {
       71, 3.4127028985921227`*^-6}, {72, 4.483027776331109*^-6}, {
       73, 0.000016050128525853894`}, {74, 0.000027151677460214952`}, {
       75, 0.000035505285315733285`}, {76, 0.00003660924174679686}, {
       77, 0.000025647597058514927`}, {78, 8.356035870788908*^-7}, {
       79, 0.00004009766811245034}, {80, 0.00008602760772684786}, {
       81, 0.00012642893916107407`}, {82, 0.00014365102492685858`}, {
       83, 0.00012318069575425173`}, {84, 0.00005126437237197791}, {
       85, 0.00007882500105720271}, {86, 0.00023778525628870314`}, {
       87, 0.0003930665144030543}, {88, 0.0004912575420470028}, {
       89, 0.0004727311763910969}, {90, 0.00029430748580079206`}, {
       91, 0.00011740502345334292`}, {92, 0.0005769127362770662}, {
       93, 0.0010928114531620662`}, {94, 0.0014777623710194756`}, {
       95, 0.001549272439795872}, {96, 0.001161992173624931}, {
       97, 0.0003942926735988468}, {98, 0.001243140552018069}, {
       99, 0.002747803634833306}, {100, 0.004007286629535681}, {
       101, 0.004501994724711602}, {102, 0.0038064319734818185`}, {
       103, 0.0018781085646712083`}, {104, 0.0023593124203412546`}, {
       105, 0.0062396105389808925`}, {106, 0.009891904213206754}, {
       107, 0.011915409055655536`}, {108, 0.011161803479006237`}, {
       109, 0.007197329097712717}, {110, 0.004178384670773133}, {
       111, 0.012301690197007564`}, {112, 0.022052108560225263`}, {
       113, 0.029248867161787675`}, {114, 0.031050819450525107`}, {
       115, 0.02569362070221519}, {116, 0.014640225612128084`}, {
       117, 0.017601648250927453`}, {118, 0.040028297599298435`}, {
       119, 0.06389930114848874}, {120, 0.08222092136134718}, {
       121, 0.08983109658350619}, {122, 0.08341276279776674}, {
       123, 0.06361073359505347}, {124, 0.0463297536197526}, {
       125, 0.07569790584388097}, {126, 0.13985529758348023`}, {
       127, 0.21704656685070572`}, {128, 0.29910422914198026`}, {
       129, 0.3808455360779263}, {130, 0.4583950615642814}, {
       131, 0.5289761690873539}, {132, 0.5908207667302015}, {
       133, 0.6430361002137149}, {134, 0.6854282144974818}, {
       135, 0.7183048103294406}, {136, 0.7423027731031244}, {
       137, 0.7582383920951351}, {138, 0.767002957596903}, {
       139, 0.7694888277761736}, {140, 0.7665396547500308}, {
       141, 0.7589294406918144}, {142, 0.7473555060335769}, {
       143, 0.7324333712853562}, {144, 0.714705652641966}, {
       145, 0.6946481542498032}, {146, 0.6726809941314018}, {
       147, 0.6491733320677741}, {148, 0.6244535646622339}, {
       149, 0.5988134359622094}, {150, 0.5725150569269272}, {
       151, 0.5457907809056177}, {152, 0.5188517459204195}, {
       153, 0.4918850974657282}, {154, 0.4650609426653706}, {
       155, 0.4385284580432407}, {156, 0.41242391380365534`}, {
       157, 0.38686449193825734`}, {158, 0.36195245822798616`}, {
       159, 0.3377778499806569}, {160, 0.31441637387129057`}, {
       161, 0.29192950025027925`}, {162, 0.2703679794461138}, {
       163, 0.24976980536518267`}, {164, 0.23016402587299736`}, {
       165, 0.21156662835981846`}, {166, 0.19398778945503897`}, {
       167, 0.17742723439500027`}, {168, 0.1618792319502118}, {
       169, 0.14732734151510063`}, {170, 0.13375128083363577`}, {
       171, 0.12112680945088165`}, {172, 0.10942268349709008`}, {
       173, 0.09860594824919854}, {174, 0.08863999779100278}, {
       175, 0.07948470290587986}, {176, 0.0710992520080257}, {
       177, 0.06344223741638448}, {178, 0.056470654955747025`}, {
       179, 0.050141861919215}, {180, 0.0444125623705725}, {
       181, 0.03924076322558548}, {182, 0.034586353462668064`}, {
       183, 0.030409361724688326`}, {184, 0.02667092254792324}, {
       185, 0.02333450808967844}, {186, 0.020364749863763312`}, {
       187, 0.01772947898612808}, {188, 0.015397459441858894`}, {
       189, 0.013338801356442501`}, {190, 0.011526950765240997`}, {
       191, 0.009936577884119712}, {192, 0.008544951968553446}, {
       193, 0.007330315677783491}, {194, 0.006271892728590545}, {
       195, 0.0053528491714311946`}, {196, 0.00455780672792188}, {
       197, 0.0038711876315099597`}, {198, 0.003279415810681045}, {
       199, 0.0027714821274414845`}, {200, 0.002336752306047296}, {
       201, 0.0019650292793226}, {202, 0.0016482241262018693`}, {
       203, 0.0013794275726454264`}, {204, 0.0011515286174456506`}, {
       205, 0.0009585524383507281}, {206, 0.0007962705305678472}, {
       207, 0.000660047342607368}, {208, 0.0005453469371544426}, {
       209, 0.00044967000914317384`}, {210, 0.00037030170950913175`}, {
       211, 0.0003035938337062031}, {212, 0.00024793018820254176`}, {
       213, 0.00020264624208489855`}, {214, 0.0001654277558815003}, {
       215, 0.0001342232075630044}, {216, 0.0001086588848457038}, {
       217, 0.00008809691164050676}, {218, 0.0000710648938132382}, {
       219, 0.000056791536662500116`}, {220, 0.000045352747704607155`}, {
       221, 0.000036395800431590996`}, {222, 0.000029023543349811118`}, {
       223, 0.00002283672780455612}, {224, 0.000017976851260834618`}, {
       225, 0.000014276745780969344`}, {226, 0.000011258062664574723`}, {
       227, 8.774733210704453*^-6}, {228, 6.953985392772539*^-6}, {
       229, 5.653861783513688*^-6}, {230, 4.472346382540365*^-6}, {
       231, 3.281034507168652*^-6}, {232, 2.4224831741165236`*^-6}, {
       233, 2.148626485932848*^-6}, {234, 1.9820437854623916`*^-6}, {
       235, 1.5333194641015504`*^-6}, {236, 9.581124662188444*^-7}, {
       237, 6.973707942666438*^-7}, {238, 7.532124124138606*^-7}, {
       239, 7.363704676220984*^-7}, {240, 6.509440332529519*^-7}, {
       241, 6.190513563661241*^-7}, {242, 6.05922147678224*^-7}, {
       243, 5.153985542583597*^-7}, {244, 3.2611470458455117`*^-7}, {
       245, 1.3622647588506856`*^-7}, {246, 3.0766703331644535`*^-7}, {
       247, 5.335819969931085*^-7}, {248, 6.501272689296098*^-7}, {
       249, 6.209152995562534*^-7}, {250, 4.6385849328152057`*^-7}, {
       251, 2.4084287890744645`*^-7}}],
    Frame->True,
    FrameTicks->{{{1., 
        FormBox[
         RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
        FormBox[
         RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
        FormBox[
         RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
        FormBox[
         RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
        FormBox["0", TraditionalForm]}, {157.25, 
        FormBox["2.5`", TraditionalForm]}, {188.5, 
        FormBox["5", TraditionalForm]}, {219.75, 
        FormBox["7.5`", TraditionalForm]}, {251., 
        FormBox["10", TraditionalForm]}}, Automatic, None, None},
    PlotRange->{-0.1, 1.1},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], Automatic}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF12dYFOcaBuAB9WiiBkWMqBBWJURJMIAUFZDHBoqNIlWQYdmll8Ry7Dhi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     "], {
      {GrayLevel[0.5], EdgeForm[None], 
       GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ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         "]]]}, {{}, {}, {
        {Hue[0.5491258269772031, 1, 1], LineBox[{1, 2}]}, 
        {Hue[0.2902796450076607, 1, 1], LineBox[{2, 3}]}, 
        {Hue[0.3186348964907807, 1, 1], LineBox[{3, 4}]}, 
        {Hue[0.2284208522423258, 1, 1], LineBox[{4, 5}]}, 
        {Hue[0.12232320112241715`, 1, 1], LineBox[{5, 6}]}, 
        {Hue[0.005010772832046706, 1, 1], LineBox[{6, 7}]}, 
        {Hue[0.9903585419751219, 1, 1], LineBox[{7, 8}]}, 
        {Hue[0.9685718751320311, 1, 1], LineBox[{8, 9}]}, 
        {Hue[0.7718361162578176, 1, 1], LineBox[{9, 10}]}, 
        {Hue[0.5534039765471076, 1, 1], LineBox[{10, 11}]}, 
        {Hue[0.23758362191745483`, 1, 1], LineBox[{11, 12}]}, 
        {Hue[0.6659801651782383, 1, 1], LineBox[{12, 13}]}, 
        {Hue[0.49931996895800457`, 1, 1], LineBox[{13, 14}]}, 
        {Hue[0.5318520191878946, 1, 1], LineBox[{14, 15}]}, 
        {Hue[0.30564357579768725`, 1, 1], LineBox[{15, 16}]}, 
        {Hue[0.9594404820243042, 1, 1], LineBox[{16, 17}]}, 
        {Hue[0.7149204571508756, 1, 1], LineBox[{17, 18}]}, 
        {Hue[0.6678921749008587, 1, 1], LineBox[{18, 19}]}, 
        {Hue[0.44835131514272236`, 1, 1], LineBox[{19, 20}]}, 
        {Hue[0.48327914376204195`, 1, 1], LineBox[{20, 21}]}, 
        {Hue[0.24750800559439334`, 1, 1], LineBox[{21, 22}]}, 
        {Hue[0.11519603984478773`, 1, 1], LineBox[{22, 23}]}, 
        {Hue[0.01612414670244009, 1, 1], LineBox[{23, 24}]}, 
        {Hue[0.8180816680337696, 1, 1], LineBox[{24, 25}]}, 
        {Hue[0.6573322503084915, 1, 1], LineBox[{25, 26}]}, 
        {Hue[0.5305707884219157, 1, 1], LineBox[{26, 27}]}, 
        {Hue[0.49633086365786905`, 1, 1], LineBox[{27, 28}]}, 
        {Hue[0.3204740927659566, 1, 1], LineBox[{28, 29}]}, 
        {Hue[0.2517279746461726, 1, 1], LineBox[{29, 30}]}, 
        {Hue[0.1268451327024544, 1, 1], LineBox[{30, 31}]}, 
        {Hue[0.04492836641888041, 1, 1], LineBox[{31, 32}]}, 
        {Hue[0.8506860772805684, 1, 1], LineBox[{32, 33}]}, 
        {Hue[0.7245694467189367, 1, 1], LineBox[{33, 34}]}, 
        {Hue[0.7509139492274237, 1, 1], LineBox[{34, 35}]}, 
        {Hue[0.6843863315113896, 1, 1], LineBox[{35, 36}]}, 
        {Hue[0.5370274938430456, 1, 1], LineBox[{36, 37}]}, 
        {Hue[0.49195828763052996`, 1, 1], LineBox[{37, 38}]}, 
        {Hue[0.308347341344207, 1, 1], LineBox[{38, 39}]}, 
        {Hue[0.12705074290335513`, 1, 1], LineBox[{39, 40}]}, 
        {Hue[0.10331329426250534`, 1, 1], LineBox[{40, 41}]}, 
        {Hue[0.982509099525184, 1, 1], LineBox[{41, 42}]}, 
        {Hue[0.787209221145821, 1, 1], LineBox[{42, 43}]}, 
        {Hue[0.7700877038771718, 1, 1], LineBox[{43, 44}]}, 
        {Hue[0.6361643271409572, 1, 1], LineBox[{44, 45}]}, 
        {Hue[0.6068466023971579, 1, 1], LineBox[{45, 46}]}, 
        {Hue[0.4791298717905439, 1, 1], LineBox[{46, 47}]}, 
        {Hue[0.4357091773859966, 1, 1], LineBox[{47, 48}]}, 
        {Hue[0.33105954022102446`, 1, 1], LineBox[{48, 49}]}, 
        {Hue[0.2141133712894142, 1, 1], LineBox[{49, 50}]}, 
        {Hue[0.15756837809381175`, 1, 1], LineBox[{50, 51}]}, 
        {Hue[0.9332747186039484, 1, 1], LineBox[{51, 52}]}, 
        {Hue[0.9791335752533095, 1, 1], LineBox[{52, 53}]}, 
        {Hue[0.8384726387092356, 1, 1], LineBox[{53, 54}]}, 
        {Hue[0.7364619005298396, 1, 1], LineBox[{54, 55}]}, 
        {Hue[0.6629901252207137, 1, 1], LineBox[{55, 56}]}, 
        {Hue[0.5783395681331325, 1, 1], LineBox[{56, 57}]}, 
        {Hue[0.4684978175622351, 1, 1], LineBox[{57, 58}]}, 
        {Hue[0.37244889106677387`, 1, 1], LineBox[{58, 59}]}, 
        {Hue[0.29411967947531414`, 1, 1], LineBox[{59, 60}]}, 
        {Hue[0.19682968958288313`, 1, 1], LineBox[{60, 61}]}, 
        {Hue[0.08992445985436712, 1, 1], LineBox[{61, 62}]}, 
        {Hue[0.9933438259434542, 1, 1], LineBox[{62, 63}]}, 
        {Hue[0.8724541681175237, 1, 1], LineBox[{63, 64}]}, 
        {Hue[0.7760773796938453, 1, 1], LineBox[{64, 65}]}, 
        {Hue[0.6796254362241916, 1, 1], LineBox[{65, 66}]}, 
        {Hue[0.5939543082492782, 1, 1], LineBox[{66, 67}]}, 
        {Hue[0.5028392579847493, 1, 1], LineBox[{67, 68}]}, 
        {Hue[0.4036498776674032, 1, 1], LineBox[{68, 69}]}, 
        {Hue[0.3003020928823953, 1, 1], LineBox[{69, 70}]}, 
        {Hue[0.1967022508173054, 1, 1], LineBox[{70, 71}]}, 
        {Hue[0.09650347712384397, 1, 1], LineBox[{71, 72}]}, 
        {Hue[0.0027723402406860794`, 1, 1], LineBox[{72, 73}]}, 
        {Hue[0.9113108697618242, 1, 1], LineBox[{73, 74}]}, 
        {Hue[0.8187928748678927, 1, 1], LineBox[{74, 75}]}, 
        {Hue[0.7208690487729366, 1, 1], LineBox[{75, 76}]}, 
        {Hue[0.6185444591778555, 1, 1], LineBox[{76, 77}]}, 
        {Hue[0.5165902196272736, 1, 1], LineBox[{77, 78}]}, 
        {Hue[0.42051768332086603`, 1, 1], LineBox[{78, 79}]}, 
        {Hue[0.33070211397551663`, 1, 1], LineBox[{79, 80}]}, 
        {Hue[0.2426103255226291, 1, 1], LineBox[{80, 81}]}, 
        {Hue[0.15138932879895664`, 1, 1], LineBox[{81, 82}]}, 
        {Hue[0.05249602680951814, 1, 1], LineBox[{82, 83}]}, 
        {Hue[0.9478367133607696, 1, 1], LineBox[{83, 84}]}, 
        {Hue[0.8475875861469279, 1, 1], LineBox[{84, 85}]}, 
        {Hue[0.756300580286778, 1, 1], LineBox[{85, 86}]}, 
        {Hue[0.6716889199006466, 1, 1], LineBox[{86, 87}]}, 
        {Hue[0.5870825798302917, 1, 1], LineBox[{87, 88}]}, 
        {Hue[0.495050509223539, 1, 1], LineBox[{88, 89}]}, 
        {Hue[0.39262780127384156`, 1, 1], LineBox[{89, 90}]}, 
        {Hue[0.28714126331748524`, 1, 1], LineBox[{90, 91}]}, 
        {Hue[0.19112805442353883`, 1, 1], LineBox[{91, 92}]}, 
        {Hue[0.10673525827672053`, 1, 1], LineBox[{92, 93}]}, 
        {Hue[0.027522840384905385`, 1, 1], LineBox[{93, 94}]}, 
        {Hue[0.9444803685835561, 1, 1], LineBox[{94, 95}]}, 
        {Hue[0.8490249070559226, 1, 1], LineBox[{95, 96}]}, 
        {Hue[0.7411649498814663, 1, 1], LineBox[{96, 97}]}, 
        {Hue[0.6374666215429812, 1, 1], LineBox[{97, 98}]}, 
        {Hue[0.5497931223572055, 1, 1], LineBox[{98, 99}]}, 
        {Hue[0.4737291123864164, 1, 1], LineBox[{99, 100}]}, 
        {Hue[0.3992080090157962, 1, 1], LineBox[{100, 101}]}, 
        {Hue[0.31482774791091384`, 1, 1], LineBox[{101, 102}]}, 
        {Hue[0.2113292305432758, 1, 1], LineBox[{102, 103}]}, 
        {Hue[0.09908154232394249, 1, 1], LineBox[{103, 104}]}, 
        {Hue[0.003580719173392775, 1, 1], LineBox[{104, 105}]}, 
        {Hue[0.9277707136658423, 1, 1], LineBox[{105, 106}]}, 
        {Hue[0.8605404856953217, 1, 1], LineBox[{106, 107}]}, 
        {Hue[0.7895048468780809, 1, 1], LineBox[{107, 108}]}, 
        {Hue[0.7002054335577579, 1, 1], LineBox[{108, 109}]}, 
        {Hue[0.5856439885046414, 1, 1], LineBox[{109, 110}]}, 
        {Hue[0.47569786975257744`, 1, 1], LineBox[{110, 111}]}, 
        {Hue[0.3944836420368127, 1, 1], LineBox[{111, 112}]}, 
        {Hue[0.33207192193375373`, 1, 1], LineBox[{112, 113}]}, 
        {Hue[0.2748339851375885, 1, 1], LineBox[{113, 114}]}, 
        {Hue[0.20987049241422776`, 1, 1], LineBox[{114, 115}]}, 
        {Hue[0.11924584561484784`, 1, 1], LineBox[{115, 116}]}, 
        {Hue[0.9968614524798072, 1, 1], LineBox[{116, 117}]}, 
        {Hue[0.8915858603078806, 1, 1], LineBox[{117, 118}]}, 
        {Hue[0.8232626433318473, 1, 1], LineBox[{118, 119}]}, 
        {Hue[0.7746340198278019, 1, 1], LineBox[{119, 120}]}, 
        {Hue[0.7331489905094091, 1, 1], LineBox[{120, 121}]}, 
        {Hue[0.6902763475191502, 1, 1], LineBox[{121, 122}]}, 
        {Hue[0.6346335888705981, 1, 1], LineBox[{122, 123}]}, 
        {Hue[0.5435719785474447, 1, 1], LineBox[{123, 124}]}, 
        {Hue[0.4138905816180223, 1, 1], LineBox[{124, 125}]}, 
        {Hue[0.3198427828740491, 1, 1], LineBox[{125, 126}]}, 
        {Hue[0.2689200438620375, 1, 1], LineBox[{126, 127}]}, 
        {Hue[0.23808301889067057`, 1, 1], LineBox[{127, 128}]}, 
        {Hue[0.21700129017557332`, 1, 1], LineBox[{128, 129}]}, 
        {Hue[0.2015965309429917, 1, 1], LineBox[{129, 130}]}, 
        {Hue[0.1900849351301606, 1, 1], LineBox[{130, 131}]}, 
        {Hue[0.18161565069571153`, 1, 1], LineBox[{131, 132}]}, 
        {Hue[0.17576281859083673`, 1, 1], LineBox[{132, 133}]}, 
        {Hue[0.17231111257394194`, 1, 1], LineBox[{133, 134}]}, 
        {Hue[0.171157003537818, 1, 1], LineBox[{134, 135}]}, 
        {Hue[0.17225561589785562`, 1, 1], LineBox[{135, 136}]}, 
        {Hue[0.1755919846585013, 1, 1], LineBox[{136, 137}]}, 
        {Hue[0.18116294003375108`, 1, 1], LineBox[{137, 138}]}, 
        {Hue[0.18896824793199118`, 1, 1], LineBox[{138, 139}]}, 
        {Hue[0.19900410308070351`, 1, 1], LineBox[{139, 140}]}, 
        {Hue[0.21126095026360003`, 1, 1], LineBox[{140, 141}]}, 
        {Hue[0.2257205157148169, 1, 1], LineBox[{141, 142}]}, 
        {Hue[0.24235848928686538`, 1, 1], LineBox[{142, 143}]}, 
        {Hue[0.2611420249768013, 1, 1], LineBox[{143, 144}]}, 
        {Hue[0.2820336741647051, 1, 1], LineBox[{144, 145}]}, 
        {Hue[0.3049911010578967, 1, 1], LineBox[{145, 146}]}, 
        {Hue[0.32996816191755635`, 1, 1], LineBox[{146, 147}]}, 
        {Hue[0.3569163337053618, 1, 1], LineBox[{147, 148}]}, 
        {Hue[0.38578591401286044`, 1, 1], LineBox[{148, 149}]}, 
        {Hue[0.4165259364321822, 1, 1], LineBox[{149, 150}]}, 
        {Hue[0.4490866391723868, 1, 1], LineBox[{150, 151}]}, 
        {Hue[0.4834177366510568, 1, 1], LineBox[{151, 152}]}, 
        {Hue[0.5194701836085935, 1, 1], LineBox[{152, 153}]}, 
        {Hue[0.557196825271485, 1, 1], LineBox[{153, 154}]}, 
        {Hue[0.5965518215974968, 1, 1], LineBox[{154, 155}]}, 
        {Hue[0.6374904602151458, 1, 1], LineBox[{155, 156}]}, 
        {Hue[0.6799708772044095, 1, 1], LineBox[{156, 157}]}, 
        {Hue[0.7239522976890813, 1, 1], LineBox[{157, 158}]}, 
        {Hue[0.769396386916378, 1, 1], LineBox[{158, 159}]}, 
        {Hue[0.8162657220231337, 1, 1], LineBox[{159, 160}]}, 
        {Hue[0.8645258254775823, 1, 1], LineBox[{160, 161}]}, 
        {Hue[0.9141429735217279, 1, 1], LineBox[{161, 162}]}, 
        {Hue[0.96508566814464, 1, 1], LineBox[{162, 163}]}, 
        {Hue[0.017324549387385067`, 1, 1], LineBox[{163, 164}]}, 
        {Hue[0.07083024111942525, 1, 1], LineBox[{164, 165}]}, 
        {Hue[0.12557661019562788`, 1, 1], LineBox[{165, 166}]}, 
        {Hue[0.18153833105298484`, 1, 1], LineBox[{166, 167}]}, 
        {Hue[0.23869059105266235`, 1, 1], LineBox[{167, 168}]}, 
        {Hue[0.29701003205840226`, 1, 1], LineBox[{168, 169}]}, 
        {Hue[0.3564750346896973, 1, 1], LineBox[{169, 170}]}, 
        {Hue[0.4170657262681432, 1, 1], LineBox[{170, 171}]}, 
        {Hue[0.47876096011285807`, 1, 1], LineBox[{171, 172}]}, 
        {Hue[0.5415437206250882, 1, 1], LineBox[{172, 173}]}, 
        {Hue[0.60539444097929, 1, 1], LineBox[{173, 174}]}, 
        {Hue[0.6702963986746799, 1, 1], LineBox[{174, 175}]}, 
        {Hue[0.7362341144996116, 1, 1], LineBox[{175, 176}]}, 
        {Hue[0.8031923601931057, 1, 1], LineBox[{176, 177}]}, 
        {Hue[0.8711544244586391, 1, 1], LineBox[{177, 178}]}, 
        {Hue[0.9401074600065064, 1, 1], LineBox[{178, 179}]}, 
        {Hue[0.010038309244361136`, 1, 1], LineBox[{179, 180}]}, 
        {Hue[0.08093288218141492, 1, 1], LineBox[{180, 181}]}, 
        {Hue[0.15277982353789266`, 1, 1], LineBox[{181, 182}]}, 
        {Hue[0.22556566530655825`, 1, 1], LineBox[{182, 183}]}, 
        {Hue[0.29928098086556476`, 1, 1], LineBox[{183, 184}]}, 
        {Hue[0.37391601020656645`, 1, 1], LineBox[{184, 185}]}, 
        {Hue[0.4494558054736254, 1, 1], LineBox[{185, 186}]}, 
        {Hue[0.5258909455116205, 1, 1], LineBox[{186, 187}]}, 
        {Hue[0.6032152788066548, 1, 1], LineBox[{187, 188}]}, 
        {Hue[0.6814168667752996, 1, 1], LineBox[{188, 189}]}, 
        {Hue[0.7604874600919083, 1, 1], LineBox[{189, 190}]}, 
        {Hue[0.840418989301121, 1, 1], LineBox[{190, 191}]}, 
        {Hue[0.9211998295028575, 1, 1], LineBox[{191, 192}]}, 
        {Hue[0.0028226054862767027`, 1, 1], LineBox[{192, 193}]}, 
        {Hue[0.08528186990748743, 1, 1], LineBox[{193, 194}]}, 
        {Hue[0.16856912526236964`, 1, 1], LineBox[{194, 195}]}, 
        {Hue[0.2526801036479619, 1, 1], LineBox[{195, 196}]}, 
        {Hue[0.3376012187689896, 1, 1], LineBox[{196, 197}]}, 
        {Hue[0.4233218065639601, 1, 1], LineBox[{197, 198}]}, 
        {Hue[0.5098467270130229, 1, 1], LineBox[{198, 199}]}, 
        {Hue[0.5971651206611441, 1, 1], LineBox[{199, 200}]}, 
        {Hue[0.6852741643229945, 1, 1], LineBox[{200, 201}]}, 
        {Hue[0.7741608030557923, 1, 1], LineBox[{201, 202}]}, 
        {Hue[0.8638069094138561, 1, 1], LineBox[{202, 203}]}, 
        {Hue[0.9542258377497206, 1, 1], LineBox[{203, 204}]}, 
        {Hue[0.0454115137177682, 1, 1], LineBox[{204, 205}]}, 
        {Hue[0.13735385516522644`, 1, 1], LineBox[{205, 206}]}, 
        {Hue[0.23004905610161971`, 1, 1], LineBox[{206, 207}]}, 
        {Hue[0.3234724822822555, 1, 1], LineBox[{207, 208}]}, 
        {Hue[0.4176385390663141, 1, 1], LineBox[{208, 209}]}, 
        {Hue[0.5125795503518117, 1, 1], LineBox[{209, 210}]}, 
        {Hue[0.6082348081135551, 1, 1], LineBox[{210, 211}]}, 
        {Hue[0.7045341876617609, 1, 1], LineBox[{211, 212}]}, 
        {Hue[0.801554485238327, 1, 1], LineBox[{212, 213}]}, 
        {Hue[0.8993717427787195, 1, 1], LineBox[{213, 214}]}, 
        {Hue[0.9978884459814782, 1, 1], LineBox[{214, 215}]}, 
        {Hue[0.09703239989098028, 1, 1], LineBox[{215, 216}]}, 
        {Hue[0.1969254467984167, 1, 1], LineBox[{216, 217}]}, 
        {Hue[0.29759581352233877`, 1, 1], LineBox[{217, 218}]}, 
        {Hue[0.39884143768934943`, 1, 1], LineBox[{218, 219}]}, 
        {Hue[0.5007358447293813, 1, 1], LineBox[{219, 220}]}, 
        {Hue[0.6035162687108159, 1, 1], LineBox[{220, 221}]}, 
        {Hue[0.706861582499672, 1, 1], LineBox[{221, 222}]}, 
        {Hue[0.8106624428033508, 1, 1], LineBox[{222, 223}]}, 
        {Hue[0.9155282350120744, 1, 1], LineBox[{223, 224}]}, 
        {Hue[0.021180426611272417`, 1, 1], LineBox[{224, 225}]}, 
        {Hue[0.12692796944485524`, 1, 1], LineBox[{225, 226}]}, 
        {Hue[0.233700700997123, 1, 1], LineBox[{226, 227}]}, 
        {Hue[0.341946802462442, 1, 1], LineBox[{227, 228}]}, 
        {Hue[0.4498968429690764, 1, 1], LineBox[{228, 229}]}, 
        {Hue[0.5573824103226421, 1, 1], LineBox[{229, 230}]}, 
        {Hue[0.6670150151902048, 1, 1], LineBox[{230, 231}]}, 
        {Hue[0.7789105505834107, 1, 1], LineBox[{231, 232}]}, 
        {Hue[0.8902108107720045, 1, 1], LineBox[{232, 233}]}, 
        {Hue[0.0010083993451186876`, 1, 1], LineBox[{233, 234}]}, 
        {Hue[0.11447518261633102`, 1, 1], LineBox[{234, 235}]}, 
        {Hue[0.2295246674513252, 1, 1], LineBox[{235, 236}]}, 
        {Hue[0.34142421738429607`, 1, 1], LineBox[{236, 237}]}, 
        {Hue[0.451574669431246, 1, 1], LineBox[{237, 238}]}, 
        {Hue[0.5666763034107078, 1, 1], LineBox[{238, 239}]}, 
        {Hue[0.6850507481624744, 1, 1], LineBox[{239, 240}]}, 
        {Hue[0.7984248249966265, 1, 1], LineBox[{240, 241}]}, 
        {Hue[0.9072367859105431, 1, 1], LineBox[{241, 242}]}, 
        {Hue[0.021987436943994295`, 1, 1], LineBox[{242, 243}]}, 
        {Hue[0.1426344355952877, 1, 1], LineBox[{243, 244}]}, 
        {Hue[0.2575711118171542, 1, 1], LineBox[{244, 245}]}, 
        {Hue[0.3682158022952389, 1, 1], LineBox[{245, 246}]}, 
        {Hue[0.49806656828861817`, 1, 1], LineBox[{246, 247}]}, 
        {Hue[0.649111403365899, 1, 1], LineBox[{247, 248}]}, 
        {Hue[0.7870001113529548, 1, 1], LineBox[{248, 249}]}, 
        {Hue[0.8400106170584366, 1, 1], LineBox[{249, 250}]}, 
        {Hue[0.9521427724380148, 1, 1], LineBox[{250, 251}]}}}},
     VertexColors->{
       Hue[0.5491258269772031, 1, 1], 
       Hue[0.2902796450076607, 1, 1], 
       Hue[0.3186348964907807, 1, 1], 
       Hue[0.2284208522423258, 1, 1], 
       Hue[0.12232320112241715`, 1, 1], 
       Hue[0.005010772832046706, 1, 1], 
       Hue[0.9903585419751219, 1, 1], 
       Hue[0.9685718751320311, 1, 1], 
       Hue[0.7718361162578176, 1, 1], 
       Hue[0.5534039765471076, 1, 1], 
       Hue[0.23758362191745483`, 1, 1], 
       Hue[0.6659801651782383, 1, 1], 
       Hue[0.49931996895800457`, 1, 1], 
       Hue[0.5318520191878946, 1, 1], 
       Hue[0.30564357579768725`, 1, 1], 
       Hue[0.9594404820243042, 1, 1], 
       Hue[0.7149204571508756, 1, 1], 
       Hue[0.6678921749008587, 1, 1], 
       Hue[0.44835131514272236`, 1, 1], 
       Hue[0.48327914376204195`, 1, 1], 
       Hue[0.24750800559439334`, 1, 1], 
       Hue[0.11519603984478773`, 1, 1], 
       Hue[0.01612414670244009, 1, 1], 
       Hue[0.8180816680337696, 1, 1], 
       Hue[0.6573322503084915, 1, 1], 
       Hue[0.5305707884219157, 1, 1], 
       Hue[0.49633086365786905`, 1, 1], 
       Hue[0.3204740927659566, 1, 1], 
       Hue[0.2517279746461726, 1, 1], 
       Hue[0.1268451327024544, 1, 1], 
       Hue[0.04492836641888041, 1, 1], 
       Hue[0.8506860772805684, 1, 1], 
       Hue[0.7245694467189367, 1, 1], 
       Hue[0.7509139492274237, 1, 1], 
       Hue[0.6843863315113896, 1, 1], 
       Hue[0.5370274938430456, 1, 1], 
       Hue[0.49195828763052996`, 1, 1], 
       Hue[0.308347341344207, 1, 1], 
       Hue[0.12705074290335513`, 1, 1], 
       Hue[0.10331329426250534`, 1, 1], 
       Hue[0.982509099525184, 1, 1], 
       Hue[0.787209221145821, 1, 1], 
       Hue[0.7700877038771718, 1, 1], 
       Hue[0.6361643271409572, 1, 1], 
       Hue[0.6068466023971579, 1, 1], 
       Hue[0.4791298717905439, 1, 1], 
       Hue[0.4357091773859966, 1, 1], 
       Hue[0.33105954022102446`, 1, 1], 
       Hue[0.2141133712894142, 1, 1], 
       Hue[0.15756837809381175`, 1, 1], 
       Hue[0.9332747186039484, 1, 1], 
       Hue[0.9791335752533095, 1, 1], 
       Hue[0.8384726387092356, 1, 1], 
       Hue[0.7364619005298396, 1, 1], 
       Hue[0.6629901252207137, 1, 1], 
       Hue[0.5783395681331325, 1, 1], 
       Hue[0.4684978175622351, 1, 1], 
       Hue[0.37244889106677387`, 1, 1], 
       Hue[0.29411967947531414`, 1, 1], 
       Hue[0.19682968958288313`, 1, 1], 
       Hue[0.08992445985436712, 1, 1], 
       Hue[0.9933438259434542, 1, 1], 
       Hue[0.8724541681175237, 1, 1], 
       Hue[0.7760773796938453, 1, 1], 
       Hue[0.6796254362241916, 1, 1], 
       Hue[0.5939543082492782, 1, 1], 
       Hue[0.5028392579847493, 1, 1], 
       Hue[0.4036498776674032, 1, 1], 
       Hue[0.3003020928823953, 1, 1], 
       Hue[0.1967022508173054, 1, 1], 
       Hue[0.09650347712384397, 1, 1], 
       Hue[0.0027723402406860794`, 1, 1], 
       Hue[0.9113108697618242, 1, 1], 
       Hue[0.8187928748678927, 1, 1], 
       Hue[0.7208690487729366, 1, 1], 
       Hue[0.6185444591778555, 1, 1], 
       Hue[0.5165902196272736, 1, 1], 
       Hue[0.42051768332086603`, 1, 1], 
       Hue[0.33070211397551663`, 1, 1], 
       Hue[0.2426103255226291, 1, 1], 
       Hue[0.15138932879895664`, 1, 1], 
       Hue[0.05249602680951814, 1, 1], 
       Hue[0.9478367133607696, 1, 1], 
       Hue[0.8475875861469279, 1, 1], 
       Hue[0.756300580286778, 1, 1], 
       Hue[0.6716889199006466, 1, 1], 
       Hue[0.5870825798302917, 1, 1], 
       Hue[0.495050509223539, 1, 1], 
       Hue[0.39262780127384156`, 1, 1], 
       Hue[0.28714126331748524`, 1, 1], 
       Hue[0.19112805442353883`, 1, 1], 
       Hue[0.10673525827672053`, 1, 1], 
       Hue[0.027522840384905385`, 1, 1], 
       Hue[0.9444803685835561, 1, 1], 
       Hue[0.8490249070559226, 1, 1], 
       Hue[0.7411649498814663, 1, 1], 
       Hue[0.6374666215429812, 1, 1], 
       Hue[0.5497931223572055, 1, 1], 
       Hue[0.4737291123864164, 1, 1], 
       Hue[0.3992080090157962, 1, 1], 
       Hue[0.31482774791091384`, 1, 1], 
       Hue[0.2113292305432758, 1, 1], 
       Hue[0.09908154232394249, 1, 1], 
       Hue[0.003580719173392775, 1, 1], 
       Hue[0.9277707136658423, 1, 1], 
       Hue[0.8605404856953217, 1, 1], 
       Hue[0.7895048468780809, 1, 1], 
       Hue[0.7002054335577579, 1, 1], 
       Hue[0.5856439885046414, 1, 1], 
       Hue[0.47569786975257744`, 1, 1], 
       Hue[0.3944836420368127, 1, 1], 
       Hue[0.33207192193375373`, 1, 1], 
       Hue[0.2748339851375885, 1, 1], 
       Hue[0.20987049241422776`, 1, 1], 
       Hue[0.11924584561484784`, 1, 1], 
       Hue[0.9968614524798072, 1, 1], 
       Hue[0.8915858603078806, 1, 1], 
       Hue[0.8232626433318473, 1, 1], 
       Hue[0.7746340198278019, 1, 1], 
       Hue[0.7331489905094091, 1, 1], 
       Hue[0.6902763475191502, 1, 1], 
       Hue[0.6346335888705981, 1, 1], 
       Hue[0.5435719785474447, 1, 1], 
       Hue[0.4138905816180223, 1, 1], 
       Hue[0.3198427828740491, 1, 1], 
       Hue[0.2689200438620375, 1, 1], 
       Hue[0.23808301889067057`, 1, 1], 
       Hue[0.21700129017557332`, 1, 1], 
       Hue[0.2015965309429917, 1, 1], 
       Hue[0.1900849351301606, 1, 1], 
       Hue[0.18161565069571153`, 1, 1], 
       Hue[0.17576281859083673`, 1, 1], 
       Hue[0.17231111257394194`, 1, 1], 
       Hue[0.171157003537818, 1, 1], 
       Hue[0.17225561589785562`, 1, 1], 
       Hue[0.1755919846585013, 1, 1], 
       Hue[0.18116294003375108`, 1, 1], 
       Hue[0.18896824793199118`, 1, 1], 
       Hue[0.19900410308070351`, 1, 1], 
       Hue[0.21126095026360003`, 1, 1], 
       Hue[0.2257205157148169, 1, 1], 
       Hue[0.24235848928686538`, 1, 1], 
       Hue[0.2611420249768013, 1, 1], 
       Hue[0.2820336741647051, 1, 1], 
       Hue[0.3049911010578967, 1, 1], 
       Hue[0.32996816191755635`, 1, 1], 
       Hue[0.3569163337053618, 1, 1], 
       Hue[0.38578591401286044`, 1, 1], 
       Hue[0.4165259364321822, 1, 1], 
       Hue[0.4490866391723868, 1, 1], 
       Hue[0.4834177366510568, 1, 1], 
       Hue[0.5194701836085935, 1, 1], 
       Hue[0.557196825271485, 1, 1], 
       Hue[0.5965518215974968, 1, 1], 
       Hue[0.6374904602151458, 1, 1], 
       Hue[0.6799708772044095, 1, 1], 
       Hue[0.7239522976890813, 1, 1], 
       Hue[0.769396386916378, 1, 1], 
       Hue[0.8162657220231337, 1, 1], 
       Hue[0.8645258254775823, 1, 1], 
       Hue[0.9141429735217279, 1, 1], 
       Hue[0.96508566814464, 1, 1], 
       Hue[0.017324549387385067`, 1, 1], 
       Hue[0.07083024111942525, 1, 1], 
       Hue[0.12557661019562788`, 1, 1], 
       Hue[0.18153833105298484`, 1, 1], 
       Hue[0.23869059105266235`, 1, 1], 
       Hue[0.29701003205840226`, 1, 1], 
       Hue[0.3564750346896973, 1, 1], 
       Hue[0.4170657262681432, 1, 1], 
       Hue[0.47876096011285807`, 1, 1], 
       Hue[0.5415437206250882, 1, 1], 
       Hue[0.60539444097929, 1, 1], 
       Hue[0.6702963986746799, 1, 1], 
       Hue[0.7362341144996116, 1, 1], 
       Hue[0.8031923601931057, 1, 1], 
       Hue[0.8711544244586391, 1, 1], 
       Hue[0.9401074600065064, 1, 1], 
       Hue[0.010038309244361136`, 1, 1], 
       Hue[0.08093288218141492, 1, 1], 
       Hue[0.15277982353789266`, 1, 1], 
       Hue[0.22556566530655825`, 1, 1], 
       Hue[0.29928098086556476`, 1, 1], 
       Hue[0.37391601020656645`, 1, 1], 
       Hue[0.4494558054736254, 1, 1], 
       Hue[0.5258909455116205, 1, 1], 
       Hue[0.6032152788066548, 1, 1], 
       Hue[0.6814168667752996, 1, 1], 
       Hue[0.7604874600919083, 1, 1], 
       Hue[0.840418989301121, 1, 1], 
       Hue[0.9211998295028575, 1, 1], 
       Hue[0.0028226054862767027`, 1, 1], 
       Hue[0.08528186990748743, 1, 1], 
       Hue[0.16856912526236964`, 1, 1], 
       Hue[0.2526801036479619, 1, 1], 
       Hue[0.3376012187689896, 1, 1], 
       Hue[0.4233218065639601, 1, 1], 
       Hue[0.5098467270130229, 1, 1], 
       Hue[0.5971651206611441, 1, 1], 
       Hue[0.6852741643229945, 1, 1], 
       Hue[0.7741608030557923, 1, 1], 
       Hue[0.8638069094138561, 1, 1], 
       Hue[0.9542258377497206, 1, 1], 
       Hue[0.0454115137177682, 1, 1], 
       Hue[0.13735385516522644`, 1, 1], 
       Hue[0.23004905610161971`, 1, 1], 
       Hue[0.3234724822822555, 1, 1], 
       Hue[0.4176385390663141, 1, 1], 
       Hue[0.5125795503518117, 1, 1], 
       Hue[0.6082348081135551, 1, 1], 
       Hue[0.7045341876617609, 1, 1], 
       Hue[0.801554485238327, 1, 1], 
       Hue[0.8993717427787195, 1, 1], 
       Hue[0.9978884459814782, 1, 1], 
       Hue[0.09703239989098028, 1, 1], 
       Hue[0.1969254467984167, 1, 1], 
       Hue[0.29759581352233877`, 1, 1], 
       Hue[0.39884143768934943`, 1, 1], 
       Hue[0.5007358447293813, 1, 1], 
       Hue[0.6035162687108159, 1, 1], 
       Hue[0.706861582499672, 1, 1], 
       Hue[0.8106624428033508, 1, 1], 
       Hue[0.9155282350120744, 1, 1], 
       Hue[0.021180426611272417`, 1, 1], 
       Hue[0.12692796944485524`, 1, 1], 
       Hue[0.233700700997123, 1, 1], 
       Hue[0.341946802462442, 1, 1], 
       Hue[0.4498968429690764, 1, 1], 
       Hue[0.5573824103226421, 1, 1], 
       Hue[0.6670150151902048, 1, 1], 
       Hue[0.7789105505834107, 1, 1], 
       Hue[0.8902108107720045, 1, 1], 
       Hue[0.0010083993451186876`, 1, 1], 
       Hue[0.11447518261633102`, 1, 1], 
       Hue[0.2295246674513252, 1, 1], 
       Hue[0.34142421738429607`, 1, 1], 
       Hue[0.451574669431246, 1, 1], 
       Hue[0.5666763034107078, 1, 1], 
       Hue[0.6850507481624744, 1, 1], 
       Hue[0.7984248249966265, 1, 1], 
       Hue[0.9072367859105431, 1, 1], 
       Hue[0.021987436943994295`, 1, 1], 
       Hue[0.1426344355952877, 1, 1], 
       Hue[0.2575711118171542, 1, 1], 
       Hue[0.3682158022952389, 1, 1], 
       Hue[0.49806656828861817`, 1, 1], 
       Hue[0.649111403365899, 1, 1], 
       Hue[0.7870001113529548, 1, 1], 
       Hue[0.8400106170584366, 1, 1], 
       Hue[0.9521427724380148, 1, 1], 
       Hue[0.1497588270495352, 1, 1], 
       Hue[0.5491258269772031, 1, 1], 
       Hue[0.2902796450076607, 1, 1], 
       Hue[0.2902796450076607, 1, 1], 
       Hue[0.3186348964907807, 1, 1], 
       Hue[0.3186348964907807, 1, 1], 
       Hue[0.2284208522423258, 1, 1], 
       Hue[0.2284208522423258, 1, 1], 
       Hue[0.12232320112241715`, 1, 1], 
       Hue[0.12232320112241715`, 1, 1], 
       Hue[0.005010772832046706, 1, 1], 
       Hue[0.005010772832046706, 1, 1], 
       Hue[0.9903585419751219, 1, 1], 
       Hue[0.9903585419751219, 1, 1], 
       Hue[0.9685718751320311, 1, 1], 
       Hue[0.9685718751320311, 1, 1], 
       Hue[0.7718361162578176, 1, 1], 
       Hue[0.7718361162578176, 1, 1], 
       Hue[0.5534039765471076, 1, 1], 
       Hue[0.5534039765471076, 1, 1], 
       Hue[0.23758362191745483`, 1, 1], 
       Hue[0.23758362191745483`, 1, 1], 
       Hue[0.6659801651782383, 1, 1], 
       Hue[0.6659801651782383, 1, 1], 
       Hue[0.49931996895800457`, 1, 1], 
       Hue[0.49931996895800457`, 1, 1], 
       Hue[0.5318520191878946, 1, 1], 
       Hue[0.5318520191878946, 1, 1], 
       Hue[0.30564357579768725`, 1, 1], 
       Hue[0.30564357579768725`, 1, 1], 
       Hue[0.9594404820243042, 1, 1], 
       Hue[0.9594404820243042, 1, 1], 
       Hue[0.7149204571508756, 1, 1], 
       Hue[0.7149204571508756, 1, 1], 
       Hue[0.6678921749008587, 1, 1], 
       Hue[0.6678921749008587, 1, 1], 
       Hue[0.44835131514272236`, 1, 1], 
       Hue[0.44835131514272236`, 1, 1], 
       Hue[0.48327914376204195`, 1, 1], 
       Hue[0.48327914376204195`, 1, 1], 
       Hue[0.24750800559439334`, 1, 1], 
       Hue[0.24750800559439334`, 1, 1], 
       Hue[0.11519603984478773`, 1, 1], 
       Hue[0.11519603984478773`, 1, 1], 
       Hue[0.01612414670244009, 1, 1], 
       Hue[0.01612414670244009, 1, 1], 
       Hue[0.8180816680337696, 1, 1], 
       Hue[0.8180816680337696, 1, 1], 
       Hue[0.6573322503084915, 1, 1], 
       Hue[0.6573322503084915, 1, 1], 
       Hue[0.5305707884219157, 1, 1], 
       Hue[0.5305707884219157, 1, 1], 
       Hue[0.49633086365786905`, 1, 1], 
       Hue[0.49633086365786905`, 1, 1], 
       Hue[0.3204740927659566, 1, 1], 
       Hue[0.3204740927659566, 1, 1], 
       Hue[0.2517279746461726, 1, 1], 
       Hue[0.2517279746461726, 1, 1], 
       Hue[0.1268451327024544, 1, 1], 
       Hue[0.1268451327024544, 1, 1], 
       Hue[0.04492836641888041, 1, 1], 
       Hue[0.04492836641888041, 1, 1], 
       Hue[0.8506860772805684, 1, 1], 
       Hue[0.8506860772805684, 1, 1], 
       Hue[0.7245694467189367, 1, 1], 
       Hue[0.7245694467189367, 1, 1], 
       Hue[0.7509139492274237, 1, 1], 
       Hue[0.7509139492274237, 1, 1], 
       Hue[0.6843863315113896, 1, 1], 
       Hue[0.6843863315113896, 1, 1], 
       Hue[0.5370274938430456, 1, 1], 
       Hue[0.5370274938430456, 1, 1], 
       Hue[0.49195828763052996`, 1, 1], 
       Hue[0.49195828763052996`, 1, 1], 
       Hue[0.308347341344207, 1, 1], 
       Hue[0.308347341344207, 1, 1], 
       Hue[0.12705074290335513`, 1, 1], 
       Hue[0.12705074290335513`, 1, 1], 
       Hue[0.10331329426250534`, 1, 1], 
       Hue[0.10331329426250534`, 1, 1], 
       Hue[0.982509099525184, 1, 1], 
       Hue[0.982509099525184, 1, 1], 
       Hue[0.787209221145821, 1, 1], 
       Hue[0.787209221145821, 1, 1], 
       Hue[0.7700877038771718, 1, 1], 
       Hue[0.7700877038771718, 1, 1], 
       Hue[0.6361643271409572, 1, 1], 
       Hue[0.6361643271409572, 1, 1], 
       Hue[0.6068466023971579, 1, 1], 
       Hue[0.6068466023971579, 1, 1], 
       Hue[0.4791298717905439, 1, 1], 
       Hue[0.4791298717905439, 1, 1], 
       Hue[0.4357091773859966, 1, 1], 
       Hue[0.4357091773859966, 1, 1], 
       Hue[0.33105954022102446`, 1, 1], 
       Hue[0.33105954022102446`, 1, 1], 
       Hue[0.2141133712894142, 1, 1], 
       Hue[0.2141133712894142, 1, 1], 
       Hue[0.15756837809381175`, 1, 1], 
       Hue[0.15756837809381175`, 1, 1], 
       Hue[0.9332747186039484, 1, 1], 
       Hue[0.9332747186039484, 1, 1], 
       Hue[0.9791335752533095, 1, 1], 
       Hue[0.9791335752533095, 1, 1], 
       Hue[0.8384726387092356, 1, 1], 
       Hue[0.8384726387092356, 1, 1], 
       Hue[0.7364619005298396, 1, 1], 
       Hue[0.7364619005298396, 1, 1], 
       Hue[0.6629901252207137, 1, 1], 
       Hue[0.6629901252207137, 1, 1], 
       Hue[0.5783395681331325, 1, 1], 
       Hue[0.5783395681331325, 1, 1], 
       Hue[0.4684978175622351, 1, 1], 
       Hue[0.4684978175622351, 1, 1], 
       Hue[0.37244889106677387`, 1, 1], 
       Hue[0.37244889106677387`, 1, 1], 
       Hue[0.29411967947531414`, 1, 1], 
       Hue[0.29411967947531414`, 1, 1], 
       Hue[0.19682968958288313`, 1, 1], 
       Hue[0.19682968958288313`, 1, 1], 
       Hue[0.08992445985436712, 1, 1], 
       Hue[0.08992445985436712, 1, 1], 
       Hue[0.9933438259434542, 1, 1], 
       Hue[0.9933438259434542, 1, 1], 
       Hue[0.8724541681175237, 1, 1], 
       Hue[0.8724541681175237, 1, 1], 
       Hue[0.7760773796938453, 1, 1], 
       Hue[0.7760773796938453, 1, 1], 
       Hue[0.6796254362241916, 1, 1], 
       Hue[0.6796254362241916, 1, 1], 
       Hue[0.5939543082492782, 1, 1], 
       Hue[0.5939543082492782, 1, 1], 
       Hue[0.5028392579847493, 1, 1], 
       Hue[0.5028392579847493, 1, 1], 
       Hue[0.4036498776674032, 1, 1], 
       Hue[0.4036498776674032, 1, 1], 
       Hue[0.3003020928823953, 1, 1], 
       Hue[0.3003020928823953, 1, 1], 
       Hue[0.1967022508173054, 1, 1], 
       Hue[0.1967022508173054, 1, 1], 
       Hue[0.09650347712384397, 1, 1], 
       Hue[0.09650347712384397, 1, 1], 
       Hue[0.0027723402406860794`, 1, 1], 
       Hue[0.0027723402406860794`, 1, 1], 
       Hue[0.9113108697618242, 1, 1], 
       Hue[0.9113108697618242, 1, 1], 
       Hue[0.8187928748678927, 1, 1], 
       Hue[0.8187928748678927, 1, 1], 
       Hue[0.7208690487729366, 1, 1], 
       Hue[0.7208690487729366, 1, 1], 
       Hue[0.6185444591778555, 1, 1], 
       Hue[0.6185444591778555, 1, 1], 
       Hue[0.5165902196272736, 1, 1], 
       Hue[0.5165902196272736, 1, 1], 
       Hue[0.42051768332086603`, 1, 1], 
       Hue[0.42051768332086603`, 1, 1], 
       Hue[0.33070211397551663`, 1, 1], 
       Hue[0.33070211397551663`, 1, 1], 
       Hue[0.2426103255226291, 1, 1], 
       Hue[0.2426103255226291, 1, 1], 
       Hue[0.15138932879895664`, 1, 1], 
       Hue[0.15138932879895664`, 1, 1], 
       Hue[0.05249602680951814, 1, 1], 
       Hue[0.05249602680951814, 1, 1], 
       Hue[0.9478367133607696, 1, 1], 
       Hue[0.9478367133607696, 1, 1], 
       Hue[0.8475875861469279, 1, 1], 
       Hue[0.8475875861469279, 1, 1], 
       Hue[0.756300580286778, 1, 1], 
       Hue[0.756300580286778, 1, 1], 
       Hue[0.6716889199006466, 1, 1], 
       Hue[0.6716889199006466, 1, 1], 
       Hue[0.5870825798302917, 1, 1], 
       Hue[0.5870825798302917, 1, 1], 
       Hue[0.495050509223539, 1, 1], 
       Hue[0.495050509223539, 1, 1], 
       Hue[0.39262780127384156`, 1, 1], 
       Hue[0.39262780127384156`, 1, 1], 
       Hue[0.28714126331748524`, 1, 1], 
       Hue[0.28714126331748524`, 1, 1], 
       Hue[0.19112805442353883`, 1, 1], 
       Hue[0.19112805442353883`, 1, 1], 
       Hue[0.10673525827672053`, 1, 1], 
       Hue[0.10673525827672053`, 1, 1], 
       Hue[0.027522840384905385`, 1, 1], 
       Hue[0.027522840384905385`, 1, 1], 
       Hue[0.9444803685835561, 1, 1], 
       Hue[0.9444803685835561, 1, 1], 
       Hue[0.8490249070559226, 1, 1], 
       Hue[0.8490249070559226, 1, 1], 
       Hue[0.7411649498814663, 1, 1], 
       Hue[0.7411649498814663, 1, 1], 
       Hue[0.6374666215429812, 1, 1], 
       Hue[0.6374666215429812, 1, 1], 
       Hue[0.5497931223572055, 1, 1], 
       Hue[0.5497931223572055, 1, 1], 
       Hue[0.4737291123864164, 1, 1], 
       Hue[0.4737291123864164, 1, 1], 
       Hue[0.3992080090157962, 1, 1], 
       Hue[0.3992080090157962, 1, 1], 
       Hue[0.31482774791091384`, 1, 1], 
       Hue[0.31482774791091384`, 1, 1], 
       Hue[0.2113292305432758, 1, 1], 
       Hue[0.2113292305432758, 1, 1], 
       Hue[0.09908154232394249, 1, 1], 
       Hue[0.09908154232394249, 1, 1], 
       Hue[0.003580719173392775, 1, 1], 
       Hue[0.003580719173392775, 1, 1], 
       Hue[0.9277707136658423, 1, 1], 
       Hue[0.9277707136658423, 1, 1], 
       Hue[0.8605404856953217, 1, 1], 
       Hue[0.8605404856953217, 1, 1], 
       Hue[0.7895048468780809, 1, 1], 
       Hue[0.7895048468780809, 1, 1], 
       Hue[0.7002054335577579, 1, 1], 
       Hue[0.7002054335577579, 1, 1], 
       Hue[0.5856439885046414, 1, 1], 
       Hue[0.5856439885046414, 1, 1], 
       Hue[0.47569786975257744`, 1, 1], 
       Hue[0.47569786975257744`, 1, 1], 
       Hue[0.3944836420368127, 1, 1], 
       Hue[0.3944836420368127, 1, 1], 
       Hue[0.33207192193375373`, 1, 1], 
       Hue[0.33207192193375373`, 1, 1], 
       Hue[0.2748339851375885, 1, 1], 
       Hue[0.2748339851375885, 1, 1], 
       Hue[0.20987049241422776`, 1, 1], 
       Hue[0.20987049241422776`, 1, 1], 
       Hue[0.11924584561484784`, 1, 1], 
       Hue[0.11924584561484784`, 1, 1], 
       Hue[0.9968614524798072, 1, 1], 
       Hue[0.9968614524798072, 1, 1], 
       Hue[0.8915858603078806, 1, 1], 
       Hue[0.8915858603078806, 1, 1], 
       Hue[0.8232626433318473, 1, 1], 
       Hue[0.8232626433318473, 1, 1], 
       Hue[0.7746340198278019, 1, 1], 
       Hue[0.7746340198278019, 1, 1], 
       Hue[0.7331489905094091, 1, 1], 
       Hue[0.7331489905094091, 1, 1], 
       Hue[0.6902763475191502, 1, 1], 
       Hue[0.6902763475191502, 1, 1], 
       Hue[0.6346335888705981, 1, 1], 
       Hue[0.6346335888705981, 1, 1], 
       Hue[0.5435719785474447, 1, 1], 
       Hue[0.5435719785474447, 1, 1], 
       Hue[0.4138905816180223, 1, 1], 
       Hue[0.4138905816180223, 1, 1], 
       Hue[0.3198427828740491, 1, 1], 
       Hue[0.3198427828740491, 1, 1], 
       Hue[0.2689200438620375, 1, 1], 
       Hue[0.2689200438620375, 1, 1], 
       Hue[0.23808301889067057`, 1, 1], 
       Hue[0.23808301889067057`, 1, 1], 
       Hue[0.21700129017557332`, 1, 1], 
       Hue[0.21700129017557332`, 1, 1], 
       Hue[0.2015965309429917, 1, 1], 
       Hue[0.2015965309429917, 1, 1], 
       Hue[0.1900849351301606, 1, 1], 
       Hue[0.1900849351301606, 1, 1], 
       Hue[0.18161565069571153`, 1, 1], 
       Hue[0.18161565069571153`, 1, 1], 
       Hue[0.17576281859083673`, 1, 1], 
       Hue[0.17576281859083673`, 1, 1], 
       Hue[0.17231111257394194`, 1, 1], 
       Hue[0.17231111257394194`, 1, 1], 
       Hue[0.171157003537818, 1, 1], 
       Hue[0.171157003537818, 1, 1], 
       Hue[0.17225561589785562`, 1, 1], 
       Hue[0.17225561589785562`, 1, 1], 
       Hue[0.1755919846585013, 1, 1], 
       Hue[0.1755919846585013, 1, 1], 
       Hue[0.18116294003375108`, 1, 1], 
       Hue[0.18116294003375108`, 1, 1], 
       Hue[0.18896824793199118`, 1, 1], 
       Hue[0.18896824793199118`, 1, 1], 
       Hue[0.19900410308070351`, 1, 1], 
       Hue[0.19900410308070351`, 1, 1], 
       Hue[0.21126095026360003`, 1, 1], 
       Hue[0.21126095026360003`, 1, 1], 
       Hue[0.2257205157148169, 1, 1], 
       Hue[0.2257205157148169, 1, 1], 
       Hue[0.24235848928686538`, 1, 1], 
       Hue[0.24235848928686538`, 1, 1], 
       Hue[0.2611420249768013, 1, 1], 
       Hue[0.2611420249768013, 1, 1], 
       Hue[0.2820336741647051, 1, 1], 
       Hue[0.2820336741647051, 1, 1], 
       Hue[0.3049911010578967, 1, 1], 
       Hue[0.3049911010578967, 1, 1], 
       Hue[0.32996816191755635`, 1, 1], 
       Hue[0.32996816191755635`, 1, 1], 
       Hue[0.3569163337053618, 1, 1], 
       Hue[0.3569163337053618, 1, 1], 
       Hue[0.38578591401286044`, 1, 1], 
       Hue[0.38578591401286044`, 1, 1], 
       Hue[0.4165259364321822, 1, 1], 
       Hue[0.4165259364321822, 1, 1], 
       Hue[0.4490866391723868, 1, 1], 
       Hue[0.4490866391723868, 1, 1], 
       Hue[0.4834177366510568, 1, 1], 
       Hue[0.4834177366510568, 1, 1], 
       Hue[0.5194701836085935, 1, 1], 
       Hue[0.5194701836085935, 1, 1], 
       Hue[0.557196825271485, 1, 1], 
       Hue[0.557196825271485, 1, 1], 
       Hue[0.5965518215974968, 1, 1], 
       Hue[0.5965518215974968, 1, 1], 
       Hue[0.6374904602151458, 1, 1], 
       Hue[0.6374904602151458, 1, 1], 
       Hue[0.6799708772044095, 1, 1], 
       Hue[0.6799708772044095, 1, 1], 
       Hue[0.7239522976890813, 1, 1], 
       Hue[0.7239522976890813, 1, 1], 
       Hue[0.769396386916378, 1, 1], 
       Hue[0.769396386916378, 1, 1], 
       Hue[0.8162657220231337, 1, 1], 
       Hue[0.8162657220231337, 1, 1], 
       Hue[0.8645258254775823, 1, 1], 
       Hue[0.8645258254775823, 1, 1], 
       Hue[0.9141429735217279, 1, 1], 
       Hue[0.9141429735217279, 1, 1], 
       Hue[0.96508566814464, 1, 1], 
       Hue[0.96508566814464, 1, 1], 
       Hue[0.017324549387385067`, 1, 1], 
       Hue[0.017324549387385067`, 1, 1], 
       Hue[0.07083024111942525, 1, 1], 
       Hue[0.07083024111942525, 1, 1], 
       Hue[0.12557661019562788`, 1, 1], 
       Hue[0.12557661019562788`, 1, 1], 
       Hue[0.18153833105298484`, 1, 1], 
       Hue[0.18153833105298484`, 1, 1], 
       Hue[0.23869059105266235`, 1, 1], 
       Hue[0.23869059105266235`, 1, 1], 
       Hue[0.29701003205840226`, 1, 1], 
       Hue[0.29701003205840226`, 1, 1], 
       Hue[0.3564750346896973, 1, 1], 
       Hue[0.3564750346896973, 1, 1], 
       Hue[0.4170657262681432, 1, 1], 
       Hue[0.4170657262681432, 1, 1], 
       Hue[0.47876096011285807`, 1, 1], 
       Hue[0.47876096011285807`, 1, 1], 
       Hue[0.5415437206250882, 1, 1], 
       Hue[0.5415437206250882, 1, 1], 
       Hue[0.60539444097929, 1, 1], 
       Hue[0.60539444097929, 1, 1], 
       Hue[0.6702963986746799, 1, 1], 
       Hue[0.6702963986746799, 1, 1], 
       Hue[0.7362341144996116, 1, 1], 
       Hue[0.7362341144996116, 1, 1], 
       Hue[0.8031923601931057, 1, 1], 
       Hue[0.8031923601931057, 1, 1], 
       Hue[0.8711544244586391, 1, 1], 
       Hue[0.8711544244586391, 1, 1], 
       Hue[0.9401074600065064, 1, 1], 
       Hue[0.9401074600065064, 1, 1], 
       Hue[0.010038309244361136`, 1, 1], 
       Hue[0.010038309244361136`, 1, 1], 
       Hue[0.08093288218141492, 1, 1], 
       Hue[0.08093288218141492, 1, 1], 
       Hue[0.15277982353789266`, 1, 1], 
       Hue[0.15277982353789266`, 1, 1], 
       Hue[0.22556566530655825`, 1, 1], 
       Hue[0.22556566530655825`, 1, 1], 
       Hue[0.29928098086556476`, 1, 1], 
       Hue[0.29928098086556476`, 1, 1], 
       Hue[0.37391601020656645`, 1, 1], 
       Hue[0.37391601020656645`, 1, 1], 
       Hue[0.4494558054736254, 1, 1], 
       Hue[0.4494558054736254, 1, 1], 
       Hue[0.5258909455116205, 1, 1], 
       Hue[0.5258909455116205, 1, 1], 
       Hue[0.6032152788066548, 1, 1], 
       Hue[0.6032152788066548, 1, 1], 
       Hue[0.6814168667752996, 1, 1], 
       Hue[0.6814168667752996, 1, 1], 
       Hue[0.7604874600919083, 1, 1], 
       Hue[0.7604874600919083, 1, 1], 
       Hue[0.840418989301121, 1, 1], 
       Hue[0.840418989301121, 1, 1], 
       Hue[0.9211998295028575, 1, 1], 
       Hue[0.9211998295028575, 1, 1], 
       Hue[0.0028226054862767027`, 1, 1], 
       Hue[0.0028226054862767027`, 1, 1], 
       Hue[0.08528186990748743, 1, 1], 
       Hue[0.08528186990748743, 1, 1], 
       Hue[0.16856912526236964`, 1, 1], 
       Hue[0.16856912526236964`, 1, 1], 
       Hue[0.2526801036479619, 1, 1], 
       Hue[0.2526801036479619, 1, 1], 
       Hue[0.3376012187689896, 1, 1], 
       Hue[0.3376012187689896, 1, 1], 
       Hue[0.4233218065639601, 1, 1], 
       Hue[0.4233218065639601, 1, 1], 
       Hue[0.5098467270130229, 1, 1], 
       Hue[0.5098467270130229, 1, 1], 
       Hue[0.5971651206611441, 1, 1], 
       Hue[0.5971651206611441, 1, 1], 
       Hue[0.6852741643229945, 1, 1], 
       Hue[0.6852741643229945, 1, 1], 
       Hue[0.7741608030557923, 1, 1], 
       Hue[0.7741608030557923, 1, 1], 
       Hue[0.8638069094138561, 1, 1], 
       Hue[0.8638069094138561, 1, 1], 
       Hue[0.9542258377497206, 1, 1], 
       Hue[0.9542258377497206, 1, 1], 
       Hue[0.0454115137177682, 1, 1], 
       Hue[0.0454115137177682, 1, 1], 
       Hue[0.13735385516522644`, 1, 1], 
       Hue[0.13735385516522644`, 1, 1], 
       Hue[0.23004905610161971`, 1, 1], 
       Hue[0.23004905610161971`, 1, 1], 
       Hue[0.3234724822822555, 1, 1], 
       Hue[0.3234724822822555, 1, 1], 
       Hue[0.4176385390663141, 1, 1], 
       Hue[0.4176385390663141, 1, 1], 
       Hue[0.5125795503518117, 1, 1], 
       Hue[0.5125795503518117, 1, 1], 
       Hue[0.6082348081135551, 1, 1], 
       Hue[0.6082348081135551, 1, 1], 
       Hue[0.7045341876617609, 1, 1], 
       Hue[0.7045341876617609, 1, 1], 
       Hue[0.801554485238327, 1, 1], 
       Hue[0.801554485238327, 1, 1], 
       Hue[0.8993717427787195, 1, 1], 
       Hue[0.8993717427787195, 1, 1], 
       Hue[0.9978884459814782, 1, 1], 
       Hue[0.9978884459814782, 1, 1], 
       Hue[0.09703239989098028, 1, 1], 
       Hue[0.09703239989098028, 1, 1], 
       Hue[0.1969254467984167, 1, 1], 
       Hue[0.1969254467984167, 1, 1], 
       Hue[0.29759581352233877`, 1, 1], 
       Hue[0.29759581352233877`, 1, 1], 
       Hue[0.39884143768934943`, 1, 1], 
       Hue[0.39884143768934943`, 1, 1], 
       Hue[0.5007358447293813, 1, 1], 
       Hue[0.5007358447293813, 1, 1], 
       Hue[0.6035162687108159, 1, 1], 
       Hue[0.6035162687108159, 1, 1], 
       Hue[0.706861582499672, 1, 1], 
       Hue[0.706861582499672, 1, 1], 
       Hue[0.8106624428033508, 1, 1], 
       Hue[0.8106624428033508, 1, 1], 
       Hue[0.9155282350120744, 1, 1], 
       Hue[0.9155282350120744, 1, 1], 
       Hue[0.021180426611272417`, 1, 1], 
       Hue[0.021180426611272417`, 1, 1], 
       Hue[0.12692796944485524`, 1, 1], 
       Hue[0.12692796944485524`, 1, 1], 
       Hue[0.233700700997123, 1, 1], 
       Hue[0.233700700997123, 1, 1], 
       Hue[0.341946802462442, 1, 1], 
       Hue[0.341946802462442, 1, 1], 
       Hue[0.4498968429690764, 1, 1], 
       Hue[0.4498968429690764, 1, 1], 
       Hue[0.5573824103226421, 1, 1], 
       Hue[0.5573824103226421, 1, 1], 
       Hue[0.6670150151902048, 1, 1], 
       Hue[0.6670150151902048, 1, 1], 
       Hue[0.7789105505834107, 1, 1], 
       Hue[0.7789105505834107, 1, 1], 
       Hue[0.8902108107720045, 1, 1], 
       Hue[0.8902108107720045, 1, 1], 
       Hue[0.0010083993451186876`, 1, 1], 
       Hue[0.0010083993451186876`, 1, 1], 
       Hue[0.11447518261633102`, 1, 1], 
       Hue[0.11447518261633102`, 1, 1], 
       Hue[0.2295246674513252, 1, 1], 
       Hue[0.2295246674513252, 1, 1], 
       Hue[0.34142421738429607`, 1, 1], 
       Hue[0.34142421738429607`, 1, 1], 
       Hue[0.451574669431246, 1, 1], 
       Hue[0.451574669431246, 1, 1], 
       Hue[0.5666763034107078, 1, 1], 
       Hue[0.5666763034107078, 1, 1], 
       Hue[0.6850507481624744, 1, 1], 
       Hue[0.6850507481624744, 1, 1], 
       Hue[0.7984248249966265, 1, 1], 
       Hue[0.7984248249966265, 1, 1], 
       Hue[0.9072367859105431, 1, 1], 
       Hue[0.9072367859105431, 1, 1], 
       Hue[0.021987436943994295`, 1, 1], 
       Hue[0.021987436943994295`, 1, 1], 
       Hue[0.1426344355952877, 1, 1], 
       Hue[0.1426344355952877, 1, 1], 
       Hue[0.2575711118171542, 1, 1], 
       Hue[0.2575711118171542, 1, 1], 
       Hue[0.3682158022952389, 1, 1], 
       Hue[0.3682158022952389, 1, 1], 
       Hue[0.49806656828861817`, 1, 1], 
       Hue[0.49806656828861817`, 1, 1], 
       Hue[0.649111403365899, 1, 1], 
       Hue[0.649111403365899, 1, 1], 
       Hue[0.7870001113529548, 1, 1], 
       Hue[0.7870001113529548, 1, 1], 
       Hue[0.8400106170584366, 1, 1], 
       Hue[0.8400106170584366, 1, 1], 
       Hue[0.9521427724380148, 1, 1], 
       Hue[0.9521427724380148, 1, 1], 
       Hue[0.1497588270495352, 1, 1]}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->{True, None},
    AxesOrigin->{0, 0},
    Epilog->
     LineBox[{{1, 2.7840904793647274`*^-7}, {2, 2.3476257877317546`*^-7}, {
       3, 3.7725605325779893`*^-7}, {4, 5.807490365437347*^-7}, {
       5, 7.951051815569195*^-7}, {6, 6.659267701819666*^-7}, {
       7, 1.397677971224363*^-7}, {8, 9.840351126654453*^-7}, {
       9, 7.9846795250673*^-7}, {10, 5.119020834217228*^-7}, {
       11, 2.2216477783161265`*^-7}, {12, 4.2435686756495455`*^-7}, {
       13, 2.649908625297579*^-7}, {14, 1.248799406657194*^-7}, {
       15, 2.877389767383945*^-7}, {16, 3.572019594724289*^-7}, {
       17, 4.351030916368993*^-7}, {18, 3.3635434343040655`*^-7}, {
       19, 7.387204878867423*^-7}, {20, 2.554036706313653*^-7}, {
       21, 6.028635670759956*^-7}, {22, 3.035024548502732*^-7}, {
       23, 7.790968413916318*^-7}, {24, 1.3220492103806469`*^-6}, {
       25, 1.1906080581749486`*^-6}, {26, 1.0494557179140746`*^-6}, {
       27, 1.0562509774555886`*^-6}, {28, 1.0023116103436475`*^-6}, {
       29, 7.424402387187357*^-7}, {30, 7.46988550868365*^-7}, {
       31, 5.082324094038862*^-7}, {32, 8.617668494863171*^-7}, {
       33, 2.4326689626910326`*^-7}, {34, 7.959911776922238*^-7}, {
       35, 5.174320123754048*^-7}, {36, 9.977542287057794*^-7}, {
       37, 3.226818329820662*^-7}, {38, 8.765669391453493*^-7}, {
       39, 4.1125255842224804`*^-7}, {40, 9.16886371658223*^-8}, {
       41, 7.761352642843013*^-7}, {42, 1.0669064162438863`*^-6}, {
       43, 7.348082300949503*^-7}, {44, 1.59826264803697*^-6}, {
       45, 1.1762956643339447`*^-6}, {46, 1.221418004506105*^-6}, {
       47, 1.6255506956415514`*^-6}, {48, 1.196383493766683*^-6}, {
       49, 9.21535848364169*^-7}, {50, 1.0676248885012513`*^-6}, {
       51, 7.074365585755711*^-7}, {52, 9.853935178781322*^-7}, {
       53, 1.778535674658169*^-6}, {54, 2.0455115818133567`*^-6}, {
       55, 2.218837704874427*^-6}, {56, 3.4472579681669604`*^-6}, {
       57, 3.839052117698447*^-6}, {58, 4.363762494023019*^-6}, {
       59, 4.841612582311263*^-6}, {60, 5.536577620935596*^-6}, {
       61, 6.6136678926983065`*^-6}, {62, 8.802235873570966*^-6}, {
       63, 0.000010961041456343846`}, {64, 0.000013859787605476625`}, {
       65, 0.00001703461906682615}, {66, 0.000021058748655187032`}, {
       67, 0.000025199897993291865`}, {68, 0.00002968139579200682}, {
       69, 0.00003505716324792632}, {70, 0.000043027678566643007`}, {
       71, 0.000053954964007524}, {72, 0.0000674634696323547}, {
       73, 0.0000820412324144472}, {74, 0.00009594861934168996}, {
       75, 0.00011136185459504488`}, {76, 0.00013071075086409833`}, {
       77, 0.00016094618403406774`}, {78, 0.00020203501529720246`}, {
       79, 0.0002484081950853924}, {80, 0.0002927796913361347}, {
       81, 0.00033190795889119354`}, {82, 0.00037483760022563166`}, {
       83, 0.000441990107494212}, {84, 0.0005475902126215767}, {
       85, 0.0006847814276824861}, {86, 0.0008231390791378697}, {
       87, 0.0009358093207514065}, {88, 0.0010224511692107133`}, {
       89, 0.0011346930763279886`}, {90, 0.0013574854783087106`}, {
       91, 0.001711758260331793}, {92, 0.0021169962664522764`}, {
       93, 0.0024565875977275043`}, {94, 0.0026595633984730993`}, {
       95, 0.0027946489095873185`}, {96, 0.003122221829525069}, {
       97, 0.0038771431050995775`}, {98, 0.004938041067166719}, {
       99, 0.005933654904485582}, {100, 0.006522960679212453}, {
       101, 0.006646645721509049}, {102, 0.006815774826748525}, {
       103, 0.007995616351049803}, {104, 0.010401970352974136`}, {
       105, 0.0131112683669604}, {106, 0.014996118300357308`}, {
       107, 0.015383408230869281`}, {108, 0.014707498975775442`}, {
       109, 0.015279614176830825`}, {110, 0.019522882649120124`}, {
       111, 0.026195959185253765`}, {112, 0.03223282989301264}, {
       113, 0.03512422024252793}, {114, 0.03382957236915019}, {
       115, 0.030359564971308782`}, {116, 0.03191601408726447}, {
       117, 0.043967561790244136`}, {118, 0.0614696303372614}, {
       119, 0.07745165464862012}, {120, 0.08678105086415389}, {
       121, 0.08631817638949765}, {122, 0.07606107119310992}, {
       123, 0.06332279108865645}, {124, 0.06996213922705054}, {
       125, 0.10830584212055046`}, {126, 0.16518187021499342`}, {
       127, 0.23012602028306}, {128, 0.2973641949329308}, {
       129, 0.36308371405726186`}, {130, 0.42465269752608714`}, {
       131, 0.4803726447834743}, {132, 0.5293185168190553}, {
       133, 0.5711653158390205}, {134, 0.6060251493048092}, {
       135, 0.6342845937069549}, {136, 0.6564912589824866}, {
       137, 0.6732482548101423}, {138, 0.6851609047002907}, {
       139, 0.6927974799320984}, {140, 0.6966676555879119}, {
       141, 0.6972205616176433}, {142, 0.6948459889541057}, {
       143, 0.6898781833862501}, {144, 0.6826077883376961}, {
       145, 0.6732861426825543}, {146, 0.6621378439744305}, {
       147, 0.6493586675192086}, {148, 0.6351329283672433}, {
       149, 0.619624248208783}, {150, 0.6029899872483413}, {
       151, 0.5853752041782484}, {152, 0.5669185809560062}, {
       153, 0.547755670582892}, {154, 0.5280112638957404}, {
       155, 0.5078087505867818}, {156, 0.48726598742742233`}, {
       157, 0.46649501622549405`}, {158, 0.4456039760172254}, {
       159, 0.4246928445443121}, {160, 0.403859037576679}, {
       161, 0.38319154208479095`}, {162, 0.3627745293670292}, {
       163, 0.342681881856025}, {164, 0.32298571760164974`}, {
       165, 0.3037490307176687}, {166, 0.28502721555113136`}, {
       167, 0.26686868740425307`}, {168, 0.24931736548735584`}, {
       169, 0.2324077319936288}, {170, 0.21616823838143148`}, {
       171, 0.2006211696912877}, {172, 0.18578336826996716`}, {
       173, 0.17166381484572382`}, {174, 0.15827036168535133`}, {
       175, 0.14560119569910537`}, {176, 0.13365203258494107`}, {
       177, 0.12241325831655088`}, {178, 0.11187428259503832`}, {
       179, 0.10201751800398683`}, {180, 0.09282514563869629}, {
       181, 0.08427549843078563}, {182, 0.07634530184867783}, {
       183, 0.06900983716985953}, {184, 0.06224155456563654}, {
       185, 0.0560127945042058}, {186, 0.05029729477061629}, {
       187, 0.04506571145298718}, {188, 0.0402890136549176}, {
       189, 0.035939385923790416`}, {190, 0.031988294945593895`}, {
       191, 0.028408497138459745`}, {192, 0.025173877161639536`}, {
       193, 0.022258328403307194`}, {194, 0.01963687681627681}, {
       195, 0.017285679964406917`}, {196, 0.015181673147389647`}, {
       197, 0.013304571684119854`}, {198, 0.01163387118578884}, {
       199, 0.010150172804532266`}, {200, 0.00883592106230871}, {
       201, 0.007674116818949747}, {202, 0.006650366590519392}, {
       203, 0.0057507066452891185`}, {204, 0.004961468204804808}, {
       205, 0.004270926570672727}, {206, 0.003668038868378991}, {
       207, 0.003143129406593967}, {208, 0.0026877077089083115`}, {
       209, 0.0022929361538310397`}, {210, 0.0019509801712304187`}, {
       211, 0.001656376827453113}, {212, 0.0014038468977090168`}, {
       213, 0.0011871230539388104`}, {214, 0.001001101083255173}, {
       215, 0.0008424619081592663}, {216, 0.0007077217647467498}, {
       217, 0.000592899530952019}, {218, 0.0004953684037376003}, {
       219, 0.00041337045860092794`}, {220, 0.00034412820398229673`}, {
       221, 0.00028544258860818965`}, {222, 0.00023660607159289897`}, {
       223, 0.00019592876995973903`}, {224, 0.00016137920551830606`}, {
       225, 0.00013268643497727102`}, {226, 0.00010931356231395121`}, {
       227, 0.00008948279035806813}, {228, 0.00007264891849158014}, {
       229, 0.00005931949564526404}, {230, 0.00004874442622113992}, {
       231, 0.00003960138217951687}, {232, 0.00003178478022753637}, {
       233, 0.00002566743430033447}, {234, 0.00002077716461379871}, {
       235, 0.000016463803053333834`}, {236, 0.000012875728193564668`}, {
       237, 0.000010309613392873033`}, {238, 8.366117099388916*^-6}, {
       239, 6.571072696292503*^-6}, {240, 5.033348848483136*^-6}, {
       241, 3.994919817546296*^-6}, {242, 3.285267772814738*^-6}, {
       243, 2.6209797736733804`*^-6}, {244, 2.0340228396403652`*^-6}, {
       245, 1.7022350329921032`*^-6}, {246, 1.5138562678469977`*^-6}, {
       247, 1.192198926530175*^-6}, {248, 7.25560419081644*^-7}, {
       249, 5.052093341425101*^-7}, {250, 7.06364939206284*^-7}, {
       251, 8.052640550483517*^-7}}],
    Frame->True,
    FrameTicks->{{{1., 
        FormBox[
         RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
        FormBox[
         RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
        FormBox[
         RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
        FormBox[
         RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
        FormBox["0", TraditionalForm]}, {157.25, 
        FormBox["2.5`", TraditionalForm]}, {188.5, 
        FormBox["5", TraditionalForm]}, {219.75, 
        FormBox["7.5`", TraditionalForm]}, {251., 
        FormBox["10", TraditionalForm]}}, Automatic, None, None},
    PlotRange->{-0.1, 1.1},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], Automatic}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF2HdcFHcaBvDBWNDIhYuKiCijIQQQUZoiGHxEEBAFVpYuMCwsZWlGgkaj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     "], {
      {GrayLevel[0.5], EdgeForm[None], 
       GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ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         "]]]}, {{}, {}, {
        {Hue[0.05956624042050353, 1, 1], LineBox[{1, 2}]}, 
        {Hue[0.9831017166735266, 1, 1], LineBox[{2, 3}]}, 
        {Hue[0.053784012938536886`, 1, 1], LineBox[{3, 4}]}, 
        {Hue[0.8965646702957913, 1, 1], LineBox[{4, 5}]}, 
        {Hue[0.7133375980565537, 1, 1], LineBox[{5, 6}]}, 
        {Hue[0.472416995417405, 1, 1], LineBox[{6, 7}]}, 
        {Hue[0.22615072621804672`, 1, 1], LineBox[{7, 8}]}, 
        {Hue[0.08421188501446454, 1, 1], LineBox[{8, 9}]}, 
        {Hue[0.8829614102660753, 1, 1], LineBox[{9, 10}]}, 
        {Hue[0.8222896416930823, 1, 1], LineBox[{10, 11}]}, 
        {Hue[0.6400882201477065, 1, 1], LineBox[{11, 12}]}, 
        {Hue[0.5078569467204729, 1, 1], LineBox[{12, 13}]}, 
        {Hue[0.4582011452917198, 1, 1], LineBox[{13, 14}]}, 
        {Hue[0.2860972878164505, 1, 1], LineBox[{14, 15}]}, 
        {Hue[0.1269644106342963, 1, 1], LineBox[{15, 16}]}, 
        {Hue[0.008489555056382105, 1, 1], LineBox[{16, 17}]}, 
        {Hue[0.9047969445291322, 1, 1], LineBox[{17, 18}]}, 
        {Hue[0.8069026689219323, 1, 1], LineBox[{18, 19}]}, 
        {Hue[0.7433426798313757, 1, 1], LineBox[{19, 20}]}, 
        {Hue[0.6211645129944474, 1, 1], LineBox[{20, 21}]}, 
        {Hue[0.570574249490098, 1, 1], LineBox[{21, 22}]}, 
        {Hue[0.37442686964880406`, 1, 1], LineBox[{22, 23}]}, 
        {Hue[0.41406408292314084`, 1, 1], LineBox[{23, 24}]}, 
        {Hue[0.27304668255560127`, 1, 1], LineBox[{24, 25}]}, 
        {Hue[0.1640613943112119, 1, 1], LineBox[{25, 26}]}, 
        {Hue[0.9217837241712366, 1, 1], LineBox[{26, 27}]}, 
        {Hue[0.04927441714522789, 1, 1], LineBox[{27, 28}]}, 
        {Hue[0.7395544576770468, 1, 1], LineBox[{28, 29}]}, 
        {Hue[0.8366427329778339, 1, 1], LineBox[{29, 30}]}, 
        {Hue[0.640224275610829, 1, 1], LineBox[{30, 31}]}, 
        {Hue[0.5005906513158629, 1, 1], LineBox[{31, 32}]}, 
        {Hue[0.39893509617051615`, 1, 1], LineBox[{32, 33}]}, 
        {Hue[0.31827148243229864`, 1, 1], LineBox[{33, 34}]}, 
        {Hue[0.10562517832932644`, 1, 1], LineBox[{34, 35}]}, 
        {Hue[0.05647944373999831, 1, 1], LineBox[{35, 36}]}, 
        {Hue[0.954476650596741, 1, 1], LineBox[{36, 37}]}, 
        {Hue[0.8121914267182476, 1, 1], LineBox[{37, 38}]}, 
        {Hue[0.6843275724593121, 1, 1], LineBox[{38, 39}]}, 
        {Hue[0.8357639889934845, 1, 1], LineBox[{39, 40}]}, 
        {Hue[0.6544318649455261, 1, 1], LineBox[{40, 41}]}, 
        {Hue[0.571673756180975, 1, 1], LineBox[{41, 42}]}, 
        {Hue[0.5103863038769656, 1, 1], LineBox[{42, 43}]}, 
        {Hue[0.38946345539840677`, 1, 1], LineBox[{43, 44}]}, 
        {Hue[0.3407740913415771, 1, 1], LineBox[{44, 45}]}, 
        {Hue[0.2881490527713575, 1, 1], LineBox[{45, 46}]}, 
        {Hue[0.14006130480093046`, 1, 1], LineBox[{46, 47}]}, 
        {Hue[0.03270695109683777, 1, 1], LineBox[{47, 48}]}, 
        {Hue[0.9222580029127315, 1, 1], LineBox[{48, 49}]}, 
        {Hue[0.8216153254234894, 1, 1], LineBox[{49, 50}]}, 
        {Hue[0.7327219348514502, 1, 1], LineBox[{50, 51}]}, 
        {Hue[0.6329691624014069, 1, 1], LineBox[{51, 52}]}, 
        {Hue[0.5441018001510657, 1, 1], LineBox[{52, 53}]}, 
        {Hue[0.451179412264641, 1, 1], LineBox[{53, 54}]}, 
        {Hue[0.34884452009062944`, 1, 1], LineBox[{54, 55}]}, 
        {Hue[0.24649878947472764`, 1, 1], LineBox[{55, 56}]}, 
        {Hue[0.15143639484703394`, 1, 1], LineBox[{56, 57}]}, 
        {Hue[0.050421982901845644`, 1, 1], LineBox[{57, 58}]}, 
        {Hue[0.9481497781591328, 1, 1], LineBox[{58, 59}]}, 
        {Hue[0.8502040510520766, 1, 1], LineBox[{59, 60}]}, 
        {Hue[0.7515057307791237, 1, 1], LineBox[{60, 61}]}, 
        {Hue[0.6545229892266976, 1, 1], LineBox[{61, 62}]}, 
        {Hue[0.556780119053588, 1, 1], LineBox[{62, 63}]}, 
        {Hue[0.45912955175820863`, 1, 1], LineBox[{63, 64}]}, 
        {Hue[0.36229952563725926`, 1, 1], LineBox[{64, 65}]}, 
        {Hue[0.2647079518516234, 1, 1], LineBox[{65, 66}]}, 
        {Hue[0.16676307041787705`, 1, 1], LineBox[{66, 67}]}, 
        {Hue[0.06850347252601507, 1, 1], LineBox[{67, 68}]}, 
        {Hue[0.9717018103355191, 1, 1], LineBox[{68, 69}]}, 
        {Hue[0.8748549989781255, 1, 1], LineBox[{69, 70}]}, 
        {Hue[0.7789328517480816, 1, 1], LineBox[{70, 71}]}, 
        {Hue[0.6833400310612193, 1, 1], LineBox[{71, 72}]}, 
        {Hue[0.587582947708543, 1, 1], LineBox[{72, 73}]}, 
        {Hue[0.49205787070990953`, 1, 1], LineBox[{73, 74}]}, 
        {Hue[0.3962264733104969, 1, 1], LineBox[{74, 75}]}, 
        {Hue[0.30073678619523775`, 1, 1], LineBox[{75, 76}]}, 
        {Hue[0.20606099121402044`, 1, 1], LineBox[{76, 77}]}, 
        {Hue[0.11180750743429678`, 1, 1], LineBox[{77, 78}]}, 
        {Hue[0.01786908293360081, 1, 1], LineBox[{78, 79}]}, 
        {Hue[0.9238158665680171, 1, 1], LineBox[{79, 80}]}, 
        {Hue[0.8295682733677235, 1, 1], LineBox[{80, 81}]}, 
        {Hue[0.735411875619816, 1, 1], LineBox[{81, 82}]}, 
        {Hue[0.6416165761393002, 1, 1], LineBox[{82, 83}]}, 
        {Hue[0.5485735773427743, 1, 1], LineBox[{83, 84}]}, 
        {Hue[0.4562559025427074, 1, 1], LineBox[{84, 85}]}, 
        {Hue[0.3641662230314104, 1, 1], LineBox[{85, 86}]}, 
        {Hue[0.2717745783461878, 1, 1], LineBox[{86, 87}]}, 
        {Hue[0.17887247985716348`, 1, 1], LineBox[{87, 88}]}, 
        {Hue[0.08588874690629737, 1, 1], LineBox[{88, 89}]}, 
        {Hue[0.9936406858076294, 1, 1], LineBox[{89, 90}]}, 
        {Hue[0.9025475885767389, 1, 1], LineBox[{90, 91}]}, 
        {Hue[0.8123557106102066, 1, 1], LineBox[{91, 92}]}, 
        {Hue[0.722196790536449, 1, 1], LineBox[{92, 93}]}, 
        {Hue[0.63124672944796, 1, 1], LineBox[{93, 94}]}, 
        {Hue[0.5394334217154759, 1, 1], LineBox[{94, 95}]}, 
        {Hue[0.4476048393319134, 1, 1], LineBox[{95, 96}]}, 
        {Hue[0.35699908783455947`, 1, 1], LineBox[{96, 97}]}, 
        {Hue[0.2681214791428311, 1, 1], LineBox[{97, 98}]}, 
        {Hue[0.18030782830551154`, 1, 1], LineBox[{98, 99}]}, 
        {Hue[0.09215150663418024, 1, 1], LineBox[{99, 100}]}, 
        {Hue[0.0025077892689761004`, 1, 1], LineBox[{100, 101}]}, 
        {Hue[0.9115947297733166, 1, 1], LineBox[{101, 102}]}, 
        {Hue[0.8210927146819142, 1, 1], LineBox[{102, 103}]}, 
        {Hue[0.732829693010617, 1, 1], LineBox[{103, 104}]}, 
        {Hue[0.647133972193977, 1, 1], LineBox[{104, 105}]}, 
        {Hue[0.5625815606607734, 1, 1], LineBox[{105, 106}]}, 
        {Hue[0.47697251741405106`, 1, 1], LineBox[{106, 107}]}, 
        {Hue[0.38892812303980323`, 1, 1], LineBox[{107, 108}]}, 
        {Hue[0.29936197072723353`, 1, 1], LineBox[{108, 109}]}, 
        {Hue[0.2112842587929123, 1, 1], LineBox[{109, 110}]}, 
        {Hue[0.12719223182875483`, 1, 1], LineBox[{110, 111}]}, 
        {Hue[0.04695078965265709, 1, 1], LineBox[{111, 112}]}, 
        {Hue[0.9681574476610211, 1, 1], LineBox[{112, 113}]}, 
        {Hue[0.887706749483258, 1, 1], LineBox[{113, 114}]}, 
        {Hue[0.8037701829451505, 1, 1], LineBox[{114, 115}]}, 
        {Hue[0.7178930970056254, 1, 1], LineBox[{115, 116}]}, 
        {Hue[0.6347073831219328, 1, 1], LineBox[{116, 117}]}, 
        {Hue[0.5579546515943477, 1, 1], LineBox[{117, 118}]}, 
        {Hue[0.48772321475245084`, 1, 1], LineBox[{118, 119}]}, 
        {Hue[0.42141514090553406`, 1, 1], LineBox[{119, 120}]}, 
        {Hue[0.3554913039774097, 1, 1], LineBox[{120, 121}]}, 
        {Hue[0.28669414064235854`, 1, 1], LineBox[{121, 122}]}, 
        {Hue[0.21367847985739843`, 1, 1], LineBox[{122, 123}]}, 
        {Hue[0.13916554587353325`, 1, 1], LineBox[{123, 124}]}, 
        {Hue[0.06927835345538431, 1, 1], LineBox[{124, 125}]}, 
        {Hue[0.008753732053659424, 1, 1], LineBox[{125, 126}]}, 
        {Hue[0.958384426153491, 1, 1], LineBox[{126, 127}]}, 
        {Hue[0.9166770183971185, 1, 1], LineBox[{127, 128}]}, 
        {Hue[0.8817915576244195, 1, 1], LineBox[{128, 129}]}, 
        {Hue[0.8522672078087553, 1, 1], LineBox[{129, 130}]}, 
        {Hue[0.8270975424776865, 1, 1], LineBox[{130, 131}]}, 
        {Hue[0.8056284375105165, 1, 1], LineBox[{131, 132}]}, 
        {Hue[0.7874454690796864, 1, 1], LineBox[{132, 133}]}, 
        {Hue[0.7722914136674517, 1, 1], LineBox[{133, 134}]}, 
        {Hue[0.7600076628418265, 1, 1], LineBox[{134, 135}]}, 
        {Hue[0.750496805403843, 1, 1], LineBox[{135, 136}]}, 
        {Hue[0.7436936708530469, 1, 1], LineBox[{136, 137}]}, 
        {Hue[0.7395522002789043, 1, 1], LineBox[{137, 138}]}, 
        {Hue[0.7380306364234124, 1, 1], LineBox[{138, 139}]}, 
        {Hue[0.739088410762426, 1, 1], LineBox[{139, 140}]}, 
        {Hue[0.7426813910790278, 1, 1], LineBox[{140, 141}]}, 
        {Hue[0.7487603009809475, 1, 1], LineBox[{141, 142}]}, 
        {Hue[0.7572706370390397, 1, 1], LineBox[{142, 143}]}, 
        {Hue[0.7681537835764389, 1, 1], LineBox[{143, 144}]}, 
        {Hue[0.7813485124592214, 1, 1], LineBox[{144, 145}]}, 
        {Hue[0.7967903032737629, 1, 1], LineBox[{145, 146}]}, 
        {Hue[0.8144135638472767, 1, 1], LineBox[{146, 147}]}, 
        {Hue[0.8341527576324702, 1, 1], LineBox[{147, 148}]}, 
        {Hue[0.8559423552343227, 1, 1], LineBox[{148, 149}]}, 
        {Hue[0.8797184779594373, 1, 1], LineBox[{149, 150}]}, 
        {Hue[0.9054178672267975, 1, 1], LineBox[{150, 151}]}, 
        {Hue[0.9329806227146357, 1, 1], LineBox[{151, 152}]}, 
        {Hue[0.9623475758499133, 1, 1], LineBox[{152, 153}]}, 
        {Hue[0.9934629887045462, 1, 1], LineBox[{153, 154}]}, 
        {Hue[0.02627405855083121, 1, 1], LineBox[{154, 155}]}, 
        {Hue[0.06072941346367099, 1, 1], LineBox[{155, 156}]}, 
        {Hue[0.09678062203177676, 1, 1], LineBox[{156, 157}]}, 
        {Hue[0.1343824932010361, 1, 1], LineBox[{157, 158}]}, 
        {Hue[0.17349119558818452`, 1, 1], LineBox[{158, 159}]}, 
        {Hue[0.21406617571740993`, 1, 1], LineBox[{159, 160}]}, 
        {Hue[0.2560679976886549, 1, 1], LineBox[{160, 161}]}, 
        {Hue[0.299460457403954, 1, 1], LineBox[{161, 162}]}, 
        {Hue[0.3442092278042504, 1, 1], LineBox[{162, 163}]}, 
        {Hue[0.3902814284807919, 1, 1], LineBox[{163, 164}]}, 
        {Hue[0.4376464529812861, 1, 1], LineBox[{164, 165}]}, 
        {Hue[0.486275510377512, 1, 1], LineBox[{165, 166}]}, 
        {Hue[0.5361404086810144, 1, 1], LineBox[{166, 167}]}, 
        {Hue[0.5872156253981717, 1, 1], LineBox[{167, 168}]}, 
        {Hue[0.63947675730222, 1, 1], LineBox[{168, 169}]}, 
        {Hue[0.6928994274576521, 1, 1], LineBox[{169, 170}]}, 
        {Hue[0.7474627961633682, 1, 1], LineBox[{170, 171}]}, 
        {Hue[0.8031448831692406, 1, 1], LineBox[{171, 172}]}, 
        {Hue[0.8599277071610487, 1, 1], LineBox[{172, 173}]}, 
        {Hue[0.9177905921948738, 1, 1], LineBox[{173, 174}]}, 
        {Hue[0.9767162994358801, 1, 1], LineBox[{174, 175}]}, 
        {Hue[0.03668769482659254, 1, 1], LineBox[{175, 176}]}, 
        {Hue[0.09768870176319887, 1, 1], LineBox[{176, 177}]}, 
        {Hue[0.15970390592623193`, 1, 1], LineBox[{177, 178}]}, 
        {Hue[0.22271848133218072`, 1, 1], LineBox[{178, 179}]}, 
        {Hue[0.2867176866955756, 1, 1], LineBox[{179, 180}]}, 
        {Hue[0.3516896132531366, 1, 1], LineBox[{180, 181}]}, 
        {Hue[0.41762027877249996`, 1, 1], LineBox[{181, 182}]}, 
        {Hue[0.4844980226954021, 1, 1], LineBox[{182, 183}]}, 
        {Hue[0.5523092351240234, 1, 1], LineBox[{183, 184}]}, 
        {Hue[0.621046141242471, 1, 1], LineBox[{184, 185}]}, 
        {Hue[0.6906954003728225, 1, 1], LineBox[{185, 186}]}, 
        {Hue[0.7612464295655417, 1, 1], LineBox[{186, 187}]}, 
        {Hue[0.8326892519615281, 1, 1], LineBox[{187, 188}]}, 
        {Hue[0.9050154457110178, 1, 1], LineBox[{188, 189}]}, 
        {Hue[0.9782137976593589, 1, 1], LineBox[{189, 190}]}, 
        {Hue[0.05227788399332845, 1, 1], LineBox[{190, 191}]}, 
        {Hue[0.1271986005150233, 1, 1], LineBox[{191, 192}]}, 
        {Hue[0.2029642440043483, 1, 1], LineBox[{192, 193}]}, 
        {Hue[0.27956900942770624`, 1, 1], LineBox[{193, 194}]}, 
        {Hue[0.3570070126386154, 1, 1], LineBox[{194, 195}]}, 
        {Hue[0.43526876191562286`, 1, 1], LineBox[{195, 196}]}, 
        {Hue[0.5143483531822721, 1, 1], LineBox[{196, 197}]}, 
        {Hue[0.5942364871856414, 1, 1], LineBox[{197, 198}]}, 
        {Hue[0.6749261104680981, 1, 1], LineBox[{198, 199}]}, 
        {Hue[0.7564165186521847, 1, 1], LineBox[{199, 200}]}, 
        {Hue[0.8386941839035054, 1, 1], LineBox[{200, 201}]}, 
        {Hue[0.9217502032467062, 1, 1], LineBox[{201, 202}]}, 
        {Hue[0.005588047415432818, 1, 1], LineBox[{202, 203}]}, 
        {Hue[0.09020190054450962, 1, 1], LineBox[{203, 204}]}, 
        {Hue[0.17557830629088042`, 1, 1], LineBox[{204, 205}]}, 
        {Hue[0.2617155460289287, 1, 1], LineBox[{205, 206}]}, 
        {Hue[0.3486030056287242, 1, 1], LineBox[{206, 207}]}, 
        {Hue[0.43623852605324986`, 1, 1], LineBox[{207, 208}]}, 
        {Hue[0.5246243796516621, 1, 1], LineBox[{208, 209}]}, 
        {Hue[0.6137467720443291, 1, 1], LineBox[{209, 210}]}, 
        {Hue[0.7036113582715795, 1, 1], LineBox[{210, 211}]}, 
        {Hue[0.7942020535195174, 1, 1], LineBox[{211, 212}]}, 
        {Hue[0.8855024036439301, 1, 1], LineBox[{212, 213}]}, 
        {Hue[0.9775292546983889, 1, 1], LineBox[{213, 214}]}, 
        {Hue[0.07027539516154879, 1, 1], LineBox[{214, 215}]}, 
        {Hue[0.16374124482826072`, 1, 1], LineBox[{215, 216}]}, 
        {Hue[0.25791700557200636`, 1, 1], LineBox[{216, 217}]}, 
        {Hue[0.3527640228073858, 1, 1], LineBox[{217, 218}]}, 
        {Hue[0.4482987662581584, 1, 1], LineBox[{218, 219}]}, 
        {Hue[0.5445531907493291, 1, 1], LineBox[{219, 220}]}, 
        {Hue[0.6415090340163123, 1, 1], LineBox[{220, 221}]}, 
        {Hue[0.739158911610914, 1, 1], LineBox[{221, 222}]}, 
        {Hue[0.8374834060735057, 1, 1], LineBox[{222, 223}]}, 
        {Hue[0.9364240543729676, 1, 1], LineBox[{223, 224}]}, 
        {Hue[0.03603829961410431, 1, 1], LineBox[{224, 225}]}, 
        {Hue[0.13642177385132023`, 1, 1], LineBox[{225, 226}]}, 
        {Hue[0.23743312162973995`, 1, 1], LineBox[{226, 227}]}, 
        {Hue[0.33890501166499326`, 1, 1], LineBox[{227, 228}]}, 
        {Hue[0.4410449979957778, 1, 1], LineBox[{228, 229}]}, 
        {Hue[0.5441512204674002, 1, 1], LineBox[{229, 230}]}, 
        {Hue[0.6479683063190227, 1, 1], LineBox[{230, 231}]}, 
        {Hue[0.7520934886159469, 1, 1], LineBox[{231, 232}]}, 
        {Hue[0.8569136201029212, 1, 1], LineBox[{232, 233}]}, 
        {Hue[0.9627047539340904, 1, 1], LineBox[{233, 234}]}, 
        {Hue[0.06879089341509073, 1, 1], LineBox[{234, 235}]}, 
        {Hue[0.17533461697758274`, 1, 1], LineBox[{235, 236}]}, 
        {Hue[0.28319618900543175`, 1, 1], LineBox[{236, 237}]}, 
        {Hue[0.39149295315047145`, 1, 1], LineBox[{237, 238}]}, 
        {Hue[0.4996425226407696, 1, 1], LineBox[{238, 239}]}, 
        {Hue[0.6094061045208964, 1, 1], LineBox[{239, 240}]}, 
        {Hue[0.7205987326917802, 1, 1], LineBox[{240, 241}]}, 
        {Hue[0.8307909058918155, 1, 1], LineBox[{241, 242}]}, 
        {Hue[0.9413621288318382, 1, 1], LineBox[{242, 243}]}, 
        {Hue[0.055154985274585344`, 1, 1], LineBox[{243, 244}]}, 
        {Hue[0.16908344480096296`, 1, 1], LineBox[{244, 245}]}, 
        {Hue[0.27949834648938626`, 1, 1], LineBox[{245, 246}]}, 
        {Hue[0.39178904539044196`, 1, 1], LineBox[{246, 247}]}, 
        {Hue[0.5113599233497237, 1, 1], LineBox[{247, 248}]}, 
        {Hue[0.6321214530795161, 1, 1], LineBox[{248, 249}]}, 
        {Hue[0.7464805255537773, 1, 1], LineBox[{249, 250}]}, 
        {Hue[0.8602620296701579, 1, 1], LineBox[{250, 251}]}}}},
     VertexColors->{
       Hue[0.05956624042050353, 1, 1], 
       Hue[0.9831017166735266, 1, 1], 
       Hue[0.053784012938536886`, 1, 1], 
       Hue[0.8965646702957913, 1, 1], 
       Hue[0.7133375980565537, 1, 1], 
       Hue[0.472416995417405, 1, 1], 
       Hue[0.22615072621804672`, 1, 1], 
       Hue[0.08421188501446454, 1, 1], 
       Hue[0.8829614102660753, 1, 1], 
       Hue[0.8222896416930823, 1, 1], 
       Hue[0.6400882201477065, 1, 1], 
       Hue[0.5078569467204729, 1, 1], 
       Hue[0.4582011452917198, 1, 1], 
       Hue[0.2860972878164505, 1, 1], 
       Hue[0.1269644106342963, 1, 1], 
       Hue[0.008489555056382105, 1, 1], 
       Hue[0.9047969445291322, 1, 1], 
       Hue[0.8069026689219323, 1, 1], 
       Hue[0.7433426798313757, 1, 1], 
       Hue[0.6211645129944474, 1, 1], 
       Hue[0.570574249490098, 1, 1], 
       Hue[0.37442686964880406`, 1, 1], 
       Hue[0.41406408292314084`, 1, 1], 
       Hue[0.27304668255560127`, 1, 1], 
       Hue[0.1640613943112119, 1, 1], 
       Hue[0.9217837241712366, 1, 1], 
       Hue[0.04927441714522789, 1, 1], 
       Hue[0.7395544576770468, 1, 1], 
       Hue[0.8366427329778339, 1, 1], 
       Hue[0.640224275610829, 1, 1], 
       Hue[0.5005906513158629, 1, 1], 
       Hue[0.39893509617051615`, 1, 1], 
       Hue[0.31827148243229864`, 1, 1], 
       Hue[0.10562517832932644`, 1, 1], 
       Hue[0.05647944373999831, 1, 1], 
       Hue[0.954476650596741, 1, 1], 
       Hue[0.8121914267182476, 1, 1], 
       Hue[0.6843275724593121, 1, 1], 
       Hue[0.8357639889934845, 1, 1], 
       Hue[0.6544318649455261, 1, 1], 
       Hue[0.571673756180975, 1, 1], 
       Hue[0.5103863038769656, 1, 1], 
       Hue[0.38946345539840677`, 1, 1], 
       Hue[0.3407740913415771, 1, 1], 
       Hue[0.2881490527713575, 1, 1], 
       Hue[0.14006130480093046`, 1, 1], 
       Hue[0.03270695109683777, 1, 1], 
       Hue[0.9222580029127315, 1, 1], 
       Hue[0.8216153254234894, 1, 1], 
       Hue[0.7327219348514502, 1, 1], 
       Hue[0.6329691624014069, 1, 1], 
       Hue[0.5441018001510657, 1, 1], 
       Hue[0.451179412264641, 1, 1], 
       Hue[0.34884452009062944`, 1, 1], 
       Hue[0.24649878947472764`, 1, 1], 
       Hue[0.15143639484703394`, 1, 1], 
       Hue[0.050421982901845644`, 1, 1], 
       Hue[0.9481497781591328, 1, 1], 
       Hue[0.8502040510520766, 1, 1], 
       Hue[0.7515057307791237, 1, 1], 
       Hue[0.6545229892266976, 1, 1], 
       Hue[0.556780119053588, 1, 1], 
       Hue[0.45912955175820863`, 1, 1], 
       Hue[0.36229952563725926`, 1, 1], 
       Hue[0.2647079518516234, 1, 1], 
       Hue[0.16676307041787705`, 1, 1], 
       Hue[0.06850347252601507, 1, 1], 
       Hue[0.9717018103355191, 1, 1], 
       Hue[0.8748549989781255, 1, 1], 
       Hue[0.7789328517480816, 1, 1], 
       Hue[0.6833400310612193, 1, 1], 
       Hue[0.587582947708543, 1, 1], 
       Hue[0.49205787070990953`, 1, 1], 
       Hue[0.3962264733104969, 1, 1], 
       Hue[0.30073678619523775`, 1, 1], 
       Hue[0.20606099121402044`, 1, 1], 
       Hue[0.11180750743429678`, 1, 1], 
       Hue[0.01786908293360081, 1, 1], 
       Hue[0.9238158665680171, 1, 1], 
       Hue[0.8295682733677235, 1, 1], 
       Hue[0.735411875619816, 1, 1], 
       Hue[0.6416165761393002, 1, 1], 
       Hue[0.5485735773427743, 1, 1], 
       Hue[0.4562559025427074, 1, 1], 
       Hue[0.3641662230314104, 1, 1], 
       Hue[0.2717745783461878, 1, 1], 
       Hue[0.17887247985716348`, 1, 1], 
       Hue[0.08588874690629737, 1, 1], 
       Hue[0.9936406858076294, 1, 1], 
       Hue[0.9025475885767389, 1, 1], 
       Hue[0.8123557106102066, 1, 1], 
       Hue[0.722196790536449, 1, 1], 
       Hue[0.63124672944796, 1, 1], 
       Hue[0.5394334217154759, 1, 1], 
       Hue[0.4476048393319134, 1, 1], 
       Hue[0.35699908783455947`, 1, 1], 
       Hue[0.2681214791428311, 1, 1], 
       Hue[0.18030782830551154`, 1, 1], 
       Hue[0.09215150663418024, 1, 1], 
       Hue[0.0025077892689761004`, 1, 1], 
       Hue[0.9115947297733166, 1, 1], 
       Hue[0.8210927146819142, 1, 1], 
       Hue[0.732829693010617, 1, 1], 
       Hue[0.647133972193977, 1, 1], 
       Hue[0.5625815606607734, 1, 1], 
       Hue[0.47697251741405106`, 1, 1], 
       Hue[0.38892812303980323`, 1, 1], 
       Hue[0.29936197072723353`, 1, 1], 
       Hue[0.2112842587929123, 1, 1], 
       Hue[0.12719223182875483`, 1, 1], 
       Hue[0.04695078965265709, 1, 1], 
       Hue[0.9681574476610211, 1, 1], 
       Hue[0.887706749483258, 1, 1], 
       Hue[0.8037701829451505, 1, 1], 
       Hue[0.7178930970056254, 1, 1], 
       Hue[0.6347073831219328, 1, 1], 
       Hue[0.5579546515943477, 1, 1], 
       Hue[0.48772321475245084`, 1, 1], 
       Hue[0.42141514090553406`, 1, 1], 
       Hue[0.3554913039774097, 1, 1], 
       Hue[0.28669414064235854`, 1, 1], 
       Hue[0.21367847985739843`, 1, 1], 
       Hue[0.13916554587353325`, 1, 1], 
       Hue[0.06927835345538431, 1, 1], 
       Hue[0.008753732053659424, 1, 1], 
       Hue[0.958384426153491, 1, 1], 
       Hue[0.9166770183971185, 1, 1], 
       Hue[0.8817915576244195, 1, 1], 
       Hue[0.8522672078087553, 1, 1], 
       Hue[0.8270975424776865, 1, 1], 
       Hue[0.8056284375105165, 1, 1], 
       Hue[0.7874454690796864, 1, 1], 
       Hue[0.7722914136674517, 1, 1], 
       Hue[0.7600076628418265, 1, 1], 
       Hue[0.750496805403843, 1, 1], 
       Hue[0.7436936708530469, 1, 1], 
       Hue[0.7395522002789043, 1, 1], 
       Hue[0.7380306364234124, 1, 1], 
       Hue[0.739088410762426, 1, 1], 
       Hue[0.7426813910790278, 1, 1], 
       Hue[0.7487603009809475, 1, 1], 
       Hue[0.7572706370390397, 1, 1], 
       Hue[0.7681537835764389, 1, 1], 
       Hue[0.7813485124592214, 1, 1], 
       Hue[0.7967903032737629, 1, 1], 
       Hue[0.8144135638472767, 1, 1], 
       Hue[0.8341527576324702, 1, 1], 
       Hue[0.8559423552343227, 1, 1], 
       Hue[0.8797184779594373, 1, 1], 
       Hue[0.9054178672267975, 1, 1], 
       Hue[0.9329806227146357, 1, 1], 
       Hue[0.9623475758499133, 1, 1], 
       Hue[0.9934629887045462, 1, 1], 
       Hue[0.02627405855083121, 1, 1], 
       Hue[0.06072941346367099, 1, 1], 
       Hue[0.09678062203177676, 1, 1], 
       Hue[0.1343824932010361, 1, 1], 
       Hue[0.17349119558818452`, 1, 1], 
       Hue[0.21406617571740993`, 1, 1], 
       Hue[0.2560679976886549, 1, 1], 
       Hue[0.299460457403954, 1, 1], 
       Hue[0.3442092278042504, 1, 1], 
       Hue[0.3902814284807919, 1, 1], 
       Hue[0.4376464529812861, 1, 1], 
       Hue[0.486275510377512, 1, 1], 
       Hue[0.5361404086810144, 1, 1], 
       Hue[0.5872156253981717, 1, 1], 
       Hue[0.63947675730222, 1, 1], 
       Hue[0.6928994274576521, 1, 1], 
       Hue[0.7474627961633682, 1, 1], 
       Hue[0.8031448831692406, 1, 1], 
       Hue[0.8599277071610487, 1, 1], 
       Hue[0.9177905921948738, 1, 1], 
       Hue[0.9767162994358801, 1, 1], 
       Hue[0.03668769482659254, 1, 1], 
       Hue[0.09768870176319887, 1, 1], 
       Hue[0.15970390592623193`, 1, 1], 
       Hue[0.22271848133218072`, 1, 1], 
       Hue[0.2867176866955756, 1, 1], 
       Hue[0.3516896132531366, 1, 1], 
       Hue[0.41762027877249996`, 1, 1], 
       Hue[0.4844980226954021, 1, 1], 
       Hue[0.5523092351240234, 1, 1], 
       Hue[0.621046141242471, 1, 1], 
       Hue[0.6906954003728225, 1, 1], 
       Hue[0.7612464295655417, 1, 1], 
       Hue[0.8326892519615281, 1, 1], 
       Hue[0.9050154457110178, 1, 1], 
       Hue[0.9782137976593589, 1, 1], 
       Hue[0.05227788399332845, 1, 1], 
       Hue[0.1271986005150233, 1, 1], 
       Hue[0.2029642440043483, 1, 1], 
       Hue[0.27956900942770624`, 1, 1], 
       Hue[0.3570070126386154, 1, 1], 
       Hue[0.43526876191562286`, 1, 1], 
       Hue[0.5143483531822721, 1, 1], 
       Hue[0.5942364871856414, 1, 1], 
       Hue[0.6749261104680981, 1, 1], 
       Hue[0.7564165186521847, 1, 1], 
       Hue[0.8386941839035054, 1, 1], 
       Hue[0.9217502032467062, 1, 1], 
       Hue[0.005588047415432818, 1, 1], 
       Hue[0.09020190054450962, 1, 1], 
       Hue[0.17557830629088042`, 1, 1], 
       Hue[0.2617155460289287, 1, 1], 
       Hue[0.3486030056287242, 1, 1], 
       Hue[0.43623852605324986`, 1, 1], 
       Hue[0.5246243796516621, 1, 1], 
       Hue[0.6137467720443291, 1, 1], 
       Hue[0.7036113582715795, 1, 1], 
       Hue[0.7942020535195174, 1, 1], 
       Hue[0.8855024036439301, 1, 1], 
       Hue[0.9775292546983889, 1, 1], 
       Hue[0.07027539516154879, 1, 1], 
       Hue[0.16374124482826072`, 1, 1], 
       Hue[0.25791700557200636`, 1, 1], 
       Hue[0.3527640228073858, 1, 1], 
       Hue[0.4482987662581584, 1, 1], 
       Hue[0.5445531907493291, 1, 1], 
       Hue[0.6415090340163123, 1, 1], 
       Hue[0.739158911610914, 1, 1], 
       Hue[0.8374834060735057, 1, 1], 
       Hue[0.9364240543729676, 1, 1], 
       Hue[0.03603829961410431, 1, 1], 
       Hue[0.13642177385132023`, 1, 1], 
       Hue[0.23743312162973995`, 1, 1], 
       Hue[0.33890501166499326`, 1, 1], 
       Hue[0.4410449979957778, 1, 1], 
       Hue[0.5441512204674002, 1, 1], 
       Hue[0.6479683063190227, 1, 1], 
       Hue[0.7520934886159469, 1, 1], 
       Hue[0.8569136201029212, 1, 1], 
       Hue[0.9627047539340904, 1, 1], 
       Hue[0.06879089341509073, 1, 1], 
       Hue[0.17533461697758274`, 1, 1], 
       Hue[0.28319618900543175`, 1, 1], 
       Hue[0.39149295315047145`, 1, 1], 
       Hue[0.4996425226407696, 1, 1], 
       Hue[0.6094061045208964, 1, 1], 
       Hue[0.7205987326917802, 1, 1], 
       Hue[0.8307909058918155, 1, 1], 
       Hue[0.9413621288318382, 1, 1], 
       Hue[0.055154985274585344`, 1, 1], 
       Hue[0.16908344480096296`, 1, 1], 
       Hue[0.27949834648938626`, 1, 1], 
       Hue[0.39178904539044196`, 1, 1], 
       Hue[0.5113599233497237, 1, 1], 
       Hue[0.6321214530795161, 1, 1], 
       Hue[0.7464805255537773, 1, 1], 
       Hue[0.8602620296701579, 1, 1], 
       Hue[0.9811501935614256, 1, 1], 
       Hue[0.05956624042050353, 1, 1], 
       Hue[0.9831017166735266, 1, 1], 
       Hue[0.9831017166735266, 1, 1], 
       Hue[0.053784012938536886`, 1, 1], 
       Hue[0.053784012938536886`, 1, 1], 
       Hue[0.8965646702957913, 1, 1], 
       Hue[0.8965646702957913, 1, 1], 
       Hue[0.7133375980565537, 1, 1], 
       Hue[0.7133375980565537, 1, 1], 
       Hue[0.472416995417405, 1, 1], 
       Hue[0.472416995417405, 1, 1], 
       Hue[0.22615072621804672`, 1, 1], 
       Hue[0.22615072621804672`, 1, 1], 
       Hue[0.08421188501446454, 1, 1], 
       Hue[0.08421188501446454, 1, 1], 
       Hue[0.8829614102660753, 1, 1], 
       Hue[0.8829614102660753, 1, 1], 
       Hue[0.8222896416930823, 1, 1], 
       Hue[0.8222896416930823, 1, 1], 
       Hue[0.6400882201477065, 1, 1], 
       Hue[0.6400882201477065, 1, 1], 
       Hue[0.5078569467204729, 1, 1], 
       Hue[0.5078569467204729, 1, 1], 
       Hue[0.4582011452917198, 1, 1], 
       Hue[0.4582011452917198, 1, 1], 
       Hue[0.2860972878164505, 1, 1], 
       Hue[0.2860972878164505, 1, 1], 
       Hue[0.1269644106342963, 1, 1], 
       Hue[0.1269644106342963, 1, 1], 
       Hue[0.008489555056382105, 1, 1], 
       Hue[0.008489555056382105, 1, 1], 
       Hue[0.9047969445291322, 1, 1], 
       Hue[0.9047969445291322, 1, 1], 
       Hue[0.8069026689219323, 1, 1], 
       Hue[0.8069026689219323, 1, 1], 
       Hue[0.7433426798313757, 1, 1], 
       Hue[0.7433426798313757, 1, 1], 
       Hue[0.6211645129944474, 1, 1], 
       Hue[0.6211645129944474, 1, 1], 
       Hue[0.570574249490098, 1, 1], 
       Hue[0.570574249490098, 1, 1], 
       Hue[0.37442686964880406`, 1, 1], 
       Hue[0.37442686964880406`, 1, 1], 
       Hue[0.41406408292314084`, 1, 1], 
       Hue[0.41406408292314084`, 1, 1], 
       Hue[0.27304668255560127`, 1, 1], 
       Hue[0.27304668255560127`, 1, 1], 
       Hue[0.1640613943112119, 1, 1], 
       Hue[0.1640613943112119, 1, 1], 
       Hue[0.9217837241712366, 1, 1], 
       Hue[0.9217837241712366, 1, 1], 
       Hue[0.04927441714522789, 1, 1], 
       Hue[0.04927441714522789, 1, 1], 
       Hue[0.7395544576770468, 1, 1], 
       Hue[0.7395544576770468, 1, 1], 
       Hue[0.8366427329778339, 1, 1], 
       Hue[0.8366427329778339, 1, 1], 
       Hue[0.640224275610829, 1, 1], 
       Hue[0.640224275610829, 1, 1], 
       Hue[0.5005906513158629, 1, 1], 
       Hue[0.5005906513158629, 1, 1], 
       Hue[0.39893509617051615`, 1, 1], 
       Hue[0.39893509617051615`, 1, 1], 
       Hue[0.31827148243229864`, 1, 1], 
       Hue[0.31827148243229864`, 1, 1], 
       Hue[0.10562517832932644`, 1, 1], 
       Hue[0.10562517832932644`, 1, 1], 
       Hue[0.05647944373999831, 1, 1], 
       Hue[0.05647944373999831, 1, 1], 
       Hue[0.954476650596741, 1, 1], 
       Hue[0.954476650596741, 1, 1], 
       Hue[0.8121914267182476, 1, 1], 
       Hue[0.8121914267182476, 1, 1], 
       Hue[0.6843275724593121, 1, 1], 
       Hue[0.6843275724593121, 1, 1], 
       Hue[0.8357639889934845, 1, 1], 
       Hue[0.8357639889934845, 1, 1], 
       Hue[0.6544318649455261, 1, 1], 
       Hue[0.6544318649455261, 1, 1], 
       Hue[0.571673756180975, 1, 1], 
       Hue[0.571673756180975, 1, 1], 
       Hue[0.5103863038769656, 1, 1], 
       Hue[0.5103863038769656, 1, 1], 
       Hue[0.38946345539840677`, 1, 1], 
       Hue[0.38946345539840677`, 1, 1], 
       Hue[0.3407740913415771, 1, 1], 
       Hue[0.3407740913415771, 1, 1], 
       Hue[0.2881490527713575, 1, 1], 
       Hue[0.2881490527713575, 1, 1], 
       Hue[0.14006130480093046`, 1, 1], 
       Hue[0.14006130480093046`, 1, 1], 
       Hue[0.03270695109683777, 1, 1], 
       Hue[0.03270695109683777, 1, 1], 
       Hue[0.9222580029127315, 1, 1], 
       Hue[0.9222580029127315, 1, 1], 
       Hue[0.8216153254234894, 1, 1], 
       Hue[0.8216153254234894, 1, 1], 
       Hue[0.7327219348514502, 1, 1], 
       Hue[0.7327219348514502, 1, 1], 
       Hue[0.6329691624014069, 1, 1], 
       Hue[0.6329691624014069, 1, 1], 
       Hue[0.5441018001510657, 1, 1], 
       Hue[0.5441018001510657, 1, 1], 
       Hue[0.451179412264641, 1, 1], 
       Hue[0.451179412264641, 1, 1], 
       Hue[0.34884452009062944`, 1, 1], 
       Hue[0.34884452009062944`, 1, 1], 
       Hue[0.24649878947472764`, 1, 1], 
       Hue[0.24649878947472764`, 1, 1], 
       Hue[0.15143639484703394`, 1, 1], 
       Hue[0.15143639484703394`, 1, 1], 
       Hue[0.050421982901845644`, 1, 1], 
       Hue[0.050421982901845644`, 1, 1], 
       Hue[0.9481497781591328, 1, 1], 
       Hue[0.9481497781591328, 1, 1], 
       Hue[0.8502040510520766, 1, 1], 
       Hue[0.8502040510520766, 1, 1], 
       Hue[0.7515057307791237, 1, 1], 
       Hue[0.7515057307791237, 1, 1], 
       Hue[0.6545229892266976, 1, 1], 
       Hue[0.6545229892266976, 1, 1], 
       Hue[0.556780119053588, 1, 1], 
       Hue[0.556780119053588, 1, 1], 
       Hue[0.45912955175820863`, 1, 1], 
       Hue[0.45912955175820863`, 1, 1], 
       Hue[0.36229952563725926`, 1, 1], 
       Hue[0.36229952563725926`, 1, 1], 
       Hue[0.2647079518516234, 1, 1], 
       Hue[0.2647079518516234, 1, 1], 
       Hue[0.16676307041787705`, 1, 1], 
       Hue[0.16676307041787705`, 1, 1], 
       Hue[0.06850347252601507, 1, 1], 
       Hue[0.06850347252601507, 1, 1], 
       Hue[0.9717018103355191, 1, 1], 
       Hue[0.9717018103355191, 1, 1], 
       Hue[0.8748549989781255, 1, 1], 
       Hue[0.8748549989781255, 1, 1], 
       Hue[0.7789328517480816, 1, 1], 
       Hue[0.7789328517480816, 1, 1], 
       Hue[0.6833400310612193, 1, 1], 
       Hue[0.6833400310612193, 1, 1], 
       Hue[0.587582947708543, 1, 1], 
       Hue[0.587582947708543, 1, 1], 
       Hue[0.49205787070990953`, 1, 1], 
       Hue[0.49205787070990953`, 1, 1], 
       Hue[0.3962264733104969, 1, 1], 
       Hue[0.3962264733104969, 1, 1], 
       Hue[0.30073678619523775`, 1, 1], 
       Hue[0.30073678619523775`, 1, 1], 
       Hue[0.20606099121402044`, 1, 1], 
       Hue[0.20606099121402044`, 1, 1], 
       Hue[0.11180750743429678`, 1, 1], 
       Hue[0.11180750743429678`, 1, 1], 
       Hue[0.01786908293360081, 1, 1], 
       Hue[0.01786908293360081, 1, 1], 
       Hue[0.9238158665680171, 1, 1], 
       Hue[0.9238158665680171, 1, 1], 
       Hue[0.8295682733677235, 1, 1], 
       Hue[0.8295682733677235, 1, 1], 
       Hue[0.735411875619816, 1, 1], 
       Hue[0.735411875619816, 1, 1], 
       Hue[0.6416165761393002, 1, 1], 
       Hue[0.6416165761393002, 1, 1], 
       Hue[0.5485735773427743, 1, 1], 
       Hue[0.5485735773427743, 1, 1], 
       Hue[0.4562559025427074, 1, 1], 
       Hue[0.4562559025427074, 1, 1], 
       Hue[0.3641662230314104, 1, 1], 
       Hue[0.3641662230314104, 1, 1], 
       Hue[0.2717745783461878, 1, 1], 
       Hue[0.2717745783461878, 1, 1], 
       Hue[0.17887247985716348`, 1, 1], 
       Hue[0.17887247985716348`, 1, 1], 
       Hue[0.08588874690629737, 1, 1], 
       Hue[0.08588874690629737, 1, 1], 
       Hue[0.9936406858076294, 1, 1], 
       Hue[0.9936406858076294, 1, 1], 
       Hue[0.9025475885767389, 1, 1], 
       Hue[0.9025475885767389, 1, 1], 
       Hue[0.8123557106102066, 1, 1], 
       Hue[0.8123557106102066, 1, 1], 
       Hue[0.722196790536449, 1, 1], 
       Hue[0.722196790536449, 1, 1], 
       Hue[0.63124672944796, 1, 1], 
       Hue[0.63124672944796, 1, 1], 
       Hue[0.5394334217154759, 1, 1], 
       Hue[0.5394334217154759, 1, 1], 
       Hue[0.4476048393319134, 1, 1], 
       Hue[0.4476048393319134, 1, 1], 
       Hue[0.35699908783455947`, 1, 1], 
       Hue[0.35699908783455947`, 1, 1], 
       Hue[0.2681214791428311, 1, 1], 
       Hue[0.2681214791428311, 1, 1], 
       Hue[0.18030782830551154`, 1, 1], 
       Hue[0.18030782830551154`, 1, 1], 
       Hue[0.09215150663418024, 1, 1], 
       Hue[0.09215150663418024, 1, 1], 
       Hue[0.0025077892689761004`, 1, 1], 
       Hue[0.0025077892689761004`, 1, 1], 
       Hue[0.9115947297733166, 1, 1], 
       Hue[0.9115947297733166, 1, 1], 
       Hue[0.8210927146819142, 1, 1], 
       Hue[0.8210927146819142, 1, 1], 
       Hue[0.732829693010617, 1, 1], 
       Hue[0.732829693010617, 1, 1], 
       Hue[0.647133972193977, 1, 1], 
       Hue[0.647133972193977, 1, 1], 
       Hue[0.5625815606607734, 1, 1], 
       Hue[0.5625815606607734, 1, 1], 
       Hue[0.47697251741405106`, 1, 1], 
       Hue[0.47697251741405106`, 1, 1], 
       Hue[0.38892812303980323`, 1, 1], 
       Hue[0.38892812303980323`, 1, 1], 
       Hue[0.29936197072723353`, 1, 1], 
       Hue[0.29936197072723353`, 1, 1], 
       Hue[0.2112842587929123, 1, 1], 
       Hue[0.2112842587929123, 1, 1], 
       Hue[0.12719223182875483`, 1, 1], 
       Hue[0.12719223182875483`, 1, 1], 
       Hue[0.04695078965265709, 1, 1], 
       Hue[0.04695078965265709, 1, 1], 
       Hue[0.9681574476610211, 1, 1], 
       Hue[0.9681574476610211, 1, 1], 
       Hue[0.887706749483258, 1, 1], 
       Hue[0.887706749483258, 1, 1], 
       Hue[0.8037701829451505, 1, 1], 
       Hue[0.8037701829451505, 1, 1], 
       Hue[0.7178930970056254, 1, 1], 
       Hue[0.7178930970056254, 1, 1], 
       Hue[0.6347073831219328, 1, 1], 
       Hue[0.6347073831219328, 1, 1], 
       Hue[0.5579546515943477, 1, 1], 
       Hue[0.5579546515943477, 1, 1], 
       Hue[0.48772321475245084`, 1, 1], 
       Hue[0.48772321475245084`, 1, 1], 
       Hue[0.42141514090553406`, 1, 1], 
       Hue[0.42141514090553406`, 1, 1], 
       Hue[0.3554913039774097, 1, 1], 
       Hue[0.3554913039774097, 1, 1], 
       Hue[0.28669414064235854`, 1, 1], 
       Hue[0.28669414064235854`, 1, 1], 
       Hue[0.21367847985739843`, 1, 1], 
       Hue[0.21367847985739843`, 1, 1], 
       Hue[0.13916554587353325`, 1, 1], 
       Hue[0.13916554587353325`, 1, 1], 
       Hue[0.06927835345538431, 1, 1], 
       Hue[0.06927835345538431, 1, 1], 
       Hue[0.008753732053659424, 1, 1], 
       Hue[0.008753732053659424, 1, 1], 
       Hue[0.958384426153491, 1, 1], 
       Hue[0.958384426153491, 1, 1], 
       Hue[0.9166770183971185, 1, 1], 
       Hue[0.9166770183971185, 1, 1], 
       Hue[0.8817915576244195, 1, 1], 
       Hue[0.8817915576244195, 1, 1], 
       Hue[0.8522672078087553, 1, 1], 
       Hue[0.8522672078087553, 1, 1], 
       Hue[0.8270975424776865, 1, 1], 
       Hue[0.8270975424776865, 1, 1], 
       Hue[0.8056284375105165, 1, 1], 
       Hue[0.8056284375105165, 1, 1], 
       Hue[0.7874454690796864, 1, 1], 
       Hue[0.7874454690796864, 1, 1], 
       Hue[0.7722914136674517, 1, 1], 
       Hue[0.7722914136674517, 1, 1], 
       Hue[0.7600076628418265, 1, 1], 
       Hue[0.7600076628418265, 1, 1], 
       Hue[0.750496805403843, 1, 1], 
       Hue[0.750496805403843, 1, 1], 
       Hue[0.7436936708530469, 1, 1], 
       Hue[0.7436936708530469, 1, 1], 
       Hue[0.7395522002789043, 1, 1], 
       Hue[0.7395522002789043, 1, 1], 
       Hue[0.7380306364234124, 1, 1], 
       Hue[0.7380306364234124, 1, 1], 
       Hue[0.739088410762426, 1, 1], 
       Hue[0.739088410762426, 1, 1], 
       Hue[0.7426813910790278, 1, 1], 
       Hue[0.7426813910790278, 1, 1], 
       Hue[0.7487603009809475, 1, 1], 
       Hue[0.7487603009809475, 1, 1], 
       Hue[0.7572706370390397, 1, 1], 
       Hue[0.7572706370390397, 1, 1], 
       Hue[0.7681537835764389, 1, 1], 
       Hue[0.7681537835764389, 1, 1], 
       Hue[0.7813485124592214, 1, 1], 
       Hue[0.7813485124592214, 1, 1], 
       Hue[0.7967903032737629, 1, 1], 
       Hue[0.7967903032737629, 1, 1], 
       Hue[0.8144135638472767, 1, 1], 
       Hue[0.8144135638472767, 1, 1], 
       Hue[0.8341527576324702, 1, 1], 
       Hue[0.8341527576324702, 1, 1], 
       Hue[0.8559423552343227, 1, 1], 
       Hue[0.8559423552343227, 1, 1], 
       Hue[0.8797184779594373, 1, 1], 
       Hue[0.8797184779594373, 1, 1], 
       Hue[0.9054178672267975, 1, 1], 
       Hue[0.9054178672267975, 1, 1], 
       Hue[0.9329806227146357, 1, 1], 
       Hue[0.9329806227146357, 1, 1], 
       Hue[0.9623475758499133, 1, 1], 
       Hue[0.9623475758499133, 1, 1], 
       Hue[0.9934629887045462, 1, 1], 
       Hue[0.9934629887045462, 1, 1], 
       Hue[0.02627405855083121, 1, 1], 
       Hue[0.02627405855083121, 1, 1], 
       Hue[0.06072941346367099, 1, 1], 
       Hue[0.06072941346367099, 1, 1], 
       Hue[0.09678062203177676, 1, 1], 
       Hue[0.09678062203177676, 1, 1], 
       Hue[0.1343824932010361, 1, 1], 
       Hue[0.1343824932010361, 1, 1], 
       Hue[0.17349119558818452`, 1, 1], 
       Hue[0.17349119558818452`, 1, 1], 
       Hue[0.21406617571740993`, 1, 1], 
       Hue[0.21406617571740993`, 1, 1], 
       Hue[0.2560679976886549, 1, 1], 
       Hue[0.2560679976886549, 1, 1], 
       Hue[0.299460457403954, 1, 1], 
       Hue[0.299460457403954, 1, 1], 
       Hue[0.3442092278042504, 1, 1], 
       Hue[0.3442092278042504, 1, 1], 
       Hue[0.3902814284807919, 1, 1], 
       Hue[0.3902814284807919, 1, 1], 
       Hue[0.4376464529812861, 1, 1], 
       Hue[0.4376464529812861, 1, 1], 
       Hue[0.486275510377512, 1, 1], 
       Hue[0.486275510377512, 1, 1], 
       Hue[0.5361404086810144, 1, 1], 
       Hue[0.5361404086810144, 1, 1], 
       Hue[0.5872156253981717, 1, 1], 
       Hue[0.5872156253981717, 1, 1], 
       Hue[0.63947675730222, 1, 1], 
       Hue[0.63947675730222, 1, 1], 
       Hue[0.6928994274576521, 1, 1], 
       Hue[0.6928994274576521, 1, 1], 
       Hue[0.7474627961633682, 1, 1], 
       Hue[0.7474627961633682, 1, 1], 
       Hue[0.8031448831692406, 1, 1], 
       Hue[0.8031448831692406, 1, 1], 
       Hue[0.8599277071610487, 1, 1], 
       Hue[0.8599277071610487, 1, 1], 
       Hue[0.9177905921948738, 1, 1], 
       Hue[0.9177905921948738, 1, 1], 
       Hue[0.9767162994358801, 1, 1], 
       Hue[0.9767162994358801, 1, 1], 
       Hue[0.03668769482659254, 1, 1], 
       Hue[0.03668769482659254, 1, 1], 
       Hue[0.09768870176319887, 1, 1], 
       Hue[0.09768870176319887, 1, 1], 
       Hue[0.15970390592623193`, 1, 1], 
       Hue[0.15970390592623193`, 1, 1], 
       Hue[0.22271848133218072`, 1, 1], 
       Hue[0.22271848133218072`, 1, 1], 
       Hue[0.2867176866955756, 1, 1], 
       Hue[0.2867176866955756, 1, 1], 
       Hue[0.3516896132531366, 1, 1], 
       Hue[0.3516896132531366, 1, 1], 
       Hue[0.41762027877249996`, 1, 1], 
       Hue[0.41762027877249996`, 1, 1], 
       Hue[0.4844980226954021, 1, 1], 
       Hue[0.4844980226954021, 1, 1], 
       Hue[0.5523092351240234, 1, 1], 
       Hue[0.5523092351240234, 1, 1], 
       Hue[0.621046141242471, 1, 1], 
       Hue[0.621046141242471, 1, 1], 
       Hue[0.6906954003728225, 1, 1], 
       Hue[0.6906954003728225, 1, 1], 
       Hue[0.7612464295655417, 1, 1], 
       Hue[0.7612464295655417, 1, 1], 
       Hue[0.8326892519615281, 1, 1], 
       Hue[0.8326892519615281, 1, 1], 
       Hue[0.9050154457110178, 1, 1], 
       Hue[0.9050154457110178, 1, 1], 
       Hue[0.9782137976593589, 1, 1], 
       Hue[0.9782137976593589, 1, 1], 
       Hue[0.05227788399332845, 1, 1], 
       Hue[0.05227788399332845, 1, 1], 
       Hue[0.1271986005150233, 1, 1], 
       Hue[0.1271986005150233, 1, 1], 
       Hue[0.2029642440043483, 1, 1], 
       Hue[0.2029642440043483, 1, 1], 
       Hue[0.27956900942770624`, 1, 1], 
       Hue[0.27956900942770624`, 1, 1], 
       Hue[0.3570070126386154, 1, 1], 
       Hue[0.3570070126386154, 1, 1], 
       Hue[0.43526876191562286`, 1, 1], 
       Hue[0.43526876191562286`, 1, 1], 
       Hue[0.5143483531822721, 1, 1], 
       Hue[0.5143483531822721, 1, 1], 
       Hue[0.5942364871856414, 1, 1], 
       Hue[0.5942364871856414, 1, 1], 
       Hue[0.6749261104680981, 1, 1], 
       Hue[0.6749261104680981, 1, 1], 
       Hue[0.7564165186521847, 1, 1], 
       Hue[0.7564165186521847, 1, 1], 
       Hue[0.8386941839035054, 1, 1], 
       Hue[0.8386941839035054, 1, 1], 
       Hue[0.9217502032467062, 1, 1], 
       Hue[0.9217502032467062, 1, 1], 
       Hue[0.005588047415432818, 1, 1], 
       Hue[0.005588047415432818, 1, 1], 
       Hue[0.09020190054450962, 1, 1], 
       Hue[0.09020190054450962, 1, 1], 
       Hue[0.17557830629088042`, 1, 1], 
       Hue[0.17557830629088042`, 1, 1], 
       Hue[0.2617155460289287, 1, 1], 
       Hue[0.2617155460289287, 1, 1], 
       Hue[0.3486030056287242, 1, 1], 
       Hue[0.3486030056287242, 1, 1], 
       Hue[0.43623852605324986`, 1, 1], 
       Hue[0.43623852605324986`, 1, 1], 
       Hue[0.5246243796516621, 1, 1], 
       Hue[0.5246243796516621, 1, 1], 
       Hue[0.6137467720443291, 1, 1], 
       Hue[0.6137467720443291, 1, 1], 
       Hue[0.7036113582715795, 1, 1], 
       Hue[0.7036113582715795, 1, 1], 
       Hue[0.7942020535195174, 1, 1], 
       Hue[0.7942020535195174, 1, 1], 
       Hue[0.8855024036439301, 1, 1], 
       Hue[0.8855024036439301, 1, 1], 
       Hue[0.9775292546983889, 1, 1], 
       Hue[0.9775292546983889, 1, 1], 
       Hue[0.07027539516154879, 1, 1], 
       Hue[0.07027539516154879, 1, 1], 
       Hue[0.16374124482826072`, 1, 1], 
       Hue[0.16374124482826072`, 1, 1], 
       Hue[0.25791700557200636`, 1, 1], 
       Hue[0.25791700557200636`, 1, 1], 
       Hue[0.3527640228073858, 1, 1], 
       Hue[0.3527640228073858, 1, 1], 
       Hue[0.4482987662581584, 1, 1], 
       Hue[0.4482987662581584, 1, 1], 
       Hue[0.5445531907493291, 1, 1], 
       Hue[0.5445531907493291, 1, 1], 
       Hue[0.6415090340163123, 1, 1], 
       Hue[0.6415090340163123, 1, 1], 
       Hue[0.739158911610914, 1, 1], 
       Hue[0.739158911610914, 1, 1], 
       Hue[0.8374834060735057, 1, 1], 
       Hue[0.8374834060735057, 1, 1], 
       Hue[0.9364240543729676, 1, 1], 
       Hue[0.9364240543729676, 1, 1], 
       Hue[0.03603829961410431, 1, 1], 
       Hue[0.03603829961410431, 1, 1], 
       Hue[0.13642177385132023`, 1, 1], 
       Hue[0.13642177385132023`, 1, 1], 
       Hue[0.23743312162973995`, 1, 1], 
       Hue[0.23743312162973995`, 1, 1], 
       Hue[0.33890501166499326`, 1, 1], 
       Hue[0.33890501166499326`, 1, 1], 
       Hue[0.4410449979957778, 1, 1], 
       Hue[0.4410449979957778, 1, 1], 
       Hue[0.5441512204674002, 1, 1], 
       Hue[0.5441512204674002, 1, 1], 
       Hue[0.6479683063190227, 1, 1], 
       Hue[0.6479683063190227, 1, 1], 
       Hue[0.7520934886159469, 1, 1], 
       Hue[0.7520934886159469, 1, 1], 
       Hue[0.8569136201029212, 1, 1], 
       Hue[0.8569136201029212, 1, 1], 
       Hue[0.9627047539340904, 1, 1], 
       Hue[0.9627047539340904, 1, 1], 
       Hue[0.06879089341509073, 1, 1], 
       Hue[0.06879089341509073, 1, 1], 
       Hue[0.17533461697758274`, 1, 1], 
       Hue[0.17533461697758274`, 1, 1], 
       Hue[0.28319618900543175`, 1, 1], 
       Hue[0.28319618900543175`, 1, 1], 
       Hue[0.39149295315047145`, 1, 1], 
       Hue[0.39149295315047145`, 1, 1], 
       Hue[0.4996425226407696, 1, 1], 
       Hue[0.4996425226407696, 1, 1], 
       Hue[0.6094061045208964, 1, 1], 
       Hue[0.6094061045208964, 1, 1], 
       Hue[0.7205987326917802, 1, 1], 
       Hue[0.7205987326917802, 1, 1], 
       Hue[0.8307909058918155, 1, 1], 
       Hue[0.8307909058918155, 1, 1], 
       Hue[0.9413621288318382, 1, 1], 
       Hue[0.9413621288318382, 1, 1], 
       Hue[0.055154985274585344`, 1, 1], 
       Hue[0.055154985274585344`, 1, 1], 
       Hue[0.16908344480096296`, 1, 1], 
       Hue[0.16908344480096296`, 1, 1], 
       Hue[0.27949834648938626`, 1, 1], 
       Hue[0.27949834648938626`, 1, 1], 
       Hue[0.39178904539044196`, 1, 1], 
       Hue[0.39178904539044196`, 1, 1], 
       Hue[0.5113599233497237, 1, 1], 
       Hue[0.5113599233497237, 1, 1], 
       Hue[0.6321214530795161, 1, 1], 
       Hue[0.6321214530795161, 1, 1], 
       Hue[0.7464805255537773, 1, 1], 
       Hue[0.7464805255537773, 1, 1], 
       Hue[0.8602620296701579, 1, 1], 
       Hue[0.8602620296701579, 1, 1], 
       Hue[0.9811501935614256, 1, 1]}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->{True, None},
    AxesOrigin->{0, 0},
    Epilog->
     LineBox[{{1, 3.4150615472307896`*^-7}, {2, 8.326392522476883*^-7}, {
       3, 2.859048395689338*^-7}, {4, 3.953536088470222*^-7}, {
       5, 6.351631059237657*^-7}, {6, 3.894539832031944*^-7}, {
       7, 6.586814227083131*^-7}, {8, 7.510154018765163*^-7}, {
       9, 4.227642051301257*^-7}, {10, 8.383393464990893*^-7}, {
       11, 1.032485409675302*^-6}, {12, 4.941444249232437*^-7}, {
       13, 1.0881791935708725`*^-6}, {14, 7.218617213599437*^-7}, {
       15, 1.0227181383285202`*^-6}, {16, 4.985029921358325*^-7}, {
       17, 6.258330298641342*^-7}, {18, 7.27637374271006*^-7}, {
       19, 7.708110456728936*^-7}, {20, 8.678819833415261*^-7}, {
       21, 1.0716205001774192`*^-6}, {22, 7.435967511779358*^-7}, {
       23, 6.641269062648287*^-7}, {24, 5.501954937648988*^-7}, {
       25, 6.658192669437551*^-7}, {26, 6.535545165656654*^-7}, {
       27, 4.6667900441271827`*^-7}, {28, 7.565506936805698*^-7}, {
       29, 1.3821364559304605`*^-7}, {30, 9.152074515538222*^-7}, {
       31, 7.424893027570349*^-7}, {32, 5.907814572336757*^-7}, {
       33, 8.83102574584025*^-7}, {34, 5.522986306869286*^-7}, {
       35, 8.459179833732249*^-7}, {36, 7.169306372930248*^-7}, {
       37, 8.44350804618063*^-7}, {38, 5.927428025714487*^-7}, {
       39, 1.1978202269684304`*^-6}, {40, 1.634963344743732*^-6}, {
       41, 1.7854314793513283`*^-6}, {42, 2.0073168838370047`*^-6}, {
       43, 1.5419112921733522`*^-6}, {44, 1.717365162019975*^-6}, {
       45, 2.0267033489479267`*^-6}, {46, 2.820588186792828*^-6}, {
       47, 3.0168859771533597`*^-6}, {48, 3.6208161270000935`*^-6}, {
       49, 4.517308708685305*^-6}, {50, 5.597084412290063*^-6}, {
       51, 6.904492751504067*^-6}, {52, 8.590829410804584*^-6}, {
       53, 0.000010364899646600366`}, {54, 0.000012674838565826955`}, {
       55, 0.000014677383465063058`}, {56, 0.000018530602034053334`}, {
       57, 0.000021879664582344023`}, {58, 0.00002740514740495939}, {
       59, 0.00003302039946077479}, {60, 0.00004007680766577907}, {
       61, 0.00004915721318423795}, {62, 0.00005967600343351285}, {
       63, 0.00007132251573900435}, {64, 0.00008510615624805744}, {
       65, 0.00010153862867520497`}, {66, 0.0001223953997284478}, {
       67, 0.0001456486479400899}, {68, 0.00017417020316132464`}, {
       69, 0.00020841130300822193`}, {70, 0.0002473682266457697}, {
       71, 0.00029365700045998444`}, {72, 0.00034651286008924374`}, {
       73, 0.00040818754207493107`}, {74, 0.0004814114763619019}, {
       75, 0.0005669946159744187}, {76, 0.0006666384195951068}, {
       77, 0.0007829210181043247}, {78, 0.0009147297986318929}, {
       79, 0.0010637780697234705`}, {80, 0.0012355887978018766`}, {
       81, 0.0014365669639994304`}, {82, 0.0016725545299133711`}, {
       83, 0.0019457368886742626`}, {84, 0.00225230897792296}, {
       85, 0.0025913788164238292`}, {86, 0.002966934515081171}, {
       87, 0.0033962259402968093`}, {88, 0.003898185716157752}, {
       89, 0.004486263571616521}, {90, 0.005155713635330152}, {
       91, 0.005886974507878243}, {92, 0.0066641588426744275`}, {
       93, 0.0075042385315737346`}, {94, 0.00846178480498481}, {
       95, 0.0096020983264912}, {96, 0.010946061400524376`}, {
       97, 0.01243978640152333}, {98, 0.013990437302815404`}, {
       99, 0.015550841999877631`}, {100, 0.01719956338118665}, {
       101, 0.019134177746136533`}, {102, 0.021536210833143093`}, {
       103, 0.024383921831166423`}, {104, 0.027418550004813615`}, {
       105, 0.030326530631184095`}, {106, 0.03302736870720255}, {
       107, 0.0358720965542906}, {108, 0.0395122163239013}, {
       109, 0.04437791884143114}, {110, 0.05017589150553378}, {
       111, 0.05600599810739158}, {112, 0.0610046778863704}, {
       113, 0.0650948966996837}, {114, 0.06939751214550856}, {
       115, 0.07580898835647554}, {116, 0.08553688954242283}, {
       117, 0.09785492383741799}, {118, 0.11050150808855051`}, {
       119, 0.12111840769427733`}, {120, 0.12859344456760172`}, {
       121, 0.134032829700865}, {122, 0.1411244888268904}, {
       123, 0.15498461252600032`}, {124, 0.17918326432910084`}, {
       125, 0.21357677996332655`}, {126, 0.255231688443048}, {
       127, 0.3004992585990988}, {128, 0.3462055832432251}, {
       129, 0.3900125249745804}, {130, 0.4304013996463122}, {
       131, 0.4665460145571011}, {132, 0.4981404183785623}, {
       133, 0.5252393663008192}, {134, 0.5481156545916485}, {
       135, 0.5671531175218409}, {136, 0.5827668510676232}, {
       137, 0.5953588515241416}, {138, 0.6052863815846707}, {
       139, 0.6128573082252979}, {140, 0.6183266041864458}, {
       141, 0.621901066307627}, {142, 0.6237432684263104}, {
       143, 0.6239906869910626}, {144, 0.6227496936719461}, {
       145, 0.6201109653312574}, {146, 0.6161508541851485}, {
       147, 0.6109385447498156}, {148, 0.6045357420982319}, {
       149, 0.5970040668252714}, {150, 0.5884030914674586}, {
       151, 0.5787907422470377}, {152, 0.5682299701611412}, {
       153, 0.5567823827512517}, {154, 0.5445112446496322}, {
       155, 0.5314860261248231}, {156, 0.517772145746944}, {
       157, 0.5034429193957675}, {158, 0.48856976394841967`}, {
       159, 0.47322240515913677`}, {160, 0.45747600274916994`}, {
       161, 0.4414014197716264}, {162, 0.4250710828279676}, {
       163, 0.408555695309991}, {164, 0.39192351837439604`}, {
       165, 0.3752446819665927}, {166, 0.3585804974155168}, {
       167, 0.3419960623351603}, {168, 0.325547006081104}, {
       169, 0.3092903168809777}, {170, 0.2932770082171458}, {
       171, 0.27755497601246615`}, {172, 0.2621669180765608}, {
       173, 0.2471516967282329}, {174, 0.23254580107335218`}, {
       175, 0.21837846809359532`}, {176, 0.20467642500234118`}, {
       177, 0.19146183794629634`}, {178, 0.17875167407500447`}, {
       179, 0.16656162614965095`}, {180, 0.1549010885025859}, {
       181, 0.14377693159047808`}, {182, 0.13319050721421524`}, {
       183, 0.12314405738145211`}, {184, 0.11363363298652249`}, {
       185, 0.10465212554227128`}, {186, 0.09619221982734708}, {
       187, 0.08824340229725057}, {188, 0.08079288682804961}, {
       189, 0.07382641517495493}, {190, 0.06732890296889077}, {
       191, 0.061281772559463626`}, {192, 0.05566798340887635}, {
       193, 0.05046959806798442}, {194, 0.045666359914472716`}, {
       195, 0.041238698412168484`}, {196, 0.03716681068531065}, {
       197, 0.033430316193146854`}, {198, 0.03001037692370112}, {
       199, 0.026886711722907297`}, {200, 0.024039528047743727`}, {
       201, 0.02145161294807903}, {202, 0.01910484112329695}, {
       203, 0.016980392726056056`}, {204, 0.01506175346484479}, {
       205, 0.013333090980926728`}, {206, 0.011778970847107622`}, {
       207, 0.010385550777830857`}, {208, 0.00913845984920063}, {
       209, 0.008024883664764117}, {210, 0.007032790109201361}, {
       211, 0.006150319492525345}, {212, 0.005367872366509762}, {
       213, 0.004675754637250524}, {214, 0.004064534653631685}, {
       215, 0.0035260121634039933`}, {216, 0.00305217400503307}, {
       217, 0.0026364810795332207`}, {218, 0.002273194257521188}, {
       219, 0.0019560774853568555`}, {220, 0.0016795982191183387`}, {
       221, 0.0014391215327172158`}, {222, 0.001230249416843563}, {
       223, 0.001049515424687823}, {224, 0.000893923723498234}, {
       225, 0.000759743329498674}, {226, 0.0006438014811699403}, {
       227, 0.000544558600428294}, {228, 0.0004604300661542428}, {
       229, 0.00038857954949786573`}, {230, 0.0003265822911236298}, {
       231, 0.00027385383194716796`}, {232, 0.00022966188700515662`}, {
       233, 0.0001919906738366209}, {234, 0.00015989314521156376`}, {
       235, 0.0001333344062333693}, {236, 0.00011098563653175951`}, {
       237, 0.00009175955142057913}, {238, 0.0000759780766423948}, {
       239, 0.00006310342740921583}, {240, 0.00005186949048771065}, {
       241, 0.000042382776042547845`}, {242, 0.00003500724731100024}, {
       243, 0.000028827735342542557`}, {244, 0.00002321197667845493}, {
       245, 0.000018756374384474497`}, {246, 0.00001564463200573973}, {
       247, 0.00001295156033672626}, {248, 0.000010217746008293889`}, {
       249, 7.9524150327209*^-6}, {250, 6.423517060508637*^-6}, {
       251, 5.140334711654943*^-6}}],
    Frame->True,
    FrameTicks->{{{1., 
        FormBox[
         RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
        FormBox[
         RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
        FormBox[
         RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
        FormBox[
         RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
        FormBox["0", TraditionalForm]}, {157.25, 
        FormBox["2.5`", TraditionalForm]}, {188.5, 
        FormBox["5", TraditionalForm]}, {219.75, 
        FormBox["7.5`", TraditionalForm]}, {251., 
        FormBox["10", TraditionalForm]}}, Automatic, None, None},
    PlotRange->{-0.1, 1.1},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], Automatic}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF13tcjvf/B/DLqZlfyERWzCUhzTEpKvVShM4nnQ9X58PdwXHYZvuE2FjO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     "], {
      {GrayLevel[0.5], EdgeForm[None], 
       GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ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         "]]]}, {{}, {}, {
        {Hue[0.03996625933509463, 1, 1], LineBox[{1, 2}]}, 
        {Hue[0.013716043603985754`, 1, 1], LineBox[{2, 3}]}, 
        {Hue[0.931478219924176, 1, 1], LineBox[{3, 4}]}, 
        {Hue[0.7933295525960278, 1, 1], LineBox[{4, 5}]}, 
        {Hue[0.671505196618513, 1, 1], LineBox[{5, 6}]}, 
        {Hue[0.48600936817024304`, 1, 1], LineBox[{6, 7}]}, 
        {Hue[0.4514066116504211, 1, 1], LineBox[{7, 8}]}, 
        {Hue[0.41317640143488066`, 1, 1], LineBox[{8, 9}]}, 
        {Hue[0.1728699714822034, 1, 1], LineBox[{9, 10}]}, 
        {Hue[0.16149973504533702`, 1, 1], LineBox[{10, 11}]}, 
        {Hue[0.014380992997808584`, 1, 1], LineBox[{11, 12}]}, 
        {Hue[0.8384753595018941, 1, 1], LineBox[{12, 13}]}, 
        {Hue[0.8680556074999903, 1, 1], LineBox[{13, 14}]}, 
        {Hue[0.7534222753972779, 1, 1], LineBox[{14, 15}]}, 
        {Hue[0.6098627904806089, 1, 1], LineBox[{15, 16}]}, 
        {Hue[0.6284046995572222, 1, 1], LineBox[{16, 17}]}, 
        {Hue[0.5287100058968992, 1, 1], LineBox[{17, 18}]}, 
        {Hue[0.3429159109541728, 1, 1], LineBox[{18, 19}]}, 
        {Hue[0.31298068804368645`, 1, 1], LineBox[{19, 20}]}, 
        {Hue[0.03115863318914752, 1, 1], LineBox[{20, 21}]}, 
        {Hue[0.8085133232198168, 1, 1], LineBox[{21, 22}]}, 
        {Hue[0.7785666480644399, 1, 1], LineBox[{22, 23}]}, 
        {Hue[0.8813213024321782, 1, 1], LineBox[{23, 24}]}, 
        {Hue[0.6921781949154409, 1, 1], LineBox[{24, 25}]}, 
        {Hue[0.6809606252489155, 1, 1], LineBox[{25, 26}]}, 
        {Hue[0.503465470130599, 1, 1], LineBox[{26, 27}]}, 
        {Hue[0.34795262488670103`, 1, 1], LineBox[{27, 28}]}, 
        {Hue[0.515525312724336, 1, 1], LineBox[{28, 29}]}, 
        {Hue[0.29980886210904617`, 1, 1], LineBox[{29, 30}]}, 
        {Hue[0.18072515446336535`, 1, 1], LineBox[{30, 31}]}, 
        {Hue[0.10297890719872621`, 1, 1], LineBox[{31, 32}]}, 
        {Hue[0.0023782259479396594`, 1, 1], LineBox[{32, 33}]}, 
        {Hue[0.914852704835432, 1, 1], LineBox[{33, 34}]}, 
        {Hue[0.8537396651102496, 1, 1], LineBox[{34, 35}]}, 
        {Hue[0.7448026676434731, 1, 1], LineBox[{35, 36}]}, 
        {Hue[0.6777817332505758, 1, 1], LineBox[{36, 37}]}, 
        {Hue[0.5860571244254931, 1, 1], LineBox[{37, 38}]}, 
        {Hue[0.4871012967944014, 1, 1], LineBox[{38, 39}]}, 
        {Hue[0.42687764741931494`, 1, 1], LineBox[{39, 40}]}, 
        {Hue[0.30671059341868573`, 1, 1], LineBox[{40, 41}]}, 
        {Hue[0.20522452942419225`, 1, 1], LineBox[{41, 42}]}, 
        {Hue[0.10111217553275076`, 1, 1], LineBox[{42, 43}]}, 
        {Hue[0.00422564743866865, 1, 1], LineBox[{43, 44}]}, 
        {Hue[0.9028465739827174, 1, 1], LineBox[{44, 45}]}, 
        {Hue[0.8018316233387496, 1, 1], LineBox[{45, 46}]}, 
        {Hue[0.6960993570726182, 1, 1], LineBox[{46, 47}]}, 
        {Hue[0.6037710348526477, 1, 1], LineBox[{47, 48}]}, 
        {Hue[0.5044937932940825, 1, 1], LineBox[{48, 49}]}, 
        {Hue[0.405547663514985, 1, 1], LineBox[{49, 50}]}, 
        {Hue[0.3059439158901264, 1, 1], LineBox[{50, 51}]}, 
        {Hue[0.20567206969598656`, 1, 1], LineBox[{51, 52}]}, 
        {Hue[0.10677120539249679`, 1, 1], LineBox[{52, 53}]}, 
        {Hue[0.007189722064902663, 1, 1], LineBox[{53, 54}]}, 
        {Hue[0.9072237591440739, 1, 1], LineBox[{54, 55}]}, 
        {Hue[0.8089701107334102, 1, 1], LineBox[{55, 56}]}, 
        {Hue[0.7109228185973671, 1, 1], LineBox[{56, 57}]}, 
        {Hue[0.6136604440188916, 1, 1], LineBox[{57, 58}]}, 
        {Hue[0.5156829678350188, 1, 1], LineBox[{58, 59}]}, 
        {Hue[0.418564420178921, 1, 1], LineBox[{59, 60}]}, 
        {Hue[0.32190746830302974`, 1, 1], LineBox[{60, 61}]}, 
        {Hue[0.2244393090199423, 1, 1], LineBox[{61, 62}]}, 
        {Hue[0.1274760151920486, 1, 1], LineBox[{62, 63}]}, 
        {Hue[0.030720452166069608`, 1, 1], LineBox[{63, 64}]}, 
        {Hue[0.9343594345237914, 1, 1], LineBox[{64, 65}]}, 
        {Hue[0.8384755979716438, 1, 1], LineBox[{65, 66}]}, 
        {Hue[0.7428164098352281, 1, 1], LineBox[{66, 67}]}, 
        {Hue[0.6473269281698956, 1, 1], LineBox[{67, 68}]}, 
        {Hue[0.5521603287832787, 1, 1], LineBox[{68, 69}]}, 
        {Hue[0.4571550579953574, 1, 1], LineBox[{69, 70}]}, 
        {Hue[0.36236152697481405`, 1, 1], LineBox[{70, 71}]}, 
        {Hue[0.2676557170894482, 1, 1], LineBox[{71, 72}]}, 
        {Hue[0.17322328832724745`, 1, 1], LineBox[{72, 73}]}, 
        {Hue[0.07913979099701683, 1, 1], LineBox[{73, 74}]}, 
        {Hue[0.985264744242385, 1, 1], LineBox[{74, 75}]}, 
        {Hue[0.8916843385002036, 1, 1], LineBox[{75, 76}]}, 
        {Hue[0.7982718205710239, 1, 1], LineBox[{76, 77}]}, 
        {Hue[0.7050922404061961, 1, 1], LineBox[{77, 78}]}, 
        {Hue[0.6121218907259338, 1, 1], LineBox[{78, 79}]}, 
        {Hue[0.5193234039088777, 1, 1], LineBox[{79, 80}]}, 
        {Hue[0.42677303315299064`, 1, 1], LineBox[{80, 81}]}, 
        {Hue[0.3344796073799292, 1, 1], LineBox[{81, 82}]}, 
        {Hue[0.24245262399604153`, 1, 1], LineBox[{82, 83}]}, 
        {Hue[0.15061363241763906`, 1, 1], LineBox[{83, 84}]}, 
        {Hue[0.05892932224611956, 1, 1], LineBox[{84, 85}]}, 
        {Hue[0.9673846491267268, 1, 1], LineBox[{85, 86}]}, 
        {Hue[0.8760413632447122, 1, 1], LineBox[{86, 87}]}, 
        {Hue[0.7849367791857623, 1, 1], LineBox[{87, 88}]}, 
        {Hue[0.6941299685528552, 1, 1], LineBox[{88, 89}]}, 
        {Hue[0.6035794489207947, 1, 1], LineBox[{89, 90}]}, 
        {Hue[0.5132120685336232, 1, 1], LineBox[{90, 91}]}, 
        {Hue[0.42295124545510054`, 1, 1], LineBox[{91, 92}]}, 
        {Hue[0.33277204168311014`, 1, 1], LineBox[{92, 93}]}, 
        {Hue[0.2427622335008183, 1, 1], LineBox[{93, 94}]}, 
        {Hue[0.15303822097131786`, 1, 1], LineBox[{94, 95}]}, 
        {Hue[0.06368438721457334, 1, 1], LineBox[{95, 96}]}, 
        {Hue[0.9746626610273306, 1, 1], LineBox[{96, 97}]}, 
        {Hue[0.8858311772112987, 1, 1], LineBox[{97, 98}]}, 
        {Hue[0.7970390083799735, 1, 1], LineBox[{98, 99}]}, 
        {Hue[0.7082714984768756, 1, 1], LineBox[{99, 100}]}, 
        {Hue[0.6196841994470936, 1, 1], LineBox[{100, 101}]}, 
        {Hue[0.5315215505517608, 1, 1], LineBox[{101, 102}]}, 
        {Hue[0.44394494742856366`, 1, 1], LineBox[{102, 103}]}, 
        {Hue[0.35688764850602844`, 1, 1], LineBox[{103, 104}]}, 
        {Hue[0.27009555087369436`, 1, 1], LineBox[{104, 105}]}, 
        {Hue[0.18331540575024727`, 1, 1], LineBox[{105, 106}]}, 
        {Hue[0.09654267138370616, 1, 1], LineBox[{106, 107}]}, 
        {Hue[0.010094811765891522`, 1, 1], LineBox[{107, 108}]}, 
        {Hue[0.9244607391604271, 1, 1], LineBox[{108, 109}]}, 
        {Hue[0.8399492499879724, 1, 1], LineBox[{109, 110}]}, 
        {Hue[0.7564654779058759, 1, 1], LineBox[{110, 111}]}, 
        {Hue[0.6735803972243876, 1, 1], LineBox[{111, 112}]}, 
        {Hue[0.5908683135133599, 1, 1], LineBox[{112, 113}]}, 
        {Hue[0.5083034172256362, 1, 1], LineBox[{113, 114}]}, 
        {Hue[0.4264514587862894, 1, 1], LineBox[{114, 115}]}, 
        {Hue[0.3462206847563066, 1, 1], LineBox[{115, 116}]}, 
        {Hue[0.26831235013189686`, 1, 1], LineBox[{116, 117}]}, 
        {Hue[0.19281231771676, 1, 1], LineBox[{117, 118}]}, 
        {Hue[0.11921971156371028`, 1, 1], LineBox[{118, 119}]}, 
        {Hue[0.0468063636139859, 1, 1], LineBox[{119, 120}]}, 
        {Hue[0.975109336835882, 1, 1], LineBox[{120, 121}]}, 
        {Hue[0.90435059561364, 1, 1], LineBox[{121, 122}]}, 
        {Hue[0.8355467810001385, 1, 1], LineBox[{122, 123}]}, 
        {Hue[0.7701334240767309, 1, 1], LineBox[{123, 124}]}, 
        {Hue[0.7093276744535297, 1, 1], LineBox[{124, 125}]}, 
        {Hue[0.6537249557854934, 1, 1], LineBox[{125, 126}]}, 
        {Hue[0.6033177014107454, 1, 1], LineBox[{126, 127}]}, 
        {Hue[0.5577557908523973, 1, 1], LineBox[{127, 128}]}, 
        {Hue[0.5165809575936386, 1, 1], LineBox[{128, 129}]}, 
        {Hue[0.47936904837080296`, 1, 1], LineBox[{129, 130}]}, 
        {Hue[0.44578257043933917`, 1, 1], LineBox[{130, 131}]}, 
        {Hue[0.41557954902743466`, 1, 1], LineBox[{131, 132}]}, 
        {Hue[0.3885948026145062, 1, 1], LineBox[{132, 133}]}, 
        {Hue[0.36472011513852276`, 1, 1], LineBox[{133, 134}]}, 
        {Hue[0.3438849218716845, 1, 1], LineBox[{134, 135}]}, 
        {Hue[0.32603800153585283`, 1, 1], LineBox[{135, 136}]}, 
        {Hue[0.3111376031158137, 1, 1], LineBox[{136, 137}]}, 
        {Hue[0.2991410864394776, 1, 1], LineBox[{137, 138}]}, 
        {Hue[0.2900012220807621, 1, 1], LineBox[{138, 139}]}, 
        {Hue[0.2836643128481897, 1, 1], LineBox[{139, 140}]}, 
        {Hue[0.2800696616453035, 1, 1], LineBox[{140, 141}]}, 
        {Hue[0.2791497176725532, 1, 1], LineBox[{141, 142}]}, 
        {Hue[0.2808316877825547, 1, 1], LineBox[{142, 143}]}, 
        {Hue[0.28503905898740767`, 1, 1], LineBox[{143, 144}]}, 
        {Hue[0.2916934185745679, 1, 1], LineBox[{144, 145}]}, 
        {Hue[0.30071520604920704`, 1, 1], LineBox[{145, 146}]}, 
        {Hue[0.31202487261304357`, 1, 1], LineBox[{146, 147}]}, 
        {Hue[0.3255441697020442, 1, 1], LineBox[{147, 148}]}, 
        {Hue[0.3411962371961555, 1, 1], LineBox[{148, 149}]}, 
        {Hue[0.3589074694876606, 1, 1], LineBox[{149, 150}]}, 
        {Hue[0.3786067400687169, 1, 1], LineBox[{150, 151}]}, 
        {Hue[0.4002252438807113, 1, 1], LineBox[{151, 152}]}, 
        {Hue[0.4236980743800631, 1, 1], LineBox[{152, 153}]}, 
        {Hue[0.4489635507509118, 1, 1], LineBox[{153, 154}]}, 
        {Hue[0.47596293667158696`, 1, 1], LineBox[{154, 155}]}, 
        {Hue[0.5046413799058753, 1, 1], LineBox[{155, 156}]}, 
        {Hue[0.5349458661619949, 1, 1], LineBox[{156, 157}]}, 
        {Hue[0.5668267075469183, 1, 1], LineBox[{157, 158}]}, 
        {Hue[0.6002383395664004, 1, 1], LineBox[{158, 159}]}, 
        {Hue[0.6351364442922158, 1, 1], LineBox[{159, 160}]}, 
        {Hue[0.6714794781732603, 1, 1], LineBox[{160, 161}]}, 
        {Hue[0.7092297851685113, 1, 1], LineBox[{161, 162}]}, 
        {Hue[0.7483498438618739, 1, 1], LineBox[{162, 163}]}, 
        {Hue[0.7888059216275258, 1, 1], LineBox[{163, 164}]}, 
        {Hue[0.8305658029377244, 1, 1], LineBox[{164, 165}]}, 
        {Hue[0.8735985589444489, 1, 1], LineBox[{165, 166}]}, 
        {Hue[0.9178756154733176, 1, 1], LineBox[{166, 167}]}, 
        {Hue[0.9633697925167194, 1, 1], LineBox[{167, 168}]}, 
        {Hue[0.010055769432912077`, 1, 1], LineBox[{168, 169}]}, 
        {Hue[0.05790905682561934, 1, 1], LineBox[{169, 170}]}, 
        {Hue[0.1069068241608519, 1, 1], LineBox[{170, 171}]}, 
        {Hue[0.1570279699615826, 1, 1], LineBox[{171, 172}]}, 
        {Hue[0.20825197197012807`, 1, 1], LineBox[{172, 173}]}, 
        {Hue[0.2605589059274433, 1, 1], LineBox[{173, 174}]}, 
        {Hue[0.31393019394989297`, 1, 1], LineBox[{174, 175}]}, 
        {Hue[0.36834904422164405`, 1, 1], LineBox[{175, 176}]}, 
        {Hue[0.4237981000610914, 1, 1], LineBox[{176, 177}]}, 
        {Hue[0.480262570763982, 1, 1], LineBox[{177, 178}]}, 
        {Hue[0.5377260144795512, 1, 1], LineBox[{178, 179}]}, 
        {Hue[0.5961750384728735, 1, 1], LineBox[{179, 180}]}, 
        {Hue[0.6555958067322936, 1, 1], LineBox[{180, 181}]}, 
        {Hue[0.7159748673000743, 1, 1], LineBox[{181, 182}]}, 
        {Hue[0.7773000790053467, 1, 1], LineBox[{182, 183}]}, 
        {Hue[0.8395593248056266, 1, 1], LineBox[{183, 184}]}, 
        {Hue[0.9027412896333733, 1, 1], LineBox[{184, 185}]}, 
        {Hue[0.9668345230053836, 1, 1], LineBox[{185, 186}]}, 
        {Hue[0.03182905417736379, 1, 1], LineBox[{186, 187}]}, 
        {Hue[0.09771495862869105, 1, 1], LineBox[{187, 188}]}, 
        {Hue[0.1644804683063499, 1, 1], LineBox[{188, 189}]}, 
        {Hue[0.232118474204226, 1, 1], LineBox[{189, 190}]}, 
        {Hue[0.3006190916938448, 1, 1], LineBox[{190, 191}]}, 
        {Hue[0.36997412177509265`, 1, 1], LineBox[{191, 192}]}, 
        {Hue[0.44017466975917585`, 1, 1], LineBox[{192, 193}]}, 
        {Hue[0.5112124994864206, 1, 1], LineBox[{193, 194}]}, 
        {Hue[0.5830804527288551, 1, 1], LineBox[{194, 195}]}, 
        {Hue[0.6557702517243329, 1, 1], LineBox[{195, 196}]}, 
        {Hue[0.7292765966264289, 1, 1], LineBox[{196, 197}]}, 
        {Hue[0.8035895255914776, 1, 1], LineBox[{197, 198}]}, 
        {Hue[0.878701965966677, 1, 1], LineBox[{198, 199}]}, 
        {Hue[0.95461010302941, 1, 1], LineBox[{199, 200}]}, 
        {Hue[0.03130808683880092, 1, 1], LineBox[{200, 201}]}, 
        {Hue[0.10878523846407365`, 1, 1], LineBox[{201, 202}]}, 
        {Hue[0.187036929367527, 1, 1], LineBox[{202, 203}]}, 
        {Hue[0.26606114164798506`, 1, 1], LineBox[{203, 204}]}, 
        {Hue[0.3458479697912301, 1, 1], LineBox[{204, 205}]}, 
        {Hue[0.42639267303198874`, 1, 1], LineBox[{205, 206}]}, 
        {Hue[0.5076933544748576, 1, 1], LineBox[{206, 207}]}, 
        {Hue[0.5897379334740892, 1, 1], LineBox[{207, 208}]}, 
        {Hue[0.6725241340879914, 1, 1], LineBox[{208, 209}]}, 
        {Hue[0.7560528613554944, 1, 1], LineBox[{209, 210}]}, 
        {Hue[0.8403144520020706, 1, 1], LineBox[{210, 211}]}, 
        {Hue[0.9253050137763409, 1, 1], LineBox[{211, 212}]}, 
        {Hue[0.011017846082229929`, 1, 1], LineBox[{212, 213}]}, 
        {Hue[0.09744381217502465, 1, 1], LineBox[{213, 214}]}, 
        {Hue[0.18458837476421924`, 1, 1], LineBox[{214, 215}]}, 
        {Hue[0.2724509652327783, 1, 1], LineBox[{215, 216}]}, 
        {Hue[0.3610067027049437, 1, 1], LineBox[{216, 217}]}, 
        {Hue[0.4502651974747799, 1, 1], LineBox[{217, 218}]}, 
        {Hue[0.5402360248028113, 1, 1], LineBox[{218, 219}]}, 
        {Hue[0.6308979533152481, 1, 1], LineBox[{219, 220}]}, 
        {Hue[0.7222476450207425, 1, 1], LineBox[{220, 221}]}, 
        {Hue[0.8142704033445503, 1, 1], LineBox[{221, 222}]}, 
        {Hue[0.9069734465940373, 1, 1], LineBox[{222, 223}]}, 
        {Hue[0.0003751091215660667, 1, 1], LineBox[{223, 224}]}, 
        {Hue[0.09444859931788216, 1, 1], LineBox[{224, 225}]}, 
        {Hue[0.18919701920563461`, 1, 1], LineBox[{225, 226}]}, 
        {Hue[0.2846160837458422, 1, 1], LineBox[{226, 227}]}, 
        {Hue[0.3806721820941903, 1, 1], LineBox[{227, 228}]}, 
        {Hue[0.4773855325816415, 1, 1], LineBox[{228, 229}]}, 
        {Hue[0.5747717077904234, 1, 1], LineBox[{229, 230}]}, 
        {Hue[0.6728222034045579, 1, 1], LineBox[{230, 231}]}, 
        {Hue[0.7715478143526769, 1, 1], LineBox[{231, 232}]}, 
        {Hue[0.8708970154927892, 1, 1], LineBox[{232, 233}]}, 
        {Hue[0.9708163161382002, 1, 1], LineBox[{233, 234}]}, 
        {Hue[0.07139599464877043, 1, 1], LineBox[{234, 235}]}, 
        {Hue[0.172692594788118, 1, 1], LineBox[{235, 236}]}, 
        {Hue[0.2746347559316637, 1, 1], LineBox[{236, 237}]}, 
        {Hue[0.37720554526915917`, 1, 1], LineBox[{237, 238}]}, 
        {Hue[0.4803521818452014, 1, 1], LineBox[{238, 239}]}, 
        {Hue[0.5839602695737227, 1, 1], LineBox[{239, 240}]}, 
        {Hue[0.6881708094172927, 1, 1], LineBox[{240, 241}]}, 
        {Hue[0.7932710636593592, 1, 1], LineBox[{241, 242}]}, 
        {Hue[0.8990904369082359, 1, 1], LineBox[{242, 243}]}, 
        {Hue[0.005048224852851583, 1, 1], LineBox[{243, 244}]}, 
        {Hue[0.1114275878230381, 1, 1], LineBox[{244, 245}]}, 
        {Hue[0.2193610331407116, 1, 1], LineBox[{245, 246}]}, 
        {Hue[0.3283303659856307, 1, 1], LineBox[{246, 247}]}, 
        {Hue[0.4367141006325967, 1, 1], LineBox[{247, 248}]}, 
        {Hue[0.5453967024411587, 1, 1], LineBox[{248, 249}]}, 
        {Hue[0.6557591931182715, 1, 1], LineBox[{249, 250}]}, 
        {Hue[0.7662084995133627, 1, 1], LineBox[{250, 251}]}}}},
     VertexColors->{
       Hue[0.03996625933509463, 1, 1], 
       Hue[0.013716043603985754`, 1, 1], 
       Hue[0.931478219924176, 1, 1], 
       Hue[0.7933295525960278, 1, 1], 
       Hue[0.671505196618513, 1, 1], 
       Hue[0.48600936817024304`, 1, 1], 
       Hue[0.4514066116504211, 1, 1], 
       Hue[0.41317640143488066`, 1, 1], 
       Hue[0.1728699714822034, 1, 1], 
       Hue[0.16149973504533702`, 1, 1], 
       Hue[0.014380992997808584`, 1, 1], 
       Hue[0.8384753595018941, 1, 1], 
       Hue[0.8680556074999903, 1, 1], 
       Hue[0.7534222753972779, 1, 1], 
       Hue[0.6098627904806089, 1, 1], 
       Hue[0.6284046995572222, 1, 1], 
       Hue[0.5287100058968992, 1, 1], 
       Hue[0.3429159109541728, 1, 1], 
       Hue[0.31298068804368645`, 1, 1], 
       Hue[0.03115863318914752, 1, 1], 
       Hue[0.8085133232198168, 1, 1], 
       Hue[0.7785666480644399, 1, 1], 
       Hue[0.8813213024321782, 1, 1], 
       Hue[0.6921781949154409, 1, 1], 
       Hue[0.6809606252489155, 1, 1], 
       Hue[0.503465470130599, 1, 1], 
       Hue[0.34795262488670103`, 1, 1], 
       Hue[0.515525312724336, 1, 1], 
       Hue[0.29980886210904617`, 1, 1], 
       Hue[0.18072515446336535`, 1, 1], 
       Hue[0.10297890719872621`, 1, 1], 
       Hue[0.0023782259479396594`, 1, 1], 
       Hue[0.914852704835432, 1, 1], 
       Hue[0.8537396651102496, 1, 1], 
       Hue[0.7448026676434731, 1, 1], 
       Hue[0.6777817332505758, 1, 1], 
       Hue[0.5860571244254931, 1, 1], 
       Hue[0.4871012967944014, 1, 1], 
       Hue[0.42687764741931494`, 1, 1], 
       Hue[0.30671059341868573`, 1, 1], 
       Hue[0.20522452942419225`, 1, 1], 
       Hue[0.10111217553275076`, 1, 1], 
       Hue[0.00422564743866865, 1, 1], 
       Hue[0.9028465739827174, 1, 1], 
       Hue[0.8018316233387496, 1, 1], 
       Hue[0.6960993570726182, 1, 1], 
       Hue[0.6037710348526477, 1, 1], 
       Hue[0.5044937932940825, 1, 1], 
       Hue[0.405547663514985, 1, 1], 
       Hue[0.3059439158901264, 1, 1], 
       Hue[0.20567206969598656`, 1, 1], 
       Hue[0.10677120539249679`, 1, 1], 
       Hue[0.007189722064902663, 1, 1], 
       Hue[0.9072237591440739, 1, 1], 
       Hue[0.8089701107334102, 1, 1], 
       Hue[0.7109228185973671, 1, 1], 
       Hue[0.6136604440188916, 1, 1], 
       Hue[0.5156829678350188, 1, 1], 
       Hue[0.418564420178921, 1, 1], 
       Hue[0.32190746830302974`, 1, 1], 
       Hue[0.2244393090199423, 1, 1], 
       Hue[0.1274760151920486, 1, 1], 
       Hue[0.030720452166069608`, 1, 1], 
       Hue[0.9343594345237914, 1, 1], 
       Hue[0.8384755979716438, 1, 1], 
       Hue[0.7428164098352281, 1, 1], 
       Hue[0.6473269281698956, 1, 1], 
       Hue[0.5521603287832787, 1, 1], 
       Hue[0.4571550579953574, 1, 1], 
       Hue[0.36236152697481405`, 1, 1], 
       Hue[0.2676557170894482, 1, 1], 
       Hue[0.17322328832724745`, 1, 1], 
       Hue[0.07913979099701683, 1, 1], 
       Hue[0.985264744242385, 1, 1], 
       Hue[0.8916843385002036, 1, 1], 
       Hue[0.7982718205710239, 1, 1], 
       Hue[0.7050922404061961, 1, 1], 
       Hue[0.6121218907259338, 1, 1], 
       Hue[0.5193234039088777, 1, 1], 
       Hue[0.42677303315299064`, 1, 1], 
       Hue[0.3344796073799292, 1, 1], 
       Hue[0.24245262399604153`, 1, 1], 
       Hue[0.15061363241763906`, 1, 1], 
       Hue[0.05892932224611956, 1, 1], 
       Hue[0.9673846491267268, 1, 1], 
       Hue[0.8760413632447122, 1, 1], 
       Hue[0.7849367791857623, 1, 1], 
       Hue[0.6941299685528552, 1, 1], 
       Hue[0.6035794489207947, 1, 1], 
       Hue[0.5132120685336232, 1, 1], 
       Hue[0.42295124545510054`, 1, 1], 
       Hue[0.33277204168311014`, 1, 1], 
       Hue[0.2427622335008183, 1, 1], 
       Hue[0.15303822097131786`, 1, 1], 
       Hue[0.06368438721457334, 1, 1], 
       Hue[0.9746626610273306, 1, 1], 
       Hue[0.8858311772112987, 1, 1], 
       Hue[0.7970390083799735, 1, 1], 
       Hue[0.7082714984768756, 1, 1], 
       Hue[0.6196841994470936, 1, 1], 
       Hue[0.5315215505517608, 1, 1], 
       Hue[0.44394494742856366`, 1, 1], 
       Hue[0.35688764850602844`, 1, 1], 
       Hue[0.27009555087369436`, 1, 1], 
       Hue[0.18331540575024727`, 1, 1], 
       Hue[0.09654267138370616, 1, 1], 
       Hue[0.010094811765891522`, 1, 1], 
       Hue[0.9244607391604271, 1, 1], 
       Hue[0.8399492499879724, 1, 1], 
       Hue[0.7564654779058759, 1, 1], 
       Hue[0.6735803972243876, 1, 1], 
       Hue[0.5908683135133599, 1, 1], 
       Hue[0.5083034172256362, 1, 1], 
       Hue[0.4264514587862894, 1, 1], 
       Hue[0.3462206847563066, 1, 1], 
       Hue[0.26831235013189686`, 1, 1], 
       Hue[0.19281231771676, 1, 1], 
       Hue[0.11921971156371028`, 1, 1], 
       Hue[0.0468063636139859, 1, 1], 
       Hue[0.975109336835882, 1, 1], 
       Hue[0.90435059561364, 1, 1], 
       Hue[0.8355467810001385, 1, 1], 
       Hue[0.7701334240767309, 1, 1], 
       Hue[0.7093276744535297, 1, 1], 
       Hue[0.6537249557854934, 1, 1], 
       Hue[0.6033177014107454, 1, 1], 
       Hue[0.5577557908523973, 1, 1], 
       Hue[0.5165809575936386, 1, 1], 
       Hue[0.47936904837080296`, 1, 1], 
       Hue[0.44578257043933917`, 1, 1], 
       Hue[0.41557954902743466`, 1, 1], 
       Hue[0.3885948026145062, 1, 1], 
       Hue[0.36472011513852276`, 1, 1], 
       Hue[0.3438849218716845, 1, 1], 
       Hue[0.32603800153585283`, 1, 1], 
       Hue[0.3111376031158137, 1, 1], 
       Hue[0.2991410864394776, 1, 1], 
       Hue[0.2900012220807621, 1, 1], 
       Hue[0.2836643128481897, 1, 1], 
       Hue[0.2800696616453035, 1, 1], 
       Hue[0.2791497176725532, 1, 1], 
       Hue[0.2808316877825547, 1, 1], 
       Hue[0.28503905898740767`, 1, 1], 
       Hue[0.2916934185745679, 1, 1], 
       Hue[0.30071520604920704`, 1, 1], 
       Hue[0.31202487261304357`, 1, 1], 
       Hue[0.3255441697020442, 1, 1], 
       Hue[0.3411962371961555, 1, 1], 
       Hue[0.3589074694876606, 1, 1], 
       Hue[0.3786067400687169, 1, 1], 
       Hue[0.4002252438807113, 1, 1], 
       Hue[0.4236980743800631, 1, 1], 
       Hue[0.4489635507509118, 1, 1], 
       Hue[0.47596293667158696`, 1, 1], 
       Hue[0.5046413799058753, 1, 1], 
       Hue[0.5349458661619949, 1, 1], 
       Hue[0.5668267075469183, 1, 1], 
       Hue[0.6002383395664004, 1, 1], 
       Hue[0.6351364442922158, 1, 1], 
       Hue[0.6714794781732603, 1, 1], 
       Hue[0.7092297851685113, 1, 1], 
       Hue[0.7483498438618739, 1, 1], 
       Hue[0.7888059216275258, 1, 1], 
       Hue[0.8305658029377244, 1, 1], 
       Hue[0.8735985589444489, 1, 1], 
       Hue[0.9178756154733176, 1, 1], 
       Hue[0.9633697925167194, 1, 1], 
       Hue[0.010055769432912077`, 1, 1], 
       Hue[0.05790905682561934, 1, 1], 
       Hue[0.1069068241608519, 1, 1], 
       Hue[0.1570279699615826, 1, 1], 
       Hue[0.20825197197012807`, 1, 1], 
       Hue[0.2605589059274433, 1, 1], 
       Hue[0.31393019394989297`, 1, 1], 
       Hue[0.36834904422164405`, 1, 1], 
       Hue[0.4237981000610914, 1, 1], 
       Hue[0.480262570763982, 1, 1], 
       Hue[0.5377260144795512, 1, 1], 
       Hue[0.5961750384728735, 1, 1], 
       Hue[0.6555958067322936, 1, 1], 
       Hue[0.7159748673000743, 1, 1], 
       Hue[0.7773000790053467, 1, 1], 
       Hue[0.8395593248056266, 1, 1], 
       Hue[0.9027412896333733, 1, 1], 
       Hue[0.9668345230053836, 1, 1], 
       Hue[0.03182905417736379, 1, 1], 
       Hue[0.09771495862869105, 1, 1], 
       Hue[0.1644804683063499, 1, 1], 
       Hue[0.232118474204226, 1, 1], 
       Hue[0.3006190916938448, 1, 1], 
       Hue[0.36997412177509265`, 1, 1], 
       Hue[0.44017466975917585`, 1, 1], 
       Hue[0.5112124994864206, 1, 1], 
       Hue[0.5830804527288551, 1, 1], 
       Hue[0.6557702517243329, 1, 1], 
       Hue[0.7292765966264289, 1, 1], 
       Hue[0.8035895255914776, 1, 1], 
       Hue[0.878701965966677, 1, 1], 
       Hue[0.95461010302941, 1, 1], 
       Hue[0.03130808683880092, 1, 1], 
       Hue[0.10878523846407365`, 1, 1], 
       Hue[0.187036929367527, 1, 1], 
       Hue[0.26606114164798506`, 1, 1], 
       Hue[0.3458479697912301, 1, 1], 
       Hue[0.42639267303198874`, 1, 1], 
       Hue[0.5076933544748576, 1, 1], 
       Hue[0.5897379334740892, 1, 1], 
       Hue[0.6725241340879914, 1, 1], 
       Hue[0.7560528613554944, 1, 1], 
       Hue[0.8403144520020706, 1, 1], 
       Hue[0.9253050137763409, 1, 1], 
       Hue[0.011017846082229929`, 1, 1], 
       Hue[0.09744381217502465, 1, 1], 
       Hue[0.18458837476421924`, 1, 1], 
       Hue[0.2724509652327783, 1, 1], 
       Hue[0.3610067027049437, 1, 1], 
       Hue[0.4502651974747799, 1, 1], 
       Hue[0.5402360248028113, 1, 1], 
       Hue[0.6308979533152481, 1, 1], 
       Hue[0.7222476450207425, 1, 1], 
       Hue[0.8142704033445503, 1, 1], 
       Hue[0.9069734465940373, 1, 1], 
       Hue[0.0003751091215660667, 1, 1], 
       Hue[0.09444859931788216, 1, 1], 
       Hue[0.18919701920563461`, 1, 1], 
       Hue[0.2846160837458422, 1, 1], 
       Hue[0.3806721820941903, 1, 1], 
       Hue[0.4773855325816415, 1, 1], 
       Hue[0.5747717077904234, 1, 1], 
       Hue[0.6728222034045579, 1, 1], 
       Hue[0.7715478143526769, 1, 1], 
       Hue[0.8708970154927892, 1, 1], 
       Hue[0.9708163161382002, 1, 1], 
       Hue[0.07139599464877043, 1, 1], 
       Hue[0.172692594788118, 1, 1], 
       Hue[0.2746347559316637, 1, 1], 
       Hue[0.37720554526915917`, 1, 1], 
       Hue[0.4803521818452014, 1, 1], 
       Hue[0.5839602695737227, 1, 1], 
       Hue[0.6881708094172927, 1, 1], 
       Hue[0.7932710636593592, 1, 1], 
       Hue[0.8990904369082359, 1, 1], 
       Hue[0.005048224852851583, 1, 1], 
       Hue[0.1114275878230381, 1, 1], 
       Hue[0.2193610331407116, 1, 1], 
       Hue[0.3283303659856307, 1, 1], 
       Hue[0.4367141006325967, 1, 1], 
       Hue[0.5453967024411587, 1, 1], 
       Hue[0.6557591931182715, 1, 1], 
       Hue[0.7662084995133627, 1, 1], 
       Hue[0.8765508443172676, 1, 1], 
       Hue[0.03996625933509463, 1, 1], 
       Hue[0.013716043603985754`, 1, 1], 
       Hue[0.013716043603985754`, 1, 1], 
       Hue[0.931478219924176, 1, 1], 
       Hue[0.931478219924176, 1, 1], 
       Hue[0.7933295525960278, 1, 1], 
       Hue[0.7933295525960278, 1, 1], 
       Hue[0.671505196618513, 1, 1], 
       Hue[0.671505196618513, 1, 1], 
       Hue[0.48600936817024304`, 1, 1], 
       Hue[0.48600936817024304`, 1, 1], 
       Hue[0.4514066116504211, 1, 1], 
       Hue[0.4514066116504211, 1, 1], 
       Hue[0.41317640143488066`, 1, 1], 
       Hue[0.41317640143488066`, 1, 1], 
       Hue[0.1728699714822034, 1, 1], 
       Hue[0.1728699714822034, 1, 1], 
       Hue[0.16149973504533702`, 1, 1], 
       Hue[0.16149973504533702`, 1, 1], 
       Hue[0.014380992997808584`, 1, 1], 
       Hue[0.014380992997808584`, 1, 1], 
       Hue[0.8384753595018941, 1, 1], 
       Hue[0.8384753595018941, 1, 1], 
       Hue[0.8680556074999903, 1, 1], 
       Hue[0.8680556074999903, 1, 1], 
       Hue[0.7534222753972779, 1, 1], 
       Hue[0.7534222753972779, 1, 1], 
       Hue[0.6098627904806089, 1, 1], 
       Hue[0.6098627904806089, 1, 1], 
       Hue[0.6284046995572222, 1, 1], 
       Hue[0.6284046995572222, 1, 1], 
       Hue[0.5287100058968992, 1, 1], 
       Hue[0.5287100058968992, 1, 1], 
       Hue[0.3429159109541728, 1, 1], 
       Hue[0.3429159109541728, 1, 1], 
       Hue[0.31298068804368645`, 1, 1], 
       Hue[0.31298068804368645`, 1, 1], 
       Hue[0.03115863318914752, 1, 1], 
       Hue[0.03115863318914752, 1, 1], 
       Hue[0.8085133232198168, 1, 1], 
       Hue[0.8085133232198168, 1, 1], 
       Hue[0.7785666480644399, 1, 1], 
       Hue[0.7785666480644399, 1, 1], 
       Hue[0.8813213024321782, 1, 1], 
       Hue[0.8813213024321782, 1, 1], 
       Hue[0.6921781949154409, 1, 1], 
       Hue[0.6921781949154409, 1, 1], 
       Hue[0.6809606252489155, 1, 1], 
       Hue[0.6809606252489155, 1, 1], 
       Hue[0.503465470130599, 1, 1], 
       Hue[0.503465470130599, 1, 1], 
       Hue[0.34795262488670103`, 1, 1], 
       Hue[0.34795262488670103`, 1, 1], 
       Hue[0.515525312724336, 1, 1], 
       Hue[0.515525312724336, 1, 1], 
       Hue[0.29980886210904617`, 1, 1], 
       Hue[0.29980886210904617`, 1, 1], 
       Hue[0.18072515446336535`, 1, 1], 
       Hue[0.18072515446336535`, 1, 1], 
       Hue[0.10297890719872621`, 1, 1], 
       Hue[0.10297890719872621`, 1, 1], 
       Hue[0.0023782259479396594`, 1, 1], 
       Hue[0.0023782259479396594`, 1, 1], 
       Hue[0.914852704835432, 1, 1], 
       Hue[0.914852704835432, 1, 1], 
       Hue[0.8537396651102496, 1, 1], 
       Hue[0.8537396651102496, 1, 1], 
       Hue[0.7448026676434731, 1, 1], 
       Hue[0.7448026676434731, 1, 1], 
       Hue[0.6777817332505758, 1, 1], 
       Hue[0.6777817332505758, 1, 1], 
       Hue[0.5860571244254931, 1, 1], 
       Hue[0.5860571244254931, 1, 1], 
       Hue[0.4871012967944014, 1, 1], 
       Hue[0.4871012967944014, 1, 1], 
       Hue[0.42687764741931494`, 1, 1], 
       Hue[0.42687764741931494`, 1, 1], 
       Hue[0.30671059341868573`, 1, 1], 
       Hue[0.30671059341868573`, 1, 1], 
       Hue[0.20522452942419225`, 1, 1], 
       Hue[0.20522452942419225`, 1, 1], 
       Hue[0.10111217553275076`, 1, 1], 
       Hue[0.10111217553275076`, 1, 1], 
       Hue[0.00422564743866865, 1, 1], 
       Hue[0.00422564743866865, 1, 1], 
       Hue[0.9028465739827174, 1, 1], 
       Hue[0.9028465739827174, 1, 1], 
       Hue[0.8018316233387496, 1, 1], 
       Hue[0.8018316233387496, 1, 1], 
       Hue[0.6960993570726182, 1, 1], 
       Hue[0.6960993570726182, 1, 1], 
       Hue[0.6037710348526477, 1, 1], 
       Hue[0.6037710348526477, 1, 1], 
       Hue[0.5044937932940825, 1, 1], 
       Hue[0.5044937932940825, 1, 1], 
       Hue[0.405547663514985, 1, 1], 
       Hue[0.405547663514985, 1, 1], 
       Hue[0.3059439158901264, 1, 1], 
       Hue[0.3059439158901264, 1, 1], 
       Hue[0.20567206969598656`, 1, 1], 
       Hue[0.20567206969598656`, 1, 1], 
       Hue[0.10677120539249679`, 1, 1], 
       Hue[0.10677120539249679`, 1, 1], 
       Hue[0.007189722064902663, 1, 1], 
       Hue[0.007189722064902663, 1, 1], 
       Hue[0.9072237591440739, 1, 1], 
       Hue[0.9072237591440739, 1, 1], 
       Hue[0.8089701107334102, 1, 1], 
       Hue[0.8089701107334102, 1, 1], 
       Hue[0.7109228185973671, 1, 1], 
       Hue[0.7109228185973671, 1, 1], 
       Hue[0.6136604440188916, 1, 1], 
       Hue[0.6136604440188916, 1, 1], 
       Hue[0.5156829678350188, 1, 1], 
       Hue[0.5156829678350188, 1, 1], 
       Hue[0.418564420178921, 1, 1], 
       Hue[0.418564420178921, 1, 1], 
       Hue[0.32190746830302974`, 1, 1], 
       Hue[0.32190746830302974`, 1, 1], 
       Hue[0.2244393090199423, 1, 1], 
       Hue[0.2244393090199423, 1, 1], 
       Hue[0.1274760151920486, 1, 1], 
       Hue[0.1274760151920486, 1, 1], 
       Hue[0.030720452166069608`, 1, 1], 
       Hue[0.030720452166069608`, 1, 1], 
       Hue[0.9343594345237914, 1, 1], 
       Hue[0.9343594345237914, 1, 1], 
       Hue[0.8384755979716438, 1, 1], 
       Hue[0.8384755979716438, 1, 1], 
       Hue[0.7428164098352281, 1, 1], 
       Hue[0.7428164098352281, 1, 1], 
       Hue[0.6473269281698956, 1, 1], 
       Hue[0.6473269281698956, 1, 1], 
       Hue[0.5521603287832787, 1, 1], 
       Hue[0.5521603287832787, 1, 1], 
       Hue[0.4571550579953574, 1, 1], 
       Hue[0.4571550579953574, 1, 1], 
       Hue[0.36236152697481405`, 1, 1], 
       Hue[0.36236152697481405`, 1, 1], 
       Hue[0.2676557170894482, 1, 1], 
       Hue[0.2676557170894482, 1, 1], 
       Hue[0.17322328832724745`, 1, 1], 
       Hue[0.17322328832724745`, 1, 1], 
       Hue[0.07913979099701683, 1, 1], 
       Hue[0.07913979099701683, 1, 1], 
       Hue[0.985264744242385, 1, 1], 
       Hue[0.985264744242385, 1, 1], 
       Hue[0.8916843385002036, 1, 1], 
       Hue[0.8916843385002036, 1, 1], 
       Hue[0.7982718205710239, 1, 1], 
       Hue[0.7982718205710239, 1, 1], 
       Hue[0.7050922404061961, 1, 1], 
       Hue[0.7050922404061961, 1, 1], 
       Hue[0.6121218907259338, 1, 1], 
       Hue[0.6121218907259338, 1, 1], 
       Hue[0.5193234039088777, 1, 1], 
       Hue[0.5193234039088777, 1, 1], 
       Hue[0.42677303315299064`, 1, 1], 
       Hue[0.42677303315299064`, 1, 1], 
       Hue[0.3344796073799292, 1, 1], 
       Hue[0.3344796073799292, 1, 1], 
       Hue[0.24245262399604153`, 1, 1], 
       Hue[0.24245262399604153`, 1, 1], 
       Hue[0.15061363241763906`, 1, 1], 
       Hue[0.15061363241763906`, 1, 1], 
       Hue[0.05892932224611956, 1, 1], 
       Hue[0.05892932224611956, 1, 1], 
       Hue[0.9673846491267268, 1, 1], 
       Hue[0.9673846491267268, 1, 1], 
       Hue[0.8760413632447122, 1, 1], 
       Hue[0.8760413632447122, 1, 1], 
       Hue[0.7849367791857623, 1, 1], 
       Hue[0.7849367791857623, 1, 1], 
       Hue[0.6941299685528552, 1, 1], 
       Hue[0.6941299685528552, 1, 1], 
       Hue[0.6035794489207947, 1, 1], 
       Hue[0.6035794489207947, 1, 1], 
       Hue[0.5132120685336232, 1, 1], 
       Hue[0.5132120685336232, 1, 1], 
       Hue[0.42295124545510054`, 1, 1], 
       Hue[0.42295124545510054`, 1, 1], 
       Hue[0.33277204168311014`, 1, 1], 
       Hue[0.33277204168311014`, 1, 1], 
       Hue[0.2427622335008183, 1, 1], 
       Hue[0.2427622335008183, 1, 1], 
       Hue[0.15303822097131786`, 1, 1], 
       Hue[0.15303822097131786`, 1, 1], 
       Hue[0.06368438721457334, 1, 1], 
       Hue[0.06368438721457334, 1, 1], 
       Hue[0.9746626610273306, 1, 1], 
       Hue[0.9746626610273306, 1, 1], 
       Hue[0.8858311772112987, 1, 1], 
       Hue[0.8858311772112987, 1, 1], 
       Hue[0.7970390083799735, 1, 1], 
       Hue[0.7970390083799735, 1, 1], 
       Hue[0.7082714984768756, 1, 1], 
       Hue[0.7082714984768756, 1, 1], 
       Hue[0.6196841994470936, 1, 1], 
       Hue[0.6196841994470936, 1, 1], 
       Hue[0.5315215505517608, 1, 1], 
       Hue[0.5315215505517608, 1, 1], 
       Hue[0.44394494742856366`, 1, 1], 
       Hue[0.44394494742856366`, 1, 1], 
       Hue[0.35688764850602844`, 1, 1], 
       Hue[0.35688764850602844`, 1, 1], 
       Hue[0.27009555087369436`, 1, 1], 
       Hue[0.27009555087369436`, 1, 1], 
       Hue[0.18331540575024727`, 1, 1], 
       Hue[0.18331540575024727`, 1, 1], 
       Hue[0.09654267138370616, 1, 1], 
       Hue[0.09654267138370616, 1, 1], 
       Hue[0.010094811765891522`, 1, 1], 
       Hue[0.010094811765891522`, 1, 1], 
       Hue[0.9244607391604271, 1, 1], 
       Hue[0.9244607391604271, 1, 1], 
       Hue[0.8399492499879724, 1, 1], 
       Hue[0.8399492499879724, 1, 1], 
       Hue[0.7564654779058759, 1, 1], 
       Hue[0.7564654779058759, 1, 1], 
       Hue[0.6735803972243876, 1, 1], 
       Hue[0.6735803972243876, 1, 1], 
       Hue[0.5908683135133599, 1, 1], 
       Hue[0.5908683135133599, 1, 1], 
       Hue[0.5083034172256362, 1, 1], 
       Hue[0.5083034172256362, 1, 1], 
       Hue[0.4264514587862894, 1, 1], 
       Hue[0.4264514587862894, 1, 1], 
       Hue[0.3462206847563066, 1, 1], 
       Hue[0.3462206847563066, 1, 1], 
       Hue[0.26831235013189686`, 1, 1], 
       Hue[0.26831235013189686`, 1, 1], 
       Hue[0.19281231771676, 1, 1], 
       Hue[0.19281231771676, 1, 1], 
       Hue[0.11921971156371028`, 1, 1], 
       Hue[0.11921971156371028`, 1, 1], 
       Hue[0.0468063636139859, 1, 1], 
       Hue[0.0468063636139859, 1, 1], 
       Hue[0.975109336835882, 1, 1], 
       Hue[0.975109336835882, 1, 1], 
       Hue[0.90435059561364, 1, 1], 
       Hue[0.90435059561364, 1, 1], 
       Hue[0.8355467810001385, 1, 1], 
       Hue[0.8355467810001385, 1, 1], 
       Hue[0.7701334240767309, 1, 1], 
       Hue[0.7701334240767309, 1, 1], 
       Hue[0.7093276744535297, 1, 1], 
       Hue[0.7093276744535297, 1, 1], 
       Hue[0.6537249557854934, 1, 1], 
       Hue[0.6537249557854934, 1, 1], 
       Hue[0.6033177014107454, 1, 1], 
       Hue[0.6033177014107454, 1, 1], 
       Hue[0.5577557908523973, 1, 1], 
       Hue[0.5577557908523973, 1, 1], 
       Hue[0.5165809575936386, 1, 1], 
       Hue[0.5165809575936386, 1, 1], 
       Hue[0.47936904837080296`, 1, 1], 
       Hue[0.47936904837080296`, 1, 1], 
       Hue[0.44578257043933917`, 1, 1], 
       Hue[0.44578257043933917`, 1, 1], 
       Hue[0.41557954902743466`, 1, 1], 
       Hue[0.41557954902743466`, 1, 1], 
       Hue[0.3885948026145062, 1, 1], 
       Hue[0.3885948026145062, 1, 1], 
       Hue[0.36472011513852276`, 1, 1], 
       Hue[0.36472011513852276`, 1, 1], 
       Hue[0.3438849218716845, 1, 1], 
       Hue[0.3438849218716845, 1, 1], 
       Hue[0.32603800153585283`, 1, 1], 
       Hue[0.32603800153585283`, 1, 1], 
       Hue[0.3111376031158137, 1, 1], 
       Hue[0.3111376031158137, 1, 1], 
       Hue[0.2991410864394776, 1, 1], 
       Hue[0.2991410864394776, 1, 1], 
       Hue[0.2900012220807621, 1, 1], 
       Hue[0.2900012220807621, 1, 1], 
       Hue[0.2836643128481897, 1, 1], 
       Hue[0.2836643128481897, 1, 1], 
       Hue[0.2800696616453035, 1, 1], 
       Hue[0.2800696616453035, 1, 1], 
       Hue[0.2791497176725532, 1, 1], 
       Hue[0.2791497176725532, 1, 1], 
       Hue[0.2808316877825547, 1, 1], 
       Hue[0.2808316877825547, 1, 1], 
       Hue[0.28503905898740767`, 1, 1], 
       Hue[0.28503905898740767`, 1, 1], 
       Hue[0.2916934185745679, 1, 1], 
       Hue[0.2916934185745679, 1, 1], 
       Hue[0.30071520604920704`, 1, 1], 
       Hue[0.30071520604920704`, 1, 1], 
       Hue[0.31202487261304357`, 1, 1], 
       Hue[0.31202487261304357`, 1, 1], 
       Hue[0.3255441697020442, 1, 1], 
       Hue[0.3255441697020442, 1, 1], 
       Hue[0.3411962371961555, 1, 1], 
       Hue[0.3411962371961555, 1, 1], 
       Hue[0.3589074694876606, 1, 1], 
       Hue[0.3589074694876606, 1, 1], 
       Hue[0.3786067400687169, 1, 1], 
       Hue[0.3786067400687169, 1, 1], 
       Hue[0.4002252438807113, 1, 1], 
       Hue[0.4002252438807113, 1, 1], 
       Hue[0.4236980743800631, 1, 1], 
       Hue[0.4236980743800631, 1, 1], 
       Hue[0.4489635507509118, 1, 1], 
       Hue[0.4489635507509118, 1, 1], 
       Hue[0.47596293667158696`, 1, 1], 
       Hue[0.47596293667158696`, 1, 1], 
       Hue[0.5046413799058753, 1, 1], 
       Hue[0.5046413799058753, 1, 1], 
       Hue[0.5349458661619949, 1, 1], 
       Hue[0.5349458661619949, 1, 1], 
       Hue[0.5668267075469183, 1, 1], 
       Hue[0.5668267075469183, 1, 1], 
       Hue[0.6002383395664004, 1, 1], 
       Hue[0.6002383395664004, 1, 1], 
       Hue[0.6351364442922158, 1, 1], 
       Hue[0.6351364442922158, 1, 1], 
       Hue[0.6714794781732603, 1, 1], 
       Hue[0.6714794781732603, 1, 1], 
       Hue[0.7092297851685113, 1, 1], 
       Hue[0.7092297851685113, 1, 1], 
       Hue[0.7483498438618739, 1, 1], 
       Hue[0.7483498438618739, 1, 1], 
       Hue[0.7888059216275258, 1, 1], 
       Hue[0.7888059216275258, 1, 1], 
       Hue[0.8305658029377244, 1, 1], 
       Hue[0.8305658029377244, 1, 1], 
       Hue[0.8735985589444489, 1, 1], 
       Hue[0.8735985589444489, 1, 1], 
       Hue[0.9178756154733176, 1, 1], 
       Hue[0.9178756154733176, 1, 1], 
       Hue[0.9633697925167194, 1, 1], 
       Hue[0.9633697925167194, 1, 1], 
       Hue[0.010055769432912077`, 1, 1], 
       Hue[0.010055769432912077`, 1, 1], 
       Hue[0.05790905682561934, 1, 1], 
       Hue[0.05790905682561934, 1, 1], 
       Hue[0.1069068241608519, 1, 1], 
       Hue[0.1069068241608519, 1, 1], 
       Hue[0.1570279699615826, 1, 1], 
       Hue[0.1570279699615826, 1, 1], 
       Hue[0.20825197197012807`, 1, 1], 
       Hue[0.20825197197012807`, 1, 1], 
       Hue[0.2605589059274433, 1, 1], 
       Hue[0.2605589059274433, 1, 1], 
       Hue[0.31393019394989297`, 1, 1], 
       Hue[0.31393019394989297`, 1, 1], 
       Hue[0.36834904422164405`, 1, 1], 
       Hue[0.36834904422164405`, 1, 1], 
       Hue[0.4237981000610914, 1, 1], 
       Hue[0.4237981000610914, 1, 1], 
       Hue[0.480262570763982, 1, 1], 
       Hue[0.480262570763982, 1, 1], 
       Hue[0.5377260144795512, 1, 1], 
       Hue[0.5377260144795512, 1, 1], 
       Hue[0.5961750384728735, 1, 1], 
       Hue[0.5961750384728735, 1, 1], 
       Hue[0.6555958067322936, 1, 1], 
       Hue[0.6555958067322936, 1, 1], 
       Hue[0.7159748673000743, 1, 1], 
       Hue[0.7159748673000743, 1, 1], 
       Hue[0.7773000790053467, 1, 1], 
       Hue[0.7773000790053467, 1, 1], 
       Hue[0.8395593248056266, 1, 1], 
       Hue[0.8395593248056266, 1, 1], 
       Hue[0.9027412896333733, 1, 1], 
       Hue[0.9027412896333733, 1, 1], 
       Hue[0.9668345230053836, 1, 1], 
       Hue[0.9668345230053836, 1, 1], 
       Hue[0.03182905417736379, 1, 1], 
       Hue[0.03182905417736379, 1, 1], 
       Hue[0.09771495862869105, 1, 1], 
       Hue[0.09771495862869105, 1, 1], 
       Hue[0.1644804683063499, 1, 1], 
       Hue[0.1644804683063499, 1, 1], 
       Hue[0.232118474204226, 1, 1], 
       Hue[0.232118474204226, 1, 1], 
       Hue[0.3006190916938448, 1, 1], 
       Hue[0.3006190916938448, 1, 1], 
       Hue[0.36997412177509265`, 1, 1], 
       Hue[0.36997412177509265`, 1, 1], 
       Hue[0.44017466975917585`, 1, 1], 
       Hue[0.44017466975917585`, 1, 1], 
       Hue[0.5112124994864206, 1, 1], 
       Hue[0.5112124994864206, 1, 1], 
       Hue[0.5830804527288551, 1, 1], 
       Hue[0.5830804527288551, 1, 1], 
       Hue[0.6557702517243329, 1, 1], 
       Hue[0.6557702517243329, 1, 1], 
       Hue[0.7292765966264289, 1, 1], 
       Hue[0.7292765966264289, 1, 1], 
       Hue[0.8035895255914776, 1, 1], 
       Hue[0.8035895255914776, 1, 1], 
       Hue[0.878701965966677, 1, 1], 
       Hue[0.878701965966677, 1, 1], 
       Hue[0.95461010302941, 1, 1], 
       Hue[0.95461010302941, 1, 1], 
       Hue[0.03130808683880092, 1, 1], 
       Hue[0.03130808683880092, 1, 1], 
       Hue[0.10878523846407365`, 1, 1], 
       Hue[0.10878523846407365`, 1, 1], 
       Hue[0.187036929367527, 1, 1], 
       Hue[0.187036929367527, 1, 1], 
       Hue[0.26606114164798506`, 1, 1], 
       Hue[0.26606114164798506`, 1, 1], 
       Hue[0.3458479697912301, 1, 1], 
       Hue[0.3458479697912301, 1, 1], 
       Hue[0.42639267303198874`, 1, 1], 
       Hue[0.42639267303198874`, 1, 1], 
       Hue[0.5076933544748576, 1, 1], 
       Hue[0.5076933544748576, 1, 1], 
       Hue[0.5897379334740892, 1, 1], 
       Hue[0.5897379334740892, 1, 1], 
       Hue[0.6725241340879914, 1, 1], 
       Hue[0.6725241340879914, 1, 1], 
       Hue[0.7560528613554944, 1, 1], 
       Hue[0.7560528613554944, 1, 1], 
       Hue[0.8403144520020706, 1, 1], 
       Hue[0.8403144520020706, 1, 1], 
       Hue[0.9253050137763409, 1, 1], 
       Hue[0.9253050137763409, 1, 1], 
       Hue[0.011017846082229929`, 1, 1], 
       Hue[0.011017846082229929`, 1, 1], 
       Hue[0.09744381217502465, 1, 1], 
       Hue[0.09744381217502465, 1, 1], 
       Hue[0.18458837476421924`, 1, 1], 
       Hue[0.18458837476421924`, 1, 1], 
       Hue[0.2724509652327783, 1, 1], 
       Hue[0.2724509652327783, 1, 1], 
       Hue[0.3610067027049437, 1, 1], 
       Hue[0.3610067027049437, 1, 1], 
       Hue[0.4502651974747799, 1, 1], 
       Hue[0.4502651974747799, 1, 1], 
       Hue[0.5402360248028113, 1, 1], 
       Hue[0.5402360248028113, 1, 1], 
       Hue[0.6308979533152481, 1, 1], 
       Hue[0.6308979533152481, 1, 1], 
       Hue[0.7222476450207425, 1, 1], 
       Hue[0.7222476450207425, 1, 1], 
       Hue[0.8142704033445503, 1, 1], 
       Hue[0.8142704033445503, 1, 1], 
       Hue[0.9069734465940373, 1, 1], 
       Hue[0.9069734465940373, 1, 1], 
       Hue[0.0003751091215660667, 1, 1], 
       Hue[0.0003751091215660667, 1, 1], 
       Hue[0.09444859931788216, 1, 1], 
       Hue[0.09444859931788216, 1, 1], 
       Hue[0.18919701920563461`, 1, 1], 
       Hue[0.18919701920563461`, 1, 1], 
       Hue[0.2846160837458422, 1, 1], 
       Hue[0.2846160837458422, 1, 1], 
       Hue[0.3806721820941903, 1, 1], 
       Hue[0.3806721820941903, 1, 1], 
       Hue[0.4773855325816415, 1, 1], 
       Hue[0.4773855325816415, 1, 1], 
       Hue[0.5747717077904234, 1, 1], 
       Hue[0.5747717077904234, 1, 1], 
       Hue[0.6728222034045579, 1, 1], 
       Hue[0.6728222034045579, 1, 1], 
       Hue[0.7715478143526769, 1, 1], 
       Hue[0.7715478143526769, 1, 1], 
       Hue[0.8708970154927892, 1, 1], 
       Hue[0.8708970154927892, 1, 1], 
       Hue[0.9708163161382002, 1, 1], 
       Hue[0.9708163161382002, 1, 1], 
       Hue[0.07139599464877043, 1, 1], 
       Hue[0.07139599464877043, 1, 1], 
       Hue[0.172692594788118, 1, 1], 
       Hue[0.172692594788118, 1, 1], 
       Hue[0.2746347559316637, 1, 1], 
       Hue[0.2746347559316637, 1, 1], 
       Hue[0.37720554526915917`, 1, 1], 
       Hue[0.37720554526915917`, 1, 1], 
       Hue[0.4803521818452014, 1, 1], 
       Hue[0.4803521818452014, 1, 1], 
       Hue[0.5839602695737227, 1, 1], 
       Hue[0.5839602695737227, 1, 1], 
       Hue[0.6881708094172927, 1, 1], 
       Hue[0.6881708094172927, 1, 1], 
       Hue[0.7932710636593592, 1, 1], 
       Hue[0.7932710636593592, 1, 1], 
       Hue[0.8990904369082359, 1, 1], 
       Hue[0.8990904369082359, 1, 1], 
       Hue[0.005048224852851583, 1, 1], 
       Hue[0.005048224852851583, 1, 1], 
       Hue[0.1114275878230381, 1, 1], 
       Hue[0.1114275878230381, 1, 1], 
       Hue[0.2193610331407116, 1, 1], 
       Hue[0.2193610331407116, 1, 1], 
       Hue[0.3283303659856307, 1, 1], 
       Hue[0.3283303659856307, 1, 1], 
       Hue[0.4367141006325967, 1, 1], 
       Hue[0.4367141006325967, 1, 1], 
       Hue[0.5453967024411587, 1, 1], 
       Hue[0.5453967024411587, 1, 1], 
       Hue[0.6557591931182715, 1, 1], 
       Hue[0.6557591931182715, 1, 1], 
       Hue[0.7662084995133627, 1, 1], 
       Hue[0.7662084995133627, 1, 1], 
       Hue[0.8765508443172676, 1, 1]}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->{True, None},
    AxesOrigin->{0, 0},
    Epilog->
     LineBox[{{1, 4.774163964623409*^-7}, {2, 7.568037735656116*^-7}, {
       3, 8.425183630226619*^-7}, {4, 7.917231523205861*^-7}, {
       5, 8.554168651769534*^-7}, {6, 2.672277370165971*^-7}, {
       7, 1.0040845588091148`*^-6}, {8, 6.829271023376666*^-7}, {
       9, 7.03086746489622*^-7}, {10, 7.435894236299269*^-7}, {
       11, 6.771128449083131*^-7}, {12, 5.815877964078606*^-7}, {
       13, 8.202379833185497*^-7}, {14, 5.937719042033464*^-7}, {
       15, 8.078166423174234*^-7}, {16, 5.086819889358714*^-7}, {
       17, 1.1878770347658353`*^-6}, {18, 4.5767974221630423`*^-7}, {
       19, 1.285767114195014*^-6}, {20, 9.987594815592835*^-7}, {
       21, 5.216225831754935*^-7}, {22, 2.7597912050209344`*^-7}, {
       23, 3.381099420630693*^-7}, {24, 5.129014284753304*^-7}, {
       25, 7.486717557597984*^-7}, {26, 1.0194041304552456`*^-6}, {
       27, 1.1199915232620164`*^-6}, {28, 1.4729528050601566`*^-7}, {
       29, 9.149084668926432*^-7}, {30, 6.790996535943909*^-7}, {
       31, 1.0088465419129672`*^-6}, {32, 1.2571103411588953`*^-6}, {
       33, 1.6101247730073053`*^-6}, {34, 1.6571629696646278`*^-6}, {
       35, 2.257069457133729*^-6}, {36, 2.7084090806485868`*^-6}, {
       37, 2.8848308038545456`*^-6}, {38, 3.8144957415019324`*^-6}, {
       39, 4.139539260535415*^-6}, {40, 5.280530651613977*^-6}, {
       41, 6.364055669957215*^-6}, {42, 7.431702353783643*^-6}, {
       43, 9.162140016540454*^-6}, {44, 0.000011392767771047028`}, {
       45, 0.00001447310790630038}, {46, 0.00001764347631662918}, {
       47, 0.000020869685237656144`}, {48, 0.00002566824990876318}, {
       49, 0.00003112225832156994}, {50, 0.00003744839707612007}, {
       51, 0.00004465548565402208}, {52, 0.00005395965578331769}, {
       53, 0.0000651462060312024}, {54, 0.00007749863683983881}, {
       55, 0.00009315560511470502}, {56, 0.00011139576418451982`}, {
       57, 0.00013237136919133893`}, {58, 0.00015811023723313907`}, {
       59, 0.00018632942795518157`}, {60, 0.00022112273953046554`}, {
       61, 0.0002617510012985398}, {62, 0.0003080954313370296}, {
       63, 0.00036302525968071866`}, {64, 0.0004264547182322541}, {
       65, 0.0005003821649413357}, {66, 0.0005873538216184944}, {
       67, 0.0006864116432464454}, {68, 0.0008014674138896858}, {
       69, 0.0009333994867948699}, {70, 0.0010847876822632763`}, {
       71, 0.0012593227345309218`}, {72, 0.001458228594512158}, {
       73, 0.0016859147516438064`}, {74, 0.0019463420713989252`}, {
       75, 0.0022425844949409057`}, {76, 0.0025777013687523073`}, {
       77, 0.0029580128365886494`}, {78, 0.0033875670127808126`}, {
       79, 0.003874037458736444}, {80, 0.0044227766864224135`}, {
       81, 0.005039860646506613}, {82, 0.005730631198991554}, {
       83, 0.006502357545690735}, {84, 0.007361882997114838}, {
       85, 0.008320421400344356}, {86, 0.009389875448602624}, {
       87, 0.010581091734063677`}, {88, 0.0119010820665137}, {
       89, 0.013353791144902543`}, {90, 0.014945917078016499`}, {
       91, 0.016689261702392058`}, {92, 0.01860538709566863}, {
       93, 0.020718611614680704`}, {94, 0.02304453228252633}, {
       95, 0.025584912527564854`}, {96, 0.028326367942955868`}, {
       97, 0.03126478541924108}, {98, 0.03441944719033271}, {
       99, 0.03784125064069806}, {100, 0.04158886136121419}, {
       101, 0.04568761779698456}, {102, 0.05010135941548635}, {
       103, 0.05475780915237832}, {104, 0.0596079890235699}, {
       105, 0.06470225013523107}, {106, 0.07019202632746886}, {
       107, 0.07624640551664297}, {108, 0.08290956559240623}, {
       109, 0.09003160896853278}, {110, 0.0973476745313712}, {
       111, 0.10469178807119026`}, {112, 0.11219896188055296`}, {
       113, 0.12032496238710254`}, {114, 0.12960077353847177`}, {
       115, 0.14023838052557958`}, {116, 0.1519029118363534}, {
       117, 0.16386240023452045`}, {118, 0.17545841255185335`}, {
       119, 0.18661894340048335`}, {120, 0.19811546100743763`}, {
       121, 0.21140514326324156`}, {122, 0.2280214318631956}, {
       123, 0.24883856143769392`}, {124, 0.27365480899730454`}, {
       125, 0.3013369856185038}, {126, 0.3303064652291558}, {
       127, 0.35901634257470755`}, {128, 0.38625267810842195`}, {
       129, 0.41122460759822177`}, {130, 0.43355125980323717`}, {
       131, 0.453162031114551}, {132, 0.47019803764867696`}, {
       133, 0.48491470269211906`}, {134, 0.4976061062157364}, {
       135, 0.5085583260585054}, {136, 0.518026485174305}, {
       137, 0.5262148396818539}, {138, 0.5332818671277445}, {
       139, 0.5393454860076266}, {140, 0.5444852770071663}, {
       141, 0.5487545918203182}, {142, 0.5521857094414232}, {
       143, 0.5547959634375633}, {144, 0.5565913203376384}, {
       145, 0.5575738932945108}, {146, 0.5577419273135292}, {
       147, 0.5570914786882771}, {148, 0.5556181975564911}, {
       149, 0.5533241726653495}, {150, 0.5502100168343496}, {
       151, 0.5462833887290199}, {152, 0.541552576757865}, {
       153, 0.5360299293130407}, {154, 0.5297356037531146}, {
       155, 0.5226889068390108}, {156, 0.5149168060814912}, {
       157, 0.506448587559631}, {158, 0.4973173609167664}, {
       159, 0.4875612372675214}, {160, 0.4772174953570008}, {
       161, 0.4663292306681738}, {162, 0.45494021484118075`}, {
       163, 0.4430965697404628}, {164, 0.4308475471281948}, {
       165, 0.4182400033512116}, {166, 0.4053267858564404}, {
       167, 0.3921556043066159}, {168, 0.3787761315316371}, {
       169, 0.3652404352476588}, {170, 0.3515956610054241}, {
       171, 0.33788993126205324`}, {172, 0.324170855316314}, {
       173, 0.3104820235253137}, {174, 0.2968681721873051}, {
       175, 0.28336996537216935`}, {176, 0.27002584096169524`}, {
       177, 0.25687251926572385`}, {178, 0.2439423001070239}, {
       179, 0.23126918083697703`}, {180, 0.21887819900453515`}, {
       181, 0.20679809804262636`}, {182, 0.1950491841962314}, {
       183, 0.18365240035483898`}, {184, 0.17262431658449057`}, {
       185, 0.161979885755609}, {186, 0.15172988008392838`}, {
       187, 0.14188261174244587`}, {188, 0.13244471277942968`}, {
       189, 0.12342183358708786`}, {190, 0.11481381001265807`}, {
       191, 0.10662116322805637`}, {192, 0.0988402429713177}, {
       193, 0.09146813543930105}, {194, 0.08449788088019945}, {
       195, 0.07792314701235764}, {196, 0.07173402566589034}, {
       197, 0.0659200818992794}, {198, 0.0604712572639386}, {
       199, 0.05537587343623979}, {200, 0.050619753553637604`}, {
       201, 0.04619008938718013}, {202, 0.04207448190356889}, {
       203, 0.03825780940923136}, {204, 0.03472502594628561}, {
       205, 0.031462799843238534`}, {206, 0.028455982257468645`}, {
       207, 0.025690166887511016`}, {208, 0.023152469210702176`}, {
       209, 0.020828052788340642`}, {210, 0.018703075944314102`}, {
       211, 0.01676459062141027}, {212, 0.014999586189779266`}, {
       213, 0.013396312640309977`}, {214, 0.011943315448921446`}, {
       215, 0.010628030423965784`}, {216, 0.009440115327044463}, {
       217, 0.008370508934478776}, {218, 0.007408313505786503}, {
       219, 0.006544331687595083}, {220, 0.005770344999403386}, {
       221, 0.005078447211523219}, {222, 0.004461822932292769}, {
       223, 0.003912784782162757}, {224, 0.003424695005263588}, {
       225, 0.002991979232332229}, {226, 0.002608754893855015}, {
       227, 0.0022703234368165624`}, {228, 0.001972412154716224}, {
       229, 0.0017104358088202098`}, {230, 0.0014804759671325658`}, {
       231, 0.0012788762024786508`}, {232, 0.0011022869387119502`}, {
       233, 0.0009484298322187346}, {234, 0.0008149359203244266}, {
       235, 0.0006988635078325137}, {236, 0.000597980578326035}, {
       237, 0.0005105670292123982}, {238, 0.00043489340690183514`}, {
       239, 0.0003698071468226065}, {240, 0.00031431605300783155`}, {
       241, 0.00026676746792321325`}, {242, 0.0002255104356572774}, {
       243, 0.00019012324368898693`}, {244, 0.00016069555881611866`}, {
       245, 0.0001358313504358103}, {246, 0.00011379949585837613`}, {
       247, 0.00009489449949196286}, {248, 0.0000795723425611223}, {
       249, 0.00006659351660543365}, {250, 0.00005531721537167322}, {
       251, 0.0000461994764096256}}],
    Frame->True,
    FrameTicks->{{{1., 
        FormBox[
         RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
        FormBox[
         RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
        FormBox[
         RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
        FormBox[
         RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
        FormBox["0", TraditionalForm]}, {157.25, 
        FormBox["2.5`", TraditionalForm]}, {188.5, 
        FormBox["5", TraditionalForm]}, {219.75, 
        FormBox["7.5`", TraditionalForm]}, {251., 
        FormBox["10", TraditionalForm]}}, Automatic, None, None},
    PlotRange->{-0.1, 1.1},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], Automatic}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF1nlcTXkfB/CjIWUJyTIGczVqQtZQFH0klKJ9307rvbfVUDEMzlhmLCPJ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     "], {
      {GrayLevel[0.5], EdgeForm[None], 
       GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ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         "]]]}, {{}, {}, {
        {Hue[0.6093005755620837, 1, 1], LineBox[{1, 2}]}, 
        {Hue[0.5883030831109552, 1, 1], LineBox[{2, 3}]}, 
        {Hue[0.34522610533488746`, 1, 1], LineBox[{3, 4}]}, 
        {Hue[0.21884433245913584`, 1, 1], LineBox[{4, 5}]}, 
        {Hue[0.05273737922507198, 1, 1], LineBox[{5, 6}]}, 
        {Hue[0.0800429915891726, 1, 1], LineBox[{6, 7}]}, 
        {Hue[0.8622231077078536, 1, 1], LineBox[{7, 8}]}, 
        {Hue[0.9811433145234884, 1, 1], LineBox[{8, 9}]}, 
        {Hue[0.7656439891671794, 1, 1], LineBox[{9, 10}]}, 
        {Hue[0.575197751929063, 1, 1], LineBox[{10, 11}]}, 
        {Hue[0.5610195544187918, 1, 1], LineBox[{11, 12}]}, 
        {Hue[0.4383547897938635, 1, 1], LineBox[{12, 13}]}, 
        {Hue[0.3897875310877631, 1, 1], LineBox[{13, 14}]}, 
        {Hue[0.86236870675324, 1, 1], LineBox[{14, 15}]}, 
        {Hue[0.15167136723202979`, 1, 1], LineBox[{15, 16}]}, 
        {Hue[0.08718633620831152, 1, 1], LineBox[{16, 17}]}, 
        {Hue[0.6976688277163468, 1, 1], LineBox[{17, 18}]}, 
        {Hue[0.9140501077809731, 1, 1], LineBox[{18, 19}]}, 
        {Hue[0.7775122878648932, 1, 1], LineBox[{19, 20}]}, 
        {Hue[0.6750104754075937, 1, 1], LineBox[{20, 21}]}, 
        {Hue[0.601540385332832, 1, 1], LineBox[{21, 22}]}, 
        {Hue[0.599162923023778, 1, 1], LineBox[{22, 23}]}, 
        {Hue[0.4732816613374724, 1, 1], LineBox[{23, 24}]}, 
        {Hue[0.42205667261767177`, 1, 1], LineBox[{24, 25}]}, 
        {Hue[0.283293118551893, 1, 1], LineBox[{25, 26}]}, 
        {Hue[0.25052329442331206`, 1, 1], LineBox[{26, 27}]}, 
        {Hue[0.16345913842696996`, 1, 1], LineBox[{27, 28}]}, 
        {Hue[0.042713628129332624`, 1, 1], LineBox[{28, 29}]}, 
        {Hue[0.9476834322091386, 1, 1], LineBox[{29, 30}]}, 
        {Hue[0.8542985795573227, 1, 1], LineBox[{30, 31}]}, 
        {Hue[0.762040334551915, 1, 1], LineBox[{31, 32}]}, 
        {Hue[0.6562323051332618, 1, 1], LineBox[{32, 33}]}, 
        {Hue[0.5611032812885219, 1, 1], LineBox[{33, 34}]}, 
        {Hue[0.46177778309986633`, 1, 1], LineBox[{34, 35}]}, 
        {Hue[0.36288473900116, 1, 1], LineBox[{35, 36}]}, 
        {Hue[0.2613185162117361, 1, 1], LineBox[{36, 37}]}, 
        {Hue[0.15485214106860662`, 1, 1], LineBox[{37, 38}]}, 
        {Hue[0.05584266875309582, 1, 1], LineBox[{38, 39}]}, 
        {Hue[0.9554896802057098, 1, 1], LineBox[{39, 40}]}, 
        {Hue[0.8511929684959288, 1, 1], LineBox[{40, 41}]}, 
        {Hue[0.752637322873851, 1, 1], LineBox[{41, 42}]}, 
        {Hue[0.6511751745758734, 1, 1], LineBox[{42, 43}]}, 
        {Hue[0.5544346510193291, 1, 1], LineBox[{43, 44}]}, 
        {Hue[0.45624627578443544`, 1, 1], LineBox[{44, 45}]}, 
        {Hue[0.3557371285223141, 1, 1], LineBox[{45, 46}]}, 
        {Hue[0.25651723449211883`, 1, 1], LineBox[{46, 47}]}, 
        {Hue[0.15787678371539723`, 1, 1], LineBox[{47, 48}]}, 
        {Hue[0.05829216344985523, 1, 1], LineBox[{48, 49}]}, 
        {Hue[0.9603759234494436, 1, 1], LineBox[{49, 50}]}, 
        {Hue[0.8622537572154851, 1, 1], LineBox[{50, 51}]}, 
        {Hue[0.7641289269589387, 1, 1], LineBox[{51, 52}]}, 
        {Hue[0.6671796192899093, 1, 1], LineBox[{52, 53}]}, 
        {Hue[0.570035522629933, 1, 1], LineBox[{53, 54}]}, 
        {Hue[0.47298974794991105`, 1, 1], LineBox[{54, 55}]}, 
        {Hue[0.37603519041829964`, 1, 1], LineBox[{55, 56}]}, 
        {Hue[0.27935036622232096`, 1, 1], LineBox[{56, 57}]}, 
        {Hue[0.18303412229352026`, 1, 1], LineBox[{57, 58}]}, 
        {Hue[0.0868397578505655, 1, 1], LineBox[{58, 59}]}, 
        {Hue[0.9908567670859162, 1, 1], LineBox[{59, 60}]}, 
        {Hue[0.8951640675116295, 1, 1], LineBox[{60, 61}]}, 
        {Hue[0.7997576994445039, 1, 1], LineBox[{61, 62}]}, 
        {Hue[0.704677263864956, 1, 1], LineBox[{62, 63}]}, 
        {Hue[0.6097791393361929, 1, 1], LineBox[{63, 64}]}, 
        {Hue[0.51511325912361, 1, 1], LineBox[{64, 65}]}, 
        {Hue[0.4206932535946379, 1, 1], LineBox[{65, 66}]}, 
        {Hue[0.3264908912211963, 1, 1], LineBox[{66, 67}]}, 
        {Hue[0.23247587761718255`, 1, 1], LineBox[{67, 68}]}, 
        {Hue[0.1386847975223601, 1, 1], LineBox[{68, 69}]}, 
        {Hue[0.04511404796464239, 1, 1], LineBox[{69, 70}]}, 
        {Hue[0.9517719103841894, 1, 1], LineBox[{70, 71}]}, 
        {Hue[0.8586357506433461, 1, 1], LineBox[{71, 72}]}, 
        {Hue[0.7657480313793572, 1, 1], LineBox[{72, 73}]}, 
        {Hue[0.6730497593471026, 1, 1], LineBox[{73, 74}]}, 
        {Hue[0.5805547319386801, 1, 1], LineBox[{74, 75}]}, 
        {Hue[0.4882651524400857, 1, 1], LineBox[{75, 76}]}, 
        {Hue[0.39616028884303134`, 1, 1], LineBox[{76, 77}]}, 
        {Hue[0.3042682849642356, 1, 1], LineBox[{77, 78}]}, 
        {Hue[0.21256569508200795`, 1, 1], LineBox[{78, 79}]}, 
        {Hue[0.12105676008102176`, 1, 1], LineBox[{79, 80}]}, 
        {Hue[0.02974915153054283, 1, 1], LineBox[{80, 81}]}, 
        {Hue[0.9386188542407726, 1, 1], LineBox[{81, 82}]}, 
        {Hue[0.8476676926099618, 1, 1], LineBox[{82, 83}]}, 
        {Hue[0.7568938711280264, 1, 1], LineBox[{83, 84}]}, 
        {Hue[0.666294270544339, 1, 1], LineBox[{84, 85}]}, 
        {Hue[0.5758728368553732, 1, 1], LineBox[{85, 86}]}, 
        {Hue[0.4856387576042597, 1, 1], LineBox[{86, 87}]}, 
        {Hue[0.39558025894890825`, 1, 1], LineBox[{87, 88}]}, 
        {Hue[0.30568691036491824`, 1, 1], LineBox[{88, 89}]}, 
        {Hue[0.21595156833836465`, 1, 1], LineBox[{89, 90}]}, 
        {Hue[0.12637324682863935`, 1, 1], LineBox[{90, 91}]}, 
        {Hue[0.03695515584535301, 1, 1], LineBox[{91, 92}]}, 
        {Hue[0.9477192956297038, 1, 1], LineBox[{92, 93}]}, 
        {Hue[0.858670444255665, 1, 1], LineBox[{93, 94}]}, 
        {Hue[0.7698077509905155, 1, 1], LineBox[{94, 95}]}, 
        {Hue[0.6811137464946615, 1, 1], LineBox[{95, 96}]}, 
        {Hue[0.5925780318446903, 1, 1], LineBox[{96, 97}]}, 
        {Hue[0.5041925162411358, 1, 1], LineBox[{97, 98}]}, 
        {Hue[0.41598740282012486`, 1, 1], LineBox[{98, 99}]}, 
        {Hue[0.3280008129379155, 1, 1], LineBox[{99, 100}]}, 
        {Hue[0.24027124169676167`, 1, 1], LineBox[{100, 101}]}, 
        {Hue[0.1528061272499971, 1, 1], LineBox[{101, 102}]}, 
        {Hue[0.06558679955777666, 1, 1], LineBox[{102, 103}]}, 
        {Hue[0.978589469438239, 1, 1], LineBox[{103, 104}]}, 
        {Hue[0.891817591134329, 1, 1], LineBox[{104, 105}]}, 
        {Hue[0.8053329095049395, 1, 1], LineBox[{105, 106}]}, 
        {Hue[0.7192408998844428, 1, 1], LineBox[{106, 107}]}, 
        {Hue[0.6336444994087578, 1, 1], LineBox[{107, 108}]}, 
        {Hue[0.5485979186574695, 1, 1], LineBox[{108, 109}]}, 
        {Hue[0.4640872331328644, 1, 1], LineBox[{109, 110}]}, 
        {Hue[0.3800792298129656, 1, 1], LineBox[{110, 111}]}, 
        {Hue[0.29658892069408954`, 1, 1], LineBox[{111, 112}]}, 
        {Hue[0.2137498070449708, 1, 1], LineBox[{112, 113}]}, 
        {Hue[0.1317985031971102, 1, 1], LineBox[{113, 114}]}, 
        {Hue[0.05100279554728599, 1, 1], LineBox[{114, 115}]}, 
        {Hue[0.9715544937306999, 1, 1], LineBox[{115, 116}]}, 
        {Hue[0.8935117552806451, 1, 1], LineBox[{116, 117}]}, 
        {Hue[0.8168378148577125, 1, 1], LineBox[{117, 118}]}, 
        {Hue[0.741507451773556, 1, 1], LineBox[{118, 119}]}, 
        {Hue[0.6676383743085668, 1, 1], LineBox[{119, 120}]}, 
        {Hue[0.5955532908070316, 1, 1], LineBox[{120, 121}]}, 
        {Hue[0.525747511026724, 1, 1], LineBox[{121, 122}]}, 
        {Hue[0.4587611269785911, 1, 1], LineBox[{122, 123}]}, 
        {Hue[0.3950341786702998, 1, 1], LineBox[{123, 124}]}, 
        {Hue[0.3348239923730795, 1, 1], LineBox[{124, 125}]}, 
        {Hue[0.278206263626647, 1, 1], LineBox[{125, 126}]}, 
        {Hue[0.22512972585836025`, 1, 1], LineBox[{126, 127}]}, 
        {Hue[0.17548723112854178`, 1, 1], LineBox[{127, 128}]}, 
        {Hue[0.1291654748489493, 1, 1], LineBox[{128, 129}]}, 
        {Hue[0.08607432464118504, 1, 1], LineBox[{129, 130}]}, 
        {Hue[0.04615674856215593, 1, 1], LineBox[{130, 131}]}, 
        {Hue[0.009383628358470269, 1, 1], LineBox[{131, 132}]}, 
        {Hue[0.9757461448592223, 1, 1], LineBox[{132, 133}]}, 
        {Hue[0.9452432063344988, 1, 1], LineBox[{133, 134}]}, 
        {Hue[0.9178706741768455, 1, 1], LineBox[{134, 135}]}, 
        {Hue[0.8936176561725973, 1, 1], LineBox[{135, 136}]}, 
        {Hue[0.8724589000455976, 1, 1], LineBox[{136, 137}]}, 
        {Hue[0.8543561829042435, 1, 1], LineBox[{137, 138}]}, 
        {Hue[0.8392565806338904, 1, 1], LineBox[{138, 139}]}, 
        {Hue[0.8270954879357122, 1, 1], LineBox[{139, 140}]}, 
        {Hue[0.8177994855524553, 1, 1], LineBox[{140, 141}]}, 
        {Hue[0.8112868389087454, 1, 1], LineBox[{141, 142}]}, 
        {Hue[0.8074714251134552, 1, 1], LineBox[{142, 143}]}, 
        {Hue[0.8062642398689808, 1, 1], LineBox[{143, 144}]}, 
        {Hue[0.80757390034191, 1, 1], LineBox[{144, 145}]}, 
        {Hue[0.8113117373256622, 1, 1], LineBox[{145, 146}]}, 
        {Hue[0.8173886140816178, 1, 1], LineBox[{146, 147}]}, 
        {Hue[0.8257180119328547, 1, 1], LineBox[{147, 148}]}, 
        {Hue[0.8362167368039202, 1, 1], LineBox[{148, 149}]}, 
        {Hue[0.8488048851013356, 1, 1], LineBox[{149, 150}]}, 
        {Hue[0.8634056201434694, 1, 1], LineBox[{150, 151}]}, 
        {Hue[0.8799464877098273, 1, 1], LineBox[{151, 152}]}, 
        {Hue[0.8983581839384799, 1, 1], LineBox[{152, 153}]}, 
        {Hue[0.9185754267824033, 1, 1], LineBox[{153, 154}]}, 
        {Hue[0.9405376116358177, 1, 1], LineBox[{154, 155}]}, 
        {Hue[0.9641858131985447, 1, 1], LineBox[{155, 156}]}, 
        {Hue[0.9894658914039416, 1, 1], LineBox[{156, 157}]}, 
        {Hue[0.016326442168588418`, 1, 1], LineBox[{157, 158}]}, 
        {Hue[0.04471969994336976, 1, 1], LineBox[{158, 159}]}, 
        {Hue[0.07460025787099636, 1, 1], LineBox[{159, 160}]}, 
        {Hue[0.1059256870147042, 1, 1], LineBox[{160, 161}]}, 
        {Hue[0.13865648207719755`, 1, 1], LineBox[{161, 162}]}, 
        {Hue[0.17275516346313075`, 1, 1], LineBox[{162, 163}]}, 
        {Hue[0.20818654705850076`, 1, 1], LineBox[{163, 164}]}, 
        {Hue[0.24491776062038736`, 1, 1], LineBox[{164, 165}]}, 
        {Hue[0.2829175843834493, 1, 1], LineBox[{165, 166}]}, 
        {Hue[0.32215693783560934`, 1, 1], LineBox[{166, 167}]}, 
        {Hue[0.36260803122091423`, 1, 1], LineBox[{167, 168}]}, 
        {Hue[0.40424526696122776`, 1, 1], LineBox[{168, 169}]}, 
        {Hue[0.4470438610211715, 1, 1], LineBox[{169, 170}]}, 
        {Hue[0.4909809957748204, 1, 1], LineBox[{170, 171}]}, 
        {Hue[0.5360342305294032, 1, 1], LineBox[{171, 172}]}, 
        {Hue[0.5821842310355752, 1, 1], LineBox[{172, 173}]}, 
        {Hue[0.6294105205595869, 1, 1], LineBox[{173, 174}]}, 
        {Hue[0.6776942738642395, 1, 1], LineBox[{174, 175}]}, 
        {Hue[0.7270182530399725, 1, 1], LineBox[{175, 176}]}, 
        {Hue[0.7773654671943058, 1, 1], LineBox[{176, 177}]}, 
        {Hue[0.8287197761195882, 1, 1], LineBox[{177, 178}]}, 
        {Hue[0.881066826233025, 1, 1], LineBox[{178, 179}]}, 
        {Hue[0.9343919814094939, 1, 1], LineBox[{179, 180}]}, 
        {Hue[0.9886810103437937, 1, 1], LineBox[{180, 181}]}, 
        {Hue[0.04392134041543094, 1, 1], LineBox[{181, 182}]}, 
        {Hue[0.10010062521493725`, 1, 1], LineBox[{182, 183}]}, 
        {Hue[0.1572060901918438, 1, 1], LineBox[{183, 184}]}, 
        {Hue[0.2152267037775803, 1, 1], LineBox[{184, 185}]}, 
        {Hue[0.2741518685733359, 1, 1], LineBox[{185, 186}]}, 
        {Hue[0.33396976725536087`, 1, 1], LineBox[{186, 187}]}, 
        {Hue[0.39467201119426304`, 1, 1], LineBox[{187, 188}]}, 
        {Hue[0.4562467952731411, 1, 1], LineBox[{188, 189}]}, 
        {Hue[0.5186871754839177, 1, 1], LineBox[{189, 190}]}, 
        {Hue[0.5819821910854881, 1, 1], LineBox[{190, 191}]}, 
        {Hue[0.6461249212559091, 1, 1], LineBox[{191, 192}]}, 
        {Hue[0.7111053868113398, 1, 1], LineBox[{192, 193}]}, 
        {Hue[0.7769163365513685, 1, 1], LineBox[{193, 194}]}, 
        {Hue[0.8435502311470209, 1, 1], LineBox[{194, 195}]}, 
        {Hue[0.9109997258954383, 1, 1], LineBox[{195, 196}]}, 
        {Hue[0.9792567724428458, 1, 1], LineBox[{196, 197}]}, 
        {Hue[0.048314893032024846`, 1, 1], LineBox[{197, 198}]}, 
        {Hue[0.11816659846490493`, 1, 1], LineBox[{198, 199}]}, 
        {Hue[0.18880748145033227`, 1, 1], LineBox[{199, 200}]}, 
        {Hue[0.2602281081067364, 1, 1], LineBox[{200, 201}]}, 
        {Hue[0.3324261821884439, 1, 1], LineBox[{201, 202}]}, 
        {Hue[0.4053919971395003, 1, 1], LineBox[{202, 203}]}, 
        {Hue[0.4791202853011942, 1, 1], LineBox[{203, 204}]}, 
        {Hue[0.5536061692980763, 1, 1], LineBox[{204, 205}]}, 
        {Hue[0.6288460163903289, 1, 1], LineBox[{205, 206}]}, 
        {Hue[0.704833005736574, 1, 1], LineBox[{206, 207}]}, 
        {Hue[0.7815601974591878, 1, 1], LineBox[{207, 208}]}, 
        {Hue[0.8590240975898622, 1, 1], LineBox[{208, 209}]}, 
        {Hue[0.9372201321618259, 1, 1], LineBox[{209, 210}]}, 
        {Hue[0.01614330869885718, 1, 1], LineBox[{210, 211}]}, 
        {Hue[0.09579005917412756, 1, 1], LineBox[{211, 212}]}, 
        {Hue[0.17615510022340541`, 1, 1], LineBox[{212, 213}]}, 
        {Hue[0.25722998241254924`, 1, 1], LineBox[{213, 214}]}, 
        {Hue[0.33901361326646806`, 1, 1], LineBox[{214, 215}]}, 
        {Hue[0.4215079868432836, 1, 1], LineBox[{215, 216}]}, 
        {Hue[0.5047018519445817, 1, 1], LineBox[{216, 217}]}, 
        {Hue[0.5885854873834779, 1, 1], LineBox[{217, 218}]}, 
        {Hue[0.6731690217175137, 1, 1], LineBox[{218, 219}]}, 
        {Hue[0.7584443045132689, 1, 1], LineBox[{219, 220}]}, 
        {Hue[0.8443996576338665, 1, 1], LineBox[{220, 221}]}, 
        {Hue[0.9310392668849811, 1, 1], LineBox[{221, 222}]}, 
        {Hue[0.01835608108419255, 1, 1], LineBox[{222, 223}]}, 
        {Hue[0.10634281440537748`, 1, 1], LineBox[{223, 224}]}, 
        {Hue[0.19500751539705116`, 1, 1], LineBox[{224, 225}]}, 
        {Hue[0.28434507288419764`, 1, 1], LineBox[{225, 226}]}, 
        {Hue[0.37433969563046116`, 1, 1], LineBox[{226, 227}]}, 
        {Hue[0.46499878959259816`, 1, 1], LineBox[{227, 228}]}, 
        {Hue[0.5563051143788902, 1, 1], LineBox[{228, 229}]}, 
        {Hue[0.6482599370262787, 1, 1], LineBox[{229, 230}]}, 
        {Hue[0.7408961732407697, 1, 1], LineBox[{230, 231}]}, 
        {Hue[0.8341759038366332, 1, 1], LineBox[{231, 232}]}, 
        {Hue[0.9280821964872943, 1, 1], LineBox[{232, 233}]}, 
        {Hue[0.02264091417215117, 1, 1], LineBox[{233, 234}]}, 
        {Hue[0.11784663968249516`, 1, 1], LineBox[{234, 235}]}, 
        {Hue[0.21369717943481406`, 1, 1], LineBox[{235, 236}]}, 
        {Hue[0.31015717277902966`, 1, 1], LineBox[{236, 237}]}, 
        {Hue[0.40721972489202457`, 1, 1], LineBox[{237, 238}]}, 
        {Hue[0.5049504162563396, 1, 1], LineBox[{238, 239}]}, 
        {Hue[0.6033365493871723, 1, 1], LineBox[{239, 240}]}, 
        {Hue[0.702329216982111, 1, 1], LineBox[{240, 241}]}, 
        {Hue[0.8019357595066028, 1, 1], LineBox[{241, 242}]}, 
        {Hue[0.9021269477589214, 1, 1], LineBox[{242, 243}]}, 
        {Hue[0.0028973395460384008`, 1, 1], LineBox[{243, 244}]}, 
        {Hue[0.10431645639875466`, 1, 1], LineBox[{244, 245}]}, 
        {Hue[0.20637297304353572`, 1, 1], LineBox[{245, 246}]}, 
        {Hue[0.30905173195330454`, 1, 1], LineBox[{246, 247}]}, 
        {Hue[0.41237782325228056`, 1, 1], LineBox[{247, 248}]}, 
        {Hue[0.5162096888172363, 1, 1], LineBox[{248, 249}]}, 
        {Hue[0.6204876662046706, 1, 1], LineBox[{249, 250}]}, 
        {Hue[0.7254548945286261, 1, 1], LineBox[{250, 251}]}}}},
     VertexColors->{
       Hue[0.6093005755620837, 1, 1], 
       Hue[0.5883030831109552, 1, 1], 
       Hue[0.34522610533488746`, 1, 1], 
       Hue[0.21884433245913584`, 1, 1], 
       Hue[0.05273737922507198, 1, 1], 
       Hue[0.0800429915891726, 1, 1], 
       Hue[0.8622231077078536, 1, 1], 
       Hue[0.9811433145234884, 1, 1], 
       Hue[0.7656439891671794, 1, 1], 
       Hue[0.575197751929063, 1, 1], 
       Hue[0.5610195544187918, 1, 1], 
       Hue[0.4383547897938635, 1, 1], 
       Hue[0.3897875310877631, 1, 1], 
       Hue[0.86236870675324, 1, 1], 
       Hue[0.15167136723202979`, 1, 1], 
       Hue[0.08718633620831152, 1, 1], 
       Hue[0.6976688277163468, 1, 1], 
       Hue[0.9140501077809731, 1, 1], 
       Hue[0.7775122878648932, 1, 1], 
       Hue[0.6750104754075937, 1, 1], 
       Hue[0.601540385332832, 1, 1], 
       Hue[0.599162923023778, 1, 1], 
       Hue[0.4732816613374724, 1, 1], 
       Hue[0.42205667261767177`, 1, 1], 
       Hue[0.283293118551893, 1, 1], 
       Hue[0.25052329442331206`, 1, 1], 
       Hue[0.16345913842696996`, 1, 1], 
       Hue[0.042713628129332624`, 1, 1], 
       Hue[0.9476834322091386, 1, 1], 
       Hue[0.8542985795573227, 1, 1], 
       Hue[0.762040334551915, 1, 1], 
       Hue[0.6562323051332618, 1, 1], 
       Hue[0.5611032812885219, 1, 1], 
       Hue[0.46177778309986633`, 1, 1], 
       Hue[0.36288473900116, 1, 1], 
       Hue[0.2613185162117361, 1, 1], 
       Hue[0.15485214106860662`, 1, 1], 
       Hue[0.05584266875309582, 1, 1], 
       Hue[0.9554896802057098, 1, 1], 
       Hue[0.8511929684959288, 1, 1], 
       Hue[0.752637322873851, 1, 1], 
       Hue[0.6511751745758734, 1, 1], 
       Hue[0.5544346510193291, 1, 1], 
       Hue[0.45624627578443544`, 1, 1], 
       Hue[0.3557371285223141, 1, 1], 
       Hue[0.25651723449211883`, 1, 1], 
       Hue[0.15787678371539723`, 1, 1], 
       Hue[0.05829216344985523, 1, 1], 
       Hue[0.9603759234494436, 1, 1], 
       Hue[0.8622537572154851, 1, 1], 
       Hue[0.7641289269589387, 1, 1], 
       Hue[0.6671796192899093, 1, 1], 
       Hue[0.570035522629933, 1, 1], 
       Hue[0.47298974794991105`, 1, 1], 
       Hue[0.37603519041829964`, 1, 1], 
       Hue[0.27935036622232096`, 1, 1], 
       Hue[0.18303412229352026`, 1, 1], 
       Hue[0.0868397578505655, 1, 1], 
       Hue[0.9908567670859162, 1, 1], 
       Hue[0.8951640675116295, 1, 1], 
       Hue[0.7997576994445039, 1, 1], 
       Hue[0.704677263864956, 1, 1], 
       Hue[0.6097791393361929, 1, 1], 
       Hue[0.51511325912361, 1, 1], 
       Hue[0.4206932535946379, 1, 1], 
       Hue[0.3264908912211963, 1, 1], 
       Hue[0.23247587761718255`, 1, 1], 
       Hue[0.1386847975223601, 1, 1], 
       Hue[0.04511404796464239, 1, 1], 
       Hue[0.9517719103841894, 1, 1], 
       Hue[0.8586357506433461, 1, 1], 
       Hue[0.7657480313793572, 1, 1], 
       Hue[0.6730497593471026, 1, 1], 
       Hue[0.5805547319386801, 1, 1], 
       Hue[0.4882651524400857, 1, 1], 
       Hue[0.39616028884303134`, 1, 1], 
       Hue[0.3042682849642356, 1, 1], 
       Hue[0.21256569508200795`, 1, 1], 
       Hue[0.12105676008102176`, 1, 1], 
       Hue[0.02974915153054283, 1, 1], 
       Hue[0.9386188542407726, 1, 1], 
       Hue[0.8476676926099618, 1, 1], 
       Hue[0.7568938711280264, 1, 1], 
       Hue[0.666294270544339, 1, 1], 
       Hue[0.5758728368553732, 1, 1], 
       Hue[0.4856387576042597, 1, 1], 
       Hue[0.39558025894890825`, 1, 1], 
       Hue[0.30568691036491824`, 1, 1], 
       Hue[0.21595156833836465`, 1, 1], 
       Hue[0.12637324682863935`, 1, 1], 
       Hue[0.03695515584535301, 1, 1], 
       Hue[0.9477192956297038, 1, 1], 
       Hue[0.858670444255665, 1, 1], 
       Hue[0.7698077509905155, 1, 1], 
       Hue[0.6811137464946615, 1, 1], 
       Hue[0.5925780318446903, 1, 1], 
       Hue[0.5041925162411358, 1, 1], 
       Hue[0.41598740282012486`, 1, 1], 
       Hue[0.3280008129379155, 1, 1], 
       Hue[0.24027124169676167`, 1, 1], 
       Hue[0.1528061272499971, 1, 1], 
       Hue[0.06558679955777666, 1, 1], 
       Hue[0.978589469438239, 1, 1], 
       Hue[0.891817591134329, 1, 1], 
       Hue[0.8053329095049395, 1, 1], 
       Hue[0.7192408998844428, 1, 1], 
       Hue[0.6336444994087578, 1, 1], 
       Hue[0.5485979186574695, 1, 1], 
       Hue[0.4640872331328644, 1, 1], 
       Hue[0.3800792298129656, 1, 1], 
       Hue[0.29658892069408954`, 1, 1], 
       Hue[0.2137498070449708, 1, 1], 
       Hue[0.1317985031971102, 1, 1], 
       Hue[0.05100279554728599, 1, 1], 
       Hue[0.9715544937306999, 1, 1], 
       Hue[0.8935117552806451, 1, 1], 
       Hue[0.8168378148577125, 1, 1], 
       Hue[0.741507451773556, 1, 1], 
       Hue[0.6676383743085668, 1, 1], 
       Hue[0.5955532908070316, 1, 1], 
       Hue[0.525747511026724, 1, 1], 
       Hue[0.4587611269785911, 1, 1], 
       Hue[0.3950341786702998, 1, 1], 
       Hue[0.3348239923730795, 1, 1], 
       Hue[0.278206263626647, 1, 1], 
       Hue[0.22512972585836025`, 1, 1], 
       Hue[0.17548723112854178`, 1, 1], 
       Hue[0.1291654748489493, 1, 1], 
       Hue[0.08607432464118504, 1, 1], 
       Hue[0.04615674856215593, 1, 1], 
       Hue[0.009383628358470269, 1, 1], 
       Hue[0.9757461448592223, 1, 1], 
       Hue[0.9452432063344988, 1, 1], 
       Hue[0.9178706741768455, 1, 1], 
       Hue[0.8936176561725973, 1, 1], 
       Hue[0.8724589000455976, 1, 1], 
       Hue[0.8543561829042435, 1, 1], 
       Hue[0.8392565806338904, 1, 1], 
       Hue[0.8270954879357122, 1, 1], 
       Hue[0.8177994855524553, 1, 1], 
       Hue[0.8112868389087454, 1, 1], 
       Hue[0.8074714251134552, 1, 1], 
       Hue[0.8062642398689808, 1, 1], 
       Hue[0.80757390034191, 1, 1], 
       Hue[0.8113117373256622, 1, 1], 
       Hue[0.8173886140816178, 1, 1], 
       Hue[0.8257180119328547, 1, 1], 
       Hue[0.8362167368039202, 1, 1], 
       Hue[0.8488048851013356, 1, 1], 
       Hue[0.8634056201434694, 1, 1], 
       Hue[0.8799464877098273, 1, 1], 
       Hue[0.8983581839384799, 1, 1], 
       Hue[0.9185754267824033, 1, 1], 
       Hue[0.9405376116358177, 1, 1], 
       Hue[0.9641858131985447, 1, 1], 
       Hue[0.9894658914039416, 1, 1], 
       Hue[0.016326442168588418`, 1, 1], 
       Hue[0.04471969994336976, 1, 1], 
       Hue[0.07460025787099636, 1, 1], 
       Hue[0.1059256870147042, 1, 1], 
       Hue[0.13865648207719755`, 1, 1], 
       Hue[0.17275516346313075`, 1, 1], 
       Hue[0.20818654705850076`, 1, 1], 
       Hue[0.24491776062038736`, 1, 1], 
       Hue[0.2829175843834493, 1, 1], 
       Hue[0.32215693783560934`, 1, 1], 
       Hue[0.36260803122091423`, 1, 1], 
       Hue[0.40424526696122776`, 1, 1], 
       Hue[0.4470438610211715, 1, 1], 
       Hue[0.4909809957748204, 1, 1], 
       Hue[0.5360342305294032, 1, 1], 
       Hue[0.5821842310355752, 1, 1], 
       Hue[0.6294105205595869, 1, 1], 
       Hue[0.6776942738642395, 1, 1], 
       Hue[0.7270182530399725, 1, 1], 
       Hue[0.7773654671943058, 1, 1], 
       Hue[0.8287197761195882, 1, 1], 
       Hue[0.881066826233025, 1, 1], 
       Hue[0.9343919814094939, 1, 1], 
       Hue[0.9886810103437937, 1, 1], 
       Hue[0.04392134041543094, 1, 1], 
       Hue[0.10010062521493725`, 1, 1], 
       Hue[0.1572060901918438, 1, 1], 
       Hue[0.2152267037775803, 1, 1], 
       Hue[0.2741518685733359, 1, 1], 
       Hue[0.33396976725536087`, 1, 1], 
       Hue[0.39467201119426304`, 1, 1], 
       Hue[0.4562467952731411, 1, 1], 
       Hue[0.5186871754839177, 1, 1], 
       Hue[0.5819821910854881, 1, 1], 
       Hue[0.6461249212559091, 1, 1], 
       Hue[0.7111053868113398, 1, 1], 
       Hue[0.7769163365513685, 1, 1], 
       Hue[0.8435502311470209, 1, 1], 
       Hue[0.9109997258954383, 1, 1], 
       Hue[0.9792567724428458, 1, 1], 
       Hue[0.048314893032024846`, 1, 1], 
       Hue[0.11816659846490493`, 1, 1], 
       Hue[0.18880748145033227`, 1, 1], 
       Hue[0.2602281081067364, 1, 1], 
       Hue[0.3324261821884439, 1, 1], 
       Hue[0.4053919971395003, 1, 1], 
       Hue[0.4791202853011942, 1, 1], 
       Hue[0.5536061692980763, 1, 1], 
       Hue[0.6288460163903289, 1, 1], 
       Hue[0.704833005736574, 1, 1], 
       Hue[0.7815601974591878, 1, 1], 
       Hue[0.8590240975898622, 1, 1], 
       Hue[0.9372201321618259, 1, 1], 
       Hue[0.01614330869885718, 1, 1], 
       Hue[0.09579005917412756, 1, 1], 
       Hue[0.17615510022340541`, 1, 1], 
       Hue[0.25722998241254924`, 1, 1], 
       Hue[0.33901361326646806`, 1, 1], 
       Hue[0.4215079868432836, 1, 1], 
       Hue[0.5047018519445817, 1, 1], 
       Hue[0.5885854873834779, 1, 1], 
       Hue[0.6731690217175137, 1, 1], 
       Hue[0.7584443045132689, 1, 1], 
       Hue[0.8443996576338665, 1, 1], 
       Hue[0.9310392668849811, 1, 1], 
       Hue[0.01835608108419255, 1, 1], 
       Hue[0.10634281440537748`, 1, 1], 
       Hue[0.19500751539705116`, 1, 1], 
       Hue[0.28434507288419764`, 1, 1], 
       Hue[0.37433969563046116`, 1, 1], 
       Hue[0.46499878959259816`, 1, 1], 
       Hue[0.5563051143788902, 1, 1], 
       Hue[0.6482599370262787, 1, 1], 
       Hue[0.7408961732407697, 1, 1], 
       Hue[0.8341759038366332, 1, 1], 
       Hue[0.9280821964872943, 1, 1], 
       Hue[0.02264091417215117, 1, 1], 
       Hue[0.11784663968249516`, 1, 1], 
       Hue[0.21369717943481406`, 1, 1], 
       Hue[0.31015717277902966`, 1, 1], 
       Hue[0.40721972489202457`, 1, 1], 
       Hue[0.5049504162563396, 1, 1], 
       Hue[0.6033365493871723, 1, 1], 
       Hue[0.702329216982111, 1, 1], 
       Hue[0.8019357595066028, 1, 1], 
       Hue[0.9021269477589214, 1, 1], 
       Hue[0.0028973395460384008`, 1, 1], 
       Hue[0.10431645639875466`, 1, 1], 
       Hue[0.20637297304353572`, 1, 1], 
       Hue[0.30905173195330454`, 1, 1], 
       Hue[0.41237782325228056`, 1, 1], 
       Hue[0.5162096888172363, 1, 1], 
       Hue[0.6204876662046706, 1, 1], 
       Hue[0.7254548945286261, 1, 1], 
       Hue[0.8312043329782834, 1, 1], 
       Hue[0.6093005755620837, 1, 1], 
       Hue[0.5883030831109552, 1, 1], 
       Hue[0.5883030831109552, 1, 1], 
       Hue[0.34522610533488746`, 1, 1], 
       Hue[0.34522610533488746`, 1, 1], 
       Hue[0.21884433245913584`, 1, 1], 
       Hue[0.21884433245913584`, 1, 1], 
       Hue[0.05273737922507198, 1, 1], 
       Hue[0.05273737922507198, 1, 1], 
       Hue[0.0800429915891726, 1, 1], 
       Hue[0.0800429915891726, 1, 1], 
       Hue[0.8622231077078536, 1, 1], 
       Hue[0.8622231077078536, 1, 1], 
       Hue[0.9811433145234884, 1, 1], 
       Hue[0.9811433145234884, 1, 1], 
       Hue[0.7656439891671794, 1, 1], 
       Hue[0.7656439891671794, 1, 1], 
       Hue[0.575197751929063, 1, 1], 
       Hue[0.575197751929063, 1, 1], 
       Hue[0.5610195544187918, 1, 1], 
       Hue[0.5610195544187918, 1, 1], 
       Hue[0.4383547897938635, 1, 1], 
       Hue[0.4383547897938635, 1, 1], 
       Hue[0.3897875310877631, 1, 1], 
       Hue[0.3897875310877631, 1, 1], 
       Hue[0.86236870675324, 1, 1], 
       Hue[0.86236870675324, 1, 1], 
       Hue[0.15167136723202979`, 1, 1], 
       Hue[0.15167136723202979`, 1, 1], 
       Hue[0.08718633620831152, 1, 1], 
       Hue[0.08718633620831152, 1, 1], 
       Hue[0.6976688277163468, 1, 1], 
       Hue[0.6976688277163468, 1, 1], 
       Hue[0.9140501077809731, 1, 1], 
       Hue[0.9140501077809731, 1, 1], 
       Hue[0.7775122878648932, 1, 1], 
       Hue[0.7775122878648932, 1, 1], 
       Hue[0.6750104754075937, 1, 1], 
       Hue[0.6750104754075937, 1, 1], 
       Hue[0.601540385332832, 1, 1], 
       Hue[0.601540385332832, 1, 1], 
       Hue[0.599162923023778, 1, 1], 
       Hue[0.599162923023778, 1, 1], 
       Hue[0.4732816613374724, 1, 1], 
       Hue[0.4732816613374724, 1, 1], 
       Hue[0.42205667261767177`, 1, 1], 
       Hue[0.42205667261767177`, 1, 1], 
       Hue[0.283293118551893, 1, 1], 
       Hue[0.283293118551893, 1, 1], 
       Hue[0.25052329442331206`, 1, 1], 
       Hue[0.25052329442331206`, 1, 1], 
       Hue[0.16345913842696996`, 1, 1], 
       Hue[0.16345913842696996`, 1, 1], 
       Hue[0.042713628129332624`, 1, 1], 
       Hue[0.042713628129332624`, 1, 1], 
       Hue[0.9476834322091386, 1, 1], 
       Hue[0.9476834322091386, 1, 1], 
       Hue[0.8542985795573227, 1, 1], 
       Hue[0.8542985795573227, 1, 1], 
       Hue[0.762040334551915, 1, 1], 
       Hue[0.762040334551915, 1, 1], 
       Hue[0.6562323051332618, 1, 1], 
       Hue[0.6562323051332618, 1, 1], 
       Hue[0.5611032812885219, 1, 1], 
       Hue[0.5611032812885219, 1, 1], 
       Hue[0.46177778309986633`, 1, 1], 
       Hue[0.46177778309986633`, 1, 1], 
       Hue[0.36288473900116, 1, 1], 
       Hue[0.36288473900116, 1, 1], 
       Hue[0.2613185162117361, 1, 1], 
       Hue[0.2613185162117361, 1, 1], 
       Hue[0.15485214106860662`, 1, 1], 
       Hue[0.15485214106860662`, 1, 1], 
       Hue[0.05584266875309582, 1, 1], 
       Hue[0.05584266875309582, 1, 1], 
       Hue[0.9554896802057098, 1, 1], 
       Hue[0.9554896802057098, 1, 1], 
       Hue[0.8511929684959288, 1, 1], 
       Hue[0.8511929684959288, 1, 1], 
       Hue[0.752637322873851, 1, 1], 
       Hue[0.752637322873851, 1, 1], 
       Hue[0.6511751745758734, 1, 1], 
       Hue[0.6511751745758734, 1, 1], 
       Hue[0.5544346510193291, 1, 1], 
       Hue[0.5544346510193291, 1, 1], 
       Hue[0.45624627578443544`, 1, 1], 
       Hue[0.45624627578443544`, 1, 1], 
       Hue[0.3557371285223141, 1, 1], 
       Hue[0.3557371285223141, 1, 1], 
       Hue[0.25651723449211883`, 1, 1], 
       Hue[0.25651723449211883`, 1, 1], 
       Hue[0.15787678371539723`, 1, 1], 
       Hue[0.15787678371539723`, 1, 1], 
       Hue[0.05829216344985523, 1, 1], 
       Hue[0.05829216344985523, 1, 1], 
       Hue[0.9603759234494436, 1, 1], 
       Hue[0.9603759234494436, 1, 1], 
       Hue[0.8622537572154851, 1, 1], 
       Hue[0.8622537572154851, 1, 1], 
       Hue[0.7641289269589387, 1, 1], 
       Hue[0.7641289269589387, 1, 1], 
       Hue[0.6671796192899093, 1, 1], 
       Hue[0.6671796192899093, 1, 1], 
       Hue[0.570035522629933, 1, 1], 
       Hue[0.570035522629933, 1, 1], 
       Hue[0.47298974794991105`, 1, 1], 
       Hue[0.47298974794991105`, 1, 1], 
       Hue[0.37603519041829964`, 1, 1], 
       Hue[0.37603519041829964`, 1, 1], 
       Hue[0.27935036622232096`, 1, 1], 
       Hue[0.27935036622232096`, 1, 1], 
       Hue[0.18303412229352026`, 1, 1], 
       Hue[0.18303412229352026`, 1, 1], 
       Hue[0.0868397578505655, 1, 1], 
       Hue[0.0868397578505655, 1, 1], 
       Hue[0.9908567670859162, 1, 1], 
       Hue[0.9908567670859162, 1, 1], 
       Hue[0.8951640675116295, 1, 1], 
       Hue[0.8951640675116295, 1, 1], 
       Hue[0.7997576994445039, 1, 1], 
       Hue[0.7997576994445039, 1, 1], 
       Hue[0.704677263864956, 1, 1], 
       Hue[0.704677263864956, 1, 1], 
       Hue[0.6097791393361929, 1, 1], 
       Hue[0.6097791393361929, 1, 1], 
       Hue[0.51511325912361, 1, 1], 
       Hue[0.51511325912361, 1, 1], 
       Hue[0.4206932535946379, 1, 1], 
       Hue[0.4206932535946379, 1, 1], 
       Hue[0.3264908912211963, 1, 1], 
       Hue[0.3264908912211963, 1, 1], 
       Hue[0.23247587761718255`, 1, 1], 
       Hue[0.23247587761718255`, 1, 1], 
       Hue[0.1386847975223601, 1, 1], 
       Hue[0.1386847975223601, 1, 1], 
       Hue[0.04511404796464239, 1, 1], 
       Hue[0.04511404796464239, 1, 1], 
       Hue[0.9517719103841894, 1, 1], 
       Hue[0.9517719103841894, 1, 1], 
       Hue[0.8586357506433461, 1, 1], 
       Hue[0.8586357506433461, 1, 1], 
       Hue[0.7657480313793572, 1, 1], 
       Hue[0.7657480313793572, 1, 1], 
       Hue[0.6730497593471026, 1, 1], 
       Hue[0.6730497593471026, 1, 1], 
       Hue[0.5805547319386801, 1, 1], 
       Hue[0.5805547319386801, 1, 1], 
       Hue[0.4882651524400857, 1, 1], 
       Hue[0.4882651524400857, 1, 1], 
       Hue[0.39616028884303134`, 1, 1], 
       Hue[0.39616028884303134`, 1, 1], 
       Hue[0.3042682849642356, 1, 1], 
       Hue[0.3042682849642356, 1, 1], 
       Hue[0.21256569508200795`, 1, 1], 
       Hue[0.21256569508200795`, 1, 1], 
       Hue[0.12105676008102176`, 1, 1], 
       Hue[0.12105676008102176`, 1, 1], 
       Hue[0.02974915153054283, 1, 1], 
       Hue[0.02974915153054283, 1, 1], 
       Hue[0.9386188542407726, 1, 1], 
       Hue[0.9386188542407726, 1, 1], 
       Hue[0.8476676926099618, 1, 1], 
       Hue[0.8476676926099618, 1, 1], 
       Hue[0.7568938711280264, 1, 1], 
       Hue[0.7568938711280264, 1, 1], 
       Hue[0.666294270544339, 1, 1], 
       Hue[0.666294270544339, 1, 1], 
       Hue[0.5758728368553732, 1, 1], 
       Hue[0.5758728368553732, 1, 1], 
       Hue[0.4856387576042597, 1, 1], 
       Hue[0.4856387576042597, 1, 1], 
       Hue[0.39558025894890825`, 1, 1], 
       Hue[0.39558025894890825`, 1, 1], 
       Hue[0.30568691036491824`, 1, 1], 
       Hue[0.30568691036491824`, 1, 1], 
       Hue[0.21595156833836465`, 1, 1], 
       Hue[0.21595156833836465`, 1, 1], 
       Hue[0.12637324682863935`, 1, 1], 
       Hue[0.12637324682863935`, 1, 1], 
       Hue[0.03695515584535301, 1, 1], 
       Hue[0.03695515584535301, 1, 1], 
       Hue[0.9477192956297038, 1, 1], 
       Hue[0.9477192956297038, 1, 1], 
       Hue[0.858670444255665, 1, 1], 
       Hue[0.858670444255665, 1, 1], 
       Hue[0.7698077509905155, 1, 1], 
       Hue[0.7698077509905155, 1, 1], 
       Hue[0.6811137464946615, 1, 1], 
       Hue[0.6811137464946615, 1, 1], 
       Hue[0.5925780318446903, 1, 1], 
       Hue[0.5925780318446903, 1, 1], 
       Hue[0.5041925162411358, 1, 1], 
       Hue[0.5041925162411358, 1, 1], 
       Hue[0.41598740282012486`, 1, 1], 
       Hue[0.41598740282012486`, 1, 1], 
       Hue[0.3280008129379155, 1, 1], 
       Hue[0.3280008129379155, 1, 1], 
       Hue[0.24027124169676167`, 1, 1], 
       Hue[0.24027124169676167`, 1, 1], 
       Hue[0.1528061272499971, 1, 1], 
       Hue[0.1528061272499971, 1, 1], 
       Hue[0.06558679955777666, 1, 1], 
       Hue[0.06558679955777666, 1, 1], 
       Hue[0.978589469438239, 1, 1], 
       Hue[0.978589469438239, 1, 1], 
       Hue[0.891817591134329, 1, 1], 
       Hue[0.891817591134329, 1, 1], 
       Hue[0.8053329095049395, 1, 1], 
       Hue[0.8053329095049395, 1, 1], 
       Hue[0.7192408998844428, 1, 1], 
       Hue[0.7192408998844428, 1, 1], 
       Hue[0.6336444994087578, 1, 1], 
       Hue[0.6336444994087578, 1, 1], 
       Hue[0.5485979186574695, 1, 1], 
       Hue[0.5485979186574695, 1, 1], 
       Hue[0.4640872331328644, 1, 1], 
       Hue[0.4640872331328644, 1, 1], 
       Hue[0.3800792298129656, 1, 1], 
       Hue[0.3800792298129656, 1, 1], 
       Hue[0.29658892069408954`, 1, 1], 
       Hue[0.29658892069408954`, 1, 1], 
       Hue[0.2137498070449708, 1, 1], 
       Hue[0.2137498070449708, 1, 1], 
       Hue[0.1317985031971102, 1, 1], 
       Hue[0.1317985031971102, 1, 1], 
       Hue[0.05100279554728599, 1, 1], 
       Hue[0.05100279554728599, 1, 1], 
       Hue[0.9715544937306999, 1, 1], 
       Hue[0.9715544937306999, 1, 1], 
       Hue[0.8935117552806451, 1, 1], 
       Hue[0.8935117552806451, 1, 1], 
       Hue[0.8168378148577125, 1, 1], 
       Hue[0.8168378148577125, 1, 1], 
       Hue[0.741507451773556, 1, 1], 
       Hue[0.741507451773556, 1, 1], 
       Hue[0.6676383743085668, 1, 1], 
       Hue[0.6676383743085668, 1, 1], 
       Hue[0.5955532908070316, 1, 1], 
       Hue[0.5955532908070316, 1, 1], 
       Hue[0.525747511026724, 1, 1], 
       Hue[0.525747511026724, 1, 1], 
       Hue[0.4587611269785911, 1, 1], 
       Hue[0.4587611269785911, 1, 1], 
       Hue[0.3950341786702998, 1, 1], 
       Hue[0.3950341786702998, 1, 1], 
       Hue[0.3348239923730795, 1, 1], 
       Hue[0.3348239923730795, 1, 1], 
       Hue[0.278206263626647, 1, 1], 
       Hue[0.278206263626647, 1, 1], 
       Hue[0.22512972585836025`, 1, 1], 
       Hue[0.22512972585836025`, 1, 1], 
       Hue[0.17548723112854178`, 1, 1], 
       Hue[0.17548723112854178`, 1, 1], 
       Hue[0.1291654748489493, 1, 1], 
       Hue[0.1291654748489493, 1, 1], 
       Hue[0.08607432464118504, 1, 1], 
       Hue[0.08607432464118504, 1, 1], 
       Hue[0.04615674856215593, 1, 1], 
       Hue[0.04615674856215593, 1, 1], 
       Hue[0.009383628358470269, 1, 1], 
       Hue[0.009383628358470269, 1, 1], 
       Hue[0.9757461448592223, 1, 1], 
       Hue[0.9757461448592223, 1, 1], 
       Hue[0.9452432063344988, 1, 1], 
       Hue[0.9452432063344988, 1, 1], 
       Hue[0.9178706741768455, 1, 1], 
       Hue[0.9178706741768455, 1, 1], 
       Hue[0.8936176561725973, 1, 1], 
       Hue[0.8936176561725973, 1, 1], 
       Hue[0.8724589000455976, 1, 1], 
       Hue[0.8724589000455976, 1, 1], 
       Hue[0.8543561829042435, 1, 1], 
       Hue[0.8543561829042435, 1, 1], 
       Hue[0.8392565806338904, 1, 1], 
       Hue[0.8392565806338904, 1, 1], 
       Hue[0.8270954879357122, 1, 1], 
       Hue[0.8270954879357122, 1, 1], 
       Hue[0.8177994855524553, 1, 1], 
       Hue[0.8177994855524553, 1, 1], 
       Hue[0.8112868389087454, 1, 1], 
       Hue[0.8112868389087454, 1, 1], 
       Hue[0.8074714251134552, 1, 1], 
       Hue[0.8074714251134552, 1, 1], 
       Hue[0.8062642398689808, 1, 1], 
       Hue[0.8062642398689808, 1, 1], 
       Hue[0.80757390034191, 1, 1], 
       Hue[0.80757390034191, 1, 1], 
       Hue[0.8113117373256622, 1, 1], 
       Hue[0.8113117373256622, 1, 1], 
       Hue[0.8173886140816178, 1, 1], 
       Hue[0.8173886140816178, 1, 1], 
       Hue[0.8257180119328547, 1, 1], 
       Hue[0.8257180119328547, 1, 1], 
       Hue[0.8362167368039202, 1, 1], 
       Hue[0.8362167368039202, 1, 1], 
       Hue[0.8488048851013356, 1, 1], 
       Hue[0.8488048851013356, 1, 1], 
       Hue[0.8634056201434694, 1, 1], 
       Hue[0.8634056201434694, 1, 1], 
       Hue[0.8799464877098273, 1, 1], 
       Hue[0.8799464877098273, 1, 1], 
       Hue[0.8983581839384799, 1, 1], 
       Hue[0.8983581839384799, 1, 1], 
       Hue[0.9185754267824033, 1, 1], 
       Hue[0.9185754267824033, 1, 1], 
       Hue[0.9405376116358177, 1, 1], 
       Hue[0.9405376116358177, 1, 1], 
       Hue[0.9641858131985447, 1, 1], 
       Hue[0.9641858131985447, 1, 1], 
       Hue[0.9894658914039416, 1, 1], 
       Hue[0.9894658914039416, 1, 1], 
       Hue[0.016326442168588418`, 1, 1], 
       Hue[0.016326442168588418`, 1, 1], 
       Hue[0.04471969994336976, 1, 1], 
       Hue[0.04471969994336976, 1, 1], 
       Hue[0.07460025787099636, 1, 1], 
       Hue[0.07460025787099636, 1, 1], 
       Hue[0.1059256870147042, 1, 1], 
       Hue[0.1059256870147042, 1, 1], 
       Hue[0.13865648207719755`, 1, 1], 
       Hue[0.13865648207719755`, 1, 1], 
       Hue[0.17275516346313075`, 1, 1], 
       Hue[0.17275516346313075`, 1, 1], 
       Hue[0.20818654705850076`, 1, 1], 
       Hue[0.20818654705850076`, 1, 1], 
       Hue[0.24491776062038736`, 1, 1], 
       Hue[0.24491776062038736`, 1, 1], 
       Hue[0.2829175843834493, 1, 1], 
       Hue[0.2829175843834493, 1, 1], 
       Hue[0.32215693783560934`, 1, 1], 
       Hue[0.32215693783560934`, 1, 1], 
       Hue[0.36260803122091423`, 1, 1], 
       Hue[0.36260803122091423`, 1, 1], 
       Hue[0.40424526696122776`, 1, 1], 
       Hue[0.40424526696122776`, 1, 1], 
       Hue[0.4470438610211715, 1, 1], 
       Hue[0.4470438610211715, 1, 1], 
       Hue[0.4909809957748204, 1, 1], 
       Hue[0.4909809957748204, 1, 1], 
       Hue[0.5360342305294032, 1, 1], 
       Hue[0.5360342305294032, 1, 1], 
       Hue[0.5821842310355752, 1, 1], 
       Hue[0.5821842310355752, 1, 1], 
       Hue[0.6294105205595869, 1, 1], 
       Hue[0.6294105205595869, 1, 1], 
       Hue[0.6776942738642395, 1, 1], 
       Hue[0.6776942738642395, 1, 1], 
       Hue[0.7270182530399725, 1, 1], 
       Hue[0.7270182530399725, 1, 1], 
       Hue[0.7773654671943058, 1, 1], 
       Hue[0.7773654671943058, 1, 1], 
       Hue[0.8287197761195882, 1, 1], 
       Hue[0.8287197761195882, 1, 1], 
       Hue[0.881066826233025, 1, 1], 
       Hue[0.881066826233025, 1, 1], 
       Hue[0.9343919814094939, 1, 1], 
       Hue[0.9343919814094939, 1, 1], 
       Hue[0.9886810103437937, 1, 1], 
       Hue[0.9886810103437937, 1, 1], 
       Hue[0.04392134041543094, 1, 1], 
       Hue[0.04392134041543094, 1, 1], 
       Hue[0.10010062521493725`, 1, 1], 
       Hue[0.10010062521493725`, 1, 1], 
       Hue[0.1572060901918438, 1, 1], 
       Hue[0.1572060901918438, 1, 1], 
       Hue[0.2152267037775803, 1, 1], 
       Hue[0.2152267037775803, 1, 1], 
       Hue[0.2741518685733359, 1, 1], 
       Hue[0.2741518685733359, 1, 1], 
       Hue[0.33396976725536087`, 1, 1], 
       Hue[0.33396976725536087`, 1, 1], 
       Hue[0.39467201119426304`, 1, 1], 
       Hue[0.39467201119426304`, 1, 1], 
       Hue[0.4562467952731411, 1, 1], 
       Hue[0.4562467952731411, 1, 1], 
       Hue[0.5186871754839177, 1, 1], 
       Hue[0.5186871754839177, 1, 1], 
       Hue[0.5819821910854881, 1, 1], 
       Hue[0.5819821910854881, 1, 1], 
       Hue[0.6461249212559091, 1, 1], 
       Hue[0.6461249212559091, 1, 1], 
       Hue[0.7111053868113398, 1, 1], 
       Hue[0.7111053868113398, 1, 1], 
       Hue[0.7769163365513685, 1, 1], 
       Hue[0.7769163365513685, 1, 1], 
       Hue[0.8435502311470209, 1, 1], 
       Hue[0.8435502311470209, 1, 1], 
       Hue[0.9109997258954383, 1, 1], 
       Hue[0.9109997258954383, 1, 1], 
       Hue[0.9792567724428458, 1, 1], 
       Hue[0.9792567724428458, 1, 1], 
       Hue[0.048314893032024846`, 1, 1], 
       Hue[0.048314893032024846`, 1, 1], 
       Hue[0.11816659846490493`, 1, 1], 
       Hue[0.11816659846490493`, 1, 1], 
       Hue[0.18880748145033227`, 1, 1], 
       Hue[0.18880748145033227`, 1, 1], 
       Hue[0.2602281081067364, 1, 1], 
       Hue[0.2602281081067364, 1, 1], 
       Hue[0.3324261821884439, 1, 1], 
       Hue[0.3324261821884439, 1, 1], 
       Hue[0.4053919971395003, 1, 1], 
       Hue[0.4053919971395003, 1, 1], 
       Hue[0.4791202853011942, 1, 1], 
       Hue[0.4791202853011942, 1, 1], 
       Hue[0.5536061692980763, 1, 1], 
       Hue[0.5536061692980763, 1, 1], 
       Hue[0.6288460163903289, 1, 1], 
       Hue[0.6288460163903289, 1, 1], 
       Hue[0.704833005736574, 1, 1], 
       Hue[0.704833005736574, 1, 1], 
       Hue[0.7815601974591878, 1, 1], 
       Hue[0.7815601974591878, 1, 1], 
       Hue[0.8590240975898622, 1, 1], 
       Hue[0.8590240975898622, 1, 1], 
       Hue[0.9372201321618259, 1, 1], 
       Hue[0.9372201321618259, 1, 1], 
       Hue[0.01614330869885718, 1, 1], 
       Hue[0.01614330869885718, 1, 1], 
       Hue[0.09579005917412756, 1, 1], 
       Hue[0.09579005917412756, 1, 1], 
       Hue[0.17615510022340541`, 1, 1], 
       Hue[0.17615510022340541`, 1, 1], 
       Hue[0.25722998241254924`, 1, 1], 
       Hue[0.25722998241254924`, 1, 1], 
       Hue[0.33901361326646806`, 1, 1], 
       Hue[0.33901361326646806`, 1, 1], 
       Hue[0.4215079868432836, 1, 1], 
       Hue[0.4215079868432836, 1, 1], 
       Hue[0.5047018519445817, 1, 1], 
       Hue[0.5047018519445817, 1, 1], 
       Hue[0.5885854873834779, 1, 1], 
       Hue[0.5885854873834779, 1, 1], 
       Hue[0.6731690217175137, 1, 1], 
       Hue[0.6731690217175137, 1, 1], 
       Hue[0.7584443045132689, 1, 1], 
       Hue[0.7584443045132689, 1, 1], 
       Hue[0.8443996576338665, 1, 1], 
       Hue[0.8443996576338665, 1, 1], 
       Hue[0.9310392668849811, 1, 1], 
       Hue[0.9310392668849811, 1, 1], 
       Hue[0.01835608108419255, 1, 1], 
       Hue[0.01835608108419255, 1, 1], 
       Hue[0.10634281440537748`, 1, 1], 
       Hue[0.10634281440537748`, 1, 1], 
       Hue[0.19500751539705116`, 1, 1], 
       Hue[0.19500751539705116`, 1, 1], 
       Hue[0.28434507288419764`, 1, 1], 
       Hue[0.28434507288419764`, 1, 1], 
       Hue[0.37433969563046116`, 1, 1], 
       Hue[0.37433969563046116`, 1, 1], 
       Hue[0.46499878959259816`, 1, 1], 
       Hue[0.46499878959259816`, 1, 1], 
       Hue[0.5563051143788902, 1, 1], 
       Hue[0.5563051143788902, 1, 1], 
       Hue[0.6482599370262787, 1, 1], 
       Hue[0.6482599370262787, 1, 1], 
       Hue[0.7408961732407697, 1, 1], 
       Hue[0.7408961732407697, 1, 1], 
       Hue[0.8341759038366332, 1, 1], 
       Hue[0.8341759038366332, 1, 1], 
       Hue[0.9280821964872943, 1, 1], 
       Hue[0.9280821964872943, 1, 1], 
       Hue[0.02264091417215117, 1, 1], 
       Hue[0.02264091417215117, 1, 1], 
       Hue[0.11784663968249516`, 1, 1], 
       Hue[0.11784663968249516`, 1, 1], 
       Hue[0.21369717943481406`, 1, 1], 
       Hue[0.21369717943481406`, 1, 1], 
       Hue[0.31015717277902966`, 1, 1], 
       Hue[0.31015717277902966`, 1, 1], 
       Hue[0.40721972489202457`, 1, 1], 
       Hue[0.40721972489202457`, 1, 1], 
       Hue[0.5049504162563396, 1, 1], 
       Hue[0.5049504162563396, 1, 1], 
       Hue[0.6033365493871723, 1, 1], 
       Hue[0.6033365493871723, 1, 1], 
       Hue[0.702329216982111, 1, 1], 
       Hue[0.702329216982111, 1, 1], 
       Hue[0.8019357595066028, 1, 1], 
       Hue[0.8019357595066028, 1, 1], 
       Hue[0.9021269477589214, 1, 1], 
       Hue[0.9021269477589214, 1, 1], 
       Hue[0.0028973395460384008`, 1, 1], 
       Hue[0.0028973395460384008`, 1, 1], 
       Hue[0.10431645639875466`, 1, 1], 
       Hue[0.10431645639875466`, 1, 1], 
       Hue[0.20637297304353572`, 1, 1], 
       Hue[0.20637297304353572`, 1, 1], 
       Hue[0.30905173195330454`, 1, 1], 
       Hue[0.30905173195330454`, 1, 1], 
       Hue[0.41237782325228056`, 1, 1], 
       Hue[0.41237782325228056`, 1, 1], 
       Hue[0.5162096888172363, 1, 1], 
       Hue[0.5162096888172363, 1, 1], 
       Hue[0.6204876662046706, 1, 1], 
       Hue[0.6204876662046706, 1, 1], 
       Hue[0.7254548945286261, 1, 1], 
       Hue[0.7254548945286261, 1, 1], 
       Hue[0.8312043329782834, 1, 1]}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->{True, None},
    AxesOrigin->{0, 0},
    Epilog->
     LineBox[{{1, 4.465961166467968*^-7}, {2, 7.956089980721662*^-7}, {
       3, 1.2080462257273376`*^-6}, {4, 1.068437575775247*^-6}, {
       5, 5.783416278576244*^-7}, {6, 7.699168023451992*^-7}, {
       7, 6.049501091558932*^-7}, {8, 4.593198849520388*^-7}, {
       9, 1.3289285724059744`*^-6}, {10, 6.452925093537202*^-7}, {
       11, 5.323775076689481*^-7}, {12, 2.661859769000134*^-7}, {
       13, 9.908697260728364*^-7}, {14, 2.8771192160845204`*^-7}, {
       15, 5.49455345501363*^-7}, {16, 2.875210030077873*^-7}, {
       17, 3.783729415361154*^-7}, {18, 7.2092942490568*^-7}, {
       19, 5.429028169661039*^-7}, {20, 8.480552527456583*^-7}, {
       21, 9.928841664717074*^-7}, {22, 5.090482571075096*^-7}, {
       23, 1.2956855266877236`*^-6}, {24, 1.6052093239676597`*^-6}, {
       25, 1.3821741390300245`*^-6}, {26, 2.079933766701361*^-6}, {
       27, 2.6198680558333978`*^-6}, {28, 3.4199793959022483`*^-6}, {
       29, 4.5356718587931005`*^-6}, {30, 4.990965537211067*^-6}, {
       31, 6.247978888870258*^-6}, {32, 8.144676894575314*^-6}, {
       33, 9.09621745593498*^-6}, {34, 0.00001103982832802499}, {
       35, 0.00001418603554700873}, {36, 0.000016210381076184154`}, {
       37, 0.000019943012021794054`}, {38, 0.000024022313091069023`}, {
       39, 0.000029033335269879555`}, {40, 0.00003552562486850368}, {
       41, 0.000042287283084219806`}, {42, 0.000050465723101957854`}, {
       43, 0.000060375069789874526`}, {44, 0.00007231518735858472}, {
       45, 0.00008646481315146338}, {46, 0.0001023073509581926}, {
       47, 0.00012183229038690085`}, {48, 0.00014332374050077062`}, {
       49, 0.00017013987185814292`}, {50, 0.00020106627376102645`}, {
       51, 0.00023618233001864057`}, {52, 0.0002783877043128726}, {
       53, 0.0003276220413634956}, {54, 0.0003843109066576748}, {
       55, 0.0004498940119490794}, {56, 0.0005248824613025863}, {
       57, 0.0006121304561945996}, {58, 0.0007132589928315171}, {
       59, 0.0008284807997185334}, {60, 0.0009618688217196192}, {
       61, 0.0011138459555453213`}, {62, 0.0012879124808189448`}, {
       63, 0.001487112254795767}, {64, 0.0017131633586123921`}, {
       65, 0.0019699980085649185`}, {66, 0.0022621774959527387`}, {
       67, 0.0025925151253452373`}, {68, 0.0029666619252550395`}, {
       69, 0.0033888531319688675`}, {70, 0.003863913236082229}, {
       71, 0.004398905981830261}, {72, 0.0049981488505563545`}, {
       73, 0.005669303481718499}, {74, 0.006418870247413546}, {
       75, 0.007254137956871396}, {76, 0.008183309315890376}, {
       77, 0.009215234568250226}, {78, 0.010358846712774742`}, {
       79, 0.01162279081313599}, {80, 0.013016926180056872`}, {
       81, 0.014550518226422011`}, {82, 0.01623478918132964}, {
       83, 0.018080280439427037`}, {84, 0.020099078029177986`}, {
       85, 0.02230204409530182}, {86, 0.024698638050735213`}, {
       87, 0.02730082934138697}, {88, 0.030115977937487996`}, {
       89, 0.03315764653793455}, {90, 0.03643837639987169}, {
       91, 0.03997007899404878}, {92, 0.04376345263026441}, {
       93, 0.04782469749544438}, {94, 0.05215550130440996}, {
       95, 0.05675983829722015}, {96, 0.061645550241447}, {
       97, 0.06682722152150794}, {98, 0.07232018000639912}, {
       99, 0.07813122502774716}, {100, 0.08425353955078142}, {
       101, 0.09066587560436452}, {102, 0.0973511213608786}, {
       103, 0.10431155230083176`}, {104, 0.11157561285659069`}, {
       105, 0.11918352951669064`}, {106, 0.12715345143951956`}, {
       107, 0.13545342025215026`}, {108, 0.14401189217854204`}, {
       109, 0.15275824670694696`}, {110, 0.16169236975100565`}, {
       111, 0.17090715568257578`}, {112, 0.18055095169812352`}, {
       113, 0.19073222664377687`}, {114, 0.20141742888704986`}, {
       115, 0.21243157765857199`}, {116, 0.22355089824579477`}, {
       117, 0.23467241982895418`}, {118, 0.24594363426387572`}, {
       119, 0.25775687988104573`}, {120, 0.2706069451816341}, {
       121, 0.28484427188009753`}, {122, 0.3004896800197246}, {
       123, 0.31717859539988275`}, {124, 0.33427521306359387`}, {
       125, 0.35106287785100576`}, {126, 0.36691675930138756`}, {
       127, 0.3814179673828288}, {128, 0.39437344029634863`}, {
       129, 0.40579020997355647`}, {130, 0.41581185940282833`}, {
       131, 0.42464925596550906`}, {132, 0.43253455180840056`}, {
       133, 0.4396808372154018}, {134, 0.4462618613503635}, {
       135, 0.4524082082767881}, {136, 0.45820952320515523`}, {
       137, 0.46371594332370797`}, {138, 0.46895300495824227`}, {
       139, 0.4739211809053794}, {140, 0.4786091120732276}, {
       141, 0.4829951729187367}, {142, 0.48704818160531754`}, {
       143, 0.4907365221711765}, {144, 0.49402681247525965`}, {
       145, 0.49688457539343917`}, {146, 0.4992784950098724}, {
       147, 0.5011769092625394}, {148, 0.5025519359075454}, {
       149, 0.503379309609334}, {150, 0.5036369734614191}, {
       151, 0.5033067946044762}, {152, 0.502375472466543}, {
       153, 0.5008309046922802}, {154, 0.49866534275152036`}, {
       155, 0.49587637025536624`}, {156, 0.49246489846108077`}, {
       157, 0.4884334308206991}, {158, 0.48378952558757127`}, {
       159, 0.4785432437504738}, {160, 0.4727106531708105}, {
       161, 0.46630814214898114`}, {162, 0.4593524595846046}, {
       163, 0.4518710858115378}, {164, 0.4438852721467415}, {
       165, 0.43542381962426596`}, {166, 0.42651748287801783`}, {
       167, 0.41719437991439223`}, {168, 0.40749117788610206`}, {
       169, 0.39743840845142464`}, {170, 0.3870722471142686}, {
       171, 0.3764296358984598}, {172, 0.3655451509810529}, {
       173, 0.35445639036631627`}, {174, 0.34319919555697326`}, {
       175, 0.33181037438936445`}, {176, 0.32032657501523804`}, {
       177, 0.30878196432282806`}, {178, 0.29721233639493516`}, {
       179, 0.2856504017606239}, {180, 0.27412895404080057`}, {
       181, 0.2626801510002149}, {182, 0.25133258410027887`}, {
       183, 0.2401131115826976}, {184, 0.22905106780074028`}, {
       185, 0.218168219911465}, {186, 0.20748991653301638`}, {
       187, 0.19703445152676527`}, {188, 0.186822890463749}, {
       189, 0.17687185602353517`}, {190, 0.16719540737274122`}, {
       191, 0.15780729351444053`}, {192, 0.14871788106288572`}, {
       193, 0.13993771818468892`}, {194, 0.13147425653188277`}, {
       195, 0.12333253909428953`}, {196, 0.11551711256741437`}, {
       197, 0.10802912396361104`}, {198, 0.10087039053467728`}, {
       199, 0.09403950368811381}, {200, 0.0875349254452988}, {
       201, 0.08135384426409242}, {202, 0.07549051693968799}, {
       203, 0.06994069338837774}, {204, 0.06469741521537807}, {
       205, 0.059753169951092956`}, {206, 0.05509900599724413}, {
       207, 0.05072712555093638}, {208, 0.04662850209658327}, {
       209, 0.04279299963219529}, {210, 0.03921058983260664}, {
       211, 0.03587082165381312}, {212, 0.03276261741917028}, {
       213, 0.02987573659595402}, {214, 0.027200285753796166`}, {
       215, 0.024724577684119318`}, {216, 0.022436967451613533`}, {
       217, 0.02032862304938372}, {218, 0.01838917013977974}, {
       219, 0.016606933025539533`}, {220, 0.014973020854909832`}, {
       221, 0.013477982160610285`}, {222, 0.012112063805714723`}, {
       223, 0.010866943725446975`}, {224, 0.009734101423890442}, {
       225, 0.008704466483636833}, {226, 0.007770765960616775}, {
       227, 0.006925721481079195}, {228, 0.006162027251691595}, {
       229, 0.005474153459128695}, {230, 0.004854976216746123}, {
       231, 0.0042978386710745935`}, {232, 0.0037984637658225805`}, {
       233, 0.003351650941346346}, {234, 0.002952316082461568}, {
       235, 0.0025959261289751983`}, {236, 0.002278380309434855}, {
       237, 0.00199669948742157}, {238, 0.0017471936926599964`}, {
       239, 0.001525909042535809}, {240, 0.0013301677381813964`}, {
       241, 0.0011574625176969822`}, {242, 0.0010053340615007662`}, {
       243, 0.0008719166539826102}, {244, 0.0007550806587226983}, {
       245, 0.0006526897535448405}, {246, 0.0005631967025225873}, {
       247, 0.00048494635841474253`}, {248, 0.0004165759108779093}, {
       249, 0.0003574616153745001}, {250, 0.0003065509025968455}, {
       251, 0.0002623382601653005}}],
    Frame->True,
    FrameTicks->{{{1., 
        FormBox[
         RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
        FormBox[
         RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
        FormBox[
         RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
        FormBox[
         RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
        FormBox["0", TraditionalForm]}, {157.25, 
        FormBox["2.5`", TraditionalForm]}, {188.5, 
        FormBox["5", TraditionalForm]}, {219.75, 
        FormBox["7.5`", TraditionalForm]}, {251., 
        FormBox["10", TraditionalForm]}}, Automatic, None, None},
    PlotRange->{-0.1, 1.1},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], Automatic}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF1nlYlOUaBvAvXMMNXJAIbVwyRFIUESGVW0AWBdkEQUG+AWaAYQC3sNz6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     "], {
      {GrayLevel[0.5], EdgeForm[None], 
       GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ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         "]]]}, {{}, {}, {
        {Hue[0.974286712374846, 1, 1], LineBox[{1, 2}]}, 
        {Hue[0.08446566071998841, 1, 1], LineBox[{2, 3}]}, 
        {Hue[0.7942339412854, 1, 1], LineBox[{3, 4}]}, 
        {Hue[0.8492028857989099, 1, 1], LineBox[{4, 5}]}, 
        {Hue[0.6022978636183001, 1, 1], LineBox[{5, 6}]}, 
        {Hue[0.7606479149378287, 1, 1], LineBox[{6, 7}]}, 
        {Hue[0.578859153398882, 1, 1], LineBox[{7, 8}]}, 
        {Hue[0.30877927036783076`, 1, 1], LineBox[{8, 9}]}, 
        {Hue[0.31482924609675245`, 1, 1], LineBox[{9, 10}]}, 
        {Hue[0.6200413157728587, 1, 1], LineBox[{10, 11}]}, 
        {Hue[0.09442359141006498, 1, 1], LineBox[{11, 12}]}, 
        {Hue[0.23001877330545184`, 1, 1], LineBox[{12, 13}]}, 
        {Hue[0.9760886121604145, 1, 1], LineBox[{13, 14}]}, 
        {Hue[0.9467880234325805, 1, 1], LineBox[{14, 15}]}, 
        {Hue[0.9028940736581215, 1, 1], LineBox[{15, 16}]}, 
        {Hue[0.752209199149744, 1, 1], LineBox[{16, 17}]}, 
        {Hue[0.6911669551999422, 1, 1], LineBox[{17, 18}]}, 
        {Hue[0.6219826929738932, 1, 1], LineBox[{18, 19}]}, 
        {Hue[0.5012750928019923, 1, 1], LineBox[{19, 20}]}, 
        {Hue[0.4060824274587364, 1, 1], LineBox[{20, 21}]}, 
        {Hue[0.3222783622768835, 1, 1], LineBox[{21, 22}]}, 
        {Hue[0.22185466654030803`, 1, 1], LineBox[{22, 23}]}, 
        {Hue[0.11723128500502794`, 1, 1], LineBox[{23, 24}]}, 
        {Hue[0.005902445675020168, 1, 1], LineBox[{24, 25}]}, 
        {Hue[0.9069522018498736, 1, 1], LineBox[{25, 26}]}, 
        {Hue[0.8068527599341031, 1, 1], LineBox[{26, 27}]}, 
        {Hue[0.708878572463234, 1, 1], LineBox[{27, 28}]}, 
        {Hue[0.6079610261392703, 1, 1], LineBox[{28, 29}]}, 
        {Hue[0.5040843867085472, 1, 1], LineBox[{29, 30}]}, 
        {Hue[0.40698470072433135`, 1, 1], LineBox[{30, 31}]}, 
        {Hue[0.3020483743054194, 1, 1], LineBox[{31, 32}]}, 
        {Hue[0.20223825272965743`, 1, 1], LineBox[{32, 33}]}, 
        {Hue[0.09995156837997692, 1, 1], LineBox[{33, 34}]}, 
        {Hue[0.9973121322051787, 1, 1], LineBox[{34, 35}]}, 
        {Hue[0.8993192587880702, 1, 1], LineBox[{35, 36}]}, 
        {Hue[0.7994909922000972, 1, 1], LineBox[{36, 37}]}, 
        {Hue[0.7001832107207604, 1, 1], LineBox[{37, 38}]}, 
        {Hue[0.6012734955736141, 1, 1], LineBox[{38, 39}]}, 
        {Hue[0.5018059684686444, 1, 1], LineBox[{39, 40}]}, 
        {Hue[0.4036261093302133, 1, 1], LineBox[{40, 41}]}, 
        {Hue[0.3050426183203099, 1, 1], LineBox[{41, 42}]}, 
        {Hue[0.20640472381167882`, 1, 1], LineBox[{42, 43}]}, 
        {Hue[0.10812979713222298`, 1, 1], LineBox[{43, 44}]}, 
        {Hue[0.010246775437848158`, 1, 1], LineBox[{44, 45}]}, 
        {Hue[0.9125933430525395, 1, 1], LineBox[{45, 46}]}, 
        {Hue[0.8151261615159525, 1, 1], LineBox[{46, 47}]}, 
        {Hue[0.7180481601790828, 1, 1], LineBox[{47, 48}]}, 
        {Hue[0.6211638292725423, 1, 1], LineBox[{48, 49}]}, 
        {Hue[0.5245698891455377, 1, 1], LineBox[{49, 50}]}, 
        {Hue[0.4283083559045919, 1, 1], LineBox[{50, 51}]}, 
        {Hue[0.3321726452629961, 1, 1], LineBox[{51, 52}]}, 
        {Hue[0.23626986406217826`, 1, 1], LineBox[{52, 53}]}, 
        {Hue[0.14059393740808274`, 1, 1], LineBox[{53, 54}]}, 
        {Hue[0.04514473076327826, 1, 1], LineBox[{54, 55}]}, 
        {Hue[0.9499413082274694, 1, 1], LineBox[{55, 56}]}, 
        {Hue[0.8549726697253752, 1, 1], LineBox[{56, 57}]}, 
        {Hue[0.760249505394373, 1, 1], LineBox[{57, 58}]}, 
        {Hue[0.6657461239613698, 1, 1], LineBox[{58, 59}]}, 
        {Hue[0.5714732426671167, 1, 1], LineBox[{59, 60}]}, 
        {Hue[0.4774118319071657, 1, 1], LineBox[{60, 61}]}, 
        {Hue[0.3835360586193685, 1, 1], LineBox[{61, 62}]}, 
        {Hue[0.28990642954043827`, 1, 1], LineBox[{62, 63}]}, 
        {Hue[0.19646904601881474`, 1, 1], LineBox[{63, 64}]}, 
        {Hue[0.10324490144884163`, 1, 1], LineBox[{64, 65}]}, 
        {Hue[0.010205847579708893`, 1, 1], LineBox[{65, 66}]}, 
        {Hue[0.9173868767508842, 1, 1], LineBox[{66, 67}]}, 
        {Hue[0.8247637208190898, 1, 1], LineBox[{67, 68}]}, 
        {Hue[0.7323406252996997, 1, 1], LineBox[{68, 69}]}, 
        {Hue[0.6401150014496022, 1, 1], LineBox[{69, 70}]}, 
        {Hue[0.5480750215091739, 1, 1], LineBox[{70, 71}]}, 
        {Hue[0.4562199245354366, 1, 1], LineBox[{71, 72}]}, 
        {Hue[0.3645427633380875, 1, 1], LineBox[{72, 73}]}, 
        {Hue[0.2730536155226162, 1, 1], LineBox[{73, 74}]}, 
        {Hue[0.18173241675375307`, 1, 1], LineBox[{74, 75}]}, 
        {Hue[0.09059190059179349, 1, 1], LineBox[{75, 76}]}, 
        {Hue[0.9996208878511171, 1, 1], LineBox[{76, 77}]}, 
        {Hue[0.9088202168452655, 1, 1], LineBox[{77, 78}]}, 
        {Hue[0.8181778536844899, 1, 1], LineBox[{78, 79}]}, 
        {Hue[0.7276989234844825, 1, 1], LineBox[{79, 80}]}, 
        {Hue[0.6373785352377027, 1, 1], LineBox[{80, 81}]}, 
        {Hue[0.5472076289406496, 1, 1], LineBox[{81, 82}]}, 
        {Hue[0.4571930226530651, 1, 1], LineBox[{82, 83}]}, 
        {Hue[0.36732415631788506`, 1, 1], LineBox[{83, 84}]}, 
        {Hue[0.27760258708611185`, 1, 1], LineBox[{84, 85}]}, 
        {Hue[0.18802137824938803`, 1, 1], LineBox[{85, 86}]}, 
        {Hue[0.0985833771491798, 1, 1], LineBox[{86, 87}]}, 
        {Hue[0.009279955353814947, 1, 1], LineBox[{87, 88}]}, 
        {Hue[0.9201105090605955, 1, 1], LineBox[{88, 89}]}, 
        {Hue[0.8310739664271323, 1, 1], LineBox[{89, 90}]}, 
        {Hue[0.7421732143735179, 1, 1], LineBox[{90, 91}]}, 
        {Hue[0.6534046333614987, 1, 1], LineBox[{91, 92}]}, 
        {Hue[0.5647705320621916, 1, 1], LineBox[{92, 93}]}, 
        {Hue[0.4762657974965162, 1, 1], LineBox[{93, 94}]}, 
        {Hue[0.38788992140175527`, 1, 1], LineBox[{94, 95}]}, 
        {Hue[0.2996453744549128, 1, 1], LineBox[{95, 96}]}, 
        {Hue[0.21153790378021053`, 1, 1], LineBox[{96, 97}]}, 
        {Hue[0.1235785372270038, 1, 1], LineBox[{97, 98}]}, 
        {Hue[0.03577515435765192, 1, 1], LineBox[{98, 99}]}, 
        {Hue[0.9481387495866579, 1, 1], LineBox[{99, 100}]}, 
        {Hue[0.8606718183156182, 1, 1], LineBox[{100, 101}]}, 
        {Hue[0.7733802821522706, 1, 1], LineBox[{101, 102}]}, 
        {Hue[0.6862775149694955, 1, 1], LineBox[{102, 103}]}, 
        {Hue[0.5993876090244299, 1, 1], LineBox[{103, 104}]}, 
        {Hue[0.5127503459429368, 1, 1], LineBox[{104, 105}]}, 
        {Hue[0.4264049556156808, 1, 1], LineBox[{105, 106}]}, 
        {Hue[0.340391290138817, 1, 1], LineBox[{106, 107}]}, 
        {Hue[0.2547382414912006, 1, 1], LineBox[{107, 108}]}, 
        {Hue[0.16947463648226757`, 1, 1], LineBox[{108, 109}]}, 
        {Hue[0.08464337306181322, 1, 1], LineBox[{109, 110}]}, 
        {Hue[0.0003142444700447079, 1, 1], LineBox[{110, 111}]}, 
        {Hue[0.9165919072600075, 1, 1], LineBox[{111, 112}]}, 
        {Hue[0.8335995002856345, 1, 1], LineBox[{112, 113}]}, 
        {Hue[0.751460053715487, 1, 1], LineBox[{113, 114}]}, 
        {Hue[0.6702749790326042, 1, 1], LineBox[{114, 115}]}, 
        {Hue[0.5901288601313921, 1, 1], LineBox[{115, 116}]}, 
        {Hue[0.5111107307817198, 1, 1], LineBox[{116, 117}]}, 
        {Hue[0.43335061972018757`, 1, 1], LineBox[{117, 118}]}, 
        {Hue[0.3570418522475413, 1, 1], LineBox[{118, 119}]}, 
        {Hue[0.2824419456203651, 1, 1], LineBox[{119, 120}]}, 
        {Hue[0.20983775878173655`, 1, 1], LineBox[{120, 121}]}, 
        {Hue[0.1395014372655653, 1, 1], LineBox[{121, 122}]}, 
        {Hue[0.07165023971034112, 1, 1], LineBox[{122, 123}]}, 
        {Hue[0.006431618568068022, 1, 1], LineBox[{123, 124}]}, 
        {Hue[0.9439325440916397, 1, 1], LineBox[{124, 125}]}, 
        {Hue[0.8842032007094285, 1, 1], LineBox[{125, 126}]}, 
        {Hue[0.8272827330826636, 1, 1], LineBox[{126, 127}]}, 
        {Hue[0.773216326087761, 1, 1], LineBox[{127, 128}]}, 
        {Hue[0.7220643652275629, 1, 1], LineBox[{128, 129}]}, 
        {Hue[0.6738998532258365, 1, 1], LineBox[{129, 130}]}, 
        {Hue[0.6287990685292446, 1, 1], LineBox[{130, 131}]}, 
        {Hue[0.5868341652474119, 1, 1], LineBox[{131, 132}]}, 
        {Hue[0.5480628738596105, 1, 1], LineBox[{132, 133}]}, 
        {Hue[0.5125236261206041, 1, 1], LineBox[{133, 134}]}, 
        {Hue[0.48022998813085527`, 1, 1], LineBox[{134, 135}]}, 
        {Hue[0.4511728596684511, 1, 1], LineBox[{135, 136}]}, 
        {Hue[0.4253201936061853, 1, 1], LineBox[{136, 137}]}, 
        {Hue[0.4026220415933217, 1, 1], LineBox[{137, 138}]}, 
        {Hue[0.38301220867514024`, 1, 1], LineBox[{138, 139}]}, 
        {Hue[0.36641158831638115`, 1, 1], LineBox[{139, 140}]}, 
        {Hue[0.35273364848390876`, 1, 1], LineBox[{140, 141}]}, 
        {Hue[0.3418848504579338, 1, 1], LineBox[{141, 142}]}, 
        {Hue[0.3337689696405779, 1, 1], LineBox[{142, 143}]}, 
        {Hue[0.3282876739531856, 1, 1], LineBox[{143, 144}]}, 
        {Hue[0.32534234933601514`, 1, 1], LineBox[{144, 145}]}, 
        {Hue[0.3248378520983274, 1, 1], LineBox[{145, 146}]}, 
        {Hue[0.3266794362989239, 1, 1], LineBox[{146, 147}]}, 
        {Hue[0.3307767146838967, 1, 1], LineBox[{147, 148}]}, 
        {Hue[0.33704224104116143`, 1, 1], LineBox[{148, 149}]}, 
        {Hue[0.3453928205084022, 1, 1], LineBox[{149, 150}]}, 
        {Hue[0.3557493476841345, 1, 1], LineBox[{150, 151}]}, 
        {Hue[0.36803727733320324`, 1, 1], LineBox[{151, 152}]}, 
        {Hue[0.38218531751463014`, 1, 1], LineBox[{152, 153}]}, 
        {Hue[0.3981269793902958, 1, 1], LineBox[{153, 154}]}, 
        {Hue[0.415800089200965, 1, 1], LineBox[{154, 155}]}, 
        {Hue[0.43514489696901626`, 1, 1], LineBox[{155, 156}]}, 
        {Hue[0.456107123450531, 1, 1], LineBox[{156, 157}]}, 
        {Hue[0.4786340231754143, 1, 1], LineBox[{157, 158}]}, 
        {Hue[0.5026780811248652, 1, 1], LineBox[{158, 159}]}, 
        {Hue[0.5281933975945959, 1, 1], LineBox[{159, 160}]}, 
        {Hue[0.555136891773129, 1, 1], LineBox[{160, 161}]}, 
        {Hue[0.5834691607915969, 1, 1], LineBox[{161, 162}]}, 
        {Hue[0.6131526256736353, 1, 1], LineBox[{162, 163}]}, 
        {Hue[0.6441525006497791, 1, 1], LineBox[{163, 164}]}, 
        {Hue[0.6764355622085817, 1, 1], LineBox[{164, 165}]}, 
        {Hue[0.7099706569321134, 1, 1], LineBox[{165, 166}]}, 
        {Hue[0.7447288937616423, 1, 1], LineBox[{166, 167}]}, 
        {Hue[0.7806831764942391, 1, 1], LineBox[{167, 168}]}, 
        {Hue[0.817806593136449, 1, 1], LineBox[{168, 169}]}, 
        {Hue[0.856076141486128, 1, 1], LineBox[{169, 170}]}, 
        {Hue[0.8954681694293263, 1, 1], LineBox[{170, 171}]}, 
        {Hue[0.9359614830231154, 1, 1], LineBox[{171, 172}]}, 
        {Hue[0.9775353594293265, 1, 1], LineBox[{172, 173}]}, 
        {Hue[0.02017020566685116, 1, 1], LineBox[{173, 174}]}, 
        {Hue[0.06384797101923008, 1, 1], LineBox[{174, 175}]}, 
        {Hue[0.10855083967090552`, 1, 1], LineBox[{175, 176}]}, 
        {Hue[0.15426253041459265`, 1, 1], LineBox[{176, 177}]}, 
        {Hue[0.20096706165192937`, 1, 1], LineBox[{177, 178}]}, 
        {Hue[0.2486501360831702, 1, 1], LineBox[{178, 179}]}, 
        {Hue[0.2972974266819873, 1, 1], LineBox[{179, 180}]}, 
        {Hue[0.3468949648771804, 1, 1], LineBox[{180, 181}]}, 
        {Hue[0.3974301349949464, 1, 1], LineBox[{181, 182}]}, 
        {Hue[0.44889051576090216`, 1, 1], LineBox[{182, 183}]}, 
        {Hue[0.5012641862857377, 1, 1], LineBox[{183, 184}]}, 
        {Hue[0.5545404603455091, 1, 1], LineBox[{184, 185}]}, 
        {Hue[0.6087074756721977, 1, 1], LineBox[{185, 186}]}, 
        {Hue[0.6637555293688033, 1, 1], LineBox[{186, 187}]}, 
        {Hue[0.7196745235769924, 1, 1], LineBox[{187, 188}]}, 
        {Hue[0.776454638312091, 1, 1], LineBox[{188, 189}]}, 
        {Hue[0.8340862590134883, 1, 1], LineBox[{189, 190}]}, 
        {Hue[0.8925618762146349, 1, 1], LineBox[{190, 191}]}, 
        {Hue[0.9518725157540959, 1, 1], LineBox[{191, 192}]}, 
        {Hue[0.01201033697292499, 1, 1], LineBox[{192, 193}]}, 
        {Hue[0.07296659247411895, 1, 1], LineBox[{193, 194}]}, 
        {Hue[0.134734925300045, 1, 1], LineBox[{194, 195}]}, 
        {Hue[0.197305564158579, 1, 1], LineBox[{195, 196}]}, 
        {Hue[0.2606739988665761, 1, 1], LineBox[{196, 197}]}, 
        {Hue[0.32483191760817787`, 1, 1], LineBox[{197, 198}]}, 
        {Hue[0.38977237153196254`, 1, 1], LineBox[{198, 199}]}, 
        {Hue[0.4554907569809305, 1, 1], LineBox[{199, 200}]}, 
        {Hue[0.5219797009100673, 1, 1], LineBox[{200, 201}]}, 
        {Hue[0.5892341605561915, 1, 1], LineBox[{201, 202}]}, 
        {Hue[0.657246872448343, 1, 1], LineBox[{202, 203}]}, 
        {Hue[0.7260117989041965, 1, 1], LineBox[{203, 204}]}, 
        {Hue[0.7955250253211213, 1, 1], LineBox[{204, 205}]}, 
        {Hue[0.8657797896021193, 1, 1], LineBox[{205, 206}]}, 
        {Hue[0.9367719499178244, 1, 1], LineBox[{206, 207}]}, 
        {Hue[0.008495195879954513, 1, 1], LineBox[{207, 208}]}, 
        {Hue[0.0809459758358265, 1, 1], LineBox[{208, 209}]}, 
        {Hue[0.15412001963161848`, 1, 1], LineBox[{209, 210}]}, 
        {Hue[0.2280115923818089, 1, 1], LineBox[{210, 211}]}, 
        {Hue[0.3026172384645148, 1, 1], LineBox[{211, 212}]}, 
        {Hue[0.3779292229301128, 1, 1], LineBox[{212, 213}]}, 
        {Hue[0.4539469430882079, 1, 1], LineBox[{213, 214}]}, 
        {Hue[0.5306651254733961, 1, 1], LineBox[{214, 215}]}, 
        {Hue[0.6080786482755536, 1, 1], LineBox[{215, 216}]}, 
        {Hue[0.6861849685526418, 1, 1], LineBox[{216, 217}]}, 
        {Hue[0.7649800556325819, 1, 1], LineBox[{217, 218}]}, 
        {Hue[0.8444590226034463, 1, 1], LineBox[{218, 219}]}, 
        {Hue[0.9246179861927123, 1, 1], LineBox[{219, 220}]}, 
        {Hue[0.0054564117259276864`, 1, 1], LineBox[{220, 221}]}, 
        {Hue[0.08696888006741216, 1, 1], LineBox[{221, 222}]}, 
        {Hue[0.16914750535384115`, 1, 1], LineBox[{222, 223}]}, 
        {Hue[0.2519914361670223, 1, 1], LineBox[{223, 224}]}, 
        {Hue[0.3355017653036084, 1, 1], LineBox[{224, 225}]}, 
        {Hue[0.41967365987477934`, 1, 1], LineBox[{225, 226}]}, 
        {Hue[0.5045004497746628, 1, 1], LineBox[{226, 227}]}, 
        {Hue[0.5899817557355067, 1, 1], LineBox[{227, 228}]}, 
        {Hue[0.6761137981804346, 1, 1], LineBox[{228, 229}]}, 
        {Hue[0.7628854856167173, 1, 1], LineBox[{229, 230}]}, 
        {Hue[0.8503116541986834, 1, 1], LineBox[{230, 231}]}, 
        {Hue[0.9383888140536001, 1, 1], LineBox[{231, 232}]}, 
        {Hue[0.02709017049981145, 1, 1], LineBox[{232, 233}]}, 
        {Hue[0.11642414389020968`, 1, 1], LineBox[{233, 234}]}, 
        {Hue[0.20640715237073284`, 1, 1], LineBox[{234, 235}]}, 
        {Hue[0.297017601531361, 1, 1], LineBox[{235, 236}]}, 
        {Hue[0.38824632762828315`, 1, 1], LineBox[{236, 237}]}, 
        {Hue[0.4801090758515208, 1, 1], LineBox[{237, 238}]}, 
        {Hue[0.5725864431600319, 1, 1], LineBox[{238, 239}]}, 
        {Hue[0.665687847174255, 1, 1], LineBox[{239, 240}]}, 
        {Hue[0.7594259975222694, 1, 1], LineBox[{240, 241}]}, 
        {Hue[0.8537592849553937, 1, 1], LineBox[{241, 242}]}, 
        {Hue[0.948706513230035, 1, 1], LineBox[{242, 243}]}, 
        {Hue[0.04426775150904171, 1, 1], LineBox[{243, 244}]}, 
        {Hue[0.1404108073613589, 1, 1], LineBox[{244, 245}]}, 
        {Hue[0.237191244358674, 1, 1], LineBox[{245, 246}]}, 
        {Hue[0.3346110854878864, 1, 1], LineBox[{246, 247}]}, 
        {Hue[0.43260862207299355`, 1, 1], LineBox[{247, 248}]}, 
        {Hue[0.5311723937875716, 1, 1], LineBox[{248, 249}]}, 
        {Hue[0.6303165816593157, 1, 1], LineBox[{249, 250}]}, 
        {Hue[0.7300926755163386, 1, 1], LineBox[{250, 251}]}}}},
     VertexColors->{
       Hue[0.974286712374846, 1, 1], 
       Hue[0.08446566071998841, 1, 1], 
       Hue[0.7942339412854, 1, 1], 
       Hue[0.8492028857989099, 1, 1], 
       Hue[0.6022978636183001, 1, 1], 
       Hue[0.7606479149378287, 1, 1], 
       Hue[0.578859153398882, 1, 1], 
       Hue[0.30877927036783076`, 1, 1], 
       Hue[0.31482924609675245`, 1, 1], 
       Hue[0.6200413157728587, 1, 1], 
       Hue[0.09442359141006498, 1, 1], 
       Hue[0.23001877330545184`, 1, 1], 
       Hue[0.9760886121604145, 1, 1], 
       Hue[0.9467880234325805, 1, 1], 
       Hue[0.9028940736581215, 1, 1], 
       Hue[0.752209199149744, 1, 1], 
       Hue[0.6911669551999422, 1, 1], 
       Hue[0.6219826929738932, 1, 1], 
       Hue[0.5012750928019923, 1, 1], 
       Hue[0.4060824274587364, 1, 1], 
       Hue[0.3222783622768835, 1, 1], 
       Hue[0.22185466654030803`, 1, 1], 
       Hue[0.11723128500502794`, 1, 1], 
       Hue[0.005902445675020168, 1, 1], 
       Hue[0.9069522018498736, 1, 1], 
       Hue[0.8068527599341031, 1, 1], 
       Hue[0.708878572463234, 1, 1], 
       Hue[0.6079610261392703, 1, 1], 
       Hue[0.5040843867085472, 1, 1], 
       Hue[0.40698470072433135`, 1, 1], 
       Hue[0.3020483743054194, 1, 1], 
       Hue[0.20223825272965743`, 1, 1], 
       Hue[0.09995156837997692, 1, 1], 
       Hue[0.9973121322051787, 1, 1], 
       Hue[0.8993192587880702, 1, 1], 
       Hue[0.7994909922000972, 1, 1], 
       Hue[0.7001832107207604, 1, 1], 
       Hue[0.6012734955736141, 1, 1], 
       Hue[0.5018059684686444, 1, 1], 
       Hue[0.4036261093302133, 1, 1], 
       Hue[0.3050426183203099, 1, 1], 
       Hue[0.20640472381167882`, 1, 1], 
       Hue[0.10812979713222298`, 1, 1], 
       Hue[0.010246775437848158`, 1, 1], 
       Hue[0.9125933430525395, 1, 1], 
       Hue[0.8151261615159525, 1, 1], 
       Hue[0.7180481601790828, 1, 1], 
       Hue[0.6211638292725423, 1, 1], 
       Hue[0.5245698891455377, 1, 1], 
       Hue[0.4283083559045919, 1, 1], 
       Hue[0.3321726452629961, 1, 1], 
       Hue[0.23626986406217826`, 1, 1], 
       Hue[0.14059393740808274`, 1, 1], 
       Hue[0.04514473076327826, 1, 1], 
       Hue[0.9499413082274694, 1, 1], 
       Hue[0.8549726697253752, 1, 1], 
       Hue[0.760249505394373, 1, 1], 
       Hue[0.6657461239613698, 1, 1], 
       Hue[0.5714732426671167, 1, 1], 
       Hue[0.4774118319071657, 1, 1], 
       Hue[0.3835360586193685, 1, 1], 
       Hue[0.28990642954043827`, 1, 1], 
       Hue[0.19646904601881474`, 1, 1], 
       Hue[0.10324490144884163`, 1, 1], 
       Hue[0.010205847579708893`, 1, 1], 
       Hue[0.9173868767508842, 1, 1], 
       Hue[0.8247637208190898, 1, 1], 
       Hue[0.7323406252996997, 1, 1], 
       Hue[0.6401150014496022, 1, 1], 
       Hue[0.5480750215091739, 1, 1], 
       Hue[0.4562199245354366, 1, 1], 
       Hue[0.3645427633380875, 1, 1], 
       Hue[0.2730536155226162, 1, 1], 
       Hue[0.18173241675375307`, 1, 1], 
       Hue[0.09059190059179349, 1, 1], 
       Hue[0.9996208878511171, 1, 1], 
       Hue[0.9088202168452655, 1, 1], 
       Hue[0.8181778536844899, 1, 1], 
       Hue[0.7276989234844825, 1, 1], 
       Hue[0.6373785352377027, 1, 1], 
       Hue[0.5472076289406496, 1, 1], 
       Hue[0.4571930226530651, 1, 1], 
       Hue[0.36732415631788506`, 1, 1], 
       Hue[0.27760258708611185`, 1, 1], 
       Hue[0.18802137824938803`, 1, 1], 
       Hue[0.0985833771491798, 1, 1], 
       Hue[0.009279955353814947, 1, 1], 
       Hue[0.9201105090605955, 1, 1], 
       Hue[0.8310739664271323, 1, 1], 
       Hue[0.7421732143735179, 1, 1], 
       Hue[0.6534046333614987, 1, 1], 
       Hue[0.5647705320621916, 1, 1], 
       Hue[0.4762657974965162, 1, 1], 
       Hue[0.38788992140175527`, 1, 1], 
       Hue[0.2996453744549128, 1, 1], 
       Hue[0.21153790378021053`, 1, 1], 
       Hue[0.1235785372270038, 1, 1], 
       Hue[0.03577515435765192, 1, 1], 
       Hue[0.9481387495866579, 1, 1], 
       Hue[0.8606718183156182, 1, 1], 
       Hue[0.7733802821522706, 1, 1], 
       Hue[0.6862775149694955, 1, 1], 
       Hue[0.5993876090244299, 1, 1], 
       Hue[0.5127503459429368, 1, 1], 
       Hue[0.4264049556156808, 1, 1], 
       Hue[0.340391290138817, 1, 1], 
       Hue[0.2547382414912006, 1, 1], 
       Hue[0.16947463648226757`, 1, 1], 
       Hue[0.08464337306181322, 1, 1], 
       Hue[0.0003142444700447079, 1, 1], 
       Hue[0.9165919072600075, 1, 1], 
       Hue[0.8335995002856345, 1, 1], 
       Hue[0.751460053715487, 1, 1], 
       Hue[0.6702749790326042, 1, 1], 
       Hue[0.5901288601313921, 1, 1], 
       Hue[0.5111107307817198, 1, 1], 
       Hue[0.43335061972018757`, 1, 1], 
       Hue[0.3570418522475413, 1, 1], 
       Hue[0.2824419456203651, 1, 1], 
       Hue[0.20983775878173655`, 1, 1], 
       Hue[0.1395014372655653, 1, 1], 
       Hue[0.07165023971034112, 1, 1], 
       Hue[0.006431618568068022, 1, 1], 
       Hue[0.9439325440916397, 1, 1], 
       Hue[0.8842032007094285, 1, 1], 
       Hue[0.8272827330826636, 1, 1], 
       Hue[0.773216326087761, 1, 1], 
       Hue[0.7220643652275629, 1, 1], 
       Hue[0.6738998532258365, 1, 1], 
       Hue[0.6287990685292446, 1, 1], 
       Hue[0.5868341652474119, 1, 1], 
       Hue[0.5480628738596105, 1, 1], 
       Hue[0.5125236261206041, 1, 1], 
       Hue[0.48022998813085527`, 1, 1], 
       Hue[0.4511728596684511, 1, 1], 
       Hue[0.4253201936061853, 1, 1], 
       Hue[0.4026220415933217, 1, 1], 
       Hue[0.38301220867514024`, 1, 1], 
       Hue[0.36641158831638115`, 1, 1], 
       Hue[0.35273364848390876`, 1, 1], 
       Hue[0.3418848504579338, 1, 1], 
       Hue[0.3337689696405779, 1, 1], 
       Hue[0.3282876739531856, 1, 1], 
       Hue[0.32534234933601514`, 1, 1], 
       Hue[0.3248378520983274, 1, 1], 
       Hue[0.3266794362989239, 1, 1], 
       Hue[0.3307767146838967, 1, 1], 
       Hue[0.33704224104116143`, 1, 1], 
       Hue[0.3453928205084022, 1, 1], 
       Hue[0.3557493476841345, 1, 1], 
       Hue[0.36803727733320324`, 1, 1], 
       Hue[0.38218531751463014`, 1, 1], 
       Hue[0.3981269793902958, 1, 1], 
       Hue[0.415800089200965, 1, 1], 
       Hue[0.43514489696901626`, 1, 1], 
       Hue[0.456107123450531, 1, 1], 
       Hue[0.4786340231754143, 1, 1], 
       Hue[0.5026780811248652, 1, 1], 
       Hue[0.5281933975945959, 1, 1], 
       Hue[0.555136891773129, 1, 1], 
       Hue[0.5834691607915969, 1, 1], 
       Hue[0.6131526256736353, 1, 1], 
       Hue[0.6441525006497791, 1, 1], 
       Hue[0.6764355622085817, 1, 1], 
       Hue[0.7099706569321134, 1, 1], 
       Hue[0.7447288937616423, 1, 1], 
       Hue[0.7806831764942391, 1, 1], 
       Hue[0.817806593136449, 1, 1], 
       Hue[0.856076141486128, 1, 1], 
       Hue[0.8954681694293263, 1, 1], 
       Hue[0.9359614830231154, 1, 1], 
       Hue[0.9775353594293265, 1, 1], 
       Hue[0.02017020566685116, 1, 1], 
       Hue[0.06384797101923008, 1, 1], 
       Hue[0.10855083967090552`, 1, 1], 
       Hue[0.15426253041459265`, 1, 1], 
       Hue[0.20096706165192937`, 1, 1], 
       Hue[0.2486501360831702, 1, 1], 
       Hue[0.2972974266819873, 1, 1], 
       Hue[0.3468949648771804, 1, 1], 
       Hue[0.3974301349949464, 1, 1], 
       Hue[0.44889051576090216`, 1, 1], 
       Hue[0.5012641862857377, 1, 1], 
       Hue[0.5545404603455091, 1, 1], 
       Hue[0.6087074756721977, 1, 1], 
       Hue[0.6637555293688033, 1, 1], 
       Hue[0.7196745235769924, 1, 1], 
       Hue[0.776454638312091, 1, 1], 
       Hue[0.8340862590134883, 1, 1], 
       Hue[0.8925618762146349, 1, 1], 
       Hue[0.9518725157540959, 1, 1], 
       Hue[0.01201033697292499, 1, 1], 
       Hue[0.07296659247411895, 1, 1], 
       Hue[0.134734925300045, 1, 1], 
       Hue[0.197305564158579, 1, 1], 
       Hue[0.2606739988665761, 1, 1], 
       Hue[0.32483191760817787`, 1, 1], 
       Hue[0.38977237153196254`, 1, 1], 
       Hue[0.4554907569809305, 1, 1], 
       Hue[0.5219797009100673, 1, 1], 
       Hue[0.5892341605561915, 1, 1], 
       Hue[0.657246872448343, 1, 1], 
       Hue[0.7260117989041965, 1, 1], 
       Hue[0.7955250253211213, 1, 1], 
       Hue[0.8657797896021193, 1, 1], 
       Hue[0.9367719499178244, 1, 1], 
       Hue[0.008495195879954513, 1, 1], 
       Hue[0.0809459758358265, 1, 1], 
       Hue[0.15412001963161848`, 1, 1], 
       Hue[0.2280115923818089, 1, 1], 
       Hue[0.3026172384645148, 1, 1], 
       Hue[0.3779292229301128, 1, 1], 
       Hue[0.4539469430882079, 1, 1], 
       Hue[0.5306651254733961, 1, 1], 
       Hue[0.6080786482755536, 1, 1], 
       Hue[0.6861849685526418, 1, 1], 
       Hue[0.7649800556325819, 1, 1], 
       Hue[0.8444590226034463, 1, 1], 
       Hue[0.9246179861927123, 1, 1], 
       Hue[0.0054564117259276864`, 1, 1], 
       Hue[0.08696888006741216, 1, 1], 
       Hue[0.16914750535384115`, 1, 1], 
       Hue[0.2519914361670223, 1, 1], 
       Hue[0.3355017653036084, 1, 1], 
       Hue[0.41967365987477934`, 1, 1], 
       Hue[0.5045004497746628, 1, 1], 
       Hue[0.5899817557355067, 1, 1], 
       Hue[0.6761137981804346, 1, 1], 
       Hue[0.7628854856167173, 1, 1], 
       Hue[0.8503116541986834, 1, 1], 
       Hue[0.9383888140536001, 1, 1], 
       Hue[0.02709017049981145, 1, 1], 
       Hue[0.11642414389020968`, 1, 1], 
       Hue[0.20640715237073284`, 1, 1], 
       Hue[0.297017601531361, 1, 1], 
       Hue[0.38824632762828315`, 1, 1], 
       Hue[0.4801090758515208, 1, 1], 
       Hue[0.5725864431600319, 1, 1], 
       Hue[0.665687847174255, 1, 1], 
       Hue[0.7594259975222694, 1, 1], 
       Hue[0.8537592849553937, 1, 1], 
       Hue[0.948706513230035, 1, 1], 
       Hue[0.04426775150904171, 1, 1], 
       Hue[0.1404108073613589, 1, 1], 
       Hue[0.237191244358674, 1, 1], 
       Hue[0.3346110854878864, 1, 1], 
       Hue[0.43260862207299355`, 1, 1], 
       Hue[0.5311723937875716, 1, 1], 
       Hue[0.6303165816593157, 1, 1], 
       Hue[0.7300926755163386, 1, 1], 
       Hue[0.830492702282897, 1, 1], 
       Hue[0.974286712374846, 1, 1], 
       Hue[0.08446566071998841, 1, 1], 
       Hue[0.08446566071998841, 1, 1], 
       Hue[0.7942339412854, 1, 1], 
       Hue[0.7942339412854, 1, 1], 
       Hue[0.8492028857989099, 1, 1], 
       Hue[0.8492028857989099, 1, 1], 
       Hue[0.6022978636183001, 1, 1], 
       Hue[0.6022978636183001, 1, 1], 
       Hue[0.7606479149378287, 1, 1], 
       Hue[0.7606479149378287, 1, 1], 
       Hue[0.578859153398882, 1, 1], 
       Hue[0.578859153398882, 1, 1], 
       Hue[0.30877927036783076`, 1, 1], 
       Hue[0.30877927036783076`, 1, 1], 
       Hue[0.31482924609675245`, 1, 1], 
       Hue[0.31482924609675245`, 1, 1], 
       Hue[0.6200413157728587, 1, 1], 
       Hue[0.6200413157728587, 1, 1], 
       Hue[0.09442359141006498, 1, 1], 
       Hue[0.09442359141006498, 1, 1], 
       Hue[0.23001877330545184`, 1, 1], 
       Hue[0.23001877330545184`, 1, 1], 
       Hue[0.9760886121604145, 1, 1], 
       Hue[0.9760886121604145, 1, 1], 
       Hue[0.9467880234325805, 1, 1], 
       Hue[0.9467880234325805, 1, 1], 
       Hue[0.9028940736581215, 1, 1], 
       Hue[0.9028940736581215, 1, 1], 
       Hue[0.752209199149744, 1, 1], 
       Hue[0.752209199149744, 1, 1], 
       Hue[0.6911669551999422, 1, 1], 
       Hue[0.6911669551999422, 1, 1], 
       Hue[0.6219826929738932, 1, 1], 
       Hue[0.6219826929738932, 1, 1], 
       Hue[0.5012750928019923, 1, 1], 
       Hue[0.5012750928019923, 1, 1], 
       Hue[0.4060824274587364, 1, 1], 
       Hue[0.4060824274587364, 1, 1], 
       Hue[0.3222783622768835, 1, 1], 
       Hue[0.3222783622768835, 1, 1], 
       Hue[0.22185466654030803`, 1, 1], 
       Hue[0.22185466654030803`, 1, 1], 
       Hue[0.11723128500502794`, 1, 1], 
       Hue[0.11723128500502794`, 1, 1], 
       Hue[0.005902445675020168, 1, 1], 
       Hue[0.005902445675020168, 1, 1], 
       Hue[0.9069522018498736, 1, 1], 
       Hue[0.9069522018498736, 1, 1], 
       Hue[0.8068527599341031, 1, 1], 
       Hue[0.8068527599341031, 1, 1], 
       Hue[0.708878572463234, 1, 1], 
       Hue[0.708878572463234, 1, 1], 
       Hue[0.6079610261392703, 1, 1], 
       Hue[0.6079610261392703, 1, 1], 
       Hue[0.5040843867085472, 1, 1], 
       Hue[0.5040843867085472, 1, 1], 
       Hue[0.40698470072433135`, 1, 1], 
       Hue[0.40698470072433135`, 1, 1], 
       Hue[0.3020483743054194, 1, 1], 
       Hue[0.3020483743054194, 1, 1], 
       Hue[0.20223825272965743`, 1, 1], 
       Hue[0.20223825272965743`, 1, 1], 
       Hue[0.09995156837997692, 1, 1], 
       Hue[0.09995156837997692, 1, 1], 
       Hue[0.9973121322051787, 1, 1], 
       Hue[0.9973121322051787, 1, 1], 
       Hue[0.8993192587880702, 1, 1], 
       Hue[0.8993192587880702, 1, 1], 
       Hue[0.7994909922000972, 1, 1], 
       Hue[0.7994909922000972, 1, 1], 
       Hue[0.7001832107207604, 1, 1], 
       Hue[0.7001832107207604, 1, 1], 
       Hue[0.6012734955736141, 1, 1], 
       Hue[0.6012734955736141, 1, 1], 
       Hue[0.5018059684686444, 1, 1], 
       Hue[0.5018059684686444, 1, 1], 
       Hue[0.4036261093302133, 1, 1], 
       Hue[0.4036261093302133, 1, 1], 
       Hue[0.3050426183203099, 1, 1], 
       Hue[0.3050426183203099, 1, 1], 
       Hue[0.20640472381167882`, 1, 1], 
       Hue[0.20640472381167882`, 1, 1], 
       Hue[0.10812979713222298`, 1, 1], 
       Hue[0.10812979713222298`, 1, 1], 
       Hue[0.010246775437848158`, 1, 1], 
       Hue[0.010246775437848158`, 1, 1], 
       Hue[0.9125933430525395, 1, 1], 
       Hue[0.9125933430525395, 1, 1], 
       Hue[0.8151261615159525, 1, 1], 
       Hue[0.8151261615159525, 1, 1], 
       Hue[0.7180481601790828, 1, 1], 
       Hue[0.7180481601790828, 1, 1], 
       Hue[0.6211638292725423, 1, 1], 
       Hue[0.6211638292725423, 1, 1], 
       Hue[0.5245698891455377, 1, 1], 
       Hue[0.5245698891455377, 1, 1], 
       Hue[0.4283083559045919, 1, 1], 
       Hue[0.4283083559045919, 1, 1], 
       Hue[0.3321726452629961, 1, 1], 
       Hue[0.3321726452629961, 1, 1], 
       Hue[0.23626986406217826`, 1, 1], 
       Hue[0.23626986406217826`, 1, 1], 
       Hue[0.14059393740808274`, 1, 1], 
       Hue[0.14059393740808274`, 1, 1], 
       Hue[0.04514473076327826, 1, 1], 
       Hue[0.04514473076327826, 1, 1], 
       Hue[0.9499413082274694, 1, 1], 
       Hue[0.9499413082274694, 1, 1], 
       Hue[0.8549726697253752, 1, 1], 
       Hue[0.8549726697253752, 1, 1], 
       Hue[0.760249505394373, 1, 1], 
       Hue[0.760249505394373, 1, 1], 
       Hue[0.6657461239613698, 1, 1], 
       Hue[0.6657461239613698, 1, 1], 
       Hue[0.5714732426671167, 1, 1], 
       Hue[0.5714732426671167, 1, 1], 
       Hue[0.4774118319071657, 1, 1], 
       Hue[0.4774118319071657, 1, 1], 
       Hue[0.3835360586193685, 1, 1], 
       Hue[0.3835360586193685, 1, 1], 
       Hue[0.28990642954043827`, 1, 1], 
       Hue[0.28990642954043827`, 1, 1], 
       Hue[0.19646904601881474`, 1, 1], 
       Hue[0.19646904601881474`, 1, 1], 
       Hue[0.10324490144884163`, 1, 1], 
       Hue[0.10324490144884163`, 1, 1], 
       Hue[0.010205847579708893`, 1, 1], 
       Hue[0.010205847579708893`, 1, 1], 
       Hue[0.9173868767508842, 1, 1], 
       Hue[0.9173868767508842, 1, 1], 
       Hue[0.8247637208190898, 1, 1], 
       Hue[0.8247637208190898, 1, 1], 
       Hue[0.7323406252996997, 1, 1], 
       Hue[0.7323406252996997, 1, 1], 
       Hue[0.6401150014496022, 1, 1], 
       Hue[0.6401150014496022, 1, 1], 
       Hue[0.5480750215091739, 1, 1], 
       Hue[0.5480750215091739, 1, 1], 
       Hue[0.4562199245354366, 1, 1], 
       Hue[0.4562199245354366, 1, 1], 
       Hue[0.3645427633380875, 1, 1], 
       Hue[0.3645427633380875, 1, 1], 
       Hue[0.2730536155226162, 1, 1], 
       Hue[0.2730536155226162, 1, 1], 
       Hue[0.18173241675375307`, 1, 1], 
       Hue[0.18173241675375307`, 1, 1], 
       Hue[0.09059190059179349, 1, 1], 
       Hue[0.09059190059179349, 1, 1], 
       Hue[0.9996208878511171, 1, 1], 
       Hue[0.9996208878511171, 1, 1], 
       Hue[0.9088202168452655, 1, 1], 
       Hue[0.9088202168452655, 1, 1], 
       Hue[0.8181778536844899, 1, 1], 
       Hue[0.8181778536844899, 1, 1], 
       Hue[0.7276989234844825, 1, 1], 
       Hue[0.7276989234844825, 1, 1], 
       Hue[0.6373785352377027, 1, 1], 
       Hue[0.6373785352377027, 1, 1], 
       Hue[0.5472076289406496, 1, 1], 
       Hue[0.5472076289406496, 1, 1], 
       Hue[0.4571930226530651, 1, 1], 
       Hue[0.4571930226530651, 1, 1], 
       Hue[0.36732415631788506`, 1, 1], 
       Hue[0.36732415631788506`, 1, 1], 
       Hue[0.27760258708611185`, 1, 1], 
       Hue[0.27760258708611185`, 1, 1], 
       Hue[0.18802137824938803`, 1, 1], 
       Hue[0.18802137824938803`, 1, 1], 
       Hue[0.0985833771491798, 1, 1], 
       Hue[0.0985833771491798, 1, 1], 
       Hue[0.009279955353814947, 1, 1], 
       Hue[0.009279955353814947, 1, 1], 
       Hue[0.9201105090605955, 1, 1], 
       Hue[0.9201105090605955, 1, 1], 
       Hue[0.8310739664271323, 1, 1], 
       Hue[0.8310739664271323, 1, 1], 
       Hue[0.7421732143735179, 1, 1], 
       Hue[0.7421732143735179, 1, 1], 
       Hue[0.6534046333614987, 1, 1], 
       Hue[0.6534046333614987, 1, 1], 
       Hue[0.5647705320621916, 1, 1], 
       Hue[0.5647705320621916, 1, 1], 
       Hue[0.4762657974965162, 1, 1], 
       Hue[0.4762657974965162, 1, 1], 
       Hue[0.38788992140175527`, 1, 1], 
       Hue[0.38788992140175527`, 1, 1], 
       Hue[0.2996453744549128, 1, 1], 
       Hue[0.2996453744549128, 1, 1], 
       Hue[0.21153790378021053`, 1, 1], 
       Hue[0.21153790378021053`, 1, 1], 
       Hue[0.1235785372270038, 1, 1], 
       Hue[0.1235785372270038, 1, 1], 
       Hue[0.03577515435765192, 1, 1], 
       Hue[0.03577515435765192, 1, 1], 
       Hue[0.9481387495866579, 1, 1], 
       Hue[0.9481387495866579, 1, 1], 
       Hue[0.8606718183156182, 1, 1], 
       Hue[0.8606718183156182, 1, 1], 
       Hue[0.7733802821522706, 1, 1], 
       Hue[0.7733802821522706, 1, 1], 
       Hue[0.6862775149694955, 1, 1], 
       Hue[0.6862775149694955, 1, 1], 
       Hue[0.5993876090244299, 1, 1], 
       Hue[0.5993876090244299, 1, 1], 
       Hue[0.5127503459429368, 1, 1], 
       Hue[0.5127503459429368, 1, 1], 
       Hue[0.4264049556156808, 1, 1], 
       Hue[0.4264049556156808, 1, 1], 
       Hue[0.340391290138817, 1, 1], 
       Hue[0.340391290138817, 1, 1], 
       Hue[0.2547382414912006, 1, 1], 
       Hue[0.2547382414912006, 1, 1], 
       Hue[0.16947463648226757`, 1, 1], 
       Hue[0.16947463648226757`, 1, 1], 
       Hue[0.08464337306181322, 1, 1], 
       Hue[0.08464337306181322, 1, 1], 
       Hue[0.0003142444700447079, 1, 1], 
       Hue[0.0003142444700447079, 1, 1], 
       Hue[0.9165919072600075, 1, 1], 
       Hue[0.9165919072600075, 1, 1], 
       Hue[0.8335995002856345, 1, 1], 
       Hue[0.8335995002856345, 1, 1], 
       Hue[0.751460053715487, 1, 1], 
       Hue[0.751460053715487, 1, 1], 
       Hue[0.6702749790326042, 1, 1], 
       Hue[0.6702749790326042, 1, 1], 
       Hue[0.5901288601313921, 1, 1], 
       Hue[0.5901288601313921, 1, 1], 
       Hue[0.5111107307817198, 1, 1], 
       Hue[0.5111107307817198, 1, 1], 
       Hue[0.43335061972018757`, 1, 1], 
       Hue[0.43335061972018757`, 1, 1], 
       Hue[0.3570418522475413, 1, 1], 
       Hue[0.3570418522475413, 1, 1], 
       Hue[0.2824419456203651, 1, 1], 
       Hue[0.2824419456203651, 1, 1], 
       Hue[0.20983775878173655`, 1, 1], 
       Hue[0.20983775878173655`, 1, 1], 
       Hue[0.1395014372655653, 1, 1], 
       Hue[0.1395014372655653, 1, 1], 
       Hue[0.07165023971034112, 1, 1], 
       Hue[0.07165023971034112, 1, 1], 
       Hue[0.006431618568068022, 1, 1], 
       Hue[0.006431618568068022, 1, 1], 
       Hue[0.9439325440916397, 1, 1], 
       Hue[0.9439325440916397, 1, 1], 
       Hue[0.8842032007094285, 1, 1], 
       Hue[0.8842032007094285, 1, 1], 
       Hue[0.8272827330826636, 1, 1], 
       Hue[0.8272827330826636, 1, 1], 
       Hue[0.773216326087761, 1, 1], 
       Hue[0.773216326087761, 1, 1], 
       Hue[0.7220643652275629, 1, 1], 
       Hue[0.7220643652275629, 1, 1], 
       Hue[0.6738998532258365, 1, 1], 
       Hue[0.6738998532258365, 1, 1], 
       Hue[0.6287990685292446, 1, 1], 
       Hue[0.6287990685292446, 1, 1], 
       Hue[0.5868341652474119, 1, 1], 
       Hue[0.5868341652474119, 1, 1], 
       Hue[0.5480628738596105, 1, 1], 
       Hue[0.5480628738596105, 1, 1], 
       Hue[0.5125236261206041, 1, 1], 
       Hue[0.5125236261206041, 1, 1], 
       Hue[0.48022998813085527`, 1, 1], 
       Hue[0.48022998813085527`, 1, 1], 
       Hue[0.4511728596684511, 1, 1], 
       Hue[0.4511728596684511, 1, 1], 
       Hue[0.4253201936061853, 1, 1], 
       Hue[0.4253201936061853, 1, 1], 
       Hue[0.4026220415933217, 1, 1], 
       Hue[0.4026220415933217, 1, 1], 
       Hue[0.38301220867514024`, 1, 1], 
       Hue[0.38301220867514024`, 1, 1], 
       Hue[0.36641158831638115`, 1, 1], 
       Hue[0.36641158831638115`, 1, 1], 
       Hue[0.35273364848390876`, 1, 1], 
       Hue[0.35273364848390876`, 1, 1], 
       Hue[0.3418848504579338, 1, 1], 
       Hue[0.3418848504579338, 1, 1], 
       Hue[0.3337689696405779, 1, 1], 
       Hue[0.3337689696405779, 1, 1], 
       Hue[0.3282876739531856, 1, 1], 
       Hue[0.3282876739531856, 1, 1], 
       Hue[0.32534234933601514`, 1, 1], 
       Hue[0.32534234933601514`, 1, 1], 
       Hue[0.3248378520983274, 1, 1], 
       Hue[0.3248378520983274, 1, 1], 
       Hue[0.3266794362989239, 1, 1], 
       Hue[0.3266794362989239, 1, 1], 
       Hue[0.3307767146838967, 1, 1], 
       Hue[0.3307767146838967, 1, 1], 
       Hue[0.33704224104116143`, 1, 1], 
       Hue[0.33704224104116143`, 1, 1], 
       Hue[0.3453928205084022, 1, 1], 
       Hue[0.3453928205084022, 1, 1], 
       Hue[0.3557493476841345, 1, 1], 
       Hue[0.3557493476841345, 1, 1], 
       Hue[0.36803727733320324`, 1, 1], 
       Hue[0.36803727733320324`, 1, 1], 
       Hue[0.38218531751463014`, 1, 1], 
       Hue[0.38218531751463014`, 1, 1], 
       Hue[0.3981269793902958, 1, 1], 
       Hue[0.3981269793902958, 1, 1], 
       Hue[0.415800089200965, 1, 1], 
       Hue[0.415800089200965, 1, 1], 
       Hue[0.43514489696901626`, 1, 1], 
       Hue[0.43514489696901626`, 1, 1], 
       Hue[0.456107123450531, 1, 1], 
       Hue[0.456107123450531, 1, 1], 
       Hue[0.4786340231754143, 1, 1], 
       Hue[0.4786340231754143, 1, 1], 
       Hue[0.5026780811248652, 1, 1], 
       Hue[0.5026780811248652, 1, 1], 
       Hue[0.5281933975945959, 1, 1], 
       Hue[0.5281933975945959, 1, 1], 
       Hue[0.555136891773129, 1, 1], 
       Hue[0.555136891773129, 1, 1], 
       Hue[0.5834691607915969, 1, 1], 
       Hue[0.5834691607915969, 1, 1], 
       Hue[0.6131526256736353, 1, 1], 
       Hue[0.6131526256736353, 1, 1], 
       Hue[0.6441525006497791, 1, 1], 
       Hue[0.6441525006497791, 1, 1], 
       Hue[0.6764355622085817, 1, 1], 
       Hue[0.6764355622085817, 1, 1], 
       Hue[0.7099706569321134, 1, 1], 
       Hue[0.7099706569321134, 1, 1], 
       Hue[0.7447288937616423, 1, 1], 
       Hue[0.7447288937616423, 1, 1], 
       Hue[0.7806831764942391, 1, 1], 
       Hue[0.7806831764942391, 1, 1], 
       Hue[0.817806593136449, 1, 1], 
       Hue[0.817806593136449, 1, 1], 
       Hue[0.856076141486128, 1, 1], 
       Hue[0.856076141486128, 1, 1], 
       Hue[0.8954681694293263, 1, 1], 
       Hue[0.8954681694293263, 1, 1], 
       Hue[0.9359614830231154, 1, 1], 
       Hue[0.9359614830231154, 1, 1], 
       Hue[0.9775353594293265, 1, 1], 
       Hue[0.9775353594293265, 1, 1], 
       Hue[0.02017020566685116, 1, 1], 
       Hue[0.02017020566685116, 1, 1], 
       Hue[0.06384797101923008, 1, 1], 
       Hue[0.06384797101923008, 1, 1], 
       Hue[0.10855083967090552`, 1, 1], 
       Hue[0.10855083967090552`, 1, 1], 
       Hue[0.15426253041459265`, 1, 1], 
       Hue[0.15426253041459265`, 1, 1], 
       Hue[0.20096706165192937`, 1, 1], 
       Hue[0.20096706165192937`, 1, 1], 
       Hue[0.2486501360831702, 1, 1], 
       Hue[0.2486501360831702, 1, 1], 
       Hue[0.2972974266819873, 1, 1], 
       Hue[0.2972974266819873, 1, 1], 
       Hue[0.3468949648771804, 1, 1], 
       Hue[0.3468949648771804, 1, 1], 
       Hue[0.3974301349949464, 1, 1], 
       Hue[0.3974301349949464, 1, 1], 
       Hue[0.44889051576090216`, 1, 1], 
       Hue[0.44889051576090216`, 1, 1], 
       Hue[0.5012641862857377, 1, 1], 
       Hue[0.5012641862857377, 1, 1], 
       Hue[0.5545404603455091, 1, 1], 
       Hue[0.5545404603455091, 1, 1], 
       Hue[0.6087074756721977, 1, 1], 
       Hue[0.6087074756721977, 1, 1], 
       Hue[0.6637555293688033, 1, 1], 
       Hue[0.6637555293688033, 1, 1], 
       Hue[0.7196745235769924, 1, 1], 
       Hue[0.7196745235769924, 1, 1], 
       Hue[0.776454638312091, 1, 1], 
       Hue[0.776454638312091, 1, 1], 
       Hue[0.8340862590134883, 1, 1], 
       Hue[0.8340862590134883, 1, 1], 
       Hue[0.8925618762146349, 1, 1], 
       Hue[0.8925618762146349, 1, 1], 
       Hue[0.9518725157540959, 1, 1], 
       Hue[0.9518725157540959, 1, 1], 
       Hue[0.01201033697292499, 1, 1], 
       Hue[0.01201033697292499, 1, 1], 
       Hue[0.07296659247411895, 1, 1], 
       Hue[0.07296659247411895, 1, 1], 
       Hue[0.134734925300045, 1, 1], 
       Hue[0.134734925300045, 1, 1], 
       Hue[0.197305564158579, 1, 1], 
       Hue[0.197305564158579, 1, 1], 
       Hue[0.2606739988665761, 1, 1], 
       Hue[0.2606739988665761, 1, 1], 
       Hue[0.32483191760817787`, 1, 1], 
       Hue[0.32483191760817787`, 1, 1], 
       Hue[0.38977237153196254`, 1, 1], 
       Hue[0.38977237153196254`, 1, 1], 
       Hue[0.4554907569809305, 1, 1], 
       Hue[0.4554907569809305, 1, 1], 
       Hue[0.5219797009100673, 1, 1], 
       Hue[0.5219797009100673, 1, 1], 
       Hue[0.5892341605561915, 1, 1], 
       Hue[0.5892341605561915, 1, 1], 
       Hue[0.657246872448343, 1, 1], 
       Hue[0.657246872448343, 1, 1], 
       Hue[0.7260117989041965, 1, 1], 
       Hue[0.7260117989041965, 1, 1], 
       Hue[0.7955250253211213, 1, 1], 
       Hue[0.7955250253211213, 1, 1], 
       Hue[0.8657797896021193, 1, 1], 
       Hue[0.8657797896021193, 1, 1], 
       Hue[0.9367719499178244, 1, 1], 
       Hue[0.9367719499178244, 1, 1], 
       Hue[0.008495195879954513, 1, 1], 
       Hue[0.008495195879954513, 1, 1], 
       Hue[0.0809459758358265, 1, 1], 
       Hue[0.0809459758358265, 1, 1], 
       Hue[0.15412001963161848`, 1, 1], 
       Hue[0.15412001963161848`, 1, 1], 
       Hue[0.2280115923818089, 1, 1], 
       Hue[0.2280115923818089, 1, 1], 
       Hue[0.3026172384645148, 1, 1], 
       Hue[0.3026172384645148, 1, 1], 
       Hue[0.3779292229301128, 1, 1], 
       Hue[0.3779292229301128, 1, 1], 
       Hue[0.4539469430882079, 1, 1], 
       Hue[0.4539469430882079, 1, 1], 
       Hue[0.5306651254733961, 1, 1], 
       Hue[0.5306651254733961, 1, 1], 
       Hue[0.6080786482755536, 1, 1], 
       Hue[0.6080786482755536, 1, 1], 
       Hue[0.6861849685526418, 1, 1], 
       Hue[0.6861849685526418, 1, 1], 
       Hue[0.7649800556325819, 1, 1], 
       Hue[0.7649800556325819, 1, 1], 
       Hue[0.8444590226034463, 1, 1], 
       Hue[0.8444590226034463, 1, 1], 
       Hue[0.9246179861927123, 1, 1], 
       Hue[0.9246179861927123, 1, 1], 
       Hue[0.0054564117259276864`, 1, 1], 
       Hue[0.0054564117259276864`, 1, 1], 
       Hue[0.08696888006741216, 1, 1], 
       Hue[0.08696888006741216, 1, 1], 
       Hue[0.16914750535384115`, 1, 1], 
       Hue[0.16914750535384115`, 1, 1], 
       Hue[0.2519914361670223, 1, 1], 
       Hue[0.2519914361670223, 1, 1], 
       Hue[0.3355017653036084, 1, 1], 
       Hue[0.3355017653036084, 1, 1], 
       Hue[0.41967365987477934`, 1, 1], 
       Hue[0.41967365987477934`, 1, 1], 
       Hue[0.5045004497746628, 1, 1], 
       Hue[0.5045004497746628, 1, 1], 
       Hue[0.5899817557355067, 1, 1], 
       Hue[0.5899817557355067, 1, 1], 
       Hue[0.6761137981804346, 1, 1], 
       Hue[0.6761137981804346, 1, 1], 
       Hue[0.7628854856167173, 1, 1], 
       Hue[0.7628854856167173, 1, 1], 
       Hue[0.8503116541986834, 1, 1], 
       Hue[0.8503116541986834, 1, 1], 
       Hue[0.9383888140536001, 1, 1], 
       Hue[0.9383888140536001, 1, 1], 
       Hue[0.02709017049981145, 1, 1], 
       Hue[0.02709017049981145, 1, 1], 
       Hue[0.11642414389020968`, 1, 1], 
       Hue[0.11642414389020968`, 1, 1], 
       Hue[0.20640715237073284`, 1, 1], 
       Hue[0.20640715237073284`, 1, 1], 
       Hue[0.297017601531361, 1, 1], 
       Hue[0.297017601531361, 1, 1], 
       Hue[0.38824632762828315`, 1, 1], 
       Hue[0.38824632762828315`, 1, 1], 
       Hue[0.4801090758515208, 1, 1], 
       Hue[0.4801090758515208, 1, 1], 
       Hue[0.5725864431600319, 1, 1], 
       Hue[0.5725864431600319, 1, 1], 
       Hue[0.665687847174255, 1, 1], 
       Hue[0.665687847174255, 1, 1], 
       Hue[0.7594259975222694, 1, 1], 
       Hue[0.7594259975222694, 1, 1], 
       Hue[0.8537592849553937, 1, 1], 
       Hue[0.8537592849553937, 1, 1], 
       Hue[0.948706513230035, 1, 1], 
       Hue[0.948706513230035, 1, 1], 
       Hue[0.04426775150904171, 1, 1], 
       Hue[0.04426775150904171, 1, 1], 
       Hue[0.1404108073613589, 1, 1], 
       Hue[0.1404108073613589, 1, 1], 
       Hue[0.237191244358674, 1, 1], 
       Hue[0.237191244358674, 1, 1], 
       Hue[0.3346110854878864, 1, 1], 
       Hue[0.3346110854878864, 1, 1], 
       Hue[0.43260862207299355`, 1, 1], 
       Hue[0.43260862207299355`, 1, 1], 
       Hue[0.5311723937875716, 1, 1], 
       Hue[0.5311723937875716, 1, 1], 
       Hue[0.6303165816593157, 1, 1], 
       Hue[0.6303165816593157, 1, 1], 
       Hue[0.7300926755163386, 1, 1], 
       Hue[0.7300926755163386, 1, 1], 
       Hue[0.830492702282897, 1, 1]}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->{True, None},
    AxesOrigin->{0, 0},
    Epilog->
     LineBox[{{1, 4.215889287240056*^-7}, {2, 7.385109444239298*^-7}, {
       3, 7.48374071250106*^-7}, {4, 2.6652854374760806`*^-7}, {
       5, 4.798609177591426*^-7}, {6, 3.532519923160807*^-7}, {
       7, 7.123917481828207*^-7}, {8, 6.909353181437822*^-7}, {
       9, 1.9759178912124717`*^-7}, {10, 2.393601123186988*^-7}, {
       11, 4.828020902023037*^-7}, {12, 8.773562552193309*^-7}, {
       13, 4.248603103100271*^-8}, {14, 1.477728486073649*^-6}, {
       15, 1.5846186351308708`*^-6}, {16, 1.69166190676397*^-6}, {
       17, 2.34058125304706*^-6}, {18, 3.554587234487345*^-6}, {
       19, 3.831295720116572*^-6}, {20, 4.731550913422753*^-6}, {
       21, 5.9575568633773365`*^-6}, {22, 7.3826308054585405`*^-6}, {
       23, 9.530826377004312*^-6}, {24, 0.000011463024279875113`}, {
       25, 0.000013184062642876242`}, {26, 0.0000159089229447806}, {
       27, 0.000019595835566520584`}, {28, 0.000023108736904217507`}, {
       29, 0.000028166462883705582`}, {30, 0.00003315047789316038}, {
       31, 0.000040285191412516525`}, {32, 0.000048272136165939263`}, {
       33, 0.00005701038041956149}, {34, 0.0000682453506668145}, {
       35, 0.00007997113108573856}, {36, 0.00009523751708395127}, {
       37, 0.00011356382394225856`}, {38, 0.00013311779268841388`}, {
       39, 0.0001578317207724534}, {40, 0.00018507493658040212`}, {
       41, 0.00021774480566805912`}, {42, 0.0002557736635614455}, {
       43, 0.00029904170290528476`}, {44, 0.0003498760289205983}, {
       45, 0.00040801102113755065`}, {46, 0.0004759445049697905}, {
       47, 0.0005543224411507603}, {48, 0.0006425758268428648}, {
       49, 0.000745431425894238}, {50, 0.0008631067028350429}, {
       51, 0.0009969789551432051}, {52, 0.0011504304418261725`}, {
       53, 0.0013256681176326212`}, {54, 0.0015242978070044162`}, {
       55, 0.0017503527918363007`}, {56, 0.002007248413198682}, {
       57, 0.0022979630591541153`}, {58, 0.002626312689812334}, {
       59, 0.0029962275891283236`}, {60, 0.0034127797367231485`}, {
       61, 0.0038803848474610668`}, {62, 0.004404852813413841}, {
       63, 0.0049919366631887814`}, {64, 0.0056480662653384086`}, {
       65, 0.006379657094258243}, {66, 0.0071942900978688145`}, {
       67, 0.008098769418912096}, {68, 0.009102294715914463}, {
       69, 0.01021143521177961}, {70, 0.011436257957032027`}, {
       71, 0.012786061349124859`}, {72, 0.014268745654730438`}, {
       73, 0.01589614104546038}, {74, 0.01767833770868489}, {
       75, 0.019624460169121425`}, {76, 0.021746717622342312`}, {
       77, 0.024054984909734324`}, {78, 0.026558748346400075`}, {
       79, 0.029270303800456907`}, {80, 0.03219841963981317}, {
       81, 0.03535409367203304}, {82, 0.03874723365223627}, {
       83, 0.04238699729530048}, {84, 0.046281460250175044`}, {
       85, 0.05043665357110623}, {86, 0.05486012978319206}, {
       87, 0.05955704623884705}, {88, 0.0645328826156716}, {
       89, 0.06978965755752117}, {90, 0.07532973747245739}, {
       91, 0.08114959140450825}, {92, 0.08724851626590913}, {
       93, 0.09361915039560055}, {94, 0.10025689006048194`}, {
       95, 0.1071560545246914}, {96, 0.11430844585670448`}, {
       97, 0.1217033838577436}, {98, 0.12932311012371334`}, {
       99, 0.13714631260939314`}, {100, 0.14515314446069377`}, {
       101, 0.1533225483246502}, {102, 0.16164517698975367`}, {
       103, 0.170107823104253}, {104, 0.17869580288708334`}, {
       105, 0.1873811868573901}, {106, 0.19612157929459}, {
       107, 0.2048782210202883}, {108, 0.21362578329327053`}, {
       109, 0.22236768199526083`}, {110, 0.2311279808598423}, {
       111, 0.23992239248151626`}, {112, 0.24873665542739315`}, {
       113, 0.25751020274435915`}, {114, 0.2661673454572213}, {
       115, 0.2746601312010308}, {116, 0.2830112541443783}, {
       117, 0.29132449869745236`}, {118, 0.29973787954650555`}, {
       119, 0.3083481319002654}, {120, 0.3171415798200614}, {
       121, 0.32596780260225977`}, {122, 0.3345722322862818}, {
       123, 0.34267581651223095`}, {124, 0.3500500412713707}, {
       125, 0.3565744228400174}, {126, 0.36225350837323567`}, {
       127, 0.36719096937395757`}, {128, 0.37155669156323456`}, {
       129, 0.37554513537073886`}, {130, 0.3793342390760843}, {
       131, 0.383070882922276}, {132, 0.38685987576150027`}, {
       133, 0.3907668786756913}, {134, 0.3948211899784529}, {
       135, 0.39902737129769483`}, {136, 0.4033689823506277}, {
       137, 0.4078172008469166}, {138, 0.4123371396033073}, {
       139, 0.4168862802789701}, {140, 0.4214220314019419}, {
       141, 0.425900984459955}, {142, 0.4302779950089401}, {
       143, 0.4345118627144108}, {144, 0.43856150938285676`}, {
       145, 0.4423885570987661}, {146, 0.4459556271664869}, {
       147, 0.44922767103676287`}, {148, 0.4521742493551205}, {
       149, 0.4547645001233238}, {150, 0.4569722752408065}, {
       151, 0.4587738125954219}, {152, 0.4601466464293833}, {
       153, 0.4610721755790764}, {154, 0.46153249695102144`}, {
       155, 0.4615143984795248}, {156, 0.46100938060962837`}, {
       157, 0.4600049132332509}, {158, 0.45849662946161834`}, {
       159, 0.4564828634399432}, {160, 0.45396219768511425`}, {
       161, 0.450936116960236}, {162, 0.44740859123356497`}, {
       163, 0.4433867912658347}, {164, 0.438879462850403}, {
       165, 0.4338985513881026}, {166, 0.42845478754416755`}, {
       167, 0.4225665240673203}, {168, 0.41624864924289}, {
       169, 0.4095187777843303}, {170, 0.4024008964244921}, {
       171, 0.3949125247572609}, {172, 0.3870783569783996}, {
       173, 0.37892367210298405`}, {174, 0.37047182114071425`}, {
       175, 0.36174970352543406`}, {176, 0.352780671349876}, {
       177, 0.343596368276955}, {178, 0.3342206882740408}, {
       179, 0.32468199662872743`}, {180, 0.3150088024522138}, {
       181, 0.3052269417747082}, {182, 0.2953639033841867}, {
       183, 0.2854471251902111}, {184, 0.275501946528256}, {
       185, 0.26555429731156105`}, {186, 0.25562929095379955`}, {
       187, 0.24574980272923372`}, {188, 0.23593916528863643`}, {
       189, 0.2262202951364634}, {190, 0.21661314779490026`}, {
       191, 0.2071363545603469}, {192, 0.1978089932307676}, {
       193, 0.18864847033508253`}, {194, 0.17967008927981218`}, {
       195, 0.17088784556493286`}, {196, 0.1623168684138531}, {
       197, 0.15396397886999108`}, {198, 0.14584376484138376`}, {
       199, 0.13796257681651314`}, {200, 0.1303298695954158}, {
       201, 0.12295059035179237`}, {202, 0.1158297673681628}, {
       203, 0.10897126286438472`}, {204, 0.1023770354799672}, {
       205, 0.09604813521545022}, {206, 0.08998563300230253}, {
       207, 0.08418782678720049}, {208, 0.07865477049212187}, {
       209, 0.07338171811921562}, {210, 0.06836641389119585}, {
       211, 0.06360356224483525}, {212, 0.05908912332527798}, {
       213, 0.05481777429581342}, {214, 0.050782717577456456`}, {
       215, 0.04697788285463465}, {216, 0.04339621374915486}, {
       217, 0.04003011539821377}, {218, 0.03687195775869723}, {
       219, 0.0339143293995455}, {220, 0.03114900901600543}, {
       221, 0.02856720206032563}, {222, 0.02616122116658831}, {
       223, 0.023923567369434084`}, {224, 0.021845571235873974`}, {
       225, 0.0199186159693755}, {226, 0.018134829043343997`}, {
       227, 0.01648668676323527}, {228, 0.014965779804103633`}, {
       229, 0.013565434145760535`}, {230, 0.012278629494649397`}, {
       231, 0.01109639496970884}, {232, 0.010012581937976927`}, {
       233, 0.00902195676969396}, {234, 0.008117145537325893}, {
       235, 0.007291412425290575}, {236, 0.0065400445677629695`}, {
       237, 0.005857281883346607}, {238, 0.005237637309735599}, {
       239, 0.004676884487185301}, {240, 0.0041693991956940945`}, {
       241, 0.0037109828014421893`}, {242, 0.0032982276237409273`}, {
       243, 0.0029265655390342407`}, {244, 0.0025930137278625865`}, {
       245, 0.0022943769171634154`}, {246, 0.0020265170894258814`}, {
       247, 0.0017867259123326017`}, {248, 0.001572863957162767}, {
       249, 0.0013826901707517044`}, {250, 0.001213800584536969}, {
       251, 0.0010634958312205779`}}],
    Frame->True,
    FrameTicks->{{{1., 
        FormBox[
         RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
        FormBox[
         RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
        FormBox[
         RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
        FormBox[
         RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
        FormBox["0", TraditionalForm]}, {157.25, 
        FormBox["2.5`", TraditionalForm]}, {188.5, 
        FormBox["5", TraditionalForm]}, {219.75, 
        FormBox["7.5`", TraditionalForm]}, {251., 
        FormBox["10", TraditionalForm]}}, Automatic, None, None},
    PlotRange->{-0.1, 1.1},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], Automatic}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF13lcTXkfB/CTnUhGxCAHaVJJ04Ip8qksJWkvlep025db9i3LUcY+2dcx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     "], {
      {GrayLevel[0.5], EdgeForm[None], 
       GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ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         "]]]}, {{}, {}, {
        {Hue[0.9510900371407752, 1, 1], LineBox[{1, 2}]}, 
        {Hue[0.6484361366317299, 1, 1], LineBox[{2, 3}]}, 
        {Hue[0.5882020378479695, 1, 1], LineBox[{3, 4}]}, 
        {Hue[0.7008368986972322, 1, 1], LineBox[{4, 5}]}, 
        {Hue[0.5201140580590689, 1, 1], LineBox[{5, 6}]}, 
        {Hue[0.41353824718892657`, 1, 1], LineBox[{6, 7}]}, 
        {Hue[0.2895856188102126, 1, 1], LineBox[{7, 8}]}, 
        {Hue[0.16031823714158067`, 1, 1], LineBox[{8, 9}]}, 
        {Hue[0.06451994409427123, 1, 1], LineBox[{9, 10}]}, 
        {Hue[0.9714549908564133, 1, 1], LineBox[{10, 11}]}, 
        {Hue[0.8495956515111321, 1, 1], LineBox[{11, 12}]}, 
        {Hue[0.7600017408631082, 1, 1], LineBox[{12, 13}]}, 
        {Hue[0.6660078883104852, 1, 1], LineBox[{13, 14}]}, 
        {Hue[0.5667738672038776, 1, 1], LineBox[{14, 15}]}, 
        {Hue[0.46747375697182053`, 1, 1], LineBox[{15, 16}]}, 
        {Hue[0.3613541730677306, 1, 1], LineBox[{16, 17}]}, 
        {Hue[0.2556959501890631, 1, 1], LineBox[{17, 18}]}, 
        {Hue[0.15091913030375073`, 1, 1], LineBox[{18, 19}]}, 
        {Hue[0.05350912890499731, 1, 1], LineBox[{19, 20}]}, 
        {Hue[0.9485998845077259, 1, 1], LineBox[{20, 21}]}, 
        {Hue[0.8458132399311028, 1, 1], LineBox[{21, 22}]}, 
        {Hue[0.7464255892667264, 1, 1], LineBox[{22, 23}]}, 
        {Hue[0.6438737768454463, 1, 1], LineBox[{23, 24}]}, 
        {Hue[0.5448327863032787, 1, 1], LineBox[{24, 25}]}, 
        {Hue[0.44362298657761645`, 1, 1], LineBox[{25, 26}]}, 
        {Hue[0.3424111543609978, 1, 1], LineBox[{26, 27}]}, 
        {Hue[0.24250115263866376`, 1, 1], LineBox[{27, 28}]}, 
        {Hue[0.14262103084386551`, 1, 1], LineBox[{28, 29}]}, 
        {Hue[0.04215699495357819, 1, 1], LineBox[{29, 30}]}, 
        {Hue[0.9430897023244098, 1, 1], LineBox[{30, 31}]}, 
        {Hue[0.8433311435994333, 1, 1], LineBox[{31, 32}]}, 
        {Hue[0.7441450909718147, 1, 1], LineBox[{32, 33}]}, 
        {Hue[0.6455832938839826, 1, 1], LineBox[{33, 34}]}, 
        {Hue[0.547121062223861, 1, 1], LineBox[{34, 35}]}, 
        {Hue[0.44874714130339627`, 1, 1], LineBox[{35, 36}]}, 
        {Hue[0.35090824263074716`, 1, 1], LineBox[{36, 37}]}, 
        {Hue[0.25295460506672574`, 1, 1], LineBox[{37, 38}]}, 
        {Hue[0.15541997209474684`, 1, 1], LineBox[{38, 39}]}, 
        {Hue[0.05812139640804242, 1, 1], LineBox[{39, 40}]}, 
        {Hue[0.9609925689645509, 1, 1], LineBox[{40, 41}]}, 
        {Hue[0.8642354511482375, 1, 1], LineBox[{41, 42}]}, 
        {Hue[0.7676332199817453, 1, 1], LineBox[{42, 43}]}, 
        {Hue[0.6713081978198241, 1, 1], LineBox[{43, 44}]}, 
        {Hue[0.575243248554518, 1, 1], LineBox[{44, 45}]}, 
        {Hue[0.4793506629061221, 1, 1], LineBox[{45, 46}]}, 
        {Hue[0.3837279679573816, 1, 1], LineBox[{46, 47}]}, 
        {Hue[0.2883292695427502, 1, 1], LineBox[{47, 48}]}, 
        {Hue[0.1931101132324467, 1, 1], LineBox[{48, 49}]}, 
        {Hue[0.09812102246936363, 1, 1], LineBox[{49, 50}]}, 
        {Hue[0.0033744796127046918`, 1, 1], LineBox[{50, 51}]}, 
        {Hue[0.9088358401103226, 1, 1], LineBox[{51, 52}]}, 
        {Hue[0.8145021934498513, 1, 1], LineBox[{52, 53}]}, 
        {Hue[0.7204197384565719, 1, 1], LineBox[{53, 54}]}, 
        {Hue[0.6265192014901787, 1, 1], LineBox[{54, 55}]}, 
        {Hue[0.5328296738760616, 1, 1], LineBox[{55, 56}]}, 
        {Hue[0.43936094810292264`, 1, 1], LineBox[{56, 57}]}, 
        {Hue[0.34605544314130104`, 1, 1], LineBox[{57, 58}]}, 
        {Hue[0.25296083618775117`, 1, 1], LineBox[{58, 59}]}, 
        {Hue[0.16006161221491183`, 1, 1], LineBox[{59, 60}]}, 
        {Hue[0.06735816359866889, 1, 1], LineBox[{60, 61}]}, 
        {Hue[0.9748321901010782, 1, 1], LineBox[{61, 62}]}, 
        {Hue[0.8824954713004484, 1, 1], LineBox[{62, 63}]}, 
        {Hue[0.7903531934070253, 1, 1], LineBox[{63, 64}]}, 
        {Hue[0.6983774005102628, 1, 1], LineBox[{64, 65}]}, 
        {Hue[0.6065873723032656, 1, 1], LineBox[{65, 66}]}, 
        {Hue[0.5149750446861332, 1, 1], LineBox[{66, 67}]}, 
        {Hue[0.4235279081245064, 1, 1], LineBox[{67, 68}]}, 
        {Hue[0.3322425528657921, 1, 1], LineBox[{68, 69}]}, 
        {Hue[0.24112435275911037`, 1, 1], LineBox[{69, 70}]}, 
        {Hue[0.1501653187547825, 1, 1], LineBox[{70, 71}]}, 
        {Hue[0.059361581806637885`, 1, 1], LineBox[{71, 72}]}, 
        {Hue[0.9687129393384285, 1, 1], LineBox[{72, 73}]}, 
        {Hue[0.8782140429001116, 1, 1], LineBox[{73, 74}]}, 
        {Hue[0.787862506183403, 1, 1], LineBox[{74, 75}]}, 
        {Hue[0.6976545951114453, 1, 1], LineBox[{75, 76}]}, 
        {Hue[0.6075843114664345, 1, 1], LineBox[{76, 77}]}, 
        {Hue[0.5176506612513734, 1, 1], LineBox[{77, 78}]}, 
        {Hue[0.4278474844771643, 1, 1], LineBox[{78, 79}]}, 
        {Hue[0.3381731805405101, 1, 1], LineBox[{79, 80}]}, 
        {Hue[0.24862091634891256`, 1, 1], LineBox[{80, 81}]}, 
        {Hue[0.15918999842875362`, 1, 1], LineBox[{81, 82}]}, 
        {Hue[0.0698751897027739, 1, 1], LineBox[{82, 83}]}, 
        {Hue[0.980675785967886, 1, 1], LineBox[{83, 84}]}, 
        {Hue[0.8915862507778612, 1, 1], LineBox[{84, 85}]}, 
        {Hue[0.8026015226029558, 1, 1], LineBox[{85, 86}]}, 
        {Hue[0.713720578442604, 1, 1], LineBox[{86, 87}]}, 
        {Hue[0.6249388091684106, 1, 1], LineBox[{87, 88}]}, 
        {Hue[0.536254069922283, 1, 1], LineBox[{88, 89}]}, 
        {Hue[0.4476648163707883, 1, 1], LineBox[{89, 90}]}, 
        {Hue[0.3591665087143565, 1, 1], LineBox[{90, 91}]}, 
        {Hue[0.2707589943993129, 1, 1], LineBox[{91, 92}]}, 
        {Hue[0.18243795461731413`, 1, 1], LineBox[{92, 93}]}, 
        {Hue[0.09420522987410568, 1, 1], LineBox[{93, 94}]}, 
        {Hue[0.0060603981071102615`, 1, 1], LineBox[{94, 95}]}, 
        {Hue[0.9180051669045146, 1, 1], LineBox[{95, 96}]}, 
        {Hue[0.8300432916737577, 1, 1], LineBox[{96, 97}]}, 
        {Hue[0.7421771414291274, 1, 1], LineBox[{97, 98}]}, 
        {Hue[0.6544112325841327, 1, 1], LineBox[{98, 99}]}, 
        {Hue[0.5667496960540034, 1, 1], LineBox[{99, 100}]}, 
        {Hue[0.47920205752405715`, 1, 1], LineBox[{100, 101}]}, 
        {Hue[0.39178214935172256`, 1, 1], LineBox[{101, 102}]}, 
        {Hue[0.3045062754819063, 1, 1], LineBox[{102, 103}]}, 
        {Hue[0.21739684886962882`, 1, 1], LineBox[{103, 104}]}, 
        {Hue[0.13047574231371242`, 1, 1], LineBox[{104, 105}]}, 
        {Hue[0.04376797122875783, 1, 1], LineBox[{105, 106}]}, 
        {Hue[0.9573010894767487, 1, 1], LineBox[{106, 107}]}, 
        {Hue[0.8711118842833472, 1, 1], LineBox[{107, 108}]}, 
        {Hue[0.7852469427284352, 1, 1], LineBox[{108, 109}]}, 
        {Hue[0.6997674748187139, 1, 1], LineBox[{109, 110}]}, 
        {Hue[0.614745443539114, 1, 1], LineBox[{110, 111}]}, 
        {Hue[0.5302588229741672, 1, 1], LineBox[{111, 112}]}, 
        {Hue[0.44638856015757233`, 1, 1], LineBox[{112, 113}]}, 
        {Hue[0.3632192567550071, 1, 1], LineBox[{113, 114}]}, 
        {Hue[0.28084379762907674`, 1, 1], LineBox[{114, 115}]}, 
        {Hue[0.1993762420489269, 1, 1], LineBox[{115, 116}]}, 
        {Hue[0.11895902434852888`, 1, 1], LineBox[{116, 117}]}, 
        {Hue[0.039760565769007494`, 1, 1], LineBox[{117, 118}]}, 
        {Hue[0.9619676922291895, 1, 1], LineBox[{118, 119}]}, 
        {Hue[0.8857671042243551, 1, 1], LineBox[{119, 120}]}, 
        {Hue[0.8113310485318781, 1, 1], LineBox[{120, 121}]}, 
        {Hue[0.7388111219616389, 1, 1], LineBox[{121, 122}]}, 
        {Hue[0.6683388300052338, 1, 1], LineBox[{122, 123}]}, 
        {Hue[0.6000397402029956, 1, 1], LineBox[{123, 124}]}, 
        {Hue[0.5340405316188332, 1, 1], LineBox[{124, 125}]}, 
        {Hue[0.47048248692041184`, 1, 1], LineBox[{125, 126}]}, 
        {Hue[0.4095173680515788, 1, 1], LineBox[{126, 127}]}, 
        {Hue[0.35130480479686726`, 1, 1], LineBox[{127, 128}]}, 
        {Hue[0.296000726032468, 1, 1], LineBox[{128, 129}]}, 
        {Hue[0.24374770049128452`, 1, 1], LineBox[{129, 130}]}, 
        {Hue[0.194663579371375, 1, 1], LineBox[{130, 131}]}, 
        {Hue[0.1488380422997798, 1, 1], LineBox[{131, 132}]}, 
        {Hue[0.10632763311726107`, 1, 1], LineBox[{132, 133}]}, 
        {Hue[0.06715869091663118, 1, 1], LineBox[{133, 134}]}, 
        {Hue[0.03132826385598596, 1, 1], LineBox[{134, 135}]}, 
        {Hue[0.9988084729013986, 1, 1], LineBox[{135, 136}]}, 
        {Hue[0.9695497880118302, 1, 1], LineBox[{136, 137}]}, 
        {Hue[0.9434863875857, 1, 1], LineBox[{137, 138}]}, 
        {Hue[0.9205386371332858, 1, 1], LineBox[{138, 139}]}, 
        {Hue[0.9006162006692988, 1, 1], LineBox[{139, 140}]}, 
        {Hue[0.8836237766941165, 1, 1], LineBox[{140, 141}]}, 
        {Hue[0.8694597112637464, 1, 1], LineBox[{141, 142}]}, 
        {Hue[0.85802144418141, 1, 1], LineBox[{142, 143}]}, 
        {Hue[0.8492062740198365, 1, 1], LineBox[{143, 144}]}, 
        {Hue[0.8429116052544968, 1, 1], LineBox[{144, 145}]}, 
        {Hue[0.8390381714029644, 1, 1], LineBox[{145, 146}]}, 
        {Hue[0.8374890855147265, 1, 1], LineBox[{146, 147}]}, 
        {Hue[0.8381717639477806, 1, 1], LineBox[{147, 148}]}, 
        {Hue[0.8409968658980163, 1, 1], LineBox[{148, 149}]}, 
        {Hue[0.8458806855527117, 1, 1], LineBox[{149, 150}]}, 
        {Hue[0.8527420684162792, 1, 1], LineBox[{150, 151}]}, 
        {Hue[0.8615061666150565, 1, 1], LineBox[{151, 152}]}, 
        {Hue[0.8721015386835536, 1, 1], LineBox[{152, 153}]}, 
        {Hue[0.8844612676447515, 1, 1], LineBox[{153, 154}]}, 
        {Hue[0.898522586062694, 1, 1], LineBox[{154, 155}]}, 
        {Hue[0.9142262597826759, 1, 1], LineBox[{155, 156}]}, 
        {Hue[0.9315171971116003, 1, 1], LineBox[{156, 157}]}, 
        {Hue[0.9503444226329854, 1, 1], LineBox[{157, 158}]}, 
        {Hue[0.970659049396332, 1, 1], LineBox[{158, 159}]}, 
        {Hue[0.9924165973493029, 1, 1], LineBox[{159, 160}]}, 
        {Hue[0.01557412362952203, 1, 1], LineBox[{160, 161}]}, 
        {Hue[0.04009276576009518, 1, 1], LineBox[{161, 162}]}, 
        {Hue[0.06593516184045774, 1, 1], LineBox[{162, 163}]}, 
        {Hue[0.09306685228699166, 1, 1], LineBox[{163, 164}]}, 
        {Hue[0.1214549126453047, 1, 1], LineBox[{164, 165}]}, 
        {Hue[0.15106943285175675`, 1, 1], LineBox[{165, 166}]}, 
        {Hue[0.18188156318791293`, 1, 1], LineBox[{166, 167}]}, 
        {Hue[0.2138645476271862, 1, 1], LineBox[{167, 168}]}, 
        {Hue[0.24699313254215138`, 1, 1], LineBox[{168, 169}]}, 
        {Hue[0.28124306964296225`, 1, 1], LineBox[{169, 170}]}, 
        {Hue[0.31659254647657986`, 1, 1], LineBox[{170, 171}]}, 
        {Hue[0.3530201981651622, 1, 1], LineBox[{171, 172}]}, 
        {Hue[0.39050554418891786`, 1, 1], LineBox[{172, 173}]}, 
        {Hue[0.4290303734910717, 1, 1], LineBox[{173, 174}]}, 
        {Hue[0.4685763131441898, 1, 1], LineBox[{174, 175}]}, 
        {Hue[0.5091260640647721, 1, 1], LineBox[{175, 176}]}, 
        {Hue[0.5506645361703, 1, 1], LineBox[{176, 177}]}, 
        {Hue[0.5931751705100399, 1, 1], LineBox[{177, 178}]}, 
        {Hue[0.6366445397467777, 1, 1], LineBox[{178, 179}]}, 
        {Hue[0.681058239309198, 1, 1], LineBox[{179, 180}]}, 
        {Hue[0.7264031386886708, 1, 1], LineBox[{180, 181}]}, 
        {Hue[0.7726674425113982, 1, 1], LineBox[{181, 182}]}, 
        {Hue[0.819837918765256, 1, 1], LineBox[{182, 183}]}, 
        {Hue[0.8679043674511565, 1, 1], LineBox[{183, 184}]}, 
        {Hue[0.9168543511248141, 1, 1], LineBox[{184, 185}]}, 
        {Hue[0.9666787266131991, 1, 1], LineBox[{185, 186}]}, 
        {Hue[0.01736632464259125, 1, 1], LineBox[{186, 187}]}, 
        {Hue[0.06890794089973355, 1, 1], LineBox[{187, 188}]}, 
        {Hue[0.12129416957634404`, 1, 1], LineBox[{188, 189}]}, 
        {Hue[0.1745166808135452, 1, 1], LineBox[{189, 190}]}, 
        {Hue[0.2285663347303934, 1, 1], LineBox[{190, 191}]}, 
        {Hue[0.2834351028766379, 1, 1], LineBox[{191, 192}]}, 
        {Hue[0.3391154820244295, 1, 1], LineBox[{192, 193}]}, 
        {Hue[0.3955997937834623, 1, 1], LineBox[{193, 194}]}, 
        {Hue[0.45287936850823385`, 1, 1], LineBox[{194, 195}]}, 
        {Hue[0.5109476098222223, 1, 1], LineBox[{195, 196}]}, 
        {Hue[0.5697982658184998, 1, 1], LineBox[{196, 197}]}, 
        {Hue[0.6294239155644445, 1, 1], LineBox[{197, 198}]}, 
        {Hue[0.689818553661153, 1, 1], LineBox[{198, 199}]}, 
        {Hue[0.750976510534746, 1, 1], LineBox[{199, 200}]}, 
        {Hue[0.8128912929293306, 1, 1], LineBox[{200, 201}]}, 
        {Hue[0.8755568751916872, 1, 1], LineBox[{201, 202}]}, 
        {Hue[0.9389682660024058, 1, 1], LineBox[{202, 203}]}, 
        {Hue[0.0031178192768510317`, 1, 1], LineBox[{203, 204}]}, 
        {Hue[0.06800275159963283, 1, 1], LineBox[{204, 205}]}, 
        {Hue[0.1336157129120437, 1, 1], LineBox[{205, 206}]}, 
        {Hue[0.19995455989782093`, 1, 1], LineBox[{206, 207}]}, 
        {Hue[0.2670094640187561, 1, 1], LineBox[{207, 208}]}, 
        {Hue[0.3347809278038839, 1, 1], LineBox[{208, 209}]}, 
        {Hue[0.4032618549635441, 1, 1], LineBox[{209, 210}]}, 
        {Hue[0.4724477701914594, 1, 1], LineBox[{210, 211}]}, 
        {Hue[0.5423359564001664, 1, 1], LineBox[{211, 212}]}, 
        {Hue[0.6129204992088491, 1, 1], LineBox[{212, 213}]}, 
        {Hue[0.6841963849615734, 1, 1], LineBox[{213, 214}]}, 
        {Hue[0.7561619322423868, 1, 1], LineBox[{214, 215}]}, 
        {Hue[0.82881140408447, 1, 1], LineBox[{215, 216}]}, 
        {Hue[0.902143300822762, 1, 1], LineBox[{216, 217}]}, 
        {Hue[0.9761493691266603, 1, 1], LineBox[{217, 218}]}, 
        {Hue[0.05083015294981264, 1, 1], LineBox[{218, 219}]}, 
        {Hue[0.12617992086590107`, 1, 1], LineBox[{219, 220}]}, 
        {Hue[0.2021963971348071, 1, 1], LineBox[{220, 221}]}, 
        {Hue[0.27887567203613706`, 1, 1], LineBox[{221, 222}]}, 
        {Hue[0.356212734612382, 1, 1], LineBox[{222, 223}]}, 
        {Hue[0.4342080079703386, 1, 1], LineBox[{223, 224}]}, 
        {Hue[0.5128557849609001, 1, 1], LineBox[{224, 225}]}, 
        {Hue[0.5921532077475121, 1, 1], LineBox[{225, 226}]}, 
        {Hue[0.6720964647032827, 1, 1], LineBox[{226, 227}]}, 
        {Hue[0.752681238076534, 1, 1], LineBox[{227, 228}]}, 
        {Hue[0.8339078459155588, 1, 1], LineBox[{228, 229}]}, 
        {Hue[0.9157742987005092, 1, 1], LineBox[{229, 230}]}, 
        {Hue[0.998275547272889, 1, 1], LineBox[{230, 231}]}, 
        {Hue[0.08140743783099906, 1, 1], LineBox[{231, 232}]}, 
        {Hue[0.16516937419891978`, 1, 1], LineBox[{232, 233}]}, 
        {Hue[0.249558861554528, 1, 1], LineBox[{233, 234}]}, 
        {Hue[0.33456939691468257`, 1, 1], LineBox[{234, 235}]}, 
        {Hue[0.42020253714678935`, 1, 1], LineBox[{235, 236}]}, 
        {Hue[0.5064565805732228, 1, 1], LineBox[{236, 237}]}, 
        {Hue[0.5933250672959842, 1, 1], LineBox[{237, 238}]}, 
        {Hue[0.6808033250107832, 1, 1], LineBox[{238, 239}]}, 
        {Hue[0.7688952871616338, 1, 1], LineBox[{239, 240}]}, 
        {Hue[0.8576085005203646, 1, 1], LineBox[{240, 241}]}, 
        {Hue[0.9469223721299237, 1, 1], LineBox[{241, 242}]}, 
        {Hue[0.03683378074896659, 1, 1], LineBox[{242, 243}]}, 
        {Hue[0.12735624017682576`, 1, 1], LineBox[{243, 244}]}, 
        {Hue[0.2184734433831852, 1, 1], LineBox[{244, 245}]}, 
        {Hue[0.31018333537811105`, 1, 1], LineBox[{245, 246}]}, 
        {Hue[0.40251345092619834`, 1, 1], LineBox[{246, 247}]}, 
        {Hue[0.4954396434137711, 1, 1], LineBox[{247, 248}]}, 
        {Hue[0.5889176172549687, 1, 1], LineBox[{248, 249}]}, 
        {Hue[0.6829974887015433, 1, 1], LineBox[{249, 250}]}, 
        {Hue[0.7776940568752733, 1, 1], LineBox[{250, 251}]}}}},
     VertexColors->{
       Hue[0.9510900371407752, 1, 1], 
       Hue[0.6484361366317299, 1, 1], 
       Hue[0.5882020378479695, 1, 1], 
       Hue[0.7008368986972322, 1, 1], 
       Hue[0.5201140580590689, 1, 1], 
       Hue[0.41353824718892657`, 1, 1], 
       Hue[0.2895856188102126, 1, 1], 
       Hue[0.16031823714158067`, 1, 1], 
       Hue[0.06451994409427123, 1, 1], 
       Hue[0.9714549908564133, 1, 1], 
       Hue[0.8495956515111321, 1, 1], 
       Hue[0.7600017408631082, 1, 1], 
       Hue[0.6660078883104852, 1, 1], 
       Hue[0.5667738672038776, 1, 1], 
       Hue[0.46747375697182053`, 1, 1], 
       Hue[0.3613541730677306, 1, 1], 
       Hue[0.2556959501890631, 1, 1], 
       Hue[0.15091913030375073`, 1, 1], 
       Hue[0.05350912890499731, 1, 1], 
       Hue[0.9485998845077259, 1, 1], 
       Hue[0.8458132399311028, 1, 1], 
       Hue[0.7464255892667264, 1, 1], 
       Hue[0.6438737768454463, 1, 1], 
       Hue[0.5448327863032787, 1, 1], 
       Hue[0.44362298657761645`, 1, 1], 
       Hue[0.3424111543609978, 1, 1], 
       Hue[0.24250115263866376`, 1, 1], 
       Hue[0.14262103084386551`, 1, 1], 
       Hue[0.04215699495357819, 1, 1], 
       Hue[0.9430897023244098, 1, 1], 
       Hue[0.8433311435994333, 1, 1], 
       Hue[0.7441450909718147, 1, 1], 
       Hue[0.6455832938839826, 1, 1], 
       Hue[0.547121062223861, 1, 1], 
       Hue[0.44874714130339627`, 1, 1], 
       Hue[0.35090824263074716`, 1, 1], 
       Hue[0.25295460506672574`, 1, 1], 
       Hue[0.15541997209474684`, 1, 1], 
       Hue[0.05812139640804242, 1, 1], 
       Hue[0.9609925689645509, 1, 1], 
       Hue[0.8642354511482375, 1, 1], 
       Hue[0.7676332199817453, 1, 1], 
       Hue[0.6713081978198241, 1, 1], 
       Hue[0.575243248554518, 1, 1], 
       Hue[0.4793506629061221, 1, 1], 
       Hue[0.3837279679573816, 1, 1], 
       Hue[0.2883292695427502, 1, 1], 
       Hue[0.1931101132324467, 1, 1], 
       Hue[0.09812102246936363, 1, 1], 
       Hue[0.0033744796127046918`, 1, 1], 
       Hue[0.9088358401103226, 1, 1], 
       Hue[0.8145021934498513, 1, 1], 
       Hue[0.7204197384565719, 1, 1], 
       Hue[0.6265192014901787, 1, 1], 
       Hue[0.5328296738760616, 1, 1], 
       Hue[0.43936094810292264`, 1, 1], 
       Hue[0.34605544314130104`, 1, 1], 
       Hue[0.25296083618775117`, 1, 1], 
       Hue[0.16006161221491183`, 1, 1], 
       Hue[0.06735816359866889, 1, 1], 
       Hue[0.9748321901010782, 1, 1], 
       Hue[0.8824954713004484, 1, 1], 
       Hue[0.7903531934070253, 1, 1], 
       Hue[0.6983774005102628, 1, 1], 
       Hue[0.6065873723032656, 1, 1], 
       Hue[0.5149750446861332, 1, 1], 
       Hue[0.4235279081245064, 1, 1], 
       Hue[0.3322425528657921, 1, 1], 
       Hue[0.24112435275911037`, 1, 1], 
       Hue[0.1501653187547825, 1, 1], 
       Hue[0.059361581806637885`, 1, 1], 
       Hue[0.9687129393384285, 1, 1], 
       Hue[0.8782140429001116, 1, 1], 
       Hue[0.787862506183403, 1, 1], 
       Hue[0.6976545951114453, 1, 1], 
       Hue[0.6075843114664345, 1, 1], 
       Hue[0.5176506612513734, 1, 1], 
       Hue[0.4278474844771643, 1, 1], 
       Hue[0.3381731805405101, 1, 1], 
       Hue[0.24862091634891256`, 1, 1], 
       Hue[0.15918999842875362`, 1, 1], 
       Hue[0.0698751897027739, 1, 1], 
       Hue[0.980675785967886, 1, 1], 
       Hue[0.8915862507778612, 1, 1], 
       Hue[0.8026015226029558, 1, 1], 
       Hue[0.713720578442604, 1, 1], 
       Hue[0.6249388091684106, 1, 1], 
       Hue[0.536254069922283, 1, 1], 
       Hue[0.4476648163707883, 1, 1], 
       Hue[0.3591665087143565, 1, 1], 
       Hue[0.2707589943993129, 1, 1], 
       Hue[0.18243795461731413`, 1, 1], 
       Hue[0.09420522987410568, 1, 1], 
       Hue[0.0060603981071102615`, 1, 1], 
       Hue[0.9180051669045146, 1, 1], 
       Hue[0.8300432916737577, 1, 1], 
       Hue[0.7421771414291274, 1, 1], 
       Hue[0.6544112325841327, 1, 1], 
       Hue[0.5667496960540034, 1, 1], 
       Hue[0.47920205752405715`, 1, 1], 
       Hue[0.39178214935172256`, 1, 1], 
       Hue[0.3045062754819063, 1, 1], 
       Hue[0.21739684886962882`, 1, 1], 
       Hue[0.13047574231371242`, 1, 1], 
       Hue[0.04376797122875783, 1, 1], 
       Hue[0.9573010894767487, 1, 1], 
       Hue[0.8711118842833472, 1, 1], 
       Hue[0.7852469427284352, 1, 1], 
       Hue[0.6997674748187139, 1, 1], 
       Hue[0.614745443539114, 1, 1], 
       Hue[0.5302588229741672, 1, 1], 
       Hue[0.44638856015757233`, 1, 1], 
       Hue[0.3632192567550071, 1, 1], 
       Hue[0.28084379762907674`, 1, 1], 
       Hue[0.1993762420489269, 1, 1], 
       Hue[0.11895902434852888`, 1, 1], 
       Hue[0.039760565769007494`, 1, 1], 
       Hue[0.9619676922291895, 1, 1], 
       Hue[0.8857671042243551, 1, 1], 
       Hue[0.8113310485318781, 1, 1], 
       Hue[0.7388111219616389, 1, 1], 
       Hue[0.6683388300052338, 1, 1], 
       Hue[0.6000397402029956, 1, 1], 
       Hue[0.5340405316188332, 1, 1], 
       Hue[0.47048248692041184`, 1, 1], 
       Hue[0.4095173680515788, 1, 1], 
       Hue[0.35130480479686726`, 1, 1], 
       Hue[0.296000726032468, 1, 1], 
       Hue[0.24374770049128452`, 1, 1], 
       Hue[0.194663579371375, 1, 1], 
       Hue[0.1488380422997798, 1, 1], 
       Hue[0.10632763311726107`, 1, 1], 
       Hue[0.06715869091663118, 1, 1], 
       Hue[0.03132826385598596, 1, 1], 
       Hue[0.9988084729013986, 1, 1], 
       Hue[0.9695497880118302, 1, 1], 
       Hue[0.9434863875857, 1, 1], 
       Hue[0.9205386371332858, 1, 1], 
       Hue[0.9006162006692988, 1, 1], 
       Hue[0.8836237766941165, 1, 1], 
       Hue[0.8694597112637464, 1, 1], 
       Hue[0.85802144418141, 1, 1], 
       Hue[0.8492062740198365, 1, 1], 
       Hue[0.8429116052544968, 1, 1], 
       Hue[0.8390381714029644, 1, 1], 
       Hue[0.8374890855147265, 1, 1], 
       Hue[0.8381717639477806, 1, 1], 
       Hue[0.8409968658980163, 1, 1], 
       Hue[0.8458806855527117, 1, 1], 
       Hue[0.8527420684162792, 1, 1], 
       Hue[0.8615061666150565, 1, 1], 
       Hue[0.8721015386835536, 1, 1], 
       Hue[0.8844612676447515, 1, 1], 
       Hue[0.898522586062694, 1, 1], 
       Hue[0.9142262597826759, 1, 1], 
       Hue[0.9315171971116003, 1, 1], 
       Hue[0.9503444226329854, 1, 1], 
       Hue[0.970659049396332, 1, 1], 
       Hue[0.9924165973493029, 1, 1], 
       Hue[0.01557412362952203, 1, 1], 
       Hue[0.04009276576009518, 1, 1], 
       Hue[0.06593516184045774, 1, 1], 
       Hue[0.09306685228699166, 1, 1], 
       Hue[0.1214549126453047, 1, 1], 
       Hue[0.15106943285175675`, 1, 1], 
       Hue[0.18188156318791293`, 1, 1], 
       Hue[0.2138645476271862, 1, 1], 
       Hue[0.24699313254215138`, 1, 1], 
       Hue[0.28124306964296225`, 1, 1], 
       Hue[0.31659254647657986`, 1, 1], 
       Hue[0.3530201981651622, 1, 1], 
       Hue[0.39050554418891786`, 1, 1], 
       Hue[0.4290303734910717, 1, 1], 
       Hue[0.4685763131441898, 1, 1], 
       Hue[0.5091260640647721, 1, 1], 
       Hue[0.5506645361703, 1, 1], 
       Hue[0.5931751705100399, 1, 1], 
       Hue[0.6366445397467777, 1, 1], 
       Hue[0.681058239309198, 1, 1], 
       Hue[0.7264031386886708, 1, 1], 
       Hue[0.7726674425113982, 1, 1], 
       Hue[0.819837918765256, 1, 1], 
       Hue[0.8679043674511565, 1, 1], 
       Hue[0.9168543511248141, 1, 1], 
       Hue[0.9666787266131991, 1, 1], 
       Hue[0.01736632464259125, 1, 1], 
       Hue[0.06890794089973355, 1, 1], 
       Hue[0.12129416957634404`, 1, 1], 
       Hue[0.1745166808135452, 1, 1], 
       Hue[0.2285663347303934, 1, 1], 
       Hue[0.2834351028766379, 1, 1], 
       Hue[0.3391154820244295, 1, 1], 
       Hue[0.3955997937834623, 1, 1], 
       Hue[0.45287936850823385`, 1, 1], 
       Hue[0.5109476098222223, 1, 1], 
       Hue[0.5697982658184998, 1, 1], 
       Hue[0.6294239155644445, 1, 1], 
       Hue[0.689818553661153, 1, 1], 
       Hue[0.750976510534746, 1, 1], 
       Hue[0.8128912929293306, 1, 1], 
       Hue[0.8755568751916872, 1, 1], 
       Hue[0.9389682660024058, 1, 1], 
       Hue[0.0031178192768510317`, 1, 1], 
       Hue[0.06800275159963283, 1, 1], 
       Hue[0.1336157129120437, 1, 1], 
       Hue[0.19995455989782093`, 1, 1], 
       Hue[0.2670094640187561, 1, 1], 
       Hue[0.3347809278038839, 1, 1], 
       Hue[0.4032618549635441, 1, 1], 
       Hue[0.4724477701914594, 1, 1], 
       Hue[0.5423359564001664, 1, 1], 
       Hue[0.6129204992088491, 1, 1], 
       Hue[0.6841963849615734, 1, 1], 
       Hue[0.7561619322423868, 1, 1], 
       Hue[0.82881140408447, 1, 1], 
       Hue[0.902143300822762, 1, 1], 
       Hue[0.9761493691266603, 1, 1], 
       Hue[0.05083015294981264, 1, 1], 
       Hue[0.12617992086590107`, 1, 1], 
       Hue[0.2021963971348071, 1, 1], 
       Hue[0.27887567203613706`, 1, 1], 
       Hue[0.356212734612382, 1, 1], 
       Hue[0.4342080079703386, 1, 1], 
       Hue[0.5128557849609001, 1, 1], 
       Hue[0.5921532077475121, 1, 1], 
       Hue[0.6720964647032827, 1, 1], 
       Hue[0.752681238076534, 1, 1], 
       Hue[0.8339078459155588, 1, 1], 
       Hue[0.9157742987005092, 1, 1], 
       Hue[0.998275547272889, 1, 1], 
       Hue[0.08140743783099906, 1, 1], 
       Hue[0.16516937419891978`, 1, 1], 
       Hue[0.249558861554528, 1, 1], 
       Hue[0.33456939691468257`, 1, 1], 
       Hue[0.42020253714678935`, 1, 1], 
       Hue[0.5064565805732228, 1, 1], 
       Hue[0.5933250672959842, 1, 1], 
       Hue[0.6808033250107832, 1, 1], 
       Hue[0.7688952871616338, 1, 1], 
       Hue[0.8576085005203646, 1, 1], 
       Hue[0.9469223721299237, 1, 1], 
       Hue[0.03683378074896659, 1, 1], 
       Hue[0.12735624017682576`, 1, 1], 
       Hue[0.2184734433831852, 1, 1], 
       Hue[0.31018333537811105`, 1, 1], 
       Hue[0.40251345092619834`, 1, 1], 
       Hue[0.4954396434137711, 1, 1], 
       Hue[0.5889176172549687, 1, 1], 
       Hue[0.6829974887015433, 1, 1], 
       Hue[0.7776940568752733, 1, 1], 
       Hue[0.8729551243678357, 1, 1], 
       Hue[0.9510900371407752, 1, 1], 
       Hue[0.6484361366317299, 1, 1], 
       Hue[0.6484361366317299, 1, 1], 
       Hue[0.5882020378479695, 1, 1], 
       Hue[0.5882020378479695, 1, 1], 
       Hue[0.7008368986972322, 1, 1], 
       Hue[0.7008368986972322, 1, 1], 
       Hue[0.5201140580590689, 1, 1], 
       Hue[0.5201140580590689, 1, 1], 
       Hue[0.41353824718892657`, 1, 1], 
       Hue[0.41353824718892657`, 1, 1], 
       Hue[0.2895856188102126, 1, 1], 
       Hue[0.2895856188102126, 1, 1], 
       Hue[0.16031823714158067`, 1, 1], 
       Hue[0.16031823714158067`, 1, 1], 
       Hue[0.06451994409427123, 1, 1], 
       Hue[0.06451994409427123, 1, 1], 
       Hue[0.9714549908564133, 1, 1], 
       Hue[0.9714549908564133, 1, 1], 
       Hue[0.8495956515111321, 1, 1], 
       Hue[0.8495956515111321, 1, 1], 
       Hue[0.7600017408631082, 1, 1], 
       Hue[0.7600017408631082, 1, 1], 
       Hue[0.6660078883104852, 1, 1], 
       Hue[0.6660078883104852, 1, 1], 
       Hue[0.5667738672038776, 1, 1], 
       Hue[0.5667738672038776, 1, 1], 
       Hue[0.46747375697182053`, 1, 1], 
       Hue[0.46747375697182053`, 1, 1], 
       Hue[0.3613541730677306, 1, 1], 
       Hue[0.3613541730677306, 1, 1], 
       Hue[0.2556959501890631, 1, 1], 
       Hue[0.2556959501890631, 1, 1], 
       Hue[0.15091913030375073`, 1, 1], 
       Hue[0.15091913030375073`, 1, 1], 
       Hue[0.05350912890499731, 1, 1], 
       Hue[0.05350912890499731, 1, 1], 
       Hue[0.9485998845077259, 1, 1], 
       Hue[0.9485998845077259, 1, 1], 
       Hue[0.8458132399311028, 1, 1], 
       Hue[0.8458132399311028, 1, 1], 
       Hue[0.7464255892667264, 1, 1], 
       Hue[0.7464255892667264, 1, 1], 
       Hue[0.6438737768454463, 1, 1], 
       Hue[0.6438737768454463, 1, 1], 
       Hue[0.5448327863032787, 1, 1], 
       Hue[0.5448327863032787, 1, 1], 
       Hue[0.44362298657761645`, 1, 1], 
       Hue[0.44362298657761645`, 1, 1], 
       Hue[0.3424111543609978, 1, 1], 
       Hue[0.3424111543609978, 1, 1], 
       Hue[0.24250115263866376`, 1, 1], 
       Hue[0.24250115263866376`, 1, 1], 
       Hue[0.14262103084386551`, 1, 1], 
       Hue[0.14262103084386551`, 1, 1], 
       Hue[0.04215699495357819, 1, 1], 
       Hue[0.04215699495357819, 1, 1], 
       Hue[0.9430897023244098, 1, 1], 
       Hue[0.9430897023244098, 1, 1], 
       Hue[0.8433311435994333, 1, 1], 
       Hue[0.8433311435994333, 1, 1], 
       Hue[0.7441450909718147, 1, 1], 
       Hue[0.7441450909718147, 1, 1], 
       Hue[0.6455832938839826, 1, 1], 
       Hue[0.6455832938839826, 1, 1], 
       Hue[0.547121062223861, 1, 1], 
       Hue[0.547121062223861, 1, 1], 
       Hue[0.44874714130339627`, 1, 1], 
       Hue[0.44874714130339627`, 1, 1], 
       Hue[0.35090824263074716`, 1, 1], 
       Hue[0.35090824263074716`, 1, 1], 
       Hue[0.25295460506672574`, 1, 1], 
       Hue[0.25295460506672574`, 1, 1], 
       Hue[0.15541997209474684`, 1, 1], 
       Hue[0.15541997209474684`, 1, 1], 
       Hue[0.05812139640804242, 1, 1], 
       Hue[0.05812139640804242, 1, 1], 
       Hue[0.9609925689645509, 1, 1], 
       Hue[0.9609925689645509, 1, 1], 
       Hue[0.8642354511482375, 1, 1], 
       Hue[0.8642354511482375, 1, 1], 
       Hue[0.7676332199817453, 1, 1], 
       Hue[0.7676332199817453, 1, 1], 
       Hue[0.6713081978198241, 1, 1], 
       Hue[0.6713081978198241, 1, 1], 
       Hue[0.575243248554518, 1, 1], 
       Hue[0.575243248554518, 1, 1], 
       Hue[0.4793506629061221, 1, 1], 
       Hue[0.4793506629061221, 1, 1], 
       Hue[0.3837279679573816, 1, 1], 
       Hue[0.3837279679573816, 1, 1], 
       Hue[0.2883292695427502, 1, 1], 
       Hue[0.2883292695427502, 1, 1], 
       Hue[0.1931101132324467, 1, 1], 
       Hue[0.1931101132324467, 1, 1], 
       Hue[0.09812102246936363, 1, 1], 
       Hue[0.09812102246936363, 1, 1], 
       Hue[0.0033744796127046918`, 1, 1], 
       Hue[0.0033744796127046918`, 1, 1], 
       Hue[0.9088358401103226, 1, 1], 
       Hue[0.9088358401103226, 1, 1], 
       Hue[0.8145021934498513, 1, 1], 
       Hue[0.8145021934498513, 1, 1], 
       Hue[0.7204197384565719, 1, 1], 
       Hue[0.7204197384565719, 1, 1], 
       Hue[0.6265192014901787, 1, 1], 
       Hue[0.6265192014901787, 1, 1], 
       Hue[0.5328296738760616, 1, 1], 
       Hue[0.5328296738760616, 1, 1], 
       Hue[0.43936094810292264`, 1, 1], 
       Hue[0.43936094810292264`, 1, 1], 
       Hue[0.34605544314130104`, 1, 1], 
       Hue[0.34605544314130104`, 1, 1], 
       Hue[0.25296083618775117`, 1, 1], 
       Hue[0.25296083618775117`, 1, 1], 
       Hue[0.16006161221491183`, 1, 1], 
       Hue[0.16006161221491183`, 1, 1], 
       Hue[0.06735816359866889, 1, 1], 
       Hue[0.06735816359866889, 1, 1], 
       Hue[0.9748321901010782, 1, 1], 
       Hue[0.9748321901010782, 1, 1], 
       Hue[0.8824954713004484, 1, 1], 
       Hue[0.8824954713004484, 1, 1], 
       Hue[0.7903531934070253, 1, 1], 
       Hue[0.7903531934070253, 1, 1], 
       Hue[0.6983774005102628, 1, 1], 
       Hue[0.6983774005102628, 1, 1], 
       Hue[0.6065873723032656, 1, 1], 
       Hue[0.6065873723032656, 1, 1], 
       Hue[0.5149750446861332, 1, 1], 
       Hue[0.5149750446861332, 1, 1], 
       Hue[0.4235279081245064, 1, 1], 
       Hue[0.4235279081245064, 1, 1], 
       Hue[0.3322425528657921, 1, 1], 
       Hue[0.3322425528657921, 1, 1], 
       Hue[0.24112435275911037`, 1, 1], 
       Hue[0.24112435275911037`, 1, 1], 
       Hue[0.1501653187547825, 1, 1], 
       Hue[0.1501653187547825, 1, 1], 
       Hue[0.059361581806637885`, 1, 1], 
       Hue[0.059361581806637885`, 1, 1], 
       Hue[0.9687129393384285, 1, 1], 
       Hue[0.9687129393384285, 1, 1], 
       Hue[0.8782140429001116, 1, 1], 
       Hue[0.8782140429001116, 1, 1], 
       Hue[0.787862506183403, 1, 1], 
       Hue[0.787862506183403, 1, 1], 
       Hue[0.6976545951114453, 1, 1], 
       Hue[0.6976545951114453, 1, 1], 
       Hue[0.6075843114664345, 1, 1], 
       Hue[0.6075843114664345, 1, 1], 
       Hue[0.5176506612513734, 1, 1], 
       Hue[0.5176506612513734, 1, 1], 
       Hue[0.4278474844771643, 1, 1], 
       Hue[0.4278474844771643, 1, 1], 
       Hue[0.3381731805405101, 1, 1], 
       Hue[0.3381731805405101, 1, 1], 
       Hue[0.24862091634891256`, 1, 1], 
       Hue[0.24862091634891256`, 1, 1], 
       Hue[0.15918999842875362`, 1, 1], 
       Hue[0.15918999842875362`, 1, 1], 
       Hue[0.0698751897027739, 1, 1], 
       Hue[0.0698751897027739, 1, 1], 
       Hue[0.980675785967886, 1, 1], 
       Hue[0.980675785967886, 1, 1], 
       Hue[0.8915862507778612, 1, 1], 
       Hue[0.8915862507778612, 1, 1], 
       Hue[0.8026015226029558, 1, 1], 
       Hue[0.8026015226029558, 1, 1], 
       Hue[0.713720578442604, 1, 1], 
       Hue[0.713720578442604, 1, 1], 
       Hue[0.6249388091684106, 1, 1], 
       Hue[0.6249388091684106, 1, 1], 
       Hue[0.536254069922283, 1, 1], 
       Hue[0.536254069922283, 1, 1], 
       Hue[0.4476648163707883, 1, 1], 
       Hue[0.4476648163707883, 1, 1], 
       Hue[0.3591665087143565, 1, 1], 
       Hue[0.3591665087143565, 1, 1], 
       Hue[0.2707589943993129, 1, 1], 
       Hue[0.2707589943993129, 1, 1], 
       Hue[0.18243795461731413`, 1, 1], 
       Hue[0.18243795461731413`, 1, 1], 
       Hue[0.09420522987410568, 1, 1], 
       Hue[0.09420522987410568, 1, 1], 
       Hue[0.0060603981071102615`, 1, 1], 
       Hue[0.0060603981071102615`, 1, 1], 
       Hue[0.9180051669045146, 1, 1], 
       Hue[0.9180051669045146, 1, 1], 
       Hue[0.8300432916737577, 1, 1], 
       Hue[0.8300432916737577, 1, 1], 
       Hue[0.7421771414291274, 1, 1], 
       Hue[0.7421771414291274, 1, 1], 
       Hue[0.6544112325841327, 1, 1], 
       Hue[0.6544112325841327, 1, 1], 
       Hue[0.5667496960540034, 1, 1], 
       Hue[0.5667496960540034, 1, 1], 
       Hue[0.47920205752405715`, 1, 1], 
       Hue[0.47920205752405715`, 1, 1], 
       Hue[0.39178214935172256`, 1, 1], 
       Hue[0.39178214935172256`, 1, 1], 
       Hue[0.3045062754819063, 1, 1], 
       Hue[0.3045062754819063, 1, 1], 
       Hue[0.21739684886962882`, 1, 1], 
       Hue[0.21739684886962882`, 1, 1], 
       Hue[0.13047574231371242`, 1, 1], 
       Hue[0.13047574231371242`, 1, 1], 
       Hue[0.04376797122875783, 1, 1], 
       Hue[0.04376797122875783, 1, 1], 
       Hue[0.9573010894767487, 1, 1], 
       Hue[0.9573010894767487, 1, 1], 
       Hue[0.8711118842833472, 1, 1], 
       Hue[0.8711118842833472, 1, 1], 
       Hue[0.7852469427284352, 1, 1], 
       Hue[0.7852469427284352, 1, 1], 
       Hue[0.6997674748187139, 1, 1], 
       Hue[0.6997674748187139, 1, 1], 
       Hue[0.614745443539114, 1, 1], 
       Hue[0.614745443539114, 1, 1], 
       Hue[0.5302588229741672, 1, 1], 
       Hue[0.5302588229741672, 1, 1], 
       Hue[0.44638856015757233`, 1, 1], 
       Hue[0.44638856015757233`, 1, 1], 
       Hue[0.3632192567550071, 1, 1], 
       Hue[0.3632192567550071, 1, 1], 
       Hue[0.28084379762907674`, 1, 1], 
       Hue[0.28084379762907674`, 1, 1], 
       Hue[0.1993762420489269, 1, 1], 
       Hue[0.1993762420489269, 1, 1], 
       Hue[0.11895902434852888`, 1, 1], 
       Hue[0.11895902434852888`, 1, 1], 
       Hue[0.039760565769007494`, 1, 1], 
       Hue[0.039760565769007494`, 1, 1], 
       Hue[0.9619676922291895, 1, 1], 
       Hue[0.9619676922291895, 1, 1], 
       Hue[0.8857671042243551, 1, 1], 
       Hue[0.8857671042243551, 1, 1], 
       Hue[0.8113310485318781, 1, 1], 
       Hue[0.8113310485318781, 1, 1], 
       Hue[0.7388111219616389, 1, 1], 
       Hue[0.7388111219616389, 1, 1], 
       Hue[0.6683388300052338, 1, 1], 
       Hue[0.6683388300052338, 1, 1], 
       Hue[0.6000397402029956, 1, 1], 
       Hue[0.6000397402029956, 1, 1], 
       Hue[0.5340405316188332, 1, 1], 
       Hue[0.5340405316188332, 1, 1], 
       Hue[0.47048248692041184`, 1, 1], 
       Hue[0.47048248692041184`, 1, 1], 
       Hue[0.4095173680515788, 1, 1], 
       Hue[0.4095173680515788, 1, 1], 
       Hue[0.35130480479686726`, 1, 1], 
       Hue[0.35130480479686726`, 1, 1], 
       Hue[0.296000726032468, 1, 1], 
       Hue[0.296000726032468, 1, 1], 
       Hue[0.24374770049128452`, 1, 1], 
       Hue[0.24374770049128452`, 1, 1], 
       Hue[0.194663579371375, 1, 1], 
       Hue[0.194663579371375, 1, 1], 
       Hue[0.1488380422997798, 1, 1], 
       Hue[0.1488380422997798, 1, 1], 
       Hue[0.10632763311726107`, 1, 1], 
       Hue[0.10632763311726107`, 1, 1], 
       Hue[0.06715869091663118, 1, 1], 
       Hue[0.06715869091663118, 1, 1], 
       Hue[0.03132826385598596, 1, 1], 
       Hue[0.03132826385598596, 1, 1], 
       Hue[0.9988084729013986, 1, 1], 
       Hue[0.9988084729013986, 1, 1], 
       Hue[0.9695497880118302, 1, 1], 
       Hue[0.9695497880118302, 1, 1], 
       Hue[0.9434863875857, 1, 1], 
       Hue[0.9434863875857, 1, 1], 
       Hue[0.9205386371332858, 1, 1], 
       Hue[0.9205386371332858, 1, 1], 
       Hue[0.9006162006692988, 1, 1], 
       Hue[0.9006162006692988, 1, 1], 
       Hue[0.8836237766941165, 1, 1], 
       Hue[0.8836237766941165, 1, 1], 
       Hue[0.8694597112637464, 1, 1], 
       Hue[0.8694597112637464, 1, 1], 
       Hue[0.85802144418141, 1, 1], 
       Hue[0.85802144418141, 1, 1], 
       Hue[0.8492062740198365, 1, 1], 
       Hue[0.8492062740198365, 1, 1], 
       Hue[0.8429116052544968, 1, 1], 
       Hue[0.8429116052544968, 1, 1], 
       Hue[0.8390381714029644, 1, 1], 
       Hue[0.8390381714029644, 1, 1], 
       Hue[0.8374890855147265, 1, 1], 
       Hue[0.8374890855147265, 1, 1], 
       Hue[0.8381717639477806, 1, 1], 
       Hue[0.8381717639477806, 1, 1], 
       Hue[0.8409968658980163, 1, 1], 
       Hue[0.8409968658980163, 1, 1], 
       Hue[0.8458806855527117, 1, 1], 
       Hue[0.8458806855527117, 1, 1], 
       Hue[0.8527420684162792, 1, 1], 
       Hue[0.8527420684162792, 1, 1], 
       Hue[0.8615061666150565, 1, 1], 
       Hue[0.8615061666150565, 1, 1], 
       Hue[0.8721015386835536, 1, 1], 
       Hue[0.8721015386835536, 1, 1], 
       Hue[0.8844612676447515, 1, 1], 
       Hue[0.8844612676447515, 1, 1], 
       Hue[0.898522586062694, 1, 1], 
       Hue[0.898522586062694, 1, 1], 
       Hue[0.9142262597826759, 1, 1], 
       Hue[0.9142262597826759, 1, 1], 
       Hue[0.9315171971116003, 1, 1], 
       Hue[0.9315171971116003, 1, 1], 
       Hue[0.9503444226329854, 1, 1], 
       Hue[0.9503444226329854, 1, 1], 
       Hue[0.970659049396332, 1, 1], 
       Hue[0.970659049396332, 1, 1], 
       Hue[0.9924165973493029, 1, 1], 
       Hue[0.9924165973493029, 1, 1], 
       Hue[0.01557412362952203, 1, 1], 
       Hue[0.01557412362952203, 1, 1], 
       Hue[0.04009276576009518, 1, 1], 
       Hue[0.04009276576009518, 1, 1], 
       Hue[0.06593516184045774, 1, 1], 
       Hue[0.06593516184045774, 1, 1], 
       Hue[0.09306685228699166, 1, 1], 
       Hue[0.09306685228699166, 1, 1], 
       Hue[0.1214549126453047, 1, 1], 
       Hue[0.1214549126453047, 1, 1], 
       Hue[0.15106943285175675`, 1, 1], 
       Hue[0.15106943285175675`, 1, 1], 
       Hue[0.18188156318791293`, 1, 1], 
       Hue[0.18188156318791293`, 1, 1], 
       Hue[0.2138645476271862, 1, 1], 
       Hue[0.2138645476271862, 1, 1], 
       Hue[0.24699313254215138`, 1, 1], 
       Hue[0.24699313254215138`, 1, 1], 
       Hue[0.28124306964296225`, 1, 1], 
       Hue[0.28124306964296225`, 1, 1], 
       Hue[0.31659254647657986`, 1, 1], 
       Hue[0.31659254647657986`, 1, 1], 
       Hue[0.3530201981651622, 1, 1], 
       Hue[0.3530201981651622, 1, 1], 
       Hue[0.39050554418891786`, 1, 1], 
       Hue[0.39050554418891786`, 1, 1], 
       Hue[0.4290303734910717, 1, 1], 
       Hue[0.4290303734910717, 1, 1], 
       Hue[0.4685763131441898, 1, 1], 
       Hue[0.4685763131441898, 1, 1], 
       Hue[0.5091260640647721, 1, 1], 
       Hue[0.5091260640647721, 1, 1], 
       Hue[0.5506645361703, 1, 1], 
       Hue[0.5506645361703, 1, 1], 
       Hue[0.5931751705100399, 1, 1], 
       Hue[0.5931751705100399, 1, 1], 
       Hue[0.6366445397467777, 1, 1], 
       Hue[0.6366445397467777, 1, 1], 
       Hue[0.681058239309198, 1, 1], 
       Hue[0.681058239309198, 1, 1], 
       Hue[0.7264031386886708, 1, 1], 
       Hue[0.7264031386886708, 1, 1], 
       Hue[0.7726674425113982, 1, 1], 
       Hue[0.7726674425113982, 1, 1], 
       Hue[0.819837918765256, 1, 1], 
       Hue[0.819837918765256, 1, 1], 
       Hue[0.8679043674511565, 1, 1], 
       Hue[0.8679043674511565, 1, 1], 
       Hue[0.9168543511248141, 1, 1], 
       Hue[0.9168543511248141, 1, 1], 
       Hue[0.9666787266131991, 1, 1], 
       Hue[0.9666787266131991, 1, 1], 
       Hue[0.01736632464259125, 1, 1], 
       Hue[0.01736632464259125, 1, 1], 
       Hue[0.06890794089973355, 1, 1], 
       Hue[0.06890794089973355, 1, 1], 
       Hue[0.12129416957634404`, 1, 1], 
       Hue[0.12129416957634404`, 1, 1], 
       Hue[0.1745166808135452, 1, 1], 
       Hue[0.1745166808135452, 1, 1], 
       Hue[0.2285663347303934, 1, 1], 
       Hue[0.2285663347303934, 1, 1], 
       Hue[0.2834351028766379, 1, 1], 
       Hue[0.2834351028766379, 1, 1], 
       Hue[0.3391154820244295, 1, 1], 
       Hue[0.3391154820244295, 1, 1], 
       Hue[0.3955997937834623, 1, 1], 
       Hue[0.3955997937834623, 1, 1], 
       Hue[0.45287936850823385`, 1, 1], 
       Hue[0.45287936850823385`, 1, 1], 
       Hue[0.5109476098222223, 1, 1], 
       Hue[0.5109476098222223, 1, 1], 
       Hue[0.5697982658184998, 1, 1], 
       Hue[0.5697982658184998, 1, 1], 
       Hue[0.6294239155644445, 1, 1], 
       Hue[0.6294239155644445, 1, 1], 
       Hue[0.689818553661153, 1, 1], 
       Hue[0.689818553661153, 1, 1], 
       Hue[0.750976510534746, 1, 1], 
       Hue[0.750976510534746, 1, 1], 
       Hue[0.8128912929293306, 1, 1], 
       Hue[0.8128912929293306, 1, 1], 
       Hue[0.8755568751916872, 1, 1], 
       Hue[0.8755568751916872, 1, 1], 
       Hue[0.9389682660024058, 1, 1], 
       Hue[0.9389682660024058, 1, 1], 
       Hue[0.0031178192768510317`, 1, 1], 
       Hue[0.0031178192768510317`, 1, 1], 
       Hue[0.06800275159963283, 1, 1], 
       Hue[0.06800275159963283, 1, 1], 
       Hue[0.1336157129120437, 1, 1], 
       Hue[0.1336157129120437, 1, 1], 
       Hue[0.19995455989782093`, 1, 1], 
       Hue[0.19995455989782093`, 1, 1], 
       Hue[0.2670094640187561, 1, 1], 
       Hue[0.2670094640187561, 1, 1], 
       Hue[0.3347809278038839, 1, 1], 
       Hue[0.3347809278038839, 1, 1], 
       Hue[0.4032618549635441, 1, 1], 
       Hue[0.4032618549635441, 1, 1], 
       Hue[0.4724477701914594, 1, 1], 
       Hue[0.4724477701914594, 1, 1], 
       Hue[0.5423359564001664, 1, 1], 
       Hue[0.5423359564001664, 1, 1], 
       Hue[0.6129204992088491, 1, 1], 
       Hue[0.6129204992088491, 1, 1], 
       Hue[0.6841963849615734, 1, 1], 
       Hue[0.6841963849615734, 1, 1], 
       Hue[0.7561619322423868, 1, 1], 
       Hue[0.7561619322423868, 1, 1], 
       Hue[0.82881140408447, 1, 1], 
       Hue[0.82881140408447, 1, 1], 
       Hue[0.902143300822762, 1, 1], 
       Hue[0.902143300822762, 1, 1], 
       Hue[0.9761493691266603, 1, 1], 
       Hue[0.9761493691266603, 1, 1], 
       Hue[0.05083015294981264, 1, 1], 
       Hue[0.05083015294981264, 1, 1], 
       Hue[0.12617992086590107`, 1, 1], 
       Hue[0.12617992086590107`, 1, 1], 
       Hue[0.2021963971348071, 1, 1], 
       Hue[0.2021963971348071, 1, 1], 
       Hue[0.27887567203613706`, 1, 1], 
       Hue[0.27887567203613706`, 1, 1], 
       Hue[0.356212734612382, 1, 1], 
       Hue[0.356212734612382, 1, 1], 
       Hue[0.4342080079703386, 1, 1], 
       Hue[0.4342080079703386, 1, 1], 
       Hue[0.5128557849609001, 1, 1], 
       Hue[0.5128557849609001, 1, 1], 
       Hue[0.5921532077475121, 1, 1], 
       Hue[0.5921532077475121, 1, 1], 
       Hue[0.6720964647032827, 1, 1], 
       Hue[0.6720964647032827, 1, 1], 
       Hue[0.752681238076534, 1, 1], 
       Hue[0.752681238076534, 1, 1], 
       Hue[0.8339078459155588, 1, 1], 
       Hue[0.8339078459155588, 1, 1], 
       Hue[0.9157742987005092, 1, 1], 
       Hue[0.9157742987005092, 1, 1], 
       Hue[0.998275547272889, 1, 1], 
       Hue[0.998275547272889, 1, 1], 
       Hue[0.08140743783099906, 1, 1], 
       Hue[0.08140743783099906, 1, 1], 
       Hue[0.16516937419891978`, 1, 1], 
       Hue[0.16516937419891978`, 1, 1], 
       Hue[0.249558861554528, 1, 1], 
       Hue[0.249558861554528, 1, 1], 
       Hue[0.33456939691468257`, 1, 1], 
       Hue[0.33456939691468257`, 1, 1], 
       Hue[0.42020253714678935`, 1, 1], 
       Hue[0.42020253714678935`, 1, 1], 
       Hue[0.5064565805732228, 1, 1], 
       Hue[0.5064565805732228, 1, 1], 
       Hue[0.5933250672959842, 1, 1], 
       Hue[0.5933250672959842, 1, 1], 
       Hue[0.6808033250107832, 1, 1], 
       Hue[0.6808033250107832, 1, 1], 
       Hue[0.7688952871616338, 1, 1], 
       Hue[0.7688952871616338, 1, 1], 
       Hue[0.8576085005203646, 1, 1], 
       Hue[0.8576085005203646, 1, 1], 
       Hue[0.9469223721299237, 1, 1], 
       Hue[0.9469223721299237, 1, 1], 
       Hue[0.03683378074896659, 1, 1], 
       Hue[0.03683378074896659, 1, 1], 
       Hue[0.12735624017682576`, 1, 1], 
       Hue[0.12735624017682576`, 1, 1], 
       Hue[0.2184734433831852, 1, 1], 
       Hue[0.2184734433831852, 1, 1], 
       Hue[0.31018333537811105`, 1, 1], 
       Hue[0.31018333537811105`, 1, 1], 
       Hue[0.40251345092619834`, 1, 1], 
       Hue[0.40251345092619834`, 1, 1], 
       Hue[0.4954396434137711, 1, 1], 
       Hue[0.4954396434137711, 1, 1], 
       Hue[0.5889176172549687, 1, 1], 
       Hue[0.5889176172549687, 1, 1], 
       Hue[0.6829974887015433, 1, 1], 
       Hue[0.6829974887015433, 1, 1], 
       Hue[0.7776940568752733, 1, 1], 
       Hue[0.7776940568752733, 1, 1], 
       Hue[0.8729551243678357, 1, 1]}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->{True, None},
    AxesOrigin->{0, 0},
    Epilog->
     LineBox[{{1, 6.428999298885704*^-7}, {2, 8.300133162933977*^-7}, {
       3, 7.781726838131124*^-7}, {4, 4.6764095957323114`*^-7}, {
       5, 1.043910806555466*^-6}, {6, 1.9363121266104814`*^-6}, {
       7, 2.5371543977632263`*^-6}, {8, 2.9470838347403353`*^-6}, {
       9, 3.647988578059276*^-6}, {10, 5.0652959303135986`*^-6}, {
       11, 5.647471884200192*^-6}, {12, 7.440611653765158*^-6}, {
       13, 8.541042412232665*^-6}, {14, 0.000010785657623601734`}, {
       15, 0.000013548039552556158`}, {16, 0.000016389288145999142`}, {
       17, 0.000019370341822961964`}, {18, 0.000023151874274806162`}, {
       19, 0.000027214964775029584`}, {20, 0.00003298613458447244}, {
       21, 0.000038910018195724345`}, {22, 0.000045552561335197265`}, {
       23, 0.00005479060691654177}, {24, 0.00006429922128257208}, {
       25, 0.00007616278239534474}, {26, 0.00009026280868556289}, {
       27, 0.00010603783951001782`}, {28, 0.00012536245386256408`}, {
       29, 0.00014726329321518414`}, {30, 0.00017338687560810236`}, {
       31, 0.0002027286733535676}, {32, 0.0002371285977790524}, {
       33, 0.0002773573152497216}, {34, 0.00032229759398258136`}, {
       35, 0.0003752971395910895}, {36, 0.00043650373002215647`}, {
       37, 0.0005055760066950821}, {38, 0.0005858805900362918}, {
       39, 0.0006775500520590249}, {40, 0.0007826219342702768}, {
       41, 0.0009025388206575009}, {42, 0.001039305480485389}, {
       43, 0.0011956701636203152`}, {44, 0.001372333453100962}, {
       45, 0.0015727408523432125`}, {46, 0.0018001736459574338`}, {
       47, 0.0020573569444218187`}, {48, 0.00234708618430293}, {
       49, 0.002673761595434414}, {50, 0.003041592942615149}, {
       51, 0.0034546082645969473`}, {52, 0.0039174323095321}, {
       53, 0.0044360271345818}, {54, 0.0050145885076479735`}, {
       55, 0.0056602214293028155`}, {56, 0.006378338695183325}, {
       57, 0.0071761982598486295`}, {58, 0.00806089836674371}, {
       59, 0.009039100367280487}, {60, 0.010121896114862725`}, {
       61, 0.011314413403477289`}, {62, 0.012626930974954228`}, {
       63, 0.014069555145806113`}, {64, 0.015649020171983965`}, {
       65, 0.01737713002968264}, {66, 0.01926404616792687}, {
       67, 0.021318576871179752`}, {68, 0.02355248498593144}, {
       69, 0.025974838130848006`}, {70, 0.02859608903775095}, {
       71, 0.031428301565331176`}, {72, 0.03447886426534259}, {
       73, 0.037758732652052956`}, {74, 0.041276655981403156`}, {
       75, 0.0450404461900716}, {76, 0.049058554378111977`}, {
       77, 0.053337067512784796`}, {78, 0.05788217737092372}, {
       79, 0.0626976824505124}, {80, 0.06778815063729648}, {
       81, 0.07315282505761574}, {82, 0.07879337065944557}, {
       83, 0.08470596116609645}, {84, 0.09088890961699303}, {
       85, 0.0973337917778841}, {86, 0.10403398711129828`}, {
       87, 0.11097887172341742`}, {88, 0.11815468760280255`}, {
       89, 0.12554810935190835`}, {90, 0.13313862309563368`}, {
       91, 0.14090767831896422`}, {92, 0.14883345370905351`}, {
       93, 0.15688969021725852`}, {94, 0.16505291151658366`}, {
       95, 0.1732923863175134}, {96, 0.18157883363457714`}, {
       97, 0.18987738440254207`}, {98, 0.19815578080140567`}, {
       99, 0.20638166931513177`}, {100, 0.21452211060539822`}, {
       101, 0.2225470466340991}, {102, 0.2304236458477509}, {
       103, 0.2381170557929398}, {104, 0.24559214920246428`}, {
       105, 0.25281425541466324`}, {106, 0.25975711992967254`}, {
       107, 0.2664048807938117}, {108, 0.2727466756392955}, {
       109, 0.278774745993297}, {110, 0.28447451710706007`}, {
       111, 0.2898232323527047}, {112, 0.2947941444576497}, {
       113, 0.2993768817121557}, {114, 0.3035834295729913}, {
       115, 0.30745293029535886`}, {116, 0.3110359940147565}, {
       117, 0.31436830790722187`}, {118, 0.31744870567324335`}, {
       119, 0.32023568186056617`}, {120, 0.32266584728751735`}, {
       121, 0.3246810154757576}, {122, 0.32626575448776735`}, {
       123, 0.32746468417830193`}, {124, 0.32838004366121354`}, {
       125, 0.3291580087116228}, {126, 0.32995500289739244`}, {
       127, 0.3309153591396115}, {128, 0.3321546296531875}, {
       129, 0.3337488229745268}, {130, 0.3357352960301967}, {
       131, 0.33812513072989614`}, {132, 0.34090558013197403`}, {
       133, 0.3440479812524503}, {134, 0.3475157319581264}, {
       135, 0.35126779411939724`}, {136, 0.3552635995234062}, {
       137, 0.35945605076740555`}, {138, 0.36380309502165364`}, {
       139, 0.3682618170894707}, {140, 0.3727896873575898}, {
       141, 0.3773481146696921}, {142, 0.3818964500724351}, {
       143, 0.386399963189704}, {144, 0.3908213403070935}, {
       145, 0.3951275291388884}, {146, 0.3992872173908136}, {
       147, 0.40326908378486537`}, {148, 0.40704615649228887`}, {
       149, 0.41058933302994277`}, {150, 0.4138767086550128}, {
       151, 0.4168848119349655}, {152, 0.41959003672901507`}, {
       153, 0.42197400083078695`}, {154, 0.4240182342563073}, {
       155, 0.42570842610467846`}, {156, 0.4270284084276151}, {
       157, 0.4279667787715813}, {158, 0.4285117872098575}, {
       159, 0.4286552152486011}, {160, 0.428391072101157}, {
       161, 0.42771182751484066`}, {162, 0.42661412552348493`}, {
       163, 0.42509566140722016`}, {164, 0.42315834601746694`}, {
       165, 0.42080139151857593`}, {166, 0.41802887142544404`}, {
       167, 0.4148446193135151}, {168, 0.41125615472689175`}, {
       169, 0.40727037232262336`}, {170, 0.4028965365295396}, {
       171, 0.39814485447257336`}, {172, 0.39302834073862397`}, {
       173, 0.38755873051365386`}, {174, 0.3817520163820842}, {
       175, 0.3756213983417457}, {176, 0.36918390254305716`}, {
       177, 0.3624575783871716}, {178, 0.3554597230174296}, {
       179, 0.3482104862813334}, {180, 0.3407274621368877}, {
       181, 0.33303186966591164`}, {182, 0.32514403899090183`}, {
       183, 0.3170826846752446}, {184, 0.30887216512654314`}, {
       185, 0.30053109021402163`}, {186, 0.29208089286694594`}, {
       187, 0.28354352534977745`}, {188, 0.27493784463691473`}, {
       189, 0.2662854865892516}, {190, 0.2576057530193071}, {
       191, 0.24891875615456377`}, {192, 0.2402446784624649}, {
       193, 0.23160046630781933`}, {194, 0.22300477914381434`}, {
       195, 0.21447612893178863`}, {196, 0.20602928892683842`}, {
       197, 0.19767930070599377`}, {198, 0.1894417438033198}, {
       199, 0.18133075612402988`}, {200, 0.1733587833081945}, {
       201, 0.1655395991734645}, {202, 0.15788202557966252`}, {
       203, 0.15039717537554026`}, {204, 0.14309223512486796`}, {
       205, 0.13597691847616825`}, {206, 0.12905654372321876`}, {
       207, 0.12233837713459317`}, {208, 0.11582877008669083`}, {
       209, 0.10952930416528646`}, {210, 0.10344509549717357`}, {
       211, 0.09757778376986678}, {212, 0.09192792049054814}, {
       213, 0.0864977348675539}, {214, 0.0812864745877578}, {
       215, 0.07629347556054447}, {216, 0.07151652572376288}, {
       217, 0.06695372335837245}, {218, 0.06260329711239984}, {
       219, 0.05846071053734583}, {220, 0.05452300698730784}, {
       221, 0.050784942749714054`}, {222, 0.04724294396141986}, {
       223, 0.043891502726421776`}, {224, 0.04072478506425511}, {
       225, 0.03773764484303948}, {226, 0.03492385838782062}, {
       227, 0.03227800698373038}, {228, 0.029794069529439693`}, {
       229, 0.02746507363996537}, {230, 0.025284546999225974`}, {
       231, 0.023246418865307334`}, {232, 0.02134442934102172}, {
       233, 0.01957179487247538}, {234, 0.017922448814677548`}, {
       235, 0.016390454128966114`}, {236, 0.014969138688617007`}, {
       237, 0.013652521746965117`}, {238, 0.012435042110557354`}, {
       239, 0.011311441399693273`}, {240, 0.010275117299394908`}, {
       241, 0.009320368359099152}, {242, 0.00844323104166659}, {
       243, 0.007638196675815911}, {244, 0.006900104815892843}, {
       245, 0.006225281234642151}, {246, 0.005608822675763076}, {
       247, 0.005045307563466428}, {248, 0.004532173679180542}, {
       249, 0.004066527000051145}, {250, 0.0036431332194328714`}, {
       251, 0.003258975402727404}}],
    Frame->True,
    FrameTicks->{{{1., 
        FormBox[
         RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
        FormBox[
         RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
        FormBox[
         RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
        FormBox[
         RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
        FormBox["0", TraditionalForm]}, {157.25, 
        FormBox["2.5`", TraditionalForm]}, {188.5, 
        FormBox["5", TraditionalForm]}, {219.75, 
        FormBox["7.5`", TraditionalForm]}, {251., 
        FormBox["10", TraditionalForm]}}, Automatic, None, None},
    PlotRange->{-0.1, 1.1},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], Automatic}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF13lcVXUaBvCTpEHQgKlkpHEkRVTCFRBEeEBkURFZZBcO97JeuGBCLrmd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     "], {
      {GrayLevel[0.5], EdgeForm[None], 
       GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ
2qYUCakkqbSrJBXatNLyfM7n/eM55zkzc2fud+be9/dW7t6/Tb8cEX/DAZBg
P/wDGi8KxRg8EIrhhcKLUvL6worDC0ORgph/4b+CtRorwlzRVLAXFMdLaE+8
FByKHwYHh8dK4gdB8YKcMl4o+awSUBovAyXDlMUPUUyYcnj55L2bwkB8EDQL
n1EBPxxK42XgSPwoKBumEn609lRuUB0/TjmGqYEfDxXw8lAVr5a8VjHH4JWh
XPgbSunbknNXTE28lnLQ2VAfP0k54BWhLn4CHBHOsaI8OVfF1MbrJK9VTAP8
ZKgUjqmHn5i8t3I4Fq+SnKvWnIKfqm9UrtAYP00541XgDPxMqIpXg7Pxc6B6
eOwsvHnyXB24DG8LdXUWNMNPT95LMefi5+kOw7TAz4caeC24GL8Eaof3aIdf
rjsIj7XBL00+SzEt8Qv0BmFa4a2hZjimPd4B6oXpiHfSneCN4Fr8Omisu4Ce
eC/dSXhNZ/wKqB+mC95VbxTmSrwbNAjTHb9KbxC+04Z4o+S71VgP/Orks3TH
TfCmyXfVEPri1yTnphx6432S55RjP7y/3ijM9fgN0CTMAPzG5Fr/C3bRhLuz
e1o1P5i5IXoTvS0Mw2/WG+ttYBw+Xm+kt4UR+K16Y709jMJHqwbCMcPxW5LX
amwMPjZ5L+0xAb8dWuCt4W78HrgwPDYRn6QaCJ9xGz4y+ayWMAW/U28cZip+
F7QKMw2fnry3vukmfGjyt+mMGfi9cJFqC2bh96vGwszGH4A2eFt4FH8M2oV5
HH9CNajahrn4I8lrFTMHf1A1Gf6GyfgdybkrZh7+JLTHO8EifDF0xjvAAvxp
6BjOcSZ+X3KuipmPP5W8VjFL8KWqyXDMQvyZ5L01tgxfDl3CPIs/p5oN8zz+
Alyp2ofN+HvqAbwbvIS/DN3DrMRfUU2HeRVfBT3CrMZfU02H99iCvw/9wmNr
8NehJ94L1uNvQG+8D7yFvw19w2vW4uuS12psI/6OeiIcswF/MzlWY5vwd5O/
RT1/od46+W+JcvgA/1A9o9qA7/EfVCP4INiOfwOD1UvwMf6JeirMp/hnMCDM
5/gX6rEw2/AvYWD4jK34R8l7aewr/Ovks3TnL+Irku96COzEv0vOTTnswL9N
nlOOu/DdMCzMHvxH9WyYn/CfYXiYX/Bf1ZNh9uK/wYgwv+N/qKfxkbAf/wdG
4aNBP9oJxoTH/mXuv+S5SVCauTIwWb0KfzO3L3kvxWTmCsHYMAfghWEcfjuU
xA+CieE9yuKHwB3hsVL4wdlnKaYIXhTGhymGHwgTwjHl8PIwJcyh+GFwJ34v
1MCPh5n43XAsXgXuCa+pgB8OU8NUxI+Au8IciR8F08JUwo+G6eEciuMlsr9N
Y8fglbPPmg318ZPgAXwGVMePy85NOVTFq2XPKceaeC24L0xtvA7MClMXPwHu
D1MPPzH7LJ3RAD8Z5qjW1AN4f1iBz1d/4BfBU/iD0BBvBA+FaYyfBg+HaYI3
hblhmuGnwyPhPS7GL4EF4bEz8DPhUfwxOBs/Bx7Hn4AW+PkwL7zmLLx59lqN
tcQvgCfDMefi52XHaqwV3jr7W/RN1+M3wEvhHNrgl8LT+LPQE+8Fz+FL1YN4
N1iGL4R2+OXwTJj2eAdYFKYj3gkWh+mMXwFLwmdchrfN3ktjXfCu2WfpDU7B
T82+6+XQA786Ozfl0B2/KntOOfbG+8DzYfri18ALYa7Fr8t+W/3mPETvPZz8
26Q7GMDcjfAyvgPW4uvgW/wVuAkfCq/iq2A4fgusDo8Nw2/OnnsNbsNHwhp8
JQzGh2TvpZgR+K3Za7VmFD4aXsfXqUfUG7A+/DfnT3L9K/lvxxswWb0PG8JM
Ue/Cm+GYifik7LUam6rehLfwt9XT6mXYiH+gmlGtwIf4e7of1TJsCa+Zod6D
d8LMVK/BpjCz1FvwbpjZ6l3YHP6mMfhYWBsem6O3zT5LOU3Dp2fn9r5qXLWd
nZtymKveyZ5TjvNU67A1zHz1JnwUZoFqGT4OnzkOH599txpbqNqFT8IsUq3C
p/gXqiHVDmzDv1OOyg2+x79UzahW4Kvw2EbdDfwQjnkRX5G9VmObdDewC/9c
Pabeyj7ra9WQage2h2tuvd4OdobPWKnayV6rsQ166+zcFLNatQTfhFmjWsqu
ZdX0QHxQdi0qh826e9iN79UZ2ht+w/eoJlQL8CP+i3JWrvAr/pPuUHcHP4fv
bIl6Fz4Lx2zVW2Sv1Z7blRv8Hj5zi94y+yztuU13lZ2L1uxQ78EfUXCG9s7O
5U/due46+391nbkMX559t4rZqbvJXqv/4ffoW2Af/j8egu84
         "]]]}, {{}, {}, {
        {Hue[0.4393534391365692, 1, 1], LineBox[{1, 2}]}, 
        {Hue[0.3201372697153097, 1, 1], LineBox[{2, 3}]}, 
        {Hue[0.2232096319850034, 1, 1], LineBox[{3, 4}]}, 
        {Hue[0.10358516356125722`, 1, 1], LineBox[{4, 5}]}, 
        {Hue[0.005372599255212061, 1, 1], LineBox[{5, 6}]}, 
        {Hue[0.9035686603544933, 1, 1], LineBox[{6, 7}]}, 
        {Hue[0.7988842591287169, 1, 1], LineBox[{7, 8}]}, 
        {Hue[0.6980701443109908, 1, 1], LineBox[{8, 9}]}, 
        {Hue[0.5965944068167406, 1, 1], LineBox[{9, 10}]}, 
        {Hue[0.4928555528039237, 1, 1], LineBox[{10, 11}]}, 
        {Hue[0.39265585709506984`, 1, 1], LineBox[{11, 12}]}, 
        {Hue[0.29253193566333907`, 1, 1], LineBox[{12, 13}]}, 
        {Hue[0.1882309916688273, 1, 1], LineBox[{13, 14}]}, 
        {Hue[0.08806702992125463, 1, 1], LineBox[{14, 15}]}, 
        {Hue[0.9854789907817174, 1, 1], LineBox[{15, 16}]}, 
        {Hue[0.8845172055478656, 1, 1], LineBox[{16, 17}]}, 
        {Hue[0.7827332249025398, 1, 1], LineBox[{17, 18}]}, 
        {Hue[0.6824754438075233, 1, 1], LineBox[{18, 19}]}, 
        {Hue[0.5820498737159026, 1, 1], LineBox[{19, 20}]}, 
        {Hue[0.4812617808739333, 1, 1], LineBox[{20, 21}]}, 
        {Hue[0.38155270567862865`, 1, 1], LineBox[{21, 22}]}, 
        {Hue[0.2821086601880322, 1, 1], LineBox[{22, 23}]}, 
        {Hue[0.1819929337444189, 1, 1], LineBox[{23, 24}]}, 
        {Hue[0.08325129391839824, 1, 1], LineBox[{24, 25}]}, 
        {Hue[0.9841663633956332, 1, 1], LineBox[{25, 26}]}, 
        {Hue[0.8857141077554178, 1, 1], LineBox[{26, 27}]}, 
        {Hue[0.7871772298162246, 1, 1], LineBox[{27, 28}]}, 
        {Hue[0.6891118122079494, 1, 1], LineBox[{28, 29}]}, 
        {Hue[0.5912062960826425, 1, 1], LineBox[{29, 30}]}, 
        {Hue[0.4935392024672945, 1, 1], LineBox[{30, 31}]}, 
        {Hue[0.3961935772395639, 1, 1], LineBox[{31, 32}]}, 
        {Hue[0.2989048819635487, 1, 1], LineBox[{32, 33}]}, 
        {Hue[0.20194144472414555`, 1, 1], LineBox[{33, 34}]}, 
        {Hue[0.10517975842540783`, 1, 1], LineBox[{34, 35}]}, 
        {Hue[0.008676147230962257, 1, 1], LineBox[{35, 36}]}, 
        {Hue[0.9123650895683947, 1, 1], LineBox[{36, 37}]}, 
        {Hue[0.8163537610262777, 1, 1], LineBox[{37, 38}]}, 
        {Hue[0.7205515282763565, 1, 1], LineBox[{38, 39}]}, 
        {Hue[0.6249695450063499, 1, 1], LineBox[{39, 40}]}, 
        {Hue[0.5296108292359909, 1, 1], LineBox[{40, 41}]}, 
        {Hue[0.43443600295767226`, 1, 1], LineBox[{41, 42}]}, 
        {Hue[0.3394953919330468, 1, 1], LineBox[{42, 43}]}, 
        {Hue[0.24476243828602443`, 1, 1], LineBox[{43, 44}]}, 
        {Hue[0.15023452379705038`, 1, 1], LineBox[{44, 45}]}, 
        {Hue[0.05591726332051126, 1, 1], LineBox[{45, 46}]}, 
        {Hue[0.9618102877749671, 1, 1], LineBox[{46, 47}]}, 
        {Hue[0.8678849255300521, 1, 1], LineBox[{47, 48}]}, 
        {Hue[0.7741471790260839, 1, 1], LineBox[{48, 49}]}, 
        {Hue[0.6806528796568211, 1, 1], LineBox[{49, 50}]}, 
        {Hue[0.5873248986019096, 1, 1], LineBox[{50, 51}]}, 
        {Hue[0.49418451596794727`, 1, 1], LineBox[{51, 52}]}, 
        {Hue[0.4012513718543245, 1, 1], LineBox[{52, 53}]}, 
        {Hue[0.30847550574319005`, 1, 1], LineBox[{53, 54}]}, 
        {Hue[0.21589285716824616`, 1, 1], LineBox[{54, 55}]}, 
        {Hue[0.12349670073992161`, 1, 1], LineBox[{55, 56}]}, 
        {Hue[0.031259713016860584`, 1, 1], LineBox[{56, 57}]}, 
        {Hue[0.9392091910858187, 1, 1], LineBox[{57, 58}]}, 
        {Hue[0.8473239080259873, 1, 1], LineBox[{58, 59}]}, 
        {Hue[0.7556065340967153, 1, 1], LineBox[{59, 60}]}, 
        {Hue[0.6640608936363054, 1, 1], LineBox[{60, 61}]}, 
        {Hue[0.5726668749139376, 1, 1], LineBox[{61, 62}]}, 
        {Hue[0.48143870204130346`, 1, 1], LineBox[{62, 63}]}, 
        {Hue[0.39036514813311196`, 1, 1], LineBox[{63, 64}]}, 
        {Hue[0.2994379002421825, 1, 1], LineBox[{64, 65}]}, 
        {Hue[0.20865980656813093`, 1, 1], LineBox[{65, 66}]}, 
        {Hue[0.11802481179332475`, 1, 1], LineBox[{66, 67}]}, 
        {Hue[0.02753031297037304, 1, 1], LineBox[{67, 68}]}, 
        {Hue[0.937172298998918, 1, 1], LineBox[{68, 69}]}, 
        {Hue[0.8469470877910599, 1, 1], LineBox[{69, 70}]}, 
        {Hue[0.7568529898751493, 1, 1], LineBox[{70, 71}]}, 
        {Hue[0.6668838918648523, 1, 1], LineBox[{71, 72}]}, 
        {Hue[0.5770355187840224, 1, 1], LineBox[{72, 73}]}, 
        {Hue[0.4873067584282177, 1, 1], LineBox[{73, 74}]}, 
        {Hue[0.3976885574940337, 1, 1], LineBox[{74, 75}]}, 
        {Hue[0.3081836156328967, 1, 1], LineBox[{75, 76}]}, 
        {Hue[0.21877826866051225`, 1, 1], LineBox[{76, 77}]}, 
        {Hue[0.12947777818206113`, 1, 1], LineBox[{77, 78}]}, 
        {Hue[0.040272061779198676`, 1, 1], LineBox[{78, 79}]}, 
        {Hue[0.9511561661238163, 1, 1], LineBox[{79, 80}]}, 
        {Hue[0.8621287005273579, 1, 1], LineBox[{80, 81}]}, 
        {Hue[0.7731830030987756, 1, 1], LineBox[{81, 82}]}, 
        {Hue[0.6843154147654915, 1, 1], LineBox[{82, 83}]}, 
        {Hue[0.5955212624534687, 1, 1], LineBox[{83, 84}]}, 
        {Hue[0.5067941698730325, 1, 1], LineBox[{84, 85}]}, 
        {Hue[0.41813217963473104`, 1, 1], LineBox[{85, 86}]}, 
        {Hue[0.3295280433802835, 1, 1], LineBox[{86, 87}]}, 
        {Hue[0.24097871888163253`, 1, 1], LineBox[{87, 88}]}, 
        {Hue[0.1524804620347105, 1, 1], LineBox[{88, 89}]}, 
        {Hue[0.06402804928679805, 1, 1], LineBox[{89, 90}]}, 
        {Hue[0.9756183067398464, 1, 1], LineBox[{90, 91}]}, 
        {Hue[0.8872471615914255, 1, 1], LineBox[{91, 92}]}, 
        {Hue[0.7989117139209876, 1, 1], LineBox[{92, 93}]}, 
        {Hue[0.7106100480325939, 1, 1], LineBox[{93, 94}]}, 
        {Hue[0.6223405196721529, 1, 1], LineBox[{94, 95}]}, 
        {Hue[0.5341007127148718, 1, 1], LineBox[{95, 96}]}, 
        {Hue[0.44589148097804515`, 1, 1], LineBox[{96, 97}]}, 
        {Hue[0.35771438988960935`, 1, 1], LineBox[{97, 98}]}, 
        {Hue[0.2695704180261129, 1, 1], LineBox[{98, 99}]}, 
        {Hue[0.18146586950745308`, 1, 1], LineBox[{99, 100}]}, 
        {Hue[0.09340746986017222, 1, 1], LineBox[{100, 101}]}, 
        {Hue[0.005404085323135338, 1, 1], LineBox[{101, 102}]}, 
        {Hue[0.9174681454422887, 1, 1], LineBox[{102, 103}]}, 
        {Hue[0.8296137987629909, 1, 1], LineBox[{103, 104}]}, 
        {Hue[0.7418598218721183, 1, 1], LineBox[{104, 105}]}, 
        {Hue[0.6542291760888137, 1, 1], LineBox[{105, 106}]}, 
        {Hue[0.5667513430325848, 1, 1], LineBox[{106, 107}]}, 
        {Hue[0.47946412748234774`, 1, 1], LineBox[{107, 108}]}, 
        {Hue[0.39241165346008955`, 1, 1], LineBox[{108, 109}]}, 
        {Hue[0.3056448023252869, 1, 1], LineBox[{109, 110}]}, 
        {Hue[0.21922055749572877`, 1, 1], LineBox[{110, 111}]}, 
        {Hue[0.1332022361576938, 1, 1], LineBox[{111, 112}]}, 
        {Hue[0.047666850681859045`, 1, 1], LineBox[{112, 113}]}, 
        {Hue[0.9627024797532431, 1, 1], LineBox[{113, 114}]}, 
        {Hue[0.8784143218371727, 1, 1], LineBox[{114, 115}]}, 
        {Hue[0.7949263299068562, 1, 1], LineBox[{115, 116}]}, 
        {Hue[0.7123730704290766, 1, 1], LineBox[{116, 117}]}, 
        {Hue[0.6308993085613088, 1, 1], LineBox[{117, 118}]}, 
        {Hue[0.5506492528355513, 1, 1], LineBox[{118, 119}]}, 
        {Hue[0.4717723067273212, 1, 1], LineBox[{119, 120}]}, 
        {Hue[0.39442199474621936`, 1, 1], LineBox[{120, 121}]}, 
        {Hue[0.3187645584683982, 1, 1], LineBox[{121, 122}]}, 
        {Hue[0.2449824694851805, 1, 1], LineBox[{122, 123}]}, 
        {Hue[0.17327582930586308`, 1, 1], LineBox[{123, 124}]}, 
        {Hue[0.10385569030144716`, 1, 1], LineBox[{124, 125}]}, 
        {Hue[0.03693914308290094, 1, 1], LineBox[{125, 126}]}, 
        {Hue[0.9727328323643722, 1, 1], LineBox[{126, 127}]}, 
        {Hue[0.9114250873976556, 1, 1], LineBox[{127, 128}]}, 
        {Hue[0.8531770723884569, 1, 1], LineBox[{128, 129}]}, 
        {Hue[0.7981179230268365, 1, 1], LineBox[{129, 130}]}, 
        {Hue[0.7463434738319357, 1, 1], LineBox[{130, 131}]}, 
        {Hue[0.6979169230722659, 1, 1], LineBox[{131, 132}]}, 
        {Hue[0.652870401171725, 1, 1], LineBox[{132, 133}]}, 
        {Hue[0.6112085161386327, 1, 1], LineBox[{133, 134}]}, 
        {Hue[0.5729111580662459, 1, 1], LineBox[{134, 135}]}, 
        {Hue[0.5379361605368185, 1, 1], LineBox[{135, 136}]}, 
        {Hue[0.5062229764889836, 1, 1], LineBox[{136, 137}]}, 
        {Hue[0.4776979103140166, 1, 1], LineBox[{137, 138}]}, 
        {Hue[0.45227423713150755`, 1, 1], LineBox[{138, 139}]}, 
        {Hue[0.42985665197459183`, 1, 1], LineBox[{139, 140}]}, 
        {Hue[0.41034533929896294`, 1, 1], LineBox[{140, 141}]}, 
        {Hue[0.39363480451596733`, 1, 1], LineBox[{141, 142}]}, 
        {Hue[0.3796206968339214, 1, 1], LineBox[{142, 143}]}, 
        {Hue[0.36819635676874646`, 1, 1], LineBox[{143, 144}]}, 
        {Hue[0.3592588810689745, 1, 1], LineBox[{144, 145}]}, 
        {Hue[0.35270652242754846`, 1, 1], LineBox[{145, 146}]}, 
        {Hue[0.348441715653491, 1, 1], LineBox[{146, 147}]}, 
        {Hue[0.34637025854678805`, 1, 1], LineBox[{147, 148}]}, 
        {Hue[0.34640317257292524`, 1, 1], LineBox[{148, 149}]}, 
        {Hue[0.3484549273788225, 1, 1], LineBox[{149, 150}]}, 
        {Hue[0.35244536014669253`, 1, 1], LineBox[{150, 151}]}, 
        {Hue[0.35829899021649714`, 1, 1], LineBox[{151, 152}]}, 
        {Hue[0.3659444120977315, 1, 1], LineBox[{152, 153}]}, 
        {Hue[0.37531470466298017`, 1, 1], LineBox[{153, 154}]}, 
        {Hue[0.38634778739783043`, 1, 1], LineBox[{154, 155}]}, 
        {Hue[0.3989849084100206, 1, 1], LineBox[{155, 156}]}, 
        {Hue[0.4131714890798039, 1, 1], LineBox[{156, 157}]}, 
        {Hue[0.4288562177677756, 1, 1], LineBox[{157, 158}]}, 
        {Hue[0.4459917475621054, 1, 1], LineBox[{158, 159}]}, 
        {Hue[0.46453343042731615`, 1, 1], LineBox[{159, 160}]}, 
        {Hue[0.484439541654788, 1, 1], LineBox[{160, 161}]}, 
        {Hue[0.5056713520991165, 1, 1], LineBox[{161, 162}]}, 
        {Hue[0.5281925529249144, 1, 1], LineBox[{162, 163}]}, 
        {Hue[0.5519691183846559, 1, 1], LineBox[{163, 164}]}, 
        {Hue[0.5769693278688665, 1, 1], LineBox[{164, 165}]}, 
        {Hue[0.603163245870465, 1, 1], LineBox[{165, 166}]}, 
        {Hue[0.6305229668447081, 1, 1], LineBox[{166, 167}]}, 
        {Hue[0.6590229110973536, 1, 1], LineBox[{167, 168}]}, 
        {Hue[0.6886380618054015, 1, 1], LineBox[{168, 169}]}, 
        {Hue[0.7193449813186077, 1, 1], LineBox[{169, 170}]}, 
        {Hue[0.7511224993169587, 1, 1], LineBox[{170, 171}]}, 
        {Hue[0.7839493701684175, 1, 1], LineBox[{171, 172}]}, 
        {Hue[0.817806735017353, 1, 1], LineBox[{172, 173}]}, 
        {Hue[0.8526758485656166, 1, 1], LineBox[{173, 174}]}, 
        {Hue[0.8885399846257238, 1, 1], LineBox[{174, 175}]}, 
        {Hue[0.9253823506081306, 1, 1], LineBox[{175, 176}]}, 
        {Hue[0.9631867200833969, 1, 1], LineBox[{176, 177}]}, 
        {Hue[0.0019395945408895489`, 1, 1], LineBox[{177, 178}]}, 
        {Hue[0.04162675481913263, 1, 1], LineBox[{178, 179}]}, 
        {Hue[0.08223416167340272, 1, 1], LineBox[{179, 180}]}, 
        {Hue[0.12375029778114359`, 1, 1], LineBox[{180, 181}]}, 
        {Hue[0.16616162602928783`, 1, 1], LineBox[{181, 182}]}, 
        {Hue[0.20945806400140338`, 1, 1], LineBox[{182, 183}]}, 
        {Hue[0.2536276524015086, 1, 1], LineBox[{183, 184}]}, 
        {Hue[0.2986598507746231, 1, 1], LineBox[{184, 185}]}, 
        {Hue[0.3445451940568987, 1, 1], LineBox[{185, 186}]}, 
        {Hue[0.39127272249841616`, 1, 1], LineBox[{186, 187}]}, 
        {Hue[0.4388341517654788, 1, 1], LineBox[{187, 188}]}, 
        {Hue[0.48722035714046374`, 1, 1], LineBox[{188, 189}]}, 
        {Hue[0.5364224284486252, 1, 1], LineBox[{189, 190}]}, 
        {Hue[0.5864334679766994, 1, 1], LineBox[{190, 191}]}, 
        {Hue[0.6372440491692886, 1, 1], LineBox[{191, 192}]}, 
        {Hue[0.6888480265350307, 1, 1], LineBox[{192, 193}]}, 
        {Hue[0.7412365911076693, 1, 1], LineBox[{193, 194}]}, 
        {Hue[0.7944032577634785, 1, 1], LineBox[{194, 195}]}, 
        {Hue[0.8483407608749416, 1, 1], LineBox[{195, 196}]}, 
        {Hue[0.903042428880277, 1, 1], LineBox[{196, 197}]}, 
        {Hue[0.9585025639644492, 1, 1], LineBox[{197, 198}]}, 
        {Hue[0.014714511057447872`, 1, 1], LineBox[{198, 199}]}, 
        {Hue[0.07167338075811815, 1, 1], LineBox[{199, 200}]}, 
        {Hue[0.12937282649114865`, 1, 1], LineBox[{200, 201}]}, 
        {Hue[0.18780666443835414`, 1, 1], LineBox[{201, 202}]}, 
        {Hue[0.24696913203339063`, 1, 1], LineBox[{202, 203}]}, 
        {Hue[0.30685523730739184`, 1, 1], LineBox[{203, 204}]}, 
        {Hue[0.36745957905062887`, 1, 1], LineBox[{204, 205}]}, 
        {Hue[0.42877769280745087`, 1, 1], LineBox[{205, 206}]}, 
        {Hue[0.4908037925029819, 1, 1], LineBox[{206, 207}]}, 
        {Hue[0.5535350384234963, 1, 1], LineBox[{207, 208}]}, 
        {Hue[0.6169653916379854, 1, 1], LineBox[{208, 209}]}, 
        {Hue[0.6810907257805103, 1, 1], LineBox[{209, 210}]}, 
        {Hue[0.7459063147912967, 1, 1], LineBox[{210, 211}]}, 
        {Hue[0.8114104826374526, 1, 1], LineBox[{211, 212}]}, 
        {Hue[0.8775950861296535, 1, 1], LineBox[{212, 213}]}, 
        {Hue[0.9444588876589283, 1, 1], LineBox[{213, 214}]}, 
        {Hue[0.0119970511399642, 1, 1], LineBox[{214, 215}]}, 
        {Hue[0.08020606705135687, 1, 1], LineBox[{215, 216}]}, 
        {Hue[0.14908284555554469`, 1, 1], LineBox[{216, 217}]}, 
        {Hue[0.2186219206976702, 1, 1], LineBox[{217, 218}]}, 
        {Hue[0.28882197240286506`, 1, 1], LineBox[{218, 219}]}, 
        {Hue[0.35967627266925456`, 1, 1], LineBox[{219, 220}]}, 
        {Hue[0.43118455611638384`, 1, 1], LineBox[{220, 221}]}, 
        {Hue[0.5033433290086244, 1, 1], LineBox[{221, 222}]}, 
        {Hue[0.5761460418402145, 1, 1], LineBox[{222, 223}]}, 
        {Hue[0.6495947288059509, 1, 1], LineBox[{223, 224}]}, 
        {Hue[0.7236843398166319, 1, 1], LineBox[{224, 225}]}, 
        {Hue[0.7984095434383008, 1, 1], LineBox[{225, 226}]}, 
        {Hue[0.873769870187243, 1, 1], LineBox[{226, 227}]}, 
        {Hue[0.9497602611869326, 1, 1], LineBox[{227, 228}]}, 
        {Hue[0.02638015960819491, 1, 1], LineBox[{228, 229}]}, 
        {Hue[0.10362426395156402`, 1, 1], LineBox[{229, 230}]}, 
        {Hue[0.18149402581251858`, 1, 1], LineBox[{230, 231}]}, 
        {Hue[0.25998363569095884`, 1, 1], LineBox[{231, 232}]}, 
        {Hue[0.33908888485147026`, 1, 1], LineBox[{232, 233}]}, 
        {Hue[0.41881004827500784`, 1, 1], LineBox[{233, 234}]}, 
        {Hue[0.4991438872643397, 1, 1], LineBox[{234, 235}]}, 
        {Hue[0.5800901965951237, 1, 1], LineBox[{235, 236}]}, 
        {Hue[0.6616434324791175, 1, 1], LineBox[{236, 237}]}, 
        {Hue[0.7437997538299062, 1, 1], LineBox[{237, 238}]}, 
        {Hue[0.8265609418880175, 1, 1], LineBox[{238, 239}]}, 
        {Hue[0.9099231632063347, 1, 1], LineBox[{239, 240}]}, 
        {Hue[0.9938830944227226, 1, 1], LineBox[{240, 241}]}, 
        {Hue[0.07843854256843062, 1, 1], LineBox[{241, 242}]}, 
        {Hue[0.16358722556755453`, 1, 1], LineBox[{242, 243}]}, 
        {Hue[0.24932940137544837`, 1, 1], LineBox[{243, 244}]}, 
        {Hue[0.33566483509107636`, 1, 1], LineBox[{244, 245}]}, 
        {Hue[0.4225903879562467, 1, 1], LineBox[{245, 246}]}, 
        {Hue[0.5100948438311672, 1, 1], LineBox[{246, 247}]}, 
        {Hue[0.598181316104291, 1, 1], LineBox[{247, 248}]}, 
        {Hue[0.6868528306413736, 1, 1], LineBox[{248, 249}]}, 
        {Hue[0.7761040425171196, 1, 1], LineBox[{249, 250}]}, 
        {Hue[0.8659330990069755, 1, 1], LineBox[{250, 251}]}}}},
     VertexColors->{
       Hue[0.4393534391365692, 1, 1], 
       Hue[0.3201372697153097, 1, 1], 
       Hue[0.2232096319850034, 1, 1], 
       Hue[0.10358516356125722`, 1, 1], 
       Hue[0.005372599255212061, 1, 1], 
       Hue[0.9035686603544933, 1, 1], 
       Hue[0.7988842591287169, 1, 1], 
       Hue[0.6980701443109908, 1, 1], 
       Hue[0.5965944068167406, 1, 1], 
       Hue[0.4928555528039237, 1, 1], 
       Hue[0.39265585709506984`, 1, 1], 
       Hue[0.29253193566333907`, 1, 1], 
       Hue[0.1882309916688273, 1, 1], 
       Hue[0.08806702992125463, 1, 1], 
       Hue[0.9854789907817174, 1, 1], 
       Hue[0.8845172055478656, 1, 1], 
       Hue[0.7827332249025398, 1, 1], 
       Hue[0.6824754438075233, 1, 1], 
       Hue[0.5820498737159026, 1, 1], 
       Hue[0.4812617808739333, 1, 1], 
       Hue[0.38155270567862865`, 1, 1], 
       Hue[0.2821086601880322, 1, 1], 
       Hue[0.1819929337444189, 1, 1], 
       Hue[0.08325129391839824, 1, 1], 
       Hue[0.9841663633956332, 1, 1], 
       Hue[0.8857141077554178, 1, 1], 
       Hue[0.7871772298162246, 1, 1], 
       Hue[0.6891118122079494, 1, 1], 
       Hue[0.5912062960826425, 1, 1], 
       Hue[0.4935392024672945, 1, 1], 
       Hue[0.3961935772395639, 1, 1], 
       Hue[0.2989048819635487, 1, 1], 
       Hue[0.20194144472414555`, 1, 1], 
       Hue[0.10517975842540783`, 1, 1], 
       Hue[0.008676147230962257, 1, 1], 
       Hue[0.9123650895683947, 1, 1], 
       Hue[0.8163537610262777, 1, 1], 
       Hue[0.7205515282763565, 1, 1], 
       Hue[0.6249695450063499, 1, 1], 
       Hue[0.5296108292359909, 1, 1], 
       Hue[0.43443600295767226`, 1, 1], 
       Hue[0.3394953919330468, 1, 1], 
       Hue[0.24476243828602443`, 1, 1], 
       Hue[0.15023452379705038`, 1, 1], 
       Hue[0.05591726332051126, 1, 1], 
       Hue[0.9618102877749671, 1, 1], 
       Hue[0.8678849255300521, 1, 1], 
       Hue[0.7741471790260839, 1, 1], 
       Hue[0.6806528796568211, 1, 1], 
       Hue[0.5873248986019096, 1, 1], 
       Hue[0.49418451596794727`, 1, 1], 
       Hue[0.4012513718543245, 1, 1], 
       Hue[0.30847550574319005`, 1, 1], 
       Hue[0.21589285716824616`, 1, 1], 
       Hue[0.12349670073992161`, 1, 1], 
       Hue[0.031259713016860584`, 1, 1], 
       Hue[0.9392091910858187, 1, 1], 
       Hue[0.8473239080259873, 1, 1], 
       Hue[0.7556065340967153, 1, 1], 
       Hue[0.6640608936363054, 1, 1], 
       Hue[0.5726668749139376, 1, 1], 
       Hue[0.48143870204130346`, 1, 1], 
       Hue[0.39036514813311196`, 1, 1], 
       Hue[0.2994379002421825, 1, 1], 
       Hue[0.20865980656813093`, 1, 1], 
       Hue[0.11802481179332475`, 1, 1], 
       Hue[0.02753031297037304, 1, 1], 
       Hue[0.937172298998918, 1, 1], 
       Hue[0.8469470877910599, 1, 1], 
       Hue[0.7568529898751493, 1, 1], 
       Hue[0.6668838918648523, 1, 1], 
       Hue[0.5770355187840224, 1, 1], 
       Hue[0.4873067584282177, 1, 1], 
       Hue[0.3976885574940337, 1, 1], 
       Hue[0.3081836156328967, 1, 1], 
       Hue[0.21877826866051225`, 1, 1], 
       Hue[0.12947777818206113`, 1, 1], 
       Hue[0.040272061779198676`, 1, 1], 
       Hue[0.9511561661238163, 1, 1], 
       Hue[0.8621287005273579, 1, 1], 
       Hue[0.7731830030987756, 1, 1], 
       Hue[0.6843154147654915, 1, 1], 
       Hue[0.5955212624534687, 1, 1], 
       Hue[0.5067941698730325, 1, 1], 
       Hue[0.41813217963473104`, 1, 1], 
       Hue[0.3295280433802835, 1, 1], 
       Hue[0.24097871888163253`, 1, 1], 
       Hue[0.1524804620347105, 1, 1], 
       Hue[0.06402804928679805, 1, 1], 
       Hue[0.9756183067398464, 1, 1], 
       Hue[0.8872471615914255, 1, 1], 
       Hue[0.7989117139209876, 1, 1], 
       Hue[0.7106100480325939, 1, 1], 
       Hue[0.6223405196721529, 1, 1], 
       Hue[0.5341007127148718, 1, 1], 
       Hue[0.44589148097804515`, 1, 1], 
       Hue[0.35771438988960935`, 1, 1], 
       Hue[0.2695704180261129, 1, 1], 
       Hue[0.18146586950745308`, 1, 1], 
       Hue[0.09340746986017222, 1, 1], 
       Hue[0.005404085323135338, 1, 1], 
       Hue[0.9174681454422887, 1, 1], 
       Hue[0.8296137987629909, 1, 1], 
       Hue[0.7418598218721183, 1, 1], 
       Hue[0.6542291760888137, 1, 1], 
       Hue[0.5667513430325848, 1, 1], 
       Hue[0.47946412748234774`, 1, 1], 
       Hue[0.39241165346008955`, 1, 1], 
       Hue[0.3056448023252869, 1, 1], 
       Hue[0.21922055749572877`, 1, 1], 
       Hue[0.1332022361576938, 1, 1], 
       Hue[0.047666850681859045`, 1, 1], 
       Hue[0.9627024797532431, 1, 1], 
       Hue[0.8784143218371727, 1, 1], 
       Hue[0.7949263299068562, 1, 1], 
       Hue[0.7123730704290766, 1, 1], 
       Hue[0.6308993085613088, 1, 1], 
       Hue[0.5506492528355513, 1, 1], 
       Hue[0.4717723067273212, 1, 1], 
       Hue[0.39442199474621936`, 1, 1], 
       Hue[0.3187645584683982, 1, 1], 
       Hue[0.2449824694851805, 1, 1], 
       Hue[0.17327582930586308`, 1, 1], 
       Hue[0.10385569030144716`, 1, 1], 
       Hue[0.03693914308290094, 1, 1], 
       Hue[0.9727328323643722, 1, 1], 
       Hue[0.9114250873976556, 1, 1], 
       Hue[0.8531770723884569, 1, 1], 
       Hue[0.7981179230268365, 1, 1], 
       Hue[0.7463434738319357, 1, 1], 
       Hue[0.6979169230722659, 1, 1], 
       Hue[0.652870401171725, 1, 1], 
       Hue[0.6112085161386327, 1, 1], 
       Hue[0.5729111580662459, 1, 1], 
       Hue[0.5379361605368185, 1, 1], 
       Hue[0.5062229764889836, 1, 1], 
       Hue[0.4776979103140166, 1, 1], 
       Hue[0.45227423713150755`, 1, 1], 
       Hue[0.42985665197459183`, 1, 1], 
       Hue[0.41034533929896294`, 1, 1], 
       Hue[0.39363480451596733`, 1, 1], 
       Hue[0.3796206968339214, 1, 1], 
       Hue[0.36819635676874646`, 1, 1], 
       Hue[0.3592588810689745, 1, 1], 
       Hue[0.35270652242754846`, 1, 1], 
       Hue[0.348441715653491, 1, 1], 
       Hue[0.34637025854678805`, 1, 1], 
       Hue[0.34640317257292524`, 1, 1], 
       Hue[0.3484549273788225, 1, 1], 
       Hue[0.35244536014669253`, 1, 1], 
       Hue[0.35829899021649714`, 1, 1], 
       Hue[0.3659444120977315, 1, 1], 
       Hue[0.37531470466298017`, 1, 1], 
       Hue[0.38634778739783043`, 1, 1], 
       Hue[0.3989849084100206, 1, 1], 
       Hue[0.4131714890798039, 1, 1], 
       Hue[0.4288562177677756, 1, 1], 
       Hue[0.4459917475621054, 1, 1], 
       Hue[0.46453343042731615`, 1, 1], 
       Hue[0.484439541654788, 1, 1], 
       Hue[0.5056713520991165, 1, 1], 
       Hue[0.5281925529249144, 1, 1], 
       Hue[0.5519691183846559, 1, 1], 
       Hue[0.5769693278688665, 1, 1], 
       Hue[0.603163245870465, 1, 1], 
       Hue[0.6305229668447081, 1, 1], 
       Hue[0.6590229110973536, 1, 1], 
       Hue[0.6886380618054015, 1, 1], 
       Hue[0.7193449813186077, 1, 1], 
       Hue[0.7511224993169587, 1, 1], 
       Hue[0.7839493701684175, 1, 1], 
       Hue[0.817806735017353, 1, 1], 
       Hue[0.8526758485656166, 1, 1], 
       Hue[0.8885399846257238, 1, 1], 
       Hue[0.9253823506081306, 1, 1], 
       Hue[0.9631867200833969, 1, 1], 
       Hue[0.0019395945408895489`, 1, 1], 
       Hue[0.04162675481913263, 1, 1], 
       Hue[0.08223416167340272, 1, 1], 
       Hue[0.12375029778114359`, 1, 1], 
       Hue[0.16616162602928783`, 1, 1], 
       Hue[0.20945806400140338`, 1, 1], 
       Hue[0.2536276524015086, 1, 1], 
       Hue[0.2986598507746231, 1, 1], 
       Hue[0.3445451940568987, 1, 1], 
       Hue[0.39127272249841616`, 1, 1], 
       Hue[0.4388341517654788, 1, 1], 
       Hue[0.48722035714046374`, 1, 1], 
       Hue[0.5364224284486252, 1, 1], 
       Hue[0.5864334679766994, 1, 1], 
       Hue[0.6372440491692886, 1, 1], 
       Hue[0.6888480265350307, 1, 1], 
       Hue[0.7412365911076693, 1, 1], 
       Hue[0.7944032577634785, 1, 1], 
       Hue[0.8483407608749416, 1, 1], 
       Hue[0.903042428880277, 1, 1], 
       Hue[0.9585025639644492, 1, 1], 
       Hue[0.014714511057447872`, 1, 1], 
       Hue[0.07167338075811815, 1, 1], 
       Hue[0.12937282649114865`, 1, 1], 
       Hue[0.18780666443835414`, 1, 1], 
       Hue[0.24696913203339063`, 1, 1], 
       Hue[0.30685523730739184`, 1, 1], 
       Hue[0.36745957905062887`, 1, 1], 
       Hue[0.42877769280745087`, 1, 1], 
       Hue[0.4908037925029819, 1, 1], 
       Hue[0.5535350384234963, 1, 1], 
       Hue[0.6169653916379854, 1, 1], 
       Hue[0.6810907257805103, 1, 1], 
       Hue[0.7459063147912967, 1, 1], 
       Hue[0.8114104826374526, 1, 1], 
       Hue[0.8775950861296535, 1, 1], 
       Hue[0.9444588876589283, 1, 1], 
       Hue[0.0119970511399642, 1, 1], 
       Hue[0.08020606705135687, 1, 1], 
       Hue[0.14908284555554469`, 1, 1], 
       Hue[0.2186219206976702, 1, 1], 
       Hue[0.28882197240286506`, 1, 1], 
       Hue[0.35967627266925456`, 1, 1], 
       Hue[0.43118455611638384`, 1, 1], 
       Hue[0.5033433290086244, 1, 1], 
       Hue[0.5761460418402145, 1, 1], 
       Hue[0.6495947288059509, 1, 1], 
       Hue[0.7236843398166319, 1, 1], 
       Hue[0.7984095434383008, 1, 1], 
       Hue[0.873769870187243, 1, 1], 
       Hue[0.9497602611869326, 1, 1], 
       Hue[0.02638015960819491, 1, 1], 
       Hue[0.10362426395156402`, 1, 1], 
       Hue[0.18149402581251858`, 1, 1], 
       Hue[0.25998363569095884`, 1, 1], 
       Hue[0.33908888485147026`, 1, 1], 
       Hue[0.41881004827500784`, 1, 1], 
       Hue[0.4991438872643397, 1, 1], 
       Hue[0.5800901965951237, 1, 1], 
       Hue[0.6616434324791175, 1, 1], 
       Hue[0.7437997538299062, 1, 1], 
       Hue[0.8265609418880175, 1, 1], 
       Hue[0.9099231632063347, 1, 1], 
       Hue[0.9938830944227226, 1, 1], 
       Hue[0.07843854256843062, 1, 1], 
       Hue[0.16358722556755453`, 1, 1], 
       Hue[0.24932940137544837`, 1, 1], 
       Hue[0.33566483509107636`, 1, 1], 
       Hue[0.4225903879562467, 1, 1], 
       Hue[0.5100948438311672, 1, 1], 
       Hue[0.598181316104291, 1, 1], 
       Hue[0.6868528306413736, 1, 1], 
       Hue[0.7761040425171196, 1, 1], 
       Hue[0.8659330990069755, 1, 1], 
       Hue[0.9563368183458169, 1, 1], 
       Hue[0.4393534391365692, 1, 1], 
       Hue[0.3201372697153097, 1, 1], 
       Hue[0.3201372697153097, 1, 1], 
       Hue[0.2232096319850034, 1, 1], 
       Hue[0.2232096319850034, 1, 1], 
       Hue[0.10358516356125722`, 1, 1], 
       Hue[0.10358516356125722`, 1, 1], 
       Hue[0.005372599255212061, 1, 1], 
       Hue[0.005372599255212061, 1, 1], 
       Hue[0.9035686603544933, 1, 1], 
       Hue[0.9035686603544933, 1, 1], 
       Hue[0.7988842591287169, 1, 1], 
       Hue[0.7988842591287169, 1, 1], 
       Hue[0.6980701443109908, 1, 1], 
       Hue[0.6980701443109908, 1, 1], 
       Hue[0.5965944068167406, 1, 1], 
       Hue[0.5965944068167406, 1, 1], 
       Hue[0.4928555528039237, 1, 1], 
       Hue[0.4928555528039237, 1, 1], 
       Hue[0.39265585709506984`, 1, 1], 
       Hue[0.39265585709506984`, 1, 1], 
       Hue[0.29253193566333907`, 1, 1], 
       Hue[0.29253193566333907`, 1, 1], 
       Hue[0.1882309916688273, 1, 1], 
       Hue[0.1882309916688273, 1, 1], 
       Hue[0.08806702992125463, 1, 1], 
       Hue[0.08806702992125463, 1, 1], 
       Hue[0.9854789907817174, 1, 1], 
       Hue[0.9854789907817174, 1, 1], 
       Hue[0.8845172055478656, 1, 1], 
       Hue[0.8845172055478656, 1, 1], 
       Hue[0.7827332249025398, 1, 1], 
       Hue[0.7827332249025398, 1, 1], 
       Hue[0.6824754438075233, 1, 1], 
       Hue[0.6824754438075233, 1, 1], 
       Hue[0.5820498737159026, 1, 1], 
       Hue[0.5820498737159026, 1, 1], 
       Hue[0.4812617808739333, 1, 1], 
       Hue[0.4812617808739333, 1, 1], 
       Hue[0.38155270567862865`, 1, 1], 
       Hue[0.38155270567862865`, 1, 1], 
       Hue[0.2821086601880322, 1, 1], 
       Hue[0.2821086601880322, 1, 1], 
       Hue[0.1819929337444189, 1, 1], 
       Hue[0.1819929337444189, 1, 1], 
       Hue[0.08325129391839824, 1, 1], 
       Hue[0.08325129391839824, 1, 1], 
       Hue[0.9841663633956332, 1, 1], 
       Hue[0.9841663633956332, 1, 1], 
       Hue[0.8857141077554178, 1, 1], 
       Hue[0.8857141077554178, 1, 1], 
       Hue[0.7871772298162246, 1, 1], 
       Hue[0.7871772298162246, 1, 1], 
       Hue[0.6891118122079494, 1, 1], 
       Hue[0.6891118122079494, 1, 1], 
       Hue[0.5912062960826425, 1, 1], 
       Hue[0.5912062960826425, 1, 1], 
       Hue[0.4935392024672945, 1, 1], 
       Hue[0.4935392024672945, 1, 1], 
       Hue[0.3961935772395639, 1, 1], 
       Hue[0.3961935772395639, 1, 1], 
       Hue[0.2989048819635487, 1, 1], 
       Hue[0.2989048819635487, 1, 1], 
       Hue[0.20194144472414555`, 1, 1], 
       Hue[0.20194144472414555`, 1, 1], 
       Hue[0.10517975842540783`, 1, 1], 
       Hue[0.10517975842540783`, 1, 1], 
       Hue[0.008676147230962257, 1, 1], 
       Hue[0.008676147230962257, 1, 1], 
       Hue[0.9123650895683947, 1, 1], 
       Hue[0.9123650895683947, 1, 1], 
       Hue[0.8163537610262777, 1, 1], 
       Hue[0.8163537610262777, 1, 1], 
       Hue[0.7205515282763565, 1, 1], 
       Hue[0.7205515282763565, 1, 1], 
       Hue[0.6249695450063499, 1, 1], 
       Hue[0.6249695450063499, 1, 1], 
       Hue[0.5296108292359909, 1, 1], 
       Hue[0.5296108292359909, 1, 1], 
       Hue[0.43443600295767226`, 1, 1], 
       Hue[0.43443600295767226`, 1, 1], 
       Hue[0.3394953919330468, 1, 1], 
       Hue[0.3394953919330468, 1, 1], 
       Hue[0.24476243828602443`, 1, 1], 
       Hue[0.24476243828602443`, 1, 1], 
       Hue[0.15023452379705038`, 1, 1], 
       Hue[0.15023452379705038`, 1, 1], 
       Hue[0.05591726332051126, 1, 1], 
       Hue[0.05591726332051126, 1, 1], 
       Hue[0.9618102877749671, 1, 1], 
       Hue[0.9618102877749671, 1, 1], 
       Hue[0.8678849255300521, 1, 1], 
       Hue[0.8678849255300521, 1, 1], 
       Hue[0.7741471790260839, 1, 1], 
       Hue[0.7741471790260839, 1, 1], 
       Hue[0.6806528796568211, 1, 1], 
       Hue[0.6806528796568211, 1, 1], 
       Hue[0.5873248986019096, 1, 1], 
       Hue[0.5873248986019096, 1, 1], 
       Hue[0.49418451596794727`, 1, 1], 
       Hue[0.49418451596794727`, 1, 1], 
       Hue[0.4012513718543245, 1, 1], 
       Hue[0.4012513718543245, 1, 1], 
       Hue[0.30847550574319005`, 1, 1], 
       Hue[0.30847550574319005`, 1, 1], 
       Hue[0.21589285716824616`, 1, 1], 
       Hue[0.21589285716824616`, 1, 1], 
       Hue[0.12349670073992161`, 1, 1], 
       Hue[0.12349670073992161`, 1, 1], 
       Hue[0.031259713016860584`, 1, 1], 
       Hue[0.031259713016860584`, 1, 1], 
       Hue[0.9392091910858187, 1, 1], 
       Hue[0.9392091910858187, 1, 1], 
       Hue[0.8473239080259873, 1, 1], 
       Hue[0.8473239080259873, 1, 1], 
       Hue[0.7556065340967153, 1, 1], 
       Hue[0.7556065340967153, 1, 1], 
       Hue[0.6640608936363054, 1, 1], 
       Hue[0.6640608936363054, 1, 1], 
       Hue[0.5726668749139376, 1, 1], 
       Hue[0.5726668749139376, 1, 1], 
       Hue[0.48143870204130346`, 1, 1], 
       Hue[0.48143870204130346`, 1, 1], 
       Hue[0.39036514813311196`, 1, 1], 
       Hue[0.39036514813311196`, 1, 1], 
       Hue[0.2994379002421825, 1, 1], 
       Hue[0.2994379002421825, 1, 1], 
       Hue[0.20865980656813093`, 1, 1], 
       Hue[0.20865980656813093`, 1, 1], 
       Hue[0.11802481179332475`, 1, 1], 
       Hue[0.11802481179332475`, 1, 1], 
       Hue[0.02753031297037304, 1, 1], 
       Hue[0.02753031297037304, 1, 1], 
       Hue[0.937172298998918, 1, 1], 
       Hue[0.937172298998918, 1, 1], 
       Hue[0.8469470877910599, 1, 1], 
       Hue[0.8469470877910599, 1, 1], 
       Hue[0.7568529898751493, 1, 1], 
       Hue[0.7568529898751493, 1, 1], 
       Hue[0.6668838918648523, 1, 1], 
       Hue[0.6668838918648523, 1, 1], 
       Hue[0.5770355187840224, 1, 1], 
       Hue[0.5770355187840224, 1, 1], 
       Hue[0.4873067584282177, 1, 1], 
       Hue[0.4873067584282177, 1, 1], 
       Hue[0.3976885574940337, 1, 1], 
       Hue[0.3976885574940337, 1, 1], 
       Hue[0.3081836156328967, 1, 1], 
       Hue[0.3081836156328967, 1, 1], 
       Hue[0.21877826866051225`, 1, 1], 
       Hue[0.21877826866051225`, 1, 1], 
       Hue[0.12947777818206113`, 1, 1], 
       Hue[0.12947777818206113`, 1, 1], 
       Hue[0.040272061779198676`, 1, 1], 
       Hue[0.040272061779198676`, 1, 1], 
       Hue[0.9511561661238163, 1, 1], 
       Hue[0.9511561661238163, 1, 1], 
       Hue[0.8621287005273579, 1, 1], 
       Hue[0.8621287005273579, 1, 1], 
       Hue[0.7731830030987756, 1, 1], 
       Hue[0.7731830030987756, 1, 1], 
       Hue[0.6843154147654915, 1, 1], 
       Hue[0.6843154147654915, 1, 1], 
       Hue[0.5955212624534687, 1, 1], 
       Hue[0.5955212624534687, 1, 1], 
       Hue[0.5067941698730325, 1, 1], 
       Hue[0.5067941698730325, 1, 1], 
       Hue[0.41813217963473104`, 1, 1], 
       Hue[0.41813217963473104`, 1, 1], 
       Hue[0.3295280433802835, 1, 1], 
       Hue[0.3295280433802835, 1, 1], 
       Hue[0.24097871888163253`, 1, 1], 
       Hue[0.24097871888163253`, 1, 1], 
       Hue[0.1524804620347105, 1, 1], 
       Hue[0.1524804620347105, 1, 1], 
       Hue[0.06402804928679805, 1, 1], 
       Hue[0.06402804928679805, 1, 1], 
       Hue[0.9756183067398464, 1, 1], 
       Hue[0.9756183067398464, 1, 1], 
       Hue[0.8872471615914255, 1, 1], 
       Hue[0.8872471615914255, 1, 1], 
       Hue[0.7989117139209876, 1, 1], 
       Hue[0.7989117139209876, 1, 1], 
       Hue[0.7106100480325939, 1, 1], 
       Hue[0.7106100480325939, 1, 1], 
       Hue[0.6223405196721529, 1, 1], 
       Hue[0.6223405196721529, 1, 1], 
       Hue[0.5341007127148718, 1, 1], 
       Hue[0.5341007127148718, 1, 1], 
       Hue[0.44589148097804515`, 1, 1], 
       Hue[0.44589148097804515`, 1, 1], 
       Hue[0.35771438988960935`, 1, 1], 
       Hue[0.35771438988960935`, 1, 1], 
       Hue[0.2695704180261129, 1, 1], 
       Hue[0.2695704180261129, 1, 1], 
       Hue[0.18146586950745308`, 1, 1], 
       Hue[0.18146586950745308`, 1, 1], 
       Hue[0.09340746986017222, 1, 1], 
       Hue[0.09340746986017222, 1, 1], 
       Hue[0.005404085323135338, 1, 1], 
       Hue[0.005404085323135338, 1, 1], 
       Hue[0.9174681454422887, 1, 1], 
       Hue[0.9174681454422887, 1, 1], 
       Hue[0.8296137987629909, 1, 1], 
       Hue[0.8296137987629909, 1, 1], 
       Hue[0.7418598218721183, 1, 1], 
       Hue[0.7418598218721183, 1, 1], 
       Hue[0.6542291760888137, 1, 1], 
       Hue[0.6542291760888137, 1, 1], 
       Hue[0.5667513430325848, 1, 1], 
       Hue[0.5667513430325848, 1, 1], 
       Hue[0.47946412748234774`, 1, 1], 
       Hue[0.47946412748234774`, 1, 1], 
       Hue[0.39241165346008955`, 1, 1], 
       Hue[0.39241165346008955`, 1, 1], 
       Hue[0.3056448023252869, 1, 1], 
       Hue[0.3056448023252869, 1, 1], 
       Hue[0.21922055749572877`, 1, 1], 
       Hue[0.21922055749572877`, 1, 1], 
       Hue[0.1332022361576938, 1, 1], 
       Hue[0.1332022361576938, 1, 1], 
       Hue[0.047666850681859045`, 1, 1], 
       Hue[0.047666850681859045`, 1, 1], 
       Hue[0.9627024797532431, 1, 1], 
       Hue[0.9627024797532431, 1, 1], 
       Hue[0.8784143218371727, 1, 1], 
       Hue[0.8784143218371727, 1, 1], 
       Hue[0.7949263299068562, 1, 1], 
       Hue[0.7949263299068562, 1, 1], 
       Hue[0.7123730704290766, 1, 1], 
       Hue[0.7123730704290766, 1, 1], 
       Hue[0.6308993085613088, 1, 1], 
       Hue[0.6308993085613088, 1, 1], 
       Hue[0.5506492528355513, 1, 1], 
       Hue[0.5506492528355513, 1, 1], 
       Hue[0.4717723067273212, 1, 1], 
       Hue[0.4717723067273212, 1, 1], 
       Hue[0.39442199474621936`, 1, 1], 
       Hue[0.39442199474621936`, 1, 1], 
       Hue[0.3187645584683982, 1, 1], 
       Hue[0.3187645584683982, 1, 1], 
       Hue[0.2449824694851805, 1, 1], 
       Hue[0.2449824694851805, 1, 1], 
       Hue[0.17327582930586308`, 1, 1], 
       Hue[0.17327582930586308`, 1, 1], 
       Hue[0.10385569030144716`, 1, 1], 
       Hue[0.10385569030144716`, 1, 1], 
       Hue[0.03693914308290094, 1, 1], 
       Hue[0.03693914308290094, 1, 1], 
       Hue[0.9727328323643722, 1, 1], 
       Hue[0.9727328323643722, 1, 1], 
       Hue[0.9114250873976556, 1, 1], 
       Hue[0.9114250873976556, 1, 1], 
       Hue[0.8531770723884569, 1, 1], 
       Hue[0.8531770723884569, 1, 1], 
       Hue[0.7981179230268365, 1, 1], 
       Hue[0.7981179230268365, 1, 1], 
       Hue[0.7463434738319357, 1, 1], 
       Hue[0.7463434738319357, 1, 1], 
       Hue[0.6979169230722659, 1, 1], 
       Hue[0.6979169230722659, 1, 1], 
       Hue[0.652870401171725, 1, 1], 
       Hue[0.652870401171725, 1, 1], 
       Hue[0.6112085161386327, 1, 1], 
       Hue[0.6112085161386327, 1, 1], 
       Hue[0.5729111580662459, 1, 1], 
       Hue[0.5729111580662459, 1, 1], 
       Hue[0.5379361605368185, 1, 1], 
       Hue[0.5379361605368185, 1, 1], 
       Hue[0.5062229764889836, 1, 1], 
       Hue[0.5062229764889836, 1, 1], 
       Hue[0.4776979103140166, 1, 1], 
       Hue[0.4776979103140166, 1, 1], 
       Hue[0.45227423713150755`, 1, 1], 
       Hue[0.45227423713150755`, 1, 1], 
       Hue[0.42985665197459183`, 1, 1], 
       Hue[0.42985665197459183`, 1, 1], 
       Hue[0.41034533929896294`, 1, 1], 
       Hue[0.41034533929896294`, 1, 1], 
       Hue[0.39363480451596733`, 1, 1], 
       Hue[0.39363480451596733`, 1, 1], 
       Hue[0.3796206968339214, 1, 1], 
       Hue[0.3796206968339214, 1, 1], 
       Hue[0.36819635676874646`, 1, 1], 
       Hue[0.36819635676874646`, 1, 1], 
       Hue[0.3592588810689745, 1, 1], 
       Hue[0.3592588810689745, 1, 1], 
       Hue[0.35270652242754846`, 1, 1], 
       Hue[0.35270652242754846`, 1, 1], 
       Hue[0.348441715653491, 1, 1], 
       Hue[0.348441715653491, 1, 1], 
       Hue[0.34637025854678805`, 1, 1], 
       Hue[0.34637025854678805`, 1, 1], 
       Hue[0.34640317257292524`, 1, 1], 
       Hue[0.34640317257292524`, 1, 1], 
       Hue[0.3484549273788225, 1, 1], 
       Hue[0.3484549273788225, 1, 1], 
       Hue[0.35244536014669253`, 1, 1], 
       Hue[0.35244536014669253`, 1, 1], 
       Hue[0.35829899021649714`, 1, 1], 
       Hue[0.35829899021649714`, 1, 1], 
       Hue[0.3659444120977315, 1, 1], 
       Hue[0.3659444120977315, 1, 1], 
       Hue[0.37531470466298017`, 1, 1], 
       Hue[0.37531470466298017`, 1, 1], 
       Hue[0.38634778739783043`, 1, 1], 
       Hue[0.38634778739783043`, 1, 1], 
       Hue[0.3989849084100206, 1, 1], 
       Hue[0.3989849084100206, 1, 1], 
       Hue[0.4131714890798039, 1, 1], 
       Hue[0.4131714890798039, 1, 1], 
       Hue[0.4288562177677756, 1, 1], 
       Hue[0.4288562177677756, 1, 1], 
       Hue[0.4459917475621054, 1, 1], 
       Hue[0.4459917475621054, 1, 1], 
       Hue[0.46453343042731615`, 1, 1], 
       Hue[0.46453343042731615`, 1, 1], 
       Hue[0.484439541654788, 1, 1], 
       Hue[0.484439541654788, 1, 1], 
       Hue[0.5056713520991165, 1, 1], 
       Hue[0.5056713520991165, 1, 1], 
       Hue[0.5281925529249144, 1, 1], 
       Hue[0.5281925529249144, 1, 1], 
       Hue[0.5519691183846559, 1, 1], 
       Hue[0.5519691183846559, 1, 1], 
       Hue[0.5769693278688665, 1, 1], 
       Hue[0.5769693278688665, 1, 1], 
       Hue[0.603163245870465, 1, 1], 
       Hue[0.603163245870465, 1, 1], 
       Hue[0.6305229668447081, 1, 1], 
       Hue[0.6305229668447081, 1, 1], 
       Hue[0.6590229110973536, 1, 1], 
       Hue[0.6590229110973536, 1, 1], 
       Hue[0.6886380618054015, 1, 1], 
       Hue[0.6886380618054015, 1, 1], 
       Hue[0.7193449813186077, 1, 1], 
       Hue[0.7193449813186077, 1, 1], 
       Hue[0.7511224993169587, 1, 1], 
       Hue[0.7511224993169587, 1, 1], 
       Hue[0.7839493701684175, 1, 1], 
       Hue[0.7839493701684175, 1, 1], 
       Hue[0.817806735017353, 1, 1], 
       Hue[0.817806735017353, 1, 1], 
       Hue[0.8526758485656166, 1, 1], 
       Hue[0.8526758485656166, 1, 1], 
       Hue[0.8885399846257238, 1, 1], 
       Hue[0.8885399846257238, 1, 1], 
       Hue[0.9253823506081306, 1, 1], 
       Hue[0.9253823506081306, 1, 1], 
       Hue[0.9631867200833969, 1, 1], 
       Hue[0.9631867200833969, 1, 1], 
       Hue[0.0019395945408895489`, 1, 1], 
       Hue[0.0019395945408895489`, 1, 1], 
       Hue[0.04162675481913263, 1, 1], 
       Hue[0.04162675481913263, 1, 1], 
       Hue[0.08223416167340272, 1, 1], 
       Hue[0.08223416167340272, 1, 1], 
       Hue[0.12375029778114359`, 1, 1], 
       Hue[0.12375029778114359`, 1, 1], 
       Hue[0.16616162602928783`, 1, 1], 
       Hue[0.16616162602928783`, 1, 1], 
       Hue[0.20945806400140338`, 1, 1], 
       Hue[0.20945806400140338`, 1, 1], 
       Hue[0.2536276524015086, 1, 1], 
       Hue[0.2536276524015086, 1, 1], 
       Hue[0.2986598507746231, 1, 1], 
       Hue[0.2986598507746231, 1, 1], 
       Hue[0.3445451940568987, 1, 1], 
       Hue[0.3445451940568987, 1, 1], 
       Hue[0.39127272249841616`, 1, 1], 
       Hue[0.39127272249841616`, 1, 1], 
       Hue[0.4388341517654788, 1, 1], 
       Hue[0.4388341517654788, 1, 1], 
       Hue[0.48722035714046374`, 1, 1], 
       Hue[0.48722035714046374`, 1, 1], 
       Hue[0.5364224284486252, 1, 1], 
       Hue[0.5364224284486252, 1, 1], 
       Hue[0.5864334679766994, 1, 1], 
       Hue[0.5864334679766994, 1, 1], 
       Hue[0.6372440491692886, 1, 1], 
       Hue[0.6372440491692886, 1, 1], 
       Hue[0.6888480265350307, 1, 1], 
       Hue[0.6888480265350307, 1, 1], 
       Hue[0.7412365911076693, 1, 1], 
       Hue[0.7412365911076693, 1, 1], 
       Hue[0.7944032577634785, 1, 1], 
       Hue[0.7944032577634785, 1, 1], 
       Hue[0.8483407608749416, 1, 1], 
       Hue[0.8483407608749416, 1, 1], 
       Hue[0.903042428880277, 1, 1], 
       Hue[0.903042428880277, 1, 1], 
       Hue[0.9585025639644492, 1, 1], 
       Hue[0.9585025639644492, 1, 1], 
       Hue[0.014714511057447872`, 1, 1], 
       Hue[0.014714511057447872`, 1, 1], 
       Hue[0.07167338075811815, 1, 1], 
       Hue[0.07167338075811815, 1, 1], 
       Hue[0.12937282649114865`, 1, 1], 
       Hue[0.12937282649114865`, 1, 1], 
       Hue[0.18780666443835414`, 1, 1], 
       Hue[0.18780666443835414`, 1, 1], 
       Hue[0.24696913203339063`, 1, 1], 
       Hue[0.24696913203339063`, 1, 1], 
       Hue[0.30685523730739184`, 1, 1], 
       Hue[0.30685523730739184`, 1, 1], 
       Hue[0.36745957905062887`, 1, 1], 
       Hue[0.36745957905062887`, 1, 1], 
       Hue[0.42877769280745087`, 1, 1], 
       Hue[0.42877769280745087`, 1, 1], 
       Hue[0.4908037925029819, 1, 1], 
       Hue[0.4908037925029819, 1, 1], 
       Hue[0.5535350384234963, 1, 1], 
       Hue[0.5535350384234963, 1, 1], 
       Hue[0.6169653916379854, 1, 1], 
       Hue[0.6169653916379854, 1, 1], 
       Hue[0.6810907257805103, 1, 1], 
       Hue[0.6810907257805103, 1, 1], 
       Hue[0.7459063147912967, 1, 1], 
       Hue[0.7459063147912967, 1, 1], 
       Hue[0.8114104826374526, 1, 1], 
       Hue[0.8114104826374526, 1, 1], 
       Hue[0.8775950861296535, 1, 1], 
       Hue[0.8775950861296535, 1, 1], 
       Hue[0.9444588876589283, 1, 1], 
       Hue[0.9444588876589283, 1, 1], 
       Hue[0.0119970511399642, 1, 1], 
       Hue[0.0119970511399642, 1, 1], 
       Hue[0.08020606705135687, 1, 1], 
       Hue[0.08020606705135687, 1, 1], 
       Hue[0.14908284555554469`, 1, 1], 
       Hue[0.14908284555554469`, 1, 1], 
       Hue[0.2186219206976702, 1, 1], 
       Hue[0.2186219206976702, 1, 1], 
       Hue[0.28882197240286506`, 1, 1], 
       Hue[0.28882197240286506`, 1, 1], 
       Hue[0.35967627266925456`, 1, 1], 
       Hue[0.35967627266925456`, 1, 1], 
       Hue[0.43118455611638384`, 1, 1], 
       Hue[0.43118455611638384`, 1, 1], 
       Hue[0.5033433290086244, 1, 1], 
       Hue[0.5033433290086244, 1, 1], 
       Hue[0.5761460418402145, 1, 1], 
       Hue[0.5761460418402145, 1, 1], 
       Hue[0.6495947288059509, 1, 1], 
       Hue[0.6495947288059509, 1, 1], 
       Hue[0.7236843398166319, 1, 1], 
       Hue[0.7236843398166319, 1, 1], 
       Hue[0.7984095434383008, 1, 1], 
       Hue[0.7984095434383008, 1, 1], 
       Hue[0.873769870187243, 1, 1], 
       Hue[0.873769870187243, 1, 1], 
       Hue[0.9497602611869326, 1, 1], 
       Hue[0.9497602611869326, 1, 1], 
       Hue[0.02638015960819491, 1, 1], 
       Hue[0.02638015960819491, 1, 1], 
       Hue[0.10362426395156402`, 1, 1], 
       Hue[0.10362426395156402`, 1, 1], 
       Hue[0.18149402581251858`, 1, 1], 
       Hue[0.18149402581251858`, 1, 1], 
       Hue[0.25998363569095884`, 1, 1], 
       Hue[0.25998363569095884`, 1, 1], 
       Hue[0.33908888485147026`, 1, 1], 
       Hue[0.33908888485147026`, 1, 1], 
       Hue[0.41881004827500784`, 1, 1], 
       Hue[0.41881004827500784`, 1, 1], 
       Hue[0.4991438872643397, 1, 1], 
       Hue[0.4991438872643397, 1, 1], 
       Hue[0.5800901965951237, 1, 1], 
       Hue[0.5800901965951237, 1, 1], 
       Hue[0.6616434324791175, 1, 1], 
       Hue[0.6616434324791175, 1, 1], 
       Hue[0.7437997538299062, 1, 1], 
       Hue[0.7437997538299062, 1, 1], 
       Hue[0.8265609418880175, 1, 1], 
       Hue[0.8265609418880175, 1, 1], 
       Hue[0.9099231632063347, 1, 1], 
       Hue[0.9099231632063347, 1, 1], 
       Hue[0.9938830944227226, 1, 1], 
       Hue[0.9938830944227226, 1, 1], 
       Hue[0.07843854256843062, 1, 1], 
       Hue[0.07843854256843062, 1, 1], 
       Hue[0.16358722556755453`, 1, 1], 
       Hue[0.16358722556755453`, 1, 1], 
       Hue[0.24932940137544837`, 1, 1], 
       Hue[0.24932940137544837`, 1, 1], 
       Hue[0.33566483509107636`, 1, 1], 
       Hue[0.33566483509107636`, 1, 1], 
       Hue[0.4225903879562467, 1, 1], 
       Hue[0.4225903879562467, 1, 1], 
       Hue[0.5100948438311672, 1, 1], 
       Hue[0.5100948438311672, 1, 1], 
       Hue[0.598181316104291, 1, 1], 
       Hue[0.598181316104291, 1, 1], 
       Hue[0.6868528306413736, 1, 1], 
       Hue[0.6868528306413736, 1, 1], 
       Hue[0.7761040425171196, 1, 1], 
       Hue[0.7761040425171196, 1, 1], 
       Hue[0.8659330990069755, 1, 1], 
       Hue[0.8659330990069755, 1, 1], 
       Hue[0.9563368183458169, 1, 1]}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->{True, None},
    AxesOrigin->{0, 0},
    Epilog->
     LineBox[{{1, 5.6522849888109*^-6}, {2, 7.493856079868243*^-6}, {
       3, 9.361903170769543*^-6}, {4, 0.000011108968790438951`}, {
       5, 0.000013292523855721003`}, {6, 0.00001567890748870991}, {
       7, 0.000019300826157275456`}, {8, 0.00002243631785471867}, {
       9, 0.00002728836735193481}, {10, 0.000031547215782990524`}, {
       11, 0.000037647963946252064`}, {12, 0.00004469129785170058}, {
       13, 0.0000523083795771476}, {14, 0.000062522934026128}, {
       15, 0.00007379014496043094}, {16, 0.00008629399260704188}, {
       17, 0.00010184073462433887`}, {18, 0.0001194013252743524}, {
       19, 0.00013937242829593257`}, {20, 0.0001636449673548106}, {
       21, 0.00019047444025661412`}, {22, 0.00022196161200206042`}, {
       23, 0.0002585859321539324}, {24, 0.00030043458691523584`}, {
       25, 0.0003482834115170974}, {26, 0.0004041598604892873}, {
       27, 0.00046719032058893087`}, {28, 0.0005404554662575303}, {
       29, 0.0006225420668444395}, {30, 0.0007181610827485022}, {
       31, 0.0008255281555854794}, {32, 0.0009485977272661631}, {
       33, 0.0010883170572777196`}, {34, 0.0012469241820886098`}, {
       35, 0.0014263219788284173`}, {36, 0.0016292419422861593`}, {
       37, 0.0018589202825967464`}, {38, 0.002117233077416884}, {
       39, 0.0024086141520167107`}, {40, 0.002736330463350093}, {
       41, 0.003103541487495681}, {42, 0.0035143825427192766`}, {
       43, 0.003974838956634007}, {44, 0.004488583025357468}, {
       45, 0.005060898957280843}, {46, 0.0056987912461398255`}, {
       47, 0.006407424419902048}, {48, 0.007192658635084565}, {
       49, 0.008061732910749672}, {50, 0.009022297740079803}, {
       51, 0.010082722219571393`}, {52, 0.011248908531296002`}, {
       53, 0.012530647562538397`}, {54, 0.013937308753915805`}, {
       55, 0.015476623303451844`}, {56, 0.017159058766566718`}, {
       57, 0.01899510851148176}, {58, 0.020991777418775462`}, {
       59, 0.02316200866272084}, {60, 0.0255142343712261}, {
       61, 0.028058049671868524`}, {62, 0.030804477420809325`}, {
       63, 0.03376307952461599}, {64, 0.0369433250245183}, {
       65, 0.040353950385624185`}, {66, 0.04400369360069891}, {
       67, 0.04790044658369485}, {68, 0.052051667868831884`}, {
       69, 0.05646288077286757}, {70, 0.061138643794485754`}, {
       71, 0.06608402922126116}, {72, 0.07129932909191282}, {
       73, 0.07678617178699923}, {74, 0.08254486999138924}, {
       75, 0.08857066836529483}, {76, 0.09486070727329636}, {
       77, 0.10140691704013148`}, {78, 0.10820040498121343`}, {
       79, 0.11522989692240476`}, {80, 0.12247925564214131`}, {
       81, 0.12993462991311364`}, {82, 0.13757559653046492`}, {
       83, 0.14538169218847222`}, {84, 0.1533280355145805}, {
       85, 0.16138964674434755`}, {86, 0.16953654983092106`}, {
       87, 0.1777384322542062}, {88, 0.18596333219333522`}, {
       89, 0.19417419431505537`}, {90, 0.2023383864716978}, {
       91, 0.21041745276281484`}, {92, 0.218372130610057}, {
       93, 0.22616571392729926`}, {94, 0.23375931093027244`}, {
       95, 0.2411138052960808}, {96, 0.24819276173761884`}, {
       97, 0.25496052072831815`}, {98, 0.26138279154564387`}, {
       99, 0.26742979989231797`}, {100, 0.2730713432144373}, {
       101, 0.27828199547269045`}, {102, 0.28303668560906775`}, {
       103, 0.2873190530457891}, {104, 0.29111461944185024`}, {
       105, 0.29441863672680496`}, {106, 0.29723049047426026`}, {
       107, 0.2995542353596208}, {108, 0.3013953345112595}, {
       109, 0.3027616203282248}, {110, 0.3036651375773494}, {
       111, 0.304128056563314}, {112, 0.304185768598205}, {
       113, 0.30388262309975167`}, {114, 0.3032668561299882}, {
       115, 0.3023846046142477}, {116, 0.3012724946513947}, {
       117, 0.2999622330218534}, {118, 0.29848152589850435`}, {
       119, 0.29687906582650525`}, {120, 0.2952239545087466}, {
       121, 0.29361060244024717`}, {122, 0.2921542584785492}, {
       123, 0.29096647352276866`}, {124, 0.2901484313952193}, {
       125, 0.2897730126065047}, {126, 0.28988429551438105`}, {
       127, 0.2904969316890593}, {128, 0.29160211860018714`}, {
       129, 0.2931792766969426}, {130, 0.2951972492883752}, {
       131, 0.2976212415637846}, {132, 0.30041704859499585`}, {
       133, 0.30354776772839553`}, {134, 0.30697990660895746`}, {
       135, 0.31067657657821374`}, {136, 0.31460387555610997`}, {
       137, 0.3187259141998557}, {138, 0.3230072791715519}, {
       139, 0.32741599378577807`}, {140, 0.33191750810603293`}, {
       141, 0.3364799219044072}, {142, 0.3410721734176782}, {
       143, 0.3456634644312373}, {144, 0.35022591697757116`}, {
       145, 0.3547304534466183}, {146, 0.3591516322835139}, {
       147, 0.3634645792148149}, {148, 0.36764472653667896`}, {
       149, 0.37167114636571835`}, {150, 0.37552266147302954`}, {
       151, 0.37917826094575385`}, {152, 0.382620340373909}, {
       153, 0.38583081259268975`}, {154, 0.38879304717095553`}, {
       155, 0.3914923418384644}, {156, 0.39391496620426814`}, {
       157, 0.3960491604294993}, {158, 0.3978817601129005}, {
       159, 0.3994029102098073}, {160, 0.40060382678811673`}, {
       161, 0.4014763827448637}, {162, 0.40201320650983136`}, {
       163, 0.40220877817913}, {164, 0.4020596379079551}, {
       165, 0.40156053180909207`}, {166, 0.40071130856384024`}, {
       167, 0.3995097978874194}, {168, 0.3979571867374357}, {
       169, 0.3960532509583461}, {170, 0.3938010527158964}, {
       171, 0.3912044641117032}, {172, 0.3882674271615195}, {
       173, 0.3849960108732703}, {174, 0.38139616514859}, {
       175, 0.377476672231851}, {176, 0.3732452805785937}, {
       177, 0.3687106584673623}, {178, 0.36388510816809233`}, {
       179, 0.35877722556847225`}, {180, 0.35339987022905417`}, {
       181, 0.3477671550716122}, {182, 0.3418903016180166}, {
       183, 0.33578618402163607`}, {184, 0.3294663815661966}, {
       185, 0.32294686438337383`}, {186, 0.3162430597784358}, {
       187, 0.30936969254715996`}, {188, 0.3023435982829913}, {
       189, 0.29517900906165695`}, {190, 0.2878947820982167}, {
       191, 0.2805053064998337}, {192, 0.27302872184533394`}, {
       193, 0.2654797263227138}, {194, 0.25787590264687776`}, {
       195, 0.2502318165479045}, {196, 0.24256408092125306`}, {
       197, 0.2348861366093799}, {198, 0.2272146654583238}, {
       199, 0.21956522476328721`}, {200, 0.21195029496825143`}, {
       201, 0.20438481288933247`}, {202, 0.19688170729809698`}, {
       203, 0.18945310873947227`}, {204, 0.1821102691001891}, {
       205, 0.17486450657556654`}, {206, 0.16772748488167488`}, {
       207, 0.1607093723086569}, {208, 0.15381766399013522`}, {
       209, 0.14706248767861638`}, {210, 0.14045181504777782`}, {
       211, 0.1339920959179641}, {212, 0.12768797002110663`}, {
       213, 0.12154877417117764`}, {214, 0.11557589800546827`}, {
       215, 0.10977615334457266`}, {216, 0.1041507102783259}, {
       217, 0.0987042107678967}, {218, 0.09343733781565668}, {
       219, 0.08835230014682988}, {220, 0.0834509207156375}, {
       221, 0.07873168199141253}, {222, 0.07419540279580301}, {
       223, 0.06984225126755672}, {224, 0.06566798441257996}, {
       225, 0.0616731481904196}, {226, 0.05785435998510809}, {
       227, 0.0542097540193168}, {228, 0.050735933935916196`}, {
       229, 0.04742980734801528}, {230, 0.044287888983703665`}, {
       231, 0.04130493677009453}, {232, 0.03847803216215673}, {
       233, 0.035802623779484295`}, {234, 0.03327410378351036}, {
       235, 0.03088753042728579}, {236, 0.028637517638561483`}, {
       237, 0.0265200339393725}, {238, 0.024529894058588228`}, {
       239, 0.022661673157524227`}, {240, 0.02091051709060792}, {
       241, 0.01927136934054423}, {242, 0.0177393465627438}, {
       243, 0.016309428448624385`}, {244, 0.014976490853957705`}, {
       245, 0.013735190975997291`}, {246, 0.012581058234864132`}, {
       247, 0.011510126540786694`}, {248, 0.010517351802369235`}, {
       249, 0.00959824873699968}, {250, 0.00874852546905899}, {
       251, 0.007964239394287614}}],
    Frame->True,
    FrameTicks->{{{1., 
        FormBox[
         RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
        FormBox[
         RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
        FormBox[
         RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
        FormBox[
         RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
        FormBox["0", TraditionalForm]}, {157.25, 
        FormBox["2.5`", TraditionalForm]}, {188.5, 
        FormBox["5", TraditionalForm]}, {219.75, 
        FormBox["7.5`", TraditionalForm]}, {251., 
        FormBox["10", TraditionalForm]}}, Automatic, None, None},
    PlotRange->{-0.1, 1.1},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], Automatic}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF13tcTocfB/AzWwkxloglJyyEoa1EySdKV93vN+fp3tNTuYy1zByXWMQs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     "], {
      {GrayLevel[0.5], EdgeForm[None], 
       GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ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         "]]]}, {{}, {}, {
        {Hue[0.9355110244011989, 1, 1], LineBox[{1, 2}]}, 
        {Hue[0.8331243269093445, 1, 1], LineBox[{2, 3}]}, 
        {Hue[0.7307595415030963, 1, 1], LineBox[{3, 4}]}, 
        {Hue[0.6279388039622625, 1, 1], LineBox[{4, 5}]}, 
        {Hue[0.5268258073343911, 1, 1], LineBox[{5, 6}]}, 
        {Hue[0.424171083302787, 1, 1], LineBox[{6, 7}]}, 
        {Hue[0.3222947041560654, 1, 1], LineBox[{7, 8}]}, 
        {Hue[0.22127598997976075`, 1, 1], LineBox[{8, 9}]}, 
        {Hue[0.12037676786958348`, 1, 1], LineBox[{9, 10}]}, 
        {Hue[0.018976825257496246`, 1, 1], LineBox[{10, 11}]}, 
        {Hue[0.918522893746869, 1, 1], LineBox[{11, 12}]}, 
        {Hue[0.8182612468644723, 1, 1], LineBox[{12, 13}]}, 
        {Hue[0.7183833795476677, 1, 1], LineBox[{13, 14}]}, 
        {Hue[0.6183986488335731, 1, 1], LineBox[{14, 15}]}, 
        {Hue[0.5193758685502148, 1, 1], LineBox[{15, 16}]}, 
        {Hue[0.41994575252986666`, 1, 1], LineBox[{16, 17}]}, 
        {Hue[0.32105900216888295`, 1, 1], LineBox[{17, 18}]}, 
        {Hue[0.22234529701340938`, 1, 1], LineBox[{18, 19}]}, 
        {Hue[0.1237823642680303, 1, 1], LineBox[{19, 20}]}, 
        {Hue[0.02539869814779855, 1, 1], LineBox[{20, 21}]}, 
        {Hue[0.9275293115368062, 1, 1], LineBox[{21, 22}]}, 
        {Hue[0.8297207691016857, 1, 1], LineBox[{22, 23}]}, 
        {Hue[0.7323239009278713, 1, 1], LineBox[{23, 24}]}, 
        {Hue[0.6349851438514744, 1, 1], LineBox[{24, 25}]}, 
        {Hue[0.5378773905896601, 1, 1], LineBox[{25, 26}]}, 
        {Hue[0.4410809731310571, 1, 1], LineBox[{26, 27}]}, 
        {Hue[0.34442190104855774`, 1, 1], LineBox[{27, 28}]}, 
        {Hue[0.24803948086947397`, 1, 1], LineBox[{28, 29}]}, 
        {Hue[0.15187714678567418`, 1, 1], LineBox[{29, 30}]}, 
        {Hue[0.055945236434117585`, 1, 1], LineBox[{30, 31}]}, 
        {Hue[0.9601410826422264, 1, 1], LineBox[{31, 32}]}, 
        {Hue[0.8646436606294935, 1, 1], LineBox[{32, 33}]}, 
        {Hue[0.7693298530870238, 1, 1], LineBox[{33, 34}]}, 
        {Hue[0.6742223440667043, 1, 1], LineBox[{34, 35}]}, 
        {Hue[0.5793187421548431, 1, 1], LineBox[{35, 36}]}, 
        {Hue[0.4846333395900681, 1, 1], LineBox[{36, 37}]}, 
        {Hue[0.3901292437334285, 1, 1], LineBox[{37, 38}]}, 
        {Hue[0.2958404119994364, 1, 1], LineBox[{38, 39}]}, 
        {Hue[0.20173313272304044`, 1, 1], LineBox[{39, 40}]}, 
        {Hue[0.1078217796597472, 1, 1], LineBox[{40, 41}]}, 
        {Hue[0.01410988454345159, 1, 1], LineBox[{41, 42}]}, 
        {Hue[0.9205757406417387, 1, 1], LineBox[{42, 43}]}, 
        {Hue[0.827240765562894, 1, 1], LineBox[{43, 44}]}, 
        {Hue[0.734071085562777, 1, 1], LineBox[{44, 45}]}, 
        {Hue[0.64109432272257, 1, 1], LineBox[{45, 46}]}, 
        {Hue[0.5482985086259914, 1, 1], LineBox[{46, 47}]}, 
        {Hue[0.45567974176311293`, 1, 1], LineBox[{47, 48}]}, 
        {Hue[0.363222210809178, 1, 1], LineBox[{48, 49}]}, 
        {Hue[0.2709463557656617, 1, 1], LineBox[{49, 50}]}, 
        {Hue[0.1788348021421335, 1, 1], LineBox[{50, 51}]}, 
        {Hue[0.0868829353325018, 1, 1], LineBox[{51, 52}]}, 
        {Hue[0.9950982708020901, 1, 1], LineBox[{52, 53}]}, 
        {Hue[0.9034663428818953, 1, 1], LineBox[{53, 54}]}, 
        {Hue[0.8119948684127546, 1, 1], LineBox[{54, 55}]}, 
        {Hue[0.7206732687351798, 1, 1], LineBox[{55, 56}]}, 
        {Hue[0.6295015001961014, 1, 1], LineBox[{56, 57}]}, 
        {Hue[0.5384757740462534, 1, 1], LineBox[{57, 58}]}, 
        {Hue[0.44759116579465397`, 1, 1], LineBox[{58, 59}]}, 
        {Hue[0.3568458792103737, 1, 1], LineBox[{59, 60}]}, 
        {Hue[0.2662389480082245, 1, 1], LineBox[{60, 61}]}, 
        {Hue[0.1757615783609027, 1, 1], LineBox[{61, 62}]}, 
        {Hue[0.0854139219264677, 1, 1], LineBox[{62, 63}]}, 
        {Hue[0.9951890441798408, 1, 1], LineBox[{63, 64}]}, 
        {Hue[0.9050842641126323, 1, 1], LineBox[{64, 65}]}, 
        {Hue[0.8150993924344233, 1, 1], LineBox[{65, 66}]}, 
        {Hue[0.7252233314084258, 1, 1], LineBox[{66, 67}]}, 
        {Hue[0.6354592261172873, 1, 1], LineBox[{67, 68}]}, 
        {Hue[0.5457985330640146, 1, 1], LineBox[{68, 69}]}, 
        {Hue[0.45623669902216535`, 1, 1], LineBox[{69, 70}]}, 
        {Hue[0.36677041065441346`, 1, 1], LineBox[{70, 71}]}, 
        {Hue[0.27739418643777836`, 1, 1], LineBox[{71, 72}]}, 
        {Hue[0.18810275745714697`, 1, 1], LineBox[{72, 73}]}, 
        {Hue[0.09889230647015664, 1, 1], LineBox[{73, 74}]}, 
        {Hue[0.00975804126335437, 1, 1], LineBox[{74, 75}]}, 
        {Hue[0.9206921227058936, 1, 1], LineBox[{75, 76}]}, 
        {Hue[0.8316919469100537, 1, 1], LineBox[{76, 77}]}, 
        {Hue[0.7427524318223738, 1, 1], LineBox[{77, 78}]}, 
        {Hue[0.6538648668741538, 1, 1], LineBox[{78, 79}]}, 
        {Hue[0.5650260486075918, 1, 1], LineBox[{79, 80}]}, 
        {Hue[0.47622861244209524`, 1, 1], LineBox[{80, 81}]}, 
        {Hue[0.3874663043663873, 1, 1], LineBox[{81, 82}]}, 
        {Hue[0.29873346458885097`, 1, 1], LineBox[{82, 83}]}, 
        {Hue[0.2100241658752546, 1, 1], LineBox[{83, 84}]}, 
        {Hue[0.12133163459447596`, 1, 1], LineBox[{84, 85}]}, 
        {Hue[0.032649092430081426`, 1, 1], LineBox[{85, 86}]}, 
        {Hue[0.9439705938460063, 1, 1], LineBox[{86, 87}]}, 
        {Hue[0.8552884086498929, 1, 1], LineBox[{87, 88}]}, 
        {Hue[0.7665970492852087, 1, 1], LineBox[{88, 89}]}, 
        {Hue[0.6778898537541902, 1, 1], LineBox[{89, 90}]}, 
        {Hue[0.5891600056030785, 1, 1], LineBox[{90, 91}]}, 
        {Hue[0.5004025675691461, 1, 1], LineBox[{91, 92}]}, 
        {Hue[0.4116114355042888, 1, 1], LineBox[{92, 93}]}, 
        {Hue[0.322779950578861, 1, 1], LineBox[{93, 94}]}, 
        {Hue[0.23390508369437094`, 1, 1], LineBox[{94, 95}]}, 
        {Hue[0.14498100702236932`, 1, 1], LineBox[{95, 96}]}, 
        {Hue[0.05600704660552275, 1, 1], LineBox[{96, 97}]}, 
        {Hue[0.9669795709678686, 1, 1], LineBox[{97, 98}]}, 
        {Hue[0.8778982582793236, 1, 1], LineBox[{98, 99}]}, 
        {Hue[0.7887652557309699, 1, 1], LineBox[{99, 100}]}, 
        {Hue[0.6995823800456853, 1, 1], LineBox[{100, 101}]}, 
        {Hue[0.6103564710800458, 1, 1], LineBox[{101, 102}]}, 
        {Hue[0.5210960147936234, 1, 1], LineBox[{102, 103}]}, 
        {Hue[0.4318125288607402, 1, 1], LineBox[{103, 104}]}, 
        {Hue[0.34252493459971806`, 1, 1], LineBox[{104, 105}]}, 
        {Hue[0.25325369870288117`, 1, 1], LineBox[{105, 106}]}, 
        {Hue[0.16402728053127844`, 1, 1], LineBox[{106, 107}]}, 
        {Hue[0.07488002620663553, 1, 1], LineBox[{107, 108}]}, 
        {Hue[0.9858520065701789, 1, 1], LineBox[{108, 109}]}, 
        {Hue[0.8969931708519823, 1, 1], LineBox[{109, 110}]}, 
        {Hue[0.8083612816537772, 1, 1], LineBox[{110, 111}]}, 
        {Hue[0.720027846091456, 1, 1], LineBox[{111, 112}]}, 
        {Hue[0.6320783020478189, 1, 1], LineBox[{112, 113}]}, 
        {Hue[0.5446126226524881, 1, 1], LineBox[{113, 114}]}, 
        {Hue[0.4577416941256004, 1, 1], LineBox[{114, 115}]}, 
        {Hue[0.371589448811156, 1, 1], LineBox[{115, 116}]}, 
        {Hue[0.2862928742918964, 1, 1], LineBox[{116, 117}]}, 
        {Hue[0.20200228023477101`, 1, 1], LineBox[{117, 118}]}, 
        {Hue[0.11888507280599135`, 1, 1], LineBox[{118, 119}]}, 
        {Hue[0.037127910830192574`, 1, 1], LineBox[{119, 120}]}, 
        {Hue[0.9569362186420415, 1, 1], LineBox[{120, 121}]}, 
        {Hue[0.8785305257889706, 1, 1], LineBox[{121, 122}]}, 
        {Hue[0.8021410588218201, 1, 1], LineBox[{122, 123}]}, 
        {Hue[0.7279945898245506, 1, 1], LineBox[{123, 124}]}, 
        {Hue[0.656309101852733, 1, 1], LineBox[{124, 125}]}, 
        {Hue[0.5872863814003371, 1, 1], LineBox[{125, 126}]}, 
        {Hue[0.5211049841628967, 1, 1], LineBox[{126, 127}]}, 
        {Hue[0.4579218320357416, 1, 1], LineBox[{127, 128}]}, 
        {Hue[0.3978644395073952, 1, 1], LineBox[{128, 129}]}, 
        {Hue[0.34103635547553546`, 1, 1], LineBox[{129, 130}]}, 
        {Hue[0.28751299833736366`, 1, 1], LineBox[{130, 131}]}, 
        {Hue[0.2373425683885671, 1, 1], LineBox[{131, 132}]}, 
        {Hue[0.19054743917487552`, 1, 1], LineBox[{132, 133}]}, 
        {Hue[0.14712457912982915`, 1, 1], LineBox[{133, 134}]}, 
        {Hue[0.10704787791664298`, 1, 1], LineBox[{134, 135}]}, 
        {Hue[0.07027258996258669, 1, 1], LineBox[{135, 136}]}, 
        {Hue[0.036734633034547576`, 1, 1], LineBox[{136, 137}]}, 
        {Hue[0.0063562804960939565`, 1, 1], LineBox[{137, 138}]}, 
        {Hue[0.9790486019005894, 1, 1], LineBox[{138, 139}]}, 
        {Hue[0.954714129564768, 1, 1], LineBox[{139, 140}]}, 
        {Hue[0.9332507821720971, 1, 1], LineBox[{140, 141}]}, 
        {Hue[0.9145521366166071, 1, 1], LineBox[{141, 142}]}, 
        {Hue[0.8985108022332381, 1, 1], LineBox[{142, 143}]}, 
        {Hue[0.885019537265343, 1, 1], LineBox[{143, 144}]}, 
        {Hue[0.8739740784029262, 1, 1], LineBox[{144, 145}]}, 
        {Hue[0.8652717100434656, 1, 1], LineBox[{145, 146}]}, 
        {Hue[0.8588139578755996, 1, 1], LineBox[{146, 147}]}, 
        {Hue[0.8545070219354367, 1, 1], LineBox[{147, 148}]}, 
        {Hue[0.8522605424068836, 1, 1], LineBox[{148, 149}]}, 
        {Hue[0.8519894958550597, 1, 1], LineBox[{149, 150}]}, 
        {Hue[0.8536135359957169, 1, 1], LineBox[{150, 151}]}, 
        {Hue[0.8570565069011515, 1, 1], LineBox[{151, 152}]}, 
        {Hue[0.8622480974679313, 1, 1], LineBox[{152, 153}]}, 
        {Hue[0.8691216861664274, 1, 1], LineBox[{153, 154}]}, 
        {Hue[0.877614308802046, 1, 1], LineBox[{154, 155}]}, 
        {Hue[0.8876687311186671, 1, 1], LineBox[{155, 156}]}, 
        {Hue[0.899230510278553, 1, 1], LineBox[{156, 157}]}, 
        {Hue[0.9122490135588197, 1, 1], LineBox[{157, 158}]}, 
        {Hue[0.9266773800415274, 1, 1], LineBox[{158, 159}]}, 
        {Hue[0.9424715311467944, 1, 1], LineBox[{159, 160}]}, 
        {Hue[0.9595905066764843, 1, 1], LineBox[{160, 161}]}, 
        {Hue[0.9779961683060914, 1, 1], LineBox[{161, 162}]}, 
        {Hue[0.9976531263524046, 1, 1], LineBox[{162, 163}]}, 
        {Hue[0.018527973642258822`, 1, 1], LineBox[{163, 164}]}, 
        {Hue[0.040589846530470396`, 1, 1], LineBox[{164, 165}]}, 
        {Hue[0.06380921313635418, 1, 1], LineBox[{165, 166}]}, 
        {Hue[0.088159456147119, 1, 1], LineBox[{166, 167}]}, 
        {Hue[0.11361500882265167`, 1, 1], LineBox[{167, 168}]}, 
        {Hue[0.14015184759702176`, 1, 1], LineBox[{168, 169}]}, 
        {Hue[0.1677470050651769, 1, 1], LineBox[{169, 170}]}, 
        {Hue[0.1963805235167193, 1, 1], LineBox[{170, 171}]}, 
        {Hue[0.226032063227786, 1, 1], LineBox[{171, 172}]}, 
        {Hue[0.2566825269000016, 1, 1], LineBox[{172, 173}]}, 
        {Hue[0.2883143142722514, 1, 1], LineBox[{173, 174}]}, 
        {Hue[0.32091103841903484`, 1, 1], LineBox[{174, 175}]}, 
        {Hue[0.3544570383872956, 1, 1], LineBox[{175, 176}]}, 
        {Hue[0.3889373946660122, 1, 1], LineBox[{176, 177}]}, 
        {Hue[0.42433703731545136`, 1, 1], LineBox[{177, 178}]}, 
        {Hue[0.4606438306174138, 1, 1], LineBox[{178, 179}]}, 
        {Hue[0.49784493890218584`, 1, 1], LineBox[{179, 180}]}, 
        {Hue[0.535927662459045, 1, 1], LineBox[{180, 181}]}, 
        {Hue[0.5748807416320714, 1, 1], LineBox[{181, 182}]}, 
        {Hue[0.6146930444978822, 1, 1], LineBox[{182, 183}]}, 
        {Hue[0.6553538884368986, 1, 1], LineBox[{183, 184}]}, 
        {Hue[0.696853218504837, 1, 1], LineBox[{184, 185}]}, 
        {Hue[0.7391814889795953, 1, 1], LineBox[{185, 186}]}, 
        {Hue[0.7823289599944764, 1, 1], LineBox[{186, 187}]}, 
        {Hue[0.8262874078226671, 1, 1], LineBox[{187, 188}]}, 
        {Hue[0.8710478798485146, 1, 1], LineBox[{188, 189}]}, 
        {Hue[0.9166034157050914, 1, 1], LineBox[{189, 190}]}, 
        {Hue[0.9629448223502752, 1, 1], LineBox[{190, 191}]}, 
        {Hue[0.01006559471412739, 1, 1], LineBox[{191, 192}]}, 
        {Hue[0.05795832163985246, 1, 1], LineBox[{192, 193}]}, 
        {Hue[0.10661432626890296`, 1, 1], LineBox[{193, 194}]}, 
        {Hue[0.1560288107312348, 1, 1], LineBox[{194, 195}]}, 
        {Hue[0.2061934463881734, 1, 1], LineBox[{195, 196}]}, 
        {Hue[0.25710371868991605`, 1, 1], LineBox[{196, 197}]}, 
        {Hue[0.30875258049480325`, 1, 1], LineBox[{197, 198}]}, 
        {Hue[0.3611346594763807, 1, 1], LineBox[{198, 199}]}, 
        {Hue[0.4142443931279519, 1, 1], LineBox[{199, 200}]}, 
        {Hue[0.46807592250982194`, 1, 1], LineBox[{200, 201}]}, 
        {Hue[0.5226235169268131, 1, 1], LineBox[{201, 202}]}, 
        {Hue[0.5778827231313868, 1, 1], LineBox[{202, 203}]}, 
        {Hue[0.6338467284901352, 1, 1], LineBox[{203, 204}]}, 
        {Hue[0.6905119716638414, 1, 1], LineBox[{204, 205}]}, 
        {Hue[0.7478729708429508, 1, 1], LineBox[{205, 206}]}, 
        {Hue[0.8059262182338722, 1, 1], LineBox[{206, 207}]}, 
        {Hue[0.8646667029288123, 1, 1], LineBox[{207, 208}]}, 
        {Hue[0.9240900148564036, 1, 1], LineBox[{208, 209}]}, 
        {Hue[0.9841921136405349, 1, 1], LineBox[{209, 210}]}, 
        {Hue[0.044968567409435826`, 1, 1], LineBox[{210, 211}]}, 
        {Hue[0.1064158884423126, 1, 1], LineBox[{211, 212}]}, 
        {Hue[0.16852978608792193`, 1, 1], LineBox[{212, 213}]}, 
        {Hue[0.23130554990275484`, 1, 1], LineBox[{213, 214}]}, 
        {Hue[0.29474084889917324`, 1, 1], LineBox[{214, 215}]}, 
        {Hue[0.358831588919603, 1, 1], LineBox[{215, 216}]}, 
        {Hue[0.423572986863044, 1, 1], LineBox[{216, 217}]}, 
        {Hue[0.4889633994506269, 1, 1], LineBox[{217, 218}]}, 
        {Hue[0.5549981042468378, 1, 1], LineBox[{218, 219}]}, 
        {Hue[0.6216756108260031, 1, 1], LineBox[{219, 220}]}, 
        {Hue[0.6889900341282765, 1, 1], LineBox[{220, 221}]}, 
        {Hue[0.7569404375017312, 1, 1], LineBox[{221, 222}]}, 
        {Hue[0.8255219568966075, 1, 1], LineBox[{222, 223}]}, 
        {Hue[0.8947351781930797, 1, 1], LineBox[{223, 224}]}, 
        {Hue[0.9645711467107132, 1, 1], LineBox[{224, 225}]}, 
        {Hue[0.035032200808264795`, 1, 1], LineBox[{225, 226}]}, 
        {Hue[0.10611430302560833`, 1, 1], LineBox[{226, 227}]}, 
        {Hue[0.1778127455017217, 1, 1], LineBox[{227, 228}]}, 
        {Hue[0.25012585040377056`, 1, 1], LineBox[{228, 229}]}, 
        {Hue[0.3230512831048453, 1, 1], LineBox[{229, 230}]}, 
        {Hue[0.39658786037578897`, 1, 1], LineBox[{230, 231}]}, 
        {Hue[0.47073048283222535`, 1, 1], LineBox[{231, 232}]}, 
        {Hue[0.5454760758758606, 1, 1], LineBox[{232, 233}]}, 
        {Hue[0.6208268462480444, 1, 1], LineBox[{233, 234}]}, 
        {Hue[0.6967736619070093, 1, 1], LineBox[{234, 235}]}, 
        {Hue[0.7733207693695567, 1, 1], LineBox[{235, 236}]}, 
        {Hue[0.85046436527129, 1, 1], LineBox[{236, 237}]}, 
        {Hue[0.9281979578049646, 1, 1], LineBox[{237, 238}]}, 
        {Hue[0.006524208684187778, 1, 1], LineBox[{238, 239}]}, 
        {Hue[0.08543857693114915, 1, 1], LineBox[{239, 240}]}, 
        {Hue[0.1649389112410558, 1, 1], LineBox[{240, 241}]}, 
        {Hue[0.24502025669775915`, 1, 1], LineBox[{241, 242}]}, 
        {Hue[0.3256855387817807, 1, 1], LineBox[{242, 243}]}, 
        {Hue[0.4069328254093805, 1, 1], LineBox[{243, 244}]}, 
        {Hue[0.48875648074544686`, 1, 1], LineBox[{244, 245}]}, 
        {Hue[0.5711582052233701, 1, 1], LineBox[{245, 246}]}, 
        {Hue[0.6541335516799798, 1, 1], LineBox[{246, 247}]}, 
        {Hue[0.7376792030777755, 1, 1], LineBox[{247, 248}]}, 
        {Hue[0.8217951224193869, 1, 1], LineBox[{248, 249}]}, 
        {Hue[0.906479842223855, 1, 1], LineBox[{249, 250}]}, 
        {Hue[0.9917352697531961, 1, 1], LineBox[{250, 251}]}}}},
     VertexColors->{
       Hue[0.9355110244011989, 1, 1], 
       Hue[0.8331243269093445, 1, 1], 
       Hue[0.7307595415030963, 1, 1], 
       Hue[0.6279388039622625, 1, 1], 
       Hue[0.5268258073343911, 1, 1], 
       Hue[0.424171083302787, 1, 1], 
       Hue[0.3222947041560654, 1, 1], 
       Hue[0.22127598997976075`, 1, 1], 
       Hue[0.12037676786958348`, 1, 1], 
       Hue[0.018976825257496246`, 1, 1], 
       Hue[0.918522893746869, 1, 1], 
       Hue[0.8182612468644723, 1, 1], 
       Hue[0.7183833795476677, 1, 1], 
       Hue[0.6183986488335731, 1, 1], 
       Hue[0.5193758685502148, 1, 1], 
       Hue[0.41994575252986666`, 1, 1], 
       Hue[0.32105900216888295`, 1, 1], 
       Hue[0.22234529701340938`, 1, 1], 
       Hue[0.1237823642680303, 1, 1], 
       Hue[0.02539869814779855, 1, 1], 
       Hue[0.9275293115368062, 1, 1], 
       Hue[0.8297207691016857, 1, 1], 
       Hue[0.7323239009278713, 1, 1], 
       Hue[0.6349851438514744, 1, 1], 
       Hue[0.5378773905896601, 1, 1], 
       Hue[0.4410809731310571, 1, 1], 
       Hue[0.34442190104855774`, 1, 1], 
       Hue[0.24803948086947397`, 1, 1], 
       Hue[0.15187714678567418`, 1, 1], 
       Hue[0.055945236434117585`, 1, 1], 
       Hue[0.9601410826422264, 1, 1], 
       Hue[0.8646436606294935, 1, 1], 
       Hue[0.7693298530870238, 1, 1], 
       Hue[0.6742223440667043, 1, 1], 
       Hue[0.5793187421548431, 1, 1], 
       Hue[0.4846333395900681, 1, 1], 
       Hue[0.3901292437334285, 1, 1], 
       Hue[0.2958404119994364, 1, 1], 
       Hue[0.20173313272304044`, 1, 1], 
       Hue[0.1078217796597472, 1, 1], 
       Hue[0.01410988454345159, 1, 1], 
       Hue[0.9205757406417387, 1, 1], 
       Hue[0.827240765562894, 1, 1], 
       Hue[0.734071085562777, 1, 1], 
       Hue[0.64109432272257, 1, 1], 
       Hue[0.5482985086259914, 1, 1], 
       Hue[0.45567974176311293`, 1, 1], 
       Hue[0.363222210809178, 1, 1], 
       Hue[0.2709463557656617, 1, 1], 
       Hue[0.1788348021421335, 1, 1], 
       Hue[0.0868829353325018, 1, 1], 
       Hue[0.9950982708020901, 1, 1], 
       Hue[0.9034663428818953, 1, 1], 
       Hue[0.8119948684127546, 1, 1], 
       Hue[0.7206732687351798, 1, 1], 
       Hue[0.6295015001961014, 1, 1], 
       Hue[0.5384757740462534, 1, 1], 
       Hue[0.44759116579465397`, 1, 1], 
       Hue[0.3568458792103737, 1, 1], 
       Hue[0.2662389480082245, 1, 1], 
       Hue[0.1757615783609027, 1, 1], 
       Hue[0.0854139219264677, 1, 1], 
       Hue[0.9951890441798408, 1, 1], 
       Hue[0.9050842641126323, 1, 1], 
       Hue[0.8150993924344233, 1, 1], 
       Hue[0.7252233314084258, 1, 1], 
       Hue[0.6354592261172873, 1, 1], 
       Hue[0.5457985330640146, 1, 1], 
       Hue[0.45623669902216535`, 1, 1], 
       Hue[0.36677041065441346`, 1, 1], 
       Hue[0.27739418643777836`, 1, 1], 
       Hue[0.18810275745714697`, 1, 1], 
       Hue[0.09889230647015664, 1, 1], 
       Hue[0.00975804126335437, 1, 1], 
       Hue[0.9206921227058936, 1, 1], 
       Hue[0.8316919469100537, 1, 1], 
       Hue[0.7427524318223738, 1, 1], 
       Hue[0.6538648668741538, 1, 1], 
       Hue[0.5650260486075918, 1, 1], 
       Hue[0.47622861244209524`, 1, 1], 
       Hue[0.3874663043663873, 1, 1], 
       Hue[0.29873346458885097`, 1, 1], 
       Hue[0.2100241658752546, 1, 1], 
       Hue[0.12133163459447596`, 1, 1], 
       Hue[0.032649092430081426`, 1, 1], 
       Hue[0.9439705938460063, 1, 1], 
       Hue[0.8552884086498929, 1, 1], 
       Hue[0.7665970492852087, 1, 1], 
       Hue[0.6778898537541902, 1, 1], 
       Hue[0.5891600056030785, 1, 1], 
       Hue[0.5004025675691461, 1, 1], 
       Hue[0.4116114355042888, 1, 1], 
       Hue[0.322779950578861, 1, 1], 
       Hue[0.23390508369437094`, 1, 1], 
       Hue[0.14498100702236932`, 1, 1], 
       Hue[0.05600704660552275, 1, 1], 
       Hue[0.9669795709678686, 1, 1], 
       Hue[0.8778982582793236, 1, 1], 
       Hue[0.7887652557309699, 1, 1], 
       Hue[0.6995823800456853, 1, 1], 
       Hue[0.6103564710800458, 1, 1], 
       Hue[0.5210960147936234, 1, 1], 
       Hue[0.4318125288607402, 1, 1], 
       Hue[0.34252493459971806`, 1, 1], 
       Hue[0.25325369870288117`, 1, 1], 
       Hue[0.16402728053127844`, 1, 1], 
       Hue[0.07488002620663553, 1, 1], 
       Hue[0.9858520065701789, 1, 1], 
       Hue[0.8969931708519823, 1, 1], 
       Hue[0.8083612816537772, 1, 1], 
       Hue[0.720027846091456, 1, 1], 
       Hue[0.6320783020478189, 1, 1], 
       Hue[0.5446126226524881, 1, 1], 
       Hue[0.4577416941256004, 1, 1], 
       Hue[0.371589448811156, 1, 1], 
       Hue[0.2862928742918964, 1, 1], 
       Hue[0.20200228023477101`, 1, 1], 
       Hue[0.11888507280599135`, 1, 1], 
       Hue[0.037127910830192574`, 1, 1], 
       Hue[0.9569362186420415, 1, 1], 
       Hue[0.8785305257889706, 1, 1], 
       Hue[0.8021410588218201, 1, 1], 
       Hue[0.7279945898245506, 1, 1], 
       Hue[0.656309101852733, 1, 1], 
       Hue[0.5872863814003371, 1, 1], 
       Hue[0.5211049841628967, 1, 1], 
       Hue[0.4579218320357416, 1, 1], 
       Hue[0.3978644395073952, 1, 1], 
       Hue[0.34103635547553546`, 1, 1], 
       Hue[0.28751299833736366`, 1, 1], 
       Hue[0.2373425683885671, 1, 1], 
       Hue[0.19054743917487552`, 1, 1], 
       Hue[0.14712457912982915`, 1, 1], 
       Hue[0.10704787791664298`, 1, 1], 
       Hue[0.07027258996258669, 1, 1], 
       Hue[0.036734633034547576`, 1, 1], 
       Hue[0.0063562804960939565`, 1, 1], 
       Hue[0.9790486019005894, 1, 1], 
       Hue[0.954714129564768, 1, 1], 
       Hue[0.9332507821720971, 1, 1], 
       Hue[0.9145521366166071, 1, 1], 
       Hue[0.8985108022332381, 1, 1], 
       Hue[0.885019537265343, 1, 1], 
       Hue[0.8739740784029262, 1, 1], 
       Hue[0.8652717100434656, 1, 1], 
       Hue[0.8588139578755996, 1, 1], 
       Hue[0.8545070219354367, 1, 1], 
       Hue[0.8522605424068836, 1, 1], 
       Hue[0.8519894958550597, 1, 1], 
       Hue[0.8536135359957169, 1, 1], 
       Hue[0.8570565069011515, 1, 1], 
       Hue[0.8622480974679313, 1, 1], 
       Hue[0.8691216861664274, 1, 1], 
       Hue[0.877614308802046, 1, 1], 
       Hue[0.8876687311186671, 1, 1], 
       Hue[0.899230510278553, 1, 1], 
       Hue[0.9122490135588197, 1, 1], 
       Hue[0.9266773800415274, 1, 1], 
       Hue[0.9424715311467944, 1, 1], 
       Hue[0.9595905066764843, 1, 1], 
       Hue[0.9779961683060914, 1, 1], 
       Hue[0.9976531263524046, 1, 1], 
       Hue[0.018527973642258822`, 1, 1], 
       Hue[0.040589846530470396`, 1, 1], 
       Hue[0.06380921313635418, 1, 1], 
       Hue[0.088159456147119, 1, 1], 
       Hue[0.11361500882265167`, 1, 1], 
       Hue[0.14015184759702176`, 1, 1], 
       Hue[0.1677470050651769, 1, 1], 
       Hue[0.1963805235167193, 1, 1], 
       Hue[0.226032063227786, 1, 1], 
       Hue[0.2566825269000016, 1, 1], 
       Hue[0.2883143142722514, 1, 1], 
       Hue[0.32091103841903484`, 1, 1], 
       Hue[0.3544570383872956, 1, 1], 
       Hue[0.3889373946660122, 1, 1], 
       Hue[0.42433703731545136`, 1, 1], 
       Hue[0.4606438306174138, 1, 1], 
       Hue[0.49784493890218584`, 1, 1], 
       Hue[0.535927662459045, 1, 1], 
       Hue[0.5748807416320714, 1, 1], 
       Hue[0.6146930444978822, 1, 1], 
       Hue[0.6553538884368986, 1, 1], 
       Hue[0.696853218504837, 1, 1], 
       Hue[0.7391814889795953, 1, 1], 
       Hue[0.7823289599944764, 1, 1], 
       Hue[0.8262874078226671, 1, 1], 
       Hue[0.8710478798485146, 1, 1], 
       Hue[0.9166034157050914, 1, 1], 
       Hue[0.9629448223502752, 1, 1], 
       Hue[0.01006559471412739, 1, 1], 
       Hue[0.05795832163985246, 1, 1], 
       Hue[0.10661432626890296`, 1, 1], 
       Hue[0.1560288107312348, 1, 1], 
       Hue[0.2061934463881734, 1, 1], 
       Hue[0.25710371868991605`, 1, 1], 
       Hue[0.30875258049480325`, 1, 1], 
       Hue[0.3611346594763807, 1, 1], 
       Hue[0.4142443931279519, 1, 1], 
       Hue[0.46807592250982194`, 1, 1], 
       Hue[0.5226235169268131, 1, 1], 
       Hue[0.5778827231313868, 1, 1], 
       Hue[0.6338467284901352, 1, 1], 
       Hue[0.6905119716638414, 1, 1], 
       Hue[0.7478729708429508, 1, 1], 
       Hue[0.8059262182338722, 1, 1], 
       Hue[0.8646667029288123, 1, 1], 
       Hue[0.9240900148564036, 1, 1], 
       Hue[0.9841921136405349, 1, 1], 
       Hue[0.044968567409435826`, 1, 1], 
       Hue[0.1064158884423126, 1, 1], 
       Hue[0.16852978608792193`, 1, 1], 
       Hue[0.23130554990275484`, 1, 1], 
       Hue[0.29474084889917324`, 1, 1], 
       Hue[0.358831588919603, 1, 1], 
       Hue[0.423572986863044, 1, 1], 
       Hue[0.4889633994506269, 1, 1], 
       Hue[0.5549981042468378, 1, 1], 
       Hue[0.6216756108260031, 1, 1], 
       Hue[0.6889900341282765, 1, 1], 
       Hue[0.7569404375017312, 1, 1], 
       Hue[0.8255219568966075, 1, 1], 
       Hue[0.8947351781930797, 1, 1], 
       Hue[0.9645711467107132, 1, 1], 
       Hue[0.035032200808264795`, 1, 1], 
       Hue[0.10611430302560833`, 1, 1], 
       Hue[0.1778127455017217, 1, 1], 
       Hue[0.25012585040377056`, 1, 1], 
       Hue[0.3230512831048453, 1, 1], 
       Hue[0.39658786037578897`, 1, 1], 
       Hue[0.47073048283222535`, 1, 1], 
       Hue[0.5454760758758606, 1, 1], 
       Hue[0.6208268462480444, 1, 1], 
       Hue[0.6967736619070093, 1, 1], 
       Hue[0.7733207693695567, 1, 1], 
       Hue[0.85046436527129, 1, 1], 
       Hue[0.9281979578049646, 1, 1], 
       Hue[0.006524208684187778, 1, 1], 
       Hue[0.08543857693114915, 1, 1], 
       Hue[0.1649389112410558, 1, 1], 
       Hue[0.24502025669775915`, 1, 1], 
       Hue[0.3256855387817807, 1, 1], 
       Hue[0.4069328254093805, 1, 1], 
       Hue[0.48875648074544686`, 1, 1], 
       Hue[0.5711582052233701, 1, 1], 
       Hue[0.6541335516799798, 1, 1], 
       Hue[0.7376792030777755, 1, 1], 
       Hue[0.8217951224193869, 1, 1], 
       Hue[0.906479842223855, 1, 1], 
       Hue[0.9917352697531961, 1, 1], 
       Hue[0.07755478805897177, 1, 1], 
       Hue[0.9355110244011989, 1, 1], 
       Hue[0.8331243269093445, 1, 1], 
       Hue[0.8331243269093445, 1, 1], 
       Hue[0.7307595415030963, 1, 1], 
       Hue[0.7307595415030963, 1, 1], 
       Hue[0.6279388039622625, 1, 1], 
       Hue[0.6279388039622625, 1, 1], 
       Hue[0.5268258073343911, 1, 1], 
       Hue[0.5268258073343911, 1, 1], 
       Hue[0.424171083302787, 1, 1], 
       Hue[0.424171083302787, 1, 1], 
       Hue[0.3222947041560654, 1, 1], 
       Hue[0.3222947041560654, 1, 1], 
       Hue[0.22127598997976075`, 1, 1], 
       Hue[0.22127598997976075`, 1, 1], 
       Hue[0.12037676786958348`, 1, 1], 
       Hue[0.12037676786958348`, 1, 1], 
       Hue[0.018976825257496246`, 1, 1], 
       Hue[0.018976825257496246`, 1, 1], 
       Hue[0.918522893746869, 1, 1], 
       Hue[0.918522893746869, 1, 1], 
       Hue[0.8182612468644723, 1, 1], 
       Hue[0.8182612468644723, 1, 1], 
       Hue[0.7183833795476677, 1, 1], 
       Hue[0.7183833795476677, 1, 1], 
       Hue[0.6183986488335731, 1, 1], 
       Hue[0.6183986488335731, 1, 1], 
       Hue[0.5193758685502148, 1, 1], 
       Hue[0.5193758685502148, 1, 1], 
       Hue[0.41994575252986666`, 1, 1], 
       Hue[0.41994575252986666`, 1, 1], 
       Hue[0.32105900216888295`, 1, 1], 
       Hue[0.32105900216888295`, 1, 1], 
       Hue[0.22234529701340938`, 1, 1], 
       Hue[0.22234529701340938`, 1, 1], 
       Hue[0.1237823642680303, 1, 1], 
       Hue[0.1237823642680303, 1, 1], 
       Hue[0.02539869814779855, 1, 1], 
       Hue[0.02539869814779855, 1, 1], 
       Hue[0.9275293115368062, 1, 1], 
       Hue[0.9275293115368062, 1, 1], 
       Hue[0.8297207691016857, 1, 1], 
       Hue[0.8297207691016857, 1, 1], 
       Hue[0.7323239009278713, 1, 1], 
       Hue[0.7323239009278713, 1, 1], 
       Hue[0.6349851438514744, 1, 1], 
       Hue[0.6349851438514744, 1, 1], 
       Hue[0.5378773905896601, 1, 1], 
       Hue[0.5378773905896601, 1, 1], 
       Hue[0.4410809731310571, 1, 1], 
       Hue[0.4410809731310571, 1, 1], 
       Hue[0.34442190104855774`, 1, 1], 
       Hue[0.34442190104855774`, 1, 1], 
       Hue[0.24803948086947397`, 1, 1], 
       Hue[0.24803948086947397`, 1, 1], 
       Hue[0.15187714678567418`, 1, 1], 
       Hue[0.15187714678567418`, 1, 1], 
       Hue[0.055945236434117585`, 1, 1], 
       Hue[0.055945236434117585`, 1, 1], 
       Hue[0.9601410826422264, 1, 1], 
       Hue[0.9601410826422264, 1, 1], 
       Hue[0.8646436606294935, 1, 1], 
       Hue[0.8646436606294935, 1, 1], 
       Hue[0.7693298530870238, 1, 1], 
       Hue[0.7693298530870238, 1, 1], 
       Hue[0.6742223440667043, 1, 1], 
       Hue[0.6742223440667043, 1, 1], 
       Hue[0.5793187421548431, 1, 1], 
       Hue[0.5793187421548431, 1, 1], 
       Hue[0.4846333395900681, 1, 1], 
       Hue[0.4846333395900681, 1, 1], 
       Hue[0.3901292437334285, 1, 1], 
       Hue[0.3901292437334285, 1, 1], 
       Hue[0.2958404119994364, 1, 1], 
       Hue[0.2958404119994364, 1, 1], 
       Hue[0.20173313272304044`, 1, 1], 
       Hue[0.20173313272304044`, 1, 1], 
       Hue[0.1078217796597472, 1, 1], 
       Hue[0.1078217796597472, 1, 1], 
       Hue[0.01410988454345159, 1, 1], 
       Hue[0.01410988454345159, 1, 1], 
       Hue[0.9205757406417387, 1, 1], 
       Hue[0.9205757406417387, 1, 1], 
       Hue[0.827240765562894, 1, 1], 
       Hue[0.827240765562894, 1, 1], 
       Hue[0.734071085562777, 1, 1], 
       Hue[0.734071085562777, 1, 1], 
       Hue[0.64109432272257, 1, 1], 
       Hue[0.64109432272257, 1, 1], 
       Hue[0.5482985086259914, 1, 1], 
       Hue[0.5482985086259914, 1, 1], 
       Hue[0.45567974176311293`, 1, 1], 
       Hue[0.45567974176311293`, 1, 1], 
       Hue[0.363222210809178, 1, 1], 
       Hue[0.363222210809178, 1, 1], 
       Hue[0.2709463557656617, 1, 1], 
       Hue[0.2709463557656617, 1, 1], 
       Hue[0.1788348021421335, 1, 1], 
       Hue[0.1788348021421335, 1, 1], 
       Hue[0.0868829353325018, 1, 1], 
       Hue[0.0868829353325018, 1, 1], 
       Hue[0.9950982708020901, 1, 1], 
       Hue[0.9950982708020901, 1, 1], 
       Hue[0.9034663428818953, 1, 1], 
       Hue[0.9034663428818953, 1, 1], 
       Hue[0.8119948684127546, 1, 1], 
       Hue[0.8119948684127546, 1, 1], 
       Hue[0.7206732687351798, 1, 1], 
       Hue[0.7206732687351798, 1, 1], 
       Hue[0.6295015001961014, 1, 1], 
       Hue[0.6295015001961014, 1, 1], 
       Hue[0.5384757740462534, 1, 1], 
       Hue[0.5384757740462534, 1, 1], 
       Hue[0.44759116579465397`, 1, 1], 
       Hue[0.44759116579465397`, 1, 1], 
       Hue[0.3568458792103737, 1, 1], 
       Hue[0.3568458792103737, 1, 1], 
       Hue[0.2662389480082245, 1, 1], 
       Hue[0.2662389480082245, 1, 1], 
       Hue[0.1757615783609027, 1, 1], 
       Hue[0.1757615783609027, 1, 1], 
       Hue[0.0854139219264677, 1, 1], 
       Hue[0.0854139219264677, 1, 1], 
       Hue[0.9951890441798408, 1, 1], 
       Hue[0.9951890441798408, 1, 1], 
       Hue[0.9050842641126323, 1, 1], 
       Hue[0.9050842641126323, 1, 1], 
       Hue[0.8150993924344233, 1, 1], 
       Hue[0.8150993924344233, 1, 1], 
       Hue[0.7252233314084258, 1, 1], 
       Hue[0.7252233314084258, 1, 1], 
       Hue[0.6354592261172873, 1, 1], 
       Hue[0.6354592261172873, 1, 1], 
       Hue[0.5457985330640146, 1, 1], 
       Hue[0.5457985330640146, 1, 1], 
       Hue[0.45623669902216535`, 1, 1], 
       Hue[0.45623669902216535`, 1, 1], 
       Hue[0.36677041065441346`, 1, 1], 
       Hue[0.36677041065441346`, 1, 1], 
       Hue[0.27739418643777836`, 1, 1], 
       Hue[0.27739418643777836`, 1, 1], 
       Hue[0.18810275745714697`, 1, 1], 
       Hue[0.18810275745714697`, 1, 1], 
       Hue[0.09889230647015664, 1, 1], 
       Hue[0.09889230647015664, 1, 1], 
       Hue[0.00975804126335437, 1, 1], 
       Hue[0.00975804126335437, 1, 1], 
       Hue[0.9206921227058936, 1, 1], 
       Hue[0.9206921227058936, 1, 1], 
       Hue[0.8316919469100537, 1, 1], 
       Hue[0.8316919469100537, 1, 1], 
       Hue[0.7427524318223738, 1, 1], 
       Hue[0.7427524318223738, 1, 1], 
       Hue[0.6538648668741538, 1, 1], 
       Hue[0.6538648668741538, 1, 1], 
       Hue[0.5650260486075918, 1, 1], 
       Hue[0.5650260486075918, 1, 1], 
       Hue[0.47622861244209524`, 1, 1], 
       Hue[0.47622861244209524`, 1, 1], 
       Hue[0.3874663043663873, 1, 1], 
       Hue[0.3874663043663873, 1, 1], 
       Hue[0.29873346458885097`, 1, 1], 
       Hue[0.29873346458885097`, 1, 1], 
       Hue[0.2100241658752546, 1, 1], 
       Hue[0.2100241658752546, 1, 1], 
       Hue[0.12133163459447596`, 1, 1], 
       Hue[0.12133163459447596`, 1, 1], 
       Hue[0.032649092430081426`, 1, 1], 
       Hue[0.032649092430081426`, 1, 1], 
       Hue[0.9439705938460063, 1, 1], 
       Hue[0.9439705938460063, 1, 1], 
       Hue[0.8552884086498929, 1, 1], 
       Hue[0.8552884086498929, 1, 1], 
       Hue[0.7665970492852087, 1, 1], 
       Hue[0.7665970492852087, 1, 1], 
       Hue[0.6778898537541902, 1, 1], 
       Hue[0.6778898537541902, 1, 1], 
       Hue[0.5891600056030785, 1, 1], 
       Hue[0.5891600056030785, 1, 1], 
       Hue[0.5004025675691461, 1, 1], 
       Hue[0.5004025675691461, 1, 1], 
       Hue[0.4116114355042888, 1, 1], 
       Hue[0.4116114355042888, 1, 1], 
       Hue[0.322779950578861, 1, 1], 
       Hue[0.322779950578861, 1, 1], 
       Hue[0.23390508369437094`, 1, 1], 
       Hue[0.23390508369437094`, 1, 1], 
       Hue[0.14498100702236932`, 1, 1], 
       Hue[0.14498100702236932`, 1, 1], 
       Hue[0.05600704660552275, 1, 1], 
       Hue[0.05600704660552275, 1, 1], 
       Hue[0.9669795709678686, 1, 1], 
       Hue[0.9669795709678686, 1, 1], 
       Hue[0.8778982582793236, 1, 1], 
       Hue[0.8778982582793236, 1, 1], 
       Hue[0.7887652557309699, 1, 1], 
       Hue[0.7887652557309699, 1, 1], 
       Hue[0.6995823800456853, 1, 1], 
       Hue[0.6995823800456853, 1, 1], 
       Hue[0.6103564710800458, 1, 1], 
       Hue[0.6103564710800458, 1, 1], 
       Hue[0.5210960147936234, 1, 1], 
       Hue[0.5210960147936234, 1, 1], 
       Hue[0.4318125288607402, 1, 1], 
       Hue[0.4318125288607402, 1, 1], 
       Hue[0.34252493459971806`, 1, 1], 
       Hue[0.34252493459971806`, 1, 1], 
       Hue[0.25325369870288117`, 1, 1], 
       Hue[0.25325369870288117`, 1, 1], 
       Hue[0.16402728053127844`, 1, 1], 
       Hue[0.16402728053127844`, 1, 1], 
       Hue[0.07488002620663553, 1, 1], 
       Hue[0.07488002620663553, 1, 1], 
       Hue[0.9858520065701789, 1, 1], 
       Hue[0.9858520065701789, 1, 1], 
       Hue[0.8969931708519823, 1, 1], 
       Hue[0.8969931708519823, 1, 1], 
       Hue[0.8083612816537772, 1, 1], 
       Hue[0.8083612816537772, 1, 1], 
       Hue[0.720027846091456, 1, 1], 
       Hue[0.720027846091456, 1, 1], 
       Hue[0.6320783020478189, 1, 1], 
       Hue[0.6320783020478189, 1, 1], 
       Hue[0.5446126226524881, 1, 1], 
       Hue[0.5446126226524881, 1, 1], 
       Hue[0.4577416941256004, 1, 1], 
       Hue[0.4577416941256004, 1, 1], 
       Hue[0.371589448811156, 1, 1], 
       Hue[0.371589448811156, 1, 1], 
       Hue[0.2862928742918964, 1, 1], 
       Hue[0.2862928742918964, 1, 1], 
       Hue[0.20200228023477101`, 1, 1], 
       Hue[0.20200228023477101`, 1, 1], 
       Hue[0.11888507280599135`, 1, 1], 
       Hue[0.11888507280599135`, 1, 1], 
       Hue[0.037127910830192574`, 1, 1], 
       Hue[0.037127910830192574`, 1, 1], 
       Hue[0.9569362186420415, 1, 1], 
       Hue[0.9569362186420415, 1, 1], 
       Hue[0.8785305257889706, 1, 1], 
       Hue[0.8785305257889706, 1, 1], 
       Hue[0.8021410588218201, 1, 1], 
       Hue[0.8021410588218201, 1, 1], 
       Hue[0.7279945898245506, 1, 1], 
       Hue[0.7279945898245506, 1, 1], 
       Hue[0.656309101852733, 1, 1], 
       Hue[0.656309101852733, 1, 1], 
       Hue[0.5872863814003371, 1, 1], 
       Hue[0.5872863814003371, 1, 1], 
       Hue[0.5211049841628967, 1, 1], 
       Hue[0.5211049841628967, 1, 1], 
       Hue[0.4579218320357416, 1, 1], 
       Hue[0.4579218320357416, 1, 1], 
       Hue[0.3978644395073952, 1, 1], 
       Hue[0.3978644395073952, 1, 1], 
       Hue[0.34103635547553546`, 1, 1], 
       Hue[0.34103635547553546`, 1, 1], 
       Hue[0.28751299833736366`, 1, 1], 
       Hue[0.28751299833736366`, 1, 1], 
       Hue[0.2373425683885671, 1, 1], 
       Hue[0.2373425683885671, 1, 1], 
       Hue[0.19054743917487552`, 1, 1], 
       Hue[0.19054743917487552`, 1, 1], 
       Hue[0.14712457912982915`, 1, 1], 
       Hue[0.14712457912982915`, 1, 1], 
       Hue[0.10704787791664298`, 1, 1], 
       Hue[0.10704787791664298`, 1, 1], 
       Hue[0.07027258996258669, 1, 1], 
       Hue[0.07027258996258669, 1, 1], 
       Hue[0.036734633034547576`, 1, 1], 
       Hue[0.036734633034547576`, 1, 1], 
       Hue[0.0063562804960939565`, 1, 1], 
       Hue[0.0063562804960939565`, 1, 1], 
       Hue[0.9790486019005894, 1, 1], 
       Hue[0.9790486019005894, 1, 1], 
       Hue[0.954714129564768, 1, 1], 
       Hue[0.954714129564768, 1, 1], 
       Hue[0.9332507821720971, 1, 1], 
       Hue[0.9332507821720971, 1, 1], 
       Hue[0.9145521366166071, 1, 1], 
       Hue[0.9145521366166071, 1, 1], 
       Hue[0.8985108022332381, 1, 1], 
       Hue[0.8985108022332381, 1, 1], 
       Hue[0.885019537265343, 1, 1], 
       Hue[0.885019537265343, 1, 1], 
       Hue[0.8739740784029262, 1, 1], 
       Hue[0.8739740784029262, 1, 1], 
       Hue[0.8652717100434656, 1, 1], 
       Hue[0.8652717100434656, 1, 1], 
       Hue[0.8588139578755996, 1, 1], 
       Hue[0.8588139578755996, 1, 1], 
       Hue[0.8545070219354367, 1, 1], 
       Hue[0.8545070219354367, 1, 1], 
       Hue[0.8522605424068836, 1, 1], 
       Hue[0.8522605424068836, 1, 1], 
       Hue[0.8519894958550597, 1, 1], 
       Hue[0.8519894958550597, 1, 1], 
       Hue[0.8536135359957169, 1, 1], 
       Hue[0.8536135359957169, 1, 1], 
       Hue[0.8570565069011515, 1, 1], 
       Hue[0.8570565069011515, 1, 1], 
       Hue[0.8622480974679313, 1, 1], 
       Hue[0.8622480974679313, 1, 1], 
       Hue[0.8691216861664274, 1, 1], 
       Hue[0.8691216861664274, 1, 1], 
       Hue[0.877614308802046, 1, 1], 
       Hue[0.877614308802046, 1, 1], 
       Hue[0.8876687311186671, 1, 1], 
       Hue[0.8876687311186671, 1, 1], 
       Hue[0.899230510278553, 1, 1], 
       Hue[0.899230510278553, 1, 1], 
       Hue[0.9122490135588197, 1, 1], 
       Hue[0.9122490135588197, 1, 1], 
       Hue[0.9266773800415274, 1, 1], 
       Hue[0.9266773800415274, 1, 1], 
       Hue[0.9424715311467944, 1, 1], 
       Hue[0.9424715311467944, 1, 1], 
       Hue[0.9595905066764843, 1, 1], 
       Hue[0.9595905066764843, 1, 1], 
       Hue[0.9779961683060914, 1, 1], 
       Hue[0.9779961683060914, 1, 1], 
       Hue[0.9976531263524046, 1, 1], 
       Hue[0.9976531263524046, 1, 1], 
       Hue[0.018527973642258822`, 1, 1], 
       Hue[0.018527973642258822`, 1, 1], 
       Hue[0.040589846530470396`, 1, 1], 
       Hue[0.040589846530470396`, 1, 1], 
       Hue[0.06380921313635418, 1, 1], 
       Hue[0.06380921313635418, 1, 1], 
       Hue[0.088159456147119, 1, 1], 
       Hue[0.088159456147119, 1, 1], 
       Hue[0.11361500882265167`, 1, 1], 
       Hue[0.11361500882265167`, 1, 1], 
       Hue[0.14015184759702176`, 1, 1], 
       Hue[0.14015184759702176`, 1, 1], 
       Hue[0.1677470050651769, 1, 1], 
       Hue[0.1677470050651769, 1, 1], 
       Hue[0.1963805235167193, 1, 1], 
       Hue[0.1963805235167193, 1, 1], 
       Hue[0.226032063227786, 1, 1], 
       Hue[0.226032063227786, 1, 1], 
       Hue[0.2566825269000016, 1, 1], 
       Hue[0.2566825269000016, 1, 1], 
       Hue[0.2883143142722514, 1, 1], 
       Hue[0.2883143142722514, 1, 1], 
       Hue[0.32091103841903484`, 1, 1], 
       Hue[0.32091103841903484`, 1, 1], 
       Hue[0.3544570383872956, 1, 1], 
       Hue[0.3544570383872956, 1, 1], 
       Hue[0.3889373946660122, 1, 1], 
       Hue[0.3889373946660122, 1, 1], 
       Hue[0.42433703731545136`, 1, 1], 
       Hue[0.42433703731545136`, 1, 1], 
       Hue[0.4606438306174138, 1, 1], 
       Hue[0.4606438306174138, 1, 1], 
       Hue[0.49784493890218584`, 1, 1], 
       Hue[0.49784493890218584`, 1, 1], 
       Hue[0.535927662459045, 1, 1], 
       Hue[0.535927662459045, 1, 1], 
       Hue[0.5748807416320714, 1, 1], 
       Hue[0.5748807416320714, 1, 1], 
       Hue[0.6146930444978822, 1, 1], 
       Hue[0.6146930444978822, 1, 1], 
       Hue[0.6553538884368986, 1, 1], 
       Hue[0.6553538884368986, 1, 1], 
       Hue[0.696853218504837, 1, 1], 
       Hue[0.696853218504837, 1, 1], 
       Hue[0.7391814889795953, 1, 1], 
       Hue[0.7391814889795953, 1, 1], 
       Hue[0.7823289599944764, 1, 1], 
       Hue[0.7823289599944764, 1, 1], 
       Hue[0.8262874078226671, 1, 1], 
       Hue[0.8262874078226671, 1, 1], 
       Hue[0.8710478798485146, 1, 1], 
       Hue[0.8710478798485146, 1, 1], 
       Hue[0.9166034157050914, 1, 1], 
       Hue[0.9166034157050914, 1, 1], 
       Hue[0.9629448223502752, 1, 1], 
       Hue[0.9629448223502752, 1, 1], 
       Hue[0.01006559471412739, 1, 1], 
       Hue[0.01006559471412739, 1, 1], 
       Hue[0.05795832163985246, 1, 1], 
       Hue[0.05795832163985246, 1, 1], 
       Hue[0.10661432626890296`, 1, 1], 
       Hue[0.10661432626890296`, 1, 1], 
       Hue[0.1560288107312348, 1, 1], 
       Hue[0.1560288107312348, 1, 1], 
       Hue[0.2061934463881734, 1, 1], 
       Hue[0.2061934463881734, 1, 1], 
       Hue[0.25710371868991605`, 1, 1], 
       Hue[0.25710371868991605`, 1, 1], 
       Hue[0.30875258049480325`, 1, 1], 
       Hue[0.30875258049480325`, 1, 1], 
       Hue[0.3611346594763807, 1, 1], 
       Hue[0.3611346594763807, 1, 1], 
       Hue[0.4142443931279519, 1, 1], 
       Hue[0.4142443931279519, 1, 1], 
       Hue[0.46807592250982194`, 1, 1], 
       Hue[0.46807592250982194`, 1, 1], 
       Hue[0.5226235169268131, 1, 1], 
       Hue[0.5226235169268131, 1, 1], 
       Hue[0.5778827231313868, 1, 1], 
       Hue[0.5778827231313868, 1, 1], 
       Hue[0.6338467284901352, 1, 1], 
       Hue[0.6338467284901352, 1, 1], 
       Hue[0.6905119716638414, 1, 1], 
       Hue[0.6905119716638414, 1, 1], 
       Hue[0.7478729708429508, 1, 1], 
       Hue[0.7478729708429508, 1, 1], 
       Hue[0.8059262182338722, 1, 1], 
       Hue[0.8059262182338722, 1, 1], 
       Hue[0.8646667029288123, 1, 1], 
       Hue[0.8646667029288123, 1, 1], 
       Hue[0.9240900148564036, 1, 1], 
       Hue[0.9240900148564036, 1, 1], 
       Hue[0.9841921136405349, 1, 1], 
       Hue[0.9841921136405349, 1, 1], 
       Hue[0.044968567409435826`, 1, 1], 
       Hue[0.044968567409435826`, 1, 1], 
       Hue[0.1064158884423126, 1, 1], 
       Hue[0.1064158884423126, 1, 1], 
       Hue[0.16852978608792193`, 1, 1], 
       Hue[0.16852978608792193`, 1, 1], 
       Hue[0.23130554990275484`, 1, 1], 
       Hue[0.23130554990275484`, 1, 1], 
       Hue[0.29474084889917324`, 1, 1], 
       Hue[0.29474084889917324`, 1, 1], 
       Hue[0.358831588919603, 1, 1], 
       Hue[0.358831588919603, 1, 1], 
       Hue[0.423572986863044, 1, 1], 
       Hue[0.423572986863044, 1, 1], 
       Hue[0.4889633994506269, 1, 1], 
       Hue[0.4889633994506269, 1, 1], 
       Hue[0.5549981042468378, 1, 1], 
       Hue[0.5549981042468378, 1, 1], 
       Hue[0.6216756108260031, 1, 1], 
       Hue[0.6216756108260031, 1, 1], 
       Hue[0.6889900341282765, 1, 1], 
       Hue[0.6889900341282765, 1, 1], 
       Hue[0.7569404375017312, 1, 1], 
       Hue[0.7569404375017312, 1, 1], 
       Hue[0.8255219568966075, 1, 1], 
       Hue[0.8255219568966075, 1, 1], 
       Hue[0.8947351781930797, 1, 1], 
       Hue[0.8947351781930797, 1, 1], 
       Hue[0.9645711467107132, 1, 1], 
       Hue[0.9645711467107132, 1, 1], 
       Hue[0.035032200808264795`, 1, 1], 
       Hue[0.035032200808264795`, 1, 1], 
       Hue[0.10611430302560833`, 1, 1], 
       Hue[0.10611430302560833`, 1, 1], 
       Hue[0.1778127455017217, 1, 1], 
       Hue[0.1778127455017217, 1, 1], 
       Hue[0.25012585040377056`, 1, 1], 
       Hue[0.25012585040377056`, 1, 1], 
       Hue[0.3230512831048453, 1, 1], 
       Hue[0.3230512831048453, 1, 1], 
       Hue[0.39658786037578897`, 1, 1], 
       Hue[0.39658786037578897`, 1, 1], 
       Hue[0.47073048283222535`, 1, 1], 
       Hue[0.47073048283222535`, 1, 1], 
       Hue[0.5454760758758606, 1, 1], 
       Hue[0.5454760758758606, 1, 1], 
       Hue[0.6208268462480444, 1, 1], 
       Hue[0.6208268462480444, 1, 1], 
       Hue[0.6967736619070093, 1, 1], 
       Hue[0.6967736619070093, 1, 1], 
       Hue[0.7733207693695567, 1, 1], 
       Hue[0.7733207693695567, 1, 1], 
       Hue[0.85046436527129, 1, 1], 
       Hue[0.85046436527129, 1, 1], 
       Hue[0.9281979578049646, 1, 1], 
       Hue[0.9281979578049646, 1, 1], 
       Hue[0.006524208684187778, 1, 1], 
       Hue[0.006524208684187778, 1, 1], 
       Hue[0.08543857693114915, 1, 1], 
       Hue[0.08543857693114915, 1, 1], 
       Hue[0.1649389112410558, 1, 1], 
       Hue[0.1649389112410558, 1, 1], 
       Hue[0.24502025669775915`, 1, 1], 
       Hue[0.24502025669775915`, 1, 1], 
       Hue[0.3256855387817807, 1, 1], 
       Hue[0.3256855387817807, 1, 1], 
       Hue[0.4069328254093805, 1, 1], 
       Hue[0.4069328254093805, 1, 1], 
       Hue[0.48875648074544686`, 1, 1], 
       Hue[0.48875648074544686`, 1, 1], 
       Hue[0.5711582052233701, 1, 1], 
       Hue[0.5711582052233701, 1, 1], 
       Hue[0.6541335516799798, 1, 1], 
       Hue[0.6541335516799798, 1, 1], 
       Hue[0.7376792030777755, 1, 1], 
       Hue[0.7376792030777755, 1, 1], 
       Hue[0.8217951224193869, 1, 1], 
       Hue[0.8217951224193869, 1, 1], 
       Hue[0.906479842223855, 1, 1], 
       Hue[0.906479842223855, 1, 1], 
       Hue[0.9917352697531961, 1, 1], 
       Hue[0.9917352697531961, 1, 1], 
       Hue[0.07755478805897177, 1, 1]}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->{True, None},
    AxesOrigin->{0, 0},
    Epilog->
     LineBox[{{1, 0.000037384552214890954`}, {2, 0.00004402948613875358}, {
       3, 0.000051761509053314036`}, {4, 0.00006154112652061493}, {
       5, 0.00007129265584977108}, {6, 0.00008429356487128198}, {
       7, 0.00009786336683845261}, {8, 0.00011430847100552931`}, {
       9, 0.00013355977665037894`}, {10, 0.00015538535790489972`}, {
       11, 0.00018083824857960948`}, {12, 0.00021050258563672333`}, {
       13, 0.00024321827031647451`}, {14, 0.00028263518192711486`}, {
       15, 0.00032649003177730835`}, {16, 0.00037718146299153924`}, {
       17, 0.000435795985409994}, {18, 0.00050232323577247}, {
       19, 0.0005776155910632372}, {20, 0.0006644539097170476}, {
       21, 0.0007620135685188869}, {22, 0.0008731886911432147}, {
       23, 0.0010001350116070125`}, {24, 0.001143123728409559}, {
       25, 0.0013051216676307573`}, {26, 0.0014880213521412684`}, {
       27, 0.0016943350635444461`}, {28, 0.0019260706898049165`}, {
       29, 0.002187562648643401}, {30, 0.002481723571716071}, {
       31, 0.0028101637290371163`}, {32, 0.0031786688904200102`}, {
       33, 0.003590314030067674}, {34, 0.0040498708684125985`}, {
       35, 0.004561420661685151}, {36, 0.005130894265673581}, {
       37, 0.005762892484561625}, {38, 0.006463654247981446}, {
       39, 0.007239383930991065}, {40, 0.008097136754030598}, {
       41, 0.009042375426619398}, {42, 0.010084897585188126`}, {
       43, 0.011230435006260767`}, {44, 0.012487825541477017`}, {
       45, 0.013864730135505642`}, {46, 0.015370506867482283`}, {
       47, 0.01701470814643129}, {48, 0.018805220620043087`}, {
       49, 0.020753075395326538`}, {50, 0.022866156456796422`}, {
       51, 0.025157125478979536`}, {52, 0.027632741975517}, {
       53, 0.03030508902617005}, {54, 0.03318214429475098}, {
       55, 0.036273892000890616`}, {56, 0.03958943843236792}, {
       57, 0.043136872914803295`}, {58, 0.046924200150548524`}, {
       59, 0.050958100744902785`}, {60, 0.05524616600624071}, {
       61, 0.05979357034383282}, {62, 0.06460545857600497}, {
       63, 0.06968448109281535}, {64, 0.0750306336272056}, {
       65, 0.08064676880379408}, {66, 0.08652820335801074}, {
       67, 0.09267434195367426}, {68, 0.09907724300020347}, {
       69, 0.10573009546343393`}, {70, 0.11262438392336507`}, {
       71, 0.11974584688114864`}, {72, 0.12708009217218974`}, {
       73, 0.1346119798886439}, {74, 0.1423190200126163}, {
       75, 0.1501826093307986}, {76, 0.15817439298282238`}, {
       77, 0.16627015680145368`}, {78, 0.17443902310026682`}, {
       79, 0.18264896287551163`}, {80, 0.19086781133950695`}, {
       81, 0.19905972762046856`}, {82, 0.20718772486422074`}, {
       83, 0.21521353571782176`}, {84, 0.2230989797948063}, {
       85, 0.2308025473616139}, {86, 0.2382846656374238}, {
       87, 0.24550667239894747`}, {88, 0.25242750091102595`}, {
       89, 0.2590099116346867}, {90, 0.26521846963637796`}, {
       91, 0.2710176137304671}, {92, 0.2763757443645039}, {
       93, 0.28126612769396603`}, {94, 0.2856600497356623}, {
       95, 0.2895389425361334}, {96, 0.2928859736946418}, {
       97, 0.295687634002137}, {98, 0.29793733927320293`}, {
       99, 0.2996329881110879}, {100, 0.300777890812787}, {
       101, 0.30138186881696644`}, {102, 0.30145863776203535`}, {
       103, 0.3010310062849546}, {104, 0.3001246185941583}, {
       105, 0.29876994684973646`}, {106, 0.29700409555525575`}, {
       107, 0.2948647785484878}, {108, 0.29239590625113954`}, {
       109, 0.2896487260600834}, {110, 0.2866757332737224}, {
       111, 0.28353739376526355`}, {112, 0.2802907660208733}, {
       113, 0.27699704388066265`}, {114, 0.273707514503783}, {
       115, 0.2704771336942364}, {116, 0.2673619437912488}, {
       117, 0.26442202824185496`}, {118, 0.2617249807156796}, {
       119, 0.2593407517771608}, {120, 0.2573297646852573}, {
       121, 0.2557435848444642}, {122, 0.25461327771510117`}, {
       123, 0.2539501659222143}, {124, 0.25375136688691413`}, {
       125, 0.25399999852132193`}, {126, 0.25467875554345276`}, {
       127, 0.25576625455596436`}, {128, 0.257243947282749}, {
       129, 0.25909216236266086`}, {130, 0.26129096993058065`}, {
       131, 0.26382137143434875`}, {132, 0.2666626203609747}, {
       133, 0.26979376536190497`}, {134, 0.27318882931382676`}, {
       135, 0.27682364633786005`}, {136, 0.28067007181340226`}, {
       137, 0.2847010666573463}, {138, 0.2888885257514357}, {
       139, 0.2932036919363435}, {140, 0.2976202185530137}, {
       141, 0.302110800305226}, {142, 0.30665090097031883`}, {
       143, 0.3112136114449349}, {144, 0.315775543719446}, {
       145, 0.32031390472244775`}, {146, 0.3248075085857351}, {
       147, 0.3292343138568991}, {148, 0.33357550995829394`}, {
       149, 0.33781177620868075`}, {150, 0.3419268659251715}, {
       151, 0.3459030940795283}, {152, 0.3497240536290127}, {
       153, 0.35337687725593236`}, {154, 0.35684513180557403`}, {
       155, 0.36011682825784586`}, {156, 0.3631802982693446}, {
       157, 0.3660233981142989}, {158, 0.3686364060016859}, {
       159, 0.3710087472278385}, {160, 0.37313154198769394`}, {
       161, 0.3749957509631288}, {162, 0.3765943588219932}, {
       163, 0.3779208757816985}, {164, 0.378970122151151}, {
       165, 0.3797353396675913}, {166, 0.38021419916020566`}, {
       167, 0.380400824885497}, {168, 0.38029480909763397`}, {
       169, 0.3798923538121558}, {170, 0.3791930901611081}, {
       171, 0.37819724537290583`}, {172, 0.3769040474757793}, {
       173, 0.375315410672962}, {174, 0.37343305455546755`}, {
       175, 0.3712586038147189}, {176, 0.36879682254070145`}, {
       177, 0.36605107206463466`}, {178, 0.3630254292273829}, {
       179, 0.35972695156883566`}, {180, 0.3561614496555175}, {
       181, 0.3523355653192698}, {182, 0.34825833821609914`}, {
       183, 0.34393768469083075`}, {184, 0.3393823953274659}, {
       185, 0.3346015326908261}, {186, 0.32960342093517686`}, {
       187, 0.32440010159890276`}, {188, 0.31900082162894383`}, {
       189, 0.31341823271936964`}, {190, 0.3076638373365698}, {
       191, 0.3017483127538634}, {192, 0.2956867298977733}, {
       193, 0.28948979342724523`}, {194, 0.2831696854830931}, {
       195, 0.2767399651114634}, {196, 0.27021151480125527`}, {
       197, 0.2635982369028219}, {198, 0.25691235246307187`}, {
       199, 0.25016916076570367`}, {200, 0.243379391922137}, {
       201, 0.23655790750821795`}, {202, 0.22971530703969972`}, {
       203, 0.22286324741091776`}, {204, 0.2160137766597309}, {
       205, 0.20917829308173305`}, {206, 0.20236973400469568`}, {
       207, 0.19559751050315222`}, {208, 0.18887337060305354`}, {
       209, 0.18220652420117284`}, {210, 0.1756079779774803}, {
       211, 0.16908609084408524`}, {212, 0.16264924399885328`}, {
       213, 0.15630631151119792`}, {214, 0.15006587297945836`}, {
       215, 0.1439333876300512}, {216, 0.13791722198206383`}, {
       217, 0.13202273678927795`}, {218, 0.12625657363446724`}, {
       219, 0.12062195427074902`}, {220, 0.1151250566638951}, {
       221, 0.10976891453715534`}, {222, 0.10455841317114434`}, {
       223, 0.09949437718192684}, {224, 0.09457986354538578}, {
       225, 0.0898187626147869}, {226, 0.08520947027257554}, {
       227, 0.08075462687615945}, {228, 0.0764546782384582}, {
       229, 0.07230964461521586}, {230, 0.06831898665297638}, {
       231, 0.06448096174343011}, {232, 0.06079646515377699}, {
       233, 0.05726254278424689}, {234, 0.053877517143560594`}, {
       235, 0.05064064429104906}, {236, 0.047546790386842196`}, {
       237, 0.04459526631409795}, {238, 0.041782708429035934`}, {
       239, 0.03910561142264679}, {240, 0.03656073444471586}, {
       241, 0.034144912485959605`}, {242, 0.031855197659735913`}, {
       243, 0.029686480997383226`}, {244, 0.027635566299171126`}, {
       245, 0.025698430074535802`}, {246, 0.023870612213392872`}, {
       247, 0.022148636911160226`}, {248, 0.020528428487954296`}, {
       249, 0.019005997000147268`}, {250, 0.017576959858977854`}, {
       251, 0.016236802368262523`}}],
    Frame->True,
    FrameTicks->{{{1., 
        FormBox[
         RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
        FormBox[
         RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
        FormBox[
         RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
        FormBox[
         RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
        FormBox["0", TraditionalForm]}, {157.25, 
        FormBox["2.5`", TraditionalForm]}, {188.5, 
        FormBox["5", TraditionalForm]}, {219.75, 
        FormBox["7.5`", TraditionalForm]}, {251., 
        FormBox["10", TraditionalForm]}}, Automatic, None, None},
    PlotRange->{-0.1, 1.1},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], Automatic}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF13tcjncfB/AfT8jksMwhpy45LDkmKaf6UJIi6Xygrrvuznfl0ci2x7ps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     "], {
      {GrayLevel[0.5], EdgeForm[None], 
       GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ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         "]]]}, {{}, {}, {
        {Hue[0.45492651067977985`, 1, 1], LineBox[{1, 2}]}, 
        {Hue[0.35453989828871174`, 1, 1], LineBox[{2, 3}]}, 
        {Hue[0.25371662554201285`, 1, 1], LineBox[{3, 4}]}, 
        {Hue[0.153334251777296, 1, 1], LineBox[{4, 5}]}, 
        {Hue[0.05307399626199472, 1, 1], LineBox[{5, 6}]}, 
        {Hue[0.95363236378411, 1, 1], LineBox[{6, 7}]}, 
        {Hue[0.8541318403485163, 1, 1], LineBox[{7, 8}]}, 
        {Hue[0.7547295042198715, 1, 1], LineBox[{8, 9}]}, 
        {Hue[0.6559085725275098, 1, 1], LineBox[{9, 10}]}, 
        {Hue[0.5571483343007815, 1, 1], LineBox[{10, 11}]}, 
        {Hue[0.4584138934345222, 1, 1], LineBox[{11, 12}]}, 
        {Hue[0.36039912597627266`, 1, 1], LineBox[{12, 13}]}, 
        {Hue[0.2622953759685387, 1, 1], LineBox[{13, 14}]}, 
        {Hue[0.16436459436553116`, 1, 1], LineBox[{14, 15}]}, 
        {Hue[0.06686467199637351, 1, 1], LineBox[{15, 16}]}, 
        {Hue[0.9695170994455039, 1, 1], LineBox[{16, 17}]}, 
        {Hue[0.8723303846594687, 1, 1], LineBox[{17, 18}]}, 
        {Hue[0.7754090552434305, 1, 1], LineBox[{18, 19}]}, 
        {Hue[0.67874465847557, 1, 1], LineBox[{19, 20}]}, 
        {Hue[0.582229927875765, 1, 1], LineBox[{20, 21}]}, 
        {Hue[0.4859598807852869, 1, 1], LineBox[{21, 22}]}, 
        {Hue[0.38987575474619246`, 1, 1], LineBox[{22, 23}]}, 
        {Hue[0.2940365317975417, 1, 1], LineBox[{23, 24}]}, 
        {Hue[0.19836684198245164`, 1, 1], LineBox[{24, 25}]}, 
        {Hue[0.10294494847248514`, 1, 1], LineBox[{25, 26}]}, 
        {Hue[0.007707782375784896, 1, 1], LineBox[{26, 27}]}, 
        {Hue[0.9126637472741359, 1, 1], LineBox[{27, 28}]}, 
        {Hue[0.8178222771749548, 1, 1], LineBox[{28, 29}]}, 
        {Hue[0.72318127803347, 1, 1], LineBox[{29, 30}]}, 
        {Hue[0.6287338651278908, 1, 1], LineBox[{30, 31}]}, 
        {Hue[0.5344701180581571, 1, 1], LineBox[{31, 32}]}, 
        {Hue[0.44039426020930045`, 1, 1], LineBox[{32, 33}]}, 
        {Hue[0.3465144635884028, 1, 1], LineBox[{33, 34}]}, 
        {Hue[0.252808072997363, 1, 1], LineBox[{34, 35}]}, 
        {Hue[0.15928966223813107`, 1, 1], LineBox[{35, 36}]}, 
        {Hue[0.06594733545312544, 1, 1], LineBox[{36, 37}]}, 
        {Hue[0.9727848443406211, 1, 1], LineBox[{37, 38}]}, 
        {Hue[0.8797973542288998, 1, 1], LineBox[{38, 39}]}, 
        {Hue[0.7869821562937429, 1, 1], LineBox[{39, 40}]}, 
        {Hue[0.6943399395807146, 1, 1], LineBox[{40, 41}]}, 
        {Hue[0.6018559509399378, 1, 1], LineBox[{41, 42}]}, 
        {Hue[0.5095393776462689, 1, 1], LineBox[{42, 43}]}, 
        {Hue[0.4173796678168414, 1, 1], LineBox[{43, 44}]}, 
        {Hue[0.32538385171789214`, 1, 1], LineBox[{44, 45}]}, 
        {Hue[0.23354205356651353`, 1, 1], LineBox[{45, 46}]}, 
        {Hue[0.14185471410378753`, 1, 1], LineBox[{46, 47}]}, 
        {Hue[0.05031447265582115, 1, 1], LineBox[{47, 48}]}, 
        {Hue[0.958923662715511, 1, 1], LineBox[{48, 49}]}, 
        {Hue[0.8676772117323288, 1, 1], LineBox[{49, 50}]}, 
        {Hue[0.7765680700686013, 1, 1], LineBox[{50, 51}]}, 
        {Hue[0.6856019387109209, 1, 1], LineBox[{51, 52}]}, 
        {Hue[0.5947691460253403, 1, 1], LineBox[{52, 53}]}, 
        {Hue[0.5040652266289894, 1, 1], LineBox[{53, 54}]}, 
        {Hue[0.4134896901706374, 1, 1], LineBox[{54, 55}]}, 
        {Hue[0.3230371814924714, 1, 1], LineBox[{55, 56}]}, 
        {Hue[0.2327048098212888, 1, 1], LineBox[{56, 57}]}, 
        {Hue[0.14248935709303306`, 1, 1], LineBox[{57, 58}]}, 
        {Hue[0.05238415176542811, 1, 1], LineBox[{58, 59}]}, 
        {Hue[0.9623891562027246, 1, 1], LineBox[{59, 60}]}, 
        {Hue[0.8724970950084328, 1, 1], LineBox[{60, 61}]}, 
        {Hue[0.782705234830449, 1, 1], LineBox[{61, 62}]}, 
        {Hue[0.6930094276719068, 1, 1], LineBox[{62, 63}]}, 
        {Hue[0.603403558095029, 1, 1], LineBox[{63, 64}]}, 
        {Hue[0.5138848451851199, 1, 1], LineBox[{64, 65}]}, 
        {Hue[0.4244454572392602, 1, 1], LineBox[{65, 66}]}, 
        {Hue[0.3350842173811413, 1, 1], LineBox[{66, 67}]}, 
        {Hue[0.24579166251417273`, 1, 1], LineBox[{67, 68}]}, 
        {Hue[0.15656623339073467`, 1, 1], LineBox[{68, 69}]}, 
        {Hue[0.06739962559988893, 1, 1], LineBox[{69, 70}]}, 
        {Hue[0.9782875994973265, 1, 1], LineBox[{70, 71}]}, 
        {Hue[0.8892234142617812, 1, 1], LineBox[{71, 72}]}, 
        {Hue[0.8002011485261, 1, 1], LineBox[{72, 73}]}, 
        {Hue[0.7112140191474132, 1, 1], LineBox[{73, 74}]}, 
        {Hue[0.6222542253030292, 1, 1], LineBox[{74, 75}]}, 
        {Hue[0.5333177457842979, 1, 1], LineBox[{75, 76}]}, 
        {Hue[0.44439506979053767`, 1, 1], LineBox[{76, 77}]}, 
        {Hue[0.3554794286460181, 1, 1], LineBox[{77, 78}]}, 
        {Hue[0.26656299622171675`, 1, 1], LineBox[{78, 79}]}, 
        {Hue[0.17763793048842322`, 1, 1], LineBox[{79, 80}]}, 
        {Hue[0.08869610636692972, 1, 1], LineBox[{80, 81}]}, 
        {Hue[0.9997283819955648, 1, 1], LineBox[{81, 82}]}, 
        {Hue[0.9107265702305547, 1, 1], LineBox[{82, 83}]}, 
        {Hue[0.821681754524422, 1, 1], LineBox[{83, 84}]}, 
        {Hue[0.7325848999670737, 1, 1], LineBox[{84, 85}]}, 
        {Hue[0.6434256600246612, 1, 1], LineBox[{85, 86}]}, 
        {Hue[0.5541954241832242, 1, 1], LineBox[{86, 87}]}, 
        {Hue[0.46488295327061496`, 1, 1], LineBox[{87, 88}]}, 
        {Hue[0.3754802808713916, 1, 1], LineBox[{88, 89}]}, 
        {Hue[0.28597636002231513`, 1, 1], LineBox[{89, 90}]}, 
        {Hue[0.1963625454614133, 1, 1], LineBox[{90, 91}]}, 
        {Hue[0.10662868654545397`, 1, 1], LineBox[{91, 92}]}, 
        {Hue[0.016766218274584323`, 1, 1], LineBox[{92, 93}]}, 
        {Hue[0.9267666720232012, 1, 1], LineBox[{93, 94}]}, 
        {Hue[0.8366209546576345, 1, 1], LineBox[{94, 95}]}, 
        {Hue[0.7463234324270356, 1, 1], LineBox[{95, 96}]}, 
        {Hue[0.6558683946811299, 1, 1], LineBox[{96, 97}]}, 
        {Hue[0.5652519424005065, 1, 1], LineBox[{97, 98}]}, 
        {Hue[0.4744728032182932, 1, 1], LineBox[{98, 99}]}, 
        {Hue[0.38353075616650895`, 1, 1], LineBox[{99, 100}]}, 
        {Hue[0.29242972469312417`, 1, 1], LineBox[{100, 101}]}, 
        {Hue[0.20117765285064337`, 1, 1], LineBox[{101, 102}]}, 
        {Hue[0.1097862900027622, 1, 1], LineBox[{102, 103}]}, 
        {Hue[0.01827287832674748, 1, 1], LineBox[{103, 104}]}, 
        {Hue[0.9266611974571188, 1, 1], LineBox[{104, 105}]}, 
        {Hue[0.8349800174043959, 1, 1], LineBox[{105, 106}]}, 
        {Hue[0.7432670997335856, 1, 1], LineBox[{106, 107}]}, 
        {Hue[0.6515686779597296, 1, 1], LineBox[{107, 108}]}, 
        {Hue[0.5599424694431372, 1, 1], LineBox[{108, 109}]}, 
        {Hue[0.46845481596904104`, 1, 1], LineBox[{109, 110}]}, 
        {Hue[0.3771862786362149, 1, 1], LineBox[{110, 111}]}, 
        {Hue[0.2862282662686505, 1, 1], LineBox[{111, 112}]}, 
        {Hue[0.19568641537022163`, 1, 1], LineBox[{112, 113}]}, 
        {Hue[0.10567797607262314`, 1, 1], LineBox[{113, 114}]}, 
        {Hue[0.016332731301606014`, 1, 1], LineBox[{114, 115}]}, 
        {Hue[0.927795591376938, 1, 1], LineBox[{115, 116}]}, 
        {Hue[0.8402276070843839, 1, 1], LineBox[{116, 117}]}, 
        {Hue[0.7538035231069398, 1, 1], LineBox[{117, 118}]}, 
        {Hue[0.6687130433992411, 1, 1], LineBox[{118, 119}]}, 
        {Hue[0.5851522832589728, 1, 1], LineBox[{119, 120}]}, 
        {Hue[0.5033241434569828, 1, 1], LineBox[{120, 121}]}, 
        {Hue[0.42343048382940907`, 1, 1], LineBox[{121, 122}]}, 
        {Hue[0.3456705554356446, 1, 1], LineBox[{122, 123}]}, 
        {Hue[0.27023803792139145`, 1, 1], LineBox[{123, 124}]}, 
        {Hue[0.19731503301541775`, 1, 1], LineBox[{124, 125}]}, 
        {Hue[0.1270771087857017, 1, 1], LineBox[{125, 126}]}, 
        {Hue[0.05968324559803208, 1, 1], LineBox[{126, 127}]}, 
        {Hue[0.9952806746680052, 1, 1], LineBox[{127, 128}]}, 
        {Hue[0.9339888537835502, 1, 1], LineBox[{128, 129}]}, 
        {Hue[0.8759098972504199, 1, 1], LineBox[{129, 130}]}, 
        {Hue[0.8211168140301675, 1, 1], LineBox[{130, 131}]}, 
        {Hue[0.769657595305148, 1, 1], LineBox[{131, 132}]}, 
        {Hue[0.72155306419796, 1, 1], LineBox[{132, 133}]}, 
        {Hue[0.6767989892674167, 1, 1], LineBox[{133, 134}]}, 
        {Hue[0.6353668692228582, 1, 1], LineBox[{134, 135}]}, 
        {Hue[0.5972087793115762, 1, 1], LineBox[{135, 136}]}, 
        {Hue[0.5622604326917748, 1, 1], LineBox[{136, 137}]}, 
        {Hue[0.5304403852817042, 1, 1], LineBox[{137, 138}]}, 
        {Hue[0.501658519810958, 1, 1], LineBox[{138, 139}]}, 
        {Hue[0.47581545273971154`, 1, 1], LineBox[{139, 140}]}, 
        {Hue[0.45280660192802846`, 1, 1], LineBox[{140, 141}]}, 
        {Hue[0.4325244262555183, 1, 1], LineBox[{141, 142}]}, 
        {Hue[0.4148607958877386, 1, 1], LineBox[{142, 143}]}, 
        {Hue[0.3997064885717363, 1, 1], LineBox[{143, 144}]}, 
        {Hue[0.3869563679568564, 1, 1], LineBox[{144, 145}]}, 
        {Hue[0.37650730231757107`, 1, 1], LineBox[{145, 146}]}, 
        {Hue[0.3682607261039661, 1, 1], LineBox[{146, 147}]}, 
        {Hue[0.3621209982291931, 1, 1], LineBox[{147, 148}]}, 
        {Hue[0.3579978426514018, 1, 1], LineBox[{148, 149}]}, 
        {Hue[0.35580683697359283`, 1, 1], LineBox[{149, 150}]}, 
        {Hue[0.3554662759685706, 1, 1], LineBox[{150, 151}]}, 
        {Hue[0.3569013213821672, 1, 1], LineBox[{151, 152}]}, 
        {Hue[0.36004081361650353`, 1, 1], LineBox[{152, 153}]}, 
        {Hue[0.3648181761077391, 1, 1], LineBox[{153, 154}]}, 
        {Hue[0.37117149197985105`, 1, 1], LineBox[{154, 155}]}, 
        {Hue[0.37904337644688246`, 1, 1], LineBox[{155, 156}]}, 
        {Hue[0.38838003033941537`, 1, 1], LineBox[{156, 157}]}, 
        {Hue[0.39913105246117875`, 1, 1], LineBox[{157, 158}]}, 
        {Hue[0.411249920977416, 1, 1], LineBox[{158, 159}]}, 
        {Hue[0.424693391058465, 1, 1], LineBox[{159, 160}]}, 
        {Hue[0.4394211657571874, 1, 1], LineBox[{160, 161}]}, 
        {Hue[0.45539537047986617`, 1, 1], LineBox[{161, 162}]}, 
        {Hue[0.47258130536614373`, 1, 1], LineBox[{162, 163}]}, 
        {Hue[0.4909467062963725, 1, 1], LineBox[{163, 164}]}, 
        {Hue[0.5104604752212907, 1, 1], LineBox[{164, 165}]}, 
        {Hue[0.5310943454416712, 1, 1], LineBox[{165, 166}]}, 
        {Hue[0.552822658366199, 1, 1], LineBox[{166, 167}]}, 
        {Hue[0.5756200347081709, 1, 1], LineBox[{167, 168}]}, 
        {Hue[0.5994628887166491, 1, 1], LineBox[{168, 169}]}, 
        {Hue[0.6243296837994351, 1, 1], LineBox[{169, 170}]}, 
        {Hue[0.6502004900681735, 1, 1], LineBox[{170, 171}]}, 
        {Hue[0.6770556375018364, 1, 1], LineBox[{171, 172}]}, 
        {Hue[0.7048773963672461, 1, 1], LineBox[{172, 173}]}, 
        {Hue[0.7336477827191039, 1, 1], LineBox[{173, 174}]}, 
        {Hue[0.7633516997074754, 1, 1], LineBox[{174, 175}]}, 
        {Hue[0.7939737135007793, 1, 1], LineBox[{175, 176}]}, 
        {Hue[0.8254995106024579, 1, 1], LineBox[{176, 177}]}, 
        {Hue[0.8579156762552613, 1, 1], LineBox[{177, 178}]}, 
        {Hue[0.8912095063230834, 1, 1], LineBox[{178, 179}]}, 
        {Hue[0.925368717798873, 1, 1], LineBox[{179, 180}]}, 
        {Hue[0.9603824546877948, 1, 1], LineBox[{180, 181}]}, 
        {Hue[0.9962385682294179, 1, 1], LineBox[{181, 182}]}, 
        {Hue[0.03292654017297939, 1, 1], LineBox[{182, 183}]}, 
        {Hue[0.07043714394267478, 1, 1], LineBox[{183, 184}]}, 
        {Hue[0.10875968330459024`, 1, 1], LineBox[{184, 185}]}, 
        {Hue[0.14788561304282966`, 1, 1], LineBox[{185, 186}]}, 
        {Hue[0.18780615362640285`, 1, 1], LineBox[{186, 187}]}, 
        {Hue[0.22851270426524767`, 1, 1], LineBox[{187, 188}]}, 
        {Hue[0.26999801955043884`, 1, 1], LineBox[{188, 189}]}, 
        {Hue[0.3122539822073713, 1, 1], LineBox[{189, 190}]}, 
        {Hue[0.35527287615958714`, 1, 1], LineBox[{190, 191}]}, 
        {Hue[0.39904799194802826`, 1, 1], LineBox[{191, 192}]}, 
        {Hue[0.44357149723895656`, 1, 1], LineBox[{192, 193}]}, 
        {Hue[0.48883781441948254`, 1, 1], LineBox[{193, 194}]}, 
        {Hue[0.534839279152306, 1, 1], LineBox[{194, 195}]}, 
        {Hue[0.5815702894260358, 1, 1], LineBox[{195, 196}]}, 
        {Hue[0.6290256025850298, 1, 1], LineBox[{196, 197}]}, 
        {Hue[0.6771988392967269, 1, 1], LineBox[{197, 198}]}, 
        {Hue[0.7260852621317265, 1, 1], LineBox[{198, 199}]}, 
        {Hue[0.7756785902064173, 1, 1], LineBox[{199, 200}]}, 
        {Hue[0.8259737909039171, 1, 1], LineBox[{200, 201}]}, 
        {Hue[0.8769657593370601, 1, 1], LineBox[{201, 202}]}, 
        {Hue[0.9286486911989955, 1, 1], LineBox[{202, 203}]}, 
        {Hue[0.9810188386237015, 1, 1], LineBox[{203, 204}]}, 
        {Hue[0.034069971038059345`, 1, 1], LineBox[{204, 205}]}, 
        {Hue[0.08779934893459017, 1, 1], LineBox[{205, 206}]}, 
        {Hue[0.1422018167527364, 1, 1], LineBox[{206, 207}]}, 
        {Hue[0.19727378340379548`, 1, 1], LineBox[{207, 208}]}, 
        {Hue[0.2530108487712756, 1, 1], LineBox[{208, 209}]}, 
        {Hue[0.30940900332769283`, 1, 1], LineBox[{209, 210}]}, 
        {Hue[0.3664635294786629, 1, 1], LineBox[{210, 211}]}, 
        {Hue[0.42417153233580135`, 1, 1], LineBox[{211, 212}]}, 
        {Hue[0.48252852701740456`, 1, 1], LineBox[{212, 213}]}, 
        {Hue[0.5415311942928878, 1, 1], LineBox[{213, 214}]}, 
        {Hue[0.6011771487708214, 1, 1], LineBox[{214, 215}]}, 
        {Hue[0.6614611660479162, 1, 1], LineBox[{215, 216}]}, 
        {Hue[0.7223807452582915, 1, 1], LineBox[{216, 217}]}, 
        {Hue[0.7839319540579166, 1, 1], LineBox[{217, 218}]}, 
        {Hue[0.8461120801534268, 1, 1], LineBox[{218, 219}]}, 
        {Hue[0.9089177824574007, 1, 1], LineBox[{219, 220}]}, 
        {Hue[0.972346258437172, 1, 1], LineBox[{220, 221}]}, 
        {Hue[0.03639461960368258, 1, 1], LineBox[{221, 222}]}, 
        {Hue[0.10105942154813613`, 1, 1], LineBox[{222, 223}]}, 
        {Hue[0.16633766436514846`, 1, 1], LineBox[{223, 224}]}, 
        {Hue[0.23222719273173698`, 1, 1], LineBox[{224, 225}]}, 
        {Hue[0.2987238647155857, 1, 1], LineBox[{225, 226}]}, 
        {Hue[0.36582744102738923`, 1, 1], LineBox[{226, 227}]}, 
        {Hue[0.4335325386500767, 1, 1], LineBox[{227, 228}]}, 
        {Hue[0.5018379833859594, 1, 1], LineBox[{228, 229}]}, 
        {Hue[0.5707408969998853, 1, 1], LineBox[{229, 230}]}, 
        {Hue[0.6402401203183907, 1, 1], LineBox[{230, 231}]}, 
        {Hue[0.7103314443312188, 1, 1], LineBox[{231, 232}]}, 
        {Hue[0.7810129385835942, 1, 1], LineBox[{232, 233}]}, 
        {Hue[0.8522840511817902, 1, 1], LineBox[{233, 234}]}, 
        {Hue[0.9241392387208713, 1, 1], LineBox[{234, 235}]}, 
        {Hue[0.9965798263202051, 1, 1], LineBox[{235, 236}]}, 
        {Hue[0.06959900745981731, 1, 1], LineBox[{236, 237}]}, 
        {Hue[0.14319934802733422`, 1, 1], LineBox[{237, 238}]}, 
        {Hue[0.21737807633544506`, 1, 1], LineBox[{238, 239}]}, 
        {Hue[0.2921281655178836, 1, 1], LineBox[{239, 240}]}, 
        {Hue[0.3674526228331577, 1, 1], LineBox[{240, 241}]}, 
        {Hue[0.4433487386760116, 1, 1], LineBox[{241, 242}]}, 
        {Hue[0.519813479068898, 1, 1], LineBox[{242, 243}]}, 
        {Hue[0.5968450073967411, 1, 1], LineBox[{243, 244}]}, 
        {Hue[0.6744391898025186, 1, 1], LineBox[{244, 245}]}, 
        {Hue[0.7526026203570177, 1, 1], LineBox[{245, 246}]}, 
        {Hue[0.8313227714345117, 1, 1], LineBox[{246, 247}]}, 
        {Hue[0.9106020572940575, 1, 1], LineBox[{247, 248}]}, 
        {Hue[0.99044294769886, 1, 1], LineBox[{248, 249}]}, 
        {Hue[0.07083722612697897, 1, 1], LineBox[{249, 250}]}, 
        {Hue[0.15178796486556909`, 1, 1], LineBox[{250, 251}]}}}},
     VertexColors->{
       Hue[0.45492651067977985`, 1, 1], 
       Hue[0.35453989828871174`, 1, 1], 
       Hue[0.25371662554201285`, 1, 1], 
       Hue[0.153334251777296, 1, 1], 
       Hue[0.05307399626199472, 1, 1], 
       Hue[0.95363236378411, 1, 1], 
       Hue[0.8541318403485163, 1, 1], 
       Hue[0.7547295042198715, 1, 1], 
       Hue[0.6559085725275098, 1, 1], 
       Hue[0.5571483343007815, 1, 1], 
       Hue[0.4584138934345222, 1, 1], 
       Hue[0.36039912597627266`, 1, 1], 
       Hue[0.2622953759685387, 1, 1], 
       Hue[0.16436459436553116`, 1, 1], 
       Hue[0.06686467199637351, 1, 1], 
       Hue[0.9695170994455039, 1, 1], 
       Hue[0.8723303846594687, 1, 1], 
       Hue[0.7754090552434305, 1, 1], 
       Hue[0.67874465847557, 1, 1], 
       Hue[0.582229927875765, 1, 1], 
       Hue[0.4859598807852869, 1, 1], 
       Hue[0.38987575474619246`, 1, 1], 
       Hue[0.2940365317975417, 1, 1], 
       Hue[0.19836684198245164`, 1, 1], 
       Hue[0.10294494847248514`, 1, 1], 
       Hue[0.007707782375784896, 1, 1], 
       Hue[0.9126637472741359, 1, 1], 
       Hue[0.8178222771749548, 1, 1], 
       Hue[0.72318127803347, 1, 1], 
       Hue[0.6287338651278908, 1, 1], 
       Hue[0.5344701180581571, 1, 1], 
       Hue[0.44039426020930045`, 1, 1], 
       Hue[0.3465144635884028, 1, 1], 
       Hue[0.252808072997363, 1, 1], 
       Hue[0.15928966223813107`, 1, 1], 
       Hue[0.06594733545312544, 1, 1], 
       Hue[0.9727848443406211, 1, 1], 
       Hue[0.8797973542288998, 1, 1], 
       Hue[0.7869821562937429, 1, 1], 
       Hue[0.6943399395807146, 1, 1], 
       Hue[0.6018559509399378, 1, 1], 
       Hue[0.5095393776462689, 1, 1], 
       Hue[0.4173796678168414, 1, 1], 
       Hue[0.32538385171789214`, 1, 1], 
       Hue[0.23354205356651353`, 1, 1], 
       Hue[0.14185471410378753`, 1, 1], 
       Hue[0.05031447265582115, 1, 1], 
       Hue[0.958923662715511, 1, 1], 
       Hue[0.8676772117323288, 1, 1], 
       Hue[0.7765680700686013, 1, 1], 
       Hue[0.6856019387109209, 1, 1], 
       Hue[0.5947691460253403, 1, 1], 
       Hue[0.5040652266289894, 1, 1], 
       Hue[0.4134896901706374, 1, 1], 
       Hue[0.3230371814924714, 1, 1], 
       Hue[0.2327048098212888, 1, 1], 
       Hue[0.14248935709303306`, 1, 1], 
       Hue[0.05238415176542811, 1, 1], 
       Hue[0.9623891562027246, 1, 1], 
       Hue[0.8724970950084328, 1, 1], 
       Hue[0.782705234830449, 1, 1], 
       Hue[0.6930094276719068, 1, 1], 
       Hue[0.603403558095029, 1, 1], 
       Hue[0.5138848451851199, 1, 1], 
       Hue[0.4244454572392602, 1, 1], 
       Hue[0.3350842173811413, 1, 1], 
       Hue[0.24579166251417273`, 1, 1], 
       Hue[0.15656623339073467`, 1, 1], 
       Hue[0.06739962559988893, 1, 1], 
       Hue[0.9782875994973265, 1, 1], 
       Hue[0.8892234142617812, 1, 1], 
       Hue[0.8002011485261, 1, 1], 
       Hue[0.7112140191474132, 1, 1], 
       Hue[0.6222542253030292, 1, 1], 
       Hue[0.5333177457842979, 1, 1], 
       Hue[0.44439506979053767`, 1, 1], 
       Hue[0.3554794286460181, 1, 1], 
       Hue[0.26656299622171675`, 1, 1], 
       Hue[0.17763793048842322`, 1, 1], 
       Hue[0.08869610636692972, 1, 1], 
       Hue[0.9997283819955648, 1, 1], 
       Hue[0.9107265702305547, 1, 1], 
       Hue[0.821681754524422, 1, 1], 
       Hue[0.7325848999670737, 1, 1], 
       Hue[0.6434256600246612, 1, 1], 
       Hue[0.5541954241832242, 1, 1], 
       Hue[0.46488295327061496`, 1, 1], 
       Hue[0.3754802808713916, 1, 1], 
       Hue[0.28597636002231513`, 1, 1], 
       Hue[0.1963625454614133, 1, 1], 
       Hue[0.10662868654545397`, 1, 1], 
       Hue[0.016766218274584323`, 1, 1], 
       Hue[0.9267666720232012, 1, 1], 
       Hue[0.8366209546576345, 1, 1], 
       Hue[0.7463234324270356, 1, 1], 
       Hue[0.6558683946811299, 1, 1], 
       Hue[0.5652519424005065, 1, 1], 
       Hue[0.4744728032182932, 1, 1], 
       Hue[0.38353075616650895`, 1, 1], 
       Hue[0.29242972469312417`, 1, 1], 
       Hue[0.20117765285064337`, 1, 1], 
       Hue[0.1097862900027622, 1, 1], 
       Hue[0.01827287832674748, 1, 1], 
       Hue[0.9266611974571188, 1, 1], 
       Hue[0.8349800174043959, 1, 1], 
       Hue[0.7432670997335856, 1, 1], 
       Hue[0.6515686779597296, 1, 1], 
       Hue[0.5599424694431372, 1, 1], 
       Hue[0.46845481596904104`, 1, 1], 
       Hue[0.3771862786362149, 1, 1], 
       Hue[0.2862282662686505, 1, 1], 
       Hue[0.19568641537022163`, 1, 1], 
       Hue[0.10567797607262314`, 1, 1], 
       Hue[0.016332731301606014`, 1, 1], 
       Hue[0.927795591376938, 1, 1], 
       Hue[0.8402276070843839, 1, 1], 
       Hue[0.7538035231069398, 1, 1], 
       Hue[0.6687130433992411, 1, 1], 
       Hue[0.5851522832589728, 1, 1], 
       Hue[0.5033241434569828, 1, 1], 
       Hue[0.42343048382940907`, 1, 1], 
       Hue[0.3456705554356446, 1, 1], 
       Hue[0.27023803792139145`, 1, 1], 
       Hue[0.19731503301541775`, 1, 1], 
       Hue[0.1270771087857017, 1, 1], 
       Hue[0.05968324559803208, 1, 1], 
       Hue[0.9952806746680052, 1, 1], 
       Hue[0.9339888537835502, 1, 1], 
       Hue[0.8759098972504199, 1, 1], 
       Hue[0.8211168140301675, 1, 1], 
       Hue[0.769657595305148, 1, 1], 
       Hue[0.72155306419796, 1, 1], 
       Hue[0.6767989892674167, 1, 1], 
       Hue[0.6353668692228582, 1, 1], 
       Hue[0.5972087793115762, 1, 1], 
       Hue[0.5622604326917748, 1, 1], 
       Hue[0.5304403852817042, 1, 1], 
       Hue[0.501658519810958, 1, 1], 
       Hue[0.47581545273971154`, 1, 1], 
       Hue[0.45280660192802846`, 1, 1], 
       Hue[0.4325244262555183, 1, 1], 
       Hue[0.4148607958877386, 1, 1], 
       Hue[0.3997064885717363, 1, 1], 
       Hue[0.3869563679568564, 1, 1], 
       Hue[0.37650730231757107`, 1, 1], 
       Hue[0.3682607261039661, 1, 1], 
       Hue[0.3621209982291931, 1, 1], 
       Hue[0.3579978426514018, 1, 1], 
       Hue[0.35580683697359283`, 1, 1], 
       Hue[0.3554662759685706, 1, 1], 
       Hue[0.3569013213821672, 1, 1], 
       Hue[0.36004081361650353`, 1, 1], 
       Hue[0.3648181761077391, 1, 1], 
       Hue[0.37117149197985105`, 1, 1], 
       Hue[0.37904337644688246`, 1, 1], 
       Hue[0.38838003033941537`, 1, 1], 
       Hue[0.39913105246117875`, 1, 1], 
       Hue[0.411249920977416, 1, 1], 
       Hue[0.424693391058465, 1, 1], 
       Hue[0.4394211657571874, 1, 1], 
       Hue[0.45539537047986617`, 1, 1], 
       Hue[0.47258130536614373`, 1, 1], 
       Hue[0.4909467062963725, 1, 1], 
       Hue[0.5104604752212907, 1, 1], 
       Hue[0.5310943454416712, 1, 1], 
       Hue[0.552822658366199, 1, 1], 
       Hue[0.5756200347081709, 1, 1], 
       Hue[0.5994628887166491, 1, 1], 
       Hue[0.6243296837994351, 1, 1], 
       Hue[0.6502004900681735, 1, 1], 
       Hue[0.6770556375018364, 1, 1], 
       Hue[0.7048773963672461, 1, 1], 
       Hue[0.7336477827191039, 1, 1], 
       Hue[0.7633516997074754, 1, 1], 
       Hue[0.7939737135007793, 1, 1], 
       Hue[0.8254995106024579, 1, 1], 
       Hue[0.8579156762552613, 1, 1], 
       Hue[0.8912095063230834, 1, 1], 
       Hue[0.925368717798873, 1, 1], 
       Hue[0.9603824546877948, 1, 1], 
       Hue[0.9962385682294179, 1, 1], 
       Hue[0.03292654017297939, 1, 1], 
       Hue[0.07043714394267478, 1, 1], 
       Hue[0.10875968330459024`, 1, 1], 
       Hue[0.14788561304282966`, 1, 1], 
       Hue[0.18780615362640285`, 1, 1], 
       Hue[0.22851270426524767`, 1, 1], 
       Hue[0.26999801955043884`, 1, 1], 
       Hue[0.3122539822073713, 1, 1], 
       Hue[0.35527287615958714`, 1, 1], 
       Hue[0.39904799194802826`, 1, 1], 
       Hue[0.44357149723895656`, 1, 1], 
       Hue[0.48883781441948254`, 1, 1], 
       Hue[0.534839279152306, 1, 1], 
       Hue[0.5815702894260358, 1, 1], 
       Hue[0.6290256025850298, 1, 1], 
       Hue[0.6771988392967269, 1, 1], 
       Hue[0.7260852621317265, 1, 1], 
       Hue[0.7756785902064173, 1, 1], 
       Hue[0.8259737909039171, 1, 1], 
       Hue[0.8769657593370601, 1, 1], 
       Hue[0.9286486911989955, 1, 1], 
       Hue[0.9810188386237015, 1, 1], 
       Hue[0.034069971038059345`, 1, 1], 
       Hue[0.08779934893459017, 1, 1], 
       Hue[0.1422018167527364, 1, 1], 
       Hue[0.19727378340379548`, 1, 1], 
       Hue[0.2530108487712756, 1, 1], 
       Hue[0.30940900332769283`, 1, 1], 
       Hue[0.3664635294786629, 1, 1], 
       Hue[0.42417153233580135`, 1, 1], 
       Hue[0.48252852701740456`, 1, 1], 
       Hue[0.5415311942928878, 1, 1], 
       Hue[0.6011771487708214, 1, 1], 
       Hue[0.6614611660479162, 1, 1], 
       Hue[0.7223807452582915, 1, 1], 
       Hue[0.7839319540579166, 1, 1], 
       Hue[0.8461120801534268, 1, 1], 
       Hue[0.9089177824574007, 1, 1], 
       Hue[0.972346258437172, 1, 1], 
       Hue[0.03639461960368258, 1, 1], 
       Hue[0.10105942154813613`, 1, 1], 
       Hue[0.16633766436514846`, 1, 1], 
       Hue[0.23222719273173698`, 1, 1], 
       Hue[0.2987238647155857, 1, 1], 
       Hue[0.36582744102738923`, 1, 1], 
       Hue[0.4335325386500767, 1, 1], 
       Hue[0.5018379833859594, 1, 1], 
       Hue[0.5707408969998853, 1, 1], 
       Hue[0.6402401203183907, 1, 1], 
       Hue[0.7103314443312188, 1, 1], 
       Hue[0.7810129385835942, 1, 1], 
       Hue[0.8522840511817902, 1, 1], 
       Hue[0.9241392387208713, 1, 1], 
       Hue[0.9965798263202051, 1, 1], 
       Hue[0.06959900745981731, 1, 1], 
       Hue[0.14319934802733422`, 1, 1], 
       Hue[0.21737807633544506`, 1, 1], 
       Hue[0.2921281655178836, 1, 1], 
       Hue[0.3674526228331577, 1, 1], 
       Hue[0.4433487386760116, 1, 1], 
       Hue[0.519813479068898, 1, 1], 
       Hue[0.5968450073967411, 1, 1], 
       Hue[0.6744391898025186, 1, 1], 
       Hue[0.7526026203570177, 1, 1], 
       Hue[0.8313227714345117, 1, 1], 
       Hue[0.9106020572940575, 1, 1], 
       Hue[0.99044294769886, 1, 1], 
       Hue[0.07083722612697897, 1, 1], 
       Hue[0.15178796486556909`, 1, 1], 
       Hue[0.2332917296675744, 1, 1], 
       Hue[0.45492651067977985`, 1, 1], 
       Hue[0.35453989828871174`, 1, 1], 
       Hue[0.35453989828871174`, 1, 1], 
       Hue[0.25371662554201285`, 1, 1], 
       Hue[0.25371662554201285`, 1, 1], 
       Hue[0.153334251777296, 1, 1], 
       Hue[0.153334251777296, 1, 1], 
       Hue[0.05307399626199472, 1, 1], 
       Hue[0.05307399626199472, 1, 1], 
       Hue[0.95363236378411, 1, 1], 
       Hue[0.95363236378411, 1, 1], 
       Hue[0.8541318403485163, 1, 1], 
       Hue[0.8541318403485163, 1, 1], 
       Hue[0.7547295042198715, 1, 1], 
       Hue[0.7547295042198715, 1, 1], 
       Hue[0.6559085725275098, 1, 1], 
       Hue[0.6559085725275098, 1, 1], 
       Hue[0.5571483343007815, 1, 1], 
       Hue[0.5571483343007815, 1, 1], 
       Hue[0.4584138934345222, 1, 1], 
       Hue[0.4584138934345222, 1, 1], 
       Hue[0.36039912597627266`, 1, 1], 
       Hue[0.36039912597627266`, 1, 1], 
       Hue[0.2622953759685387, 1, 1], 
       Hue[0.2622953759685387, 1, 1], 
       Hue[0.16436459436553116`, 1, 1], 
       Hue[0.16436459436553116`, 1, 1], 
       Hue[0.06686467199637351, 1, 1], 
       Hue[0.06686467199637351, 1, 1], 
       Hue[0.9695170994455039, 1, 1], 
       Hue[0.9695170994455039, 1, 1], 
       Hue[0.8723303846594687, 1, 1], 
       Hue[0.8723303846594687, 1, 1], 
       Hue[0.7754090552434305, 1, 1], 
       Hue[0.7754090552434305, 1, 1], 
       Hue[0.67874465847557, 1, 1], 
       Hue[0.67874465847557, 1, 1], 
       Hue[0.582229927875765, 1, 1], 
       Hue[0.582229927875765, 1, 1], 
       Hue[0.4859598807852869, 1, 1], 
       Hue[0.4859598807852869, 1, 1], 
       Hue[0.38987575474619246`, 1, 1], 
       Hue[0.38987575474619246`, 1, 1], 
       Hue[0.2940365317975417, 1, 1], 
       Hue[0.2940365317975417, 1, 1], 
       Hue[0.19836684198245164`, 1, 1], 
       Hue[0.19836684198245164`, 1, 1], 
       Hue[0.10294494847248514`, 1, 1], 
       Hue[0.10294494847248514`, 1, 1], 
       Hue[0.007707782375784896, 1, 1], 
       Hue[0.007707782375784896, 1, 1], 
       Hue[0.9126637472741359, 1, 1], 
       Hue[0.9126637472741359, 1, 1], 
       Hue[0.8178222771749548, 1, 1], 
       Hue[0.8178222771749548, 1, 1], 
       Hue[0.72318127803347, 1, 1], 
       Hue[0.72318127803347, 1, 1], 
       Hue[0.6287338651278908, 1, 1], 
       Hue[0.6287338651278908, 1, 1], 
       Hue[0.5344701180581571, 1, 1], 
       Hue[0.5344701180581571, 1, 1], 
       Hue[0.44039426020930045`, 1, 1], 
       Hue[0.44039426020930045`, 1, 1], 
       Hue[0.3465144635884028, 1, 1], 
       Hue[0.3465144635884028, 1, 1], 
       Hue[0.252808072997363, 1, 1], 
       Hue[0.252808072997363, 1, 1], 
       Hue[0.15928966223813107`, 1, 1], 
       Hue[0.15928966223813107`, 1, 1], 
       Hue[0.06594733545312544, 1, 1], 
       Hue[0.06594733545312544, 1, 1], 
       Hue[0.9727848443406211, 1, 1], 
       Hue[0.9727848443406211, 1, 1], 
       Hue[0.8797973542288998, 1, 1], 
       Hue[0.8797973542288998, 1, 1], 
       Hue[0.7869821562937429, 1, 1], 
       Hue[0.7869821562937429, 1, 1], 
       Hue[0.6943399395807146, 1, 1], 
       Hue[0.6943399395807146, 1, 1], 
       Hue[0.6018559509399378, 1, 1], 
       Hue[0.6018559509399378, 1, 1], 
       Hue[0.5095393776462689, 1, 1], 
       Hue[0.5095393776462689, 1, 1], 
       Hue[0.4173796678168414, 1, 1], 
       Hue[0.4173796678168414, 1, 1], 
       Hue[0.32538385171789214`, 1, 1], 
       Hue[0.32538385171789214`, 1, 1], 
       Hue[0.23354205356651353`, 1, 1], 
       Hue[0.23354205356651353`, 1, 1], 
       Hue[0.14185471410378753`, 1, 1], 
       Hue[0.14185471410378753`, 1, 1], 
       Hue[0.05031447265582115, 1, 1], 
       Hue[0.05031447265582115, 1, 1], 
       Hue[0.958923662715511, 1, 1], 
       Hue[0.958923662715511, 1, 1], 
       Hue[0.8676772117323288, 1, 1], 
       Hue[0.8676772117323288, 1, 1], 
       Hue[0.7765680700686013, 1, 1], 
       Hue[0.7765680700686013, 1, 1], 
       Hue[0.6856019387109209, 1, 1], 
       Hue[0.6856019387109209, 1, 1], 
       Hue[0.5947691460253403, 1, 1], 
       Hue[0.5947691460253403, 1, 1], 
       Hue[0.5040652266289894, 1, 1], 
       Hue[0.5040652266289894, 1, 1], 
       Hue[0.4134896901706374, 1, 1], 
       Hue[0.4134896901706374, 1, 1], 
       Hue[0.3230371814924714, 1, 1], 
       Hue[0.3230371814924714, 1, 1], 
       Hue[0.2327048098212888, 1, 1], 
       Hue[0.2327048098212888, 1, 1], 
       Hue[0.14248935709303306`, 1, 1], 
       Hue[0.14248935709303306`, 1, 1], 
       Hue[0.05238415176542811, 1, 1], 
       Hue[0.05238415176542811, 1, 1], 
       Hue[0.9623891562027246, 1, 1], 
       Hue[0.9623891562027246, 1, 1], 
       Hue[0.8724970950084328, 1, 1], 
       Hue[0.8724970950084328, 1, 1], 
       Hue[0.782705234830449, 1, 1], 
       Hue[0.782705234830449, 1, 1], 
       Hue[0.6930094276719068, 1, 1], 
       Hue[0.6930094276719068, 1, 1], 
       Hue[0.603403558095029, 1, 1], 
       Hue[0.603403558095029, 1, 1], 
       Hue[0.5138848451851199, 1, 1], 
       Hue[0.5138848451851199, 1, 1], 
       Hue[0.4244454572392602, 1, 1], 
       Hue[0.4244454572392602, 1, 1], 
       Hue[0.3350842173811413, 1, 1], 
       Hue[0.3350842173811413, 1, 1], 
       Hue[0.24579166251417273`, 1, 1], 
       Hue[0.24579166251417273`, 1, 1], 
       Hue[0.15656623339073467`, 1, 1], 
       Hue[0.15656623339073467`, 1, 1], 
       Hue[0.06739962559988893, 1, 1], 
       Hue[0.06739962559988893, 1, 1], 
       Hue[0.9782875994973265, 1, 1], 
       Hue[0.9782875994973265, 1, 1], 
       Hue[0.8892234142617812, 1, 1], 
       Hue[0.8892234142617812, 1, 1], 
       Hue[0.8002011485261, 1, 1], 
       Hue[0.8002011485261, 1, 1], 
       Hue[0.7112140191474132, 1, 1], 
       Hue[0.7112140191474132, 1, 1], 
       Hue[0.6222542253030292, 1, 1], 
       Hue[0.6222542253030292, 1, 1], 
       Hue[0.5333177457842979, 1, 1], 
       Hue[0.5333177457842979, 1, 1], 
       Hue[0.44439506979053767`, 1, 1], 
       Hue[0.44439506979053767`, 1, 1], 
       Hue[0.3554794286460181, 1, 1], 
       Hue[0.3554794286460181, 1, 1], 
       Hue[0.26656299622171675`, 1, 1], 
       Hue[0.26656299622171675`, 1, 1], 
       Hue[0.17763793048842322`, 1, 1], 
       Hue[0.17763793048842322`, 1, 1], 
       Hue[0.08869610636692972, 1, 1], 
       Hue[0.08869610636692972, 1, 1], 
       Hue[0.9997283819955648, 1, 1], 
       Hue[0.9997283819955648, 1, 1], 
       Hue[0.9107265702305547, 1, 1], 
       Hue[0.9107265702305547, 1, 1], 
       Hue[0.821681754524422, 1, 1], 
       Hue[0.821681754524422, 1, 1], 
       Hue[0.7325848999670737, 1, 1], 
       Hue[0.7325848999670737, 1, 1], 
       Hue[0.6434256600246612, 1, 1], 
       Hue[0.6434256600246612, 1, 1], 
       Hue[0.5541954241832242, 1, 1], 
       Hue[0.5541954241832242, 1, 1], 
       Hue[0.46488295327061496`, 1, 1], 
       Hue[0.46488295327061496`, 1, 1], 
       Hue[0.3754802808713916, 1, 1], 
       Hue[0.3754802808713916, 1, 1], 
       Hue[0.28597636002231513`, 1, 1], 
       Hue[0.28597636002231513`, 1, 1], 
       Hue[0.1963625454614133, 1, 1], 
       Hue[0.1963625454614133, 1, 1], 
       Hue[0.10662868654545397`, 1, 1], 
       Hue[0.10662868654545397`, 1, 1], 
       Hue[0.016766218274584323`, 1, 1], 
       Hue[0.016766218274584323`, 1, 1], 
       Hue[0.9267666720232012, 1, 1], 
       Hue[0.9267666720232012, 1, 1], 
       Hue[0.8366209546576345, 1, 1], 
       Hue[0.8366209546576345, 1, 1], 
       Hue[0.7463234324270356, 1, 1], 
       Hue[0.7463234324270356, 1, 1], 
       Hue[0.6558683946811299, 1, 1], 
       Hue[0.6558683946811299, 1, 1], 
       Hue[0.5652519424005065, 1, 1], 
       Hue[0.5652519424005065, 1, 1], 
       Hue[0.4744728032182932, 1, 1], 
       Hue[0.4744728032182932, 1, 1], 
       Hue[0.38353075616650895`, 1, 1], 
       Hue[0.38353075616650895`, 1, 1], 
       Hue[0.29242972469312417`, 1, 1], 
       Hue[0.29242972469312417`, 1, 1], 
       Hue[0.20117765285064337`, 1, 1], 
       Hue[0.20117765285064337`, 1, 1], 
       Hue[0.1097862900027622, 1, 1], 
       Hue[0.1097862900027622, 1, 1], 
       Hue[0.01827287832674748, 1, 1], 
       Hue[0.01827287832674748, 1, 1], 
       Hue[0.9266611974571188, 1, 1], 
       Hue[0.9266611974571188, 1, 1], 
       Hue[0.8349800174043959, 1, 1], 
       Hue[0.8349800174043959, 1, 1], 
       Hue[0.7432670997335856, 1, 1], 
       Hue[0.7432670997335856, 1, 1], 
       Hue[0.6515686779597296, 1, 1], 
       Hue[0.6515686779597296, 1, 1], 
       Hue[0.5599424694431372, 1, 1], 
       Hue[0.5599424694431372, 1, 1], 
       Hue[0.46845481596904104`, 1, 1], 
       Hue[0.46845481596904104`, 1, 1], 
       Hue[0.3771862786362149, 1, 1], 
       Hue[0.3771862786362149, 1, 1], 
       Hue[0.2862282662686505, 1, 1], 
       Hue[0.2862282662686505, 1, 1], 
       Hue[0.19568641537022163`, 1, 1], 
       Hue[0.19568641537022163`, 1, 1], 
       Hue[0.10567797607262314`, 1, 1], 
       Hue[0.10567797607262314`, 1, 1], 
       Hue[0.016332731301606014`, 1, 1], 
       Hue[0.016332731301606014`, 1, 1], 
       Hue[0.927795591376938, 1, 1], 
       Hue[0.927795591376938, 1, 1], 
       Hue[0.8402276070843839, 1, 1], 
       Hue[0.8402276070843839, 1, 1], 
       Hue[0.7538035231069398, 1, 1], 
       Hue[0.7538035231069398, 1, 1], 
       Hue[0.6687130433992411, 1, 1], 
       Hue[0.6687130433992411, 1, 1], 
       Hue[0.5851522832589728, 1, 1], 
       Hue[0.5851522832589728, 1, 1], 
       Hue[0.5033241434569828, 1, 1], 
       Hue[0.5033241434569828, 1, 1], 
       Hue[0.42343048382940907`, 1, 1], 
       Hue[0.42343048382940907`, 1, 1], 
       Hue[0.3456705554356446, 1, 1], 
       Hue[0.3456705554356446, 1, 1], 
       Hue[0.27023803792139145`, 1, 1], 
       Hue[0.27023803792139145`, 1, 1], 
       Hue[0.19731503301541775`, 1, 1], 
       Hue[0.19731503301541775`, 1, 1], 
       Hue[0.1270771087857017, 1, 1], 
       Hue[0.1270771087857017, 1, 1], 
       Hue[0.05968324559803208, 1, 1], 
       Hue[0.05968324559803208, 1, 1], 
       Hue[0.9952806746680052, 1, 1], 
       Hue[0.9952806746680052, 1, 1], 
       Hue[0.9339888537835502, 1, 1], 
       Hue[0.9339888537835502, 1, 1], 
       Hue[0.8759098972504199, 1, 1], 
       Hue[0.8759098972504199, 1, 1], 
       Hue[0.8211168140301675, 1, 1], 
       Hue[0.8211168140301675, 1, 1], 
       Hue[0.769657595305148, 1, 1], 
       Hue[0.769657595305148, 1, 1], 
       Hue[0.72155306419796, 1, 1], 
       Hue[0.72155306419796, 1, 1], 
       Hue[0.6767989892674167, 1, 1], 
       Hue[0.6767989892674167, 1, 1], 
       Hue[0.6353668692228582, 1, 1], 
       Hue[0.6353668692228582, 1, 1], 
       Hue[0.5972087793115762, 1, 1], 
       Hue[0.5972087793115762, 1, 1], 
       Hue[0.5622604326917748, 1, 1], 
       Hue[0.5622604326917748, 1, 1], 
       Hue[0.5304403852817042, 1, 1], 
       Hue[0.5304403852817042, 1, 1], 
       Hue[0.501658519810958, 1, 1], 
       Hue[0.501658519810958, 1, 1], 
       Hue[0.47581545273971154`, 1, 1], 
       Hue[0.47581545273971154`, 1, 1], 
       Hue[0.45280660192802846`, 1, 1], 
       Hue[0.45280660192802846`, 1, 1], 
       Hue[0.4325244262555183, 1, 1], 
       Hue[0.4325244262555183, 1, 1], 
       Hue[0.4148607958877386, 1, 1], 
       Hue[0.4148607958877386, 1, 1], 
       Hue[0.3997064885717363, 1, 1], 
       Hue[0.3997064885717363, 1, 1], 
       Hue[0.3869563679568564, 1, 1], 
       Hue[0.3869563679568564, 1, 1], 
       Hue[0.37650730231757107`, 1, 1], 
       Hue[0.37650730231757107`, 1, 1], 
       Hue[0.3682607261039661, 1, 1], 
       Hue[0.3682607261039661, 1, 1], 
       Hue[0.3621209982291931, 1, 1], 
       Hue[0.3621209982291931, 1, 1], 
       Hue[0.3579978426514018, 1, 1], 
       Hue[0.3579978426514018, 1, 1], 
       Hue[0.35580683697359283`, 1, 1], 
       Hue[0.35580683697359283`, 1, 1], 
       Hue[0.3554662759685706, 1, 1], 
       Hue[0.3554662759685706, 1, 1], 
       Hue[0.3569013213821672, 1, 1], 
       Hue[0.3569013213821672, 1, 1], 
       Hue[0.36004081361650353`, 1, 1], 
       Hue[0.36004081361650353`, 1, 1], 
       Hue[0.3648181761077391, 1, 1], 
       Hue[0.3648181761077391, 1, 1], 
       Hue[0.37117149197985105`, 1, 1], 
       Hue[0.37117149197985105`, 1, 1], 
       Hue[0.37904337644688246`, 1, 1], 
       Hue[0.37904337644688246`, 1, 1], 
       Hue[0.38838003033941537`, 1, 1], 
       Hue[0.38838003033941537`, 1, 1], 
       Hue[0.39913105246117875`, 1, 1], 
       Hue[0.39913105246117875`, 1, 1], 
       Hue[0.411249920977416, 1, 1], 
       Hue[0.411249920977416, 1, 1], 
       Hue[0.424693391058465, 1, 1], 
       Hue[0.424693391058465, 1, 1], 
       Hue[0.4394211657571874, 1, 1], 
       Hue[0.4394211657571874, 1, 1], 
       Hue[0.45539537047986617`, 1, 1], 
       Hue[0.45539537047986617`, 1, 1], 
       Hue[0.47258130536614373`, 1, 1], 
       Hue[0.47258130536614373`, 1, 1], 
       Hue[0.4909467062963725, 1, 1], 
       Hue[0.4909467062963725, 1, 1], 
       Hue[0.5104604752212907, 1, 1], 
       Hue[0.5104604752212907, 1, 1], 
       Hue[0.5310943454416712, 1, 1], 
       Hue[0.5310943454416712, 1, 1], 
       Hue[0.552822658366199, 1, 1], 
       Hue[0.552822658366199, 1, 1], 
       Hue[0.5756200347081709, 1, 1], 
       Hue[0.5756200347081709, 1, 1], 
       Hue[0.5994628887166491, 1, 1], 
       Hue[0.5994628887166491, 1, 1], 
       Hue[0.6243296837994351, 1, 1], 
       Hue[0.6243296837994351, 1, 1], 
       Hue[0.6502004900681735, 1, 1], 
       Hue[0.6502004900681735, 1, 1], 
       Hue[0.6770556375018364, 1, 1], 
       Hue[0.6770556375018364, 1, 1], 
       Hue[0.7048773963672461, 1, 1], 
       Hue[0.7048773963672461, 1, 1], 
       Hue[0.7336477827191039, 1, 1], 
       Hue[0.7336477827191039, 1, 1], 
       Hue[0.7633516997074754, 1, 1], 
       Hue[0.7633516997074754, 1, 1], 
       Hue[0.7939737135007793, 1, 1], 
       Hue[0.7939737135007793, 1, 1], 
       Hue[0.8254995106024579, 1, 1], 
       Hue[0.8254995106024579, 1, 1], 
       Hue[0.8579156762552613, 1, 1], 
       Hue[0.8579156762552613, 1, 1], 
       Hue[0.8912095063230834, 1, 1], 
       Hue[0.8912095063230834, 1, 1], 
       Hue[0.925368717798873, 1, 1], 
       Hue[0.925368717798873, 1, 1], 
       Hue[0.9603824546877948, 1, 1], 
       Hue[0.9603824546877948, 1, 1], 
       Hue[0.9962385682294179, 1, 1], 
       Hue[0.9962385682294179, 1, 1], 
       Hue[0.03292654017297939, 1, 1], 
       Hue[0.03292654017297939, 1, 1], 
       Hue[0.07043714394267478, 1, 1], 
       Hue[0.07043714394267478, 1, 1], 
       Hue[0.10875968330459024`, 1, 1], 
       Hue[0.10875968330459024`, 1, 1], 
       Hue[0.14788561304282966`, 1, 1], 
       Hue[0.14788561304282966`, 1, 1], 
       Hue[0.18780615362640285`, 1, 1], 
       Hue[0.18780615362640285`, 1, 1], 
       Hue[0.22851270426524767`, 1, 1], 
       Hue[0.22851270426524767`, 1, 1], 
       Hue[0.26999801955043884`, 1, 1], 
       Hue[0.26999801955043884`, 1, 1], 
       Hue[0.3122539822073713, 1, 1], 
       Hue[0.3122539822073713, 1, 1], 
       Hue[0.35527287615958714`, 1, 1], 
       Hue[0.35527287615958714`, 1, 1], 
       Hue[0.39904799194802826`, 1, 1], 
       Hue[0.39904799194802826`, 1, 1], 
       Hue[0.44357149723895656`, 1, 1], 
       Hue[0.44357149723895656`, 1, 1], 
       Hue[0.48883781441948254`, 1, 1], 
       Hue[0.48883781441948254`, 1, 1], 
       Hue[0.534839279152306, 1, 1], 
       Hue[0.534839279152306, 1, 1], 
       Hue[0.5815702894260358, 1, 1], 
       Hue[0.5815702894260358, 1, 1], 
       Hue[0.6290256025850298, 1, 1], 
       Hue[0.6290256025850298, 1, 1], 
       Hue[0.6771988392967269, 1, 1], 
       Hue[0.6771988392967269, 1, 1], 
       Hue[0.7260852621317265, 1, 1], 
       Hue[0.7260852621317265, 1, 1], 
       Hue[0.7756785902064173, 1, 1], 
       Hue[0.7756785902064173, 1, 1], 
       Hue[0.8259737909039171, 1, 1], 
       Hue[0.8259737909039171, 1, 1], 
       Hue[0.8769657593370601, 1, 1], 
       Hue[0.8769657593370601, 1, 1], 
       Hue[0.9286486911989955, 1, 1], 
       Hue[0.9286486911989955, 1, 1], 
       Hue[0.9810188386237015, 1, 1], 
       Hue[0.9810188386237015, 1, 1], 
       Hue[0.034069971038059345`, 1, 1], 
       Hue[0.034069971038059345`, 1, 1], 
       Hue[0.08779934893459017, 1, 1], 
       Hue[0.08779934893459017, 1, 1], 
       Hue[0.1422018167527364, 1, 1], 
       Hue[0.1422018167527364, 1, 1], 
       Hue[0.19727378340379548`, 1, 1], 
       Hue[0.19727378340379548`, 1, 1], 
       Hue[0.2530108487712756, 1, 1], 
       Hue[0.2530108487712756, 1, 1], 
       Hue[0.30940900332769283`, 1, 1], 
       Hue[0.30940900332769283`, 1, 1], 
       Hue[0.3664635294786629, 1, 1], 
       Hue[0.3664635294786629, 1, 1], 
       Hue[0.42417153233580135`, 1, 1], 
       Hue[0.42417153233580135`, 1, 1], 
       Hue[0.48252852701740456`, 1, 1], 
       Hue[0.48252852701740456`, 1, 1], 
       Hue[0.5415311942928878, 1, 1], 
       Hue[0.5415311942928878, 1, 1], 
       Hue[0.6011771487708214, 1, 1], 
       Hue[0.6011771487708214, 1, 1], 
       Hue[0.6614611660479162, 1, 1], 
       Hue[0.6614611660479162, 1, 1], 
       Hue[0.7223807452582915, 1, 1], 
       Hue[0.7223807452582915, 1, 1], 
       Hue[0.7839319540579166, 1, 1], 
       Hue[0.7839319540579166, 1, 1], 
       Hue[0.8461120801534268, 1, 1], 
       Hue[0.8461120801534268, 1, 1], 
       Hue[0.9089177824574007, 1, 1], 
       Hue[0.9089177824574007, 1, 1], 
       Hue[0.972346258437172, 1, 1], 
       Hue[0.972346258437172, 1, 1], 
       Hue[0.03639461960368258, 1, 1], 
       Hue[0.03639461960368258, 1, 1], 
       Hue[0.10105942154813613`, 1, 1], 
       Hue[0.10105942154813613`, 1, 1], 
       Hue[0.16633766436514846`, 1, 1], 
       Hue[0.16633766436514846`, 1, 1], 
       Hue[0.23222719273173698`, 1, 1], 
       Hue[0.23222719273173698`, 1, 1], 
       Hue[0.2987238647155857, 1, 1], 
       Hue[0.2987238647155857, 1, 1], 
       Hue[0.36582744102738923`, 1, 1], 
       Hue[0.36582744102738923`, 1, 1], 
       Hue[0.4335325386500767, 1, 1], 
       Hue[0.4335325386500767, 1, 1], 
       Hue[0.5018379833859594, 1, 1], 
       Hue[0.5018379833859594, 1, 1], 
       Hue[0.5707408969998853, 1, 1], 
       Hue[0.5707408969998853, 1, 1], 
       Hue[0.6402401203183907, 1, 1], 
       Hue[0.6402401203183907, 1, 1], 
       Hue[0.7103314443312188, 1, 1], 
       Hue[0.7103314443312188, 1, 1], 
       Hue[0.7810129385835942, 1, 1], 
       Hue[0.7810129385835942, 1, 1], 
       Hue[0.8522840511817902, 1, 1], 
       Hue[0.8522840511817902, 1, 1], 
       Hue[0.9241392387208713, 1, 1], 
       Hue[0.9241392387208713, 1, 1], 
       Hue[0.9965798263202051, 1, 1], 
       Hue[0.9965798263202051, 1, 1], 
       Hue[0.06959900745981731, 1, 1], 
       Hue[0.06959900745981731, 1, 1], 
       Hue[0.14319934802733422`, 1, 1], 
       Hue[0.14319934802733422`, 1, 1], 
       Hue[0.21737807633544506`, 1, 1], 
       Hue[0.21737807633544506`, 1, 1], 
       Hue[0.2921281655178836, 1, 1], 
       Hue[0.2921281655178836, 1, 1], 
       Hue[0.3674526228331577, 1, 1], 
       Hue[0.3674526228331577, 1, 1], 
       Hue[0.4433487386760116, 1, 1], 
       Hue[0.4433487386760116, 1, 1], 
       Hue[0.519813479068898, 1, 1], 
       Hue[0.519813479068898, 1, 1], 
       Hue[0.5968450073967411, 1, 1], 
       Hue[0.5968450073967411, 1, 1], 
       Hue[0.6744391898025186, 1, 1], 
       Hue[0.6744391898025186, 1, 1], 
       Hue[0.7526026203570177, 1, 1], 
       Hue[0.7526026203570177, 1, 1], 
       Hue[0.8313227714345117, 1, 1], 
       Hue[0.8313227714345117, 1, 1], 
       Hue[0.9106020572940575, 1, 1], 
       Hue[0.9106020572940575, 1, 1], 
       Hue[0.99044294769886, 1, 1], 
       Hue[0.99044294769886, 1, 1], 
       Hue[0.07083722612697897, 1, 1], 
       Hue[0.07083722612697897, 1, 1], 
       Hue[0.15178796486556909`, 1, 1], 
       Hue[0.15178796486556909`, 1, 1], 
       Hue[0.2332917296675744, 1, 1]}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->{True, None},
    AxesOrigin->{0, 0},
    Epilog->
     LineBox[{{1, 0.00017331360906365425`}, {2, 0.0002004461808351119}, {
       3, 0.00023209713512794408`}, {4, 0.0002676601467031059}, {
       5, 0.0003087125258518049}, {6, 0.00035601856270682205`}, {
       7, 0.00040921890624941425`}, {8, 0.00047047823447631036`}, {
       9, 0.0005408591138593625}, {10, 0.0006191840054251076}, {
       11, 0.000709080914569019}, {12, 0.0008108019552094106}, {
       13, 0.0009260520917183954}, {14, 0.0010567383942408253`}, {
       15, 0.001203589144498999}, {16, 0.001369595381751855}, {
       17, 0.001556798677256322}, {18, 0.0017671588589336738`}, {
       19, 0.002003161644340483}, {20, 0.0022683286188256734`}, {
       21, 0.002564484755366951}, {22, 0.0028965581741394954`}, {
       23, 0.003266999043243304}, {24, 0.003679718150811597}, {
       25, 0.004139597446106104}, {26, 0.004651357662952773}, {
       27, 0.005219157882640485}, {28, 0.005848647155893587}, {
       29, 0.006544771804776567}, {30, 0.007314549033472592}, {
       31, 0.008163307664132327}, {32, 0.009097766833732104}, {
       33, 0.010125651977032576`}, {34, 0.011253800023434424`}, {
       35, 0.012490321227922572`}, {36, 0.013843050812300686`}, {
       37, 0.015320781611294472`}, {38, 0.016931734649757835`}, {
       39, 0.01868550424974008}, {40, 0.020590763001179613`}, {
       41, 0.022657460746511208`}, {42, 0.02489424843224105}, {
       43, 0.027309952530464643`}, {44, 0.029915507633075825`}, {
       45, 0.03271946707746229}, {46, 0.03573130775371684}, {
       47, 0.038960198487364}, {48, 0.042415244819097794`}, {
       49, 0.04610289409974477}, {50, 0.05003236483245365}, {
       51, 0.05420899154698012}, {52, 0.05863889792005716}, {
       53, 0.06332690652005153}, {54, 0.06827741625858946}, {
       55, 0.07349179029983526}, {56, 0.07897109342627537}, {
       57, 0.08471680264858389}, {58, 0.09072300813638602}, {
       59, 0.09698886835011952}, {60, 0.10350497014937741`}, {
       61, 0.11026774539193047`}, {62, 0.11726170125230698`}, {
       63, 0.12447599750403575`}, {64, 0.1318964919738219}, {
       65, 0.13950329194724798`}, {66, 0.14727665871607312`}, {
       67, 0.15519608126465115`}, {68, 0.1632344398253523}, {
       69, 0.1713644947197756}, {70, 0.17955757773348607`}, {
       71, 0.18778228063222607`}, {72, 0.1960031327336788}, {
       73, 0.20418488385356307`}, {74, 0.21228982250938633`}, {
       75, 0.22027943389973836`}, {76, 0.22811543342899146`}, {
       77, 0.23575713640247425`}, {78, 0.2431623336675769}, {
       79, 0.2502926032747923}, {80, 0.257107721944049}, {
       81, 0.26356920240138426`}, {82, 0.26964041115952253`}, {
       83, 0.27528476901325505`}, {84, 0.280471331611325}, {
       85, 0.2851677251561855}, {86, 0.28934997381540073`}, {
       87, 0.2929952468495493}, {88, 0.29608487661443056`}, {
       89, 0.2986072234382678}, {90, 0.30055166088767266`}, {
       91, 0.30191413901786507`}, {92, 0.30269958653023915`}, {
       93, 0.3029140568249175}, {94, 0.3025722637243245}, {
       95, 0.3016927811930327}, {96, 0.30029936131922935`}, {
       97, 0.298422595194389}, {98, 0.29609755507909635`}, {
       99, 0.2933624150108186}, {100, 0.29026302920979236`}, {
       101, 0.286845190950363}, {102, 0.2831569528517122}, {
       103, 0.2792543839598428}, {104, 0.27518591752285343`}, {
       105, 0.27100818902992324`}, {106, 0.26677321461853376`}, {
       107, 0.2625346332313765}, {108, 0.25834460634723033`}, {
       109, 0.2542509824968922}, {110, 0.2502986191492596}, {
       111, 0.2465274341001233}, {112, 0.24297202419410358`}, {
       113, 0.23966451598179678`}, {114, 0.23663128278311762`}, {
       115, 0.23390103667854123`}, {116, 0.23149503843117236`}, {
       117, 0.22943181659834055`}, {118, 0.227721992744214}, {
       119, 0.22636634281838655`}, {120, 0.22536101369420056`}, {
       121, 0.2246947834068738}, {122, 0.22436027860410757`}, {
       123, 0.22435101244766212`}, {124, 0.22466262388700653`}, {
       125, 0.22529472387529653`}, {126, 0.2262498670299155}, {
       127, 0.2275275484347227}, {128, 0.22912754265719926`}, {
       129, 0.23104560277962916`}, {130, 0.23327282372492836`}, {
       131, 0.23579644777173359`}, {132, 0.23860069202274067`}, {
       133, 0.24167043021890522`}, {134, 0.24498347104904492`}, {
       135, 0.24851775123571604`}, {136, 0.25225101398229677`}, {
       137, 0.2561585438676063}, {138, 0.26021802672829825`}, {
       139, 0.26440430659264025`}, {140, 0.2686946553889191}, {
       141, 0.27306451128652093`}, {142, 0.2774921983657016}, {
       143, 0.2819580316031532}, {144, 0.28644029650886804`}, {
       145, 0.2909186135018893}, {146, 0.2953764858811382}, {
       147, 0.2997955161814758}, {148, 0.3041572159402033}, {
       149, 0.3084482026663686}, {150, 0.31265286760456545`}, {
       151, 0.3167559555142252}, {152, 0.3207462080944464}, {
       153, 0.3246101344601126}, {154, 0.32833554355847333`}, {
       155, 0.33191168048336966`}, {156, 0.3353278337157109}, {
       157, 0.33857372031040317`}, {158, 0.34163980647816605`}, {
       159, 0.3445196047504053}, {160, 0.34720328429045594`}, {
       161, 0.34968315332650624`}, {162, 0.3519532090939946}, {
       163, 0.35400553299927073`}, {164, 0.355835932042}, {
       165, 0.35743740457042233`}, {166, 0.3588061600083007}, {
       167, 0.3599380251656484}, {168, 0.36082873894708223`}, {
       169, 0.3614752729586731}, {170, 0.3618740414578474}, {
       171, 0.36202326204878194`}, {172, 0.3619225665184547}, {
       173, 0.3615701460598568}, {174, 0.3609634715525623}, {
       175, 0.3601041589814839}, {176, 0.3589912735630775}, {
       177, 0.35762675577858566`}, {178, 0.3560114742371367}, {
       179, 0.35414883846456613`}, {180, 0.35204110725361115`}, {
       181, 0.3496927266882332}, {182, 0.34710523535039056`}, {
       183, 0.3442845086427027}, {184, 0.3412346921588113}, {
       185, 0.3379589979277237}, {186, 0.3344658081951963}, {
       187, 0.33075967688214025`}, {188, 0.32684660456093056`}, {
       189, 0.3227360764781115}, {190, 0.31843382693867706`}, {
       191, 0.31394905753557445`}, {192, 0.3092903483080025}, {
       193, 0.3044647642369086}, {194, 0.29948218731334375`}, {
       195, 0.29434880562639715`}, {196, 0.2890779807748132}, {
       197, 0.28367575492880553`}, {198, 0.2781530167818283}, {
       199, 0.2725223654171132}, {200, 0.26679207144047457`}, {
       201, 0.26097372002960206`}, {202, 0.2550758806462184}, {
       203, 0.24910940445230206`}, {204, 0.24308241575262116`}, {
       205, 0.23700763672712055`}, {206, 0.23089362972065583`}, {
       207, 0.22475210603935666`}, {208, 0.2185920309509046}, {
       209, 0.21242353315960877`}, {210, 0.20625598550913202`}, {
       211, 0.20009937456319532`}, {212, 0.19396179537919936`}, {
       213, 0.18785405181512924`}, {214, 0.18178284167940054`}, {
       215, 0.17575729087552489`}, {216, 0.16978546960300106`}, {
       217, 0.16387457420798704`}, {218, 0.15803288760853612`}, {
       219, 0.15226633476887902`}, {220, 0.14658267456394228`}, {
       221, 0.1409865044606825}, {222, 0.135484535537044}, {
       223, 0.13008182482847933`}, {224, 0.1247839027199716}, {
       225, 0.11959491469218655`}, {226, 0.11451929151160208`}, {
       227, 0.10955929120759636`}, {228, 0.10472080290491217`}, {
       229, 0.10000468295437415`}, {230, 0.09541473481728896}, {
       231, 0.09095112056823589}, {232, 0.08661814601089213}, {
       233, 0.08241452964810968}, {234, 0.07834296446027422}, {
       235, 0.07440297853840688}, {236, 0.07059524256318644}, {
       237, 0.06692074572861295}, {238, 0.06337686152312463}, {
       239, 0.059963828819883436`}, {240, 0.05668196958223771}, {
       241, 0.05352822472308272}, {242, 0.05050196948533514}, {
       243, 0.04760062289923075}, {244, 0.04482390245047077}, {
       245, 0.04216821657723882}, {246, 0.0396303648274698}, {
       247, 0.037210444023152885`}, {248, 0.034903687340650254`}, {
       249, 0.032707795639734534`}, {250, 0.030620143186840535`}, {
       251, 0.028637042566020763`}}],
    Frame->True,
    FrameTicks->{{{1., 
        FormBox[
         RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
        FormBox[
         RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
        FormBox[
         RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
        FormBox[
         RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
        FormBox["0", TraditionalForm]}, {157.25, 
        FormBox["2.5`", TraditionalForm]}, {188.5, 
        FormBox["5", TraditionalForm]}, {219.75, 
        FormBox["7.5`", TraditionalForm]}, {251., 
        FormBox["10", TraditionalForm]}}, Automatic, None, None},
    PlotRange->{-0.1, 1.1},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], Automatic}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF13lcVVW7B/DtlGZa5ICi2N04kPKiKYNYpvwUEWSeQcYNh+Ewg0pYmu5I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     "], {
      {GrayLevel[0.5], EdgeForm[None], 
       GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ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         "]]]}, {{}, {}, {
        {Hue[0.9890850417084717, 1, 1], LineBox[{1, 2}]}, 
        {Hue[0.890444563820459, 1, 1], LineBox[{2, 3}]}, 
        {Hue[0.7918350921023508, 1, 1], LineBox[{3, 4}]}, 
        {Hue[0.6935923344905128, 1, 1], LineBox[{4, 5}]}, 
        {Hue[0.5955973533557162, 1, 1], LineBox[{5, 6}]}, 
        {Hue[0.49769643430527716`, 1, 1], LineBox[{6, 7}]}, 
        {Hue[0.3999909708344182, 1, 1], LineBox[{7, 8}]}, 
        {Hue[0.30260995757912756`, 1, 1], LineBox[{8, 9}]}, 
        {Hue[0.20549048009062573`, 1, 1], LineBox[{9, 10}]}, 
        {Hue[0.10841445415142113`, 1, 1], LineBox[{10, 11}]}, 
        {Hue[0.011694343660073115`, 1, 1], LineBox[{11, 12}]}, 
        {Hue[0.915136789552362, 1, 1], LineBox[{12, 13}]}, 
        {Hue[0.8187862586312527, 1, 1], LineBox[{13, 14}]}, 
        {Hue[0.7226446234905665, 1, 1], LineBox[{14, 15}]}, 
        {Hue[0.6266956278780738, 1, 1], LineBox[{15, 16}]}, 
        {Hue[0.5309424388597916, 1, 1], LineBox[{16, 17}]}, 
        {Hue[0.4354159473821284, 1, 1], LineBox[{17, 18}]}, 
        {Hue[0.34006128509633016`, 1, 1], LineBox[{18, 19}]}, 
        {Hue[0.24488834017086, 1, 1], LineBox[{19, 20}]}, 
        {Hue[0.1499381665888046, 1, 1], LineBox[{20, 21}]}, 
        {Hue[0.0551497709061452, 1, 1], LineBox[{21, 22}]}, 
        {Hue[0.9605747897944017, 1, 1], LineBox[{22, 23}]}, 
        {Hue[0.8661822384266291, 1, 1], LineBox[{23, 24}]}, 
        {Hue[0.7719815454219947, 1, 1], LineBox[{24, 25}]}, 
        {Hue[0.6779589248969495, 1, 1], LineBox[{25, 26}]}, 
        {Hue[0.584107788669658, 1, 1], LineBox[{26, 27}]}, 
        {Hue[0.4904321659403038, 1, 1], LineBox[{27, 28}]}, 
        {Hue[0.3969391264652255, 1, 1], LineBox[{28, 29}]}, 
        {Hue[0.3036082892055693, 1, 1], LineBox[{29, 30}]}, 
        {Hue[0.21045143453790424`, 1, 1], LineBox[{30, 31}]}, 
        {Hue[0.1174679950897775, 1, 1], LineBox[{31, 32}]}, 
        {Hue[0.024647137886225627`, 1, 1], LineBox[{32, 33}]}, 
        {Hue[0.9319857820365661, 1, 1], LineBox[{33, 34}]}, 
        {Hue[0.8394897332935869, 1, 1], LineBox[{34, 35}]}, 
        {Hue[0.7471528779093941, 1, 1], LineBox[{35, 36}]}, 
        {Hue[0.6549682687227965, 1, 1], LineBox[{36, 37}]}, 
        {Hue[0.562940161123826, 1, 1], LineBox[{37, 38}]}, 
        {Hue[0.4710601350866123, 1, 1], LineBox[{38, 39}]}, 
        {Hue[0.379328026400307, 1, 1], LineBox[{39, 40}]}, 
        {Hue[0.2877415089286468, 1, 1], LineBox[{40, 41}]}, 
        {Hue[0.19629620846912263`, 1, 1], LineBox[{41, 42}]}, 
        {Hue[0.10498770191782866`, 1, 1], LineBox[{42, 43}]}, 
        {Hue[0.013818248266201809`, 1, 1], LineBox[{43, 44}]}, 
        {Hue[0.922779355202294, 1, 1], LineBox[{44, 45}]}, 
        {Hue[0.8318706546694346, 1, 1], LineBox[{45, 46}]}, 
        {Hue[0.7410886274394506, 1, 1], LineBox[{46, 47}]}, 
        {Hue[0.6504288919503963, 1, 1], LineBox[{47, 48}]}, 
        {Hue[0.5598898317429462, 1, 1], LineBox[{48, 49}]}, 
        {Hue[0.46946392923718244`, 1, 1], LineBox[{49, 50}]}, 
        {Hue[0.3791530821283772, 1, 1], LineBox[{50, 51}]}, 
        {Hue[0.2889467711448164, 1, 1], LineBox[{51, 52}]}, 
        {Hue[0.19884605935101357`, 1, 1], LineBox[{52, 53}]}, 
        {Hue[0.10884451371828528`, 1, 1], LineBox[{53, 54}]}, 
        {Hue[0.01893840191355155, 1, 1], LineBox[{54, 55}]}, 
        {Hue[0.9291246203627189, 1, 1], LineBox[{55, 56}]}, 
        {Hue[0.839398027108012, 1, 1], LineBox[{56, 57}]}, 
        {Hue[0.7497518597451729, 1, 1], LineBox[{57, 58}]}, 
        {Hue[0.6601838746800621, 1, 1], LineBox[{58, 59}]}, 
        {Hue[0.5706881943866222, 1, 1], LineBox[{59, 60}]}, 
        {Hue[0.48125901280784156`, 1, 1], LineBox[{60, 61}]}, 
        {Hue[0.39189064851607397`, 1, 1], LineBox[{61, 62}]}, 
        {Hue[0.30257891317312197`, 1, 1], LineBox[{62, 63}]}, 
        {Hue[0.21331627749924545`, 1, 1], LineBox[{63, 64}]}, 
        {Hue[0.1240969762593327, 1, 1], LineBox[{64, 65}]}, 
        {Hue[0.034915561215019375`, 1, 1], LineBox[{65, 66}]}, 
        {Hue[0.9457641347466291, 1, 1], LineBox[{66, 67}]}, 
        {Hue[0.8566369738272935, 1, 1], LineBox[{67, 68}]}, 
        {Hue[0.7675262150857177, 1, 1], LineBox[{68, 69}]}, 
        {Hue[0.6784238011201688, 1, 1], LineBox[{69, 70}]}, 
        {Hue[0.5893230207040819, 1, 1], LineBox[{70, 71}]}, 
        {Hue[0.50021495606295, 1, 1], LineBox[{71, 72}]}, 
        {Hue[0.4110913775684948, 1, 1], LineBox[{72, 73}]}, 
        {Hue[0.3219424614705361, 1, 1], LineBox[{73, 74}]}, 
        {Hue[0.23275925793759258`, 1, 1], LineBox[{74, 75}]}, 
        {Hue[0.143532350562769, 1, 1], LineBox[{75, 76}]}, 
        {Hue[0.05425179677263266, 1, 1], LineBox[{76, 77}]}, 
        {Hue[0.9649067193442054, 1, 1], LineBox[{77, 78}]}, 
        {Hue[0.8754870852363708, 1, 1], LineBox[{78, 79}]}, 
        {Hue[0.7859800701598657, 1, 1], LineBox[{79, 80}]}, 
        {Hue[0.6963751868911885, 1, 1], LineBox[{80, 81}]}, 
        {Hue[0.6066591090207734, 1, 1], LineBox[{81, 82}]}, 
        {Hue[0.5168198134156986, 1, 1], LineBox[{82, 83}]}, 
        {Hue[0.42684569537233746`, 1, 1], LineBox[{83, 84}]}, 
        {Hue[0.33672288201474887`, 1, 1], LineBox[{84, 85}]}, 
        {Hue[0.24643890665017157`, 1, 1], LineBox[{85, 86}]}, 
        {Hue[0.1559790483703322, 1, 1], LineBox[{86, 87}]}, 
        {Hue[0.06533250362229022, 1, 1], LineBox[{87, 88}]}, 
        {Hue[0.9744852437056882, 1, 1], LineBox[{88, 89}]}, 
        {Hue[0.8834255839195694, 1, 1], LineBox[{89, 90}]}, 
        {Hue[0.7921421823685508, 1, 1], LineBox[{90, 91}]}, 
        {Hue[0.7006241285792536, 1, 1], LineBox[{91, 92}]}, 
        {Hue[0.6088632573112414, 1, 1], LineBox[{92, 93}]}, 
        {Hue[0.5168507348555242, 1, 1], LineBox[{93, 94}]}, 
        {Hue[0.42458376599519027`, 1, 1], LineBox[{94, 95}]}, 
        {Hue[0.33205862577194645`, 1, 1], LineBox[{95, 96}]}, 
        {Hue[0.2392775083989476, 1, 1], LineBox[{96, 97}]}, 
        {Hue[0.14624478418476983`, 1, 1], LineBox[{97, 98}]}, 
        {Hue[0.052970346936419506`, 1, 1], LineBox[{98, 99}]}, 
        {Hue[0.9594676360426614, 1, 1], LineBox[{99, 100}]}, 
        {Hue[0.8657577363329141, 1, 1], LineBox[{100, 101}]}, 
        {Hue[0.7718665933810455, 1, 1], LineBox[{101, 102}]}, 
        {Hue[0.6778270601834724, 1, 1], LineBox[{102, 103}]}, 
        {Hue[0.5836773317911002, 1, 1], LineBox[{103, 104}]}, 
        {Hue[0.48946425451082115`, 1, 1], LineBox[{104, 105}]}, 
        {Hue[0.3952418144069382, 1, 1], LineBox[{105, 106}]}, 
        {Hue[0.3010708889653632, 1, 1], LineBox[{106, 107}]}, 
        {Hue[0.207018848047681, 1, 1], LineBox[{107, 108}]}, 
        {Hue[0.11316324104925135`, 1, 1], LineBox[{108, 109}]}, 
        {Hue[0.019585516746522633`, 1, 1], LineBox[{109, 110}]}, 
        {Hue[0.9263748173191635, 1, 1], LineBox[{110, 111}]}, 
        {Hue[0.8336288784715873, 1, 1], LineBox[{111, 112}]}, 
        {Hue[0.7414537703560893, 1, 1], LineBox[{112, 113}]}, 
        {Hue[0.6499644952350254, 1, 1], LineBox[{113, 114}]}, 
        {Hue[0.559286572819959, 1, 1], LineBox[{114, 115}]}, 
        {Hue[0.46955356366737094`, 1, 1], LineBox[{115, 116}]}, 
        {Hue[0.38090894736594, 1, 1], LineBox[{116, 117}]}, 
        {Hue[0.2935085010777411, 1, 1], LineBox[{117, 118}]}, 
        {Hue[0.20750998380319777`, 1, 1], LineBox[{118, 119}]}, 
        {Hue[0.12308448701168116`, 1, 1], LineBox[{119, 120}]}, 
        {Hue[0.040405866180151344`, 1, 1], LineBox[{120, 121}]}, 
        {Hue[0.9596577933041008, 1, 1], LineBox[{121, 122}]}, 
        {Hue[0.8810254867376445, 1, 1], LineBox[{122, 123}]}, 
        {Hue[0.8046989087260993, 1, 1], LineBox[{123, 124}]}, 
        {Hue[0.7308629301685772, 1, 1], LineBox[{124, 125}]}, 
        {Hue[0.659694095351627, 1, 1], LineBox[{125, 126}]}, 
        {Hue[0.591355858433406, 1, 1], LineBox[{126, 127}]}, 
        {Hue[0.5259960179322813, 1, 1], LineBox[{127, 128}]}, 
        {Hue[0.4637384836446359, 1, 1], LineBox[{128, 129}]}, 
        {Hue[0.4046827309417128, 1, 1], LineBox[{129, 130}]}, 
        {Hue[0.34890027740507623`, 1, 1], LineBox[{130, 131}]}, 
        {Hue[0.2964351349133147, 1, 1], LineBox[{131, 132}]}, 
        {Hue[0.24730822874130168`, 1, 1], LineBox[{132, 133}]}, 
        {Hue[0.20151138082108272`, 1, 1], LineBox[{133, 134}]}, 
        {Hue[0.15901329786186943`, 1, 1], LineBox[{134, 135}]}, 
        {Hue[0.1197649271403978, 1, 1], LineBox[{135, 136}]}, 
        {Hue[0.08369802893832214, 1, 1], LineBox[{136, 137}]}, 
        {Hue[0.05073030875715411, 1, 1], LineBox[{137, 138}]}, 
        {Hue[0.020769647506886663`, 1, 1], LineBox[{138, 139}]}, 
        {Hue[0.9937147157833544, 1, 1], LineBox[{139, 140}]}, 
        {Hue[0.9694593009365832, 1, 1], LineBox[{140, 141}]}, 
        {Hue[0.9478941084618844, 1, 1], LineBox[{141, 142}]}, 
        {Hue[0.9289103885338673, 1, 1], LineBox[{142, 143}]}, 
        {Hue[0.9123988367658288, 1, 1], LineBox[{143, 144}]}, 
        {Hue[0.8982520275469468, 1, 1], LineBox[{144, 145}]}, 
        {Hue[0.886366658292858, 1, 1], LineBox[{145, 146}]}, 
        {Hue[0.8766426050628463, 1, 1], LineBox[{146, 147}]}, 
        {Hue[0.8689841847218216, 1, 1], LineBox[{147, 148}]}, 
        {Hue[0.8633013381767353, 1, 1], LineBox[{148, 149}]}, 
        {Hue[0.8595080716578948, 1, 1], LineBox[{149, 150}]}, 
        {Hue[0.8575234682582267, 1, 1], LineBox[{150, 151}]}, 
        {Hue[0.8572726383404587, 1, 1], LineBox[{151, 152}]}, 
        {Hue[0.8586834192166569, 1, 1], LineBox[{152, 153}]}, 
        {Hue[0.8616906928539774, 1, 1], LineBox[{153, 154}]}, 
        {Hue[0.8662315826883799, 1, 1], LineBox[{154, 155}]}, 
        {Hue[0.8722494769006455, 1, 1], LineBox[{155, 156}]}, 
        {Hue[0.8796904511863467, 1, 1], LineBox[{156, 157}]}, 
        {Hue[0.8885044010740425, 1, 1], LineBox[{157, 158}]}, 
        {Hue[0.8986452227927522, 1, 1], LineBox[{158, 159}]}, 
        {Hue[0.9100709017328726, 1, 1], LineBox[{159, 160}]}, 
        {Hue[0.922740508081968, 1, 1], LineBox[{160, 161}]}, 
        {Hue[0.9366170036872785, 1, 1], LineBox[{161, 162}]}, 
        {Hue[0.9516664083103602, 1, 1], LineBox[{162, 163}]}, 
        {Hue[0.9678565539344235, 1, 1], LineBox[{163, 164}]}, 
        {Hue[0.9851573817997091, 1, 1], LineBox[{164, 165}]}, 
        {Hue[0.003541029282514706, 1, 1], LineBox[{165, 166}]}, 
        {Hue[0.022982276990593284`, 1, 1], LineBox[{166, 167}]}, 
        {Hue[0.04345618989206919, 1, 1], LineBox[{167, 168}]}, 
        {Hue[0.06494019180123378, 1, 1], LineBox[{168, 169}]}, 
        {Hue[0.0874131980234504, 1, 1], LineBox[{169, 170}]}, 
        {Hue[0.11085512776100503`, 1, 1], LineBox[{170, 171}]}, 
        {Hue[0.1352477810807344, 1, 1], LineBox[{171, 172}]}, 
        {Hue[0.16057339342287885`, 1, 1], LineBox[{172, 173}]}, 
        {Hue[0.18681539630932423`, 1, 1], LineBox[{173, 174}]}, 
        {Hue[0.21395846771373908`, 1, 1], LineBox[{174, 175}]}, 
        {Hue[0.24198818379898965`, 1, 1], LineBox[{175, 176}]}, 
        {Hue[0.2708910987900743, 1, 1], LineBox[{176, 177}]}, 
        {Hue[0.3006539059544776, 1, 1], LineBox[{177, 178}]}, 
        {Hue[0.33126453314316606`, 1, 1], LineBox[{178, 179}]}, 
        {Hue[0.3627118699332237, 1, 1], LineBox[{179, 180}]}, 
        {Hue[0.3949837005086097, 1, 1], LineBox[{180, 181}]}, 
        {Hue[0.42806988647193905`, 1, 1], LineBox[{181, 182}]}, 
        {Hue[0.4619600062641948, 1, 1], LineBox[{182, 183}]}, 
        {Hue[0.4966450931302264, 1, 1], LineBox[{183, 184}]}, 
        {Hue[0.5321158935007508, 1, 1], LineBox[{184, 185}]}, 
        {Hue[0.5683632217963384, 1, 1], LineBox[{185, 186}]}, 
        {Hue[0.6053791741127653, 1, 1], LineBox[{186, 187}]}, 
        {Hue[0.6431566313515203, 1, 1], LineBox[{187, 188}]}, 
        {Hue[0.6816868752353401, 1, 1], LineBox[{188, 189}]}, 
        {Hue[0.72096364429756, 1, 1], LineBox[{189, 190}]}, 
        {Hue[0.7609790655744311, 1, 1], LineBox[{190, 191}]}, 
        {Hue[0.801726159467002, 1, 1], LineBox[{191, 192}]}, 
        {Hue[0.8431986948347605, 1, 1], LineBox[{192, 193}]}, 
        {Hue[0.8853905620106385, 1, 1], LineBox[{193, 194}]}, 
        {Hue[0.928294741191026, 1, 1], LineBox[{194, 195}]}, 
        {Hue[0.9719066681331644, 1, 1], LineBox[{195, 196}]}, 
        {Hue[0.016219788090091947`, 1, 1], LineBox[{196, 197}]}, 
        {Hue[0.06123023095704857, 1, 1], LineBox[{197, 198}]}, 
        {Hue[0.10693189420594468`, 1, 1], LineBox[{198, 199}]}, 
        {Hue[0.15331972133290422`, 1, 1], LineBox[{199, 200}]}, 
        {Hue[0.20038888796819257`, 1, 1], LineBox[{200, 201}]}, 
        {Hue[0.24813299763561023`, 1, 1], LineBox[{201, 202}]}, 
        {Hue[0.29654916208262616`, 1, 1], LineBox[{202, 203}]}, 
        {Hue[0.3456310281848976, 1, 1], LineBox[{203, 204}]}, 
        {Hue[0.3953752971354173, 1, 1], LineBox[{204, 205}]}, 
        {Hue[0.4457779494411485, 1, 1], LineBox[{205, 206}]}, 
        {Hue[0.49683472538993373`, 1, 1], LineBox[{206, 207}]}, 
        {Hue[0.5485422000086002, 1, 1], LineBox[{207, 208}]}, 
        {Hue[0.6008954329863241, 1, 1], LineBox[{208, 209}]}, 
        {Hue[0.6538909685908945, 1, 1], LineBox[{209, 210}]}, 
        {Hue[0.7075250668459268, 1, 1], LineBox[{210, 211}]}, 
        {Hue[0.7617944518713861, 1, 1], LineBox[{211, 212}]}, 
        {Hue[0.8166949979465602, 1, 1], LineBox[{212, 213}]}, 
        {Hue[0.872223828978173, 1, 1], LineBox[{213, 214}]}, 
        {Hue[0.9283771405632653, 1, 1], LineBox[{214, 215}]}, 
        {Hue[0.9851524220111436, 1, 1], LineBox[{215, 216}]}, 
        {Hue[0.04254550702833396, 1, 1], LineBox[{216, 217}]}, 
        {Hue[0.10055367721831523`, 1, 1], LineBox[{217, 218}]}, 
        {Hue[0.15917356261530025`, 1, 1], LineBox[{218, 219}]}, 
        {Hue[0.21840264135365023`, 1, 1], LineBox[{219, 220}]}, 
        {Hue[0.2782378071599018, 1, 1], LineBox[{220, 221}]}, 
        {Hue[0.3386767001197204, 1, 1], LineBox[{221, 222}]}, 
        {Hue[0.39971523956125754`, 1, 1], LineBox[{222, 223}]}, 
        {Hue[0.46135214668637253`, 1, 1], LineBox[{223, 224}]}, 
        {Hue[0.5235833706306552, 1, 1], LineBox[{224, 225}]}, 
        {Hue[0.5864068661836146, 1, 1], LineBox[{225, 226}]}, 
        {Hue[0.649821296542562, 1, 1], LineBox[{226, 227}]}, 
        {Hue[0.7138220064371237, 1, 1], LineBox[{227, 228}]}, 
        {Hue[0.7784087549190002, 1, 1], LineBox[{228, 229}]}, 
        {Hue[0.8435773141078866, 1, 1], LineBox[{229, 230}]}, 
        {Hue[0.9093257782873416, 1, 1], LineBox[{230, 231}]}, 
        {Hue[0.9756515583544458, 1, 1], LineBox[{231, 232}]}, 
        {Hue[0.04255353459618736, 1, 1], LineBox[{232, 233}]}, 
        {Hue[0.11002894234378277`, 1, 1], LineBox[{233, 234}]}, 
        {Hue[0.17807576974946937`, 1, 1], LineBox[{234, 235}]}, 
        {Hue[0.2466904245232185, 1, 1], LineBox[{235, 236}]}, 
        {Hue[0.3158733471799709, 1, 1], LineBox[{236, 237}]}, 
        {Hue[0.3856187575594051, 1, 1], LineBox[{237, 238}]}, 
        {Hue[0.4559291256723374, 1, 1], LineBox[{238, 239}]}, 
        {Hue[0.5267985007359997, 1, 1], LineBox[{239, 240}]}, 
        {Hue[0.5982288136813674, 1, 1], LineBox[{240, 241}]}, 
        {Hue[0.6702147558152386, 1, 1], LineBox[{241, 242}]}, 
        {Hue[0.7427566853759748, 1, 1], LineBox[{242, 243}]}, 
        {Hue[0.8158503838879019, 1, 1], LineBox[{243, 244}]}, 
        {Hue[0.8894967525160438, 1, 1], LineBox[{244, 245}]}, 
        {Hue[0.9636945233767934, 1, 1], LineBox[{245, 246}]}, 
        {Hue[0.03843758514675253, 1, 1], LineBox[{246, 247}]}, 
        {Hue[0.11372770534876697`, 1, 1], LineBox[{247, 248}]}, 
        {Hue[0.1895624517637856, 1, 1], LineBox[{248, 249}]}, 
        {Hue[0.26593871306289235`, 1, 1], LineBox[{249, 250}]}, 
        {Hue[0.34285896982386044`, 1, 1], LineBox[{250, 251}]}}}},
     VertexColors->{
       Hue[0.9890850417084717, 1, 1], 
       Hue[0.890444563820459, 1, 1], 
       Hue[0.7918350921023508, 1, 1], 
       Hue[0.6935923344905128, 1, 1], 
       Hue[0.5955973533557162, 1, 1], 
       Hue[0.49769643430527716`, 1, 1], 
       Hue[0.3999909708344182, 1, 1], 
       Hue[0.30260995757912756`, 1, 1], 
       Hue[0.20549048009062573`, 1, 1], 
       Hue[0.10841445415142113`, 1, 1], 
       Hue[0.011694343660073115`, 1, 1], 
       Hue[0.915136789552362, 1, 1], 
       Hue[0.8187862586312527, 1, 1], 
       Hue[0.7226446234905665, 1, 1], 
       Hue[0.6266956278780738, 1, 1], 
       Hue[0.5309424388597916, 1, 1], 
       Hue[0.4354159473821284, 1, 1], 
       Hue[0.34006128509633016`, 1, 1], 
       Hue[0.24488834017086, 1, 1], 
       Hue[0.1499381665888046, 1, 1], 
       Hue[0.0551497709061452, 1, 1], 
       Hue[0.9605747897944017, 1, 1], 
       Hue[0.8661822384266291, 1, 1], 
       Hue[0.7719815454219947, 1, 1], 
       Hue[0.6779589248969495, 1, 1], 
       Hue[0.584107788669658, 1, 1], 
       Hue[0.4904321659403038, 1, 1], 
       Hue[0.3969391264652255, 1, 1], 
       Hue[0.3036082892055693, 1, 1], 
       Hue[0.21045143453790424`, 1, 1], 
       Hue[0.1174679950897775, 1, 1], 
       Hue[0.024647137886225627`, 1, 1], 
       Hue[0.9319857820365661, 1, 1], 
       Hue[0.8394897332935869, 1, 1], 
       Hue[0.7471528779093941, 1, 1], 
       Hue[0.6549682687227965, 1, 1], 
       Hue[0.562940161123826, 1, 1], 
       Hue[0.4710601350866123, 1, 1], 
       Hue[0.379328026400307, 1, 1], 
       Hue[0.2877415089286468, 1, 1], 
       Hue[0.19629620846912263`, 1, 1], 
       Hue[0.10498770191782866`, 1, 1], 
       Hue[0.013818248266201809`, 1, 1], 
       Hue[0.922779355202294, 1, 1], 
       Hue[0.8318706546694346, 1, 1], 
       Hue[0.7410886274394506, 1, 1], 
       Hue[0.6504288919503963, 1, 1], 
       Hue[0.5598898317429462, 1, 1], 
       Hue[0.46946392923718244`, 1, 1], 
       Hue[0.3791530821283772, 1, 1], 
       Hue[0.2889467711448164, 1, 1], 
       Hue[0.19884605935101357`, 1, 1], 
       Hue[0.10884451371828528`, 1, 1], 
       Hue[0.01893840191355155, 1, 1], 
       Hue[0.9291246203627189, 1, 1], 
       Hue[0.839398027108012, 1, 1], 
       Hue[0.7497518597451729, 1, 1], 
       Hue[0.6601838746800621, 1, 1], 
       Hue[0.5706881943866222, 1, 1], 
       Hue[0.48125901280784156`, 1, 1], 
       Hue[0.39189064851607397`, 1, 1], 
       Hue[0.30257891317312197`, 1, 1], 
       Hue[0.21331627749924545`, 1, 1], 
       Hue[0.1240969762593327, 1, 1], 
       Hue[0.034915561215019375`, 1, 1], 
       Hue[0.9457641347466291, 1, 1], 
       Hue[0.8566369738272935, 1, 1], 
       Hue[0.7675262150857177, 1, 1], 
       Hue[0.6784238011201688, 1, 1], 
       Hue[0.5893230207040819, 1, 1], 
       Hue[0.50021495606295, 1, 1], 
       Hue[0.4110913775684948, 1, 1], 
       Hue[0.3219424614705361, 1, 1], 
       Hue[0.23275925793759258`, 1, 1], 
       Hue[0.143532350562769, 1, 1], 
       Hue[0.05425179677263266, 1, 1], 
       Hue[0.9649067193442054, 1, 1], 
       Hue[0.8754870852363708, 1, 1], 
       Hue[0.7859800701598657, 1, 1], 
       Hue[0.6963751868911885, 1, 1], 
       Hue[0.6066591090207734, 1, 1], 
       Hue[0.5168198134156986, 1, 1], 
       Hue[0.42684569537233746`, 1, 1], 
       Hue[0.33672288201474887`, 1, 1], 
       Hue[0.24643890665017157`, 1, 1], 
       Hue[0.1559790483703322, 1, 1], 
       Hue[0.06533250362229022, 1, 1], 
       Hue[0.9744852437056882, 1, 1], 
       Hue[0.8834255839195694, 1, 1], 
       Hue[0.7921421823685508, 1, 1], 
       Hue[0.7006241285792536, 1, 1], 
       Hue[0.6088632573112414, 1, 1], 
       Hue[0.5168507348555242, 1, 1], 
       Hue[0.42458376599519027`, 1, 1], 
       Hue[0.33205862577194645`, 1, 1], 
       Hue[0.2392775083989476, 1, 1], 
       Hue[0.14624478418476983`, 1, 1], 
       Hue[0.052970346936419506`, 1, 1], 
       Hue[0.9594676360426614, 1, 1], 
       Hue[0.8657577363329141, 1, 1], 
       Hue[0.7718665933810455, 1, 1], 
       Hue[0.6778270601834724, 1, 1], 
       Hue[0.5836773317911002, 1, 1], 
       Hue[0.48946425451082115`, 1, 1], 
       Hue[0.3952418144069382, 1, 1], 
       Hue[0.3010708889653632, 1, 1], 
       Hue[0.207018848047681, 1, 1], 
       Hue[0.11316324104925135`, 1, 1], 
       Hue[0.019585516746522633`, 1, 1], 
       Hue[0.9263748173191635, 1, 1], 
       Hue[0.8336288784715873, 1, 1], 
       Hue[0.7414537703560893, 1, 1], 
       Hue[0.6499644952350254, 1, 1], 
       Hue[0.559286572819959, 1, 1], 
       Hue[0.46955356366737094`, 1, 1], 
       Hue[0.38090894736594, 1, 1], 
       Hue[0.2935085010777411, 1, 1], 
       Hue[0.20750998380319777`, 1, 1], 
       Hue[0.12308448701168116`, 1, 1], 
       Hue[0.040405866180151344`, 1, 1], 
       Hue[0.9596577933041008, 1, 1], 
       Hue[0.8810254867376445, 1, 1], 
       Hue[0.8046989087260993, 1, 1], 
       Hue[0.7308629301685772, 1, 1], 
       Hue[0.659694095351627, 1, 1], 
       Hue[0.591355858433406, 1, 1], 
       Hue[0.5259960179322813, 1, 1], 
       Hue[0.4637384836446359, 1, 1], 
       Hue[0.4046827309417128, 1, 1], 
       Hue[0.34890027740507623`, 1, 1], 
       Hue[0.2964351349133147, 1, 1], 
       Hue[0.24730822874130168`, 1, 1], 
       Hue[0.20151138082108272`, 1, 1], 
       Hue[0.15901329786186943`, 1, 1], 
       Hue[0.1197649271403978, 1, 1], 
       Hue[0.08369802893832214, 1, 1], 
       Hue[0.05073030875715411, 1, 1], 
       Hue[0.020769647506886663`, 1, 1], 
       Hue[0.9937147157833544, 1, 1], 
       Hue[0.9694593009365832, 1, 1], 
       Hue[0.9478941084618844, 1, 1], 
       Hue[0.9289103885338673, 1, 1], 
       Hue[0.9123988367658288, 1, 1], 
       Hue[0.8982520275469468, 1, 1], 
       Hue[0.886366658292858, 1, 1], 
       Hue[0.8766426050628463, 1, 1], 
       Hue[0.8689841847218216, 1, 1], 
       Hue[0.8633013381767353, 1, 1], 
       Hue[0.8595080716578948, 1, 1], 
       Hue[0.8575234682582267, 1, 1], 
       Hue[0.8572726383404587, 1, 1], 
       Hue[0.8586834192166569, 1, 1], 
       Hue[0.8616906928539774, 1, 1], 
       Hue[0.8662315826883799, 1, 1], 
       Hue[0.8722494769006455, 1, 1], 
       Hue[0.8796904511863467, 1, 1], 
       Hue[0.8885044010740425, 1, 1], 
       Hue[0.8986452227927522, 1, 1], 
       Hue[0.9100709017328726, 1, 1], 
       Hue[0.922740508081968, 1, 1], 
       Hue[0.9366170036872785, 1, 1], 
       Hue[0.9516664083103602, 1, 1], 
       Hue[0.9678565539344235, 1, 1], 
       Hue[0.9851573817997091, 1, 1], 
       Hue[0.003541029282514706, 1, 1], 
       Hue[0.022982276990593284`, 1, 1], 
       Hue[0.04345618989206919, 1, 1], 
       Hue[0.06494019180123378, 1, 1], 
       Hue[0.0874131980234504, 1, 1], 
       Hue[0.11085512776100503`, 1, 1], 
       Hue[0.1352477810807344, 1, 1], 
       Hue[0.16057339342287885`, 1, 1], 
       Hue[0.18681539630932423`, 1, 1], 
       Hue[0.21395846771373908`, 1, 1], 
       Hue[0.24198818379898965`, 1, 1], 
       Hue[0.2708910987900743, 1, 1], 
       Hue[0.3006539059544776, 1, 1], 
       Hue[0.33126453314316606`, 1, 1], 
       Hue[0.3627118699332237, 1, 1], 
       Hue[0.3949837005086097, 1, 1], 
       Hue[0.42806988647193905`, 1, 1], 
       Hue[0.4619600062641948, 1, 1], 
       Hue[0.4966450931302264, 1, 1], 
       Hue[0.5321158935007508, 1, 1], 
       Hue[0.5683632217963384, 1, 1], 
       Hue[0.6053791741127653, 1, 1], 
       Hue[0.6431566313515203, 1, 1], 
       Hue[0.6816868752353401, 1, 1], 
       Hue[0.72096364429756, 1, 1], 
       Hue[0.7609790655744311, 1, 1], 
       Hue[0.801726159467002, 1, 1], 
       Hue[0.8431986948347605, 1, 1], 
       Hue[0.8853905620106385, 1, 1], 
       Hue[0.928294741191026, 1, 1], 
       Hue[0.9719066681331644, 1, 1], 
       Hue[0.016219788090091947`, 1, 1], 
       Hue[0.06123023095704857, 1, 1], 
       Hue[0.10693189420594468`, 1, 1], 
       Hue[0.15331972133290422`, 1, 1], 
       Hue[0.20038888796819257`, 1, 1], 
       Hue[0.24813299763561023`, 1, 1], 
       Hue[0.29654916208262616`, 1, 1], 
       Hue[0.3456310281848976, 1, 1], 
       Hue[0.3953752971354173, 1, 1], 
       Hue[0.4457779494411485, 1, 1], 
       Hue[0.49683472538993373`, 1, 1], 
       Hue[0.5485422000086002, 1, 1], 
       Hue[0.6008954329863241, 1, 1], 
       Hue[0.6538909685908945, 1, 1], 
       Hue[0.7075250668459268, 1, 1], 
       Hue[0.7617944518713861, 1, 1], 
       Hue[0.8166949979465602, 1, 1], 
       Hue[0.872223828978173, 1, 1], 
       Hue[0.9283771405632653, 1, 1], 
       Hue[0.9851524220111436, 1, 1], 
       Hue[0.04254550702833396, 1, 1], 
       Hue[0.10055367721831523`, 1, 1], 
       Hue[0.15917356261530025`, 1, 1], 
       Hue[0.21840264135365023`, 1, 1], 
       Hue[0.2782378071599018, 1, 1], 
       Hue[0.3386767001197204, 1, 1], 
       Hue[0.39971523956125754`, 1, 1], 
       Hue[0.46135214668637253`, 1, 1], 
       Hue[0.5235833706306552, 1, 1], 
       Hue[0.5864068661836146, 1, 1], 
       Hue[0.649821296542562, 1, 1], 
       Hue[0.7138220064371237, 1, 1], 
       Hue[0.7784087549190002, 1, 1], 
       Hue[0.8435773141078866, 1, 1], 
       Hue[0.9093257782873416, 1, 1], 
       Hue[0.9756515583544458, 1, 1], 
       Hue[0.04255353459618736, 1, 1], 
       Hue[0.11002894234378277`, 1, 1], 
       Hue[0.17807576974946937`, 1, 1], 
       Hue[0.2466904245232185, 1, 1], 
       Hue[0.3158733471799709, 1, 1], 
       Hue[0.3856187575594051, 1, 1], 
       Hue[0.4559291256723374, 1, 1], 
       Hue[0.5267985007359997, 1, 1], 
       Hue[0.5982288136813674, 1, 1], 
       Hue[0.6702147558152386, 1, 1], 
       Hue[0.7427566853759748, 1, 1], 
       Hue[0.8158503838879019, 1, 1], 
       Hue[0.8894967525160438, 1, 1], 
       Hue[0.9636945233767934, 1, 1], 
       Hue[0.03843758514675253, 1, 1], 
       Hue[0.11372770534876697`, 1, 1], 
       Hue[0.1895624517637856, 1, 1], 
       Hue[0.26593871306289235`, 1, 1], 
       Hue[0.34285896982386044`, 1, 1], 
       Hue[0.42031468485448653`, 1, 1], 
       Hue[0.9890850417084717, 1, 1], 
       Hue[0.890444563820459, 1, 1], 
       Hue[0.890444563820459, 1, 1], 
       Hue[0.7918350921023508, 1, 1], 
       Hue[0.7918350921023508, 1, 1], 
       Hue[0.6935923344905128, 1, 1], 
       Hue[0.6935923344905128, 1, 1], 
       Hue[0.5955973533557162, 1, 1], 
       Hue[0.5955973533557162, 1, 1], 
       Hue[0.49769643430527716`, 1, 1], 
       Hue[0.49769643430527716`, 1, 1], 
       Hue[0.3999909708344182, 1, 1], 
       Hue[0.3999909708344182, 1, 1], 
       Hue[0.30260995757912756`, 1, 1], 
       Hue[0.30260995757912756`, 1, 1], 
       Hue[0.20549048009062573`, 1, 1], 
       Hue[0.20549048009062573`, 1, 1], 
       Hue[0.10841445415142113`, 1, 1], 
       Hue[0.10841445415142113`, 1, 1], 
       Hue[0.011694343660073115`, 1, 1], 
       Hue[0.011694343660073115`, 1, 1], 
       Hue[0.915136789552362, 1, 1], 
       Hue[0.915136789552362, 1, 1], 
       Hue[0.8187862586312527, 1, 1], 
       Hue[0.8187862586312527, 1, 1], 
       Hue[0.7226446234905665, 1, 1], 
       Hue[0.7226446234905665, 1, 1], 
       Hue[0.6266956278780738, 1, 1], 
       Hue[0.6266956278780738, 1, 1], 
       Hue[0.5309424388597916, 1, 1], 
       Hue[0.5309424388597916, 1, 1], 
       Hue[0.4354159473821284, 1, 1], 
       Hue[0.4354159473821284, 1, 1], 
       Hue[0.34006128509633016`, 1, 1], 
       Hue[0.34006128509633016`, 1, 1], 
       Hue[0.24488834017086, 1, 1], 
       Hue[0.24488834017086, 1, 1], 
       Hue[0.1499381665888046, 1, 1], 
       Hue[0.1499381665888046, 1, 1], 
       Hue[0.0551497709061452, 1, 1], 
       Hue[0.0551497709061452, 1, 1], 
       Hue[0.9605747897944017, 1, 1], 
       Hue[0.9605747897944017, 1, 1], 
       Hue[0.8661822384266291, 1, 1], 
       Hue[0.8661822384266291, 1, 1], 
       Hue[0.7719815454219947, 1, 1], 
       Hue[0.7719815454219947, 1, 1], 
       Hue[0.6779589248969495, 1, 1], 
       Hue[0.6779589248969495, 1, 1], 
       Hue[0.584107788669658, 1, 1], 
       Hue[0.584107788669658, 1, 1], 
       Hue[0.4904321659403038, 1, 1], 
       Hue[0.4904321659403038, 1, 1], 
       Hue[0.3969391264652255, 1, 1], 
       Hue[0.3969391264652255, 1, 1], 
       Hue[0.3036082892055693, 1, 1], 
       Hue[0.3036082892055693, 1, 1], 
       Hue[0.21045143453790424`, 1, 1], 
       Hue[0.21045143453790424`, 1, 1], 
       Hue[0.1174679950897775, 1, 1], 
       Hue[0.1174679950897775, 1, 1], 
       Hue[0.024647137886225627`, 1, 1], 
       Hue[0.024647137886225627`, 1, 1], 
       Hue[0.9319857820365661, 1, 1], 
       Hue[0.9319857820365661, 1, 1], 
       Hue[0.8394897332935869, 1, 1], 
       Hue[0.8394897332935869, 1, 1], 
       Hue[0.7471528779093941, 1, 1], 
       Hue[0.7471528779093941, 1, 1], 
       Hue[0.6549682687227965, 1, 1], 
       Hue[0.6549682687227965, 1, 1], 
       Hue[0.562940161123826, 1, 1], 
       Hue[0.562940161123826, 1, 1], 
       Hue[0.4710601350866123, 1, 1], 
       Hue[0.4710601350866123, 1, 1], 
       Hue[0.379328026400307, 1, 1], 
       Hue[0.379328026400307, 1, 1], 
       Hue[0.2877415089286468, 1, 1], 
       Hue[0.2877415089286468, 1, 1], 
       Hue[0.19629620846912263`, 1, 1], 
       Hue[0.19629620846912263`, 1, 1], 
       Hue[0.10498770191782866`, 1, 1], 
       Hue[0.10498770191782866`, 1, 1], 
       Hue[0.013818248266201809`, 1, 1], 
       Hue[0.013818248266201809`, 1, 1], 
       Hue[0.922779355202294, 1, 1], 
       Hue[0.922779355202294, 1, 1], 
       Hue[0.8318706546694346, 1, 1], 
       Hue[0.8318706546694346, 1, 1], 
       Hue[0.7410886274394506, 1, 1], 
       Hue[0.7410886274394506, 1, 1], 
       Hue[0.6504288919503963, 1, 1], 
       Hue[0.6504288919503963, 1, 1], 
       Hue[0.5598898317429462, 1, 1], 
       Hue[0.5598898317429462, 1, 1], 
       Hue[0.46946392923718244`, 1, 1], 
       Hue[0.46946392923718244`, 1, 1], 
       Hue[0.3791530821283772, 1, 1], 
       Hue[0.3791530821283772, 1, 1], 
       Hue[0.2889467711448164, 1, 1], 
       Hue[0.2889467711448164, 1, 1], 
       Hue[0.19884605935101357`, 1, 1], 
       Hue[0.19884605935101357`, 1, 1], 
       Hue[0.10884451371828528`, 1, 1], 
       Hue[0.10884451371828528`, 1, 1], 
       Hue[0.01893840191355155, 1, 1], 
       Hue[0.01893840191355155, 1, 1], 
       Hue[0.9291246203627189, 1, 1], 
       Hue[0.9291246203627189, 1, 1], 
       Hue[0.839398027108012, 1, 1], 
       Hue[0.839398027108012, 1, 1], 
       Hue[0.7497518597451729, 1, 1], 
       Hue[0.7497518597451729, 1, 1], 
       Hue[0.6601838746800621, 1, 1], 
       Hue[0.6601838746800621, 1, 1], 
       Hue[0.5706881943866222, 1, 1], 
       Hue[0.5706881943866222, 1, 1], 
       Hue[0.48125901280784156`, 1, 1], 
       Hue[0.48125901280784156`, 1, 1], 
       Hue[0.39189064851607397`, 1, 1], 
       Hue[0.39189064851607397`, 1, 1], 
       Hue[0.30257891317312197`, 1, 1], 
       Hue[0.30257891317312197`, 1, 1], 
       Hue[0.21331627749924545`, 1, 1], 
       Hue[0.21331627749924545`, 1, 1], 
       Hue[0.1240969762593327, 1, 1], 
       Hue[0.1240969762593327, 1, 1], 
       Hue[0.034915561215019375`, 1, 1], 
       Hue[0.034915561215019375`, 1, 1], 
       Hue[0.9457641347466291, 1, 1], 
       Hue[0.9457641347466291, 1, 1], 
       Hue[0.8566369738272935, 1, 1], 
       Hue[0.8566369738272935, 1, 1], 
       Hue[0.7675262150857177, 1, 1], 
       Hue[0.7675262150857177, 1, 1], 
       Hue[0.6784238011201688, 1, 1], 
       Hue[0.6784238011201688, 1, 1], 
       Hue[0.5893230207040819, 1, 1], 
       Hue[0.5893230207040819, 1, 1], 
       Hue[0.50021495606295, 1, 1], 
       Hue[0.50021495606295, 1, 1], 
       Hue[0.4110913775684948, 1, 1], 
       Hue[0.4110913775684948, 1, 1], 
       Hue[0.3219424614705361, 1, 1], 
       Hue[0.3219424614705361, 1, 1], 
       Hue[0.23275925793759258`, 1, 1], 
       Hue[0.23275925793759258`, 1, 1], 
       Hue[0.143532350562769, 1, 1], 
       Hue[0.143532350562769, 1, 1], 
       Hue[0.05425179677263266, 1, 1], 
       Hue[0.05425179677263266, 1, 1], 
       Hue[0.9649067193442054, 1, 1], 
       Hue[0.9649067193442054, 1, 1], 
       Hue[0.8754870852363708, 1, 1], 
       Hue[0.8754870852363708, 1, 1], 
       Hue[0.7859800701598657, 1, 1], 
       Hue[0.7859800701598657, 1, 1], 
       Hue[0.6963751868911885, 1, 1], 
       Hue[0.6963751868911885, 1, 1], 
       Hue[0.6066591090207734, 1, 1], 
       Hue[0.6066591090207734, 1, 1], 
       Hue[0.5168198134156986, 1, 1], 
       Hue[0.5168198134156986, 1, 1], 
       Hue[0.42684569537233746`, 1, 1], 
       Hue[0.42684569537233746`, 1, 1], 
       Hue[0.33672288201474887`, 1, 1], 
       Hue[0.33672288201474887`, 1, 1], 
       Hue[0.24643890665017157`, 1, 1], 
       Hue[0.24643890665017157`, 1, 1], 
       Hue[0.1559790483703322, 1, 1], 
       Hue[0.1559790483703322, 1, 1], 
       Hue[0.06533250362229022, 1, 1], 
       Hue[0.06533250362229022, 1, 1], 
       Hue[0.9744852437056882, 1, 1], 
       Hue[0.9744852437056882, 1, 1], 
       Hue[0.8834255839195694, 1, 1], 
       Hue[0.8834255839195694, 1, 1], 
       Hue[0.7921421823685508, 1, 1], 
       Hue[0.7921421823685508, 1, 1], 
       Hue[0.7006241285792536, 1, 1], 
       Hue[0.7006241285792536, 1, 1], 
       Hue[0.6088632573112414, 1, 1], 
       Hue[0.6088632573112414, 1, 1], 
       Hue[0.5168507348555242, 1, 1], 
       Hue[0.5168507348555242, 1, 1], 
       Hue[0.42458376599519027`, 1, 1], 
       Hue[0.42458376599519027`, 1, 1], 
       Hue[0.33205862577194645`, 1, 1], 
       Hue[0.33205862577194645`, 1, 1], 
       Hue[0.2392775083989476, 1, 1], 
       Hue[0.2392775083989476, 1, 1], 
       Hue[0.14624478418476983`, 1, 1], 
       Hue[0.14624478418476983`, 1, 1], 
       Hue[0.052970346936419506`, 1, 1], 
       Hue[0.052970346936419506`, 1, 1], 
       Hue[0.9594676360426614, 1, 1], 
       Hue[0.9594676360426614, 1, 1], 
       Hue[0.8657577363329141, 1, 1], 
       Hue[0.8657577363329141, 1, 1], 
       Hue[0.7718665933810455, 1, 1], 
       Hue[0.7718665933810455, 1, 1], 
       Hue[0.6778270601834724, 1, 1], 
       Hue[0.6778270601834724, 1, 1], 
       Hue[0.5836773317911002, 1, 1], 
       Hue[0.5836773317911002, 1, 1], 
       Hue[0.48946425451082115`, 1, 1], 
       Hue[0.48946425451082115`, 1, 1], 
       Hue[0.3952418144069382, 1, 1], 
       Hue[0.3952418144069382, 1, 1], 
       Hue[0.3010708889653632, 1, 1], 
       Hue[0.3010708889653632, 1, 1], 
       Hue[0.207018848047681, 1, 1], 
       Hue[0.207018848047681, 1, 1], 
       Hue[0.11316324104925135`, 1, 1], 
       Hue[0.11316324104925135`, 1, 1], 
       Hue[0.019585516746522633`, 1, 1], 
       Hue[0.019585516746522633`, 1, 1], 
       Hue[0.9263748173191635, 1, 1], 
       Hue[0.9263748173191635, 1, 1], 
       Hue[0.8336288784715873, 1, 1], 
       Hue[0.8336288784715873, 1, 1], 
       Hue[0.7414537703560893, 1, 1], 
       Hue[0.7414537703560893, 1, 1], 
       Hue[0.6499644952350254, 1, 1], 
       Hue[0.6499644952350254, 1, 1], 
       Hue[0.559286572819959, 1, 1], 
       Hue[0.559286572819959, 1, 1], 
       Hue[0.46955356366737094`, 1, 1], 
       Hue[0.46955356366737094`, 1, 1], 
       Hue[0.38090894736594, 1, 1], 
       Hue[0.38090894736594, 1, 1], 
       Hue[0.2935085010777411, 1, 1], 
       Hue[0.2935085010777411, 1, 1], 
       Hue[0.20750998380319777`, 1, 1], 
       Hue[0.20750998380319777`, 1, 1], 
       Hue[0.12308448701168116`, 1, 1], 
       Hue[0.12308448701168116`, 1, 1], 
       Hue[0.040405866180151344`, 1, 1], 
       Hue[0.040405866180151344`, 1, 1], 
       Hue[0.9596577933041008, 1, 1], 
       Hue[0.9596577933041008, 1, 1], 
       Hue[0.8810254867376445, 1, 1], 
       Hue[0.8810254867376445, 1, 1], 
       Hue[0.8046989087260993, 1, 1], 
       Hue[0.8046989087260993, 1, 1], 
       Hue[0.7308629301685772, 1, 1], 
       Hue[0.7308629301685772, 1, 1], 
       Hue[0.659694095351627, 1, 1], 
       Hue[0.659694095351627, 1, 1], 
       Hue[0.591355858433406, 1, 1], 
       Hue[0.591355858433406, 1, 1], 
       Hue[0.5259960179322813, 1, 1], 
       Hue[0.5259960179322813, 1, 1], 
       Hue[0.4637384836446359, 1, 1], 
       Hue[0.4637384836446359, 1, 1], 
       Hue[0.4046827309417128, 1, 1], 
       Hue[0.4046827309417128, 1, 1], 
       Hue[0.34890027740507623`, 1, 1], 
       Hue[0.34890027740507623`, 1, 1], 
       Hue[0.2964351349133147, 1, 1], 
       Hue[0.2964351349133147, 1, 1], 
       Hue[0.24730822874130168`, 1, 1], 
       Hue[0.24730822874130168`, 1, 1], 
       Hue[0.20151138082108272`, 1, 1], 
       Hue[0.20151138082108272`, 1, 1], 
       Hue[0.15901329786186943`, 1, 1], 
       Hue[0.15901329786186943`, 1, 1], 
       Hue[0.1197649271403978, 1, 1], 
       Hue[0.1197649271403978, 1, 1], 
       Hue[0.08369802893832214, 1, 1], 
       Hue[0.08369802893832214, 1, 1], 
       Hue[0.05073030875715411, 1, 1], 
       Hue[0.05073030875715411, 1, 1], 
       Hue[0.020769647506886663`, 1, 1], 
       Hue[0.020769647506886663`, 1, 1], 
       Hue[0.9937147157833544, 1, 1], 
       Hue[0.9937147157833544, 1, 1], 
       Hue[0.9694593009365832, 1, 1], 
       Hue[0.9694593009365832, 1, 1], 
       Hue[0.9478941084618844, 1, 1], 
       Hue[0.9478941084618844, 1, 1], 
       Hue[0.9289103885338673, 1, 1], 
       Hue[0.9289103885338673, 1, 1], 
       Hue[0.9123988367658288, 1, 1], 
       Hue[0.9123988367658288, 1, 1], 
       Hue[0.8982520275469468, 1, 1], 
       Hue[0.8982520275469468, 1, 1], 
       Hue[0.886366658292858, 1, 1], 
       Hue[0.886366658292858, 1, 1], 
       Hue[0.8766426050628463, 1, 1], 
       Hue[0.8766426050628463, 1, 1], 
       Hue[0.8689841847218216, 1, 1], 
       Hue[0.8689841847218216, 1, 1], 
       Hue[0.8633013381767353, 1, 1], 
       Hue[0.8633013381767353, 1, 1], 
       Hue[0.8595080716578948, 1, 1], 
       Hue[0.8595080716578948, 1, 1], 
       Hue[0.8575234682582267, 1, 1], 
       Hue[0.8575234682582267, 1, 1], 
       Hue[0.8572726383404587, 1, 1], 
       Hue[0.8572726383404587, 1, 1], 
       Hue[0.8586834192166569, 1, 1], 
       Hue[0.8586834192166569, 1, 1], 
       Hue[0.8616906928539774, 1, 1], 
       Hue[0.8616906928539774, 1, 1], 
       Hue[0.8662315826883799, 1, 1], 
       Hue[0.8662315826883799, 1, 1], 
       Hue[0.8722494769006455, 1, 1], 
       Hue[0.8722494769006455, 1, 1], 
       Hue[0.8796904511863467, 1, 1], 
       Hue[0.8796904511863467, 1, 1], 
       Hue[0.8885044010740425, 1, 1], 
       Hue[0.8885044010740425, 1, 1], 
       Hue[0.8986452227927522, 1, 1], 
       Hue[0.8986452227927522, 1, 1], 
       Hue[0.9100709017328726, 1, 1], 
       Hue[0.9100709017328726, 1, 1], 
       Hue[0.922740508081968, 1, 1], 
       Hue[0.922740508081968, 1, 1], 
       Hue[0.9366170036872785, 1, 1], 
       Hue[0.9366170036872785, 1, 1], 
       Hue[0.9516664083103602, 1, 1], 
       Hue[0.9516664083103602, 1, 1], 
       Hue[0.9678565539344235, 1, 1], 
       Hue[0.9678565539344235, 1, 1], 
       Hue[0.9851573817997091, 1, 1], 
       Hue[0.9851573817997091, 1, 1], 
       Hue[0.003541029282514706, 1, 1], 
       Hue[0.003541029282514706, 1, 1], 
       Hue[0.022982276990593284`, 1, 1], 
       Hue[0.022982276990593284`, 1, 1], 
       Hue[0.04345618989206919, 1, 1], 
       Hue[0.04345618989206919, 1, 1], 
       Hue[0.06494019180123378, 1, 1], 
       Hue[0.06494019180123378, 1, 1], 
       Hue[0.0874131980234504, 1, 1], 
       Hue[0.0874131980234504, 1, 1], 
       Hue[0.11085512776100503`, 1, 1], 
       Hue[0.11085512776100503`, 1, 1], 
       Hue[0.1352477810807344, 1, 1], 
       Hue[0.1352477810807344, 1, 1], 
       Hue[0.16057339342287885`, 1, 1], 
       Hue[0.16057339342287885`, 1, 1], 
       Hue[0.18681539630932423`, 1, 1], 
       Hue[0.18681539630932423`, 1, 1], 
       Hue[0.21395846771373908`, 1, 1], 
       Hue[0.21395846771373908`, 1, 1], 
       Hue[0.24198818379898965`, 1, 1], 
       Hue[0.24198818379898965`, 1, 1], 
       Hue[0.2708910987900743, 1, 1], 
       Hue[0.2708910987900743, 1, 1], 
       Hue[0.3006539059544776, 1, 1], 
       Hue[0.3006539059544776, 1, 1], 
       Hue[0.33126453314316606`, 1, 1], 
       Hue[0.33126453314316606`, 1, 1], 
       Hue[0.3627118699332237, 1, 1], 
       Hue[0.3627118699332237, 1, 1], 
       Hue[0.3949837005086097, 1, 1], 
       Hue[0.3949837005086097, 1, 1], 
       Hue[0.42806988647193905`, 1, 1], 
       Hue[0.42806988647193905`, 1, 1], 
       Hue[0.4619600062641948, 1, 1], 
       Hue[0.4619600062641948, 1, 1], 
       Hue[0.4966450931302264, 1, 1], 
       Hue[0.4966450931302264, 1, 1], 
       Hue[0.5321158935007508, 1, 1], 
       Hue[0.5321158935007508, 1, 1], 
       Hue[0.5683632217963384, 1, 1], 
       Hue[0.5683632217963384, 1, 1], 
       Hue[0.6053791741127653, 1, 1], 
       Hue[0.6053791741127653, 1, 1], 
       Hue[0.6431566313515203, 1, 1], 
       Hue[0.6431566313515203, 1, 1], 
       Hue[0.6816868752353401, 1, 1], 
       Hue[0.6816868752353401, 1, 1], 
       Hue[0.72096364429756, 1, 1], 
       Hue[0.72096364429756, 1, 1], 
       Hue[0.7609790655744311, 1, 1], 
       Hue[0.7609790655744311, 1, 1], 
       Hue[0.801726159467002, 1, 1], 
       Hue[0.801726159467002, 1, 1], 
       Hue[0.8431986948347605, 1, 1], 
       Hue[0.8431986948347605, 1, 1], 
       Hue[0.8853905620106385, 1, 1], 
       Hue[0.8853905620106385, 1, 1], 
       Hue[0.928294741191026, 1, 1], 
       Hue[0.928294741191026, 1, 1], 
       Hue[0.9719066681331644, 1, 1], 
       Hue[0.9719066681331644, 1, 1], 
       Hue[0.016219788090091947`, 1, 1], 
       Hue[0.016219788090091947`, 1, 1], 
       Hue[0.06123023095704857, 1, 1], 
       Hue[0.06123023095704857, 1, 1], 
       Hue[0.10693189420594468`, 1, 1], 
       Hue[0.10693189420594468`, 1, 1], 
       Hue[0.15331972133290422`, 1, 1], 
       Hue[0.15331972133290422`, 1, 1], 
       Hue[0.20038888796819257`, 1, 1], 
       Hue[0.20038888796819257`, 1, 1], 
       Hue[0.24813299763561023`, 1, 1], 
       Hue[0.24813299763561023`, 1, 1], 
       Hue[0.29654916208262616`, 1, 1], 
       Hue[0.29654916208262616`, 1, 1], 
       Hue[0.3456310281848976, 1, 1], 
       Hue[0.3456310281848976, 1, 1], 
       Hue[0.3953752971354173, 1, 1], 
       Hue[0.3953752971354173, 1, 1], 
       Hue[0.4457779494411485, 1, 1], 
       Hue[0.4457779494411485, 1, 1], 
       Hue[0.49683472538993373`, 1, 1], 
       Hue[0.49683472538993373`, 1, 1], 
       Hue[0.5485422000086002, 1, 1], 
       Hue[0.5485422000086002, 1, 1], 
       Hue[0.6008954329863241, 1, 1], 
       Hue[0.6008954329863241, 1, 1], 
       Hue[0.6538909685908945, 1, 1], 
       Hue[0.6538909685908945, 1, 1], 
       Hue[0.7075250668459268, 1, 1], 
       Hue[0.7075250668459268, 1, 1], 
       Hue[0.7617944518713861, 1, 1], 
       Hue[0.7617944518713861, 1, 1], 
       Hue[0.8166949979465602, 1, 1], 
       Hue[0.8166949979465602, 1, 1], 
       Hue[0.872223828978173, 1, 1], 
       Hue[0.872223828978173, 1, 1], 
       Hue[0.9283771405632653, 1, 1], 
       Hue[0.9283771405632653, 1, 1], 
       Hue[0.9851524220111436, 1, 1], 
       Hue[0.9851524220111436, 1, 1], 
       Hue[0.04254550702833396, 1, 1], 
       Hue[0.04254550702833396, 1, 1], 
       Hue[0.10055367721831523`, 1, 1], 
       Hue[0.10055367721831523`, 1, 1], 
       Hue[0.15917356261530025`, 1, 1], 
       Hue[0.15917356261530025`, 1, 1], 
       Hue[0.21840264135365023`, 1, 1], 
       Hue[0.21840264135365023`, 1, 1], 
       Hue[0.2782378071599018, 1, 1], 
       Hue[0.2782378071599018, 1, 1], 
       Hue[0.3386767001197204, 1, 1], 
       Hue[0.3386767001197204, 1, 1], 
       Hue[0.39971523956125754`, 1, 1], 
       Hue[0.39971523956125754`, 1, 1], 
       Hue[0.46135214668637253`, 1, 1], 
       Hue[0.46135214668637253`, 1, 1], 
       Hue[0.5235833706306552, 1, 1], 
       Hue[0.5235833706306552, 1, 1], 
       Hue[0.5864068661836146, 1, 1], 
       Hue[0.5864068661836146, 1, 1], 
       Hue[0.649821296542562, 1, 1], 
       Hue[0.649821296542562, 1, 1], 
       Hue[0.7138220064371237, 1, 1], 
       Hue[0.7138220064371237, 1, 1], 
       Hue[0.7784087549190002, 1, 1], 
       Hue[0.7784087549190002, 1, 1], 
       Hue[0.8435773141078866, 1, 1], 
       Hue[0.8435773141078866, 1, 1], 
       Hue[0.9093257782873416, 1, 1], 
       Hue[0.9093257782873416, 1, 1], 
       Hue[0.9756515583544458, 1, 1], 
       Hue[0.9756515583544458, 1, 1], 
       Hue[0.04255353459618736, 1, 1], 
       Hue[0.04255353459618736, 1, 1], 
       Hue[0.11002894234378277`, 1, 1], 
       Hue[0.11002894234378277`, 1, 1], 
       Hue[0.17807576974946937`, 1, 1], 
       Hue[0.17807576974946937`, 1, 1], 
       Hue[0.2466904245232185, 1, 1], 
       Hue[0.2466904245232185, 1, 1], 
       Hue[0.3158733471799709, 1, 1], 
       Hue[0.3158733471799709, 1, 1], 
       Hue[0.3856187575594051, 1, 1], 
       Hue[0.3856187575594051, 1, 1], 
       Hue[0.4559291256723374, 1, 1], 
       Hue[0.4559291256723374, 1, 1], 
       Hue[0.5267985007359997, 1, 1], 
       Hue[0.5267985007359997, 1, 1], 
       Hue[0.5982288136813674, 1, 1], 
       Hue[0.5982288136813674, 1, 1], 
       Hue[0.6702147558152386, 1, 1], 
       Hue[0.6702147558152386, 1, 1], 
       Hue[0.7427566853759748, 1, 1], 
       Hue[0.7427566853759748, 1, 1], 
       Hue[0.8158503838879019, 1, 1], 
       Hue[0.8158503838879019, 1, 1], 
       Hue[0.8894967525160438, 1, 1], 
       Hue[0.8894967525160438, 1, 1], 
       Hue[0.9636945233767934, 1, 1], 
       Hue[0.9636945233767934, 1, 1], 
       Hue[0.03843758514675253, 1, 1], 
       Hue[0.03843758514675253, 1, 1], 
       Hue[0.11372770534876697`, 1, 1], 
       Hue[0.11372770534876697`, 1, 1], 
       Hue[0.1895624517637856, 1, 1], 
       Hue[0.1895624517637856, 1, 1], 
       Hue[0.26593871306289235`, 1, 1], 
       Hue[0.26593871306289235`, 1, 1], 
       Hue[0.34285896982386044`, 1, 1], 
       Hue[0.34285896982386044`, 1, 1], 
       Hue[0.42031468485448653`, 1, 1]}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->{True, None},
    AxesOrigin->{0, 0},
    Epilog->
     LineBox[{{1, 0.000664668848283562}, {2, 0.0007578249757784219}, {
       3, 0.0008642419812624661}, {4, 0.0009837440297558759}, {
       5, 0.0011187530665935752`}, {6, 0.00127026705304837}, {
       7, 0.0014407827005841357`}, {8, 0.0016324575192494697`}, {
       9, 0.0018469143646733174`}, {10, 0.0020875272332460154`}, {
       11, 0.0023571779792675752`}, {12, 0.002657382543073407}, {
       13, 0.0029934640484062455`}, {14, 0.0033673795303978604`}, {
       15, 0.0037830445000899115`}, {16, 0.004245348595680484}, {
       17, 0.004758296131193935}, {18, 0.005325861929652516}, {
       19, 0.005954032823546849}, {20, 0.006648894812744901}, {
       21, 0.007413337365274173}, {22, 0.008255961576166157}, {
       23, 0.009181721326599274}, {24, 0.010199667647376759`}, {
       25, 0.011314832152383335`}, {26, 0.012534941237711757`}, {
       27, 0.013868397105800144`}, {28, 0.015323063450822292`}, {
       29, 0.016906143722724996`}, {30, 0.018628298995364164`}, {
       31, 0.020496553105252308`}, {32, 0.02252104662660519}, {
       33, 0.024710776316784058`}, {34, 0.027074982320335568`}, {
       35, 0.029621695252019756`}, {36, 0.032362003145395654`}, {
       37, 0.03530363221228}, {38, 0.038455441062931875`}, {
       39, 0.04182646577968378}, {40, 0.045423915091767325`}, {
       41, 0.04925692036876082}, {42, 0.05333020871688937}, {
       43, 0.05765115187281245}, {44, 0.0622257674990335}, {
       45, 0.06705670452812336}, {46, 0.07214708022323275}, {
       47, 0.07749983054642036}, {48, 0.08311439321732401}, {
       49, 0.08898877829707996}, {50, 0.09512052037227856}, {
       51, 0.10150663774809097`}, {52, 0.1081354253800436}, {
       53, 0.1150019061040271}, {54, 0.1220936144131497}, {
       55, 0.1293972064525951}, {56, 0.13689892989489563`}, {
       57, 0.14457712758239238`}, {58, 0.15241315214460283`}, {
       59, 0.16038190903242153`}, {60, 0.1684615566075817}, {
       61, 0.17662172799118284`}, {62, 0.18483196842932537`}, {
       63, 0.19306250340813078`}, {64, 0.2012767371394998}, {
       65, 0.20944014406619127`}, {66, 0.21751560159964423`}, {
       67, 0.2254641288338429}, {68, 0.2332454219851741}, {
       69, 0.24081985861409405`}, {70, 0.24814634601642171`}, {
       71, 0.2551860454597357}, {72, 0.26189651111972095`}, {
       73, 0.26824207172577524`}, {74, 0.2741828891974974}, {
       75, 0.27968416290233983`}, {76, 0.2847149872488188}, {
       77, 0.28924272537030293`}, {78, 0.29324323480954684`}, {
       79, 0.29669005447292734`}, {80, 0.2995684888957312}, {
       81, 0.3018620657122918}, {82, 0.3035644537678138}, {
       83, 0.30466888785328816`}, {84, 0.305180352289716}, {
       85, 0.3051033785076924}, {86, 0.3044531797231474}, {
       87, 0.3032499216205878}, {88, 0.30151529073836747`}, {
       89, 0.29928130094478006`}, {90, 0.29658211575418286`}, {
       91, 0.29345613837786805`}, {92, 0.2899500583888158}, {
       93, 0.28611061885544065`}, {94, 0.281987712769225}, {
       95, 0.27763458297817006`}, {96, 0.2731040735661529}, {
       97, 0.2684499236456921}, {98, 0.26372519850162407`}, {
       99, 0.25898111007698604`}, {100, 0.2542669640422208}, {
       101, 0.24962607547812987`}, {102, 0.2451006904660024}, {
       103, 0.2407254513760431}, {104, 0.23652999170688682`}, {
       105, 0.23253907066447196`}, {106, 0.22877078069950094`}, {
       107, 0.22523885874077418`}, {108, 0.2219497583587372}, {
       109, 0.21890453854473463`}, {110, 0.21610496605481777`}, {
       111, 0.21354656728857724`}, {112, 0.21122466686562755`}, {
       113, 0.20913711767375845`}, {114, 0.2072758475357398}, {
       115, 0.20563683972122598`}, {116, 0.2042144849955796}, {
       117, 0.20300468425376453`}, {118, 0.2020066182357582}, {
       119, 0.2012241133441278}, {120, 0.20066355626120483`}, {
       121, 0.20033675032379364`}, {122, 0.2002585178414794}, {
       123, 0.20044349024995206`}, {124, 0.200904788079577}, {
       125, 0.20165375261870563`}, {126, 0.2026986286273951}, {
       127, 0.2040397524877515}, {128, 0.205676877836494}, {
       129, 0.2076062432423218}, {130, 0.20981912254695656`}, {
       131, 0.21230330197935168`}, {132, 0.2150453624253769}, {
       133, 0.21803000030396835`}, {134, 0.2212398536506614}, {
       135, 0.22465651833649647`}, {136, 0.22825894875125857`}, {
       137, 0.23202606115224791`}, {138, 0.23593876903811686`}, {
       139, 0.23997423756735528`}, {140, 0.244113217539779}, {
       141, 0.2483363620080084}, {142, 0.2526211336341694}, {
       143, 0.2569523185113574}, {144, 0.2613107331206272}, {
       145, 0.2656778731696579}, {146, 0.2700404107000231}, {
       147, 0.2743815278442918}, {148, 0.278685809033522}, {
       149, 0.28294240676743787`}, {150, 0.2871360731217418}, {
       151, 0.29125595042959995`}, {152, 0.29528955271190627`}, {
       153, 0.299228184700775}, {154, 0.3030599076373242}, {
       155, 0.3067753173207391}, {156, 0.31036533305072056`}, {
       157, 0.31382251148178053`}, {158, 0.31713754919580145`}, {
       159, 0.32030483692599415`}, {160, 0.3233157296612615}, {
       161, 0.3261646838367019}, {162, 0.3288455578407144}, {
       163, 0.3313507737565988}, {164, 0.33367703222519535`}, {
       165, 0.33581938918725435`}, {166, 0.3377711371853463}, {
       167, 0.33953009665770134`}, {168, 0.34109267616594374`}, {
       169, 0.3424544603684473}, {170, 0.3436120225553153}, {
       171, 0.34456225034033994`}, {172, 0.34530432483727314`}, {
       173, 0.34583417891188684`}, {174, 0.34614845491660057`}, {
       175, 0.34624903348363184`}, {176, 0.3461332009271847}, {
       177, 0.3457996058748039}, {178, 0.3452491686542008}, {
       179, 0.34448279393761716`}, {180, 0.3435000913327909}, {
       181, 0.3423032677719288}, {182, 0.3408921639498197}, {
       183, 0.33926889642340363`}, {184, 0.3374349908636319}, {
       185, 0.3353925758858055}, {186, 0.33314500092582017`}, {
       187, 0.3306949005584111}, {188, 0.32804702465442587`}, {
       189, 0.32520372622224464`}, {190, 0.322173103737576}, {
       191, 0.3189573533941454}, {192, 0.31556094764975107`}, {
       193, 0.3119908533406933}, {194, 0.30825051131030745`}, {
       195, 0.30434627720431023`}, {196, 0.3002852528222341}, {
       197, 0.29607335693637815`}, {198, 0.29171910266007883`}, {
       199, 0.2872284276059091}, {200, 0.28261059079894446`}, {
       201, 0.2778724288784171}, {202, 0.27301980928850683`}, {
       203, 0.26806223527064726`}, {204, 0.2630046812916387}, {
       205, 0.25785719771069154`}, {206, 0.2526286355583951}, {
       207, 0.24732585659345746`}, {208, 0.24195855321537207`}, {
       209, 0.23653358351771342`}, {210, 0.2310612431805267}, {
       211, 0.22554784551728368`}, {212, 0.22000273469118897`}, {
       213, 0.21443353450328032`}, {214, 0.20884957007158925`}, {
       215, 0.2032569076949167}, {216, 0.19766514228703305`}, {
       217, 0.1920801360379522}, {218, 0.18651072625107135`}, {
       219, 0.1809638615685871}, {220, 0.17544693144378382`}, {
       221, 0.1699662029126255}, {222, 0.16452835193572998`}, {
       223, 0.1591405414104012}, {224, 0.1538075784555804}, {
       225, 0.14853739118751566`}, {226, 0.14333374957006437`}, {
       227, 0.13820238522581701`}, {228, 0.1331482959727786}, {
       229, 0.12817604197316176`}, {230, 0.12328979286015337`}, {
       231, 0.1184944185453906}, {232, 0.11379308044375856`}, {
       233, 0.10918961605542017`}, {234, 0.10468534367902582`}, {
       235, 0.10028529231125609`}, {236, 0.09599010278565637}, {
       237, 0.09180311548526479}, {238, 0.08772586588157019}, {
       239, 0.08375958829813208}, {240, 0.07990594418213398}, {
       241, 0.07616473206131731}, {242, 0.07253762318455177}, {
       243, 0.06902405594024237}, {244, 0.06562589736100348}, {
       245, 0.06234053998751813}, {246, 0.059168502737609416`}, {
       247, 0.056110074025049764`}, {248, 0.05316302748491828}, {
       249, 0.0503274687300253}, {250, 0.047601231688745645`}, {
       251, 0.04498285349295762}}],
    Frame->True,
    FrameTicks->{{{1., 
        FormBox[
         RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
        FormBox[
         RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
        FormBox[
         RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
        FormBox[
         RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
        FormBox["0", TraditionalForm]}, {157.25, 
        FormBox["2.5`", TraditionalForm]}, {188.5, 
        FormBox["5", TraditionalForm]}, {219.75, 
        FormBox["7.5`", TraditionalForm]}, {251., 
        FormBox["10", TraditionalForm]}}, Automatic, None, None},
    PlotRange->{-0.1, 1.1},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], Automatic}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF12lQFVcWB/DGGESjBnFFyfgwQhwVFRBw94/sLggIyE7De+w8UCdoXGIa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     "], {
      {GrayLevel[0.5], EdgeForm[None], 
       GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ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         "]]]}, {{}, {}, {
        {Hue[0.5373228843195937, 1, 1], LineBox[{1, 2}]}, 
        {Hue[0.4403401308194928, 1, 1], LineBox[{2, 3}]}, 
        {Hue[0.3435613420702442, 1, 1], LineBox[{3, 4}]}, 
        {Hue[0.24690243660616393`, 1, 1], LineBox[{4, 5}]}, 
        {Hue[0.150454945879873, 1, 1], LineBox[{5, 6}]}, 
        {Hue[0.054253709597173946`, 1, 1], LineBox[{6, 7}]}, 
        {Hue[0.9582329689095465, 1, 1], LineBox[{7, 8}]}, 
        {Hue[0.8624154616652879, 1, 1], LineBox[{8, 9}]}, 
        {Hue[0.7667920547714719, 1, 1], LineBox[{9, 10}]}, 
        {Hue[0.6713539032786822, 1, 1], LineBox[{10, 11}]}, 
        {Hue[0.5761322441426646, 1, 1], LineBox[{11, 12}]}, 
        {Hue[0.4810681788903602, 1, 1], LineBox[{12, 13}]}, 
        {Hue[0.38618845496889015`, 1, 1], LineBox[{13, 14}]}, 
        {Hue[0.2914937307005535, 1, 1], LineBox[{14, 15}]}, 
        {Hue[0.19699709955287387`, 1, 1], LineBox[{15, 16}]}, 
        {Hue[0.1026633796553843, 1, 1], LineBox[{16, 17}]}, 
        {Hue[0.008503512405973614, 1, 1], LineBox[{17, 18}]}, 
        {Hue[0.9145316811967742, 1, 1], LineBox[{18, 19}]}, 
        {Hue[0.8207243806703278, 1, 1], LineBox[{19, 20}]}, 
        {Hue[0.7270951531531911, 1, 1], LineBox[{20, 21}]}, 
        {Hue[0.6336280106405164, 1, 1], LineBox[{21, 22}]}, 
        {Hue[0.5403238375965598, 1, 1], LineBox[{22, 23}]}, 
        {Hue[0.44719515461189513`, 1, 1], LineBox[{23, 24}]}, 
        {Hue[0.3542219104665249, 1, 1], LineBox[{24, 25}]}, 
        {Hue[0.261406034757453, 1, 1], LineBox[{25, 26}]}, 
        {Hue[0.16874846532010704`, 1, 1], LineBox[{26, 27}]}, 
        {Hue[0.07625049917109356, 1, 1], LineBox[{27, 28}]}, 
        {Hue[0.983898824560589, 1, 1], LineBox[{28, 29}]}, 
        {Hue[0.8917010932212673, 1, 1], LineBox[{29, 30}]}, 
        {Hue[0.7996520993560525, 1, 1], LineBox[{30, 31}]}, 
        {Hue[0.707743391289746, 1, 1], LineBox[{31, 32}]}, 
        {Hue[0.6159813669193629, 1, 1], LineBox[{32, 33}]}, 
        {Hue[0.5243603446357614, 1, 1], LineBox[{33, 34}]}, 
        {Hue[0.4328738691798968, 1, 1], LineBox[{34, 35}]}, 
        {Hue[0.3415208785271995, 1, 1], LineBox[{35, 36}]}, 
        {Hue[0.2502971034620979, 1, 1], LineBox[{36, 37}]}, 
        {Hue[0.15920419790548215`, 1, 1], LineBox[{37, 38}]}, 
        {Hue[0.06823149246682571, 1, 1], LineBox[{38, 39}]}, 
        {Hue[0.9773857509141324, 1, 1], LineBox[{39, 40}]}, 
        {Hue[0.8866550914096966, 1, 1], LineBox[{40, 41}]}, 
        {Hue[0.7960381183778849, 1, 1], LineBox[{41, 42}]}, 
        {Hue[0.7055349171841978, 1, 1], LineBox[{42, 43}]}, 
        {Hue[0.6151380108443617, 1, 1], LineBox[{43, 44}]}, 
        {Hue[0.5248452588742869, 1, 1], LineBox[{44, 45}]}, 
        {Hue[0.4346517215644478, 1, 1], LineBox[{45, 46}]}, 
        {Hue[0.3445541576573799, 1, 1], LineBox[{46, 47}]}, 
        {Hue[0.2545477968642205, 1, 1], LineBox[{47, 48}]}, 
        {Hue[0.16462741410865095`, 1, 1], LineBox[{48, 49}]}, 
        {Hue[0.07479106859346756, 1, 1], LineBox[{49, 50}]}, 
        {Hue[0.9850329040069344, 1, 1], LineBox[{50, 51}]}, 
        {Hue[0.8953473747186199, 1, 1], LineBox[{51, 52}]}, 
        {Hue[0.8057307072880856, 1, 1], LineBox[{52, 53}]}, 
        {Hue[0.7161762779604128, 1, 1], LineBox[{53, 54}]}, 
        {Hue[0.6266791615905233, 1, 1], LineBox[{54, 55}]}, 
        {Hue[0.537233630987699, 1, 1], LineBox[{55, 56}]}, 
        {Hue[0.44783486011628426`, 1, 1], LineBox[{56, 57}]}, 
        {Hue[0.35847490168618695`, 1, 1], LineBox[{57, 58}]}, 
        {Hue[0.2691480839616139, 1, 1], LineBox[{58, 59}]}, 
        {Hue[0.17984719749350703`, 1, 1], LineBox[{59, 60}]}, 
        {Hue[0.0905655025466636, 1, 1], LineBox[{60, 61}]}, 
        {Hue[0.0012950694149223545`, 1, 1], LineBox[{61, 62}]}, 
        {Hue[0.9120293898934388, 1, 1], LineBox[{62, 63}]}, 
        {Hue[0.8227595802498255, 1, 1], LineBox[{63, 64}]}, 
        {Hue[0.7334770677884576, 1, 1], LineBox[{64, 65}]}, 
        {Hue[0.6441724629825096, 1, 1], LineBox[{65, 66}]}, 
        {Hue[0.5548371466919941, 1, 1], LineBox[{66, 67}]}, 
        {Hue[0.465461449193391, 1, 1], LineBox[{67, 68}]}, 
        {Hue[0.376034319947191, 1, 1], LineBox[{68, 69}]}, 
        {Hue[0.2865456196687959, 1, 1], LineBox[{69, 70}]}, 
        {Hue[0.19698327643107333`, 1, 1], LineBox[{70, 71}]}, 
        {Hue[0.1073368478949821, 1, 1], LineBox[{71, 72}]}, 
        {Hue[0.01759296362047231, 1, 1], LineBox[{72, 73}]}, 
        {Hue[0.9277393289210646, 1, 1], LineBox[{73, 74}]}, 
        {Hue[0.8377620797664849, 1, 1], LineBox[{74, 75}]}, 
        {Hue[0.7476481320246258, 1, 1], LineBox[{75, 76}]}, 
        {Hue[0.6573829603765364, 1, 1], LineBox[{76, 77}]}, 
        {Hue[0.5669519357335853, 1, 1], LineBox[{77, 78}]}, 
        {Hue[0.47633986833715763`, 1, 1], LineBox[{78, 79}]}, 
        {Hue[0.3855322472772245, 1, 1], LineBox[{79, 80}]}, 
        {Hue[0.29451264153189877`, 1, 1], LineBox[{80, 81}]}, 
        {Hue[0.2032663941206125, 1, 1], LineBox[{81, 82}]}, 
        {Hue[0.1117779759296776, 1, 1], LineBox[{82, 83}]}, 
        {Hue[0.020033942761572048`, 1, 1], LineBox[{83, 84}]}, 
        {Hue[0.9280205709389144, 1, 1], LineBox[{84, 85}]}, 
        {Hue[0.8357247215490928, 1, 1], LineBox[{85, 86}]}, 
        {Hue[0.7431357669534746, 1, 1], LineBox[{86, 87}]}, 
        {Hue[0.6502444277652975, 1, 1], LineBox[{87, 88}]}, 
        {Hue[0.5570441913618924, 1, 1], LineBox[{88, 89}]}, 
        {Hue[0.4635317224138139, 1, 1], LineBox[{89, 90}]}, 
        {Hue[0.36970776965283664`, 1, 1], LineBox[{90, 91}]}, 
        {Hue[0.2755740273533197, 1, 1], LineBox[{91, 92}]}, 
        {Hue[0.18114110308239148`, 1, 1], LineBox[{92, 93}]}, 
        {Hue[0.08642018714458502, 1, 1], LineBox[{93, 94}]}, 
        {Hue[0.991429466836158, 1, 1], LineBox[{94, 95}]}, 
        {Hue[0.8961918398709002, 1, 1], LineBox[{95, 96}]}, 
        {Hue[0.8007328384798726, 1, 1], LineBox[{96, 97}]}, 
        {Hue[0.7050864037553836, 1, 1], LineBox[{97, 98}]}, 
        {Hue[0.609285542248224, 1, 1], LineBox[{98, 99}]}, 
        {Hue[0.5133709156062596, 1, 1], LineBox[{99, 100}]}, 
        {Hue[0.4173825756025821, 1, 1], LineBox[{100, 101}]}, 
        {Hue[0.32136569104301854`, 1, 1], LineBox[{101, 102}]}, 
        {Hue[0.22536565076658127`, 1, 1], LineBox[{102, 103}]}, 
        {Hue[0.12942796312066054`, 1, 1], LineBox[{103, 104}]}, 
        {Hue[0.03360344686736547, 1, 1], LineBox[{104, 105}]}, 
        {Hue[0.9379393315284854, 1, 1], LineBox[{105, 106}]}, 
        {Hue[0.8424879868571606, 1, 1], LineBox[{106, 107}]}, 
        {Hue[0.7473036316593907, 1, 1], LineBox[{107, 108}]}, 
        {Hue[0.6524441982990108, 1, 1], LineBox[{108, 109}]}, 
        {Hue[0.5579734697208485, 1, 1], LineBox[{109, 110}]}, 
        {Hue[0.46395883956449696`, 1, 1], LineBox[{110, 111}]}, 
        {Hue[0.3704781574462508, 1, 1], LineBox[{111, 112}]}, 
        {Hue[0.2776173958945541, 1, 1], LineBox[{112, 113}]}, 
        {Hue[0.18547449142920566`, 1, 1], LineBox[{113, 114}]}, 
        {Hue[0.09415329417638389, 1, 1], LineBox[{114, 115}]}, 
        {Hue[0.003775040291353455, 1, 1], LineBox[{115, 116}]}, 
        {Hue[0.9144694333143899, 1, 1], LineBox[{116, 117}]}, 
        {Hue[0.8263845311262752, 1, 1], LineBox[{117, 118}]}, 
        {Hue[0.7396779689740469, 1, 1], LineBox[{118, 119}]}, 
        {Hue[0.6545208857676232, 1, 1], LineBox[{119, 120}]}, 
        {Hue[0.5710974969169937, 1, 1], LineBox[{120, 121}]}, 
        {Hue[0.4895946637982166, 1, 1], LineBox[{121, 122}]}, 
        {Hue[0.4102067570615117, 1, 1], LineBox[{122, 123}]}, 
        {Hue[0.3331259132411515, 1, 1], LineBox[{123, 124}]}, 
        {Hue[0.2585394496739037, 1, 1], LineBox[{124, 125}]}, 
        {Hue[0.18662342332781198`, 1, 1], LineBox[{125, 126}]}, 
        {Hue[0.11753800147794824`, 1, 1], LineBox[{126, 127}]}, 
        {Hue[0.051427112466138326`, 1, 1], LineBox[{127, 128}]}, 
        {Hue[0.9884129955448848, 1, 1], LineBox[{128, 129}]}, 
        {Hue[0.9285892300665948, 1, 1], LineBox[{129, 130}]}, 
        {Hue[0.8720256023365494, 1, 1], LineBox[{130, 131}]}, 
        {Hue[0.8187663658254357, 1, 1], LineBox[{131, 132}]}, 
        {Hue[0.7688265810123591, 1, 1], LineBox[{132, 133}]}, 
        {Hue[0.7221955842685372, 1, 1], LineBox[{133, 134}]}, 
        {Hue[0.6788437009263398, 1, 1], LineBox[{134, 135}]}, 
        {Hue[0.6387174301250554, 1, 1], LineBox[{135, 136}]}, 
        {Hue[0.6017459196746494, 1, 1], LineBox[{136, 137}]}, 
        {Hue[0.5678475644593504, 1, 1], LineBox[{137, 138}]}, 
        {Hue[0.5369273311968021, 1, 1], LineBox[{138, 139}]}, 
        {Hue[0.5088816899028001, 1, 1], LineBox[{139, 140}]}, 
        {Hue[0.48360528170390377`, 1, 1], LineBox[{140, 141}]}, 
        {Hue[0.46098584079126487`, 1, 1], LineBox[{141, 142}]}, 
        {Hue[0.4409134101930792, 1, 1], LineBox[{142, 143}]}, 
        {Hue[0.42327813781734536`, 1, 1], LineBox[{143, 144}]}, 
        {Hue[0.40797200785093196`, 1, 1], LineBox[{144, 145}]}, 
        {Hue[0.39489008560322125`, 1, 1], LineBox[{145, 146}]}, 
        {Hue[0.38393221617634643`, 1, 1], LineBox[{146, 147}]}, 
        {Hue[0.37500196058241536`, 1, 1], LineBox[{147, 148}]}, 
        {Hue[0.368008847957833, 1, 1], LineBox[{148, 149}]}, 
        {Hue[0.3628666282170082, 1, 1], LineBox[{149, 150}]}, 
        {Hue[0.3594945306893157, 1, 1], LineBox[{150, 151}]}, 
        {Hue[0.35781626459383525`, 1, 1], LineBox[{151, 152}]}, 
        {Hue[0.35775992995884326`, 1, 1], LineBox[{152, 153}]}, 
        {Hue[0.35926092258923653`, 1, 1], LineBox[{153, 154}]}, 
        {Hue[0.3622564401320628, 1, 1], LineBox[{154, 155}]}, 
        {Hue[0.3666882794674252, 1, 1], LineBox[{155, 156}]}, 
        {Hue[0.3725045100663471, 1, 1], LineBox[{156, 157}]}, 
        {Hue[0.3796554784319523, 1, 1], LineBox[{157, 158}]}, 
        {Hue[0.3880942947049093, 1, 1], LineBox[{158, 159}]}, 
        {Hue[0.3977782989289779, 1, 1], LineBox[{159, 160}]}, 
        {Hue[0.40866872513605323`, 1, 1], LineBox[{160, 161}]}, 
        {Hue[0.42072856583263124`, 1, 1], LineBox[{161, 162}]}, 
        {Hue[0.4339235440994428, 1, 1], LineBox[{162, 163}]}, 
        {Hue[0.4482219112825864, 1, 1], LineBox[{163, 164}]}, 
        {Hue[0.46359477563959944`, 1, 1], LineBox[{164, 165}]}, 
        {Hue[0.48001457607847264`, 1, 1], LineBox[{165, 166}]}, 
        {Hue[0.4974556703406297, 1, 1], LineBox[{166, 167}]}, 
        {Hue[0.5158939565017163, 1, 1], LineBox[{167, 168}]}, 
        {Hue[0.5353076938087875, 1, 1], LineBox[{168, 169}]}, 
        {Hue[0.5556757445982718, 1, 1], LineBox[{169, 170}]}, 
        {Hue[0.5769792836174885, 1, 1], LineBox[{170, 171}]}, 
        {Hue[0.5991999701653299, 1, 1], LineBox[{171, 172}]}, 
        {Hue[0.6223206152611922, 1, 1], LineBox[{172, 173}]}, 
        {Hue[0.646325446642212, 1, 1], LineBox[{173, 174}]}, 
        {Hue[0.6711998264592366, 1, 1], LineBox[{174, 175}]}, 
        {Hue[0.6969297314660567, 1, 1], LineBox[{175, 176}]}, 
        {Hue[0.7235023463562673, 1, 1], LineBox[{176, 177}]}, 
        {Hue[0.7509043991326707, 1, 1], LineBox[{177, 178}]}, 
        {Hue[0.7791243549562274, 1, 1], LineBox[{178, 179}]}, 
        {Hue[0.8081515576934792, 1, 1], LineBox[{179, 180}]}, 
        {Hue[0.8379740999023698, 1, 1], LineBox[{180, 181}]}, 
        {Hue[0.8685825170069534, 1, 1], LineBox[{181, 182}]}, 
        {Hue[0.8999669535324006, 1, 1], LineBox[{182, 183}]}, 
        {Hue[0.9321197069880646, 1, 1], LineBox[{183, 184}]}, 
        {Hue[0.9650299290140577, 1, 1], LineBox[{184, 185}]}, 
        {Hue[0.9986909188738518, 1, 1], LineBox[{185, 186}]}, 
        {Hue[0.033094624152977085`, 1, 1], LineBox[{186, 187}]}, 
        {Hue[0.06823373640233694, 1, 1], LineBox[{187, 188}]}, 
        {Hue[0.10410062095224143`, 1, 1], LineBox[{188, 189}]}, 
        {Hue[0.14068810334390153`, 1, 1], LineBox[{189, 190}]}, 
        {Hue[0.17799007885133333`, 1, 1], LineBox[{190, 191}]}, 
        {Hue[0.21599943175394315`, 1, 1], LineBox[{191, 192}]}, 
        {Hue[0.25471007336292956`, 1, 1], LineBox[{192, 193}]}, 
        {Hue[0.2941159403636716, 1, 1], LineBox[{193, 194}]}, 
        {Hue[0.33421239620143495`, 1, 1], LineBox[{194, 195}]}, 
        {Hue[0.3749927116198588, 1, 1], LineBox[{195, 196}]}, 
        {Hue[0.416453227649966, 1, 1], LineBox[{196, 197}]}, 
        {Hue[0.458587613960305, 1, 1], LineBox[{197, 198}]}, 
        {Hue[0.5013919317739264, 1, 1], LineBox[{198, 199}]}, 
        {Hue[0.5448606766593951, 1, 1], LineBox[{199, 200}]}, 
        {Hue[0.5889888658916277, 1, 1], LineBox[{200, 201}]}, 
        {Hue[0.6337722749365959, 1, 1], LineBox[{201, 202}]}, 
        {Hue[0.67920575497398, 1, 1], LineBox[{202, 203}]}, 
        {Hue[0.7252846301057936, 1, 1], LineBox[{203, 204}]}, 
        {Hue[0.7720062490737557, 1, 1], LineBox[{204, 205}]}, 
        {Hue[0.8193656677120444, 1, 1], LineBox[{205, 206}]}, 
        {Hue[0.8673604139213364, 1, 1], LineBox[{206, 207}]}, 
        {Hue[0.9159854061521892, 1, 1], LineBox[{207, 208}]}, 
        {Hue[0.965236857420268, 1, 1], LineBox[{208, 209}]}, 
        {Hue[0.015112380299624755`, 1, 1], LineBox[{209, 210}]}, 
        {Hue[0.0656073763500262, 1, 1], LineBox[{210, 211}]}, 
        {Hue[0.11671944049250602`, 1, 1], LineBox[{211, 212}]}, 
        {Hue[0.16844370339115078`, 1, 1], LineBox[{212, 213}]}, 
        {Hue[0.22077839817692413`, 1, 1], LineBox[{213, 214}]}, 
        {Hue[0.273719747035185, 1, 1], LineBox[{214, 215}]}, 
        {Hue[0.3272645018509859, 1, 1], LineBox[{215, 216}]}, 
        {Hue[0.3814105632880518, 1, 1], LineBox[{216, 217}]}, 
        {Hue[0.4361535802050153, 1, 1], LineBox[{217, 218}]}, 
        {Hue[0.4914914479566433, 1, 1], LineBox[{218, 219}]}, 
        {Hue[0.5474209272105655, 1, 1], LineBox[{219, 220}]}, 
        {Hue[0.6039401865467807, 1, 1], LineBox[{220, 221}]}, 
        {Hue[0.6610450494096778, 1, 1], LineBox[{221, 222}]}, 
        {Hue[0.7187346323562827, 1, 1], LineBox[{222, 223}]}, 
        {Hue[0.7770045221176122, 1, 1], LineBox[{223, 224}]}, 
        {Hue[0.8358535452751399, 1, 1], LineBox[{224, 225}]}, 
        {Hue[0.8952789269389672, 1, 1], LineBox[{225, 226}]}, 
        {Hue[0.955278384025335, 1, 1], LineBox[{226, 227}]}, 
        {Hue[0.015848820485411037`, 1, 1], LineBox[{227, 228}]}, 
        {Hue[0.07698831686773512, 1, 1], LineBox[{228, 229}]}, 
        {Hue[0.1386951331217639, 1, 1], LineBox[{229, 230}]}, 
        {Hue[0.2009663078345991, 1, 1], LineBox[{230, 231}]}, 
        {Hue[0.2637996869744032, 1, 1], LineBox[{231, 232}]}, 
        {Hue[0.32719312809287177`, 1, 1], LineBox[{232, 233}]}, 
        {Hue[0.39114533126753376`, 1, 1], LineBox[{233, 234}]}, 
        {Hue[0.455652644567371, 1, 1], LineBox[{234, 235}]}, 
        {Hue[0.5207154494556953, 1, 1], LineBox[{235, 236}]}, 
        {Hue[0.5863291466008134, 1, 1], LineBox[{236, 237}]}, 
        {Hue[0.6524927029187558, 1, 1], LineBox[{237, 238}]}, 
        {Hue[0.7192051438604465, 1, 1], LineBox[{238, 239}]}, 
        {Hue[0.7864639377353686, 1, 1], LineBox[{239, 240}]}, 
        {Hue[0.854266613804419, 1, 1], LineBox[{240, 241}]}, 
        {Hue[0.9226132153595316, 1, 1], LineBox[{241, 242}]}, 
        {Hue[0.9914990995606757, 1, 1], LineBox[{242, 243}]}, 
        {Hue[0.06092463240056644, 1, 1], LineBox[{243, 244}]}, 
        {Hue[0.13088683050341132`, 1, 1], LineBox[{244, 245}]}, 
        {Hue[0.20138520126493187`, 1, 1], LineBox[{245, 246}]}, 
        {Hue[0.27241755097104603`, 1, 1], LineBox[{246, 247}]}, 
        {Hue[0.34398228203464143`, 1, 1], LineBox[{247, 248}]}, 
        {Hue[0.4160792472892838, 1, 1], LineBox[{248, 249}]}, 
        {Hue[0.4887026604895621, 1, 1], LineBox[{249, 250}]}, 
        {Hue[0.5618548264506966, 1, 1], LineBox[{250, 251}]}}}},
     VertexColors->{
       Hue[0.5373228843195937, 1, 1], 
       Hue[0.4403401308194928, 1, 1], 
       Hue[0.3435613420702442, 1, 1], 
       Hue[0.24690243660616393`, 1, 1], 
       Hue[0.150454945879873, 1, 1], 
       Hue[0.054253709597173946`, 1, 1], 
       Hue[0.9582329689095465, 1, 1], 
       Hue[0.8624154616652879, 1, 1], 
       Hue[0.7667920547714719, 1, 1], 
       Hue[0.6713539032786822, 1, 1], 
       Hue[0.5761322441426646, 1, 1], 
       Hue[0.4810681788903602, 1, 1], 
       Hue[0.38618845496889015`, 1, 1], 
       Hue[0.2914937307005535, 1, 1], 
       Hue[0.19699709955287387`, 1, 1], 
       Hue[0.1026633796553843, 1, 1], 
       Hue[0.008503512405973614, 1, 1], 
       Hue[0.9145316811967742, 1, 1], 
       Hue[0.8207243806703278, 1, 1], 
       Hue[0.7270951531531911, 1, 1], 
       Hue[0.6336280106405164, 1, 1], 
       Hue[0.5403238375965598, 1, 1], 
       Hue[0.44719515461189513`, 1, 1], 
       Hue[0.3542219104665249, 1, 1], 
       Hue[0.261406034757453, 1, 1], 
       Hue[0.16874846532010704`, 1, 1], 
       Hue[0.07625049917109356, 1, 1], 
       Hue[0.983898824560589, 1, 1], 
       Hue[0.8917010932212673, 1, 1], 
       Hue[0.7996520993560525, 1, 1], 
       Hue[0.707743391289746, 1, 1], 
       Hue[0.6159813669193629, 1, 1], 
       Hue[0.5243603446357614, 1, 1], 
       Hue[0.4328738691798968, 1, 1], 
       Hue[0.3415208785271995, 1, 1], 
       Hue[0.2502971034620979, 1, 1], 
       Hue[0.15920419790548215`, 1, 1], 
       Hue[0.06823149246682571, 1, 1], 
       Hue[0.9773857509141324, 1, 1], 
       Hue[0.8866550914096966, 1, 1], 
       Hue[0.7960381183778849, 1, 1], 
       Hue[0.7055349171841978, 1, 1], 
       Hue[0.6151380108443617, 1, 1], 
       Hue[0.5248452588742869, 1, 1], 
       Hue[0.4346517215644478, 1, 1], 
       Hue[0.3445541576573799, 1, 1], 
       Hue[0.2545477968642205, 1, 1], 
       Hue[0.16462741410865095`, 1, 1], 
       Hue[0.07479106859346756, 1, 1], 
       Hue[0.9850329040069344, 1, 1], 
       Hue[0.8953473747186199, 1, 1], 
       Hue[0.8057307072880856, 1, 1], 
       Hue[0.7161762779604128, 1, 1], 
       Hue[0.6266791615905233, 1, 1], 
       Hue[0.537233630987699, 1, 1], 
       Hue[0.44783486011628426`, 1, 1], 
       Hue[0.35847490168618695`, 1, 1], 
       Hue[0.2691480839616139, 1, 1], 
       Hue[0.17984719749350703`, 1, 1], 
       Hue[0.0905655025466636, 1, 1], 
       Hue[0.0012950694149223545`, 1, 1], 
       Hue[0.9120293898934388, 1, 1], 
       Hue[0.8227595802498255, 1, 1], 
       Hue[0.7334770677884576, 1, 1], 
       Hue[0.6441724629825096, 1, 1], 
       Hue[0.5548371466919941, 1, 1], 
       Hue[0.465461449193391, 1, 1], 
       Hue[0.376034319947191, 1, 1], 
       Hue[0.2865456196687959, 1, 1], 
       Hue[0.19698327643107333`, 1, 1], 
       Hue[0.1073368478949821, 1, 1], 
       Hue[0.01759296362047231, 1, 1], 
       Hue[0.9277393289210646, 1, 1], 
       Hue[0.8377620797664849, 1, 1], 
       Hue[0.7476481320246258, 1, 1], 
       Hue[0.6573829603765364, 1, 1], 
       Hue[0.5669519357335853, 1, 1], 
       Hue[0.47633986833715763`, 1, 1], 
       Hue[0.3855322472772245, 1, 1], 
       Hue[0.29451264153189877`, 1, 1], 
       Hue[0.2032663941206125, 1, 1], 
       Hue[0.1117779759296776, 1, 1], 
       Hue[0.020033942761572048`, 1, 1], 
       Hue[0.9280205709389144, 1, 1], 
       Hue[0.8357247215490928, 1, 1], 
       Hue[0.7431357669534746, 1, 1], 
       Hue[0.6502444277652975, 1, 1], 
       Hue[0.5570441913618924, 1, 1], 
       Hue[0.4635317224138139, 1, 1], 
       Hue[0.36970776965283664`, 1, 1], 
       Hue[0.2755740273533197, 1, 1], 
       Hue[0.18114110308239148`, 1, 1], 
       Hue[0.08642018714458502, 1, 1], 
       Hue[0.991429466836158, 1, 1], 
       Hue[0.8961918398709002, 1, 1], 
       Hue[0.8007328384798726, 1, 1], 
       Hue[0.7050864037553836, 1, 1], 
       Hue[0.609285542248224, 1, 1], 
       Hue[0.5133709156062596, 1, 1], 
       Hue[0.4173825756025821, 1, 1], 
       Hue[0.32136569104301854`, 1, 1], 
       Hue[0.22536565076658127`, 1, 1], 
       Hue[0.12942796312066054`, 1, 1], 
       Hue[0.03360344686736547, 1, 1], 
       Hue[0.9379393315284854, 1, 1], 
       Hue[0.8424879868571606, 1, 1], 
       Hue[0.7473036316593907, 1, 1], 
       Hue[0.6524441982990108, 1, 1], 
       Hue[0.5579734697208485, 1, 1], 
       Hue[0.46395883956449696`, 1, 1], 
       Hue[0.3704781574462508, 1, 1], 
       Hue[0.2776173958945541, 1, 1], 
       Hue[0.18547449142920566`, 1, 1], 
       Hue[0.09415329417638389, 1, 1], 
       Hue[0.003775040291353455, 1, 1], 
       Hue[0.9144694333143899, 1, 1], 
       Hue[0.8263845311262752, 1, 1], 
       Hue[0.7396779689740469, 1, 1], 
       Hue[0.6545208857676232, 1, 1], 
       Hue[0.5710974969169937, 1, 1], 
       Hue[0.4895946637982166, 1, 1], 
       Hue[0.4102067570615117, 1, 1], 
       Hue[0.3331259132411515, 1, 1], 
       Hue[0.2585394496739037, 1, 1], 
       Hue[0.18662342332781198`, 1, 1], 
       Hue[0.11753800147794824`, 1, 1], 
       Hue[0.051427112466138326`, 1, 1], 
       Hue[0.9884129955448848, 1, 1], 
       Hue[0.9285892300665948, 1, 1], 
       Hue[0.8720256023365494, 1, 1], 
       Hue[0.8187663658254357, 1, 1], 
       Hue[0.7688265810123591, 1, 1], 
       Hue[0.7221955842685372, 1, 1], 
       Hue[0.6788437009263398, 1, 1], 
       Hue[0.6387174301250554, 1, 1], 
       Hue[0.6017459196746494, 1, 1], 
       Hue[0.5678475644593504, 1, 1], 
       Hue[0.5369273311968021, 1, 1], 
       Hue[0.5088816899028001, 1, 1], 
       Hue[0.48360528170390377`, 1, 1], 
       Hue[0.46098584079126487`, 1, 1], 
       Hue[0.4409134101930792, 1, 1], 
       Hue[0.42327813781734536`, 1, 1], 
       Hue[0.40797200785093196`, 1, 1], 
       Hue[0.39489008560322125`, 1, 1], 
       Hue[0.38393221617634643`, 1, 1], 
       Hue[0.37500196058241536`, 1, 1], 
       Hue[0.368008847957833, 1, 1], 
       Hue[0.3628666282170082, 1, 1], 
       Hue[0.3594945306893157, 1, 1], 
       Hue[0.35781626459383525`, 1, 1], 
       Hue[0.35775992995884326`, 1, 1], 
       Hue[0.35926092258923653`, 1, 1], 
       Hue[0.3622564401320628, 1, 1], 
       Hue[0.3666882794674252, 1, 1], 
       Hue[0.3725045100663471, 1, 1], 
       Hue[0.3796554784319523, 1, 1], 
       Hue[0.3880942947049093, 1, 1], 
       Hue[0.3977782989289779, 1, 1], 
       Hue[0.40866872513605323`, 1, 1], 
       Hue[0.42072856583263124`, 1, 1], 
       Hue[0.4339235440994428, 1, 1], 
       Hue[0.4482219112825864, 1, 1], 
       Hue[0.46359477563959944`, 1, 1], 
       Hue[0.48001457607847264`, 1, 1], 
       Hue[0.4974556703406297, 1, 1], 
       Hue[0.5158939565017163, 1, 1], 
       Hue[0.5353076938087875, 1, 1], 
       Hue[0.5556757445982718, 1, 1], 
       Hue[0.5769792836174885, 1, 1], 
       Hue[0.5991999701653299, 1, 1], 
       Hue[0.6223206152611922, 1, 1], 
       Hue[0.646325446642212, 1, 1], 
       Hue[0.6711998264592366, 1, 1], 
       Hue[0.6969297314660567, 1, 1], 
       Hue[0.7235023463562673, 1, 1], 
       Hue[0.7509043991326707, 1, 1], 
       Hue[0.7791243549562274, 1, 1], 
       Hue[0.8081515576934792, 1, 1], 
       Hue[0.8379740999023698, 1, 1], 
       Hue[0.8685825170069534, 1, 1], 
       Hue[0.8999669535324006, 1, 1], 
       Hue[0.9321197069880646, 1, 1], 
       Hue[0.9650299290140577, 1, 1], 
       Hue[0.9986909188738518, 1, 1], 
       Hue[0.033094624152977085`, 1, 1], 
       Hue[0.06823373640233694, 1, 1], 
       Hue[0.10410062095224143`, 1, 1], 
       Hue[0.14068810334390153`, 1, 1], 
       Hue[0.17799007885133333`, 1, 1], 
       Hue[0.21599943175394315`, 1, 1], 
       Hue[0.25471007336292956`, 1, 1], 
       Hue[0.2941159403636716, 1, 1], 
       Hue[0.33421239620143495`, 1, 1], 
       Hue[0.3749927116198588, 1, 1], 
       Hue[0.416453227649966, 1, 1], 
       Hue[0.458587613960305, 1, 1], 
       Hue[0.5013919317739264, 1, 1], 
       Hue[0.5448606766593951, 1, 1], 
       Hue[0.5889888658916277, 1, 1], 
       Hue[0.6337722749365959, 1, 1], 
       Hue[0.67920575497398, 1, 1], 
       Hue[0.7252846301057936, 1, 1], 
       Hue[0.7720062490737557, 1, 1], 
       Hue[0.8193656677120444, 1, 1], 
       Hue[0.8673604139213364, 1, 1], 
       Hue[0.9159854061521892, 1, 1], 
       Hue[0.965236857420268, 1, 1], 
       Hue[0.015112380299624755`, 1, 1], 
       Hue[0.0656073763500262, 1, 1], 
       Hue[0.11671944049250602`, 1, 1], 
       Hue[0.16844370339115078`, 1, 1], 
       Hue[0.22077839817692413`, 1, 1], 
       Hue[0.273719747035185, 1, 1], 
       Hue[0.3272645018509859, 1, 1], 
       Hue[0.3814105632880518, 1, 1], 
       Hue[0.4361535802050153, 1, 1], 
       Hue[0.4914914479566433, 1, 1], 
       Hue[0.5474209272105655, 1, 1], 
       Hue[0.6039401865467807, 1, 1], 
       Hue[0.6610450494096778, 1, 1], 
       Hue[0.7187346323562827, 1, 1], 
       Hue[0.7770045221176122, 1, 1], 
       Hue[0.8358535452751399, 1, 1], 
       Hue[0.8952789269389672, 1, 1], 
       Hue[0.955278384025335, 1, 1], 
       Hue[0.015848820485411037`, 1, 1], 
       Hue[0.07698831686773512, 1, 1], 
       Hue[0.1386951331217639, 1, 1], 
       Hue[0.2009663078345991, 1, 1], 
       Hue[0.2637996869744032, 1, 1], 
       Hue[0.32719312809287177`, 1, 1], 
       Hue[0.39114533126753376`, 1, 1], 
       Hue[0.455652644567371, 1, 1], 
       Hue[0.5207154494556953, 1, 1], 
       Hue[0.5863291466008134, 1, 1], 
       Hue[0.6524927029187558, 1, 1], 
       Hue[0.7192051438604465, 1, 1], 
       Hue[0.7864639377353686, 1, 1], 
       Hue[0.854266613804419, 1, 1], 
       Hue[0.9226132153595316, 1, 1], 
       Hue[0.9914990995606757, 1, 1], 
       Hue[0.06092463240056644, 1, 1], 
       Hue[0.13088683050341132`, 1, 1], 
       Hue[0.20138520126493187`, 1, 1], 
       Hue[0.27241755097104603`, 1, 1], 
       Hue[0.34398228203464143`, 1, 1], 
       Hue[0.4160792472892838, 1, 1], 
       Hue[0.4887026604895621, 1, 1], 
       Hue[0.5618548264506966, 1, 1], 
       Hue[0.6355323297586595, 1, 1], 
       Hue[0.5373228843195937, 1, 1], 
       Hue[0.4403401308194928, 1, 1], 
       Hue[0.4403401308194928, 1, 1], 
       Hue[0.3435613420702442, 1, 1], 
       Hue[0.3435613420702442, 1, 1], 
       Hue[0.24690243660616393`, 1, 1], 
       Hue[0.24690243660616393`, 1, 1], 
       Hue[0.150454945879873, 1, 1], 
       Hue[0.150454945879873, 1, 1], 
       Hue[0.054253709597173946`, 1, 1], 
       Hue[0.054253709597173946`, 1, 1], 
       Hue[0.9582329689095465, 1, 1], 
       Hue[0.9582329689095465, 1, 1], 
       Hue[0.8624154616652879, 1, 1], 
       Hue[0.8624154616652879, 1, 1], 
       Hue[0.7667920547714719, 1, 1], 
       Hue[0.7667920547714719, 1, 1], 
       Hue[0.6713539032786822, 1, 1], 
       Hue[0.6713539032786822, 1, 1], 
       Hue[0.5761322441426646, 1, 1], 
       Hue[0.5761322441426646, 1, 1], 
       Hue[0.4810681788903602, 1, 1], 
       Hue[0.4810681788903602, 1, 1], 
       Hue[0.38618845496889015`, 1, 1], 
       Hue[0.38618845496889015`, 1, 1], 
       Hue[0.2914937307005535, 1, 1], 
       Hue[0.2914937307005535, 1, 1], 
       Hue[0.19699709955287387`, 1, 1], 
       Hue[0.19699709955287387`, 1, 1], 
       Hue[0.1026633796553843, 1, 1], 
       Hue[0.1026633796553843, 1, 1], 
       Hue[0.008503512405973614, 1, 1], 
       Hue[0.008503512405973614, 1, 1], 
       Hue[0.9145316811967742, 1, 1], 
       Hue[0.9145316811967742, 1, 1], 
       Hue[0.8207243806703278, 1, 1], 
       Hue[0.8207243806703278, 1, 1], 
       Hue[0.7270951531531911, 1, 1], 
       Hue[0.7270951531531911, 1, 1], 
       Hue[0.6336280106405164, 1, 1], 
       Hue[0.6336280106405164, 1, 1], 
       Hue[0.5403238375965598, 1, 1], 
       Hue[0.5403238375965598, 1, 1], 
       Hue[0.44719515461189513`, 1, 1], 
       Hue[0.44719515461189513`, 1, 1], 
       Hue[0.3542219104665249, 1, 1], 
       Hue[0.3542219104665249, 1, 1], 
       Hue[0.261406034757453, 1, 1], 
       Hue[0.261406034757453, 1, 1], 
       Hue[0.16874846532010704`, 1, 1], 
       Hue[0.16874846532010704`, 1, 1], 
       Hue[0.07625049917109356, 1, 1], 
       Hue[0.07625049917109356, 1, 1], 
       Hue[0.983898824560589, 1, 1], 
       Hue[0.983898824560589, 1, 1], 
       Hue[0.8917010932212673, 1, 1], 
       Hue[0.8917010932212673, 1, 1], 
       Hue[0.7996520993560525, 1, 1], 
       Hue[0.7996520993560525, 1, 1], 
       Hue[0.707743391289746, 1, 1], 
       Hue[0.707743391289746, 1, 1], 
       Hue[0.6159813669193629, 1, 1], 
       Hue[0.6159813669193629, 1, 1], 
       Hue[0.5243603446357614, 1, 1], 
       Hue[0.5243603446357614, 1, 1], 
       Hue[0.4328738691798968, 1, 1], 
       Hue[0.4328738691798968, 1, 1], 
       Hue[0.3415208785271995, 1, 1], 
       Hue[0.3415208785271995, 1, 1], 
       Hue[0.2502971034620979, 1, 1], 
       Hue[0.2502971034620979, 1, 1], 
       Hue[0.15920419790548215`, 1, 1], 
       Hue[0.15920419790548215`, 1, 1], 
       Hue[0.06823149246682571, 1, 1], 
       Hue[0.06823149246682571, 1, 1], 
       Hue[0.9773857509141324, 1, 1], 
       Hue[0.9773857509141324, 1, 1], 
       Hue[0.8866550914096966, 1, 1], 
       Hue[0.8866550914096966, 1, 1], 
       Hue[0.7960381183778849, 1, 1], 
       Hue[0.7960381183778849, 1, 1], 
       Hue[0.7055349171841978, 1, 1], 
       Hue[0.7055349171841978, 1, 1], 
       Hue[0.6151380108443617, 1, 1], 
       Hue[0.6151380108443617, 1, 1], 
       Hue[0.5248452588742869, 1, 1], 
       Hue[0.5248452588742869, 1, 1], 
       Hue[0.4346517215644478, 1, 1], 
       Hue[0.4346517215644478, 1, 1], 
       Hue[0.3445541576573799, 1, 1], 
       Hue[0.3445541576573799, 1, 1], 
       Hue[0.2545477968642205, 1, 1], 
       Hue[0.2545477968642205, 1, 1], 
       Hue[0.16462741410865095`, 1, 1], 
       Hue[0.16462741410865095`, 1, 1], 
       Hue[0.07479106859346756, 1, 1], 
       Hue[0.07479106859346756, 1, 1], 
       Hue[0.9850329040069344, 1, 1], 
       Hue[0.9850329040069344, 1, 1], 
       Hue[0.8953473747186199, 1, 1], 
       Hue[0.8953473747186199, 1, 1], 
       Hue[0.8057307072880856, 1, 1], 
       Hue[0.8057307072880856, 1, 1], 
       Hue[0.7161762779604128, 1, 1], 
       Hue[0.7161762779604128, 1, 1], 
       Hue[0.6266791615905233, 1, 1], 
       Hue[0.6266791615905233, 1, 1], 
       Hue[0.537233630987699, 1, 1], 
       Hue[0.537233630987699, 1, 1], 
       Hue[0.44783486011628426`, 1, 1], 
       Hue[0.44783486011628426`, 1, 1], 
       Hue[0.35847490168618695`, 1, 1], 
       Hue[0.35847490168618695`, 1, 1], 
       Hue[0.2691480839616139, 1, 1], 
       Hue[0.2691480839616139, 1, 1], 
       Hue[0.17984719749350703`, 1, 1], 
       Hue[0.17984719749350703`, 1, 1], 
       Hue[0.0905655025466636, 1, 1], 
       Hue[0.0905655025466636, 1, 1], 
       Hue[0.0012950694149223545`, 1, 1], 
       Hue[0.0012950694149223545`, 1, 1], 
       Hue[0.9120293898934388, 1, 1], 
       Hue[0.9120293898934388, 1, 1], 
       Hue[0.8227595802498255, 1, 1], 
       Hue[0.8227595802498255, 1, 1], 
       Hue[0.7334770677884576, 1, 1], 
       Hue[0.7334770677884576, 1, 1], 
       Hue[0.6441724629825096, 1, 1], 
       Hue[0.6441724629825096, 1, 1], 
       Hue[0.5548371466919941, 1, 1], 
       Hue[0.5548371466919941, 1, 1], 
       Hue[0.465461449193391, 1, 1], 
       Hue[0.465461449193391, 1, 1], 
       Hue[0.376034319947191, 1, 1], 
       Hue[0.376034319947191, 1, 1], 
       Hue[0.2865456196687959, 1, 1], 
       Hue[0.2865456196687959, 1, 1], 
       Hue[0.19698327643107333`, 1, 1], 
       Hue[0.19698327643107333`, 1, 1], 
       Hue[0.1073368478949821, 1, 1], 
       Hue[0.1073368478949821, 1, 1], 
       Hue[0.01759296362047231, 1, 1], 
       Hue[0.01759296362047231, 1, 1], 
       Hue[0.9277393289210646, 1, 1], 
       Hue[0.9277393289210646, 1, 1], 
       Hue[0.8377620797664849, 1, 1], 
       Hue[0.8377620797664849, 1, 1], 
       Hue[0.7476481320246258, 1, 1], 
       Hue[0.7476481320246258, 1, 1], 
       Hue[0.6573829603765364, 1, 1], 
       Hue[0.6573829603765364, 1, 1], 
       Hue[0.5669519357335853, 1, 1], 
       Hue[0.5669519357335853, 1, 1], 
       Hue[0.47633986833715763`, 1, 1], 
       Hue[0.47633986833715763`, 1, 1], 
       Hue[0.3855322472772245, 1, 1], 
       Hue[0.3855322472772245, 1, 1], 
       Hue[0.29451264153189877`, 1, 1], 
       Hue[0.29451264153189877`, 1, 1], 
       Hue[0.2032663941206125, 1, 1], 
       Hue[0.2032663941206125, 1, 1], 
       Hue[0.1117779759296776, 1, 1], 
       Hue[0.1117779759296776, 1, 1], 
       Hue[0.020033942761572048`, 1, 1], 
       Hue[0.020033942761572048`, 1, 1], 
       Hue[0.9280205709389144, 1, 1], 
       Hue[0.9280205709389144, 1, 1], 
       Hue[0.8357247215490928, 1, 1], 
       Hue[0.8357247215490928, 1, 1], 
       Hue[0.7431357669534746, 1, 1], 
       Hue[0.7431357669534746, 1, 1], 
       Hue[0.6502444277652975, 1, 1], 
       Hue[0.6502444277652975, 1, 1], 
       Hue[0.5570441913618924, 1, 1], 
       Hue[0.5570441913618924, 1, 1], 
       Hue[0.4635317224138139, 1, 1], 
       Hue[0.4635317224138139, 1, 1], 
       Hue[0.36970776965283664`, 1, 1], 
       Hue[0.36970776965283664`, 1, 1], 
       Hue[0.2755740273533197, 1, 1], 
       Hue[0.2755740273533197, 1, 1], 
       Hue[0.18114110308239148`, 1, 1], 
       Hue[0.18114110308239148`, 1, 1], 
       Hue[0.08642018714458502, 1, 1], 
       Hue[0.08642018714458502, 1, 1], 
       Hue[0.991429466836158, 1, 1], 
       Hue[0.991429466836158, 1, 1], 
       Hue[0.8961918398709002, 1, 1], 
       Hue[0.8961918398709002, 1, 1], 
       Hue[0.8007328384798726, 1, 1], 
       Hue[0.8007328384798726, 1, 1], 
       Hue[0.7050864037553836, 1, 1], 
       Hue[0.7050864037553836, 1, 1], 
       Hue[0.609285542248224, 1, 1], 
       Hue[0.609285542248224, 1, 1], 
       Hue[0.5133709156062596, 1, 1], 
       Hue[0.5133709156062596, 1, 1], 
       Hue[0.4173825756025821, 1, 1], 
       Hue[0.4173825756025821, 1, 1], 
       Hue[0.32136569104301854`, 1, 1], 
       Hue[0.32136569104301854`, 1, 1], 
       Hue[0.22536565076658127`, 1, 1], 
       Hue[0.22536565076658127`, 1, 1], 
       Hue[0.12942796312066054`, 1, 1], 
       Hue[0.12942796312066054`, 1, 1], 
       Hue[0.03360344686736547, 1, 1], 
       Hue[0.03360344686736547, 1, 1], 
       Hue[0.9379393315284854, 1, 1], 
       Hue[0.9379393315284854, 1, 1], 
       Hue[0.8424879868571606, 1, 1], 
       Hue[0.8424879868571606, 1, 1], 
       Hue[0.7473036316593907, 1, 1], 
       Hue[0.7473036316593907, 1, 1], 
       Hue[0.6524441982990108, 1, 1], 
       Hue[0.6524441982990108, 1, 1], 
       Hue[0.5579734697208485, 1, 1], 
       Hue[0.5579734697208485, 1, 1], 
       Hue[0.46395883956449696`, 1, 1], 
       Hue[0.46395883956449696`, 1, 1], 
       Hue[0.3704781574462508, 1, 1], 
       Hue[0.3704781574462508, 1, 1], 
       Hue[0.2776173958945541, 1, 1], 
       Hue[0.2776173958945541, 1, 1], 
       Hue[0.18547449142920566`, 1, 1], 
       Hue[0.18547449142920566`, 1, 1], 
       Hue[0.09415329417638389, 1, 1], 
       Hue[0.09415329417638389, 1, 1], 
       Hue[0.003775040291353455, 1, 1], 
       Hue[0.003775040291353455, 1, 1], 
       Hue[0.9144694333143899, 1, 1], 
       Hue[0.9144694333143899, 1, 1], 
       Hue[0.8263845311262752, 1, 1], 
       Hue[0.8263845311262752, 1, 1], 
       Hue[0.7396779689740469, 1, 1], 
       Hue[0.7396779689740469, 1, 1], 
       Hue[0.6545208857676232, 1, 1], 
       Hue[0.6545208857676232, 1, 1], 
       Hue[0.5710974969169937, 1, 1], 
       Hue[0.5710974969169937, 1, 1], 
       Hue[0.4895946637982166, 1, 1], 
       Hue[0.4895946637982166, 1, 1], 
       Hue[0.4102067570615117, 1, 1], 
       Hue[0.4102067570615117, 1, 1], 
       Hue[0.3331259132411515, 1, 1], 
       Hue[0.3331259132411515, 1, 1], 
       Hue[0.2585394496739037, 1, 1], 
       Hue[0.2585394496739037, 1, 1], 
       Hue[0.18662342332781198`, 1, 1], 
       Hue[0.18662342332781198`, 1, 1], 
       Hue[0.11753800147794824`, 1, 1], 
       Hue[0.11753800147794824`, 1, 1], 
       Hue[0.051427112466138326`, 1, 1], 
       Hue[0.051427112466138326`, 1, 1], 
       Hue[0.9884129955448848, 1, 1], 
       Hue[0.9884129955448848, 1, 1], 
       Hue[0.9285892300665948, 1, 1], 
       Hue[0.9285892300665948, 1, 1], 
       Hue[0.8720256023365494, 1, 1], 
       Hue[0.8720256023365494, 1, 1], 
       Hue[0.8187663658254357, 1, 1], 
       Hue[0.8187663658254357, 1, 1], 
       Hue[0.7688265810123591, 1, 1], 
       Hue[0.7688265810123591, 1, 1], 
       Hue[0.7221955842685372, 1, 1], 
       Hue[0.7221955842685372, 1, 1], 
       Hue[0.6788437009263398, 1, 1], 
       Hue[0.6788437009263398, 1, 1], 
       Hue[0.6387174301250554, 1, 1], 
       Hue[0.6387174301250554, 1, 1], 
       Hue[0.6017459196746494, 1, 1], 
       Hue[0.6017459196746494, 1, 1], 
       Hue[0.5678475644593504, 1, 1], 
       Hue[0.5678475644593504, 1, 1], 
       Hue[0.5369273311968021, 1, 1], 
       Hue[0.5369273311968021, 1, 1], 
       Hue[0.5088816899028001, 1, 1], 
       Hue[0.5088816899028001, 1, 1], 
       Hue[0.48360528170390377`, 1, 1], 
       Hue[0.48360528170390377`, 1, 1], 
       Hue[0.46098584079126487`, 1, 1], 
       Hue[0.46098584079126487`, 1, 1], 
       Hue[0.4409134101930792, 1, 1], 
       Hue[0.4409134101930792, 1, 1], 
       Hue[0.42327813781734536`, 1, 1], 
       Hue[0.42327813781734536`, 1, 1], 
       Hue[0.40797200785093196`, 1, 1], 
       Hue[0.40797200785093196`, 1, 1], 
       Hue[0.39489008560322125`, 1, 1], 
       Hue[0.39489008560322125`, 1, 1], 
       Hue[0.38393221617634643`, 1, 1], 
       Hue[0.38393221617634643`, 1, 1], 
       Hue[0.37500196058241536`, 1, 1], 
       Hue[0.37500196058241536`, 1, 1], 
       Hue[0.368008847957833, 1, 1], 
       Hue[0.368008847957833, 1, 1], 
       Hue[0.3628666282170082, 1, 1], 
       Hue[0.3628666282170082, 1, 1], 
       Hue[0.3594945306893157, 1, 1], 
       Hue[0.3594945306893157, 1, 1], 
       Hue[0.35781626459383525`, 1, 1], 
       Hue[0.35781626459383525`, 1, 1], 
       Hue[0.35775992995884326`, 1, 1], 
       Hue[0.35775992995884326`, 1, 1], 
       Hue[0.35926092258923653`, 1, 1], 
       Hue[0.35926092258923653`, 1, 1], 
       Hue[0.3622564401320628, 1, 1], 
       Hue[0.3622564401320628, 1, 1], 
       Hue[0.3666882794674252, 1, 1], 
       Hue[0.3666882794674252, 1, 1], 
       Hue[0.3725045100663471, 1, 1], 
       Hue[0.3725045100663471, 1, 1], 
       Hue[0.3796554784319523, 1, 1], 
       Hue[0.3796554784319523, 1, 1], 
       Hue[0.3880942947049093, 1, 1], 
       Hue[0.3880942947049093, 1, 1], 
       Hue[0.3977782989289779, 1, 1], 
       Hue[0.3977782989289779, 1, 1], 
       Hue[0.40866872513605323`, 1, 1], 
       Hue[0.40866872513605323`, 1, 1], 
       Hue[0.42072856583263124`, 1, 1], 
       Hue[0.42072856583263124`, 1, 1], 
       Hue[0.4339235440994428, 1, 1], 
       Hue[0.4339235440994428, 1, 1], 
       Hue[0.4482219112825864, 1, 1], 
       Hue[0.4482219112825864, 1, 1], 
       Hue[0.46359477563959944`, 1, 1], 
       Hue[0.46359477563959944`, 1, 1], 
       Hue[0.48001457607847264`, 1, 1], 
       Hue[0.48001457607847264`, 1, 1], 
       Hue[0.4974556703406297, 1, 1], 
       Hue[0.4974556703406297, 1, 1], 
       Hue[0.5158939565017163, 1, 1], 
       Hue[0.5158939565017163, 1, 1], 
       Hue[0.5353076938087875, 1, 1], 
       Hue[0.5353076938087875, 1, 1], 
       Hue[0.5556757445982718, 1, 1], 
       Hue[0.5556757445982718, 1, 1], 
       Hue[0.5769792836174885, 1, 1], 
       Hue[0.5769792836174885, 1, 1], 
       Hue[0.5991999701653299, 1, 1], 
       Hue[0.5991999701653299, 1, 1], 
       Hue[0.6223206152611922, 1, 1], 
       Hue[0.6223206152611922, 1, 1], 
       Hue[0.646325446642212, 1, 1], 
       Hue[0.646325446642212, 1, 1], 
       Hue[0.6711998264592366, 1, 1], 
       Hue[0.6711998264592366, 1, 1], 
       Hue[0.6969297314660567, 1, 1], 
       Hue[0.6969297314660567, 1, 1], 
       Hue[0.7235023463562673, 1, 1], 
       Hue[0.7235023463562673, 1, 1], 
       Hue[0.7509043991326707, 1, 1], 
       Hue[0.7509043991326707, 1, 1], 
       Hue[0.7791243549562274, 1, 1], 
       Hue[0.7791243549562274, 1, 1], 
       Hue[0.8081515576934792, 1, 1], 
       Hue[0.8081515576934792, 1, 1], 
       Hue[0.8379740999023698, 1, 1], 
       Hue[0.8379740999023698, 1, 1], 
       Hue[0.8685825170069534, 1, 1], 
       Hue[0.8685825170069534, 1, 1], 
       Hue[0.8999669535324006, 1, 1], 
       Hue[0.8999669535324006, 1, 1], 
       Hue[0.9321197069880646, 1, 1], 
       Hue[0.9321197069880646, 1, 1], 
       Hue[0.9650299290140577, 1, 1], 
       Hue[0.9650299290140577, 1, 1], 
       Hue[0.9986909188738518, 1, 1], 
       Hue[0.9986909188738518, 1, 1], 
       Hue[0.033094624152977085`, 1, 1], 
       Hue[0.033094624152977085`, 1, 1], 
       Hue[0.06823373640233694, 1, 1], 
       Hue[0.06823373640233694, 1, 1], 
       Hue[0.10410062095224143`, 1, 1], 
       Hue[0.10410062095224143`, 1, 1], 
       Hue[0.14068810334390153`, 1, 1], 
       Hue[0.14068810334390153`, 1, 1], 
       Hue[0.17799007885133333`, 1, 1], 
       Hue[0.17799007885133333`, 1, 1], 
       Hue[0.21599943175394315`, 1, 1], 
       Hue[0.21599943175394315`, 1, 1], 
       Hue[0.25471007336292956`, 1, 1], 
       Hue[0.25471007336292956`, 1, 1], 
       Hue[0.2941159403636716, 1, 1], 
       Hue[0.2941159403636716, 1, 1], 
       Hue[0.33421239620143495`, 1, 1], 
       Hue[0.33421239620143495`, 1, 1], 
       Hue[0.3749927116198588, 1, 1], 
       Hue[0.3749927116198588, 1, 1], 
       Hue[0.416453227649966, 1, 1], 
       Hue[0.416453227649966, 1, 1], 
       Hue[0.458587613960305, 1, 1], 
       Hue[0.458587613960305, 1, 1], 
       Hue[0.5013919317739264, 1, 1], 
       Hue[0.5013919317739264, 1, 1], 
       Hue[0.5448606766593951, 1, 1], 
       Hue[0.5448606766593951, 1, 1], 
       Hue[0.5889888658916277, 1, 1], 
       Hue[0.5889888658916277, 1, 1], 
       Hue[0.6337722749365959, 1, 1], 
       Hue[0.6337722749365959, 1, 1], 
       Hue[0.67920575497398, 1, 1], 
       Hue[0.67920575497398, 1, 1], 
       Hue[0.7252846301057936, 1, 1], 
       Hue[0.7252846301057936, 1, 1], 
       Hue[0.7720062490737557, 1, 1], 
       Hue[0.7720062490737557, 1, 1], 
       Hue[0.8193656677120444, 1, 1], 
       Hue[0.8193656677120444, 1, 1], 
       Hue[0.8673604139213364, 1, 1], 
       Hue[0.8673604139213364, 1, 1], 
       Hue[0.9159854061521892, 1, 1], 
       Hue[0.9159854061521892, 1, 1], 
       Hue[0.965236857420268, 1, 1], 
       Hue[0.965236857420268, 1, 1], 
       Hue[0.015112380299624755`, 1, 1], 
       Hue[0.015112380299624755`, 1, 1], 
       Hue[0.0656073763500262, 1, 1], 
       Hue[0.0656073763500262, 1, 1], 
       Hue[0.11671944049250602`, 1, 1], 
       Hue[0.11671944049250602`, 1, 1], 
       Hue[0.16844370339115078`, 1, 1], 
       Hue[0.16844370339115078`, 1, 1], 
       Hue[0.22077839817692413`, 1, 1], 
       Hue[0.22077839817692413`, 1, 1], 
       Hue[0.273719747035185, 1, 1], 
       Hue[0.273719747035185, 1, 1], 
       Hue[0.3272645018509859, 1, 1], 
       Hue[0.3272645018509859, 1, 1], 
       Hue[0.3814105632880518, 1, 1], 
       Hue[0.3814105632880518, 1, 1], 
       Hue[0.4361535802050153, 1, 1], 
       Hue[0.4361535802050153, 1, 1], 
       Hue[0.4914914479566433, 1, 1], 
       Hue[0.4914914479566433, 1, 1], 
       Hue[0.5474209272105655, 1, 1], 
       Hue[0.5474209272105655, 1, 1], 
       Hue[0.6039401865467807, 1, 1], 
       Hue[0.6039401865467807, 1, 1], 
       Hue[0.6610450494096778, 1, 1], 
       Hue[0.6610450494096778, 1, 1], 
       Hue[0.7187346323562827, 1, 1], 
       Hue[0.7187346323562827, 1, 1], 
       Hue[0.7770045221176122, 1, 1], 
       Hue[0.7770045221176122, 1, 1], 
       Hue[0.8358535452751399, 1, 1], 
       Hue[0.8358535452751399, 1, 1], 
       Hue[0.8952789269389672, 1, 1], 
       Hue[0.8952789269389672, 1, 1], 
       Hue[0.955278384025335, 1, 1], 
       Hue[0.955278384025335, 1, 1], 
       Hue[0.015848820485411037`, 1, 1], 
       Hue[0.015848820485411037`, 1, 1], 
       Hue[0.07698831686773512, 1, 1], 
       Hue[0.07698831686773512, 1, 1], 
       Hue[0.1386951331217639, 1, 1], 
       Hue[0.1386951331217639, 1, 1], 
       Hue[0.2009663078345991, 1, 1], 
       Hue[0.2009663078345991, 1, 1], 
       Hue[0.2637996869744032, 1, 1], 
       Hue[0.2637996869744032, 1, 1], 
       Hue[0.32719312809287177`, 1, 1], 
       Hue[0.32719312809287177`, 1, 1], 
       Hue[0.39114533126753376`, 1, 1], 
       Hue[0.39114533126753376`, 1, 1], 
       Hue[0.455652644567371, 1, 1], 
       Hue[0.455652644567371, 1, 1], 
       Hue[0.5207154494556953, 1, 1], 
       Hue[0.5207154494556953, 1, 1], 
       Hue[0.5863291466008134, 1, 1], 
       Hue[0.5863291466008134, 1, 1], 
       Hue[0.6524927029187558, 1, 1], 
       Hue[0.6524927029187558, 1, 1], 
       Hue[0.7192051438604465, 1, 1], 
       Hue[0.7192051438604465, 1, 1], 
       Hue[0.7864639377353686, 1, 1], 
       Hue[0.7864639377353686, 1, 1], 
       Hue[0.854266613804419, 1, 1], 
       Hue[0.854266613804419, 1, 1], 
       Hue[0.9226132153595316, 1, 1], 
       Hue[0.9226132153595316, 1, 1], 
       Hue[0.9914990995606757, 1, 1], 
       Hue[0.9914990995606757, 1, 1], 
       Hue[0.06092463240056644, 1, 1], 
       Hue[0.06092463240056644, 1, 1], 
       Hue[0.13088683050341132`, 1, 1], 
       Hue[0.13088683050341132`, 1, 1], 
       Hue[0.20138520126493187`, 1, 1], 
       Hue[0.20138520126493187`, 1, 1], 
       Hue[0.27241755097104603`, 1, 1], 
       Hue[0.27241755097104603`, 1, 1], 
       Hue[0.34398228203464143`, 1, 1], 
       Hue[0.34398228203464143`, 1, 1], 
       Hue[0.4160792472892838, 1, 1], 
       Hue[0.4160792472892838, 1, 1], 
       Hue[0.4887026604895621, 1, 1], 
       Hue[0.4887026604895621, 1, 1], 
       Hue[0.5618548264506966, 1, 1], 
       Hue[0.5618548264506966, 1, 1], 
       Hue[0.6355323297586595, 1, 1]}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    Axes->{True, None},
    AxesOrigin->{0, 0},
    Epilog->
     LineBox[{{1, 0.0021799831813656945`}, {2, 0.0024547471130588697`}, {
       3, 0.0027606662867666098`}, {4, 0.00310096922303252}, {
       5, 0.003479630423965041}, {6, 0.0038990756191110173`}, {
       7, 0.004364457528543681}, {8, 0.004879509214405107}, {
       9, 0.005448609775370151}, {10, 0.00607783337340048}, {
       11, 0.006771229639254447}, {12, 0.007534672312490112}, {
       13, 0.008373304709795179}, {14, 0.009294845349407829}, {
       15, 0.010304618153907819`}, {16, 0.01140898182274556}, {
       17, 0.012617009801659366`}, {18, 0.01393417377741211}, {
       19, 0.015369334707733887`}, {20, 0.01693052603426591}, {
       21, 0.018625045106631745`}, {22, 0.0204627300752764}, {
       23, 0.02245186102823825}, {24, 0.024600353194320804`}, {
       25, 0.026918694107456217`}, {26, 0.029414581495396434`}, {
       27, 0.03209752393067593}, {28, 0.03497663756977691}, {
       29, 0.03805943900264641}, {30, 0.04135527058194878}, {
       31, 0.04487205944262558}, {32, 0.04861669908277569}, {
       33, 0.052595813402006496`}, {34, 0.05681765288026682}, {
       35, 0.06128684124972052}, {36, 0.06600735661718311}, {
       37, 0.07098133867238944}, {38, 0.07621429108125545}, {
       39, 0.08170396646219354}, {40, 0.08745337841366976}, {
       41, 0.09345684393239075}, {42, 0.09971207493206483}, {
       43, 0.1062145504230983}, {44, 0.11295475444444614`}, {
       45, 0.11992402182272366`}, {46, 0.12711014757064265`}, {
       47, 0.13449927717565893`}, {48, 0.14207473459179756`}, {
       49, 0.14981788177608477`}, {50, 0.1577090626447986}, {
       51, 0.16572195132175516`}, {52, 0.17383363997322535`}, {
       53, 0.18201327708641205`}, {54, 0.19023090517269958`}, {
       55, 0.19845666922086197`}, {56, 0.2066547490415461}, {
       57, 0.2147892752119631}, {58, 0.22282207387879305`}, {
       59, 0.2307139277821593}, {60, 0.23842856312732666`}, {
       61, 0.24592331614768356`}, {62, 0.2531579881197967}, {
       63, 0.26009347663882154`}, {64, 0.266687990684189}, {
       65, 0.27290474123558606`}, {66, 0.2787056248084115}, {
       67, 0.28405546539111104`}, {68, 0.288922326327844}, {
       69, 0.2932728914984872}, {70, 0.2970847720774838}, {
       71, 0.300333536278017}, {72, 0.30300070228846354`}, {
       73, 0.30507237121874037`}, {74, 0.3065387874003445}, {
       75, 0.3073973403517825}, {76, 0.3076497579563843}, {
       77, 0.30730408686856986`}, {78, 0.3063712315994363}, {
       79, 0.30487258459097555`}, {80, 0.3028320925175755}, {
       81, 0.3002819571326402}, {82, 0.29725865947961133`}, {
       83, 0.29379979535068473`}, {84, 0.28995294789514875`}, {
       85, 0.2857648880509278}, {86, 0.28129062388327647`}, {
       87, 0.27658269844021716`}, {88, 0.27169718740444215`}, {
       89, 0.2666906976134521}, {90, 0.2616168629701307}, {
       91, 0.2565280602176279}, {92, 0.2514780628083413}, {
       93, 0.24650941000045604`}, {94, 0.24166636363156704`}, {
       95, 0.23698007578970054`}, {96, 0.23248304222392727`}, {
       97, 0.22819453905150927`}, {98, 0.22413178162004652`}, {
       99, 0.22030220801739753`}, {100, 0.2167062743997629}, {
       101, 0.21334160263783833`}, {102, 0.21019975129940682`}, {
       103, 0.2072683036175195}, {104, 0.20453335001586842`}, {
       105, 0.20197927439289626`}, {106, 0.1995901673952055}, {
       107, 0.1973507795540104}, {108, 0.19524792079993175`}, {
       109, 0.19327337457769886`}, {110, 0.1914172015968598}, {
       111, 0.1896776782531}, {112, 0.1880523688496779}, {
       113, 0.18654420131680388`}, {114, 0.18515664205590052`}, {
       115, 0.18390070631243616`}, {116, 0.18278607177438055`}, {
       117, 0.1818305129298614}, {118, 0.1810474019790032}, {
       119, 0.18045700911436888`}, {120, 0.1800751183748118}, {
       121, 0.17991766815397456`}, {122, 0.17999760725938194`}, {
       123, 0.18032655323051994`}, {124, 0.18091272810541784`}, {
       125, 0.18176197203862812`}, {126, 0.18287977591601534`}, {
       127, 0.18426627239305343`}, {128, 0.18592112250725912`}, {
       129, 0.1878390559561203}, {130, 0.19001498019456217`}, {
       131, 0.19244006202174596`}, {132, 0.1951012138829728}, {
       133, 0.197986338017044}, {134, 0.20108064539539905`}, {
       135, 0.2043662895322691}, {136, 0.20782814554787238`}, {
       137, 0.21144653564173965`}, {138, 0.21520250197148622`}, {
       139, 0.21907806251327414`}, {140, 0.22305640919252961`}, {
       141, 0.22711730009352807`}, {142, 0.23124504948911348`}, {
       143, 0.23542218374800802`}, {144, 0.23963353016472685`}, {
       145, 0.24386401835164245`}, {146, 0.24809965530234476`}, {
       147, 0.2523260908466059}, {148, 0.2565302391304768}, {
       149, 0.2607016770294811}, {150, 0.2648274984786279}, {
       151, 0.26889803443234606`}, {152, 0.27290320631651044`}, {
       153, 0.27683363423461865`}, {154, 0.2806801991248366}, {
       155, 0.28443525302491}, {156, 0.28809024064643235`}, {
       157, 0.29163901097496614`}, {158, 0.29507401902021624`}, {
       159, 0.29838910968607785`}, {160, 0.3015772042376815}, {
       161, 0.3046348330229162}, {162, 0.30755399493607855`}, {
       163, 0.31033221756447726`}, {164, 0.3129636336740557}, {
       165, 0.3154441980736416}, {166, 0.31777007818309055`}, {
       167, 0.31993764570871985`}, {168, 0.3219434577418436}, {
       169, 0.32378341534983596`}, {170, 0.32545472489057453`}, {
       171, 0.32695347780764084`}, {172, 0.32827824120537646`}, {
       173, 0.32942539135230975`}, {174, 0.33039275650375993`}, {
       175, 0.3311792980508751}, {176, 0.331782687461141}, {
       177, 0.3322021952409746}, {178, 0.3324371612101211}, {
       179, 0.33248655859580284`}, {180, 0.33234942612588225`}, {
       181, 0.3320263443622201}, {182, 0.3315154262703894}, {
       183, 0.33081648467692026`}, {184, 0.32993197057730134`}, {
       185, 0.32886029091017266`}, {186, 0.32760457894796874`}, {
       187, 0.32616510408389376`}, {188, 0.3245444672501573}, {
       189, 0.3227446550864884}, {190, 0.32076765029176696`}, {
       191, 0.31861567058417606`}, {192, 0.31629297780033966`}, {
       193, 0.3137996491750622}, {194, 0.3111404317865209}, {
       195, 0.30831810564048734`}, {196, 0.30533689423288973`}, {
       197, 0.3022020131806279}, {198, 0.29891760907581105`}, {
       199, 0.2954889170169475}, {200, 0.2919226989198764}, {
       201, 0.28822068394914535`}, {202, 0.28439015450288807`}, {
       203, 0.280433669537068}, {204, 0.27635872207917583`}, {
       205, 0.2721720199165121}, {206, 0.2678772039533209}, {
       207, 0.2634829490418796}, {208, 0.25899486782896647`}, {
       209, 0.25441951975904553`}, {210, 0.24976359895689187`}, {
       211, 0.24503284301055434`}, {212, 0.24023487783718145`}, {
       213, 0.23537589294801745`}, {214, 0.2304633271025684}, {
       215, 0.22550377899840038`}, {216, 0.220503698786649}, {
       217, 0.2154693931477791}, {218, 0.21040862038725164`}, {
       219, 0.20532763200609877`}, {220, 0.20023297795475978`}, {
       221, 0.19513113616630748`}, {222, 0.1900287749004839}, {
       223, 0.18493251329302843`}, {224, 0.17984846819312714`}, {
       225, 0.17478194192086524`}, {226, 0.16973955556593626`}, {
       227, 0.16472611577448287`}, {228, 0.1597483479342035}, {
       229, 0.15481068329760453`}, {230, 0.14991918292058082`}, {
       231, 0.14507750129117297`}, {232, 0.14029210921109336`}, {
       233, 0.13556633011942473`}, {234, 0.13090441351713136`}, {
       235, 0.12631114216836556`}, {236, 0.12178844192530047`}, {
       237, 0.11734271191792324`}, {238, 0.11297526442350818`}, {
       239, 0.10868992111755553`}, {240, 0.10448912435596365`}, {
       241, 0.10037555424227927`}, {242, 0.09635068888676308}, {
       243, 0.09241834313596697}, {244, 0.08857861685360004}, {
       245, 0.08483489754908917}, {246, 0.08118628463297588}, {
       247, 0.07763606160726062}, {248, 0.074182574803797}, {
       249, 0.0708284200420116}, {250, 0.06757324956683444}, {
       251, 0.06441805511228864}}],
    Frame->True,
    FrameTicks->{{{1., 
        FormBox[
         RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
        FormBox[
         RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
        FormBox[
         RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
        FormBox[
         RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
        FormBox["0", TraditionalForm]}, {157.25, 
        FormBox["2.5`", TraditionalForm]}, {188.5, 
        FormBox["5", TraditionalForm]}, {219.75, 
        FormBox["7.5`", TraditionalForm]}, {251., 
        FormBox["10", TraditionalForm]}}, Automatic, None, None},
    PlotRange->{-0.1, 1.1},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], Automatic}]}], "}"}]], "Output",
 CellChangeTimes->{3.3917582652551756`*^9, 3.391758485515954*^9, 
  3.3917585789532895`*^9, 3.3917586907248497`*^9, 3.391758736615117*^9, 
  3.391758878849937*^9, 3.3917589605333624`*^9, 3.391758996920468*^9, 
  3.391850171359375*^9, 3.392472445730389*^9, 3.392472481037429*^9},
 CellLabel->"Out[64]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ListAnimate", "[", "qtab", "]"}]], "Input",
 CellChangeTimes->{{3.3917585282176604`*^9, 3.3917585308122735`*^9}, {
  3.3917586148871193`*^9, 3.391758615418546*^9}},
 CellLabel->"In[65]:="],

Cell[BoxData[
 TagBox[
  StyleBox[
   DynamicModuleBox[{$CellContext`i$$ = 19, Typeset`show$$ = True, 
    Typeset`bookmarkList$$ = {}, Typeset`bookmarkMode$$ = "Menu", 
    Typeset`animator$$, Typeset`animvar$$ = 1, Typeset`name$$ = 
    "\"untitled\"", Typeset`specs$$ = {{{
       Hold[$CellContext`i$$], 1, ""}, 1, 24, 1}}, Typeset`size$$ = Automatic,
     Typeset`update$$ = 0, Typeset`initDone$$, Typeset`skipInitDone$$ = 
    True, $CellContext`i$4984$$ = 0}, 
    PaneBox[
     PanelBox[
      DynamicWrapperBox[
       TagBox[GridBox[{
          {
           ItemBox[
            ItemBox[
             StyleBox[
              TagBox[GridBox[{
                 {"\<\"\\!\\(\\*TagBox[\\\"\\\\\\\"\\\\\\\"\\\", HoldForm]\\)\
\"\>", 
                  AnimatorBox[Dynamic[$CellContext`i$$], {1, 24, 1},
                   AnimationRate->Automatic,
                   
                   AppearanceElements->{
                    "ProgressSlider", "PlayPauseButton", 
                    "FasterSlowerButtons", "DirectionButton"},
                   AutoAction->False,
                   ContinuousAction->True,
                   DisplayAllSteps->True]}
                },
                ColumnsEqual->False,
                
                GridBoxAlignment->{
                 "Columns" -> {Right, {Left}}, "ColumnsIndexed" -> {}, 
                  "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
                
                GridBoxItemSize->{
                 "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
                RowsEqual->False],
               "Grid"], "ManipulateLabel",
              StripOnInput->False],
             Alignment->{Automatic, Inherited},
             StripOnInput->False],
            Background->None,
            StripOnInput->False]},
          {
           ItemBox[
            TagBox[
             StyleBox[
              PaneBox[Cell[BoxData[
                TagBox[
                 PaneSelectorBox[{1->
                  GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF2Xl0jXcex/EfghhLU8tUCb0Vu4QsJEHCRxZLgmyyb09uYokgjWliachv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                    "], {
                    {GrayLevel[0.5], EdgeForm[None], 
                    GraphicsGroupBox[{PolygonBox[{{154, 558, 155}}], 
                    PolygonBox[CompressedData["
1:eJw1lgV0FWQYhr//jo2xjW2MGtuIjTFghKiIiICFBYg6AZFQHFiohGKBEkqH
dNrd3d3d3d3dLfG859Vz7nPOc/74vj++/95bWT+pbmImIjZAA5Bvhi2Q9R+R
+CT3Z2scng05eC7k4fnQKNyWgzcU/8fEs5LndoAB+J5QE55TgDeGPDwfivEm
UBCmBG+qMWGa4c2hMJwjV7mT11IEpXgrKA5ThpcrZpgKvDWUhHMW4kXJuRWz
Bd4yOZbGtMHbag3KDVV4e60BL4ce+LZQoblQg3dUjDCd8M5QGqYW76I1humK
d4OycIzt8O21xjA98R2gTZhe+I5aE14Nu+N7JJ9tJfTF+0FVeExvfCdoF95D
O7wyeS9q64PvnDxXe6pWrOS9tIdd8d2Scylmf3yX5D7VTVeKphtsDN/pXvTt
rTPQ3mEgPkhnEGYwvh/UhhmC768zUSyoww/SmeDbwAj8EOgRbhuKD4Pu4TkH
4Acmz1XOffB9k3Nrzkh8lO5IZwuH4ofpjMOMxQ+HnmHq8XE683CO4fjByWvp
BUfiR+kOwhyNHwO9w0zAj9WZh3OOxsck51bbcfjx0CfMRHyS7iDMZHwK9NXd
wgx8pu4Y7wdT8ZOgf7htFj4bBoTbTsZP0R3p7mAaPl13GOZ0/Izk2BpzKn5a
8ljVYHftNbm2FfNM/Cy9WXwQLMdXwGCdNSzCF+vMdfcwD5+vGgjPmYPPTe5T
2wJ8YfJc7fEE/MTkvQ2Es/FlybkUcwm+NLlPOVfiq1RDYVbja2BImLX4OtUU
PgwuxS+D4aoV2Iifo5oJcy5+HtSFOR+/QDUY5kL8IhgajnE5foVqIsyV+FUw
IszV+DWqWdUa3I7foZrDR8GN+E0wOjzmWvw6GBnOcTF+SfLa1XY9fkPy3LFw
N36PahYfA7fityXnUsyb8VuS+5TzTvyu5Lmacy9+H9Tr7OF9/APdgWoZnsaf
UU3j4+BB/CEYH257Fn8OJoTbHsYfgSP0NuBx/Am9kTBP4k8lx9aYR/HHkscq
54f4RzA1HPN5/AW9IXwKvIu/l7zWifA6/obejN4SvIy/ojcVnvMi/lJyn9pe
xV9Lnqs9348/kLy3yfA2/k5yLsV8E38ruU/fAeO1t+S3rzV+jH+iN6hahRZ8
97WEZXpL8DV93+hN6S3C5/gXepNhvsS/Sh6rMd/i38E0vU34Gf9FbzRcw+vx
Dcm1q7Zf8d9gBj4T/sT/gln4XGjAWrJhnt4ubKZvi95weMzf+D8wO5zze/wH
mB5u+xfflDxXOX7H/0jONQcyxM7KOJdi6k9DyrhPMX5k7E/Je9EacuhrCPPD
5OKNYAG+HMrwclgRPsNSvFXGfWqrwFvDSnwRFOJFsBhfAiV4U1gaphnePOO7
0ZhivEnGY3Vnn7K2z5LvRjHb0NcWVuHroBPeGdaH15iH58NCfA1U4x1gbXhM
Ld4l499CxWiHV8Lq8JgavGPGsRWjAG+c8V40pgpvn3HspPOHTT7W2AqLWufS

                    "]]}]}, {{}, {}, {
                    {Hue[0.7749898951612308, 1, 1], LineBox[{1, 2}]}, 
                    {Hue[0.8722290584923409, 1, 1], LineBox[{2, 3}]}, 
                    {Hue[0.9695841775471524, 1, 1], LineBox[{3, 4}]}, 
                    {Hue[0.06702656516515425, 1, 1], LineBox[{4, 5}]}, 
                    {Hue[0.16458210327625292`, 1, 1], LineBox[{5, 6}]}, 
                    {Hue[0.2622279195690579, 1, 1], LineBox[{6, 7}]}, 
                    {Hue[0.35998704518367436`, 1, 1], LineBox[{7, 8}]}, 
                    {Hue[0.45783566493262245`, 1, 1], LineBox[{8, 9}]}, 
                    {Hue[0.5557890356018238, 1, 1], LineBox[{9, 10}]}, 
                    {Hue[0.653844920101932, 1, 1], LineBox[{10, 11}]}, 
                    {Hue[0.7519981602293165, 1, 1], LineBox[{11, 12}]}, 
                    {Hue[0.8502551726866501, 1, 1], LineBox[{12, 13}]}, 
                    {Hue[0.9486085695831875, 1, 1], LineBox[{13, 14}]}, 
                    {Hue[0.0470609664999944, 1, 1], LineBox[{14, 15}]}, 
                    {Hue[0.14561566828510647`, 1, 1], LineBox[{15, 16}]}, 
                    {Hue[0.24427022386525324`, 1, 1], LineBox[{16, 17}]}, 
                    {Hue[0.343021643237129, 1, 1], LineBox[{17, 18}]}, 
                    {Hue[0.44187379272680516`, 1, 1], LineBox[{18, 19}]}, 
                    {Hue[0.5408248371910815, 1, 1], LineBox[{19, 20}]}, 
                    {Hue[0.6398744958829277, 1, 1], LineBox[{20, 21}]}, 
                    {Hue[0.7390232792464664, 1, 1], LineBox[{21, 22}]}, 
                    {Hue[0.838270524168504, 1, 1], LineBox[{22, 23}]}, 
                    {Hue[0.9376160928746887, 1, 1], LineBox[{23, 24}]}, 
                    {Hue[0.03706054230140248, 1, 1], LineBox[{24, 25}]}, 
                    {Hue[0.13660362360586886`, 1, 1], LineBox[{25, 26}]}, 
                    {Hue[0.23624319452712514`, 1, 1], LineBox[{26, 27}]}, 
                    {Hue[0.33598260881557607`, 1, 1], LineBox[{27, 28}]}, 
                    {Hue[0.435818781503373, 1, 1], LineBox[{28, 29}]}, 
                    {Hue[0.5357533041230387, 1, 1], LineBox[{29, 30}]}, 
                    {Hue[0.6357851273644934, 1, 1], LineBox[{30, 31}]}, 
                    {Hue[0.735914881028622, 1, 1], LineBox[{31, 32}]}, 
                    {Hue[0.8361419713192397, 1, 1], LineBox[{32, 33}]}, 
                    {Hue[0.9364664851620053, 1, 1], LineBox[{33, 34}]}, 
                    {Hue[0.03688834509492618, 1, 1], LineBox[{34, 35}]}, 
                    {Hue[0.13740705788019347`, 1, 1], LineBox[{35, 36}]}, 
                    {Hue[0.23802342482553834`, 1, 1], LineBox[{36, 37}]}, 
                    {Hue[0.33873639122288024`, 1, 1], LineBox[{37, 38}]}, 
                    {Hue[0.43954626861141466`, 1, 1], LineBox[{38, 39}]}, 
                    {Hue[0.5404528403401376, 1, 1], LineBox[{39, 40}]}, 
                    {Hue[0.6414564693287275, 1, 1], LineBox[{40, 41}]}, 
                    {Hue[0.742557176478602, 1, 1], LineBox[{41, 42}]}, 
                    {Hue[0.843754208203586, 1, 1], LineBox[{42, 43}]}, 
                    {Hue[0.9450473394371686, 1, 1], LineBox[{43, 44}]}, 
                    {Hue[0.04643704926151799, 1, 1], LineBox[{44, 45}]}, 
                    {Hue[0.14792346076075114`, 1, 1], LineBox[{45, 46}]}, 
                    {Hue[0.24950675009915688`, 1, 1], LineBox[{46, 47}]}, 
                    {Hue[0.35118600578268955`, 1, 1], LineBox[{47, 48}]}, 
                    {Hue[0.4529611660509637, 1, 1], LineBox[{48, 49}]}, 
                    {Hue[0.5548322704627584, 1, 1], LineBox[{49, 50}]}, 
                    {Hue[0.6567991697961184, 1, 1], LineBox[{50, 51}]}, 
                    {Hue[0.7588625703273734, 1, 1], LineBox[{51, 52}]}, 
                    {Hue[0.8610221377534003, 1, 1], LineBox[{52, 53}]}, 
                    {Hue[0.9632780091731467, 1, 1], LineBox[{53, 54}]}, 
                    {Hue[0.06562985710310025, 1, 1], LineBox[{54, 55}]}, 
                    {Hue[0.168077395961652, 1, 1], LineBox[{55, 56}]}, 
                    {Hue[0.2706209976066884, 1, 1], LineBox[{56, 57}]}, 
                    {Hue[0.37326058880617385`, 1, 1], LineBox[{57, 58}]}, 
                    {Hue[0.4759959473707332, 1, 1], LineBox[{58, 59}]}, 
                    {Hue[0.5788270998124714, 1, 1], LineBox[{59, 60}]}, 
                    {Hue[0.6817539353409436, 1, 1], LineBox[{60, 61}]}, 
                    {Hue[0.7847767535259931, 1, 1], LineBox[{61, 62}]}, 
                    {Hue[0.8878954180530303, 1, 1], LineBox[{62, 63}]}, 
                    {Hue[0.9911097590231914, 1, 1], LineBox[{63, 64}]}, 
                    {Hue[0.09441974610041529, 1, 1], LineBox[{64, 65}]}, 
                    {Hue[0.1978253627188138, 1, 1], LineBox[{65, 66}]}, 
                    {Hue[0.3013268292330902, 1, 1], LineBox[{66, 67}]}, 
                    {Hue[0.40492419793388484`, 1, 1], LineBox[{67, 68}]}, 
                    {Hue[0.508616503730313, 1, 1], LineBox[{68, 69}]}, 
                    {Hue[0.6124040131257059, 1, 1], LineBox[{69, 70}]}, 
                    {Hue[0.7162870697903432, 1, 1], LineBox[{70, 71}]}, 
                    {Hue[0.820265753877946, 1, 1], LineBox[{71, 72}]}, 
                    {Hue[0.9243396644570422, 1, 1], LineBox[{72, 73}]}, 
                    {Hue[0.028508445063498762`, 1, 1], LineBox[{73, 74}]}, 
                    {Hue[0.13277225503614024`, 1, 1], LineBox[{74, 75}]}, 
                    {Hue[0.23713093589427062`, 1, 1], LineBox[{75, 76}]}, 
                    {Hue[0.3415839747802137, 1, 1], LineBox[{76, 77}]}, 
                    {Hue[0.4461313425988212, 1, 1], LineBox[{77, 78}]}, 
                    {Hue[0.5507727312686279, 1, 1], LineBox[{78, 79}]}, 
                    {Hue[0.6555078730306969, 1, 1], LineBox[{79, 80}]}, 
                    {Hue[0.7603362356798644, 1, 1], LineBox[{80, 81}]}, 
                    {Hue[0.8652575837223297, 1, 1], LineBox[{81, 82}]}, 
                    {Hue[0.9702710585264879, 1, 1], LineBox[{82, 83}]}, 
                    {Hue[0.07537595344508677, 1, 1], LineBox[{83, 84}]}, 
                    {Hue[0.1805713877230246, 1, 1], LineBox[{84, 85}]}, 
                    {Hue[0.28585628035949945`, 1, 1], LineBox[{85, 86}]}, 
                    {Hue[0.3912295774894582, 1, 1], LineBox[{86, 87}]}, 
                    {Hue[0.4966904112408541, 1, 1], LineBox[{87, 88}]}, 
                    {Hue[0.6022370667295118, 1, 1], LineBox[{88, 89}]}, 
                    {Hue[0.70786676887722, 1, 1], LineBox[{89, 90}]}, 
                    {Hue[0.8135773679517109, 1, 1], LineBox[{90, 91}]}, 
                    {Hue[0.9193669962723475, 1, 1], LineBox[{91, 92}]}, 
                    {Hue[0.02523258448171614, 1, 1], LineBox[{92, 93}]}, 
                    {Hue[0.1311700458947892, 1, 1], LineBox[{93, 94}]}, 
                    {Hue[0.23717520772041145`, 1, 1], LineBox[{94, 95}]}, 
                    {Hue[0.3432433458032074, 1, 1], LineBox[{95, 96}]}, 
                    {Hue[0.4493686985356077, 1, 1], LineBox[{96, 97}]}, 
                    {Hue[0.5555437495804476, 1, 1], LineBox[{97, 98}]}, 
                    {Hue[0.6617608642013143, 1, 1], LineBox[{98, 99}]}, 
                    {Hue[0.7680114358431309, 1, 1], LineBox[{99, 100}]}, 
                    {Hue[0.8742849075770401, 1, 1], LineBox[{100, 101}]}, 
                    {Hue[0.9805688072229441, 1, 1], LineBox[{101, 102}]}, 
                    {Hue[0.08684850786555587, 1, 1], LineBox[{102, 103}]}, 
                    {Hue[0.1931073492585604, 1, 1], LineBox[{103, 104}]}, 
                    {Hue[0.29932613581724765`, 1, 1], LineBox[{104, 105}]}, 
                    {Hue[0.4054832774981767, 1, 1], LineBox[{105, 106}]}, 
                    {Hue[0.511554075856528, 1, 1], LineBox[{106, 107}]}, 
                    {Hue[0.6175095361091129, 1, 1], LineBox[{107, 108}]}, 
                    {Hue[0.7233171354656784, 1, 1], LineBox[{108, 109}]}, 
                    {Hue[0.8289397327072737, 1, 1], LineBox[{109, 110}]}, 
                    {Hue[0.9343363087113633, 1, 1], LineBox[{110, 111}]}, 
                    {Hue[0.03946140402520984, 1, 1], LineBox[{111, 112}]}, 
                    {Hue[0.14426430790282302`, 1, 1], LineBox[{112, 113}]}, 
                    {Hue[0.2486887623010038, 1, 1], LineBox[{113, 114}]}, 
                    {Hue[0.3526741834381149, 1, 1], LineBox[{114, 115}]}, 
                    {Hue[0.45615598834782883`, 1, 1], LineBox[{115, 116}]}, 
                    {Hue[0.5590664490818031, 1, 1], LineBox[{116, 117}]}, 
                    {Hue[0.6613355896608559, 1, 1], LineBox[{117, 118}]}, 
                    {Hue[0.7628925659823964, 1, 1], LineBox[{118, 119}]}, 
                    {Hue[0.8636679499759634, 1, 1], LineBox[{119, 120}]}, 
                    {Hue[0.9635960997077214, 1, 1], LineBox[{120, 121}]}, 
                    {Hue[0.06261731240532682, 1, 1], LineBox[{121, 122}]}, 
                    {Hue[0.16067962802358593`, 1, 1], LineBox[{122, 123}]}, 
                    {Hue[0.257741847061097, 1, 1], LineBox[{123, 124}]}, 
                    {Hue[0.3537773726657826, 1, 1], LineBox[{124, 125}]}, 
                    {Hue[0.44877573855416353`, 1, 1], LineBox[{125, 126}]}, 
                    {Hue[0.5427446454772279, 1, 1], LineBox[{126, 127}]}, 
                    {Hue[0.6357130551987016, 1, 1], LineBox[{127, 128}]}, 
                    {Hue[0.7277350307086694, 1, 1], LineBox[{128, 129}]}, 
                    {Hue[0.818891906069626, 1, 1], LineBox[{129, 130}]}, 
                    {Hue[0.9092929010507722, 1, 1], LineBox[{130, 131}]}, 
                    {Hue[0.9990790665473557, 1, 1], LineBox[{131, 132}]}, 
                    {Hue[0.0884304839703989, 1, 1], LineBox[{132, 133}]}, 
                    {Hue[0.17756605313563623`, 1, 1], LineBox[{133, 134}]}, 
                    {Hue[0.2667301868473516, 1, 1], LineBox[{134, 135}]}, 
                    {Hue[0.3561835023519964, 1, 1], LineBox[{135, 136}]}, 
                    {Hue[0.4461975135842855, 1, 1], LineBox[{136, 137}]}, 
                    {Hue[0.5370394232765678, 1, 1], LineBox[{137, 138}]}, 
                    {Hue[0.6289504058124807, 1, 1], LineBox[{138, 139}]}, 
                    {Hue[0.722133989077812, 1, 1], LineBox[{139, 140}]}, 
                    {Hue[0.8167482696906572, 1, 1], LineBox[{140, 141}]}, 
                    {Hue[0.9128958389165969, 1, 1], LineBox[{141, 142}]}, 
                    {Hue[0.01062320557995812, 1, 1], LineBox[{142, 143}]}, 
                    {Hue[0.10993026899221506`, 1, 1], LineBox[{143, 144}]}, 
                    {Hue[0.2107766388559164, 1, 1], LineBox[{144, 145}]}, 
                    {Hue[0.31309037761082614`, 1, 1], LineBox[{145, 146}]}, 
                    {Hue[0.4167761395354911, 1, 1], LineBox[{146, 147}]}, 
                    {Hue[0.5217208944670482, 1, 1], LineBox[{147, 148}]}, 
                    {Hue[0.6277983545596304, 1, 1], LineBox[{148, 149}]}, 
                    {Hue[0.7348794471728705, 1, 1], LineBox[{149, 150}]}, 
                    {Hue[0.8428413068614792, 1, 1], LineBox[{150, 151}]}, 
                    {Hue[0.9515712395853337, 1, 1], LineBox[{151, 152}]}, 
                    {Hue[0.06096695036510244, 1, 1], LineBox[{152, 153}]}, 
                    {Hue[0.1709356742763382, 1, 1], LineBox[{153, 154}]}, 
                    {Hue[0.28139292300740415`, 1, 1], LineBox[{154, 155}]}, 
                    {Hue[0.39226373560099825`, 1, 1], LineBox[{155, 156}]}, 
                    {Hue[0.503488967576359, 1, 1], LineBox[{156, 157}]}, 
                    {Hue[0.6150271693063969, 1, 1], LineBox[{157, 158}]}, 
                    {Hue[0.7268457566459958, 1, 1], LineBox[{158, 159}]}, 
                    {Hue[0.8389106407489604, 1, 1], LineBox[{159, 160}]}, 
                    {Hue[0.9511904422560358, 1, 1], LineBox[{160, 161}]}, 
                    {Hue[0.06366457578475905, 1, 1], LineBox[{161, 162}]}, 
                    {Hue[0.17632399770969037`, 1, 1], LineBox[{162, 163}]}, 
                    {Hue[0.2891634352733916, 1, 1], LineBox[{163, 164}]}, 
                    {Hue[0.40217446463777096`, 1, 1], LineBox[{164, 165}]}, 
                    {Hue[0.5153428538195237, 1, 1], LineBox[{165, 166}]}, 
                    {Hue[0.6286535390852653, 1, 1], LineBox[{166, 167}]}, 
                    {Hue[0.7420989550247571, 1, 1], LineBox[{167, 168}]}, 
                    {Hue[0.8556824988993403, 1, 1], LineBox[{168, 169}]}, 
                    {Hue[0.9694108265582719, 1, 1], LineBox[{169, 170}]}, 
                    {Hue[0.08328308408885278, 1, 1], LineBox[{170, 171}]}, 
                    {Hue[0.197289107027446, 1, 1], LineBox[{171, 172}]}, 
                    {Hue[0.3114172647845141, 1, 1], LineBox[{172, 173}]}, 
                    {Hue[0.425663848694229, 1, 1], LineBox[{173, 174}]}, 
                    {Hue[0.5400364449850104, 1, 1], LineBox[{174, 175}]}, 
                    {Hue[0.654545337569995, 1, 1], LineBox[{175, 176}]}, 
                    {Hue[0.7691919589545142, 1, 1], LineBox[{176, 177}]}, 
                    {Hue[0.8839673278665687, 1, 1], LineBox[{177, 178}]}, 
                    {Hue[0.9988648083453614, 1, 1], LineBox[{178, 179}]}, 
                    {Hue[0.11389045041208445`, 1, 1], LineBox[{179, 180}]}, 
                    {Hue[0.22905317250681512`, 1, 1], LineBox[{180, 181}]}, 
                    {Hue[0.3443477222842044, 1, 1], LineBox[{181, 182}]}, 
                    {Hue[0.4597564467478896, 1, 1], LineBox[{182, 183}]}, 
                    {Hue[0.575272954437088, 1, 1], LineBox[{183, 184}]}, 
                    {Hue[0.690916850151059, 1, 1], LineBox[{184, 185}]}, 
                    {Hue[0.8067172895240435, 1, 1], LineBox[{185, 186}]}, 
                    {Hue[0.9226830304114683, 1, 1], LineBox[{186, 187}]}, 
                    {Hue[0.03879193317028482, 1, 1], LineBox[{187, 188}]}, 
                    {Hue[0.15501007233955133`, 1, 1], LineBox[{188, 189}]}, 
                    {Hue[0.271322652888202, 1, 1], LineBox[{189, 190}]}, 
                    {Hue[0.38774699933720297`, 1, 1], LineBox[{190, 191}]}, 
                    {Hue[0.5043184018206746, 1, 1], LineBox[{191, 192}]}, 
                    {Hue[0.6210616490432017, 1, 1], LineBox[{192, 193}]}, 
                    {Hue[0.7379741223799546, 1, 1], LineBox[{193, 194}]}, 
                    {Hue[0.8550330753268756, 1, 1], LineBox[{194, 195}]}, 
                    {Hue[0.9722146191777067, 1, 1], LineBox[{195, 196}]}, 
                    {Hue[0.08950548352353373, 1, 1], LineBox[{196, 197}]}, 
                    {Hue[0.20690376138675443`, 1, 1], LineBox[{197, 198}]}, 
                    {Hue[0.3244227059977963, 1, 1], LineBox[{198, 199}]}, 
                    {Hue[0.44209693973826225`, 1, 1], LineBox[{199, 200}]}, 
                    {Hue[0.5599691644159934, 1, 1], LineBox[{200, 201}]}, 
                    {Hue[0.6780539150001488, 1, 1], LineBox[{201, 202}]}, 
                    {Hue[0.7963115534339857, 1, 1], LineBox[{202, 203}]}, 
                    {Hue[0.9146706065985621, 1, 1], LineBox[{203, 204}]}, 
                    {Hue[0.03308838080242426, 1, 1], LineBox[{204, 205}]}, 
                    {Hue[0.1515920280512786, 1, 1], LineBox[{205, 206}]}, 
                    {Hue[0.27025289101267647`, 1, 1], LineBox[{206, 207}]}, 
                    {Hue[0.389119757508248, 1, 1], LineBox[{207, 208}]}, 
                    {Hue[0.5081848430282914, 1, 1], LineBox[{208, 209}]}, 
                    {Hue[0.627414082696829, 1, 1], LineBox[{209, 210}]}, 
                    {Hue[0.7467971716276445, 1, 1], LineBox[{210, 211}]}, 
                    {Hue[0.8663546614564116, 1, 1], LineBox[{211, 212}]}, 
                    {Hue[0.9860997934088671, 1, 1], LineBox[{212, 213}]}, 
                    {Hue[0.10600667585355196`, 1, 1], LineBox[{213, 214}]}, 
                    {Hue[0.2260244737159512, 1, 1], LineBox[{214, 215}]}, 
                    {Hue[0.3461272749252381, 1, 1], LineBox[{215, 216}]}, 
                    {Hue[0.466347997981795, 1, 1], LineBox[{216, 217}]}, 
                    {Hue[0.5867510597322951, 1, 1], LineBox[{217, 218}]}, 
                    {Hue[0.7073591620063187, 1, 1], LineBox[{218, 219}]}, 
                    {Hue[0.8281193177149418, 1, 1], LineBox[{219, 220}]}, 
                    {Hue[0.9489791169151242, 1, 1], LineBox[{220, 221}]}, 
                    {Hue[0.06995961206559426, 1, 1], LineBox[{221, 222}]}, 
                    {Hue[0.19134767685266463`, 1, 1], LineBox[{222, 223}]}, 
                    {Hue[0.3131695453007627, 1, 1], LineBox[{223, 224}]}, 
                    {Hue[0.43461463065006734`, 1, 1], LineBox[{224, 225}]}, 
                    {Hue[0.5587062218899088, 1, 1], LineBox[{225, 226}]}, 
                    {Hue[0.7011321539939399, 1, 1], LineBox[{226, 227}]}, 
                    {Hue[0.8279791303773694, 1, 1], LineBox[{227, 228}]}, 
                    {Hue[0.8573883562613613, 1, 1], LineBox[{228, 229}]}, 
                    {Hue[0.8656487203793617, 1, 1], LineBox[{229, 230}]}, 
                    {Hue[0.8656487203793617, 1, 1], LineBox[{230, 231}]}, 
                    {Hue[0.8656487203793617, 1, 1], LineBox[{231, 232}]}, 
                    {Hue[0.8656487203793617, 1, 1], LineBox[{232, 233}]}, 
                    {Hue[0.8656487203793617, 1, 1], LineBox[{233, 234}]}, 
                    {Hue[0.8656487203793617, 1, 1], LineBox[{234, 235}]}, 
                    {Hue[0.8656487203793617, 1, 1], LineBox[{235, 236}]}, 
                    {Hue[0.8656487203793617, 1, 1], LineBox[{236, 237}]}, 
                    {Hue[0.8656487203793617, 1, 1], LineBox[{237, 238}]}, 
                    {Hue[0.8656487203793617, 1, 1], LineBox[{238, 239}]}, 
                    {Hue[0.8656487203793617, 1, 1], LineBox[{239, 240}]}, 
                    {Hue[0.8656487203793617, 1, 1], LineBox[{240, 241}]}, 
                    {Hue[0.8656487203793617, 1, 1], LineBox[{241, 242}]}, 
                    {Hue[0.8656487203793617, 1, 1], LineBox[{242, 243}]}, 
                    {Hue[0.8656487203793617, 1, 1], LineBox[{243, 244}]}, 
                    {Hue[0.8656487203793617, 1, 1], LineBox[{244, 245}]}, 
                    {Hue[0.8656487203793617, 1, 1], LineBox[{245, 246}]}, 
                    {Hue[0.8656487203793617, 1, 1], LineBox[{246, 247}]}, 
                    {Hue[0.8656487203793617, 1, 1], LineBox[{247, 248}]}, 
                    {Hue[0.8656487203793617, 1, 1], LineBox[{248, 249}]}, 
                    {Hue[0.8656487203793617, 1, 1], LineBox[{249, 250}]}, 
                    {Hue[0.8656487203793617, 1, 1], LineBox[{250, 251}]}}}},
                    VertexColors->{
                    Hue[0.7749898951612308, 1, 1], 
                    Hue[0.8722290584923409, 1, 1], 
                    Hue[0.9695841775471524, 1, 1], 
                    Hue[0.06702656516515425, 1, 1], 
                    Hue[0.16458210327625292`, 1, 1], 
                    Hue[0.2622279195690579, 1, 1], 
                    Hue[0.35998704518367436`, 1, 1], 
                    Hue[0.45783566493262245`, 1, 1], 
                    Hue[0.5557890356018238, 1, 1], 
                    Hue[0.653844920101932, 1, 1], 
                    Hue[0.7519981602293165, 1, 1], 
                    Hue[0.8502551726866501, 1, 1], 
                    Hue[0.9486085695831875, 1, 1], 
                    Hue[0.0470609664999944, 1, 1], 
                    Hue[0.14561566828510647`, 1, 1], 
                    Hue[0.24427022386525324`, 1, 1], 
                    Hue[0.343021643237129, 1, 1], 
                    Hue[0.44187379272680516`, 1, 1], 
                    Hue[0.5408248371910815, 1, 1], 
                    Hue[0.6398744958829277, 1, 1], 
                    Hue[0.7390232792464664, 1, 1], 
                    Hue[0.838270524168504, 1, 1], 
                    Hue[0.9376160928746887, 1, 1], 
                    Hue[0.03706054230140248, 1, 1], 
                    Hue[0.13660362360586886`, 1, 1], 
                    Hue[0.23624319452712514`, 1, 1], 
                    Hue[0.33598260881557607`, 1, 1], 
                    Hue[0.435818781503373, 1, 1], 
                    Hue[0.5357533041230387, 1, 1], 
                    Hue[0.6357851273644934, 1, 1], 
                    Hue[0.735914881028622, 1, 1], 
                    Hue[0.8361419713192397, 1, 1], 
                    Hue[0.9364664851620053, 1, 1], 
                    Hue[0.03688834509492618, 1, 1], 
                    Hue[0.13740705788019347`, 1, 1], 
                    Hue[0.23802342482553834`, 1, 1], 
                    Hue[0.33873639122288024`, 1, 1], 
                    Hue[0.43954626861141466`, 1, 1], 
                    Hue[0.5404528403401376, 1, 1], 
                    Hue[0.6414564693287275, 1, 1], 
                    Hue[0.742557176478602, 1, 1], 
                    Hue[0.843754208203586, 1, 1], 
                    Hue[0.9450473394371686, 1, 1], 
                    Hue[0.04643704926151799, 1, 1], 
                    Hue[0.14792346076075114`, 1, 1], 
                    Hue[0.24950675009915688`, 1, 1], 
                    Hue[0.35118600578268955`, 1, 1], 
                    Hue[0.4529611660509637, 1, 1], 
                    Hue[0.5548322704627584, 1, 1], 
                    Hue[0.6567991697961184, 1, 1], 
                    Hue[0.7588625703273734, 1, 1], 
                    Hue[0.8610221377534003, 1, 1], 
                    Hue[0.9632780091731467, 1, 1], 
                    Hue[0.06562985710310025, 1, 1], 
                    Hue[0.168077395961652, 1, 1], 
                    Hue[0.2706209976066884, 1, 1], 
                    Hue[0.37326058880617385`, 1, 1], 
                    Hue[0.4759959473707332, 1, 1], 
                    Hue[0.5788270998124714, 1, 1], 
                    Hue[0.6817539353409436, 1, 1], 
                    Hue[0.7847767535259931, 1, 1], 
                    Hue[0.8878954180530303, 1, 1], 
                    Hue[0.9911097590231914, 1, 1], 
                    Hue[0.09441974610041529, 1, 1], 
                    Hue[0.1978253627188138, 1, 1], 
                    Hue[0.3013268292330902, 1, 1], 
                    Hue[0.40492419793388484`, 1, 1], 
                    Hue[0.508616503730313, 1, 1], 
                    Hue[0.6124040131257059, 1, 1], 
                    Hue[0.7162870697903432, 1, 1], 
                    Hue[0.820265753877946, 1, 1], 
                    Hue[0.9243396644570422, 1, 1], 
                    Hue[0.028508445063498762`, 1, 1], 
                    Hue[0.13277225503614024`, 1, 1], 
                    Hue[0.23713093589427062`, 1, 1], 
                    Hue[0.3415839747802137, 1, 1], 
                    Hue[0.4461313425988212, 1, 1], 
                    Hue[0.5507727312686279, 1, 1], 
                    Hue[0.6555078730306969, 1, 1], 
                    Hue[0.7603362356798644, 1, 1], 
                    Hue[0.8652575837223297, 1, 1], 
                    Hue[0.9702710585264879, 1, 1], 
                    Hue[0.07537595344508677, 1, 1], 
                    Hue[0.1805713877230246, 1, 1], 
                    Hue[0.28585628035949945`, 1, 1], 
                    Hue[0.3912295774894582, 1, 1], 
                    Hue[0.4966904112408541, 1, 1], 
                    Hue[0.6022370667295118, 1, 1], 
                    Hue[0.70786676887722, 1, 1], 
                    Hue[0.8135773679517109, 1, 1], 
                    Hue[0.9193669962723475, 1, 1], 
                    Hue[0.02523258448171614, 1, 1], 
                    Hue[0.1311700458947892, 1, 1], 
                    Hue[0.23717520772041145`, 1, 1], 
                    Hue[0.3432433458032074, 1, 1], 
                    Hue[0.4493686985356077, 1, 1], 
                    Hue[0.5555437495804476, 1, 1], 
                    Hue[0.6617608642013143, 1, 1], 
                    Hue[0.7680114358431309, 1, 1], 
                    Hue[0.8742849075770401, 1, 1], 
                    Hue[0.9805688072229441, 1, 1], 
                    Hue[0.08684850786555587, 1, 1], 
                    Hue[0.1931073492585604, 1, 1], 
                    Hue[0.29932613581724765`, 1, 1], 
                    Hue[0.4054832774981767, 1, 1], 
                    Hue[0.511554075856528, 1, 1], 
                    Hue[0.6175095361091129, 1, 1], 
                    Hue[0.7233171354656784, 1, 1], 
                    Hue[0.8289397327072737, 1, 1], 
                    Hue[0.9343363087113633, 1, 1], 
                    Hue[0.03946140402520984, 1, 1], 
                    Hue[0.14426430790282302`, 1, 1], 
                    Hue[0.2486887623010038, 1, 1], 
                    Hue[0.3526741834381149, 1, 1], 
                    Hue[0.45615598834782883`, 1, 1], 
                    Hue[0.5590664490818031, 1, 1], 
                    Hue[0.6613355896608559, 1, 1], 
                    Hue[0.7628925659823964, 1, 1], 
                    Hue[0.8636679499759634, 1, 1], 
                    Hue[0.9635960997077214, 1, 1], 
                    Hue[0.06261731240532682, 1, 1], 
                    Hue[0.16067962802358593`, 1, 1], 
                    Hue[0.257741847061097, 1, 1], 
                    Hue[0.3537773726657826, 1, 1], 
                    Hue[0.44877573855416353`, 1, 1], 
                    Hue[0.5427446454772279, 1, 1], 
                    Hue[0.6357130551987016, 1, 1], 
                    Hue[0.7277350307086694, 1, 1], 
                    Hue[0.818891906069626, 1, 1], 
                    Hue[0.9092929010507722, 1, 1], 
                    Hue[0.9990790665473557, 1, 1], 
                    Hue[0.0884304839703989, 1, 1], 
                    Hue[0.17756605313563623`, 1, 1], 
                    Hue[0.2667301868473516, 1, 1], 
                    Hue[0.3561835023519964, 1, 1], 
                    Hue[0.4461975135842855, 1, 1], 
                    Hue[0.5370394232765678, 1, 1], 
                    Hue[0.6289504058124807, 1, 1], 
                    Hue[0.722133989077812, 1, 1], 
                    Hue[0.8167482696906572, 1, 1], 
                    Hue[0.9128958389165969, 1, 1], 
                    Hue[0.01062320557995812, 1, 1], 
                    Hue[0.10993026899221506`, 1, 1], 
                    Hue[0.2107766388559164, 1, 1], 
                    Hue[0.31309037761082614`, 1, 1], 
                    Hue[0.4167761395354911, 1, 1], 
                    Hue[0.5217208944670482, 1, 1], 
                    Hue[0.6277983545596304, 1, 1], 
                    Hue[0.7348794471728705, 1, 1], 
                    Hue[0.8428413068614792, 1, 1], 
                    Hue[0.9515712395853337, 1, 1], 
                    Hue[0.06096695036510244, 1, 1], 
                    Hue[0.1709356742763382, 1, 1], 
                    Hue[0.28139292300740415`, 1, 1], 
                    Hue[0.39226373560099825`, 1, 1], 
                    Hue[0.503488967576359, 1, 1], 
                    Hue[0.6150271693063969, 1, 1], 
                    Hue[0.7268457566459958, 1, 1], 
                    Hue[0.8389106407489604, 1, 1], 
                    Hue[0.9511904422560358, 1, 1], 
                    Hue[0.06366457578475905, 1, 1], 
                    Hue[0.17632399770969037`, 1, 1], 
                    Hue[0.2891634352733916, 1, 1], 
                    Hue[0.40217446463777096`, 1, 1], 
                    Hue[0.5153428538195237, 1, 1], 
                    Hue[0.6286535390852653, 1, 1], 
                    Hue[0.7420989550247571, 1, 1], 
                    Hue[0.8556824988993403, 1, 1], 
                    Hue[0.9694108265582719, 1, 1], 
                    Hue[0.08328308408885278, 1, 1], 
                    Hue[0.197289107027446, 1, 1], 
                    Hue[0.3114172647845141, 1, 1], 
                    Hue[0.425663848694229, 1, 1], 
                    Hue[0.5400364449850104, 1, 1], 
                    Hue[0.654545337569995, 1, 1], 
                    Hue[0.7691919589545142, 1, 1], 
                    Hue[0.8839673278665687, 1, 1], 
                    Hue[0.9988648083453614, 1, 1], 
                    Hue[0.11389045041208445`, 1, 1], 
                    Hue[0.22905317250681512`, 1, 1], 
                    Hue[0.3443477222842044, 1, 1], 
                    Hue[0.4597564467478896, 1, 1], 
                    Hue[0.575272954437088, 1, 1], 
                    Hue[0.690916850151059, 1, 1], 
                    Hue[0.8067172895240435, 1, 1], 
                    Hue[0.9226830304114683, 1, 1], 
                    Hue[0.03879193317028482, 1, 1], 
                    Hue[0.15501007233955133`, 1, 1], 
                    Hue[0.271322652888202, 1, 1], 
                    Hue[0.38774699933720297`, 1, 1], 
                    Hue[0.5043184018206746, 1, 1], 
                    Hue[0.6210616490432017, 1, 1], 
                    Hue[0.7379741223799546, 1, 1], 
                    Hue[0.8550330753268756, 1, 1], 
                    Hue[0.9722146191777067, 1, 1], 
                    Hue[0.08950548352353373, 1, 1], 
                    Hue[0.20690376138675443`, 1, 1], 
                    Hue[0.3244227059977963, 1, 1], 
                    Hue[0.44209693973826225`, 1, 1], 
                    Hue[0.5599691644159934, 1, 1], 
                    Hue[0.6780539150001488, 1, 1], 
                    Hue[0.7963115534339857, 1, 1], 
                    Hue[0.9146706065985621, 1, 1], 
                    Hue[0.03308838080242426, 1, 1], 
                    Hue[0.1515920280512786, 1, 1], 
                    Hue[0.27025289101267647`, 1, 1], 
                    Hue[0.389119757508248, 1, 1], 
                    Hue[0.5081848430282914, 1, 1], 
                    Hue[0.627414082696829, 1, 1], 
                    Hue[0.7467971716276445, 1, 1], 
                    Hue[0.8663546614564116, 1, 1], 
                    Hue[0.9860997934088671, 1, 1], 
                    Hue[0.10600667585355196`, 1, 1], 
                    Hue[0.2260244737159512, 1, 1], 
                    Hue[0.3461272749252381, 1, 1], 
                    Hue[0.466347997981795, 1, 1], 
                    Hue[0.5867510597322951, 1, 1], 
                    Hue[0.7073591620063187, 1, 1], 
                    Hue[0.8281193177149418, 1, 1], 
                    Hue[0.9489791169151242, 1, 1], 
                    Hue[0.06995961206559426, 1, 1], 
                    Hue[0.19134767685266463`, 1, 1], 
                    Hue[0.3131695453007627, 1, 1], 
                    Hue[0.43461463065006734`, 1, 1], 
                    Hue[0.5587062218899088, 1, 1], 
                    Hue[0.7011321539939399, 1, 1], 
                    Hue[0.8279791303773694, 1, 1], 
                    Hue[0.8573883562613613, 1, 1], 
                    Hue[0.8656487203793617, 1, 1], 
                    Hue[0.8656487203793617, 1, 1], 
                    Hue[0.8656487203793617, 1, 1], 
                    Hue[0.8656487203793617, 1, 1], 
                    Hue[0.8656487203793617, 1, 1], 
                    Hue[0.8656487203793617, 1, 1], 
                    Hue[0.8656487203793617, 1, 1], 
                    Hue[0.8656487203793617, 1, 1], 
                    Hue[0.8656487203793617, 1, 1], 
                    Hue[0.8656487203793617, 1, 1], 
                    Hue[0.8656487203793617, 1, 1], 
                    Hue[0.8656487203793617, 1, 1], 
                    Hue[0.8656487203793617, 1, 1], 
                    Hue[0.8656487203793617, 1, 1], 
                    Hue[0.8656487203793617, 1, 1], 
                    Hue[0.8656487203793617, 1, 1], 
                    Hue[0.8656487203793617, 1, 1], 
                    Hue[0.8656487203793617, 1, 1], 
                    Hue[0.8656487203793617, 1, 1], 
                    Hue[0.8656487203793617, 1, 1], 
                    Hue[0.8656487203793617, 1, 1], 
                    Hue[0.8656487203793617, 1, 1], 
                    Hue[0.8656487203793617, 1, 1], 
                    Hue[0.7749898951612308, 1, 1], 
                    Hue[0.8722290584923409, 1, 1], 
                    Hue[0.8722290584923409, 1, 1], 
                    Hue[0.9695841775471524, 1, 1], 
                    Hue[0.9695841775471524, 1, 1], 
                    Hue[0.06702656516515425, 1, 1], 
                    Hue[0.06702656516515425, 1, 1], 
                    Hue[0.16458210327625292`, 1, 1], 
                    Hue[0.16458210327625292`, 1, 1], 
                    Hue[0.2622279195690579, 1, 1], 
                    Hue[0.2622279195690579, 1, 1], 
                    Hue[0.35998704518367436`, 1, 1], 
                    Hue[0.35998704518367436`, 1, 1], 
                    Hue[0.45783566493262245`, 1, 1], 
                    Hue[0.45783566493262245`, 1, 1], 
                    Hue[0.5557890356018238, 1, 1], 
                    Hue[0.5557890356018238, 1, 1], 
                    Hue[0.653844920101932, 1, 1], 
                    Hue[0.653844920101932, 1, 1], 
                    Hue[0.7519981602293165, 1, 1], 
                    Hue[0.7519981602293165, 1, 1], 
                    Hue[0.8502551726866501, 1, 1], 
                    Hue[0.8502551726866501, 1, 1], 
                    Hue[0.9486085695831875, 1, 1], 
                    Hue[0.9486085695831875, 1, 1], 
                    Hue[0.0470609664999944, 1, 1], 
                    Hue[0.0470609664999944, 1, 1], 
                    Hue[0.14561566828510647`, 1, 1], 
                    Hue[0.14561566828510647`, 1, 1], 
                    Hue[0.24427022386525324`, 1, 1], 
                    Hue[0.24427022386525324`, 1, 1], 
                    Hue[0.343021643237129, 1, 1], 
                    Hue[0.343021643237129, 1, 1], 
                    Hue[0.44187379272680516`, 1, 1], 
                    Hue[0.44187379272680516`, 1, 1], 
                    Hue[0.5408248371910815, 1, 1], 
                    Hue[0.5408248371910815, 1, 1], 
                    Hue[0.6398744958829277, 1, 1], 
                    Hue[0.6398744958829277, 1, 1], 
                    Hue[0.7390232792464664, 1, 1], 
                    Hue[0.7390232792464664, 1, 1], 
                    Hue[0.838270524168504, 1, 1], 
                    Hue[0.838270524168504, 1, 1], 
                    Hue[0.9376160928746887, 1, 1], 
                    Hue[0.9376160928746887, 1, 1], 
                    Hue[0.03706054230140248, 1, 1], 
                    Hue[0.03706054230140248, 1, 1], 
                    Hue[0.13660362360586886`, 1, 1], 
                    Hue[0.13660362360586886`, 1, 1], 
                    Hue[0.23624319452712514`, 1, 1], 
                    Hue[0.23624319452712514`, 1, 1], 
                    Hue[0.33598260881557607`, 1, 1], 
                    Hue[0.33598260881557607`, 1, 1], 
                    Hue[0.435818781503373, 1, 1], 
                    Hue[0.435818781503373, 1, 1], 
                    Hue[0.5357533041230387, 1, 1], 
                    Hue[0.5357533041230387, 1, 1], 
                    Hue[0.6357851273644934, 1, 1], 
                    Hue[0.6357851273644934, 1, 1], 
                    Hue[0.735914881028622, 1, 1], 
                    Hue[0.735914881028622, 1, 1], 
                    Hue[0.8361419713192397, 1, 1], 
                    Hue[0.8361419713192397, 1, 1], 
                    Hue[0.9364664851620053, 1, 1], 
                    Hue[0.9364664851620053, 1, 1], 
                    Hue[0.03688834509492618, 1, 1], 
                    Hue[0.03688834509492618, 1, 1], 
                    Hue[0.13740705788019347`, 1, 1], 
                    Hue[0.13740705788019347`, 1, 1], 
                    Hue[0.23802342482553834`, 1, 1], 
                    Hue[0.23802342482553834`, 1, 1], 
                    Hue[0.33873639122288024`, 1, 1], 
                    Hue[0.33873639122288024`, 1, 1], 
                    Hue[0.43954626861141466`, 1, 1], 
                    Hue[0.43954626861141466`, 1, 1], 
                    Hue[0.5404528403401376, 1, 1], 
                    Hue[0.5404528403401376, 1, 1], 
                    Hue[0.6414564693287275, 1, 1], 
                    Hue[0.6414564693287275, 1, 1], 
                    Hue[0.742557176478602, 1, 1], 
                    Hue[0.742557176478602, 1, 1], 
                    Hue[0.843754208203586, 1, 1], 
                    Hue[0.843754208203586, 1, 1], 
                    Hue[0.9450473394371686, 1, 1], 
                    Hue[0.9450473394371686, 1, 1], 
                    Hue[0.04643704926151799, 1, 1], 
                    Hue[0.04643704926151799, 1, 1], 
                    Hue[0.14792346076075114`, 1, 1], 
                    Hue[0.14792346076075114`, 1, 1], 
                    Hue[0.24950675009915688`, 1, 1], 
                    Hue[0.24950675009915688`, 1, 1], 
                    Hue[0.35118600578268955`, 1, 1], 
                    Hue[0.35118600578268955`, 1, 1], 
                    Hue[0.4529611660509637, 1, 1], 
                    Hue[0.4529611660509637, 1, 1], 
                    Hue[0.5548322704627584, 1, 1], 
                    Hue[0.5548322704627584, 1, 1], 
                    Hue[0.6567991697961184, 1, 1], 
                    Hue[0.6567991697961184, 1, 1], 
                    Hue[0.7588625703273734, 1, 1], 
                    Hue[0.7588625703273734, 1, 1], 
                    Hue[0.8610221377534003, 1, 1], 
                    Hue[0.8610221377534003, 1, 1], 
                    Hue[0.9632780091731467, 1, 1], 
                    Hue[0.9632780091731467, 1, 1], 
                    Hue[0.06562985710310025, 1, 1], 
                    Hue[0.06562985710310025, 1, 1], 
                    Hue[0.168077395961652, 1, 1], 
                    Hue[0.168077395961652, 1, 1], 
                    Hue[0.2706209976066884, 1, 1], 
                    Hue[0.2706209976066884, 1, 1], 
                    Hue[0.37326058880617385`, 1, 1], 
                    Hue[0.37326058880617385`, 1, 1], 
                    Hue[0.4759959473707332, 1, 1], 
                    Hue[0.4759959473707332, 1, 1], 
                    Hue[0.5788270998124714, 1, 1], 
                    Hue[0.5788270998124714, 1, 1], 
                    Hue[0.6817539353409436, 1, 1], 
                    Hue[0.6817539353409436, 1, 1], 
                    Hue[0.7847767535259931, 1, 1], 
                    Hue[0.7847767535259931, 1, 1], 
                    Hue[0.8878954180530303, 1, 1], 
                    Hue[0.8878954180530303, 1, 1], 
                    Hue[0.9911097590231914, 1, 1], 
                    Hue[0.9911097590231914, 1, 1], 
                    Hue[0.09441974610041529, 1, 1], 
                    Hue[0.09441974610041529, 1, 1], 
                    Hue[0.1978253627188138, 1, 1], 
                    Hue[0.1978253627188138, 1, 1], 
                    Hue[0.3013268292330902, 1, 1], 
                    Hue[0.3013268292330902, 1, 1], 
                    Hue[0.40492419793388484`, 1, 1], 
                    Hue[0.40492419793388484`, 1, 1], 
                    Hue[0.508616503730313, 1, 1], 
                    Hue[0.508616503730313, 1, 1], 
                    Hue[0.6124040131257059, 1, 1], 
                    Hue[0.6124040131257059, 1, 1], 
                    Hue[0.7162870697903432, 1, 1], 
                    Hue[0.7162870697903432, 1, 1], 
                    Hue[0.820265753877946, 1, 1], 
                    Hue[0.820265753877946, 1, 1], 
                    Hue[0.9243396644570422, 1, 1], 
                    Hue[0.9243396644570422, 1, 1], 
                    Hue[0.028508445063498762`, 1, 1], 
                    Hue[0.028508445063498762`, 1, 1], 
                    Hue[0.13277225503614024`, 1, 1], 
                    Hue[0.13277225503614024`, 1, 1], 
                    Hue[0.23713093589427062`, 1, 1], 
                    Hue[0.23713093589427062`, 1, 1], 
                    Hue[0.3415839747802137, 1, 1], 
                    Hue[0.3415839747802137, 1, 1], 
                    Hue[0.4461313425988212, 1, 1], 
                    Hue[0.4461313425988212, 1, 1], 
                    Hue[0.5507727312686279, 1, 1], 
                    Hue[0.5507727312686279, 1, 1], 
                    Hue[0.6555078730306969, 1, 1], 
                    Hue[0.6555078730306969, 1, 1], 
                    Hue[0.7603362356798644, 1, 1], 
                    Hue[0.7603362356798644, 1, 1], 
                    Hue[0.8652575837223297, 1, 1], 
                    Hue[0.8652575837223297, 1, 1], 
                    Hue[0.9702710585264879, 1, 1], 
                    Hue[0.9702710585264879, 1, 1], 
                    Hue[0.07537595344508677, 1, 1], 
                    Hue[0.07537595344508677, 1, 1], 
                    Hue[0.1805713877230246, 1, 1], 
                    Hue[0.1805713877230246, 1, 1], 
                    Hue[0.28585628035949945`, 1, 1], 
                    Hue[0.28585628035949945`, 1, 1], 
                    Hue[0.3912295774894582, 1, 1], 
                    Hue[0.3912295774894582, 1, 1], 
                    Hue[0.4966904112408541, 1, 1], 
                    Hue[0.4966904112408541, 1, 1], 
                    Hue[0.6022370667295118, 1, 1], 
                    Hue[0.6022370667295118, 1, 1], 
                    Hue[0.70786676887722, 1, 1], 
                    Hue[0.70786676887722, 1, 1], 
                    Hue[0.8135773679517109, 1, 1], 
                    Hue[0.8135773679517109, 1, 1], 
                    Hue[0.9193669962723475, 1, 1], 
                    Hue[0.9193669962723475, 1, 1], 
                    Hue[0.02523258448171614, 1, 1], 
                    Hue[0.02523258448171614, 1, 1], 
                    Hue[0.1311700458947892, 1, 1], 
                    Hue[0.1311700458947892, 1, 1], 
                    Hue[0.23717520772041145`, 1, 1], 
                    Hue[0.23717520772041145`, 1, 1], 
                    Hue[0.3432433458032074, 1, 1], 
                    Hue[0.3432433458032074, 1, 1], 
                    Hue[0.4493686985356077, 1, 1], 
                    Hue[0.4493686985356077, 1, 1], 
                    Hue[0.5555437495804476, 1, 1], 
                    Hue[0.5555437495804476, 1, 1], 
                    Hue[0.6617608642013143, 1, 1], 
                    Hue[0.6617608642013143, 1, 1], 
                    Hue[0.7680114358431309, 1, 1], 
                    Hue[0.7680114358431309, 1, 1], 
                    Hue[0.8742849075770401, 1, 1], 
                    Hue[0.8742849075770401, 1, 1], 
                    Hue[0.9805688072229441, 1, 1], 
                    Hue[0.9805688072229441, 1, 1], 
                    Hue[0.08684850786555587, 1, 1], 
                    Hue[0.08684850786555587, 1, 1], 
                    Hue[0.1931073492585604, 1, 1], 
                    Hue[0.1931073492585604, 1, 1], 
                    Hue[0.29932613581724765`, 1, 1], 
                    Hue[0.29932613581724765`, 1, 1], 
                    Hue[0.4054832774981767, 1, 1], 
                    Hue[0.4054832774981767, 1, 1], 
                    Hue[0.511554075856528, 1, 1], 
                    Hue[0.511554075856528, 1, 1], 
                    Hue[0.6175095361091129, 1, 1], 
                    Hue[0.6175095361091129, 1, 1], 
                    Hue[0.7233171354656784, 1, 1], 
                    Hue[0.7233171354656784, 1, 1], 
                    Hue[0.8289397327072737, 1, 1], 
                    Hue[0.8289397327072737, 1, 1], 
                    Hue[0.9343363087113633, 1, 1], 
                    Hue[0.9343363087113633, 1, 1], 
                    Hue[0.03946140402520984, 1, 1], 
                    Hue[0.03946140402520984, 1, 1], 
                    Hue[0.14426430790282302`, 1, 1], 
                    Hue[0.14426430790282302`, 1, 1], 
                    Hue[0.2486887623010038, 1, 1], 
                    Hue[0.2486887623010038, 1, 1], 
                    Hue[0.3526741834381149, 1, 1], 
                    Hue[0.3526741834381149, 1, 1], 
                    Hue[0.45615598834782883`, 1, 1], 
                    Hue[0.45615598834782883`, 1, 1], 
                    Hue[0.5590664490818031, 1, 1], 
                    Hue[0.5590664490818031, 1, 1], 
                    Hue[0.6613355896608559, 1, 1], 
                    Hue[0.6613355896608559, 1, 1], 
                    Hue[0.7628925659823964, 1, 1], 
                    Hue[0.7628925659823964, 1, 1], 
                    Hue[0.8636679499759634, 1, 1], 
                    Hue[0.8636679499759634, 1, 1], 
                    Hue[0.9635960997077214, 1, 1], 
                    Hue[0.9635960997077214, 1, 1], 
                    Hue[0.06261731240532682, 1, 1], 
                    Hue[0.06261731240532682, 1, 1], 
                    Hue[0.16067962802358593`, 1, 1], 
                    Hue[0.16067962802358593`, 1, 1], 
                    Hue[0.257741847061097, 1, 1], 
                    Hue[0.257741847061097, 1, 1], 
                    Hue[0.3537773726657826, 1, 1], 
                    Hue[0.3537773726657826, 1, 1], 
                    Hue[0.44877573855416353`, 1, 1], 
                    Hue[0.44877573855416353`, 1, 1], 
                    Hue[0.5427446454772279, 1, 1], 
                    Hue[0.5427446454772279, 1, 1], 
                    Hue[0.6357130551987016, 1, 1], 
                    Hue[0.6357130551987016, 1, 1], 
                    Hue[0.7277350307086694, 1, 1], 
                    Hue[0.7277350307086694, 1, 1], 
                    Hue[0.818891906069626, 1, 1], 
                    Hue[0.818891906069626, 1, 1], 
                    Hue[0.9092929010507722, 1, 1], 
                    Hue[0.9092929010507722, 1, 1], 
                    Hue[0.9990790665473557, 1, 1], 
                    Hue[0.9990790665473557, 1, 1], 
                    Hue[0.0884304839703989, 1, 1], 
                    Hue[0.0884304839703989, 1, 1], 
                    Hue[0.17756605313563623`, 1, 1], 
                    Hue[0.17756605313563623`, 1, 1], 
                    Hue[0.2667301868473516, 1, 1], 
                    Hue[0.2667301868473516, 1, 1], 
                    Hue[0.3561835023519964, 1, 1], 
                    Hue[0.3561835023519964, 1, 1], 
                    Hue[0.4461975135842855, 1, 1], 
                    Hue[0.4461975135842855, 1, 1], 
                    Hue[0.5370394232765678, 1, 1], 
                    Hue[0.5370394232765678, 1, 1], 
                    Hue[0.6289504058124807, 1, 1], 
                    Hue[0.6289504058124807, 1, 1], 
                    Hue[0.722133989077812, 1, 1], 
                    Hue[0.722133989077812, 1, 1], 
                    Hue[0.8167482696906572, 1, 1], 
                    Hue[0.8167482696906572, 1, 1], 
                    Hue[0.9128958389165969, 1, 1], 
                    Hue[0.9128958389165969, 1, 1], 
                    Hue[0.01062320557995812, 1, 1], 
                    Hue[0.01062320557995812, 1, 1], 
                    Hue[0.10993026899221506`, 1, 1], 
                    Hue[0.10993026899221506`, 1, 1], 
                    Hue[0.2107766388559164, 1, 1], 
                    Hue[0.2107766388559164, 1, 1], 
                    Hue[0.31309037761082614`, 1, 1], 
                    Hue[0.31309037761082614`, 1, 1], 
                    Hue[0.4167761395354911, 1, 1], 
                    Hue[0.4167761395354911, 1, 1], 
                    Hue[0.5217208944670482, 1, 1], 
                    Hue[0.5217208944670482, 1, 1], 
                    Hue[0.6277983545596304, 1, 1], 
                    Hue[0.6277983545596304, 1, 1], 
                    Hue[0.7348794471728705, 1, 1], 
                    Hue[0.7348794471728705, 1, 1], 
                    Hue[0.8428413068614792, 1, 1], 
                    Hue[0.8428413068614792, 1, 1], 
                    Hue[0.9515712395853337, 1, 1], 
                    Hue[0.9515712395853337, 1, 1], 
                    Hue[0.06096695036510244, 1, 1], 
                    Hue[0.06096695036510244, 1, 1], 
                    Hue[0.1709356742763382, 1, 1], 
                    Hue[0.1709356742763382, 1, 1], 
                    Hue[0.28139292300740415`, 1, 1], 
                    Hue[0.28139292300740415`, 1, 1]}],
                   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
                   Axes->{True, None},
                   AxesOrigin->{0, 0},
                   
                   Epilog->LineBox[{{1, 0.0005978036838236523}, {
                    2, 0.0008053826706155865}, {3, 0.0010783281061545478`}, {
                    4, 0.0014344865472014994`}, {5, 0.001896207945665435}, {
                    6, 0.002490598524366172}, {7, 0.0032505695525940704`}, {
                    8, 0.004215593487139871}, {9, 0.005432165107508066}, {
                    10, 0.0069555683735350775`}, {11, 0.008849580652412104}, {
                    12, 0.011187903541496564`}, {13, 0.014054462469394725`}, {
                    14, 0.017543191377605898`}, {15, 0.021758767049308975`}, {
                    16, 0.02681647778534678}, {17, 0.03283999937702506}, {
                    18, 0.039961194393297074`}, {19, 0.04831812268402073}, {
                    20, 0.05805212604061957}, {21, 0.06930424471365276}, {
                    22, 0.08221315192120926}, {23, 0.09690711648582484}, {
                    24, 0.11350362619005733`}, {25, 0.13209941153498989`}, {
                    26, 0.15276682324916818`}, {27, 0.17554686627183963`}, {
                    28, 0.2004460270980389}, {29, 0.2274252707278486}, {
                    30, 0.25640041172926176`}, {31, 0.28723533620292424`}, {
                    32, 0.3197393342617633}, {33, 0.3536664144879694}, {
                    34, 0.388714848345864}, {35, 0.42453053069338237`}, {
                    36, 0.4607088517733009}, {37, 0.4968048013040667}, {
                    38, 0.5323364841100944}, {39, 0.566796855209019}, {
                    40, 0.5996671663438695}, {41, 0.6304280335187514}, {
                    42, 0.6585737678217439}, {43, 0.6836221641113502}, {
                    44, 0.7051321016931162}, {45, 0.7227158273629761}, {
                    46, 0.7360535445923382}, {47, 0.7448993289637308}, {
                    48, 0.7490843082657368}, {49, 0.7485282257909024}, {
                    50, 0.7432430662614795}, {51, 0.733328891710943}, {
                    52, 0.7189740119957582}, {53, 0.7004475353997589}, {
                    54, 0.6780871567544573}, {55, 0.6522926770282905}, {
                    56, 0.6235164860247326}, {57, 0.5922461540627697}, {
                    58, 0.558992725196141}, {59, 0.5242751410811061}, {
                    60, 0.48860999161873314`}, {61, 0.45249624849738507`}, {
                    62, 0.4164082649414819}, {63, 0.38078040520181694`}, {
                    64, 0.3460043305384273}, {65, 0.3124209307742743}, {
                    66, 0.2803180865445572}, {67, 0.2499280460164679}, {
                    68, 0.2214275591471378}, {69, 0.19494063287362245`}, {
                    70, 0.1705404969474726}, {71, 0.1482541170912516}, {
                    72, 0.1280679962072261}, {73, 0.10993357751924059`}, {
                    74, 0.0937725318990729}, {75, 0.0794835218552028}, {
                    76, 0.0669474933402219}, {77, 0.05603350931486707}, {
                    78, 0.046603485356780036`}, {79, 0.03851640133275329}, {
                    80, 0.031632254938806115`}, {81, 0.02581501719694651}, {
                    82, 0.02093490744124618}, {83, 0.016870470504046636`}, {
                    84, 0.013509519133381805`}, {85, 0.010750021626502956`}, {
                    86, 0.008500339470996873}, {87, 0.006679120039506415}, {
                    88, 0.005215018758582762}, {89, 0.004046182627650348}, {
                    90, 0.003119522808065956}, {91, 0.0023899180121836425`}, {
                    92, 0.0018193916773749262`}, {
                    93, 0.0013763050395730976`}, {94, 0.001034537652863203}, {
                    95, 0.0007727106562897323}, {96, 0.0005734831336549152}, {
                    97, 0.00042291475674991456`}, {
                    98, 0.00030989002993746955`}, {
                    99, 0.00022561955752772904`}, {
                    100, 0.00016321121780623593`}, {
                    101, 0.00011730456036459204`}, {
                    102, 0.000083764047120075}, {
                    103, 0.000059424096198848745`}, {
                    104, 0.00004188041764585625}, {
                    105, 0.000029321150905136}, {
                    106, 0.000020391297449909892`}, {
                    107, 0.000014085479428168673`}, {
                    108, 9.663304179834228*^-6}, {
                    109, 6.583634863668514*^-6}, {
                    110, 4.453958166502931*^-6}, {
                    111, 2.9916622901711298`*^-6}, {
                    112, 1.9948146181534026`*^-6}, {
                    113, 1.3202243416442894`*^-6}, {
                    114, 8.671105099514225*^-7}, {
                    115, 5.650692673957306*^-7}, {
                    116, 3.652965795644892*^-7}, {
                    117, 2.3421768562283517`*^-7}, {
                    118, 1.489147050614743*^-7}, {
                    119, 9.386888440090016*^-8}, {
                    120, 5.865502851490493*^-8}, {
                    121, 3.632797659857819*^-8}, {
                    122, 2.2300329433110763`*^-8}, {
                    123, 1.3568690646364813`*^-8}, {
                    124, 8.184536091772735*^-9}, {
                    125, 4.895643838211951*^-9}, {
                    126, 2.905235223785311*^-9}, {
                    127, 1.7115079763678964`*^-9}, {
                    128, 1.001718225125788*^-9}, {
                    129, 5.830244280446148*^-10}, {
                    130, 3.3780105496683146`*^-10}, {
                    131, 1.9506109654349634`*^-10}, {
                    132, 1.1239068223834314`*^-10}, {
                    133, 6.468622379687457*^-11}, {
                    134, 3.722222072882125*^-11}, {
                    135, 2.1428379887313934`*^-11}, {
                    136, 1.2346278825006124`*^-11}, {
                    137, 7.119413969897597*^-12}, {
                    138, 4.107335708559928*^-12}, {
                    139, 2.3692065077203713`*^-12}, {
                    140, 1.3651622175567317`*^-12}, {
                    141, 7.849507907633935*^-13}, {
                    142, 4.498774206551361*^-13}, {
                    143, 2.5672636079354647`*^-13}, {
                    144, 1.4572481897223778`*^-13}, {
                    145, 8.220449138279655*^-14}, {
                    146, 4.604962947625631*^-14}, {
                    147, 2.5600720899091714`*^-14}, {
                    148, 1.4117895812743714`*^-14}, {
                    149, 7.720615155415827*^-15}, {
                    150, 4.186322110517518*^-15}, {
                    151, 2.2505628162462104`*^-15}, {
                    152, 1.1996034260298354`*^-15}, {
                    153, 6.340236047782132*^-16}, {
                    154, 3.32313812207569*^-16}, {
                    155, 1.7276135800859778`*^-16}, {
                    156, 8.910471161341757*^-17}, {
                    157, 4.560421538440369*^-17}, {
                    158, 2.3164728544126162`*^-17}, {
                    159, 1.1679503097307834`*^-17}, {
                    160, 5.846132471219055*^-18}, {
                    161, 2.905633862934292*^-18}, {
                    162, 1.4341861814967735`*^-18}, {
                    163, 7.030721417189377*^-19}, {
                    164, 3.423274259667461*^-19}, {
                    165, 1.6555775751413006`*^-19}, {
                    166, 7.953522417306659*^-20}, {
                    167, 3.7959688096682886`*^-20}, {
                    168, 1.8000289902287722`*^-20}, {
                    169, 8.480856435530723*^-21}, {
                    170, 3.970002716637773*^-21}, {
                    171, 1.8464101911452966`*^-21}, {
                    172, 8.532433986826816*^-22}, {
                    173, 3.91800403598377*^-22}, {
                    174, 1.7878642617973032`*^-22}, {
                    175, 8.107362796764301*^-23}, {
                    176, 3.653226629432639*^-23}, {
                    177, 1.635748211101474*^-23}, {
                    178, 7.278170092615176*^-24}, {
                    179, 3.2182283844083147`*^-24}, {
                    180, 1.4141216048569485`*^-24}, {
                    181, 6.174475400430278*^-25}, {
                    182, 2.678945688733389*^-25}, {
                    183, 1.155151110511368*^-25}, {
                    184, 4.950815150661714*^-26}, {
                    185, 2.1088987092434958`*^-26}, {
                    186, 8.926723624621219*^-27}, {
                    187, 3.7542069263221626`*^-27}, {
                    188, 1.5687315282769104`*^-27}, {
                    189, 6.514345816862464*^-28}, {
                    190, 2.688871098587076*^-28}, {
                    191, 1.103214026704079*^-28}, {
                    192, 4.4985869886202567`*^-29}, {
                    193, 1.822776961971838*^-29}, {
                    194, 7.33816645881475*^-30}, {
                    195, 2.9352948551187117`*^-30}, {
                    196, 1.166737308083731*^-30}, {
                    197, 4.609059873028647*^-31}, {
                    198, 1.809820168594749*^-31}, {
                    199, 7.064454269561081*^-32}, {
                    200, 2.740858340767667*^-32}, {
                    201, 1.0565945346847473`*^-32}, {
                    202, 4.045783589742848*^-33}, {
                    203, 1.53876838255855*^-33}, {
                    204, 5.8155468969536534`*^-34}, {
                    205, 2.1849935211684695`*^-34}, {
                    206, 8.162098256350196*^-35}, {
                    207, 3.030598622460437*^-35}, {
                    208, 1.1181252666740965`*^-35}, {
                    209, 4.0988108118497964`*^-36}, {
                    210, 1.4930859785358631`*^-36}, {
                    211, 5.404894314409645*^-37}, {
                    212, 1.9439680428861522`*^-37}, {
                    213, 6.94546997570754*^-38}, {
                    214, 2.465139926227765*^-38}, {
                    215, 8.694671700419666*^-39}, {
                    216, 3.048500162067952*^-39}, {
                    217, 1.0625106717845274`*^-39}, {
                    218, 3.679906245531537*^-40}, {
                    219, 1.2661667988279013`*^-40}, {
                    220, 4.3300427408341196`*^-41}, {
                    221, 1.4732497407239412`*^-41}, {
                    222, 4.994657259048155*^-42}, {
                    223, 1.6819930873413686`*^-42}, {
                    224, 5.578963232753847*^-43}, {
                    225, 1.7877954506559312`*^-43}, {
                    226, 6.027212593164703*^-44}, {
                    227, 2.382207389352189*^-44}, {
                    228, 1.7945376321591332`*^-44}, {
                    229, 1.6873867991283687`*^-44}, {
                    230, 1.6873867991283687`*^-44}, {
                    231, 1.6873867991283687`*^-44}, {
                    232, 1.6873867991283687`*^-44}, {
                    233, 1.6873867991283687`*^-44}, {
                    234, 1.6873867991283687`*^-44}, {
                    235, 1.6873867991283687`*^-44}, {
                    236, 1.6873867991283687`*^-44}, {
                    237, 1.6873867991283687`*^-44}, {
                    238, 1.6873867991283687`*^-44}, {
                    239, 1.6873867991283687`*^-44}, {
                    240, 1.6873867991283687`*^-44}, {
                    241, 1.6873867991283687`*^-44}, {
                    242, 1.6873867991283687`*^-44}, {
                    243, 1.6873867991283687`*^-44}, {
                    244, 1.6873867991283687`*^-44}, {
                    245, 1.6873867991283687`*^-44}, {
                    246, 1.6873867991283687`*^-44}, {
                    247, 1.6873867991283687`*^-44}, {
                    248, 1.6873867991283687`*^-44}, {
                    249, 1.6873867991283687`*^-44}, {
                    250, 1.6873867991283687`*^-44}, {
                    251, 1.6873867991283687`*^-44}}],
                   Frame->True,
                   FrameTicks->{{{1., 
                    FormBox[
                    RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
                    FormBox[
                    RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
                    FormBox[
                    RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
                    FormBox[
                    RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
                    FormBox["0", TraditionalForm]}, {157.25, 
                    FormBox["2.5`", TraditionalForm]}, {188.5, 
                    FormBox["5", TraditionalForm]}, {219.75, 
                    FormBox["7.5`", TraditionalForm]}, {251., 
                    FormBox["10", TraditionalForm]}}, Automatic, None, None},
                   PlotRange->{-0.1, 1.1},
                   PlotRangeClipping->True,
                   PlotRangePadding->{
                    Scaled[0.02], Automatic}], 2->
                  GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF2XtQ1PUax/FvejBEDUM0JTJE85h3RQRF4cNdEXG5CrLAj2UxDI2SFDWp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                    "], {
                    {GrayLevel[0.5], EdgeForm[None], 
                    GraphicsGroupBox[{PolygonBox[{{166, 582, 167}}], 
                    PolygonBox[CompressedData["
1:eJwtlmW0llUQRue8dHd3d126GxEVAbEQEBAUAQFRAQFppLu7BLu7u7u7u7sF
3M967o+91l5n5sQ7Z853b60xU4ZMziLiEOTNJUsReSCfHI7CMcgDhaAosWJQ
OHcs8JQ8tyAUxosk5+YXeAERXjOv1k6OaaygclPuXCiOl9AaeGmojteAMtpb
Y3KdAS8LtfE6UC48pyReKjm3OJTHK2hNxaAyXkU5Yari1ZL3Uk5FvFJyrvas
iddK3kt71MXrQXm8IQzAT4BGeCVoijeDyngF5SgGFfUt0A5vr28KjzXGmyTP
rQZt8LbJuVWgJd4KqobXbI63SI5pjQ54R6gZphPeWWfGG0B//Pjks9aGbnh3
1Syc0wXvmhzTN9XHGySfXTk98J5QN0wvvLdqELl3oNon17o+9MOPS95bOX3w
vskx9VkfmqovHA7X7ERiJ0FjvAkMwgdD0zBD8FNU0zBD8VOheZjT8NNVkzBn
4GdCyzDD8LNUwzDD8RGQE2Ykfja0DjMKHw1twozBz9GdhBmLj4N2Yc7Fz9Od
qrYwHZ+hGutuYAI+UXcUZhJ+AXQKMxmfojvDu8LF+CXJa3WBafhFyTHlTMUv
TI6pJ3J09uTe0Z4z8Uuhh2oLG/CNqjHeE+bgl0GvMHPxebrTMPPxBdAnzEJ8
ke4wzGJ8CfQLczm+VHceZhm+HPqHWYGvVA+GWYWvhgFh1uBr9YbCrMPXJ59d
3zALn518dn3DJnwzDAyzBd8KJ4fZhm+HQeGeGqhYcm8Nhl34bhgSztmB70yO
aWwPvlc9F2Yfvh+GhjmAH1QPqtfgGvxa9Zx6EQ7jV6on8RFwG347jAyPXYVf
nTx3ONyC35qcOwxuwG9Uz4bXvA6/PjmmNe7A71TPhrkLvxtGqRfhCfxJ9SQ+
Gu7D74cx4Zx78HuTY7PhJ/xnmBPOeQB/UD0f5iH8YRgbPtNN+M3JZx8Hj+GP
J++tnEfwR5NjWvMX/Ff1XPhMT+FPw3j8fHgOfx4m4BPhJfxlmBTmFfxVvZkw
r+Gvw+Qwb+Bv6g2FeQt/G6aGeQd/V28mzHv4+zAtvOcL+IvJe2vsA/xDvTm9
RfgE/1RvEp8BX+Ffw8zw2Gf45zA9zBf4l8m5yvkG/1ZvMrzmR/jHyWurBs/g
zybXYhb8gP+YfDea8x3+fXJMvzHjlZv826Ka/ob/DnPx7dCU39ZmsANfAP8R
OwIL8XnwF/43zNfbhfzkFoCl4bF/iP2bPHcJ5CWWL3PuItA/BQkWh9c8Su6x
5JjWKEisECwLUxgvAsvxtVARrwTr8BVQHC8BK8M5RfFimWProSpeDTaEc0ri
pWBVmNJ4GVgdPlOG58l89jVQHq+QeW/llMXLZY6pZn9w9j+Ta6MzVSZWJfPe
euNXEDuU/La3QH1iDWArvglq43VgM74TWuKtYFd4rC5eL/Nc3UlzvEXm3G3Q
GG+S+e60ZkO8UeaY1sjBW8PuMG3wtrBHZ4NeeO/Mf1v3Qge8I+wL57TD22eO
bYSaeK3MZ1dOJ7wz7A/TBe8KB8I1r47XyDz3IPTAe2beWznd8O6ZY0n9Bkfc
JvE/xHj8cg==
                    "]]}]}, {{}, {}, {
                    {Hue[0.4651544301308045, 1, 1], LineBox[{1, 2}]}, 
                    {Hue[0.5554105816111728, 1, 1], LineBox[{2, 3}]}, 
                    {Hue[0.6470845464116377, 1, 1], LineBox[{3, 4}]}, 
                    {Hue[0.7379011536823059, 1, 1], LineBox[{4, 5}]}, 
                    {Hue[0.8295664135425962, 1, 1], LineBox[{5, 6}]}, 
                    {Hue[0.9214195643235872, 1, 1], LineBox[{6, 7}]}, 
                    {Hue[0.013054762008105212`, 1, 1], LineBox[{7, 8}]}, 
                    {Hue[0.10496038717414542`, 1, 1], LineBox[{8, 9}]}, 
                    {Hue[0.19751458917685086`, 1, 1], LineBox[{9, 10}]}, 
                    {Hue[0.2897640620074146, 1, 1], LineBox[{10, 11}]}, 
                    {Hue[0.3824828729642137, 1, 1], LineBox[{11, 12}]}, 
                    {Hue[0.4753040283446231, 1, 1], LineBox[{12, 13}]}, 
                    {Hue[0.5684100351355065, 1, 1], LineBox[{13, 14}]}, 
                    {Hue[0.6616080261793253, 1, 1], LineBox[{14, 15}]}, 
                    {Hue[0.7550855940217207, 1, 1], LineBox[{15, 16}]}, 
                    {Hue[0.8487100201155822, 1, 1], LineBox[{16, 17}]}, 
                    {Hue[0.9425509991658001, 1, 1], LineBox[{17, 18}]}, 
                    {Hue[0.036609959351899514`, 1, 1], LineBox[{18, 19}]}, 
                    {Hue[0.1308358768599252, 1, 1], LineBox[{19, 20}]}, 
                    {Hue[0.22527489516639876`, 1, 1], LineBox[{20, 21}]}, 
                    {Hue[0.3199020157682274, 1, 1], LineBox[{21, 22}]}, 
                    {Hue[0.41473227333091395`, 1, 1], LineBox[{22, 23}]}, 
                    {Hue[0.509757605107152, 1, 1], LineBox[{23, 24}]}, 
                    {Hue[0.6049705984441132, 1, 1], LineBox[{24, 25}]}, 
                    {Hue[0.7003881175130392, 1, 1], LineBox[{25, 26}]}, 
                    {Hue[0.7960010298713771, 1, 1], LineBox[{26, 27}]}, 
                    {Hue[0.8918060581525241, 1, 1], LineBox[{27, 28}]}, 
                    {Hue[0.987804312899045, 1, 1], LineBox[{28, 29}]}, 
                    {Hue[0.08399480518255445, 1, 1], LineBox[{29, 30}]}, 
                    {Hue[0.18038477209683, 1, 1], LineBox[{30, 31}]}, 
                    {Hue[0.27696843286290557`, 1, 1], LineBox[{31, 32}]}, 
                    {Hue[0.37374369528133805`, 1, 1], LineBox[{32, 33}]}, 
                    {Hue[0.4707099465669432, 1, 1], LineBox[{33, 34}]}, 
                    {Hue[0.567869481609075, 1, 1], LineBox[{34, 35}]}, 
                    {Hue[0.6652240721329183, 1, 1], LineBox[{35, 36}]}, 
                    {Hue[0.7627725097368594, 1, 1], LineBox[{36, 37}]}, 
                    {Hue[0.8605108972765441, 1, 1], LineBox[{37, 38}]}, 
                    {Hue[0.958440389766678, 1, 1], LineBox[{38, 39}]}, 
                    {Hue[0.05656129026424089, 1, 1], LineBox[{39, 40}]}, 
                    {Hue[0.15487414605873545`, 1, 1], LineBox[{40, 41}]}, 
                    {Hue[0.2533794441638736, 1, 1], LineBox[{41, 42}]}, 
                    {Hue[0.35207574889560544`, 1, 1], LineBox[{42, 43}]}, 
                    {Hue[0.4509616563755431, 1, 1], LineBox[{43, 44}]}, 
                    {Hue[0.5500368479038305, 1, 1], LineBox[{44, 45}]}, 
                    {Hue[0.6493029200071799, 1, 1], LineBox[{45, 46}]}, 
                    {Hue[0.7487594785778681, 1, 1], LineBox[{46, 47}]}, 
                    {Hue[0.8484065019661553, 1, 1], LineBox[{47, 48}]}, 
                    {Hue[0.9482424326521234, 1, 1], LineBox[{48, 49}]}, 
                    {Hue[0.0482667009607877, 1, 1], LineBox[{49, 50}]}, 
                    {Hue[0.14848009586554878`, 1, 1], LineBox[{50, 51}]}, 
                    {Hue[0.24888253739037916`, 1, 1], LineBox[{51, 52}]}, 
                    {Hue[0.3494738481964279, 1, 1], LineBox[{52, 53}]}, 
                    {Hue[0.4502538400841612, 1, 1], LineBox[{53, 54}]}, 
                    {Hue[0.5512215313210937, 1, 1], LineBox[{54, 55}]}, 
                    {Hue[0.6523770888719254, 1, 1], LineBox[{55, 56}]}, 
                    {Hue[0.7537211195726249, 1, 1], LineBox[{56, 57}]}, 
                    {Hue[0.8552526263705517, 1, 1], LineBox[{57, 58}]}, 
                    {Hue[0.956971653741671, 1, 1], LineBox[{58, 59}]}, 
                    {Hue[0.05887788638436986, 1, 1], LineBox[{59, 60}]}, 
                    {Hue[0.1609711068475429, 1, 1], LineBox[{60, 61}]}, 
                    {Hue[0.263251461198306, 1, 1], LineBox[{61, 62}]}, 
                    {Hue[0.36571881160627995`, 1, 1], LineBox[{62, 63}]}, 
                    {Hue[0.46837235562530183`, 1, 1], LineBox[{63, 64}]}, 
                    {Hue[0.571212188793546, 1, 1], LineBox[{64, 65}]}, 
                    {Hue[0.6742384387634822, 1, 1], LineBox[{65, 66}]}, 
                    {Hue[0.7774505905900797, 1, 1], LineBox[{66, 67}]}, 
                    {Hue[0.88084887240729, 1, 1], LineBox[{67, 68}]}, 
                    {Hue[0.9844326836374937, 1, 1], LineBox[{68, 69}]}, 
                    {Hue[0.08820140979164921, 1, 1], LineBox[{69, 70}]}, 
                    {Hue[0.1921553760160884, 1, 1], LineBox[{70, 71}]}, 
                    {Hue[0.2962945397620034, 1, 1], LineBox[{71, 72}]}, 
                    {Hue[0.40061857163953374`, 1, 1], LineBox[{72, 73}]}, 
                    {Hue[0.5051269690985097, 1, 1], LineBox[{73, 74}]}, 
                    {Hue[0.6098193140164784, 1, 1], LineBox[{74, 75}]}, 
                    {Hue[0.7146956052782951, 1, 1], LineBox[{75, 76}]}, 
                    {Hue[0.8197550348620458, 1, 1], LineBox[{76, 77}]}, 
                    {Hue[0.92499766979028, 1, 1], LineBox[{77, 78}]}, 
                    {Hue[0.030422802443574908`, 1, 1], LineBox[{78, 79}]}, 
                    {Hue[0.13603006751000404`, 1, 1], LineBox[{79, 80}]}, 
                    {Hue[0.24181881541715228`, 1, 1], LineBox[{80, 81}]}, 
                    {Hue[0.34778816817848446`, 1, 1], LineBox[{81, 82}]}, 
                    {Hue[0.4539371094543324, 1, 1], LineBox[{82, 83}]}, 
                    {Hue[0.5602651505544005, 1, 1], LineBox[{83, 84}]}, 
                    {Hue[0.6667712634439584, 1, 1], LineBox[{84, 85}]}, 
                    {Hue[0.7734535461630561, 1, 1], LineBox[{85, 86}]}, 
                    {Hue[0.8803101814100706, 1, 1], LineBox[{86, 87}]}, 
                    {Hue[0.9873409380665711, 1, 1], LineBox[{87, 88}]}, 
                    {Hue[0.09454433494227246, 1, 1], LineBox[{88, 89}]}, 
                    {Hue[0.20191668511525024`, 1, 1], LineBox[{89, 90}]}, 
                    {Hue[0.3094548457672087, 1, 1], LineBox[{90, 91}]}, 
                    {Hue[0.4171562070936044, 1, 1], LineBox[{91, 92}]}, 
                    {Hue[0.5250179871183952, 1, 1], LineBox[{92, 93}]}, 
                    {Hue[0.6330355272821271, 1, 1], LineBox[{93, 94}]}, 
                    {Hue[0.7412034759242306, 1, 1], LineBox[{94, 95}]}, 
                    {Hue[0.849516723315727, 1, 1], LineBox[{95, 96}]}, 
                    {Hue[0.9579680811132738, 1, 1], LineBox[{96, 97}]}, 
                    {Hue[0.06654996696082718, 1, 1], LineBox[{97, 98}]}, 
                    {Hue[0.1752542034971408, 1, 1], LineBox[{98, 99}]}, 
                    {Hue[0.28406982820136284`, 1, 1], LineBox[{99, 100}]}, 
                    {Hue[0.39298448987471163`, 1, 1], LineBox[{100, 101}]}, 
                    {Hue[0.5019841493283912, 1, 1], LineBox[{101, 102}]}, 
                    {Hue[0.6110525350493271, 1, 1], LineBox[{102, 103}]}, 
                    {Hue[0.7201713819939524, 1, 1], LineBox[{103, 104}]}, 
                    {Hue[0.8293217984945689, 1, 1], LineBox[{104, 105}]}, 
                    {Hue[0.9384800401343466, 1, 1], LineBox[{105, 106}]}, 
                    {Hue[0.04761399197175331, 1, 1], LineBox[{106, 107}]}, 
                    {Hue[0.15668862421436977`, 1, 1], LineBox[{107, 108}]}, 
                    {Hue[0.26567158533576374`, 1, 1], LineBox[{108, 109}]}, 
                    {Hue[0.37452804323486194`, 1, 1], LineBox[{109, 110}]}, 
                    {Hue[0.48321158195502617`, 1, 1], LineBox[{110, 111}]}, 
                    {Hue[0.591665801751543, 1, 1], LineBox[{111, 112}]}, 
                    {Hue[0.6998335322551498, 1, 1], LineBox[{112, 113}]}, 
                    {Hue[0.8076561474609649, 1, 1], LineBox[{113, 114}]}, 
                    {Hue[0.9150674728730183, 1, 1], LineBox[{114, 115}]}, 
                    {Hue[0.021995142902907924`, 1, 1], LineBox[{115, 116}]}, 
                    {Hue[0.12836837744267132`, 1, 1], LineBox[{116, 117}]}, 
                    {Hue[0.2341169635451068, 1, 1], LineBox[{117, 118}]}, 
                    {Hue[0.3391660604792021, 1, 1], LineBox[{118, 119}]}, 
                    {Hue[0.4434364948665046, 1, 1], LineBox[{119, 120}]}, 
                    {Hue[0.546852954234411, 1, 1], LineBox[{120, 121}]}, 
                    {Hue[0.6493540844131561, 1, 1], LineBox[{121, 122}]}, 
                    {Hue[0.7508968981911877, 1, 1], LineBox[{122, 123}]}, 
                    {Hue[0.851451910761745, 1, 1], LineBox[{123, 124}]}, 
                    {Hue[0.9509963989571802, 1, 1], LineBox[{124, 125}]}, 
                    {Hue[0.04951252491739281, 1, 1], LineBox[{125, 126}]}, 
                    {Hue[0.1469910489023467, 1, 1], LineBox[{126, 127}]}, 
                    {Hue[0.2434442346796176, 1, 1], LineBox[{127, 128}]}, 
                    {Hue[0.33891490119097384`, 1, 1], LineBox[{128, 129}]}, 
                    {Hue[0.43345638334230663`, 1, 1], LineBox[{129, 130}]}, 
                    {Hue[0.527095344156284, 1, 1], LineBox[{130, 131}]}, 
                    {Hue[0.6198251547570155, 1, 1], LineBox[{131, 132}]}, 
                    {Hue[0.7116325629947977, 1, 1], LineBox[{132, 133}]}, 
                    {Hue[0.8025203294870583, 1, 1], LineBox[{133, 134}]}, 
                    {Hue[0.892489925483982, 1, 1], LineBox[{134, 135}]}, 
                    {Hue[0.9814950793701384, 1, 1], LineBox[{135, 136}]}, 
                    {Hue[0.06940879585274738, 1, 1], LineBox[{136, 137}]}, 
                    {Hue[0.1560379167494746, 1, 1], LineBox[{137, 138}]}, 
                    {Hue[0.2411397908261848, 1, 1], LineBox[{138, 139}]}, 
                    {Hue[0.3243570164916532, 1, 1], LineBox[{139, 140}]}, 
                    {Hue[0.40512776526272315`, 1, 1], LineBox[{140, 141}]}, 
                    {Hue[0.4827689796728388, 1, 1], LineBox[{141, 142}]}, 
                    {Hue[0.5567366414205508, 1, 1], LineBox[{142, 143}]}, 
                    {Hue[0.6268618216854998, 1, 1], LineBox[{143, 144}]}, 
                    {Hue[0.6936551818635579, 1, 1], LineBox[{144, 145}]}, 
                    {Hue[0.7588789911312471, 1, 1], LineBox[{145, 146}]}, 
                    {Hue[0.8254234195078273, 1, 1], LineBox[{146, 147}]}, 
                    {Hue[0.8957871458702045, 1, 1], LineBox[{147, 148}]}, 
                    {Hue[0.9712803960295606, 1, 1], LineBox[{148, 149}]}, 
                    {Hue[0.0526909438225915, 1, 1], LineBox[{149, 150}]}, 
                    {Hue[0.14028738520395323`, 1, 1], LineBox[{150, 151}]}, 
                    {Hue[0.23330222879300616`, 1, 1], LineBox[{151, 152}]}, 
                    {Hue[0.3304408906001906, 1, 1], LineBox[{152, 153}]}, 
                    {Hue[0.43089611044311255`, 1, 1], LineBox[{153, 154}]}, 
                    {Hue[0.5345867051543077, 1, 1], LineBox[{154, 155}]}, 
                    {Hue[0.6416275575907403, 1, 1], LineBox[{155, 156}]}, 
                    {Hue[0.7519045675691594, 1, 1], LineBox[{156, 157}]}, 
                    {Hue[0.865098064379245, 1, 1], LineBox[{157, 158}]}, 
                    {Hue[0.9808402665833201, 1, 1], LineBox[{158, 159}]}, 
                    {Hue[0.09878598465087692, 1, 1], LineBox[{159, 160}]}, 
                    {Hue[0.2186316551202424, 1, 1], LineBox[{160, 161}]}, 
                    {Hue[0.3401338285178514, 1, 1], LineBox[{161, 162}]}, 
                    {Hue[0.4631214068126678, 1, 1], LineBox[{162, 163}]}, 
                    {Hue[0.5874801753732921, 1, 1], LineBox[{163, 164}]}, 
                    {Hue[0.7130934559046723, 1, 1], LineBox[{164, 165}]}, 
                    {Hue[0.8397487315274839, 1, 1], LineBox[{165, 166}]}, 
                    {Hue[0.9670748643982374, 1, 1], LineBox[{166, 167}]}, 
                    {Hue[0.09457976930817419, 1, 1], LineBox[{167, 168}]}, 
                    {Hue[0.22179504405250422`, 1, 1], LineBox[{168, 169}]}, 
                    {Hue[0.3484568885698326, 1, 1], LineBox[{169, 170}]}, 
                    {Hue[0.4746096680737803, 1, 1], LineBox[{170, 171}]}, 
                    {Hue[0.6005198489903314, 1, 1], LineBox[{171, 172}]}, 
                    {Hue[0.7263863088486961, 1, 1], LineBox[{172, 173}]}, 
                    {Hue[0.8520468555923485, 1, 1], LineBox[{173, 174}]}, 
                    {Hue[0.9770017580001946, 1, 1], LineBox[{174, 175}]}, 
                    {Hue[0.10087798050703355`, 1, 1], LineBox[{175, 176}]}, 
                    {Hue[0.22399620151989078`, 1, 1], LineBox[{176, 177}]}, 
                    {Hue[0.34734765213365987`, 1, 1], LineBox[{177, 178}]}, 
                    {Hue[0.47175243518309923`, 1, 1], LineBox[{178, 179}]}, 
                    {Hue[0.5970599486235515, 1, 1], LineBox[{179, 180}]}, 
                    {Hue[0.722304988072564, 1, 1], LineBox[{180, 181}]}, 
                    {Hue[0.8467009602545368, 1, 1], LineBox[{181, 182}]}, 
                    {Hue[0.9706106571787333, 1, 1], LineBox[{182, 183}]}, 
                    {Hue[0.09543940162180947, 1, 1], LineBox[{183, 184}]}, 
                    {Hue[0.2222604194299728, 1, 1], LineBox[{184, 185}]}, 
                    {Hue[0.3505885939092142, 1, 1], LineBox[{185, 186}]}, 
                    {Hue[0.47851986808596186`, 1, 1], LineBox[{186, 187}]}, 
                    {Hue[0.6040683153079265, 1, 1], LineBox[{187, 188}]}, 
                    {Hue[0.7270629798245565, 1, 1], LineBox[{188, 189}]}, 
                    {Hue[0.8500757901129055, 1, 1], LineBox[{189, 190}]}, 
                    {Hue[0.9763751814294643, 1, 1], LineBox[{190, 191}]}, 
                    {Hue[0.10674965681726335`, 1, 1], LineBox[{191, 192}]}, 
                    {Hue[0.23891176768458297`, 1, 1], LineBox[{192, 193}]}, 
                    {Hue[0.3690996557259483, 1, 1], LineBox[{193, 194}]}, 
                    {Hue[0.4944875289910466, 1, 1], LineBox[{194, 195}]}, 
                    {Hue[0.6160926085787073, 1, 1], LineBox[{195, 196}]}, 
                    {Hue[0.7389647173416496, 1, 1], LineBox[{196, 197}]}, 
                    {Hue[0.8670570961233259, 1, 1], LineBox[{197, 198}]}, 
                    {Hue[0.999696967846253, 1, 1], LineBox[{198, 199}]}, 
                    {Hue[0.13360266491970105`, 1, 1], LineBox[{199, 200}]}, 
                    {Hue[0.2661866503459161, 1, 1], LineBox[{200, 201}]}, 
                    {Hue[0.3973874472708366, 1, 1], LineBox[{201, 202}]}, 
                    {Hue[0.528600014946299, 1, 1], LineBox[{202, 203}]}, 
                    {Hue[0.6591720785215255, 1, 1], LineBox[{203, 204}]}, 
                    {Hue[0.7845489645344041, 1, 1], LineBox[{204, 205}]}, 
                    {Hue[0.9003656193469529, 1, 1], LineBox[{205, 206}]}, 
                    {Hue[0.013714912174225602`, 1, 1], LineBox[{206, 207}]}, 
                    {Hue[0.1408471718351541, 1, 1], LineBox[{207, 208}]}, 
                    {Hue[0.28661504066258015`, 1, 1], LineBox[{208, 209}]}, 
                    {Hue[0.44534697210746116`, 1, 1], LineBox[{209, 210}]}, 
                    {Hue[0.6095009105377235, 1, 1], LineBox[{210, 211}]}, 
                    {Hue[0.7690038856007232, 1, 1], LineBox[{211, 212}]}, 
                    {Hue[0.8949038714677751, 1, 1], LineBox[{212, 213}]}, 
                    {Hue[0.9086342977794758, 1, 1], LineBox[{213, 214}]}, 
                    {Hue[0.9845249830133344, 1, 1], LineBox[{214, 215}]}, 
                    {Hue[0.14123330332232742`, 1, 1], LineBox[{215, 216}]}, 
                    {Hue[0.31951791804120616`, 1, 1], LineBox[{216, 217}]}, 
                    {Hue[0.5065938980359681, 1, 1], LineBox[{217, 218}]}, 
                    {Hue[0.6998846256103239, 1, 1], LineBox[{218, 219}]}, 
                    {Hue[0.9016850504011885, 1, 1], LineBox[{219, 220}]}, 
                    {Hue[0.12781140313159747`, 1, 1], LineBox[{220, 221}]}, 
                    {Hue[0.5089285539277778, 1, 1], LineBox[{221, 222}]}, 
                    {Hue[0.8765034481206417, 1, 1], LineBox[{222, 223}]}, 
                    {Hue[0.10736464903210238`, 1, 1], LineBox[{223, 224}]}, 
                    {Hue[0.3213807049021133, 1, 1], LineBox[{224, 225}]}, 
                    {Hue[0.5341230584416392, 1, 1], LineBox[{225, 226}]}, 
                    {Hue[0.7506241615262762, 1, 1], LineBox[{226, 227}]}, 
                    {Hue[0.9751545195339061, 1, 1], LineBox[{227, 228}]}, 
                    {Hue[0.2140165315639541, 1, 1], LineBox[{228, 229}]}, 
                    {Hue[0.4755204440520452, 1, 1], LineBox[{229, 230}]}, 
                    {Hue[0.756792506749526, 1, 1], LineBox[{230, 231}]}, 
                    {Hue[0.030535038964111475`, 1, 1], LineBox[{231, 232}]}, 
                    {Hue[0.28164947178050603`, 1, 1], LineBox[{232, 233}]}, 
                    {Hue[0.5162105627879208, 1, 1], LineBox[{233, 234}]}, 
                    {Hue[0.7416446994261499, 1, 1], LineBox[{234, 235}]}, 
                    {Hue[0.9624384494465095, 1, 1], LineBox[{235, 236}]}, 
                    {Hue[0.1814804022546692, 1, 1], LineBox[{236, 237}]}, 
                    {Hue[0.401511966546178, 1, 1], LineBox[{237, 238}]}, 
                    {Hue[0.6248849205239658, 1, 1], LineBox[{238, 239}]}, 
                    {Hue[0.8526414952731264, 1, 1], LineBox[{239, 240}]}, 
                    {Hue[0.10869555432495916`, 1, 1], LineBox[{240, 241}]}, 
                    {Hue[0.42386784600606003`, 1, 1], LineBox[{241, 242}]}, 
                    {Hue[0.6301322507388738, 1, 1], LineBox[{242, 243}]}, 
                    {Hue[0.65362457352435, 1, 1], LineBox[{243, 244}]}, 
                    {Hue[0.6475836176504333, 1, 1], LineBox[{244, 245}]}, 
                    {Hue[0.6475836176504333, 1, 1], LineBox[{245, 246}]}, 
                    {Hue[0.6475836176504333, 1, 1], LineBox[{246, 247}]}, 
                    {Hue[0.6475836176504333, 1, 1], LineBox[{247, 248}]}, 
                    {Hue[0.6475836176504333, 1, 1], LineBox[{248, 249}]}, 
                    {Hue[0.6475836176504333, 1, 1], LineBox[{249, 250}]}, 
                    {Hue[0.6475836176504333, 1, 1], LineBox[{250, 251}]}}}},
                    VertexColors->{
                    Hue[0.4651544301308045, 1, 1], 
                    Hue[0.5554105816111728, 1, 1], 
                    Hue[0.6470845464116377, 1, 1], 
                    Hue[0.7379011536823059, 1, 1], 
                    Hue[0.8295664135425962, 1, 1], 
                    Hue[0.9214195643235872, 1, 1], 
                    Hue[0.013054762008105212`, 1, 1], 
                    Hue[0.10496038717414542`, 1, 1], 
                    Hue[0.19751458917685086`, 1, 1], 
                    Hue[0.2897640620074146, 1, 1], 
                    Hue[0.3824828729642137, 1, 1], 
                    Hue[0.4753040283446231, 1, 1], 
                    Hue[0.5684100351355065, 1, 1], 
                    Hue[0.6616080261793253, 1, 1], 
                    Hue[0.7550855940217207, 1, 1], 
                    Hue[0.8487100201155822, 1, 1], 
                    Hue[0.9425509991658001, 1, 1], 
                    Hue[0.036609959351899514`, 1, 1], 
                    Hue[0.1308358768599252, 1, 1], 
                    Hue[0.22527489516639876`, 1, 1], 
                    Hue[0.3199020157682274, 1, 1], 
                    Hue[0.41473227333091395`, 1, 1], 
                    Hue[0.509757605107152, 1, 1], 
                    Hue[0.6049705984441132, 1, 1], 
                    Hue[0.7003881175130392, 1, 1], 
                    Hue[0.7960010298713771, 1, 1], 
                    Hue[0.8918060581525241, 1, 1], 
                    Hue[0.987804312899045, 1, 1], 
                    Hue[0.08399480518255445, 1, 1], 
                    Hue[0.18038477209683, 1, 1], 
                    Hue[0.27696843286290557`, 1, 1], 
                    Hue[0.37374369528133805`, 1, 1], 
                    Hue[0.4707099465669432, 1, 1], 
                    Hue[0.567869481609075, 1, 1], 
                    Hue[0.6652240721329183, 1, 1], 
                    Hue[0.7627725097368594, 1, 1], 
                    Hue[0.8605108972765441, 1, 1], 
                    Hue[0.958440389766678, 1, 1], 
                    Hue[0.05656129026424089, 1, 1], 
                    Hue[0.15487414605873545`, 1, 1], 
                    Hue[0.2533794441638736, 1, 1], 
                    Hue[0.35207574889560544`, 1, 1], 
                    Hue[0.4509616563755431, 1, 1], 
                    Hue[0.5500368479038305, 1, 1], 
                    Hue[0.6493029200071799, 1, 1], 
                    Hue[0.7487594785778681, 1, 1], 
                    Hue[0.8484065019661553, 1, 1], 
                    Hue[0.9482424326521234, 1, 1], 
                    Hue[0.0482667009607877, 1, 1], 
                    Hue[0.14848009586554878`, 1, 1], 
                    Hue[0.24888253739037916`, 1, 1], 
                    Hue[0.3494738481964279, 1, 1], 
                    Hue[0.4502538400841612, 1, 1], 
                    Hue[0.5512215313210937, 1, 1], 
                    Hue[0.6523770888719254, 1, 1], 
                    Hue[0.7537211195726249, 1, 1], 
                    Hue[0.8552526263705517, 1, 1], 
                    Hue[0.956971653741671, 1, 1], 
                    Hue[0.05887788638436986, 1, 1], 
                    Hue[0.1609711068475429, 1, 1], 
                    Hue[0.263251461198306, 1, 1], 
                    Hue[0.36571881160627995`, 1, 1], 
                    Hue[0.46837235562530183`, 1, 1], 
                    Hue[0.571212188793546, 1, 1], 
                    Hue[0.6742384387634822, 1, 1], 
                    Hue[0.7774505905900797, 1, 1], 
                    Hue[0.88084887240729, 1, 1], 
                    Hue[0.9844326836374937, 1, 1], 
                    Hue[0.08820140979164921, 1, 1], 
                    Hue[0.1921553760160884, 1, 1], 
                    Hue[0.2962945397620034, 1, 1], 
                    Hue[0.40061857163953374`, 1, 1], 
                    Hue[0.5051269690985097, 1, 1], 
                    Hue[0.6098193140164784, 1, 1], 
                    Hue[0.7146956052782951, 1, 1], 
                    Hue[0.8197550348620458, 1, 1], 
                    Hue[0.92499766979028, 1, 1], 
                    Hue[0.030422802443574908`, 1, 1], 
                    Hue[0.13603006751000404`, 1, 1], 
                    Hue[0.24181881541715228`, 1, 1], 
                    Hue[0.34778816817848446`, 1, 1], 
                    Hue[0.4539371094543324, 1, 1], 
                    Hue[0.5602651505544005, 1, 1], 
                    Hue[0.6667712634439584, 1, 1], 
                    Hue[0.7734535461630561, 1, 1], 
                    Hue[0.8803101814100706, 1, 1], 
                    Hue[0.9873409380665711, 1, 1], 
                    Hue[0.09454433494227246, 1, 1], 
                    Hue[0.20191668511525024`, 1, 1], 
                    Hue[0.3094548457672087, 1, 1], 
                    Hue[0.4171562070936044, 1, 1], 
                    Hue[0.5250179871183952, 1, 1], 
                    Hue[0.6330355272821271, 1, 1], 
                    Hue[0.7412034759242306, 1, 1], 
                    Hue[0.849516723315727, 1, 1], 
                    Hue[0.9579680811132738, 1, 1], 
                    Hue[0.06654996696082718, 1, 1], 
                    Hue[0.1752542034971408, 1, 1], 
                    Hue[0.28406982820136284`, 1, 1], 
                    Hue[0.39298448987471163`, 1, 1], 
                    Hue[0.5019841493283912, 1, 1], 
                    Hue[0.6110525350493271, 1, 1], 
                    Hue[0.7201713819939524, 1, 1], 
                    Hue[0.8293217984945689, 1, 1], 
                    Hue[0.9384800401343466, 1, 1], 
                    Hue[0.04761399197175331, 1, 1], 
                    Hue[0.15668862421436977`, 1, 1], 
                    Hue[0.26567158533576374`, 1, 1], 
                    Hue[0.37452804323486194`, 1, 1], 
                    Hue[0.48321158195502617`, 1, 1], 
                    Hue[0.591665801751543, 1, 1], 
                    Hue[0.6998335322551498, 1, 1], 
                    Hue[0.8076561474609649, 1, 1], 
                    Hue[0.9150674728730183, 1, 1], 
                    Hue[0.021995142902907924`, 1, 1], 
                    Hue[0.12836837744267132`, 1, 1], 
                    Hue[0.2341169635451068, 1, 1], 
                    Hue[0.3391660604792021, 1, 1], 
                    Hue[0.4434364948665046, 1, 1], 
                    Hue[0.546852954234411, 1, 1], 
                    Hue[0.6493540844131561, 1, 1], 
                    Hue[0.7508968981911877, 1, 1], 
                    Hue[0.851451910761745, 1, 1], 
                    Hue[0.9509963989571802, 1, 1], 
                    Hue[0.04951252491739281, 1, 1], 
                    Hue[0.1469910489023467, 1, 1], 
                    Hue[0.2434442346796176, 1, 1], 
                    Hue[0.33891490119097384`, 1, 1], 
                    Hue[0.43345638334230663`, 1, 1], 
                    Hue[0.527095344156284, 1, 1], 
                    Hue[0.6198251547570155, 1, 1], 
                    Hue[0.7116325629947977, 1, 1], 
                    Hue[0.8025203294870583, 1, 1], 
                    Hue[0.892489925483982, 1, 1], 
                    Hue[0.9814950793701384, 1, 1], 
                    Hue[0.06940879585274738, 1, 1], 
                    Hue[0.1560379167494746, 1, 1], 
                    Hue[0.2411397908261848, 1, 1], 
                    Hue[0.3243570164916532, 1, 1], 
                    Hue[0.40512776526272315`, 1, 1], 
                    Hue[0.4827689796728388, 1, 1], 
                    Hue[0.5567366414205508, 1, 1], 
                    Hue[0.6268618216854998, 1, 1], 
                    Hue[0.6936551818635579, 1, 1], 
                    Hue[0.7588789911312471, 1, 1], 
                    Hue[0.8254234195078273, 1, 1], 
                    Hue[0.8957871458702045, 1, 1], 
                    Hue[0.9712803960295606, 1, 1], 
                    Hue[0.0526909438225915, 1, 1], 
                    Hue[0.14028738520395323`, 1, 1], 
                    Hue[0.23330222879300616`, 1, 1], 
                    Hue[0.3304408906001906, 1, 1], 
                    Hue[0.43089611044311255`, 1, 1], 
                    Hue[0.5345867051543077, 1, 1], 
                    Hue[0.6416275575907403, 1, 1], 
                    Hue[0.7519045675691594, 1, 1], 
                    Hue[0.865098064379245, 1, 1], 
                    Hue[0.9808402665833201, 1, 1], 
                    Hue[0.09878598465087692, 1, 1], 
                    Hue[0.2186316551202424, 1, 1], 
                    Hue[0.3401338285178514, 1, 1], 
                    Hue[0.4631214068126678, 1, 1], 
                    Hue[0.5874801753732921, 1, 1], 
                    Hue[0.7130934559046723, 1, 1], 
                    Hue[0.8397487315274839, 1, 1], 
                    Hue[0.9670748643982374, 1, 1], 
                    Hue[0.09457976930817419, 1, 1], 
                    Hue[0.22179504405250422`, 1, 1], 
                    Hue[0.3484568885698326, 1, 1], 
                    Hue[0.4746096680737803, 1, 1], 
                    Hue[0.6005198489903314, 1, 1], 
                    Hue[0.7263863088486961, 1, 1], 
                    Hue[0.8520468555923485, 1, 1], 
                    Hue[0.9770017580001946, 1, 1], 
                    Hue[0.10087798050703355`, 1, 1], 
                    Hue[0.22399620151989078`, 1, 1], 
                    Hue[0.34734765213365987`, 1, 1], 
                    Hue[0.47175243518309923`, 1, 1], 
                    Hue[0.5970599486235515, 1, 1], 
                    Hue[0.722304988072564, 1, 1], 
                    Hue[0.8467009602545368, 1, 1], 
                    Hue[0.9706106571787333, 1, 1], 
                    Hue[0.09543940162180947, 1, 1], 
                    Hue[0.2222604194299728, 1, 1], 
                    Hue[0.3505885939092142, 1, 1], 
                    Hue[0.47851986808596186`, 1, 1], 
                    Hue[0.6040683153079265, 1, 1], 
                    Hue[0.7270629798245565, 1, 1], 
                    Hue[0.8500757901129055, 1, 1], 
                    Hue[0.9763751814294643, 1, 1], 
                    Hue[0.10674965681726335`, 1, 1], 
                    Hue[0.23891176768458297`, 1, 1], 
                    Hue[0.3690996557259483, 1, 1], 
                    Hue[0.4944875289910466, 1, 1], 
                    Hue[0.6160926085787073, 1, 1], 
                    Hue[0.7389647173416496, 1, 1], 
                    Hue[0.8670570961233259, 1, 1], 
                    Hue[0.999696967846253, 1, 1], 
                    Hue[0.13360266491970105`, 1, 1], 
                    Hue[0.2661866503459161, 1, 1], 
                    Hue[0.3973874472708366, 1, 1], 
                    Hue[0.528600014946299, 1, 1], 
                    Hue[0.6591720785215255, 1, 1], 
                    Hue[0.7845489645344041, 1, 1], 
                    Hue[0.9003656193469529, 1, 1], 
                    Hue[0.013714912174225602`, 1, 1], 
                    Hue[0.1408471718351541, 1, 1], 
                    Hue[0.28661504066258015`, 1, 1], 
                    Hue[0.44534697210746116`, 1, 1], 
                    Hue[0.6095009105377235, 1, 1], 
                    Hue[0.7690038856007232, 1, 1], 
                    Hue[0.8949038714677751, 1, 1], 
                    Hue[0.9086342977794758, 1, 1], 
                    Hue[0.9845249830133344, 1, 1], 
                    Hue[0.14123330332232742`, 1, 1], 
                    Hue[0.31951791804120616`, 1, 1], 
                    Hue[0.5065938980359681, 1, 1], 
                    Hue[0.6998846256103239, 1, 1], 
                    Hue[0.9016850504011885, 1, 1], 
                    Hue[0.12781140313159747`, 1, 1], 
                    Hue[0.5089285539277778, 1, 1], 
                    Hue[0.8765034481206417, 1, 1], 
                    Hue[0.10736464903210238`, 1, 1], 
                    Hue[0.3213807049021133, 1, 1], 
                    Hue[0.5341230584416392, 1, 1], 
                    Hue[0.7506241615262762, 1, 1], 
                    Hue[0.9751545195339061, 1, 1], 
                    Hue[0.2140165315639541, 1, 1], 
                    Hue[0.4755204440520452, 1, 1], 
                    Hue[0.756792506749526, 1, 1], 
                    Hue[0.030535038964111475`, 1, 1], 
                    Hue[0.28164947178050603`, 1, 1], 
                    Hue[0.5162105627879208, 1, 1], 
                    Hue[0.7416446994261499, 1, 1], 
                    Hue[0.9624384494465095, 1, 1], 
                    Hue[0.1814804022546692, 1, 1], 
                    Hue[0.401511966546178, 1, 1], 
                    Hue[0.6248849205239658, 1, 1], 
                    Hue[0.8526414952731264, 1, 1], 
                    Hue[0.10869555432495916`, 1, 1], 
                    Hue[0.42386784600606003`, 1, 1], 
                    Hue[0.6301322507388738, 1, 1], 
                    Hue[0.65362457352435, 1, 1], 
                    Hue[0.6475836176504333, 1, 1], 
                    Hue[0.6475836176504333, 1, 1], 
                    Hue[0.6475836176504333, 1, 1], 
                    Hue[0.6475836176504333, 1, 1], 
                    Hue[0.6475836176504333, 1, 1], 
                    Hue[0.6475836176504333, 1, 1], 
                    Hue[0.6475836176504333, 1, 1], 
                    Hue[0.6475836176504333, 1, 1], 
                    Hue[0.4651544301308045, 1, 1], 
                    Hue[0.5554105816111728, 1, 1], 
                    Hue[0.5554105816111728, 1, 1], 
                    Hue[0.6470845464116377, 1, 1], 
                    Hue[0.6470845464116377, 1, 1], 
                    Hue[0.7379011536823059, 1, 1], 
                    Hue[0.7379011536823059, 1, 1], 
                    Hue[0.8295664135425962, 1, 1], 
                    Hue[0.8295664135425962, 1, 1], 
                    Hue[0.9214195643235872, 1, 1], 
                    Hue[0.9214195643235872, 1, 1], 
                    Hue[0.013054762008105212`, 1, 1], 
                    Hue[0.013054762008105212`, 1, 1], 
                    Hue[0.10496038717414542`, 1, 1], 
                    Hue[0.10496038717414542`, 1, 1], 
                    Hue[0.19751458917685086`, 1, 1], 
                    Hue[0.19751458917685086`, 1, 1], 
                    Hue[0.2897640620074146, 1, 1], 
                    Hue[0.2897640620074146, 1, 1], 
                    Hue[0.3824828729642137, 1, 1], 
                    Hue[0.3824828729642137, 1, 1], 
                    Hue[0.4753040283446231, 1, 1], 
                    Hue[0.4753040283446231, 1, 1], 
                    Hue[0.5684100351355065, 1, 1], 
                    Hue[0.5684100351355065, 1, 1], 
                    Hue[0.6616080261793253, 1, 1], 
                    Hue[0.6616080261793253, 1, 1], 
                    Hue[0.7550855940217207, 1, 1], 
                    Hue[0.7550855940217207, 1, 1], 
                    Hue[0.8487100201155822, 1, 1], 
                    Hue[0.8487100201155822, 1, 1], 
                    Hue[0.9425509991658001, 1, 1], 
                    Hue[0.9425509991658001, 1, 1], 
                    Hue[0.036609959351899514`, 1, 1], 
                    Hue[0.036609959351899514`, 1, 1], 
                    Hue[0.1308358768599252, 1, 1], 
                    Hue[0.1308358768599252, 1, 1], 
                    Hue[0.22527489516639876`, 1, 1], 
                    Hue[0.22527489516639876`, 1, 1], 
                    Hue[0.3199020157682274, 1, 1], 
                    Hue[0.3199020157682274, 1, 1], 
                    Hue[0.41473227333091395`, 1, 1], 
                    Hue[0.41473227333091395`, 1, 1], 
                    Hue[0.509757605107152, 1, 1], 
                    Hue[0.509757605107152, 1, 1], 
                    Hue[0.6049705984441132, 1, 1], 
                    Hue[0.6049705984441132, 1, 1], 
                    Hue[0.7003881175130392, 1, 1], 
                    Hue[0.7003881175130392, 1, 1], 
                    Hue[0.7960010298713771, 1, 1], 
                    Hue[0.7960010298713771, 1, 1], 
                    Hue[0.8918060581525241, 1, 1], 
                    Hue[0.8918060581525241, 1, 1], 
                    Hue[0.987804312899045, 1, 1], 
                    Hue[0.987804312899045, 1, 1], 
                    Hue[0.08399480518255445, 1, 1], 
                    Hue[0.08399480518255445, 1, 1], 
                    Hue[0.18038477209683, 1, 1], 
                    Hue[0.18038477209683, 1, 1], 
                    Hue[0.27696843286290557`, 1, 1], 
                    Hue[0.27696843286290557`, 1, 1], 
                    Hue[0.37374369528133805`, 1, 1], 
                    Hue[0.37374369528133805`, 1, 1], 
                    Hue[0.4707099465669432, 1, 1], 
                    Hue[0.4707099465669432, 1, 1], 
                    Hue[0.567869481609075, 1, 1], 
                    Hue[0.567869481609075, 1, 1], 
                    Hue[0.6652240721329183, 1, 1], 
                    Hue[0.6652240721329183, 1, 1], 
                    Hue[0.7627725097368594, 1, 1], 
                    Hue[0.7627725097368594, 1, 1], 
                    Hue[0.8605108972765441, 1, 1], 
                    Hue[0.8605108972765441, 1, 1], 
                    Hue[0.958440389766678, 1, 1], 
                    Hue[0.958440389766678, 1, 1], 
                    Hue[0.05656129026424089, 1, 1], 
                    Hue[0.05656129026424089, 1, 1], 
                    Hue[0.15487414605873545`, 1, 1], 
                    Hue[0.15487414605873545`, 1, 1], 
                    Hue[0.2533794441638736, 1, 1], 
                    Hue[0.2533794441638736, 1, 1], 
                    Hue[0.35207574889560544`, 1, 1], 
                    Hue[0.35207574889560544`, 1, 1], 
                    Hue[0.4509616563755431, 1, 1], 
                    Hue[0.4509616563755431, 1, 1], 
                    Hue[0.5500368479038305, 1, 1], 
                    Hue[0.5500368479038305, 1, 1], 
                    Hue[0.6493029200071799, 1, 1], 
                    Hue[0.6493029200071799, 1, 1], 
                    Hue[0.7487594785778681, 1, 1], 
                    Hue[0.7487594785778681, 1, 1], 
                    Hue[0.8484065019661553, 1, 1], 
                    Hue[0.8484065019661553, 1, 1], 
                    Hue[0.9482424326521234, 1, 1], 
                    Hue[0.9482424326521234, 1, 1], 
                    Hue[0.0482667009607877, 1, 1], 
                    Hue[0.0482667009607877, 1, 1], 
                    Hue[0.14848009586554878`, 1, 1], 
                    Hue[0.14848009586554878`, 1, 1], 
                    Hue[0.24888253739037916`, 1, 1], 
                    Hue[0.24888253739037916`, 1, 1], 
                    Hue[0.3494738481964279, 1, 1], 
                    Hue[0.3494738481964279, 1, 1], 
                    Hue[0.4502538400841612, 1, 1], 
                    Hue[0.4502538400841612, 1, 1], 
                    Hue[0.5512215313210937, 1, 1], 
                    Hue[0.5512215313210937, 1, 1], 
                    Hue[0.6523770888719254, 1, 1], 
                    Hue[0.6523770888719254, 1, 1], 
                    Hue[0.7537211195726249, 1, 1], 
                    Hue[0.7537211195726249, 1, 1], 
                    Hue[0.8552526263705517, 1, 1], 
                    Hue[0.8552526263705517, 1, 1], 
                    Hue[0.956971653741671, 1, 1], 
                    Hue[0.956971653741671, 1, 1], 
                    Hue[0.05887788638436986, 1, 1], 
                    Hue[0.05887788638436986, 1, 1], 
                    Hue[0.1609711068475429, 1, 1], 
                    Hue[0.1609711068475429, 1, 1], 
                    Hue[0.263251461198306, 1, 1], 
                    Hue[0.263251461198306, 1, 1], 
                    Hue[0.36571881160627995`, 1, 1], 
                    Hue[0.36571881160627995`, 1, 1], 
                    Hue[0.46837235562530183`, 1, 1], 
                    Hue[0.46837235562530183`, 1, 1], 
                    Hue[0.571212188793546, 1, 1], 
                    Hue[0.571212188793546, 1, 1], 
                    Hue[0.6742384387634822, 1, 1], 
                    Hue[0.6742384387634822, 1, 1], 
                    Hue[0.7774505905900797, 1, 1], 
                    Hue[0.7774505905900797, 1, 1], 
                    Hue[0.88084887240729, 1, 1], 
                    Hue[0.88084887240729, 1, 1], 
                    Hue[0.9844326836374937, 1, 1], 
                    Hue[0.9844326836374937, 1, 1], 
                    Hue[0.08820140979164921, 1, 1], 
                    Hue[0.08820140979164921, 1, 1], 
                    Hue[0.1921553760160884, 1, 1], 
                    Hue[0.1921553760160884, 1, 1], 
                    Hue[0.2962945397620034, 1, 1], 
                    Hue[0.2962945397620034, 1, 1], 
                    Hue[0.40061857163953374`, 1, 1], 
                    Hue[0.40061857163953374`, 1, 1], 
                    Hue[0.5051269690985097, 1, 1], 
                    Hue[0.5051269690985097, 1, 1], 
                    Hue[0.6098193140164784, 1, 1], 
                    Hue[0.6098193140164784, 1, 1], 
                    Hue[0.7146956052782951, 1, 1], 
                    Hue[0.7146956052782951, 1, 1], 
                    Hue[0.8197550348620458, 1, 1], 
                    Hue[0.8197550348620458, 1, 1], 
                    Hue[0.92499766979028, 1, 1], 
                    Hue[0.92499766979028, 1, 1], 
                    Hue[0.030422802443574908`, 1, 1], 
                    Hue[0.030422802443574908`, 1, 1], 
                    Hue[0.13603006751000404`, 1, 1], 
                    Hue[0.13603006751000404`, 1, 1], 
                    Hue[0.24181881541715228`, 1, 1], 
                    Hue[0.24181881541715228`, 1, 1], 
                    Hue[0.34778816817848446`, 1, 1], 
                    Hue[0.34778816817848446`, 1, 1], 
                    Hue[0.4539371094543324, 1, 1], 
                    Hue[0.4539371094543324, 1, 1], 
                    Hue[0.5602651505544005, 1, 1], 
                    Hue[0.5602651505544005, 1, 1], 
                    Hue[0.6667712634439584, 1, 1], 
                    Hue[0.6667712634439584, 1, 1], 
                    Hue[0.7734535461630561, 1, 1], 
                    Hue[0.7734535461630561, 1, 1], 
                    Hue[0.8803101814100706, 1, 1], 
                    Hue[0.8803101814100706, 1, 1], 
                    Hue[0.9873409380665711, 1, 1], 
                    Hue[0.9873409380665711, 1, 1], 
                    Hue[0.09454433494227246, 1, 1], 
                    Hue[0.09454433494227246, 1, 1], 
                    Hue[0.20191668511525024`, 1, 1], 
                    Hue[0.20191668511525024`, 1, 1], 
                    Hue[0.3094548457672087, 1, 1], 
                    Hue[0.3094548457672087, 1, 1], 
                    Hue[0.4171562070936044, 1, 1], 
                    Hue[0.4171562070936044, 1, 1], 
                    Hue[0.5250179871183952, 1, 1], 
                    Hue[0.5250179871183952, 1, 1], 
                    Hue[0.6330355272821271, 1, 1], 
                    Hue[0.6330355272821271, 1, 1], 
                    Hue[0.7412034759242306, 1, 1], 
                    Hue[0.7412034759242306, 1, 1], 
                    Hue[0.849516723315727, 1, 1], 
                    Hue[0.849516723315727, 1, 1], 
                    Hue[0.9579680811132738, 1, 1], 
                    Hue[0.9579680811132738, 1, 1], 
                    Hue[0.06654996696082718, 1, 1], 
                    Hue[0.06654996696082718, 1, 1], 
                    Hue[0.1752542034971408, 1, 1], 
                    Hue[0.1752542034971408, 1, 1], 
                    Hue[0.28406982820136284`, 1, 1], 
                    Hue[0.28406982820136284`, 1, 1], 
                    Hue[0.39298448987471163`, 1, 1], 
                    Hue[0.39298448987471163`, 1, 1], 
                    Hue[0.5019841493283912, 1, 1], 
                    Hue[0.5019841493283912, 1, 1], 
                    Hue[0.6110525350493271, 1, 1], 
                    Hue[0.6110525350493271, 1, 1], 
                    Hue[0.7201713819939524, 1, 1], 
                    Hue[0.7201713819939524, 1, 1], 
                    Hue[0.8293217984945689, 1, 1], 
                    Hue[0.8293217984945689, 1, 1], 
                    Hue[0.9384800401343466, 1, 1], 
                    Hue[0.9384800401343466, 1, 1], 
                    Hue[0.04761399197175331, 1, 1], 
                    Hue[0.04761399197175331, 1, 1], 
                    Hue[0.15668862421436977`, 1, 1], 
                    Hue[0.15668862421436977`, 1, 1], 
                    Hue[0.26567158533576374`, 1, 1], 
                    Hue[0.26567158533576374`, 1, 1], 
                    Hue[0.37452804323486194`, 1, 1], 
                    Hue[0.37452804323486194`, 1, 1], 
                    Hue[0.48321158195502617`, 1, 1], 
                    Hue[0.48321158195502617`, 1, 1], 
                    Hue[0.591665801751543, 1, 1], 
                    Hue[0.591665801751543, 1, 1], 
                    Hue[0.6998335322551498, 1, 1], 
                    Hue[0.6998335322551498, 1, 1], 
                    Hue[0.8076561474609649, 1, 1], 
                    Hue[0.8076561474609649, 1, 1], 
                    Hue[0.9150674728730183, 1, 1], 
                    Hue[0.9150674728730183, 1, 1], 
                    Hue[0.021995142902907924`, 1, 1], 
                    Hue[0.021995142902907924`, 1, 1], 
                    Hue[0.12836837744267132`, 1, 1], 
                    Hue[0.12836837744267132`, 1, 1], 
                    Hue[0.2341169635451068, 1, 1], 
                    Hue[0.2341169635451068, 1, 1], 
                    Hue[0.3391660604792021, 1, 1], 
                    Hue[0.3391660604792021, 1, 1], 
                    Hue[0.4434364948665046, 1, 1], 
                    Hue[0.4434364948665046, 1, 1], 
                    Hue[0.546852954234411, 1, 1], 
                    Hue[0.546852954234411, 1, 1], 
                    Hue[0.6493540844131561, 1, 1], 
                    Hue[0.6493540844131561, 1, 1], 
                    Hue[0.7508968981911877, 1, 1], 
                    Hue[0.7508968981911877, 1, 1], 
                    Hue[0.851451910761745, 1, 1], 
                    Hue[0.851451910761745, 1, 1], 
                    Hue[0.9509963989571802, 1, 1], 
                    Hue[0.9509963989571802, 1, 1], 
                    Hue[0.04951252491739281, 1, 1], 
                    Hue[0.04951252491739281, 1, 1], 
                    Hue[0.1469910489023467, 1, 1], 
                    Hue[0.1469910489023467, 1, 1], 
                    Hue[0.2434442346796176, 1, 1], 
                    Hue[0.2434442346796176, 1, 1], 
                    Hue[0.33891490119097384`, 1, 1], 
                    Hue[0.33891490119097384`, 1, 1], 
                    Hue[0.43345638334230663`, 1, 1], 
                    Hue[0.43345638334230663`, 1, 1], 
                    Hue[0.527095344156284, 1, 1], 
                    Hue[0.527095344156284, 1, 1], 
                    Hue[0.6198251547570155, 1, 1], 
                    Hue[0.6198251547570155, 1, 1], 
                    Hue[0.7116325629947977, 1, 1], 
                    Hue[0.7116325629947977, 1, 1], 
                    Hue[0.8025203294870583, 1, 1], 
                    Hue[0.8025203294870583, 1, 1], 
                    Hue[0.892489925483982, 1, 1], 
                    Hue[0.892489925483982, 1, 1], 
                    Hue[0.9814950793701384, 1, 1], 
                    Hue[0.9814950793701384, 1, 1], 
                    Hue[0.06940879585274738, 1, 1], 
                    Hue[0.06940879585274738, 1, 1], 
                    Hue[0.1560379167494746, 1, 1], 
                    Hue[0.1560379167494746, 1, 1], 
                    Hue[0.2411397908261848, 1, 1], 
                    Hue[0.2411397908261848, 1, 1], 
                    Hue[0.3243570164916532, 1, 1], 
                    Hue[0.3243570164916532, 1, 1], 
                    Hue[0.40512776526272315`, 1, 1], 
                    Hue[0.40512776526272315`, 1, 1], 
                    Hue[0.4827689796728388, 1, 1], 
                    Hue[0.4827689796728388, 1, 1], 
                    Hue[0.5567366414205508, 1, 1], 
                    Hue[0.5567366414205508, 1, 1], 
                    Hue[0.6268618216854998, 1, 1], 
                    Hue[0.6268618216854998, 1, 1], 
                    Hue[0.6936551818635579, 1, 1], 
                    Hue[0.6936551818635579, 1, 1], 
                    Hue[0.7588789911312471, 1, 1], 
                    Hue[0.7588789911312471, 1, 1], 
                    Hue[0.8254234195078273, 1, 1], 
                    Hue[0.8254234195078273, 1, 1], 
                    Hue[0.8957871458702045, 1, 1], 
                    Hue[0.8957871458702045, 1, 1], 
                    Hue[0.9712803960295606, 1, 1], 
                    Hue[0.9712803960295606, 1, 1], 
                    Hue[0.0526909438225915, 1, 1], 
                    Hue[0.0526909438225915, 1, 1], 
                    Hue[0.14028738520395323`, 1, 1], 
                    Hue[0.14028738520395323`, 1, 1], 
                    Hue[0.23330222879300616`, 1, 1], 
                    Hue[0.23330222879300616`, 1, 1], 
                    Hue[0.3304408906001906, 1, 1], 
                    Hue[0.3304408906001906, 1, 1], 
                    Hue[0.43089611044311255`, 1, 1], 
                    Hue[0.43089611044311255`, 1, 1], 
                    Hue[0.5345867051543077, 1, 1], 
                    Hue[0.5345867051543077, 1, 1], 
                    Hue[0.6416275575907403, 1, 1], 
                    Hue[0.6416275575907403, 1, 1], 
                    Hue[0.7519045675691594, 1, 1], 
                    Hue[0.7519045675691594, 1, 1], 
                    Hue[0.865098064379245, 1, 1], 
                    Hue[0.865098064379245, 1, 1], 
                    Hue[0.9808402665833201, 1, 1], 
                    Hue[0.9808402665833201, 1, 1], 
                    Hue[0.09878598465087692, 1, 1], 
                    Hue[0.09878598465087692, 1, 1], 
                    Hue[0.2186316551202424, 1, 1], 
                    Hue[0.2186316551202424, 1, 1], 
                    Hue[0.3401338285178514, 1, 1], 
                    Hue[0.3401338285178514, 1, 1], 
                    Hue[0.4631214068126678, 1, 1], 
                    Hue[0.4631214068126678, 1, 1], 
                    Hue[0.5874801753732921, 1, 1], 
                    Hue[0.5874801753732921, 1, 1], 
                    Hue[0.7130934559046723, 1, 1], 
                    Hue[0.7130934559046723, 1, 1], 
                    Hue[0.8397487315274839, 1, 1], 
                    Hue[0.8397487315274839, 1, 1], 
                    Hue[0.9670748643982374, 1, 1], 
                    Hue[0.9670748643982374, 1, 1]}],
                   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
                   Axes->{True, None},
                   AxesOrigin->{0, 0},
                   
                   Epilog->LineBox[{{1, 0.00002891020892332036}, {
                    2, 0.00004092709715677925}, {3, 0.0000578442623831742}, {
                    4, 0.00008132170006101293}, {5, 0.0001132123506546545}, {
                    6, 0.00015732521161777313`}, {
                    7, 0.00021706148730958605`}, {
                    8, 0.00029679391155740205`}, {9, 0.0004044437869046104}, {
                    10, 0.0005472342408465536}, {11, 0.0007357821766619172}, {
                    12, 0.0009829499454222842}, {
                    13, 0.0013059764142051215`}, {
                    14, 0.0017232519495184514`}, {15, 0.002260713272967188}, {
                    16, 0.002946800006478862}, {17, 0.0038173768386365785`}, {
                    18, 0.0049149116761541415`}, {19, 0.006288876610526376}, {
                    20, 0.007996683145507118}, {21, 0.010105666130880783`}, {
                    22, 0.012691372899594027`}, {23, 0.015840722427496182`}, {
                    24, 0.019649107316154976`}, {25, 0.02422033336326641}, {
                    26, 0.029673292220142568`}, {27, 0.0361270537731897}, {
                    28, 0.04371311709373623}, {29, 0.052564193855269684`}, {
                    30, 0.06281535460071211}, {31, 0.07460291718896549}, {
                    32, 0.0880555151108855}, {33, 0.10328974726463552`}, {
                    34, 0.12040811749593244`}, {35, 0.1394954020401599}, {
                    36, 0.16061091395511376`}, {37, 0.18378087874642873`}, {
                    38, 0.2089922199749991}, {39, 0.2361917925602136}, {
                    40, 0.2652790478412556}, {41, 0.29610895455565495`}, {
                    42, 0.3284817261818962}, {43, 0.3621433275612805}, {
                    44, 0.3967865039045263}, {45, 0.4320552259778682}, {
                    46, 0.46755521182451004`}, {47, 0.502848612260845}, {
                    48, 0.5374715173738412}, {49, 0.5709318093085728}, {
                    50, 0.6027321885660719}, {51, 0.6323769993170906}, {
                    52, 0.6593878623229199}, {53, 0.683314527710014}, {
                    54, 0.7037438284627603}, {55, 0.720314922534282}, {
                    56, 0.7327336595044417}, {57, 0.7407756379784874}, {
                    58, 0.7442929847235761}, {59, 0.7432220041413445}, {
                    60, 0.7375829745887565}, {61, 0.727482398832054}, {
                    62, 0.713104883727343}, {63, 0.6947127448381861}, {
                    64, 0.672633440982742}, {65, 0.6472501550561022}, {
                    66, 0.6189974934494784}, {67, 0.5883416268738719}, {
                    68, 0.5557685597855335}, {69, 0.5217745903424703}, {
                    70, 0.48685238284288923`}, {71, 0.45147933044212796`}, {
                    72, 0.41610763357236824`}, {73, 0.3811551145094326}, {
                    74, 0.3469971943359706}, {75, 0.31396435207500445`}, {
                    76, 0.28233459460566235`}, {77, 0.25233526155361025`}, {
                    78, 0.22414177952445052`}, {79, 0.1978791948170029}, {
                    80, 0.17362383911706927`}, {81, 0.15140880388809472`}, {
                    82, 0.13122793688418682`}, {83, 0.11304092722139888`}, {
                    84, 0.09677834565813032}, {85, 0.08234803303136913}, {
                    86, 0.06964079160188397}, {87, 0.05853433492781175}, {
                    88, 0.048897844664849976`}, {89, 0.04059741069384246}, {
                    90, 0.03349942904664628}, {91, 0.027473050215111466`}, {
                    92, 0.022392600022269244`}, {93, 0.018139521535058042`}, {
                    94, 0.014603855708234886`}, {95, 0.011684918935038858`}, {
                    96, 0.009291702582069678}, {97, 0.007342946920873598}, {
                    98, 0.0057668916461557414`}, {99, 0.004500893449613142}, {
                    100, 0.003490844303002224}, {101, 0.002690439568320477}, {
                    102, 0.002060441646598003}, {
                    103, 0.0015679265151644249`}, {
                    104, 0.001185481301271069}, {
                    105, 0.0008904909429065257}, {
                    106, 0.0006644926244939608}, {
                    107, 0.0004925486128969269}, {
                    108, 0.00036263689520873944`}, {
                    109, 0.0002651530607565262}, {
                    110, 0.00019250490900931877`}, {
                    111, 0.00013874917878154179`}, {
                    112, 0.00009926295614784529}, {
                    113, 0.00007047173943797045}, {
                    114, 0.000049635798552467466`}, {
                    115, 0.00003467388849944929}, {
                    116, 0.000024016154323433876`}, {
                    117, 0.000016486707346631874`}, {
                    118, 0.000011212525533547908`}, {
                    119, 7.551223400127575*^-6}, {
                    120, 5.033782062115359*^-6}, {
                    121, 3.3202087014511472`*^-6}, {
                    122, 2.1659889767730572`*^-6}, {
                    123, 1.3969490532916797`*^-6}, {
                    124, 8.903416292425542*^-7}, {
                    125, 5.605784753529043*^-7}, {
                    126, 3.4860769672483537`*^-7}, {
                    127, 2.1411714295504502`*^-7}, {
                    128, 1.298982749564206*^-7}, {
                    129, 7.783615867321636*^-8}, {
                    130, 4.606425442606486*^-8}, {
                    131, 2.6928475360203448`*^-8}, {
                    132, 1.5556195071803505`*^-8}, {
                    133, 8.885186758671052*^-9}, {
                    134, 5.0196350717437815`*^-9}, {
                    135, 2.8055407531168087`*^-9}, {
                    136, 1.5517446726957088`*^-9}, {
                    137, 8.498258748257083*^-10}, {
                    138, 4.6119296558491886`*^-10}, {
                    139, 2.4824138882945997`*^-10}, {
                    140, 1.3277177808329952`*^-10}, {
                    141, 7.086064999210945*^-11}, {
                    142, 3.801026333431942*^-11}, {
                    143, 2.0696721414753352`*^-11}, {
                    144, 1.1582566254040135`*^-11}, {
                    145, 6.7437210789930295`*^-12}, {
                    146, 4.101660830478816*^-12}, {
                    147, 2.588258987503109*^-12}, {
                    148, 1.677398476264057*^-12}, {
                    149, 1.1056530443190712`*^-12}, {
                    150, 7.326545945600222*^-13}, {
                    151, 4.829412105180564*^-13}, {
                    152, 3.1537391770488354`*^-13}, {
                    153, 2.043063784043066*^-13}, {
                    154, 1.314820599904095*^-13}, {
                    155, 8.394517153718718*^-14}, {
                    156, 5.302690052076813*^-14}, {
                    157, 3.306900582764145*^-14}, {
                    158, 2.033386408969542*^-14}, {
                    159, 1.2320213377579618`*^-14}, {
                    160, 7.354143039946028*^-15}, {
                    161, 4.325176770077754*^-15}, {
                    162, 2.5066362010932763`*^-15}, {
                    163, 1.4312908779257063`*^-15}, {
                    164, 8.046569771216889*^-16}, {
                    165, 4.44886744753384*^-16}, {
                    166, 2.416796690954281*^-16}, {
                    167, 1.290153750284501*^-16}, {
                    168, 6.779355076761803*^-17}, {
                    169, 3.5168030021614194`*^-17}, {
                    170, 1.8063215544407602`*^-17}, {
                    171, 9.199179135910566*^-18}, {
                    172, 4.641987822757133*^-18}, {
                    173, 2.3174779056761736`*^-18}, {
                    174, 1.1452059894028952`*^-18}, {
                    175, 5.624104126413987*^-19}, {
                    176, 2.7599947295600498`*^-19}, {
                    177, 1.3560619278634575`*^-19}, {
                    178, 6.642886965129633*^-20}, {
                    179, 3.2237954242053554`*^-20}, {
                    180, 1.546624066782157*^-20}, {
                    181, 7.370221724968152*^-21}, {
                    182, 3.514752502332634*^-21}, {
                    183, 1.6802961176895993`*^-21}, {
                    184, 7.991471929231893*^-22}, {
                    185, 3.7399007907499105`*^-22}, {
                    186, 1.7145451406333492`*^-22}, {
                    187, 7.761154651199172*^-23}, {
                    188, 3.52984693233935*^-23}, {
                    189, 1.6326750179855814`*^-23}, {
                    190, 7.619470907545238*^-24}, {
                    191, 3.509935742563039*^-24}, {
                    192, 1.568446653381133*^-24}, {
                    193, 6.7949655651238565`*^-25}, {
                    194, 2.9111091180487185`*^-25}, {
                    195, 1.2710382446206862`*^-25}, {
                    196, 5.71941839854841*^-26}, {
                    197, 2.596278265435298*^-26}, {
                    198, 1.1558779943164204`*^-26}, {
                    199, 4.9935204122097425`*^-27}, {
                    200, 2.1103569849562185`*^-27}, {
                    201, 8.838585429776227*^-28}, {
                    202, 3.668692009429326*^-28}, {
                    203, 1.4892269509956578`*^-28}, {
                    204, 5.90475119302719*^-29}, {
                    205, 2.390957975287992*^-29}, {
                    206, 1.0603451847817696`*^-29}, {
                    207, 5.060315596891892*^-30}, {
                    208, 2.3681572225553716`*^-30}, {
                    209, 1.0163040672217539`*^-30}, {
                    210, 3.8457874062906504`*^-31}, {
                    211, 1.2256088027958108`*^-31}, {
                    212, 3.0687848060388567`*^-32}, {
                    213, 9.531978210287135*^-33}, {
                    214, 6.97059595648626*^-33}, {
                    215, 4.267400202337467*^-33}, {
                    216, 2.1179288063670586`*^-33}, {
                    217, 8.987222616149098*^-34}, {
                    218, 3.2995954930822204`*^-34}, {
                    219, 1.01816079322637*^-34}, {
                    220, 2.356542731308659*^-35}, {
                    221, 4.2842353129339825`*^-36}, {
                    222, 4.054623859961678*^-36}, {
                    223, 2.8234781589665136`*^-36}, {
                    224, 1.499188288927744*^-36}, {
                    225, 6.793560946884133*^-37}, {
                    226, 2.733625636757853*^-37}, {
                    227, 9.934019281650187*^-38}, {
                    228, 3.320485400670432*^-38}, {
                    229, 1.0831024208197172`*^-38}, {
                    230, 3.945846408887182*^-39}, {
                    231, 1.688584083746897*^-39}, {
                    232, 7.477084301291403*^-40}, {
                    233, 3.1290048397775705`*^-40}, {
                    234, 1.2107924998030126`*^-40}, {
                    235, 4.3371552934346244`*^-41}, {
                    236, 1.4474842703160065`*^-41}, {
                    237, 4.5536452248293894`*^-42}, {
                    238, 1.370370313465693*^-42}, {
                    239, 3.962876021177806*^-43}, {
                    240, 1.0658162261243229`*^-43}, {
                    241, 4.261882896279717*^-44}, {
                    242, 3.073287462511899*^-44}, {
                    243, 2.2156474144813128`*^-44}, {
                    244, 2.1019476964872256`*^-44}, {
                    245, 2.1019476964872256`*^-44}, {
                    246, 2.1019476964872256`*^-44}, {
                    247, 2.1019476964872256`*^-44}, {
                    248, 2.1019476964872256`*^-44}, {
                    249, 2.1019476964872256`*^-44}, {
                    250, 2.1019476964872256`*^-44}, {
                    251, 2.1019476964872256`*^-44}}],
                   Frame->True,
                   FrameTicks->{{{1., 
                    FormBox[
                    RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
                    FormBox[
                    RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
                    FormBox[
                    RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
                    FormBox[
                    RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
                    FormBox["0", TraditionalForm]}, {157.25, 
                    FormBox["2.5`", TraditionalForm]}, {188.5, 
                    FormBox["5", TraditionalForm]}, {219.75, 
                    FormBox["7.5`", TraditionalForm]}, {251., 
                    FormBox["10", TraditionalForm]}}, Automatic, None, None},
                   PlotRange->{-0.1, 1.1},
                   PlotRangeClipping->True,
                   PlotRangePadding->{
                    Scaled[0.02], Automatic}], 3->
                  GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF2Xl0jPceBvDfEQ0uCWm1dRF36maxVZFELBFPFmSVyCr7m5lM1NYoEm1V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                    "], {
                    {GrayLevel[0.5], EdgeForm[None], 
                    GraphicsGroupBox[{PolygonBox[{{179, 608, 180}}], 
                    PolygonBox[CompressedData["
1:eJwtlgOQnVkQRrv/2LaNiW3bmWhibrBIstEmu0k2uxvbts21bdu27eB0fVM1
p+pMd1+82/e+euXGTEmdnJjZecgIGcCcP9f/kbsEl9Nz2SAnuVyQPX1MgmeA
TCYyhkNmU01uPA/kwLNAVjwbZDXVZMazeHouxkSta60YkxfPBznx/FAaLwMF
8FxQEC8EuaMWiuMlYgyeB4rixVy5iJXES7nmSoE+eF+oYelrxNyuuWONsng5
KGiiPF4h1jRREa8EhU1UxqtAERNV8WpQFC8GNfCaUDzmhmZ481gDLwF18LpQ
0kQ9vH7s2UQDvCGUNs1RC6/tGlsGmuBNXXNHTSO8sSsXe6iOp7j2Emu2wFtC
ebwCtMHbQkVTrBXe2pWLWDu8fXxmEx3wjlDZ1IPC8dldZx+xTnjnOJM4C+iG
d48zMcW64F1duerQC+/t6k3U9MB7unJpcBw/AYNNnMRPwZA4CxiID4ozibOE
IfjQOFNTLC3GunKNYDw+Ic7IVDMMHw71TIzAR0YPTDUT8UnQJHoBY/Cx0RNT
zSh8tCsXsXH4Va61YszV+DXRI7w1zMbnQJu4CzAZnxJ3Am8JM/CZ0Cp6BdPw
6a5cxGbhN7jmqgn98QGus4g1rsWvc80da9yI3xQ9NjEXnwftTMzHb44em1iA
3wIdTNyK3xY9j97CYnxJ9BjvCRvxTdAregvL8RXRYxMr8VXQzcRqfE3cCdMc
S/FlrrE9YD2+wTV31KzF17lysYeF+CLXXmLNzfiWuEPxtmE7vgP6mmJb8W2u
XMR24rsg1cRufA/0M30npIa7zjZie/F90B8fAAfxQzDQFNuPH3DlBsFR/Jjr
7kbNYfyIKxc9nopf7+pt3OHT+Jm4s/imeHN8CVeCzfhwuJ3cHTAi7iqcxy+4
cmPhYfwRGGequRO/K+6wibvxe2CUqeZR/LG4o/houB9/AMaYau7F73PlIvYg
/pBrrRjzOP4EjI+7Bq/ir8WdwyfA0/gzMDHuPryIvxRvAJ8Ez+MvuHIRexl/
xTXXIsjIZ88Ei01rPEnuKdfcscbr+Bsw2cSb+Fvxhky8jb8DU028i78XZ27i
ffwDmIZPh4/xT2BGvBX4Ef8p3gw+Ez7Hv4BZJr7Ev4o3aOJr/BuYbZrjU/wz
19g58D3+g2vuqPkW/86Viz18iH/k2kus+TP+C8zF58Hv+B8w3xT7Ff/NlYvY
n/hf8YZN/I3/AwtMPXgWf8519hH7F/8v3ni8bbiEX443bor9j1905RZCQi8y
JOpN1MSPBE+Ui98RI/CRcMF0h88y9pzrri6F7ORywDJ8OeTG88AKUywnnitR
bi0Ux0vAOlNNXjwfrDSRHy8Aq0w1JfFSsB5fDYXxIrDGVFMQL5QoF7GieLFE
a8WY0ngZ2IBvgxp4TdiOb4TyeIVEb3MLVMWrwVbTG62MV0mUi1h1PCXRXEsg
K54t0VnEGmXxconmjjVq4bVhh4k6eF3YaaIeXh92mWiAN4TdJhrhjWEPvhea
4c1hH34EuuBd4Si+H1rhreGAiTZ4Wzhooh3eHg6Z5miBt0w09jB0wjsnmjtq
OuAdE+ViD03wpon2Emt2w7vDMfw49MJ7wwlTrAfeM1EuYn3wvnDSRCreD06Z
vhMy41kSnW3E+uMD4DR+BtLwwXDWFBuID0qUOwfD8OGJ7m7UDMGHJsp53H+4
qGtuVwBRgRao
                    "]]}]}, {{}, {}, {
                    {Hue[0.2740759612382822, 1, 1], LineBox[{1, 2}]}, 
                    {Hue[0.34061641190314307`, 1, 1], LineBox[{2, 3}]}, 
                    {Hue[0.42976605858521966`, 1, 1], LineBox[{3, 4}]}, 
                    {Hue[0.5138753870902316, 1, 1], LineBox[{4, 5}]}, 
                    {Hue[0.5840491086657771, 1, 1], LineBox[{5, 6}]}, 
                    {Hue[0.6852575862750414, 1, 1], LineBox[{6, 7}]}, 
                    {Hue[0.7690215103953824, 1, 1], LineBox[{7, 8}]}, 
                    {Hue[0.8448677927359287, 1, 1], LineBox[{8, 9}]}, 
                    {Hue[0.9361130126135394, 1, 1], LineBox[{9, 10}]}, 
                    {Hue[0.018348444993657845`, 1, 1], LineBox[{10, 11}]}, 
                    {Hue[0.1067341007690407, 1, 1], LineBox[{11, 12}]}, 
                    {Hue[0.1916021807806745, 1, 1], LineBox[{12, 13}]}, 
                    {Hue[0.27772484219464055`, 1, 1], LineBox[{13, 14}]}, 
                    {Hue[0.3643349903317177, 1, 1], LineBox[{14, 15}]}, 
                    {Hue[0.45177282749741526`, 1, 1], LineBox[{15, 16}]}, 
                    {Hue[0.538908701070393, 1, 1], LineBox[{16, 17}]}, 
                    {Hue[0.6261843843359687, 1, 1], LineBox[{17, 18}]}, 
                    {Hue[0.7137393391661621, 1, 1], LineBox[{18, 19}]}, 
                    {Hue[0.8018583969042162, 1, 1], LineBox[{19, 20}]}, 
                    {Hue[0.8904918701851595, 1, 1], LineBox[{20, 21}]}, 
                    {Hue[0.9790975713397599, 1, 1], LineBox[{21, 22}]}, 
                    {Hue[0.06789470579575665, 1, 1], LineBox[{22, 23}]}, 
                    {Hue[0.15697474778197495`, 1, 1], LineBox[{23, 24}]}, 
                    {Hue[0.24655060468658918`, 1, 1], LineBox[{24, 25}]}, 
                    {Hue[0.3363665345986199, 1, 1], LineBox[{25, 26}]}, 
                    {Hue[0.4264547735004385, 1, 1], LineBox[{26, 27}]}, 
                    {Hue[0.5167730399489824, 1, 1], LineBox[{27, 28}]}, 
                    {Hue[0.6073369670309214, 1, 1], LineBox[{28, 29}]}, 
                    {Hue[0.6982587635747064, 1, 1], LineBox[{29, 30}]}, 
                    {Hue[0.7894629494103195, 1, 1], LineBox[{30, 31}]}, 
                    {Hue[0.8809698434134233, 1, 1], LineBox[{31, 32}]}, 
                    {Hue[0.9727289977353122, 1, 1], LineBox[{32, 33}]}, 
                    {Hue[0.06475513559485649, 1, 1], LineBox[{33, 34}]}, 
                    {Hue[0.15706346690015022`, 1, 1], LineBox[{34, 35}]}, 
                    {Hue[0.24967597355959564`, 1, 1], LineBox[{35, 36}]}, 
                    {Hue[0.34256934792907295`, 1, 1], LineBox[{36, 37}]}, 
                    {Hue[0.43574581435487925`, 1, 1], LineBox[{37, 38}]}, 
                    {Hue[0.5291912090168738, 1, 1], LineBox[{38, 39}]}, 
                    {Hue[0.6229120360801792, 1, 1], LineBox[{39, 40}]}, 
                    {Hue[0.7169177088892706, 1, 1], LineBox[{40, 41}]}, 
                    {Hue[0.8112095824461873, 1, 1], LineBox[{41, 42}]}, 
                    {Hue[0.9057801263860004, 1, 1], LineBox[{42, 43}]}, 
                    {Hue[0.0006257096597658727, 1, 1], LineBox[{43, 44}]}, 
                    {Hue[0.09574830413526374, 1, 1], LineBox[{44, 45}]}, 
                    {Hue[0.19114909128921104`, 1, 1], LineBox[{45, 46}]}, 
                    {Hue[0.2868292849773195, 1, 1], LineBox[{46, 47}]}, 
                    {Hue[0.3827869813995296, 1, 1], LineBox[{47, 48}]}, 
                    {Hue[0.47902067137753845`, 1, 1], LineBox[{48, 49}]}, 
                    {Hue[0.5755290682770862, 1, 1], LineBox[{49, 50}]}, 
                    {Hue[0.6723137955010745, 1, 1], LineBox[{50, 51}]}, 
                    {Hue[0.7693745554168671, 1, 1], LineBox[{51, 52}]}, 
                    {Hue[0.8667116851118895, 1, 1], LineBox[{52, 53}]}, 
                    {Hue[0.9643234476361405, 1, 1], LineBox[{53, 54}]}, 
                    {Hue[0.062208530739026324`, 1, 1], LineBox[{54, 55}]}, 
                    {Hue[0.16036799852267106`, 1, 1], LineBox[{55, 56}]}, 
                    {Hue[0.258801810944857, 1, 1], LineBox[{56, 57}]}, 
                    {Hue[0.3575091461923081, 1, 1], LineBox[{57, 58}]}, 
                    {Hue[0.4564888992590564, 1, 1], LineBox[{58, 59}]}, 
                    {Hue[0.5557409571993345, 1, 1], LineBox[{59, 60}]}, 
                    {Hue[0.655265089711039, 1, 1], LineBox[{60, 61}]}, 
                    {Hue[0.7550614056935411, 1, 1], LineBox[{61, 62}]}, 
                    {Hue[0.8551296658837929, 1, 1], LineBox[{62, 63}]}, 
                    {Hue[0.9554684222196816, 1, 1], LineBox[{63, 64}]}, 
                    {Hue[0.056077780299213835`, 1, 1], LineBox[{64, 65}]}, 
                    {Hue[0.1569576070864384, 1, 1], LineBox[{65, 66}]}, 
                    {Hue[0.2581074846488935, 1, 1], LineBox[{66, 67}]}, 
                    {Hue[0.35952720768489815`, 1, 1], LineBox[{67, 68}]}, 
                    {Hue[0.4612154447314414, 1, 1], LineBox[{68, 69}]}, 
                    {Hue[0.5631724067610944, 1, 1], LineBox[{69, 70}]}, 
                    {Hue[0.6653976393554915, 1, 1], LineBox[{70, 71}]}, 
                    {Hue[0.7678905877870311, 1, 1], LineBox[{71, 72}]}, 
                    {Hue[0.8706510630208145, 1, 1], LineBox[{72, 73}]}, 
                    {Hue[0.9736784294934824, 1, 1], LineBox[{73, 74}]}, 
                    {Hue[0.07697211848436797, 1, 1], LineBox[{74, 75}]}, 
                    {Hue[0.18053149069666805`, 1, 1], LineBox[{75, 76}]}, 
                    {Hue[0.28435568477136225`, 1, 1], LineBox[{76, 77}]}, 
                    {Hue[0.3884441911870997, 1, 1], LineBox[{77, 78}]}, 
                    {Hue[0.4927967517078755, 1, 1], LineBox[{78, 79}]}, 
                    {Hue[0.5974123653015715, 1, 1], LineBox[{79, 80}]}, 
                    {Hue[0.7022899159669522, 1, 1], LineBox[{80, 81}]}, 
                    {Hue[0.8074287288457369, 1, 1], LineBox[{81, 82}]}, 
                    {Hue[0.9128275507242074, 1, 1], LineBox[{82, 83}]}, 
                    {Hue[0.018485500919929655`, 1, 1], LineBox[{83, 84}]}, 
                    {Hue[0.12440106059642532`, 1, 1], LineBox[{84, 85}]}, 
                    {Hue[0.2305726939366271, 1, 1], LineBox[{85, 86}]}, 
                    {Hue[0.3369984854894055, 1, 1], LineBox[{86, 87}]}, 
                    {Hue[0.4436764093552376, 1, 1], LineBox[{87, 88}]}, 
                    {Hue[0.5506045244741555, 1, 1], LineBox[{88, 89}]}, 
                    {Hue[0.6577800886416443, 1, 1], LineBox[{89, 90}]}, 
                    {Hue[0.7652001869956759, 1, 1], LineBox[{90, 91}]}, 
                    {Hue[0.8728611074438166, 1, 1], LineBox[{91, 92}]}, 
                    {Hue[0.9807589144955988, 1, 1], LineBox[{92, 93}]}, 
                    {Hue[0.08888919496304422, 1, 1], LineBox[{93, 94}]}, 
                    {Hue[0.1972463957493058, 1, 1], LineBox[{94, 95}]}, 
                    {Hue[0.30582418937682976`, 1, 1], LineBox[{95, 96}]}, 
                    {Hue[0.41461542530876894`, 1, 1], LineBox[{96, 97}]}, 
                    {Hue[0.523611607046055, 1, 1], LineBox[{97, 98}]}, 
                    {Hue[0.6328035280323666, 1, 1], LineBox[{98, 99}]}, 
                    {Hue[0.7421791292100071, 1, 1], LineBox[{99, 100}]}, 
                    {Hue[0.8517262210776776, 1, 1], LineBox[{100, 101}]}, 
                    {Hue[0.9614313829679946, 1, 1], LineBox[{101, 102}]}, 
                    {Hue[0.07127581809817282, 1, 1], LineBox[{102, 103}]}, 
                    {Hue[0.18123831391879858`, 1, 1], LineBox[{103, 104}]}, 
                    {Hue[0.2912992515505204, 1, 1], LineBox[{104, 105}]}, 
                    {Hue[0.4014357821497693, 1, 1], LineBox[{105, 106}]}, 
                    {Hue[0.5116154678989273, 1, 1], LineBox[{106, 107}]}, 
                    {Hue[0.6217988884400221, 1, 1], LineBox[{107, 108}]}, 
                    {Hue[0.7319483440755872, 1, 1], LineBox[{108, 109}]}, 
                    {Hue[0.8420262140980949, 1, 1], LineBox[{109, 110}]}, 
                    {Hue[0.9519858790730588, 1, 1], LineBox[{110, 111}]}, 
                    {Hue[0.061770917376315, 1, 1], LineBox[{111, 112}]}, 
                    {Hue[0.17131950476346894`, 1, 1], LineBox[{112, 113}]}, 
                    {Hue[0.2805652433311565, 1, 1], LineBox[{113, 114}]}, 
                    {Hue[0.3894387845260012, 1, 1], LineBox[{114, 115}]}, 
                    {Hue[0.497868118280707, 1, 1], LineBox[{115, 116}]}, 
                    {Hue[0.6057737050224179, 1, 1], LineBox[{116, 117}]}, 
                    {Hue[0.7130713214880129, 1, 1], LineBox[{117, 118}]}, 
                    {Hue[0.8196801378961807, 1, 1], LineBox[{118, 119}]}, 
                    {Hue[0.9255229902660722, 1, 1], LineBox[{119, 120}]}, 
                    {Hue[0.030529801027407266`, 1, 1], LineBox[{120, 121}]}, 
                    {Hue[0.13465081125510253`, 1, 1], LineBox[{121, 122}]}, 
                    {Hue[0.23784994238387336`, 1, 1], LineBox[{122, 123}]}, 
                    {Hue[0.34007354031929565`, 1, 1], LineBox[{123, 124}]}, 
                    {Hue[0.4412627880888048, 1, 1], LineBox[{124, 125}]}, 
                    {Hue[0.5414319533204015, 1, 1], LineBox[{125, 126}]}, 
                    {Hue[0.6406952567590228, 1, 1], LineBox[{126, 127}]}, 
                    {Hue[0.7391561792610386, 1, 1], LineBox[{127, 128}]}, 
                    {Hue[0.8368067224869767, 1, 1], LineBox[{128, 129}]}, 
                    {Hue[0.933611299963676, 1, 1], LineBox[{129, 130}]}, 
                    {Hue[0.02966183917474022, 1, 1], LineBox[{130, 131}]}, 
                    {Hue[0.12515565615431143`, 1, 1], LineBox[{131, 132}]}, 
                    {Hue[0.22025100378644757`, 1, 1], LineBox[{132, 133}]}, 
                    {Hue[0.3150509908005036, 1, 1], LineBox[{133, 134}]}, 
                    {Hue[0.409700986425785, 1, 1], LineBox[{134, 135}]}, 
                    {Hue[0.5043456473843696, 1, 1], LineBox[{135, 136}]}, 
                    {Hue[0.5989362670288179, 1, 1], LineBox[{136, 137}]}, 
                    {Hue[0.6932230603805059, 1, 1], LineBox[{137, 138}]}, 
                    {Hue[0.7871257717961646, 1, 1], LineBox[{138, 139}]}, 
                    {Hue[0.8811210035608937, 1, 1], LineBox[{139, 140}]}, 
                    {Hue[0.976043668502006, 1, 1], LineBox[{140, 141}]}, 
                    {Hue[0.07233130360111649, 1, 1], LineBox[{141, 142}]}, 
                    {Hue[0.1694413486033889, 1, 1], LineBox[{142, 143}]}, 
                    {Hue[0.2660171617229767, 1, 1], LineBox[{143, 144}]}, 
                    {Hue[0.36108283356206833`, 1, 1], LineBox[{144, 145}]}, 
                    {Hue[0.45592188250768106`, 1, 1], LineBox[{145, 146}]}, 
                    {Hue[0.5541536405087248, 1, 1], LineBox[{146, 147}]}, 
                    {Hue[0.6582616045411482, 1, 1], LineBox[{147, 148}]}, 
                    {Hue[0.766613722852846, 1, 1], LineBox[{148, 149}]}, 
                    {Hue[0.874029838680562, 1, 1], LineBox[{149, 150}]}, 
                    {Hue[0.9752100114454223, 1, 1], LineBox[{150, 151}]}, 
                    {Hue[0.07288244035433329, 1, 1], LineBox[{151, 152}]}, 
                    {Hue[0.18183762518900837`, 1, 1], LineBox[{152, 153}]}, 
                    {Hue[0.3128118513040628, 1, 1], LineBox[{153, 154}]}, 
                    {Hue[0.4627513612045987, 1, 1], LineBox[{154, 155}]}, 
                    {Hue[0.6188156983257607, 1, 1], LineBox[{155, 156}]}, 
                    {Hue[0.7659699254591066, 1, 1], LineBox[{156, 157}]}, 
                    {Hue[0.900255093224109, 1, 1], LineBox[{157, 158}]}, 
                    {Hue[0.027221273580469205`, 1, 1], LineBox[{158, 159}]}, 
                    {Hue[0.14925197585231578`, 1, 1], LineBox[{159, 160}]}, 
                    {Hue[0.2642182068786057, 1, 1], LineBox[{160, 161}]}, 
                    {Hue[0.37219080794297776`, 1, 1], LineBox[{161, 162}]}, 
                    {Hue[0.47834136069253086`, 1, 1], LineBox[{162, 163}]}, 
                    {Hue[0.5866752936506326, 1, 1], LineBox[{163, 164}]}, 
                    {Hue[0.6967194221160611, 1, 1], LineBox[{164, 165}]}, 
                    {Hue[0.8091853189345573, 1, 1], LineBox[{165, 166}]}, 
                    {Hue[0.9280958405079539, 1, 1], LineBox[{166, 167}]}, 
                    {Hue[0.05495597451428205, 1, 1], LineBox[{167, 168}]}, 
                    {Hue[0.1858218646652519, 1, 1], LineBox[{168, 169}]}, 
                    {Hue[0.31408752473670615`, 1, 1], LineBox[{169, 170}]}, 
                    {Hue[0.43511710210004056`, 1, 1], LineBox[{170, 171}]}, 
                    {Hue[0.5521633524083553, 1, 1], LineBox[{171, 172}]}, 
                    {Hue[0.6750664035148826, 1, 1], LineBox[{172, 173}]}, 
                    {Hue[0.8088793717674257, 1, 1], LineBox[{173, 174}]}, 
                    {Hue[0.9514016888140933, 1, 1], LineBox[{174, 175}]}, 
                    {Hue[0.09849358523699003, 1, 1], LineBox[{175, 176}]}, 
                    {Hue[0.2469004214429708, 1, 1], LineBox[{176, 177}]}, 
                    {Hue[0.3946861926401874, 1, 1], LineBox[{177, 178}]}, 
                    {Hue[0.5410819268855549, 1, 1], LineBox[{178, 179}]}, 
                    {Hue[0.6862933635086946, 1, 1], LineBox[{179, 180}]}, 
                    {Hue[0.8313368849143955, 1, 1], LineBox[{180, 181}]}, 
                    {Hue[0.9778423483567932, 1, 1], LineBox[{181, 182}]}, 
                    {Hue[0.1276628869951627, 1, 1], LineBox[{182, 183}]}, 
                    {Hue[0.28227537150474946`, 1, 1], LineBox[{183, 184}]}, 
                    {Hue[0.44229630855671426`, 1, 1], LineBox[{184, 185}]}, 
                    {Hue[0.6073850295098555, 1, 1], LineBox[{185, 186}]}, 
                    {Hue[0.7763711353670936, 1, 1], LineBox[{186, 187}]}, 
                    {Hue[0.9471867145675702, 1, 1], LineBox[{187, 188}]}, 
                    {Hue[0.11600160479303977`, 1, 1], LineBox[{188, 189}]}, 
                    {Hue[0.27387095448388904`, 1, 1], LineBox[{189, 190}]}, 
                    {Hue[0.3962254851792768, 1, 1], LineBox[{190, 191}]}, 
                    {Hue[0.46305170608323365`, 1, 1], LineBox[{191, 192}]}, 
                    {Hue[0.5694959575492422, 1, 1], LineBox[{192, 193}]}, 
                    {Hue[0.7301243631668375, 1, 1], LineBox[{193, 194}]}, 
                    {Hue[0.9137879918990288, 1, 1], LineBox[{194, 195}]}, 
                    {Hue[0.1106167560401981, 1, 1], LineBox[{195, 196}]}, 
                    {Hue[0.7913631345256851, 1, 1], LineBox[{196, 197}]}, 
                    {Hue[0.0028615328745636315`, 1, 1], 
                    LineBox[{197, 198}]}, 
                    {Hue[0.2096236893305576, 1, 1], LineBox[{198, 199}]}, 
                    {Hue[0.4177109221353927, 1, 1], LineBox[{199, 200}]}, 
                    {Hue[0.6269797211930057, 1, 1], LineBox[{200, 201}]}, 
                    {Hue[0.8360059584538889, 1, 1], LineBox[{201, 202}]}, 
                    {Hue[0.0348818559659671, 1, 1], LineBox[{202, 203}]}, 
                    {Hue[0.847839796942091, 1, 1], LineBox[{203, 204}]}, 
                    {Hue[0.020982790830345505`, 1, 1], LineBox[{204, 205}]}, 
                    {Hue[0.24158613432752232`, 1, 1], LineBox[{205, 206}]}, 
                    {Hue[0.4741393505207045, 1, 1], LineBox[{206, 207}]}, 
                    {Hue[0.7208471264822096, 1, 1], LineBox[{207, 208}]}, 
                    {Hue[0.9892888113983458, 1, 1], LineBox[{208, 209}]}, 
                    {Hue[0.2828148033760653, 1, 1], LineBox[{209, 210}]}, 
                    {Hue[0.5800212428125786, 1, 1], LineBox[{210, 211}]}, 
                    {Hue[0.8581499613212836, 1, 1], LineBox[{211, 212}]}, 
                    {Hue[0.12028273327694287`, 1, 1], LineBox[{212, 213}]}, 
                    {Hue[0.37594161632222606`, 1, 1], LineBox[{213, 214}]}, 
                    {Hue[0.6320919036749972, 1, 1], LineBox[{214, 215}]}, 
                    {Hue[0.8934095514045742, 1, 1], LineBox[{215, 216}]}, 
                    {Hue[0.16192256336513522`, 1, 1], LineBox[{216, 217}]}, 
                    {Hue[0.43495904779326117`, 1, 1], LineBox[{217, 218}]}, 
                    {Hue[0.7055894196988116, 1, 1], LineBox[{218, 219}]}, 
                    {Hue[0.9681377653949481, 1, 1], LineBox[{219, 220}]}, 
                    {Hue[0.2213746800354963, 1, 1], LineBox[{220, 221}]}, 
                    {Hue[0.4665413997702319, 1, 1], LineBox[{221, 222}]}, 
                    {Hue[0.7051081290350656, 1, 1], LineBox[{222, 223}]}, 
                    {Hue[0.9382072452706954, 1, 1], LineBox[{223, 224}]}, 
                    {Hue[0.1672102846488853, 1, 1], LineBox[{224, 225}]}, 
                    {Hue[0.3945864673089715, 1, 1], LineBox[{225, 226}]}, 
                    {Hue[0.6236533885898886, 1, 1], LineBox[{226, 227}]}, 
                    {Hue[0.8568562407239165, 1, 1], LineBox[{227, 228}]}, 
                    {Hue[0.09470133370263792, 1, 1], LineBox[{228, 229}]}, 
                    {Hue[0.33644601534657576`, 1, 1], LineBox[{229, 230}]}, 
                    {Hue[0.5811669538459834, 1, 1], LineBox[{230, 231}]}, 
                    {Hue[0.8282212401023837, 1, 1], LineBox[{231, 232}]}, 
                    {Hue[0.07723329757905519, 1, 1], LineBox[{232, 233}]}, 
                    {Hue[0.3279418824207741, 1, 1], LineBox[{233, 234}]}, 
                    {Hue[0.5800695997267706, 1, 1], LineBox[{234, 235}]}, 
                    {Hue[0.8332640886537754, 1, 1], LineBox[{235, 236}]}, 
                    {Hue[0.087111534514982, 1, 1], LineBox[{236, 237}]}, 
                    {Hue[0.341199953193986, 1, 1], LineBox[{237, 238}]}, 
                    {Hue[0.5951926565049659, 1, 1], LineBox[{238, 239}]}, 
                    {Hue[0.8488741618835237, 1, 1], LineBox[{239, 240}]}, 
                    {Hue[0.10215276053117331`, 1, 1], LineBox[{240, 241}]}, 
                    {Hue[0.3550286828792829, 1, 1], LineBox[{241, 242}]}, 
                    {Hue[0.607548195949124, 1, 1], LineBox[{242, 243}]}, 
                    {Hue[0.8597629474201154, 1, 1], LineBox[{243, 244}]}, 
                    {Hue[0.11170520728591252`, 1, 1], LineBox[{244, 245}]}, 
                    {Hue[0.36338197592002286`, 1, 1], LineBox[{245, 246}]}, 
                    {Hue[0.614783091609361, 1, 1], LineBox[{246, 247}]}, 
                    {Hue[0.8658951219648834, 1, 1], LineBox[{247, 248}]}, 
                    {Hue[0.11671322034353572`, 1, 1], LineBox[{248, 249}]}, 
                    {Hue[0.3672457427656668, 1, 1], LineBox[{249, 250}]}, 
                    {Hue[0.6175115430478646, 1, 1], LineBox[{250, 251}]}}}},
                    VertexColors->{
                    Hue[0.2740759612382822, 1, 1], 
                    Hue[0.34061641190314307`, 1, 1], 
                    Hue[0.42976605858521966`, 1, 1], 
                    Hue[0.5138753870902316, 1, 1], 
                    Hue[0.5840491086657771, 1, 1], 
                    Hue[0.6852575862750414, 1, 1], 
                    Hue[0.7690215103953824, 1, 1], 
                    Hue[0.8448677927359287, 1, 1], 
                    Hue[0.9361130126135394, 1, 1], 
                    Hue[0.018348444993657845`, 1, 1], 
                    Hue[0.1067341007690407, 1, 1], 
                    Hue[0.1916021807806745, 1, 1], 
                    Hue[0.27772484219464055`, 1, 1], 
                    Hue[0.3643349903317177, 1, 1], 
                    Hue[0.45177282749741526`, 1, 1], 
                    Hue[0.538908701070393, 1, 1], 
                    Hue[0.6261843843359687, 1, 1], 
                    Hue[0.7137393391661621, 1, 1], 
                    Hue[0.8018583969042162, 1, 1], 
                    Hue[0.8904918701851595, 1, 1], 
                    Hue[0.9790975713397599, 1, 1], 
                    Hue[0.06789470579575665, 1, 1], 
                    Hue[0.15697474778197495`, 1, 1], 
                    Hue[0.24655060468658918`, 1, 1], 
                    Hue[0.3363665345986199, 1, 1], 
                    Hue[0.4264547735004385, 1, 1], 
                    Hue[0.5167730399489824, 1, 1], 
                    Hue[0.6073369670309214, 1, 1], 
                    Hue[0.6982587635747064, 1, 1], 
                    Hue[0.7894629494103195, 1, 1], 
                    Hue[0.8809698434134233, 1, 1], 
                    Hue[0.9727289977353122, 1, 1], 
                    Hue[0.06475513559485649, 1, 1], 
                    Hue[0.15706346690015022`, 1, 1], 
                    Hue[0.24967597355959564`, 1, 1], 
                    Hue[0.34256934792907295`, 1, 1], 
                    Hue[0.43574581435487925`, 1, 1], 
                    Hue[0.5291912090168738, 1, 1], 
                    Hue[0.6229120360801792, 1, 1], 
                    Hue[0.7169177088892706, 1, 1], 
                    Hue[0.8112095824461873, 1, 1], 
                    Hue[0.9057801263860004, 1, 1], 
                    Hue[0.0006257096597658727, 1, 1], 
                    Hue[0.09574830413526374, 1, 1], 
                    Hue[0.19114909128921104`, 1, 1], 
                    Hue[0.2868292849773195, 1, 1], 
                    Hue[0.3827869813995296, 1, 1], 
                    Hue[0.47902067137753845`, 1, 1], 
                    Hue[0.5755290682770862, 1, 1], 
                    Hue[0.6723137955010745, 1, 1], 
                    Hue[0.7693745554168671, 1, 1], 
                    Hue[0.8667116851118895, 1, 1], 
                    Hue[0.9643234476361405, 1, 1], 
                    Hue[0.062208530739026324`, 1, 1], 
                    Hue[0.16036799852267106`, 1, 1], 
                    Hue[0.258801810944857, 1, 1], 
                    Hue[0.3575091461923081, 1, 1], 
                    Hue[0.4564888992590564, 1, 1], 
                    Hue[0.5557409571993345, 1, 1], 
                    Hue[0.655265089711039, 1, 1], 
                    Hue[0.7550614056935411, 1, 1], 
                    Hue[0.8551296658837929, 1, 1], 
                    Hue[0.9554684222196816, 1, 1], 
                    Hue[0.056077780299213835`, 1, 1], 
                    Hue[0.1569576070864384, 1, 1], 
                    Hue[0.2581074846488935, 1, 1], 
                    Hue[0.35952720768489815`, 1, 1], 
                    Hue[0.4612154447314414, 1, 1], 
                    Hue[0.5631724067610944, 1, 1], 
                    Hue[0.6653976393554915, 1, 1], 
                    Hue[0.7678905877870311, 1, 1], 
                    Hue[0.8706510630208145, 1, 1], 
                    Hue[0.9736784294934824, 1, 1], 
                    Hue[0.07697211848436797, 1, 1], 
                    Hue[0.18053149069666805`, 1, 1], 
                    Hue[0.28435568477136225`, 1, 1], 
                    Hue[0.3884441911870997, 1, 1], 
                    Hue[0.4927967517078755, 1, 1], 
                    Hue[0.5974123653015715, 1, 1], 
                    Hue[0.7022899159669522, 1, 1], 
                    Hue[0.8074287288457369, 1, 1], 
                    Hue[0.9128275507242074, 1, 1], 
                    Hue[0.018485500919929655`, 1, 1], 
                    Hue[0.12440106059642532`, 1, 1], 
                    Hue[0.2305726939366271, 1, 1], 
                    Hue[0.3369984854894055, 1, 1], 
                    Hue[0.4436764093552376, 1, 1], 
                    Hue[0.5506045244741555, 1, 1], 
                    Hue[0.6577800886416443, 1, 1], 
                    Hue[0.7652001869956759, 1, 1], 
                    Hue[0.8728611074438166, 1, 1], 
                    Hue[0.9807589144955988, 1, 1], 
                    Hue[0.08888919496304422, 1, 1], 
                    Hue[0.1972463957493058, 1, 1], 
                    Hue[0.30582418937682976`, 1, 1], 
                    Hue[0.41461542530876894`, 1, 1], 
                    Hue[0.523611607046055, 1, 1], 
                    Hue[0.6328035280323666, 1, 1], 
                    Hue[0.7421791292100071, 1, 1], 
                    Hue[0.8517262210776776, 1, 1], 
                    Hue[0.9614313829679946, 1, 1], 
                    Hue[0.07127581809817282, 1, 1], 
                    Hue[0.18123831391879858`, 1, 1], 
                    Hue[0.2912992515505204, 1, 1], 
                    Hue[0.4014357821497693, 1, 1], 
                    Hue[0.5116154678989273, 1, 1], 
                    Hue[0.6217988884400221, 1, 1], 
                    Hue[0.7319483440755872, 1, 1], 
                    Hue[0.8420262140980949, 1, 1], 
                    Hue[0.9519858790730588, 1, 1], 
                    Hue[0.061770917376315, 1, 1], 
                    Hue[0.17131950476346894`, 1, 1], 
                    Hue[0.2805652433311565, 1, 1], 
                    Hue[0.3894387845260012, 1, 1], 
                    Hue[0.497868118280707, 1, 1], 
                    Hue[0.6057737050224179, 1, 1], 
                    Hue[0.7130713214880129, 1, 1], 
                    Hue[0.8196801378961807, 1, 1], 
                    Hue[0.9255229902660722, 1, 1], 
                    Hue[0.030529801027407266`, 1, 1], 
                    Hue[0.13465081125510253`, 1, 1], 
                    Hue[0.23784994238387336`, 1, 1], 
                    Hue[0.34007354031929565`, 1, 1], 
                    Hue[0.4412627880888048, 1, 1], 
                    Hue[0.5414319533204015, 1, 1], 
                    Hue[0.6406952567590228, 1, 1], 
                    Hue[0.7391561792610386, 1, 1], 
                    Hue[0.8368067224869767, 1, 1], 
                    Hue[0.933611299963676, 1, 1], 
                    Hue[0.02966183917474022, 1, 1], 
                    Hue[0.12515565615431143`, 1, 1], 
                    Hue[0.22025100378644757`, 1, 1], 
                    Hue[0.3150509908005036, 1, 1], 
                    Hue[0.409700986425785, 1, 1], 
                    Hue[0.5043456473843696, 1, 1], 
                    Hue[0.5989362670288179, 1, 1], 
                    Hue[0.6932230603805059, 1, 1], 
                    Hue[0.7871257717961646, 1, 1], 
                    Hue[0.8811210035608937, 1, 1], 
                    Hue[0.976043668502006, 1, 1], 
                    Hue[0.07233130360111649, 1, 1], 
                    Hue[0.1694413486033889, 1, 1], 
                    Hue[0.2660171617229767, 1, 1], 
                    Hue[0.36108283356206833`, 1, 1], 
                    Hue[0.45592188250768106`, 1, 1], 
                    Hue[0.5541536405087248, 1, 1], 
                    Hue[0.6582616045411482, 1, 1], 
                    Hue[0.766613722852846, 1, 1], 
                    Hue[0.874029838680562, 1, 1], 
                    Hue[0.9752100114454223, 1, 1], 
                    Hue[0.07288244035433329, 1, 1], 
                    Hue[0.18183762518900837`, 1, 1], 
                    Hue[0.3128118513040628, 1, 1], 
                    Hue[0.4627513612045987, 1, 1], 
                    Hue[0.6188156983257607, 1, 1], 
                    Hue[0.7659699254591066, 1, 1], 
                    Hue[0.900255093224109, 1, 1], 
                    Hue[0.027221273580469205`, 1, 1], 
                    Hue[0.14925197585231578`, 1, 1], 
                    Hue[0.2642182068786057, 1, 1], 
                    Hue[0.37219080794297776`, 1, 1], 
                    Hue[0.47834136069253086`, 1, 1], 
                    Hue[0.5866752936506326, 1, 1], 
                    Hue[0.6967194221160611, 1, 1], 
                    Hue[0.8091853189345573, 1, 1], 
                    Hue[0.9280958405079539, 1, 1], 
                    Hue[0.05495597451428205, 1, 1], 
                    Hue[0.1858218646652519, 1, 1], 
                    Hue[0.31408752473670615`, 1, 1], 
                    Hue[0.43511710210004056`, 1, 1], 
                    Hue[0.5521633524083553, 1, 1], 
                    Hue[0.6750664035148826, 1, 1], 
                    Hue[0.8088793717674257, 1, 1], 
                    Hue[0.9514016888140933, 1, 1], 
                    Hue[0.09849358523699003, 1, 1], 
                    Hue[0.2469004214429708, 1, 1], 
                    Hue[0.3946861926401874, 1, 1], 
                    Hue[0.5410819268855549, 1, 1], 
                    Hue[0.6862933635086946, 1, 1], 
                    Hue[0.8313368849143955, 1, 1], 
                    Hue[0.9778423483567932, 1, 1], 
                    Hue[0.1276628869951627, 1, 1], 
                    Hue[0.28227537150474946`, 1, 1], 
                    Hue[0.44229630855671426`, 1, 1], 
                    Hue[0.6073850295098555, 1, 1], 
                    Hue[0.7763711353670936, 1, 1], 
                    Hue[0.9471867145675702, 1, 1], 
                    Hue[0.11600160479303977`, 1, 1], 
                    Hue[0.27387095448388904`, 1, 1], 
                    Hue[0.3962254851792768, 1, 1], 
                    Hue[0.46305170608323365`, 1, 1], 
                    Hue[0.5694959575492422, 1, 1], 
                    Hue[0.7301243631668375, 1, 1], 
                    Hue[0.9137879918990288, 1, 1], 
                    Hue[0.1106167560401981, 1, 1], 
                    Hue[0.7913631345256851, 1, 1], 
                    Hue[0.0028615328745636315`, 1, 1], 
                    Hue[0.2096236893305576, 1, 1], 
                    Hue[0.4177109221353927, 1, 1], 
                    Hue[0.6269797211930057, 1, 1], 
                    Hue[0.8360059584538889, 1, 1], 
                    Hue[0.0348818559659671, 1, 1], 
                    Hue[0.847839796942091, 1, 1], 
                    Hue[0.020982790830345505`, 1, 1], 
                    Hue[0.24158613432752232`, 1, 1], 
                    Hue[0.4741393505207045, 1, 1], 
                    Hue[0.7208471264822096, 1, 1], 
                    Hue[0.9892888113983458, 1, 1], 
                    Hue[0.2828148033760653, 1, 1], 
                    Hue[0.5800212428125786, 1, 1], 
                    Hue[0.8581499613212836, 1, 1], 
                    Hue[0.12028273327694287`, 1, 1], 
                    Hue[0.37594161632222606`, 1, 1], 
                    Hue[0.6320919036749972, 1, 1], 
                    Hue[0.8934095514045742, 1, 1], 
                    Hue[0.16192256336513522`, 1, 1], 
                    Hue[0.43495904779326117`, 1, 1], 
                    Hue[0.7055894196988116, 1, 1], 
                    Hue[0.9681377653949481, 1, 1], 
                    Hue[0.2213746800354963, 1, 1], 
                    Hue[0.4665413997702319, 1, 1], 
                    Hue[0.7051081290350656, 1, 1], 
                    Hue[0.9382072452706954, 1, 1], 
                    Hue[0.1672102846488853, 1, 1], 
                    Hue[0.3945864673089715, 1, 1], 
                    Hue[0.6236533885898886, 1, 1], 
                    Hue[0.8568562407239165, 1, 1], 
                    Hue[0.09470133370263792, 1, 1], 
                    Hue[0.33644601534657576`, 1, 1], 
                    Hue[0.5811669538459834, 1, 1], 
                    Hue[0.8282212401023837, 1, 1], 
                    Hue[0.07723329757905519, 1, 1], 
                    Hue[0.3279418824207741, 1, 1], 
                    Hue[0.5800695997267706, 1, 1], 
                    Hue[0.8332640886537754, 1, 1], 
                    Hue[0.087111534514982, 1, 1], 
                    Hue[0.341199953193986, 1, 1], 
                    Hue[0.5951926565049659, 1, 1], 
                    Hue[0.8488741618835237, 1, 1], 
                    Hue[0.10215276053117331`, 1, 1], 
                    Hue[0.3550286828792829, 1, 1], 
                    Hue[0.607548195949124, 1, 1], 
                    Hue[0.8597629474201154, 1, 1], 
                    Hue[0.11170520728591252`, 1, 1], 
                    Hue[0.36338197592002286`, 1, 1], 
                    Hue[0.614783091609361, 1, 1], 
                    Hue[0.8658951219648834, 1, 1], 
                    Hue[0.11671322034353572`, 1, 1], 
                    Hue[0.3672457427656668, 1, 1], 
                    Hue[0.6175115430478646, 1, 1], 
                    Hue[0.8675314868122538, 1, 1], 
                    Hue[0.2740759612382822, 1, 1], 
                    Hue[0.34061641190314307`, 1, 1], 
                    Hue[0.34061641190314307`, 1, 1], 
                    Hue[0.42976605858521966`, 1, 1], 
                    Hue[0.42976605858521966`, 1, 1], 
                    Hue[0.5138753870902316, 1, 1], 
                    Hue[0.5138753870902316, 1, 1], 
                    Hue[0.5840491086657771, 1, 1], 
                    Hue[0.5840491086657771, 1, 1], 
                    Hue[0.6852575862750414, 1, 1], 
                    Hue[0.6852575862750414, 1, 1], 
                    Hue[0.7690215103953824, 1, 1], 
                    Hue[0.7690215103953824, 1, 1], 
                    Hue[0.8448677927359287, 1, 1], 
                    Hue[0.8448677927359287, 1, 1], 
                    Hue[0.9361130126135394, 1, 1], 
                    Hue[0.9361130126135394, 1, 1], 
                    Hue[0.018348444993657845`, 1, 1], 
                    Hue[0.018348444993657845`, 1, 1], 
                    Hue[0.1067341007690407, 1, 1], 
                    Hue[0.1067341007690407, 1, 1], 
                    Hue[0.1916021807806745, 1, 1], 
                    Hue[0.1916021807806745, 1, 1], 
                    Hue[0.27772484219464055`, 1, 1], 
                    Hue[0.27772484219464055`, 1, 1], 
                    Hue[0.3643349903317177, 1, 1], 
                    Hue[0.3643349903317177, 1, 1], 
                    Hue[0.45177282749741526`, 1, 1], 
                    Hue[0.45177282749741526`, 1, 1], 
                    Hue[0.538908701070393, 1, 1], 
                    Hue[0.538908701070393, 1, 1], 
                    Hue[0.6261843843359687, 1, 1], 
                    Hue[0.6261843843359687, 1, 1], 
                    Hue[0.7137393391661621, 1, 1], 
                    Hue[0.7137393391661621, 1, 1], 
                    Hue[0.8018583969042162, 1, 1], 
                    Hue[0.8018583969042162, 1, 1], 
                    Hue[0.8904918701851595, 1, 1], 
                    Hue[0.8904918701851595, 1, 1], 
                    Hue[0.9790975713397599, 1, 1], 
                    Hue[0.9790975713397599, 1, 1], 
                    Hue[0.06789470579575665, 1, 1], 
                    Hue[0.06789470579575665, 1, 1], 
                    Hue[0.15697474778197495`, 1, 1], 
                    Hue[0.15697474778197495`, 1, 1], 
                    Hue[0.24655060468658918`, 1, 1], 
                    Hue[0.24655060468658918`, 1, 1], 
                    Hue[0.3363665345986199, 1, 1], 
                    Hue[0.3363665345986199, 1, 1], 
                    Hue[0.4264547735004385, 1, 1], 
                    Hue[0.4264547735004385, 1, 1], 
                    Hue[0.5167730399489824, 1, 1], 
                    Hue[0.5167730399489824, 1, 1], 
                    Hue[0.6073369670309214, 1, 1], 
                    Hue[0.6073369670309214, 1, 1], 
                    Hue[0.6982587635747064, 1, 1], 
                    Hue[0.6982587635747064, 1, 1], 
                    Hue[0.7894629494103195, 1, 1], 
                    Hue[0.7894629494103195, 1, 1], 
                    Hue[0.8809698434134233, 1, 1], 
                    Hue[0.8809698434134233, 1, 1], 
                    Hue[0.9727289977353122, 1, 1], 
                    Hue[0.9727289977353122, 1, 1], 
                    Hue[0.06475513559485649, 1, 1], 
                    Hue[0.06475513559485649, 1, 1], 
                    Hue[0.15706346690015022`, 1, 1], 
                    Hue[0.15706346690015022`, 1, 1], 
                    Hue[0.24967597355959564`, 1, 1], 
                    Hue[0.24967597355959564`, 1, 1], 
                    Hue[0.34256934792907295`, 1, 1], 
                    Hue[0.34256934792907295`, 1, 1], 
                    Hue[0.43574581435487925`, 1, 1], 
                    Hue[0.43574581435487925`, 1, 1], 
                    Hue[0.5291912090168738, 1, 1], 
                    Hue[0.5291912090168738, 1, 1], 
                    Hue[0.6229120360801792, 1, 1], 
                    Hue[0.6229120360801792, 1, 1], 
                    Hue[0.7169177088892706, 1, 1], 
                    Hue[0.7169177088892706, 1, 1], 
                    Hue[0.8112095824461873, 1, 1], 
                    Hue[0.8112095824461873, 1, 1], 
                    Hue[0.9057801263860004, 1, 1], 
                    Hue[0.9057801263860004, 1, 1], 
                    Hue[0.0006257096597658727, 1, 1], 
                    Hue[0.0006257096597658727, 1, 1], 
                    Hue[0.09574830413526374, 1, 1], 
                    Hue[0.09574830413526374, 1, 1], 
                    Hue[0.19114909128921104`, 1, 1], 
                    Hue[0.19114909128921104`, 1, 1], 
                    Hue[0.2868292849773195, 1, 1], 
                    Hue[0.2868292849773195, 1, 1], 
                    Hue[0.3827869813995296, 1, 1], 
                    Hue[0.3827869813995296, 1, 1], 
                    Hue[0.47902067137753845`, 1, 1], 
                    Hue[0.47902067137753845`, 1, 1], 
                    Hue[0.5755290682770862, 1, 1], 
                    Hue[0.5755290682770862, 1, 1], 
                    Hue[0.6723137955010745, 1, 1], 
                    Hue[0.6723137955010745, 1, 1], 
                    Hue[0.7693745554168671, 1, 1], 
                    Hue[0.7693745554168671, 1, 1], 
                    Hue[0.8667116851118895, 1, 1], 
                    Hue[0.8667116851118895, 1, 1], 
                    Hue[0.9643234476361405, 1, 1], 
                    Hue[0.9643234476361405, 1, 1], 
                    Hue[0.062208530739026324`, 1, 1], 
                    Hue[0.062208530739026324`, 1, 1], 
                    Hue[0.16036799852267106`, 1, 1], 
                    Hue[0.16036799852267106`, 1, 1], 
                    Hue[0.258801810944857, 1, 1], 
                    Hue[0.258801810944857, 1, 1], 
                    Hue[0.3575091461923081, 1, 1], 
                    Hue[0.3575091461923081, 1, 1], 
                    Hue[0.4564888992590564, 1, 1], 
                    Hue[0.4564888992590564, 1, 1], 
                    Hue[0.5557409571993345, 1, 1], 
                    Hue[0.5557409571993345, 1, 1], 
                    Hue[0.655265089711039, 1, 1], 
                    Hue[0.655265089711039, 1, 1], 
                    Hue[0.7550614056935411, 1, 1], 
                    Hue[0.7550614056935411, 1, 1], 
                    Hue[0.8551296658837929, 1, 1], 
                    Hue[0.8551296658837929, 1, 1], 
                    Hue[0.9554684222196816, 1, 1], 
                    Hue[0.9554684222196816, 1, 1], 
                    Hue[0.056077780299213835`, 1, 1], 
                    Hue[0.056077780299213835`, 1, 1], 
                    Hue[0.1569576070864384, 1, 1], 
                    Hue[0.1569576070864384, 1, 1], 
                    Hue[0.2581074846488935, 1, 1], 
                    Hue[0.2581074846488935, 1, 1], 
                    Hue[0.35952720768489815`, 1, 1], 
                    Hue[0.35952720768489815`, 1, 1], 
                    Hue[0.4612154447314414, 1, 1], 
                    Hue[0.4612154447314414, 1, 1], 
                    Hue[0.5631724067610944, 1, 1], 
                    Hue[0.5631724067610944, 1, 1], 
                    Hue[0.6653976393554915, 1, 1], 
                    Hue[0.6653976393554915, 1, 1], 
                    Hue[0.7678905877870311, 1, 1], 
                    Hue[0.7678905877870311, 1, 1], 
                    Hue[0.8706510630208145, 1, 1], 
                    Hue[0.8706510630208145, 1, 1], 
                    Hue[0.9736784294934824, 1, 1], 
                    Hue[0.9736784294934824, 1, 1], 
                    Hue[0.07697211848436797, 1, 1], 
                    Hue[0.07697211848436797, 1, 1], 
                    Hue[0.18053149069666805`, 1, 1], 
                    Hue[0.18053149069666805`, 1, 1], 
                    Hue[0.28435568477136225`, 1, 1], 
                    Hue[0.28435568477136225`, 1, 1], 
                    Hue[0.3884441911870997, 1, 1], 
                    Hue[0.3884441911870997, 1, 1], 
                    Hue[0.4927967517078755, 1, 1], 
                    Hue[0.4927967517078755, 1, 1], 
                    Hue[0.5974123653015715, 1, 1], 
                    Hue[0.5974123653015715, 1, 1], 
                    Hue[0.7022899159669522, 1, 1], 
                    Hue[0.7022899159669522, 1, 1], 
                    Hue[0.8074287288457369, 1, 1], 
                    Hue[0.8074287288457369, 1, 1], 
                    Hue[0.9128275507242074, 1, 1], 
                    Hue[0.9128275507242074, 1, 1], 
                    Hue[0.018485500919929655`, 1, 1], 
                    Hue[0.018485500919929655`, 1, 1], 
                    Hue[0.12440106059642532`, 1, 1], 
                    Hue[0.12440106059642532`, 1, 1], 
                    Hue[0.2305726939366271, 1, 1], 
                    Hue[0.2305726939366271, 1, 1], 
                    Hue[0.3369984854894055, 1, 1], 
                    Hue[0.3369984854894055, 1, 1], 
                    Hue[0.4436764093552376, 1, 1], 
                    Hue[0.4436764093552376, 1, 1], 
                    Hue[0.5506045244741555, 1, 1], 
                    Hue[0.5506045244741555, 1, 1], 
                    Hue[0.6577800886416443, 1, 1], 
                    Hue[0.6577800886416443, 1, 1], 
                    Hue[0.7652001869956759, 1, 1], 
                    Hue[0.7652001869956759, 1, 1], 
                    Hue[0.8728611074438166, 1, 1], 
                    Hue[0.8728611074438166, 1, 1], 
                    Hue[0.9807589144955988, 1, 1], 
                    Hue[0.9807589144955988, 1, 1], 
                    Hue[0.08888919496304422, 1, 1], 
                    Hue[0.08888919496304422, 1, 1], 
                    Hue[0.1972463957493058, 1, 1], 
                    Hue[0.1972463957493058, 1, 1], 
                    Hue[0.30582418937682976`, 1, 1], 
                    Hue[0.30582418937682976`, 1, 1], 
                    Hue[0.41461542530876894`, 1, 1], 
                    Hue[0.41461542530876894`, 1, 1], 
                    Hue[0.523611607046055, 1, 1], 
                    Hue[0.523611607046055, 1, 1], 
                    Hue[0.6328035280323666, 1, 1], 
                    Hue[0.6328035280323666, 1, 1], 
                    Hue[0.7421791292100071, 1, 1], 
                    Hue[0.7421791292100071, 1, 1], 
                    Hue[0.8517262210776776, 1, 1], 
                    Hue[0.8517262210776776, 1, 1], 
                    Hue[0.9614313829679946, 1, 1], 
                    Hue[0.9614313829679946, 1, 1], 
                    Hue[0.07127581809817282, 1, 1], 
                    Hue[0.07127581809817282, 1, 1], 
                    Hue[0.18123831391879858`, 1, 1], 
                    Hue[0.18123831391879858`, 1, 1], 
                    Hue[0.2912992515505204, 1, 1], 
                    Hue[0.2912992515505204, 1, 1], 
                    Hue[0.4014357821497693, 1, 1], 
                    Hue[0.4014357821497693, 1, 1], 
                    Hue[0.5116154678989273, 1, 1], 
                    Hue[0.5116154678989273, 1, 1], 
                    Hue[0.6217988884400221, 1, 1], 
                    Hue[0.6217988884400221, 1, 1], 
                    Hue[0.7319483440755872, 1, 1], 
                    Hue[0.7319483440755872, 1, 1], 
                    Hue[0.8420262140980949, 1, 1], 
                    Hue[0.8420262140980949, 1, 1], 
                    Hue[0.9519858790730588, 1, 1], 
                    Hue[0.9519858790730588, 1, 1], 
                    Hue[0.061770917376315, 1, 1], 
                    Hue[0.061770917376315, 1, 1], 
                    Hue[0.17131950476346894`, 1, 1], 
                    Hue[0.17131950476346894`, 1, 1], 
                    Hue[0.2805652433311565, 1, 1], 
                    Hue[0.2805652433311565, 1, 1], 
                    Hue[0.3894387845260012, 1, 1], 
                    Hue[0.3894387845260012, 1, 1], 
                    Hue[0.497868118280707, 1, 1], 
                    Hue[0.497868118280707, 1, 1], 
                    Hue[0.6057737050224179, 1, 1], 
                    Hue[0.6057737050224179, 1, 1], 
                    Hue[0.7130713214880129, 1, 1], 
                    Hue[0.7130713214880129, 1, 1], 
                    Hue[0.8196801378961807, 1, 1], 
                    Hue[0.8196801378961807, 1, 1], 
                    Hue[0.9255229902660722, 1, 1], 
                    Hue[0.9255229902660722, 1, 1], 
                    Hue[0.030529801027407266`, 1, 1], 
                    Hue[0.030529801027407266`, 1, 1], 
                    Hue[0.13465081125510253`, 1, 1], 
                    Hue[0.13465081125510253`, 1, 1], 
                    Hue[0.23784994238387336`, 1, 1], 
                    Hue[0.23784994238387336`, 1, 1], 
                    Hue[0.34007354031929565`, 1, 1], 
                    Hue[0.34007354031929565`, 1, 1], 
                    Hue[0.4412627880888048, 1, 1], 
                    Hue[0.4412627880888048, 1, 1], 
                    Hue[0.5414319533204015, 1, 1], 
                    Hue[0.5414319533204015, 1, 1], 
                    Hue[0.6406952567590228, 1, 1], 
                    Hue[0.6406952567590228, 1, 1], 
                    Hue[0.7391561792610386, 1, 1], 
                    Hue[0.7391561792610386, 1, 1], 
                    Hue[0.8368067224869767, 1, 1], 
                    Hue[0.8368067224869767, 1, 1], 
                    Hue[0.933611299963676, 1, 1], 
                    Hue[0.933611299963676, 1, 1], 
                    Hue[0.02966183917474022, 1, 1], 
                    Hue[0.02966183917474022, 1, 1], 
                    Hue[0.12515565615431143`, 1, 1], 
                    Hue[0.12515565615431143`, 1, 1], 
                    Hue[0.22025100378644757`, 1, 1], 
                    Hue[0.22025100378644757`, 1, 1], 
                    Hue[0.3150509908005036, 1, 1], 
                    Hue[0.3150509908005036, 1, 1], 
                    Hue[0.409700986425785, 1, 1], 
                    Hue[0.409700986425785, 1, 1], 
                    Hue[0.5043456473843696, 1, 1], 
                    Hue[0.5043456473843696, 1, 1], 
                    Hue[0.5989362670288179, 1, 1], 
                    Hue[0.5989362670288179, 1, 1], 
                    Hue[0.6932230603805059, 1, 1], 
                    Hue[0.6932230603805059, 1, 1], 
                    Hue[0.7871257717961646, 1, 1], 
                    Hue[0.7871257717961646, 1, 1], 
                    Hue[0.8811210035608937, 1, 1], 
                    Hue[0.8811210035608937, 1, 1], 
                    Hue[0.976043668502006, 1, 1], 
                    Hue[0.976043668502006, 1, 1], 
                    Hue[0.07233130360111649, 1, 1], 
                    Hue[0.07233130360111649, 1, 1], 
                    Hue[0.1694413486033889, 1, 1], 
                    Hue[0.1694413486033889, 1, 1], 
                    Hue[0.2660171617229767, 1, 1], 
                    Hue[0.2660171617229767, 1, 1], 
                    Hue[0.36108283356206833`, 1, 1], 
                    Hue[0.36108283356206833`, 1, 1], 
                    Hue[0.45592188250768106`, 1, 1], 
                    Hue[0.45592188250768106`, 1, 1], 
                    Hue[0.5541536405087248, 1, 1], 
                    Hue[0.5541536405087248, 1, 1], 
                    Hue[0.6582616045411482, 1, 1], 
                    Hue[0.6582616045411482, 1, 1], 
                    Hue[0.766613722852846, 1, 1], 
                    Hue[0.766613722852846, 1, 1], 
                    Hue[0.874029838680562, 1, 1], 
                    Hue[0.874029838680562, 1, 1], 
                    Hue[0.9752100114454223, 1, 1], 
                    Hue[0.9752100114454223, 1, 1], 
                    Hue[0.07288244035433329, 1, 1], 
                    Hue[0.07288244035433329, 1, 1], 
                    Hue[0.18183762518900837`, 1, 1], 
                    Hue[0.18183762518900837`, 1, 1], 
                    Hue[0.3128118513040628, 1, 1], 
                    Hue[0.3128118513040628, 1, 1], 
                    Hue[0.4627513612045987, 1, 1], 
                    Hue[0.4627513612045987, 1, 1], 
                    Hue[0.6188156983257607, 1, 1], 
                    Hue[0.6188156983257607, 1, 1], 
                    Hue[0.7659699254591066, 1, 1], 
                    Hue[0.7659699254591066, 1, 1], 
                    Hue[0.900255093224109, 1, 1], 
                    Hue[0.900255093224109, 1, 1], 
                    Hue[0.027221273580469205`, 1, 1], 
                    Hue[0.027221273580469205`, 1, 1], 
                    Hue[0.14925197585231578`, 1, 1], 
                    Hue[0.14925197585231578`, 1, 1], 
                    Hue[0.2642182068786057, 1, 1], 
                    Hue[0.2642182068786057, 1, 1], 
                    Hue[0.37219080794297776`, 1, 1], 
                    Hue[0.37219080794297776`, 1, 1], 
                    Hue[0.47834136069253086`, 1, 1], 
                    Hue[0.47834136069253086`, 1, 1], 
                    Hue[0.5866752936506326, 1, 1], 
                    Hue[0.5866752936506326, 1, 1], 
                    Hue[0.6967194221160611, 1, 1], 
                    Hue[0.6967194221160611, 1, 1], 
                    Hue[0.8091853189345573, 1, 1], 
                    Hue[0.8091853189345573, 1, 1], 
                    Hue[0.9280958405079539, 1, 1], 
                    Hue[0.9280958405079539, 1, 1], 
                    Hue[0.05495597451428205, 1, 1], 
                    Hue[0.05495597451428205, 1, 1], 
                    Hue[0.1858218646652519, 1, 1], 
                    Hue[0.1858218646652519, 1, 1], 
                    Hue[0.31408752473670615`, 1, 1], 
                    Hue[0.31408752473670615`, 1, 1], 
                    Hue[0.43511710210004056`, 1, 1], 
                    Hue[0.43511710210004056`, 1, 1], 
                    Hue[0.5521633524083553, 1, 1], 
                    Hue[0.5521633524083553, 1, 1], 
                    Hue[0.6750664035148826, 1, 1], 
                    Hue[0.6750664035148826, 1, 1], 
                    Hue[0.8088793717674257, 1, 1], 
                    Hue[0.8088793717674257, 1, 1], 
                    Hue[0.9514016888140933, 1, 1], 
                    Hue[0.9514016888140933, 1, 1], 
                    Hue[0.09849358523699003, 1, 1], 
                    Hue[0.09849358523699003, 1, 1], 
                    Hue[0.2469004214429708, 1, 1], 
                    Hue[0.2469004214429708, 1, 1], 
                    Hue[0.3946861926401874, 1, 1], 
                    Hue[0.3946861926401874, 1, 1], 
                    Hue[0.5410819268855549, 1, 1], 
                    Hue[0.5410819268855549, 1, 1], 
                    Hue[0.6862933635086946, 1, 1], 
                    Hue[0.6862933635086946, 1, 1]}],
                   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
                   Axes->{True, None},
                   AxesOrigin->{0, 0},
                   
                   Epilog->LineBox[{{1, 1.0060939673004288`*^-6}, {
                    2, 1.9028034995904528`*^-6}, {3, 2.214491810956645*^-6}, {
                    4, 4.087228850688351*^-6}, {5, 5.134365838566222*^-6}, {
                    6, 7.371767357820723*^-6}, {7, 0.00001165372710711609}, {
                    8, 0.000015942184345643765`}, {
                    9, 0.000022865159820867053`}, {
                    10, 0.000032216550384681804`}, {
                    11, 0.00004531982940408456}, {
                    12, 0.00006391139327604779}, {
                    13, 0.00008879778874723313}, {
                    14, 0.00012220914588380733`}, {
                    15, 0.00016837634707687606`}, {
                    16, 0.0002304191416659198}, {17, 0.000313330230771826}, {
                    18, 0.00042327333137639}, {19, 0.0005665090879436251}, {
                    20, 0.0007572810178161954}, {21, 0.001005831880287899}, {
                    22, 0.0013268803199349435`}, {
                    23, 0.0017391163480508696`}, {
                    24, 0.0022660961357999173`}, {
                    25, 0.0029364923325162057`}, {26, 0.003783476455054223}, {
                    27, 0.004846710171243579}, {28, 0.006168486543946245}, {
                    29, 0.007804057557584411}, {30, 0.009815973448589142}, {
                    31, 0.012274732172491804`}, {32, 0.015261005919086659`}, {
                    33, 0.018858882179092214`}, {34, 0.023163979488719316`}, {
                    35, 0.028283904251113486`}, {36, 0.03433193553691799}, {
                    37, 0.0414287305216337}, {38, 0.04969702664817599}, {
                    39, 0.05925947235567039}, {40, 0.07024061686016954}, {
                    41, 0.08276481303417263}, {42, 0.09694643359644554}, {
                    43, 0.11288613149152181`}, {44, 0.130666962000904}, {
                    45, 0.15035010155817907`}, {46, 0.17197400269955715`}, {
                    47, 0.1955431220108868}, {48, 0.22102763981507995`}, {
                    49, 0.24835302349513763`}, {50, 0.27740154811758816`}, {
                    51, 0.3080119086144073}, {52, 0.3399762036602181}, {
                    53, 0.3730382159591162}, {54, 0.40689263790584307`}, {
                    55, 0.4411897581894739}, {56, 0.4755466371172123}, {
                    57, 0.509548967173053}, {58, 0.542754466090125}, {
                    59, 0.5747023054196585}, {60, 0.6049310886711522}, {
                    61, 0.6329850962818948}, {62, 0.6584265248926926}, {
                    63, 0.6808433345012275}, {64, 0.6998643430946516}, {
                    65, 0.715165637206513}, {66, 0.7264858990053652}, {
                    67, 0.7336304023963114}, {68, 0.7364734571070242}, {
                    69, 0.7349657214266813}, {70, 0.729133566232684}, {
                    71, 0.7190808722418666}, {72, 0.7049882415274422}, {
                    73, 0.6870981021966865}, {74, 0.665717646082974}, {
                    75, 0.6412035112765463}, {76, 0.6139555205838362}, {
                    77, 0.5844051790844851}, {78, 0.5530048503076144}, {
                    79, 0.5202132421740492}, {80, 0.4864885030547204}, {
                    81, 0.45227595486284317`}, {82, 0.41799797463716465`}, {
                    83, 0.384048462118922}, {84, 0.350783370503897}, {
                    85, 0.3185176293754406}, {86, 0.28752103645030824`}, {
                    87, 0.2580170663937899}, {88, 0.23018141936997993`}, {
                    89, 0.20414282734699182`}, {90, 0.1799860714977934}, {
                    91, 0.157755243147453}, {92, 0.13745759396913695`}, {
                    93, 0.11906694362998292`}, {94, 0.1025294296099039}, {
                    95, 0.08776820549138109}, {96, 0.07468858883400269}, {
                    97, 0.0631822748244973}, {98, 0.053131568411669855`}, {
                    99, 0.044413889468005625`}, {
                    100, 0.036905342135417546`}, {101, 0.03048234076287701}, {
                    102, 0.025025243195574372`}, {
                    103, 0.020420649337431384`}, {104, 0.01656181108565891}, {
                    105, 0.01334932748083014}, {106, 0.010692574790119742`}, {
                    107, 0.008510368599458121}, {108, 0.00673027892552953}, {
                    109, 0.005287974898104728}, {110, 0.00412713655811019}, {
                    111, 0.003199170136874578}, {112, 0.00246250767529086}, {
                    113, 0.0018818411486922785`}, {
                    114, 0.0014274258169291948`}, {
                    115, 0.0010743963007635695`}, {
                    116, 0.0008021734836276542}, {
                    117, 0.0005938981170839203}, {
                    118, 0.0004358437750483404}, {
                    119, 0.0003169143098870495}, {
                    120, 0.00022822345065750627`}, {
                    121, 0.0001627001272008966}, {
                    122, 0.00011475425052318849`}, {
                    123, 0.0000800260541898452}, {
                    124, 0.00005516292207792344}, {
                    125, 0.00003758789282828423}, {
                    126, 0.000025314466690062543`}, {
                    127, 0.000016838027811155756`}, {
                    128, 0.000011054501119483348`}, {
                    129, 7.1655136221218016`*^-6}, {
                    130, 4.589964828963215*^-6}, {
                    131, 2.906933090609105*^-6}, {
                    132, 1.8201919219658998`*^-6}, {
                    133, 1.1270270792791408`*^-6}, {
                    134, 6.902640131868779*^-7}, {
                    135, 4.180183801076222*^-7}, {
                    136, 2.501184398881478*^-7}, {
                    137, 1.479636453594186*^-7}, {
                    138, 8.680633038614577*^-8}, {
                    139, 5.06653280168099*^-8}, {
                    140, 2.9399614941896473`*^-8}, {
                    141, 1.6869931056056762`*^-8}, {
                    142, 9.514779299252841*^-9}, {
                    143, 5.272670355686785*^-9}, {
                    144, 2.898929397941354*^-9}, {
                    145, 1.6041878654532542`*^-9}, {
                    146, 8.936820823976941*^-10}, {
                    147, 4.896731263798972*^-10}, {
                    148, 2.562848336534942*^-10}, {
                    149, 1.2681716371188922`*^-10}, {
                    150, 6.111460626146627*^-11}, {
                    151, 3.070893633672525*^-11}, {
                    152, 1.6575077692769548`*^-11}, {
                    153, 9.195631750182398*^-12}, {
                    154, 5.153797796699761*^-12}, {
                    155, 3.0522209672263964`*^-12}, {
                    156, 1.993857641881664*^-12}, {
                    157, 1.4221696991339278`*^-12}, {
                    158, 1.0490133273302367`*^-12}, {
                    159, 7.588589589693029*^-13}, {
                    160, 5.331956471141299*^-13}, {
                    161, 3.7457829774578716`*^-13}, {
                    162, 2.685778504148808*^-13}, {
                    163, 1.940500524749891*^-13}, {
                    164, 1.4001556424587258`*^-13}, {
                    165, 1.0204355672203919`*^-13}, {
                    166, 7.483665694650635*^-14}, {
                    167, 5.347910219035631*^-14}, {
                    168, 3.620484922047855*^-14}, {
                    169, 2.326471685926075*^-14}, {
                    170, 1.47617064363966*^-14}, {
                    171, 9.769967251951077*^-15}, {
                    172, 6.796912093454265*^-15}, {
                    173, 4.7512022351669356`*^-15}, {
                    174, 3.2089960637776472`*^-15}, {
                    175, 2.0671743953597076`*^-15}, {
                    176, 1.2740320892155748`*^-15}, {
                    177, 7.586352984322208*^-16}, {
                    178, 4.415716037259406*^-16}, {
                    179, 2.5399341718929953`*^-16}, {
                    180, 1.4557341737399026`*^-16}, {
                    181, 8.343536015433051*^-17}, {
                    182, 4.768197795701147*^-17}, {
                    183, 2.6907693223305463`*^-17}, {
                    184, 1.4782036064853723`*^-17}, {
                    185, 7.780973176767766*^-18}, {
                    186, 3.860579985642399*^-18}, {
                    187, 1.7726870071704155`*^-18}, {
                    188, 7.357054351639424*^-19}, {
                    189, 2.677124945936663*^-19}, {
                    190, 8.646722352541604*^-20}, {
                    191, 3.5795685934324046`*^-20}, {
                    192, 2.2260313008291673`*^-20}, {
                    193, 1.2353234324457536`*^-20}, {
                    194, 5.171882233315641*^-21}, {
                    195, 1.2329921260399653`*^-21}, {
                    196, 3.5697379682382633`*^-22}, {
                    197, 6.984088193968665*^-22}, {
                    198, 5.614517877255946*^-22}, {
                    199, 3.3495412370170086`*^-22}, {
                    200, 1.606895604155378*^-22}, {
                    201, 6.012656079710269*^-23}, {
                    202, 1.3702289045464478`*^-23}, {
                    203, 3.176748949691128*^-24}, {
                    204, 5.9906052281992155`*^-24}, {
                    205, 4.634817176155701*^-24}, {
                    206, 2.7094443837280076`*^-24}, {
                    207, 1.3425486156671524`*^-24}, {
                    208, 6.075732300979954*^-25}, {
                    209, 2.8664838988972673`*^-25}, {
                    210, 1.5814663506259344`*^-25}, {
                    211, 9.342359763911288*^-26}, {
                    212, 5.272603860863187*^-26}, {
                    213, 2.751337247881723*^-26}, {
                    214, 1.3341384879998547`*^-26}, {
                    215, 6.139988063566283*^-27}, {
                    216, 2.7772905130818968`*^-27}, {
                    217, 1.2833952994966071`*^-27}, {
                    218, 6.140892038997121*^-28}, {
                    219, 2.9761548591698386`*^-28}, {
                    220, 1.418088674611575*^-28}, {
                    221, 6.5433295675369*^-29}, {
                    222, 2.929250527447788*^-29}, {
                    223, 1.2926786050298013`*^-29}, {
                    224, 5.773637459621317*^-30}, {
                    225, 2.683219739682183*^-30}, {
                    226, 1.3154971342703058`*^-30}, {
                    227, 6.749788967552416*^-31}, {
                    228, 3.548869862050382*^-31}, {
                    229, 1.8721015041609338`*^-31}, {
                    230, 9.764670895508588*^-32}, {
                    231, 4.997300003623216*^-32}, {
                    232, 2.5031563237895134`*^-32}, {
                    233, 1.2282006772383095`*^-32}, {
                    234, 5.917888472830059*^-33}, {
                    235, 2.8084316007143253`*^-33}, {
                    236, 1.3160390749146207`*^-33}, {
                    237, 6.100008144197109*^-34}, {
                    238, 2.799268290862233*^-34}, {
                    239, 1.2722999305358613`*^-34}, {
                    240, 5.729239788761659*^-35}, {
                    241, 2.557255695078455*^-35}, {
                    242, 1.1322487175699076`*^-35}, {
                    243, 4.9773435952795905`*^-36}, {
                    244, 2.1743796476380444`*^-36}, {
                    245, 9.44627126821288*^-37}, {
                    246, 4.082537411854193*^-37}, {
                    247, 1.7552364589517332`*^-37}, {
                    248, 7.504517103744936*^-38}, {
                    249, 3.1888794961033176`*^-38}, {
                    250, 1.345796770018393*^-38}, {
                    251, 5.6368908933762926`*^-39}}],
                   Frame->True,
                   FrameTicks->{{{1., 
                    FormBox[
                    RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
                    FormBox[
                    RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
                    FormBox[
                    RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
                    FormBox[
                    RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
                    FormBox["0", TraditionalForm]}, {157.25, 
                    FormBox["2.5`", TraditionalForm]}, {188.5, 
                    FormBox["5", TraditionalForm]}, {219.75, 
                    FormBox["7.5`", TraditionalForm]}, {251., 
                    FormBox["10", TraditionalForm]}}, Automatic, None, None},
                   PlotRange->{-0.1, 1.1},
                   PlotRangeClipping->True,
                   PlotRangePadding->{
                    Scaled[0.02], Automatic}], 4->
                  GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF2XlYVPUeBvBDZu4pGqClNgipuWW4C8iLgAIi+76eAWYGvBJ6zSWl6480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                    "], {
                    {GrayLevel[0.5], EdgeForm[None], 
                    GraphicsGroupBox[{PolygonBox[{{194, 638, 195}}], 
                    PolygonBox[CompressedData["
1:eJwtlmV0VUcURs/c4E5wJxDc3d2CQ3CH4BrcaSnuFHd3h+IudXd3d/cW6T79
+LFX9po5ct/cM3kvLik5cUxkZjchBaT0v4G/kAr3vbtwD2LuY+yFoPj0kAXP
Chnu14jwmKBa6SATnjko1mumwlM7eBqP8T1IayIDnjEo9/8Y3wuK9R6xeDaP
wbNDEbwo5PBekBPP5T1N5MbzQBYTefF8/sx2v4bnBuX6Wn68AMTi2aAwHhfU
y9cK4oWC9rxnPF4McpoojpfwZ8ArQHe8B1Q0rZXES0FuPA+UxctBXq8N1fEa
3gPPBxXxSpDfRGW8ij+jKac8XiEo1teq4tWCahWFhngjiDfVrInXgsJ4HNTF
60ERE/XxBkG5/oyl8TJBz+o1GuNN/DP7s0Mi3sk/g8dCO7y95+AlIAFvCSVN
tMJb+xn4WUFzvEVQrK+1wdsG1fIeTfFmQbFeswPeMai39+yMd4HyJrri3YLO
3ud6JkM8C26Z3kFP9nr5mfpZQl+8n5+pnz0MwYf6OzCt9ccHQFUTSfhAP2M/
OxiPT/AzNK0NwgcH1aoDY/FxQbFecxg+HGritWAUPhpqmxiDJwfleswIfGRQ
rPeYiE+C+ngzmIfPh+Z4A5iKT4OGJqbjM3wGTMzEZ0FjU43J+JSgXF97AH/Q
37GfPczB5wb18rXZ+ENBe95zAb7Q36GJRfhiSDCxBF/qM2BiGb4cWplYga/0
mfDZgfX4Bp8hExvxTdDBZwNW42t8RkysxdcF5XrMZnyLzwjeCXbiu6Czid34
Hp8ZU8xWfBskmtiO7wjK9Wd6GF8V1Ntz9uL7oCveDQ7ih6A73gtO4qegt2nt
MH4Eepg4ih+DniaO4yeCcj3nEfw09DH12I8fCOrla2fws9DXxDn8vM+wiQv4
RehvuuO1fZaC7rqvXcIv+0ybuIJfhSQT1/DrPuP4UHgKfxqG+WzDLfxRn3ET
j+GPwxB8GnyHfw/TTWtP4E8G1RoFr+Cv+sybaj6DP+t3wmcdXsBf9Jk38RL+
clCuxzyHPx8U6zVew1+HMfgk+Az/HCbjE+Aj/GOY6HcR3sHf9Ttp4j38fRiP
J8Nb+NtBsb72Af5hUC3v8Qb+ZlCs1/wE/zSot/f8Av/S75SJr/CvYaqJb/Bv
g87Kz+gH/Ee/oz5rkJf/Vflgld9F+IO9P/1OmvgL/xtm+12GX/Bf/U6b+A3/
PSjXY/7B//U7i88F/1IPMM9EhMfAfFPMbWLvwBwTd/F7Qbn+jD/hPwf19pwU
5KaEBfhCSIOnhUX4UsiCZ4VlprV0eHpYbCIDnhGWmMiEZ46U6zmxeDZYbuqR
Ck8dqZevZcdzwAoTOfFcsNJEbjxPpLP1M82PF4DVpu+A3n7Xgr4LfK0ge4Vg
jd9Fv2N4PziNr4UieFFYZyIeLwbr8W1QFa8G201Ux2vADnwDlMRLwUYTpfEy
sMlUozheIlLsFqiIV4KtJirjVSL12gzl8QqRYr1HTbwW7MQPQALeEg7iu6Au
Xg92m6iPN4A9pjt/g7O4GXTX90Nz9lpEquU1a+N1ItXaC43xJrDPVKMh3ijS
nvdshbeGQ6aYpnizSLX9M5fFy0X6LB7TBm8Lh020w9vDEfwk9MR7wSn8GHTG
u8BxE13xbnACPwqJeKdIsb7WHe8RqdZZGIQPhnOmmr3xPpHevb/z/vgAOGMi
CR8YKddzhuBD4Tx+AUbgI+Eifg0m41PgumltFD4aLpnwH+DJcNn0GTvgHSM9
+xUYj0+Aq6aYsfi4SHu+NhGfFKmX95iKT4MbJqbjMyL9FvJnHIYPj/SsPuOF
8bhIs81rt9twx/Tb/z9UaTnB
                    "]]}]}, {{}, {}, {
                    {Hue[0.9640532089338167, 1, 1], LineBox[{1, 2}]}, 
                    {Hue[0.7638938367871528, 1, 1], LineBox[{2, 3}]}, 
                    {Hue[0.5456953793797118, 1, 1], LineBox[{3, 4}]}, 
                    {Hue[0.4880359025202404, 1, 1], LineBox[{4, 5}]}, 
                    {Hue[0.418755936350388, 1, 1], LineBox[{5, 6}]}, 
                    {Hue[0.22471186555443298`, 1, 1], LineBox[{6, 7}]}, 
                    {Hue[0.7664365867469122, 1, 1], LineBox[{7, 8}]}, 
                    {Hue[0.7569067121630682, 1, 1], LineBox[{8, 9}]}, 
                    {Hue[0.8201765986296474, 1, 1], LineBox[{9, 10}]}, 
                    {Hue[0.8383737951258301, 1, 1], LineBox[{10, 11}]}, 
                    {Hue[0.8528114312520936, 1, 1], LineBox[{11, 12}]}, 
                    {Hue[0.0005814275632171318, 1, 1], LineBox[{12, 13}]}, 
                    {Hue[0.09900120701277339, 1, 1], LineBox[{13, 14}]}, 
                    {Hue[0.1775661426240637, 1, 1], LineBox[{14, 15}]}, 
                    {Hue[0.24423919184988493`, 1, 1], LineBox[{15, 16}]}, 
                    {Hue[0.3156688798563375, 1, 1], LineBox[{16, 17}]}, 
                    {Hue[0.38950220432683996`, 1, 1], LineBox[{17, 18}]}, 
                    {Hue[0.4814958525831977, 1, 1], LineBox[{18, 19}]}, 
                    {Hue[0.5657261147473843, 1, 1], LineBox[{19, 20}]}, 
                    {Hue[0.6467337892415539, 1, 1], LineBox[{20, 21}]}, 
                    {Hue[0.7248416611021938, 1, 1], LineBox[{21, 22}]}, 
                    {Hue[0.8055142767348117, 1, 1], LineBox[{22, 23}]}, 
                    {Hue[0.8872992703340177, 1, 1], LineBox[{23, 24}]}, 
                    {Hue[0.9720085880616776, 1, 1], LineBox[{24, 25}]}, 
                    {Hue[0.05610154862290952, 1, 1], LineBox[{25, 26}]}, 
                    {Hue[0.13926945181155012`, 1, 1], LineBox[{26, 27}]}, 
                    {Hue[0.22225896671723153`, 1, 1], LineBox[{27, 28}]}, 
                    {Hue[0.30594469461240686`, 1, 1], LineBox[{28, 29}]}, 
                    {Hue[0.3907297817738525, 1, 1], LineBox[{29, 30}]}, 
                    {Hue[0.47584462841575326`, 1, 1], LineBox[{30, 31}]}, 
                    {Hue[0.5613011947128719, 1, 1], LineBox[{31, 32}]}, 
                    {Hue[0.6467734454202569, 1, 1], LineBox[{32, 33}]}, 
                    {Hue[0.732620054471488, 1, 1], LineBox[{33, 34}]}, 
                    {Hue[0.8188508419403508, 1, 1], LineBox[{34, 35}]}, 
                    {Hue[0.9055713829157245, 1, 1], LineBox[{35, 36}]}, 
                    {Hue[0.9926765120282037, 1, 1], LineBox[{36, 37}]}, 
                    {Hue[0.08008348174035529, 1, 1], LineBox[{37, 38}]}, 
                    {Hue[0.1678032957843583, 1, 1], LineBox[{38, 39}]}, 
                    {Hue[0.2558772079638403, 1, 1], LineBox[{39, 40}]}, 
                    {Hue[0.34433867572467236`, 1, 1], LineBox[{40, 41}]}, 
                    {Hue[0.43317536470416307`, 1, 1], LineBox[{41, 42}]}, 
                    {Hue[0.5223512697273097, 1, 1], LineBox[{42, 43}]}, 
                    {Hue[0.611865760320544, 1, 1], LineBox[{43, 44}]}, 
                    {Hue[0.7017376629319375, 1, 1], LineBox[{44, 45}]}, 
                    {Hue[0.791973646004684, 1, 1], LineBox[{45, 46}]}, 
                    {Hue[0.8825715119856232, 1, 1], LineBox[{46, 47}]}, 
                    {Hue[0.9735201231658492, 1, 1], LineBox[{47, 48}]}, 
                    {Hue[0.06481664673905892, 1, 1], LineBox[{48, 49}]}, 
                    {Hue[0.1564624526544423, 1, 1], LineBox[{49, 50}]}, 
                    {Hue[0.24846339708111093`, 1, 1], LineBox[{50, 51}]}, 
                    {Hue[0.34082044254039434`, 1, 1], LineBox[{51, 52}]}, 
                    {Hue[0.4335285188995974, 1, 1], LineBox[{52, 53}]}, 
                    {Hue[0.5265858504718376, 1, 1], LineBox[{53, 54}]}, 
                    {Hue[0.619991238019795, 1, 1], LineBox[{54, 55}]}, 
                    {Hue[0.7137489787936226, 1, 1], LineBox[{55, 56}]}, 
                    {Hue[0.8078581491198726, 1, 1], LineBox[{56, 57}]}, 
                    {Hue[0.9023167908162214, 1, 1], LineBox[{57, 58}]}, 
                    {Hue[0.997123518820222, 1, 1], LineBox[{58, 59}]}, 
                    {Hue[0.09227809771084206, 1, 1], LineBox[{59, 60}]}, 
                    {Hue[0.18778045984197525`, 1, 1], LineBox[{60, 61}]}, 
                    {Hue[0.28363150538320664`, 1, 1], LineBox[{61, 62}]}, 
                    {Hue[0.37982913467653123`, 1, 1], LineBox[{62, 63}]}, 
                    {Hue[0.4763730534549754, 1, 1], LineBox[{63, 64}]}, 
                    {Hue[0.5732620582331205, 1, 1], LineBox[{64, 65}]}, 
                    {Hue[0.6704969545124291, 1, 1], LineBox[{65, 66}]}, 
                    {Hue[0.7680769044926821, 1, 1], LineBox[{66, 67}]}, 
                    {Hue[0.866001209191981, 1, 1], LineBox[{67, 68}]}, 
                    {Hue[0.9642694798933042, 1, 1], LineBox[{68, 69}]}, 
                    {Hue[0.06288033404720918, 1, 1], LineBox[{69, 70}]}, 
                    {Hue[0.1618340190608663, 1, 1], LineBox[{70, 71}]}, 
                    {Hue[0.26112965826933277`, 1, 1], LineBox[{71, 72}]}, 
                    {Hue[0.36076682669457827`, 1, 1], LineBox[{72, 73}]}, 
                    {Hue[0.4607446702920191, 1, 1], LineBox[{73, 74}]}, 
                    {Hue[0.5610625757967369, 1, 1], LineBox[{74, 75}]}, 
                    {Hue[0.6617196759615321, 1, 1], LineBox[{75, 76}]}, 
                    {Hue[0.7627147629687548, 1, 1], LineBox[{76, 77}]}, 
                    {Hue[0.8640473849020327, 1, 1], LineBox[{77, 78}]}, 
                    {Hue[0.9657168711695278, 1, 1], LineBox[{78, 79}]}, 
                    {Hue[0.06772203209084474, 1, 1], LineBox[{79, 80}]}, 
                    {Hue[0.17006179329564264`, 1, 1], LineBox[{80, 81}]}, 
                    {Hue[0.2727348799293318, 1, 1], LineBox[{81, 82}]}, 
                    {Hue[0.3757400482244432, 1, 1], LineBox[{82, 83}]}, 
                    {Hue[0.47907615599813785`, 1, 1], LineBox[{83, 84}]}, 
                    {Hue[0.5827411225608394, 1, 1], LineBox[{84, 85}]}, 
                    {Hue[0.686732977202875, 1, 1], LineBox[{85, 86}]}, 
                    {Hue[0.7910505657857274, 1, 1], LineBox[{86, 87}]}, 
                    {Hue[0.8956910816409434, 1, 1], LineBox[{87, 88}]}, 
                    {Hue[0.0006518319749389761, 1, 1], LineBox[{88, 89}]}, 
                    {Hue[0.10592980643142957`, 1, 1], LineBox[{89, 90}]}, 
                    {Hue[0.21152177735264863`, 1, 1], LineBox[{90, 91}]}, 
                    {Hue[0.3174239892158998, 1, 1], LineBox[{91, 92}]}, 
                    {Hue[0.4236321033035602, 1, 1], LineBox[{92, 93}]}, 
                    {Hue[0.5301403332116573, 1, 1], LineBox[{93, 94}]}, 
                    {Hue[0.6369434399221541, 1, 1], LineBox[{94, 95}]}, 
                    {Hue[0.7440350187497498, 1, 1], LineBox[{95, 96}]}, 
                    {Hue[0.8514071856099105, 1, 1], LineBox[{96, 97}]}, 
                    {Hue[0.9590505875381703, 1, 1], LineBox[{97, 98}]}, 
                    {Hue[0.06695495506670603, 1, 1], LineBox[{98, 99}]}, 
                    {Hue[0.17510840763985963`, 1, 1], LineBox[{99, 100}]}, 
                    {Hue[0.2834976571366844, 1, 1], LineBox[{100, 101}]}, 
                    {Hue[0.3921067036965846, 1, 1], LineBox[{101, 102}]}, 
                    {Hue[0.5009171822537385, 1, 1], LineBox[{102, 103}]}, 
                    {Hue[0.609907960840949, 1, 1], LineBox[{103, 104}]}, 
                    {Hue[0.7190549385107554, 1, 1], LineBox[{104, 105}]}, 
                    {Hue[0.8283307021079086, 1, 1], LineBox[{105, 106}]}, 
                    {Hue[0.9377040687499487, 1, 1], LineBox[{106, 107}]}, 
                    {Hue[0.04713886558145295, 1, 1], LineBox[{107, 108}]}, 
                    {Hue[0.15659333346659132`, 1, 1], LineBox[{108, 109}]}, 
                    {Hue[0.26602139648303896`, 1, 1], LineBox[{109, 110}]}, 
                    {Hue[0.37537095342567955`, 1, 1], LineBox[{110, 111}]}, 
                    {Hue[0.48458443441848675`, 1, 1], LineBox[{111, 112}]}, 
                    {Hue[0.5935967404345542, 1, 1], LineBox[{112, 113}]}, 
                    {Hue[0.7023349586497987, 1, 1], LineBox[{113, 114}]}, 
                    {Hue[0.8107237607449748, 1, 1], LineBox[{114, 115}]}, 
                    {Hue[0.9186813036861303, 1, 1], LineBox[{115, 116}]}, 
                    {Hue[0.02611720447668775, 1, 1], LineBox[{116, 117}]}, 
                    {Hue[0.1329418218181866, 1, 1], LineBox[{117, 118}]}, 
                    {Hue[0.23906924624217446`, 1, 1], LineBox[{118, 119}]}, 
                    {Hue[0.34440835836416867`, 1, 1], LineBox[{119, 120}]}, 
                    {Hue[0.4488665860210912, 1, 1], LineBox[{120, 121}]}, 
                    {Hue[0.5523753592326575, 1, 1], LineBox[{121, 122}]}, 
                    {Hue[0.6548918734757004, 1, 1], LineBox[{122, 123}]}, 
                    {Hue[0.7563690714498079, 1, 1], LineBox[{123, 124}]}, 
                    {Hue[0.856754833013501, 1, 1], LineBox[{124, 125}]}, 
                    {Hue[0.9560476299526799, 1, 1], LineBox[{125, 126}]}, 
                    {Hue[0.05431861348849255, 1, 1], LineBox[{126, 127}]}, 
                    {Hue[0.15165859477128055`, 1, 1], LineBox[{127, 128}]}, 
                    {Hue[0.2481278362722127, 1, 1], LineBox[{128, 129}]}, 
                    {Hue[0.34377181562590414`, 1, 1], LineBox[{129, 130}]}, 
                    {Hue[0.43867250905938415`, 1, 1], LineBox[{130, 131}]}, 
                    {Hue[0.5329818091731069, 1, 1], LineBox[{131, 132}]}, 
                    {Hue[0.6268784974511232, 1, 1], LineBox[{132, 133}]}, 
                    {Hue[0.7204800452917036, 1, 1], LineBox[{133, 134}]}, 
                    {Hue[0.8138447115607892, 1, 1], LineBox[{134, 135}]}, 
                    {Hue[0.9070647396475882, 1, 1], LineBox[{135, 136}]}, 
                    {Hue[0.00027500323336431757`, 1, 1], 
                    LineBox[{136, 137}]}, 
                    {Hue[0.0936287218150815, 1, 1], LineBox[{137, 138}]}, 
                    {Hue[0.18738898166912799`, 1, 1], LineBox[{138, 139}]}, 
                    {Hue[0.2817608025135326, 1, 1], LineBox[{139, 140}]}, 
                    {Hue[0.3762560119614834, 1, 1], LineBox[{140, 141}]}, 
                    {Hue[0.46983126202546877`, 1, 1], LineBox[{141, 142}]}, 
                    {Hue[0.5628818631021897, 1, 1], LineBox[{142, 143}]}, 
                    {Hue[0.6583764075712767, 1, 1], LineBox[{143, 144}]}, 
                    {Hue[0.7583085727453893, 1, 1], LineBox[{144, 145}]}, 
                    {Hue[0.8597558378859961, 1, 1], LineBox[{145, 146}]}, 
                    {Hue[0.9565076209795028, 1, 1], LineBox[{146, 147}]}, 
                    {Hue[0.04627851579084141, 1, 1], LineBox[{147, 148}]}, 
                    {Hue[0.13696150887572137`, 1, 1], LineBox[{148, 149}]}, 
                    {Hue[0.236959712104721, 1, 1], LineBox[{149, 150}]}, 
                    {Hue[0.34512012689175703`, 1, 1], LineBox[{150, 151}]}, 
                    {Hue[0.45688774460841647`, 1, 1], LineBox[{151, 152}]}, 
                    {Hue[0.5703780558213525, 1, 1], LineBox[{152, 153}]}, 
                    {Hue[0.6836540521049628, 1, 1], LineBox[{153, 154}]}, 
                    {Hue[0.7838772463981823, 1, 1], LineBox[{154, 155}]}, 
                    {Hue[0.8381977472273094, 1, 1], LineBox[{155, 156}]}, 
                    {Hue[0.8870544606419305, 1, 1], LineBox[{156, 157}]}, 
                    {Hue[0.003981834612291627, 1, 1], LineBox[{157, 158}]}, 
                    {Hue[0.16018129844586784`, 1, 1], LineBox[{158, 159}]}, 
                    {Hue[0.33780729398489723`, 1, 1], LineBox[{159, 160}]}, 
                    {Hue[0.5369775037050852, 1, 1], LineBox[{160, 161}]}, 
                    {Hue[0.7905257246587302, 1, 1], LineBox[{161, 162}]}, 
                    {Hue[0.1945036657769734, 1, 1], LineBox[{162, 163}]}, 
                    {Hue[0.48496065810113864`, 1, 1], LineBox[{163, 164}]}, 
                    {Hue[0.7153646411488792, 1, 1], LineBox[{164, 165}]}, 
                    {Hue[0.9394067463789025, 1, 1], LineBox[{165, 166}]}, 
                    {Hue[0.17080211619971455`, 1, 1], LineBox[{166, 167}]}, 
                    {Hue[0.4207562168980021, 1, 1], LineBox[{167, 168}]}, 
                    {Hue[0.6977457893116823, 1, 1], LineBox[{168, 169}]}, 
                    {Hue[0.9810396549863314, 1, 1], LineBox[{169, 170}]}, 
                    {Hue[0.23870865443606856`, 1, 1], LineBox[{170, 171}]}, 
                    {Hue[0.4717879390337842, 1, 1], LineBox[{171, 172}]}, 
                    {Hue[0.6898493957007891, 1, 1], LineBox[{172, 173}]}, 
                    {Hue[0.8977125550908651, 1, 1], LineBox[{173, 174}]}, 
                    {Hue[0.09510862803082076, 1, 1], LineBox[{174, 175}]}, 
                    {Hue[0.22425606418634192`, 1, 1], LineBox[{175, 176}]}, 
                    {Hue[0.9819966346546343, 1, 1], LineBox[{176, 177}]}, 
                    {Hue[0.12933728393861235`, 1, 1], LineBox[{177, 178}]}, 
                    {Hue[0.24718059981823032`, 1, 1], LineBox[{178, 179}]}, 
                    {Hue[0.3560042502224064, 1, 1], LineBox[{179, 180}]}, 
                    {Hue[0.4832174243664257, 1, 1], LineBox[{180, 181}]}, 
                    {Hue[0.6225466914902852, 1, 1], LineBox[{181, 182}]}, 
                    {Hue[0.7669134155846555, 1, 1], LineBox[{182, 183}]}, 
                    {Hue[0.919797571218377, 1, 1], LineBox[{183, 184}]}, 
                    {Hue[0.08538170051501558, 1, 1], LineBox[{184, 185}]}, 
                    {Hue[0.2620427531752457, 1, 1], LineBox[{185, 186}]}, 
                    {Hue[0.44406556173858014`, 1, 1], LineBox[{186, 187}]}, 
                    {Hue[0.6185988292105646, 1, 1], LineBox[{187, 188}]}, 
                    {Hue[0.6838184160682961, 1, 1], LineBox[{188, 189}]}, 
                    {Hue[0.6378535206406792, 1, 1], LineBox[{189, 190}]}, 
                    {Hue[0.8073537712650531, 1, 1], LineBox[{190, 191}]}, 
                    {Hue[0.0008353865620092372, 1, 1], LineBox[{191, 192}]}, 
                    {Hue[0.1996041412947276, 1, 1], LineBox[{192, 193}]}, 
                    {Hue[0.3994928084092794, 1, 1], LineBox[{193, 194}]}, 
                    {Hue[0.598581266602507, 1, 1], LineBox[{194, 195}]}, 
                    {Hue[0.795286513537276, 1, 1], LineBox[{195, 196}]}, 
                    {Hue[0.9874661970418543, 1, 1], LineBox[{196, 197}]}, 
                    {Hue[0.17098866130101048`, 1, 1], LineBox[{197, 198}]}, 
                    {Hue[0.33608016823445647`, 1, 1], LineBox[{198, 199}]}, 
                    {Hue[0.4626610597119734, 1, 1], LineBox[{199, 200}]}, 
                    {Hue[0.5600193469710358, 1, 1], LineBox[{200, 201}]}, 
                    {Hue[0.6951140435126271, 1, 1], LineBox[{201, 202}]}, 
                    {Hue[0.8688175337237586, 1, 1], LineBox[{202, 203}]}, 
                    {Hue[0.06217243610467935, 1, 1], LineBox[{203, 204}]}, 
                    {Hue[0.26673052970331346`, 1, 1], LineBox[{204, 205}]}, 
                    {Hue[0.47918613708383234`, 1, 1], LineBox[{205, 206}]}, 
                    {Hue[0.6981752604591175, 1, 1], LineBox[{206, 207}]}, 
                    {Hue[0.923139519550703, 1, 1], LineBox[{207, 208}]}, 
                    {Hue[0.15391413553106462`, 1, 1], LineBox[{208, 209}]}, 
                    {Hue[0.39058746840666203`, 1, 1], LineBox[{209, 210}]}, 
                    {Hue[0.6335130531533011, 1, 1], LineBox[{210, 211}]}, 
                    {Hue[0.8834878616778492, 1, 1], LineBox[{211, 212}]}, 
                    {Hue[0.14218772425266332`, 1, 1], LineBox[{212, 213}]}, 
                    {Hue[0.4129145318615668, 1, 1], LineBox[{213, 214}]}, 
                    {Hue[0.7008647172568718, 1, 1], LineBox[{214, 215}]}, 
                    {Hue[0.008276153030424088, 1, 1], LineBox[{215, 216}]}, 
                    {Hue[0.32203152553002135`, 1, 1], LineBox[{216, 217}]}, 
                    {Hue[0.6228272969220539, 1, 1], LineBox[{217, 218}]}, 
                    {Hue[0.9077377802688978, 1, 1], LineBox[{218, 219}]}, 
                    {Hue[0.18295813338288494`, 1, 1], LineBox[{219, 220}]}, 
                    {Hue[0.45436977284719, 1, 1], LineBox[{220, 221}]}, 
                    {Hue[0.7264230564635294, 1, 1], LineBox[{221, 222}]}, 
                    {Hue[0.0029732826562475275`, 1, 1], 
                    LineBox[{222, 223}]}, 
                    {Hue[0.28772899918756284`, 1, 1], LineBox[{223, 224}]}, 
                    {Hue[0.5832291550635875, 1, 1], LineBox[{224, 225}]}, 
                    {Hue[0.8874633543999638, 1, 1], LineBox[{225, 226}]}, 
                    {Hue[0.1922430080867037, 1, 1], LineBox[{226, 227}]}, 
                    {Hue[0.489719235032304, 1, 1], LineBox[{227, 228}]}, 
                    {Hue[0.7780535457697576, 1, 1], LineBox[{228, 229}]}, 
                    {Hue[0.0590819613523457, 1, 1], LineBox[{229, 230}]}, 
                    {Hue[0.3350544804696013, 1, 1], LineBox[{230, 231}]}, 
                    {Hue[0.6076027369926792, 1, 1], LineBox[{231, 232}]}, 
                    {Hue[0.8777648644902436, 1, 1], LineBox[{232, 233}]}, 
                    {Hue[0.1461932685057524, 1, 1], LineBox[{233, 234}]}, 
                    {Hue[0.4133070810184004, 1, 1], LineBox[{234, 235}]}, 
                    {Hue[0.6793688455124909, 1, 1], LineBox[{235, 236}]}, 
                    {Hue[0.944513321626013, 1, 1], LineBox[{236, 237}]}, 
                    {Hue[0.20875530569204304`, 1, 1], LineBox[{237, 238}]}, 
                    {Hue[0.4719956959154934, 1, 1], LineBox[{238, 239}]}, 
                    {Hue[0.7340353362877327, 1, 1], LineBox[{239, 240}]}, 
                    {Hue[0.9945952634521681, 1, 1], LineBox[{240, 241}]}, 
                    {Hue[0.2533389529365845, 1, 1], LineBox[{241, 242}]}, 
                    {Hue[0.5098912530483598, 1, 1], LineBox[{242, 243}]}, 
                    {Hue[0.7638545596346901, 1, 1], LineBox[{243, 244}]}, 
                    {Hue[0.01483438802715526, 1, 1], LineBox[{244, 245}]}, 
                    {Hue[0.2624959819063229, 1, 1], LineBox[{245, 246}]}, 
                    {Hue[0.5066777947454011, 1, 1], LineBox[{246, 247}]}, 
                    {Hue[0.7475644420744317, 1, 1], LineBox[{247, 248}]}, 
                    {Hue[0.9858518958082757, 1, 1], LineBox[{248, 249}]}, 
                    {Hue[0.2227535812882585, 1, 1], LineBox[{249, 250}]}, 
                    {Hue[0.45973174016904605`, 1, 1], LineBox[{250, 251}]}}}},
                    
                    VertexColors->{
                    Hue[0.9640532089338167, 1, 1], 
                    Hue[0.7638938367871528, 1, 1], 
                    Hue[0.5456953793797118, 1, 1], 
                    Hue[0.4880359025202404, 1, 1], 
                    Hue[0.418755936350388, 1, 1], 
                    Hue[0.22471186555443298`, 1, 1], 
                    Hue[0.7664365867469122, 1, 1], 
                    Hue[0.7569067121630682, 1, 1], 
                    Hue[0.8201765986296474, 1, 1], 
                    Hue[0.8383737951258301, 1, 1], 
                    Hue[0.8528114312520936, 1, 1], 
                    Hue[0.0005814275632171318, 1, 1], 
                    Hue[0.09900120701277339, 1, 1], 
                    Hue[0.1775661426240637, 1, 1], 
                    Hue[0.24423919184988493`, 1, 1], 
                    Hue[0.3156688798563375, 1, 1], 
                    Hue[0.38950220432683996`, 1, 1], 
                    Hue[0.4814958525831977, 1, 1], 
                    Hue[0.5657261147473843, 1, 1], 
                    Hue[0.6467337892415539, 1, 1], 
                    Hue[0.7248416611021938, 1, 1], 
                    Hue[0.8055142767348117, 1, 1], 
                    Hue[0.8872992703340177, 1, 1], 
                    Hue[0.9720085880616776, 1, 1], 
                    Hue[0.05610154862290952, 1, 1], 
                    Hue[0.13926945181155012`, 1, 1], 
                    Hue[0.22225896671723153`, 1, 1], 
                    Hue[0.30594469461240686`, 1, 1], 
                    Hue[0.3907297817738525, 1, 1], 
                    Hue[0.47584462841575326`, 1, 1], 
                    Hue[0.5613011947128719, 1, 1], 
                    Hue[0.6467734454202569, 1, 1], 
                    Hue[0.732620054471488, 1, 1], 
                    Hue[0.8188508419403508, 1, 1], 
                    Hue[0.9055713829157245, 1, 1], 
                    Hue[0.9926765120282037, 1, 1], 
                    Hue[0.08008348174035529, 1, 1], 
                    Hue[0.1678032957843583, 1, 1], 
                    Hue[0.2558772079638403, 1, 1], 
                    Hue[0.34433867572467236`, 1, 1], 
                    Hue[0.43317536470416307`, 1, 1], 
                    Hue[0.5223512697273097, 1, 1], 
                    Hue[0.611865760320544, 1, 1], 
                    Hue[0.7017376629319375, 1, 1], 
                    Hue[0.791973646004684, 1, 1], 
                    Hue[0.8825715119856232, 1, 1], 
                    Hue[0.9735201231658492, 1, 1], 
                    Hue[0.06481664673905892, 1, 1], 
                    Hue[0.1564624526544423, 1, 1], 
                    Hue[0.24846339708111093`, 1, 1], 
                    Hue[0.34082044254039434`, 1, 1], 
                    Hue[0.4335285188995974, 1, 1], 
                    Hue[0.5265858504718376, 1, 1], 
                    Hue[0.619991238019795, 1, 1], 
                    Hue[0.7137489787936226, 1, 1], 
                    Hue[0.8078581491198726, 1, 1], 
                    Hue[0.9023167908162214, 1, 1], 
                    Hue[0.997123518820222, 1, 1], 
                    Hue[0.09227809771084206, 1, 1], 
                    Hue[0.18778045984197525`, 1, 1], 
                    Hue[0.28363150538320664`, 1, 1], 
                    Hue[0.37982913467653123`, 1, 1], 
                    Hue[0.4763730534549754, 1, 1], 
                    Hue[0.5732620582331205, 1, 1], 
                    Hue[0.6704969545124291, 1, 1], 
                    Hue[0.7680769044926821, 1, 1], 
                    Hue[0.866001209191981, 1, 1], 
                    Hue[0.9642694798933042, 1, 1], 
                    Hue[0.06288033404720918, 1, 1], 
                    Hue[0.1618340190608663, 1, 1], 
                    Hue[0.26112965826933277`, 1, 1], 
                    Hue[0.36076682669457827`, 1, 1], 
                    Hue[0.4607446702920191, 1, 1], 
                    Hue[0.5610625757967369, 1, 1], 
                    Hue[0.6617196759615321, 1, 1], 
                    Hue[0.7627147629687548, 1, 1], 
                    Hue[0.8640473849020327, 1, 1], 
                    Hue[0.9657168711695278, 1, 1], 
                    Hue[0.06772203209084474, 1, 1], 
                    Hue[0.17006179329564264`, 1, 1], 
                    Hue[0.2727348799293318, 1, 1], 
                    Hue[0.3757400482244432, 1, 1], 
                    Hue[0.47907615599813785`, 1, 1], 
                    Hue[0.5827411225608394, 1, 1], 
                    Hue[0.686732977202875, 1, 1], 
                    Hue[0.7910505657857274, 1, 1], 
                    Hue[0.8956910816409434, 1, 1], 
                    Hue[0.0006518319749389761, 1, 1], 
                    Hue[0.10592980643142957`, 1, 1], 
                    Hue[0.21152177735264863`, 1, 1], 
                    Hue[0.3174239892158998, 1, 1], 
                    Hue[0.4236321033035602, 1, 1], 
                    Hue[0.5301403332116573, 1, 1], 
                    Hue[0.6369434399221541, 1, 1], 
                    Hue[0.7440350187497498, 1, 1], 
                    Hue[0.8514071856099105, 1, 1], 
                    Hue[0.9590505875381703, 1, 1], 
                    Hue[0.06695495506670603, 1, 1], 
                    Hue[0.17510840763985963`, 1, 1], 
                    Hue[0.2834976571366844, 1, 1], 
                    Hue[0.3921067036965846, 1, 1], 
                    Hue[0.5009171822537385, 1, 1], 
                    Hue[0.609907960840949, 1, 1], 
                    Hue[0.7190549385107554, 1, 1], 
                    Hue[0.8283307021079086, 1, 1], 
                    Hue[0.9377040687499487, 1, 1], 
                    Hue[0.04713886558145295, 1, 1], 
                    Hue[0.15659333346659132`, 1, 1], 
                    Hue[0.26602139648303896`, 1, 1], 
                    Hue[0.37537095342567955`, 1, 1], 
                    Hue[0.48458443441848675`, 1, 1], 
                    Hue[0.5935967404345542, 1, 1], 
                    Hue[0.7023349586497987, 1, 1], 
                    Hue[0.8107237607449748, 1, 1], 
                    Hue[0.9186813036861303, 1, 1], 
                    Hue[0.02611720447668775, 1, 1], 
                    Hue[0.1329418218181866, 1, 1], 
                    Hue[0.23906924624217446`, 1, 1], 
                    Hue[0.34440835836416867`, 1, 1], 
                    Hue[0.4488665860210912, 1, 1], 
                    Hue[0.5523753592326575, 1, 1], 
                    Hue[0.6548918734757004, 1, 1], 
                    Hue[0.7563690714498079, 1, 1], 
                    Hue[0.856754833013501, 1, 1], 
                    Hue[0.9560476299526799, 1, 1], 
                    Hue[0.05431861348849255, 1, 1], 
                    Hue[0.15165859477128055`, 1, 1], 
                    Hue[0.2481278362722127, 1, 1], 
                    Hue[0.34377181562590414`, 1, 1], 
                    Hue[0.43867250905938415`, 1, 1], 
                    Hue[0.5329818091731069, 1, 1], 
                    Hue[0.6268784974511232, 1, 1], 
                    Hue[0.7204800452917036, 1, 1], 
                    Hue[0.8138447115607892, 1, 1], 
                    Hue[0.9070647396475882, 1, 1], 
                    Hue[0.00027500323336431757`, 1, 1], 
                    Hue[0.0936287218150815, 1, 1], 
                    Hue[0.18738898166912799`, 1, 1], 
                    Hue[0.2817608025135326, 1, 1], 
                    Hue[0.3762560119614834, 1, 1], 
                    Hue[0.46983126202546877`, 1, 1], 
                    Hue[0.5628818631021897, 1, 1], 
                    Hue[0.6583764075712767, 1, 1], 
                    Hue[0.7583085727453893, 1, 1], 
                    Hue[0.8597558378859961, 1, 1], 
                    Hue[0.9565076209795028, 1, 1], 
                    Hue[0.04627851579084141, 1, 1], 
                    Hue[0.13696150887572137`, 1, 1], 
                    Hue[0.236959712104721, 1, 1], 
                    Hue[0.34512012689175703`, 1, 1], 
                    Hue[0.45688774460841647`, 1, 1], 
                    Hue[0.5703780558213525, 1, 1], 
                    Hue[0.6836540521049628, 1, 1], 
                    Hue[0.7838772463981823, 1, 1], 
                    Hue[0.8381977472273094, 1, 1], 
                    Hue[0.8870544606419305, 1, 1], 
                    Hue[0.003981834612291627, 1, 1], 
                    Hue[0.16018129844586784`, 1, 1], 
                    Hue[0.33780729398489723`, 1, 1], 
                    Hue[0.5369775037050852, 1, 1], 
                    Hue[0.7905257246587302, 1, 1], 
                    Hue[0.1945036657769734, 1, 1], 
                    Hue[0.48496065810113864`, 1, 1], 
                    Hue[0.7153646411488792, 1, 1], 
                    Hue[0.9394067463789025, 1, 1], 
                    Hue[0.17080211619971455`, 1, 1], 
                    Hue[0.4207562168980021, 1, 1], 
                    Hue[0.6977457893116823, 1, 1], 
                    Hue[0.9810396549863314, 1, 1], 
                    Hue[0.23870865443606856`, 1, 1], 
                    Hue[0.4717879390337842, 1, 1], 
                    Hue[0.6898493957007891, 1, 1], 
                    Hue[0.8977125550908651, 1, 1], 
                    Hue[0.09510862803082076, 1, 1], 
                    Hue[0.22425606418634192`, 1, 1], 
                    Hue[0.9819966346546343, 1, 1], 
                    Hue[0.12933728393861235`, 1, 1], 
                    Hue[0.24718059981823032`, 1, 1], 
                    Hue[0.3560042502224064, 1, 1], 
                    Hue[0.4832174243664257, 1, 1], 
                    Hue[0.6225466914902852, 1, 1], 
                    Hue[0.7669134155846555, 1, 1], 
                    Hue[0.919797571218377, 1, 1], 
                    Hue[0.08538170051501558, 1, 1], 
                    Hue[0.2620427531752457, 1, 1], 
                    Hue[0.44406556173858014`, 1, 1], 
                    Hue[0.6185988292105646, 1, 1], 
                    Hue[0.6838184160682961, 1, 1], 
                    Hue[0.6378535206406792, 1, 1], 
                    Hue[0.8073537712650531, 1, 1], 
                    Hue[0.0008353865620092372, 1, 1], 
                    Hue[0.1996041412947276, 1, 1], 
                    Hue[0.3994928084092794, 1, 1], 
                    Hue[0.598581266602507, 1, 1], 
                    Hue[0.795286513537276, 1, 1], 
                    Hue[0.9874661970418543, 1, 1], 
                    Hue[0.17098866130101048`, 1, 1], 
                    Hue[0.33608016823445647`, 1, 1], 
                    Hue[0.4626610597119734, 1, 1], 
                    Hue[0.5600193469710358, 1, 1], 
                    Hue[0.6951140435126271, 1, 1], 
                    Hue[0.8688175337237586, 1, 1], 
                    Hue[0.06217243610467935, 1, 1], 
                    Hue[0.26673052970331346`, 1, 1], 
                    Hue[0.47918613708383234`, 1, 1], 
                    Hue[0.6981752604591175, 1, 1], 
                    Hue[0.923139519550703, 1, 1], 
                    Hue[0.15391413553106462`, 1, 1], 
                    Hue[0.39058746840666203`, 1, 1], 
                    Hue[0.6335130531533011, 1, 1], 
                    Hue[0.8834878616778492, 1, 1], 
                    Hue[0.14218772425266332`, 1, 1], 
                    Hue[0.4129145318615668, 1, 1], 
                    Hue[0.7008647172568718, 1, 1], 
                    Hue[0.008276153030424088, 1, 1], 
                    Hue[0.32203152553002135`, 1, 1], 
                    Hue[0.6228272969220539, 1, 1], 
                    Hue[0.9077377802688978, 1, 1], 
                    Hue[0.18295813338288494`, 1, 1], 
                    Hue[0.45436977284719, 1, 1], 
                    Hue[0.7264230564635294, 1, 1], 
                    Hue[0.0029732826562475275`, 1, 1], 
                    Hue[0.28772899918756284`, 1, 1], 
                    Hue[0.5832291550635875, 1, 1], 
                    Hue[0.8874633543999638, 1, 1], 
                    Hue[0.1922430080867037, 1, 1], 
                    Hue[0.489719235032304, 1, 1], 
                    Hue[0.7780535457697576, 1, 1], 
                    Hue[0.0590819613523457, 1, 1], 
                    Hue[0.3350544804696013, 1, 1], 
                    Hue[0.6076027369926792, 1, 1], 
                    Hue[0.8777648644902436, 1, 1], 
                    Hue[0.1461932685057524, 1, 1], 
                    Hue[0.4133070810184004, 1, 1], 
                    Hue[0.6793688455124909, 1, 1], 
                    Hue[0.944513321626013, 1, 1], 
                    Hue[0.20875530569204304`, 1, 1], 
                    Hue[0.4719956959154934, 1, 1], 
                    Hue[0.7340353362877327, 1, 1], 
                    Hue[0.9945952634521681, 1, 1], 
                    Hue[0.2533389529365845, 1, 1], 
                    Hue[0.5098912530483598, 1, 1], 
                    Hue[0.7638545596346901, 1, 1], 
                    Hue[0.01483438802715526, 1, 1], 
                    Hue[0.2624959819063229, 1, 1], 
                    Hue[0.5066777947454011, 1, 1], 
                    Hue[0.7475644420744317, 1, 1], 
                    Hue[0.9858518958082757, 1, 1], 
                    Hue[0.2227535812882585, 1, 1], 
                    Hue[0.45973174016904605`, 1, 1], 
                    Hue[0.6980657527124632, 1, 1], 
                    Hue[0.9640532089338167, 1, 1], 
                    Hue[0.7638938367871528, 1, 1], 
                    Hue[0.7638938367871528, 1, 1], 
                    Hue[0.5456953793797118, 1, 1], 
                    Hue[0.5456953793797118, 1, 1], 
                    Hue[0.4880359025202404, 1, 1], 
                    Hue[0.4880359025202404, 1, 1], 
                    Hue[0.418755936350388, 1, 1], 
                    Hue[0.418755936350388, 1, 1], 
                    Hue[0.22471186555443298`, 1, 1], 
                    Hue[0.22471186555443298`, 1, 1], 
                    Hue[0.7664365867469122, 1, 1], 
                    Hue[0.7664365867469122, 1, 1], 
                    Hue[0.7569067121630682, 1, 1], 
                    Hue[0.7569067121630682, 1, 1], 
                    Hue[0.8201765986296474, 1, 1], 
                    Hue[0.8201765986296474, 1, 1], 
                    Hue[0.8383737951258301, 1, 1], 
                    Hue[0.8383737951258301, 1, 1], 
                    Hue[0.8528114312520936, 1, 1], 
                    Hue[0.8528114312520936, 1, 1], 
                    Hue[0.0005814275632171318, 1, 1], 
                    Hue[0.0005814275632171318, 1, 1], 
                    Hue[0.09900120701277339, 1, 1], 
                    Hue[0.09900120701277339, 1, 1], 
                    Hue[0.1775661426240637, 1, 1], 
                    Hue[0.1775661426240637, 1, 1], 
                    Hue[0.24423919184988493`, 1, 1], 
                    Hue[0.24423919184988493`, 1, 1], 
                    Hue[0.3156688798563375, 1, 1], 
                    Hue[0.3156688798563375, 1, 1], 
                    Hue[0.38950220432683996`, 1, 1], 
                    Hue[0.38950220432683996`, 1, 1], 
                    Hue[0.4814958525831977, 1, 1], 
                    Hue[0.4814958525831977, 1, 1], 
                    Hue[0.5657261147473843, 1, 1], 
                    Hue[0.5657261147473843, 1, 1], 
                    Hue[0.6467337892415539, 1, 1], 
                    Hue[0.6467337892415539, 1, 1], 
                    Hue[0.7248416611021938, 1, 1], 
                    Hue[0.7248416611021938, 1, 1], 
                    Hue[0.8055142767348117, 1, 1], 
                    Hue[0.8055142767348117, 1, 1], 
                    Hue[0.8872992703340177, 1, 1], 
                    Hue[0.8872992703340177, 1, 1], 
                    Hue[0.9720085880616776, 1, 1], 
                    Hue[0.9720085880616776, 1, 1], 
                    Hue[0.05610154862290952, 1, 1], 
                    Hue[0.05610154862290952, 1, 1], 
                    Hue[0.13926945181155012`, 1, 1], 
                    Hue[0.13926945181155012`, 1, 1], 
                    Hue[0.22225896671723153`, 1, 1], 
                    Hue[0.22225896671723153`, 1, 1], 
                    Hue[0.30594469461240686`, 1, 1], 
                    Hue[0.30594469461240686`, 1, 1], 
                    Hue[0.3907297817738525, 1, 1], 
                    Hue[0.3907297817738525, 1, 1], 
                    Hue[0.47584462841575326`, 1, 1], 
                    Hue[0.47584462841575326`, 1, 1], 
                    Hue[0.5613011947128719, 1, 1], 
                    Hue[0.5613011947128719, 1, 1], 
                    Hue[0.6467734454202569, 1, 1], 
                    Hue[0.6467734454202569, 1, 1], 
                    Hue[0.732620054471488, 1, 1], 
                    Hue[0.732620054471488, 1, 1], 
                    Hue[0.8188508419403508, 1, 1], 
                    Hue[0.8188508419403508, 1, 1], 
                    Hue[0.9055713829157245, 1, 1], 
                    Hue[0.9055713829157245, 1, 1], 
                    Hue[0.9926765120282037, 1, 1], 
                    Hue[0.9926765120282037, 1, 1], 
                    Hue[0.08008348174035529, 1, 1], 
                    Hue[0.08008348174035529, 1, 1], 
                    Hue[0.1678032957843583, 1, 1], 
                    Hue[0.1678032957843583, 1, 1], 
                    Hue[0.2558772079638403, 1, 1], 
                    Hue[0.2558772079638403, 1, 1], 
                    Hue[0.34433867572467236`, 1, 1], 
                    Hue[0.34433867572467236`, 1, 1], 
                    Hue[0.43317536470416307`, 1, 1], 
                    Hue[0.43317536470416307`, 1, 1], 
                    Hue[0.5223512697273097, 1, 1], 
                    Hue[0.5223512697273097, 1, 1], 
                    Hue[0.611865760320544, 1, 1], 
                    Hue[0.611865760320544, 1, 1], 
                    Hue[0.7017376629319375, 1, 1], 
                    Hue[0.7017376629319375, 1, 1], 
                    Hue[0.791973646004684, 1, 1], 
                    Hue[0.791973646004684, 1, 1], 
                    Hue[0.8825715119856232, 1, 1], 
                    Hue[0.8825715119856232, 1, 1], 
                    Hue[0.9735201231658492, 1, 1], 
                    Hue[0.9735201231658492, 1, 1], 
                    Hue[0.06481664673905892, 1, 1], 
                    Hue[0.06481664673905892, 1, 1], 
                    Hue[0.1564624526544423, 1, 1], 
                    Hue[0.1564624526544423, 1, 1], 
                    Hue[0.24846339708111093`, 1, 1], 
                    Hue[0.24846339708111093`, 1, 1], 
                    Hue[0.34082044254039434`, 1, 1], 
                    Hue[0.34082044254039434`, 1, 1], 
                    Hue[0.4335285188995974, 1, 1], 
                    Hue[0.4335285188995974, 1, 1], 
                    Hue[0.5265858504718376, 1, 1], 
                    Hue[0.5265858504718376, 1, 1], 
                    Hue[0.619991238019795, 1, 1], 
                    Hue[0.619991238019795, 1, 1], 
                    Hue[0.7137489787936226, 1, 1], 
                    Hue[0.7137489787936226, 1, 1], 
                    Hue[0.8078581491198726, 1, 1], 
                    Hue[0.8078581491198726, 1, 1], 
                    Hue[0.9023167908162214, 1, 1], 
                    Hue[0.9023167908162214, 1, 1], 
                    Hue[0.997123518820222, 1, 1], 
                    Hue[0.997123518820222, 1, 1], 
                    Hue[0.09227809771084206, 1, 1], 
                    Hue[0.09227809771084206, 1, 1], 
                    Hue[0.18778045984197525`, 1, 1], 
                    Hue[0.18778045984197525`, 1, 1], 
                    Hue[0.28363150538320664`, 1, 1], 
                    Hue[0.28363150538320664`, 1, 1], 
                    Hue[0.37982913467653123`, 1, 1], 
                    Hue[0.37982913467653123`, 1, 1], 
                    Hue[0.4763730534549754, 1, 1], 
                    Hue[0.4763730534549754, 1, 1], 
                    Hue[0.5732620582331205, 1, 1], 
                    Hue[0.5732620582331205, 1, 1], 
                    Hue[0.6704969545124291, 1, 1], 
                    Hue[0.6704969545124291, 1, 1], 
                    Hue[0.7680769044926821, 1, 1], 
                    Hue[0.7680769044926821, 1, 1], 
                    Hue[0.866001209191981, 1, 1], 
                    Hue[0.866001209191981, 1, 1], 
                    Hue[0.9642694798933042, 1, 1], 
                    Hue[0.9642694798933042, 1, 1], 
                    Hue[0.06288033404720918, 1, 1], 
                    Hue[0.06288033404720918, 1, 1], 
                    Hue[0.1618340190608663, 1, 1], 
                    Hue[0.1618340190608663, 1, 1], 
                    Hue[0.26112965826933277`, 1, 1], 
                    Hue[0.26112965826933277`, 1, 1], 
                    Hue[0.36076682669457827`, 1, 1], 
                    Hue[0.36076682669457827`, 1, 1], 
                    Hue[0.4607446702920191, 1, 1], 
                    Hue[0.4607446702920191, 1, 1], 
                    Hue[0.5610625757967369, 1, 1], 
                    Hue[0.5610625757967369, 1, 1], 
                    Hue[0.6617196759615321, 1, 1], 
                    Hue[0.6617196759615321, 1, 1], 
                    Hue[0.7627147629687548, 1, 1], 
                    Hue[0.7627147629687548, 1, 1], 
                    Hue[0.8640473849020327, 1, 1], 
                    Hue[0.8640473849020327, 1, 1], 
                    Hue[0.9657168711695278, 1, 1], 
                    Hue[0.9657168711695278, 1, 1], 
                    Hue[0.06772203209084474, 1, 1], 
                    Hue[0.06772203209084474, 1, 1], 
                    Hue[0.17006179329564264`, 1, 1], 
                    Hue[0.17006179329564264`, 1, 1], 
                    Hue[0.2727348799293318, 1, 1], 
                    Hue[0.2727348799293318, 1, 1], 
                    Hue[0.3757400482244432, 1, 1], 
                    Hue[0.3757400482244432, 1, 1], 
                    Hue[0.47907615599813785`, 1, 1], 
                    Hue[0.47907615599813785`, 1, 1], 
                    Hue[0.5827411225608394, 1, 1], 
                    Hue[0.5827411225608394, 1, 1], 
                    Hue[0.686732977202875, 1, 1], 
                    Hue[0.686732977202875, 1, 1], 
                    Hue[0.7910505657857274, 1, 1], 
                    Hue[0.7910505657857274, 1, 1], 
                    Hue[0.8956910816409434, 1, 1], 
                    Hue[0.8956910816409434, 1, 1], 
                    Hue[0.0006518319749389761, 1, 1], 
                    Hue[0.0006518319749389761, 1, 1], 
                    Hue[0.10592980643142957`, 1, 1], 
                    Hue[0.10592980643142957`, 1, 1], 
                    Hue[0.21152177735264863`, 1, 1], 
                    Hue[0.21152177735264863`, 1, 1], 
                    Hue[0.3174239892158998, 1, 1], 
                    Hue[0.3174239892158998, 1, 1], 
                    Hue[0.4236321033035602, 1, 1], 
                    Hue[0.4236321033035602, 1, 1], 
                    Hue[0.5301403332116573, 1, 1], 
                    Hue[0.5301403332116573, 1, 1], 
                    Hue[0.6369434399221541, 1, 1], 
                    Hue[0.6369434399221541, 1, 1], 
                    Hue[0.7440350187497498, 1, 1], 
                    Hue[0.7440350187497498, 1, 1], 
                    Hue[0.8514071856099105, 1, 1], 
                    Hue[0.8514071856099105, 1, 1], 
                    Hue[0.9590505875381703, 1, 1], 
                    Hue[0.9590505875381703, 1, 1], 
                    Hue[0.06695495506670603, 1, 1], 
                    Hue[0.06695495506670603, 1, 1], 
                    Hue[0.17510840763985963`, 1, 1], 
                    Hue[0.17510840763985963`, 1, 1], 
                    Hue[0.2834976571366844, 1, 1], 
                    Hue[0.2834976571366844, 1, 1], 
                    Hue[0.3921067036965846, 1, 1], 
                    Hue[0.3921067036965846, 1, 1], 
                    Hue[0.5009171822537385, 1, 1], 
                    Hue[0.5009171822537385, 1, 1], 
                    Hue[0.609907960840949, 1, 1], 
                    Hue[0.609907960840949, 1, 1], 
                    Hue[0.7190549385107554, 1, 1], 
                    Hue[0.7190549385107554, 1, 1], 
                    Hue[0.8283307021079086, 1, 1], 
                    Hue[0.8283307021079086, 1, 1], 
                    Hue[0.9377040687499487, 1, 1], 
                    Hue[0.9377040687499487, 1, 1], 
                    Hue[0.04713886558145295, 1, 1], 
                    Hue[0.04713886558145295, 1, 1], 
                    Hue[0.15659333346659132`, 1, 1], 
                    Hue[0.15659333346659132`, 1, 1], 
                    Hue[0.26602139648303896`, 1, 1], 
                    Hue[0.26602139648303896`, 1, 1], 
                    Hue[0.37537095342567955`, 1, 1], 
                    Hue[0.37537095342567955`, 1, 1], 
                    Hue[0.48458443441848675`, 1, 1], 
                    Hue[0.48458443441848675`, 1, 1], 
                    Hue[0.5935967404345542, 1, 1], 
                    Hue[0.5935967404345542, 1, 1], 
                    Hue[0.7023349586497987, 1, 1], 
                    Hue[0.7023349586497987, 1, 1], 
                    Hue[0.8107237607449748, 1, 1], 
                    Hue[0.8107237607449748, 1, 1], 
                    Hue[0.9186813036861303, 1, 1], 
                    Hue[0.9186813036861303, 1, 1], 
                    Hue[0.02611720447668775, 1, 1], 
                    Hue[0.02611720447668775, 1, 1], 
                    Hue[0.1329418218181866, 1, 1], 
                    Hue[0.1329418218181866, 1, 1], 
                    Hue[0.23906924624217446`, 1, 1], 
                    Hue[0.23906924624217446`, 1, 1], 
                    Hue[0.34440835836416867`, 1, 1], 
                    Hue[0.34440835836416867`, 1, 1], 
                    Hue[0.4488665860210912, 1, 1], 
                    Hue[0.4488665860210912, 1, 1], 
                    Hue[0.5523753592326575, 1, 1], 
                    Hue[0.5523753592326575, 1, 1], 
                    Hue[0.6548918734757004, 1, 1], 
                    Hue[0.6548918734757004, 1, 1], 
                    Hue[0.7563690714498079, 1, 1], 
                    Hue[0.7563690714498079, 1, 1], 
                    Hue[0.856754833013501, 1, 1], 
                    Hue[0.856754833013501, 1, 1], 
                    Hue[0.9560476299526799, 1, 1], 
                    Hue[0.9560476299526799, 1, 1], 
                    Hue[0.05431861348849255, 1, 1], 
                    Hue[0.05431861348849255, 1, 1], 
                    Hue[0.15165859477128055`, 1, 1], 
                    Hue[0.15165859477128055`, 1, 1], 
                    Hue[0.2481278362722127, 1, 1], 
                    Hue[0.2481278362722127, 1, 1], 
                    Hue[0.34377181562590414`, 1, 1], 
                    Hue[0.34377181562590414`, 1, 1], 
                    Hue[0.43867250905938415`, 1, 1], 
                    Hue[0.43867250905938415`, 1, 1], 
                    Hue[0.5329818091731069, 1, 1], 
                    Hue[0.5329818091731069, 1, 1], 
                    Hue[0.6268784974511232, 1, 1], 
                    Hue[0.6268784974511232, 1, 1], 
                    Hue[0.7204800452917036, 1, 1], 
                    Hue[0.7204800452917036, 1, 1], 
                    Hue[0.8138447115607892, 1, 1], 
                    Hue[0.8138447115607892, 1, 1], 
                    Hue[0.9070647396475882, 1, 1], 
                    Hue[0.9070647396475882, 1, 1], 
                    Hue[0.00027500323336431757`, 1, 1], 
                    Hue[0.00027500323336431757`, 1, 1], 
                    Hue[0.0936287218150815, 1, 1], 
                    Hue[0.0936287218150815, 1, 1], 
                    Hue[0.18738898166912799`, 1, 1], 
                    Hue[0.18738898166912799`, 1, 1], 
                    Hue[0.2817608025135326, 1, 1], 
                    Hue[0.2817608025135326, 1, 1], 
                    Hue[0.3762560119614834, 1, 1], 
                    Hue[0.3762560119614834, 1, 1], 
                    Hue[0.46983126202546877`, 1, 1], 
                    Hue[0.46983126202546877`, 1, 1], 
                    Hue[0.5628818631021897, 1, 1], 
                    Hue[0.5628818631021897, 1, 1], 
                    Hue[0.6583764075712767, 1, 1], 
                    Hue[0.6583764075712767, 1, 1], 
                    Hue[0.7583085727453893, 1, 1], 
                    Hue[0.7583085727453893, 1, 1], 
                    Hue[0.8597558378859961, 1, 1], 
                    Hue[0.8597558378859961, 1, 1], 
                    Hue[0.9565076209795028, 1, 1], 
                    Hue[0.9565076209795028, 1, 1], 
                    Hue[0.04627851579084141, 1, 1], 
                    Hue[0.04627851579084141, 1, 1], 
                    Hue[0.13696150887572137`, 1, 1], 
                    Hue[0.13696150887572137`, 1, 1], 
                    Hue[0.236959712104721, 1, 1], 
                    Hue[0.236959712104721, 1, 1], 
                    Hue[0.34512012689175703`, 1, 1], 
                    Hue[0.34512012689175703`, 1, 1], 
                    Hue[0.45688774460841647`, 1, 1], 
                    Hue[0.45688774460841647`, 1, 1], 
                    Hue[0.5703780558213525, 1, 1], 
                    Hue[0.5703780558213525, 1, 1], 
                    Hue[0.6836540521049628, 1, 1], 
                    Hue[0.6836540521049628, 1, 1], 
                    Hue[0.7838772463981823, 1, 1], 
                    Hue[0.7838772463981823, 1, 1], 
                    Hue[0.8381977472273094, 1, 1], 
                    Hue[0.8381977472273094, 1, 1], 
                    Hue[0.8870544606419305, 1, 1], 
                    Hue[0.8870544606419305, 1, 1], 
                    Hue[0.003981834612291627, 1, 1], 
                    Hue[0.003981834612291627, 1, 1], 
                    Hue[0.16018129844586784`, 1, 1], 
                    Hue[0.16018129844586784`, 1, 1], 
                    Hue[0.33780729398489723`, 1, 1], 
                    Hue[0.33780729398489723`, 1, 1], 
                    Hue[0.5369775037050852, 1, 1], 
                    Hue[0.5369775037050852, 1, 1], 
                    Hue[0.7905257246587302, 1, 1], 
                    Hue[0.7905257246587302, 1, 1], 
                    Hue[0.1945036657769734, 1, 1], 
                    Hue[0.1945036657769734, 1, 1], 
                    Hue[0.48496065810113864`, 1, 1], 
                    Hue[0.48496065810113864`, 1, 1], 
                    Hue[0.7153646411488792, 1, 1], 
                    Hue[0.7153646411488792, 1, 1], 
                    Hue[0.9394067463789025, 1, 1], 
                    Hue[0.9394067463789025, 1, 1], 
                    Hue[0.17080211619971455`, 1, 1], 
                    Hue[0.17080211619971455`, 1, 1], 
                    Hue[0.4207562168980021, 1, 1], 
                    Hue[0.4207562168980021, 1, 1], 
                    Hue[0.6977457893116823, 1, 1], 
                    Hue[0.6977457893116823, 1, 1], 
                    Hue[0.9810396549863314, 1, 1], 
                    Hue[0.9810396549863314, 1, 1], 
                    Hue[0.23870865443606856`, 1, 1], 
                    Hue[0.23870865443606856`, 1, 1], 
                    Hue[0.4717879390337842, 1, 1], 
                    Hue[0.4717879390337842, 1, 1], 
                    Hue[0.6898493957007891, 1, 1], 
                    Hue[0.6898493957007891, 1, 1], 
                    Hue[0.8977125550908651, 1, 1], 
                    Hue[0.8977125550908651, 1, 1], 
                    Hue[0.09510862803082076, 1, 1], 
                    Hue[0.09510862803082076, 1, 1], 
                    Hue[0.22425606418634192`, 1, 1], 
                    Hue[0.22425606418634192`, 1, 1], 
                    Hue[0.9819966346546343, 1, 1], 
                    Hue[0.9819966346546343, 1, 1], 
                    Hue[0.12933728393861235`, 1, 1], 
                    Hue[0.12933728393861235`, 1, 1], 
                    Hue[0.24718059981823032`, 1, 1], 
                    Hue[0.24718059981823032`, 1, 1], 
                    Hue[0.3560042502224064, 1, 1], 
                    Hue[0.3560042502224064, 1, 1], 
                    Hue[0.4832174243664257, 1, 1], 
                    Hue[0.4832174243664257, 1, 1], 
                    Hue[0.6225466914902852, 1, 1], 
                    Hue[0.6225466914902852, 1, 1], 
                    Hue[0.7669134155846555, 1, 1], 
                    Hue[0.7669134155846555, 1, 1], 
                    Hue[0.919797571218377, 1, 1], 
                    Hue[0.919797571218377, 1, 1], 
                    Hue[0.08538170051501558, 1, 1], 
                    Hue[0.08538170051501558, 1, 1], 
                    Hue[0.2620427531752457, 1, 1], 
                    Hue[0.2620427531752457, 1, 1], 
                    Hue[0.44406556173858014`, 1, 1], 
                    Hue[0.44406556173858014`, 1, 1], 
                    Hue[0.6185988292105646, 1, 1], 
                    Hue[0.6185988292105646, 1, 1], 
                    Hue[0.6838184160682961, 1, 1], 
                    Hue[0.6838184160682961, 1, 1], 
                    Hue[0.6378535206406792, 1, 1], 
                    Hue[0.6378535206406792, 1, 1], 
                    Hue[0.8073537712650531, 1, 1], 
                    Hue[0.8073537712650531, 1, 1], 
                    Hue[0.0008353865620092372, 1, 1], 
                    Hue[0.0008353865620092372, 1, 1], 
                    Hue[0.1996041412947276, 1, 1], 
                    Hue[0.1996041412947276, 1, 1], 
                    Hue[0.3994928084092794, 1, 1], 
                    Hue[0.3994928084092794, 1, 1], 
                    Hue[0.598581266602507, 1, 1], 
                    Hue[0.598581266602507, 1, 1]}],
                   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
                   Axes->{True, None},
                   AxesOrigin->{0, 0},
                   
                   Epilog->LineBox[{{1, 4.994783395296727*^-7}, {
                    2, 3.986765672925796*^-7}, {3, 3.6484116999930404`*^-7}, {
                    4, 4.244605140076791*^-7}, {5, 9.418473172586208*^-7}, {
                    6, 6.977534201348251*^-7}, {7, 4.324780565545246*^-7}, {
                    8, 7.739143884695598*^-7}, {9, 1.4925662927061932`*^-6}, {
                    10, 2.7363528323711107`*^-6}, {
                    11, 2.3694317865726362`*^-6}, {
                    12, 2.9428984745531942`*^-6}, {
                    13, 4.927896848144337*^-6}, {14, 7.473823446927416*^-6}, {
                    15, 0.000011730229772105373`}, {
                    16, 0.00001624049470443861}, {
                    17, 0.000020944855409634024`}, {
                    18, 0.000028871176479729104`}, {
                    19, 0.00004168759055026988}, {
                    20, 0.000058977781410749604`}, {
                    21, 0.0000821512438370269}, {
                    22, 0.00011153933646128924`}, {
                    23, 0.0001501877725257336}, {24, 0.0002028127511679077}, {
                    25, 0.0002752063422036464}, {
                    26, 0.00037196451476663195`}, {
                    27, 0.0004968254289896502}, {28, 0.0006569383380542082}, {
                    29, 0.0008646777264225473}, {
                    30, 0.0011341787163496409`}, {
                    31, 0.0014812983035309155`}, {32, 0.00192451958873768}, {
                    33, 0.002482691121822296}, {34, 0.003184244258854491}, {
                    35, 0.004061454108004423}, {36, 0.005153334725356818}, {
                    37, 0.006503897404234565}, {38, 0.00816267274936306}, {
                    39, 0.010184664622306338`}, {40, 0.012635410541207903`}, {
                    41, 0.015590589359775657`}, {42, 0.019132140781103642`}, {
                    43, 0.02334596316779318}, {44, 0.028326687478641765`}, {
                    45, 0.03417704305872553}, {46, 0.04100739099097344}, {
                    47, 0.04893038528968952}, {48, 0.05805861289827473}, {
                    49, 0.06850378714107186}, {50, 0.08037362824657279}, {
                    51, 0.09377546516286478}, {52, 0.10880283771743832`}, {
                    53, 0.12553376031808236`}, {54, 0.1440262124893879}, {
                    55, 0.1643184702438419}, {56, 0.18642212994643712`}, {
                    57, 0.21031955947502556`}, {58, 0.23595640277360674`}, {
                    59, 0.2632369775285807}, {60, 0.2920304586291676}, {
                    61, 0.3221626679815606}, {62, 0.35342105068282276`}, {
                    63, 0.38554662093534187`}, {64, 0.4182415652473806}, {
                    65, 0.4511732152331836}, {66, 0.4839799614117237}, {
                    67, 0.516273069711347}, {68, 0.5476464942019678}, {
                    69, 0.577682177984169}, {70, 0.6059621045677198}, {
                    71, 0.6320778322936406}, {72, 0.6556404966435839}, {
                    73, 0.676285525540514}, {74, 0.693689416263403}, {
                    75, 0.7075731820501371}, {76, 0.7177089826948957}, {
                    77, 0.7239293602809228}, {78, 0.7261337407825015}, {
                    79, 0.7242859668634876}, {80, 0.7184165466300587}, {
                    81, 0.7086250347294344}, {82, 0.695074795190095}, {
                    83, 0.677987255579029}, {84, 0.6576373569956018}, {
                    85, 0.6343483778240276}, {86, 0.6084786474910284}, {
                    87, 0.5804163306991071}, {88, 0.5505677145457509}, {
                    89, 0.519347793284551}, {90, 0.4871724957189612}, {
                    91, 0.45444776977078666`}, {92, 0.42156145609977047`}, {
                    93, 0.3888776271948038}, {94, 0.3567295637615142}, {
                    95, 0.3254148972148669}, {96, 0.2951925286448773}, {
                    97, 0.2662803790311598}, {98, 0.23885501015649402`}, {
                    99, 0.2130524435276181}, {100, 0.18896874158666613`}, {
                    101, 0.16666166593939843`}, {102, 0.14615450600623264`}, {
                    103, 0.1274408271590448}, {104, 0.11048699731783652`}, {
                    105, 0.09523581019732066}, {106, 0.08161186277289911}, {
                    107, 0.0695251588752548}, {108, 0.058875386586847635`}, {
                    109, 0.04955547836873383}, {110, 0.041454307948559935`}, {
                    111, 0.034459699409057146`}, {
                    112, 0.028460898980589244`}, {113, 0.02335110917215513}, {
                    114, 0.0190283327245}, {115, 0.01539628957952017}, {
                    116, 0.012366074596434767`}, {117, 0.00985640965709907}, {
                    118, 0.007793159716799637}, {119, 0.006109706828236233}, {
                    120, 0.004747389885704473}, {121, 0.003654630612157861}, {
                    122, 0.00278583449591614}, {123, 0.002101416925266802}, {
                    124, 0.0015679027167395609`}, {
                    125, 0.0011569275591605903`}, {
                    126, 0.0008440839473146381}, {
                    127, 0.000608650078946116}, {
                    128, 0.0004335854234812388}, {
                    129, 0.000305138037729062}, {
                    130, 0.0002122319275392624}, {
                    131, 0.00014596258855150972`}, {
                    132, 0.00009928326149451607}, {
                    133, 0.00006679085306027818}, {
                    134, 0.00004445655855107141}, {
                    135, 0.000029301007706538234`}, {
                    136, 0.000019137126774696508`}, {
                    137, 0.000012394565342042676`}, {
                    138, 7.962990185967206*^-6}, {
                    139, 5.065137310831807*^-6}, {
                    140, 3.181557334776959*^-6}, {
                    141, 1.981775686439238*^-6}, {
                    142, 1.2399440938483532`*^-6}, {
                    143, 7.818038533064367*^-7}, {
                    144, 4.862508096344292*^-7}, {
                    145, 2.909588468730301*^-7}, {
                    146, 1.6860679387934232`*^-7}, {
                    147, 9.943081735171632*^-8}, {
                    148, 6.198255773861349*^-8}, {
                    149, 3.9474549989156254`*^-8}, {
                    150, 2.44297731769855*^-8}, {
                    151, 1.447854293784885*^-8}, {
                    152, 8.20793397722054*^-9}, {
                    153, 4.342580316686153*^-9}, {
                    154, 2.062342025287604*^-9}, {
                    155, 1.0076368918492748`*^-9}, {
                    156, 7.896011603640955*^-10}, {
                    157, 6.98709259034934*^-10}, {
                    158, 5.239002823885162*^-10}, {
                    159, 3.247461338186444*^-10}, {
                    160, 1.6094234553971135`*^-10}, {
                    161, 5.7890093799645316`*^-11}, {
                    162, 3.277946154119626*^-11}, {
                    163, 4.214313430738415*^-11}, {
                    164, 3.9492506979803*^-11}, {
                    165, 2.955361322243714*^-11}, {
                    166, 1.884218596997743*^-11}, {
                    167, 1.0776428920419602`*^-11}, {
                    168, 6.264934023401239*^-12}, {
                    169, 4.331228658996611*^-12}, {
                    170, 3.241645366087931*^-12}, {
                    171, 2.2306143417328605`*^-12}, {
                    172, 1.31707993640091*^-12}, {
                    173, 6.323571184904889*^-13}, {
                    174, 2.1377788431473829`*^-13}, {
                    175, 1.4334996999488572`*^-14}, {
                    176, 5.101102014106216*^-14}, {
                    177, 5.2624323987021615`*^-14}, {
                    178, 4.124389086521208*^-14}, {
                    179, 3.5078488374246167`*^-14}, {
                    180, 3.0903911663198325`*^-14}, {
                    181, 2.5791482943213817`*^-14}, {
                    182, 2.079675721174681*^-14}, {
                    183, 1.6507204738868056`*^-14}, {
                    184, 1.2465274374698773`*^-14}, {
                    185, 8.476546445570054*^-15}, {
                    186, 4.885914903938889*^-15}, {
                    187, 2.1220261024332918`*^-15}, {
                    188, 5.151240139758338*^-16}, {
                    189, 7.99574431085779*^-16}, {
                    190, 1.056800930724421*^-15}, {
                    191, 9.846417193976231*^-16}, {
                    192, 7.662773364131214*^-16}, {
                    193, 5.274523526772276*^-16}, {
                    194, 3.290537145353*^-16}, {
                    195, 1.8798858806434994`*^-16}, {
                    196, 9.851637199785806*^-17}, {
                    197, 4.713023954228207*^-17}, {
                    198, 2.057607043224922*^-17}, {
                    199, 8.950115268951903*^-18}, {
                    200, 5.351109179036183*^-18}, {
                    201, 4.237162057364687*^-18}, {
                    202, 3.2896698581742215`*^-18}, {
                    203, 2.354117431939352*^-18}, {
                    204, 1.5660407574147986`*^-18}, {
                    205, 9.800090437312581*^-19}, {
                    206, 5.816804785448641*^-19}, {
                    207, 3.2922684628414906`*^-19}, {
                    208, 1.7831273345047157`*^-19}, {
                    209, 9.261475510174089*^-20}, {
                    210, 4.6173453453776934`*^-20}, {
                    211, 2.2091537893628526`*^-20}, {
                    212, 1.0139860177673312`*^-20}, {
                    213, 4.4851396665016416`*^-21}, {
                    214, 1.9624031100872154`*^-21}, {
                    215, 9.160410256889905*^-22}, {
                    216, 4.953388953388919*^-22}, {
                    217, 2.9782906916741086`*^-22}, {
                    218, 1.8038045164791116`*^-22}, {
                    219, 1.04671288746965*^-22}, {
                    220, 5.74468597913101*^-23}, {
                    221, 2.984204839762825*^-23}, {
                    222, 1.478437538719076*^-23}, {
                    223, 7.107342540248776*^-24}, {
                    224, 3.4315368410250665`*^-24}, {
                    225, 1.745920460011025*^-24}, {
                    226, 9.606622142992313*^-25}, {
                    227, 5.585118706486809*^-25}, {
                    228, 3.2956046235983315`*^-25}, {
                    229, 1.9246520742020925`*^-25}, {
                    230, 1.1018257114828082`*^-25}, {
                    231, 6.170538318072313*^-26}, {
                    232, 3.382433919856732*^-26}, {
                    233, 1.817069358656203*^-26}, {
                    234, 9.579390093348711*^-27}, {
                    235, 4.9627919182631355`*^-27}, {
                    236, 2.5301041513653094`*^-27}, {
                    237, 1.2710356470657414`*^-27}, {
                    238, 6.2994704638294175`*^-28}, {
                    239, 3.0831482420624158`*^-28}, {
                    240, 1.4912070596623323`*^-28}, {
                    241, 7.131500414102161*^-29}, {
                    242, 3.3745137194828654`*^-29}, {
                    243, 1.5816624089791583`*^-29}, {
                    244, 7.358064935844816*^-30}, {
                    245, 3.408962563464699*^-30}, {
                    246, 1.5807318097306387`*^-30}, {
                    247, 7.3829435981384236`*^-31}, {
                    248, 3.495756697896012*^-31}, {
                    249, 1.6854287796342513`*^-31}, {
                    250, 8.278063589009809*^-32}, {
                    251, 4.1264891084231985`*^-32}}],
                   Frame->True,
                   FrameTicks->{{{1., 
                    FormBox[
                    RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
                    FormBox[
                    RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
                    FormBox[
                    RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
                    FormBox[
                    RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
                    FormBox["0", TraditionalForm]}, {157.25, 
                    FormBox["2.5`", TraditionalForm]}, {188.5, 
                    FormBox["5", TraditionalForm]}, {219.75, 
                    FormBox["7.5`", TraditionalForm]}, {251., 
                    FormBox["10", TraditionalForm]}}, Automatic, None, None},
                   PlotRange->{-0.1, 1.1},
                   PlotRangeClipping->True,
                   PlotRangePadding->{
                    Scaled[0.02], Automatic}], 5->
                  GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF2XdUFIcaBfDR6BNLwKgxIOpbFdHEhlQRkauASFHpvQy9xIYlMQTJJAbU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                    "], {
                    {GrayLevel[0.5], EdgeForm[None], 
                    GraphicsGroupBox[{PolygonBox[{{212, 674, 213}}], 
                    PolygonBox[CompressedData["
1:eJwtl3fYjmUYh6/7sRVlhJRZmSUhykhkE7Iqe5PssmUnIUWyEkmU1TZDqSia
aO+99550/o6fP87jOL/nua/rvu5xve/3lus7osPwLCKOQs4TZCkiB+TCT4JT
8UJwcpjCeBEogOeAwFM6EQvH4PiJd8qRU7kgN54fCuKnJOfOA3nxfJA3PCY3
nif5neYoip8GBfF8qkFzpxO5oBheXDnx4lAJrwwlVDuUxM/QGsKciZeCwngR
KIuXg6LhHCXw05NjNaY0XiZ5rMaUx89STeE5quBVFRN+djZ+DhTDa0JvvA/U
Cj+rgFdMrlVrzq+9SF6bcpyLnwclw1TDz9caVAvUxuuopjAX4RdDWeWGZnhz
zYGXglr4hcmxGlMXr6c1h6mPN4Dyqh0a45dpDWGa4E2Tc2vMJXhD7YH2Emrg
NZPn0pwt8JZQCa8MbfDLoQpeHbri3eACrRU64B215nBMK7x1cqzWXF1jk+eq
Cu3xK5Jjtad98X5aY3iOtni75LHVoAt+pfYwPGd3vAfUCNMT75V8VhpzFX51
cq2qqRPeOTmX1nwp3ih5rzRnf3wA1FYsbMQ3KYfOBobg1+qMwgzFh0Fd5YKJ
+CTlxOvAYPya5FiNGY6P0JmFGYmPgvo6CxiLj9OZhBmPT0jOrTGj8et0xuE5
JuM3QOPws+vxMcm59GwKPlV3AG8Nt+ELoY3uAszAZ+pOhJmF3wjNwszGb9Id
DDMHvxla4C1hPn4LtMLbwTJ8ObQPz7EIv113Jsxi/A5oG45ZgN+aXJtqnIZP
T65NY5bgS5NzK+cK/E7dmfDn3N18SK2BV/GO+pt3a6BTmHvwtTpzvBs8hD8M
3XVXYRW+OjlWY+7F10GXMOvx+3TnwndgM74FuoZ5AH8wObfG3I9vSL47muMR
/FHd0TCP4VuhZ5ht+HbdWXwgHMCfhUF4b9iFPw59wuzG90BfvB88ie+D/uEc
O/CdybG6wwOVK/kuKmYv/kRyrPZwJX5X8l5ozufwgzA4zCH8ed1hfAA8g+9P
rlVzPoU/nfxOY17AX4QhYV7CX1bPhHkFPwxD1QvwOv6GeiLMm/hbMFK9AJ/j
X6gn8GHwKv5acqzGvI2/ox4K8y7+HoxWL8DH+CfqiTCf4p8l59aY9/EP1FPh
Ob7Ev4Lx4Wcf4h8l59Kzr/Fv1JP4dPgX/w9m4BPhe/wHmBTmR/wnmBzmZ/wX
9WyYX/HfYAo+Ff7E/4Jpups6Q+52HVgdnuMY746rZ8PoSzrBrHDM37z7J7k2
1fgt/l1ybRqTMTYH3BgmJ54LZofJjeeBm/A5kB8/CW4OczJeAOaGKYifAvPC
nIoXgvlhCuNF4JYwRfHTYEGYYnhxuDVMCfx0uC1MSfwMWBjmTLwULArXmBfP
l7lWPSuNl4Hb8cVQHj8L7sCXQAW8IiwNUwmvDMvCMWXxcpljFXM2fk7mWJ3R
7+zlH8lnpZgqvKsKy/EVUA0/H+4MUx2/AFaGqYHXhLvC1MIvzHzWynEufl7m
XOqRI8x1NPnu6w5cxLuL4W58DdTHG8A9+Fq4FG8E94ZpjF8G6/BN0B6/AjaH
Yy7BG2aO1ZgmeFNYH6YZ3hzuwzdAG/xy2BimLd4uc26NaYG3hPvDc3TAO8KW
MJ3wzvBAmC74lfAgvg0G4oNgO/4QdMW7wcNhuuM94BH8UeiN94HHwjmuwq/O
HKsaWuGtM9eumJ54r8yx2sO6eL3Me6k5B+PXwI4wQ/BrYSe+FfrjAzLXqjn7
4v0yv9N34lzOal7yd6NihvJuGOzCH4eR+CjYje+B6/ExsDfMWHwcPBFmPD4B
ngwzEZ8E+8JMxm+Ap8JMwafC02Gm4dPhmTAz8JmwP8wsfRbAgTCz1fvwbLjG
0fh1mWvVszn6LIDn8IMwX70Nh/Dn1bPqVXghzCL1IrwYjpmLz8scq5gF6v3M
sdoj/YAZkXmvFLNYvQsv4S+r59Rr8EqYFeo1OBxmpXoLjoRZha/O/L+CcixR
72fOxTHFv/Bf+LfO/83MatU=
                    "]]}]}, {{}, {}, {
                    {Hue[0.1880703612269147, 1, 1], LineBox[{1, 2}]}, 
                    {Hue[0.07884389201651706, 1, 1], LineBox[{2, 3}]}, 
                    {Hue[0.997136729011347, 1, 1], LineBox[{3, 4}]}, 
                    {Hue[0.8689395434525204, 1, 1], LineBox[{4, 5}]}, 
                    {Hue[0.7062546158712821, 1, 1], LineBox[{5, 6}]}, 
                    {Hue[0.7388461845627924, 1, 1], LineBox[{6, 7}]}, 
                    {Hue[0.5855521737701119, 1, 1], LineBox[{7, 8}]}, 
                    {Hue[0.5339300205386036, 1, 1], LineBox[{8, 9}]}, 
                    {Hue[0.46682903323219344`, 1, 1], LineBox[{9, 10}]}, 
                    {Hue[0.3723331935375797, 1, 1], LineBox[{10, 11}]}, 
                    {Hue[0.21409818648276444`, 1, 1], LineBox[{11, 12}]}, 
                    {Hue[0.12489905314846222`, 1, 1], LineBox[{12, 13}]}, 
                    {Hue[0.03131991234497626, 1, 1], LineBox[{13, 14}]}, 
                    {Hue[0.0021162108163126875`, 1, 1], LineBox[{14, 15}]}, 
                    {Hue[0.9204190719936995, 1, 1], LineBox[{15, 16}]}, 
                    {Hue[0.8532338736598175, 1, 1], LineBox[{16, 17}]}, 
                    {Hue[0.5961847194378479, 1, 1], LineBox[{17, 18}]}, 
                    {Hue[0.46862351167277533`, 1, 1], LineBox[{18, 19}]}, 
                    {Hue[0.4458251915675756, 1, 1], LineBox[{19, 20}]}, 
                    {Hue[0.46895428974704645`, 1, 1], LineBox[{20, 21}]}, 
                    {Hue[0.5125418994642816, 1, 1], LineBox[{21, 22}]}, 
                    {Hue[0.5989540200443739, 1, 1], LineBox[{22, 23}]}, 
                    {Hue[0.7063579466621666, 1, 1], LineBox[{23, 24}]}, 
                    {Hue[0.7990936586658762, 1, 1], LineBox[{24, 25}]}, 
                    {Hue[0.8690664659236853, 1, 1], LineBox[{25, 26}]}, 
                    {Hue[0.9327432316916345, 1, 1], LineBox[{26, 27}]}, 
                    {Hue[0.004137892867792425, 1, 1], LineBox[{27, 28}]}, 
                    {Hue[0.08342054374255543, 1, 1], LineBox[{28, 29}]}, 
                    {Hue[0.16503786284242647`, 1, 1], LineBox[{29, 30}]}, 
                    {Hue[0.2442067612292575, 1, 1], LineBox[{30, 31}]}, 
                    {Hue[0.32216285879652434`, 1, 1], LineBox[{31, 32}]}, 
                    {Hue[0.39923262219595296`, 1, 1], LineBox[{32, 33}]}, 
                    {Hue[0.4780029083854207, 1, 1], LineBox[{33, 34}]}, 
                    {Hue[0.5582453418924969, 1, 1], LineBox[{34, 35}]}, 
                    {Hue[0.6387858577875118, 1, 1], LineBox[{35, 36}]}, 
                    {Hue[0.7192316129758667, 1, 1], LineBox[{36, 37}]}, 
                    {Hue[0.7996332928670785, 1, 1], LineBox[{37, 38}]}, 
                    {Hue[0.8808137668476514, 1, 1], LineBox[{38, 39}]}, 
                    {Hue[0.9626442596312538, 1, 1], LineBox[{39, 40}]}, 
                    {Hue[0.044988617541437445`, 1, 1], LineBox[{40, 41}]}, 
                    {Hue[0.12760340502421383`, 1, 1], LineBox[{41, 42}]}, 
                    {Hue[0.21047848037237515`, 1, 1], LineBox[{42, 43}]}, 
                    {Hue[0.29376911801795574`, 1, 1], LineBox[{43, 44}]}, 
                    {Hue[0.3775536220592098, 1, 1], LineBox[{44, 45}]}, 
                    {Hue[0.4618126702069244, 1, 1], LineBox[{45, 46}]}, 
                    {Hue[0.5464592745186805, 1, 1], LineBox[{46, 47}]}, 
                    {Hue[0.6315026619907433, 1, 1], LineBox[{47, 48}]}, 
                    {Hue[0.7169267598757019, 1, 1], LineBox[{48, 49}]}, 
                    {Hue[0.8027916833342827, 1, 1], LineBox[{49, 50}]}, 
                    {Hue[0.889089585445414, 1, 1], LineBox[{50, 51}]}, 
                    {Hue[0.9757950557057082, 1, 1], LineBox[{51, 52}]}, 
                    {Hue[0.06291248955601675, 1, 1], LineBox[{52, 53}]}, 
                    {Hue[0.15043625406158562`, 1, 1], LineBox[{53, 54}]}, 
                    {Hue[0.23836997371235044`, 1, 1], LineBox[{54, 55}]}, 
                    {Hue[0.32672315535883, 1, 1], LineBox[{55, 56}]}, 
                    {Hue[0.4154914518436902, 1, 1], LineBox[{56, 57}]}, 
                    {Hue[0.5046658300732096, 1, 1], LineBox[{57, 58}]}, 
                    {Hue[0.5942503894406991, 1, 1], LineBox[{58, 59}]}, 
                    {Hue[0.6842458102565003, 1, 1], LineBox[{59, 60}]}, 
                    {Hue[0.7746521596919396, 1, 1], LineBox[{60, 61}]}, 
                    {Hue[0.8654707714978663, 1, 1], LineBox[{61, 62}]}, 
                    {Hue[0.9566988597906313, 1, 1], LineBox[{62, 63}]}, 
                    {Hue[0.04833514536830779, 1, 1], LineBox[{63, 64}]}, 
                    {Hue[0.14037917323177285`, 1, 1], LineBox[{64, 65}]}, 
                    {Hue[0.23283177667960436`, 1, 1], LineBox[{65, 66}]}, 
                    {Hue[0.32569164258084476`, 1, 1], LineBox[{66, 67}]}, 
                    {Hue[0.41895898555335936`, 1, 1], LineBox[{67, 68}]}, 
                    {Hue[0.5126322184725833, 1, 1], LineBox[{68, 69}]}, 
                    {Hue[0.6067109627284879, 1, 1], LineBox[{69, 70}]}, 
                    {Hue[0.7011944510053674, 1, 1], LineBox[{70, 71}]}, 
                    {Hue[0.7960825734629512, 1, 1], LineBox[{71, 72}]}, 
                    {Hue[0.8913750773674782, 1, 1], LineBox[{72, 73}]}, 
                    {Hue[0.9870703085292256, 1, 1], LineBox[{73, 74}]}, 
                    {Hue[0.0831682078717523, 1, 1], LineBox[{74, 75}]}, 
                    {Hue[0.17966690754711842`, 1, 1], LineBox[{75, 76}]}, 
                    {Hue[0.27656651526156967`, 1, 1], LineBox[{76, 77}]}, 
                    {Hue[0.37386606936161204`, 1, 1], LineBox[{77, 78}]}, 
                    {Hue[0.47156409420583095`, 1, 1], LineBox[{78, 79}]}, 
                    {Hue[0.5696598302845104, 1, 1], LineBox[{79, 80}]}, 
                    {Hue[0.6681520841585072, 1, 1], LineBox[{80, 81}]}, 
                    {Hue[0.7670396100066637, 1, 1], LineBox[{81, 82}]}, 
                    {Hue[0.8663210495276291, 1, 1], LineBox[{82, 83}]}, 
                    {Hue[0.965994561204372, 1, 1], LineBox[{83, 84}]}, 
                    {Hue[0.06605854132798218, 1, 1], LineBox[{84, 85}]}, 
                    {Hue[0.1665109206955648, 1, 1], LineBox[{85, 86}]}, 
                    {Hue[0.26735002732299895`, 1, 1], LineBox[{86, 87}]}, 
                    {Hue[0.3685727399772961, 1, 1], LineBox[{87, 88}]}, 
                    {Hue[0.4701766148566998, 1, 1], LineBox[{88, 89}]}, 
                    {Hue[0.5721579648495672, 1, 1], LineBox[{89, 90}]}, 
                    {Hue[0.6745138217897079, 1, 1], LineBox[{90, 91}]}, 
                    {Hue[0.7772401123597215, 1, 1], LineBox[{91, 92}]}, 
                    {Hue[0.8803315074975259, 1, 1], LineBox[{92, 93}]}, 
                    {Hue[0.9837828931126193, 1, 1], LineBox[{93, 94}]}, 
                    {Hue[0.08758813029091003, 1, 1], LineBox[{94, 95}]}, 
                    {Hue[0.19173995962207716`, 1, 1], LineBox[{95, 96}]}, 
                    {Hue[0.29623052831179936`, 1, 1], LineBox[{96, 97}]}, 
                    {Hue[0.40104982353295887`, 1, 1], LineBox[{97, 98}]}, 
                    {Hue[0.5061869922694149, 1, 1], LineBox[{98, 99}]}, 
                    {Hue[0.6116299342344147, 1, 1], LineBox[{99, 100}]}, 
                    {Hue[0.7173639634709316, 1, 1], LineBox[{100, 101}]}, 
                    {Hue[0.8233718474500051, 1, 1], LineBox[{101, 102}]}, 
                    {Hue[0.9296346319294032, 1, 1], LineBox[{102, 103}]}, 
                    {Hue[0.03613037661478721, 1, 1], LineBox[{103, 104}]}, 
                    {Hue[0.1428334271268505, 1, 1], LineBox[{104, 105}]}, 
                    {Hue[0.24971412507695165`, 1, 1], LineBox[{105, 106}]}, 
                    {Hue[0.356738931200211, 1, 1], LineBox[{106, 107}]}, 
                    {Hue[0.4638696268399593, 1, 1], LineBox[{107, 108}]}, 
                    {Hue[0.571063319566488, 1, 1], LineBox[{108, 109}]}, 
                    {Hue[0.6782700343654368, 1, 1], LineBox[{109, 110}]}, 
                    {Hue[0.785434006690692, 1, 1], LineBox[{110, 111}]}, 
                    {Hue[0.8924930729167271, 1, 1], LineBox[{111, 112}]}, 
                    {Hue[0.9993779783296385, 1, 1], LineBox[{112, 113}]}, 
                    {Hue[0.10601149166352605`, 1, 1], LineBox[{113, 114}]}, 
                    {Hue[0.2123087313751443, 1, 1], LineBox[{114, 115}]}, 
                    {Hue[0.3181787875662695, 1, 1], LineBox[{115, 116}]}, 
                    {Hue[0.4235247873915342, 1, 1], LineBox[{116, 117}]}, 
                    {Hue[0.5282462551631992, 1, 1], LineBox[{117, 118}]}, 
                    {Hue[0.6322395345155183, 1, 1], LineBox[{118, 119}]}, 
                    {Hue[0.7354006500082138, 1, 1], LineBox[{119, 120}]}, 
                    {Hue[0.8376300003692168, 1, 1], LineBox[{120, 121}]}, 
                    {Hue[0.9388378605035128, 1, 1], LineBox[{121, 122}]}, 
                    {Hue[0.038949607055956306`, 1, 1], LineBox[{122, 123}]}, 
                    {Hue[0.13790895816925902`, 1, 1], LineBox[{123, 124}]}, 
                    {Hue[0.23568267151472239`, 1, 1], LineBox[{124, 125}]}, 
                    {Hue[0.3322654544287647, 1, 1], LineBox[{125, 126}]}, 
                    {Hue[0.42768181401220956`, 1, 1], LineBox[{126, 127}]}, 
                    {Hue[0.5219917613987968, 1, 1], LineBox[{127, 128}]}, 
                    {Hue[0.6152841180730468, 1, 1], LineBox[{128, 129}]}, 
                    {Hue[0.7076595650334099, 1, 1], LineBox[{129, 130}]}, 
                    {Hue[0.7992366467292306, 1, 1], LineBox[{130, 131}]}, 
                    {Hue[0.8901637845684595, 1, 1], LineBox[{131, 132}]}, 
                    {Hue[0.980586996534017, 1, 1], LineBox[{132, 133}]}, 
                    {Hue[0.0706304063174194, 1, 1], LineBox[{133, 134}]}, 
                    {Hue[0.16044150686162284`, 1, 1], LineBox[{134, 135}]}, 
                    {Hue[0.25018705154374354`, 1, 1], LineBox[{135, 136}]}, 
                    {Hue[0.33995086535139046`, 1, 1], LineBox[{136, 137}]}, 
                    {Hue[0.4297694192998456, 1, 1], LineBox[{137, 138}]}, 
                    {Hue[0.5198289187276111, 1, 1], LineBox[{138, 139}]}, 
                    {Hue[0.610354057984487, 1, 1], LineBox[{139, 140}]}, 
                    {Hue[0.7012144543506329, 1, 1], LineBox[{140, 141}]}, 
                    {Hue[0.7921081889537456, 1, 1], LineBox[{141, 142}]}, 
                    {Hue[0.8833380798161025, 1, 1], LineBox[{142, 143}]}, 
                    {Hue[0.975815306966425, 1, 1], LineBox[{143, 144}]}, 
                    {Hue[0.0698586820590084, 1, 1], LineBox[{144, 145}]}, 
                    {Hue[0.16445521240978872`, 1, 1], LineBox[{145, 146}]}, 
                    {Hue[0.2582012082242161, 1, 1], LineBox[{146, 147}]}, 
                    {Hue[0.35121116346775805`, 1, 1], LineBox[{147, 148}]}, 
                    {Hue[0.44563606809176354`, 1, 1], LineBox[{148, 149}]}, 
                    {Hue[0.5430933601796459, 1, 1], LineBox[{149, 150}]}, 
                    {Hue[0.6423155915986974, 1, 1], LineBox[{150, 151}]}, 
                    {Hue[0.7407113090471817, 1, 1], LineBox[{151, 152}]}, 
                    {Hue[0.8376881189539714, 1, 1], LineBox[{152, 153}]}, 
                    {Hue[0.9352232891503853, 1, 1], LineBox[{153, 154}]}, 
                    {Hue[0.03614858908370498, 1, 1], LineBox[{154, 155}]}, 
                    {Hue[0.14398074114436193`, 1, 1], LineBox[{155, 156}]}, 
                    {Hue[0.25920378352449885`, 1, 1], LineBox[{156, 157}]}, 
                    {Hue[0.3668844355750923, 1, 1], LineBox[{157, 158}]}, 
                    {Hue[0.41640280589271683`, 1, 1], LineBox[{158, 159}]}, 
                    {Hue[0.46331902891034815`, 1, 1], LineBox[{159, 160}]}, 
                    {Hue[0.6033248517219278, 1, 1], LineBox[{160, 161}]}, 
                    {Hue[0.7883325306388308, 1, 1], LineBox[{161, 162}]}, 
                    {Hue[0.0056238243984809925`, 1, 1], 
                    LineBox[{162, 163}]}, 
                    {Hue[0.2774077069850332, 1, 1], LineBox[{163, 164}]}, 
                    {Hue[0.6155398081712723, 1, 1], LineBox[{164, 165}]}, 
                    {Hue[0.9030578930956729, 1, 1], LineBox[{165, 166}]}, 
                    {Hue[0.14414120650805495`, 1, 1], LineBox[{166, 167}]}, 
                    {Hue[0.3701881994243456, 1, 1], LineBox[{167, 168}]}, 
                    {Hue[0.5916882192261339, 1, 1], LineBox[{168, 169}]}, 
                    {Hue[0.8128425253061538, 1, 1], LineBox[{169, 170}]}, 
                    {Hue[0.03606014119192609, 1, 1], LineBox[{170, 171}]}, 
                    {Hue[0.26223236189121907`, 1, 1], LineBox[{171, 172}]}, 
                    {Hue[0.49054148919426266`, 1, 1], LineBox[{172, 173}]}, 
                    {Hue[0.7203779998491203, 1, 1], LineBox[{173, 174}]}, 
                    {Hue[0.9525369026809333, 1, 1], LineBox[{174, 175}]}, 
                    {Hue[0.18843993116706134`, 1, 1], LineBox[{175, 176}]}, 
                    {Hue[0.42932440021741863`, 1, 1], LineBox[{176, 177}]}, 
                    {Hue[0.6757389699208721, 1, 1], LineBox[{177, 178}]}, 
                    {Hue[0.9271433109474645, 1, 1], LineBox[{178, 179}]}, 
                    {Hue[0.181969958793826, 1, 1], LineBox[{179, 180}]}, 
                    {Hue[0.438446713394616, 1, 1], LineBox[{180, 181}]}, 
                    {Hue[0.6955392300974138, 1, 1], LineBox[{181, 182}]}, 
                    {Hue[0.9530958902650895, 1, 1], LineBox[{182, 183}]}, 
                    {Hue[0.21123375064817898`, 1, 1], LineBox[{183, 184}]}, 
                    {Hue[0.46967263406929427`, 1, 1], LineBox[{184, 185}]}, 
                    {Hue[0.7276023629921933, 1, 1], LineBox[{185, 186}]}, 
                    {Hue[0.984161950447084, 1, 1], LineBox[{186, 187}]}, 
                    {Hue[0.2390623682607009, 1, 1], LineBox[{187, 188}]}, 
                    {Hue[0.49283556399150985`, 1, 1], LineBox[{188, 189}]}, 
                    {Hue[0.7466324493665115, 1, 1], LineBox[{189, 190}]}, 
                    {Hue[0.0017163865090559496`, 1, 1], 
                    LineBox[{190, 191}]}, 
                    {Hue[0.2586526882451303, 1, 1], LineBox[{191, 192}]}, 
                    {Hue[0.5162546501565902, 1, 1], LineBox[{192, 193}]}, 
                    {Hue[0.7710500352229288, 1, 1], LineBox[{193, 194}]}, 
                    {Hue[0.018209373829782306`, 1, 1], LineBox[{194, 195}]}, 
                    {Hue[0.25272791954391893`, 1, 1], LineBox[{195, 196}]}, 
                    {Hue[0.46754293320048695`, 1, 1], LineBox[{196, 197}]}, 
                    {Hue[0.6414161184083459, 1, 1], LineBox[{197, 198}]}, 
                    {Hue[0.7116095092051098, 1, 1], LineBox[{198, 199}]}, 
                    {Hue[0.7941755173099261, 1, 1], LineBox[{199, 200}]}, 
                    {Hue[0.959677603338433, 1, 1], LineBox[{200, 201}]}, 
                    {Hue[0.15416041543491266`, 1, 1], LineBox[{201, 202}]}, 
                    {Hue[0.3659312105320584, 1, 1], LineBox[{202, 203}]}, 
                    {Hue[0.5945187685231991, 1, 1], LineBox[{203, 204}]}, 
                    {Hue[0.8435841217998448, 1, 1], LineBox[{204, 205}]}, 
                    {Hue[0.11798106092260775`, 1, 1], LineBox[{205, 206}]}, 
                    {Hue[0.40976957642296785`, 1, 1], LineBox[{206, 207}]}, 
                    {Hue[0.6931950268848085, 1, 1], LineBox[{207, 208}]}, 
                    {Hue[0.9572768482725862, 1, 1], LineBox[{208, 209}]}, 
                    {Hue[0.20736297205040422`, 1, 1], LineBox[{209, 210}]}, 
                    {Hue[0.4495073129009342, 1, 1], LineBox[{210, 211}]}, 
                    {Hue[0.6873302371223096, 1, 1], LineBox[{211, 212}]}, 
                    {Hue[0.922917256847574, 1, 1], LineBox[{212, 213}]}, 
                    {Hue[0.15766446546505655`, 1, 1], LineBox[{213, 214}]}, 
                    {Hue[0.39281267806319864`, 1, 1], LineBox[{214, 215}]}, 
                    {Hue[0.6298954761469904, 1, 1], LineBox[{215, 216}]}, 
                    {Hue[0.8713537747234714, 1, 1], LineBox[{216, 217}]}, 
                    {Hue[0.12165943440260661`, 1, 1], LineBox[{217, 218}]}, 
                    {Hue[0.3889776048863425, 1, 1], LineBox[{218, 219}]}, 
                    {Hue[0.6821005554196526, 1, 1], LineBox[{219, 220}]}, 
                    {Hue[0.9874793453943027, 1, 1], LineBox[{220, 221}]}, 
                    {Hue[0.27333428148134026`, 1, 1], LineBox[{221, 222}]}, 
                    {Hue[0.536210098334362, 1, 1], LineBox[{222, 223}]}, 
                    {Hue[0.7860931382583406, 1, 1], LineBox[{223, 224}]}, 
                    {Hue[0.029915025712982726`, 1, 1], LineBox[{224, 225}]}, 
                    {Hue[0.27135597860730676`, 1, 1], LineBox[{225, 226}]}, 
                    {Hue[0.5124025961869167, 1, 1], LineBox[{226, 227}]}, 
                    {Hue[0.7542039770961328, 1, 1], LineBox[{227, 228}]}, 
                    {Hue[0.9974861242682816, 1, 1], LineBox[{228, 229}]}, 
                    {Hue[0.2427641987352073, 1, 1], LineBox[{229, 230}]}, 
                    {Hue[0.49046522259802094`, 1, 1], LineBox[{230, 231}]}, 
                    {Hue[0.7410080688609925, 1, 1], LineBox[{231, 232}]}, 
                    {Hue[0.9948612689328367, 1, 1], LineBox[{232, 233}]}, 
                    {Hue[0.25258201027232025`, 1, 1], LineBox[{233, 234}]}, 
                    {Hue[0.5148223127241571, 1, 1], LineBox[{234, 235}]}, 
                    {Hue[0.7822622604242208, 1, 1], LineBox[{235, 236}]}, 
                    {Hue[0.05540122424524702, 1, 1], LineBox[{236, 237}]}, 
                    {Hue[0.33416566373549533`, 1, 1], LineBox[{237, 238}]}, 
                    {Hue[0.6174964318648639, 1, 1], LineBox[{238, 239}]}, 
                    {Hue[0.9033933394447586, 1, 1], LineBox[{239, 240}]}, 
                    {Hue[0.18972674509790277`, 1, 1], LineBox[{240, 241}]}, 
                    {Hue[0.4752451532292422, 1, 1], LineBox[{241, 242}]}, 
                    {Hue[0.7599383987575214, 1, 1], LineBox[{242, 243}]}, 
                    {Hue[0.044725871382062, 1, 1], LineBox[{243, 244}]}, 
                    {Hue[0.3309301529199232, 1, 1], LineBox[{244, 245}]}, 
                    {Hue[0.619740435821263, 1, 1], LineBox[{245, 246}]}, 
                    {Hue[0.9115976593850698, 1, 1], LineBox[{246, 247}]}, 
                    {Hue[0.2055958773373284, 1, 1], LineBox[{247, 248}]}, 
                    {Hue[0.49947597567191315`, 1, 1], LineBox[{248, 249}]}, 
                    {Hue[0.79064522017179, 1, 1], LineBox[{249, 250}]}, 
                    {Hue[0.0774344648203574, 1, 1], LineBox[{250, 251}]}}}},
                    VertexColors->{
                    Hue[0.1880703612269147, 1, 1], 
                    Hue[0.07884389201651706, 1, 1], 
                    Hue[0.997136729011347, 1, 1], 
                    Hue[0.8689395434525204, 1, 1], 
                    Hue[0.7062546158712821, 1, 1], 
                    Hue[0.7388461845627924, 1, 1], 
                    Hue[0.5855521737701119, 1, 1], 
                    Hue[0.5339300205386036, 1, 1], 
                    Hue[0.46682903323219344`, 1, 1], 
                    Hue[0.3723331935375797, 1, 1], 
                    Hue[0.21409818648276444`, 1, 1], 
                    Hue[0.12489905314846222`, 1, 1], 
                    Hue[0.03131991234497626, 1, 1], 
                    Hue[0.0021162108163126875`, 1, 1], 
                    Hue[0.9204190719936995, 1, 1], 
                    Hue[0.8532338736598175, 1, 1], 
                    Hue[0.5961847194378479, 1, 1], 
                    Hue[0.46862351167277533`, 1, 1], 
                    Hue[0.4458251915675756, 1, 1], 
                    Hue[0.46895428974704645`, 1, 1], 
                    Hue[0.5125418994642816, 1, 1], 
                    Hue[0.5989540200443739, 1, 1], 
                    Hue[0.7063579466621666, 1, 1], 
                    Hue[0.7990936586658762, 1, 1], 
                    Hue[0.8690664659236853, 1, 1], 
                    Hue[0.9327432316916345, 1, 1], 
                    Hue[0.004137892867792425, 1, 1], 
                    Hue[0.08342054374255543, 1, 1], 
                    Hue[0.16503786284242647`, 1, 1], 
                    Hue[0.2442067612292575, 1, 1], 
                    Hue[0.32216285879652434`, 1, 1], 
                    Hue[0.39923262219595296`, 1, 1], 
                    Hue[0.4780029083854207, 1, 1], 
                    Hue[0.5582453418924969, 1, 1], 
                    Hue[0.6387858577875118, 1, 1], 
                    Hue[0.7192316129758667, 1, 1], 
                    Hue[0.7996332928670785, 1, 1], 
                    Hue[0.8808137668476514, 1, 1], 
                    Hue[0.9626442596312538, 1, 1], 
                    Hue[0.044988617541437445`, 1, 1], 
                    Hue[0.12760340502421383`, 1, 1], 
                    Hue[0.21047848037237515`, 1, 1], 
                    Hue[0.29376911801795574`, 1, 1], 
                    Hue[0.3775536220592098, 1, 1], 
                    Hue[0.4618126702069244, 1, 1], 
                    Hue[0.5464592745186805, 1, 1], 
                    Hue[0.6315026619907433, 1, 1], 
                    Hue[0.7169267598757019, 1, 1], 
                    Hue[0.8027916833342827, 1, 1], 
                    Hue[0.889089585445414, 1, 1], 
                    Hue[0.9757950557057082, 1, 1], 
                    Hue[0.06291248955601675, 1, 1], 
                    Hue[0.15043625406158562`, 1, 1], 
                    Hue[0.23836997371235044`, 1, 1], 
                    Hue[0.32672315535883, 1, 1], 
                    Hue[0.4154914518436902, 1, 1], 
                    Hue[0.5046658300732096, 1, 1], 
                    Hue[0.5942503894406991, 1, 1], 
                    Hue[0.6842458102565003, 1, 1], 
                    Hue[0.7746521596919396, 1, 1], 
                    Hue[0.8654707714978663, 1, 1], 
                    Hue[0.9566988597906313, 1, 1], 
                    Hue[0.04833514536830779, 1, 1], 
                    Hue[0.14037917323177285`, 1, 1], 
                    Hue[0.23283177667960436`, 1, 1], 
                    Hue[0.32569164258084476`, 1, 1], 
                    Hue[0.41895898555335936`, 1, 1], 
                    Hue[0.5126322184725833, 1, 1], 
                    Hue[0.6067109627284879, 1, 1], 
                    Hue[0.7011944510053674, 1, 1], 
                    Hue[0.7960825734629512, 1, 1], 
                    Hue[0.8913750773674782, 1, 1], 
                    Hue[0.9870703085292256, 1, 1], 
                    Hue[0.0831682078717523, 1, 1], 
                    Hue[0.17966690754711842`, 1, 1], 
                    Hue[0.27656651526156967`, 1, 1], 
                    Hue[0.37386606936161204`, 1, 1], 
                    Hue[0.47156409420583095`, 1, 1], 
                    Hue[0.5696598302845104, 1, 1], 
                    Hue[0.6681520841585072, 1, 1], 
                    Hue[0.7670396100066637, 1, 1], 
                    Hue[0.8663210495276291, 1, 1], 
                    Hue[0.965994561204372, 1, 1], 
                    Hue[0.06605854132798218, 1, 1], 
                    Hue[0.1665109206955648, 1, 1], 
                    Hue[0.26735002732299895`, 1, 1], 
                    Hue[0.3685727399772961, 1, 1], 
                    Hue[0.4701766148566998, 1, 1], 
                    Hue[0.5721579648495672, 1, 1], 
                    Hue[0.6745138217897079, 1, 1], 
                    Hue[0.7772401123597215, 1, 1], 
                    Hue[0.8803315074975259, 1, 1], 
                    Hue[0.9837828931126193, 1, 1], 
                    Hue[0.08758813029091003, 1, 1], 
                    Hue[0.19173995962207716`, 1, 1], 
                    Hue[0.29623052831179936`, 1, 1], 
                    Hue[0.40104982353295887`, 1, 1], 
                    Hue[0.5061869922694149, 1, 1], 
                    Hue[0.6116299342344147, 1, 1], 
                    Hue[0.7173639634709316, 1, 1], 
                    Hue[0.8233718474500051, 1, 1], 
                    Hue[0.9296346319294032, 1, 1], 
                    Hue[0.03613037661478721, 1, 1], 
                    Hue[0.1428334271268505, 1, 1], 
                    Hue[0.24971412507695165`, 1, 1], 
                    Hue[0.356738931200211, 1, 1], 
                    Hue[0.4638696268399593, 1, 1], 
                    Hue[0.571063319566488, 1, 1], 
                    Hue[0.6782700343654368, 1, 1], 
                    Hue[0.785434006690692, 1, 1], 
                    Hue[0.8924930729167271, 1, 1], 
                    Hue[0.9993779783296385, 1, 1], 
                    Hue[0.10601149166352605`, 1, 1], 
                    Hue[0.2123087313751443, 1, 1], 
                    Hue[0.3181787875662695, 1, 1], 
                    Hue[0.4235247873915342, 1, 1], 
                    Hue[0.5282462551631992, 1, 1], 
                    Hue[0.6322395345155183, 1, 1], 
                    Hue[0.7354006500082138, 1, 1], 
                    Hue[0.8376300003692168, 1, 1], 
                    Hue[0.9388378605035128, 1, 1], 
                    Hue[0.038949607055956306`, 1, 1], 
                    Hue[0.13790895816925902`, 1, 1], 
                    Hue[0.23568267151472239`, 1, 1], 
                    Hue[0.3322654544287647, 1, 1], 
                    Hue[0.42768181401220956`, 1, 1], 
                    Hue[0.5219917613987968, 1, 1], 
                    Hue[0.6152841180730468, 1, 1], 
                    Hue[0.7076595650334099, 1, 1], 
                    Hue[0.7992366467292306, 1, 1], 
                    Hue[0.8901637845684595, 1, 1], 
                    Hue[0.980586996534017, 1, 1], 
                    Hue[0.0706304063174194, 1, 1], 
                    Hue[0.16044150686162284`, 1, 1], 
                    Hue[0.25018705154374354`, 1, 1], 
                    Hue[0.33995086535139046`, 1, 1], 
                    Hue[0.4297694192998456, 1, 1], 
                    Hue[0.5198289187276111, 1, 1], 
                    Hue[0.610354057984487, 1, 1], 
                    Hue[0.7012144543506329, 1, 1], 
                    Hue[0.7921081889537456, 1, 1], 
                    Hue[0.8833380798161025, 1, 1], 
                    Hue[0.975815306966425, 1, 1], 
                    Hue[0.0698586820590084, 1, 1], 
                    Hue[0.16445521240978872`, 1, 1], 
                    Hue[0.2582012082242161, 1, 1], 
                    Hue[0.35121116346775805`, 1, 1], 
                    Hue[0.44563606809176354`, 1, 1], 
                    Hue[0.5430933601796459, 1, 1], 
                    Hue[0.6423155915986974, 1, 1], 
                    Hue[0.7407113090471817, 1, 1], 
                    Hue[0.8376881189539714, 1, 1], 
                    Hue[0.9352232891503853, 1, 1], 
                    Hue[0.03614858908370498, 1, 1], 
                    Hue[0.14398074114436193`, 1, 1], 
                    Hue[0.25920378352449885`, 1, 1], 
                    Hue[0.3668844355750923, 1, 1], 
                    Hue[0.41640280589271683`, 1, 1], 
                    Hue[0.46331902891034815`, 1, 1], 
                    Hue[0.6033248517219278, 1, 1], 
                    Hue[0.7883325306388308, 1, 1], 
                    Hue[0.0056238243984809925`, 1, 1], 
                    Hue[0.2774077069850332, 1, 1], 
                    Hue[0.6155398081712723, 1, 1], 
                    Hue[0.9030578930956729, 1, 1], 
                    Hue[0.14414120650805495`, 1, 1], 
                    Hue[0.3701881994243456, 1, 1], 
                    Hue[0.5916882192261339, 1, 1], 
                    Hue[0.8128425253061538, 1, 1], 
                    Hue[0.03606014119192609, 1, 1], 
                    Hue[0.26223236189121907`, 1, 1], 
                    Hue[0.49054148919426266`, 1, 1], 
                    Hue[0.7203779998491203, 1, 1], 
                    Hue[0.9525369026809333, 1, 1], 
                    Hue[0.18843993116706134`, 1, 1], 
                    Hue[0.42932440021741863`, 1, 1], 
                    Hue[0.6757389699208721, 1, 1], 
                    Hue[0.9271433109474645, 1, 1], 
                    Hue[0.181969958793826, 1, 1], 
                    Hue[0.438446713394616, 1, 1], 
                    Hue[0.6955392300974138, 1, 1], 
                    Hue[0.9530958902650895, 1, 1], 
                    Hue[0.21123375064817898`, 1, 1], 
                    Hue[0.46967263406929427`, 1, 1], 
                    Hue[0.7276023629921933, 1, 1], 
                    Hue[0.984161950447084, 1, 1], 
                    Hue[0.2390623682607009, 1, 1], 
                    Hue[0.49283556399150985`, 1, 1], 
                    Hue[0.7466324493665115, 1, 1], 
                    Hue[0.0017163865090559496`, 1, 1], 
                    Hue[0.2586526882451303, 1, 1], 
                    Hue[0.5162546501565902, 1, 1], 
                    Hue[0.7710500352229288, 1, 1], 
                    Hue[0.018209373829782306`, 1, 1], 
                    Hue[0.25272791954391893`, 1, 1], 
                    Hue[0.46754293320048695`, 1, 1], 
                    Hue[0.6414161184083459, 1, 1], 
                    Hue[0.7116095092051098, 1, 1], 
                    Hue[0.7941755173099261, 1, 1], 
                    Hue[0.959677603338433, 1, 1], 
                    Hue[0.15416041543491266`, 1, 1], 
                    Hue[0.3659312105320584, 1, 1], 
                    Hue[0.5945187685231991, 1, 1], 
                    Hue[0.8435841217998448, 1, 1], 
                    Hue[0.11798106092260775`, 1, 1], 
                    Hue[0.40976957642296785`, 1, 1], 
                    Hue[0.6931950268848085, 1, 1], 
                    Hue[0.9572768482725862, 1, 1], 
                    Hue[0.20736297205040422`, 1, 1], 
                    Hue[0.4495073129009342, 1, 1], 
                    Hue[0.6873302371223096, 1, 1], 
                    Hue[0.922917256847574, 1, 1], 
                    Hue[0.15766446546505655`, 1, 1], 
                    Hue[0.39281267806319864`, 1, 1], 
                    Hue[0.6298954761469904, 1, 1], 
                    Hue[0.8713537747234714, 1, 1], 
                    Hue[0.12165943440260661`, 1, 1], 
                    Hue[0.3889776048863425, 1, 1], 
                    Hue[0.6821005554196526, 1, 1], 
                    Hue[0.9874793453943027, 1, 1], 
                    Hue[0.27333428148134026`, 1, 1], 
                    Hue[0.536210098334362, 1, 1], 
                    Hue[0.7860931382583406, 1, 1], 
                    Hue[0.029915025712982726`, 1, 1], 
                    Hue[0.27135597860730676`, 1, 1], 
                    Hue[0.5124025961869167, 1, 1], 
                    Hue[0.7542039770961328, 1, 1], 
                    Hue[0.9974861242682816, 1, 1], 
                    Hue[0.2427641987352073, 1, 1], 
                    Hue[0.49046522259802094`, 1, 1], 
                    Hue[0.7410080688609925, 1, 1], 
                    Hue[0.9948612689328367, 1, 1], 
                    Hue[0.25258201027232025`, 1, 1], 
                    Hue[0.5148223127241571, 1, 1], 
                    Hue[0.7822622604242208, 1, 1], 
                    Hue[0.05540122424524702, 1, 1], 
                    Hue[0.33416566373549533`, 1, 1], 
                    Hue[0.6174964318648639, 1, 1], 
                    Hue[0.9033933394447586, 1, 1], 
                    Hue[0.18972674509790277`, 1, 1], 
                    Hue[0.4752451532292422, 1, 1], 
                    Hue[0.7599383987575214, 1, 1], 
                    Hue[0.044725871382062, 1, 1], 
                    Hue[0.3309301529199232, 1, 1], 
                    Hue[0.619740435821263, 1, 1], 
                    Hue[0.9115976593850698, 1, 1], 
                    Hue[0.2055958773373284, 1, 1], 
                    Hue[0.49947597567191315`, 1, 1], 
                    Hue[0.79064522017179, 1, 1], 
                    Hue[0.0774344648203574, 1, 1], 
                    Hue[0.359464892025414, 1, 1], 
                    Hue[0.1880703612269147, 1, 1], 
                    Hue[0.07884389201651706, 1, 1], 
                    Hue[0.07884389201651706, 1, 1], 
                    Hue[0.997136729011347, 1, 1], 
                    Hue[0.997136729011347, 1, 1], 
                    Hue[0.8689395434525204, 1, 1], 
                    Hue[0.8689395434525204, 1, 1], 
                    Hue[0.7062546158712821, 1, 1], 
                    Hue[0.7062546158712821, 1, 1], 
                    Hue[0.7388461845627924, 1, 1], 
                    Hue[0.7388461845627924, 1, 1], 
                    Hue[0.5855521737701119, 1, 1], 
                    Hue[0.5855521737701119, 1, 1], 
                    Hue[0.5339300205386036, 1, 1], 
                    Hue[0.5339300205386036, 1, 1], 
                    Hue[0.46682903323219344`, 1, 1], 
                    Hue[0.46682903323219344`, 1, 1], 
                    Hue[0.3723331935375797, 1, 1], 
                    Hue[0.3723331935375797, 1, 1], 
                    Hue[0.21409818648276444`, 1, 1], 
                    Hue[0.21409818648276444`, 1, 1], 
                    Hue[0.12489905314846222`, 1, 1], 
                    Hue[0.12489905314846222`, 1, 1], 
                    Hue[0.03131991234497626, 1, 1], 
                    Hue[0.03131991234497626, 1, 1], 
                    Hue[0.0021162108163126875`, 1, 1], 
                    Hue[0.0021162108163126875`, 1, 1], 
                    Hue[0.9204190719936995, 1, 1], 
                    Hue[0.9204190719936995, 1, 1], 
                    Hue[0.8532338736598175, 1, 1], 
                    Hue[0.8532338736598175, 1, 1], 
                    Hue[0.5961847194378479, 1, 1], 
                    Hue[0.5961847194378479, 1, 1], 
                    Hue[0.46862351167277533`, 1, 1], 
                    Hue[0.46862351167277533`, 1, 1], 
                    Hue[0.4458251915675756, 1, 1], 
                    Hue[0.4458251915675756, 1, 1], 
                    Hue[0.46895428974704645`, 1, 1], 
                    Hue[0.46895428974704645`, 1, 1], 
                    Hue[0.5125418994642816, 1, 1], 
                    Hue[0.5125418994642816, 1, 1], 
                    Hue[0.5989540200443739, 1, 1], 
                    Hue[0.5989540200443739, 1, 1], 
                    Hue[0.7063579466621666, 1, 1], 
                    Hue[0.7063579466621666, 1, 1], 
                    Hue[0.7990936586658762, 1, 1], 
                    Hue[0.7990936586658762, 1, 1], 
                    Hue[0.8690664659236853, 1, 1], 
                    Hue[0.8690664659236853, 1, 1], 
                    Hue[0.9327432316916345, 1, 1], 
                    Hue[0.9327432316916345, 1, 1], 
                    Hue[0.004137892867792425, 1, 1], 
                    Hue[0.004137892867792425, 1, 1], 
                    Hue[0.08342054374255543, 1, 1], 
                    Hue[0.08342054374255543, 1, 1], 
                    Hue[0.16503786284242647`, 1, 1], 
                    Hue[0.16503786284242647`, 1, 1], 
                    Hue[0.2442067612292575, 1, 1], 
                    Hue[0.2442067612292575, 1, 1], 
                    Hue[0.32216285879652434`, 1, 1], 
                    Hue[0.32216285879652434`, 1, 1], 
                    Hue[0.39923262219595296`, 1, 1], 
                    Hue[0.39923262219595296`, 1, 1], 
                    Hue[0.4780029083854207, 1, 1], 
                    Hue[0.4780029083854207, 1, 1], 
                    Hue[0.5582453418924969, 1, 1], 
                    Hue[0.5582453418924969, 1, 1], 
                    Hue[0.6387858577875118, 1, 1], 
                    Hue[0.6387858577875118, 1, 1], 
                    Hue[0.7192316129758667, 1, 1], 
                    Hue[0.7192316129758667, 1, 1], 
                    Hue[0.7996332928670785, 1, 1], 
                    Hue[0.7996332928670785, 1, 1], 
                    Hue[0.8808137668476514, 1, 1], 
                    Hue[0.8808137668476514, 1, 1], 
                    Hue[0.9626442596312538, 1, 1], 
                    Hue[0.9626442596312538, 1, 1], 
                    Hue[0.044988617541437445`, 1, 1], 
                    Hue[0.044988617541437445`, 1, 1], 
                    Hue[0.12760340502421383`, 1, 1], 
                    Hue[0.12760340502421383`, 1, 1], 
                    Hue[0.21047848037237515`, 1, 1], 
                    Hue[0.21047848037237515`, 1, 1], 
                    Hue[0.29376911801795574`, 1, 1], 
                    Hue[0.29376911801795574`, 1, 1], 
                    Hue[0.3775536220592098, 1, 1], 
                    Hue[0.3775536220592098, 1, 1], 
                    Hue[0.4618126702069244, 1, 1], 
                    Hue[0.4618126702069244, 1, 1], 
                    Hue[0.5464592745186805, 1, 1], 
                    Hue[0.5464592745186805, 1, 1], 
                    Hue[0.6315026619907433, 1, 1], 
                    Hue[0.6315026619907433, 1, 1], 
                    Hue[0.7169267598757019, 1, 1], 
                    Hue[0.7169267598757019, 1, 1], 
                    Hue[0.8027916833342827, 1, 1], 
                    Hue[0.8027916833342827, 1, 1], 
                    Hue[0.889089585445414, 1, 1], 
                    Hue[0.889089585445414, 1, 1], 
                    Hue[0.9757950557057082, 1, 1], 
                    Hue[0.9757950557057082, 1, 1], 
                    Hue[0.06291248955601675, 1, 1], 
                    Hue[0.06291248955601675, 1, 1], 
                    Hue[0.15043625406158562`, 1, 1], 
                    Hue[0.15043625406158562`, 1, 1], 
                    Hue[0.23836997371235044`, 1, 1], 
                    Hue[0.23836997371235044`, 1, 1], 
                    Hue[0.32672315535883, 1, 1], 
                    Hue[0.32672315535883, 1, 1], 
                    Hue[0.4154914518436902, 1, 1], 
                    Hue[0.4154914518436902, 1, 1], 
                    Hue[0.5046658300732096, 1, 1], 
                    Hue[0.5046658300732096, 1, 1], 
                    Hue[0.5942503894406991, 1, 1], 
                    Hue[0.5942503894406991, 1, 1], 
                    Hue[0.6842458102565003, 1, 1], 
                    Hue[0.6842458102565003, 1, 1], 
                    Hue[0.7746521596919396, 1, 1], 
                    Hue[0.7746521596919396, 1, 1], 
                    Hue[0.8654707714978663, 1, 1], 
                    Hue[0.8654707714978663, 1, 1], 
                    Hue[0.9566988597906313, 1, 1], 
                    Hue[0.9566988597906313, 1, 1], 
                    Hue[0.04833514536830779, 1, 1], 
                    Hue[0.04833514536830779, 1, 1], 
                    Hue[0.14037917323177285`, 1, 1], 
                    Hue[0.14037917323177285`, 1, 1], 
                    Hue[0.23283177667960436`, 1, 1], 
                    Hue[0.23283177667960436`, 1, 1], 
                    Hue[0.32569164258084476`, 1, 1], 
                    Hue[0.32569164258084476`, 1, 1], 
                    Hue[0.41895898555335936`, 1, 1], 
                    Hue[0.41895898555335936`, 1, 1], 
                    Hue[0.5126322184725833, 1, 1], 
                    Hue[0.5126322184725833, 1, 1], 
                    Hue[0.6067109627284879, 1, 1], 
                    Hue[0.6067109627284879, 1, 1], 
                    Hue[0.7011944510053674, 1, 1], 
                    Hue[0.7011944510053674, 1, 1], 
                    Hue[0.7960825734629512, 1, 1], 
                    Hue[0.7960825734629512, 1, 1], 
                    Hue[0.8913750773674782, 1, 1], 
                    Hue[0.8913750773674782, 1, 1], 
                    Hue[0.9870703085292256, 1, 1], 
                    Hue[0.9870703085292256, 1, 1], 
                    Hue[0.0831682078717523, 1, 1], 
                    Hue[0.0831682078717523, 1, 1], 
                    Hue[0.17966690754711842`, 1, 1], 
                    Hue[0.17966690754711842`, 1, 1], 
                    Hue[0.27656651526156967`, 1, 1], 
                    Hue[0.27656651526156967`, 1, 1], 
                    Hue[0.37386606936161204`, 1, 1], 
                    Hue[0.37386606936161204`, 1, 1], 
                    Hue[0.47156409420583095`, 1, 1], 
                    Hue[0.47156409420583095`, 1, 1], 
                    Hue[0.5696598302845104, 1, 1], 
                    Hue[0.5696598302845104, 1, 1], 
                    Hue[0.6681520841585072, 1, 1], 
                    Hue[0.6681520841585072, 1, 1], 
                    Hue[0.7670396100066637, 1, 1], 
                    Hue[0.7670396100066637, 1, 1], 
                    Hue[0.8663210495276291, 1, 1], 
                    Hue[0.8663210495276291, 1, 1], 
                    Hue[0.965994561204372, 1, 1], 
                    Hue[0.965994561204372, 1, 1], 
                    Hue[0.06605854132798218, 1, 1], 
                    Hue[0.06605854132798218, 1, 1], 
                    Hue[0.1665109206955648, 1, 1], 
                    Hue[0.1665109206955648, 1, 1], 
                    Hue[0.26735002732299895`, 1, 1], 
                    Hue[0.26735002732299895`, 1, 1], 
                    Hue[0.3685727399772961, 1, 1], 
                    Hue[0.3685727399772961, 1, 1], 
                    Hue[0.4701766148566998, 1, 1], 
                    Hue[0.4701766148566998, 1, 1], 
                    Hue[0.5721579648495672, 1, 1], 
                    Hue[0.5721579648495672, 1, 1], 
                    Hue[0.6745138217897079, 1, 1], 
                    Hue[0.6745138217897079, 1, 1], 
                    Hue[0.7772401123597215, 1, 1], 
                    Hue[0.7772401123597215, 1, 1], 
                    Hue[0.8803315074975259, 1, 1], 
                    Hue[0.8803315074975259, 1, 1], 
                    Hue[0.9837828931126193, 1, 1], 
                    Hue[0.9837828931126193, 1, 1], 
                    Hue[0.08758813029091003, 1, 1], 
                    Hue[0.08758813029091003, 1, 1], 
                    Hue[0.19173995962207716`, 1, 1], 
                    Hue[0.19173995962207716`, 1, 1], 
                    Hue[0.29623052831179936`, 1, 1], 
                    Hue[0.29623052831179936`, 1, 1], 
                    Hue[0.40104982353295887`, 1, 1], 
                    Hue[0.40104982353295887`, 1, 1], 
                    Hue[0.5061869922694149, 1, 1], 
                    Hue[0.5061869922694149, 1, 1], 
                    Hue[0.6116299342344147, 1, 1], 
                    Hue[0.6116299342344147, 1, 1], 
                    Hue[0.7173639634709316, 1, 1], 
                    Hue[0.7173639634709316, 1, 1], 
                    Hue[0.8233718474500051, 1, 1], 
                    Hue[0.8233718474500051, 1, 1], 
                    Hue[0.9296346319294032, 1, 1], 
                    Hue[0.9296346319294032, 1, 1], 
                    Hue[0.03613037661478721, 1, 1], 
                    Hue[0.03613037661478721, 1, 1], 
                    Hue[0.1428334271268505, 1, 1], 
                    Hue[0.1428334271268505, 1, 1], 
                    Hue[0.24971412507695165`, 1, 1], 
                    Hue[0.24971412507695165`, 1, 1], 
                    Hue[0.356738931200211, 1, 1], 
                    Hue[0.356738931200211, 1, 1], 
                    Hue[0.4638696268399593, 1, 1], 
                    Hue[0.4638696268399593, 1, 1], 
                    Hue[0.571063319566488, 1, 1], 
                    Hue[0.571063319566488, 1, 1], 
                    Hue[0.6782700343654368, 1, 1], 
                    Hue[0.6782700343654368, 1, 1], 
                    Hue[0.785434006690692, 1, 1], 
                    Hue[0.785434006690692, 1, 1], 
                    Hue[0.8924930729167271, 1, 1], 
                    Hue[0.8924930729167271, 1, 1], 
                    Hue[0.9993779783296385, 1, 1], 
                    Hue[0.9993779783296385, 1, 1], 
                    Hue[0.10601149166352605`, 1, 1], 
                    Hue[0.10601149166352605`, 1, 1], 
                    Hue[0.2123087313751443, 1, 1], 
                    Hue[0.2123087313751443, 1, 1], 
                    Hue[0.3181787875662695, 1, 1], 
                    Hue[0.3181787875662695, 1, 1], 
                    Hue[0.4235247873915342, 1, 1], 
                    Hue[0.4235247873915342, 1, 1], 
                    Hue[0.5282462551631992, 1, 1], 
                    Hue[0.5282462551631992, 1, 1], 
                    Hue[0.6322395345155183, 1, 1], 
                    Hue[0.6322395345155183, 1, 1], 
                    Hue[0.7354006500082138, 1, 1], 
                    Hue[0.7354006500082138, 1, 1], 
                    Hue[0.8376300003692168, 1, 1], 
                    Hue[0.8376300003692168, 1, 1], 
                    Hue[0.9388378605035128, 1, 1], 
                    Hue[0.9388378605035128, 1, 1], 
                    Hue[0.038949607055956306`, 1, 1], 
                    Hue[0.038949607055956306`, 1, 1], 
                    Hue[0.13790895816925902`, 1, 1], 
                    Hue[0.13790895816925902`, 1, 1], 
                    Hue[0.23568267151472239`, 1, 1], 
                    Hue[0.23568267151472239`, 1, 1], 
                    Hue[0.3322654544287647, 1, 1], 
                    Hue[0.3322654544287647, 1, 1], 
                    Hue[0.42768181401220956`, 1, 1], 
                    Hue[0.42768181401220956`, 1, 1], 
                    Hue[0.5219917613987968, 1, 1], 
                    Hue[0.5219917613987968, 1, 1], 
                    Hue[0.6152841180730468, 1, 1], 
                    Hue[0.6152841180730468, 1, 1], 
                    Hue[0.7076595650334099, 1, 1], 
                    Hue[0.7076595650334099, 1, 1], 
                    Hue[0.7992366467292306, 1, 1], 
                    Hue[0.7992366467292306, 1, 1], 
                    Hue[0.8901637845684595, 1, 1], 
                    Hue[0.8901637845684595, 1, 1], 
                    Hue[0.980586996534017, 1, 1], 
                    Hue[0.980586996534017, 1, 1], 
                    Hue[0.0706304063174194, 1, 1], 
                    Hue[0.0706304063174194, 1, 1], 
                    Hue[0.16044150686162284`, 1, 1], 
                    Hue[0.16044150686162284`, 1, 1], 
                    Hue[0.25018705154374354`, 1, 1], 
                    Hue[0.25018705154374354`, 1, 1], 
                    Hue[0.33995086535139046`, 1, 1], 
                    Hue[0.33995086535139046`, 1, 1], 
                    Hue[0.4297694192998456, 1, 1], 
                    Hue[0.4297694192998456, 1, 1], 
                    Hue[0.5198289187276111, 1, 1], 
                    Hue[0.5198289187276111, 1, 1], 
                    Hue[0.610354057984487, 1, 1], 
                    Hue[0.610354057984487, 1, 1], 
                    Hue[0.7012144543506329, 1, 1], 
                    Hue[0.7012144543506329, 1, 1], 
                    Hue[0.7921081889537456, 1, 1], 
                    Hue[0.7921081889537456, 1, 1], 
                    Hue[0.8833380798161025, 1, 1], 
                    Hue[0.8833380798161025, 1, 1], 
                    Hue[0.975815306966425, 1, 1], 
                    Hue[0.975815306966425, 1, 1], 
                    Hue[0.0698586820590084, 1, 1], 
                    Hue[0.0698586820590084, 1, 1], 
                    Hue[0.16445521240978872`, 1, 1], 
                    Hue[0.16445521240978872`, 1, 1], 
                    Hue[0.2582012082242161, 1, 1], 
                    Hue[0.2582012082242161, 1, 1], 
                    Hue[0.35121116346775805`, 1, 1], 
                    Hue[0.35121116346775805`, 1, 1], 
                    Hue[0.44563606809176354`, 1, 1], 
                    Hue[0.44563606809176354`, 1, 1], 
                    Hue[0.5430933601796459, 1, 1], 
                    Hue[0.5430933601796459, 1, 1], 
                    Hue[0.6423155915986974, 1, 1], 
                    Hue[0.6423155915986974, 1, 1], 
                    Hue[0.7407113090471817, 1, 1], 
                    Hue[0.7407113090471817, 1, 1], 
                    Hue[0.8376881189539714, 1, 1], 
                    Hue[0.8376881189539714, 1, 1], 
                    Hue[0.9352232891503853, 1, 1], 
                    Hue[0.9352232891503853, 1, 1], 
                    Hue[0.03614858908370498, 1, 1], 
                    Hue[0.03614858908370498, 1, 1], 
                    Hue[0.14398074114436193`, 1, 1], 
                    Hue[0.14398074114436193`, 1, 1], 
                    Hue[0.25920378352449885`, 1, 1], 
                    Hue[0.25920378352449885`, 1, 1], 
                    Hue[0.3668844355750923, 1, 1], 
                    Hue[0.3668844355750923, 1, 1], 
                    Hue[0.41640280589271683`, 1, 1], 
                    Hue[0.41640280589271683`, 1, 1], 
                    Hue[0.46331902891034815`, 1, 1], 
                    Hue[0.46331902891034815`, 1, 1], 
                    Hue[0.6033248517219278, 1, 1], 
                    Hue[0.6033248517219278, 1, 1], 
                    Hue[0.7883325306388308, 1, 1], 
                    Hue[0.7883325306388308, 1, 1], 
                    Hue[0.0056238243984809925`, 1, 1], 
                    Hue[0.0056238243984809925`, 1, 1], 
                    Hue[0.2774077069850332, 1, 1], 
                    Hue[0.2774077069850332, 1, 1], 
                    Hue[0.6155398081712723, 1, 1], 
                    Hue[0.6155398081712723, 1, 1], 
                    Hue[0.9030578930956729, 1, 1], 
                    Hue[0.9030578930956729, 1, 1], 
                    Hue[0.14414120650805495`, 1, 1], 
                    Hue[0.14414120650805495`, 1, 1], 
                    Hue[0.3701881994243456, 1, 1], 
                    Hue[0.3701881994243456, 1, 1], 
                    Hue[0.5916882192261339, 1, 1], 
                    Hue[0.5916882192261339, 1, 1], 
                    Hue[0.8128425253061538, 1, 1], 
                    Hue[0.8128425253061538, 1, 1], 
                    Hue[0.03606014119192609, 1, 1], 
                    Hue[0.03606014119192609, 1, 1], 
                    Hue[0.26223236189121907`, 1, 1], 
                    Hue[0.26223236189121907`, 1, 1], 
                    Hue[0.49054148919426266`, 1, 1], 
                    Hue[0.49054148919426266`, 1, 1], 
                    Hue[0.7203779998491203, 1, 1], 
                    Hue[0.7203779998491203, 1, 1], 
                    Hue[0.9525369026809333, 1, 1], 
                    Hue[0.9525369026809333, 1, 1], 
                    Hue[0.18843993116706134`, 1, 1], 
                    Hue[0.18843993116706134`, 1, 1], 
                    Hue[0.42932440021741863`, 1, 1], 
                    Hue[0.42932440021741863`, 1, 1], 
                    Hue[0.6757389699208721, 1, 1], 
                    Hue[0.6757389699208721, 1, 1], 
                    Hue[0.9271433109474645, 1, 1], 
                    Hue[0.9271433109474645, 1, 1], 
                    Hue[0.181969958793826, 1, 1], 
                    Hue[0.181969958793826, 1, 1], 
                    Hue[0.438446713394616, 1, 1], 
                    Hue[0.438446713394616, 1, 1], 
                    Hue[0.6955392300974138, 1, 1], 
                    Hue[0.6955392300974138, 1, 1], 
                    Hue[0.9530958902650895, 1, 1], 
                    Hue[0.9530958902650895, 1, 1], 
                    Hue[0.21123375064817898`, 1, 1], 
                    Hue[0.21123375064817898`, 1, 1], 
                    Hue[0.46967263406929427`, 1, 1], 
                    Hue[0.46967263406929427`, 1, 1], 
                    Hue[0.7276023629921933, 1, 1], 
                    Hue[0.7276023629921933, 1, 1], 
                    Hue[0.984161950447084, 1, 1], 
                    Hue[0.984161950447084, 1, 1], 
                    Hue[0.2390623682607009, 1, 1], 
                    Hue[0.2390623682607009, 1, 1], 
                    Hue[0.49283556399150985`, 1, 1], 
                    Hue[0.49283556399150985`, 1, 1], 
                    Hue[0.7466324493665115, 1, 1], 
                    Hue[0.7466324493665115, 1, 1], 
                    Hue[0.0017163865090559496`, 1, 1], 
                    Hue[0.0017163865090559496`, 1, 1], 
                    Hue[0.2586526882451303, 1, 1], 
                    Hue[0.2586526882451303, 1, 1], 
                    Hue[0.5162546501565902, 1, 1], 
                    Hue[0.5162546501565902, 1, 1], 
                    Hue[0.7710500352229288, 1, 1], 
                    Hue[0.7710500352229288, 1, 1], 
                    Hue[0.018209373829782306`, 1, 1], 
                    Hue[0.018209373829782306`, 1, 1], 
                    Hue[0.25272791954391893`, 1, 1], 
                    Hue[0.25272791954391893`, 1, 1], 
                    Hue[0.46754293320048695`, 1, 1], 
                    Hue[0.46754293320048695`, 1, 1], 
                    Hue[0.6414161184083459, 1, 1], 
                    Hue[0.6414161184083459, 1, 1], 
                    Hue[0.7116095092051098, 1, 1], 
                    Hue[0.7116095092051098, 1, 1], 
                    Hue[0.7941755173099261, 1, 1], 
                    Hue[0.7941755173099261, 1, 1], 
                    Hue[0.959677603338433, 1, 1], 
                    Hue[0.959677603338433, 1, 1], 
                    Hue[0.15416041543491266`, 1, 1], 
                    Hue[0.15416041543491266`, 1, 1], 
                    Hue[0.3659312105320584, 1, 1], 
                    Hue[0.3659312105320584, 1, 1], 
                    Hue[0.5945187685231991, 1, 1], 
                    Hue[0.5945187685231991, 1, 1], 
                    Hue[0.8435841217998448, 1, 1], 
                    Hue[0.8435841217998448, 1, 1], 
                    Hue[0.11798106092260775`, 1, 1], 
                    Hue[0.11798106092260775`, 1, 1], 
                    Hue[0.40976957642296785`, 1, 1], 
                    Hue[0.40976957642296785`, 1, 1], 
                    Hue[0.6931950268848085, 1, 1], 
                    Hue[0.6931950268848085, 1, 1], 
                    Hue[0.9572768482725862, 1, 1], 
                    Hue[0.9572768482725862, 1, 1], 
                    Hue[0.20736297205040422`, 1, 1], 
                    Hue[0.20736297205040422`, 1, 1], 
                    Hue[0.4495073129009342, 1, 1], 
                    Hue[0.4495073129009342, 1, 1], 
                    Hue[0.6873302371223096, 1, 1], 
                    Hue[0.6873302371223096, 1, 1], 
                    Hue[0.922917256847574, 1, 1], 
                    Hue[0.922917256847574, 1, 1]}],
                   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
                   Axes->{True, None},
                   AxesOrigin->{0, 0},
                   
                   Epilog->LineBox[{{1, 7.414327510823197*^-7}, {
                    2, 7.726648117048535*^-7}, {3, 8.227827566517121*^-7}, {
                    4, 1.093774842111724*^-6}, {5, 6.92663342057923*^-7}, {
                    6, 9.029940042057901*^-7}, {7, 1.0029934330924888`*^-6}, {
                    8, 9.594195588106386*^-7}, {9, 1.0271893547518258`*^-6}, {
                    10, 1.553112599900039*^-6}, {
                    11, 1.4122340889127595`*^-6}, {
                    12, 1.6304166533229733`*^-6}, {
                    13, 1.6350909292463403`*^-6}, {
                    14, 2.2090677784137976`*^-6}, {
                    15, 2.2962723623266234`*^-6}, {
                    16, 9.707863278121864*^-7}, {17, 8.020548870281221*^-7}, {
                    18, 2.822056189897182*^-6}, {
                    19, 4.6869507720681674`*^-6}, {
                    20, 6.096744079244237*^-6}, {21, 7.063427491395416*^-6}, {
                    22, 7.794429600735892*^-6}, {
                    23, 0.000010000334409831495`}, {
                    24, 0.000016079909290535753`}, {
                    25, 0.000024694801992314865`}, {
                    26, 0.00003455279454292162}, {
                    27, 0.000045972828071435314`}, {
                    28, 0.000060129516385812365`}, {
                    29, 0.00008244508585235948}, {
                    30, 0.00011348064676914458`}, {
                    31, 0.00015421730440595075`}, {
                    32, 0.00020666137672112157`}, {
                    33, 0.0002738907954709313}, {
                    34, 0.00036249393886701866`}, {
                    35, 0.0004795055246468662}, {36, 0.0006319610183347425}, {
                    37, 0.0008262340614665134}, {38, 0.001072238789977809}, {
                    39, 0.0013856598500324628`}, {
                    40, 0.0017828545468639974`}, {
                    41, 0.0022847224707939176`}, {
                    42, 0.0029112548477058732`}, {
                    43, 0.0036885158297529684`}, {
                    44, 0.0046471797210618495`}, {45, 0.005826297023032688}, {
                    46, 0.007269174042799757}, {47, 0.00902337255522693}, {
                    48, 0.011139495212259065`}, {49, 0.013680367241996328`}, {
                    50, 0.01671405533605794}, {51, 0.0203167530574952}, {
                    52, 0.02456848322042616}, {53, 0.02955560705012436}, {
                    54, 0.03536918925939972}, {55, 0.0421061246410018}, {
                    56, 0.04986818001732029}, {57, 0.05875546153974489}, {
                    58, 0.06886598944801064}, {59, 0.08029625234723621}, {
                    60, 0.09313682382931113}, {61, 0.10747049427939853`}, {
                    62, 0.12336581515893144`}, {63, 0.140877328810592}, {
                    64, 0.1600380256187275}, {65, 0.18085763736287935`}, {
                    66, 0.20332458653902022`}, {67, 0.22739331558537393`}, {
                    68, 0.2529884165191143}, {69, 0.280001793265108}, {
                    70, 0.30828551653997743`}, {71, 0.33765916254082884`}, {
                    72, 0.36790720370645474`}, {73, 0.39877914143774007`}, {
                    74, 0.42999236379254935`}, {75, 0.4612356746723151}, {
                    76, 0.49217361421945327`}, {77, 0.5224524023023419}, {
                    78, 0.5517088725374619}, {79, 0.5795702050496168}, {
                    80, 0.6056696818733874}, {81, 0.6296501632222378}, {
                    82, 0.6511746336594946}, {83, 0.669930757354472}, {
                    84, 0.6856399305512036}, {85, 0.6980665916937486}, {
                    86, 0.7070203315403472}, {87, 0.7123637287383301}, {
                    88, 0.7140158929677963}, {89, 0.7119476364974316}, {
                    90, 0.7061922836568995}, {91, 0.6968398384595758}, {
                    92, 0.6840349839321099}, {93, 0.6679709575650906}, {
                    94, 0.6488889052524311}, {95, 0.6270693205699949}, {
                    96, 0.6028275804551093}, {97, 0.5765001309514965}, {
                    98, 0.548444547215035}, {99, 0.5190284406133917}, {
                    100, 0.48861740532525}, {101, 0.4575741128314588}, {
                    102, 0.42624811702523574`}, {103, 0.39496988820163076`}, {
                    104, 0.3640457908144311}, {105, 0.33375305133422806`}, {
                    106, 0.30433775400326585`}, {107, 0.2760132868026079}, {
                    108, 0.2489571357334717}, {109, 0.22331019272275382`}, {
                    110, 0.1991815252064953}, {111, 0.17664572645663257`}, {
                    112, 0.15574708320501904`}, {113, 0.13650094059901738`}, {
                    114, 0.11889856500892278`}, {115, 0.1029094474280986}, {
                    116, 0.08848510494641651}, {117, 0.07556141916118712}, {
                    118, 0.06406190154534439}, {119, 0.05390217484054021}, {
                    120, 0.0449922563097948}, {121, 0.0372386216672715}, {
                    122, 0.030546115015454664`}, {123, 0.02481945644097266}, {
                    124, 0.019965172713635293`}, {
                    125, 0.015892201526474237`}, {
                    126, 0.012512529524703274`}, {
                    127, 0.009741382974183893}, {128, 0.007497271014814265}, {
                    129, 0.0057035103917303775`}, {
                    130, 0.004289283991860947}, {
                    131, 0.0031895139578431916`}, {
                    132, 0.0023455531991862415`}, {
                    133, 0.001706513453597146}, {
                    134, 0.0012290554977053695`}, {
                    135, 0.0008765476755997755}, {
                    136, 0.0006191399800800749}, {
                    137, 0.00043348917358537286`}, {
                    138, 0.0003011735024753659}, {
                    139, 0.00020754971025556095`}, {
                    140, 0.00014173769256671488`}, {
                    141, 0.00009611499901884002}, {
                    142, 0.00006498672083488544}, {
                    143, 0.00004380237613635753}, {
                    144, 0.00002924481123072321}, {
                    145, 0.000019246461737039464`}, {
                    146, 0.000012545604184898961`}, {
                    147, 8.200360770300677*^-6}, {
                    148, 5.401433218187356*^-6}, {
                    149, 3.5431454016851195`*^-6}, {
                    150, 2.282352420502683*^-6}, {
                    151, 1.4475910571191847`*^-6}, {
                    152, 9.189024635576254*^-7}, {
                    153, 5.901948711551626*^-7}, {
                    154, 3.8311119386201366`*^-7}, {
                    155, 2.4667724473902263`*^-7}, {
                    156, 1.4865099651632655`*^-7}, {
                    157, 7.696523884268475*^-8}, {
                    158, 3.89326617851635*^-8}, {
                    159, 3.726456412680317*^-8}, {
                    160, 3.7605901478163517`*^-8}, {
                    161, 2.9453433551011027`*^-8}, {
                    162, 1.7993605160619925`*^-8}, {
                    163, 9.103960578601715*^-9}, {
                    164, 6.311695715418187*^-9}, {
                    165, 6.401799679296129*^-9}, {
                    166, 5.68501651573685*^-9}, {
                    167, 4.269643070896361*^-9}, {
                    168, 2.847684714471986*^-9}, {
                    169, 1.7895282937427172`*^-9}, {
                    170, 1.1437952757066059`*^-9}, {
                    171, 7.999984398510176*^-10}, {
                    172, 6.218463821877656*^-10}, {
                    173, 5.115524015505435*^-10}, {
                    174, 4.207630480745786*^-10}, {
                    175, 3.357366216645681*^-10}, {
                    176, 2.5843136097154293`*^-10}, {
                    177, 1.935474922958874*^-10}, {
                    178, 1.4293116852212076`*^-10}, {
                    179, 1.0511453421243559`*^-10}, {
                    180, 7.712115487772525*^-11}, {
                    181, 5.621667430914929*^-11}, {
                    182, 4.054390586868913*^-11}, {
                    183, 2.8908104750926594`*^-11}, {
                    184, 2.040498913143773*^-11}, {
                    185, 1.4251912976180901`*^-11}, {
                    186, 9.800630048217516*^-12}, {
                    187, 6.57913131445202*^-12}, {
                    188, 4.274770790377179*^-12}, {
                    189, 2.6749152561166425`*^-12}, {
                    190, 1.6133543412842098`*^-12}, {
                    191, 9.448194700992246*^-13}, {
                    192, 5.42731984055033*^-13}, {
                    193, 3.067097325077313*^-13}, {
                    194, 1.6801927044294415`*^-13}, {
                    195, 8.6261312595424*^-14}, {
                    196, 3.948342048738686*^-14}, {
                    197, 1.5193867717940518`*^-14}, {
                    198, 6.5694239209621976`*^-15}, {
                    199, 6.708009929094076*^-15}, {
                    200, 6.9279436136124596`*^-15}, {
                    201, 6.194400127951439*^-15}, {
                    202, 4.965903588682642*^-15}, {
                    203, 3.618768463808099*^-15}, {
                    204, 2.442251038285703*^-15}, {
                    205, 1.6301870902702573`*^-15}, {
                    206, 1.2158632258205339`*^-15}, {
                    207, 1.0318398450867304`*^-15}, {
                    208, 8.898102756987287*^-16}, {
                    209, 7.263451976926728*^-16}, {
                    210, 5.522281081260929*^-16}, {
                    211, 3.9170248855427457`*^-16}, {
                    212, 2.6044039812402194`*^-16}, {
                    213, 1.6295044994113111`*^-16}, {
                    214, 9.614260209824502*^-17}, {
                    215, 5.350234815246673*^-17}, {
                    216, 2.8047113721690084`*^-17}, {
                    217, 1.3861995886414788`*^-17}, {
                    218, 6.5904240015867805`*^-18}, {
                    219, 3.3080978813061304`*^-18}, {
                    220, 2.0503510579974192`*^-18}, {
                    221, 1.508888521740555*^-18}, {
                    222, 1.132026207373764*^-18}, {
                    223, 8.160692801667178*^-19}, {
                    224, 5.613574032985331*^-19}, {
                    225, 3.7000252062908056`*^-19}, {
                    226, 2.347957359001564*^-19}, {
                    227, 1.4396579220806302`*^-19}, {
                    228, 8.550562430544295*^-20}, {
                    229, 4.927676895626111*^-20}, {
                    230, 2.758946893870306*^-20}, {
                    231, 1.5023139820600678`*^-20}, {
                    232, 7.96560350376841*^-21}, {
                    233, 4.119992897606218*^-21}, {
                    234, 2.085043581322321*^-21}, {
                    235, 1.0378245174651536`*^-21}, {
                    236, 5.122350162534242*^-22}, {
                    237, 2.5346963760975466`*^-22}, {
                    238, 1.271573166528436*^-22}, {
                    239, 6.507491161079296*^-23}, {
                    240, 3.3893415267401256`*^-23}, {
                    241, 1.7805307740970742`*^-23}, {
                    242, 9.34133877014123*^-24}, {
                    243, 4.863817014848145*^-24}, {
                    244, 2.5120476447117986`*^-24}, {
                    245, 1.2943882778761253`*^-24}, {
                    246, 6.729119257503642*^-25}, {
                    247, 3.575203807497975*^-25}, {
                    248, 1.955977041851322*^-25}, {
                    249, 1.0987051004439517`*^-25}, {
                    250, 6.267042066341017*^-26}, {
                    251, 3.5851017056091055`*^-26}}],
                   Frame->True,
                   FrameTicks->{{{1., 
                    FormBox[
                    RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
                    FormBox[
                    RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
                    FormBox[
                    RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
                    FormBox[
                    RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
                    FormBox["0", TraditionalForm]}, {157.25, 
                    FormBox["2.5`", TraditionalForm]}, {188.5, 
                    FormBox["5", TraditionalForm]}, {219.75, 
                    FormBox["7.5`", TraditionalForm]}, {251., 
                    FormBox["10", TraditionalForm]}}, Automatic, None, None},
                   PlotRange->{-0.1, 1.1},
                   PlotRangeClipping->True,
                   PlotRangePadding->{
                    Scaled[0.02], Automatic}], 6->
                  GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF2XdYlXUfx/EfrnBkuCd6UCTMrag4kA9LUUEEGQdZN/swJYRUQLlBsWFm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                    "], {
                    {GrayLevel[0.5], EdgeForm[None], 
                    GraphicsGroupBox[{PolygonBox[{{230, 710, 231}}], 
                    PolygonBox[CompressedData["
1:eJwtlwfYFlIYht9zCiHRIJJoD1kt2kmSJikqSZK0l1S0QxIKpW2UrFQISUJS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                    
                    "]]}]}, {{}, {}, {
                    {Hue[0.7796801289652671, 1, 1], LineBox[{1, 2}]}, 
                    {Hue[0.6407245824031771, 1, 1], LineBox[{2, 3}]}, 
                    {Hue[0.5503816896589118, 1, 1], LineBox[{3, 4}]}, 
                    {Hue[0.46693777518468293`, 1, 1], LineBox[{4, 5}]}, 
                    {Hue[0.34337602294665853`, 1, 1], LineBox[{5, 6}]}, 
                    {Hue[0.2800240470435314, 1, 1], LineBox[{6, 7}]}, 
                    {Hue[0.19178230855508877`, 1, 1], LineBox[{7, 8}]}, 
                    {Hue[0.09597816212744277, 1, 1], LineBox[{8, 9}]}, 
                    {Hue[0.03441314240637002, 1, 1], LineBox[{9, 10}]}, 
                    {Hue[0.9619307728034987, 1, 1], LineBox[{10, 11}]}, 
                    {Hue[0.8556957513387586, 1, 1], LineBox[{11, 12}]}, 
                    {Hue[0.7617493391570483, 1, 1], LineBox[{12, 13}]}, 
                    {Hue[0.6649750661560606, 1, 1], LineBox[{13, 14}]}, 
                    {Hue[0.6003782106730116, 1, 1], LineBox[{14, 15}]}, 
                    {Hue[0.5016480547751627, 1, 1], LineBox[{15, 16}]}, 
                    {Hue[0.40821669939596, 1, 1], LineBox[{16, 17}]}, 
                    {Hue[0.3228877865793172, 1, 1], LineBox[{17, 18}]}, 
                    {Hue[0.2545692858199149, 1, 1], LineBox[{18, 19}]}, 
                    {Hue[0.16962359970764626`, 1, 1], LineBox[{19, 20}]}, 
                    {Hue[0.056865225111496887`, 1, 1], LineBox[{20, 21}]}, 
                    {Hue[0.9539397202584826, 1, 1], LineBox[{21, 22}]}, 
                    {Hue[0.7837294353244291, 1, 1], LineBox[{22, 23}]}, 
                    {Hue[0.6953650268548688, 1, 1], LineBox[{23, 24}]}, 
                    {Hue[0.6305218286702476, 1, 1], LineBox[{24, 25}]}, 
                    {Hue[0.6528261996783604, 1, 1], LineBox[{25, 26}]}, 
                    {Hue[0.7458918504427492, 1, 1], LineBox[{26, 27}]}, 
                    {Hue[0.92126851876369, 1, 1], LineBox[{27, 28}]}, 
                    {Hue[0.9782053362672661, 1, 1], LineBox[{28, 29}]}, 
                    {Hue[0.020789729978857467`, 1, 1], LineBox[{29, 30}]}, 
                    {Hue[0.07110836451469632, 1, 1], LineBox[{30, 31}]}, 
                    {Hue[0.13731102313054688`, 1, 1], LineBox[{31, 32}]}, 
                    {Hue[0.2237270578911424, 1, 1], LineBox[{32, 33}]}, 
                    {Hue[0.308233058166907, 1, 1], LineBox[{33, 34}]}, 
                    {Hue[0.37858341969070897`, 1, 1], LineBox[{34, 35}]}, 
                    {Hue[0.44582144082943587`, 1, 1], LineBox[{35, 36}]}, 
                    {Hue[0.5171911046750017, 1, 1], LineBox[{36, 37}]}, 
                    {Hue[0.5903988201832231, 1, 1], LineBox[{37, 38}]}, 
                    {Hue[0.6664515169027294, 1, 1], LineBox[{38, 39}]}, 
                    {Hue[0.7432363389081607, 1, 1], LineBox[{39, 40}]}, 
                    {Hue[0.8184062390390154, 1, 1], LineBox[{40, 41}]}, 
                    {Hue[0.8937470175078367, 1, 1], LineBox[{41, 42}]}, 
                    {Hue[0.9698398309055655, 1, 1], LineBox[{42, 43}]}, 
                    {Hue[0.046820964778370125`, 1, 1], LineBox[{43, 44}]}, 
                    {Hue[0.12449143522202596`, 1, 1], LineBox[{44, 45}]}, 
                    {Hue[0.20232629915042974`, 1, 1], LineBox[{45, 46}]}, 
                    {Hue[0.2805663074213705, 1, 1], LineBox[{46, 47}]}, 
                    {Hue[0.3590896587353866, 1, 1], LineBox[{47, 48}]}, 
                    {Hue[0.4382390460728896, 1, 1], LineBox[{48, 49}]}, 
                    {Hue[0.517902737197748, 1, 1], LineBox[{49, 50}]}, 
                    {Hue[0.5980097653972472, 1, 1], LineBox[{50, 51}]}, 
                    {Hue[0.6785327051929539, 1, 1], LineBox[{51, 52}]}, 
                    {Hue[0.7595095736388215, 1, 1], LineBox[{52, 53}]}, 
                    {Hue[0.8409610804245629, 1, 1], LineBox[{53, 54}]}, 
                    {Hue[0.9228629740322556, 1, 1], LineBox[{54, 55}]}, 
                    {Hue[0.005253870251030078, 1, 1], LineBox[{55, 56}]}, 
                    {Hue[0.0881095112778214, 1, 1], LineBox[{56, 57}]}, 
                    {Hue[0.17139434226227557`, 1, 1], LineBox[{57, 58}]}, 
                    {Hue[0.2551463423565107, 1, 1], LineBox[{58, 59}]}, 
                    {Hue[0.33933976584552206`, 1, 1], LineBox[{59, 60}]}, 
                    {Hue[0.4239966318214204, 1, 1], LineBox[{60, 61}]}, 
                    {Hue[0.5091224007223621, 1, 1], LineBox[{61, 62}]}, 
                    {Hue[0.5946966107789967, 1, 1], LineBox[{62, 63}]}, 
                    {Hue[0.6807271939580889, 1, 1], LineBox[{63, 64}]}, 
                    {Hue[0.7672101480855866, 1, 1], LineBox[{64, 65}]}, 
                    {Hue[0.8541512225764117, 1, 1], LineBox[{65, 66}]}, 
                    {Hue[0.941550181896359, 1, 1], LineBox[{66, 67}]}, 
                    {Hue[0.029403984208433615`, 1, 1], LineBox[{67, 68}]}, 
                    {Hue[0.11770952561423882`, 1, 1], LineBox[{68, 69}]}, 
                    {Hue[0.20646973711035155`, 1, 1], LineBox[{69, 70}]}, 
                    {Hue[0.295683004779304, 1, 1], LineBox[{70, 71}]}, 
                    {Hue[0.3853477154083707, 1, 1], LineBox[{71, 72}]}, 
                    {Hue[0.4754668125881876, 1, 1], LineBox[{72, 73}]}, 
                    {Hue[0.5660352585959618, 1, 1], LineBox[{73, 74}]}, 
                    {Hue[0.6570548476475564, 1, 1], LineBox[{74, 75}]}, 
                    {Hue[0.7485248677659424, 1, 1], LineBox[{75, 76}]}, 
                    {Hue[0.8404436664280217, 1, 1], LineBox[{76, 77}]}, 
                    {Hue[0.9328115568614501, 1, 1], LineBox[{77, 78}]}, 
                    {Hue[0.025625635071901066`, 1, 1], LineBox[{78, 79}]}, 
                    {Hue[0.11888645287894296`, 1, 1], LineBox[{79, 80}]}, 
                    {Hue[0.21259272180999705`, 1, 1], LineBox[{80, 81}]}, 
                    {Hue[0.3067437454522678, 1, 1], LineBox[{81, 82}]}, 
                    {Hue[0.4013375289297376, 1, 1], LineBox[{82, 83}]}, 
                    {Hue[0.49637135639775043`, 1, 1], LineBox[{83, 84}]}, 
                    {Hue[0.5918450111035345, 1, 1], LineBox[{84, 85}]}, 
                    {Hue[0.6877562589564038, 1, 1], LineBox[{85, 86}]}, 
                    {Hue[0.78410256268569, 1, 1], LineBox[{86, 87}]}, 
                    {Hue[0.8808823473932245, 1, 1], LineBox[{87, 88}]}, 
                    {Hue[0.9780911430522481, 1, 1], LineBox[{88, 89}]}, 
                    {Hue[0.07572664367055548, 1, 1], LineBox[{89, 90}]}, 
                    {Hue[0.1737853758905005, 1, 1], LineBox[{90, 91}]}, 
                    {Hue[0.272262768039806, 1, 1], LineBox[{91, 92}]}, 
                    {Hue[0.371153556618099, 1, 1], LineBox[{92, 93}]}, 
                    {Hue[0.4704529220778693, 1, 1], LineBox[{93, 94}]}, 
                    {Hue[0.5701542763470691, 1, 1], LineBox[{94, 95}]}, 
                    {Hue[0.6702500724172249, 1, 1], LineBox[{95, 96}]}, 
                    {Hue[0.7707317610985269, 1, 1], LineBox[{96, 97}]}, 
                    {Hue[0.8715892246887066, 1, 1], LineBox[{97, 98}]}, 
                    {Hue[0.9728114871992333, 1, 1], LineBox[{98, 99}]}, 
                    {Hue[0.0743857178097356, 1, 1], LineBox[{99, 100}]}, 
                    {Hue[0.17629597248002574`, 1, 1], LineBox[{100, 101}]}, 
                    {Hue[0.2785251813360693, 1, 1], LineBox[{101, 102}]}, 
                    {Hue[0.38105343827909777`, 1, 1], LineBox[{102, 103}]}, 
                    {Hue[0.4838574827290812, 1, 1], LineBox[{103, 104}]}, 
                    {Hue[0.5869100103646415, 1, 1], LineBox[{104, 105}]}, 
                    {Hue[0.6901806989907271, 1, 1], LineBox[{105, 106}]}, 
                    {Hue[0.7936343981668283, 1, 1], LineBox[{106, 107}]}, 
                    {Hue[0.89723070358846, 1, 1], LineBox[{107, 108}]}, 
                    {Hue[0.0009237190342984455, 1, 1], LineBox[{108, 109}]}, 
                    {Hue[0.10466113495925189`, 1, 1], LineBox[{109, 110}]}, 
                    {Hue[0.20838404049157222`, 1, 1], LineBox[{110, 111}]}, 
                    {Hue[0.3120259981560382, 1, 1], LineBox[{111, 112}]}, 
                    {Hue[0.4155125941036146, 1, 1], LineBox[{112, 113}]}, 
                    {Hue[0.5187618099741917, 1, 1], LineBox[{113, 114}]}, 
                    {Hue[0.6216818288821319, 1, 1], LineBox[{114, 115}]}, 
                    {Hue[0.7241743766947099, 1, 1], LineBox[{115, 116}]}, 
                    {Hue[0.8261341567272162, 1, 1], LineBox[{116, 117}]}, 
                    {Hue[0.9274483379521479, 1, 1], LineBox[{117, 118}]}, 
                    {Hue[0.027999913112641616`, 1, 1], LineBox[{118, 119}]}, 
                    {Hue[0.12767141780559033`, 1, 1], LineBox[{119, 120}]}, 
                    {Hue[0.226347300174275, 1, 1], LineBox[{120, 121}]}, 
                    {Hue[0.3239190041183996, 1, 1], LineBox[{121, 122}]}, 
                    {Hue[0.42028976211005264`, 1, 1], LineBox[{122, 123}]}, 
                    {Hue[0.5153815945444176, 1, 1], LineBox[{123, 124}]}, 
                    {Hue[0.6091439972339776, 1, 1], LineBox[{124, 125}]}, 
                    {Hue[0.7015583727446617, 1, 1], LineBox[{125, 126}]}, 
                    {Hue[0.7926410197746079, 1, 1], LineBox[{126, 127}]}, 
                    {Hue[0.8824458895514251, 1, 1], LineBox[{127, 128}]}, 
                    {Hue[0.9710638880548629, 1, 1], LineBox[{128, 129}]}, 
                    {Hue[0.05861534812403368, 1, 1], LineBox[{129, 130}]}, 
                    {Hue[0.14524236947836713`, 1, 1], LineBox[{130, 131}]}, 
                    {Hue[0.2311013783598076, 1, 1], LineBox[{131, 132}]}, 
                    {Hue[0.3163582834028976, 1, 1], LineBox[{132, 133}]}, 
                    {Hue[0.4011833062789287, 1, 1], LineBox[{133, 134}]}, 
                    {Hue[0.48573932037648593`, 1, 1], LineBox[{134, 135}]}, 
                    {Hue[0.5701759410951197, 1, 1], LineBox[{135, 136}]}, 
                    {Hue[0.6546269333325413, 1, 1], LineBox[{136, 137}]}, 
                    {Hue[0.7392210534650951, 1, 1], LineBox[{137, 138}]}, 
                    {Hue[0.8240822234173037, 1, 1], LineBox[{138, 139}]}, 
                    {Hue[0.9093113235791046, 1, 1], LineBox[{139, 140}]}, 
                    {Hue[0.9949861214641248, 1, 1], LineBox[{140, 141}]}, 
                    {Hue[0.08117017774335357, 1, 1], LineBox[{141, 142}]}, 
                    {Hue[0.16791383557109385`, 1, 1], LineBox[{142, 143}]}, 
                    {Hue[0.25530021949010917`, 1, 1], LineBox[{143, 144}]}, 
                    {Hue[0.34343541671925093`, 1, 1], LineBox[{144, 145}]}, 
                    {Hue[0.43231232279874066`, 1, 1], LineBox[{145, 146}]}, 
                    {Hue[0.5218524601207256, 1, 1], LineBox[{146, 147}]}, 
                    {Hue[0.612173062059559, 1, 1], LineBox[{147, 148}]}, 
                    {Hue[0.7034698819664108, 1, 1], LineBox[{148, 149}]}, 
                    {Hue[0.7955898813013543, 1, 1], LineBox[{149, 150}]}, 
                    {Hue[0.888364574202874, 1, 1], LineBox[{150, 151}]}, 
                    {Hue[0.9822000502315932, 1, 1], LineBox[{151, 152}]}, 
                    {Hue[0.0771550576325835, 1, 1], LineBox[{152, 153}]}, 
                    {Hue[0.1720959120088873, 1, 1], LineBox[{153, 154}]}, 
                    {Hue[0.26688998366448946`, 1, 1], LineBox[{154, 155}]}, 
                    {Hue[0.3639293449103173, 1, 1], LineBox[{155, 156}]}, 
                    {Hue[0.4634940195293576, 1, 1], LineBox[{156, 157}]}, 
                    {Hue[0.5603649207077858, 1, 1], LineBox[{157, 158}]}, 
                    {Hue[0.6519246304843664, 1, 1], LineBox[{158, 159}]}, 
                    {Hue[0.7495382230954826, 1, 1], LineBox[{159, 160}]}, 
                    {Hue[0.8643080095757046, 1, 1], LineBox[{160, 161}]}, 
                    {Hue[0.9906253074833521, 1, 1], LineBox[{161, 162}]}, 
                    {Hue[0.10779399475058227`, 1, 1], LineBox[{162, 163}]}, 
                    {Hue[0.1661984781166469, 1, 1], LineBox[{163, 164}]}, 
                    {Hue[0.2013592736498602, 1, 1], LineBox[{164, 165}]}, 
                    {Hue[0.32046871065631627`, 1, 1], LineBox[{165, 166}]}, 
                    {Hue[0.4756075296246733, 1, 1], LineBox[{166, 167}]}, 
                    {Hue[0.6385764348360206, 1, 1], LineBox[{167, 168}]}, 
                    {Hue[0.7869345435130637, 1, 1], LineBox[{168, 169}]}, 
                    {Hue[0.8313036715488447, 1, 1], LineBox[{169, 170}]}, 
                    {Hue[0.8287660083714148, 1, 1], LineBox[{170, 171}]}, 
                    {Hue[0.9760417001103902, 1, 1], LineBox[{171, 172}]}, 
                    {Hue[0.17203733354249606`, 1, 1], LineBox[{172, 173}]}, 
                    {Hue[0.41546928442185527`, 1, 1], LineBox[{173, 174}]}, 
                    {Hue[0.7130492258316945, 1, 1], LineBox[{174, 175}]}, 
                    {Hue[0.013352993052113508`, 1, 1], LineBox[{175, 176}]}, 
                    {Hue[0.2865809602909467, 1, 1], LineBox[{176, 177}]}, 
                    {Hue[0.5485481811412078, 1, 1], LineBox[{177, 178}]}, 
                    {Hue[0.8100483978103337, 1, 1], LineBox[{178, 179}]}, 
                    {Hue[0.07438821094553806, 1, 1], LineBox[{179, 180}]}, 
                    {Hue[0.34079459211773067`, 1, 1], LineBox[{180, 181}]}, 
                    {Hue[0.6067300720056761, 1, 1], LineBox[{181, 182}]}, 
                    {Hue[0.8699265236196507, 1, 1], LineBox[{182, 183}]}, 
                    {Hue[0.12928216875586565`, 1, 1], LineBox[{183, 184}]}, 
                    {Hue[0.3844879401823965, 1, 1], LineBox[{184, 185}]}, 
                    {Hue[0.6353011644887285, 1, 1], LineBox[{185, 186}]}, 
                    {Hue[0.8810951367672261, 1, 1], LineBox[{186, 187}]}, 
                    {Hue[0.12083724148470129`, 1, 1], LineBox[{187, 188}]}, 
                    {Hue[0.35359810181285867`, 1, 1], LineBox[{188, 189}]}, 
                    {Hue[0.5797037414262387, 1, 1], LineBox[{189, 190}]}, 
                    {Hue[0.801927284818749, 1, 1], LineBox[{190, 191}]}, 
                    {Hue[0.02490207809077299, 1, 1], LineBox[{191, 192}]}, 
                    {Hue[0.2524952787854635, 1, 1], LineBox[{192, 193}]}, 
                    {Hue[0.4861575156782732, 1, 1], LineBox[{193, 194}]}, 
                    {Hue[0.7255279616649843, 1, 1], LineBox[{194, 195}]}, 
                    {Hue[0.9696088981681895, 1, 1], LineBox[{195, 196}]}, 
                    {Hue[0.21741244056167142`, 1, 1], LineBox[{196, 197}]}, 
                    {Hue[0.46813755920083117`, 1, 1], LineBox[{197, 198}]}, 
                    {Hue[0.7211432166767873, 1, 1], LineBox[{198, 199}]}, 
                    {Hue[0.975875284824123, 1, 1], LineBox[{199, 200}]}, 
                    {Hue[0.23181597238659138`, 1, 1], LineBox[{200, 201}]}, 
                    {Hue[0.48848723203835615`, 1, 1], LineBox[{201, 202}]}, 
                    {Hue[0.745497138351656, 1, 1], LineBox[{202, 203}]}, 
                    {Hue[0.0025923563209303813`, 1, 1], 
                    LineBox[{203, 204}]}, 
                    {Hue[0.25967466350231044`, 1, 1], LineBox[{204, 205}]}, 
                    {Hue[0.5167645910330184, 1, 1], LineBox[{205, 206}]}, 
                    {Hue[0.7739220318465841, 1, 1], LineBox[{206, 207}]}, 
                    {Hue[0.031150257131776243`, 1, 1], LineBox[{207, 208}]}, 
                    {Hue[0.2883194585058068, 1, 1], LineBox[{208, 209}]}, 
                    {Hue[0.5451448055773551, 1, 1], LineBox[{209, 210}]}, 
                    {Hue[0.801241512943925, 1, 1], LineBox[{210, 211}]}, 
                    {Hue[0.05625312288155043, 1, 1], LineBox[{211, 212}]}, 
                    {Hue[0.31002956423589995`, 1, 1], LineBox[{212, 213}]}, 
                    {Hue[0.5628570448388254, 1, 1], LineBox[{213, 214}]}, 
                    {Hue[0.8158451729427638, 1, 1], LineBox[{214, 215}]}, 
                    {Hue[0.07184273217758473, 1, 1], LineBox[{215, 216}]}, 
                    {Hue[0.33804700588730185`, 1, 1], LineBox[{216, 217}]}, 
                    {Hue[0.6330401547699546, 1, 1], LineBox[{217, 218}]}, 
                    {Hue[0.9793681228383752, 1, 1], LineBox[{218, 219}]}, 
                    {Hue[0.31651445767840364`, 1, 1], LineBox[{219, 220}]}, 
                    {Hue[0.6077541122814277, 1, 1], LineBox[{220, 221}]}, 
                    {Hue[0.8802124139644494, 1, 1], LineBox[{221, 222}]}, 
                    {Hue[0.14828103651834512`, 1, 1], LineBox[{222, 223}]}, 
                    {Hue[0.4157813282383809, 1, 1], LineBox[{223, 224}]}, 
                    {Hue[0.6821139026892058, 1, 1], LineBox[{224, 225}]}, 
                    {Hue[0.9456040737426483, 1, 1], LineBox[{225, 226}]}, 
                    {Hue[0.20528754302196955`, 1, 1], LineBox[{226, 227}]}, 
                    {Hue[0.46120974183242636`, 1, 1], LineBox[{227, 228}]}, 
                    {Hue[0.7139993425376341, 1, 1], LineBox[{228, 229}]}, 
                    {Hue[0.9644555293237684, 1, 1], LineBox[{229, 230}]}, 
                    {Hue[0.2133525195580862, 1, 1], LineBox[{230, 231}]}, 
                    {Hue[0.4613964047250365, 1, 1], LineBox[{231, 232}]}, 
                    {Hue[0.7092509138974926, 1, 1], LineBox[{232, 233}]}, 
                    {Hue[0.9575856208274757, 1, 1], LineBox[{233, 234}]}, 
                    {Hue[0.20712353313556747`, 1, 1], LineBox[{234, 235}]}, 
                    {Hue[0.4586661154329677, 1, 1], LineBox[{235, 236}]}, 
                    {Hue[0.7130546309764122, 1, 1], LineBox[{236, 237}]}, 
                    {Hue[0.9709943653608248, 1, 1], LineBox[{237, 238}]}, 
                    {Hue[0.23267334689881333`, 1, 1], LineBox[{238, 239}]}, 
                    {Hue[0.49727980286706513`, 1, 1], LineBox[{239, 240}]}, 
                    {Hue[0.7628869304352861, 1, 1], LineBox[{240, 241}]}, 
                    {Hue[0.02716246416619776, 1, 1], LineBox[{241, 242}]}, 
                    {Hue[0.28845986121524614`, 1, 1], LineBox[{242, 243}]}, 
                    {Hue[0.5463100846568116, 1, 1], LineBox[{243, 244}]}, 
                    {Hue[0.8011400052763996, 1, 1], LineBox[{244, 245}]}, 
                    {Hue[0.053766015412985584`, 1, 1], LineBox[{245, 246}]}, 
                    {Hue[0.3050462366515147, 1, 1], LineBox[{246, 247}]}, 
                    {Hue[0.555729431609835, 1, 1], LineBox[{247, 248}]}, 
                    {Hue[0.8064210755015563, 1, 1], LineBox[{248, 249}]}, 
                    {Hue[0.05759929417499629, 1, 1], LineBox[{249, 250}]}, 
                    {Hue[0.3096448548061538, 1, 1], LineBox[{250, 251}]}}}},
                    VertexColors->{
                    Hue[0.7796801289652671, 1, 1], 
                    Hue[0.6407245824031771, 1, 1], 
                    Hue[0.5503816896589118, 1, 1], 
                    Hue[0.46693777518468293`, 1, 1], 
                    Hue[0.34337602294665853`, 1, 1], 
                    Hue[0.2800240470435314, 1, 1], 
                    Hue[0.19178230855508877`, 1, 1], 
                    Hue[0.09597816212744277, 1, 1], 
                    Hue[0.03441314240637002, 1, 1], 
                    Hue[0.9619307728034987, 1, 1], 
                    Hue[0.8556957513387586, 1, 1], 
                    Hue[0.7617493391570483, 1, 1], 
                    Hue[0.6649750661560606, 1, 1], 
                    Hue[0.6003782106730116, 1, 1], 
                    Hue[0.5016480547751627, 1, 1], 
                    Hue[0.40821669939596, 1, 1], 
                    Hue[0.3228877865793172, 1, 1], 
                    Hue[0.2545692858199149, 1, 1], 
                    Hue[0.16962359970764626`, 1, 1], 
                    Hue[0.056865225111496887`, 1, 1], 
                    Hue[0.9539397202584826, 1, 1], 
                    Hue[0.7837294353244291, 1, 1], 
                    Hue[0.6953650268548688, 1, 1], 
                    Hue[0.6305218286702476, 1, 1], 
                    Hue[0.6528261996783604, 1, 1], 
                    Hue[0.7458918504427492, 1, 1], 
                    Hue[0.92126851876369, 1, 1], 
                    Hue[0.9782053362672661, 1, 1], 
                    Hue[0.020789729978857467`, 1, 1], 
                    Hue[0.07110836451469632, 1, 1], 
                    Hue[0.13731102313054688`, 1, 1], 
                    Hue[0.2237270578911424, 1, 1], 
                    Hue[0.308233058166907, 1, 1], 
                    Hue[0.37858341969070897`, 1, 1], 
                    Hue[0.44582144082943587`, 1, 1], 
                    Hue[0.5171911046750017, 1, 1], 
                    Hue[0.5903988201832231, 1, 1], 
                    Hue[0.6664515169027294, 1, 1], 
                    Hue[0.7432363389081607, 1, 1], 
                    Hue[0.8184062390390154, 1, 1], 
                    Hue[0.8937470175078367, 1, 1], 
                    Hue[0.9698398309055655, 1, 1], 
                    Hue[0.046820964778370125`, 1, 1], 
                    Hue[0.12449143522202596`, 1, 1], 
                    Hue[0.20232629915042974`, 1, 1], 
                    Hue[0.2805663074213705, 1, 1], 
                    Hue[0.3590896587353866, 1, 1], 
                    Hue[0.4382390460728896, 1, 1], 
                    Hue[0.517902737197748, 1, 1], 
                    Hue[0.5980097653972472, 1, 1], 
                    Hue[0.6785327051929539, 1, 1], 
                    Hue[0.7595095736388215, 1, 1], 
                    Hue[0.8409610804245629, 1, 1], 
                    Hue[0.9228629740322556, 1, 1], 
                    Hue[0.005253870251030078, 1, 1], 
                    Hue[0.0881095112778214, 1, 1], 
                    Hue[0.17139434226227557`, 1, 1], 
                    Hue[0.2551463423565107, 1, 1], 
                    Hue[0.33933976584552206`, 1, 1], 
                    Hue[0.4239966318214204, 1, 1], 
                    Hue[0.5091224007223621, 1, 1], 
                    Hue[0.5946966107789967, 1, 1], 
                    Hue[0.6807271939580889, 1, 1], 
                    Hue[0.7672101480855866, 1, 1], 
                    Hue[0.8541512225764117, 1, 1], 
                    Hue[0.941550181896359, 1, 1], 
                    Hue[0.029403984208433615`, 1, 1], 
                    Hue[0.11770952561423882`, 1, 1], 
                    Hue[0.20646973711035155`, 1, 1], 
                    Hue[0.295683004779304, 1, 1], 
                    Hue[0.3853477154083707, 1, 1], 
                    Hue[0.4754668125881876, 1, 1], 
                    Hue[0.5660352585959618, 1, 1], 
                    Hue[0.6570548476475564, 1, 1], 
                    Hue[0.7485248677659424, 1, 1], 
                    Hue[0.8404436664280217, 1, 1], 
                    Hue[0.9328115568614501, 1, 1], 
                    Hue[0.025625635071901066`, 1, 1], 
                    Hue[0.11888645287894296`, 1, 1], 
                    Hue[0.21259272180999705`, 1, 1], 
                    Hue[0.3067437454522678, 1, 1], 
                    Hue[0.4013375289297376, 1, 1], 
                    Hue[0.49637135639775043`, 1, 1], 
                    Hue[0.5918450111035345, 1, 1], 
                    Hue[0.6877562589564038, 1, 1], 
                    Hue[0.78410256268569, 1, 1], 
                    Hue[0.8808823473932245, 1, 1], 
                    Hue[0.9780911430522481, 1, 1], 
                    Hue[0.07572664367055548, 1, 1], 
                    Hue[0.1737853758905005, 1, 1], 
                    Hue[0.272262768039806, 1, 1], 
                    Hue[0.371153556618099, 1, 1], 
                    Hue[0.4704529220778693, 1, 1], 
                    Hue[0.5701542763470691, 1, 1], 
                    Hue[0.6702500724172249, 1, 1], 
                    Hue[0.7707317610985269, 1, 1], 
                    Hue[0.8715892246887066, 1, 1], 
                    Hue[0.9728114871992333, 1, 1], 
                    Hue[0.0743857178097356, 1, 1], 
                    Hue[0.17629597248002574`, 1, 1], 
                    Hue[0.2785251813360693, 1, 1], 
                    Hue[0.38105343827909777`, 1, 1], 
                    Hue[0.4838574827290812, 1, 1], 
                    Hue[0.5869100103646415, 1, 1], 
                    Hue[0.6901806989907271, 1, 1], 
                    Hue[0.7936343981668283, 1, 1], 
                    Hue[0.89723070358846, 1, 1], 
                    Hue[0.0009237190342984455, 1, 1], 
                    Hue[0.10466113495925189`, 1, 1], 
                    Hue[0.20838404049157222`, 1, 1], 
                    Hue[0.3120259981560382, 1, 1], 
                    Hue[0.4155125941036146, 1, 1], 
                    Hue[0.5187618099741917, 1, 1], 
                    Hue[0.6216818288821319, 1, 1], 
                    Hue[0.7241743766947099, 1, 1], 
                    Hue[0.8261341567272162, 1, 1], 
                    Hue[0.9274483379521479, 1, 1], 
                    Hue[0.027999913112641616`, 1, 1], 
                    Hue[0.12767141780559033`, 1, 1], 
                    Hue[0.226347300174275, 1, 1], 
                    Hue[0.3239190041183996, 1, 1], 
                    Hue[0.42028976211005264`, 1, 1], 
                    Hue[0.5153815945444176, 1, 1], 
                    Hue[0.6091439972339776, 1, 1], 
                    Hue[0.7015583727446617, 1, 1], 
                    Hue[0.7926410197746079, 1, 1], 
                    Hue[0.8824458895514251, 1, 1], 
                    Hue[0.9710638880548629, 1, 1], 
                    Hue[0.05861534812403368, 1, 1], 
                    Hue[0.14524236947836713`, 1, 1], 
                    Hue[0.2311013783598076, 1, 1], 
                    Hue[0.3163582834028976, 1, 1], 
                    Hue[0.4011833062789287, 1, 1], 
                    Hue[0.48573932037648593`, 1, 1], 
                    Hue[0.5701759410951197, 1, 1], 
                    Hue[0.6546269333325413, 1, 1], 
                    Hue[0.7392210534650951, 1, 1], 
                    Hue[0.8240822234173037, 1, 1], 
                    Hue[0.9093113235791046, 1, 1], 
                    Hue[0.9949861214641248, 1, 1], 
                    Hue[0.08117017774335357, 1, 1], 
                    Hue[0.16791383557109385`, 1, 1], 
                    Hue[0.25530021949010917`, 1, 1], 
                    Hue[0.34343541671925093`, 1, 1], 
                    Hue[0.43231232279874066`, 1, 1], 
                    Hue[0.5218524601207256, 1, 1], 
                    Hue[0.612173062059559, 1, 1], 
                    Hue[0.7034698819664108, 1, 1], 
                    Hue[0.7955898813013543, 1, 1], 
                    Hue[0.888364574202874, 1, 1], 
                    Hue[0.9822000502315932, 1, 1], 
                    Hue[0.0771550576325835, 1, 1], 
                    Hue[0.1720959120088873, 1, 1], 
                    Hue[0.26688998366448946`, 1, 1], 
                    Hue[0.3639293449103173, 1, 1], 
                    Hue[0.4634940195293576, 1, 1], 
                    Hue[0.5603649207077858, 1, 1], 
                    Hue[0.6519246304843664, 1, 1], 
                    Hue[0.7495382230954826, 1, 1], 
                    Hue[0.8643080095757046, 1, 1], 
                    Hue[0.9906253074833521, 1, 1], 
                    Hue[0.10779399475058227`, 1, 1], 
                    Hue[0.1661984781166469, 1, 1], 
                    Hue[0.2013592736498602, 1, 1], 
                    Hue[0.32046871065631627`, 1, 1], 
                    Hue[0.4756075296246733, 1, 1], 
                    Hue[0.6385764348360206, 1, 1], 
                    Hue[0.7869345435130637, 1, 1], 
                    Hue[0.8313036715488447, 1, 1], 
                    Hue[0.8287660083714148, 1, 1], 
                    Hue[0.9760417001103902, 1, 1], 
                    Hue[0.17203733354249606`, 1, 1], 
                    Hue[0.41546928442185527`, 1, 1], 
                    Hue[0.7130492258316945, 1, 1], 
                    Hue[0.013352993052113508`, 1, 1], 
                    Hue[0.2865809602909467, 1, 1], 
                    Hue[0.5485481811412078, 1, 1], 
                    Hue[0.8100483978103337, 1, 1], 
                    Hue[0.07438821094553806, 1, 1], 
                    Hue[0.34079459211773067`, 1, 1], 
                    Hue[0.6067300720056761, 1, 1], 
                    Hue[0.8699265236196507, 1, 1], 
                    Hue[0.12928216875586565`, 1, 1], 
                    Hue[0.3844879401823965, 1, 1], 
                    Hue[0.6353011644887285, 1, 1], 
                    Hue[0.8810951367672261, 1, 1], 
                    Hue[0.12083724148470129`, 1, 1], 
                    Hue[0.35359810181285867`, 1, 1], 
                    Hue[0.5797037414262387, 1, 1], 
                    Hue[0.801927284818749, 1, 1], 
                    Hue[0.02490207809077299, 1, 1], 
                    Hue[0.2524952787854635, 1, 1], 
                    Hue[0.4861575156782732, 1, 1], 
                    Hue[0.7255279616649843, 1, 1], 
                    Hue[0.9696088981681895, 1, 1], 
                    Hue[0.21741244056167142`, 1, 1], 
                    Hue[0.46813755920083117`, 1, 1], 
                    Hue[0.7211432166767873, 1, 1], 
                    Hue[0.975875284824123, 1, 1], 
                    Hue[0.23181597238659138`, 1, 1], 
                    Hue[0.48848723203835615`, 1, 1], 
                    Hue[0.745497138351656, 1, 1], 
                    Hue[0.0025923563209303813`, 1, 1], 
                    Hue[0.25967466350231044`, 1, 1], 
                    Hue[0.5167645910330184, 1, 1], 
                    Hue[0.7739220318465841, 1, 1], 
                    Hue[0.031150257131776243`, 1, 1], 
                    Hue[0.2883194585058068, 1, 1], 
                    Hue[0.5451448055773551, 1, 1], 
                    Hue[0.801241512943925, 1, 1], 
                    Hue[0.05625312288155043, 1, 1], 
                    Hue[0.31002956423589995`, 1, 1], 
                    Hue[0.5628570448388254, 1, 1], 
                    Hue[0.8158451729427638, 1, 1], 
                    Hue[0.07184273217758473, 1, 1], 
                    Hue[0.33804700588730185`, 1, 1], 
                    Hue[0.6330401547699546, 1, 1], 
                    Hue[0.9793681228383752, 1, 1], 
                    Hue[0.31651445767840364`, 1, 1], 
                    Hue[0.6077541122814277, 1, 1], 
                    Hue[0.8802124139644494, 1, 1], 
                    Hue[0.14828103651834512`, 1, 1], 
                    Hue[0.4157813282383809, 1, 1], 
                    Hue[0.6821139026892058, 1, 1], 
                    Hue[0.9456040737426483, 1, 1], 
                    Hue[0.20528754302196955`, 1, 1], 
                    Hue[0.46120974183242636`, 1, 1], 
                    Hue[0.7139993425376341, 1, 1], 
                    Hue[0.9644555293237684, 1, 1], 
                    Hue[0.2133525195580862, 1, 1], 
                    Hue[0.4613964047250365, 1, 1], 
                    Hue[0.7092509138974926, 1, 1], 
                    Hue[0.9575856208274757, 1, 1], 
                    Hue[0.20712353313556747`, 1, 1], 
                    Hue[0.4586661154329677, 1, 1], 
                    Hue[0.7130546309764122, 1, 1], 
                    Hue[0.9709943653608248, 1, 1], 
                    Hue[0.23267334689881333`, 1, 1], 
                    Hue[0.49727980286706513`, 1, 1], 
                    Hue[0.7628869304352861, 1, 1], 
                    Hue[0.02716246416619776, 1, 1], 
                    Hue[0.28845986121524614`, 1, 1], 
                    Hue[0.5463100846568116, 1, 1], 
                    Hue[0.8011400052763996, 1, 1], 
                    Hue[0.053766015412985584`, 1, 1], 
                    Hue[0.3050462366515147, 1, 1], 
                    Hue[0.555729431609835, 1, 1], 
                    Hue[0.8064210755015563, 1, 1], 
                    Hue[0.05759929417499629, 1, 1], 
                    Hue[0.3096448548061538, 1, 1], 
                    Hue[0.562871819288882, 1, 1], 
                    Hue[0.7796801289652671, 1, 1], 
                    Hue[0.6407245824031771, 1, 1], 
                    Hue[0.6407245824031771, 1, 1], 
                    Hue[0.5503816896589118, 1, 1], 
                    Hue[0.5503816896589118, 1, 1], 
                    Hue[0.46693777518468293`, 1, 1], 
                    Hue[0.46693777518468293`, 1, 1], 
                    Hue[0.34337602294665853`, 1, 1], 
                    Hue[0.34337602294665853`, 1, 1], 
                    Hue[0.2800240470435314, 1, 1], 
                    Hue[0.2800240470435314, 1, 1], 
                    Hue[0.19178230855508877`, 1, 1], 
                    Hue[0.19178230855508877`, 1, 1], 
                    Hue[0.09597816212744277, 1, 1], 
                    Hue[0.09597816212744277, 1, 1], 
                    Hue[0.03441314240637002, 1, 1], 
                    Hue[0.03441314240637002, 1, 1], 
                    Hue[0.9619307728034987, 1, 1], 
                    Hue[0.9619307728034987, 1, 1], 
                    Hue[0.8556957513387586, 1, 1], 
                    Hue[0.8556957513387586, 1, 1], 
                    Hue[0.7617493391570483, 1, 1], 
                    Hue[0.7617493391570483, 1, 1], 
                    Hue[0.6649750661560606, 1, 1], 
                    Hue[0.6649750661560606, 1, 1], 
                    Hue[0.6003782106730116, 1, 1], 
                    Hue[0.6003782106730116, 1, 1], 
                    Hue[0.5016480547751627, 1, 1], 
                    Hue[0.5016480547751627, 1, 1], 
                    Hue[0.40821669939596, 1, 1], 
                    Hue[0.40821669939596, 1, 1], 
                    Hue[0.3228877865793172, 1, 1], 
                    Hue[0.3228877865793172, 1, 1], 
                    Hue[0.2545692858199149, 1, 1], 
                    Hue[0.2545692858199149, 1, 1], 
                    Hue[0.16962359970764626`, 1, 1], 
                    Hue[0.16962359970764626`, 1, 1], 
                    Hue[0.056865225111496887`, 1, 1], 
                    Hue[0.056865225111496887`, 1, 1], 
                    Hue[0.9539397202584826, 1, 1], 
                    Hue[0.9539397202584826, 1, 1], 
                    Hue[0.7837294353244291, 1, 1], 
                    Hue[0.7837294353244291, 1, 1], 
                    Hue[0.6953650268548688, 1, 1], 
                    Hue[0.6953650268548688, 1, 1], 
                    Hue[0.6305218286702476, 1, 1], 
                    Hue[0.6305218286702476, 1, 1], 
                    Hue[0.6528261996783604, 1, 1], 
                    Hue[0.6528261996783604, 1, 1], 
                    Hue[0.7458918504427492, 1, 1], 
                    Hue[0.7458918504427492, 1, 1], 
                    Hue[0.92126851876369, 1, 1], 
                    Hue[0.92126851876369, 1, 1], 
                    Hue[0.9782053362672661, 1, 1], 
                    Hue[0.9782053362672661, 1, 1], 
                    Hue[0.020789729978857467`, 1, 1], 
                    Hue[0.020789729978857467`, 1, 1], 
                    Hue[0.07110836451469632, 1, 1], 
                    Hue[0.07110836451469632, 1, 1], 
                    Hue[0.13731102313054688`, 1, 1], 
                    Hue[0.13731102313054688`, 1, 1], 
                    Hue[0.2237270578911424, 1, 1], 
                    Hue[0.2237270578911424, 1, 1], 
                    Hue[0.308233058166907, 1, 1], 
                    Hue[0.308233058166907, 1, 1], 
                    Hue[0.37858341969070897`, 1, 1], 
                    Hue[0.37858341969070897`, 1, 1], 
                    Hue[0.44582144082943587`, 1, 1], 
                    Hue[0.44582144082943587`, 1, 1], 
                    Hue[0.5171911046750017, 1, 1], 
                    Hue[0.5171911046750017, 1, 1], 
                    Hue[0.5903988201832231, 1, 1], 
                    Hue[0.5903988201832231, 1, 1], 
                    Hue[0.6664515169027294, 1, 1], 
                    Hue[0.6664515169027294, 1, 1], 
                    Hue[0.7432363389081607, 1, 1], 
                    Hue[0.7432363389081607, 1, 1], 
                    Hue[0.8184062390390154, 1, 1], 
                    Hue[0.8184062390390154, 1, 1], 
                    Hue[0.8937470175078367, 1, 1], 
                    Hue[0.8937470175078367, 1, 1], 
                    Hue[0.9698398309055655, 1, 1], 
                    Hue[0.9698398309055655, 1, 1], 
                    Hue[0.046820964778370125`, 1, 1], 
                    Hue[0.046820964778370125`, 1, 1], 
                    Hue[0.12449143522202596`, 1, 1], 
                    Hue[0.12449143522202596`, 1, 1], 
                    Hue[0.20232629915042974`, 1, 1], 
                    Hue[0.20232629915042974`, 1, 1], 
                    Hue[0.2805663074213705, 1, 1], 
                    Hue[0.2805663074213705, 1, 1], 
                    Hue[0.3590896587353866, 1, 1], 
                    Hue[0.3590896587353866, 1, 1], 
                    Hue[0.4382390460728896, 1, 1], 
                    Hue[0.4382390460728896, 1, 1], 
                    Hue[0.517902737197748, 1, 1], 
                    Hue[0.517902737197748, 1, 1], 
                    Hue[0.5980097653972472, 1, 1], 
                    Hue[0.5980097653972472, 1, 1], 
                    Hue[0.6785327051929539, 1, 1], 
                    Hue[0.6785327051929539, 1, 1], 
                    Hue[0.7595095736388215, 1, 1], 
                    Hue[0.7595095736388215, 1, 1], 
                    Hue[0.8409610804245629, 1, 1], 
                    Hue[0.8409610804245629, 1, 1], 
                    Hue[0.9228629740322556, 1, 1], 
                    Hue[0.9228629740322556, 1, 1], 
                    Hue[0.005253870251030078, 1, 1], 
                    Hue[0.005253870251030078, 1, 1], 
                    Hue[0.0881095112778214, 1, 1], 
                    Hue[0.0881095112778214, 1, 1], 
                    Hue[0.17139434226227557`, 1, 1], 
                    Hue[0.17139434226227557`, 1, 1], 
                    Hue[0.2551463423565107, 1, 1], 
                    Hue[0.2551463423565107, 1, 1], 
                    Hue[0.33933976584552206`, 1, 1], 
                    Hue[0.33933976584552206`, 1, 1], 
                    Hue[0.4239966318214204, 1, 1], 
                    Hue[0.4239966318214204, 1, 1], 
                    Hue[0.5091224007223621, 1, 1], 
                    Hue[0.5091224007223621, 1, 1], 
                    Hue[0.5946966107789967, 1, 1], 
                    Hue[0.5946966107789967, 1, 1], 
                    Hue[0.6807271939580889, 1, 1], 
                    Hue[0.6807271939580889, 1, 1], 
                    Hue[0.7672101480855866, 1, 1], 
                    Hue[0.7672101480855866, 1, 1], 
                    Hue[0.8541512225764117, 1, 1], 
                    Hue[0.8541512225764117, 1, 1], 
                    Hue[0.941550181896359, 1, 1], 
                    Hue[0.941550181896359, 1, 1], 
                    Hue[0.029403984208433615`, 1, 1], 
                    Hue[0.029403984208433615`, 1, 1], 
                    Hue[0.11770952561423882`, 1, 1], 
                    Hue[0.11770952561423882`, 1, 1], 
                    Hue[0.20646973711035155`, 1, 1], 
                    Hue[0.20646973711035155`, 1, 1], 
                    Hue[0.295683004779304, 1, 1], 
                    Hue[0.295683004779304, 1, 1], 
                    Hue[0.3853477154083707, 1, 1], 
                    Hue[0.3853477154083707, 1, 1], 
                    Hue[0.4754668125881876, 1, 1], 
                    Hue[0.4754668125881876, 1, 1], 
                    Hue[0.5660352585959618, 1, 1], 
                    Hue[0.5660352585959618, 1, 1], 
                    Hue[0.6570548476475564, 1, 1], 
                    Hue[0.6570548476475564, 1, 1], 
                    Hue[0.7485248677659424, 1, 1], 
                    Hue[0.7485248677659424, 1, 1], 
                    Hue[0.8404436664280217, 1, 1], 
                    Hue[0.8404436664280217, 1, 1], 
                    Hue[0.9328115568614501, 1, 1], 
                    Hue[0.9328115568614501, 1, 1], 
                    Hue[0.025625635071901066`, 1, 1], 
                    Hue[0.025625635071901066`, 1, 1], 
                    Hue[0.11888645287894296`, 1, 1], 
                    Hue[0.11888645287894296`, 1, 1], 
                    Hue[0.21259272180999705`, 1, 1], 
                    Hue[0.21259272180999705`, 1, 1], 
                    Hue[0.3067437454522678, 1, 1], 
                    Hue[0.3067437454522678, 1, 1], 
                    Hue[0.4013375289297376, 1, 1], 
                    Hue[0.4013375289297376, 1, 1], 
                    Hue[0.49637135639775043`, 1, 1], 
                    Hue[0.49637135639775043`, 1, 1], 
                    Hue[0.5918450111035345, 1, 1], 
                    Hue[0.5918450111035345, 1, 1], 
                    Hue[0.6877562589564038, 1, 1], 
                    Hue[0.6877562589564038, 1, 1], 
                    Hue[0.78410256268569, 1, 1], 
                    Hue[0.78410256268569, 1, 1], 
                    Hue[0.8808823473932245, 1, 1], 
                    Hue[0.8808823473932245, 1, 1], 
                    Hue[0.9780911430522481, 1, 1], 
                    Hue[0.9780911430522481, 1, 1], 
                    Hue[0.07572664367055548, 1, 1], 
                    Hue[0.07572664367055548, 1, 1], 
                    Hue[0.1737853758905005, 1, 1], 
                    Hue[0.1737853758905005, 1, 1], 
                    Hue[0.272262768039806, 1, 1], 
                    Hue[0.272262768039806, 1, 1], 
                    Hue[0.371153556618099, 1, 1], 
                    Hue[0.371153556618099, 1, 1], 
                    Hue[0.4704529220778693, 1, 1], 
                    Hue[0.4704529220778693, 1, 1], 
                    Hue[0.5701542763470691, 1, 1], 
                    Hue[0.5701542763470691, 1, 1], 
                    Hue[0.6702500724172249, 1, 1], 
                    Hue[0.6702500724172249, 1, 1], 
                    Hue[0.7707317610985269, 1, 1], 
                    Hue[0.7707317610985269, 1, 1], 
                    Hue[0.8715892246887066, 1, 1], 
                    Hue[0.8715892246887066, 1, 1], 
                    Hue[0.9728114871992333, 1, 1], 
                    Hue[0.9728114871992333, 1, 1], 
                    Hue[0.0743857178097356, 1, 1], 
                    Hue[0.0743857178097356, 1, 1], 
                    Hue[0.17629597248002574`, 1, 1], 
                    Hue[0.17629597248002574`, 1, 1], 
                    Hue[0.2785251813360693, 1, 1], 
                    Hue[0.2785251813360693, 1, 1], 
                    Hue[0.38105343827909777`, 1, 1], 
                    Hue[0.38105343827909777`, 1, 1], 
                    Hue[0.4838574827290812, 1, 1], 
                    Hue[0.4838574827290812, 1, 1], 
                    Hue[0.5869100103646415, 1, 1], 
                    Hue[0.5869100103646415, 1, 1], 
                    Hue[0.6901806989907271, 1, 1], 
                    Hue[0.6901806989907271, 1, 1], 
                    Hue[0.7936343981668283, 1, 1], 
                    Hue[0.7936343981668283, 1, 1], 
                    Hue[0.89723070358846, 1, 1], 
                    Hue[0.89723070358846, 1, 1], 
                    Hue[0.0009237190342984455, 1, 1], 
                    Hue[0.0009237190342984455, 1, 1], 
                    Hue[0.10466113495925189`, 1, 1], 
                    Hue[0.10466113495925189`, 1, 1], 
                    Hue[0.20838404049157222`, 1, 1], 
                    Hue[0.20838404049157222`, 1, 1], 
                    Hue[0.3120259981560382, 1, 1], 
                    Hue[0.3120259981560382, 1, 1], 
                    Hue[0.4155125941036146, 1, 1], 
                    Hue[0.4155125941036146, 1, 1], 
                    Hue[0.5187618099741917, 1, 1], 
                    Hue[0.5187618099741917, 1, 1], 
                    Hue[0.6216818288821319, 1, 1], 
                    Hue[0.6216818288821319, 1, 1], 
                    Hue[0.7241743766947099, 1, 1], 
                    Hue[0.7241743766947099, 1, 1], 
                    Hue[0.8261341567272162, 1, 1], 
                    Hue[0.8261341567272162, 1, 1], 
                    Hue[0.9274483379521479, 1, 1], 
                    Hue[0.9274483379521479, 1, 1], 
                    Hue[0.027999913112641616`, 1, 1], 
                    Hue[0.027999913112641616`, 1, 1], 
                    Hue[0.12767141780559033`, 1, 1], 
                    Hue[0.12767141780559033`, 1, 1], 
                    Hue[0.226347300174275, 1, 1], 
                    Hue[0.226347300174275, 1, 1], 
                    Hue[0.3239190041183996, 1, 1], 
                    Hue[0.3239190041183996, 1, 1], 
                    Hue[0.42028976211005264`, 1, 1], 
                    Hue[0.42028976211005264`, 1, 1], 
                    Hue[0.5153815945444176, 1, 1], 
                    Hue[0.5153815945444176, 1, 1], 
                    Hue[0.6091439972339776, 1, 1], 
                    Hue[0.6091439972339776, 1, 1], 
                    Hue[0.7015583727446617, 1, 1], 
                    Hue[0.7015583727446617, 1, 1], 
                    Hue[0.7926410197746079, 1, 1], 
                    Hue[0.7926410197746079, 1, 1], 
                    Hue[0.8824458895514251, 1, 1], 
                    Hue[0.8824458895514251, 1, 1], 
                    Hue[0.9710638880548629, 1, 1], 
                    Hue[0.9710638880548629, 1, 1], 
                    Hue[0.05861534812403368, 1, 1], 
                    Hue[0.05861534812403368, 1, 1], 
                    Hue[0.14524236947836713`, 1, 1], 
                    Hue[0.14524236947836713`, 1, 1], 
                    Hue[0.2311013783598076, 1, 1], 
                    Hue[0.2311013783598076, 1, 1], 
                    Hue[0.3163582834028976, 1, 1], 
                    Hue[0.3163582834028976, 1, 1], 
                    Hue[0.4011833062789287, 1, 1], 
                    Hue[0.4011833062789287, 1, 1], 
                    Hue[0.48573932037648593`, 1, 1], 
                    Hue[0.48573932037648593`, 1, 1], 
                    Hue[0.5701759410951197, 1, 1], 
                    Hue[0.5701759410951197, 1, 1], 
                    Hue[0.6546269333325413, 1, 1], 
                    Hue[0.6546269333325413, 1, 1], 
                    Hue[0.7392210534650951, 1, 1], 
                    Hue[0.7392210534650951, 1, 1], 
                    Hue[0.8240822234173037, 1, 1], 
                    Hue[0.8240822234173037, 1, 1], 
                    Hue[0.9093113235791046, 1, 1], 
                    Hue[0.9093113235791046, 1, 1], 
                    Hue[0.9949861214641248, 1, 1], 
                    Hue[0.9949861214641248, 1, 1], 
                    Hue[0.08117017774335357, 1, 1], 
                    Hue[0.08117017774335357, 1, 1], 
                    Hue[0.16791383557109385`, 1, 1], 
                    Hue[0.16791383557109385`, 1, 1], 
                    Hue[0.25530021949010917`, 1, 1], 
                    Hue[0.25530021949010917`, 1, 1], 
                    Hue[0.34343541671925093`, 1, 1], 
                    Hue[0.34343541671925093`, 1, 1], 
                    Hue[0.43231232279874066`, 1, 1], 
                    Hue[0.43231232279874066`, 1, 1], 
                    Hue[0.5218524601207256, 1, 1], 
                    Hue[0.5218524601207256, 1, 1], 
                    Hue[0.612173062059559, 1, 1], 
                    Hue[0.612173062059559, 1, 1], 
                    Hue[0.7034698819664108, 1, 1], 
                    Hue[0.7034698819664108, 1, 1], 
                    Hue[0.7955898813013543, 1, 1], 
                    Hue[0.7955898813013543, 1, 1], 
                    Hue[0.888364574202874, 1, 1], 
                    Hue[0.888364574202874, 1, 1], 
                    Hue[0.9822000502315932, 1, 1], 
                    Hue[0.9822000502315932, 1, 1], 
                    Hue[0.0771550576325835, 1, 1], 
                    Hue[0.0771550576325835, 1, 1], 
                    Hue[0.1720959120088873, 1, 1], 
                    Hue[0.1720959120088873, 1, 1], 
                    Hue[0.26688998366448946`, 1, 1], 
                    Hue[0.26688998366448946`, 1, 1], 
                    Hue[0.3639293449103173, 1, 1], 
                    Hue[0.3639293449103173, 1, 1], 
                    Hue[0.4634940195293576, 1, 1], 
                    Hue[0.4634940195293576, 1, 1], 
                    Hue[0.5603649207077858, 1, 1], 
                    Hue[0.5603649207077858, 1, 1], 
                    Hue[0.6519246304843664, 1, 1], 
                    Hue[0.6519246304843664, 1, 1], 
                    Hue[0.7495382230954826, 1, 1], 
                    Hue[0.7495382230954826, 1, 1], 
                    Hue[0.8643080095757046, 1, 1], 
                    Hue[0.8643080095757046, 1, 1], 
                    Hue[0.9906253074833521, 1, 1], 
                    Hue[0.9906253074833521, 1, 1], 
                    Hue[0.10779399475058227`, 1, 1], 
                    Hue[0.10779399475058227`, 1, 1], 
                    Hue[0.1661984781166469, 1, 1], 
                    Hue[0.1661984781166469, 1, 1], 
                    Hue[0.2013592736498602, 1, 1], 
                    Hue[0.2013592736498602, 1, 1], 
                    Hue[0.32046871065631627`, 1, 1], 
                    Hue[0.32046871065631627`, 1, 1], 
                    Hue[0.4756075296246733, 1, 1], 
                    Hue[0.4756075296246733, 1, 1], 
                    Hue[0.6385764348360206, 1, 1], 
                    Hue[0.6385764348360206, 1, 1], 
                    Hue[0.7869345435130637, 1, 1], 
                    Hue[0.7869345435130637, 1, 1], 
                    Hue[0.8313036715488447, 1, 1], 
                    Hue[0.8313036715488447, 1, 1], 
                    Hue[0.8287660083714148, 1, 1], 
                    Hue[0.8287660083714148, 1, 1], 
                    Hue[0.9760417001103902, 1, 1], 
                    Hue[0.9760417001103902, 1, 1], 
                    Hue[0.17203733354249606`, 1, 1], 
                    Hue[0.17203733354249606`, 1, 1], 
                    Hue[0.41546928442185527`, 1, 1], 
                    Hue[0.41546928442185527`, 1, 1], 
                    Hue[0.7130492258316945, 1, 1], 
                    Hue[0.7130492258316945, 1, 1], 
                    Hue[0.013352993052113508`, 1, 1], 
                    Hue[0.013352993052113508`, 1, 1], 
                    Hue[0.2865809602909467, 1, 1], 
                    Hue[0.2865809602909467, 1, 1], 
                    Hue[0.5485481811412078, 1, 1], 
                    Hue[0.5485481811412078, 1, 1], 
                    Hue[0.8100483978103337, 1, 1], 
                    Hue[0.8100483978103337, 1, 1], 
                    Hue[0.07438821094553806, 1, 1], 
                    Hue[0.07438821094553806, 1, 1], 
                    Hue[0.34079459211773067`, 1, 1], 
                    Hue[0.34079459211773067`, 1, 1], 
                    Hue[0.6067300720056761, 1, 1], 
                    Hue[0.6067300720056761, 1, 1], 
                    Hue[0.8699265236196507, 1, 1], 
                    Hue[0.8699265236196507, 1, 1], 
                    Hue[0.12928216875586565`, 1, 1], 
                    Hue[0.12928216875586565`, 1, 1], 
                    Hue[0.3844879401823965, 1, 1], 
                    Hue[0.3844879401823965, 1, 1], 
                    Hue[0.6353011644887285, 1, 1], 
                    Hue[0.6353011644887285, 1, 1], 
                    Hue[0.8810951367672261, 1, 1], 
                    Hue[0.8810951367672261, 1, 1], 
                    Hue[0.12083724148470129`, 1, 1], 
                    Hue[0.12083724148470129`, 1, 1], 
                    Hue[0.35359810181285867`, 1, 1], 
                    Hue[0.35359810181285867`, 1, 1], 
                    Hue[0.5797037414262387, 1, 1], 
                    Hue[0.5797037414262387, 1, 1], 
                    Hue[0.801927284818749, 1, 1], 
                    Hue[0.801927284818749, 1, 1], 
                    Hue[0.02490207809077299, 1, 1], 
                    Hue[0.02490207809077299, 1, 1], 
                    Hue[0.2524952787854635, 1, 1], 
                    Hue[0.2524952787854635, 1, 1], 
                    Hue[0.4861575156782732, 1, 1], 
                    Hue[0.4861575156782732, 1, 1], 
                    Hue[0.7255279616649843, 1, 1], 
                    Hue[0.7255279616649843, 1, 1], 
                    Hue[0.9696088981681895, 1, 1], 
                    Hue[0.9696088981681895, 1, 1], 
                    Hue[0.21741244056167142`, 1, 1], 
                    Hue[0.21741244056167142`, 1, 1], 
                    Hue[0.46813755920083117`, 1, 1], 
                    Hue[0.46813755920083117`, 1, 1], 
                    Hue[0.7211432166767873, 1, 1], 
                    Hue[0.7211432166767873, 1, 1], 
                    Hue[0.975875284824123, 1, 1], 
                    Hue[0.975875284824123, 1, 1], 
                    Hue[0.23181597238659138`, 1, 1], 
                    Hue[0.23181597238659138`, 1, 1], 
                    Hue[0.48848723203835615`, 1, 1], 
                    Hue[0.48848723203835615`, 1, 1], 
                    Hue[0.745497138351656, 1, 1], 
                    Hue[0.745497138351656, 1, 1], 
                    Hue[0.0025923563209303813`, 1, 1], 
                    Hue[0.0025923563209303813`, 1, 1], 
                    Hue[0.25967466350231044`, 1, 1], 
                    Hue[0.25967466350231044`, 1, 1], 
                    Hue[0.5167645910330184, 1, 1], 
                    Hue[0.5167645910330184, 1, 1], 
                    Hue[0.7739220318465841, 1, 1], 
                    Hue[0.7739220318465841, 1, 1], 
                    Hue[0.031150257131776243`, 1, 1], 
                    Hue[0.031150257131776243`, 1, 1], 
                    Hue[0.2883194585058068, 1, 1], 
                    Hue[0.2883194585058068, 1, 1], 
                    Hue[0.5451448055773551, 1, 1], 
                    Hue[0.5451448055773551, 1, 1], 
                    Hue[0.801241512943925, 1, 1], 
                    Hue[0.801241512943925, 1, 1], 
                    Hue[0.05625312288155043, 1, 1], 
                    Hue[0.05625312288155043, 1, 1], 
                    Hue[0.31002956423589995`, 1, 1], 
                    Hue[0.31002956423589995`, 1, 1], 
                    Hue[0.5628570448388254, 1, 1], 
                    Hue[0.5628570448388254, 1, 1], 
                    Hue[0.8158451729427638, 1, 1], 
                    Hue[0.8158451729427638, 1, 1], 
                    Hue[0.07184273217758473, 1, 1], 
                    Hue[0.07184273217758473, 1, 1], 
                    Hue[0.33804700588730185`, 1, 1], 
                    Hue[0.33804700588730185`, 1, 1], 
                    Hue[0.6330401547699546, 1, 1], 
                    Hue[0.6330401547699546, 1, 1], 
                    Hue[0.9793681228383752, 1, 1], 
                    Hue[0.9793681228383752, 1, 1], 
                    Hue[0.31651445767840364`, 1, 1], 
                    Hue[0.31651445767840364`, 1, 1], 
                    Hue[0.6077541122814277, 1, 1], 
                    Hue[0.6077541122814277, 1, 1], 
                    Hue[0.8802124139644494, 1, 1], 
                    Hue[0.8802124139644494, 1, 1], 
                    Hue[0.14828103651834512`, 1, 1], 
                    Hue[0.14828103651834512`, 1, 1], 
                    Hue[0.4157813282383809, 1, 1], 
                    Hue[0.4157813282383809, 1, 1], 
                    Hue[0.6821139026892058, 1, 1], 
                    Hue[0.6821139026892058, 1, 1], 
                    Hue[0.9456040737426483, 1, 1], 
                    Hue[0.9456040737426483, 1, 1], 
                    Hue[0.20528754302196955`, 1, 1], 
                    Hue[0.20528754302196955`, 1, 1], 
                    Hue[0.46120974183242636`, 1, 1], 
                    Hue[0.46120974183242636`, 1, 1], 
                    Hue[0.7139993425376341, 1, 1], 
                    Hue[0.7139993425376341, 1, 1], 
                    Hue[0.9644555293237684, 1, 1], 
                    Hue[0.9644555293237684, 1, 1], 
                    Hue[0.2133525195580862, 1, 1], 
                    Hue[0.2133525195580862, 1, 1]}],
                   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
                   Axes->{True, None},
                   AxesOrigin->{0, 0},
                   
                   Epilog->LineBox[{{1, 1.3578616670247881`*^-6}, {
                    2, 1.7875241658911324`*^-6}, {
                    3, 1.3103993465932616`*^-6}, {
                    4, 1.8216579940536379`*^-6}, {
                    5, 1.5182456124162612`*^-6}, {
                    6, 1.5915857528156287`*^-6}, {
                    7, 1.6562529840457176`*^-6}, {
                    8, 1.6155346667930268`*^-6}, {
                    9, 1.6093333732317148`*^-6}, {10, 1.81716422545438*^-6}, {
                    11, 2.241367317592486*^-6}, {12, 2.300923295728431*^-6}, {
                    13, 2.0346675200493288`*^-6}, {
                    14, 1.9750831096706663`*^-6}, {
                    15, 2.225434568468253*^-6}, {16, 1.935483230855609*^-6}, {
                    17, 2.1734845906928817`*^-6}, {
                    18, 2.210747147632964*^-6}, {
                    19, 2.3999074824891674`*^-6}, {
                    20, 1.709320260604519*^-6}, {
                    21, 1.2232913505398706`*^-6}, {
                    22, 1.2807414685936205`*^-6}, {
                    23, 2.4656133378253376`*^-6}, {
                    24, 2.6317470425984405`*^-6}, {
                    25, 2.73191883772434*^-6}, {26, 2.161439319185387*^-6}, {
                    27, 2.9993781934814074`*^-6}, {
                    28, 5.885392164480087*^-6}, {29, 9.140637439099608*^-6}, {
                    30, 0.000012537549689778418`}, {
                    31, 0.000015125531152692758`}, {
                    32, 0.000019317912257021085`}, {
                    33, 0.000027799915899672324`}, {
                    34, 0.00004062888695791183}, {
                    35, 0.00005493732462684299}, {
                    36, 0.0000734364992144021}, {
                    37, 0.00009658428414418722}, {
                    38, 0.00012796459911777018`}, {
                    39, 0.000170813089428751}, {40, 0.00022696971291477682`},
                     {41, 0.000299636428986054}, {
                    42, 0.0003910582358853813}, {43, 0.0005084328649574035}, {
                    44, 0.0006601813774843741}, {45, 0.0008533502212744484}, {
                    46, 0.0010973566411053204`}, {
                    47, 0.0014042731783203211`}, {48, 0.001786221219027729}, {
                    49, 0.0022622360056601017`}, {
                    50, 0.0028531421052515322`}, {51, 0.003581392976911822}, {
                    52, 0.004473592560734086}, {53, 0.005560497943693344}, {
                    54, 0.006878054778466725}, {55, 0.008468844177113395}, {
                    56, 0.010378608142441953`}, {57, 0.01265858088407264}, {
                    58, 0.01536651990372765}, {59, 0.018563495254488412`}, {
                    60, 0.022318225943976984`}, {61, 0.02670515095385385}, {
                    62, 0.03180287667558553}, {63, 0.037692974885989126`}, {
                    64, 0.04446019492143582}, {65, 0.05219142104214091}, {
                    66, 0.06097357503607669}, {67, 0.07089558283135813}, {
                    68, 0.08203792902250882}, {69, 0.09447815368599492}, {
                    70, 0.10828358624555064`}, {71, 0.123512893412521}, {
                    72, 0.1402095530049863}, {73, 0.1584024435663279}, {
                    74, 0.17809894639984414`}, {75, 0.19928566276647516`}, {
                    76, 0.22192477190074908`}, {77, 0.24595194505811113`}, {
                    78, 0.2712737623793337}, {79, 0.2977683872323064}, {
                    80, 0.3252825318464745}, {81, 0.35363557733727796`}, {
                    82, 0.3826182999584437}, {83, 0.4119903692221059}, {
                    84, 0.4414879086848425}, {85, 0.470827754776116}, {
                    86, 0.49970964595311035`}, {87, 0.5278201645933404}, {
                    88, 0.5548366243419628}, {89, 0.580437763650691}, {
                    90, 0.6043074113454318}, {91, 0.6261409988330071}, {
                    92, 0.6456514554388889}, {93, 0.6625749076950869}, {
                    94, 0.6766801964750884}, {95, 0.687771076988495}, {
                    96, 0.6956898827986803}, {97, 0.7003218973983162}, {
                    98, 0.7015987452623819}, {99, 0.6995035557496568}, {
                    100, 0.6940614186036079}, {101, 0.6853459264951608}, {
                    102, 0.6734791966567298}, {103, 0.658621962544615}, {
                    104, 0.6409760879553257}, {105, 0.6207734769911902}, {
                    106, 0.5982750040816684}, {107, 0.5737647870662654}, {
                    108, 0.5475430917750653}, {109, 0.5199189419161926}, {
                    110, 0.4912039516155621}, {111, 0.4617090541035805}, {
                    112, 0.4317386345853279}, {113, 0.40158017517091327`}, {
                    114, 0.3715076168379809}, {115, 0.3417749018112787}, {
                    116, 0.31261349515941717`}, {117, 0.2842295081237318}, {
                    118, 0.25680583426915427`}, {119, 0.23050129959149365`}, {
                    120, 0.20545165508947386`}, {121, 0.18176941902956462`}, {
                    122, 0.15954718950965407`}, {123, 0.13885984045994407`}, {
                    124, 0.1197651230651645}, {125, 0.10230357089374113`}, {
                    126, 0.08649928792914117}, {127, 0.07235827507074691}, {
                    128, 0.05986317239481794}, {129, 0.048969742650728634`}, {
                    130, 0.039606133182448336`}, {131, 0.03167408322519318}, {
                    132, 0.025053002326810153`}, {
                    133, 0.019605476376695116`}, {134, 0.01518553197705968}, {
                    135, 0.011646988506975207`}, {
                    136, 0.008850222590473791}, {137, 0.006666504517610634}, {
                    138, 0.004980372895791658}, {139, 0.003691677467039283}, {
                    140, 0.0027162303507356615`}, {
                    141, 0.0019846048042396606`}, {
                    142, 0.0014406680748210495`}, {
                    143, 0.0010396299889217052`}, {
                    144, 0.0007458456831020373}, {
                    145, 0.0005318076067174114}, {
                    146, 0.00037710631583470663`}, {
                    147, 0.0002662201787648565}, {
                    148, 0.00018698485377092197`}, {
                    149, 0.0001305203043121926}, {
                    150, 0.00009068959598857076}, {
                    151, 0.0000627773097781486}, {
                    152, 0.000043106948920869545`}, {
                    153, 0.000029353360974000126`}, {
                    154, 0.000020034389591908615`}, {
                    155, 0.000013722976025379663`}, {
                    156, 9.232758252075829*^-6}, {
                    157, 6.069303133707478*^-6}, {
                    158, 4.102082951343716*^-6}, {
                    159, 2.960733311087227*^-6}, {
                    160, 2.11683677966345*^-6}, {
                    161, 1.3455872632310667`*^-6}, {
                    162, 7.057570641144632*^-7}, {
                    163, 3.4137138754889196`*^-7}, {
                    164, 2.977709987555004*^-7}, {
                    165, 2.9413439060320487`*^-7}, {
                    166, 2.279128080971359*^-7}, {
                    167, 1.3669536634937134`*^-7}, {
                    168, 6.006314143393278*^-8}, {
                    169, 1.971630225127637*^-8}, {
                    170, 2.221893726538528*^-8}, {
                    171, 2.539935454267735*^-8}, {
                    172, 2.232795126757919*^-8}, {
                    173, 1.7191890733676927`*^-8}, {
                    174, 1.4736212748019013`*^-8}, {
                    175, 1.6669375500447787`*^-8}, {
                    176, 1.9242945537034262`*^-8}, {
                    177, 2.0007141834995885`*^-8}, {
                    178, 1.891303765524477*^-8}, {
                    179, 1.6775224667260526`*^-8}, {
                    180, 1.4328663110985998`*^-8}, {
                    181, 1.1926660424116892`*^-8}, {
                    182, 9.662128318034177*^-9}, {
                    183, 7.572349252150447*^-9}, {
                    184, 5.718835914263641*^-9}, {
                    185, 4.1648231855897414`*^-9}, {
                    186, 2.9401470573357896`*^-9}, {
                    187, 2.0323836352054213`*^-9}, {
                    188, 1.3982041579841351`*^-9}, {
                    189, 9.78824556477881*^-10}, {
                    190, 7.117852341948728*^-10}, {
                    191, 5.405773407421047*^-10}, {
                    192, 4.231994735814963*^-10}, {
                    193, 3.3492866075375823`*^-10}, {
                    194, 2.6408642276844017`*^-10}, {
                    195, 2.0591097065050514`*^-10}, {
                    196, 1.5835781028446087`*^-10}, {
                    197, 1.2012255869962224`*^-10}, {
                    198, 8.996850196999382*^-11}, {
                    199, 6.661255499962208*^-11}, {
                    200, 4.878971711098*^-11}, {
                    201, 3.5344813557897254`*^-11}, {
                    202, 2.5295063913191857`*^-11}, {
                    203, 1.7849693821885105`*^-11}, {
                    204, 1.2393415426388838`*^-11}, {
                    205, 8.451361843885001*^-12}, {
                    206, 5.653391888322103*^-12}, {
                    207, 3.707196895984505*^-12}, {
                    208, 2.3817498112488497`*^-12}, {
                    209, 1.4975256882161408`*^-12}, {
                    210, 9.19255175704502*^-13}, {
                    211, 5.486164926513217*^-13}, {
                    212, 3.163935523475936*^-13}, {
                    213, 1.7491807169023416`*^-13}, {
                    214, 9.176024729416635*^-14}, {
                    215, 4.5074835322737055`*^-14}, {
                    216, 2.0425273096470337`*^-14}, {
                    217, 8.683533237441414*^-15}, {
                    218, 4.458770092149619*^-15}, {
                    219, 3.658535436267328*^-15}, {
                    220, 3.3309519957799806`*^-15}, {
                    221, 2.878677603272283*^-15}, {
                    222, 2.3890902074055295`*^-15}, {
                    223, 1.949497394214547*^-15}, {
                    224, 1.5820279836881724`*^-15}, {
                    225, 1.2738135802945798`*^-15}, {
                    226, 1.0079058960944382`*^-15}, {
                    227, 7.76197669192485*^-16}, {
                    228, 5.781262065644672*^-16}, {
                    229, 4.1523156990714817`*^-16}, {
                    230, 2.8741102700943933`*^-16}, {
                    231, 1.918665072443445*^-16}, {
                    232, 1.23734807678485*^-16}, {
                    233, 7.726864601608759*^-17}, {
                    234, 4.687384324679332*^-17}, {
                    235, 2.774965603840204*^-17}, {
                    236, 1.613999563539118*^-17}, {
                    237, 9.313541658879721*^-18}, {
                    238, 5.401540449087361*^-18}, {
                    239, 3.192373235958878*^-18}, {
                    240, 1.9406722239128857`*^-18}, {
                    241, 1.2134259763425004`*^-18}, {
                    242, 7.736389261906779*^-19}, {
                    243, 4.970535617599421*^-19}, {
                    244, 3.1853882222610627`*^-19}, {
                    245, 2.021750204960456*^-19}, {
                    246, 1.2652925461690181`*^-19}, {
                    247, 7.788315533966009*^-20}, {
                    248, 4.708293883442487*^-20}, {
                    249, 2.793320747395414*^-20}, {
                    250, 1.625792464700552*^-20}, {
                    251, 9.282592651594108*^-21}}],
                   Frame->True,
                   FrameTicks->{{{1., 
                    FormBox[
                    RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
                    FormBox[
                    RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
                    FormBox[
                    RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
                    FormBox[
                    RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
                    FormBox["0", TraditionalForm]}, {157.25, 
                    FormBox["2.5`", TraditionalForm]}, {188.5, 
                    FormBox["5", TraditionalForm]}, {219.75, 
                    FormBox["7.5`", TraditionalForm]}, {251., 
                    FormBox["10", TraditionalForm]}}, Automatic, None, None},
                   PlotRange->{-0.1, 1.1},
                   PlotRangeClipping->True,
                   PlotRangePadding->{
                    Scaled[0.02], Automatic}], 7->
                  GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF2XlUlIUax/HXBQVvKAaVWzYSoJQbyCaB/JBVRIEBZIcZBphh1euKqN2X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                    "], {
                    {GrayLevel[0.5], EdgeForm[None], 
                    GraphicsGroupBox[{PolygonBox[{{249, 748, 250}}], 
                    PolygonBox[CompressedData["
1:eJwtl3n0FkIUhu9MSqi0KO3KkgrJFpL2HZUiIUUKLSpFu7IUJVJEVEgokrSQ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                    "]]}]}, {{}, {}, {
                    {Hue[0.3607696028161082, 1, 1], LineBox[{1, 2}]}, 
                    {Hue[0.267057753080544, 1, 1], LineBox[{2, 3}]}, 
                    {Hue[0.1686832503326415, 1, 1], LineBox[{3, 4}]}, 
                    {Hue[0.0839902357583648, 1, 1], LineBox[{4, 5}]}, 
                    {Hue[0.9997109477118303, 1, 1], LineBox[{5, 6}]}, 
                    {Hue[0.9203737430652636, 1, 1], LineBox[{6, 7}]}, 
                    {Hue[0.8006315064242483, 1, 1], LineBox[{7, 8}]}, 
                    {Hue[0.7290597125194981, 1, 1], LineBox[{8, 9}]}, 
                    {Hue[0.6389748737056098, 1, 1], LineBox[{9, 10}]}, 
                    {Hue[0.5422263215544533, 1, 1], LineBox[{10, 11}]}, 
                    {Hue[0.43031212923343726`, 1, 1], LineBox[{11, 12}]}, 
                    {Hue[0.362450423374641, 1, 1], LineBox[{12, 13}]}, 
                    {Hue[0.2585024535212568, 1, 1], LineBox[{13, 14}]}, 
                    {Hue[0.17167954532191643`, 1, 1], LineBox[{14, 15}]}, 
                    {Hue[0.08752763848837851, 1, 1], LineBox[{15, 16}]}, 
                    {Hue[0.9634484512814975, 1, 1], LineBox[{16, 17}]}, 
                    {Hue[0.8125204942144689, 1, 1], LineBox[{17, 18}]}, 
                    {Hue[0.6799926598771155, 1, 1], LineBox[{18, 19}]}, 
                    {Hue[0.58093094876618, 1, 1], LineBox[{19, 20}]}, 
                    {Hue[0.46481982190682386`, 1, 1], LineBox[{20, 21}]}, 
                    {Hue[0.32700913340710674`, 1, 1], LineBox[{21, 22}]}, 
                    {Hue[0.28037209140499164`, 1, 1], LineBox[{22, 23}]}, 
                    {Hue[0.1675991623172561, 1, 1], LineBox[{23, 24}]}, 
                    {Hue[0.07255009251719302, 1, 1], LineBox[{24, 25}]}, 
                    {Hue[0.9911313410433275, 1, 1], LineBox[{25, 26}]}, 
                    {Hue[0.8390801589542314, 1, 1], LineBox[{26, 27}]}, 
                    {Hue[0.7723732432560815, 1, 1], LineBox[{27, 28}]}, 
                    {Hue[0.7250092002080438, 1, 1], LineBox[{28, 29}]}, 
                    {Hue[0.704030291923211, 1, 1], LineBox[{29, 30}]}, 
                    {Hue[0.6606090358406091, 1, 1], LineBox[{30, 31}]}, 
                    {Hue[0.18458027210925845`, 1, 1], LineBox[{31, 32}]}, 
                    {Hue[0.20736414094701383`, 1, 1], LineBox[{32, 33}]}, 
                    {Hue[0.19627293875574253`, 1, 1], LineBox[{33, 34}]}, 
                    {Hue[0.22709226991981238`, 1, 1], LineBox[{34, 35}]}, 
                    {Hue[0.309336981052953, 1, 1], LineBox[{35, 36}]}, 
                    {Hue[0.38668987546377964`, 1, 1], LineBox[{36, 37}]}, 
                    {Hue[0.4580886691838912, 1, 1], LineBox[{37, 38}]}, 
                    {Hue[0.527587973755167, 1, 1], LineBox[{38, 39}]}, 
                    {Hue[0.5843741913273628, 1, 1], LineBox[{39, 40}]}, 
                    {Hue[0.6529738372754882, 1, 1], LineBox[{40, 41}]}, 
                    {Hue[0.7231717946712797, 1, 1], LineBox[{41, 42}]}, 
                    {Hue[0.7940198162849148, 1, 1], LineBox[{42, 43}]}, 
                    {Hue[0.8670233124123604, 1, 1], LineBox[{43, 44}]}, 
                    {Hue[0.9376088157986627, 1, 1], LineBox[{44, 45}]}, 
                    {Hue[0.008213234289735359, 1, 1], LineBox[{45, 46}]}, 
                    {Hue[0.07954226466320549, 1, 1], LineBox[{46, 47}]}, 
                    {Hue[0.15182109711666714`, 1, 1], LineBox[{47, 48}]}, 
                    {Hue[0.22507428783865907`, 1, 1], LineBox[{48, 49}]}, 
                    {Hue[0.298601788645345, 1, 1], LineBox[{49, 50}]}, 
                    {Hue[0.37261978757492364`, 1, 1], LineBox[{50, 51}]}, 
                    {Hue[0.4464695067172625, 1, 1], LineBox[{51, 52}]}, 
                    {Hue[0.520985703519601, 1, 1], LineBox[{52, 53}]}, 
                    {Hue[0.5962503222699094, 1, 1], LineBox[{53, 54}]}, 
                    {Hue[0.6720211162908327, 1, 1], LineBox[{54, 55}]}, 
                    {Hue[0.7482495212329322, 1, 1], LineBox[{55, 56}]}, 
                    {Hue[0.8250046849443391, 1, 1], LineBox[{56, 57}]}, 
                    {Hue[0.9021656385273559, 1, 1], LineBox[{57, 58}]}, 
                    {Hue[0.9798307401607916, 1, 1], LineBox[{58, 59}]}, 
                    {Hue[0.05800235227659842, 1, 1], LineBox[{59, 60}]}, 
                    {Hue[0.13669273281546193`, 1, 1], LineBox[{60, 61}]}, 
                    {Hue[0.21585977065513692`, 1, 1], LineBox[{61, 62}]}, 
                    {Hue[0.29548816845309706`, 1, 1], LineBox[{62, 63}]}, 
                    {Hue[0.37560341603106856`, 1, 1], LineBox[{63, 64}]}, 
                    {Hue[0.4562040007508113, 1, 1], LineBox[{64, 65}]}, 
                    {Hue[0.5373038430435617, 1, 1], LineBox[{65, 66}]}, 
                    {Hue[0.6188951496447265, 1, 1], LineBox[{66, 67}]}, 
                    {Hue[0.7009511042349899, 1, 1], LineBox[{67, 68}]}, 
                    {Hue[0.7835073631502996, 1, 1], LineBox[{68, 69}]}, 
                    {Hue[0.8665536235626842, 1, 1], LineBox[{69, 70}]}, 
                    {Hue[0.9500892860673488, 1, 1], LineBox[{70, 71}]}, 
                    {Hue[0.034106981671226305`, 1, 1], LineBox[{71, 72}]}, 
                    {Hue[0.11860815259491314`, 1, 1], LineBox[{72, 73}]}, 
                    {Hue[0.2035904553821011, 1, 1], LineBox[{73, 74}]}, 
                    {Hue[0.28906409794720567`, 1, 1], LineBox[{74, 75}]}, 
                    {Hue[0.37502184323541427`, 1, 1], LineBox[{75, 76}]}, 
                    {Hue[0.4614619759364616, 1, 1], LineBox[{76, 77}]}, 
                    {Hue[0.5483815239488417, 1, 1], LineBox[{77, 78}]}, 
                    {Hue[0.63578442230926, 1, 1], LineBox[{78, 79}]}, 
                    {Hue[0.7236676785821858, 1, 1], LineBox[{79, 80}]}, 
                    {Hue[0.8120314596811757, 1, 1], LineBox[{80, 81}]}, 
                    {Hue[0.900873601632316, 1, 1], LineBox[{81, 82}]}, 
                    {Hue[0.9901942213417271, 1, 1], LineBox[{82, 83}]}, 
                    {Hue[0.0799895702292984, 1, 1], LineBox[{83, 84}]}, 
                    {Hue[0.170260706962758, 1, 1], LineBox[{84, 85}]}, 
                    {Hue[0.2610034850012405, 1, 1], LineBox[{85, 86}]}, 
                    {Hue[0.35221637057648686`, 1, 1], LineBox[{86, 87}]}, 
                    {Hue[0.44389826161878776`, 1, 1], LineBox[{87, 88}]}, 
                    {Hue[0.5360449736481047, 1, 1], LineBox[{88, 89}]}, 
                    {Hue[0.6286541129036205, 1, 1], LineBox[{89, 90}]}, 
                    {Hue[0.7217217985068132, 1, 1], LineBox[{90, 91}]}, 
                    {Hue[0.8152433603000737, 1, 1], LineBox[{91, 92}]}, 
                    {Hue[0.9092139707072192, 1, 1], LineBox[{92, 93}]}, 
                    {Hue[0.0036290315923241745`, 1, 1], LineBox[{93, 94}]}, 
                    {Hue[0.09848073066420522, 1, 1], LineBox[{94, 95}]}, 
                    {Hue[0.19376227185500616`, 1, 1], LineBox[{95, 96}]}, 
                    {Hue[0.289464672794272, 1, 1], LineBox[{96, 97}]}, 
                    {Hue[0.3855781333910314, 1, 1], LineBox[{97, 98}]}, 
                    {Hue[0.4820908974496971, 1, 1], LineBox[{98, 99}]}, 
                    {Hue[0.5789897695238124, 1, 1], LineBox[{99, 100}]}, 
                    {Hue[0.6762580878511103, 1, 1], LineBox[{100, 101}]}, 
                    {Hue[0.7738792026198127, 1, 1], LineBox[{101, 102}]}, 
                    {Hue[0.8718316149733819, 1, 1], LineBox[{102, 103}]}, 
                    {Hue[0.9700914448448266, 1, 1], LineBox[{103, 104}]}, 
                    {Hue[0.06863078135729278, 1, 1], LineBox[{104, 105}]}, 
                    {Hue[0.1674173699230757, 1, 1], LineBox[{105, 106}]}, 
                    {Hue[0.2664150890214937, 1, 1], LineBox[{106, 107}]}, 
                    {Hue[0.36558201499268755`, 1, 1], LineBox[{107, 108}]}, 
                    {Hue[0.4648696110448278, 1, 1], LineBox[{108, 109}]}, 
                    {Hue[0.5642232715551221, 1, 1], LineBox[{109, 110}]}, 
                    {Hue[0.6635811232888242, 1, 1], LineBox[{110, 111}]}, 
                    {Hue[0.7628731825198262, 1, 1], LineBox[{111, 112}]}, 
                    {Hue[0.8620209439738609, 1, 1], LineBox[{112, 113}]}, 
                    {Hue[0.960936178647772, 1, 1], LineBox[{113, 114}]}, 
                    {Hue[0.05952243736322674, 1, 1], LineBox[{114, 115}]}, 
                    {Hue[0.1576739410013476, 1, 1], LineBox[{115, 116}]}, 
                    {Hue[0.25527613254844134`, 1, 1], LineBox[{116, 117}]}, 
                    {Hue[0.3522059883891288, 1, 1], LineBox[{117, 118}]}, 
                    {Hue[0.4483344792343556, 1, 1], LineBox[{118, 119}]}, 
                    {Hue[0.5435302454469617, 1, 1], LineBox[{119, 120}]}, 
                    {Hue[0.6376602769840898, 1, 1], LineBox[{120, 121}]}, 
                    {Hue[0.7305964827538581, 1, 1], LineBox[{121, 122}]}, 
                    {Hue[0.822219008183593, 1, 1], LineBox[{122, 123}]}, 
                    {Hue[0.9124241755271605, 1, 1], LineBox[{123, 124}]}, 
                    {Hue[0.0011313352912279788`, 1, 1], 
                    LineBox[{124, 125}]}, 
                    {Hue[0.08829537377171129, 1, 1], LineBox[{125, 126}]}, 
                    {Hue[0.173914091059342, 1, 1], LineBox[{126, 127}]}, 
                    {Hue[0.2580327570304501, 1, 1], LineBox[{127, 128}]}, 
                    {Hue[0.34074391743273624`, 1, 1], LineBox[{128, 129}]}, 
                    {Hue[0.42218350829950074`, 1, 1], LineBox[{129, 130}]}, 
                    {Hue[0.5025176745363464, 1, 1], LineBox[{130, 131}]}, 
                    {Hue[0.5819340841751708, 1, 1], LineBox[{131, 132}]}, 
                    {Hue[0.6606322089404484, 1, 1], LineBox[{132, 133}]}, 
                    {Hue[0.7388112677112173, 1, 1], LineBox[{133, 134}]}, 
                    {Hue[0.8166629771094296, 1, 1], LineBox[{134, 135}]}, 
                    {Hue[0.894367291049327, 1, 1], LineBox[{135, 136}]}, 
                    {Hue[0.9720925084202446, 1, 1], LineBox[{136, 137}]}, 
                    {Hue[0.04998896335048285, 1, 1], LineBox[{137, 138}]}, 
                    {Hue[0.1281877525899212, 1, 1], LineBox[{138, 139}]}, 
                    {Hue[0.20680136452066197`, 1, 1], LineBox[{139, 140}]}, 
                    {Hue[0.28593303904703327`, 1, 1], LineBox[{140, 141}]}, 
                    {Hue[0.36567016562670085`, 1, 1], LineBox[{141, 142}]}, 
                    {Hue[0.44607683475779675`, 1, 1], LineBox[{142, 143}]}, 
                    {Hue[0.5272080444436378, 1, 1], LineBox[{143, 144}]}, 
                    {Hue[0.6091162047792973, 1, 1], LineBox[{144, 145}]}, 
                    {Hue[0.691847083043619, 1, 1], LineBox[{145, 146}]}, 
                    {Hue[0.7754258035423244, 1, 1], LineBox[{146, 147}]}, 
                    {Hue[0.8598681254734846, 1, 1], LineBox[{147, 148}]}, 
                    {Hue[0.9452056790005225, 1, 1], LineBox[{148, 149}]}, 
                    {Hue[0.03147188112713823, 1, 1], LineBox[{149, 150}]}, 
                    {Hue[0.11867304215521769`, 1, 1], LineBox[{150, 151}]}, 
                    {Hue[0.20676513826419657`, 1, 1], LineBox[{151, 152}]}, 
                    {Hue[0.2956907086256168, 1, 1], LineBox[{152, 153}]}, 
                    {Hue[0.3854998251506179, 1, 1], LineBox[{153, 154}]}, 
                    {Hue[0.4763031405657942, 1, 1], LineBox[{154, 155}]}, 
                    {Hue[0.5680698599260738, 1, 1], LineBox[{155, 156}]}, 
                    {Hue[0.6606728731331002, 1, 1], LineBox[{156, 157}]}, 
                    {Hue[0.754025841465475, 1, 1], LineBox[{157, 158}]}, 
                    {Hue[0.8480804931757541, 1, 1], LineBox[{158, 159}]}, 
                    {Hue[0.9431181536174291, 1, 1], LineBox[{159, 160}]}, 
                    {Hue[0.0397869542393286, 1, 1], LineBox[{160, 161}]}, 
                    {Hue[0.137840700355691, 1, 1], LineBox[{161, 162}]}, 
                    {Hue[0.23574526072148794`, 1, 1], LineBox[{162, 163}]}, 
                    {Hue[0.333496700037884, 1, 1], LineBox[{163, 164}]}, 
                    {Hue[0.43380775123773196`, 1, 1], LineBox[{164, 165}]}, 
                    {Hue[0.536904103339749, 1, 1], LineBox[{165, 166}]}, 
                    {Hue[0.6388111314179277, 1, 1], LineBox[{166, 167}]}, 
                    {Hue[0.7395416522567456, 1, 1], LineBox[{167, 168}]}, 
                    {Hue[0.8447363652466423, 1, 1], LineBox[{168, 169}]}, 
                    {Hue[0.9501147795477853, 1, 1], LineBox[{169, 170}]}, 
                    {Hue[0.04045361918740696, 1, 1], LineBox[{170, 171}]}, 
                    {Hue[0.1286078858347576, 1, 1], LineBox[{171, 172}]}, 
                    {Hue[0.24672799717042326`, 1, 1], LineBox[{172, 173}]}, 
                    {Hue[0.37861522354390637`, 1, 1], LineBox[{173, 174}]}, 
                    {Hue[0.45736632935843075`, 1, 1], LineBox[{174, 175}]}, 
                    {Hue[0.49308730974207005`, 1, 1], LineBox[{175, 176}]}, 
                    {Hue[0.6470686768988838, 1, 1], LineBox[{176, 177}]}, 
                    {Hue[0.8568677735491859, 1, 1], LineBox[{177, 178}]}, 
                    {Hue[0.10431225644341774`, 1, 1], LineBox[{178, 179}]}, 
                    {Hue[0.3894102905074262, 1, 1], LineBox[{179, 180}]}, 
                    {Hue[0.6838263781448488, 1, 1], LineBox[{180, 181}]}, 
                    {Hue[0.9569069071984381, 1, 1], LineBox[{181, 182}]}, 
                    {Hue[0.21395671894896143`, 1, 1], LineBox[{182, 183}]}, 
                    {Hue[0.4664485349712569, 1, 1], LineBox[{183, 184}]}, 
                    {Hue[0.7219258772292536, 1, 1], LineBox[{184, 185}]}, 
                    {Hue[0.9856372041087308, 1, 1], LineBox[{185, 186}]}, 
                    {Hue[0.2608722601555482, 1, 1], LineBox[{186, 187}]}, 
                    {Hue[0.5470837038038487, 1, 1], LineBox[{187, 188}]}, 
                    {Hue[0.8401196088581602, 1, 1], LineBox[{188, 189}]}, 
                    {Hue[0.13624437333826112`, 1, 1], LineBox[{189, 190}]}, 
                    {Hue[0.4309682311873325, 1, 1], LineBox[{190, 191}]}, 
                    {Hue[0.7184660058323185, 1, 1], LineBox[{191, 192}]}, 
                    {Hue[0.9973525953613468, 1, 1], LineBox[{192, 193}]}, 
                    {Hue[0.2702841777599323, 1, 1], LineBox[{193, 194}]}, 
                    {Hue[0.5401052324973117, 1, 1], LineBox[{194, 195}]}, 
                    {Hue[0.8084590354995888, 1, 1], LineBox[{195, 196}]}, 
                    {Hue[0.07594959588080527, 1, 1], LineBox[{196, 197}]}, 
                    {Hue[0.34257008552859086`, 1, 1], LineBox[{197, 198}]}, 
                    {Hue[0.608041503369714, 1, 1], LineBox[{198, 199}]}, 
                    {Hue[0.8720135963849045, 1, 1], LineBox[{199, 200}]}, 
                    {Hue[0.13414354537958378`, 1, 1], LineBox[{200, 201}]}, 
                    {Hue[0.3940928156514941, 1, 1], LineBox[{201, 202}]}, 
                    {Hue[0.6514812971209296, 1, 1], LineBox[{202, 203}]}, 
                    {Hue[0.9058301300769357, 1, 1], LineBox[{203, 204}]}, 
                    {Hue[0.1565116040087216, 1, 1], LineBox[{204, 205}]}, 
                    {Hue[0.4027360261695492, 1, 1], LineBox[{205, 206}]}, 
                    {Hue[0.6436450448936091, 1, 1], LineBox[{206, 207}]}, 
                    {Hue[0.8786518091264015, 1, 1], LineBox[{207, 208}]}, 
                    {Hue[0.10814245304970796`, 1, 1], LineBox[{208, 209}]}, 
                    {Hue[0.3342040087120603, 1, 1], LineBox[{209, 210}]}, 
                    {Hue[0.5603417438283603, 1, 1], LineBox[{210, 211}]}, 
                    {Hue[0.7898375376869549, 1, 1], LineBox[{211, 212}]}, 
                    {Hue[0.02435059756972669, 1, 1], LineBox[{212, 213}]}, 
                    {Hue[0.263972200373601, 1, 1], LineBox[{213, 214}]}, 
                    {Hue[0.5080091112724794, 1, 1], LineBox[{214, 215}]}, 
                    {Hue[0.7555873580958158, 1, 1], LineBox[{215, 216}]}, 
                    {Hue[0.005910038993356071, 1, 1], LineBox[{216, 217}]}, 
                    {Hue[0.2583208773049551, 1, 1], LineBox[{217, 218}]}, 
                    {Hue[0.5122990607419959, 1, 1], LineBox[{218, 219}]}, 
                    {Hue[0.7674399787929954, 1, 1], LineBox[{219, 220}]}, 
                    {Hue[0.02343753155512082, 1, 1], LineBox[{220, 221}]}, 
                    {Hue[0.2800651076356245, 1, 1], LineBox[{221, 222}]}, 
                    {Hue[0.5371545059966867, 1, 1], LineBox[{222, 223}]}, 
                    {Hue[0.7945733996717301, 1, 1], LineBox[{223, 224}]}, 
                    {Hue[0.052204389580622186`, 1, 1], LineBox[{224, 225}]}, 
                    {Hue[0.30993216728720485`, 1, 1], LineBox[{225, 226}]}, 
                    {Hue[0.5676409258558874, 1, 1], LineBox[{226, 227}]}, 
                    {Hue[0.8252245236518228, 1, 1], LineBox[{227, 228}]}, 
                    {Hue[0.08260688618784486, 1, 1], LineBox[{228, 229}]}, 
                    {Hue[0.3397704033682906, 1, 1], LineBox[{229, 230}]}, 
                    {Hue[0.5967914666911367, 1, 1], LineBox[{230, 231}]}, 
                    {Hue[0.8538866103988598, 1, 1], LineBox[{231, 232}]}, 
                    {Hue[0.11148110441383434`, 1, 1], LineBox[{232, 233}]}, 
                    {Hue[0.37032221044370817`, 1, 1], LineBox[{233, 234}]}, 
                    {Hue[0.6316738642015586, 1, 1], LineBox[{234, 235}]}, 
                    {Hue[0.8976395052931159, 1, 1], LineBox[{235, 236}]}, 
                    {Hue[0.17158596189859066`, 1, 1], LineBox[{236, 237}]}, 
                    {Hue[0.45806602624555387`, 1, 1], LineBox[{237, 238}]}, 
                    {Hue[0.759613669603346, 1, 1], LineBox[{238, 239}]}, 
                    {Hue[0.06876575145420652, 1, 1], LineBox[{239, 240}]}, 
                    {Hue[0.3704520490070801, 1, 1], LineBox[{240, 241}]}, 
                    {Hue[0.6581400003275106, 1, 1], LineBox[{241, 242}]}, 
                    {Hue[0.9342542877440975, 1, 1], LineBox[{242, 243}]}, 
                    {Hue[0.20258989419990772`, 1, 1], LineBox[{243, 244}]}, 
                    {Hue[0.46580311190130813`, 1, 1], LineBox[{244, 245}]}, 
                    {Hue[0.7255298424272546, 1, 1], LineBox[{245, 246}]}, 
                    {Hue[0.9828031804668734, 1, 1], LineBox[{246, 247}]}, 
                    {Hue[0.2383262257411595, 1, 1], LineBox[{247, 248}]}, 
                    {Hue[0.4926179409024767, 1, 1], LineBox[{248, 249}]}, 
                    {Hue[0.7460921021000613, 1, 1], LineBox[{249, 250}]}, 
                    {Hue[0.9991008808619908, 1, 1], LineBox[{250, 251}]}}}},
                    VertexColors->{
                    Hue[0.3607696028161082, 1, 1], 
                    Hue[0.267057753080544, 1, 1], 
                    Hue[0.1686832503326415, 1, 1], 
                    Hue[0.0839902357583648, 1, 1], 
                    Hue[0.9997109477118303, 1, 1], 
                    Hue[0.9203737430652636, 1, 1], 
                    Hue[0.8006315064242483, 1, 1], 
                    Hue[0.7290597125194981, 1, 1], 
                    Hue[0.6389748737056098, 1, 1], 
                    Hue[0.5422263215544533, 1, 1], 
                    Hue[0.43031212923343726`, 1, 1], 
                    Hue[0.362450423374641, 1, 1], 
                    Hue[0.2585024535212568, 1, 1], 
                    Hue[0.17167954532191643`, 1, 1], 
                    Hue[0.08752763848837851, 1, 1], 
                    Hue[0.9634484512814975, 1, 1], 
                    Hue[0.8125204942144689, 1, 1], 
                    Hue[0.6799926598771155, 1, 1], 
                    Hue[0.58093094876618, 1, 1], 
                    Hue[0.46481982190682386`, 1, 1], 
                    Hue[0.32700913340710674`, 1, 1], 
                    Hue[0.28037209140499164`, 1, 1], 
                    Hue[0.1675991623172561, 1, 1], 
                    Hue[0.07255009251719302, 1, 1], 
                    Hue[0.9911313410433275, 1, 1], 
                    Hue[0.8390801589542314, 1, 1], 
                    Hue[0.7723732432560815, 1, 1], 
                    Hue[0.7250092002080438, 1, 1], 
                    Hue[0.704030291923211, 1, 1], 
                    Hue[0.6606090358406091, 1, 1], 
                    Hue[0.18458027210925845`, 1, 1], 
                    Hue[0.20736414094701383`, 1, 1], 
                    Hue[0.19627293875574253`, 1, 1], 
                    Hue[0.22709226991981238`, 1, 1], 
                    Hue[0.309336981052953, 1, 1], 
                    Hue[0.38668987546377964`, 1, 1], 
                    Hue[0.4580886691838912, 1, 1], 
                    Hue[0.527587973755167, 1, 1], 
                    Hue[0.5843741913273628, 1, 1], 
                    Hue[0.6529738372754882, 1, 1], 
                    Hue[0.7231717946712797, 1, 1], 
                    Hue[0.7940198162849148, 1, 1], 
                    Hue[0.8670233124123604, 1, 1], 
                    Hue[0.9376088157986627, 1, 1], 
                    Hue[0.008213234289735359, 1, 1], 
                    Hue[0.07954226466320549, 1, 1], 
                    Hue[0.15182109711666714`, 1, 1], 
                    Hue[0.22507428783865907`, 1, 1], 
                    Hue[0.298601788645345, 1, 1], 
                    Hue[0.37261978757492364`, 1, 1], 
                    Hue[0.4464695067172625, 1, 1], 
                    Hue[0.520985703519601, 1, 1], 
                    Hue[0.5962503222699094, 1, 1], 
                    Hue[0.6720211162908327, 1, 1], 
                    Hue[0.7482495212329322, 1, 1], 
                    Hue[0.8250046849443391, 1, 1], 
                    Hue[0.9021656385273559, 1, 1], 
                    Hue[0.9798307401607916, 1, 1], 
                    Hue[0.05800235227659842, 1, 1], 
                    Hue[0.13669273281546193`, 1, 1], 
                    Hue[0.21585977065513692`, 1, 1], 
                    Hue[0.29548816845309706`, 1, 1], 
                    Hue[0.37560341603106856`, 1, 1], 
                    Hue[0.4562040007508113, 1, 1], 
                    Hue[0.5373038430435617, 1, 1], 
                    Hue[0.6188951496447265, 1, 1], 
                    Hue[0.7009511042349899, 1, 1], 
                    Hue[0.7835073631502996, 1, 1], 
                    Hue[0.8665536235626842, 1, 1], 
                    Hue[0.9500892860673488, 1, 1], 
                    Hue[0.034106981671226305`, 1, 1], 
                    Hue[0.11860815259491314`, 1, 1], 
                    Hue[0.2035904553821011, 1, 1], 
                    Hue[0.28906409794720567`, 1, 1], 
                    Hue[0.37502184323541427`, 1, 1], 
                    Hue[0.4614619759364616, 1, 1], 
                    Hue[0.5483815239488417, 1, 1], 
                    Hue[0.63578442230926, 1, 1], 
                    Hue[0.7236676785821858, 1, 1], 
                    Hue[0.8120314596811757, 1, 1], 
                    Hue[0.900873601632316, 1, 1], 
                    Hue[0.9901942213417271, 1, 1], 
                    Hue[0.0799895702292984, 1, 1], 
                    Hue[0.170260706962758, 1, 1], 
                    Hue[0.2610034850012405, 1, 1], 
                    Hue[0.35221637057648686`, 1, 1], 
                    Hue[0.44389826161878776`, 1, 1], 
                    Hue[0.5360449736481047, 1, 1], 
                    Hue[0.6286541129036205, 1, 1], 
                    Hue[0.7217217985068132, 1, 1], 
                    Hue[0.8152433603000737, 1, 1], 
                    Hue[0.9092139707072192, 1, 1], 
                    Hue[0.0036290315923241745`, 1, 1], 
                    Hue[0.09848073066420522, 1, 1], 
                    Hue[0.19376227185500616`, 1, 1], 
                    Hue[0.289464672794272, 1, 1], 
                    Hue[0.3855781333910314, 1, 1], 
                    Hue[0.4820908974496971, 1, 1], 
                    Hue[0.5789897695238124, 1, 1], 
                    Hue[0.6762580878511103, 1, 1], 
                    Hue[0.7738792026198127, 1, 1], 
                    Hue[0.8718316149733819, 1, 1], 
                    Hue[0.9700914448448266, 1, 1], 
                    Hue[0.06863078135729278, 1, 1], 
                    Hue[0.1674173699230757, 1, 1], 
                    Hue[0.2664150890214937, 1, 1], 
                    Hue[0.36558201499268755`, 1, 1], 
                    Hue[0.4648696110448278, 1, 1], 
                    Hue[0.5642232715551221, 1, 1], 
                    Hue[0.6635811232888242, 1, 1], 
                    Hue[0.7628731825198262, 1, 1], 
                    Hue[0.8620209439738609, 1, 1], 
                    Hue[0.960936178647772, 1, 1], 
                    Hue[0.05952243736322674, 1, 1], 
                    Hue[0.1576739410013476, 1, 1], 
                    Hue[0.25527613254844134`, 1, 1], 
                    Hue[0.3522059883891288, 1, 1], 
                    Hue[0.4483344792343556, 1, 1], 
                    Hue[0.5435302454469617, 1, 1], 
                    Hue[0.6376602769840898, 1, 1], 
                    Hue[0.7305964827538581, 1, 1], 
                    Hue[0.822219008183593, 1, 1], 
                    Hue[0.9124241755271605, 1, 1], 
                    Hue[0.0011313352912279788`, 1, 1], 
                    Hue[0.08829537377171129, 1, 1], 
                    Hue[0.173914091059342, 1, 1], 
                    Hue[0.2580327570304501, 1, 1], 
                    Hue[0.34074391743273624`, 1, 1], 
                    Hue[0.42218350829950074`, 1, 1], 
                    Hue[0.5025176745363464, 1, 1], 
                    Hue[0.5819340841751708, 1, 1], 
                    Hue[0.6606322089404484, 1, 1], 
                    Hue[0.7388112677112173, 1, 1], 
                    Hue[0.8166629771094296, 1, 1], 
                    Hue[0.894367291049327, 1, 1], 
                    Hue[0.9720925084202446, 1, 1], 
                    Hue[0.04998896335048285, 1, 1], 
                    Hue[0.1281877525899212, 1, 1], 
                    Hue[0.20680136452066197`, 1, 1], 
                    Hue[0.28593303904703327`, 1, 1], 
                    Hue[0.36567016562670085`, 1, 1], 
                    Hue[0.44607683475779675`, 1, 1], 
                    Hue[0.5272080444436378, 1, 1], 
                    Hue[0.6091162047792973, 1, 1], 
                    Hue[0.691847083043619, 1, 1], 
                    Hue[0.7754258035423244, 1, 1], 
                    Hue[0.8598681254734846, 1, 1], 
                    Hue[0.9452056790005225, 1, 1], 
                    Hue[0.03147188112713823, 1, 1], 
                    Hue[0.11867304215521769`, 1, 1], 
                    Hue[0.20676513826419657`, 1, 1], 
                    Hue[0.2956907086256168, 1, 1], 
                    Hue[0.3854998251506179, 1, 1], 
                    Hue[0.4763031405657942, 1, 1], 
                    Hue[0.5680698599260738, 1, 1], 
                    Hue[0.6606728731331002, 1, 1], 
                    Hue[0.754025841465475, 1, 1], 
                    Hue[0.8480804931757541, 1, 1], 
                    Hue[0.9431181536174291, 1, 1], 
                    Hue[0.0397869542393286, 1, 1], 
                    Hue[0.137840700355691, 1, 1], 
                    Hue[0.23574526072148794`, 1, 1], 
                    Hue[0.333496700037884, 1, 1], 
                    Hue[0.43380775123773196`, 1, 1], 
                    Hue[0.536904103339749, 1, 1], 
                    Hue[0.6388111314179277, 1, 1], 
                    Hue[0.7395416522567456, 1, 1], 
                    Hue[0.8447363652466423, 1, 1], 
                    Hue[0.9501147795477853, 1, 1], 
                    Hue[0.04045361918740696, 1, 1], 
                    Hue[0.1286078858347576, 1, 1], 
                    Hue[0.24672799717042326`, 1, 1], 
                    Hue[0.37861522354390637`, 1, 1], 
                    Hue[0.45736632935843075`, 1, 1], 
                    Hue[0.49308730974207005`, 1, 1], 
                    Hue[0.6470686768988838, 1, 1], 
                    Hue[0.8568677735491859, 1, 1], 
                    Hue[0.10431225644341774`, 1, 1], 
                    Hue[0.3894102905074262, 1, 1], 
                    Hue[0.6838263781448488, 1, 1], 
                    Hue[0.9569069071984381, 1, 1], 
                    Hue[0.21395671894896143`, 1, 1], 
                    Hue[0.4664485349712569, 1, 1], 
                    Hue[0.7219258772292536, 1, 1], 
                    Hue[0.9856372041087308, 1, 1], 
                    Hue[0.2608722601555482, 1, 1], 
                    Hue[0.5470837038038487, 1, 1], 
                    Hue[0.8401196088581602, 1, 1], 
                    Hue[0.13624437333826112`, 1, 1], 
                    Hue[0.4309682311873325, 1, 1], 
                    Hue[0.7184660058323185, 1, 1], 
                    Hue[0.9973525953613468, 1, 1], 
                    Hue[0.2702841777599323, 1, 1], 
                    Hue[0.5401052324973117, 1, 1], 
                    Hue[0.8084590354995888, 1, 1], 
                    Hue[0.07594959588080527, 1, 1], 
                    Hue[0.34257008552859086`, 1, 1], 
                    Hue[0.608041503369714, 1, 1], 
                    Hue[0.8720135963849045, 1, 1], 
                    Hue[0.13414354537958378`, 1, 1], 
                    Hue[0.3940928156514941, 1, 1], 
                    Hue[0.6514812971209296, 1, 1], 
                    Hue[0.9058301300769357, 1, 1], 
                    Hue[0.1565116040087216, 1, 1], 
                    Hue[0.4027360261695492, 1, 1], 
                    Hue[0.6436450448936091, 1, 1], 
                    Hue[0.8786518091264015, 1, 1], 
                    Hue[0.10814245304970796`, 1, 1], 
                    Hue[0.3342040087120603, 1, 1], 
                    Hue[0.5603417438283603, 1, 1], 
                    Hue[0.7898375376869549, 1, 1], 
                    Hue[0.02435059756972669, 1, 1], 
                    Hue[0.263972200373601, 1, 1], 
                    Hue[0.5080091112724794, 1, 1], 
                    Hue[0.7555873580958158, 1, 1], 
                    Hue[0.005910038993356071, 1, 1], 
                    Hue[0.2583208773049551, 1, 1], 
                    Hue[0.5122990607419959, 1, 1], 
                    Hue[0.7674399787929954, 1, 1], 
                    Hue[0.02343753155512082, 1, 1], 
                    Hue[0.2800651076356245, 1, 1], 
                    Hue[0.5371545059966867, 1, 1], 
                    Hue[0.7945733996717301, 1, 1], 
                    Hue[0.052204389580622186`, 1, 1], 
                    Hue[0.30993216728720485`, 1, 1], 
                    Hue[0.5676409258558874, 1, 1], 
                    Hue[0.8252245236518228, 1, 1], 
                    Hue[0.08260688618784486, 1, 1], 
                    Hue[0.3397704033682906, 1, 1], 
                    Hue[0.5967914666911367, 1, 1], 
                    Hue[0.8538866103988598, 1, 1], 
                    Hue[0.11148110441383434`, 1, 1], 
                    Hue[0.37032221044370817`, 1, 1], 
                    Hue[0.6316738642015586, 1, 1], 
                    Hue[0.8976395052931159, 1, 1], 
                    Hue[0.17158596189859066`, 1, 1], 
                    Hue[0.45806602624555387`, 1, 1], 
                    Hue[0.759613669603346, 1, 1], 
                    Hue[0.06876575145420652, 1, 1], 
                    Hue[0.3704520490070801, 1, 1], 
                    Hue[0.6581400003275106, 1, 1], 
                    Hue[0.9342542877440975, 1, 1], 
                    Hue[0.20258989419990772`, 1, 1], 
                    Hue[0.46580311190130813`, 1, 1], 
                    Hue[0.7255298424272546, 1, 1], 
                    Hue[0.9828031804668734, 1, 1], 
                    Hue[0.2383262257411595, 1, 1], 
                    Hue[0.4926179409024767, 1, 1], 
                    Hue[0.7460921021000613, 1, 1], 
                    Hue[0.9991008808619908, 1, 1], 
                    Hue[0.25196147333959246`, 1, 1], 
                    Hue[0.3607696028161082, 1, 1], 
                    Hue[0.267057753080544, 1, 1], 
                    Hue[0.267057753080544, 1, 1], 
                    Hue[0.1686832503326415, 1, 1], 
                    Hue[0.1686832503326415, 1, 1], 
                    Hue[0.0839902357583648, 1, 1], 
                    Hue[0.0839902357583648, 1, 1], 
                    Hue[0.9997109477118303, 1, 1], 
                    Hue[0.9997109477118303, 1, 1], 
                    Hue[0.9203737430652636, 1, 1], 
                    Hue[0.9203737430652636, 1, 1], 
                    Hue[0.8006315064242483, 1, 1], 
                    Hue[0.8006315064242483, 1, 1], 
                    Hue[0.7290597125194981, 1, 1], 
                    Hue[0.7290597125194981, 1, 1], 
                    Hue[0.6389748737056098, 1, 1], 
                    Hue[0.6389748737056098, 1, 1], 
                    Hue[0.5422263215544533, 1, 1], 
                    Hue[0.5422263215544533, 1, 1], 
                    Hue[0.43031212923343726`, 1, 1], 
                    Hue[0.43031212923343726`, 1, 1], 
                    Hue[0.362450423374641, 1, 1], 
                    Hue[0.362450423374641, 1, 1], 
                    Hue[0.2585024535212568, 1, 1], 
                    Hue[0.2585024535212568, 1, 1], 
                    Hue[0.17167954532191643`, 1, 1], 
                    Hue[0.17167954532191643`, 1, 1], 
                    Hue[0.08752763848837851, 1, 1], 
                    Hue[0.08752763848837851, 1, 1], 
                    Hue[0.9634484512814975, 1, 1], 
                    Hue[0.9634484512814975, 1, 1], 
                    Hue[0.8125204942144689, 1, 1], 
                    Hue[0.8125204942144689, 1, 1], 
                    Hue[0.6799926598771155, 1, 1], 
                    Hue[0.6799926598771155, 1, 1], 
                    Hue[0.58093094876618, 1, 1], 
                    Hue[0.58093094876618, 1, 1], 
                    Hue[0.46481982190682386`, 1, 1], 
                    Hue[0.46481982190682386`, 1, 1], 
                    Hue[0.32700913340710674`, 1, 1], 
                    Hue[0.32700913340710674`, 1, 1], 
                    Hue[0.28037209140499164`, 1, 1], 
                    Hue[0.28037209140499164`, 1, 1], 
                    Hue[0.1675991623172561, 1, 1], 
                    Hue[0.1675991623172561, 1, 1], 
                    Hue[0.07255009251719302, 1, 1], 
                    Hue[0.07255009251719302, 1, 1], 
                    Hue[0.9911313410433275, 1, 1], 
                    Hue[0.9911313410433275, 1, 1], 
                    Hue[0.8390801589542314, 1, 1], 
                    Hue[0.8390801589542314, 1, 1], 
                    Hue[0.7723732432560815, 1, 1], 
                    Hue[0.7723732432560815, 1, 1], 
                    Hue[0.7250092002080438, 1, 1], 
                    Hue[0.7250092002080438, 1, 1], 
                    Hue[0.704030291923211, 1, 1], 
                    Hue[0.704030291923211, 1, 1], 
                    Hue[0.6606090358406091, 1, 1], 
                    Hue[0.6606090358406091, 1, 1], 
                    Hue[0.18458027210925845`, 1, 1], 
                    Hue[0.18458027210925845`, 1, 1], 
                    Hue[0.20736414094701383`, 1, 1], 
                    Hue[0.20736414094701383`, 1, 1], 
                    Hue[0.19627293875574253`, 1, 1], 
                    Hue[0.19627293875574253`, 1, 1], 
                    Hue[0.22709226991981238`, 1, 1], 
                    Hue[0.22709226991981238`, 1, 1], 
                    Hue[0.309336981052953, 1, 1], 
                    Hue[0.309336981052953, 1, 1], 
                    Hue[0.38668987546377964`, 1, 1], 
                    Hue[0.38668987546377964`, 1, 1], 
                    Hue[0.4580886691838912, 1, 1], 
                    Hue[0.4580886691838912, 1, 1], 
                    Hue[0.527587973755167, 1, 1], 
                    Hue[0.527587973755167, 1, 1], 
                    Hue[0.5843741913273628, 1, 1], 
                    Hue[0.5843741913273628, 1, 1], 
                    Hue[0.6529738372754882, 1, 1], 
                    Hue[0.6529738372754882, 1, 1], 
                    Hue[0.7231717946712797, 1, 1], 
                    Hue[0.7231717946712797, 1, 1], 
                    Hue[0.7940198162849148, 1, 1], 
                    Hue[0.7940198162849148, 1, 1], 
                    Hue[0.8670233124123604, 1, 1], 
                    Hue[0.8670233124123604, 1, 1], 
                    Hue[0.9376088157986627, 1, 1], 
                    Hue[0.9376088157986627, 1, 1], 
                    Hue[0.008213234289735359, 1, 1], 
                    Hue[0.008213234289735359, 1, 1], 
                    Hue[0.07954226466320549, 1, 1], 
                    Hue[0.07954226466320549, 1, 1], 
                    Hue[0.15182109711666714`, 1, 1], 
                    Hue[0.15182109711666714`, 1, 1], 
                    Hue[0.22507428783865907`, 1, 1], 
                    Hue[0.22507428783865907`, 1, 1], 
                    Hue[0.298601788645345, 1, 1], 
                    Hue[0.298601788645345, 1, 1], 
                    Hue[0.37261978757492364`, 1, 1], 
                    Hue[0.37261978757492364`, 1, 1], 
                    Hue[0.4464695067172625, 1, 1], 
                    Hue[0.4464695067172625, 1, 1], 
                    Hue[0.520985703519601, 1, 1], 
                    Hue[0.520985703519601, 1, 1], 
                    Hue[0.5962503222699094, 1, 1], 
                    Hue[0.5962503222699094, 1, 1], 
                    Hue[0.6720211162908327, 1, 1], 
                    Hue[0.6720211162908327, 1, 1], 
                    Hue[0.7482495212329322, 1, 1], 
                    Hue[0.7482495212329322, 1, 1], 
                    Hue[0.8250046849443391, 1, 1], 
                    Hue[0.8250046849443391, 1, 1], 
                    Hue[0.9021656385273559, 1, 1], 
                    Hue[0.9021656385273559, 1, 1], 
                    Hue[0.9798307401607916, 1, 1], 
                    Hue[0.9798307401607916, 1, 1], 
                    Hue[0.05800235227659842, 1, 1], 
                    Hue[0.05800235227659842, 1, 1], 
                    Hue[0.13669273281546193`, 1, 1], 
                    Hue[0.13669273281546193`, 1, 1], 
                    Hue[0.21585977065513692`, 1, 1], 
                    Hue[0.21585977065513692`, 1, 1], 
                    Hue[0.29548816845309706`, 1, 1], 
                    Hue[0.29548816845309706`, 1, 1], 
                    Hue[0.37560341603106856`, 1, 1], 
                    Hue[0.37560341603106856`, 1, 1], 
                    Hue[0.4562040007508113, 1, 1], 
                    Hue[0.4562040007508113, 1, 1], 
                    Hue[0.5373038430435617, 1, 1], 
                    Hue[0.5373038430435617, 1, 1], 
                    Hue[0.6188951496447265, 1, 1], 
                    Hue[0.6188951496447265, 1, 1], 
                    Hue[0.7009511042349899, 1, 1], 
                    Hue[0.7009511042349899, 1, 1], 
                    Hue[0.7835073631502996, 1, 1], 
                    Hue[0.7835073631502996, 1, 1], 
                    Hue[0.8665536235626842, 1, 1], 
                    Hue[0.8665536235626842, 1, 1], 
                    Hue[0.9500892860673488, 1, 1], 
                    Hue[0.9500892860673488, 1, 1], 
                    Hue[0.034106981671226305`, 1, 1], 
                    Hue[0.034106981671226305`, 1, 1], 
                    Hue[0.11860815259491314`, 1, 1], 
                    Hue[0.11860815259491314`, 1, 1], 
                    Hue[0.2035904553821011, 1, 1], 
                    Hue[0.2035904553821011, 1, 1], 
                    Hue[0.28906409794720567`, 1, 1], 
                    Hue[0.28906409794720567`, 1, 1], 
                    Hue[0.37502184323541427`, 1, 1], 
                    Hue[0.37502184323541427`, 1, 1], 
                    Hue[0.4614619759364616, 1, 1], 
                    Hue[0.4614619759364616, 1, 1], 
                    Hue[0.5483815239488417, 1, 1], 
                    Hue[0.5483815239488417, 1, 1], 
                    Hue[0.63578442230926, 1, 1], 
                    Hue[0.63578442230926, 1, 1], 
                    Hue[0.7236676785821858, 1, 1], 
                    Hue[0.7236676785821858, 1, 1], 
                    Hue[0.8120314596811757, 1, 1], 
                    Hue[0.8120314596811757, 1, 1], 
                    Hue[0.900873601632316, 1, 1], 
                    Hue[0.900873601632316, 1, 1], 
                    Hue[0.9901942213417271, 1, 1], 
                    Hue[0.9901942213417271, 1, 1], 
                    Hue[0.0799895702292984, 1, 1], 
                    Hue[0.0799895702292984, 1, 1], 
                    Hue[0.170260706962758, 1, 1], 
                    Hue[0.170260706962758, 1, 1], 
                    Hue[0.2610034850012405, 1, 1], 
                    Hue[0.2610034850012405, 1, 1], 
                    Hue[0.35221637057648686`, 1, 1], 
                    Hue[0.35221637057648686`, 1, 1], 
                    Hue[0.44389826161878776`, 1, 1], 
                    Hue[0.44389826161878776`, 1, 1], 
                    Hue[0.5360449736481047, 1, 1], 
                    Hue[0.5360449736481047, 1, 1], 
                    Hue[0.6286541129036205, 1, 1], 
                    Hue[0.6286541129036205, 1, 1], 
                    Hue[0.7217217985068132, 1, 1], 
                    Hue[0.7217217985068132, 1, 1], 
                    Hue[0.8152433603000737, 1, 1], 
                    Hue[0.8152433603000737, 1, 1], 
                    Hue[0.9092139707072192, 1, 1], 
                    Hue[0.9092139707072192, 1, 1], 
                    Hue[0.0036290315923241745`, 1, 1], 
                    Hue[0.0036290315923241745`, 1, 1], 
                    Hue[0.09848073066420522, 1, 1], 
                    Hue[0.09848073066420522, 1, 1], 
                    Hue[0.19376227185500616`, 1, 1], 
                    Hue[0.19376227185500616`, 1, 1], 
                    Hue[0.289464672794272, 1, 1], 
                    Hue[0.289464672794272, 1, 1], 
                    Hue[0.3855781333910314, 1, 1], 
                    Hue[0.3855781333910314, 1, 1], 
                    Hue[0.4820908974496971, 1, 1], 
                    Hue[0.4820908974496971, 1, 1], 
                    Hue[0.5789897695238124, 1, 1], 
                    Hue[0.5789897695238124, 1, 1], 
                    Hue[0.6762580878511103, 1, 1], 
                    Hue[0.6762580878511103, 1, 1], 
                    Hue[0.7738792026198127, 1, 1], 
                    Hue[0.7738792026198127, 1, 1], 
                    Hue[0.8718316149733819, 1, 1], 
                    Hue[0.8718316149733819, 1, 1], 
                    Hue[0.9700914448448266, 1, 1], 
                    Hue[0.9700914448448266, 1, 1], 
                    Hue[0.06863078135729278, 1, 1], 
                    Hue[0.06863078135729278, 1, 1], 
                    Hue[0.1674173699230757, 1, 1], 
                    Hue[0.1674173699230757, 1, 1], 
                    Hue[0.2664150890214937, 1, 1], 
                    Hue[0.2664150890214937, 1, 1], 
                    Hue[0.36558201499268755`, 1, 1], 
                    Hue[0.36558201499268755`, 1, 1], 
                    Hue[0.4648696110448278, 1, 1], 
                    Hue[0.4648696110448278, 1, 1], 
                    Hue[0.5642232715551221, 1, 1], 
                    Hue[0.5642232715551221, 1, 1], 
                    Hue[0.6635811232888242, 1, 1], 
                    Hue[0.6635811232888242, 1, 1], 
                    Hue[0.7628731825198262, 1, 1], 
                    Hue[0.7628731825198262, 1, 1], 
                    Hue[0.8620209439738609, 1, 1], 
                    Hue[0.8620209439738609, 1, 1], 
                    Hue[0.960936178647772, 1, 1], 
                    Hue[0.960936178647772, 1, 1], 
                    Hue[0.05952243736322674, 1, 1], 
                    Hue[0.05952243736322674, 1, 1], 
                    Hue[0.1576739410013476, 1, 1], 
                    Hue[0.1576739410013476, 1, 1], 
                    Hue[0.25527613254844134`, 1, 1], 
                    Hue[0.25527613254844134`, 1, 1], 
                    Hue[0.3522059883891288, 1, 1], 
                    Hue[0.3522059883891288, 1, 1], 
                    Hue[0.4483344792343556, 1, 1], 
                    Hue[0.4483344792343556, 1, 1], 
                    Hue[0.5435302454469617, 1, 1], 
                    Hue[0.5435302454469617, 1, 1], 
                    Hue[0.6376602769840898, 1, 1], 
                    Hue[0.6376602769840898, 1, 1], 
                    Hue[0.7305964827538581, 1, 1], 
                    Hue[0.7305964827538581, 1, 1], 
                    Hue[0.822219008183593, 1, 1], 
                    Hue[0.822219008183593, 1, 1], 
                    Hue[0.9124241755271605, 1, 1], 
                    Hue[0.9124241755271605, 1, 1], 
                    Hue[0.0011313352912279788`, 1, 1], 
                    Hue[0.0011313352912279788`, 1, 1], 
                    Hue[0.08829537377171129, 1, 1], 
                    Hue[0.08829537377171129, 1, 1], 
                    Hue[0.173914091059342, 1, 1], 
                    Hue[0.173914091059342, 1, 1], 
                    Hue[0.2580327570304501, 1, 1], 
                    Hue[0.2580327570304501, 1, 1], 
                    Hue[0.34074391743273624`, 1, 1], 
                    Hue[0.34074391743273624`, 1, 1], 
                    Hue[0.42218350829950074`, 1, 1], 
                    Hue[0.42218350829950074`, 1, 1], 
                    Hue[0.5025176745363464, 1, 1], 
                    Hue[0.5025176745363464, 1, 1], 
                    Hue[0.5819340841751708, 1, 1], 
                    Hue[0.5819340841751708, 1, 1], 
                    Hue[0.6606322089404484, 1, 1], 
                    Hue[0.6606322089404484, 1, 1], 
                    Hue[0.7388112677112173, 1, 1], 
                    Hue[0.7388112677112173, 1, 1], 
                    Hue[0.8166629771094296, 1, 1], 
                    Hue[0.8166629771094296, 1, 1], 
                    Hue[0.894367291049327, 1, 1], 
                    Hue[0.894367291049327, 1, 1], 
                    Hue[0.9720925084202446, 1, 1], 
                    Hue[0.9720925084202446, 1, 1], 
                    Hue[0.04998896335048285, 1, 1], 
                    Hue[0.04998896335048285, 1, 1], 
                    Hue[0.1281877525899212, 1, 1], 
                    Hue[0.1281877525899212, 1, 1], 
                    Hue[0.20680136452066197`, 1, 1], 
                    Hue[0.20680136452066197`, 1, 1], 
                    Hue[0.28593303904703327`, 1, 1], 
                    Hue[0.28593303904703327`, 1, 1], 
                    Hue[0.36567016562670085`, 1, 1], 
                    Hue[0.36567016562670085`, 1, 1], 
                    Hue[0.44607683475779675`, 1, 1], 
                    Hue[0.44607683475779675`, 1, 1], 
                    Hue[0.5272080444436378, 1, 1], 
                    Hue[0.5272080444436378, 1, 1], 
                    Hue[0.6091162047792973, 1, 1], 
                    Hue[0.6091162047792973, 1, 1], 
                    Hue[0.691847083043619, 1, 1], 
                    Hue[0.691847083043619, 1, 1], 
                    Hue[0.7754258035423244, 1, 1], 
                    Hue[0.7754258035423244, 1, 1], 
                    Hue[0.8598681254734846, 1, 1], 
                    Hue[0.8598681254734846, 1, 1], 
                    Hue[0.9452056790005225, 1, 1], 
                    Hue[0.9452056790005225, 1, 1], 
                    Hue[0.03147188112713823, 1, 1], 
                    Hue[0.03147188112713823, 1, 1], 
                    Hue[0.11867304215521769`, 1, 1], 
                    Hue[0.11867304215521769`, 1, 1], 
                    Hue[0.20676513826419657`, 1, 1], 
                    Hue[0.20676513826419657`, 1, 1], 
                    Hue[0.2956907086256168, 1, 1], 
                    Hue[0.2956907086256168, 1, 1], 
                    Hue[0.3854998251506179, 1, 1], 
                    Hue[0.3854998251506179, 1, 1], 
                    Hue[0.4763031405657942, 1, 1], 
                    Hue[0.4763031405657942, 1, 1], 
                    Hue[0.5680698599260738, 1, 1], 
                    Hue[0.5680698599260738, 1, 1], 
                    Hue[0.6606728731331002, 1, 1], 
                    Hue[0.6606728731331002, 1, 1], 
                    Hue[0.754025841465475, 1, 1], 
                    Hue[0.754025841465475, 1, 1], 
                    Hue[0.8480804931757541, 1, 1], 
                    Hue[0.8480804931757541, 1, 1], 
                    Hue[0.9431181536174291, 1, 1], 
                    Hue[0.9431181536174291, 1, 1], 
                    Hue[0.0397869542393286, 1, 1], 
                    Hue[0.0397869542393286, 1, 1], 
                    Hue[0.137840700355691, 1, 1], 
                    Hue[0.137840700355691, 1, 1], 
                    Hue[0.23574526072148794`, 1, 1], 
                    Hue[0.23574526072148794`, 1, 1], 
                    Hue[0.333496700037884, 1, 1], 
                    Hue[0.333496700037884, 1, 1], 
                    Hue[0.43380775123773196`, 1, 1], 
                    Hue[0.43380775123773196`, 1, 1], 
                    Hue[0.536904103339749, 1, 1], 
                    Hue[0.536904103339749, 1, 1], 
                    Hue[0.6388111314179277, 1, 1], 
                    Hue[0.6388111314179277, 1, 1], 
                    Hue[0.7395416522567456, 1, 1], 
                    Hue[0.7395416522567456, 1, 1], 
                    Hue[0.8447363652466423, 1, 1], 
                    Hue[0.8447363652466423, 1, 1], 
                    Hue[0.9501147795477853, 1, 1], 
                    Hue[0.9501147795477853, 1, 1], 
                    Hue[0.04045361918740696, 1, 1], 
                    Hue[0.04045361918740696, 1, 1], 
                    Hue[0.1286078858347576, 1, 1], 
                    Hue[0.1286078858347576, 1, 1], 
                    Hue[0.24672799717042326`, 1, 1], 
                    Hue[0.24672799717042326`, 1, 1], 
                    Hue[0.37861522354390637`, 1, 1], 
                    Hue[0.37861522354390637`, 1, 1], 
                    Hue[0.45736632935843075`, 1, 1], 
                    Hue[0.45736632935843075`, 1, 1], 
                    Hue[0.49308730974207005`, 1, 1], 
                    Hue[0.49308730974207005`, 1, 1], 
                    Hue[0.6470686768988838, 1, 1], 
                    Hue[0.6470686768988838, 1, 1], 
                    Hue[0.8568677735491859, 1, 1], 
                    Hue[0.8568677735491859, 1, 1], 
                    Hue[0.10431225644341774`, 1, 1], 
                    Hue[0.10431225644341774`, 1, 1], 
                    Hue[0.3894102905074262, 1, 1], 
                    Hue[0.3894102905074262, 1, 1], 
                    Hue[0.6838263781448488, 1, 1], 
                    Hue[0.6838263781448488, 1, 1], 
                    Hue[0.9569069071984381, 1, 1], 
                    Hue[0.9569069071984381, 1, 1], 
                    Hue[0.21395671894896143`, 1, 1], 
                    Hue[0.21395671894896143`, 1, 1], 
                    Hue[0.4664485349712569, 1, 1], 
                    Hue[0.4664485349712569, 1, 1], 
                    Hue[0.7219258772292536, 1, 1], 
                    Hue[0.7219258772292536, 1, 1], 
                    Hue[0.9856372041087308, 1, 1], 
                    Hue[0.9856372041087308, 1, 1], 
                    Hue[0.2608722601555482, 1, 1], 
                    Hue[0.2608722601555482, 1, 1], 
                    Hue[0.5470837038038487, 1, 1], 
                    Hue[0.5470837038038487, 1, 1], 
                    Hue[0.8401196088581602, 1, 1], 
                    Hue[0.8401196088581602, 1, 1], 
                    Hue[0.13624437333826112`, 1, 1], 
                    Hue[0.13624437333826112`, 1, 1], 
                    Hue[0.4309682311873325, 1, 1], 
                    Hue[0.4309682311873325, 1, 1], 
                    Hue[0.7184660058323185, 1, 1], 
                    Hue[0.7184660058323185, 1, 1], 
                    Hue[0.9973525953613468, 1, 1], 
                    Hue[0.9973525953613468, 1, 1], 
                    Hue[0.2702841777599323, 1, 1], 
                    Hue[0.2702841777599323, 1, 1], 
                    Hue[0.5401052324973117, 1, 1], 
                    Hue[0.5401052324973117, 1, 1], 
                    Hue[0.8084590354995888, 1, 1], 
                    Hue[0.8084590354995888, 1, 1], 
                    Hue[0.07594959588080527, 1, 1], 
                    Hue[0.07594959588080527, 1, 1], 
                    Hue[0.34257008552859086`, 1, 1], 
                    Hue[0.34257008552859086`, 1, 1], 
                    Hue[0.608041503369714, 1, 1], 
                    Hue[0.608041503369714, 1, 1], 
                    Hue[0.8720135963849045, 1, 1], 
                    Hue[0.8720135963849045, 1, 1], 
                    Hue[0.13414354537958378`, 1, 1], 
                    Hue[0.13414354537958378`, 1, 1], 
                    Hue[0.3940928156514941, 1, 1], 
                    Hue[0.3940928156514941, 1, 1], 
                    Hue[0.6514812971209296, 1, 1], 
                    Hue[0.6514812971209296, 1, 1], 
                    Hue[0.9058301300769357, 1, 1], 
                    Hue[0.9058301300769357, 1, 1], 
                    Hue[0.1565116040087216, 1, 1], 
                    Hue[0.1565116040087216, 1, 1], 
                    Hue[0.4027360261695492, 1, 1], 
                    Hue[0.4027360261695492, 1, 1], 
                    Hue[0.6436450448936091, 1, 1], 
                    Hue[0.6436450448936091, 1, 1], 
                    Hue[0.8786518091264015, 1, 1], 
                    Hue[0.8786518091264015, 1, 1], 
                    Hue[0.10814245304970796`, 1, 1], 
                    Hue[0.10814245304970796`, 1, 1], 
                    Hue[0.3342040087120603, 1, 1], 
                    Hue[0.3342040087120603, 1, 1], 
                    Hue[0.5603417438283603, 1, 1], 
                    Hue[0.5603417438283603, 1, 1], 
                    Hue[0.7898375376869549, 1, 1], 
                    Hue[0.7898375376869549, 1, 1], 
                    Hue[0.02435059756972669, 1, 1], 
                    Hue[0.02435059756972669, 1, 1], 
                    Hue[0.263972200373601, 1, 1], 
                    Hue[0.263972200373601, 1, 1], 
                    Hue[0.5080091112724794, 1, 1], 
                    Hue[0.5080091112724794, 1, 1], 
                    Hue[0.7555873580958158, 1, 1], 
                    Hue[0.7555873580958158, 1, 1], 
                    Hue[0.005910038993356071, 1, 1], 
                    Hue[0.005910038993356071, 1, 1], 
                    Hue[0.2583208773049551, 1, 1], 
                    Hue[0.2583208773049551, 1, 1], 
                    Hue[0.5122990607419959, 1, 1], 
                    Hue[0.5122990607419959, 1, 1], 
                    Hue[0.7674399787929954, 1, 1], 
                    Hue[0.7674399787929954, 1, 1], 
                    Hue[0.02343753155512082, 1, 1], 
                    Hue[0.02343753155512082, 1, 1], 
                    Hue[0.2800651076356245, 1, 1], 
                    Hue[0.2800651076356245, 1, 1], 
                    Hue[0.5371545059966867, 1, 1], 
                    Hue[0.5371545059966867, 1, 1], 
                    Hue[0.7945733996717301, 1, 1], 
                    Hue[0.7945733996717301, 1, 1], 
                    Hue[0.052204389580622186`, 1, 1], 
                    Hue[0.052204389580622186`, 1, 1], 
                    Hue[0.30993216728720485`, 1, 1], 
                    Hue[0.30993216728720485`, 1, 1], 
                    Hue[0.5676409258558874, 1, 1], 
                    Hue[0.5676409258558874, 1, 1], 
                    Hue[0.8252245236518228, 1, 1], 
                    Hue[0.8252245236518228, 1, 1], 
                    Hue[0.08260688618784486, 1, 1], 
                    Hue[0.08260688618784486, 1, 1], 
                    Hue[0.3397704033682906, 1, 1], 
                    Hue[0.3397704033682906, 1, 1], 
                    Hue[0.5967914666911367, 1, 1], 
                    Hue[0.5967914666911367, 1, 1], 
                    Hue[0.8538866103988598, 1, 1], 
                    Hue[0.8538866103988598, 1, 1], 
                    Hue[0.11148110441383434`, 1, 1], 
                    Hue[0.11148110441383434`, 1, 1], 
                    Hue[0.37032221044370817`, 1, 1], 
                    Hue[0.37032221044370817`, 1, 1], 
                    Hue[0.6316738642015586, 1, 1], 
                    Hue[0.6316738642015586, 1, 1], 
                    Hue[0.8976395052931159, 1, 1], 
                    Hue[0.8976395052931159, 1, 1], 
                    Hue[0.17158596189859066`, 1, 1], 
                    Hue[0.17158596189859066`, 1, 1], 
                    Hue[0.45806602624555387`, 1, 1], 
                    Hue[0.45806602624555387`, 1, 1], 
                    Hue[0.759613669603346, 1, 1], 
                    Hue[0.759613669603346, 1, 1], 
                    Hue[0.06876575145420652, 1, 1], 
                    Hue[0.06876575145420652, 1, 1], 
                    Hue[0.3704520490070801, 1, 1], 
                    Hue[0.3704520490070801, 1, 1], 
                    Hue[0.6581400003275106, 1, 1], 
                    Hue[0.6581400003275106, 1, 1], 
                    Hue[0.9342542877440975, 1, 1], 
                    Hue[0.9342542877440975, 1, 1], 
                    Hue[0.20258989419990772`, 1, 1], 
                    Hue[0.20258989419990772`, 1, 1], 
                    Hue[0.46580311190130813`, 1, 1], 
                    Hue[0.46580311190130813`, 1, 1], 
                    Hue[0.7255298424272546, 1, 1], 
                    Hue[0.7255298424272546, 1, 1], 
                    Hue[0.9828031804668734, 1, 1], 
                    Hue[0.9828031804668734, 1, 1], 
                    Hue[0.2383262257411595, 1, 1], 
                    Hue[0.2383262257411595, 1, 1], 
                    Hue[0.4926179409024767, 1, 1], 
                    Hue[0.4926179409024767, 1, 1], 
                    Hue[0.7460921021000613, 1, 1], 
                    Hue[0.7460921021000613, 1, 1]}],
                   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
                   Axes->{True, None},
                   AxesOrigin->{0, 0},
                   
                   Epilog->LineBox[{{1, 1.6982610874419622`*^-6}, {
                    2, 1.9647701693871465`*^-6}, {
                    3, 2.0948437179045684`*^-6}, {
                    4, 2.0302758518194934`*^-6}, {
                    5, 2.0415101989582863`*^-6}, {
                    6, 2.4237144694841933`*^-6}, {7, 2.263507117589174*^-6}, {
                    8, 2.15188375044499*^-6}, {9, 2.389458724731309*^-6}, {
                    10, 2.2209139272981044`*^-6}, {
                    11, 2.0436269132659483`*^-6}, {
                    12, 1.7497485462304771`*^-6}, {
                    13, 1.7704495747455398`*^-6}, {
                    14, 1.4545976910234285`*^-6}, {
                    15, 1.76508901076846*^-6}, {
                    16, 1.6365180323201764`*^-6}, {
                    17, 1.7531282690360914`*^-6}, {
                    18, 1.1382589000422142`*^-6}, {
                    19, 1.2741942634533994`*^-6}, {
                    20, 1.2716663098030212`*^-6}, {
                    21, 1.5070659336485792`*^-6}, {
                    22, 1.358278950778587*^-6}, {
                    23, 1.2723159736908188`*^-6}, {
                    24, 1.428478902232972*^-6}, {
                    25, 1.5594612339969622`*^-6}, {
                    26, 2.0812536031578334`*^-6}, {
                    27, 2.0599880018011587`*^-6}, {
                    28, 1.911509335940612*^-6}, {29, 1.275693584243608*^-6}, {
                    30, 5.592922461325838*^-7}, {
                    31, 1.5033435767768933`*^-6}, {
                    32, 2.6308798571919916`*^-6}, {
                    33, 4.4215774115370905`*^-6}, {
                    34, 5.12757909305354*^-6}, {35, 5.72760521533108*^-6}, {
                    36, 8.227998861627223*^-6}, {
                    37, 0.000011496751643690234`}, {
                    38, 0.00001641178926049975}, {
                    39, 0.000022328379297166416`}, {
                    40, 0.000029130572054353337`}, {
                    41, 0.000038573511936454706`}, {
                    42, 0.00005142662480180197}, {
                    43, 0.00006810732508637353}, {
                    44, 0.0000911546459833953}, {
                    45, 0.00012077470985243289`}, {
                    46, 0.00015796610167937512`}, {
                    47, 0.0002064050644225185}, {
                    48, 0.00026808475547251704`}, {
                    49, 0.0003470294687037318}, {
                    50, 0.00044956544971822574`}, {
                    51, 0.0005780051125679727}, {52, 0.0007399352695906527}, {
                    53, 0.0009408092852317185}, {
                    54, 0.0011929766658488746`}, {
                    55, 0.0015061783531894648`}, {56, 0.001894756352925618}, {
                    57, 0.0023719167714152003`}, {58, 0.002956496462836706}, {
                    59, 0.003668471692467315}, {60, 0.004533488441720348}, {
                    61, 0.005578673811343152}, {62, 0.006834720140660297}, {
                    63, 0.008337922235369858}, {64, 0.010125773580268912`}, {
                    65, 0.012244541904668123`}, {66, 0.014742906615013941`}, {
                    67, 0.017672704373976108`}, {68, 0.02109220059254929}, {
                    69, 0.02506509972756395}, {70, 0.029656494270082816`}, {
                    71, 0.03493720991780949}, {72, 0.04097893986486279}, {
                    73, 0.04785463741800247}, {74, 0.055640216363719255`}, {
                    75, 0.0644125128869491}, {76, 0.07424265631887662}, {
                    77, 0.08519935490738817}, {78, 0.09734634691951272}, {
                    79, 0.11073877360399402`}, {80, 0.1254249289938402}, {
                    81, 0.14143861107288347`}, {82, 0.15879884542870257`}, {
                    83, 0.1775115044913922}, {84, 0.19756209466556782`}, {
                    85, 0.2189171098734944}, {86, 0.24151841102316288`}, {
                    87, 0.26528960762072673`}, {88, 0.29012622675921884`}, {
                    89, 0.315901005078622}, {90, 0.3424631209840068}, {
                    91, 0.36963656854100463`}, {92, 0.39722223248745586`}, {
                    93, 0.4250025479830893}, {94, 0.4527414477605905}, {
                    95, 0.4801855720565026}, {96, 0.5070706215499177}, {
                    97, 0.5331265865110192}, {98, 0.5580784883728015}, {
                    99, 0.5816552522714593}, {100, 0.6035882809440296}, {
                    101, 0.6236209000280355}, {102, 0.6415165287012181}, {
                    103, 0.6570557732577346}, {104, 0.6700446279125143}, {
                    105, 0.6803152854825002}, {106, 0.687734108920405}, {
                    107, 0.6921986813412877}, {108, 0.6936453454100678}, {
                    109, 0.6920399413480028}, {110, 0.6873869749461042}, {
                    111, 0.6797249679215843}, {112, 0.6691278665235665}, {
                    113, 0.6556921635290287}, {114, 0.6395453447840154}, {
                    115, 0.6208382616754691}, {116, 0.5997411293807583}, {
                    117, 0.5764401704231145}, {118, 0.5511334776968638}, {
                    119, 0.5240305068689864}, {120, 0.49534997192265817`}, {
                    121, 0.4653249818133817}, {122, 0.43419970261802016`}, {
                    123, 0.40223057501093107`}, {124, 0.36969224814526735`}, {
                    125, 0.3368861729576676}, {126, 0.3041515786051314}, {
                    127, 0.2718703623851708}, {128, 0.24046192898104318`}, {
                    129, 0.21035706497599305`}, {130, 0.1819649617957836}, {
                    131, 0.1556411215041802}, {132, 0.1316517549384237}, {
                    133, 0.11015913659298437`}, {134, 0.09121860431563601}, {
                    135, 0.07478808492258321}, {136, 0.06074390577387306}, {
                    137, 0.048902053562975036`}, {138, 0.03904220194724473}, {
                    139, 0.030928136282631313`}, {
                    140, 0.024322179759817087`}, {141, 0.01899566338179031}, {
                    142, 0.014739178891111785`}, {143, 0.01136650432361027}, {
                    144, 0.008714954975170101}, {145, 0.006644985971218271}, {
                    146, 0.005039590827510158}, {147, 0.003802605791960672}, {
                    148, 0.0028553383581204775`}, {
                    149, 0.00213379177038994}, {
                    150, 0.0015868030531876213`}, {
                    151, 0.001174223915727052}, {
                    152, 0.0008650182387141355}, {
                    153, 0.0006347903835024599}, {
                    154, 0.00046396334584149164`}, {
                    155, 0.00033749842409038503`}, {
                    156, 0.0002443422560242348}, {
                    157, 0.00017616107162067316`}, {
                    158, 0.0001266309479384828}, {
                    159, 0.00009092834283296823}, {
                    160, 0.00006509498741269296}, {
                    161, 0.00004614899305824531}, {
                    162, 0.00003242317121705183}, {
                    163, 0.000022869994455753856`}, {
                    164, 0.000016203980099513416`}, {
                    165, 0.000011289214982628973`}, {
                    166, 7.715153041070073*^-6}, {
                    167, 5.322149728176927*^-6}, {
                    168, 3.6676550396962726`*^-6}, {
                    169, 2.3935315225552227`*^-6}, {
                    170, 1.549880981005627*^-6}, {
                    171, 1.167067405975017*^-6}, {
                    172, 9.062389395707861*^-7}, {
                    173, 5.708653782239252*^-7}, {
                    174, 2.8945899973879995`*^-7}, {
                    175, 2.967207380330455*^-7}, {
                    176, 3.653042334172365*^-7}, {
                    177, 3.4204143315030274`*^-7}, {
                    178, 2.667783534666879*^-7}, {
                    179, 2.0697146870377512`*^-7}, {
                    180, 1.896090490189446*^-7}, {
                    181, 1.832458510126788*^-7}, {
                    182, 1.6261564615772594`*^-7}, {
                    183, 1.286417963815216*^-7}, {
                    184, 9.169261386046591*^-8}, {
                    185, 6.053939613475948*^-8}, {
                    186, 3.885686033006999*^-8}, {
                    187, 2.5900849836937016`*^-8}, {
                    188, 1.899370168661009*^-8}, {
                    189, 1.5862338164688535`*^-8}, {
                    190, 1.5241901735972995`*^-8}, {
                    191, 1.6187204930059355`*^-8}, {
                    192, 1.7713463652247404`*^-8}, {
                    193, 1.9005458583186304`*^-8}, {
                    194, 1.9590719033333195`*^-8}, {
                    195, 1.932363266290875*^-8}, {
                    196, 1.827823135123405*^-8}, {
                    197, 1.6635355337833728`*^-8}, {
                    198, 1.4605923721175196`*^-8}, {
                    199, 1.2392123394311898`*^-8}, {
                    200, 1.0169865918038008`*^-8}, {
                    201, 8.079218824983797*^-9}, {
                    202, 6.218712054094677*^-9}, {
                    203, 4.644218033499428*^-9}, {
                    204, 3.3732348073758506`*^-9}, {
                    205, 2.3930921865282627`*^-9}, {
                    206, 1.6708413960370641`*^-9}, {
                    207, 1.1623881569200185`*^-9}, {
                    208, 8.195943176920906*^-10}, {
                    209, 5.954823945304898*^-10}, {
                    210, 4.4886240724376257`*^-10}, {
                    211, 3.4852163178805556`*^-10}, {
                    212, 2.746945337663734*^-10}, {
                    213, 2.1685384809247283`*^-10}, {
                    214, 1.7002102768423432`*^-10}, {
                    215, 1.318386451064352*^-10}, {
                    216, 1.0094732808474467`*^-10}, {
                    217, 7.629426594331476*^-11}, {
                    218, 5.6915407756874474`*^-11}, {
                    219, 4.190743787146259*^-11}, {
                    220, 3.0449648929368786`*^-11}, {
                    221, 2.1824113674691968`*^-11}, {
                    222, 1.5421888997366352`*^-11}, {
                    223, 1.0738372429719186`*^-11}, {
                    224, 7.36311575431131*^-12}, {
                    225, 4.967952894865655*^-12}, {
                    226, 3.295153981128586*^-12}, {
                    227, 2.1460575514314102`*^-12}, {
                    228, 1.3704134411195273`*^-12}, {
                    229, 8.566435530940083*^-13}, {
                    230, 5.232977769443833*^-13}, {
                    231, 3.11890589369434*^-13}, {
                    232, 1.8113341000573393`*^-13}, {
                    233, 1.0243158000355152`*^-13}, {
                    234, 5.6426822851840584`*^-14}, {
                    235, 3.036609779468772*^-14}, {
                    236, 1.6112304934858916`*^-14}, {
                    237, 8.65417514361298*^-15}, {
                    238, 4.993682472955497*^-15}, {
                    239, 3.321981832551148*^-15}, {
                    240, 2.5355995858752223`*^-15}, {
                    241, 2.0660815329268104`*^-15}, {
                    242, 1.7009075383757556`*^-15}, {
                    243, 1.3815175723394891`*^-15}, {
                    244, 1.0973438723643075`*^-15}, {
                    245, 8.494648561693853*^-16}, {
                    246, 6.399647522652412*^-16}, {
                    247, 4.690292980897647*^-16}, {
                    248, 3.344884025643928*^-16}, {
                    249, 2.322805531343931*^-16}, {
                    250, 1.5724757171557174`*^-16}, {
                    251, 1.0393113940608272`*^-16}}],
                   Frame->True,
                   FrameTicks->{{{1., 
                    FormBox[
                    RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
                    FormBox[
                    RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
                    FormBox[
                    RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
                    FormBox[
                    RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
                    FormBox["0", TraditionalForm]}, {157.25, 
                    FormBox["2.5`", TraditionalForm]}, {188.5, 
                    FormBox["5", TraditionalForm]}, {219.75, 
                    FormBox["7.5`", TraditionalForm]}, {251., 
                    FormBox["10", TraditionalForm]}}, Automatic, None, None},
                   PlotRange->{-0.1, 1.1},
                   PlotRangeClipping->True,
                   PlotRangePadding->{
                    Scaled[0.02], Automatic}], 8->
                  GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF1mdYVAcWxvGLWQP2soqxRAY1FgQEVMT+ilIEUXpHLkMZOhFRs6jJlYDR
BA2KRmOKg4UoLgiCYogbr4qsxC5hESNhsIs0S3AtxHWfObP77R4/6PPzzOuf
5syYq1O8o7sJgtD+9rf//qn/1TF3eG55v/b2U3PoL5D/TrFuyO4zZBN8Ncsp
pLDN4P7Y7ZhR91u14fGDkGa1p5eb6Unye2j81/1tA4b9RB6BZXG7nI6MKier
0G+QnXdt/VHyKMz9OO1gD7sfyWOQumnfVs2jI+Sx8O3lmhvaVkoeDzcx82BC
9zKyBeSXPa1vTj5GtkRDa4jlw52Gx1vDeKxDipBrsA1uFm48mehm6NkifcWs
VwVTfybbIf+3j41WTzR8PpPh+ZmNPMHH8PFPgUVvjev3fob9VOw6OQhHPSrI
9jg0ccbdwiKDp8EK7ReMHhj6DjBJ7WNy+QvD5zcdXVlLJ9bbHiDPgLvREZNh
QhF5Jp6WDj4140geeRaKHvTUZLTnk2cjxHPX3imq78hzMCtjcrapbHj8XIgl
5m2/Xc4hAz813jZuOqjVWwKGtlX90hW9he7zYLJjVOma5B10n4ejuq33H089
THdH7Nhg+e30Lf+guyOMalPiTfZX0n0+xmpas2/anaP7fJjYLp7SHFJN9wUY
c3L/hIDci3RfgHnHOl0q7l6juxN+ObAy2NdeR3cnDC0c3uNB3/t0d8ZN09m2
g3e30N0Z6ZNezilY+ITuLlgZ7pa3OPzfdHdBhOWmReW1b+juivh1TyYuWdN9
rv7uinEFvr5HzvbWW1iIjPftbwWUDqT7QqTnSuLC4Pfo7ob42GuZP54wo7sb
PK0L+rd+PYbu7midjaK131rQ3R2J440Ovjazpfsi/BLc9ujP36fSfRHWdQzs
t/rWTLp7YM6AnenfHZ9Hdw+MbrwduTzRme6LcbzBt0R0XkT3xSiZ1aP1yAkv
ui9Bj5X2Vc1W/nRfgt/+GD306d4Qunti5KOaKwfGR+gNT3Qfa16wziGaHu+J
NwExey+q4vWWPdFoOWekbfcPae+FCymB5VFbl9PeC0+1tdvvXV1Fey+MH25W
U9a4hvZeKHiTWN2etI723jjjnOO0vDOT9t7IzHl5uX74Rtp7I6chb7n4xyba
e+OYs2tpzsUttPfB6oaIMX3/up32PihT23tMrt9Jex+0X99za8Ck72jvg2Fr
Rzf9MSCP9r7o1+3B6/bbe2nvi1V/jsivGPwD7X1R3/dz48rDBbT3Rd/jg58V
VRTR3g+jn5mbztldQns/fLRpY8EG8zLa+yH+fuagYQ+O0d4PXqPLfVw7f6S9
P9Km5UqDlvyD9v6oP1gQtzxdpr0/NpX760rTTtPeH9J9T3XdV5W0D4BL3814
pquifQA0xpdS93tX0z4Ak3DP53nzedoH4MmGu/7eBy7RPhA9jWvDgyqv0D4Q
gYF+G8sWXKN9IA6meTtk19TQPhCmFcV9HibX0j4IU/bWvk4fVEf7IMT5fTy/
9NR12gch3HjCoDPLb9A+CAktK2v3jLtJ+2BMyA1w76pvoH0wanRPvNWfNdI+
GFZ+afYjv9LRPhg5prdfrVU30T4EMV1hPTvG3qJ9CMqtRgZ830SWQpDY0Tbz
eM5t2oegZNjOJsnuDu1DYR3h6Tn/LBmhyI8Tg/xc7tI+FC4WhTYjy8lyKPoF
Xh429K/3aB8G11a7G41BZIShrEOaaPo5WQrD9sDdkeIeshyGgvLWn5zyDPul
0Fxd7uqdZdgvxeObC8w+8DDsl2LEztTnOc8N/aW4/bqy74pPyUI4Wurqh+e1
Gz7+cGRtsXj66wyyFI4VqRfSLaMMn384mqU9984k0NdHEDHlmNvnn/rQ11Ml
InNc5xrJjL7eENEknnZ/c4i+P6KIrdcfZdzoou+nJEJ7dubXA53p+60VseiU
c3vXdvr5kEX0bmmtceqinyedCNP1WZ+d2kA/b0IE1ruVXL356QXqRyBWWuJ4
Kewc9d96wT0ft/308y5GwNhCcOkKPkX9CDhZtS7q9fwE9SPwuuvWWBej49SP
wObQdb1nV5ZSPwKhj748k29VTH01HJw/0pn50f9vlRr9q/YPiPXcT3015pXN
H3NiAj1fiGos+7vc337dN9RXY/O7b0ZNaqfnH60aLZ1WRo5x9Pwkq+Hx068j
t2R+QX01aoe8s+Zyfhb1I1GxbMa2050S9SPx58Q+nwkfpVM/Et280vo2fplG
/UiYDGtrOHQ1hfqRGPj0x2VPsmKpH4kBPxc3HLqmpn4kOtKvBvxQFUr9SHyY
Ffr+rYd+1I/CxbX7iv3zPKkfBbshhXfOjVlI/Sh0i3/erbnFkfpR6Iy1s1gg
zqJ+FLbl1ZQF2NHrl/bt4//ywn34F5OoH4XZ7xq9UgeNo34UTnZtvpFnbU79
aLhkv04IcxhK/Wg8DSxpyc6m11tEY3HwOMeZzr2oH42moK2enzS9Q/1orKv+
+6HJpa/1r+faaGT0SbnYJ/+Z3nI0zHqNWDX1RqveumiMWx150qH1Lr3+x2BL
cfBRi4X0/kIVg6raxzen+tfpjRisHhf2fV3LJb3FGByvWzxg9GJ6fyPFvH1/
GvmdyWF6v6h9u/9kyyvnYnp/KMfgbEreneM76P2bLgZ9/Qa0TVtJ77cEDao2
jfUx37Cd+hpkvBfb9C/jzdTX4OddHY8Lk5dTX4PCBSPOTXfNoL4GZz44HNxH
Q4/XalC8ovvRM1M2UV8De+/qDxqb1lFfgxfXNjg0H1tK/VhM252ucXm1ivqx
KMruNMqRs6gfi4tF089XHvmC+rEYaGt/QK6inhSLhK/P7d1z5Uvqx+Kb4Zfi
r5mR5VhcODbTJaY+m/qxmPSs5VVPz43Uj8NV8cpDj97UU8XhdGiRS8k+ifpx
WPHPix6jJvyN+nH4MHjADS/PVOrHQZPc37eyM476cVjvOTvCaKGa+nFIezex
h2V2MPXjIDVtWv/8XT/qx8MrWeiR/dyb+vF4kZVU7drdh/rxqP/YerTjPbIY
j4hgs5b8Cl/qx+Op965ba3vRv6eNh8ujxLqOArrL8ShPfbzVZhftdfFYYjNU
ruz0on4CdKevbnWrX0L9BPT8aPLzqgB36ifAsWGt5ZbTztRPwGvzk8PcIh2p
nwBc7lj9qGEW9RNwuXlPVWZPe+onoHLV0fK10TbUT8Dt6xfqnWrGUT8R9WN2
tokvVNRPfPsOZlTwIt0Q6ifizq/3bczl/tRPxA7jHK9lZ42pn4jfbVLfD3Q3
on4iNv/wzXTzly9m6/uJePnQ9f2Yfs/01iWieaOp651P2vUWkjDJKrq66PoD
vVVJaLoseldn3tYbSRi61uPuHvXveotJKEsesv/z8Dq9pSQE+66v2Hbyqt7a
JGRfKZ7ybOcv1E9CvyP15389eIb6Sah0T96mPnyC+snYmeFmbbuvjPrJsD71
To2VRyH1k9FtwcpPe2fso34yDoxIO1/h/i31k1Eu9t+nmplL/WQ88g/4ev+J
jdRPxofncxPbH0vUT4Z66hbd3o4V1E+BxV8+SY1uTqB+Cl4Mntg9d1kE9VNQ
kpli+qfGj/op6Gr5Sup9fiHtO+h59H+/oGwTxv0ZD2L8HuMRjFWMRzEew3gs
4/GMLRhbMrZmbMPYlrEd48mMpzCeytie8TTGDoynM57BeCbjWYxnM57DeC5j
KFtiLMxTtsRYcFS2xFiYr2yJsbBA2RJjwUnZEmPBWdkSY8FF2RJjwVXZEmNh
obIlxoKbsiXGgruyJcbCImVLjAUPZUuMhcXKlhgLS5QtMRY8lQ3GEmOZseCl
bDCWGMuMBW9lg7HEWGYs+CgbjCXGMmPBV9lgLDGWGQt+ygZjibHMWPBXNhhL
jGXGQoCywVhiLDMWApUNxhJjmbEQpGwwlhjLjIVgZYOxxFhmLIQoG4wlxjJj
IVTZYCwxlhkLYcoGY4mxzFhYqmwwlhjLjIVwZYOxxFhmLIjKVjEGY5GxxFjL
WGasYyxEKFvFGIxFxhJjLWOZsY6xoFa2ijEYi4wlxlrGMmMdYyFS2SrGYCwy
lhhrGcuMdYyFKGWrGIOxyFhirGUsM9YxFqKVrWIMxiJjibGWscxYx1iIUbaK
MRiLjCXGWsYyYx1jQaNsFWMwFhlLjLWMZcY6xkKsslWMwVhkLDHWMpYZ6xgL
ccpWMQZjkbHEWMtYZqxjLMQrW8UYjEXGEmMtY5mxjrGQoGwVYzAWGUuMtYxl
xjrGQqKyVYzBWGQsMdYylhnrGAtJylYxBmORscRYy1hmrGMsJCtbxRiMRcYS
Yy1jmbGOsZCibBVjMBb/7/8ALqGHeA==
                    "], {
                    {GrayLevel[0.5], EdgeForm[None], 
                    GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ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                    
                    "]]]}, {{}, {}, {
                    {Hue[0.9407694159419471, 1, 1], LineBox[{1, 2}]}, 
                    {Hue[0.8652990571156856, 1, 1], LineBox[{2, 3}]}, 
                    {Hue[0.7534371156171316, 1, 1], LineBox[{3, 4}]}, 
                    {Hue[0.6481847977203763, 1, 1], LineBox[{4, 5}]}, 
                    {Hue[0.5354894789766873, 1, 1], LineBox[{5, 6}]}, 
                    {Hue[0.46816222537833896`, 1, 1], LineBox[{6, 7}]}, 
                    {Hue[0.35585854458977223`, 1, 1], LineBox[{7, 8}]}, 
                    {Hue[0.2817377719638701, 1, 1], LineBox[{8, 9}]}, 
                    {Hue[0.14460000879947793`, 1, 1], LineBox[{9, 10}]}, 
                    {Hue[0.01469903437668414, 1, 1], LineBox[{10, 11}]}, 
                    {Hue[0.8988936709144447, 1, 1], LineBox[{11, 12}]}, 
                    {Hue[0.810522269771311, 1, 1], LineBox[{12, 13}]}, 
                    {Hue[0.6480896528173896, 1, 1], LineBox[{13, 14}]}, 
                    {Hue[0.5591795772773027, 1, 1], LineBox[{14, 15}]}, 
                    {Hue[0.4540232922687597, 1, 1], LineBox[{15, 16}]}, 
                    {Hue[0.40055182835091585`, 1, 1], LineBox[{16, 17}]}, 
                    {Hue[0.27385460721166377`, 1, 1], LineBox[{17, 18}]}, 
                    {Hue[0.14121327890295116`, 1, 1], LineBox[{18, 19}]}, 
                    {Hue[0.11383263176537513`, 1, 1], LineBox[{19, 20}]}, 
                    {Hue[0.9936820338959563, 1, 1], LineBox[{20, 21}]}, 
                    {Hue[0.9282939753793533, 1, 1], LineBox[{21, 22}]}, 
                    {Hue[0.812205984476198, 1, 1], LineBox[{22, 23}]}, 
                    {Hue[0.6632661263018338, 1, 1], LineBox[{23, 24}]}, 
                    {Hue[0.6142780797353196, 1, 1], LineBox[{24, 25}]}, 
                    {Hue[0.5144926704498913, 1, 1], LineBox[{25, 26}]}, 
                    {Hue[0.5040634907503884, 1, 1], LineBox[{26, 27}]}, 
                    {Hue[0.43416845221881806`, 1, 1], LineBox[{27, 28}]}, 
                    {Hue[0.26450211931177586`, 1, 1], LineBox[{28, 29}]}, 
                    {Hue[0.11758076126626354`, 1, 1], LineBox[{29, 30}]}, 
                    {Hue[0.9354851250970341, 1, 1], LineBox[{30, 31}]}, 
                    {Hue[0.7503898389339675, 1, 1], LineBox[{31, 32}]}, 
                    {Hue[0.9769323213302773, 1, 1], LineBox[{32, 33}]}, 
                    {Hue[0.932109019870338, 1, 1], LineBox[{33, 34}]}, 
                    {Hue[0.6669326654057793, 1, 1], LineBox[{34, 35}]}, 
                    {Hue[0.43016993026155, 1, 1], LineBox[{35, 36}]}, 
                    {Hue[0.39716477337106854`, 1, 1], LineBox[{36, 37}]}, 
                    {Hue[0.36493756558008444`, 1, 1], LineBox[{37, 38}]}, 
                    {Hue[0.4080060559607373, 1, 1], LineBox[{38, 39}]}, 
                    {Hue[0.4309672884697319, 1, 1], LineBox[{39, 40}]}, 
                    {Hue[0.5565706017286116, 1, 1], LineBox[{40, 41}]}, 
                    {Hue[0.6190552894798738, 1, 1], LineBox[{41, 42}]}, 
                    {Hue[0.6934773859825238, 1, 1], LineBox[{42, 43}]}, 
                    {Hue[0.7431865445042352, 1, 1], LineBox[{43, 44}]}, 
                    {Hue[0.7979053944788896, 1, 1], LineBox[{44, 45}]}, 
                    {Hue[0.8639723377143136, 1, 1], LineBox[{45, 46}]}, 
                    {Hue[0.931354074203931, 1, 1], LineBox[{46, 47}]}, 
                    {Hue[0.0019813757552162815`, 1, 1], LineBox[{47, 48}]}, 
                    {Hue[0.0721006651445593, 1, 1], LineBox[{48, 49}]}, 
                    {Hue[0.13836744610171361`, 1, 1], LineBox[{49, 50}]}, 
                    {Hue[0.20386232588584846`, 1, 1], LineBox[{50, 51}]}, 
                    {Hue[0.27293800925391504`, 1, 1], LineBox[{51, 52}]}, 
                    {Hue[0.3407829136148149, 1, 1], LineBox[{52, 53}]}, 
                    {Hue[0.41103261850344186`, 1, 1], LineBox[{53, 54}]}, 
                    {Hue[0.48015625333872314`, 1, 1], LineBox[{54, 55}]}, 
                    {Hue[0.5500525672930505, 1, 1], LineBox[{55, 56}]}, 
                    {Hue[0.620445851492982, 1, 1], LineBox[{56, 57}]}, 
                    {Hue[0.6917554854759792, 1, 1], LineBox[{57, 58}]}, 
                    {Hue[0.7634716246789225, 1, 1], LineBox[{58, 59}]}, 
                    {Hue[0.8356895882785431, 1, 1], LineBox[{59, 60}]}, 
                    {Hue[0.9082214829959486, 1, 1], LineBox[{60, 61}]}, 
                    {Hue[0.9815323348687448, 1, 1], LineBox[{61, 62}]}, 
                    {Hue[0.055317258485876146`, 1, 1], LineBox[{62, 63}]}, 
                    {Hue[0.12961575956451213`, 1, 1], LineBox[{63, 64}]}, 
                    {Hue[0.20433090079601726`, 1, 1], LineBox[{64, 65}]}, 
                    {Hue[0.2795540383337438, 1, 1], LineBox[{65, 66}]}, 
                    {Hue[0.35524739473256994`, 1, 1], LineBox[{66, 67}]}, 
                    {Hue[0.4315296393312207, 1, 1], LineBox[{67, 68}]}, 
                    {Hue[0.5082753399542235, 1, 1], LineBox[{68, 69}]}, 
                    {Hue[0.5855122705463676, 1, 1], LineBox[{69, 70}]}, 
                    {Hue[0.6632910732135462, 1, 1], LineBox[{70, 71}]}, 
                    {Hue[0.7415493785590683, 1, 1], LineBox[{71, 72}]}, 
                    {Hue[0.8203168163406356, 1, 1], LineBox[{72, 73}]}, 
                    {Hue[0.899594466264572, 1, 1], LineBox[{73, 74}]}, 
                    {Hue[0.9793808535751918, 1, 1], LineBox[{74, 75}]}, 
                    {Hue[0.05966611579066608, 1, 1], LineBox[{75, 76}]}, 
                    {Hue[0.14046103040636695`, 1, 1], LineBox[{76, 77}]}, 
                    {Hue[0.22176099485644657`, 1, 1], LineBox[{77, 78}]}, 
                    {Hue[0.30355640042518384`, 1, 1], LineBox[{78, 79}]}, 
                    {Hue[0.3858558561080394, 1, 1], LineBox[{79, 80}]}, 
                    {Hue[0.46865920579523773`, 1, 1], LineBox[{80, 81}]}, 
                    {Hue[0.5519572630975732, 1, 1], LineBox[{81, 82}]}, 
                    {Hue[0.6357616925070961, 1, 1], LineBox[{82, 83}]}, 
                    {Hue[0.7200614226226909, 1, 1], LineBox[{83, 84}]}, 
                    {Hue[0.8048540983332827, 1, 1], LineBox[{84, 85}]}, 
                    {Hue[0.8901452092823243, 1, 1], LineBox[{85, 86}]}, 
                    {Hue[0.9759256850504838, 1, 1], LineBox[{86, 87}]}, 
                    {Hue[0.06219940245881586, 1, 1], LineBox[{87, 88}]}, 
                    {Hue[0.14895952789651382`, 1, 1], LineBox[{88, 89}]}, 
                    {Hue[0.23620276109148372`, 1, 1], LineBox[{89, 90}]}, 
                    {Hue[0.32392749417342787`, 1, 1], LineBox[{90, 91}]}, 
                    {Hue[0.41212937430084273`, 1, 1], LineBox[{91, 92}]}, 
                    {Hue[0.5008039055547484, 1, 1], LineBox[{92, 93}]}, 
                    {Hue[0.5899448498811499, 1, 1], LineBox[{93, 94}]}, 
                    {Hue[0.6795444249013187, 1, 1], LineBox[{94, 95}]}, 
                    {Hue[0.7695963834453866, 1, 1], LineBox[{95, 96}]}, 
                    {Hue[0.8600924523736628, 1, 1], LineBox[{96, 97}]}, 
                    {Hue[0.9510219118737493, 1, 1], LineBox[{97, 98}]}, 
                    {Hue[0.04237273932485363, 1, 1], LineBox[{98, 99}]}, 
                    {Hue[0.1341303083120591, 1, 1], LineBox[{99, 100}]}, 
                    {Hue[0.2262799008469206, 1, 1], LineBox[{100, 101}]}, 
                    {Hue[0.318803207291762, 1, 1], LineBox[{101, 102}]}, 
                    {Hue[0.41167848966155357`, 1, 1], LineBox[{102, 103}]}, 
                    {Hue[0.5048808051332369, 1, 1], LineBox[{103, 104}]}, 
                    {Hue[0.5983810634330753, 1, 1], LineBox[{104, 105}]}, 
                    {Hue[0.6921470473254048, 1, 1], LineBox[{105, 106}]}, 
                    {Hue[0.7861407557274118, 1, 1], LineBox[{106, 107}]}, 
                    {Hue[0.8803183444062389, 1, 1], LineBox[{107, 108}]}, 
                    {Hue[0.9746296130822348, 1, 1], LineBox[{108, 109}]}, 
                    {Hue[0.06901767356908702, 1, 1], LineBox[{109, 110}]}, 
                    {Hue[0.1634186711999563, 1, 1], LineBox[{110, 111}]}, 
                    {Hue[0.257759298319671, 1, 1], LineBox[{111, 112}]}, 
                    {Hue[0.35195625424943705`, 1, 1], LineBox[{112, 113}]}, 
                    {Hue[0.4459170200063353, 1, 1], LineBox[{113, 114}]}, 
                    {Hue[0.5395389647253133, 1, 1], LineBox[{114, 115}]}, 
                    {Hue[0.6327115014530438, 1, 1], LineBox[{115, 116}]}, 
                    {Hue[0.7253120626933802, 1, 1], LineBox[{116, 117}]}, 
                    {Hue[0.8172080503626297, 1, 1], LineBox[{117, 118}]}, 
                    {Hue[0.9082571817465559, 1, 1], LineBox[{118, 119}]}, 
                    {Hue[0.9983125469259199, 1, 1], LineBox[{119, 120}]}, 
                    {Hue[0.0872270893257851, 1, 1], LineBox[{120, 121}]}, 
                    {Hue[0.17485685640692983`, 1, 1], LineBox[{121, 122}]}, 
                    {Hue[0.26106162246570447`, 1, 1], LineBox[{122, 123}]}, 
                    {Hue[0.3457071509121968, 1, 1], LineBox[{123, 124}]}, 
                    {Hue[0.4286721641297487, 1, 1], LineBox[{124, 125}]}, 
                    {Hue[0.5098651650273955, 1, 1], LineBox[{125, 126}]}, 
                    {Hue[0.5892417687680118, 1, 1], LineBox[{126, 127}]}, 
                    {Hue[0.6668211853508332, 1, 1], LineBox[{127, 128}]}, 
                    {Hue[0.7426906112473048, 1, 1], LineBox[{128, 129}]}, 
                    {Hue[0.8170015469243537, 1, 1], LineBox[{129, 130}]}, 
                    {Hue[0.8899533966442782, 1, 1], LineBox[{130, 131}]}, 
                    {Hue[0.9617783185528035, 1, 1], LineBox[{131, 132}]}, 
                    {Hue[0.03272320056978751, 1, 1], LineBox[{132, 133}]}, 
                    {Hue[0.10303459335594814`, 1, 1], LineBox[{133, 134}]}, 
                    {Hue[0.17294878870530267`, 1, 1], LineBox[{134, 135}]}, 
                    {Hue[0.2426863090271892, 1, 1], LineBox[{135, 136}]}, 
                    {Hue[0.31244541224626743`, 1, 1], LineBox[{136, 137}]}, 
                    {Hue[0.3824019140435007, 1, 1], LineBox[{137, 138}]}, 
                    {Hue[0.4527103697766939, 1, 1], LineBox[{138, 139}]}, 
                    {Hue[0.5235041204448956, 1, 1], LineBox[{139, 140}]}, 
                    {Hue[0.5948940264243402, 1, 1], LineBox[{140, 141}]}, 
                    {Hue[0.6669759737360023, 1, 1], LineBox[{141, 142}]}, 
                    {Hue[0.7398286321251204, 1, 1], LineBox[{142, 143}]}, 
                    {Hue[0.8135130404584696, 1, 1], LineBox[{143, 144}]}, 
                    {Hue[0.8880832031234254, 1, 1], LineBox[{144, 145}]}, 
                    {Hue[0.9635811958810462, 1, 1], LineBox[{145, 146}]}, 
                    {Hue[0.04003531910755127, 1, 1], LineBox[{146, 147}]}, 
                    {Hue[0.11746914394991234`, 1, 1], LineBox[{147, 148}]}, 
                    {Hue[0.195904600981633, 1, 1], LineBox[{148, 149}]}, 
                    {Hue[0.27535047794179796`, 1, 1], LineBox[{149, 150}]}, 
                    {Hue[0.3558085223791187, 1, 1], LineBox[{150, 151}]}, 
                    {Hue[0.4372840484618632, 1, 1], LineBox[{151, 152}]}, 
                    {Hue[0.5197882895126544, 1, 1], LineBox[{152, 153}]}, 
                    {Hue[0.6033219242113339, 1, 1], LineBox[{153, 154}]}, 
                    {Hue[0.6878536441947171, 1, 1], LineBox[{154, 155}]}, 
                    {Hue[0.7733697728793487, 1, 1], LineBox[{155, 156}]}, 
                    {Hue[0.8598942801945528, 1, 1], LineBox[{156, 157}]}, 
                    {Hue[0.9474247816814833, 1, 1], LineBox[{157, 158}]}, 
                    {Hue[0.035907408921143254`, 1, 1], LineBox[{158, 159}]}, 
                    {Hue[0.1253083000319576, 1, 1], LineBox[{159, 160}]}, 
                    {Hue[0.2156850527530413, 1, 1], LineBox[{160, 161}]}, 
                    {Hue[0.30709160770072824`, 1, 1], LineBox[{161, 162}]}, 
                    {Hue[0.3994547063543386, 1, 1], LineBox[{162, 163}]}, 
                    {Hue[0.4926921458642045, 1, 1], LineBox[{163, 164}]}, 
                    {Hue[0.5868410658775327, 1, 1], LineBox[{164, 165}]}, 
                    {Hue[0.6819856197390084, 1, 1], LineBox[{165, 166}]}, 
                    {Hue[0.7780905715872407, 1, 1], LineBox[{166, 167}]}, 
                    {Hue[0.8747788347944276, 1, 1], LineBox[{167, 168}]}, 
                    {Hue[0.9716418911349147, 1, 1], LineBox[{168, 169}]}, 
                    {Hue[0.06920173882244322, 1, 1], LineBox[{169, 170}]}, 
                    {Hue[0.16854962278205396`, 1, 1], LineBox[{170, 171}]}, 
                    {Hue[0.2695185878541521, 1, 1], LineBox[{171, 172}]}, 
                    {Hue[0.3707722030072139, 1, 1], LineBox[{172, 173}]}, 
                    {Hue[0.47160607498768725`, 1, 1], LineBox[{173, 174}]}, 
                    {Hue[0.5721384519808977, 1, 1], LineBox[{174, 175}]}, 
                    {Hue[0.6738855996878312, 1, 1], LineBox[{175, 176}]}, 
                    {Hue[0.78145634117785, 1, 1], LineBox[{176, 177}]}, 
                    {Hue[0.8966100687768956, 1, 1], LineBox[{177, 178}]}, 
                    {Hue[0.00987144155480758, 1, 1], LineBox[{178, 179}]}, 
                    {Hue[0.10665989586565482`, 1, 1], LineBox[{179, 180}]}, 
                    {Hue[0.2017755608777928, 1, 1], LineBox[{180, 181}]}, 
                    {Hue[0.3243851462421015, 1, 1], LineBox[{181, 182}]}, 
                    {Hue[0.46035977061619515`, 1, 1], LineBox[{182, 183}]}, 
                    {Hue[0.5698711687553977, 1, 1], LineBox[{183, 184}]}, 
                    {Hue[0.6446141696538195, 1, 1], LineBox[{184, 185}]}, 
                    {Hue[0.7649959746908312, 1, 1], LineBox[{185, 186}]}, 
                    {Hue[0.8499991304409061, 1, 1], LineBox[{186, 187}]}, 
                    {Hue[0.7527311439527943, 1, 1], LineBox[{187, 188}]}, 
                    {Hue[0.9407324783675022, 1, 1], LineBox[{188, 189}]}, 
                    {Hue[0.16279964711927528`, 1, 1], LineBox[{189, 190}]}, 
                    {Hue[0.40538122810309657`, 1, 1], LineBox[{190, 191}]}, 
                    {Hue[0.7662466372646557, 1, 1], LineBox[{191, 192}]}, 
                    {Hue[0.325746588547473, 1, 1], LineBox[{192, 193}]}, 
                    {Hue[0.6014346559379302, 1, 1], LineBox[{193, 194}]}, 
                    {Hue[0.8702034520468045, 1, 1], LineBox[{194, 195}]}, 
                    {Hue[0.14000852383014625`, 1, 1], LineBox[{195, 196}]}, 
                    {Hue[0.4094397952932074, 1, 1], LineBox[{196, 197}]}, 
                    {Hue[0.6768951372489489, 1, 1], LineBox[{197, 198}]}, 
                    {Hue[0.9421369682877887, 1, 1], LineBox[{198, 199}]}, 
                    {Hue[0.2058797630336536, 1, 1], LineBox[{199, 200}]}, 
                    {Hue[0.4690627681848623, 1, 1], LineBox[{200, 201}]}, 
                    {Hue[0.7324642750692172, 1, 1], LineBox[{201, 202}]}, 
                    {Hue[0.996675128822844, 1, 1], LineBox[{202, 203}]}, 
                    {Hue[0.2623619251828773, 1, 1], LineBox[{203, 204}]}, 
                    {Hue[0.5308036556013873, 1, 1], LineBox[{204, 205}]}, 
                    {Hue[0.8045450818299433, 1, 1], LineBox[{205, 206}]}, 
                    {Hue[0.08721915474396433, 1, 1], LineBox[{206, 207}]}, 
                    {Hue[0.38020271952928, 1, 1], LineBox[{207, 208}]}, 
                    {Hue[0.6773830849870384, 1, 1], LineBox[{208, 209}]}, 
                    {Hue[0.969304645903326, 1, 1], LineBox[{209, 210}]}, 
                    {Hue[0.25270882801694694`, 1, 1], LineBox[{210, 211}]}, 
                    {Hue[0.5294894977396489, 1, 1], LineBox[{211, 212}]}, 
                    {Hue[0.8021274837723045, 1, 1], LineBox[{212, 213}]}, 
                    {Hue[0.07220163138290007, 1, 1], LineBox[{213, 214}]}, 
                    {Hue[0.3404993776492227, 1, 1], LineBox[{214, 215}]}, 
                    {Hue[0.607341117597648, 1, 1], LineBox[{215, 216}]}, 
                    {Hue[0.8728024358639596, 1, 1], LineBox[{216, 217}]}, 
                    {Hue[0.13683263563569506`, 1, 1], LineBox[{217, 218}]}, 
                    {Hue[0.3993081326856766, 1, 1], LineBox[{218, 219}]}, 
                    {Hue[0.6600489781726215, 1, 1], LineBox[{219, 220}]}, 
                    {Hue[0.9188134565589243, 1, 1], LineBox[{220, 221}]}, 
                    {Hue[0.17528128961485592`, 1, 1], LineBox[{221, 222}]}, 
                    {Hue[0.42902982578944954`, 1, 1], LineBox[{222, 223}]}, 
                    {Hue[0.6795145524088642, 1, 1], LineBox[{223, 224}]}, 
                    {Hue[0.9260767691882732, 1, 1], LineBox[{224, 225}]}, 
                    {Hue[0.16803549279117072`, 1, 1], LineBox[{225, 226}]}, 
                    {Hue[0.40495850769743, 1, 1], LineBox[{226, 227}]}, 
                    {Hue[0.6371737992290389, 1, 1], LineBox[{227, 228}]}, 
                    {Hue[0.8662893283077272, 1, 1], LineBox[{228, 229}]}, 
                    {Hue[0.09505552883252164, 1, 1], LineBox[{229, 230}]}, 
                    {Hue[0.3262617090805855, 1, 1], LineBox[{230, 231}]}, 
                    {Hue[0.5615905657572104, 1, 1], LineBox[{231, 232}]}, 
                    {Hue[0.8014193438577213, 1, 1], LineBox[{232, 233}]}, 
                    {Hue[0.04532016587981987, 1, 1], LineBox[{233, 234}]}, 
                    {Hue[0.2925849492448882, 1, 1], LineBox[{234, 235}]}, 
                    {Hue[0.5425124691739087, 1, 1], LineBox[{235, 236}]}, 
                    {Hue[0.7945105484655892, 1, 1], LineBox[{236, 237}]}, 
                    {Hue[0.048109368347345416`, 1, 1], LineBox[{237, 238}]}, 
                    {Hue[0.3029441434058117, 1, 1], LineBox[{238, 239}]}, 
                    {Hue[0.5587316333455382, 1, 1], LineBox[{239, 240}]}, 
                    {Hue[0.8152486151476237, 1, 1], LineBox[{240, 241}]}, 
                    {Hue[0.07231525149390143, 1, 1], LineBox[{241, 242}]}, 
                    {Hue[0.3297838895830974, 1, 1], LineBox[{242, 243}]}, 
                    {Hue[0.587531735292107, 1, 1], LineBox[{243, 244}]}, 
                    {Hue[0.8454588286735683, 1, 1], LineBox[{244, 245}]}, 
                    {Hue[0.10348856732185839`, 1, 1], LineBox[{245, 246}]}, 
                    {Hue[0.36157113441264427`, 1, 1], LineBox[{246, 247}]}, 
                    {Hue[0.6196890157024028, 1, 1], LineBox[{247, 248}]}, 
                    {Hue[0.877865442493381, 1, 1], LineBox[{248, 249}]}, 
                    {Hue[0.136176524325591, 1, 1], LineBox[{249, 250}]}, 
                    {Hue[0.3947708920661652, 1, 1], LineBox[{250, 251}]}}}},
                    VertexColors->{
                    Hue[0.9407694159419471, 1, 1], 
                    Hue[0.8652990571156856, 1, 1], 
                    Hue[0.7534371156171316, 1, 1], 
                    Hue[0.6481847977203763, 1, 1], 
                    Hue[0.5354894789766873, 1, 1], 
                    Hue[0.46816222537833896`, 1, 1], 
                    Hue[0.35585854458977223`, 1, 1], 
                    Hue[0.2817377719638701, 1, 1], 
                    Hue[0.14460000879947793`, 1, 1], 
                    Hue[0.01469903437668414, 1, 1], 
                    Hue[0.8988936709144447, 1, 1], 
                    Hue[0.810522269771311, 1, 1], 
                    Hue[0.6480896528173896, 1, 1], 
                    Hue[0.5591795772773027, 1, 1], 
                    Hue[0.4540232922687597, 1, 1], 
                    Hue[0.40055182835091585`, 1, 1], 
                    Hue[0.27385460721166377`, 1, 1], 
                    Hue[0.14121327890295116`, 1, 1], 
                    Hue[0.11383263176537513`, 1, 1], 
                    Hue[0.9936820338959563, 1, 1], 
                    Hue[0.9282939753793533, 1, 1], 
                    Hue[0.812205984476198, 1, 1], 
                    Hue[0.6632661263018338, 1, 1], 
                    Hue[0.6142780797353196, 1, 1], 
                    Hue[0.5144926704498913, 1, 1], 
                    Hue[0.5040634907503884, 1, 1], 
                    Hue[0.43416845221881806`, 1, 1], 
                    Hue[0.26450211931177586`, 1, 1], 
                    Hue[0.11758076126626354`, 1, 1], 
                    Hue[0.9354851250970341, 1, 1], 
                    Hue[0.7503898389339675, 1, 1], 
                    Hue[0.9769323213302773, 1, 1], 
                    Hue[0.932109019870338, 1, 1], 
                    Hue[0.6669326654057793, 1, 1], 
                    Hue[0.43016993026155, 1, 1], 
                    Hue[0.39716477337106854`, 1, 1], 
                    Hue[0.36493756558008444`, 1, 1], 
                    Hue[0.4080060559607373, 1, 1], 
                    Hue[0.4309672884697319, 1, 1], 
                    Hue[0.5565706017286116, 1, 1], 
                    Hue[0.6190552894798738, 1, 1], 
                    Hue[0.6934773859825238, 1, 1], 
                    Hue[0.7431865445042352, 1, 1], 
                    Hue[0.7979053944788896, 1, 1], 
                    Hue[0.8639723377143136, 1, 1], 
                    Hue[0.931354074203931, 1, 1], 
                    Hue[0.0019813757552162815`, 1, 1], 
                    Hue[0.0721006651445593, 1, 1], 
                    Hue[0.13836744610171361`, 1, 1], 
                    Hue[0.20386232588584846`, 1, 1], 
                    Hue[0.27293800925391504`, 1, 1], 
                    Hue[0.3407829136148149, 1, 1], 
                    Hue[0.41103261850344186`, 1, 1], 
                    Hue[0.48015625333872314`, 1, 1], 
                    Hue[0.5500525672930505, 1, 1], 
                    Hue[0.620445851492982, 1, 1], 
                    Hue[0.6917554854759792, 1, 1], 
                    Hue[0.7634716246789225, 1, 1], 
                    Hue[0.8356895882785431, 1, 1], 
                    Hue[0.9082214829959486, 1, 1], 
                    Hue[0.9815323348687448, 1, 1], 
                    Hue[0.055317258485876146`, 1, 1], 
                    Hue[0.12961575956451213`, 1, 1], 
                    Hue[0.20433090079601726`, 1, 1], 
                    Hue[0.2795540383337438, 1, 1], 
                    Hue[0.35524739473256994`, 1, 1], 
                    Hue[0.4315296393312207, 1, 1], 
                    Hue[0.5082753399542235, 1, 1], 
                    Hue[0.5855122705463676, 1, 1], 
                    Hue[0.6632910732135462, 1, 1], 
                    Hue[0.7415493785590683, 1, 1], 
                    Hue[0.8203168163406356, 1, 1], 
                    Hue[0.899594466264572, 1, 1], 
                    Hue[0.9793808535751918, 1, 1], 
                    Hue[0.05966611579066608, 1, 1], 
                    Hue[0.14046103040636695`, 1, 1], 
                    Hue[0.22176099485644657`, 1, 1], 
                    Hue[0.30355640042518384`, 1, 1], 
                    Hue[0.3858558561080394, 1, 1], 
                    Hue[0.46865920579523773`, 1, 1], 
                    Hue[0.5519572630975732, 1, 1], 
                    Hue[0.6357616925070961, 1, 1], 
                    Hue[0.7200614226226909, 1, 1], 
                    Hue[0.8048540983332827, 1, 1], 
                    Hue[0.8901452092823243, 1, 1], 
                    Hue[0.9759256850504838, 1, 1], 
                    Hue[0.06219940245881586, 1, 1], 
                    Hue[0.14895952789651382`, 1, 1], 
                    Hue[0.23620276109148372`, 1, 1], 
                    Hue[0.32392749417342787`, 1, 1], 
                    Hue[0.41212937430084273`, 1, 1], 
                    Hue[0.5008039055547484, 1, 1], 
                    Hue[0.5899448498811499, 1, 1], 
                    Hue[0.6795444249013187, 1, 1], 
                    Hue[0.7695963834453866, 1, 1], 
                    Hue[0.8600924523736628, 1, 1], 
                    Hue[0.9510219118737493, 1, 1], 
                    Hue[0.04237273932485363, 1, 1], 
                    Hue[0.1341303083120591, 1, 1], 
                    Hue[0.2262799008469206, 1, 1], 
                    Hue[0.318803207291762, 1, 1], 
                    Hue[0.41167848966155357`, 1, 1], 
                    Hue[0.5048808051332369, 1, 1], 
                    Hue[0.5983810634330753, 1, 1], 
                    Hue[0.6921470473254048, 1, 1], 
                    Hue[0.7861407557274118, 1, 1], 
                    Hue[0.8803183444062389, 1, 1], 
                    Hue[0.9746296130822348, 1, 1], 
                    Hue[0.06901767356908702, 1, 1], 
                    Hue[0.1634186711999563, 1, 1], 
                    Hue[0.257759298319671, 1, 1], 
                    Hue[0.35195625424943705`, 1, 1], 
                    Hue[0.4459170200063353, 1, 1], 
                    Hue[0.5395389647253133, 1, 1], 
                    Hue[0.6327115014530438, 1, 1], 
                    Hue[0.7253120626933802, 1, 1], 
                    Hue[0.8172080503626297, 1, 1], 
                    Hue[0.9082571817465559, 1, 1], 
                    Hue[0.9983125469259199, 1, 1], 
                    Hue[0.0872270893257851, 1, 1], 
                    Hue[0.17485685640692983`, 1, 1], 
                    Hue[0.26106162246570447`, 1, 1], 
                    Hue[0.3457071509121968, 1, 1], 
                    Hue[0.4286721641297487, 1, 1], 
                    Hue[0.5098651650273955, 1, 1], 
                    Hue[0.5892417687680118, 1, 1], 
                    Hue[0.6668211853508332, 1, 1], 
                    Hue[0.7426906112473048, 1, 1], 
                    Hue[0.8170015469243537, 1, 1], 
                    Hue[0.8899533966442782, 1, 1], 
                    Hue[0.9617783185528035, 1, 1], 
                    Hue[0.03272320056978751, 1, 1], 
                    Hue[0.10303459335594814`, 1, 1], 
                    Hue[0.17294878870530267`, 1, 1], 
                    Hue[0.2426863090271892, 1, 1], 
                    Hue[0.31244541224626743`, 1, 1], 
                    Hue[0.3824019140435007, 1, 1], 
                    Hue[0.4527103697766939, 1, 1], 
                    Hue[0.5235041204448956, 1, 1], 
                    Hue[0.5948940264243402, 1, 1], 
                    Hue[0.6669759737360023, 1, 1], 
                    Hue[0.7398286321251204, 1, 1], 
                    Hue[0.8135130404584696, 1, 1], 
                    Hue[0.8880832031234254, 1, 1], 
                    Hue[0.9635811958810462, 1, 1], 
                    Hue[0.04003531910755127, 1, 1], 
                    Hue[0.11746914394991234`, 1, 1], 
                    Hue[0.195904600981633, 1, 1], 
                    Hue[0.27535047794179796`, 1, 1], 
                    Hue[0.3558085223791187, 1, 1], 
                    Hue[0.4372840484618632, 1, 1], 
                    Hue[0.5197882895126544, 1, 1], 
                    Hue[0.6033219242113339, 1, 1], 
                    Hue[0.6878536441947171, 1, 1], 
                    Hue[0.7733697728793487, 1, 1], 
                    Hue[0.8598942801945528, 1, 1], 
                    Hue[0.9474247816814833, 1, 1], 
                    Hue[0.035907408921143254`, 1, 1], 
                    Hue[0.1253083000319576, 1, 1], 
                    Hue[0.2156850527530413, 1, 1], 
                    Hue[0.30709160770072824`, 1, 1], 
                    Hue[0.3994547063543386, 1, 1], 
                    Hue[0.4926921458642045, 1, 1], 
                    Hue[0.5868410658775327, 1, 1], 
                    Hue[0.6819856197390084, 1, 1], 
                    Hue[0.7780905715872407, 1, 1], 
                    Hue[0.8747788347944276, 1, 1], 
                    Hue[0.9716418911349147, 1, 1], 
                    Hue[0.06920173882244322, 1, 1], 
                    Hue[0.16854962278205396`, 1, 1], 
                    Hue[0.2695185878541521, 1, 1], 
                    Hue[0.3707722030072139, 1, 1], 
                    Hue[0.47160607498768725`, 1, 1], 
                    Hue[0.5721384519808977, 1, 1], 
                    Hue[0.6738855996878312, 1, 1], 
                    Hue[0.78145634117785, 1, 1], 
                    Hue[0.8966100687768956, 1, 1], 
                    Hue[0.00987144155480758, 1, 1], 
                    Hue[0.10665989586565482`, 1, 1], 
                    Hue[0.2017755608777928, 1, 1], 
                    Hue[0.3243851462421015, 1, 1], 
                    Hue[0.46035977061619515`, 1, 1], 
                    Hue[0.5698711687553977, 1, 1], 
                    Hue[0.6446141696538195, 1, 1], 
                    Hue[0.7649959746908312, 1, 1], 
                    Hue[0.8499991304409061, 1, 1], 
                    Hue[0.7527311439527943, 1, 1], 
                    Hue[0.9407324783675022, 1, 1], 
                    Hue[0.16279964711927528`, 1, 1], 
                    Hue[0.40538122810309657`, 1, 1], 
                    Hue[0.7662466372646557, 1, 1], 
                    Hue[0.325746588547473, 1, 1], 
                    Hue[0.6014346559379302, 1, 1], 
                    Hue[0.8702034520468045, 1, 1], 
                    Hue[0.14000852383014625`, 1, 1], 
                    Hue[0.4094397952932074, 1, 1], 
                    Hue[0.6768951372489489, 1, 1], 
                    Hue[0.9421369682877887, 1, 1], 
                    Hue[0.2058797630336536, 1, 1], 
                    Hue[0.4690627681848623, 1, 1], 
                    Hue[0.7324642750692172, 1, 1], 
                    Hue[0.996675128822844, 1, 1], 
                    Hue[0.2623619251828773, 1, 1], 
                    Hue[0.5308036556013873, 1, 1], 
                    Hue[0.8045450818299433, 1, 1], 
                    Hue[0.08721915474396433, 1, 1], 
                    Hue[0.38020271952928, 1, 1], 
                    Hue[0.6773830849870384, 1, 1], 
                    Hue[0.969304645903326, 1, 1], 
                    Hue[0.25270882801694694`, 1, 1], 
                    Hue[0.5294894977396489, 1, 1], 
                    Hue[0.8021274837723045, 1, 1], 
                    Hue[0.07220163138290007, 1, 1], 
                    Hue[0.3404993776492227, 1, 1], 
                    Hue[0.607341117597648, 1, 1], 
                    Hue[0.8728024358639596, 1, 1], 
                    Hue[0.13683263563569506`, 1, 1], 
                    Hue[0.3993081326856766, 1, 1], 
                    Hue[0.6600489781726215, 1, 1], 
                    Hue[0.9188134565589243, 1, 1], 
                    Hue[0.17528128961485592`, 1, 1], 
                    Hue[0.42902982578944954`, 1, 1], 
                    Hue[0.6795145524088642, 1, 1], 
                    Hue[0.9260767691882732, 1, 1], 
                    Hue[0.16803549279117072`, 1, 1], 
                    Hue[0.40495850769743, 1, 1], 
                    Hue[0.6371737992290389, 1, 1], 
                    Hue[0.8662893283077272, 1, 1], 
                    Hue[0.09505552883252164, 1, 1], 
                    Hue[0.3262617090805855, 1, 1], 
                    Hue[0.5615905657572104, 1, 1], 
                    Hue[0.8014193438577213, 1, 1], 
                    Hue[0.04532016587981987, 1, 1], 
                    Hue[0.2925849492448882, 1, 1], 
                    Hue[0.5425124691739087, 1, 1], 
                    Hue[0.7945105484655892, 1, 1], 
                    Hue[0.048109368347345416`, 1, 1], 
                    Hue[0.3029441434058117, 1, 1], 
                    Hue[0.5587316333455382, 1, 1], 
                    Hue[0.8152486151476237, 1, 1], 
                    Hue[0.07231525149390143, 1, 1], 
                    Hue[0.3297838895830974, 1, 1], 
                    Hue[0.587531735292107, 1, 1], 
                    Hue[0.8454588286735683, 1, 1], 
                    Hue[0.10348856732185839`, 1, 1], 
                    Hue[0.36157113441264427`, 1, 1], 
                    Hue[0.6196890157024028, 1, 1], 
                    Hue[0.877865442493381, 1, 1], 
                    Hue[0.136176524325591, 1, 1], 
                    Hue[0.3947708920661652, 1, 1], 
                    Hue[0.6539029141821653, 1, 1], 
                    Hue[0.9407694159419471, 1, 1], 
                    Hue[0.8652990571156856, 1, 1], 
                    Hue[0.8652990571156856, 1, 1], 
                    Hue[0.7534371156171316, 1, 1], 
                    Hue[0.7534371156171316, 1, 1], 
                    Hue[0.6481847977203763, 1, 1], 
                    Hue[0.6481847977203763, 1, 1], 
                    Hue[0.5354894789766873, 1, 1], 
                    Hue[0.5354894789766873, 1, 1], 
                    Hue[0.46816222537833896`, 1, 1], 
                    Hue[0.46816222537833896`, 1, 1], 
                    Hue[0.35585854458977223`, 1, 1], 
                    Hue[0.35585854458977223`, 1, 1], 
                    Hue[0.2817377719638701, 1, 1], 
                    Hue[0.2817377719638701, 1, 1], 
                    Hue[0.14460000879947793`, 1, 1], 
                    Hue[0.14460000879947793`, 1, 1], 
                    Hue[0.01469903437668414, 1, 1], 
                    Hue[0.01469903437668414, 1, 1], 
                    Hue[0.8988936709144447, 1, 1], 
                    Hue[0.8988936709144447, 1, 1], 
                    Hue[0.810522269771311, 1, 1], 
                    Hue[0.810522269771311, 1, 1], 
                    Hue[0.6480896528173896, 1, 1], 
                    Hue[0.6480896528173896, 1, 1], 
                    Hue[0.5591795772773027, 1, 1], 
                    Hue[0.5591795772773027, 1, 1], 
                    Hue[0.4540232922687597, 1, 1], 
                    Hue[0.4540232922687597, 1, 1], 
                    Hue[0.40055182835091585`, 1, 1], 
                    Hue[0.40055182835091585`, 1, 1], 
                    Hue[0.27385460721166377`, 1, 1], 
                    Hue[0.27385460721166377`, 1, 1], 
                    Hue[0.14121327890295116`, 1, 1], 
                    Hue[0.14121327890295116`, 1, 1], 
                    Hue[0.11383263176537513`, 1, 1], 
                    Hue[0.11383263176537513`, 1, 1], 
                    Hue[0.9936820338959563, 1, 1], 
                    Hue[0.9936820338959563, 1, 1], 
                    Hue[0.9282939753793533, 1, 1], 
                    Hue[0.9282939753793533, 1, 1], 
                    Hue[0.812205984476198, 1, 1], 
                    Hue[0.812205984476198, 1, 1], 
                    Hue[0.6632661263018338, 1, 1], 
                    Hue[0.6632661263018338, 1, 1], 
                    Hue[0.6142780797353196, 1, 1], 
                    Hue[0.6142780797353196, 1, 1], 
                    Hue[0.5144926704498913, 1, 1], 
                    Hue[0.5144926704498913, 1, 1], 
                    Hue[0.5040634907503884, 1, 1], 
                    Hue[0.5040634907503884, 1, 1], 
                    Hue[0.43416845221881806`, 1, 1], 
                    Hue[0.43416845221881806`, 1, 1], 
                    Hue[0.26450211931177586`, 1, 1], 
                    Hue[0.26450211931177586`, 1, 1], 
                    Hue[0.11758076126626354`, 1, 1], 
                    Hue[0.11758076126626354`, 1, 1], 
                    Hue[0.9354851250970341, 1, 1], 
                    Hue[0.9354851250970341, 1, 1], 
                    Hue[0.7503898389339675, 1, 1], 
                    Hue[0.7503898389339675, 1, 1], 
                    Hue[0.9769323213302773, 1, 1], 
                    Hue[0.9769323213302773, 1, 1], 
                    Hue[0.932109019870338, 1, 1], 
                    Hue[0.932109019870338, 1, 1], 
                    Hue[0.6669326654057793, 1, 1], 
                    Hue[0.6669326654057793, 1, 1], 
                    Hue[0.43016993026155, 1, 1], 
                    Hue[0.43016993026155, 1, 1], 
                    Hue[0.39716477337106854`, 1, 1], 
                    Hue[0.39716477337106854`, 1, 1], 
                    Hue[0.36493756558008444`, 1, 1], 
                    Hue[0.36493756558008444`, 1, 1], 
                    Hue[0.4080060559607373, 1, 1], 
                    Hue[0.4080060559607373, 1, 1], 
                    Hue[0.4309672884697319, 1, 1], 
                    Hue[0.4309672884697319, 1, 1], 
                    Hue[0.5565706017286116, 1, 1], 
                    Hue[0.5565706017286116, 1, 1], 
                    Hue[0.6190552894798738, 1, 1], 
                    Hue[0.6190552894798738, 1, 1], 
                    Hue[0.6934773859825238, 1, 1], 
                    Hue[0.6934773859825238, 1, 1], 
                    Hue[0.7431865445042352, 1, 1], 
                    Hue[0.7431865445042352, 1, 1], 
                    Hue[0.7979053944788896, 1, 1], 
                    Hue[0.7979053944788896, 1, 1], 
                    Hue[0.8639723377143136, 1, 1], 
                    Hue[0.8639723377143136, 1, 1], 
                    Hue[0.931354074203931, 1, 1], 
                    Hue[0.931354074203931, 1, 1], 
                    Hue[0.0019813757552162815`, 1, 1], 
                    Hue[0.0019813757552162815`, 1, 1], 
                    Hue[0.0721006651445593, 1, 1], 
                    Hue[0.0721006651445593, 1, 1], 
                    Hue[0.13836744610171361`, 1, 1], 
                    Hue[0.13836744610171361`, 1, 1], 
                    Hue[0.20386232588584846`, 1, 1], 
                    Hue[0.20386232588584846`, 1, 1], 
                    Hue[0.27293800925391504`, 1, 1], 
                    Hue[0.27293800925391504`, 1, 1], 
                    Hue[0.3407829136148149, 1, 1], 
                    Hue[0.3407829136148149, 1, 1], 
                    Hue[0.41103261850344186`, 1, 1], 
                    Hue[0.41103261850344186`, 1, 1], 
                    Hue[0.48015625333872314`, 1, 1], 
                    Hue[0.48015625333872314`, 1, 1], 
                    Hue[0.5500525672930505, 1, 1], 
                    Hue[0.5500525672930505, 1, 1], 
                    Hue[0.620445851492982, 1, 1], 
                    Hue[0.620445851492982, 1, 1], 
                    Hue[0.6917554854759792, 1, 1], 
                    Hue[0.6917554854759792, 1, 1], 
                    Hue[0.7634716246789225, 1, 1], 
                    Hue[0.7634716246789225, 1, 1], 
                    Hue[0.8356895882785431, 1, 1], 
                    Hue[0.8356895882785431, 1, 1], 
                    Hue[0.9082214829959486, 1, 1], 
                    Hue[0.9082214829959486, 1, 1], 
                    Hue[0.9815323348687448, 1, 1], 
                    Hue[0.9815323348687448, 1, 1], 
                    Hue[0.055317258485876146`, 1, 1], 
                    Hue[0.055317258485876146`, 1, 1], 
                    Hue[0.12961575956451213`, 1, 1], 
                    Hue[0.12961575956451213`, 1, 1], 
                    Hue[0.20433090079601726`, 1, 1], 
                    Hue[0.20433090079601726`, 1, 1], 
                    Hue[0.2795540383337438, 1, 1], 
                    Hue[0.2795540383337438, 1, 1], 
                    Hue[0.35524739473256994`, 1, 1], 
                    Hue[0.35524739473256994`, 1, 1], 
                    Hue[0.4315296393312207, 1, 1], 
                    Hue[0.4315296393312207, 1, 1], 
                    Hue[0.5082753399542235, 1, 1], 
                    Hue[0.5082753399542235, 1, 1], 
                    Hue[0.5855122705463676, 1, 1], 
                    Hue[0.5855122705463676, 1, 1], 
                    Hue[0.6632910732135462, 1, 1], 
                    Hue[0.6632910732135462, 1, 1], 
                    Hue[0.7415493785590683, 1, 1], 
                    Hue[0.7415493785590683, 1, 1], 
                    Hue[0.8203168163406356, 1, 1], 
                    Hue[0.8203168163406356, 1, 1], 
                    Hue[0.899594466264572, 1, 1], 
                    Hue[0.899594466264572, 1, 1], 
                    Hue[0.9793808535751918, 1, 1], 
                    Hue[0.9793808535751918, 1, 1], 
                    Hue[0.05966611579066608, 1, 1], 
                    Hue[0.05966611579066608, 1, 1], 
                    Hue[0.14046103040636695`, 1, 1], 
                    Hue[0.14046103040636695`, 1, 1], 
                    Hue[0.22176099485644657`, 1, 1], 
                    Hue[0.22176099485644657`, 1, 1], 
                    Hue[0.30355640042518384`, 1, 1], 
                    Hue[0.30355640042518384`, 1, 1], 
                    Hue[0.3858558561080394, 1, 1], 
                    Hue[0.3858558561080394, 1, 1], 
                    Hue[0.46865920579523773`, 1, 1], 
                    Hue[0.46865920579523773`, 1, 1], 
                    Hue[0.5519572630975732, 1, 1], 
                    Hue[0.5519572630975732, 1, 1], 
                    Hue[0.6357616925070961, 1, 1], 
                    Hue[0.6357616925070961, 1, 1], 
                    Hue[0.7200614226226909, 1, 1], 
                    Hue[0.7200614226226909, 1, 1], 
                    Hue[0.8048540983332827, 1, 1], 
                    Hue[0.8048540983332827, 1, 1], 
                    Hue[0.8901452092823243, 1, 1], 
                    Hue[0.8901452092823243, 1, 1], 
                    Hue[0.9759256850504838, 1, 1], 
                    Hue[0.9759256850504838, 1, 1], 
                    Hue[0.06219940245881586, 1, 1], 
                    Hue[0.06219940245881586, 1, 1], 
                    Hue[0.14895952789651382`, 1, 1], 
                    Hue[0.14895952789651382`, 1, 1], 
                    Hue[0.23620276109148372`, 1, 1], 
                    Hue[0.23620276109148372`, 1, 1], 
                    Hue[0.32392749417342787`, 1, 1], 
                    Hue[0.32392749417342787`, 1, 1], 
                    Hue[0.41212937430084273`, 1, 1], 
                    Hue[0.41212937430084273`, 1, 1], 
                    Hue[0.5008039055547484, 1, 1], 
                    Hue[0.5008039055547484, 1, 1], 
                    Hue[0.5899448498811499, 1, 1], 
                    Hue[0.5899448498811499, 1, 1], 
                    Hue[0.6795444249013187, 1, 1], 
                    Hue[0.6795444249013187, 1, 1], 
                    Hue[0.7695963834453866, 1, 1], 
                    Hue[0.7695963834453866, 1, 1], 
                    Hue[0.8600924523736628, 1, 1], 
                    Hue[0.8600924523736628, 1, 1], 
                    Hue[0.9510219118737493, 1, 1], 
                    Hue[0.9510219118737493, 1, 1], 
                    Hue[0.04237273932485363, 1, 1], 
                    Hue[0.04237273932485363, 1, 1], 
                    Hue[0.1341303083120591, 1, 1], 
                    Hue[0.1341303083120591, 1, 1], 
                    Hue[0.2262799008469206, 1, 1], 
                    Hue[0.2262799008469206, 1, 1], 
                    Hue[0.318803207291762, 1, 1], 
                    Hue[0.318803207291762, 1, 1], 
                    Hue[0.41167848966155357`, 1, 1], 
                    Hue[0.41167848966155357`, 1, 1], 
                    Hue[0.5048808051332369, 1, 1], 
                    Hue[0.5048808051332369, 1, 1], 
                    Hue[0.5983810634330753, 1, 1], 
                    Hue[0.5983810634330753, 1, 1], 
                    Hue[0.6921470473254048, 1, 1], 
                    Hue[0.6921470473254048, 1, 1], 
                    Hue[0.7861407557274118, 1, 1], 
                    Hue[0.7861407557274118, 1, 1], 
                    Hue[0.8803183444062389, 1, 1], 
                    Hue[0.8803183444062389, 1, 1], 
                    Hue[0.9746296130822348, 1, 1], 
                    Hue[0.9746296130822348, 1, 1], 
                    Hue[0.06901767356908702, 1, 1], 
                    Hue[0.06901767356908702, 1, 1], 
                    Hue[0.1634186711999563, 1, 1], 
                    Hue[0.1634186711999563, 1, 1], 
                    Hue[0.257759298319671, 1, 1], 
                    Hue[0.257759298319671, 1, 1], 
                    Hue[0.35195625424943705`, 1, 1], 
                    Hue[0.35195625424943705`, 1, 1], 
                    Hue[0.4459170200063353, 1, 1], 
                    Hue[0.4459170200063353, 1, 1], 
                    Hue[0.5395389647253133, 1, 1], 
                    Hue[0.5395389647253133, 1, 1], 
                    Hue[0.6327115014530438, 1, 1], 
                    Hue[0.6327115014530438, 1, 1], 
                    Hue[0.7253120626933802, 1, 1], 
                    Hue[0.7253120626933802, 1, 1], 
                    Hue[0.8172080503626297, 1, 1], 
                    Hue[0.8172080503626297, 1, 1], 
                    Hue[0.9082571817465559, 1, 1], 
                    Hue[0.9082571817465559, 1, 1], 
                    Hue[0.9983125469259199, 1, 1], 
                    Hue[0.9983125469259199, 1, 1], 
                    Hue[0.0872270893257851, 1, 1], 
                    Hue[0.0872270893257851, 1, 1], 
                    Hue[0.17485685640692983`, 1, 1], 
                    Hue[0.17485685640692983`, 1, 1], 
                    Hue[0.26106162246570447`, 1, 1], 
                    Hue[0.26106162246570447`, 1, 1], 
                    Hue[0.3457071509121968, 1, 1], 
                    Hue[0.3457071509121968, 1, 1], 
                    Hue[0.4286721641297487, 1, 1], 
                    Hue[0.4286721641297487, 1, 1], 
                    Hue[0.5098651650273955, 1, 1], 
                    Hue[0.5098651650273955, 1, 1], 
                    Hue[0.5892417687680118, 1, 1], 
                    Hue[0.5892417687680118, 1, 1], 
                    Hue[0.6668211853508332, 1, 1], 
                    Hue[0.6668211853508332, 1, 1], 
                    Hue[0.7426906112473048, 1, 1], 
                    Hue[0.7426906112473048, 1, 1], 
                    Hue[0.8170015469243537, 1, 1], 
                    Hue[0.8170015469243537, 1, 1], 
                    Hue[0.8899533966442782, 1, 1], 
                    Hue[0.8899533966442782, 1, 1], 
                    Hue[0.9617783185528035, 1, 1], 
                    Hue[0.9617783185528035, 1, 1], 
                    Hue[0.03272320056978751, 1, 1], 
                    Hue[0.03272320056978751, 1, 1], 
                    Hue[0.10303459335594814`, 1, 1], 
                    Hue[0.10303459335594814`, 1, 1], 
                    Hue[0.17294878870530267`, 1, 1], 
                    Hue[0.17294878870530267`, 1, 1], 
                    Hue[0.2426863090271892, 1, 1], 
                    Hue[0.2426863090271892, 1, 1], 
                    Hue[0.31244541224626743`, 1, 1], 
                    Hue[0.31244541224626743`, 1, 1], 
                    Hue[0.3824019140435007, 1, 1], 
                    Hue[0.3824019140435007, 1, 1], 
                    Hue[0.4527103697766939, 1, 1], 
                    Hue[0.4527103697766939, 1, 1], 
                    Hue[0.5235041204448956, 1, 1], 
                    Hue[0.5235041204448956, 1, 1], 
                    Hue[0.5948940264243402, 1, 1], 
                    Hue[0.5948940264243402, 1, 1], 
                    Hue[0.6669759737360023, 1, 1], 
                    Hue[0.6669759737360023, 1, 1], 
                    Hue[0.7398286321251204, 1, 1], 
                    Hue[0.7398286321251204, 1, 1], 
                    Hue[0.8135130404584696, 1, 1], 
                    Hue[0.8135130404584696, 1, 1], 
                    Hue[0.8880832031234254, 1, 1], 
                    Hue[0.8880832031234254, 1, 1], 
                    Hue[0.9635811958810462, 1, 1], 
                    Hue[0.9635811958810462, 1, 1], 
                    Hue[0.04003531910755127, 1, 1], 
                    Hue[0.04003531910755127, 1, 1], 
                    Hue[0.11746914394991234`, 1, 1], 
                    Hue[0.11746914394991234`, 1, 1], 
                    Hue[0.195904600981633, 1, 1], 
                    Hue[0.195904600981633, 1, 1], 
                    Hue[0.27535047794179796`, 1, 1], 
                    Hue[0.27535047794179796`, 1, 1], 
                    Hue[0.3558085223791187, 1, 1], 
                    Hue[0.3558085223791187, 1, 1], 
                    Hue[0.4372840484618632, 1, 1], 
                    Hue[0.4372840484618632, 1, 1], 
                    Hue[0.5197882895126544, 1, 1], 
                    Hue[0.5197882895126544, 1, 1], 
                    Hue[0.6033219242113339, 1, 1], 
                    Hue[0.6033219242113339, 1, 1], 
                    Hue[0.6878536441947171, 1, 1], 
                    Hue[0.6878536441947171, 1, 1], 
                    Hue[0.7733697728793487, 1, 1], 
                    Hue[0.7733697728793487, 1, 1], 
                    Hue[0.8598942801945528, 1, 1], 
                    Hue[0.8598942801945528, 1, 1], 
                    Hue[0.9474247816814833, 1, 1], 
                    Hue[0.9474247816814833, 1, 1], 
                    Hue[0.035907408921143254`, 1, 1], 
                    Hue[0.035907408921143254`, 1, 1], 
                    Hue[0.1253083000319576, 1, 1], 
                    Hue[0.1253083000319576, 1, 1], 
                    Hue[0.2156850527530413, 1, 1], 
                    Hue[0.2156850527530413, 1, 1], 
                    Hue[0.30709160770072824`, 1, 1], 
                    Hue[0.30709160770072824`, 1, 1], 
                    Hue[0.3994547063543386, 1, 1], 
                    Hue[0.3994547063543386, 1, 1], 
                    Hue[0.4926921458642045, 1, 1], 
                    Hue[0.4926921458642045, 1, 1], 
                    Hue[0.5868410658775327, 1, 1], 
                    Hue[0.5868410658775327, 1, 1], 
                    Hue[0.6819856197390084, 1, 1], 
                    Hue[0.6819856197390084, 1, 1], 
                    Hue[0.7780905715872407, 1, 1], 
                    Hue[0.7780905715872407, 1, 1], 
                    Hue[0.8747788347944276, 1, 1], 
                    Hue[0.8747788347944276, 1, 1], 
                    Hue[0.9716418911349147, 1, 1], 
                    Hue[0.9716418911349147, 1, 1], 
                    Hue[0.06920173882244322, 1, 1], 
                    Hue[0.06920173882244322, 1, 1], 
                    Hue[0.16854962278205396`, 1, 1], 
                    Hue[0.16854962278205396`, 1, 1], 
                    Hue[0.2695185878541521, 1, 1], 
                    Hue[0.2695185878541521, 1, 1], 
                    Hue[0.3707722030072139, 1, 1], 
                    Hue[0.3707722030072139, 1, 1], 
                    Hue[0.47160607498768725`, 1, 1], 
                    Hue[0.47160607498768725`, 1, 1], 
                    Hue[0.5721384519808977, 1, 1], 
                    Hue[0.5721384519808977, 1, 1], 
                    Hue[0.6738855996878312, 1, 1], 
                    Hue[0.6738855996878312, 1, 1], 
                    Hue[0.78145634117785, 1, 1], 
                    Hue[0.78145634117785, 1, 1], 
                    Hue[0.8966100687768956, 1, 1], 
                    Hue[0.8966100687768956, 1, 1], 
                    Hue[0.00987144155480758, 1, 1], 
                    Hue[0.00987144155480758, 1, 1], 
                    Hue[0.10665989586565482`, 1, 1], 
                    Hue[0.10665989586565482`, 1, 1], 
                    Hue[0.2017755608777928, 1, 1], 
                    Hue[0.2017755608777928, 1, 1], 
                    Hue[0.3243851462421015, 1, 1], 
                    Hue[0.3243851462421015, 1, 1], 
                    Hue[0.46035977061619515`, 1, 1], 
                    Hue[0.46035977061619515`, 1, 1], 
                    Hue[0.5698711687553977, 1, 1], 
                    Hue[0.5698711687553977, 1, 1], 
                    Hue[0.6446141696538195, 1, 1], 
                    Hue[0.6446141696538195, 1, 1], 
                    Hue[0.7649959746908312, 1, 1], 
                    Hue[0.7649959746908312, 1, 1], 
                    Hue[0.8499991304409061, 1, 1], 
                    Hue[0.8499991304409061, 1, 1], 
                    Hue[0.7527311439527943, 1, 1], 
                    Hue[0.7527311439527943, 1, 1], 
                    Hue[0.9407324783675022, 1, 1], 
                    Hue[0.9407324783675022, 1, 1], 
                    Hue[0.16279964711927528`, 1, 1], 
                    Hue[0.16279964711927528`, 1, 1], 
                    Hue[0.40538122810309657`, 1, 1], 
                    Hue[0.40538122810309657`, 1, 1], 
                    Hue[0.7662466372646557, 1, 1], 
                    Hue[0.7662466372646557, 1, 1], 
                    Hue[0.325746588547473, 1, 1], 
                    Hue[0.325746588547473, 1, 1], 
                    Hue[0.6014346559379302, 1, 1], 
                    Hue[0.6014346559379302, 1, 1], 
                    Hue[0.8702034520468045, 1, 1], 
                    Hue[0.8702034520468045, 1, 1], 
                    Hue[0.14000852383014625`, 1, 1], 
                    Hue[0.14000852383014625`, 1, 1], 
                    Hue[0.4094397952932074, 1, 1], 
                    Hue[0.4094397952932074, 1, 1], 
                    Hue[0.6768951372489489, 1, 1], 
                    Hue[0.6768951372489489, 1, 1], 
                    Hue[0.9421369682877887, 1, 1], 
                    Hue[0.9421369682877887, 1, 1], 
                    Hue[0.2058797630336536, 1, 1], 
                    Hue[0.2058797630336536, 1, 1], 
                    Hue[0.4690627681848623, 1, 1], 
                    Hue[0.4690627681848623, 1, 1], 
                    Hue[0.7324642750692172, 1, 1], 
                    Hue[0.7324642750692172, 1, 1], 
                    Hue[0.996675128822844, 1, 1], 
                    Hue[0.996675128822844, 1, 1], 
                    Hue[0.2623619251828773, 1, 1], 
                    Hue[0.2623619251828773, 1, 1], 
                    Hue[0.5308036556013873, 1, 1], 
                    Hue[0.5308036556013873, 1, 1], 
                    Hue[0.8045450818299433, 1, 1], 
                    Hue[0.8045450818299433, 1, 1], 
                    Hue[0.08721915474396433, 1, 1], 
                    Hue[0.08721915474396433, 1, 1], 
                    Hue[0.38020271952928, 1, 1], 
                    Hue[0.38020271952928, 1, 1], 
                    Hue[0.6773830849870384, 1, 1], 
                    Hue[0.6773830849870384, 1, 1], 
                    Hue[0.969304645903326, 1, 1], 
                    Hue[0.969304645903326, 1, 1], 
                    Hue[0.25270882801694694`, 1, 1], 
                    Hue[0.25270882801694694`, 1, 1], 
                    Hue[0.5294894977396489, 1, 1], 
                    Hue[0.5294894977396489, 1, 1], 
                    Hue[0.8021274837723045, 1, 1], 
                    Hue[0.8021274837723045, 1, 1], 
                    Hue[0.07220163138290007, 1, 1], 
                    Hue[0.07220163138290007, 1, 1], 
                    Hue[0.3404993776492227, 1, 1], 
                    Hue[0.3404993776492227, 1, 1], 
                    Hue[0.607341117597648, 1, 1], 
                    Hue[0.607341117597648, 1, 1], 
                    Hue[0.8728024358639596, 1, 1], 
                    Hue[0.8728024358639596, 1, 1], 
                    Hue[0.13683263563569506`, 1, 1], 
                    Hue[0.13683263563569506`, 1, 1], 
                    Hue[0.3993081326856766, 1, 1], 
                    Hue[0.3993081326856766, 1, 1], 
                    Hue[0.6600489781726215, 1, 1], 
                    Hue[0.6600489781726215, 1, 1], 
                    Hue[0.9188134565589243, 1, 1], 
                    Hue[0.9188134565589243, 1, 1], 
                    Hue[0.17528128961485592`, 1, 1], 
                    Hue[0.17528128961485592`, 1, 1], 
                    Hue[0.42902982578944954`, 1, 1], 
                    Hue[0.42902982578944954`, 1, 1], 
                    Hue[0.6795145524088642, 1, 1], 
                    Hue[0.6795145524088642, 1, 1], 
                    Hue[0.9260767691882732, 1, 1], 
                    Hue[0.9260767691882732, 1, 1], 
                    Hue[0.16803549279117072`, 1, 1], 
                    Hue[0.16803549279117072`, 1, 1], 
                    Hue[0.40495850769743, 1, 1], 
                    Hue[0.40495850769743, 1, 1], 
                    Hue[0.6371737992290389, 1, 1], 
                    Hue[0.6371737992290389, 1, 1], 
                    Hue[0.8662893283077272, 1, 1], 
                    Hue[0.8662893283077272, 1, 1], 
                    Hue[0.09505552883252164, 1, 1], 
                    Hue[0.09505552883252164, 1, 1], 
                    Hue[0.3262617090805855, 1, 1], 
                    Hue[0.3262617090805855, 1, 1], 
                    Hue[0.5615905657572104, 1, 1], 
                    Hue[0.5615905657572104, 1, 1], 
                    Hue[0.8014193438577213, 1, 1], 
                    Hue[0.8014193438577213, 1, 1], 
                    Hue[0.04532016587981987, 1, 1], 
                    Hue[0.04532016587981987, 1, 1], 
                    Hue[0.2925849492448882, 1, 1], 
                    Hue[0.2925849492448882, 1, 1], 
                    Hue[0.5425124691739087, 1, 1], 
                    Hue[0.5425124691739087, 1, 1], 
                    Hue[0.7945105484655892, 1, 1], 
                    Hue[0.7945105484655892, 1, 1], 
                    Hue[0.048109368347345416`, 1, 1], 
                    Hue[0.048109368347345416`, 1, 1], 
                    Hue[0.3029441434058117, 1, 1], 
                    Hue[0.3029441434058117, 1, 1], 
                    Hue[0.5587316333455382, 1, 1], 
                    Hue[0.5587316333455382, 1, 1], 
                    Hue[0.8152486151476237, 1, 1], 
                    Hue[0.8152486151476237, 1, 1], 
                    Hue[0.07231525149390143, 1, 1], 
                    Hue[0.07231525149390143, 1, 1], 
                    Hue[0.3297838895830974, 1, 1], 
                    Hue[0.3297838895830974, 1, 1], 
                    Hue[0.587531735292107, 1, 1], 
                    Hue[0.587531735292107, 1, 1], 
                    Hue[0.8454588286735683, 1, 1], 
                    Hue[0.8454588286735683, 1, 1], 
                    Hue[0.10348856732185839`, 1, 1], 
                    Hue[0.10348856732185839`, 1, 1], 
                    Hue[0.36157113441264427`, 1, 1], 
                    Hue[0.36157113441264427`, 1, 1], 
                    Hue[0.6196890157024028, 1, 1], 
                    Hue[0.6196890157024028, 1, 1], 
                    Hue[0.877865442493381, 1, 1], 
                    Hue[0.877865442493381, 1, 1], 
                    Hue[0.136176524325591, 1, 1], 
                    Hue[0.136176524325591, 1, 1], 
                    Hue[0.3947708920661652, 1, 1], 
                    Hue[0.3947708920661652, 1, 1], 
                    Hue[0.6539029141821653, 1, 1]}],
                   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
                   Axes->{True, None},
                   AxesOrigin->{0, 0},
                   
                   Epilog->LineBox[{{1, 2.1382858201739083`*^-6}, {
                    2, 2.339060079404161*^-6}, {3, 2.376119221526492*^-6}, {
                    4, 2.120555731073843*^-6}, {5, 1.8529336061359418`*^-6}, {
                    6, 1.6153949960995193`*^-6}, {
                    7, 1.3789852090552707`*^-6}, {8, 1.302415804345069*^-6}, {
                    9, 1.501766439045054*^-6}, {
                    10, 1.1276930061497953`*^-6}, {
                    11, 1.1879861406833316`*^-6}, {
                    12, 1.193354966904871*^-6}, {
                    13, 1.3233768002965314`*^-6}, {
                    14, 1.1680905784284983`*^-6}, {
                    15, 1.1667147504799766`*^-6}, {
                    16, 1.507444492921938*^-6}, {
                    17, 1.8006348740448443`*^-6}, {
                    18, 1.857867578875766*^-6}, {
                    19, 1.3902697453345237`*^-6}, {
                    20, 1.510050681483681*^-6}, {21, 1.567813569924143*^-6}, {
                    22, 1.5903914531180477`*^-6}, {
                    23, 1.1865065976286821`*^-6}, {
                    24, 1.1054353594364884`*^-6}, {
                    25, 7.210588452885815*^-7}, {26, 8.940815105079953*^-7}, {
                    27, 5.273881262098304*^-7}, {28, 6.835336546419997*^-7}, {
                    29, 4.340082885104023*^-7}, {30, 5.290014758706824*^-7}, {
                    31, 2.5051757124893184`*^-7}, {
                    32, 4.966136797615175*^-7}, {33, 2.592564196897756*^-7}, {
                    34, 3.4915790359867765`*^-7}, {
                    35, 9.30152011544545*^-7}, {
                    36, 1.7623475378555014`*^-6}, {
                    37, 2.461035211615893*^-6}, {38, 2.884707192602002*^-6}, {
                    39, 3.0230334310874387`*^-6}, {
                    40, 3.5256801542079176`*^-6}, {
                    41, 4.505484980139137*^-6}, {42, 7.730591120875744*^-6}, {
                    43, 0.000010995019951795655`}, {
                    44, 0.000013647755202971156`}, {
                    45, 0.000017440561292533342`}, {
                    46, 0.00002324323038979623}, {
                    47, 0.000030360057994099544`}, {
                    48, 0.000040980497481915245`}, {
                    49, 0.00005487871616545832}, {
                    50, 0.00007133697413596286}, {
                    51, 0.00009289810998376431}, {
                    52, 0.00012106783215629224`}, {53, 0.00015717524395985}, {
                    54, 0.00020302805126036442`}, {
                    55, 0.00026130598508145616`}, {
                    56, 0.0003337261550141855}, {
                    57, 0.00042548660628561474`}, {
                    58, 0.0005408066692216618}, {59, 0.000684315054768601}, {
                    60, 0.0008628818638162705}, {
                    61, 0.0010825869868716193`}, {
                    62, 0.0013537209241357726`}, {63, 0.001687037849989684}, {
                    64, 0.0020945832426354635`}, {
                    65, 0.0025904209624385576`}, {
                    66, 0.0031894699452079673`}, {67, 0.003911201350898368}, {
                    68, 0.00477828352225269}, {69, 0.005815411304276308}, {
                    70, 0.007049280497034606}, {71, 0.008510468802000865}, {
                    72, 0.010235691605007152`}, {73, 0.012260152492054262`}, {
                    74, 0.014628175286118844`}, {75, 0.01738573164227005}, {
                    76, 0.020580502482031342`}, {77, 0.024266053760217246`}, {
                    78, 0.028499869783183248`}, {79, 0.03334015248560203}, {
                    80, 0.03884839975581055}, {81, 0.04508765485524873}, {
                    82, 0.0521222361758455}, {83, 0.060018031402123126`}, {
                    84, 0.06883609810206841}, {85, 0.07863668597706475}, {
                    86, 0.08947909356075248}, {87, 0.10141422313530525`}, {
                    88, 0.11448791636504813`}, {89, 0.12873690280441855`}, {
                    90, 0.14418648199119524`}, {91, 0.16085331249797363`}, {
                    92, 0.17874005462737072`}, {93, 0.19783466439917705`}, {
                    94, 0.2181080970444795}, {95, 0.2395117687511355}, {
                    96, 0.26198419912917414`}, {97, 0.28544343387091464`}, {
                    98, 0.3097868490238605}, {99, 0.33489443146655284`}, {
                    100, 0.36062502566214794`}, {101, 0.38682252494012076`}, {
                    102, 0.4133155533673882}, {103, 0.43991919370791405`}, {
                    104, 0.4664300189745512}, {105, 0.4926381883665389}, {
                    106, 0.5183249534190161}, {107, 0.5432729644459915}, {
                    108, 0.5672531321032863}, {109, 0.5900413228532021}, {
                    110, 0.6114159746716843}, {111, 0.6311649709831127}, {
                    112, 0.6490799492485502}, {113, 0.6649576898228602}, {
                    114, 0.6786032629691555}, {115, 0.689831306803855}, {
                    116, 0.6984709989259636}, {117, 0.7043562745743857}, {
                    118, 0.7073213525534522}, {119, 0.7071865721206809}, {
                    120, 0.7037745871626101}, {121, 0.6969171152803978}, {
                    122, 0.6864704926454155}, {123, 0.672297028395317}, {
                    124, 0.6542516273464094}, {125, 0.6321816205350365}, {
                    126, 0.6059782408152176}, {127, 0.575654936274103}, {
                    128, 0.5414445591938546}, {129, 0.5038454476027572}, {
                    130, 0.4636228790127586}, {131, 0.4217439917114315}, {
                    132, 0.3792768951817269}, {133, 0.33727978102526346`}, {
                    134, 0.29670854278782666`}, {135, 0.25834692225007083`}, {
                    136, 0.22277412642377586`}, {137, 0.19036272275752997`}, {
                    138, 0.16129426094679222`}, {139, 0.13558927504375637`}, {
                    140, 0.11314459446037002`}, {141, 0.09376949977285415}, {
                    142, 0.07721311145120266}, {143, 0.06319658541171463}, {
                    144, 0.05143069853887677}, {145, 0.041629078058473934`}, {
                    146, 0.03352153723412999}, {147, 0.026860089085101572`}, {
                    148, 0.021420227538107066`}, {
                    149, 0.017003089258188206`}, {
                    150, 0.013436445001017506`}, {
                    151, 0.010572072002457059`}, {
                    152, 0.008283157490318927}, {
                    153, 0.0064620991332260385`}, {
                    154, 0.005020018877321288}, {
                    155, 0.0038841587806388612`}, {
                    156, 0.0029935584296353424`}, {
                    157, 0.00229772155355313}, {
                    158, 0.0017563471040131194`}, {
                    159, 0.0013375003958987185`}, {
                    160, 0.001015028627598602}, {
                    161, 0.0007673245357657564}, {
                    162, 0.0005775587687211761}, {
                    163, 0.00043298401051007575`}, {
                    164, 0.00032343522327817867`}, {
                    165, 0.00024065693104272095`}, {
                    166, 0.00017814000375619864`}, {
                    167, 0.000131089456116824}, {
                    168, 0.00009620733453323634}, {
                    169, 0.00007075891623862015}, {
                    170, 0.00005202167837452654}, {
                    171, 0.000037919710004153324`}, {
                    172, 0.000027370459729067403`}, {
                    173, 0.00001969208537012718}, {
                    174, 0.000014234723241473395`}, {
                    175, 0.000010455440115850874`}, {
                    176, 7.765678371761643*^-6}, {
                    177, 5.563917013442224*^-6}, {
                    178, 3.686566958292898*^-6}, {
                    179, 2.420571159035341*^-6}, {
                    180, 1.8288925410163764`*^-6}, {
                    181, 1.4119495814965288`*^-6}, {
                    182, 9.117324648688405*^-7}, {
                    183, 4.905996001761964*^-7}, {
                    184, 3.208482041388912*^-7}, {
                    185, 2.2374557380483393`*^-7}, {
                    186, 7.239111397880053*^-8}, {
                    187, 1.3621249555414346`*^-7}, {
                    188, 2.2651663064551017`*^-7}, {
                    189, 2.1760248581751497`*^-7}, {
                    190, 1.3323360094581935`*^-7}, {
                    191, 2.8605117859651873`*^-8}, {
                    192, 8.93382664466351*^-8}, {
                    193, 1.6206649975950964`*^-7}, {
                    194, 2.0636853080860873`*^-7}, {
                    195, 2.293018789119855*^-7}, {
                    196, 2.3704008524187569`*^-7}, {
                    197, 2.318941389157153*^-7}, {
                    198, 2.1494341561434727`*^-7}, {
                    199, 1.8859407010545268`*^-7}, {
                    200, 1.568202962216502*^-7}, {
                    201, 1.2400247214876664`*^-7}, {
                    202, 9.373083593807168*^-8}, {
                    203, 6.822403132828696*^-8}, {
                    204, 4.836159429191074*^-8}, {
                    205, 3.405879680945633*^-8}, {
                    206, 2.4724329773627344`*^-8}, {
                    207, 1.9583119490142026`*^-8}, {
                    208, 1.7664377127784197`*^-8}, {
                    209, 1.771435317475953*^-8}, {
                    210, 1.8532803912708786`*^-8}, {
                    211, 1.930548765974165*^-8}, {
                    212, 1.9599580769626324`*^-8}, {
                    213, 1.924833045316238*^-8}, {
                    214, 1.8256067931996337`*^-8}, {
                    215, 1.6730222546134842`*^-8}, {
                    216, 1.4830750483790228`*^-8}, {
                    217, 1.2732520571241701`*^-8}, {
                    218, 1.0598463786916573`*^-8}, {
                    219, 8.56250966648243*^-9}, {
                    220, 6.721092959136898*^-9}, {
                    221, 5.131840822242209*^-9}, {
                    222, 3.81759590202339*^-9}, {
                    223, 2.7736281729708353`*^-9}, {
                    224, 1.976049425438888*^-9}, {
                    225, 1.3897757185833489`*^-9}, {
                    226, 9.75007799863787*^-10}, {
                    227, 6.91834731105331*^-10}, {
                    228, 5.033307615248293*^-10}, {
                    229, 3.780161128633329*^-10}, {
                    230, 2.9191362980048405`*^-10}, {
                    231, 2.2918958748551362`*^-10}, {
                    232, 1.807814771992523*^-10}, {
                    233, 1.420172115229306*^-10}, {
                    234, 1.105410705034356*^-10}, {
                    235, 8.503124009844324*^-11}, {
                    236, 6.456816906764726*^-11}, {
                    237, 4.838052752373304*^-11}, {
                    238, 3.5767522174338105`*^-11}, {
                    239, 2.60889446130894*^-11}, {
                    240, 1.877322159982756*^-11}, {
                    241, 1.3324770622130949`*^-11}, {
                    242, 9.32596023686214*^-12}, {
                    243, 6.433724368330998*^-12}, {
                    244, 4.372615605967666*^-12}, {
                    245, 2.925946982945934*^-12}, {
                    246, 1.926430237779039*^-12}, {
                    247, 1.2471446420856075`*^-12}, {
                    248, 7.934093691289404*^-13}, {
                    249, 4.957783403383221*^-13}, {
                    250, 3.0420069407013193`*^-13}, {
                    251, 1.83271421292399*^-13}}],
                   Frame->True,
                   FrameTicks->{{{1., 
                    FormBox[
                    RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
                    FormBox[
                    RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
                    FormBox[
                    RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
                    FormBox[
                    RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
                    FormBox["0", TraditionalForm]}, {157.25, 
                    FormBox["2.5`", TraditionalForm]}, {188.5, 
                    FormBox["5", TraditionalForm]}, {219.75, 
                    FormBox["7.5`", TraditionalForm]}, {251., 
                    FormBox["10", TraditionalForm]}}, Automatic, None, None},
                   PlotRange->{-0.1, 1.1},
                   PlotRangeClipping->True,
                   PlotRangePadding->{
                    Scaled[0.02], Automatic}], 9->
                  GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF2Hlczfkex/GfLJPMDGaYGuvR2K4lFRKl3qWkfY+0/ep02k6LMbmjMvql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                    "], {
                    {GrayLevel[0.5], EdgeForm[None], 
                    GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ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                    
                    "]]]}, {{}, {}, {
                    {Hue[0.4168600680892608, 1, 1], LineBox[{1, 2}]}, 
                    {Hue[0.3284984963233482, 1, 1], LineBox[{2, 3}]}, 
                    {Hue[0.24932412379762658`, 1, 1], LineBox[{3, 4}]}, 
                    {Hue[0.13504033356722836`, 1, 1], LineBox[{4, 5}]}, 
                    {Hue[0.9913179497102926, 1, 1], LineBox[{5, 6}]}, 
                    {Hue[0.9044701728619814, 1, 1], LineBox[{6, 7}]}, 
                    {Hue[0.7391832250120645, 1, 1], LineBox[{7, 8}]}, 
                    {Hue[0.6580534334747035, 1, 1], LineBox[{8, 9}]}, 
                    {Hue[0.5388133016638648, 1, 1], LineBox[{9, 10}]}, 
                    {Hue[0.4425053349418048, 1, 1], LineBox[{10, 11}]}, 
                    {Hue[0.4131213114772012, 1, 1], LineBox[{11, 12}]}, 
                    {Hue[0.2819609923741898, 1, 1], LineBox[{12, 13}]}, 
                    {Hue[0.23054292097004633`, 1, 1], LineBox[{13, 14}]}, 
                    {Hue[0.07316661394169041, 1, 1], LineBox[{14, 15}]}, 
                    {Hue[0.02333481242210441, 1, 1], LineBox[{15, 16}]}, 
                    {Hue[0.9691545115150134, 1, 1], LineBox[{16, 17}]}, 
                    {Hue[0.8837532272468197, 1, 1], LineBox[{17, 18}]}, 
                    {Hue[0.8375921797780413, 1, 1], LineBox[{18, 19}]}, 
                    {Hue[0.7888660194019311, 1, 1], LineBox[{19, 20}]}, 
                    {Hue[0.6068473004411263, 1, 1], LineBox[{20, 21}]}, 
                    {Hue[0.586597492966769, 1, 1], LineBox[{21, 22}]}, 
                    {Hue[0.3828866975293449, 1, 1], LineBox[{22, 23}]}, 
                    {Hue[0.339288400593652, 1, 1], LineBox[{23, 24}]}, 
                    {Hue[0.18584920775669142`, 1, 1], LineBox[{24, 25}]}, 
                    {Hue[0.21568596776334784`, 1, 1], LineBox[{25, 26}]}, 
                    {Hue[0.9680533520465127, 1, 1], LineBox[{26, 27}]}, 
                    {Hue[0.7800790594740148, 1, 1], LineBox[{27, 28}]}, 
                    {Hue[0.7392259322801114, 1, 1], LineBox[{28, 29}]}, 
                    {Hue[0.5822206436209341, 1, 1], LineBox[{29, 30}]}, 
                    {Hue[0.6048288102153387, 1, 1], LineBox[{30, 31}]}, 
                    {Hue[0.5106564504880033, 1, 1], LineBox[{31, 32}]}, 
                    {Hue[0.45122902163414735`, 1, 1], LineBox[{32, 33}]}, 
                    {Hue[0.34778934937168415`, 1, 1], LineBox[{33, 34}]}, 
                    {Hue[0.2321365582273194, 1, 1], LineBox[{34, 35}]}, 
                    {Hue[0.18910973578281579`, 1, 1], LineBox[{35, 36}]}, 
                    {Hue[0.09788173808824142, 1, 1], LineBox[{36, 37}]}, 
                    {Hue[0.9791798487066989, 1, 1], LineBox[{37, 38}]}, 
                    {Hue[0.900859846867653, 1, 1], LineBox[{38, 39}]}, 
                    {Hue[0.6167238393623564, 1, 1], LineBox[{39, 40}]}, 
                    {Hue[0.534487194829494, 1, 1], LineBox[{40, 41}]}, 
                    {Hue[0.5813004010570966, 1, 1], LineBox[{41, 42}]}, 
                    {Hue[0.515034083624454, 1, 1], LineBox[{42, 43}]}, 
                    {Hue[0.6103811277352958, 1, 1], LineBox[{43, 44}]}, 
                    {Hue[0.7170574961996892, 1, 1], LineBox[{44, 45}]}, 
                    {Hue[0.789739331819158, 1, 1], LineBox[{45, 46}]}, 
                    {Hue[0.8290180831382562, 1, 1], LineBox[{46, 47}]}, 
                    {Hue[0.8874325399101823, 1, 1], LineBox[{47, 48}]}, 
                    {Hue[0.951266500016831, 1, 1], LineBox[{48, 49}]}, 
                    {Hue[0.025375996397833736`, 1, 1], LineBox[{49, 50}]}, 
                    {Hue[0.09518071115517238, 1, 1], LineBox[{50, 51}]}, 
                    {Hue[0.15224845113862764`, 1, 1], LineBox[{51, 52}]}, 
                    {Hue[0.20884666010258823`, 1, 1], LineBox[{52, 53}]}, 
                    {Hue[0.27363270321178623`, 1, 1], LineBox[{53, 54}]}, 
                    {Hue[0.33988303732676983`, 1, 1], LineBox[{54, 55}]}, 
                    {Hue[0.40458577082292346`, 1, 1], LineBox[{55, 56}]}, 
                    {Hue[0.46924365834371023`, 1, 1], LineBox[{56, 57}]}, 
                    {Hue[0.5337862177226609, 1, 1], LineBox[{57, 58}]}, 
                    {Hue[0.5977805099182919, 1, 1], LineBox[{58, 59}]}, 
                    {Hue[0.6644374554337116, 1, 1], LineBox[{59, 60}]}, 
                    {Hue[0.7316138102104992, 1, 1], LineBox[{60, 61}]}, 
                    {Hue[0.7988801075922869, 1, 1], LineBox[{61, 62}]}, 
                    {Hue[0.8676110266452448, 1, 1], LineBox[{62, 63}]}, 
                    {Hue[0.9359784086629239, 1, 1], LineBox[{63, 64}]}, 
                    {Hue[0.004853955433337728, 1, 1], LineBox[{64, 65}]}, 
                    {Hue[0.07487819872850442, 1, 1], LineBox[{65, 66}]}, 
                    {Hue[0.1448715690079386, 1, 1], LineBox[{66, 67}]}, 
                    {Hue[0.21537600544420657`, 1, 1], LineBox[{67, 68}]}, 
                    {Hue[0.28661749291712185`, 1, 1], LineBox[{68, 69}]}, 
                    {Hue[0.3582009007979773, 1, 1], LineBox[{69, 70}]}, 
                    {Hue[0.4303446834356813, 1, 1], LineBox[{70, 71}]}, 
                    {Hue[0.5030331518422426, 1, 1], LineBox[{71, 72}]}, 
                    {Hue[0.5761971000286218, 1, 1], LineBox[{72, 73}]}, 
                    {Hue[0.6498180906501252, 1, 1], LineBox[{73, 74}]}, 
                    {Hue[0.7240252819100452, 1, 1], LineBox[{74, 75}]}, 
                    {Hue[0.7987410737277055, 1, 1], LineBox[{75, 76}]}, 
                    {Hue[0.8739893909809054, 1, 1], LineBox[{76, 77}]}, 
                    {Hue[0.9497212641793275, 1, 1], LineBox[{77, 78}]}, 
                    {Hue[0.025953380835229217`, 1, 1], LineBox[{78, 79}]}, 
                    {Hue[0.1027263958323548, 1, 1], LineBox[{79, 80}]}, 
                    {Hue[0.18001765715205367`, 1, 1], LineBox[{80, 81}]}, 
                    {Hue[0.2578107175059787, 1, 1], LineBox[{81, 82}]}, 
                    {Hue[0.3361008681236953, 1, 1], LineBox[{82, 83}]}, 
                    {Hue[0.41490177682211926`, 1, 1], LineBox[{83, 84}]}, 
                    {Hue[0.49421569243712077`, 1, 1], LineBox[{84, 85}]}, 
                    {Hue[0.5740441176728341, 1, 1], LineBox[{85, 86}]}, 
                    {Hue[0.654369993646139, 1, 1], LineBox[{86, 87}]}, 
                    {Hue[0.7351953469422576, 1, 1], LineBox[{87, 88}]}, 
                    {Hue[0.81651418750048, 1, 1], LineBox[{88, 89}]}, 
                    {Hue[0.8983316598890928, 1, 1], LineBox[{89, 90}]}, 
                    {Hue[0.9806458205478827, 1, 1], LineBox[{90, 91}]}, 
                    {Hue[0.06344814211594595, 1, 1], LineBox[{91, 92}]}, 
                    {Hue[0.14673048541690584`, 1, 1], LineBox[{92, 93}]}, 
                    {Hue[0.2304863422174567, 1, 1], LineBox[{93, 94}]}, 
                    {Hue[0.31471886861517806`, 1, 1], LineBox[{94, 95}]}, 
                    {Hue[0.39941476841128004`, 1, 1], LineBox[{95, 96}]}, 
                    {Hue[0.48456659088365756`, 1, 1], LineBox[{96, 97}]}, 
                    {Hue[0.5701597559844244, 1, 1], LineBox[{97, 98}]}, 
                    {Hue[0.6561853153134112, 1, 1], LineBox[{98, 99}]}, 
                    {Hue[0.7426309139751042, 1, 1], LineBox[{99, 100}]}, 
                    {Hue[0.8294782523205144, 1, 1], LineBox[{100, 101}]}, 
                    {Hue[0.9167098749001431, 1, 1], LineBox[{101, 102}]}, 
                    {Hue[0.004301163725584873, 1, 1], LineBox[{102, 103}]}, 
                    {Hue[0.09222752982918228, 1, 1], LineBox[{103, 104}]}, 
                    {Hue[0.18046028996418825`, 1, 1], LineBox[{104, 105}]}, 
                    {Hue[0.26896728276140036`, 1, 1], LineBox[{105, 106}]}, 
                    {Hue[0.3577101757639991, 1, 1], LineBox[{106, 107}]}, 
                    {Hue[0.44663945537731664`, 1, 1], LineBox[{107, 108}]}, 
                    {Hue[0.5357014982016148, 1, 1], LineBox[{108, 109}]}, 
                    {Hue[0.62483975362368, 1, 1], LineBox[{109, 110}]}, 
                    {Hue[0.7139918935953822, 1, 1], LineBox[{110, 111}]}, 
                    {Hue[0.8030846259919716, 1, 1], LineBox[{111, 112}]}, 
                    {Hue[0.892026133441005, 1, 1], LineBox[{112, 113}]}, 
                    {Hue[0.9807129381766048, 1, 1], LineBox[{113, 114}]}, 
                    {Hue[0.06903599557663395, 1, 1], LineBox[{114, 115}]}, 
                    {Hue[0.15688514799288122`, 1, 1], LineBox[{115, 116}]}, 
                    {Hue[0.24414326416059198`, 1, 1], LineBox[{116, 117}]}, 
                    {Hue[0.3306707147725039, 1, 1], LineBox[{117, 118}]}, 
                    {Hue[0.41630227337717696`, 1, 1], LineBox[{118, 119}]}, 
                    {Hue[0.500860694705684, 1, 1], LineBox[{119, 120}]}, 
                    {Hue[0.5841800911294179, 1, 1], LineBox[{120, 121}]}, 
                    {Hue[0.6661179424522261, 1, 1], LineBox[{121, 122}]}, 
                    {Hue[0.7465420638122583, 1, 1], LineBox[{122, 123}]}, 
                    {Hue[0.8253073721807094, 1, 1], LineBox[{123, 124}]}, 
                    {Hue[0.9022415102401787, 1, 1], LineBox[{124, 125}]}, 
                    {Hue[0.9771627557972062, 1, 1], LineBox[{125, 126}]}, 
                    {Hue[0.04992714740017764, 1, 1], LineBox[{126, 127}]}, 
                    {Hue[0.12047663023156521`, 1, 1], LineBox[{127, 128}]}, 
                    {Hue[0.18886801330590997`, 1, 1], LineBox[{128, 129}]}, 
                    {Hue[0.2552696366393901, 1, 1], LineBox[{129, 130}]}, 
                    {Hue[0.31993599055739874`, 1, 1], LineBox[{130, 131}]}, 
                    {Hue[0.3831737264972136, 1, 1], LineBox[{131, 132}]}, 
                    {Hue[0.4453093562026673, 1, 1], LineBox[{132, 133}]}, 
                    {Hue[0.5066649663249492, 1, 1], LineBox[{133, 134}]}, 
                    {Hue[0.5675424486716532, 1, 1], LineBox[{134, 135}]}, 
                    {Hue[0.6282157733354248, 1, 1], LineBox[{135, 136}]}, 
                    {Hue[0.6889261744265864, 1, 1], LineBox[{136, 137}]}, 
                    {Hue[0.7498836141773992, 1, 1], LineBox[{137, 138}]}, 
                    {Hue[0.8112670476816446, 1, 1], LineBox[{138, 139}]}, 
                    {Hue[0.8732283191632844, 1, 1], LineBox[{139, 140}]}, 
                    {Hue[0.9358926524849951, 1, 1], LineBox[{140, 141}]}, 
                    {Hue[0.9993644809767988, 1, 1], LineBox[{141, 142}]}, 
                    {Hue[0.06372770144449971, 1, 1], LineBox[{142, 143}]}, 
                    {Hue[0.12904974272742295`, 1, 1], LineBox[{143, 144}]}, 
                    {Hue[0.19538451094276366`, 1, 1], LineBox[{144, 145}]}, 
                    {Hue[0.26277242101711923`, 1, 1], LineBox[{145, 146}]}, 
                    {Hue[0.33124284884960614`, 1, 1], LineBox[{146, 147}]}, 
                    {Hue[0.40081906563092584`, 1, 1], LineBox[{147, 148}]}, 
                    {Hue[0.4715144182185708, 1, 1], LineBox[{148, 149}]}, 
                    {Hue[0.5433368070487188, 1, 1], LineBox[{149, 150}]}, 
                    {Hue[0.6162905659653262, 1, 1], LineBox[{150, 151}]}, 
                    {Hue[0.6903767969105157, 1, 1], LineBox[{151, 152}]}, 
                    {Hue[0.7655875006949686, 1, 1], LineBox[{152, 153}]}, 
                    {Hue[0.841917362845512, 1, 1], LineBox[{153, 154}]}, 
                    {Hue[0.9193636432215327, 1, 1], LineBox[{154, 155}]}, 
                    {Hue[0.9979094392787704, 1, 1], LineBox[{155, 156}]}, 
                    {Hue[0.07754188310626753, 1, 1], LineBox[{156, 157}]}, 
                    {Hue[0.15825696090997052`, 1, 1], LineBox[{157, 158}]}, 
                    {Hue[0.2400363503843214, 1, 1], LineBox[{158, 159}]}, 
                    {Hue[0.32286155835865576`, 1, 1], LineBox[{159, 160}]}, 
                    {Hue[0.4067358030853234, 1, 1], LineBox[{160, 161}]}, 
                    {Hue[0.49164280861494974`, 1, 1], LineBox[{161, 162}]}, 
                    {Hue[0.5775504068185704, 1, 1], LineBox[{162, 163}]}, 
                    {Hue[0.664455646848052, 1, 1], LineBox[{163, 164}]}, 
                    {Hue[0.7523608376696441, 1, 1], LineBox[{164, 165}]}, 
                    {Hue[0.8412461268252658, 1, 1], LineBox[{165, 166}]}, 
                    {Hue[0.9310707885385651, 1, 1], LineBox[{166, 167}]}, 
                    {Hue[0.021832696308046587`, 1, 1], LineBox[{167, 168}]}, 
                    {Hue[0.11358777873201778`, 1, 1], LineBox[{168, 169}]}, 
                    {Hue[0.20629340625631443`, 1, 1], LineBox[{169, 170}]}, 
                    {Hue[0.2997868713777356, 1, 1], LineBox[{170, 171}]}, 
                    {Hue[0.3940467464102866, 1, 1], LineBox[{171, 172}]}, 
                    {Hue[0.48928446488002764`, 1, 1], LineBox[{172, 173}]}, 
                    {Hue[0.5856052024320231, 1, 1], LineBox[{173, 174}]}, 
                    {Hue[0.6827196800638444, 1, 1], LineBox[{174, 175}]}, 
                    {Hue[0.7803999870354645, 1, 1], LineBox[{175, 176}]}, 
                    {Hue[0.8790687016307588, 1, 1], LineBox[{176, 177}]}, 
                    {Hue[0.9791807877677713, 1, 1], LineBox[{177, 178}]}, 
                    {Hue[0.08034201063185673, 1, 1], LineBox[{178, 179}]}, 
                    {Hue[0.18179689340613547`, 1, 1], LineBox[{179, 180}]}, 
                    {Hue[0.283346050031219, 1, 1], LineBox[{180, 181}]}, 
                    {Hue[0.3855860147913132, 1, 1], LineBox[{181, 182}]}, 
                    {Hue[0.4891772391858982, 1, 1], LineBox[{182, 183}]}, 
                    {Hue[0.5928155783977882, 1, 1], LineBox[{183, 184}]}, 
                    {Hue[0.6934891136564652, 1, 1], LineBox[{184, 185}]}, 
                    {Hue[0.793080936386119, 1, 1], LineBox[{185, 186}]}, 
                    {Hue[0.8998311366440165, 1, 1], LineBox[{186, 187}]}, 
                    {Hue[0.015176503380478905`, 1, 1], LineBox[{187, 188}]}, 
                    {Hue[0.13044450564833976`, 1, 1], LineBox[{188, 189}]}, 
                    {Hue[0.23708488707843128`, 1, 1], LineBox[{189, 190}]}, 
                    {Hue[0.33533235811624007`, 1, 1], LineBox[{190, 191}]}, 
                    {Hue[0.4339269518627205, 1, 1], LineBox[{191, 192}]}, 
                    {Hue[0.551635230054077, 1, 1], LineBox[{192, 193}]}, 
                    {Hue[0.7025576980009608, 1, 1], LineBox[{193, 194}]}, 
                    {Hue[0.8827000587356161, 1, 1], LineBox[{194, 195}]}, 
                    {Hue[0.08418342314818303, 1, 1], LineBox[{195, 196}]}, 
                    {Hue[0.7699995303057103, 1, 1], LineBox[{196, 197}]}, 
                    {Hue[0.02429688523103022, 1, 1], LineBox[{197, 198}]}, 
                    {Hue[0.27179627875331236`, 1, 1], LineBox[{198, 199}]}, 
                    {Hue[0.5495531291764919, 1, 1], LineBox[{199, 200}]}, 
                    {Hue[0.8756326603745737, 1, 1], LineBox[{200, 201}]}, 
                    {Hue[0.1870945145097854, 1, 1], LineBox[{201, 202}]}, 
                    {Hue[0.3864954222807409, 1, 1], LineBox[{202, 203}]}, 
                    {Hue[0.44858201992730534`, 1, 1], LineBox[{203, 204}]}, 
                    {Hue[0.6439880334050112, 1, 1], LineBox[{204, 205}]}, 
                    {Hue[0.8858058217557241, 1, 1], LineBox[{205, 206}]}, 
                    {Hue[0.14297565563919354`, 1, 1], LineBox[{206, 207}]}, 
                    {Hue[0.4141276726633181, 1, 1], LineBox[{207, 208}]}, 
                    {Hue[0.7168368232742479, 1, 1], LineBox[{208, 209}]}, 
                    {Hue[0.14197056687390944`, 1, 1], LineBox[{209, 210}]}, 
                    {Hue[0.5524620912908854, 1, 1], LineBox[{210, 211}]}, 
                    {Hue[0.8553893891830866, 1, 1], LineBox[{211, 212}]}, 
                    {Hue[0.13438705262114328`, 1, 1], LineBox[{212, 213}]}, 
                    {Hue[0.4058233803609347, 1, 1], LineBox[{213, 214}]}, 
                    {Hue[0.6738903749915348, 1, 1], LineBox[{214, 215}]}, 
                    {Hue[0.9401789901094688, 1, 1], LineBox[{215, 216}]}, 
                    {Hue[0.20547177438491626`, 1, 1], LineBox[{216, 217}]}, 
                    {Hue[0.47020848142795374`, 1, 1], LineBox[{217, 218}]}, 
                    {Hue[0.734657523019001, 1, 1], LineBox[{218, 219}]}, 
                    {Hue[0.9990322339559546, 1, 1], LineBox[{219, 220}]}, 
                    {Hue[0.2636330251999652, 1, 1], LineBox[{220, 221}]}, 
                    {Hue[0.5290628537980339, 1, 1], LineBox[{221, 222}]}, 
                    {Hue[0.7965350950639479, 1, 1], LineBox[{222, 223}]}, 
                    {Hue[0.06817073638838694, 1, 1], LineBox[{223, 224}]}, 
                    {Hue[0.3467897323971712, 1, 1], LineBox[{224, 225}]}, 
                    {Hue[0.6341252035470084, 1, 1], LineBox[{225, 226}]}, 
                    {Hue[0.9275574020329773, 1, 1], LineBox[{226, 227}]}, 
                    {Hue[0.22028416339828585`, 1, 1], LineBox[{227, 228}]}, 
                    {Hue[0.5071579118616438, 1, 1], LineBox[{228, 229}]}, 
                    {Hue[0.7874886413105265, 1, 1], LineBox[{229, 230}]}, 
                    {Hue[0.06275314718690028, 1, 1], LineBox[{230, 231}]}, 
                    {Hue[0.3345026343916775, 1, 1], LineBox[{231, 232}]}, 
                    {Hue[0.6037857107221525, 1, 1], LineBox[{232, 233}]}, 
                    {Hue[0.8712023572484743, 1, 1], LineBox[{233, 234}]}, 
                    {Hue[0.13705768198816723`, 1, 1], LineBox[{234, 235}]}, 
                    {Hue[0.401476064729792, 1, 1], LineBox[{235, 236}]}, 
                    {Hue[0.6644687447989364, 1, 1], LineBox[{236, 237}]}, 
                    {Hue[0.9259678258280817, 1, 1], LineBox[{237, 238}]}, 
                    {Hue[0.18583835754938358`, 1, 1], LineBox[{238, 239}]}, 
                    {Hue[0.44387781820781336`, 1, 1], LineBox[{239, 240}]}, 
                    {Hue[0.6998052797815966, 1, 1], LineBox[{240, 241}]}, 
                    {Hue[0.9532478642107147, 1, 1], LineBox[{241, 242}]}, 
                    {Hue[0.20373127791721357`, 1, 1], LineBox[{242, 243}]}, 
                    {Hue[0.45069692166927705`, 1, 1], LineBox[{243, 244}]}, 
                    {Hue[0.693588719734344, 1, 1], LineBox[{244, 245}]}, 
                    {Hue[0.9320772375617843, 1, 1], LineBox[{245, 246}]}, 
                    {Hue[0.16645123969962386`, 1, 1], LineBox[{246, 247}]}, 
                    {Hue[0.398004171513236, 1, 1], LineBox[{247, 248}]}, 
                    {Hue[0.6289620661548705, 1, 1], LineBox[{248, 249}]}, 
                    {Hue[0.8617056861385596, 1, 1], LineBox[{249, 250}]}, 
                    {Hue[0.09785397911062299, 1, 1], LineBox[{250, 251}]}}}},
                    VertexColors->{
                    Hue[0.4168600680892608, 1, 1], 
                    Hue[0.3284984963233482, 1, 1], 
                    Hue[0.24932412379762658`, 1, 1], 
                    Hue[0.13504033356722836`, 1, 1], 
                    Hue[0.9913179497102926, 1, 1], 
                    Hue[0.9044701728619814, 1, 1], 
                    Hue[0.7391832250120645, 1, 1], 
                    Hue[0.6580534334747035, 1, 1], 
                    Hue[0.5388133016638648, 1, 1], 
                    Hue[0.4425053349418048, 1, 1], 
                    Hue[0.4131213114772012, 1, 1], 
                    Hue[0.2819609923741898, 1, 1], 
                    Hue[0.23054292097004633`, 1, 1], 
                    Hue[0.07316661394169041, 1, 1], 
                    Hue[0.02333481242210441, 1, 1], 
                    Hue[0.9691545115150134, 1, 1], 
                    Hue[0.8837532272468197, 1, 1], 
                    Hue[0.8375921797780413, 1, 1], 
                    Hue[0.7888660194019311, 1, 1], 
                    Hue[0.6068473004411263, 1, 1], 
                    Hue[0.586597492966769, 1, 1], 
                    Hue[0.3828866975293449, 1, 1], 
                    Hue[0.339288400593652, 1, 1], 
                    Hue[0.18584920775669142`, 1, 1], 
                    Hue[0.21568596776334784`, 1, 1], 
                    Hue[0.9680533520465127, 1, 1], 
                    Hue[0.7800790594740148, 1, 1], 
                    Hue[0.7392259322801114, 1, 1], 
                    Hue[0.5822206436209341, 1, 1], 
                    Hue[0.6048288102153387, 1, 1], 
                    Hue[0.5106564504880033, 1, 1], 
                    Hue[0.45122902163414735`, 1, 1], 
                    Hue[0.34778934937168415`, 1, 1], 
                    Hue[0.2321365582273194, 1, 1], 
                    Hue[0.18910973578281579`, 1, 1], 
                    Hue[0.09788173808824142, 1, 1], 
                    Hue[0.9791798487066989, 1, 1], 
                    Hue[0.900859846867653, 1, 1], 
                    Hue[0.6167238393623564, 1, 1], 
                    Hue[0.534487194829494, 1, 1], 
                    Hue[0.5813004010570966, 1, 1], 
                    Hue[0.515034083624454, 1, 1], 
                    Hue[0.6103811277352958, 1, 1], 
                    Hue[0.7170574961996892, 1, 1], 
                    Hue[0.789739331819158, 1, 1], 
                    Hue[0.8290180831382562, 1, 1], 
                    Hue[0.8874325399101823, 1, 1], 
                    Hue[0.951266500016831, 1, 1], 
                    Hue[0.025375996397833736`, 1, 1], 
                    Hue[0.09518071115517238, 1, 1], 
                    Hue[0.15224845113862764`, 1, 1], 
                    Hue[0.20884666010258823`, 1, 1], 
                    Hue[0.27363270321178623`, 1, 1], 
                    Hue[0.33988303732676983`, 1, 1], 
                    Hue[0.40458577082292346`, 1, 1], 
                    Hue[0.46924365834371023`, 1, 1], 
                    Hue[0.5337862177226609, 1, 1], 
                    Hue[0.5977805099182919, 1, 1], 
                    Hue[0.6644374554337116, 1, 1], 
                    Hue[0.7316138102104992, 1, 1], 
                    Hue[0.7988801075922869, 1, 1], 
                    Hue[0.8676110266452448, 1, 1], 
                    Hue[0.9359784086629239, 1, 1], 
                    Hue[0.004853955433337728, 1, 1], 
                    Hue[0.07487819872850442, 1, 1], 
                    Hue[0.1448715690079386, 1, 1], 
                    Hue[0.21537600544420657`, 1, 1], 
                    Hue[0.28661749291712185`, 1, 1], 
                    Hue[0.3582009007979773, 1, 1], 
                    Hue[0.4303446834356813, 1, 1], 
                    Hue[0.5030331518422426, 1, 1], 
                    Hue[0.5761971000286218, 1, 1], 
                    Hue[0.6498180906501252, 1, 1], 
                    Hue[0.7240252819100452, 1, 1], 
                    Hue[0.7987410737277055, 1, 1], 
                    Hue[0.8739893909809054, 1, 1], 
                    Hue[0.9497212641793275, 1, 1], 
                    Hue[0.025953380835229217`, 1, 1], 
                    Hue[0.1027263958323548, 1, 1], 
                    Hue[0.18001765715205367`, 1, 1], 
                    Hue[0.2578107175059787, 1, 1], 
                    Hue[0.3361008681236953, 1, 1], 
                    Hue[0.41490177682211926`, 1, 1], 
                    Hue[0.49421569243712077`, 1, 1], 
                    Hue[0.5740441176728341, 1, 1], 
                    Hue[0.654369993646139, 1, 1], 
                    Hue[0.7351953469422576, 1, 1], 
                    Hue[0.81651418750048, 1, 1], 
                    Hue[0.8983316598890928, 1, 1], 
                    Hue[0.9806458205478827, 1, 1], 
                    Hue[0.06344814211594595, 1, 1], 
                    Hue[0.14673048541690584`, 1, 1], 
                    Hue[0.2304863422174567, 1, 1], 
                    Hue[0.31471886861517806`, 1, 1], 
                    Hue[0.39941476841128004`, 1, 1], 
                    Hue[0.48456659088365756`, 1, 1], 
                    Hue[0.5701597559844244, 1, 1], 
                    Hue[0.6561853153134112, 1, 1], 
                    Hue[0.7426309139751042, 1, 1], 
                    Hue[0.8294782523205144, 1, 1], 
                    Hue[0.9167098749001431, 1, 1], 
                    Hue[0.004301163725584873, 1, 1], 
                    Hue[0.09222752982918228, 1, 1], 
                    Hue[0.18046028996418825`, 1, 1], 
                    Hue[0.26896728276140036`, 1, 1], 
                    Hue[0.3577101757639991, 1, 1], 
                    Hue[0.44663945537731664`, 1, 1], 
                    Hue[0.5357014982016148, 1, 1], 
                    Hue[0.62483975362368, 1, 1], 
                    Hue[0.7139918935953822, 1, 1], 
                    Hue[0.8030846259919716, 1, 1], 
                    Hue[0.892026133441005, 1, 1], 
                    Hue[0.9807129381766048, 1, 1], 
                    Hue[0.06903599557663395, 1, 1], 
                    Hue[0.15688514799288122`, 1, 1], 
                    Hue[0.24414326416059198`, 1, 1], 
                    Hue[0.3306707147725039, 1, 1], 
                    Hue[0.41630227337717696`, 1, 1], 
                    Hue[0.500860694705684, 1, 1], 
                    Hue[0.5841800911294179, 1, 1], 
                    Hue[0.6661179424522261, 1, 1], 
                    Hue[0.7465420638122583, 1, 1], 
                    Hue[0.8253073721807094, 1, 1], 
                    Hue[0.9022415102401787, 1, 1], 
                    Hue[0.9771627557972062, 1, 1], 
                    Hue[0.04992714740017764, 1, 1], 
                    Hue[0.12047663023156521`, 1, 1], 
                    Hue[0.18886801330590997`, 1, 1], 
                    Hue[0.2552696366393901, 1, 1], 
                    Hue[0.31993599055739874`, 1, 1], 
                    Hue[0.3831737264972136, 1, 1], 
                    Hue[0.4453093562026673, 1, 1], 
                    Hue[0.5066649663249492, 1, 1], 
                    Hue[0.5675424486716532, 1, 1], 
                    Hue[0.6282157733354248, 1, 1], 
                    Hue[0.6889261744265864, 1, 1], 
                    Hue[0.7498836141773992, 1, 1], 
                    Hue[0.8112670476816446, 1, 1], 
                    Hue[0.8732283191632844, 1, 1], 
                    Hue[0.9358926524849951, 1, 1], 
                    Hue[0.9993644809767988, 1, 1], 
                    Hue[0.06372770144449971, 1, 1], 
                    Hue[0.12904974272742295`, 1, 1], 
                    Hue[0.19538451094276366`, 1, 1], 
                    Hue[0.26277242101711923`, 1, 1], 
                    Hue[0.33124284884960614`, 1, 1], 
                    Hue[0.40081906563092584`, 1, 1], 
                    Hue[0.4715144182185708, 1, 1], 
                    Hue[0.5433368070487188, 1, 1], 
                    Hue[0.6162905659653262, 1, 1], 
                    Hue[0.6903767969105157, 1, 1], 
                    Hue[0.7655875006949686, 1, 1], 
                    Hue[0.841917362845512, 1, 1], 
                    Hue[0.9193636432215327, 1, 1], 
                    Hue[0.9979094392787704, 1, 1], 
                    Hue[0.07754188310626753, 1, 1], 
                    Hue[0.15825696090997052`, 1, 1], 
                    Hue[0.2400363503843214, 1, 1], 
                    Hue[0.32286155835865576`, 1, 1], 
                    Hue[0.4067358030853234, 1, 1], 
                    Hue[0.49164280861494974`, 1, 1], 
                    Hue[0.5775504068185704, 1, 1], 
                    Hue[0.664455646848052, 1, 1], 
                    Hue[0.7523608376696441, 1, 1], 
                    Hue[0.8412461268252658, 1, 1], 
                    Hue[0.9310707885385651, 1, 1], 
                    Hue[0.021832696308046587`, 1, 1], 
                    Hue[0.11358777873201778`, 1, 1], 
                    Hue[0.20629340625631443`, 1, 1], 
                    Hue[0.2997868713777356, 1, 1], 
                    Hue[0.3940467464102866, 1, 1], 
                    Hue[0.48928446488002764`, 1, 1], 
                    Hue[0.5856052024320231, 1, 1], 
                    Hue[0.6827196800638444, 1, 1], 
                    Hue[0.7803999870354645, 1, 1], 
                    Hue[0.8790687016307588, 1, 1], 
                    Hue[0.9791807877677713, 1, 1], 
                    Hue[0.08034201063185673, 1, 1], 
                    Hue[0.18179689340613547`, 1, 1], 
                    Hue[0.283346050031219, 1, 1], 
                    Hue[0.3855860147913132, 1, 1], 
                    Hue[0.4891772391858982, 1, 1], 
                    Hue[0.5928155783977882, 1, 1], 
                    Hue[0.6934891136564652, 1, 1], 
                    Hue[0.793080936386119, 1, 1], 
                    Hue[0.8998311366440165, 1, 1], 
                    Hue[0.015176503380478905`, 1, 1], 
                    Hue[0.13044450564833976`, 1, 1], 
                    Hue[0.23708488707843128`, 1, 1], 
                    Hue[0.33533235811624007`, 1, 1], 
                    Hue[0.4339269518627205, 1, 1], 
                    Hue[0.551635230054077, 1, 1], 
                    Hue[0.7025576980009608, 1, 1], 
                    Hue[0.8827000587356161, 1, 1], 
                    Hue[0.08418342314818303, 1, 1], 
                    Hue[0.7699995303057103, 1, 1], 
                    Hue[0.02429688523103022, 1, 1], 
                    Hue[0.27179627875331236`, 1, 1], 
                    Hue[0.5495531291764919, 1, 1], 
                    Hue[0.8756326603745737, 1, 1], 
                    Hue[0.1870945145097854, 1, 1], 
                    Hue[0.3864954222807409, 1, 1], 
                    Hue[0.44858201992730534`, 1, 1], 
                    Hue[0.6439880334050112, 1, 1], 
                    Hue[0.8858058217557241, 1, 1], 
                    Hue[0.14297565563919354`, 1, 1], 
                    Hue[0.4141276726633181, 1, 1], 
                    Hue[0.7168368232742479, 1, 1], 
                    Hue[0.14197056687390944`, 1, 1], 
                    Hue[0.5524620912908854, 1, 1], 
                    Hue[0.8553893891830866, 1, 1], 
                    Hue[0.13438705262114328`, 1, 1], 
                    Hue[0.4058233803609347, 1, 1], 
                    Hue[0.6738903749915348, 1, 1], 
                    Hue[0.9401789901094688, 1, 1], 
                    Hue[0.20547177438491626`, 1, 1], 
                    Hue[0.47020848142795374`, 1, 1], 
                    Hue[0.734657523019001, 1, 1], 
                    Hue[0.9990322339559546, 1, 1], 
                    Hue[0.2636330251999652, 1, 1], 
                    Hue[0.5290628537980339, 1, 1], 
                    Hue[0.7965350950639479, 1, 1], 
                    Hue[0.06817073638838694, 1, 1], 
                    Hue[0.3467897323971712, 1, 1], 
                    Hue[0.6341252035470084, 1, 1], 
                    Hue[0.9275574020329773, 1, 1], 
                    Hue[0.22028416339828585`, 1, 1], 
                    Hue[0.5071579118616438, 1, 1], 
                    Hue[0.7874886413105265, 1, 1], 
                    Hue[0.06275314718690028, 1, 1], 
                    Hue[0.3345026343916775, 1, 1], 
                    Hue[0.6037857107221525, 1, 1], 
                    Hue[0.8712023572484743, 1, 1], 
                    Hue[0.13705768198816723`, 1, 1], 
                    Hue[0.401476064729792, 1, 1], 
                    Hue[0.6644687447989364, 1, 1], 
                    Hue[0.9259678258280817, 1, 1], 
                    Hue[0.18583835754938358`, 1, 1], 
                    Hue[0.44387781820781336`, 1, 1], 
                    Hue[0.6998052797815966, 1, 1], 
                    Hue[0.9532478642107147, 1, 1], 
                    Hue[0.20373127791721357`, 1, 1], 
                    Hue[0.45069692166927705`, 1, 1], 
                    Hue[0.693588719734344, 1, 1], 
                    Hue[0.9320772375617843, 1, 1], 
                    Hue[0.16645123969962386`, 1, 1], 
                    Hue[0.398004171513236, 1, 1], 
                    Hue[0.6289620661548705, 1, 1], 
                    Hue[0.8617056861385596, 1, 1], 
                    Hue[0.09785397911062299, 1, 1], 
                    Hue[0.33796398999826804`, 1, 1], 
                    Hue[0.4168600680892608, 1, 1], 
                    Hue[0.3284984963233482, 1, 1], 
                    Hue[0.3284984963233482, 1, 1], 
                    Hue[0.24932412379762658`, 1, 1], 
                    Hue[0.24932412379762658`, 1, 1], 
                    Hue[0.13504033356722836`, 1, 1], 
                    Hue[0.13504033356722836`, 1, 1], 
                    Hue[0.9913179497102926, 1, 1], 
                    Hue[0.9913179497102926, 1, 1], 
                    Hue[0.9044701728619814, 1, 1], 
                    Hue[0.9044701728619814, 1, 1], 
                    Hue[0.7391832250120645, 1, 1], 
                    Hue[0.7391832250120645, 1, 1], 
                    Hue[0.6580534334747035, 1, 1], 
                    Hue[0.6580534334747035, 1, 1], 
                    Hue[0.5388133016638648, 1, 1], 
                    Hue[0.5388133016638648, 1, 1], 
                    Hue[0.4425053349418048, 1, 1], 
                    Hue[0.4425053349418048, 1, 1], 
                    Hue[0.4131213114772012, 1, 1], 
                    Hue[0.4131213114772012, 1, 1], 
                    Hue[0.2819609923741898, 1, 1], 
                    Hue[0.2819609923741898, 1, 1], 
                    Hue[0.23054292097004633`, 1, 1], 
                    Hue[0.23054292097004633`, 1, 1], 
                    Hue[0.07316661394169041, 1, 1], 
                    Hue[0.07316661394169041, 1, 1], 
                    Hue[0.02333481242210441, 1, 1], 
                    Hue[0.02333481242210441, 1, 1], 
                    Hue[0.9691545115150134, 1, 1], 
                    Hue[0.9691545115150134, 1, 1], 
                    Hue[0.8837532272468197, 1, 1], 
                    Hue[0.8837532272468197, 1, 1], 
                    Hue[0.8375921797780413, 1, 1], 
                    Hue[0.8375921797780413, 1, 1], 
                    Hue[0.7888660194019311, 1, 1], 
                    Hue[0.7888660194019311, 1, 1], 
                    Hue[0.6068473004411263, 1, 1], 
                    Hue[0.6068473004411263, 1, 1], 
                    Hue[0.586597492966769, 1, 1], 
                    Hue[0.586597492966769, 1, 1], 
                    Hue[0.3828866975293449, 1, 1], 
                    Hue[0.3828866975293449, 1, 1], 
                    Hue[0.339288400593652, 1, 1], 
                    Hue[0.339288400593652, 1, 1], 
                    Hue[0.18584920775669142`, 1, 1], 
                    Hue[0.18584920775669142`, 1, 1], 
                    Hue[0.21568596776334784`, 1, 1], 
                    Hue[0.21568596776334784`, 1, 1], 
                    Hue[0.9680533520465127, 1, 1], 
                    Hue[0.9680533520465127, 1, 1], 
                    Hue[0.7800790594740148, 1, 1], 
                    Hue[0.7800790594740148, 1, 1], 
                    Hue[0.7392259322801114, 1, 1], 
                    Hue[0.7392259322801114, 1, 1], 
                    Hue[0.5822206436209341, 1, 1], 
                    Hue[0.5822206436209341, 1, 1], 
                    Hue[0.6048288102153387, 1, 1], 
                    Hue[0.6048288102153387, 1, 1], 
                    Hue[0.5106564504880033, 1, 1], 
                    Hue[0.5106564504880033, 1, 1], 
                    Hue[0.45122902163414735`, 1, 1], 
                    Hue[0.45122902163414735`, 1, 1], 
                    Hue[0.34778934937168415`, 1, 1], 
                    Hue[0.34778934937168415`, 1, 1], 
                    Hue[0.2321365582273194, 1, 1], 
                    Hue[0.2321365582273194, 1, 1], 
                    Hue[0.18910973578281579`, 1, 1], 
                    Hue[0.18910973578281579`, 1, 1], 
                    Hue[0.09788173808824142, 1, 1], 
                    Hue[0.09788173808824142, 1, 1], 
                    Hue[0.9791798487066989, 1, 1], 
                    Hue[0.9791798487066989, 1, 1], 
                    Hue[0.900859846867653, 1, 1], 
                    Hue[0.900859846867653, 1, 1], 
                    Hue[0.6167238393623564, 1, 1], 
                    Hue[0.6167238393623564, 1, 1], 
                    Hue[0.534487194829494, 1, 1], 
                    Hue[0.534487194829494, 1, 1], 
                    Hue[0.5813004010570966, 1, 1], 
                    Hue[0.5813004010570966, 1, 1], 
                    Hue[0.515034083624454, 1, 1], 
                    Hue[0.515034083624454, 1, 1], 
                    Hue[0.6103811277352958, 1, 1], 
                    Hue[0.6103811277352958, 1, 1], 
                    Hue[0.7170574961996892, 1, 1], 
                    Hue[0.7170574961996892, 1, 1], 
                    Hue[0.789739331819158, 1, 1], 
                    Hue[0.789739331819158, 1, 1], 
                    Hue[0.8290180831382562, 1, 1], 
                    Hue[0.8290180831382562, 1, 1], 
                    Hue[0.8874325399101823, 1, 1], 
                    Hue[0.8874325399101823, 1, 1], 
                    Hue[0.951266500016831, 1, 1], 
                    Hue[0.951266500016831, 1, 1], 
                    Hue[0.025375996397833736`, 1, 1], 
                    Hue[0.025375996397833736`, 1, 1], 
                    Hue[0.09518071115517238, 1, 1], 
                    Hue[0.09518071115517238, 1, 1], 
                    Hue[0.15224845113862764`, 1, 1], 
                    Hue[0.15224845113862764`, 1, 1], 
                    Hue[0.20884666010258823`, 1, 1], 
                    Hue[0.20884666010258823`, 1, 1], 
                    Hue[0.27363270321178623`, 1, 1], 
                    Hue[0.27363270321178623`, 1, 1], 
                    Hue[0.33988303732676983`, 1, 1], 
                    Hue[0.33988303732676983`, 1, 1], 
                    Hue[0.40458577082292346`, 1, 1], 
                    Hue[0.40458577082292346`, 1, 1], 
                    Hue[0.46924365834371023`, 1, 1], 
                    Hue[0.46924365834371023`, 1, 1], 
                    Hue[0.5337862177226609, 1, 1], 
                    Hue[0.5337862177226609, 1, 1], 
                    Hue[0.5977805099182919, 1, 1], 
                    Hue[0.5977805099182919, 1, 1], 
                    Hue[0.6644374554337116, 1, 1], 
                    Hue[0.6644374554337116, 1, 1], 
                    Hue[0.7316138102104992, 1, 1], 
                    Hue[0.7316138102104992, 1, 1], 
                    Hue[0.7988801075922869, 1, 1], 
                    Hue[0.7988801075922869, 1, 1], 
                    Hue[0.8676110266452448, 1, 1], 
                    Hue[0.8676110266452448, 1, 1], 
                    Hue[0.9359784086629239, 1, 1], 
                    Hue[0.9359784086629239, 1, 1], 
                    Hue[0.004853955433337728, 1, 1], 
                    Hue[0.004853955433337728, 1, 1], 
                    Hue[0.07487819872850442, 1, 1], 
                    Hue[0.07487819872850442, 1, 1], 
                    Hue[0.1448715690079386, 1, 1], 
                    Hue[0.1448715690079386, 1, 1], 
                    Hue[0.21537600544420657`, 1, 1], 
                    Hue[0.21537600544420657`, 1, 1], 
                    Hue[0.28661749291712185`, 1, 1], 
                    Hue[0.28661749291712185`, 1, 1], 
                    Hue[0.3582009007979773, 1, 1], 
                    Hue[0.3582009007979773, 1, 1], 
                    Hue[0.4303446834356813, 1, 1], 
                    Hue[0.4303446834356813, 1, 1], 
                    Hue[0.5030331518422426, 1, 1], 
                    Hue[0.5030331518422426, 1, 1], 
                    Hue[0.5761971000286218, 1, 1], 
                    Hue[0.5761971000286218, 1, 1], 
                    Hue[0.6498180906501252, 1, 1], 
                    Hue[0.6498180906501252, 1, 1], 
                    Hue[0.7240252819100452, 1, 1], 
                    Hue[0.7240252819100452, 1, 1], 
                    Hue[0.7987410737277055, 1, 1], 
                    Hue[0.7987410737277055, 1, 1], 
                    Hue[0.8739893909809054, 1, 1], 
                    Hue[0.8739893909809054, 1, 1], 
                    Hue[0.9497212641793275, 1, 1], 
                    Hue[0.9497212641793275, 1, 1], 
                    Hue[0.025953380835229217`, 1, 1], 
                    Hue[0.025953380835229217`, 1, 1], 
                    Hue[0.1027263958323548, 1, 1], 
                    Hue[0.1027263958323548, 1, 1], 
                    Hue[0.18001765715205367`, 1, 1], 
                    Hue[0.18001765715205367`, 1, 1], 
                    Hue[0.2578107175059787, 1, 1], 
                    Hue[0.2578107175059787, 1, 1], 
                    Hue[0.3361008681236953, 1, 1], 
                    Hue[0.3361008681236953, 1, 1], 
                    Hue[0.41490177682211926`, 1, 1], 
                    Hue[0.41490177682211926`, 1, 1], 
                    Hue[0.49421569243712077`, 1, 1], 
                    Hue[0.49421569243712077`, 1, 1], 
                    Hue[0.5740441176728341, 1, 1], 
                    Hue[0.5740441176728341, 1, 1], 
                    Hue[0.654369993646139, 1, 1], 
                    Hue[0.654369993646139, 1, 1], 
                    Hue[0.7351953469422576, 1, 1], 
                    Hue[0.7351953469422576, 1, 1], 
                    Hue[0.81651418750048, 1, 1], 
                    Hue[0.81651418750048, 1, 1], 
                    Hue[0.8983316598890928, 1, 1], 
                    Hue[0.8983316598890928, 1, 1], 
                    Hue[0.9806458205478827, 1, 1], 
                    Hue[0.9806458205478827, 1, 1], 
                    Hue[0.06344814211594595, 1, 1], 
                    Hue[0.06344814211594595, 1, 1], 
                    Hue[0.14673048541690584`, 1, 1], 
                    Hue[0.14673048541690584`, 1, 1], 
                    Hue[0.2304863422174567, 1, 1], 
                    Hue[0.2304863422174567, 1, 1], 
                    Hue[0.31471886861517806`, 1, 1], 
                    Hue[0.31471886861517806`, 1, 1], 
                    Hue[0.39941476841128004`, 1, 1], 
                    Hue[0.39941476841128004`, 1, 1], 
                    Hue[0.48456659088365756`, 1, 1], 
                    Hue[0.48456659088365756`, 1, 1], 
                    Hue[0.5701597559844244, 1, 1], 
                    Hue[0.5701597559844244, 1, 1], 
                    Hue[0.6561853153134112, 1, 1], 
                    Hue[0.6561853153134112, 1, 1], 
                    Hue[0.7426309139751042, 1, 1], 
                    Hue[0.7426309139751042, 1, 1], 
                    Hue[0.8294782523205144, 1, 1], 
                    Hue[0.8294782523205144, 1, 1], 
                    Hue[0.9167098749001431, 1, 1], 
                    Hue[0.9167098749001431, 1, 1], 
                    Hue[0.004301163725584873, 1, 1], 
                    Hue[0.004301163725584873, 1, 1], 
                    Hue[0.09222752982918228, 1, 1], 
                    Hue[0.09222752982918228, 1, 1], 
                    Hue[0.18046028996418825`, 1, 1], 
                    Hue[0.18046028996418825`, 1, 1], 
                    Hue[0.26896728276140036`, 1, 1], 
                    Hue[0.26896728276140036`, 1, 1], 
                    Hue[0.3577101757639991, 1, 1], 
                    Hue[0.3577101757639991, 1, 1], 
                    Hue[0.44663945537731664`, 1, 1], 
                    Hue[0.44663945537731664`, 1, 1], 
                    Hue[0.5357014982016148, 1, 1], 
                    Hue[0.5357014982016148, 1, 1], 
                    Hue[0.62483975362368, 1, 1], 
                    Hue[0.62483975362368, 1, 1], 
                    Hue[0.7139918935953822, 1, 1], 
                    Hue[0.7139918935953822, 1, 1], 
                    Hue[0.8030846259919716, 1, 1], 
                    Hue[0.8030846259919716, 1, 1], 
                    Hue[0.892026133441005, 1, 1], 
                    Hue[0.892026133441005, 1, 1], 
                    Hue[0.9807129381766048, 1, 1], 
                    Hue[0.9807129381766048, 1, 1], 
                    Hue[0.06903599557663395, 1, 1], 
                    Hue[0.06903599557663395, 1, 1], 
                    Hue[0.15688514799288122`, 1, 1], 
                    Hue[0.15688514799288122`, 1, 1], 
                    Hue[0.24414326416059198`, 1, 1], 
                    Hue[0.24414326416059198`, 1, 1], 
                    Hue[0.3306707147725039, 1, 1], 
                    Hue[0.3306707147725039, 1, 1], 
                    Hue[0.41630227337717696`, 1, 1], 
                    Hue[0.41630227337717696`, 1, 1], 
                    Hue[0.500860694705684, 1, 1], 
                    Hue[0.500860694705684, 1, 1], 
                    Hue[0.5841800911294179, 1, 1], 
                    Hue[0.5841800911294179, 1, 1], 
                    Hue[0.6661179424522261, 1, 1], 
                    Hue[0.6661179424522261, 1, 1], 
                    Hue[0.7465420638122583, 1, 1], 
                    Hue[0.7465420638122583, 1, 1], 
                    Hue[0.8253073721807094, 1, 1], 
                    Hue[0.8253073721807094, 1, 1], 
                    Hue[0.9022415102401787, 1, 1], 
                    Hue[0.9022415102401787, 1, 1], 
                    Hue[0.9771627557972062, 1, 1], 
                    Hue[0.9771627557972062, 1, 1], 
                    Hue[0.04992714740017764, 1, 1], 
                    Hue[0.04992714740017764, 1, 1], 
                    Hue[0.12047663023156521`, 1, 1], 
                    Hue[0.12047663023156521`, 1, 1], 
                    Hue[0.18886801330590997`, 1, 1], 
                    Hue[0.18886801330590997`, 1, 1], 
                    Hue[0.2552696366393901, 1, 1], 
                    Hue[0.2552696366393901, 1, 1], 
                    Hue[0.31993599055739874`, 1, 1], 
                    Hue[0.31993599055739874`, 1, 1], 
                    Hue[0.3831737264972136, 1, 1], 
                    Hue[0.3831737264972136, 1, 1], 
                    Hue[0.4453093562026673, 1, 1], 
                    Hue[0.4453093562026673, 1, 1], 
                    Hue[0.5066649663249492, 1, 1], 
                    Hue[0.5066649663249492, 1, 1], 
                    Hue[0.5675424486716532, 1, 1], 
                    Hue[0.5675424486716532, 1, 1], 
                    Hue[0.6282157733354248, 1, 1], 
                    Hue[0.6282157733354248, 1, 1], 
                    Hue[0.6889261744265864, 1, 1], 
                    Hue[0.6889261744265864, 1, 1], 
                    Hue[0.7498836141773992, 1, 1], 
                    Hue[0.7498836141773992, 1, 1], 
                    Hue[0.8112670476816446, 1, 1], 
                    Hue[0.8112670476816446, 1, 1], 
                    Hue[0.8732283191632844, 1, 1], 
                    Hue[0.8732283191632844, 1, 1], 
                    Hue[0.9358926524849951, 1, 1], 
                    Hue[0.9358926524849951, 1, 1], 
                    Hue[0.9993644809767988, 1, 1], 
                    Hue[0.9993644809767988, 1, 1], 
                    Hue[0.06372770144449971, 1, 1], 
                    Hue[0.06372770144449971, 1, 1], 
                    Hue[0.12904974272742295`, 1, 1], 
                    Hue[0.12904974272742295`, 1, 1], 
                    Hue[0.19538451094276366`, 1, 1], 
                    Hue[0.19538451094276366`, 1, 1], 
                    Hue[0.26277242101711923`, 1, 1], 
                    Hue[0.26277242101711923`, 1, 1], 
                    Hue[0.33124284884960614`, 1, 1], 
                    Hue[0.33124284884960614`, 1, 1], 
                    Hue[0.40081906563092584`, 1, 1], 
                    Hue[0.40081906563092584`, 1, 1], 
                    Hue[0.4715144182185708, 1, 1], 
                    Hue[0.4715144182185708, 1, 1], 
                    Hue[0.5433368070487188, 1, 1], 
                    Hue[0.5433368070487188, 1, 1], 
                    Hue[0.6162905659653262, 1, 1], 
                    Hue[0.6162905659653262, 1, 1], 
                    Hue[0.6903767969105157, 1, 1], 
                    Hue[0.6903767969105157, 1, 1], 
                    Hue[0.7655875006949686, 1, 1], 
                    Hue[0.7655875006949686, 1, 1], 
                    Hue[0.841917362845512, 1, 1], 
                    Hue[0.841917362845512, 1, 1], 
                    Hue[0.9193636432215327, 1, 1], 
                    Hue[0.9193636432215327, 1, 1], 
                    Hue[0.9979094392787704, 1, 1], 
                    Hue[0.9979094392787704, 1, 1], 
                    Hue[0.07754188310626753, 1, 1], 
                    Hue[0.07754188310626753, 1, 1], 
                    Hue[0.15825696090997052`, 1, 1], 
                    Hue[0.15825696090997052`, 1, 1], 
                    Hue[0.2400363503843214, 1, 1], 
                    Hue[0.2400363503843214, 1, 1], 
                    Hue[0.32286155835865576`, 1, 1], 
                    Hue[0.32286155835865576`, 1, 1], 
                    Hue[0.4067358030853234, 1, 1], 
                    Hue[0.4067358030853234, 1, 1], 
                    Hue[0.49164280861494974`, 1, 1], 
                    Hue[0.49164280861494974`, 1, 1], 
                    Hue[0.5775504068185704, 1, 1], 
                    Hue[0.5775504068185704, 1, 1], 
                    Hue[0.664455646848052, 1, 1], 
                    Hue[0.664455646848052, 1, 1], 
                    Hue[0.7523608376696441, 1, 1], 
                    Hue[0.7523608376696441, 1, 1], 
                    Hue[0.8412461268252658, 1, 1], 
                    Hue[0.8412461268252658, 1, 1], 
                    Hue[0.9310707885385651, 1, 1], 
                    Hue[0.9310707885385651, 1, 1], 
                    Hue[0.021832696308046587`, 1, 1], 
                    Hue[0.021832696308046587`, 1, 1], 
                    Hue[0.11358777873201778`, 1, 1], 
                    Hue[0.11358777873201778`, 1, 1], 
                    Hue[0.20629340625631443`, 1, 1], 
                    Hue[0.20629340625631443`, 1, 1], 
                    Hue[0.2997868713777356, 1, 1], 
                    Hue[0.2997868713777356, 1, 1], 
                    Hue[0.3940467464102866, 1, 1], 
                    Hue[0.3940467464102866, 1, 1], 
                    Hue[0.48928446488002764`, 1, 1], 
                    Hue[0.48928446488002764`, 1, 1], 
                    Hue[0.5856052024320231, 1, 1], 
                    Hue[0.5856052024320231, 1, 1], 
                    Hue[0.6827196800638444, 1, 1], 
                    Hue[0.6827196800638444, 1, 1], 
                    Hue[0.7803999870354645, 1, 1], 
                    Hue[0.7803999870354645, 1, 1], 
                    Hue[0.8790687016307588, 1, 1], 
                    Hue[0.8790687016307588, 1, 1], 
                    Hue[0.9791807877677713, 1, 1], 
                    Hue[0.9791807877677713, 1, 1], 
                    Hue[0.08034201063185673, 1, 1], 
                    Hue[0.08034201063185673, 1, 1], 
                    Hue[0.18179689340613547`, 1, 1], 
                    Hue[0.18179689340613547`, 1, 1], 
                    Hue[0.283346050031219, 1, 1], 
                    Hue[0.283346050031219, 1, 1], 
                    Hue[0.3855860147913132, 1, 1], 
                    Hue[0.3855860147913132, 1, 1], 
                    Hue[0.4891772391858982, 1, 1], 
                    Hue[0.4891772391858982, 1, 1], 
                    Hue[0.5928155783977882, 1, 1], 
                    Hue[0.5928155783977882, 1, 1], 
                    Hue[0.6934891136564652, 1, 1], 
                    Hue[0.6934891136564652, 1, 1], 
                    Hue[0.793080936386119, 1, 1], 
                    Hue[0.793080936386119, 1, 1], 
                    Hue[0.8998311366440165, 1, 1], 
                    Hue[0.8998311366440165, 1, 1], 
                    Hue[0.015176503380478905`, 1, 1], 
                    Hue[0.015176503380478905`, 1, 1], 
                    Hue[0.13044450564833976`, 1, 1], 
                    Hue[0.13044450564833976`, 1, 1], 
                    Hue[0.23708488707843128`, 1, 1], 
                    Hue[0.23708488707843128`, 1, 1], 
                    Hue[0.33533235811624007`, 1, 1], 
                    Hue[0.33533235811624007`, 1, 1], 
                    Hue[0.4339269518627205, 1, 1], 
                    Hue[0.4339269518627205, 1, 1], 
                    Hue[0.551635230054077, 1, 1], 
                    Hue[0.551635230054077, 1, 1], 
                    Hue[0.7025576980009608, 1, 1], 
                    Hue[0.7025576980009608, 1, 1], 
                    Hue[0.8827000587356161, 1, 1], 
                    Hue[0.8827000587356161, 1, 1], 
                    Hue[0.08418342314818303, 1, 1], 
                    Hue[0.08418342314818303, 1, 1], 
                    Hue[0.7699995303057103, 1, 1], 
                    Hue[0.7699995303057103, 1, 1], 
                    Hue[0.02429688523103022, 1, 1], 
                    Hue[0.02429688523103022, 1, 1], 
                    Hue[0.27179627875331236`, 1, 1], 
                    Hue[0.27179627875331236`, 1, 1], 
                    Hue[0.5495531291764919, 1, 1], 
                    Hue[0.5495531291764919, 1, 1], 
                    Hue[0.8756326603745737, 1, 1], 
                    Hue[0.8756326603745737, 1, 1], 
                    Hue[0.1870945145097854, 1, 1], 
                    Hue[0.1870945145097854, 1, 1], 
                    Hue[0.3864954222807409, 1, 1], 
                    Hue[0.3864954222807409, 1, 1], 
                    Hue[0.44858201992730534`, 1, 1], 
                    Hue[0.44858201992730534`, 1, 1], 
                    Hue[0.6439880334050112, 1, 1], 
                    Hue[0.6439880334050112, 1, 1], 
                    Hue[0.8858058217557241, 1, 1], 
                    Hue[0.8858058217557241, 1, 1], 
                    Hue[0.14297565563919354`, 1, 1], 
                    Hue[0.14297565563919354`, 1, 1], 
                    Hue[0.4141276726633181, 1, 1], 
                    Hue[0.4141276726633181, 1, 1], 
                    Hue[0.7168368232742479, 1, 1], 
                    Hue[0.7168368232742479, 1, 1], 
                    Hue[0.14197056687390944`, 1, 1], 
                    Hue[0.14197056687390944`, 1, 1], 
                    Hue[0.5524620912908854, 1, 1], 
                    Hue[0.5524620912908854, 1, 1], 
                    Hue[0.8553893891830866, 1, 1], 
                    Hue[0.8553893891830866, 1, 1], 
                    Hue[0.13438705262114328`, 1, 1], 
                    Hue[0.13438705262114328`, 1, 1], 
                    Hue[0.4058233803609347, 1, 1], 
                    Hue[0.4058233803609347, 1, 1], 
                    Hue[0.6738903749915348, 1, 1], 
                    Hue[0.6738903749915348, 1, 1], 
                    Hue[0.9401789901094688, 1, 1], 
                    Hue[0.9401789901094688, 1, 1], 
                    Hue[0.20547177438491626`, 1, 1], 
                    Hue[0.20547177438491626`, 1, 1], 
                    Hue[0.47020848142795374`, 1, 1], 
                    Hue[0.47020848142795374`, 1, 1], 
                    Hue[0.734657523019001, 1, 1], 
                    Hue[0.734657523019001, 1, 1], 
                    Hue[0.9990322339559546, 1, 1], 
                    Hue[0.9990322339559546, 1, 1], 
                    Hue[0.2636330251999652, 1, 1], 
                    Hue[0.2636330251999652, 1, 1], 
                    Hue[0.5290628537980339, 1, 1], 
                    Hue[0.5290628537980339, 1, 1], 
                    Hue[0.7965350950639479, 1, 1], 
                    Hue[0.7965350950639479, 1, 1], 
                    Hue[0.06817073638838694, 1, 1], 
                    Hue[0.06817073638838694, 1, 1], 
                    Hue[0.3467897323971712, 1, 1], 
                    Hue[0.3467897323971712, 1, 1], 
                    Hue[0.6341252035470084, 1, 1], 
                    Hue[0.6341252035470084, 1, 1], 
                    Hue[0.9275574020329773, 1, 1], 
                    Hue[0.9275574020329773, 1, 1], 
                    Hue[0.22028416339828585`, 1, 1], 
                    Hue[0.22028416339828585`, 1, 1], 
                    Hue[0.5071579118616438, 1, 1], 
                    Hue[0.5071579118616438, 1, 1], 
                    Hue[0.7874886413105265, 1, 1], 
                    Hue[0.7874886413105265, 1, 1], 
                    Hue[0.06275314718690028, 1, 1], 
                    Hue[0.06275314718690028, 1, 1], 
                    Hue[0.3345026343916775, 1, 1], 
                    Hue[0.3345026343916775, 1, 1], 
                    Hue[0.6037857107221525, 1, 1], 
                    Hue[0.6037857107221525, 1, 1], 
                    Hue[0.8712023572484743, 1, 1], 
                    Hue[0.8712023572484743, 1, 1], 
                    Hue[0.13705768198816723`, 1, 1], 
                    Hue[0.13705768198816723`, 1, 1], 
                    Hue[0.401476064729792, 1, 1], 
                    Hue[0.401476064729792, 1, 1], 
                    Hue[0.6644687447989364, 1, 1], 
                    Hue[0.6644687447989364, 1, 1], 
                    Hue[0.9259678258280817, 1, 1], 
                    Hue[0.9259678258280817, 1, 1], 
                    Hue[0.18583835754938358`, 1, 1], 
                    Hue[0.18583835754938358`, 1, 1], 
                    Hue[0.44387781820781336`, 1, 1], 
                    Hue[0.44387781820781336`, 1, 1], 
                    Hue[0.6998052797815966, 1, 1], 
                    Hue[0.6998052797815966, 1, 1], 
                    Hue[0.9532478642107147, 1, 1], 
                    Hue[0.9532478642107147, 1, 1], 
                    Hue[0.20373127791721357`, 1, 1], 
                    Hue[0.20373127791721357`, 1, 1], 
                    Hue[0.45069692166927705`, 1, 1], 
                    Hue[0.45069692166927705`, 1, 1], 
                    Hue[0.693588719734344, 1, 1], 
                    Hue[0.693588719734344, 1, 1], 
                    Hue[0.9320772375617843, 1, 1], 
                    Hue[0.9320772375617843, 1, 1], 
                    Hue[0.16645123969962386`, 1, 1], 
                    Hue[0.16645123969962386`, 1, 1], 
                    Hue[0.398004171513236, 1, 1], 
                    Hue[0.398004171513236, 1, 1], 
                    Hue[0.6289620661548705, 1, 1], 
                    Hue[0.6289620661548705, 1, 1], 
                    Hue[0.8617056861385596, 1, 1], 
                    Hue[0.8617056861385596, 1, 1], 
                    Hue[0.09785397911062299, 1, 1], 
                    Hue[0.09785397911062299, 1, 1], 
                    Hue[0.33796398999826804`, 1, 1]}],
                   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
                   Axes->{True, None},
                   AxesOrigin->{0, 0},
                   
                   Epilog->LineBox[{{1, 1.304380383737595*^-6}, {
                    2, 9.629156898030052*^-7}, {3, 9.032900884567711*^-7}, {
                    4, 1.0373227579521636`*^-6}, {
                    5, 1.1315018027794283`*^-6}, {6, 1.432676966517547*^-6}, {
                    7, 1.5242952737204173`*^-6}, {
                    8, 1.4674159318854584`*^-6}, {
                    9, 1.4190676919269421`*^-6}, {
                    10, 1.2139812463018507`*^-6}, {
                    11, 1.2904452266002735`*^-6}, {
                    12, 1.538037344047358*^-6}, {
                    13, 1.5125623946061144`*^-6}, {
                    14, 1.397844282342397*^-6}, {
                    15, 1.0782540134931227`*^-6}, {
                    16, 9.061955126660892*^-7}, {17, 5.362202811743981*^-7}, {
                    18, 8.152745909536184*^-7}, {19, 6.332578572294198*^-7}, {
                    20, 6.449360220860603*^-7}, {21, 6.33376259274902*^-7}, {
                    22, 7.887259241188358*^-7}, {
                    23, 3.6743487421042405`*^-7}, {
                    24, 1.235500858470756*^-7}, {25, 5.064205796295638*^-7}, {
                    26, 1.0841054261785144`*^-6}, {
                    27, 7.044021040626456*^-7}, {28, 9.808584854079515*^-7}, {
                    29, 8.956095248079672*^-7}, {30, 5.439898906183192*^-7}, {
                    31, 1.103570430545134*^-6}, {32, 8.54911256353534*^-7}, {
                    33, 1.07874097863397*^-6}, {34, 7.811829745586429*^-7}, {
                    35, 9.5592624380291*^-7}, {36, 5.298554720683576*^-7}, {
                    37, 3.8995572112254075`*^-7}, {
                    38, 4.427745130584191*^-7}, {39, 9.928965231940917*^-7}, {
                    40, 1.112065013186867*^-6}, {
                    41, 1.7190219615230559`*^-6}, {
                    42, 1.8282257305026606`*^-6}, {
                    43, 1.4711942109645388`*^-6}, {
                    44, 2.2477877159309747`*^-6}, {
                    45, 3.172475454053349*^-6}, {46, 3.723124351694845*^-6}, {
                    47, 5.458674755317944*^-6}, {
                    48, 6.6410853868583535`*^-6}, {
                    49, 8.03685681726843*^-6}, {
                    50, 0.000012243606787475356`}, {
                    51, 0.000016202644180892942`}, {
                    52, 0.000020916086173850225`}, {
                    53, 0.00002715864602049856}, {
                    54, 0.00003578593120562926}, {
                    55, 0.000046707612943799305`}, {
                    56, 0.00006093738990640693}, {
                    57, 0.00007931776292023355}, {
                    58, 0.00010124319103865022`}, {
                    59, 0.00012965715251245875`}, {
                    60, 0.0001652184262445919}, {
                    61, 0.00021070607757578402`}, {
                    62, 0.00026706057946122885`}, {
                    63, 0.0003375466976779362}, {64, 0.000424750641144717}, {
                    65, 0.0005335389928184603}, {66, 0.0006685709324789817}, {
                    67, 0.000833697099042285}, {68, 0.0010351930740482775`}, {
                    69, 0.001281545042875129}, {70, 0.0015817836788480037`}, {
                    71, 0.0019445059845353218`}, {
                    72, 0.0023819412998479996`}, {
                    73, 0.0029071103864555577`}, {
                    74, 0.0035356522085702603`}, {75, 0.004284295821573266}, {
                    76, 0.005174325557735281}, {77, 0.0062266481535458015`}, {
                    78, 0.007465823368299143}, {79, 0.008919008605186053}, {
                    80, 0.01061821172547095}, {81, 0.012597352819328065`}, {
                    82, 0.014891544699183724`}, {83, 0.017540079728647955`}, {
                    84, 0.020585731204115325`}, {85, 0.02407462916734266}, {
                    86, 0.028056806327298605`}, {87, 0.03258154002325569}, {
                    88, 0.03770128574982179}, {89, 0.04346872822828755}, {
                    90, 0.049941338680835376`}, {91, 0.0571771557928496}, {
                    92, 0.06523001180642563}, {93, 0.07415248764148753}, {
                    94, 0.0839979716623994}, {95, 0.09481631665433748}, {
                    96, 0.10665439417967083`}, {97, 0.11955198565177566`}, {
                    98, 0.1335406133311027}, {99, 0.14864848607449443`}, {
                    100, 0.16489410852233816`}, {101, 0.18228798800728993`}, {
                    102, 0.20082834815434092`}, {103, 0.22050392699708302`}, {
                    104, 0.24128891036818118`}, {105, 0.263147951520146}, {
                    106, 0.28604035243811315`}, {107, 0.3099115410549996}, {
                    108, 0.334683039515788}, {109, 0.3602663228412355}, {
                    110, 0.38656268562356294`}, {111, 0.4134835469962792}, {
                    112, 0.4409265278656139}, {113, 0.4687620701608355}, {
                    114, 0.49682142648520244`}, {115, 0.5249285694578344}, {
                    116, 0.552924946944084}, {117, 0.5806733894641674}, {
                    118, 0.6079994045309144}, {119, 0.63462523765809}, {
                    120, 0.6601575908398875}, {121, 0.6841621818362645}, {
                    122, 0.7062554709286154}, {123, 0.7261075128035555}, {
                    124, 0.7433148537999551}, {125, 0.7572078110597844}, {
                    126, 0.7667599545390897}, {127, 0.7706756788784703}, {
                    128, 0.7676502807652559}, {129, 0.7566859746593542}, {
                    130, 0.7373488418893955}, {131, 0.7098626238028884}, {
                    132, 0.6750710832816553}, {133, 0.6342832833534616}, {
                    134, 0.5890813679254239}, {135, 0.5411240287219006}, {
                    136, 0.4919916061629801}, {137, 0.4430774442912328}, {
                    138, 0.3955303462601081}, {139, 0.3502314542429103}, {
                    140, 0.3078085569932737}, {141, 0.26866006459640857`}, {
                    142, 0.23299054352683432`}, {143, 0.2008530161252165}, {
                    144, 0.17218190691170104`}, {145, 0.14682603144900405`}, {
                    146, 0.12458004102748296`}, {147, 0.10520184293587102`}, {
                    148, 0.08843238184821163}, {149, 0.0740093904889083}, {
                    150, 0.0616756501235092}, {151, 0.05118426414991482}, {
                    152, 0.042305326067561856`}, {
                    153, 0.034828991055787774`}, {154, 0.02856219766143268}, {
                    155, 0.023332515058396282`}, {156, 0.01898868316559714}, {
                    157, 0.015395866592511739`}, {
                    158, 0.012436135985384074`}, {
                    159, 0.010008781861144097`}, {
                    160, 0.008026172857656336}, {161, 0.006412440134820296}, {
                    162, 0.00510450378905798}, {163, 0.004049021298917947}, {
                    164, 0.003200306833496937}, {
                    165, 0.0025201728799015273`}, {
                    166, 0.0019773560552377393`}, {
                    167, 0.001546238502737701}, {168, 0.001204890761178659}, {
                    169, 0.0009350022339971854}, {
                    170, 0.0007226613520117723}, {
                    171, 0.000556982984964478}, {
                    172, 0.0004282547875772621}, {
                    173, 0.0003279812780222784}, {
                    174, 0.00024993972448124314`}, {
                    175, 0.00018993337322628718`}, {
                    176, 0.000144209093655313}, {
                    177, 0.00010907024451671457`}, {
                    178, 0.00008184276173099311}, {
                    179, 0.00006097626515968016}, {
                    180, 0.00004530442269158052}, {
                    181, 0.000033658099287237654`}, {
                    182, 0.0000248843899827779}, {
                    183, 0.000018153907330621393`}, {
                    184, 0.000013223451202338003`}, {
                    185, 9.963764949990425*^-6}, {
                    186, 7.724307684131922*^-6}, {
                    187, 5.80543629987835*^-6}, {
                    188, 4.102828122612841*^-6}, {
                    189, 2.827451607612426*^-6}, {
                    190, 2.032892416954505*^-6}, {
                    191, 1.6106628657117503`*^-6}, {
                    192, 1.4041728160521102`*^-6}, {
                    193, 1.1798805921599137`*^-6}, {
                    194, 8.079934780963365*^-7}, {
                    195, 3.556446757918429*^-7}, {
                    196, 2.546949050174494*^-8}, {
                    197, 2.2850477355468987`*^-7}, {
                    198, 2.528800192503595*^-7}, {
                    199, 1.806456782338626*^-7}, {
                    200, 1.1667792907080793`*^-7}, {
                    201, 9.006204444596412*^-8}, {
                    202, 5.635361049712508*^-8}, {
                    203, 7.32045438122571*^-8}, {
                    204, 1.3707651137223234`*^-7}, {
                    205, 1.778813899422182*^-7}, {
                    206, 1.7806604029286668`*^-7}, {
                    207, 1.4189267605809225`*^-7}, {
                    208, 8.578004895793163*^-8}, {
                    209, 4.9819202470450704`*^-8}, {
                    210, 8.972369549354587*^-8}, {
                    211, 1.4506298276404592`*^-7}, {
                    212, 1.8975320201858187`*^-7}, {
                    213, 2.1856689255803505`*^-7}, {
                    214, 2.3055555397033795`*^-7}, {
                    215, 2.2708109218557778`*^-7}, {
                    216, 2.1110640057225708`*^-7}, {
                    217, 1.8650892436038908`*^-7}, {
                    218, 1.5736542112103807`*^-7}, {
                    219, 1.2733903044577994`*^-7}, {
                    220, 9.92704270549027*^-8}, {
                    221, 7.500484520111254*^-8}, {
                    222, 5.542957376388741*^-8}, {223, 4.066821604665421*^-8},
                     {224, 3.036283441282917*^-8}, {
                    225, 2.3928998369784318`*^-8}, {
                    226, 2.0627484727031363`*^-8}, {
                    227, 1.9495985991114893`*^-8}, {
                    228, 1.9482414369140438`*^-8}, {
                    229, 1.9744327165034357`*^-8}, {
                    230, 1.9768316718986537`*^-8}, {
                    231, 1.9314466284698768`*^-8}, {
                    232, 1.8328711162169572`*^-8}, {
                    233, 1.6872297479929105`*^-8}, {
                    234, 1.5069496006086792`*^-8}, {
                    235, 1.3068841459865362`*^-8}, {
                    236, 1.1015253781473726`*^-8}, {
                    237, 9.032110441132644*^-9}, {
                    238, 7.211733456886917*^-9}, {
                    239, 5.612885300624509*^-9}, {
                    240, 4.2632885328740104`*^-9}, {
                    241, 3.1652340412508406`*^-9}, {
                    242, 2.3025167109426824`*^-9}, {
                    243, 1.6473203311378306`*^-9}, {
                    244, 1.1661145086376094`*^-9}, {
                    245, 8.241084823545091*^-10}, {
                    246, 5.882170342111297*^-10}, {
                    247, 4.2890653665416393`*^-10}, {
                    248, 3.215122368663073*^-10}, {
                    249, 2.472321589515314*^-10}, {
                    250, 1.9331961245459556`*^-10}, {
                    251, 1.521207633440912*^-10}}],
                   Frame->True,
                   FrameTicks->{{{1., 
                    FormBox[
                    RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
                    FormBox[
                    RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
                    FormBox[
                    RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
                    FormBox[
                    RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
                    FormBox["0", TraditionalForm]}, {157.25, 
                    FormBox["2.5`", TraditionalForm]}, {188.5, 
                    FormBox["5", TraditionalForm]}, {219.75, 
                    FormBox["7.5`", TraditionalForm]}, {251., 
                    FormBox["10", TraditionalForm]}}, Automatic, None, None},
                   PlotRange->{-0.1, 1.1},
                   PlotRangeClipping->True,
                   PlotRangePadding->{
                    Scaled[0.02], Automatic}], 10->
                  GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF2WlUFGcWBuBSiZJIEpXEGEBpUCPirhjQILyIQhQJW7MvFg00W9OAuERE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                    "], {
                    {GrayLevel[0.5], EdgeForm[None], 
                    GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ
2qYUCakkqbSrJBXatNLyfM7n/eM55zkzc2fud+be9/dW7t6/Tb8cEX/DAZBg
P/wDGi8KxRg8EIrhhcKLUvL6worDC0ORgph/4b+CtRorwlzRVLAXFMdLaE+8
FByKHwYHh8dK4gdB8YKcMl4o+awSUBovAyXDlMUPUUyYcnj55L2bwkB8EDQL
n1EBPxxK42XgSPwoKBumEn609lRuUB0/TjmGqYEfDxXw8lAVr5a8VjHH4JWh
XPgbSunbknNXTE28lnLQ2VAfP0k54BWhLn4CHBHOsaI8OVfF1MbrJK9VTAP8
ZKgUjqmHn5i8t3I4Fq+SnKvWnIKfqm9UrtAYP00541XgDPxMqIpXg7Pxc6B6
eOwsvHnyXB24DG8LdXUWNMNPT95LMefi5+kOw7TAz4caeC24GL8Eaof3aIdf
rjsIj7XBL00+SzEt8Qv0BmFa4a2hZjimPd4B6oXpiHfSneCN4Fr8Omisu4Ce
eC/dSXhNZ/wKqB+mC95VbxTmSrwbNAjTHb9KbxC+04Z4o+S71VgP/Orks3TH
TfCmyXfVEPri1yTnphx6432S55RjP7y/3ijM9fgN0CTMAPzG5Fr/C3bRhLuz
e1o1P5i5IXoTvS0Mw2/WG+ttYBw+Xm+kt4UR+K16Y709jMJHqwbCMcPxW5LX
amwMPjZ5L+0xAb8dWuCt4W78HrgwPDYRn6QaCJ9xGz4y+ayWMAW/U28cZip+
F7QKMw2fnry3vukmfGjyt+mMGfi9cJFqC2bh96vGwszGH4A2eFt4FH8M2oV5
HH9CNajahrn4I8lrFTMHf1A1Gf6GyfgdybkrZh7+JLTHO8EifDF0xjvAAvxp
6BjOcSZ+X3KuipmPP5W8VjFL8KWqyXDMQvyZ5L01tgxfDl3CPIs/p5oN8zz+
Alyp2ofN+HvqAbwbvIS/DN3DrMRfUU2HeRVfBT3CrMZfU02H99iCvw/9wmNr
8NehJ94L1uNvQG+8D7yFvw19w2vW4uuS12psI/6OeiIcswF/MzlWY5vwd5O/
RT1/od46+W+JcvgA/1A9o9qA7/EfVCP4INiOfwOD1UvwMf6JeirMp/hnMCDM
5/gX6rEw2/AvYWD4jK34R8l7aewr/Ovks3TnL+Irku96COzEv0vOTTnswL9N
nlOOu/DdMCzMHvxH9WyYn/CfYXiYX/Bf1ZNh9uK/wYgwv+N/qKfxkbAf/wdG
4aNBP9oJxoTH/mXuv+S5SVCauTIwWb0KfzO3L3kvxWTmCsHYMAfghWEcfjuU
xA+CieE9yuKHwB3hsVL4wdlnKaYIXhTGhymGHwgTwjHl8PIwJcyh+GFwJ34v
1MCPh5n43XAsXgXuCa+pgB8OU8NUxI+Au8IciR8F08JUwo+G6eEciuMlsr9N
Y8fglbPPmg318ZPgAXwGVMePy85NOVTFq2XPKceaeC24L0xtvA7MClMXPwHu
D1MPPzH7LJ3RAD8Z5qjW1AN4f1iBz1d/4BfBU/iD0BBvBA+FaYyfBg+HaYI3
hblhmuGnwyPhPS7GL4EF4bEz8DPhUfwxOBs/Bx7Hn4AW+PkwL7zmLLx59lqN
tcQvgCfDMefi52XHaqwV3jr7W/RN1+M3wEvhHNrgl8LT+LPQE+8Fz+FL1YN4
N1iGL4R2+OXwTJj2eAdYFKYj3gkWh+mMXwFLwmdchrfN3ktjXfCu2WfpDU7B
T82+6+XQA786Ozfl0B2/KntOOfbG+8DzYfri18ALYa7Fr8t+W/3mPETvPZz8
26Q7GMDcjfAyvgPW4uvgW/wVuAkfCq/iq2A4fgusDo8Nw2/OnnsNbsNHwhp8
JQzGh2TvpZgR+K3Za7VmFD4aXsfXqUfUG7A+/DfnT3L9K/lvxxswWb0PG8JM
Ue/Cm+GYifik7LUam6rehLfwt9XT6mXYiH+gmlGtwIf4e7of1TJsCa+Zod6D
d8LMVK/BpjCz1FvwbpjZ6l3YHP6mMfhYWBsem6O3zT5LOU3Dp2fn9r5qXLWd
nZtymKveyZ5TjvNU67A1zHz1JnwUZoFqGT4OnzkOH599txpbqNqFT8IsUq3C
p/gXqiHVDmzDv1OOyg2+x79UzahW4Kvw2EbdDfwQjnkRX5G9VmObdDewC/9c
Pabeyj7ra9WQage2h2tuvd4OdobPWKnayV6rsQ166+zcFLNatQTfhFmjWsqu
ZdX0QHxQdi0qh826e9iN79UZ2ht+w/eoJlQL8CP+i3JWrvAr/pPuUHcHP4fv
bIl6Fz4Lx2zVW2Sv1Z7blRv8Hj5zi94y+yztuU13lZ2L1uxQ78EfUXCG9s7O
5U/due46+391nbkMX559t4rZqbvJXqv/4ffoW2Af/j8egu84
                    
                    "]]]}, {{}, {}, {
                    {Hue[0.8135838915291268, 1, 1], LineBox[{1, 2}]}, 
                    {Hue[0.7439114747630289, 1, 1], LineBox[{2, 3}]}, 
                    {Hue[0.6845940903831713, 1, 1], LineBox[{3, 4}]}, 
                    {Hue[0.5899282786123202, 1, 1], LineBox[{4, 5}]}, 
                    {Hue[0.5537484979137711, 1, 1], LineBox[{5, 6}]}, 
                    {Hue[0.38417448662969855`, 1, 1], LineBox[{6, 7}]}, 
                    {Hue[0.34907528892008843`, 1, 1], LineBox[{7, 8}]}, 
                    {Hue[0.33302783616838333`, 1, 1], LineBox[{8, 9}]}, 
                    {Hue[0.23767149843348115`, 1, 1], LineBox[{9, 10}]}, 
                    {Hue[0.14225313723343055`, 1, 1], LineBox[{10, 11}]}, 
                    {Hue[0.043298420103147786`, 1, 1], LineBox[{11, 12}]}, 
                    {Hue[0.907956457126252, 1, 1], LineBox[{12, 13}]}, 
                    {Hue[0.8863378129830908, 1, 1], LineBox[{13, 14}]}, 
                    {Hue[0.7324579169392005, 1, 1], LineBox[{14, 15}]}, 
                    {Hue[0.6938488004081815, 1, 1], LineBox[{15, 16}]}, 
                    {Hue[0.39015314796355327`, 1, 1], LineBox[{16, 17}]}, 
                    {Hue[0.47524975557936294`, 1, 1], LineBox[{17, 18}]}, 
                    {Hue[0.23061422533859277`, 1, 1], LineBox[{18, 19}]}, 
                    {Hue[0.18759108425177645`, 1, 1], LineBox[{19, 20}]}, 
                    {Hue[0.10579347430510228`, 1, 1], LineBox[{20, 21}]}, 
                    {Hue[0.9460988040503769, 1, 1], LineBox[{21, 22}]}, 
                    {Hue[0.883853234242489, 1, 1], LineBox[{22, 23}]}, 
                    {Hue[0.7416327814295387, 1, 1], LineBox[{23, 24}]}, 
                    {Hue[0.7749364531338783, 1, 1], LineBox[{24, 25}]}, 
                    {Hue[0.6422587050581101, 1, 1], LineBox[{25, 26}]}, 
                    {Hue[0.5160848082862535, 1, 1], LineBox[{26, 27}]}, 
                    {Hue[0.41998315715370105`, 1, 1], LineBox[{27, 28}]}, 
                    {Hue[0.3677902653730239, 1, 1], LineBox[{28, 29}]}, 
                    {Hue[0.2596215547704807, 1, 1], LineBox[{29, 30}]}, 
                    {Hue[0.3028772709352434, 1, 1], LineBox[{30, 31}]}, 
                    {Hue[0.041896744764891025`, 1, 1], LineBox[{31, 32}]}, 
                    {Hue[0.8588600627762908, 1, 1], LineBox[{32, 33}]}, 
                    {Hue[0.7343447439619594, 1, 1], LineBox[{33, 34}]}, 
                    {Hue[0.6365872375656859, 1, 1], LineBox[{34, 35}]}, 
                    {Hue[0.47100754517141513`, 1, 1], LineBox[{35, 36}]}, 
                    {Hue[0.2813280179514035, 1, 1], LineBox[{36, 37}]}, 
                    {Hue[0.9735888980687284, 1, 1], LineBox[{37, 38}]}, 
                    {Hue[0.5552827557441284, 1, 1], LineBox[{38, 39}]}, 
                    {Hue[0.9381015599604834, 1, 1], LineBox[{39, 40}]}, 
                    {Hue[0.6549908474106629, 1, 1], LineBox[{40, 41}]}, 
                    {Hue[0.6518281404891956, 1, 1], LineBox[{41, 42}]}, 
                    {Hue[0.5400340083574924, 1, 1], LineBox[{42, 43}]}, 
                    {Hue[0.506450929345753, 1, 1], LineBox[{43, 44}]}, 
                    {Hue[0.5285733574892757, 1, 1], LineBox[{44, 45}]}, 
                    {Hue[0.5921813181707918, 1, 1], LineBox[{45, 46}]}, 
                    {Hue[0.9333562836332183, 1, 1], LineBox[{46, 47}]}, 
                    {Hue[0.8853458163957009, 1, 1], LineBox[{47, 48}]}, 
                    {Hue[0.853662012170136, 1, 1], LineBox[{48, 49}]}, 
                    {Hue[0.8968065708923977, 1, 1], LineBox[{49, 50}]}, 
                    {Hue[0.9826277227569763, 1, 1], LineBox[{50, 51}]}, 
                    {Hue[0.05395650772204826, 1, 1], LineBox[{51, 52}]}, 
                    {Hue[0.1370908908529463, 1, 1], LineBox[{52, 53}]}, 
                    {Hue[0.18163710723817444`, 1, 1], LineBox[{53, 54}]}, 
                    {Hue[0.24445818798964217`, 1, 1], LineBox[{54, 55}]}, 
                    {Hue[0.30857439043837853`, 1, 1], LineBox[{55, 56}]}, 
                    {Hue[0.3606133290881668, 1, 1], LineBox[{56, 57}]}, 
                    {Hue[0.420044573230797, 1, 1], LineBox[{57, 58}]}, 
                    {Hue[0.48419173721395464`, 1, 1], LineBox[{58, 59}]}, 
                    {Hue[0.5361081510744629, 1, 1], LineBox[{59, 60}]}, 
                    {Hue[0.5980659943808304, 1, 1], LineBox[{60, 61}]}, 
                    {Hue[0.6600463844653425, 1, 1], LineBox[{61, 62}]}, 
                    {Hue[0.7235566724925602, 1, 1], LineBox[{62, 63}]}, 
                    {Hue[0.7878680606789363, 1, 1], LineBox[{63, 64}]}, 
                    {Hue[0.8513131084760756, 1, 1], LineBox[{64, 65}]}, 
                    {Hue[0.9141237531356108, 1, 1], LineBox[{65, 66}]}, 
                    {Hue[0.9796550132334485, 1, 1], LineBox[{66, 67}]}, 
                    {Hue[0.04554279610193454, 1, 1], LineBox[{67, 68}]}, 
                    {Hue[0.11187390374902148`, 1, 1], LineBox[{68, 69}]}, 
                    {Hue[0.17762159787379167`, 1, 1], LineBox[{69, 70}]}, 
                    {Hue[0.24427137933553897`, 1, 1], LineBox[{70, 71}]}, 
                    {Hue[0.3117162635752856, 1, 1], LineBox[{71, 72}]}, 
                    {Hue[0.37969314417642314`, 1, 1], LineBox[{72, 73}]}, 
                    {Hue[0.44815967961866965`, 1, 1], LineBox[{73, 74}]}, 
                    {Hue[0.5171221163629012, 1, 1], LineBox[{74, 75}]}, 
                    {Hue[0.5862931145848354, 1, 1], LineBox[{75, 76}]}, 
                    {Hue[0.6560422323667225, 1, 1], LineBox[{76, 77}]}, 
                    {Hue[0.7265285874458343, 1, 1], LineBox[{77, 78}]}, 
                    {Hue[0.797530226193851, 1, 1], LineBox[{78, 79}]}, 
                    {Hue[0.8690849455760121, 1, 1], LineBox[{79, 80}]}, 
                    {Hue[0.9409585144911263, 1, 1], LineBox[{80, 81}]}, 
                    {Hue[0.013394114338599833`, 1, 1], LineBox[{81, 82}]}, 
                    {Hue[0.08635321152175987, 1, 1], LineBox[{82, 83}]}, 
                    {Hue[0.15990432497714052`, 1, 1], LineBox[{83, 84}]}, 
                    {Hue[0.2339381258426344, 1, 1], LineBox[{84, 85}]}, 
                    {Hue[0.30843325664141336`, 1, 1], LineBox[{85, 86}]}, 
                    {Hue[0.38345074264343776`, 1, 1], LineBox[{86, 87}]}, 
                    {Hue[0.4589748378028819, 1, 1], LineBox[{87, 88}]}, 
                    {Hue[0.5350070831107332, 1, 1], LineBox[{88, 89}]}, 
                    {Hue[0.6115514859638331, 1, 1], LineBox[{89, 90}]}, 
                    {Hue[0.688588607439024, 1, 1], LineBox[{90, 91}]}, 
                    {Hue[0.7660936824814056, 1, 1], LineBox[{91, 92}]}, 
                    {Hue[0.8440844237695915, 1, 1], LineBox[{92, 93}]}, 
                    {Hue[0.9225589053693233, 1, 1], LineBox[{93, 94}]}, 
                    {Hue[0.0015271511254317436`, 1, 1], LineBox[{94, 95}]}, 
                    {Hue[0.08096308242320557, 1, 1], LineBox[{95, 96}]}, 
                    {Hue[0.16084335970662272`, 1, 1], LineBox[{96, 97}]}, 
                    {Hue[0.241147292875211, 1, 1], LineBox[{97, 98}]}, 
                    {Hue[0.32189121078452165`, 1, 1], LineBox[{98, 99}]}, 
                    {Hue[0.40307511451608796`, 1, 1], LineBox[{99, 100}]}, 
                    {Hue[0.48468106235468705`, 1, 1], LineBox[{100, 101}]}, 
                    {Hue[0.5666666550014439, 1, 1], LineBox[{101, 102}]}, 
                    {Hue[0.648984876342054, 1, 1], LineBox[{102, 103}]}, 
                    {Hue[0.7316139984139767, 1, 1], LineBox[{103, 104}]}, 
                    {Hue[0.8145654914194567, 1, 1], LineBox[{104, 105}]}, 
                    {Hue[0.8978277043733119, 1, 1], LineBox[{105, 106}]}, 
                    {Hue[0.9813450138645246, 1, 1], LineBox[{106, 107}]}, 
                    {Hue[0.06501907948890219, 1, 1], LineBox[{107, 108}]}, 
                    {Hue[0.14876757678712846`, 1, 1], LineBox[{108, 109}]}, 
                    {Hue[0.23256003250733165`, 1, 1], LineBox[{109, 110}]}, 
                    {Hue[0.31639830470966585`, 1, 1], LineBox[{110, 111}]}, 
                    {Hue[0.40023530109883876`, 1, 1], LineBox[{111, 112}]}, 
                    {Hue[0.48393024058144063`, 1, 1], LineBox[{112, 113}]}, 
                    {Hue[0.5672876532384993, 1, 1], LineBox[{113, 114}]}, 
                    {Hue[0.6501592758058539, 1, 1], LineBox[{114, 115}]}, 
                    {Hue[0.7324987736444111, 1, 1], LineBox[{115, 116}]}, 
                    {Hue[0.8142983145499002, 1, 1], LineBox[{116, 117}]}, 
                    {Hue[0.8954590993731899, 1, 1], LineBox[{117, 118}]}, 
                    {Hue[0.975722643047566, 1, 1], LineBox[{118, 119}]}, 
                    {Hue[0.05474766844007228, 1, 1], LineBox[{119, 120}]}, 
                    {Hue[0.13227308453669961`, 1, 1], LineBox[{120, 121}]}, 
                    {Hue[0.20821636750529138`, 1, 1], LineBox[{121, 122}]}, 
                    {Hue[0.2826194654284094, 1, 1], LineBox[{122, 123}]}, 
                    {Hue[0.3554752741691703, 1, 1], LineBox[{123, 124}]}, 
                    {Hue[0.4265889065828907, 1, 1], LineBox[{124, 125}]}, 
                    {Hue[0.4955650664400778, 1, 1], LineBox[{125, 126}]}, 
                    {Hue[0.5619584530314572, 1, 1], LineBox[{126, 127}]}, 
                    {Hue[0.6254635887802759, 1, 1], LineBox[{127, 128}]}, 
                    {Hue[0.6860331182124972, 1, 1], LineBox[{128, 129}]}, 
                    {Hue[0.7438757636849702, 1, 1], LineBox[{129, 130}]}, 
                    {Hue[0.799376342278605, 1, 1], LineBox[{130, 131}]}, 
                    {Hue[0.8530037452326122, 1, 1], LineBox[{131, 132}]}, 
                    {Hue[0.9052386573064211, 1, 1], LineBox[{132, 133}]}, 
                    {Hue[0.9565330054338083, 1, 1], LineBox[{133, 134}]}, 
                    {Hue[0.007289019921117269, 1, 1], LineBox[{134, 135}]}, 
                    {Hue[0.05785436886123326, 1, 1], LineBox[{135, 136}]}, 
                    {Hue[0.10852361694626046`, 1, 1], LineBox[{136, 137}]}, 
                    {Hue[0.1595424939905707, 1, 1], LineBox[{137, 138}]}, 
                    {Hue[0.21111490902177088`, 1, 1], LineBox[{138, 139}]}, 
                    {Hue[0.2634076300324872, 1, 1], LineBox[{139, 140}]}, 
                    {Hue[0.31655628975593003`, 1, 1], LineBox[{140, 141}]}, 
                    {Hue[0.37066907391576337`, 1, 1], LineBox[{141, 142}]}, 
                    {Hue[0.42583171930078956`, 1, 1], LineBox[{142, 143}]}, 
                    {Hue[0.4821112905542982, 1, 1], LineBox[{143, 144}]}, 
                    {Hue[0.5395581595296779, 1, 1], LineBox[{144, 145}]}, 
                    {Hue[0.5982089184764874, 1, 1], LineBox[{145, 146}]}, 
                    {Hue[0.6580902546204306, 1, 1], LineBox[{146, 147}]}, 
                    {Hue[0.7192184186201758, 1, 1], LineBox[{147, 148}]}, 
                    {Hue[0.7816029392773315, 1, 1], LineBox[{148, 149}]}, 
                    {Hue[0.845246320651291, 1, 1], LineBox[{149, 150}]}, 
                    {Hue[0.9101471280449227, 1, 1], LineBox[{150, 151}]}, 
                    {Hue[0.9762994778057256, 1, 1], LineBox[{151, 152}]}, 
                    {Hue[0.043693647064499526`, 1, 1], LineBox[{152, 153}]}, 
                    {Hue[0.1123192138725254, 1, 1], LineBox[{153, 154}]}, 
                    {Hue[0.18216349795160328`, 1, 1], LineBox[{154, 155}]}, 
                    {Hue[0.2532094583491348, 1, 1], LineBox[{155, 156}]}, 
                    {Hue[0.32544339799515104`, 1, 1], LineBox[{156, 157}]}, 
                    {Hue[0.39884894807087073`, 1, 1], LineBox[{157, 158}]}, 
                    {Hue[0.4734080173466877, 1, 1], LineBox[{158, 159}]}, 
                    {Hue[0.5491064682724095, 1, 1], LineBox[{159, 160}]}, 
                    {Hue[0.6259260435306573, 1, 1], LineBox[{160, 161}]}, 
                    {Hue[0.7038469620065598, 1, 1], LineBox[{161, 162}]}, 
                    {Hue[0.7828539447311839, 1, 1], LineBox[{162, 163}]}, 
                    {Hue[0.86293510134423, 1, 1], LineBox[{163, 164}]}, 
                    {Hue[0.9440694759822695, 1, 1], LineBox[{164, 165}]}, 
                    {Hue[0.026237977530103335`, 1, 1], LineBox[{165, 166}]}, 
                    {Hue[0.10943912867403406`, 1, 1], LineBox[{166, 167}]}, 
                    {Hue[0.1936492364678344, 1, 1], LineBox[{167, 168}]}, 
                    {Hue[0.2788390158538952, 1, 1], LineBox[{168, 169}]}, 
                    {Hue[0.36501718255201326`, 1, 1], LineBox[{169, 170}]}, 
                    {Hue[0.4521779601579034, 1, 1], LineBox[{170, 171}]}, 
                    {Hue[0.5402709397872767, 1, 1], LineBox[{171, 172}]}, 
                    {Hue[0.6292987165623611, 1, 1], LineBox[{172, 173}]}, 
                    {Hue[0.7192852763459747, 1, 1], LineBox[{173, 174}]}, 
                    {Hue[0.8101807715827263, 1, 1], LineBox[{174, 175}]}, 
                    {Hue[0.9019724559267499, 1, 1], LineBox[{175, 176}]}, 
                    {Hue[0.99470033906565, 1, 1], LineBox[{176, 177}]}, 
                    {Hue[0.08832783678624961, 1, 1], LineBox[{177, 178}]}, 
                    {Hue[0.18277987722455066`, 1, 1], LineBox[{178, 179}]}, 
                    {Hue[0.2780640171066228, 1, 1], LineBox[{179, 180}]}, 
                    {Hue[0.3742762888141277, 1, 1], LineBox[{180, 181}]}, 
                    {Hue[0.4714059652914306, 1, 1], LineBox[{181, 182}]}, 
                    {Hue[0.5692514338009139, 1, 1], LineBox[{182, 183}]}, 
                    {Hue[0.6677981386855287, 1, 1], LineBox[{183, 184}]}, 
                    {Hue[0.7673407367346702, 1, 1], LineBox[{184, 185}]}, 
                    {Hue[0.8678504620247419, 1, 1], LineBox[{185, 186}]}, 
                    {Hue[0.9686973038820387, 1, 1], LineBox[{186, 187}]}, 
                    {Hue[0.06963060970896097, 1, 1], LineBox[{187, 188}]}, 
                    {Hue[0.17180207357002875`, 1, 1], LineBox[{188, 189}]}, 
                    {Hue[0.27636830103356114`, 1, 1], LineBox[{189, 190}]}, 
                    {Hue[0.38209981443647867`, 1, 1], LineBox[{190, 191}]}, 
                    {Hue[0.48684916138316015`, 1, 1], LineBox[{191, 192}]}, 
                    {Hue[0.591638674225299, 1, 1], LineBox[{192, 193}]}, 
                    {Hue[0.6996440399500621, 1, 1], LineBox[{193, 194}]}, 
                    {Hue[0.8106408769056764, 1, 1], LineBox[{194, 195}]}, 
                    {Hue[0.9203208846246161, 1, 1], LineBox[{195, 196}]}, 
                    {Hue[0.02405236895185515, 1, 1], LineBox[{196, 197}]}, 
                    {Hue[0.1228424398166391, 1, 1], LineBox[{197, 198}]}, 
                    {Hue[0.22619536848471866`, 1, 1], LineBox[{198, 199}]}, 
                    {Hue[0.3370375970414235, 1, 1], LineBox[{199, 200}]}, 
                    {Hue[0.4399614845447659, 1, 1], LineBox[{200, 201}]}, 
                    {Hue[0.5268701196885841, 1, 1], LineBox[{201, 202}]}, 
                    {Hue[0.6371482431217275, 1, 1], LineBox[{202, 203}]}, 
                    {Hue[0.7836147903328474, 1, 1], LineBox[{203, 204}]}, 
                    {Hue[0.9442860013131829, 1, 1], LineBox[{204, 205}]}, 
                    {Hue[0.09297398599726875, 1, 1], LineBox[{205, 206}]}, 
                    {Hue[0.17887958096169843`, 1, 1], LineBox[{206, 207}]}, 
                    {Hue[0.19638153085605062`, 1, 1], LineBox[{207, 208}]}, 
                    {Hue[0.30819964433545805`, 1, 1], LineBox[{208, 209}]}, 
                    {Hue[0.4887367267379165, 1, 1], LineBox[{209, 210}]}, 
                    {Hue[0.7100622885787811, 1, 1], LineBox[{210, 211}]}, 
                    {Hue[0.9616419546043693, 1, 1], LineBox[{211, 212}]}, 
                    {Hue[0.23801211969541905`, 1, 1], LineBox[{212, 213}]}, 
                    {Hue[0.5347134004266552, 1, 1], LineBox[{213, 214}]}, 
                    {Hue[0.851720999705813, 1, 1], LineBox[{214, 215}]}, 
                    {Hue[0.19772709565797011`, 1, 1], LineBox[{215, 216}]}, 
                    {Hue[0.5542643360085939, 1, 1], LineBox[{216, 217}]}, 
                    {Hue[0.8726282053207132, 1, 1], LineBox[{217, 218}]}, 
                    {Hue[0.1490042529253488, 1, 1], LineBox[{218, 219}]}, 
                    {Hue[0.37880560395471286`, 1, 1], LineBox[{219, 220}]}, 
                    {Hue[0.5180102868178059, 1, 1], LineBox[{220, 221}]}, 
                    {Hue[0.6661968199894867, 1, 1], LineBox[{221, 222}]}, 
                    {Hue[0.8912713919398864, 1, 1], LineBox[{222, 223}]}, 
                    {Hue[0.14372955017319863`, 1, 1], LineBox[{223, 224}]}, 
                    {Hue[0.40895675323907044`, 1, 1], LineBox[{224, 225}]}, 
                    {Hue[0.687498852931313, 1, 1], LineBox[{225, 226}]}, 
                    {Hue[0.9983927521379041, 1, 1], LineBox[{226, 227}]}, 
                    {Hue[0.41402290612424353`, 1, 1], LineBox[{227, 228}]}, 
                    {Hue[0.8107856520326657, 1, 1], LineBox[{228, 229}]}, 
                    {Hue[0.11575640376966859`, 1, 1], LineBox[{229, 230}]}, 
                    {Hue[0.39601159431580035`, 1, 1], LineBox[{230, 231}]}, 
                    {Hue[0.6681929871460615, 1, 1], LineBox[{231, 232}]}, 
                    {Hue[0.9369112472018684, 1, 1], LineBox[{232, 233}]}, 
                    {Hue[0.20387105287694343`, 1, 1], LineBox[{233, 234}]}, 
                    {Hue[0.46982427789568754`, 1, 1], LineBox[{234, 235}]}, 
                    {Hue[0.7351428404407656, 1, 1], LineBox[{235, 236}]}, 
                    {Hue[0.000045043010442031465`, 1, 1], 
                    LineBox[{236, 237}]}, 
                    {Hue[0.26472928293010783`, 1, 1], LineBox[{237, 238}]}, 
                    {Hue[0.529502741806376, 1, 1], LineBox[{238, 239}]}, 
                    {Hue[0.7949436009945829, 1, 1], LineBox[{239, 240}]}, 
                    {Hue[0.06210321930714255, 1, 1], LineBox[{240, 241}]}, 
                    {Hue[0.33266837165463214`, 1, 1], LineBox[{241, 242}]}, 
                    {Hue[0.6088044722098869, 1, 1], LineBox[{242, 243}]}, 
                    {Hue[0.892140756761359, 1, 1], LineBox[{243, 244}]}, 
                    {Hue[0.18175714747005245`, 1, 1], LineBox[{244, 245}]}, 
                    {Hue[0.47326275941150026`, 1, 1], LineBox[{245, 246}]}, 
                    {Hue[0.7616040472763749, 1, 1], LineBox[{246, 247}]}, 
                    {Hue[0.044437360962688716`, 1, 1], LineBox[{247, 248}]}, 
                    {Hue[0.32202075940832503`, 1, 1], LineBox[{248, 249}]}, 
                    {Hue[0.5955234481590492, 1, 1], LineBox[{249, 250}]}, 
                    {Hue[0.8660326380659066, 1, 1], LineBox[{250, 251}]}}}},
                    VertexColors->{
                    Hue[0.8135838915291268, 1, 1], 
                    Hue[0.7439114747630289, 1, 1], 
                    Hue[0.6845940903831713, 1, 1], 
                    Hue[0.5899282786123202, 1, 1], 
                    Hue[0.5537484979137711, 1, 1], 
                    Hue[0.38417448662969855`, 1, 1], 
                    Hue[0.34907528892008843`, 1, 1], 
                    Hue[0.33302783616838333`, 1, 1], 
                    Hue[0.23767149843348115`, 1, 1], 
                    Hue[0.14225313723343055`, 1, 1], 
                    Hue[0.043298420103147786`, 1, 1], 
                    Hue[0.907956457126252, 1, 1], 
                    Hue[0.8863378129830908, 1, 1], 
                    Hue[0.7324579169392005, 1, 1], 
                    Hue[0.6938488004081815, 1, 1], 
                    Hue[0.39015314796355327`, 1, 1], 
                    Hue[0.47524975557936294`, 1, 1], 
                    Hue[0.23061422533859277`, 1, 1], 
                    Hue[0.18759108425177645`, 1, 1], 
                    Hue[0.10579347430510228`, 1, 1], 
                    Hue[0.9460988040503769, 1, 1], 
                    Hue[0.883853234242489, 1, 1], 
                    Hue[0.7416327814295387, 1, 1], 
                    Hue[0.7749364531338783, 1, 1], 
                    Hue[0.6422587050581101, 1, 1], 
                    Hue[0.5160848082862535, 1, 1], 
                    Hue[0.41998315715370105`, 1, 1], 
                    Hue[0.3677902653730239, 1, 1], 
                    Hue[0.2596215547704807, 1, 1], 
                    Hue[0.3028772709352434, 1, 1], 
                    Hue[0.041896744764891025`, 1, 1], 
                    Hue[0.8588600627762908, 1, 1], 
                    Hue[0.7343447439619594, 1, 1], 
                    Hue[0.6365872375656859, 1, 1], 
                    Hue[0.47100754517141513`, 1, 1], 
                    Hue[0.2813280179514035, 1, 1], 
                    Hue[0.9735888980687284, 1, 1], 
                    Hue[0.5552827557441284, 1, 1], 
                    Hue[0.9381015599604834, 1, 1], 
                    Hue[0.6549908474106629, 1, 1], 
                    Hue[0.6518281404891956, 1, 1], 
                    Hue[0.5400340083574924, 1, 1], 
                    Hue[0.506450929345753, 1, 1], 
                    Hue[0.5285733574892757, 1, 1], 
                    Hue[0.5921813181707918, 1, 1], 
                    Hue[0.9333562836332183, 1, 1], 
                    Hue[0.8853458163957009, 1, 1], 
                    Hue[0.853662012170136, 1, 1], 
                    Hue[0.8968065708923977, 1, 1], 
                    Hue[0.9826277227569763, 1, 1], 
                    Hue[0.05395650772204826, 1, 1], 
                    Hue[0.1370908908529463, 1, 1], 
                    Hue[0.18163710723817444`, 1, 1], 
                    Hue[0.24445818798964217`, 1, 1], 
                    Hue[0.30857439043837853`, 1, 1], 
                    Hue[0.3606133290881668, 1, 1], 
                    Hue[0.420044573230797, 1, 1], 
                    Hue[0.48419173721395464`, 1, 1], 
                    Hue[0.5361081510744629, 1, 1], 
                    Hue[0.5980659943808304, 1, 1], 
                    Hue[0.6600463844653425, 1, 1], 
                    Hue[0.7235566724925602, 1, 1], 
                    Hue[0.7878680606789363, 1, 1], 
                    Hue[0.8513131084760756, 1, 1], 
                    Hue[0.9141237531356108, 1, 1], 
                    Hue[0.9796550132334485, 1, 1], 
                    Hue[0.04554279610193454, 1, 1], 
                    Hue[0.11187390374902148`, 1, 1], 
                    Hue[0.17762159787379167`, 1, 1], 
                    Hue[0.24427137933553897`, 1, 1], 
                    Hue[0.3117162635752856, 1, 1], 
                    Hue[0.37969314417642314`, 1, 1], 
                    Hue[0.44815967961866965`, 1, 1], 
                    Hue[0.5171221163629012, 1, 1], 
                    Hue[0.5862931145848354, 1, 1], 
                    Hue[0.6560422323667225, 1, 1], 
                    Hue[0.7265285874458343, 1, 1], 
                    Hue[0.797530226193851, 1, 1], 
                    Hue[0.8690849455760121, 1, 1], 
                    Hue[0.9409585144911263, 1, 1], 
                    Hue[0.013394114338599833`, 1, 1], 
                    Hue[0.08635321152175987, 1, 1], 
                    Hue[0.15990432497714052`, 1, 1], 
                    Hue[0.2339381258426344, 1, 1], 
                    Hue[0.30843325664141336`, 1, 1], 
                    Hue[0.38345074264343776`, 1, 1], 
                    Hue[0.4589748378028819, 1, 1], 
                    Hue[0.5350070831107332, 1, 1], 
                    Hue[0.6115514859638331, 1, 1], 
                    Hue[0.688588607439024, 1, 1], 
                    Hue[0.7660936824814056, 1, 1], 
                    Hue[0.8440844237695915, 1, 1], 
                    Hue[0.9225589053693233, 1, 1], 
                    Hue[0.0015271511254317436`, 1, 1], 
                    Hue[0.08096308242320557, 1, 1], 
                    Hue[0.16084335970662272`, 1, 1], 
                    Hue[0.241147292875211, 1, 1], 
                    Hue[0.32189121078452165`, 1, 1], 
                    Hue[0.40307511451608796`, 1, 1], 
                    Hue[0.48468106235468705`, 1, 1], 
                    Hue[0.5666666550014439, 1, 1], 
                    Hue[0.648984876342054, 1, 1], 
                    Hue[0.7316139984139767, 1, 1], 
                    Hue[0.8145654914194567, 1, 1], 
                    Hue[0.8978277043733119, 1, 1], 
                    Hue[0.9813450138645246, 1, 1], 
                    Hue[0.06501907948890219, 1, 1], 
                    Hue[0.14876757678712846`, 1, 1], 
                    Hue[0.23256003250733165`, 1, 1], 
                    Hue[0.31639830470966585`, 1, 1], 
                    Hue[0.40023530109883876`, 1, 1], 
                    Hue[0.48393024058144063`, 1, 1], 
                    Hue[0.5672876532384993, 1, 1], 
                    Hue[0.6501592758058539, 1, 1], 
                    Hue[0.7324987736444111, 1, 1], 
                    Hue[0.8142983145499002, 1, 1], 
                    Hue[0.8954590993731899, 1, 1], 
                    Hue[0.975722643047566, 1, 1], 
                    Hue[0.05474766844007228, 1, 1], 
                    Hue[0.13227308453669961`, 1, 1], 
                    Hue[0.20821636750529138`, 1, 1], 
                    Hue[0.2826194654284094, 1, 1], 
                    Hue[0.3554752741691703, 1, 1], 
                    Hue[0.4265889065828907, 1, 1], 
                    Hue[0.4955650664400778, 1, 1], 
                    Hue[0.5619584530314572, 1, 1], 
                    Hue[0.6254635887802759, 1, 1], 
                    Hue[0.6860331182124972, 1, 1], 
                    Hue[0.7438757636849702, 1, 1], 
                    Hue[0.799376342278605, 1, 1], 
                    Hue[0.8530037452326122, 1, 1], 
                    Hue[0.9052386573064211, 1, 1], 
                    Hue[0.9565330054338083, 1, 1], 
                    Hue[0.007289019921117269, 1, 1], 
                    Hue[0.05785436886123326, 1, 1], 
                    Hue[0.10852361694626046`, 1, 1], 
                    Hue[0.1595424939905707, 1, 1], 
                    Hue[0.21111490902177088`, 1, 1], 
                    Hue[0.2634076300324872, 1, 1], 
                    Hue[0.31655628975593003`, 1, 1], 
                    Hue[0.37066907391576337`, 1, 1], 
                    Hue[0.42583171930078956`, 1, 1], 
                    Hue[0.4821112905542982, 1, 1], 
                    Hue[0.5395581595296779, 1, 1], 
                    Hue[0.5982089184764874, 1, 1], 
                    Hue[0.6580902546204306, 1, 1], 
                    Hue[0.7192184186201758, 1, 1], 
                    Hue[0.7816029392773315, 1, 1], 
                    Hue[0.845246320651291, 1, 1], 
                    Hue[0.9101471280449227, 1, 1], 
                    Hue[0.9762994778057256, 1, 1], 
                    Hue[0.043693647064499526`, 1, 1], 
                    Hue[0.1123192138725254, 1, 1], 
                    Hue[0.18216349795160328`, 1, 1], 
                    Hue[0.2532094583491348, 1, 1], 
                    Hue[0.32544339799515104`, 1, 1], 
                    Hue[0.39884894807087073`, 1, 1], 
                    Hue[0.4734080173466877, 1, 1], 
                    Hue[0.5491064682724095, 1, 1], 
                    Hue[0.6259260435306573, 1, 1], 
                    Hue[0.7038469620065598, 1, 1], 
                    Hue[0.7828539447311839, 1, 1], 
                    Hue[0.86293510134423, 1, 1], 
                    Hue[0.9440694759822695, 1, 1], 
                    Hue[0.026237977530103335`, 1, 1], 
                    Hue[0.10943912867403406`, 1, 1], 
                    Hue[0.1936492364678344, 1, 1], 
                    Hue[0.2788390158538952, 1, 1], 
                    Hue[0.36501718255201326`, 1, 1], 
                    Hue[0.4521779601579034, 1, 1], 
                    Hue[0.5402709397872767, 1, 1], 
                    Hue[0.6292987165623611, 1, 1], 
                    Hue[0.7192852763459747, 1, 1], 
                    Hue[0.8101807715827263, 1, 1], 
                    Hue[0.9019724559267499, 1, 1], 
                    Hue[0.99470033906565, 1, 1], 
                    Hue[0.08832783678624961, 1, 1], 
                    Hue[0.18277987722455066`, 1, 1], 
                    Hue[0.2780640171066228, 1, 1], 
                    Hue[0.3742762888141277, 1, 1], 
                    Hue[0.4714059652914306, 1, 1], 
                    Hue[0.5692514338009139, 1, 1], 
                    Hue[0.6677981386855287, 1, 1], 
                    Hue[0.7673407367346702, 1, 1], 
                    Hue[0.8678504620247419, 1, 1], 
                    Hue[0.9686973038820387, 1, 1], 
                    Hue[0.06963060970896097, 1, 1], 
                    Hue[0.17180207357002875`, 1, 1], 
                    Hue[0.27636830103356114`, 1, 1], 
                    Hue[0.38209981443647867`, 1, 1], 
                    Hue[0.48684916138316015`, 1, 1], 
                    Hue[0.591638674225299, 1, 1], 
                    Hue[0.6996440399500621, 1, 1], 
                    Hue[0.8106408769056764, 1, 1], 
                    Hue[0.9203208846246161, 1, 1], 
                    Hue[0.02405236895185515, 1, 1], 
                    Hue[0.1228424398166391, 1, 1], 
                    Hue[0.22619536848471866`, 1, 1], 
                    Hue[0.3370375970414235, 1, 1], 
                    Hue[0.4399614845447659, 1, 1], 
                    Hue[0.5268701196885841, 1, 1], 
                    Hue[0.6371482431217275, 1, 1], 
                    Hue[0.7836147903328474, 1, 1], 
                    Hue[0.9442860013131829, 1, 1], 
                    Hue[0.09297398599726875, 1, 1], 
                    Hue[0.17887958096169843`, 1, 1], 
                    Hue[0.19638153085605062`, 1, 1], 
                    Hue[0.30819964433545805`, 1, 1], 
                    Hue[0.4887367267379165, 1, 1], 
                    Hue[0.7100622885787811, 1, 1], 
                    Hue[0.9616419546043693, 1, 1], 
                    Hue[0.23801211969541905`, 1, 1], 
                    Hue[0.5347134004266552, 1, 1], 
                    Hue[0.851720999705813, 1, 1], 
                    Hue[0.19772709565797011`, 1, 1], 
                    Hue[0.5542643360085939, 1, 1], 
                    Hue[0.8726282053207132, 1, 1], 
                    Hue[0.1490042529253488, 1, 1], 
                    Hue[0.37880560395471286`, 1, 1], 
                    Hue[0.5180102868178059, 1, 1], 
                    Hue[0.6661968199894867, 1, 1], 
                    Hue[0.8912713919398864, 1, 1], 
                    Hue[0.14372955017319863`, 1, 1], 
                    Hue[0.40895675323907044`, 1, 1], 
                    Hue[0.687498852931313, 1, 1], 
                    Hue[0.9983927521379041, 1, 1], 
                    Hue[0.41402290612424353`, 1, 1], 
                    Hue[0.8107856520326657, 1, 1], 
                    Hue[0.11575640376966859`, 1, 1], 
                    Hue[0.39601159431580035`, 1, 1], 
                    Hue[0.6681929871460615, 1, 1], 
                    Hue[0.9369112472018684, 1, 1], 
                    Hue[0.20387105287694343`, 1, 1], 
                    Hue[0.46982427789568754`, 1, 1], 
                    Hue[0.7351428404407656, 1, 1], 
                    Hue[0.000045043010442031465`, 1, 1], 
                    Hue[0.26472928293010783`, 1, 1], 
                    Hue[0.529502741806376, 1, 1], 
                    Hue[0.7949436009945829, 1, 1], 
                    Hue[0.06210321930714255, 1, 1], 
                    Hue[0.33266837165463214`, 1, 1], 
                    Hue[0.6088044722098869, 1, 1], 
                    Hue[0.892140756761359, 1, 1], 
                    Hue[0.18175714747005245`, 1, 1], 
                    Hue[0.47326275941150026`, 1, 1], 
                    Hue[0.7616040472763749, 1, 1], 
                    Hue[0.044437360962688716`, 1, 1], 
                    Hue[0.32202075940832503`, 1, 1], 
                    Hue[0.5955234481590492, 1, 1], 
                    Hue[0.8660326380659066, 1, 1], 
                    Hue[0.1343067124487573, 1, 1], 
                    Hue[0.8135838915291268, 1, 1], 
                    Hue[0.7439114747630289, 1, 1], 
                    Hue[0.7439114747630289, 1, 1], 
                    Hue[0.6845940903831713, 1, 1], 
                    Hue[0.6845940903831713, 1, 1], 
                    Hue[0.5899282786123202, 1, 1], 
                    Hue[0.5899282786123202, 1, 1], 
                    Hue[0.5537484979137711, 1, 1], 
                    Hue[0.5537484979137711, 1, 1], 
                    Hue[0.38417448662969855`, 1, 1], 
                    Hue[0.38417448662969855`, 1, 1], 
                    Hue[0.34907528892008843`, 1, 1], 
                    Hue[0.34907528892008843`, 1, 1], 
                    Hue[0.33302783616838333`, 1, 1], 
                    Hue[0.33302783616838333`, 1, 1], 
                    Hue[0.23767149843348115`, 1, 1], 
                    Hue[0.23767149843348115`, 1, 1], 
                    Hue[0.14225313723343055`, 1, 1], 
                    Hue[0.14225313723343055`, 1, 1], 
                    Hue[0.043298420103147786`, 1, 1], 
                    Hue[0.043298420103147786`, 1, 1], 
                    Hue[0.907956457126252, 1, 1], 
                    Hue[0.907956457126252, 1, 1], 
                    Hue[0.8863378129830908, 1, 1], 
                    Hue[0.8863378129830908, 1, 1], 
                    Hue[0.7324579169392005, 1, 1], 
                    Hue[0.7324579169392005, 1, 1], 
                    Hue[0.6938488004081815, 1, 1], 
                    Hue[0.6938488004081815, 1, 1], 
                    Hue[0.39015314796355327`, 1, 1], 
                    Hue[0.39015314796355327`, 1, 1], 
                    Hue[0.47524975557936294`, 1, 1], 
                    Hue[0.47524975557936294`, 1, 1], 
                    Hue[0.23061422533859277`, 1, 1], 
                    Hue[0.23061422533859277`, 1, 1], 
                    Hue[0.18759108425177645`, 1, 1], 
                    Hue[0.18759108425177645`, 1, 1], 
                    Hue[0.10579347430510228`, 1, 1], 
                    Hue[0.10579347430510228`, 1, 1], 
                    Hue[0.9460988040503769, 1, 1], 
                    Hue[0.9460988040503769, 1, 1], 
                    Hue[0.883853234242489, 1, 1], 
                    Hue[0.883853234242489, 1, 1], 
                    Hue[0.7416327814295387, 1, 1], 
                    Hue[0.7416327814295387, 1, 1], 
                    Hue[0.7749364531338783, 1, 1], 
                    Hue[0.7749364531338783, 1, 1], 
                    Hue[0.6422587050581101, 1, 1], 
                    Hue[0.6422587050581101, 1, 1], 
                    Hue[0.5160848082862535, 1, 1], 
                    Hue[0.5160848082862535, 1, 1], 
                    Hue[0.41998315715370105`, 1, 1], 
                    Hue[0.41998315715370105`, 1, 1], 
                    Hue[0.3677902653730239, 1, 1], 
                    Hue[0.3677902653730239, 1, 1], 
                    Hue[0.2596215547704807, 1, 1], 
                    Hue[0.2596215547704807, 1, 1], 
                    Hue[0.3028772709352434, 1, 1], 
                    Hue[0.3028772709352434, 1, 1], 
                    Hue[0.041896744764891025`, 1, 1], 
                    Hue[0.041896744764891025`, 1, 1], 
                    Hue[0.8588600627762908, 1, 1], 
                    Hue[0.8588600627762908, 1, 1], 
                    Hue[0.7343447439619594, 1, 1], 
                    Hue[0.7343447439619594, 1, 1], 
                    Hue[0.6365872375656859, 1, 1], 
                    Hue[0.6365872375656859, 1, 1], 
                    Hue[0.47100754517141513`, 1, 1], 
                    Hue[0.47100754517141513`, 1, 1], 
                    Hue[0.2813280179514035, 1, 1], 
                    Hue[0.2813280179514035, 1, 1], 
                    Hue[0.9735888980687284, 1, 1], 
                    Hue[0.9735888980687284, 1, 1], 
                    Hue[0.5552827557441284, 1, 1], 
                    Hue[0.5552827557441284, 1, 1], 
                    Hue[0.9381015599604834, 1, 1], 
                    Hue[0.9381015599604834, 1, 1], 
                    Hue[0.6549908474106629, 1, 1], 
                    Hue[0.6549908474106629, 1, 1], 
                    Hue[0.6518281404891956, 1, 1], 
                    Hue[0.6518281404891956, 1, 1], 
                    Hue[0.5400340083574924, 1, 1], 
                    Hue[0.5400340083574924, 1, 1], 
                    Hue[0.506450929345753, 1, 1], 
                    Hue[0.506450929345753, 1, 1], 
                    Hue[0.5285733574892757, 1, 1], 
                    Hue[0.5285733574892757, 1, 1], 
                    Hue[0.5921813181707918, 1, 1], 
                    Hue[0.5921813181707918, 1, 1], 
                    Hue[0.9333562836332183, 1, 1], 
                    Hue[0.9333562836332183, 1, 1], 
                    Hue[0.8853458163957009, 1, 1], 
                    Hue[0.8853458163957009, 1, 1], 
                    Hue[0.853662012170136, 1, 1], 
                    Hue[0.853662012170136, 1, 1], 
                    Hue[0.8968065708923977, 1, 1], 
                    Hue[0.8968065708923977, 1, 1], 
                    Hue[0.9826277227569763, 1, 1], 
                    Hue[0.9826277227569763, 1, 1], 
                    Hue[0.05395650772204826, 1, 1], 
                    Hue[0.05395650772204826, 1, 1], 
                    Hue[0.1370908908529463, 1, 1], 
                    Hue[0.1370908908529463, 1, 1], 
                    Hue[0.18163710723817444`, 1, 1], 
                    Hue[0.18163710723817444`, 1, 1], 
                    Hue[0.24445818798964217`, 1, 1], 
                    Hue[0.24445818798964217`, 1, 1], 
                    Hue[0.30857439043837853`, 1, 1], 
                    Hue[0.30857439043837853`, 1, 1], 
                    Hue[0.3606133290881668, 1, 1], 
                    Hue[0.3606133290881668, 1, 1], 
                    Hue[0.420044573230797, 1, 1], 
                    Hue[0.420044573230797, 1, 1], 
                    Hue[0.48419173721395464`, 1, 1], 
                    Hue[0.48419173721395464`, 1, 1], 
                    Hue[0.5361081510744629, 1, 1], 
                    Hue[0.5361081510744629, 1, 1], 
                    Hue[0.5980659943808304, 1, 1], 
                    Hue[0.5980659943808304, 1, 1], 
                    Hue[0.6600463844653425, 1, 1], 
                    Hue[0.6600463844653425, 1, 1], 
                    Hue[0.7235566724925602, 1, 1], 
                    Hue[0.7235566724925602, 1, 1], 
                    Hue[0.7878680606789363, 1, 1], 
                    Hue[0.7878680606789363, 1, 1], 
                    Hue[0.8513131084760756, 1, 1], 
                    Hue[0.8513131084760756, 1, 1], 
                    Hue[0.9141237531356108, 1, 1], 
                    Hue[0.9141237531356108, 1, 1], 
                    Hue[0.9796550132334485, 1, 1], 
                    Hue[0.9796550132334485, 1, 1], 
                    Hue[0.04554279610193454, 1, 1], 
                    Hue[0.04554279610193454, 1, 1], 
                    Hue[0.11187390374902148`, 1, 1], 
                    Hue[0.11187390374902148`, 1, 1], 
                    Hue[0.17762159787379167`, 1, 1], 
                    Hue[0.17762159787379167`, 1, 1], 
                    Hue[0.24427137933553897`, 1, 1], 
                    Hue[0.24427137933553897`, 1, 1], 
                    Hue[0.3117162635752856, 1, 1], 
                    Hue[0.3117162635752856, 1, 1], 
                    Hue[0.37969314417642314`, 1, 1], 
                    Hue[0.37969314417642314`, 1, 1], 
                    Hue[0.44815967961866965`, 1, 1], 
                    Hue[0.44815967961866965`, 1, 1], 
                    Hue[0.5171221163629012, 1, 1], 
                    Hue[0.5171221163629012, 1, 1], 
                    Hue[0.5862931145848354, 1, 1], 
                    Hue[0.5862931145848354, 1, 1], 
                    Hue[0.6560422323667225, 1, 1], 
                    Hue[0.6560422323667225, 1, 1], 
                    Hue[0.7265285874458343, 1, 1], 
                    Hue[0.7265285874458343, 1, 1], 
                    Hue[0.797530226193851, 1, 1], 
                    Hue[0.797530226193851, 1, 1], 
                    Hue[0.8690849455760121, 1, 1], 
                    Hue[0.8690849455760121, 1, 1], 
                    Hue[0.9409585144911263, 1, 1], 
                    Hue[0.9409585144911263, 1, 1], 
                    Hue[0.013394114338599833`, 1, 1], 
                    Hue[0.013394114338599833`, 1, 1], 
                    Hue[0.08635321152175987, 1, 1], 
                    Hue[0.08635321152175987, 1, 1], 
                    Hue[0.15990432497714052`, 1, 1], 
                    Hue[0.15990432497714052`, 1, 1], 
                    Hue[0.2339381258426344, 1, 1], 
                    Hue[0.2339381258426344, 1, 1], 
                    Hue[0.30843325664141336`, 1, 1], 
                    Hue[0.30843325664141336`, 1, 1], 
                    Hue[0.38345074264343776`, 1, 1], 
                    Hue[0.38345074264343776`, 1, 1], 
                    Hue[0.4589748378028819, 1, 1], 
                    Hue[0.4589748378028819, 1, 1], 
                    Hue[0.5350070831107332, 1, 1], 
                    Hue[0.5350070831107332, 1, 1], 
                    Hue[0.6115514859638331, 1, 1], 
                    Hue[0.6115514859638331, 1, 1], 
                    Hue[0.688588607439024, 1, 1], 
                    Hue[0.688588607439024, 1, 1], 
                    Hue[0.7660936824814056, 1, 1], 
                    Hue[0.7660936824814056, 1, 1], 
                    Hue[0.8440844237695915, 1, 1], 
                    Hue[0.8440844237695915, 1, 1], 
                    Hue[0.9225589053693233, 1, 1], 
                    Hue[0.9225589053693233, 1, 1], 
                    Hue[0.0015271511254317436`, 1, 1], 
                    Hue[0.0015271511254317436`, 1, 1], 
                    Hue[0.08096308242320557, 1, 1], 
                    Hue[0.08096308242320557, 1, 1], 
                    Hue[0.16084335970662272`, 1, 1], 
                    Hue[0.16084335970662272`, 1, 1], 
                    Hue[0.241147292875211, 1, 1], 
                    Hue[0.241147292875211, 1, 1], 
                    Hue[0.32189121078452165`, 1, 1], 
                    Hue[0.32189121078452165`, 1, 1], 
                    Hue[0.40307511451608796`, 1, 1], 
                    Hue[0.40307511451608796`, 1, 1], 
                    Hue[0.48468106235468705`, 1, 1], 
                    Hue[0.48468106235468705`, 1, 1], 
                    Hue[0.5666666550014439, 1, 1], 
                    Hue[0.5666666550014439, 1, 1], 
                    Hue[0.648984876342054, 1, 1], 
                    Hue[0.648984876342054, 1, 1], 
                    Hue[0.7316139984139767, 1, 1], 
                    Hue[0.7316139984139767, 1, 1], 
                    Hue[0.8145654914194567, 1, 1], 
                    Hue[0.8145654914194567, 1, 1], 
                    Hue[0.8978277043733119, 1, 1], 
                    Hue[0.8978277043733119, 1, 1], 
                    Hue[0.9813450138645246, 1, 1], 
                    Hue[0.9813450138645246, 1, 1], 
                    Hue[0.06501907948890219, 1, 1], 
                    Hue[0.06501907948890219, 1, 1], 
                    Hue[0.14876757678712846`, 1, 1], 
                    Hue[0.14876757678712846`, 1, 1], 
                    Hue[0.23256003250733165`, 1, 1], 
                    Hue[0.23256003250733165`, 1, 1], 
                    Hue[0.31639830470966585`, 1, 1], 
                    Hue[0.31639830470966585`, 1, 1], 
                    Hue[0.40023530109883876`, 1, 1], 
                    Hue[0.40023530109883876`, 1, 1], 
                    Hue[0.48393024058144063`, 1, 1], 
                    Hue[0.48393024058144063`, 1, 1], 
                    Hue[0.5672876532384993, 1, 1], 
                    Hue[0.5672876532384993, 1, 1], 
                    Hue[0.6501592758058539, 1, 1], 
                    Hue[0.6501592758058539, 1, 1], 
                    Hue[0.7324987736444111, 1, 1], 
                    Hue[0.7324987736444111, 1, 1], 
                    Hue[0.8142983145499002, 1, 1], 
                    Hue[0.8142983145499002, 1, 1], 
                    Hue[0.8954590993731899, 1, 1], 
                    Hue[0.8954590993731899, 1, 1], 
                    Hue[0.975722643047566, 1, 1], 
                    Hue[0.975722643047566, 1, 1], 
                    Hue[0.05474766844007228, 1, 1], 
                    Hue[0.05474766844007228, 1, 1], 
                    Hue[0.13227308453669961`, 1, 1], 
                    Hue[0.13227308453669961`, 1, 1], 
                    Hue[0.20821636750529138`, 1, 1], 
                    Hue[0.20821636750529138`, 1, 1], 
                    Hue[0.2826194654284094, 1, 1], 
                    Hue[0.2826194654284094, 1, 1], 
                    Hue[0.3554752741691703, 1, 1], 
                    Hue[0.3554752741691703, 1, 1], 
                    Hue[0.4265889065828907, 1, 1], 
                    Hue[0.4265889065828907, 1, 1], 
                    Hue[0.4955650664400778, 1, 1], 
                    Hue[0.4955650664400778, 1, 1], 
                    Hue[0.5619584530314572, 1, 1], 
                    Hue[0.5619584530314572, 1, 1], 
                    Hue[0.6254635887802759, 1, 1], 
                    Hue[0.6254635887802759, 1, 1], 
                    Hue[0.6860331182124972, 1, 1], 
                    Hue[0.6860331182124972, 1, 1], 
                    Hue[0.7438757636849702, 1, 1], 
                    Hue[0.7438757636849702, 1, 1], 
                    Hue[0.799376342278605, 1, 1], 
                    Hue[0.799376342278605, 1, 1], 
                    Hue[0.8530037452326122, 1, 1], 
                    Hue[0.8530037452326122, 1, 1], 
                    Hue[0.9052386573064211, 1, 1], 
                    Hue[0.9052386573064211, 1, 1], 
                    Hue[0.9565330054338083, 1, 1], 
                    Hue[0.9565330054338083, 1, 1], 
                    Hue[0.007289019921117269, 1, 1], 
                    Hue[0.007289019921117269, 1, 1], 
                    Hue[0.05785436886123326, 1, 1], 
                    Hue[0.05785436886123326, 1, 1], 
                    Hue[0.10852361694626046`, 1, 1], 
                    Hue[0.10852361694626046`, 1, 1], 
                    Hue[0.1595424939905707, 1, 1], 
                    Hue[0.1595424939905707, 1, 1], 
                    Hue[0.21111490902177088`, 1, 1], 
                    Hue[0.21111490902177088`, 1, 1], 
                    Hue[0.2634076300324872, 1, 1], 
                    Hue[0.2634076300324872, 1, 1], 
                    Hue[0.31655628975593003`, 1, 1], 
                    Hue[0.31655628975593003`, 1, 1], 
                    Hue[0.37066907391576337`, 1, 1], 
                    Hue[0.37066907391576337`, 1, 1], 
                    Hue[0.42583171930078956`, 1, 1], 
                    Hue[0.42583171930078956`, 1, 1], 
                    Hue[0.4821112905542982, 1, 1], 
                    Hue[0.4821112905542982, 1, 1], 
                    Hue[0.5395581595296779, 1, 1], 
                    Hue[0.5395581595296779, 1, 1], 
                    Hue[0.5982089184764874, 1, 1], 
                    Hue[0.5982089184764874, 1, 1], 
                    Hue[0.6580902546204306, 1, 1], 
                    Hue[0.6580902546204306, 1, 1], 
                    Hue[0.7192184186201758, 1, 1], 
                    Hue[0.7192184186201758, 1, 1], 
                    Hue[0.7816029392773315, 1, 1], 
                    Hue[0.7816029392773315, 1, 1], 
                    Hue[0.845246320651291, 1, 1], 
                    Hue[0.845246320651291, 1, 1], 
                    Hue[0.9101471280449227, 1, 1], 
                    Hue[0.9101471280449227, 1, 1], 
                    Hue[0.9762994778057256, 1, 1], 
                    Hue[0.9762994778057256, 1, 1], 
                    Hue[0.043693647064499526`, 1, 1], 
                    Hue[0.043693647064499526`, 1, 1], 
                    Hue[0.1123192138725254, 1, 1], 
                    Hue[0.1123192138725254, 1, 1], 
                    Hue[0.18216349795160328`, 1, 1], 
                    Hue[0.18216349795160328`, 1, 1], 
                    Hue[0.2532094583491348, 1, 1], 
                    Hue[0.2532094583491348, 1, 1], 
                    Hue[0.32544339799515104`, 1, 1], 
                    Hue[0.32544339799515104`, 1, 1], 
                    Hue[0.39884894807087073`, 1, 1], 
                    Hue[0.39884894807087073`, 1, 1], 
                    Hue[0.4734080173466877, 1, 1], 
                    Hue[0.4734080173466877, 1, 1], 
                    Hue[0.5491064682724095, 1, 1], 
                    Hue[0.5491064682724095, 1, 1], 
                    Hue[0.6259260435306573, 1, 1], 
                    Hue[0.6259260435306573, 1, 1], 
                    Hue[0.7038469620065598, 1, 1], 
                    Hue[0.7038469620065598, 1, 1], 
                    Hue[0.7828539447311839, 1, 1], 
                    Hue[0.7828539447311839, 1, 1], 
                    Hue[0.86293510134423, 1, 1], 
                    Hue[0.86293510134423, 1, 1], 
                    Hue[0.9440694759822695, 1, 1], 
                    Hue[0.9440694759822695, 1, 1], 
                    Hue[0.026237977530103335`, 1, 1], 
                    Hue[0.026237977530103335`, 1, 1], 
                    Hue[0.10943912867403406`, 1, 1], 
                    Hue[0.10943912867403406`, 1, 1], 
                    Hue[0.1936492364678344, 1, 1], 
                    Hue[0.1936492364678344, 1, 1], 
                    Hue[0.2788390158538952, 1, 1], 
                    Hue[0.2788390158538952, 1, 1], 
                    Hue[0.36501718255201326`, 1, 1], 
                    Hue[0.36501718255201326`, 1, 1], 
                    Hue[0.4521779601579034, 1, 1], 
                    Hue[0.4521779601579034, 1, 1], 
                    Hue[0.5402709397872767, 1, 1], 
                    Hue[0.5402709397872767, 1, 1], 
                    Hue[0.6292987165623611, 1, 1], 
                    Hue[0.6292987165623611, 1, 1], 
                    Hue[0.7192852763459747, 1, 1], 
                    Hue[0.7192852763459747, 1, 1], 
                    Hue[0.8101807715827263, 1, 1], 
                    Hue[0.8101807715827263, 1, 1], 
                    Hue[0.9019724559267499, 1, 1], 
                    Hue[0.9019724559267499, 1, 1], 
                    Hue[0.99470033906565, 1, 1], 
                    Hue[0.99470033906565, 1, 1], 
                    Hue[0.08832783678624961, 1, 1], 
                    Hue[0.08832783678624961, 1, 1], 
                    Hue[0.18277987722455066`, 1, 1], 
                    Hue[0.18277987722455066`, 1, 1], 
                    Hue[0.2780640171066228, 1, 1], 
                    Hue[0.2780640171066228, 1, 1], 
                    Hue[0.3742762888141277, 1, 1], 
                    Hue[0.3742762888141277, 1, 1], 
                    Hue[0.4714059652914306, 1, 1], 
                    Hue[0.4714059652914306, 1, 1], 
                    Hue[0.5692514338009139, 1, 1], 
                    Hue[0.5692514338009139, 1, 1], 
                    Hue[0.6677981386855287, 1, 1], 
                    Hue[0.6677981386855287, 1, 1], 
                    Hue[0.7673407367346702, 1, 1], 
                    Hue[0.7673407367346702, 1, 1], 
                    Hue[0.8678504620247419, 1, 1], 
                    Hue[0.8678504620247419, 1, 1], 
                    Hue[0.9686973038820387, 1, 1], 
                    Hue[0.9686973038820387, 1, 1], 
                    Hue[0.06963060970896097, 1, 1], 
                    Hue[0.06963060970896097, 1, 1], 
                    Hue[0.17180207357002875`, 1, 1], 
                    Hue[0.17180207357002875`, 1, 1], 
                    Hue[0.27636830103356114`, 1, 1], 
                    Hue[0.27636830103356114`, 1, 1], 
                    Hue[0.38209981443647867`, 1, 1], 
                    Hue[0.38209981443647867`, 1, 1], 
                    Hue[0.48684916138316015`, 1, 1], 
                    Hue[0.48684916138316015`, 1, 1], 
                    Hue[0.591638674225299, 1, 1], 
                    Hue[0.591638674225299, 1, 1], 
                    Hue[0.6996440399500621, 1, 1], 
                    Hue[0.6996440399500621, 1, 1], 
                    Hue[0.8106408769056764, 1, 1], 
                    Hue[0.8106408769056764, 1, 1], 
                    Hue[0.9203208846246161, 1, 1], 
                    Hue[0.9203208846246161, 1, 1], 
                    Hue[0.02405236895185515, 1, 1], 
                    Hue[0.02405236895185515, 1, 1], 
                    Hue[0.1228424398166391, 1, 1], 
                    Hue[0.1228424398166391, 1, 1], 
                    Hue[0.22619536848471866`, 1, 1], 
                    Hue[0.22619536848471866`, 1, 1], 
                    Hue[0.3370375970414235, 1, 1], 
                    Hue[0.3370375970414235, 1, 1], 
                    Hue[0.4399614845447659, 1, 1], 
                    Hue[0.4399614845447659, 1, 1], 
                    Hue[0.5268701196885841, 1, 1], 
                    Hue[0.5268701196885841, 1, 1], 
                    Hue[0.6371482431217275, 1, 1], 
                    Hue[0.6371482431217275, 1, 1], 
                    Hue[0.7836147903328474, 1, 1], 
                    Hue[0.7836147903328474, 1, 1], 
                    Hue[0.9442860013131829, 1, 1], 
                    Hue[0.9442860013131829, 1, 1], 
                    Hue[0.09297398599726875, 1, 1], 
                    Hue[0.09297398599726875, 1, 1], 
                    Hue[0.17887958096169843`, 1, 1], 
                    Hue[0.17887958096169843`, 1, 1], 
                    Hue[0.19638153085605062`, 1, 1], 
                    Hue[0.19638153085605062`, 1, 1], 
                    Hue[0.30819964433545805`, 1, 1], 
                    Hue[0.30819964433545805`, 1, 1], 
                    Hue[0.4887367267379165, 1, 1], 
                    Hue[0.4887367267379165, 1, 1], 
                    Hue[0.7100622885787811, 1, 1], 
                    Hue[0.7100622885787811, 1, 1], 
                    Hue[0.9616419546043693, 1, 1], 
                    Hue[0.9616419546043693, 1, 1], 
                    Hue[0.23801211969541905`, 1, 1], 
                    Hue[0.23801211969541905`, 1, 1], 
                    Hue[0.5347134004266552, 1, 1], 
                    Hue[0.5347134004266552, 1, 1], 
                    Hue[0.851720999705813, 1, 1], 
                    Hue[0.851720999705813, 1, 1], 
                    Hue[0.19772709565797011`, 1, 1], 
                    Hue[0.19772709565797011`, 1, 1], 
                    Hue[0.5542643360085939, 1, 1], 
                    Hue[0.5542643360085939, 1, 1], 
                    Hue[0.8726282053207132, 1, 1], 
                    Hue[0.8726282053207132, 1, 1], 
                    Hue[0.1490042529253488, 1, 1], 
                    Hue[0.1490042529253488, 1, 1], 
                    Hue[0.37880560395471286`, 1, 1], 
                    Hue[0.37880560395471286`, 1, 1], 
                    Hue[0.5180102868178059, 1, 1], 
                    Hue[0.5180102868178059, 1, 1], 
                    Hue[0.6661968199894867, 1, 1], 
                    Hue[0.6661968199894867, 1, 1], 
                    Hue[0.8912713919398864, 1, 1], 
                    Hue[0.8912713919398864, 1, 1], 
                    Hue[0.14372955017319863`, 1, 1], 
                    Hue[0.14372955017319863`, 1, 1], 
                    Hue[0.40895675323907044`, 1, 1], 
                    Hue[0.40895675323907044`, 1, 1], 
                    Hue[0.687498852931313, 1, 1], 
                    Hue[0.687498852931313, 1, 1], 
                    Hue[0.9983927521379041, 1, 1], 
                    Hue[0.9983927521379041, 1, 1], 
                    Hue[0.41402290612424353`, 1, 1], 
                    Hue[0.41402290612424353`, 1, 1], 
                    Hue[0.8107856520326657, 1, 1], 
                    Hue[0.8107856520326657, 1, 1], 
                    Hue[0.11575640376966859`, 1, 1], 
                    Hue[0.11575640376966859`, 1, 1], 
                    Hue[0.39601159431580035`, 1, 1], 
                    Hue[0.39601159431580035`, 1, 1], 
                    Hue[0.6681929871460615, 1, 1], 
                    Hue[0.6681929871460615, 1, 1], 
                    Hue[0.9369112472018684, 1, 1], 
                    Hue[0.9369112472018684, 1, 1], 
                    Hue[0.20387105287694343`, 1, 1], 
                    Hue[0.20387105287694343`, 1, 1], 
                    Hue[0.46982427789568754`, 1, 1], 
                    Hue[0.46982427789568754`, 1, 1], 
                    Hue[0.7351428404407656, 1, 1], 
                    Hue[0.7351428404407656, 1, 1], 
                    Hue[0.000045043010442031465`, 1, 1], 
                    Hue[0.000045043010442031465`, 1, 1], 
                    Hue[0.26472928293010783`, 1, 1], 
                    Hue[0.26472928293010783`, 1, 1], 
                    Hue[0.529502741806376, 1, 1], 
                    Hue[0.529502741806376, 1, 1], 
                    Hue[0.7949436009945829, 1, 1], 
                    Hue[0.7949436009945829, 1, 1], 
                    Hue[0.06210321930714255, 1, 1], 
                    Hue[0.06210321930714255, 1, 1], 
                    Hue[0.33266837165463214`, 1, 1], 
                    Hue[0.33266837165463214`, 1, 1], 
                    Hue[0.6088044722098869, 1, 1], 
                    Hue[0.6088044722098869, 1, 1], 
                    Hue[0.892140756761359, 1, 1], 
                    Hue[0.892140756761359, 1, 1], 
                    Hue[0.18175714747005245`, 1, 1], 
                    Hue[0.18175714747005245`, 1, 1], 
                    Hue[0.47326275941150026`, 1, 1], 
                    Hue[0.47326275941150026`, 1, 1], 
                    Hue[0.7616040472763749, 1, 1], 
                    Hue[0.7616040472763749, 1, 1], 
                    Hue[0.044437360962688716`, 1, 1], 
                    Hue[0.044437360962688716`, 1, 1], 
                    Hue[0.32202075940832503`, 1, 1], 
                    Hue[0.32202075940832503`, 1, 1], 
                    Hue[0.5955234481590492, 1, 1], 
                    Hue[0.5955234481590492, 1, 1], 
                    Hue[0.8660326380659066, 1, 1], 
                    Hue[0.8660326380659066, 1, 1], 
                    Hue[0.1343067124487573, 1, 1]}],
                   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
                   Axes->{True, None},
                   AxesOrigin->{0, 0},
                   
                   Epilog->LineBox[{{1, 1.263148251317129*^-6}, {
                    2, 1.0988000515666537`*^-6}, {
                    3, 1.0817263863807828`*^-6}, {
                    4, 1.0119678170251821`*^-6}, {
                    5, 1.4268495912945795`*^-6}, {
                    6, 1.2158198379080188`*^-6}, {7, 8.503215924495397*^-7}, {
                    8, 7.119171371667034*^-7}, {9, 1.2699117946955453`*^-6}, {
                    10, 1.1097360510145725`*^-6}, {
                    11, 9.686198262614006*^-7}, {
                    12, 1.0440379619942911`*^-6}, {
                    13, 5.361746432048057*^-7}, {14, 2.912173475928828*^-7}, {
                    15, 7.050495895397025*^-7}, {
                    16, 3.5814730863871657`*^-7}, {
                    17, 5.396703710757984*^-7}, {18, 7.664627341804243*^-7}, {
                    19, 4.0801767795511484`*^-7}, {
                    20, 6.991707873749146*^-7}, {21, 5.263730152481841*^-7}, {
                    22, 9.70780317961668*^-7}, {23, 8.644035522275964*^-7}, {
                    24, 6.937235396962769*^-7}, {
                    25, 1.3516433050748546`*^-6}, {
                    26, 1.2768069719849307`*^-6}, {
                    27, 1.0238978125815495`*^-6}, {
                    28, 8.898613927937892*^-7}, {29, 5.341727732913914*^-7}, {
                    30, 6.209342000461613*^-7}, {31, 9.009864392952196*^-7}, {
                    32, 5.475783116943175*^-7}, {33, 4.523964194472669*^-7}, {
                    34, 7.779450560138102*^-7}, {35, 5.653741555509746*^-7}, {
                    36, 6.678906401350125*^-7}, {37, 3.048580799157426*^-7}, {
                    38, 1.7862246494954367`*^-7}, {
                    39, 3.1412611311898113`*^-7}, {
                    40, 4.728133286029297*^-7}, {
                    41, 1.1059742026166754`*^-6}, {
                    42, 9.711285501908659*^-7}, {43, 6.804852580820673*^-7}, {
                    44, 4.0418701019270225`*^-7}, {
                    45, 8.061249137868119*^-7}, {
                    46, 1.6011490721009806`*^-7}, {
                    47, 1.1980869562401297`*^-6}, {
                    48, 1.7954501904300843`*^-6}, {
                    49, 1.764768036572435*^-6}, {50, 1.741554027841161*^-6}, {
                    51, 2.547808912351531*^-6}, {52, 3.967349067635245*^-6}, {
                    53, 4.675277820415082*^-6}, {54, 6.705248721334987*^-6}, {
                    55, 9.1105241168038*^-6}, {
                    56, 0.000012079194735455833`}, {
                    57, 0.000016305790019528445`}, {
                    58, 0.000020496125247843785`}, {
                    59, 0.000026719599565117968`}, {
                    60, 0.00003372651932261041}, {
                    61, 0.00004303338915770956}, {
                    62, 0.00005491463309140772}, {
                    63, 0.00007083746466335644}, {
                    64, 0.00008993613583068241}, {
                    65, 0.00011330692805068455`}, {
                    66, 0.00014343820280123958`}, {
                    67, 0.00017999827480562385`}, {
                    68, 0.00022746334371359815`}, {
                    69, 0.000284740801932554}, {70, 0.0003560084751828962}, {
                    71, 0.00044258016571382285`}, {72, 0.00054888069144532}, {
                    73, 0.0006797480336058403}, {74, 0.0008386814466846738}, {
                    75, 0.0010318864144614696`}, {
                    76, 0.0012624659024528951`}, {
                    77, 0.0015406394425161105`}, {
                    78, 0.0018749729185660578`}, {
                    79, 0.0022757260584992576`}, {80, 0.0027541709743578217`},
                     {81, 0.0033193790856586147`}, {
                    82, 0.0039877325408871804`}, {
                    83, 0.0047761189427326925`}, {84, 0.005703182499766771}, {
                    85, 0.006788783044794935}, {86, 0.00805369122293874}, {
                    87, 0.009524395031424866}, {88, 0.01122698434952529}, {
                    89, 0.013192045776814772`}, {90, 0.015454561606664587`}, {
                    91, 0.018046737583053998`}, {92, 0.021006611899095824`}, {
                    93, 0.024372539765145262`}, {94, 0.028189047597036564`}, {
                    95, 0.03250456613877092}, {96, 0.037367231421487866`}, {
                    97, 0.0428209875868495}, {98, 0.048912629874264715`}, {
                    99, 0.055696952208377455`}, {100, 0.06323502455006375}, {
                    101, 0.07158860340452554}, {102, 0.08080870784750611}, {
                    103, 0.0909318631190164}, {104, 0.10200305896600698`}, {
                    105, 0.1140904859340883}, {106, 0.12728168839326717`}, {
                    107, 0.141643848728605}, {108, 0.15719544874447805`}, {
                    109, 0.1739270345635291}, {110, 0.1918782184002109}, {
                    111, 0.21117046433785905`}, {112, 0.23196424641881064`}, {
                    113, 0.25431892722897964`}, {114, 0.27814526654604743`}, {
                    115, 0.30331112666387466`}, {116, 0.32986095364484397`}, {
                    117, 0.35809505922371804`}, {118, 0.3883717365143623}, {
                    119, 0.4207607713769854}, {120, 0.4548740029477722}, {
                    121, 0.490110334868377}, {122, 0.5261493338945208}, {
                    123, 0.5632673404284136}, {124, 0.6021412600336473}, {
                    125, 0.6431759683372389}, {126, 0.6857803321079176}, {
                    127, 0.728075313886562}, {128, 0.7672047976373915}, {
                    129, 0.800014715616252}, {130, 0.8237640602600242}, {
                    131, 0.8365869153357057}, {132, 0.8376782875817699}, {
                    133, 0.8272295868527458}, {134, 0.8062211834197465}, {
                    135, 0.7761660585836947}, {136, 0.7388520666179677}, {
                    137, 0.6961367814201835}, {138, 0.6497960193904806}, {
                    139, 0.6014253585615972}, {140, 0.5523908288486171}, {
                    141, 0.5038091195575622}, {142, 0.4565611420950155}, {
                    143, 0.4113057663216284}, {144, 0.36851019085678266`}, {
                    145, 0.328482001850657}, {146, 0.29139632688865214`}, {
                    147, 0.25732124094024783`}, {148, 0.2262453381993219}, {
                    149, 0.1980946081755368}, {150, 0.17275083936779465`}, {
                    151, 0.15006261914988958`}, {152, 0.12986009072911917`}, {
                    153, 0.11196120849466382`}, {154, 0.09617775359400356}, {
                    155, 0.08232318253314833}, {156, 0.07021616422173953}, {
                    157, 0.0596802037105046}, {158, 0.05054994265191247}, {
                    159, 0.042669791788711206`}, {160, 0.03589490528875932}, {
                    161, 0.030093437210281387`}, {
                    162, 0.025145102355836807`}, {
                    163, 0.020940067802506627`}, {
                    164, 0.017379509419725825`}, {
                    165, 0.014376847488959774`}, {
                    166, 0.011853757125176678`}, {
                    167, 0.009740389731214268}, {168, 0.007977573433021856}, {
                    169, 0.006512956797516867}, {170, 0.005299299322158557}, {
                    171, 0.004297337987412937}, {
                    172, 0.0034740358857803864`}, {
                    173, 0.002799294019160139}, {
                    174, 0.0022479382273943526`}, {
                    175, 0.0017995581984829431`}, {
                    176, 0.0014359787938274453`}, {
                    177, 0.001141760038936783}, {
                    178, 0.0009047212899598665}, {
                    179, 0.0007147979075875034}, {180, 0.00056305882367613}, {
                    181, 0.0004418018253960445}, {
                    182, 0.0003452920105716575}, {
                    183, 0.000269206229752395}, {
                    184, 0.00020936042558596652`}, {
                    185, 0.00016198302063449976`}, {
                    186, 0.00012465461503090568`}, {
                    187, 0.00009595256142123079}, {
                    188, 0.00007417392975360292}, {
                    189, 0.00005711911105166829}, {
                    190, 0.00004335333802676895}, {
                    191, 0.00003265278475943498}, {
                    192, 0.00002480302838990986}, {
                    193, 0.00001888902636019418}, {
                    194, 0.000014069409021099529`}, {
                    195, 0.000010140238061285121`}, {
                    196, 7.223437696523649*^-6}, {
                    197, 5.331976073950158*^-6}, {
                    198, 4.116946903917992*^-6}, {
                    199, 3.1000435757834277`*^-6}, {
                    200, 2.212493793781606*^-6}, {
                    201, 1.7640415257678798`*^-6}, {
                    202, 1.6774412610150379`*^-6}, {
                    203, 1.4863064073338804`*^-6}, {
                    204, 1.073783501540085*^-6}, {
                    205, 6.067001431692634*^-7}, {
                    206, 2.8887998994694905`*^-7}, {
                    207, 2.5692678868152457`*^-7}, {
                    208, 3.6375127468225415`*^-7}, {
                    209, 4.586372637019976*^-7}, {
                    210, 4.920130033778911*^-7}, {
                    211, 4.5768653173881383`*^-7}, {
                    212, 3.833207615838428*^-7}, {
                    213, 2.985179898232454*^-7}, {
                    214, 2.187705880180826*^-7}, {
                    215, 1.6257060200826846`*^-7}, {
                    216, 1.5012810121569064`*^-7}, {
                    217, 1.551910978414857*^-7}, {
                    218, 1.393170641899645*^-7}, {
                    219, 9.902569698696222*^-8}, {
                    220, 7.324759957669555*^-8}, {
                    221, 1.0613153206144688`*^-7}, {
                    222, 1.4904763485985006`*^-7}, {
                    223, 1.6954928874328623`*^-7}, {
                    224, 1.6091045023207582`*^-7}, {
                    225, 1.270671391642489*^-7}, {
                    226, 7.982965316575254*^-8}, {
                    227, 5.301977242089482*^-8}, {228, 8.8312965017922*^-8}, {
                    229, 1.3920213858281637`*^-7}, {
                    230, 1.8171512090898223`*^-7}, {
                    231, 2.0987282765413233`*^-7}, {
                    232, 2.2238103460634007`*^-7}, {
                    233, 2.204307549653656*^-7}, {
                    234, 2.0675662642762887`*^-7}, {
                    235, 1.8488282323233655`*^-7}, {
                    236, 1.5847104530444418`*^-7}, {
                    237, 1.3080985986707924`*^-7}, {
                    238, 1.0448319462058098`*^-7}, {
                    239, 8.122465889264352*^-8}, {
                    240, 6.194217610725269*^-8}, {
                    241, 4.687140568053066*^-8}, {
                    242, 3.580397842895375*^-8}, {
                    243, 2.830297650785658*^-8}, {
                    244, 2.3799015296220542`*^-8}, {
                    245, 2.15533365868183*^-8}, {
                    246, 2.0687898647831395`*^-8}, {
                    247, 2.039239155408677*^-8}, {
                    248, 2.009900121546737*^-8}, {
                    249, 1.949959901940705*^-8}, {
                    250, 1.8482672936234614`*^-8}, {
                    251, 1.7064321965583427`*^-8}}],
                   Frame->True,
                   FrameTicks->{{{1., 
                    FormBox[
                    RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
                    FormBox[
                    RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
                    FormBox[
                    RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
                    FormBox[
                    RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
                    FormBox["0", TraditionalForm]}, {157.25, 
                    FormBox["2.5`", TraditionalForm]}, {188.5, 
                    FormBox["5", TraditionalForm]}, {219.75, 
                    FormBox["7.5`", TraditionalForm]}, {251., 
                    FormBox["10", TraditionalForm]}}, Automatic, None, None},
                   PlotRange->{-0.1, 1.1},
                   PlotRangeClipping->True,
                   PlotRangePadding->{
                    Scaled[0.02], Automatic}], 11->
                  GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF13dYFHcaB/BJsJ7GS0CiMZZVERsgEQU7XwXBBtI7y1B2gaWdothOGUEE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                    "], {
                    {GrayLevel[0.5], EdgeForm[None], 
                    GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ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                    
                    "]]]}, {{}, {}, {
                    {Hue[0.5134986713962597, 1, 1], LineBox[{1, 2}]}, 
                    {Hue[0.429675798262789, 1, 1], LineBox[{2, 3}]}, 
                    {Hue[0.36537759142896803`, 1, 1], LineBox[{3, 4}]}, 
                    {Hue[0.287489306520422, 1, 1], LineBox[{4, 5}]}, 
                    {Hue[0.11714250593167995`, 1, 1], LineBox[{5, 6}]}, 
                    {Hue[0.030302165263602618`, 1, 1], LineBox[{6, 7}]}, 
                    {Hue[0.0651831562564822, 1, 1], LineBox[{7, 8}]}, 
                    {Hue[0.03005619215611696, 1, 1], LineBox[{8, 9}]}, 
                    {Hue[0.8086350452323195, 1, 1], LineBox[{9, 10}]}, 
                    {Hue[0.5538481087637268, 1, 1], LineBox[{10, 11}]}, 
                    {Hue[0.5327463853830385, 1, 1], LineBox[{11, 12}]}, 
                    {Hue[0.3282688019815088, 1, 1], LineBox[{12, 13}]}, 
                    {Hue[0.2199518739415025, 1, 1], LineBox[{13, 14}]}, 
                    {Hue[0.22297225551656205`, 1, 1], LineBox[{14, 15}]}, 
                    {Hue[0.08319886681240779, 1, 1], LineBox[{15, 16}]}, 
                    {Hue[0.059918982847335045`, 1, 1], LineBox[{16, 17}]}, 
                    {Hue[0.9702160847775785, 1, 1], LineBox[{17, 18}]}, 
                    {Hue[0.7991004302923844, 1, 1], LineBox[{18, 19}]}, 
                    {Hue[0.644117418730759, 1, 1], LineBox[{19, 20}]}, 
                    {Hue[0.6356139264194032, 1, 1], LineBox[{20, 21}]}, 
                    {Hue[0.5268321040319055, 1, 1], LineBox[{21, 22}]}, 
                    {Hue[0.4849149904511286, 1, 1], LineBox[{22, 23}]}, 
                    {Hue[0.2951625421267708, 1, 1], LineBox[{23, 24}]}, 
                    {Hue[0.17072137554429173`, 1, 1], LineBox[{24, 25}]}, 
                    {Hue[0.8425020370839564, 1, 1], LineBox[{25, 26}]}, 
                    {Hue[0.5623136607528382, 1, 1], LineBox[{26, 27}]}, 
                    {Hue[0.3847582675161996, 1, 1], LineBox[{27, 28}]}, 
                    {Hue[0.6570983483750655, 1, 1], LineBox[{28, 29}]}, 
                    {Hue[0.41201678385957174`, 1, 1], LineBox[{29, 30}]}, 
                    {Hue[0.35912242361652047`, 1, 1], LineBox[{30, 31}]}, 
                    {Hue[0.18101548966175515`, 1, 1], LineBox[{31, 32}]}, 
                    {Hue[0.16450131136982224`, 1, 1], LineBox[{32, 33}]}, 
                    {Hue[0.9480130500106693, 1, 1], LineBox[{33, 34}]}, 
                    {Hue[0.7136847614331843, 1, 1], LineBox[{34, 35}]}, 
                    {Hue[0.6820379269327059, 1, 1], LineBox[{35, 36}]}, 
                    {Hue[0.5438279527273757, 1, 1], LineBox[{36, 37}]}, 
                    {Hue[0.4815860989652991, 1, 1], LineBox[{37, 38}]}, 
                    {Hue[0.4540529949539039, 1, 1], LineBox[{38, 39}]}, 
                    {Hue[0.588981116835043, 1, 1], LineBox[{39, 40}]}, 
                    {Hue[0.44981566460748473`, 1, 1], LineBox[{40, 41}]}, 
                    {Hue[0.1962791259326371, 1, 1], LineBox[{41, 42}]}, 
                    {Hue[0.5211218777608584, 1, 1], LineBox[{42, 43}]}, 
                    {Hue[0.47547988701683014`, 1, 1], LineBox[{43, 44}]}, 
                    {Hue[0.7495477847379686, 1, 1], LineBox[{44, 45}]}, 
                    {Hue[0.6681734265858652, 1, 1], LineBox[{45, 46}]}, 
                    {Hue[0.5077961245052601, 1, 1], LineBox[{46, 47}]}, 
                    {Hue[0.6866704623527522, 1, 1], LineBox[{47, 48}]}, 
                    {Hue[0.6452365448802283, 1, 1], LineBox[{48, 49}]}, 
                    {Hue[0.8005378726770057, 1, 1], LineBox[{49, 50}]}, 
                    {Hue[0.8643750617374688, 1, 1], LineBox[{50, 51}]}, 
                    {Hue[0.6693497106381865, 1, 1], LineBox[{51, 52}]}, 
                    {Hue[0.9575525251427208, 1, 1], LineBox[{52, 53}]}, 
                    {Hue[0.22592667680464848`, 1, 1], LineBox[{53, 54}]}, 
                    {Hue[0.2661863141424179, 1, 1], LineBox[{54, 55}]}, 
                    {Hue[0.22840163982044662`, 1, 1], LineBox[{55, 56}]}, 
                    {Hue[0.31215738198099585`, 1, 1], LineBox[{56, 57}]}, 
                    {Hue[0.31477941715247937`, 1, 1], LineBox[{57, 58}]}, 
                    {Hue[0.39024845611634024`, 1, 1], LineBox[{58, 59}]}, 
                    {Hue[0.47248760524899003`, 1, 1], LineBox[{59, 60}]}, 
                    {Hue[0.5211974196257148, 1, 1], LineBox[{60, 61}]}, 
                    {Hue[0.5636352335907608, 1, 1], LineBox[{61, 62}]}, 
                    {Hue[0.6027404106171735, 1, 1], LineBox[{62, 63}]}, 
                    {Hue[0.6707034118133188, 1, 1], LineBox[{63, 64}]}, 
                    {Hue[0.7327249096922388, 1, 1], LineBox[{64, 65}]}, 
                    {Hue[0.7986624195234826, 1, 1], LineBox[{65, 66}]}, 
                    {Hue[0.855887994596807, 1, 1], LineBox[{66, 67}]}, 
                    {Hue[0.9095456426708814, 1, 1], LineBox[{67, 68}]}, 
                    {Hue[0.970077867929754, 1, 1], LineBox[{68, 69}]}, 
                    {Hue[0.036075328277316174`, 1, 1], LineBox[{69, 70}]}, 
                    {Hue[0.0986472384956598, 1, 1], LineBox[{70, 71}]}, 
                    {Hue[0.16258323239669467`, 1, 1], LineBox[{71, 72}]}, 
                    {Hue[0.22439962875667932`, 1, 1], LineBox[{72, 73}]}, 
                    {Hue[0.2868228301934304, 1, 1], LineBox[{73, 74}]}, 
                    {Hue[0.3506068304944729, 1, 1], LineBox[{74, 75}]}, 
                    {Hue[0.4151345914868187, 1, 1], LineBox[{75, 76}]}, 
                    {Hue[0.4806693457773988, 1, 1], LineBox[{76, 77}]}, 
                    {Hue[0.5460311877138803, 1, 1], LineBox[{77, 78}]}, 
                    {Hue[0.6120088180229494, 1, 1], LineBox[{78, 79}]}, 
                    {Hue[0.6780807385652687, 1, 1], LineBox[{79, 80}]}, 
                    {Hue[0.7447573890440767, 1, 1], LineBox[{80, 81}]}, 
                    {Hue[0.8124943399697049, 1, 1], LineBox[{81, 82}]}, 
                    {Hue[0.8807725622213715, 1, 1], LineBox[{82, 83}]}, 
                    {Hue[0.9493408685771565, 1, 1], LineBox[{83, 84}]}, 
                    {Hue[0.018181533898883856`, 1, 1], LineBox[{84, 85}]}, 
                    {Hue[0.08740416222963696, 1, 1], LineBox[{85, 86}]}, 
                    {Hue[0.15726713787309227`, 1, 1], LineBox[{86, 87}]}, 
                    {Hue[0.22793403650425142`, 1, 1], LineBox[{87, 88}]}, 
                    {Hue[0.2992108352550998, 1, 1], LineBox[{88, 89}]}, 
                    {Hue[0.37080822391280877`, 1, 1], LineBox[{89, 90}]}, 
                    {Hue[0.44261246362794693`, 1, 1], LineBox[{90, 91}]}, 
                    {Hue[0.5147944799691418, 1, 1], LineBox[{91, 92}]}, 
                    {Hue[0.5876513479851337, 1, 1], LineBox[{92, 93}]}, 
                    {Hue[0.6613124979945698, 1, 1], LineBox[{93, 94}]}, 
                    {Hue[0.7355536948539427, 1, 1], LineBox[{94, 95}]}, 
                    {Hue[0.8100331941773403, 1, 1], LineBox[{95, 96}]}, 
                    {Hue[0.8845760918967533, 1, 1], LineBox[{96, 97}]}, 
                    {Hue[0.9594392237794754, 1, 1], LineBox[{97, 98}]}, 
                    {Hue[0.035026619894552934`, 1, 1], LineBox[{98, 99}]}, 
                    {Hue[0.11148126313743942`, 1, 1], LineBox[{99, 100}]}, 
                    {Hue[0.18846222297838114`, 1, 1], LineBox[{100, 101}]}, 
                    {Hue[0.2654320874095458, 1, 1], LineBox[{101, 102}]}, 
                    {Hue[0.3422347714240084, 1, 1], LineBox[{102, 103}]}, 
                    {Hue[0.4192481284631285, 1, 1], LineBox[{103, 104}]}, 
                    {Hue[0.4970426128281789, 1, 1], LineBox[{104, 105}]}, 
                    {Hue[0.5757258532795645, 1, 1], LineBox[{105, 106}]}, 
                    {Hue[0.6547295590250144, 1, 1], LineBox[{106, 107}]}, 
                    {Hue[0.7333012536351259, 1, 1], LineBox[{107, 108}]}, 
                    {Hue[0.8112505621189473, 1, 1], LineBox[{108, 109}]}, 
                    {Hue[0.8891688664450038, 1, 1], LineBox[{109, 110}]}, 
                    {Hue[0.9678089829795671, 1, 1], LineBox[{110, 111}]}, 
                    {Hue[0.04724536239282264, 1, 1], LineBox[{111, 112}]}, 
                    {Hue[0.12659362795393675`, 1, 1], LineBox[{112, 113}]}, 
                    {Hue[0.2047569751445727, 1, 1], LineBox[{113, 114}]}, 
                    {Hue[0.2814529462598848, 1, 1], LineBox[{114, 115}]}, 
                    {Hue[0.3574199080216351, 1, 1], LineBox[{115, 116}]}, 
                    {Hue[0.4336652855048778, 1, 1], LineBox[{116, 117}]}, 
                    {Hue[0.5104069744697515, 1, 1], LineBox[{117, 118}]}, 
                    {Hue[0.5866138643940774, 1, 1], LineBox[{118, 119}]}, 
                    {Hue[0.6607219582217274, 1, 1], LineBox[{119, 120}]}, 
                    {Hue[0.7319085375689991, 1, 1], LineBox[{120, 121}]}, 
                    {Hue[0.800597168484313, 1, 1], LineBox[{121, 122}]}, 
                    {Hue[0.8679469213743725, 1, 1], LineBox[{122, 123}]}, 
                    {Hue[0.9349223090369303, 1, 1], LineBox[{123, 124}]}, 
                    {Hue[0.001510620429620076, 1, 1], LineBox[{124, 125}]}, 
                    {Hue[0.06652459152823118, 1, 1], LineBox[{125, 126}]}, 
                    {Hue[0.1282599143269595, 1, 1], LineBox[{126, 127}]}, 
                    {Hue[0.18550179423443017`, 1, 1], LineBox[{127, 128}]}, 
                    {Hue[0.23800735641251244`, 1, 1], LineBox[{128, 129}]}, 
                    {Hue[0.2862968986993083, 1, 1], LineBox[{129, 130}]}, 
                    {Hue[0.33123280398036214`, 1, 1], LineBox[{130, 131}]}, 
                    {Hue[0.3737126292032006, 1, 1], LineBox[{131, 132}]}, 
                    {Hue[0.41453560107924475`, 1, 1], LineBox[{132, 133}]}, 
                    {Hue[0.4543646210431929, 1, 1], LineBox[{133, 134}]}, 
                    {Hue[0.49373324008370056`, 1, 1], LineBox[{134, 135}]}, 
                    {Hue[0.5330669882843551, 1, 1], LineBox[{135, 136}]}, 
                    {Hue[0.5727040733701112, 1, 1], LineBox[{136, 137}]}, 
                    {Hue[0.6129127180864538, 1, 1], LineBox[{137, 138}]}, 
                    {Hue[0.6539059380017792, 1, 1], LineBox[{138, 139}]}, 
                    {Hue[0.6958512749606287, 1, 1], LineBox[{139, 140}]}, 
                    {Hue[0.7388802296152222, 1, 1], LineBox[{140, 141}]}, 
                    {Hue[0.7830946197868673, 1, 1], LineBox[{141, 142}]}, 
                    {Hue[0.8285721896317425, 1, 1], LineBox[{142, 143}]}, 
                    {Hue[0.8753695061192841, 1, 1], LineBox[{143, 144}]}, 
                    {Hue[0.923527316882723, 1, 1], LineBox[{144, 145}]}, 
                    {Hue[0.9730728782416933, 1, 1], LineBox[{145, 146}]}, 
                    {Hue[0.024022163083270768`, 1, 1], LineBox[{146, 147}]}, 
                    {Hue[0.07638221788367007, 1, 1], LineBox[{147, 148}]}, 
                    {Hue[0.13015318159727385`, 1, 1], LineBox[{148, 149}]}, 
                    {Hue[0.1853285701027619, 1, 1], LineBox[{149, 150}]}, 
                    {Hue[0.24189914564441517`, 1, 1], LineBox[{150, 151}]}, 
                    {Hue[0.2998500303351519, 1, 1], LineBox[{151, 152}]}, 
                    {Hue[0.35916544194696576`, 1, 1], LineBox[{152, 153}]}, 
                    {Hue[0.4198266093930038, 1, 1], LineBox[{153, 154}]}, 
                    {Hue[0.4818143337854014, 1, 1], LineBox[{154, 155}]}, 
                    {Hue[0.5451075206194197, 1, 1], LineBox[{155, 156}]}, 
                    {Hue[0.6096848949214557, 1, 1], LineBox[{156, 157}]}, 
                    {Hue[0.6755247680557995, 1, 1], LineBox[{157, 158}]}, 
                    {Hue[0.7426074280870318, 1, 1], LineBox[{158, 159}]}, 
                    {Hue[0.8109106936417033, 1, 1], LineBox[{159, 160}]}, 
                    {Hue[0.8804129310143901, 1, 1], LineBox[{160, 161}]}, 
                    {Hue[0.951093924998524, 1, 1], LineBox[{161, 162}]}, 
                    {Hue[0.022934143089617415`, 1, 1], LineBox[{162, 163}]}, 
                    {Hue[0.09591393913800811, 1, 1], LineBox[{163, 164}]}, 
                    {Hue[0.17001339104666313`, 1, 1], LineBox[{164, 165}]}, 
                    {Hue[0.2452148794921656, 1, 1], LineBox[{165, 166}]}, 
                    {Hue[0.3215035907254118, 1, 1], LineBox[{166, 167}]}, 
                    {Hue[0.3988570522565961, 1, 1], LineBox[{167, 168}]}, 
                    {Hue[0.4772609021301015, 1, 1], LineBox[{168, 169}]}, 
                    {Hue[0.5567029416512079, 1, 1], LineBox[{169, 170}]}, 
                    {Hue[0.6371604032186986, 1, 1], LineBox[{170, 171}]}, 
                    {Hue[0.7186211088714941, 1, 1], LineBox[{171, 172}]}, 
                    {Hue[0.8010744330456714, 1, 1], LineBox[{172, 173}]}, 
                    {Hue[0.8845040486856267, 1, 1], LineBox[{173, 174}]}, 
                    {Hue[0.9688939180842179, 1, 1], LineBox[{174, 175}]}, 
                    {Hue[0.05423573609039471, 1, 1], LineBox[{175, 176}]}, 
                    {Hue[0.14051347720991264`, 1, 1], LineBox[{176, 177}]}, 
                    {Hue[0.22770374758516046`, 1, 1], LineBox[{177, 178}]}, 
                    {Hue[0.3158165858776364, 1, 1], LineBox[{178, 179}]}, 
                    {Hue[0.40485117363829604`, 1, 1], LineBox[{179, 180}]}, 
                    {Hue[0.4947574943312316, 1, 1], LineBox[{180, 181}]}, 
                    {Hue[0.5855326413478238, 1, 1], LineBox[{181, 182}]}, 
                    {Hue[0.6772189855349321, 1, 1], LineBox[{182, 183}]}, 
                    {Hue[0.76975196888853, 1, 1], LineBox[{183, 184}]}, 
                    {Hue[0.863080250307886, 1, 1], LineBox[{184, 185}]}, 
                    {Hue[0.95731350477976, 1, 1], LineBox[{185, 186}]}, 
                    {Hue[0.052435140580638674`, 1, 1], LineBox[{186, 187}]}, 
                    {Hue[0.14829772018432766`, 1, 1], LineBox[{187, 188}]}, 
                    {Hue[0.24497580630907112`, 1, 1], LineBox[{188, 189}]}, 
                    {Hue[0.34257966189593014`, 1, 1], LineBox[{189, 190}]}, 
                    {Hue[0.44099189618055484`, 1, 1], LineBox[{190, 191}]}, 
                    {Hue[0.5401359020862424, 1, 1], LineBox[{191, 192}]}, 
                    {Hue[0.6401264403994193, 1, 1], LineBox[{192, 193}]}, 
                    {Hue[0.7409793231976449, 1, 1], LineBox[{193, 194}]}, 
                    {Hue[0.842431620268548, 1, 1], LineBox[{194, 195}]}, 
                    {Hue[0.9444064645043521, 1, 1], LineBox[{195, 196}]}, 
                    {Hue[0.04739874473607524, 1, 1], LineBox[{196, 197}]}, 
                    {Hue[0.15160986904984824`, 1, 1], LineBox[{197, 198}]}, 
                    {Hue[0.2562192662631057, 1, 1], LineBox[{198, 199}]}, 
                    {Hue[0.36062671712047917`, 1, 1], LineBox[{199, 200}]}, 
                    {Hue[0.46586838643294537`, 1, 1], LineBox[{200, 201}]}, 
                    {Hue[0.5731461208010898, 1, 1], LineBox[{201, 202}]}, 
                    {Hue[0.680934766329103, 1, 1], LineBox[{202, 203}]}, 
                    {Hue[0.7865115346926551, 1, 1], LineBox[{203, 204}]}, 
                    {Hue[0.8927582528892654, 1, 1], LineBox[{204, 205}]}, 
                    {Hue[0.00683152188586909, 1, 1], LineBox[{205, 206}]}, 
                    {Hue[0.12756989049417516`, 1, 1], LineBox[{206, 207}]}, 
                    {Hue[0.24415429036566494`, 1, 1], LineBox[{207, 208}]}, 
                    {Hue[0.3490163234031179, 1, 1], LineBox[{208, 209}]}, 
                    {Hue[0.45444314884782067`, 1, 1], LineBox[{209, 210}]}, 
                    {Hue[0.5747967080651628, 1, 1], LineBox[{210, 211}]}, 
                    {Hue[0.7022712739746917, 1, 1], LineBox[{211, 212}]}, 
                    {Hue[0.8051517666191494, 1, 1], LineBox[{212, 213}]}, 
                    {Hue[0.8245393293497276, 1, 1], LineBox[{213, 214}]}, 
                    {Hue[0.8981395757716686, 1, 1], LineBox[{214, 215}]}, 
                    {Hue[0.043647678620711894`, 1, 1], LineBox[{215, 216}]}, 
                    {Hue[0.1952359421759699, 1, 1], LineBox[{216, 217}]}, 
                    {Hue[0.2979846970988019, 1, 1], LineBox[{217, 218}]}, 
                    {Hue[0.3842789352023804, 1, 1], LineBox[{218, 219}]}, 
                    {Hue[0.5490636286686705, 1, 1], LineBox[{219, 220}]}, 
                    {Hue[0.7531878470074891, 1, 1], LineBox[{220, 221}]}, 
                    {Hue[0.9741316560530584, 1, 1], LineBox[{221, 222}]}, 
                    {Hue[0.21069204938267144`, 1, 1], LineBox[{222, 223}]}, 
                    {Hue[0.5179625464986409, 1, 1], LineBox[{223, 224}]}, 
                    {Hue[0.9515960101338363, 1, 1], LineBox[{224, 225}]}, 
                    {Hue[0.21823819534412597`, 1, 1], LineBox[{225, 226}]}, 
                    {Hue[0.46127061830899735`, 1, 1], LineBox[{226, 227}]}, 
                    {Hue[0.7122031613185107, 1, 1], LineBox[{227, 228}]}, 
                    {Hue[0.9728813343268621, 1, 1], LineBox[{228, 229}]}, 
                    {Hue[0.2420268926362587, 1, 1], LineBox[{229, 230}]}, 
                    {Hue[0.5202779494326837, 1, 1], LineBox[{230, 231}]}, 
                    {Hue[0.8127800983269537, 1, 1], LineBox[{231, 232}]}, 
                    {Hue[0.13233314843452446`, 1, 1], LineBox[{232, 233}]}, 
                    {Hue[0.48454859416905616`, 1, 1], LineBox[{233, 234}]}, 
                    {Hue[0.8231200751796759, 1, 1], LineBox[{234, 235}]}, 
                    {Hue[0.11904642996065473`, 1, 1], LineBox[{235, 236}]}, 
                    {Hue[0.3777046432339246, 1, 1], LineBox[{236, 237}]}, 
                    {Hue[0.5843606419824321, 1, 1], LineBox[{237, 238}]}, 
                    {Hue[0.7304586385055036, 1, 1], LineBox[{238, 239}]}, 
                    {Hue[0.9181566980435721, 1, 1], LineBox[{239, 240}]}, 
                    {Hue[0.15404779540900293`, 1, 1], LineBox[{240, 241}]}, 
                    {Hue[0.409814915165141, 1, 1], LineBox[{241, 242}]}, 
                    {Hue[0.6769779427689762, 1, 1], LineBox[{242, 243}]}, 
                    {Hue[0.9577656899112758, 1, 1], LineBox[{243, 244}]}, 
                    {Hue[0.2723615776834439, 1, 1], LineBox[{244, 245}]}, 
                    {Hue[0.6820614538136163, 1, 1], LineBox[{245, 246}]}, 
                    {Hue[0.06994172851893946, 1, 1], LineBox[{246, 247}]}, 
                    {Hue[0.37591322907590524`, 1, 1], LineBox[{247, 248}]}, 
                    {Hue[0.6574196870928151, 1, 1], LineBox[{248, 249}]}, 
                    {Hue[0.9305902035811251, 1, 1], LineBox[{249, 250}]}, 
                    {Hue[0.20010644558572172`, 1, 1], LineBox[{250, 251}]}}}},
                    
                    VertexColors->{
                    Hue[0.5134986713962597, 1, 1], 
                    Hue[0.429675798262789, 1, 1], 
                    Hue[0.36537759142896803`, 1, 1], 
                    Hue[0.287489306520422, 1, 1], 
                    Hue[0.11714250593167995`, 1, 1], 
                    Hue[0.030302165263602618`, 1, 1], 
                    Hue[0.0651831562564822, 1, 1], 
                    Hue[0.03005619215611696, 1, 1], 
                    Hue[0.8086350452323195, 1, 1], 
                    Hue[0.5538481087637268, 1, 1], 
                    Hue[0.5327463853830385, 1, 1], 
                    Hue[0.3282688019815088, 1, 1], 
                    Hue[0.2199518739415025, 1, 1], 
                    Hue[0.22297225551656205`, 1, 1], 
                    Hue[0.08319886681240779, 1, 1], 
                    Hue[0.059918982847335045`, 1, 1], 
                    Hue[0.9702160847775785, 1, 1], 
                    Hue[0.7991004302923844, 1, 1], 
                    Hue[0.644117418730759, 1, 1], 
                    Hue[0.6356139264194032, 1, 1], 
                    Hue[0.5268321040319055, 1, 1], 
                    Hue[0.4849149904511286, 1, 1], 
                    Hue[0.2951625421267708, 1, 1], 
                    Hue[0.17072137554429173`, 1, 1], 
                    Hue[0.8425020370839564, 1, 1], 
                    Hue[0.5623136607528382, 1, 1], 
                    Hue[0.3847582675161996, 1, 1], 
                    Hue[0.6570983483750655, 1, 1], 
                    Hue[0.41201678385957174`, 1, 1], 
                    Hue[0.35912242361652047`, 1, 1], 
                    Hue[0.18101548966175515`, 1, 1], 
                    Hue[0.16450131136982224`, 1, 1], 
                    Hue[0.9480130500106693, 1, 1], 
                    Hue[0.7136847614331843, 1, 1], 
                    Hue[0.6820379269327059, 1, 1], 
                    Hue[0.5438279527273757, 1, 1], 
                    Hue[0.4815860989652991, 1, 1], 
                    Hue[0.4540529949539039, 1, 1], 
                    Hue[0.588981116835043, 1, 1], 
                    Hue[0.44981566460748473`, 1, 1], 
                    Hue[0.1962791259326371, 1, 1], 
                    Hue[0.5211218777608584, 1, 1], 
                    Hue[0.47547988701683014`, 1, 1], 
                    Hue[0.7495477847379686, 1, 1], 
                    Hue[0.6681734265858652, 1, 1], 
                    Hue[0.5077961245052601, 1, 1], 
                    Hue[0.6866704623527522, 1, 1], 
                    Hue[0.6452365448802283, 1, 1], 
                    Hue[0.8005378726770057, 1, 1], 
                    Hue[0.8643750617374688, 1, 1], 
                    Hue[0.6693497106381865, 1, 1], 
                    Hue[0.9575525251427208, 1, 1], 
                    Hue[0.22592667680464848`, 1, 1], 
                    Hue[0.2661863141424179, 1, 1], 
                    Hue[0.22840163982044662`, 1, 1], 
                    Hue[0.31215738198099585`, 1, 1], 
                    Hue[0.31477941715247937`, 1, 1], 
                    Hue[0.39024845611634024`, 1, 1], 
                    Hue[0.47248760524899003`, 1, 1], 
                    Hue[0.5211974196257148, 1, 1], 
                    Hue[0.5636352335907608, 1, 1], 
                    Hue[0.6027404106171735, 1, 1], 
                    Hue[0.6707034118133188, 1, 1], 
                    Hue[0.7327249096922388, 1, 1], 
                    Hue[0.7986624195234826, 1, 1], 
                    Hue[0.855887994596807, 1, 1], 
                    Hue[0.9095456426708814, 1, 1], 
                    Hue[0.970077867929754, 1, 1], 
                    Hue[0.036075328277316174`, 1, 1], 
                    Hue[0.0986472384956598, 1, 1], 
                    Hue[0.16258323239669467`, 1, 1], 
                    Hue[0.22439962875667932`, 1, 1], 
                    Hue[0.2868228301934304, 1, 1], 
                    Hue[0.3506068304944729, 1, 1], 
                    Hue[0.4151345914868187, 1, 1], 
                    Hue[0.4806693457773988, 1, 1], 
                    Hue[0.5460311877138803, 1, 1], 
                    Hue[0.6120088180229494, 1, 1], 
                    Hue[0.6780807385652687, 1, 1], 
                    Hue[0.7447573890440767, 1, 1], 
                    Hue[0.8124943399697049, 1, 1], 
                    Hue[0.8807725622213715, 1, 1], 
                    Hue[0.9493408685771565, 1, 1], 
                    Hue[0.018181533898883856`, 1, 1], 
                    Hue[0.08740416222963696, 1, 1], 
                    Hue[0.15726713787309227`, 1, 1], 
                    Hue[0.22793403650425142`, 1, 1], 
                    Hue[0.2992108352550998, 1, 1], 
                    Hue[0.37080822391280877`, 1, 1], 
                    Hue[0.44261246362794693`, 1, 1], 
                    Hue[0.5147944799691418, 1, 1], 
                    Hue[0.5876513479851337, 1, 1], 
                    Hue[0.6613124979945698, 1, 1], 
                    Hue[0.7355536948539427, 1, 1], 
                    Hue[0.8100331941773403, 1, 1], 
                    Hue[0.8845760918967533, 1, 1], 
                    Hue[0.9594392237794754, 1, 1], 
                    Hue[0.035026619894552934`, 1, 1], 
                    Hue[0.11148126313743942`, 1, 1], 
                    Hue[0.18846222297838114`, 1, 1], 
                    Hue[0.2654320874095458, 1, 1], 
                    Hue[0.3422347714240084, 1, 1], 
                    Hue[0.4192481284631285, 1, 1], 
                    Hue[0.4970426128281789, 1, 1], 
                    Hue[0.5757258532795645, 1, 1], 
                    Hue[0.6547295590250144, 1, 1], 
                    Hue[0.7333012536351259, 1, 1], 
                    Hue[0.8112505621189473, 1, 1], 
                    Hue[0.8891688664450038, 1, 1], 
                    Hue[0.9678089829795671, 1, 1], 
                    Hue[0.04724536239282264, 1, 1], 
                    Hue[0.12659362795393675`, 1, 1], 
                    Hue[0.2047569751445727, 1, 1], 
                    Hue[0.2814529462598848, 1, 1], 
                    Hue[0.3574199080216351, 1, 1], 
                    Hue[0.4336652855048778, 1, 1], 
                    Hue[0.5104069744697515, 1, 1], 
                    Hue[0.5866138643940774, 1, 1], 
                    Hue[0.6607219582217274, 1, 1], 
                    Hue[0.7319085375689991, 1, 1], 
                    Hue[0.800597168484313, 1, 1], 
                    Hue[0.8679469213743725, 1, 1], 
                    Hue[0.9349223090369303, 1, 1], 
                    Hue[0.001510620429620076, 1, 1], 
                    Hue[0.06652459152823118, 1, 1], 
                    Hue[0.1282599143269595, 1, 1], 
                    Hue[0.18550179423443017`, 1, 1], 
                    Hue[0.23800735641251244`, 1, 1], 
                    Hue[0.2862968986993083, 1, 1], 
                    Hue[0.33123280398036214`, 1, 1], 
                    Hue[0.3737126292032006, 1, 1], 
                    Hue[0.41453560107924475`, 1, 1], 
                    Hue[0.4543646210431929, 1, 1], 
                    Hue[0.49373324008370056`, 1, 1], 
                    Hue[0.5330669882843551, 1, 1], 
                    Hue[0.5727040733701112, 1, 1], 
                    Hue[0.6129127180864538, 1, 1], 
                    Hue[0.6539059380017792, 1, 1], 
                    Hue[0.6958512749606287, 1, 1], 
                    Hue[0.7388802296152222, 1, 1], 
                    Hue[0.7830946197868673, 1, 1], 
                    Hue[0.8285721896317425, 1, 1], 
                    Hue[0.8753695061192841, 1, 1], 
                    Hue[0.923527316882723, 1, 1], 
                    Hue[0.9730728782416933, 1, 1], 
                    Hue[0.024022163083270768`, 1, 1], 
                    Hue[0.07638221788367007, 1, 1], 
                    Hue[0.13015318159727385`, 1, 1], 
                    Hue[0.1853285701027619, 1, 1], 
                    Hue[0.24189914564441517`, 1, 1], 
                    Hue[0.2998500303351519, 1, 1], 
                    Hue[0.35916544194696576`, 1, 1], 
                    Hue[0.4198266093930038, 1, 1], 
                    Hue[0.4818143337854014, 1, 1], 
                    Hue[0.5451075206194197, 1, 1], 
                    Hue[0.6096848949214557, 1, 1], 
                    Hue[0.6755247680557995, 1, 1], 
                    Hue[0.7426074280870318, 1, 1], 
                    Hue[0.8109106936417033, 1, 1], 
                    Hue[0.8804129310143901, 1, 1], 
                    Hue[0.951093924998524, 1, 1], 
                    Hue[0.022934143089617415`, 1, 1], 
                    Hue[0.09591393913800811, 1, 1], 
                    Hue[0.17001339104666313`, 1, 1], 
                    Hue[0.2452148794921656, 1, 1], 
                    Hue[0.3215035907254118, 1, 1], 
                    Hue[0.3988570522565961, 1, 1], 
                    Hue[0.4772609021301015, 1, 1], 
                    Hue[0.5567029416512079, 1, 1], 
                    Hue[0.6371604032186986, 1, 1], 
                    Hue[0.7186211088714941, 1, 1], 
                    Hue[0.8010744330456714, 1, 1], 
                    Hue[0.8845040486856267, 1, 1], 
                    Hue[0.9688939180842179, 1, 1], 
                    Hue[0.05423573609039471, 1, 1], 
                    Hue[0.14051347720991264`, 1, 1], 
                    Hue[0.22770374758516046`, 1, 1], 
                    Hue[0.3158165858776364, 1, 1], 
                    Hue[0.40485117363829604`, 1, 1], 
                    Hue[0.4947574943312316, 1, 1], 
                    Hue[0.5855326413478238, 1, 1], 
                    Hue[0.6772189855349321, 1, 1], 
                    Hue[0.76975196888853, 1, 1], 
                    Hue[0.863080250307886, 1, 1], 
                    Hue[0.95731350477976, 1, 1], 
                    Hue[0.052435140580638674`, 1, 1], 
                    Hue[0.14829772018432766`, 1, 1], 
                    Hue[0.24497580630907112`, 1, 1], 
                    Hue[0.34257966189593014`, 1, 1], 
                    Hue[0.44099189618055484`, 1, 1], 
                    Hue[0.5401359020862424, 1, 1], 
                    Hue[0.6401264403994193, 1, 1], 
                    Hue[0.7409793231976449, 1, 1], 
                    Hue[0.842431620268548, 1, 1], 
                    Hue[0.9444064645043521, 1, 1], 
                    Hue[0.04739874473607524, 1, 1], 
                    Hue[0.15160986904984824`, 1, 1], 
                    Hue[0.2562192662631057, 1, 1], 
                    Hue[0.36062671712047917`, 1, 1], 
                    Hue[0.46586838643294537`, 1, 1], 
                    Hue[0.5731461208010898, 1, 1], 
                    Hue[0.680934766329103, 1, 1], 
                    Hue[0.7865115346926551, 1, 1], 
                    Hue[0.8927582528892654, 1, 1], 
                    Hue[0.00683152188586909, 1, 1], 
                    Hue[0.12756989049417516`, 1, 1], 
                    Hue[0.24415429036566494`, 1, 1], 
                    Hue[0.3490163234031179, 1, 1], 
                    Hue[0.45444314884782067`, 1, 1], 
                    Hue[0.5747967080651628, 1, 1], 
                    Hue[0.7022712739746917, 1, 1], 
                    Hue[0.8051517666191494, 1, 1], 
                    Hue[0.8245393293497276, 1, 1], 
                    Hue[0.8981395757716686, 1, 1], 
                    Hue[0.043647678620711894`, 1, 1], 
                    Hue[0.1952359421759699, 1, 1], 
                    Hue[0.2979846970988019, 1, 1], 
                    Hue[0.3842789352023804, 1, 1], 
                    Hue[0.5490636286686705, 1, 1], 
                    Hue[0.7531878470074891, 1, 1], 
                    Hue[0.9741316560530584, 1, 1], 
                    Hue[0.21069204938267144`, 1, 1], 
                    Hue[0.5179625464986409, 1, 1], 
                    Hue[0.9515960101338363, 1, 1], 
                    Hue[0.21823819534412597`, 1, 1], 
                    Hue[0.46127061830899735`, 1, 1], 
                    Hue[0.7122031613185107, 1, 1], 
                    Hue[0.9728813343268621, 1, 1], 
                    Hue[0.2420268926362587, 1, 1], 
                    Hue[0.5202779494326837, 1, 1], 
                    Hue[0.8127800983269537, 1, 1], 
                    Hue[0.13233314843452446`, 1, 1], 
                    Hue[0.48454859416905616`, 1, 1], 
                    Hue[0.8231200751796759, 1, 1], 
                    Hue[0.11904642996065473`, 1, 1], 
                    Hue[0.3777046432339246, 1, 1], 
                    Hue[0.5843606419824321, 1, 1], 
                    Hue[0.7304586385055036, 1, 1], 
                    Hue[0.9181566980435721, 1, 1], 
                    Hue[0.15404779540900293`, 1, 1], 
                    Hue[0.409814915165141, 1, 1], 
                    Hue[0.6769779427689762, 1, 1], 
                    Hue[0.9577656899112758, 1, 1], 
                    Hue[0.2723615776834439, 1, 1], 
                    Hue[0.6820614538136163, 1, 1], 
                    Hue[0.06994172851893946, 1, 1], 
                    Hue[0.37591322907590524`, 1, 1], 
                    Hue[0.6574196870928151, 1, 1], 
                    Hue[0.9305902035811251, 1, 1], 
                    Hue[0.20010644558572172`, 1, 1], 
                    Hue[0.4677155460079821, 1, 1], 
                    Hue[0.5134986713962597, 1, 1], 
                    Hue[0.429675798262789, 1, 1], 
                    Hue[0.429675798262789, 1, 1], 
                    Hue[0.36537759142896803`, 1, 1], 
                    Hue[0.36537759142896803`, 1, 1], 
                    Hue[0.287489306520422, 1, 1], 
                    Hue[0.287489306520422, 1, 1], 
                    Hue[0.11714250593167995`, 1, 1], 
                    Hue[0.11714250593167995`, 1, 1], 
                    Hue[0.030302165263602618`, 1, 1], 
                    Hue[0.030302165263602618`, 1, 1], 
                    Hue[0.0651831562564822, 1, 1], 
                    Hue[0.0651831562564822, 1, 1], 
                    Hue[0.03005619215611696, 1, 1], 
                    Hue[0.03005619215611696, 1, 1], 
                    Hue[0.8086350452323195, 1, 1], 
                    Hue[0.8086350452323195, 1, 1], 
                    Hue[0.5538481087637268, 1, 1], 
                    Hue[0.5538481087637268, 1, 1], 
                    Hue[0.5327463853830385, 1, 1], 
                    Hue[0.5327463853830385, 1, 1], 
                    Hue[0.3282688019815088, 1, 1], 
                    Hue[0.3282688019815088, 1, 1], 
                    Hue[0.2199518739415025, 1, 1], 
                    Hue[0.2199518739415025, 1, 1], 
                    Hue[0.22297225551656205`, 1, 1], 
                    Hue[0.22297225551656205`, 1, 1], 
                    Hue[0.08319886681240779, 1, 1], 
                    Hue[0.08319886681240779, 1, 1], 
                    Hue[0.059918982847335045`, 1, 1], 
                    Hue[0.059918982847335045`, 1, 1], 
                    Hue[0.9702160847775785, 1, 1], 
                    Hue[0.9702160847775785, 1, 1], 
                    Hue[0.7991004302923844, 1, 1], 
                    Hue[0.7991004302923844, 1, 1], 
                    Hue[0.644117418730759, 1, 1], 
                    Hue[0.644117418730759, 1, 1], 
                    Hue[0.6356139264194032, 1, 1], 
                    Hue[0.6356139264194032, 1, 1], 
                    Hue[0.5268321040319055, 1, 1], 
                    Hue[0.5268321040319055, 1, 1], 
                    Hue[0.4849149904511286, 1, 1], 
                    Hue[0.4849149904511286, 1, 1], 
                    Hue[0.2951625421267708, 1, 1], 
                    Hue[0.2951625421267708, 1, 1], 
                    Hue[0.17072137554429173`, 1, 1], 
                    Hue[0.17072137554429173`, 1, 1], 
                    Hue[0.8425020370839564, 1, 1], 
                    Hue[0.8425020370839564, 1, 1], 
                    Hue[0.5623136607528382, 1, 1], 
                    Hue[0.5623136607528382, 1, 1], 
                    Hue[0.3847582675161996, 1, 1], 
                    Hue[0.3847582675161996, 1, 1], 
                    Hue[0.6570983483750655, 1, 1], 
                    Hue[0.6570983483750655, 1, 1], 
                    Hue[0.41201678385957174`, 1, 1], 
                    Hue[0.41201678385957174`, 1, 1], 
                    Hue[0.35912242361652047`, 1, 1], 
                    Hue[0.35912242361652047`, 1, 1], 
                    Hue[0.18101548966175515`, 1, 1], 
                    Hue[0.18101548966175515`, 1, 1], 
                    Hue[0.16450131136982224`, 1, 1], 
                    Hue[0.16450131136982224`, 1, 1], 
                    Hue[0.9480130500106693, 1, 1], 
                    Hue[0.9480130500106693, 1, 1], 
                    Hue[0.7136847614331843, 1, 1], 
                    Hue[0.7136847614331843, 1, 1], 
                    Hue[0.6820379269327059, 1, 1], 
                    Hue[0.6820379269327059, 1, 1], 
                    Hue[0.5438279527273757, 1, 1], 
                    Hue[0.5438279527273757, 1, 1], 
                    Hue[0.4815860989652991, 1, 1], 
                    Hue[0.4815860989652991, 1, 1], 
                    Hue[0.4540529949539039, 1, 1], 
                    Hue[0.4540529949539039, 1, 1], 
                    Hue[0.588981116835043, 1, 1], 
                    Hue[0.588981116835043, 1, 1], 
                    Hue[0.44981566460748473`, 1, 1], 
                    Hue[0.44981566460748473`, 1, 1], 
                    Hue[0.1962791259326371, 1, 1], 
                    Hue[0.1962791259326371, 1, 1], 
                    Hue[0.5211218777608584, 1, 1], 
                    Hue[0.5211218777608584, 1, 1], 
                    Hue[0.47547988701683014`, 1, 1], 
                    Hue[0.47547988701683014`, 1, 1], 
                    Hue[0.7495477847379686, 1, 1], 
                    Hue[0.7495477847379686, 1, 1], 
                    Hue[0.6681734265858652, 1, 1], 
                    Hue[0.6681734265858652, 1, 1], 
                    Hue[0.5077961245052601, 1, 1], 
                    Hue[0.5077961245052601, 1, 1], 
                    Hue[0.6866704623527522, 1, 1], 
                    Hue[0.6866704623527522, 1, 1], 
                    Hue[0.6452365448802283, 1, 1], 
                    Hue[0.6452365448802283, 1, 1], 
                    Hue[0.8005378726770057, 1, 1], 
                    Hue[0.8005378726770057, 1, 1], 
                    Hue[0.8643750617374688, 1, 1], 
                    Hue[0.8643750617374688, 1, 1], 
                    Hue[0.6693497106381865, 1, 1], 
                    Hue[0.6693497106381865, 1, 1], 
                    Hue[0.9575525251427208, 1, 1], 
                    Hue[0.9575525251427208, 1, 1], 
                    Hue[0.22592667680464848`, 1, 1], 
                    Hue[0.22592667680464848`, 1, 1], 
                    Hue[0.2661863141424179, 1, 1], 
                    Hue[0.2661863141424179, 1, 1], 
                    Hue[0.22840163982044662`, 1, 1], 
                    Hue[0.22840163982044662`, 1, 1], 
                    Hue[0.31215738198099585`, 1, 1], 
                    Hue[0.31215738198099585`, 1, 1], 
                    Hue[0.31477941715247937`, 1, 1], 
                    Hue[0.31477941715247937`, 1, 1], 
                    Hue[0.39024845611634024`, 1, 1], 
                    Hue[0.39024845611634024`, 1, 1], 
                    Hue[0.47248760524899003`, 1, 1], 
                    Hue[0.47248760524899003`, 1, 1], 
                    Hue[0.5211974196257148, 1, 1], 
                    Hue[0.5211974196257148, 1, 1], 
                    Hue[0.5636352335907608, 1, 1], 
                    Hue[0.5636352335907608, 1, 1], 
                    Hue[0.6027404106171735, 1, 1], 
                    Hue[0.6027404106171735, 1, 1], 
                    Hue[0.6707034118133188, 1, 1], 
                    Hue[0.6707034118133188, 1, 1], 
                    Hue[0.7327249096922388, 1, 1], 
                    Hue[0.7327249096922388, 1, 1], 
                    Hue[0.7986624195234826, 1, 1], 
                    Hue[0.7986624195234826, 1, 1], 
                    Hue[0.855887994596807, 1, 1], 
                    Hue[0.855887994596807, 1, 1], 
                    Hue[0.9095456426708814, 1, 1], 
                    Hue[0.9095456426708814, 1, 1], 
                    Hue[0.970077867929754, 1, 1], 
                    Hue[0.970077867929754, 1, 1], 
                    Hue[0.036075328277316174`, 1, 1], 
                    Hue[0.036075328277316174`, 1, 1], 
                    Hue[0.0986472384956598, 1, 1], 
                    Hue[0.0986472384956598, 1, 1], 
                    Hue[0.16258323239669467`, 1, 1], 
                    Hue[0.16258323239669467`, 1, 1], 
                    Hue[0.22439962875667932`, 1, 1], 
                    Hue[0.22439962875667932`, 1, 1], 
                    Hue[0.2868228301934304, 1, 1], 
                    Hue[0.2868228301934304, 1, 1], 
                    Hue[0.3506068304944729, 1, 1], 
                    Hue[0.3506068304944729, 1, 1], 
                    Hue[0.4151345914868187, 1, 1], 
                    Hue[0.4151345914868187, 1, 1], 
                    Hue[0.4806693457773988, 1, 1], 
                    Hue[0.4806693457773988, 1, 1], 
                    Hue[0.5460311877138803, 1, 1], 
                    Hue[0.5460311877138803, 1, 1], 
                    Hue[0.6120088180229494, 1, 1], 
                    Hue[0.6120088180229494, 1, 1], 
                    Hue[0.6780807385652687, 1, 1], 
                    Hue[0.6780807385652687, 1, 1], 
                    Hue[0.7447573890440767, 1, 1], 
                    Hue[0.7447573890440767, 1, 1], 
                    Hue[0.8124943399697049, 1, 1], 
                    Hue[0.8124943399697049, 1, 1], 
                    Hue[0.8807725622213715, 1, 1], 
                    Hue[0.8807725622213715, 1, 1], 
                    Hue[0.9493408685771565, 1, 1], 
                    Hue[0.9493408685771565, 1, 1], 
                    Hue[0.018181533898883856`, 1, 1], 
                    Hue[0.018181533898883856`, 1, 1], 
                    Hue[0.08740416222963696, 1, 1], 
                    Hue[0.08740416222963696, 1, 1], 
                    Hue[0.15726713787309227`, 1, 1], 
                    Hue[0.15726713787309227`, 1, 1], 
                    Hue[0.22793403650425142`, 1, 1], 
                    Hue[0.22793403650425142`, 1, 1], 
                    Hue[0.2992108352550998, 1, 1], 
                    Hue[0.2992108352550998, 1, 1], 
                    Hue[0.37080822391280877`, 1, 1], 
                    Hue[0.37080822391280877`, 1, 1], 
                    Hue[0.44261246362794693`, 1, 1], 
                    Hue[0.44261246362794693`, 1, 1], 
                    Hue[0.5147944799691418, 1, 1], 
                    Hue[0.5147944799691418, 1, 1], 
                    Hue[0.5876513479851337, 1, 1], 
                    Hue[0.5876513479851337, 1, 1], 
                    Hue[0.6613124979945698, 1, 1], 
                    Hue[0.6613124979945698, 1, 1], 
                    Hue[0.7355536948539427, 1, 1], 
                    Hue[0.7355536948539427, 1, 1], 
                    Hue[0.8100331941773403, 1, 1], 
                    Hue[0.8100331941773403, 1, 1], 
                    Hue[0.8845760918967533, 1, 1], 
                    Hue[0.8845760918967533, 1, 1], 
                    Hue[0.9594392237794754, 1, 1], 
                    Hue[0.9594392237794754, 1, 1], 
                    Hue[0.035026619894552934`, 1, 1], 
                    Hue[0.035026619894552934`, 1, 1], 
                    Hue[0.11148126313743942`, 1, 1], 
                    Hue[0.11148126313743942`, 1, 1], 
                    Hue[0.18846222297838114`, 1, 1], 
                    Hue[0.18846222297838114`, 1, 1], 
                    Hue[0.2654320874095458, 1, 1], 
                    Hue[0.2654320874095458, 1, 1], 
                    Hue[0.3422347714240084, 1, 1], 
                    Hue[0.3422347714240084, 1, 1], 
                    Hue[0.4192481284631285, 1, 1], 
                    Hue[0.4192481284631285, 1, 1], 
                    Hue[0.4970426128281789, 1, 1], 
                    Hue[0.4970426128281789, 1, 1], 
                    Hue[0.5757258532795645, 1, 1], 
                    Hue[0.5757258532795645, 1, 1], 
                    Hue[0.6547295590250144, 1, 1], 
                    Hue[0.6547295590250144, 1, 1], 
                    Hue[0.7333012536351259, 1, 1], 
                    Hue[0.7333012536351259, 1, 1], 
                    Hue[0.8112505621189473, 1, 1], 
                    Hue[0.8112505621189473, 1, 1], 
                    Hue[0.8891688664450038, 1, 1], 
                    Hue[0.8891688664450038, 1, 1], 
                    Hue[0.9678089829795671, 1, 1], 
                    Hue[0.9678089829795671, 1, 1], 
                    Hue[0.04724536239282264, 1, 1], 
                    Hue[0.04724536239282264, 1, 1], 
                    Hue[0.12659362795393675`, 1, 1], 
                    Hue[0.12659362795393675`, 1, 1], 
                    Hue[0.2047569751445727, 1, 1], 
                    Hue[0.2047569751445727, 1, 1], 
                    Hue[0.2814529462598848, 1, 1], 
                    Hue[0.2814529462598848, 1, 1], 
                    Hue[0.3574199080216351, 1, 1], 
                    Hue[0.3574199080216351, 1, 1], 
                    Hue[0.4336652855048778, 1, 1], 
                    Hue[0.4336652855048778, 1, 1], 
                    Hue[0.5104069744697515, 1, 1], 
                    Hue[0.5104069744697515, 1, 1], 
                    Hue[0.5866138643940774, 1, 1], 
                    Hue[0.5866138643940774, 1, 1], 
                    Hue[0.6607219582217274, 1, 1], 
                    Hue[0.6607219582217274, 1, 1], 
                    Hue[0.7319085375689991, 1, 1], 
                    Hue[0.7319085375689991, 1, 1], 
                    Hue[0.800597168484313, 1, 1], 
                    Hue[0.800597168484313, 1, 1], 
                    Hue[0.8679469213743725, 1, 1], 
                    Hue[0.8679469213743725, 1, 1], 
                    Hue[0.9349223090369303, 1, 1], 
                    Hue[0.9349223090369303, 1, 1], 
                    Hue[0.001510620429620076, 1, 1], 
                    Hue[0.001510620429620076, 1, 1], 
                    Hue[0.06652459152823118, 1, 1], 
                    Hue[0.06652459152823118, 1, 1], 
                    Hue[0.1282599143269595, 1, 1], 
                    Hue[0.1282599143269595, 1, 1], 
                    Hue[0.18550179423443017`, 1, 1], 
                    Hue[0.18550179423443017`, 1, 1], 
                    Hue[0.23800735641251244`, 1, 1], 
                    Hue[0.23800735641251244`, 1, 1], 
                    Hue[0.2862968986993083, 1, 1], 
                    Hue[0.2862968986993083, 1, 1], 
                    Hue[0.33123280398036214`, 1, 1], 
                    Hue[0.33123280398036214`, 1, 1], 
                    Hue[0.3737126292032006, 1, 1], 
                    Hue[0.3737126292032006, 1, 1], 
                    Hue[0.41453560107924475`, 1, 1], 
                    Hue[0.41453560107924475`, 1, 1], 
                    Hue[0.4543646210431929, 1, 1], 
                    Hue[0.4543646210431929, 1, 1], 
                    Hue[0.49373324008370056`, 1, 1], 
                    Hue[0.49373324008370056`, 1, 1], 
                    Hue[0.5330669882843551, 1, 1], 
                    Hue[0.5330669882843551, 1, 1], 
                    Hue[0.5727040733701112, 1, 1], 
                    Hue[0.5727040733701112, 1, 1], 
                    Hue[0.6129127180864538, 1, 1], 
                    Hue[0.6129127180864538, 1, 1], 
                    Hue[0.6539059380017792, 1, 1], 
                    Hue[0.6539059380017792, 1, 1], 
                    Hue[0.6958512749606287, 1, 1], 
                    Hue[0.6958512749606287, 1, 1], 
                    Hue[0.7388802296152222, 1, 1], 
                    Hue[0.7388802296152222, 1, 1], 
                    Hue[0.7830946197868673, 1, 1], 
                    Hue[0.7830946197868673, 1, 1], 
                    Hue[0.8285721896317425, 1, 1], 
                    Hue[0.8285721896317425, 1, 1], 
                    Hue[0.8753695061192841, 1, 1], 
                    Hue[0.8753695061192841, 1, 1], 
                    Hue[0.923527316882723, 1, 1], 
                    Hue[0.923527316882723, 1, 1], 
                    Hue[0.9730728782416933, 1, 1], 
                    Hue[0.9730728782416933, 1, 1], 
                    Hue[0.024022163083270768`, 1, 1], 
                    Hue[0.024022163083270768`, 1, 1], 
                    Hue[0.07638221788367007, 1, 1], 
                    Hue[0.07638221788367007, 1, 1], 
                    Hue[0.13015318159727385`, 1, 1], 
                    Hue[0.13015318159727385`, 1, 1], 
                    Hue[0.1853285701027619, 1, 1], 
                    Hue[0.1853285701027619, 1, 1], 
                    Hue[0.24189914564441517`, 1, 1], 
                    Hue[0.24189914564441517`, 1, 1], 
                    Hue[0.2998500303351519, 1, 1], 
                    Hue[0.2998500303351519, 1, 1], 
                    Hue[0.35916544194696576`, 1, 1], 
                    Hue[0.35916544194696576`, 1, 1], 
                    Hue[0.4198266093930038, 1, 1], 
                    Hue[0.4198266093930038, 1, 1], 
                    Hue[0.4818143337854014, 1, 1], 
                    Hue[0.4818143337854014, 1, 1], 
                    Hue[0.5451075206194197, 1, 1], 
                    Hue[0.5451075206194197, 1, 1], 
                    Hue[0.6096848949214557, 1, 1], 
                    Hue[0.6096848949214557, 1, 1], 
                    Hue[0.6755247680557995, 1, 1], 
                    Hue[0.6755247680557995, 1, 1], 
                    Hue[0.7426074280870318, 1, 1], 
                    Hue[0.7426074280870318, 1, 1], 
                    Hue[0.8109106936417033, 1, 1], 
                    Hue[0.8109106936417033, 1, 1], 
                    Hue[0.8804129310143901, 1, 1], 
                    Hue[0.8804129310143901, 1, 1], 
                    Hue[0.951093924998524, 1, 1], 
                    Hue[0.951093924998524, 1, 1], 
                    Hue[0.022934143089617415`, 1, 1], 
                    Hue[0.022934143089617415`, 1, 1], 
                    Hue[0.09591393913800811, 1, 1], 
                    Hue[0.09591393913800811, 1, 1], 
                    Hue[0.17001339104666313`, 1, 1], 
                    Hue[0.17001339104666313`, 1, 1], 
                    Hue[0.2452148794921656, 1, 1], 
                    Hue[0.2452148794921656, 1, 1], 
                    Hue[0.3215035907254118, 1, 1], 
                    Hue[0.3215035907254118, 1, 1], 
                    Hue[0.3988570522565961, 1, 1], 
                    Hue[0.3988570522565961, 1, 1], 
                    Hue[0.4772609021301015, 1, 1], 
                    Hue[0.4772609021301015, 1, 1], 
                    Hue[0.5567029416512079, 1, 1], 
                    Hue[0.5567029416512079, 1, 1], 
                    Hue[0.6371604032186986, 1, 1], 
                    Hue[0.6371604032186986, 1, 1], 
                    Hue[0.7186211088714941, 1, 1], 
                    Hue[0.7186211088714941, 1, 1], 
                    Hue[0.8010744330456714, 1, 1], 
                    Hue[0.8010744330456714, 1, 1], 
                    Hue[0.8845040486856267, 1, 1], 
                    Hue[0.8845040486856267, 1, 1], 
                    Hue[0.9688939180842179, 1, 1], 
                    Hue[0.9688939180842179, 1, 1], 
                    Hue[0.05423573609039471, 1, 1], 
                    Hue[0.05423573609039471, 1, 1], 
                    Hue[0.14051347720991264`, 1, 1], 
                    Hue[0.14051347720991264`, 1, 1], 
                    Hue[0.22770374758516046`, 1, 1], 
                    Hue[0.22770374758516046`, 1, 1], 
                    Hue[0.3158165858776364, 1, 1], 
                    Hue[0.3158165858776364, 1, 1], 
                    Hue[0.40485117363829604`, 1, 1], 
                    Hue[0.40485117363829604`, 1, 1], 
                    Hue[0.4947574943312316, 1, 1], 
                    Hue[0.4947574943312316, 1, 1], 
                    Hue[0.5855326413478238, 1, 1], 
                    Hue[0.5855326413478238, 1, 1], 
                    Hue[0.6772189855349321, 1, 1], 
                    Hue[0.6772189855349321, 1, 1], 
                    Hue[0.76975196888853, 1, 1], 
                    Hue[0.76975196888853, 1, 1], 
                    Hue[0.863080250307886, 1, 1], 
                    Hue[0.863080250307886, 1, 1], 
                    Hue[0.95731350477976, 1, 1], 
                    Hue[0.95731350477976, 1, 1], 
                    Hue[0.052435140580638674`, 1, 1], 
                    Hue[0.052435140580638674`, 1, 1], 
                    Hue[0.14829772018432766`, 1, 1], 
                    Hue[0.14829772018432766`, 1, 1], 
                    Hue[0.24497580630907112`, 1, 1], 
                    Hue[0.24497580630907112`, 1, 1], 
                    Hue[0.34257966189593014`, 1, 1], 
                    Hue[0.34257966189593014`, 1, 1], 
                    Hue[0.44099189618055484`, 1, 1], 
                    Hue[0.44099189618055484`, 1, 1], 
                    Hue[0.5401359020862424, 1, 1], 
                    Hue[0.5401359020862424, 1, 1], 
                    Hue[0.6401264403994193, 1, 1], 
                    Hue[0.6401264403994193, 1, 1], 
                    Hue[0.7409793231976449, 1, 1], 
                    Hue[0.7409793231976449, 1, 1], 
                    Hue[0.842431620268548, 1, 1], 
                    Hue[0.842431620268548, 1, 1], 
                    Hue[0.9444064645043521, 1, 1], 
                    Hue[0.9444064645043521, 1, 1], 
                    Hue[0.04739874473607524, 1, 1], 
                    Hue[0.04739874473607524, 1, 1], 
                    Hue[0.15160986904984824`, 1, 1], 
                    Hue[0.15160986904984824`, 1, 1], 
                    Hue[0.2562192662631057, 1, 1], 
                    Hue[0.2562192662631057, 1, 1], 
                    Hue[0.36062671712047917`, 1, 1], 
                    Hue[0.36062671712047917`, 1, 1], 
                    Hue[0.46586838643294537`, 1, 1], 
                    Hue[0.46586838643294537`, 1, 1], 
                    Hue[0.5731461208010898, 1, 1], 
                    Hue[0.5731461208010898, 1, 1], 
                    Hue[0.680934766329103, 1, 1], 
                    Hue[0.680934766329103, 1, 1], 
                    Hue[0.7865115346926551, 1, 1], 
                    Hue[0.7865115346926551, 1, 1], 
                    Hue[0.8927582528892654, 1, 1], 
                    Hue[0.8927582528892654, 1, 1], 
                    Hue[0.00683152188586909, 1, 1], 
                    Hue[0.00683152188586909, 1, 1], 
                    Hue[0.12756989049417516`, 1, 1], 
                    Hue[0.12756989049417516`, 1, 1], 
                    Hue[0.24415429036566494`, 1, 1], 
                    Hue[0.24415429036566494`, 1, 1], 
                    Hue[0.3490163234031179, 1, 1], 
                    Hue[0.3490163234031179, 1, 1], 
                    Hue[0.45444314884782067`, 1, 1], 
                    Hue[0.45444314884782067`, 1, 1], 
                    Hue[0.5747967080651628, 1, 1], 
                    Hue[0.5747967080651628, 1, 1], 
                    Hue[0.7022712739746917, 1, 1], 
                    Hue[0.7022712739746917, 1, 1], 
                    Hue[0.8051517666191494, 1, 1], 
                    Hue[0.8051517666191494, 1, 1], 
                    Hue[0.8245393293497276, 1, 1], 
                    Hue[0.8245393293497276, 1, 1], 
                    Hue[0.8981395757716686, 1, 1], 
                    Hue[0.8981395757716686, 1, 1], 
                    Hue[0.043647678620711894`, 1, 1], 
                    Hue[0.043647678620711894`, 1, 1], 
                    Hue[0.1952359421759699, 1, 1], 
                    Hue[0.1952359421759699, 1, 1], 
                    Hue[0.2979846970988019, 1, 1], 
                    Hue[0.2979846970988019, 1, 1], 
                    Hue[0.3842789352023804, 1, 1], 
                    Hue[0.3842789352023804, 1, 1], 
                    Hue[0.5490636286686705, 1, 1], 
                    Hue[0.5490636286686705, 1, 1], 
                    Hue[0.7531878470074891, 1, 1], 
                    Hue[0.7531878470074891, 1, 1], 
                    Hue[0.9741316560530584, 1, 1], 
                    Hue[0.9741316560530584, 1, 1], 
                    Hue[0.21069204938267144`, 1, 1], 
                    Hue[0.21069204938267144`, 1, 1], 
                    Hue[0.5179625464986409, 1, 1], 
                    Hue[0.5179625464986409, 1, 1], 
                    Hue[0.9515960101338363, 1, 1], 
                    Hue[0.9515960101338363, 1, 1], 
                    Hue[0.21823819534412597`, 1, 1], 
                    Hue[0.21823819534412597`, 1, 1], 
                    Hue[0.46127061830899735`, 1, 1], 
                    Hue[0.46127061830899735`, 1, 1], 
                    Hue[0.7122031613185107, 1, 1], 
                    Hue[0.7122031613185107, 1, 1], 
                    Hue[0.9728813343268621, 1, 1], 
                    Hue[0.9728813343268621, 1, 1], 
                    Hue[0.2420268926362587, 1, 1], 
                    Hue[0.2420268926362587, 1, 1], 
                    Hue[0.5202779494326837, 1, 1], 
                    Hue[0.5202779494326837, 1, 1], 
                    Hue[0.8127800983269537, 1, 1], 
                    Hue[0.8127800983269537, 1, 1], 
                    Hue[0.13233314843452446`, 1, 1], 
                    Hue[0.13233314843452446`, 1, 1], 
                    Hue[0.48454859416905616`, 1, 1], 
                    Hue[0.48454859416905616`, 1, 1], 
                    Hue[0.8231200751796759, 1, 1], 
                    Hue[0.8231200751796759, 1, 1], 
                    Hue[0.11904642996065473`, 1, 1], 
                    Hue[0.11904642996065473`, 1, 1], 
                    Hue[0.3777046432339246, 1, 1], 
                    Hue[0.3777046432339246, 1, 1], 
                    Hue[0.5843606419824321, 1, 1], 
                    Hue[0.5843606419824321, 1, 1], 
                    Hue[0.7304586385055036, 1, 1], 
                    Hue[0.7304586385055036, 1, 1], 
                    Hue[0.9181566980435721, 1, 1], 
                    Hue[0.9181566980435721, 1, 1], 
                    Hue[0.15404779540900293`, 1, 1], 
                    Hue[0.15404779540900293`, 1, 1], 
                    Hue[0.409814915165141, 1, 1], 
                    Hue[0.409814915165141, 1, 1], 
                    Hue[0.6769779427689762, 1, 1], 
                    Hue[0.6769779427689762, 1, 1], 
                    Hue[0.9577656899112758, 1, 1], 
                    Hue[0.9577656899112758, 1, 1], 
                    Hue[0.2723615776834439, 1, 1], 
                    Hue[0.2723615776834439, 1, 1], 
                    Hue[0.6820614538136163, 1, 1], 
                    Hue[0.6820614538136163, 1, 1], 
                    Hue[0.06994172851893946, 1, 1], 
                    Hue[0.06994172851893946, 1, 1], 
                    Hue[0.37591322907590524`, 1, 1], 
                    Hue[0.37591322907590524`, 1, 1], 
                    Hue[0.6574196870928151, 1, 1], 
                    Hue[0.6574196870928151, 1, 1], 
                    Hue[0.9305902035811251, 1, 1], 
                    Hue[0.9305902035811251, 1, 1], 
                    Hue[0.20010644558572172`, 1, 1], 
                    Hue[0.20010644558572172`, 1, 1], 
                    Hue[0.4677155460079821, 1, 1]}],
                   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
                   Axes->{True, None},
                   AxesOrigin->{0, 0},
                   
                   Epilog->LineBox[{{1, 1.293512231656321*^-6}, {
                    2, 9.141179925278823*^-7}, {3, 8.738486673240133*^-7}, {
                    4, 1.1210907591190583`*^-6}, {5, 9.07458607036266*^-7}, {
                    6, 4.335430178556219*^-7}, {7, 5.74511132360924*^-7}, {
                    8, 2.9722907071946993`*^-7}, {
                    9, 1.1114240457962253`*^-6}, {
                    10, 3.9462169867884443`*^-7}, {11, 6.72138609389269*^-7},
                     {12, 9.01247445382832*^-7}, {
                    13, 4.255413005141942*^-7}, {14, 8.722579684489437*^-7}, {
                    15, 8.593168312381147*^-7}, {16, 8.669063730412045*^-7}, {
                    17, 1.2595782450007813`*^-6}, {
                    18, 1.506617868651792*^-6}, {
                    19, 1.1757365608115608`*^-6}, {
                    20, 8.025743290499689*^-7}, {21, 6.732285678809414*^-7}, {
                    22, 7.758828188347845*^-7}, {23, 6.850954466267486*^-7}, {
                    24, 2.7640494086808906`*^-7}, {
                    25, 4.1610743188772513`*^-7}, {
                    26, 5.497172127152359*^-7}, {27, 3.509473577203337*^-7}, {
                    28, 3.1532358712482945`*^-7}, {
                    29, 3.1602394629538306`*^-7}, {
                    30, 4.802712349263791*^-7}, {31, 3.059337111566676*^-7}, {
                    32, 2.9289214899306654`*^-7}, {
                    33, 7.446363940210739*^-7}, {34, 4.781745861647592*^-7}, {
                    35, 6.884589351806089*^-7}, {36, 4.043057013188036*^-7}, {
                    37, 6.26413944214673*^-7}, {
                    38, 3.9242325745653125`*^-7}, {
                    39, 2.6984107852635816`*^-7}, {
                    40, 4.0388859979536897`*^-7}, {
                    41, 1.481126130275057*^-7}, {
                    42, 2.7924034872987627`*^-7}, {
                    43, 5.920943221166556*^-8}, {
                    44, 2.5574650870813133`*^-7}, {
                    45, 5.338061392273096*^-7}, {
                    46, 3.3157916898313786`*^-7}, {
                    47, 5.080266787759876*^-7}, {
                    48, 1.9576585161032869`*^-7}, {
                    49, 6.446274351345195*^-7}, {50, 9.117893641634066*^-7}, {
                    51, 5.641773057275045*^-7}, {
                    52, 1.4713599820993626`*^-7}, {
                    53, 9.276509431557103*^-7}, {54, 1.949581240149834*^-6}, {
                    55, 1.8429925869274515`*^-6}, {
                    56, 3.0501785922866223`*^-6}, {
                    57, 3.1801313669957977`*^-6}, {
                    58, 4.240223537517871*^-6}, {59, 5.770471339188001*^-6}, {
                    60, 7.806637939346476*^-6}, {
                    61, 0.000010836651825653717`}, {
                    62, 0.000013213601151066506`}, {
                    63, 0.00001573018688427932}, {
                    64, 0.0000194955168537409}, {
                    65, 0.000025278389727982726`}, {
                    66, 0.000034283650623131885`}, {
                    67, 0.000042413992302537525`}, {
                    68, 0.000052378440784301256`}, {
                    69, 0.00006514038828741289}, {
                    70, 0.00008276211956885553}, {
                    71, 0.00010436266510232029`}, {
                    72, 0.00013147205706328773`}, {
                    73, 0.0001644640094295517}, {
                    74, 0.00020369831563033585`}, {
                    75, 0.0002527719879515726}, {76, 0.0003120986993609501}, {
                    77, 0.0003859504945095197}, {78, 0.0004756812335648483}, {
                    79, 0.000583959707457682}, {80, 0.0007120971327799281}, {
                    81, 0.0008670458036575534}, {
                    82, 0.0010546593415718998`}, {
                    83, 0.0012807496458557576`}, {84, 0.001549727376410733}, {
                    85, 0.0018683441728276685`}, {
                    86, 0.0022410959858374887`}, {87, 0.002682757294860802}, {
                    88, 0.003206744685566178}, {89, 0.003827240618801887}, {
                    90, 0.00455353926527167}, {91, 0.005393639532321564}, {
                    92, 0.006360409592698273}, {93, 0.007483240821529209}, {
                    94, 0.008796005862820604}, {95, 0.010328902748770472`}, {
                    96, 0.012089150433735006`}, {97, 0.01407695310547073}, {
                    98, 0.01631329752193934}, {99, 0.01886838983075003}, {
                    100, 0.021829441161571064`}, {101, 0.0252472239761647}, {
                    102, 0.029092384280423406`}, {
                    103, 0.033316826391802966`}, {104, 0.03796235936310983}, {
                    105, 0.04321439645996892}, {106, 0.049296394577029806`}, {
                    107, 0.056266371961745026`}, {108, 0.06394939612262282}, {
                    109, 0.0721401259564591}, {110, 0.08094319415817058}, {
                    111, 0.09087443155879571}, {112, 0.10250964089336785`}, {
                    113, 0.11591118174746179`}, {114, 0.13049409990327585`}, {
                    115, 0.14561428723859315`}, {116, 0.16141534152146533`}, {
                    117, 0.17913859651728026`}, {118, 0.20036233552247132`}, {
                    119, 0.22563468200675418`}, {120, 0.2538608223489282}, {
                    121, 0.2831438247640217}, {122, 0.31245646125942195`}, {
                    123, 0.34289984266884477`}, {124, 0.3776610436133015}, {
                    125, 0.42039205520094763`}, {126, 0.4729304085167958}, {
                    127, 0.5339993839351291}, {128, 0.5996979492315827}, {
                    129, 0.6649636411211871}, {130, 0.7249393249452404}, {
                    131, 0.7758037070419624}, {132, 0.8150649936086384}, {
                    133, 0.8415434412900966}, {134, 0.8551543054082128}, {
                    135, 0.8566266116156382}, {136, 0.84722963235018}, {
                    137, 0.8285176908095345}, {138, 0.8021545435679595}, {
                    139, 0.7697728830438603}, {140, 0.732890414207474}, {
                    141, 0.6928638366334374}, {142, 0.6508648879340589}, {
                    143, 0.6078840960132798}, {144, 0.5647401608361832}, {
                    145, 0.5220934545210268}, {146, 0.4804649624244304}, {
                    147, 0.4402581863244774}, {148, 0.4017737863835495}, {
                    149, 0.36522951065326015`}, {150, 0.33076851972809906`}, {
                    151, 0.2984767440061327}, {152, 0.26839148065552443`}, {
                    153, 0.240511846373238}, {154, 0.21480407111506367`}, {
                    155, 0.1912106729030629}, {156, 0.1696543138854673}, {
                    157, 0.15004499743935715`}, {158, 0.13228013405326708`}, {
                    159, 0.11624986241032739`}, {160, 0.10184224373137564`}, {
                    161, 0.08894298053627564}, {162, 0.07743707595025123}, {
                    163, 0.06721160522174467}, {164, 0.05815722715814426}, {
                    165, 0.05016927570122365}, {166, 0.043146144449944426`}, {
                    167, 0.03699272744649877}, {168, 0.031621311760072705`}, {
                    169, 0.026947529670115163`}, {170, 0.02289472101164024}, {
                    171, 0.01939310538204206}, {172, 0.01637747784421382}, {
                    173, 0.013788899138288105`}, {
                    174, 0.011574518784269747`}, {
                    175, 0.009686638180006514}, {176, 0.008081974594870676}, {
                    177, 0.006723110362494051}, {178, 0.0055765664924465}, {
                    179, 0.004611307803165932}, {180, 0.003801297502792852}, {
                    181, 0.003124782105194667}, {182, 0.002561037097792583}, {
                    183, 0.0020920854609505655`}, {
                    184, 0.0017043262606185644`}, {
                    185, 0.001384830922755092}, {186, 0.001121349919165515}, {
                    187, 0.0009051277606995639}, {
                    188, 0.000728888605462127}, {
                    189, 0.0005852038490514287}, {
                    190, 0.000468143977313343}, {
                    191, 0.0003733955357207099}, {
                    192, 0.00029701716047071864`}, {
                    193, 0.00023536789815051942`}, {
                    194, 0.00018576725940270443`}, {
                    195, 0.00014636873569506386`}, {
                    196, 0.00011521799623727578`}, {
                    197, 0.0000902380920373945}, {
                    198, 0.00007015764286455641}, {
                    199, 0.00005446901393564296}, {
                    200, 0.000042464233711868196`}, {
                    201, 0.00003299611122747644}, {
                    202, 0.000025283501423243874`}, {
                    203, 0.000019365025353130492`}, {
                    204, 0.000015261501326867872`}, {
                    205, 0.000012149528911031253`}, {
                    206, 9.204816704419756*^-6}, {
                    207, 6.523031805848554*^-6}, {
                    208, 4.643962784735902*^-6}, {
                    209, 3.5683167936200154`*^-6}, {
                    210, 2.7322297961770383`*^-6}, {
                    211, 1.837123085155947*^-6}, {
                    212, 1.0056988926017879`*^-6}, {
                    213, 6.747753956907239*^-7}, {
                    214, 8.69852940691384*^-7}, {215, 9.22063894374567*^-7}, {
                    216, 7.188839891749446*^-7}, {
                    217, 4.904189636537491*^-7}, {
                    218, 5.405660028472834*^-7}, {
                    219, 6.534520250027231*^-7}, {
                    220, 6.180973262270588*^-7}, {
                    221, 4.5248445263946285`*^-7}, {
                    222, 2.478787272298718*^-7}, {
                    223, 9.296103639784852*^-8}, {
                    224, 1.0921799299772099`*^-7}, {
                    225, 2.0711755693726188`*^-7}, {
                    226, 3.081392505696856*^-7}, {
                    227, 3.923024023110412*^-7}, {
                    228, 4.3769496072188006`*^-7}, {
                    229, 4.312443204886432*^-7}, {
                    230, 3.747963886256413*^-7}, {
                    231, 2.866613694737511*^-7}, {
                    232, 2.005305727655022*^-7}, {
                    233, 1.5631262708631255`*^-7}, {
                    234, 1.5449683109040095`*^-7}, {
                    235, 1.5098429014051752`*^-7}, {
                    236, 1.2522275526235434`*^-7}, {
                    237, 8.995853327982884*^-8}, {
                    238, 8.55037910558022*^-8}, {
                    239, 1.1932707796824371`*^-7}, {
                    240, 1.521606743728176*^-7}, {
                    241, 1.647117895354992*^-7}, {
                    242, 1.5272485327266351`*^-7}, {
                    243, 1.1977216631568382`*^-7}, {
                    244, 7.664213250933791*^-8}, {
                    245, 5.466237583583313*^-8}, {
                    246, 8.715763702470234*^-8}, {
                    247, 1.3428755108958065`*^-7}, {
                    248, 1.744023835071272*^-7}, {
                    249, 2.0163074144323127`*^-7}, {
                    250, 2.1456332049311917`*^-7}, {
                    251, 2.1413213272558206`*^-7}}],
                   Frame->True,
                   FrameTicks->{{{1., 
                    FormBox[
                    RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
                    FormBox[
                    RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
                    FormBox[
                    RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
                    FormBox[
                    RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
                    FormBox["0", TraditionalForm]}, {157.25, 
                    FormBox["2.5`", TraditionalForm]}, {188.5, 
                    FormBox["5", TraditionalForm]}, {219.75, 
                    FormBox["7.5`", TraditionalForm]}, {251., 
                    FormBox["10", TraditionalForm]}}, Automatic, None, None},
                   PlotRange->{-0.1, 1.1},
                   PlotRangeClipping->True,
                   PlotRangePadding->{
                    Scaled[0.02], Automatic}], 12->
                  GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF13tcTekeBvCV0SCXiUM1hD1FQxMquTd5KilF6a6LWrV3u/tlmjiMwQop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                    "], {
                    {GrayLevel[0.5], EdgeForm[None], 
                    GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ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                    
                    "]]]}, {{}, {}, {
                    {Hue[0.10248796621842285`, 1, 1], LineBox[{1, 2}]}, 
                    {Hue[0.8290457167148891, 1, 1], LineBox[{2, 3}]}, 
                    {Hue[0.8703632125845846, 1, 1], LineBox[{3, 4}]}, 
                    {Hue[0.7393175820802007, 1, 1], LineBox[{4, 5}]}, 
                    {Hue[0.6032998790616205, 1, 1], LineBox[{5, 6}]}, 
                    {Hue[0.5291282883599169, 1, 1], LineBox[{6, 7}]}, 
                    {Hue[0.37551039951532045`, 1, 1], LineBox[{7, 8}]}, 
                    {Hue[0.22433259918415324`, 1, 1], LineBox[{8, 9}]}, 
                    {Hue[0.11852503188071434`, 1, 1], LineBox[{9, 10}]}, 
                    {Hue[0.07197136740119216, 1, 1], LineBox[{10, 11}]}, 
                    {Hue[0.027235479998395193`, 1, 1], LineBox[{11, 12}]}, 
                    {Hue[0.9072061553893599, 1, 1], LineBox[{12, 13}]}, 
                    {Hue[0.8405170901747704, 1, 1], LineBox[{13, 14}]}, 
                    {Hue[0.8075393950498422, 1, 1], LineBox[{14, 15}]}, 
                    {Hue[0.5331074488604866, 1, 1], LineBox[{15, 16}]}, 
                    {Hue[0.5358981991195568, 1, 1], LineBox[{16, 17}]}, 
                    {Hue[0.25004222875796983`, 1, 1], LineBox[{17, 18}]}, 
                    {Hue[0.7962568815322872, 1, 1], LineBox[{18, 19}]}, 
                    {Hue[0.8602643648636271, 1, 1], LineBox[{19, 20}]}, 
                    {Hue[0.7061829013619123, 1, 1], LineBox[{20, 21}]}, 
                    {Hue[0.5353893983812265, 1, 1], LineBox[{21, 22}]}, 
                    {Hue[0.5574883411356903, 1, 1], LineBox[{22, 23}]}, 
                    {Hue[0.3982347746773217, 1, 1], LineBox[{23, 24}]}, 
                    {Hue[0.376614336574381, 1, 1], LineBox[{24, 25}]}, 
                    {Hue[0.1848960953069641, 1, 1], LineBox[{25, 26}]}, 
                    {Hue[0.39778971853907324`, 1, 1], LineBox[{26, 27}]}, 
                    {Hue[0.09352019075476536, 1, 1], LineBox[{27, 28}]}, 
                    {Hue[0.783376412070276, 1, 1], LineBox[{28, 29}]}, 
                    {Hue[0.7824547112898794, 1, 1], LineBox[{29, 30}]}, 
                    {Hue[0.7761971219426127, 1, 1], LineBox[{30, 31}]}, 
                    {Hue[0.6038072405559677, 1, 1], LineBox[{31, 32}]}, 
                    {Hue[0.560930797116872, 1, 1], LineBox[{32, 33}]}, 
                    {Hue[0.33776760760892577`, 1, 1], LineBox[{33, 34}]}, 
                    {Hue[0.35334587499948134`, 1, 1], LineBox[{34, 35}]}, 
                    {Hue[0.06870296802393036, 1, 1], LineBox[{35, 36}]}, 
                    {Hue[0.6821357962793559, 1, 1], LineBox[{36, 37}]}, 
                    {Hue[0.6351997075059883, 1, 1], LineBox[{37, 38}]}, 
                    {Hue[0.40994418070536254`, 1, 1], LineBox[{38, 39}]}, 
                    {Hue[0.6600956815350549, 1, 1], LineBox[{39, 40}]}, 
                    {Hue[0.3763685961693272, 1, 1], LineBox[{40, 41}]}, 
                    {Hue[0.0525203213503501, 1, 1], LineBox[{41, 42}]}, 
                    {Hue[0.9908307255976239, 1, 1], LineBox[{42, 43}]}, 
                    {Hue[0.009823911060358891, 1, 1], LineBox[{43, 44}]}, 
                    {Hue[0.8095599146726241, 1, 1], LineBox[{44, 45}]}, 
                    {Hue[0.6425002953713946, 1, 1], LineBox[{45, 46}]}, 
                    {Hue[0.6328654713099459, 1, 1], LineBox[{46, 47}]}, 
                    {Hue[0.6176065487895835, 1, 1], LineBox[{47, 48}]}, 
                    {Hue[0.491800422525271, 1, 1], LineBox[{48, 49}]}, 
                    {Hue[0.3386878267347081, 1, 1], LineBox[{49, 50}]}, 
                    {Hue[0.19665319598299283`, 1, 1], LineBox[{50, 51}]}, 
                    {Hue[0.0021306817817562038`, 1, 1], LineBox[{51, 52}]}, 
                    {Hue[0.8729735920333851, 1, 1], LineBox[{52, 53}]}, 
                    {Hue[0.7003776259936819, 1, 1], LineBox[{53, 54}]}, 
                    {Hue[0.6587624394900979, 1, 1], LineBox[{54, 55}]}, 
                    {Hue[0.46120340580633984`, 1, 1], LineBox[{55, 56}]}, 
                    {Hue[0.36582744313624216`, 1, 1], LineBox[{56, 57}]}, 
                    {Hue[0.36577014645724054`, 1, 1], LineBox[{57, 58}]}, 
                    {Hue[0.2768708630533676, 1, 1], LineBox[{58, 59}]}, 
                    {Hue[0.401572703318089, 1, 1], LineBox[{59, 60}]}, 
                    {Hue[0.4543051107625647, 1, 1], LineBox[{60, 61}]}, 
                    {Hue[0.4950883665628377, 1, 1], LineBox[{61, 62}]}, 
                    {Hue[0.5727212045820049, 1, 1], LineBox[{62, 63}]}, 
                    {Hue[0.6144593839253265, 1, 1], LineBox[{63, 64}]}, 
                    {Hue[0.650777198099475, 1, 1], LineBox[{64, 65}]}, 
                    {Hue[0.6812023476718669, 1, 1], LineBox[{65, 66}]}, 
                    {Hue[0.7366775496335508, 1, 1], LineBox[{66, 67}]}, 
                    {Hue[0.8040407063237689, 1, 1], LineBox[{67, 68}]}, 
                    {Hue[0.8751337841577034, 1, 1], LineBox[{68, 69}]}, 
                    {Hue[0.9349979895740793, 1, 1], LineBox[{69, 70}]}, 
                    {Hue[0.9831611795516255, 1, 1], LineBox[{70, 71}]}, 
                    {Hue[0.03690540732882161, 1, 1], LineBox[{71, 72}]}, 
                    {Hue[0.09340805898442196, 1, 1], LineBox[{72, 73}]}, 
                    {Hue[0.15555150481302726`, 1, 1], LineBox[{73, 74}]}, 
                    {Hue[0.22260102211802169`, 1, 1], LineBox[{74, 75}]}, 
                    {Hue[0.2874873707395083, 1, 1], LineBox[{75, 76}]}, 
                    {Hue[0.34371795285997764`, 1, 1], LineBox[{76, 77}]}, 
                    {Hue[0.3987686908705344, 1, 1], LineBox[{77, 78}]}, 
                    {Hue[0.45584195734129307`, 1, 1], LineBox[{78, 79}]}, 
                    {Hue[0.5194171517525992, 1, 1], LineBox[{79, 80}]}, 
                    {Hue[0.5871948626381034, 1, 1], LineBox[{80, 81}]}, 
                    {Hue[0.6550071558423163, 1, 1], LineBox[{81, 82}]}, 
                    {Hue[0.7169971729064637, 1, 1], LineBox[{82, 83}]}, 
                    {Hue[0.7758915686233734, 1, 1], LineBox[{83, 84}]}, 
                    {Hue[0.8351048410044376, 1, 1], LineBox[{84, 85}]}, 
                    {Hue[0.8997708000560913, 1, 1], LineBox[{85, 86}]}, 
                    {Hue[0.9703622628350036, 1, 1], LineBox[{86, 87}]}, 
                    {Hue[0.04204741694621801, 1, 1], LineBox[{87, 88}]}, 
                    {Hue[0.1084845737470324, 1, 1], LineBox[{88, 89}]}, 
                    {Hue[0.1701666187456011, 1, 1], LineBox[{89, 90}]}, 
                    {Hue[0.2313775399901313, 1, 1], LineBox[{90, 91}]}, 
                    {Hue[0.2976877502367062, 1, 1], LineBox[{91, 92}]}, 
                    {Hue[0.3711740550744846, 1, 1], LineBox[{92, 93}]}, 
                    {Hue[0.44687625426845595`, 1, 1], LineBox[{93, 94}]}, 
                    {Hue[0.5176126759525406, 1, 1], LineBox[{94, 95}]}, 
                    {Hue[0.5819664960418984, 1, 1], LineBox[{95, 96}]}, 
                    {Hue[0.6448711364223789, 1, 1], LineBox[{96, 97}]}, 
                    {Hue[0.7127443005598428, 1, 1], LineBox[{97, 98}]}, 
                    {Hue[0.7888911244253934, 1, 1], LineBox[{98, 99}]}, 
                    {Hue[0.8685972235020938, 1, 1], LineBox[{99, 100}]}, 
                    {Hue[0.9429413869593951, 1, 1], LineBox[{100, 101}]}, 
                    {Hue[0.009597528089488936, 1, 1], LineBox[{101, 102}]}, 
                    {Hue[0.07369556643187233, 1, 1], LineBox[{102, 103}]}, 
                    {Hue[0.14259002452799643`, 1, 1], LineBox[{103, 104}]}, 
                    {Hue[0.2208055343087441, 1, 1], LineBox[{104, 105}]}, 
                    {Hue[0.30397005118067305`, 1, 1], LineBox[{105, 106}]}, 
                    {Hue[0.3814022118325686, 1, 1], LineBox[{106, 107}]}, 
                    {Hue[0.44958930198492464`, 1, 1], LineBox[{107, 108}]}, 
                    {Hue[0.5137878132576624, 1, 1], LineBox[{108, 109}]}, 
                    {Hue[0.5819989623965363, 1, 1], LineBox[{109, 110}]}, 
                    {Hue[0.6601775740658046, 1, 1], LineBox[{110, 111}]}, 
                    {Hue[0.745202837419231, 1, 1], LineBox[{111, 112}]}, 
                    {Hue[0.8250475018811899, 1, 1], LineBox[{112, 113}]}, 
                    {Hue[0.8938729371910848, 1, 1], LineBox[{113, 114}]}, 
                    {Hue[0.9561034162561497, 1, 1], LineBox[{114, 115}]}, 
                    {Hue[0.019638189549080177`, 1, 1], LineBox[{115, 116}]}, 
                    {Hue[0.09164212060033193, 1, 1], LineBox[{116, 117}]}, 
                    {Hue[0.17317222581323283`, 1, 1], LineBox[{117, 118}]}, 
                    {Hue[0.2541781134560693, 1, 1], LineBox[{118, 119}]}, 
                    {Hue[0.3241269011239155, 1, 1], LineBox[{119, 120}]}, 
                    {Hue[0.3830525677867494, 1, 1], LineBox[{120, 121}]}, 
                    {Hue[0.43671758530245763`, 1, 1], LineBox[{121, 122}]}, 
                    {Hue[0.491429567541259, 1, 1], LineBox[{122, 123}]}, 
                    {Hue[0.5524263664053911, 1, 1], LineBox[{123, 124}]}, 
                    {Hue[0.6213032633035116, 1, 1], LineBox[{124, 125}]}, 
                    {Hue[0.6922386416207904, 1, 1], LineBox[{125, 126}]}, 
                    {Hue[0.7556807223227779, 1, 1], LineBox[{126, 127}]}, 
                    {Hue[0.8075261375163918, 1, 1], LineBox[{127, 128}]}, 
                    {Hue[0.8494469370502941, 1, 1], LineBox[{128, 129}]}, 
                    {Hue[0.8845456784920832, 1, 1], LineBox[{129, 130}]}, 
                    {Hue[0.9153607707770892, 1, 1], LineBox[{130, 131}]}, 
                    {Hue[0.9436483964312276, 1, 1], LineBox[{131, 132}]}, 
                    {Hue[0.9705913152413616, 1, 1], LineBox[{132, 133}]}, 
                    {Hue[0.9969991385198878, 1, 1], LineBox[{133, 134}]}, 
                    {Hue[0.023443344689877202`, 1, 1], LineBox[{134, 135}]}, 
                    {Hue[0.0503392189712252, 1, 1], LineBox[{135, 136}]}, 
                    {Hue[0.07799583048219152, 1, 1], LineBox[{136, 137}]}, 
                    {Hue[0.10664739890542389`, 1, 1], LineBox[{137, 138}]}, 
                    {Hue[0.13647223751425389`, 1, 1], LineBox[{138, 139}]}, 
                    {Hue[0.16760651012276648`, 1, 1], LineBox[{139, 140}]}, 
                    {Hue[0.20015193386020552`, 1, 1], LineBox[{140, 141}]}, 
                    {Hue[0.2341836723124326, 1, 1], LineBox[{141, 142}]}, 
                    {Hue[0.2697549633000815, 1, 1], LineBox[{142, 143}]}, 
                    {Hue[0.3069010347031737, 1, 1], LineBox[{143, 144}]}, 
                    {Hue[0.34564242381955096`, 1, 1], LineBox[{144, 145}]}, 
                    {Hue[0.385988114107708, 1, 1], LineBox[{145, 146}]}, 
                    {Hue[0.4279373960933569, 1, 1], LineBox[{146, 147}]}, 
                    {Hue[0.4714821138884691, 1, 1], LineBox[{147, 148}]}, 
                    {Hue[0.516608142439269, 1, 1], LineBox[{148, 149}]}, 
                    {Hue[0.5632967986431676, 1, 1], LineBox[{149, 150}]}, 
                    {Hue[0.6115262291158869, 1, 1], LineBox[{150, 151}]}, 
                    {Hue[0.6612723650240759, 1, 1], LineBox[{151, 152}]}, 
                    {Hue[0.7125082110086003, 1, 1], LineBox[{152, 153}]}, 
                    {Hue[0.7652067245285928, 1, 1], LineBox[{153, 154}]}, 
                    {Hue[0.8193404899939781, 1, 1], LineBox[{154, 155}]}, 
                    {Hue[0.8748814082258843, 1, 1], LineBox[{155, 156}]}, 
                    {Hue[0.93180121767383, 1, 1], LineBox[{156, 157}]}, 
                    {Hue[0.9900732339915105, 1, 1], LineBox[{157, 158}]}, 
                    {Hue[0.04966897366675908, 1, 1], LineBox[{158, 159}]}, 
                    {Hue[0.11056355278904713`, 1, 1], LineBox[{159, 160}]}, 
                    {Hue[0.17273127731108673`, 1, 1], LineBox[{160, 161}]}, 
                    {Hue[0.23614688428905453`, 1, 1], LineBox[{161, 162}]}, 
                    {Hue[0.30078631795382943`, 1, 1], LineBox[{162, 163}]}, 
                    {Hue[0.36662673011340297`, 1, 1], LineBox[{163, 164}]}, 
                    {Hue[0.4336462625630184, 1, 1], LineBox[{164, 165}]}, 
                    {Hue[0.5018243590466764, 1, 1], LineBox[{165, 166}]}, 
                    {Hue[0.5711391738638567, 1, 1], LineBox[{166, 167}]}, 
                    {Hue[0.6415712630612694, 1, 1], LineBox[{167, 168}]}, 
                    {Hue[0.7131026687380215, 1, 1], LineBox[{168, 169}]}, 
                    {Hue[0.785714952794411, 1, 1], LineBox[{169, 170}]}, 
                    {Hue[0.85939125988999, 1, 1], LineBox[{170, 171}]}, 
                    {Hue[0.9341142030507813, 1, 1], LineBox[{171, 172}]}, 
                    {Hue[0.009867814283582065, 1, 1], LineBox[{172, 173}]}, 
                    {Hue[0.08663975672705043, 1, 1], LineBox[{173, 174}]}, 
                    {Hue[0.16441331632110967`, 1, 1], LineBox[{174, 175}]}, 
                    {Hue[0.24317140299829, 1, 1], LineBox[{175, 176}]}, 
                    {Hue[0.32290236585478704`, 1, 1], LineBox[{176, 177}]}, 
                    {Hue[0.4035943587363709, 1, 1], LineBox[{177, 178}]}, 
                    {Hue[0.48523999204304397`, 1, 1], LineBox[{178, 179}]}, 
                    {Hue[0.5678193374677074, 1, 1], LineBox[{179, 180}]}, 
                    {Hue[0.6513185536279276, 1, 1], LineBox[{180, 181}]}, 
                    {Hue[0.7357401903243395, 1, 1], LineBox[{181, 182}]}, 
                    {Hue[0.8210622956913834, 1, 1], LineBox[{182, 183}]}, 
                    {Hue[0.9072682139525383, 1, 1], LineBox[{183, 184}]}, 
                    {Hue[0.9943604247210516, 1, 1], LineBox[{184, 185}]}, 
                    {Hue[0.08232920234124495, 1, 1], LineBox[{185, 186}]}, 
                    {Hue[0.17115182211425745`, 1, 1], LineBox[{186, 187}]}, 
                    {Hue[0.2608259833213761, 1, 1], LineBox[{187, 188}]}, 
                    {Hue[0.3513592109082063, 1, 1], LineBox[{188, 189}]}, 
                    {Hue[0.44272159494008767`, 1, 1], LineBox[{189, 190}]}, 
                    {Hue[0.5348962505409033, 1, 1], LineBox[{190, 191}]}, 
                    {Hue[0.6279072641081033, 1, 1], LineBox[{191, 192}]}, 
                    {Hue[0.7217267880302329, 1, 1], LineBox[{192, 193}]}, 
                    {Hue[0.816313267532884, 1, 1], LineBox[{193, 194}]}, 
                    {Hue[0.9117512965679803, 1, 1], LineBox[{194, 195}]}, 
                    {Hue[0.008047879736664992, 1, 1], LineBox[{195, 196}]}, 
                    {Hue[0.10501187209075581`, 1, 1], LineBox[{196, 197}]}, 
                    {Hue[0.20270535185021696`, 1, 1], LineBox[{197, 198}]}, 
                    {Hue[0.30133780047475683`, 1, 1], LineBox[{198, 199}]}, 
                    {Hue[0.40068737757147616`, 1, 1], LineBox[{199, 200}]}, 
                    {Hue[0.500621695745771, 1, 1], LineBox[{200, 201}]}, 
                    {Hue[0.6015194562805606, 1, 1], LineBox[{201, 202}]}, 
                    {Hue[0.7033866071661006, 1, 1], LineBox[{202, 203}]}, 
                    {Hue[0.8057600247529666, 1, 1], LineBox[{203, 204}]}, 
                    {Hue[0.9087115831851035, 1, 1], LineBox[{204, 205}]}, 
                    {Hue[0.012728494365962887`, 1, 1], LineBox[{205, 206}]}, 
                    {Hue[0.11768230946122678`, 1, 1], LineBox[{206, 207}]}, 
                    {Hue[0.22299007300419238`, 1, 1], LineBox[{207, 208}]}, 
                    {Hue[0.32878942455954685`, 1, 1], LineBox[{208, 209}]}, 
                    {Hue[0.43566326944187145`, 1, 1], LineBox[{209, 210}]}, 
                    {Hue[0.5430660387149626, 1, 1], LineBox[{210, 211}]}, 
                    {Hue[0.6501242831582152, 1, 1], LineBox[{211, 212}]}, 
                    {Hue[0.758048676916268, 1, 1], LineBox[{212, 213}]}, 
                    {Hue[0.8686784353266866, 1, 1], LineBox[{213, 214}]}, 
                    {Hue[0.9804881391600404, 1, 1], LineBox[{214, 215}]}, 
                    {Hue[0.08932909517187086, 1, 1], LineBox[{215, 216}]}, 
                    {Hue[0.1955941259474388, 1, 1], LineBox[{216, 217}]}, 
                    {Hue[0.30741713430835516`, 1, 1], LineBox[{217, 218}]}, 
                    {Hue[0.4270155355366292, 1, 1], LineBox[{218, 219}]}, 
                    {Hue[0.5429924302256508, 1, 1], LineBox[{219, 220}]}, 
                    {Hue[0.6483558235535518, 1, 1], LineBox[{220, 221}]}, 
                    {Hue[0.7667154238986771, 1, 1], LineBox[{221, 222}]}, 
                    {Hue[0.9142402763382171, 1, 1], LineBox[{222, 223}]}, 
                    {Hue[0.07850281472384095, 1, 1], LineBox[{223, 224}]}, 
                    {Hue[0.2333380105758965, 1, 1], LineBox[{224, 225}]}, 
                    {Hue[0.23584220459241986`, 1, 1], LineBox[{225, 226}]}, 
                    {Hue[0.2548056612472624, 1, 1], LineBox[{226, 227}]}, 
                    {Hue[0.41845090004777136`, 1, 1], LineBox[{227, 228}]}, 
                    {Hue[0.5707092498069801, 1, 1], LineBox[{228, 229}]}, 
                    {Hue[0.33232299147995975`, 1, 1], LineBox[{229, 230}]}, 
                    {Hue[0.531991234171352, 1, 1], LineBox[{230, 231}]}, 
                    {Hue[0.7431920582950459, 1, 1], LineBox[{231, 232}]}, 
                    {Hue[0.9572424609776871, 1, 1], LineBox[{232, 233}]}, 
                    {Hue[0.16180360112036146`, 1, 1], LineBox[{233, 234}]}, 
                    {Hue[0.30347340649352716`, 1, 1], LineBox[{234, 235}]}, 
                    {Hue[0.35228833642655205`, 1, 1], LineBox[{235, 236}]}, 
                    {Hue[0.5320890788639869, 1, 1], LineBox[{236, 237}]}, 
                    {Hue[0.7606664269256596, 1, 1], LineBox[{237, 238}]}, 
                    {Hue[0.00496704327457589, 1, 1], LineBox[{238, 239}]}, 
                    {Hue[0.26145708925610656`, 1, 1], LineBox[{239, 240}]}, 
                    {Hue[0.543029142422131, 1, 1], LineBox[{240, 241}]}, 
                    {Hue[0.8648449595129252, 1, 1], LineBox[{241, 242}]}, 
                    {Hue[0.16854952740027157`, 1, 1], LineBox[{242, 243}]}, 
                    {Hue[0.43839966896805005`, 1, 1], LineBox[{243, 244}]}, 
                    {Hue[0.7007213800317906, 1, 1], LineBox[{244, 245}]}, 
                    {Hue[0.9650895094938333, 1, 1], LineBox[{245, 246}]}, 
                    {Hue[0.23385603595579127`, 1, 1], LineBox[{246, 247}]}, 
                    {Hue[0.5085921548983394, 1, 1], LineBox[{247, 248}]}, 
                    {Hue[0.7925154147969324, 1, 1], LineBox[{248, 249}]}, 
                    {Hue[0.09250870720938896, 1, 1], LineBox[{249, 250}]}, 
                    {Hue[0.417902691531048, 1, 1], LineBox[{250, 251}]}}}},
                    VertexColors->{
                    Hue[0.10248796621842285`, 1, 1], 
                    Hue[0.8290457167148891, 1, 1], 
                    Hue[0.8703632125845846, 1, 1], 
                    Hue[0.7393175820802007, 1, 1], 
                    Hue[0.6032998790616205, 1, 1], 
                    Hue[0.5291282883599169, 1, 1], 
                    Hue[0.37551039951532045`, 1, 1], 
                    Hue[0.22433259918415324`, 1, 1], 
                    Hue[0.11852503188071434`, 1, 1], 
                    Hue[0.07197136740119216, 1, 1], 
                    Hue[0.027235479998395193`, 1, 1], 
                    Hue[0.9072061553893599, 1, 1], 
                    Hue[0.8405170901747704, 1, 1], 
                    Hue[0.8075393950498422, 1, 1], 
                    Hue[0.5331074488604866, 1, 1], 
                    Hue[0.5358981991195568, 1, 1], 
                    Hue[0.25004222875796983`, 1, 1], 
                    Hue[0.7962568815322872, 1, 1], 
                    Hue[0.8602643648636271, 1, 1], 
                    Hue[0.7061829013619123, 1, 1], 
                    Hue[0.5353893983812265, 1, 1], 
                    Hue[0.5574883411356903, 1, 1], 
                    Hue[0.3982347746773217, 1, 1], 
                    Hue[0.376614336574381, 1, 1], 
                    Hue[0.1848960953069641, 1, 1], 
                    Hue[0.39778971853907324`, 1, 1], 
                    Hue[0.09352019075476536, 1, 1], 
                    Hue[0.783376412070276, 1, 1], 
                    Hue[0.7824547112898794, 1, 1], 
                    Hue[0.7761971219426127, 1, 1], 
                    Hue[0.6038072405559677, 1, 1], 
                    Hue[0.560930797116872, 1, 1], 
                    Hue[0.33776760760892577`, 1, 1], 
                    Hue[0.35334587499948134`, 1, 1], 
                    Hue[0.06870296802393036, 1, 1], 
                    Hue[0.6821357962793559, 1, 1], 
                    Hue[0.6351997075059883, 1, 1], 
                    Hue[0.40994418070536254`, 1, 1], 
                    Hue[0.6600956815350549, 1, 1], 
                    Hue[0.3763685961693272, 1, 1], 
                    Hue[0.0525203213503501, 1, 1], 
                    Hue[0.9908307255976239, 1, 1], 
                    Hue[0.009823911060358891, 1, 1], 
                    Hue[0.8095599146726241, 1, 1], 
                    Hue[0.6425002953713946, 1, 1], 
                    Hue[0.6328654713099459, 1, 1], 
                    Hue[0.6176065487895835, 1, 1], 
                    Hue[0.491800422525271, 1, 1], 
                    Hue[0.3386878267347081, 1, 1], 
                    Hue[0.19665319598299283`, 1, 1], 
                    Hue[0.0021306817817562038`, 1, 1], 
                    Hue[0.8729735920333851, 1, 1], 
                    Hue[0.7003776259936819, 1, 1], 
                    Hue[0.6587624394900979, 1, 1], 
                    Hue[0.46120340580633984`, 1, 1], 
                    Hue[0.36582744313624216`, 1, 1], 
                    Hue[0.36577014645724054`, 1, 1], 
                    Hue[0.2768708630533676, 1, 1], 
                    Hue[0.401572703318089, 1, 1], 
                    Hue[0.4543051107625647, 1, 1], 
                    Hue[0.4950883665628377, 1, 1], 
                    Hue[0.5727212045820049, 1, 1], 
                    Hue[0.6144593839253265, 1, 1], 
                    Hue[0.650777198099475, 1, 1], 
                    Hue[0.6812023476718669, 1, 1], 
                    Hue[0.7366775496335508, 1, 1], 
                    Hue[0.8040407063237689, 1, 1], 
                    Hue[0.8751337841577034, 1, 1], 
                    Hue[0.9349979895740793, 1, 1], 
                    Hue[0.9831611795516255, 1, 1], 
                    Hue[0.03690540732882161, 1, 1], 
                    Hue[0.09340805898442196, 1, 1], 
                    Hue[0.15555150481302726`, 1, 1], 
                    Hue[0.22260102211802169`, 1, 1], 
                    Hue[0.2874873707395083, 1, 1], 
                    Hue[0.34371795285997764`, 1, 1], 
                    Hue[0.3987686908705344, 1, 1], 
                    Hue[0.45584195734129307`, 1, 1], 
                    Hue[0.5194171517525992, 1, 1], 
                    Hue[0.5871948626381034, 1, 1], 
                    Hue[0.6550071558423163, 1, 1], 
                    Hue[0.7169971729064637, 1, 1], 
                    Hue[0.7758915686233734, 1, 1], 
                    Hue[0.8351048410044376, 1, 1], 
                    Hue[0.8997708000560913, 1, 1], 
                    Hue[0.9703622628350036, 1, 1], 
                    Hue[0.04204741694621801, 1, 1], 
                    Hue[0.1084845737470324, 1, 1], 
                    Hue[0.1701666187456011, 1, 1], 
                    Hue[0.2313775399901313, 1, 1], 
                    Hue[0.2976877502367062, 1, 1], 
                    Hue[0.3711740550744846, 1, 1], 
                    Hue[0.44687625426845595`, 1, 1], 
                    Hue[0.5176126759525406, 1, 1], 
                    Hue[0.5819664960418984, 1, 1], 
                    Hue[0.6448711364223789, 1, 1], 
                    Hue[0.7127443005598428, 1, 1], 
                    Hue[0.7888911244253934, 1, 1], 
                    Hue[0.8685972235020938, 1, 1], 
                    Hue[0.9429413869593951, 1, 1], 
                    Hue[0.009597528089488936, 1, 1], 
                    Hue[0.07369556643187233, 1, 1], 
                    Hue[0.14259002452799643`, 1, 1], 
                    Hue[0.2208055343087441, 1, 1], 
                    Hue[0.30397005118067305`, 1, 1], 
                    Hue[0.3814022118325686, 1, 1], 
                    Hue[0.44958930198492464`, 1, 1], 
                    Hue[0.5137878132576624, 1, 1], 
                    Hue[0.5819989623965363, 1, 1], 
                    Hue[0.6601775740658046, 1, 1], 
                    Hue[0.745202837419231, 1, 1], 
                    Hue[0.8250475018811899, 1, 1], 
                    Hue[0.8938729371910848, 1, 1], 
                    Hue[0.9561034162561497, 1, 1], 
                    Hue[0.019638189549080177`, 1, 1], 
                    Hue[0.09164212060033193, 1, 1], 
                    Hue[0.17317222581323283`, 1, 1], 
                    Hue[0.2541781134560693, 1, 1], 
                    Hue[0.3241269011239155, 1, 1], 
                    Hue[0.3830525677867494, 1, 1], 
                    Hue[0.43671758530245763`, 1, 1], 
                    Hue[0.491429567541259, 1, 1], 
                    Hue[0.5524263664053911, 1, 1], 
                    Hue[0.6213032633035116, 1, 1], 
                    Hue[0.6922386416207904, 1, 1], 
                    Hue[0.7556807223227779, 1, 1], 
                    Hue[0.8075261375163918, 1, 1], 
                    Hue[0.8494469370502941, 1, 1], 
                    Hue[0.8845456784920832, 1, 1], 
                    Hue[0.9153607707770892, 1, 1], 
                    Hue[0.9436483964312276, 1, 1], 
                    Hue[0.9705913152413616, 1, 1], 
                    Hue[0.9969991385198878, 1, 1], 
                    Hue[0.023443344689877202`, 1, 1], 
                    Hue[0.0503392189712252, 1, 1], 
                    Hue[0.07799583048219152, 1, 1], 
                    Hue[0.10664739890542389`, 1, 1], 
                    Hue[0.13647223751425389`, 1, 1], 
                    Hue[0.16760651012276648`, 1, 1], 
                    Hue[0.20015193386020552`, 1, 1], 
                    Hue[0.2341836723124326, 1, 1], 
                    Hue[0.2697549633000815, 1, 1], 
                    Hue[0.3069010347031737, 1, 1], 
                    Hue[0.34564242381955096`, 1, 1], 
                    Hue[0.385988114107708, 1, 1], 
                    Hue[0.4279373960933569, 1, 1], 
                    Hue[0.4714821138884691, 1, 1], 
                    Hue[0.516608142439269, 1, 1], 
                    Hue[0.5632967986431676, 1, 1], 
                    Hue[0.6115262291158869, 1, 1], 
                    Hue[0.6612723650240759, 1, 1], 
                    Hue[0.7125082110086003, 1, 1], 
                    Hue[0.7652067245285928, 1, 1], 
                    Hue[0.8193404899939781, 1, 1], 
                    Hue[0.8748814082258843, 1, 1], 
                    Hue[0.93180121767383, 1, 1], 
                    Hue[0.9900732339915105, 1, 1], 
                    Hue[0.04966897366675908, 1, 1], 
                    Hue[0.11056355278904713`, 1, 1], 
                    Hue[0.17273127731108673`, 1, 1], 
                    Hue[0.23614688428905453`, 1, 1], 
                    Hue[0.30078631795382943`, 1, 1], 
                    Hue[0.36662673011340297`, 1, 1], 
                    Hue[0.4336462625630184, 1, 1], 
                    Hue[0.5018243590466764, 1, 1], 
                    Hue[0.5711391738638567, 1, 1], 
                    Hue[0.6415712630612694, 1, 1], 
                    Hue[0.7131026687380215, 1, 1], 
                    Hue[0.785714952794411, 1, 1], 
                    Hue[0.85939125988999, 1, 1], 
                    Hue[0.9341142030507813, 1, 1], 
                    Hue[0.009867814283582065, 1, 1], 
                    Hue[0.08663975672705043, 1, 1], 
                    Hue[0.16441331632110967`, 1, 1], 
                    Hue[0.24317140299829, 1, 1], 
                    Hue[0.32290236585478704`, 1, 1], 
                    Hue[0.4035943587363709, 1, 1], 
                    Hue[0.48523999204304397`, 1, 1], 
                    Hue[0.5678193374677074, 1, 1], 
                    Hue[0.6513185536279276, 1, 1], 
                    Hue[0.7357401903243395, 1, 1], 
                    Hue[0.8210622956913834, 1, 1], 
                    Hue[0.9072682139525383, 1, 1], 
                    Hue[0.9943604247210516, 1, 1], 
                    Hue[0.08232920234124495, 1, 1], 
                    Hue[0.17115182211425745`, 1, 1], 
                    Hue[0.2608259833213761, 1, 1], 
                    Hue[0.3513592109082063, 1, 1], 
                    Hue[0.44272159494008767`, 1, 1], 
                    Hue[0.5348962505409033, 1, 1], 
                    Hue[0.6279072641081033, 1, 1], 
                    Hue[0.7217267880302329, 1, 1], 
                    Hue[0.816313267532884, 1, 1], 
                    Hue[0.9117512965679803, 1, 1], 
                    Hue[0.008047879736664992, 1, 1], 
                    Hue[0.10501187209075581`, 1, 1], 
                    Hue[0.20270535185021696`, 1, 1], 
                    Hue[0.30133780047475683`, 1, 1], 
                    Hue[0.40068737757147616`, 1, 1], 
                    Hue[0.500621695745771, 1, 1], 
                    Hue[0.6015194562805606, 1, 1], 
                    Hue[0.7033866071661006, 1, 1], 
                    Hue[0.8057600247529666, 1, 1], 
                    Hue[0.9087115831851035, 1, 1], 
                    Hue[0.012728494365962887`, 1, 1], 
                    Hue[0.11768230946122678`, 1, 1], 
                    Hue[0.22299007300419238`, 1, 1], 
                    Hue[0.32878942455954685`, 1, 1], 
                    Hue[0.43566326944187145`, 1, 1], 
                    Hue[0.5430660387149626, 1, 1], 
                    Hue[0.6501242831582152, 1, 1], 
                    Hue[0.758048676916268, 1, 1], 
                    Hue[0.8686784353266866, 1, 1], 
                    Hue[0.9804881391600404, 1, 1], 
                    Hue[0.08932909517187086, 1, 1], 
                    Hue[0.1955941259474388, 1, 1], 
                    Hue[0.30741713430835516`, 1, 1], 
                    Hue[0.4270155355366292, 1, 1], 
                    Hue[0.5429924302256508, 1, 1], 
                    Hue[0.6483558235535518, 1, 1], 
                    Hue[0.7667154238986771, 1, 1], 
                    Hue[0.9142402763382171, 1, 1], 
                    Hue[0.07850281472384095, 1, 1], 
                    Hue[0.2333380105758965, 1, 1], 
                    Hue[0.23584220459241986`, 1, 1], 
                    Hue[0.2548056612472624, 1, 1], 
                    Hue[0.41845090004777136`, 1, 1], 
                    Hue[0.5707092498069801, 1, 1], 
                    Hue[0.33232299147995975`, 1, 1], 
                    Hue[0.531991234171352, 1, 1], 
                    Hue[0.7431920582950459, 1, 1], 
                    Hue[0.9572424609776871, 1, 1], 
                    Hue[0.16180360112036146`, 1, 1], 
                    Hue[0.30347340649352716`, 1, 1], 
                    Hue[0.35228833642655205`, 1, 1], 
                    Hue[0.5320890788639869, 1, 1], 
                    Hue[0.7606664269256596, 1, 1], 
                    Hue[0.00496704327457589, 1, 1], 
                    Hue[0.26145708925610656`, 1, 1], 
                    Hue[0.543029142422131, 1, 1], 
                    Hue[0.8648449595129252, 1, 1], 
                    Hue[0.16854952740027157`, 1, 1], 
                    Hue[0.43839966896805005`, 1, 1], 
                    Hue[0.7007213800317906, 1, 1], 
                    Hue[0.9650895094938333, 1, 1], 
                    Hue[0.23385603595579127`, 1, 1], 
                    Hue[0.5085921548983394, 1, 1], 
                    Hue[0.7925154147969324, 1, 1], 
                    Hue[0.09250870720938896, 1, 1], 
                    Hue[0.417902691531048, 1, 1], 
                    Hue[0.7584781495977884, 1, 1], 
                    Hue[0.10248796621842285`, 1, 1], 
                    Hue[0.8290457167148891, 1, 1], 
                    Hue[0.8290457167148891, 1, 1], 
                    Hue[0.8703632125845846, 1, 1], 
                    Hue[0.8703632125845846, 1, 1], 
                    Hue[0.7393175820802007, 1, 1], 
                    Hue[0.7393175820802007, 1, 1], 
                    Hue[0.6032998790616205, 1, 1], 
                    Hue[0.6032998790616205, 1, 1], 
                    Hue[0.5291282883599169, 1, 1], 
                    Hue[0.5291282883599169, 1, 1], 
                    Hue[0.37551039951532045`, 1, 1], 
                    Hue[0.37551039951532045`, 1, 1], 
                    Hue[0.22433259918415324`, 1, 1], 
                    Hue[0.22433259918415324`, 1, 1], 
                    Hue[0.11852503188071434`, 1, 1], 
                    Hue[0.11852503188071434`, 1, 1], 
                    Hue[0.07197136740119216, 1, 1], 
                    Hue[0.07197136740119216, 1, 1], 
                    Hue[0.027235479998395193`, 1, 1], 
                    Hue[0.027235479998395193`, 1, 1], 
                    Hue[0.9072061553893599, 1, 1], 
                    Hue[0.9072061553893599, 1, 1], 
                    Hue[0.8405170901747704, 1, 1], 
                    Hue[0.8405170901747704, 1, 1], 
                    Hue[0.8075393950498422, 1, 1], 
                    Hue[0.8075393950498422, 1, 1], 
                    Hue[0.5331074488604866, 1, 1], 
                    Hue[0.5331074488604866, 1, 1], 
                    Hue[0.5358981991195568, 1, 1], 
                    Hue[0.5358981991195568, 1, 1], 
                    Hue[0.25004222875796983`, 1, 1], 
                    Hue[0.25004222875796983`, 1, 1], 
                    Hue[0.7962568815322872, 1, 1], 
                    Hue[0.7962568815322872, 1, 1], 
                    Hue[0.8602643648636271, 1, 1], 
                    Hue[0.8602643648636271, 1, 1], 
                    Hue[0.7061829013619123, 1, 1], 
                    Hue[0.7061829013619123, 1, 1], 
                    Hue[0.5353893983812265, 1, 1], 
                    Hue[0.5353893983812265, 1, 1], 
                    Hue[0.5574883411356903, 1, 1], 
                    Hue[0.5574883411356903, 1, 1], 
                    Hue[0.3982347746773217, 1, 1], 
                    Hue[0.3982347746773217, 1, 1], 
                    Hue[0.376614336574381, 1, 1], 
                    Hue[0.376614336574381, 1, 1], 
                    Hue[0.1848960953069641, 1, 1], 
                    Hue[0.1848960953069641, 1, 1], 
                    Hue[0.39778971853907324`, 1, 1], 
                    Hue[0.39778971853907324`, 1, 1], 
                    Hue[0.09352019075476536, 1, 1], 
                    Hue[0.09352019075476536, 1, 1], 
                    Hue[0.783376412070276, 1, 1], 
                    Hue[0.783376412070276, 1, 1], 
                    Hue[0.7824547112898794, 1, 1], 
                    Hue[0.7824547112898794, 1, 1], 
                    Hue[0.7761971219426127, 1, 1], 
                    Hue[0.7761971219426127, 1, 1], 
                    Hue[0.6038072405559677, 1, 1], 
                    Hue[0.6038072405559677, 1, 1], 
                    Hue[0.560930797116872, 1, 1], 
                    Hue[0.560930797116872, 1, 1], 
                    Hue[0.33776760760892577`, 1, 1], 
                    Hue[0.33776760760892577`, 1, 1], 
                    Hue[0.35334587499948134`, 1, 1], 
                    Hue[0.35334587499948134`, 1, 1], 
                    Hue[0.06870296802393036, 1, 1], 
                    Hue[0.06870296802393036, 1, 1], 
                    Hue[0.6821357962793559, 1, 1], 
                    Hue[0.6821357962793559, 1, 1], 
                    Hue[0.6351997075059883, 1, 1], 
                    Hue[0.6351997075059883, 1, 1], 
                    Hue[0.40994418070536254`, 1, 1], 
                    Hue[0.40994418070536254`, 1, 1], 
                    Hue[0.6600956815350549, 1, 1], 
                    Hue[0.6600956815350549, 1, 1], 
                    Hue[0.3763685961693272, 1, 1], 
                    Hue[0.3763685961693272, 1, 1], 
                    Hue[0.0525203213503501, 1, 1], 
                    Hue[0.0525203213503501, 1, 1], 
                    Hue[0.9908307255976239, 1, 1], 
                    Hue[0.9908307255976239, 1, 1], 
                    Hue[0.009823911060358891, 1, 1], 
                    Hue[0.009823911060358891, 1, 1], 
                    Hue[0.8095599146726241, 1, 1], 
                    Hue[0.8095599146726241, 1, 1], 
                    Hue[0.6425002953713946, 1, 1], 
                    Hue[0.6425002953713946, 1, 1], 
                    Hue[0.6328654713099459, 1, 1], 
                    Hue[0.6328654713099459, 1, 1], 
                    Hue[0.6176065487895835, 1, 1], 
                    Hue[0.6176065487895835, 1, 1], 
                    Hue[0.491800422525271, 1, 1], 
                    Hue[0.491800422525271, 1, 1], 
                    Hue[0.3386878267347081, 1, 1], 
                    Hue[0.3386878267347081, 1, 1], 
                    Hue[0.19665319598299283`, 1, 1], 
                    Hue[0.19665319598299283`, 1, 1], 
                    Hue[0.0021306817817562038`, 1, 1], 
                    Hue[0.0021306817817562038`, 1, 1], 
                    Hue[0.8729735920333851, 1, 1], 
                    Hue[0.8729735920333851, 1, 1], 
                    Hue[0.7003776259936819, 1, 1], 
                    Hue[0.7003776259936819, 1, 1], 
                    Hue[0.6587624394900979, 1, 1], 
                    Hue[0.6587624394900979, 1, 1], 
                    Hue[0.46120340580633984`, 1, 1], 
                    Hue[0.46120340580633984`, 1, 1], 
                    Hue[0.36582744313624216`, 1, 1], 
                    Hue[0.36582744313624216`, 1, 1], 
                    Hue[0.36577014645724054`, 1, 1], 
                    Hue[0.36577014645724054`, 1, 1], 
                    Hue[0.2768708630533676, 1, 1], 
                    Hue[0.2768708630533676, 1, 1], 
                    Hue[0.401572703318089, 1, 1], 
                    Hue[0.401572703318089, 1, 1], 
                    Hue[0.4543051107625647, 1, 1], 
                    Hue[0.4543051107625647, 1, 1], 
                    Hue[0.4950883665628377, 1, 1], 
                    Hue[0.4950883665628377, 1, 1], 
                    Hue[0.5727212045820049, 1, 1], 
                    Hue[0.5727212045820049, 1, 1], 
                    Hue[0.6144593839253265, 1, 1], 
                    Hue[0.6144593839253265, 1, 1], 
                    Hue[0.650777198099475, 1, 1], 
                    Hue[0.650777198099475, 1, 1], 
                    Hue[0.6812023476718669, 1, 1], 
                    Hue[0.6812023476718669, 1, 1], 
                    Hue[0.7366775496335508, 1, 1], 
                    Hue[0.7366775496335508, 1, 1], 
                    Hue[0.8040407063237689, 1, 1], 
                    Hue[0.8040407063237689, 1, 1], 
                    Hue[0.8751337841577034, 1, 1], 
                    Hue[0.8751337841577034, 1, 1], 
                    Hue[0.9349979895740793, 1, 1], 
                    Hue[0.9349979895740793, 1, 1], 
                    Hue[0.9831611795516255, 1, 1], 
                    Hue[0.9831611795516255, 1, 1], 
                    Hue[0.03690540732882161, 1, 1], 
                    Hue[0.03690540732882161, 1, 1], 
                    Hue[0.09340805898442196, 1, 1], 
                    Hue[0.09340805898442196, 1, 1], 
                    Hue[0.15555150481302726`, 1, 1], 
                    Hue[0.15555150481302726`, 1, 1], 
                    Hue[0.22260102211802169`, 1, 1], 
                    Hue[0.22260102211802169`, 1, 1], 
                    Hue[0.2874873707395083, 1, 1], 
                    Hue[0.2874873707395083, 1, 1], 
                    Hue[0.34371795285997764`, 1, 1], 
                    Hue[0.34371795285997764`, 1, 1], 
                    Hue[0.3987686908705344, 1, 1], 
                    Hue[0.3987686908705344, 1, 1], 
                    Hue[0.45584195734129307`, 1, 1], 
                    Hue[0.45584195734129307`, 1, 1], 
                    Hue[0.5194171517525992, 1, 1], 
                    Hue[0.5194171517525992, 1, 1], 
                    Hue[0.5871948626381034, 1, 1], 
                    Hue[0.5871948626381034, 1, 1], 
                    Hue[0.6550071558423163, 1, 1], 
                    Hue[0.6550071558423163, 1, 1], 
                    Hue[0.7169971729064637, 1, 1], 
                    Hue[0.7169971729064637, 1, 1], 
                    Hue[0.7758915686233734, 1, 1], 
                    Hue[0.7758915686233734, 1, 1], 
                    Hue[0.8351048410044376, 1, 1], 
                    Hue[0.8351048410044376, 1, 1], 
                    Hue[0.8997708000560913, 1, 1], 
                    Hue[0.8997708000560913, 1, 1], 
                    Hue[0.9703622628350036, 1, 1], 
                    Hue[0.9703622628350036, 1, 1], 
                    Hue[0.04204741694621801, 1, 1], 
                    Hue[0.04204741694621801, 1, 1], 
                    Hue[0.1084845737470324, 1, 1], 
                    Hue[0.1084845737470324, 1, 1], 
                    Hue[0.1701666187456011, 1, 1], 
                    Hue[0.1701666187456011, 1, 1], 
                    Hue[0.2313775399901313, 1, 1], 
                    Hue[0.2313775399901313, 1, 1], 
                    Hue[0.2976877502367062, 1, 1], 
                    Hue[0.2976877502367062, 1, 1], 
                    Hue[0.3711740550744846, 1, 1], 
                    Hue[0.3711740550744846, 1, 1], 
                    Hue[0.44687625426845595`, 1, 1], 
                    Hue[0.44687625426845595`, 1, 1], 
                    Hue[0.5176126759525406, 1, 1], 
                    Hue[0.5176126759525406, 1, 1], 
                    Hue[0.5819664960418984, 1, 1], 
                    Hue[0.5819664960418984, 1, 1], 
                    Hue[0.6448711364223789, 1, 1], 
                    Hue[0.6448711364223789, 1, 1], 
                    Hue[0.7127443005598428, 1, 1], 
                    Hue[0.7127443005598428, 1, 1], 
                    Hue[0.7888911244253934, 1, 1], 
                    Hue[0.7888911244253934, 1, 1], 
                    Hue[0.8685972235020938, 1, 1], 
                    Hue[0.8685972235020938, 1, 1], 
                    Hue[0.9429413869593951, 1, 1], 
                    Hue[0.9429413869593951, 1, 1], 
                    Hue[0.009597528089488936, 1, 1], 
                    Hue[0.009597528089488936, 1, 1], 
                    Hue[0.07369556643187233, 1, 1], 
                    Hue[0.07369556643187233, 1, 1], 
                    Hue[0.14259002452799643`, 1, 1], 
                    Hue[0.14259002452799643`, 1, 1], 
                    Hue[0.2208055343087441, 1, 1], 
                    Hue[0.2208055343087441, 1, 1], 
                    Hue[0.30397005118067305`, 1, 1], 
                    Hue[0.30397005118067305`, 1, 1], 
                    Hue[0.3814022118325686, 1, 1], 
                    Hue[0.3814022118325686, 1, 1], 
                    Hue[0.44958930198492464`, 1, 1], 
                    Hue[0.44958930198492464`, 1, 1], 
                    Hue[0.5137878132576624, 1, 1], 
                    Hue[0.5137878132576624, 1, 1], 
                    Hue[0.5819989623965363, 1, 1], 
                    Hue[0.5819989623965363, 1, 1], 
                    Hue[0.6601775740658046, 1, 1], 
                    Hue[0.6601775740658046, 1, 1], 
                    Hue[0.745202837419231, 1, 1], 
                    Hue[0.745202837419231, 1, 1], 
                    Hue[0.8250475018811899, 1, 1], 
                    Hue[0.8250475018811899, 1, 1], 
                    Hue[0.8938729371910848, 1, 1], 
                    Hue[0.8938729371910848, 1, 1], 
                    Hue[0.9561034162561497, 1, 1], 
                    Hue[0.9561034162561497, 1, 1], 
                    Hue[0.019638189549080177`, 1, 1], 
                    Hue[0.019638189549080177`, 1, 1], 
                    Hue[0.09164212060033193, 1, 1], 
                    Hue[0.09164212060033193, 1, 1], 
                    Hue[0.17317222581323283`, 1, 1], 
                    Hue[0.17317222581323283`, 1, 1], 
                    Hue[0.2541781134560693, 1, 1], 
                    Hue[0.2541781134560693, 1, 1], 
                    Hue[0.3241269011239155, 1, 1], 
                    Hue[0.3241269011239155, 1, 1], 
                    Hue[0.3830525677867494, 1, 1], 
                    Hue[0.3830525677867494, 1, 1], 
                    Hue[0.43671758530245763`, 1, 1], 
                    Hue[0.43671758530245763`, 1, 1], 
                    Hue[0.491429567541259, 1, 1], 
                    Hue[0.491429567541259, 1, 1], 
                    Hue[0.5524263664053911, 1, 1], 
                    Hue[0.5524263664053911, 1, 1], 
                    Hue[0.6213032633035116, 1, 1], 
                    Hue[0.6213032633035116, 1, 1], 
                    Hue[0.6922386416207904, 1, 1], 
                    Hue[0.6922386416207904, 1, 1], 
                    Hue[0.7556807223227779, 1, 1], 
                    Hue[0.7556807223227779, 1, 1], 
                    Hue[0.8075261375163918, 1, 1], 
                    Hue[0.8075261375163918, 1, 1], 
                    Hue[0.8494469370502941, 1, 1], 
                    Hue[0.8494469370502941, 1, 1], 
                    Hue[0.8845456784920832, 1, 1], 
                    Hue[0.8845456784920832, 1, 1], 
                    Hue[0.9153607707770892, 1, 1], 
                    Hue[0.9153607707770892, 1, 1], 
                    Hue[0.9436483964312276, 1, 1], 
                    Hue[0.9436483964312276, 1, 1], 
                    Hue[0.9705913152413616, 1, 1], 
                    Hue[0.9705913152413616, 1, 1], 
                    Hue[0.9969991385198878, 1, 1], 
                    Hue[0.9969991385198878, 1, 1], 
                    Hue[0.023443344689877202`, 1, 1], 
                    Hue[0.023443344689877202`, 1, 1], 
                    Hue[0.0503392189712252, 1, 1], 
                    Hue[0.0503392189712252, 1, 1], 
                    Hue[0.07799583048219152, 1, 1], 
                    Hue[0.07799583048219152, 1, 1], 
                    Hue[0.10664739890542389`, 1, 1], 
                    Hue[0.10664739890542389`, 1, 1], 
                    Hue[0.13647223751425389`, 1, 1], 
                    Hue[0.13647223751425389`, 1, 1], 
                    Hue[0.16760651012276648`, 1, 1], 
                    Hue[0.16760651012276648`, 1, 1], 
                    Hue[0.20015193386020552`, 1, 1], 
                    Hue[0.20015193386020552`, 1, 1], 
                    Hue[0.2341836723124326, 1, 1], 
                    Hue[0.2341836723124326, 1, 1], 
                    Hue[0.2697549633000815, 1, 1], 
                    Hue[0.2697549633000815, 1, 1], 
                    Hue[0.3069010347031737, 1, 1], 
                    Hue[0.3069010347031737, 1, 1], 
                    Hue[0.34564242381955096`, 1, 1], 
                    Hue[0.34564242381955096`, 1, 1], 
                    Hue[0.385988114107708, 1, 1], 
                    Hue[0.385988114107708, 1, 1], 
                    Hue[0.4279373960933569, 1, 1], 
                    Hue[0.4279373960933569, 1, 1], 
                    Hue[0.4714821138884691, 1, 1], 
                    Hue[0.4714821138884691, 1, 1], 
                    Hue[0.516608142439269, 1, 1], 
                    Hue[0.516608142439269, 1, 1], 
                    Hue[0.5632967986431676, 1, 1], 
                    Hue[0.5632967986431676, 1, 1], 
                    Hue[0.6115262291158869, 1, 1], 
                    Hue[0.6115262291158869, 1, 1], 
                    Hue[0.6612723650240759, 1, 1], 
                    Hue[0.6612723650240759, 1, 1], 
                    Hue[0.7125082110086003, 1, 1], 
                    Hue[0.7125082110086003, 1, 1], 
                    Hue[0.7652067245285928, 1, 1], 
                    Hue[0.7652067245285928, 1, 1], 
                    Hue[0.8193404899939781, 1, 1], 
                    Hue[0.8193404899939781, 1, 1], 
                    Hue[0.8748814082258843, 1, 1], 
                    Hue[0.8748814082258843, 1, 1], 
                    Hue[0.93180121767383, 1, 1], 
                    Hue[0.93180121767383, 1, 1], 
                    Hue[0.9900732339915105, 1, 1], 
                    Hue[0.9900732339915105, 1, 1], 
                    Hue[0.04966897366675908, 1, 1], 
                    Hue[0.04966897366675908, 1, 1], 
                    Hue[0.11056355278904713`, 1, 1], 
                    Hue[0.11056355278904713`, 1, 1], 
                    Hue[0.17273127731108673`, 1, 1], 
                    Hue[0.17273127731108673`, 1, 1], 
                    Hue[0.23614688428905453`, 1, 1], 
                    Hue[0.23614688428905453`, 1, 1], 
                    Hue[0.30078631795382943`, 1, 1], 
                    Hue[0.30078631795382943`, 1, 1], 
                    Hue[0.36662673011340297`, 1, 1], 
                    Hue[0.36662673011340297`, 1, 1], 
                    Hue[0.4336462625630184, 1, 1], 
                    Hue[0.4336462625630184, 1, 1], 
                    Hue[0.5018243590466764, 1, 1], 
                    Hue[0.5018243590466764, 1, 1], 
                    Hue[0.5711391738638567, 1, 1], 
                    Hue[0.5711391738638567, 1, 1], 
                    Hue[0.6415712630612694, 1, 1], 
                    Hue[0.6415712630612694, 1, 1], 
                    Hue[0.7131026687380215, 1, 1], 
                    Hue[0.7131026687380215, 1, 1], 
                    Hue[0.785714952794411, 1, 1], 
                    Hue[0.785714952794411, 1, 1], 
                    Hue[0.85939125988999, 1, 1], 
                    Hue[0.85939125988999, 1, 1], 
                    Hue[0.9341142030507813, 1, 1], 
                    Hue[0.9341142030507813, 1, 1], 
                    Hue[0.009867814283582065, 1, 1], 
                    Hue[0.009867814283582065, 1, 1], 
                    Hue[0.08663975672705043, 1, 1], 
                    Hue[0.08663975672705043, 1, 1], 
                    Hue[0.16441331632110967`, 1, 1], 
                    Hue[0.16441331632110967`, 1, 1], 
                    Hue[0.24317140299829, 1, 1], 
                    Hue[0.24317140299829, 1, 1], 
                    Hue[0.32290236585478704`, 1, 1], 
                    Hue[0.32290236585478704`, 1, 1], 
                    Hue[0.4035943587363709, 1, 1], 
                    Hue[0.4035943587363709, 1, 1], 
                    Hue[0.48523999204304397`, 1, 1], 
                    Hue[0.48523999204304397`, 1, 1], 
                    Hue[0.5678193374677074, 1, 1], 
                    Hue[0.5678193374677074, 1, 1], 
                    Hue[0.6513185536279276, 1, 1], 
                    Hue[0.6513185536279276, 1, 1], 
                    Hue[0.7357401903243395, 1, 1], 
                    Hue[0.7357401903243395, 1, 1], 
                    Hue[0.8210622956913834, 1, 1], 
                    Hue[0.8210622956913834, 1, 1], 
                    Hue[0.9072682139525383, 1, 1], 
                    Hue[0.9072682139525383, 1, 1], 
                    Hue[0.9943604247210516, 1, 1], 
                    Hue[0.9943604247210516, 1, 1], 
                    Hue[0.08232920234124495, 1, 1], 
                    Hue[0.08232920234124495, 1, 1], 
                    Hue[0.17115182211425745`, 1, 1], 
                    Hue[0.17115182211425745`, 1, 1], 
                    Hue[0.2608259833213761, 1, 1], 
                    Hue[0.2608259833213761, 1, 1], 
                    Hue[0.3513592109082063, 1, 1], 
                    Hue[0.3513592109082063, 1, 1], 
                    Hue[0.44272159494008767`, 1, 1], 
                    Hue[0.44272159494008767`, 1, 1], 
                    Hue[0.5348962505409033, 1, 1], 
                    Hue[0.5348962505409033, 1, 1], 
                    Hue[0.6279072641081033, 1, 1], 
                    Hue[0.6279072641081033, 1, 1], 
                    Hue[0.7217267880302329, 1, 1], 
                    Hue[0.7217267880302329, 1, 1], 
                    Hue[0.816313267532884, 1, 1], 
                    Hue[0.816313267532884, 1, 1], 
                    Hue[0.9117512965679803, 1, 1], 
                    Hue[0.9117512965679803, 1, 1], 
                    Hue[0.008047879736664992, 1, 1], 
                    Hue[0.008047879736664992, 1, 1], 
                    Hue[0.10501187209075581`, 1, 1], 
                    Hue[0.10501187209075581`, 1, 1], 
                    Hue[0.20270535185021696`, 1, 1], 
                    Hue[0.20270535185021696`, 1, 1], 
                    Hue[0.30133780047475683`, 1, 1], 
                    Hue[0.30133780047475683`, 1, 1], 
                    Hue[0.40068737757147616`, 1, 1], 
                    Hue[0.40068737757147616`, 1, 1], 
                    Hue[0.500621695745771, 1, 1], 
                    Hue[0.500621695745771, 1, 1], 
                    Hue[0.6015194562805606, 1, 1], 
                    Hue[0.6015194562805606, 1, 1], 
                    Hue[0.7033866071661006, 1, 1], 
                    Hue[0.7033866071661006, 1, 1], 
                    Hue[0.8057600247529666, 1, 1], 
                    Hue[0.8057600247529666, 1, 1], 
                    Hue[0.9087115831851035, 1, 1], 
                    Hue[0.9087115831851035, 1, 1], 
                    Hue[0.012728494365962887`, 1, 1], 
                    Hue[0.012728494365962887`, 1, 1], 
                    Hue[0.11768230946122678`, 1, 1], 
                    Hue[0.11768230946122678`, 1, 1], 
                    Hue[0.22299007300419238`, 1, 1], 
                    Hue[0.22299007300419238`, 1, 1], 
                    Hue[0.32878942455954685`, 1, 1], 
                    Hue[0.32878942455954685`, 1, 1], 
                    Hue[0.43566326944187145`, 1, 1], 
                    Hue[0.43566326944187145`, 1, 1], 
                    Hue[0.5430660387149626, 1, 1], 
                    Hue[0.5430660387149626, 1, 1], 
                    Hue[0.6501242831582152, 1, 1], 
                    Hue[0.6501242831582152, 1, 1], 
                    Hue[0.758048676916268, 1, 1], 
                    Hue[0.758048676916268, 1, 1], 
                    Hue[0.8686784353266866, 1, 1], 
                    Hue[0.8686784353266866, 1, 1], 
                    Hue[0.9804881391600404, 1, 1], 
                    Hue[0.9804881391600404, 1, 1], 
                    Hue[0.08932909517187086, 1, 1], 
                    Hue[0.08932909517187086, 1, 1], 
                    Hue[0.1955941259474388, 1, 1], 
                    Hue[0.1955941259474388, 1, 1], 
                    Hue[0.30741713430835516`, 1, 1], 
                    Hue[0.30741713430835516`, 1, 1], 
                    Hue[0.4270155355366292, 1, 1], 
                    Hue[0.4270155355366292, 1, 1], 
                    Hue[0.5429924302256508, 1, 1], 
                    Hue[0.5429924302256508, 1, 1], 
                    Hue[0.6483558235535518, 1, 1], 
                    Hue[0.6483558235535518, 1, 1], 
                    Hue[0.7667154238986771, 1, 1], 
                    Hue[0.7667154238986771, 1, 1], 
                    Hue[0.9142402763382171, 1, 1], 
                    Hue[0.9142402763382171, 1, 1], 
                    Hue[0.07850281472384095, 1, 1], 
                    Hue[0.07850281472384095, 1, 1], 
                    Hue[0.2333380105758965, 1, 1], 
                    Hue[0.2333380105758965, 1, 1], 
                    Hue[0.23584220459241986`, 1, 1], 
                    Hue[0.23584220459241986`, 1, 1], 
                    Hue[0.2548056612472624, 1, 1], 
                    Hue[0.2548056612472624, 1, 1], 
                    Hue[0.41845090004777136`, 1, 1], 
                    Hue[0.41845090004777136`, 1, 1], 
                    Hue[0.5707092498069801, 1, 1], 
                    Hue[0.5707092498069801, 1, 1], 
                    Hue[0.33232299147995975`, 1, 1], 
                    Hue[0.33232299147995975`, 1, 1], 
                    Hue[0.531991234171352, 1, 1], 
                    Hue[0.531991234171352, 1, 1], 
                    Hue[0.7431920582950459, 1, 1], 
                    Hue[0.7431920582950459, 1, 1], 
                    Hue[0.9572424609776871, 1, 1], 
                    Hue[0.9572424609776871, 1, 1], 
                    Hue[0.16180360112036146`, 1, 1], 
                    Hue[0.16180360112036146`, 1, 1], 
                    Hue[0.30347340649352716`, 1, 1], 
                    Hue[0.30347340649352716`, 1, 1], 
                    Hue[0.35228833642655205`, 1, 1], 
                    Hue[0.35228833642655205`, 1, 1], 
                    Hue[0.5320890788639869, 1, 1], 
                    Hue[0.5320890788639869, 1, 1], 
                    Hue[0.7606664269256596, 1, 1], 
                    Hue[0.7606664269256596, 1, 1], 
                    Hue[0.00496704327457589, 1, 1], 
                    Hue[0.00496704327457589, 1, 1], 
                    Hue[0.26145708925610656`, 1, 1], 
                    Hue[0.26145708925610656`, 1, 1], 
                    Hue[0.543029142422131, 1, 1], 
                    Hue[0.543029142422131, 1, 1], 
                    Hue[0.8648449595129252, 1, 1], 
                    Hue[0.8648449595129252, 1, 1], 
                    Hue[0.16854952740027157`, 1, 1], 
                    Hue[0.16854952740027157`, 1, 1], 
                    Hue[0.43839966896805005`, 1, 1], 
                    Hue[0.43839966896805005`, 1, 1], 
                    Hue[0.7007213800317906, 1, 1], 
                    Hue[0.7007213800317906, 1, 1], 
                    Hue[0.9650895094938333, 1, 1], 
                    Hue[0.9650895094938333, 1, 1], 
                    Hue[0.23385603595579127`, 1, 1], 
                    Hue[0.23385603595579127`, 1, 1], 
                    Hue[0.5085921548983394, 1, 1], 
                    Hue[0.5085921548983394, 1, 1], 
                    Hue[0.7925154147969324, 1, 1], 
                    Hue[0.7925154147969324, 1, 1], 
                    Hue[0.09250870720938896, 1, 1], 
                    Hue[0.09250870720938896, 1, 1], 
                    Hue[0.417902691531048, 1, 1], 
                    Hue[0.417902691531048, 1, 1], 
                    Hue[0.7584781495977884, 1, 1]}],
                   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
                   Axes->{True, None},
                   AxesOrigin->{0, 0},
                   
                   Epilog->LineBox[{{1, 1.3438449168047713`*^-6}, {
                    2, 5.050073771653918*^-7}, {3, 6.855638028766111*^-7}, {
                    4, 3.8179203479447143`*^-7}, {5, 7.508603360329421*^-7}, {
                    6, 9.863227720421532*^-7}, {7, 1.2155460876151615`*^-6}, {
                    8, 1.3764484103715475`*^-6}, {9, 8.860643544638477*^-7}, {
                    10, 7.970423102534875*^-7}, {11, 7.277582299714451*^-7}, {
                    12, 8.800794860302492*^-7}, {13, 3.582363026860292*^-7}, {
                    14, 6.570367993999595*^-7}, {
                    15, 4.2600734910275776`*^-7}, {
                    16, 3.117866987224697*^-7}, {17, 7.755335491452408*^-8}, {
                    18, 1.7280895051173313`*^-7}, {
                    19, 3.238175698959648*^-7}, {20, 6.444259440967693*^-7}, {
                    21, 6.001646297483413*^-7}, {22, 5.308526312369224*^-7}, {
                    23, 5.501795048657428*^-7}, {24, 5.880886923884123*^-7}, {
                    25, 3.6339121006225795`*^-7}, {
                    26, 7.828114432257981*^-8}, {27, 4.009478784132785*^-7}, {
                    28, 1.5077047672234011`*^-7}, {
                    29, 3.7956738003471573`*^-7}, {
                    30, 3.675254922703899*^-7}, {31, 5.729003618224869*^-7}, {
                    32, 4.896950804797941*^-7}, {
                    33, 3.6930699272705243`*^-7}, {
                    34, 2.2832008136322795`*^-7}, {
                    35, 2.672340564393481*^-7}, {36, 5.297091315972509*^-7}, {
                    37, 4.4744785302170775`*^-7}, {
                    38, 4.707307285350399*^-7}, {
                    39, 3.2214292679393745`*^-7}, {
                    40, 6.049555476280254*^-7}, {
                    41, 3.2998149166183465`*^-7}, {
                    42, 1.3751834669131524`*^-7}, {
                    43, 5.158705154512046*^-7}, {44, 8.862478157837731*^-7}, {
                    45, 9.80485350079701*^-7}, {
                    46, 1.0092223842856086`*^-6}, {
                    47, 8.579114723324283*^-7}, {
                    48, 1.7486290941597078`*^-6}, {
                    49, 9.054521848466083*^-7}, {50, 8.259347454515807*^-7}, {
                    51, 1.070798592799538*^-6}, {52, 8.190627097875383*^-7}, {
                    53, 9.593272749811963*^-7}, {54, 7.478993037142122*^-7}, {
                    55, 1.0712710648962929`*^-6}, {
                    56, 1.6523331545411954`*^-6}, {
                    57, 1.5654480130518108`*^-6}, {
                    58, 1.9173987565508257`*^-6}, {
                    59, 1.4079807461187133`*^-6}, {
                    60, 1.6069212552473312`*^-6}, {
                    61, 2.227450433275266*^-6}, {62, 2.487365436638597*^-6}, {
                    63, 5.026716832323899*^-6}, {
                    64, 6.0071582606822355`*^-6}, {
                    65, 7.72925896247523*^-6}, {66, 7.968046530132231*^-6}, {
                    67, 9.750304369810336*^-6}, {
                    68, 0.000012010820018980802`}, {
                    69, 0.00001695662743718514}, {
                    70, 0.000021838288456610445`}, {
                    71, 0.000027545364569211193`}, {
                    72, 0.00003280517031879668}, {
                    73, 0.0000397289539871845}, {
                    74, 0.00004990496590362894}, {
                    75, 0.00006507609589356898}, {
                    76, 0.00008400469294037497}, {
                    77, 0.00010488977155735191`}, {
                    78, 0.00012595144863133373`}, {
                    79, 0.0001504478447275558}, {
                    80, 0.00018302024466741338`}, {81, 0.0002306537564104}, {
                    82, 0.00029305570926458355`}, {
                    83, 0.0003639439168739834}, {84, 0.0004355366064725881}, {
                    85, 0.0005094748215040786}, {86, 0.0006043612252946217}, {
                    87, 0.0007452875180149037}, {88, 0.0009368617929305138}, {
                    89, 0.0011538333711350818`}, {
                    90, 0.0013649377313992438`}, {91, 0.00156766787445566}, {
                    92, 0.0018152004024004707`}, {
                    93, 0.0021935312331993633`}, {
                    94, 0.0027275459679547242`}, {
                    95, 0.0033369844639424976`}, {96, 0.003904867095906774}, {
                    97, 0.004401431136862781}, {98, 0.004978110175082453}, {
                    99, 0.005904094500136208}, {100, 0.007275753779572891}, {
                    101, 0.008847806987205883}, {
                    102, 0.010248771318140405`}, {
                    103, 0.011345142560833464`}, {104, 0.01253285738118848}, {
                    105, 0.014599867811876072`}, {
                    106, 0.017867544513828392`}, {
                    107, 0.021667703537470377`}, {
                    108, 0.024950541768689296`}, {
                    109, 0.027238495986368776`}, {110, 0.02940311931934233}, {
                    111, 0.03351230587360265}, {112, 0.040729395517057}, {
                    113, 0.049659882684868796`}, {
                    114, 0.057673069959102916`}, {115, 0.06308854116158401}, {
                    116, 0.06690215058106808}, {117, 0.07339236308727035}, {
                    118, 0.0870608107984569}, {119, 0.10753201867699332`}, {
                    120, 0.12995958304616997`}, {121, 0.14911920520456104`}, {
                    122, 0.16244801066148032`}, {123, 0.1721619288517853}, {
                    124, 0.1862680126406907}, {125, 0.21539485625645893`}, {
                    126, 0.264976114350546}, {127, 0.3320452268584973}, {
                    128, 0.40947303322547146`}, {129, 0.4899994582841429}, {
                    130, 0.5676920888365797}, {131, 0.6382087421275998}, {
                    132, 0.6987166706429022}, {133, 0.7476863091312034}, {
                    134, 0.7846285759743065}, {135, 0.8098267519905125}, {
                    136, 0.8240797004065663}, {137, 0.8284917687953577}, {
                    138, 0.8243133093150494}, {139, 0.8128235362431939}, {
                    140, 0.7952521187231224}, {141, 0.7727324104388693}, {
                    142, 0.7462865979566424}, {143, 0.7168179897770113}, {
                    144, 0.6851062078648421}, {145, 0.6518299674536496}, {
                    146, 0.6175629545140168}, {147, 0.5827943279857422}, {
                    148, 0.5479388160825146}, {149, 0.5133441422392822}, {
                    150, 0.4792999587345991}, {151, 0.4460434487292549}, {
                    152, 0.4137704736498192}, {153, 0.38263950511929795`}, {
                    154, 0.3527714554612603}, {155, 0.3242602513218213}, {
                    156, 0.2971724532008716}, {157, 0.2715527340965929}, {
                    158, 0.24742353029016012`}, {159, 0.2247939525022326}, {
                    160, 0.2036523446691271}, {161, 0.18397777905830978`}, {
                    162, 0.16573598265025766`}, {163, 0.14888508429895186`}, {
                    164, 0.13337462413585321`}, {165, 0.11914782680738818`}, {
                    166, 0.10614276729661636`}, {167, 0.09429614807521265}, {
                    168, 0.08354068220910087}, {169, 0.07380815169384132}, {
                    170, 0.06503013721849481}, {171, 0.05713849101411949}, {
                    172, 0.0500671793562392}, {173, 0.04375092686238379}, {
                    174, 0.03812614549723036}, {175, 0.033133619668249}, {
                    176, 0.028716380125324337`}, {
                    177, 0.024820460704744866`}, {
                    178, 0.021394451298417515`}, {
                    179, 0.018390240342439877`}, {
                    180, 0.015765329077248633`}, {
                    181, 0.013478310748997345`}, {
                    182, 0.011490914705107165`}, {183, 0.00977006632718522}, {
                    184, 0.008284577191326116}, {185, 0.007005526316663681}, {
                    186, 0.005907672408679385}, {187, 0.004968658789847037}, {
                    188, 0.0041674666728667275`}, {
                    189, 0.003485596864708379}, {
                    190, 0.0029075710061982753`}, {
                    191, 0.002418950582181375}, {
                    192, 0.0020066691709867504`}, {
                    193, 0.001660402413691285}, {194, 0.001370625758092963}, {
                    195, 0.0011277504204931037`}, {
                    196, 0.0009250077954023635}, {
                    197, 0.0007573544376017891}, {
                    198, 0.0006184492461656033}, {
                    199, 0.0005031056181638236}, {
                    200, 0.0004084998824557169}, {
                    201, 0.0003311899695683833}, {
                    202, 0.00026737501939824715`}, {
                    203, 0.00021498646877285307`}, {
                    204, 0.00017267039016009137`}, {
                    205, 0.00013842908104329103`}, {
                    206, 0.00011039992670949446`}, {
                    207, 0.00008764947102213703}, {
                    208, 0.00006952916199485672}, {
                    209, 0.00005501169866745292}, {
                    210, 0.00004323365295301365}, {
                    211, 0.00003393569435602977}, {
                    212, 0.00002681571119707127}, {
                    213, 0.00002113504143743491}, {
                    214, 0.00001633126975793672}, {
                    215, 0.000012465478675476022`}, {
                    216, 9.765833054529133*^-6}, {
                    217, 7.891181340785691*^-6}, {
                    218, 6.189870811177567*^-6}, {
                    219, 4.591315917875258*^-6}, {
                    220, 3.560800344707003*^-6}, {
                    221, 3.0834467563532345`*^-6}, {
                    222, 2.532692419875134*^-6}, {
                    223, 1.6778020217233635`*^-6}, {
                    224, 7.648025130755926*^-7}, {
                    225, 2.3711932750827905`*^-7}, {
                    226, 3.7414362600598133`*^-7}, {
                    227, 3.3167912302392594`*^-7}, {
                    228, 7.902566195493602*^-8}, {
                    229, 2.654603186421689*^-7}, {
                    230, 5.60193173383047*^-7}, {
                    231, 7.079174757880079*^-7}, {
                    232, 6.63482538974332*^-7}, {
                    233, 4.624047157019311*^-7}, {
                    234, 2.3394632954248082`*^-7}, {
                    235, 2.472913142786779*^-7}, {
                    236, 3.69627078493883*^-7}, {
                    237, 4.0112951083116867`*^-7}, {
                    238, 3.426030704612548*^-7}, {
                    239, 2.425202527781865*^-7}, {
                    240, 1.5532513664155531`*^-7}, {
                    241, 1.2833865924653207`*^-7}, {
                    242, 1.7388447828129748`*^-7}, {
                    243, 2.5838021233589175`*^-7}, {
                    244, 3.493734919940622*^-7}, {
                    245, 4.175911984705985*^-7}, {
                    246, 4.414058280392368*^-7}, {
                    247, 4.139526252759938*^-7}, {
                    248, 3.453080914589958*^-7}, {
                    249, 2.599588323658625*^-7}, {
                    250, 1.9088605678547244`*^-7}, {
                    251, 1.615070789048257*^-7}}],
                   Frame->True,
                   FrameTicks->{{{1., 
                    FormBox[
                    RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
                    FormBox[
                    RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
                    FormBox[
                    RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
                    FormBox[
                    RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
                    FormBox["0", TraditionalForm]}, {157.25, 
                    FormBox["2.5`", TraditionalForm]}, {188.5, 
                    FormBox["5", TraditionalForm]}, {219.75, 
                    FormBox["7.5`", TraditionalForm]}, {251., 
                    FormBox["10", TraditionalForm]}}, Automatic, None, None},
                   PlotRange->{-0.1, 1.1},
                   PlotRangeClipping->True,
                   PlotRangePadding->{
                    Scaled[0.02], Automatic}], 13->
                  GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF13tcTfkaBvAVEsOhmXEJyc4lI4QpoRl5VKgo3XRVrdpdd7VzHRmD37iO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                    "], {
                    {GrayLevel[0.5], EdgeForm[None], 
                    GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ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                    
                    "]]]}, {{}, {}, {
                    {Hue[0.46903121084379074`, 1, 1], LineBox[{1, 2}]}, 
                    {Hue[0.3276739181280018, 1, 1], LineBox[{2, 3}]}, 
                    {Hue[0.27302120989995315`, 1, 1], LineBox[{3, 4}]}, 
                    {Hue[0.1503982909211788, 1, 1], LineBox[{4, 5}]}, 
                    {Hue[0.1159540677729317, 1, 1], LineBox[{5, 6}]}, 
                    {Hue[0.9885034259412886, 1, 1], LineBox[{6, 7}]}, 
                    {Hue[0.555919303286334, 1, 1], LineBox[{7, 8}]}, 
                    {Hue[0.014674981446927316`, 1, 1], LineBox[{8, 9}]}, 
                    {Hue[0.2505963104170238, 1, 1], LineBox[{9, 10}]}, 
                    {Hue[0.08114438222060529, 1, 1], LineBox[{10, 11}]}, 
                    {Hue[0.01429524804121616, 1, 1], LineBox[{11, 12}]}, 
                    {Hue[0.887197895954603, 1, 1], LineBox[{12, 13}]}, 
                    {Hue[0.776711378633425, 1, 1], LineBox[{13, 14}]}, 
                    {Hue[0.8220325704507755, 1, 1], LineBox[{14, 15}]}, 
                    {Hue[0.6796760798270274, 1, 1], LineBox[{15, 16}]}, 
                    {Hue[0.7081680584902381, 1, 1], LineBox[{16, 17}]}, 
                    {Hue[0.5453095997956825, 1, 1], LineBox[{17, 18}]}, 
                    {Hue[0.4431782547061368, 1, 1], LineBox[{18, 19}]}, 
                    {Hue[0.24332838511522192`, 1, 1], LineBox[{19, 20}]}, 
                    {Hue[0.11867547147282674`, 1, 1], LineBox[{20, 21}]}, 
                    {Hue[0.20671099307287558`, 1, 1], LineBox[{21, 22}]}, 
                    {Hue[0.874019357415786, 1, 1], LineBox[{22, 23}]}, 
                    {Hue[0.6591515413554456, 1, 1], LineBox[{23, 24}]}, 
                    {Hue[0.18066228033355544`, 1, 1], LineBox[{24, 25}]}, 
                    {Hue[0.6678350196899061, 1, 1], LineBox[{25, 26}]}, 
                    {Hue[0.5241258581961699, 1, 1], LineBox[{26, 27}]}, 
                    {Hue[0.1431727845363231, 1, 1], LineBox[{27, 28}]}, 
                    {Hue[0.7756794556885923, 1, 1], LineBox[{28, 29}]}, 
                    {Hue[0.5095140846201118, 1, 1], LineBox[{29, 30}]}, 
                    {Hue[0.6861711805072839, 1, 1], LineBox[{30, 31}]}, 
                    {Hue[0.8636511930406199, 1, 1], LineBox[{31, 32}]}, 
                    {Hue[0.5811421883741066, 1, 1], LineBox[{32, 33}]}, 
                    {Hue[0.5145867440569433, 1, 1], LineBox[{33, 34}]}, 
                    {Hue[0.3775090338495559, 1, 1], LineBox[{34, 35}]}, 
                    {Hue[0.24014184273348146`, 1, 1], LineBox[{35, 36}]}, 
                    {Hue[0.0808948788107402, 1, 1], LineBox[{36, 37}]}, 
                    {Hue[0.00828126453665834, 1, 1], LineBox[{37, 38}]}, 
                    {Hue[0.8486886497933208, 1, 1], LineBox[{38, 39}]}, 
                    {Hue[0.7430682886052937, 1, 1], LineBox[{39, 40}]}, 
                    {Hue[0.661458391422076, 1, 1], LineBox[{40, 41}]}, 
                    {Hue[0.502941759363241, 1, 1], LineBox[{41, 42}]}, 
                    {Hue[0.3928074615720261, 1, 1], LineBox[{42, 43}]}, 
                    {Hue[0.17436336899653096`, 1, 1], LineBox[{43, 44}]}, 
                    {Hue[0.19971063365582253`, 1, 1], LineBox[{44, 45}]}, 
                    {Hue[0.9072160164407977, 1, 1], LineBox[{45, 46}]}, 
                    {Hue[0.7435128384384582, 1, 1], LineBox[{46, 47}]}, 
                    {Hue[0.7542167129026032, 1, 1], LineBox[{47, 48}]}, 
                    {Hue[0.7220480636268023, 1, 1], LineBox[{48, 49}]}, 
                    {Hue[0.5960170125858861, 1, 1], LineBox[{49, 50}]}, 
                    {Hue[0.504394885168866, 1, 1], LineBox[{50, 51}]}, 
                    {Hue[0.5980761131160199, 1, 1], LineBox[{51, 52}]}, 
                    {Hue[0.39175536296490443`, 1, 1], LineBox[{52, 53}]}, 
                    {Hue[0.3080870960109756, 1, 1], LineBox[{53, 54}]}, 
                    {Hue[0.22637965615689903`, 1, 1], LineBox[{54, 55}]}, 
                    {Hue[0.04078695456709334, 1, 1], LineBox[{55, 56}]}, 
                    {Hue[0.9979339064190902, 1, 1], LineBox[{56, 57}]}, 
                    {Hue[0.9379513734489464, 1, 1], LineBox[{57, 58}]}, 
                    {Hue[0.7568969236098246, 1, 1], LineBox[{58, 59}]}, 
                    {Hue[0.6044031428332308, 1, 1], LineBox[{59, 60}]}, 
                    {Hue[0.546835631606627, 1, 1], LineBox[{60, 61}]}, 
                    {Hue[0.48445950589899756`, 1, 1], LineBox[{61, 62}]}, 
                    {Hue[0.45836739143999605`, 1, 1], LineBox[{62, 63}]}, 
                    {Hue[0.4050876322941351, 1, 1], LineBox[{63, 64}]}, 
                    {Hue[0.3310440859556128, 1, 1], LineBox[{64, 65}]}, 
                    {Hue[0.8952884846740966, 1, 1], LineBox[{65, 66}]}, 
                    {Hue[0.849914391383057, 1, 1], LineBox[{66, 67}]}, 
                    {Hue[0.8210799392004464, 1, 1], LineBox[{67, 68}]}, 
                    {Hue[0.797476270966503, 1, 1], LineBox[{68, 69}]}, 
                    {Hue[0.7677111204344772, 1, 1], LineBox[{69, 70}]}, 
                    {Hue[0.7373996974703723, 1, 1], LineBox[{70, 71}]}, 
                    {Hue[0.6751855643100823, 1, 1], LineBox[{71, 72}]}, 
                    {Hue[0.24133341544108203`, 1, 1], LineBox[{72, 73}]}, 
                    {Hue[0.20414494146012305`, 1, 1], LineBox[{73, 74}]}, 
                    {Hue[0.17127988511038536`, 1, 1], LineBox[{74, 75}]}, 
                    {Hue[0.15097260410212796`, 1, 1], LineBox[{75, 76}]}, 
                    {Hue[0.1323205065914355, 1, 1], LineBox[{76, 77}]}, 
                    {Hue[0.11577470259370665`, 1, 1], LineBox[{77, 78}]}, 
                    {Hue[0.40072236254851873`, 1, 1], LineBox[{78, 79}]}, 
                    {Hue[0.5642972190009667, 1, 1], LineBox[{79, 80}]}, 
                    {Hue[0.5486440775627401, 1, 1], LineBox[{80, 81}]}, 
                    {Hue[0.5323579238759228, 1, 1], LineBox[{81, 82}]}, 
                    {Hue[0.5186050976851513, 1, 1], LineBox[{82, 83}]}, 
                    {Hue[0.5076758901256943, 1, 1], LineBox[{83, 84}]}, 
                    {Hue[0.5231123375576997, 1, 1], LineBox[{84, 85}]}, 
                    {Hue[0.9249542200277258, 1, 1], LineBox[{85, 86}]}, 
                    {Hue[0.9326440595614945, 1, 1], LineBox[{86, 87}]}, 
                    {Hue[0.92414635791773, 1, 1], LineBox[{87, 88}]}, 
                    {Hue[0.914328727297987, 1, 1], LineBox[{88, 89}]}, 
                    {Hue[0.9075469174696386, 1, 1], LineBox[{89, 90}]}, 
                    {Hue[0.9162986729778786, 1, 1], LineBox[{90, 91}]}, 
                    {Hue[0.22438990344755358`, 1, 1], LineBox[{91, 92}]}, 
                    {Hue[0.3233701921895654, 1, 1], LineBox[{92, 93}]}, 
                    {Hue[0.3255239702566573, 1, 1], LineBox[{93, 94}]}, 
                    {Hue[0.32135596500062585`, 1, 1], LineBox[{94, 95}]}, 
                    {Hue[0.3185714943521732, 1, 1], LineBox[{95, 96}]}, 
                    {Hue[0.3260807234884014, 1, 1], LineBox[{96, 97}]}, 
                    {Hue[0.44731094999721865`, 1, 1], LineBox[{97, 98}]}, 
                    {Hue[0.718402716585326, 1, 1], LineBox[{98, 99}]}, 
                    {Hue[0.7378073209369546, 1, 1], LineBox[{99, 100}]}, 
                    {Hue[0.7400657932223309, 1, 1], LineBox[{100, 101}]}, 
                    {Hue[0.7412978514163524, 1, 1], LineBox[{101, 102}]}, 
                    {Hue[0.7498344855551069, 1, 1], LineBox[{102, 103}]}, 
                    {Hue[0.8080132497531595, 1, 1], LineBox[{103, 104}]}, 
                    {Hue[0.10678166006413865`, 1, 1], LineBox[{104, 105}]}, 
                    {Hue[0.15908580322017676`, 1, 1], LineBox[{105, 106}]}, 
                    {Hue[0.16950671056358974`, 1, 1], LineBox[{106, 107}]}, 
                    {Hue[0.1750900190849448, 1, 1], LineBox[{107, 108}]}, 
                    {Hue[0.18498588973514118`, 1, 1], LineBox[{108, 109}]}, 
                    {Hue[0.2205776860799222, 1, 1], LineBox[{109, 110}]}, 
                    {Hue[0.4486392775157134, 1, 1], LineBox[{110, 111}]}, 
                    {Hue[0.5823447125514417, 1, 1], LineBox[{111, 112}]}, 
                    {Hue[0.6062875308462049, 1, 1], LineBox[{112, 113}]}, 
                    {Hue[0.6170867116073702, 1, 1], LineBox[{113, 114}]}, 
                    {Hue[0.6280375327039411, 1, 1], LineBox[{114, 115}]}, 
                    {Hue[0.6498635797223722, 1, 1], LineBox[{115, 116}]}, 
                    {Hue[0.7294631065042633, 1, 1], LineBox[{116, 117}]}, 
                    {Hue[0.9619442698698356, 1, 1], LineBox[{117, 118}]}, 
                    {Hue[0.0358694738259074, 1, 1], LineBox[{118, 119}]}, 
                    {Hue[0.058560492456003414`, 1, 1], LineBox[{119, 120}]}, 
                    {Hue[0.07200337794989134, 1, 1], LineBox[{120, 121}]}, 
                    {Hue[0.08546181681129635, 1, 1], LineBox[{121, 122}]}, 
                    {Hue[0.10595949252681901`, 1, 1], LineBox[{122, 123}]}, 
                    {Hue[0.15064552410101642`, 1, 1], LineBox[{123, 124}]}, 
                    {Hue[0.2776938160586117, 1, 1], LineBox[{124, 125}]}, 
                    {Hue[0.42749515722735437`, 1, 1], LineBox[{125, 126}]}, 
                    {Hue[0.48338919600598795`, 1, 1], LineBox[{126, 127}]}, 
                    {Hue[0.5077457331726007, 1, 1], LineBox[{127, 128}]}, 
                    {Hue[0.522453270537981, 1, 1], LineBox[{128, 129}]}, 
                    {Hue[0.533643024834802, 1, 1], LineBox[{129, 130}]}, 
                    {Hue[0.5436190540030256, 1, 1], LineBox[{130, 131}]}, 
                    {Hue[0.5534698627463082, 1, 1], LineBox[{131, 132}]}, 
                    {Hue[0.5638043360423671, 1, 1], LineBox[{132, 133}]}, 
                    {Hue[0.5750106310394598, 1, 1], LineBox[{133, 134}]}, 
                    {Hue[0.5873592564502069, 1, 1], LineBox[{134, 135}]}, 
                    {Hue[0.6010505587702185, 1, 1], LineBox[{135, 136}]}, 
                    {Hue[0.6162377444445248, 1, 1], LineBox[{136, 137}]}, 
                    {Hue[0.633039328584811, 1, 1], LineBox[{137, 138}]}, 
                    {Hue[0.6515452342370982, 1, 1], LineBox[{138, 139}]}, 
                    {Hue[0.671822399406282, 1, 1], LineBox[{139, 140}]}, 
                    {Hue[0.6939174624732731, 1, 1], LineBox[{140, 141}]}, 
                    {Hue[0.7178601571527807, 1, 1], LineBox[{141, 142}]}, 
                    {Hue[0.743664904009497, 1, 1], LineBox[{142, 143}]}, 
                    {Hue[0.7713339799014306, 1, 1], LineBox[{143, 144}]}, 
                    {Hue[0.8008597716600634, 1, 1], LineBox[{144, 145}]}, 
                    {Hue[0.8322257562878227, 1, 1], LineBox[{145, 146}]}, 
                    {Hue[0.8654094415339608, 1, 1], LineBox[{146, 147}]}, 
                    {Hue[0.900382675750429, 1, 1], LineBox[{147, 148}]}, 
                    {Hue[0.9371137956725565, 1, 1], LineBox[{148, 149}]}, 
                    {Hue[0.9755688441042102, 1, 1], LineBox[{149, 150}]}, 
                    {Hue[0.015710822193555896`, 1, 1], LineBox[{150, 151}]}, 
                    {Hue[0.05750341769511634, 1, 1], LineBox[{151, 152}]}, 
                    {Hue[0.1009088276816796, 1, 1], LineBox[{152, 153}]}, 
                    {Hue[0.14588984153644521`, 1, 1], LineBox[{153, 154}]}, 
                    {Hue[0.19240917392454862`, 1, 1], LineBox[{154, 155}]}, 
                    {Hue[0.2404306955446981, 1, 1], LineBox[{155, 156}]}, 
                    {Hue[0.28991896844010645`, 1, 1], LineBox[{156, 157}]}, 
                    {Hue[0.34084052389359315`, 1, 1], LineBox[{157, 158}]}, 
                    {Hue[0.3931618756369694, 1, 1], LineBox[{158, 159}]}, 
                    {Hue[0.44685127226500027`, 1, 1], LineBox[{159, 160}]}, 
                    {Hue[0.5018784756128514, 1, 1], LineBox[{160, 161}]}, 
                    {Hue[0.5582146923969578, 1, 1], LineBox[{161, 162}]}, 
                    {Hue[0.6158309711166918, 1, 1], LineBox[{162, 163}]}, 
                    {Hue[0.6747017479612505, 1, 1], LineBox[{163, 164}]}, 
                    {Hue[0.7348012513906026, 1, 1], LineBox[{164, 165}]}, 
                    {Hue[0.7961058202286575, 1, 1], LineBox[{165, 166}]}, 
                    {Hue[0.8585911448709992, 1, 1], LineBox[{166, 167}]}, 
                    {Hue[0.9222357148841215, 1, 1], LineBox[{167, 168}]}, 
                    {Hue[0.9870184060201163, 1, 1], LineBox[{168, 169}]}, 
                    {Hue[0.05292099295597615, 1, 1], LineBox[{169, 170}]}, 
                    {Hue[0.11992120925220558`, 1, 1], LineBox[{170, 171}]}, 
                    {Hue[0.1880026820180965, 1, 1], LineBox[{171, 172}]}, 
                    {Hue[0.2571478502170692, 1, 1], LineBox[{172, 173}]}, 
                    {Hue[0.3273391787139689, 1, 1], LineBox[{173, 174}]}, 
                    {Hue[0.39856225931539346`, 1, 1], LineBox[{174, 175}]}, 
                    {Hue[0.47080188987063876`, 1, 1], LineBox[{175, 176}]}, 
                    {Hue[0.5440415604188835, 1, 1], LineBox[{176, 177}]}, 
                    {Hue[0.6182681067487781, 1, 1], LineBox[{177, 178}]}, 
                    {Hue[0.6934684017449322, 1, 1], LineBox[{178, 179}]}, 
                    {Hue[0.7696304170032996, 1, 1], LineBox[{179, 180}]}, 
                    {Hue[0.8467428738832437, 1, 1], LineBox[{180, 181}]}, 
                    {Hue[0.924790561415135, 1, 1], LineBox[{181, 182}]}, 
                    {Hue[0.003762021946077935, 1, 1], LineBox[{182, 183}]}, 
                    {Hue[0.08364972930613161, 1, 1], LineBox[{183, 184}]}, 
                    {Hue[0.16444234996376278`, 1, 1], LineBox[{184, 185}]}, 
                    {Hue[0.24613038943135804`, 1, 1], LineBox[{185, 186}]}, 
                    {Hue[0.32870200662097754`, 1, 1], LineBox[{186, 187}]}, 
                    {Hue[0.4121436885489237, 1, 1], LineBox[{187, 188}]}, 
                    {Hue[0.4964553658376828, 1, 1], LineBox[{188, 189}]}, 
                    {Hue[0.5816237800416533, 1, 1], LineBox[{189, 190}]}, 
                    {Hue[0.6676374651199203, 1, 1], LineBox[{190, 191}]}, 
                    {Hue[0.7544888084455762, 1, 1], LineBox[{191, 192}]}, 
                    {Hue[0.8421678489328173, 1, 1], LineBox[{192, 193}]}, 
                    {Hue[0.9306872201453219, 1, 1], LineBox[{193, 194}]}, 
                    {Hue[0.020028044921177234`, 1, 1], LineBox[{194, 195}]}, 
                    {Hue[0.11015384531023154`, 1, 1], LineBox[{195, 196}]}, 
                    {Hue[0.2010779247037523, 1, 1], LineBox[{196, 197}]}, 
                    {Hue[0.2928164041304295, 1, 1], LineBox[{197, 198}]}, 
                    {Hue[0.38533319026587, 1, 1], LineBox[{198, 199}]}, 
                    {Hue[0.47860709073898694`, 1, 1], LineBox[{199, 200}]}, 
                    {Hue[0.5726744637481624, 1, 1], LineBox[{200, 201}]}, 
                    {Hue[0.6675282615363339, 1, 1], LineBox[{201, 202}]}, 
                    {Hue[0.7631119700483477, 1, 1], LineBox[{202, 203}]}, 
                    {Hue[0.8594478928195965, 1, 1], LineBox[{203, 204}]}, 
                    {Hue[0.9565833397615986, 1, 1], LineBox[{204, 205}]}, 
                    {Hue[0.05442891042184869, 1, 1], LineBox[{205, 206}]}, 
                    {Hue[0.15295126944831824`, 1, 1], LineBox[{206, 207}]}, 
                    {Hue[0.2523057656500986, 1, 1], LineBox[{207, 208}]}, 
                    {Hue[0.35240163726979257`, 1, 1], LineBox[{208, 209}]}, 
                    {Hue[0.4530685174367239, 1, 1], LineBox[{209, 210}]}, 
                    {Hue[0.5546446182690019, 1, 1], LineBox[{210, 211}]}, 
                    {Hue[0.6571564465191764, 1, 1], LineBox[{211, 212}]}, 
                    {Hue[0.7599530157392963, 1, 1], LineBox[{212, 213}]}, 
                    {Hue[0.86323881815049, 1, 1], LineBox[{213, 214}]}, 
                    {Hue[0.9678183223563761, 1, 1], LineBox[{214, 215}]}, 
                    {Hue[0.07322977077513246, 1, 1], LineBox[{215, 216}]}, 
                    {Hue[0.17878726522152358`, 1, 1], LineBox[{216, 217}]}, 
                    {Hue[0.2853080718697084, 1, 1], LineBox[{217, 218}]}, 
                    {Hue[0.39338501586632146`, 1, 1], LineBox[{218, 219}]}, 
                    {Hue[0.5017840227505358, 1, 1], LineBox[{219, 220}]}, 
                    {Hue[0.6096976623540389, 1, 1], LineBox[{220, 221}]}, 
                    {Hue[0.7186763960117638, 1, 1], LineBox[{221, 222}]}, 
                    {Hue[0.8296894368998564, 1, 1], LineBox[{222, 223}]}, 
                    {Hue[0.9407097079137103, 1, 1], LineBox[{223, 224}]}, 
                    {Hue[0.05042165514197633, 1, 1], LineBox[{224, 225}]}, 
                    {Hue[0.1611490802839811, 1, 1], LineBox[{225, 226}]}, 
                    {Hue[0.2739566445765149, 1, 1], LineBox[{226, 227}]}, 
                    {Hue[0.38568383428444303`, 1, 1], LineBox[{227, 228}]}, 
                    {Hue[0.4956018360147952, 1, 1], LineBox[{228, 229}]}, 
                    {Hue[0.6101557229342287, 1, 1], LineBox[{229, 230}]}, 
                    {Hue[0.7326512288615409, 1, 1], LineBox[{230, 231}]}, 
                    {Hue[0.8528047721115746, 1, 1], LineBox[{231, 232}]}, 
                    {Hue[0.9542542608112418, 1, 1], LineBox[{232, 233}]}, 
                    {Hue[0.0558345373254985, 1, 1], LineBox[{233, 234}]}, 
                    {Hue[0.1919635521635686, 1, 1], LineBox[{234, 235}]}, 
                    {Hue[0.34847173818108124`, 1, 1], LineBox[{235, 236}]}, 
                    {Hue[0.48833365764146647`, 1, 1], LineBox[{236, 237}]}, 
                    {Hue[0.5804563486032157, 1, 1], LineBox[{237, 238}]}, 
                    {Hue[0.7220462748464456, 1, 1], LineBox[{238, 239}]}, 
                    {Hue[0.939032460554266, 1, 1], LineBox[{239, 240}]}, 
                    {Hue[0.20366116429204512`, 1, 1], LineBox[{240, 241}]}, 
                    {Hue[0.4854402698286375, 1, 1], LineBox[{241, 242}]}, 
                    {Hue[0.7471307691604958, 1, 1], LineBox[{242, 243}]}, 
                    {Hue[0.982320906152381, 1, 1], LineBox[{243, 244}]}, 
                    {Hue[0.18815268440253508`, 1, 1], LineBox[{244, 245}]}, 
                    {Hue[0.2570783535583873, 1, 1], LineBox[{245, 246}]}, 
                    {Hue[0.28697439108716605`, 1, 1], LineBox[{246, 247}]}, 
                    {Hue[0.4877808018408883, 1, 1], LineBox[{247, 248}]}, 
                    {Hue[0.7145537078305133, 1, 1], LineBox[{248, 249}]}, 
                    {Hue[0.9491777610707849, 1, 1], LineBox[{249, 250}]}, 
                    {Hue[0.18421774847407577`, 1, 1], LineBox[{250, 251}]}}}},
                    
                    VertexColors->{
                    Hue[0.46903121084379074`, 1, 1], 
                    Hue[0.3276739181280018, 1, 1], 
                    Hue[0.27302120989995315`, 1, 1], 
                    Hue[0.1503982909211788, 1, 1], 
                    Hue[0.1159540677729317, 1, 1], 
                    Hue[0.9885034259412886, 1, 1], 
                    Hue[0.555919303286334, 1, 1], 
                    Hue[0.014674981446927316`, 1, 1], 
                    Hue[0.2505963104170238, 1, 1], 
                    Hue[0.08114438222060529, 1, 1], 
                    Hue[0.01429524804121616, 1, 1], 
                    Hue[0.887197895954603, 1, 1], 
                    Hue[0.776711378633425, 1, 1], 
                    Hue[0.8220325704507755, 1, 1], 
                    Hue[0.6796760798270274, 1, 1], 
                    Hue[0.7081680584902381, 1, 1], 
                    Hue[0.5453095997956825, 1, 1], 
                    Hue[0.4431782547061368, 1, 1], 
                    Hue[0.24332838511522192`, 1, 1], 
                    Hue[0.11867547147282674`, 1, 1], 
                    Hue[0.20671099307287558`, 1, 1], 
                    Hue[0.874019357415786, 1, 1], 
                    Hue[0.6591515413554456, 1, 1], 
                    Hue[0.18066228033355544`, 1, 1], 
                    Hue[0.6678350196899061, 1, 1], 
                    Hue[0.5241258581961699, 1, 1], 
                    Hue[0.1431727845363231, 1, 1], 
                    Hue[0.7756794556885923, 1, 1], 
                    Hue[0.5095140846201118, 1, 1], 
                    Hue[0.6861711805072839, 1, 1], 
                    Hue[0.8636511930406199, 1, 1], 
                    Hue[0.5811421883741066, 1, 1], 
                    Hue[0.5145867440569433, 1, 1], 
                    Hue[0.3775090338495559, 1, 1], 
                    Hue[0.24014184273348146`, 1, 1], 
                    Hue[0.0808948788107402, 1, 1], 
                    Hue[0.00828126453665834, 1, 1], 
                    Hue[0.8486886497933208, 1, 1], 
                    Hue[0.7430682886052937, 1, 1], 
                    Hue[0.661458391422076, 1, 1], 
                    Hue[0.502941759363241, 1, 1], 
                    Hue[0.3928074615720261, 1, 1], 
                    Hue[0.17436336899653096`, 1, 1], 
                    Hue[0.19971063365582253`, 1, 1], 
                    Hue[0.9072160164407977, 1, 1], 
                    Hue[0.7435128384384582, 1, 1], 
                    Hue[0.7542167129026032, 1, 1], 
                    Hue[0.7220480636268023, 1, 1], 
                    Hue[0.5960170125858861, 1, 1], 
                    Hue[0.504394885168866, 1, 1], 
                    Hue[0.5980761131160199, 1, 1], 
                    Hue[0.39175536296490443`, 1, 1], 
                    Hue[0.3080870960109756, 1, 1], 
                    Hue[0.22637965615689903`, 1, 1], 
                    Hue[0.04078695456709334, 1, 1], 
                    Hue[0.9979339064190902, 1, 1], 
                    Hue[0.9379513734489464, 1, 1], 
                    Hue[0.7568969236098246, 1, 1], 
                    Hue[0.6044031428332308, 1, 1], 
                    Hue[0.546835631606627, 1, 1], 
                    Hue[0.48445950589899756`, 1, 1], 
                    Hue[0.45836739143999605`, 1, 1], 
                    Hue[0.4050876322941351, 1, 1], 
                    Hue[0.3310440859556128, 1, 1], 
                    Hue[0.8952884846740966, 1, 1], 
                    Hue[0.849914391383057, 1, 1], 
                    Hue[0.8210799392004464, 1, 1], 
                    Hue[0.797476270966503, 1, 1], 
                    Hue[0.7677111204344772, 1, 1], 
                    Hue[0.7373996974703723, 1, 1], 
                    Hue[0.6751855643100823, 1, 1], 
                    Hue[0.24133341544108203`, 1, 1], 
                    Hue[0.20414494146012305`, 1, 1], 
                    Hue[0.17127988511038536`, 1, 1], 
                    Hue[0.15097260410212796`, 1, 1], 
                    Hue[0.1323205065914355, 1, 1], 
                    Hue[0.11577470259370665`, 1, 1], 
                    Hue[0.40072236254851873`, 1, 1], 
                    Hue[0.5642972190009667, 1, 1], 
                    Hue[0.5486440775627401, 1, 1], 
                    Hue[0.5323579238759228, 1, 1], 
                    Hue[0.5186050976851513, 1, 1], 
                    Hue[0.5076758901256943, 1, 1], 
                    Hue[0.5231123375576997, 1, 1], 
                    Hue[0.9249542200277258, 1, 1], 
                    Hue[0.9326440595614945, 1, 1], 
                    Hue[0.92414635791773, 1, 1], 
                    Hue[0.914328727297987, 1, 1], 
                    Hue[0.9075469174696386, 1, 1], 
                    Hue[0.9162986729778786, 1, 1], 
                    Hue[0.22438990344755358`, 1, 1], 
                    Hue[0.3233701921895654, 1, 1], 
                    Hue[0.3255239702566573, 1, 1], 
                    Hue[0.32135596500062585`, 1, 1], 
                    Hue[0.3185714943521732, 1, 1], 
                    Hue[0.3260807234884014, 1, 1], 
                    Hue[0.44731094999721865`, 1, 1], 
                    Hue[0.718402716585326, 1, 1], 
                    Hue[0.7378073209369546, 1, 1], 
                    Hue[0.7400657932223309, 1, 1], 
                    Hue[0.7412978514163524, 1, 1], 
                    Hue[0.7498344855551069, 1, 1], 
                    Hue[0.8080132497531595, 1, 1], 
                    Hue[0.10678166006413865`, 1, 1], 
                    Hue[0.15908580322017676`, 1, 1], 
                    Hue[0.16950671056358974`, 1, 1], 
                    Hue[0.1750900190849448, 1, 1], 
                    Hue[0.18498588973514118`, 1, 1], 
                    Hue[0.2205776860799222, 1, 1], 
                    Hue[0.4486392775157134, 1, 1], 
                    Hue[0.5823447125514417, 1, 1], 
                    Hue[0.6062875308462049, 1, 1], 
                    Hue[0.6170867116073702, 1, 1], 
                    Hue[0.6280375327039411, 1, 1], 
                    Hue[0.6498635797223722, 1, 1], 
                    Hue[0.7294631065042633, 1, 1], 
                    Hue[0.9619442698698356, 1, 1], 
                    Hue[0.0358694738259074, 1, 1], 
                    Hue[0.058560492456003414`, 1, 1], 
                    Hue[0.07200337794989134, 1, 1], 
                    Hue[0.08546181681129635, 1, 1], 
                    Hue[0.10595949252681901`, 1, 1], 
                    Hue[0.15064552410101642`, 1, 1], 
                    Hue[0.2776938160586117, 1, 1], 
                    Hue[0.42749515722735437`, 1, 1], 
                    Hue[0.48338919600598795`, 1, 1], 
                    Hue[0.5077457331726007, 1, 1], 
                    Hue[0.522453270537981, 1, 1], 
                    Hue[0.533643024834802, 1, 1], 
                    Hue[0.5436190540030256, 1, 1], 
                    Hue[0.5534698627463082, 1, 1], 
                    Hue[0.5638043360423671, 1, 1], 
                    Hue[0.5750106310394598, 1, 1], 
                    Hue[0.5873592564502069, 1, 1], 
                    Hue[0.6010505587702185, 1, 1], 
                    Hue[0.6162377444445248, 1, 1], 
                    Hue[0.633039328584811, 1, 1], 
                    Hue[0.6515452342370982, 1, 1], 
                    Hue[0.671822399406282, 1, 1], 
                    Hue[0.6939174624732731, 1, 1], 
                    Hue[0.7178601571527807, 1, 1], 
                    Hue[0.743664904009497, 1, 1], 
                    Hue[0.7713339799014306, 1, 1], 
                    Hue[0.8008597716600634, 1, 1], 
                    Hue[0.8322257562878227, 1, 1], 
                    Hue[0.8654094415339608, 1, 1], 
                    Hue[0.900382675750429, 1, 1], 
                    Hue[0.9371137956725565, 1, 1], 
                    Hue[0.9755688441042102, 1, 1], 
                    Hue[0.015710822193555896`, 1, 1], 
                    Hue[0.05750341769511634, 1, 1], 
                    Hue[0.1009088276816796, 1, 1], 
                    Hue[0.14588984153644521`, 1, 1], 
                    Hue[0.19240917392454862`, 1, 1], 
                    Hue[0.2404306955446981, 1, 1], 
                    Hue[0.28991896844010645`, 1, 1], 
                    Hue[0.34084052389359315`, 1, 1], 
                    Hue[0.3931618756369694, 1, 1], 
                    Hue[0.44685127226500027`, 1, 1], 
                    Hue[0.5018784756128514, 1, 1], 
                    Hue[0.5582146923969578, 1, 1], 
                    Hue[0.6158309711166918, 1, 1], 
                    Hue[0.6747017479612505, 1, 1], 
                    Hue[0.7348012513906026, 1, 1], 
                    Hue[0.7961058202286575, 1, 1], 
                    Hue[0.8585911448709992, 1, 1], 
                    Hue[0.9222357148841215, 1, 1], 
                    Hue[0.9870184060201163, 1, 1], 
                    Hue[0.05292099295597615, 1, 1], 
                    Hue[0.11992120925220558`, 1, 1], 
                    Hue[0.1880026820180965, 1, 1], 
                    Hue[0.2571478502170692, 1, 1], 
                    Hue[0.3273391787139689, 1, 1], 
                    Hue[0.39856225931539346`, 1, 1], 
                    Hue[0.47080188987063876`, 1, 1], 
                    Hue[0.5440415604188835, 1, 1], 
                    Hue[0.6182681067487781, 1, 1], 
                    Hue[0.6934684017449322, 1, 1], 
                    Hue[0.7696304170032996, 1, 1], 
                    Hue[0.8467428738832437, 1, 1], 
                    Hue[0.924790561415135, 1, 1], 
                    Hue[0.003762021946077935, 1, 1], 
                    Hue[0.08364972930613161, 1, 1], 
                    Hue[0.16444234996376278`, 1, 1], 
                    Hue[0.24613038943135804`, 1, 1], 
                    Hue[0.32870200662097754`, 1, 1], 
                    Hue[0.4121436885489237, 1, 1], 
                    Hue[0.4964553658376828, 1, 1], 
                    Hue[0.5816237800416533, 1, 1], 
                    Hue[0.6676374651199203, 1, 1], 
                    Hue[0.7544888084455762, 1, 1], 
                    Hue[0.8421678489328173, 1, 1], 
                    Hue[0.9306872201453219, 1, 1], 
                    Hue[0.020028044921177234`, 1, 1], 
                    Hue[0.11015384531023154`, 1, 1], 
                    Hue[0.2010779247037523, 1, 1], 
                    Hue[0.2928164041304295, 1, 1], 
                    Hue[0.38533319026587, 1, 1], 
                    Hue[0.47860709073898694`, 1, 1], 
                    Hue[0.5726744637481624, 1, 1], 
                    Hue[0.6675282615363339, 1, 1], 
                    Hue[0.7631119700483477, 1, 1], 
                    Hue[0.8594478928195965, 1, 1], 
                    Hue[0.9565833397615986, 1, 1], 
                    Hue[0.05442891042184869, 1, 1], 
                    Hue[0.15295126944831824`, 1, 1], 
                    Hue[0.2523057656500986, 1, 1], 
                    Hue[0.35240163726979257`, 1, 1], 
                    Hue[0.4530685174367239, 1, 1], 
                    Hue[0.5546446182690019, 1, 1], 
                    Hue[0.6571564465191764, 1, 1], 
                    Hue[0.7599530157392963, 1, 1], 
                    Hue[0.86323881815049, 1, 1], 
                    Hue[0.9678183223563761, 1, 1], 
                    Hue[0.07322977077513246, 1, 1], 
                    Hue[0.17878726522152358`, 1, 1], 
                    Hue[0.2853080718697084, 1, 1], 
                    Hue[0.39338501586632146`, 1, 1], 
                    Hue[0.5017840227505358, 1, 1], 
                    Hue[0.6096976623540389, 1, 1], 
                    Hue[0.7186763960117638, 1, 1], 
                    Hue[0.8296894368998564, 1, 1], 
                    Hue[0.9407097079137103, 1, 1], 
                    Hue[0.05042165514197633, 1, 1], 
                    Hue[0.1611490802839811, 1, 1], 
                    Hue[0.2739566445765149, 1, 1], 
                    Hue[0.38568383428444303`, 1, 1], 
                    Hue[0.4956018360147952, 1, 1], 
                    Hue[0.6101557229342287, 1, 1], 
                    Hue[0.7326512288615409, 1, 1], 
                    Hue[0.8528047721115746, 1, 1], 
                    Hue[0.9542542608112418, 1, 1], 
                    Hue[0.0558345373254985, 1, 1], 
                    Hue[0.1919635521635686, 1, 1], 
                    Hue[0.34847173818108124`, 1, 1], 
                    Hue[0.48833365764146647`, 1, 1], 
                    Hue[0.5804563486032157, 1, 1], 
                    Hue[0.7220462748464456, 1, 1], 
                    Hue[0.939032460554266, 1, 1], 
                    Hue[0.20366116429204512`, 1, 1], 
                    Hue[0.4854402698286375, 1, 1], 
                    Hue[0.7471307691604958, 1, 1], 
                    Hue[0.982320906152381, 1, 1], 
                    Hue[0.18815268440253508`, 1, 1], 
                    Hue[0.2570783535583873, 1, 1], 
                    Hue[0.28697439108716605`, 1, 1], 
                    Hue[0.4877808018408883, 1, 1], 
                    Hue[0.7145537078305133, 1, 1], 
                    Hue[0.9491777610707849, 1, 1], 
                    Hue[0.18421774847407577`, 1, 1], 
                    Hue[0.40213423036871093`, 1, 1], 
                    Hue[0.46903121084379074`, 1, 1], 
                    Hue[0.3276739181280018, 1, 1], 
                    Hue[0.3276739181280018, 1, 1], 
                    Hue[0.27302120989995315`, 1, 1], 
                    Hue[0.27302120989995315`, 1, 1], 
                    Hue[0.1503982909211788, 1, 1], 
                    Hue[0.1503982909211788, 1, 1], 
                    Hue[0.1159540677729317, 1, 1], 
                    Hue[0.1159540677729317, 1, 1], 
                    Hue[0.9885034259412886, 1, 1], 
                    Hue[0.9885034259412886, 1, 1], 
                    Hue[0.555919303286334, 1, 1], 
                    Hue[0.555919303286334, 1, 1], 
                    Hue[0.014674981446927316`, 1, 1], 
                    Hue[0.014674981446927316`, 1, 1], 
                    Hue[0.2505963104170238, 1, 1], 
                    Hue[0.2505963104170238, 1, 1], 
                    Hue[0.08114438222060529, 1, 1], 
                    Hue[0.08114438222060529, 1, 1], 
                    Hue[0.01429524804121616, 1, 1], 
                    Hue[0.01429524804121616, 1, 1], 
                    Hue[0.887197895954603, 1, 1], 
                    Hue[0.887197895954603, 1, 1], 
                    Hue[0.776711378633425, 1, 1], 
                    Hue[0.776711378633425, 1, 1], 
                    Hue[0.8220325704507755, 1, 1], 
                    Hue[0.8220325704507755, 1, 1], 
                    Hue[0.6796760798270274, 1, 1], 
                    Hue[0.6796760798270274, 1, 1], 
                    Hue[0.7081680584902381, 1, 1], 
                    Hue[0.7081680584902381, 1, 1], 
                    Hue[0.5453095997956825, 1, 1], 
                    Hue[0.5453095997956825, 1, 1], 
                    Hue[0.4431782547061368, 1, 1], 
                    Hue[0.4431782547061368, 1, 1], 
                    Hue[0.24332838511522192`, 1, 1], 
                    Hue[0.24332838511522192`, 1, 1], 
                    Hue[0.11867547147282674`, 1, 1], 
                    Hue[0.11867547147282674`, 1, 1], 
                    Hue[0.20671099307287558`, 1, 1], 
                    Hue[0.20671099307287558`, 1, 1], 
                    Hue[0.874019357415786, 1, 1], 
                    Hue[0.874019357415786, 1, 1], 
                    Hue[0.6591515413554456, 1, 1], 
                    Hue[0.6591515413554456, 1, 1], 
                    Hue[0.18066228033355544`, 1, 1], 
                    Hue[0.18066228033355544`, 1, 1], 
                    Hue[0.6678350196899061, 1, 1], 
                    Hue[0.6678350196899061, 1, 1], 
                    Hue[0.5241258581961699, 1, 1], 
                    Hue[0.5241258581961699, 1, 1], 
                    Hue[0.1431727845363231, 1, 1], 
                    Hue[0.1431727845363231, 1, 1], 
                    Hue[0.7756794556885923, 1, 1], 
                    Hue[0.7756794556885923, 1, 1], 
                    Hue[0.5095140846201118, 1, 1], 
                    Hue[0.5095140846201118, 1, 1], 
                    Hue[0.6861711805072839, 1, 1], 
                    Hue[0.6861711805072839, 1, 1], 
                    Hue[0.8636511930406199, 1, 1], 
                    Hue[0.8636511930406199, 1, 1], 
                    Hue[0.5811421883741066, 1, 1], 
                    Hue[0.5811421883741066, 1, 1], 
                    Hue[0.5145867440569433, 1, 1], 
                    Hue[0.5145867440569433, 1, 1], 
                    Hue[0.3775090338495559, 1, 1], 
                    Hue[0.3775090338495559, 1, 1], 
                    Hue[0.24014184273348146`, 1, 1], 
                    Hue[0.24014184273348146`, 1, 1], 
                    Hue[0.0808948788107402, 1, 1], 
                    Hue[0.0808948788107402, 1, 1], 
                    Hue[0.00828126453665834, 1, 1], 
                    Hue[0.00828126453665834, 1, 1], 
                    Hue[0.8486886497933208, 1, 1], 
                    Hue[0.8486886497933208, 1, 1], 
                    Hue[0.7430682886052937, 1, 1], 
                    Hue[0.7430682886052937, 1, 1], 
                    Hue[0.661458391422076, 1, 1], 
                    Hue[0.661458391422076, 1, 1], 
                    Hue[0.502941759363241, 1, 1], 
                    Hue[0.502941759363241, 1, 1], 
                    Hue[0.3928074615720261, 1, 1], 
                    Hue[0.3928074615720261, 1, 1], 
                    Hue[0.17436336899653096`, 1, 1], 
                    Hue[0.17436336899653096`, 1, 1], 
                    Hue[0.19971063365582253`, 1, 1], 
                    Hue[0.19971063365582253`, 1, 1], 
                    Hue[0.9072160164407977, 1, 1], 
                    Hue[0.9072160164407977, 1, 1], 
                    Hue[0.7435128384384582, 1, 1], 
                    Hue[0.7435128384384582, 1, 1], 
                    Hue[0.7542167129026032, 1, 1], 
                    Hue[0.7542167129026032, 1, 1], 
                    Hue[0.7220480636268023, 1, 1], 
                    Hue[0.7220480636268023, 1, 1], 
                    Hue[0.5960170125858861, 1, 1], 
                    Hue[0.5960170125858861, 1, 1], 
                    Hue[0.504394885168866, 1, 1], 
                    Hue[0.504394885168866, 1, 1], 
                    Hue[0.5980761131160199, 1, 1], 
                    Hue[0.5980761131160199, 1, 1], 
                    Hue[0.39175536296490443`, 1, 1], 
                    Hue[0.39175536296490443`, 1, 1], 
                    Hue[0.3080870960109756, 1, 1], 
                    Hue[0.3080870960109756, 1, 1], 
                    Hue[0.22637965615689903`, 1, 1], 
                    Hue[0.22637965615689903`, 1, 1], 
                    Hue[0.04078695456709334, 1, 1], 
                    Hue[0.04078695456709334, 1, 1], 
                    Hue[0.9979339064190902, 1, 1], 
                    Hue[0.9979339064190902, 1, 1], 
                    Hue[0.9379513734489464, 1, 1], 
                    Hue[0.9379513734489464, 1, 1], 
                    Hue[0.7568969236098246, 1, 1], 
                    Hue[0.7568969236098246, 1, 1], 
                    Hue[0.6044031428332308, 1, 1], 
                    Hue[0.6044031428332308, 1, 1], 
                    Hue[0.546835631606627, 1, 1], 
                    Hue[0.546835631606627, 1, 1], 
                    Hue[0.48445950589899756`, 1, 1], 
                    Hue[0.48445950589899756`, 1, 1], 
                    Hue[0.45836739143999605`, 1, 1], 
                    Hue[0.45836739143999605`, 1, 1], 
                    Hue[0.4050876322941351, 1, 1], 
                    Hue[0.4050876322941351, 1, 1], 
                    Hue[0.3310440859556128, 1, 1], 
                    Hue[0.3310440859556128, 1, 1], 
                    Hue[0.8952884846740966, 1, 1], 
                    Hue[0.8952884846740966, 1, 1], 
                    Hue[0.849914391383057, 1, 1], 
                    Hue[0.849914391383057, 1, 1], 
                    Hue[0.8210799392004464, 1, 1], 
                    Hue[0.8210799392004464, 1, 1], 
                    Hue[0.797476270966503, 1, 1], 
                    Hue[0.797476270966503, 1, 1], 
                    Hue[0.7677111204344772, 1, 1], 
                    Hue[0.7677111204344772, 1, 1], 
                    Hue[0.7373996974703723, 1, 1], 
                    Hue[0.7373996974703723, 1, 1], 
                    Hue[0.6751855643100823, 1, 1], 
                    Hue[0.6751855643100823, 1, 1], 
                    Hue[0.24133341544108203`, 1, 1], 
                    Hue[0.24133341544108203`, 1, 1], 
                    Hue[0.20414494146012305`, 1, 1], 
                    Hue[0.20414494146012305`, 1, 1], 
                    Hue[0.17127988511038536`, 1, 1], 
                    Hue[0.17127988511038536`, 1, 1], 
                    Hue[0.15097260410212796`, 1, 1], 
                    Hue[0.15097260410212796`, 1, 1], 
                    Hue[0.1323205065914355, 1, 1], 
                    Hue[0.1323205065914355, 1, 1], 
                    Hue[0.11577470259370665`, 1, 1], 
                    Hue[0.11577470259370665`, 1, 1], 
                    Hue[0.40072236254851873`, 1, 1], 
                    Hue[0.40072236254851873`, 1, 1], 
                    Hue[0.5642972190009667, 1, 1], 
                    Hue[0.5642972190009667, 1, 1], 
                    Hue[0.5486440775627401, 1, 1], 
                    Hue[0.5486440775627401, 1, 1], 
                    Hue[0.5323579238759228, 1, 1], 
                    Hue[0.5323579238759228, 1, 1], 
                    Hue[0.5186050976851513, 1, 1], 
                    Hue[0.5186050976851513, 1, 1], 
                    Hue[0.5076758901256943, 1, 1], 
                    Hue[0.5076758901256943, 1, 1], 
                    Hue[0.5231123375576997, 1, 1], 
                    Hue[0.5231123375576997, 1, 1], 
                    Hue[0.9249542200277258, 1, 1], 
                    Hue[0.9249542200277258, 1, 1], 
                    Hue[0.9326440595614945, 1, 1], 
                    Hue[0.9326440595614945, 1, 1], 
                    Hue[0.92414635791773, 1, 1], 
                    Hue[0.92414635791773, 1, 1], 
                    Hue[0.914328727297987, 1, 1], 
                    Hue[0.914328727297987, 1, 1], 
                    Hue[0.9075469174696386, 1, 1], 
                    Hue[0.9075469174696386, 1, 1], 
                    Hue[0.9162986729778786, 1, 1], 
                    Hue[0.9162986729778786, 1, 1], 
                    Hue[0.22438990344755358`, 1, 1], 
                    Hue[0.22438990344755358`, 1, 1], 
                    Hue[0.3233701921895654, 1, 1], 
                    Hue[0.3233701921895654, 1, 1], 
                    Hue[0.3255239702566573, 1, 1], 
                    Hue[0.3255239702566573, 1, 1], 
                    Hue[0.32135596500062585`, 1, 1], 
                    Hue[0.32135596500062585`, 1, 1], 
                    Hue[0.3185714943521732, 1, 1], 
                    Hue[0.3185714943521732, 1, 1], 
                    Hue[0.3260807234884014, 1, 1], 
                    Hue[0.3260807234884014, 1, 1], 
                    Hue[0.44731094999721865`, 1, 1], 
                    Hue[0.44731094999721865`, 1, 1], 
                    Hue[0.718402716585326, 1, 1], 
                    Hue[0.718402716585326, 1, 1], 
                    Hue[0.7378073209369546, 1, 1], 
                    Hue[0.7378073209369546, 1, 1], 
                    Hue[0.7400657932223309, 1, 1], 
                    Hue[0.7400657932223309, 1, 1], 
                    Hue[0.7412978514163524, 1, 1], 
                    Hue[0.7412978514163524, 1, 1], 
                    Hue[0.7498344855551069, 1, 1], 
                    Hue[0.7498344855551069, 1, 1], 
                    Hue[0.8080132497531595, 1, 1], 
                    Hue[0.8080132497531595, 1, 1], 
                    Hue[0.10678166006413865`, 1, 1], 
                    Hue[0.10678166006413865`, 1, 1], 
                    Hue[0.15908580322017676`, 1, 1], 
                    Hue[0.15908580322017676`, 1, 1], 
                    Hue[0.16950671056358974`, 1, 1], 
                    Hue[0.16950671056358974`, 1, 1], 
                    Hue[0.1750900190849448, 1, 1], 
                    Hue[0.1750900190849448, 1, 1], 
                    Hue[0.18498588973514118`, 1, 1], 
                    Hue[0.18498588973514118`, 1, 1], 
                    Hue[0.2205776860799222, 1, 1], 
                    Hue[0.2205776860799222, 1, 1], 
                    Hue[0.4486392775157134, 1, 1], 
                    Hue[0.4486392775157134, 1, 1], 
                    Hue[0.5823447125514417, 1, 1], 
                    Hue[0.5823447125514417, 1, 1], 
                    Hue[0.6062875308462049, 1, 1], 
                    Hue[0.6062875308462049, 1, 1], 
                    Hue[0.6170867116073702, 1, 1], 
                    Hue[0.6170867116073702, 1, 1], 
                    Hue[0.6280375327039411, 1, 1], 
                    Hue[0.6280375327039411, 1, 1], 
                    Hue[0.6498635797223722, 1, 1], 
                    Hue[0.6498635797223722, 1, 1], 
                    Hue[0.7294631065042633, 1, 1], 
                    Hue[0.7294631065042633, 1, 1], 
                    Hue[0.9619442698698356, 1, 1], 
                    Hue[0.9619442698698356, 1, 1], 
                    Hue[0.0358694738259074, 1, 1], 
                    Hue[0.0358694738259074, 1, 1], 
                    Hue[0.058560492456003414`, 1, 1], 
                    Hue[0.058560492456003414`, 1, 1], 
                    Hue[0.07200337794989134, 1, 1], 
                    Hue[0.07200337794989134, 1, 1], 
                    Hue[0.08546181681129635, 1, 1], 
                    Hue[0.08546181681129635, 1, 1], 
                    Hue[0.10595949252681901`, 1, 1], 
                    Hue[0.10595949252681901`, 1, 1], 
                    Hue[0.15064552410101642`, 1, 1], 
                    Hue[0.15064552410101642`, 1, 1], 
                    Hue[0.2776938160586117, 1, 1], 
                    Hue[0.2776938160586117, 1, 1], 
                    Hue[0.42749515722735437`, 1, 1], 
                    Hue[0.42749515722735437`, 1, 1], 
                    Hue[0.48338919600598795`, 1, 1], 
                    Hue[0.48338919600598795`, 1, 1], 
                    Hue[0.5077457331726007, 1, 1], 
                    Hue[0.5077457331726007, 1, 1], 
                    Hue[0.522453270537981, 1, 1], 
                    Hue[0.522453270537981, 1, 1], 
                    Hue[0.533643024834802, 1, 1], 
                    Hue[0.533643024834802, 1, 1], 
                    Hue[0.5436190540030256, 1, 1], 
                    Hue[0.5436190540030256, 1, 1], 
                    Hue[0.5534698627463082, 1, 1], 
                    Hue[0.5534698627463082, 1, 1], 
                    Hue[0.5638043360423671, 1, 1], 
                    Hue[0.5638043360423671, 1, 1], 
                    Hue[0.5750106310394598, 1, 1], 
                    Hue[0.5750106310394598, 1, 1], 
                    Hue[0.5873592564502069, 1, 1], 
                    Hue[0.5873592564502069, 1, 1], 
                    Hue[0.6010505587702185, 1, 1], 
                    Hue[0.6010505587702185, 1, 1], 
                    Hue[0.6162377444445248, 1, 1], 
                    Hue[0.6162377444445248, 1, 1], 
                    Hue[0.633039328584811, 1, 1], 
                    Hue[0.633039328584811, 1, 1], 
                    Hue[0.6515452342370982, 1, 1], 
                    Hue[0.6515452342370982, 1, 1], 
                    Hue[0.671822399406282, 1, 1], 
                    Hue[0.671822399406282, 1, 1], 
                    Hue[0.6939174624732731, 1, 1], 
                    Hue[0.6939174624732731, 1, 1], 
                    Hue[0.7178601571527807, 1, 1], 
                    Hue[0.7178601571527807, 1, 1], 
                    Hue[0.743664904009497, 1, 1], 
                    Hue[0.743664904009497, 1, 1], 
                    Hue[0.7713339799014306, 1, 1], 
                    Hue[0.7713339799014306, 1, 1], 
                    Hue[0.8008597716600634, 1, 1], 
                    Hue[0.8008597716600634, 1, 1], 
                    Hue[0.8322257562878227, 1, 1], 
                    Hue[0.8322257562878227, 1, 1], 
                    Hue[0.8654094415339608, 1, 1], 
                    Hue[0.8654094415339608, 1, 1], 
                    Hue[0.900382675750429, 1, 1], 
                    Hue[0.900382675750429, 1, 1], 
                    Hue[0.9371137956725565, 1, 1], 
                    Hue[0.9371137956725565, 1, 1], 
                    Hue[0.9755688441042102, 1, 1], 
                    Hue[0.9755688441042102, 1, 1], 
                    Hue[0.015710822193555896`, 1, 1], 
                    Hue[0.015710822193555896`, 1, 1], 
                    Hue[0.05750341769511634, 1, 1], 
                    Hue[0.05750341769511634, 1, 1], 
                    Hue[0.1009088276816796, 1, 1], 
                    Hue[0.1009088276816796, 1, 1], 
                    Hue[0.14588984153644521`, 1, 1], 
                    Hue[0.14588984153644521`, 1, 1], 
                    Hue[0.19240917392454862`, 1, 1], 
                    Hue[0.19240917392454862`, 1, 1], 
                    Hue[0.2404306955446981, 1, 1], 
                    Hue[0.2404306955446981, 1, 1], 
                    Hue[0.28991896844010645`, 1, 1], 
                    Hue[0.28991896844010645`, 1, 1], 
                    Hue[0.34084052389359315`, 1, 1], 
                    Hue[0.34084052389359315`, 1, 1], 
                    Hue[0.3931618756369694, 1, 1], 
                    Hue[0.3931618756369694, 1, 1], 
                    Hue[0.44685127226500027`, 1, 1], 
                    Hue[0.44685127226500027`, 1, 1], 
                    Hue[0.5018784756128514, 1, 1], 
                    Hue[0.5018784756128514, 1, 1], 
                    Hue[0.5582146923969578, 1, 1], 
                    Hue[0.5582146923969578, 1, 1], 
                    Hue[0.6158309711166918, 1, 1], 
                    Hue[0.6158309711166918, 1, 1], 
                    Hue[0.6747017479612505, 1, 1], 
                    Hue[0.6747017479612505, 1, 1], 
                    Hue[0.7348012513906026, 1, 1], 
                    Hue[0.7348012513906026, 1, 1], 
                    Hue[0.7961058202286575, 1, 1], 
                    Hue[0.7961058202286575, 1, 1], 
                    Hue[0.8585911448709992, 1, 1], 
                    Hue[0.8585911448709992, 1, 1], 
                    Hue[0.9222357148841215, 1, 1], 
                    Hue[0.9222357148841215, 1, 1], 
                    Hue[0.9870184060201163, 1, 1], 
                    Hue[0.9870184060201163, 1, 1], 
                    Hue[0.05292099295597615, 1, 1], 
                    Hue[0.05292099295597615, 1, 1], 
                    Hue[0.11992120925220558`, 1, 1], 
                    Hue[0.11992120925220558`, 1, 1], 
                    Hue[0.1880026820180965, 1, 1], 
                    Hue[0.1880026820180965, 1, 1], 
                    Hue[0.2571478502170692, 1, 1], 
                    Hue[0.2571478502170692, 1, 1], 
                    Hue[0.3273391787139689, 1, 1], 
                    Hue[0.3273391787139689, 1, 1], 
                    Hue[0.39856225931539346`, 1, 1], 
                    Hue[0.39856225931539346`, 1, 1], 
                    Hue[0.47080188987063876`, 1, 1], 
                    Hue[0.47080188987063876`, 1, 1], 
                    Hue[0.5440415604188835, 1, 1], 
                    Hue[0.5440415604188835, 1, 1], 
                    Hue[0.6182681067487781, 1, 1], 
                    Hue[0.6182681067487781, 1, 1], 
                    Hue[0.6934684017449322, 1, 1], 
                    Hue[0.6934684017449322, 1, 1], 
                    Hue[0.7696304170032996, 1, 1], 
                    Hue[0.7696304170032996, 1, 1], 
                    Hue[0.8467428738832437, 1, 1], 
                    Hue[0.8467428738832437, 1, 1], 
                    Hue[0.924790561415135, 1, 1], 
                    Hue[0.924790561415135, 1, 1], 
                    Hue[0.003762021946077935, 1, 1], 
                    Hue[0.003762021946077935, 1, 1], 
                    Hue[0.08364972930613161, 1, 1], 
                    Hue[0.08364972930613161, 1, 1], 
                    Hue[0.16444234996376278`, 1, 1], 
                    Hue[0.16444234996376278`, 1, 1], 
                    Hue[0.24613038943135804`, 1, 1], 
                    Hue[0.24613038943135804`, 1, 1], 
                    Hue[0.32870200662097754`, 1, 1], 
                    Hue[0.32870200662097754`, 1, 1], 
                    Hue[0.4121436885489237, 1, 1], 
                    Hue[0.4121436885489237, 1, 1], 
                    Hue[0.4964553658376828, 1, 1], 
                    Hue[0.4964553658376828, 1, 1], 
                    Hue[0.5816237800416533, 1, 1], 
                    Hue[0.5816237800416533, 1, 1], 
                    Hue[0.6676374651199203, 1, 1], 
                    Hue[0.6676374651199203, 1, 1], 
                    Hue[0.7544888084455762, 1, 1], 
                    Hue[0.7544888084455762, 1, 1], 
                    Hue[0.8421678489328173, 1, 1], 
                    Hue[0.8421678489328173, 1, 1], 
                    Hue[0.9306872201453219, 1, 1], 
                    Hue[0.9306872201453219, 1, 1], 
                    Hue[0.020028044921177234`, 1, 1], 
                    Hue[0.020028044921177234`, 1, 1], 
                    Hue[0.11015384531023154`, 1, 1], 
                    Hue[0.11015384531023154`, 1, 1], 
                    Hue[0.2010779247037523, 1, 1], 
                    Hue[0.2010779247037523, 1, 1], 
                    Hue[0.2928164041304295, 1, 1], 
                    Hue[0.2928164041304295, 1, 1], 
                    Hue[0.38533319026587, 1, 1], 
                    Hue[0.38533319026587, 1, 1], 
                    Hue[0.47860709073898694`, 1, 1], 
                    Hue[0.47860709073898694`, 1, 1], 
                    Hue[0.5726744637481624, 1, 1], 
                    Hue[0.5726744637481624, 1, 1], 
                    Hue[0.6675282615363339, 1, 1], 
                    Hue[0.6675282615363339, 1, 1], 
                    Hue[0.7631119700483477, 1, 1], 
                    Hue[0.7631119700483477, 1, 1], 
                    Hue[0.8594478928195965, 1, 1], 
                    Hue[0.8594478928195965, 1, 1], 
                    Hue[0.9565833397615986, 1, 1], 
                    Hue[0.9565833397615986, 1, 1], 
                    Hue[0.05442891042184869, 1, 1], 
                    Hue[0.05442891042184869, 1, 1], 
                    Hue[0.15295126944831824`, 1, 1], 
                    Hue[0.15295126944831824`, 1, 1], 
                    Hue[0.2523057656500986, 1, 1], 
                    Hue[0.2523057656500986, 1, 1], 
                    Hue[0.35240163726979257`, 1, 1], 
                    Hue[0.35240163726979257`, 1, 1], 
                    Hue[0.4530685174367239, 1, 1], 
                    Hue[0.4530685174367239, 1, 1], 
                    Hue[0.5546446182690019, 1, 1], 
                    Hue[0.5546446182690019, 1, 1], 
                    Hue[0.6571564465191764, 1, 1], 
                    Hue[0.6571564465191764, 1, 1], 
                    Hue[0.7599530157392963, 1, 1], 
                    Hue[0.7599530157392963, 1, 1], 
                    Hue[0.86323881815049, 1, 1], 
                    Hue[0.86323881815049, 1, 1], 
                    Hue[0.9678183223563761, 1, 1], 
                    Hue[0.9678183223563761, 1, 1], 
                    Hue[0.07322977077513246, 1, 1], 
                    Hue[0.07322977077513246, 1, 1], 
                    Hue[0.17878726522152358`, 1, 1], 
                    Hue[0.17878726522152358`, 1, 1], 
                    Hue[0.2853080718697084, 1, 1], 
                    Hue[0.2853080718697084, 1, 1], 
                    Hue[0.39338501586632146`, 1, 1], 
                    Hue[0.39338501586632146`, 1, 1], 
                    Hue[0.5017840227505358, 1, 1], 
                    Hue[0.5017840227505358, 1, 1], 
                    Hue[0.6096976623540389, 1, 1], 
                    Hue[0.6096976623540389, 1, 1], 
                    Hue[0.7186763960117638, 1, 1], 
                    Hue[0.7186763960117638, 1, 1], 
                    Hue[0.8296894368998564, 1, 1], 
                    Hue[0.8296894368998564, 1, 1], 
                    Hue[0.9407097079137103, 1, 1], 
                    Hue[0.9407097079137103, 1, 1], 
                    Hue[0.05042165514197633, 1, 1], 
                    Hue[0.05042165514197633, 1, 1], 
                    Hue[0.1611490802839811, 1, 1], 
                    Hue[0.1611490802839811, 1, 1], 
                    Hue[0.2739566445765149, 1, 1], 
                    Hue[0.2739566445765149, 1, 1], 
                    Hue[0.38568383428444303`, 1, 1], 
                    Hue[0.38568383428444303`, 1, 1], 
                    Hue[0.4956018360147952, 1, 1], 
                    Hue[0.4956018360147952, 1, 1], 
                    Hue[0.6101557229342287, 1, 1], 
                    Hue[0.6101557229342287, 1, 1], 
                    Hue[0.7326512288615409, 1, 1], 
                    Hue[0.7326512288615409, 1, 1], 
                    Hue[0.8528047721115746, 1, 1], 
                    Hue[0.8528047721115746, 1, 1], 
                    Hue[0.9542542608112418, 1, 1], 
                    Hue[0.9542542608112418, 1, 1], 
                    Hue[0.0558345373254985, 1, 1], 
                    Hue[0.0558345373254985, 1, 1], 
                    Hue[0.1919635521635686, 1, 1], 
                    Hue[0.1919635521635686, 1, 1], 
                    Hue[0.34847173818108124`, 1, 1], 
                    Hue[0.34847173818108124`, 1, 1], 
                    Hue[0.48833365764146647`, 1, 1], 
                    Hue[0.48833365764146647`, 1, 1], 
                    Hue[0.5804563486032157, 1, 1], 
                    Hue[0.5804563486032157, 1, 1], 
                    Hue[0.7220462748464456, 1, 1], 
                    Hue[0.7220462748464456, 1, 1], 
                    Hue[0.939032460554266, 1, 1], 
                    Hue[0.939032460554266, 1, 1], 
                    Hue[0.20366116429204512`, 1, 1], 
                    Hue[0.20366116429204512`, 1, 1], 
                    Hue[0.4854402698286375, 1, 1], 
                    Hue[0.4854402698286375, 1, 1], 
                    Hue[0.7471307691604958, 1, 1], 
                    Hue[0.7471307691604958, 1, 1], 
                    Hue[0.982320906152381, 1, 1], 
                    Hue[0.982320906152381, 1, 1], 
                    Hue[0.18815268440253508`, 1, 1], 
                    Hue[0.18815268440253508`, 1, 1], 
                    Hue[0.2570783535583873, 1, 1], 
                    Hue[0.2570783535583873, 1, 1], 
                    Hue[0.28697439108716605`, 1, 1], 
                    Hue[0.28697439108716605`, 1, 1], 
                    Hue[0.4877808018408883, 1, 1], 
                    Hue[0.4877808018408883, 1, 1], 
                    Hue[0.7145537078305133, 1, 1], 
                    Hue[0.7145537078305133, 1, 1], 
                    Hue[0.9491777610707849, 1, 1], 
                    Hue[0.9491777610707849, 1, 1], 
                    Hue[0.18421774847407577`, 1, 1], 
                    Hue[0.18421774847407577`, 1, 1], 
                    Hue[0.40213423036871093`, 1, 1]}],
                   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
                   Axes->{True, None},
                   AxesOrigin->{0, 0},
                   
                   Epilog->LineBox[{{1, 8.282894589782796*^-7}, {
                    2, 1.1202877807983794`*^-6}, {3, 6.401588066558767*^-7}, {
                    4, 7.044868800422878*^-7}, {5, 5.063156044883944*^-7}, {
                    6, 6.359801451412574*^-7}, {7, 2.8240946626931734`*^-8}, {
                    8, 2.5856877142292223`*^-7}, {
                    9, 1.8892115337134525`*^-7}, {
                    10, 3.4125006734039956`*^-7}, {
                    11, 5.949025187346364*^-7}, {12, 3.991365478167852*^-7}, {
                    13, 9.308445072940111*^-8}, {
                    14, 4.5679961770593727`*^-7}, {
                    15, 4.261737112417841*^-7}, {16, 7.24800811334896*^-7}, {
                    17, 6.665932762372686*^-7}, {18, 7.570937846972064*^-7}, {
                    19, 4.5653925275661676`*^-7}, {
                    20, 8.792856427796261*^-8}, {
                    21, 2.0756682956855616`*^-7}, {
                    22, 1.4595892735127165`*^-7}, {
                    23, 4.128528806868028*^-7}, {24, 8.025050299090712*^-8}, {
                    25, 3.0109430416895856`*^-7}, {
                    26, 6.32527447117659*^-7}, {27, 7.220476592701592*^-7}, {
                    28, 6.078546112048344*^-7}, {29, 2.836595495698056*^-7}, {
                    30, 7.817803951595045*^-8}, {31, 4.130836281500989*^-7}, {
                    32, 8.077501389718034*^-7}, {33, 7.695854456873939*^-7}, {
                    34, 1.085921208119925*^-6}, {35, 8.271624497134395*^-7}, {
                    36, 8.101373205921064*^-7}, {37, 6.602088573970645*^-7}, {
                    38, 1.254642413330229*^-6}, {39, 7.852945997554673*^-7}, {
                    40, 1.4316997818164336`*^-6}, {
                    41, 1.155711798328248*^-6}, {42, 1.364144920999599*^-6}, {
                    43, 1.6561765717457883`*^-7}, {
                    44, 2.8662991973381915`*^-7}, {
                    45, 2.4458657582565824`*^-7}, {
                    46, 2.9177512669774036`*^-7}, {
                    47, 5.36454314871079*^-7}, {48, 7.171172377656772*^-7}, {
                    49, 9.139045972448797*^-7}, {
                    50, 1.1798845935274491`*^-6}, {
                    51, 1.087532731691727*^-6}, {
                    52, 1.3128950449757542`*^-6}, {
                    53, 8.650038653407684*^-7}, {54, 9.860043111151074*^-7}, {
                    55, 8.160602897951036*^-7}, {56, 4.202865967114354*^-7}, {
                    57, 7.461161585658179*^-7}, {
                    58, 1.2222415058272401`*^-6}, {
                    59, 1.3812463545144571`*^-6}, {
                    60, 1.943249590853916*^-6}, {
                    61, 2.7829321649514686`*^-6}, {
                    62, 2.517934741324609*^-6}, {
                    63, 2.3081514522406843`*^-6}, {
                    64, 1.2016925075575837`*^-6}, {
                    65, 6.561629017829647*^-7}, {66, 2.639759589456634*^-6}, {
                    67, 5.830265056689041*^-6}, {68, 8.096040582559952*^-6}, {
                    69, 9.49287197447064*^-6}, {70, 8.368529338993974*^-6}, {
                    71, 3.4127028985921227`*^-6}, {
                    72, 4.483027776331109*^-6}, {
                    73, 0.000016050128525853894`}, {
                    74, 0.000027151677460214952`}, {
                    75, 0.000035505285315733285`}, {
                    76, 0.00003660924174679686}, {
                    77, 0.000025647597058514927`}, {
                    78, 8.356035870788908*^-7}, {
                    79, 0.00004009766811245034}, {
                    80, 0.00008602760772684786}, {
                    81, 0.00012642893916107407`}, {
                    82, 0.00014365102492685858`}, {
                    83, 0.00012318069575425173`}, {
                    84, 0.00005126437237197791}, {
                    85, 0.00007882500105720271}, {
                    86, 0.00023778525628870314`}, {
                    87, 0.0003930665144030543}, {88, 0.0004912575420470028}, {
                    89, 0.0004727311763910969}, {
                    90, 0.00029430748580079206`}, {
                    91, 0.00011740502345334292`}, {
                    92, 0.0005769127362770662}, {
                    93, 0.0010928114531620662`}, {
                    94, 0.0014777623710194756`}, {95, 0.001549272439795872}, {
                    96, 0.001161992173624931}, {97, 0.0003942926735988468}, {
                    98, 0.001243140552018069}, {99, 0.002747803634833306}, {
                    100, 0.004007286629535681}, {101, 0.004501994724711602}, {
                    102, 0.0038064319734818185`}, {
                    103, 0.0018781085646712083`}, {
                    104, 0.0023593124203412546`}, {
                    105, 0.0062396105389808925`}, {
                    106, 0.009891904213206754}, {
                    107, 0.011915409055655536`}, {
                    108, 0.011161803479006237`}, {
                    109, 0.007197329097712717}, {110, 0.004178384670773133}, {
                    111, 0.012301690197007564`}, {
                    112, 0.022052108560225263`}, {
                    113, 0.029248867161787675`}, {
                    114, 0.031050819450525107`}, {115, 0.02569362070221519}, {
                    116, 0.014640225612128084`}, {
                    117, 0.017601648250927453`}, {
                    118, 0.040028297599298435`}, {119, 0.06389930114848874}, {
                    120, 0.08222092136134718}, {121, 0.08983109658350619}, {
                    122, 0.08341276279776674}, {123, 0.06361073359505347}, {
                    124, 0.0463297536197526}, {125, 0.07569790584388097}, {
                    126, 0.13985529758348023`}, {127, 0.21704656685070572`}, {
                    128, 0.29910422914198026`}, {129, 0.3808455360779263}, {
                    130, 0.4583950615642814}, {131, 0.5289761690873539}, {
                    132, 0.5908207667302015}, {133, 0.6430361002137149}, {
                    134, 0.6854282144974818}, {135, 0.7183048103294406}, {
                    136, 0.7423027731031244}, {137, 0.7582383920951351}, {
                    138, 0.767002957596903}, {139, 0.7694888277761736}, {
                    140, 0.7665396547500308}, {141, 0.7589294406918144}, {
                    142, 0.7473555060335769}, {143, 0.7324333712853562}, {
                    144, 0.714705652641966}, {145, 0.6946481542498032}, {
                    146, 0.6726809941314018}, {147, 0.6491733320677741}, {
                    148, 0.6244535646622339}, {149, 0.5988134359622094}, {
                    150, 0.5725150569269272}, {151, 0.5457907809056177}, {
                    152, 0.5188517459204195}, {153, 0.4918850974657282}, {
                    154, 0.4650609426653706}, {155, 0.4385284580432407}, {
                    156, 0.41242391380365534`}, {157, 0.38686449193825734`}, {
                    158, 0.36195245822798616`}, {159, 0.3377778499806569}, {
                    160, 0.31441637387129057`}, {161, 0.29192950025027925`}, {
                    162, 0.2703679794461138}, {163, 0.24976980536518267`}, {
                    164, 0.23016402587299736`}, {165, 0.21156662835981846`}, {
                    166, 0.19398778945503897`}, {167, 0.17742723439500027`}, {
                    168, 0.1618792319502118}, {169, 0.14732734151510063`}, {
                    170, 0.13375128083363577`}, {171, 0.12112680945088165`}, {
                    172, 0.10942268349709008`}, {173, 0.09860594824919854}, {
                    174, 0.08863999779100278}, {175, 0.07948470290587986}, {
                    176, 0.0710992520080257}, {177, 0.06344223741638448}, {
                    178, 0.056470654955747025`}, {179, 0.050141861919215}, {
                    180, 0.0444125623705725}, {181, 0.03924076322558548}, {
                    182, 0.034586353462668064`}, {
                    183, 0.030409361724688326`}, {184, 0.02667092254792324}, {
                    185, 0.02333450808967844}, {186, 0.020364749863763312`}, {
                    187, 0.01772947898612808}, {188, 0.015397459441858894`}, {
                    189, 0.013338801356442501`}, {
                    190, 0.011526950765240997`}, {
                    191, 0.009936577884119712}, {192, 0.008544951968553446}, {
                    193, 0.007330315677783491}, {194, 0.006271892728590545}, {
                    195, 0.0053528491714311946`}, {
                    196, 0.00455780672792188}, {
                    197, 0.0038711876315099597`}, {
                    198, 0.003279415810681045}, {
                    199, 0.0027714821274414845`}, {
                    200, 0.002336752306047296}, {201, 0.0019650292793226}, {
                    202, 0.0016482241262018693`}, {
                    203, 0.0013794275726454264`}, {
                    204, 0.0011515286174456506`}, {
                    205, 0.0009585524383507281}, {
                    206, 0.0007962705305678472}, {
                    207, 0.000660047342607368}, {
                    208, 0.0005453469371544426}, {
                    209, 0.00044967000914317384`}, {
                    210, 0.00037030170950913175`}, {
                    211, 0.0003035938337062031}, {
                    212, 0.00024793018820254176`}, {
                    213, 0.00020264624208489855`}, {
                    214, 0.0001654277558815003}, {
                    215, 0.0001342232075630044}, {
                    216, 0.0001086588848457038}, {
                    217, 0.00008809691164050676}, {
                    218, 0.0000710648938132382}, {
                    219, 0.000056791536662500116`}, {
                    220, 0.000045352747704607155`}, {
                    221, 0.000036395800431590996`}, {
                    222, 0.000029023543349811118`}, {
                    223, 0.00002283672780455612}, {
                    224, 0.000017976851260834618`}, {
                    225, 0.000014276745780969344`}, {
                    226, 0.000011258062664574723`}, {
                    227, 8.774733210704453*^-6}, {
                    228, 6.953985392772539*^-6}, {
                    229, 5.653861783513688*^-6}, {
                    230, 4.472346382540365*^-6}, {
                    231, 3.281034507168652*^-6}, {
                    232, 2.4224831741165236`*^-6}, {
                    233, 2.148626485932848*^-6}, {
                    234, 1.9820437854623916`*^-6}, {
                    235, 1.5333194641015504`*^-6}, {
                    236, 9.581124662188444*^-7}, {
                    237, 6.973707942666438*^-7}, {
                    238, 7.532124124138606*^-7}, {
                    239, 7.363704676220984*^-7}, {
                    240, 6.509440332529519*^-7}, {
                    241, 6.190513563661241*^-7}, {
                    242, 6.05922147678224*^-7}, {
                    243, 5.153985542583597*^-7}, {
                    244, 3.2611470458455117`*^-7}, {
                    245, 1.3622647588506856`*^-7}, {
                    246, 3.0766703331644535`*^-7}, {
                    247, 5.335819969931085*^-7}, {
                    248, 6.501272689296098*^-7}, {
                    249, 6.209152995562534*^-7}, {
                    250, 4.6385849328152057`*^-7}, {
                    251, 2.4084287890744645`*^-7}}],
                   Frame->True,
                   FrameTicks->{{{1., 
                    FormBox[
                    RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
                    FormBox[
                    RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
                    FormBox[
                    RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
                    FormBox[
                    RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
                    FormBox["0", TraditionalForm]}, {157.25, 
                    FormBox["2.5`", TraditionalForm]}, {188.5, 
                    FormBox["5", TraditionalForm]}, {219.75, 
                    FormBox["7.5`", TraditionalForm]}, {251., 
                    FormBox["10", TraditionalForm]}}, Automatic, None, None},
                   PlotRange->{-0.1, 1.1},
                   PlotRangeClipping->True,
                   PlotRangePadding->{
                    Scaled[0.02], Automatic}], 14->
                  GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF12dYFOcaBuAB9WiiBkWMqBBWJURJMIAUFZDHBoqNIlWQYdmll8Ry7Dhi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                    "], {
                    {GrayLevel[0.5], EdgeForm[None], 
                    GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ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                    
                    "]]]}, {{}, {}, {
                    {Hue[0.5491258269772031, 1, 1], LineBox[{1, 2}]}, 
                    {Hue[0.2902796450076607, 1, 1], LineBox[{2, 3}]}, 
                    {Hue[0.3186348964907807, 1, 1], LineBox[{3, 4}]}, 
                    {Hue[0.2284208522423258, 1, 1], LineBox[{4, 5}]}, 
                    {Hue[0.12232320112241715`, 1, 1], LineBox[{5, 6}]}, 
                    {Hue[0.005010772832046706, 1, 1], LineBox[{6, 7}]}, 
                    {Hue[0.9903585419751219, 1, 1], LineBox[{7, 8}]}, 
                    {Hue[0.9685718751320311, 1, 1], LineBox[{8, 9}]}, 
                    {Hue[0.7718361162578176, 1, 1], LineBox[{9, 10}]}, 
                    {Hue[0.5534039765471076, 1, 1], LineBox[{10, 11}]}, 
                    {Hue[0.23758362191745483`, 1, 1], LineBox[{11, 12}]}, 
                    {Hue[0.6659801651782383, 1, 1], LineBox[{12, 13}]}, 
                    {Hue[0.49931996895800457`, 1, 1], LineBox[{13, 14}]}, 
                    {Hue[0.5318520191878946, 1, 1], LineBox[{14, 15}]}, 
                    {Hue[0.30564357579768725`, 1, 1], LineBox[{15, 16}]}, 
                    {Hue[0.9594404820243042, 1, 1], LineBox[{16, 17}]}, 
                    {Hue[0.7149204571508756, 1, 1], LineBox[{17, 18}]}, 
                    {Hue[0.6678921749008587, 1, 1], LineBox[{18, 19}]}, 
                    {Hue[0.44835131514272236`, 1, 1], LineBox[{19, 20}]}, 
                    {Hue[0.48327914376204195`, 1, 1], LineBox[{20, 21}]}, 
                    {Hue[0.24750800559439334`, 1, 1], LineBox[{21, 22}]}, 
                    {Hue[0.11519603984478773`, 1, 1], LineBox[{22, 23}]}, 
                    {Hue[0.01612414670244009, 1, 1], LineBox[{23, 24}]}, 
                    {Hue[0.8180816680337696, 1, 1], LineBox[{24, 25}]}, 
                    {Hue[0.6573322503084915, 1, 1], LineBox[{25, 26}]}, 
                    {Hue[0.5305707884219157, 1, 1], LineBox[{26, 27}]}, 
                    {Hue[0.49633086365786905`, 1, 1], LineBox[{27, 28}]}, 
                    {Hue[0.3204740927659566, 1, 1], LineBox[{28, 29}]}, 
                    {Hue[0.2517279746461726, 1, 1], LineBox[{29, 30}]}, 
                    {Hue[0.1268451327024544, 1, 1], LineBox[{30, 31}]}, 
                    {Hue[0.04492836641888041, 1, 1], LineBox[{31, 32}]}, 
                    {Hue[0.8506860772805684, 1, 1], LineBox[{32, 33}]}, 
                    {Hue[0.7245694467189367, 1, 1], LineBox[{33, 34}]}, 
                    {Hue[0.7509139492274237, 1, 1], LineBox[{34, 35}]}, 
                    {Hue[0.6843863315113896, 1, 1], LineBox[{35, 36}]}, 
                    {Hue[0.5370274938430456, 1, 1], LineBox[{36, 37}]}, 
                    {Hue[0.49195828763052996`, 1, 1], LineBox[{37, 38}]}, 
                    {Hue[0.308347341344207, 1, 1], LineBox[{38, 39}]}, 
                    {Hue[0.12705074290335513`, 1, 1], LineBox[{39, 40}]}, 
                    {Hue[0.10331329426250534`, 1, 1], LineBox[{40, 41}]}, 
                    {Hue[0.982509099525184, 1, 1], LineBox[{41, 42}]}, 
                    {Hue[0.787209221145821, 1, 1], LineBox[{42, 43}]}, 
                    {Hue[0.7700877038771718, 1, 1], LineBox[{43, 44}]}, 
                    {Hue[0.6361643271409572, 1, 1], LineBox[{44, 45}]}, 
                    {Hue[0.6068466023971579, 1, 1], LineBox[{45, 46}]}, 
                    {Hue[0.4791298717905439, 1, 1], LineBox[{46, 47}]}, 
                    {Hue[0.4357091773859966, 1, 1], LineBox[{47, 48}]}, 
                    {Hue[0.33105954022102446`, 1, 1], LineBox[{48, 49}]}, 
                    {Hue[0.2141133712894142, 1, 1], LineBox[{49, 50}]}, 
                    {Hue[0.15756837809381175`, 1, 1], LineBox[{50, 51}]}, 
                    {Hue[0.9332747186039484, 1, 1], LineBox[{51, 52}]}, 
                    {Hue[0.9791335752533095, 1, 1], LineBox[{52, 53}]}, 
                    {Hue[0.8384726387092356, 1, 1], LineBox[{53, 54}]}, 
                    {Hue[0.7364619005298396, 1, 1], LineBox[{54, 55}]}, 
                    {Hue[0.6629901252207137, 1, 1], LineBox[{55, 56}]}, 
                    {Hue[0.5783395681331325, 1, 1], LineBox[{56, 57}]}, 
                    {Hue[0.4684978175622351, 1, 1], LineBox[{57, 58}]}, 
                    {Hue[0.37244889106677387`, 1, 1], LineBox[{58, 59}]}, 
                    {Hue[0.29411967947531414`, 1, 1], LineBox[{59, 60}]}, 
                    {Hue[0.19682968958288313`, 1, 1], LineBox[{60, 61}]}, 
                    {Hue[0.08992445985436712, 1, 1], LineBox[{61, 62}]}, 
                    {Hue[0.9933438259434542, 1, 1], LineBox[{62, 63}]}, 
                    {Hue[0.8724541681175237, 1, 1], LineBox[{63, 64}]}, 
                    {Hue[0.7760773796938453, 1, 1], LineBox[{64, 65}]}, 
                    {Hue[0.6796254362241916, 1, 1], LineBox[{65, 66}]}, 
                    {Hue[0.5939543082492782, 1, 1], LineBox[{66, 67}]}, 
                    {Hue[0.5028392579847493, 1, 1], LineBox[{67, 68}]}, 
                    {Hue[0.4036498776674032, 1, 1], LineBox[{68, 69}]}, 
                    {Hue[0.3003020928823953, 1, 1], LineBox[{69, 70}]}, 
                    {Hue[0.1967022508173054, 1, 1], LineBox[{70, 71}]}, 
                    {Hue[0.09650347712384397, 1, 1], LineBox[{71, 72}]}, 
                    {Hue[0.0027723402406860794`, 1, 1], LineBox[{72, 73}]}, 
                    {Hue[0.9113108697618242, 1, 1], LineBox[{73, 74}]}, 
                    {Hue[0.8187928748678927, 1, 1], LineBox[{74, 75}]}, 
                    {Hue[0.7208690487729366, 1, 1], LineBox[{75, 76}]}, 
                    {Hue[0.6185444591778555, 1, 1], LineBox[{76, 77}]}, 
                    {Hue[0.5165902196272736, 1, 1], LineBox[{77, 78}]}, 
                    {Hue[0.42051768332086603`, 1, 1], LineBox[{78, 79}]}, 
                    {Hue[0.33070211397551663`, 1, 1], LineBox[{79, 80}]}, 
                    {Hue[0.2426103255226291, 1, 1], LineBox[{80, 81}]}, 
                    {Hue[0.15138932879895664`, 1, 1], LineBox[{81, 82}]}, 
                    {Hue[0.05249602680951814, 1, 1], LineBox[{82, 83}]}, 
                    {Hue[0.9478367133607696, 1, 1], LineBox[{83, 84}]}, 
                    {Hue[0.8475875861469279, 1, 1], LineBox[{84, 85}]}, 
                    {Hue[0.756300580286778, 1, 1], LineBox[{85, 86}]}, 
                    {Hue[0.6716889199006466, 1, 1], LineBox[{86, 87}]}, 
                    {Hue[0.5870825798302917, 1, 1], LineBox[{87, 88}]}, 
                    {Hue[0.495050509223539, 1, 1], LineBox[{88, 89}]}, 
                    {Hue[0.39262780127384156`, 1, 1], LineBox[{89, 90}]}, 
                    {Hue[0.28714126331748524`, 1, 1], LineBox[{90, 91}]}, 
                    {Hue[0.19112805442353883`, 1, 1], LineBox[{91, 92}]}, 
                    {Hue[0.10673525827672053`, 1, 1], LineBox[{92, 93}]}, 
                    {Hue[0.027522840384905385`, 1, 1], LineBox[{93, 94}]}, 
                    {Hue[0.9444803685835561, 1, 1], LineBox[{94, 95}]}, 
                    {Hue[0.8490249070559226, 1, 1], LineBox[{95, 96}]}, 
                    {Hue[0.7411649498814663, 1, 1], LineBox[{96, 97}]}, 
                    {Hue[0.6374666215429812, 1, 1], LineBox[{97, 98}]}, 
                    {Hue[0.5497931223572055, 1, 1], LineBox[{98, 99}]}, 
                    {Hue[0.4737291123864164, 1, 1], LineBox[{99, 100}]}, 
                    {Hue[0.3992080090157962, 1, 1], LineBox[{100, 101}]}, 
                    {Hue[0.31482774791091384`, 1, 1], LineBox[{101, 102}]}, 
                    {Hue[0.2113292305432758, 1, 1], LineBox[{102, 103}]}, 
                    {Hue[0.09908154232394249, 1, 1], LineBox[{103, 104}]}, 
                    {Hue[0.003580719173392775, 1, 1], LineBox[{104, 105}]}, 
                    {Hue[0.9277707136658423, 1, 1], LineBox[{105, 106}]}, 
                    {Hue[0.8605404856953217, 1, 1], LineBox[{106, 107}]}, 
                    {Hue[0.7895048468780809, 1, 1], LineBox[{107, 108}]}, 
                    {Hue[0.7002054335577579, 1, 1], LineBox[{108, 109}]}, 
                    {Hue[0.5856439885046414, 1, 1], LineBox[{109, 110}]}, 
                    {Hue[0.47569786975257744`, 1, 1], LineBox[{110, 111}]}, 
                    {Hue[0.3944836420368127, 1, 1], LineBox[{111, 112}]}, 
                    {Hue[0.33207192193375373`, 1, 1], LineBox[{112, 113}]}, 
                    {Hue[0.2748339851375885, 1, 1], LineBox[{113, 114}]}, 
                    {Hue[0.20987049241422776`, 1, 1], LineBox[{114, 115}]}, 
                    {Hue[0.11924584561484784`, 1, 1], LineBox[{115, 116}]}, 
                    {Hue[0.9968614524798072, 1, 1], LineBox[{116, 117}]}, 
                    {Hue[0.8915858603078806, 1, 1], LineBox[{117, 118}]}, 
                    {Hue[0.8232626433318473, 1, 1], LineBox[{118, 119}]}, 
                    {Hue[0.7746340198278019, 1, 1], LineBox[{119, 120}]}, 
                    {Hue[0.7331489905094091, 1, 1], LineBox[{120, 121}]}, 
                    {Hue[0.6902763475191502, 1, 1], LineBox[{121, 122}]}, 
                    {Hue[0.6346335888705981, 1, 1], LineBox[{122, 123}]}, 
                    {Hue[0.5435719785474447, 1, 1], LineBox[{123, 124}]}, 
                    {Hue[0.4138905816180223, 1, 1], LineBox[{124, 125}]}, 
                    {Hue[0.3198427828740491, 1, 1], LineBox[{125, 126}]}, 
                    {Hue[0.2689200438620375, 1, 1], LineBox[{126, 127}]}, 
                    {Hue[0.23808301889067057`, 1, 1], LineBox[{127, 128}]}, 
                    {Hue[0.21700129017557332`, 1, 1], LineBox[{128, 129}]}, 
                    {Hue[0.2015965309429917, 1, 1], LineBox[{129, 130}]}, 
                    {Hue[0.1900849351301606, 1, 1], LineBox[{130, 131}]}, 
                    {Hue[0.18161565069571153`, 1, 1], LineBox[{131, 132}]}, 
                    {Hue[0.17576281859083673`, 1, 1], LineBox[{132, 133}]}, 
                    {Hue[0.17231111257394194`, 1, 1], LineBox[{133, 134}]}, 
                    {Hue[0.171157003537818, 1, 1], LineBox[{134, 135}]}, 
                    {Hue[0.17225561589785562`, 1, 1], LineBox[{135, 136}]}, 
                    {Hue[0.1755919846585013, 1, 1], LineBox[{136, 137}]}, 
                    {Hue[0.18116294003375108`, 1, 1], LineBox[{137, 138}]}, 
                    {Hue[0.18896824793199118`, 1, 1], LineBox[{138, 139}]}, 
                    {Hue[0.19900410308070351`, 1, 1], LineBox[{139, 140}]}, 
                    {Hue[0.21126095026360003`, 1, 1], LineBox[{140, 141}]}, 
                    {Hue[0.2257205157148169, 1, 1], LineBox[{141, 142}]}, 
                    {Hue[0.24235848928686538`, 1, 1], LineBox[{142, 143}]}, 
                    {Hue[0.2611420249768013, 1, 1], LineBox[{143, 144}]}, 
                    {Hue[0.2820336741647051, 1, 1], LineBox[{144, 145}]}, 
                    {Hue[0.3049911010578967, 1, 1], LineBox[{145, 146}]}, 
                    {Hue[0.32996816191755635`, 1, 1], LineBox[{146, 147}]}, 
                    {Hue[0.3569163337053618, 1, 1], LineBox[{147, 148}]}, 
                    {Hue[0.38578591401286044`, 1, 1], LineBox[{148, 149}]}, 
                    {Hue[0.4165259364321822, 1, 1], LineBox[{149, 150}]}, 
                    {Hue[0.4490866391723868, 1, 1], LineBox[{150, 151}]}, 
                    {Hue[0.4834177366510568, 1, 1], LineBox[{151, 152}]}, 
                    {Hue[0.5194701836085935, 1, 1], LineBox[{152, 153}]}, 
                    {Hue[0.557196825271485, 1, 1], LineBox[{153, 154}]}, 
                    {Hue[0.5965518215974968, 1, 1], LineBox[{154, 155}]}, 
                    {Hue[0.6374904602151458, 1, 1], LineBox[{155, 156}]}, 
                    {Hue[0.6799708772044095, 1, 1], LineBox[{156, 157}]}, 
                    {Hue[0.7239522976890813, 1, 1], LineBox[{157, 158}]}, 
                    {Hue[0.769396386916378, 1, 1], LineBox[{158, 159}]}, 
                    {Hue[0.8162657220231337, 1, 1], LineBox[{159, 160}]}, 
                    {Hue[0.8645258254775823, 1, 1], LineBox[{160, 161}]}, 
                    {Hue[0.9141429735217279, 1, 1], LineBox[{161, 162}]}, 
                    {Hue[0.96508566814464, 1, 1], LineBox[{162, 163}]}, 
                    {Hue[0.017324549387385067`, 1, 1], LineBox[{163, 164}]}, 
                    {Hue[0.07083024111942525, 1, 1], LineBox[{164, 165}]}, 
                    {Hue[0.12557661019562788`, 1, 1], LineBox[{165, 166}]}, 
                    {Hue[0.18153833105298484`, 1, 1], LineBox[{166, 167}]}, 
                    {Hue[0.23869059105266235`, 1, 1], LineBox[{167, 168}]}, 
                    {Hue[0.29701003205840226`, 1, 1], LineBox[{168, 169}]}, 
                    {Hue[0.3564750346896973, 1, 1], LineBox[{169, 170}]}, 
                    {Hue[0.4170657262681432, 1, 1], LineBox[{170, 171}]}, 
                    {Hue[0.47876096011285807`, 1, 1], LineBox[{171, 172}]}, 
                    {Hue[0.5415437206250882, 1, 1], LineBox[{172, 173}]}, 
                    {Hue[0.60539444097929, 1, 1], LineBox[{173, 174}]}, 
                    {Hue[0.6702963986746799, 1, 1], LineBox[{174, 175}]}, 
                    {Hue[0.7362341144996116, 1, 1], LineBox[{175, 176}]}, 
                    {Hue[0.8031923601931057, 1, 1], LineBox[{176, 177}]}, 
                    {Hue[0.8711544244586391, 1, 1], LineBox[{177, 178}]}, 
                    {Hue[0.9401074600065064, 1, 1], LineBox[{178, 179}]}, 
                    {Hue[0.010038309244361136`, 1, 1], LineBox[{179, 180}]}, 
                    {Hue[0.08093288218141492, 1, 1], LineBox[{180, 181}]}, 
                    {Hue[0.15277982353789266`, 1, 1], LineBox[{181, 182}]}, 
                    {Hue[0.22556566530655825`, 1, 1], LineBox[{182, 183}]}, 
                    {Hue[0.29928098086556476`, 1, 1], LineBox[{183, 184}]}, 
                    {Hue[0.37391601020656645`, 1, 1], LineBox[{184, 185}]}, 
                    {Hue[0.4494558054736254, 1, 1], LineBox[{185, 186}]}, 
                    {Hue[0.5258909455116205, 1, 1], LineBox[{186, 187}]}, 
                    {Hue[0.6032152788066548, 1, 1], LineBox[{187, 188}]}, 
                    {Hue[0.6814168667752996, 1, 1], LineBox[{188, 189}]}, 
                    {Hue[0.7604874600919083, 1, 1], LineBox[{189, 190}]}, 
                    {Hue[0.840418989301121, 1, 1], LineBox[{190, 191}]}, 
                    {Hue[0.9211998295028575, 1, 1], LineBox[{191, 192}]}, 
                    {Hue[0.0028226054862767027`, 1, 1], 
                    LineBox[{192, 193}]}, 
                    {Hue[0.08528186990748743, 1, 1], LineBox[{193, 194}]}, 
                    {Hue[0.16856912526236964`, 1, 1], LineBox[{194, 195}]}, 
                    {Hue[0.2526801036479619, 1, 1], LineBox[{195, 196}]}, 
                    {Hue[0.3376012187689896, 1, 1], LineBox[{196, 197}]}, 
                    {Hue[0.4233218065639601, 1, 1], LineBox[{197, 198}]}, 
                    {Hue[0.5098467270130229, 1, 1], LineBox[{198, 199}]}, 
                    {Hue[0.5971651206611441, 1, 1], LineBox[{199, 200}]}, 
                    {Hue[0.6852741643229945, 1, 1], LineBox[{200, 201}]}, 
                    {Hue[0.7741608030557923, 1, 1], LineBox[{201, 202}]}, 
                    {Hue[0.8638069094138561, 1, 1], LineBox[{202, 203}]}, 
                    {Hue[0.9542258377497206, 1, 1], LineBox[{203, 204}]}, 
                    {Hue[0.0454115137177682, 1, 1], LineBox[{204, 205}]}, 
                    {Hue[0.13735385516522644`, 1, 1], LineBox[{205, 206}]}, 
                    {Hue[0.23004905610161971`, 1, 1], LineBox[{206, 207}]}, 
                    {Hue[0.3234724822822555, 1, 1], LineBox[{207, 208}]}, 
                    {Hue[0.4176385390663141, 1, 1], LineBox[{208, 209}]}, 
                    {Hue[0.5125795503518117, 1, 1], LineBox[{209, 210}]}, 
                    {Hue[0.6082348081135551, 1, 1], LineBox[{210, 211}]}, 
                    {Hue[0.7045341876617609, 1, 1], LineBox[{211, 212}]}, 
                    {Hue[0.801554485238327, 1, 1], LineBox[{212, 213}]}, 
                    {Hue[0.8993717427787195, 1, 1], LineBox[{213, 214}]}, 
                    {Hue[0.9978884459814782, 1, 1], LineBox[{214, 215}]}, 
                    {Hue[0.09703239989098028, 1, 1], LineBox[{215, 216}]}, 
                    {Hue[0.1969254467984167, 1, 1], LineBox[{216, 217}]}, 
                    {Hue[0.29759581352233877`, 1, 1], LineBox[{217, 218}]}, 
                    {Hue[0.39884143768934943`, 1, 1], LineBox[{218, 219}]}, 
                    {Hue[0.5007358447293813, 1, 1], LineBox[{219, 220}]}, 
                    {Hue[0.6035162687108159, 1, 1], LineBox[{220, 221}]}, 
                    {Hue[0.706861582499672, 1, 1], LineBox[{221, 222}]}, 
                    {Hue[0.8106624428033508, 1, 1], LineBox[{222, 223}]}, 
                    {Hue[0.9155282350120744, 1, 1], LineBox[{223, 224}]}, 
                    {Hue[0.021180426611272417`, 1, 1], LineBox[{224, 225}]}, 
                    {Hue[0.12692796944485524`, 1, 1], LineBox[{225, 226}]}, 
                    {Hue[0.233700700997123, 1, 1], LineBox[{226, 227}]}, 
                    {Hue[0.341946802462442, 1, 1], LineBox[{227, 228}]}, 
                    {Hue[0.4498968429690764, 1, 1], LineBox[{228, 229}]}, 
                    {Hue[0.5573824103226421, 1, 1], LineBox[{229, 230}]}, 
                    {Hue[0.6670150151902048, 1, 1], LineBox[{230, 231}]}, 
                    {Hue[0.7789105505834107, 1, 1], LineBox[{231, 232}]}, 
                    {Hue[0.8902108107720045, 1, 1], LineBox[{232, 233}]}, 
                    {Hue[0.0010083993451186876`, 1, 1], 
                    LineBox[{233, 234}]}, 
                    {Hue[0.11447518261633102`, 1, 1], LineBox[{234, 235}]}, 
                    {Hue[0.2295246674513252, 1, 1], LineBox[{235, 236}]}, 
                    {Hue[0.34142421738429607`, 1, 1], LineBox[{236, 237}]}, 
                    {Hue[0.451574669431246, 1, 1], LineBox[{237, 238}]}, 
                    {Hue[0.5666763034107078, 1, 1], LineBox[{238, 239}]}, 
                    {Hue[0.6850507481624744, 1, 1], LineBox[{239, 240}]}, 
                    {Hue[0.7984248249966265, 1, 1], LineBox[{240, 241}]}, 
                    {Hue[0.9072367859105431, 1, 1], LineBox[{241, 242}]}, 
                    {Hue[0.021987436943994295`, 1, 1], LineBox[{242, 243}]}, 
                    {Hue[0.1426344355952877, 1, 1], LineBox[{243, 244}]}, 
                    {Hue[0.2575711118171542, 1, 1], LineBox[{244, 245}]}, 
                    {Hue[0.3682158022952389, 1, 1], LineBox[{245, 246}]}, 
                    {Hue[0.49806656828861817`, 1, 1], LineBox[{246, 247}]}, 
                    {Hue[0.649111403365899, 1, 1], LineBox[{247, 248}]}, 
                    {Hue[0.7870001113529548, 1, 1], LineBox[{248, 249}]}, 
                    {Hue[0.8400106170584366, 1, 1], LineBox[{249, 250}]}, 
                    {Hue[0.9521427724380148, 1, 1], LineBox[{250, 251}]}}}},
                    VertexColors->{
                    Hue[0.5491258269772031, 1, 1], 
                    Hue[0.2902796450076607, 1, 1], 
                    Hue[0.3186348964907807, 1, 1], 
                    Hue[0.2284208522423258, 1, 1], 
                    Hue[0.12232320112241715`, 1, 1], 
                    Hue[0.005010772832046706, 1, 1], 
                    Hue[0.9903585419751219, 1, 1], 
                    Hue[0.9685718751320311, 1, 1], 
                    Hue[0.7718361162578176, 1, 1], 
                    Hue[0.5534039765471076, 1, 1], 
                    Hue[0.23758362191745483`, 1, 1], 
                    Hue[0.6659801651782383, 1, 1], 
                    Hue[0.49931996895800457`, 1, 1], 
                    Hue[0.5318520191878946, 1, 1], 
                    Hue[0.30564357579768725`, 1, 1], 
                    Hue[0.9594404820243042, 1, 1], 
                    Hue[0.7149204571508756, 1, 1], 
                    Hue[0.6678921749008587, 1, 1], 
                    Hue[0.44835131514272236`, 1, 1], 
                    Hue[0.48327914376204195`, 1, 1], 
                    Hue[0.24750800559439334`, 1, 1], 
                    Hue[0.11519603984478773`, 1, 1], 
                    Hue[0.01612414670244009, 1, 1], 
                    Hue[0.8180816680337696, 1, 1], 
                    Hue[0.6573322503084915, 1, 1], 
                    Hue[0.5305707884219157, 1, 1], 
                    Hue[0.49633086365786905`, 1, 1], 
                    Hue[0.3204740927659566, 1, 1], 
                    Hue[0.2517279746461726, 1, 1], 
                    Hue[0.1268451327024544, 1, 1], 
                    Hue[0.04492836641888041, 1, 1], 
                    Hue[0.8506860772805684, 1, 1], 
                    Hue[0.7245694467189367, 1, 1], 
                    Hue[0.7509139492274237, 1, 1], 
                    Hue[0.6843863315113896, 1, 1], 
                    Hue[0.5370274938430456, 1, 1], 
                    Hue[0.49195828763052996`, 1, 1], 
                    Hue[0.308347341344207, 1, 1], 
                    Hue[0.12705074290335513`, 1, 1], 
                    Hue[0.10331329426250534`, 1, 1], 
                    Hue[0.982509099525184, 1, 1], 
                    Hue[0.787209221145821, 1, 1], 
                    Hue[0.7700877038771718, 1, 1], 
                    Hue[0.6361643271409572, 1, 1], 
                    Hue[0.6068466023971579, 1, 1], 
                    Hue[0.4791298717905439, 1, 1], 
                    Hue[0.4357091773859966, 1, 1], 
                    Hue[0.33105954022102446`, 1, 1], 
                    Hue[0.2141133712894142, 1, 1], 
                    Hue[0.15756837809381175`, 1, 1], 
                    Hue[0.9332747186039484, 1, 1], 
                    Hue[0.9791335752533095, 1, 1], 
                    Hue[0.8384726387092356, 1, 1], 
                    Hue[0.7364619005298396, 1, 1], 
                    Hue[0.6629901252207137, 1, 1], 
                    Hue[0.5783395681331325, 1, 1], 
                    Hue[0.4684978175622351, 1, 1], 
                    Hue[0.37244889106677387`, 1, 1], 
                    Hue[0.29411967947531414`, 1, 1], 
                    Hue[0.19682968958288313`, 1, 1], 
                    Hue[0.08992445985436712, 1, 1], 
                    Hue[0.9933438259434542, 1, 1], 
                    Hue[0.8724541681175237, 1, 1], 
                    Hue[0.7760773796938453, 1, 1], 
                    Hue[0.6796254362241916, 1, 1], 
                    Hue[0.5939543082492782, 1, 1], 
                    Hue[0.5028392579847493, 1, 1], 
                    Hue[0.4036498776674032, 1, 1], 
                    Hue[0.3003020928823953, 1, 1], 
                    Hue[0.1967022508173054, 1, 1], 
                    Hue[0.09650347712384397, 1, 1], 
                    Hue[0.0027723402406860794`, 1, 1], 
                    Hue[0.9113108697618242, 1, 1], 
                    Hue[0.8187928748678927, 1, 1], 
                    Hue[0.7208690487729366, 1, 1], 
                    Hue[0.6185444591778555, 1, 1], 
                    Hue[0.5165902196272736, 1, 1], 
                    Hue[0.42051768332086603`, 1, 1], 
                    Hue[0.33070211397551663`, 1, 1], 
                    Hue[0.2426103255226291, 1, 1], 
                    Hue[0.15138932879895664`, 1, 1], 
                    Hue[0.05249602680951814, 1, 1], 
                    Hue[0.9478367133607696, 1, 1], 
                    Hue[0.8475875861469279, 1, 1], 
                    Hue[0.756300580286778, 1, 1], 
                    Hue[0.6716889199006466, 1, 1], 
                    Hue[0.5870825798302917, 1, 1], 
                    Hue[0.495050509223539, 1, 1], 
                    Hue[0.39262780127384156`, 1, 1], 
                    Hue[0.28714126331748524`, 1, 1], 
                    Hue[0.19112805442353883`, 1, 1], 
                    Hue[0.10673525827672053`, 1, 1], 
                    Hue[0.027522840384905385`, 1, 1], 
                    Hue[0.9444803685835561, 1, 1], 
                    Hue[0.8490249070559226, 1, 1], 
                    Hue[0.7411649498814663, 1, 1], 
                    Hue[0.6374666215429812, 1, 1], 
                    Hue[0.5497931223572055, 1, 1], 
                    Hue[0.4737291123864164, 1, 1], 
                    Hue[0.3992080090157962, 1, 1], 
                    Hue[0.31482774791091384`, 1, 1], 
                    Hue[0.2113292305432758, 1, 1], 
                    Hue[0.09908154232394249, 1, 1], 
                    Hue[0.003580719173392775, 1, 1], 
                    Hue[0.9277707136658423, 1, 1], 
                    Hue[0.8605404856953217, 1, 1], 
                    Hue[0.7895048468780809, 1, 1], 
                    Hue[0.7002054335577579, 1, 1], 
                    Hue[0.5856439885046414, 1, 1], 
                    Hue[0.47569786975257744`, 1, 1], 
                    Hue[0.3944836420368127, 1, 1], 
                    Hue[0.33207192193375373`, 1, 1], 
                    Hue[0.2748339851375885, 1, 1], 
                    Hue[0.20987049241422776`, 1, 1], 
                    Hue[0.11924584561484784`, 1, 1], 
                    Hue[0.9968614524798072, 1, 1], 
                    Hue[0.8915858603078806, 1, 1], 
                    Hue[0.8232626433318473, 1, 1], 
                    Hue[0.7746340198278019, 1, 1], 
                    Hue[0.7331489905094091, 1, 1], 
                    Hue[0.6902763475191502, 1, 1], 
                    Hue[0.6346335888705981, 1, 1], 
                    Hue[0.5435719785474447, 1, 1], 
                    Hue[0.4138905816180223, 1, 1], 
                    Hue[0.3198427828740491, 1, 1], 
                    Hue[0.2689200438620375, 1, 1], 
                    Hue[0.23808301889067057`, 1, 1], 
                    Hue[0.21700129017557332`, 1, 1], 
                    Hue[0.2015965309429917, 1, 1], 
                    Hue[0.1900849351301606, 1, 1], 
                    Hue[0.18161565069571153`, 1, 1], 
                    Hue[0.17576281859083673`, 1, 1], 
                    Hue[0.17231111257394194`, 1, 1], 
                    Hue[0.171157003537818, 1, 1], 
                    Hue[0.17225561589785562`, 1, 1], 
                    Hue[0.1755919846585013, 1, 1], 
                    Hue[0.18116294003375108`, 1, 1], 
                    Hue[0.18896824793199118`, 1, 1], 
                    Hue[0.19900410308070351`, 1, 1], 
                    Hue[0.21126095026360003`, 1, 1], 
                    Hue[0.2257205157148169, 1, 1], 
                    Hue[0.24235848928686538`, 1, 1], 
                    Hue[0.2611420249768013, 1, 1], 
                    Hue[0.2820336741647051, 1, 1], 
                    Hue[0.3049911010578967, 1, 1], 
                    Hue[0.32996816191755635`, 1, 1], 
                    Hue[0.3569163337053618, 1, 1], 
                    Hue[0.38578591401286044`, 1, 1], 
                    Hue[0.4165259364321822, 1, 1], 
                    Hue[0.4490866391723868, 1, 1], 
                    Hue[0.4834177366510568, 1, 1], 
                    Hue[0.5194701836085935, 1, 1], 
                    Hue[0.557196825271485, 1, 1], 
                    Hue[0.5965518215974968, 1, 1], 
                    Hue[0.6374904602151458, 1, 1], 
                    Hue[0.6799708772044095, 1, 1], 
                    Hue[0.7239522976890813, 1, 1], 
                    Hue[0.769396386916378, 1, 1], 
                    Hue[0.8162657220231337, 1, 1], 
                    Hue[0.8645258254775823, 1, 1], 
                    Hue[0.9141429735217279, 1, 1], 
                    Hue[0.96508566814464, 1, 1], 
                    Hue[0.017324549387385067`, 1, 1], 
                    Hue[0.07083024111942525, 1, 1], 
                    Hue[0.12557661019562788`, 1, 1], 
                    Hue[0.18153833105298484`, 1, 1], 
                    Hue[0.23869059105266235`, 1, 1], 
                    Hue[0.29701003205840226`, 1, 1], 
                    Hue[0.3564750346896973, 1, 1], 
                    Hue[0.4170657262681432, 1, 1], 
                    Hue[0.47876096011285807`, 1, 1], 
                    Hue[0.5415437206250882, 1, 1], 
                    Hue[0.60539444097929, 1, 1], 
                    Hue[0.6702963986746799, 1, 1], 
                    Hue[0.7362341144996116, 1, 1], 
                    Hue[0.8031923601931057, 1, 1], 
                    Hue[0.8711544244586391, 1, 1], 
                    Hue[0.9401074600065064, 1, 1], 
                    Hue[0.010038309244361136`, 1, 1], 
                    Hue[0.08093288218141492, 1, 1], 
                    Hue[0.15277982353789266`, 1, 1], 
                    Hue[0.22556566530655825`, 1, 1], 
                    Hue[0.29928098086556476`, 1, 1], 
                    Hue[0.37391601020656645`, 1, 1], 
                    Hue[0.4494558054736254, 1, 1], 
                    Hue[0.5258909455116205, 1, 1], 
                    Hue[0.6032152788066548, 1, 1], 
                    Hue[0.6814168667752996, 1, 1], 
                    Hue[0.7604874600919083, 1, 1], 
                    Hue[0.840418989301121, 1, 1], 
                    Hue[0.9211998295028575, 1, 1], 
                    Hue[0.0028226054862767027`, 1, 1], 
                    Hue[0.08528186990748743, 1, 1], 
                    Hue[0.16856912526236964`, 1, 1], 
                    Hue[0.2526801036479619, 1, 1], 
                    Hue[0.3376012187689896, 1, 1], 
                    Hue[0.4233218065639601, 1, 1], 
                    Hue[0.5098467270130229, 1, 1], 
                    Hue[0.5971651206611441, 1, 1], 
                    Hue[0.6852741643229945, 1, 1], 
                    Hue[0.7741608030557923, 1, 1], 
                    Hue[0.8638069094138561, 1, 1], 
                    Hue[0.9542258377497206, 1, 1], 
                    Hue[0.0454115137177682, 1, 1], 
                    Hue[0.13735385516522644`, 1, 1], 
                    Hue[0.23004905610161971`, 1, 1], 
                    Hue[0.3234724822822555, 1, 1], 
                    Hue[0.4176385390663141, 1, 1], 
                    Hue[0.5125795503518117, 1, 1], 
                    Hue[0.6082348081135551, 1, 1], 
                    Hue[0.7045341876617609, 1, 1], 
                    Hue[0.801554485238327, 1, 1], 
                    Hue[0.8993717427787195, 1, 1], 
                    Hue[0.9978884459814782, 1, 1], 
                    Hue[0.09703239989098028, 1, 1], 
                    Hue[0.1969254467984167, 1, 1], 
                    Hue[0.29759581352233877`, 1, 1], 
                    Hue[0.39884143768934943`, 1, 1], 
                    Hue[0.5007358447293813, 1, 1], 
                    Hue[0.6035162687108159, 1, 1], 
                    Hue[0.706861582499672, 1, 1], 
                    Hue[0.8106624428033508, 1, 1], 
                    Hue[0.9155282350120744, 1, 1], 
                    Hue[0.021180426611272417`, 1, 1], 
                    Hue[0.12692796944485524`, 1, 1], 
                    Hue[0.233700700997123, 1, 1], 
                    Hue[0.341946802462442, 1, 1], 
                    Hue[0.4498968429690764, 1, 1], 
                    Hue[0.5573824103226421, 1, 1], 
                    Hue[0.6670150151902048, 1, 1], 
                    Hue[0.7789105505834107, 1, 1], 
                    Hue[0.8902108107720045, 1, 1], 
                    Hue[0.0010083993451186876`, 1, 1], 
                    Hue[0.11447518261633102`, 1, 1], 
                    Hue[0.2295246674513252, 1, 1], 
                    Hue[0.34142421738429607`, 1, 1], 
                    Hue[0.451574669431246, 1, 1], 
                    Hue[0.5666763034107078, 1, 1], 
                    Hue[0.6850507481624744, 1, 1], 
                    Hue[0.7984248249966265, 1, 1], 
                    Hue[0.9072367859105431, 1, 1], 
                    Hue[0.021987436943994295`, 1, 1], 
                    Hue[0.1426344355952877, 1, 1], 
                    Hue[0.2575711118171542, 1, 1], 
                    Hue[0.3682158022952389, 1, 1], 
                    Hue[0.49806656828861817`, 1, 1], 
                    Hue[0.649111403365899, 1, 1], 
                    Hue[0.7870001113529548, 1, 1], 
                    Hue[0.8400106170584366, 1, 1], 
                    Hue[0.9521427724380148, 1, 1], 
                    Hue[0.1497588270495352, 1, 1], 
                    Hue[0.5491258269772031, 1, 1], 
                    Hue[0.2902796450076607, 1, 1], 
                    Hue[0.2902796450076607, 1, 1], 
                    Hue[0.3186348964907807, 1, 1], 
                    Hue[0.3186348964907807, 1, 1], 
                    Hue[0.2284208522423258, 1, 1], 
                    Hue[0.2284208522423258, 1, 1], 
                    Hue[0.12232320112241715`, 1, 1], 
                    Hue[0.12232320112241715`, 1, 1], 
                    Hue[0.005010772832046706, 1, 1], 
                    Hue[0.005010772832046706, 1, 1], 
                    Hue[0.9903585419751219, 1, 1], 
                    Hue[0.9903585419751219, 1, 1], 
                    Hue[0.9685718751320311, 1, 1], 
                    Hue[0.9685718751320311, 1, 1], 
                    Hue[0.7718361162578176, 1, 1], 
                    Hue[0.7718361162578176, 1, 1], 
                    Hue[0.5534039765471076, 1, 1], 
                    Hue[0.5534039765471076, 1, 1], 
                    Hue[0.23758362191745483`, 1, 1], 
                    Hue[0.23758362191745483`, 1, 1], 
                    Hue[0.6659801651782383, 1, 1], 
                    Hue[0.6659801651782383, 1, 1], 
                    Hue[0.49931996895800457`, 1, 1], 
                    Hue[0.49931996895800457`, 1, 1], 
                    Hue[0.5318520191878946, 1, 1], 
                    Hue[0.5318520191878946, 1, 1], 
                    Hue[0.30564357579768725`, 1, 1], 
                    Hue[0.30564357579768725`, 1, 1], 
                    Hue[0.9594404820243042, 1, 1], 
                    Hue[0.9594404820243042, 1, 1], 
                    Hue[0.7149204571508756, 1, 1], 
                    Hue[0.7149204571508756, 1, 1], 
                    Hue[0.6678921749008587, 1, 1], 
                    Hue[0.6678921749008587, 1, 1], 
                    Hue[0.44835131514272236`, 1, 1], 
                    Hue[0.44835131514272236`, 1, 1], 
                    Hue[0.48327914376204195`, 1, 1], 
                    Hue[0.48327914376204195`, 1, 1], 
                    Hue[0.24750800559439334`, 1, 1], 
                    Hue[0.24750800559439334`, 1, 1], 
                    Hue[0.11519603984478773`, 1, 1], 
                    Hue[0.11519603984478773`, 1, 1], 
                    Hue[0.01612414670244009, 1, 1], 
                    Hue[0.01612414670244009, 1, 1], 
                    Hue[0.8180816680337696, 1, 1], 
                    Hue[0.8180816680337696, 1, 1], 
                    Hue[0.6573322503084915, 1, 1], 
                    Hue[0.6573322503084915, 1, 1], 
                    Hue[0.5305707884219157, 1, 1], 
                    Hue[0.5305707884219157, 1, 1], 
                    Hue[0.49633086365786905`, 1, 1], 
                    Hue[0.49633086365786905`, 1, 1], 
                    Hue[0.3204740927659566, 1, 1], 
                    Hue[0.3204740927659566, 1, 1], 
                    Hue[0.2517279746461726, 1, 1], 
                    Hue[0.2517279746461726, 1, 1], 
                    Hue[0.1268451327024544, 1, 1], 
                    Hue[0.1268451327024544, 1, 1], 
                    Hue[0.04492836641888041, 1, 1], 
                    Hue[0.04492836641888041, 1, 1], 
                    Hue[0.8506860772805684, 1, 1], 
                    Hue[0.8506860772805684, 1, 1], 
                    Hue[0.7245694467189367, 1, 1], 
                    Hue[0.7245694467189367, 1, 1], 
                    Hue[0.7509139492274237, 1, 1], 
                    Hue[0.7509139492274237, 1, 1], 
                    Hue[0.6843863315113896, 1, 1], 
                    Hue[0.6843863315113896, 1, 1], 
                    Hue[0.5370274938430456, 1, 1], 
                    Hue[0.5370274938430456, 1, 1], 
                    Hue[0.49195828763052996`, 1, 1], 
                    Hue[0.49195828763052996`, 1, 1], 
                    Hue[0.308347341344207, 1, 1], 
                    Hue[0.308347341344207, 1, 1], 
                    Hue[0.12705074290335513`, 1, 1], 
                    Hue[0.12705074290335513`, 1, 1], 
                    Hue[0.10331329426250534`, 1, 1], 
                    Hue[0.10331329426250534`, 1, 1], 
                    Hue[0.982509099525184, 1, 1], 
                    Hue[0.982509099525184, 1, 1], 
                    Hue[0.787209221145821, 1, 1], 
                    Hue[0.787209221145821, 1, 1], 
                    Hue[0.7700877038771718, 1, 1], 
                    Hue[0.7700877038771718, 1, 1], 
                    Hue[0.6361643271409572, 1, 1], 
                    Hue[0.6361643271409572, 1, 1], 
                    Hue[0.6068466023971579, 1, 1], 
                    Hue[0.6068466023971579, 1, 1], 
                    Hue[0.4791298717905439, 1, 1], 
                    Hue[0.4791298717905439, 1, 1], 
                    Hue[0.4357091773859966, 1, 1], 
                    Hue[0.4357091773859966, 1, 1], 
                    Hue[0.33105954022102446`, 1, 1], 
                    Hue[0.33105954022102446`, 1, 1], 
                    Hue[0.2141133712894142, 1, 1], 
                    Hue[0.2141133712894142, 1, 1], 
                    Hue[0.15756837809381175`, 1, 1], 
                    Hue[0.15756837809381175`, 1, 1], 
                    Hue[0.9332747186039484, 1, 1], 
                    Hue[0.9332747186039484, 1, 1], 
                    Hue[0.9791335752533095, 1, 1], 
                    Hue[0.9791335752533095, 1, 1], 
                    Hue[0.8384726387092356, 1, 1], 
                    Hue[0.8384726387092356, 1, 1], 
                    Hue[0.7364619005298396, 1, 1], 
                    Hue[0.7364619005298396, 1, 1], 
                    Hue[0.6629901252207137, 1, 1], 
                    Hue[0.6629901252207137, 1, 1], 
                    Hue[0.5783395681331325, 1, 1], 
                    Hue[0.5783395681331325, 1, 1], 
                    Hue[0.4684978175622351, 1, 1], 
                    Hue[0.4684978175622351, 1, 1], 
                    Hue[0.37244889106677387`, 1, 1], 
                    Hue[0.37244889106677387`, 1, 1], 
                    Hue[0.29411967947531414`, 1, 1], 
                    Hue[0.29411967947531414`, 1, 1], 
                    Hue[0.19682968958288313`, 1, 1], 
                    Hue[0.19682968958288313`, 1, 1], 
                    Hue[0.08992445985436712, 1, 1], 
                    Hue[0.08992445985436712, 1, 1], 
                    Hue[0.9933438259434542, 1, 1], 
                    Hue[0.9933438259434542, 1, 1], 
                    Hue[0.8724541681175237, 1, 1], 
                    Hue[0.8724541681175237, 1, 1], 
                    Hue[0.7760773796938453, 1, 1], 
                    Hue[0.7760773796938453, 1, 1], 
                    Hue[0.6796254362241916, 1, 1], 
                    Hue[0.6796254362241916, 1, 1], 
                    Hue[0.5939543082492782, 1, 1], 
                    Hue[0.5939543082492782, 1, 1], 
                    Hue[0.5028392579847493, 1, 1], 
                    Hue[0.5028392579847493, 1, 1], 
                    Hue[0.4036498776674032, 1, 1], 
                    Hue[0.4036498776674032, 1, 1], 
                    Hue[0.3003020928823953, 1, 1], 
                    Hue[0.3003020928823953, 1, 1], 
                    Hue[0.1967022508173054, 1, 1], 
                    Hue[0.1967022508173054, 1, 1], 
                    Hue[0.09650347712384397, 1, 1], 
                    Hue[0.09650347712384397, 1, 1], 
                    Hue[0.0027723402406860794`, 1, 1], 
                    Hue[0.0027723402406860794`, 1, 1], 
                    Hue[0.9113108697618242, 1, 1], 
                    Hue[0.9113108697618242, 1, 1], 
                    Hue[0.8187928748678927, 1, 1], 
                    Hue[0.8187928748678927, 1, 1], 
                    Hue[0.7208690487729366, 1, 1], 
                    Hue[0.7208690487729366, 1, 1], 
                    Hue[0.6185444591778555, 1, 1], 
                    Hue[0.6185444591778555, 1, 1], 
                    Hue[0.5165902196272736, 1, 1], 
                    Hue[0.5165902196272736, 1, 1], 
                    Hue[0.42051768332086603`, 1, 1], 
                    Hue[0.42051768332086603`, 1, 1], 
                    Hue[0.33070211397551663`, 1, 1], 
                    Hue[0.33070211397551663`, 1, 1], 
                    Hue[0.2426103255226291, 1, 1], 
                    Hue[0.2426103255226291, 1, 1], 
                    Hue[0.15138932879895664`, 1, 1], 
                    Hue[0.15138932879895664`, 1, 1], 
                    Hue[0.05249602680951814, 1, 1], 
                    Hue[0.05249602680951814, 1, 1], 
                    Hue[0.9478367133607696, 1, 1], 
                    Hue[0.9478367133607696, 1, 1], 
                    Hue[0.8475875861469279, 1, 1], 
                    Hue[0.8475875861469279, 1, 1], 
                    Hue[0.756300580286778, 1, 1], 
                    Hue[0.756300580286778, 1, 1], 
                    Hue[0.6716889199006466, 1, 1], 
                    Hue[0.6716889199006466, 1, 1], 
                    Hue[0.5870825798302917, 1, 1], 
                    Hue[0.5870825798302917, 1, 1], 
                    Hue[0.495050509223539, 1, 1], 
                    Hue[0.495050509223539, 1, 1], 
                    Hue[0.39262780127384156`, 1, 1], 
                    Hue[0.39262780127384156`, 1, 1], 
                    Hue[0.28714126331748524`, 1, 1], 
                    Hue[0.28714126331748524`, 1, 1], 
                    Hue[0.19112805442353883`, 1, 1], 
                    Hue[0.19112805442353883`, 1, 1], 
                    Hue[0.10673525827672053`, 1, 1], 
                    Hue[0.10673525827672053`, 1, 1], 
                    Hue[0.027522840384905385`, 1, 1], 
                    Hue[0.027522840384905385`, 1, 1], 
                    Hue[0.9444803685835561, 1, 1], 
                    Hue[0.9444803685835561, 1, 1], 
                    Hue[0.8490249070559226, 1, 1], 
                    Hue[0.8490249070559226, 1, 1], 
                    Hue[0.7411649498814663, 1, 1], 
                    Hue[0.7411649498814663, 1, 1], 
                    Hue[0.6374666215429812, 1, 1], 
                    Hue[0.6374666215429812, 1, 1], 
                    Hue[0.5497931223572055, 1, 1], 
                    Hue[0.5497931223572055, 1, 1], 
                    Hue[0.4737291123864164, 1, 1], 
                    Hue[0.4737291123864164, 1, 1], 
                    Hue[0.3992080090157962, 1, 1], 
                    Hue[0.3992080090157962, 1, 1], 
                    Hue[0.31482774791091384`, 1, 1], 
                    Hue[0.31482774791091384`, 1, 1], 
                    Hue[0.2113292305432758, 1, 1], 
                    Hue[0.2113292305432758, 1, 1], 
                    Hue[0.09908154232394249, 1, 1], 
                    Hue[0.09908154232394249, 1, 1], 
                    Hue[0.003580719173392775, 1, 1], 
                    Hue[0.003580719173392775, 1, 1], 
                    Hue[0.9277707136658423, 1, 1], 
                    Hue[0.9277707136658423, 1, 1], 
                    Hue[0.8605404856953217, 1, 1], 
                    Hue[0.8605404856953217, 1, 1], 
                    Hue[0.7895048468780809, 1, 1], 
                    Hue[0.7895048468780809, 1, 1], 
                    Hue[0.7002054335577579, 1, 1], 
                    Hue[0.7002054335577579, 1, 1], 
                    Hue[0.5856439885046414, 1, 1], 
                    Hue[0.5856439885046414, 1, 1], 
                    Hue[0.47569786975257744`, 1, 1], 
                    Hue[0.47569786975257744`, 1, 1], 
                    Hue[0.3944836420368127, 1, 1], 
                    Hue[0.3944836420368127, 1, 1], 
                    Hue[0.33207192193375373`, 1, 1], 
                    Hue[0.33207192193375373`, 1, 1], 
                    Hue[0.2748339851375885, 1, 1], 
                    Hue[0.2748339851375885, 1, 1], 
                    Hue[0.20987049241422776`, 1, 1], 
                    Hue[0.20987049241422776`, 1, 1], 
                    Hue[0.11924584561484784`, 1, 1], 
                    Hue[0.11924584561484784`, 1, 1], 
                    Hue[0.9968614524798072, 1, 1], 
                    Hue[0.9968614524798072, 1, 1], 
                    Hue[0.8915858603078806, 1, 1], 
                    Hue[0.8915858603078806, 1, 1], 
                    Hue[0.8232626433318473, 1, 1], 
                    Hue[0.8232626433318473, 1, 1], 
                    Hue[0.7746340198278019, 1, 1], 
                    Hue[0.7746340198278019, 1, 1], 
                    Hue[0.7331489905094091, 1, 1], 
                    Hue[0.7331489905094091, 1, 1], 
                    Hue[0.6902763475191502, 1, 1], 
                    Hue[0.6902763475191502, 1, 1], 
                    Hue[0.6346335888705981, 1, 1], 
                    Hue[0.6346335888705981, 1, 1], 
                    Hue[0.5435719785474447, 1, 1], 
                    Hue[0.5435719785474447, 1, 1], 
                    Hue[0.4138905816180223, 1, 1], 
                    Hue[0.4138905816180223, 1, 1], 
                    Hue[0.3198427828740491, 1, 1], 
                    Hue[0.3198427828740491, 1, 1], 
                    Hue[0.2689200438620375, 1, 1], 
                    Hue[0.2689200438620375, 1, 1], 
                    Hue[0.23808301889067057`, 1, 1], 
                    Hue[0.23808301889067057`, 1, 1], 
                    Hue[0.21700129017557332`, 1, 1], 
                    Hue[0.21700129017557332`, 1, 1], 
                    Hue[0.2015965309429917, 1, 1], 
                    Hue[0.2015965309429917, 1, 1], 
                    Hue[0.1900849351301606, 1, 1], 
                    Hue[0.1900849351301606, 1, 1], 
                    Hue[0.18161565069571153`, 1, 1], 
                    Hue[0.18161565069571153`, 1, 1], 
                    Hue[0.17576281859083673`, 1, 1], 
                    Hue[0.17576281859083673`, 1, 1], 
                    Hue[0.17231111257394194`, 1, 1], 
                    Hue[0.17231111257394194`, 1, 1], 
                    Hue[0.171157003537818, 1, 1], 
                    Hue[0.171157003537818, 1, 1], 
                    Hue[0.17225561589785562`, 1, 1], 
                    Hue[0.17225561589785562`, 1, 1], 
                    Hue[0.1755919846585013, 1, 1], 
                    Hue[0.1755919846585013, 1, 1], 
                    Hue[0.18116294003375108`, 1, 1], 
                    Hue[0.18116294003375108`, 1, 1], 
                    Hue[0.18896824793199118`, 1, 1], 
                    Hue[0.18896824793199118`, 1, 1], 
                    Hue[0.19900410308070351`, 1, 1], 
                    Hue[0.19900410308070351`, 1, 1], 
                    Hue[0.21126095026360003`, 1, 1], 
                    Hue[0.21126095026360003`, 1, 1], 
                    Hue[0.2257205157148169, 1, 1], 
                    Hue[0.2257205157148169, 1, 1], 
                    Hue[0.24235848928686538`, 1, 1], 
                    Hue[0.24235848928686538`, 1, 1], 
                    Hue[0.2611420249768013, 1, 1], 
                    Hue[0.2611420249768013, 1, 1], 
                    Hue[0.2820336741647051, 1, 1], 
                    Hue[0.2820336741647051, 1, 1], 
                    Hue[0.3049911010578967, 1, 1], 
                    Hue[0.3049911010578967, 1, 1], 
                    Hue[0.32996816191755635`, 1, 1], 
                    Hue[0.32996816191755635`, 1, 1], 
                    Hue[0.3569163337053618, 1, 1], 
                    Hue[0.3569163337053618, 1, 1], 
                    Hue[0.38578591401286044`, 1, 1], 
                    Hue[0.38578591401286044`, 1, 1], 
                    Hue[0.4165259364321822, 1, 1], 
                    Hue[0.4165259364321822, 1, 1], 
                    Hue[0.4490866391723868, 1, 1], 
                    Hue[0.4490866391723868, 1, 1], 
                    Hue[0.4834177366510568, 1, 1], 
                    Hue[0.4834177366510568, 1, 1], 
                    Hue[0.5194701836085935, 1, 1], 
                    Hue[0.5194701836085935, 1, 1], 
                    Hue[0.557196825271485, 1, 1], 
                    Hue[0.557196825271485, 1, 1], 
                    Hue[0.5965518215974968, 1, 1], 
                    Hue[0.5965518215974968, 1, 1], 
                    Hue[0.6374904602151458, 1, 1], 
                    Hue[0.6374904602151458, 1, 1], 
                    Hue[0.6799708772044095, 1, 1], 
                    Hue[0.6799708772044095, 1, 1], 
                    Hue[0.7239522976890813, 1, 1], 
                    Hue[0.7239522976890813, 1, 1], 
                    Hue[0.769396386916378, 1, 1], 
                    Hue[0.769396386916378, 1, 1], 
                    Hue[0.8162657220231337, 1, 1], 
                    Hue[0.8162657220231337, 1, 1], 
                    Hue[0.8645258254775823, 1, 1], 
                    Hue[0.8645258254775823, 1, 1], 
                    Hue[0.9141429735217279, 1, 1], 
                    Hue[0.9141429735217279, 1, 1], 
                    Hue[0.96508566814464, 1, 1], 
                    Hue[0.96508566814464, 1, 1], 
                    Hue[0.017324549387385067`, 1, 1], 
                    Hue[0.017324549387385067`, 1, 1], 
                    Hue[0.07083024111942525, 1, 1], 
                    Hue[0.07083024111942525, 1, 1], 
                    Hue[0.12557661019562788`, 1, 1], 
                    Hue[0.12557661019562788`, 1, 1], 
                    Hue[0.18153833105298484`, 1, 1], 
                    Hue[0.18153833105298484`, 1, 1], 
                    Hue[0.23869059105266235`, 1, 1], 
                    Hue[0.23869059105266235`, 1, 1], 
                    Hue[0.29701003205840226`, 1, 1], 
                    Hue[0.29701003205840226`, 1, 1], 
                    Hue[0.3564750346896973, 1, 1], 
                    Hue[0.3564750346896973, 1, 1], 
                    Hue[0.4170657262681432, 1, 1], 
                    Hue[0.4170657262681432, 1, 1], 
                    Hue[0.47876096011285807`, 1, 1], 
                    Hue[0.47876096011285807`, 1, 1], 
                    Hue[0.5415437206250882, 1, 1], 
                    Hue[0.5415437206250882, 1, 1], 
                    Hue[0.60539444097929, 1, 1], 
                    Hue[0.60539444097929, 1, 1], 
                    Hue[0.6702963986746799, 1, 1], 
                    Hue[0.6702963986746799, 1, 1], 
                    Hue[0.7362341144996116, 1, 1], 
                    Hue[0.7362341144996116, 1, 1], 
                    Hue[0.8031923601931057, 1, 1], 
                    Hue[0.8031923601931057, 1, 1], 
                    Hue[0.8711544244586391, 1, 1], 
                    Hue[0.8711544244586391, 1, 1], 
                    Hue[0.9401074600065064, 1, 1], 
                    Hue[0.9401074600065064, 1, 1], 
                    Hue[0.010038309244361136`, 1, 1], 
                    Hue[0.010038309244361136`, 1, 1], 
                    Hue[0.08093288218141492, 1, 1], 
                    Hue[0.08093288218141492, 1, 1], 
                    Hue[0.15277982353789266`, 1, 1], 
                    Hue[0.15277982353789266`, 1, 1], 
                    Hue[0.22556566530655825`, 1, 1], 
                    Hue[0.22556566530655825`, 1, 1], 
                    Hue[0.29928098086556476`, 1, 1], 
                    Hue[0.29928098086556476`, 1, 1], 
                    Hue[0.37391601020656645`, 1, 1], 
                    Hue[0.37391601020656645`, 1, 1], 
                    Hue[0.4494558054736254, 1, 1], 
                    Hue[0.4494558054736254, 1, 1], 
                    Hue[0.5258909455116205, 1, 1], 
                    Hue[0.5258909455116205, 1, 1], 
                    Hue[0.6032152788066548, 1, 1], 
                    Hue[0.6032152788066548, 1, 1], 
                    Hue[0.6814168667752996, 1, 1], 
                    Hue[0.6814168667752996, 1, 1], 
                    Hue[0.7604874600919083, 1, 1], 
                    Hue[0.7604874600919083, 1, 1], 
                    Hue[0.840418989301121, 1, 1], 
                    Hue[0.840418989301121, 1, 1], 
                    Hue[0.9211998295028575, 1, 1], 
                    Hue[0.9211998295028575, 1, 1], 
                    Hue[0.0028226054862767027`, 1, 1], 
                    Hue[0.0028226054862767027`, 1, 1], 
                    Hue[0.08528186990748743, 1, 1], 
                    Hue[0.08528186990748743, 1, 1], 
                    Hue[0.16856912526236964`, 1, 1], 
                    Hue[0.16856912526236964`, 1, 1], 
                    Hue[0.2526801036479619, 1, 1], 
                    Hue[0.2526801036479619, 1, 1], 
                    Hue[0.3376012187689896, 1, 1], 
                    Hue[0.3376012187689896, 1, 1], 
                    Hue[0.4233218065639601, 1, 1], 
                    Hue[0.4233218065639601, 1, 1], 
                    Hue[0.5098467270130229, 1, 1], 
                    Hue[0.5098467270130229, 1, 1], 
                    Hue[0.5971651206611441, 1, 1], 
                    Hue[0.5971651206611441, 1, 1], 
                    Hue[0.6852741643229945, 1, 1], 
                    Hue[0.6852741643229945, 1, 1], 
                    Hue[0.7741608030557923, 1, 1], 
                    Hue[0.7741608030557923, 1, 1], 
                    Hue[0.8638069094138561, 1, 1], 
                    Hue[0.8638069094138561, 1, 1], 
                    Hue[0.9542258377497206, 1, 1], 
                    Hue[0.9542258377497206, 1, 1], 
                    Hue[0.0454115137177682, 1, 1], 
                    Hue[0.0454115137177682, 1, 1], 
                    Hue[0.13735385516522644`, 1, 1], 
                    Hue[0.13735385516522644`, 1, 1], 
                    Hue[0.23004905610161971`, 1, 1], 
                    Hue[0.23004905610161971`, 1, 1], 
                    Hue[0.3234724822822555, 1, 1], 
                    Hue[0.3234724822822555, 1, 1], 
                    Hue[0.4176385390663141, 1, 1], 
                    Hue[0.4176385390663141, 1, 1], 
                    Hue[0.5125795503518117, 1, 1], 
                    Hue[0.5125795503518117, 1, 1], 
                    Hue[0.6082348081135551, 1, 1], 
                    Hue[0.6082348081135551, 1, 1], 
                    Hue[0.7045341876617609, 1, 1], 
                    Hue[0.7045341876617609, 1, 1], 
                    Hue[0.801554485238327, 1, 1], 
                    Hue[0.801554485238327, 1, 1], 
                    Hue[0.8993717427787195, 1, 1], 
                    Hue[0.8993717427787195, 1, 1], 
                    Hue[0.9978884459814782, 1, 1], 
                    Hue[0.9978884459814782, 1, 1], 
                    Hue[0.09703239989098028, 1, 1], 
                    Hue[0.09703239989098028, 1, 1], 
                    Hue[0.1969254467984167, 1, 1], 
                    Hue[0.1969254467984167, 1, 1], 
                    Hue[0.29759581352233877`, 1, 1], 
                    Hue[0.29759581352233877`, 1, 1], 
                    Hue[0.39884143768934943`, 1, 1], 
                    Hue[0.39884143768934943`, 1, 1], 
                    Hue[0.5007358447293813, 1, 1], 
                    Hue[0.5007358447293813, 1, 1], 
                    Hue[0.6035162687108159, 1, 1], 
                    Hue[0.6035162687108159, 1, 1], 
                    Hue[0.706861582499672, 1, 1], 
                    Hue[0.706861582499672, 1, 1], 
                    Hue[0.8106624428033508, 1, 1], 
                    Hue[0.8106624428033508, 1, 1], 
                    Hue[0.9155282350120744, 1, 1], 
                    Hue[0.9155282350120744, 1, 1], 
                    Hue[0.021180426611272417`, 1, 1], 
                    Hue[0.021180426611272417`, 1, 1], 
                    Hue[0.12692796944485524`, 1, 1], 
                    Hue[0.12692796944485524`, 1, 1], 
                    Hue[0.233700700997123, 1, 1], 
                    Hue[0.233700700997123, 1, 1], 
                    Hue[0.341946802462442, 1, 1], 
                    Hue[0.341946802462442, 1, 1], 
                    Hue[0.4498968429690764, 1, 1], 
                    Hue[0.4498968429690764, 1, 1], 
                    Hue[0.5573824103226421, 1, 1], 
                    Hue[0.5573824103226421, 1, 1], 
                    Hue[0.6670150151902048, 1, 1], 
                    Hue[0.6670150151902048, 1, 1], 
                    Hue[0.7789105505834107, 1, 1], 
                    Hue[0.7789105505834107, 1, 1], 
                    Hue[0.8902108107720045, 1, 1], 
                    Hue[0.8902108107720045, 1, 1], 
                    Hue[0.0010083993451186876`, 1, 1], 
                    Hue[0.0010083993451186876`, 1, 1], 
                    Hue[0.11447518261633102`, 1, 1], 
                    Hue[0.11447518261633102`, 1, 1], 
                    Hue[0.2295246674513252, 1, 1], 
                    Hue[0.2295246674513252, 1, 1], 
                    Hue[0.34142421738429607`, 1, 1], 
                    Hue[0.34142421738429607`, 1, 1], 
                    Hue[0.451574669431246, 1, 1], 
                    Hue[0.451574669431246, 1, 1], 
                    Hue[0.5666763034107078, 1, 1], 
                    Hue[0.5666763034107078, 1, 1], 
                    Hue[0.6850507481624744, 1, 1], 
                    Hue[0.6850507481624744, 1, 1], 
                    Hue[0.7984248249966265, 1, 1], 
                    Hue[0.7984248249966265, 1, 1], 
                    Hue[0.9072367859105431, 1, 1], 
                    Hue[0.9072367859105431, 1, 1], 
                    Hue[0.021987436943994295`, 1, 1], 
                    Hue[0.021987436943994295`, 1, 1], 
                    Hue[0.1426344355952877, 1, 1], 
                    Hue[0.1426344355952877, 1, 1], 
                    Hue[0.2575711118171542, 1, 1], 
                    Hue[0.2575711118171542, 1, 1], 
                    Hue[0.3682158022952389, 1, 1], 
                    Hue[0.3682158022952389, 1, 1], 
                    Hue[0.49806656828861817`, 1, 1], 
                    Hue[0.49806656828861817`, 1, 1], 
                    Hue[0.649111403365899, 1, 1], 
                    Hue[0.649111403365899, 1, 1], 
                    Hue[0.7870001113529548, 1, 1], 
                    Hue[0.7870001113529548, 1, 1], 
                    Hue[0.8400106170584366, 1, 1], 
                    Hue[0.8400106170584366, 1, 1], 
                    Hue[0.9521427724380148, 1, 1], 
                    Hue[0.9521427724380148, 1, 1], 
                    Hue[0.1497588270495352, 1, 1]}],
                   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
                   Axes->{True, None},
                   AxesOrigin->{0, 0},
                   
                   Epilog->LineBox[{{1, 2.7840904793647274`*^-7}, {
                    2, 2.3476257877317546`*^-7}, {
                    3, 3.7725605325779893`*^-7}, {4, 5.807490365437347*^-7}, {
                    5, 7.951051815569195*^-7}, {6, 6.659267701819666*^-7}, {
                    7, 1.397677971224363*^-7}, {8, 9.840351126654453*^-7}, {
                    9, 7.9846795250673*^-7}, {10, 5.119020834217228*^-7}, {
                    11, 2.2216477783161265`*^-7}, {
                    12, 4.2435686756495455`*^-7}, {
                    13, 2.649908625297579*^-7}, {14, 1.248799406657194*^-7}, {
                    15, 2.877389767383945*^-7}, {16, 3.572019594724289*^-7}, {
                    17, 4.351030916368993*^-7}, {
                    18, 3.3635434343040655`*^-7}, {
                    19, 7.387204878867423*^-7}, {20, 2.554036706313653*^-7}, {
                    21, 6.028635670759956*^-7}, {22, 3.035024548502732*^-7}, {
                    23, 7.790968413916318*^-7}, {
                    24, 1.3220492103806469`*^-6}, {
                    25, 1.1906080581749486`*^-6}, {
                    26, 1.0494557179140746`*^-6}, {
                    27, 1.0562509774555886`*^-6}, {
                    28, 1.0023116103436475`*^-6}, {
                    29, 7.424402387187357*^-7}, {30, 7.46988550868365*^-7}, {
                    31, 5.082324094038862*^-7}, {32, 8.617668494863171*^-7}, {
                    33, 2.4326689626910326`*^-7}, {
                    34, 7.959911776922238*^-7}, {35, 5.174320123754048*^-7}, {
                    36, 9.977542287057794*^-7}, {37, 3.226818329820662*^-7}, {
                    38, 8.765669391453493*^-7}, {
                    39, 4.1125255842224804`*^-7}, {
                    40, 9.16886371658223*^-8}, {41, 7.761352642843013*^-7}, {
                    42, 1.0669064162438863`*^-6}, {
                    43, 7.348082300949503*^-7}, {44, 1.59826264803697*^-6}, {
                    45, 1.1762956643339447`*^-6}, {
                    46, 1.221418004506105*^-6}, {
                    47, 1.6255506956415514`*^-6}, {
                    48, 1.196383493766683*^-6}, {49, 9.21535848364169*^-7}, {
                    50, 1.0676248885012513`*^-6}, {
                    51, 7.074365585755711*^-7}, {52, 9.853935178781322*^-7}, {
                    53, 1.778535674658169*^-6}, {
                    54, 2.0455115818133567`*^-6}, {
                    55, 2.218837704874427*^-6}, {
                    56, 3.4472579681669604`*^-6}, {
                    57, 3.839052117698447*^-6}, {58, 4.363762494023019*^-6}, {
                    59, 4.841612582311263*^-6}, {60, 5.536577620935596*^-6}, {
                    61, 6.6136678926983065`*^-6}, {
                    62, 8.802235873570966*^-6}, {
                    63, 0.000010961041456343846`}, {
                    64, 0.000013859787605476625`}, {
                    65, 0.00001703461906682615}, {
                    66, 0.000021058748655187032`}, {
                    67, 0.000025199897993291865`}, {
                    68, 0.00002968139579200682}, {
                    69, 0.00003505716324792632}, {
                    70, 0.000043027678566643007`}, {
                    71, 0.000053954964007524}, {72, 0.0000674634696323547}, {
                    73, 0.0000820412324144472}, {
                    74, 0.00009594861934168996}, {
                    75, 0.00011136185459504488`}, {
                    76, 0.00013071075086409833`}, {
                    77, 0.00016094618403406774`}, {
                    78, 0.00020203501529720246`}, {
                    79, 0.0002484081950853924}, {80, 0.0002927796913361347}, {
                    81, 0.00033190795889119354`}, {
                    82, 0.00037483760022563166`}, {
                    83, 0.000441990107494212}, {84, 0.0005475902126215767}, {
                    85, 0.0006847814276824861}, {86, 0.0008231390791378697}, {
                    87, 0.0009358093207514065}, {
                    88, 0.0010224511692107133`}, {
                    89, 0.0011346930763279886`}, {
                    90, 0.0013574854783087106`}, {91, 0.001711758260331793}, {
                    92, 0.0021169962664522764`}, {
                    93, 0.0024565875977275043`}, {
                    94, 0.0026595633984730993`}, {
                    95, 0.0027946489095873185`}, {96, 0.003122221829525069}, {
                    97, 0.0038771431050995775`}, {98, 0.004938041067166719}, {
                    99, 0.005933654904485582}, {100, 0.006522960679212453}, {
                    101, 0.006646645721509049}, {102, 0.006815774826748525}, {
                    103, 0.007995616351049803}, {
                    104, 0.010401970352974136`}, {105, 0.0131112683669604}, {
                    106, 0.014996118300357308`}, {
                    107, 0.015383408230869281`}, {
                    108, 0.014707498975775442`}, {
                    109, 0.015279614176830825`}, {
                    110, 0.019522882649120124`}, {
                    111, 0.026195959185253765`}, {112, 0.03223282989301264}, {
                    113, 0.03512422024252793}, {114, 0.03382957236915019}, {
                    115, 0.030359564971308782`}, {116, 0.03191601408726447}, {
                    117, 0.043967561790244136`}, {118, 0.0614696303372614}, {
                    119, 0.07745165464862012}, {120, 0.08678105086415389}, {
                    121, 0.08631817638949765}, {122, 0.07606107119310992}, {
                    123, 0.06332279108865645}, {124, 0.06996213922705054}, {
                    125, 0.10830584212055046`}, {126, 0.16518187021499342`}, {
                    127, 0.23012602028306}, {128, 0.2973641949329308}, {
                    129, 0.36308371405726186`}, {130, 0.42465269752608714`}, {
                    131, 0.4803726447834743}, {132, 0.5293185168190553}, {
                    133, 0.5711653158390205}, {134, 0.6060251493048092}, {
                    135, 0.6342845937069549}, {136, 0.6564912589824866}, {
                    137, 0.6732482548101423}, {138, 0.6851609047002907}, {
                    139, 0.6927974799320984}, {140, 0.6966676555879119}, {
                    141, 0.6972205616176433}, {142, 0.6948459889541057}, {
                    143, 0.6898781833862501}, {144, 0.6826077883376961}, {
                    145, 0.6732861426825543}, {146, 0.6621378439744305}, {
                    147, 0.6493586675192086}, {148, 0.6351329283672433}, {
                    149, 0.619624248208783}, {150, 0.6029899872483413}, {
                    151, 0.5853752041782484}, {152, 0.5669185809560062}, {
                    153, 0.547755670582892}, {154, 0.5280112638957404}, {
                    155, 0.5078087505867818}, {156, 0.48726598742742233`}, {
                    157, 0.46649501622549405`}, {158, 0.4456039760172254}, {
                    159, 0.4246928445443121}, {160, 0.403859037576679}, {
                    161, 0.38319154208479095`}, {162, 0.3627745293670292}, {
                    163, 0.342681881856025}, {164, 0.32298571760164974`}, {
                    165, 0.3037490307176687}, {166, 0.28502721555113136`}, {
                    167, 0.26686868740425307`}, {168, 0.24931736548735584`}, {
                    169, 0.2324077319936288}, {170, 0.21616823838143148`}, {
                    171, 0.2006211696912877}, {172, 0.18578336826996716`}, {
                    173, 0.17166381484572382`}, {174, 0.15827036168535133`}, {
                    175, 0.14560119569910537`}, {176, 0.13365203258494107`}, {
                    177, 0.12241325831655088`}, {178, 0.11187428259503832`}, {
                    179, 0.10201751800398683`}, {180, 0.09282514563869629}, {
                    181, 0.08427549843078563}, {182, 0.07634530184867783}, {
                    183, 0.06900983716985953}, {184, 0.06224155456563654}, {
                    185, 0.0560127945042058}, {186, 0.05029729477061629}, {
                    187, 0.04506571145298718}, {188, 0.0402890136549176}, {
                    189, 0.035939385923790416`}, {
                    190, 0.031988294945593895`}, {
                    191, 0.028408497138459745`}, {
                    192, 0.025173877161639536`}, {
                    193, 0.022258328403307194`}, {194, 0.01963687681627681}, {
                    195, 0.017285679964406917`}, {
                    196, 0.015181673147389647`}, {
                    197, 0.013304571684119854`}, {198, 0.01163387118578884}, {
                    199, 0.010150172804532266`}, {200, 0.00883592106230871}, {
                    201, 0.007674116818949747}, {202, 0.006650366590519392}, {
                    203, 0.0057507066452891185`}, {
                    204, 0.004961468204804808}, {205, 0.004270926570672727}, {
                    206, 0.003668038868378991}, {207, 0.003143129406593967}, {
                    208, 0.0026877077089083115`}, {
                    209, 0.0022929361538310397`}, {
                    210, 0.0019509801712304187`}, {
                    211, 0.001656376827453113}, {
                    212, 0.0014038468977090168`}, {
                    213, 0.0011871230539388104`}, {
                    214, 0.001001101083255173}, {
                    215, 0.0008424619081592663}, {
                    216, 0.0007077217647467498}, {
                    217, 0.000592899530952019}, {
                    218, 0.0004953684037376003}, {
                    219, 0.00041337045860092794`}, {
                    220, 0.00034412820398229673`}, {
                    221, 0.00028544258860818965`}, {
                    222, 0.00023660607159289897`}, {
                    223, 0.00019592876995973903`}, {
                    224, 0.00016137920551830606`}, {
                    225, 0.00013268643497727102`}, {
                    226, 0.00010931356231395121`}, {
                    227, 0.00008948279035806813}, {
                    228, 0.00007264891849158014}, {
                    229, 0.00005931949564526404}, {
                    230, 0.00004874442622113992}, {
                    231, 0.00003960138217951687}, {
                    232, 0.00003178478022753637}, {
                    233, 0.00002566743430033447}, {
                    234, 0.00002077716461379871}, {
                    235, 0.000016463803053333834`}, {
                    236, 0.000012875728193564668`}, {
                    237, 0.000010309613392873033`}, {
                    238, 8.366117099388916*^-6}, {
                    239, 6.571072696292503*^-6}, {
                    240, 5.033348848483136*^-6}, {
                    241, 3.994919817546296*^-6}, {
                    242, 3.285267772814738*^-6}, {
                    243, 2.6209797736733804`*^-6}, {
                    244, 2.0340228396403652`*^-6}, {
                    245, 1.7022350329921032`*^-6}, {
                    246, 1.5138562678469977`*^-6}, {
                    247, 1.192198926530175*^-6}, {
                    248, 7.25560419081644*^-7}, {
                    249, 5.052093341425101*^-7}, {
                    250, 7.06364939206284*^-7}, {
                    251, 8.052640550483517*^-7}}],
                   Frame->True,
                   FrameTicks->{{{1., 
                    FormBox[
                    RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
                    FormBox[
                    RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
                    FormBox[
                    RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
                    FormBox[
                    RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
                    FormBox["0", TraditionalForm]}, {157.25, 
                    FormBox["2.5`", TraditionalForm]}, {188.5, 
                    FormBox["5", TraditionalForm]}, {219.75, 
                    FormBox["7.5`", TraditionalForm]}, {251., 
                    FormBox["10", TraditionalForm]}}, Automatic, None, None},
                   PlotRange->{-0.1, 1.1},
                   PlotRangeClipping->True,
                   PlotRangePadding->{
                    Scaled[0.02], Automatic}], 15->
                  GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF2HdcFHcaBvDBWNDIhYuKiCijIQQQUZoiGHxEEBAFVpYuMCwsZWlGgkaj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                    
                    "], {
                    {GrayLevel[0.5], EdgeForm[None], 
                    GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ
2qYUCakkqbSrJBXatNLyfM7n/eM55zkzc2fud+be9/dW7t6/Tb8cEX/DAZBg
P/wDGi8KxRg8EIrhhcKLUvL6worDC0ORgph/4b+CtRorwlzRVLAXFMdLaE+8
FByKHwYHh8dK4gdB8YKcMl4o+awSUBovAyXDlMUPUUyYcnj55L2bwkB8EDQL
n1EBPxxK42XgSPwoKBumEn609lRuUB0/TjmGqYEfDxXw8lAVr5a8VjHH4JWh
XPgbSunbknNXTE28lnLQ2VAfP0k54BWhLn4CHBHOsaI8OVfF1MbrJK9VTAP8
ZKgUjqmHn5i8t3I4Fq+SnKvWnIKfqm9UrtAYP00541XgDPxMqIpXg7Pxc6B6
eOwsvHnyXB24DG8LdXUWNMNPT95LMefi5+kOw7TAz4caeC24GL8Eaof3aIdf
rjsIj7XBL00+SzEt8Qv0BmFa4a2hZjimPd4B6oXpiHfSneCN4Fr8Omisu4Ce
eC/dSXhNZ/wKqB+mC95VbxTmSrwbNAjTHb9KbxC+04Z4o+S71VgP/Orks3TH
TfCmyXfVEPri1yTnphx6432S55RjP7y/3ijM9fgN0CTMAPzG5Fr/C3bRhLuz
e1o1P5i5IXoTvS0Mw2/WG+ttYBw+Xm+kt4UR+K16Y709jMJHqwbCMcPxW5LX
amwMPjZ5L+0xAb8dWuCt4W78HrgwPDYRn6QaCJ9xGz4y+ayWMAW/U28cZip+
F7QKMw2fnry3vukmfGjyt+mMGfi9cJFqC2bh96vGwszGH4A2eFt4FH8M2oV5
HH9CNajahrn4I8lrFTMHf1A1Gf6GyfgdybkrZh7+JLTHO8EifDF0xjvAAvxp
6BjOcSZ+X3KuipmPP5W8VjFL8KWqyXDMQvyZ5L01tgxfDl3CPIs/p5oN8zz+
Alyp2ofN+HvqAbwbvIS/DN3DrMRfUU2HeRVfBT3CrMZfU02H99iCvw/9wmNr
8NehJ94L1uNvQG+8D7yFvw19w2vW4uuS12psI/6OeiIcswF/MzlWY5vwd5O/
RT1/od46+W+JcvgA/1A9o9qA7/EfVCP4INiOfwOD1UvwMf6JeirMp/hnMCDM
5/gX6rEw2/AvYWD4jK34R8l7aewr/Ovks3TnL+Irku96COzEv0vOTTnswL9N
nlOOu/DdMCzMHvxH9WyYn/CfYXiYX/Bf1ZNh9uK/wYgwv+N/qKfxkbAf/wdG
4aNBP9oJxoTH/mXuv+S5SVCauTIwWb0KfzO3L3kvxWTmCsHYMAfghWEcfjuU
xA+CieE9yuKHwB3hsVL4wdlnKaYIXhTGhymGHwgTwjHl8PIwJcyh+GFwJ34v
1MCPh5n43XAsXgXuCa+pgB8OU8NUxI+Au8IciR8F08JUwo+G6eEciuMlsr9N
Y8fglbPPmg318ZPgAXwGVMePy85NOVTFq2XPKceaeC24L0xtvA7MClMXPwHu
D1MPPzH7LJ3RAD8Z5qjW1AN4f1iBz1d/4BfBU/iD0BBvBA+FaYyfBg+HaYI3
hblhmuGnwyPhPS7GL4EF4bEz8DPhUfwxOBs/Bx7Hn4AW+PkwL7zmLLx59lqN
tcQvgCfDMefi52XHaqwV3jr7W/RN1+M3wEvhHNrgl8LT+LPQE+8Fz+FL1YN4
N1iGL4R2+OXwTJj2eAdYFKYj3gkWh+mMXwFLwmdchrfN3ktjXfCu2WfpDU7B
T82+6+XQA786Ozfl0B2/KntOOfbG+8DzYfri18ALYa7Fr8t+W/3mPETvPZz8
26Q7GMDcjfAyvgPW4uvgW/wVuAkfCq/iq2A4fgusDo8Nw2/OnnsNbsNHwhp8
JQzGh2TvpZgR+K3Za7VmFD4aXsfXqUfUG7A+/DfnT3L9K/lvxxswWb0PG8JM
Ue/Cm+GYifik7LUam6rehLfwt9XT6mXYiH+gmlGtwIf4e7of1TJsCa+Zod6D
d8LMVK/BpjCz1FvwbpjZ6l3YHP6mMfhYWBsem6O3zT5LOU3Dp2fn9r5qXLWd
nZtymKveyZ5TjvNU67A1zHz1JnwUZoFqGT4OnzkOH599txpbqNqFT8IsUq3C
p/gXqiHVDmzDv1OOyg2+x79UzahW4Kvw2EbdDfwQjnkRX5G9VmObdDewC/9c
Pabeyj7ra9WQage2h2tuvd4OdobPWKnayV6rsQ166+zcFLNatQTfhFmjWsqu
ZdX0QHxQdi0qh826e9iN79UZ2ht+w/eoJlQL8CP+i3JWrvAr/pPuUHcHP4fv
bIl6Fz4Lx2zVW2Sv1Z7blRv8Hj5zi94y+yztuU13lZ2L1uxQ78EfUXCG9s7O
5U/due46+391nbkMX559t4rZqbvJXqv/4ffoW2Af/j8egu84
                    
                    "]]]}, {{}, {}, {
                    {Hue[0.05956624042050353, 1, 1], LineBox[{1, 2}]}, 
                    {Hue[0.9831017166735266, 1, 1], LineBox[{2, 3}]}, 
                    {Hue[0.053784012938536886`, 1, 1], LineBox[{3, 4}]}, 
                    {Hue[0.8965646702957913, 1, 1], LineBox[{4, 5}]}, 
                    {Hue[0.7133375980565537, 1, 1], LineBox[{5, 6}]}, 
                    {Hue[0.472416995417405, 1, 1], LineBox[{6, 7}]}, 
                    {Hue[0.22615072621804672`, 1, 1], LineBox[{7, 8}]}, 
                    {Hue[0.08421188501446454, 1, 1], LineBox[{8, 9}]}, 
                    {Hue[0.8829614102660753, 1, 1], LineBox[{9, 10}]}, 
                    {Hue[0.8222896416930823, 1, 1], LineBox[{10, 11}]}, 
                    {Hue[0.6400882201477065, 1, 1], LineBox[{11, 12}]}, 
                    {Hue[0.5078569467204729, 1, 1], LineBox[{12, 13}]}, 
                    {Hue[0.4582011452917198, 1, 1], LineBox[{13, 14}]}, 
                    {Hue[0.2860972878164505, 1, 1], LineBox[{14, 15}]}, 
                    {Hue[0.1269644106342963, 1, 1], LineBox[{15, 16}]}, 
                    {Hue[0.008489555056382105, 1, 1], LineBox[{16, 17}]}, 
                    {Hue[0.9047969445291322, 1, 1], LineBox[{17, 18}]}, 
                    {Hue[0.8069026689219323, 1, 1], LineBox[{18, 19}]}, 
                    {Hue[0.7433426798313757, 1, 1], LineBox[{19, 20}]}, 
                    {Hue[0.6211645129944474, 1, 1], LineBox[{20, 21}]}, 
                    {Hue[0.570574249490098, 1, 1], LineBox[{21, 22}]}, 
                    {Hue[0.37442686964880406`, 1, 1], LineBox[{22, 23}]}, 
                    {Hue[0.41406408292314084`, 1, 1], LineBox[{23, 24}]}, 
                    {Hue[0.27304668255560127`, 1, 1], LineBox[{24, 25}]}, 
                    {Hue[0.1640613943112119, 1, 1], LineBox[{25, 26}]}, 
                    {Hue[0.9217837241712366, 1, 1], LineBox[{26, 27}]}, 
                    {Hue[0.04927441714522789, 1, 1], LineBox[{27, 28}]}, 
                    {Hue[0.7395544576770468, 1, 1], LineBox[{28, 29}]}, 
                    {Hue[0.8366427329778339, 1, 1], LineBox[{29, 30}]}, 
                    {Hue[0.640224275610829, 1, 1], LineBox[{30, 31}]}, 
                    {Hue[0.5005906513158629, 1, 1], LineBox[{31, 32}]}, 
                    {Hue[0.39893509617051615`, 1, 1], LineBox[{32, 33}]}, 
                    {Hue[0.31827148243229864`, 1, 1], LineBox[{33, 34}]}, 
                    {Hue[0.10562517832932644`, 1, 1], LineBox[{34, 35}]}, 
                    {Hue[0.05647944373999831, 1, 1], LineBox[{35, 36}]}, 
                    {Hue[0.954476650596741, 1, 1], LineBox[{36, 37}]}, 
                    {Hue[0.8121914267182476, 1, 1], LineBox[{37, 38}]}, 
                    {Hue[0.6843275724593121, 1, 1], LineBox[{38, 39}]}, 
                    {Hue[0.8357639889934845, 1, 1], LineBox[{39, 40}]}, 
                    {Hue[0.6544318649455261, 1, 1], LineBox[{40, 41}]}, 
                    {Hue[0.571673756180975, 1, 1], LineBox[{41, 42}]}, 
                    {Hue[0.5103863038769656, 1, 1], LineBox[{42, 43}]}, 
                    {Hue[0.38946345539840677`, 1, 1], LineBox[{43, 44}]}, 
                    {Hue[0.3407740913415771, 1, 1], LineBox[{44, 45}]}, 
                    {Hue[0.2881490527713575, 1, 1], LineBox[{45, 46}]}, 
                    {Hue[0.14006130480093046`, 1, 1], LineBox[{46, 47}]}, 
                    {Hue[0.03270695109683777, 1, 1], LineBox[{47, 48}]}, 
                    {Hue[0.9222580029127315, 1, 1], LineBox[{48, 49}]}, 
                    {Hue[0.8216153254234894, 1, 1], LineBox[{49, 50}]}, 
                    {Hue[0.7327219348514502, 1, 1], LineBox[{50, 51}]}, 
                    {Hue[0.6329691624014069, 1, 1], LineBox[{51, 52}]}, 
                    {Hue[0.5441018001510657, 1, 1], LineBox[{52, 53}]}, 
                    {Hue[0.451179412264641, 1, 1], LineBox[{53, 54}]}, 
                    {Hue[0.34884452009062944`, 1, 1], LineBox[{54, 55}]}, 
                    {Hue[0.24649878947472764`, 1, 1], LineBox[{55, 56}]}, 
                    {Hue[0.15143639484703394`, 1, 1], LineBox[{56, 57}]}, 
                    {Hue[0.050421982901845644`, 1, 1], LineBox[{57, 58}]}, 
                    {Hue[0.9481497781591328, 1, 1], LineBox[{58, 59}]}, 
                    {Hue[0.8502040510520766, 1, 1], LineBox[{59, 60}]}, 
                    {Hue[0.7515057307791237, 1, 1], LineBox[{60, 61}]}, 
                    {Hue[0.6545229892266976, 1, 1], LineBox[{61, 62}]}, 
                    {Hue[0.556780119053588, 1, 1], LineBox[{62, 63}]}, 
                    {Hue[0.45912955175820863`, 1, 1], LineBox[{63, 64}]}, 
                    {Hue[0.36229952563725926`, 1, 1], LineBox[{64, 65}]}, 
                    {Hue[0.2647079518516234, 1, 1], LineBox[{65, 66}]}, 
                    {Hue[0.16676307041787705`, 1, 1], LineBox[{66, 67}]}, 
                    {Hue[0.06850347252601507, 1, 1], LineBox[{67, 68}]}, 
                    {Hue[0.9717018103355191, 1, 1], LineBox[{68, 69}]}, 
                    {Hue[0.8748549989781255, 1, 1], LineBox[{69, 70}]}, 
                    {Hue[0.7789328517480816, 1, 1], LineBox[{70, 71}]}, 
                    {Hue[0.6833400310612193, 1, 1], LineBox[{71, 72}]}, 
                    {Hue[0.587582947708543, 1, 1], LineBox[{72, 73}]}, 
                    {Hue[0.49205787070990953`, 1, 1], LineBox[{73, 74}]}, 
                    {Hue[0.3962264733104969, 1, 1], LineBox[{74, 75}]}, 
                    {Hue[0.30073678619523775`, 1, 1], LineBox[{75, 76}]}, 
                    {Hue[0.20606099121402044`, 1, 1], LineBox[{76, 77}]}, 
                    {Hue[0.11180750743429678`, 1, 1], LineBox[{77, 78}]}, 
                    {Hue[0.01786908293360081, 1, 1], LineBox[{78, 79}]}, 
                    {Hue[0.9238158665680171, 1, 1], LineBox[{79, 80}]}, 
                    {Hue[0.8295682733677235, 1, 1], LineBox[{80, 81}]}, 
                    {Hue[0.735411875619816, 1, 1], LineBox[{81, 82}]}, 
                    {Hue[0.6416165761393002, 1, 1], LineBox[{82, 83}]}, 
                    {Hue[0.5485735773427743, 1, 1], LineBox[{83, 84}]}, 
                    {Hue[0.4562559025427074, 1, 1], LineBox[{84, 85}]}, 
                    {Hue[0.3641662230314104, 1, 1], LineBox[{85, 86}]}, 
                    {Hue[0.2717745783461878, 1, 1], LineBox[{86, 87}]}, 
                    {Hue[0.17887247985716348`, 1, 1], LineBox[{87, 88}]}, 
                    {Hue[0.08588874690629737, 1, 1], LineBox[{88, 89}]}, 
                    {Hue[0.9936406858076294, 1, 1], LineBox[{89, 90}]}, 
                    {Hue[0.9025475885767389, 1, 1], LineBox[{90, 91}]}, 
                    {Hue[0.8123557106102066, 1, 1], LineBox[{91, 92}]}, 
                    {Hue[0.722196790536449, 1, 1], LineBox[{92, 93}]}, 
                    {Hue[0.63124672944796, 1, 1], LineBox[{93, 94}]}, 
                    {Hue[0.5394334217154759, 1, 1], LineBox[{94, 95}]}, 
                    {Hue[0.4476048393319134, 1, 1], LineBox[{95, 96}]}, 
                    {Hue[0.35699908783455947`, 1, 1], LineBox[{96, 97}]}, 
                    {Hue[0.2681214791428311, 1, 1], LineBox[{97, 98}]}, 
                    {Hue[0.18030782830551154`, 1, 1], LineBox[{98, 99}]}, 
                    {Hue[0.09215150663418024, 1, 1], LineBox[{99, 100}]}, 
                    {Hue[0.0025077892689761004`, 1, 1], 
                    LineBox[{100, 101}]}, 
                    {Hue[0.9115947297733166, 1, 1], LineBox[{101, 102}]}, 
                    {Hue[0.8210927146819142, 1, 1], LineBox[{102, 103}]}, 
                    {Hue[0.732829693010617, 1, 1], LineBox[{103, 104}]}, 
                    {Hue[0.647133972193977, 1, 1], LineBox[{104, 105}]}, 
                    {Hue[0.5625815606607734, 1, 1], LineBox[{105, 106}]}, 
                    {Hue[0.47697251741405106`, 1, 1], LineBox[{106, 107}]}, 
                    {Hue[0.38892812303980323`, 1, 1], LineBox[{107, 108}]}, 
                    {Hue[0.29936197072723353`, 1, 1], LineBox[{108, 109}]}, 
                    {Hue[0.2112842587929123, 1, 1], LineBox[{109, 110}]}, 
                    {Hue[0.12719223182875483`, 1, 1], LineBox[{110, 111}]}, 
                    {Hue[0.04695078965265709, 1, 1], LineBox[{111, 112}]}, 
                    {Hue[0.9681574476610211, 1, 1], LineBox[{112, 113}]}, 
                    {Hue[0.887706749483258, 1, 1], LineBox[{113, 114}]}, 
                    {Hue[0.8037701829451505, 1, 1], LineBox[{114, 115}]}, 
                    {Hue[0.7178930970056254, 1, 1], LineBox[{115, 116}]}, 
                    {Hue[0.6347073831219328, 1, 1], LineBox[{116, 117}]}, 
                    {Hue[0.5579546515943477, 1, 1], LineBox[{117, 118}]}, 
                    {Hue[0.48772321475245084`, 1, 1], LineBox[{118, 119}]}, 
                    {Hue[0.42141514090553406`, 1, 1], LineBox[{119, 120}]}, 
                    {Hue[0.3554913039774097, 1, 1], LineBox[{120, 121}]}, 
                    {Hue[0.28669414064235854`, 1, 1], LineBox[{121, 122}]}, 
                    {Hue[0.21367847985739843`, 1, 1], LineBox[{122, 123}]}, 
                    {Hue[0.13916554587353325`, 1, 1], LineBox[{123, 124}]}, 
                    {Hue[0.06927835345538431, 1, 1], LineBox[{124, 125}]}, 
                    {Hue[0.008753732053659424, 1, 1], LineBox[{125, 126}]}, 
                    {Hue[0.958384426153491, 1, 1], LineBox[{126, 127}]}, 
                    {Hue[0.9166770183971185, 1, 1], LineBox[{127, 128}]}, 
                    {Hue[0.8817915576244195, 1, 1], LineBox[{128, 129}]}, 
                    {Hue[0.8522672078087553, 1, 1], LineBox[{129, 130}]}, 
                    {Hue[0.8270975424776865, 1, 1], LineBox[{130, 131}]}, 
                    {Hue[0.8056284375105165, 1, 1], LineBox[{131, 132}]}, 
                    {Hue[0.7874454690796864, 1, 1], LineBox[{132, 133}]}, 
                    {Hue[0.7722914136674517, 1, 1], LineBox[{133, 134}]}, 
                    {Hue[0.7600076628418265, 1, 1], LineBox[{134, 135}]}, 
                    {Hue[0.750496805403843, 1, 1], LineBox[{135, 136}]}, 
                    {Hue[0.7436936708530469, 1, 1], LineBox[{136, 137}]}, 
                    {Hue[0.7395522002789043, 1, 1], LineBox[{137, 138}]}, 
                    {Hue[0.7380306364234124, 1, 1], LineBox[{138, 139}]}, 
                    {Hue[0.739088410762426, 1, 1], LineBox[{139, 140}]}, 
                    {Hue[0.7426813910790278, 1, 1], LineBox[{140, 141}]}, 
                    {Hue[0.7487603009809475, 1, 1], LineBox[{141, 142}]}, 
                    {Hue[0.7572706370390397, 1, 1], LineBox[{142, 143}]}, 
                    {Hue[0.7681537835764389, 1, 1], LineBox[{143, 144}]}, 
                    {Hue[0.7813485124592214, 1, 1], LineBox[{144, 145}]}, 
                    {Hue[0.7967903032737629, 1, 1], LineBox[{145, 146}]}, 
                    {Hue[0.8144135638472767, 1, 1], LineBox[{146, 147}]}, 
                    {Hue[0.8341527576324702, 1, 1], LineBox[{147, 148}]}, 
                    {Hue[0.8559423552343227, 1, 1], LineBox[{148, 149}]}, 
                    {Hue[0.8797184779594373, 1, 1], LineBox[{149, 150}]}, 
                    {Hue[0.9054178672267975, 1, 1], LineBox[{150, 151}]}, 
                    {Hue[0.9329806227146357, 1, 1], LineBox[{151, 152}]}, 
                    {Hue[0.9623475758499133, 1, 1], LineBox[{152, 153}]}, 
                    {Hue[0.9934629887045462, 1, 1], LineBox[{153, 154}]}, 
                    {Hue[0.02627405855083121, 1, 1], LineBox[{154, 155}]}, 
                    {Hue[0.06072941346367099, 1, 1], LineBox[{155, 156}]}, 
                    {Hue[0.09678062203177676, 1, 1], LineBox[{156, 157}]}, 
                    {Hue[0.1343824932010361, 1, 1], LineBox[{157, 158}]}, 
                    {Hue[0.17349119558818452`, 1, 1], LineBox[{158, 159}]}, 
                    {Hue[0.21406617571740993`, 1, 1], LineBox[{159, 160}]}, 
                    {Hue[0.2560679976886549, 1, 1], LineBox[{160, 161}]}, 
                    {Hue[0.299460457403954, 1, 1], LineBox[{161, 162}]}, 
                    {Hue[0.3442092278042504, 1, 1], LineBox[{162, 163}]}, 
                    {Hue[0.3902814284807919, 1, 1], LineBox[{163, 164}]}, 
                    {Hue[0.4376464529812861, 1, 1], LineBox[{164, 165}]}, 
                    {Hue[0.486275510377512, 1, 1], LineBox[{165, 166}]}, 
                    {Hue[0.5361404086810144, 1, 1], LineBox[{166, 167}]}, 
                    {Hue[0.5872156253981717, 1, 1], LineBox[{167, 168}]}, 
                    {Hue[0.63947675730222, 1, 1], LineBox[{168, 169}]}, 
                    {Hue[0.6928994274576521, 1, 1], LineBox[{169, 170}]}, 
                    {Hue[0.7474627961633682, 1, 1], LineBox[{170, 171}]}, 
                    {Hue[0.8031448831692406, 1, 1], LineBox[{171, 172}]}, 
                    {Hue[0.8599277071610487, 1, 1], LineBox[{172, 173}]}, 
                    {Hue[0.9177905921948738, 1, 1], LineBox[{173, 174}]}, 
                    {Hue[0.9767162994358801, 1, 1], LineBox[{174, 175}]}, 
                    {Hue[0.03668769482659254, 1, 1], LineBox[{175, 176}]}, 
                    {Hue[0.09768870176319887, 1, 1], LineBox[{176, 177}]}, 
                    {Hue[0.15970390592623193`, 1, 1], LineBox[{177, 178}]}, 
                    {Hue[0.22271848133218072`, 1, 1], LineBox[{178, 179}]}, 
                    {Hue[0.2867176866955756, 1, 1], LineBox[{179, 180}]}, 
                    {Hue[0.3516896132531366, 1, 1], LineBox[{180, 181}]}, 
                    {Hue[0.41762027877249996`, 1, 1], LineBox[{181, 182}]}, 
                    {Hue[0.4844980226954021, 1, 1], LineBox[{182, 183}]}, 
                    {Hue[0.5523092351240234, 1, 1], LineBox[{183, 184}]}, 
                    {Hue[0.621046141242471, 1, 1], LineBox[{184, 185}]}, 
                    {Hue[0.6906954003728225, 1, 1], LineBox[{185, 186}]}, 
                    {Hue[0.7612464295655417, 1, 1], LineBox[{186, 187}]}, 
                    {Hue[0.8326892519615281, 1, 1], LineBox[{187, 188}]}, 
                    {Hue[0.9050154457110178, 1, 1], LineBox[{188, 189}]}, 
                    {Hue[0.9782137976593589, 1, 1], LineBox[{189, 190}]}, 
                    {Hue[0.05227788399332845, 1, 1], LineBox[{190, 191}]}, 
                    {Hue[0.1271986005150233, 1, 1], LineBox[{191, 192}]}, 
                    {Hue[0.2029642440043483, 1, 1], LineBox[{192, 193}]}, 
                    {Hue[0.27956900942770624`, 1, 1], LineBox[{193, 194}]}, 
                    {Hue[0.3570070126386154, 1, 1], LineBox[{194, 195}]}, 
                    {Hue[0.43526876191562286`, 1, 1], LineBox[{195, 196}]}, 
                    {Hue[0.5143483531822721, 1, 1], LineBox[{196, 197}]}, 
                    {Hue[0.5942364871856414, 1, 1], LineBox[{197, 198}]}, 
                    {Hue[0.6749261104680981, 1, 1], LineBox[{198, 199}]}, 
                    {Hue[0.7564165186521847, 1, 1], LineBox[{199, 200}]}, 
                    {Hue[0.8386941839035054, 1, 1], LineBox[{200, 201}]}, 
                    {Hue[0.9217502032467062, 1, 1], LineBox[{201, 202}]}, 
                    {Hue[0.005588047415432818, 1, 1], LineBox[{202, 203}]}, 
                    {Hue[0.09020190054450962, 1, 1], LineBox[{203, 204}]}, 
                    {Hue[0.17557830629088042`, 1, 1], LineBox[{204, 205}]}, 
                    {Hue[0.2617155460289287, 1, 1], LineBox[{205, 206}]}, 
                    {Hue[0.3486030056287242, 1, 1], LineBox[{206, 207}]}, 
                    {Hue[0.43623852605324986`, 1, 1], LineBox[{207, 208}]}, 
                    {Hue[0.5246243796516621, 1, 1], LineBox[{208, 209}]}, 
                    {Hue[0.6137467720443291, 1, 1], LineBox[{209, 210}]}, 
                    {Hue[0.7036113582715795, 1, 1], LineBox[{210, 211}]}, 
                    {Hue[0.7942020535195174, 1, 1], LineBox[{211, 212}]}, 
                    {Hue[0.8855024036439301, 1, 1], LineBox[{212, 213}]}, 
                    {Hue[0.9775292546983889, 1, 1], LineBox[{213, 214}]}, 
                    {Hue[0.07027539516154879, 1, 1], LineBox[{214, 215}]}, 
                    {Hue[0.16374124482826072`, 1, 1], LineBox[{215, 216}]}, 
                    {Hue[0.25791700557200636`, 1, 1], LineBox[{216, 217}]}, 
                    {Hue[0.3527640228073858, 1, 1], LineBox[{217, 218}]}, 
                    {Hue[0.4482987662581584, 1, 1], LineBox[{218, 219}]}, 
                    {Hue[0.5445531907493291, 1, 1], LineBox[{219, 220}]}, 
                    {Hue[0.6415090340163123, 1, 1], LineBox[{220, 221}]}, 
                    {Hue[0.739158911610914, 1, 1], LineBox[{221, 222}]}, 
                    {Hue[0.8374834060735057, 1, 1], LineBox[{222, 223}]}, 
                    {Hue[0.9364240543729676, 1, 1], LineBox[{223, 224}]}, 
                    {Hue[0.03603829961410431, 1, 1], LineBox[{224, 225}]}, 
                    {Hue[0.13642177385132023`, 1, 1], LineBox[{225, 226}]}, 
                    {Hue[0.23743312162973995`, 1, 1], LineBox[{226, 227}]}, 
                    {Hue[0.33890501166499326`, 1, 1], LineBox[{227, 228}]}, 
                    {Hue[0.4410449979957778, 1, 1], LineBox[{228, 229}]}, 
                    {Hue[0.5441512204674002, 1, 1], LineBox[{229, 230}]}, 
                    {Hue[0.6479683063190227, 1, 1], LineBox[{230, 231}]}, 
                    {Hue[0.7520934886159469, 1, 1], LineBox[{231, 232}]}, 
                    {Hue[0.8569136201029212, 1, 1], LineBox[{232, 233}]}, 
                    {Hue[0.9627047539340904, 1, 1], LineBox[{233, 234}]}, 
                    {Hue[0.06879089341509073, 1, 1], LineBox[{234, 235}]}, 
                    {Hue[0.17533461697758274`, 1, 1], LineBox[{235, 236}]}, 
                    {Hue[0.28319618900543175`, 1, 1], LineBox[{236, 237}]}, 
                    {Hue[0.39149295315047145`, 1, 1], LineBox[{237, 238}]}, 
                    {Hue[0.4996425226407696, 1, 1], LineBox[{238, 239}]}, 
                    {Hue[0.6094061045208964, 1, 1], LineBox[{239, 240}]}, 
                    {Hue[0.7205987326917802, 1, 1], LineBox[{240, 241}]}, 
                    {Hue[0.8307909058918155, 1, 1], LineBox[{241, 242}]}, 
                    {Hue[0.9413621288318382, 1, 1], LineBox[{242, 243}]}, 
                    {Hue[0.055154985274585344`, 1, 1], LineBox[{243, 244}]}, 
                    {Hue[0.16908344480096296`, 1, 1], LineBox[{244, 245}]}, 
                    {Hue[0.27949834648938626`, 1, 1], LineBox[{245, 246}]}, 
                    {Hue[0.39178904539044196`, 1, 1], LineBox[{246, 247}]}, 
                    {Hue[0.5113599233497237, 1, 1], LineBox[{247, 248}]}, 
                    {Hue[0.6321214530795161, 1, 1], LineBox[{248, 249}]}, 
                    {Hue[0.7464805255537773, 1, 1], LineBox[{249, 250}]}, 
                    {Hue[0.8602620296701579, 1, 1], LineBox[{250, 251}]}}}},
                    VertexColors->{
                    Hue[0.05956624042050353, 1, 1], 
                    Hue[0.9831017166735266, 1, 1], 
                    Hue[0.053784012938536886`, 1, 1], 
                    Hue[0.8965646702957913, 1, 1], 
                    Hue[0.7133375980565537, 1, 1], 
                    Hue[0.472416995417405, 1, 1], 
                    Hue[0.22615072621804672`, 1, 1], 
                    Hue[0.08421188501446454, 1, 1], 
                    Hue[0.8829614102660753, 1, 1], 
                    Hue[0.8222896416930823, 1, 1], 
                    Hue[0.6400882201477065, 1, 1], 
                    Hue[0.5078569467204729, 1, 1], 
                    Hue[0.4582011452917198, 1, 1], 
                    Hue[0.2860972878164505, 1, 1], 
                    Hue[0.1269644106342963, 1, 1], 
                    Hue[0.008489555056382105, 1, 1], 
                    Hue[0.9047969445291322, 1, 1], 
                    Hue[0.8069026689219323, 1, 1], 
                    Hue[0.7433426798313757, 1, 1], 
                    Hue[0.6211645129944474, 1, 1], 
                    Hue[0.570574249490098, 1, 1], 
                    Hue[0.37442686964880406`, 1, 1], 
                    Hue[0.41406408292314084`, 1, 1], 
                    Hue[0.27304668255560127`, 1, 1], 
                    Hue[0.1640613943112119, 1, 1], 
                    Hue[0.9217837241712366, 1, 1], 
                    Hue[0.04927441714522789, 1, 1], 
                    Hue[0.7395544576770468, 1, 1], 
                    Hue[0.8366427329778339, 1, 1], 
                    Hue[0.640224275610829, 1, 1], 
                    Hue[0.5005906513158629, 1, 1], 
                    Hue[0.39893509617051615`, 1, 1], 
                    Hue[0.31827148243229864`, 1, 1], 
                    Hue[0.10562517832932644`, 1, 1], 
                    Hue[0.05647944373999831, 1, 1], 
                    Hue[0.954476650596741, 1, 1], 
                    Hue[0.8121914267182476, 1, 1], 
                    Hue[0.6843275724593121, 1, 1], 
                    Hue[0.8357639889934845, 1, 1], 
                    Hue[0.6544318649455261, 1, 1], 
                    Hue[0.571673756180975, 1, 1], 
                    Hue[0.5103863038769656, 1, 1], 
                    Hue[0.38946345539840677`, 1, 1], 
                    Hue[0.3407740913415771, 1, 1], 
                    Hue[0.2881490527713575, 1, 1], 
                    Hue[0.14006130480093046`, 1, 1], 
                    Hue[0.03270695109683777, 1, 1], 
                    Hue[0.9222580029127315, 1, 1], 
                    Hue[0.8216153254234894, 1, 1], 
                    Hue[0.7327219348514502, 1, 1], 
                    Hue[0.6329691624014069, 1, 1], 
                    Hue[0.5441018001510657, 1, 1], 
                    Hue[0.451179412264641, 1, 1], 
                    Hue[0.34884452009062944`, 1, 1], 
                    Hue[0.24649878947472764`, 1, 1], 
                    Hue[0.15143639484703394`, 1, 1], 
                    Hue[0.050421982901845644`, 1, 1], 
                    Hue[0.9481497781591328, 1, 1], 
                    Hue[0.8502040510520766, 1, 1], 
                    Hue[0.7515057307791237, 1, 1], 
                    Hue[0.6545229892266976, 1, 1], 
                    Hue[0.556780119053588, 1, 1], 
                    Hue[0.45912955175820863`, 1, 1], 
                    Hue[0.36229952563725926`, 1, 1], 
                    Hue[0.2647079518516234, 1, 1], 
                    Hue[0.16676307041787705`, 1, 1], 
                    Hue[0.06850347252601507, 1, 1], 
                    Hue[0.9717018103355191, 1, 1], 
                    Hue[0.8748549989781255, 1, 1], 
                    Hue[0.7789328517480816, 1, 1], 
                    Hue[0.6833400310612193, 1, 1], 
                    Hue[0.587582947708543, 1, 1], 
                    Hue[0.49205787070990953`, 1, 1], 
                    Hue[0.3962264733104969, 1, 1], 
                    Hue[0.30073678619523775`, 1, 1], 
                    Hue[0.20606099121402044`, 1, 1], 
                    Hue[0.11180750743429678`, 1, 1], 
                    Hue[0.01786908293360081, 1, 1], 
                    Hue[0.9238158665680171, 1, 1], 
                    Hue[0.8295682733677235, 1, 1], 
                    Hue[0.735411875619816, 1, 1], 
                    Hue[0.6416165761393002, 1, 1], 
                    Hue[0.5485735773427743, 1, 1], 
                    Hue[0.4562559025427074, 1, 1], 
                    Hue[0.3641662230314104, 1, 1], 
                    Hue[0.2717745783461878, 1, 1], 
                    Hue[0.17887247985716348`, 1, 1], 
                    Hue[0.08588874690629737, 1, 1], 
                    Hue[0.9936406858076294, 1, 1], 
                    Hue[0.9025475885767389, 1, 1], 
                    Hue[0.8123557106102066, 1, 1], 
                    Hue[0.722196790536449, 1, 1], 
                    Hue[0.63124672944796, 1, 1], 
                    Hue[0.5394334217154759, 1, 1], 
                    Hue[0.4476048393319134, 1, 1], 
                    Hue[0.35699908783455947`, 1, 1], 
                    Hue[0.2681214791428311, 1, 1], 
                    Hue[0.18030782830551154`, 1, 1], 
                    Hue[0.09215150663418024, 1, 1], 
                    Hue[0.0025077892689761004`, 1, 1], 
                    Hue[0.9115947297733166, 1, 1], 
                    Hue[0.8210927146819142, 1, 1], 
                    Hue[0.732829693010617, 1, 1], 
                    Hue[0.647133972193977, 1, 1], 
                    Hue[0.5625815606607734, 1, 1], 
                    Hue[0.47697251741405106`, 1, 1], 
                    Hue[0.38892812303980323`, 1, 1], 
                    Hue[0.29936197072723353`, 1, 1], 
                    Hue[0.2112842587929123, 1, 1], 
                    Hue[0.12719223182875483`, 1, 1], 
                    Hue[0.04695078965265709, 1, 1], 
                    Hue[0.9681574476610211, 1, 1], 
                    Hue[0.887706749483258, 1, 1], 
                    Hue[0.8037701829451505, 1, 1], 
                    Hue[0.7178930970056254, 1, 1], 
                    Hue[0.6347073831219328, 1, 1], 
                    Hue[0.5579546515943477, 1, 1], 
                    Hue[0.48772321475245084`, 1, 1], 
                    Hue[0.42141514090553406`, 1, 1], 
                    Hue[0.3554913039774097, 1, 1], 
                    Hue[0.28669414064235854`, 1, 1], 
                    Hue[0.21367847985739843`, 1, 1], 
                    Hue[0.13916554587353325`, 1, 1], 
                    Hue[0.06927835345538431, 1, 1], 
                    Hue[0.008753732053659424, 1, 1], 
                    Hue[0.958384426153491, 1, 1], 
                    Hue[0.9166770183971185, 1, 1], 
                    Hue[0.8817915576244195, 1, 1], 
                    Hue[0.8522672078087553, 1, 1], 
                    Hue[0.8270975424776865, 1, 1], 
                    Hue[0.8056284375105165, 1, 1], 
                    Hue[0.7874454690796864, 1, 1], 
                    Hue[0.7722914136674517, 1, 1], 
                    Hue[0.7600076628418265, 1, 1], 
                    Hue[0.750496805403843, 1, 1], 
                    Hue[0.7436936708530469, 1, 1], 
                    Hue[0.7395522002789043, 1, 1], 
                    Hue[0.7380306364234124, 1, 1], 
                    Hue[0.739088410762426, 1, 1], 
                    Hue[0.7426813910790278, 1, 1], 
                    Hue[0.7487603009809475, 1, 1], 
                    Hue[0.7572706370390397, 1, 1], 
                    Hue[0.7681537835764389, 1, 1], 
                    Hue[0.7813485124592214, 1, 1], 
                    Hue[0.7967903032737629, 1, 1], 
                    Hue[0.8144135638472767, 1, 1], 
                    Hue[0.8341527576324702, 1, 1], 
                    Hue[0.8559423552343227, 1, 1], 
                    Hue[0.8797184779594373, 1, 1], 
                    Hue[0.9054178672267975, 1, 1], 
                    Hue[0.9329806227146357, 1, 1], 
                    Hue[0.9623475758499133, 1, 1], 
                    Hue[0.9934629887045462, 1, 1], 
                    Hue[0.02627405855083121, 1, 1], 
                    Hue[0.06072941346367099, 1, 1], 
                    Hue[0.09678062203177676, 1, 1], 
                    Hue[0.1343824932010361, 1, 1], 
                    Hue[0.17349119558818452`, 1, 1], 
                    Hue[0.21406617571740993`, 1, 1], 
                    Hue[0.2560679976886549, 1, 1], 
                    Hue[0.299460457403954, 1, 1], 
                    Hue[0.3442092278042504, 1, 1], 
                    Hue[0.3902814284807919, 1, 1], 
                    Hue[0.4376464529812861, 1, 1], 
                    Hue[0.486275510377512, 1, 1], 
                    Hue[0.5361404086810144, 1, 1], 
                    Hue[0.5872156253981717, 1, 1], 
                    Hue[0.63947675730222, 1, 1], 
                    Hue[0.6928994274576521, 1, 1], 
                    Hue[0.7474627961633682, 1, 1], 
                    Hue[0.8031448831692406, 1, 1], 
                    Hue[0.8599277071610487, 1, 1], 
                    Hue[0.9177905921948738, 1, 1], 
                    Hue[0.9767162994358801, 1, 1], 
                    Hue[0.03668769482659254, 1, 1], 
                    Hue[0.09768870176319887, 1, 1], 
                    Hue[0.15970390592623193`, 1, 1], 
                    Hue[0.22271848133218072`, 1, 1], 
                    Hue[0.2867176866955756, 1, 1], 
                    Hue[0.3516896132531366, 1, 1], 
                    Hue[0.41762027877249996`, 1, 1], 
                    Hue[0.4844980226954021, 1, 1], 
                    Hue[0.5523092351240234, 1, 1], 
                    Hue[0.621046141242471, 1, 1], 
                    Hue[0.6906954003728225, 1, 1], 
                    Hue[0.7612464295655417, 1, 1], 
                    Hue[0.8326892519615281, 1, 1], 
                    Hue[0.9050154457110178, 1, 1], 
                    Hue[0.9782137976593589, 1, 1], 
                    Hue[0.05227788399332845, 1, 1], 
                    Hue[0.1271986005150233, 1, 1], 
                    Hue[0.2029642440043483, 1, 1], 
                    Hue[0.27956900942770624`, 1, 1], 
                    Hue[0.3570070126386154, 1, 1], 
                    Hue[0.43526876191562286`, 1, 1], 
                    Hue[0.5143483531822721, 1, 1], 
                    Hue[0.5942364871856414, 1, 1], 
                    Hue[0.6749261104680981, 1, 1], 
                    Hue[0.7564165186521847, 1, 1], 
                    Hue[0.8386941839035054, 1, 1], 
                    Hue[0.9217502032467062, 1, 1], 
                    Hue[0.005588047415432818, 1, 1], 
                    Hue[0.09020190054450962, 1, 1], 
                    Hue[0.17557830629088042`, 1, 1], 
                    Hue[0.2617155460289287, 1, 1], 
                    Hue[0.3486030056287242, 1, 1], 
                    Hue[0.43623852605324986`, 1, 1], 
                    Hue[0.5246243796516621, 1, 1], 
                    Hue[0.6137467720443291, 1, 1], 
                    Hue[0.7036113582715795, 1, 1], 
                    Hue[0.7942020535195174, 1, 1], 
                    Hue[0.8855024036439301, 1, 1], 
                    Hue[0.9775292546983889, 1, 1], 
                    Hue[0.07027539516154879, 1, 1], 
                    Hue[0.16374124482826072`, 1, 1], 
                    Hue[0.25791700557200636`, 1, 1], 
                    Hue[0.3527640228073858, 1, 1], 
                    Hue[0.4482987662581584, 1, 1], 
                    Hue[0.5445531907493291, 1, 1], 
                    Hue[0.6415090340163123, 1, 1], 
                    Hue[0.739158911610914, 1, 1], 
                    Hue[0.8374834060735057, 1, 1], 
                    Hue[0.9364240543729676, 1, 1], 
                    Hue[0.03603829961410431, 1, 1], 
                    Hue[0.13642177385132023`, 1, 1], 
                    Hue[0.23743312162973995`, 1, 1], 
                    Hue[0.33890501166499326`, 1, 1], 
                    Hue[0.4410449979957778, 1, 1], 
                    Hue[0.5441512204674002, 1, 1], 
                    Hue[0.6479683063190227, 1, 1], 
                    Hue[0.7520934886159469, 1, 1], 
                    Hue[0.8569136201029212, 1, 1], 
                    Hue[0.9627047539340904, 1, 1], 
                    Hue[0.06879089341509073, 1, 1], 
                    Hue[0.17533461697758274`, 1, 1], 
                    Hue[0.28319618900543175`, 1, 1], 
                    Hue[0.39149295315047145`, 1, 1], 
                    Hue[0.4996425226407696, 1, 1], 
                    Hue[0.6094061045208964, 1, 1], 
                    Hue[0.7205987326917802, 1, 1], 
                    Hue[0.8307909058918155, 1, 1], 
                    Hue[0.9413621288318382, 1, 1], 
                    Hue[0.055154985274585344`, 1, 1], 
                    Hue[0.16908344480096296`, 1, 1], 
                    Hue[0.27949834648938626`, 1, 1], 
                    Hue[0.39178904539044196`, 1, 1], 
                    Hue[0.5113599233497237, 1, 1], 
                    Hue[0.6321214530795161, 1, 1], 
                    Hue[0.7464805255537773, 1, 1], 
                    Hue[0.8602620296701579, 1, 1], 
                    Hue[0.9811501935614256, 1, 1], 
                    Hue[0.05956624042050353, 1, 1], 
                    Hue[0.9831017166735266, 1, 1], 
                    Hue[0.9831017166735266, 1, 1], 
                    Hue[0.053784012938536886`, 1, 1], 
                    Hue[0.053784012938536886`, 1, 1], 
                    Hue[0.8965646702957913, 1, 1], 
                    Hue[0.8965646702957913, 1, 1], 
                    Hue[0.7133375980565537, 1, 1], 
                    Hue[0.7133375980565537, 1, 1], 
                    Hue[0.472416995417405, 1, 1], 
                    Hue[0.472416995417405, 1, 1], 
                    Hue[0.22615072621804672`, 1, 1], 
                    Hue[0.22615072621804672`, 1, 1], 
                    Hue[0.08421188501446454, 1, 1], 
                    Hue[0.08421188501446454, 1, 1], 
                    Hue[0.8829614102660753, 1, 1], 
                    Hue[0.8829614102660753, 1, 1], 
                    Hue[0.8222896416930823, 1, 1], 
                    Hue[0.8222896416930823, 1, 1], 
                    Hue[0.6400882201477065, 1, 1], 
                    Hue[0.6400882201477065, 1, 1], 
                    Hue[0.5078569467204729, 1, 1], 
                    Hue[0.5078569467204729, 1, 1], 
                    Hue[0.4582011452917198, 1, 1], 
                    Hue[0.4582011452917198, 1, 1], 
                    Hue[0.2860972878164505, 1, 1], 
                    Hue[0.2860972878164505, 1, 1], 
                    Hue[0.1269644106342963, 1, 1], 
                    Hue[0.1269644106342963, 1, 1], 
                    Hue[0.008489555056382105, 1, 1], 
                    Hue[0.008489555056382105, 1, 1], 
                    Hue[0.9047969445291322, 1, 1], 
                    Hue[0.9047969445291322, 1, 1], 
                    Hue[0.8069026689219323, 1, 1], 
                    Hue[0.8069026689219323, 1, 1], 
                    Hue[0.7433426798313757, 1, 1], 
                    Hue[0.7433426798313757, 1, 1], 
                    Hue[0.6211645129944474, 1, 1], 
                    Hue[0.6211645129944474, 1, 1], 
                    Hue[0.570574249490098, 1, 1], 
                    Hue[0.570574249490098, 1, 1], 
                    Hue[0.37442686964880406`, 1, 1], 
                    Hue[0.37442686964880406`, 1, 1], 
                    Hue[0.41406408292314084`, 1, 1], 
                    Hue[0.41406408292314084`, 1, 1], 
                    Hue[0.27304668255560127`, 1, 1], 
                    Hue[0.27304668255560127`, 1, 1], 
                    Hue[0.1640613943112119, 1, 1], 
                    Hue[0.1640613943112119, 1, 1], 
                    Hue[0.9217837241712366, 1, 1], 
                    Hue[0.9217837241712366, 1, 1], 
                    Hue[0.04927441714522789, 1, 1], 
                    Hue[0.04927441714522789, 1, 1], 
                    Hue[0.7395544576770468, 1, 1], 
                    Hue[0.7395544576770468, 1, 1], 
                    Hue[0.8366427329778339, 1, 1], 
                    Hue[0.8366427329778339, 1, 1], 
                    Hue[0.640224275610829, 1, 1], 
                    Hue[0.640224275610829, 1, 1], 
                    Hue[0.5005906513158629, 1, 1], 
                    Hue[0.5005906513158629, 1, 1], 
                    Hue[0.39893509617051615`, 1, 1], 
                    Hue[0.39893509617051615`, 1, 1], 
                    Hue[0.31827148243229864`, 1, 1], 
                    Hue[0.31827148243229864`, 1, 1], 
                    Hue[0.10562517832932644`, 1, 1], 
                    Hue[0.10562517832932644`, 1, 1], 
                    Hue[0.05647944373999831, 1, 1], 
                    Hue[0.05647944373999831, 1, 1], 
                    Hue[0.954476650596741, 1, 1], 
                    Hue[0.954476650596741, 1, 1], 
                    Hue[0.8121914267182476, 1, 1], 
                    Hue[0.8121914267182476, 1, 1], 
                    Hue[0.6843275724593121, 1, 1], 
                    Hue[0.6843275724593121, 1, 1], 
                    Hue[0.8357639889934845, 1, 1], 
                    Hue[0.8357639889934845, 1, 1], 
                    Hue[0.6544318649455261, 1, 1], 
                    Hue[0.6544318649455261, 1, 1], 
                    Hue[0.571673756180975, 1, 1], 
                    Hue[0.571673756180975, 1, 1], 
                    Hue[0.5103863038769656, 1, 1], 
                    Hue[0.5103863038769656, 1, 1], 
                    Hue[0.38946345539840677`, 1, 1], 
                    Hue[0.38946345539840677`, 1, 1], 
                    Hue[0.3407740913415771, 1, 1], 
                    Hue[0.3407740913415771, 1, 1], 
                    Hue[0.2881490527713575, 1, 1], 
                    Hue[0.2881490527713575, 1, 1], 
                    Hue[0.14006130480093046`, 1, 1], 
                    Hue[0.14006130480093046`, 1, 1], 
                    Hue[0.03270695109683777, 1, 1], 
                    Hue[0.03270695109683777, 1, 1], 
                    Hue[0.9222580029127315, 1, 1], 
                    Hue[0.9222580029127315, 1, 1], 
                    Hue[0.8216153254234894, 1, 1], 
                    Hue[0.8216153254234894, 1, 1], 
                    Hue[0.7327219348514502, 1, 1], 
                    Hue[0.7327219348514502, 1, 1], 
                    Hue[0.6329691624014069, 1, 1], 
                    Hue[0.6329691624014069, 1, 1], 
                    Hue[0.5441018001510657, 1, 1], 
                    Hue[0.5441018001510657, 1, 1], 
                    Hue[0.451179412264641, 1, 1], 
                    Hue[0.451179412264641, 1, 1], 
                    Hue[0.34884452009062944`, 1, 1], 
                    Hue[0.34884452009062944`, 1, 1], 
                    Hue[0.24649878947472764`, 1, 1], 
                    Hue[0.24649878947472764`, 1, 1], 
                    Hue[0.15143639484703394`, 1, 1], 
                    Hue[0.15143639484703394`, 1, 1], 
                    Hue[0.050421982901845644`, 1, 1], 
                    Hue[0.050421982901845644`, 1, 1], 
                    Hue[0.9481497781591328, 1, 1], 
                    Hue[0.9481497781591328, 1, 1], 
                    Hue[0.8502040510520766, 1, 1], 
                    Hue[0.8502040510520766, 1, 1], 
                    Hue[0.7515057307791237, 1, 1], 
                    Hue[0.7515057307791237, 1, 1], 
                    Hue[0.6545229892266976, 1, 1], 
                    Hue[0.6545229892266976, 1, 1], 
                    Hue[0.556780119053588, 1, 1], 
                    Hue[0.556780119053588, 1, 1], 
                    Hue[0.45912955175820863`, 1, 1], 
                    Hue[0.45912955175820863`, 1, 1], 
                    Hue[0.36229952563725926`, 1, 1], 
                    Hue[0.36229952563725926`, 1, 1], 
                    Hue[0.2647079518516234, 1, 1], 
                    Hue[0.2647079518516234, 1, 1], 
                    Hue[0.16676307041787705`, 1, 1], 
                    Hue[0.16676307041787705`, 1, 1], 
                    Hue[0.06850347252601507, 1, 1], 
                    Hue[0.06850347252601507, 1, 1], 
                    Hue[0.9717018103355191, 1, 1], 
                    Hue[0.9717018103355191, 1, 1], 
                    Hue[0.8748549989781255, 1, 1], 
                    Hue[0.8748549989781255, 1, 1], 
                    Hue[0.7789328517480816, 1, 1], 
                    Hue[0.7789328517480816, 1, 1], 
                    Hue[0.6833400310612193, 1, 1], 
                    Hue[0.6833400310612193, 1, 1], 
                    Hue[0.587582947708543, 1, 1], 
                    Hue[0.587582947708543, 1, 1], 
                    Hue[0.49205787070990953`, 1, 1], 
                    Hue[0.49205787070990953`, 1, 1], 
                    Hue[0.3962264733104969, 1, 1], 
                    Hue[0.3962264733104969, 1, 1], 
                    Hue[0.30073678619523775`, 1, 1], 
                    Hue[0.30073678619523775`, 1, 1], 
                    Hue[0.20606099121402044`, 1, 1], 
                    Hue[0.20606099121402044`, 1, 1], 
                    Hue[0.11180750743429678`, 1, 1], 
                    Hue[0.11180750743429678`, 1, 1], 
                    Hue[0.01786908293360081, 1, 1], 
                    Hue[0.01786908293360081, 1, 1], 
                    Hue[0.9238158665680171, 1, 1], 
                    Hue[0.9238158665680171, 1, 1], 
                    Hue[0.8295682733677235, 1, 1], 
                    Hue[0.8295682733677235, 1, 1], 
                    Hue[0.735411875619816, 1, 1], 
                    Hue[0.735411875619816, 1, 1], 
                    Hue[0.6416165761393002, 1, 1], 
                    Hue[0.6416165761393002, 1, 1], 
                    Hue[0.5485735773427743, 1, 1], 
                    Hue[0.5485735773427743, 1, 1], 
                    Hue[0.4562559025427074, 1, 1], 
                    Hue[0.4562559025427074, 1, 1], 
                    Hue[0.3641662230314104, 1, 1], 
                    Hue[0.3641662230314104, 1, 1], 
                    Hue[0.2717745783461878, 1, 1], 
                    Hue[0.2717745783461878, 1, 1], 
                    Hue[0.17887247985716348`, 1, 1], 
                    Hue[0.17887247985716348`, 1, 1], 
                    Hue[0.08588874690629737, 1, 1], 
                    Hue[0.08588874690629737, 1, 1], 
                    Hue[0.9936406858076294, 1, 1], 
                    Hue[0.9936406858076294, 1, 1], 
                    Hue[0.9025475885767389, 1, 1], 
                    Hue[0.9025475885767389, 1, 1], 
                    Hue[0.8123557106102066, 1, 1], 
                    Hue[0.8123557106102066, 1, 1], 
                    Hue[0.722196790536449, 1, 1], 
                    Hue[0.722196790536449, 1, 1], 
                    Hue[0.63124672944796, 1, 1], 
                    Hue[0.63124672944796, 1, 1], 
                    Hue[0.5394334217154759, 1, 1], 
                    Hue[0.5394334217154759, 1, 1], 
                    Hue[0.4476048393319134, 1, 1], 
                    Hue[0.4476048393319134, 1, 1], 
                    Hue[0.35699908783455947`, 1, 1], 
                    Hue[0.35699908783455947`, 1, 1], 
                    Hue[0.2681214791428311, 1, 1], 
                    Hue[0.2681214791428311, 1, 1], 
                    Hue[0.18030782830551154`, 1, 1], 
                    Hue[0.18030782830551154`, 1, 1], 
                    Hue[0.09215150663418024, 1, 1], 
                    Hue[0.09215150663418024, 1, 1], 
                    Hue[0.0025077892689761004`, 1, 1], 
                    Hue[0.0025077892689761004`, 1, 1], 
                    Hue[0.9115947297733166, 1, 1], 
                    Hue[0.9115947297733166, 1, 1], 
                    Hue[0.8210927146819142, 1, 1], 
                    Hue[0.8210927146819142, 1, 1], 
                    Hue[0.732829693010617, 1, 1], 
                    Hue[0.732829693010617, 1, 1], 
                    Hue[0.647133972193977, 1, 1], 
                    Hue[0.647133972193977, 1, 1], 
                    Hue[0.5625815606607734, 1, 1], 
                    Hue[0.5625815606607734, 1, 1], 
                    Hue[0.47697251741405106`, 1, 1], 
                    Hue[0.47697251741405106`, 1, 1], 
                    Hue[0.38892812303980323`, 1, 1], 
                    Hue[0.38892812303980323`, 1, 1], 
                    Hue[0.29936197072723353`, 1, 1], 
                    Hue[0.29936197072723353`, 1, 1], 
                    Hue[0.2112842587929123, 1, 1], 
                    Hue[0.2112842587929123, 1, 1], 
                    Hue[0.12719223182875483`, 1, 1], 
                    Hue[0.12719223182875483`, 1, 1], 
                    Hue[0.04695078965265709, 1, 1], 
                    Hue[0.04695078965265709, 1, 1], 
                    Hue[0.9681574476610211, 1, 1], 
                    Hue[0.9681574476610211, 1, 1], 
                    Hue[0.887706749483258, 1, 1], 
                    Hue[0.887706749483258, 1, 1], 
                    Hue[0.8037701829451505, 1, 1], 
                    Hue[0.8037701829451505, 1, 1], 
                    Hue[0.7178930970056254, 1, 1], 
                    Hue[0.7178930970056254, 1, 1], 
                    Hue[0.6347073831219328, 1, 1], 
                    Hue[0.6347073831219328, 1, 1], 
                    Hue[0.5579546515943477, 1, 1], 
                    Hue[0.5579546515943477, 1, 1], 
                    Hue[0.48772321475245084`, 1, 1], 
                    Hue[0.48772321475245084`, 1, 1], 
                    Hue[0.42141514090553406`, 1, 1], 
                    Hue[0.42141514090553406`, 1, 1], 
                    Hue[0.3554913039774097, 1, 1], 
                    Hue[0.3554913039774097, 1, 1], 
                    Hue[0.28669414064235854`, 1, 1], 
                    Hue[0.28669414064235854`, 1, 1], 
                    Hue[0.21367847985739843`, 1, 1], 
                    Hue[0.21367847985739843`, 1, 1], 
                    Hue[0.13916554587353325`, 1, 1], 
                    Hue[0.13916554587353325`, 1, 1], 
                    Hue[0.06927835345538431, 1, 1], 
                    Hue[0.06927835345538431, 1, 1], 
                    Hue[0.008753732053659424, 1, 1], 
                    Hue[0.008753732053659424, 1, 1], 
                    Hue[0.958384426153491, 1, 1], 
                    Hue[0.958384426153491, 1, 1], 
                    Hue[0.9166770183971185, 1, 1], 
                    Hue[0.9166770183971185, 1, 1], 
                    Hue[0.8817915576244195, 1, 1], 
                    Hue[0.8817915576244195, 1, 1], 
                    Hue[0.8522672078087553, 1, 1], 
                    Hue[0.8522672078087553, 1, 1], 
                    Hue[0.8270975424776865, 1, 1], 
                    Hue[0.8270975424776865, 1, 1], 
                    Hue[0.8056284375105165, 1, 1], 
                    Hue[0.8056284375105165, 1, 1], 
                    Hue[0.7874454690796864, 1, 1], 
                    Hue[0.7874454690796864, 1, 1], 
                    Hue[0.7722914136674517, 1, 1], 
                    Hue[0.7722914136674517, 1, 1], 
                    Hue[0.7600076628418265, 1, 1], 
                    Hue[0.7600076628418265, 1, 1], 
                    Hue[0.750496805403843, 1, 1], 
                    Hue[0.750496805403843, 1, 1], 
                    Hue[0.7436936708530469, 1, 1], 
                    Hue[0.7436936708530469, 1, 1], 
                    Hue[0.7395522002789043, 1, 1], 
                    Hue[0.7395522002789043, 1, 1], 
                    Hue[0.7380306364234124, 1, 1], 
                    Hue[0.7380306364234124, 1, 1], 
                    Hue[0.739088410762426, 1, 1], 
                    Hue[0.739088410762426, 1, 1], 
                    Hue[0.7426813910790278, 1, 1], 
                    Hue[0.7426813910790278, 1, 1], 
                    Hue[0.7487603009809475, 1, 1], 
                    Hue[0.7487603009809475, 1, 1], 
                    Hue[0.7572706370390397, 1, 1], 
                    Hue[0.7572706370390397, 1, 1], 
                    Hue[0.7681537835764389, 1, 1], 
                    Hue[0.7681537835764389, 1, 1], 
                    Hue[0.7813485124592214, 1, 1], 
                    Hue[0.7813485124592214, 1, 1], 
                    Hue[0.7967903032737629, 1, 1], 
                    Hue[0.7967903032737629, 1, 1], 
                    Hue[0.8144135638472767, 1, 1], 
                    Hue[0.8144135638472767, 1, 1], 
                    Hue[0.8341527576324702, 1, 1], 
                    Hue[0.8341527576324702, 1, 1], 
                    Hue[0.8559423552343227, 1, 1], 
                    Hue[0.8559423552343227, 1, 1], 
                    Hue[0.8797184779594373, 1, 1], 
                    Hue[0.8797184779594373, 1, 1], 
                    Hue[0.9054178672267975, 1, 1], 
                    Hue[0.9054178672267975, 1, 1], 
                    Hue[0.9329806227146357, 1, 1], 
                    Hue[0.9329806227146357, 1, 1], 
                    Hue[0.9623475758499133, 1, 1], 
                    Hue[0.9623475758499133, 1, 1], 
                    Hue[0.9934629887045462, 1, 1], 
                    Hue[0.9934629887045462, 1, 1], 
                    Hue[0.02627405855083121, 1, 1], 
                    Hue[0.02627405855083121, 1, 1], 
                    Hue[0.06072941346367099, 1, 1], 
                    Hue[0.06072941346367099, 1, 1], 
                    Hue[0.09678062203177676, 1, 1], 
                    Hue[0.09678062203177676, 1, 1], 
                    Hue[0.1343824932010361, 1, 1], 
                    Hue[0.1343824932010361, 1, 1], 
                    Hue[0.17349119558818452`, 1, 1], 
                    Hue[0.17349119558818452`, 1, 1], 
                    Hue[0.21406617571740993`, 1, 1], 
                    Hue[0.21406617571740993`, 1, 1], 
                    Hue[0.2560679976886549, 1, 1], 
                    Hue[0.2560679976886549, 1, 1], 
                    Hue[0.299460457403954, 1, 1], 
                    Hue[0.299460457403954, 1, 1], 
                    Hue[0.3442092278042504, 1, 1], 
                    Hue[0.3442092278042504, 1, 1], 
                    Hue[0.3902814284807919, 1, 1], 
                    Hue[0.3902814284807919, 1, 1], 
                    Hue[0.4376464529812861, 1, 1], 
                    Hue[0.4376464529812861, 1, 1], 
                    Hue[0.486275510377512, 1, 1], 
                    Hue[0.486275510377512, 1, 1], 
                    Hue[0.5361404086810144, 1, 1], 
                    Hue[0.5361404086810144, 1, 1], 
                    Hue[0.5872156253981717, 1, 1], 
                    Hue[0.5872156253981717, 1, 1], 
                    Hue[0.63947675730222, 1, 1], 
                    Hue[0.63947675730222, 1, 1], 
                    Hue[0.6928994274576521, 1, 1], 
                    Hue[0.6928994274576521, 1, 1], 
                    Hue[0.7474627961633682, 1, 1], 
                    Hue[0.7474627961633682, 1, 1], 
                    Hue[0.8031448831692406, 1, 1], 
                    Hue[0.8031448831692406, 1, 1], 
                    Hue[0.8599277071610487, 1, 1], 
                    Hue[0.8599277071610487, 1, 1], 
                    Hue[0.9177905921948738, 1, 1], 
                    Hue[0.9177905921948738, 1, 1], 
                    Hue[0.9767162994358801, 1, 1], 
                    Hue[0.9767162994358801, 1, 1], 
                    Hue[0.03668769482659254, 1, 1], 
                    Hue[0.03668769482659254, 1, 1], 
                    Hue[0.09768870176319887, 1, 1], 
                    Hue[0.09768870176319887, 1, 1], 
                    Hue[0.15970390592623193`, 1, 1], 
                    Hue[0.15970390592623193`, 1, 1], 
                    Hue[0.22271848133218072`, 1, 1], 
                    Hue[0.22271848133218072`, 1, 1], 
                    Hue[0.2867176866955756, 1, 1], 
                    Hue[0.2867176866955756, 1, 1], 
                    Hue[0.3516896132531366, 1, 1], 
                    Hue[0.3516896132531366, 1, 1], 
                    Hue[0.41762027877249996`, 1, 1], 
                    Hue[0.41762027877249996`, 1, 1], 
                    Hue[0.4844980226954021, 1, 1], 
                    Hue[0.4844980226954021, 1, 1], 
                    Hue[0.5523092351240234, 1, 1], 
                    Hue[0.5523092351240234, 1, 1], 
                    Hue[0.621046141242471, 1, 1], 
                    Hue[0.621046141242471, 1, 1], 
                    Hue[0.6906954003728225, 1, 1], 
                    Hue[0.6906954003728225, 1, 1], 
                    Hue[0.7612464295655417, 1, 1], 
                    Hue[0.7612464295655417, 1, 1], 
                    Hue[0.8326892519615281, 1, 1], 
                    Hue[0.8326892519615281, 1, 1], 
                    Hue[0.9050154457110178, 1, 1], 
                    Hue[0.9050154457110178, 1, 1], 
                    Hue[0.9782137976593589, 1, 1], 
                    Hue[0.9782137976593589, 1, 1], 
                    Hue[0.05227788399332845, 1, 1], 
                    Hue[0.05227788399332845, 1, 1], 
                    Hue[0.1271986005150233, 1, 1], 
                    Hue[0.1271986005150233, 1, 1], 
                    Hue[0.2029642440043483, 1, 1], 
                    Hue[0.2029642440043483, 1, 1], 
                    Hue[0.27956900942770624`, 1, 1], 
                    Hue[0.27956900942770624`, 1, 1], 
                    Hue[0.3570070126386154, 1, 1], 
                    Hue[0.3570070126386154, 1, 1], 
                    Hue[0.43526876191562286`, 1, 1], 
                    Hue[0.43526876191562286`, 1, 1], 
                    Hue[0.5143483531822721, 1, 1], 
                    Hue[0.5143483531822721, 1, 1], 
                    Hue[0.5942364871856414, 1, 1], 
                    Hue[0.5942364871856414, 1, 1], 
                    Hue[0.6749261104680981, 1, 1], 
                    Hue[0.6749261104680981, 1, 1], 
                    Hue[0.7564165186521847, 1, 1], 
                    Hue[0.7564165186521847, 1, 1], 
                    Hue[0.8386941839035054, 1, 1], 
                    Hue[0.8386941839035054, 1, 1], 
                    Hue[0.9217502032467062, 1, 1], 
                    Hue[0.9217502032467062, 1, 1], 
                    Hue[0.005588047415432818, 1, 1], 
                    Hue[0.005588047415432818, 1, 1], 
                    Hue[0.09020190054450962, 1, 1], 
                    Hue[0.09020190054450962, 1, 1], 
                    Hue[0.17557830629088042`, 1, 1], 
                    Hue[0.17557830629088042`, 1, 1], 
                    Hue[0.2617155460289287, 1, 1], 
                    Hue[0.2617155460289287, 1, 1], 
                    Hue[0.3486030056287242, 1, 1], 
                    Hue[0.3486030056287242, 1, 1], 
                    Hue[0.43623852605324986`, 1, 1], 
                    Hue[0.43623852605324986`, 1, 1], 
                    Hue[0.5246243796516621, 1, 1], 
                    Hue[0.5246243796516621, 1, 1], 
                    Hue[0.6137467720443291, 1, 1], 
                    Hue[0.6137467720443291, 1, 1], 
                    Hue[0.7036113582715795, 1, 1], 
                    Hue[0.7036113582715795, 1, 1], 
                    Hue[0.7942020535195174, 1, 1], 
                    Hue[0.7942020535195174, 1, 1], 
                    Hue[0.8855024036439301, 1, 1], 
                    Hue[0.8855024036439301, 1, 1], 
                    Hue[0.9775292546983889, 1, 1], 
                    Hue[0.9775292546983889, 1, 1], 
                    Hue[0.07027539516154879, 1, 1], 
                    Hue[0.07027539516154879, 1, 1], 
                    Hue[0.16374124482826072`, 1, 1], 
                    Hue[0.16374124482826072`, 1, 1], 
                    Hue[0.25791700557200636`, 1, 1], 
                    Hue[0.25791700557200636`, 1, 1], 
                    Hue[0.3527640228073858, 1, 1], 
                    Hue[0.3527640228073858, 1, 1], 
                    Hue[0.4482987662581584, 1, 1], 
                    Hue[0.4482987662581584, 1, 1], 
                    Hue[0.5445531907493291, 1, 1], 
                    Hue[0.5445531907493291, 1, 1], 
                    Hue[0.6415090340163123, 1, 1], 
                    Hue[0.6415090340163123, 1, 1], 
                    Hue[0.739158911610914, 1, 1], 
                    Hue[0.739158911610914, 1, 1], 
                    Hue[0.8374834060735057, 1, 1], 
                    Hue[0.8374834060735057, 1, 1], 
                    Hue[0.9364240543729676, 1, 1], 
                    Hue[0.9364240543729676, 1, 1], 
                    Hue[0.03603829961410431, 1, 1], 
                    Hue[0.03603829961410431, 1, 1], 
                    Hue[0.13642177385132023`, 1, 1], 
                    Hue[0.13642177385132023`, 1, 1], 
                    Hue[0.23743312162973995`, 1, 1], 
                    Hue[0.23743312162973995`, 1, 1], 
                    Hue[0.33890501166499326`, 1, 1], 
                    Hue[0.33890501166499326`, 1, 1], 
                    Hue[0.4410449979957778, 1, 1], 
                    Hue[0.4410449979957778, 1, 1], 
                    Hue[0.5441512204674002, 1, 1], 
                    Hue[0.5441512204674002, 1, 1], 
                    Hue[0.6479683063190227, 1, 1], 
                    Hue[0.6479683063190227, 1, 1], 
                    Hue[0.7520934886159469, 1, 1], 
                    Hue[0.7520934886159469, 1, 1], 
                    Hue[0.8569136201029212, 1, 1], 
                    Hue[0.8569136201029212, 1, 1], 
                    Hue[0.9627047539340904, 1, 1], 
                    Hue[0.9627047539340904, 1, 1], 
                    Hue[0.06879089341509073, 1, 1], 
                    Hue[0.06879089341509073, 1, 1], 
                    Hue[0.17533461697758274`, 1, 1], 
                    Hue[0.17533461697758274`, 1, 1], 
                    Hue[0.28319618900543175`, 1, 1], 
                    Hue[0.28319618900543175`, 1, 1], 
                    Hue[0.39149295315047145`, 1, 1], 
                    Hue[0.39149295315047145`, 1, 1], 
                    Hue[0.4996425226407696, 1, 1], 
                    Hue[0.4996425226407696, 1, 1], 
                    Hue[0.6094061045208964, 1, 1], 
                    Hue[0.6094061045208964, 1, 1], 
                    Hue[0.7205987326917802, 1, 1], 
                    Hue[0.7205987326917802, 1, 1], 
                    Hue[0.8307909058918155, 1, 1], 
                    Hue[0.8307909058918155, 1, 1], 
                    Hue[0.9413621288318382, 1, 1], 
                    Hue[0.9413621288318382, 1, 1], 
                    Hue[0.055154985274585344`, 1, 1], 
                    Hue[0.055154985274585344`, 1, 1], 
                    Hue[0.16908344480096296`, 1, 1], 
                    Hue[0.16908344480096296`, 1, 1], 
                    Hue[0.27949834648938626`, 1, 1], 
                    Hue[0.27949834648938626`, 1, 1], 
                    Hue[0.39178904539044196`, 1, 1], 
                    Hue[0.39178904539044196`, 1, 1], 
                    Hue[0.5113599233497237, 1, 1], 
                    Hue[0.5113599233497237, 1, 1], 
                    Hue[0.6321214530795161, 1, 1], 
                    Hue[0.6321214530795161, 1, 1], 
                    Hue[0.7464805255537773, 1, 1], 
                    Hue[0.7464805255537773, 1, 1], 
                    Hue[0.8602620296701579, 1, 1], 
                    Hue[0.8602620296701579, 1, 1], 
                    Hue[0.9811501935614256, 1, 1]}],
                   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
                   Axes->{True, None},
                   AxesOrigin->{0, 0},
                   
                   Epilog->LineBox[{{1, 3.4150615472307896`*^-7}, {
                    2, 8.326392522476883*^-7}, {3, 2.859048395689338*^-7}, {
                    4, 3.953536088470222*^-7}, {5, 6.351631059237657*^-7}, {
                    6, 3.894539832031944*^-7}, {7, 6.586814227083131*^-7}, {
                    8, 7.510154018765163*^-7}, {9, 4.227642051301257*^-7}, {
                    10, 8.383393464990893*^-7}, {11, 1.032485409675302*^-6}, {
                    12, 4.941444249232437*^-7}, {
                    13, 1.0881791935708725`*^-6}, {
                    14, 7.218617213599437*^-7}, {
                    15, 1.0227181383285202`*^-6}, {
                    16, 4.985029921358325*^-7}, {17, 6.258330298641342*^-7}, {
                    18, 7.27637374271006*^-7}, {19, 7.708110456728936*^-7}, {
                    20, 8.678819833415261*^-7}, {
                    21, 1.0716205001774192`*^-6}, {
                    22, 7.435967511779358*^-7}, {23, 6.641269062648287*^-7}, {
                    24, 5.501954937648988*^-7}, {25, 6.658192669437551*^-7}, {
                    26, 6.535545165656654*^-7}, {
                    27, 4.6667900441271827`*^-7}, {
                    28, 7.565506936805698*^-7}, {
                    29, 1.3821364559304605`*^-7}, {
                    30, 9.152074515538222*^-7}, {31, 7.424893027570349*^-7}, {
                    32, 5.907814572336757*^-7}, {33, 8.83102574584025*^-7}, {
                    34, 5.522986306869286*^-7}, {35, 8.459179833732249*^-7}, {
                    36, 7.169306372930248*^-7}, {37, 8.44350804618063*^-7}, {
                    38, 5.927428025714487*^-7}, {
                    39, 1.1978202269684304`*^-6}, {
                    40, 1.634963344743732*^-6}, {
                    41, 1.7854314793513283`*^-6}, {
                    42, 2.0073168838370047`*^-6}, {
                    43, 1.5419112921733522`*^-6}, {
                    44, 1.717365162019975*^-6}, {
                    45, 2.0267033489479267`*^-6}, {
                    46, 2.820588186792828*^-6}, {
                    47, 3.0168859771533597`*^-6}, {
                    48, 3.6208161270000935`*^-6}, {
                    49, 4.517308708685305*^-6}, {50, 5.597084412290063*^-6}, {
                    51, 6.904492751504067*^-6}, {52, 8.590829410804584*^-6}, {
                    53, 0.000010364899646600366`}, {
                    54, 0.000012674838565826955`}, {
                    55, 0.000014677383465063058`}, {
                    56, 0.000018530602034053334`}, {
                    57, 0.000021879664582344023`}, {
                    58, 0.00002740514740495939}, {
                    59, 0.00003302039946077479}, {
                    60, 0.00004007680766577907}, {
                    61, 0.00004915721318423795}, {
                    62, 0.00005967600343351285}, {
                    63, 0.00007132251573900435}, {
                    64, 0.00008510615624805744}, {
                    65, 0.00010153862867520497`}, {
                    66, 0.0001223953997284478}, {67, 0.0001456486479400899}, {
                    68, 0.00017417020316132464`}, {
                    69, 0.00020841130300822193`}, {
                    70, 0.0002473682266457697}, {
                    71, 0.00029365700045998444`}, {
                    72, 0.00034651286008924374`}, {
                    73, 0.00040818754207493107`}, {
                    74, 0.0004814114763619019}, {75, 0.0005669946159744187}, {
                    76, 0.0006666384195951068}, {77, 0.0007829210181043247}, {
                    78, 0.0009147297986318929}, {
                    79, 0.0010637780697234705`}, {
                    80, 0.0012355887978018766`}, {
                    81, 0.0014365669639994304`}, {
                    82, 0.0016725545299133711`}, {
                    83, 0.0019457368886742626`}, {84, 0.00225230897792296}, {
                    85, 0.0025913788164238292`}, {86, 0.002966934515081171}, {
                    87, 0.0033962259402968093`}, {88, 0.003898185716157752}, {
                    89, 0.004486263571616521}, {90, 0.005155713635330152}, {
                    91, 0.005886974507878243}, {92, 0.0066641588426744275`}, {
                    93, 0.0075042385315737346`}, {94, 0.00846178480498481}, {
                    95, 0.0096020983264912}, {96, 0.010946061400524376`}, {
                    97, 0.01243978640152333}, {98, 0.013990437302815404`}, {
                    99, 0.015550841999877631`}, {100, 0.01719956338118665}, {
                    101, 0.019134177746136533`}, {
                    102, 0.021536210833143093`}, {
                    103, 0.024383921831166423`}, {
                    104, 0.027418550004813615`}, {
                    105, 0.030326530631184095`}, {106, 0.03302736870720255}, {
                    107, 0.0358720965542906}, {108, 0.0395122163239013}, {
                    109, 0.04437791884143114}, {110, 0.05017589150553378}, {
                    111, 0.05600599810739158}, {112, 0.0610046778863704}, {
                    113, 0.0650948966996837}, {114, 0.06939751214550856}, {
                    115, 0.07580898835647554}, {116, 0.08553688954242283}, {
                    117, 0.09785492383741799}, {118, 0.11050150808855051`}, {
                    119, 0.12111840769427733`}, {120, 0.12859344456760172`}, {
                    121, 0.134032829700865}, {122, 0.1411244888268904}, {
                    123, 0.15498461252600032`}, {124, 0.17918326432910084`}, {
                    125, 0.21357677996332655`}, {126, 0.255231688443048}, {
                    127, 0.3004992585990988}, {128, 0.3462055832432251}, {
                    129, 0.3900125249745804}, {130, 0.4304013996463122}, {
                    131, 0.4665460145571011}, {132, 0.4981404183785623}, {
                    133, 0.5252393663008192}, {134, 0.5481156545916485}, {
                    135, 0.5671531175218409}, {136, 0.5827668510676232}, {
                    137, 0.5953588515241416}, {138, 0.6052863815846707}, {
                    139, 0.6128573082252979}, {140, 0.6183266041864458}, {
                    141, 0.621901066307627}, {142, 0.6237432684263104}, {
                    143, 0.6239906869910626}, {144, 0.6227496936719461}, {
                    145, 0.6201109653312574}, {146, 0.6161508541851485}, {
                    147, 0.6109385447498156}, {148, 0.6045357420982319}, {
                    149, 0.5970040668252714}, {150, 0.5884030914674586}, {
                    151, 0.5787907422470377}, {152, 0.5682299701611412}, {
                    153, 0.5567823827512517}, {154, 0.5445112446496322}, {
                    155, 0.5314860261248231}, {156, 0.517772145746944}, {
                    157, 0.5034429193957675}, {158, 0.48856976394841967`}, {
                    159, 0.47322240515913677`}, {160, 0.45747600274916994`}, {
                    161, 0.4414014197716264}, {162, 0.4250710828279676}, {
                    163, 0.408555695309991}, {164, 0.39192351837439604`}, {
                    165, 0.3752446819665927}, {166, 0.3585804974155168}, {
                    167, 0.3419960623351603}, {168, 0.325547006081104}, {
                    169, 0.3092903168809777}, {170, 0.2932770082171458}, {
                    171, 0.27755497601246615`}, {172, 0.2621669180765608}, {
                    173, 0.2471516967282329}, {174, 0.23254580107335218`}, {
                    175, 0.21837846809359532`}, {176, 0.20467642500234118`}, {
                    177, 0.19146183794629634`}, {178, 0.17875167407500447`}, {
                    179, 0.16656162614965095`}, {180, 0.1549010885025859}, {
                    181, 0.14377693159047808`}, {182, 0.13319050721421524`}, {
                    183, 0.12314405738145211`}, {184, 0.11363363298652249`}, {
                    185, 0.10465212554227128`}, {186, 0.09619221982734708}, {
                    187, 0.08824340229725057}, {188, 0.08079288682804961}, {
                    189, 0.07382641517495493}, {190, 0.06732890296889077}, {
                    191, 0.061281772559463626`}, {192, 0.05566798340887635}, {
                    193, 0.05046959806798442}, {194, 0.045666359914472716`}, {
                    195, 0.041238698412168484`}, {196, 0.03716681068531065}, {
                    197, 0.033430316193146854`}, {198, 0.03001037692370112}, {
                    199, 0.026886711722907297`}, {
                    200, 0.024039528047743727`}, {201, 0.02145161294807903}, {
                    202, 0.01910484112329695}, {203, 0.016980392726056056`}, {
                    204, 0.01506175346484479}, {205, 0.013333090980926728`}, {
                    206, 0.011778970847107622`}, {
                    207, 0.010385550777830857`}, {208, 0.00913845984920063}, {
                    209, 0.008024883664764117}, {210, 0.007032790109201361}, {
                    211, 0.006150319492525345}, {212, 0.005367872366509762}, {
                    213, 0.004675754637250524}, {214, 0.004064534653631685}, {
                    215, 0.0035260121634039933`}, {
                    216, 0.00305217400503307}, {
                    217, 0.0026364810795332207`}, {
                    218, 0.002273194257521188}, {
                    219, 0.0019560774853568555`}, {
                    220, 0.0016795982191183387`}, {
                    221, 0.0014391215327172158`}, {
                    222, 0.001230249416843563}, {223, 0.001049515424687823}, {
                    224, 0.000893923723498234}, {225, 0.000759743329498674}, {
                    226, 0.0006438014811699403}, {
                    227, 0.000544558600428294}, {
                    228, 0.0004604300661542428}, {
                    229, 0.00038857954949786573`}, {
                    230, 0.0003265822911236298}, {
                    231, 0.00027385383194716796`}, {
                    232, 0.00022966188700515662`}, {
                    233, 0.0001919906738366209}, {
                    234, 0.00015989314521156376`}, {
                    235, 0.0001333344062333693}, {
                    236, 0.00011098563653175951`}, {
                    237, 0.00009175955142057913}, {
                    238, 0.0000759780766423948}, {
                    239, 0.00006310342740921583}, {
                    240, 0.00005186949048771065}, {
                    241, 0.000042382776042547845`}, {
                    242, 0.00003500724731100024}, {
                    243, 0.000028827735342542557`}, {
                    244, 0.00002321197667845493}, {
                    245, 0.000018756374384474497`}, {
                    246, 0.00001564463200573973}, {
                    247, 0.00001295156033672626}, {
                    248, 0.000010217746008293889`}, {
                    249, 7.9524150327209*^-6}, {250, 6.423517060508637*^-6}, {
                    251, 5.140334711654943*^-6}}],
                   Frame->True,
                   FrameTicks->{{{1., 
                    FormBox[
                    RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
                    FormBox[
                    RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
                    FormBox[
                    RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
                    FormBox[
                    RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
                    FormBox["0", TraditionalForm]}, {157.25, 
                    FormBox["2.5`", TraditionalForm]}, {188.5, 
                    FormBox["5", TraditionalForm]}, {219.75, 
                    FormBox["7.5`", TraditionalForm]}, {251., 
                    FormBox["10", TraditionalForm]}}, Automatic, None, None},
                   PlotRange->{-0.1, 1.1},
                   PlotRangeClipping->True,
                   PlotRangePadding->{
                    Scaled[0.02], Automatic}], 16->
                  GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF13tcjvf/B/DLqZlfyERWzCUhzTEpKvVShM4nnQ9X58PdwXHYZvuE2FjO
rGxxZ2sNm2nYKswlI7IW0yYz3L9qTkty/hL283t437/ff9e7P+rx7H2/7lfd
Xff1eTckMTc4pbMkSXdefvrfr68+2t1v7R5e/aSr3o2+gbutfd/8O3kXuTtW
xk9aMnr6N2Qz/L6vYsfF9bvJ5ghzTd/8ZL9xPgA333Hs2t62gTwQAbpfS5c1
7iPLeLp0/mbfW1+RrRH6YqzV4k/LyDa4+WVKbuL9r8nDYb5atERVGB9vC+nu
3/3rln9BtsPlmyK1Nn8PeRR2Zp2d/E6rcT4G1Ql2WyysjPNxGO7zNH9NnxLy
eERaf/95y+39ZHtYmshnDD9sJ0/Aso78lpCNB8kOaK+q3/XFMePvMxEO96+N
OyCMz+eIaWFmnWflbyQ74ezi6fhu31byJLQNmP527UTj4ydjY1r3bT0GG19v
Z4S3Gcr9LcvJLvjY+vb+kNrvya5onDLhucmJ5eQpGJT92wtp/16yG8w7xbTp
ThtfL3ekHWoWj1qNPy/QaCXNcbeg30cA80503MjpcojmU/F3jfeDhXVkMRXO
J+f06nanmuYeqOo0/wvPipM094DfkN158oRTNPfEJ16S44Z/62nuiaw21+Yz
MedoPg1zD65xvD3sD5pPg8vQeL+o8j9pPh13bh4+XfX2VZpPR+PZxXeaHJtp
7oUr9Yr5vJbrNPfC6MxLdeOi2mg+A4kWvy+0FvdoPgPnS5Mujmx6SPOZqI9u
2D/t1lOaz0T5k6lltkGS+6v5LIhzjvKUnV1eWcyCv3q8bm3GazT3hkXKj2Xh
3qY090bC3mG+zX370NwHpWlxcxsDzWnuA3W52cANugE098WqLiZffzlhEM19
Ebt5Ulpp5BCa++G1huJC30obmvuhJGla0egsW5r746M+666UdoyiuT9CP1z2
fP6I8TQPQIf7pLqOcQ40D0BhZ/vtBbVONA/EDxsz+1Tec3llBMK6NHzFshTQ
4wORt9Stytzd85XVQJRHF9a8/oEX5YOwZEm2nb2nN+WDsGzXGuvWLf6UD0KH
5+brt08EUT4IxcG5SztKQikfjIaa669NbQmnfDAMJ5zGTNkRTfmX/uBU0yfN
8ZQPRk5pr8Qe8YmUD8H2N95vahiUQvkQbOy73TTNJZ3yIcj7qOd4N1VH+RBs
ndrDZN7hXMqH4tq8z/tuHzSP8qHI8Jrv/e2XCygfCsvCmCJpySLKh2KVpUWg
0853KT8bmxZF6T4Y8QHlZ2NUanYv3x55lJ+Nvpcqjv7rvJzys2EVXmKxek8+
5cOQ8uxN+8L4jygfBs/9D/NcItdQPgwVcxZ3f/zZWsqHIdnqfPLz/hsoH47p
jbpk756bKB+O5Zcdl3o4baF8OKaM65b4rPBTyofjwTH/zDb7bZSPwPXjP8Zv
7VRM+Qj0O3ZqUHsnPeUjYHW6l5S9o4TyEbDr07TndvwXlI9Em+vF7dY+pZSP
hF34arO45DLKR2JBVuTXTV/tonwkFtt2G+v0+jeUj8KxFVcd21fspXwUHB76
60PW76N8FHo6rjht+Z9yykch8JshmU0r9lM+GsMP/rRv3eSDlI9G2UxZDTD9
kfLRcLFf8OjkPxWUj4bjndYj5nVVlI9Btlmr0xuVhykfA/fQiKbiup8oH4NR
tnf7VW9VKR+Ds6OkutD7xygfC4cCdBTMPU75WPyzwWTu3Cc/Uz4Wv7RfuqBb
eJLysbDpGTNySHMN5eNQ3n67vTH6NOXjcKPWwxDc+wzl43DwUlD6r2PrKB+H
bte+HXb3Yj3l49EQVr1w3fJzlI/HTf2mVstZ5ykfjzseC6qODP2d8vEYrc/f
O/jpH5RXMOzhoBGxFY2vLCv4aJFr17ygP+n5FLSXfnjK7+ilV1YULPC4FPSZ
dJmeX4HyX1uXVVpceWW9gsqEqG8vdL9KfQq+th48s6aRbFAgPQgcONDLQP0J
WG/98YxvNpHlBGQHNAv1DzISUFISbV1q9t/Un4Bphx+8E+RBFgm4ftI1KSuL
rE/AhCOjalPWkdUEHPxNF5u3h2xIgGfk5XeaVbKUiGN11UWF9WQ5ETUHihxK
L5CRiLzs0UXdLhr7E/G4n8uy8vPG/kSoI+ed3VFj7E/EqaaJfzccMPYnwmvl
qteDPzP2J6Kl51t+I94z9idhye6aPqmhxv4khC/oPMV2hLE/CWXfHTAvvE+v
h5KEbYpV/yuVZJGEoc9m1tu/S9YnYfGGfnvOO5DVJNz5wcnetMH4+iehYLCT
qYMDWUpGZO0N90Or6e8nJ2Pr1Yq1YY3090UyPrEfsP7IILKSjH4h0o/l0X9R
fzKeFLz1aadNdH3ok/FTTp+KuT/T9aMmo2xtTPXV9ovUn4z3R/51ru+bZCkF
HdXRblemGK+/FNiOt9G9qVyg/hTEHo/uv0bQ9aqk4NpxS4vnO+h6FilwKVwy
yOloA/WnYKri8eLOFbr+1RSMGVoe0/zvb9SfgnKf92sOWZOlVLwVIS2ynEnv
HzkVtePjc4vmnKX+VDiYfhwqJtP7TUmFn5u1cN5G70eRira+0zImdP2F+lNh
datjdMziWupPRcTz4mHxj09RfyrGrfv2xT95xvd/Gux/HlhZMIDuD3IaEns/
Pn7sEN0/kIaxe5Lfb9LR/UVJg2RS9vnEkdXUn4ajTg9P3rtP9yd9GrrOuVK9
sfAo9aeheIy1xZzkI9SfhkmQZ630OUT96XB528Kq0K+S+tORLMJ2L8ik+yfS
cXviI/uyErq/Ki/nz7ouPnGP7r8iHf0a3NxGxn5P/emY6HBj/lAD3b/VdGRc
+XRf/0d0fzekY9eZ2qeTo4z3/wxsqe5v/vstOh/kDPjk7nR0/orOD2Tgrvua
8NOr6XxRMuCsLDRbtZXOH5GBsFVb8t+tofNJnwH73oeVFis6v9QMBHz43Zfr
h9L5ZsjAW75zb+X7G8+/TPhtDLkx4yCdj3Imnl30eHQvhM5PZOJEXe8x14bT
+apkomDzLyWH7ej8FS/nrVYJlhV0PuszEX40s3RlLp3faiYWznpj9+GFdL4b
Xvb1Pjxg+M/G81+Hpz3rGsJm034g69Dl2ot9a21of4AON0YUdxkTQfuFosOw
1ba7nH+l/UPo4POfu+NKd9J+otfhwF+dfIbW0f6i6nDR2n7JOi/abww62Gyo
t/vTZA71ZyHp4XtDrHplUX8WEnLF9p3LaH9CFuofLNn+II72KyULw+/bFzsW
0f4lsmD6RswP82wV6s9C6liLRf1NYqg/C2dC5cfrp0ZQ/8u+wTWXX9w37n/Z
6Lpt/daQ3sHUn40LVSFrO5kHUH82Yv61WT/6AO2XSjZOrF7zYOk12j9FNlKv
rlw+uZD2U3027ibvi7hXRfurmo3GgEMPfYUr9WejNuRx0WzXSdSfAzs787cj
i2g/lnNwubHlk+L3aH9GDqpmJrRcXTua+nMwxMTH+d1/aP8WOdi/5lqYqdkw
6s+B94s5mT/V0f6u5uDW3sr5DxpovzfkwGDI6O9pMO7/ubA5tzm381/0/4Gc
i7SEuOtdFtL/D8iFe/5Q0wKpJ/XnokdB39bTk7pTvp2+/t8HtN2dsRljc8YD
GA9kLDO2ZmzDeDhjW8Z2jEcxHsN4HOPxjO0ZT2DswHgiY0fGTownMZ7M2Jmx
C2NXxlMYuzF2ZwxtC8bSVG0LxpKHtgVjyVPbgrE0TduCsTRd24Kx5KVtwVia
oW3BWJqpbcFYmqVtwVjy1rZgLPloWzCWfLUtGEt+2haMJX9tC8ZSgLYFYylQ
22AsGKuMpSBtg7FgrDKWgrUNxoKxylgK0TYYC8YqYylU22AsGKuMpdnaBmPB
WGUshWkbjAVjlbEUrm0wFoxVxlKEtsFYMFYZS5HaBmPBWGUsRWkbjAVjlbEU
rW0wFoxVxlKMtsFYMFYZS7HaBmPBWGUsxWkbjAVjlbEUr20wFoxVxpKibZkx
GCuMBWM9Y5WxgbGUoG2ZMRgrjAVjPWOVsYGxlKhtmTEYK4wFYz1jlbGBsZSk
bZkxGCuMBWM9Y5WxgbGUrG2ZMRgrjAVjPWOVsYGxlKJtmTEYK4wFYz1jlbGB
sZSqbZkxGCuMBWM9Y5WxgbGUpm2ZMRgrjAVjPWOVsYGxlK5tmTEYK4wFYz1j
lbGBsZShbZkxGCuMBWM9Y5WxgbGUqW2ZMRgrjAVjPWOVsYGxpNO2zBiMFcaC
sZ6xytjAWMrStswYjBXGgrGescrYwFjK1rbMGIwVxoKxnrHK2MBYytG2zBiM
FcaCsZ6xytjAWMrVtswYjJX/9/8APtx4tw==
                    "], {
                    {GrayLevel[0.5], EdgeForm[None], 
                    GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ
2qYUCakkqbSrJBXatNLyfM7n/eM55zkzc2fud+be9/dW7t6/Tb8cEX/DAZBg
P/wDGi8KxRg8EIrhhcKLUvL6worDC0ORgph/4b+CtRorwlzRVLAXFMdLaE+8
FByKHwYHh8dK4gdB8YKcMl4o+awSUBovAyXDlMUPUUyYcnj55L2bwkB8EDQL
n1EBPxxK42XgSPwoKBumEn609lRuUB0/TjmGqYEfDxXw8lAVr5a8VjHH4JWh
XPgbSunbknNXTE28lnLQ2VAfP0k54BWhLn4CHBHOsaI8OVfF1MbrJK9VTAP8
ZKgUjqmHn5i8t3I4Fq+SnKvWnIKfqm9UrtAYP00541XgDPxMqIpXg7Pxc6B6
eOwsvHnyXB24DG8LdXUWNMNPT95LMefi5+kOw7TAz4caeC24GL8Eaof3aIdf
rjsIj7XBL00+SzEt8Qv0BmFa4a2hZjimPd4B6oXpiHfSneCN4Fr8Omisu4Ce
eC/dSXhNZ/wKqB+mC95VbxTmSrwbNAjTHb9KbxC+04Z4o+S71VgP/Orks3TH
TfCmyXfVEPri1yTnphx6432S55RjP7y/3ijM9fgN0CTMAPzG5Fr/C3bRhLuz
e1o1P5i5IXoTvS0Mw2/WG+ttYBw+Xm+kt4UR+K16Y709jMJHqwbCMcPxW5LX
amwMPjZ5L+0xAb8dWuCt4W78HrgwPDYRn6QaCJ9xGz4y+ayWMAW/U28cZip+
F7QKMw2fnry3vukmfGjyt+mMGfi9cJFqC2bh96vGwszGH4A2eFt4FH8M2oV5
HH9CNajahrn4I8lrFTMHf1A1Gf6GyfgdybkrZh7+JLTHO8EifDF0xjvAAvxp
6BjOcSZ+X3KuipmPP5W8VjFL8KWqyXDMQvyZ5L01tgxfDl3CPIs/p5oN8zz+
Alyp2ofN+HvqAbwbvIS/DN3DrMRfUU2HeRVfBT3CrMZfU02H99iCvw/9wmNr
8NehJ94L1uNvQG+8D7yFvw19w2vW4uuS12psI/6OeiIcswF/MzlWY5vwd5O/
RT1/od46+W+JcvgA/1A9o9qA7/EfVCP4INiOfwOD1UvwMf6JeirMp/hnMCDM
5/gX6rEw2/AvYWD4jK34R8l7aewr/Ovks3TnL+Irku96COzEv0vOTTnswL9N
nlOOu/DdMCzMHvxH9WyYn/CfYXiYX/Bf1ZNh9uK/wYgwv+N/qKfxkbAf/wdG
4aNBP9oJxoTH/mXuv+S5SVCauTIwWb0KfzO3L3kvxWTmCsHYMAfghWEcfjuU
xA+CieE9yuKHwB3hsVL4wdlnKaYIXhTGhymGHwgTwjHl8PIwJcyh+GFwJ34v
1MCPh5n43XAsXgXuCa+pgB8OU8NUxI+Au8IciR8F08JUwo+G6eEciuMlsr9N
Y8fglbPPmg318ZPgAXwGVMePy85NOVTFq2XPKceaeC24L0xtvA7MClMXPwHu
D1MPPzH7LJ3RAD8Z5qjW1AN4f1iBz1d/4BfBU/iD0BBvBA+FaYyfBg+HaYI3
hblhmuGnwyPhPS7GL4EF4bEz8DPhUfwxOBs/Bx7Hn4AW+PkwL7zmLLx59lqN
tcQvgCfDMefi52XHaqwV3jr7W/RN1+M3wEvhHNrgl8LT+LPQE+8Fz+FL1YN4
N1iGL4R2+OXwTJj2eAdYFKYj3gkWh+mMXwFLwmdchrfN3ktjXfCu2WfpDU7B
T82+6+XQA786Ozfl0B2/KntOOfbG+8DzYfri18ALYa7Fr8t+W/3mPETvPZz8
26Q7GMDcjfAyvgPW4uvgW/wVuAkfCq/iq2A4fgusDo8Nw2/OnnsNbsNHwhp8
JQzGh2TvpZgR+K3Za7VmFD4aXsfXqUfUG7A+/DfnT3L9K/lvxxswWb0PG8JM
Ue/Cm+GYifik7LUam6rehLfwt9XT6mXYiH+gmlGtwIf4e7of1TJsCa+Zod6D
d8LMVK/BpjCz1FvwbpjZ6l3YHP6mMfhYWBsem6O3zT5LOU3Dp2fn9r5qXLWd
nZtymKveyZ5TjvNU67A1zHz1JnwUZoFqGT4OnzkOH599txpbqNqFT8IsUq3C
p/gXqiHVDmzDv1OOyg2+x79UzahW4Kvw2EbdDfwQjnkRX5G9VmObdDewC/9c
Pabeyj7ra9WQage2h2tuvd4OdobPWKnayV6rsQ166+zcFLNatQTfhFmjWsqu
ZdX0QHxQdi0qh826e9iN79UZ2ht+w/eoJlQL8CP+i3JWrvAr/pPuUHcHP4fv
bIl6Fz4Lx2zVW2Sv1Z7blRv8Hj5zi94y+yztuU13lZ2L1uxQ78EfUXCG9s7O
5U/due46+391nbkMX559t4rZqbvJXqv/4ffoW2Af/j8egu84
                    
                    "]]]}, {{}, {}, {
                    {Hue[0.03996625933509463, 1, 1], LineBox[{1, 2}]}, 
                    {Hue[0.013716043603985754`, 1, 1], LineBox[{2, 3}]}, 
                    {Hue[0.931478219924176, 1, 1], LineBox[{3, 4}]}, 
                    {Hue[0.7933295525960278, 1, 1], LineBox[{4, 5}]}, 
                    {Hue[0.671505196618513, 1, 1], LineBox[{5, 6}]}, 
                    {Hue[0.48600936817024304`, 1, 1], LineBox[{6, 7}]}, 
                    {Hue[0.4514066116504211, 1, 1], LineBox[{7, 8}]}, 
                    {Hue[0.41317640143488066`, 1, 1], LineBox[{8, 9}]}, 
                    {Hue[0.1728699714822034, 1, 1], LineBox[{9, 10}]}, 
                    {Hue[0.16149973504533702`, 1, 1], LineBox[{10, 11}]}, 
                    {Hue[0.014380992997808584`, 1, 1], LineBox[{11, 12}]}, 
                    {Hue[0.8384753595018941, 1, 1], LineBox[{12, 13}]}, 
                    {Hue[0.8680556074999903, 1, 1], LineBox[{13, 14}]}, 
                    {Hue[0.7534222753972779, 1, 1], LineBox[{14, 15}]}, 
                    {Hue[0.6098627904806089, 1, 1], LineBox[{15, 16}]}, 
                    {Hue[0.6284046995572222, 1, 1], LineBox[{16, 17}]}, 
                    {Hue[0.5287100058968992, 1, 1], LineBox[{17, 18}]}, 
                    {Hue[0.3429159109541728, 1, 1], LineBox[{18, 19}]}, 
                    {Hue[0.31298068804368645`, 1, 1], LineBox[{19, 20}]}, 
                    {Hue[0.03115863318914752, 1, 1], LineBox[{20, 21}]}, 
                    {Hue[0.8085133232198168, 1, 1], LineBox[{21, 22}]}, 
                    {Hue[0.7785666480644399, 1, 1], LineBox[{22, 23}]}, 
                    {Hue[0.8813213024321782, 1, 1], LineBox[{23, 24}]}, 
                    {Hue[0.6921781949154409, 1, 1], LineBox[{24, 25}]}, 
                    {Hue[0.6809606252489155, 1, 1], LineBox[{25, 26}]}, 
                    {Hue[0.503465470130599, 1, 1], LineBox[{26, 27}]}, 
                    {Hue[0.34795262488670103`, 1, 1], LineBox[{27, 28}]}, 
                    {Hue[0.515525312724336, 1, 1], LineBox[{28, 29}]}, 
                    {Hue[0.29980886210904617`, 1, 1], LineBox[{29, 30}]}, 
                    {Hue[0.18072515446336535`, 1, 1], LineBox[{30, 31}]}, 
                    {Hue[0.10297890719872621`, 1, 1], LineBox[{31, 32}]}, 
                    {Hue[0.0023782259479396594`, 1, 1], LineBox[{32, 33}]}, 
                    {Hue[0.914852704835432, 1, 1], LineBox[{33, 34}]}, 
                    {Hue[0.8537396651102496, 1, 1], LineBox[{34, 35}]}, 
                    {Hue[0.7448026676434731, 1, 1], LineBox[{35, 36}]}, 
                    {Hue[0.6777817332505758, 1, 1], LineBox[{36, 37}]}, 
                    {Hue[0.5860571244254931, 1, 1], LineBox[{37, 38}]}, 
                    {Hue[0.4871012967944014, 1, 1], LineBox[{38, 39}]}, 
                    {Hue[0.42687764741931494`, 1, 1], LineBox[{39, 40}]}, 
                    {Hue[0.30671059341868573`, 1, 1], LineBox[{40, 41}]}, 
                    {Hue[0.20522452942419225`, 1, 1], LineBox[{41, 42}]}, 
                    {Hue[0.10111217553275076`, 1, 1], LineBox[{42, 43}]}, 
                    {Hue[0.00422564743866865, 1, 1], LineBox[{43, 44}]}, 
                    {Hue[0.9028465739827174, 1, 1], LineBox[{44, 45}]}, 
                    {Hue[0.8018316233387496, 1, 1], LineBox[{45, 46}]}, 
                    {Hue[0.6960993570726182, 1, 1], LineBox[{46, 47}]}, 
                    {Hue[0.6037710348526477, 1, 1], LineBox[{47, 48}]}, 
                    {Hue[0.5044937932940825, 1, 1], LineBox[{48, 49}]}, 
                    {Hue[0.405547663514985, 1, 1], LineBox[{49, 50}]}, 
                    {Hue[0.3059439158901264, 1, 1], LineBox[{50, 51}]}, 
                    {Hue[0.20567206969598656`, 1, 1], LineBox[{51, 52}]}, 
                    {Hue[0.10677120539249679`, 1, 1], LineBox[{52, 53}]}, 
                    {Hue[0.007189722064902663, 1, 1], LineBox[{53, 54}]}, 
                    {Hue[0.9072237591440739, 1, 1], LineBox[{54, 55}]}, 
                    {Hue[0.8089701107334102, 1, 1], LineBox[{55, 56}]}, 
                    {Hue[0.7109228185973671, 1, 1], LineBox[{56, 57}]}, 
                    {Hue[0.6136604440188916, 1, 1], LineBox[{57, 58}]}, 
                    {Hue[0.5156829678350188, 1, 1], LineBox[{58, 59}]}, 
                    {Hue[0.418564420178921, 1, 1], LineBox[{59, 60}]}, 
                    {Hue[0.32190746830302974`, 1, 1], LineBox[{60, 61}]}, 
                    {Hue[0.2244393090199423, 1, 1], LineBox[{61, 62}]}, 
                    {Hue[0.1274760151920486, 1, 1], LineBox[{62, 63}]}, 
                    {Hue[0.030720452166069608`, 1, 1], LineBox[{63, 64}]}, 
                    {Hue[0.9343594345237914, 1, 1], LineBox[{64, 65}]}, 
                    {Hue[0.8384755979716438, 1, 1], LineBox[{65, 66}]}, 
                    {Hue[0.7428164098352281, 1, 1], LineBox[{66, 67}]}, 
                    {Hue[0.6473269281698956, 1, 1], LineBox[{67, 68}]}, 
                    {Hue[0.5521603287832787, 1, 1], LineBox[{68, 69}]}, 
                    {Hue[0.4571550579953574, 1, 1], LineBox[{69, 70}]}, 
                    {Hue[0.36236152697481405`, 1, 1], LineBox[{70, 71}]}, 
                    {Hue[0.2676557170894482, 1, 1], LineBox[{71, 72}]}, 
                    {Hue[0.17322328832724745`, 1, 1], LineBox[{72, 73}]}, 
                    {Hue[0.07913979099701683, 1, 1], LineBox[{73, 74}]}, 
                    {Hue[0.985264744242385, 1, 1], LineBox[{74, 75}]}, 
                    {Hue[0.8916843385002036, 1, 1], LineBox[{75, 76}]}, 
                    {Hue[0.7982718205710239, 1, 1], LineBox[{76, 77}]}, 
                    {Hue[0.7050922404061961, 1, 1], LineBox[{77, 78}]}, 
                    {Hue[0.6121218907259338, 1, 1], LineBox[{78, 79}]}, 
                    {Hue[0.5193234039088777, 1, 1], LineBox[{79, 80}]}, 
                    {Hue[0.42677303315299064`, 1, 1], LineBox[{80, 81}]}, 
                    {Hue[0.3344796073799292, 1, 1], LineBox[{81, 82}]}, 
                    {Hue[0.24245262399604153`, 1, 1], LineBox[{82, 83}]}, 
                    {Hue[0.15061363241763906`, 1, 1], LineBox[{83, 84}]}, 
                    {Hue[0.05892932224611956, 1, 1], LineBox[{84, 85}]}, 
                    {Hue[0.9673846491267268, 1, 1], LineBox[{85, 86}]}, 
                    {Hue[0.8760413632447122, 1, 1], LineBox[{86, 87}]}, 
                    {Hue[0.7849367791857623, 1, 1], LineBox[{87, 88}]}, 
                    {Hue[0.6941299685528552, 1, 1], LineBox[{88, 89}]}, 
                    {Hue[0.6035794489207947, 1, 1], LineBox[{89, 90}]}, 
                    {Hue[0.5132120685336232, 1, 1], LineBox[{90, 91}]}, 
                    {Hue[0.42295124545510054`, 1, 1], LineBox[{91, 92}]}, 
                    {Hue[0.33277204168311014`, 1, 1], LineBox[{92, 93}]}, 
                    {Hue[0.2427622335008183, 1, 1], LineBox[{93, 94}]}, 
                    {Hue[0.15303822097131786`, 1, 1], LineBox[{94, 95}]}, 
                    {Hue[0.06368438721457334, 1, 1], LineBox[{95, 96}]}, 
                    {Hue[0.9746626610273306, 1, 1], LineBox[{96, 97}]}, 
                    {Hue[0.8858311772112987, 1, 1], LineBox[{97, 98}]}, 
                    {Hue[0.7970390083799735, 1, 1], LineBox[{98, 99}]}, 
                    {Hue[0.7082714984768756, 1, 1], LineBox[{99, 100}]}, 
                    {Hue[0.6196841994470936, 1, 1], LineBox[{100, 101}]}, 
                    {Hue[0.5315215505517608, 1, 1], LineBox[{101, 102}]}, 
                    {Hue[0.44394494742856366`, 1, 1], LineBox[{102, 103}]}, 
                    {Hue[0.35688764850602844`, 1, 1], LineBox[{103, 104}]}, 
                    {Hue[0.27009555087369436`, 1, 1], LineBox[{104, 105}]}, 
                    {Hue[0.18331540575024727`, 1, 1], LineBox[{105, 106}]}, 
                    {Hue[0.09654267138370616, 1, 1], LineBox[{106, 107}]}, 
                    {Hue[0.010094811765891522`, 1, 1], LineBox[{107, 108}]}, 
                    {Hue[0.9244607391604271, 1, 1], LineBox[{108, 109}]}, 
                    {Hue[0.8399492499879724, 1, 1], LineBox[{109, 110}]}, 
                    {Hue[0.7564654779058759, 1, 1], LineBox[{110, 111}]}, 
                    {Hue[0.6735803972243876, 1, 1], LineBox[{111, 112}]}, 
                    {Hue[0.5908683135133599, 1, 1], LineBox[{112, 113}]}, 
                    {Hue[0.5083034172256362, 1, 1], LineBox[{113, 114}]}, 
                    {Hue[0.4264514587862894, 1, 1], LineBox[{114, 115}]}, 
                    {Hue[0.3462206847563066, 1, 1], LineBox[{115, 116}]}, 
                    {Hue[0.26831235013189686`, 1, 1], LineBox[{116, 117}]}, 
                    {Hue[0.19281231771676, 1, 1], LineBox[{117, 118}]}, 
                    {Hue[0.11921971156371028`, 1, 1], LineBox[{118, 119}]}, 
                    {Hue[0.0468063636139859, 1, 1], LineBox[{119, 120}]}, 
                    {Hue[0.975109336835882, 1, 1], LineBox[{120, 121}]}, 
                    {Hue[0.90435059561364, 1, 1], LineBox[{121, 122}]}, 
                    {Hue[0.8355467810001385, 1, 1], LineBox[{122, 123}]}, 
                    {Hue[0.7701334240767309, 1, 1], LineBox[{123, 124}]}, 
                    {Hue[0.7093276744535297, 1, 1], LineBox[{124, 125}]}, 
                    {Hue[0.6537249557854934, 1, 1], LineBox[{125, 126}]}, 
                    {Hue[0.6033177014107454, 1, 1], LineBox[{126, 127}]}, 
                    {Hue[0.5577557908523973, 1, 1], LineBox[{127, 128}]}, 
                    {Hue[0.5165809575936386, 1, 1], LineBox[{128, 129}]}, 
                    {Hue[0.47936904837080296`, 1, 1], LineBox[{129, 130}]}, 
                    {Hue[0.44578257043933917`, 1, 1], LineBox[{130, 131}]}, 
                    {Hue[0.41557954902743466`, 1, 1], LineBox[{131, 132}]}, 
                    {Hue[0.3885948026145062, 1, 1], LineBox[{132, 133}]}, 
                    {Hue[0.36472011513852276`, 1, 1], LineBox[{133, 134}]}, 
                    {Hue[0.3438849218716845, 1, 1], LineBox[{134, 135}]}, 
                    {Hue[0.32603800153585283`, 1, 1], LineBox[{135, 136}]}, 
                    {Hue[0.3111376031158137, 1, 1], LineBox[{136, 137}]}, 
                    {Hue[0.2991410864394776, 1, 1], LineBox[{137, 138}]}, 
                    {Hue[0.2900012220807621, 1, 1], LineBox[{138, 139}]}, 
                    {Hue[0.2836643128481897, 1, 1], LineBox[{139, 140}]}, 
                    {Hue[0.2800696616453035, 1, 1], LineBox[{140, 141}]}, 
                    {Hue[0.2791497176725532, 1, 1], LineBox[{141, 142}]}, 
                    {Hue[0.2808316877825547, 1, 1], LineBox[{142, 143}]}, 
                    {Hue[0.28503905898740767`, 1, 1], LineBox[{143, 144}]}, 
                    {Hue[0.2916934185745679, 1, 1], LineBox[{144, 145}]}, 
                    {Hue[0.30071520604920704`, 1, 1], LineBox[{145, 146}]}, 
                    {Hue[0.31202487261304357`, 1, 1], LineBox[{146, 147}]}, 
                    {Hue[0.3255441697020442, 1, 1], LineBox[{147, 148}]}, 
                    {Hue[0.3411962371961555, 1, 1], LineBox[{148, 149}]}, 
                    {Hue[0.3589074694876606, 1, 1], LineBox[{149, 150}]}, 
                    {Hue[0.3786067400687169, 1, 1], LineBox[{150, 151}]}, 
                    {Hue[0.4002252438807113, 1, 1], LineBox[{151, 152}]}, 
                    {Hue[0.4236980743800631, 1, 1], LineBox[{152, 153}]}, 
                    {Hue[0.4489635507509118, 1, 1], LineBox[{153, 154}]}, 
                    {Hue[0.47596293667158696`, 1, 1], LineBox[{154, 155}]}, 
                    {Hue[0.5046413799058753, 1, 1], LineBox[{155, 156}]}, 
                    {Hue[0.5349458661619949, 1, 1], LineBox[{156, 157}]}, 
                    {Hue[0.5668267075469183, 1, 1], LineBox[{157, 158}]}, 
                    {Hue[0.6002383395664004, 1, 1], LineBox[{158, 159}]}, 
                    {Hue[0.6351364442922158, 1, 1], LineBox[{159, 160}]}, 
                    {Hue[0.6714794781732603, 1, 1], LineBox[{160, 161}]}, 
                    {Hue[0.7092297851685113, 1, 1], LineBox[{161, 162}]}, 
                    {Hue[0.7483498438618739, 1, 1], LineBox[{162, 163}]}, 
                    {Hue[0.7888059216275258, 1, 1], LineBox[{163, 164}]}, 
                    {Hue[0.8305658029377244, 1, 1], LineBox[{164, 165}]}, 
                    {Hue[0.8735985589444489, 1, 1], LineBox[{165, 166}]}, 
                    {Hue[0.9178756154733176, 1, 1], LineBox[{166, 167}]}, 
                    {Hue[0.9633697925167194, 1, 1], LineBox[{167, 168}]}, 
                    {Hue[0.010055769432912077`, 1, 1], LineBox[{168, 169}]}, 
                    {Hue[0.05790905682561934, 1, 1], LineBox[{169, 170}]}, 
                    {Hue[0.1069068241608519, 1, 1], LineBox[{170, 171}]}, 
                    {Hue[0.1570279699615826, 1, 1], LineBox[{171, 172}]}, 
                    {Hue[0.20825197197012807`, 1, 1], LineBox[{172, 173}]}, 
                    {Hue[0.2605589059274433, 1, 1], LineBox[{173, 174}]}, 
                    {Hue[0.31393019394989297`, 1, 1], LineBox[{174, 175}]}, 
                    {Hue[0.36834904422164405`, 1, 1], LineBox[{175, 176}]}, 
                    {Hue[0.4237981000610914, 1, 1], LineBox[{176, 177}]}, 
                    {Hue[0.480262570763982, 1, 1], LineBox[{177, 178}]}, 
                    {Hue[0.5377260144795512, 1, 1], LineBox[{178, 179}]}, 
                    {Hue[0.5961750384728735, 1, 1], LineBox[{179, 180}]}, 
                    {Hue[0.6555958067322936, 1, 1], LineBox[{180, 181}]}, 
                    {Hue[0.7159748673000743, 1, 1], LineBox[{181, 182}]}, 
                    {Hue[0.7773000790053467, 1, 1], LineBox[{182, 183}]}, 
                    {Hue[0.8395593248056266, 1, 1], LineBox[{183, 184}]}, 
                    {Hue[0.9027412896333733, 1, 1], LineBox[{184, 185}]}, 
                    {Hue[0.9668345230053836, 1, 1], LineBox[{185, 186}]}, 
                    {Hue[0.03182905417736379, 1, 1], LineBox[{186, 187}]}, 
                    {Hue[0.09771495862869105, 1, 1], LineBox[{187, 188}]}, 
                    {Hue[0.1644804683063499, 1, 1], LineBox[{188, 189}]}, 
                    {Hue[0.232118474204226, 1, 1], LineBox[{189, 190}]}, 
                    {Hue[0.3006190916938448, 1, 1], LineBox[{190, 191}]}, 
                    {Hue[0.36997412177509265`, 1, 1], LineBox[{191, 192}]}, 
                    {Hue[0.44017466975917585`, 1, 1], LineBox[{192, 193}]}, 
                    {Hue[0.5112124994864206, 1, 1], LineBox[{193, 194}]}, 
                    {Hue[0.5830804527288551, 1, 1], LineBox[{194, 195}]}, 
                    {Hue[0.6557702517243329, 1, 1], LineBox[{195, 196}]}, 
                    {Hue[0.7292765966264289, 1, 1], LineBox[{196, 197}]}, 
                    {Hue[0.8035895255914776, 1, 1], LineBox[{197, 198}]}, 
                    {Hue[0.878701965966677, 1, 1], LineBox[{198, 199}]}, 
                    {Hue[0.95461010302941, 1, 1], LineBox[{199, 200}]}, 
                    {Hue[0.03130808683880092, 1, 1], LineBox[{200, 201}]}, 
                    {Hue[0.10878523846407365`, 1, 1], LineBox[{201, 202}]}, 
                    {Hue[0.187036929367527, 1, 1], LineBox[{202, 203}]}, 
                    {Hue[0.26606114164798506`, 1, 1], LineBox[{203, 204}]}, 
                    {Hue[0.3458479697912301, 1, 1], LineBox[{204, 205}]}, 
                    {Hue[0.42639267303198874`, 1, 1], LineBox[{205, 206}]}, 
                    {Hue[0.5076933544748576, 1, 1], LineBox[{206, 207}]}, 
                    {Hue[0.5897379334740892, 1, 1], LineBox[{207, 208}]}, 
                    {Hue[0.6725241340879914, 1, 1], LineBox[{208, 209}]}, 
                    {Hue[0.7560528613554944, 1, 1], LineBox[{209, 210}]}, 
                    {Hue[0.8403144520020706, 1, 1], LineBox[{210, 211}]}, 
                    {Hue[0.9253050137763409, 1, 1], LineBox[{211, 212}]}, 
                    {Hue[0.011017846082229929`, 1, 1], LineBox[{212, 213}]}, 
                    {Hue[0.09744381217502465, 1, 1], LineBox[{213, 214}]}, 
                    {Hue[0.18458837476421924`, 1, 1], LineBox[{214, 215}]}, 
                    {Hue[0.2724509652327783, 1, 1], LineBox[{215, 216}]}, 
                    {Hue[0.3610067027049437, 1, 1], LineBox[{216, 217}]}, 
                    {Hue[0.4502651974747799, 1, 1], LineBox[{217, 218}]}, 
                    {Hue[0.5402360248028113, 1, 1], LineBox[{218, 219}]}, 
                    {Hue[0.6308979533152481, 1, 1], LineBox[{219, 220}]}, 
                    {Hue[0.7222476450207425, 1, 1], LineBox[{220, 221}]}, 
                    {Hue[0.8142704033445503, 1, 1], LineBox[{221, 222}]}, 
                    {Hue[0.9069734465940373, 1, 1], LineBox[{222, 223}]}, 
                    {Hue[0.0003751091215660667, 1, 1], LineBox[{223, 224}]}, 
                    {Hue[0.09444859931788216, 1, 1], LineBox[{224, 225}]}, 
                    {Hue[0.18919701920563461`, 1, 1], LineBox[{225, 226}]}, 
                    {Hue[0.2846160837458422, 1, 1], LineBox[{226, 227}]}, 
                    {Hue[0.3806721820941903, 1, 1], LineBox[{227, 228}]}, 
                    {Hue[0.4773855325816415, 1, 1], LineBox[{228, 229}]}, 
                    {Hue[0.5747717077904234, 1, 1], LineBox[{229, 230}]}, 
                    {Hue[0.6728222034045579, 1, 1], LineBox[{230, 231}]}, 
                    {Hue[0.7715478143526769, 1, 1], LineBox[{231, 232}]}, 
                    {Hue[0.8708970154927892, 1, 1], LineBox[{232, 233}]}, 
                    {Hue[0.9708163161382002, 1, 1], LineBox[{233, 234}]}, 
                    {Hue[0.07139599464877043, 1, 1], LineBox[{234, 235}]}, 
                    {Hue[0.172692594788118, 1, 1], LineBox[{235, 236}]}, 
                    {Hue[0.2746347559316637, 1, 1], LineBox[{236, 237}]}, 
                    {Hue[0.37720554526915917`, 1, 1], LineBox[{237, 238}]}, 
                    {Hue[0.4803521818452014, 1, 1], LineBox[{238, 239}]}, 
                    {Hue[0.5839602695737227, 1, 1], LineBox[{239, 240}]}, 
                    {Hue[0.6881708094172927, 1, 1], LineBox[{240, 241}]}, 
                    {Hue[0.7932710636593592, 1, 1], LineBox[{241, 242}]}, 
                    {Hue[0.8990904369082359, 1, 1], LineBox[{242, 243}]}, 
                    {Hue[0.005048224852851583, 1, 1], LineBox[{243, 244}]}, 
                    {Hue[0.1114275878230381, 1, 1], LineBox[{244, 245}]}, 
                    {Hue[0.2193610331407116, 1, 1], LineBox[{245, 246}]}, 
                    {Hue[0.3283303659856307, 1, 1], LineBox[{246, 247}]}, 
                    {Hue[0.4367141006325967, 1, 1], LineBox[{247, 248}]}, 
                    {Hue[0.5453967024411587, 1, 1], LineBox[{248, 249}]}, 
                    {Hue[0.6557591931182715, 1, 1], LineBox[{249, 250}]}, 
                    {Hue[0.7662084995133627, 1, 1], LineBox[{250, 251}]}}}},
                    VertexColors->{
                    Hue[0.03996625933509463, 1, 1], 
                    Hue[0.013716043603985754`, 1, 1], 
                    Hue[0.931478219924176, 1, 1], 
                    Hue[0.7933295525960278, 1, 1], 
                    Hue[0.671505196618513, 1, 1], 
                    Hue[0.48600936817024304`, 1, 1], 
                    Hue[0.4514066116504211, 1, 1], 
                    Hue[0.41317640143488066`, 1, 1], 
                    Hue[0.1728699714822034, 1, 1], 
                    Hue[0.16149973504533702`, 1, 1], 
                    Hue[0.014380992997808584`, 1, 1], 
                    Hue[0.8384753595018941, 1, 1], 
                    Hue[0.8680556074999903, 1, 1], 
                    Hue[0.7534222753972779, 1, 1], 
                    Hue[0.6098627904806089, 1, 1], 
                    Hue[0.6284046995572222, 1, 1], 
                    Hue[0.5287100058968992, 1, 1], 
                    Hue[0.3429159109541728, 1, 1], 
                    Hue[0.31298068804368645`, 1, 1], 
                    Hue[0.03115863318914752, 1, 1], 
                    Hue[0.8085133232198168, 1, 1], 
                    Hue[0.7785666480644399, 1, 1], 
                    Hue[0.8813213024321782, 1, 1], 
                    Hue[0.6921781949154409, 1, 1], 
                    Hue[0.6809606252489155, 1, 1], 
                    Hue[0.503465470130599, 1, 1], 
                    Hue[0.34795262488670103`, 1, 1], 
                    Hue[0.515525312724336, 1, 1], 
                    Hue[0.29980886210904617`, 1, 1], 
                    Hue[0.18072515446336535`, 1, 1], 
                    Hue[0.10297890719872621`, 1, 1], 
                    Hue[0.0023782259479396594`, 1, 1], 
                    Hue[0.914852704835432, 1, 1], 
                    Hue[0.8537396651102496, 1, 1], 
                    Hue[0.7448026676434731, 1, 1], 
                    Hue[0.6777817332505758, 1, 1], 
                    Hue[0.5860571244254931, 1, 1], 
                    Hue[0.4871012967944014, 1, 1], 
                    Hue[0.42687764741931494`, 1, 1], 
                    Hue[0.30671059341868573`, 1, 1], 
                    Hue[0.20522452942419225`, 1, 1], 
                    Hue[0.10111217553275076`, 1, 1], 
                    Hue[0.00422564743866865, 1, 1], 
                    Hue[0.9028465739827174, 1, 1], 
                    Hue[0.8018316233387496, 1, 1], 
                    Hue[0.6960993570726182, 1, 1], 
                    Hue[0.6037710348526477, 1, 1], 
                    Hue[0.5044937932940825, 1, 1], 
                    Hue[0.405547663514985, 1, 1], 
                    Hue[0.3059439158901264, 1, 1], 
                    Hue[0.20567206969598656`, 1, 1], 
                    Hue[0.10677120539249679`, 1, 1], 
                    Hue[0.007189722064902663, 1, 1], 
                    Hue[0.9072237591440739, 1, 1], 
                    Hue[0.8089701107334102, 1, 1], 
                    Hue[0.7109228185973671, 1, 1], 
                    Hue[0.6136604440188916, 1, 1], 
                    Hue[0.5156829678350188, 1, 1], 
                    Hue[0.418564420178921, 1, 1], 
                    Hue[0.32190746830302974`, 1, 1], 
                    Hue[0.2244393090199423, 1, 1], 
                    Hue[0.1274760151920486, 1, 1], 
                    Hue[0.030720452166069608`, 1, 1], 
                    Hue[0.9343594345237914, 1, 1], 
                    Hue[0.8384755979716438, 1, 1], 
                    Hue[0.7428164098352281, 1, 1], 
                    Hue[0.6473269281698956, 1, 1], 
                    Hue[0.5521603287832787, 1, 1], 
                    Hue[0.4571550579953574, 1, 1], 
                    Hue[0.36236152697481405`, 1, 1], 
                    Hue[0.2676557170894482, 1, 1], 
                    Hue[0.17322328832724745`, 1, 1], 
                    Hue[0.07913979099701683, 1, 1], 
                    Hue[0.985264744242385, 1, 1], 
                    Hue[0.8916843385002036, 1, 1], 
                    Hue[0.7982718205710239, 1, 1], 
                    Hue[0.7050922404061961, 1, 1], 
                    Hue[0.6121218907259338, 1, 1], 
                    Hue[0.5193234039088777, 1, 1], 
                    Hue[0.42677303315299064`, 1, 1], 
                    Hue[0.3344796073799292, 1, 1], 
                    Hue[0.24245262399604153`, 1, 1], 
                    Hue[0.15061363241763906`, 1, 1], 
                    Hue[0.05892932224611956, 1, 1], 
                    Hue[0.9673846491267268, 1, 1], 
                    Hue[0.8760413632447122, 1, 1], 
                    Hue[0.7849367791857623, 1, 1], 
                    Hue[0.6941299685528552, 1, 1], 
                    Hue[0.6035794489207947, 1, 1], 
                    Hue[0.5132120685336232, 1, 1], 
                    Hue[0.42295124545510054`, 1, 1], 
                    Hue[0.33277204168311014`, 1, 1], 
                    Hue[0.2427622335008183, 1, 1], 
                    Hue[0.15303822097131786`, 1, 1], 
                    Hue[0.06368438721457334, 1, 1], 
                    Hue[0.9746626610273306, 1, 1], 
                    Hue[0.8858311772112987, 1, 1], 
                    Hue[0.7970390083799735, 1, 1], 
                    Hue[0.7082714984768756, 1, 1], 
                    Hue[0.6196841994470936, 1, 1], 
                    Hue[0.5315215505517608, 1, 1], 
                    Hue[0.44394494742856366`, 1, 1], 
                    Hue[0.35688764850602844`, 1, 1], 
                    Hue[0.27009555087369436`, 1, 1], 
                    Hue[0.18331540575024727`, 1, 1], 
                    Hue[0.09654267138370616, 1, 1], 
                    Hue[0.010094811765891522`, 1, 1], 
                    Hue[0.9244607391604271, 1, 1], 
                    Hue[0.8399492499879724, 1, 1], 
                    Hue[0.7564654779058759, 1, 1], 
                    Hue[0.6735803972243876, 1, 1], 
                    Hue[0.5908683135133599, 1, 1], 
                    Hue[0.5083034172256362, 1, 1], 
                    Hue[0.4264514587862894, 1, 1], 
                    Hue[0.3462206847563066, 1, 1], 
                    Hue[0.26831235013189686`, 1, 1], 
                    Hue[0.19281231771676, 1, 1], 
                    Hue[0.11921971156371028`, 1, 1], 
                    Hue[0.0468063636139859, 1, 1], 
                    Hue[0.975109336835882, 1, 1], 
                    Hue[0.90435059561364, 1, 1], 
                    Hue[0.8355467810001385, 1, 1], 
                    Hue[0.7701334240767309, 1, 1], 
                    Hue[0.7093276744535297, 1, 1], 
                    Hue[0.6537249557854934, 1, 1], 
                    Hue[0.6033177014107454, 1, 1], 
                    Hue[0.5577557908523973, 1, 1], 
                    Hue[0.5165809575936386, 1, 1], 
                    Hue[0.47936904837080296`, 1, 1], 
                    Hue[0.44578257043933917`, 1, 1], 
                    Hue[0.41557954902743466`, 1, 1], 
                    Hue[0.3885948026145062, 1, 1], 
                    Hue[0.36472011513852276`, 1, 1], 
                    Hue[0.3438849218716845, 1, 1], 
                    Hue[0.32603800153585283`, 1, 1], 
                    Hue[0.3111376031158137, 1, 1], 
                    Hue[0.2991410864394776, 1, 1], 
                    Hue[0.2900012220807621, 1, 1], 
                    Hue[0.2836643128481897, 1, 1], 
                    Hue[0.2800696616453035, 1, 1], 
                    Hue[0.2791497176725532, 1, 1], 
                    Hue[0.2808316877825547, 1, 1], 
                    Hue[0.28503905898740767`, 1, 1], 
                    Hue[0.2916934185745679, 1, 1], 
                    Hue[0.30071520604920704`, 1, 1], 
                    Hue[0.31202487261304357`, 1, 1], 
                    Hue[0.3255441697020442, 1, 1], 
                    Hue[0.3411962371961555, 1, 1], 
                    Hue[0.3589074694876606, 1, 1], 
                    Hue[0.3786067400687169, 1, 1], 
                    Hue[0.4002252438807113, 1, 1], 
                    Hue[0.4236980743800631, 1, 1], 
                    Hue[0.4489635507509118, 1, 1], 
                    Hue[0.47596293667158696`, 1, 1], 
                    Hue[0.5046413799058753, 1, 1], 
                    Hue[0.5349458661619949, 1, 1], 
                    Hue[0.5668267075469183, 1, 1], 
                    Hue[0.6002383395664004, 1, 1], 
                    Hue[0.6351364442922158, 1, 1], 
                    Hue[0.6714794781732603, 1, 1], 
                    Hue[0.7092297851685113, 1, 1], 
                    Hue[0.7483498438618739, 1, 1], 
                    Hue[0.7888059216275258, 1, 1], 
                    Hue[0.8305658029377244, 1, 1], 
                    Hue[0.8735985589444489, 1, 1], 
                    Hue[0.9178756154733176, 1, 1], 
                    Hue[0.9633697925167194, 1, 1], 
                    Hue[0.010055769432912077`, 1, 1], 
                    Hue[0.05790905682561934, 1, 1], 
                    Hue[0.1069068241608519, 1, 1], 
                    Hue[0.1570279699615826, 1, 1], 
                    Hue[0.20825197197012807`, 1, 1], 
                    Hue[0.2605589059274433, 1, 1], 
                    Hue[0.31393019394989297`, 1, 1], 
                    Hue[0.36834904422164405`, 1, 1], 
                    Hue[0.4237981000610914, 1, 1], 
                    Hue[0.480262570763982, 1, 1], 
                    Hue[0.5377260144795512, 1, 1], 
                    Hue[0.5961750384728735, 1, 1], 
                    Hue[0.6555958067322936, 1, 1], 
                    Hue[0.7159748673000743, 1, 1], 
                    Hue[0.7773000790053467, 1, 1], 
                    Hue[0.8395593248056266, 1, 1], 
                    Hue[0.9027412896333733, 1, 1], 
                    Hue[0.9668345230053836, 1, 1], 
                    Hue[0.03182905417736379, 1, 1], 
                    Hue[0.09771495862869105, 1, 1], 
                    Hue[0.1644804683063499, 1, 1], 
                    Hue[0.232118474204226, 1, 1], 
                    Hue[0.3006190916938448, 1, 1], 
                    Hue[0.36997412177509265`, 1, 1], 
                    Hue[0.44017466975917585`, 1, 1], 
                    Hue[0.5112124994864206, 1, 1], 
                    Hue[0.5830804527288551, 1, 1], 
                    Hue[0.6557702517243329, 1, 1], 
                    Hue[0.7292765966264289, 1, 1], 
                    Hue[0.8035895255914776, 1, 1], 
                    Hue[0.878701965966677, 1, 1], 
                    Hue[0.95461010302941, 1, 1], 
                    Hue[0.03130808683880092, 1, 1], 
                    Hue[0.10878523846407365`, 1, 1], 
                    Hue[0.187036929367527, 1, 1], 
                    Hue[0.26606114164798506`, 1, 1], 
                    Hue[0.3458479697912301, 1, 1], 
                    Hue[0.42639267303198874`, 1, 1], 
                    Hue[0.5076933544748576, 1, 1], 
                    Hue[0.5897379334740892, 1, 1], 
                    Hue[0.6725241340879914, 1, 1], 
                    Hue[0.7560528613554944, 1, 1], 
                    Hue[0.8403144520020706, 1, 1], 
                    Hue[0.9253050137763409, 1, 1], 
                    Hue[0.011017846082229929`, 1, 1], 
                    Hue[0.09744381217502465, 1, 1], 
                    Hue[0.18458837476421924`, 1, 1], 
                    Hue[0.2724509652327783, 1, 1], 
                    Hue[0.3610067027049437, 1, 1], 
                    Hue[0.4502651974747799, 1, 1], 
                    Hue[0.5402360248028113, 1, 1], 
                    Hue[0.6308979533152481, 1, 1], 
                    Hue[0.7222476450207425, 1, 1], 
                    Hue[0.8142704033445503, 1, 1], 
                    Hue[0.9069734465940373, 1, 1], 
                    Hue[0.0003751091215660667, 1, 1], 
                    Hue[0.09444859931788216, 1, 1], 
                    Hue[0.18919701920563461`, 1, 1], 
                    Hue[0.2846160837458422, 1, 1], 
                    Hue[0.3806721820941903, 1, 1], 
                    Hue[0.4773855325816415, 1, 1], 
                    Hue[0.5747717077904234, 1, 1], 
                    Hue[0.6728222034045579, 1, 1], 
                    Hue[0.7715478143526769, 1, 1], 
                    Hue[0.8708970154927892, 1, 1], 
                    Hue[0.9708163161382002, 1, 1], 
                    Hue[0.07139599464877043, 1, 1], 
                    Hue[0.172692594788118, 1, 1], 
                    Hue[0.2746347559316637, 1, 1], 
                    Hue[0.37720554526915917`, 1, 1], 
                    Hue[0.4803521818452014, 1, 1], 
                    Hue[0.5839602695737227, 1, 1], 
                    Hue[0.6881708094172927, 1, 1], 
                    Hue[0.7932710636593592, 1, 1], 
                    Hue[0.8990904369082359, 1, 1], 
                    Hue[0.005048224852851583, 1, 1], 
                    Hue[0.1114275878230381, 1, 1], 
                    Hue[0.2193610331407116, 1, 1], 
                    Hue[0.3283303659856307, 1, 1], 
                    Hue[0.4367141006325967, 1, 1], 
                    Hue[0.5453967024411587, 1, 1], 
                    Hue[0.6557591931182715, 1, 1], 
                    Hue[0.7662084995133627, 1, 1], 
                    Hue[0.8765508443172676, 1, 1], 
                    Hue[0.03996625933509463, 1, 1], 
                    Hue[0.013716043603985754`, 1, 1], 
                    Hue[0.013716043603985754`, 1, 1], 
                    Hue[0.931478219924176, 1, 1], 
                    Hue[0.931478219924176, 1, 1], 
                    Hue[0.7933295525960278, 1, 1], 
                    Hue[0.7933295525960278, 1, 1], 
                    Hue[0.671505196618513, 1, 1], 
                    Hue[0.671505196618513, 1, 1], 
                    Hue[0.48600936817024304`, 1, 1], 
                    Hue[0.48600936817024304`, 1, 1], 
                    Hue[0.4514066116504211, 1, 1], 
                    Hue[0.4514066116504211, 1, 1], 
                    Hue[0.41317640143488066`, 1, 1], 
                    Hue[0.41317640143488066`, 1, 1], 
                    Hue[0.1728699714822034, 1, 1], 
                    Hue[0.1728699714822034, 1, 1], 
                    Hue[0.16149973504533702`, 1, 1], 
                    Hue[0.16149973504533702`, 1, 1], 
                    Hue[0.014380992997808584`, 1, 1], 
                    Hue[0.014380992997808584`, 1, 1], 
                    Hue[0.8384753595018941, 1, 1], 
                    Hue[0.8384753595018941, 1, 1], 
                    Hue[0.8680556074999903, 1, 1], 
                    Hue[0.8680556074999903, 1, 1], 
                    Hue[0.7534222753972779, 1, 1], 
                    Hue[0.7534222753972779, 1, 1], 
                    Hue[0.6098627904806089, 1, 1], 
                    Hue[0.6098627904806089, 1, 1], 
                    Hue[0.6284046995572222, 1, 1], 
                    Hue[0.6284046995572222, 1, 1], 
                    Hue[0.5287100058968992, 1, 1], 
                    Hue[0.5287100058968992, 1, 1], 
                    Hue[0.3429159109541728, 1, 1], 
                    Hue[0.3429159109541728, 1, 1], 
                    Hue[0.31298068804368645`, 1, 1], 
                    Hue[0.31298068804368645`, 1, 1], 
                    Hue[0.03115863318914752, 1, 1], 
                    Hue[0.03115863318914752, 1, 1], 
                    Hue[0.8085133232198168, 1, 1], 
                    Hue[0.8085133232198168, 1, 1], 
                    Hue[0.7785666480644399, 1, 1], 
                    Hue[0.7785666480644399, 1, 1], 
                    Hue[0.8813213024321782, 1, 1], 
                    Hue[0.8813213024321782, 1, 1], 
                    Hue[0.6921781949154409, 1, 1], 
                    Hue[0.6921781949154409, 1, 1], 
                    Hue[0.6809606252489155, 1, 1], 
                    Hue[0.6809606252489155, 1, 1], 
                    Hue[0.503465470130599, 1, 1], 
                    Hue[0.503465470130599, 1, 1], 
                    Hue[0.34795262488670103`, 1, 1], 
                    Hue[0.34795262488670103`, 1, 1], 
                    Hue[0.515525312724336, 1, 1], 
                    Hue[0.515525312724336, 1, 1], 
                    Hue[0.29980886210904617`, 1, 1], 
                    Hue[0.29980886210904617`, 1, 1], 
                    Hue[0.18072515446336535`, 1, 1], 
                    Hue[0.18072515446336535`, 1, 1], 
                    Hue[0.10297890719872621`, 1, 1], 
                    Hue[0.10297890719872621`, 1, 1], 
                    Hue[0.0023782259479396594`, 1, 1], 
                    Hue[0.0023782259479396594`, 1, 1], 
                    Hue[0.914852704835432, 1, 1], 
                    Hue[0.914852704835432, 1, 1], 
                    Hue[0.8537396651102496, 1, 1], 
                    Hue[0.8537396651102496, 1, 1], 
                    Hue[0.7448026676434731, 1, 1], 
                    Hue[0.7448026676434731, 1, 1], 
                    Hue[0.6777817332505758, 1, 1], 
                    Hue[0.6777817332505758, 1, 1], 
                    Hue[0.5860571244254931, 1, 1], 
                    Hue[0.5860571244254931, 1, 1], 
                    Hue[0.4871012967944014, 1, 1], 
                    Hue[0.4871012967944014, 1, 1], 
                    Hue[0.42687764741931494`, 1, 1], 
                    Hue[0.42687764741931494`, 1, 1], 
                    Hue[0.30671059341868573`, 1, 1], 
                    Hue[0.30671059341868573`, 1, 1], 
                    Hue[0.20522452942419225`, 1, 1], 
                    Hue[0.20522452942419225`, 1, 1], 
                    Hue[0.10111217553275076`, 1, 1], 
                    Hue[0.10111217553275076`, 1, 1], 
                    Hue[0.00422564743866865, 1, 1], 
                    Hue[0.00422564743866865, 1, 1], 
                    Hue[0.9028465739827174, 1, 1], 
                    Hue[0.9028465739827174, 1, 1], 
                    Hue[0.8018316233387496, 1, 1], 
                    Hue[0.8018316233387496, 1, 1], 
                    Hue[0.6960993570726182, 1, 1], 
                    Hue[0.6960993570726182, 1, 1], 
                    Hue[0.6037710348526477, 1, 1], 
                    Hue[0.6037710348526477, 1, 1], 
                    Hue[0.5044937932940825, 1, 1], 
                    Hue[0.5044937932940825, 1, 1], 
                    Hue[0.405547663514985, 1, 1], 
                    Hue[0.405547663514985, 1, 1], 
                    Hue[0.3059439158901264, 1, 1], 
                    Hue[0.3059439158901264, 1, 1], 
                    Hue[0.20567206969598656`, 1, 1], 
                    Hue[0.20567206969598656`, 1, 1], 
                    Hue[0.10677120539249679`, 1, 1], 
                    Hue[0.10677120539249679`, 1, 1], 
                    Hue[0.007189722064902663, 1, 1], 
                    Hue[0.007189722064902663, 1, 1], 
                    Hue[0.9072237591440739, 1, 1], 
                    Hue[0.9072237591440739, 1, 1], 
                    Hue[0.8089701107334102, 1, 1], 
                    Hue[0.8089701107334102, 1, 1], 
                    Hue[0.7109228185973671, 1, 1], 
                    Hue[0.7109228185973671, 1, 1], 
                    Hue[0.6136604440188916, 1, 1], 
                    Hue[0.6136604440188916, 1, 1], 
                    Hue[0.5156829678350188, 1, 1], 
                    Hue[0.5156829678350188, 1, 1], 
                    Hue[0.418564420178921, 1, 1], 
                    Hue[0.418564420178921, 1, 1], 
                    Hue[0.32190746830302974`, 1, 1], 
                    Hue[0.32190746830302974`, 1, 1], 
                    Hue[0.2244393090199423, 1, 1], 
                    Hue[0.2244393090199423, 1, 1], 
                    Hue[0.1274760151920486, 1, 1], 
                    Hue[0.1274760151920486, 1, 1], 
                    Hue[0.030720452166069608`, 1, 1], 
                    Hue[0.030720452166069608`, 1, 1], 
                    Hue[0.9343594345237914, 1, 1], 
                    Hue[0.9343594345237914, 1, 1], 
                    Hue[0.8384755979716438, 1, 1], 
                    Hue[0.8384755979716438, 1, 1], 
                    Hue[0.7428164098352281, 1, 1], 
                    Hue[0.7428164098352281, 1, 1], 
                    Hue[0.6473269281698956, 1, 1], 
                    Hue[0.6473269281698956, 1, 1], 
                    Hue[0.5521603287832787, 1, 1], 
                    Hue[0.5521603287832787, 1, 1], 
                    Hue[0.4571550579953574, 1, 1], 
                    Hue[0.4571550579953574, 1, 1], 
                    Hue[0.36236152697481405`, 1, 1], 
                    Hue[0.36236152697481405`, 1, 1], 
                    Hue[0.2676557170894482, 1, 1], 
                    Hue[0.2676557170894482, 1, 1], 
                    Hue[0.17322328832724745`, 1, 1], 
                    Hue[0.17322328832724745`, 1, 1], 
                    Hue[0.07913979099701683, 1, 1], 
                    Hue[0.07913979099701683, 1, 1], 
                    Hue[0.985264744242385, 1, 1], 
                    Hue[0.985264744242385, 1, 1], 
                    Hue[0.8916843385002036, 1, 1], 
                    Hue[0.8916843385002036, 1, 1], 
                    Hue[0.7982718205710239, 1, 1], 
                    Hue[0.7982718205710239, 1, 1], 
                    Hue[0.7050922404061961, 1, 1], 
                    Hue[0.7050922404061961, 1, 1], 
                    Hue[0.6121218907259338, 1, 1], 
                    Hue[0.6121218907259338, 1, 1], 
                    Hue[0.5193234039088777, 1, 1], 
                    Hue[0.5193234039088777, 1, 1], 
                    Hue[0.42677303315299064`, 1, 1], 
                    Hue[0.42677303315299064`, 1, 1], 
                    Hue[0.3344796073799292, 1, 1], 
                    Hue[0.3344796073799292, 1, 1], 
                    Hue[0.24245262399604153`, 1, 1], 
                    Hue[0.24245262399604153`, 1, 1], 
                    Hue[0.15061363241763906`, 1, 1], 
                    Hue[0.15061363241763906`, 1, 1], 
                    Hue[0.05892932224611956, 1, 1], 
                    Hue[0.05892932224611956, 1, 1], 
                    Hue[0.9673846491267268, 1, 1], 
                    Hue[0.9673846491267268, 1, 1], 
                    Hue[0.8760413632447122, 1, 1], 
                    Hue[0.8760413632447122, 1, 1], 
                    Hue[0.7849367791857623, 1, 1], 
                    Hue[0.7849367791857623, 1, 1], 
                    Hue[0.6941299685528552, 1, 1], 
                    Hue[0.6941299685528552, 1, 1], 
                    Hue[0.6035794489207947, 1, 1], 
                    Hue[0.6035794489207947, 1, 1], 
                    Hue[0.5132120685336232, 1, 1], 
                    Hue[0.5132120685336232, 1, 1], 
                    Hue[0.42295124545510054`, 1, 1], 
                    Hue[0.42295124545510054`, 1, 1], 
                    Hue[0.33277204168311014`, 1, 1], 
                    Hue[0.33277204168311014`, 1, 1], 
                    Hue[0.2427622335008183, 1, 1], 
                    Hue[0.2427622335008183, 1, 1], 
                    Hue[0.15303822097131786`, 1, 1], 
                    Hue[0.15303822097131786`, 1, 1], 
                    Hue[0.06368438721457334, 1, 1], 
                    Hue[0.06368438721457334, 1, 1], 
                    Hue[0.9746626610273306, 1, 1], 
                    Hue[0.9746626610273306, 1, 1], 
                    Hue[0.8858311772112987, 1, 1], 
                    Hue[0.8858311772112987, 1, 1], 
                    Hue[0.7970390083799735, 1, 1], 
                    Hue[0.7970390083799735, 1, 1], 
                    Hue[0.7082714984768756, 1, 1], 
                    Hue[0.7082714984768756, 1, 1], 
                    Hue[0.6196841994470936, 1, 1], 
                    Hue[0.6196841994470936, 1, 1], 
                    Hue[0.5315215505517608, 1, 1], 
                    Hue[0.5315215505517608, 1, 1], 
                    Hue[0.44394494742856366`, 1, 1], 
                    Hue[0.44394494742856366`, 1, 1], 
                    Hue[0.35688764850602844`, 1, 1], 
                    Hue[0.35688764850602844`, 1, 1], 
                    Hue[0.27009555087369436`, 1, 1], 
                    Hue[0.27009555087369436`, 1, 1], 
                    Hue[0.18331540575024727`, 1, 1], 
                    Hue[0.18331540575024727`, 1, 1], 
                    Hue[0.09654267138370616, 1, 1], 
                    Hue[0.09654267138370616, 1, 1], 
                    Hue[0.010094811765891522`, 1, 1], 
                    Hue[0.010094811765891522`, 1, 1], 
                    Hue[0.9244607391604271, 1, 1], 
                    Hue[0.9244607391604271, 1, 1], 
                    Hue[0.8399492499879724, 1, 1], 
                    Hue[0.8399492499879724, 1, 1], 
                    Hue[0.7564654779058759, 1, 1], 
                    Hue[0.7564654779058759, 1, 1], 
                    Hue[0.6735803972243876, 1, 1], 
                    Hue[0.6735803972243876, 1, 1], 
                    Hue[0.5908683135133599, 1, 1], 
                    Hue[0.5908683135133599, 1, 1], 
                    Hue[0.5083034172256362, 1, 1], 
                    Hue[0.5083034172256362, 1, 1], 
                    Hue[0.4264514587862894, 1, 1], 
                    Hue[0.4264514587862894, 1, 1], 
                    Hue[0.3462206847563066, 1, 1], 
                    Hue[0.3462206847563066, 1, 1], 
                    Hue[0.26831235013189686`, 1, 1], 
                    Hue[0.26831235013189686`, 1, 1], 
                    Hue[0.19281231771676, 1, 1], 
                    Hue[0.19281231771676, 1, 1], 
                    Hue[0.11921971156371028`, 1, 1], 
                    Hue[0.11921971156371028`, 1, 1], 
                    Hue[0.0468063636139859, 1, 1], 
                    Hue[0.0468063636139859, 1, 1], 
                    Hue[0.975109336835882, 1, 1], 
                    Hue[0.975109336835882, 1, 1], 
                    Hue[0.90435059561364, 1, 1], 
                    Hue[0.90435059561364, 1, 1], 
                    Hue[0.8355467810001385, 1, 1], 
                    Hue[0.8355467810001385, 1, 1], 
                    Hue[0.7701334240767309, 1, 1], 
                    Hue[0.7701334240767309, 1, 1], 
                    Hue[0.7093276744535297, 1, 1], 
                    Hue[0.7093276744535297, 1, 1], 
                    Hue[0.6537249557854934, 1, 1], 
                    Hue[0.6537249557854934, 1, 1], 
                    Hue[0.6033177014107454, 1, 1], 
                    Hue[0.6033177014107454, 1, 1], 
                    Hue[0.5577557908523973, 1, 1], 
                    Hue[0.5577557908523973, 1, 1], 
                    Hue[0.5165809575936386, 1, 1], 
                    Hue[0.5165809575936386, 1, 1], 
                    Hue[0.47936904837080296`, 1, 1], 
                    Hue[0.47936904837080296`, 1, 1], 
                    Hue[0.44578257043933917`, 1, 1], 
                    Hue[0.44578257043933917`, 1, 1], 
                    Hue[0.41557954902743466`, 1, 1], 
                    Hue[0.41557954902743466`, 1, 1], 
                    Hue[0.3885948026145062, 1, 1], 
                    Hue[0.3885948026145062, 1, 1], 
                    Hue[0.36472011513852276`, 1, 1], 
                    Hue[0.36472011513852276`, 1, 1], 
                    Hue[0.3438849218716845, 1, 1], 
                    Hue[0.3438849218716845, 1, 1], 
                    Hue[0.32603800153585283`, 1, 1], 
                    Hue[0.32603800153585283`, 1, 1], 
                    Hue[0.3111376031158137, 1, 1], 
                    Hue[0.3111376031158137, 1, 1], 
                    Hue[0.2991410864394776, 1, 1], 
                    Hue[0.2991410864394776, 1, 1], 
                    Hue[0.2900012220807621, 1, 1], 
                    Hue[0.2900012220807621, 1, 1], 
                    Hue[0.2836643128481897, 1, 1], 
                    Hue[0.2836643128481897, 1, 1], 
                    Hue[0.2800696616453035, 1, 1], 
                    Hue[0.2800696616453035, 1, 1], 
                    Hue[0.2791497176725532, 1, 1], 
                    Hue[0.2791497176725532, 1, 1], 
                    Hue[0.2808316877825547, 1, 1], 
                    Hue[0.2808316877825547, 1, 1], 
                    Hue[0.28503905898740767`, 1, 1], 
                    Hue[0.28503905898740767`, 1, 1], 
                    Hue[0.2916934185745679, 1, 1], 
                    Hue[0.2916934185745679, 1, 1], 
                    Hue[0.30071520604920704`, 1, 1], 
                    Hue[0.30071520604920704`, 1, 1], 
                    Hue[0.31202487261304357`, 1, 1], 
                    Hue[0.31202487261304357`, 1, 1], 
                    Hue[0.3255441697020442, 1, 1], 
                    Hue[0.3255441697020442, 1, 1], 
                    Hue[0.3411962371961555, 1, 1], 
                    Hue[0.3411962371961555, 1, 1], 
                    Hue[0.3589074694876606, 1, 1], 
                    Hue[0.3589074694876606, 1, 1], 
                    Hue[0.3786067400687169, 1, 1], 
                    Hue[0.3786067400687169, 1, 1], 
                    Hue[0.4002252438807113, 1, 1], 
                    Hue[0.4002252438807113, 1, 1], 
                    Hue[0.4236980743800631, 1, 1], 
                    Hue[0.4236980743800631, 1, 1], 
                    Hue[0.4489635507509118, 1, 1], 
                    Hue[0.4489635507509118, 1, 1], 
                    Hue[0.47596293667158696`, 1, 1], 
                    Hue[0.47596293667158696`, 1, 1], 
                    Hue[0.5046413799058753, 1, 1], 
                    Hue[0.5046413799058753, 1, 1], 
                    Hue[0.5349458661619949, 1, 1], 
                    Hue[0.5349458661619949, 1, 1], 
                    Hue[0.5668267075469183, 1, 1], 
                    Hue[0.5668267075469183, 1, 1], 
                    Hue[0.6002383395664004, 1, 1], 
                    Hue[0.6002383395664004, 1, 1], 
                    Hue[0.6351364442922158, 1, 1], 
                    Hue[0.6351364442922158, 1, 1], 
                    Hue[0.6714794781732603, 1, 1], 
                    Hue[0.6714794781732603, 1, 1], 
                    Hue[0.7092297851685113, 1, 1], 
                    Hue[0.7092297851685113, 1, 1], 
                    Hue[0.7483498438618739, 1, 1], 
                    Hue[0.7483498438618739, 1, 1], 
                    Hue[0.7888059216275258, 1, 1], 
                    Hue[0.7888059216275258, 1, 1], 
                    Hue[0.8305658029377244, 1, 1], 
                    Hue[0.8305658029377244, 1, 1], 
                    Hue[0.8735985589444489, 1, 1], 
                    Hue[0.8735985589444489, 1, 1], 
                    Hue[0.9178756154733176, 1, 1], 
                    Hue[0.9178756154733176, 1, 1], 
                    Hue[0.9633697925167194, 1, 1], 
                    Hue[0.9633697925167194, 1, 1], 
                    Hue[0.010055769432912077`, 1, 1], 
                    Hue[0.010055769432912077`, 1, 1], 
                    Hue[0.05790905682561934, 1, 1], 
                    Hue[0.05790905682561934, 1, 1], 
                    Hue[0.1069068241608519, 1, 1], 
                    Hue[0.1069068241608519, 1, 1], 
                    Hue[0.1570279699615826, 1, 1], 
                    Hue[0.1570279699615826, 1, 1], 
                    Hue[0.20825197197012807`, 1, 1], 
                    Hue[0.20825197197012807`, 1, 1], 
                    Hue[0.2605589059274433, 1, 1], 
                    Hue[0.2605589059274433, 1, 1], 
                    Hue[0.31393019394989297`, 1, 1], 
                    Hue[0.31393019394989297`, 1, 1], 
                    Hue[0.36834904422164405`, 1, 1], 
                    Hue[0.36834904422164405`, 1, 1], 
                    Hue[0.4237981000610914, 1, 1], 
                    Hue[0.4237981000610914, 1, 1], 
                    Hue[0.480262570763982, 1, 1], 
                    Hue[0.480262570763982, 1, 1], 
                    Hue[0.5377260144795512, 1, 1], 
                    Hue[0.5377260144795512, 1, 1], 
                    Hue[0.5961750384728735, 1, 1], 
                    Hue[0.5961750384728735, 1, 1], 
                    Hue[0.6555958067322936, 1, 1], 
                    Hue[0.6555958067322936, 1, 1], 
                    Hue[0.7159748673000743, 1, 1], 
                    Hue[0.7159748673000743, 1, 1], 
                    Hue[0.7773000790053467, 1, 1], 
                    Hue[0.7773000790053467, 1, 1], 
                    Hue[0.8395593248056266, 1, 1], 
                    Hue[0.8395593248056266, 1, 1], 
                    Hue[0.9027412896333733, 1, 1], 
                    Hue[0.9027412896333733, 1, 1], 
                    Hue[0.9668345230053836, 1, 1], 
                    Hue[0.9668345230053836, 1, 1], 
                    Hue[0.03182905417736379, 1, 1], 
                    Hue[0.03182905417736379, 1, 1], 
                    Hue[0.09771495862869105, 1, 1], 
                    Hue[0.09771495862869105, 1, 1], 
                    Hue[0.1644804683063499, 1, 1], 
                    Hue[0.1644804683063499, 1, 1], 
                    Hue[0.232118474204226, 1, 1], 
                    Hue[0.232118474204226, 1, 1], 
                    Hue[0.3006190916938448, 1, 1], 
                    Hue[0.3006190916938448, 1, 1], 
                    Hue[0.36997412177509265`, 1, 1], 
                    Hue[0.36997412177509265`, 1, 1], 
                    Hue[0.44017466975917585`, 1, 1], 
                    Hue[0.44017466975917585`, 1, 1], 
                    Hue[0.5112124994864206, 1, 1], 
                    Hue[0.5112124994864206, 1, 1], 
                    Hue[0.5830804527288551, 1, 1], 
                    Hue[0.5830804527288551, 1, 1], 
                    Hue[0.6557702517243329, 1, 1], 
                    Hue[0.6557702517243329, 1, 1], 
                    Hue[0.7292765966264289, 1, 1], 
                    Hue[0.7292765966264289, 1, 1], 
                    Hue[0.8035895255914776, 1, 1], 
                    Hue[0.8035895255914776, 1, 1], 
                    Hue[0.878701965966677, 1, 1], 
                    Hue[0.878701965966677, 1, 1], 
                    Hue[0.95461010302941, 1, 1], 
                    Hue[0.95461010302941, 1, 1], 
                    Hue[0.03130808683880092, 1, 1], 
                    Hue[0.03130808683880092, 1, 1], 
                    Hue[0.10878523846407365`, 1, 1], 
                    Hue[0.10878523846407365`, 1, 1], 
                    Hue[0.187036929367527, 1, 1], 
                    Hue[0.187036929367527, 1, 1], 
                    Hue[0.26606114164798506`, 1, 1], 
                    Hue[0.26606114164798506`, 1, 1], 
                    Hue[0.3458479697912301, 1, 1], 
                    Hue[0.3458479697912301, 1, 1], 
                    Hue[0.42639267303198874`, 1, 1], 
                    Hue[0.42639267303198874`, 1, 1], 
                    Hue[0.5076933544748576, 1, 1], 
                    Hue[0.5076933544748576, 1, 1], 
                    Hue[0.5897379334740892, 1, 1], 
                    Hue[0.5897379334740892, 1, 1], 
                    Hue[0.6725241340879914, 1, 1], 
                    Hue[0.6725241340879914, 1, 1], 
                    Hue[0.7560528613554944, 1, 1], 
                    Hue[0.7560528613554944, 1, 1], 
                    Hue[0.8403144520020706, 1, 1], 
                    Hue[0.8403144520020706, 1, 1], 
                    Hue[0.9253050137763409, 1, 1], 
                    Hue[0.9253050137763409, 1, 1], 
                    Hue[0.011017846082229929`, 1, 1], 
                    Hue[0.011017846082229929`, 1, 1], 
                    Hue[0.09744381217502465, 1, 1], 
                    Hue[0.09744381217502465, 1, 1], 
                    Hue[0.18458837476421924`, 1, 1], 
                    Hue[0.18458837476421924`, 1, 1], 
                    Hue[0.2724509652327783, 1, 1], 
                    Hue[0.2724509652327783, 1, 1], 
                    Hue[0.3610067027049437, 1, 1], 
                    Hue[0.3610067027049437, 1, 1], 
                    Hue[0.4502651974747799, 1, 1], 
                    Hue[0.4502651974747799, 1, 1], 
                    Hue[0.5402360248028113, 1, 1], 
                    Hue[0.5402360248028113, 1, 1], 
                    Hue[0.6308979533152481, 1, 1], 
                    Hue[0.6308979533152481, 1, 1], 
                    Hue[0.7222476450207425, 1, 1], 
                    Hue[0.7222476450207425, 1, 1], 
                    Hue[0.8142704033445503, 1, 1], 
                    Hue[0.8142704033445503, 1, 1], 
                    Hue[0.9069734465940373, 1, 1], 
                    Hue[0.9069734465940373, 1, 1], 
                    Hue[0.0003751091215660667, 1, 1], 
                    Hue[0.0003751091215660667, 1, 1], 
                    Hue[0.09444859931788216, 1, 1], 
                    Hue[0.09444859931788216, 1, 1], 
                    Hue[0.18919701920563461`, 1, 1], 
                    Hue[0.18919701920563461`, 1, 1], 
                    Hue[0.2846160837458422, 1, 1], 
                    Hue[0.2846160837458422, 1, 1], 
                    Hue[0.3806721820941903, 1, 1], 
                    Hue[0.3806721820941903, 1, 1], 
                    Hue[0.4773855325816415, 1, 1], 
                    Hue[0.4773855325816415, 1, 1], 
                    Hue[0.5747717077904234, 1, 1], 
                    Hue[0.5747717077904234, 1, 1], 
                    Hue[0.6728222034045579, 1, 1], 
                    Hue[0.6728222034045579, 1, 1], 
                    Hue[0.7715478143526769, 1, 1], 
                    Hue[0.7715478143526769, 1, 1], 
                    Hue[0.8708970154927892, 1, 1], 
                    Hue[0.8708970154927892, 1, 1], 
                    Hue[0.9708163161382002, 1, 1], 
                    Hue[0.9708163161382002, 1, 1], 
                    Hue[0.07139599464877043, 1, 1], 
                    Hue[0.07139599464877043, 1, 1], 
                    Hue[0.172692594788118, 1, 1], 
                    Hue[0.172692594788118, 1, 1], 
                    Hue[0.2746347559316637, 1, 1], 
                    Hue[0.2746347559316637, 1, 1], 
                    Hue[0.37720554526915917`, 1, 1], 
                    Hue[0.37720554526915917`, 1, 1], 
                    Hue[0.4803521818452014, 1, 1], 
                    Hue[0.4803521818452014, 1, 1], 
                    Hue[0.5839602695737227, 1, 1], 
                    Hue[0.5839602695737227, 1, 1], 
                    Hue[0.6881708094172927, 1, 1], 
                    Hue[0.6881708094172927, 1, 1], 
                    Hue[0.7932710636593592, 1, 1], 
                    Hue[0.7932710636593592, 1, 1], 
                    Hue[0.8990904369082359, 1, 1], 
                    Hue[0.8990904369082359, 1, 1], 
                    Hue[0.005048224852851583, 1, 1], 
                    Hue[0.005048224852851583, 1, 1], 
                    Hue[0.1114275878230381, 1, 1], 
                    Hue[0.1114275878230381, 1, 1], 
                    Hue[0.2193610331407116, 1, 1], 
                    Hue[0.2193610331407116, 1, 1], 
                    Hue[0.3283303659856307, 1, 1], 
                    Hue[0.3283303659856307, 1, 1], 
                    Hue[0.4367141006325967, 1, 1], 
                    Hue[0.4367141006325967, 1, 1], 
                    Hue[0.5453967024411587, 1, 1], 
                    Hue[0.5453967024411587, 1, 1], 
                    Hue[0.6557591931182715, 1, 1], 
                    Hue[0.6557591931182715, 1, 1], 
                    Hue[0.7662084995133627, 1, 1], 
                    Hue[0.7662084995133627, 1, 1], 
                    Hue[0.8765508443172676, 1, 1]}],
                   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
                   Axes->{True, None},
                   AxesOrigin->{0, 0},
                   
                   Epilog->LineBox[{{1, 4.774163964623409*^-7}, {
                    2, 7.568037735656116*^-7}, {3, 8.425183630226619*^-7}, {
                    4, 7.917231523205861*^-7}, {5, 8.554168651769534*^-7}, {
                    6, 2.672277370165971*^-7}, {7, 1.0040845588091148`*^-6}, {
                    8, 6.829271023376666*^-7}, {9, 7.03086746489622*^-7}, {
                    10, 7.435894236299269*^-7}, {11, 6.771128449083131*^-7}, {
                    12, 5.815877964078606*^-7}, {13, 8.202379833185497*^-7}, {
                    14, 5.937719042033464*^-7}, {15, 8.078166423174234*^-7}, {
                    16, 5.086819889358714*^-7}, {
                    17, 1.1878770347658353`*^-6}, {
                    18, 4.5767974221630423`*^-7}, {
                    19, 1.285767114195014*^-6}, {20, 9.987594815592835*^-7}, {
                    21, 5.216225831754935*^-7}, {
                    22, 2.7597912050209344`*^-7}, {
                    23, 3.381099420630693*^-7}, {24, 5.129014284753304*^-7}, {
                    25, 7.486717557597984*^-7}, {
                    26, 1.0194041304552456`*^-6}, {
                    27, 1.1199915232620164`*^-6}, {
                    28, 1.4729528050601566`*^-7}, {
                    29, 9.149084668926432*^-7}, {30, 6.790996535943909*^-7}, {
                    31, 1.0088465419129672`*^-6}, {
                    32, 1.2571103411588953`*^-6}, {
                    33, 1.6101247730073053`*^-6}, {
                    34, 1.6571629696646278`*^-6}, {
                    35, 2.257069457133729*^-6}, {
                    36, 2.7084090806485868`*^-6}, {
                    37, 2.8848308038545456`*^-6}, {
                    38, 3.8144957415019324`*^-6}, {
                    39, 4.139539260535415*^-6}, {40, 5.280530651613977*^-6}, {
                    41, 6.364055669957215*^-6}, {42, 7.431702353783643*^-6}, {
                    43, 9.162140016540454*^-6}, {
                    44, 0.000011392767771047028`}, {
                    45, 0.00001447310790630038}, {
                    46, 0.00001764347631662918}, {
                    47, 0.000020869685237656144`}, {
                    48, 0.00002566824990876318}, {
                    49, 0.00003112225832156994}, {
                    50, 0.00003744839707612007}, {
                    51, 0.00004465548565402208}, {
                    52, 0.00005395965578331769}, {
                    53, 0.0000651462060312024}, {
                    54, 0.00007749863683983881}, {
                    55, 0.00009315560511470502}, {
                    56, 0.00011139576418451982`}, {
                    57, 0.00013237136919133893`}, {
                    58, 0.00015811023723313907`}, {
                    59, 0.00018632942795518157`}, {
                    60, 0.00022112273953046554`}, {
                    61, 0.0002617510012985398}, {62, 0.0003080954313370296}, {
                    63, 0.00036302525968071866`}, {
                    64, 0.0004264547182322541}, {65, 0.0005003821649413357}, {
                    66, 0.0005873538216184944}, {67, 0.0006864116432464454}, {
                    68, 0.0008014674138896858}, {69, 0.0009333994867948699}, {
                    70, 0.0010847876822632763`}, {
                    71, 0.0012593227345309218`}, {72, 0.001458228594512158}, {
                    73, 0.0016859147516438064`}, {
                    74, 0.0019463420713989252`}, {
                    75, 0.0022425844949409057`}, {
                    76, 0.0025777013687523073`}, {
                    77, 0.0029580128365886494`}, {
                    78, 0.0033875670127808126`}, {79, 0.003874037458736444}, {
                    80, 0.0044227766864224135`}, {81, 0.005039860646506613}, {
                    82, 0.005730631198991554}, {83, 0.006502357545690735}, {
                    84, 0.007361882997114838}, {85, 0.008320421400344356}, {
                    86, 0.009389875448602624}, {87, 0.010581091734063677`}, {
                    88, 0.0119010820665137}, {89, 0.013353791144902543`}, {
                    90, 0.014945917078016499`}, {91, 0.016689261702392058`}, {
                    92, 0.01860538709566863}, {93, 0.020718611614680704`}, {
                    94, 0.02304453228252633}, {95, 0.025584912527564854`}, {
                    96, 0.028326367942955868`}, {97, 0.03126478541924108}, {
                    98, 0.03441944719033271}, {99, 0.03784125064069806}, {
                    100, 0.04158886136121419}, {101, 0.04568761779698456}, {
                    102, 0.05010135941548635}, {103, 0.05475780915237832}, {
                    104, 0.0596079890235699}, {105, 0.06470225013523107}, {
                    106, 0.07019202632746886}, {107, 0.07624640551664297}, {
                    108, 0.08290956559240623}, {109, 0.09003160896853278}, {
                    110, 0.0973476745313712}, {111, 0.10469178807119026`}, {
                    112, 0.11219896188055296`}, {113, 0.12032496238710254`}, {
                    114, 0.12960077353847177`}, {115, 0.14023838052557958`}, {
                    116, 0.1519029118363534}, {117, 0.16386240023452045`}, {
                    118, 0.17545841255185335`}, {119, 0.18661894340048335`}, {
                    120, 0.19811546100743763`}, {121, 0.21140514326324156`}, {
                    122, 0.2280214318631956}, {123, 0.24883856143769392`}, {
                    124, 0.27365480899730454`}, {125, 0.3013369856185038}, {
                    126, 0.3303064652291558}, {127, 0.35901634257470755`}, {
                    128, 0.38625267810842195`}, {129, 0.41122460759822177`}, {
                    130, 0.43355125980323717`}, {131, 0.453162031114551}, {
                    132, 0.47019803764867696`}, {133, 0.48491470269211906`}, {
                    134, 0.4976061062157364}, {135, 0.5085583260585054}, {
                    136, 0.518026485174305}, {137, 0.5262148396818539}, {
                    138, 0.5332818671277445}, {139, 0.5393454860076266}, {
                    140, 0.5444852770071663}, {141, 0.5487545918203182}, {
                    142, 0.5521857094414232}, {143, 0.5547959634375633}, {
                    144, 0.5565913203376384}, {145, 0.5575738932945108}, {
                    146, 0.5577419273135292}, {147, 0.5570914786882771}, {
                    148, 0.5556181975564911}, {149, 0.5533241726653495}, {
                    150, 0.5502100168343496}, {151, 0.5462833887290199}, {
                    152, 0.541552576757865}, {153, 0.5360299293130407}, {
                    154, 0.5297356037531146}, {155, 0.5226889068390108}, {
                    156, 0.5149168060814912}, {157, 0.506448587559631}, {
                    158, 0.4973173609167664}, {159, 0.4875612372675214}, {
                    160, 0.4772174953570008}, {161, 0.4663292306681738}, {
                    162, 0.45494021484118075`}, {163, 0.4430965697404628}, {
                    164, 0.4308475471281948}, {165, 0.4182400033512116}, {
                    166, 0.4053267858564404}, {167, 0.3921556043066159}, {
                    168, 0.3787761315316371}, {169, 0.3652404352476588}, {
                    170, 0.3515956610054241}, {171, 0.33788993126205324`}, {
                    172, 0.324170855316314}, {173, 0.3104820235253137}, {
                    174, 0.2968681721873051}, {175, 0.28336996537216935`}, {
                    176, 0.27002584096169524`}, {177, 0.25687251926572385`}, {
                    178, 0.2439423001070239}, {179, 0.23126918083697703`}, {
                    180, 0.21887819900453515`}, {181, 0.20679809804262636`}, {
                    182, 0.1950491841962314}, {183, 0.18365240035483898`}, {
                    184, 0.17262431658449057`}, {185, 0.161979885755609}, {
                    186, 0.15172988008392838`}, {187, 0.14188261174244587`}, {
                    188, 0.13244471277942968`}, {189, 0.12342183358708786`}, {
                    190, 0.11481381001265807`}, {191, 0.10662116322805637`}, {
                    192, 0.0988402429713177}, {193, 0.09146813543930105}, {
                    194, 0.08449788088019945}, {195, 0.07792314701235764}, {
                    196, 0.07173402566589034}, {197, 0.0659200818992794}, {
                    198, 0.0604712572639386}, {199, 0.05537587343623979}, {
                    200, 0.050619753553637604`}, {201, 0.04619008938718013}, {
                    202, 0.04207448190356889}, {203, 0.03825780940923136}, {
                    204, 0.03472502594628561}, {205, 0.031462799843238534`}, {
                    206, 0.028455982257468645`}, {
                    207, 0.025690166887511016`}, {
                    208, 0.023152469210702176`}, {
                    209, 0.020828052788340642`}, {
                    210, 0.018703075944314102`}, {211, 0.01676459062141027}, {
                    212, 0.014999586189779266`}, {
                    213, 0.013396312640309977`}, {
                    214, 0.011943315448921446`}, {
                    215, 0.010628030423965784`}, {
                    216, 0.009440115327044463}, {217, 0.008370508934478776}, {
                    218, 0.007408313505786503}, {219, 0.006544331687595083}, {
                    220, 0.005770344999403386}, {221, 0.005078447211523219}, {
                    222, 0.004461822932292769}, {223, 0.003912784782162757}, {
                    224, 0.003424695005263588}, {225, 0.002991979232332229}, {
                    226, 0.002608754893855015}, {
                    227, 0.0022703234368165624`}, {
                    228, 0.001972412154716224}, {
                    229, 0.0017104358088202098`}, {
                    230, 0.0014804759671325658`}, {
                    231, 0.0012788762024786508`}, {
                    232, 0.0011022869387119502`}, {
                    233, 0.0009484298322187346}, {
                    234, 0.0008149359203244266}, {
                    235, 0.0006988635078325137}, {
                    236, 0.000597980578326035}, {
                    237, 0.0005105670292123982}, {
                    238, 0.00043489340690183514`}, {
                    239, 0.0003698071468226065}, {
                    240, 0.00031431605300783155`}, {
                    241, 0.00026676746792321325`}, {
                    242, 0.0002255104356572774}, {
                    243, 0.00019012324368898693`}, {
                    244, 0.00016069555881611866`}, {
                    245, 0.0001358313504358103}, {
                    246, 0.00011379949585837613`}, {
                    247, 0.00009489449949196286}, {
                    248, 0.0000795723425611223}, {
                    249, 0.00006659351660543365}, {
                    250, 0.00005531721537167322}, {
                    251, 0.0000461994764096256}}],
                   Frame->True,
                   FrameTicks->{{{1., 
                    FormBox[
                    RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
                    FormBox[
                    RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
                    FormBox[
                    RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
                    FormBox[
                    RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
                    FormBox["0", TraditionalForm]}, {157.25, 
                    FormBox["2.5`", TraditionalForm]}, {188.5, 
                    FormBox["5", TraditionalForm]}, {219.75, 
                    FormBox["7.5`", TraditionalForm]}, {251., 
                    FormBox["10", TraditionalForm]}}, Automatic, None, None},
                   PlotRange->{-0.1, 1.1},
                   PlotRangeClipping->True,
                   PlotRangePadding->{
                    Scaled[0.02], Automatic}], 17->
                  GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF1nlcTXkfB/CjIWUJyTIGczVqQtZQFH0klKJ9307rvbfVUDEMzlhmLCPJ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                    "], {
                    {GrayLevel[0.5], EdgeForm[None], 
                    GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ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                    
                    "]]]}, {{}, {}, {
                    {Hue[0.6093005755620837, 1, 1], LineBox[{1, 2}]}, 
                    {Hue[0.5883030831109552, 1, 1], LineBox[{2, 3}]}, 
                    {Hue[0.34522610533488746`, 1, 1], LineBox[{3, 4}]}, 
                    {Hue[0.21884433245913584`, 1, 1], LineBox[{4, 5}]}, 
                    {Hue[0.05273737922507198, 1, 1], LineBox[{5, 6}]}, 
                    {Hue[0.0800429915891726, 1, 1], LineBox[{6, 7}]}, 
                    {Hue[0.8622231077078536, 1, 1], LineBox[{7, 8}]}, 
                    {Hue[0.9811433145234884, 1, 1], LineBox[{8, 9}]}, 
                    {Hue[0.7656439891671794, 1, 1], LineBox[{9, 10}]}, 
                    {Hue[0.575197751929063, 1, 1], LineBox[{10, 11}]}, 
                    {Hue[0.5610195544187918, 1, 1], LineBox[{11, 12}]}, 
                    {Hue[0.4383547897938635, 1, 1], LineBox[{12, 13}]}, 
                    {Hue[0.3897875310877631, 1, 1], LineBox[{13, 14}]}, 
                    {Hue[0.86236870675324, 1, 1], LineBox[{14, 15}]}, 
                    {Hue[0.15167136723202979`, 1, 1], LineBox[{15, 16}]}, 
                    {Hue[0.08718633620831152, 1, 1], LineBox[{16, 17}]}, 
                    {Hue[0.6976688277163468, 1, 1], LineBox[{17, 18}]}, 
                    {Hue[0.9140501077809731, 1, 1], LineBox[{18, 19}]}, 
                    {Hue[0.7775122878648932, 1, 1], LineBox[{19, 20}]}, 
                    {Hue[0.6750104754075937, 1, 1], LineBox[{20, 21}]}, 
                    {Hue[0.601540385332832, 1, 1], LineBox[{21, 22}]}, 
                    {Hue[0.599162923023778, 1, 1], LineBox[{22, 23}]}, 
                    {Hue[0.4732816613374724, 1, 1], LineBox[{23, 24}]}, 
                    {Hue[0.42205667261767177`, 1, 1], LineBox[{24, 25}]}, 
                    {Hue[0.283293118551893, 1, 1], LineBox[{25, 26}]}, 
                    {Hue[0.25052329442331206`, 1, 1], LineBox[{26, 27}]}, 
                    {Hue[0.16345913842696996`, 1, 1], LineBox[{27, 28}]}, 
                    {Hue[0.042713628129332624`, 1, 1], LineBox[{28, 29}]}, 
                    {Hue[0.9476834322091386, 1, 1], LineBox[{29, 30}]}, 
                    {Hue[0.8542985795573227, 1, 1], LineBox[{30, 31}]}, 
                    {Hue[0.762040334551915, 1, 1], LineBox[{31, 32}]}, 
                    {Hue[0.6562323051332618, 1, 1], LineBox[{32, 33}]}, 
                    {Hue[0.5611032812885219, 1, 1], LineBox[{33, 34}]}, 
                    {Hue[0.46177778309986633`, 1, 1], LineBox[{34, 35}]}, 
                    {Hue[0.36288473900116, 1, 1], LineBox[{35, 36}]}, 
                    {Hue[0.2613185162117361, 1, 1], LineBox[{36, 37}]}, 
                    {Hue[0.15485214106860662`, 1, 1], LineBox[{37, 38}]}, 
                    {Hue[0.05584266875309582, 1, 1], LineBox[{38, 39}]}, 
                    {Hue[0.9554896802057098, 1, 1], LineBox[{39, 40}]}, 
                    {Hue[0.8511929684959288, 1, 1], LineBox[{40, 41}]}, 
                    {Hue[0.752637322873851, 1, 1], LineBox[{41, 42}]}, 
                    {Hue[0.6511751745758734, 1, 1], LineBox[{42, 43}]}, 
                    {Hue[0.5544346510193291, 1, 1], LineBox[{43, 44}]}, 
                    {Hue[0.45624627578443544`, 1, 1], LineBox[{44, 45}]}, 
                    {Hue[0.3557371285223141, 1, 1], LineBox[{45, 46}]}, 
                    {Hue[0.25651723449211883`, 1, 1], LineBox[{46, 47}]}, 
                    {Hue[0.15787678371539723`, 1, 1], LineBox[{47, 48}]}, 
                    {Hue[0.05829216344985523, 1, 1], LineBox[{48, 49}]}, 
                    {Hue[0.9603759234494436, 1, 1], LineBox[{49, 50}]}, 
                    {Hue[0.8622537572154851, 1, 1], LineBox[{50, 51}]}, 
                    {Hue[0.7641289269589387, 1, 1], LineBox[{51, 52}]}, 
                    {Hue[0.6671796192899093, 1, 1], LineBox[{52, 53}]}, 
                    {Hue[0.570035522629933, 1, 1], LineBox[{53, 54}]}, 
                    {Hue[0.47298974794991105`, 1, 1], LineBox[{54, 55}]}, 
                    {Hue[0.37603519041829964`, 1, 1], LineBox[{55, 56}]}, 
                    {Hue[0.27935036622232096`, 1, 1], LineBox[{56, 57}]}, 
                    {Hue[0.18303412229352026`, 1, 1], LineBox[{57, 58}]}, 
                    {Hue[0.0868397578505655, 1, 1], LineBox[{58, 59}]}, 
                    {Hue[0.9908567670859162, 1, 1], LineBox[{59, 60}]}, 
                    {Hue[0.8951640675116295, 1, 1], LineBox[{60, 61}]}, 
                    {Hue[0.7997576994445039, 1, 1], LineBox[{61, 62}]}, 
                    {Hue[0.704677263864956, 1, 1], LineBox[{62, 63}]}, 
                    {Hue[0.6097791393361929, 1, 1], LineBox[{63, 64}]}, 
                    {Hue[0.51511325912361, 1, 1], LineBox[{64, 65}]}, 
                    {Hue[0.4206932535946379, 1, 1], LineBox[{65, 66}]}, 
                    {Hue[0.3264908912211963, 1, 1], LineBox[{66, 67}]}, 
                    {Hue[0.23247587761718255`, 1, 1], LineBox[{67, 68}]}, 
                    {Hue[0.1386847975223601, 1, 1], LineBox[{68, 69}]}, 
                    {Hue[0.04511404796464239, 1, 1], LineBox[{69, 70}]}, 
                    {Hue[0.9517719103841894, 1, 1], LineBox[{70, 71}]}, 
                    {Hue[0.8586357506433461, 1, 1], LineBox[{71, 72}]}, 
                    {Hue[0.7657480313793572, 1, 1], LineBox[{72, 73}]}, 
                    {Hue[0.6730497593471026, 1, 1], LineBox[{73, 74}]}, 
                    {Hue[0.5805547319386801, 1, 1], LineBox[{74, 75}]}, 
                    {Hue[0.4882651524400857, 1, 1], LineBox[{75, 76}]}, 
                    {Hue[0.39616028884303134`, 1, 1], LineBox[{76, 77}]}, 
                    {Hue[0.3042682849642356, 1, 1], LineBox[{77, 78}]}, 
                    {Hue[0.21256569508200795`, 1, 1], LineBox[{78, 79}]}, 
                    {Hue[0.12105676008102176`, 1, 1], LineBox[{79, 80}]}, 
                    {Hue[0.02974915153054283, 1, 1], LineBox[{80, 81}]}, 
                    {Hue[0.9386188542407726, 1, 1], LineBox[{81, 82}]}, 
                    {Hue[0.8476676926099618, 1, 1], LineBox[{82, 83}]}, 
                    {Hue[0.7568938711280264, 1, 1], LineBox[{83, 84}]}, 
                    {Hue[0.666294270544339, 1, 1], LineBox[{84, 85}]}, 
                    {Hue[0.5758728368553732, 1, 1], LineBox[{85, 86}]}, 
                    {Hue[0.4856387576042597, 1, 1], LineBox[{86, 87}]}, 
                    {Hue[0.39558025894890825`, 1, 1], LineBox[{87, 88}]}, 
                    {Hue[0.30568691036491824`, 1, 1], LineBox[{88, 89}]}, 
                    {Hue[0.21595156833836465`, 1, 1], LineBox[{89, 90}]}, 
                    {Hue[0.12637324682863935`, 1, 1], LineBox[{90, 91}]}, 
                    {Hue[0.03695515584535301, 1, 1], LineBox[{91, 92}]}, 
                    {Hue[0.9477192956297038, 1, 1], LineBox[{92, 93}]}, 
                    {Hue[0.858670444255665, 1, 1], LineBox[{93, 94}]}, 
                    {Hue[0.7698077509905155, 1, 1], LineBox[{94, 95}]}, 
                    {Hue[0.6811137464946615, 1, 1], LineBox[{95, 96}]}, 
                    {Hue[0.5925780318446903, 1, 1], LineBox[{96, 97}]}, 
                    {Hue[0.5041925162411358, 1, 1], LineBox[{97, 98}]}, 
                    {Hue[0.41598740282012486`, 1, 1], LineBox[{98, 99}]}, 
                    {Hue[0.3280008129379155, 1, 1], LineBox[{99, 100}]}, 
                    {Hue[0.24027124169676167`, 1, 1], LineBox[{100, 101}]}, 
                    {Hue[0.1528061272499971, 1, 1], LineBox[{101, 102}]}, 
                    {Hue[0.06558679955777666, 1, 1], LineBox[{102, 103}]}, 
                    {Hue[0.978589469438239, 1, 1], LineBox[{103, 104}]}, 
                    {Hue[0.891817591134329, 1, 1], LineBox[{104, 105}]}, 
                    {Hue[0.8053329095049395, 1, 1], LineBox[{105, 106}]}, 
                    {Hue[0.7192408998844428, 1, 1], LineBox[{106, 107}]}, 
                    {Hue[0.6336444994087578, 1, 1], LineBox[{107, 108}]}, 
                    {Hue[0.5485979186574695, 1, 1], LineBox[{108, 109}]}, 
                    {Hue[0.4640872331328644, 1, 1], LineBox[{109, 110}]}, 
                    {Hue[0.3800792298129656, 1, 1], LineBox[{110, 111}]}, 
                    {Hue[0.29658892069408954`, 1, 1], LineBox[{111, 112}]}, 
                    {Hue[0.2137498070449708, 1, 1], LineBox[{112, 113}]}, 
                    {Hue[0.1317985031971102, 1, 1], LineBox[{113, 114}]}, 
                    {Hue[0.05100279554728599, 1, 1], LineBox[{114, 115}]}, 
                    {Hue[0.9715544937306999, 1, 1], LineBox[{115, 116}]}, 
                    {Hue[0.8935117552806451, 1, 1], LineBox[{116, 117}]}, 
                    {Hue[0.8168378148577125, 1, 1], LineBox[{117, 118}]}, 
                    {Hue[0.741507451773556, 1, 1], LineBox[{118, 119}]}, 
                    {Hue[0.6676383743085668, 1, 1], LineBox[{119, 120}]}, 
                    {Hue[0.5955532908070316, 1, 1], LineBox[{120, 121}]}, 
                    {Hue[0.525747511026724, 1, 1], LineBox[{121, 122}]}, 
                    {Hue[0.4587611269785911, 1, 1], LineBox[{122, 123}]}, 
                    {Hue[0.3950341786702998, 1, 1], LineBox[{123, 124}]}, 
                    {Hue[0.3348239923730795, 1, 1], LineBox[{124, 125}]}, 
                    {Hue[0.278206263626647, 1, 1], LineBox[{125, 126}]}, 
                    {Hue[0.22512972585836025`, 1, 1], LineBox[{126, 127}]}, 
                    {Hue[0.17548723112854178`, 1, 1], LineBox[{127, 128}]}, 
                    {Hue[0.1291654748489493, 1, 1], LineBox[{128, 129}]}, 
                    {Hue[0.08607432464118504, 1, 1], LineBox[{129, 130}]}, 
                    {Hue[0.04615674856215593, 1, 1], LineBox[{130, 131}]}, 
                    {Hue[0.009383628358470269, 1, 1], LineBox[{131, 132}]}, 
                    {Hue[0.9757461448592223, 1, 1], LineBox[{132, 133}]}, 
                    {Hue[0.9452432063344988, 1, 1], LineBox[{133, 134}]}, 
                    {Hue[0.9178706741768455, 1, 1], LineBox[{134, 135}]}, 
                    {Hue[0.8936176561725973, 1, 1], LineBox[{135, 136}]}, 
                    {Hue[0.8724589000455976, 1, 1], LineBox[{136, 137}]}, 
                    {Hue[0.8543561829042435, 1, 1], LineBox[{137, 138}]}, 
                    {Hue[0.8392565806338904, 1, 1], LineBox[{138, 139}]}, 
                    {Hue[0.8270954879357122, 1, 1], LineBox[{139, 140}]}, 
                    {Hue[0.8177994855524553, 1, 1], LineBox[{140, 141}]}, 
                    {Hue[0.8112868389087454, 1, 1], LineBox[{141, 142}]}, 
                    {Hue[0.8074714251134552, 1, 1], LineBox[{142, 143}]}, 
                    {Hue[0.8062642398689808, 1, 1], LineBox[{143, 144}]}, 
                    {Hue[0.80757390034191, 1, 1], LineBox[{144, 145}]}, 
                    {Hue[0.8113117373256622, 1, 1], LineBox[{145, 146}]}, 
                    {Hue[0.8173886140816178, 1, 1], LineBox[{146, 147}]}, 
                    {Hue[0.8257180119328547, 1, 1], LineBox[{147, 148}]}, 
                    {Hue[0.8362167368039202, 1, 1], LineBox[{148, 149}]}, 
                    {Hue[0.8488048851013356, 1, 1], LineBox[{149, 150}]}, 
                    {Hue[0.8634056201434694, 1, 1], LineBox[{150, 151}]}, 
                    {Hue[0.8799464877098273, 1, 1], LineBox[{151, 152}]}, 
                    {Hue[0.8983581839384799, 1, 1], LineBox[{152, 153}]}, 
                    {Hue[0.9185754267824033, 1, 1], LineBox[{153, 154}]}, 
                    {Hue[0.9405376116358177, 1, 1], LineBox[{154, 155}]}, 
                    {Hue[0.9641858131985447, 1, 1], LineBox[{155, 156}]}, 
                    {Hue[0.9894658914039416, 1, 1], LineBox[{156, 157}]}, 
                    {Hue[0.016326442168588418`, 1, 1], LineBox[{157, 158}]}, 
                    {Hue[0.04471969994336976, 1, 1], LineBox[{158, 159}]}, 
                    {Hue[0.07460025787099636, 1, 1], LineBox[{159, 160}]}, 
                    {Hue[0.1059256870147042, 1, 1], LineBox[{160, 161}]}, 
                    {Hue[0.13865648207719755`, 1, 1], LineBox[{161, 162}]}, 
                    {Hue[0.17275516346313075`, 1, 1], LineBox[{162, 163}]}, 
                    {Hue[0.20818654705850076`, 1, 1], LineBox[{163, 164}]}, 
                    {Hue[0.24491776062038736`, 1, 1], LineBox[{164, 165}]}, 
                    {Hue[0.2829175843834493, 1, 1], LineBox[{165, 166}]}, 
                    {Hue[0.32215693783560934`, 1, 1], LineBox[{166, 167}]}, 
                    {Hue[0.36260803122091423`, 1, 1], LineBox[{167, 168}]}, 
                    {Hue[0.40424526696122776`, 1, 1], LineBox[{168, 169}]}, 
                    {Hue[0.4470438610211715, 1, 1], LineBox[{169, 170}]}, 
                    {Hue[0.4909809957748204, 1, 1], LineBox[{170, 171}]}, 
                    {Hue[0.5360342305294032, 1, 1], LineBox[{171, 172}]}, 
                    {Hue[0.5821842310355752, 1, 1], LineBox[{172, 173}]}, 
                    {Hue[0.6294105205595869, 1, 1], LineBox[{173, 174}]}, 
                    {Hue[0.6776942738642395, 1, 1], LineBox[{174, 175}]}, 
                    {Hue[0.7270182530399725, 1, 1], LineBox[{175, 176}]}, 
                    {Hue[0.7773654671943058, 1, 1], LineBox[{176, 177}]}, 
                    {Hue[0.8287197761195882, 1, 1], LineBox[{177, 178}]}, 
                    {Hue[0.881066826233025, 1, 1], LineBox[{178, 179}]}, 
                    {Hue[0.9343919814094939, 1, 1], LineBox[{179, 180}]}, 
                    {Hue[0.9886810103437937, 1, 1], LineBox[{180, 181}]}, 
                    {Hue[0.04392134041543094, 1, 1], LineBox[{181, 182}]}, 
                    {Hue[0.10010062521493725`, 1, 1], LineBox[{182, 183}]}, 
                    {Hue[0.1572060901918438, 1, 1], LineBox[{183, 184}]}, 
                    {Hue[0.2152267037775803, 1, 1], LineBox[{184, 185}]}, 
                    {Hue[0.2741518685733359, 1, 1], LineBox[{185, 186}]}, 
                    {Hue[0.33396976725536087`, 1, 1], LineBox[{186, 187}]}, 
                    {Hue[0.39467201119426304`, 1, 1], LineBox[{187, 188}]}, 
                    {Hue[0.4562467952731411, 1, 1], LineBox[{188, 189}]}, 
                    {Hue[0.5186871754839177, 1, 1], LineBox[{189, 190}]}, 
                    {Hue[0.5819821910854881, 1, 1], LineBox[{190, 191}]}, 
                    {Hue[0.6461249212559091, 1, 1], LineBox[{191, 192}]}, 
                    {Hue[0.7111053868113398, 1, 1], LineBox[{192, 193}]}, 
                    {Hue[0.7769163365513685, 1, 1], LineBox[{193, 194}]}, 
                    {Hue[0.8435502311470209, 1, 1], LineBox[{194, 195}]}, 
                    {Hue[0.9109997258954383, 1, 1], LineBox[{195, 196}]}, 
                    {Hue[0.9792567724428458, 1, 1], LineBox[{196, 197}]}, 
                    {Hue[0.048314893032024846`, 1, 1], LineBox[{197, 198}]}, 
                    {Hue[0.11816659846490493`, 1, 1], LineBox[{198, 199}]}, 
                    {Hue[0.18880748145033227`, 1, 1], LineBox[{199, 200}]}, 
                    {Hue[0.2602281081067364, 1, 1], LineBox[{200, 201}]}, 
                    {Hue[0.3324261821884439, 1, 1], LineBox[{201, 202}]}, 
                    {Hue[0.4053919971395003, 1, 1], LineBox[{202, 203}]}, 
                    {Hue[0.4791202853011942, 1, 1], LineBox[{203, 204}]}, 
                    {Hue[0.5536061692980763, 1, 1], LineBox[{204, 205}]}, 
                    {Hue[0.6288460163903289, 1, 1], LineBox[{205, 206}]}, 
                    {Hue[0.704833005736574, 1, 1], LineBox[{206, 207}]}, 
                    {Hue[0.7815601974591878, 1, 1], LineBox[{207, 208}]}, 
                    {Hue[0.8590240975898622, 1, 1], LineBox[{208, 209}]}, 
                    {Hue[0.9372201321618259, 1, 1], LineBox[{209, 210}]}, 
                    {Hue[0.01614330869885718, 1, 1], LineBox[{210, 211}]}, 
                    {Hue[0.09579005917412756, 1, 1], LineBox[{211, 212}]}, 
                    {Hue[0.17615510022340541`, 1, 1], LineBox[{212, 213}]}, 
                    {Hue[0.25722998241254924`, 1, 1], LineBox[{213, 214}]}, 
                    {Hue[0.33901361326646806`, 1, 1], LineBox[{214, 215}]}, 
                    {Hue[0.4215079868432836, 1, 1], LineBox[{215, 216}]}, 
                    {Hue[0.5047018519445817, 1, 1], LineBox[{216, 217}]}, 
                    {Hue[0.5885854873834779, 1, 1], LineBox[{217, 218}]}, 
                    {Hue[0.6731690217175137, 1, 1], LineBox[{218, 219}]}, 
                    {Hue[0.7584443045132689, 1, 1], LineBox[{219, 220}]}, 
                    {Hue[0.8443996576338665, 1, 1], LineBox[{220, 221}]}, 
                    {Hue[0.9310392668849811, 1, 1], LineBox[{221, 222}]}, 
                    {Hue[0.01835608108419255, 1, 1], LineBox[{222, 223}]}, 
                    {Hue[0.10634281440537748`, 1, 1], LineBox[{223, 224}]}, 
                    {Hue[0.19500751539705116`, 1, 1], LineBox[{224, 225}]}, 
                    {Hue[0.28434507288419764`, 1, 1], LineBox[{225, 226}]}, 
                    {Hue[0.37433969563046116`, 1, 1], LineBox[{226, 227}]}, 
                    {Hue[0.46499878959259816`, 1, 1], LineBox[{227, 228}]}, 
                    {Hue[0.5563051143788902, 1, 1], LineBox[{228, 229}]}, 
                    {Hue[0.6482599370262787, 1, 1], LineBox[{229, 230}]}, 
                    {Hue[0.7408961732407697, 1, 1], LineBox[{230, 231}]}, 
                    {Hue[0.8341759038366332, 1, 1], LineBox[{231, 232}]}, 
                    {Hue[0.9280821964872943, 1, 1], LineBox[{232, 233}]}, 
                    {Hue[0.02264091417215117, 1, 1], LineBox[{233, 234}]}, 
                    {Hue[0.11784663968249516`, 1, 1], LineBox[{234, 235}]}, 
                    {Hue[0.21369717943481406`, 1, 1], LineBox[{235, 236}]}, 
                    {Hue[0.31015717277902966`, 1, 1], LineBox[{236, 237}]}, 
                    {Hue[0.40721972489202457`, 1, 1], LineBox[{237, 238}]}, 
                    {Hue[0.5049504162563396, 1, 1], LineBox[{238, 239}]}, 
                    {Hue[0.6033365493871723, 1, 1], LineBox[{239, 240}]}, 
                    {Hue[0.702329216982111, 1, 1], LineBox[{240, 241}]}, 
                    {Hue[0.8019357595066028, 1, 1], LineBox[{241, 242}]}, 
                    {Hue[0.9021269477589214, 1, 1], LineBox[{242, 243}]}, 
                    {Hue[0.0028973395460384008`, 1, 1], 
                    LineBox[{243, 244}]}, 
                    {Hue[0.10431645639875466`, 1, 1], LineBox[{244, 245}]}, 
                    {Hue[0.20637297304353572`, 1, 1], LineBox[{245, 246}]}, 
                    {Hue[0.30905173195330454`, 1, 1], LineBox[{246, 247}]}, 
                    {Hue[0.41237782325228056`, 1, 1], LineBox[{247, 248}]}, 
                    {Hue[0.5162096888172363, 1, 1], LineBox[{248, 249}]}, 
                    {Hue[0.6204876662046706, 1, 1], LineBox[{249, 250}]}, 
                    {Hue[0.7254548945286261, 1, 1], LineBox[{250, 251}]}}}},
                    VertexColors->{
                    Hue[0.6093005755620837, 1, 1], 
                    Hue[0.5883030831109552, 1, 1], 
                    Hue[0.34522610533488746`, 1, 1], 
                    Hue[0.21884433245913584`, 1, 1], 
                    Hue[0.05273737922507198, 1, 1], 
                    Hue[0.0800429915891726, 1, 1], 
                    Hue[0.8622231077078536, 1, 1], 
                    Hue[0.9811433145234884, 1, 1], 
                    Hue[0.7656439891671794, 1, 1], 
                    Hue[0.575197751929063, 1, 1], 
                    Hue[0.5610195544187918, 1, 1], 
                    Hue[0.4383547897938635, 1, 1], 
                    Hue[0.3897875310877631, 1, 1], 
                    Hue[0.86236870675324, 1, 1], 
                    Hue[0.15167136723202979`, 1, 1], 
                    Hue[0.08718633620831152, 1, 1], 
                    Hue[0.6976688277163468, 1, 1], 
                    Hue[0.9140501077809731, 1, 1], 
                    Hue[0.7775122878648932, 1, 1], 
                    Hue[0.6750104754075937, 1, 1], 
                    Hue[0.601540385332832, 1, 1], 
                    Hue[0.599162923023778, 1, 1], 
                    Hue[0.4732816613374724, 1, 1], 
                    Hue[0.42205667261767177`, 1, 1], 
                    Hue[0.283293118551893, 1, 1], 
                    Hue[0.25052329442331206`, 1, 1], 
                    Hue[0.16345913842696996`, 1, 1], 
                    Hue[0.042713628129332624`, 1, 1], 
                    Hue[0.9476834322091386, 1, 1], 
                    Hue[0.8542985795573227, 1, 1], 
                    Hue[0.762040334551915, 1, 1], 
                    Hue[0.6562323051332618, 1, 1], 
                    Hue[0.5611032812885219, 1, 1], 
                    Hue[0.46177778309986633`, 1, 1], 
                    Hue[0.36288473900116, 1, 1], 
                    Hue[0.2613185162117361, 1, 1], 
                    Hue[0.15485214106860662`, 1, 1], 
                    Hue[0.05584266875309582, 1, 1], 
                    Hue[0.9554896802057098, 1, 1], 
                    Hue[0.8511929684959288, 1, 1], 
                    Hue[0.752637322873851, 1, 1], 
                    Hue[0.6511751745758734, 1, 1], 
                    Hue[0.5544346510193291, 1, 1], 
                    Hue[0.45624627578443544`, 1, 1], 
                    Hue[0.3557371285223141, 1, 1], 
                    Hue[0.25651723449211883`, 1, 1], 
                    Hue[0.15787678371539723`, 1, 1], 
                    Hue[0.05829216344985523, 1, 1], 
                    Hue[0.9603759234494436, 1, 1], 
                    Hue[0.8622537572154851, 1, 1], 
                    Hue[0.7641289269589387, 1, 1], 
                    Hue[0.6671796192899093, 1, 1], 
                    Hue[0.570035522629933, 1, 1], 
                    Hue[0.47298974794991105`, 1, 1], 
                    Hue[0.37603519041829964`, 1, 1], 
                    Hue[0.27935036622232096`, 1, 1], 
                    Hue[0.18303412229352026`, 1, 1], 
                    Hue[0.0868397578505655, 1, 1], 
                    Hue[0.9908567670859162, 1, 1], 
                    Hue[0.8951640675116295, 1, 1], 
                    Hue[0.7997576994445039, 1, 1], 
                    Hue[0.704677263864956, 1, 1], 
                    Hue[0.6097791393361929, 1, 1], 
                    Hue[0.51511325912361, 1, 1], 
                    Hue[0.4206932535946379, 1, 1], 
                    Hue[0.3264908912211963, 1, 1], 
                    Hue[0.23247587761718255`, 1, 1], 
                    Hue[0.1386847975223601, 1, 1], 
                    Hue[0.04511404796464239, 1, 1], 
                    Hue[0.9517719103841894, 1, 1], 
                    Hue[0.8586357506433461, 1, 1], 
                    Hue[0.7657480313793572, 1, 1], 
                    Hue[0.6730497593471026, 1, 1], 
                    Hue[0.5805547319386801, 1, 1], 
                    Hue[0.4882651524400857, 1, 1], 
                    Hue[0.39616028884303134`, 1, 1], 
                    Hue[0.3042682849642356, 1, 1], 
                    Hue[0.21256569508200795`, 1, 1], 
                    Hue[0.12105676008102176`, 1, 1], 
                    Hue[0.02974915153054283, 1, 1], 
                    Hue[0.9386188542407726, 1, 1], 
                    Hue[0.8476676926099618, 1, 1], 
                    Hue[0.7568938711280264, 1, 1], 
                    Hue[0.666294270544339, 1, 1], 
                    Hue[0.5758728368553732, 1, 1], 
                    Hue[0.4856387576042597, 1, 1], 
                    Hue[0.39558025894890825`, 1, 1], 
                    Hue[0.30568691036491824`, 1, 1], 
                    Hue[0.21595156833836465`, 1, 1], 
                    Hue[0.12637324682863935`, 1, 1], 
                    Hue[0.03695515584535301, 1, 1], 
                    Hue[0.9477192956297038, 1, 1], 
                    Hue[0.858670444255665, 1, 1], 
                    Hue[0.7698077509905155, 1, 1], 
                    Hue[0.6811137464946615, 1, 1], 
                    Hue[0.5925780318446903, 1, 1], 
                    Hue[0.5041925162411358, 1, 1], 
                    Hue[0.41598740282012486`, 1, 1], 
                    Hue[0.3280008129379155, 1, 1], 
                    Hue[0.24027124169676167`, 1, 1], 
                    Hue[0.1528061272499971, 1, 1], 
                    Hue[0.06558679955777666, 1, 1], 
                    Hue[0.978589469438239, 1, 1], 
                    Hue[0.891817591134329, 1, 1], 
                    Hue[0.8053329095049395, 1, 1], 
                    Hue[0.7192408998844428, 1, 1], 
                    Hue[0.6336444994087578, 1, 1], 
                    Hue[0.5485979186574695, 1, 1], 
                    Hue[0.4640872331328644, 1, 1], 
                    Hue[0.3800792298129656, 1, 1], 
                    Hue[0.29658892069408954`, 1, 1], 
                    Hue[0.2137498070449708, 1, 1], 
                    Hue[0.1317985031971102, 1, 1], 
                    Hue[0.05100279554728599, 1, 1], 
                    Hue[0.9715544937306999, 1, 1], 
                    Hue[0.8935117552806451, 1, 1], 
                    Hue[0.8168378148577125, 1, 1], 
                    Hue[0.741507451773556, 1, 1], 
                    Hue[0.6676383743085668, 1, 1], 
                    Hue[0.5955532908070316, 1, 1], 
                    Hue[0.525747511026724, 1, 1], 
                    Hue[0.4587611269785911, 1, 1], 
                    Hue[0.3950341786702998, 1, 1], 
                    Hue[0.3348239923730795, 1, 1], 
                    Hue[0.278206263626647, 1, 1], 
                    Hue[0.22512972585836025`, 1, 1], 
                    Hue[0.17548723112854178`, 1, 1], 
                    Hue[0.1291654748489493, 1, 1], 
                    Hue[0.08607432464118504, 1, 1], 
                    Hue[0.04615674856215593, 1, 1], 
                    Hue[0.009383628358470269, 1, 1], 
                    Hue[0.9757461448592223, 1, 1], 
                    Hue[0.9452432063344988, 1, 1], 
                    Hue[0.9178706741768455, 1, 1], 
                    Hue[0.8936176561725973, 1, 1], 
                    Hue[0.8724589000455976, 1, 1], 
                    Hue[0.8543561829042435, 1, 1], 
                    Hue[0.8392565806338904, 1, 1], 
                    Hue[0.8270954879357122, 1, 1], 
                    Hue[0.8177994855524553, 1, 1], 
                    Hue[0.8112868389087454, 1, 1], 
                    Hue[0.8074714251134552, 1, 1], 
                    Hue[0.8062642398689808, 1, 1], 
                    Hue[0.80757390034191, 1, 1], 
                    Hue[0.8113117373256622, 1, 1], 
                    Hue[0.8173886140816178, 1, 1], 
                    Hue[0.8257180119328547, 1, 1], 
                    Hue[0.8362167368039202, 1, 1], 
                    Hue[0.8488048851013356, 1, 1], 
                    Hue[0.8634056201434694, 1, 1], 
                    Hue[0.8799464877098273, 1, 1], 
                    Hue[0.8983581839384799, 1, 1], 
                    Hue[0.9185754267824033, 1, 1], 
                    Hue[0.9405376116358177, 1, 1], 
                    Hue[0.9641858131985447, 1, 1], 
                    Hue[0.9894658914039416, 1, 1], 
                    Hue[0.016326442168588418`, 1, 1], 
                    Hue[0.04471969994336976, 1, 1], 
                    Hue[0.07460025787099636, 1, 1], 
                    Hue[0.1059256870147042, 1, 1], 
                    Hue[0.13865648207719755`, 1, 1], 
                    Hue[0.17275516346313075`, 1, 1], 
                    Hue[0.20818654705850076`, 1, 1], 
                    Hue[0.24491776062038736`, 1, 1], 
                    Hue[0.2829175843834493, 1, 1], 
                    Hue[0.32215693783560934`, 1, 1], 
                    Hue[0.36260803122091423`, 1, 1], 
                    Hue[0.40424526696122776`, 1, 1], 
                    Hue[0.4470438610211715, 1, 1], 
                    Hue[0.4909809957748204, 1, 1], 
                    Hue[0.5360342305294032, 1, 1], 
                    Hue[0.5821842310355752, 1, 1], 
                    Hue[0.6294105205595869, 1, 1], 
                    Hue[0.6776942738642395, 1, 1], 
                    Hue[0.7270182530399725, 1, 1], 
                    Hue[0.7773654671943058, 1, 1], 
                    Hue[0.8287197761195882, 1, 1], 
                    Hue[0.881066826233025, 1, 1], 
                    Hue[0.9343919814094939, 1, 1], 
                    Hue[0.9886810103437937, 1, 1], 
                    Hue[0.04392134041543094, 1, 1], 
                    Hue[0.10010062521493725`, 1, 1], 
                    Hue[0.1572060901918438, 1, 1], 
                    Hue[0.2152267037775803, 1, 1], 
                    Hue[0.2741518685733359, 1, 1], 
                    Hue[0.33396976725536087`, 1, 1], 
                    Hue[0.39467201119426304`, 1, 1], 
                    Hue[0.4562467952731411, 1, 1], 
                    Hue[0.5186871754839177, 1, 1], 
                    Hue[0.5819821910854881, 1, 1], 
                    Hue[0.6461249212559091, 1, 1], 
                    Hue[0.7111053868113398, 1, 1], 
                    Hue[0.7769163365513685, 1, 1], 
                    Hue[0.8435502311470209, 1, 1], 
                    Hue[0.9109997258954383, 1, 1], 
                    Hue[0.9792567724428458, 1, 1], 
                    Hue[0.048314893032024846`, 1, 1], 
                    Hue[0.11816659846490493`, 1, 1], 
                    Hue[0.18880748145033227`, 1, 1], 
                    Hue[0.2602281081067364, 1, 1], 
                    Hue[0.3324261821884439, 1, 1], 
                    Hue[0.4053919971395003, 1, 1], 
                    Hue[0.4791202853011942, 1, 1], 
                    Hue[0.5536061692980763, 1, 1], 
                    Hue[0.6288460163903289, 1, 1], 
                    Hue[0.704833005736574, 1, 1], 
                    Hue[0.7815601974591878, 1, 1], 
                    Hue[0.8590240975898622, 1, 1], 
                    Hue[0.9372201321618259, 1, 1], 
                    Hue[0.01614330869885718, 1, 1], 
                    Hue[0.09579005917412756, 1, 1], 
                    Hue[0.17615510022340541`, 1, 1], 
                    Hue[0.25722998241254924`, 1, 1], 
                    Hue[0.33901361326646806`, 1, 1], 
                    Hue[0.4215079868432836, 1, 1], 
                    Hue[0.5047018519445817, 1, 1], 
                    Hue[0.5885854873834779, 1, 1], 
                    Hue[0.6731690217175137, 1, 1], 
                    Hue[0.7584443045132689, 1, 1], 
                    Hue[0.8443996576338665, 1, 1], 
                    Hue[0.9310392668849811, 1, 1], 
                    Hue[0.01835608108419255, 1, 1], 
                    Hue[0.10634281440537748`, 1, 1], 
                    Hue[0.19500751539705116`, 1, 1], 
                    Hue[0.28434507288419764`, 1, 1], 
                    Hue[0.37433969563046116`, 1, 1], 
                    Hue[0.46499878959259816`, 1, 1], 
                    Hue[0.5563051143788902, 1, 1], 
                    Hue[0.6482599370262787, 1, 1], 
                    Hue[0.7408961732407697, 1, 1], 
                    Hue[0.8341759038366332, 1, 1], 
                    Hue[0.9280821964872943, 1, 1], 
                    Hue[0.02264091417215117, 1, 1], 
                    Hue[0.11784663968249516`, 1, 1], 
                    Hue[0.21369717943481406`, 1, 1], 
                    Hue[0.31015717277902966`, 1, 1], 
                    Hue[0.40721972489202457`, 1, 1], 
                    Hue[0.5049504162563396, 1, 1], 
                    Hue[0.6033365493871723, 1, 1], 
                    Hue[0.702329216982111, 1, 1], 
                    Hue[0.8019357595066028, 1, 1], 
                    Hue[0.9021269477589214, 1, 1], 
                    Hue[0.0028973395460384008`, 1, 1], 
                    Hue[0.10431645639875466`, 1, 1], 
                    Hue[0.20637297304353572`, 1, 1], 
                    Hue[0.30905173195330454`, 1, 1], 
                    Hue[0.41237782325228056`, 1, 1], 
                    Hue[0.5162096888172363, 1, 1], 
                    Hue[0.6204876662046706, 1, 1], 
                    Hue[0.7254548945286261, 1, 1], 
                    Hue[0.8312043329782834, 1, 1], 
                    Hue[0.6093005755620837, 1, 1], 
                    Hue[0.5883030831109552, 1, 1], 
                    Hue[0.5883030831109552, 1, 1], 
                    Hue[0.34522610533488746`, 1, 1], 
                    Hue[0.34522610533488746`, 1, 1], 
                    Hue[0.21884433245913584`, 1, 1], 
                    Hue[0.21884433245913584`, 1, 1], 
                    Hue[0.05273737922507198, 1, 1], 
                    Hue[0.05273737922507198, 1, 1], 
                    Hue[0.0800429915891726, 1, 1], 
                    Hue[0.0800429915891726, 1, 1], 
                    Hue[0.8622231077078536, 1, 1], 
                    Hue[0.8622231077078536, 1, 1], 
                    Hue[0.9811433145234884, 1, 1], 
                    Hue[0.9811433145234884, 1, 1], 
                    Hue[0.7656439891671794, 1, 1], 
                    Hue[0.7656439891671794, 1, 1], 
                    Hue[0.575197751929063, 1, 1], 
                    Hue[0.575197751929063, 1, 1], 
                    Hue[0.5610195544187918, 1, 1], 
                    Hue[0.5610195544187918, 1, 1], 
                    Hue[0.4383547897938635, 1, 1], 
                    Hue[0.4383547897938635, 1, 1], 
                    Hue[0.3897875310877631, 1, 1], 
                    Hue[0.3897875310877631, 1, 1], 
                    Hue[0.86236870675324, 1, 1], 
                    Hue[0.86236870675324, 1, 1], 
                    Hue[0.15167136723202979`, 1, 1], 
                    Hue[0.15167136723202979`, 1, 1], 
                    Hue[0.08718633620831152, 1, 1], 
                    Hue[0.08718633620831152, 1, 1], 
                    Hue[0.6976688277163468, 1, 1], 
                    Hue[0.6976688277163468, 1, 1], 
                    Hue[0.9140501077809731, 1, 1], 
                    Hue[0.9140501077809731, 1, 1], 
                    Hue[0.7775122878648932, 1, 1], 
                    Hue[0.7775122878648932, 1, 1], 
                    Hue[0.6750104754075937, 1, 1], 
                    Hue[0.6750104754075937, 1, 1], 
                    Hue[0.601540385332832, 1, 1], 
                    Hue[0.601540385332832, 1, 1], 
                    Hue[0.599162923023778, 1, 1], 
                    Hue[0.599162923023778, 1, 1], 
                    Hue[0.4732816613374724, 1, 1], 
                    Hue[0.4732816613374724, 1, 1], 
                    Hue[0.42205667261767177`, 1, 1], 
                    Hue[0.42205667261767177`, 1, 1], 
                    Hue[0.283293118551893, 1, 1], 
                    Hue[0.283293118551893, 1, 1], 
                    Hue[0.25052329442331206`, 1, 1], 
                    Hue[0.25052329442331206`, 1, 1], 
                    Hue[0.16345913842696996`, 1, 1], 
                    Hue[0.16345913842696996`, 1, 1], 
                    Hue[0.042713628129332624`, 1, 1], 
                    Hue[0.042713628129332624`, 1, 1], 
                    Hue[0.9476834322091386, 1, 1], 
                    Hue[0.9476834322091386, 1, 1], 
                    Hue[0.8542985795573227, 1, 1], 
                    Hue[0.8542985795573227, 1, 1], 
                    Hue[0.762040334551915, 1, 1], 
                    Hue[0.762040334551915, 1, 1], 
                    Hue[0.6562323051332618, 1, 1], 
                    Hue[0.6562323051332618, 1, 1], 
                    Hue[0.5611032812885219, 1, 1], 
                    Hue[0.5611032812885219, 1, 1], 
                    Hue[0.46177778309986633`, 1, 1], 
                    Hue[0.46177778309986633`, 1, 1], 
                    Hue[0.36288473900116, 1, 1], 
                    Hue[0.36288473900116, 1, 1], 
                    Hue[0.2613185162117361, 1, 1], 
                    Hue[0.2613185162117361, 1, 1], 
                    Hue[0.15485214106860662`, 1, 1], 
                    Hue[0.15485214106860662`, 1, 1], 
                    Hue[0.05584266875309582, 1, 1], 
                    Hue[0.05584266875309582, 1, 1], 
                    Hue[0.9554896802057098, 1, 1], 
                    Hue[0.9554896802057098, 1, 1], 
                    Hue[0.8511929684959288, 1, 1], 
                    Hue[0.8511929684959288, 1, 1], 
                    Hue[0.752637322873851, 1, 1], 
                    Hue[0.752637322873851, 1, 1], 
                    Hue[0.6511751745758734, 1, 1], 
                    Hue[0.6511751745758734, 1, 1], 
                    Hue[0.5544346510193291, 1, 1], 
                    Hue[0.5544346510193291, 1, 1], 
                    Hue[0.45624627578443544`, 1, 1], 
                    Hue[0.45624627578443544`, 1, 1], 
                    Hue[0.3557371285223141, 1, 1], 
                    Hue[0.3557371285223141, 1, 1], 
                    Hue[0.25651723449211883`, 1, 1], 
                    Hue[0.25651723449211883`, 1, 1], 
                    Hue[0.15787678371539723`, 1, 1], 
                    Hue[0.15787678371539723`, 1, 1], 
                    Hue[0.05829216344985523, 1, 1], 
                    Hue[0.05829216344985523, 1, 1], 
                    Hue[0.9603759234494436, 1, 1], 
                    Hue[0.9603759234494436, 1, 1], 
                    Hue[0.8622537572154851, 1, 1], 
                    Hue[0.8622537572154851, 1, 1], 
                    Hue[0.7641289269589387, 1, 1], 
                    Hue[0.7641289269589387, 1, 1], 
                    Hue[0.6671796192899093, 1, 1], 
                    Hue[0.6671796192899093, 1, 1], 
                    Hue[0.570035522629933, 1, 1], 
                    Hue[0.570035522629933, 1, 1], 
                    Hue[0.47298974794991105`, 1, 1], 
                    Hue[0.47298974794991105`, 1, 1], 
                    Hue[0.37603519041829964`, 1, 1], 
                    Hue[0.37603519041829964`, 1, 1], 
                    Hue[0.27935036622232096`, 1, 1], 
                    Hue[0.27935036622232096`, 1, 1], 
                    Hue[0.18303412229352026`, 1, 1], 
                    Hue[0.18303412229352026`, 1, 1], 
                    Hue[0.0868397578505655, 1, 1], 
                    Hue[0.0868397578505655, 1, 1], 
                    Hue[0.9908567670859162, 1, 1], 
                    Hue[0.9908567670859162, 1, 1], 
                    Hue[0.8951640675116295, 1, 1], 
                    Hue[0.8951640675116295, 1, 1], 
                    Hue[0.7997576994445039, 1, 1], 
                    Hue[0.7997576994445039, 1, 1], 
                    Hue[0.704677263864956, 1, 1], 
                    Hue[0.704677263864956, 1, 1], 
                    Hue[0.6097791393361929, 1, 1], 
                    Hue[0.6097791393361929, 1, 1], 
                    Hue[0.51511325912361, 1, 1], 
                    Hue[0.51511325912361, 1, 1], 
                    Hue[0.4206932535946379, 1, 1], 
                    Hue[0.4206932535946379, 1, 1], 
                    Hue[0.3264908912211963, 1, 1], 
                    Hue[0.3264908912211963, 1, 1], 
                    Hue[0.23247587761718255`, 1, 1], 
                    Hue[0.23247587761718255`, 1, 1], 
                    Hue[0.1386847975223601, 1, 1], 
                    Hue[0.1386847975223601, 1, 1], 
                    Hue[0.04511404796464239, 1, 1], 
                    Hue[0.04511404796464239, 1, 1], 
                    Hue[0.9517719103841894, 1, 1], 
                    Hue[0.9517719103841894, 1, 1], 
                    Hue[0.8586357506433461, 1, 1], 
                    Hue[0.8586357506433461, 1, 1], 
                    Hue[0.7657480313793572, 1, 1], 
                    Hue[0.7657480313793572, 1, 1], 
                    Hue[0.6730497593471026, 1, 1], 
                    Hue[0.6730497593471026, 1, 1], 
                    Hue[0.5805547319386801, 1, 1], 
                    Hue[0.5805547319386801, 1, 1], 
                    Hue[0.4882651524400857, 1, 1], 
                    Hue[0.4882651524400857, 1, 1], 
                    Hue[0.39616028884303134`, 1, 1], 
                    Hue[0.39616028884303134`, 1, 1], 
                    Hue[0.3042682849642356, 1, 1], 
                    Hue[0.3042682849642356, 1, 1], 
                    Hue[0.21256569508200795`, 1, 1], 
                    Hue[0.21256569508200795`, 1, 1], 
                    Hue[0.12105676008102176`, 1, 1], 
                    Hue[0.12105676008102176`, 1, 1], 
                    Hue[0.02974915153054283, 1, 1], 
                    Hue[0.02974915153054283, 1, 1], 
                    Hue[0.9386188542407726, 1, 1], 
                    Hue[0.9386188542407726, 1, 1], 
                    Hue[0.8476676926099618, 1, 1], 
                    Hue[0.8476676926099618, 1, 1], 
                    Hue[0.7568938711280264, 1, 1], 
                    Hue[0.7568938711280264, 1, 1], 
                    Hue[0.666294270544339, 1, 1], 
                    Hue[0.666294270544339, 1, 1], 
                    Hue[0.5758728368553732, 1, 1], 
                    Hue[0.5758728368553732, 1, 1], 
                    Hue[0.4856387576042597, 1, 1], 
                    Hue[0.4856387576042597, 1, 1], 
                    Hue[0.39558025894890825`, 1, 1], 
                    Hue[0.39558025894890825`, 1, 1], 
                    Hue[0.30568691036491824`, 1, 1], 
                    Hue[0.30568691036491824`, 1, 1], 
                    Hue[0.21595156833836465`, 1, 1], 
                    Hue[0.21595156833836465`, 1, 1], 
                    Hue[0.12637324682863935`, 1, 1], 
                    Hue[0.12637324682863935`, 1, 1], 
                    Hue[0.03695515584535301, 1, 1], 
                    Hue[0.03695515584535301, 1, 1], 
                    Hue[0.9477192956297038, 1, 1], 
                    Hue[0.9477192956297038, 1, 1], 
                    Hue[0.858670444255665, 1, 1], 
                    Hue[0.858670444255665, 1, 1], 
                    Hue[0.7698077509905155, 1, 1], 
                    Hue[0.7698077509905155, 1, 1], 
                    Hue[0.6811137464946615, 1, 1], 
                    Hue[0.6811137464946615, 1, 1], 
                    Hue[0.5925780318446903, 1, 1], 
                    Hue[0.5925780318446903, 1, 1], 
                    Hue[0.5041925162411358, 1, 1], 
                    Hue[0.5041925162411358, 1, 1], 
                    Hue[0.41598740282012486`, 1, 1], 
                    Hue[0.41598740282012486`, 1, 1], 
                    Hue[0.3280008129379155, 1, 1], 
                    Hue[0.3280008129379155, 1, 1], 
                    Hue[0.24027124169676167`, 1, 1], 
                    Hue[0.24027124169676167`, 1, 1], 
                    Hue[0.1528061272499971, 1, 1], 
                    Hue[0.1528061272499971, 1, 1], 
                    Hue[0.06558679955777666, 1, 1], 
                    Hue[0.06558679955777666, 1, 1], 
                    Hue[0.978589469438239, 1, 1], 
                    Hue[0.978589469438239, 1, 1], 
                    Hue[0.891817591134329, 1, 1], 
                    Hue[0.891817591134329, 1, 1], 
                    Hue[0.8053329095049395, 1, 1], 
                    Hue[0.8053329095049395, 1, 1], 
                    Hue[0.7192408998844428, 1, 1], 
                    Hue[0.7192408998844428, 1, 1], 
                    Hue[0.6336444994087578, 1, 1], 
                    Hue[0.6336444994087578, 1, 1], 
                    Hue[0.5485979186574695, 1, 1], 
                    Hue[0.5485979186574695, 1, 1], 
                    Hue[0.4640872331328644, 1, 1], 
                    Hue[0.4640872331328644, 1, 1], 
                    Hue[0.3800792298129656, 1, 1], 
                    Hue[0.3800792298129656, 1, 1], 
                    Hue[0.29658892069408954`, 1, 1], 
                    Hue[0.29658892069408954`, 1, 1], 
                    Hue[0.2137498070449708, 1, 1], 
                    Hue[0.2137498070449708, 1, 1], 
                    Hue[0.1317985031971102, 1, 1], 
                    Hue[0.1317985031971102, 1, 1], 
                    Hue[0.05100279554728599, 1, 1], 
                    Hue[0.05100279554728599, 1, 1], 
                    Hue[0.9715544937306999, 1, 1], 
                    Hue[0.9715544937306999, 1, 1], 
                    Hue[0.8935117552806451, 1, 1], 
                    Hue[0.8935117552806451, 1, 1], 
                    Hue[0.8168378148577125, 1, 1], 
                    Hue[0.8168378148577125, 1, 1], 
                    Hue[0.741507451773556, 1, 1], 
                    Hue[0.741507451773556, 1, 1], 
                    Hue[0.6676383743085668, 1, 1], 
                    Hue[0.6676383743085668, 1, 1], 
                    Hue[0.5955532908070316, 1, 1], 
                    Hue[0.5955532908070316, 1, 1], 
                    Hue[0.525747511026724, 1, 1], 
                    Hue[0.525747511026724, 1, 1], 
                    Hue[0.4587611269785911, 1, 1], 
                    Hue[0.4587611269785911, 1, 1], 
                    Hue[0.3950341786702998, 1, 1], 
                    Hue[0.3950341786702998, 1, 1], 
                    Hue[0.3348239923730795, 1, 1], 
                    Hue[0.3348239923730795, 1, 1], 
                    Hue[0.278206263626647, 1, 1], 
                    Hue[0.278206263626647, 1, 1], 
                    Hue[0.22512972585836025`, 1, 1], 
                    Hue[0.22512972585836025`, 1, 1], 
                    Hue[0.17548723112854178`, 1, 1], 
                    Hue[0.17548723112854178`, 1, 1], 
                    Hue[0.1291654748489493, 1, 1], 
                    Hue[0.1291654748489493, 1, 1], 
                    Hue[0.08607432464118504, 1, 1], 
                    Hue[0.08607432464118504, 1, 1], 
                    Hue[0.04615674856215593, 1, 1], 
                    Hue[0.04615674856215593, 1, 1], 
                    Hue[0.009383628358470269, 1, 1], 
                    Hue[0.009383628358470269, 1, 1], 
                    Hue[0.9757461448592223, 1, 1], 
                    Hue[0.9757461448592223, 1, 1], 
                    Hue[0.9452432063344988, 1, 1], 
                    Hue[0.9452432063344988, 1, 1], 
                    Hue[0.9178706741768455, 1, 1], 
                    Hue[0.9178706741768455, 1, 1], 
                    Hue[0.8936176561725973, 1, 1], 
                    Hue[0.8936176561725973, 1, 1], 
                    Hue[0.8724589000455976, 1, 1], 
                    Hue[0.8724589000455976, 1, 1], 
                    Hue[0.8543561829042435, 1, 1], 
                    Hue[0.8543561829042435, 1, 1], 
                    Hue[0.8392565806338904, 1, 1], 
                    Hue[0.8392565806338904, 1, 1], 
                    Hue[0.8270954879357122, 1, 1], 
                    Hue[0.8270954879357122, 1, 1], 
                    Hue[0.8177994855524553, 1, 1], 
                    Hue[0.8177994855524553, 1, 1], 
                    Hue[0.8112868389087454, 1, 1], 
                    Hue[0.8112868389087454, 1, 1], 
                    Hue[0.8074714251134552, 1, 1], 
                    Hue[0.8074714251134552, 1, 1], 
                    Hue[0.8062642398689808, 1, 1], 
                    Hue[0.8062642398689808, 1, 1], 
                    Hue[0.80757390034191, 1, 1], 
                    Hue[0.80757390034191, 1, 1], 
                    Hue[0.8113117373256622, 1, 1], 
                    Hue[0.8113117373256622, 1, 1], 
                    Hue[0.8173886140816178, 1, 1], 
                    Hue[0.8173886140816178, 1, 1], 
                    Hue[0.8257180119328547, 1, 1], 
                    Hue[0.8257180119328547, 1, 1], 
                    Hue[0.8362167368039202, 1, 1], 
                    Hue[0.8362167368039202, 1, 1], 
                    Hue[0.8488048851013356, 1, 1], 
                    Hue[0.8488048851013356, 1, 1], 
                    Hue[0.8634056201434694, 1, 1], 
                    Hue[0.8634056201434694, 1, 1], 
                    Hue[0.8799464877098273, 1, 1], 
                    Hue[0.8799464877098273, 1, 1], 
                    Hue[0.8983581839384799, 1, 1], 
                    Hue[0.8983581839384799, 1, 1], 
                    Hue[0.9185754267824033, 1, 1], 
                    Hue[0.9185754267824033, 1, 1], 
                    Hue[0.9405376116358177, 1, 1], 
                    Hue[0.9405376116358177, 1, 1], 
                    Hue[0.9641858131985447, 1, 1], 
                    Hue[0.9641858131985447, 1, 1], 
                    Hue[0.9894658914039416, 1, 1], 
                    Hue[0.9894658914039416, 1, 1], 
                    Hue[0.016326442168588418`, 1, 1], 
                    Hue[0.016326442168588418`, 1, 1], 
                    Hue[0.04471969994336976, 1, 1], 
                    Hue[0.04471969994336976, 1, 1], 
                    Hue[0.07460025787099636, 1, 1], 
                    Hue[0.07460025787099636, 1, 1], 
                    Hue[0.1059256870147042, 1, 1], 
                    Hue[0.1059256870147042, 1, 1], 
                    Hue[0.13865648207719755`, 1, 1], 
                    Hue[0.13865648207719755`, 1, 1], 
                    Hue[0.17275516346313075`, 1, 1], 
                    Hue[0.17275516346313075`, 1, 1], 
                    Hue[0.20818654705850076`, 1, 1], 
                    Hue[0.20818654705850076`, 1, 1], 
                    Hue[0.24491776062038736`, 1, 1], 
                    Hue[0.24491776062038736`, 1, 1], 
                    Hue[0.2829175843834493, 1, 1], 
                    Hue[0.2829175843834493, 1, 1], 
                    Hue[0.32215693783560934`, 1, 1], 
                    Hue[0.32215693783560934`, 1, 1], 
                    Hue[0.36260803122091423`, 1, 1], 
                    Hue[0.36260803122091423`, 1, 1], 
                    Hue[0.40424526696122776`, 1, 1], 
                    Hue[0.40424526696122776`, 1, 1], 
                    Hue[0.4470438610211715, 1, 1], 
                    Hue[0.4470438610211715, 1, 1], 
                    Hue[0.4909809957748204, 1, 1], 
                    Hue[0.4909809957748204, 1, 1], 
                    Hue[0.5360342305294032, 1, 1], 
                    Hue[0.5360342305294032, 1, 1], 
                    Hue[0.5821842310355752, 1, 1], 
                    Hue[0.5821842310355752, 1, 1], 
                    Hue[0.6294105205595869, 1, 1], 
                    Hue[0.6294105205595869, 1, 1], 
                    Hue[0.6776942738642395, 1, 1], 
                    Hue[0.6776942738642395, 1, 1], 
                    Hue[0.7270182530399725, 1, 1], 
                    Hue[0.7270182530399725, 1, 1], 
                    Hue[0.7773654671943058, 1, 1], 
                    Hue[0.7773654671943058, 1, 1], 
                    Hue[0.8287197761195882, 1, 1], 
                    Hue[0.8287197761195882, 1, 1], 
                    Hue[0.881066826233025, 1, 1], 
                    Hue[0.881066826233025, 1, 1], 
                    Hue[0.9343919814094939, 1, 1], 
                    Hue[0.9343919814094939, 1, 1], 
                    Hue[0.9886810103437937, 1, 1], 
                    Hue[0.9886810103437937, 1, 1], 
                    Hue[0.04392134041543094, 1, 1], 
                    Hue[0.04392134041543094, 1, 1], 
                    Hue[0.10010062521493725`, 1, 1], 
                    Hue[0.10010062521493725`, 1, 1], 
                    Hue[0.1572060901918438, 1, 1], 
                    Hue[0.1572060901918438, 1, 1], 
                    Hue[0.2152267037775803, 1, 1], 
                    Hue[0.2152267037775803, 1, 1], 
                    Hue[0.2741518685733359, 1, 1], 
                    Hue[0.2741518685733359, 1, 1], 
                    Hue[0.33396976725536087`, 1, 1], 
                    Hue[0.33396976725536087`, 1, 1], 
                    Hue[0.39467201119426304`, 1, 1], 
                    Hue[0.39467201119426304`, 1, 1], 
                    Hue[0.4562467952731411, 1, 1], 
                    Hue[0.4562467952731411, 1, 1], 
                    Hue[0.5186871754839177, 1, 1], 
                    Hue[0.5186871754839177, 1, 1], 
                    Hue[0.5819821910854881, 1, 1], 
                    Hue[0.5819821910854881, 1, 1], 
                    Hue[0.6461249212559091, 1, 1], 
                    Hue[0.6461249212559091, 1, 1], 
                    Hue[0.7111053868113398, 1, 1], 
                    Hue[0.7111053868113398, 1, 1], 
                    Hue[0.7769163365513685, 1, 1], 
                    Hue[0.7769163365513685, 1, 1], 
                    Hue[0.8435502311470209, 1, 1], 
                    Hue[0.8435502311470209, 1, 1], 
                    Hue[0.9109997258954383, 1, 1], 
                    Hue[0.9109997258954383, 1, 1], 
                    Hue[0.9792567724428458, 1, 1], 
                    Hue[0.9792567724428458, 1, 1], 
                    Hue[0.048314893032024846`, 1, 1], 
                    Hue[0.048314893032024846`, 1, 1], 
                    Hue[0.11816659846490493`, 1, 1], 
                    Hue[0.11816659846490493`, 1, 1], 
                    Hue[0.18880748145033227`, 1, 1], 
                    Hue[0.18880748145033227`, 1, 1], 
                    Hue[0.2602281081067364, 1, 1], 
                    Hue[0.2602281081067364, 1, 1], 
                    Hue[0.3324261821884439, 1, 1], 
                    Hue[0.3324261821884439, 1, 1], 
                    Hue[0.4053919971395003, 1, 1], 
                    Hue[0.4053919971395003, 1, 1], 
                    Hue[0.4791202853011942, 1, 1], 
                    Hue[0.4791202853011942, 1, 1], 
                    Hue[0.5536061692980763, 1, 1], 
                    Hue[0.5536061692980763, 1, 1], 
                    Hue[0.6288460163903289, 1, 1], 
                    Hue[0.6288460163903289, 1, 1], 
                    Hue[0.704833005736574, 1, 1], 
                    Hue[0.704833005736574, 1, 1], 
                    Hue[0.7815601974591878, 1, 1], 
                    Hue[0.7815601974591878, 1, 1], 
                    Hue[0.8590240975898622, 1, 1], 
                    Hue[0.8590240975898622, 1, 1], 
                    Hue[0.9372201321618259, 1, 1], 
                    Hue[0.9372201321618259, 1, 1], 
                    Hue[0.01614330869885718, 1, 1], 
                    Hue[0.01614330869885718, 1, 1], 
                    Hue[0.09579005917412756, 1, 1], 
                    Hue[0.09579005917412756, 1, 1], 
                    Hue[0.17615510022340541`, 1, 1], 
                    Hue[0.17615510022340541`, 1, 1], 
                    Hue[0.25722998241254924`, 1, 1], 
                    Hue[0.25722998241254924`, 1, 1], 
                    Hue[0.33901361326646806`, 1, 1], 
                    Hue[0.33901361326646806`, 1, 1], 
                    Hue[0.4215079868432836, 1, 1], 
                    Hue[0.4215079868432836, 1, 1], 
                    Hue[0.5047018519445817, 1, 1], 
                    Hue[0.5047018519445817, 1, 1], 
                    Hue[0.5885854873834779, 1, 1], 
                    Hue[0.5885854873834779, 1, 1], 
                    Hue[0.6731690217175137, 1, 1], 
                    Hue[0.6731690217175137, 1, 1], 
                    Hue[0.7584443045132689, 1, 1], 
                    Hue[0.7584443045132689, 1, 1], 
                    Hue[0.8443996576338665, 1, 1], 
                    Hue[0.8443996576338665, 1, 1], 
                    Hue[0.9310392668849811, 1, 1], 
                    Hue[0.9310392668849811, 1, 1], 
                    Hue[0.01835608108419255, 1, 1], 
                    Hue[0.01835608108419255, 1, 1], 
                    Hue[0.10634281440537748`, 1, 1], 
                    Hue[0.10634281440537748`, 1, 1], 
                    Hue[0.19500751539705116`, 1, 1], 
                    Hue[0.19500751539705116`, 1, 1], 
                    Hue[0.28434507288419764`, 1, 1], 
                    Hue[0.28434507288419764`, 1, 1], 
                    Hue[0.37433969563046116`, 1, 1], 
                    Hue[0.37433969563046116`, 1, 1], 
                    Hue[0.46499878959259816`, 1, 1], 
                    Hue[0.46499878959259816`, 1, 1], 
                    Hue[0.5563051143788902, 1, 1], 
                    Hue[0.5563051143788902, 1, 1], 
                    Hue[0.6482599370262787, 1, 1], 
                    Hue[0.6482599370262787, 1, 1], 
                    Hue[0.7408961732407697, 1, 1], 
                    Hue[0.7408961732407697, 1, 1], 
                    Hue[0.8341759038366332, 1, 1], 
                    Hue[0.8341759038366332, 1, 1], 
                    Hue[0.9280821964872943, 1, 1], 
                    Hue[0.9280821964872943, 1, 1], 
                    Hue[0.02264091417215117, 1, 1], 
                    Hue[0.02264091417215117, 1, 1], 
                    Hue[0.11784663968249516`, 1, 1], 
                    Hue[0.11784663968249516`, 1, 1], 
                    Hue[0.21369717943481406`, 1, 1], 
                    Hue[0.21369717943481406`, 1, 1], 
                    Hue[0.31015717277902966`, 1, 1], 
                    Hue[0.31015717277902966`, 1, 1], 
                    Hue[0.40721972489202457`, 1, 1], 
                    Hue[0.40721972489202457`, 1, 1], 
                    Hue[0.5049504162563396, 1, 1], 
                    Hue[0.5049504162563396, 1, 1], 
                    Hue[0.6033365493871723, 1, 1], 
                    Hue[0.6033365493871723, 1, 1], 
                    Hue[0.702329216982111, 1, 1], 
                    Hue[0.702329216982111, 1, 1], 
                    Hue[0.8019357595066028, 1, 1], 
                    Hue[0.8019357595066028, 1, 1], 
                    Hue[0.9021269477589214, 1, 1], 
                    Hue[0.9021269477589214, 1, 1], 
                    Hue[0.0028973395460384008`, 1, 1], 
                    Hue[0.0028973395460384008`, 1, 1], 
                    Hue[0.10431645639875466`, 1, 1], 
                    Hue[0.10431645639875466`, 1, 1], 
                    Hue[0.20637297304353572`, 1, 1], 
                    Hue[0.20637297304353572`, 1, 1], 
                    Hue[0.30905173195330454`, 1, 1], 
                    Hue[0.30905173195330454`, 1, 1], 
                    Hue[0.41237782325228056`, 1, 1], 
                    Hue[0.41237782325228056`, 1, 1], 
                    Hue[0.5162096888172363, 1, 1], 
                    Hue[0.5162096888172363, 1, 1], 
                    Hue[0.6204876662046706, 1, 1], 
                    Hue[0.6204876662046706, 1, 1], 
                    Hue[0.7254548945286261, 1, 1], 
                    Hue[0.7254548945286261, 1, 1], 
                    Hue[0.8312043329782834, 1, 1]}],
                   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
                   Axes->{True, None},
                   AxesOrigin->{0, 0},
                   
                   Epilog->LineBox[{{1, 4.465961166467968*^-7}, {
                    2, 7.956089980721662*^-7}, {3, 1.2080462257273376`*^-6}, {
                    4, 1.068437575775247*^-6}, {5, 5.783416278576244*^-7}, {
                    6, 7.699168023451992*^-7}, {7, 6.049501091558932*^-7}, {
                    8, 4.593198849520388*^-7}, {9, 1.3289285724059744`*^-6}, {
                    10, 6.452925093537202*^-7}, {11, 5.323775076689481*^-7}, {
                    12, 2.661859769000134*^-7}, {13, 9.908697260728364*^-7}, {
                    14, 2.8771192160845204`*^-7}, {
                    15, 5.49455345501363*^-7}, {16, 2.875210030077873*^-7}, {
                    17, 3.783729415361154*^-7}, {18, 7.2092942490568*^-7}, {
                    19, 5.429028169661039*^-7}, {20, 8.480552527456583*^-7}, {
                    21, 9.928841664717074*^-7}, {22, 5.090482571075096*^-7}, {
                    23, 1.2956855266877236`*^-6}, {
                    24, 1.6052093239676597`*^-6}, {
                    25, 1.3821741390300245`*^-6}, {
                    26, 2.079933766701361*^-6}, {
                    27, 2.6198680558333978`*^-6}, {
                    28, 3.4199793959022483`*^-6}, {
                    29, 4.5356718587931005`*^-6}, {
                    30, 4.990965537211067*^-6}, {31, 6.247978888870258*^-6}, {
                    32, 8.144676894575314*^-6}, {33, 9.09621745593498*^-6}, {
                    34, 0.00001103982832802499}, {
                    35, 0.00001418603554700873}, {
                    36, 0.000016210381076184154`}, {
                    37, 0.000019943012021794054`}, {
                    38, 0.000024022313091069023`}, {
                    39, 0.000029033335269879555`}, {
                    40, 0.00003552562486850368}, {
                    41, 0.000042287283084219806`}, {
                    42, 0.000050465723101957854`}, {
                    43, 0.000060375069789874526`}, {
                    44, 0.00007231518735858472}, {
                    45, 0.00008646481315146338}, {
                    46, 0.0001023073509581926}, {
                    47, 0.00012183229038690085`}, {
                    48, 0.00014332374050077062`}, {
                    49, 0.00017013987185814292`}, {
                    50, 0.00020106627376102645`}, {
                    51, 0.00023618233001864057`}, {
                    52, 0.0002783877043128726}, {53, 0.0003276220413634956}, {
                    54, 0.0003843109066576748}, {55, 0.0004498940119490794}, {
                    56, 0.0005248824613025863}, {57, 0.0006121304561945996}, {
                    58, 0.0007132589928315171}, {59, 0.0008284807997185334}, {
                    60, 0.0009618688217196192}, {
                    61, 0.0011138459555453213`}, {
                    62, 0.0012879124808189448`}, {63, 0.001487112254795767}, {
                    64, 0.0017131633586123921`}, {
                    65, 0.0019699980085649185`}, {
                    66, 0.0022621774959527387`}, {
                    67, 0.0025925151253452373`}, {
                    68, 0.0029666619252550395`}, {
                    69, 0.0033888531319688675`}, {70, 0.003863913236082229}, {
                    71, 0.004398905981830261}, {72, 0.0049981488505563545`}, {
                    73, 0.005669303481718499}, {74, 0.006418870247413546}, {
                    75, 0.007254137956871396}, {76, 0.008183309315890376}, {
                    77, 0.009215234568250226}, {78, 0.010358846712774742`}, {
                    79, 0.01162279081313599}, {80, 0.013016926180056872`}, {
                    81, 0.014550518226422011`}, {82, 0.01623478918132964}, {
                    83, 0.018080280439427037`}, {84, 0.020099078029177986`}, {
                    85, 0.02230204409530182}, {86, 0.024698638050735213`}, {
                    87, 0.02730082934138697}, {88, 0.030115977937487996`}, {
                    89, 0.03315764653793455}, {90, 0.03643837639987169}, {
                    91, 0.03997007899404878}, {92, 0.04376345263026441}, {
                    93, 0.04782469749544438}, {94, 0.05215550130440996}, {
                    95, 0.05675983829722015}, {96, 0.061645550241447}, {
                    97, 0.06682722152150794}, {98, 0.07232018000639912}, {
                    99, 0.07813122502774716}, {100, 0.08425353955078142}, {
                    101, 0.09066587560436452}, {102, 0.0973511213608786}, {
                    103, 0.10431155230083176`}, {104, 0.11157561285659069`}, {
                    105, 0.11918352951669064`}, {106, 0.12715345143951956`}, {
                    107, 0.13545342025215026`}, {108, 0.14401189217854204`}, {
                    109, 0.15275824670694696`}, {110, 0.16169236975100565`}, {
                    111, 0.17090715568257578`}, {112, 0.18055095169812352`}, {
                    113, 0.19073222664377687`}, {114, 0.20141742888704986`}, {
                    115, 0.21243157765857199`}, {116, 0.22355089824579477`}, {
                    117, 0.23467241982895418`}, {118, 0.24594363426387572`}, {
                    119, 0.25775687988104573`}, {120, 0.2706069451816341}, {
                    121, 0.28484427188009753`}, {122, 0.3004896800197246}, {
                    123, 0.31717859539988275`}, {124, 0.33427521306359387`}, {
                    125, 0.35106287785100576`}, {126, 0.36691675930138756`}, {
                    127, 0.3814179673828288}, {128, 0.39437344029634863`}, {
                    129, 0.40579020997355647`}, {130, 0.41581185940282833`}, {
                    131, 0.42464925596550906`}, {132, 0.43253455180840056`}, {
                    133, 0.4396808372154018}, {134, 0.4462618613503635}, {
                    135, 0.4524082082767881}, {136, 0.45820952320515523`}, {
                    137, 0.46371594332370797`}, {138, 0.46895300495824227`}, {
                    139, 0.4739211809053794}, {140, 0.4786091120732276}, {
                    141, 0.4829951729187367}, {142, 0.48704818160531754`}, {
                    143, 0.4907365221711765}, {144, 0.49402681247525965`}, {
                    145, 0.49688457539343917`}, {146, 0.4992784950098724}, {
                    147, 0.5011769092625394}, {148, 0.5025519359075454}, {
                    149, 0.503379309609334}, {150, 0.5036369734614191}, {
                    151, 0.5033067946044762}, {152, 0.502375472466543}, {
                    153, 0.5008309046922802}, {154, 0.49866534275152036`}, {
                    155, 0.49587637025536624`}, {156, 0.49246489846108077`}, {
                    157, 0.4884334308206991}, {158, 0.48378952558757127`}, {
                    159, 0.4785432437504738}, {160, 0.4727106531708105}, {
                    161, 0.46630814214898114`}, {162, 0.4593524595846046}, {
                    163, 0.4518710858115378}, {164, 0.4438852721467415}, {
                    165, 0.43542381962426596`}, {166, 0.42651748287801783`}, {
                    167, 0.41719437991439223`}, {168, 0.40749117788610206`}, {
                    169, 0.39743840845142464`}, {170, 0.3870722471142686}, {
                    171, 0.3764296358984598}, {172, 0.3655451509810529}, {
                    173, 0.35445639036631627`}, {174, 0.34319919555697326`}, {
                    175, 0.33181037438936445`}, {176, 0.32032657501523804`}, {
                    177, 0.30878196432282806`}, {178, 0.29721233639493516`}, {
                    179, 0.2856504017606239}, {180, 0.27412895404080057`}, {
                    181, 0.2626801510002149}, {182, 0.25133258410027887`}, {
                    183, 0.2401131115826976}, {184, 0.22905106780074028`}, {
                    185, 0.218168219911465}, {186, 0.20748991653301638`}, {
                    187, 0.19703445152676527`}, {188, 0.186822890463749}, {
                    189, 0.17687185602353517`}, {190, 0.16719540737274122`}, {
                    191, 0.15780729351444053`}, {192, 0.14871788106288572`}, {
                    193, 0.13993771818468892`}, {194, 0.13147425653188277`}, {
                    195, 0.12333253909428953`}, {196, 0.11551711256741437`}, {
                    197, 0.10802912396361104`}, {198, 0.10087039053467728`}, {
                    199, 0.09403950368811381}, {200, 0.0875349254452988}, {
                    201, 0.08135384426409242}, {202, 0.07549051693968799}, {
                    203, 0.06994069338837774}, {204, 0.06469741521537807}, {
                    205, 0.059753169951092956`}, {206, 0.05509900599724413}, {
                    207, 0.05072712555093638}, {208, 0.04662850209658327}, {
                    209, 0.04279299963219529}, {210, 0.03921058983260664}, {
                    211, 0.03587082165381312}, {212, 0.03276261741917028}, {
                    213, 0.02987573659595402}, {214, 0.027200285753796166`}, {
                    215, 0.024724577684119318`}, {
                    216, 0.022436967451613533`}, {217, 0.02032862304938372}, {
                    218, 0.01838917013977974}, {219, 0.016606933025539533`}, {
                    220, 0.014973020854909832`}, {
                    221, 0.013477982160610285`}, {
                    222, 0.012112063805714723`}, {
                    223, 0.010866943725446975`}, {
                    224, 0.009734101423890442}, {225, 0.008704466483636833}, {
                    226, 0.007770765960616775}, {227, 0.006925721481079195}, {
                    228, 0.006162027251691595}, {229, 0.005474153459128695}, {
                    230, 0.004854976216746123}, {
                    231, 0.0042978386710745935`}, {
                    232, 0.0037984637658225805`}, {
                    233, 0.003351650941346346}, {234, 0.002952316082461568}, {
                    235, 0.0025959261289751983`}, {
                    236, 0.002278380309434855}, {237, 0.00199669948742157}, {
                    238, 0.0017471936926599964`}, {
                    239, 0.001525909042535809}, {
                    240, 0.0013301677381813964`}, {
                    241, 0.0011574625176969822`}, {
                    242, 0.0010053340615007662`}, {
                    243, 0.0008719166539826102}, {
                    244, 0.0007550806587226983}, {
                    245, 0.0006526897535448405}, {
                    246, 0.0005631967025225873}, {
                    247, 0.00048494635841474253`}, {
                    248, 0.0004165759108779093}, {
                    249, 0.0003574616153745001}, {
                    250, 0.0003065509025968455}, {
                    251, 0.0002623382601653005}}],
                   Frame->True,
                   FrameTicks->{{{1., 
                    FormBox[
                    RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
                    FormBox[
                    RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
                    FormBox[
                    RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
                    FormBox[
                    RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
                    FormBox["0", TraditionalForm]}, {157.25, 
                    FormBox["2.5`", TraditionalForm]}, {188.5, 
                    FormBox["5", TraditionalForm]}, {219.75, 
                    FormBox["7.5`", TraditionalForm]}, {251., 
                    FormBox["10", TraditionalForm]}}, Automatic, None, None},
                   PlotRange->{-0.1, 1.1},
                   PlotRangeClipping->True,
                   PlotRangePadding->{
                    Scaled[0.02], Automatic}], 18->
                  GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF1nlYlOUaBvAvXMMNXJAIbVwyRFIUESGVW0AWBdkEQUG+AWaAYQC3sNz6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                    "], {
                    {GrayLevel[0.5], EdgeForm[None], 
                    GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ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                    
                    "]]]}, {{}, {}, {
                    {Hue[0.974286712374846, 1, 1], LineBox[{1, 2}]}, 
                    {Hue[0.08446566071998841, 1, 1], LineBox[{2, 3}]}, 
                    {Hue[0.7942339412854, 1, 1], LineBox[{3, 4}]}, 
                    {Hue[0.8492028857989099, 1, 1], LineBox[{4, 5}]}, 
                    {Hue[0.6022978636183001, 1, 1], LineBox[{5, 6}]}, 
                    {Hue[0.7606479149378287, 1, 1], LineBox[{6, 7}]}, 
                    {Hue[0.578859153398882, 1, 1], LineBox[{7, 8}]}, 
                    {Hue[0.30877927036783076`, 1, 1], LineBox[{8, 9}]}, 
                    {Hue[0.31482924609675245`, 1, 1], LineBox[{9, 10}]}, 
                    {Hue[0.6200413157728587, 1, 1], LineBox[{10, 11}]}, 
                    {Hue[0.09442359141006498, 1, 1], LineBox[{11, 12}]}, 
                    {Hue[0.23001877330545184`, 1, 1], LineBox[{12, 13}]}, 
                    {Hue[0.9760886121604145, 1, 1], LineBox[{13, 14}]}, 
                    {Hue[0.9467880234325805, 1, 1], LineBox[{14, 15}]}, 
                    {Hue[0.9028940736581215, 1, 1], LineBox[{15, 16}]}, 
                    {Hue[0.752209199149744, 1, 1], LineBox[{16, 17}]}, 
                    {Hue[0.6911669551999422, 1, 1], LineBox[{17, 18}]}, 
                    {Hue[0.6219826929738932, 1, 1], LineBox[{18, 19}]}, 
                    {Hue[0.5012750928019923, 1, 1], LineBox[{19, 20}]}, 
                    {Hue[0.4060824274587364, 1, 1], LineBox[{20, 21}]}, 
                    {Hue[0.3222783622768835, 1, 1], LineBox[{21, 22}]}, 
                    {Hue[0.22185466654030803`, 1, 1], LineBox[{22, 23}]}, 
                    {Hue[0.11723128500502794`, 1, 1], LineBox[{23, 24}]}, 
                    {Hue[0.005902445675020168, 1, 1], LineBox[{24, 25}]}, 
                    {Hue[0.9069522018498736, 1, 1], LineBox[{25, 26}]}, 
                    {Hue[0.8068527599341031, 1, 1], LineBox[{26, 27}]}, 
                    {Hue[0.708878572463234, 1, 1], LineBox[{27, 28}]}, 
                    {Hue[0.6079610261392703, 1, 1], LineBox[{28, 29}]}, 
                    {Hue[0.5040843867085472, 1, 1], LineBox[{29, 30}]}, 
                    {Hue[0.40698470072433135`, 1, 1], LineBox[{30, 31}]}, 
                    {Hue[0.3020483743054194, 1, 1], LineBox[{31, 32}]}, 
                    {Hue[0.20223825272965743`, 1, 1], LineBox[{32, 33}]}, 
                    {Hue[0.09995156837997692, 1, 1], LineBox[{33, 34}]}, 
                    {Hue[0.9973121322051787, 1, 1], LineBox[{34, 35}]}, 
                    {Hue[0.8993192587880702, 1, 1], LineBox[{35, 36}]}, 
                    {Hue[0.7994909922000972, 1, 1], LineBox[{36, 37}]}, 
                    {Hue[0.7001832107207604, 1, 1], LineBox[{37, 38}]}, 
                    {Hue[0.6012734955736141, 1, 1], LineBox[{38, 39}]}, 
                    {Hue[0.5018059684686444, 1, 1], LineBox[{39, 40}]}, 
                    {Hue[0.4036261093302133, 1, 1], LineBox[{40, 41}]}, 
                    {Hue[0.3050426183203099, 1, 1], LineBox[{41, 42}]}, 
                    {Hue[0.20640472381167882`, 1, 1], LineBox[{42, 43}]}, 
                    {Hue[0.10812979713222298`, 1, 1], LineBox[{43, 44}]}, 
                    {Hue[0.010246775437848158`, 1, 1], LineBox[{44, 45}]}, 
                    {Hue[0.9125933430525395, 1, 1], LineBox[{45, 46}]}, 
                    {Hue[0.8151261615159525, 1, 1], LineBox[{46, 47}]}, 
                    {Hue[0.7180481601790828, 1, 1], LineBox[{47, 48}]}, 
                    {Hue[0.6211638292725423, 1, 1], LineBox[{48, 49}]}, 
                    {Hue[0.5245698891455377, 1, 1], LineBox[{49, 50}]}, 
                    {Hue[0.4283083559045919, 1, 1], LineBox[{50, 51}]}, 
                    {Hue[0.3321726452629961, 1, 1], LineBox[{51, 52}]}, 
                    {Hue[0.23626986406217826`, 1, 1], LineBox[{52, 53}]}, 
                    {Hue[0.14059393740808274`, 1, 1], LineBox[{53, 54}]}, 
                    {Hue[0.04514473076327826, 1, 1], LineBox[{54, 55}]}, 
                    {Hue[0.9499413082274694, 1, 1], LineBox[{55, 56}]}, 
                    {Hue[0.8549726697253752, 1, 1], LineBox[{56, 57}]}, 
                    {Hue[0.760249505394373, 1, 1], LineBox[{57, 58}]}, 
                    {Hue[0.6657461239613698, 1, 1], LineBox[{58, 59}]}, 
                    {Hue[0.5714732426671167, 1, 1], LineBox[{59, 60}]}, 
                    {Hue[0.4774118319071657, 1, 1], LineBox[{60, 61}]}, 
                    {Hue[0.3835360586193685, 1, 1], LineBox[{61, 62}]}, 
                    {Hue[0.28990642954043827`, 1, 1], LineBox[{62, 63}]}, 
                    {Hue[0.19646904601881474`, 1, 1], LineBox[{63, 64}]}, 
                    {Hue[0.10324490144884163`, 1, 1], LineBox[{64, 65}]}, 
                    {Hue[0.010205847579708893`, 1, 1], LineBox[{65, 66}]}, 
                    {Hue[0.9173868767508842, 1, 1], LineBox[{66, 67}]}, 
                    {Hue[0.8247637208190898, 1, 1], LineBox[{67, 68}]}, 
                    {Hue[0.7323406252996997, 1, 1], LineBox[{68, 69}]}, 
                    {Hue[0.6401150014496022, 1, 1], LineBox[{69, 70}]}, 
                    {Hue[0.5480750215091739, 1, 1], LineBox[{70, 71}]}, 
                    {Hue[0.4562199245354366, 1, 1], LineBox[{71, 72}]}, 
                    {Hue[0.3645427633380875, 1, 1], LineBox[{72, 73}]}, 
                    {Hue[0.2730536155226162, 1, 1], LineBox[{73, 74}]}, 
                    {Hue[0.18173241675375307`, 1, 1], LineBox[{74, 75}]}, 
                    {Hue[0.09059190059179349, 1, 1], LineBox[{75, 76}]}, 
                    {Hue[0.9996208878511171, 1, 1], LineBox[{76, 77}]}, 
                    {Hue[0.9088202168452655, 1, 1], LineBox[{77, 78}]}, 
                    {Hue[0.8181778536844899, 1, 1], LineBox[{78, 79}]}, 
                    {Hue[0.7276989234844825, 1, 1], LineBox[{79, 80}]}, 
                    {Hue[0.6373785352377027, 1, 1], LineBox[{80, 81}]}, 
                    {Hue[0.5472076289406496, 1, 1], LineBox[{81, 82}]}, 
                    {Hue[0.4571930226530651, 1, 1], LineBox[{82, 83}]}, 
                    {Hue[0.36732415631788506`, 1, 1], LineBox[{83, 84}]}, 
                    {Hue[0.27760258708611185`, 1, 1], LineBox[{84, 85}]}, 
                    {Hue[0.18802137824938803`, 1, 1], LineBox[{85, 86}]}, 
                    {Hue[0.0985833771491798, 1, 1], LineBox[{86, 87}]}, 
                    {Hue[0.009279955353814947, 1, 1], LineBox[{87, 88}]}, 
                    {Hue[0.9201105090605955, 1, 1], LineBox[{88, 89}]}, 
                    {Hue[0.8310739664271323, 1, 1], LineBox[{89, 90}]}, 
                    {Hue[0.7421732143735179, 1, 1], LineBox[{90, 91}]}, 
                    {Hue[0.6534046333614987, 1, 1], LineBox[{91, 92}]}, 
                    {Hue[0.5647705320621916, 1, 1], LineBox[{92, 93}]}, 
                    {Hue[0.4762657974965162, 1, 1], LineBox[{93, 94}]}, 
                    {Hue[0.38788992140175527`, 1, 1], LineBox[{94, 95}]}, 
                    {Hue[0.2996453744549128, 1, 1], LineBox[{95, 96}]}, 
                    {Hue[0.21153790378021053`, 1, 1], LineBox[{96, 97}]}, 
                    {Hue[0.1235785372270038, 1, 1], LineBox[{97, 98}]}, 
                    {Hue[0.03577515435765192, 1, 1], LineBox[{98, 99}]}, 
                    {Hue[0.9481387495866579, 1, 1], LineBox[{99, 100}]}, 
                    {Hue[0.8606718183156182, 1, 1], LineBox[{100, 101}]}, 
                    {Hue[0.7733802821522706, 1, 1], LineBox[{101, 102}]}, 
                    {Hue[0.6862775149694955, 1, 1], LineBox[{102, 103}]}, 
                    {Hue[0.5993876090244299, 1, 1], LineBox[{103, 104}]}, 
                    {Hue[0.5127503459429368, 1, 1], LineBox[{104, 105}]}, 
                    {Hue[0.4264049556156808, 1, 1], LineBox[{105, 106}]}, 
                    {Hue[0.340391290138817, 1, 1], LineBox[{106, 107}]}, 
                    {Hue[0.2547382414912006, 1, 1], LineBox[{107, 108}]}, 
                    {Hue[0.16947463648226757`, 1, 1], LineBox[{108, 109}]}, 
                    {Hue[0.08464337306181322, 1, 1], LineBox[{109, 110}]}, 
                    {Hue[0.0003142444700447079, 1, 1], LineBox[{110, 111}]}, 
                    {Hue[0.9165919072600075, 1, 1], LineBox[{111, 112}]}, 
                    {Hue[0.8335995002856345, 1, 1], LineBox[{112, 113}]}, 
                    {Hue[0.751460053715487, 1, 1], LineBox[{113, 114}]}, 
                    {Hue[0.6702749790326042, 1, 1], LineBox[{114, 115}]}, 
                    {Hue[0.5901288601313921, 1, 1], LineBox[{115, 116}]}, 
                    {Hue[0.5111107307817198, 1, 1], LineBox[{116, 117}]}, 
                    {Hue[0.43335061972018757`, 1, 1], LineBox[{117, 118}]}, 
                    {Hue[0.3570418522475413, 1, 1], LineBox[{118, 119}]}, 
                    {Hue[0.2824419456203651, 1, 1], LineBox[{119, 120}]}, 
                    {Hue[0.20983775878173655`, 1, 1], LineBox[{120, 121}]}, 
                    {Hue[0.1395014372655653, 1, 1], LineBox[{121, 122}]}, 
                    {Hue[0.07165023971034112, 1, 1], LineBox[{122, 123}]}, 
                    {Hue[0.006431618568068022, 1, 1], LineBox[{123, 124}]}, 
                    {Hue[0.9439325440916397, 1, 1], LineBox[{124, 125}]}, 
                    {Hue[0.8842032007094285, 1, 1], LineBox[{125, 126}]}, 
                    {Hue[0.8272827330826636, 1, 1], LineBox[{126, 127}]}, 
                    {Hue[0.773216326087761, 1, 1], LineBox[{127, 128}]}, 
                    {Hue[0.7220643652275629, 1, 1], LineBox[{128, 129}]}, 
                    {Hue[0.6738998532258365, 1, 1], LineBox[{129, 130}]}, 
                    {Hue[0.6287990685292446, 1, 1], LineBox[{130, 131}]}, 
                    {Hue[0.5868341652474119, 1, 1], LineBox[{131, 132}]}, 
                    {Hue[0.5480628738596105, 1, 1], LineBox[{132, 133}]}, 
                    {Hue[0.5125236261206041, 1, 1], LineBox[{133, 134}]}, 
                    {Hue[0.48022998813085527`, 1, 1], LineBox[{134, 135}]}, 
                    {Hue[0.4511728596684511, 1, 1], LineBox[{135, 136}]}, 
                    {Hue[0.4253201936061853, 1, 1], LineBox[{136, 137}]}, 
                    {Hue[0.4026220415933217, 1, 1], LineBox[{137, 138}]}, 
                    {Hue[0.38301220867514024`, 1, 1], LineBox[{138, 139}]}, 
                    {Hue[0.36641158831638115`, 1, 1], LineBox[{139, 140}]}, 
                    {Hue[0.35273364848390876`, 1, 1], LineBox[{140, 141}]}, 
                    {Hue[0.3418848504579338, 1, 1], LineBox[{141, 142}]}, 
                    {Hue[0.3337689696405779, 1, 1], LineBox[{142, 143}]}, 
                    {Hue[0.3282876739531856, 1, 1], LineBox[{143, 144}]}, 
                    {Hue[0.32534234933601514`, 1, 1], LineBox[{144, 145}]}, 
                    {Hue[0.3248378520983274, 1, 1], LineBox[{145, 146}]}, 
                    {Hue[0.3266794362989239, 1, 1], LineBox[{146, 147}]}, 
                    {Hue[0.3307767146838967, 1, 1], LineBox[{147, 148}]}, 
                    {Hue[0.33704224104116143`, 1, 1], LineBox[{148, 149}]}, 
                    {Hue[0.3453928205084022, 1, 1], LineBox[{149, 150}]}, 
                    {Hue[0.3557493476841345, 1, 1], LineBox[{150, 151}]}, 
                    {Hue[0.36803727733320324`, 1, 1], LineBox[{151, 152}]}, 
                    {Hue[0.38218531751463014`, 1, 1], LineBox[{152, 153}]}, 
                    {Hue[0.3981269793902958, 1, 1], LineBox[{153, 154}]}, 
                    {Hue[0.415800089200965, 1, 1], LineBox[{154, 155}]}, 
                    {Hue[0.43514489696901626`, 1, 1], LineBox[{155, 156}]}, 
                    {Hue[0.456107123450531, 1, 1], LineBox[{156, 157}]}, 
                    {Hue[0.4786340231754143, 1, 1], LineBox[{157, 158}]}, 
                    {Hue[0.5026780811248652, 1, 1], LineBox[{158, 159}]}, 
                    {Hue[0.5281933975945959, 1, 1], LineBox[{159, 160}]}, 
                    {Hue[0.555136891773129, 1, 1], LineBox[{160, 161}]}, 
                    {Hue[0.5834691607915969, 1, 1], LineBox[{161, 162}]}, 
                    {Hue[0.6131526256736353, 1, 1], LineBox[{162, 163}]}, 
                    {Hue[0.6441525006497791, 1, 1], LineBox[{163, 164}]}, 
                    {Hue[0.6764355622085817, 1, 1], LineBox[{164, 165}]}, 
                    {Hue[0.7099706569321134, 1, 1], LineBox[{165, 166}]}, 
                    {Hue[0.7447288937616423, 1, 1], LineBox[{166, 167}]}, 
                    {Hue[0.7806831764942391, 1, 1], LineBox[{167, 168}]}, 
                    {Hue[0.817806593136449, 1, 1], LineBox[{168, 169}]}, 
                    {Hue[0.856076141486128, 1, 1], LineBox[{169, 170}]}, 
                    {Hue[0.8954681694293263, 1, 1], LineBox[{170, 171}]}, 
                    {Hue[0.9359614830231154, 1, 1], LineBox[{171, 172}]}, 
                    {Hue[0.9775353594293265, 1, 1], LineBox[{172, 173}]}, 
                    {Hue[0.02017020566685116, 1, 1], LineBox[{173, 174}]}, 
                    {Hue[0.06384797101923008, 1, 1], LineBox[{174, 175}]}, 
                    {Hue[0.10855083967090552`, 1, 1], LineBox[{175, 176}]}, 
                    {Hue[0.15426253041459265`, 1, 1], LineBox[{176, 177}]}, 
                    {Hue[0.20096706165192937`, 1, 1], LineBox[{177, 178}]}, 
                    {Hue[0.2486501360831702, 1, 1], LineBox[{178, 179}]}, 
                    {Hue[0.2972974266819873, 1, 1], LineBox[{179, 180}]}, 
                    {Hue[0.3468949648771804, 1, 1], LineBox[{180, 181}]}, 
                    {Hue[0.3974301349949464, 1, 1], LineBox[{181, 182}]}, 
                    {Hue[0.44889051576090216`, 1, 1], LineBox[{182, 183}]}, 
                    {Hue[0.5012641862857377, 1, 1], LineBox[{183, 184}]}, 
                    {Hue[0.5545404603455091, 1, 1], LineBox[{184, 185}]}, 
                    {Hue[0.6087074756721977, 1, 1], LineBox[{185, 186}]}, 
                    {Hue[0.6637555293688033, 1, 1], LineBox[{186, 187}]}, 
                    {Hue[0.7196745235769924, 1, 1], LineBox[{187, 188}]}, 
                    {Hue[0.776454638312091, 1, 1], LineBox[{188, 189}]}, 
                    {Hue[0.8340862590134883, 1, 1], LineBox[{189, 190}]}, 
                    {Hue[0.8925618762146349, 1, 1], LineBox[{190, 191}]}, 
                    {Hue[0.9518725157540959, 1, 1], LineBox[{191, 192}]}, 
                    {Hue[0.01201033697292499, 1, 1], LineBox[{192, 193}]}, 
                    {Hue[0.07296659247411895, 1, 1], LineBox[{193, 194}]}, 
                    {Hue[0.134734925300045, 1, 1], LineBox[{194, 195}]}, 
                    {Hue[0.197305564158579, 1, 1], LineBox[{195, 196}]}, 
                    {Hue[0.2606739988665761, 1, 1], LineBox[{196, 197}]}, 
                    {Hue[0.32483191760817787`, 1, 1], LineBox[{197, 198}]}, 
                    {Hue[0.38977237153196254`, 1, 1], LineBox[{198, 199}]}, 
                    {Hue[0.4554907569809305, 1, 1], LineBox[{199, 200}]}, 
                    {Hue[0.5219797009100673, 1, 1], LineBox[{200, 201}]}, 
                    {Hue[0.5892341605561915, 1, 1], LineBox[{201, 202}]}, 
                    {Hue[0.657246872448343, 1, 1], LineBox[{202, 203}]}, 
                    {Hue[0.7260117989041965, 1, 1], LineBox[{203, 204}]}, 
                    {Hue[0.7955250253211213, 1, 1], LineBox[{204, 205}]}, 
                    {Hue[0.8657797896021193, 1, 1], LineBox[{205, 206}]}, 
                    {Hue[0.9367719499178244, 1, 1], LineBox[{206, 207}]}, 
                    {Hue[0.008495195879954513, 1, 1], LineBox[{207, 208}]}, 
                    {Hue[0.0809459758358265, 1, 1], LineBox[{208, 209}]}, 
                    {Hue[0.15412001963161848`, 1, 1], LineBox[{209, 210}]}, 
                    {Hue[0.2280115923818089, 1, 1], LineBox[{210, 211}]}, 
                    {Hue[0.3026172384645148, 1, 1], LineBox[{211, 212}]}, 
                    {Hue[0.3779292229301128, 1, 1], LineBox[{212, 213}]}, 
                    {Hue[0.4539469430882079, 1, 1], LineBox[{213, 214}]}, 
                    {Hue[0.5306651254733961, 1, 1], LineBox[{214, 215}]}, 
                    {Hue[0.6080786482755536, 1, 1], LineBox[{215, 216}]}, 
                    {Hue[0.6861849685526418, 1, 1], LineBox[{216, 217}]}, 
                    {Hue[0.7649800556325819, 1, 1], LineBox[{217, 218}]}, 
                    {Hue[0.8444590226034463, 1, 1], LineBox[{218, 219}]}, 
                    {Hue[0.9246179861927123, 1, 1], LineBox[{219, 220}]}, 
                    {Hue[0.0054564117259276864`, 1, 1], 
                    LineBox[{220, 221}]}, 
                    {Hue[0.08696888006741216, 1, 1], LineBox[{221, 222}]}, 
                    {Hue[0.16914750535384115`, 1, 1], LineBox[{222, 223}]}, 
                    {Hue[0.2519914361670223, 1, 1], LineBox[{223, 224}]}, 
                    {Hue[0.3355017653036084, 1, 1], LineBox[{224, 225}]}, 
                    {Hue[0.41967365987477934`, 1, 1], LineBox[{225, 226}]}, 
                    {Hue[0.5045004497746628, 1, 1], LineBox[{226, 227}]}, 
                    {Hue[0.5899817557355067, 1, 1], LineBox[{227, 228}]}, 
                    {Hue[0.6761137981804346, 1, 1], LineBox[{228, 229}]}, 
                    {Hue[0.7628854856167173, 1, 1], LineBox[{229, 230}]}, 
                    {Hue[0.8503116541986834, 1, 1], LineBox[{230, 231}]}, 
                    {Hue[0.9383888140536001, 1, 1], LineBox[{231, 232}]}, 
                    {Hue[0.02709017049981145, 1, 1], LineBox[{232, 233}]}, 
                    {Hue[0.11642414389020968`, 1, 1], LineBox[{233, 234}]}, 
                    {Hue[0.20640715237073284`, 1, 1], LineBox[{234, 235}]}, 
                    {Hue[0.297017601531361, 1, 1], LineBox[{235, 236}]}, 
                    {Hue[0.38824632762828315`, 1, 1], LineBox[{236, 237}]}, 
                    {Hue[0.4801090758515208, 1, 1], LineBox[{237, 238}]}, 
                    {Hue[0.5725864431600319, 1, 1], LineBox[{238, 239}]}, 
                    {Hue[0.665687847174255, 1, 1], LineBox[{239, 240}]}, 
                    {Hue[0.7594259975222694, 1, 1], LineBox[{240, 241}]}, 
                    {Hue[0.8537592849553937, 1, 1], LineBox[{241, 242}]}, 
                    {Hue[0.948706513230035, 1, 1], LineBox[{242, 243}]}, 
                    {Hue[0.04426775150904171, 1, 1], LineBox[{243, 244}]}, 
                    {Hue[0.1404108073613589, 1, 1], LineBox[{244, 245}]}, 
                    {Hue[0.237191244358674, 1, 1], LineBox[{245, 246}]}, 
                    {Hue[0.3346110854878864, 1, 1], LineBox[{246, 247}]}, 
                    {Hue[0.43260862207299355`, 1, 1], LineBox[{247, 248}]}, 
                    {Hue[0.5311723937875716, 1, 1], LineBox[{248, 249}]}, 
                    {Hue[0.6303165816593157, 1, 1], LineBox[{249, 250}]}, 
                    {Hue[0.7300926755163386, 1, 1], LineBox[{250, 251}]}}}},
                    VertexColors->{
                    Hue[0.974286712374846, 1, 1], 
                    Hue[0.08446566071998841, 1, 1], 
                    Hue[0.7942339412854, 1, 1], 
                    Hue[0.8492028857989099, 1, 1], 
                    Hue[0.6022978636183001, 1, 1], 
                    Hue[0.7606479149378287, 1, 1], 
                    Hue[0.578859153398882, 1, 1], 
                    Hue[0.30877927036783076`, 1, 1], 
                    Hue[0.31482924609675245`, 1, 1], 
                    Hue[0.6200413157728587, 1, 1], 
                    Hue[0.09442359141006498, 1, 1], 
                    Hue[0.23001877330545184`, 1, 1], 
                    Hue[0.9760886121604145, 1, 1], 
                    Hue[0.9467880234325805, 1, 1], 
                    Hue[0.9028940736581215, 1, 1], 
                    Hue[0.752209199149744, 1, 1], 
                    Hue[0.6911669551999422, 1, 1], 
                    Hue[0.6219826929738932, 1, 1], 
                    Hue[0.5012750928019923, 1, 1], 
                    Hue[0.4060824274587364, 1, 1], 
                    Hue[0.3222783622768835, 1, 1], 
                    Hue[0.22185466654030803`, 1, 1], 
                    Hue[0.11723128500502794`, 1, 1], 
                    Hue[0.005902445675020168, 1, 1], 
                    Hue[0.9069522018498736, 1, 1], 
                    Hue[0.8068527599341031, 1, 1], 
                    Hue[0.708878572463234, 1, 1], 
                    Hue[0.6079610261392703, 1, 1], 
                    Hue[0.5040843867085472, 1, 1], 
                    Hue[0.40698470072433135`, 1, 1], 
                    Hue[0.3020483743054194, 1, 1], 
                    Hue[0.20223825272965743`, 1, 1], 
                    Hue[0.09995156837997692, 1, 1], 
                    Hue[0.9973121322051787, 1, 1], 
                    Hue[0.8993192587880702, 1, 1], 
                    Hue[0.7994909922000972, 1, 1], 
                    Hue[0.7001832107207604, 1, 1], 
                    Hue[0.6012734955736141, 1, 1], 
                    Hue[0.5018059684686444, 1, 1], 
                    Hue[0.4036261093302133, 1, 1], 
                    Hue[0.3050426183203099, 1, 1], 
                    Hue[0.20640472381167882`, 1, 1], 
                    Hue[0.10812979713222298`, 1, 1], 
                    Hue[0.010246775437848158`, 1, 1], 
                    Hue[0.9125933430525395, 1, 1], 
                    Hue[0.8151261615159525, 1, 1], 
                    Hue[0.7180481601790828, 1, 1], 
                    Hue[0.6211638292725423, 1, 1], 
                    Hue[0.5245698891455377, 1, 1], 
                    Hue[0.4283083559045919, 1, 1], 
                    Hue[0.3321726452629961, 1, 1], 
                    Hue[0.23626986406217826`, 1, 1], 
                    Hue[0.14059393740808274`, 1, 1], 
                    Hue[0.04514473076327826, 1, 1], 
                    Hue[0.9499413082274694, 1, 1], 
                    Hue[0.8549726697253752, 1, 1], 
                    Hue[0.760249505394373, 1, 1], 
                    Hue[0.6657461239613698, 1, 1], 
                    Hue[0.5714732426671167, 1, 1], 
                    Hue[0.4774118319071657, 1, 1], 
                    Hue[0.3835360586193685, 1, 1], 
                    Hue[0.28990642954043827`, 1, 1], 
                    Hue[0.19646904601881474`, 1, 1], 
                    Hue[0.10324490144884163`, 1, 1], 
                    Hue[0.010205847579708893`, 1, 1], 
                    Hue[0.9173868767508842, 1, 1], 
                    Hue[0.8247637208190898, 1, 1], 
                    Hue[0.7323406252996997, 1, 1], 
                    Hue[0.6401150014496022, 1, 1], 
                    Hue[0.5480750215091739, 1, 1], 
                    Hue[0.4562199245354366, 1, 1], 
                    Hue[0.3645427633380875, 1, 1], 
                    Hue[0.2730536155226162, 1, 1], 
                    Hue[0.18173241675375307`, 1, 1], 
                    Hue[0.09059190059179349, 1, 1], 
                    Hue[0.9996208878511171, 1, 1], 
                    Hue[0.9088202168452655, 1, 1], 
                    Hue[0.8181778536844899, 1, 1], 
                    Hue[0.7276989234844825, 1, 1], 
                    Hue[0.6373785352377027, 1, 1], 
                    Hue[0.5472076289406496, 1, 1], 
                    Hue[0.4571930226530651, 1, 1], 
                    Hue[0.36732415631788506`, 1, 1], 
                    Hue[0.27760258708611185`, 1, 1], 
                    Hue[0.18802137824938803`, 1, 1], 
                    Hue[0.0985833771491798, 1, 1], 
                    Hue[0.009279955353814947, 1, 1], 
                    Hue[0.9201105090605955, 1, 1], 
                    Hue[0.8310739664271323, 1, 1], 
                    Hue[0.7421732143735179, 1, 1], 
                    Hue[0.6534046333614987, 1, 1], 
                    Hue[0.5647705320621916, 1, 1], 
                    Hue[0.4762657974965162, 1, 1], 
                    Hue[0.38788992140175527`, 1, 1], 
                    Hue[0.2996453744549128, 1, 1], 
                    Hue[0.21153790378021053`, 1, 1], 
                    Hue[0.1235785372270038, 1, 1], 
                    Hue[0.03577515435765192, 1, 1], 
                    Hue[0.9481387495866579, 1, 1], 
                    Hue[0.8606718183156182, 1, 1], 
                    Hue[0.7733802821522706, 1, 1], 
                    Hue[0.6862775149694955, 1, 1], 
                    Hue[0.5993876090244299, 1, 1], 
                    Hue[0.5127503459429368, 1, 1], 
                    Hue[0.4264049556156808, 1, 1], 
                    Hue[0.340391290138817, 1, 1], 
                    Hue[0.2547382414912006, 1, 1], 
                    Hue[0.16947463648226757`, 1, 1], 
                    Hue[0.08464337306181322, 1, 1], 
                    Hue[0.0003142444700447079, 1, 1], 
                    Hue[0.9165919072600075, 1, 1], 
                    Hue[0.8335995002856345, 1, 1], 
                    Hue[0.751460053715487, 1, 1], 
                    Hue[0.6702749790326042, 1, 1], 
                    Hue[0.5901288601313921, 1, 1], 
                    Hue[0.5111107307817198, 1, 1], 
                    Hue[0.43335061972018757`, 1, 1], 
                    Hue[0.3570418522475413, 1, 1], 
                    Hue[0.2824419456203651, 1, 1], 
                    Hue[0.20983775878173655`, 1, 1], 
                    Hue[0.1395014372655653, 1, 1], 
                    Hue[0.07165023971034112, 1, 1], 
                    Hue[0.006431618568068022, 1, 1], 
                    Hue[0.9439325440916397, 1, 1], 
                    Hue[0.8842032007094285, 1, 1], 
                    Hue[0.8272827330826636, 1, 1], 
                    Hue[0.773216326087761, 1, 1], 
                    Hue[0.7220643652275629, 1, 1], 
                    Hue[0.6738998532258365, 1, 1], 
                    Hue[0.6287990685292446, 1, 1], 
                    Hue[0.5868341652474119, 1, 1], 
                    Hue[0.5480628738596105, 1, 1], 
                    Hue[0.5125236261206041, 1, 1], 
                    Hue[0.48022998813085527`, 1, 1], 
                    Hue[0.4511728596684511, 1, 1], 
                    Hue[0.4253201936061853, 1, 1], 
                    Hue[0.4026220415933217, 1, 1], 
                    Hue[0.38301220867514024`, 1, 1], 
                    Hue[0.36641158831638115`, 1, 1], 
                    Hue[0.35273364848390876`, 1, 1], 
                    Hue[0.3418848504579338, 1, 1], 
                    Hue[0.3337689696405779, 1, 1], 
                    Hue[0.3282876739531856, 1, 1], 
                    Hue[0.32534234933601514`, 1, 1], 
                    Hue[0.3248378520983274, 1, 1], 
                    Hue[0.3266794362989239, 1, 1], 
                    Hue[0.3307767146838967, 1, 1], 
                    Hue[0.33704224104116143`, 1, 1], 
                    Hue[0.3453928205084022, 1, 1], 
                    Hue[0.3557493476841345, 1, 1], 
                    Hue[0.36803727733320324`, 1, 1], 
                    Hue[0.38218531751463014`, 1, 1], 
                    Hue[0.3981269793902958, 1, 1], 
                    Hue[0.415800089200965, 1, 1], 
                    Hue[0.43514489696901626`, 1, 1], 
                    Hue[0.456107123450531, 1, 1], 
                    Hue[0.4786340231754143, 1, 1], 
                    Hue[0.5026780811248652, 1, 1], 
                    Hue[0.5281933975945959, 1, 1], 
                    Hue[0.555136891773129, 1, 1], 
                    Hue[0.5834691607915969, 1, 1], 
                    Hue[0.6131526256736353, 1, 1], 
                    Hue[0.6441525006497791, 1, 1], 
                    Hue[0.6764355622085817, 1, 1], 
                    Hue[0.7099706569321134, 1, 1], 
                    Hue[0.7447288937616423, 1, 1], 
                    Hue[0.7806831764942391, 1, 1], 
                    Hue[0.817806593136449, 1, 1], 
                    Hue[0.856076141486128, 1, 1], 
                    Hue[0.8954681694293263, 1, 1], 
                    Hue[0.9359614830231154, 1, 1], 
                    Hue[0.9775353594293265, 1, 1], 
                    Hue[0.02017020566685116, 1, 1], 
                    Hue[0.06384797101923008, 1, 1], 
                    Hue[0.10855083967090552`, 1, 1], 
                    Hue[0.15426253041459265`, 1, 1], 
                    Hue[0.20096706165192937`, 1, 1], 
                    Hue[0.2486501360831702, 1, 1], 
                    Hue[0.2972974266819873, 1, 1], 
                    Hue[0.3468949648771804, 1, 1], 
                    Hue[0.3974301349949464, 1, 1], 
                    Hue[0.44889051576090216`, 1, 1], 
                    Hue[0.5012641862857377, 1, 1], 
                    Hue[0.5545404603455091, 1, 1], 
                    Hue[0.6087074756721977, 1, 1], 
                    Hue[0.6637555293688033, 1, 1], 
                    Hue[0.7196745235769924, 1, 1], 
                    Hue[0.776454638312091, 1, 1], 
                    Hue[0.8340862590134883, 1, 1], 
                    Hue[0.8925618762146349, 1, 1], 
                    Hue[0.9518725157540959, 1, 1], 
                    Hue[0.01201033697292499, 1, 1], 
                    Hue[0.07296659247411895, 1, 1], 
                    Hue[0.134734925300045, 1, 1], 
                    Hue[0.197305564158579, 1, 1], 
                    Hue[0.2606739988665761, 1, 1], 
                    Hue[0.32483191760817787`, 1, 1], 
                    Hue[0.38977237153196254`, 1, 1], 
                    Hue[0.4554907569809305, 1, 1], 
                    Hue[0.5219797009100673, 1, 1], 
                    Hue[0.5892341605561915, 1, 1], 
                    Hue[0.657246872448343, 1, 1], 
                    Hue[0.7260117989041965, 1, 1], 
                    Hue[0.7955250253211213, 1, 1], 
                    Hue[0.8657797896021193, 1, 1], 
                    Hue[0.9367719499178244, 1, 1], 
                    Hue[0.008495195879954513, 1, 1], 
                    Hue[0.0809459758358265, 1, 1], 
                    Hue[0.15412001963161848`, 1, 1], 
                    Hue[0.2280115923818089, 1, 1], 
                    Hue[0.3026172384645148, 1, 1], 
                    Hue[0.3779292229301128, 1, 1], 
                    Hue[0.4539469430882079, 1, 1], 
                    Hue[0.5306651254733961, 1, 1], 
                    Hue[0.6080786482755536, 1, 1], 
                    Hue[0.6861849685526418, 1, 1], 
                    Hue[0.7649800556325819, 1, 1], 
                    Hue[0.8444590226034463, 1, 1], 
                    Hue[0.9246179861927123, 1, 1], 
                    Hue[0.0054564117259276864`, 1, 1], 
                    Hue[0.08696888006741216, 1, 1], 
                    Hue[0.16914750535384115`, 1, 1], 
                    Hue[0.2519914361670223, 1, 1], 
                    Hue[0.3355017653036084, 1, 1], 
                    Hue[0.41967365987477934`, 1, 1], 
                    Hue[0.5045004497746628, 1, 1], 
                    Hue[0.5899817557355067, 1, 1], 
                    Hue[0.6761137981804346, 1, 1], 
                    Hue[0.7628854856167173, 1, 1], 
                    Hue[0.8503116541986834, 1, 1], 
                    Hue[0.9383888140536001, 1, 1], 
                    Hue[0.02709017049981145, 1, 1], 
                    Hue[0.11642414389020968`, 1, 1], 
                    Hue[0.20640715237073284`, 1, 1], 
                    Hue[0.297017601531361, 1, 1], 
                    Hue[0.38824632762828315`, 1, 1], 
                    Hue[0.4801090758515208, 1, 1], 
                    Hue[0.5725864431600319, 1, 1], 
                    Hue[0.665687847174255, 1, 1], 
                    Hue[0.7594259975222694, 1, 1], 
                    Hue[0.8537592849553937, 1, 1], 
                    Hue[0.948706513230035, 1, 1], 
                    Hue[0.04426775150904171, 1, 1], 
                    Hue[0.1404108073613589, 1, 1], 
                    Hue[0.237191244358674, 1, 1], 
                    Hue[0.3346110854878864, 1, 1], 
                    Hue[0.43260862207299355`, 1, 1], 
                    Hue[0.5311723937875716, 1, 1], 
                    Hue[0.6303165816593157, 1, 1], 
                    Hue[0.7300926755163386, 1, 1], 
                    Hue[0.830492702282897, 1, 1], 
                    Hue[0.974286712374846, 1, 1], 
                    Hue[0.08446566071998841, 1, 1], 
                    Hue[0.08446566071998841, 1, 1], 
                    Hue[0.7942339412854, 1, 1], 
                    Hue[0.7942339412854, 1, 1], 
                    Hue[0.8492028857989099, 1, 1], 
                    Hue[0.8492028857989099, 1, 1], 
                    Hue[0.6022978636183001, 1, 1], 
                    Hue[0.6022978636183001, 1, 1], 
                    Hue[0.7606479149378287, 1, 1], 
                    Hue[0.7606479149378287, 1, 1], 
                    Hue[0.578859153398882, 1, 1], 
                    Hue[0.578859153398882, 1, 1], 
                    Hue[0.30877927036783076`, 1, 1], 
                    Hue[0.30877927036783076`, 1, 1], 
                    Hue[0.31482924609675245`, 1, 1], 
                    Hue[0.31482924609675245`, 1, 1], 
                    Hue[0.6200413157728587, 1, 1], 
                    Hue[0.6200413157728587, 1, 1], 
                    Hue[0.09442359141006498, 1, 1], 
                    Hue[0.09442359141006498, 1, 1], 
                    Hue[0.23001877330545184`, 1, 1], 
                    Hue[0.23001877330545184`, 1, 1], 
                    Hue[0.9760886121604145, 1, 1], 
                    Hue[0.9760886121604145, 1, 1], 
                    Hue[0.9467880234325805, 1, 1], 
                    Hue[0.9467880234325805, 1, 1], 
                    Hue[0.9028940736581215, 1, 1], 
                    Hue[0.9028940736581215, 1, 1], 
                    Hue[0.752209199149744, 1, 1], 
                    Hue[0.752209199149744, 1, 1], 
                    Hue[0.6911669551999422, 1, 1], 
                    Hue[0.6911669551999422, 1, 1], 
                    Hue[0.6219826929738932, 1, 1], 
                    Hue[0.6219826929738932, 1, 1], 
                    Hue[0.5012750928019923, 1, 1], 
                    Hue[0.5012750928019923, 1, 1], 
                    Hue[0.4060824274587364, 1, 1], 
                    Hue[0.4060824274587364, 1, 1], 
                    Hue[0.3222783622768835, 1, 1], 
                    Hue[0.3222783622768835, 1, 1], 
                    Hue[0.22185466654030803`, 1, 1], 
                    Hue[0.22185466654030803`, 1, 1], 
                    Hue[0.11723128500502794`, 1, 1], 
                    Hue[0.11723128500502794`, 1, 1], 
                    Hue[0.005902445675020168, 1, 1], 
                    Hue[0.005902445675020168, 1, 1], 
                    Hue[0.9069522018498736, 1, 1], 
                    Hue[0.9069522018498736, 1, 1], 
                    Hue[0.8068527599341031, 1, 1], 
                    Hue[0.8068527599341031, 1, 1], 
                    Hue[0.708878572463234, 1, 1], 
                    Hue[0.708878572463234, 1, 1], 
                    Hue[0.6079610261392703, 1, 1], 
                    Hue[0.6079610261392703, 1, 1], 
                    Hue[0.5040843867085472, 1, 1], 
                    Hue[0.5040843867085472, 1, 1], 
                    Hue[0.40698470072433135`, 1, 1], 
                    Hue[0.40698470072433135`, 1, 1], 
                    Hue[0.3020483743054194, 1, 1], 
                    Hue[0.3020483743054194, 1, 1], 
                    Hue[0.20223825272965743`, 1, 1], 
                    Hue[0.20223825272965743`, 1, 1], 
                    Hue[0.09995156837997692, 1, 1], 
                    Hue[0.09995156837997692, 1, 1], 
                    Hue[0.9973121322051787, 1, 1], 
                    Hue[0.9973121322051787, 1, 1], 
                    Hue[0.8993192587880702, 1, 1], 
                    Hue[0.8993192587880702, 1, 1], 
                    Hue[0.7994909922000972, 1, 1], 
                    Hue[0.7994909922000972, 1, 1], 
                    Hue[0.7001832107207604, 1, 1], 
                    Hue[0.7001832107207604, 1, 1], 
                    Hue[0.6012734955736141, 1, 1], 
                    Hue[0.6012734955736141, 1, 1], 
                    Hue[0.5018059684686444, 1, 1], 
                    Hue[0.5018059684686444, 1, 1], 
                    Hue[0.4036261093302133, 1, 1], 
                    Hue[0.4036261093302133, 1, 1], 
                    Hue[0.3050426183203099, 1, 1], 
                    Hue[0.3050426183203099, 1, 1], 
                    Hue[0.20640472381167882`, 1, 1], 
                    Hue[0.20640472381167882`, 1, 1], 
                    Hue[0.10812979713222298`, 1, 1], 
                    Hue[0.10812979713222298`, 1, 1], 
                    Hue[0.010246775437848158`, 1, 1], 
                    Hue[0.010246775437848158`, 1, 1], 
                    Hue[0.9125933430525395, 1, 1], 
                    Hue[0.9125933430525395, 1, 1], 
                    Hue[0.8151261615159525, 1, 1], 
                    Hue[0.8151261615159525, 1, 1], 
                    Hue[0.7180481601790828, 1, 1], 
                    Hue[0.7180481601790828, 1, 1], 
                    Hue[0.6211638292725423, 1, 1], 
                    Hue[0.6211638292725423, 1, 1], 
                    Hue[0.5245698891455377, 1, 1], 
                    Hue[0.5245698891455377, 1, 1], 
                    Hue[0.4283083559045919, 1, 1], 
                    Hue[0.4283083559045919, 1, 1], 
                    Hue[0.3321726452629961, 1, 1], 
                    Hue[0.3321726452629961, 1, 1], 
                    Hue[0.23626986406217826`, 1, 1], 
                    Hue[0.23626986406217826`, 1, 1], 
                    Hue[0.14059393740808274`, 1, 1], 
                    Hue[0.14059393740808274`, 1, 1], 
                    Hue[0.04514473076327826, 1, 1], 
                    Hue[0.04514473076327826, 1, 1], 
                    Hue[0.9499413082274694, 1, 1], 
                    Hue[0.9499413082274694, 1, 1], 
                    Hue[0.8549726697253752, 1, 1], 
                    Hue[0.8549726697253752, 1, 1], 
                    Hue[0.760249505394373, 1, 1], 
                    Hue[0.760249505394373, 1, 1], 
                    Hue[0.6657461239613698, 1, 1], 
                    Hue[0.6657461239613698, 1, 1], 
                    Hue[0.5714732426671167, 1, 1], 
                    Hue[0.5714732426671167, 1, 1], 
                    Hue[0.4774118319071657, 1, 1], 
                    Hue[0.4774118319071657, 1, 1], 
                    Hue[0.3835360586193685, 1, 1], 
                    Hue[0.3835360586193685, 1, 1], 
                    Hue[0.28990642954043827`, 1, 1], 
                    Hue[0.28990642954043827`, 1, 1], 
                    Hue[0.19646904601881474`, 1, 1], 
                    Hue[0.19646904601881474`, 1, 1], 
                    Hue[0.10324490144884163`, 1, 1], 
                    Hue[0.10324490144884163`, 1, 1], 
                    Hue[0.010205847579708893`, 1, 1], 
                    Hue[0.010205847579708893`, 1, 1], 
                    Hue[0.9173868767508842, 1, 1], 
                    Hue[0.9173868767508842, 1, 1], 
                    Hue[0.8247637208190898, 1, 1], 
                    Hue[0.8247637208190898, 1, 1], 
                    Hue[0.7323406252996997, 1, 1], 
                    Hue[0.7323406252996997, 1, 1], 
                    Hue[0.6401150014496022, 1, 1], 
                    Hue[0.6401150014496022, 1, 1], 
                    Hue[0.5480750215091739, 1, 1], 
                    Hue[0.5480750215091739, 1, 1], 
                    Hue[0.4562199245354366, 1, 1], 
                    Hue[0.4562199245354366, 1, 1], 
                    Hue[0.3645427633380875, 1, 1], 
                    Hue[0.3645427633380875, 1, 1], 
                    Hue[0.2730536155226162, 1, 1], 
                    Hue[0.2730536155226162, 1, 1], 
                    Hue[0.18173241675375307`, 1, 1], 
                    Hue[0.18173241675375307`, 1, 1], 
                    Hue[0.09059190059179349, 1, 1], 
                    Hue[0.09059190059179349, 1, 1], 
                    Hue[0.9996208878511171, 1, 1], 
                    Hue[0.9996208878511171, 1, 1], 
                    Hue[0.9088202168452655, 1, 1], 
                    Hue[0.9088202168452655, 1, 1], 
                    Hue[0.8181778536844899, 1, 1], 
                    Hue[0.8181778536844899, 1, 1], 
                    Hue[0.7276989234844825, 1, 1], 
                    Hue[0.7276989234844825, 1, 1], 
                    Hue[0.6373785352377027, 1, 1], 
                    Hue[0.6373785352377027, 1, 1], 
                    Hue[0.5472076289406496, 1, 1], 
                    Hue[0.5472076289406496, 1, 1], 
                    Hue[0.4571930226530651, 1, 1], 
                    Hue[0.4571930226530651, 1, 1], 
                    Hue[0.36732415631788506`, 1, 1], 
                    Hue[0.36732415631788506`, 1, 1], 
                    Hue[0.27760258708611185`, 1, 1], 
                    Hue[0.27760258708611185`, 1, 1], 
                    Hue[0.18802137824938803`, 1, 1], 
                    Hue[0.18802137824938803`, 1, 1], 
                    Hue[0.0985833771491798, 1, 1], 
                    Hue[0.0985833771491798, 1, 1], 
                    Hue[0.009279955353814947, 1, 1], 
                    Hue[0.009279955353814947, 1, 1], 
                    Hue[0.9201105090605955, 1, 1], 
                    Hue[0.9201105090605955, 1, 1], 
                    Hue[0.8310739664271323, 1, 1], 
                    Hue[0.8310739664271323, 1, 1], 
                    Hue[0.7421732143735179, 1, 1], 
                    Hue[0.7421732143735179, 1, 1], 
                    Hue[0.6534046333614987, 1, 1], 
                    Hue[0.6534046333614987, 1, 1], 
                    Hue[0.5647705320621916, 1, 1], 
                    Hue[0.5647705320621916, 1, 1], 
                    Hue[0.4762657974965162, 1, 1], 
                    Hue[0.4762657974965162, 1, 1], 
                    Hue[0.38788992140175527`, 1, 1], 
                    Hue[0.38788992140175527`, 1, 1], 
                    Hue[0.2996453744549128, 1, 1], 
                    Hue[0.2996453744549128, 1, 1], 
                    Hue[0.21153790378021053`, 1, 1], 
                    Hue[0.21153790378021053`, 1, 1], 
                    Hue[0.1235785372270038, 1, 1], 
                    Hue[0.1235785372270038, 1, 1], 
                    Hue[0.03577515435765192, 1, 1], 
                    Hue[0.03577515435765192, 1, 1], 
                    Hue[0.9481387495866579, 1, 1], 
                    Hue[0.9481387495866579, 1, 1], 
                    Hue[0.8606718183156182, 1, 1], 
                    Hue[0.8606718183156182, 1, 1], 
                    Hue[0.7733802821522706, 1, 1], 
                    Hue[0.7733802821522706, 1, 1], 
                    Hue[0.6862775149694955, 1, 1], 
                    Hue[0.6862775149694955, 1, 1], 
                    Hue[0.5993876090244299, 1, 1], 
                    Hue[0.5993876090244299, 1, 1], 
                    Hue[0.5127503459429368, 1, 1], 
                    Hue[0.5127503459429368, 1, 1], 
                    Hue[0.4264049556156808, 1, 1], 
                    Hue[0.4264049556156808, 1, 1], 
                    Hue[0.340391290138817, 1, 1], 
                    Hue[0.340391290138817, 1, 1], 
                    Hue[0.2547382414912006, 1, 1], 
                    Hue[0.2547382414912006, 1, 1], 
                    Hue[0.16947463648226757`, 1, 1], 
                    Hue[0.16947463648226757`, 1, 1], 
                    Hue[0.08464337306181322, 1, 1], 
                    Hue[0.08464337306181322, 1, 1], 
                    Hue[0.0003142444700447079, 1, 1], 
                    Hue[0.0003142444700447079, 1, 1], 
                    Hue[0.9165919072600075, 1, 1], 
                    Hue[0.9165919072600075, 1, 1], 
                    Hue[0.8335995002856345, 1, 1], 
                    Hue[0.8335995002856345, 1, 1], 
                    Hue[0.751460053715487, 1, 1], 
                    Hue[0.751460053715487, 1, 1], 
                    Hue[0.6702749790326042, 1, 1], 
                    Hue[0.6702749790326042, 1, 1], 
                    Hue[0.5901288601313921, 1, 1], 
                    Hue[0.5901288601313921, 1, 1], 
                    Hue[0.5111107307817198, 1, 1], 
                    Hue[0.5111107307817198, 1, 1], 
                    Hue[0.43335061972018757`, 1, 1], 
                    Hue[0.43335061972018757`, 1, 1], 
                    Hue[0.3570418522475413, 1, 1], 
                    Hue[0.3570418522475413, 1, 1], 
                    Hue[0.2824419456203651, 1, 1], 
                    Hue[0.2824419456203651, 1, 1], 
                    Hue[0.20983775878173655`, 1, 1], 
                    Hue[0.20983775878173655`, 1, 1], 
                    Hue[0.1395014372655653, 1, 1], 
                    Hue[0.1395014372655653, 1, 1], 
                    Hue[0.07165023971034112, 1, 1], 
                    Hue[0.07165023971034112, 1, 1], 
                    Hue[0.006431618568068022, 1, 1], 
                    Hue[0.006431618568068022, 1, 1], 
                    Hue[0.9439325440916397, 1, 1], 
                    Hue[0.9439325440916397, 1, 1], 
                    Hue[0.8842032007094285, 1, 1], 
                    Hue[0.8842032007094285, 1, 1], 
                    Hue[0.8272827330826636, 1, 1], 
                    Hue[0.8272827330826636, 1, 1], 
                    Hue[0.773216326087761, 1, 1], 
                    Hue[0.773216326087761, 1, 1], 
                    Hue[0.7220643652275629, 1, 1], 
                    Hue[0.7220643652275629, 1, 1], 
                    Hue[0.6738998532258365, 1, 1], 
                    Hue[0.6738998532258365, 1, 1], 
                    Hue[0.6287990685292446, 1, 1], 
                    Hue[0.6287990685292446, 1, 1], 
                    Hue[0.5868341652474119, 1, 1], 
                    Hue[0.5868341652474119, 1, 1], 
                    Hue[0.5480628738596105, 1, 1], 
                    Hue[0.5480628738596105, 1, 1], 
                    Hue[0.5125236261206041, 1, 1], 
                    Hue[0.5125236261206041, 1, 1], 
                    Hue[0.48022998813085527`, 1, 1], 
                    Hue[0.48022998813085527`, 1, 1], 
                    Hue[0.4511728596684511, 1, 1], 
                    Hue[0.4511728596684511, 1, 1], 
                    Hue[0.4253201936061853, 1, 1], 
                    Hue[0.4253201936061853, 1, 1], 
                    Hue[0.4026220415933217, 1, 1], 
                    Hue[0.4026220415933217, 1, 1], 
                    Hue[0.38301220867514024`, 1, 1], 
                    Hue[0.38301220867514024`, 1, 1], 
                    Hue[0.36641158831638115`, 1, 1], 
                    Hue[0.36641158831638115`, 1, 1], 
                    Hue[0.35273364848390876`, 1, 1], 
                    Hue[0.35273364848390876`, 1, 1], 
                    Hue[0.3418848504579338, 1, 1], 
                    Hue[0.3418848504579338, 1, 1], 
                    Hue[0.3337689696405779, 1, 1], 
                    Hue[0.3337689696405779, 1, 1], 
                    Hue[0.3282876739531856, 1, 1], 
                    Hue[0.3282876739531856, 1, 1], 
                    Hue[0.32534234933601514`, 1, 1], 
                    Hue[0.32534234933601514`, 1, 1], 
                    Hue[0.3248378520983274, 1, 1], 
                    Hue[0.3248378520983274, 1, 1], 
                    Hue[0.3266794362989239, 1, 1], 
                    Hue[0.3266794362989239, 1, 1], 
                    Hue[0.3307767146838967, 1, 1], 
                    Hue[0.3307767146838967, 1, 1], 
                    Hue[0.33704224104116143`, 1, 1], 
                    Hue[0.33704224104116143`, 1, 1], 
                    Hue[0.3453928205084022, 1, 1], 
                    Hue[0.3453928205084022, 1, 1], 
                    Hue[0.3557493476841345, 1, 1], 
                    Hue[0.3557493476841345, 1, 1], 
                    Hue[0.36803727733320324`, 1, 1], 
                    Hue[0.36803727733320324`, 1, 1], 
                    Hue[0.38218531751463014`, 1, 1], 
                    Hue[0.38218531751463014`, 1, 1], 
                    Hue[0.3981269793902958, 1, 1], 
                    Hue[0.3981269793902958, 1, 1], 
                    Hue[0.415800089200965, 1, 1], 
                    Hue[0.415800089200965, 1, 1], 
                    Hue[0.43514489696901626`, 1, 1], 
                    Hue[0.43514489696901626`, 1, 1], 
                    Hue[0.456107123450531, 1, 1], 
                    Hue[0.456107123450531, 1, 1], 
                    Hue[0.4786340231754143, 1, 1], 
                    Hue[0.4786340231754143, 1, 1], 
                    Hue[0.5026780811248652, 1, 1], 
                    Hue[0.5026780811248652, 1, 1], 
                    Hue[0.5281933975945959, 1, 1], 
                    Hue[0.5281933975945959, 1, 1], 
                    Hue[0.555136891773129, 1, 1], 
                    Hue[0.555136891773129, 1, 1], 
                    Hue[0.5834691607915969, 1, 1], 
                    Hue[0.5834691607915969, 1, 1], 
                    Hue[0.6131526256736353, 1, 1], 
                    Hue[0.6131526256736353, 1, 1], 
                    Hue[0.6441525006497791, 1, 1], 
                    Hue[0.6441525006497791, 1, 1], 
                    Hue[0.6764355622085817, 1, 1], 
                    Hue[0.6764355622085817, 1, 1], 
                    Hue[0.7099706569321134, 1, 1], 
                    Hue[0.7099706569321134, 1, 1], 
                    Hue[0.7447288937616423, 1, 1], 
                    Hue[0.7447288937616423, 1, 1], 
                    Hue[0.7806831764942391, 1, 1], 
                    Hue[0.7806831764942391, 1, 1], 
                    Hue[0.817806593136449, 1, 1], 
                    Hue[0.817806593136449, 1, 1], 
                    Hue[0.856076141486128, 1, 1], 
                    Hue[0.856076141486128, 1, 1], 
                    Hue[0.8954681694293263, 1, 1], 
                    Hue[0.8954681694293263, 1, 1], 
                    Hue[0.9359614830231154, 1, 1], 
                    Hue[0.9359614830231154, 1, 1], 
                    Hue[0.9775353594293265, 1, 1], 
                    Hue[0.9775353594293265, 1, 1], 
                    Hue[0.02017020566685116, 1, 1], 
                    Hue[0.02017020566685116, 1, 1], 
                    Hue[0.06384797101923008, 1, 1], 
                    Hue[0.06384797101923008, 1, 1], 
                    Hue[0.10855083967090552`, 1, 1], 
                    Hue[0.10855083967090552`, 1, 1], 
                    Hue[0.15426253041459265`, 1, 1], 
                    Hue[0.15426253041459265`, 1, 1], 
                    Hue[0.20096706165192937`, 1, 1], 
                    Hue[0.20096706165192937`, 1, 1], 
                    Hue[0.2486501360831702, 1, 1], 
                    Hue[0.2486501360831702, 1, 1], 
                    Hue[0.2972974266819873, 1, 1], 
                    Hue[0.2972974266819873, 1, 1], 
                    Hue[0.3468949648771804, 1, 1], 
                    Hue[0.3468949648771804, 1, 1], 
                    Hue[0.3974301349949464, 1, 1], 
                    Hue[0.3974301349949464, 1, 1], 
                    Hue[0.44889051576090216`, 1, 1], 
                    Hue[0.44889051576090216`, 1, 1], 
                    Hue[0.5012641862857377, 1, 1], 
                    Hue[0.5012641862857377, 1, 1], 
                    Hue[0.5545404603455091, 1, 1], 
                    Hue[0.5545404603455091, 1, 1], 
                    Hue[0.6087074756721977, 1, 1], 
                    Hue[0.6087074756721977, 1, 1], 
                    Hue[0.6637555293688033, 1, 1], 
                    Hue[0.6637555293688033, 1, 1], 
                    Hue[0.7196745235769924, 1, 1], 
                    Hue[0.7196745235769924, 1, 1], 
                    Hue[0.776454638312091, 1, 1], 
                    Hue[0.776454638312091, 1, 1], 
                    Hue[0.8340862590134883, 1, 1], 
                    Hue[0.8340862590134883, 1, 1], 
                    Hue[0.8925618762146349, 1, 1], 
                    Hue[0.8925618762146349, 1, 1], 
                    Hue[0.9518725157540959, 1, 1], 
                    Hue[0.9518725157540959, 1, 1], 
                    Hue[0.01201033697292499, 1, 1], 
                    Hue[0.01201033697292499, 1, 1], 
                    Hue[0.07296659247411895, 1, 1], 
                    Hue[0.07296659247411895, 1, 1], 
                    Hue[0.134734925300045, 1, 1], 
                    Hue[0.134734925300045, 1, 1], 
                    Hue[0.197305564158579, 1, 1], 
                    Hue[0.197305564158579, 1, 1], 
                    Hue[0.2606739988665761, 1, 1], 
                    Hue[0.2606739988665761, 1, 1], 
                    Hue[0.32483191760817787`, 1, 1], 
                    Hue[0.32483191760817787`, 1, 1], 
                    Hue[0.38977237153196254`, 1, 1], 
                    Hue[0.38977237153196254`, 1, 1], 
                    Hue[0.4554907569809305, 1, 1], 
                    Hue[0.4554907569809305, 1, 1], 
                    Hue[0.5219797009100673, 1, 1], 
                    Hue[0.5219797009100673, 1, 1], 
                    Hue[0.5892341605561915, 1, 1], 
                    Hue[0.5892341605561915, 1, 1], 
                    Hue[0.657246872448343, 1, 1], 
                    Hue[0.657246872448343, 1, 1], 
                    Hue[0.7260117989041965, 1, 1], 
                    Hue[0.7260117989041965, 1, 1], 
                    Hue[0.7955250253211213, 1, 1], 
                    Hue[0.7955250253211213, 1, 1], 
                    Hue[0.8657797896021193, 1, 1], 
                    Hue[0.8657797896021193, 1, 1], 
                    Hue[0.9367719499178244, 1, 1], 
                    Hue[0.9367719499178244, 1, 1], 
                    Hue[0.008495195879954513, 1, 1], 
                    Hue[0.008495195879954513, 1, 1], 
                    Hue[0.0809459758358265, 1, 1], 
                    Hue[0.0809459758358265, 1, 1], 
                    Hue[0.15412001963161848`, 1, 1], 
                    Hue[0.15412001963161848`, 1, 1], 
                    Hue[0.2280115923818089, 1, 1], 
                    Hue[0.2280115923818089, 1, 1], 
                    Hue[0.3026172384645148, 1, 1], 
                    Hue[0.3026172384645148, 1, 1], 
                    Hue[0.3779292229301128, 1, 1], 
                    Hue[0.3779292229301128, 1, 1], 
                    Hue[0.4539469430882079, 1, 1], 
                    Hue[0.4539469430882079, 1, 1], 
                    Hue[0.5306651254733961, 1, 1], 
                    Hue[0.5306651254733961, 1, 1], 
                    Hue[0.6080786482755536, 1, 1], 
                    Hue[0.6080786482755536, 1, 1], 
                    Hue[0.6861849685526418, 1, 1], 
                    Hue[0.6861849685526418, 1, 1], 
                    Hue[0.7649800556325819, 1, 1], 
                    Hue[0.7649800556325819, 1, 1], 
                    Hue[0.8444590226034463, 1, 1], 
                    Hue[0.8444590226034463, 1, 1], 
                    Hue[0.9246179861927123, 1, 1], 
                    Hue[0.9246179861927123, 1, 1], 
                    Hue[0.0054564117259276864`, 1, 1], 
                    Hue[0.0054564117259276864`, 1, 1], 
                    Hue[0.08696888006741216, 1, 1], 
                    Hue[0.08696888006741216, 1, 1], 
                    Hue[0.16914750535384115`, 1, 1], 
                    Hue[0.16914750535384115`, 1, 1], 
                    Hue[0.2519914361670223, 1, 1], 
                    Hue[0.2519914361670223, 1, 1], 
                    Hue[0.3355017653036084, 1, 1], 
                    Hue[0.3355017653036084, 1, 1], 
                    Hue[0.41967365987477934`, 1, 1], 
                    Hue[0.41967365987477934`, 1, 1], 
                    Hue[0.5045004497746628, 1, 1], 
                    Hue[0.5045004497746628, 1, 1], 
                    Hue[0.5899817557355067, 1, 1], 
                    Hue[0.5899817557355067, 1, 1], 
                    Hue[0.6761137981804346, 1, 1], 
                    Hue[0.6761137981804346, 1, 1], 
                    Hue[0.7628854856167173, 1, 1], 
                    Hue[0.7628854856167173, 1, 1], 
                    Hue[0.8503116541986834, 1, 1], 
                    Hue[0.8503116541986834, 1, 1], 
                    Hue[0.9383888140536001, 1, 1], 
                    Hue[0.9383888140536001, 1, 1], 
                    Hue[0.02709017049981145, 1, 1], 
                    Hue[0.02709017049981145, 1, 1], 
                    Hue[0.11642414389020968`, 1, 1], 
                    Hue[0.11642414389020968`, 1, 1], 
                    Hue[0.20640715237073284`, 1, 1], 
                    Hue[0.20640715237073284`, 1, 1], 
                    Hue[0.297017601531361, 1, 1], 
                    Hue[0.297017601531361, 1, 1], 
                    Hue[0.38824632762828315`, 1, 1], 
                    Hue[0.38824632762828315`, 1, 1], 
                    Hue[0.4801090758515208, 1, 1], 
                    Hue[0.4801090758515208, 1, 1], 
                    Hue[0.5725864431600319, 1, 1], 
                    Hue[0.5725864431600319, 1, 1], 
                    Hue[0.665687847174255, 1, 1], 
                    Hue[0.665687847174255, 1, 1], 
                    Hue[0.7594259975222694, 1, 1], 
                    Hue[0.7594259975222694, 1, 1], 
                    Hue[0.8537592849553937, 1, 1], 
                    Hue[0.8537592849553937, 1, 1], 
                    Hue[0.948706513230035, 1, 1], 
                    Hue[0.948706513230035, 1, 1], 
                    Hue[0.04426775150904171, 1, 1], 
                    Hue[0.04426775150904171, 1, 1], 
                    Hue[0.1404108073613589, 1, 1], 
                    Hue[0.1404108073613589, 1, 1], 
                    Hue[0.237191244358674, 1, 1], 
                    Hue[0.237191244358674, 1, 1], 
                    Hue[0.3346110854878864, 1, 1], 
                    Hue[0.3346110854878864, 1, 1], 
                    Hue[0.43260862207299355`, 1, 1], 
                    Hue[0.43260862207299355`, 1, 1], 
                    Hue[0.5311723937875716, 1, 1], 
                    Hue[0.5311723937875716, 1, 1], 
                    Hue[0.6303165816593157, 1, 1], 
                    Hue[0.6303165816593157, 1, 1], 
                    Hue[0.7300926755163386, 1, 1], 
                    Hue[0.7300926755163386, 1, 1], 
                    Hue[0.830492702282897, 1, 1]}],
                   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
                   Axes->{True, None},
                   AxesOrigin->{0, 0},
                   
                   Epilog->LineBox[{{1, 4.215889287240056*^-7}, {
                    2, 7.385109444239298*^-7}, {3, 7.48374071250106*^-7}, {
                    4, 2.6652854374760806`*^-7}, {5, 4.798609177591426*^-7}, {
                    6, 3.532519923160807*^-7}, {7, 7.123917481828207*^-7}, {
                    8, 6.909353181437822*^-7}, {9, 1.9759178912124717`*^-7}, {
                    10, 2.393601123186988*^-7}, {11, 4.828020902023037*^-7}, {
                    12, 8.773562552193309*^-7}, {13, 4.248603103100271*^-8}, {
                    14, 1.477728486073649*^-6}, {
                    15, 1.5846186351308708`*^-6}, {
                    16, 1.69166190676397*^-6}, {17, 2.34058125304706*^-6}, {
                    18, 3.554587234487345*^-6}, {19, 3.831295720116572*^-6}, {
                    20, 4.731550913422753*^-6}, {
                    21, 5.9575568633773365`*^-6}, {
                    22, 7.3826308054585405`*^-6}, {
                    23, 9.530826377004312*^-6}, {
                    24, 0.000011463024279875113`}, {
                    25, 0.000013184062642876242`}, {
                    26, 0.0000159089229447806}, {
                    27, 0.000019595835566520584`}, {
                    28, 0.000023108736904217507`}, {
                    29, 0.000028166462883705582`}, {
                    30, 0.00003315047789316038}, {
                    31, 0.000040285191412516525`}, {
                    32, 0.000048272136165939263`}, {
                    33, 0.00005701038041956149}, {
                    34, 0.0000682453506668145}, {
                    35, 0.00007997113108573856}, {
                    36, 0.00009523751708395127}, {
                    37, 0.00011356382394225856`}, {
                    38, 0.00013311779268841388`}, {
                    39, 0.0001578317207724534}, {
                    40, 0.00018507493658040212`}, {
                    41, 0.00021774480566805912`}, {
                    42, 0.0002557736635614455}, {
                    43, 0.00029904170290528476`}, {
                    44, 0.0003498760289205983}, {
                    45, 0.00040801102113755065`}, {
                    46, 0.0004759445049697905}, {47, 0.0005543224411507603}, {
                    48, 0.0006425758268428648}, {49, 0.000745431425894238}, {
                    50, 0.0008631067028350429}, {51, 0.0009969789551432051}, {
                    52, 0.0011504304418261725`}, {
                    53, 0.0013256681176326212`}, {
                    54, 0.0015242978070044162`}, {
                    55, 0.0017503527918363007`}, {56, 0.002007248413198682}, {
                    57, 0.0022979630591541153`}, {58, 0.002626312689812334}, {
                    59, 0.0029962275891283236`}, {
                    60, 0.0034127797367231485`}, {
                    61, 0.0038803848474610668`}, {62, 0.004404852813413841}, {
                    63, 0.0049919366631887814`}, {
                    64, 0.0056480662653384086`}, {65, 0.006379657094258243}, {
                    66, 0.0071942900978688145`}, {67, 0.008098769418912096}, {
                    68, 0.009102294715914463}, {69, 0.01021143521177961}, {
                    70, 0.011436257957032027`}, {71, 0.012786061349124859`}, {
                    72, 0.014268745654730438`}, {73, 0.01589614104546038}, {
                    74, 0.01767833770868489}, {75, 0.019624460169121425`}, {
                    76, 0.021746717622342312`}, {77, 0.024054984909734324`}, {
                    78, 0.026558748346400075`}, {79, 0.029270303800456907`}, {
                    80, 0.03219841963981317}, {81, 0.03535409367203304}, {
                    82, 0.03874723365223627}, {83, 0.04238699729530048}, {
                    84, 0.046281460250175044`}, {85, 0.05043665357110623}, {
                    86, 0.05486012978319206}, {87, 0.05955704623884705}, {
                    88, 0.0645328826156716}, {89, 0.06978965755752117}, {
                    90, 0.07532973747245739}, {91, 0.08114959140450825}, {
                    92, 0.08724851626590913}, {93, 0.09361915039560055}, {
                    94, 0.10025689006048194`}, {95, 0.1071560545246914}, {
                    96, 0.11430844585670448`}, {97, 0.1217033838577436}, {
                    98, 0.12932311012371334`}, {99, 0.13714631260939314`}, {
                    100, 0.14515314446069377`}, {101, 0.1533225483246502}, {
                    102, 0.16164517698975367`}, {103, 0.170107823104253}, {
                    104, 0.17869580288708334`}, {105, 0.1873811868573901}, {
                    106, 0.19612157929459}, {107, 0.2048782210202883}, {
                    108, 0.21362578329327053`}, {109, 0.22236768199526083`}, {
                    110, 0.2311279808598423}, {111, 0.23992239248151626`}, {
                    112, 0.24873665542739315`}, {113, 0.25751020274435915`}, {
                    114, 0.2661673454572213}, {115, 0.2746601312010308}, {
                    116, 0.2830112541443783}, {117, 0.29132449869745236`}, {
                    118, 0.29973787954650555`}, {119, 0.3083481319002654}, {
                    120, 0.3171415798200614}, {121, 0.32596780260225977`}, {
                    122, 0.3345722322862818}, {123, 0.34267581651223095`}, {
                    124, 0.3500500412713707}, {125, 0.3565744228400174}, {
                    126, 0.36225350837323567`}, {127, 0.36719096937395757`}, {
                    128, 0.37155669156323456`}, {129, 0.37554513537073886`}, {
                    130, 0.3793342390760843}, {131, 0.383070882922276}, {
                    132, 0.38685987576150027`}, {133, 0.3907668786756913}, {
                    134, 0.3948211899784529}, {135, 0.39902737129769483`}, {
                    136, 0.4033689823506277}, {137, 0.4078172008469166}, {
                    138, 0.4123371396033073}, {139, 0.4168862802789701}, {
                    140, 0.4214220314019419}, {141, 0.425900984459955}, {
                    142, 0.4302779950089401}, {143, 0.4345118627144108}, {
                    144, 0.43856150938285676`}, {145, 0.4423885570987661}, {
                    146, 0.4459556271664869}, {147, 0.44922767103676287`}, {
                    148, 0.4521742493551205}, {149, 0.4547645001233238}, {
                    150, 0.4569722752408065}, {151, 0.4587738125954219}, {
                    152, 0.4601466464293833}, {153, 0.4610721755790764}, {
                    154, 0.46153249695102144`}, {155, 0.4615143984795248}, {
                    156, 0.46100938060962837`}, {157, 0.4600049132332509}, {
                    158, 0.45849662946161834`}, {159, 0.4564828634399432}, {
                    160, 0.45396219768511425`}, {161, 0.450936116960236}, {
                    162, 0.44740859123356497`}, {163, 0.4433867912658347}, {
                    164, 0.438879462850403}, {165, 0.4338985513881026}, {
                    166, 0.42845478754416755`}, {167, 0.4225665240673203}, {
                    168, 0.41624864924289}, {169, 0.4095187777843303}, {
                    170, 0.4024008964244921}, {171, 0.3949125247572609}, {
                    172, 0.3870783569783996}, {173, 0.37892367210298405`}, {
                    174, 0.37047182114071425`}, {175, 0.36174970352543406`}, {
                    176, 0.352780671349876}, {177, 0.343596368276955}, {
                    178, 0.3342206882740408}, {179, 0.32468199662872743`}, {
                    180, 0.3150088024522138}, {181, 0.3052269417747082}, {
                    182, 0.2953639033841867}, {183, 0.2854471251902111}, {
                    184, 0.275501946528256}, {185, 0.26555429731156105`}, {
                    186, 0.25562929095379955`}, {187, 0.24574980272923372`}, {
                    188, 0.23593916528863643`}, {189, 0.2262202951364634}, {
                    190, 0.21661314779490026`}, {191, 0.2071363545603469}, {
                    192, 0.1978089932307676}, {193, 0.18864847033508253`}, {
                    194, 0.17967008927981218`}, {195, 0.17088784556493286`}, {
                    196, 0.1623168684138531}, {197, 0.15396397886999108`}, {
                    198, 0.14584376484138376`}, {199, 0.13796257681651314`}, {
                    200, 0.1303298695954158}, {201, 0.12295059035179237`}, {
                    202, 0.1158297673681628}, {203, 0.10897126286438472`}, {
                    204, 0.1023770354799672}, {205, 0.09604813521545022}, {
                    206, 0.08998563300230253}, {207, 0.08418782678720049}, {
                    208, 0.07865477049212187}, {209, 0.07338171811921562}, {
                    210, 0.06836641389119585}, {211, 0.06360356224483525}, {
                    212, 0.05908912332527798}, {213, 0.05481777429581342}, {
                    214, 0.050782717577456456`}, {215, 0.04697788285463465}, {
                    216, 0.04339621374915486}, {217, 0.04003011539821377}, {
                    218, 0.03687195775869723}, {219, 0.0339143293995455}, {
                    220, 0.03114900901600543}, {221, 0.02856720206032563}, {
                    222, 0.02616122116658831}, {223, 0.023923567369434084`}, {
                    224, 0.021845571235873974`}, {225, 0.0199186159693755}, {
                    226, 0.018134829043343997`}, {227, 0.01648668676323527}, {
                    228, 0.014965779804103633`}, {
                    229, 0.013565434145760535`}, {
                    230, 0.012278629494649397`}, {231, 0.01109639496970884}, {
                    232, 0.010012581937976927`}, {233, 0.00902195676969396}, {
                    234, 0.008117145537325893}, {235, 0.007291412425290575}, {
                    236, 0.0065400445677629695`}, {
                    237, 0.005857281883346607}, {238, 0.005237637309735599}, {
                    239, 0.004676884487185301}, {
                    240, 0.0041693991956940945`}, {
                    241, 0.0037109828014421893`}, {
                    242, 0.0032982276237409273`}, {
                    243, 0.0029265655390342407`}, {
                    244, 0.0025930137278625865`}, {
                    245, 0.0022943769171634154`}, {
                    246, 0.0020265170894258814`}, {
                    247, 0.0017867259123326017`}, {
                    248, 0.001572863957162767}, {
                    249, 0.0013826901707517044`}, {
                    250, 0.001213800584536969}, {
                    251, 0.0010634958312205779`}}],
                   Frame->True,
                   FrameTicks->{{{1., 
                    FormBox[
                    RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
                    FormBox[
                    RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
                    FormBox[
                    RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
                    FormBox[
                    RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
                    FormBox["0", TraditionalForm]}, {157.25, 
                    FormBox["2.5`", TraditionalForm]}, {188.5, 
                    FormBox["5", TraditionalForm]}, {219.75, 
                    FormBox["7.5`", TraditionalForm]}, {251., 
                    FormBox["10", TraditionalForm]}}, Automatic, None, None},
                   PlotRange->{-0.1, 1.1},
                   PlotRangeClipping->True,
                   PlotRangePadding->{
                    Scaled[0.02], Automatic}], 19->
                  GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF13lcTXkfB/CTnUhGxCAHaVJJ04Ip8qksJWkvlep025db9i3LUcY+2dcx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                    
                    "], {
                    {GrayLevel[0.5], EdgeForm[None], 
                    GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ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                    
                    "]]]}, {{}, {}, {
                    {Hue[0.9510900371407752, 1, 1], LineBox[{1, 2}]}, 
                    {Hue[0.6484361366317299, 1, 1], LineBox[{2, 3}]}, 
                    {Hue[0.5882020378479695, 1, 1], LineBox[{3, 4}]}, 
                    {Hue[0.7008368986972322, 1, 1], LineBox[{4, 5}]}, 
                    {Hue[0.5201140580590689, 1, 1], LineBox[{5, 6}]}, 
                    {Hue[0.41353824718892657`, 1, 1], LineBox[{6, 7}]}, 
                    {Hue[0.2895856188102126, 1, 1], LineBox[{7, 8}]}, 
                    {Hue[0.16031823714158067`, 1, 1], LineBox[{8, 9}]}, 
                    {Hue[0.06451994409427123, 1, 1], LineBox[{9, 10}]}, 
                    {Hue[0.9714549908564133, 1, 1], LineBox[{10, 11}]}, 
                    {Hue[0.8495956515111321, 1, 1], LineBox[{11, 12}]}, 
                    {Hue[0.7600017408631082, 1, 1], LineBox[{12, 13}]}, 
                    {Hue[0.6660078883104852, 1, 1], LineBox[{13, 14}]}, 
                    {Hue[0.5667738672038776, 1, 1], LineBox[{14, 15}]}, 
                    {Hue[0.46747375697182053`, 1, 1], LineBox[{15, 16}]}, 
                    {Hue[0.3613541730677306, 1, 1], LineBox[{16, 17}]}, 
                    {Hue[0.2556959501890631, 1, 1], LineBox[{17, 18}]}, 
                    {Hue[0.15091913030375073`, 1, 1], LineBox[{18, 19}]}, 
                    {Hue[0.05350912890499731, 1, 1], LineBox[{19, 20}]}, 
                    {Hue[0.9485998845077259, 1, 1], LineBox[{20, 21}]}, 
                    {Hue[0.8458132399311028, 1, 1], LineBox[{21, 22}]}, 
                    {Hue[0.7464255892667264, 1, 1], LineBox[{22, 23}]}, 
                    {Hue[0.6438737768454463, 1, 1], LineBox[{23, 24}]}, 
                    {Hue[0.5448327863032787, 1, 1], LineBox[{24, 25}]}, 
                    {Hue[0.44362298657761645`, 1, 1], LineBox[{25, 26}]}, 
                    {Hue[0.3424111543609978, 1, 1], LineBox[{26, 27}]}, 
                    {Hue[0.24250115263866376`, 1, 1], LineBox[{27, 28}]}, 
                    {Hue[0.14262103084386551`, 1, 1], LineBox[{28, 29}]}, 
                    {Hue[0.04215699495357819, 1, 1], LineBox[{29, 30}]}, 
                    {Hue[0.9430897023244098, 1, 1], LineBox[{30, 31}]}, 
                    {Hue[0.8433311435994333, 1, 1], LineBox[{31, 32}]}, 
                    {Hue[0.7441450909718147, 1, 1], LineBox[{32, 33}]}, 
                    {Hue[0.6455832938839826, 1, 1], LineBox[{33, 34}]}, 
                    {Hue[0.547121062223861, 1, 1], LineBox[{34, 35}]}, 
                    {Hue[0.44874714130339627`, 1, 1], LineBox[{35, 36}]}, 
                    {Hue[0.35090824263074716`, 1, 1], LineBox[{36, 37}]}, 
                    {Hue[0.25295460506672574`, 1, 1], LineBox[{37, 38}]}, 
                    {Hue[0.15541997209474684`, 1, 1], LineBox[{38, 39}]}, 
                    {Hue[0.05812139640804242, 1, 1], LineBox[{39, 40}]}, 
                    {Hue[0.9609925689645509, 1, 1], LineBox[{40, 41}]}, 
                    {Hue[0.8642354511482375, 1, 1], LineBox[{41, 42}]}, 
                    {Hue[0.7676332199817453, 1, 1], LineBox[{42, 43}]}, 
                    {Hue[0.6713081978198241, 1, 1], LineBox[{43, 44}]}, 
                    {Hue[0.575243248554518, 1, 1], LineBox[{44, 45}]}, 
                    {Hue[0.4793506629061221, 1, 1], LineBox[{45, 46}]}, 
                    {Hue[0.3837279679573816, 1, 1], LineBox[{46, 47}]}, 
                    {Hue[0.2883292695427502, 1, 1], LineBox[{47, 48}]}, 
                    {Hue[0.1931101132324467, 1, 1], LineBox[{48, 49}]}, 
                    {Hue[0.09812102246936363, 1, 1], LineBox[{49, 50}]}, 
                    {Hue[0.0033744796127046918`, 1, 1], LineBox[{50, 51}]}, 
                    {Hue[0.9088358401103226, 1, 1], LineBox[{51, 52}]}, 
                    {Hue[0.8145021934498513, 1, 1], LineBox[{52, 53}]}, 
                    {Hue[0.7204197384565719, 1, 1], LineBox[{53, 54}]}, 
                    {Hue[0.6265192014901787, 1, 1], LineBox[{54, 55}]}, 
                    {Hue[0.5328296738760616, 1, 1], LineBox[{55, 56}]}, 
                    {Hue[0.43936094810292264`, 1, 1], LineBox[{56, 57}]}, 
                    {Hue[0.34605544314130104`, 1, 1], LineBox[{57, 58}]}, 
                    {Hue[0.25296083618775117`, 1, 1], LineBox[{58, 59}]}, 
                    {Hue[0.16006161221491183`, 1, 1], LineBox[{59, 60}]}, 
                    {Hue[0.06735816359866889, 1, 1], LineBox[{60, 61}]}, 
                    {Hue[0.9748321901010782, 1, 1], LineBox[{61, 62}]}, 
                    {Hue[0.8824954713004484, 1, 1], LineBox[{62, 63}]}, 
                    {Hue[0.7903531934070253, 1, 1], LineBox[{63, 64}]}, 
                    {Hue[0.6983774005102628, 1, 1], LineBox[{64, 65}]}, 
                    {Hue[0.6065873723032656, 1, 1], LineBox[{65, 66}]}, 
                    {Hue[0.5149750446861332, 1, 1], LineBox[{66, 67}]}, 
                    {Hue[0.4235279081245064, 1, 1], LineBox[{67, 68}]}, 
                    {Hue[0.3322425528657921, 1, 1], LineBox[{68, 69}]}, 
                    {Hue[0.24112435275911037`, 1, 1], LineBox[{69, 70}]}, 
                    {Hue[0.1501653187547825, 1, 1], LineBox[{70, 71}]}, 
                    {Hue[0.059361581806637885`, 1, 1], LineBox[{71, 72}]}, 
                    {Hue[0.9687129393384285, 1, 1], LineBox[{72, 73}]}, 
                    {Hue[0.8782140429001116, 1, 1], LineBox[{73, 74}]}, 
                    {Hue[0.787862506183403, 1, 1], LineBox[{74, 75}]}, 
                    {Hue[0.6976545951114453, 1, 1], LineBox[{75, 76}]}, 
                    {Hue[0.6075843114664345, 1, 1], LineBox[{76, 77}]}, 
                    {Hue[0.5176506612513734, 1, 1], LineBox[{77, 78}]}, 
                    {Hue[0.4278474844771643, 1, 1], LineBox[{78, 79}]}, 
                    {Hue[0.3381731805405101, 1, 1], LineBox[{79, 80}]}, 
                    {Hue[0.24862091634891256`, 1, 1], LineBox[{80, 81}]}, 
                    {Hue[0.15918999842875362`, 1, 1], LineBox[{81, 82}]}, 
                    {Hue[0.0698751897027739, 1, 1], LineBox[{82, 83}]}, 
                    {Hue[0.980675785967886, 1, 1], LineBox[{83, 84}]}, 
                    {Hue[0.8915862507778612, 1, 1], LineBox[{84, 85}]}, 
                    {Hue[0.8026015226029558, 1, 1], LineBox[{85, 86}]}, 
                    {Hue[0.713720578442604, 1, 1], LineBox[{86, 87}]}, 
                    {Hue[0.6249388091684106, 1, 1], LineBox[{87, 88}]}, 
                    {Hue[0.536254069922283, 1, 1], LineBox[{88, 89}]}, 
                    {Hue[0.4476648163707883, 1, 1], LineBox[{89, 90}]}, 
                    {Hue[0.3591665087143565, 1, 1], LineBox[{90, 91}]}, 
                    {Hue[0.2707589943993129, 1, 1], LineBox[{91, 92}]}, 
                    {Hue[0.18243795461731413`, 1, 1], LineBox[{92, 93}]}, 
                    {Hue[0.09420522987410568, 1, 1], LineBox[{93, 94}]}, 
                    {Hue[0.0060603981071102615`, 1, 1], LineBox[{94, 95}]}, 
                    {Hue[0.9180051669045146, 1, 1], LineBox[{95, 96}]}, 
                    {Hue[0.8300432916737577, 1, 1], LineBox[{96, 97}]}, 
                    {Hue[0.7421771414291274, 1, 1], LineBox[{97, 98}]}, 
                    {Hue[0.6544112325841327, 1, 1], LineBox[{98, 99}]}, 
                    {Hue[0.5667496960540034, 1, 1], LineBox[{99, 100}]}, 
                    {Hue[0.47920205752405715`, 1, 1], LineBox[{100, 101}]}, 
                    {Hue[0.39178214935172256`, 1, 1], LineBox[{101, 102}]}, 
                    {Hue[0.3045062754819063, 1, 1], LineBox[{102, 103}]}, 
                    {Hue[0.21739684886962882`, 1, 1], LineBox[{103, 104}]}, 
                    {Hue[0.13047574231371242`, 1, 1], LineBox[{104, 105}]}, 
                    {Hue[0.04376797122875783, 1, 1], LineBox[{105, 106}]}, 
                    {Hue[0.9573010894767487, 1, 1], LineBox[{106, 107}]}, 
                    {Hue[0.8711118842833472, 1, 1], LineBox[{107, 108}]}, 
                    {Hue[0.7852469427284352, 1, 1], LineBox[{108, 109}]}, 
                    {Hue[0.6997674748187139, 1, 1], LineBox[{109, 110}]}, 
                    {Hue[0.614745443539114, 1, 1], LineBox[{110, 111}]}, 
                    {Hue[0.5302588229741672, 1, 1], LineBox[{111, 112}]}, 
                    {Hue[0.44638856015757233`, 1, 1], LineBox[{112, 113}]}, 
                    {Hue[0.3632192567550071, 1, 1], LineBox[{113, 114}]}, 
                    {Hue[0.28084379762907674`, 1, 1], LineBox[{114, 115}]}, 
                    {Hue[0.1993762420489269, 1, 1], LineBox[{115, 116}]}, 
                    {Hue[0.11895902434852888`, 1, 1], LineBox[{116, 117}]}, 
                    {Hue[0.039760565769007494`, 1, 1], LineBox[{117, 118}]}, 
                    {Hue[0.9619676922291895, 1, 1], LineBox[{118, 119}]}, 
                    {Hue[0.8857671042243551, 1, 1], LineBox[{119, 120}]}, 
                    {Hue[0.8113310485318781, 1, 1], LineBox[{120, 121}]}, 
                    {Hue[0.7388111219616389, 1, 1], LineBox[{121, 122}]}, 
                    {Hue[0.6683388300052338, 1, 1], LineBox[{122, 123}]}, 
                    {Hue[0.6000397402029956, 1, 1], LineBox[{123, 124}]}, 
                    {Hue[0.5340405316188332, 1, 1], LineBox[{124, 125}]}, 
                    {Hue[0.47048248692041184`, 1, 1], LineBox[{125, 126}]}, 
                    {Hue[0.4095173680515788, 1, 1], LineBox[{126, 127}]}, 
                    {Hue[0.35130480479686726`, 1, 1], LineBox[{127, 128}]}, 
                    {Hue[0.296000726032468, 1, 1], LineBox[{128, 129}]}, 
                    {Hue[0.24374770049128452`, 1, 1], LineBox[{129, 130}]}, 
                    {Hue[0.194663579371375, 1, 1], LineBox[{130, 131}]}, 
                    {Hue[0.1488380422997798, 1, 1], LineBox[{131, 132}]}, 
                    {Hue[0.10632763311726107`, 1, 1], LineBox[{132, 133}]}, 
                    {Hue[0.06715869091663118, 1, 1], LineBox[{133, 134}]}, 
                    {Hue[0.03132826385598596, 1, 1], LineBox[{134, 135}]}, 
                    {Hue[0.9988084729013986, 1, 1], LineBox[{135, 136}]}, 
                    {Hue[0.9695497880118302, 1, 1], LineBox[{136, 137}]}, 
                    {Hue[0.9434863875857, 1, 1], LineBox[{137, 138}]}, 
                    {Hue[0.9205386371332858, 1, 1], LineBox[{138, 139}]}, 
                    {Hue[0.9006162006692988, 1, 1], LineBox[{139, 140}]}, 
                    {Hue[0.8836237766941165, 1, 1], LineBox[{140, 141}]}, 
                    {Hue[0.8694597112637464, 1, 1], LineBox[{141, 142}]}, 
                    {Hue[0.85802144418141, 1, 1], LineBox[{142, 143}]}, 
                    {Hue[0.8492062740198365, 1, 1], LineBox[{143, 144}]}, 
                    {Hue[0.8429116052544968, 1, 1], LineBox[{144, 145}]}, 
                    {Hue[0.8390381714029644, 1, 1], LineBox[{145, 146}]}, 
                    {Hue[0.8374890855147265, 1, 1], LineBox[{146, 147}]}, 
                    {Hue[0.8381717639477806, 1, 1], LineBox[{147, 148}]}, 
                    {Hue[0.8409968658980163, 1, 1], LineBox[{148, 149}]}, 
                    {Hue[0.8458806855527117, 1, 1], LineBox[{149, 150}]}, 
                    {Hue[0.8527420684162792, 1, 1], LineBox[{150, 151}]}, 
                    {Hue[0.8615061666150565, 1, 1], LineBox[{151, 152}]}, 
                    {Hue[0.8721015386835536, 1, 1], LineBox[{152, 153}]}, 
                    {Hue[0.8844612676447515, 1, 1], LineBox[{153, 154}]}, 
                    {Hue[0.898522586062694, 1, 1], LineBox[{154, 155}]}, 
                    {Hue[0.9142262597826759, 1, 1], LineBox[{155, 156}]}, 
                    {Hue[0.9315171971116003, 1, 1], LineBox[{156, 157}]}, 
                    {Hue[0.9503444226329854, 1, 1], LineBox[{157, 158}]}, 
                    {Hue[0.970659049396332, 1, 1], LineBox[{158, 159}]}, 
                    {Hue[0.9924165973493029, 1, 1], LineBox[{159, 160}]}, 
                    {Hue[0.01557412362952203, 1, 1], LineBox[{160, 161}]}, 
                    {Hue[0.04009276576009518, 1, 1], LineBox[{161, 162}]}, 
                    {Hue[0.06593516184045774, 1, 1], LineBox[{162, 163}]}, 
                    {Hue[0.09306685228699166, 1, 1], LineBox[{163, 164}]}, 
                    {Hue[0.1214549126453047, 1, 1], LineBox[{164, 165}]}, 
                    {Hue[0.15106943285175675`, 1, 1], LineBox[{165, 166}]}, 
                    {Hue[0.18188156318791293`, 1, 1], LineBox[{166, 167}]}, 
                    {Hue[0.2138645476271862, 1, 1], LineBox[{167, 168}]}, 
                    {Hue[0.24699313254215138`, 1, 1], LineBox[{168, 169}]}, 
                    {Hue[0.28124306964296225`, 1, 1], LineBox[{169, 170}]}, 
                    {Hue[0.31659254647657986`, 1, 1], LineBox[{170, 171}]}, 
                    {Hue[0.3530201981651622, 1, 1], LineBox[{171, 172}]}, 
                    {Hue[0.39050554418891786`, 1, 1], LineBox[{172, 173}]}, 
                    {Hue[0.4290303734910717, 1, 1], LineBox[{173, 174}]}, 
                    {Hue[0.4685763131441898, 1, 1], LineBox[{174, 175}]}, 
                    {Hue[0.5091260640647721, 1, 1], LineBox[{175, 176}]}, 
                    {Hue[0.5506645361703, 1, 1], LineBox[{176, 177}]}, 
                    {Hue[0.5931751705100399, 1, 1], LineBox[{177, 178}]}, 
                    {Hue[0.6366445397467777, 1, 1], LineBox[{178, 179}]}, 
                    {Hue[0.681058239309198, 1, 1], LineBox[{179, 180}]}, 
                    {Hue[0.7264031386886708, 1, 1], LineBox[{180, 181}]}, 
                    {Hue[0.7726674425113982, 1, 1], LineBox[{181, 182}]}, 
                    {Hue[0.819837918765256, 1, 1], LineBox[{182, 183}]}, 
                    {Hue[0.8679043674511565, 1, 1], LineBox[{183, 184}]}, 
                    {Hue[0.9168543511248141, 1, 1], LineBox[{184, 185}]}, 
                    {Hue[0.9666787266131991, 1, 1], LineBox[{185, 186}]}, 
                    {Hue[0.01736632464259125, 1, 1], LineBox[{186, 187}]}, 
                    {Hue[0.06890794089973355, 1, 1], LineBox[{187, 188}]}, 
                    {Hue[0.12129416957634404`, 1, 1], LineBox[{188, 189}]}, 
                    {Hue[0.1745166808135452, 1, 1], LineBox[{189, 190}]}, 
                    {Hue[0.2285663347303934, 1, 1], LineBox[{190, 191}]}, 
                    {Hue[0.2834351028766379, 1, 1], LineBox[{191, 192}]}, 
                    {Hue[0.3391154820244295, 1, 1], LineBox[{192, 193}]}, 
                    {Hue[0.3955997937834623, 1, 1], LineBox[{193, 194}]}, 
                    {Hue[0.45287936850823385`, 1, 1], LineBox[{194, 195}]}, 
                    {Hue[0.5109476098222223, 1, 1], LineBox[{195, 196}]}, 
                    {Hue[0.5697982658184998, 1, 1], LineBox[{196, 197}]}, 
                    {Hue[0.6294239155644445, 1, 1], LineBox[{197, 198}]}, 
                    {Hue[0.689818553661153, 1, 1], LineBox[{198, 199}]}, 
                    {Hue[0.750976510534746, 1, 1], LineBox[{199, 200}]}, 
                    {Hue[0.8128912929293306, 1, 1], LineBox[{200, 201}]}, 
                    {Hue[0.8755568751916872, 1, 1], LineBox[{201, 202}]}, 
                    {Hue[0.9389682660024058, 1, 1], LineBox[{202, 203}]}, 
                    {Hue[0.0031178192768510317`, 1, 1], 
                    LineBox[{203, 204}]}, 
                    {Hue[0.06800275159963283, 1, 1], LineBox[{204, 205}]}, 
                    {Hue[0.1336157129120437, 1, 1], LineBox[{205, 206}]}, 
                    {Hue[0.19995455989782093`, 1, 1], LineBox[{206, 207}]}, 
                    {Hue[0.2670094640187561, 1, 1], LineBox[{207, 208}]}, 
                    {Hue[0.3347809278038839, 1, 1], LineBox[{208, 209}]}, 
                    {Hue[0.4032618549635441, 1, 1], LineBox[{209, 210}]}, 
                    {Hue[0.4724477701914594, 1, 1], LineBox[{210, 211}]}, 
                    {Hue[0.5423359564001664, 1, 1], LineBox[{211, 212}]}, 
                    {Hue[0.6129204992088491, 1, 1], LineBox[{212, 213}]}, 
                    {Hue[0.6841963849615734, 1, 1], LineBox[{213, 214}]}, 
                    {Hue[0.7561619322423868, 1, 1], LineBox[{214, 215}]}, 
                    {Hue[0.82881140408447, 1, 1], LineBox[{215, 216}]}, 
                    {Hue[0.902143300822762, 1, 1], LineBox[{216, 217}]}, 
                    {Hue[0.9761493691266603, 1, 1], LineBox[{217, 218}]}, 
                    {Hue[0.05083015294981264, 1, 1], LineBox[{218, 219}]}, 
                    {Hue[0.12617992086590107`, 1, 1], LineBox[{219, 220}]}, 
                    {Hue[0.2021963971348071, 1, 1], LineBox[{220, 221}]}, 
                    {Hue[0.27887567203613706`, 1, 1], LineBox[{221, 222}]}, 
                    {Hue[0.356212734612382, 1, 1], LineBox[{222, 223}]}, 
                    {Hue[0.4342080079703386, 1, 1], LineBox[{223, 224}]}, 
                    {Hue[0.5128557849609001, 1, 1], LineBox[{224, 225}]}, 
                    {Hue[0.5921532077475121, 1, 1], LineBox[{225, 226}]}, 
                    {Hue[0.6720964647032827, 1, 1], LineBox[{226, 227}]}, 
                    {Hue[0.752681238076534, 1, 1], LineBox[{227, 228}]}, 
                    {Hue[0.8339078459155588, 1, 1], LineBox[{228, 229}]}, 
                    {Hue[0.9157742987005092, 1, 1], LineBox[{229, 230}]}, 
                    {Hue[0.998275547272889, 1, 1], LineBox[{230, 231}]}, 
                    {Hue[0.08140743783099906, 1, 1], LineBox[{231, 232}]}, 
                    {Hue[0.16516937419891978`, 1, 1], LineBox[{232, 233}]}, 
                    {Hue[0.249558861554528, 1, 1], LineBox[{233, 234}]}, 
                    {Hue[0.33456939691468257`, 1, 1], LineBox[{234, 235}]}, 
                    {Hue[0.42020253714678935`, 1, 1], LineBox[{235, 236}]}, 
                    {Hue[0.5064565805732228, 1, 1], LineBox[{236, 237}]}, 
                    {Hue[0.5933250672959842, 1, 1], LineBox[{237, 238}]}, 
                    {Hue[0.6808033250107832, 1, 1], LineBox[{238, 239}]}, 
                    {Hue[0.7688952871616338, 1, 1], LineBox[{239, 240}]}, 
                    {Hue[0.8576085005203646, 1, 1], LineBox[{240, 241}]}, 
                    {Hue[0.9469223721299237, 1, 1], LineBox[{241, 242}]}, 
                    {Hue[0.03683378074896659, 1, 1], LineBox[{242, 243}]}, 
                    {Hue[0.12735624017682576`, 1, 1], LineBox[{243, 244}]}, 
                    {Hue[0.2184734433831852, 1, 1], LineBox[{244, 245}]}, 
                    {Hue[0.31018333537811105`, 1, 1], LineBox[{245, 246}]}, 
                    {Hue[0.40251345092619834`, 1, 1], LineBox[{246, 247}]}, 
                    {Hue[0.4954396434137711, 1, 1], LineBox[{247, 248}]}, 
                    {Hue[0.5889176172549687, 1, 1], LineBox[{248, 249}]}, 
                    {Hue[0.6829974887015433, 1, 1], LineBox[{249, 250}]}, 
                    {Hue[0.7776940568752733, 1, 1], LineBox[{250, 251}]}}}},
                    VertexColors->{
                    Hue[0.9510900371407752, 1, 1], 
                    Hue[0.6484361366317299, 1, 1], 
                    Hue[0.5882020378479695, 1, 1], 
                    Hue[0.7008368986972322, 1, 1], 
                    Hue[0.5201140580590689, 1, 1], 
                    Hue[0.41353824718892657`, 1, 1], 
                    Hue[0.2895856188102126, 1, 1], 
                    Hue[0.16031823714158067`, 1, 1], 
                    Hue[0.06451994409427123, 1, 1], 
                    Hue[0.9714549908564133, 1, 1], 
                    Hue[0.8495956515111321, 1, 1], 
                    Hue[0.7600017408631082, 1, 1], 
                    Hue[0.6660078883104852, 1, 1], 
                    Hue[0.5667738672038776, 1, 1], 
                    Hue[0.46747375697182053`, 1, 1], 
                    Hue[0.3613541730677306, 1, 1], 
                    Hue[0.2556959501890631, 1, 1], 
                    Hue[0.15091913030375073`, 1, 1], 
                    Hue[0.05350912890499731, 1, 1], 
                    Hue[0.9485998845077259, 1, 1], 
                    Hue[0.8458132399311028, 1, 1], 
                    Hue[0.7464255892667264, 1, 1], 
                    Hue[0.6438737768454463, 1, 1], 
                    Hue[0.5448327863032787, 1, 1], 
                    Hue[0.44362298657761645`, 1, 1], 
                    Hue[0.3424111543609978, 1, 1], 
                    Hue[0.24250115263866376`, 1, 1], 
                    Hue[0.14262103084386551`, 1, 1], 
                    Hue[0.04215699495357819, 1, 1], 
                    Hue[0.9430897023244098, 1, 1], 
                    Hue[0.8433311435994333, 1, 1], 
                    Hue[0.7441450909718147, 1, 1], 
                    Hue[0.6455832938839826, 1, 1], 
                    Hue[0.547121062223861, 1, 1], 
                    Hue[0.44874714130339627`, 1, 1], 
                    Hue[0.35090824263074716`, 1, 1], 
                    Hue[0.25295460506672574`, 1, 1], 
                    Hue[0.15541997209474684`, 1, 1], 
                    Hue[0.05812139640804242, 1, 1], 
                    Hue[0.9609925689645509, 1, 1], 
                    Hue[0.8642354511482375, 1, 1], 
                    Hue[0.7676332199817453, 1, 1], 
                    Hue[0.6713081978198241, 1, 1], 
                    Hue[0.575243248554518, 1, 1], 
                    Hue[0.4793506629061221, 1, 1], 
                    Hue[0.3837279679573816, 1, 1], 
                    Hue[0.2883292695427502, 1, 1], 
                    Hue[0.1931101132324467, 1, 1], 
                    Hue[0.09812102246936363, 1, 1], 
                    Hue[0.0033744796127046918`, 1, 1], 
                    Hue[0.9088358401103226, 1, 1], 
                    Hue[0.8145021934498513, 1, 1], 
                    Hue[0.7204197384565719, 1, 1], 
                    Hue[0.6265192014901787, 1, 1], 
                    Hue[0.5328296738760616, 1, 1], 
                    Hue[0.43936094810292264`, 1, 1], 
                    Hue[0.34605544314130104`, 1, 1], 
                    Hue[0.25296083618775117`, 1, 1], 
                    Hue[0.16006161221491183`, 1, 1], 
                    Hue[0.06735816359866889, 1, 1], 
                    Hue[0.9748321901010782, 1, 1], 
                    Hue[0.8824954713004484, 1, 1], 
                    Hue[0.7903531934070253, 1, 1], 
                    Hue[0.6983774005102628, 1, 1], 
                    Hue[0.6065873723032656, 1, 1], 
                    Hue[0.5149750446861332, 1, 1], 
                    Hue[0.4235279081245064, 1, 1], 
                    Hue[0.3322425528657921, 1, 1], 
                    Hue[0.24112435275911037`, 1, 1], 
                    Hue[0.1501653187547825, 1, 1], 
                    Hue[0.059361581806637885`, 1, 1], 
                    Hue[0.9687129393384285, 1, 1], 
                    Hue[0.8782140429001116, 1, 1], 
                    Hue[0.787862506183403, 1, 1], 
                    Hue[0.6976545951114453, 1, 1], 
                    Hue[0.6075843114664345, 1, 1], 
                    Hue[0.5176506612513734, 1, 1], 
                    Hue[0.4278474844771643, 1, 1], 
                    Hue[0.3381731805405101, 1, 1], 
                    Hue[0.24862091634891256`, 1, 1], 
                    Hue[0.15918999842875362`, 1, 1], 
                    Hue[0.0698751897027739, 1, 1], 
                    Hue[0.980675785967886, 1, 1], 
                    Hue[0.8915862507778612, 1, 1], 
                    Hue[0.8026015226029558, 1, 1], 
                    Hue[0.713720578442604, 1, 1], 
                    Hue[0.6249388091684106, 1, 1], 
                    Hue[0.536254069922283, 1, 1], 
                    Hue[0.4476648163707883, 1, 1], 
                    Hue[0.3591665087143565, 1, 1], 
                    Hue[0.2707589943993129, 1, 1], 
                    Hue[0.18243795461731413`, 1, 1], 
                    Hue[0.09420522987410568, 1, 1], 
                    Hue[0.0060603981071102615`, 1, 1], 
                    Hue[0.9180051669045146, 1, 1], 
                    Hue[0.8300432916737577, 1, 1], 
                    Hue[0.7421771414291274, 1, 1], 
                    Hue[0.6544112325841327, 1, 1], 
                    Hue[0.5667496960540034, 1, 1], 
                    Hue[0.47920205752405715`, 1, 1], 
                    Hue[0.39178214935172256`, 1, 1], 
                    Hue[0.3045062754819063, 1, 1], 
                    Hue[0.21739684886962882`, 1, 1], 
                    Hue[0.13047574231371242`, 1, 1], 
                    Hue[0.04376797122875783, 1, 1], 
                    Hue[0.9573010894767487, 1, 1], 
                    Hue[0.8711118842833472, 1, 1], 
                    Hue[0.7852469427284352, 1, 1], 
                    Hue[0.6997674748187139, 1, 1], 
                    Hue[0.614745443539114, 1, 1], 
                    Hue[0.5302588229741672, 1, 1], 
                    Hue[0.44638856015757233`, 1, 1], 
                    Hue[0.3632192567550071, 1, 1], 
                    Hue[0.28084379762907674`, 1, 1], 
                    Hue[0.1993762420489269, 1, 1], 
                    Hue[0.11895902434852888`, 1, 1], 
                    Hue[0.039760565769007494`, 1, 1], 
                    Hue[0.9619676922291895, 1, 1], 
                    Hue[0.8857671042243551, 1, 1], 
                    Hue[0.8113310485318781, 1, 1], 
                    Hue[0.7388111219616389, 1, 1], 
                    Hue[0.6683388300052338, 1, 1], 
                    Hue[0.6000397402029956, 1, 1], 
                    Hue[0.5340405316188332, 1, 1], 
                    Hue[0.47048248692041184`, 1, 1], 
                    Hue[0.4095173680515788, 1, 1], 
                    Hue[0.35130480479686726`, 1, 1], 
                    Hue[0.296000726032468, 1, 1], 
                    Hue[0.24374770049128452`, 1, 1], 
                    Hue[0.194663579371375, 1, 1], 
                    Hue[0.1488380422997798, 1, 1], 
                    Hue[0.10632763311726107`, 1, 1], 
                    Hue[0.06715869091663118, 1, 1], 
                    Hue[0.03132826385598596, 1, 1], 
                    Hue[0.9988084729013986, 1, 1], 
                    Hue[0.9695497880118302, 1, 1], 
                    Hue[0.9434863875857, 1, 1], 
                    Hue[0.9205386371332858, 1, 1], 
                    Hue[0.9006162006692988, 1, 1], 
                    Hue[0.8836237766941165, 1, 1], 
                    Hue[0.8694597112637464, 1, 1], 
                    Hue[0.85802144418141, 1, 1], 
                    Hue[0.8492062740198365, 1, 1], 
                    Hue[0.8429116052544968, 1, 1], 
                    Hue[0.8390381714029644, 1, 1], 
                    Hue[0.8374890855147265, 1, 1], 
                    Hue[0.8381717639477806, 1, 1], 
                    Hue[0.8409968658980163, 1, 1], 
                    Hue[0.8458806855527117, 1, 1], 
                    Hue[0.8527420684162792, 1, 1], 
                    Hue[0.8615061666150565, 1, 1], 
                    Hue[0.8721015386835536, 1, 1], 
                    Hue[0.8844612676447515, 1, 1], 
                    Hue[0.898522586062694, 1, 1], 
                    Hue[0.9142262597826759, 1, 1], 
                    Hue[0.9315171971116003, 1, 1], 
                    Hue[0.9503444226329854, 1, 1], 
                    Hue[0.970659049396332, 1, 1], 
                    Hue[0.9924165973493029, 1, 1], 
                    Hue[0.01557412362952203, 1, 1], 
                    Hue[0.04009276576009518, 1, 1], 
                    Hue[0.06593516184045774, 1, 1], 
                    Hue[0.09306685228699166, 1, 1], 
                    Hue[0.1214549126453047, 1, 1], 
                    Hue[0.15106943285175675`, 1, 1], 
                    Hue[0.18188156318791293`, 1, 1], 
                    Hue[0.2138645476271862, 1, 1], 
                    Hue[0.24699313254215138`, 1, 1], 
                    Hue[0.28124306964296225`, 1, 1], 
                    Hue[0.31659254647657986`, 1, 1], 
                    Hue[0.3530201981651622, 1, 1], 
                    Hue[0.39050554418891786`, 1, 1], 
                    Hue[0.4290303734910717, 1, 1], 
                    Hue[0.4685763131441898, 1, 1], 
                    Hue[0.5091260640647721, 1, 1], 
                    Hue[0.5506645361703, 1, 1], 
                    Hue[0.5931751705100399, 1, 1], 
                    Hue[0.6366445397467777, 1, 1], 
                    Hue[0.681058239309198, 1, 1], 
                    Hue[0.7264031386886708, 1, 1], 
                    Hue[0.7726674425113982, 1, 1], 
                    Hue[0.819837918765256, 1, 1], 
                    Hue[0.8679043674511565, 1, 1], 
                    Hue[0.9168543511248141, 1, 1], 
                    Hue[0.9666787266131991, 1, 1], 
                    Hue[0.01736632464259125, 1, 1], 
                    Hue[0.06890794089973355, 1, 1], 
                    Hue[0.12129416957634404`, 1, 1], 
                    Hue[0.1745166808135452, 1, 1], 
                    Hue[0.2285663347303934, 1, 1], 
                    Hue[0.2834351028766379, 1, 1], 
                    Hue[0.3391154820244295, 1, 1], 
                    Hue[0.3955997937834623, 1, 1], 
                    Hue[0.45287936850823385`, 1, 1], 
                    Hue[0.5109476098222223, 1, 1], 
                    Hue[0.5697982658184998, 1, 1], 
                    Hue[0.6294239155644445, 1, 1], 
                    Hue[0.689818553661153, 1, 1], 
                    Hue[0.750976510534746, 1, 1], 
                    Hue[0.8128912929293306, 1, 1], 
                    Hue[0.8755568751916872, 1, 1], 
                    Hue[0.9389682660024058, 1, 1], 
                    Hue[0.0031178192768510317`, 1, 1], 
                    Hue[0.06800275159963283, 1, 1], 
                    Hue[0.1336157129120437, 1, 1], 
                    Hue[0.19995455989782093`, 1, 1], 
                    Hue[0.2670094640187561, 1, 1], 
                    Hue[0.3347809278038839, 1, 1], 
                    Hue[0.4032618549635441, 1, 1], 
                    Hue[0.4724477701914594, 1, 1], 
                    Hue[0.5423359564001664, 1, 1], 
                    Hue[0.6129204992088491, 1, 1], 
                    Hue[0.6841963849615734, 1, 1], 
                    Hue[0.7561619322423868, 1, 1], 
                    Hue[0.82881140408447, 1, 1], 
                    Hue[0.902143300822762, 1, 1], 
                    Hue[0.9761493691266603, 1, 1], 
                    Hue[0.05083015294981264, 1, 1], 
                    Hue[0.12617992086590107`, 1, 1], 
                    Hue[0.2021963971348071, 1, 1], 
                    Hue[0.27887567203613706`, 1, 1], 
                    Hue[0.356212734612382, 1, 1], 
                    Hue[0.4342080079703386, 1, 1], 
                    Hue[0.5128557849609001, 1, 1], 
                    Hue[0.5921532077475121, 1, 1], 
                    Hue[0.6720964647032827, 1, 1], 
                    Hue[0.752681238076534, 1, 1], 
                    Hue[0.8339078459155588, 1, 1], 
                    Hue[0.9157742987005092, 1, 1], 
                    Hue[0.998275547272889, 1, 1], 
                    Hue[0.08140743783099906, 1, 1], 
                    Hue[0.16516937419891978`, 1, 1], 
                    Hue[0.249558861554528, 1, 1], 
                    Hue[0.33456939691468257`, 1, 1], 
                    Hue[0.42020253714678935`, 1, 1], 
                    Hue[0.5064565805732228, 1, 1], 
                    Hue[0.5933250672959842, 1, 1], 
                    Hue[0.6808033250107832, 1, 1], 
                    Hue[0.7688952871616338, 1, 1], 
                    Hue[0.8576085005203646, 1, 1], 
                    Hue[0.9469223721299237, 1, 1], 
                    Hue[0.03683378074896659, 1, 1], 
                    Hue[0.12735624017682576`, 1, 1], 
                    Hue[0.2184734433831852, 1, 1], 
                    Hue[0.31018333537811105`, 1, 1], 
                    Hue[0.40251345092619834`, 1, 1], 
                    Hue[0.4954396434137711, 1, 1], 
                    Hue[0.5889176172549687, 1, 1], 
                    Hue[0.6829974887015433, 1, 1], 
                    Hue[0.7776940568752733, 1, 1], 
                    Hue[0.8729551243678357, 1, 1], 
                    Hue[0.9510900371407752, 1, 1], 
                    Hue[0.6484361366317299, 1, 1], 
                    Hue[0.6484361366317299, 1, 1], 
                    Hue[0.5882020378479695, 1, 1], 
                    Hue[0.5882020378479695, 1, 1], 
                    Hue[0.7008368986972322, 1, 1], 
                    Hue[0.7008368986972322, 1, 1], 
                    Hue[0.5201140580590689, 1, 1], 
                    Hue[0.5201140580590689, 1, 1], 
                    Hue[0.41353824718892657`, 1, 1], 
                    Hue[0.41353824718892657`, 1, 1], 
                    Hue[0.2895856188102126, 1, 1], 
                    Hue[0.2895856188102126, 1, 1], 
                    Hue[0.16031823714158067`, 1, 1], 
                    Hue[0.16031823714158067`, 1, 1], 
                    Hue[0.06451994409427123, 1, 1], 
                    Hue[0.06451994409427123, 1, 1], 
                    Hue[0.9714549908564133, 1, 1], 
                    Hue[0.9714549908564133, 1, 1], 
                    Hue[0.8495956515111321, 1, 1], 
                    Hue[0.8495956515111321, 1, 1], 
                    Hue[0.7600017408631082, 1, 1], 
                    Hue[0.7600017408631082, 1, 1], 
                    Hue[0.6660078883104852, 1, 1], 
                    Hue[0.6660078883104852, 1, 1], 
                    Hue[0.5667738672038776, 1, 1], 
                    Hue[0.5667738672038776, 1, 1], 
                    Hue[0.46747375697182053`, 1, 1], 
                    Hue[0.46747375697182053`, 1, 1], 
                    Hue[0.3613541730677306, 1, 1], 
                    Hue[0.3613541730677306, 1, 1], 
                    Hue[0.2556959501890631, 1, 1], 
                    Hue[0.2556959501890631, 1, 1], 
                    Hue[0.15091913030375073`, 1, 1], 
                    Hue[0.15091913030375073`, 1, 1], 
                    Hue[0.05350912890499731, 1, 1], 
                    Hue[0.05350912890499731, 1, 1], 
                    Hue[0.9485998845077259, 1, 1], 
                    Hue[0.9485998845077259, 1, 1], 
                    Hue[0.8458132399311028, 1, 1], 
                    Hue[0.8458132399311028, 1, 1], 
                    Hue[0.7464255892667264, 1, 1], 
                    Hue[0.7464255892667264, 1, 1], 
                    Hue[0.6438737768454463, 1, 1], 
                    Hue[0.6438737768454463, 1, 1], 
                    Hue[0.5448327863032787, 1, 1], 
                    Hue[0.5448327863032787, 1, 1], 
                    Hue[0.44362298657761645`, 1, 1], 
                    Hue[0.44362298657761645`, 1, 1], 
                    Hue[0.3424111543609978, 1, 1], 
                    Hue[0.3424111543609978, 1, 1], 
                    Hue[0.24250115263866376`, 1, 1], 
                    Hue[0.24250115263866376`, 1, 1], 
                    Hue[0.14262103084386551`, 1, 1], 
                    Hue[0.14262103084386551`, 1, 1], 
                    Hue[0.04215699495357819, 1, 1], 
                    Hue[0.04215699495357819, 1, 1], 
                    Hue[0.9430897023244098, 1, 1], 
                    Hue[0.9430897023244098, 1, 1], 
                    Hue[0.8433311435994333, 1, 1], 
                    Hue[0.8433311435994333, 1, 1], 
                    Hue[0.7441450909718147, 1, 1], 
                    Hue[0.7441450909718147, 1, 1], 
                    Hue[0.6455832938839826, 1, 1], 
                    Hue[0.6455832938839826, 1, 1], 
                    Hue[0.547121062223861, 1, 1], 
                    Hue[0.547121062223861, 1, 1], 
                    Hue[0.44874714130339627`, 1, 1], 
                    Hue[0.44874714130339627`, 1, 1], 
                    Hue[0.35090824263074716`, 1, 1], 
                    Hue[0.35090824263074716`, 1, 1], 
                    Hue[0.25295460506672574`, 1, 1], 
                    Hue[0.25295460506672574`, 1, 1], 
                    Hue[0.15541997209474684`, 1, 1], 
                    Hue[0.15541997209474684`, 1, 1], 
                    Hue[0.05812139640804242, 1, 1], 
                    Hue[0.05812139640804242, 1, 1], 
                    Hue[0.9609925689645509, 1, 1], 
                    Hue[0.9609925689645509, 1, 1], 
                    Hue[0.8642354511482375, 1, 1], 
                    Hue[0.8642354511482375, 1, 1], 
                    Hue[0.7676332199817453, 1, 1], 
                    Hue[0.7676332199817453, 1, 1], 
                    Hue[0.6713081978198241, 1, 1], 
                    Hue[0.6713081978198241, 1, 1], 
                    Hue[0.575243248554518, 1, 1], 
                    Hue[0.575243248554518, 1, 1], 
                    Hue[0.4793506629061221, 1, 1], 
                    Hue[0.4793506629061221, 1, 1], 
                    Hue[0.3837279679573816, 1, 1], 
                    Hue[0.3837279679573816, 1, 1], 
                    Hue[0.2883292695427502, 1, 1], 
                    Hue[0.2883292695427502, 1, 1], 
                    Hue[0.1931101132324467, 1, 1], 
                    Hue[0.1931101132324467, 1, 1], 
                    Hue[0.09812102246936363, 1, 1], 
                    Hue[0.09812102246936363, 1, 1], 
                    Hue[0.0033744796127046918`, 1, 1], 
                    Hue[0.0033744796127046918`, 1, 1], 
                    Hue[0.9088358401103226, 1, 1], 
                    Hue[0.9088358401103226, 1, 1], 
                    Hue[0.8145021934498513, 1, 1], 
                    Hue[0.8145021934498513, 1, 1], 
                    Hue[0.7204197384565719, 1, 1], 
                    Hue[0.7204197384565719, 1, 1], 
                    Hue[0.6265192014901787, 1, 1], 
                    Hue[0.6265192014901787, 1, 1], 
                    Hue[0.5328296738760616, 1, 1], 
                    Hue[0.5328296738760616, 1, 1], 
                    Hue[0.43936094810292264`, 1, 1], 
                    Hue[0.43936094810292264`, 1, 1], 
                    Hue[0.34605544314130104`, 1, 1], 
                    Hue[0.34605544314130104`, 1, 1], 
                    Hue[0.25296083618775117`, 1, 1], 
                    Hue[0.25296083618775117`, 1, 1], 
                    Hue[0.16006161221491183`, 1, 1], 
                    Hue[0.16006161221491183`, 1, 1], 
                    Hue[0.06735816359866889, 1, 1], 
                    Hue[0.06735816359866889, 1, 1], 
                    Hue[0.9748321901010782, 1, 1], 
                    Hue[0.9748321901010782, 1, 1], 
                    Hue[0.8824954713004484, 1, 1], 
                    Hue[0.8824954713004484, 1, 1], 
                    Hue[0.7903531934070253, 1, 1], 
                    Hue[0.7903531934070253, 1, 1], 
                    Hue[0.6983774005102628, 1, 1], 
                    Hue[0.6983774005102628, 1, 1], 
                    Hue[0.6065873723032656, 1, 1], 
                    Hue[0.6065873723032656, 1, 1], 
                    Hue[0.5149750446861332, 1, 1], 
                    Hue[0.5149750446861332, 1, 1], 
                    Hue[0.4235279081245064, 1, 1], 
                    Hue[0.4235279081245064, 1, 1], 
                    Hue[0.3322425528657921, 1, 1], 
                    Hue[0.3322425528657921, 1, 1], 
                    Hue[0.24112435275911037`, 1, 1], 
                    Hue[0.24112435275911037`, 1, 1], 
                    Hue[0.1501653187547825, 1, 1], 
                    Hue[0.1501653187547825, 1, 1], 
                    Hue[0.059361581806637885`, 1, 1], 
                    Hue[0.059361581806637885`, 1, 1], 
                    Hue[0.9687129393384285, 1, 1], 
                    Hue[0.9687129393384285, 1, 1], 
                    Hue[0.8782140429001116, 1, 1], 
                    Hue[0.8782140429001116, 1, 1], 
                    Hue[0.787862506183403, 1, 1], 
                    Hue[0.787862506183403, 1, 1], 
                    Hue[0.6976545951114453, 1, 1], 
                    Hue[0.6976545951114453, 1, 1], 
                    Hue[0.6075843114664345, 1, 1], 
                    Hue[0.6075843114664345, 1, 1], 
                    Hue[0.5176506612513734, 1, 1], 
                    Hue[0.5176506612513734, 1, 1], 
                    Hue[0.4278474844771643, 1, 1], 
                    Hue[0.4278474844771643, 1, 1], 
                    Hue[0.3381731805405101, 1, 1], 
                    Hue[0.3381731805405101, 1, 1], 
                    Hue[0.24862091634891256`, 1, 1], 
                    Hue[0.24862091634891256`, 1, 1], 
                    Hue[0.15918999842875362`, 1, 1], 
                    Hue[0.15918999842875362`, 1, 1], 
                    Hue[0.0698751897027739, 1, 1], 
                    Hue[0.0698751897027739, 1, 1], 
                    Hue[0.980675785967886, 1, 1], 
                    Hue[0.980675785967886, 1, 1], 
                    Hue[0.8915862507778612, 1, 1], 
                    Hue[0.8915862507778612, 1, 1], 
                    Hue[0.8026015226029558, 1, 1], 
                    Hue[0.8026015226029558, 1, 1], 
                    Hue[0.713720578442604, 1, 1], 
                    Hue[0.713720578442604, 1, 1], 
                    Hue[0.6249388091684106, 1, 1], 
                    Hue[0.6249388091684106, 1, 1], 
                    Hue[0.536254069922283, 1, 1], 
                    Hue[0.536254069922283, 1, 1], 
                    Hue[0.4476648163707883, 1, 1], 
                    Hue[0.4476648163707883, 1, 1], 
                    Hue[0.3591665087143565, 1, 1], 
                    Hue[0.3591665087143565, 1, 1], 
                    Hue[0.2707589943993129, 1, 1], 
                    Hue[0.2707589943993129, 1, 1], 
                    Hue[0.18243795461731413`, 1, 1], 
                    Hue[0.18243795461731413`, 1, 1], 
                    Hue[0.09420522987410568, 1, 1], 
                    Hue[0.09420522987410568, 1, 1], 
                    Hue[0.0060603981071102615`, 1, 1], 
                    Hue[0.0060603981071102615`, 1, 1], 
                    Hue[0.9180051669045146, 1, 1], 
                    Hue[0.9180051669045146, 1, 1], 
                    Hue[0.8300432916737577, 1, 1], 
                    Hue[0.8300432916737577, 1, 1], 
                    Hue[0.7421771414291274, 1, 1], 
                    Hue[0.7421771414291274, 1, 1], 
                    Hue[0.6544112325841327, 1, 1], 
                    Hue[0.6544112325841327, 1, 1], 
                    Hue[0.5667496960540034, 1, 1], 
                    Hue[0.5667496960540034, 1, 1], 
                    Hue[0.47920205752405715`, 1, 1], 
                    Hue[0.47920205752405715`, 1, 1], 
                    Hue[0.39178214935172256`, 1, 1], 
                    Hue[0.39178214935172256`, 1, 1], 
                    Hue[0.3045062754819063, 1, 1], 
                    Hue[0.3045062754819063, 1, 1], 
                    Hue[0.21739684886962882`, 1, 1], 
                    Hue[0.21739684886962882`, 1, 1], 
                    Hue[0.13047574231371242`, 1, 1], 
                    Hue[0.13047574231371242`, 1, 1], 
                    Hue[0.04376797122875783, 1, 1], 
                    Hue[0.04376797122875783, 1, 1], 
                    Hue[0.9573010894767487, 1, 1], 
                    Hue[0.9573010894767487, 1, 1], 
                    Hue[0.8711118842833472, 1, 1], 
                    Hue[0.8711118842833472, 1, 1], 
                    Hue[0.7852469427284352, 1, 1], 
                    Hue[0.7852469427284352, 1, 1], 
                    Hue[0.6997674748187139, 1, 1], 
                    Hue[0.6997674748187139, 1, 1], 
                    Hue[0.614745443539114, 1, 1], 
                    Hue[0.614745443539114, 1, 1], 
                    Hue[0.5302588229741672, 1, 1], 
                    Hue[0.5302588229741672, 1, 1], 
                    Hue[0.44638856015757233`, 1, 1], 
                    Hue[0.44638856015757233`, 1, 1], 
                    Hue[0.3632192567550071, 1, 1], 
                    Hue[0.3632192567550071, 1, 1], 
                    Hue[0.28084379762907674`, 1, 1], 
                    Hue[0.28084379762907674`, 1, 1], 
                    Hue[0.1993762420489269, 1, 1], 
                    Hue[0.1993762420489269, 1, 1], 
                    Hue[0.11895902434852888`, 1, 1], 
                    Hue[0.11895902434852888`, 1, 1], 
                    Hue[0.039760565769007494`, 1, 1], 
                    Hue[0.039760565769007494`, 1, 1], 
                    Hue[0.9619676922291895, 1, 1], 
                    Hue[0.9619676922291895, 1, 1], 
                    Hue[0.8857671042243551, 1, 1], 
                    Hue[0.8857671042243551, 1, 1], 
                    Hue[0.8113310485318781, 1, 1], 
                    Hue[0.8113310485318781, 1, 1], 
                    Hue[0.7388111219616389, 1, 1], 
                    Hue[0.7388111219616389, 1, 1], 
                    Hue[0.6683388300052338, 1, 1], 
                    Hue[0.6683388300052338, 1, 1], 
                    Hue[0.6000397402029956, 1, 1], 
                    Hue[0.6000397402029956, 1, 1], 
                    Hue[0.5340405316188332, 1, 1], 
                    Hue[0.5340405316188332, 1, 1], 
                    Hue[0.47048248692041184`, 1, 1], 
                    Hue[0.47048248692041184`, 1, 1], 
                    Hue[0.4095173680515788, 1, 1], 
                    Hue[0.4095173680515788, 1, 1], 
                    Hue[0.35130480479686726`, 1, 1], 
                    Hue[0.35130480479686726`, 1, 1], 
                    Hue[0.296000726032468, 1, 1], 
                    Hue[0.296000726032468, 1, 1], 
                    Hue[0.24374770049128452`, 1, 1], 
                    Hue[0.24374770049128452`, 1, 1], 
                    Hue[0.194663579371375, 1, 1], 
                    Hue[0.194663579371375, 1, 1], 
                    Hue[0.1488380422997798, 1, 1], 
                    Hue[0.1488380422997798, 1, 1], 
                    Hue[0.10632763311726107`, 1, 1], 
                    Hue[0.10632763311726107`, 1, 1], 
                    Hue[0.06715869091663118, 1, 1], 
                    Hue[0.06715869091663118, 1, 1], 
                    Hue[0.03132826385598596, 1, 1], 
                    Hue[0.03132826385598596, 1, 1], 
                    Hue[0.9988084729013986, 1, 1], 
                    Hue[0.9988084729013986, 1, 1], 
                    Hue[0.9695497880118302, 1, 1], 
                    Hue[0.9695497880118302, 1, 1], 
                    Hue[0.9434863875857, 1, 1], 
                    Hue[0.9434863875857, 1, 1], 
                    Hue[0.9205386371332858, 1, 1], 
                    Hue[0.9205386371332858, 1, 1], 
                    Hue[0.9006162006692988, 1, 1], 
                    Hue[0.9006162006692988, 1, 1], 
                    Hue[0.8836237766941165, 1, 1], 
                    Hue[0.8836237766941165, 1, 1], 
                    Hue[0.8694597112637464, 1, 1], 
                    Hue[0.8694597112637464, 1, 1], 
                    Hue[0.85802144418141, 1, 1], 
                    Hue[0.85802144418141, 1, 1], 
                    Hue[0.8492062740198365, 1, 1], 
                    Hue[0.8492062740198365, 1, 1], 
                    Hue[0.8429116052544968, 1, 1], 
                    Hue[0.8429116052544968, 1, 1], 
                    Hue[0.8390381714029644, 1, 1], 
                    Hue[0.8390381714029644, 1, 1], 
                    Hue[0.8374890855147265, 1, 1], 
                    Hue[0.8374890855147265, 1, 1], 
                    Hue[0.8381717639477806, 1, 1], 
                    Hue[0.8381717639477806, 1, 1], 
                    Hue[0.8409968658980163, 1, 1], 
                    Hue[0.8409968658980163, 1, 1], 
                    Hue[0.8458806855527117, 1, 1], 
                    Hue[0.8458806855527117, 1, 1], 
                    Hue[0.8527420684162792, 1, 1], 
                    Hue[0.8527420684162792, 1, 1], 
                    Hue[0.8615061666150565, 1, 1], 
                    Hue[0.8615061666150565, 1, 1], 
                    Hue[0.8721015386835536, 1, 1], 
                    Hue[0.8721015386835536, 1, 1], 
                    Hue[0.8844612676447515, 1, 1], 
                    Hue[0.8844612676447515, 1, 1], 
                    Hue[0.898522586062694, 1, 1], 
                    Hue[0.898522586062694, 1, 1], 
                    Hue[0.9142262597826759, 1, 1], 
                    Hue[0.9142262597826759, 1, 1], 
                    Hue[0.9315171971116003, 1, 1], 
                    Hue[0.9315171971116003, 1, 1], 
                    Hue[0.9503444226329854, 1, 1], 
                    Hue[0.9503444226329854, 1, 1], 
                    Hue[0.970659049396332, 1, 1], 
                    Hue[0.970659049396332, 1, 1], 
                    Hue[0.9924165973493029, 1, 1], 
                    Hue[0.9924165973493029, 1, 1], 
                    Hue[0.01557412362952203, 1, 1], 
                    Hue[0.01557412362952203, 1, 1], 
                    Hue[0.04009276576009518, 1, 1], 
                    Hue[0.04009276576009518, 1, 1], 
                    Hue[0.06593516184045774, 1, 1], 
                    Hue[0.06593516184045774, 1, 1], 
                    Hue[0.09306685228699166, 1, 1], 
                    Hue[0.09306685228699166, 1, 1], 
                    Hue[0.1214549126453047, 1, 1], 
                    Hue[0.1214549126453047, 1, 1], 
                    Hue[0.15106943285175675`, 1, 1], 
                    Hue[0.15106943285175675`, 1, 1], 
                    Hue[0.18188156318791293`, 1, 1], 
                    Hue[0.18188156318791293`, 1, 1], 
                    Hue[0.2138645476271862, 1, 1], 
                    Hue[0.2138645476271862, 1, 1], 
                    Hue[0.24699313254215138`, 1, 1], 
                    Hue[0.24699313254215138`, 1, 1], 
                    Hue[0.28124306964296225`, 1, 1], 
                    Hue[0.28124306964296225`, 1, 1], 
                    Hue[0.31659254647657986`, 1, 1], 
                    Hue[0.31659254647657986`, 1, 1], 
                    Hue[0.3530201981651622, 1, 1], 
                    Hue[0.3530201981651622, 1, 1], 
                    Hue[0.39050554418891786`, 1, 1], 
                    Hue[0.39050554418891786`, 1, 1], 
                    Hue[0.4290303734910717, 1, 1], 
                    Hue[0.4290303734910717, 1, 1], 
                    Hue[0.4685763131441898, 1, 1], 
                    Hue[0.4685763131441898, 1, 1], 
                    Hue[0.5091260640647721, 1, 1], 
                    Hue[0.5091260640647721, 1, 1], 
                    Hue[0.5506645361703, 1, 1], 
                    Hue[0.5506645361703, 1, 1], 
                    Hue[0.5931751705100399, 1, 1], 
                    Hue[0.5931751705100399, 1, 1], 
                    Hue[0.6366445397467777, 1, 1], 
                    Hue[0.6366445397467777, 1, 1], 
                    Hue[0.681058239309198, 1, 1], 
                    Hue[0.681058239309198, 1, 1], 
                    Hue[0.7264031386886708, 1, 1], 
                    Hue[0.7264031386886708, 1, 1], 
                    Hue[0.7726674425113982, 1, 1], 
                    Hue[0.7726674425113982, 1, 1], 
                    Hue[0.819837918765256, 1, 1], 
                    Hue[0.819837918765256, 1, 1], 
                    Hue[0.8679043674511565, 1, 1], 
                    Hue[0.8679043674511565, 1, 1], 
                    Hue[0.9168543511248141, 1, 1], 
                    Hue[0.9168543511248141, 1, 1], 
                    Hue[0.9666787266131991, 1, 1], 
                    Hue[0.9666787266131991, 1, 1], 
                    Hue[0.01736632464259125, 1, 1], 
                    Hue[0.01736632464259125, 1, 1], 
                    Hue[0.06890794089973355, 1, 1], 
                    Hue[0.06890794089973355, 1, 1], 
                    Hue[0.12129416957634404`, 1, 1], 
                    Hue[0.12129416957634404`, 1, 1], 
                    Hue[0.1745166808135452, 1, 1], 
                    Hue[0.1745166808135452, 1, 1], 
                    Hue[0.2285663347303934, 1, 1], 
                    Hue[0.2285663347303934, 1, 1], 
                    Hue[0.2834351028766379, 1, 1], 
                    Hue[0.2834351028766379, 1, 1], 
                    Hue[0.3391154820244295, 1, 1], 
                    Hue[0.3391154820244295, 1, 1], 
                    Hue[0.3955997937834623, 1, 1], 
                    Hue[0.3955997937834623, 1, 1], 
                    Hue[0.45287936850823385`, 1, 1], 
                    Hue[0.45287936850823385`, 1, 1], 
                    Hue[0.5109476098222223, 1, 1], 
                    Hue[0.5109476098222223, 1, 1], 
                    Hue[0.5697982658184998, 1, 1], 
                    Hue[0.5697982658184998, 1, 1], 
                    Hue[0.6294239155644445, 1, 1], 
                    Hue[0.6294239155644445, 1, 1], 
                    Hue[0.689818553661153, 1, 1], 
                    Hue[0.689818553661153, 1, 1], 
                    Hue[0.750976510534746, 1, 1], 
                    Hue[0.750976510534746, 1, 1], 
                    Hue[0.8128912929293306, 1, 1], 
                    Hue[0.8128912929293306, 1, 1], 
                    Hue[0.8755568751916872, 1, 1], 
                    Hue[0.8755568751916872, 1, 1], 
                    Hue[0.9389682660024058, 1, 1], 
                    Hue[0.9389682660024058, 1, 1], 
                    Hue[0.0031178192768510317`, 1, 1], 
                    Hue[0.0031178192768510317`, 1, 1], 
                    Hue[0.06800275159963283, 1, 1], 
                    Hue[0.06800275159963283, 1, 1], 
                    Hue[0.1336157129120437, 1, 1], 
                    Hue[0.1336157129120437, 1, 1], 
                    Hue[0.19995455989782093`, 1, 1], 
                    Hue[0.19995455989782093`, 1, 1], 
                    Hue[0.2670094640187561, 1, 1], 
                    Hue[0.2670094640187561, 1, 1], 
                    Hue[0.3347809278038839, 1, 1], 
                    Hue[0.3347809278038839, 1, 1], 
                    Hue[0.4032618549635441, 1, 1], 
                    Hue[0.4032618549635441, 1, 1], 
                    Hue[0.4724477701914594, 1, 1], 
                    Hue[0.4724477701914594, 1, 1], 
                    Hue[0.5423359564001664, 1, 1], 
                    Hue[0.5423359564001664, 1, 1], 
                    Hue[0.6129204992088491, 1, 1], 
                    Hue[0.6129204992088491, 1, 1], 
                    Hue[0.6841963849615734, 1, 1], 
                    Hue[0.6841963849615734, 1, 1], 
                    Hue[0.7561619322423868, 1, 1], 
                    Hue[0.7561619322423868, 1, 1], 
                    Hue[0.82881140408447, 1, 1], 
                    Hue[0.82881140408447, 1, 1], 
                    Hue[0.902143300822762, 1, 1], 
                    Hue[0.902143300822762, 1, 1], 
                    Hue[0.9761493691266603, 1, 1], 
                    Hue[0.9761493691266603, 1, 1], 
                    Hue[0.05083015294981264, 1, 1], 
                    Hue[0.05083015294981264, 1, 1], 
                    Hue[0.12617992086590107`, 1, 1], 
                    Hue[0.12617992086590107`, 1, 1], 
                    Hue[0.2021963971348071, 1, 1], 
                    Hue[0.2021963971348071, 1, 1], 
                    Hue[0.27887567203613706`, 1, 1], 
                    Hue[0.27887567203613706`, 1, 1], 
                    Hue[0.356212734612382, 1, 1], 
                    Hue[0.356212734612382, 1, 1], 
                    Hue[0.4342080079703386, 1, 1], 
                    Hue[0.4342080079703386, 1, 1], 
                    Hue[0.5128557849609001, 1, 1], 
                    Hue[0.5128557849609001, 1, 1], 
                    Hue[0.5921532077475121, 1, 1], 
                    Hue[0.5921532077475121, 1, 1], 
                    Hue[0.6720964647032827, 1, 1], 
                    Hue[0.6720964647032827, 1, 1], 
                    Hue[0.752681238076534, 1, 1], 
                    Hue[0.752681238076534, 1, 1], 
                    Hue[0.8339078459155588, 1, 1], 
                    Hue[0.8339078459155588, 1, 1], 
                    Hue[0.9157742987005092, 1, 1], 
                    Hue[0.9157742987005092, 1, 1], 
                    Hue[0.998275547272889, 1, 1], 
                    Hue[0.998275547272889, 1, 1], 
                    Hue[0.08140743783099906, 1, 1], 
                    Hue[0.08140743783099906, 1, 1], 
                    Hue[0.16516937419891978`, 1, 1], 
                    Hue[0.16516937419891978`, 1, 1], 
                    Hue[0.249558861554528, 1, 1], 
                    Hue[0.249558861554528, 1, 1], 
                    Hue[0.33456939691468257`, 1, 1], 
                    Hue[0.33456939691468257`, 1, 1], 
                    Hue[0.42020253714678935`, 1, 1], 
                    Hue[0.42020253714678935`, 1, 1], 
                    Hue[0.5064565805732228, 1, 1], 
                    Hue[0.5064565805732228, 1, 1], 
                    Hue[0.5933250672959842, 1, 1], 
                    Hue[0.5933250672959842, 1, 1], 
                    Hue[0.6808033250107832, 1, 1], 
                    Hue[0.6808033250107832, 1, 1], 
                    Hue[0.7688952871616338, 1, 1], 
                    Hue[0.7688952871616338, 1, 1], 
                    Hue[0.8576085005203646, 1, 1], 
                    Hue[0.8576085005203646, 1, 1], 
                    Hue[0.9469223721299237, 1, 1], 
                    Hue[0.9469223721299237, 1, 1], 
                    Hue[0.03683378074896659, 1, 1], 
                    Hue[0.03683378074896659, 1, 1], 
                    Hue[0.12735624017682576`, 1, 1], 
                    Hue[0.12735624017682576`, 1, 1], 
                    Hue[0.2184734433831852, 1, 1], 
                    Hue[0.2184734433831852, 1, 1], 
                    Hue[0.31018333537811105`, 1, 1], 
                    Hue[0.31018333537811105`, 1, 1], 
                    Hue[0.40251345092619834`, 1, 1], 
                    Hue[0.40251345092619834`, 1, 1], 
                    Hue[0.4954396434137711, 1, 1], 
                    Hue[0.4954396434137711, 1, 1], 
                    Hue[0.5889176172549687, 1, 1], 
                    Hue[0.5889176172549687, 1, 1], 
                    Hue[0.6829974887015433, 1, 1], 
                    Hue[0.6829974887015433, 1, 1], 
                    Hue[0.7776940568752733, 1, 1], 
                    Hue[0.7776940568752733, 1, 1], 
                    Hue[0.8729551243678357, 1, 1]}],
                   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
                   Axes->{True, None},
                   AxesOrigin->{0, 0},
                   
                   Epilog->LineBox[{{1, 6.428999298885704*^-7}, {
                    2, 8.300133162933977*^-7}, {3, 7.781726838131124*^-7}, {
                    4, 4.6764095957323114`*^-7}, {5, 1.043910806555466*^-6}, {
                    6, 1.9363121266104814`*^-6}, {
                    7, 2.5371543977632263`*^-6}, {
                    8, 2.9470838347403353`*^-6}, {9, 3.647988578059276*^-6}, {
                    10, 5.0652959303135986`*^-6}, {
                    11, 5.647471884200192*^-6}, {12, 7.440611653765158*^-6}, {
                    13, 8.541042412232665*^-6}, {
                    14, 0.000010785657623601734`}, {
                    15, 0.000013548039552556158`}, {
                    16, 0.000016389288145999142`}, {
                    17, 0.000019370341822961964`}, {
                    18, 0.000023151874274806162`}, {
                    19, 0.000027214964775029584`}, {
                    20, 0.00003298613458447244}, {
                    21, 0.000038910018195724345`}, {
                    22, 0.000045552561335197265`}, {
                    23, 0.00005479060691654177}, {
                    24, 0.00006429922128257208}, {
                    25, 0.00007616278239534474}, {
                    26, 0.00009026280868556289}, {
                    27, 0.00010603783951001782`}, {
                    28, 0.00012536245386256408`}, {
                    29, 0.00014726329321518414`}, {
                    30, 0.00017338687560810236`}, {
                    31, 0.0002027286733535676}, {32, 0.0002371285977790524}, {
                    33, 0.0002773573152497216}, {
                    34, 0.00032229759398258136`}, {
                    35, 0.0003752971395910895}, {
                    36, 0.00043650373002215647`}, {
                    37, 0.0005055760066950821}, {38, 0.0005858805900362918}, {
                    39, 0.0006775500520590249}, {40, 0.0007826219342702768}, {
                    41, 0.0009025388206575009}, {42, 0.001039305480485389}, {
                    43, 0.0011956701636203152`}, {44, 0.001372333453100962}, {
                    45, 0.0015727408523432125`}, {
                    46, 0.0018001736459574338`}, {
                    47, 0.0020573569444218187`}, {48, 0.00234708618430293}, {
                    49, 0.002673761595434414}, {50, 0.003041592942615149}, {
                    51, 0.0034546082645969473`}, {52, 0.0039174323095321}, {
                    53, 0.0044360271345818}, {54, 0.0050145885076479735`}, {
                    55, 0.0056602214293028155`}, {56, 0.006378338695183325}, {
                    57, 0.0071761982598486295`}, {58, 0.00806089836674371}, {
                    59, 0.009039100367280487}, {60, 0.010121896114862725`}, {
                    61, 0.011314413403477289`}, {62, 0.012626930974954228`}, {
                    63, 0.014069555145806113`}, {64, 0.015649020171983965`}, {
                    65, 0.01737713002968264}, {66, 0.01926404616792687}, {
                    67, 0.021318576871179752`}, {68, 0.02355248498593144}, {
                    69, 0.025974838130848006`}, {70, 0.02859608903775095}, {
                    71, 0.031428301565331176`}, {72, 0.03447886426534259}, {
                    73, 0.037758732652052956`}, {74, 0.041276655981403156`}, {
                    75, 0.0450404461900716}, {76, 0.049058554378111977`}, {
                    77, 0.053337067512784796`}, {78, 0.05788217737092372}, {
                    79, 0.0626976824505124}, {80, 0.06778815063729648}, {
                    81, 0.07315282505761574}, {82, 0.07879337065944557}, {
                    83, 0.08470596116609645}, {84, 0.09088890961699303}, {
                    85, 0.0973337917778841}, {86, 0.10403398711129828`}, {
                    87, 0.11097887172341742`}, {88, 0.11815468760280255`}, {
                    89, 0.12554810935190835`}, {90, 0.13313862309563368`}, {
                    91, 0.14090767831896422`}, {92, 0.14883345370905351`}, {
                    93, 0.15688969021725852`}, {94, 0.16505291151658366`}, {
                    95, 0.1732923863175134}, {96, 0.18157883363457714`}, {
                    97, 0.18987738440254207`}, {98, 0.19815578080140567`}, {
                    99, 0.20638166931513177`}, {100, 0.21452211060539822`}, {
                    101, 0.2225470466340991}, {102, 0.2304236458477509}, {
                    103, 0.2381170557929398}, {104, 0.24559214920246428`}, {
                    105, 0.25281425541466324`}, {106, 0.25975711992967254`}, {
                    107, 0.2664048807938117}, {108, 0.2727466756392955}, {
                    109, 0.278774745993297}, {110, 0.28447451710706007`}, {
                    111, 0.2898232323527047}, {112, 0.2947941444576497}, {
                    113, 0.2993768817121557}, {114, 0.3035834295729913}, {
                    115, 0.30745293029535886`}, {116, 0.3110359940147565}, {
                    117, 0.31436830790722187`}, {118, 0.31744870567324335`}, {
                    119, 0.32023568186056617`}, {120, 0.32266584728751735`}, {
                    121, 0.3246810154757576}, {122, 0.32626575448776735`}, {
                    123, 0.32746468417830193`}, {124, 0.32838004366121354`}, {
                    125, 0.3291580087116228}, {126, 0.32995500289739244`}, {
                    127, 0.3309153591396115}, {128, 0.3321546296531875}, {
                    129, 0.3337488229745268}, {130, 0.3357352960301967}, {
                    131, 0.33812513072989614`}, {132, 0.34090558013197403`}, {
                    133, 0.3440479812524503}, {134, 0.3475157319581264}, {
                    135, 0.35126779411939724`}, {136, 0.3552635995234062}, {
                    137, 0.35945605076740555`}, {138, 0.36380309502165364`}, {
                    139, 0.3682618170894707}, {140, 0.3727896873575898}, {
                    141, 0.3773481146696921}, {142, 0.3818964500724351}, {
                    143, 0.386399963189704}, {144, 0.3908213403070935}, {
                    145, 0.3951275291388884}, {146, 0.3992872173908136}, {
                    147, 0.40326908378486537`}, {148, 0.40704615649228887`}, {
                    149, 0.41058933302994277`}, {150, 0.4138767086550128}, {
                    151, 0.4168848119349655}, {152, 0.41959003672901507`}, {
                    153, 0.42197400083078695`}, {154, 0.4240182342563073}, {
                    155, 0.42570842610467846`}, {156, 0.4270284084276151}, {
                    157, 0.4279667787715813}, {158, 0.4285117872098575}, {
                    159, 0.4286552152486011}, {160, 0.428391072101157}, {
                    161, 0.42771182751484066`}, {162, 0.42661412552348493`}, {
                    163, 0.42509566140722016`}, {164, 0.42315834601746694`}, {
                    165, 0.42080139151857593`}, {166, 0.41802887142544404`}, {
                    167, 0.4148446193135151}, {168, 0.41125615472689175`}, {
                    169, 0.40727037232262336`}, {170, 0.4028965365295396}, {
                    171, 0.39814485447257336`}, {172, 0.39302834073862397`}, {
                    173, 0.38755873051365386`}, {174, 0.3817520163820842}, {
                    175, 0.3756213983417457}, {176, 0.36918390254305716`}, {
                    177, 0.3624575783871716}, {178, 0.3554597230174296}, {
                    179, 0.3482104862813334}, {180, 0.3407274621368877}, {
                    181, 0.33303186966591164`}, {182, 0.32514403899090183`}, {
                    183, 0.3170826846752446}, {184, 0.30887216512654314`}, {
                    185, 0.30053109021402163`}, {186, 0.29208089286694594`}, {
                    187, 0.28354352534977745`}, {188, 0.27493784463691473`}, {
                    189, 0.2662854865892516}, {190, 0.2576057530193071}, {
                    191, 0.24891875615456377`}, {192, 0.2402446784624649}, {
                    193, 0.23160046630781933`}, {194, 0.22300477914381434`}, {
                    195, 0.21447612893178863`}, {196, 0.20602928892683842`}, {
                    197, 0.19767930070599377`}, {198, 0.1894417438033198}, {
                    199, 0.18133075612402988`}, {200, 0.1733587833081945}, {
                    201, 0.1655395991734645}, {202, 0.15788202557966252`}, {
                    203, 0.15039717537554026`}, {204, 0.14309223512486796`}, {
                    205, 0.13597691847616825`}, {206, 0.12905654372321876`}, {
                    207, 0.12233837713459317`}, {208, 0.11582877008669083`}, {
                    209, 0.10952930416528646`}, {210, 0.10344509549717357`}, {
                    211, 0.09757778376986678}, {212, 0.09192792049054814}, {
                    213, 0.0864977348675539}, {214, 0.0812864745877578}, {
                    215, 0.07629347556054447}, {216, 0.07151652572376288}, {
                    217, 0.06695372335837245}, {218, 0.06260329711239984}, {
                    219, 0.05846071053734583}, {220, 0.05452300698730784}, {
                    221, 0.050784942749714054`}, {222, 0.04724294396141986}, {
                    223, 0.043891502726421776`}, {224, 0.04072478506425511}, {
                    225, 0.03773764484303948}, {226, 0.03492385838782062}, {
                    227, 0.03227800698373038}, {228, 0.029794069529439693`}, {
                    229, 0.02746507363996537}, {230, 0.025284546999225974`}, {
                    231, 0.023246418865307334`}, {232, 0.02134442934102172}, {
                    233, 0.01957179487247538}, {234, 0.017922448814677548`}, {
                    235, 0.016390454128966114`}, {
                    236, 0.014969138688617007`}, {
                    237, 0.013652521746965117`}, {
                    238, 0.012435042110557354`}, {
                    239, 0.011311441399693273`}, {
                    240, 0.010275117299394908`}, {
                    241, 0.009320368359099152}, {242, 0.00844323104166659}, {
                    243, 0.007638196675815911}, {244, 0.006900104815892843}, {
                    245, 0.006225281234642151}, {246, 0.005608822675763076}, {
                    247, 0.005045307563466428}, {248, 0.004532173679180542}, {
                    249, 0.004066527000051145}, {
                    250, 0.0036431332194328714`}, {
                    251, 0.003258975402727404}}],
                   Frame->True,
                   FrameTicks->{{{1., 
                    FormBox[
                    RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
                    FormBox[
                    RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
                    FormBox[
                    RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
                    FormBox[
                    RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
                    FormBox["0", TraditionalForm]}, {157.25, 
                    FormBox["2.5`", TraditionalForm]}, {188.5, 
                    FormBox["5", TraditionalForm]}, {219.75, 
                    FormBox["7.5`", TraditionalForm]}, {251., 
                    FormBox["10", TraditionalForm]}}, Automatic, None, None},
                   PlotRange->{-0.1, 1.1},
                   PlotRangeClipping->True,
                   PlotRangePadding->{
                    Scaled[0.02], Automatic},
                   ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]], 20->
                  GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF13lcVXUaBvCTpEHQgKlkpHEkRVTCFRBEeEBkURFZZBcO97JeuGBCLrmd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                    "], {
                    {GrayLevel[0.5], EdgeForm[None], 
                    GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ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                    
                    "]]]}, {{}, {}, {
                    {Hue[0.4393534391365692, 1, 1], LineBox[{1, 2}]}, 
                    {Hue[0.3201372697153097, 1, 1], LineBox[{2, 3}]}, 
                    {Hue[0.2232096319850034, 1, 1], LineBox[{3, 4}]}, 
                    {Hue[0.10358516356125722`, 1, 1], LineBox[{4, 5}]}, 
                    {Hue[0.005372599255212061, 1, 1], LineBox[{5, 6}]}, 
                    {Hue[0.9035686603544933, 1, 1], LineBox[{6, 7}]}, 
                    {Hue[0.7988842591287169, 1, 1], LineBox[{7, 8}]}, 
                    {Hue[0.6980701443109908, 1, 1], LineBox[{8, 9}]}, 
                    {Hue[0.5965944068167406, 1, 1], LineBox[{9, 10}]}, 
                    {Hue[0.4928555528039237, 1, 1], LineBox[{10, 11}]}, 
                    {Hue[0.39265585709506984`, 1, 1], LineBox[{11, 12}]}, 
                    {Hue[0.29253193566333907`, 1, 1], LineBox[{12, 13}]}, 
                    {Hue[0.1882309916688273, 1, 1], LineBox[{13, 14}]}, 
                    {Hue[0.08806702992125463, 1, 1], LineBox[{14, 15}]}, 
                    {Hue[0.9854789907817174, 1, 1], LineBox[{15, 16}]}, 
                    {Hue[0.8845172055478656, 1, 1], LineBox[{16, 17}]}, 
                    {Hue[0.7827332249025398, 1, 1], LineBox[{17, 18}]}, 
                    {Hue[0.6824754438075233, 1, 1], LineBox[{18, 19}]}, 
                    {Hue[0.5820498737159026, 1, 1], LineBox[{19, 20}]}, 
                    {Hue[0.4812617808739333, 1, 1], LineBox[{20, 21}]}, 
                    {Hue[0.38155270567862865`, 1, 1], LineBox[{21, 22}]}, 
                    {Hue[0.2821086601880322, 1, 1], LineBox[{22, 23}]}, 
                    {Hue[0.1819929337444189, 1, 1], LineBox[{23, 24}]}, 
                    {Hue[0.08325129391839824, 1, 1], LineBox[{24, 25}]}, 
                    {Hue[0.9841663633956332, 1, 1], LineBox[{25, 26}]}, 
                    {Hue[0.8857141077554178, 1, 1], LineBox[{26, 27}]}, 
                    {Hue[0.7871772298162246, 1, 1], LineBox[{27, 28}]}, 
                    {Hue[0.6891118122079494, 1, 1], LineBox[{28, 29}]}, 
                    {Hue[0.5912062960826425, 1, 1], LineBox[{29, 30}]}, 
                    {Hue[0.4935392024672945, 1, 1], LineBox[{30, 31}]}, 
                    {Hue[0.3961935772395639, 1, 1], LineBox[{31, 32}]}, 
                    {Hue[0.2989048819635487, 1, 1], LineBox[{32, 33}]}, 
                    {Hue[0.20194144472414555`, 1, 1], LineBox[{33, 34}]}, 
                    {Hue[0.10517975842540783`, 1, 1], LineBox[{34, 35}]}, 
                    {Hue[0.008676147230962257, 1, 1], LineBox[{35, 36}]}, 
                    {Hue[0.9123650895683947, 1, 1], LineBox[{36, 37}]}, 
                    {Hue[0.8163537610262777, 1, 1], LineBox[{37, 38}]}, 
                    {Hue[0.7205515282763565, 1, 1], LineBox[{38, 39}]}, 
                    {Hue[0.6249695450063499, 1, 1], LineBox[{39, 40}]}, 
                    {Hue[0.5296108292359909, 1, 1], LineBox[{40, 41}]}, 
                    {Hue[0.43443600295767226`, 1, 1], LineBox[{41, 42}]}, 
                    {Hue[0.3394953919330468, 1, 1], LineBox[{42, 43}]}, 
                    {Hue[0.24476243828602443`, 1, 1], LineBox[{43, 44}]}, 
                    {Hue[0.15023452379705038`, 1, 1], LineBox[{44, 45}]}, 
                    {Hue[0.05591726332051126, 1, 1], LineBox[{45, 46}]}, 
                    {Hue[0.9618102877749671, 1, 1], LineBox[{46, 47}]}, 
                    {Hue[0.8678849255300521, 1, 1], LineBox[{47, 48}]}, 
                    {Hue[0.7741471790260839, 1, 1], LineBox[{48, 49}]}, 
                    {Hue[0.6806528796568211, 1, 1], LineBox[{49, 50}]}, 
                    {Hue[0.5873248986019096, 1, 1], LineBox[{50, 51}]}, 
                    {Hue[0.49418451596794727`, 1, 1], LineBox[{51, 52}]}, 
                    {Hue[0.4012513718543245, 1, 1], LineBox[{52, 53}]}, 
                    {Hue[0.30847550574319005`, 1, 1], LineBox[{53, 54}]}, 
                    {Hue[0.21589285716824616`, 1, 1], LineBox[{54, 55}]}, 
                    {Hue[0.12349670073992161`, 1, 1], LineBox[{55, 56}]}, 
                    {Hue[0.031259713016860584`, 1, 1], LineBox[{56, 57}]}, 
                    {Hue[0.9392091910858187, 1, 1], LineBox[{57, 58}]}, 
                    {Hue[0.8473239080259873, 1, 1], LineBox[{58, 59}]}, 
                    {Hue[0.7556065340967153, 1, 1], LineBox[{59, 60}]}, 
                    {Hue[0.6640608936363054, 1, 1], LineBox[{60, 61}]}, 
                    {Hue[0.5726668749139376, 1, 1], LineBox[{61, 62}]}, 
                    {Hue[0.48143870204130346`, 1, 1], LineBox[{62, 63}]}, 
                    {Hue[0.39036514813311196`, 1, 1], LineBox[{63, 64}]}, 
                    {Hue[0.2994379002421825, 1, 1], LineBox[{64, 65}]}, 
                    {Hue[0.20865980656813093`, 1, 1], LineBox[{65, 66}]}, 
                    {Hue[0.11802481179332475`, 1, 1], LineBox[{66, 67}]}, 
                    {Hue[0.02753031297037304, 1, 1], LineBox[{67, 68}]}, 
                    {Hue[0.937172298998918, 1, 1], LineBox[{68, 69}]}, 
                    {Hue[0.8469470877910599, 1, 1], LineBox[{69, 70}]}, 
                    {Hue[0.7568529898751493, 1, 1], LineBox[{70, 71}]}, 
                    {Hue[0.6668838918648523, 1, 1], LineBox[{71, 72}]}, 
                    {Hue[0.5770355187840224, 1, 1], LineBox[{72, 73}]}, 
                    {Hue[0.4873067584282177, 1, 1], LineBox[{73, 74}]}, 
                    {Hue[0.3976885574940337, 1, 1], LineBox[{74, 75}]}, 
                    {Hue[0.3081836156328967, 1, 1], LineBox[{75, 76}]}, 
                    {Hue[0.21877826866051225`, 1, 1], LineBox[{76, 77}]}, 
                    {Hue[0.12947777818206113`, 1, 1], LineBox[{77, 78}]}, 
                    {Hue[0.040272061779198676`, 1, 1], LineBox[{78, 79}]}, 
                    {Hue[0.9511561661238163, 1, 1], LineBox[{79, 80}]}, 
                    {Hue[0.8621287005273579, 1, 1], LineBox[{80, 81}]}, 
                    {Hue[0.7731830030987756, 1, 1], LineBox[{81, 82}]}, 
                    {Hue[0.6843154147654915, 1, 1], LineBox[{82, 83}]}, 
                    {Hue[0.5955212624534687, 1, 1], LineBox[{83, 84}]}, 
                    {Hue[0.5067941698730325, 1, 1], LineBox[{84, 85}]}, 
                    {Hue[0.41813217963473104`, 1, 1], LineBox[{85, 86}]}, 
                    {Hue[0.3295280433802835, 1, 1], LineBox[{86, 87}]}, 
                    {Hue[0.24097871888163253`, 1, 1], LineBox[{87, 88}]}, 
                    {Hue[0.1524804620347105, 1, 1], LineBox[{88, 89}]}, 
                    {Hue[0.06402804928679805, 1, 1], LineBox[{89, 90}]}, 
                    {Hue[0.9756183067398464, 1, 1], LineBox[{90, 91}]}, 
                    {Hue[0.8872471615914255, 1, 1], LineBox[{91, 92}]}, 
                    {Hue[0.7989117139209876, 1, 1], LineBox[{92, 93}]}, 
                    {Hue[0.7106100480325939, 1, 1], LineBox[{93, 94}]}, 
                    {Hue[0.6223405196721529, 1, 1], LineBox[{94, 95}]}, 
                    {Hue[0.5341007127148718, 1, 1], LineBox[{95, 96}]}, 
                    {Hue[0.44589148097804515`, 1, 1], LineBox[{96, 97}]}, 
                    {Hue[0.35771438988960935`, 1, 1], LineBox[{97, 98}]}, 
                    {Hue[0.2695704180261129, 1, 1], LineBox[{98, 99}]}, 
                    {Hue[0.18146586950745308`, 1, 1], LineBox[{99, 100}]}, 
                    {Hue[0.09340746986017222, 1, 1], LineBox[{100, 101}]}, 
                    {Hue[0.005404085323135338, 1, 1], LineBox[{101, 102}]}, 
                    {Hue[0.9174681454422887, 1, 1], LineBox[{102, 103}]}, 
                    {Hue[0.8296137987629909, 1, 1], LineBox[{103, 104}]}, 
                    {Hue[0.7418598218721183, 1, 1], LineBox[{104, 105}]}, 
                    {Hue[0.6542291760888137, 1, 1], LineBox[{105, 106}]}, 
                    {Hue[0.5667513430325848, 1, 1], LineBox[{106, 107}]}, 
                    {Hue[0.47946412748234774`, 1, 1], LineBox[{107, 108}]}, 
                    {Hue[0.39241165346008955`, 1, 1], LineBox[{108, 109}]}, 
                    {Hue[0.3056448023252869, 1, 1], LineBox[{109, 110}]}, 
                    {Hue[0.21922055749572877`, 1, 1], LineBox[{110, 111}]}, 
                    {Hue[0.1332022361576938, 1, 1], LineBox[{111, 112}]}, 
                    {Hue[0.047666850681859045`, 1, 1], LineBox[{112, 113}]}, 
                    {Hue[0.9627024797532431, 1, 1], LineBox[{113, 114}]}, 
                    {Hue[0.8784143218371727, 1, 1], LineBox[{114, 115}]}, 
                    {Hue[0.7949263299068562, 1, 1], LineBox[{115, 116}]}, 
                    {Hue[0.7123730704290766, 1, 1], LineBox[{116, 117}]}, 
                    {Hue[0.6308993085613088, 1, 1], LineBox[{117, 118}]}, 
                    {Hue[0.5506492528355513, 1, 1], LineBox[{118, 119}]}, 
                    {Hue[0.4717723067273212, 1, 1], LineBox[{119, 120}]}, 
                    {Hue[0.39442199474621936`, 1, 1], LineBox[{120, 121}]}, 
                    {Hue[0.3187645584683982, 1, 1], LineBox[{121, 122}]}, 
                    {Hue[0.2449824694851805, 1, 1], LineBox[{122, 123}]}, 
                    {Hue[0.17327582930586308`, 1, 1], LineBox[{123, 124}]}, 
                    {Hue[0.10385569030144716`, 1, 1], LineBox[{124, 125}]}, 
                    {Hue[0.03693914308290094, 1, 1], LineBox[{125, 126}]}, 
                    {Hue[0.9727328323643722, 1, 1], LineBox[{126, 127}]}, 
                    {Hue[0.9114250873976556, 1, 1], LineBox[{127, 128}]}, 
                    {Hue[0.8531770723884569, 1, 1], LineBox[{128, 129}]}, 
                    {Hue[0.7981179230268365, 1, 1], LineBox[{129, 130}]}, 
                    {Hue[0.7463434738319357, 1, 1], LineBox[{130, 131}]}, 
                    {Hue[0.6979169230722659, 1, 1], LineBox[{131, 132}]}, 
                    {Hue[0.652870401171725, 1, 1], LineBox[{132, 133}]}, 
                    {Hue[0.6112085161386327, 1, 1], LineBox[{133, 134}]}, 
                    {Hue[0.5729111580662459, 1, 1], LineBox[{134, 135}]}, 
                    {Hue[0.5379361605368185, 1, 1], LineBox[{135, 136}]}, 
                    {Hue[0.5062229764889836, 1, 1], LineBox[{136, 137}]}, 
                    {Hue[0.4776979103140166, 1, 1], LineBox[{137, 138}]}, 
                    {Hue[0.45227423713150755`, 1, 1], LineBox[{138, 139}]}, 
                    {Hue[0.42985665197459183`, 1, 1], LineBox[{139, 140}]}, 
                    {Hue[0.41034533929896294`, 1, 1], LineBox[{140, 141}]}, 
                    {Hue[0.39363480451596733`, 1, 1], LineBox[{141, 142}]}, 
                    {Hue[0.3796206968339214, 1, 1], LineBox[{142, 143}]}, 
                    {Hue[0.36819635676874646`, 1, 1], LineBox[{143, 144}]}, 
                    {Hue[0.3592588810689745, 1, 1], LineBox[{144, 145}]}, 
                    {Hue[0.35270652242754846`, 1, 1], LineBox[{145, 146}]}, 
                    {Hue[0.348441715653491, 1, 1], LineBox[{146, 147}]}, 
                    {Hue[0.34637025854678805`, 1, 1], LineBox[{147, 148}]}, 
                    {Hue[0.34640317257292524`, 1, 1], LineBox[{148, 149}]}, 
                    {Hue[0.3484549273788225, 1, 1], LineBox[{149, 150}]}, 
                    {Hue[0.35244536014669253`, 1, 1], LineBox[{150, 151}]}, 
                    {Hue[0.35829899021649714`, 1, 1], LineBox[{151, 152}]}, 
                    {Hue[0.3659444120977315, 1, 1], LineBox[{152, 153}]}, 
                    {Hue[0.37531470466298017`, 1, 1], LineBox[{153, 154}]}, 
                    {Hue[0.38634778739783043`, 1, 1], LineBox[{154, 155}]}, 
                    {Hue[0.3989849084100206, 1, 1], LineBox[{155, 156}]}, 
                    {Hue[0.4131714890798039, 1, 1], LineBox[{156, 157}]}, 
                    {Hue[0.4288562177677756, 1, 1], LineBox[{157, 158}]}, 
                    {Hue[0.4459917475621054, 1, 1], LineBox[{158, 159}]}, 
                    {Hue[0.46453343042731615`, 1, 1], LineBox[{159, 160}]}, 
                    {Hue[0.484439541654788, 1, 1], LineBox[{160, 161}]}, 
                    {Hue[0.5056713520991165, 1, 1], LineBox[{161, 162}]}, 
                    {Hue[0.5281925529249144, 1, 1], LineBox[{162, 163}]}, 
                    {Hue[0.5519691183846559, 1, 1], LineBox[{163, 164}]}, 
                    {Hue[0.5769693278688665, 1, 1], LineBox[{164, 165}]}, 
                    {Hue[0.603163245870465, 1, 1], LineBox[{165, 166}]}, 
                    {Hue[0.6305229668447081, 1, 1], LineBox[{166, 167}]}, 
                    {Hue[0.6590229110973536, 1, 1], LineBox[{167, 168}]}, 
                    {Hue[0.6886380618054015, 1, 1], LineBox[{168, 169}]}, 
                    {Hue[0.7193449813186077, 1, 1], LineBox[{169, 170}]}, 
                    {Hue[0.7511224993169587, 1, 1], LineBox[{170, 171}]}, 
                    {Hue[0.7839493701684175, 1, 1], LineBox[{171, 172}]}, 
                    {Hue[0.817806735017353, 1, 1], LineBox[{172, 173}]}, 
                    {Hue[0.8526758485656166, 1, 1], LineBox[{173, 174}]}, 
                    {Hue[0.8885399846257238, 1, 1], LineBox[{174, 175}]}, 
                    {Hue[0.9253823506081306, 1, 1], LineBox[{175, 176}]}, 
                    {Hue[0.9631867200833969, 1, 1], LineBox[{176, 177}]}, 
                    {Hue[0.0019395945408895489`, 1, 1], 
                    LineBox[{177, 178}]}, 
                    {Hue[0.04162675481913263, 1, 1], LineBox[{178, 179}]}, 
                    {Hue[0.08223416167340272, 1, 1], LineBox[{179, 180}]}, 
                    {Hue[0.12375029778114359`, 1, 1], LineBox[{180, 181}]}, 
                    {Hue[0.16616162602928783`, 1, 1], LineBox[{181, 182}]}, 
                    {Hue[0.20945806400140338`, 1, 1], LineBox[{182, 183}]}, 
                    {Hue[0.2536276524015086, 1, 1], LineBox[{183, 184}]}, 
                    {Hue[0.2986598507746231, 1, 1], LineBox[{184, 185}]}, 
                    {Hue[0.3445451940568987, 1, 1], LineBox[{185, 186}]}, 
                    {Hue[0.39127272249841616`, 1, 1], LineBox[{186, 187}]}, 
                    {Hue[0.4388341517654788, 1, 1], LineBox[{187, 188}]}, 
                    {Hue[0.48722035714046374`, 1, 1], LineBox[{188, 189}]}, 
                    {Hue[0.5364224284486252, 1, 1], LineBox[{189, 190}]}, 
                    {Hue[0.5864334679766994, 1, 1], LineBox[{190, 191}]}, 
                    {Hue[0.6372440491692886, 1, 1], LineBox[{191, 192}]}, 
                    {Hue[0.6888480265350307, 1, 1], LineBox[{192, 193}]}, 
                    {Hue[0.7412365911076693, 1, 1], LineBox[{193, 194}]}, 
                    {Hue[0.7944032577634785, 1, 1], LineBox[{194, 195}]}, 
                    {Hue[0.8483407608749416, 1, 1], LineBox[{195, 196}]}, 
                    {Hue[0.903042428880277, 1, 1], LineBox[{196, 197}]}, 
                    {Hue[0.9585025639644492, 1, 1], LineBox[{197, 198}]}, 
                    {Hue[0.014714511057447872`, 1, 1], LineBox[{198, 199}]}, 
                    {Hue[0.07167338075811815, 1, 1], LineBox[{199, 200}]}, 
                    {Hue[0.12937282649114865`, 1, 1], LineBox[{200, 201}]}, 
                    {Hue[0.18780666443835414`, 1, 1], LineBox[{201, 202}]}, 
                    {Hue[0.24696913203339063`, 1, 1], LineBox[{202, 203}]}, 
                    {Hue[0.30685523730739184`, 1, 1], LineBox[{203, 204}]}, 
                    {Hue[0.36745957905062887`, 1, 1], LineBox[{204, 205}]}, 
                    {Hue[0.42877769280745087`, 1, 1], LineBox[{205, 206}]}, 
                    {Hue[0.4908037925029819, 1, 1], LineBox[{206, 207}]}, 
                    {Hue[0.5535350384234963, 1, 1], LineBox[{207, 208}]}, 
                    {Hue[0.6169653916379854, 1, 1], LineBox[{208, 209}]}, 
                    {Hue[0.6810907257805103, 1, 1], LineBox[{209, 210}]}, 
                    {Hue[0.7459063147912967, 1, 1], LineBox[{210, 211}]}, 
                    {Hue[0.8114104826374526, 1, 1], LineBox[{211, 212}]}, 
                    {Hue[0.8775950861296535, 1, 1], LineBox[{212, 213}]}, 
                    {Hue[0.9444588876589283, 1, 1], LineBox[{213, 214}]}, 
                    {Hue[0.0119970511399642, 1, 1], LineBox[{214, 215}]}, 
                    {Hue[0.08020606705135687, 1, 1], LineBox[{215, 216}]}, 
                    {Hue[0.14908284555554469`, 1, 1], LineBox[{216, 217}]}, 
                    {Hue[0.2186219206976702, 1, 1], LineBox[{217, 218}]}, 
                    {Hue[0.28882197240286506`, 1, 1], LineBox[{218, 219}]}, 
                    {Hue[0.35967627266925456`, 1, 1], LineBox[{219, 220}]}, 
                    {Hue[0.43118455611638384`, 1, 1], LineBox[{220, 221}]}, 
                    {Hue[0.5033433290086244, 1, 1], LineBox[{221, 222}]}, 
                    {Hue[0.5761460418402145, 1, 1], LineBox[{222, 223}]}, 
                    {Hue[0.6495947288059509, 1, 1], LineBox[{223, 224}]}, 
                    {Hue[0.7236843398166319, 1, 1], LineBox[{224, 225}]}, 
                    {Hue[0.7984095434383008, 1, 1], LineBox[{225, 226}]}, 
                    {Hue[0.873769870187243, 1, 1], LineBox[{226, 227}]}, 
                    {Hue[0.9497602611869326, 1, 1], LineBox[{227, 228}]}, 
                    {Hue[0.02638015960819491, 1, 1], LineBox[{228, 229}]}, 
                    {Hue[0.10362426395156402`, 1, 1], LineBox[{229, 230}]}, 
                    {Hue[0.18149402581251858`, 1, 1], LineBox[{230, 231}]}, 
                    {Hue[0.25998363569095884`, 1, 1], LineBox[{231, 232}]}, 
                    {Hue[0.33908888485147026`, 1, 1], LineBox[{232, 233}]}, 
                    {Hue[0.41881004827500784`, 1, 1], LineBox[{233, 234}]}, 
                    {Hue[0.4991438872643397, 1, 1], LineBox[{234, 235}]}, 
                    {Hue[0.5800901965951237, 1, 1], LineBox[{235, 236}]}, 
                    {Hue[0.6616434324791175, 1, 1], LineBox[{236, 237}]}, 
                    {Hue[0.7437997538299062, 1, 1], LineBox[{237, 238}]}, 
                    {Hue[0.8265609418880175, 1, 1], LineBox[{238, 239}]}, 
                    {Hue[0.9099231632063347, 1, 1], LineBox[{239, 240}]}, 
                    {Hue[0.9938830944227226, 1, 1], LineBox[{240, 241}]}, 
                    {Hue[0.07843854256843062, 1, 1], LineBox[{241, 242}]}, 
                    {Hue[0.16358722556755453`, 1, 1], LineBox[{242, 243}]}, 
                    {Hue[0.24932940137544837`, 1, 1], LineBox[{243, 244}]}, 
                    {Hue[0.33566483509107636`, 1, 1], LineBox[{244, 245}]}, 
                    {Hue[0.4225903879562467, 1, 1], LineBox[{245, 246}]}, 
                    {Hue[0.5100948438311672, 1, 1], LineBox[{246, 247}]}, 
                    {Hue[0.598181316104291, 1, 1], LineBox[{247, 248}]}, 
                    {Hue[0.6868528306413736, 1, 1], LineBox[{248, 249}]}, 
                    {Hue[0.7761040425171196, 1, 1], LineBox[{249, 250}]}, 
                    {Hue[0.8659330990069755, 1, 1], LineBox[{250, 251}]}}}},
                    VertexColors->{
                    Hue[0.4393534391365692, 1, 1], 
                    Hue[0.3201372697153097, 1, 1], 
                    Hue[0.2232096319850034, 1, 1], 
                    Hue[0.10358516356125722`, 1, 1], 
                    Hue[0.005372599255212061, 1, 1], 
                    Hue[0.9035686603544933, 1, 1], 
                    Hue[0.7988842591287169, 1, 1], 
                    Hue[0.6980701443109908, 1, 1], 
                    Hue[0.5965944068167406, 1, 1], 
                    Hue[0.4928555528039237, 1, 1], 
                    Hue[0.39265585709506984`, 1, 1], 
                    Hue[0.29253193566333907`, 1, 1], 
                    Hue[0.1882309916688273, 1, 1], 
                    Hue[0.08806702992125463, 1, 1], 
                    Hue[0.9854789907817174, 1, 1], 
                    Hue[0.8845172055478656, 1, 1], 
                    Hue[0.7827332249025398, 1, 1], 
                    Hue[0.6824754438075233, 1, 1], 
                    Hue[0.5820498737159026, 1, 1], 
                    Hue[0.4812617808739333, 1, 1], 
                    Hue[0.38155270567862865`, 1, 1], 
                    Hue[0.2821086601880322, 1, 1], 
                    Hue[0.1819929337444189, 1, 1], 
                    Hue[0.08325129391839824, 1, 1], 
                    Hue[0.9841663633956332, 1, 1], 
                    Hue[0.8857141077554178, 1, 1], 
                    Hue[0.7871772298162246, 1, 1], 
                    Hue[0.6891118122079494, 1, 1], 
                    Hue[0.5912062960826425, 1, 1], 
                    Hue[0.4935392024672945, 1, 1], 
                    Hue[0.3961935772395639, 1, 1], 
                    Hue[0.2989048819635487, 1, 1], 
                    Hue[0.20194144472414555`, 1, 1], 
                    Hue[0.10517975842540783`, 1, 1], 
                    Hue[0.008676147230962257, 1, 1], 
                    Hue[0.9123650895683947, 1, 1], 
                    Hue[0.8163537610262777, 1, 1], 
                    Hue[0.7205515282763565, 1, 1], 
                    Hue[0.6249695450063499, 1, 1], 
                    Hue[0.5296108292359909, 1, 1], 
                    Hue[0.43443600295767226`, 1, 1], 
                    Hue[0.3394953919330468, 1, 1], 
                    Hue[0.24476243828602443`, 1, 1], 
                    Hue[0.15023452379705038`, 1, 1], 
                    Hue[0.05591726332051126, 1, 1], 
                    Hue[0.9618102877749671, 1, 1], 
                    Hue[0.8678849255300521, 1, 1], 
                    Hue[0.7741471790260839, 1, 1], 
                    Hue[0.6806528796568211, 1, 1], 
                    Hue[0.5873248986019096, 1, 1], 
                    Hue[0.49418451596794727`, 1, 1], 
                    Hue[0.4012513718543245, 1, 1], 
                    Hue[0.30847550574319005`, 1, 1], 
                    Hue[0.21589285716824616`, 1, 1], 
                    Hue[0.12349670073992161`, 1, 1], 
                    Hue[0.031259713016860584`, 1, 1], 
                    Hue[0.9392091910858187, 1, 1], 
                    Hue[0.8473239080259873, 1, 1], 
                    Hue[0.7556065340967153, 1, 1], 
                    Hue[0.6640608936363054, 1, 1], 
                    Hue[0.5726668749139376, 1, 1], 
                    Hue[0.48143870204130346`, 1, 1], 
                    Hue[0.39036514813311196`, 1, 1], 
                    Hue[0.2994379002421825, 1, 1], 
                    Hue[0.20865980656813093`, 1, 1], 
                    Hue[0.11802481179332475`, 1, 1], 
                    Hue[0.02753031297037304, 1, 1], 
                    Hue[0.937172298998918, 1, 1], 
                    Hue[0.8469470877910599, 1, 1], 
                    Hue[0.7568529898751493, 1, 1], 
                    Hue[0.6668838918648523, 1, 1], 
                    Hue[0.5770355187840224, 1, 1], 
                    Hue[0.4873067584282177, 1, 1], 
                    Hue[0.3976885574940337, 1, 1], 
                    Hue[0.3081836156328967, 1, 1], 
                    Hue[0.21877826866051225`, 1, 1], 
                    Hue[0.12947777818206113`, 1, 1], 
                    Hue[0.040272061779198676`, 1, 1], 
                    Hue[0.9511561661238163, 1, 1], 
                    Hue[0.8621287005273579, 1, 1], 
                    Hue[0.7731830030987756, 1, 1], 
                    Hue[0.6843154147654915, 1, 1], 
                    Hue[0.5955212624534687, 1, 1], 
                    Hue[0.5067941698730325, 1, 1], 
                    Hue[0.41813217963473104`, 1, 1], 
                    Hue[0.3295280433802835, 1, 1], 
                    Hue[0.24097871888163253`, 1, 1], 
                    Hue[0.1524804620347105, 1, 1], 
                    Hue[0.06402804928679805, 1, 1], 
                    Hue[0.9756183067398464, 1, 1], 
                    Hue[0.8872471615914255, 1, 1], 
                    Hue[0.7989117139209876, 1, 1], 
                    Hue[0.7106100480325939, 1, 1], 
                    Hue[0.6223405196721529, 1, 1], 
                    Hue[0.5341007127148718, 1, 1], 
                    Hue[0.44589148097804515`, 1, 1], 
                    Hue[0.35771438988960935`, 1, 1], 
                    Hue[0.2695704180261129, 1, 1], 
                    Hue[0.18146586950745308`, 1, 1], 
                    Hue[0.09340746986017222, 1, 1], 
                    Hue[0.005404085323135338, 1, 1], 
                    Hue[0.9174681454422887, 1, 1], 
                    Hue[0.8296137987629909, 1, 1], 
                    Hue[0.7418598218721183, 1, 1], 
                    Hue[0.6542291760888137, 1, 1], 
                    Hue[0.5667513430325848, 1, 1], 
                    Hue[0.47946412748234774`, 1, 1], 
                    Hue[0.39241165346008955`, 1, 1], 
                    Hue[0.3056448023252869, 1, 1], 
                    Hue[0.21922055749572877`, 1, 1], 
                    Hue[0.1332022361576938, 1, 1], 
                    Hue[0.047666850681859045`, 1, 1], 
                    Hue[0.9627024797532431, 1, 1], 
                    Hue[0.8784143218371727, 1, 1], 
                    Hue[0.7949263299068562, 1, 1], 
                    Hue[0.7123730704290766, 1, 1], 
                    Hue[0.6308993085613088, 1, 1], 
                    Hue[0.5506492528355513, 1, 1], 
                    Hue[0.4717723067273212, 1, 1], 
                    Hue[0.39442199474621936`, 1, 1], 
                    Hue[0.3187645584683982, 1, 1], 
                    Hue[0.2449824694851805, 1, 1], 
                    Hue[0.17327582930586308`, 1, 1], 
                    Hue[0.10385569030144716`, 1, 1], 
                    Hue[0.03693914308290094, 1, 1], 
                    Hue[0.9727328323643722, 1, 1], 
                    Hue[0.9114250873976556, 1, 1], 
                    Hue[0.8531770723884569, 1, 1], 
                    Hue[0.7981179230268365, 1, 1], 
                    Hue[0.7463434738319357, 1, 1], 
                    Hue[0.6979169230722659, 1, 1], 
                    Hue[0.652870401171725, 1, 1], 
                    Hue[0.6112085161386327, 1, 1], 
                    Hue[0.5729111580662459, 1, 1], 
                    Hue[0.5379361605368185, 1, 1], 
                    Hue[0.5062229764889836, 1, 1], 
                    Hue[0.4776979103140166, 1, 1], 
                    Hue[0.45227423713150755`, 1, 1], 
                    Hue[0.42985665197459183`, 1, 1], 
                    Hue[0.41034533929896294`, 1, 1], 
                    Hue[0.39363480451596733`, 1, 1], 
                    Hue[0.3796206968339214, 1, 1], 
                    Hue[0.36819635676874646`, 1, 1], 
                    Hue[0.3592588810689745, 1, 1], 
                    Hue[0.35270652242754846`, 1, 1], 
                    Hue[0.348441715653491, 1, 1], 
                    Hue[0.34637025854678805`, 1, 1], 
                    Hue[0.34640317257292524`, 1, 1], 
                    Hue[0.3484549273788225, 1, 1], 
                    Hue[0.35244536014669253`, 1, 1], 
                    Hue[0.35829899021649714`, 1, 1], 
                    Hue[0.3659444120977315, 1, 1], 
                    Hue[0.37531470466298017`, 1, 1], 
                    Hue[0.38634778739783043`, 1, 1], 
                    Hue[0.3989849084100206, 1, 1], 
                    Hue[0.4131714890798039, 1, 1], 
                    Hue[0.4288562177677756, 1, 1], 
                    Hue[0.4459917475621054, 1, 1], 
                    Hue[0.46453343042731615`, 1, 1], 
                    Hue[0.484439541654788, 1, 1], 
                    Hue[0.5056713520991165, 1, 1], 
                    Hue[0.5281925529249144, 1, 1], 
                    Hue[0.5519691183846559, 1, 1], 
                    Hue[0.5769693278688665, 1, 1], 
                    Hue[0.603163245870465, 1, 1], 
                    Hue[0.6305229668447081, 1, 1], 
                    Hue[0.6590229110973536, 1, 1], 
                    Hue[0.6886380618054015, 1, 1], 
                    Hue[0.7193449813186077, 1, 1], 
                    Hue[0.7511224993169587, 1, 1], 
                    Hue[0.7839493701684175, 1, 1], 
                    Hue[0.817806735017353, 1, 1], 
                    Hue[0.8526758485656166, 1, 1], 
                    Hue[0.8885399846257238, 1, 1], 
                    Hue[0.9253823506081306, 1, 1], 
                    Hue[0.9631867200833969, 1, 1], 
                    Hue[0.0019395945408895489`, 1, 1], 
                    Hue[0.04162675481913263, 1, 1], 
                    Hue[0.08223416167340272, 1, 1], 
                    Hue[0.12375029778114359`, 1, 1], 
                    Hue[0.16616162602928783`, 1, 1], 
                    Hue[0.20945806400140338`, 1, 1], 
                    Hue[0.2536276524015086, 1, 1], 
                    Hue[0.2986598507746231, 1, 1], 
                    Hue[0.3445451940568987, 1, 1], 
                    Hue[0.39127272249841616`, 1, 1], 
                    Hue[0.4388341517654788, 1, 1], 
                    Hue[0.48722035714046374`, 1, 1], 
                    Hue[0.5364224284486252, 1, 1], 
                    Hue[0.5864334679766994, 1, 1], 
                    Hue[0.6372440491692886, 1, 1], 
                    Hue[0.6888480265350307, 1, 1], 
                    Hue[0.7412365911076693, 1, 1], 
                    Hue[0.7944032577634785, 1, 1], 
                    Hue[0.8483407608749416, 1, 1], 
                    Hue[0.903042428880277, 1, 1], 
                    Hue[0.9585025639644492, 1, 1], 
                    Hue[0.014714511057447872`, 1, 1], 
                    Hue[0.07167338075811815, 1, 1], 
                    Hue[0.12937282649114865`, 1, 1], 
                    Hue[0.18780666443835414`, 1, 1], 
                    Hue[0.24696913203339063`, 1, 1], 
                    Hue[0.30685523730739184`, 1, 1], 
                    Hue[0.36745957905062887`, 1, 1], 
                    Hue[0.42877769280745087`, 1, 1], 
                    Hue[0.4908037925029819, 1, 1], 
                    Hue[0.5535350384234963, 1, 1], 
                    Hue[0.6169653916379854, 1, 1], 
                    Hue[0.6810907257805103, 1, 1], 
                    Hue[0.7459063147912967, 1, 1], 
                    Hue[0.8114104826374526, 1, 1], 
                    Hue[0.8775950861296535, 1, 1], 
                    Hue[0.9444588876589283, 1, 1], 
                    Hue[0.0119970511399642, 1, 1], 
                    Hue[0.08020606705135687, 1, 1], 
                    Hue[0.14908284555554469`, 1, 1], 
                    Hue[0.2186219206976702, 1, 1], 
                    Hue[0.28882197240286506`, 1, 1], 
                    Hue[0.35967627266925456`, 1, 1], 
                    Hue[0.43118455611638384`, 1, 1], 
                    Hue[0.5033433290086244, 1, 1], 
                    Hue[0.5761460418402145, 1, 1], 
                    Hue[0.6495947288059509, 1, 1], 
                    Hue[0.7236843398166319, 1, 1], 
                    Hue[0.7984095434383008, 1, 1], 
                    Hue[0.873769870187243, 1, 1], 
                    Hue[0.9497602611869326, 1, 1], 
                    Hue[0.02638015960819491, 1, 1], 
                    Hue[0.10362426395156402`, 1, 1], 
                    Hue[0.18149402581251858`, 1, 1], 
                    Hue[0.25998363569095884`, 1, 1], 
                    Hue[0.33908888485147026`, 1, 1], 
                    Hue[0.41881004827500784`, 1, 1], 
                    Hue[0.4991438872643397, 1, 1], 
                    Hue[0.5800901965951237, 1, 1], 
                    Hue[0.6616434324791175, 1, 1], 
                    Hue[0.7437997538299062, 1, 1], 
                    Hue[0.8265609418880175, 1, 1], 
                    Hue[0.9099231632063347, 1, 1], 
                    Hue[0.9938830944227226, 1, 1], 
                    Hue[0.07843854256843062, 1, 1], 
                    Hue[0.16358722556755453`, 1, 1], 
                    Hue[0.24932940137544837`, 1, 1], 
                    Hue[0.33566483509107636`, 1, 1], 
                    Hue[0.4225903879562467, 1, 1], 
                    Hue[0.5100948438311672, 1, 1], 
                    Hue[0.598181316104291, 1, 1], 
                    Hue[0.6868528306413736, 1, 1], 
                    Hue[0.7761040425171196, 1, 1], 
                    Hue[0.8659330990069755, 1, 1], 
                    Hue[0.9563368183458169, 1, 1], 
                    Hue[0.4393534391365692, 1, 1], 
                    Hue[0.3201372697153097, 1, 1], 
                    Hue[0.3201372697153097, 1, 1], 
                    Hue[0.2232096319850034, 1, 1], 
                    Hue[0.2232096319850034, 1, 1], 
                    Hue[0.10358516356125722`, 1, 1], 
                    Hue[0.10358516356125722`, 1, 1], 
                    Hue[0.005372599255212061, 1, 1], 
                    Hue[0.005372599255212061, 1, 1], 
                    Hue[0.9035686603544933, 1, 1], 
                    Hue[0.9035686603544933, 1, 1], 
                    Hue[0.7988842591287169, 1, 1], 
                    Hue[0.7988842591287169, 1, 1], 
                    Hue[0.6980701443109908, 1, 1], 
                    Hue[0.6980701443109908, 1, 1], 
                    Hue[0.5965944068167406, 1, 1], 
                    Hue[0.5965944068167406, 1, 1], 
                    Hue[0.4928555528039237, 1, 1], 
                    Hue[0.4928555528039237, 1, 1], 
                    Hue[0.39265585709506984`, 1, 1], 
                    Hue[0.39265585709506984`, 1, 1], 
                    Hue[0.29253193566333907`, 1, 1], 
                    Hue[0.29253193566333907`, 1, 1], 
                    Hue[0.1882309916688273, 1, 1], 
                    Hue[0.1882309916688273, 1, 1], 
                    Hue[0.08806702992125463, 1, 1], 
                    Hue[0.08806702992125463, 1, 1], 
                    Hue[0.9854789907817174, 1, 1], 
                    Hue[0.9854789907817174, 1, 1], 
                    Hue[0.8845172055478656, 1, 1], 
                    Hue[0.8845172055478656, 1, 1], 
                    Hue[0.7827332249025398, 1, 1], 
                    Hue[0.7827332249025398, 1, 1], 
                    Hue[0.6824754438075233, 1, 1], 
                    Hue[0.6824754438075233, 1, 1], 
                    Hue[0.5820498737159026, 1, 1], 
                    Hue[0.5820498737159026, 1, 1], 
                    Hue[0.4812617808739333, 1, 1], 
                    Hue[0.4812617808739333, 1, 1], 
                    Hue[0.38155270567862865`, 1, 1], 
                    Hue[0.38155270567862865`, 1, 1], 
                    Hue[0.2821086601880322, 1, 1], 
                    Hue[0.2821086601880322, 1, 1], 
                    Hue[0.1819929337444189, 1, 1], 
                    Hue[0.1819929337444189, 1, 1], 
                    Hue[0.08325129391839824, 1, 1], 
                    Hue[0.08325129391839824, 1, 1], 
                    Hue[0.9841663633956332, 1, 1], 
                    Hue[0.9841663633956332, 1, 1], 
                    Hue[0.8857141077554178, 1, 1], 
                    Hue[0.8857141077554178, 1, 1], 
                    Hue[0.7871772298162246, 1, 1], 
                    Hue[0.7871772298162246, 1, 1], 
                    Hue[0.6891118122079494, 1, 1], 
                    Hue[0.6891118122079494, 1, 1], 
                    Hue[0.5912062960826425, 1, 1], 
                    Hue[0.5912062960826425, 1, 1], 
                    Hue[0.4935392024672945, 1, 1], 
                    Hue[0.4935392024672945, 1, 1], 
                    Hue[0.3961935772395639, 1, 1], 
                    Hue[0.3961935772395639, 1, 1], 
                    Hue[0.2989048819635487, 1, 1], 
                    Hue[0.2989048819635487, 1, 1], 
                    Hue[0.20194144472414555`, 1, 1], 
                    Hue[0.20194144472414555`, 1, 1], 
                    Hue[0.10517975842540783`, 1, 1], 
                    Hue[0.10517975842540783`, 1, 1], 
                    Hue[0.008676147230962257, 1, 1], 
                    Hue[0.008676147230962257, 1, 1], 
                    Hue[0.9123650895683947, 1, 1], 
                    Hue[0.9123650895683947, 1, 1], 
                    Hue[0.8163537610262777, 1, 1], 
                    Hue[0.8163537610262777, 1, 1], 
                    Hue[0.7205515282763565, 1, 1], 
                    Hue[0.7205515282763565, 1, 1], 
                    Hue[0.6249695450063499, 1, 1], 
                    Hue[0.6249695450063499, 1, 1], 
                    Hue[0.5296108292359909, 1, 1], 
                    Hue[0.5296108292359909, 1, 1], 
                    Hue[0.43443600295767226`, 1, 1], 
                    Hue[0.43443600295767226`, 1, 1], 
                    Hue[0.3394953919330468, 1, 1], 
                    Hue[0.3394953919330468, 1, 1], 
                    Hue[0.24476243828602443`, 1, 1], 
                    Hue[0.24476243828602443`, 1, 1], 
                    Hue[0.15023452379705038`, 1, 1], 
                    Hue[0.15023452379705038`, 1, 1], 
                    Hue[0.05591726332051126, 1, 1], 
                    Hue[0.05591726332051126, 1, 1], 
                    Hue[0.9618102877749671, 1, 1], 
                    Hue[0.9618102877749671, 1, 1], 
                    Hue[0.8678849255300521, 1, 1], 
                    Hue[0.8678849255300521, 1, 1], 
                    Hue[0.7741471790260839, 1, 1], 
                    Hue[0.7741471790260839, 1, 1], 
                    Hue[0.6806528796568211, 1, 1], 
                    Hue[0.6806528796568211, 1, 1], 
                    Hue[0.5873248986019096, 1, 1], 
                    Hue[0.5873248986019096, 1, 1], 
                    Hue[0.49418451596794727`, 1, 1], 
                    Hue[0.49418451596794727`, 1, 1], 
                    Hue[0.4012513718543245, 1, 1], 
                    Hue[0.4012513718543245, 1, 1], 
                    Hue[0.30847550574319005`, 1, 1], 
                    Hue[0.30847550574319005`, 1, 1], 
                    Hue[0.21589285716824616`, 1, 1], 
                    Hue[0.21589285716824616`, 1, 1], 
                    Hue[0.12349670073992161`, 1, 1], 
                    Hue[0.12349670073992161`, 1, 1], 
                    Hue[0.031259713016860584`, 1, 1], 
                    Hue[0.031259713016860584`, 1, 1], 
                    Hue[0.9392091910858187, 1, 1], 
                    Hue[0.9392091910858187, 1, 1], 
                    Hue[0.8473239080259873, 1, 1], 
                    Hue[0.8473239080259873, 1, 1], 
                    Hue[0.7556065340967153, 1, 1], 
                    Hue[0.7556065340967153, 1, 1], 
                    Hue[0.6640608936363054, 1, 1], 
                    Hue[0.6640608936363054, 1, 1], 
                    Hue[0.5726668749139376, 1, 1], 
                    Hue[0.5726668749139376, 1, 1], 
                    Hue[0.48143870204130346`, 1, 1], 
                    Hue[0.48143870204130346`, 1, 1], 
                    Hue[0.39036514813311196`, 1, 1], 
                    Hue[0.39036514813311196`, 1, 1], 
                    Hue[0.2994379002421825, 1, 1], 
                    Hue[0.2994379002421825, 1, 1], 
                    Hue[0.20865980656813093`, 1, 1], 
                    Hue[0.20865980656813093`, 1, 1], 
                    Hue[0.11802481179332475`, 1, 1], 
                    Hue[0.11802481179332475`, 1, 1], 
                    Hue[0.02753031297037304, 1, 1], 
                    Hue[0.02753031297037304, 1, 1], 
                    Hue[0.937172298998918, 1, 1], 
                    Hue[0.937172298998918, 1, 1], 
                    Hue[0.8469470877910599, 1, 1], 
                    Hue[0.8469470877910599, 1, 1], 
                    Hue[0.7568529898751493, 1, 1], 
                    Hue[0.7568529898751493, 1, 1], 
                    Hue[0.6668838918648523, 1, 1], 
                    Hue[0.6668838918648523, 1, 1], 
                    Hue[0.5770355187840224, 1, 1], 
                    Hue[0.5770355187840224, 1, 1], 
                    Hue[0.4873067584282177, 1, 1], 
                    Hue[0.4873067584282177, 1, 1], 
                    Hue[0.3976885574940337, 1, 1], 
                    Hue[0.3976885574940337, 1, 1], 
                    Hue[0.3081836156328967, 1, 1], 
                    Hue[0.3081836156328967, 1, 1], 
                    Hue[0.21877826866051225`, 1, 1], 
                    Hue[0.21877826866051225`, 1, 1], 
                    Hue[0.12947777818206113`, 1, 1], 
                    Hue[0.12947777818206113`, 1, 1], 
                    Hue[0.040272061779198676`, 1, 1], 
                    Hue[0.040272061779198676`, 1, 1], 
                    Hue[0.9511561661238163, 1, 1], 
                    Hue[0.9511561661238163, 1, 1], 
                    Hue[0.8621287005273579, 1, 1], 
                    Hue[0.8621287005273579, 1, 1], 
                    Hue[0.7731830030987756, 1, 1], 
                    Hue[0.7731830030987756, 1, 1], 
                    Hue[0.6843154147654915, 1, 1], 
                    Hue[0.6843154147654915, 1, 1], 
                    Hue[0.5955212624534687, 1, 1], 
                    Hue[0.5955212624534687, 1, 1], 
                    Hue[0.5067941698730325, 1, 1], 
                    Hue[0.5067941698730325, 1, 1], 
                    Hue[0.41813217963473104`, 1, 1], 
                    Hue[0.41813217963473104`, 1, 1], 
                    Hue[0.3295280433802835, 1, 1], 
                    Hue[0.3295280433802835, 1, 1], 
                    Hue[0.24097871888163253`, 1, 1], 
                    Hue[0.24097871888163253`, 1, 1], 
                    Hue[0.1524804620347105, 1, 1], 
                    Hue[0.1524804620347105, 1, 1], 
                    Hue[0.06402804928679805, 1, 1], 
                    Hue[0.06402804928679805, 1, 1], 
                    Hue[0.9756183067398464, 1, 1], 
                    Hue[0.9756183067398464, 1, 1], 
                    Hue[0.8872471615914255, 1, 1], 
                    Hue[0.8872471615914255, 1, 1], 
                    Hue[0.7989117139209876, 1, 1], 
                    Hue[0.7989117139209876, 1, 1], 
                    Hue[0.7106100480325939, 1, 1], 
                    Hue[0.7106100480325939, 1, 1], 
                    Hue[0.6223405196721529, 1, 1], 
                    Hue[0.6223405196721529, 1, 1], 
                    Hue[0.5341007127148718, 1, 1], 
                    Hue[0.5341007127148718, 1, 1], 
                    Hue[0.44589148097804515`, 1, 1], 
                    Hue[0.44589148097804515`, 1, 1], 
                    Hue[0.35771438988960935`, 1, 1], 
                    Hue[0.35771438988960935`, 1, 1], 
                    Hue[0.2695704180261129, 1, 1], 
                    Hue[0.2695704180261129, 1, 1], 
                    Hue[0.18146586950745308`, 1, 1], 
                    Hue[0.18146586950745308`, 1, 1], 
                    Hue[0.09340746986017222, 1, 1], 
                    Hue[0.09340746986017222, 1, 1], 
                    Hue[0.005404085323135338, 1, 1], 
                    Hue[0.005404085323135338, 1, 1], 
                    Hue[0.9174681454422887, 1, 1], 
                    Hue[0.9174681454422887, 1, 1], 
                    Hue[0.8296137987629909, 1, 1], 
                    Hue[0.8296137987629909, 1, 1], 
                    Hue[0.7418598218721183, 1, 1], 
                    Hue[0.7418598218721183, 1, 1], 
                    Hue[0.6542291760888137, 1, 1], 
                    Hue[0.6542291760888137, 1, 1], 
                    Hue[0.5667513430325848, 1, 1], 
                    Hue[0.5667513430325848, 1, 1], 
                    Hue[0.47946412748234774`, 1, 1], 
                    Hue[0.47946412748234774`, 1, 1], 
                    Hue[0.39241165346008955`, 1, 1], 
                    Hue[0.39241165346008955`, 1, 1], 
                    Hue[0.3056448023252869, 1, 1], 
                    Hue[0.3056448023252869, 1, 1], 
                    Hue[0.21922055749572877`, 1, 1], 
                    Hue[0.21922055749572877`, 1, 1], 
                    Hue[0.1332022361576938, 1, 1], 
                    Hue[0.1332022361576938, 1, 1], 
                    Hue[0.047666850681859045`, 1, 1], 
                    Hue[0.047666850681859045`, 1, 1], 
                    Hue[0.9627024797532431, 1, 1], 
                    Hue[0.9627024797532431, 1, 1], 
                    Hue[0.8784143218371727, 1, 1], 
                    Hue[0.8784143218371727, 1, 1], 
                    Hue[0.7949263299068562, 1, 1], 
                    Hue[0.7949263299068562, 1, 1], 
                    Hue[0.7123730704290766, 1, 1], 
                    Hue[0.7123730704290766, 1, 1], 
                    Hue[0.6308993085613088, 1, 1], 
                    Hue[0.6308993085613088, 1, 1], 
                    Hue[0.5506492528355513, 1, 1], 
                    Hue[0.5506492528355513, 1, 1], 
                    Hue[0.4717723067273212, 1, 1], 
                    Hue[0.4717723067273212, 1, 1], 
                    Hue[0.39442199474621936`, 1, 1], 
                    Hue[0.39442199474621936`, 1, 1], 
                    Hue[0.3187645584683982, 1, 1], 
                    Hue[0.3187645584683982, 1, 1], 
                    Hue[0.2449824694851805, 1, 1], 
                    Hue[0.2449824694851805, 1, 1], 
                    Hue[0.17327582930586308`, 1, 1], 
                    Hue[0.17327582930586308`, 1, 1], 
                    Hue[0.10385569030144716`, 1, 1], 
                    Hue[0.10385569030144716`, 1, 1], 
                    Hue[0.03693914308290094, 1, 1], 
                    Hue[0.03693914308290094, 1, 1], 
                    Hue[0.9727328323643722, 1, 1], 
                    Hue[0.9727328323643722, 1, 1], 
                    Hue[0.9114250873976556, 1, 1], 
                    Hue[0.9114250873976556, 1, 1], 
                    Hue[0.8531770723884569, 1, 1], 
                    Hue[0.8531770723884569, 1, 1], 
                    Hue[0.7981179230268365, 1, 1], 
                    Hue[0.7981179230268365, 1, 1], 
                    Hue[0.7463434738319357, 1, 1], 
                    Hue[0.7463434738319357, 1, 1], 
                    Hue[0.6979169230722659, 1, 1], 
                    Hue[0.6979169230722659, 1, 1], 
                    Hue[0.652870401171725, 1, 1], 
                    Hue[0.652870401171725, 1, 1], 
                    Hue[0.6112085161386327, 1, 1], 
                    Hue[0.6112085161386327, 1, 1], 
                    Hue[0.5729111580662459, 1, 1], 
                    Hue[0.5729111580662459, 1, 1], 
                    Hue[0.5379361605368185, 1, 1], 
                    Hue[0.5379361605368185, 1, 1], 
                    Hue[0.5062229764889836, 1, 1], 
                    Hue[0.5062229764889836, 1, 1], 
                    Hue[0.4776979103140166, 1, 1], 
                    Hue[0.4776979103140166, 1, 1], 
                    Hue[0.45227423713150755`, 1, 1], 
                    Hue[0.45227423713150755`, 1, 1], 
                    Hue[0.42985665197459183`, 1, 1], 
                    Hue[0.42985665197459183`, 1, 1], 
                    Hue[0.41034533929896294`, 1, 1], 
                    Hue[0.41034533929896294`, 1, 1], 
                    Hue[0.39363480451596733`, 1, 1], 
                    Hue[0.39363480451596733`, 1, 1], 
                    Hue[0.3796206968339214, 1, 1], 
                    Hue[0.3796206968339214, 1, 1], 
                    Hue[0.36819635676874646`, 1, 1], 
                    Hue[0.36819635676874646`, 1, 1], 
                    Hue[0.3592588810689745, 1, 1], 
                    Hue[0.3592588810689745, 1, 1], 
                    Hue[0.35270652242754846`, 1, 1], 
                    Hue[0.35270652242754846`, 1, 1], 
                    Hue[0.348441715653491, 1, 1], 
                    Hue[0.348441715653491, 1, 1], 
                    Hue[0.34637025854678805`, 1, 1], 
                    Hue[0.34637025854678805`, 1, 1], 
                    Hue[0.34640317257292524`, 1, 1], 
                    Hue[0.34640317257292524`, 1, 1], 
                    Hue[0.3484549273788225, 1, 1], 
                    Hue[0.3484549273788225, 1, 1], 
                    Hue[0.35244536014669253`, 1, 1], 
                    Hue[0.35244536014669253`, 1, 1], 
                    Hue[0.35829899021649714`, 1, 1], 
                    Hue[0.35829899021649714`, 1, 1], 
                    Hue[0.3659444120977315, 1, 1], 
                    Hue[0.3659444120977315, 1, 1], 
                    Hue[0.37531470466298017`, 1, 1], 
                    Hue[0.37531470466298017`, 1, 1], 
                    Hue[0.38634778739783043`, 1, 1], 
                    Hue[0.38634778739783043`, 1, 1], 
                    Hue[0.3989849084100206, 1, 1], 
                    Hue[0.3989849084100206, 1, 1], 
                    Hue[0.4131714890798039, 1, 1], 
                    Hue[0.4131714890798039, 1, 1], 
                    Hue[0.4288562177677756, 1, 1], 
                    Hue[0.4288562177677756, 1, 1], 
                    Hue[0.4459917475621054, 1, 1], 
                    Hue[0.4459917475621054, 1, 1], 
                    Hue[0.46453343042731615`, 1, 1], 
                    Hue[0.46453343042731615`, 1, 1], 
                    Hue[0.484439541654788, 1, 1], 
                    Hue[0.484439541654788, 1, 1], 
                    Hue[0.5056713520991165, 1, 1], 
                    Hue[0.5056713520991165, 1, 1], 
                    Hue[0.5281925529249144, 1, 1], 
                    Hue[0.5281925529249144, 1, 1], 
                    Hue[0.5519691183846559, 1, 1], 
                    Hue[0.5519691183846559, 1, 1], 
                    Hue[0.5769693278688665, 1, 1], 
                    Hue[0.5769693278688665, 1, 1], 
                    Hue[0.603163245870465, 1, 1], 
                    Hue[0.603163245870465, 1, 1], 
                    Hue[0.6305229668447081, 1, 1], 
                    Hue[0.6305229668447081, 1, 1], 
                    Hue[0.6590229110973536, 1, 1], 
                    Hue[0.6590229110973536, 1, 1], 
                    Hue[0.6886380618054015, 1, 1], 
                    Hue[0.6886380618054015, 1, 1], 
                    Hue[0.7193449813186077, 1, 1], 
                    Hue[0.7193449813186077, 1, 1], 
                    Hue[0.7511224993169587, 1, 1], 
                    Hue[0.7511224993169587, 1, 1], 
                    Hue[0.7839493701684175, 1, 1], 
                    Hue[0.7839493701684175, 1, 1], 
                    Hue[0.817806735017353, 1, 1], 
                    Hue[0.817806735017353, 1, 1], 
                    Hue[0.8526758485656166, 1, 1], 
                    Hue[0.8526758485656166, 1, 1], 
                    Hue[0.8885399846257238, 1, 1], 
                    Hue[0.8885399846257238, 1, 1], 
                    Hue[0.9253823506081306, 1, 1], 
                    Hue[0.9253823506081306, 1, 1], 
                    Hue[0.9631867200833969, 1, 1], 
                    Hue[0.9631867200833969, 1, 1], 
                    Hue[0.0019395945408895489`, 1, 1], 
                    Hue[0.0019395945408895489`, 1, 1], 
                    Hue[0.04162675481913263, 1, 1], 
                    Hue[0.04162675481913263, 1, 1], 
                    Hue[0.08223416167340272, 1, 1], 
                    Hue[0.08223416167340272, 1, 1], 
                    Hue[0.12375029778114359`, 1, 1], 
                    Hue[0.12375029778114359`, 1, 1], 
                    Hue[0.16616162602928783`, 1, 1], 
                    Hue[0.16616162602928783`, 1, 1], 
                    Hue[0.20945806400140338`, 1, 1], 
                    Hue[0.20945806400140338`, 1, 1], 
                    Hue[0.2536276524015086, 1, 1], 
                    Hue[0.2536276524015086, 1, 1], 
                    Hue[0.2986598507746231, 1, 1], 
                    Hue[0.2986598507746231, 1, 1], 
                    Hue[0.3445451940568987, 1, 1], 
                    Hue[0.3445451940568987, 1, 1], 
                    Hue[0.39127272249841616`, 1, 1], 
                    Hue[0.39127272249841616`, 1, 1], 
                    Hue[0.4388341517654788, 1, 1], 
                    Hue[0.4388341517654788, 1, 1], 
                    Hue[0.48722035714046374`, 1, 1], 
                    Hue[0.48722035714046374`, 1, 1], 
                    Hue[0.5364224284486252, 1, 1], 
                    Hue[0.5364224284486252, 1, 1], 
                    Hue[0.5864334679766994, 1, 1], 
                    Hue[0.5864334679766994, 1, 1], 
                    Hue[0.6372440491692886, 1, 1], 
                    Hue[0.6372440491692886, 1, 1], 
                    Hue[0.6888480265350307, 1, 1], 
                    Hue[0.6888480265350307, 1, 1], 
                    Hue[0.7412365911076693, 1, 1], 
                    Hue[0.7412365911076693, 1, 1], 
                    Hue[0.7944032577634785, 1, 1], 
                    Hue[0.7944032577634785, 1, 1], 
                    Hue[0.8483407608749416, 1, 1], 
                    Hue[0.8483407608749416, 1, 1], 
                    Hue[0.903042428880277, 1, 1], 
                    Hue[0.903042428880277, 1, 1], 
                    Hue[0.9585025639644492, 1, 1], 
                    Hue[0.9585025639644492, 1, 1], 
                    Hue[0.014714511057447872`, 1, 1], 
                    Hue[0.014714511057447872`, 1, 1], 
                    Hue[0.07167338075811815, 1, 1], 
                    Hue[0.07167338075811815, 1, 1], 
                    Hue[0.12937282649114865`, 1, 1], 
                    Hue[0.12937282649114865`, 1, 1], 
                    Hue[0.18780666443835414`, 1, 1], 
                    Hue[0.18780666443835414`, 1, 1], 
                    Hue[0.24696913203339063`, 1, 1], 
                    Hue[0.24696913203339063`, 1, 1], 
                    Hue[0.30685523730739184`, 1, 1], 
                    Hue[0.30685523730739184`, 1, 1], 
                    Hue[0.36745957905062887`, 1, 1], 
                    Hue[0.36745957905062887`, 1, 1], 
                    Hue[0.42877769280745087`, 1, 1], 
                    Hue[0.42877769280745087`, 1, 1], 
                    Hue[0.4908037925029819, 1, 1], 
                    Hue[0.4908037925029819, 1, 1], 
                    Hue[0.5535350384234963, 1, 1], 
                    Hue[0.5535350384234963, 1, 1], 
                    Hue[0.6169653916379854, 1, 1], 
                    Hue[0.6169653916379854, 1, 1], 
                    Hue[0.6810907257805103, 1, 1], 
                    Hue[0.6810907257805103, 1, 1], 
                    Hue[0.7459063147912967, 1, 1], 
                    Hue[0.7459063147912967, 1, 1], 
                    Hue[0.8114104826374526, 1, 1], 
                    Hue[0.8114104826374526, 1, 1], 
                    Hue[0.8775950861296535, 1, 1], 
                    Hue[0.8775950861296535, 1, 1], 
                    Hue[0.9444588876589283, 1, 1], 
                    Hue[0.9444588876589283, 1, 1], 
                    Hue[0.0119970511399642, 1, 1], 
                    Hue[0.0119970511399642, 1, 1], 
                    Hue[0.08020606705135687, 1, 1], 
                    Hue[0.08020606705135687, 1, 1], 
                    Hue[0.14908284555554469`, 1, 1], 
                    Hue[0.14908284555554469`, 1, 1], 
                    Hue[0.2186219206976702, 1, 1], 
                    Hue[0.2186219206976702, 1, 1], 
                    Hue[0.28882197240286506`, 1, 1], 
                    Hue[0.28882197240286506`, 1, 1], 
                    Hue[0.35967627266925456`, 1, 1], 
                    Hue[0.35967627266925456`, 1, 1], 
                    Hue[0.43118455611638384`, 1, 1], 
                    Hue[0.43118455611638384`, 1, 1], 
                    Hue[0.5033433290086244, 1, 1], 
                    Hue[0.5033433290086244, 1, 1], 
                    Hue[0.5761460418402145, 1, 1], 
                    Hue[0.5761460418402145, 1, 1], 
                    Hue[0.6495947288059509, 1, 1], 
                    Hue[0.6495947288059509, 1, 1], 
                    Hue[0.7236843398166319, 1, 1], 
                    Hue[0.7236843398166319, 1, 1], 
                    Hue[0.7984095434383008, 1, 1], 
                    Hue[0.7984095434383008, 1, 1], 
                    Hue[0.873769870187243, 1, 1], 
                    Hue[0.873769870187243, 1, 1], 
                    Hue[0.9497602611869326, 1, 1], 
                    Hue[0.9497602611869326, 1, 1], 
                    Hue[0.02638015960819491, 1, 1], 
                    Hue[0.02638015960819491, 1, 1], 
                    Hue[0.10362426395156402`, 1, 1], 
                    Hue[0.10362426395156402`, 1, 1], 
                    Hue[0.18149402581251858`, 1, 1], 
                    Hue[0.18149402581251858`, 1, 1], 
                    Hue[0.25998363569095884`, 1, 1], 
                    Hue[0.25998363569095884`, 1, 1], 
                    Hue[0.33908888485147026`, 1, 1], 
                    Hue[0.33908888485147026`, 1, 1], 
                    Hue[0.41881004827500784`, 1, 1], 
                    Hue[0.41881004827500784`, 1, 1], 
                    Hue[0.4991438872643397, 1, 1], 
                    Hue[0.4991438872643397, 1, 1], 
                    Hue[0.5800901965951237, 1, 1], 
                    Hue[0.5800901965951237, 1, 1], 
                    Hue[0.6616434324791175, 1, 1], 
                    Hue[0.6616434324791175, 1, 1], 
                    Hue[0.7437997538299062, 1, 1], 
                    Hue[0.7437997538299062, 1, 1], 
                    Hue[0.8265609418880175, 1, 1], 
                    Hue[0.8265609418880175, 1, 1], 
                    Hue[0.9099231632063347, 1, 1], 
                    Hue[0.9099231632063347, 1, 1], 
                    Hue[0.9938830944227226, 1, 1], 
                    Hue[0.9938830944227226, 1, 1], 
                    Hue[0.07843854256843062, 1, 1], 
                    Hue[0.07843854256843062, 1, 1], 
                    Hue[0.16358722556755453`, 1, 1], 
                    Hue[0.16358722556755453`, 1, 1], 
                    Hue[0.24932940137544837`, 1, 1], 
                    Hue[0.24932940137544837`, 1, 1], 
                    Hue[0.33566483509107636`, 1, 1], 
                    Hue[0.33566483509107636`, 1, 1], 
                    Hue[0.4225903879562467, 1, 1], 
                    Hue[0.4225903879562467, 1, 1], 
                    Hue[0.5100948438311672, 1, 1], 
                    Hue[0.5100948438311672, 1, 1], 
                    Hue[0.598181316104291, 1, 1], 
                    Hue[0.598181316104291, 1, 1], 
                    Hue[0.6868528306413736, 1, 1], 
                    Hue[0.6868528306413736, 1, 1], 
                    Hue[0.7761040425171196, 1, 1], 
                    Hue[0.7761040425171196, 1, 1], 
                    Hue[0.8659330990069755, 1, 1], 
                    Hue[0.8659330990069755, 1, 1], 
                    Hue[0.9563368183458169, 1, 1]}],
                   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
                   Axes->{True, None},
                   AxesOrigin->{0, 0},
                   
                   Epilog->LineBox[{{1, 5.6522849888109*^-6}, {
                    2, 7.493856079868243*^-6}, {3, 9.361903170769543*^-6}, {
                    4, 0.000011108968790438951`}, {
                    5, 0.000013292523855721003`}, {
                    6, 0.00001567890748870991}, {
                    7, 0.000019300826157275456`}, {
                    8, 0.00002243631785471867}, {9, 0.00002728836735193481}, {
                    10, 0.000031547215782990524`}, {
                    11, 0.000037647963946252064`}, {
                    12, 0.00004469129785170058}, {
                    13, 0.0000523083795771476}, {14, 0.000062522934026128}, {
                    15, 0.00007379014496043094}, {
                    16, 0.00008629399260704188}, {
                    17, 0.00010184073462433887`}, {
                    18, 0.0001194013252743524}, {
                    19, 0.00013937242829593257`}, {
                    20, 0.0001636449673548106}, {
                    21, 0.00019047444025661412`}, {
                    22, 0.00022196161200206042`}, {
                    23, 0.0002585859321539324}, {
                    24, 0.00030043458691523584`}, {
                    25, 0.0003482834115170974}, {26, 0.0004041598604892873}, {
                    27, 0.00046719032058893087`}, {
                    28, 0.0005404554662575303}, {29, 0.0006225420668444395}, {
                    30, 0.0007181610827485022}, {31, 0.0008255281555854794}, {
                    32, 0.0009485977272661631}, {
                    33, 0.0010883170572777196`}, {
                    34, 0.0012469241820886098`}, {
                    35, 0.0014263219788284173`}, {
                    36, 0.0016292419422861593`}, {
                    37, 0.0018589202825967464`}, {38, 0.002117233077416884}, {
                    39, 0.0024086141520167107`}, {40, 0.002736330463350093}, {
                    41, 0.003103541487495681}, {42, 0.0035143825427192766`}, {
                    43, 0.003974838956634007}, {44, 0.004488583025357468}, {
                    45, 0.005060898957280843}, {46, 0.0056987912461398255`}, {
                    47, 0.006407424419902048}, {48, 0.007192658635084565}, {
                    49, 0.008061732910749672}, {50, 0.009022297740079803}, {
                    51, 0.010082722219571393`}, {52, 0.011248908531296002`}, {
                    53, 0.012530647562538397`}, {54, 0.013937308753915805`}, {
                    55, 0.015476623303451844`}, {56, 0.017159058766566718`}, {
                    57, 0.01899510851148176}, {58, 0.020991777418775462`}, {
                    59, 0.02316200866272084}, {60, 0.0255142343712261}, {
                    61, 0.028058049671868524`}, {62, 0.030804477420809325`}, {
                    63, 0.03376307952461599}, {64, 0.0369433250245183}, {
                    65, 0.040353950385624185`}, {66, 0.04400369360069891}, {
                    67, 0.04790044658369485}, {68, 0.052051667868831884`}, {
                    69, 0.05646288077286757}, {70, 0.061138643794485754`}, {
                    71, 0.06608402922126116}, {72, 0.07129932909191282}, {
                    73, 0.07678617178699923}, {74, 0.08254486999138924}, {
                    75, 0.08857066836529483}, {76, 0.09486070727329636}, {
                    77, 0.10140691704013148`}, {78, 0.10820040498121343`}, {
                    79, 0.11522989692240476`}, {80, 0.12247925564214131`}, {
                    81, 0.12993462991311364`}, {82, 0.13757559653046492`}, {
                    83, 0.14538169218847222`}, {84, 0.1533280355145805}, {
                    85, 0.16138964674434755`}, {86, 0.16953654983092106`}, {
                    87, 0.1777384322542062}, {88, 0.18596333219333522`}, {
                    89, 0.19417419431505537`}, {90, 0.2023383864716978}, {
                    91, 0.21041745276281484`}, {92, 0.218372130610057}, {
                    93, 0.22616571392729926`}, {94, 0.23375931093027244`}, {
                    95, 0.2411138052960808}, {96, 0.24819276173761884`}, {
                    97, 0.25496052072831815`}, {98, 0.26138279154564387`}, {
                    99, 0.26742979989231797`}, {100, 0.2730713432144373}, {
                    101, 0.27828199547269045`}, {102, 0.28303668560906775`}, {
                    103, 0.2873190530457891}, {104, 0.29111461944185024`}, {
                    105, 0.29441863672680496`}, {106, 0.29723049047426026`}, {
                    107, 0.2995542353596208}, {108, 0.3013953345112595}, {
                    109, 0.3027616203282248}, {110, 0.3036651375773494}, {
                    111, 0.304128056563314}, {112, 0.304185768598205}, {
                    113, 0.30388262309975167`}, {114, 0.3032668561299882}, {
                    115, 0.3023846046142477}, {116, 0.3012724946513947}, {
                    117, 0.2999622330218534}, {118, 0.29848152589850435`}, {
                    119, 0.29687906582650525`}, {120, 0.2952239545087466}, {
                    121, 0.29361060244024717`}, {122, 0.2921542584785492}, {
                    123, 0.29096647352276866`}, {124, 0.2901484313952193}, {
                    125, 0.2897730126065047}, {126, 0.28988429551438105`}, {
                    127, 0.2904969316890593}, {128, 0.29160211860018714`}, {
                    129, 0.2931792766969426}, {130, 0.2951972492883752}, {
                    131, 0.2976212415637846}, {132, 0.30041704859499585`}, {
                    133, 0.30354776772839553`}, {134, 0.30697990660895746`}, {
                    135, 0.31067657657821374`}, {136, 0.31460387555610997`}, {
                    137, 0.3187259141998557}, {138, 0.3230072791715519}, {
                    139, 0.32741599378577807`}, {140, 0.33191750810603293`}, {
                    141, 0.3364799219044072}, {142, 0.3410721734176782}, {
                    143, 0.3456634644312373}, {144, 0.35022591697757116`}, {
                    145, 0.3547304534466183}, {146, 0.3591516322835139}, {
                    147, 0.3634645792148149}, {148, 0.36764472653667896`}, {
                    149, 0.37167114636571835`}, {150, 0.37552266147302954`}, {
                    151, 0.37917826094575385`}, {152, 0.382620340373909}, {
                    153, 0.38583081259268975`}, {154, 0.38879304717095553`}, {
                    155, 0.3914923418384644}, {156, 0.39391496620426814`}, {
                    157, 0.3960491604294993}, {158, 0.3978817601129005}, {
                    159, 0.3994029102098073}, {160, 0.40060382678811673`}, {
                    161, 0.4014763827448637}, {162, 0.40201320650983136`}, {
                    163, 0.40220877817913}, {164, 0.4020596379079551}, {
                    165, 0.40156053180909207`}, {166, 0.40071130856384024`}, {
                    167, 0.3995097978874194}, {168, 0.3979571867374357}, {
                    169, 0.3960532509583461}, {170, 0.3938010527158964}, {
                    171, 0.3912044641117032}, {172, 0.3882674271615195}, {
                    173, 0.3849960108732703}, {174, 0.38139616514859}, {
                    175, 0.377476672231851}, {176, 0.3732452805785937}, {
                    177, 0.3687106584673623}, {178, 0.36388510816809233`}, {
                    179, 0.35877722556847225`}, {180, 0.35339987022905417`}, {
                    181, 0.3477671550716122}, {182, 0.3418903016180166}, {
                    183, 0.33578618402163607`}, {184, 0.3294663815661966}, {
                    185, 0.32294686438337383`}, {186, 0.3162430597784358}, {
                    187, 0.30936969254715996`}, {188, 0.3023435982829913}, {
                    189, 0.29517900906165695`}, {190, 0.2878947820982167}, {
                    191, 0.2805053064998337}, {192, 0.27302872184533394`}, {
                    193, 0.2654797263227138}, {194, 0.25787590264687776`}, {
                    195, 0.2502318165479045}, {196, 0.24256408092125306`}, {
                    197, 0.2348861366093799}, {198, 0.2272146654583238}, {
                    199, 0.21956522476328721`}, {200, 0.21195029496825143`}, {
                    201, 0.20438481288933247`}, {202, 0.19688170729809698`}, {
                    203, 0.18945310873947227`}, {204, 0.1821102691001891}, {
                    205, 0.17486450657556654`}, {206, 0.16772748488167488`}, {
                    207, 0.1607093723086569}, {208, 0.15381766399013522`}, {
                    209, 0.14706248767861638`}, {210, 0.14045181504777782`}, {
                    211, 0.1339920959179641}, {212, 0.12768797002110663`}, {
                    213, 0.12154877417117764`}, {214, 0.11557589800546827`}, {
                    215, 0.10977615334457266`}, {216, 0.1041507102783259}, {
                    217, 0.0987042107678967}, {218, 0.09343733781565668}, {
                    219, 0.08835230014682988}, {220, 0.0834509207156375}, {
                    221, 0.07873168199141253}, {222, 0.07419540279580301}, {
                    223, 0.06984225126755672}, {224, 0.06566798441257996}, {
                    225, 0.0616731481904196}, {226, 0.05785435998510809}, {
                    227, 0.0542097540193168}, {228, 0.050735933935916196`}, {
                    229, 0.04742980734801528}, {230, 0.044287888983703665`}, {
                    231, 0.04130493677009453}, {232, 0.03847803216215673}, {
                    233, 0.035802623779484295`}, {234, 0.03327410378351036}, {
                    235, 0.03088753042728579}, {236, 0.028637517638561483`}, {
                    237, 0.0265200339393725}, {238, 0.024529894058588228`}, {
                    239, 0.022661673157524227`}, {240, 0.02091051709060792}, {
                    241, 0.01927136934054423}, {242, 0.0177393465627438}, {
                    243, 0.016309428448624385`}, {
                    244, 0.014976490853957705`}, {
                    245, 0.013735190975997291`}, {
                    246, 0.012581058234864132`}, {
                    247, 0.011510126540786694`}, {
                    248, 0.010517351802369235`}, {249, 0.00959824873699968}, {
                    250, 0.00874852546905899}, {251, 0.007964239394287614}}],
                   Frame->True,
                   FrameTicks->{{{1., 
                    FormBox[
                    RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
                    FormBox[
                    RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
                    FormBox[
                    RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
                    FormBox[
                    RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
                    FormBox["0", TraditionalForm]}, {157.25, 
                    FormBox["2.5`", TraditionalForm]}, {188.5, 
                    FormBox["5", TraditionalForm]}, {219.75, 
                    FormBox["7.5`", TraditionalForm]}, {251., 
                    FormBox["10", TraditionalForm]}}, Automatic, None, None},
                   PlotRange->{-0.1, 1.1},
                   PlotRangeClipping->True,
                   PlotRangePadding->{
                    Scaled[0.02], Automatic}], 21->
                  GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF13tcTocfB/AzWwkxloglJyyEoa1EySdKV93vN+fp3tNTuYy1zByXWMQs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                    "], {
                    {GrayLevel[0.5], EdgeForm[None], 
                    GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ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                    
                    "]]]}, {{}, {}, {
                    {Hue[0.9355110244011989, 1, 1], LineBox[{1, 2}]}, 
                    {Hue[0.8331243269093445, 1, 1], LineBox[{2, 3}]}, 
                    {Hue[0.7307595415030963, 1, 1], LineBox[{3, 4}]}, 
                    {Hue[0.6279388039622625, 1, 1], LineBox[{4, 5}]}, 
                    {Hue[0.5268258073343911, 1, 1], LineBox[{5, 6}]}, 
                    {Hue[0.424171083302787, 1, 1], LineBox[{6, 7}]}, 
                    {Hue[0.3222947041560654, 1, 1], LineBox[{7, 8}]}, 
                    {Hue[0.22127598997976075`, 1, 1], LineBox[{8, 9}]}, 
                    {Hue[0.12037676786958348`, 1, 1], LineBox[{9, 10}]}, 
                    {Hue[0.018976825257496246`, 1, 1], LineBox[{10, 11}]}, 
                    {Hue[0.918522893746869, 1, 1], LineBox[{11, 12}]}, 
                    {Hue[0.8182612468644723, 1, 1], LineBox[{12, 13}]}, 
                    {Hue[0.7183833795476677, 1, 1], LineBox[{13, 14}]}, 
                    {Hue[0.6183986488335731, 1, 1], LineBox[{14, 15}]}, 
                    {Hue[0.5193758685502148, 1, 1], LineBox[{15, 16}]}, 
                    {Hue[0.41994575252986666`, 1, 1], LineBox[{16, 17}]}, 
                    {Hue[0.32105900216888295`, 1, 1], LineBox[{17, 18}]}, 
                    {Hue[0.22234529701340938`, 1, 1], LineBox[{18, 19}]}, 
                    {Hue[0.1237823642680303, 1, 1], LineBox[{19, 20}]}, 
                    {Hue[0.02539869814779855, 1, 1], LineBox[{20, 21}]}, 
                    {Hue[0.9275293115368062, 1, 1], LineBox[{21, 22}]}, 
                    {Hue[0.8297207691016857, 1, 1], LineBox[{22, 23}]}, 
                    {Hue[0.7323239009278713, 1, 1], LineBox[{23, 24}]}, 
                    {Hue[0.6349851438514744, 1, 1], LineBox[{24, 25}]}, 
                    {Hue[0.5378773905896601, 1, 1], LineBox[{25, 26}]}, 
                    {Hue[0.4410809731310571, 1, 1], LineBox[{26, 27}]}, 
                    {Hue[0.34442190104855774`, 1, 1], LineBox[{27, 28}]}, 
                    {Hue[0.24803948086947397`, 1, 1], LineBox[{28, 29}]}, 
                    {Hue[0.15187714678567418`, 1, 1], LineBox[{29, 30}]}, 
                    {Hue[0.055945236434117585`, 1, 1], LineBox[{30, 31}]}, 
                    {Hue[0.9601410826422264, 1, 1], LineBox[{31, 32}]}, 
                    {Hue[0.8646436606294935, 1, 1], LineBox[{32, 33}]}, 
                    {Hue[0.7693298530870238, 1, 1], LineBox[{33, 34}]}, 
                    {Hue[0.6742223440667043, 1, 1], LineBox[{34, 35}]}, 
                    {Hue[0.5793187421548431, 1, 1], LineBox[{35, 36}]}, 
                    {Hue[0.4846333395900681, 1, 1], LineBox[{36, 37}]}, 
                    {Hue[0.3901292437334285, 1, 1], LineBox[{37, 38}]}, 
                    {Hue[0.2958404119994364, 1, 1], LineBox[{38, 39}]}, 
                    {Hue[0.20173313272304044`, 1, 1], LineBox[{39, 40}]}, 
                    {Hue[0.1078217796597472, 1, 1], LineBox[{40, 41}]}, 
                    {Hue[0.01410988454345159, 1, 1], LineBox[{41, 42}]}, 
                    {Hue[0.9205757406417387, 1, 1], LineBox[{42, 43}]}, 
                    {Hue[0.827240765562894, 1, 1], LineBox[{43, 44}]}, 
                    {Hue[0.734071085562777, 1, 1], LineBox[{44, 45}]}, 
                    {Hue[0.64109432272257, 1, 1], LineBox[{45, 46}]}, 
                    {Hue[0.5482985086259914, 1, 1], LineBox[{46, 47}]}, 
                    {Hue[0.45567974176311293`, 1, 1], LineBox[{47, 48}]}, 
                    {Hue[0.363222210809178, 1, 1], LineBox[{48, 49}]}, 
                    {Hue[0.2709463557656617, 1, 1], LineBox[{49, 50}]}, 
                    {Hue[0.1788348021421335, 1, 1], LineBox[{50, 51}]}, 
                    {Hue[0.0868829353325018, 1, 1], LineBox[{51, 52}]}, 
                    {Hue[0.9950982708020901, 1, 1], LineBox[{52, 53}]}, 
                    {Hue[0.9034663428818953, 1, 1], LineBox[{53, 54}]}, 
                    {Hue[0.8119948684127546, 1, 1], LineBox[{54, 55}]}, 
                    {Hue[0.7206732687351798, 1, 1], LineBox[{55, 56}]}, 
                    {Hue[0.6295015001961014, 1, 1], LineBox[{56, 57}]}, 
                    {Hue[0.5384757740462534, 1, 1], LineBox[{57, 58}]}, 
                    {Hue[0.44759116579465397`, 1, 1], LineBox[{58, 59}]}, 
                    {Hue[0.3568458792103737, 1, 1], LineBox[{59, 60}]}, 
                    {Hue[0.2662389480082245, 1, 1], LineBox[{60, 61}]}, 
                    {Hue[0.1757615783609027, 1, 1], LineBox[{61, 62}]}, 
                    {Hue[0.0854139219264677, 1, 1], LineBox[{62, 63}]}, 
                    {Hue[0.9951890441798408, 1, 1], LineBox[{63, 64}]}, 
                    {Hue[0.9050842641126323, 1, 1], LineBox[{64, 65}]}, 
                    {Hue[0.8150993924344233, 1, 1], LineBox[{65, 66}]}, 
                    {Hue[0.7252233314084258, 1, 1], LineBox[{66, 67}]}, 
                    {Hue[0.6354592261172873, 1, 1], LineBox[{67, 68}]}, 
                    {Hue[0.5457985330640146, 1, 1], LineBox[{68, 69}]}, 
                    {Hue[0.45623669902216535`, 1, 1], LineBox[{69, 70}]}, 
                    {Hue[0.36677041065441346`, 1, 1], LineBox[{70, 71}]}, 
                    {Hue[0.27739418643777836`, 1, 1], LineBox[{71, 72}]}, 
                    {Hue[0.18810275745714697`, 1, 1], LineBox[{72, 73}]}, 
                    {Hue[0.09889230647015664, 1, 1], LineBox[{73, 74}]}, 
                    {Hue[0.00975804126335437, 1, 1], LineBox[{74, 75}]}, 
                    {Hue[0.9206921227058936, 1, 1], LineBox[{75, 76}]}, 
                    {Hue[0.8316919469100537, 1, 1], LineBox[{76, 77}]}, 
                    {Hue[0.7427524318223738, 1, 1], LineBox[{77, 78}]}, 
                    {Hue[0.6538648668741538, 1, 1], LineBox[{78, 79}]}, 
                    {Hue[0.5650260486075918, 1, 1], LineBox[{79, 80}]}, 
                    {Hue[0.47622861244209524`, 1, 1], LineBox[{80, 81}]}, 
                    {Hue[0.3874663043663873, 1, 1], LineBox[{81, 82}]}, 
                    {Hue[0.29873346458885097`, 1, 1], LineBox[{82, 83}]}, 
                    {Hue[0.2100241658752546, 1, 1], LineBox[{83, 84}]}, 
                    {Hue[0.12133163459447596`, 1, 1], LineBox[{84, 85}]}, 
                    {Hue[0.032649092430081426`, 1, 1], LineBox[{85, 86}]}, 
                    {Hue[0.9439705938460063, 1, 1], LineBox[{86, 87}]}, 
                    {Hue[0.8552884086498929, 1, 1], LineBox[{87, 88}]}, 
                    {Hue[0.7665970492852087, 1, 1], LineBox[{88, 89}]}, 
                    {Hue[0.6778898537541902, 1, 1], LineBox[{89, 90}]}, 
                    {Hue[0.5891600056030785, 1, 1], LineBox[{90, 91}]}, 
                    {Hue[0.5004025675691461, 1, 1], LineBox[{91, 92}]}, 
                    {Hue[0.4116114355042888, 1, 1], LineBox[{92, 93}]}, 
                    {Hue[0.322779950578861, 1, 1], LineBox[{93, 94}]}, 
                    {Hue[0.23390508369437094`, 1, 1], LineBox[{94, 95}]}, 
                    {Hue[0.14498100702236932`, 1, 1], LineBox[{95, 96}]}, 
                    {Hue[0.05600704660552275, 1, 1], LineBox[{96, 97}]}, 
                    {Hue[0.9669795709678686, 1, 1], LineBox[{97, 98}]}, 
                    {Hue[0.8778982582793236, 1, 1], LineBox[{98, 99}]}, 
                    {Hue[0.7887652557309699, 1, 1], LineBox[{99, 100}]}, 
                    {Hue[0.6995823800456853, 1, 1], LineBox[{100, 101}]}, 
                    {Hue[0.6103564710800458, 1, 1], LineBox[{101, 102}]}, 
                    {Hue[0.5210960147936234, 1, 1], LineBox[{102, 103}]}, 
                    {Hue[0.4318125288607402, 1, 1], LineBox[{103, 104}]}, 
                    {Hue[0.34252493459971806`, 1, 1], LineBox[{104, 105}]}, 
                    {Hue[0.25325369870288117`, 1, 1], LineBox[{105, 106}]}, 
                    {Hue[0.16402728053127844`, 1, 1], LineBox[{106, 107}]}, 
                    {Hue[0.07488002620663553, 1, 1], LineBox[{107, 108}]}, 
                    {Hue[0.9858520065701789, 1, 1], LineBox[{108, 109}]}, 
                    {Hue[0.8969931708519823, 1, 1], LineBox[{109, 110}]}, 
                    {Hue[0.8083612816537772, 1, 1], LineBox[{110, 111}]}, 
                    {Hue[0.720027846091456, 1, 1], LineBox[{111, 112}]}, 
                    {Hue[0.6320783020478189, 1, 1], LineBox[{112, 113}]}, 
                    {Hue[0.5446126226524881, 1, 1], LineBox[{113, 114}]}, 
                    {Hue[0.4577416941256004, 1, 1], LineBox[{114, 115}]}, 
                    {Hue[0.371589448811156, 1, 1], LineBox[{115, 116}]}, 
                    {Hue[0.2862928742918964, 1, 1], LineBox[{116, 117}]}, 
                    {Hue[0.20200228023477101`, 1, 1], LineBox[{117, 118}]}, 
                    {Hue[0.11888507280599135`, 1, 1], LineBox[{118, 119}]}, 
                    {Hue[0.037127910830192574`, 1, 1], LineBox[{119, 120}]}, 
                    {Hue[0.9569362186420415, 1, 1], LineBox[{120, 121}]}, 
                    {Hue[0.8785305257889706, 1, 1], LineBox[{121, 122}]}, 
                    {Hue[0.8021410588218201, 1, 1], LineBox[{122, 123}]}, 
                    {Hue[0.7279945898245506, 1, 1], LineBox[{123, 124}]}, 
                    {Hue[0.656309101852733, 1, 1], LineBox[{124, 125}]}, 
                    {Hue[0.5872863814003371, 1, 1], LineBox[{125, 126}]}, 
                    {Hue[0.5211049841628967, 1, 1], LineBox[{126, 127}]}, 
                    {Hue[0.4579218320357416, 1, 1], LineBox[{127, 128}]}, 
                    {Hue[0.3978644395073952, 1, 1], LineBox[{128, 129}]}, 
                    {Hue[0.34103635547553546`, 1, 1], LineBox[{129, 130}]}, 
                    {Hue[0.28751299833736366`, 1, 1], LineBox[{130, 131}]}, 
                    {Hue[0.2373425683885671, 1, 1], LineBox[{131, 132}]}, 
                    {Hue[0.19054743917487552`, 1, 1], LineBox[{132, 133}]}, 
                    {Hue[0.14712457912982915`, 1, 1], LineBox[{133, 134}]}, 
                    {Hue[0.10704787791664298`, 1, 1], LineBox[{134, 135}]}, 
                    {Hue[0.07027258996258669, 1, 1], LineBox[{135, 136}]}, 
                    {Hue[0.036734633034547576`, 1, 1], LineBox[{136, 137}]}, 
                    {Hue[0.0063562804960939565`, 1, 1], 
                    LineBox[{137, 138}]}, 
                    {Hue[0.9790486019005894, 1, 1], LineBox[{138, 139}]}, 
                    {Hue[0.954714129564768, 1, 1], LineBox[{139, 140}]}, 
                    {Hue[0.9332507821720971, 1, 1], LineBox[{140, 141}]}, 
                    {Hue[0.9145521366166071, 1, 1], LineBox[{141, 142}]}, 
                    {Hue[0.8985108022332381, 1, 1], LineBox[{142, 143}]}, 
                    {Hue[0.885019537265343, 1, 1], LineBox[{143, 144}]}, 
                    {Hue[0.8739740784029262, 1, 1], LineBox[{144, 145}]}, 
                    {Hue[0.8652717100434656, 1, 1], LineBox[{145, 146}]}, 
                    {Hue[0.8588139578755996, 1, 1], LineBox[{146, 147}]}, 
                    {Hue[0.8545070219354367, 1, 1], LineBox[{147, 148}]}, 
                    {Hue[0.8522605424068836, 1, 1], LineBox[{148, 149}]}, 
                    {Hue[0.8519894958550597, 1, 1], LineBox[{149, 150}]}, 
                    {Hue[0.8536135359957169, 1, 1], LineBox[{150, 151}]}, 
                    {Hue[0.8570565069011515, 1, 1], LineBox[{151, 152}]}, 
                    {Hue[0.8622480974679313, 1, 1], LineBox[{152, 153}]}, 
                    {Hue[0.8691216861664274, 1, 1], LineBox[{153, 154}]}, 
                    {Hue[0.877614308802046, 1, 1], LineBox[{154, 155}]}, 
                    {Hue[0.8876687311186671, 1, 1], LineBox[{155, 156}]}, 
                    {Hue[0.899230510278553, 1, 1], LineBox[{156, 157}]}, 
                    {Hue[0.9122490135588197, 1, 1], LineBox[{157, 158}]}, 
                    {Hue[0.9266773800415274, 1, 1], LineBox[{158, 159}]}, 
                    {Hue[0.9424715311467944, 1, 1], LineBox[{159, 160}]}, 
                    {Hue[0.9595905066764843, 1, 1], LineBox[{160, 161}]}, 
                    {Hue[0.9779961683060914, 1, 1], LineBox[{161, 162}]}, 
                    {Hue[0.9976531263524046, 1, 1], LineBox[{162, 163}]}, 
                    {Hue[0.018527973642258822`, 1, 1], LineBox[{163, 164}]}, 
                    {Hue[0.040589846530470396`, 1, 1], LineBox[{164, 165}]}, 
                    {Hue[0.06380921313635418, 1, 1], LineBox[{165, 166}]}, 
                    {Hue[0.088159456147119, 1, 1], LineBox[{166, 167}]}, 
                    {Hue[0.11361500882265167`, 1, 1], LineBox[{167, 168}]}, 
                    {Hue[0.14015184759702176`, 1, 1], LineBox[{168, 169}]}, 
                    {Hue[0.1677470050651769, 1, 1], LineBox[{169, 170}]}, 
                    {Hue[0.1963805235167193, 1, 1], LineBox[{170, 171}]}, 
                    {Hue[0.226032063227786, 1, 1], LineBox[{171, 172}]}, 
                    {Hue[0.2566825269000016, 1, 1], LineBox[{172, 173}]}, 
                    {Hue[0.2883143142722514, 1, 1], LineBox[{173, 174}]}, 
                    {Hue[0.32091103841903484`, 1, 1], LineBox[{174, 175}]}, 
                    {Hue[0.3544570383872956, 1, 1], LineBox[{175, 176}]}, 
                    {Hue[0.3889373946660122, 1, 1], LineBox[{176, 177}]}, 
                    {Hue[0.42433703731545136`, 1, 1], LineBox[{177, 178}]}, 
                    {Hue[0.4606438306174138, 1, 1], LineBox[{178, 179}]}, 
                    {Hue[0.49784493890218584`, 1, 1], LineBox[{179, 180}]}, 
                    {Hue[0.535927662459045, 1, 1], LineBox[{180, 181}]}, 
                    {Hue[0.5748807416320714, 1, 1], LineBox[{181, 182}]}, 
                    {Hue[0.6146930444978822, 1, 1], LineBox[{182, 183}]}, 
                    {Hue[0.6553538884368986, 1, 1], LineBox[{183, 184}]}, 
                    {Hue[0.696853218504837, 1, 1], LineBox[{184, 185}]}, 
                    {Hue[0.7391814889795953, 1, 1], LineBox[{185, 186}]}, 
                    {Hue[0.7823289599944764, 1, 1], LineBox[{186, 187}]}, 
                    {Hue[0.8262874078226671, 1, 1], LineBox[{187, 188}]}, 
                    {Hue[0.8710478798485146, 1, 1], LineBox[{188, 189}]}, 
                    {Hue[0.9166034157050914, 1, 1], LineBox[{189, 190}]}, 
                    {Hue[0.9629448223502752, 1, 1], LineBox[{190, 191}]}, 
                    {Hue[0.01006559471412739, 1, 1], LineBox[{191, 192}]}, 
                    {Hue[0.05795832163985246, 1, 1], LineBox[{192, 193}]}, 
                    {Hue[0.10661432626890296`, 1, 1], LineBox[{193, 194}]}, 
                    {Hue[0.1560288107312348, 1, 1], LineBox[{194, 195}]}, 
                    {Hue[0.2061934463881734, 1, 1], LineBox[{195, 196}]}, 
                    {Hue[0.25710371868991605`, 1, 1], LineBox[{196, 197}]}, 
                    {Hue[0.30875258049480325`, 1, 1], LineBox[{197, 198}]}, 
                    {Hue[0.3611346594763807, 1, 1], LineBox[{198, 199}]}, 
                    {Hue[0.4142443931279519, 1, 1], LineBox[{199, 200}]}, 
                    {Hue[0.46807592250982194`, 1, 1], LineBox[{200, 201}]}, 
                    {Hue[0.5226235169268131, 1, 1], LineBox[{201, 202}]}, 
                    {Hue[0.5778827231313868, 1, 1], LineBox[{202, 203}]}, 
                    {Hue[0.6338467284901352, 1, 1], LineBox[{203, 204}]}, 
                    {Hue[0.6905119716638414, 1, 1], LineBox[{204, 205}]}, 
                    {Hue[0.7478729708429508, 1, 1], LineBox[{205, 206}]}, 
                    {Hue[0.8059262182338722, 1, 1], LineBox[{206, 207}]}, 
                    {Hue[0.8646667029288123, 1, 1], LineBox[{207, 208}]}, 
                    {Hue[0.9240900148564036, 1, 1], LineBox[{208, 209}]}, 
                    {Hue[0.9841921136405349, 1, 1], LineBox[{209, 210}]}, 
                    {Hue[0.044968567409435826`, 1, 1], LineBox[{210, 211}]}, 
                    {Hue[0.1064158884423126, 1, 1], LineBox[{211, 212}]}, 
                    {Hue[0.16852978608792193`, 1, 1], LineBox[{212, 213}]}, 
                    {Hue[0.23130554990275484`, 1, 1], LineBox[{213, 214}]}, 
                    {Hue[0.29474084889917324`, 1, 1], LineBox[{214, 215}]}, 
                    {Hue[0.358831588919603, 1, 1], LineBox[{215, 216}]}, 
                    {Hue[0.423572986863044, 1, 1], LineBox[{216, 217}]}, 
                    {Hue[0.4889633994506269, 1, 1], LineBox[{217, 218}]}, 
                    {Hue[0.5549981042468378, 1, 1], LineBox[{218, 219}]}, 
                    {Hue[0.6216756108260031, 1, 1], LineBox[{219, 220}]}, 
                    {Hue[0.6889900341282765, 1, 1], LineBox[{220, 221}]}, 
                    {Hue[0.7569404375017312, 1, 1], LineBox[{221, 222}]}, 
                    {Hue[0.8255219568966075, 1, 1], LineBox[{222, 223}]}, 
                    {Hue[0.8947351781930797, 1, 1], LineBox[{223, 224}]}, 
                    {Hue[0.9645711467107132, 1, 1], LineBox[{224, 225}]}, 
                    {Hue[0.035032200808264795`, 1, 1], LineBox[{225, 226}]}, 
                    {Hue[0.10611430302560833`, 1, 1], LineBox[{226, 227}]}, 
                    {Hue[0.1778127455017217, 1, 1], LineBox[{227, 228}]}, 
                    {Hue[0.25012585040377056`, 1, 1], LineBox[{228, 229}]}, 
                    {Hue[0.3230512831048453, 1, 1], LineBox[{229, 230}]}, 
                    {Hue[0.39658786037578897`, 1, 1], LineBox[{230, 231}]}, 
                    {Hue[0.47073048283222535`, 1, 1], LineBox[{231, 232}]}, 
                    {Hue[0.5454760758758606, 1, 1], LineBox[{232, 233}]}, 
                    {Hue[0.6208268462480444, 1, 1], LineBox[{233, 234}]}, 
                    {Hue[0.6967736619070093, 1, 1], LineBox[{234, 235}]}, 
                    {Hue[0.7733207693695567, 1, 1], LineBox[{235, 236}]}, 
                    {Hue[0.85046436527129, 1, 1], LineBox[{236, 237}]}, 
                    {Hue[0.9281979578049646, 1, 1], LineBox[{237, 238}]}, 
                    {Hue[0.006524208684187778, 1, 1], LineBox[{238, 239}]}, 
                    {Hue[0.08543857693114915, 1, 1], LineBox[{239, 240}]}, 
                    {Hue[0.1649389112410558, 1, 1], LineBox[{240, 241}]}, 
                    {Hue[0.24502025669775915`, 1, 1], LineBox[{241, 242}]}, 
                    {Hue[0.3256855387817807, 1, 1], LineBox[{242, 243}]}, 
                    {Hue[0.4069328254093805, 1, 1], LineBox[{243, 244}]}, 
                    {Hue[0.48875648074544686`, 1, 1], LineBox[{244, 245}]}, 
                    {Hue[0.5711582052233701, 1, 1], LineBox[{245, 246}]}, 
                    {Hue[0.6541335516799798, 1, 1], LineBox[{246, 247}]}, 
                    {Hue[0.7376792030777755, 1, 1], LineBox[{247, 248}]}, 
                    {Hue[0.8217951224193869, 1, 1], LineBox[{248, 249}]}, 
                    {Hue[0.906479842223855, 1, 1], LineBox[{249, 250}]}, 
                    {Hue[0.9917352697531961, 1, 1], LineBox[{250, 251}]}}}},
                    VertexColors->{
                    Hue[0.9355110244011989, 1, 1], 
                    Hue[0.8331243269093445, 1, 1], 
                    Hue[0.7307595415030963, 1, 1], 
                    Hue[0.6279388039622625, 1, 1], 
                    Hue[0.5268258073343911, 1, 1], 
                    Hue[0.424171083302787, 1, 1], 
                    Hue[0.3222947041560654, 1, 1], 
                    Hue[0.22127598997976075`, 1, 1], 
                    Hue[0.12037676786958348`, 1, 1], 
                    Hue[0.018976825257496246`, 1, 1], 
                    Hue[0.918522893746869, 1, 1], 
                    Hue[0.8182612468644723, 1, 1], 
                    Hue[0.7183833795476677, 1, 1], 
                    Hue[0.6183986488335731, 1, 1], 
                    Hue[0.5193758685502148, 1, 1], 
                    Hue[0.41994575252986666`, 1, 1], 
                    Hue[0.32105900216888295`, 1, 1], 
                    Hue[0.22234529701340938`, 1, 1], 
                    Hue[0.1237823642680303, 1, 1], 
                    Hue[0.02539869814779855, 1, 1], 
                    Hue[0.9275293115368062, 1, 1], 
                    Hue[0.8297207691016857, 1, 1], 
                    Hue[0.7323239009278713, 1, 1], 
                    Hue[0.6349851438514744, 1, 1], 
                    Hue[0.5378773905896601, 1, 1], 
                    Hue[0.4410809731310571, 1, 1], 
                    Hue[0.34442190104855774`, 1, 1], 
                    Hue[0.24803948086947397`, 1, 1], 
                    Hue[0.15187714678567418`, 1, 1], 
                    Hue[0.055945236434117585`, 1, 1], 
                    Hue[0.9601410826422264, 1, 1], 
                    Hue[0.8646436606294935, 1, 1], 
                    Hue[0.7693298530870238, 1, 1], 
                    Hue[0.6742223440667043, 1, 1], 
                    Hue[0.5793187421548431, 1, 1], 
                    Hue[0.4846333395900681, 1, 1], 
                    Hue[0.3901292437334285, 1, 1], 
                    Hue[0.2958404119994364, 1, 1], 
                    Hue[0.20173313272304044`, 1, 1], 
                    Hue[0.1078217796597472, 1, 1], 
                    Hue[0.01410988454345159, 1, 1], 
                    Hue[0.9205757406417387, 1, 1], 
                    Hue[0.827240765562894, 1, 1], 
                    Hue[0.734071085562777, 1, 1], 
                    Hue[0.64109432272257, 1, 1], 
                    Hue[0.5482985086259914, 1, 1], 
                    Hue[0.45567974176311293`, 1, 1], 
                    Hue[0.363222210809178, 1, 1], 
                    Hue[0.2709463557656617, 1, 1], 
                    Hue[0.1788348021421335, 1, 1], 
                    Hue[0.0868829353325018, 1, 1], 
                    Hue[0.9950982708020901, 1, 1], 
                    Hue[0.9034663428818953, 1, 1], 
                    Hue[0.8119948684127546, 1, 1], 
                    Hue[0.7206732687351798, 1, 1], 
                    Hue[0.6295015001961014, 1, 1], 
                    Hue[0.5384757740462534, 1, 1], 
                    Hue[0.44759116579465397`, 1, 1], 
                    Hue[0.3568458792103737, 1, 1], 
                    Hue[0.2662389480082245, 1, 1], 
                    Hue[0.1757615783609027, 1, 1], 
                    Hue[0.0854139219264677, 1, 1], 
                    Hue[0.9951890441798408, 1, 1], 
                    Hue[0.9050842641126323, 1, 1], 
                    Hue[0.8150993924344233, 1, 1], 
                    Hue[0.7252233314084258, 1, 1], 
                    Hue[0.6354592261172873, 1, 1], 
                    Hue[0.5457985330640146, 1, 1], 
                    Hue[0.45623669902216535`, 1, 1], 
                    Hue[0.36677041065441346`, 1, 1], 
                    Hue[0.27739418643777836`, 1, 1], 
                    Hue[0.18810275745714697`, 1, 1], 
                    Hue[0.09889230647015664, 1, 1], 
                    Hue[0.00975804126335437, 1, 1], 
                    Hue[0.9206921227058936, 1, 1], 
                    Hue[0.8316919469100537, 1, 1], 
                    Hue[0.7427524318223738, 1, 1], 
                    Hue[0.6538648668741538, 1, 1], 
                    Hue[0.5650260486075918, 1, 1], 
                    Hue[0.47622861244209524`, 1, 1], 
                    Hue[0.3874663043663873, 1, 1], 
                    Hue[0.29873346458885097`, 1, 1], 
                    Hue[0.2100241658752546, 1, 1], 
                    Hue[0.12133163459447596`, 1, 1], 
                    Hue[0.032649092430081426`, 1, 1], 
                    Hue[0.9439705938460063, 1, 1], 
                    Hue[0.8552884086498929, 1, 1], 
                    Hue[0.7665970492852087, 1, 1], 
                    Hue[0.6778898537541902, 1, 1], 
                    Hue[0.5891600056030785, 1, 1], 
                    Hue[0.5004025675691461, 1, 1], 
                    Hue[0.4116114355042888, 1, 1], 
                    Hue[0.322779950578861, 1, 1], 
                    Hue[0.23390508369437094`, 1, 1], 
                    Hue[0.14498100702236932`, 1, 1], 
                    Hue[0.05600704660552275, 1, 1], 
                    Hue[0.9669795709678686, 1, 1], 
                    Hue[0.8778982582793236, 1, 1], 
                    Hue[0.7887652557309699, 1, 1], 
                    Hue[0.6995823800456853, 1, 1], 
                    Hue[0.6103564710800458, 1, 1], 
                    Hue[0.5210960147936234, 1, 1], 
                    Hue[0.4318125288607402, 1, 1], 
                    Hue[0.34252493459971806`, 1, 1], 
                    Hue[0.25325369870288117`, 1, 1], 
                    Hue[0.16402728053127844`, 1, 1], 
                    Hue[0.07488002620663553, 1, 1], 
                    Hue[0.9858520065701789, 1, 1], 
                    Hue[0.8969931708519823, 1, 1], 
                    Hue[0.8083612816537772, 1, 1], 
                    Hue[0.720027846091456, 1, 1], 
                    Hue[0.6320783020478189, 1, 1], 
                    Hue[0.5446126226524881, 1, 1], 
                    Hue[0.4577416941256004, 1, 1], 
                    Hue[0.371589448811156, 1, 1], 
                    Hue[0.2862928742918964, 1, 1], 
                    Hue[0.20200228023477101`, 1, 1], 
                    Hue[0.11888507280599135`, 1, 1], 
                    Hue[0.037127910830192574`, 1, 1], 
                    Hue[0.9569362186420415, 1, 1], 
                    Hue[0.8785305257889706, 1, 1], 
                    Hue[0.8021410588218201, 1, 1], 
                    Hue[0.7279945898245506, 1, 1], 
                    Hue[0.656309101852733, 1, 1], 
                    Hue[0.5872863814003371, 1, 1], 
                    Hue[0.5211049841628967, 1, 1], 
                    Hue[0.4579218320357416, 1, 1], 
                    Hue[0.3978644395073952, 1, 1], 
                    Hue[0.34103635547553546`, 1, 1], 
                    Hue[0.28751299833736366`, 1, 1], 
                    Hue[0.2373425683885671, 1, 1], 
                    Hue[0.19054743917487552`, 1, 1], 
                    Hue[0.14712457912982915`, 1, 1], 
                    Hue[0.10704787791664298`, 1, 1], 
                    Hue[0.07027258996258669, 1, 1], 
                    Hue[0.036734633034547576`, 1, 1], 
                    Hue[0.0063562804960939565`, 1, 1], 
                    Hue[0.9790486019005894, 1, 1], 
                    Hue[0.954714129564768, 1, 1], 
                    Hue[0.9332507821720971, 1, 1], 
                    Hue[0.9145521366166071, 1, 1], 
                    Hue[0.8985108022332381, 1, 1], 
                    Hue[0.885019537265343, 1, 1], 
                    Hue[0.8739740784029262, 1, 1], 
                    Hue[0.8652717100434656, 1, 1], 
                    Hue[0.8588139578755996, 1, 1], 
                    Hue[0.8545070219354367, 1, 1], 
                    Hue[0.8522605424068836, 1, 1], 
                    Hue[0.8519894958550597, 1, 1], 
                    Hue[0.8536135359957169, 1, 1], 
                    Hue[0.8570565069011515, 1, 1], 
                    Hue[0.8622480974679313, 1, 1], 
                    Hue[0.8691216861664274, 1, 1], 
                    Hue[0.877614308802046, 1, 1], 
                    Hue[0.8876687311186671, 1, 1], 
                    Hue[0.899230510278553, 1, 1], 
                    Hue[0.9122490135588197, 1, 1], 
                    Hue[0.9266773800415274, 1, 1], 
                    Hue[0.9424715311467944, 1, 1], 
                    Hue[0.9595905066764843, 1, 1], 
                    Hue[0.9779961683060914, 1, 1], 
                    Hue[0.9976531263524046, 1, 1], 
                    Hue[0.018527973642258822`, 1, 1], 
                    Hue[0.040589846530470396`, 1, 1], 
                    Hue[0.06380921313635418, 1, 1], 
                    Hue[0.088159456147119, 1, 1], 
                    Hue[0.11361500882265167`, 1, 1], 
                    Hue[0.14015184759702176`, 1, 1], 
                    Hue[0.1677470050651769, 1, 1], 
                    Hue[0.1963805235167193, 1, 1], 
                    Hue[0.226032063227786, 1, 1], 
                    Hue[0.2566825269000016, 1, 1], 
                    Hue[0.2883143142722514, 1, 1], 
                    Hue[0.32091103841903484`, 1, 1], 
                    Hue[0.3544570383872956, 1, 1], 
                    Hue[0.3889373946660122, 1, 1], 
                    Hue[0.42433703731545136`, 1, 1], 
                    Hue[0.4606438306174138, 1, 1], 
                    Hue[0.49784493890218584`, 1, 1], 
                    Hue[0.535927662459045, 1, 1], 
                    Hue[0.5748807416320714, 1, 1], 
                    Hue[0.6146930444978822, 1, 1], 
                    Hue[0.6553538884368986, 1, 1], 
                    Hue[0.696853218504837, 1, 1], 
                    Hue[0.7391814889795953, 1, 1], 
                    Hue[0.7823289599944764, 1, 1], 
                    Hue[0.8262874078226671, 1, 1], 
                    Hue[0.8710478798485146, 1, 1], 
                    Hue[0.9166034157050914, 1, 1], 
                    Hue[0.9629448223502752, 1, 1], 
                    Hue[0.01006559471412739, 1, 1], 
                    Hue[0.05795832163985246, 1, 1], 
                    Hue[0.10661432626890296`, 1, 1], 
                    Hue[0.1560288107312348, 1, 1], 
                    Hue[0.2061934463881734, 1, 1], 
                    Hue[0.25710371868991605`, 1, 1], 
                    Hue[0.30875258049480325`, 1, 1], 
                    Hue[0.3611346594763807, 1, 1], 
                    Hue[0.4142443931279519, 1, 1], 
                    Hue[0.46807592250982194`, 1, 1], 
                    Hue[0.5226235169268131, 1, 1], 
                    Hue[0.5778827231313868, 1, 1], 
                    Hue[0.6338467284901352, 1, 1], 
                    Hue[0.6905119716638414, 1, 1], 
                    Hue[0.7478729708429508, 1, 1], 
                    Hue[0.8059262182338722, 1, 1], 
                    Hue[0.8646667029288123, 1, 1], 
                    Hue[0.9240900148564036, 1, 1], 
                    Hue[0.9841921136405349, 1, 1], 
                    Hue[0.044968567409435826`, 1, 1], 
                    Hue[0.1064158884423126, 1, 1], 
                    Hue[0.16852978608792193`, 1, 1], 
                    Hue[0.23130554990275484`, 1, 1], 
                    Hue[0.29474084889917324`, 1, 1], 
                    Hue[0.358831588919603, 1, 1], 
                    Hue[0.423572986863044, 1, 1], 
                    Hue[0.4889633994506269, 1, 1], 
                    Hue[0.5549981042468378, 1, 1], 
                    Hue[0.6216756108260031, 1, 1], 
                    Hue[0.6889900341282765, 1, 1], 
                    Hue[0.7569404375017312, 1, 1], 
                    Hue[0.8255219568966075, 1, 1], 
                    Hue[0.8947351781930797, 1, 1], 
                    Hue[0.9645711467107132, 1, 1], 
                    Hue[0.035032200808264795`, 1, 1], 
                    Hue[0.10611430302560833`, 1, 1], 
                    Hue[0.1778127455017217, 1, 1], 
                    Hue[0.25012585040377056`, 1, 1], 
                    Hue[0.3230512831048453, 1, 1], 
                    Hue[0.39658786037578897`, 1, 1], 
                    Hue[0.47073048283222535`, 1, 1], 
                    Hue[0.5454760758758606, 1, 1], 
                    Hue[0.6208268462480444, 1, 1], 
                    Hue[0.6967736619070093, 1, 1], 
                    Hue[0.7733207693695567, 1, 1], 
                    Hue[0.85046436527129, 1, 1], 
                    Hue[0.9281979578049646, 1, 1], 
                    Hue[0.006524208684187778, 1, 1], 
                    Hue[0.08543857693114915, 1, 1], 
                    Hue[0.1649389112410558, 1, 1], 
                    Hue[0.24502025669775915`, 1, 1], 
                    Hue[0.3256855387817807, 1, 1], 
                    Hue[0.4069328254093805, 1, 1], 
                    Hue[0.48875648074544686`, 1, 1], 
                    Hue[0.5711582052233701, 1, 1], 
                    Hue[0.6541335516799798, 1, 1], 
                    Hue[0.7376792030777755, 1, 1], 
                    Hue[0.8217951224193869, 1, 1], 
                    Hue[0.906479842223855, 1, 1], 
                    Hue[0.9917352697531961, 1, 1], 
                    Hue[0.07755478805897177, 1, 1], 
                    Hue[0.9355110244011989, 1, 1], 
                    Hue[0.8331243269093445, 1, 1], 
                    Hue[0.8331243269093445, 1, 1], 
                    Hue[0.7307595415030963, 1, 1], 
                    Hue[0.7307595415030963, 1, 1], 
                    Hue[0.6279388039622625, 1, 1], 
                    Hue[0.6279388039622625, 1, 1], 
                    Hue[0.5268258073343911, 1, 1], 
                    Hue[0.5268258073343911, 1, 1], 
                    Hue[0.424171083302787, 1, 1], 
                    Hue[0.424171083302787, 1, 1], 
                    Hue[0.3222947041560654, 1, 1], 
                    Hue[0.3222947041560654, 1, 1], 
                    Hue[0.22127598997976075`, 1, 1], 
                    Hue[0.22127598997976075`, 1, 1], 
                    Hue[0.12037676786958348`, 1, 1], 
                    Hue[0.12037676786958348`, 1, 1], 
                    Hue[0.018976825257496246`, 1, 1], 
                    Hue[0.018976825257496246`, 1, 1], 
                    Hue[0.918522893746869, 1, 1], 
                    Hue[0.918522893746869, 1, 1], 
                    Hue[0.8182612468644723, 1, 1], 
                    Hue[0.8182612468644723, 1, 1], 
                    Hue[0.7183833795476677, 1, 1], 
                    Hue[0.7183833795476677, 1, 1], 
                    Hue[0.6183986488335731, 1, 1], 
                    Hue[0.6183986488335731, 1, 1], 
                    Hue[0.5193758685502148, 1, 1], 
                    Hue[0.5193758685502148, 1, 1], 
                    Hue[0.41994575252986666`, 1, 1], 
                    Hue[0.41994575252986666`, 1, 1], 
                    Hue[0.32105900216888295`, 1, 1], 
                    Hue[0.32105900216888295`, 1, 1], 
                    Hue[0.22234529701340938`, 1, 1], 
                    Hue[0.22234529701340938`, 1, 1], 
                    Hue[0.1237823642680303, 1, 1], 
                    Hue[0.1237823642680303, 1, 1], 
                    Hue[0.02539869814779855, 1, 1], 
                    Hue[0.02539869814779855, 1, 1], 
                    Hue[0.9275293115368062, 1, 1], 
                    Hue[0.9275293115368062, 1, 1], 
                    Hue[0.8297207691016857, 1, 1], 
                    Hue[0.8297207691016857, 1, 1], 
                    Hue[0.7323239009278713, 1, 1], 
                    Hue[0.7323239009278713, 1, 1], 
                    Hue[0.6349851438514744, 1, 1], 
                    Hue[0.6349851438514744, 1, 1], 
                    Hue[0.5378773905896601, 1, 1], 
                    Hue[0.5378773905896601, 1, 1], 
                    Hue[0.4410809731310571, 1, 1], 
                    Hue[0.4410809731310571, 1, 1], 
                    Hue[0.34442190104855774`, 1, 1], 
                    Hue[0.34442190104855774`, 1, 1], 
                    Hue[0.24803948086947397`, 1, 1], 
                    Hue[0.24803948086947397`, 1, 1], 
                    Hue[0.15187714678567418`, 1, 1], 
                    Hue[0.15187714678567418`, 1, 1], 
                    Hue[0.055945236434117585`, 1, 1], 
                    Hue[0.055945236434117585`, 1, 1], 
                    Hue[0.9601410826422264, 1, 1], 
                    Hue[0.9601410826422264, 1, 1], 
                    Hue[0.8646436606294935, 1, 1], 
                    Hue[0.8646436606294935, 1, 1], 
                    Hue[0.7693298530870238, 1, 1], 
                    Hue[0.7693298530870238, 1, 1], 
                    Hue[0.6742223440667043, 1, 1], 
                    Hue[0.6742223440667043, 1, 1], 
                    Hue[0.5793187421548431, 1, 1], 
                    Hue[0.5793187421548431, 1, 1], 
                    Hue[0.4846333395900681, 1, 1], 
                    Hue[0.4846333395900681, 1, 1], 
                    Hue[0.3901292437334285, 1, 1], 
                    Hue[0.3901292437334285, 1, 1], 
                    Hue[0.2958404119994364, 1, 1], 
                    Hue[0.2958404119994364, 1, 1], 
                    Hue[0.20173313272304044`, 1, 1], 
                    Hue[0.20173313272304044`, 1, 1], 
                    Hue[0.1078217796597472, 1, 1], 
                    Hue[0.1078217796597472, 1, 1], 
                    Hue[0.01410988454345159, 1, 1], 
                    Hue[0.01410988454345159, 1, 1], 
                    Hue[0.9205757406417387, 1, 1], 
                    Hue[0.9205757406417387, 1, 1], 
                    Hue[0.827240765562894, 1, 1], 
                    Hue[0.827240765562894, 1, 1], 
                    Hue[0.734071085562777, 1, 1], 
                    Hue[0.734071085562777, 1, 1], 
                    Hue[0.64109432272257, 1, 1], 
                    Hue[0.64109432272257, 1, 1], 
                    Hue[0.5482985086259914, 1, 1], 
                    Hue[0.5482985086259914, 1, 1], 
                    Hue[0.45567974176311293`, 1, 1], 
                    Hue[0.45567974176311293`, 1, 1], 
                    Hue[0.363222210809178, 1, 1], 
                    Hue[0.363222210809178, 1, 1], 
                    Hue[0.2709463557656617, 1, 1], 
                    Hue[0.2709463557656617, 1, 1], 
                    Hue[0.1788348021421335, 1, 1], 
                    Hue[0.1788348021421335, 1, 1], 
                    Hue[0.0868829353325018, 1, 1], 
                    Hue[0.0868829353325018, 1, 1], 
                    Hue[0.9950982708020901, 1, 1], 
                    Hue[0.9950982708020901, 1, 1], 
                    Hue[0.9034663428818953, 1, 1], 
                    Hue[0.9034663428818953, 1, 1], 
                    Hue[0.8119948684127546, 1, 1], 
                    Hue[0.8119948684127546, 1, 1], 
                    Hue[0.7206732687351798, 1, 1], 
                    Hue[0.7206732687351798, 1, 1], 
                    Hue[0.6295015001961014, 1, 1], 
                    Hue[0.6295015001961014, 1, 1], 
                    Hue[0.5384757740462534, 1, 1], 
                    Hue[0.5384757740462534, 1, 1], 
                    Hue[0.44759116579465397`, 1, 1], 
                    Hue[0.44759116579465397`, 1, 1], 
                    Hue[0.3568458792103737, 1, 1], 
                    Hue[0.3568458792103737, 1, 1], 
                    Hue[0.2662389480082245, 1, 1], 
                    Hue[0.2662389480082245, 1, 1], 
                    Hue[0.1757615783609027, 1, 1], 
                    Hue[0.1757615783609027, 1, 1], 
                    Hue[0.0854139219264677, 1, 1], 
                    Hue[0.0854139219264677, 1, 1], 
                    Hue[0.9951890441798408, 1, 1], 
                    Hue[0.9951890441798408, 1, 1], 
                    Hue[0.9050842641126323, 1, 1], 
                    Hue[0.9050842641126323, 1, 1], 
                    Hue[0.8150993924344233, 1, 1], 
                    Hue[0.8150993924344233, 1, 1], 
                    Hue[0.7252233314084258, 1, 1], 
                    Hue[0.7252233314084258, 1, 1], 
                    Hue[0.6354592261172873, 1, 1], 
                    Hue[0.6354592261172873, 1, 1], 
                    Hue[0.5457985330640146, 1, 1], 
                    Hue[0.5457985330640146, 1, 1], 
                    Hue[0.45623669902216535`, 1, 1], 
                    Hue[0.45623669902216535`, 1, 1], 
                    Hue[0.36677041065441346`, 1, 1], 
                    Hue[0.36677041065441346`, 1, 1], 
                    Hue[0.27739418643777836`, 1, 1], 
                    Hue[0.27739418643777836`, 1, 1], 
                    Hue[0.18810275745714697`, 1, 1], 
                    Hue[0.18810275745714697`, 1, 1], 
                    Hue[0.09889230647015664, 1, 1], 
                    Hue[0.09889230647015664, 1, 1], 
                    Hue[0.00975804126335437, 1, 1], 
                    Hue[0.00975804126335437, 1, 1], 
                    Hue[0.9206921227058936, 1, 1], 
                    Hue[0.9206921227058936, 1, 1], 
                    Hue[0.8316919469100537, 1, 1], 
                    Hue[0.8316919469100537, 1, 1], 
                    Hue[0.7427524318223738, 1, 1], 
                    Hue[0.7427524318223738, 1, 1], 
                    Hue[0.6538648668741538, 1, 1], 
                    Hue[0.6538648668741538, 1, 1], 
                    Hue[0.5650260486075918, 1, 1], 
                    Hue[0.5650260486075918, 1, 1], 
                    Hue[0.47622861244209524`, 1, 1], 
                    Hue[0.47622861244209524`, 1, 1], 
                    Hue[0.3874663043663873, 1, 1], 
                    Hue[0.3874663043663873, 1, 1], 
                    Hue[0.29873346458885097`, 1, 1], 
                    Hue[0.29873346458885097`, 1, 1], 
                    Hue[0.2100241658752546, 1, 1], 
                    Hue[0.2100241658752546, 1, 1], 
                    Hue[0.12133163459447596`, 1, 1], 
                    Hue[0.12133163459447596`, 1, 1], 
                    Hue[0.032649092430081426`, 1, 1], 
                    Hue[0.032649092430081426`, 1, 1], 
                    Hue[0.9439705938460063, 1, 1], 
                    Hue[0.9439705938460063, 1, 1], 
                    Hue[0.8552884086498929, 1, 1], 
                    Hue[0.8552884086498929, 1, 1], 
                    Hue[0.7665970492852087, 1, 1], 
                    Hue[0.7665970492852087, 1, 1], 
                    Hue[0.6778898537541902, 1, 1], 
                    Hue[0.6778898537541902, 1, 1], 
                    Hue[0.5891600056030785, 1, 1], 
                    Hue[0.5891600056030785, 1, 1], 
                    Hue[0.5004025675691461, 1, 1], 
                    Hue[0.5004025675691461, 1, 1], 
                    Hue[0.4116114355042888, 1, 1], 
                    Hue[0.4116114355042888, 1, 1], 
                    Hue[0.322779950578861, 1, 1], 
                    Hue[0.322779950578861, 1, 1], 
                    Hue[0.23390508369437094`, 1, 1], 
                    Hue[0.23390508369437094`, 1, 1], 
                    Hue[0.14498100702236932`, 1, 1], 
                    Hue[0.14498100702236932`, 1, 1], 
                    Hue[0.05600704660552275, 1, 1], 
                    Hue[0.05600704660552275, 1, 1], 
                    Hue[0.9669795709678686, 1, 1], 
                    Hue[0.9669795709678686, 1, 1], 
                    Hue[0.8778982582793236, 1, 1], 
                    Hue[0.8778982582793236, 1, 1], 
                    Hue[0.7887652557309699, 1, 1], 
                    Hue[0.7887652557309699, 1, 1], 
                    Hue[0.6995823800456853, 1, 1], 
                    Hue[0.6995823800456853, 1, 1], 
                    Hue[0.6103564710800458, 1, 1], 
                    Hue[0.6103564710800458, 1, 1], 
                    Hue[0.5210960147936234, 1, 1], 
                    Hue[0.5210960147936234, 1, 1], 
                    Hue[0.4318125288607402, 1, 1], 
                    Hue[0.4318125288607402, 1, 1], 
                    Hue[0.34252493459971806`, 1, 1], 
                    Hue[0.34252493459971806`, 1, 1], 
                    Hue[0.25325369870288117`, 1, 1], 
                    Hue[0.25325369870288117`, 1, 1], 
                    Hue[0.16402728053127844`, 1, 1], 
                    Hue[0.16402728053127844`, 1, 1], 
                    Hue[0.07488002620663553, 1, 1], 
                    Hue[0.07488002620663553, 1, 1], 
                    Hue[0.9858520065701789, 1, 1], 
                    Hue[0.9858520065701789, 1, 1], 
                    Hue[0.8969931708519823, 1, 1], 
                    Hue[0.8969931708519823, 1, 1], 
                    Hue[0.8083612816537772, 1, 1], 
                    Hue[0.8083612816537772, 1, 1], 
                    Hue[0.720027846091456, 1, 1], 
                    Hue[0.720027846091456, 1, 1], 
                    Hue[0.6320783020478189, 1, 1], 
                    Hue[0.6320783020478189, 1, 1], 
                    Hue[0.5446126226524881, 1, 1], 
                    Hue[0.5446126226524881, 1, 1], 
                    Hue[0.4577416941256004, 1, 1], 
                    Hue[0.4577416941256004, 1, 1], 
                    Hue[0.371589448811156, 1, 1], 
                    Hue[0.371589448811156, 1, 1], 
                    Hue[0.2862928742918964, 1, 1], 
                    Hue[0.2862928742918964, 1, 1], 
                    Hue[0.20200228023477101`, 1, 1], 
                    Hue[0.20200228023477101`, 1, 1], 
                    Hue[0.11888507280599135`, 1, 1], 
                    Hue[0.11888507280599135`, 1, 1], 
                    Hue[0.037127910830192574`, 1, 1], 
                    Hue[0.037127910830192574`, 1, 1], 
                    Hue[0.9569362186420415, 1, 1], 
                    Hue[0.9569362186420415, 1, 1], 
                    Hue[0.8785305257889706, 1, 1], 
                    Hue[0.8785305257889706, 1, 1], 
                    Hue[0.8021410588218201, 1, 1], 
                    Hue[0.8021410588218201, 1, 1], 
                    Hue[0.7279945898245506, 1, 1], 
                    Hue[0.7279945898245506, 1, 1], 
                    Hue[0.656309101852733, 1, 1], 
                    Hue[0.656309101852733, 1, 1], 
                    Hue[0.5872863814003371, 1, 1], 
                    Hue[0.5872863814003371, 1, 1], 
                    Hue[0.5211049841628967, 1, 1], 
                    Hue[0.5211049841628967, 1, 1], 
                    Hue[0.4579218320357416, 1, 1], 
                    Hue[0.4579218320357416, 1, 1], 
                    Hue[0.3978644395073952, 1, 1], 
                    Hue[0.3978644395073952, 1, 1], 
                    Hue[0.34103635547553546`, 1, 1], 
                    Hue[0.34103635547553546`, 1, 1], 
                    Hue[0.28751299833736366`, 1, 1], 
                    Hue[0.28751299833736366`, 1, 1], 
                    Hue[0.2373425683885671, 1, 1], 
                    Hue[0.2373425683885671, 1, 1], 
                    Hue[0.19054743917487552`, 1, 1], 
                    Hue[0.19054743917487552`, 1, 1], 
                    Hue[0.14712457912982915`, 1, 1], 
                    Hue[0.14712457912982915`, 1, 1], 
                    Hue[0.10704787791664298`, 1, 1], 
                    Hue[0.10704787791664298`, 1, 1], 
                    Hue[0.07027258996258669, 1, 1], 
                    Hue[0.07027258996258669, 1, 1], 
                    Hue[0.036734633034547576`, 1, 1], 
                    Hue[0.036734633034547576`, 1, 1], 
                    Hue[0.0063562804960939565`, 1, 1], 
                    Hue[0.0063562804960939565`, 1, 1], 
                    Hue[0.9790486019005894, 1, 1], 
                    Hue[0.9790486019005894, 1, 1], 
                    Hue[0.954714129564768, 1, 1], 
                    Hue[0.954714129564768, 1, 1], 
                    Hue[0.9332507821720971, 1, 1], 
                    Hue[0.9332507821720971, 1, 1], 
                    Hue[0.9145521366166071, 1, 1], 
                    Hue[0.9145521366166071, 1, 1], 
                    Hue[0.8985108022332381, 1, 1], 
                    Hue[0.8985108022332381, 1, 1], 
                    Hue[0.885019537265343, 1, 1], 
                    Hue[0.885019537265343, 1, 1], 
                    Hue[0.8739740784029262, 1, 1], 
                    Hue[0.8739740784029262, 1, 1], 
                    Hue[0.8652717100434656, 1, 1], 
                    Hue[0.8652717100434656, 1, 1], 
                    Hue[0.8588139578755996, 1, 1], 
                    Hue[0.8588139578755996, 1, 1], 
                    Hue[0.8545070219354367, 1, 1], 
                    Hue[0.8545070219354367, 1, 1], 
                    Hue[0.8522605424068836, 1, 1], 
                    Hue[0.8522605424068836, 1, 1], 
                    Hue[0.8519894958550597, 1, 1], 
                    Hue[0.8519894958550597, 1, 1], 
                    Hue[0.8536135359957169, 1, 1], 
                    Hue[0.8536135359957169, 1, 1], 
                    Hue[0.8570565069011515, 1, 1], 
                    Hue[0.8570565069011515, 1, 1], 
                    Hue[0.8622480974679313, 1, 1], 
                    Hue[0.8622480974679313, 1, 1], 
                    Hue[0.8691216861664274, 1, 1], 
                    Hue[0.8691216861664274, 1, 1], 
                    Hue[0.877614308802046, 1, 1], 
                    Hue[0.877614308802046, 1, 1], 
                    Hue[0.8876687311186671, 1, 1], 
                    Hue[0.8876687311186671, 1, 1], 
                    Hue[0.899230510278553, 1, 1], 
                    Hue[0.899230510278553, 1, 1], 
                    Hue[0.9122490135588197, 1, 1], 
                    Hue[0.9122490135588197, 1, 1], 
                    Hue[0.9266773800415274, 1, 1], 
                    Hue[0.9266773800415274, 1, 1], 
                    Hue[0.9424715311467944, 1, 1], 
                    Hue[0.9424715311467944, 1, 1], 
                    Hue[0.9595905066764843, 1, 1], 
                    Hue[0.9595905066764843, 1, 1], 
                    Hue[0.9779961683060914, 1, 1], 
                    Hue[0.9779961683060914, 1, 1], 
                    Hue[0.9976531263524046, 1, 1], 
                    Hue[0.9976531263524046, 1, 1], 
                    Hue[0.018527973642258822`, 1, 1], 
                    Hue[0.018527973642258822`, 1, 1], 
                    Hue[0.040589846530470396`, 1, 1], 
                    Hue[0.040589846530470396`, 1, 1], 
                    Hue[0.06380921313635418, 1, 1], 
                    Hue[0.06380921313635418, 1, 1], 
                    Hue[0.088159456147119, 1, 1], 
                    Hue[0.088159456147119, 1, 1], 
                    Hue[0.11361500882265167`, 1, 1], 
                    Hue[0.11361500882265167`, 1, 1], 
                    Hue[0.14015184759702176`, 1, 1], 
                    Hue[0.14015184759702176`, 1, 1], 
                    Hue[0.1677470050651769, 1, 1], 
                    Hue[0.1677470050651769, 1, 1], 
                    Hue[0.1963805235167193, 1, 1], 
                    Hue[0.1963805235167193, 1, 1], 
                    Hue[0.226032063227786, 1, 1], 
                    Hue[0.226032063227786, 1, 1], 
                    Hue[0.2566825269000016, 1, 1], 
                    Hue[0.2566825269000016, 1, 1], 
                    Hue[0.2883143142722514, 1, 1], 
                    Hue[0.2883143142722514, 1, 1], 
                    Hue[0.32091103841903484`, 1, 1], 
                    Hue[0.32091103841903484`, 1, 1], 
                    Hue[0.3544570383872956, 1, 1], 
                    Hue[0.3544570383872956, 1, 1], 
                    Hue[0.3889373946660122, 1, 1], 
                    Hue[0.3889373946660122, 1, 1], 
                    Hue[0.42433703731545136`, 1, 1], 
                    Hue[0.42433703731545136`, 1, 1], 
                    Hue[0.4606438306174138, 1, 1], 
                    Hue[0.4606438306174138, 1, 1], 
                    Hue[0.49784493890218584`, 1, 1], 
                    Hue[0.49784493890218584`, 1, 1], 
                    Hue[0.535927662459045, 1, 1], 
                    Hue[0.535927662459045, 1, 1], 
                    Hue[0.5748807416320714, 1, 1], 
                    Hue[0.5748807416320714, 1, 1], 
                    Hue[0.6146930444978822, 1, 1], 
                    Hue[0.6146930444978822, 1, 1], 
                    Hue[0.6553538884368986, 1, 1], 
                    Hue[0.6553538884368986, 1, 1], 
                    Hue[0.696853218504837, 1, 1], 
                    Hue[0.696853218504837, 1, 1], 
                    Hue[0.7391814889795953, 1, 1], 
                    Hue[0.7391814889795953, 1, 1], 
                    Hue[0.7823289599944764, 1, 1], 
                    Hue[0.7823289599944764, 1, 1], 
                    Hue[0.8262874078226671, 1, 1], 
                    Hue[0.8262874078226671, 1, 1], 
                    Hue[0.8710478798485146, 1, 1], 
                    Hue[0.8710478798485146, 1, 1], 
                    Hue[0.9166034157050914, 1, 1], 
                    Hue[0.9166034157050914, 1, 1], 
                    Hue[0.9629448223502752, 1, 1], 
                    Hue[0.9629448223502752, 1, 1], 
                    Hue[0.01006559471412739, 1, 1], 
                    Hue[0.01006559471412739, 1, 1], 
                    Hue[0.05795832163985246, 1, 1], 
                    Hue[0.05795832163985246, 1, 1], 
                    Hue[0.10661432626890296`, 1, 1], 
                    Hue[0.10661432626890296`, 1, 1], 
                    Hue[0.1560288107312348, 1, 1], 
                    Hue[0.1560288107312348, 1, 1], 
                    Hue[0.2061934463881734, 1, 1], 
                    Hue[0.2061934463881734, 1, 1], 
                    Hue[0.25710371868991605`, 1, 1], 
                    Hue[0.25710371868991605`, 1, 1], 
                    Hue[0.30875258049480325`, 1, 1], 
                    Hue[0.30875258049480325`, 1, 1], 
                    Hue[0.3611346594763807, 1, 1], 
                    Hue[0.3611346594763807, 1, 1], 
                    Hue[0.4142443931279519, 1, 1], 
                    Hue[0.4142443931279519, 1, 1], 
                    Hue[0.46807592250982194`, 1, 1], 
                    Hue[0.46807592250982194`, 1, 1], 
                    Hue[0.5226235169268131, 1, 1], 
                    Hue[0.5226235169268131, 1, 1], 
                    Hue[0.5778827231313868, 1, 1], 
                    Hue[0.5778827231313868, 1, 1], 
                    Hue[0.6338467284901352, 1, 1], 
                    Hue[0.6338467284901352, 1, 1], 
                    Hue[0.6905119716638414, 1, 1], 
                    Hue[0.6905119716638414, 1, 1], 
                    Hue[0.7478729708429508, 1, 1], 
                    Hue[0.7478729708429508, 1, 1], 
                    Hue[0.8059262182338722, 1, 1], 
                    Hue[0.8059262182338722, 1, 1], 
                    Hue[0.8646667029288123, 1, 1], 
                    Hue[0.8646667029288123, 1, 1], 
                    Hue[0.9240900148564036, 1, 1], 
                    Hue[0.9240900148564036, 1, 1], 
                    Hue[0.9841921136405349, 1, 1], 
                    Hue[0.9841921136405349, 1, 1], 
                    Hue[0.044968567409435826`, 1, 1], 
                    Hue[0.044968567409435826`, 1, 1], 
                    Hue[0.1064158884423126, 1, 1], 
                    Hue[0.1064158884423126, 1, 1], 
                    Hue[0.16852978608792193`, 1, 1], 
                    Hue[0.16852978608792193`, 1, 1], 
                    Hue[0.23130554990275484`, 1, 1], 
                    Hue[0.23130554990275484`, 1, 1], 
                    Hue[0.29474084889917324`, 1, 1], 
                    Hue[0.29474084889917324`, 1, 1], 
                    Hue[0.358831588919603, 1, 1], 
                    Hue[0.358831588919603, 1, 1], 
                    Hue[0.423572986863044, 1, 1], 
                    Hue[0.423572986863044, 1, 1], 
                    Hue[0.4889633994506269, 1, 1], 
                    Hue[0.4889633994506269, 1, 1], 
                    Hue[0.5549981042468378, 1, 1], 
                    Hue[0.5549981042468378, 1, 1], 
                    Hue[0.6216756108260031, 1, 1], 
                    Hue[0.6216756108260031, 1, 1], 
                    Hue[0.6889900341282765, 1, 1], 
                    Hue[0.6889900341282765, 1, 1], 
                    Hue[0.7569404375017312, 1, 1], 
                    Hue[0.7569404375017312, 1, 1], 
                    Hue[0.8255219568966075, 1, 1], 
                    Hue[0.8255219568966075, 1, 1], 
                    Hue[0.8947351781930797, 1, 1], 
                    Hue[0.8947351781930797, 1, 1], 
                    Hue[0.9645711467107132, 1, 1], 
                    Hue[0.9645711467107132, 1, 1], 
                    Hue[0.035032200808264795`, 1, 1], 
                    Hue[0.035032200808264795`, 1, 1], 
                    Hue[0.10611430302560833`, 1, 1], 
                    Hue[0.10611430302560833`, 1, 1], 
                    Hue[0.1778127455017217, 1, 1], 
                    Hue[0.1778127455017217, 1, 1], 
                    Hue[0.25012585040377056`, 1, 1], 
                    Hue[0.25012585040377056`, 1, 1], 
                    Hue[0.3230512831048453, 1, 1], 
                    Hue[0.3230512831048453, 1, 1], 
                    Hue[0.39658786037578897`, 1, 1], 
                    Hue[0.39658786037578897`, 1, 1], 
                    Hue[0.47073048283222535`, 1, 1], 
                    Hue[0.47073048283222535`, 1, 1], 
                    Hue[0.5454760758758606, 1, 1], 
                    Hue[0.5454760758758606, 1, 1], 
                    Hue[0.6208268462480444, 1, 1], 
                    Hue[0.6208268462480444, 1, 1], 
                    Hue[0.6967736619070093, 1, 1], 
                    Hue[0.6967736619070093, 1, 1], 
                    Hue[0.7733207693695567, 1, 1], 
                    Hue[0.7733207693695567, 1, 1], 
                    Hue[0.85046436527129, 1, 1], 
                    Hue[0.85046436527129, 1, 1], 
                    Hue[0.9281979578049646, 1, 1], 
                    Hue[0.9281979578049646, 1, 1], 
                    Hue[0.006524208684187778, 1, 1], 
                    Hue[0.006524208684187778, 1, 1], 
                    Hue[0.08543857693114915, 1, 1], 
                    Hue[0.08543857693114915, 1, 1], 
                    Hue[0.1649389112410558, 1, 1], 
                    Hue[0.1649389112410558, 1, 1], 
                    Hue[0.24502025669775915`, 1, 1], 
                    Hue[0.24502025669775915`, 1, 1], 
                    Hue[0.3256855387817807, 1, 1], 
                    Hue[0.3256855387817807, 1, 1], 
                    Hue[0.4069328254093805, 1, 1], 
                    Hue[0.4069328254093805, 1, 1], 
                    Hue[0.48875648074544686`, 1, 1], 
                    Hue[0.48875648074544686`, 1, 1], 
                    Hue[0.5711582052233701, 1, 1], 
                    Hue[0.5711582052233701, 1, 1], 
                    Hue[0.6541335516799798, 1, 1], 
                    Hue[0.6541335516799798, 1, 1], 
                    Hue[0.7376792030777755, 1, 1], 
                    Hue[0.7376792030777755, 1, 1], 
                    Hue[0.8217951224193869, 1, 1], 
                    Hue[0.8217951224193869, 1, 1], 
                    Hue[0.906479842223855, 1, 1], 
                    Hue[0.906479842223855, 1, 1], 
                    Hue[0.9917352697531961, 1, 1], 
                    Hue[0.9917352697531961, 1, 1], 
                    Hue[0.07755478805897177, 1, 1]}],
                   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
                   Axes->{True, None},
                   AxesOrigin->{0, 0},
                   
                   Epilog->LineBox[{{1, 0.000037384552214890954`}, {
                    2, 0.00004402948613875358}, {
                    3, 0.000051761509053314036`}, {
                    4, 0.00006154112652061493}, {5, 0.00007129265584977108}, {
                    6, 0.00008429356487128198}, {7, 0.00009786336683845261}, {
                    8, 0.00011430847100552931`}, {
                    9, 0.00013355977665037894`}, {
                    10, 0.00015538535790489972`}, {
                    11, 0.00018083824857960948`}, {
                    12, 0.00021050258563672333`}, {
                    13, 0.00024321827031647451`}, {
                    14, 0.00028263518192711486`}, {
                    15, 0.00032649003177730835`}, {
                    16, 0.00037718146299153924`}, {
                    17, 0.000435795985409994}, {18, 0.00050232323577247}, {
                    19, 0.0005776155910632372}, {20, 0.0006644539097170476}, {
                    21, 0.0007620135685188869}, {22, 0.0008731886911432147}, {
                    23, 0.0010001350116070125`}, {24, 0.001143123728409559}, {
                    25, 0.0013051216676307573`}, {
                    26, 0.0014880213521412684`}, {
                    27, 0.0016943350635444461`}, {
                    28, 0.0019260706898049165`}, {29, 0.002187562648643401}, {
                    30, 0.002481723571716071}, {31, 0.0028101637290371163`}, {
                    32, 0.0031786688904200102`}, {33, 0.003590314030067674}, {
                    34, 0.0040498708684125985`}, {35, 0.004561420661685151}, {
                    36, 0.005130894265673581}, {37, 0.005762892484561625}, {
                    38, 0.006463654247981446}, {39, 0.007239383930991065}, {
                    40, 0.008097136754030598}, {41, 0.009042375426619398}, {
                    42, 0.010084897585188126`}, {43, 0.011230435006260767`}, {
                    44, 0.012487825541477017`}, {45, 0.013864730135505642`}, {
                    46, 0.015370506867482283`}, {47, 0.01701470814643129}, {
                    48, 0.018805220620043087`}, {49, 0.020753075395326538`}, {
                    50, 0.022866156456796422`}, {51, 0.025157125478979536`}, {
                    52, 0.027632741975517}, {53, 0.03030508902617005}, {
                    54, 0.03318214429475098}, {55, 0.036273892000890616`}, {
                    56, 0.03958943843236792}, {57, 0.043136872914803295`}, {
                    58, 0.046924200150548524`}, {59, 0.050958100744902785`}, {
                    60, 0.05524616600624071}, {61, 0.05979357034383282}, {
                    62, 0.06460545857600497}, {63, 0.06968448109281535}, {
                    64, 0.0750306336272056}, {65, 0.08064676880379408}, {
                    66, 0.08652820335801074}, {67, 0.09267434195367426}, {
                    68, 0.09907724300020347}, {69, 0.10573009546343393`}, {
                    70, 0.11262438392336507`}, {71, 0.11974584688114864`}, {
                    72, 0.12708009217218974`}, {73, 0.1346119798886439}, {
                    74, 0.1423190200126163}, {75, 0.1501826093307986}, {
                    76, 0.15817439298282238`}, {77, 0.16627015680145368`}, {
                    78, 0.17443902310026682`}, {79, 0.18264896287551163`}, {
                    80, 0.19086781133950695`}, {81, 0.19905972762046856`}, {
                    82, 0.20718772486422074`}, {83, 0.21521353571782176`}, {
                    84, 0.2230989797948063}, {85, 0.2308025473616139}, {
                    86, 0.2382846656374238}, {87, 0.24550667239894747`}, {
                    88, 0.25242750091102595`}, {89, 0.2590099116346867}, {
                    90, 0.26521846963637796`}, {91, 0.2710176137304671}, {
                    92, 0.2763757443645039}, {93, 0.28126612769396603`}, {
                    94, 0.2856600497356623}, {95, 0.2895389425361334}, {
                    96, 0.2928859736946418}, {97, 0.295687634002137}, {
                    98, 0.29793733927320293`}, {99, 0.2996329881110879}, {
                    100, 0.300777890812787}, {101, 0.30138186881696644`}, {
                    102, 0.30145863776203535`}, {103, 0.3010310062849546}, {
                    104, 0.3001246185941583}, {105, 0.29876994684973646`}, {
                    106, 0.29700409555525575`}, {107, 0.2948647785484878}, {
                    108, 0.29239590625113954`}, {109, 0.2896487260600834}, {
                    110, 0.2866757332737224}, {111, 0.28353739376526355`}, {
                    112, 0.2802907660208733}, {113, 0.27699704388066265`}, {
                    114, 0.273707514503783}, {115, 0.2704771336942364}, {
                    116, 0.2673619437912488}, {117, 0.26442202824185496`}, {
                    118, 0.2617249807156796}, {119, 0.2593407517771608}, {
                    120, 0.2573297646852573}, {121, 0.2557435848444642}, {
                    122, 0.25461327771510117`}, {123, 0.2539501659222143}, {
                    124, 0.25375136688691413`}, {125, 0.25399999852132193`}, {
                    126, 0.25467875554345276`}, {127, 0.25576625455596436`}, {
                    128, 0.257243947282749}, {129, 0.25909216236266086`}, {
                    130, 0.26129096993058065`}, {131, 0.26382137143434875`}, {
                    132, 0.2666626203609747}, {133, 0.26979376536190497`}, {
                    134, 0.27318882931382676`}, {135, 0.27682364633786005`}, {
                    136, 0.28067007181340226`}, {137, 0.2847010666573463}, {
                    138, 0.2888885257514357}, {139, 0.2932036919363435}, {
                    140, 0.2976202185530137}, {141, 0.302110800305226}, {
                    142, 0.30665090097031883`}, {143, 0.3112136114449349}, {
                    144, 0.315775543719446}, {145, 0.32031390472244775`}, {
                    146, 0.3248075085857351}, {147, 0.3292343138568991}, {
                    148, 0.33357550995829394`}, {149, 0.33781177620868075`}, {
                    150, 0.3419268659251715}, {151, 0.3459030940795283}, {
                    152, 0.3497240536290127}, {153, 0.35337687725593236`}, {
                    154, 0.35684513180557403`}, {155, 0.36011682825784586`}, {
                    156, 0.3631802982693446}, {157, 0.3660233981142989}, {
                    158, 0.3686364060016859}, {159, 0.3710087472278385}, {
                    160, 0.37313154198769394`}, {161, 0.3749957509631288}, {
                    162, 0.3765943588219932}, {163, 0.3779208757816985}, {
                    164, 0.378970122151151}, {165, 0.3797353396675913}, {
                    166, 0.38021419916020566`}, {167, 0.380400824885497}, {
                    168, 0.38029480909763397`}, {169, 0.3798923538121558}, {
                    170, 0.3791930901611081}, {171, 0.37819724537290583`}, {
                    172, 0.3769040474757793}, {173, 0.375315410672962}, {
                    174, 0.37343305455546755`}, {175, 0.3712586038147189}, {
                    176, 0.36879682254070145`}, {177, 0.36605107206463466`}, {
                    178, 0.3630254292273829}, {179, 0.35972695156883566`}, {
                    180, 0.3561614496555175}, {181, 0.3523355653192698}, {
                    182, 0.34825833821609914`}, {183, 0.34393768469083075`}, {
                    184, 0.3393823953274659}, {185, 0.3346015326908261}, {
                    186, 0.32960342093517686`}, {187, 0.32440010159890276`}, {
                    188, 0.31900082162894383`}, {189, 0.31341823271936964`}, {
                    190, 0.3076638373365698}, {191, 0.3017483127538634}, {
                    192, 0.2956867298977733}, {193, 0.28948979342724523`}, {
                    194, 0.2831696854830931}, {195, 0.2767399651114634}, {
                    196, 0.27021151480125527`}, {197, 0.2635982369028219}, {
                    198, 0.25691235246307187`}, {199, 0.25016916076570367`}, {
                    200, 0.243379391922137}, {201, 0.23655790750821795`}, {
                    202, 0.22971530703969972`}, {203, 0.22286324741091776`}, {
                    204, 0.2160137766597309}, {205, 0.20917829308173305`}, {
                    206, 0.20236973400469568`}, {207, 0.19559751050315222`}, {
                    208, 0.18887337060305354`}, {209, 0.18220652420117284`}, {
                    210, 0.1756079779774803}, {211, 0.16908609084408524`}, {
                    212, 0.16264924399885328`}, {213, 0.15630631151119792`}, {
                    214, 0.15006587297945836`}, {215, 0.1439333876300512}, {
                    216, 0.13791722198206383`}, {217, 0.13202273678927795`}, {
                    218, 0.12625657363446724`}, {219, 0.12062195427074902`}, {
                    220, 0.1151250566638951}, {221, 0.10976891453715534`}, {
                    222, 0.10455841317114434`}, {223, 0.09949437718192684}, {
                    224, 0.09457986354538578}, {225, 0.0898187626147869}, {
                    226, 0.08520947027257554}, {227, 0.08075462687615945}, {
                    228, 0.0764546782384582}, {229, 0.07230964461521586}, {
                    230, 0.06831898665297638}, {231, 0.06448096174343011}, {
                    232, 0.06079646515377699}, {233, 0.05726254278424689}, {
                    234, 0.053877517143560594`}, {235, 0.05064064429104906}, {
                    236, 0.047546790386842196`}, {237, 0.04459526631409795}, {
                    238, 0.041782708429035934`}, {239, 0.03910561142264679}, {
                    240, 0.03656073444471586}, {241, 0.034144912485959605`}, {
                    242, 0.031855197659735913`}, {
                    243, 0.029686480997383226`}, {
                    244, 0.027635566299171126`}, {
                    245, 0.025698430074535802`}, {
                    246, 0.023870612213392872`}, {
                    247, 0.022148636911160226`}, {
                    248, 0.020528428487954296`}, {
                    249, 0.019005997000147268`}, {
                    250, 0.017576959858977854`}, {
                    251, 0.016236802368262523`}}],
                   Frame->True,
                   FrameTicks->{{{1., 
                    FormBox[
                    RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
                    FormBox[
                    RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
                    FormBox[
                    RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
                    FormBox[
                    RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
                    FormBox["0", TraditionalForm]}, {157.25, 
                    FormBox["2.5`", TraditionalForm]}, {188.5, 
                    FormBox["5", TraditionalForm]}, {219.75, 
                    FormBox["7.5`", TraditionalForm]}, {251., 
                    FormBox["10", TraditionalForm]}}, Automatic, None, None},
                   PlotRange->{-0.1, 1.1},
                   PlotRangeClipping->True,
                   PlotRangePadding->{
                    Scaled[0.02], Automatic}], 22->
                  GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF13tcjncfB/AfT8jksMwhpy45LDkmKaf6UJIi6Xygrrvuznfl0ci2x7ps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                    "], {
                    {GrayLevel[0.5], EdgeForm[None], 
                    GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ
2qYUCakkqbSrJBXatNLyfM7n/eM55zkzc2fud+be9/dW7t6/Tb8cEX/DAZBg
P/wDGi8KxRg8EIrhhcKLUvL6worDC0ORgph/4b+CtRorwlzRVLAXFMdLaE+8
FByKHwYHh8dK4gdB8YKcMl4o+awSUBovAyXDlMUPUUyYcnj55L2bwkB8EDQL
n1EBPxxK42XgSPwoKBumEn609lRuUB0/TjmGqYEfDxXw8lAVr5a8VjHH4JWh
XPgbSunbknNXTE28lnLQ2VAfP0k54BWhLn4CHBHOsaI8OVfF1MbrJK9VTAP8
ZKgUjqmHn5i8t3I4Fq+SnKvWnIKfqm9UrtAYP00541XgDPxMqIpXg7Pxc6B6
eOwsvHnyXB24DG8LdXUWNMNPT95LMefi5+kOw7TAz4caeC24GL8Eaof3aIdf
rjsIj7XBL00+SzEt8Qv0BmFa4a2hZjimPd4B6oXpiHfSneCN4Fr8Omisu4Ce
eC/dSXhNZ/wKqB+mC95VbxTmSrwbNAjTHb9KbxC+04Z4o+S71VgP/Orks3TH
TfCmyXfVEPri1yTnphx6432S55RjP7y/3ijM9fgN0CTMAPzG5Fr/C3bRhLuz
e1o1P5i5IXoTvS0Mw2/WG+ttYBw+Xm+kt4UR+K16Y709jMJHqwbCMcPxW5LX
amwMPjZ5L+0xAb8dWuCt4W78HrgwPDYRn6QaCJ9xGz4y+ayWMAW/U28cZip+
F7QKMw2fnry3vukmfGjyt+mMGfi9cJFqC2bh96vGwszGH4A2eFt4FH8M2oV5
HH9CNajahrn4I8lrFTMHf1A1Gf6GyfgdybkrZh7+JLTHO8EifDF0xjvAAvxp
6BjOcSZ+X3KuipmPP5W8VjFL8KWqyXDMQvyZ5L01tgxfDl3CPIs/p5oN8zz+
Alyp2ofN+HvqAbwbvIS/DN3DrMRfUU2HeRVfBT3CrMZfU02H99iCvw/9wmNr
8NehJ94L1uNvQG+8D7yFvw19w2vW4uuS12psI/6OeiIcswF/MzlWY5vwd5O/
RT1/od46+W+JcvgA/1A9o9qA7/EfVCP4INiOfwOD1UvwMf6JeirMp/hnMCDM
5/gX6rEw2/AvYWD4jK34R8l7aewr/Ovks3TnL+Irku96COzEv0vOTTnswL9N
nlOOu/DdMCzMHvxH9WyYn/CfYXiYX/Bf1ZNh9uK/wYgwv+N/qKfxkbAf/wdG
4aNBP9oJxoTH/mXuv+S5SVCauTIwWb0KfzO3L3kvxWTmCsHYMAfghWEcfjuU
xA+CieE9yuKHwB3hsVL4wdlnKaYIXhTGhymGHwgTwjHl8PIwJcyh+GFwJ34v
1MCPh5n43XAsXgXuCa+pgB8OU8NUxI+Au8IciR8F08JUwo+G6eEciuMlsr9N
Y8fglbPPmg318ZPgAXwGVMePy85NOVTFq2XPKceaeC24L0xtvA7MClMXPwHu
D1MPPzH7LJ3RAD8Z5qjW1AN4f1iBz1d/4BfBU/iD0BBvBA+FaYyfBg+HaYI3
hblhmuGnwyPhPS7GL4EF4bEz8DPhUfwxOBs/Bx7Hn4AW+PkwL7zmLLx59lqN
tcQvgCfDMefi52XHaqwV3jr7W/RN1+M3wEvhHNrgl8LT+LPQE+8Fz+FL1YN4
N1iGL4R2+OXwTJj2eAdYFKYj3gkWh+mMXwFLwmdchrfN3ktjXfCu2WfpDU7B
T82+6+XQA786Ozfl0B2/KntOOfbG+8DzYfri18ALYa7Fr8t+W/3mPETvPZz8
26Q7GMDcjfAyvgPW4uvgW/wVuAkfCq/iq2A4fgusDo8Nw2/OnnsNbsNHwhp8
JQzGh2TvpZgR+K3Za7VmFD4aXsfXqUfUG7A+/DfnT3L9K/lvxxswWb0PG8JM
Ue/Cm+GYifik7LUam6rehLfwt9XT6mXYiH+gmlGtwIf4e7of1TJsCa+Zod6D
d8LMVK/BpjCz1FvwbpjZ6l3YHP6mMfhYWBsem6O3zT5LOU3Dp2fn9r5qXLWd
nZtymKveyZ5TjvNU67A1zHz1JnwUZoFqGT4OnzkOH599txpbqNqFT8IsUq3C
p/gXqiHVDmzDv1OOyg2+x79UzahW4Kvw2EbdDfwQjnkRX5G9VmObdDewC/9c
Pabeyj7ra9WQage2h2tuvd4OdobPWKnayV6rsQ166+zcFLNatQTfhFmjWsqu
ZdX0QHxQdi0qh826e9iN79UZ2ht+w/eoJlQL8CP+i3JWrvAr/pPuUHcHP4fv
bIl6Fz4Lx2zVW2Sv1Z7blRv8Hj5zi94y+yztuU13lZ2L1uxQ78EfUXCG9s7O
5U/due46+391nbkMX559t4rZqbvJXqv/4ffoW2Af/j8egu84
                    
                    "]]]}, {{}, {}, {
                    {Hue[0.45492651067977985`, 1, 1], LineBox[{1, 2}]}, 
                    {Hue[0.35453989828871174`, 1, 1], LineBox[{2, 3}]}, 
                    {Hue[0.25371662554201285`, 1, 1], LineBox[{3, 4}]}, 
                    {Hue[0.153334251777296, 1, 1], LineBox[{4, 5}]}, 
                    {Hue[0.05307399626199472, 1, 1], LineBox[{5, 6}]}, 
                    {Hue[0.95363236378411, 1, 1], LineBox[{6, 7}]}, 
                    {Hue[0.8541318403485163, 1, 1], LineBox[{7, 8}]}, 
                    {Hue[0.7547295042198715, 1, 1], LineBox[{8, 9}]}, 
                    {Hue[0.6559085725275098, 1, 1], LineBox[{9, 10}]}, 
                    {Hue[0.5571483343007815, 1, 1], LineBox[{10, 11}]}, 
                    {Hue[0.4584138934345222, 1, 1], LineBox[{11, 12}]}, 
                    {Hue[0.36039912597627266`, 1, 1], LineBox[{12, 13}]}, 
                    {Hue[0.2622953759685387, 1, 1], LineBox[{13, 14}]}, 
                    {Hue[0.16436459436553116`, 1, 1], LineBox[{14, 15}]}, 
                    {Hue[0.06686467199637351, 1, 1], LineBox[{15, 16}]}, 
                    {Hue[0.9695170994455039, 1, 1], LineBox[{16, 17}]}, 
                    {Hue[0.8723303846594687, 1, 1], LineBox[{17, 18}]}, 
                    {Hue[0.7754090552434305, 1, 1], LineBox[{18, 19}]}, 
                    {Hue[0.67874465847557, 1, 1], LineBox[{19, 20}]}, 
                    {Hue[0.582229927875765, 1, 1], LineBox[{20, 21}]}, 
                    {Hue[0.4859598807852869, 1, 1], LineBox[{21, 22}]}, 
                    {Hue[0.38987575474619246`, 1, 1], LineBox[{22, 23}]}, 
                    {Hue[0.2940365317975417, 1, 1], LineBox[{23, 24}]}, 
                    {Hue[0.19836684198245164`, 1, 1], LineBox[{24, 25}]}, 
                    {Hue[0.10294494847248514`, 1, 1], LineBox[{25, 26}]}, 
                    {Hue[0.007707782375784896, 1, 1], LineBox[{26, 27}]}, 
                    {Hue[0.9126637472741359, 1, 1], LineBox[{27, 28}]}, 
                    {Hue[0.8178222771749548, 1, 1], LineBox[{28, 29}]}, 
                    {Hue[0.72318127803347, 1, 1], LineBox[{29, 30}]}, 
                    {Hue[0.6287338651278908, 1, 1], LineBox[{30, 31}]}, 
                    {Hue[0.5344701180581571, 1, 1], LineBox[{31, 32}]}, 
                    {Hue[0.44039426020930045`, 1, 1], LineBox[{32, 33}]}, 
                    {Hue[0.3465144635884028, 1, 1], LineBox[{33, 34}]}, 
                    {Hue[0.252808072997363, 1, 1], LineBox[{34, 35}]}, 
                    {Hue[0.15928966223813107`, 1, 1], LineBox[{35, 36}]}, 
                    {Hue[0.06594733545312544, 1, 1], LineBox[{36, 37}]}, 
                    {Hue[0.9727848443406211, 1, 1], LineBox[{37, 38}]}, 
                    {Hue[0.8797973542288998, 1, 1], LineBox[{38, 39}]}, 
                    {Hue[0.7869821562937429, 1, 1], LineBox[{39, 40}]}, 
                    {Hue[0.6943399395807146, 1, 1], LineBox[{40, 41}]}, 
                    {Hue[0.6018559509399378, 1, 1], LineBox[{41, 42}]}, 
                    {Hue[0.5095393776462689, 1, 1], LineBox[{42, 43}]}, 
                    {Hue[0.4173796678168414, 1, 1], LineBox[{43, 44}]}, 
                    {Hue[0.32538385171789214`, 1, 1], LineBox[{44, 45}]}, 
                    {Hue[0.23354205356651353`, 1, 1], LineBox[{45, 46}]}, 
                    {Hue[0.14185471410378753`, 1, 1], LineBox[{46, 47}]}, 
                    {Hue[0.05031447265582115, 1, 1], LineBox[{47, 48}]}, 
                    {Hue[0.958923662715511, 1, 1], LineBox[{48, 49}]}, 
                    {Hue[0.8676772117323288, 1, 1], LineBox[{49, 50}]}, 
                    {Hue[0.7765680700686013, 1, 1], LineBox[{50, 51}]}, 
                    {Hue[0.6856019387109209, 1, 1], LineBox[{51, 52}]}, 
                    {Hue[0.5947691460253403, 1, 1], LineBox[{52, 53}]}, 
                    {Hue[0.5040652266289894, 1, 1], LineBox[{53, 54}]}, 
                    {Hue[0.4134896901706374, 1, 1], LineBox[{54, 55}]}, 
                    {Hue[0.3230371814924714, 1, 1], LineBox[{55, 56}]}, 
                    {Hue[0.2327048098212888, 1, 1], LineBox[{56, 57}]}, 
                    {Hue[0.14248935709303306`, 1, 1], LineBox[{57, 58}]}, 
                    {Hue[0.05238415176542811, 1, 1], LineBox[{58, 59}]}, 
                    {Hue[0.9623891562027246, 1, 1], LineBox[{59, 60}]}, 
                    {Hue[0.8724970950084328, 1, 1], LineBox[{60, 61}]}, 
                    {Hue[0.782705234830449, 1, 1], LineBox[{61, 62}]}, 
                    {Hue[0.6930094276719068, 1, 1], LineBox[{62, 63}]}, 
                    {Hue[0.603403558095029, 1, 1], LineBox[{63, 64}]}, 
                    {Hue[0.5138848451851199, 1, 1], LineBox[{64, 65}]}, 
                    {Hue[0.4244454572392602, 1, 1], LineBox[{65, 66}]}, 
                    {Hue[0.3350842173811413, 1, 1], LineBox[{66, 67}]}, 
                    {Hue[0.24579166251417273`, 1, 1], LineBox[{67, 68}]}, 
                    {Hue[0.15656623339073467`, 1, 1], LineBox[{68, 69}]}, 
                    {Hue[0.06739962559988893, 1, 1], LineBox[{69, 70}]}, 
                    {Hue[0.9782875994973265, 1, 1], LineBox[{70, 71}]}, 
                    {Hue[0.8892234142617812, 1, 1], LineBox[{71, 72}]}, 
                    {Hue[0.8002011485261, 1, 1], LineBox[{72, 73}]}, 
                    {Hue[0.7112140191474132, 1, 1], LineBox[{73, 74}]}, 
                    {Hue[0.6222542253030292, 1, 1], LineBox[{74, 75}]}, 
                    {Hue[0.5333177457842979, 1, 1], LineBox[{75, 76}]}, 
                    {Hue[0.44439506979053767`, 1, 1], LineBox[{76, 77}]}, 
                    {Hue[0.3554794286460181, 1, 1], LineBox[{77, 78}]}, 
                    {Hue[0.26656299622171675`, 1, 1], LineBox[{78, 79}]}, 
                    {Hue[0.17763793048842322`, 1, 1], LineBox[{79, 80}]}, 
                    {Hue[0.08869610636692972, 1, 1], LineBox[{80, 81}]}, 
                    {Hue[0.9997283819955648, 1, 1], LineBox[{81, 82}]}, 
                    {Hue[0.9107265702305547, 1, 1], LineBox[{82, 83}]}, 
                    {Hue[0.821681754524422, 1, 1], LineBox[{83, 84}]}, 
                    {Hue[0.7325848999670737, 1, 1], LineBox[{84, 85}]}, 
                    {Hue[0.6434256600246612, 1, 1], LineBox[{85, 86}]}, 
                    {Hue[0.5541954241832242, 1, 1], LineBox[{86, 87}]}, 
                    {Hue[0.46488295327061496`, 1, 1], LineBox[{87, 88}]}, 
                    {Hue[0.3754802808713916, 1, 1], LineBox[{88, 89}]}, 
                    {Hue[0.28597636002231513`, 1, 1], LineBox[{89, 90}]}, 
                    {Hue[0.1963625454614133, 1, 1], LineBox[{90, 91}]}, 
                    {Hue[0.10662868654545397`, 1, 1], LineBox[{91, 92}]}, 
                    {Hue[0.016766218274584323`, 1, 1], LineBox[{92, 93}]}, 
                    {Hue[0.9267666720232012, 1, 1], LineBox[{93, 94}]}, 
                    {Hue[0.8366209546576345, 1, 1], LineBox[{94, 95}]}, 
                    {Hue[0.7463234324270356, 1, 1], LineBox[{95, 96}]}, 
                    {Hue[0.6558683946811299, 1, 1], LineBox[{96, 97}]}, 
                    {Hue[0.5652519424005065, 1, 1], LineBox[{97, 98}]}, 
                    {Hue[0.4744728032182932, 1, 1], LineBox[{98, 99}]}, 
                    {Hue[0.38353075616650895`, 1, 1], LineBox[{99, 100}]}, 
                    {Hue[0.29242972469312417`, 1, 1], LineBox[{100, 101}]}, 
                    {Hue[0.20117765285064337`, 1, 1], LineBox[{101, 102}]}, 
                    {Hue[0.1097862900027622, 1, 1], LineBox[{102, 103}]}, 
                    {Hue[0.01827287832674748, 1, 1], LineBox[{103, 104}]}, 
                    {Hue[0.9266611974571188, 1, 1], LineBox[{104, 105}]}, 
                    {Hue[0.8349800174043959, 1, 1], LineBox[{105, 106}]}, 
                    {Hue[0.7432670997335856, 1, 1], LineBox[{106, 107}]}, 
                    {Hue[0.6515686779597296, 1, 1], LineBox[{107, 108}]}, 
                    {Hue[0.5599424694431372, 1, 1], LineBox[{108, 109}]}, 
                    {Hue[0.46845481596904104`, 1, 1], LineBox[{109, 110}]}, 
                    {Hue[0.3771862786362149, 1, 1], LineBox[{110, 111}]}, 
                    {Hue[0.2862282662686505, 1, 1], LineBox[{111, 112}]}, 
                    {Hue[0.19568641537022163`, 1, 1], LineBox[{112, 113}]}, 
                    {Hue[0.10567797607262314`, 1, 1], LineBox[{113, 114}]}, 
                    {Hue[0.016332731301606014`, 1, 1], LineBox[{114, 115}]}, 
                    {Hue[0.927795591376938, 1, 1], LineBox[{115, 116}]}, 
                    {Hue[0.8402276070843839, 1, 1], LineBox[{116, 117}]}, 
                    {Hue[0.7538035231069398, 1, 1], LineBox[{117, 118}]}, 
                    {Hue[0.6687130433992411, 1, 1], LineBox[{118, 119}]}, 
                    {Hue[0.5851522832589728, 1, 1], LineBox[{119, 120}]}, 
                    {Hue[0.5033241434569828, 1, 1], LineBox[{120, 121}]}, 
                    {Hue[0.42343048382940907`, 1, 1], LineBox[{121, 122}]}, 
                    {Hue[0.3456705554356446, 1, 1], LineBox[{122, 123}]}, 
                    {Hue[0.27023803792139145`, 1, 1], LineBox[{123, 124}]}, 
                    {Hue[0.19731503301541775`, 1, 1], LineBox[{124, 125}]}, 
                    {Hue[0.1270771087857017, 1, 1], LineBox[{125, 126}]}, 
                    {Hue[0.05968324559803208, 1, 1], LineBox[{126, 127}]}, 
                    {Hue[0.9952806746680052, 1, 1], LineBox[{127, 128}]}, 
                    {Hue[0.9339888537835502, 1, 1], LineBox[{128, 129}]}, 
                    {Hue[0.8759098972504199, 1, 1], LineBox[{129, 130}]}, 
                    {Hue[0.8211168140301675, 1, 1], LineBox[{130, 131}]}, 
                    {Hue[0.769657595305148, 1, 1], LineBox[{131, 132}]}, 
                    {Hue[0.72155306419796, 1, 1], LineBox[{132, 133}]}, 
                    {Hue[0.6767989892674167, 1, 1], LineBox[{133, 134}]}, 
                    {Hue[0.6353668692228582, 1, 1], LineBox[{134, 135}]}, 
                    {Hue[0.5972087793115762, 1, 1], LineBox[{135, 136}]}, 
                    {Hue[0.5622604326917748, 1, 1], LineBox[{136, 137}]}, 
                    {Hue[0.5304403852817042, 1, 1], LineBox[{137, 138}]}, 
                    {Hue[0.501658519810958, 1, 1], LineBox[{138, 139}]}, 
                    {Hue[0.47581545273971154`, 1, 1], LineBox[{139, 140}]}, 
                    {Hue[0.45280660192802846`, 1, 1], LineBox[{140, 141}]}, 
                    {Hue[0.4325244262555183, 1, 1], LineBox[{141, 142}]}, 
                    {Hue[0.4148607958877386, 1, 1], LineBox[{142, 143}]}, 
                    {Hue[0.3997064885717363, 1, 1], LineBox[{143, 144}]}, 
                    {Hue[0.3869563679568564, 1, 1], LineBox[{144, 145}]}, 
                    {Hue[0.37650730231757107`, 1, 1], LineBox[{145, 146}]}, 
                    {Hue[0.3682607261039661, 1, 1], LineBox[{146, 147}]}, 
                    {Hue[0.3621209982291931, 1, 1], LineBox[{147, 148}]}, 
                    {Hue[0.3579978426514018, 1, 1], LineBox[{148, 149}]}, 
                    {Hue[0.35580683697359283`, 1, 1], LineBox[{149, 150}]}, 
                    {Hue[0.3554662759685706, 1, 1], LineBox[{150, 151}]}, 
                    {Hue[0.3569013213821672, 1, 1], LineBox[{151, 152}]}, 
                    {Hue[0.36004081361650353`, 1, 1], LineBox[{152, 153}]}, 
                    {Hue[0.3648181761077391, 1, 1], LineBox[{153, 154}]}, 
                    {Hue[0.37117149197985105`, 1, 1], LineBox[{154, 155}]}, 
                    {Hue[0.37904337644688246`, 1, 1], LineBox[{155, 156}]}, 
                    {Hue[0.38838003033941537`, 1, 1], LineBox[{156, 157}]}, 
                    {Hue[0.39913105246117875`, 1, 1], LineBox[{157, 158}]}, 
                    {Hue[0.411249920977416, 1, 1], LineBox[{158, 159}]}, 
                    {Hue[0.424693391058465, 1, 1], LineBox[{159, 160}]}, 
                    {Hue[0.4394211657571874, 1, 1], LineBox[{160, 161}]}, 
                    {Hue[0.45539537047986617`, 1, 1], LineBox[{161, 162}]}, 
                    {Hue[0.47258130536614373`, 1, 1], LineBox[{162, 163}]}, 
                    {Hue[0.4909467062963725, 1, 1], LineBox[{163, 164}]}, 
                    {Hue[0.5104604752212907, 1, 1], LineBox[{164, 165}]}, 
                    {Hue[0.5310943454416712, 1, 1], LineBox[{165, 166}]}, 
                    {Hue[0.552822658366199, 1, 1], LineBox[{166, 167}]}, 
                    {Hue[0.5756200347081709, 1, 1], LineBox[{167, 168}]}, 
                    {Hue[0.5994628887166491, 1, 1], LineBox[{168, 169}]}, 
                    {Hue[0.6243296837994351, 1, 1], LineBox[{169, 170}]}, 
                    {Hue[0.6502004900681735, 1, 1], LineBox[{170, 171}]}, 
                    {Hue[0.6770556375018364, 1, 1], LineBox[{171, 172}]}, 
                    {Hue[0.7048773963672461, 1, 1], LineBox[{172, 173}]}, 
                    {Hue[0.7336477827191039, 1, 1], LineBox[{173, 174}]}, 
                    {Hue[0.7633516997074754, 1, 1], LineBox[{174, 175}]}, 
                    {Hue[0.7939737135007793, 1, 1], LineBox[{175, 176}]}, 
                    {Hue[0.8254995106024579, 1, 1], LineBox[{176, 177}]}, 
                    {Hue[0.8579156762552613, 1, 1], LineBox[{177, 178}]}, 
                    {Hue[0.8912095063230834, 1, 1], LineBox[{178, 179}]}, 
                    {Hue[0.925368717798873, 1, 1], LineBox[{179, 180}]}, 
                    {Hue[0.9603824546877948, 1, 1], LineBox[{180, 181}]}, 
                    {Hue[0.9962385682294179, 1, 1], LineBox[{181, 182}]}, 
                    {Hue[0.03292654017297939, 1, 1], LineBox[{182, 183}]}, 
                    {Hue[0.07043714394267478, 1, 1], LineBox[{183, 184}]}, 
                    {Hue[0.10875968330459024`, 1, 1], LineBox[{184, 185}]}, 
                    {Hue[0.14788561304282966`, 1, 1], LineBox[{185, 186}]}, 
                    {Hue[0.18780615362640285`, 1, 1], LineBox[{186, 187}]}, 
                    {Hue[0.22851270426524767`, 1, 1], LineBox[{187, 188}]}, 
                    {Hue[0.26999801955043884`, 1, 1], LineBox[{188, 189}]}, 
                    {Hue[0.3122539822073713, 1, 1], LineBox[{189, 190}]}, 
                    {Hue[0.35527287615958714`, 1, 1], LineBox[{190, 191}]}, 
                    {Hue[0.39904799194802826`, 1, 1], LineBox[{191, 192}]}, 
                    {Hue[0.44357149723895656`, 1, 1], LineBox[{192, 193}]}, 
                    {Hue[0.48883781441948254`, 1, 1], LineBox[{193, 194}]}, 
                    {Hue[0.534839279152306, 1, 1], LineBox[{194, 195}]}, 
                    {Hue[0.5815702894260358, 1, 1], LineBox[{195, 196}]}, 
                    {Hue[0.6290256025850298, 1, 1], LineBox[{196, 197}]}, 
                    {Hue[0.6771988392967269, 1, 1], LineBox[{197, 198}]}, 
                    {Hue[0.7260852621317265, 1, 1], LineBox[{198, 199}]}, 
                    {Hue[0.7756785902064173, 1, 1], LineBox[{199, 200}]}, 
                    {Hue[0.8259737909039171, 1, 1], LineBox[{200, 201}]}, 
                    {Hue[0.8769657593370601, 1, 1], LineBox[{201, 202}]}, 
                    {Hue[0.9286486911989955, 1, 1], LineBox[{202, 203}]}, 
                    {Hue[0.9810188386237015, 1, 1], LineBox[{203, 204}]}, 
                    {Hue[0.034069971038059345`, 1, 1], LineBox[{204, 205}]}, 
                    {Hue[0.08779934893459017, 1, 1], LineBox[{205, 206}]}, 
                    {Hue[0.1422018167527364, 1, 1], LineBox[{206, 207}]}, 
                    {Hue[0.19727378340379548`, 1, 1], LineBox[{207, 208}]}, 
                    {Hue[0.2530108487712756, 1, 1], LineBox[{208, 209}]}, 
                    {Hue[0.30940900332769283`, 1, 1], LineBox[{209, 210}]}, 
                    {Hue[0.3664635294786629, 1, 1], LineBox[{210, 211}]}, 
                    {Hue[0.42417153233580135`, 1, 1], LineBox[{211, 212}]}, 
                    {Hue[0.48252852701740456`, 1, 1], LineBox[{212, 213}]}, 
                    {Hue[0.5415311942928878, 1, 1], LineBox[{213, 214}]}, 
                    {Hue[0.6011771487708214, 1, 1], LineBox[{214, 215}]}, 
                    {Hue[0.6614611660479162, 1, 1], LineBox[{215, 216}]}, 
                    {Hue[0.7223807452582915, 1, 1], LineBox[{216, 217}]}, 
                    {Hue[0.7839319540579166, 1, 1], LineBox[{217, 218}]}, 
                    {Hue[0.8461120801534268, 1, 1], LineBox[{218, 219}]}, 
                    {Hue[0.9089177824574007, 1, 1], LineBox[{219, 220}]}, 
                    {Hue[0.972346258437172, 1, 1], LineBox[{220, 221}]}, 
                    {Hue[0.03639461960368258, 1, 1], LineBox[{221, 222}]}, 
                    {Hue[0.10105942154813613`, 1, 1], LineBox[{222, 223}]}, 
                    {Hue[0.16633766436514846`, 1, 1], LineBox[{223, 224}]}, 
                    {Hue[0.23222719273173698`, 1, 1], LineBox[{224, 225}]}, 
                    {Hue[0.2987238647155857, 1, 1], LineBox[{225, 226}]}, 
                    {Hue[0.36582744102738923`, 1, 1], LineBox[{226, 227}]}, 
                    {Hue[0.4335325386500767, 1, 1], LineBox[{227, 228}]}, 
                    {Hue[0.5018379833859594, 1, 1], LineBox[{228, 229}]}, 
                    {Hue[0.5707408969998853, 1, 1], LineBox[{229, 230}]}, 
                    {Hue[0.6402401203183907, 1, 1], LineBox[{230, 231}]}, 
                    {Hue[0.7103314443312188, 1, 1], LineBox[{231, 232}]}, 
                    {Hue[0.7810129385835942, 1, 1], LineBox[{232, 233}]}, 
                    {Hue[0.8522840511817902, 1, 1], LineBox[{233, 234}]}, 
                    {Hue[0.9241392387208713, 1, 1], LineBox[{234, 235}]}, 
                    {Hue[0.9965798263202051, 1, 1], LineBox[{235, 236}]}, 
                    {Hue[0.06959900745981731, 1, 1], LineBox[{236, 237}]}, 
                    {Hue[0.14319934802733422`, 1, 1], LineBox[{237, 238}]}, 
                    {Hue[0.21737807633544506`, 1, 1], LineBox[{238, 239}]}, 
                    {Hue[0.2921281655178836, 1, 1], LineBox[{239, 240}]}, 
                    {Hue[0.3674526228331577, 1, 1], LineBox[{240, 241}]}, 
                    {Hue[0.4433487386760116, 1, 1], LineBox[{241, 242}]}, 
                    {Hue[0.519813479068898, 1, 1], LineBox[{242, 243}]}, 
                    {Hue[0.5968450073967411, 1, 1], LineBox[{243, 244}]}, 
                    {Hue[0.6744391898025186, 1, 1], LineBox[{244, 245}]}, 
                    {Hue[0.7526026203570177, 1, 1], LineBox[{245, 246}]}, 
                    {Hue[0.8313227714345117, 1, 1], LineBox[{246, 247}]}, 
                    {Hue[0.9106020572940575, 1, 1], LineBox[{247, 248}]}, 
                    {Hue[0.99044294769886, 1, 1], LineBox[{248, 249}]}, 
                    {Hue[0.07083722612697897, 1, 1], LineBox[{249, 250}]}, 
                    {Hue[0.15178796486556909`, 1, 1], LineBox[{250, 251}]}}}},
                    
                    VertexColors->{
                    Hue[0.45492651067977985`, 1, 1], 
                    Hue[0.35453989828871174`, 1, 1], 
                    Hue[0.25371662554201285`, 1, 1], 
                    Hue[0.153334251777296, 1, 1], 
                    Hue[0.05307399626199472, 1, 1], 
                    Hue[0.95363236378411, 1, 1], 
                    Hue[0.8541318403485163, 1, 1], 
                    Hue[0.7547295042198715, 1, 1], 
                    Hue[0.6559085725275098, 1, 1], 
                    Hue[0.5571483343007815, 1, 1], 
                    Hue[0.4584138934345222, 1, 1], 
                    Hue[0.36039912597627266`, 1, 1], 
                    Hue[0.2622953759685387, 1, 1], 
                    Hue[0.16436459436553116`, 1, 1], 
                    Hue[0.06686467199637351, 1, 1], 
                    Hue[0.9695170994455039, 1, 1], 
                    Hue[0.8723303846594687, 1, 1], 
                    Hue[0.7754090552434305, 1, 1], 
                    Hue[0.67874465847557, 1, 1], 
                    Hue[0.582229927875765, 1, 1], 
                    Hue[0.4859598807852869, 1, 1], 
                    Hue[0.38987575474619246`, 1, 1], 
                    Hue[0.2940365317975417, 1, 1], 
                    Hue[0.19836684198245164`, 1, 1], 
                    Hue[0.10294494847248514`, 1, 1], 
                    Hue[0.007707782375784896, 1, 1], 
                    Hue[0.9126637472741359, 1, 1], 
                    Hue[0.8178222771749548, 1, 1], 
                    Hue[0.72318127803347, 1, 1], 
                    Hue[0.6287338651278908, 1, 1], 
                    Hue[0.5344701180581571, 1, 1], 
                    Hue[0.44039426020930045`, 1, 1], 
                    Hue[0.3465144635884028, 1, 1], 
                    Hue[0.252808072997363, 1, 1], 
                    Hue[0.15928966223813107`, 1, 1], 
                    Hue[0.06594733545312544, 1, 1], 
                    Hue[0.9727848443406211, 1, 1], 
                    Hue[0.8797973542288998, 1, 1], 
                    Hue[0.7869821562937429, 1, 1], 
                    Hue[0.6943399395807146, 1, 1], 
                    Hue[0.6018559509399378, 1, 1], 
                    Hue[0.5095393776462689, 1, 1], 
                    Hue[0.4173796678168414, 1, 1], 
                    Hue[0.32538385171789214`, 1, 1], 
                    Hue[0.23354205356651353`, 1, 1], 
                    Hue[0.14185471410378753`, 1, 1], 
                    Hue[0.05031447265582115, 1, 1], 
                    Hue[0.958923662715511, 1, 1], 
                    Hue[0.8676772117323288, 1, 1], 
                    Hue[0.7765680700686013, 1, 1], 
                    Hue[0.6856019387109209, 1, 1], 
                    Hue[0.5947691460253403, 1, 1], 
                    Hue[0.5040652266289894, 1, 1], 
                    Hue[0.4134896901706374, 1, 1], 
                    Hue[0.3230371814924714, 1, 1], 
                    Hue[0.2327048098212888, 1, 1], 
                    Hue[0.14248935709303306`, 1, 1], 
                    Hue[0.05238415176542811, 1, 1], 
                    Hue[0.9623891562027246, 1, 1], 
                    Hue[0.8724970950084328, 1, 1], 
                    Hue[0.782705234830449, 1, 1], 
                    Hue[0.6930094276719068, 1, 1], 
                    Hue[0.603403558095029, 1, 1], 
                    Hue[0.5138848451851199, 1, 1], 
                    Hue[0.4244454572392602, 1, 1], 
                    Hue[0.3350842173811413, 1, 1], 
                    Hue[0.24579166251417273`, 1, 1], 
                    Hue[0.15656623339073467`, 1, 1], 
                    Hue[0.06739962559988893, 1, 1], 
                    Hue[0.9782875994973265, 1, 1], 
                    Hue[0.8892234142617812, 1, 1], 
                    Hue[0.8002011485261, 1, 1], 
                    Hue[0.7112140191474132, 1, 1], 
                    Hue[0.6222542253030292, 1, 1], 
                    Hue[0.5333177457842979, 1, 1], 
                    Hue[0.44439506979053767`, 1, 1], 
                    Hue[0.3554794286460181, 1, 1], 
                    Hue[0.26656299622171675`, 1, 1], 
                    Hue[0.17763793048842322`, 1, 1], 
                    Hue[0.08869610636692972, 1, 1], 
                    Hue[0.9997283819955648, 1, 1], 
                    Hue[0.9107265702305547, 1, 1], 
                    Hue[0.821681754524422, 1, 1], 
                    Hue[0.7325848999670737, 1, 1], 
                    Hue[0.6434256600246612, 1, 1], 
                    Hue[0.5541954241832242, 1, 1], 
                    Hue[0.46488295327061496`, 1, 1], 
                    Hue[0.3754802808713916, 1, 1], 
                    Hue[0.28597636002231513`, 1, 1], 
                    Hue[0.1963625454614133, 1, 1], 
                    Hue[0.10662868654545397`, 1, 1], 
                    Hue[0.016766218274584323`, 1, 1], 
                    Hue[0.9267666720232012, 1, 1], 
                    Hue[0.8366209546576345, 1, 1], 
                    Hue[0.7463234324270356, 1, 1], 
                    Hue[0.6558683946811299, 1, 1], 
                    Hue[0.5652519424005065, 1, 1], 
                    Hue[0.4744728032182932, 1, 1], 
                    Hue[0.38353075616650895`, 1, 1], 
                    Hue[0.29242972469312417`, 1, 1], 
                    Hue[0.20117765285064337`, 1, 1], 
                    Hue[0.1097862900027622, 1, 1], 
                    Hue[0.01827287832674748, 1, 1], 
                    Hue[0.9266611974571188, 1, 1], 
                    Hue[0.8349800174043959, 1, 1], 
                    Hue[0.7432670997335856, 1, 1], 
                    Hue[0.6515686779597296, 1, 1], 
                    Hue[0.5599424694431372, 1, 1], 
                    Hue[0.46845481596904104`, 1, 1], 
                    Hue[0.3771862786362149, 1, 1], 
                    Hue[0.2862282662686505, 1, 1], 
                    Hue[0.19568641537022163`, 1, 1], 
                    Hue[0.10567797607262314`, 1, 1], 
                    Hue[0.016332731301606014`, 1, 1], 
                    Hue[0.927795591376938, 1, 1], 
                    Hue[0.8402276070843839, 1, 1], 
                    Hue[0.7538035231069398, 1, 1], 
                    Hue[0.6687130433992411, 1, 1], 
                    Hue[0.5851522832589728, 1, 1], 
                    Hue[0.5033241434569828, 1, 1], 
                    Hue[0.42343048382940907`, 1, 1], 
                    Hue[0.3456705554356446, 1, 1], 
                    Hue[0.27023803792139145`, 1, 1], 
                    Hue[0.19731503301541775`, 1, 1], 
                    Hue[0.1270771087857017, 1, 1], 
                    Hue[0.05968324559803208, 1, 1], 
                    Hue[0.9952806746680052, 1, 1], 
                    Hue[0.9339888537835502, 1, 1], 
                    Hue[0.8759098972504199, 1, 1], 
                    Hue[0.8211168140301675, 1, 1], 
                    Hue[0.769657595305148, 1, 1], 
                    Hue[0.72155306419796, 1, 1], 
                    Hue[0.6767989892674167, 1, 1], 
                    Hue[0.6353668692228582, 1, 1], 
                    Hue[0.5972087793115762, 1, 1], 
                    Hue[0.5622604326917748, 1, 1], 
                    Hue[0.5304403852817042, 1, 1], 
                    Hue[0.501658519810958, 1, 1], 
                    Hue[0.47581545273971154`, 1, 1], 
                    Hue[0.45280660192802846`, 1, 1], 
                    Hue[0.4325244262555183, 1, 1], 
                    Hue[0.4148607958877386, 1, 1], 
                    Hue[0.3997064885717363, 1, 1], 
                    Hue[0.3869563679568564, 1, 1], 
                    Hue[0.37650730231757107`, 1, 1], 
                    Hue[0.3682607261039661, 1, 1], 
                    Hue[0.3621209982291931, 1, 1], 
                    Hue[0.3579978426514018, 1, 1], 
                    Hue[0.35580683697359283`, 1, 1], 
                    Hue[0.3554662759685706, 1, 1], 
                    Hue[0.3569013213821672, 1, 1], 
                    Hue[0.36004081361650353`, 1, 1], 
                    Hue[0.3648181761077391, 1, 1], 
                    Hue[0.37117149197985105`, 1, 1], 
                    Hue[0.37904337644688246`, 1, 1], 
                    Hue[0.38838003033941537`, 1, 1], 
                    Hue[0.39913105246117875`, 1, 1], 
                    Hue[0.411249920977416, 1, 1], 
                    Hue[0.424693391058465, 1, 1], 
                    Hue[0.4394211657571874, 1, 1], 
                    Hue[0.45539537047986617`, 1, 1], 
                    Hue[0.47258130536614373`, 1, 1], 
                    Hue[0.4909467062963725, 1, 1], 
                    Hue[0.5104604752212907, 1, 1], 
                    Hue[0.5310943454416712, 1, 1], 
                    Hue[0.552822658366199, 1, 1], 
                    Hue[0.5756200347081709, 1, 1], 
                    Hue[0.5994628887166491, 1, 1], 
                    Hue[0.6243296837994351, 1, 1], 
                    Hue[0.6502004900681735, 1, 1], 
                    Hue[0.6770556375018364, 1, 1], 
                    Hue[0.7048773963672461, 1, 1], 
                    Hue[0.7336477827191039, 1, 1], 
                    Hue[0.7633516997074754, 1, 1], 
                    Hue[0.7939737135007793, 1, 1], 
                    Hue[0.8254995106024579, 1, 1], 
                    Hue[0.8579156762552613, 1, 1], 
                    Hue[0.8912095063230834, 1, 1], 
                    Hue[0.925368717798873, 1, 1], 
                    Hue[0.9603824546877948, 1, 1], 
                    Hue[0.9962385682294179, 1, 1], 
                    Hue[0.03292654017297939, 1, 1], 
                    Hue[0.07043714394267478, 1, 1], 
                    Hue[0.10875968330459024`, 1, 1], 
                    Hue[0.14788561304282966`, 1, 1], 
                    Hue[0.18780615362640285`, 1, 1], 
                    Hue[0.22851270426524767`, 1, 1], 
                    Hue[0.26999801955043884`, 1, 1], 
                    Hue[0.3122539822073713, 1, 1], 
                    Hue[0.35527287615958714`, 1, 1], 
                    Hue[0.39904799194802826`, 1, 1], 
                    Hue[0.44357149723895656`, 1, 1], 
                    Hue[0.48883781441948254`, 1, 1], 
                    Hue[0.534839279152306, 1, 1], 
                    Hue[0.5815702894260358, 1, 1], 
                    Hue[0.6290256025850298, 1, 1], 
                    Hue[0.6771988392967269, 1, 1], 
                    Hue[0.7260852621317265, 1, 1], 
                    Hue[0.7756785902064173, 1, 1], 
                    Hue[0.8259737909039171, 1, 1], 
                    Hue[0.8769657593370601, 1, 1], 
                    Hue[0.9286486911989955, 1, 1], 
                    Hue[0.9810188386237015, 1, 1], 
                    Hue[0.034069971038059345`, 1, 1], 
                    Hue[0.08779934893459017, 1, 1], 
                    Hue[0.1422018167527364, 1, 1], 
                    Hue[0.19727378340379548`, 1, 1], 
                    Hue[0.2530108487712756, 1, 1], 
                    Hue[0.30940900332769283`, 1, 1], 
                    Hue[0.3664635294786629, 1, 1], 
                    Hue[0.42417153233580135`, 1, 1], 
                    Hue[0.48252852701740456`, 1, 1], 
                    Hue[0.5415311942928878, 1, 1], 
                    Hue[0.6011771487708214, 1, 1], 
                    Hue[0.6614611660479162, 1, 1], 
                    Hue[0.7223807452582915, 1, 1], 
                    Hue[0.7839319540579166, 1, 1], 
                    Hue[0.8461120801534268, 1, 1], 
                    Hue[0.9089177824574007, 1, 1], 
                    Hue[0.972346258437172, 1, 1], 
                    Hue[0.03639461960368258, 1, 1], 
                    Hue[0.10105942154813613`, 1, 1], 
                    Hue[0.16633766436514846`, 1, 1], 
                    Hue[0.23222719273173698`, 1, 1], 
                    Hue[0.2987238647155857, 1, 1], 
                    Hue[0.36582744102738923`, 1, 1], 
                    Hue[0.4335325386500767, 1, 1], 
                    Hue[0.5018379833859594, 1, 1], 
                    Hue[0.5707408969998853, 1, 1], 
                    Hue[0.6402401203183907, 1, 1], 
                    Hue[0.7103314443312188, 1, 1], 
                    Hue[0.7810129385835942, 1, 1], 
                    Hue[0.8522840511817902, 1, 1], 
                    Hue[0.9241392387208713, 1, 1], 
                    Hue[0.9965798263202051, 1, 1], 
                    Hue[0.06959900745981731, 1, 1], 
                    Hue[0.14319934802733422`, 1, 1], 
                    Hue[0.21737807633544506`, 1, 1], 
                    Hue[0.2921281655178836, 1, 1], 
                    Hue[0.3674526228331577, 1, 1], 
                    Hue[0.4433487386760116, 1, 1], 
                    Hue[0.519813479068898, 1, 1], 
                    Hue[0.5968450073967411, 1, 1], 
                    Hue[0.6744391898025186, 1, 1], 
                    Hue[0.7526026203570177, 1, 1], 
                    Hue[0.8313227714345117, 1, 1], 
                    Hue[0.9106020572940575, 1, 1], 
                    Hue[0.99044294769886, 1, 1], 
                    Hue[0.07083722612697897, 1, 1], 
                    Hue[0.15178796486556909`, 1, 1], 
                    Hue[0.2332917296675744, 1, 1], 
                    Hue[0.45492651067977985`, 1, 1], 
                    Hue[0.35453989828871174`, 1, 1], 
                    Hue[0.35453989828871174`, 1, 1], 
                    Hue[0.25371662554201285`, 1, 1], 
                    Hue[0.25371662554201285`, 1, 1], 
                    Hue[0.153334251777296, 1, 1], 
                    Hue[0.153334251777296, 1, 1], 
                    Hue[0.05307399626199472, 1, 1], 
                    Hue[0.05307399626199472, 1, 1], 
                    Hue[0.95363236378411, 1, 1], 
                    Hue[0.95363236378411, 1, 1], 
                    Hue[0.8541318403485163, 1, 1], 
                    Hue[0.8541318403485163, 1, 1], 
                    Hue[0.7547295042198715, 1, 1], 
                    Hue[0.7547295042198715, 1, 1], 
                    Hue[0.6559085725275098, 1, 1], 
                    Hue[0.6559085725275098, 1, 1], 
                    Hue[0.5571483343007815, 1, 1], 
                    Hue[0.5571483343007815, 1, 1], 
                    Hue[0.4584138934345222, 1, 1], 
                    Hue[0.4584138934345222, 1, 1], 
                    Hue[0.36039912597627266`, 1, 1], 
                    Hue[0.36039912597627266`, 1, 1], 
                    Hue[0.2622953759685387, 1, 1], 
                    Hue[0.2622953759685387, 1, 1], 
                    Hue[0.16436459436553116`, 1, 1], 
                    Hue[0.16436459436553116`, 1, 1], 
                    Hue[0.06686467199637351, 1, 1], 
                    Hue[0.06686467199637351, 1, 1], 
                    Hue[0.9695170994455039, 1, 1], 
                    Hue[0.9695170994455039, 1, 1], 
                    Hue[0.8723303846594687, 1, 1], 
                    Hue[0.8723303846594687, 1, 1], 
                    Hue[0.7754090552434305, 1, 1], 
                    Hue[0.7754090552434305, 1, 1], 
                    Hue[0.67874465847557, 1, 1], 
                    Hue[0.67874465847557, 1, 1], 
                    Hue[0.582229927875765, 1, 1], 
                    Hue[0.582229927875765, 1, 1], 
                    Hue[0.4859598807852869, 1, 1], 
                    Hue[0.4859598807852869, 1, 1], 
                    Hue[0.38987575474619246`, 1, 1], 
                    Hue[0.38987575474619246`, 1, 1], 
                    Hue[0.2940365317975417, 1, 1], 
                    Hue[0.2940365317975417, 1, 1], 
                    Hue[0.19836684198245164`, 1, 1], 
                    Hue[0.19836684198245164`, 1, 1], 
                    Hue[0.10294494847248514`, 1, 1], 
                    Hue[0.10294494847248514`, 1, 1], 
                    Hue[0.007707782375784896, 1, 1], 
                    Hue[0.007707782375784896, 1, 1], 
                    Hue[0.9126637472741359, 1, 1], 
                    Hue[0.9126637472741359, 1, 1], 
                    Hue[0.8178222771749548, 1, 1], 
                    Hue[0.8178222771749548, 1, 1], 
                    Hue[0.72318127803347, 1, 1], 
                    Hue[0.72318127803347, 1, 1], 
                    Hue[0.6287338651278908, 1, 1], 
                    Hue[0.6287338651278908, 1, 1], 
                    Hue[0.5344701180581571, 1, 1], 
                    Hue[0.5344701180581571, 1, 1], 
                    Hue[0.44039426020930045`, 1, 1], 
                    Hue[0.44039426020930045`, 1, 1], 
                    Hue[0.3465144635884028, 1, 1], 
                    Hue[0.3465144635884028, 1, 1], 
                    Hue[0.252808072997363, 1, 1], 
                    Hue[0.252808072997363, 1, 1], 
                    Hue[0.15928966223813107`, 1, 1], 
                    Hue[0.15928966223813107`, 1, 1], 
                    Hue[0.06594733545312544, 1, 1], 
                    Hue[0.06594733545312544, 1, 1], 
                    Hue[0.9727848443406211, 1, 1], 
                    Hue[0.9727848443406211, 1, 1], 
                    Hue[0.8797973542288998, 1, 1], 
                    Hue[0.8797973542288998, 1, 1], 
                    Hue[0.7869821562937429, 1, 1], 
                    Hue[0.7869821562937429, 1, 1], 
                    Hue[0.6943399395807146, 1, 1], 
                    Hue[0.6943399395807146, 1, 1], 
                    Hue[0.6018559509399378, 1, 1], 
                    Hue[0.6018559509399378, 1, 1], 
                    Hue[0.5095393776462689, 1, 1], 
                    Hue[0.5095393776462689, 1, 1], 
                    Hue[0.4173796678168414, 1, 1], 
                    Hue[0.4173796678168414, 1, 1], 
                    Hue[0.32538385171789214`, 1, 1], 
                    Hue[0.32538385171789214`, 1, 1], 
                    Hue[0.23354205356651353`, 1, 1], 
                    Hue[0.23354205356651353`, 1, 1], 
                    Hue[0.14185471410378753`, 1, 1], 
                    Hue[0.14185471410378753`, 1, 1], 
                    Hue[0.05031447265582115, 1, 1], 
                    Hue[0.05031447265582115, 1, 1], 
                    Hue[0.958923662715511, 1, 1], 
                    Hue[0.958923662715511, 1, 1], 
                    Hue[0.8676772117323288, 1, 1], 
                    Hue[0.8676772117323288, 1, 1], 
                    Hue[0.7765680700686013, 1, 1], 
                    Hue[0.7765680700686013, 1, 1], 
                    Hue[0.6856019387109209, 1, 1], 
                    Hue[0.6856019387109209, 1, 1], 
                    Hue[0.5947691460253403, 1, 1], 
                    Hue[0.5947691460253403, 1, 1], 
                    Hue[0.5040652266289894, 1, 1], 
                    Hue[0.5040652266289894, 1, 1], 
                    Hue[0.4134896901706374, 1, 1], 
                    Hue[0.4134896901706374, 1, 1], 
                    Hue[0.3230371814924714, 1, 1], 
                    Hue[0.3230371814924714, 1, 1], 
                    Hue[0.2327048098212888, 1, 1], 
                    Hue[0.2327048098212888, 1, 1], 
                    Hue[0.14248935709303306`, 1, 1], 
                    Hue[0.14248935709303306`, 1, 1], 
                    Hue[0.05238415176542811, 1, 1], 
                    Hue[0.05238415176542811, 1, 1], 
                    Hue[0.9623891562027246, 1, 1], 
                    Hue[0.9623891562027246, 1, 1], 
                    Hue[0.8724970950084328, 1, 1], 
                    Hue[0.8724970950084328, 1, 1], 
                    Hue[0.782705234830449, 1, 1], 
                    Hue[0.782705234830449, 1, 1], 
                    Hue[0.6930094276719068, 1, 1], 
                    Hue[0.6930094276719068, 1, 1], 
                    Hue[0.603403558095029, 1, 1], 
                    Hue[0.603403558095029, 1, 1], 
                    Hue[0.5138848451851199, 1, 1], 
                    Hue[0.5138848451851199, 1, 1], 
                    Hue[0.4244454572392602, 1, 1], 
                    Hue[0.4244454572392602, 1, 1], 
                    Hue[0.3350842173811413, 1, 1], 
                    Hue[0.3350842173811413, 1, 1], 
                    Hue[0.24579166251417273`, 1, 1], 
                    Hue[0.24579166251417273`, 1, 1], 
                    Hue[0.15656623339073467`, 1, 1], 
                    Hue[0.15656623339073467`, 1, 1], 
                    Hue[0.06739962559988893, 1, 1], 
                    Hue[0.06739962559988893, 1, 1], 
                    Hue[0.9782875994973265, 1, 1], 
                    Hue[0.9782875994973265, 1, 1], 
                    Hue[0.8892234142617812, 1, 1], 
                    Hue[0.8892234142617812, 1, 1], 
                    Hue[0.8002011485261, 1, 1], 
                    Hue[0.8002011485261, 1, 1], 
                    Hue[0.7112140191474132, 1, 1], 
                    Hue[0.7112140191474132, 1, 1], 
                    Hue[0.6222542253030292, 1, 1], 
                    Hue[0.6222542253030292, 1, 1], 
                    Hue[0.5333177457842979, 1, 1], 
                    Hue[0.5333177457842979, 1, 1], 
                    Hue[0.44439506979053767`, 1, 1], 
                    Hue[0.44439506979053767`, 1, 1], 
                    Hue[0.3554794286460181, 1, 1], 
                    Hue[0.3554794286460181, 1, 1], 
                    Hue[0.26656299622171675`, 1, 1], 
                    Hue[0.26656299622171675`, 1, 1], 
                    Hue[0.17763793048842322`, 1, 1], 
                    Hue[0.17763793048842322`, 1, 1], 
                    Hue[0.08869610636692972, 1, 1], 
                    Hue[0.08869610636692972, 1, 1], 
                    Hue[0.9997283819955648, 1, 1], 
                    Hue[0.9997283819955648, 1, 1], 
                    Hue[0.9107265702305547, 1, 1], 
                    Hue[0.9107265702305547, 1, 1], 
                    Hue[0.821681754524422, 1, 1], 
                    Hue[0.821681754524422, 1, 1], 
                    Hue[0.7325848999670737, 1, 1], 
                    Hue[0.7325848999670737, 1, 1], 
                    Hue[0.6434256600246612, 1, 1], 
                    Hue[0.6434256600246612, 1, 1], 
                    Hue[0.5541954241832242, 1, 1], 
                    Hue[0.5541954241832242, 1, 1], 
                    Hue[0.46488295327061496`, 1, 1], 
                    Hue[0.46488295327061496`, 1, 1], 
                    Hue[0.3754802808713916, 1, 1], 
                    Hue[0.3754802808713916, 1, 1], 
                    Hue[0.28597636002231513`, 1, 1], 
                    Hue[0.28597636002231513`, 1, 1], 
                    Hue[0.1963625454614133, 1, 1], 
                    Hue[0.1963625454614133, 1, 1], 
                    Hue[0.10662868654545397`, 1, 1], 
                    Hue[0.10662868654545397`, 1, 1], 
                    Hue[0.016766218274584323`, 1, 1], 
                    Hue[0.016766218274584323`, 1, 1], 
                    Hue[0.9267666720232012, 1, 1], 
                    Hue[0.9267666720232012, 1, 1], 
                    Hue[0.8366209546576345, 1, 1], 
                    Hue[0.8366209546576345, 1, 1], 
                    Hue[0.7463234324270356, 1, 1], 
                    Hue[0.7463234324270356, 1, 1], 
                    Hue[0.6558683946811299, 1, 1], 
                    Hue[0.6558683946811299, 1, 1], 
                    Hue[0.5652519424005065, 1, 1], 
                    Hue[0.5652519424005065, 1, 1], 
                    Hue[0.4744728032182932, 1, 1], 
                    Hue[0.4744728032182932, 1, 1], 
                    Hue[0.38353075616650895`, 1, 1], 
                    Hue[0.38353075616650895`, 1, 1], 
                    Hue[0.29242972469312417`, 1, 1], 
                    Hue[0.29242972469312417`, 1, 1], 
                    Hue[0.20117765285064337`, 1, 1], 
                    Hue[0.20117765285064337`, 1, 1], 
                    Hue[0.1097862900027622, 1, 1], 
                    Hue[0.1097862900027622, 1, 1], 
                    Hue[0.01827287832674748, 1, 1], 
                    Hue[0.01827287832674748, 1, 1], 
                    Hue[0.9266611974571188, 1, 1], 
                    Hue[0.9266611974571188, 1, 1], 
                    Hue[0.8349800174043959, 1, 1], 
                    Hue[0.8349800174043959, 1, 1], 
                    Hue[0.7432670997335856, 1, 1], 
                    Hue[0.7432670997335856, 1, 1], 
                    Hue[0.6515686779597296, 1, 1], 
                    Hue[0.6515686779597296, 1, 1], 
                    Hue[0.5599424694431372, 1, 1], 
                    Hue[0.5599424694431372, 1, 1], 
                    Hue[0.46845481596904104`, 1, 1], 
                    Hue[0.46845481596904104`, 1, 1], 
                    Hue[0.3771862786362149, 1, 1], 
                    Hue[0.3771862786362149, 1, 1], 
                    Hue[0.2862282662686505, 1, 1], 
                    Hue[0.2862282662686505, 1, 1], 
                    Hue[0.19568641537022163`, 1, 1], 
                    Hue[0.19568641537022163`, 1, 1], 
                    Hue[0.10567797607262314`, 1, 1], 
                    Hue[0.10567797607262314`, 1, 1], 
                    Hue[0.016332731301606014`, 1, 1], 
                    Hue[0.016332731301606014`, 1, 1], 
                    Hue[0.927795591376938, 1, 1], 
                    Hue[0.927795591376938, 1, 1], 
                    Hue[0.8402276070843839, 1, 1], 
                    Hue[0.8402276070843839, 1, 1], 
                    Hue[0.7538035231069398, 1, 1], 
                    Hue[0.7538035231069398, 1, 1], 
                    Hue[0.6687130433992411, 1, 1], 
                    Hue[0.6687130433992411, 1, 1], 
                    Hue[0.5851522832589728, 1, 1], 
                    Hue[0.5851522832589728, 1, 1], 
                    Hue[0.5033241434569828, 1, 1], 
                    Hue[0.5033241434569828, 1, 1], 
                    Hue[0.42343048382940907`, 1, 1], 
                    Hue[0.42343048382940907`, 1, 1], 
                    Hue[0.3456705554356446, 1, 1], 
                    Hue[0.3456705554356446, 1, 1], 
                    Hue[0.27023803792139145`, 1, 1], 
                    Hue[0.27023803792139145`, 1, 1], 
                    Hue[0.19731503301541775`, 1, 1], 
                    Hue[0.19731503301541775`, 1, 1], 
                    Hue[0.1270771087857017, 1, 1], 
                    Hue[0.1270771087857017, 1, 1], 
                    Hue[0.05968324559803208, 1, 1], 
                    Hue[0.05968324559803208, 1, 1], 
                    Hue[0.9952806746680052, 1, 1], 
                    Hue[0.9952806746680052, 1, 1], 
                    Hue[0.9339888537835502, 1, 1], 
                    Hue[0.9339888537835502, 1, 1], 
                    Hue[0.8759098972504199, 1, 1], 
                    Hue[0.8759098972504199, 1, 1], 
                    Hue[0.8211168140301675, 1, 1], 
                    Hue[0.8211168140301675, 1, 1], 
                    Hue[0.769657595305148, 1, 1], 
                    Hue[0.769657595305148, 1, 1], 
                    Hue[0.72155306419796, 1, 1], 
                    Hue[0.72155306419796, 1, 1], 
                    Hue[0.6767989892674167, 1, 1], 
                    Hue[0.6767989892674167, 1, 1], 
                    Hue[0.6353668692228582, 1, 1], 
                    Hue[0.6353668692228582, 1, 1], 
                    Hue[0.5972087793115762, 1, 1], 
                    Hue[0.5972087793115762, 1, 1], 
                    Hue[0.5622604326917748, 1, 1], 
                    Hue[0.5622604326917748, 1, 1], 
                    Hue[0.5304403852817042, 1, 1], 
                    Hue[0.5304403852817042, 1, 1], 
                    Hue[0.501658519810958, 1, 1], 
                    Hue[0.501658519810958, 1, 1], 
                    Hue[0.47581545273971154`, 1, 1], 
                    Hue[0.47581545273971154`, 1, 1], 
                    Hue[0.45280660192802846`, 1, 1], 
                    Hue[0.45280660192802846`, 1, 1], 
                    Hue[0.4325244262555183, 1, 1], 
                    Hue[0.4325244262555183, 1, 1], 
                    Hue[0.4148607958877386, 1, 1], 
                    Hue[0.4148607958877386, 1, 1], 
                    Hue[0.3997064885717363, 1, 1], 
                    Hue[0.3997064885717363, 1, 1], 
                    Hue[0.3869563679568564, 1, 1], 
                    Hue[0.3869563679568564, 1, 1], 
                    Hue[0.37650730231757107`, 1, 1], 
                    Hue[0.37650730231757107`, 1, 1], 
                    Hue[0.3682607261039661, 1, 1], 
                    Hue[0.3682607261039661, 1, 1], 
                    Hue[0.3621209982291931, 1, 1], 
                    Hue[0.3621209982291931, 1, 1], 
                    Hue[0.3579978426514018, 1, 1], 
                    Hue[0.3579978426514018, 1, 1], 
                    Hue[0.35580683697359283`, 1, 1], 
                    Hue[0.35580683697359283`, 1, 1], 
                    Hue[0.3554662759685706, 1, 1], 
                    Hue[0.3554662759685706, 1, 1], 
                    Hue[0.3569013213821672, 1, 1], 
                    Hue[0.3569013213821672, 1, 1], 
                    Hue[0.36004081361650353`, 1, 1], 
                    Hue[0.36004081361650353`, 1, 1], 
                    Hue[0.3648181761077391, 1, 1], 
                    Hue[0.3648181761077391, 1, 1], 
                    Hue[0.37117149197985105`, 1, 1], 
                    Hue[0.37117149197985105`, 1, 1], 
                    Hue[0.37904337644688246`, 1, 1], 
                    Hue[0.37904337644688246`, 1, 1], 
                    Hue[0.38838003033941537`, 1, 1], 
                    Hue[0.38838003033941537`, 1, 1], 
                    Hue[0.39913105246117875`, 1, 1], 
                    Hue[0.39913105246117875`, 1, 1], 
                    Hue[0.411249920977416, 1, 1], 
                    Hue[0.411249920977416, 1, 1], 
                    Hue[0.424693391058465, 1, 1], 
                    Hue[0.424693391058465, 1, 1], 
                    Hue[0.4394211657571874, 1, 1], 
                    Hue[0.4394211657571874, 1, 1], 
                    Hue[0.45539537047986617`, 1, 1], 
                    Hue[0.45539537047986617`, 1, 1], 
                    Hue[0.47258130536614373`, 1, 1], 
                    Hue[0.47258130536614373`, 1, 1], 
                    Hue[0.4909467062963725, 1, 1], 
                    Hue[0.4909467062963725, 1, 1], 
                    Hue[0.5104604752212907, 1, 1], 
                    Hue[0.5104604752212907, 1, 1], 
                    Hue[0.5310943454416712, 1, 1], 
                    Hue[0.5310943454416712, 1, 1], 
                    Hue[0.552822658366199, 1, 1], 
                    Hue[0.552822658366199, 1, 1], 
                    Hue[0.5756200347081709, 1, 1], 
                    Hue[0.5756200347081709, 1, 1], 
                    Hue[0.5994628887166491, 1, 1], 
                    Hue[0.5994628887166491, 1, 1], 
                    Hue[0.6243296837994351, 1, 1], 
                    Hue[0.6243296837994351, 1, 1], 
                    Hue[0.6502004900681735, 1, 1], 
                    Hue[0.6502004900681735, 1, 1], 
                    Hue[0.6770556375018364, 1, 1], 
                    Hue[0.6770556375018364, 1, 1], 
                    Hue[0.7048773963672461, 1, 1], 
                    Hue[0.7048773963672461, 1, 1], 
                    Hue[0.7336477827191039, 1, 1], 
                    Hue[0.7336477827191039, 1, 1], 
                    Hue[0.7633516997074754, 1, 1], 
                    Hue[0.7633516997074754, 1, 1], 
                    Hue[0.7939737135007793, 1, 1], 
                    Hue[0.7939737135007793, 1, 1], 
                    Hue[0.8254995106024579, 1, 1], 
                    Hue[0.8254995106024579, 1, 1], 
                    Hue[0.8579156762552613, 1, 1], 
                    Hue[0.8579156762552613, 1, 1], 
                    Hue[0.8912095063230834, 1, 1], 
                    Hue[0.8912095063230834, 1, 1], 
                    Hue[0.925368717798873, 1, 1], 
                    Hue[0.925368717798873, 1, 1], 
                    Hue[0.9603824546877948, 1, 1], 
                    Hue[0.9603824546877948, 1, 1], 
                    Hue[0.9962385682294179, 1, 1], 
                    Hue[0.9962385682294179, 1, 1], 
                    Hue[0.03292654017297939, 1, 1], 
                    Hue[0.03292654017297939, 1, 1], 
                    Hue[0.07043714394267478, 1, 1], 
                    Hue[0.07043714394267478, 1, 1], 
                    Hue[0.10875968330459024`, 1, 1], 
                    Hue[0.10875968330459024`, 1, 1], 
                    Hue[0.14788561304282966`, 1, 1], 
                    Hue[0.14788561304282966`, 1, 1], 
                    Hue[0.18780615362640285`, 1, 1], 
                    Hue[0.18780615362640285`, 1, 1], 
                    Hue[0.22851270426524767`, 1, 1], 
                    Hue[0.22851270426524767`, 1, 1], 
                    Hue[0.26999801955043884`, 1, 1], 
                    Hue[0.26999801955043884`, 1, 1], 
                    Hue[0.3122539822073713, 1, 1], 
                    Hue[0.3122539822073713, 1, 1], 
                    Hue[0.35527287615958714`, 1, 1], 
                    Hue[0.35527287615958714`, 1, 1], 
                    Hue[0.39904799194802826`, 1, 1], 
                    Hue[0.39904799194802826`, 1, 1], 
                    Hue[0.44357149723895656`, 1, 1], 
                    Hue[0.44357149723895656`, 1, 1], 
                    Hue[0.48883781441948254`, 1, 1], 
                    Hue[0.48883781441948254`, 1, 1], 
                    Hue[0.534839279152306, 1, 1], 
                    Hue[0.534839279152306, 1, 1], 
                    Hue[0.5815702894260358, 1, 1], 
                    Hue[0.5815702894260358, 1, 1], 
                    Hue[0.6290256025850298, 1, 1], 
                    Hue[0.6290256025850298, 1, 1], 
                    Hue[0.6771988392967269, 1, 1], 
                    Hue[0.6771988392967269, 1, 1], 
                    Hue[0.7260852621317265, 1, 1], 
                    Hue[0.7260852621317265, 1, 1], 
                    Hue[0.7756785902064173, 1, 1], 
                    Hue[0.7756785902064173, 1, 1], 
                    Hue[0.8259737909039171, 1, 1], 
                    Hue[0.8259737909039171, 1, 1], 
                    Hue[0.8769657593370601, 1, 1], 
                    Hue[0.8769657593370601, 1, 1], 
                    Hue[0.9286486911989955, 1, 1], 
                    Hue[0.9286486911989955, 1, 1], 
                    Hue[0.9810188386237015, 1, 1], 
                    Hue[0.9810188386237015, 1, 1], 
                    Hue[0.034069971038059345`, 1, 1], 
                    Hue[0.034069971038059345`, 1, 1], 
                    Hue[0.08779934893459017, 1, 1], 
                    Hue[0.08779934893459017, 1, 1], 
                    Hue[0.1422018167527364, 1, 1], 
                    Hue[0.1422018167527364, 1, 1], 
                    Hue[0.19727378340379548`, 1, 1], 
                    Hue[0.19727378340379548`, 1, 1], 
                    Hue[0.2530108487712756, 1, 1], 
                    Hue[0.2530108487712756, 1, 1], 
                    Hue[0.30940900332769283`, 1, 1], 
                    Hue[0.30940900332769283`, 1, 1], 
                    Hue[0.3664635294786629, 1, 1], 
                    Hue[0.3664635294786629, 1, 1], 
                    Hue[0.42417153233580135`, 1, 1], 
                    Hue[0.42417153233580135`, 1, 1], 
                    Hue[0.48252852701740456`, 1, 1], 
                    Hue[0.48252852701740456`, 1, 1], 
                    Hue[0.5415311942928878, 1, 1], 
                    Hue[0.5415311942928878, 1, 1], 
                    Hue[0.6011771487708214, 1, 1], 
                    Hue[0.6011771487708214, 1, 1], 
                    Hue[0.6614611660479162, 1, 1], 
                    Hue[0.6614611660479162, 1, 1], 
                    Hue[0.7223807452582915, 1, 1], 
                    Hue[0.7223807452582915, 1, 1], 
                    Hue[0.7839319540579166, 1, 1], 
                    Hue[0.7839319540579166, 1, 1], 
                    Hue[0.8461120801534268, 1, 1], 
                    Hue[0.8461120801534268, 1, 1], 
                    Hue[0.9089177824574007, 1, 1], 
                    Hue[0.9089177824574007, 1, 1], 
                    Hue[0.972346258437172, 1, 1], 
                    Hue[0.972346258437172, 1, 1], 
                    Hue[0.03639461960368258, 1, 1], 
                    Hue[0.03639461960368258, 1, 1], 
                    Hue[0.10105942154813613`, 1, 1], 
                    Hue[0.10105942154813613`, 1, 1], 
                    Hue[0.16633766436514846`, 1, 1], 
                    Hue[0.16633766436514846`, 1, 1], 
                    Hue[0.23222719273173698`, 1, 1], 
                    Hue[0.23222719273173698`, 1, 1], 
                    Hue[0.2987238647155857, 1, 1], 
                    Hue[0.2987238647155857, 1, 1], 
                    Hue[0.36582744102738923`, 1, 1], 
                    Hue[0.36582744102738923`, 1, 1], 
                    Hue[0.4335325386500767, 1, 1], 
                    Hue[0.4335325386500767, 1, 1], 
                    Hue[0.5018379833859594, 1, 1], 
                    Hue[0.5018379833859594, 1, 1], 
                    Hue[0.5707408969998853, 1, 1], 
                    Hue[0.5707408969998853, 1, 1], 
                    Hue[0.6402401203183907, 1, 1], 
                    Hue[0.6402401203183907, 1, 1], 
                    Hue[0.7103314443312188, 1, 1], 
                    Hue[0.7103314443312188, 1, 1], 
                    Hue[0.7810129385835942, 1, 1], 
                    Hue[0.7810129385835942, 1, 1], 
                    Hue[0.8522840511817902, 1, 1], 
                    Hue[0.8522840511817902, 1, 1], 
                    Hue[0.9241392387208713, 1, 1], 
                    Hue[0.9241392387208713, 1, 1], 
                    Hue[0.9965798263202051, 1, 1], 
                    Hue[0.9965798263202051, 1, 1], 
                    Hue[0.06959900745981731, 1, 1], 
                    Hue[0.06959900745981731, 1, 1], 
                    Hue[0.14319934802733422`, 1, 1], 
                    Hue[0.14319934802733422`, 1, 1], 
                    Hue[0.21737807633544506`, 1, 1], 
                    Hue[0.21737807633544506`, 1, 1], 
                    Hue[0.2921281655178836, 1, 1], 
                    Hue[0.2921281655178836, 1, 1], 
                    Hue[0.3674526228331577, 1, 1], 
                    Hue[0.3674526228331577, 1, 1], 
                    Hue[0.4433487386760116, 1, 1], 
                    Hue[0.4433487386760116, 1, 1], 
                    Hue[0.519813479068898, 1, 1], 
                    Hue[0.519813479068898, 1, 1], 
                    Hue[0.5968450073967411, 1, 1], 
                    Hue[0.5968450073967411, 1, 1], 
                    Hue[0.6744391898025186, 1, 1], 
                    Hue[0.6744391898025186, 1, 1], 
                    Hue[0.7526026203570177, 1, 1], 
                    Hue[0.7526026203570177, 1, 1], 
                    Hue[0.8313227714345117, 1, 1], 
                    Hue[0.8313227714345117, 1, 1], 
                    Hue[0.9106020572940575, 1, 1], 
                    Hue[0.9106020572940575, 1, 1], 
                    Hue[0.99044294769886, 1, 1], 
                    Hue[0.99044294769886, 1, 1], 
                    Hue[0.07083722612697897, 1, 1], 
                    Hue[0.07083722612697897, 1, 1], 
                    Hue[0.15178796486556909`, 1, 1], 
                    Hue[0.15178796486556909`, 1, 1], 
                    Hue[0.2332917296675744, 1, 1]}],
                   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
                   Axes->{True, None},
                   AxesOrigin->{0, 0},
                   
                   Epilog->LineBox[{{1, 0.00017331360906365425`}, {
                    2, 0.0002004461808351119}, {3, 0.00023209713512794408`}, {
                    4, 0.0002676601467031059}, {5, 0.0003087125258518049}, {
                    6, 0.00035601856270682205`}, {
                    7, 0.00040921890624941425`}, {
                    8, 0.00047047823447631036`}, {9, 0.0005408591138593625}, {
                    10, 0.0006191840054251076}, {11, 0.000709080914569019}, {
                    12, 0.0008108019552094106}, {13, 0.0009260520917183954}, {
                    14, 0.0010567383942408253`}, {15, 0.001203589144498999}, {
                    16, 0.001369595381751855}, {17, 0.001556798677256322}, {
                    18, 0.0017671588589336738`}, {19, 0.002003161644340483}, {
                    20, 0.0022683286188256734`}, {21, 0.002564484755366951}, {
                    22, 0.0028965581741394954`}, {23, 0.003266999043243304}, {
                    24, 0.003679718150811597}, {25, 0.004139597446106104}, {
                    26, 0.004651357662952773}, {27, 0.005219157882640485}, {
                    28, 0.005848647155893587}, {29, 0.006544771804776567}, {
                    30, 0.007314549033472592}, {31, 0.008163307664132327}, {
                    32, 0.009097766833732104}, {33, 0.010125651977032576`}, {
                    34, 0.011253800023434424`}, {35, 0.012490321227922572`}, {
                    36, 0.013843050812300686`}, {37, 0.015320781611294472`}, {
                    38, 0.016931734649757835`}, {39, 0.01868550424974008}, {
                    40, 0.020590763001179613`}, {41, 0.022657460746511208`}, {
                    42, 0.02489424843224105}, {43, 0.027309952530464643`}, {
                    44, 0.029915507633075825`}, {45, 0.03271946707746229}, {
                    46, 0.03573130775371684}, {47, 0.038960198487364}, {
                    48, 0.042415244819097794`}, {49, 0.04610289409974477}, {
                    50, 0.05003236483245365}, {51, 0.05420899154698012}, {
                    52, 0.05863889792005716}, {53, 0.06332690652005153}, {
                    54, 0.06827741625858946}, {55, 0.07349179029983526}, {
                    56, 0.07897109342627537}, {57, 0.08471680264858389}, {
                    58, 0.09072300813638602}, {59, 0.09698886835011952}, {
                    60, 0.10350497014937741`}, {61, 0.11026774539193047`}, {
                    62, 0.11726170125230698`}, {63, 0.12447599750403575`}, {
                    64, 0.1318964919738219}, {65, 0.13950329194724798`}, {
                    66, 0.14727665871607312`}, {67, 0.15519608126465115`}, {
                    68, 0.1632344398253523}, {69, 0.1713644947197756}, {
                    70, 0.17955757773348607`}, {71, 0.18778228063222607`}, {
                    72, 0.1960031327336788}, {73, 0.20418488385356307`}, {
                    74, 0.21228982250938633`}, {75, 0.22027943389973836`}, {
                    76, 0.22811543342899146`}, {77, 0.23575713640247425`}, {
                    78, 0.2431623336675769}, {79, 0.2502926032747923}, {
                    80, 0.257107721944049}, {81, 0.26356920240138426`}, {
                    82, 0.26964041115952253`}, {83, 0.27528476901325505`}, {
                    84, 0.280471331611325}, {85, 0.2851677251561855}, {
                    86, 0.28934997381540073`}, {87, 0.2929952468495493}, {
                    88, 0.29608487661443056`}, {89, 0.2986072234382678}, {
                    90, 0.30055166088767266`}, {91, 0.30191413901786507`}, {
                    92, 0.30269958653023915`}, {93, 0.3029140568249175}, {
                    94, 0.3025722637243245}, {95, 0.3016927811930327}, {
                    96, 0.30029936131922935`}, {97, 0.298422595194389}, {
                    98, 0.29609755507909635`}, {99, 0.2933624150108186}, {
                    100, 0.29026302920979236`}, {101, 0.286845190950363}, {
                    102, 0.2831569528517122}, {103, 0.2792543839598428}, {
                    104, 0.27518591752285343`}, {105, 0.27100818902992324`}, {
                    106, 0.26677321461853376`}, {107, 0.2625346332313765}, {
                    108, 0.25834460634723033`}, {109, 0.2542509824968922}, {
                    110, 0.2502986191492596}, {111, 0.2465274341001233}, {
                    112, 0.24297202419410358`}, {113, 0.23966451598179678`}, {
                    114, 0.23663128278311762`}, {115, 0.23390103667854123`}, {
                    116, 0.23149503843117236`}, {117, 0.22943181659834055`}, {
                    118, 0.227721992744214}, {119, 0.22636634281838655`}, {
                    120, 0.22536101369420056`}, {121, 0.2246947834068738}, {
                    122, 0.22436027860410757`}, {123, 0.22435101244766212`}, {
                    124, 0.22466262388700653`}, {125, 0.22529472387529653`}, {
                    126, 0.2262498670299155}, {127, 0.2275275484347227}, {
                    128, 0.22912754265719926`}, {129, 0.23104560277962916`}, {
                    130, 0.23327282372492836`}, {131, 0.23579644777173359`}, {
                    132, 0.23860069202274067`}, {133, 0.24167043021890522`}, {
                    134, 0.24498347104904492`}, {135, 0.24851775123571604`}, {
                    136, 0.25225101398229677`}, {137, 0.2561585438676063}, {
                    138, 0.26021802672829825`}, {139, 0.26440430659264025`}, {
                    140, 0.2686946553889191}, {141, 0.27306451128652093`}, {
                    142, 0.2774921983657016}, {143, 0.2819580316031532}, {
                    144, 0.28644029650886804`}, {145, 0.2909186135018893}, {
                    146, 0.2953764858811382}, {147, 0.2997955161814758}, {
                    148, 0.3041572159402033}, {149, 0.3084482026663686}, {
                    150, 0.31265286760456545`}, {151, 0.3167559555142252}, {
                    152, 0.3207462080944464}, {153, 0.3246101344601126}, {
                    154, 0.32833554355847333`}, {155, 0.33191168048336966`}, {
                    156, 0.3353278337157109}, {157, 0.33857372031040317`}, {
                    158, 0.34163980647816605`}, {159, 0.3445196047504053}, {
                    160, 0.34720328429045594`}, {161, 0.34968315332650624`}, {
                    162, 0.3519532090939946}, {163, 0.35400553299927073`}, {
                    164, 0.355835932042}, {165, 0.35743740457042233`}, {
                    166, 0.3588061600083007}, {167, 0.3599380251656484}, {
                    168, 0.36082873894708223`}, {169, 0.3614752729586731}, {
                    170, 0.3618740414578474}, {171, 0.36202326204878194`}, {
                    172, 0.3619225665184547}, {173, 0.3615701460598568}, {
                    174, 0.3609634715525623}, {175, 0.3601041589814839}, {
                    176, 0.3589912735630775}, {177, 0.35762675577858566`}, {
                    178, 0.3560114742371367}, {179, 0.35414883846456613`}, {
                    180, 0.35204110725361115`}, {181, 0.3496927266882332}, {
                    182, 0.34710523535039056`}, {183, 0.3442845086427027}, {
                    184, 0.3412346921588113}, {185, 0.3379589979277237}, {
                    186, 0.3344658081951963}, {187, 0.33075967688214025`}, {
                    188, 0.32684660456093056`}, {189, 0.3227360764781115}, {
                    190, 0.31843382693867706`}, {191, 0.31394905753557445`}, {
                    192, 0.3092903483080025}, {193, 0.3044647642369086}, {
                    194, 0.29948218731334375`}, {195, 0.29434880562639715`}, {
                    196, 0.2890779807748132}, {197, 0.28367575492880553`}, {
                    198, 0.2781530167818283}, {199, 0.2725223654171132}, {
                    200, 0.26679207144047457`}, {201, 0.26097372002960206`}, {
                    202, 0.2550758806462184}, {203, 0.24910940445230206`}, {
                    204, 0.24308241575262116`}, {205, 0.23700763672712055`}, {
                    206, 0.23089362972065583`}, {207, 0.22475210603935666`}, {
                    208, 0.2185920309509046}, {209, 0.21242353315960877`}, {
                    210, 0.20625598550913202`}, {211, 0.20009937456319532`}, {
                    212, 0.19396179537919936`}, {213, 0.18785405181512924`}, {
                    214, 0.18178284167940054`}, {215, 0.17575729087552489`}, {
                    216, 0.16978546960300106`}, {217, 0.16387457420798704`}, {
                    218, 0.15803288760853612`}, {219, 0.15226633476887902`}, {
                    220, 0.14658267456394228`}, {221, 0.1409865044606825}, {
                    222, 0.135484535537044}, {223, 0.13008182482847933`}, {
                    224, 0.1247839027199716}, {225, 0.11959491469218655`}, {
                    226, 0.11451929151160208`}, {227, 0.10955929120759636`}, {
                    228, 0.10472080290491217`}, {229, 0.10000468295437415`}, {
                    230, 0.09541473481728896}, {231, 0.09095112056823589}, {
                    232, 0.08661814601089213}, {233, 0.08241452964810968}, {
                    234, 0.07834296446027422}, {235, 0.07440297853840688}, {
                    236, 0.07059524256318644}, {237, 0.06692074572861295}, {
                    238, 0.06337686152312463}, {239, 0.059963828819883436`}, {
                    240, 0.05668196958223771}, {241, 0.05352822472308272}, {
                    242, 0.05050196948533514}, {243, 0.04760062289923075}, {
                    244, 0.04482390245047077}, {245, 0.04216821657723882}, {
                    246, 0.0396303648274698}, {247, 0.037210444023152885`}, {
                    248, 0.034903687340650254`}, {
                    249, 0.032707795639734534`}, {
                    250, 0.030620143186840535`}, {
                    251, 0.028637042566020763`}}],
                   Frame->True,
                   FrameTicks->{{{1., 
                    FormBox[
                    RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
                    FormBox[
                    RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
                    FormBox[
                    RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
                    FormBox[
                    RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
                    FormBox["0", TraditionalForm]}, {157.25, 
                    FormBox["2.5`", TraditionalForm]}, {188.5, 
                    FormBox["5", TraditionalForm]}, {219.75, 
                    FormBox["7.5`", TraditionalForm]}, {251., 
                    FormBox["10", TraditionalForm]}}, Automatic, None, None},
                   PlotRange->{-0.1, 1.1},
                   PlotRangeClipping->True,
                   PlotRangePadding->{
                    Scaled[0.02], Automatic}], 23->
                  GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF13lcVVW7B/DtlGZa5ICi2N04kPKiKYNYpvwUEWSeQcYNh+Ewg0pYmu5I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                    "], {
                    {GrayLevel[0.5], EdgeForm[None], 
                    GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ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                    
                    "]]]}, {{}, {}, {
                    {Hue[0.9890850417084717, 1, 1], LineBox[{1, 2}]}, 
                    {Hue[0.890444563820459, 1, 1], LineBox[{2, 3}]}, 
                    {Hue[0.7918350921023508, 1, 1], LineBox[{3, 4}]}, 
                    {Hue[0.6935923344905128, 1, 1], LineBox[{4, 5}]}, 
                    {Hue[0.5955973533557162, 1, 1], LineBox[{5, 6}]}, 
                    {Hue[0.49769643430527716`, 1, 1], LineBox[{6, 7}]}, 
                    {Hue[0.3999909708344182, 1, 1], LineBox[{7, 8}]}, 
                    {Hue[0.30260995757912756`, 1, 1], LineBox[{8, 9}]}, 
                    {Hue[0.20549048009062573`, 1, 1], LineBox[{9, 10}]}, 
                    {Hue[0.10841445415142113`, 1, 1], LineBox[{10, 11}]}, 
                    {Hue[0.011694343660073115`, 1, 1], LineBox[{11, 12}]}, 
                    {Hue[0.915136789552362, 1, 1], LineBox[{12, 13}]}, 
                    {Hue[0.8187862586312527, 1, 1], LineBox[{13, 14}]}, 
                    {Hue[0.7226446234905665, 1, 1], LineBox[{14, 15}]}, 
                    {Hue[0.6266956278780738, 1, 1], LineBox[{15, 16}]}, 
                    {Hue[0.5309424388597916, 1, 1], LineBox[{16, 17}]}, 
                    {Hue[0.4354159473821284, 1, 1], LineBox[{17, 18}]}, 
                    {Hue[0.34006128509633016`, 1, 1], LineBox[{18, 19}]}, 
                    {Hue[0.24488834017086, 1, 1], LineBox[{19, 20}]}, 
                    {Hue[0.1499381665888046, 1, 1], LineBox[{20, 21}]}, 
                    {Hue[0.0551497709061452, 1, 1], LineBox[{21, 22}]}, 
                    {Hue[0.9605747897944017, 1, 1], LineBox[{22, 23}]}, 
                    {Hue[0.8661822384266291, 1, 1], LineBox[{23, 24}]}, 
                    {Hue[0.7719815454219947, 1, 1], LineBox[{24, 25}]}, 
                    {Hue[0.6779589248969495, 1, 1], LineBox[{25, 26}]}, 
                    {Hue[0.584107788669658, 1, 1], LineBox[{26, 27}]}, 
                    {Hue[0.4904321659403038, 1, 1], LineBox[{27, 28}]}, 
                    {Hue[0.3969391264652255, 1, 1], LineBox[{28, 29}]}, 
                    {Hue[0.3036082892055693, 1, 1], LineBox[{29, 30}]}, 
                    {Hue[0.21045143453790424`, 1, 1], LineBox[{30, 31}]}, 
                    {Hue[0.1174679950897775, 1, 1], LineBox[{31, 32}]}, 
                    {Hue[0.024647137886225627`, 1, 1], LineBox[{32, 33}]}, 
                    {Hue[0.9319857820365661, 1, 1], LineBox[{33, 34}]}, 
                    {Hue[0.8394897332935869, 1, 1], LineBox[{34, 35}]}, 
                    {Hue[0.7471528779093941, 1, 1], LineBox[{35, 36}]}, 
                    {Hue[0.6549682687227965, 1, 1], LineBox[{36, 37}]}, 
                    {Hue[0.562940161123826, 1, 1], LineBox[{37, 38}]}, 
                    {Hue[0.4710601350866123, 1, 1], LineBox[{38, 39}]}, 
                    {Hue[0.379328026400307, 1, 1], LineBox[{39, 40}]}, 
                    {Hue[0.2877415089286468, 1, 1], LineBox[{40, 41}]}, 
                    {Hue[0.19629620846912263`, 1, 1], LineBox[{41, 42}]}, 
                    {Hue[0.10498770191782866`, 1, 1], LineBox[{42, 43}]}, 
                    {Hue[0.013818248266201809`, 1, 1], LineBox[{43, 44}]}, 
                    {Hue[0.922779355202294, 1, 1], LineBox[{44, 45}]}, 
                    {Hue[0.8318706546694346, 1, 1], LineBox[{45, 46}]}, 
                    {Hue[0.7410886274394506, 1, 1], LineBox[{46, 47}]}, 
                    {Hue[0.6504288919503963, 1, 1], LineBox[{47, 48}]}, 
                    {Hue[0.5598898317429462, 1, 1], LineBox[{48, 49}]}, 
                    {Hue[0.46946392923718244`, 1, 1], LineBox[{49, 50}]}, 
                    {Hue[0.3791530821283772, 1, 1], LineBox[{50, 51}]}, 
                    {Hue[0.2889467711448164, 1, 1], LineBox[{51, 52}]}, 
                    {Hue[0.19884605935101357`, 1, 1], LineBox[{52, 53}]}, 
                    {Hue[0.10884451371828528`, 1, 1], LineBox[{53, 54}]}, 
                    {Hue[0.01893840191355155, 1, 1], LineBox[{54, 55}]}, 
                    {Hue[0.9291246203627189, 1, 1], LineBox[{55, 56}]}, 
                    {Hue[0.839398027108012, 1, 1], LineBox[{56, 57}]}, 
                    {Hue[0.7497518597451729, 1, 1], LineBox[{57, 58}]}, 
                    {Hue[0.6601838746800621, 1, 1], LineBox[{58, 59}]}, 
                    {Hue[0.5706881943866222, 1, 1], LineBox[{59, 60}]}, 
                    {Hue[0.48125901280784156`, 1, 1], LineBox[{60, 61}]}, 
                    {Hue[0.39189064851607397`, 1, 1], LineBox[{61, 62}]}, 
                    {Hue[0.30257891317312197`, 1, 1], LineBox[{62, 63}]}, 
                    {Hue[0.21331627749924545`, 1, 1], LineBox[{63, 64}]}, 
                    {Hue[0.1240969762593327, 1, 1], LineBox[{64, 65}]}, 
                    {Hue[0.034915561215019375`, 1, 1], LineBox[{65, 66}]}, 
                    {Hue[0.9457641347466291, 1, 1], LineBox[{66, 67}]}, 
                    {Hue[0.8566369738272935, 1, 1], LineBox[{67, 68}]}, 
                    {Hue[0.7675262150857177, 1, 1], LineBox[{68, 69}]}, 
                    {Hue[0.6784238011201688, 1, 1], LineBox[{69, 70}]}, 
                    {Hue[0.5893230207040819, 1, 1], LineBox[{70, 71}]}, 
                    {Hue[0.50021495606295, 1, 1], LineBox[{71, 72}]}, 
                    {Hue[0.4110913775684948, 1, 1], LineBox[{72, 73}]}, 
                    {Hue[0.3219424614705361, 1, 1], LineBox[{73, 74}]}, 
                    {Hue[0.23275925793759258`, 1, 1], LineBox[{74, 75}]}, 
                    {Hue[0.143532350562769, 1, 1], LineBox[{75, 76}]}, 
                    {Hue[0.05425179677263266, 1, 1], LineBox[{76, 77}]}, 
                    {Hue[0.9649067193442054, 1, 1], LineBox[{77, 78}]}, 
                    {Hue[0.8754870852363708, 1, 1], LineBox[{78, 79}]}, 
                    {Hue[0.7859800701598657, 1, 1], LineBox[{79, 80}]}, 
                    {Hue[0.6963751868911885, 1, 1], LineBox[{80, 81}]}, 
                    {Hue[0.6066591090207734, 1, 1], LineBox[{81, 82}]}, 
                    {Hue[0.5168198134156986, 1, 1], LineBox[{82, 83}]}, 
                    {Hue[0.42684569537233746`, 1, 1], LineBox[{83, 84}]}, 
                    {Hue[0.33672288201474887`, 1, 1], LineBox[{84, 85}]}, 
                    {Hue[0.24643890665017157`, 1, 1], LineBox[{85, 86}]}, 
                    {Hue[0.1559790483703322, 1, 1], LineBox[{86, 87}]}, 
                    {Hue[0.06533250362229022, 1, 1], LineBox[{87, 88}]}, 
                    {Hue[0.9744852437056882, 1, 1], LineBox[{88, 89}]}, 
                    {Hue[0.8834255839195694, 1, 1], LineBox[{89, 90}]}, 
                    {Hue[0.7921421823685508, 1, 1], LineBox[{90, 91}]}, 
                    {Hue[0.7006241285792536, 1, 1], LineBox[{91, 92}]}, 
                    {Hue[0.6088632573112414, 1, 1], LineBox[{92, 93}]}, 
                    {Hue[0.5168507348555242, 1, 1], LineBox[{93, 94}]}, 
                    {Hue[0.42458376599519027`, 1, 1], LineBox[{94, 95}]}, 
                    {Hue[0.33205862577194645`, 1, 1], LineBox[{95, 96}]}, 
                    {Hue[0.2392775083989476, 1, 1], LineBox[{96, 97}]}, 
                    {Hue[0.14624478418476983`, 1, 1], LineBox[{97, 98}]}, 
                    {Hue[0.052970346936419506`, 1, 1], LineBox[{98, 99}]}, 
                    {Hue[0.9594676360426614, 1, 1], LineBox[{99, 100}]}, 
                    {Hue[0.8657577363329141, 1, 1], LineBox[{100, 101}]}, 
                    {Hue[0.7718665933810455, 1, 1], LineBox[{101, 102}]}, 
                    {Hue[0.6778270601834724, 1, 1], LineBox[{102, 103}]}, 
                    {Hue[0.5836773317911002, 1, 1], LineBox[{103, 104}]}, 
                    {Hue[0.48946425451082115`, 1, 1], LineBox[{104, 105}]}, 
                    {Hue[0.3952418144069382, 1, 1], LineBox[{105, 106}]}, 
                    {Hue[0.3010708889653632, 1, 1], LineBox[{106, 107}]}, 
                    {Hue[0.207018848047681, 1, 1], LineBox[{107, 108}]}, 
                    {Hue[0.11316324104925135`, 1, 1], LineBox[{108, 109}]}, 
                    {Hue[0.019585516746522633`, 1, 1], LineBox[{109, 110}]}, 
                    {Hue[0.9263748173191635, 1, 1], LineBox[{110, 111}]}, 
                    {Hue[0.8336288784715873, 1, 1], LineBox[{111, 112}]}, 
                    {Hue[0.7414537703560893, 1, 1], LineBox[{112, 113}]}, 
                    {Hue[0.6499644952350254, 1, 1], LineBox[{113, 114}]}, 
                    {Hue[0.559286572819959, 1, 1], LineBox[{114, 115}]}, 
                    {Hue[0.46955356366737094`, 1, 1], LineBox[{115, 116}]}, 
                    {Hue[0.38090894736594, 1, 1], LineBox[{116, 117}]}, 
                    {Hue[0.2935085010777411, 1, 1], LineBox[{117, 118}]}, 
                    {Hue[0.20750998380319777`, 1, 1], LineBox[{118, 119}]}, 
                    {Hue[0.12308448701168116`, 1, 1], LineBox[{119, 120}]}, 
                    {Hue[0.040405866180151344`, 1, 1], LineBox[{120, 121}]}, 
                    {Hue[0.9596577933041008, 1, 1], LineBox[{121, 122}]}, 
                    {Hue[0.8810254867376445, 1, 1], LineBox[{122, 123}]}, 
                    {Hue[0.8046989087260993, 1, 1], LineBox[{123, 124}]}, 
                    {Hue[0.7308629301685772, 1, 1], LineBox[{124, 125}]}, 
                    {Hue[0.659694095351627, 1, 1], LineBox[{125, 126}]}, 
                    {Hue[0.591355858433406, 1, 1], LineBox[{126, 127}]}, 
                    {Hue[0.5259960179322813, 1, 1], LineBox[{127, 128}]}, 
                    {Hue[0.4637384836446359, 1, 1], LineBox[{128, 129}]}, 
                    {Hue[0.4046827309417128, 1, 1], LineBox[{129, 130}]}, 
                    {Hue[0.34890027740507623`, 1, 1], LineBox[{130, 131}]}, 
                    {Hue[0.2964351349133147, 1, 1], LineBox[{131, 132}]}, 
                    {Hue[0.24730822874130168`, 1, 1], LineBox[{132, 133}]}, 
                    {Hue[0.20151138082108272`, 1, 1], LineBox[{133, 134}]}, 
                    {Hue[0.15901329786186943`, 1, 1], LineBox[{134, 135}]}, 
                    {Hue[0.1197649271403978, 1, 1], LineBox[{135, 136}]}, 
                    {Hue[0.08369802893832214, 1, 1], LineBox[{136, 137}]}, 
                    {Hue[0.05073030875715411, 1, 1], LineBox[{137, 138}]}, 
                    {Hue[0.020769647506886663`, 1, 1], LineBox[{138, 139}]}, 
                    {Hue[0.9937147157833544, 1, 1], LineBox[{139, 140}]}, 
                    {Hue[0.9694593009365832, 1, 1], LineBox[{140, 141}]}, 
                    {Hue[0.9478941084618844, 1, 1], LineBox[{141, 142}]}, 
                    {Hue[0.9289103885338673, 1, 1], LineBox[{142, 143}]}, 
                    {Hue[0.9123988367658288, 1, 1], LineBox[{143, 144}]}, 
                    {Hue[0.8982520275469468, 1, 1], LineBox[{144, 145}]}, 
                    {Hue[0.886366658292858, 1, 1], LineBox[{145, 146}]}, 
                    {Hue[0.8766426050628463, 1, 1], LineBox[{146, 147}]}, 
                    {Hue[0.8689841847218216, 1, 1], LineBox[{147, 148}]}, 
                    {Hue[0.8633013381767353, 1, 1], LineBox[{148, 149}]}, 
                    {Hue[0.8595080716578948, 1, 1], LineBox[{149, 150}]}, 
                    {Hue[0.8575234682582267, 1, 1], LineBox[{150, 151}]}, 
                    {Hue[0.8572726383404587, 1, 1], LineBox[{151, 152}]}, 
                    {Hue[0.8586834192166569, 1, 1], LineBox[{152, 153}]}, 
                    {Hue[0.8616906928539774, 1, 1], LineBox[{153, 154}]}, 
                    {Hue[0.8662315826883799, 1, 1], LineBox[{154, 155}]}, 
                    {Hue[0.8722494769006455, 1, 1], LineBox[{155, 156}]}, 
                    {Hue[0.8796904511863467, 1, 1], LineBox[{156, 157}]}, 
                    {Hue[0.8885044010740425, 1, 1], LineBox[{157, 158}]}, 
                    {Hue[0.8986452227927522, 1, 1], LineBox[{158, 159}]}, 
                    {Hue[0.9100709017328726, 1, 1], LineBox[{159, 160}]}, 
                    {Hue[0.922740508081968, 1, 1], LineBox[{160, 161}]}, 
                    {Hue[0.9366170036872785, 1, 1], LineBox[{161, 162}]}, 
                    {Hue[0.9516664083103602, 1, 1], LineBox[{162, 163}]}, 
                    {Hue[0.9678565539344235, 1, 1], LineBox[{163, 164}]}, 
                    {Hue[0.9851573817997091, 1, 1], LineBox[{164, 165}]}, 
                    {Hue[0.003541029282514706, 1, 1], LineBox[{165, 166}]}, 
                    {Hue[0.022982276990593284`, 1, 1], LineBox[{166, 167}]}, 
                    {Hue[0.04345618989206919, 1, 1], LineBox[{167, 168}]}, 
                    {Hue[0.06494019180123378, 1, 1], LineBox[{168, 169}]}, 
                    {Hue[0.0874131980234504, 1, 1], LineBox[{169, 170}]}, 
                    {Hue[0.11085512776100503`, 1, 1], LineBox[{170, 171}]}, 
                    {Hue[0.1352477810807344, 1, 1], LineBox[{171, 172}]}, 
                    {Hue[0.16057339342287885`, 1, 1], LineBox[{172, 173}]}, 
                    {Hue[0.18681539630932423`, 1, 1], LineBox[{173, 174}]}, 
                    {Hue[0.21395846771373908`, 1, 1], LineBox[{174, 175}]}, 
                    {Hue[0.24198818379898965`, 1, 1], LineBox[{175, 176}]}, 
                    {Hue[0.2708910987900743, 1, 1], LineBox[{176, 177}]}, 
                    {Hue[0.3006539059544776, 1, 1], LineBox[{177, 178}]}, 
                    {Hue[0.33126453314316606`, 1, 1], LineBox[{178, 179}]}, 
                    {Hue[0.3627118699332237, 1, 1], LineBox[{179, 180}]}, 
                    {Hue[0.3949837005086097, 1, 1], LineBox[{180, 181}]}, 
                    {Hue[0.42806988647193905`, 1, 1], LineBox[{181, 182}]}, 
                    {Hue[0.4619600062641948, 1, 1], LineBox[{182, 183}]}, 
                    {Hue[0.4966450931302264, 1, 1], LineBox[{183, 184}]}, 
                    {Hue[0.5321158935007508, 1, 1], LineBox[{184, 185}]}, 
                    {Hue[0.5683632217963384, 1, 1], LineBox[{185, 186}]}, 
                    {Hue[0.6053791741127653, 1, 1], LineBox[{186, 187}]}, 
                    {Hue[0.6431566313515203, 1, 1], LineBox[{187, 188}]}, 
                    {Hue[0.6816868752353401, 1, 1], LineBox[{188, 189}]}, 
                    {Hue[0.72096364429756, 1, 1], LineBox[{189, 190}]}, 
                    {Hue[0.7609790655744311, 1, 1], LineBox[{190, 191}]}, 
                    {Hue[0.801726159467002, 1, 1], LineBox[{191, 192}]}, 
                    {Hue[0.8431986948347605, 1, 1], LineBox[{192, 193}]}, 
                    {Hue[0.8853905620106385, 1, 1], LineBox[{193, 194}]}, 
                    {Hue[0.928294741191026, 1, 1], LineBox[{194, 195}]}, 
                    {Hue[0.9719066681331644, 1, 1], LineBox[{195, 196}]}, 
                    {Hue[0.016219788090091947`, 1, 1], LineBox[{196, 197}]}, 
                    {Hue[0.06123023095704857, 1, 1], LineBox[{197, 198}]}, 
                    {Hue[0.10693189420594468`, 1, 1], LineBox[{198, 199}]}, 
                    {Hue[0.15331972133290422`, 1, 1], LineBox[{199, 200}]}, 
                    {Hue[0.20038888796819257`, 1, 1], LineBox[{200, 201}]}, 
                    {Hue[0.24813299763561023`, 1, 1], LineBox[{201, 202}]}, 
                    {Hue[0.29654916208262616`, 1, 1], LineBox[{202, 203}]}, 
                    {Hue[0.3456310281848976, 1, 1], LineBox[{203, 204}]}, 
                    {Hue[0.3953752971354173, 1, 1], LineBox[{204, 205}]}, 
                    {Hue[0.4457779494411485, 1, 1], LineBox[{205, 206}]}, 
                    {Hue[0.49683472538993373`, 1, 1], LineBox[{206, 207}]}, 
                    {Hue[0.5485422000086002, 1, 1], LineBox[{207, 208}]}, 
                    {Hue[0.6008954329863241, 1, 1], LineBox[{208, 209}]}, 
                    {Hue[0.6538909685908945, 1, 1], LineBox[{209, 210}]}, 
                    {Hue[0.7075250668459268, 1, 1], LineBox[{210, 211}]}, 
                    {Hue[0.7617944518713861, 1, 1], LineBox[{211, 212}]}, 
                    {Hue[0.8166949979465602, 1, 1], LineBox[{212, 213}]}, 
                    {Hue[0.872223828978173, 1, 1], LineBox[{213, 214}]}, 
                    {Hue[0.9283771405632653, 1, 1], LineBox[{214, 215}]}, 
                    {Hue[0.9851524220111436, 1, 1], LineBox[{215, 216}]}, 
                    {Hue[0.04254550702833396, 1, 1], LineBox[{216, 217}]}, 
                    {Hue[0.10055367721831523`, 1, 1], LineBox[{217, 218}]}, 
                    {Hue[0.15917356261530025`, 1, 1], LineBox[{218, 219}]}, 
                    {Hue[0.21840264135365023`, 1, 1], LineBox[{219, 220}]}, 
                    {Hue[0.2782378071599018, 1, 1], LineBox[{220, 221}]}, 
                    {Hue[0.3386767001197204, 1, 1], LineBox[{221, 222}]}, 
                    {Hue[0.39971523956125754`, 1, 1], LineBox[{222, 223}]}, 
                    {Hue[0.46135214668637253`, 1, 1], LineBox[{223, 224}]}, 
                    {Hue[0.5235833706306552, 1, 1], LineBox[{224, 225}]}, 
                    {Hue[0.5864068661836146, 1, 1], LineBox[{225, 226}]}, 
                    {Hue[0.649821296542562, 1, 1], LineBox[{226, 227}]}, 
                    {Hue[0.7138220064371237, 1, 1], LineBox[{227, 228}]}, 
                    {Hue[0.7784087549190002, 1, 1], LineBox[{228, 229}]}, 
                    {Hue[0.8435773141078866, 1, 1], LineBox[{229, 230}]}, 
                    {Hue[0.9093257782873416, 1, 1], LineBox[{230, 231}]}, 
                    {Hue[0.9756515583544458, 1, 1], LineBox[{231, 232}]}, 
                    {Hue[0.04255353459618736, 1, 1], LineBox[{232, 233}]}, 
                    {Hue[0.11002894234378277`, 1, 1], LineBox[{233, 234}]}, 
                    {Hue[0.17807576974946937`, 1, 1], LineBox[{234, 235}]}, 
                    {Hue[0.2466904245232185, 1, 1], LineBox[{235, 236}]}, 
                    {Hue[0.3158733471799709, 1, 1], LineBox[{236, 237}]}, 
                    {Hue[0.3856187575594051, 1, 1], LineBox[{237, 238}]}, 
                    {Hue[0.4559291256723374, 1, 1], LineBox[{238, 239}]}, 
                    {Hue[0.5267985007359997, 1, 1], LineBox[{239, 240}]}, 
                    {Hue[0.5982288136813674, 1, 1], LineBox[{240, 241}]}, 
                    {Hue[0.6702147558152386, 1, 1], LineBox[{241, 242}]}, 
                    {Hue[0.7427566853759748, 1, 1], LineBox[{242, 243}]}, 
                    {Hue[0.8158503838879019, 1, 1], LineBox[{243, 244}]}, 
                    {Hue[0.8894967525160438, 1, 1], LineBox[{244, 245}]}, 
                    {Hue[0.9636945233767934, 1, 1], LineBox[{245, 246}]}, 
                    {Hue[0.03843758514675253, 1, 1], LineBox[{246, 247}]}, 
                    {Hue[0.11372770534876697`, 1, 1], LineBox[{247, 248}]}, 
                    {Hue[0.1895624517637856, 1, 1], LineBox[{248, 249}]}, 
                    {Hue[0.26593871306289235`, 1, 1], LineBox[{249, 250}]}, 
                    {Hue[0.34285896982386044`, 1, 1], LineBox[{250, 251}]}}}},
                    
                    VertexColors->{
                    Hue[0.9890850417084717, 1, 1], 
                    Hue[0.890444563820459, 1, 1], 
                    Hue[0.7918350921023508, 1, 1], 
                    Hue[0.6935923344905128, 1, 1], 
                    Hue[0.5955973533557162, 1, 1], 
                    Hue[0.49769643430527716`, 1, 1], 
                    Hue[0.3999909708344182, 1, 1], 
                    Hue[0.30260995757912756`, 1, 1], 
                    Hue[0.20549048009062573`, 1, 1], 
                    Hue[0.10841445415142113`, 1, 1], 
                    Hue[0.011694343660073115`, 1, 1], 
                    Hue[0.915136789552362, 1, 1], 
                    Hue[0.8187862586312527, 1, 1], 
                    Hue[0.7226446234905665, 1, 1], 
                    Hue[0.6266956278780738, 1, 1], 
                    Hue[0.5309424388597916, 1, 1], 
                    Hue[0.4354159473821284, 1, 1], 
                    Hue[0.34006128509633016`, 1, 1], 
                    Hue[0.24488834017086, 1, 1], 
                    Hue[0.1499381665888046, 1, 1], 
                    Hue[0.0551497709061452, 1, 1], 
                    Hue[0.9605747897944017, 1, 1], 
                    Hue[0.8661822384266291, 1, 1], 
                    Hue[0.7719815454219947, 1, 1], 
                    Hue[0.6779589248969495, 1, 1], 
                    Hue[0.584107788669658, 1, 1], 
                    Hue[0.4904321659403038, 1, 1], 
                    Hue[0.3969391264652255, 1, 1], 
                    Hue[0.3036082892055693, 1, 1], 
                    Hue[0.21045143453790424`, 1, 1], 
                    Hue[0.1174679950897775, 1, 1], 
                    Hue[0.024647137886225627`, 1, 1], 
                    Hue[0.9319857820365661, 1, 1], 
                    Hue[0.8394897332935869, 1, 1], 
                    Hue[0.7471528779093941, 1, 1], 
                    Hue[0.6549682687227965, 1, 1], 
                    Hue[0.562940161123826, 1, 1], 
                    Hue[0.4710601350866123, 1, 1], 
                    Hue[0.379328026400307, 1, 1], 
                    Hue[0.2877415089286468, 1, 1], 
                    Hue[0.19629620846912263`, 1, 1], 
                    Hue[0.10498770191782866`, 1, 1], 
                    Hue[0.013818248266201809`, 1, 1], 
                    Hue[0.922779355202294, 1, 1], 
                    Hue[0.8318706546694346, 1, 1], 
                    Hue[0.7410886274394506, 1, 1], 
                    Hue[0.6504288919503963, 1, 1], 
                    Hue[0.5598898317429462, 1, 1], 
                    Hue[0.46946392923718244`, 1, 1], 
                    Hue[0.3791530821283772, 1, 1], 
                    Hue[0.2889467711448164, 1, 1], 
                    Hue[0.19884605935101357`, 1, 1], 
                    Hue[0.10884451371828528`, 1, 1], 
                    Hue[0.01893840191355155, 1, 1], 
                    Hue[0.9291246203627189, 1, 1], 
                    Hue[0.839398027108012, 1, 1], 
                    Hue[0.7497518597451729, 1, 1], 
                    Hue[0.6601838746800621, 1, 1], 
                    Hue[0.5706881943866222, 1, 1], 
                    Hue[0.48125901280784156`, 1, 1], 
                    Hue[0.39189064851607397`, 1, 1], 
                    Hue[0.30257891317312197`, 1, 1], 
                    Hue[0.21331627749924545`, 1, 1], 
                    Hue[0.1240969762593327, 1, 1], 
                    Hue[0.034915561215019375`, 1, 1], 
                    Hue[0.9457641347466291, 1, 1], 
                    Hue[0.8566369738272935, 1, 1], 
                    Hue[0.7675262150857177, 1, 1], 
                    Hue[0.6784238011201688, 1, 1], 
                    Hue[0.5893230207040819, 1, 1], 
                    Hue[0.50021495606295, 1, 1], 
                    Hue[0.4110913775684948, 1, 1], 
                    Hue[0.3219424614705361, 1, 1], 
                    Hue[0.23275925793759258`, 1, 1], 
                    Hue[0.143532350562769, 1, 1], 
                    Hue[0.05425179677263266, 1, 1], 
                    Hue[0.9649067193442054, 1, 1], 
                    Hue[0.8754870852363708, 1, 1], 
                    Hue[0.7859800701598657, 1, 1], 
                    Hue[0.6963751868911885, 1, 1], 
                    Hue[0.6066591090207734, 1, 1], 
                    Hue[0.5168198134156986, 1, 1], 
                    Hue[0.42684569537233746`, 1, 1], 
                    Hue[0.33672288201474887`, 1, 1], 
                    Hue[0.24643890665017157`, 1, 1], 
                    Hue[0.1559790483703322, 1, 1], 
                    Hue[0.06533250362229022, 1, 1], 
                    Hue[0.9744852437056882, 1, 1], 
                    Hue[0.8834255839195694, 1, 1], 
                    Hue[0.7921421823685508, 1, 1], 
                    Hue[0.7006241285792536, 1, 1], 
                    Hue[0.6088632573112414, 1, 1], 
                    Hue[0.5168507348555242, 1, 1], 
                    Hue[0.42458376599519027`, 1, 1], 
                    Hue[0.33205862577194645`, 1, 1], 
                    Hue[0.2392775083989476, 1, 1], 
                    Hue[0.14624478418476983`, 1, 1], 
                    Hue[0.052970346936419506`, 1, 1], 
                    Hue[0.9594676360426614, 1, 1], 
                    Hue[0.8657577363329141, 1, 1], 
                    Hue[0.7718665933810455, 1, 1], 
                    Hue[0.6778270601834724, 1, 1], 
                    Hue[0.5836773317911002, 1, 1], 
                    Hue[0.48946425451082115`, 1, 1], 
                    Hue[0.3952418144069382, 1, 1], 
                    Hue[0.3010708889653632, 1, 1], 
                    Hue[0.207018848047681, 1, 1], 
                    Hue[0.11316324104925135`, 1, 1], 
                    Hue[0.019585516746522633`, 1, 1], 
                    Hue[0.9263748173191635, 1, 1], 
                    Hue[0.8336288784715873, 1, 1], 
                    Hue[0.7414537703560893, 1, 1], 
                    Hue[0.6499644952350254, 1, 1], 
                    Hue[0.559286572819959, 1, 1], 
                    Hue[0.46955356366737094`, 1, 1], 
                    Hue[0.38090894736594, 1, 1], 
                    Hue[0.2935085010777411, 1, 1], 
                    Hue[0.20750998380319777`, 1, 1], 
                    Hue[0.12308448701168116`, 1, 1], 
                    Hue[0.040405866180151344`, 1, 1], 
                    Hue[0.9596577933041008, 1, 1], 
                    Hue[0.8810254867376445, 1, 1], 
                    Hue[0.8046989087260993, 1, 1], 
                    Hue[0.7308629301685772, 1, 1], 
                    Hue[0.659694095351627, 1, 1], 
                    Hue[0.591355858433406, 1, 1], 
                    Hue[0.5259960179322813, 1, 1], 
                    Hue[0.4637384836446359, 1, 1], 
                    Hue[0.4046827309417128, 1, 1], 
                    Hue[0.34890027740507623`, 1, 1], 
                    Hue[0.2964351349133147, 1, 1], 
                    Hue[0.24730822874130168`, 1, 1], 
                    Hue[0.20151138082108272`, 1, 1], 
                    Hue[0.15901329786186943`, 1, 1], 
                    Hue[0.1197649271403978, 1, 1], 
                    Hue[0.08369802893832214, 1, 1], 
                    Hue[0.05073030875715411, 1, 1], 
                    Hue[0.020769647506886663`, 1, 1], 
                    Hue[0.9937147157833544, 1, 1], 
                    Hue[0.9694593009365832, 1, 1], 
                    Hue[0.9478941084618844, 1, 1], 
                    Hue[0.9289103885338673, 1, 1], 
                    Hue[0.9123988367658288, 1, 1], 
                    Hue[0.8982520275469468, 1, 1], 
                    Hue[0.886366658292858, 1, 1], 
                    Hue[0.8766426050628463, 1, 1], 
                    Hue[0.8689841847218216, 1, 1], 
                    Hue[0.8633013381767353, 1, 1], 
                    Hue[0.8595080716578948, 1, 1], 
                    Hue[0.8575234682582267, 1, 1], 
                    Hue[0.8572726383404587, 1, 1], 
                    Hue[0.8586834192166569, 1, 1], 
                    Hue[0.8616906928539774, 1, 1], 
                    Hue[0.8662315826883799, 1, 1], 
                    Hue[0.8722494769006455, 1, 1], 
                    Hue[0.8796904511863467, 1, 1], 
                    Hue[0.8885044010740425, 1, 1], 
                    Hue[0.8986452227927522, 1, 1], 
                    Hue[0.9100709017328726, 1, 1], 
                    Hue[0.922740508081968, 1, 1], 
                    Hue[0.9366170036872785, 1, 1], 
                    Hue[0.9516664083103602, 1, 1], 
                    Hue[0.9678565539344235, 1, 1], 
                    Hue[0.9851573817997091, 1, 1], 
                    Hue[0.003541029282514706, 1, 1], 
                    Hue[0.022982276990593284`, 1, 1], 
                    Hue[0.04345618989206919, 1, 1], 
                    Hue[0.06494019180123378, 1, 1], 
                    Hue[0.0874131980234504, 1, 1], 
                    Hue[0.11085512776100503`, 1, 1], 
                    Hue[0.1352477810807344, 1, 1], 
                    Hue[0.16057339342287885`, 1, 1], 
                    Hue[0.18681539630932423`, 1, 1], 
                    Hue[0.21395846771373908`, 1, 1], 
                    Hue[0.24198818379898965`, 1, 1], 
                    Hue[0.2708910987900743, 1, 1], 
                    Hue[0.3006539059544776, 1, 1], 
                    Hue[0.33126453314316606`, 1, 1], 
                    Hue[0.3627118699332237, 1, 1], 
                    Hue[0.3949837005086097, 1, 1], 
                    Hue[0.42806988647193905`, 1, 1], 
                    Hue[0.4619600062641948, 1, 1], 
                    Hue[0.4966450931302264, 1, 1], 
                    Hue[0.5321158935007508, 1, 1], 
                    Hue[0.5683632217963384, 1, 1], 
                    Hue[0.6053791741127653, 1, 1], 
                    Hue[0.6431566313515203, 1, 1], 
                    Hue[0.6816868752353401, 1, 1], 
                    Hue[0.72096364429756, 1, 1], 
                    Hue[0.7609790655744311, 1, 1], 
                    Hue[0.801726159467002, 1, 1], 
                    Hue[0.8431986948347605, 1, 1], 
                    Hue[0.8853905620106385, 1, 1], 
                    Hue[0.928294741191026, 1, 1], 
                    Hue[0.9719066681331644, 1, 1], 
                    Hue[0.016219788090091947`, 1, 1], 
                    Hue[0.06123023095704857, 1, 1], 
                    Hue[0.10693189420594468`, 1, 1], 
                    Hue[0.15331972133290422`, 1, 1], 
                    Hue[0.20038888796819257`, 1, 1], 
                    Hue[0.24813299763561023`, 1, 1], 
                    Hue[0.29654916208262616`, 1, 1], 
                    Hue[0.3456310281848976, 1, 1], 
                    Hue[0.3953752971354173, 1, 1], 
                    Hue[0.4457779494411485, 1, 1], 
                    Hue[0.49683472538993373`, 1, 1], 
                    Hue[0.5485422000086002, 1, 1], 
                    Hue[0.6008954329863241, 1, 1], 
                    Hue[0.6538909685908945, 1, 1], 
                    Hue[0.7075250668459268, 1, 1], 
                    Hue[0.7617944518713861, 1, 1], 
                    Hue[0.8166949979465602, 1, 1], 
                    Hue[0.872223828978173, 1, 1], 
                    Hue[0.9283771405632653, 1, 1], 
                    Hue[0.9851524220111436, 1, 1], 
                    Hue[0.04254550702833396, 1, 1], 
                    Hue[0.10055367721831523`, 1, 1], 
                    Hue[0.15917356261530025`, 1, 1], 
                    Hue[0.21840264135365023`, 1, 1], 
                    Hue[0.2782378071599018, 1, 1], 
                    Hue[0.3386767001197204, 1, 1], 
                    Hue[0.39971523956125754`, 1, 1], 
                    Hue[0.46135214668637253`, 1, 1], 
                    Hue[0.5235833706306552, 1, 1], 
                    Hue[0.5864068661836146, 1, 1], 
                    Hue[0.649821296542562, 1, 1], 
                    Hue[0.7138220064371237, 1, 1], 
                    Hue[0.7784087549190002, 1, 1], 
                    Hue[0.8435773141078866, 1, 1], 
                    Hue[0.9093257782873416, 1, 1], 
                    Hue[0.9756515583544458, 1, 1], 
                    Hue[0.04255353459618736, 1, 1], 
                    Hue[0.11002894234378277`, 1, 1], 
                    Hue[0.17807576974946937`, 1, 1], 
                    Hue[0.2466904245232185, 1, 1], 
                    Hue[0.3158733471799709, 1, 1], 
                    Hue[0.3856187575594051, 1, 1], 
                    Hue[0.4559291256723374, 1, 1], 
                    Hue[0.5267985007359997, 1, 1], 
                    Hue[0.5982288136813674, 1, 1], 
                    Hue[0.6702147558152386, 1, 1], 
                    Hue[0.7427566853759748, 1, 1], 
                    Hue[0.8158503838879019, 1, 1], 
                    Hue[0.8894967525160438, 1, 1], 
                    Hue[0.9636945233767934, 1, 1], 
                    Hue[0.03843758514675253, 1, 1], 
                    Hue[0.11372770534876697`, 1, 1], 
                    Hue[0.1895624517637856, 1, 1], 
                    Hue[0.26593871306289235`, 1, 1], 
                    Hue[0.34285896982386044`, 1, 1], 
                    Hue[0.42031468485448653`, 1, 1], 
                    Hue[0.9890850417084717, 1, 1], 
                    Hue[0.890444563820459, 1, 1], 
                    Hue[0.890444563820459, 1, 1], 
                    Hue[0.7918350921023508, 1, 1], 
                    Hue[0.7918350921023508, 1, 1], 
                    Hue[0.6935923344905128, 1, 1], 
                    Hue[0.6935923344905128, 1, 1], 
                    Hue[0.5955973533557162, 1, 1], 
                    Hue[0.5955973533557162, 1, 1], 
                    Hue[0.49769643430527716`, 1, 1], 
                    Hue[0.49769643430527716`, 1, 1], 
                    Hue[0.3999909708344182, 1, 1], 
                    Hue[0.3999909708344182, 1, 1], 
                    Hue[0.30260995757912756`, 1, 1], 
                    Hue[0.30260995757912756`, 1, 1], 
                    Hue[0.20549048009062573`, 1, 1], 
                    Hue[0.20549048009062573`, 1, 1], 
                    Hue[0.10841445415142113`, 1, 1], 
                    Hue[0.10841445415142113`, 1, 1], 
                    Hue[0.011694343660073115`, 1, 1], 
                    Hue[0.011694343660073115`, 1, 1], 
                    Hue[0.915136789552362, 1, 1], 
                    Hue[0.915136789552362, 1, 1], 
                    Hue[0.8187862586312527, 1, 1], 
                    Hue[0.8187862586312527, 1, 1], 
                    Hue[0.7226446234905665, 1, 1], 
                    Hue[0.7226446234905665, 1, 1], 
                    Hue[0.6266956278780738, 1, 1], 
                    Hue[0.6266956278780738, 1, 1], 
                    Hue[0.5309424388597916, 1, 1], 
                    Hue[0.5309424388597916, 1, 1], 
                    Hue[0.4354159473821284, 1, 1], 
                    Hue[0.4354159473821284, 1, 1], 
                    Hue[0.34006128509633016`, 1, 1], 
                    Hue[0.34006128509633016`, 1, 1], 
                    Hue[0.24488834017086, 1, 1], 
                    Hue[0.24488834017086, 1, 1], 
                    Hue[0.1499381665888046, 1, 1], 
                    Hue[0.1499381665888046, 1, 1], 
                    Hue[0.0551497709061452, 1, 1], 
                    Hue[0.0551497709061452, 1, 1], 
                    Hue[0.9605747897944017, 1, 1], 
                    Hue[0.9605747897944017, 1, 1], 
                    Hue[0.8661822384266291, 1, 1], 
                    Hue[0.8661822384266291, 1, 1], 
                    Hue[0.7719815454219947, 1, 1], 
                    Hue[0.7719815454219947, 1, 1], 
                    Hue[0.6779589248969495, 1, 1], 
                    Hue[0.6779589248969495, 1, 1], 
                    Hue[0.584107788669658, 1, 1], 
                    Hue[0.584107788669658, 1, 1], 
                    Hue[0.4904321659403038, 1, 1], 
                    Hue[0.4904321659403038, 1, 1], 
                    Hue[0.3969391264652255, 1, 1], 
                    Hue[0.3969391264652255, 1, 1], 
                    Hue[0.3036082892055693, 1, 1], 
                    Hue[0.3036082892055693, 1, 1], 
                    Hue[0.21045143453790424`, 1, 1], 
                    Hue[0.21045143453790424`, 1, 1], 
                    Hue[0.1174679950897775, 1, 1], 
                    Hue[0.1174679950897775, 1, 1], 
                    Hue[0.024647137886225627`, 1, 1], 
                    Hue[0.024647137886225627`, 1, 1], 
                    Hue[0.9319857820365661, 1, 1], 
                    Hue[0.9319857820365661, 1, 1], 
                    Hue[0.8394897332935869, 1, 1], 
                    Hue[0.8394897332935869, 1, 1], 
                    Hue[0.7471528779093941, 1, 1], 
                    Hue[0.7471528779093941, 1, 1], 
                    Hue[0.6549682687227965, 1, 1], 
                    Hue[0.6549682687227965, 1, 1], 
                    Hue[0.562940161123826, 1, 1], 
                    Hue[0.562940161123826, 1, 1], 
                    Hue[0.4710601350866123, 1, 1], 
                    Hue[0.4710601350866123, 1, 1], 
                    Hue[0.379328026400307, 1, 1], 
                    Hue[0.379328026400307, 1, 1], 
                    Hue[0.2877415089286468, 1, 1], 
                    Hue[0.2877415089286468, 1, 1], 
                    Hue[0.19629620846912263`, 1, 1], 
                    Hue[0.19629620846912263`, 1, 1], 
                    Hue[0.10498770191782866`, 1, 1], 
                    Hue[0.10498770191782866`, 1, 1], 
                    Hue[0.013818248266201809`, 1, 1], 
                    Hue[0.013818248266201809`, 1, 1], 
                    Hue[0.922779355202294, 1, 1], 
                    Hue[0.922779355202294, 1, 1], 
                    Hue[0.8318706546694346, 1, 1], 
                    Hue[0.8318706546694346, 1, 1], 
                    Hue[0.7410886274394506, 1, 1], 
                    Hue[0.7410886274394506, 1, 1], 
                    Hue[0.6504288919503963, 1, 1], 
                    Hue[0.6504288919503963, 1, 1], 
                    Hue[0.5598898317429462, 1, 1], 
                    Hue[0.5598898317429462, 1, 1], 
                    Hue[0.46946392923718244`, 1, 1], 
                    Hue[0.46946392923718244`, 1, 1], 
                    Hue[0.3791530821283772, 1, 1], 
                    Hue[0.3791530821283772, 1, 1], 
                    Hue[0.2889467711448164, 1, 1], 
                    Hue[0.2889467711448164, 1, 1], 
                    Hue[0.19884605935101357`, 1, 1], 
                    Hue[0.19884605935101357`, 1, 1], 
                    Hue[0.10884451371828528`, 1, 1], 
                    Hue[0.10884451371828528`, 1, 1], 
                    Hue[0.01893840191355155, 1, 1], 
                    Hue[0.01893840191355155, 1, 1], 
                    Hue[0.9291246203627189, 1, 1], 
                    Hue[0.9291246203627189, 1, 1], 
                    Hue[0.839398027108012, 1, 1], 
                    Hue[0.839398027108012, 1, 1], 
                    Hue[0.7497518597451729, 1, 1], 
                    Hue[0.7497518597451729, 1, 1], 
                    Hue[0.6601838746800621, 1, 1], 
                    Hue[0.6601838746800621, 1, 1], 
                    Hue[0.5706881943866222, 1, 1], 
                    Hue[0.5706881943866222, 1, 1], 
                    Hue[0.48125901280784156`, 1, 1], 
                    Hue[0.48125901280784156`, 1, 1], 
                    Hue[0.39189064851607397`, 1, 1], 
                    Hue[0.39189064851607397`, 1, 1], 
                    Hue[0.30257891317312197`, 1, 1], 
                    Hue[0.30257891317312197`, 1, 1], 
                    Hue[0.21331627749924545`, 1, 1], 
                    Hue[0.21331627749924545`, 1, 1], 
                    Hue[0.1240969762593327, 1, 1], 
                    Hue[0.1240969762593327, 1, 1], 
                    Hue[0.034915561215019375`, 1, 1], 
                    Hue[0.034915561215019375`, 1, 1], 
                    Hue[0.9457641347466291, 1, 1], 
                    Hue[0.9457641347466291, 1, 1], 
                    Hue[0.8566369738272935, 1, 1], 
                    Hue[0.8566369738272935, 1, 1], 
                    Hue[0.7675262150857177, 1, 1], 
                    Hue[0.7675262150857177, 1, 1], 
                    Hue[0.6784238011201688, 1, 1], 
                    Hue[0.6784238011201688, 1, 1], 
                    Hue[0.5893230207040819, 1, 1], 
                    Hue[0.5893230207040819, 1, 1], 
                    Hue[0.50021495606295, 1, 1], 
                    Hue[0.50021495606295, 1, 1], 
                    Hue[0.4110913775684948, 1, 1], 
                    Hue[0.4110913775684948, 1, 1], 
                    Hue[0.3219424614705361, 1, 1], 
                    Hue[0.3219424614705361, 1, 1], 
                    Hue[0.23275925793759258`, 1, 1], 
                    Hue[0.23275925793759258`, 1, 1], 
                    Hue[0.143532350562769, 1, 1], 
                    Hue[0.143532350562769, 1, 1], 
                    Hue[0.05425179677263266, 1, 1], 
                    Hue[0.05425179677263266, 1, 1], 
                    Hue[0.9649067193442054, 1, 1], 
                    Hue[0.9649067193442054, 1, 1], 
                    Hue[0.8754870852363708, 1, 1], 
                    Hue[0.8754870852363708, 1, 1], 
                    Hue[0.7859800701598657, 1, 1], 
                    Hue[0.7859800701598657, 1, 1], 
                    Hue[0.6963751868911885, 1, 1], 
                    Hue[0.6963751868911885, 1, 1], 
                    Hue[0.6066591090207734, 1, 1], 
                    Hue[0.6066591090207734, 1, 1], 
                    Hue[0.5168198134156986, 1, 1], 
                    Hue[0.5168198134156986, 1, 1], 
                    Hue[0.42684569537233746`, 1, 1], 
                    Hue[0.42684569537233746`, 1, 1], 
                    Hue[0.33672288201474887`, 1, 1], 
                    Hue[0.33672288201474887`, 1, 1], 
                    Hue[0.24643890665017157`, 1, 1], 
                    Hue[0.24643890665017157`, 1, 1], 
                    Hue[0.1559790483703322, 1, 1], 
                    Hue[0.1559790483703322, 1, 1], 
                    Hue[0.06533250362229022, 1, 1], 
                    Hue[0.06533250362229022, 1, 1], 
                    Hue[0.9744852437056882, 1, 1], 
                    Hue[0.9744852437056882, 1, 1], 
                    Hue[0.8834255839195694, 1, 1], 
                    Hue[0.8834255839195694, 1, 1], 
                    Hue[0.7921421823685508, 1, 1], 
                    Hue[0.7921421823685508, 1, 1], 
                    Hue[0.7006241285792536, 1, 1], 
                    Hue[0.7006241285792536, 1, 1], 
                    Hue[0.6088632573112414, 1, 1], 
                    Hue[0.6088632573112414, 1, 1], 
                    Hue[0.5168507348555242, 1, 1], 
                    Hue[0.5168507348555242, 1, 1], 
                    Hue[0.42458376599519027`, 1, 1], 
                    Hue[0.42458376599519027`, 1, 1], 
                    Hue[0.33205862577194645`, 1, 1], 
                    Hue[0.33205862577194645`, 1, 1], 
                    Hue[0.2392775083989476, 1, 1], 
                    Hue[0.2392775083989476, 1, 1], 
                    Hue[0.14624478418476983`, 1, 1], 
                    Hue[0.14624478418476983`, 1, 1], 
                    Hue[0.052970346936419506`, 1, 1], 
                    Hue[0.052970346936419506`, 1, 1], 
                    Hue[0.9594676360426614, 1, 1], 
                    Hue[0.9594676360426614, 1, 1], 
                    Hue[0.8657577363329141, 1, 1], 
                    Hue[0.8657577363329141, 1, 1], 
                    Hue[0.7718665933810455, 1, 1], 
                    Hue[0.7718665933810455, 1, 1], 
                    Hue[0.6778270601834724, 1, 1], 
                    Hue[0.6778270601834724, 1, 1], 
                    Hue[0.5836773317911002, 1, 1], 
                    Hue[0.5836773317911002, 1, 1], 
                    Hue[0.48946425451082115`, 1, 1], 
                    Hue[0.48946425451082115`, 1, 1], 
                    Hue[0.3952418144069382, 1, 1], 
                    Hue[0.3952418144069382, 1, 1], 
                    Hue[0.3010708889653632, 1, 1], 
                    Hue[0.3010708889653632, 1, 1], 
                    Hue[0.207018848047681, 1, 1], 
                    Hue[0.207018848047681, 1, 1], 
                    Hue[0.11316324104925135`, 1, 1], 
                    Hue[0.11316324104925135`, 1, 1], 
                    Hue[0.019585516746522633`, 1, 1], 
                    Hue[0.019585516746522633`, 1, 1], 
                    Hue[0.9263748173191635, 1, 1], 
                    Hue[0.9263748173191635, 1, 1], 
                    Hue[0.8336288784715873, 1, 1], 
                    Hue[0.8336288784715873, 1, 1], 
                    Hue[0.7414537703560893, 1, 1], 
                    Hue[0.7414537703560893, 1, 1], 
                    Hue[0.6499644952350254, 1, 1], 
                    Hue[0.6499644952350254, 1, 1], 
                    Hue[0.559286572819959, 1, 1], 
                    Hue[0.559286572819959, 1, 1], 
                    Hue[0.46955356366737094`, 1, 1], 
                    Hue[0.46955356366737094`, 1, 1], 
                    Hue[0.38090894736594, 1, 1], 
                    Hue[0.38090894736594, 1, 1], 
                    Hue[0.2935085010777411, 1, 1], 
                    Hue[0.2935085010777411, 1, 1], 
                    Hue[0.20750998380319777`, 1, 1], 
                    Hue[0.20750998380319777`, 1, 1], 
                    Hue[0.12308448701168116`, 1, 1], 
                    Hue[0.12308448701168116`, 1, 1], 
                    Hue[0.040405866180151344`, 1, 1], 
                    Hue[0.040405866180151344`, 1, 1], 
                    Hue[0.9596577933041008, 1, 1], 
                    Hue[0.9596577933041008, 1, 1], 
                    Hue[0.8810254867376445, 1, 1], 
                    Hue[0.8810254867376445, 1, 1], 
                    Hue[0.8046989087260993, 1, 1], 
                    Hue[0.8046989087260993, 1, 1], 
                    Hue[0.7308629301685772, 1, 1], 
                    Hue[0.7308629301685772, 1, 1], 
                    Hue[0.659694095351627, 1, 1], 
                    Hue[0.659694095351627, 1, 1], 
                    Hue[0.591355858433406, 1, 1], 
                    Hue[0.591355858433406, 1, 1], 
                    Hue[0.5259960179322813, 1, 1], 
                    Hue[0.5259960179322813, 1, 1], 
                    Hue[0.4637384836446359, 1, 1], 
                    Hue[0.4637384836446359, 1, 1], 
                    Hue[0.4046827309417128, 1, 1], 
                    Hue[0.4046827309417128, 1, 1], 
                    Hue[0.34890027740507623`, 1, 1], 
                    Hue[0.34890027740507623`, 1, 1], 
                    Hue[0.2964351349133147, 1, 1], 
                    Hue[0.2964351349133147, 1, 1], 
                    Hue[0.24730822874130168`, 1, 1], 
                    Hue[0.24730822874130168`, 1, 1], 
                    Hue[0.20151138082108272`, 1, 1], 
                    Hue[0.20151138082108272`, 1, 1], 
                    Hue[0.15901329786186943`, 1, 1], 
                    Hue[0.15901329786186943`, 1, 1], 
                    Hue[0.1197649271403978, 1, 1], 
                    Hue[0.1197649271403978, 1, 1], 
                    Hue[0.08369802893832214, 1, 1], 
                    Hue[0.08369802893832214, 1, 1], 
                    Hue[0.05073030875715411, 1, 1], 
                    Hue[0.05073030875715411, 1, 1], 
                    Hue[0.020769647506886663`, 1, 1], 
                    Hue[0.020769647506886663`, 1, 1], 
                    Hue[0.9937147157833544, 1, 1], 
                    Hue[0.9937147157833544, 1, 1], 
                    Hue[0.9694593009365832, 1, 1], 
                    Hue[0.9694593009365832, 1, 1], 
                    Hue[0.9478941084618844, 1, 1], 
                    Hue[0.9478941084618844, 1, 1], 
                    Hue[0.9289103885338673, 1, 1], 
                    Hue[0.9289103885338673, 1, 1], 
                    Hue[0.9123988367658288, 1, 1], 
                    Hue[0.9123988367658288, 1, 1], 
                    Hue[0.8982520275469468, 1, 1], 
                    Hue[0.8982520275469468, 1, 1], 
                    Hue[0.886366658292858, 1, 1], 
                    Hue[0.886366658292858, 1, 1], 
                    Hue[0.8766426050628463, 1, 1], 
                    Hue[0.8766426050628463, 1, 1], 
                    Hue[0.8689841847218216, 1, 1], 
                    Hue[0.8689841847218216, 1, 1], 
                    Hue[0.8633013381767353, 1, 1], 
                    Hue[0.8633013381767353, 1, 1], 
                    Hue[0.8595080716578948, 1, 1], 
                    Hue[0.8595080716578948, 1, 1], 
                    Hue[0.8575234682582267, 1, 1], 
                    Hue[0.8575234682582267, 1, 1], 
                    Hue[0.8572726383404587, 1, 1], 
                    Hue[0.8572726383404587, 1, 1], 
                    Hue[0.8586834192166569, 1, 1], 
                    Hue[0.8586834192166569, 1, 1], 
                    Hue[0.8616906928539774, 1, 1], 
                    Hue[0.8616906928539774, 1, 1], 
                    Hue[0.8662315826883799, 1, 1], 
                    Hue[0.8662315826883799, 1, 1], 
                    Hue[0.8722494769006455, 1, 1], 
                    Hue[0.8722494769006455, 1, 1], 
                    Hue[0.8796904511863467, 1, 1], 
                    Hue[0.8796904511863467, 1, 1], 
                    Hue[0.8885044010740425, 1, 1], 
                    Hue[0.8885044010740425, 1, 1], 
                    Hue[0.8986452227927522, 1, 1], 
                    Hue[0.8986452227927522, 1, 1], 
                    Hue[0.9100709017328726, 1, 1], 
                    Hue[0.9100709017328726, 1, 1], 
                    Hue[0.922740508081968, 1, 1], 
                    Hue[0.922740508081968, 1, 1], 
                    Hue[0.9366170036872785, 1, 1], 
                    Hue[0.9366170036872785, 1, 1], 
                    Hue[0.9516664083103602, 1, 1], 
                    Hue[0.9516664083103602, 1, 1], 
                    Hue[0.9678565539344235, 1, 1], 
                    Hue[0.9678565539344235, 1, 1], 
                    Hue[0.9851573817997091, 1, 1], 
                    Hue[0.9851573817997091, 1, 1], 
                    Hue[0.003541029282514706, 1, 1], 
                    Hue[0.003541029282514706, 1, 1], 
                    Hue[0.022982276990593284`, 1, 1], 
                    Hue[0.022982276990593284`, 1, 1], 
                    Hue[0.04345618989206919, 1, 1], 
                    Hue[0.04345618989206919, 1, 1], 
                    Hue[0.06494019180123378, 1, 1], 
                    Hue[0.06494019180123378, 1, 1], 
                    Hue[0.0874131980234504, 1, 1], 
                    Hue[0.0874131980234504, 1, 1], 
                    Hue[0.11085512776100503`, 1, 1], 
                    Hue[0.11085512776100503`, 1, 1], 
                    Hue[0.1352477810807344, 1, 1], 
                    Hue[0.1352477810807344, 1, 1], 
                    Hue[0.16057339342287885`, 1, 1], 
                    Hue[0.16057339342287885`, 1, 1], 
                    Hue[0.18681539630932423`, 1, 1], 
                    Hue[0.18681539630932423`, 1, 1], 
                    Hue[0.21395846771373908`, 1, 1], 
                    Hue[0.21395846771373908`, 1, 1], 
                    Hue[0.24198818379898965`, 1, 1], 
                    Hue[0.24198818379898965`, 1, 1], 
                    Hue[0.2708910987900743, 1, 1], 
                    Hue[0.2708910987900743, 1, 1], 
                    Hue[0.3006539059544776, 1, 1], 
                    Hue[0.3006539059544776, 1, 1], 
                    Hue[0.33126453314316606`, 1, 1], 
                    Hue[0.33126453314316606`, 1, 1], 
                    Hue[0.3627118699332237, 1, 1], 
                    Hue[0.3627118699332237, 1, 1], 
                    Hue[0.3949837005086097, 1, 1], 
                    Hue[0.3949837005086097, 1, 1], 
                    Hue[0.42806988647193905`, 1, 1], 
                    Hue[0.42806988647193905`, 1, 1], 
                    Hue[0.4619600062641948, 1, 1], 
                    Hue[0.4619600062641948, 1, 1], 
                    Hue[0.4966450931302264, 1, 1], 
                    Hue[0.4966450931302264, 1, 1], 
                    Hue[0.5321158935007508, 1, 1], 
                    Hue[0.5321158935007508, 1, 1], 
                    Hue[0.5683632217963384, 1, 1], 
                    Hue[0.5683632217963384, 1, 1], 
                    Hue[0.6053791741127653, 1, 1], 
                    Hue[0.6053791741127653, 1, 1], 
                    Hue[0.6431566313515203, 1, 1], 
                    Hue[0.6431566313515203, 1, 1], 
                    Hue[0.6816868752353401, 1, 1], 
                    Hue[0.6816868752353401, 1, 1], 
                    Hue[0.72096364429756, 1, 1], 
                    Hue[0.72096364429756, 1, 1], 
                    Hue[0.7609790655744311, 1, 1], 
                    Hue[0.7609790655744311, 1, 1], 
                    Hue[0.801726159467002, 1, 1], 
                    Hue[0.801726159467002, 1, 1], 
                    Hue[0.8431986948347605, 1, 1], 
                    Hue[0.8431986948347605, 1, 1], 
                    Hue[0.8853905620106385, 1, 1], 
                    Hue[0.8853905620106385, 1, 1], 
                    Hue[0.928294741191026, 1, 1], 
                    Hue[0.928294741191026, 1, 1], 
                    Hue[0.9719066681331644, 1, 1], 
                    Hue[0.9719066681331644, 1, 1], 
                    Hue[0.016219788090091947`, 1, 1], 
                    Hue[0.016219788090091947`, 1, 1], 
                    Hue[0.06123023095704857, 1, 1], 
                    Hue[0.06123023095704857, 1, 1], 
                    Hue[0.10693189420594468`, 1, 1], 
                    Hue[0.10693189420594468`, 1, 1], 
                    Hue[0.15331972133290422`, 1, 1], 
                    Hue[0.15331972133290422`, 1, 1], 
                    Hue[0.20038888796819257`, 1, 1], 
                    Hue[0.20038888796819257`, 1, 1], 
                    Hue[0.24813299763561023`, 1, 1], 
                    Hue[0.24813299763561023`, 1, 1], 
                    Hue[0.29654916208262616`, 1, 1], 
                    Hue[0.29654916208262616`, 1, 1], 
                    Hue[0.3456310281848976, 1, 1], 
                    Hue[0.3456310281848976, 1, 1], 
                    Hue[0.3953752971354173, 1, 1], 
                    Hue[0.3953752971354173, 1, 1], 
                    Hue[0.4457779494411485, 1, 1], 
                    Hue[0.4457779494411485, 1, 1], 
                    Hue[0.49683472538993373`, 1, 1], 
                    Hue[0.49683472538993373`, 1, 1], 
                    Hue[0.5485422000086002, 1, 1], 
                    Hue[0.5485422000086002, 1, 1], 
                    Hue[0.6008954329863241, 1, 1], 
                    Hue[0.6008954329863241, 1, 1], 
                    Hue[0.6538909685908945, 1, 1], 
                    Hue[0.6538909685908945, 1, 1], 
                    Hue[0.7075250668459268, 1, 1], 
                    Hue[0.7075250668459268, 1, 1], 
                    Hue[0.7617944518713861, 1, 1], 
                    Hue[0.7617944518713861, 1, 1], 
                    Hue[0.8166949979465602, 1, 1], 
                    Hue[0.8166949979465602, 1, 1], 
                    Hue[0.872223828978173, 1, 1], 
                    Hue[0.872223828978173, 1, 1], 
                    Hue[0.9283771405632653, 1, 1], 
                    Hue[0.9283771405632653, 1, 1], 
                    Hue[0.9851524220111436, 1, 1], 
                    Hue[0.9851524220111436, 1, 1], 
                    Hue[0.04254550702833396, 1, 1], 
                    Hue[0.04254550702833396, 1, 1], 
                    Hue[0.10055367721831523`, 1, 1], 
                    Hue[0.10055367721831523`, 1, 1], 
                    Hue[0.15917356261530025`, 1, 1], 
                    Hue[0.15917356261530025`, 1, 1], 
                    Hue[0.21840264135365023`, 1, 1], 
                    Hue[0.21840264135365023`, 1, 1], 
                    Hue[0.2782378071599018, 1, 1], 
                    Hue[0.2782378071599018, 1, 1], 
                    Hue[0.3386767001197204, 1, 1], 
                    Hue[0.3386767001197204, 1, 1], 
                    Hue[0.39971523956125754`, 1, 1], 
                    Hue[0.39971523956125754`, 1, 1], 
                    Hue[0.46135214668637253`, 1, 1], 
                    Hue[0.46135214668637253`, 1, 1], 
                    Hue[0.5235833706306552, 1, 1], 
                    Hue[0.5235833706306552, 1, 1], 
                    Hue[0.5864068661836146, 1, 1], 
                    Hue[0.5864068661836146, 1, 1], 
                    Hue[0.649821296542562, 1, 1], 
                    Hue[0.649821296542562, 1, 1], 
                    Hue[0.7138220064371237, 1, 1], 
                    Hue[0.7138220064371237, 1, 1], 
                    Hue[0.7784087549190002, 1, 1], 
                    Hue[0.7784087549190002, 1, 1], 
                    Hue[0.8435773141078866, 1, 1], 
                    Hue[0.8435773141078866, 1, 1], 
                    Hue[0.9093257782873416, 1, 1], 
                    Hue[0.9093257782873416, 1, 1], 
                    Hue[0.9756515583544458, 1, 1], 
                    Hue[0.9756515583544458, 1, 1], 
                    Hue[0.04255353459618736, 1, 1], 
                    Hue[0.04255353459618736, 1, 1], 
                    Hue[0.11002894234378277`, 1, 1], 
                    Hue[0.11002894234378277`, 1, 1], 
                    Hue[0.17807576974946937`, 1, 1], 
                    Hue[0.17807576974946937`, 1, 1], 
                    Hue[0.2466904245232185, 1, 1], 
                    Hue[0.2466904245232185, 1, 1], 
                    Hue[0.3158733471799709, 1, 1], 
                    Hue[0.3158733471799709, 1, 1], 
                    Hue[0.3856187575594051, 1, 1], 
                    Hue[0.3856187575594051, 1, 1], 
                    Hue[0.4559291256723374, 1, 1], 
                    Hue[0.4559291256723374, 1, 1], 
                    Hue[0.5267985007359997, 1, 1], 
                    Hue[0.5267985007359997, 1, 1], 
                    Hue[0.5982288136813674, 1, 1], 
                    Hue[0.5982288136813674, 1, 1], 
                    Hue[0.6702147558152386, 1, 1], 
                    Hue[0.6702147558152386, 1, 1], 
                    Hue[0.7427566853759748, 1, 1], 
                    Hue[0.7427566853759748, 1, 1], 
                    Hue[0.8158503838879019, 1, 1], 
                    Hue[0.8158503838879019, 1, 1], 
                    Hue[0.8894967525160438, 1, 1], 
                    Hue[0.8894967525160438, 1, 1], 
                    Hue[0.9636945233767934, 1, 1], 
                    Hue[0.9636945233767934, 1, 1], 
                    Hue[0.03843758514675253, 1, 1], 
                    Hue[0.03843758514675253, 1, 1], 
                    Hue[0.11372770534876697`, 1, 1], 
                    Hue[0.11372770534876697`, 1, 1], 
                    Hue[0.1895624517637856, 1, 1], 
                    Hue[0.1895624517637856, 1, 1], 
                    Hue[0.26593871306289235`, 1, 1], 
                    Hue[0.26593871306289235`, 1, 1], 
                    Hue[0.34285896982386044`, 1, 1], 
                    Hue[0.34285896982386044`, 1, 1], 
                    Hue[0.42031468485448653`, 1, 1]}],
                   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
                   Axes->{True, None},
                   AxesOrigin->{0, 0},
                   
                   Epilog->LineBox[{{1, 0.000664668848283562}, {
                    2, 0.0007578249757784219}, {3, 0.0008642419812624661}, {
                    4, 0.0009837440297558759}, {5, 0.0011187530665935752`}, {
                    6, 0.00127026705304837}, {7, 0.0014407827005841357`}, {
                    8, 0.0016324575192494697`}, {9, 0.0018469143646733174`}, {
                    10, 0.0020875272332460154`}, {
                    11, 0.0023571779792675752`}, {12, 0.002657382543073407}, {
                    13, 0.0029934640484062455`}, {
                    14, 0.0033673795303978604`}, {
                    15, 0.0037830445000899115`}, {16, 0.004245348595680484}, {
                    17, 0.004758296131193935}, {18, 0.005325861929652516}, {
                    19, 0.005954032823546849}, {20, 0.006648894812744901}, {
                    21, 0.007413337365274173}, {22, 0.008255961576166157}, {
                    23, 0.009181721326599274}, {24, 0.010199667647376759`}, {
                    25, 0.011314832152383335`}, {26, 0.012534941237711757`}, {
                    27, 0.013868397105800144`}, {28, 0.015323063450822292`}, {
                    29, 0.016906143722724996`}, {30, 0.018628298995364164`}, {
                    31, 0.020496553105252308`}, {32, 0.02252104662660519}, {
                    33, 0.024710776316784058`}, {34, 0.027074982320335568`}, {
                    35, 0.029621695252019756`}, {36, 0.032362003145395654`}, {
                    37, 0.03530363221228}, {38, 0.038455441062931875`}, {
                    39, 0.04182646577968378}, {40, 0.045423915091767325`}, {
                    41, 0.04925692036876082}, {42, 0.05333020871688937}, {
                    43, 0.05765115187281245}, {44, 0.0622257674990335}, {
                    45, 0.06705670452812336}, {46, 0.07214708022323275}, {
                    47, 0.07749983054642036}, {48, 0.08311439321732401}, {
                    49, 0.08898877829707996}, {50, 0.09512052037227856}, {
                    51, 0.10150663774809097`}, {52, 0.1081354253800436}, {
                    53, 0.1150019061040271}, {54, 0.1220936144131497}, {
                    55, 0.1293972064525951}, {56, 0.13689892989489563`}, {
                    57, 0.14457712758239238`}, {58, 0.15241315214460283`}, {
                    59, 0.16038190903242153`}, {60, 0.1684615566075817}, {
                    61, 0.17662172799118284`}, {62, 0.18483196842932537`}, {
                    63, 0.19306250340813078`}, {64, 0.2012767371394998}, {
                    65, 0.20944014406619127`}, {66, 0.21751560159964423`}, {
                    67, 0.2254641288338429}, {68, 0.2332454219851741}, {
                    69, 0.24081985861409405`}, {70, 0.24814634601642171`}, {
                    71, 0.2551860454597357}, {72, 0.26189651111972095`}, {
                    73, 0.26824207172577524`}, {74, 0.2741828891974974}, {
                    75, 0.27968416290233983`}, {76, 0.2847149872488188}, {
                    77, 0.28924272537030293`}, {78, 0.29324323480954684`}, {
                    79, 0.29669005447292734`}, {80, 0.2995684888957312}, {
                    81, 0.3018620657122918}, {82, 0.3035644537678138}, {
                    83, 0.30466888785328816`}, {84, 0.305180352289716}, {
                    85, 0.3051033785076924}, {86, 0.3044531797231474}, {
                    87, 0.3032499216205878}, {88, 0.30151529073836747`}, {
                    89, 0.29928130094478006`}, {90, 0.29658211575418286`}, {
                    91, 0.29345613837786805`}, {92, 0.2899500583888158}, {
                    93, 0.28611061885544065`}, {94, 0.281987712769225}, {
                    95, 0.27763458297817006`}, {96, 0.2731040735661529}, {
                    97, 0.2684499236456921}, {98, 0.26372519850162407`}, {
                    99, 0.25898111007698604`}, {100, 0.2542669640422208}, {
                    101, 0.24962607547812987`}, {102, 0.2451006904660024}, {
                    103, 0.2407254513760431}, {104, 0.23652999170688682`}, {
                    105, 0.23253907066447196`}, {106, 0.22877078069950094`}, {
                    107, 0.22523885874077418`}, {108, 0.2219497583587372}, {
                    109, 0.21890453854473463`}, {110, 0.21610496605481777`}, {
                    111, 0.21354656728857724`}, {112, 0.21122466686562755`}, {
                    113, 0.20913711767375845`}, {114, 0.2072758475357398}, {
                    115, 0.20563683972122598`}, {116, 0.2042144849955796}, {
                    117, 0.20300468425376453`}, {118, 0.2020066182357582}, {
                    119, 0.2012241133441278}, {120, 0.20066355626120483`}, {
                    121, 0.20033675032379364`}, {122, 0.2002585178414794}, {
                    123, 0.20044349024995206`}, {124, 0.200904788079577}, {
                    125, 0.20165375261870563`}, {126, 0.2026986286273951}, {
                    127, 0.2040397524877515}, {128, 0.205676877836494}, {
                    129, 0.2076062432423218}, {130, 0.20981912254695656`}, {
                    131, 0.21230330197935168`}, {132, 0.2150453624253769}, {
                    133, 0.21803000030396835`}, {134, 0.2212398536506614}, {
                    135, 0.22465651833649647`}, {136, 0.22825894875125857`}, {
                    137, 0.23202606115224791`}, {138, 0.23593876903811686`}, {
                    139, 0.23997423756735528`}, {140, 0.244113217539779}, {
                    141, 0.2483363620080084}, {142, 0.2526211336341694}, {
                    143, 0.2569523185113574}, {144, 0.2613107331206272}, {
                    145, 0.2656778731696579}, {146, 0.2700404107000231}, {
                    147, 0.2743815278442918}, {148, 0.278685809033522}, {
                    149, 0.28294240676743787`}, {150, 0.2871360731217418}, {
                    151, 0.29125595042959995`}, {152, 0.29528955271190627`}, {
                    153, 0.299228184700775}, {154, 0.3030599076373242}, {
                    155, 0.3067753173207391}, {156, 0.31036533305072056`}, {
                    157, 0.31382251148178053`}, {158, 0.31713754919580145`}, {
                    159, 0.32030483692599415`}, {160, 0.3233157296612615}, {
                    161, 0.3261646838367019}, {162, 0.3288455578407144}, {
                    163, 0.3313507737565988}, {164, 0.33367703222519535`}, {
                    165, 0.33581938918725435`}, {166, 0.3377711371853463}, {
                    167, 0.33953009665770134`}, {168, 0.34109267616594374`}, {
                    169, 0.3424544603684473}, {170, 0.3436120225553153}, {
                    171, 0.34456225034033994`}, {172, 0.34530432483727314`}, {
                    173, 0.34583417891188684`}, {174, 0.34614845491660057`}, {
                    175, 0.34624903348363184`}, {176, 0.3461332009271847}, {
                    177, 0.3457996058748039}, {178, 0.3452491686542008}, {
                    179, 0.34448279393761716`}, {180, 0.3435000913327909}, {
                    181, 0.3423032677719288}, {182, 0.3408921639498197}, {
                    183, 0.33926889642340363`}, {184, 0.3374349908636319}, {
                    185, 0.3353925758858055}, {186, 0.33314500092582017`}, {
                    187, 0.3306949005584111}, {188, 0.32804702465442587`}, {
                    189, 0.32520372622224464`}, {190, 0.322173103737576}, {
                    191, 0.3189573533941454}, {192, 0.31556094764975107`}, {
                    193, 0.3119908533406933}, {194, 0.30825051131030745`}, {
                    195, 0.30434627720431023`}, {196, 0.3002852528222341}, {
                    197, 0.29607335693637815`}, {198, 0.29171910266007883`}, {
                    199, 0.2872284276059091}, {200, 0.28261059079894446`}, {
                    201, 0.2778724288784171}, {202, 0.27301980928850683`}, {
                    203, 0.26806223527064726`}, {204, 0.2630046812916387}, {
                    205, 0.25785719771069154`}, {206, 0.2526286355583951}, {
                    207, 0.24732585659345746`}, {208, 0.24195855321537207`}, {
                    209, 0.23653358351771342`}, {210, 0.2310612431805267}, {
                    211, 0.22554784551728368`}, {212, 0.22000273469118897`}, {
                    213, 0.21443353450328032`}, {214, 0.20884957007158925`}, {
                    215, 0.2032569076949167}, {216, 0.19766514228703305`}, {
                    217, 0.1920801360379522}, {218, 0.18651072625107135`}, {
                    219, 0.1809638615685871}, {220, 0.17544693144378382`}, {
                    221, 0.1699662029126255}, {222, 0.16452835193572998`}, {
                    223, 0.1591405414104012}, {224, 0.1538075784555804}, {
                    225, 0.14853739118751566`}, {226, 0.14333374957006437`}, {
                    227, 0.13820238522581701`}, {228, 0.1331482959727786}, {
                    229, 0.12817604197316176`}, {230, 0.12328979286015337`}, {
                    231, 0.1184944185453906}, {232, 0.11379308044375856`}, {
                    233, 0.10918961605542017`}, {234, 0.10468534367902582`}, {
                    235, 0.10028529231125609`}, {236, 0.09599010278565637}, {
                    237, 0.09180311548526479}, {238, 0.08772586588157019}, {
                    239, 0.08375958829813208}, {240, 0.07990594418213398}, {
                    241, 0.07616473206131731}, {242, 0.07253762318455177}, {
                    243, 0.06902405594024237}, {244, 0.06562589736100348}, {
                    245, 0.06234053998751813}, {246, 0.059168502737609416`}, {
                    247, 0.056110074025049764`}, {248, 0.05316302748491828}, {
                    249, 0.0503274687300253}, {250, 0.047601231688745645`}, {
                    251, 0.04498285349295762}}],
                   Frame->True,
                   FrameTicks->{{{1., 
                    FormBox[
                    RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
                    FormBox[
                    RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
                    FormBox[
                    RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
                    FormBox[
                    RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
                    FormBox["0", TraditionalForm]}, {157.25, 
                    FormBox["2.5`", TraditionalForm]}, {188.5, 
                    FormBox["5", TraditionalForm]}, {219.75, 
                    FormBox["7.5`", TraditionalForm]}, {251., 
                    FormBox["10", TraditionalForm]}}, Automatic, None, None},
                   PlotRange->{-0.1, 1.1},
                   PlotRangeClipping->True,
                   PlotRangePadding->{
                    Scaled[0.02], Automatic}], 24->
                  GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyF12lQFVcWB/DGGESjBnFFyfgwQhwVFRBw94/sLggIyE7De+w8UCdoXGIa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                    "], {
                    {GrayLevel[0.5], EdgeForm[None], 
                    GraphicsGroupBox[PolygonBox[CompressedData["
1:eJwtlnnUz0UUh++MLCEhSiolS7IlqRBatEi0KLJFJFtFyRJJ2YUkSpJU2iTJ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                    
                    "]]]}, {{}, {}, {
                    {Hue[0.5373228843195937, 1, 1], LineBox[{1, 2}]}, 
                    {Hue[0.4403401308194928, 1, 1], LineBox[{2, 3}]}, 
                    {Hue[0.3435613420702442, 1, 1], LineBox[{3, 4}]}, 
                    {Hue[0.24690243660616393`, 1, 1], LineBox[{4, 5}]}, 
                    {Hue[0.150454945879873, 1, 1], LineBox[{5, 6}]}, 
                    {Hue[0.054253709597173946`, 1, 1], LineBox[{6, 7}]}, 
                    {Hue[0.9582329689095465, 1, 1], LineBox[{7, 8}]}, 
                    {Hue[0.8624154616652879, 1, 1], LineBox[{8, 9}]}, 
                    {Hue[0.7667920547714719, 1, 1], LineBox[{9, 10}]}, 
                    {Hue[0.6713539032786822, 1, 1], LineBox[{10, 11}]}, 
                    {Hue[0.5761322441426646, 1, 1], LineBox[{11, 12}]}, 
                    {Hue[0.4810681788903602, 1, 1], LineBox[{12, 13}]}, 
                    {Hue[0.38618845496889015`, 1, 1], LineBox[{13, 14}]}, 
                    {Hue[0.2914937307005535, 1, 1], LineBox[{14, 15}]}, 
                    {Hue[0.19699709955287387`, 1, 1], LineBox[{15, 16}]}, 
                    {Hue[0.1026633796553843, 1, 1], LineBox[{16, 17}]}, 
                    {Hue[0.008503512405973614, 1, 1], LineBox[{17, 18}]}, 
                    {Hue[0.9145316811967742, 1, 1], LineBox[{18, 19}]}, 
                    {Hue[0.8207243806703278, 1, 1], LineBox[{19, 20}]}, 
                    {Hue[0.7270951531531911, 1, 1], LineBox[{20, 21}]}, 
                    {Hue[0.6336280106405164, 1, 1], LineBox[{21, 22}]}, 
                    {Hue[0.5403238375965598, 1, 1], LineBox[{22, 23}]}, 
                    {Hue[0.44719515461189513`, 1, 1], LineBox[{23, 24}]}, 
                    {Hue[0.3542219104665249, 1, 1], LineBox[{24, 25}]}, 
                    {Hue[0.261406034757453, 1, 1], LineBox[{25, 26}]}, 
                    {Hue[0.16874846532010704`, 1, 1], LineBox[{26, 27}]}, 
                    {Hue[0.07625049917109356, 1, 1], LineBox[{27, 28}]}, 
                    {Hue[0.983898824560589, 1, 1], LineBox[{28, 29}]}, 
                    {Hue[0.8917010932212673, 1, 1], LineBox[{29, 30}]}, 
                    {Hue[0.7996520993560525, 1, 1], LineBox[{30, 31}]}, 
                    {Hue[0.707743391289746, 1, 1], LineBox[{31, 32}]}, 
                    {Hue[0.6159813669193629, 1, 1], LineBox[{32, 33}]}, 
                    {Hue[0.5243603446357614, 1, 1], LineBox[{33, 34}]}, 
                    {Hue[0.4328738691798968, 1, 1], LineBox[{34, 35}]}, 
                    {Hue[0.3415208785271995, 1, 1], LineBox[{35, 36}]}, 
                    {Hue[0.2502971034620979, 1, 1], LineBox[{36, 37}]}, 
                    {Hue[0.15920419790548215`, 1, 1], LineBox[{37, 38}]}, 
                    {Hue[0.06823149246682571, 1, 1], LineBox[{38, 39}]}, 
                    {Hue[0.9773857509141324, 1, 1], LineBox[{39, 40}]}, 
                    {Hue[0.8866550914096966, 1, 1], LineBox[{40, 41}]}, 
                    {Hue[0.7960381183778849, 1, 1], LineBox[{41, 42}]}, 
                    {Hue[0.7055349171841978, 1, 1], LineBox[{42, 43}]}, 
                    {Hue[0.6151380108443617, 1, 1], LineBox[{43, 44}]}, 
                    {Hue[0.5248452588742869, 1, 1], LineBox[{44, 45}]}, 
                    {Hue[0.4346517215644478, 1, 1], LineBox[{45, 46}]}, 
                    {Hue[0.3445541576573799, 1, 1], LineBox[{46, 47}]}, 
                    {Hue[0.2545477968642205, 1, 1], LineBox[{47, 48}]}, 
                    {Hue[0.16462741410865095`, 1, 1], LineBox[{48, 49}]}, 
                    {Hue[0.07479106859346756, 1, 1], LineBox[{49, 50}]}, 
                    {Hue[0.9850329040069344, 1, 1], LineBox[{50, 51}]}, 
                    {Hue[0.8953473747186199, 1, 1], LineBox[{51, 52}]}, 
                    {Hue[0.8057307072880856, 1, 1], LineBox[{52, 53}]}, 
                    {Hue[0.7161762779604128, 1, 1], LineBox[{53, 54}]}, 
                    {Hue[0.6266791615905233, 1, 1], LineBox[{54, 55}]}, 
                    {Hue[0.537233630987699, 1, 1], LineBox[{55, 56}]}, 
                    {Hue[0.44783486011628426`, 1, 1], LineBox[{56, 57}]}, 
                    {Hue[0.35847490168618695`, 1, 1], LineBox[{57, 58}]}, 
                    {Hue[0.2691480839616139, 1, 1], LineBox[{58, 59}]}, 
                    {Hue[0.17984719749350703`, 1, 1], LineBox[{59, 60}]}, 
                    {Hue[0.0905655025466636, 1, 1], LineBox[{60, 61}]}, 
                    {Hue[0.0012950694149223545`, 1, 1], LineBox[{61, 62}]}, 
                    {Hue[0.9120293898934388, 1, 1], LineBox[{62, 63}]}, 
                    {Hue[0.8227595802498255, 1, 1], LineBox[{63, 64}]}, 
                    {Hue[0.7334770677884576, 1, 1], LineBox[{64, 65}]}, 
                    {Hue[0.6441724629825096, 1, 1], LineBox[{65, 66}]}, 
                    {Hue[0.5548371466919941, 1, 1], LineBox[{66, 67}]}, 
                    {Hue[0.465461449193391, 1, 1], LineBox[{67, 68}]}, 
                    {Hue[0.376034319947191, 1, 1], LineBox[{68, 69}]}, 
                    {Hue[0.2865456196687959, 1, 1], LineBox[{69, 70}]}, 
                    {Hue[0.19698327643107333`, 1, 1], LineBox[{70, 71}]}, 
                    {Hue[0.1073368478949821, 1, 1], LineBox[{71, 72}]}, 
                    {Hue[0.01759296362047231, 1, 1], LineBox[{72, 73}]}, 
                    {Hue[0.9277393289210646, 1, 1], LineBox[{73, 74}]}, 
                    {Hue[0.8377620797664849, 1, 1], LineBox[{74, 75}]}, 
                    {Hue[0.7476481320246258, 1, 1], LineBox[{75, 76}]}, 
                    {Hue[0.6573829603765364, 1, 1], LineBox[{76, 77}]}, 
                    {Hue[0.5669519357335853, 1, 1], LineBox[{77, 78}]}, 
                    {Hue[0.47633986833715763`, 1, 1], LineBox[{78, 79}]}, 
                    {Hue[0.3855322472772245, 1, 1], LineBox[{79, 80}]}, 
                    {Hue[0.29451264153189877`, 1, 1], LineBox[{80, 81}]}, 
                    {Hue[0.2032663941206125, 1, 1], LineBox[{81, 82}]}, 
                    {Hue[0.1117779759296776, 1, 1], LineBox[{82, 83}]}, 
                    {Hue[0.020033942761572048`, 1, 1], LineBox[{83, 84}]}, 
                    {Hue[0.9280205709389144, 1, 1], LineBox[{84, 85}]}, 
                    {Hue[0.8357247215490928, 1, 1], LineBox[{85, 86}]}, 
                    {Hue[0.7431357669534746, 1, 1], LineBox[{86, 87}]}, 
                    {Hue[0.6502444277652975, 1, 1], LineBox[{87, 88}]}, 
                    {Hue[0.5570441913618924, 1, 1], LineBox[{88, 89}]}, 
                    {Hue[0.4635317224138139, 1, 1], LineBox[{89, 90}]}, 
                    {Hue[0.36970776965283664`, 1, 1], LineBox[{90, 91}]}, 
                    {Hue[0.2755740273533197, 1, 1], LineBox[{91, 92}]}, 
                    {Hue[0.18114110308239148`, 1, 1], LineBox[{92, 93}]}, 
                    {Hue[0.08642018714458502, 1, 1], LineBox[{93, 94}]}, 
                    {Hue[0.991429466836158, 1, 1], LineBox[{94, 95}]}, 
                    {Hue[0.8961918398709002, 1, 1], LineBox[{95, 96}]}, 
                    {Hue[0.8007328384798726, 1, 1], LineBox[{96, 97}]}, 
                    {Hue[0.7050864037553836, 1, 1], LineBox[{97, 98}]}, 
                    {Hue[0.609285542248224, 1, 1], LineBox[{98, 99}]}, 
                    {Hue[0.5133709156062596, 1, 1], LineBox[{99, 100}]}, 
                    {Hue[0.4173825756025821, 1, 1], LineBox[{100, 101}]}, 
                    {Hue[0.32136569104301854`, 1, 1], LineBox[{101, 102}]}, 
                    {Hue[0.22536565076658127`, 1, 1], LineBox[{102, 103}]}, 
                    {Hue[0.12942796312066054`, 1, 1], LineBox[{103, 104}]}, 
                    {Hue[0.03360344686736547, 1, 1], LineBox[{104, 105}]}, 
                    {Hue[0.9379393315284854, 1, 1], LineBox[{105, 106}]}, 
                    {Hue[0.8424879868571606, 1, 1], LineBox[{106, 107}]}, 
                    {Hue[0.7473036316593907, 1, 1], LineBox[{107, 108}]}, 
                    {Hue[0.6524441982990108, 1, 1], LineBox[{108, 109}]}, 
                    {Hue[0.5579734697208485, 1, 1], LineBox[{109, 110}]}, 
                    {Hue[0.46395883956449696`, 1, 1], LineBox[{110, 111}]}, 
                    {Hue[0.3704781574462508, 1, 1], LineBox[{111, 112}]}, 
                    {Hue[0.2776173958945541, 1, 1], LineBox[{112, 113}]}, 
                    {Hue[0.18547449142920566`, 1, 1], LineBox[{113, 114}]}, 
                    {Hue[0.09415329417638389, 1, 1], LineBox[{114, 115}]}, 
                    {Hue[0.003775040291353455, 1, 1], LineBox[{115, 116}]}, 
                    {Hue[0.9144694333143899, 1, 1], LineBox[{116, 117}]}, 
                    {Hue[0.8263845311262752, 1, 1], LineBox[{117, 118}]}, 
                    {Hue[0.7396779689740469, 1, 1], LineBox[{118, 119}]}, 
                    {Hue[0.6545208857676232, 1, 1], LineBox[{119, 120}]}, 
                    {Hue[0.5710974969169937, 1, 1], LineBox[{120, 121}]}, 
                    {Hue[0.4895946637982166, 1, 1], LineBox[{121, 122}]}, 
                    {Hue[0.4102067570615117, 1, 1], LineBox[{122, 123}]}, 
                    {Hue[0.3331259132411515, 1, 1], LineBox[{123, 124}]}, 
                    {Hue[0.2585394496739037, 1, 1], LineBox[{124, 125}]}, 
                    {Hue[0.18662342332781198`, 1, 1], LineBox[{125, 126}]}, 
                    {Hue[0.11753800147794824`, 1, 1], LineBox[{126, 127}]}, 
                    {Hue[0.051427112466138326`, 1, 1], LineBox[{127, 128}]}, 
                    {Hue[0.9884129955448848, 1, 1], LineBox[{128, 129}]}, 
                    {Hue[0.9285892300665948, 1, 1], LineBox[{129, 130}]}, 
                    {Hue[0.8720256023365494, 1, 1], LineBox[{130, 131}]}, 
                    {Hue[0.8187663658254357, 1, 1], LineBox[{131, 132}]}, 
                    {Hue[0.7688265810123591, 1, 1], LineBox[{132, 133}]}, 
                    {Hue[0.7221955842685372, 1, 1], LineBox[{133, 134}]}, 
                    {Hue[0.6788437009263398, 1, 1], LineBox[{134, 135}]}, 
                    {Hue[0.6387174301250554, 1, 1], LineBox[{135, 136}]}, 
                    {Hue[0.6017459196746494, 1, 1], LineBox[{136, 137}]}, 
                    {Hue[0.5678475644593504, 1, 1], LineBox[{137, 138}]}, 
                    {Hue[0.5369273311968021, 1, 1], LineBox[{138, 139}]}, 
                    {Hue[0.5088816899028001, 1, 1], LineBox[{139, 140}]}, 
                    {Hue[0.48360528170390377`, 1, 1], LineBox[{140, 141}]}, 
                    {Hue[0.46098584079126487`, 1, 1], LineBox[{141, 142}]}, 
                    {Hue[0.4409134101930792, 1, 1], LineBox[{142, 143}]}, 
                    {Hue[0.42327813781734536`, 1, 1], LineBox[{143, 144}]}, 
                    {Hue[0.40797200785093196`, 1, 1], LineBox[{144, 145}]}, 
                    {Hue[0.39489008560322125`, 1, 1], LineBox[{145, 146}]}, 
                    {Hue[0.38393221617634643`, 1, 1], LineBox[{146, 147}]}, 
                    {Hue[0.37500196058241536`, 1, 1], LineBox[{147, 148}]}, 
                    {Hue[0.368008847957833, 1, 1], LineBox[{148, 149}]}, 
                    {Hue[0.3628666282170082, 1, 1], LineBox[{149, 150}]}, 
                    {Hue[0.3594945306893157, 1, 1], LineBox[{150, 151}]}, 
                    {Hue[0.35781626459383525`, 1, 1], LineBox[{151, 152}]}, 
                    {Hue[0.35775992995884326`, 1, 1], LineBox[{152, 153}]}, 
                    {Hue[0.35926092258923653`, 1, 1], LineBox[{153, 154}]}, 
                    {Hue[0.3622564401320628, 1, 1], LineBox[{154, 155}]}, 
                    {Hue[0.3666882794674252, 1, 1], LineBox[{155, 156}]}, 
                    {Hue[0.3725045100663471, 1, 1], LineBox[{156, 157}]}, 
                    {Hue[0.3796554784319523, 1, 1], LineBox[{157, 158}]}, 
                    {Hue[0.3880942947049093, 1, 1], LineBox[{158, 159}]}, 
                    {Hue[0.3977782989289779, 1, 1], LineBox[{159, 160}]}, 
                    {Hue[0.40866872513605323`, 1, 1], LineBox[{160, 161}]}, 
                    {Hue[0.42072856583263124`, 1, 1], LineBox[{161, 162}]}, 
                    {Hue[0.4339235440994428, 1, 1], LineBox[{162, 163}]}, 
                    {Hue[0.4482219112825864, 1, 1], LineBox[{163, 164}]}, 
                    {Hue[0.46359477563959944`, 1, 1], LineBox[{164, 165}]}, 
                    {Hue[0.48001457607847264`, 1, 1], LineBox[{165, 166}]}, 
                    {Hue[0.4974556703406297, 1, 1], LineBox[{166, 167}]}, 
                    {Hue[0.5158939565017163, 1, 1], LineBox[{167, 168}]}, 
                    {Hue[0.5353076938087875, 1, 1], LineBox[{168, 169}]}, 
                    {Hue[0.5556757445982718, 1, 1], LineBox[{169, 170}]}, 
                    {Hue[0.5769792836174885, 1, 1], LineBox[{170, 171}]}, 
                    {Hue[0.5991999701653299, 1, 1], LineBox[{171, 172}]}, 
                    {Hue[0.6223206152611922, 1, 1], LineBox[{172, 173}]}, 
                    {Hue[0.646325446642212, 1, 1], LineBox[{173, 174}]}, 
                    {Hue[0.6711998264592366, 1, 1], LineBox[{174, 175}]}, 
                    {Hue[0.6969297314660567, 1, 1], LineBox[{175, 176}]}, 
                    {Hue[0.7235023463562673, 1, 1], LineBox[{176, 177}]}, 
                    {Hue[0.7509043991326707, 1, 1], LineBox[{177, 178}]}, 
                    {Hue[0.7791243549562274, 1, 1], LineBox[{178, 179}]}, 
                    {Hue[0.8081515576934792, 1, 1], LineBox[{179, 180}]}, 
                    {Hue[0.8379740999023698, 1, 1], LineBox[{180, 181}]}, 
                    {Hue[0.8685825170069534, 1, 1], LineBox[{181, 182}]}, 
                    {Hue[0.8999669535324006, 1, 1], LineBox[{182, 183}]}, 
                    {Hue[0.9321197069880646, 1, 1], LineBox[{183, 184}]}, 
                    {Hue[0.9650299290140577, 1, 1], LineBox[{184, 185}]}, 
                    {Hue[0.9986909188738518, 1, 1], LineBox[{185, 186}]}, 
                    {Hue[0.033094624152977085`, 1, 1], LineBox[{186, 187}]}, 
                    {Hue[0.06823373640233694, 1, 1], LineBox[{187, 188}]}, 
                    {Hue[0.10410062095224143`, 1, 1], LineBox[{188, 189}]}, 
                    {Hue[0.14068810334390153`, 1, 1], LineBox[{189, 190}]}, 
                    {Hue[0.17799007885133333`, 1, 1], LineBox[{190, 191}]}, 
                    {Hue[0.21599943175394315`, 1, 1], LineBox[{191, 192}]}, 
                    {Hue[0.25471007336292956`, 1, 1], LineBox[{192, 193}]}, 
                    {Hue[0.2941159403636716, 1, 1], LineBox[{193, 194}]}, 
                    {Hue[0.33421239620143495`, 1, 1], LineBox[{194, 195}]}, 
                    {Hue[0.3749927116198588, 1, 1], LineBox[{195, 196}]}, 
                    {Hue[0.416453227649966, 1, 1], LineBox[{196, 197}]}, 
                    {Hue[0.458587613960305, 1, 1], LineBox[{197, 198}]}, 
                    {Hue[0.5013919317739264, 1, 1], LineBox[{198, 199}]}, 
                    {Hue[0.5448606766593951, 1, 1], LineBox[{199, 200}]}, 
                    {Hue[0.5889888658916277, 1, 1], LineBox[{200, 201}]}, 
                    {Hue[0.6337722749365959, 1, 1], LineBox[{201, 202}]}, 
                    {Hue[0.67920575497398, 1, 1], LineBox[{202, 203}]}, 
                    {Hue[0.7252846301057936, 1, 1], LineBox[{203, 204}]}, 
                    {Hue[0.7720062490737557, 1, 1], LineBox[{204, 205}]}, 
                    {Hue[0.8193656677120444, 1, 1], LineBox[{205, 206}]}, 
                    {Hue[0.8673604139213364, 1, 1], LineBox[{206, 207}]}, 
                    {Hue[0.9159854061521892, 1, 1], LineBox[{207, 208}]}, 
                    {Hue[0.965236857420268, 1, 1], LineBox[{208, 209}]}, 
                    {Hue[0.015112380299624755`, 1, 1], LineBox[{209, 210}]}, 
                    {Hue[0.0656073763500262, 1, 1], LineBox[{210, 211}]}, 
                    {Hue[0.11671944049250602`, 1, 1], LineBox[{211, 212}]}, 
                    {Hue[0.16844370339115078`, 1, 1], LineBox[{212, 213}]}, 
                    {Hue[0.22077839817692413`, 1, 1], LineBox[{213, 214}]}, 
                    {Hue[0.273719747035185, 1, 1], LineBox[{214, 215}]}, 
                    {Hue[0.3272645018509859, 1, 1], LineBox[{215, 216}]}, 
                    {Hue[0.3814105632880518, 1, 1], LineBox[{216, 217}]}, 
                    {Hue[0.4361535802050153, 1, 1], LineBox[{217, 218}]}, 
                    {Hue[0.4914914479566433, 1, 1], LineBox[{218, 219}]}, 
                    {Hue[0.5474209272105655, 1, 1], LineBox[{219, 220}]}, 
                    {Hue[0.6039401865467807, 1, 1], LineBox[{220, 221}]}, 
                    {Hue[0.6610450494096778, 1, 1], LineBox[{221, 222}]}, 
                    {Hue[0.7187346323562827, 1, 1], LineBox[{222, 223}]}, 
                    {Hue[0.7770045221176122, 1, 1], LineBox[{223, 224}]}, 
                    {Hue[0.8358535452751399, 1, 1], LineBox[{224, 225}]}, 
                    {Hue[0.8952789269389672, 1, 1], LineBox[{225, 226}]}, 
                    {Hue[0.955278384025335, 1, 1], LineBox[{226, 227}]}, 
                    {Hue[0.015848820485411037`, 1, 1], LineBox[{227, 228}]}, 
                    {Hue[0.07698831686773512, 1, 1], LineBox[{228, 229}]}, 
                    {Hue[0.1386951331217639, 1, 1], LineBox[{229, 230}]}, 
                    {Hue[0.2009663078345991, 1, 1], LineBox[{230, 231}]}, 
                    {Hue[0.2637996869744032, 1, 1], LineBox[{231, 232}]}, 
                    {Hue[0.32719312809287177`, 1, 1], LineBox[{232, 233}]}, 
                    {Hue[0.39114533126753376`, 1, 1], LineBox[{233, 234}]}, 
                    {Hue[0.455652644567371, 1, 1], LineBox[{234, 235}]}, 
                    {Hue[0.5207154494556953, 1, 1], LineBox[{235, 236}]}, 
                    {Hue[0.5863291466008134, 1, 1], LineBox[{236, 237}]}, 
                    {Hue[0.6524927029187558, 1, 1], LineBox[{237, 238}]}, 
                    {Hue[0.7192051438604465, 1, 1], LineBox[{238, 239}]}, 
                    {Hue[0.7864639377353686, 1, 1], LineBox[{239, 240}]}, 
                    {Hue[0.854266613804419, 1, 1], LineBox[{240, 241}]}, 
                    {Hue[0.9226132153595316, 1, 1], LineBox[{241, 242}]}, 
                    {Hue[0.9914990995606757, 1, 1], LineBox[{242, 243}]}, 
                    {Hue[0.06092463240056644, 1, 1], LineBox[{243, 244}]}, 
                    {Hue[0.13088683050341132`, 1, 1], LineBox[{244, 245}]}, 
                    {Hue[0.20138520126493187`, 1, 1], LineBox[{245, 246}]}, 
                    {Hue[0.27241755097104603`, 1, 1], LineBox[{246, 247}]}, 
                    {Hue[0.34398228203464143`, 1, 1], LineBox[{247, 248}]}, 
                    {Hue[0.4160792472892838, 1, 1], LineBox[{248, 249}]}, 
                    {Hue[0.4887026604895621, 1, 1], LineBox[{249, 250}]}, 
                    {Hue[0.5618548264506966, 1, 1], LineBox[{250, 251}]}}}},
                    VertexColors->{
                    Hue[0.5373228843195937, 1, 1], 
                    Hue[0.4403401308194928, 1, 1], 
                    Hue[0.3435613420702442, 1, 1], 
                    Hue[0.24690243660616393`, 1, 1], 
                    Hue[0.150454945879873, 1, 1], 
                    Hue[0.054253709597173946`, 1, 1], 
                    Hue[0.9582329689095465, 1, 1], 
                    Hue[0.8624154616652879, 1, 1], 
                    Hue[0.7667920547714719, 1, 1], 
                    Hue[0.6713539032786822, 1, 1], 
                    Hue[0.5761322441426646, 1, 1], 
                    Hue[0.4810681788903602, 1, 1], 
                    Hue[0.38618845496889015`, 1, 1], 
                    Hue[0.2914937307005535, 1, 1], 
                    Hue[0.19699709955287387`, 1, 1], 
                    Hue[0.1026633796553843, 1, 1], 
                    Hue[0.008503512405973614, 1, 1], 
                    Hue[0.9145316811967742, 1, 1], 
                    Hue[0.8207243806703278, 1, 1], 
                    Hue[0.7270951531531911, 1, 1], 
                    Hue[0.6336280106405164, 1, 1], 
                    Hue[0.5403238375965598, 1, 1], 
                    Hue[0.44719515461189513`, 1, 1], 
                    Hue[0.3542219104665249, 1, 1], 
                    Hue[0.261406034757453, 1, 1], 
                    Hue[0.16874846532010704`, 1, 1], 
                    Hue[0.07625049917109356, 1, 1], 
                    Hue[0.983898824560589, 1, 1], 
                    Hue[0.8917010932212673, 1, 1], 
                    Hue[0.7996520993560525, 1, 1], 
                    Hue[0.707743391289746, 1, 1], 
                    Hue[0.6159813669193629, 1, 1], 
                    Hue[0.5243603446357614, 1, 1], 
                    Hue[0.4328738691798968, 1, 1], 
                    Hue[0.3415208785271995, 1, 1], 
                    Hue[0.2502971034620979, 1, 1], 
                    Hue[0.15920419790548215`, 1, 1], 
                    Hue[0.06823149246682571, 1, 1], 
                    Hue[0.9773857509141324, 1, 1], 
                    Hue[0.8866550914096966, 1, 1], 
                    Hue[0.7960381183778849, 1, 1], 
                    Hue[0.7055349171841978, 1, 1], 
                    Hue[0.6151380108443617, 1, 1], 
                    Hue[0.5248452588742869, 1, 1], 
                    Hue[0.4346517215644478, 1, 1], 
                    Hue[0.3445541576573799, 1, 1], 
                    Hue[0.2545477968642205, 1, 1], 
                    Hue[0.16462741410865095`, 1, 1], 
                    Hue[0.07479106859346756, 1, 1], 
                    Hue[0.9850329040069344, 1, 1], 
                    Hue[0.8953473747186199, 1, 1], 
                    Hue[0.8057307072880856, 1, 1], 
                    Hue[0.7161762779604128, 1, 1], 
                    Hue[0.6266791615905233, 1, 1], 
                    Hue[0.537233630987699, 1, 1], 
                    Hue[0.44783486011628426`, 1, 1], 
                    Hue[0.35847490168618695`, 1, 1], 
                    Hue[0.2691480839616139, 1, 1], 
                    Hue[0.17984719749350703`, 1, 1], 
                    Hue[0.0905655025466636, 1, 1], 
                    Hue[0.0012950694149223545`, 1, 1], 
                    Hue[0.9120293898934388, 1, 1], 
                    Hue[0.8227595802498255, 1, 1], 
                    Hue[0.7334770677884576, 1, 1], 
                    Hue[0.6441724629825096, 1, 1], 
                    Hue[0.5548371466919941, 1, 1], 
                    Hue[0.465461449193391, 1, 1], 
                    Hue[0.376034319947191, 1, 1], 
                    Hue[0.2865456196687959, 1, 1], 
                    Hue[0.19698327643107333`, 1, 1], 
                    Hue[0.1073368478949821, 1, 1], 
                    Hue[0.01759296362047231, 1, 1], 
                    Hue[0.9277393289210646, 1, 1], 
                    Hue[0.8377620797664849, 1, 1], 
                    Hue[0.7476481320246258, 1, 1], 
                    Hue[0.6573829603765364, 1, 1], 
                    Hue[0.5669519357335853, 1, 1], 
                    Hue[0.47633986833715763`, 1, 1], 
                    Hue[0.3855322472772245, 1, 1], 
                    Hue[0.29451264153189877`, 1, 1], 
                    Hue[0.2032663941206125, 1, 1], 
                    Hue[0.1117779759296776, 1, 1], 
                    Hue[0.020033942761572048`, 1, 1], 
                    Hue[0.9280205709389144, 1, 1], 
                    Hue[0.8357247215490928, 1, 1], 
                    Hue[0.7431357669534746, 1, 1], 
                    Hue[0.6502444277652975, 1, 1], 
                    Hue[0.5570441913618924, 1, 1], 
                    Hue[0.4635317224138139, 1, 1], 
                    Hue[0.36970776965283664`, 1, 1], 
                    Hue[0.2755740273533197, 1, 1], 
                    Hue[0.18114110308239148`, 1, 1], 
                    Hue[0.08642018714458502, 1, 1], 
                    Hue[0.991429466836158, 1, 1], 
                    Hue[0.8961918398709002, 1, 1], 
                    Hue[0.8007328384798726, 1, 1], 
                    Hue[0.7050864037553836, 1, 1], 
                    Hue[0.609285542248224, 1, 1], 
                    Hue[0.5133709156062596, 1, 1], 
                    Hue[0.4173825756025821, 1, 1], 
                    Hue[0.32136569104301854`, 1, 1], 
                    Hue[0.22536565076658127`, 1, 1], 
                    Hue[0.12942796312066054`, 1, 1], 
                    Hue[0.03360344686736547, 1, 1], 
                    Hue[0.9379393315284854, 1, 1], 
                    Hue[0.8424879868571606, 1, 1], 
                    Hue[0.7473036316593907, 1, 1], 
                    Hue[0.6524441982990108, 1, 1], 
                    Hue[0.5579734697208485, 1, 1], 
                    Hue[0.46395883956449696`, 1, 1], 
                    Hue[0.3704781574462508, 1, 1], 
                    Hue[0.2776173958945541, 1, 1], 
                    Hue[0.18547449142920566`, 1, 1], 
                    Hue[0.09415329417638389, 1, 1], 
                    Hue[0.003775040291353455, 1, 1], 
                    Hue[0.9144694333143899, 1, 1], 
                    Hue[0.8263845311262752, 1, 1], 
                    Hue[0.7396779689740469, 1, 1], 
                    Hue[0.6545208857676232, 1, 1], 
                    Hue[0.5710974969169937, 1, 1], 
                    Hue[0.4895946637982166, 1, 1], 
                    Hue[0.4102067570615117, 1, 1], 
                    Hue[0.3331259132411515, 1, 1], 
                    Hue[0.2585394496739037, 1, 1], 
                    Hue[0.18662342332781198`, 1, 1], 
                    Hue[0.11753800147794824`, 1, 1], 
                    Hue[0.051427112466138326`, 1, 1], 
                    Hue[0.9884129955448848, 1, 1], 
                    Hue[0.9285892300665948, 1, 1], 
                    Hue[0.8720256023365494, 1, 1], 
                    Hue[0.8187663658254357, 1, 1], 
                    Hue[0.7688265810123591, 1, 1], 
                    Hue[0.7221955842685372, 1, 1], 
                    Hue[0.6788437009263398, 1, 1], 
                    Hue[0.6387174301250554, 1, 1], 
                    Hue[0.6017459196746494, 1, 1], 
                    Hue[0.5678475644593504, 1, 1], 
                    Hue[0.5369273311968021, 1, 1], 
                    Hue[0.5088816899028001, 1, 1], 
                    Hue[0.48360528170390377`, 1, 1], 
                    Hue[0.46098584079126487`, 1, 1], 
                    Hue[0.4409134101930792, 1, 1], 
                    Hue[0.42327813781734536`, 1, 1], 
                    Hue[0.40797200785093196`, 1, 1], 
                    Hue[0.39489008560322125`, 1, 1], 
                    Hue[0.38393221617634643`, 1, 1], 
                    Hue[0.37500196058241536`, 1, 1], 
                    Hue[0.368008847957833, 1, 1], 
                    Hue[0.3628666282170082, 1, 1], 
                    Hue[0.3594945306893157, 1, 1], 
                    Hue[0.35781626459383525`, 1, 1], 
                    Hue[0.35775992995884326`, 1, 1], 
                    Hue[0.35926092258923653`, 1, 1], 
                    Hue[0.3622564401320628, 1, 1], 
                    Hue[0.3666882794674252, 1, 1], 
                    Hue[0.3725045100663471, 1, 1], 
                    Hue[0.3796554784319523, 1, 1], 
                    Hue[0.3880942947049093, 1, 1], 
                    Hue[0.3977782989289779, 1, 1], 
                    Hue[0.40866872513605323`, 1, 1], 
                    Hue[0.42072856583263124`, 1, 1], 
                    Hue[0.4339235440994428, 1, 1], 
                    Hue[0.4482219112825864, 1, 1], 
                    Hue[0.46359477563959944`, 1, 1], 
                    Hue[0.48001457607847264`, 1, 1], 
                    Hue[0.4974556703406297, 1, 1], 
                    Hue[0.5158939565017163, 1, 1], 
                    Hue[0.5353076938087875, 1, 1], 
                    Hue[0.5556757445982718, 1, 1], 
                    Hue[0.5769792836174885, 1, 1], 
                    Hue[0.5991999701653299, 1, 1], 
                    Hue[0.6223206152611922, 1, 1], 
                    Hue[0.646325446642212, 1, 1], 
                    Hue[0.6711998264592366, 1, 1], 
                    Hue[0.6969297314660567, 1, 1], 
                    Hue[0.7235023463562673, 1, 1], 
                    Hue[0.7509043991326707, 1, 1], 
                    Hue[0.7791243549562274, 1, 1], 
                    Hue[0.8081515576934792, 1, 1], 
                    Hue[0.8379740999023698, 1, 1], 
                    Hue[0.8685825170069534, 1, 1], 
                    Hue[0.8999669535324006, 1, 1], 
                    Hue[0.9321197069880646, 1, 1], 
                    Hue[0.9650299290140577, 1, 1], 
                    Hue[0.9986909188738518, 1, 1], 
                    Hue[0.033094624152977085`, 1, 1], 
                    Hue[0.06823373640233694, 1, 1], 
                    Hue[0.10410062095224143`, 1, 1], 
                    Hue[0.14068810334390153`, 1, 1], 
                    Hue[0.17799007885133333`, 1, 1], 
                    Hue[0.21599943175394315`, 1, 1], 
                    Hue[0.25471007336292956`, 1, 1], 
                    Hue[0.2941159403636716, 1, 1], 
                    Hue[0.33421239620143495`, 1, 1], 
                    Hue[0.3749927116198588, 1, 1], 
                    Hue[0.416453227649966, 1, 1], 
                    Hue[0.458587613960305, 1, 1], 
                    Hue[0.5013919317739264, 1, 1], 
                    Hue[0.5448606766593951, 1, 1], 
                    Hue[0.5889888658916277, 1, 1], 
                    Hue[0.6337722749365959, 1, 1], 
                    Hue[0.67920575497398, 1, 1], 
                    Hue[0.7252846301057936, 1, 1], 
                    Hue[0.7720062490737557, 1, 1], 
                    Hue[0.8193656677120444, 1, 1], 
                    Hue[0.8673604139213364, 1, 1], 
                    Hue[0.9159854061521892, 1, 1], 
                    Hue[0.965236857420268, 1, 1], 
                    Hue[0.015112380299624755`, 1, 1], 
                    Hue[0.0656073763500262, 1, 1], 
                    Hue[0.11671944049250602`, 1, 1], 
                    Hue[0.16844370339115078`, 1, 1], 
                    Hue[0.22077839817692413`, 1, 1], 
                    Hue[0.273719747035185, 1, 1], 
                    Hue[0.3272645018509859, 1, 1], 
                    Hue[0.3814105632880518, 1, 1], 
                    Hue[0.4361535802050153, 1, 1], 
                    Hue[0.4914914479566433, 1, 1], 
                    Hue[0.5474209272105655, 1, 1], 
                    Hue[0.6039401865467807, 1, 1], 
                    Hue[0.6610450494096778, 1, 1], 
                    Hue[0.7187346323562827, 1, 1], 
                    Hue[0.7770045221176122, 1, 1], 
                    Hue[0.8358535452751399, 1, 1], 
                    Hue[0.8952789269389672, 1, 1], 
                    Hue[0.955278384025335, 1, 1], 
                    Hue[0.015848820485411037`, 1, 1], 
                    Hue[0.07698831686773512, 1, 1], 
                    Hue[0.1386951331217639, 1, 1], 
                    Hue[0.2009663078345991, 1, 1], 
                    Hue[0.2637996869744032, 1, 1], 
                    Hue[0.32719312809287177`, 1, 1], 
                    Hue[0.39114533126753376`, 1, 1], 
                    Hue[0.455652644567371, 1, 1], 
                    Hue[0.5207154494556953, 1, 1], 
                    Hue[0.5863291466008134, 1, 1], 
                    Hue[0.6524927029187558, 1, 1], 
                    Hue[0.7192051438604465, 1, 1], 
                    Hue[0.7864639377353686, 1, 1], 
                    Hue[0.854266613804419, 1, 1], 
                    Hue[0.9226132153595316, 1, 1], 
                    Hue[0.9914990995606757, 1, 1], 
                    Hue[0.06092463240056644, 1, 1], 
                    Hue[0.13088683050341132`, 1, 1], 
                    Hue[0.20138520126493187`, 1, 1], 
                    Hue[0.27241755097104603`, 1, 1], 
                    Hue[0.34398228203464143`, 1, 1], 
                    Hue[0.4160792472892838, 1, 1], 
                    Hue[0.4887026604895621, 1, 1], 
                    Hue[0.5618548264506966, 1, 1], 
                    Hue[0.6355323297586595, 1, 1], 
                    Hue[0.5373228843195937, 1, 1], 
                    Hue[0.4403401308194928, 1, 1], 
                    Hue[0.4403401308194928, 1, 1], 
                    Hue[0.3435613420702442, 1, 1], 
                    Hue[0.3435613420702442, 1, 1], 
                    Hue[0.24690243660616393`, 1, 1], 
                    Hue[0.24690243660616393`, 1, 1], 
                    Hue[0.150454945879873, 1, 1], 
                    Hue[0.150454945879873, 1, 1], 
                    Hue[0.054253709597173946`, 1, 1], 
                    Hue[0.054253709597173946`, 1, 1], 
                    Hue[0.9582329689095465, 1, 1], 
                    Hue[0.9582329689095465, 1, 1], 
                    Hue[0.8624154616652879, 1, 1], 
                    Hue[0.8624154616652879, 1, 1], 
                    Hue[0.7667920547714719, 1, 1], 
                    Hue[0.7667920547714719, 1, 1], 
                    Hue[0.6713539032786822, 1, 1], 
                    Hue[0.6713539032786822, 1, 1], 
                    Hue[0.5761322441426646, 1, 1], 
                    Hue[0.5761322441426646, 1, 1], 
                    Hue[0.4810681788903602, 1, 1], 
                    Hue[0.4810681788903602, 1, 1], 
                    Hue[0.38618845496889015`, 1, 1], 
                    Hue[0.38618845496889015`, 1, 1], 
                    Hue[0.2914937307005535, 1, 1], 
                    Hue[0.2914937307005535, 1, 1], 
                    Hue[0.19699709955287387`, 1, 1], 
                    Hue[0.19699709955287387`, 1, 1], 
                    Hue[0.1026633796553843, 1, 1], 
                    Hue[0.1026633796553843, 1, 1], 
                    Hue[0.008503512405973614, 1, 1], 
                    Hue[0.008503512405973614, 1, 1], 
                    Hue[0.9145316811967742, 1, 1], 
                    Hue[0.9145316811967742, 1, 1], 
                    Hue[0.8207243806703278, 1, 1], 
                    Hue[0.8207243806703278, 1, 1], 
                    Hue[0.7270951531531911, 1, 1], 
                    Hue[0.7270951531531911, 1, 1], 
                    Hue[0.6336280106405164, 1, 1], 
                    Hue[0.6336280106405164, 1, 1], 
                    Hue[0.5403238375965598, 1, 1], 
                    Hue[0.5403238375965598, 1, 1], 
                    Hue[0.44719515461189513`, 1, 1], 
                    Hue[0.44719515461189513`, 1, 1], 
                    Hue[0.3542219104665249, 1, 1], 
                    Hue[0.3542219104665249, 1, 1], 
                    Hue[0.261406034757453, 1, 1], 
                    Hue[0.261406034757453, 1, 1], 
                    Hue[0.16874846532010704`, 1, 1], 
                    Hue[0.16874846532010704`, 1, 1], 
                    Hue[0.07625049917109356, 1, 1], 
                    Hue[0.07625049917109356, 1, 1], 
                    Hue[0.983898824560589, 1, 1], 
                    Hue[0.983898824560589, 1, 1], 
                    Hue[0.8917010932212673, 1, 1], 
                    Hue[0.8917010932212673, 1, 1], 
                    Hue[0.7996520993560525, 1, 1], 
                    Hue[0.7996520993560525, 1, 1], 
                    Hue[0.707743391289746, 1, 1], 
                    Hue[0.707743391289746, 1, 1], 
                    Hue[0.6159813669193629, 1, 1], 
                    Hue[0.6159813669193629, 1, 1], 
                    Hue[0.5243603446357614, 1, 1], 
                    Hue[0.5243603446357614, 1, 1], 
                    Hue[0.4328738691798968, 1, 1], 
                    Hue[0.4328738691798968, 1, 1], 
                    Hue[0.3415208785271995, 1, 1], 
                    Hue[0.3415208785271995, 1, 1], 
                    Hue[0.2502971034620979, 1, 1], 
                    Hue[0.2502971034620979, 1, 1], 
                    Hue[0.15920419790548215`, 1, 1], 
                    Hue[0.15920419790548215`, 1, 1], 
                    Hue[0.06823149246682571, 1, 1], 
                    Hue[0.06823149246682571, 1, 1], 
                    Hue[0.9773857509141324, 1, 1], 
                    Hue[0.9773857509141324, 1, 1], 
                    Hue[0.8866550914096966, 1, 1], 
                    Hue[0.8866550914096966, 1, 1], 
                    Hue[0.7960381183778849, 1, 1], 
                    Hue[0.7960381183778849, 1, 1], 
                    Hue[0.7055349171841978, 1, 1], 
                    Hue[0.7055349171841978, 1, 1], 
                    Hue[0.6151380108443617, 1, 1], 
                    Hue[0.6151380108443617, 1, 1], 
                    Hue[0.5248452588742869, 1, 1], 
                    Hue[0.5248452588742869, 1, 1], 
                    Hue[0.4346517215644478, 1, 1], 
                    Hue[0.4346517215644478, 1, 1], 
                    Hue[0.3445541576573799, 1, 1], 
                    Hue[0.3445541576573799, 1, 1], 
                    Hue[0.2545477968642205, 1, 1], 
                    Hue[0.2545477968642205, 1, 1], 
                    Hue[0.16462741410865095`, 1, 1], 
                    Hue[0.16462741410865095`, 1, 1], 
                    Hue[0.07479106859346756, 1, 1], 
                    Hue[0.07479106859346756, 1, 1], 
                    Hue[0.9850329040069344, 1, 1], 
                    Hue[0.9850329040069344, 1, 1], 
                    Hue[0.8953473747186199, 1, 1], 
                    Hue[0.8953473747186199, 1, 1], 
                    Hue[0.8057307072880856, 1, 1], 
                    Hue[0.8057307072880856, 1, 1], 
                    Hue[0.7161762779604128, 1, 1], 
                    Hue[0.7161762779604128, 1, 1], 
                    Hue[0.6266791615905233, 1, 1], 
                    Hue[0.6266791615905233, 1, 1], 
                    Hue[0.537233630987699, 1, 1], 
                    Hue[0.537233630987699, 1, 1], 
                    Hue[0.44783486011628426`, 1, 1], 
                    Hue[0.44783486011628426`, 1, 1], 
                    Hue[0.35847490168618695`, 1, 1], 
                    Hue[0.35847490168618695`, 1, 1], 
                    Hue[0.2691480839616139, 1, 1], 
                    Hue[0.2691480839616139, 1, 1], 
                    Hue[0.17984719749350703`, 1, 1], 
                    Hue[0.17984719749350703`, 1, 1], 
                    Hue[0.0905655025466636, 1, 1], 
                    Hue[0.0905655025466636, 1, 1], 
                    Hue[0.0012950694149223545`, 1, 1], 
                    Hue[0.0012950694149223545`, 1, 1], 
                    Hue[0.9120293898934388, 1, 1], 
                    Hue[0.9120293898934388, 1, 1], 
                    Hue[0.8227595802498255, 1, 1], 
                    Hue[0.8227595802498255, 1, 1], 
                    Hue[0.7334770677884576, 1, 1], 
                    Hue[0.7334770677884576, 1, 1], 
                    Hue[0.6441724629825096, 1, 1], 
                    Hue[0.6441724629825096, 1, 1], 
                    Hue[0.5548371466919941, 1, 1], 
                    Hue[0.5548371466919941, 1, 1], 
                    Hue[0.465461449193391, 1, 1], 
                    Hue[0.465461449193391, 1, 1], 
                    Hue[0.376034319947191, 1, 1], 
                    Hue[0.376034319947191, 1, 1], 
                    Hue[0.2865456196687959, 1, 1], 
                    Hue[0.2865456196687959, 1, 1], 
                    Hue[0.19698327643107333`, 1, 1], 
                    Hue[0.19698327643107333`, 1, 1], 
                    Hue[0.1073368478949821, 1, 1], 
                    Hue[0.1073368478949821, 1, 1], 
                    Hue[0.01759296362047231, 1, 1], 
                    Hue[0.01759296362047231, 1, 1], 
                    Hue[0.9277393289210646, 1, 1], 
                    Hue[0.9277393289210646, 1, 1], 
                    Hue[0.8377620797664849, 1, 1], 
                    Hue[0.8377620797664849, 1, 1], 
                    Hue[0.7476481320246258, 1, 1], 
                    Hue[0.7476481320246258, 1, 1], 
                    Hue[0.6573829603765364, 1, 1], 
                    Hue[0.6573829603765364, 1, 1], 
                    Hue[0.5669519357335853, 1, 1], 
                    Hue[0.5669519357335853, 1, 1], 
                    Hue[0.47633986833715763`, 1, 1], 
                    Hue[0.47633986833715763`, 1, 1], 
                    Hue[0.3855322472772245, 1, 1], 
                    Hue[0.3855322472772245, 1, 1], 
                    Hue[0.29451264153189877`, 1, 1], 
                    Hue[0.29451264153189877`, 1, 1], 
                    Hue[0.2032663941206125, 1, 1], 
                    Hue[0.2032663941206125, 1, 1], 
                    Hue[0.1117779759296776, 1, 1], 
                    Hue[0.1117779759296776, 1, 1], 
                    Hue[0.020033942761572048`, 1, 1], 
                    Hue[0.020033942761572048`, 1, 1], 
                    Hue[0.9280205709389144, 1, 1], 
                    Hue[0.9280205709389144, 1, 1], 
                    Hue[0.8357247215490928, 1, 1], 
                    Hue[0.8357247215490928, 1, 1], 
                    Hue[0.7431357669534746, 1, 1], 
                    Hue[0.7431357669534746, 1, 1], 
                    Hue[0.6502444277652975, 1, 1], 
                    Hue[0.6502444277652975, 1, 1], 
                    Hue[0.5570441913618924, 1, 1], 
                    Hue[0.5570441913618924, 1, 1], 
                    Hue[0.4635317224138139, 1, 1], 
                    Hue[0.4635317224138139, 1, 1], 
                    Hue[0.36970776965283664`, 1, 1], 
                    Hue[0.36970776965283664`, 1, 1], 
                    Hue[0.2755740273533197, 1, 1], 
                    Hue[0.2755740273533197, 1, 1], 
                    Hue[0.18114110308239148`, 1, 1], 
                    Hue[0.18114110308239148`, 1, 1], 
                    Hue[0.08642018714458502, 1, 1], 
                    Hue[0.08642018714458502, 1, 1], 
                    Hue[0.991429466836158, 1, 1], 
                    Hue[0.991429466836158, 1, 1], 
                    Hue[0.8961918398709002, 1, 1], 
                    Hue[0.8961918398709002, 1, 1], 
                    Hue[0.8007328384798726, 1, 1], 
                    Hue[0.8007328384798726, 1, 1], 
                    Hue[0.7050864037553836, 1, 1], 
                    Hue[0.7050864037553836, 1, 1], 
                    Hue[0.609285542248224, 1, 1], 
                    Hue[0.609285542248224, 1, 1], 
                    Hue[0.5133709156062596, 1, 1], 
                    Hue[0.5133709156062596, 1, 1], 
                    Hue[0.4173825756025821, 1, 1], 
                    Hue[0.4173825756025821, 1, 1], 
                    Hue[0.32136569104301854`, 1, 1], 
                    Hue[0.32136569104301854`, 1, 1], 
                    Hue[0.22536565076658127`, 1, 1], 
                    Hue[0.22536565076658127`, 1, 1], 
                    Hue[0.12942796312066054`, 1, 1], 
                    Hue[0.12942796312066054`, 1, 1], 
                    Hue[0.03360344686736547, 1, 1], 
                    Hue[0.03360344686736547, 1, 1], 
                    Hue[0.9379393315284854, 1, 1], 
                    Hue[0.9379393315284854, 1, 1], 
                    Hue[0.8424879868571606, 1, 1], 
                    Hue[0.8424879868571606, 1, 1], 
                    Hue[0.7473036316593907, 1, 1], 
                    Hue[0.7473036316593907, 1, 1], 
                    Hue[0.6524441982990108, 1, 1], 
                    Hue[0.6524441982990108, 1, 1], 
                    Hue[0.5579734697208485, 1, 1], 
                    Hue[0.5579734697208485, 1, 1], 
                    Hue[0.46395883956449696`, 1, 1], 
                    Hue[0.46395883956449696`, 1, 1], 
                    Hue[0.3704781574462508, 1, 1], 
                    Hue[0.3704781574462508, 1, 1], 
                    Hue[0.2776173958945541, 1, 1], 
                    Hue[0.2776173958945541, 1, 1], 
                    Hue[0.18547449142920566`, 1, 1], 
                    Hue[0.18547449142920566`, 1, 1], 
                    Hue[0.09415329417638389, 1, 1], 
                    Hue[0.09415329417638389, 1, 1], 
                    Hue[0.003775040291353455, 1, 1], 
                    Hue[0.003775040291353455, 1, 1], 
                    Hue[0.9144694333143899, 1, 1], 
                    Hue[0.9144694333143899, 1, 1], 
                    Hue[0.8263845311262752, 1, 1], 
                    Hue[0.8263845311262752, 1, 1], 
                    Hue[0.7396779689740469, 1, 1], 
                    Hue[0.7396779689740469, 1, 1], 
                    Hue[0.6545208857676232, 1, 1], 
                    Hue[0.6545208857676232, 1, 1], 
                    Hue[0.5710974969169937, 1, 1], 
                    Hue[0.5710974969169937, 1, 1], 
                    Hue[0.4895946637982166, 1, 1], 
                    Hue[0.4895946637982166, 1, 1], 
                    Hue[0.4102067570615117, 1, 1], 
                    Hue[0.4102067570615117, 1, 1], 
                    Hue[0.3331259132411515, 1, 1], 
                    Hue[0.3331259132411515, 1, 1], 
                    Hue[0.2585394496739037, 1, 1], 
                    Hue[0.2585394496739037, 1, 1], 
                    Hue[0.18662342332781198`, 1, 1], 
                    Hue[0.18662342332781198`, 1, 1], 
                    Hue[0.11753800147794824`, 1, 1], 
                    Hue[0.11753800147794824`, 1, 1], 
                    Hue[0.051427112466138326`, 1, 1], 
                    Hue[0.051427112466138326`, 1, 1], 
                    Hue[0.9884129955448848, 1, 1], 
                    Hue[0.9884129955448848, 1, 1], 
                    Hue[0.9285892300665948, 1, 1], 
                    Hue[0.9285892300665948, 1, 1], 
                    Hue[0.8720256023365494, 1, 1], 
                    Hue[0.8720256023365494, 1, 1], 
                    Hue[0.8187663658254357, 1, 1], 
                    Hue[0.8187663658254357, 1, 1], 
                    Hue[0.7688265810123591, 1, 1], 
                    Hue[0.7688265810123591, 1, 1], 
                    Hue[0.7221955842685372, 1, 1], 
                    Hue[0.7221955842685372, 1, 1], 
                    Hue[0.6788437009263398, 1, 1], 
                    Hue[0.6788437009263398, 1, 1], 
                    Hue[0.6387174301250554, 1, 1], 
                    Hue[0.6387174301250554, 1, 1], 
                    Hue[0.6017459196746494, 1, 1], 
                    Hue[0.6017459196746494, 1, 1], 
                    Hue[0.5678475644593504, 1, 1], 
                    Hue[0.5678475644593504, 1, 1], 
                    Hue[0.5369273311968021, 1, 1], 
                    Hue[0.5369273311968021, 1, 1], 
                    Hue[0.5088816899028001, 1, 1], 
                    Hue[0.5088816899028001, 1, 1], 
                    Hue[0.48360528170390377`, 1, 1], 
                    Hue[0.48360528170390377`, 1, 1], 
                    Hue[0.46098584079126487`, 1, 1], 
                    Hue[0.46098584079126487`, 1, 1], 
                    Hue[0.4409134101930792, 1, 1], 
                    Hue[0.4409134101930792, 1, 1], 
                    Hue[0.42327813781734536`, 1, 1], 
                    Hue[0.42327813781734536`, 1, 1], 
                    Hue[0.40797200785093196`, 1, 1], 
                    Hue[0.40797200785093196`, 1, 1], 
                    Hue[0.39489008560322125`, 1, 1], 
                    Hue[0.39489008560322125`, 1, 1], 
                    Hue[0.38393221617634643`, 1, 1], 
                    Hue[0.38393221617634643`, 1, 1], 
                    Hue[0.37500196058241536`, 1, 1], 
                    Hue[0.37500196058241536`, 1, 1], 
                    Hue[0.368008847957833, 1, 1], 
                    Hue[0.368008847957833, 1, 1], 
                    Hue[0.3628666282170082, 1, 1], 
                    Hue[0.3628666282170082, 1, 1], 
                    Hue[0.3594945306893157, 1, 1], 
                    Hue[0.3594945306893157, 1, 1], 
                    Hue[0.35781626459383525`, 1, 1], 
                    Hue[0.35781626459383525`, 1, 1], 
                    Hue[0.35775992995884326`, 1, 1], 
                    Hue[0.35775992995884326`, 1, 1], 
                    Hue[0.35926092258923653`, 1, 1], 
                    Hue[0.35926092258923653`, 1, 1], 
                    Hue[0.3622564401320628, 1, 1], 
                    Hue[0.3622564401320628, 1, 1], 
                    Hue[0.3666882794674252, 1, 1], 
                    Hue[0.3666882794674252, 1, 1], 
                    Hue[0.3725045100663471, 1, 1], 
                    Hue[0.3725045100663471, 1, 1], 
                    Hue[0.3796554784319523, 1, 1], 
                    Hue[0.3796554784319523, 1, 1], 
                    Hue[0.3880942947049093, 1, 1], 
                    Hue[0.3880942947049093, 1, 1], 
                    Hue[0.3977782989289779, 1, 1], 
                    Hue[0.3977782989289779, 1, 1], 
                    Hue[0.40866872513605323`, 1, 1], 
                    Hue[0.40866872513605323`, 1, 1], 
                    Hue[0.42072856583263124`, 1, 1], 
                    Hue[0.42072856583263124`, 1, 1], 
                    Hue[0.4339235440994428, 1, 1], 
                    Hue[0.4339235440994428, 1, 1], 
                    Hue[0.4482219112825864, 1, 1], 
                    Hue[0.4482219112825864, 1, 1], 
                    Hue[0.46359477563959944`, 1, 1], 
                    Hue[0.46359477563959944`, 1, 1], 
                    Hue[0.48001457607847264`, 1, 1], 
                    Hue[0.48001457607847264`, 1, 1], 
                    Hue[0.4974556703406297, 1, 1], 
                    Hue[0.4974556703406297, 1, 1], 
                    Hue[0.5158939565017163, 1, 1], 
                    Hue[0.5158939565017163, 1, 1], 
                    Hue[0.5353076938087875, 1, 1], 
                    Hue[0.5353076938087875, 1, 1], 
                    Hue[0.5556757445982718, 1, 1], 
                    Hue[0.5556757445982718, 1, 1], 
                    Hue[0.5769792836174885, 1, 1], 
                    Hue[0.5769792836174885, 1, 1], 
                    Hue[0.5991999701653299, 1, 1], 
                    Hue[0.5991999701653299, 1, 1], 
                    Hue[0.6223206152611922, 1, 1], 
                    Hue[0.6223206152611922, 1, 1], 
                    Hue[0.646325446642212, 1, 1], 
                    Hue[0.646325446642212, 1, 1], 
                    Hue[0.6711998264592366, 1, 1], 
                    Hue[0.6711998264592366, 1, 1], 
                    Hue[0.6969297314660567, 1, 1], 
                    Hue[0.6969297314660567, 1, 1], 
                    Hue[0.7235023463562673, 1, 1], 
                    Hue[0.7235023463562673, 1, 1], 
                    Hue[0.7509043991326707, 1, 1], 
                    Hue[0.7509043991326707, 1, 1], 
                    Hue[0.7791243549562274, 1, 1], 
                    Hue[0.7791243549562274, 1, 1], 
                    Hue[0.8081515576934792, 1, 1], 
                    Hue[0.8081515576934792, 1, 1], 
                    Hue[0.8379740999023698, 1, 1], 
                    Hue[0.8379740999023698, 1, 1], 
                    Hue[0.8685825170069534, 1, 1], 
                    Hue[0.8685825170069534, 1, 1], 
                    Hue[0.8999669535324006, 1, 1], 
                    Hue[0.8999669535324006, 1, 1], 
                    Hue[0.9321197069880646, 1, 1], 
                    Hue[0.9321197069880646, 1, 1], 
                    Hue[0.9650299290140577, 1, 1], 
                    Hue[0.9650299290140577, 1, 1], 
                    Hue[0.9986909188738518, 1, 1], 
                    Hue[0.9986909188738518, 1, 1], 
                    Hue[0.033094624152977085`, 1, 1], 
                    Hue[0.033094624152977085`, 1, 1], 
                    Hue[0.06823373640233694, 1, 1], 
                    Hue[0.06823373640233694, 1, 1], 
                    Hue[0.10410062095224143`, 1, 1], 
                    Hue[0.10410062095224143`, 1, 1], 
                    Hue[0.14068810334390153`, 1, 1], 
                    Hue[0.14068810334390153`, 1, 1], 
                    Hue[0.17799007885133333`, 1, 1], 
                    Hue[0.17799007885133333`, 1, 1], 
                    Hue[0.21599943175394315`, 1, 1], 
                    Hue[0.21599943175394315`, 1, 1], 
                    Hue[0.25471007336292956`, 1, 1], 
                    Hue[0.25471007336292956`, 1, 1], 
                    Hue[0.2941159403636716, 1, 1], 
                    Hue[0.2941159403636716, 1, 1], 
                    Hue[0.33421239620143495`, 1, 1], 
                    Hue[0.33421239620143495`, 1, 1], 
                    Hue[0.3749927116198588, 1, 1], 
                    Hue[0.3749927116198588, 1, 1], 
                    Hue[0.416453227649966, 1, 1], 
                    Hue[0.416453227649966, 1, 1], 
                    Hue[0.458587613960305, 1, 1], 
                    Hue[0.458587613960305, 1, 1], 
                    Hue[0.5013919317739264, 1, 1], 
                    Hue[0.5013919317739264, 1, 1], 
                    Hue[0.5448606766593951, 1, 1], 
                    Hue[0.5448606766593951, 1, 1], 
                    Hue[0.5889888658916277, 1, 1], 
                    Hue[0.5889888658916277, 1, 1], 
                    Hue[0.6337722749365959, 1, 1], 
                    Hue[0.6337722749365959, 1, 1], 
                    Hue[0.67920575497398, 1, 1], 
                    Hue[0.67920575497398, 1, 1], 
                    Hue[0.7252846301057936, 1, 1], 
                    Hue[0.7252846301057936, 1, 1], 
                    Hue[0.7720062490737557, 1, 1], 
                    Hue[0.7720062490737557, 1, 1], 
                    Hue[0.8193656677120444, 1, 1], 
                    Hue[0.8193656677120444, 1, 1], 
                    Hue[0.8673604139213364, 1, 1], 
                    Hue[0.8673604139213364, 1, 1], 
                    Hue[0.9159854061521892, 1, 1], 
                    Hue[0.9159854061521892, 1, 1], 
                    Hue[0.965236857420268, 1, 1], 
                    Hue[0.965236857420268, 1, 1], 
                    Hue[0.015112380299624755`, 1, 1], 
                    Hue[0.015112380299624755`, 1, 1], 
                    Hue[0.0656073763500262, 1, 1], 
                    Hue[0.0656073763500262, 1, 1], 
                    Hue[0.11671944049250602`, 1, 1], 
                    Hue[0.11671944049250602`, 1, 1], 
                    Hue[0.16844370339115078`, 1, 1], 
                    Hue[0.16844370339115078`, 1, 1], 
                    Hue[0.22077839817692413`, 1, 1], 
                    Hue[0.22077839817692413`, 1, 1], 
                    Hue[0.273719747035185, 1, 1], 
                    Hue[0.273719747035185, 1, 1], 
                    Hue[0.3272645018509859, 1, 1], 
                    Hue[0.3272645018509859, 1, 1], 
                    Hue[0.3814105632880518, 1, 1], 
                    Hue[0.3814105632880518, 1, 1], 
                    Hue[0.4361535802050153, 1, 1], 
                    Hue[0.4361535802050153, 1, 1], 
                    Hue[0.4914914479566433, 1, 1], 
                    Hue[0.4914914479566433, 1, 1], 
                    Hue[0.5474209272105655, 1, 1], 
                    Hue[0.5474209272105655, 1, 1], 
                    Hue[0.6039401865467807, 1, 1], 
                    Hue[0.6039401865467807, 1, 1], 
                    Hue[0.6610450494096778, 1, 1], 
                    Hue[0.6610450494096778, 1, 1], 
                    Hue[0.7187346323562827, 1, 1], 
                    Hue[0.7187346323562827, 1, 1], 
                    Hue[0.7770045221176122, 1, 1], 
                    Hue[0.7770045221176122, 1, 1], 
                    Hue[0.8358535452751399, 1, 1], 
                    Hue[0.8358535452751399, 1, 1], 
                    Hue[0.8952789269389672, 1, 1], 
                    Hue[0.8952789269389672, 1, 1], 
                    Hue[0.955278384025335, 1, 1], 
                    Hue[0.955278384025335, 1, 1], 
                    Hue[0.015848820485411037`, 1, 1], 
                    Hue[0.015848820485411037`, 1, 1], 
                    Hue[0.07698831686773512, 1, 1], 
                    Hue[0.07698831686773512, 1, 1], 
                    Hue[0.1386951331217639, 1, 1], 
                    Hue[0.1386951331217639, 1, 1], 
                    Hue[0.2009663078345991, 1, 1], 
                    Hue[0.2009663078345991, 1, 1], 
                    Hue[0.2637996869744032, 1, 1], 
                    Hue[0.2637996869744032, 1, 1], 
                    Hue[0.32719312809287177`, 1, 1], 
                    Hue[0.32719312809287177`, 1, 1], 
                    Hue[0.39114533126753376`, 1, 1], 
                    Hue[0.39114533126753376`, 1, 1], 
                    Hue[0.455652644567371, 1, 1], 
                    Hue[0.455652644567371, 1, 1], 
                    Hue[0.5207154494556953, 1, 1], 
                    Hue[0.5207154494556953, 1, 1], 
                    Hue[0.5863291466008134, 1, 1], 
                    Hue[0.5863291466008134, 1, 1], 
                    Hue[0.6524927029187558, 1, 1], 
                    Hue[0.6524927029187558, 1, 1], 
                    Hue[0.7192051438604465, 1, 1], 
                    Hue[0.7192051438604465, 1, 1], 
                    Hue[0.7864639377353686, 1, 1], 
                    Hue[0.7864639377353686, 1, 1], 
                    Hue[0.854266613804419, 1, 1], 
                    Hue[0.854266613804419, 1, 1], 
                    Hue[0.9226132153595316, 1, 1], 
                    Hue[0.9226132153595316, 1, 1], 
                    Hue[0.9914990995606757, 1, 1], 
                    Hue[0.9914990995606757, 1, 1], 
                    Hue[0.06092463240056644, 1, 1], 
                    Hue[0.06092463240056644, 1, 1], 
                    Hue[0.13088683050341132`, 1, 1], 
                    Hue[0.13088683050341132`, 1, 1], 
                    Hue[0.20138520126493187`, 1, 1], 
                    Hue[0.20138520126493187`, 1, 1], 
                    Hue[0.27241755097104603`, 1, 1], 
                    Hue[0.27241755097104603`, 1, 1], 
                    Hue[0.34398228203464143`, 1, 1], 
                    Hue[0.34398228203464143`, 1, 1], 
                    Hue[0.4160792472892838, 1, 1], 
                    Hue[0.4160792472892838, 1, 1], 
                    Hue[0.4887026604895621, 1, 1], 
                    Hue[0.4887026604895621, 1, 1], 
                    Hue[0.5618548264506966, 1, 1], 
                    Hue[0.5618548264506966, 1, 1], 
                    Hue[0.6355323297586595, 1, 1]}],
                   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
                   Axes->{True, None},
                   AxesOrigin->{0, 0},
                   
                   Epilog->LineBox[{{1, 0.0021799831813656945`}, {
                    2, 0.0024547471130588697`}, {3, 0.0027606662867666098`}, {
                    4, 0.00310096922303252}, {5, 0.003479630423965041}, {
                    6, 0.0038990756191110173`}, {7, 0.004364457528543681}, {
                    8, 0.004879509214405107}, {9, 0.005448609775370151}, {
                    10, 0.00607783337340048}, {11, 0.006771229639254447}, {
                    12, 0.007534672312490112}, {13, 0.008373304709795179}, {
                    14, 0.009294845349407829}, {15, 0.010304618153907819`}, {
                    16, 0.01140898182274556}, {17, 0.012617009801659366`}, {
                    18, 0.01393417377741211}, {19, 0.015369334707733887`}, {
                    20, 0.01693052603426591}, {21, 0.018625045106631745`}, {
                    22, 0.0204627300752764}, {23, 0.02245186102823825}, {
                    24, 0.024600353194320804`}, {25, 0.026918694107456217`}, {
                    26, 0.029414581495396434`}, {27, 0.03209752393067593}, {
                    28, 0.03497663756977691}, {29, 0.03805943900264641}, {
                    30, 0.04135527058194878}, {31, 0.04487205944262558}, {
                    32, 0.04861669908277569}, {33, 0.052595813402006496`}, {
                    34, 0.05681765288026682}, {35, 0.06128684124972052}, {
                    36, 0.06600735661718311}, {37, 0.07098133867238944}, {
                    38, 0.07621429108125545}, {39, 0.08170396646219354}, {
                    40, 0.08745337841366976}, {41, 0.09345684393239075}, {
                    42, 0.09971207493206483}, {43, 0.1062145504230983}, {
                    44, 0.11295475444444614`}, {45, 0.11992402182272366`}, {
                    46, 0.12711014757064265`}, {47, 0.13449927717565893`}, {
                    48, 0.14207473459179756`}, {49, 0.14981788177608477`}, {
                    50, 0.1577090626447986}, {51, 0.16572195132175516`}, {
                    52, 0.17383363997322535`}, {53, 0.18201327708641205`}, {
                    54, 0.19023090517269958`}, {55, 0.19845666922086197`}, {
                    56, 0.2066547490415461}, {57, 0.2147892752119631}, {
                    58, 0.22282207387879305`}, {59, 0.2307139277821593}, {
                    60, 0.23842856312732666`}, {61, 0.24592331614768356`}, {
                    62, 0.2531579881197967}, {63, 0.26009347663882154`}, {
                    64, 0.266687990684189}, {65, 0.27290474123558606`}, {
                    66, 0.2787056248084115}, {67, 0.28405546539111104`}, {
                    68, 0.288922326327844}, {69, 0.2932728914984872}, {
                    70, 0.2970847720774838}, {71, 0.300333536278017}, {
                    72, 0.30300070228846354`}, {73, 0.30507237121874037`}, {
                    74, 0.3065387874003445}, {75, 0.3073973403517825}, {
                    76, 0.3076497579563843}, {77, 0.30730408686856986`}, {
                    78, 0.3063712315994363}, {79, 0.30487258459097555`}, {
                    80, 0.3028320925175755}, {81, 0.3002819571326402}, {
                    82, 0.29725865947961133`}, {83, 0.29379979535068473`}, {
                    84, 0.28995294789514875`}, {85, 0.2857648880509278}, {
                    86, 0.28129062388327647`}, {87, 0.27658269844021716`}, {
                    88, 0.27169718740444215`}, {89, 0.2666906976134521}, {
                    90, 0.2616168629701307}, {91, 0.2565280602176279}, {
                    92, 0.2514780628083413}, {93, 0.24650941000045604`}, {
                    94, 0.24166636363156704`}, {95, 0.23698007578970054`}, {
                    96, 0.23248304222392727`}, {97, 0.22819453905150927`}, {
                    98, 0.22413178162004652`}, {99, 0.22030220801739753`}, {
                    100, 0.2167062743997629}, {101, 0.21334160263783833`}, {
                    102, 0.21019975129940682`}, {103, 0.2072683036175195}, {
                    104, 0.20453335001586842`}, {105, 0.20197927439289626`}, {
                    106, 0.1995901673952055}, {107, 0.1973507795540104}, {
                    108, 0.19524792079993175`}, {109, 0.19327337457769886`}, {
                    110, 0.1914172015968598}, {111, 0.1896776782531}, {
                    112, 0.1880523688496779}, {113, 0.18654420131680388`}, {
                    114, 0.18515664205590052`}, {115, 0.18390070631243616`}, {
                    116, 0.18278607177438055`}, {117, 0.1818305129298614}, {
                    118, 0.1810474019790032}, {119, 0.18045700911436888`}, {
                    120, 0.1800751183748118}, {121, 0.17991766815397456`}, {
                    122, 0.17999760725938194`}, {123, 0.18032655323051994`}, {
                    124, 0.18091272810541784`}, {125, 0.18176197203862812`}, {
                    126, 0.18287977591601534`}, {127, 0.18426627239305343`}, {
                    128, 0.18592112250725912`}, {129, 0.1878390559561203}, {
                    130, 0.19001498019456217`}, {131, 0.19244006202174596`}, {
                    132, 0.1951012138829728}, {133, 0.197986338017044}, {
                    134, 0.20108064539539905`}, {135, 0.2043662895322691}, {
                    136, 0.20782814554787238`}, {137, 0.21144653564173965`}, {
                    138, 0.21520250197148622`}, {139, 0.21907806251327414`}, {
                    140, 0.22305640919252961`}, {141, 0.22711730009352807`}, {
                    142, 0.23124504948911348`}, {143, 0.23542218374800802`}, {
                    144, 0.23963353016472685`}, {145, 0.24386401835164245`}, {
                    146, 0.24809965530234476`}, {147, 0.2523260908466059}, {
                    148, 0.2565302391304768}, {149, 0.2607016770294811}, {
                    150, 0.2648274984786279}, {151, 0.26889803443234606`}, {
                    152, 0.27290320631651044`}, {153, 0.27683363423461865`}, {
                    154, 0.2806801991248366}, {155, 0.28443525302491}, {
                    156, 0.28809024064643235`}, {157, 0.29163901097496614`}, {
                    158, 0.29507401902021624`}, {159, 0.29838910968607785`}, {
                    160, 0.3015772042376815}, {161, 0.3046348330229162}, {
                    162, 0.30755399493607855`}, {163, 0.31033221756447726`}, {
                    164, 0.3129636336740557}, {165, 0.3154441980736416}, {
                    166, 0.31777007818309055`}, {167, 0.31993764570871985`}, {
                    168, 0.3219434577418436}, {169, 0.32378341534983596`}, {
                    170, 0.32545472489057453`}, {171, 0.32695347780764084`}, {
                    172, 0.32827824120537646`}, {173, 0.32942539135230975`}, {
                    174, 0.33039275650375993`}, {175, 0.3311792980508751}, {
                    176, 0.331782687461141}, {177, 0.3322021952409746}, {
                    178, 0.3324371612101211}, {179, 0.33248655859580284`}, {
                    180, 0.33234942612588225`}, {181, 0.3320263443622201}, {
                    182, 0.3315154262703894}, {183, 0.33081648467692026`}, {
                    184, 0.32993197057730134`}, {185, 0.32886029091017266`}, {
                    186, 0.32760457894796874`}, {187, 0.32616510408389376`}, {
                    188, 0.3245444672501573}, {189, 0.3227446550864884}, {
                    190, 0.32076765029176696`}, {191, 0.31861567058417606`}, {
                    192, 0.31629297780033966`}, {193, 0.3137996491750622}, {
                    194, 0.3111404317865209}, {195, 0.30831810564048734`}, {
                    196, 0.30533689423288973`}, {197, 0.3022020131806279}, {
                    198, 0.29891760907581105`}, {199, 0.2954889170169475}, {
                    200, 0.2919226989198764}, {201, 0.28822068394914535`}, {
                    202, 0.28439015450288807`}, {203, 0.280433669537068}, {
                    204, 0.27635872207917583`}, {205, 0.2721720199165121}, {
                    206, 0.2678772039533209}, {207, 0.2634829490418796}, {
                    208, 0.25899486782896647`}, {209, 0.25441951975904553`}, {
                    210, 0.24976359895689187`}, {211, 0.24503284301055434`}, {
                    212, 0.24023487783718145`}, {213, 0.23537589294801745`}, {
                    214, 0.2304633271025684}, {215, 0.22550377899840038`}, {
                    216, 0.220503698786649}, {217, 0.2154693931477791}, {
                    218, 0.21040862038725164`}, {219, 0.20532763200609877`}, {
                    220, 0.20023297795475978`}, {221, 0.19513113616630748`}, {
                    222, 0.1900287749004839}, {223, 0.18493251329302843`}, {
                    224, 0.17984846819312714`}, {225, 0.17478194192086524`}, {
                    226, 0.16973955556593626`}, {227, 0.16472611577448287`}, {
                    228, 0.1597483479342035}, {229, 0.15481068329760453`}, {
                    230, 0.14991918292058082`}, {231, 0.14507750129117297`}, {
                    232, 0.14029210921109336`}, {233, 0.13556633011942473`}, {
                    234, 0.13090441351713136`}, {235, 0.12631114216836556`}, {
                    236, 0.12178844192530047`}, {237, 0.11734271191792324`}, {
                    238, 0.11297526442350818`}, {239, 0.10868992111755553`}, {
                    240, 0.10448912435596365`}, {241, 0.10037555424227927`}, {
                    242, 0.09635068888676308}, {243, 0.09241834313596697}, {
                    244, 0.08857861685360004}, {245, 0.08483489754908917}, {
                    246, 0.08118628463297588}, {247, 0.07763606160726062}, {
                    248, 0.074182574803797}, {249, 0.0708284200420116}, {
                    250, 0.06757324956683444}, {251, 0.06441805511228864}}],
                   Frame->True,
                   FrameTicks->{{{1., 
                    FormBox[
                    RowBox[{"-", "10"}], TraditionalForm]}, {32.25, 
                    FormBox[
                    RowBox[{"-", "7.5`"}], TraditionalForm]}, {63.5, 
                    FormBox[
                    RowBox[{"-", "5"}], TraditionalForm]}, {94.75, 
                    FormBox[
                    RowBox[{"-", "2.5`"}], TraditionalForm]}, {126., 
                    FormBox["0", TraditionalForm]}, {157.25, 
                    FormBox["2.5`", TraditionalForm]}, {188.5, 
                    FormBox["5", TraditionalForm]}, {219.75, 
                    FormBox["7.5`", TraditionalForm]}, {251., 
                    FormBox["10", TraditionalForm]}}, Automatic, None, None},
                   PlotRange->{-0.1, 1.1},
                   PlotRangeClipping->True,
                   PlotRangePadding->{
                    Scaled[0.02], Automatic}]}, Dynamic[$CellContext`i$$],
                  Alignment->Automatic,
                  ImageSize->All],
                 Identity,
                 Editable->True,
                 Selectable->True]],
                ScriptLevel->0,
                Background->GrayLevel[1]],
               ImageMargins->10],
              Deployed->False,
              StripOnInput->False,
              GraphicsBoxOptions->{PreserveImageOptions->True},
              Graphics3DBoxOptions->{PreserveImageOptions->True}],
             Identity,
             Editable->False,
             Selectable->False],
            Alignment->{Left, Center},
            Background->GrayLevel[1],
            Frame->1,
            FrameStyle->GrayLevel[0, 0.2],
            StripOnInput->False]}
         },
         ColumnsEqual->False,
         GridBoxAlignment->{
          "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Top}}, 
           "RowsIndexed" -> {}},
         GridBoxDividers->{
          "Columns" -> {{False}}, "ColumnsIndexed" -> {}, "Rows" -> {{False}},
            "RowsIndexed" -> {}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.7], {
              Offset[0.5599999999999999]}, 
             Offset[0.7]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.4], {
              Offset[0.8]}, 
             Offset[0.4]}, "RowsIndexed" -> {}},
         RowsEqual->False],
        "Grid"], If[
        CurrentValue["SelectionOver"], 
        Manipulate`Dump`ReadControllerState[
         Map[Manipulate`Dump`updateOneVar[#, 
           CurrentValue["PreviousFormatTime"], 
           CurrentValue["CurrentFormatTime"]]& , {
           
           Manipulate`Dump`controllerLink[{$CellContext`i$$, \
$CellContext`i$4984$$}, "X1", 
            If["DefaultAbsolute", True, "JB1"], False, {1, 24, 1}, 24, 1.]}], 
         
         CurrentValue[{
          "ControllerData", {
           "Gamepad", "Joystick", "Multi-Axis Controller"}}], {}]],
       ImageSizeCache->{392., {138.5, 145.5}}],
      DefaultBaseStyle->{},
      FrameMargins->{{5, 5}, {5, 5}}],
     BaselinePosition->Automatic,
     ImageMargins->0],
    Deinitialization:>None,
    DynamicModuleValues:>{},
    SynchronousInitialization->True,
    UnsavedVariables:>{Typeset`initDone$$},
    UntrackedVariables:>{Typeset`size$$}], "Manipulate",
   Deployed->True,
   StripOnInput->False],
  Manipulate`InterpretManipulate[1]]], "Output",
 CellChangeTimes->{3.3917585311873984`*^9, 3.3917585882688885`*^9, 
  3.391758622889782*^9, 3.391758695523321*^9, 3.391758759403949*^9, 
  3.3917588993880196`*^9, 3.391758967707624*^9, 3.391759007095728*^9, 
  3.3918501959375*^9, 3.392472455556773*^9, 3.392472488717057*^9},
 CellLabel->"Out[65]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"AbsoluteTime", "[", "]"}], "-", "$starttime"}]], "Input",
 CellChangeTimes->{{3.391758934493449*^9, 3.3917589373225155`*^9}},
 CellLabel->"In[66]:="],

Cell[BoxData["21.365334`7.781254679941166"], "Output",
 CellChangeTimes->{3.391758975929109*^9, 3.3917590079241285`*^9, 
  3.391850196515625*^9, 3.3924724562492733`*^9, 3.3924724897731037`*^9},
 CellLabel->"Out[66]="]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
WindowSize->{615, 767},
WindowMargins->{{24, Automatic}, {Automatic, 30}},
PrintingCopies->1,
PrintingPageRange->{1, Automatic},
ShowSelection->True,
Magnification->1.,
FrontEndVersion->"6.0 for Linux x86 (64-bit) (April 20, 2007)",
StyleDefinitions->"TutorialBook.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[590, 23, 99, 2, 173, "Title"],
Cell[692, 27, 56, 3, 51, "Subsubtitle"],
Cell[CellGroupData[{
Cell[773, 34, 333, 8, 32, "Input"],
Cell[1109, 44, 544, 11, 53, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1690, 60, 32, 0, 92, "Section"],
Cell[1725, 62, 411, 9, 101, "Text"],
Cell[2139, 73, 138, 2, 32, "Input"],
Cell[2280, 77, 274, 9, 47, "Text"],
Cell[2557, 88, 278, 7, 101, "Text"],
Cell[2838, 97, 139, 2, 32, "Input"],
Cell[2980, 101, 86, 2, 26, "Text"],
Cell[3069, 105, 74, 1, 32, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[3180, 111, 35, 0, 92, "Section"],
Cell[3218, 113, 184, 4, 47, "Text"],
Cell[3405, 119, 620, 20, 42, "DisplayFormula"],
Cell[4028, 141, 514, 8, 131, "Text"],
Cell[4545, 151, 506, 8, 131, "Text"],
Cell[5054, 161, 480, 15, 374, "Input"],
Cell[5537, 178, 343, 6, 89, "Text"],
Cell[5883, 186, 145, 2, 32, "Input"],
Cell[6031, 190, 267, 5, 68, "Text"],
Cell[6301, 197, 417, 7, 110, "Text"],
Cell[6721, 206, 115, 4, 59, "Input"],
Cell[6839, 212, 259, 5, 68, "Text"],
Cell[7101, 219, 71, 0, 26, "Text"],
Cell[7175, 221, 81, 1, 32, "Input"],
Cell[7259, 224, 307, 8, 68, "Text"],
Cell[7569, 234, 94, 1, 59, "Input"],
Cell[7666, 237, 75, 1, 32, "Input"],
Cell[7744, 240, 232, 4, 68, "Text"],
Cell[7979, 246, 71, 1, 32, "Input"],
Cell[8053, 249, 233, 4, 68, "Text"],
Cell[8289, 255, 147, 3, 47, "Text"],
Cell[8439, 260, 486, 8, 131, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[8962, 273, 40, 0, 92, "Section"],
Cell[9005, 275, 267, 5, 68, "Text"],
Cell[9275, 282, 268, 7, 242, "Input"],
Cell[9546, 291, 236, 4, 68, "Text"],
Cell[9785, 297, 87, 1, 32, "Input"],
Cell[9875, 300, 147, 3, 47, "Text"],
Cell[10025, 305, 188, 6, 85, "Input"],
Cell[10216, 313, 181, 4, 47, "Text"],
Cell[10400, 319, 163, 2, 59, "Input"],
Cell[10566, 323, 194, 4, 68, "Text"],
Cell[10763, 329, 324, 12, 295, "Input"],
Cell[11090, 343, 99, 2, 26, "Text"],
Cell[CellGroupData[{
Cell[11214, 349, 71, 1, 32, "Input"],
Cell[11288, 352, 52214, 1088, 238, 46744, 997, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[63551, 1446, 28, 0, 92, "Section"],
Cell[63582, 1448, 52, 1, 32, "Input"],
Cell[63637, 1451, 182, 5, 26, "Text"],
Cell[CellGroupData[{
Cell[63844, 1460, 213, 3, 59, "Input"],
Cell[64060, 1465, 1438354, 29568, 1700, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1502451, 31038, 212, 4, 32, "Input"],
Cell[1502666, 31044, 1818537, 30951, 312, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[3321240, 62000, 184, 4, 32, "Input"],
Cell[3321427, 62006, 217, 3, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
