(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    502810,       9950]
NotebookOptionsPosition[    489458,       9502]
NotebookOutlinePosition[    490105,       9528]
CellTagsIndexPosition[    490018,       9523]
WindowFrame->Normal
ContainsDynamic->False*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"$Version", ",", 
   RowBox[{"Date", "[", "]"}]}], "}"}]], "Input",
 CellChangeTimes->{{3.3918507375625*^9, 3.39185074071875*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"6.0 for Mac OS X PowerPC (32-bit) (June 19, 2007)\"\>", ",", 
   RowBox[{"{", 
    RowBox[{
    "2007", ",", "7", ",", "19", ",", "18", ",", "58", ",", 
     "3.505478`7.2973222342130555"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.39185083946875*^9, 3.392472609607225*^9, 
  3.392472675907154*^9, 3.393850538264327*^9, 3.393851022356381*^9, 
  3.3938530835488997`*^9}]
}, Open  ]],

Cell["\<\
RM: todo (?) :Lighting is different to 5.2 (completely different ...) .
     I added Glow as an option.\
\>", "Program",
 CellChangeTimes->{{3.391850760140625*^9, 3.3918508313125*^9}}],

Cell[CellGroupData[{

Cell["The package Spinors.m", "Title"],

Cell["\<\
Bernd Thaller
Institute of Mathematics
University of Graz
Austria
2006-08-08\
\>", "Subsubtitle"],

Cell[CellGroupData[{

Cell["Abstract", "Subsection"],

Cell[TextData[{
 "The package VQM`Spinors defines basic operations in the two-dimensional \
complex Hilbert space of a qubit. It defines the Pauli matrices and provides \
tools to visualize pure and mixed states of a qubit. This package is part of \
the VQM packages which can be obtained here:  ",
 ButtonBox["http://www.uni-graz.at/imawww/vqm/software.html",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.uni-graz.at/imawww/vqm/software.html"], None}],
 " (free download). The VQM packages are part of the Visual Quantum Mechanics \
project, see ",
 ButtonBox["http://www.uni-graz.at/imawww/vqm/",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.uni-graz.at/imawww/vqm/"], None}],
 ". In particular, these packages are distributed with the book ",
 StyleBox["Advanced Visual Quantum Mechanics",
  FontSlant->"Italic"],
 ", Springer-Verlag New York, 2004."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Load package", "Section"],

Cell["Needs[\"VQM`\"];", "Input",
 CellChangeTimes->{{3.3917813578541613`*^9, 3.3917813671141*^9}, {
   3.392472651138022*^9, 3.392472656068408*^9}, 3.3938509107756233`*^9, {
   3.393851005263566*^9, 3.393851006148788*^9}, 3.393854570679183*^9}],

Cell["", "Input",
 CellChangeTimes->{{3.3917813578541613`*^9, 3.3917813671141*^9}, {
   3.392472651138022*^9, 3.392472656068408*^9}, 3.3938509107756233`*^9, 
   3.393851003787365*^9}],

Cell[TextData[{
 "This also loads the package \"",
 StyleBox["VQM`VisualizeVector", "InlineInput"],
 "\"."
}], "Text",
 CellChangeTimes->{3.39178137583251*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Bases", "Section"],

Cell[TextData[{
 "A spinor is a complex two-dimensional vector {",
 Cell[BoxData[
  RowBox[{
   SubscriptBox["c", "1"], ",", 
   SubscriptBox["c", "2"]}]]],
 "}, ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox["c", "i"], "\[Epsilon]", " "}]]],
 "\[DoubleStruckCapitalC],",
 " that is, an element of the linear space ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalC]", "2"], TraditionalForm]]],
 ". The spinors are usually defined with respect to a reference basis in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalC]", "2"], TraditionalForm]]],
 ". The package defines the canonical standard basis"
}], "Text"],

Cell[CellGroupData[{

Cell["$QSpinBasis", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.390201928208454*^9, 3.391781381966646*^9, 3.39185069625*^9, 
   3.391850839625*^9, 3.392472610314433*^9, 3.392472676625819*^9, 
   3.393850538730576*^9, {3.393850932964362*^9, 3.393850939532649*^9}, 
   3.393851023084322*^9, 3.393853084277861*^9}]
}, Open  ]],

Cell[TextData[{
 "The basis spinors are interpreted as \"spin-up\" and \"spin-down\" with \
respect to the z-axis of a Cartesian basis in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalR]", "3"], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "Other standard bases provided by the package are ",
 StyleBox["QxBasis", "InlineInput"],
 ", ",
 StyleBox["QyBasis", "InlineInput"],
 ", and ",
 StyleBox["QzBasis", "InlineInput"],
 " (",
 StyleBox["QzBasis", "InlineInput"],
 " is equal to ",
 StyleBox["$QSpinBasis", "InlineInput"],
 "). For example, ",
 StyleBox["QxBasis", "InlineInput"],
 " consists of up and down spinors with respect to the x-axis:"
}], "Text"],

Cell[CellGroupData[{

Cell["QxBasis", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     FractionBox["1", 
      SqrtBox["2"]], ",", 
     FractionBox["1", 
      SqrtBox["2"]]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SqrtBox["2"]]}], ",", 
     FractionBox["1", 
      SqrtBox["2"]]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.390201932484603*^9, 3.3917813819880533`*^9, 3.391850696296875*^9, 
   3.3918508396875*^9, 3.392472610355606*^9, 3.392472676773801*^9, 
   3.393850538787444*^9, {3.393850933241325*^9, 3.393850939737109*^9}, 
   3.393851023152471*^9, 3.3938530843287*^9}]
}, Open  ]],

Cell[TextData[{
 "Note that the symbols defined by the VQM packages all start with the letter \
Q in order to prevent conflicts with existing symbols defined in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " or in other third-party packages."
}], "Text"],

Cell[TextData[{
 "The package also defines ",
 StyleBox["QSpinBasis", "InlineInput"],
 " as a default basis for some basis-dependent commands. It is initially \
equal to ",
 StyleBox["$QSpinBasis", "InlineInput"],
 ", but can be changed with the command ",
 StyleBox["QSetSpinBasis[{", "InlineInput"],
 "basespinor1",
 StyleBox[",", "InlineInput"],
 "basespinor2",
 StyleBox["}]", "InlineInput"],
 ". This influences how the components of spinors are calculated."
}], "Text"],

Cell[CellGroupData[{

Cell["\<\
QSetSpinBasis[QxBasis];
QSpinBasis\
\>", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     FractionBox["1", 
      SqrtBox["2"]], ",", 
     FractionBox["1", 
      SqrtBox["2"]]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       SqrtBox["2"]]}], ",", 
     FractionBox["1", 
      SqrtBox["2"]]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.390201953865347*^9, 3.391781382051331*^9, 3.391850696328125*^9, 
   3.39185083971875*^9, 3.392472610389121*^9, 3.392472676930443*^9, 
   3.393850538854597*^9, {3.3938509333155327`*^9, 3.393850939814931*^9}, 
   3.3938510232351227`*^9, 3.393853084378121*^9}]
}, Open  ]],

Cell[TextData[{
 "An example of an operation that depends on the chosen basis is the \
calculation of the components of a spinor with respect to that basis. This is \
done with the command ",
 StyleBox["QSpinorToComponents", "InlineInput"],
 ". The spinor psi is given with respect to the reference basis ",
 StyleBox["$QSpinBasis", "InlineInput"],
 ". ",
 StyleBox["QSpinorToComponents[", "InlineInput"],
 "psi",
 StyleBox["]", "InlineInput"],
 " calculates the components of psi with respect to the basis ",
 StyleBox["QSpinBasis", "InlineInput"],
 ". This gives the components of the spinor {1,0} with respect to the \
previously set ",
 StyleBox["QSpinBasis", "InlineInput"],
 ":"
}], "Text"],

Cell[CellGroupData[{

Cell["components = QSpinorToComponents[{1,0}]", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox["1", 
    SqrtBox["2"]], ",", 
   RowBox[{"-", 
    FractionBox["1", 
     SqrtBox["2"]]}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.3902019559884*^9, 3.3917813821057367`*^9, 3.391850696375*^9, 
   3.391850839765625*^9, 3.3924726104208813`*^9, 3.3924726773709*^9, 
   3.393850538935712*^9, {3.393850933463881*^9, 3.39385093988135*^9}, 
   3.393851023318317*^9, 3.39385308444452*^9}]
}, Open  ]],

Cell[TextData[{
 StyleBox["components.QSpinBasis", "InlineInput"],
 " returns the original spinor. More generally, ",
 StyleBox["QComponentsToSpinor", "InlineInput"],
 " takes two complex coefficients and forms the corresponding linear \
combination of the basis vectors in ",
 StyleBox["QSpinBasis", "InlineInput"],
 ". The result is the spinor in the canonical standard basis."
}], "Text"],

Cell[CellGroupData[{

Cell["QComponentsToSpinor[components]", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{
  3.3902019579512224`*^9, 3.391781382155744*^9, 3.391850696421875*^9, 
   3.391850839796875*^9, 3.392472610457427*^9, 3.392472677537878*^9, 
   3.393850539021014*^9, {3.3938509336141367`*^9, 3.393850939948791*^9}, 
   3.3938510235397778`*^9, 3.393853084527849*^9}]
}, Open  ]],

Cell[TextData[{
 StyleBox["QProjectUp[", "InlineInput"],
 "psi",
 StyleBox["]", "InlineInput"],
 " projects a spinor psi into the direction of the first basis vector of ",
 StyleBox["QSpinBasis", "InlineInput"],
 ", "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QProjectUp", "[", 
  RowBox[{"{", 
   RowBox[{"1", ",", "0"}], "}"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox["1", "2"], ",", 
   FractionBox["1", "2"]}], "}"}]], "Output",
 CellChangeTimes->{
  3.390201959683714*^9, 3.391781382206231*^9, 3.3918506965*^9, 
   3.39185083984375*^9, 3.392472610490858*^9, 3.3924726777178497`*^9, 
   3.393850539104166*^9, {3.3938509336816187`*^9, 3.393850940015727*^9}, 
   3.393851023635273*^9, 3.3938530848199244`*^9}]
}, Open  ]],

Cell[TextData[{
 StyleBox["QProjectDown[", "InlineInput"],
 "psi",
 StyleBox["]", "InlineInput"],
 " gives the part proportional to the second basis vector. The sum of the two \
projections is the identity, that is,"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"sp", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{"1", ",", "I"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"QProjectUp", "[", "sp", "]"}], "+", 
   RowBox[{"QProjectDown", "[", "sp", "]"}]}], " ", "\[Equal]", " ", 
  "sp"}]}], "Input"],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{
  3.3902019611257873`*^9, 3.391781382260758*^9, 3.391850696546875*^9, 
   3.391850839875*^9, 3.392472610504342*^9, 3.392472677887038*^9, 
   3.393850539187612*^9, {3.39385093374746*^9, 3.393850940081464*^9}, 
   3.3938510237535067`*^9, 3.3938530848963957`*^9}]
}, Open  ]],

Cell[TextData[{
 StyleBox["QProbabilityUp[", "InlineInput"],
 "psi",
 StyleBox["]", "InlineInput"],
 " gives the probability that the spinor has spin-up with respect to the ",
 StyleBox["QSpinBasis", "InlineInput"],
 ", ",
 StyleBox["QProbabilityDown[", "InlineInput"],
 "psi",
 StyleBox["]", "InlineInput"],
 " gives the probability for spin down."
}], "Text"],

Cell["\<\
A natural method for the visualization of a spinor is to represent its \
components with respect to a basis by a bar diagram. The height of the bars \
gives the absolute value, and the color gives the phase of the component.\
\>", "Text"],

Cell[CellGroupData[{

Cell["QSpinorBarDiagram[{1,0}]", "Input"],

Cell[BoxData[
 GraphicsBox[{
   {Hue[0], 
    RectangleBox[{0.05, 0}, 
     NCache[{0.95, 2^Rational[-1, 2]}, {0.95, 0.7071067811865475}]]}, 
   {Hue[
     NCache[
      Rational[1, 2], 0.5]], 
    RectangleBox[{1.05, 0}, 
     NCache[{1.95, 2^Rational[-1, 2]}, {1.95, 0.7071067811865475}]]}},
  Epilog->{
    GrayLevel[0], 
    LineBox[{{0, 0}, {2, 0}}], 
    LineBox[{{0, 1}, {2, 1}}]},
  Frame->True,
  FrameTicks->{{{0.5, 
      FormBox["\"\\!\\(c\\_1\\)\"", TraditionalForm]}, {1.5, 
      FormBox["\"\\!\\(c\\_2\\)\"", TraditionalForm]}}, Automatic, None, 
    Automatic},
  PlotRange->{{-0.1, 2.1}, {-0.1, 1.1}}]], "Output",
 CellChangeTimes->{
  3.3902019638396893`*^9, 3.391781382333283*^9, 3.391850696734375*^9, 
   3.391850839921875*^9, 3.392472610550968*^9, 3.392472678063467*^9, 
   3.393850539275618*^9, {3.393850933864749*^9, 3.393850940149613*^9}, 
   3.393851023842098*^9, 3.393853084986668*^9}]
}, Open  ]],

Cell[TextData[{
 "Note that the image above shows the components {",
 Cell[BoxData[
  RowBox[{
   SubscriptBox["c", "1"], ",", 
   SubscriptBox["c", "2"]}]]],
 "} of the spinor with respect to the chosen ",
 StyleBox["QSpinBasis", "InlineInput"],
 " (= ",
 StyleBox["QxBasis", "InlineInput"],
 "). Let us restore the default setting. The following image then gives the \
components of the same spinor with respect to the canonical standard basis ",
 StyleBox["$QSpinBasis = QzBasis", "InlineInput"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell["\<\
QSetSpinBasis[$QSpinBasis];
QSpinorBarDiagram[{1,0}]\
\>", "Input"],

Cell[BoxData[
 GraphicsBox[{
   {Hue[0], RectangleBox[{0.05, 0}, {0.95, 1}]}, 
   {Hue[0], RectangleBox[{1.05, 0}, {1.95, 0}]}},
  Epilog->{
    GrayLevel[0], 
    LineBox[{{0, 0}, {2, 0}}], 
    LineBox[{{0, 1}, {2, 1}}]},
  Frame->True,
  FrameTicks->{{{0.5, 
      FormBox["\"\\!\\(c\\_1\\)\"", TraditionalForm]}, {1.5, 
      FormBox["\"\\!\\(c\\_2\\)\"", TraditionalForm]}}, Automatic, None, 
    Automatic},
  PlotRange->{{-0.1, 2.1}, {-0.1, 1.1}}]], "Output",
 CellChangeTimes->{
  3.3902019947541423`*^9, 3.3917813823576403`*^9, 3.39185069684375*^9, 
   3.39185083996875*^9, 3.392472610581937*^9, 3.3924726810983143`*^9, 
   3.393850539444211*^9, {3.393850933981311*^9, 3.3938509402158833`*^9}, 
   3.393851023999545*^9, 3.393853085063027*^9}]
}, Open  ]],

Cell[TextData[{
 "All the commands above accept the option ",
 StyleBox["QUseBasis", "InlineInput"],
 ". You can use this option when you want use a particular basis just for one \
command. The following command gives the spin-up part with respect to the ",
 StyleBox["QyBasis", "InlineInput"],
 " of the spinor {1,-1}."
}], "Text"],

Cell[CellGroupData[{

Cell["QProjectUp[{1,-1},QUseBasis->QyBasis]", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], "+", 
    FractionBox["\[ImaginaryI]", "2"]}], ",", 
   RowBox[{
    RowBox[{"-", 
     FractionBox["1", "2"]}], "+", 
    FractionBox["\[ImaginaryI]", "2"]}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.3902020128101053`*^9, 3.3917813824121733`*^9, 3.391850696890625*^9, 
   3.39185084003125*^9, 3.392472610606029*^9, 3.3924726815240498`*^9, 
   3.3938505395189867`*^9, {3.3938509342365093`*^9, 3.393850940281431*^9}, 
   3.393851024086286*^9, 3.393853085145088*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["QSpinorToComponents[{1,-1}, QUseBasis->QxBasis]", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", 
   RowBox[{"-", 
    SqrtBox["2"]}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.390202013871632*^9, 3.391781382465633*^9, 3.391850696921875*^9, 
   3.39185084009375*^9, 3.392472610618846*^9, 3.392472681670175*^9, 
   3.393850539586351*^9, {3.3938509342977953`*^9, 3.39385094034824*^9}, 
   3.393851024169344*^9, 3.393853085212125*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["QSpinorBarDiagram[{1,-1}, QUseBasis->QyBasis]", "Input"],

Cell[BoxData[
 GraphicsBox[{
   {Hue[
     NCache[
      Rational[1, 8], 0.125]], RectangleBox[{0.05, 0}, {0.95, 1}]}, 
   {Hue[
     NCache[
      Rational[3, 8], 0.375]], RectangleBox[{1.05, 0}, {1.95, 1}]}},
  Epilog->{
    GrayLevel[0], 
    LineBox[{{0, 0}, {2, 0}}], 
    LineBox[{{0, 1}, {2, 1}}]},
  Frame->True,
  FrameTicks->{{{0.5, 
      FormBox["\"\\!\\(c\\_1\\)\"", TraditionalForm]}, {1.5, 
      FormBox["\"\\!\\(c\\_2\\)\"", TraditionalForm]}}, Automatic, None, 
    Automatic},
  PlotRange->{{-0.1, 2.1}, {-0.1, 1.1}}]], "Output",
 CellChangeTimes->{
  3.3902020149231443`*^9, 3.391781382483757*^9, 3.391850696984375*^9, 
   3.391850840140625*^9, 3.3924726106370087`*^9, 3.392472681830912*^9, 
   3.3938505396541357`*^9, {3.393850934431279*^9, 3.3938509404154663`*^9}, 
   3.3938510243158503`*^9, 3.393853085280059*^9}]
}, Open  ]],

Cell["You can compute the norm of a spinor with", "Text"],

Cell[CellGroupData[{

Cell["Norm[{2,-I}]", "Input"],

Cell[BoxData[
 SqrtBox["5"]], "Output",
 CellChangeTimes->{
  3.3902020167257357`*^9, 3.391781382501028*^9, 3.391850697015625*^9, 
   3.391850840171875*^9, 3.392472610680442*^9, 3.3924726820728197`*^9, 
   3.3938505397443132`*^9, {3.393850934580339*^9, 3.393850940481289*^9}, 
   3.393851024465413*^9, 3.393853085361574*^9}]
}, Open  ]],

Cell[TextData[{
 "and ",
 StyleBox["QNormalize", "InlineInput"],
 " returns a spinor with norm 1"
}], "Text"],

Cell[CellGroupData[{

Cell["QNormalize[{2,-I}]", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox["2", 
    SqrtBox["5"]], ",", 
   RowBox[{"-", 
    FractionBox["\[ImaginaryI]", 
     SqrtBox["5"]]}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.3902020181377664`*^9, 3.391781382513804*^9, 3.3918506970625*^9, 
   3.39185084021875*^9, 3.3924726108698807`*^9, 3.392472682250346*^9, 
   3.393850539802939*^9, {3.393850934713208*^9, 3.393850940549192*^9}, 
   3.393851024865817*^9, 3.3938530854288797`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Norm[%]", "Input"],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{
  3.3902020193595233`*^9, 3.391781382561142*^9, 3.391850697109375*^9, 
   3.39185084025*^9, 3.3924726109190903`*^9, 3.392472682672188*^9, 
   3.393850539898178*^9, {3.393850934781204*^9, 3.393850940735621*^9}, 
   3.393851024934111*^9, 3.393853085511341*^9}]
}, Open  ]],

Cell[TextData[{
 " The function ",
 StyleBox["QNormalize", "InlineInput"],
 " can also be used to normalize arbitrary vectors"
}], "Text"],

Cell[CellGroupData[{

Cell["QNormalize[{1,2,3}]", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox["1", 
    SqrtBox["14"]], ",", 
   SqrtBox[
    FractionBox["2", "7"]], ",", 
   FractionBox["3", 
    SqrtBox["14"]]}], "}"}]], "Output",
 CellChangeTimes->{
  3.3902020208416543`*^9, 3.391781382574592*^9, 3.39185069725*^9, 
   3.391850840296875*^9, 3.392472610951498*^9, 3.392472682833084*^9, 
   3.393850539985578*^9, {3.393850934848278*^9, 3.393850940798903*^9}, 
   3.393851025066687*^9, 3.393853085594844*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Some matrices", "Section"],

Cell[TextData[{
 "The package \"",
 StyleBox["VQM`Spinors", "InlineInput"],
 "\" also defines the some important operators. First of all, we have the \
Pauli spin matrices ",
 StyleBox["QPauliSigma1", "InlineInput"],
 ", ",
 StyleBox["QPauliSigma2", "InlineInput"],
 ", and ",
 StyleBox["QPauliSigma3", "InlineInput"],
 ". Moreover, ",
 StyleBox["QPauliSigmaV", "InlineInput"],
 " is a vector with the three Pauli matrices as components"
}], "Text"],

Cell[CellGroupData[{

Cell["QPauliSigma2//MatrixForm", "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", 
      RowBox[{"-", "\[ImaginaryI]"}]},
     {"\[ImaginaryI]", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.3902020232250814`*^9, 3.391781382631901*^9, 
  3.3918506974375*^9, 3.391850840328125*^9, 3.392472610998458*^9, 
  3.392472682923409*^9, 3.393850540075253*^9, 3.3938509455099688`*^9, 
  3.393851025234386*^9, 3.393853086122623*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["QPauliSigmaV", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0"}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", 
       RowBox[{"-", "\[ImaginaryI]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"\[ImaginaryI]", ",", "0"}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", 
       RowBox[{"-", "1"}]}], "}"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.3902020244167953`*^9, 3.391781382646007*^9, 
  3.3918506975*^9, 3.391850840421875*^9, 3.3924726110332537`*^9, 
  3.3924726830253973`*^9, 3.39385054015593*^9, 3.39385094571671*^9, 
  3.3938510253832483`*^9, 3.393853086194899*^9}]
}, Open  ]],

Cell[TextData[{
 "The two-by-two identity matrix is ",
 StyleBox["QIdentity2", "InlineInput"],
 ". The Pauli sigma matrices are the quantum-mechanical observables for the \
spin measured with respect to the x-, y-, and z-axis of the reference \
coordinate system in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalR]", "3"], TraditionalForm]]],
 ". The spin-observable with respect to an arbitrary axis (represented by a \
vector ",
 StyleBox["kv", "InlineInput"],
 ") is ",
 StyleBox["QSpinHamiltonian[kv]", "InlineInput"],
 ". This matrix is obtained as\n ",
 StyleBox["QPauliSigma1.kv[[1]]+QPauliSigma2.kv[[2]]+QPauliSigma3.kv[[3]]", 
  "InlineInput"]
}], "Text"],

Cell["This is a three-dimensional vector with symbolic components", "Text"],

Cell["\<\
Clear[k0, k1, k2, k3, kv];
kv = {k1,k2,k3};\
\>", "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hs", "=", 
  StyleBox[
   RowBox[{"QSpinHamiltonian", "[", "kv", "]"}], "InlineInput"]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"k3", ",", 
     RowBox[{"k1", "-", 
      RowBox[{"\[ImaginaryI]", " ", "k2"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"k1", "+", 
      RowBox[{"\[ImaginaryI]", " ", "k2"}]}], ",", 
     RowBox[{"-", "k3"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.3902020293038225`*^9, 3.391781382706217*^9, 
  3.391850697578125*^9, 3.3918508405*^9, 3.392472611072085*^9, 
  3.392472683220715*^9, 3.393850540263027*^9, 3.393850945816477*^9, 
  3.3938510254674997`*^9, 3.393853086305575*^9}]
}, Open  ]],

Cell[TextData[{
 "A density matrix is a hermitian matrix with trace 1. It is also determined \
by a vector in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalR]", "3"], TraditionalForm]]],
 ":"
}], "Text"],

Cell[TextData[{
 "This is the density matrix describing spin in the direction of ",
 StyleBox["kv", "InlineInput"]
}], "Text"],

Cell[CellGroupData[{

Cell["ma = QVectorToDensityMatrix[kv]", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     FractionBox[
      RowBox[{"1", "+", "k3"}], "2"], ",", 
     RowBox[{
      FractionBox["1", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"k1", "-", 
        RowBox[{"\[ImaginaryI]", " ", "k2"}]}], ")"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      FractionBox["1", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"k1", "+", 
        RowBox[{"\[ImaginaryI]", " ", "k2"}]}], ")"}]}], ",", 
     FractionBox[
      RowBox[{"1", "-", "k3"}], "2"]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.390202031577091*^9, 3.391781382757916*^9, 
  3.391850697609375*^9, 3.391850840640625*^9, 3.392472611114621*^9, 
  3.392472683319537*^9, 3.393850540318858*^9, 3.393850945882504*^9, 
  3.393851025656457*^9, 3.393853086360971*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ma", "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox[
       RowBox[{"1", "+", "k3"}], "2"], 
      RowBox[{
       FractionBox["1", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"k1", "-", 
         RowBox[{"\[ImaginaryI]", " ", "k2"}]}], ")"}]}]},
     {
      RowBox[{
       FractionBox["1", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"k1", "+", 
         RowBox[{"\[ImaginaryI]", " ", "k2"}]}], ")"}]}], 
      FractionBox[
       RowBox[{"1", "-", "k3"}], "2"]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.3902020647547984`*^9, 3.391781382814259*^9, 
  3.391850697671875*^9, 3.3918508406875*^9, 3.392472611130831*^9, 
  3.3924726837219477`*^9, 3.393850540385425*^9, 3.393850945950344*^9, 
  3.39385102571809*^9, 3.39385308646824*^9}]
}, Open  ]],

Cell["\<\
A four-vector {k0,k1,k2,k3} can be converted into a Hermitian matrix by:\
\>", "Text"],

Cell[CellGroupData[{

Cell["QVectorToHermitianMatrix[{k0,k1,k2,k3}]", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"k0", "+", "k3"}], ",", 
     RowBox[{"k1", "-", 
      RowBox[{"\[ImaginaryI]", " ", "k2"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"k1", "+", 
      RowBox[{"\[ImaginaryI]", " ", "k2"}]}], ",", 
     RowBox[{"k0", "-", "k3"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.39020206714824*^9, 3.391781382852118*^9, 
  3.39185069775*^9, 3.39185084071875*^9, 3.392472611173853*^9, 
  3.392472683866301*^9, 3.393850540540615*^9, 3.3938509460346003`*^9, 
  3.393851025783833*^9, 3.393853086585496*^9}]
}, Open  ]],

Cell["and a three-vector {k1,k2,k3} by", "Text"],

Cell[CellGroupData[{

Cell["QVectorToHermitianMatrix[{k1,k2,k3}]", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"k3", ",", 
     RowBox[{"k1", "-", 
      RowBox[{"\[ImaginaryI]", " ", "k2"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"k1", "+", 
      RowBox[{"\[ImaginaryI]", " ", "k2"}]}], ",", 
     RowBox[{"-", "k3"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.390202068380011*^9, 3.391781382867155*^9, 
  3.3918506978125*^9, 3.391850840765625*^9, 3.39247261118705*^9, 
  3.392472683991275*^9, 3.393850540602408*^9, 3.393850946116417*^9, 
  3.393851025850088*^9, 3.393853086701021*^9}]
}, Open  ]],

Cell["which is the same as QSpinHamiltonian[{k1,k2,k3}]", "Text"],

Cell["\<\
There is also an inverse operation which determines the vector from a \
Hermitian matrix:\
\>", "Text"],

Cell[TextData[{
 StyleBox["QHermitianMatrixToVector", "InlineInput"],
 " is inverse to ",
 StyleBox["QVectorToHermitianMatrix", "InlineInput"],
 ":"
}], "Text"],

Cell[TextData[{
 StyleBox["QDensityMatrixToVector", "InlineInput"],
 " is inverse to ",
 StyleBox["QVectorToDensityMatrix", "InlineInput"],
 ":"
}], "Text"],

Cell[CellGroupData[{

Cell["\<\
QHermitianMatrixToVector[QVectorToHermitianMatrix[{k0,k1,k2,k3}]] // Simplify\
\
\>", "Input",
 CellChangeTimes->{{3.390202079315736*^9, 3.3902021169598656`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"k0", ",", "k1", ",", "k2", ",", "k3"}], "}"}]], "Output",
 CellChangeTimes->{{3.390202071113942*^9, 3.3902020991843057`*^9}, 
   3.391781383049754*^9, 3.391850697953125*^9, 3.391850840859375*^9, 
   3.3924726112186537`*^9, 3.392472684264398*^9, 3.3938505406686277`*^9, 
   3.393850946198501*^9, 3.393851025917292*^9, 3.393853087087468*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
QDensityMatrixToVector[QVectorToDensityMatrix[{k1,k2,k3}]] // Simplify\
\>", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"k1", ",", "k2", ",", "k3"}], "}"}]], "Output",
 CellChangeTimes->{3.3902020724759007`*^9, 3.390202122557915*^9, 
  3.3917813831028433`*^9, 3.391850697984375*^9, 3.391850840921875*^9, 
  3.392472611233101*^9, 3.392472684537385*^9, 3.393850540735445*^9, 
  3.393850946281904*^9, 3.393851025984064*^9, 3.393853087202943*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
ma == QVectorToDensityMatrix[QDensityMatrixToVector[ma]] //Simplify\
\>", "Input"],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.3902020739480176`*^9, 3.3902021232388945`*^9, 
  3.391781383118181*^9, 3.39185069803125*^9, 3.391850840953125*^9, 
  3.3924726112470818`*^9, 3.3924726848433943`*^9, 3.393850540802367*^9, 
  3.3938509464298563`*^9, 3.3938510260507097`*^9, 3.393853087321947*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["The spinors QSpinorUp and QSpinorDown", "Section"],

Cell[TextData[{
 "The package defines two basis spinors for each direction in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalR]", "3"], TraditionalForm]]],
 ". These spinors are ",
 StyleBox["QSpinorUp[", "InlineInput"],
 "vector",
 StyleBox["]", "InlineInput"],
 " and ",
 StyleBox["QSpinorDown[", "InlineInput"],
 "vector",
 StyleBox["]", "InlineInput"],
 ". Here 'vector ' defines the direction. These spinors are eigenvectors of \
the matrix ",
 StyleBox["QVectorToMatrix[", "InlineInput"],
 "vector",
 StyleBox["]", "InlineInput"],
 "."
}], "Text"],

Cell["Clear[k1, k2, k3, kv]; kv = {k1,k2,k3};", "Input"],

Cell[CellGroupData[{

Cell["\<\
QVectorToDensityMatrix[kv].QSpinorUp[kv] == Sqrt[kv.kv] \
QSpinorUp[kv]//Simplify\
\>", "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      RowBox[{
       SuperscriptBox["k1", "2"], "+", 
       SuperscriptBox["k2", "2"], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", "k3"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{"k3", "+", 
          SqrtBox[
           RowBox[{
            SuperscriptBox["k1", "2"], "+", 
            SuperscriptBox["k2", "2"], "+", 
            SuperscriptBox["k3", "2"]}]]}], ")"}]}]}], 
      RowBox[{"2", " ", 
       SqrtBox["2"], " ", 
       SqrtBox[
        RowBox[{
         SuperscriptBox["k1", "2"], "+", 
         SuperscriptBox["k2", "2"], "+", 
         RowBox[{"k3", " ", 
          RowBox[{"(", 
           RowBox[{"k3", "+", 
            SqrtBox[
             RowBox[{
              SuperscriptBox["k1", "2"], "+", 
              SuperscriptBox["k2", "2"], "+", 
              SuperscriptBox["k3", "2"]}]]}], ")"}]}]}]]}]]}], ",", 
    RowBox[{"-", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{"k1", "+", 
         RowBox[{"\[ImaginaryI]", " ", "k2"}]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         SqrtBox[
          RowBox[{
           SuperscriptBox["k1", "2"], "+", 
           SuperscriptBox["k2", "2"], "+", 
           SuperscriptBox["k3", "2"]}]]}], ")"}]}], 
      RowBox[{"2", " ", 
       SqrtBox["2"], " ", 
       SqrtBox[
        RowBox[{
         SuperscriptBox["k1", "2"], "+", 
         SuperscriptBox["k2", "2"], "+", 
         RowBox[{"k3", " ", 
          RowBox[{"(", 
           RowBox[{"k3", "+", 
            SqrtBox[
             RowBox[{
              SuperscriptBox["k1", "2"], "+", 
              SuperscriptBox["k2", "2"], "+", 
              SuperscriptBox["k3", "2"]}]]}], ")"}]}]}]]}]]}]}], "}"}], 
  "\[Equal]", 
  RowBox[{"{", 
   RowBox[{"0", ",", "0"}], "}"}]}]], "Output",
 CellChangeTimes->{3.390202139021589*^9, 3.391781383515233*^9, 
  3.391850698515625*^9, 3.391850841*^9, 3.3924726114865522`*^9, 
  3.392472685294332*^9, 3.393850541204372*^9, 3.393850950379869*^9, 
  3.393851026389057*^9, 3.393853087707086*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
QVectorToHermitianMatrix[kv].QSpinorDown[kv] == -Sqrt[kv.kv] \
QSpinorDown[kv]//Simplify\
\>", "Input"],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.3902021705769634`*^9, 3.3917813836471777`*^9, 
  3.391850698796875*^9, 3.391850841046875*^9, 3.3924726116162367`*^9, 
  3.392472685385206*^9, 3.393850541320126*^9, 3.39385095049613*^9, 
  3.393851026570148*^9, 3.393853088634796*^9}]
}, Open  ]],

Cell[TextData[{
 "Hence, for example, ",
 StyleBox["QSpinorUp[", "InlineInput"],
 "vector",
 StyleBox["]", "InlineInput"],
 " is the state with spin-up in the direction of 'vector'. The two spinors \
form an orthonormal basis in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalC]", "2"], TraditionalForm]]],
 ". For example,"
}], "Text"],

Cell[CellGroupData[{

Cell["QConjSpinorUp[kv].QSpinorUp[kv]//Simplify", "Input"],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.3902021733709807`*^9, 3.391781383670657*^9, 
  3.391850698890625*^9, 3.391850841078125*^9, 3.392472611660952*^9, 
  3.39247268540983*^9, 3.393850541385007*^9, 3.3938509505657177`*^9, 
  3.3938510267924423`*^9, 3.393853088740917*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["QConjSpinorDown[kv].QSpinorUp[kv]//Simplify", "Input"],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.390202174752968*^9, 3.391781383696581*^9, 
  3.391850698953125*^9, 3.391850841125*^9, 3.3924726118841963`*^9, 
  3.392472685474699*^9, 3.3938505414521914`*^9, 3.393850950696599*^9, 
  3.39385102689555*^9, 3.393853089182399*^9}]
}, Open  ]],

Cell[TextData[{
 "The conjugate comples spinors ",
 StyleBox["QConjSpinorUp[kv]", "InlineInput"],
 " and ",
 StyleBox["QConjSpinorDown[kv]", "InlineInput"],
 " have been defined for convenience."
}], "Text"],

Cell[TextData[{
 "We also note that ",
 StyleBox["QSpinorUp[kv]", "InlineInput"],
 " has a phase discontinuity on the negative z-axis, and ",
 StyleBox["QSpinorDown", "InlineInput"],
 StyleBox["[kv]", "InlineInput"],
 " has a phase discontinuity on the positive z-axis."
}], "Text"],

Cell[CellGroupData[{

Cell["QSpinorUp[{0,0,-1}]", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "1"}], "}"}]], "Output",
 CellChangeTimes->{3.3902021765255165`*^9, 3.3917813837100763`*^9, 
  3.391850699*^9, 3.39185084115625*^9, 3.392472611897161*^9, 
  3.392472685487866*^9, 3.3938505416146593`*^9, 3.393850950764587*^9, 
  3.393851026967766*^9, 3.3938530897433662`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Limit[QSpinorUp[{eps,0,-1}],eps->0]", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "1"}], "}"}]], "Output",
 CellChangeTimes->{3.390202178388195*^9, 3.39178138415609*^9, 
  3.391850699828125*^9, 3.39185084121875*^9, 3.392472612252*^9, 
  3.392472685758666*^9, 3.393850542008707*^9, 3.39385095093888*^9, 
  3.393851027205554*^9, 3.3938530899578457`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Limit[QSpinorUp[{-eps,0,-1}],eps->0]", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", 
   RowBox[{"-", "1"}]}], "}"}]], "Output",
 CellChangeTimes->{3.3902021791593037`*^9, 3.391781384280548*^9, 
  3.3918506999375*^9, 3.391850841296875*^9, 3.3924726122908907`*^9, 
  3.392472686166675*^9, 3.393850542069158*^9, 3.393850950997624*^9, 
  3.393851027285137*^9, 3.393853090306106*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Limit[QSpinorUp[{0,eps,-1}],eps->0]", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "\[ImaginaryI]"}], "}"}]], "Output",
 CellChangeTimes->{3.3902021810319967`*^9, 3.391781384298647*^9, 
  3.391850699984375*^9, 3.39185084134375*^9, 3.392472612310802*^9, 
  3.392472686239806*^9, 3.3938505421363173`*^9, 3.393850951184548*^9, 
  3.393851027385826*^9, 3.393853090401849*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Spinors and Vectors", "Section"],

Cell["Consider the unit vector in three-dimensional space", "Text"],

Cell[CellGroupData[{

Cell["vec = {-4/9,8/9,-1/9}", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox["4", "9"]}], ",", 
   FractionBox["8", "9"], ",", 
   RowBox[{"-", 
    FractionBox["1", "9"]}]}], "}"}]], "Output",
 CellChangeTimes->{3.390202183515568*^9, 3.3917813843124123`*^9, 
  3.391850700015625*^9, 3.391850841375*^9, 3.3924726123404922`*^9, 
  3.392472686267886*^9, 3.393850542203392*^9, 3.393850954709137*^9, 
  3.393851027468011*^9, 3.3938530908255177`*^9}]
}, Open  ]],

Cell["\<\
It defines a direction in space. The state which is \"spin-up\" with respect \
to this direction is\
\>", "Text"],

Cell[CellGroupData[{

Cell["sp = QSpinorUp[vec]", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox["2", "3"], ",", 
   RowBox[{
    RowBox[{"-", 
     FractionBox["1", "3"]}], "+", 
    FractionBox[
     RowBox[{"2", " ", "\[ImaginaryI]"}], "3"]}]}], "}"}]], "Output",
 CellChangeTimes->{3.390202185147915*^9, 3.39178138432628*^9, 
  3.3918507000625*^9, 3.391850841421875*^9, 3.3924726123548822`*^9, 
  3.392472686285006*^9, 3.39385054226952*^9, 3.393850954968391*^9, 
  3.393851027551051*^9, 3.393853090938827*^9}]
}, Open  ]],

Cell["We can convert this spinor back to the vector", "Text"],

Cell[CellGroupData[{

Cell["QSpinorToVector[sp]", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox["4", "9"]}], ",", 
   FractionBox["8", "9"], ",", 
   RowBox[{"-", 
    FractionBox["1", "9"]}]}], "}"}]], "Output",
 CellChangeTimes->{3.390202186189413*^9, 3.391781384379191*^9, 
  3.39185070009375*^9, 3.391850841453125*^9, 3.392472612368146*^9, 
  3.3924726863036222`*^9, 3.3938505423364363`*^9, 3.39385095503199*^9, 
  3.393851027652367*^9, 3.393853091058672*^9}]
}, Open  ]],

Cell[TextData[{
 "The norm (length) of the three-dimensional real vector ",
 StyleBox["QSpinorToVector[", "InlineInput"],
 "spinor",
 StyleBox["]", "InlineInput"],
 " is equal to  ",
 StyleBox["Norm[", "InlineInput"],
 "spinor",
 StyleBox["]^2", "InlineInput"],
 ", that is"
}], "Text"],

Cell[CellGroupData[{

Cell["Norm[QSpinorToVector[{3,2}]] == Norm[{3,2}]^2", "Input"],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.3902021872309103`*^9, 3.391781384423267*^9, 
  3.391850700140625*^9, 3.391850841484375*^9, 3.392472612380993*^9, 
  3.392472686353546*^9, 3.393850542403288*^9, 3.3938509550985727`*^9, 
  3.393851027853104*^9, 3.393853091772949*^9}]
}, Open  ]],

Cell[TextData[{
 "The command ",
 StyleBox["QSpinorToVector", "InlineInput"],
 " accepts the option ",
 StyleBox["QVectorLength", "InlineInput"]
}], "Text"],

Cell[TextData[{
 StyleBox["QSpinorToVector[psi, QVectorLength->", "InlineInput"],
 "n",
 StyleBox["]", "InlineInput"],
 " with",
 StyleBox["\n", "Input"],
 "n=1\t\t\tgenerates a unit vector\nn=2\t\t\tgenerates a vector with norm \
Norm[psi]^2 (default)\nn=3\t\t\tgenerates a vector with norm Norm[psi]"
}], "Text"],

Cell["\<\
There is also an inverse command QVectorToSpinor that accepts the same \
option.\
\>", "Text"],

Cell[TextData[{
 StyleBox["QVectorToSpinor[vec, QVectorLength->", "InlineInput"],
 "n",
 StyleBox["]", "InlineInput"],
 " with the option",
 StyleBox["\n", "Input"],
 "n=1\t\t\tgenerates a unit spinor\nn=2\t\t\tgenerates a vector with norm \
Norm[vec]^1/2 (default)\nn=3\t\t\tgenerates a vector with norm Norm[vec]"
}], "Text"],

Cell[TextData[{
 "We have ",
 StyleBox["QVectorToSpinor[", "InlineInput"],
 "vec",
 StyleBox[", QVectorLength->1] == QSpinorUp[", "InlineInput"],
 "vec",
 StyleBox["]", "InlineInput"],
 "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["QExtractPhase", "Section"],

Cell["\<\
Note that the relation between spinors and vectors is not one-to-one. Here is \
another normalized spinor that gives the same vector than the spinor sp \
above:\
\>", "Text"],

Cell[CellGroupData[{

Cell["sp2 =  {2/3,2 I/3-1/3}", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox["2", "3"], ",", 
   RowBox[{
    RowBox[{"-", 
     FractionBox["1", "3"]}], "+", 
    FractionBox[
     RowBox[{"2", " ", "\[ImaginaryI]"}], "3"]}]}], "}"}]], "Output",
 CellChangeTimes->{3.39020219433112*^9, 3.391781384437584*^9, 
  3.39185070025*^9, 3.39185084153125*^9, 3.392472612394058*^9, 
  3.392472686379294*^9, 3.3938505424688587`*^9, 3.393850959092909*^9, 
  3.393851027928269*^9, 3.393853091895543*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["QSpinorToVector[sp2]", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox["4", "9"]}], ",", 
   FractionBox["8", "9"], ",", 
   RowBox[{"-", 
    FractionBox["1", "9"]}]}], "}"}]], "Output",
 CellChangeTimes->{3.3902021951222577`*^9, 3.39178138453826*^9, 
  3.391850700296875*^9, 3.3918508415625*^9, 3.392472612425502*^9, 
  3.392472686442*^9, 3.393850542623679*^9, 3.393850959303911*^9, 
  3.393851027999995*^9, 3.393853092006473*^9}]
}, Open  ]],

Cell["\<\
Generally, two spinors differing by a phase, sp1=Exp[I phi] sp2, lead to the \
same vector. \
\>", "Text"],

Cell[TextData[{
 StyleBox["QExtractPhase[", "InlineInput"],
 "psi",
 StyleBox["]", "InlineInput"],
 " returns a real number for any nonzero psi={z1,z2}. This number is a phase \
determined from comparison with ",
 StyleBox["QSpinorUp", "InlineInput"],
 ", where ",
 StyleBox["QSpinorUp", "InlineInput"],
 " is taken with respect to the spin-up direction defined by psi. Let us \
illustrate this with an example:"
}], "Text"],

Cell[CellGroupData[{

Cell["psi = QNormalize[{-2 I,1}]", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"2", " ", "\[ImaginaryI]"}], 
     SqrtBox["5"]]}], ",", 
   FractionBox["1", 
    SqrtBox["5"]]}], "}"}]], "Output",
 CellChangeTimes->{3.3902021979563327`*^9, 3.391781384553912*^9, 
  3.39185070034375*^9, 3.391850841765625*^9, 3.392472612438827*^9, 
  3.392472686456159*^9, 3.393850542685882*^9, 3.393850959365612*^9, 
  3.393851028066874*^9, 3.393853092122488*^9}]
}, Open  ]],

Cell["This spinor has the spin-up direction (unit vector)", "Text"],

Cell[CellGroupData[{

Cell["dir = QSpinorToVector[psi]", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", 
   FractionBox["4", "5"], ",", 
   FractionBox["3", "5"]}], "}"}]], "Output",
 CellChangeTimes->{3.3902021995285935`*^9, 3.391781384591078*^9, 
  3.391850700390625*^9, 3.3918508418125*^9, 3.3924726124692163`*^9, 
  3.392472686499579*^9, 3.393850542884467*^9, 3.393850959431983*^9, 
  3.3938510281333303`*^9, 3.393853092513988*^9}]
}, Open  ]],

Cell[TextData[{
 "Note, however, that ",
 StyleBox["QSpinorUp[", "InlineInput"],
 "dir",
 StyleBox["]", "InlineInput"],
 " is different from ",
 StyleBox["psi", "InlineInput"]
}], "Text"],

Cell[CellGroupData[{

Cell["QSpinorUp[dir]", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox["2", 
    SqrtBox["5"]], ",", 
   FractionBox["\[ImaginaryI]", 
    SqrtBox["5"]]}], "}"}]], "Output",
 CellChangeTimes->{3.390202200670235*^9, 3.391781384605682*^9, 
  3.3918507004375*^9, 3.39185084184375*^9, 3.3924726124847517`*^9, 
  3.3924726865149927`*^9, 3.393850543017782*^9, 3.393850959497691*^9, 
  3.393851028199459*^9, 3.393853092629156*^9}]
}, Open  ]],

Cell[TextData[{
 "but the difference is only a phase factor. The phase can be obtained with \
",
 StyleBox["QExtractPhase", "InlineInput"],
 ":"
}], "Text"],

Cell[CellGroupData[{

Cell["ph = QExtractPhase[psi]", "Input"],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox["\[Pi]", "2"]}]], "Output",
 CellChangeTimes->{3.3902022017217474`*^9, 3.391781384619257*^9, 
  3.39185070046875*^9, 3.391850841890625*^9, 3.392472612497994*^9, 
  3.392472686528883*^9, 3.39385054321813*^9, 3.3938509595654287`*^9, 
  3.393851028266161*^9, 3.393853092747862*^9}]
}, Open  ]],

Cell[TextData[{
 "psi can now be recovered by multiplying ",
 StyleBox["QSpinorUp[", "InlineInput"],
 "dir",
 StyleBox["]", "InlineInput"],
 " with the appropriate phase factor"
}], "Text"],

Cell[CellGroupData[{

Cell["Exp[I ph] QSpinorUp[dir]", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"2", " ", "\[ImaginaryI]"}], 
     SqrtBox["5"]]}], ",", 
   FractionBox["1", 
    SqrtBox["5"]]}], "}"}]], "Output",
 CellChangeTimes->{3.3902022027332015`*^9, 3.391781384632826*^9, 
  3.3918507005*^9, 3.391850841921875*^9, 3.3924726125108643`*^9, 
  3.39247268654283*^9, 3.393850543350765*^9, 3.393850959650172*^9, 
  3.3938510283334427`*^9, 3.3938530928582487`*^9}]
}, Open  ]],

Cell["\<\
Any spinor psi is thus uniquely determined by a three-dimensional vector \
together with phase factor.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Rotations", "Section"],

Cell[TextData[{
 StyleBox["QRotationSO3[", "InlineInput"],
 "vector",
 StyleBox["]", "InlineInput"],
 " and ",
 StyleBox["QRotationSU2[", "InlineInput"],
 "vector",
 StyleBox["]", "InlineInput"],
 " define rotations around the axis given by vector through an angle given by \
",
 StyleBox["Norm[", "InlineInput"],
 "vector",
 StyleBox["]", "InlineInput"],
 ". The matrix ",
 StyleBox["QRotationSO3[", "InlineInput"],
 "vector",
 StyleBox["]", "InlineInput"],
 " is an orthogonal 3x3-matrix. ",
 StyleBox["QRotationSU2[", "InlineInput"],
 "vector",
 StyleBox["]", "InlineInput"],
 " is a unitary 2x2 matrix realizing the 'spinor representation' of the \
rotation group."
}], "Text"],

Cell["For example,", "Text"],

Cell[CellGroupData[{

Cell["QRotationSU2[{2 Pi,0,0}]//MatrixForm", "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"-", "1"}], "0"},
     {"0", 
      RowBox[{"-", "1"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.3902022072396812`*^9, 3.39178138467202*^9, 
  3.39185070059375*^9, 3.39185084196875*^9, 3.3924726125249777`*^9, 
  3.392472686590209*^9, 3.393850543418563*^9, 3.393850967193975*^9, 
  3.393851028400217*^9, 3.393853093159934*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["QRotationSU2[{1,1,1}]//MatrixForm", "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       RowBox[{"Cos", "[", 
        FractionBox[
         SqrtBox["3"], "2"], "]"}], "-", 
       FractionBox[
        RowBox[{"\[ImaginaryI]", " ", 
         RowBox[{"Sin", "[", 
          FractionBox[
           SqrtBox["3"], "2"], "]"}]}], 
        SqrtBox["3"]]}], 
      RowBox[{"-", 
       FractionBox[
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", "+", "\[ImaginaryI]"}], ")"}], " ", 
         RowBox[{"Sin", "[", 
          FractionBox[
           SqrtBox["3"], "2"], "]"}]}], 
        SqrtBox["3"]]}]},
     {
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "\[ImaginaryI]"}], ")"}], " ", 
        RowBox[{"Sin", "[", 
         FractionBox[
          SqrtBox["3"], "2"], "]"}]}], 
       SqrtBox["3"]], 
      RowBox[{
       RowBox[{"Cos", "[", 
        FractionBox[
         SqrtBox["3"], "2"], "]"}], "+", 
       FractionBox[
        RowBox[{"\[ImaginaryI]", " ", 
         RowBox[{"Sin", "[", 
          FractionBox[
           SqrtBox["3"], "2"], "]"}]}], 
        SqrtBox["3"]]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.3902022085615826`*^9, 3.391781384709365*^9, 
  3.39185070071875*^9, 3.39185084203125*^9, 3.3924726125804977`*^9, 
  3.392472686632729*^9, 3.39385054348663*^9, 3.393850967443213*^9, 
  3.393851028467526*^9, 3.39385309345583*^9}]
}, Open  ]],

Cell["\<\
The textbook-form of the SU(2)-rotation matrix can be obtained as follows. \
Let us first define a unit vector with symbolic components.\
\>", "Text"],

Cell["\<\
Clear[nv,n1,n2,n3];
nv = {n1,n2,n3}; (* should be a unit vector *)\
\>", "Input"],

Cell[CellGroupData[{

Cell["QRotationSU2[phi nv] // Simplify[#,{phi>0,nv.nv==1}]&", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Cos", "[", 
       FractionBox["phi", "2"], "]"}], "-", 
      RowBox[{"\[ImaginaryI]", " ", "n3", " ", 
       RowBox[{"Sin", "[", 
        FractionBox["phi", "2"], "]"}]}]}], ",", 
     RowBox[{
      RowBox[{"-", "\[ImaginaryI]"}], " ", 
      RowBox[{"(", 
       RowBox[{"n1", "-", 
        RowBox[{"\[ImaginaryI]", " ", "n2"}]}], ")"}], " ", 
      RowBox[{"Sin", "[", 
       FractionBox["phi", "2"], "]"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "\[ImaginaryI]"}], " ", "n1"}], "+", "n2"}], ")"}], " ",
       
      RowBox[{"Sin", "[", 
       FractionBox["phi", "2"], "]"}]}], ",", 
     RowBox[{
      RowBox[{"Cos", "[", 
       FractionBox["phi", "2"], "]"}], "+", 
      RowBox[{"\[ImaginaryI]", " ", "n3", " ", 
       RowBox[{"Sin", "[", 
        FractionBox["phi", "2"], "]"}]}]}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.3902022117962337`*^9, 3.391781385277275*^9, 
  3.3918507019375*^9, 3.39185084228125*^9, 3.3924726131677647`*^9, 
  3.392472687268579*^9, 3.393850544518289*^9, 3.3938509682435427`*^9, 
  3.393851029249473*^9, 3.393853094254718*^9}]
}, Open  ]],

Cell["This matrix is the same as the following:", "Text"],

Cell[CellGroupData[{

Cell["\<\
MatrixExp[-I phi/2 QVectorToHermitianMatrix[nv]] // ExpToTrig // \
Simplify[#,{phi>0,nv.nv==1}]&\
\>", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Cos", "[", 
       FractionBox["phi", "2"], "]"}], "-", 
      RowBox[{"\[ImaginaryI]", " ", "n3", " ", 
       RowBox[{"Sin", "[", 
        FractionBox["phi", "2"], "]"}]}]}], ",", 
     RowBox[{
      RowBox[{"-", "\[ImaginaryI]"}], " ", 
      RowBox[{"(", 
       RowBox[{"n1", "-", 
        RowBox[{"\[ImaginaryI]", " ", "n2"}]}], ")"}], " ", 
      RowBox[{"Sin", "[", 
       FractionBox["phi", "2"], "]"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "\[ImaginaryI]"}], " ", "n1"}], "+", "n2"}], ")"}], " ",
       
      RowBox[{"Sin", "[", 
       FractionBox["phi", "2"], "]"}]}], ",", 
     RowBox[{
      RowBox[{"Cos", "[", 
       FractionBox["phi", "2"], "]"}], "+", 
      RowBox[{"\[ImaginaryI]", " ", "n3", " ", 
       RowBox[{"Sin", "[", 
        FractionBox["phi", "2"], "]"}]}]}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.390202212387083*^9, 3.39178138575559*^9, 
  3.391850702734375*^9, 3.39185084240625*^9, 3.392472613674445*^9, 
  3.39247268795497*^9, 3.393850544984315*^9, 3.3938509687151937`*^9, 
  3.3938510298165197`*^9, 3.3938530947626553`*^9}]
}, Open  ]],

Cell["\<\
Any unitary 2x2-matrix may be considered a SU(2) rotation. The following \
example shows the connection with a basis transformation\
\>", "Text"],

Cell[CellGroupData[{

Cell["\<\
Clear[a,b];
T=QRotationSU2[{1,1,1}];
newBasis={T.QzBasis[[1]], T.QzBasis[[2]]};
QSpinorToComponents[{a,b},QUseBasis->newBasis] == Inverse[T].{a,b} // \
Simplify\
\>", "Input"],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.3902022135487537`*^9, 3.391781385980476*^9, 
  3.391850703390625*^9, 3.391850842453125*^9, 3.392472613872738*^9, 
  3.392472688254717*^9, 3.39385054520159*^9, 3.393850968931346*^9, 
  3.393851030014632*^9, 3.3938530949600697`*^9}]
}, Open  ]],

Cell["\<\
The connection between rotations in SO(3) and rotations in SU(2) is the \
following:\
\>", "Text"],

Cell["\<\
rotV = {1, 0.5, -1};
R = QRotationSO3[rotV];
T = QRotationSU2[rotV];\
\>", "Input"],

Cell[CellGroupData[{

Cell["\<\
QVectorToSpinor[ R . {1,0,0} ] == T . QVectorToSpinor[{1,0,0}]\
\>", "Input"],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.390202216372814*^9, 3.391781386036551*^9, 
  3.391850704359375*^9, 3.3918508425*^9, 3.3924726141802673`*^9, 
  3.392472688544128*^9, 3.3938505453183203`*^9, 3.3938509691811733`*^9, 
  3.393851030215005*^9, 3.3938530951121893`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
QVectorToHermitianMatrix[ R . {1,0,0} ] == Chop[T . \
QVectorToHermitianMatrix[{1,0,0}] . Inverse[T]]\
\>", "Input"],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.3902022173241825`*^9, 3.391781386151795*^9, 
  3.391850705046875*^9, 3.391850842546875*^9, 3.3924726147993813`*^9, 
  3.392472688612095*^9, 3.393850545409936*^9, 3.393850969330379*^9, 
  3.3938510302830563`*^9, 3.3938530951793127`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["QVectorToArrow"], "Input"],

Cell[BoxData["QVectorToArrow"], "Output",
 CellChangeTimes->{3.393853095244648*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QVectorToArrow", "//", "Options"}]], "Input",
 CellChangeTimes->{{3.39385182443626*^9, 3.393851825589398*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"QArrowHead", "\[Rule]", "True"}], ",", 
   RowBox[{"QArrowShaft", "\[Rule]", "False"}], ",", 
   RowBox[{"QLinePointSize", "\[Rule]", "0.001`"}], ",", 
   RowBox[{"QMinLength", "\[Rule]", "0.1`"}], ",", 
   RowBox[{"QArrowShape", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"QHeadColor", "\[Rule]", 
    RowBox[{"GrayLevel", "[", "0.5`", "]"}]}], ",", 
   RowBox[{"QArrowScale", "\[Rule]", "1"}], ",", 
   RowBox[{"QShaftColor", "\[Rule]", 
    RowBox[{"GrayLevel", "[", "0", "]"}]}], ",", 
   RowBox[{"QNeedleStyle", "\[Rule]", "False"}]}], "}"}]], "Output",
 CellChangeTimes->{3.3938518259501963`*^9, 3.393853095296537*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Visualization", "Section"],

Cell[CellGroupData[{

Cell[BoxData["coloroptt"], "Input",
 CellChangeTimes->{3.393852019823992*^9}],

Cell[BoxData["coloroptt"], "Output",
 CellChangeTimes->{3.39385202028813*^9, 3.393853095563716*^9}]
}, Open  ]],

Cell["\<\
Since spinors and Hermitian matrices can be converted into vectors, we can \
visualize them\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"??", "Glow"}]], "Input",
 CellChangeTimes->{{3.393853833264017*^9, 3.393853834626066*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  StyleBox["\<\"\\!\\(\\*RowBox[{\\\"Glow\\\", \\\"[\\\", \
StyleBox[\\\"col\\\", \\\"TI\\\"], \\\"]\\\"}]\\) is a graphics directive \
which specifies that surfaces of 3D graphics objects which follow are to be \
taken to glow with color \\!\\(\\*StyleBox[\\\"col\\\", \\\"TI\\\"]\\). \
\\n\\!\\(\\*RowBox[{\\\"Glow\\\", \\\"[\\\", \\\"]\\\"}]\\) specifies that \
there is no glow. \"\>", "MSG"], " ", 
  ButtonBox[
   StyleBox["\[RightSkeleton]", "SR"],
   Active->True,
   BaseStyle->"Link",
   ButtonData->"paclet:ref/Glow"]}]], "Print", "PrintUsage",
 CellChangeTimes->{3.39385383546307*^9},
 CellTags->"Info3393861035-8787931"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {
     RowBox[{
      RowBox[{"Attributes", "[", "Glow", "]"}], "=", 
      RowBox[{"{", 
       RowBox[{"Protected", ",", "ReadProtected"}], "}"}]}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition["Glow"],
  Editable->False]], "Print",
 CellChangeTimes->{3.393853835506009*^9},
 CellTags->"Info3393861035-8787931"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graphics3D", "[", 
  RowBox[{"QSpinorToArrow", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "I"}], ",", 
     RowBox[{"1", "+", "I"}]}], "}"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.393853697256723*^9, 3.3938537409777193`*^9}, {
  3.3938538159442387`*^9, 3.393853824354298*^9}, {3.393853859166441*^9, 
  3.393853946218874*^9}, {3.393854115308565*^9, 3.393854127360586*^9}}],

Cell[BoxData[
 Graphics3DBox[{
   {Thickness[0.001], Line3DBox[{{0, 0, 0}, {-2, 2, -1}}]}, 
   {Hue[0.75, 1., 1.], 
    Polygon3DBox[
     NCache[{{-2, 2, -1}, {
       Rational[-3, 2] + Rational[3, 8] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[3, 2] + Rational[3, 8] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-3, 4]}, {
       Rational[-3, 2] + 
        Rational[1, 16] Rational[3, 2]^Rational[1, 2]/GoldenRatio + 
        Rational[3, 16] 2^Rational[-1, 2]/GoldenRatio, Rational[3, 2] + 
        Rational[-1, 16] Rational[3, 2]^Rational[1, 2]/GoldenRatio + 
        Rational[3, 16] 2^Rational[-1, 2]/GoldenRatio, Rational[-3, 4] + 
        Rational[-1, 4] Rational[3, 2]^Rational[1, 2]/GoldenRatio}}, {{-2, 
       2, -1}, {-1.3361189908316922`, 
       1.6638810091683078`, -0.75}, {-1.370751123036651, 
       1.534632132204959, -0.9392334895167802}}]], 
    Polygon3DBox[
     NCache[{{-2, 2, -1}, {
       Rational[-3, 2] + 
        Rational[1, 16] Rational[3, 2]^Rational[1, 2]/GoldenRatio + 
        Rational[3, 16] 2^Rational[-1, 2]/GoldenRatio, Rational[3, 2] + 
        Rational[-1, 16] Rational[3, 2]^Rational[1, 2]/GoldenRatio + 
        Rational[3, 16] 2^Rational[-1, 2]/GoldenRatio, Rational[-3, 4] + 
        Rational[-1, 4] Rational[3, 2]^Rational[1, 2]/GoldenRatio}, {
       Rational[-3, 2] + 
        Rational[1, 16] Rational[3, 2]^Rational[1, 2]/GoldenRatio + 
        Rational[-3, 16] 2^Rational[-1, 2]/GoldenRatio, Rational[3, 2] + 
        Rational[-1, 16] Rational[3, 2]^Rational[1, 2]/GoldenRatio + 
        Rational[-3, 16] 2^Rational[-1, 2]/GoldenRatio, Rational[-3, 4] + 
        Rational[-1, 4] Rational[3, 2]^Rational[1, 2]/GoldenRatio}}, {{-2, 
       2, -1}, {-1.370751123036651, 
       1.534632132204959, -0.9392334895167802}, {-1.534632132204959, 
       1.370751123036651, -0.9392334895167802}}]], 
    Polygon3DBox[
     NCache[{{-2, 2, -1}, {
       Rational[-3, 2] + 
        Rational[1, 16] Rational[3, 2]^Rational[1, 2]/GoldenRatio + 
        Rational[-3, 16] 2^Rational[-1, 2]/GoldenRatio, Rational[3, 2] + 
        Rational[-1, 16] Rational[3, 2]^Rational[1, 2]/GoldenRatio + 
        Rational[-3, 16] 2^Rational[-1, 2]/GoldenRatio, Rational[-3, 4] + 
        Rational[-1, 4] Rational[3, 2]^Rational[1, 2]/GoldenRatio}, {
       Rational[-3, 2] + Rational[-3, 8] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[3, 2] + Rational[-3, 8] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-3, 4]}}, {{-2, 2, -1}, {-1.534632132204959, 
       1.370751123036651, -0.9392334895167802}, {-1.6638810091683078`, 
       1.3361189908316922`, -0.75}}]], 
    Polygon3DBox[
     NCache[{{-2, 2, -1}, {
       Rational[-3, 2] + Rational[-3, 8] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[3, 2] + Rational[-3, 8] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-3, 4]}, {
       Rational[-3, 2] + 
        Rational[-1, 16] Rational[3, 2]^Rational[1, 2]/GoldenRatio + 
        Rational[-3, 16] 2^Rational[-1, 2]/GoldenRatio, Rational[3, 2] + 
        Rational[1, 16] Rational[3, 2]^Rational[1, 2]/GoldenRatio + 
        Rational[-3, 16] 2^Rational[-1, 2]/GoldenRatio, Rational[-3, 4] + 
        Rational[1, 4] Rational[3, 2]^Rational[1, 2]/GoldenRatio}}, {{-2, 
       2, -1}, {-1.6638810091683078`, 
       1.3361189908316922`, -0.75}, {-1.629248876963349, 
       1.465367867795041, -0.5607665104832198}}]], 
    Polygon3DBox[
     NCache[{{-2, 2, -1}, {
       Rational[-3, 2] + 
        Rational[-1, 16] Rational[3, 2]^Rational[1, 2]/GoldenRatio + 
        Rational[-3, 16] 2^Rational[-1, 2]/GoldenRatio, Rational[3, 2] + 
        Rational[1, 16] Rational[3, 2]^Rational[1, 2]/GoldenRatio + 
        Rational[-3, 16] 2^Rational[-1, 2]/GoldenRatio, Rational[-3, 4] + 
        Rational[1, 4] Rational[3, 2]^Rational[1, 2]/GoldenRatio}, {
       Rational[-3, 2] + 
        Rational[-1, 16] Rational[3, 2]^Rational[1, 2]/GoldenRatio + 
        Rational[3, 16] 2^Rational[-1, 2]/GoldenRatio, Rational[3, 2] + 
        Rational[1, 16] Rational[3, 2]^Rational[1, 2]/GoldenRatio + 
        Rational[3, 16] 2^Rational[-1, 2]/GoldenRatio, Rational[-3, 4] + 
        Rational[1, 4] Rational[3, 2]^Rational[1, 2]/GoldenRatio}}, {{-2, 
       2, -1}, {-1.629248876963349, 
       1.465367867795041, -0.5607665104832198}, {-1.465367867795041, 
       1.629248876963349, -0.5607665104832198}}]], 
    Polygon3DBox[
     NCache[{{-2, 2, -1}, {
       Rational[-3, 2] + 
        Rational[-1, 16] Rational[3, 2]^Rational[1, 2]/GoldenRatio + 
        Rational[3, 16] 2^Rational[-1, 2]/GoldenRatio, Rational[3, 2] + 
        Rational[1, 16] Rational[3, 2]^Rational[1, 2]/GoldenRatio + 
        Rational[3, 16] 2^Rational[-1, 2]/GoldenRatio, Rational[-3, 4] + 
        Rational[1, 4] Rational[3, 2]^Rational[1, 2]/GoldenRatio}, {
       Rational[-3, 2] + Rational[3, 8] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[3, 2] + Rational[3, 8] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-3, 4]}}, {{-2, 2, -1}, {-1.465367867795041, 
       1.629248876963349, -0.5607665104832198}, {-1.3361189908316922`, 
       1.6638810091683078`, -0.75}}]]}},
  ImageSize->{360., 309.5895424770095},
  ViewPoint->{-3.2371995282253057`, 0.4220051088299329, 0.8901971144523532},
  ViewVertical->{-0.3319673470642641, 0.4807301302023575, 
   1.6232020482968084`}]], "Output",
 CellChangeTimes->{{3.393853697455085*^9, 3.393853727260672*^9}, {
   3.3938538107555*^9, 3.393853817060092*^9}, {3.393853860734929*^9, 
   3.39385394795044*^9}, {3.3938541156303673`*^9, 3.393854127720768*^9}, 
   3.393854602014285*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graphics3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"QSpinorToArrow", "[", 
     RowBox[{"{", 
      RowBox[{"\[ImaginaryI]", ",", "1"}], "}"}], "]"}], "/.", 
    "\[InvisibleSpace]", 
    RowBox[{
     RowBox[{"Hue", "[", "a__", "]"}], "\[RuleDelayed]", 
     RowBox[{"Glow", "[", 
      RowBox[{"Hue", "[", "a", "]"}], "]"}]}]}], ",", 
   RowBox[{"Lighting", "\[Rule]", "Automatic"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.393851684618824*^9, 3.39385169074304*^9}, {
   3.393851869765396*^9, 3.393851872207851*^9}, 3.393853479045322*^9, {
   3.393853560112652*^9, 3.393853561303982*^9}, {3.393853593638112*^9, 
   3.393853676858457*^9}}],

Cell[BoxData[
 Graphics3DBox[{
   {Thickness[0.001], Line3DBox[{{0, 0, 0}, {0, -2, 0}}]}, 
   {Glow[
     Hue[0.25, 1., 1.]], 
    Polygon3DBox[
     NCache[{{0, -2, 0}, {Rational[-1, 4]/GoldenRatio, Rational[-3, 2], 0}, {
       Rational[-1, 8]/GoldenRatio, Rational[-3, 2], Rational[-1, 8] 
        3^Rational[1, 2]/GoldenRatio}}, {{0, -2, 
       0}, {-0.1545084971874737, -1.5, 
        0}, {-0.07725424859373685, -1.5, -0.1338082836649087}}]], 
    Polygon3DBox[
     NCache[{{0, -2, 0}, {
       Rational[-1, 8]/GoldenRatio, Rational[-3, 2], Rational[-1, 8] 
        3^Rational[1, 2]/GoldenRatio}, {
       Rational[1, 8]/GoldenRatio, Rational[-3, 2], Rational[-1, 8] 
        3^Rational[1, 2]/GoldenRatio}}, {{0, -2, 
       0}, {-0.07725424859373685, -1.5, -0.1338082836649087}, {
       0.07725424859373685, -1.5, -0.1338082836649087}}]], 
    Polygon3DBox[
     NCache[{{0, -2, 0}, {
       Rational[1, 8]/GoldenRatio, Rational[-3, 2], Rational[-1, 8] 
        3^Rational[1, 2]/GoldenRatio}, {
       Rational[1, 4]/GoldenRatio, Rational[-3, 2], 0}}, {{0, -2, 0}, {
       0.07725424859373685, -1.5, -0.1338082836649087}, {
       0.1545084971874737, -1.5, 0}}]], 
    Polygon3DBox[
     NCache[{{0, -2, 0}, {Rational[1, 4]/GoldenRatio, Rational[-3, 2], 0}, {
       Rational[1, 8]/GoldenRatio, Rational[-3, 2], Rational[1, 8] 
        3^Rational[1, 2]/GoldenRatio}}, {{0, -2, 0}, {
       0.1545084971874737, -1.5, 0}, {0.07725424859373685, -1.5, 
       0.1338082836649087}}]], 
    Polygon3DBox[
     NCache[{{0, -2, 0}, {
       Rational[1, 8]/GoldenRatio, Rational[-3, 2], Rational[1, 8] 
        3^Rational[1, 2]/GoldenRatio}, {
       Rational[-1, 8]/GoldenRatio, Rational[-3, 2], Rational[1, 8] 
        3^Rational[1, 2]/GoldenRatio}}, {{0, -2, 0}, {
       0.07725424859373685, -1.5, 
       0.1338082836649087}, {-0.07725424859373685, -1.5, 
       0.1338082836649087}}]], 
    Polygon3DBox[
     NCache[{{0, -2, 0}, {
       Rational[-1, 8]/GoldenRatio, Rational[-3, 2], Rational[1, 8] 
        3^Rational[1, 2]/GoldenRatio}, {
       Rational[-1, 4]/GoldenRatio, Rational[-3, 2], 0}}, {{0, -2, 
       0}, {-0.07725424859373685, -1.5, 
       0.1338082836649087}, {-0.1545084971874737, -1.5, 0}}]]}},
  Lighting->Automatic]], "Output",
 CellChangeTimes->{
  3.393853571559692*^9, 3.393853603146605*^9, {3.393853665145288*^9, 
   3.393853677376624*^9}, 3.393854604696863*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", " ", "[", "Graphics3D", "]"}]], "Input",
 CellChangeTimes->{{3.39385357302444*^9, 3.393853578361883*^9}, {
  3.393853629938384*^9, 3.393853630392205*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"AlignmentPoint", "\[Rule]", "Center"}], ",", 
   RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"Axes", "\[Rule]", "False"}], ",", 
   RowBox[{"AxesEdge", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"AxesLabel", "\[Rule]", "None"}], ",", 
   RowBox[{"AxesStyle", "\[Rule]", 
    RowBox[{"{", "}"}]}], ",", 
   RowBox[{"Background", "\[Rule]", "None"}], ",", 
   RowBox[{"BaselinePosition", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"BaseStyle", "\[Rule]", 
    RowBox[{"{", "}"}]}], ",", 
   RowBox[{"Boxed", "\[Rule]", "True"}], ",", 
   RowBox[{"BoxRatios", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"BoxStyle", "\[Rule]", 
    RowBox[{"{", "}"}]}], ",", 
   RowBox[{"ColorOutput", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"ContentSelectable", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"ControllerLinking", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"ControllerMethod", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"ControllerPath", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"DisplayFunction", "\[RuleDelayed]", "$DisplayFunction"}], ",", 
   RowBox[{"Epilog", "\[Rule]", 
    RowBox[{"{", "}"}]}], ",", 
   RowBox[{"FaceGrids", "\[Rule]", "None"}], ",", 
   RowBox[{"FaceGridsStyle", "\[Rule]", 
    RowBox[{"{", "}"}]}], ",", 
   RowBox[{"FormatType", "\[RuleDelayed]", "TraditionalForm"}], ",", 
   RowBox[{"ImageMargins", "\[Rule]", "0.`"}], ",", 
   RowBox[{"ImagePadding", "\[Rule]", "All"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"LabelStyle", "\[Rule]", 
    RowBox[{"{", "}"}]}], ",", 
   RowBox[{"Lighting", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"Method", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"PlotLabel", "\[Rule]", "None"}], ",", 
   RowBox[{"PlotRange", "\[Rule]", "All"}], ",", 
   RowBox[{"PlotRangePadding", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"PlotRegion", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"PreserveImageOptions", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"Prolog", "\[Rule]", 
    RowBox[{"{", "}"}]}], ",", 
   RowBox[{"RotationAction", "\[Rule]", "\<\"Fit\"\>"}], ",", 
   RowBox[{"SphericalRegion", "\[Rule]", "False"}], ",", 
   RowBox[{"Ticks", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"TicksStyle", "\[Rule]", 
    RowBox[{"{", "}"}]}], ",", 
   RowBox[{"ViewAngle", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"ViewCenter", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      FractionBox["1", "2"], ",", 
      FractionBox["1", "2"], ",", 
      FractionBox["1", "2"]}], "}"}]}], ",", 
   RowBox[{"ViewMatrix", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"ViewPoint", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"1.3`", ",", 
      RowBox[{"-", "2.4`"}], ",", "2.`"}], "}"}]}], ",", 
   RowBox[{"ViewRange", "\[Rule]", "All"}], ",", 
   RowBox[{"ViewVector", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"ViewVertical", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0", ",", "1"}], "}"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.393853578790765*^9, 3.393853630791663*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"QSpinorToArrow", "[", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1"}], "}"}], "]"}], "//", "Graphics3D"}]], "Input",
 CellChangeTimes->{{3.3902027403462515`*^9, 3.3902027456638975`*^9}, 
   3.390210249675383*^9, {3.390225998471664*^9, 3.390226005880858*^9}, {
   3.3902265452964354`*^9, 3.390226574105443*^9}, {3.390226807927422*^9, 
   3.390226809804764*^9}, {3.390227203837964*^9, 3.390227205203659*^9}, 
   3.39247263166999*^9, {3.392472670590097*^9, 3.3924726723890123`*^9}, {
   3.393850753177603*^9, 3.393850780710935*^9}, {3.393851049456848*^9, 
   3.3938510536328297`*^9}}],

Cell[BoxData[
 Graphics3DBox[{
   {Thickness[0.001], Line3DBox[{{0, 0, 0}, {2, 0, 0}}]}, 
   {Hue[0., 1., 1.], 
    Polygon3DBox[
     NCache[{{2, 0, 0}, {Rational[3, 2], Rational[-1, 4]/GoldenRatio, 0}, {
       Rational[3, 2], Rational[-1, 8]/GoldenRatio, Rational[-1, 8] 
        3^Rational[1, 2]/GoldenRatio}}, {{2, 0, 0}, {
       1.5, -0.1545084971874737, 0}, {
       1.5, -0.07725424859373685, -0.1338082836649087}}]], 
    Polygon3DBox[
     NCache[{{2, 0, 0}, {
       Rational[3, 2], Rational[-1, 8]/GoldenRatio, Rational[-1, 8] 
        3^Rational[1, 2]/GoldenRatio}, {
       Rational[3, 2], Rational[1, 8]/GoldenRatio, Rational[-1, 8] 
        3^Rational[1, 2]/GoldenRatio}}, {{2, 0, 0}, {
       1.5, -0.07725424859373685, -0.1338082836649087}, {1.5, 
       0.07725424859373685, -0.1338082836649087}}]], 
    Polygon3DBox[
     NCache[{{2, 0, 0}, {
       Rational[3, 2], Rational[1, 8]/GoldenRatio, Rational[-1, 8] 
        3^Rational[1, 2]/GoldenRatio}, {
       Rational[3, 2], Rational[1, 4]/GoldenRatio, 0}}, {{2, 0, 0}, {1.5, 
       0.07725424859373685, -0.1338082836649087}, {
       1.5, 0.1545084971874737, 0}}]], 
    Polygon3DBox[
     NCache[{{2, 0, 0}, {Rational[3, 2], Rational[1, 4]/GoldenRatio, 0}, {
       Rational[3, 2], Rational[1, 8]/GoldenRatio, Rational[1, 8] 
        3^Rational[1, 2]/GoldenRatio}}, {{2, 0, 0}, {
       1.5, 0.1545084971874737, 0}, {1.5, 0.07725424859373685, 
       0.1338082836649087}}]], 
    Polygon3DBox[
     NCache[{{2, 0, 0}, {
       Rational[3, 2], Rational[1, 8]/GoldenRatio, Rational[1, 8] 
        3^Rational[1, 2]/GoldenRatio}, {
       Rational[3, 2], Rational[-1, 8]/GoldenRatio, Rational[1, 8] 
        3^Rational[1, 2]/GoldenRatio}}, {{2, 0, 0}, {1.5, 0.07725424859373685,
        0.1338082836649087}, {1.5, -0.07725424859373685, 
       0.1338082836649087}}]], 
    Polygon3DBox[
     NCache[{{2, 0, 0}, {
       Rational[3, 2], Rational[-1, 8]/GoldenRatio, Rational[1, 8] 
        3^Rational[1, 2]/GoldenRatio}, {
       Rational[3, 2], Rational[-1, 4]/GoldenRatio, 0}}, {{2, 0, 0}, {
       1.5, -0.07725424859373685, 0.1338082836649087}, {
       1.5, -0.1545084971874737, 0}}]]}}]], "Output",
 CellChangeTimes->{
  3.3902022752574863`*^9, 3.3902027462347183`*^9, 3.390222458072433*^9, 
   3.3902225038577137`*^9, 3.390223239892784*^9, 3.390223516250691*^9, 
   3.390223641119677*^9, 3.390225013066559*^9, 3.39022523094105*^9, 
   3.390225330129653*^9, 3.390225479366404*^9, 3.390225894918037*^9, {
   3.390225977841845*^9, 3.390226006508007*^9}, 3.390226142135723*^9, {
   3.3902265470923157`*^9, 3.39022657467129*^9}, {3.3902268108379173`*^9, 
   3.390226830111018*^9}, {3.390227198646706*^9, 3.390227206011322*^9}, 
   3.3902316398923883`*^9, 3.3902317299611588`*^9, 3.391781386238368*^9, 
   3.39185070571875*^9, 3.39185084265625*^9, 3.39185382965625*^9, {
   3.392472614996791*^9, 3.392472633454049*^9}, 3.392472688720983*^9, 
   3.3938505457212057`*^9, 3.39385075379838*^9, 3.393850977082827*^9, {
   3.3938510304649467`*^9, 3.3938510540871058`*^9}, 3.393851632448578*^9, 
   3.393853512655095*^9, 3.3938546110981627`*^9}]
}, Open  ]],

Cell[TextData[{
 "The package ",
 StyleBox["VQM`VisualizeVector", "InlineInput"],
 " (which is loaded automatically by ",
 StyleBox["VQM`Spinors", "InlineInput"],
 ") provides all necessary commands for visualizing vectors. These commands \
are also used by ",
 StyleBox["VQM`Spinors", "InlineInput"],
 " for the visualization of spinors and Hermitian matrices.. "
}], "Text"],

Cell[TextData[{
 "The  commands ",
 StyleBox["QVectorToArrow", "InlineInput"],
 " and ",
 StyleBox["QSpinorToArrow", "InlineInput"],
 " return a 3D-graphics primitive describing an arrow whose properties are \
determined by the following options"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "QVectorToArrow", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"QArrowHead", "\[Rule]", "True"}], ",", 
   RowBox[{"QArrowShaft", "\[Rule]", "False"}], ",", 
   RowBox[{"QLinePointSize", "\[Rule]", "0.001`"}], ",", 
   RowBox[{"QMinLength", "\[Rule]", "0.1`"}], ",", 
   RowBox[{"QArrowShape", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"QHeadColor", "\[Rule]", 
    RowBox[{"GrayLevel", "[", "0.5`", "]"}]}], ",", 
   RowBox[{"QArrowScale", "\[Rule]", "1"}], ",", 
   RowBox[{"QShaftColor", "\[Rule]", 
    RowBox[{"GrayLevel", "[", "0", "]"}]}], ",", 
   RowBox[{"QNeedleStyle", "\[Rule]", "False"}]}], "}"}]], "Output",
 CellChangeTimes->{3.3902022976596994`*^9, 3.390231751706073*^9, 
  3.391781386262101*^9, 3.3918507058125*^9, 3.391850842828125*^9, 
  3.3924726150185738`*^9, 3.3924726890253468`*^9, 3.3938505461522617`*^9, 
  3.393851030533441*^9, 3.393853521834642*^9}]
}, Open  ]],

Cell[TextData[{
 StyleBox["QArrowHead->True", "InlineInput"],
 " (default) causes the vector to be drawn with an arrow head.",
 StyleBox[" QArrowHead->Automatic", "InlineInput"],
 " draws the vector with a point of size ",
 StyleBox["2*QLinePointSize", "InlineInput"],
 " instead of the ",
 StyleBox["QArrowHead", "InlineInput"],
 ". Otherwise the vector is just represented by a line segment."
}], "Text"],

Cell[TextData[{
 StyleBox["QMinLength", "InlineInput"],
 StyleBox["->0.1", "InlineInput"],
 " (default) specifies that vectors of length less than 0.1 are to be drawn \
as points of size QLinePointSize."
}], "Text"],

Cell[TextData[{
 StyleBox["QArrowShape->{", "InlineInput"],
 StyleBox["n",
  FontSlant->"Italic"],
 ",",
 StyleBox["k",
  FontSlant->"Italic"],
 StyleBox["}", "InlineInput"],
 ": The head of the arrow is a cone with half height ",
 StyleBox["h",
  FontSlant->"Italic"],
 " = ",
 StyleBox["k",
  FontSlant->"Italic"],
 " * ",
 StyleBox["Norm[", "InlineInput"],
 "vector",
 StyleBox["]", "InlineInput"],
 " and radius ",
 StyleBox["r ",
  FontSlant->"Italic"],
 "= ",
 StyleBox["h",
  FontSlant->"Italic"],
 StyleBox["/(2 GoldenRatio)", "InlineInput"],
 " drawn using ",
 StyleBox["n",
  FontSlant->"Italic"],
 " polygons. If ",
 StyleBox["k",
  FontSlant->"Italic"],
 " is negative, then ",
 StyleBox["h",
  FontSlant->"Italic"],
 " = -",
 StyleBox["k",
  FontSlant->"Italic"],
 ". Default is (1,1/6)"
}], "Text"],

Cell[TextData[{
 StyleBox["QArrowScale", "InlineInput"],
 StyleBox["-> ", "InlineInput"],
 StyleBox["k",
  FontSlant->"Italic"],
 " scales the length of the arrow by the factor ",
 StyleBox["k",
  FontSlant->"Italic"],
 "."
}], "Text"],

Cell[TextData[{
 StyleBox["QHeadColor", "InlineInput"],
 StyleBox["->", "InlineInput"],
 StyleBox["colordirective",
  FontSlant->"Italic"],
 ": The arrow appears in the color specified by colordirective. Turn Lighting \
off to see the effect."
}], "Text"],

Cell[TextData[{
 StyleBox["QLinePointSize", "InlineInput"],
 StyleBox["->", "InlineInput"],
 StyleBox["pt",
  FontSlant->"Italic"],
 "  causes the shaft of the arrow to be drawn with relative pointsize ",
 StyleBox["pt",
  FontSlant->"Italic"],
 ". Default is  ",
 StyleBox["pt ",
  FontSlant->"Italic"],
 "= 0.001."
}], "Text"],

Cell[TextData[{
 StyleBox["QSpinorToArrow", "InlineInput"],
 " draws the arrow head by default in a color determined by the complex phase \
of the spinor. The options ",
 StyleBox["QHeadColor->QExtractPhase", "InlineInput"],
 " is also accepted. The complex phase factor ",
 StyleBox["Exp[I ", "InlineInput"],
 "phase",
 StyleBox["] ", "InlineInput"],
 "is mapped to colors according to ",
 StyleBox["Hue[", "InlineInput"],
 "phase",
 StyleBox["/(2 Pi)]", "InlineInput"],
 " (this is the standard color map of complex numbers)."
}], "Text"],

Cell["Here are some examples:", "Text"],

Cell[CellGroupData[{

Cell["\<\
Graphics3D[
\tQVectorToArrow[{1,1,-1},
\t\tQArrowShape->{12,1/3}, QLinePointSize->0.007]
]\
\>", "Input",
 CellChangeTimes->{{3.390222413776577*^9, 3.390222416879047*^9}, {
   3.390232270323421*^9, 3.390232286372056*^9}, 3.3902323268905163`*^9, {
   3.390294045113927*^9, 3.390294065147149*^9}, 3.390294211922637*^9}],

Cell[BoxData[
 Graphics3DBox[{
   {Thickness[0.007], Line3DBox[{{0, 0, 0}, {1, 1, -1}}]}, 
   {GrayLevel[1], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3]}, {
       Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio}}, {{
       1, 1, -1}, {0.7928223263445201, 
       0.5405110069888132, -0.6666666666666666}, {0.7395026707414701, 
       0.5209946585170596, -0.7395026707414701}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3], Rational[2, 3] + 
        Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, Rational[-2, 3] + 
        Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio}}, {{1, 1, -1}, {
       0.7395026707414701, 0.5209946585170596, -0.7395026707414701}, {
       0.6666666666666666, 0.5405110069888132, -0.7928223263445201}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3], Rational[2, 3] + 
        Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, Rational[-2, 3] + 
        Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[-1, 3] 2^Rational[-1, 2]/GoldenRatio}}, {{
       1, 1, -1}, {0.6666666666666666, 
       0.5405110069888132, -0.7928223263445201}, {0.5938306625918631, 
       0.5938306625918631, -0.8123386748162736}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[-1, 3] 2^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3], Rational[-2, 3] + 
        Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio}}, {{1, 1, -1}, {
       0.5938306625918631, 0.5938306625918631, -0.8123386748162736}, {
       0.5405110069888132, 0.6666666666666666, -0.7928223263445201}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3], Rational[-2, 3] + 
        Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[-1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio}}, {{
       1, 1, -1}, {0.5405110069888132, 
       0.6666666666666666, -0.7928223263445201}, {0.5209946585170596, 
       0.7395026707414701, -0.7395026707414701}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[-1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3]}}, {{1, 1, -1}, {0.5209946585170596, 
       0.7395026707414701, -0.7395026707414701}, {0.5405110069888132, 
       0.7928223263445201, -0.6666666666666666}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3]}, {
       Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio}}, {{1,
        1, -1}, {0.5405110069888132, 
       0.7928223263445201, -0.6666666666666666}, {0.5938306625918631, 
       0.8123386748162736, -0.5938306625918631}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3], Rational[2, 3] + 
        Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, Rational[-2, 3] + 
        Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio}}, {{1, 1, -1}, {
       0.5938306625918631, 0.8123386748162736, -0.5938306625918631}, {
       0.6666666666666666, 0.7928223263445201, -0.5405110069888132}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3], Rational[2, 3] + 
        Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, Rational[-2, 3] + 
        Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[1, 3] 2^Rational[-1, 2]/GoldenRatio}}, {{1,
        1, -1}, {0.6666666666666666, 
       0.7928223263445201, -0.5405110069888132}, {0.7395026707414701, 
       0.7395026707414701, -0.5209946585170596}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[1, 3] 2^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3], Rational[-2, 3] + 
        Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio}}, {{1, 1, -1}, {
       0.7395026707414701, 0.7395026707414701, -0.5209946585170596}, {
       0.7928223263445201, 0.6666666666666666, -0.5405110069888132}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3], Rational[-2, 3] + 
        Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio}}, {{1,
        1, -1}, {0.7928223263445201, 
       0.6666666666666666, -0.5405110069888132}, {0.8123386748162736, 
       0.5938306625918631, -0.5938306625918631}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3]}}, {{1, 1, -1}, {0.8123386748162736, 
       0.5938306625918631, -0.5938306625918631}, {0.7928223263445201, 
       0.5405110069888132, -0.6666666666666666}}]]}}]], "Output",
 CellChangeTimes->{
  3.39023233974994*^9, 3.3902940119666157`*^9, {3.390294048984612*^9, 
   3.390294065560966*^9}, 3.3902942129021673`*^9, 3.391781386630105*^9, 
   3.391850706765625*^9, 3.391850843125*^9, 3.391853839921875*^9, 
   3.3924726154944*^9, 3.392472689175414*^9, 3.3938505463731956`*^9, 
   3.393851030683734*^9, 3.3938539756914253`*^9, 3.3938541607058372`*^9, 
   3.393854207475073*^9, 3.393854432106103*^9, 3.393854615486845*^9, 
   3.393854726204153*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "InputForm"}]], "Input",
 CellChangeTimes->{{3.393854211124669*^9, 3.393854213762082*^9}}],

Cell["\<\
Graphics3D[{{Thickness[0.007], Line[{{0, 0, 0}, {1, 1, -1}}]}, 
  {GrayLevel[0.5], {Polygon[{{1, 1, -1}, {2/3 + 1/(2*Sqrt[6]*GoldenRatio), \
2/3 - 1/(2*Sqrt[6]*GoldenRatio), 
       -2/3}, {2/3 + 1/(6*Sqrt[2]*GoldenRatio), 2/3 - \
1/(3*Sqrt[2]*GoldenRatio), 
       -2/3 - 1/(6*Sqrt[2]*GoldenRatio)}}], Polygon[{{1, 1, -1}, {2/3 + \
1/(6*Sqrt[2]*GoldenRatio), 
       2/3 - 1/(3*Sqrt[2]*GoldenRatio), -2/3 - 1/(6*Sqrt[2]*GoldenRatio)}, 
      {2/3, 2/3 - 1/(2*Sqrt[6]*GoldenRatio), -2/3 - \
1/(2*Sqrt[6]*GoldenRatio)}}], 
    Polygon[{{1, 1, -1}, {2/3, 2/3 - 1/(2*Sqrt[6]*GoldenRatio), -2/3 - \
1/(2*Sqrt[6]*GoldenRatio)}, 
      {2/3 - 1/(6*Sqrt[2]*GoldenRatio), 2/3 - 1/(6*Sqrt[2]*GoldenRatio), -2/3 \
- 1/(3*Sqrt[2]*GoldenRatio)}}], 
    Polygon[{{1, 1, -1}, {2/3 - 1/(6*Sqrt[2]*GoldenRatio), 2/3 - \
1/(6*Sqrt[2]*GoldenRatio), 
       -2/3 - 1/(3*Sqrt[2]*GoldenRatio)}, {2/3 - 1/(2*Sqrt[6]*GoldenRatio), \
2/3, 
       -2/3 - 1/(2*Sqrt[6]*GoldenRatio)}}], Polygon[{{1, 1, -1}, {2/3 - \
1/(2*Sqrt[6]*GoldenRatio), 2/3, 
       -2/3 - 1/(2*Sqrt[6]*GoldenRatio)}, {2/3 - 1/(3*Sqrt[2]*GoldenRatio), \
2/3 + 1/(6*Sqrt[2]*GoldenRatio), 
       -2/3 - 1/(6*Sqrt[2]*GoldenRatio)}}], Polygon[{{1, 1, -1}, {2/3 - \
1/(3*Sqrt[2]*GoldenRatio), 
       2/3 + 1/(6*Sqrt[2]*GoldenRatio), -2/3 - 1/(6*Sqrt[2]*GoldenRatio)}, \
{2/3 - 1/(2*Sqrt[6]*GoldenRatio), 
       2/3 + 1/(2*Sqrt[6]*GoldenRatio), -2/3}}], 
    Polygon[{{1, 1, -1}, {2/3 - 1/(2*Sqrt[6]*GoldenRatio), 2/3 + \
1/(2*Sqrt[6]*GoldenRatio), -2/3}, 
      {2/3 - 1/(6*Sqrt[2]*GoldenRatio), 2/3 + 1/(3*Sqrt[2]*GoldenRatio), -2/3 \
+ 1/(6*Sqrt[2]*GoldenRatio)}}], 
    Polygon[{{1, 1, -1}, {2/3 - 1/(6*Sqrt[2]*GoldenRatio), 2/3 + \
1/(3*Sqrt[2]*GoldenRatio), 
       -2/3 + 1/(6*Sqrt[2]*GoldenRatio)}, {2/3, 2/3 + \
1/(2*Sqrt[6]*GoldenRatio), 
       -2/3 + 1/(2*Sqrt[6]*GoldenRatio)}}], Polygon[{{1, 1, -1}, {2/3, 2/3 + \
1/(2*Sqrt[6]*GoldenRatio), 
       -2/3 + 1/(2*Sqrt[6]*GoldenRatio)}, {2/3 + 1/(6*Sqrt[2]*GoldenRatio), \
2/3 + 1/(6*Sqrt[2]*GoldenRatio), 
       -2/3 + 1/(3*Sqrt[2]*GoldenRatio)}}], Polygon[{{1, 1, -1}, {2/3 + \
1/(6*Sqrt[2]*GoldenRatio), 
       2/3 + 1/(6*Sqrt[2]*GoldenRatio), -2/3 + 1/(3*Sqrt[2]*GoldenRatio)}, \
{2/3 + 1/(2*Sqrt[6]*GoldenRatio), 
       2/3, -2/3 + 1/(2*Sqrt[6]*GoldenRatio)}}], 
    Polygon[{{1, 1, -1}, {2/3 + 1/(2*Sqrt[6]*GoldenRatio), 2/3, -2/3 + \
1/(2*Sqrt[6]*GoldenRatio)}, 
      {2/3 + 1/(3*Sqrt[2]*GoldenRatio), 2/3 - 1/(6*Sqrt[2]*GoldenRatio), -2/3 \
+ 1/(6*Sqrt[2]*GoldenRatio)}}], 
    Polygon[{{1, 1, -1}, {2/3 + 1/(3*Sqrt[2]*GoldenRatio), 2/3 - \
1/(6*Sqrt[2]*GoldenRatio), 
       -2/3 + 1/(6*Sqrt[2]*GoldenRatio)}, {2/3 + 1/(2*Sqrt[6]*GoldenRatio), \
2/3 - 1/(2*Sqrt[6]*GoldenRatio), 
       -2/3}}]}}}]\
\>", "Output",
 CellChangeTimes->{3.393854435481262*^9, 3.3938546199633083`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Graphics3D[{{Thickness[0.007], Line[{{0, 0, 0}, {1, 1, -1}}]}, 
  {White, {Polygon[{{1, 1, -1}, {2/3 + 1/(2*Sqrt[6]*GoldenRatio), 2/3 - \
1/(2*Sqrt[6]*GoldenRatio), 
       -2/3}, {2/3 + 1/(6*Sqrt[2]*GoldenRatio), 2/3 - \
1/(3*Sqrt[2]*GoldenRatio), 
       -2/3 - 1/(6*Sqrt[2]*GoldenRatio)}}], Polygon[{{1, 1, -1}, {2/3 + \
1/(6*Sqrt[2]*GoldenRatio), 
       2/3 - 1/(3*Sqrt[2]*GoldenRatio), -2/3 - 1/(6*Sqrt[2]*GoldenRatio)}, 
      {2/3, 2/3 - 1/(2*Sqrt[6]*GoldenRatio), -2/3 - \
1/(2*Sqrt[6]*GoldenRatio)}}], 
    Polygon[{{1, 1, -1}, {2/3, 2/3 - 1/(2*Sqrt[6]*GoldenRatio), -2/3 - \
1/(2*Sqrt[6]*GoldenRatio)}, 
      {2/3 - 1/(6*Sqrt[2]*GoldenRatio), 2/3 - 1/(6*Sqrt[2]*GoldenRatio), -2/3 \
- 1/(3*Sqrt[2]*GoldenRatio)}}], 
    Polygon[{{1, 1, -1}, {2/3 - 1/(6*Sqrt[2]*GoldenRatio), 2/3 - \
1/(6*Sqrt[2]*GoldenRatio), 
       -2/3 - 1/(3*Sqrt[2]*GoldenRatio)}, {2/3 - 1/(2*Sqrt[6]*GoldenRatio), \
2/3, 
       -2/3 - 1/(2*Sqrt[6]*GoldenRatio)}}], Polygon[{{1, 1, -1}, {2/3 - \
1/(2*Sqrt[6]*GoldenRatio), 2/3, 
       -2/3 - 1/(2*Sqrt[6]*GoldenRatio)}, {2/3 - 1/(3*Sqrt[2]*GoldenRatio), \
2/3 + 1/(6*Sqrt[2]*GoldenRatio), 
       -2/3 - 1/(6*Sqrt[2]*GoldenRatio)}}], Polygon[{{1, 1, -1}, {2/3 - \
1/(3*Sqrt[2]*GoldenRatio), 
       2/3 + 1/(6*Sqrt[2]*GoldenRatio), -2/3 - 1/(6*Sqrt[2]*GoldenRatio)}, \
{2/3 - 1/(2*Sqrt[6]*GoldenRatio), 
       2/3 + 1/(2*Sqrt[6]*GoldenRatio), -2/3}}], 
    Polygon[{{1, 1, -1}, {2/3 - 1/(2*Sqrt[6]*GoldenRatio), 2/3 + \
1/(2*Sqrt[6]*GoldenRatio), -2/3}, 
      {2/3 - 1/(6*Sqrt[2]*GoldenRatio), 2/3 + 1/(3*Sqrt[2]*GoldenRatio), -2/3 \
+ 1/(6*Sqrt[2]*GoldenRatio)}}], 
    Polygon[{{1, 1, -1}, {2/3 - 1/(6*Sqrt[2]*GoldenRatio), 2/3 + \
1/(3*Sqrt[2]*GoldenRatio), 
       -2/3 + 1/(6*Sqrt[2]*GoldenRatio)}, {2/3, 2/3 + \
1/(2*Sqrt[6]*GoldenRatio), 
       -2/3 + 1/(2*Sqrt[6]*GoldenRatio)}}], Polygon[{{1, 1, -1}, {2/3, 2/3 + \
1/(2*Sqrt[6]*GoldenRatio), 
       -2/3 + 1/(2*Sqrt[6]*GoldenRatio)}, {2/3 + 1/(6*Sqrt[2]*GoldenRatio), \
2/3 + 1/(6*Sqrt[2]*GoldenRatio), 
       -2/3 + 1/(3*Sqrt[2]*GoldenRatio)}}], Polygon[{{1, 1, -1}, {2/3 + \
1/(6*Sqrt[2]*GoldenRatio), 
       2/3 + 1/(6*Sqrt[2]*GoldenRatio), -2/3 + 1/(3*Sqrt[2]*GoldenRatio)}, \
{2/3 + 1/(2*Sqrt[6]*GoldenRatio), 
       2/3, -2/3 + 1/(2*Sqrt[6]*GoldenRatio)}}], 
    Polygon[{{1, 1, -1}, {2/3 + 1/(2*Sqrt[6]*GoldenRatio), 2/3, -2/3 + \
1/(2*Sqrt[6]*GoldenRatio)}, 
      {2/3 + 1/(3*Sqrt[2]*GoldenRatio), 2/3 - 1/(6*Sqrt[2]*GoldenRatio), -2/3 \
+ 1/(6*Sqrt[2]*GoldenRatio)}}], 
    Polygon[{{1, 1, -1}, {2/3 + 1/(3*Sqrt[2]*GoldenRatio), 2/3 - \
1/(6*Sqrt[2]*GoldenRatio), 
       -2/3 + 1/(6*Sqrt[2]*GoldenRatio)}, {2/3 + 1/(2*Sqrt[6]*GoldenRatio), \
2/3 - 1/(2*Sqrt[6]*GoldenRatio), 
       -2/3}}]}}},Lighting->Automatic]\
\>", "Input",
 CellChangeTimes->{{3.3938542142488337`*^9, 3.3938542274173117`*^9}, {
  3.393854286014658*^9, 3.393854312025385*^9}}],

Cell[BoxData[
 Graphics3DBox[{
   {Thickness[0.007], Line3DBox[{{0, 0, 0}, {1, 1, -1}}]}, 
   {GrayLevel[1], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3]}, {
       Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio}}, {{
       1, 1, -1}, {0.7928223263445201, 
       0.5405110069888132, -0.6666666666666666}, {0.7395026707414701, 
       0.5209946585170596, -0.7395026707414701}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3], Rational[2, 3] + 
        Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, Rational[-2, 3] + 
        Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio}}, {{1, 1, -1}, {
       0.7395026707414701, 0.5209946585170596, -0.7395026707414701}, {
       0.6666666666666666, 0.5405110069888132, -0.7928223263445201}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3], Rational[2, 3] + 
        Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, Rational[-2, 3] + 
        Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[-1, 3] 2^Rational[-1, 2]/GoldenRatio}}, {{
       1, 1, -1}, {0.6666666666666666, 
       0.5405110069888132, -0.7928223263445201}, {0.5938306625918631, 
       0.5938306625918631, -0.8123386748162736}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[-1, 3] 2^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3], Rational[-2, 3] + 
        Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio}}, {{1, 1, -1}, {
       0.5938306625918631, 0.5938306625918631, -0.8123386748162736}, {
       0.5405110069888132, 0.6666666666666666, -0.7928223263445201}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3], Rational[-2, 3] + 
        Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[-1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio}}, {{
       1, 1, -1}, {0.5405110069888132, 
       0.6666666666666666, -0.7928223263445201}, {0.5209946585170596, 
       0.7395026707414701, -0.7395026707414701}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[-1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3]}}, {{1, 1, -1}, {0.5209946585170596, 
       0.7395026707414701, -0.7395026707414701}, {0.5405110069888132, 
       0.7928223263445201, -0.6666666666666666}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3]}, {
       Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio}}, {{1,
        1, -1}, {0.5405110069888132, 
       0.7928223263445201, -0.6666666666666666}, {0.5938306625918631, 
       0.8123386748162736, -0.5938306625918631}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3], Rational[2, 3] + 
        Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, Rational[-2, 3] + 
        Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio}}, {{1, 1, -1}, {
       0.5938306625918631, 0.8123386748162736, -0.5938306625918631}, {
       0.6666666666666666, 0.7928223263445201, -0.5405110069888132}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3], Rational[2, 3] + 
        Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, Rational[-2, 3] + 
        Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[1, 3] 2^Rational[-1, 2]/GoldenRatio}}, {{1,
        1, -1}, {0.6666666666666666, 
       0.7928223263445201, -0.5405110069888132}, {0.7395026707414701, 
       0.7395026707414701, -0.5209946585170596}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[1, 3] 2^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3], Rational[-2, 3] + 
        Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio}}, {{1, 1, -1}, {
       0.7395026707414701, 0.7395026707414701, -0.5209946585170596}, {
       0.7928223263445201, 0.6666666666666666, -0.5405110069888132}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3], Rational[-2, 3] + 
        Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio}}, {{1,
        1, -1}, {0.7928223263445201, 
       0.6666666666666666, -0.5405110069888132}, {0.8123386748162736, 
       0.5938306625918631, -0.5938306625918631}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3]}}, {{1, 1, -1}, {0.8123386748162736, 
       0.5938306625918631, -0.5938306625918631}, {0.7928223263445201, 
       0.5405110069888132, -0.6666666666666666}}]]}},
  Lighting->Automatic]], "Output",
 CellChangeTimes->{3.393854331209014*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QVectorToArrow", "//", "Options"}]], "Input",
 CellChangeTimes->{{3.390219785784399*^9, 3.390219790421028*^9}, {
  3.390219842903577*^9, 3.390219843833543*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"QArrowHead", "\[Rule]", "True"}], ",", 
   RowBox[{"QArrowShaft", "\[Rule]", "False"}], ",", 
   RowBox[{"QLinePointSize", "\[Rule]", "0.001`"}], ",", 
   RowBox[{"QMinLength", "\[Rule]", "0.1`"}], ",", 
   RowBox[{"QArrowShape", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"QHeadColor", "\[Rule]", 
    RowBox[{"GrayLevel", "[", "0.5`", "]"}]}], ",", 
   RowBox[{"QArrowScale", "\[Rule]", "1"}], ",", 
   RowBox[{"QShaftColor", "\[Rule]", 
    RowBox[{"GrayLevel", "[", "0", "]"}]}], ",", 
   RowBox[{"QNeedleStyle", "\[Rule]", "False"}], ",", 
   RowBox[{"Glow", "\[Rule]", "True"}], ",", 
   RowBox[{"Lighting", "\[Rule]", "None"}]}], "}"}]], "Output",
 CellChangeTimes->{3.3902197908173113`*^9, 3.390219844172699*^9, 
  3.390232345424114*^9, 3.391781386669127*^9, 3.391850707109375*^9, 
  3.391850843140625*^9, 3.392472616367331*^9, 3.392472689190824*^9, 
  3.393850546519372*^9, 3.393851030711912*^9}]
}, Open  ]],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.390219873706263*^9, 3.390219874375732*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graphics3D", "[", 
  RowBox[{"QVectorToArrow", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "1", ",", 
      RowBox[{"-", "1"}]}], "}"}], ",", 
    RowBox[{"QArrowShape", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"12", ",", 
       FractionBox["1", "3"]}], "}"}]}], ",", 
    RowBox[{"QLinePointSize", "\[Rule]", "0.02"}], ",", 
    RowBox[{"Lighting", "\[Rule]", "Automatic"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.3902194266242*^9, 3.390219441688385*^9}, {
   3.390219501354521*^9, 3.3902195357024927`*^9}, {3.390219879910611*^9, 
   3.390219906128868*^9}, {3.3902200204808617`*^9, 3.39022005446014*^9}, {
   3.39022223870511*^9, 3.390222239160843*^9}, {3.390232365319017*^9, 
   3.3902323732864933`*^9}, {3.390293993592916*^9, 3.390293999193632*^9}, 
   3.393850671122547*^9, {3.393850726395426*^9, 3.393850739636883*^9}}],

Cell[BoxData[
 Graphics3DBox[{
   {Thickness[0.02], Line3DBox[{{0, 0, 0}, {1, 1, -1}}]}, 
   {GrayLevel[1], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3]}, {
       Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio}}, {{
       1, 1, -1}, {0.7928223263445201, 
       0.5405110069888132, -0.6666666666666666}, {0.7395026707414701, 
       0.5209946585170596, -0.7395026707414701}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3], Rational[2, 3] + 
        Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, Rational[-2, 3] + 
        Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio}}, {{1, 1, -1}, {
       0.7395026707414701, 0.5209946585170596, -0.7395026707414701}, {
       0.6666666666666666, 0.5405110069888132, -0.7928223263445201}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3], Rational[2, 3] + 
        Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, Rational[-2, 3] + 
        Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[-1, 3] 2^Rational[-1, 2]/GoldenRatio}}, {{
       1, 1, -1}, {0.6666666666666666, 
       0.5405110069888132, -0.7928223263445201}, {0.5938306625918631, 
       0.5938306625918631, -0.8123386748162736}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[-1, 3] 2^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3], Rational[-2, 3] + 
        Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio}}, {{1, 1, -1}, {
       0.5938306625918631, 0.5938306625918631, -0.8123386748162736}, {
       0.5405110069888132, 0.6666666666666666, -0.7928223263445201}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3], Rational[-2, 3] + 
        Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[-1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio}}, {{
       1, 1, -1}, {0.5405110069888132, 
       0.6666666666666666, -0.7928223263445201}, {0.5209946585170596, 
       0.7395026707414701, -0.7395026707414701}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[-1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3]}}, {{1, 1, -1}, {0.5209946585170596, 
       0.7395026707414701, -0.7395026707414701}, {0.5405110069888132, 
       0.7928223263445201, -0.6666666666666666}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3]}, {
       Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio}}, {{1,
        1, -1}, {0.5405110069888132, 
       0.7928223263445201, -0.6666666666666666}, {0.5938306625918631, 
       0.8123386748162736, -0.5938306625918631}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3], Rational[2, 3] + 
        Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, Rational[-2, 3] + 
        Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio}}, {{1, 1, -1}, {
       0.5938306625918631, 0.8123386748162736, -0.5938306625918631}, {
       0.6666666666666666, 0.7928223263445201, -0.5405110069888132}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3], Rational[2, 3] + 
        Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, Rational[-2, 3] + 
        Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[1, 3] 2^Rational[-1, 2]/GoldenRatio}}, {{1,
        1, -1}, {0.6666666666666666, 
       0.7928223263445201, -0.5405110069888132}, {0.7395026707414701, 
       0.7395026707414701, -0.5209946585170596}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[1, 3] 2^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3], Rational[-2, 3] + 
        Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio}}, {{1, 1, -1}, {
       0.7395026707414701, 0.7395026707414701, -0.5209946585170596}, {
       0.7928223263445201, 0.6666666666666666, -0.5405110069888132}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3], Rational[-2, 3] + 
        Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio}}, {{1,
        1, -1}, {0.7928223263445201, 
       0.6666666666666666, -0.5405110069888132}, {0.8123386748162736, 
       0.5938306625918631, -0.5938306625918631}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3]}}, {{1, 1, -1}, {0.8123386748162736, 
       0.5938306625918631, -0.5938306625918631}, {0.7928223263445201, 
       0.5405110069888132, -0.6666666666666666}}]]}}]], "Output",
 CellChangeTimes->{{3.390232349029167*^9, 3.390232374311734*^9}, {
   3.390293988367948*^9, 3.3902939995946703`*^9}, 3.390294216381464*^9, 
   3.3917813869298687`*^9, 3.391850708234375*^9, 3.391850843421875*^9, 
   3.392472616556045*^9, 3.392472689420823*^9, 3.393850546748045*^9, 
   3.393850671498632*^9, {3.3938507289462347`*^9, 3.393850740289895*^9}, 
   3.3938510310039473`*^9, 3.393851064446913*^9, 3.393854742283937*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "QVectorToArrow", "]"}]], "Input",
 CellChangeTimes->{{3.390232611942411*^9, 3.390232616481105*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"QArrowHead", "\[Rule]", "True"}], ",", 
   RowBox[{"QArrowShaft", "\[Rule]", "False"}], ",", 
   RowBox[{"QLinePointSize", "\[Rule]", "0.001`"}], ",", 
   RowBox[{"QMinLength", "\[Rule]", "0.1`"}], ",", 
   RowBox[{"QArrowShape", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"QHeadColor", "\[Rule]", 
    RowBox[{"GrayLevel", "[", "0.5`", "]"}]}], ",", 
   RowBox[{"QArrowScale", "\[Rule]", "1"}], ",", 
   RowBox[{"QShaftColor", "\[Rule]", 
    RowBox[{"GrayLevel", "[", "0", "]"}]}], ",", 
   RowBox[{"QNeedleStyle", "\[Rule]", "False"}], ",", 
   RowBox[{"Glow", "\[Rule]", "True"}], ",", 
   RowBox[{"Lighting", "\[Rule]", "None"}]}], "}"}]], "Output",
 CellChangeTimes->{3.3902326171154423`*^9, 3.3917813869645653`*^9, 
  3.391850708515625*^9, 3.391850843453125*^9, 3.392472616954069*^9, 
  3.392472689794395*^9, 3.39385054691037*^9, 3.3938510311159487`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graphics3D", "[", 
  RowBox[{"QVectorToArrow", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "1", ",", 
      RowBox[{"-", "1"}]}], "}"}], ",", 
    RowBox[{"QArrowShape", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"12", ",", 
       FractionBox["1", "3"]}], "}"}]}], ",", 
    RowBox[{"QLinePointSize", "\[Rule]", "0.02"}], ",", 
    RowBox[{"QHeadColor", "\[Rule]", "Blue"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.3902194266242*^9, 3.390219441688385*^9}, {
   3.390232387855072*^9, 3.3902324824832993`*^9}, {3.3902900716971197`*^9, 
   3.390290075030418*^9}, {3.390290118627667*^9, 3.390290121254785*^9}, 
   3.390293983873961*^9, 3.393850646254328*^9}],

Cell[BoxData[
 Graphics3DBox[{
   {Thickness[0.02], Line3DBox[{{0, 0, 0}, {1, 1, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3]}, {
       Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio}}, {{
       1, 1, -1}, {0.7928223263445201, 
       0.5405110069888132, -0.6666666666666666}, {0.7395026707414701, 
       0.5209946585170596, -0.7395026707414701}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3], Rational[2, 3] + 
        Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, Rational[-2, 3] + 
        Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio}}, {{1, 1, -1}, {
       0.7395026707414701, 0.5209946585170596, -0.7395026707414701}, {
       0.6666666666666666, 0.5405110069888132, -0.7928223263445201}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3], Rational[2, 3] + 
        Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, Rational[-2, 3] + 
        Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[-1, 3] 2^Rational[-1, 2]/GoldenRatio}}, {{
       1, 1, -1}, {0.6666666666666666, 
       0.5405110069888132, -0.7928223263445201}, {0.5938306625918631, 
       0.5938306625918631, -0.8123386748162736}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[-1, 3] 2^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3], Rational[-2, 3] + 
        Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio}}, {{1, 1, -1}, {
       0.5938306625918631, 0.5938306625918631, -0.8123386748162736}, {
       0.5405110069888132, 0.6666666666666666, -0.7928223263445201}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3], Rational[-2, 3] + 
        Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[-1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio}}, {{
       1, 1, -1}, {0.5405110069888132, 
       0.6666666666666666, -0.7928223263445201}, {0.5209946585170596, 
       0.7395026707414701, -0.7395026707414701}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[-1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3]}}, {{1, 1, -1}, {0.5209946585170596, 
       0.7395026707414701, -0.7395026707414701}, {0.5405110069888132, 
       0.7928223263445201, -0.6666666666666666}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3]}, {
       Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio}}, {{1,
        1, -1}, {0.5405110069888132, 
       0.7928223263445201, -0.6666666666666666}, {0.5938306625918631, 
       0.8123386748162736, -0.5938306625918631}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3], Rational[2, 3] + 
        Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, Rational[-2, 3] + 
        Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio}}, {{1, 1, -1}, {
       0.5938306625918631, 0.8123386748162736, -0.5938306625918631}, {
       0.6666666666666666, 0.7928223263445201, -0.5405110069888132}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3], Rational[2, 3] + 
        Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, Rational[-2, 3] + 
        Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[1, 3] 2^Rational[-1, 2]/GoldenRatio}}, {{1,
        1, -1}, {0.6666666666666666, 
       0.7928223263445201, -0.5405110069888132}, {0.7395026707414701, 
       0.7395026707414701, -0.5209946585170596}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[1, 3] 2^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3], Rational[-2, 3] + 
        Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio}}, {{1, 1, -1}, {
       0.7395026707414701, 0.7395026707414701, -0.5209946585170596}, {
       0.7928223263445201, 0.6666666666666666, -0.5405110069888132}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3], Rational[-2, 3] + 
        Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio}}, {{1,
        1, -1}, {0.7928223263445201, 
       0.6666666666666666, -0.5405110069888132}, {0.8123386748162736, 
       0.5938306625918631, -0.5938306625918631}}]], 
    Polygon3DBox[
     NCache[{{1, 1, -1}, {
       Rational[2, 3] + Rational[1, 3] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 6] 2^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3] + Rational[1, 6] 2^Rational[-1, 2]/GoldenRatio}, {
       Rational[2, 3] + Rational[1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[2, 3] + Rational[-1, 2] 6^Rational[-1, 2]/GoldenRatio, 
        Rational[-2, 3]}}, {{1, 1, -1}, {0.8123386748162736, 
       0.5938306625918631, -0.5938306625918631}, {0.7928223263445201, 
       0.5405110069888132, -0.6666666666666666}}]]}}]], "Output",
 CellChangeTimes->{
  3.390233026319407*^9, 3.3902900755425577`*^9, 3.390290121912304*^9, 
   3.39029063745224*^9, {3.390293977913168*^9, 3.39029398438713*^9}, 
   3.3902942221581583`*^9, 3.391781387148355*^9, 3.391850709203125*^9, 
   3.391850843734375*^9, 3.392472617240334*^9, 3.3924726900096817`*^9, 
   3.3938505471501913`*^9, 3.393850646779047*^9, 3.393851031320139*^9, 
   3.3938549741577187`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graphics3D", "[", 
  RowBox[{
   RowBox[{"QSpinorToArrow", "[", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[ExponentialE]", 
       FractionBox[
        RowBox[{"\[ImaginaryI]", " ", "\[Pi]"}], "4"]], " ", 
      RowBox[{"{", 
       RowBox[{"1", ",", "1"}], "}"}]}], ",", 
     RowBox[{"Lighting", "\[Rule]", "None"}]}], "]"}], ",", 
   RowBox[{"PlotRange", "\[Rule]", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.3902330509952717`*^9, 3.39023305995817*^9}, {
  3.39023335103624*^9, 3.390233387181116*^9}}],

Cell[BoxData[
 Graphics3DBox[{
   {Thickness[0.001], Line3DBox[{{0, 0, 0}, {2, 0, 0}}]}, 
   {Hue[0.125, 1., 1.], 
    Polygon3DBox[
     NCache[{{2, 0, 0}, {Rational[3, 2], Rational[-1, 4]/GoldenRatio, 0}, {
       Rational[3, 2], Rational[-1, 8]/GoldenRatio, Rational[-1, 8] 
        3^Rational[1, 2]/GoldenRatio}}, {{2, 0, 0}, {
       1.5, -0.1545084971874737, 0}, {
       1.5, -0.07725424859373685, -0.1338082836649087}}]], 
    Polygon3DBox[
     NCache[{{2, 0, 0}, {
       Rational[3, 2], Rational[-1, 8]/GoldenRatio, Rational[-1, 8] 
        3^Rational[1, 2]/GoldenRatio}, {
       Rational[3, 2], Rational[1, 8]/GoldenRatio, Rational[-1, 8] 
        3^Rational[1, 2]/GoldenRatio}}, {{2, 0, 0}, {
       1.5, -0.07725424859373685, -0.1338082836649087}, {1.5, 
       0.07725424859373685, -0.1338082836649087}}]], 
    Polygon3DBox[
     NCache[{{2, 0, 0}, {
       Rational[3, 2], Rational[1, 8]/GoldenRatio, Rational[-1, 8] 
        3^Rational[1, 2]/GoldenRatio}, {
       Rational[3, 2], Rational[1, 4]/GoldenRatio, 0}}, {{2, 0, 0}, {1.5, 
       0.07725424859373685, -0.1338082836649087}, {
       1.5, 0.1545084971874737, 0}}]], 
    Polygon3DBox[
     NCache[{{2, 0, 0}, {Rational[3, 2], Rational[1, 4]/GoldenRatio, 0}, {
       Rational[3, 2], Rational[1, 8]/GoldenRatio, Rational[1, 8] 
        3^Rational[1, 2]/GoldenRatio}}, {{2, 0, 0}, {
       1.5, 0.1545084971874737, 0}, {1.5, 0.07725424859373685, 
       0.1338082836649087}}]], 
    Polygon3DBox[
     NCache[{{2, 0, 0}, {
       Rational[3, 2], Rational[1, 8]/GoldenRatio, Rational[1, 8] 
        3^Rational[1, 2]/GoldenRatio}, {
       Rational[3, 2], Rational[-1, 8]/GoldenRatio, Rational[1, 8] 
        3^Rational[1, 2]/GoldenRatio}}, {{2, 0, 0}, {1.5, 0.07725424859373685,
        0.1338082836649087}, {1.5, -0.07725424859373685, 
       0.1338082836649087}}]], 
    Polygon3DBox[
     NCache[{{2, 0, 0}, {
       Rational[3, 2], Rational[-1, 8]/GoldenRatio, Rational[1, 8] 
        3^Rational[1, 2]/GoldenRatio}, {
       Rational[3, 2], Rational[-1, 4]/GoldenRatio, 0}}, {{2, 0, 0}, {
       1.5, -0.07725424859373685, 0.1338082836649087}, {
       1.5, -0.1545084971874737, 0}}]]}},
  PlotRange->All]], "Output",
 CellChangeTimes->{{3.390233045706937*^9, 3.390233060668228*^9}, {
   3.390233324541596*^9, 3.390233387543298*^9}, 3.390233434094542*^9, 
   3.390290639690605*^9, 3.390293974669693*^9, 3.390294224061452*^9, 
   3.391781387222742*^9, 3.3918507095625*^9, 3.391850844015625*^9, 
   3.3924726175892353`*^9, 3.392472690061987*^9, 3.393850547290434*^9, 
   3.393851031710495*^9, 3.393854986561077*^9}]
}, Open  ]],

Cell[TextData[{
 "Finally, there are the commands ",
 StyleBox["QVisualizeVector", "InlineInput"],
 ", ",
 StyleBox["QVisualizeSpinor", "InlineInput"],
 ", and ",
 StyleBox["QVisualizeOperator", "InlineInput"],
 ". They convert their argument into an arrow which is then shown together \
with auxiliary graphics elements. The appearance of those auxiliary graphics \
elements is controlled by the following options"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "QVisualizeVector", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"QDrawUnitSphere", "\[Rule]", "18"}], ",", 
   RowBox[{"QDrawAxes", "\[Rule]", "True"}], ",", 
   RowBox[{"QCoordinateCube", "\[Rule]", "False"}], ",", 
   RowBox[{"QCoordinateCircles", "\[Rule]", "True"}], ",", 
   RowBox[{"QCoordinateCirclesColor", "\[Rule]", 
    RowBox[{"RGBColor", "[", 
     RowBox[{"0", ",", "0", ",", "1"}], "]"}]}], ",", 
   RowBox[{"QArrowHead", "\[Rule]", "True"}], ",", 
   RowBox[{"QArrowShaft", "\[Rule]", "False"}], ",", 
   RowBox[{"QLinePointSize", "\[Rule]", "0.001`"}], ",", 
   RowBox[{"QMinLength", "\[Rule]", "0.1`"}], ",", 
   RowBox[{"QArrowShape", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"QHeadColor", "\[Rule]", 
    RowBox[{"GrayLevel", "[", "0.5`", "]"}]}], ",", 
   RowBox[{"QArrowScale", "\[Rule]", "1"}], ",", 
   RowBox[{"QShaftColor", "\[Rule]", 
    RowBox[{"GrayLevel", "[", "0", "]"}]}], ",", 
   RowBox[{"QNeedleStyle", "\[Rule]", "False"}], ",", 
   RowBox[{"Glow", "\[Rule]", "True"}], ",", 
   RowBox[{"Lighting", "\[Rule]", "None"}]}], "}"}]], "Output",
 CellChangeTimes->{3.3902335540747833`*^9, 3.391781387246126*^9, 
  3.391850709828125*^9, 3.39185084403125*^9, 3.392472617730176*^9, 
  3.392472690123988*^9, 3.3938505473690166`*^9, 3.393851031745069*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "QVisualizeSpinor", "]"}]], "Input",
 CellChangeTimes->{{3.3902908172403173`*^9, 3.390290818091012*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"QNeedleStyle", "\[Rule]", "True"}], ",", 
   RowBox[{"QDrawUnitSphere", "\[Rule]", "18"}], ",", 
   RowBox[{"QDrawAxes", "\[Rule]", "True"}], ",", 
   RowBox[{"QCoordinateCube", "\[Rule]", "False"}], ",", 
   RowBox[{"QCoordinateCircles", "\[Rule]", "True"}], ",", 
   RowBox[{"QCoordinateCirclesColor", "\[Rule]", 
    RowBox[{"RGBColor", "[", 
     RowBox[{"0", ",", "0", ",", "1"}], "]"}]}], ",", 
   RowBox[{"QArrowHead", "\[Rule]", "True"}], ",", 
   RowBox[{"QArrowShaft", "\[Rule]", "False"}], ",", 
   RowBox[{"QLinePointSize", "\[Rule]", "0.001`"}], ",", 
   RowBox[{"QMinLength", "\[Rule]", "0.1`"}], ",", 
   RowBox[{"QArrowShape", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"QHeadColor", "\[Rule]", 
    RowBox[{"GrayLevel", "[", "0.5`", "]"}]}], ",", 
   RowBox[{"QArrowScale", "\[Rule]", "1"}], ",", 
   RowBox[{"QShaftColor", "\[Rule]", 
    RowBox[{"GrayLevel", "[", "0", "]"}]}], ",", 
   RowBox[{"Glow", "\[Rule]", "True"}], ",", 
   RowBox[{"Lighting", "\[Rule]", "None"}]}], "}"}]], "Output",
 CellChangeTimes->{3.390233555986806*^9, 3.390288811016152*^9, 
  3.390290818382369*^9, 3.391781387262019*^9, 3.391850710015625*^9, 
  3.391850844078125*^9, 3.3924726181695423`*^9, 3.392472690163885*^9, 
  3.3938505474537363`*^9, 3.393851031817664*^9}]
}, Open  ]],

Cell[TextData[{
 "This shows a spinor as a vector on the surface of the Bloch-sphere. The \
size and shape of the arrow is controlled by ",
 StyleBox["Options[QArrowToVector]", "InlineInput"],
 " as described above."
}], "Text"],

Cell[CellGroupData[{

Cell["QVisualizeSpinor[QNormalize[(-1+I) {1 , 1+I}]]", "Input",
 CellChangeTimes->{{3.390289860638399*^9, 3.3902899349073353`*^9}, {
   3.39029083164787*^9, 3.390290833950815*^9}, {3.3902908735359917`*^9, 
   3.390290875955452*^9}, {3.390291055540636*^9, 3.3902910819612617`*^9}, 
   3.390291159803007*^9, {3.390292500078312*^9, 3.39029250705829*^9}, {
   3.390293374794389*^9, 3.390293375149041*^9}, 3.390293481275622*^9, 
   3.390293550008285*^9, {3.390293811393549*^9, 3.390293816062522*^9}, 
   3.39029396620325*^9, {3.3902946220029583`*^9, 3.39029463591173*^9}, {
   3.393851551483891*^9, 3.3938516012058067`*^9}}],

Cell[BoxData[
 Graphics3DBox[{{
    {Hue[0.375, 1., 0.6666666666666667], 
     Polygon3DBox[
      NCache[{{Rational[2, 3], Rational[2, 3], Rational[-1, 3]}, {
        Rational[1, 3] + Rational[1, 8] 2^Rational[-1, 2], Rational[1, 3] + 
         Rational[-1, 8] 2^Rational[-1, 2], Rational[-1, 6]}, {
        Rational[1, 3] + Rational[1, 16] 2^Rational[-1, 2] + 
         Rational[-1, 16] 6^Rational[-1, 2], Rational[1, 3] + 
         Rational[-1, 16] 2^Rational[-1, 2] + 
         Rational[-1, 16] 6^Rational[-1, 2], Rational[-1, 6] + 
         Rational[-1, 4] 6^Rational[-1, 2]}}, {{0.6666666666666666, 
       0.6666666666666666, -0.3333333333333333}, {0.42172168098165175`, 
       0.24494498568501488`, -0.16666666666666666`}, {0.3520119890035011, 
       0.2636236413551827, -0.2687287392826324}}]], 
     Polygon3DBox[
      NCache[{{Rational[2, 3], Rational[2, 3], Rational[-1, 3]}, {
        Rational[1, 3] + Rational[1, 16] 2^Rational[-1, 2] + 
         Rational[-1, 16] 6^Rational[-1, 2], Rational[1, 3] + 
         Rational[-1, 16] 2^Rational[-1, 2] + 
         Rational[-1, 16] 6^Rational[-1, 2], Rational[-1, 6] + 
         Rational[-1, 4] 6^Rational[-1, 2]}, {
        Rational[1, 3] + Rational[-1, 16] 2^Rational[-1, 2] + 
         Rational[-1, 16] 6^Rational[-1, 2], Rational[1, 3] + 
         Rational[1, 16] 2^Rational[-1, 2] + 
         Rational[-1, 16] 6^Rational[-1, 2], Rational[-1, 6] + 
         Rational[-1, 4] 6^Rational[-1, 2]}}, {{0.6666666666666666, 
       0.6666666666666666, -0.3333333333333333}, {0.3520119890035011, 
       0.2636236413551827, -0.2687287392826324}, {0.2636236413551827, 
       0.3520119890035011, -0.2687287392826324}}]], 
     Polygon3DBox[
      NCache[{{Rational[2, 3], Rational[2, 3], Rational[-1, 3]}, {
        Rational[1, 3] + Rational[-1, 16] 2^Rational[-1, 2] + 
         Rational[-1, 16] 6^Rational[-1, 2], Rational[1, 3] + 
         Rational[1, 16] 2^Rational[-1, 2] + 
         Rational[-1, 16] 6^Rational[-1, 2], Rational[-1, 6] + 
         Rational[-1, 4] 6^Rational[-1, 2]}, {
        Rational[1, 3] + Rational[-1, 8] 2^Rational[-1, 2], Rational[1, 3] + 
         Rational[1, 8] 2^Rational[-1, 2], Rational[-1, 6]}}, {{
       0.6666666666666666, 0.6666666666666666, -0.3333333333333333}, {
       0.2636236413551827, 0.3520119890035011, -0.2687287392826324}, {
       0.24494498568501488`, 0.42172168098165175`, -0.16666666666666666`}}]], 
     Polygon3DBox[
      NCache[{{Rational[2, 3], Rational[2, 3], Rational[-1, 3]}, {
        Rational[1, 3] + Rational[-1, 8] 2^Rational[-1, 2], Rational[1, 3] + 
         Rational[1, 8] 2^Rational[-1, 2], Rational[-1, 6]}, {
        Rational[1, 3] + Rational[-1, 16] 2^Rational[-1, 2] + 
         Rational[1, 16] 6^Rational[-1, 2], Rational[1, 3] + 
         Rational[1, 16] 2^Rational[-1, 2] + 
         Rational[1, 16] 6^Rational[-1, 2], Rational[-1, 6] + 
         Rational[1, 4] 6^Rational[-1, 2]}}, {{0.6666666666666666, 
       0.6666666666666666, -0.3333333333333333}, {0.24494498568501488`, 
       0.42172168098165175`, -0.16666666666666666`}, {0.3146546776631655, 
       0.40304302531148395`, -0.06460459405070089}}]], 
     Polygon3DBox[
      NCache[{{Rational[2, 3], Rational[2, 3], Rational[-1, 3]}, {
        Rational[1, 3] + Rational[-1, 16] 2^Rational[-1, 2] + 
         Rational[1, 16] 6^Rational[-1, 2], Rational[1, 3] + 
         Rational[1, 16] 2^Rational[-1, 2] + 
         Rational[1, 16] 6^Rational[-1, 2], Rational[-1, 6] + 
         Rational[1, 4] 6^Rational[-1, 2]}, {
        Rational[1, 3] + Rational[1, 16] 2^Rational[-1, 2] + 
         Rational[1, 16] 6^Rational[-1, 2], Rational[1, 3] + 
         Rational[-1, 16] 2^Rational[-1, 2] + 
         Rational[1, 16] 6^Rational[-1, 2], Rational[-1, 6] + 
         Rational[1, 4] 6^Rational[-1, 2]}}, {{0.6666666666666666, 
       0.6666666666666666, -0.3333333333333333}, {0.3146546776631655, 
       0.40304302531148395`, -0.06460459405070089}, {0.40304302531148395`, 
       0.3146546776631655, -0.06460459405070089}}]], 
     Polygon3DBox[
      NCache[{{Rational[2, 3], Rational[2, 3], Rational[-1, 3]}, {
        Rational[1, 3] + Rational[1, 16] 2^Rational[-1, 2] + 
         Rational[1, 16] 6^Rational[-1, 2], Rational[1, 3] + 
         Rational[-1, 16] 2^Rational[-1, 2] + 
         Rational[1, 16] 6^Rational[-1, 2], Rational[-1, 6] + 
         Rational[1, 4] 6^Rational[-1, 2]}, {
        Rational[1, 3] + Rational[1, 8] 2^Rational[-1, 2], Rational[1, 3] + 
         Rational[-1, 8] 2^Rational[-1, 2], Rational[-1, 6]}}, {{
       0.6666666666666666, 0.6666666666666666, -0.3333333333333333}, {
       0.40304302531148395`, 0.3146546776631655, -0.06460459405070089}, {
       0.42172168098165175`, 
       0.24494498568501488`, -0.16666666666666666`}}]]}, 
    {Hue[0.5, 1., 0.8718115663020503], 
     Polygon3DBox[
      NCache[{{0, 0, 0}, {
        Rational[1, 3] + Rational[1, 8] 2^Rational[-1, 2], Rational[1, 3] + 
         Rational[-1, 8] 2^Rational[-1, 2], Rational[-1, 6]}, {
        Rational[1, 3] + Rational[1, 16] 2^Rational[-1, 2] + 
         Rational[-1, 16] 6^Rational[-1, 2], Rational[1, 3] + 
         Rational[-1, 16] 2^Rational[-1, 2] + 
         Rational[-1, 16] 6^Rational[-1, 2], Rational[-1, 6] + 
         Rational[-1, 4] 6^Rational[-1, 2]}}, {{0, 0, 0}, {
        0.42172168098165175`, 0.24494498568501488`, -0.16666666666666666`}, {
        0.3520119890035011, 0.2636236413551827, -0.2687287392826324}}]], 
     Polygon3DBox[
      NCache[{{0, 0, 0}, {
        Rational[1, 3] + Rational[1, 16] 2^Rational[-1, 2] + 
         Rational[-1, 16] 6^Rational[-1, 2], Rational[1, 3] + 
         Rational[-1, 16] 2^Rational[-1, 2] + 
         Rational[-1, 16] 6^Rational[-1, 2], Rational[-1, 6] + 
         Rational[-1, 4] 6^Rational[-1, 2]}, {
        Rational[1, 3] + Rational[-1, 16] 2^Rational[-1, 2] + 
         Rational[-1, 16] 6^Rational[-1, 2], Rational[1, 3] + 
         Rational[1, 16] 2^Rational[-1, 2] + 
         Rational[-1, 16] 6^Rational[-1, 2], Rational[-1, 6] + 
         Rational[-1, 4] 6^Rational[-1, 2]}}, {{0, 0, 0}, {0.3520119890035011,
         0.2636236413551827, -0.2687287392826324}, {0.2636236413551827, 
        0.3520119890035011, -0.2687287392826324}}]], 
     Polygon3DBox[
      NCache[{{0, 0, 0}, {
        Rational[1, 3] + Rational[-1, 16] 2^Rational[-1, 2] + 
         Rational[-1, 16] 6^Rational[-1, 2], Rational[1, 3] + 
         Rational[1, 16] 2^Rational[-1, 2] + 
         Rational[-1, 16] 6^Rational[-1, 2], Rational[-1, 6] + 
         Rational[-1, 4] 6^Rational[-1, 2]}, {
        Rational[1, 3] + Rational[-1, 8] 2^Rational[-1, 2], Rational[1, 3] + 
         Rational[1, 8] 2^Rational[-1, 2], Rational[-1, 6]}}, {{0, 0, 0}, {
        0.2636236413551827, 0.3520119890035011, -0.2687287392826324}, {
        0.24494498568501488`, 0.42172168098165175`, -0.16666666666666666`}}]],
      Polygon3DBox[
      NCache[{{0, 0, 0}, {
        Rational[1, 3] + Rational[-1, 8] 2^Rational[-1, 2], Rational[1, 3] + 
         Rational[1, 8] 2^Rational[-1, 2], Rational[-1, 6]}, {
        Rational[1, 3] + Rational[-1, 16] 2^Rational[-1, 2] + 
         Rational[1, 16] 6^Rational[-1, 2], Rational[1, 3] + 
         Rational[1, 16] 2^Rational[-1, 2] + 
         Rational[1, 16] 6^Rational[-1, 2], Rational[-1, 6] + 
         Rational[1, 4] 6^Rational[-1, 2]}}, {{0, 0, 0}, {
        0.24494498568501488`, 0.42172168098165175`, -0.16666666666666666`}, {
        0.3146546776631655, 0.40304302531148395`, -0.06460459405070089}}]], 
     Polygon3DBox[
      NCache[{{0, 0, 0}, {
        Rational[1, 3] + Rational[-1, 16] 2^Rational[-1, 2] + 
         Rational[1, 16] 6^Rational[-1, 2], Rational[1, 3] + 
         Rational[1, 16] 2^Rational[-1, 2] + 
         Rational[1, 16] 6^Rational[-1, 2], Rational[-1, 6] + 
         Rational[1, 4] 6^Rational[-1, 2]}, {
        Rational[1, 3] + Rational[1, 16] 2^Rational[-1, 2] + 
         Rational[1, 16] 6^Rational[-1, 2], Rational[1, 3] + 
         Rational[-1, 16] 2^Rational[-1, 2] + 
         Rational[1, 16] 6^Rational[-1, 2], Rational[-1, 6] + 
         Rational[1, 4] 6^Rational[-1, 2]}}, {{0, 0, 0}, {0.3146546776631655, 
        0.40304302531148395`, -0.06460459405070089}, {0.40304302531148395`, 
        0.3146546776631655, -0.06460459405070089}}]], 
     Polygon3DBox[
      NCache[{{0, 0, 0}, {
        Rational[1, 3] + Rational[1, 16] 2^Rational[-1, 2] + 
         Rational[1, 16] 6^Rational[-1, 2], Rational[1, 3] + 
         Rational[-1, 16] 2^Rational[-1, 2] + 
         Rational[1, 16] 6^Rational[-1, 2], Rational[-1, 6] + 
         Rational[1, 4] 6^Rational[-1, 2]}, {
        Rational[1, 3] + Rational[1, 8] 2^Rational[-1, 2], Rational[1, 3] + 
         Rational[-1, 8] 2^Rational[-1, 2], Rational[-1, 6]}}, {{0, 0, 0}, {
        0.40304302531148395`, 0.3146546776631655, -0.06460459405070089}, {
        0.42172168098165175`, 
        0.24494498568501488`, -0.16666666666666666`}}]]}}, {
    {GrayLevel[0.5], Line3DBox[CompressedData["
1:eJxN2Hk41NsfB3BLCJHqhhBKt7InQsJxs1SIsodCtla7riUTBoWEsqYQQvZ9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      "]], Line3DBox[CompressedData["
1:eJxN2Hk0ldv/B3BJ4lLGMiZuokQUEqktJKHj3kiiiERJSuaQzEVEGTMcQiqZ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      "]], Line3DBox[CompressedData["

1:eJxN2Hk4VVv4B3CKhFCOses2ikR1E6JiSSFForgVCUkDDSRDJUNlijKXKUVF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      "]], 
     Line3DBox[CompressedData["
1:eJxN2Hk41Nv/AHBJIjdabilXpUIqqcjW4ESlqBCylLImS5RMuLYwCil7Icoa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      "]], 
     Line3DBox[CompressedData["
1:eJxN2Hk4VN//AHA7kRahSCpLklJ8UkidIhGSYpI2UlKKRBRJJKVEkUTZl+yy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      "]], Line3DBox[CompressedData["

1:eJxN2Hk4lNsfAPAhSyRJKWWrKHQVkrJMjlAqEoPSr6RS2rgNCeG26CJbN0si
ciVl35I1OdaapDL2ZTArM/OGm7JUrn6e3/O759x5nvefz/M+Z877fc/3e77v
oVw+vf4siUQSXbgGFq5FgCgP5LvYZpm5Ht/q9+GFL2CS/vcDRFlO9KO6ORA5
Nql+b4Ux9lLDMFOfePBwTGNwl+Y49sLSYeu+aJBZX7TKpKgBe/bezJbfD4D8
Jcc0F/M+Y3+gXr56zTbwok3o2sgMxX6DsT2bUQ5eXtUKT0sxxe52anl33hdQ
76A66eHLwW5yaMfNmEnQ9Ezp54WqO9ilP/b6fPoJaORx/5GUW8iFnfRrtj9T
wLuac4FxneLYU8YeCCIh+DA7M5jL/R37ka0dmW/SAd04yXkonoFdzvumqcEL
0N7ZF+S2Px+5oFk0xl0qG3S6p2U2uB7C7unFGFmhAbplJxeprH6EfcUd9vF5
d9CTtKckvGwVcn5+wDKWxAnQR7YtOEB6iN08ZVtqzQ3QT2/rNgucRT7avfEP
o41egLEn3FDnXiz242dtD2kogEG6UXTP99+Qj3BnmtqLKGDYVKdgNyzC7nqK
m6GcCpiu/ky+Ex6H167JBFadgKXgLvr2Vxw33oHP1CBTA8CqaR06sTIFOfd5
Yby2Fg2wvRLFOq3I2NVcDEW88gBHVrqHT55HzonUdQyxrAKcZ2xKVB1+LjYH
mNovlQNc6zvjO1p0sVt8OPk0yRRwe/0CBzrw/Fn3xDcc4m4DvMsT/dq2eD7M
gapcD39jwJtvTyLVaGLX9T9BL58HI9cbZrovJyEfYjZM6NflgdFF0TuDXuQg
H/R17JUJUAaj12h8xXBJ5IzFhvIeu8bBKN+4em8HXj/9V1vX8CiRgH+0QyJa
9D/Ie2f6y6/KnwT8V/3epjnKyHscN4Vmf4oFAvVCuxLVGuRdTT6zLiY2QBDs
Z3A8LAZ5p5G3VW9tGxC8D7m+rFAKeXvofceOI55AqJr+dm0iDXkboV7ifdgK
CN3ZyypjSpF/cEz7QLEYAMKsdqMsW1XkLX3Lveri6oCQs/uCeagj8jfxr0FI
4U1ArNQU11S5jLxJJDu/80oMIHZ/j7BWwOu2LousuIGmBYiz05PLxPyRV5fG
U+2NPAER0h09tbEeeWnIWuFfU0cA8XBatem+M/LijSIiIsFrAPHkqWjmKjfk
OTGc0bRjuoDILTZxp29DngHO+27QOguI4gAbGBeC/P6w3ax+TCQgXlAuTnZg
vy7NmTr8cOtCXTRu3izQ+Mdh0IWAQNuFukaUWxVEbXJHngAuMkZ0vyyM8/jK
pHQF8nR6pkdQ+m1AlLhxysukkWevOL5+u7sZIPIXlfkx/kZe5Oj5H13DWUBk
1jWEuXohL7+ktn0HSRoQcSpZgww8/qvygc1RGusB4RcQMlFdh7zB06zl+Dkq
IBzp1ns+o3jC5pduV5W1EgCh89Ehlbsa+Vub2bAndqpA+MP6D/ZoKvLW5Hqe
6WQrEL62Pp94VgX5x7Zd6UvM0oAwWgqoV0wip2dID1jtuAOEe9jLBYkoj2CH
iZrdL6Q6IPh7nOV62gF554TBLQBSgCB7tvZhrzby7sRHW2LqbwOBXcIR3hSq
D7BXeaUrP/AL4H9VOSgQq0LeV7PI7a5UM+BHnQv8oW6JfMB4dI+BVQfgq0fp
WG6lImfkjA3vv6UFRisVb6s9xvEc0q8XVdtXAkZ3OWxprrZCPvzgq379uAEY
aZalrjUoRM6M4F6jzqeDEXKqeBEX5RFk7VxF03kKAK8iRaqS8gQ7o2fp0+v6
gKe3uibzJB6fHcV3+hzVC7gPwkNFBkSRczZX3hGoLwZcMSnZwQEp7PQnFq3p
HYBzoq3YPbYLOZeqpHOsNgiwKxxE4xiHsM9PWb69+xOwV12nBoadRM5L2P3B
dvNJwHJ3OC9I3oR8ZK2ljuGxHYBZUVRsSwPYk/y0A9ogYC4JmU+w70A+quRD
HX/5FxhKlk6q/0sN+x/zTJUXamBwi8z7dVfRvgb5kvMSe+X3gYGHlQ17hvH7
5fvv3Z+YNAP6F5sVSMt+wd6nflrTvBb0RkZExIqeRS4gP9d2LvgbdI+safBR
+oQ93tUgpD8BdB35XGOlp4md7eamFy0LOlpzkqnTQciF+seiv01MgPa18b0b
tu/C7psZnxiyCrTd0jhl4auPvTwsQXpKCby/1cRvLRdiHzsKqzLSQYsh+aa0
BX4vhPLrrY/Efwevv6x12ze5Hvt+BapO2kPQeMaaxAiyw36p1Mbo7G1Qt/RC
Q/jfEdgjutRpm31AZZdH/ZBnGfY/hdtcNVxASZ76q412ztizcq9cmrcG+U+a
uo8yPLEXvJOjZgyBpxPx9adOlmN/ntxfoB8GHlHOpzex8rCX3dDNUXICMeHW
v+jfEMdefs2xriIPUCpmmp6XqP3rftpT7YA5eKfPyzLG5l//W/qbNn/7nzAZ
NiiP7yVhL3qQVV9uBJ/cyTaVXXsPe07MoEn2aphbL97pHIDrA/E4Q/5c6mtY
/IOWHhaqhD2mPXeL3WlYUXp7so60Ezv1m/1Ns62wdsqQtreLhv3gzjZeeiNs
lNj3Or8H13Nio9XWs2aX4Wt7Y9budyb4/U42v1Ft5cK3A/7s/d+WY6+ZTg10
Ogjfq3NjT1PxPIVB1D3117xhmzNFaYnPv9aVwZCfXaEMbJcRS4myWIPXp7D0
z6CVJrAjnSsrHonXvyDh2rja4y+wi5K6fdrjI/ZdcSo9OqWwm20mn1CmjvOF
7bv1InEB9p7JLfN5qIX92krtRYcXwX6Rq4Z+Ctuwy5WFN0R6woFg697GzQo4
f5M3iFXRlsPBZVfKghPmsKtQvtx16oFD8Sec2u3O4PqQaZIpFngfDk+9Oa/I
If5VT4RzHo/VIbOcTish4XXFu08mHA/KQdbpNc8rPXqxL9UqGbhHhmzxqTP5
L3xwHfMVnVbe6AzZpTJHQw/i8TlMpdUqOjGQc2iWNBdljt3EX3Wl4RDkfBWE
JFwqxvX24XQ8RbQYciPG1erv4PrG+m74q6GSL+St3mYXcfk9dmvll/cnr0Je
7iBNhY76WMhMKfS8pzYNR3SdJyQM/PB+Uc3yVL2VA0cqR74WeTvh/YWcL9Q0
7oCjxk1KcBjnHaOtoU3WahMczac4uJh/w/uXWXgLOxdA/oZd4j53UT8G++q2
GZK/Z0F+pF3hqohWvD9aHq0fu7wF8oUPLph3P8X76Z90n7UzH6DANkz2lwOR
yLvkG3dZOnZCwePixgtLG/F+faLRXo1sDgUz6xhP7F3w/l4Q8NEl0BAKLae9
o5/hvGtTkDtZvuEHFIZaugZrLMH9Q9671PZMbShsli8TROA+4e0pRsJJc0so
/DafJCbEfU5zS0vR9+gpSOhIQeVxG9zPmAWwjzmHQsK5Jmr1BO5DXnWtG3/O
WQ+Jq1MmQVdkcL+UKC/OaWiFRGwsVd7pGu6vku4c8+rIhESmjxKJj74jYPZY
p8icjBYk8ix2L2uhI8/YnU+XNFKARLHfez0R1IfDREm5HNqlNEi8OBAlNnAd
+fWNihclNkss1EVjx8NPUZyB/9S9t9HP9BbcITDN3wN5/O1SnRn/pIVxGtcN
mqH/BY8VbLxIqT8hUbIj0F/xHfLsc1EnFHQNF+ZZNZ3xdgB50f0vtvKOcgvP
dVX2Nwk8fukplSX6ygtxS3FcHtGO+/9qWWufbsXtkAi1P98li/v8ugyXKpmf
eZDw2ORbQN2PvHFupXVNsyQkLMgSlEMoL8Ab870tj0cW5qmY0xY0huIMWpr9
DVfUUqCQG6gcbo+/lz9IvjP68YceFBbo7pa/OIO8rUnuvt6aASi8qELRrcX3
t/vf1FPTnYHC9aqdg2O/Iu+U2qAf87stFHRXV9NvnUPeVeE7Czy2QMEN0zNz
8c3IeyxfiT7QjYcCTfOltR8ykfcK+2OGJQchv369ZAvxF/L+237XVALeQf5h
pV9ZM3icge/6dudun4Gjn5xkbFpwPAcjJO1KI67D0SvWPv72KN/B0PRistsb
FhyZ89A4HYzPN5ib7eYXW5vDkZCA1aHM49i5po8iv3Ihb/bAc7szaJ0DVkxC
1JejPpDnPeEgE9yDnG1+nmI91Qq5/fe0VM1GsBOLbgRPakKu+QFnuO4Kck7o
XUVbVhTk5AWVBbJR3gGu5vcx+/lNkCPGV+kKxucn3Oqk62XUIcg+U1gqJKF+
BvD23DBfsvMTZDU6+XtV4PMcXnfe3iu1ayBr2aPM5XFofwQjLs3Vx743QqZ7
Q5+RXTz24V7dQG0GHD6wZb+tbxzyURfyGyWNBDg4RE53lMR5MUqjXRGvWAUZ
PuezXoa+Rs4nJ9c1lcfCfuY+dvMvndgzypRNxzmw77Rz1SlFHE+BdGVy57Mk
2PMq+awFwOdCgouLiKNOubB77bKo2An83S2oqTrBsZiCnTEOot4P8LmTcFmc
AcXiNWzvK1hSYPoK++FL4fH7d0O6mQXZMlka+/3KB+KXNeHHg63Sh+TDsXdQ
WcYfJ+E7erHVkRvpyAkJ72L1oeWQFnmmvlIVnw8QO4LfPXdphU3TKtXMT1rY
T+z9+eijEaz35M26peLzKCK4MYMkZQNrjLpJNsrF2BPjRHoj9WAZ6bu2WTWO
A5FFT8uIboD5v/UZhKZ1YC/02KU3ehc+1TJ57p04gb108YSJjSpMyQBm0zp4
vRFlaR5hoBpGPVGe22mI8/T/5421rse3+paZt/3j/wVnM05v
      "]], 
     Line3DBox[CompressedData["
1:eJxN2Gk0Vl0bB/BCgwyVKRRCeiJzylA9W4ZkaDKmQjKmpJA0SAplTBlKGTKX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      "]], 
     Line3DBox[CompressedData["
1:eJxN2Gk4lU0fAHCpKEVKhERCKWRNRRlLluhEyZZIEkkoxGOJbJUlCi1UCJVI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      "]], 
     Line3DBox[CompressedData["
1:eJxN2nc41t//B3CKhCYZlU9bQxpURuRISYNEkiIhqygrq5JVVpkZWVG27FEU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      "]], Line3DBox[CompressedData["
1:eJxN2Gk41F0bAHCpKEUiQiKhFLKmohxLlmiiZEskiSQU4rFkV1mi0EKFUIlk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      "]], 
     Line3DBox[CompressedData["
1:eJxN2HlUTV8bB3AqQxpQSUWlkp/SLBrw22mQJkOjUKRRJCrJkERFo2ggmkdS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      "]], 
     Line3DBox[CompressedData["
1:eJxN2Hk4lNsfAPAhSySVUspWUegqJGWZHKFUJAalX6FS2riNXbgtusjWLSQi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      "]], 
     Line3DBox[CompressedData["
1:eJxN2Hk4VN//AHBLlkiLLJFUliRRfFJInSIRksokqYgsKRJRJGlRShRJlH3J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      "]], 
     Line3DBox[CompressedData["
1:eJxN2Hk41Nv/AHBJIjdabilXpUIqqcjW4ESlqBCylLImS5RMuLYwCil7Icoa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      "]], 
     Line3DBox[CompressedData["
1:eJxN2Hk4VVv4B3CKhFCOses2ikR1E6JiSSFForgVCUkDDSRDJUNlijKXKUVF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      "]], 
     Line3DBox[CompressedData["
1:eJxN2Hk0ldv/B3AhcSljGRM3pUQUEqkthNBxbyRRRKIkJXNIhlJElDHDUUgy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      "]], Line3DBox[CompressedData["
1:eJxN2Hk41NsfB3BLCJHqhhBKt7InQsJxs1SIsodCtla7riUTBoWEsqYQQvZ9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      "]]}, {
     {GrayLevel[0.4], Thickness[0.005], 
      Line3DBox[{{-1., 0, 0}, {1., 0, 0}}]}, 
     {GrayLevel[0.4], Thickness[0.005], 
      Line3DBox[{{0, -1., 0}, {0, 1., 0}}]}, 
     {GrayLevel[0.4], Thickness[0.005], 
      Line3DBox[{{0, 0, -1.}, {0, 0, 1.}}]}}, 
    {RGBColor[0, 0, 1], PointSize[0.02], Thickness[0.003], 
     Point3DBox[
      NCache[{Rational[2, 3], Rational[2, 3], Rational[-1, 3]}, {
       0.6666666666666666, 0.6666666666666666, -0.3333333333333333}]], 
     Line3DBox[CompressedData["
1:eJxN1ns8VGkfAHBSJNVEWLdWloxpthqhpHgitXhLuUul0m1bWVIJobeLbghJ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      "]], 
     Line3DBox[CompressedData["
1:eJw92Ak0VP0fx3HZSh5UDyFLe2QvpfTEtwVFthItlKlsSX+0EIpC1opUlLJW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      "]]}}},
  Axes->False,
  Boxed->False]], "Output",
 CellChangeTimes->{
  3.390233705348592*^9, {3.390287928736093*^9, 3.3902879542289267`*^9}, {
   3.390289071510254*^9, 3.390289093605995*^9}, 3.390289815689598*^9, {
   3.390289865091864*^9, 3.390289935444324*^9}, 3.390290643510312*^9, 
   3.390290795663542*^9, 3.390290834529838*^9, 3.390290876468884*^9, {
   3.390291058746862*^9, 3.3902910827668467`*^9}, 3.390291160216526*^9, {
   3.390291579010453*^9, 3.3902915997549057`*^9}, {3.390291632570723*^9, 
   3.390291680301694*^9}, 3.390291722094013*^9, {3.390292484130665*^9, 
   3.390292507529818*^9}, {3.390293369793296*^9, 3.390293375531101*^9}, 
   3.3902934821959963`*^9, {3.3902935512015543`*^9, 3.390293558602696*^9}, 
   3.390293627095941*^9, {3.39029374540703*^9, 3.390293759425191*^9}, {
   3.390293805107067*^9, 3.390293816913919*^9}, 3.3902939673409843`*^9, 
   3.390294231143792*^9, {3.390294603821598*^9, 3.39029463679138*^9}, 
   3.391781387721816*^9, 3.391850712015625*^9, 3.391850844890625*^9, 
   3.392472618622245*^9, 3.392472690561982*^9, 3.393850548035213*^9, 
   3.3938510323565483`*^9, {3.393851553189289*^9, 3.3938515713706512`*^9}, 
   3.393851602490643*^9, 3.393855007036696*^9, 3.393855125181432*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
QVisualizeSpinor[QNormalize[(-1+I) {1 , 1+I}], QNeedleStyle->False]\
\>", "Input",
 CellChangeTimes->{{3.39029386237111*^9, 3.390293897491084*^9}}],

Cell[BoxData[
 Graphics3DBox[{{
    {Thickness[0.001], 
     Line3DBox[
      NCache[{{0, 0, 0}, {
        Rational[2, 3], Rational[2, 3], Rational[-1, 3]}}, {{0, 0, 0}, {
        0.6666666666666666, 0.6666666666666666, -0.3333333333333333}}]]}, 
    {Hue[0.375, 1., 0.6666666666666667], 
     Polygon3DBox[
      NCache[{{Rational[2, 3], Rational[2, 3], Rational[-1, 3]}, {
        Rational[1, 2] + Rational[1, 8] 2^Rational[-1, 2]/GoldenRatio, 
         Rational[1, 2] + Rational[-1, 8] 2^Rational[-1, 2]/GoldenRatio, 
         Rational[-1, 4]}, {
        Rational[1, 2] + Rational[1, 16] 2^Rational[-1, 2]/GoldenRatio + 
         Rational[-1, 16] 6^Rational[-1, 2]/GoldenRatio, Rational[1, 2] + 
         Rational[-1, 16] 2^Rational[-1, 2]/GoldenRatio + 
         Rational[-1, 16] 6^Rational[-1, 2]/GoldenRatio, Rational[-1, 4] + 
         Rational[-1, 4] 6^Rational[-1, 2]/GoldenRatio}}, {{
       0.6666666666666666, 0.6666666666666666, -0.3333333333333333}, {
       0.5546270030561026, 0.44537299694389737`, -0.25}, {0.5115440440683197, 
       0.456917041012217, -0.31307782983892674`}}]], 
     Polygon3DBox[
      NCache[{{Rational[2, 3], Rational[2, 3], Rational[-1, 3]}, {
        Rational[1, 2] + Rational[1, 16] 2^Rational[-1, 2]/GoldenRatio + 
         Rational[-1, 16] 6^Rational[-1, 2]/GoldenRatio, Rational[1, 2] + 
         Rational[-1, 16] 2^Rational[-1, 2]/GoldenRatio + 
         Rational[-1, 16] 6^Rational[-1, 2]/GoldenRatio, Rational[-1, 4] + 
         Rational[-1, 4] 6^Rational[-1, 2]/GoldenRatio}, {
        Rational[1, 2] + Rational[-1, 16] 2^Rational[-1, 2]/GoldenRatio + 
         Rational[-1, 16] 6^Rational[-1, 2]/GoldenRatio, Rational[1, 2] + 
         Rational[1, 16] 2^Rational[-1, 2]/GoldenRatio + 
         Rational[-1, 16] 6^Rational[-1, 2]/GoldenRatio, Rational[-1, 4] + 
         Rational[-1, 4] 6^Rational[-1, 2]/GoldenRatio}}, {{
       0.6666666666666666, 0.6666666666666666, -0.3333333333333333}, {
       0.5115440440683197, 0.456917041012217, -0.31307782983892674`}, {
       0.456917041012217, 0.5115440440683197, -0.31307782983892674`}}]], 
     Polygon3DBox[
      NCache[{{Rational[2, 3], Rational[2, 3], Rational[-1, 3]}, {
        Rational[1, 2] + Rational[-1, 16] 2^Rational[-1, 2]/GoldenRatio + 
         Rational[-1, 16] 6^Rational[-1, 2]/GoldenRatio, Rational[1, 2] + 
         Rational[1, 16] 2^Rational[-1, 2]/GoldenRatio + 
         Rational[-1, 16] 6^Rational[-1, 2]/GoldenRatio, Rational[-1, 4] + 
         Rational[-1, 4] 6^Rational[-1, 2]/GoldenRatio}, {
        Rational[1, 2] + Rational[-1, 8] 2^Rational[-1, 2]/GoldenRatio, 
         Rational[1, 2] + Rational[1, 8] 2^Rational[-1, 2]/GoldenRatio, 
         Rational[-1, 4]}}, {{0.6666666666666666, 
       0.6666666666666666, -0.3333333333333333}, {0.456917041012217, 
       0.5115440440683197, -0.31307782983892674`}, {0.44537299694389737`, 
       0.5546270030561026, -0.25}}]], 
     Polygon3DBox[
      NCache[{{Rational[2, 3], Rational[2, 3], Rational[-1, 3]}, {
        Rational[1, 2] + Rational[-1, 8] 2^Rational[-1, 2]/GoldenRatio, 
         Rational[1, 2] + Rational[1, 8] 2^Rational[-1, 2]/GoldenRatio, 
         Rational[-1, 4]}, {
        Rational[1, 2] + Rational[-1, 16] 2^Rational[-1, 2]/GoldenRatio + 
         Rational[1, 16] 6^Rational[-1, 2]/GoldenRatio, Rational[1, 2] + 
         Rational[1, 16] 2^Rational[-1, 2]/GoldenRatio + 
         Rational[1, 16] 6^Rational[-1, 2]/GoldenRatio, Rational[-1, 4] + 
         Rational[1, 4] 6^Rational[-1, 2]/GoldenRatio}}, {{0.6666666666666666,
        0.6666666666666666, -0.3333333333333333}, {0.44537299694389737`, 
       0.5546270030561026, -0.25}, {0.4884559559316804, 
       0.543082958987783, -0.18692217016107326`}}]], 
     Polygon3DBox[
      NCache[{{Rational[2, 3], Rational[2, 3], Rational[-1, 3]}, {
        Rational[1, 2] + Rational[-1, 16] 2^Rational[-1, 2]/GoldenRatio + 
         Rational[1, 16] 6^Rational[-1, 2]/GoldenRatio, Rational[1, 2] + 
         Rational[1, 16] 2^Rational[-1, 2]/GoldenRatio + 
         Rational[1, 16] 6^Rational[-1, 2]/GoldenRatio, Rational[-1, 4] + 
         Rational[1, 4] 6^Rational[-1, 2]/GoldenRatio}, {
        Rational[1, 2] + Rational[1, 16] 2^Rational[-1, 2]/GoldenRatio + 
         Rational[1, 16] 6^Rational[-1, 2]/GoldenRatio, Rational[1, 2] + 
         Rational[-1, 16] 2^Rational[-1, 2]/GoldenRatio + 
         Rational[1, 16] 6^Rational[-1, 2]/GoldenRatio, Rational[-1, 4] + 
         Rational[1, 4] 6^Rational[-1, 2]/GoldenRatio}}, {{0.6666666666666666,
        0.6666666666666666, -0.3333333333333333}, {0.4884559559316804, 
       0.543082958987783, -0.18692217016107326`}, {0.543082958987783, 
       0.4884559559316804, -0.18692217016107326`}}]], 
     Polygon3DBox[
      NCache[{{Rational[2, 3], Rational[2, 3], Rational[-1, 3]}, {
        Rational[1, 2] + Rational[1, 16] 2^Rational[-1, 2]/GoldenRatio + 
         Rational[1, 16] 6^Rational[-1, 2]/GoldenRatio, Rational[1, 2] + 
         Rational[-1, 16] 2^Rational[-1, 2]/GoldenRatio + 
         Rational[1, 16] 6^Rational[-1, 2]/GoldenRatio, Rational[-1, 4] + 
         Rational[1, 4] 6^Rational[-1, 2]/GoldenRatio}, {
        Rational[1, 2] + Rational[1, 8] 2^Rational[-1, 2]/GoldenRatio, 
         Rational[1, 2] + Rational[-1, 8] 2^Rational[-1, 2]/GoldenRatio, 
         Rational[-1, 4]}}, {{0.6666666666666666, 
       0.6666666666666666, -0.3333333333333333}, {0.543082958987783, 
       0.4884559559316804, -0.18692217016107326`}, {0.5546270030561026, 
       0.44537299694389737`, -0.25}}]]}}, {
    {GrayLevel[0.5], Line3DBox[CompressedData["
1:eJxN2Hk41NsfB3BLCJHqhhBKt7InQsJxs1SIsodCtla7riUTBoWEsqYQQvZ9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      "]], Line3DBox[CompressedData["
1:eJxN2Hk0ldv/B3BJ4lLGMiZuokQUEqktJKHj3kiiiERJSuaQzEVEGTMcQiqZ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      "]], Line3DBox[CompressedData["

1:eJxN2Hk4VVv4B3CKhFCOses2ikR1E6JiSSFForgVCUkDDSRDJUNlijKXKUVF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      "]], 
     Line3DBox[CompressedData["
1:eJxN2Hk41Nv/AHBJIjdabilXpUIqqcjW4ESlqBCylLImS5RMuLYwCil7Icoa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      "]], 
     Line3DBox[CompressedData["
1:eJxN2Hk4VN//AHA7kRahSCpLklJ8UkidIhGSYpI2UlKKRBRJJKVEkUTZl+yy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      "]], Line3DBox[CompressedData["

1:eJxN2Hk4lNsfAPAhSyRJKWWrKHQVkrJMjlAqEoPSr6RS2rgNCeG26CJbN0si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      "]], 
     Line3DBox[CompressedData["
1:eJxN2Gk0Vl0bB/BCgwyVKRRCeiJzylA9W4ZkaDKmQjKmpJA0SAplTBlKGTKX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      "]], 
     Line3DBox[CompressedData["
1:eJxN2Gk4lU0fAHCpKEVKhERCKWRNRRlLluhEyZZIEkkoxGOJbJUlCi1UCJVI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      "]], 
     Line3DBox[CompressedData["
1:eJxN2nc41t//B3CKhCYZlU9bQxpURuRISYNEkiIhqygrq5JVVpkZWVG27FEU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      "]], Line3DBox[CompressedData["
1:eJxN2Gk41F0bAHCpKEUiQiKhFLKmohxLlmiiZEskiSQU4rFkV1mi0EKFUIlk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      "]], 
     Line3DBox[CompressedData["
1:eJxN2HlUTV8bB3AqQxpQSUWlkp/SLBrw22mQJkOjUKRRJCrJkERFo2ggmkdS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      "]], 
     Line3DBox[CompressedData["
1:eJxN2Hk4lNsfAPAhSySVUspWUegqJGWZHKFUJAalX6FS2riNXbgtusjWLSQi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      "]], 
     Line3DBox[CompressedData["
1:eJxN2Hk4VN//AHBLlkiLLJFUliRRfFJInSIRksokqYgsKRJRJGlRShRJlH3J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      "]], 
     Line3DBox[CompressedData["
1:eJxN2Hk41Nv/AHBJIjdabilXpUIqqcjW4ESlqBCylLImS5RMuLYwCil7Icoa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      "]], 
     Line3DBox[CompressedData["
1:eJxN2Hk4VVv4B3CKhFCOses2ikR1E6JiSSFForgVCUkDDSRDJUNlijKXKUVF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      "]], 
     Line3DBox[CompressedData["
1:eJxN2Hk0ldv/B3AhcSljGRM3pUQUEqkthNBxbyRRRKIkJXNIhlJElDHDUUgy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      "]], Line3DBox[CompressedData["
1:eJxN2Hk41NsfB3BLCJHqhhBKt7InQsJxs1SIsodCtla7riUTBoWEsqYQQvZ9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      "]]}, {
     {GrayLevel[0.4], Thickness[0.005], 
      Line3DBox[{{-1., 0, 0}, {1., 0, 0}}]}, 
     {GrayLevel[0.4], Thickness[0.005], 
      Line3DBox[{{0, -1., 0}, {0, 1., 0}}]}, 
     {GrayLevel[0.4], Thickness[0.005], 
      Line3DBox[{{0, 0, -1.}, {0, 0, 1.}}]}}, 
    {RGBColor[0, 0, 1], PointSize[0.02], Thickness[0.003], 
     Point3DBox[
      NCache[{Rational[2, 3], Rational[2, 3], Rational[-1, 3]}, {
       0.6666666666666666, 0.6666666666666666, -0.3333333333333333}]], 
     Line3DBox[CompressedData["
1:eJxN1ns8VGkfAHBSJNVEWLdWloxpthqhpHgitXhLuUul0m1bWVIJobeLbghJ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      "]], 
     Line3DBox[CompressedData["
1:eJw92Ak0VP0fx3HZSh5UDyFLe2QvpfTEtwVFthItlKlsSX+0EIpC1opUlLJW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      "]]}}},
  Axes->False,
  Boxed->False]], "Output",
 CellChangeTimes->{
  3.390233718161047*^9, 3.3902924607692757`*^9, {3.390293844476983*^9, 
   3.3902938703376923`*^9}, {3.390293946442925*^9, 3.3902939607312593`*^9}, 
   3.390294235622725*^9, 3.390294365183837*^9, {3.3902944932155333`*^9, 
   3.3902945910148563`*^9}, 3.390294676764833*^9, 3.3917813883329363`*^9, 
   3.391850714765625*^9, 3.391850846921875*^9, 3.392472619863587*^9, 
   3.3924726911836987`*^9, 3.393850548636591*^9, 3.393851032923892*^9, 
   3.3938551300322638`*^9}]
}, Open  ]],

Cell["Additional graphics elements may sometimes be helpful", "Text"],

Cell[CellGroupData[{

Cell["\<\
QVisualizeVector[{1,1,1}/Sqrt[3],QCoordinateCube->True, \
QCoordinateCircles->False, QDrawUnitSphere->10, QDrawAxes->{True,False,True}]\
\
\>", "Input",
 CellChangeTimes->{{3.39029469726896*^9, 3.390294706877055*^9}}],

Cell[BoxData[
 Graphics3DBox[{{
    {Thickness[0.001], 
     Line3DBox[
      NCache[{{0, 0, 0}, {
        3^Rational[-1, 2], 3^Rational[-1, 2], 3^Rational[-1, 2]}}, {{0, 0, 
        0}, {0.5773502691896258, 0.5773502691896258, 
        0.5773502691896258}}]]}, 
    {GrayLevel[1], 
     Polygon3DBox[
      NCache[{{3^Rational[-1, 2], 3^Rational[-1, 2], 3^Rational[-1, 2]}, {
        Rational[1, 4] 3^Rational[1, 2] + 
         Rational[1, 8] 2^Rational[-1, 2]/GoldenRatio, 
         Rational[1, 4] 3^Rational[1, 2] + 
         Rational[-1, 8] 2^Rational[-1, 2]/GoldenRatio, Rational[1, 4] 
         3^Rational[1, 2]}, {
        Rational[1, 4] 3^Rational[1, 2] + 
         Rational[1, 8] 2^Rational[-1, 2]/GoldenRatio, Rational[1, 4] 
         3^Rational[1, 2], Rational[1, 4] 3^Rational[1, 2] + 
         Rational[-1, 8] 2^Rational[-1, 2]/GoldenRatio}}, {{
       0.5773502691896258, 0.5773502691896258, 0.5773502691896258}, {
       0.48763970494832193`, 0.37838569883611667`, 0.4330127018922193}, {
       0.48763970494832193`, 0.4330127018922193, 0.37838569883611667`}}]], 
     Polygon3DBox[
      NCache[{{3^Rational[-1, 2], 3^Rational[-1, 2], 3^Rational[-1, 2]}, {
        Rational[1, 4] 3^Rational[1, 2] + 
         Rational[1, 8] 2^Rational[-1, 2]/GoldenRatio, Rational[1, 4] 
         3^Rational[1, 2], Rational[1, 4] 3^Rational[1, 2] + 
         Rational[-1, 8] 2^Rational[-1, 2]/GoldenRatio}, {
        Rational[1, 4] 3^Rational[1, 2], Rational[1, 4] 3^Rational[1, 2] + 
         Rational[1, 8] 2^Rational[-1, 2]/GoldenRatio, 
         Rational[1, 4] 3^Rational[1, 2] + 
         Rational[-1, 8] 2^Rational[-1, 2]/GoldenRatio}}, {{
       0.5773502691896258, 0.5773502691896258, 0.5773502691896258}, {
       0.48763970494832193`, 0.4330127018922193, 0.37838569883611667`}, {
       0.4330127018922193, 0.48763970494832193`, 0.37838569883611667`}}]], 
     Polygon3DBox[
      NCache[{{3^Rational[-1, 2], 3^Rational[-1, 2], 3^Rational[-1, 2]}, {
        Rational[1, 4] 3^Rational[1, 2], Rational[1, 4] 3^Rational[1, 2] + 
         Rational[1, 8] 2^Rational[-1, 2]/GoldenRatio, 
         Rational[1, 4] 3^Rational[1, 2] + 
         Rational[-1, 8] 2^Rational[-1, 2]/GoldenRatio}, {
        Rational[1, 4] 3^Rational[1, 2] + 
         Rational[-1, 8] 2^Rational[-1, 2]/GoldenRatio, 
         Rational[1, 4] 3^Rational[1, 2] + 
         Rational[1, 8] 2^Rational[-1, 2]/GoldenRatio, Rational[1, 4] 
         3^Rational[1, 2]}}, {{0.5773502691896258, 0.5773502691896258, 
       0.5773502691896258}, {0.4330127018922193, 0.48763970494832193`, 
       0.37838569883611667`}, {0.37838569883611667`, 0.48763970494832193`, 
       0.4330127018922193}}]], 
     Polygon3DBox[
      NCache[{{3^Rational[-1, 2], 3^Rational[-1, 2], 3^Rational[-1, 2]}, {
        Rational[1, 4] 3^Rational[1, 2] + 
         Rational[-1, 8] 2^Rational[-1, 2]/GoldenRatio, 
         Rational[1, 4] 3^Rational[1, 2] + 
         Rational[1, 8] 2^Rational[-1, 2]/GoldenRatio, Rational[1, 4] 
         3^Rational[1, 2]}, {
        Rational[1, 4] 3^Rational[1, 2] + 
         Rational[-1, 8] 2^Rational[-1, 2]/GoldenRatio, Rational[1, 4] 
         3^Rational[1, 2], Rational[1, 4] 3^Rational[1, 2] + 
         Rational[1, 8] 2^Rational[-1, 2]/GoldenRatio}}, {{0.5773502691896258,
        0.5773502691896258, 0.5773502691896258}, {0.37838569883611667`, 
       0.48763970494832193`, 0.4330127018922193}, {0.37838569883611667`, 
       0.4330127018922193, 0.48763970494832193`}}]], 
     Polygon3DBox[
      NCache[{{3^Rational[-1, 2], 3^Rational[-1, 2], 3^Rational[-1, 2]}, {
        Rational[1, 4] 3^Rational[1, 2] + 
         Rational[-1, 8] 2^Rational[-1, 2]/GoldenRatio, Rational[1, 4] 
         3^Rational[1, 2], Rational[1, 4] 3^Rational[1, 2] + 
         Rational[1, 8] 2^Rational[-1, 2]/GoldenRatio}, {
        Rational[1, 4] 3^Rational[1, 2], Rational[1, 4] 3^Rational[1, 2] + 
         Rational[-1, 8] 2^Rational[-1, 2]/GoldenRatio, 
         Rational[1, 4] 3^Rational[1, 2] + 
         Rational[1, 8] 2^Rational[-1, 2]/GoldenRatio}}, {{0.5773502691896258,
        0.5773502691896258, 0.5773502691896258}, {0.37838569883611667`, 
       0.4330127018922193, 0.48763970494832193`}, {0.4330127018922193, 
       0.37838569883611667`, 0.48763970494832193`}}]], 
     Polygon3DBox[
      NCache[{{3^Rational[-1, 2], 3^Rational[-1, 2], 3^Rational[-1, 2]}, {
        Rational[1, 4] 3^Rational[1, 2], Rational[1, 4] 3^Rational[1, 2] + 
         Rational[-1, 8] 2^Rational[-1, 2]/GoldenRatio, 
         Rational[1, 4] 3^Rational[1, 2] + 
         Rational[1, 8] 2^Rational[-1, 2]/GoldenRatio}, {
        Rational[1, 4] 3^Rational[1, 2] + 
         Rational[1, 8] 2^Rational[-1, 2]/GoldenRatio, 
         Rational[1, 4] 3^Rational[1, 2] + 
         Rational[-1, 8] 2^Rational[-1, 2]/GoldenRatio, Rational[1, 4] 
         3^Rational[1, 2]}}, {{0.5773502691896258, 0.5773502691896258, 
       0.5773502691896258}, {0.4330127018922193, 0.37838569883611667`, 
       0.48763970494832193`}, {0.48763970494832193`, 0.37838569883611667`, 
       0.4330127018922193}}]]}}, {
    {GrayLevel[0.5], Line3DBox[CompressedData["
1:eJxN2Hk4lNsfAHClbgvJUirSJoXcwlVIOZHshIuKCkUlSZIQkex7skYiS4lk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      "]], 
     Line3DBox[CompressedData["
1:eJxN2Hk41Nv/AHBlTaWyVCJZSoo2UhQdKi10KVKS5ba4JFcJiWsP2SNElmSJ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      "]], Line3DBox[CompressedData["
1:eJxN2Hk0Vd3fAPAiQ4kKJT0JqUhECpnaEZmKiowlU6JoQCQkkTJWKEUyD5GI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      "]], Line3DBox[CompressedData["
1:eJxN2Gkgldn/AHBSllBaUAlZkrXNNkacsZZJI6WQROLaIoVEaJHK1oKUKZIQ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      "]], 
     Line3DBox[CompressedData["
1:eJxN2nc41t//B3CKhCYZlU9bQxpURuRISYNEkiIhqygrq5JVVpkZWVG27FEU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      "]], Line3DBox[CompressedData["
1:eJxN2Gkgldn/AHBSllBaUAlZkrXNNkacsZZJI6UoicS1RQqJkJKytiBliiSE
LFkSWY7se7bsy8W93PWRGMtI/r34/86ZF8+bz4t7n+ec7/ku59RVeylHDg6O
db+eoV8PFyCubtpmaLGiaye5Pd3w5C7YXT+jeYB+EhAe8gqa9gogOqancU/W
JHa3G36Sbx+CxCdz5/fsILCTQqJiRIpAeouLyP65o9htvRtvLu0Aua0GH3sF
E7GbvpEQ7/QExcMymivegdhVt/qlLV4ClfvM28IvjWLfTKJPzPqBmvWXY3RC
C5Gzp/L/9FTnAvU5e6i1nVrYC8IE+vSdQHOSolNc5n3svkdCy2OWQLvPo2gd
ajD236pzNR5rgk7Oir0xxqrIWWyDpQBJR9DFiE726+zB/iou5kaoMejJPTCo
WnIMu2nFk9SnbNDrsPv44bg45EzmDeusZ7qgX8uhoDTmb+zhvcthSQAM0M/2
zkXTse94Gzls1QqGil+4SLIVkTNSVG8vz7qBEa9vrkkL5thVTOZMZMbAmMDe
+Vk5vI/0t0/dZGkvAdnWKpHjYzZ2WQ+uE3MMMC4nmpjULYKc9ihMqkhxAxhf
6lC5opeCfHohJV44rw9MVDcf9JXixW6/N6Rznz+Y9E8VmsrMQT71aSaIg+c2
oBxp/BIf+Q/2nVKfIvTuAwrrZ6Ojoglyqs/ptA+uvIAa18/FI1qMnFKxFnhE
cCOYMjJ8W63shF2wjZwy3wamKO58uVbKyCdNblTb1K0H0xF1jWTBJuQTIb5B
T5bzAW3fe+7F5d+Qj9cfvyyrRABa0UF1PwofcvL8dW9auCSgG3PtrJl/jl35
bOuIMQD0hqHy6zmvkI82tgf+q7wGGHqy4bH625CPbJqocozVBYwK6mHg9Ab5
kNs1ftOVfMBUqpzMfGmFfCDRQ//aNS3AfBGj41mwD3nf9FP5vSG3AXN1S+7r
GLyevWr5b0g2CoB1dtu+pDP4e3ssZU4kaG8ArDx5R7caOeRdcQ4L9INjgLWi
4U6LKUXekf79icGLVsA26gIyfdbI225Nm/rwugP2A1GrAcMvyJsopLOtjjaA
Xd7MMXpMH3ndUhivH0MPsGfE7f9Uxl5t0hUVmBMJCJEUc2dxHM8Vpz6x2GIM
QGhYvhuUi0f+4Yq3XOFhG0Cc/Posxusb8vxP7fymuTyAOH+xfIh0D3m2kLrP
ovV2QNiLcZ6bw/khdePWCKUteoBw9nZgfTuA/PmG7UcjPzoA4kre2TqGLvLg
21yiBCj/lf/qI3qs3v/PYdCuW8tRC5a/vCvUznQj8vgj56M1lgQA4c4tba2N
9gWm3HRoMtsYBAiX65/E4qqQZ3mGyMbrSADi8qNOTxFL5PmqzmrhJYGAsI63
Ym3zR16SlNQcFysGiKMSzYYZKH5g1cnRWCtWHiDkq+sYpo+Q12qbjdPqDgFi
XVFI5K0R5A0jiXf/EmoH7L7RbavnFpG3NHQq/i6ZC9iv/c7fCu1H/sU5rSl0
wgSwnYw4JaNeI+/0nQ2ZDvkXsPf9NR5flo+820bvltcMJ2ANGtp29eHf/yru
pMnLXwVYkX+8F1VH9QL2rjr+beFAByyVOU6N6HTk/e2X/ikWNwbM5txo0pEb
yAf9RaJTP1YAptOZ11+dUb6Cw/rDf5je2QwY85YjKX7qyEemgGzgaQZg3D22
zD0SinzM3xmMt0YBhoDJQzgiiJwcd56ftCsU0EO9JUufcCAfP8dHqoc/AZ3T
Pub+7u/IJxQSHJMSnQHNLeK2+Rze34npNmFufxUwPeDvWOv7EflkYnc871AW
mDbMYCicTkJOuShspESyBlOJ4rNlQsPIqQKBpyzddQB1RVbubscc9pqWpzEH
pQHVqIDia4PONZy61pUjdk4EUKI/C5ptv4Z8elOhm80pWzA5IislZfgWe0FG
ZiT0AJM7bN19Dvohpxl1mVvGmYMJ68nMXWXPsHe42JtlrILxpC/GAQxUHyHd
Tp+/lFkKyG1C7LIJNexk8INHohKQN6jeMlMcRM5w4Pmr5mQjGD0Zo1amfAr7
6JEi/fRDYDg+Nvs48zJypsmH9KvHvcDgnINUrHkx9o8bh3nDm8HA3my/lQ9R
yFkiv1lJOB8DfZ5XElhqztiv/PAwdx0HXystRiV2/sdrSr9tWP8H6B5PVomQ
x/vOFkxSn7nIB7okV63jNdKwn1lS90puBB2qJQYZu/Owx6i4U2bSQWvDj0tW
3nh92PV6ZKfOTNCkOkffuTyDfYFBVjh8BtQdLwjWoeI4JHYTvepn1sDnZOrX
UCqOK0KXe8tPshwoM3qxg/KmA/tZY+XxnL2g0Alw06r/wH6xSU9i6QTI80q1
dbVZwO4YMr5qQwcZIrM/kgunsLu23Ct1uAeSdxbTugwysbsvv7n2eB94rFGZ
nOxnhf2qwNk5qQVg8fDm/pUiKnaPP81sgQKM/JO7cbAQxw/hVrc3YP4mfGm8
vGezfD12p8CCAKFUmOaVLjce+p/vshuW+iz0HOZcum0mGliC3UpBx9GeGxak
Z+snpm7Bru9Eod+Og5949EZcWxKwS5c60/g0YNVa9ljG8Ae8/is+SiX8y7CW
4lfsLILzHvvLU4vC0G7YeEfYvcDnAfbnUpVCm+/CVkcnUrWXAHabjYI/3vfC
Dg7loyUtBtiFt5ccvm0HO2dGXr7tUcDxVn9rWUexEnZ3pqwIhV7CfkOar1Zd
E359aLDz+3tO7KLXeB7USsI+18drTpIBOP5LONqq92jDAamrHVJ5ldhPf0uo
SA2Cg01tLkfzvPH5ImuytyuswOFHki/TejSxuycwfrNdhaMHZGWTP+B4ozOO
tUpuboJjdCpdopobu715uVpWLCQ3zQVvvYi/lzZyJnWeuQLHox6qKJn8x40j
XiR8K4ET52cWO5xxfp4uLPdw/+cRnBTl2627zRS7xPGWYqoHnGxe8cmxd8f5
7cbu11y6MpASvkWsmDSL82F33jbXD98gVd2Rj4sug128IV321TpIZYT5ce5C
fS+keCTOdmhtglMP95cSvGiugZOV2+WT6vXgtIJ5UXdFBM7ny46Nszpf4XRL
AMGMwus/oSPt751lAWkXFmq3/G2E68UVjYXVXfyQNs8Xf/d31J9A8t/n9N7A
E5B+J7qloa0W16M0Qd3F7wKQvmbtUhmFz9cI8/Tih0JByAge5JLzDcP17qi3
FtvqNWTMyQS9D9+P62Oa5sab1zZC5gWy7SMtKVxP+wPjDuSEQGbjlWdKmyeQ
98l6HRJRqYSsvfOc85Y8uF4bpar+uLofskIakileeri+u7SXNKgIQFa/vP+h
Utz/dGaaVt8hyUG25Ojn1YAi3D8EeFPTuzIg23F2TUUrC/cbY0PeURo5kJ0s
JNynjeYy2EBEmRYo5EN23wYx1wDUf8JaPv7rBd1jkFjfwRUZhd+n6lIgw++e
DCTkDyckXMTvU9LWJHtKVRcSxvfaeYRwfck/d/5z7j89kLAGS/YvcVy9s/M4
fWP5DiTsyy76nMN5LFXg0USN3m5IOF+vagvF/vxghbH1l3hIXEl/T4t3RR4c
Tcpw8s3/lf/qBbuISNRP+ps107ieP/vl5DgNf9Q/gKceakG3loZ/5Vcl8VJb
PEenVL83GSpohYSLEVgKeoo8q3WBdKCEGxKXdXtWnrJxP2w5U7fslfnruww0
BNTEcf98TIurn3EBEubaWal5ucjLE9UkCk7KQUJrc+024VXcnxt6mz21L4CE
SG+QZjuKZ1D33enyajc/ZM84nnDt3YT7/7AiycjH1ZD9uXL/XSX8v20kuwt2
ws2QHeYB1+yYyDsCzdqHeBMh+3ixKMkRIO9yuZX/TtkTstdtMl8dx/NCz9Hs
gys6EpCVTxWpf7iMvJejwsPp4++QZacqve10NPK+gZPPPFzcIYtj990DBuHI
B6LuWPM+eQuZrwILv2uj+giGbBTFBuJpkHnAsiM8A8+5I+KUl2T3dsgo7VAa
SEZ9AhgtdfWpna6HDKO0lWluPKeTNYsKblaNQHpVcVV+MMrzYJxjj0eKjDWk
6yqcHS6A2KteX1x/igfSSvRD+5ZvIp+I+mJy/9/LkLYnZNmi+D/zrHFjf+Fz
QTgdteibFYviDVD4lR5k96rCqamyO0WhZdjr7y97+q+DUzpmizJNOA6pnrr8
snbvIDWByiFNoPwJpqQT714Q64WUiQV3U4cJ7DXJFtxinJCiylmwWDOGfPqC
4tWaG7Vw8s6XZrvv+H2mV565t2/6DifKvWosX3Yip0XcFHXxYsLxtdJTOiQ0
pwC6RFt67tb1cHy/he+VMrw+9Dfr7YrX+0AyKUjbhe8rcobEy5k+na1wTOdq
sMnBrdgT22xFjPTgSEiP3OEwNDcBpuCM9vCQHhwi2wXFtulhD1aQP6JoAQe3
WtT1H8H3Qkzal4p2og/2GyW82pCwhJxlkCqhZXwP9j6Z+qR6H+8XK+mVVbPo
YdhTGPDpJ0kQO8v99gM3O9g1k10l0YbqIGAfms2Yz+uHnQfNLaXW8HuyfUrN
NvqxYHvUjAXTHp8jdk65mJhNOmye0674Pa4V+0RQe5pNCmxYv/rG7R2+PyE2
Hdr7zlMV1khTWtTW4XsA4kC2sEn7MVj5U7003xjfFxGmXGIP3u+EH4WYW/T9
8b0HcXHecPjcZphnshYf8A3fdxGk/celrRbh2yPcWjoajtjdjO5YkPlgktXX
/gTZS9g9xBgUWiZ8eK7J2k4Sx+H/3zdW2kluTxNX/fQ//z+0HI9c
      "]], 
     Line3DBox[CompressedData["
1:eJxN2Hk0Vd3fAPAiQ4kKJT0JqUhECpnaEZmKiowlU6JoQCQkkTJWKEUyD5GI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      "]], Line3DBox[CompressedData["
1:eJxN2Hk41Nv/AHBlTaWyVCJZSoo2UhQdKi10KVKS5ba4JFcJiWsP2SNElmSJ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      "]], 
     Line3DBox[CompressedData["
1:eJxN2Hk4lNsfAHClbgvJUirSJoXcwlVIOZHshIuKCkUlSZIQkex7skYiS4lk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      "]]}, 
    {GrayLevel[0.4], Thickness[0.005], 
     Line3DBox[{{-1., 0, 0}, {1., 0, 0}}]}, 
    {GrayLevel[0.4], Thickness[0.005], 
     Line3DBox[{{0, 0, -1.}, {0, 0, 1.}}]}, 
    {RGBColor[0, 0.6, 0.4], 
     Line3DBox[
      NCache[{{0, 0, 0}, {3^Rational[-1, 2], 0, 0}, {
        3^Rational[-1, 2], 3^Rational[-1, 2], 0}, {
        3^Rational[-1, 2], 3^Rational[-1, 2], 3^Rational[-1, 2]}, {
        3^Rational[-1, 2], 0, 3^Rational[-1, 2]}, {
        0, 0, 3^Rational[-1, 2]}}, {{0, 0, 0}, {0.5773502691896258, 0, 0}, {
        0.5773502691896258, 0.5773502691896258, 0}, {0.5773502691896258, 
        0.5773502691896258, 0.5773502691896258}, {
        0.5773502691896258, 0, 0.5773502691896258}, {
        0, 0, 0.5773502691896258}}]], 
     Line3DBox[
      NCache[{{0, 0, 0}, {0, 3^Rational[-1, 2], 0}, {
        3^Rational[-1, 2], 3^Rational[-1, 2], 0}, {
        3^Rational[-1, 2], 3^Rational[-1, 2], 3^Rational[-1, 2]}, {
        0, 3^Rational[-1, 2], 3^Rational[-1, 2]}, {
        0, 0, 3^Rational[-1, 2]}}, {{0, 0, 0}, {0, 0.5773502691896258, 0}, {
        0.5773502691896258, 0.5773502691896258, 0}, {0.5773502691896258, 
        0.5773502691896258, 0.5773502691896258}, {
        0, 0.5773502691896258, 0.5773502691896258}, {
        0, 0, 0.5773502691896258}}]], 
     Line3DBox[
      NCache[{{3^Rational[-1, 2], 0, 0}, {
        3^Rational[-1, 2], 0, 3^Rational[-1, 2]}}, {{
        0.5773502691896258, 0, 0}, {
        0.5773502691896258, 0, 0.5773502691896258}}]], 
     Line3DBox[
      NCache[{{0, 3^Rational[-1, 2], 0}, {
        0, 3^Rational[-1, 2], 3^Rational[-1, 2]}}, {{
        0, 0.5773502691896258, 0}, {
        0, 0.5773502691896258, 0.5773502691896258}}]]}}},
  Axes->False,
  Boxed->False]], "Output",
 CellChangeTimes->{
  3.3902337434155073`*^9, {3.390294680144495*^9, 3.390294708067894*^9}, 
   3.391781388646299*^9, 3.3918507168125*^9, 3.391850848953125*^9, 
   3.3924726205782557`*^9, 3.3924726916994247`*^9, 3.3938505489097757`*^9, 
   3.393851033201085*^9, 3.3938551347970657`*^9}]
}, Open  ]],

Cell["A Density matrix can also be visualized as a vector.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"dm", " ", "=", " ", 
  RowBox[{"QVectorToDensityMatrix", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1.", ",", 
      RowBox[{"-", ".5"}], ",", "2"}], "}"}], "//", "QNormalize"}], 
   "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0.9364357804719847`", ",", 
     RowBox[{"0.2182178902359924`", "\[InvisibleSpace]", "+", 
      RowBox[{"0.1091089451179962`", " ", "\[ImaginaryI]"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"0.2182178902359924`", "\[InvisibleSpace]", "-", 
      RowBox[{"0.1091089451179962`", " ", "\[ImaginaryI]"}]}], ",", 
     "0.06356421952801522`"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.3902337566033573`*^9, 3.390294711049708*^9, 
  3.391781388669965*^9, 3.3918507171875*^9, 3.39185084921875*^9, 
  3.392472620593741*^9, 3.3924726919826107`*^9, 3.393850548947022*^9, 
  3.3938510332284117`*^9, 3.393855141432623*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
QVisualizeDensityMatrix[dm, QArrowShape->{6,1/3}, QLinePointSize->.01]\
\>", "Input",
 CellChangeTimes->{{3.3902337888867903`*^9, 3.3902338148488903`*^9}}],

Cell[BoxData[
 Graphics3DBox[{{
    {Thickness[0.01], 
     Line3DBox[{{0, 0, 0}, {0.4364357804719848, -0.2182178902359924, 
       0.8728715609439694}}]}, 
    {GrayLevel[1], 
     Polygon3DBox[{{0.4364357804719848, -0.2182178902359924, 
      0.8728715609439694}, {0.24489165327298648`, -0.23760966090733526`, 
      0.5819143739626462}, {0.33756896081499616`, -0.2263663975429191, 
      0.5383865360327456}}], 
     Polygon3DBox[{{0.4364357804719848, -0.2182178902359924, 
      0.8728715609439694}, {0.33756896081499616`, -0.2263663975429191, 
      0.5383865360327456}, {0.38363449452333304`, -0.13423533012624547`, 
      0.5383865360327456}}], 
     Polygon3DBox[{{0.4364357804719848, -0.2182178902359924, 
      0.8728715609439694}, {0.38363449452333304`, -0.13423533012624547`, 
      0.5383865360327456}, {0.33702272068966016`, -0.05334752607398799, 
      0.5819143739626462}}], 
     Polygon3DBox[{{0.4364357804719848, -0.2182178902359924, 
      0.8728715609439694}, {0.33702272068966016`, -0.05334752607398799, 
      0.5819143739626462}, {0.24434541314765046`, -0.06459078943840416, 
      0.6254422118925471}}], 
     Polygon3DBox[{{0.4364357804719848, -0.2182178902359924, 
      0.8728715609439694}, {0.24434541314765046`, -0.06459078943840416, 
      0.6254422118925471}, {0.1982798794393136, -0.1567218568550778, 
      0.6254422118925471}}], 
     Polygon3DBox[{{0.4364357804719848, -0.2182178902359924, 
      0.8728715609439694}, {0.1982798794393136, -0.1567218568550778, 
      0.6254422118925471}, {0.24489165327298648`, -0.23760966090733526`, 
      0.5819143739626462}}]}}, {
    {GrayLevel[0.5], Line3DBox[CompressedData["
1:eJxN2Hk41NsfB3BLCJHqhhBKt7InQsJxs1SIsodCtla7riUTBoWEsqYQQvZ9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      "]], Line3DBox[CompressedData["
1:eJxN2Hk0ldv/B3BJ4lLGMiZuokQUEqktJKHj3kiiiERJSuaQzEVEGTMcQiqZ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      "]], Line3DBox[CompressedData["

1:eJxN2Hk4VVv4B3CKhFCOses2ikR1E6JiSSFForgVCUkDDSRDJUNlijKXKUVF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      "]], 
     Line3DBox[CompressedData["
1:eJxN2Hk41Nv/AHBJIjdabilXpUIqqcjW4ESlqBCylLImS5RMuLYwCil7Icoa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      "]], 
     Line3DBox[CompressedData["
1:eJxN2Hk4VN//AHA7kRahSCpLklJ8UkidIhGSYpI2UlKKRBRJJKVEkUTZl+yy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      "]], Line3DBox[CompressedData["

1:eJxN2Hk4lNsfAPAhSyRJKWWrKHQVkrJMjlAqEoPSr6RS2rgNCeG26CJbN0si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      "]], 
     Line3DBox[CompressedData["
1:eJxN2Gk0Vl0bB/BCgwyVKRRCeiJzylA9W4ZkaDKmQjKmpJA0SAplTBlKGTKX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      "]], 
     Line3DBox[CompressedData["
1:eJxN2Gk4lU0fAHCpKEVKhERCKWRNRRlLluhEyZZIEkkoxGOJbJUlCi1UCJVI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      "]], 
     Line3DBox[CompressedData["
1:eJxN2nc41t//B3CKhCYZlU9bQxpURuRISYNEkiIhqygrq5JVVpkZWVG27FEU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      "]], Line3DBox[CompressedData["
1:eJxN2Gk41F0bAHCpKEUiQiKhFLKmohxLlmiiZEskiSQU4rFkV1mi0EKFUIlk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      "]], 
     Line3DBox[CompressedData["
1:eJxN2HlUTV8bB3AqQxpQSUWlkp/SLBrw22mQJkOjUKRRJCrJkERFo2ggmkdS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      "]], 
     Line3DBox[CompressedData["
1:eJxN2Hk4lNsfAPAhSySVUspWUegqJGWZHKFUJAalX6FS2riNXbgtusjWLSQi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      "]], 
     Line3DBox[CompressedData["
1:eJxN2Hk4VN//AHBLlkiLLJFUliRRfFJInSIRksokqYgsKRJRJGlRShRJlH3J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      "]], 
     Line3DBox[CompressedData["
1:eJxN2Hk41Nv/AHBJIjdabilXpUIqqcjW4ESlqBCylLImS5RMuLYwCil7Icoa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      "]], 
     Line3DBox[CompressedData["
1:eJxN2Hk4VVv4B3CKhFCOses2ikR1E6JiSSFForgVCUkDDSRDJUNlijKXKUVF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      "]], 
     Line3DBox[CompressedData["
1:eJxN2Hk0ldv/B3AhcSljGRM3pUQUEqkthNBxbyRRRKIkJXNIhlJElDHDUUgy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      "]], Line3DBox[CompressedData["
1:eJxN2Hk41NsfB3BLCJHqhhBKt7InQsJxs1SIsodCtla7riUTBoWEsqYQQvZ9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      "]]}, {
     {GrayLevel[0.4], Thickness[0.005], 
      Line3DBox[{{-1., 0, 0}, {1., 0, 0}}]}, 
     {GrayLevel[0.4], Thickness[0.005], 
      Line3DBox[{{0, -1., 0}, {0, 1., 0}}]}, 
     {GrayLevel[0.4], Thickness[0.005], 
      Line3DBox[{{0, 0, -1.}, {0, 0, 1.}}]}}, 
    {RGBColor[0, 0, 1], PointSize[0.02], Thickness[0.003], 
     Point3DBox[{0.4364357804719848, -0.2182178902359924, 
      0.8728715609439694}], Line3DBox[CompressedData["
1:eJxN1mtcjNseB3CJFOlCVOzakiipNI2hybGI4ogtidJdKXQj7Yho0JwuDkq1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      "]], 
     Line3DBox[CompressedData["
1:eJw12HtUjdkbwHGVKMm4k3tUKJVENOQxFSZdDCNULkVhVAiZ0V2pSOmConuo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      "]]}}},
  Axes->False,
  Boxed->False]], "Output",
 CellChangeTimes->{
  3.390233757821763*^9, {3.390233789476882*^9, 3.390233815284246*^9}, 
   3.390294712597361*^9, 3.391781389304351*^9, 3.39185071825*^9, 
   3.39185085128125*^9, 3.392472621093131*^9, 3.392472692408518*^9, 
   3.393850549373179*^9, 3.393851033625442*^9, 3.393855143439775*^9}]
}, Open  ]],

Cell[TextData[{
 "Options for ",
 StyleBox["QVisualize", "InlineInput"],
 StyleBox["Vector", "InlineInput"],
 ", etc."
}], "Text"],

Cell[TextData[{
 StyleBox["QDrawUnitSphere->", "InlineInput"],
 "n \t\tdraws the outline of the unit sphere by plotting n (default: 20) \
circles.\n",
 StyleBox["QDrawAxes->True", "InlineInput"],
 " \t\t\tor, for example, ",
 StyleBox["QDrawAxes->{True, False, True}", "InlineInput"],
 " adds gray coordinate axes.\n",
 StyleBox["QCoordinateCube->->True", "InlineInput"],
 " \t\tadds a green cube indicating the components of the vector (default: \
False).\n",
 StyleBox["QCoordinateCircles->True", "InlineInput"],
 " \t\tadds blue coordinate lines indicating the polar coordinates of the \
vector.\n",
 StyleBox["QHeadColor->QExtractPhase", "InlineInput"],
 "\tworks only with ",
 StyleBox["QVisualizeSpinor", "InlineInput"],
 ", see ",
 StyleBox["QSpinorToArrow", "InlineInput"],
 "\n",
 StyleBox["QHeadColor->color", "InlineInput"],
 "\t\t\tdraws the vector with the given color"
}], "Text"]
}, Open  ]]
}, Open  ]]
},
WindowToolbars->"EditBar",
WindowSize->{936, 1065},
WindowMargins->{{Automatic, 186}, {0, Automatic}},
ShowSelection->True,
Magnification->1.,
FrontEndVersion->"6.0 for Mac OS X PowerPC (32-bit) (June 19, 2007)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "Info3393861035-8787931"->{
  Cell[54426, 1847, 657, 14, 55, "Print",
   CellTags->"Info3393861035-8787931"],
  Cell[55086, 1863, 502, 15, 20, "Print",
   CellTags->"Info3393861035-8787931"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"Info3393861035-8787931", 489830, 9515}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[590, 23, 169, 4, 27, "Input"],
Cell[762, 29, 420, 9, 27, "Output"]
}, Open  ]],
Cell[1197, 41, 194, 4, 50, "Program"],
Cell[CellGroupData[{
Cell[1416, 49, 38, 0, 76, "Title"],
Cell[1457, 51, 107, 6, 94, "Subsubtitle"],
Cell[CellGroupData[{
Cell[1589, 61, 30, 0, 34, "Subsection"],
Cell[1622, 63, 899, 19, 71, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2558, 87, 31, 0, 67, "Section"],
Cell[2592, 89, 245, 3, 27, "Input"],
Cell[2840, 94, 183, 3, 27, "Input"],
Cell[3026, 99, 159, 5, 26, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[3222, 109, 24, 0, 67, "Section"],
Cell[3249, 111, 651, 20, 47, "Text"],
Cell[CellGroupData[{
Cell[3925, 135, 28, 0, 27, "Input"],
Cell[3956, 137, 443, 11, 27, "Output"]
}, Open  ]],
Cell[4414, 151, 254, 7, 29, "Text"],
Cell[4671, 160, 435, 14, 41, "Text"],
Cell[CellGroupData[{
Cell[5131, 178, 24, 0, 27, "Input"],
Cell[5158, 180, 631, 20, 49, "Output"]
}, Open  ]],
Cell[5804, 203, 262, 6, 41, "Text"],
Cell[6069, 211, 475, 13, 41, "Text"],
Cell[CellGroupData[{
Cell[6569, 228, 59, 3, 40, "Input"],
Cell[6631, 233, 636, 20, 49, "Output"]
}, Open  ]],
Cell[7282, 256, 696, 17, 56, "Text"],
Cell[CellGroupData[{
Cell[8003, 277, 56, 0, 27, "Input"],
Cell[8062, 279, 440, 12, 49, "Output"]
}, Open  ]],
Cell[8517, 294, 391, 8, 41, "Text"],
Cell[CellGroupData[{
Cell[8933, 306, 48, 0, 27, "Input"],
Cell[8984, 308, 358, 7, 27, "Output"]
}, Open  ]],
Cell[9357, 318, 229, 7, 26, "Text"],
Cell[CellGroupData[{
Cell[9611, 329, 110, 3, 27, "Input"],
Cell[9724, 334, 396, 9, 44, "Output"]
}, Open  ]],
Cell[10135, 346, 228, 6, 26, "Text"],
Cell[CellGroupData[{
Cell[10388, 356, 303, 9, 43, "Input"],
Cell[10694, 367, 312, 5, 27, "Output"]
}, Open  ]],
Cell[11021, 375, 361, 11, 26, "Text"],
Cell[11385, 388, 248, 4, 41, "Text"],
Cell[CellGroupData[{
Cell[11658, 396, 41, 0, 27, "Input"],
Cell[11702, 398, 912, 24, 219, "Output"]
}, Open  ]],
Cell[12629, 425, 516, 14, 41, "Text"],
Cell[CellGroupData[{
Cell[13170, 443, 77, 3, 40, "Input"],
Cell[13250, 448, 751, 18, 219, "Output"]
}, Open  ]],
Cell[14016, 469, 332, 7, 41, "Text"],
Cell[CellGroupData[{
Cell[14373, 480, 54, 0, 27, "Input"],
Cell[14430, 482, 546, 14, 44, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[15013, 501, 64, 0, 27, "Input"],
Cell[15080, 503, 385, 9, 35, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[15502, 517, 62, 0, 27, "Input"],
Cell[15567, 519, 837, 22, 219, "Output"]
}, Open  ]],
Cell[16419, 544, 57, 0, 26, "Text"],
Cell[CellGroupData[{
Cell[16501, 548, 29, 0, 27, "Input"],
Cell[16533, 550, 324, 6, 32, "Output"]
}, Open  ]],
Cell[16872, 559, 109, 4, 26, "Text"],
Cell[CellGroupData[{
Cell[17006, 567, 35, 0, 27, "Input"],
Cell[17044, 569, 460, 12, 49, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[17541, 586, 24, 0, 27, "Input"],
Cell[17568, 588, 307, 5, 27, "Output"]
}, Open  ]],
Cell[17890, 596, 138, 4, 26, "Text"],
Cell[CellGroupData[{
Cell[18053, 604, 36, 0, 27, "Input"],
Cell[18092, 606, 470, 13, 55, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[18611, 625, 32, 0, 67, "Section"],
Cell[18646, 627, 449, 13, 41, "Text"],
Cell[CellGroupData[{
Cell[19120, 644, 41, 0, 27, "Input"],
Cell[19164, 646, 880, 22, 57, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[20081, 673, 29, 0, 27, "Input"],
Cell[20113, 675, 868, 26, 27, "Output"]
}, Open  ]],
Cell[20996, 704, 687, 17, 74, "Text"],
Cell[21686, 723, 75, 0, 26, "Text"],
Cell[21764, 725, 68, 3, 40, "Input"],
Cell[CellGroupData[{
Cell[21857, 732, 123, 3, 27, "Input"],
Cell[21983, 737, 576, 15, 27, "Output"]
}, Open  ]],
Cell[22574, 755, 226, 7, 29, "Text"],
Cell[22803, 764, 126, 3, 26, "Text"],
Cell[CellGroupData[{
Cell[22954, 771, 48, 0, 27, "Input"],
Cell[23005, 773, 822, 24, 44, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[23864, 802, 60, 1, 27, "Input"],
Cell[23927, 805, 1248, 35, 75, "Output"]
}, Open  ]],
Cell[25190, 843, 96, 2, 26, "Text"],
Cell[CellGroupData[{
Cell[25311, 849, 56, 0, 27, "Input"],
Cell[25370, 851, 606, 16, 27, "Output"]
}, Open  ]],
Cell[25991, 870, 48, 0, 26, "Text"],
Cell[CellGroupData[{
Cell[26064, 874, 53, 0, 27, "Input"],
Cell[26120, 876, 574, 15, 27, "Output"]
}, Open  ]],
Cell[26709, 894, 65, 0, 26, "Text"],
Cell[26777, 896, 113, 3, 26, "Text"],
Cell[26893, 901, 160, 5, 26, "Text"],
Cell[27056, 908, 156, 5, 26, "Text"],
Cell[CellGroupData[{
Cell[27237, 917, 172, 4, 27, "Input"],
Cell[27412, 923, 379, 6, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[27828, 934, 95, 2, 27, "Input"],
Cell[27926, 938, 361, 6, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[28324, 949, 92, 2, 27, "Input"],
Cell[28419, 953, 313, 4, 27, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[28781, 963, 56, 0, 67, "Section"],
Cell[28840, 965, 575, 19, 44, "Text"],
Cell[29418, 986, 56, 0, 27, "Input"],
Cell[CellGroupData[{
Cell[29499, 990, 106, 3, 27, "Input"],
Cell[29608, 995, 2189, 68, 87, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[31834, 1068, 113, 3, 27, "Input"],
Cell[31950, 1073, 285, 4, 27, "Output"]
}, Open  ]],
Cell[32250, 1080, 358, 11, 29, "Text"],
Cell[CellGroupData[{
Cell[32633, 1095, 58, 0, 27, "Input"],
Cell[32694, 1097, 282, 4, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[33013, 1106, 60, 0, 27, "Input"],
Cell[33076, 1108, 277, 4, 27, "Output"]
}, Open  ]],
Cell[33368, 1115, 207, 6, 26, "Text"],
Cell[33578, 1123, 282, 7, 26, "Text"],
Cell[CellGroupData[{
Cell[33885, 1134, 36, 0, 27, "Input"],
Cell[33924, 1136, 323, 6, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[34284, 1147, 52, 0, 27, "Input"],
Cell[34339, 1149, 318, 6, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[34694, 1160, 53, 0, 27, "Input"],
Cell[34750, 1162, 343, 7, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[35130, 1174, 52, 0, 27, "Input"],
Cell[35185, 1176, 337, 6, 27, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[35571, 1188, 38, 0, 67, "Section"],
Cell[35612, 1190, 67, 0, 26, "Text"],
Cell[CellGroupData[{
Cell[35704, 1194, 38, 0, 27, "Input"],
Cell[35745, 1196, 441, 11, 45, "Output"]
}, Open  ]],
Cell[36201, 1210, 123, 3, 26, "Text"],
Cell[CellGroupData[{
Cell[36349, 1217, 36, 0, 27, "Input"],
Cell[36388, 1219, 471, 12, 45, "Output"]
}, Open  ]],
Cell[36874, 1234, 61, 0, 26, "Text"],
Cell[CellGroupData[{
Cell[36960, 1238, 36, 0, 27, "Input"],
Cell[36999, 1240, 440, 11, 45, "Output"]
}, Open  ]],
Cell[37454, 1254, 286, 10, 26, "Text"],
Cell[CellGroupData[{
Cell[37765, 1268, 62, 0, 27, "Input"],
Cell[37830, 1270, 284, 4, 27, "Output"]
}, Open  ]],
Cell[38129, 1277, 156, 5, 26, "Text"],
Cell[38288, 1284, 314, 8, 71, "Text"],
Cell[38605, 1294, 104, 3, 26, "Text"],
Cell[38712, 1299, 327, 8, 71, "Text"],
Cell[39042, 1309, 201, 8, 26, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[39280, 1322, 32, 0, 67, "Section"],
Cell[39315, 1324, 184, 4, 26, "Text"],
Cell[CellGroupData[{
Cell[39524, 1332, 39, 0, 27, "Input"],
Cell[39566, 1334, 469, 12, 45, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[40072, 1351, 37, 0, 27, "Input"],
Cell[40112, 1353, 434, 11, 45, "Output"]
}, Open  ]],
Cell[40561, 1367, 116, 3, 26, "Text"],
Cell[40680, 1372, 424, 11, 41, "Text"],
Cell[CellGroupData[{
Cell[41129, 1387, 43, 0, 27, "Input"],
Cell[41175, 1389, 454, 12, 49, "Output"]
}, Open  ]],
Cell[41644, 1404, 67, 0, 26, "Text"],
Cell[CellGroupData[{
Cell[41736, 1408, 43, 0, 27, "Input"],
Cell[41782, 1410, 380, 8, 45, "Output"]
}, Open  ]],
Cell[42177, 1421, 187, 7, 26, "Text"],
Cell[CellGroupData[{
Cell[42389, 1432, 31, 0, 27, "Input"],
Cell[42423, 1434, 407, 10, 49, "Output"]
}, Open  ]],
Cell[42845, 1447, 156, 5, 26, "Text"],
Cell[CellGroupData[{
Cell[43026, 1456, 40, 0, 27, "Input"],
Cell[43069, 1458, 321, 6, 42, "Output"]
}, Open  ]],
Cell[43405, 1467, 189, 6, 26, "Text"],
Cell[CellGroupData[{
Cell[43619, 1477, 41, 0, 27, "Input"],
Cell[43663, 1479, 455, 12, 49, "Output"]
}, Open  ]],
Cell[44133, 1494, 126, 3, 26, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[44296, 1502, 28, 0, 67, "Section"],
Cell[44327, 1504, 681, 23, 41, "Text"],
Cell[45011, 1529, 28, 0, 26, "Text"],
Cell[CellGroupData[{
Cell[45064, 1533, 53, 0, 27, "Input"],
Cell[45120, 1535, 877, 23, 57, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[46034, 1563, 50, 0, 27, "Input"],
Cell[46087, 1565, 1872, 58, 103, "Output"]
}, Open  ]],
Cell[47974, 1626, 160, 3, 26, "Text"],
Cell[48137, 1631, 91, 3, 40, "Input"],
Cell[CellGroupData[{
Cell[48253, 1638, 70, 0, 27, "Input"],
Cell[48326, 1640, 1275, 37, 46, "Output"]
}, Open  ]],
Cell[49616, 1680, 57, 0, 26, "Text"],
Cell[CellGroupData[{
Cell[49698, 1684, 121, 3, 27, "Input"],
Cell[49822, 1689, 1275, 37, 46, "Output"]
}, Open  ]],
Cell[51112, 1729, 155, 3, 26, "Text"],
Cell[CellGroupData[{
Cell[51292, 1736, 185, 6, 66, "Input"],
Cell[51480, 1744, 283, 4, 27, "Output"]
}, Open  ]],
Cell[51778, 1751, 108, 3, 26, "Text"],
Cell[51889, 1756, 93, 4, 53, "Input"],
Cell[CellGroupData[{
Cell[52007, 1764, 87, 2, 27, "Input"],
Cell[52097, 1768, 283, 4, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[52417, 1777, 126, 3, 27, "Input"],
Cell[52546, 1782, 288, 4, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[52871, 1791, 40, 0, 27, "Input"],
Cell[52914, 1793, 83, 1, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[53034, 1799, 134, 2, 27, "Input"],
Cell[53171, 1803, 676, 14, 43, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[53896, 1823, 32, 0, 67, "Section"],
Cell[CellGroupData[{
Cell[53953, 1827, 77, 1, 27, "Input"],
Cell[54033, 1830, 99, 1, 27, "Output"]
}, Open  ]],
Cell[54147, 1834, 115, 3, 26, "Text"],
Cell[CellGroupData[{
Cell[54287, 1841, 114, 2, 27, "Input"],
Cell[CellGroupData[{
Cell[54426, 1847, 657, 14, 55, "Print",
 CellTags->"Info3393861035-8787931"],
Cell[55086, 1863, 502, 15, 20, "Print",
 CellTags->"Info3393861035-8787931"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[55637, 1884, 414, 9, 27, "Input"],
Cell[56054, 1895, 5528, 99, 325, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[61619, 1999, 668, 16, 27, "Input"],
Cell[62290, 2017, 2396, 51, 405, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[64723, 2073, 188, 3, 27, "Input"],
Cell[64914, 2078, 3042, 66, 203, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[67993, 2149, 614, 11, 27, "Input"],
Cell[68610, 2162, 3109, 59, 191, "Output"]
}, Open  ]],
Cell[71734, 2224, 376, 9, 41, "Text"],
Cell[72113, 2235, 258, 7, 26, "Text"],
Cell[CellGroupData[{
Cell[72396, 2246, 73, 1, 27, "Input"],
Cell[72472, 2249, 865, 17, 43, "Output"]
}, Open  ]],
Cell[73352, 2269, 406, 9, 41, "Text"],
Cell[73761, 2280, 215, 5, 26, "Text"],
Cell[73979, 2287, 812, 38, 41, "Text"],
Cell[74794, 2327, 235, 9, 26, "Text"],
Cell[75032, 2338, 255, 7, 26, "Text"],
Cell[75290, 2347, 328, 12, 26, "Text"],
Cell[75621, 2361, 540, 14, 41, "Text"],
Cell[76164, 2377, 39, 0, 26, "Text"],
Cell[CellGroupData[{
Cell[76228, 2381, 327, 8, 66, "Input"],
Cell[76558, 2391, 7775, 137, 406, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[84370, 2533, 124, 2, 27, "Input"],
Cell[84497, 2537, 2808, 55, 403, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[87342, 2597, 2871, 56, 391, "Input"],
Cell[90216, 2655, 7369, 131, 406, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[97622, 2791, 184, 3, 27, "Input"],
Cell[97809, 2796, 957, 19, 58, "Output"]
}, Open  ]],
Cell[98781, 2818, 92, 1, 27, "Input"],
Cell[CellGroupData[{
Cell[98898, 2823, 877, 18, 47, "Input"],
Cell[99778, 2843, 7731, 135, 406, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[107546, 2983, 139, 2, 27, "Input"],
Cell[107688, 2987, 913, 18, 58, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[108638, 3010, 704, 16, 47, "Input"],
Cell[109345, 3028, 7713, 136, 406, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[117095, 3169, 544, 14, 43, "Input"],
Cell[117642, 3185, 2591, 52, 191, "Output"]
}, Open  ]],
Cell[120248, 3240, 427, 10, 41, "Text"],
Cell[CellGroupData[{
Cell[120700, 3254, 75, 1, 27, "Input"],
Cell[120778, 3257, 1270, 25, 73, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[122085, 3287, 143, 2, 27, "Input"],
Cell[122231, 3291, 1317, 26, 73, "Output"]
}, Open  ]],
Cell[123563, 3320, 228, 5, 26, "Text"],
Cell[CellGroupData[{
Cell[123816, 3329, 619, 8, 27, "Input"],
Cell[124438, 3339, 106133, 1777, 406, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[230608, 5121, 157, 3, 27, "Input"],
Cell[230768, 5126, 102056, 1702, 406, "Output"]
}, Open  ]],
Cell[332839, 6831, 69, 0, 26, "Text"],
Cell[CellGroupData[{
Cell[332933, 6835, 227, 5, 27, "Input"],
Cell[333163, 6842, 55947, 944, 406, "Output"]
}, Open  ]],
Cell[389125, 7789, 68, 0, 26, "Text"],
Cell[CellGroupData[{
Cell[389218, 7793, 232, 7, 27, "Input"],
Cell[389453, 7802, 710, 15, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[390200, 7822, 165, 3, 27, "Input"],
Cell[390368, 7827, 98020, 1639, 406, "Output"]
}, Open  ]],
Cell[488403, 9469, 130, 5, 26, "Text"],
Cell[488536, 9476, 894, 22, 101, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
