(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[   2822806,      50010]
NotebookOptionsPosition[   2802017,      49334]
NotebookOutlinePosition[   2802423,      49352]
CellTagsIndexPosition[   2802380,      49349]
WindowFrame->Normal
ContainsDynamic->False*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Gallery of mathematical images", "Title"],

Cell[CellGroupData[{

Cell["Author", "Subsection"],

Cell["\<\
Bernd Thaller
Institute for Mathematics and Scientific Computing
University of Graz
Austria\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Source", "Subsection"],

Cell[TextData[{
 " ",
 ButtonBox["http://www.uni-graz.at/imawww/vqm/",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.uni-graz.at/imawww/vqm/"], None}]
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Abstract", "Subsection"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " source code for the images at  ",
 ButtonBox["http://www.uni-graz.at/imawww/vqm/pages/colorgallery/index.html",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.uni-graz.at/imawww/vqm/pages/colorgallery/index.html"], 
    None}]
}], "Text"],

Cell["\<\
In this notebook, the value of the option PlotPoints has been reduced for \
performance reasons.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"$starttime", "=", 
   RowBox[{"AbsoluteTime", "[", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{"$Version", ",", " ", 
   RowBox[{"Date", "[", "]"}]}], "}"}]}], "Input",
 CellChangeTimes->{{3.39184908209375*^9, 3.39184908571875*^9}, {
  3.392469791733589*^9, 3.39246981005025*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"6.0 for Linux x86 (32-bit) (June 19, 2007)\"\>", ",", 
   RowBox[{"{", 
    RowBox[{
    "2007", ",", "7", ",", "20", ",", "10", ",", "40", ",", 
     "14.173055`7.90403845779349"}], "}"}]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Load package", "Section"],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<VQM`ComplexPlot`\>\"", "]"}]], "Input",
 CellChangeTimes->{{3.39184902375*^9, 3.391849029421875*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["01: alien1", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myColorMap", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ab", ",", "ar"}], "}"}], ",", 
     RowBox[{"Hue", "[", 
      RowBox[{
       RowBox[{"Sin", "[", "ab", "]"}], ",", 
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ar", "2"]}]], ",", 
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ar", "2"]}]]}], "]"}]}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   RowBox[{"Exp", "[", 
    RowBox[{"Tan", "[", 
     RowBox[{"y", "+", 
      RowBox[{"I", " ", "x"}]}], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", 
      FractionBox["\[Pi]", "3"]}], ",", 
     FractionBox["\[Pi]", "3"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", "0", ",", "\[Pi]"}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.390026578190277*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJzd3Xn8r2ddHvgb62gXWqftdGprrTjazaWuHZdpLVatbdXWWrWLSuNa6q6o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   
   "], {{Rational[-1, 3] Pi, 0}, {Rational[1, 3] Pi, Pi}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["02: alien2", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"myColorMap", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ab", ",", "ar"}], "}"}], ",", 
     RowBox[{"Hue", "[", 
      RowBox[{
       RowBox[{"Sin", "[", "ab", "]"}], ",", 
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ar", "2"]}]], ",", 
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ar", "2"]}]]}], "]"}]}], "]"}]}], ";", 
  RowBox[{"QComplexDensityPlot", "[", 
   RowBox[{
    RowBox[{"Zeta", "[", 
     RowBox[{"Zeta", "[", 
      RowBox[{"y", "+", 
       RowBox[{"I", " ", "x"}]}], "]"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", 
      RowBox[{"-", ".5"}], ",", ".5"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"y", ",", ".5", ",", "1.2"}], "}"}], ",", 
    RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
    RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
    RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
    RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]}]], "Input",
 CellChangeTimes->{3.390026585510803*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJzt3Qm0b1V5JfqNl3svvaCgICBXEKTvG+kPtrE0nZrENCYkJtGYZ0JiytjF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   "], {{-0.5, 0.5}, {0.5, 1.2}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output",
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["03: alien3", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myColorMap", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ab", ",", "ar"}], "}"}], ",", 
     RowBox[{"Hue", "[", 
      RowBox[{
       RowBox[{"Sin", "[", "ab", "]"}], ",", 
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ar", "2"]}]], ",", 
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ar", "2"]}]]}], "]"}]}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   SuperscriptBox[
    RowBox[{"Exp", "[", 
     RowBox[{"x", "+", 
      RowBox[{"I", " ", "y"}]}], "]"}], 
    RowBox[{"Tan", "[", 
     RowBox[{"y", "+", 
      RowBox[{"I", " ", "x"}]}], "]"}]], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", 
      FractionBox["\[Pi]", "8"]}], ",", 
     FractionBox["\[Pi]", "4"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"\[Pi]", "+", "0.0001"}], ",", 
     FractionBox[
      RowBox[{"15", " ", "\[Pi]"}], "8"]}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.3900265889056845`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJyUvQvYpmPZ/n3MDFqXFkrl1VDS6m3tTaWavFQqEhKKRkhSSYloNaksKymJ
UCNlFZJ1QmO9zqL0ZDGMsTbDGDNj1s/z+X3O35z7/fzHf/u+e9vObds9cz/X
dV7HsR/7eRzHeV6PNT6/22ZfHVdVO7YB3uTJsbiNTQNv/uRY0saWgbd6cixt
4zOBtwu8/ZNjuI0dA38x8K5PjpE2dgv89cB7Bt4n8PcC7/vkqDFPjf0DHxz4
54EPC3xk4N8GPi7wiYFPDXxG4PMCXxj4ksBXBb4+8M2BhwJPDXx34PsDzwg8
K/CcwPMDLwq8JPBw4JHATzeGn+Y6CwPPHTU3/fVg4LsC3xL4msAXBD6tOn+O
CXxw4G9V5+EOgTcOvE51Pq8WeGx1/j8Y+B/1lO0YZ9ZTz8k4IvD3nxwL2vhC
PWV3xieeHE+08d4nx7w2XldP2Yjx8nrKZ4znPTkeb2PFJ8fsNvg81gZmfbSN
lZ4cj7Sx8pNjZhurPzkebuNtT46H2tjoyfFAGzs9Oe5r48dPjnvbgPPT24Cr
09pgrne28cp6iqOM/31y3N7GLk+OW9v4aT3FacapzceMq58c/2yD697UBs95
QxtL2r2NFTjBQK/kw6aBNw+8ZeCt2u8zPhN4u8DbB94x8BcD7xp4t8BfD7xn
4H0Cfy/waL1yngcH/nngwwIfGfi3gY8LfGJgbG5cnBH4vMAXBr4k8FWBrw98
c+ChwFMDTw98f+AZgWcFnhN4fuBFgZfEcw0HzqF+jcb5/bzOwsCj9Ur8YOC7
At8S+JrAFwQ+LfAxgXNt+lbgHQJvHHidmOdqgccFTl39R9jtzMDwRw38fuCd
q2vgpoHRKzXw9dU18OXVdQ+9UvdWDMzU1T2wurdidd17QXXd+6/quvfW6lr3
4XrKH4zPV9c64kit+0N1rbu62eLB9jtq3Uura937nhz3tIHN726DeLyrDeJI
fbusur7dUV3f0F/1Dbuob67rjE3Cj5sG3jzwpwKP1it9ul3g7QPvGHh0fiXe
LfDXw+97Bt4n8Peqc2DfwPsFhsNy45DAhwU+MvBvq/PnuMAnBj41MHolx84N
fGHgSwJfFfj6wDcHHgo8NfD0wPcHnhF4VuA5gecHXhRjSeDhwCOB8bH2STwc
dlsSeGHYf27gRwM/EH68K/Atga+uzocLAud6cUwtf23aK/DnA3+8PQPjnYFf
GXhs4AcCZz6cefURgXMN/ULc9xOB3xv4dfEsLw/8vLDDimG3CjwmbL5i+OsF
4Wv0Sq1Dr8z30Cu17vPVte4H1bUO/pvXXVld6x6pnte9pLq+8Vzq28TqudyP
quvbH6vrG3Ghvt1WXd+4vvkbz3BHGyPte4zR+ZV22zzwp8KGo+tBbbhd4O3D
njsG3jlsu2vY9quBvx523jPsvE9guOFas2/g/cIXBwc+JPxyWOAjw0fHBP59
9bXpxMCnBj4jfHpu4AsDX/I0fr8+8M2BhwJPDTw98P2BZ8T1ZwWeE3h+4EUx
/yWBhwNbT85ZDtZuw2HDJWHnhYHnhl8eDfxA+PHOwP8Kv6e2/y3wKdX5c1R1
Xh0Y+JvVeTgx8Eer8/btgVMrRgLfV8uvT08PfHizC+M7gbOmyLzu3YHXDrxq
4OcGXiHuVYHHxDxXCrxyPNfq8exvC/t8JGy4Q9h237D/H8IvV4VPH63u91Wq
cwa9kmPEvvkbemX+xjVntHFxdU0jr1LTZlbXNOakpvHMatrG8SybxrNsFs/y
qXiWrQJvE8+1bXVObh/PuGM8I3ol/78Uz/vVeF70ynj8Zjz73oG/W13n9w2b
7Fdd8w8K+xwSGL1yLTgi7HZMYPTKHPiEwKeGbc8IfG5g9Or+Ni4JfGXg6wPf
HHgo8NTA0wPnmBF4VuA5MZ/5gRcFXhLPNRyYYNAOo7F2sxad0a4jXhD2nxN+
yXXZuc5qzyj+Z/gdW8mHv1bnycnV+cNaI6+oO+TbN6rzcNsazCvk7VsCU9fI
88w5iQ3j4srqsX9K4F9U14e9q+vG9oHRSXXmfwKvFfilzb6M5wQeF7gCjwn8
jMAvjGuOD/yOmM/HAu8c88/a5MR49uvCJo+HrVYNG74/bL5D+GK/8NfxNbiO
63dyJrkxMwb3kHs8g7zdOK5P7ioHNot7bRH3+nR1Xm0T99027juxOk+Zv3zG
PnL+SzGfr1SPl91jbuiVegsf7AeiV2oveay9QXqG6vBBgX9WvWf4y+o1NXpl
/oleWV//PjB6ZV56SmD0yhr83MAXVM9dLwl8ZeDrA98ceCjw1MDTAz8QeEbg
WYHnxHzmB14U818a2H6ZPf7E2sf++F3t+9pzcdh5QeA54ZdHwl/3hR/vCP/e
VN3vl1fnA7aVJ8SR/Pl1dV7lOv616jxMftITlrdvqM7nF1bnObaS/zy/cYEf
jccTqmvgT6vH7B7VY/mz1WP8Q9VjH91QE9aorhUvqa4hzwo8tpafX2Xd+szA
Lwn86sDrBs6ez5fjmj8JnPsaN8c8F9Rg/uZzbRDPu0vY4eCwz8lhwyvCtneE
/R+twVpA342Efz8e/kWv1LdPVucGeiVn0Ct5tXX1NR0fycPP1fL7b9T109rg
uYwF9MoY2b16HMEBY5A+qn05cu//tIFe2aNDr/7dBjWC/Tr06l9toFfuTfy6
eh/v6Or7FMc+OW5s44TA6JX7F3+pp/qwjHMDXxAYnrvHcWXgHDcHHgo8NfD0
wA8EnhH4sbjv3MDzAy+K+S+N5xoJDI+1Q/Y5R8Juw2HPxWHn+WH/x8MvM8Nf
94Yfb4txQ/j90up8OLs6T+iBTKu+7sirSdX5tmt1HlIXyM/3V+ctuY18pv6S
57Or859nMC7wqbFzbPWY2r8GawS1jhgxHj9YPU7fXD1+iXfj+oXV4/0Z1XUA
Wcn+lVoxrrqePCfwy2qwPyZ+X/Ml49OB9wic++bnBr69ButQ8dpx39x73b0G
80+f5fR4xmvj2fGl+j87bLU47MmNtTN65RpHL0u/o1fygV6Wayt7hXIJvZJj
6JXcQ6/kJz09Ocz+qdxGr+Q/Om+MsD4aO9jTWEOvjE306ro2JlXf62SdvboN
9OqqNlgHr2wDG17RBnp1eRvo1WVtwMlL2zg+MHp1SRvo1cVtnBMYbk9p4+LA
Vwa+PvDNgYntv7cxNX4+PfADgWcEfizw3JjP/MCLYv5L47lGAsMN7QDWPiNh
t+Gw5+Kw8/yw/+zwy8PV/ZXaCz/0L/+t33kG+YCd5Ql+kT/U+PLq29X5xpoo
D8kl5Oe61Xn7qup8pl8tz5mn/Ec/jYuzqmvdb6rnfpOqax21g1rHfdW691bP
615fXetGn2HI/b7cB8z9QWM8e+zPr64JeTbjzdU1ZMPq2jKxuuZ8L/Dvmr8Z
eQ4n9x2eF/gdgbeK63w78FFx37/GfG6Ked4X858bz2g+t6DdR5tQz1ovoJOu
fdjc9ZHaUJ6wZsmfrarz6jPV125673KSWv7aNuhlyWf2CuU8a6KxwF6h8UIv
wvhi38e4wybGNfvIF7Xxw3qqf8Q4oJ7q1TLQq/PbOLTZjkG/9Lw2sO25bUyu
pzSIcXzgP9VT6z2D9eKsNs4JjF6d2cbFga8MfF1g9OqMNoYCTw08Pb7/QOCZ
gR8LPDfmsyDw4pj/0niukcCur66x2mck7LY07Lko7PxE2P+x8MtD4a9p4cd/
h3+vCb/z+/LhtOo8oWaXP9T78op9Gfk2sToPOWskP+k5y1vyEPm8tDrPsbP8
537GxZ+qax38Uev2ql7zskab13Ff8zr2ItU3ajT1LffXnl1d38ZV1zc+5hVj
q+cbz6wevy+unp9wfeP9XdU1jbpJTSDW1IqDqmsIa7HaYs490u6RZz/UqA8G
3inwAYFPiN+9JPCtca8ZNVhvOreRmDPX8rk+Gj79ePiaZ3RNRK9cK6kNXU/J
LV1/6RW4RtPLkpMTq3OV2lA+sx7Jc3pZ8p+82hih924cwUljjb1CY5M1wrhG
r05vg1z9z21Qm5/WBnw7tY3Dm58Y6NWf2phcT9XbDGqQk9rg305s4/TAzO+E
Nv4WmOc/vo0rA18X+ObAQ4GnBp4e+IHAMwM/FvedG3hBzHNx4KXxXCPxvHBD
O4C1z0jYbWnYc2HYeV7Yf1b4JTX2zvDjv8K/V4bf0UT5cFJ1ntBvlD/k0vLq
a9X5Rs4vD4kp+Ul9JG/JSeTznOo8Jz+T/9wj+wNqHbwyr/tq9byOukN9m1Bd
3/67eg27WnV94+yB+rZS9fyNUF+2x1G9R8deoZpGDaumvaK6pr2xuqZlHUou
oabtXT32yRXVNJ5XrZheXUOoT9UWnkXN2SwwfWY1KvdMzwqcZ0Kmx+/ODrwk
7sX3nIPnBRmsBa5f5FqucdSGrn3UhnJj8+qcwUfyCp7IN2pDeci64/pObSiH
ydvlNrWh/KeXZYxQCxtH2MT4wubGI713Y5a9wj+0wd7EcW3Q9/t9G5wdPbaN
X9VT2jS5+e53MX7bBtc6pg3mcXQbfw5M7B3Vxt8CX9yuy7gi8HWBbw48FHhq
4OmBHwg8M/DsuO+8wAtinosDe4bJc0w+LzzRBmDtMxJ2WxL2XBh2nhv2fyT8
cl/4647w443hX+vtU5o95QPXkifU8vKHdUpekavLN7gqD6kB5Sc1oLwl9uUz
uiDPiSn5Tx1qXBCD5nXZi2D9tZ5lrVffyG3s761VPX9jH9D87TnVa1VyKnM2
Ptan5FRqGjmVmvbi6pq2ZvU8jf1HNY34VdPIB6w9yamsPbG1OoYvrL/QOHWM
82nmOWigGkJOpbbkOe3jazCnUqNuDZzvZeT7FyPxu/nfi+NenMe4oI2Phn+p
Da0LyLXkBroqZ6gNXVupDV1/0XPXaGpD+UltKG953j+2Ad/kPL0s44J109ih
l2V80csy7uCPMUvv/cg26L0f0QY+OrwN+h6/auOwGPzOL9sgbn/Rxh8C8wyH
tsEz/7yNswOf3+7BmBL4isBw/Wdt3Bx4KPDUwNMDPxjXmRl4duB5MZ+FgZfE
/IfjuUYCwxPtYB/2sPZ97bYk7Lkg7Px42H9G+OWe8Net4cfrw79/D7+jTfIB
X8gT1h35s2d1Xk2szjf4LA/fXJ2f1F/yNmtk7CzPL67Of+5nXBxUg7mc+pb1
BbFs/kYeYv72X9VrVXIqe3TkVOZshKs5W1XXNHRVTaN3pKahIeZp3Ms+G/t0
1p7EnbUna7r7a9jTvYOLq+dm6KC5Gfmbudnbq+sYz6uOUeOoY/hYHcOOy6sr
7w2c74stCZxjOH6eNemHw48bhX8/Xt3vaLXrIOuXnNmiOpfIteQYteHkNrat
vnZPrM7bHarzmdpQzlNfGwvk28YLtaFxRN/AuKOXZTzSyzKuqRd+2ga995+0
wb8d3AbXOqiNI9r3GMTSAW3wPPu3QR21XxunBYb3P27j/MBT2jwYVwS+LvDN
gYcCTw18T+AHAz8SeHbcd17ghTHPJYFdFxn2Hew9aIeRsM9w2G1x2POJsPPs
sP9D4Zdp4a9bwo9XhX/PD7+fXJ0P7InIE+p9+UMeLq/g3uQ23ledh/Rz5Cd1
jbydUZ3P1H7ynHpV/nNf44I10RoW3pq/UY9Yq5Lb2K97TfX6lHOV5mycVTBn
I3ey/8bHPI2fm6fxffM0rmPt+drqtSf9fPdPyR/MzYgXczPWIXMzclB1jD0J
dYyPOsb1cz9OHaO3rI7hS3UMe6lj5KDWbsxJHcv3xfjeyNMMNWph4EcCcz7E
ngZrkzk8uZZ+J9ea3AZ9eNdHci3rDmzl2kquJffIteTk56qv49SG8pnaUJ5T
GxoLcNJ4oTY0jqgNjTV03hikRjB+fxiDONy3DeLqB20wj0ltwM/vt3F0uxaD
mue7bWCj77RxamB4/u02/hr4721+jMsDXxf45sBDgacGvifwg4EfCfx43PeJ
wAtjnksCD8dzwQOfF6wdRsI+S8Nui8Ke88LOnmn2PQz9MjX8dVP48bLwL5qv
3+GcfCBflCf7VOcPuiGv0A35Rl9dHlJ/yU/mLG+Zz+Q2plTnObm0/IczxgU8
NH+D59aqnJH4Sxuc6VLT6CmpaeRFahr5kprGxzyNUDRPI++y9uScg7Xn+Oq5
GT18czNi03qT2HFv9MDqPTTqPnUM+6tjeZaYPFAd46yaOkYuYV3JNfO9D+vK
S6vXldzb/hj5vznY6F66OmYuNdy+l30tNeruwOSQ5uTkWq5f5Fr6HT78ug1y
LXlCriV/yLWsR8gbXXPJtVyXt63O1YnV+cy+ofkAtaH8pzY0RsjDjSNqQ+OL
2tAYJPaMU37HWOYee7fBPPZqgxjYs43DA8Pvb7ZxbLsfg7X5G22cGhi9+nob
fw2MXu3exuWBrw18U+ChwFMD3xP4wcCPBH487vtE4EUxz6WBR+K54IHPC9YO
I2GTJWG3hWHPOWHnmWH/e8Mvt4a/rgs/XhT+PTX8jv3lw34x4ID82ao6r9AN
+UZOJQ/JqeQncShvqc3kM7WnPCfnk//wyrigtrJfx/puzoY2qmn0x+y/oTPm
aczBPI2QW7YPW32/FR2zt8ZegLkZ/XlzM87kW29+uAZ7aO4RYDt1jPmpY9Sl
6hi6Yz6Gnqtj5KXWlfRtrCtZe+z5o+f2x6ZUP5vBPeyPYWdzMPTLHMy8imEd
6FC7fDeMkfWj5wIZaKlr04bVc3VsYg5PruV6h57LE/pa8odcS159qvo6C69c
i8m1JrVBriWHybXkNrmW/KdP+K02qA2NF+LQ+OK/jbtvV49T7mEsw/uvtcFa
vlsbh7TrMujLfKUNao8vtzG5nqpRGccHPqXNj3FGYNbUXdq4KPDlgdGrL7Zx
U+ChwFMD3xP4objOo4HnBJ4f81kUeGnMfySwZ52/3LB2GA77LAm7zQ97zg47
Pxz2nxZ++Vf468rw47nh3+Or+x19+U4MebJjdf6whsor3ueVb/Sp5CHzlJ9o
prwl15HP1JPynGvJf/IK44I1Wk2jl66mUfdNboP7mqc9q3qeRn23bO+1eu1J
j93cjN67uRnnsszN2O+zh0acTmkD/VTHsIs6xjzUMfhmPjajel35oup1Je8z
WldyTftjrB+eQ0OjPZvh2RQGvTZzMP7bHAydyvdSzcHsrTPgUr6bbw6GRuUZ
rTy7pXZ9MHy3QfiUdcSeBnWi6x08cR3Ej5Pa2LQ6r9hDlGv0FuQhuZb8JNdy
7UbP5TaclPPEp3FBLBkv/Lcx9c3qscZ6b2wyD+P3h9XjnefZuQ2e+QttwNGd
2jiy+t8I+11g1qwd2qA2+Hwbfwl8buCL2rMxLgt8beCbAg8FvjPwPYEfCvxo
4Dlx3/mBFwdeGvMfieeCS4m1w3DYZ1HY7Ymw56yw8/1h/zvCLzeEvy4OP54e
/j0m/E7dKB/4N3lC3i5/1qvOq/HV+TauOg/JFSa1Qf9Q3pIPyWe4Ls93q85/
1lzjgtxDTePMp3ka79CZp6E51p7MYdl+a/XcDO0yN+PMlfsFnAdTx7i+Ojah
uo6x524+xrpsPvbb6vkYdZl1pX8LxrrSnj97pvbH0CjPoaFR9vnzfTQ4bA72
7+o5GDw0B4Nv7lHCGftgeS4djbIP9nh17YLbahfaqnah12qX+9nDzSauR+uH
Tzeo7mtyLTmwUfWcn16cdQG5lrwi15Jv+F0ekmvJT3It125yLblNXMl54sRY
4L+Nl92qxxG1jbEGj43H71aPU9ZN45pYmNjGT9q97att1wb827aNYwKTD3+2
jZPbMzBOD4xebdPGhYEvC3xN4JsCDwW+M/C9gR8KPCvw3MALYj6LAw/H/OGS
z5V4JOywNOyzMOw2J+w5M+zs+1c7tGfRL1eHv/4afjwh/PuL6n5Hd+QD15In
rJvy503VeUUNJd9Y4+UhtpWf5MDyFq2Rz+R5k9rARvKffMa4YA/OPI29Ofts
9KbsrVHTLdtXrZ6boVHuF5B3qWP0z9Ux3m1Rxz5QXcfwpTq2T/W6kt+3rkSj
3NMkj7I/9uLq59DeU73Pj//s85OPmoNxb3MwtM8cDP1yj5LrT2sDjtkHgzP5
vrz1I9yzfsQv1o/8vtpFLqh20VdRu35dXbuIa7WL2t9+woTq6xTapa83rJ7D
o11ygzpRzlAnum6Sa7mekmtZd5BryU/Oa8lb4sQ1HXvKebhuLHyxerxwXeOI
OsT4gqPGI7w0TidVj+Uft3szqA23agPOfbqNwwMf3ebNgFufauOkwPQptmjj
nMAXBr602YJxTeCbAg8FvjPwvYEfDjwrrj838ILAS2Kew4Hhlc/le2WMkXj2
JWGf+WG32TWon9r5zrC/6zfjkvDXX8KPx4R/WUf0O9okH3gGeUItJn/oacsr
OC3fqG3kIXWf/MSP8lad2qvdW56jh/Kf3pRxsXr12pMzCeZmvBezbF+1uo7R
arGHxrkpdQyNMh9Do9QxNEodw47qGPNwT3Ny9Z4/uaL9MbTAsxnkadaSXNNa
knXCHAzNNQdjzfXcBdd0j5K8yj4Y17cPRo6tdsGTaW14XoGB1tjDp1ZUu26v
rl3cy7yLvorahb3Urt2qa9cm1bWLvNocG+2yz/DB6r5mrXFdQ7vkBv6VM9SJ
cmmT6uspPXnXWbgnP4kH12h8JJ/ht2v956vHAtcyRrifsbNb9fiCi8Yg/DM2
v1c9flk7jXf7boyftbkyDgtMfr9pG/TeP9HGCYFPa8/MODvwBYHRq43buCbw
TYGHAt8Z+N7ADwd+LK4/L/DCwEtiniOBfVff9/V93uF49sVhn3lht0fDnveF
nW8N+18bfjk//HVC+PHQ8O/e4ffPxWDdlCf0i+QPWiGvmIN8I4+Vh+TD8hN9
kbdon3ymRpDn7LvJ//HV42Ll6rkZ9122l1pdx8ZU1zE0Sh1Do9Qxztibj02o
XlfyzJ7fcH/66Db/ydX7Up7FxRfmYGipOVieTULzPVeGnd2jZL2Y0oZ9LgZn
uawf0RG1C/6oXTyk2kWuo3Yxn2ltoFNqF7maeRexYM1IHFkz7ltduz5fvd+F
fezVs3dpr/494S+0y7UJfTYnR7vM1dEu1zt6XK6DG1XnEto1sQ1ixnUW3rv+
wm+5yvotn/k3ec7vy3/1jkGuZex8pXpM8QzGIGunsfmd6vFLnW68s6Z/vI2f
Bv5lewYGvayPtvG7wMe3Z2acGph17iNt/C3wJYGvDnxj4KHAdwa+L/CMwI8F
nhd4UcxnaeCRmD/cE4/E8y4NOywMPCfsNiPsOS3s/M+w/6Xhl9PDX0fV4Nqh
f3cJv3MP+cC+mDwht5E/aKa88r1BRvYV6UPJT3gsb5mffH5XdZ6vUZ3/9HyM
C84YLNs/ra5jaJQ6hkaZj7Hvpo5xzsG6ktjx/Abzsz+GxrmPyfytJYl3czDW
EXMwelz29rGve5TMyT1K9M4+GDy0frylunaRG09pg5wm321Xu3jI/Pseahda
Y80IV9Uufsd+F/mf/a5Dq/fqqYny3XB79dTd7jPS33OfEQ0011o3/IV2uTbR
Y3TNQrtcy9Au1zt6XK6DaJfrI7Ehx+C63IPTchLuui5vWZ3P21Tn+XbV+f/5
6nHBnIydL1WPKXhm3FmvMvauHrPwybiGJx9uwz2GDzXbisndN2zjt4H/2J6f
cUpgavD/beP8wBcHvjrwjYGHAt8Z+L7AMwLPDvxE4EUxn+HAIzF/eCgeiedd
Enh+2Ce18f6w521h52vD/ueHX/4Y/vpZ+HGP8O+nw+/sW8kHetryZH51/hB3
8op8Qr7hO3lIXMhPriVvub585vyDPEej5D8atWzPtLqOoVHqGP0odYx5WlcS
d9aV2GtSG6yz7mOyPpqDkZObg7E/Zw5GD98cjPOo7lHyPJ4lQwetH8m3rB+p
udQuuKR24Ue1i/xZ7eLBptRTA86oXayD1oz43bMTnBu034Wv7dV7vo1BD9lz
+J+tvs+I/T3rRf5pnUiOZY/rhuq5FrpnnYgdXHfWDT++J/z7vup+/0D1NQ7t
MrcnHux1wG85Bo9dT+GrnNw0Bv/meu36yoB/8v9z1eOCHoex88XqMUWuZazB
J+Nxr+oxiw2Na9a/9ds4IPAh7dkYnHOY0MbRgY9rtmCcHPiMwPSW39/GxYGv
Cnxj4KHAdwW+P/DMwI8Hnh/3XRx4ODCcdP6Jh+N5F4Ud5gWeGXa7O+z5z7Dz
JWH/P4dfjgh/fS/8aC7+sXZd/c47cfIh61bWb/mDbeUVuYh8QzvkIeug/ITP
8haNks/s78tzNGrZPmn1uKBnbj6GhlhX0nu3rqTf5dk25m1/DM00Bzuiei2J
rczBiFVzMK6Ze/T2wbiH2nV49fqRXpXahS/ULvIYtQtu5HvrahfYmpHcxX4X
vrbfhQZd0Qb/7XkJ1iD3GZm37xNhO89ITKh+RoKzZNaJ1HjWibdUz7X8OyOP
tec018KO5lrU7ObP/xN+XDf8i6/1O9rlGkcMyBM47ZoIj+2BwFfXUzi6cQy5
yvdcr7mW3Fb7PtzmYVyY2zHI9Y2jL1WPL9ZCY3CP6rG5T/X4nVQ93vdrz8b4
aWBqw/XaIC9/bxvHBj4p8OnNXozzAk8JfFXgGwOjV+9u4674+f2BHwk8J/CC
wEtiPiOBif3EPtfSeN4FgWeHfR4Mu8lFxrVh5/PC/r8PvxwQ/vpy+HHj8C9n
jfQ7tZV8gN/y5O/VuUNdKa+og+QbnJGHPKf8RKPkLXmUfKYftWzPtDr/OZNg
XLAPaF3JuVDPchAj9se4rvuY5FXmYMS1+5XkJOZg2N/3C3jPUe1iTmoX9bL1
I/We2pXvMbF2mHfBB2tGfGrNyL2sGVnL7HehC/l3M+zVo03uM/ou+qVtHp71
4vmtE/GxdSL9NOtENNBcC10y1+J+5lrEl7kWNjLXIlexrwVvzLV4n9H8+Z3h
R7TLdWrd6n4nHuQDnHbtg8euifDVWgBeup5+JAb8k6ubVOfwZtW5vWV1/jMn
44I6cUIb8MmY2rl6rBEXxiO8MU73Cvzd6jFOjq5WHNSemXFoYNazd7UxOfAJ
gf/cbMc4N/DfA18ZmLx3nTaGAt8V+IH4/iOB5wReGHhpzGckMLzNn/tcSwI/
EXZIbZwedvtn4IvDzn8K+/8i/LJn+OvT4cd1wr9oiH6n5pUP1CTyhP6h/CFf
kVdwQ74xD3mIRslPeubylrMHy/ZJq/N8her85/vGBX3+iW3wDO5jwj1rSThj
DobfzcGo5T1PS+1j/chzql38m9qF7qhd5DpqV553JZ6tGdEf30PHv/a7llY/
n0/M26tH3zy/Sl/CfUb/LsbZ7R6ekUCXprSBn8y1yJc8FwFnbophrsU+lLkW
fYBpbXytel+LGLevhYZ4BpXzKtaG3Ms9RM4G2wd4e/X+J9qlf4kB1y84LR/g
cfJE/nBfecUzWiNsWJ2T/JvrL9olh1kvJ7TBveU8PDcWPlM9RlgLjR1yUWNt
l+oxuFtguGLMfrt6jNOHVx8OCHxIswXj8MCsYe9o44+BT212ZJwT+KLAVwa+
IfBQ4LsCPxD40cBzAy8MvDTm49+CFDv/4XiWRfG8cwI/GPa5Pex2Tdjz7MBH
h/0nhV+2D3+9P/z48hqsbSfU/5mzkR/IE+oo+QMH5BXXlW/UkvKQ/rz8JF9a
VmNW5zM1oDznjLr8J98zLriWORixZg7G85qDUSOpXdjfPhj7BWoX66zahb6p
XdRmatdN1WtGYljtwr/5frr9LvzoWVb4YK+e9cXzq+jSsW2gS9aJB1avE4kd
9xbxjbkWGmuuBffMta6q3pNHB/OdIPta2ML3gPCBtSF8sjZkLbA2ZJ/C2pC6
29pwhep6RS7qWoNt7Q9Qj7s2wXXzcHhsfg535QnalfyRV2jX+jHk4YbV64uP
VOew2sf4RPX1HT4ZC+Z81qzG0fbV4+sL1eOOXMvY3L16/FJHGOPfC/zj9vyM
nwQm/35rG0cFPq7Z7i3N9+KzAl/YbM24IvANgYcCTwv8YOBZgefF9RcFHg4M
t51n4qXxXPMDzwo7TA98c9jtorDn8WHng8L+u4ZfPhz+WiP8uLAG62L9TizJ
h0nVeYJGyR90UE6Nr843ajd5SN9JfvKRt9Sb8plzXPKcZ5D/cMy42Ld6Dpa9
ffID9yjRXrULDls/siaqXdhK7SLO1S6uo3bhC8/tkwO754hPPctK3NqrJ19y
nxEN8awXtZ89Ln7HOpF4MdfCr+Za7B2c3sa06j15+oT2tcjh7GvRY/EMKrmE
5+dZe6wN2Ru1NsTOd7bB+V7f92G9UK/Yy1Cv+KhXY6vrFeuIaw37GvYE3lJ9
bYLH9hPgq/k5/jVvh5eug3BRXsG/D8SQh/DM9Zffl7fcQ25/vDrn4ZOxsEX1
GNm6ehxtVz2+dqgeg18MTH/BmN2jelzDK3Vg38AHNrswDg1M3v6mNo4NfHLg
M5t9fQ9MfHngfwQeCjwt8IOBHwv8RODFcd+RwJAgf+78Fwd23WI8FPj2sM9V
YbfTw56/CvzNsP8nwy9vCH+NCT/+O/z7p/A7ua58QBfkCbySP/RJ5BRrsXzj
+vIQLD+fW5235GDymd+R52iK/D+ielyQO9nbJ0+zD/bu6trFmqh2sZapXfTV
zbvQFmtG1hr7XfjFfhe6ZK+eXNdePXnJpDbIb6wT4Z51IvmZdSLPYK5FXeyZ
LvTQnjz6Yl+LGsG/O8E9PIM6sXptiB2ntOG6a21oH56c9uY20CL1Cl/Yy+Jj
Lwst8nwp74arV1xHvWLvVb3iLId5Mrx3DXpT+JS5uWbBV9cyeOkax1orf+Cf
vMKn68WQh3xPfq5fnbfYRG5vVH0dZy2U/3DIGPlU9Tii72B8fa563LHOGZu7
Bt69eix/q3rsfz8wnHhDGz8LfHizHeN3gU8MTJ7/ujb+FviywP8IPBR4WuCH
As8OPD/wkrjvSGCIIh6O+S8MnBo4PfBNYZ+/hd1+F/j7YefPhP3fEX55Zvjr
9vDjaeHfH4bf8eO7YsgT4kL+kA/IKT7yDT2Rh+z9yU+eQd7CH/nM/OU564v8
J2aMi7Wqaxf9LLULXTDv+lt17SImrRmJsXxn0z1HfGGvnlrY8xLYx3P4rHHW
ifDNOnHX6rkWz+y7QuQt5lrTqvfk0R37Wuja5DZYx30vm+e3Nnxr9T48Nve9
Ra7/9zbYp1Cv0CL1io96RR2nXj2nul5Rd5tfsX9hfvWmGvx/RqtXG1XXq7XD
R/De/PkN4VO465oFR13L4KX9B7gof+CivGIdfXcMOch85Ce/L2/hlmv0htV5
vlF1/qNdxshm1WNny+rx9dnqcbd99djcOTB5ubG8R/XY/3bgHzYbrd18LYYH
r23jmMDHByYPWauNvwa+NPA/Ag8Fvjvww4EfD7wg8NK4LwRaHl4S858X+OF4
3tsCXx72OTnsdlDgL4Sd1wv7rxx+uTv8dUb48UfhX/unDDghH1atzhPW62V7
B9U5xc/l2ytqcG2Un6xZ8pbcWD7TZ5HnzFn+k28ZF8xT7SL21S564NaM5EJq
F7pkvwtdmtgGtbbnJdgvc58Rm1gn0p8y1yKHM9fCx57p4l4/aOOS6n0tcl37
WuRtnkHlWawNsbt9eM5O+N4i+ad6Na56L4uPekUupF6xBqlX7JWoV5xFV6/g
gXqFP6wHmY96RT6qXtEbtN/Os9lv5xncH3xN+Oi14bu1w6fwdUIbcFQOvKk6
N+CinMEmrolol/l/8pB/k59ol7zluq7R61fnOdol/zeqHhebVI+dzQNvVT3W
WBONwR2qx+ku1WP5a4HJtdSB7wc+oNmLcWhg1q1Xt/GHwKcFPi8wHFuzjesD
DwW+O/CMwHPiOgsDDwdGo8QjMc9FgWfHc00PfEPY4ezAh4fddg97fijsvGrY
/4HAZ4e/fhR+3Cr8+/bwOz1e+bBCdZ7wkT+s1/KKeWcPQR7iO/lJPSNvscOE
NuCePN+rOv+JW+OCulXtYr3I95vsd7Fe2KvHX/bq8W++L2+dyDkt60TmZK5F
TeSZLvJb+1roi30tfndSG6zdvgeEHa0N8YV9eHJR9YpcyF4WH/WKfMn3E6nF
1KvMr/CfekX+ql5tXr1/hf7av2K/03OkzMN+O/ez38513R9kn8XzDHdUP89A
vuX5qzXDR68O360VPn1tdV+/rno/AY66xrG+uvbBRddEawMGnJOH/Jvr7Luq
8/Y91fn8/uprN+ui/N+weox8tHrsoF3G1ObVY23r6vFIjm7M7hT4y9VjfI/q
+vCdwD8KzBmtNdr4dWD6CePb+FPgcwJfHPi6wEOBpweeGXhu4EWBRwJDSuez
NOY2P+b/cOD/BJ4Sdjg27POdwJ8Me64Rdn4k7H9e+IVY0l/Es37E3/qXNVq/
j63OB7A84W8OyB80S16xBsk34loeovnykznJW9Yp+YyGTGiDGJP/xIlxgW2t
Gen72e+iT26vHp9aJxKb9riobc216LlNbIPcwlzr5Oo9edZK+1pokWdQ0Qpr
Q+op+/Dklp6Z56NejauuV+ROvmdNTqVeYV/1yvc0TmlztR5EK+1fsVc+pQ36
Hvbb+W/PYsFh9wfhg/uDfKa1gR89fwXPZrSBPvheIXpoDcg64fn28dXrerhn
zgx39SnXdJ2Co3IAjsoNeCln4KJcws5yDPu4hvLf8vOd1dffd1XnM9olz99X
nf/Mz7jYsHrsoF3G1CcCb1E9BrepHpvbV4/lLwberXrs7xV4Uj1VbzMOCkxt
uHobvw18UuCzAk8JbA7NGAo8PfAj8f15gZcEHgmMXokXxzwfD5x6mJp5Wtjh
wMDbht3eGPacG3a+IPCPwy+bh7/eEH5kL29Zn62634lB+cAzyBNyHflDvSCv
WOPlGzVd7sXIT3rg8pYcUj7nOwLYTf6TuxoX1DD26sk5PC9BrpNne8y1qNPN
tdBtz3TRe7Inz+/Y1+L5rQ2Zv7UhmqNe8fG9xWdW72WRR6lXmV+hreoVOZz5
Ff1Y60F0yv4VPVXPkcIN++2sd76z87Lqf8OZe3mewffKbmzP7Pkr7mvPao/q
PfZJ1WtA5moNSBxZA+Iva8Bjq78/SDzYk8w9l1eFT3MPGr7KAfgqN+CleTu8
dE18ffW1Ep7/dwzXVr/HgHPy+X+q8/zd1fmPdhkXH6weOxtWj6mPVo811r/x
bWwZ+LPV43eH6jG+a+BvVNeN7wTG76u18fPARwU+PjA8f2Ubfw98beChwPcE
fjTwE4GXxvWJNfFI/HxB4Bm1fG08P5738MC7hH3WCbstCXxx2Hm/sP8nwy+v
DX89O/zIR/+SA+h3uCEfqJ3kyQnVuQOPs1cg38gB5OE11flJDOS5WfnMdeQ5
OYH8J56Ni3y3gtzAXAvfmWvRKxj9jtkW7dmsDXlGa8Mx1WtDdMk+PHsE6hXP
bC+LHEO9olZUr/CX9SDnTNQrai37V+ibZ+BZDya38ZHqZ7Hgv2dH0WHPM9C7
v7ANbHJZG6xr9qzgjz0rerPWgNjHGpB/84wo3PNMO/sL5lTomu/g8DvmVOTJ
/n0GeD+hDTjtPgs8tg8Ad+0PyF0GHHVdg6NyBl7KJXjpWrl2de5hNznpvzFY
O60v3l6d59xP/q9bPS7UU3PD8W18qHqsfax6DH4y8Kerx+x21eP6C4G/Wl0f
9grM+v6KNn4S+NeBOZf18jZOD3xh4GsCDwW+N/CswAsCDwcmALzv0pjDnMDT
Y/5XBv5DPO9egT8Y9lkx7HZF4P3DzpuE/deIsVL4a0z4cZXwL7qg38kV5APr
tDzJNW5SdV4RP/KN/aQ8byY/0St5y7PLZ+Yvz/O8BLXKhDaIc3MtOGOuxVzt
a9EjWva3eqqfQeXn9uF5fvWK389elnqFL9QrevLqFfZXr8gprQfRQc82sJ7a
b2fOnsVirfTsKHsonmfg+p6/QhN8rxAdsWeF3aa0wVrp+zjkOp4RXVw9p8q/
MwM/zalYQ82pwOZUYPtUI9X7VCPVcyrwkjbgumsK2LXmFdXXIDhtbxMey4HV
q+fk8NVcfXx1Llk/MOCl3MPXcpI11TXX7zHgp9zmHvL/ndXjQg01HzSOMtY+
VD0GP1Y9TjcLvHX1uJ4YeJfqmvD1wHBj1Tb2D/zLwJMDnxr4b4GvDjwU+L7A
jwVeGHgkMLEpXhQ/nxn4XzH/MwPvF8/7icArh32uC3zAKHtq59XC/uPCLyuE
v14dftwu/Iue63f2zeUDua48YR2XP+Sc8irPmBED8pBaTH5y5kne0k+Wz9Q8
8pxekvznHsYF+Ume6bKvRS5kbZjvfTBn9YrcxV4WPrWXhS3UK3J+9Yo8Sb2i
l2Q9yLpmv31a9X473HB/EG54ngGt8PwVk/adaDhjzwptsQaEM76Pg154RhQd
Madi3fQdHOxsX5283T7Vw9X7VGD7VHxnZgzrPvxo3cezLGiDdXZxG/Oq/31R
zsBbv1Of2pNctfoaBKddm+Cx+TbclRtw1Vwd7sol+CrH8JHcQ8fk5Gti+G9r
tt+R22+qzn/uZ1y8o3q8vKt6HL23enxNqB6DHwr88eox698uYXwm8A6BvxJ4
r/jdHwY+JK5/dOA/Bf5r4KsCDwW+L/DjgRcHzpxqOH4+L+Zzd+BLYv6/Dvy5
wKsF/mf87kFx/Y8EXjXsPCbs/5zwy7rhrx+FH+8P/24SfsdW8iHPcnAD+cM6
La/QBPmGduQ5CvlJLiJv4Yx8flZ1ntPbl/88g3FBTm5tiN2sDe+prldvrq5X
7J+qV/Slza/Y7zO/It7VK/IG+1c8o/12fu7+YJ4dxW6evxqq/k40eaM9K7Ta
GpD86dg26K+e1Ab9InMqcjJzKrTv4ja4h30qzlHbp/LeDNY7zypYi97b5mHd
R+/Mum9y9V46c1OjWEPVKGoQNYoYGWmDvQZzYPJzc2Ow9T6a5j4ymuaatWr1
PBxOu8bBY7mE3+UY3JV78FVOwle5ylo7vg2/xyAHk/NvrB4X3Nt4eXv1OPqf
6rH23sATqsfjhwJvHN/5VOBt45o7x712D/zdmM9BgY+oHrMnBD438JW1/J7S
/YHnBF4SGG7nz73vI4FvjnmeFPhr8VxvjOedGvinT2OrF8d3KvAqcc0t4l5n
x3zWDj8eVsvv//Nc8gHdlifkCvIn83N6LvLNs84M8jT5iZbKWzgmn8mf5Tk5
hPznO8YFe3L24ck51Cs0014WcaVesXaoV8SnekV+a7+d2HR/kHzoSzE8z8C6
4Pkr+if2rMgPrQEXVv9/mXGGxHefyXXMqdAXcyp0x3Ps1PX2qcAXtXF19bMK
/4jx7xjTqvfSyZc8T4XWeD7h2dU1Ct/bm8If9qbWr65RW1bXKOygRrHW+P/z
elH4BR6aG4PNmdE0c2mw/QH46Vr20ur7O2iavVP4bV8CHss9eCwnibfxbbwq
xhrV4/HV1Tn/2uqx8PrqMQJHjZ23Vo+vdQK/pwa1K/Mur7lJ3Ct7XBNr+T35
vWLOP45n+WXg4wKfVT2vuCLwUOAHA88LPFyDcS2eHz+/O+ZzQczzx4HtUTIe
juf9Rdhhg7DPC8JuY8KeawXeJ74zO373a3HNf8W9Um8968zYL56FnEeeEKvy
h39bdia2Ot/4njxEZ+QnMSVv0Qv5TD4hz6lH5D9zMy7Qf/WKnEO9Yp7mV/DH
/hVnqOy3E+f229EOz73zNys8f4WWfbn63/K1Bnxb9RoQO+/bBvYyp1q1el+d
ePQcO3F6XBv83LNVxKZ7f/jGug+7X94GNvU81Vuqn09g3bc3tVF1jeLZ1Cie
wTNU9HjVKPI8NSrzKPpfahQ+UqOe3Xzg+xnilav7ZeXw1wvDj2Bz6RdVz7HR
NNcysDzBVq59q1RfE9E0+xXY2Rh8eXWuvrI6h+2JrBb/9sr4HcZrqsfL2oEz
Z3hL9dh5Z1zn3XGfCdVj+cMxn9xb3DrmzLpoHH0tnitj6meBfxd2+Etg1nTj
bijwQ4Hnx/dHAg/H9WcFviHmc1TMc/OYf57d+k087/+GHbKmWzHsZn+TcUrY
+T1h/78GXhrfeSCu8+3w77S412YxnxVjzqvEc+0ZdqBXJt/y7D25Wb5HKW+p
W+QzcS7P0TL5jw2NC3qV6hX9LPtXPJv9dnp07g3y8TwDew32rIgda0Dsbg2I
nppTcW9zKu5nX5372aci7/JvYfFzzyrw/cltcB3rPp7L85/wyfMJ+ODSNvKM
OjWyGoVtb2+DjxqFfKhR6LN5FNex1uMZrfWIOzWKWmZ5edTBzU+MPwf+V3yH
nNk6nTXUfDh7kmD9i6bpd7B8YJ7m5C+qHmtg4whNM3ZeUj1e0DFjBP2Xwy+v
zu1XVOc8nDYWXhU/X7N6HK1VgzmYsfDmGux9OQf8a1xMqB4XH4n5bxbPRZ2Y
5+Rd378VdqAvbeygCfZYTgu7XRp4KPDDgRfWYNx5nUVx/elx37NiPl+OeT4/
5v/HeK4N4nmfGXZ4TtjH846MPBdxYNg5+2y5p7l3+Ovy+HnW8uRR+v1Tca/F
MYffVefPwpj/jHiuA8IOS8NuaI685XfkMyEhzzkP9p4YxgU+tX9Fz9/9QdYR
zzPgA89fsS5YA9I3sAZEa3Zq47DqORVriu8MHlP9vRtyEs9W0Quy7uO/7aV/
vfp5Kvquvr88oXpvao3qZz5Z49So/1TXKLRMjUI3zKPogahR5JT2o15fXaOI
azUKfVaj+Livt0L1/4cguqH+2Jf07Jt4x/jOT6rrGz5yHyT99bxRftS/cF6/
g40dsDGCvskf5mYsrFw9FtA311A0Tf7z7HKY9VRuo2mrxvDnr4iR+dj4Gqwl
vdfaNdjDd25vr75er1s9xidU14R8D4hc397I9tVrDdZI+yTkw9Yd8NOeCT1G
+71TAg+FzWfGzxcHxpdeZ05c//q470G1/PNv+V7Mh+O5Vqzl58P7h33eFXb7
Z9hzcdg59w3PC7+8LPz18vDhRfGzn1f3+3Bc86q41+oxhy/VoMbKN9ZBeUif
Wn4SP/J29xr8W2TanHiT/9jZuKAe9zwD54LsWfG71oDsb/ou4eTqORX9ec+F
/qp6j4r1y7MKrB3WffjIXjrX9zzVbdV7U+izZ9TRbc9QMWc1ChuqUbnHR55k
rUcOfEsb6KD9KOoIe+avqn7+HM1yX++k6vt636h+9gBNU6PsXzCyvqvAKwRe
Jb4Pj9Uo8tRFbbCGmuuC3c8F24cEGxecedPvYPmf5/3QNzkPlldo2vg2wK6h
6Js8R9PkOZomt18U+CXxnZfG764a13xF3EvNci99rRjOmbiV257FZ6xbXQew
kVrxsbAP66g8Zz3KPq2c/1nYGX66XhOz7pWjV8bFo/HzpfH94bjOfXH9P8V9
PxbzuS7muUnMf4V4rjwvd0HY4cCwz3DY7JSw55Sw83PC/huEX94b/poXfrwg
8I3xnVXid9eIa94U9/LvpDAWx9zujTmTo8hPviNvWa/l8wfCPtxX/q8QduZj
DUgeaE6FzptTkVfYp+J3J7bB95ft/VWv+6iz9m6DGLE3xfOqUfjbv7HA76hR
rHfmUfTWrPXeWb0fRU5oP4p82545ebj7etjE85z4N88eeD6KPWXPR7EeLa9P
zlqgtlDHmRexx6kWvTvwKvGdsfG7Y+Oa5C1qIH5SG9E9YwRsDgy2lge7voNd
u8HyAWweDm9d78DyKusL9E0egl1b8Z1rLpomn7Pfi6bJ5xfFdV4c189eR/b8
uYd5BWuHcWqO9fb2DK658Ns9brRrQhv4y74rOuCa67sWDDgmn+nNuv6Su1sv
0EeQ53BomzZmx89H4vsL4jrXx/X3iPs+GvP5TMxzXMx/nXiupfG8t4Qdpod9
Hgm7PTvs+Y2w8wlh//vDLzeGv8aEH/NMxRnx8znV+bBOXGfzuH7FfT8W87mr
Og+p0+UncStvH6/OZ2wrz4lZ+c9ablxgc/vq/Nw+Fd/3bBUPZi+d73ieCjv4
t2KmVD/zyd7HD9rg3moUeyXmUeTJ1nroqmc7iQt75nzc18N2nuckJ7mpDc5X
eT6K2PEMJ/y0B8U7F/79FrinPvBc6gY5v7mTtSXj6+177p+Ir6yuS/vE99eL
66Qu8WueF0X/7dWPr157kkv7HjrYdfyZ1dcasGsQWD6D5TNYPtOvkz9gefWs
6nwDm/PDf/n57Oq8Rd/kM5rm+vu8+N3M2V4Q9yI3s8bBJuYM6Ji8de+QsVoN
vn+kHfj9PGOjPqBdagucto9KTJnzs/eze/W/z20fld6oZ2k42+A+NXplf3VO
/BxH+v2ZcZ3j4vqrxX2/EfNZIeb5kZj/h+K5RuJ5N6vBM8Pa54Cw2wNhz63D
zs8I+08Nv4yEv14Qfhwb/mVd1u/0uuXDtBrsFXjNjeNe8Hx569HnqvNzcnXe
ct5RPt9Rg+/jaIdzqscFdbF1HzplL5052ZviWdzvg6tqFHY2jyIvNY9ifvaj
mJvvIxOTnj/P85zo7bltoI++x4ce2YPCF9Z32NkzUXDA3IlnNneiP6YmoGP2
ndBQ8xyupc4sra4/aIq6dHngb8b3V6meO2FPazpqY/O0tWMOK1XXJexpHUo/
4YHqf9PPdRlsLxE8oQ2w9QVYDoPlMNi17Bk1yGH7omD7D2DXTbROfoLlbeJn
xXeeFb+LvtmDzbwOzmRvzdpn5epcfVE8I7Y1J+F+7keTg01sg3rDs3zEgH1R
1kf3l7Gd+zVoxB/aYA3y/axDqp97Ia9wvwa9urCN+fHz4fj+P+M6W8f1j4r7
PiPms03M85CY/13xXLvE8+4UdlgU9jkp7OZeO2Mk7Jx/E+Py8Mv88Neu4cdP
hH/PC/yv+M6/43fRh+RP7hfIN/IWeUi9JD95l1Pe0huSzx+PZ3xpdf7zMS64
lhqFjqhR5KzmUZxzs9Yzn2CcWv39PuLZv/+5WfV3ZMZUf++YHNceFNe0vrOf
xSA/NHciH3L/jj68vXHqYuMdv9h3Wqm6VsB/dQm/qDOPVtelzJ3+FXgkvsP3
1SU0WV3Kv6swrbousQarS++MeRLX6hK3MQ9kPVN7V6rBvVd9BLZmB9snAZs/
g+0DgF3LwPmuvb2FfI8MLA/B8hMsb0djawq4ag8213T0zVqAZ7dfgaZlL86+
BHnaxm2wHtkjZR33nQi03TWRe3u+hfvZN2A/2jw8z6hwXfdT0AH3UOCuOTD+
NTceCr8vip/Pj+8fG9eZGtd/cdx3h5jP9THPT8b8nxvPNTued8Oww2Fhn33C
bj8Me+4Xdt4q7L8w/PLd8Feea/1m+Pdtgckf5APruDwhNuUPeiiv6BvLN2JK
HmJD+UmMy1vmLJ9ZG+Q5fVH579++YcAlaz10wH4UeuJZqTur/z9u4NKBbWQP
Cn2c3Aa1ln+/5eLq/38HfOr+Hfps3wm7/KcNzuzd3QY5mTXdcPWa7soa/JtR
agh6KseICzUH/qhF3wn8wuo6dlR1Ts6Kay6Je3FfdWmn6rrEnOQqumq+9454
FmJZDo+L54VDvvtDz9DaAWyfBGwODHbdWTF8DZYD5Jzm22DP6YHtf4JdE/Od
MrAcXqE6b8HWCyvGd1aM3wXLW7BrfWopdrAvgabZL4WHrpX4bs820DHfFSUf
870q+GyvknrGNQs+6Pc3h3+xkdpCDPq+57fCL+RG5vBc1/fch+Pn98b33x/X
+Z+4/jZx37kxn2/GPHeK+W8RzwUvfd6fhx08W7ldm6d2Oy3suUrY+UNh/zeE
X/Lvd+0RflwQ/qVe0u9nxXe+Fr+b59lyP5r4zXOh8nBaxfnPGvx/yC7Lo8Z0
nmNr+c+6ZlxQY3v2AB9Yj/Pf7uWRJ3gmijrL3Alb2xs/tHrfCe5NaQOuWtM9
ET5iXTGWiXF1iY+69KPqWoHPzJeWVucDva3Mh9SiD+fP4vv4VV1CI1wr8Z19
rdwr5PetMam15Sf75u4z8n17++Rs9tNeXZ3nTMu88XlhE+Zjbrli9X1V9MEa
HPyZNsD2PcDW9eAJbYDlBtooZ8Dm53muD+xaCXYNBbu2guXz6J+7XoPz3KD3
Yj72kMH2Tnlee6Rg10c4Yz2Fprn/i455Xvd54WvWHdeRl1XXf2oV83lyDs/J
EMvmM9RfvhcGJ63R4KhxwT38+Rnx/UPiOu+swXdave+YmM+XYp7L9pNbD9Xn
mh/Pu2nY4QNhn+eF3bYNe06owXPF2v/68Mva4a+s9R4J/94TPyfe5AO6JE/y
3MXFcS/ySPkGloc/rs5P8kZ5y72Sz/Kc+8p/nse4QL/sJfI79sZfEbZir9Z3
XmZWz2l55r+3QR5oDB5YXZfQZXWJn5uHEMNqwlnVayt8r57AeXXmwvDxXwOz
LqhLrD3mV1zHHCnrRHJLOYOdzZG+FnO7P8aSmD85Yq6tPiN9N9dieqaeqSDP
9/1EnkUeMocpbeAD62L8dUAbYPu6YOOF67jWgM2Twa65YPffwe49gV1/6aHl
35CUe/nOBdj+Ktg1dzTO71h3gF3fwdYsYLk6+lnM28Hus/Ds7uGuGP4lT3Mt
QMcmt0FMmZOQb9jfHh/3RVvs62I7Y5x+q7GGX+0LLY2frxvfHxfXgYte/9q4
75SYz3DMc5uY/7h4rjfE894Ydnhr2OcrYbfnxXONCTt/Ouz/1/DLrPDX4+HH
/w78WHwHHZQP1FDy5K3V+UNfQ15xffnGuiAPyR/k56ur83afeBZ0RJ5Tx8l/
1gzjAj11XSDPVJf4/TPbYI/g4jbyvTt8cWsb2Mp8g7rRnITrmiczB/OZG8NH
2etmHVGLVo2fb1ldu8iFzLXULMb34l6HVM/T4Ll9JGyqFpEPqUXUtGoR11KL
+B216Efx7OR8ahF7QmrRK6trER+1iPXJPYWdw7bkVae3QWx63gwOu16MjmX7
h+Oq5wDogHU92Hc8Ux/A1g5g4w5snwFsbo8b5CpYDuc+ONg9JvD4NsaMuo5n
jcDm/2DXUPCRbTAHe3rgPB/iWsCz+04EdjPfQMfy3Fq+o7RmDGucCfEs2NQc
Yyh+/mD8/MPx8zlxnQfj2hX33SbmMzXmuTDmPzOeK5/3zhhfDvv8PuxmLcG4
L+w8HPZfI/xyRfgr3/HMs6aHxLM8L65Dn0ieZM6MBskrtFS+EVPyEH7Lz4XV
eXtRdT6zFybPiRn5jy2MC3Ik9zuwleeayGXNY4lJ6x0+1m7kIuZIzFPdQD/N
kfx7T4y/Nfsy1gl8WXUtOqy6Fr0rrklvUS0iFzIvGqmeF21X/e/mcYZ8Wht8
7NujFb7XzNppjcZzmh8yD3uk5JH23PCf+ST1u2dW1wobVg3+fXjfv6bu8D3H
G8P+rFnuKxGD6mHmGOPiOuBj2sBf7sOC3XsaW71nO7YG93k/2Qam9ywQ2DUR
nO9H2DsdU4N71tYIo7XLOmK01rkWg+1dgOUw2P4zeGYbYNcdsGsrODXZXAg8
vnovLvc3nQ+1kn1p4tCznazRnitAr+ylHBI/XyG+f3cNvgfn9fN9yTyntEXM
c6eY/+rxXFfE824fdvh82OeOsNv5Yc8XhJ0/EPP5avjlKzG33GfZJ/xLb1C/
88zygTVbnqBZ8oc+m7wiJuXbStV5yHXlJ/OUt9hNPifP2c+S/+RvxgUaZCxj
O/ewXhd2oxZTN/CRWgRnRveNGM+PnxOH5lDYyBrN/YYn2pzMganjzM2wo/0i
+gXuD9KX9Lw6+Zy9a9Y7NfYV8YzYylxoWvVciH3hKW0wb/es0SlzITTTXIj+
v/vX7BO7p8DHPh73Mkcllz22jTy3Tw1uD5B1wXVzXNhhXDw72Nwvcwyw+RvY
M2xjq58hwSW+9wS2DwA2D8/YH61j7jWDl/2tyOp1AZ/x1c8NunaPxpl75HXc
r+ez7IxT9byCjzFb1WuEirnxsSczZhT2O6nnxJFrLnmXay77R65B6JV+3DZ+
/lB8/zNxnXfW4P/jyfseF/P5n8Cz4jt/iOe6N543c6yLwj7Twm4vDXt+Jew8
M+x/SWC4NL4N9FA/nhnXyflPrs4HYkaeEEvyh/pKXrGWyzfyJ3mIreQn2ihv
2R+Uz/Tm5Dl9SPlPXBkXnAuxRiOOzHPQAvXnwcBrxXd8n2pxe0Z7RCvH9akh
l9evRn/Vn4XV9Qf9Un+wrbXYy+K58KP5D/Yy/9m4ev5DT0OO4QvzH94BN//h
GdTza6rnP/jM/Iea2fwHf6s/8F/9oZ+o/vBv5rHos+dj8Yv7PvQ5PfuB7X7W
xriwc2LulefkXTvAeaZLe2aPFzylDbAaC05N+0EbYNdEPvY/+Zhj8FnWt6lB
/fHMDx/Xbj7WU3yMWT7mA3zsb/BZ1rsOO/Bx34eP9cjo6+c+lDkzH22e+V7m
YMzJ2if7V/fFz0fi+/fUYA8wfep9c+/evzmyag3W42vHcx0Vz3t62GGTGvx/
fHvfkbDn/XF97qX9iSv9wn6S/srzqOiL/oVn+p3YlA/kkfKE78gf5iCvyDem
tEEMysMPVOcndbq8Rd/l82er89z9QsbqgYl/9Yc+j/nPqtX3y6idzLWwg72g
ql5/sS5YM6Jp9rUWxJzJA815sIU5D8+Z/R81B43IdU3NIf7VHOosNYf5qznk
LWrOUdU1h2dWc8jT1JxnVNeciTW4PyVn0GE1h3zevx9IzqnmYPOftHFB9bOC
1LH2DPHvfm1gN9ej0TmVe0Zjq9fCY8NHY8N3Y8OnmfeCXVPAriNjwnej9c21
r9Ju8Sx8XE+rBjXN3gUfe798rC/4pF6lzliP8LHu4OM+ddXgnq91Bx/7SHzy
nKFrGR91ho95Mp8PtgF/7OGgV2rClvHz42rw7JPXeWFcf9e4b54/eTDmeUfM
/9h4rr/E854cdnhslH2028NhTzRIO8NF7U8/Wb8wf/2Fji3z45juX7iUfpcP
xLM8IT7lDzEjr3gW+XZ/PNd72j0Yub9P/Suf0Xd5TrzL/7urxwX18PQ2+Fgb
cmn3L8iTb2iD+FdnzBHNE9UZ8hl1hn6UOkMOo85wb3VmQvVcET95vvTe6n/n
CvtPbgP+WPOuXL2u57nUGXpw5jasEeoMObc6c351nWHPTp0ZU913rD3qDPpm
jxQOeVbWef+w2dc9bi40qZ4a+M58Hn32PNvnqtcC48Ju4CltjKv+NwbHVl+7
weafYM9RjKm+jwC2ZgFby4Bd18Cua9XmbT4jr/iYP/ORP3zsAfLJd6m0OR/3
4vm4N8THPi2fZX8DvAZzM7WCT2qgf1OIzy/a4JO6ZG+TT8a7/Rw+qZnWTeiV
/BxTg3sKfv/7cZ23xPU3j/vmHtx5Mc/nxPzHxHO9Mp73LWGHj4V91g+7XRT2
9Lw7A45qf9Zc/cK19BfPoB+JAf1Lv0u/09uXD9RH8iT5Q54kr1j3s0aWh/SP
5Cf6Lm/hv3xG4+Q5N5hST40J1ePC90kY9G48N4JO+b7M/WFb+tjW9dOrv19D
Xfa7NqiXj24D/VJPyL3MIclN1ZMLw488l3n1+6vrCX5UT1ir1BNqQPcUFlTX
E/bd1BPyUnMYuKSeEOPmt8xnUhusF2oLa5Xagm7ao6C+dV+D9cP3kvCN71FO
rn6ejXm6r8364RkebOff21mhBmtD7TAu7MOcPaObfaqxNdivVs/HVNd5sOsm
2J7b6DrRNYWPZ32r+nrEx3qcT+qV/Qc+9kL5LDt3VE+vjcve66y+J1I1uNa7
383HPIdPalH2yjJXTN1LTfNcIh85iV5Zs4yLn98e358e19ksrr963Dd7St+O
eX4z5v/deK63xPNeG3Z4UdhnZtjti2FPtFU7k69of9Zv/YIe6a8J1f3I/PUv
PTH9TjzKB/rS8oS6Uf78qwZ9NLkN4kIe0heSn3Bd3pKLyufkOfoi/6l/jQuu
qQ5gC89fkduoA9TV5hh8z1oGXVITmL+awH6t+Qb2VBOIqUlt8PPvt0E/UX04
tLo+oNXqAzpgHoKWqhUPByYXUjd4v1HdICdXN8ZX1w38qG68ubpunBIY37tm
wQnzZ3Ji32/dPzD65rtL/6j+riXPJiep5e0JvDTwCqOwe77jRuFd28CevlcL
3qWNsdXfU4DWvlcFzvMDO7Yxuve+fRt8suby3RA+W7fBJ/Mlz/7x8X0KPtlX
93wjn8yv1m2DT/bEfK+Ezxpt8Ml+ftaYeU1rCj7ZW3M/i0/mbHmewb+98Kr4
+Tmjvu91NozrrxD3HYn53B3z/EbMf0E813A872/CDteGfe6Lax4Z9iS/0c5o
jfYnNvWLZ4OscfUjZ6X0Lxql31n35UO+f4FWy5/J1XlFniTfqEHkIXvi8pOP
vEVD5TN5oDxfEphczbj4S+A9AmNb4+vFgS8O/KbAI4E/+DTfJxjEj8fPxwQ+
NfCUwPle1fqBvxx4XOA3xvOeGfgTga8MWy0NPDvwmWHbrQLPC/sfGXjTwKuH
v1as5WvIyoH5jlyC8773mmfax1X/GxpgOZnnRdEreQu2fwKW22Oqr+9g9+hz
Tw1sTxjsmUA+7uPzyd5UakLmOVnfZW6TmpNa5DmfqsE+vPk8H/MfPu5h8Unt
yvzN90f4ZD2YeZfPgl7Ztzw/fn5ifD//dsENcf3z4r6rjpqP81w75n9hPNdQ
DZ771Q7/HfbZswb/roL2pI7SznBX+5Ov6Jf8WzTooH4kZvQva7F+p+csH+jn
yBP4L3/QDXlFfSff6LnJw0/W4DtE8hZbyGf21OQ5eaz8x57GBXmXZ9jQa3Nd
9Mczunzs0RHL6jA5zbZt4Fc1eaT6+ZAnAvOZ2Ab9JXX7lsB3BSaW1fP1azAH
ELP+7dDGvMA/Czwz8HDgGwLvFfj5gf8UeOvAawZeMXDO7cWB3xl4+8CHxfPm
e81cM99bWb8NtMvzNnkWCG7LK7BrMdh34eG2Z2nAchVsb4T55zlP3+MYU4Pn
qfL81dP1gvLn46ufefD6fDJXkZ98cn9w2dng6r1NPrlHmff173jzsX/CJ/VQ
f/Exb8+/UzEU8z981PP6/ZG4zvVx/cvjvo/FfN4U88y/x/ideK7z43mvDDv8
NuwzJe02ptuT/sj4NlaO+f8m7rs4fv72eC7yN/1L30O/j1S/Pu9ryBP0UP7M
qsG/0S3f0FV5iF7JT3JHeYt95DP2kuf0nOU/Z1aNC+od9w54Hvc0iWv1kDXD
d8PZB/EcHXuEaiP1m9rI2uM5jfXDzjy7OTY1p5pJfqJm8u6hmsl6YL56fmBy
ULX09sD4Tl29rTqmFlVjXxL45MDMSe39r8DjAq8Q+JWB/zfwnoFPDXz/01x/
08Dkw3I4/24MOiY/wfIBHZMnYGsH5pn79XKMZ5F7YDmJXslVsPnA2Bp812Z5
PefUqzE1ePYptS7zMflZNVhjZn8+++S+j8DHur5qUOtSG1MPzYf/3xSr5Tw8
i/X+M8OG2Gh8G75Lxjg7fv6R+P6iuM5/xfW/GPf9Sw3qv/McG/N/bjzX3TX4
Lph2uC/ss1vYbd2w5wZhZ2JN+6N7+uUdMQeume+E6t9zq/t9bvwuDyBPHq/B
d0V9xmfV4NonD+Gt/FwjbEjOI593DpuQe8l/+l/GBb08a2HWiQltoMNq3bwa
rCPUOnR1ebkfNZS6R66r7qF16t7q1XNCenlqIFqkBtLXsj+A/dVDamH1EN+p
h4sD0wdTG3ketXHlGuy3qJP5Dql/j4OxfeDfBIbP6ueLAm8UeO/AxwW+LPDU
wM8O3+F383NiyrNAYGuE0X/nKv9ujOcAR//NBPmGjo1vAyw/wfIZHlprJCbW
jOWxcZ2xNaiBeTbJM0jwRz6Ard9H7wV4Zg/sHjfY8ydjwl9ja/B8u/GVz5L2
IX+W29jCPgw+sDeLf+0Dbxo/nxvf/0Jc58y4/mO1fA1/ScxzXA2+s+xzvTGe
97lhBy6kfVYMu70+7LlS2PlVYf9Dwy/rhb/eEn48IPx7TOBp8SzMaVntPKZf
J8/2nx/3Qmvk4UgN9g3kLZovn/P8Pz18+U+uZVyQa6ppL4vnxQfLzsCM6fkb
+YN6jjZmD0TfoWXqG/WgNTv+UN/YpzCvQ6vVOvxh/vy28AV7l2odOZNah63V
Ouag1j0zMLqv7sFJde/wwOi2GviSwJtU18BJgakT1cPrAt8bGJ1XG4cDM1Hx
0vj5c8NfYNfKfA8OTdPvz67Bdc0eBc9uLwVsnTL6b1XJvaf7m1fEhevvivEd
sHUEmpAa6HxWiPmDXStXqMH/75KxD3bfB+x7YWDPM3PffPfZvWyexTwHO8gT
dMk+DPsa9rFfGzaBo8bs9+LZ8/3BRwN/K77/1bjOq+L6e8Z9F8R81o95rhbz
vySe67Z43meFHfK8zc5ht0+GPX8Udt437D83/HJJ+Gtc+PGY8O/M8PsF8ezb
xHc4iyB/plTn1T/CJsxfHr65Oj8fjPmw3yafye3k+ftrMFc0LtjDU8fQWHWM
Z1HH0GR17KGwya+q52nzqudp2EdNoz+mpr21uqbRQ1PTiFU1DRuZv/FR04hZ
NY15m799MfxFL1R9o1aVJ28MzDlPtQ69Uus4x6K+3RGYtVKtWxIYzVHrllbH
2EGteyjwfwLTF1X3/hJ45eprzQvCdy+owdxAX5MTygGw3EDr5GT+nfb8fxCg
dcYd2FrgWdW5Cpar+TdFnxnXyffywK6nYNd3sLwCy4dnhj2fVX0/Aux+NHPz
fSiea3ob+c4X+a05CfP2PQie2fN72NRzLNTv7rnAB/uu7K+7dgyFX3J9mRzf
z7/FtFVcfyTuu0LMh+s4z1Vq8Iyoz7VbPO9lYYdxYZ+5YbehsOdI2Hk47H9r
+GWD8Ne24cdnhH/vCr/nmXnmKU/o58if7GeeUZ1v58S90G35iR3lLXvz2aOQ
5/R75f/N1eOCvpN1JXWydeWS6u96E8uuF9hIHcNW6hhrjfmYeed6bQ7qGGu3
OvbasDO6rY5x9ul/25hWXcc4s6KOTayep5Gvq2nUnmoadZaaxh6Bmoa2LNtv
GtMxz6umsc6pafcE5syK+nZBYHJC9e0ngb9RnVfbBOaZ1bfXB4bDcowczz7A
i6uv3WC596LqufcLa/DvpbvGoYHjq/dg5RUaKN/A8hAsP58/CvsdsGvr8+P6
z485pK4+P56L68sr5iM3Xhj+4hmtkbGJtQ/zsC7j3p4D5H7mIdjL2IejnoGE
H+rD5wKzvpvPcL5QPYHb5jxL4uez4/uT4jp/CDw27ntMzOeumOekmP/q8VwH
x/PeGnY4J+xzUtjtuWHPF4edtw77/zL8cmL464jw41fCv++qwX2BfIfI76Aj
WUtmvu31sa08vC3mQNzJW9Z0+cz+qDxfrzr/b4zn4pyQeRf/pnahLbmXpE3Q
X7Vroxr8W6xqF2ffzcGeW1270AvrSvI6tQvdntAGdas6Rl6qdtGbWnZuZ0zX
MfRHHYNP+bc11LGbq+sYOqPfs/Y8KDA2Uceo69Sx9wbGB+rYMwOTj6ljtwWG
o+rYXwL/NnDmw68If708/Lhq+HfV6n6nlnf9Aruugce3QU0kr8By9aW1/P/P
4CrVuZo/z++Pvo51Qc6B33c95fddQz0X/pb2++YGzFWevKa6r18ftuJ33E+B
f+Ye8MbeNXb3fCB9SPvV1HSebWa989wjfXXPtKNX/p2fefFzuOf3p8d1Dorr
z4/7bhnzGTNqns5/bDzXJfG8w2GH34V95oTdtgt7viDsnPu5vhvJWC/8lfXd
ruHf3cLvX4ifbxHff19cBx/Jq1wfx9XgnnLm/2vFkM8frc5zer/yHx8YF+ib
uRbXVa/QZ2tGzoxaM54RdstzGtRf7l2y9ix753RMz7uGq2sXfS3XCOagdsGT
CW1cXF27WFPULs6vql17hH/JydUu7Kl28VxqF9qids0KPFRdx7iv2sXZUbXr
iMDos9q1Z2B6C2rX1oH5N7Vr/cDrBkYDXUfGh7/Ghx/hnOsmWH6C9T+8cS1b
vfoat3p1Lq0+auTZbDkJdj3NM5OJx8d3uIdr7ppx7dfEfV9bncOvjzm/KZ7r
rdX5uU51Tr63OvfggRzDH3Jpq+o8Idb0u+8YMMhhzHWp2az7yM+zHrQfPqsG
+5B+P/sD0+L6x8V9PxTz+WfMc8eY/0vjucbE874pxv5hn4VhN8/WMHIf84Gw
/9Lwy0j4a1748YHAt8V3yG3kA+ev5MnVcf3r4748o3yjryIP7485Y095Oz+e
i3kuOxs8pvOfORsX5C3q1ZywFfM3v6L3ol5dEbYlT1CvqOPUq33DF5ybVK+I
SfWK33ftwBbqFWvNhDaoT9UrciRrRnIkOcD7jGoXa6XaxRzULs6dq12ceZJL
cE694t/Uq/UC8/xqV+7FrBmY+cvVVQKz1qhXz63BfEyNWinw68IXYLn6uuq+
e134dO3wNVgOgHPtkjNrV+c2vzO+DbVPLZST3EOemw+9sgb/vlOeKXpzYDgk
598ev7tOXB+uyPnct/pgzBO/uC5vEs+4ZTz7xOo5Br0kuc0abR5yYHVu/6Y6
t8mZ5Pal1euIocAzAi+M7xNTXmdxXP/+uO/fYz4/jnmuF/OfEc/1m3jeT4Ud
1gz7rBR2GxP2zDNXbwn7fyr88v3AJ4Yfr6/Bfrt+Xymu77olT5zP22KOaIF8
I++Vh+vEM741nj35zPNkD0T+s7diXMwPOz9YPadKjboy/HVudY06oXo9SK9S
vWItsB4k91avvlRdr+j1udaQB6tXaM2ENvgd9Yo5qVc8o3rlHs8GzZ7qFf0f
9Yo8Wb2iHl9errViYPJY9Qo+qFeJ+eS5azUqMd9Xo8YGXiEw91Wv3ha+eFv4
KP9+yNvCp2D5/7bq/gebY4PlCXFlLIDl1Tuqr4n4eHwbcEx+4u/UGfmMj+X5
etVjBL8aC/jPGNkw8EbxnY3jdzePa24d96KmM053ibl9Peb83XgWajTj6Nfx
vMcHPidscmXgocAPBJ4beGkN9mTEi+O+j8R8bol5nhnz3z+ea/N43tXCDveE
fU4Lu3037LlF4HfGd1aL331uXHOFuE+Ff/MMRvajcq85e5UvrsFehHzjHvKQ
Z5OfXEvegrOOkOfcW/4zJ+OCa6pRPJsa9dIa7OOpUWi4GsWelBqFDqhRPJca
hT6rUXkem/hVo7DPhDb4qFF81Cg+ahSfPA+mRnFNNQrNUaPQBDXqGYHZh1Kj
mL8ahS/UpZUDYwc16qWBjTfGaoGxubr06sD4Uo16fWBs53r9nvAL2PX6veFH
sP4F63dsLR+wtTwBy5/3V+cVPnAd/GB1Hno+ac1ma7mKfcdXPz8pt7GdnNcW
jE8GZq1PLTKOtqseX9RKxt2XAu8eeO/4/r6BfxLX/HXg4+K+pwe+sAbrHeeZ
50XvDTwrvr8g8HBgSOl9l9Zg/ZX7buKb47nOC3xk4L3i+5vFNd8U931GzPOe
wBeHX46rQZ3Uj18L/3IOVb/De/kAZ+QJHJU/8Exe8T359l/Veci85SexI2+J
KfnM/pE8Z/9F/q9cg+fnc7/GWg+sRvG76hLXtNbjXmoU+YwaxXzUpVWr51HY
V43CjhPaQHvVKOylRqHXahR2UaOIcTUKf6lR6K8aheaqS+4NbNTmKmZ+atQH
q+vSBoH5fXWJ/1aLPhEY/qhFW9Zg/0ot2jbwxMD0H+y1bhD23yD8smH4a8Pw
o8/sc+trsHz4SPW8mmeWM9nb5NnkGM8j9z5RnZNojlzdogb3puUzzyvPP1ud
/zzv+DbQIuPFdywZ7C8bU+yrGmt7B55UPTYPCMw7/MbvkYHZbzLGTwl8buBL
arAmWt75q+mBZwSeG3hR4OHAqV3ZO1oU83888P21fM28Iuzzl7DbUWHPH4ad
dw37bxZ+eU/4a81afg+cmku/o3vyAV3Nc1zyh79TI6+oueTb0dV5yD6E/KR/
KG85MyafOf8mz/G7/OfMg3FBLq0uobHqEvxRl75SXZfoD6hLYHUJrC7xfXWJ
60xog30HdYl7qUvMQV3KdyfpY6tL1JXqEudU1SXevVKXqM3Voh8ExibqEmuK
WnRgYNZldYl+vlp0aGD6Y2rRrwMTI+rP0YFz7+/YwMcFpjaxZ7tJ2JzfN4/d
NHy0afjuk+HTzcLX1BRyAG2RG+Q2coY+susaOiOv2DOSe2iO6yNnAOQndZm8
RX/kM3sN8py8SP7DAePi64HJGYwd/GhM4Ttjbf/AP60ej4cFzpjN/IEzvcb1
WYEvDJzvCN8QeCjwtMAPBp4V+InAiwMPx3zQrsTOfzieZWE875zAM8I+d4fd
bgl7XhX4r2H/k8MvR4W/Dg4/fru6f4ll/c46Kx/gpzwhhuUP/JNX7pcw0FZ5
SG4jP59Vg///EfmMhstznl3+U1MbF9hfLSLvVYvwkXUcZ5nUJfYO1CV6XOrS
v6trEWcS1CL2C3J/RC1Cq9UisFo0GqtL/K66BFaLuJdahO/Un38HznfembP6
c2vg26vrzx2BeXb1567A2EfNuTsw65H6c2/g+wM/EBj7e/5q67D51uGLbcJH
nwnf+f4YY9vw7+fC7xOr84G9BnlCXid/ODMpr9Ac+Ub+Iw/hsPxEf+QtNZrr
KWuN3P5WYGJB/rOmGBesI8YL+ZJx9NPArMvG2hGBWQuMxz8GJo8yZs8MfH7g
iwOzz2Xs3xh4KPBdge8LPDOuMzvwE4EX1WC/K98jFhO0PtdIPONw2GFR4CfC
brPDng+HnaeH/W8LfFP466rw40Xh37PC72idfJhcnSectZY/+Ete4VP5ht/l
IeuU/GQtk7fwMPksz8nb5T9xZVwQ5+oP2qH+oDsT2nhfdS3iumoR81CLqL/U
ordW1yKeX/3BRuoPNlV/XhcYf6hFr62uRfhV/cH3+S62GA6pP9kvSsz6pOaM
xmpOYn5XbRmN1ZnErxmFPdOeeK2nwTuEnXcM++8UfvlC+Gvn6uvILuHTL4Wv
qQvkwFeqc4NcV86gOXLpG9U5lnk4+iMn96ne56TfK28nVeczvJXn+wdmTTcW
yGONkV8GRpeMI85bGl+/D0zdYQyeGvjMwPSCjN+/B7488LWBbwo8FHhq4HsC
PxT40cCPB34i5rMo8JLAw/FcaFdi7QDWPsNhqyVhz4Vh5ycCPx5+eST89WD4
8Z7w79TwO3aQD9hHntDrkz+cw5dX6J58o+6Wh9Sw8pO8V95SX8jnydV5Th44
oQ200big3lF/0En1h/VO/WEfVv0hV1d/4Kr6A4fzrIuas0dgNFb9oR5Uf4gj
NWfXwMSd+sMZOzWHmFVndgpMjKsznw9M/aLOTAxMHqK2JKb3q4ZsG/izo7Dn
cD7zNJi8yHcJnw5vHfgrYduvhs13C1/sHj76evjuG+HTb4av831PNEdu7BOY
Wkz+0EOQV5Oq843eiDzMs8HwRN4eHPhn1bnN31+S87zDZSwcGRhdMl6ODUzu
ZEydHPjP1ePuzMBolLF5YeBLA18V+PrANwceCnxH4LsD3x94RuBZgVMrnoh5
LqzBsxA+19LAI2EH9Cqxdks8EnYeDvsvCb8sCrwg/Dgv/Pt4+J1nkQ88ozyh
RpA/9waeVp1v2G1CG9Q1y6u/yGnlM+/hynN8JP/RQ+NiSnXNwb9qDvmzOnNO
4PzbC+yzqDmsceoMvFJnTgwM99Qc+gzqzLGBOcerzsBhteWowPBcbfl1YGJB
PTksMPGSf7dBTJ9WrfhZ4J8Gpq/lWcGDAx8U+MDAxK/nDPcPvF/gHwf+UeC9
wp7fCrx32Pzb4Yvs3X0v/Pj98G+uL2iOHOC+coO5yRmeRS4dXINrmXzDbhPa
wLby81eByZHkML6T578NTL5kLPwxMGeDjZdTa3CNNqbODkyvxri7MPDFga8I
nLnBDYH/GXgo8B2BpwW+N/CDcf0ZgWcFfryWrw8LAi+K510SeGnYZ2QU1p75
dwnA2h+sX0ZqEOvHpeFf7qvfFwdGY+UJPXn5w7PIK55Rvj0WmPxTfmIfeUuO
Kp/Rf3lOvif/7w58Z3WdwS/Z21FbhgKjjWoL65Hagk6qJ9cHJt9WT64OzFkX
9YT8XA25NDB8Uzf+HpicU324IPD5geGwOnBuYLR3qzbOCnxm4DMCo8lbt/Hn
wKcFPjXwnwKfHPikwGj4Nm2cEHhS2O0HgX8Ydv5R2H+/8Mv+gQ8MPx4U/s09
BfRZDqA/8uQXgdF8uUROLseODIwWyUO0SH5ODszaJJ+PD4xN5PwpgbG5cXFm
YHxn7Jwf+MLAcMa4uzzwVYGvC3xjDa774nyP+PbAdwWeHvi+wA8Fnhl4Vsxz
duC5gZ+I510QeFHYZ0ngpWHP4cAjYf/EaJf+AuvH0Vi/87vyITH3kj/5Tu6S
wGidPETr5OeCwDyvfEb35Dn9dvn/eGA0UA1BA9WQRwKjh2rIQ4FZU9QQtFGt
uDcwOqlW3B0Yv6sJdwZGM43r2wPfGpjc0hj/d+BbArNW+n7WzYFvDHxD4H9U
/xto1we+NvA1ga8OfFXgKwJfHviywJcGviTwgWGHg8JWmeP9NHDmhJkrZg75
y8CZf2ZeekRgzlfLjaMD/9/2OuVb5s8nBs58+9TApwdmjZDnZwemvjMu/hb4
osBolHF0WeCrAl8b+B+Bbwqcezf5ftbtge8MfHfgewNnnvBQ4JmBH435PxZ4
TuB5YYf5gReE3RYFTn1YEvZfGng4/DUSfkyMXun30ViejMbyiuvIt9FYfg4H
zr+nlJj5y3+ey7hYFBgNVBMWBMZW6sATgbGn8T43MHpoDM4O/Fhg1ppt23g0
8COB0cnt2ng48EOBHwxMTf25Nu4PfG/gewJPD3x34GmB7wp8Z+CpgW8PfFvg
WwP/J/BQ4H8HPjSe5ReBDwv8q7DP4YGPCNseGfio8MsxgX8bfpwc+Pfh9z8E
Ji+SJycGPjkwOZIcOy0wuiQPzwx8dmB0SQ6fH/jCGsyx5f8lgS8LfGXgawKz
BhlTNwS+OfAtgYcC3xZ4auC7Ak8PfG/c94Fa/lqfOcAjgWfF884OPCfwvLDb
E4EXBF4Y9l9Uy4/9JYGXBh4OvydGc+RJYvRKXo3G8nA0lrejsTwfqafHxsvw
KDyxjaVPg3ne7dtY/DQYW32+jYWBFzwNnh8YX+xQ/+ffZp8beM7T4McDzw78
WOBZgR8N/EjgmU+DZwR+OPBDgR8M/EA81/2B7wuMznyhDfKcndo4KvDRgdGc
Hdv4XeBjA/8+5vCHwH8MfELM4aTAJwc+JXx6WuDTA5MjTWzjrMDnBEaX5Nj5
gS8IfFHgKdW5emngywNfGfjqwNcF/kdgcmxj5J+Bbwk8FPi2wHcEvjPwtLj+
9MD31eB6urw1N9fimYEfDTs8Fnh24DmB54adnwg8P/CC8NfCGozTjF/9viTw
0lFY/gw/DUZb5OHTYTRKPv/fsHHx/xWzd75zu9f/Hzw8Cvv3NJb+f8BLngYv
DrzoafDCp8H/T3H3/VzHfZ1hfP9dT+I0J5n0OGXS66R6bCeMnViyrF5MiqJI
kBQrCIBotwP3AiDB3imCnZl3dN7Zh8f7vViIofPDmfl4bAP37p59tLdQfAjv
JPtxPij4S/g+fC/Zx+0ufKfg21V9XtScb8b8F/wD+Iew+vOLMT+C34DfhN+C
fwLrNdovxLwLvw9/WDX/nUGfwPvhA/BBWF36RsxheAY+Bn8Bn4RP49ydgWfh
OXgBXoSX4VW4C/fhIbwGj+ENeAu+CG/Dl/G8rsLX4BvwTfg2fAfH+S58v3r5
73WyH8A7OL8P4UfwY1jXoPeHfgo/K/h5sveZftHC6lIb67P2X3pNflG97N0e
c+n50s+Sm47h04KfpHPUdO7oR1W9A/TDZO/YDqzm/EbMG/Cb8I/ht+C34Xfg
d+H34A/gD2HdF/16zCfwfvgAfBD+DP4cPgzPwEfh47Dunb4VcxI+DZ+FZ+E5
eAFehJfgFbgDd+E+PIRH8Do8gTfgLfgCvA1fgq/AV3F8rsE34JvwLfg2fBe+
B9+Hv4QfwNpV7wn9EH4EPy74Cfy04GcFP2/hFy3t56X+vIp9jr6O9Z2kX0t+
Ub3sX42Z5l+JeT7FvxxD69i6t9lu5tNk3vP8fszb8Dvwe/D7sPrzBzEfwh/B
H8M/hffDB+CD8GfwIfgwfASegY/Bx+Ev4JPwKfgMfBY+B8/BC/B5eAlehlfh
DtyF+/AAHsFr8BiewBvwFnwBvghvw5fhK/BV+Bp8Hb4J34Jvw3fgu/C9gu/D
X8IP4J2C1Tfv7aOCH8NPCn6a/Hsxz1pY1+zvxpT8Ivl3qvrvcy/5t2PUn5L1
XaXfarD/vY7Z+g7lb+5idzi7qbF6nG5mbmObTrqB6o//vZQfwOrPn8V8BH8M
qz9/HrMfPgB/Ch+E1aK/iDkEfw4fgWfgo7C69Jcxx+ET8En4FHwaPgvPwufg
OXgBPg8vwkuw7qn+KmYV7sBduA8P4CE8ws9fh8fwBN6AN+Et+CK8DV+CL8NX
4KvwNfg6ztcN+CZ8C75dsPrm/bkL34PvF6y+eW8fwDvJ3v+HBatvfxrzuGD1
7U9iSlbr/jiGVt/+KCb72zHPk/8whtY17lZnu7fZ7qo6Q7uf2W5m207utY1N
DdTrr7+NUX/+LmY/rP78fcynsPrzDzGfwYdg9ecfYw7DR+CZ6qvPJjVH4WPw
cVj3SP8UcwI+CZ+C1aV/jjkDn4Vn4Tl4Hl6Az8Nq1L/ELMHL8Aq8CnfgLtyD
B/AQHsFr8Do8xuOcwBvwJrwFX4AvwtvwJRznywVfga/iPF6Dr8M3YDXNe0Lf
gtU079udgtU37+09WE3zntPqm6+LB8l/E7NTsPr21zGy//nyKNk9f5zsPj9J
dnvVNzc22119luyWym7m82Q3U3Yn2cYXyXtp46s20N1Tf/61qv8OdFv9+beY
Q8n/HvM5rBZ9J+YIPFN99bmj5iisFn035jisFn0v5gR8ElaLvh9zumB16T9i
zsKzsO6d/jNmDp6HF+Dz8CKsRu2LWYZX4FW4U3AX7sF9eAAP4RG8Bq/jcY7h
ScEb8Ca8heN2Ab6Y7OO/DV+C1TGfxyvw1WTvwzVYTfP+3Ej2vt2E1TTvJ62+
eZ/vJHv/78Lqm/tPq2/utvrmJtPqm9sru7E7ye6q+uaWyu7no2T3U31zJ2V3
8kmyOym7jeqbe5jtNqpv7mGpgbl7uzWwbfea7v3UH++b+uP9PAyrP/tiZqqv
PkfUHE3+QYxa9MOY47Ba9N8xJ2C16H9iaHXpRzGnYbXojZiz8GzymzHnYHXp
xzHz8ELBatRbMYvwErxc8AqsXv0kpgN3C+7BfXgAD/Hz6RG8Bq8XPMbzncAb
BW/ieG7BF3D86YvwNs7jpWSf98uwmuY9odU079W1ZO+h+ub9vJG8L0Z9857f
SvZ1ob65t3eS3Vj1zV2V3VL1zf28n+x+qm9upuxOqm9uo+w27iS7jeqb2yi7
h+qbeyi7h2ygWtfUwP/P7jW1bqaqd0b2Xqk/3sNjsPrjvZW982qRr6MTsPrz
doz8TswpWC16N0Ytei+GVpfej5mF1aIPYmh16cOY+YLVpY9izsOLyR/HLMHL
BatRn8SsFtyBuwX34H7BA3iY7MczgtcKXsfzHRc8wXHbKHgTx38r2efrAnwR
55dW07wP6pj3hFbTvFey9+0KrKZ5P2XvrZrmfb6e7P1X03xdyO6n+uZm3kp2
M9U3N1N2J9W3fTFqmtsou43qm3sou4fqm3sou4fqmxsou4Hqmxsou4HqmxvI
7qlvX7d76lupe7u1jq952bfS61w1x7she2fUH++V+uPdk72fJ2H1x7st+1pQ
i3wdnUn+aYxatD9GLToQQ6tFn8bMJR+MmU/+LGYBVpcOxdDq0ucxSwWrS4dj
6BV4teBO8pGYbsE9uJ/snzMoeFjwCF7D86LXk318xsk+nhN4A8ef3sT5ordw
fi8kex/UMe+J7P3ZTva+qWneQ3XM+0mrad5n2Xuupnn/ZV8X6ps7qaa5k7Lb
qKa5jbLbqL65h7J7qL65h2qaeyi7geqbGyi7geqbGyjvi1HfmrqnvjV1T31r
6p76xu7t1rpp93i73ddNe23b1Df1x/ug/nhPZO+P+uMdO53snVR/vLfqj3db
9v6rP75eZF+D55JnYtSiozHzycdi1KLjMbRa9EUMvVjVf38ovZR8Mma54JU4
LqeSVwvuJJ+O6RbcK7iPn0MPCh4m+/GPCl7DcaDXk308x8k+/pNkn68NnEd6
s6rPu+x92ILVMe+P7L1Sx7xvsvdQHfN+yt7bS8neczXN+y/7ulDT3EbZbVTT
3EA1zQ2U3UA1zQ2U3UA1zQ2U3UA1zQ2U3T31zd2T3T31ral7P4/Wtbmv261v
fB+PfeP7eOzbKZx3XQ/eB/XHeyLPxKg/3jHZe6j+eD9l77D64z2XfV3MJfua
mq/qa3M+frdmIflszPn4fbPJi/H7ziUvVfXf1U4vx++bT15JXoihVwvuxGM6
n9xt4V7B/YIH+L0lD5P9vEZT7OOzluzjuZ7s4z9O9vma4DzSGzjvsvdB9p5s
Jnuv1DTvm+w9VNO8n7L3Vk2biZG952qa91/2daGmuYeye6imuYeyu6emuXuy
u6e+uXuyu6e+uXtqmrsnu3tqmrvH1qlpTa1T05pap6axdexb031dqW9tX8P6
/o3v3bV9v67pM4szONfaJ++A9sy7IXt/tJfeq3NVvW9zVb2Tc1W9q3NVvc/z
Vb3z85iFZM/5NIuFWYqRl2OWkldilpNXY2j9d50YenWKuzGdFm4zvRbuFzxI
9uOkh1Ps4zBK9nGj13Cc6XWcl3WcK3qMczvGeZ9U9T7Qapf3R/Zeyd43dcx7
KHs/1THvrex9Vse857L3Xx3zdSG7geqYuyfPxKhj7p7s7snunprW1Do1zfd7
bJ2a1tQ69c2veXPrmu7r1Dfe1+31Xq70urXp/k1m30rv0blpsu/Z8mcTvmeT
3bRZnN9zOO/nMOyL7L3Sf/a+yd5D2buq/513mNb/xzsv+7rwvUgnfpavO1o/
txcj92No/Y5BTPYwZiV5VNXXoq3HtBZTcid5PYbu7tG9Fu5PsR/PoKX9fIfJ
TTPC8czu4/fTPl9rOI/ryT7v46reh3FV78m4qvdnkux9k72H6pj3U/beqmPe
Z9l7ro55/2VfF7K7p6a5e7K7p6Y1tU5Na2qdOjZT/Wzr1DH2zfd1bF3um+/r
/i/7Vnqt6vfo+L4cX5/KTe/FTfsMwk2T3TT1jZ+9+j5tDud0rmpujl8L+TWQ
d8mveTph755fw/h1jPeW1u/wnsu+FmRfL/rdvqayfW1qX8dVvcv0JGYleSOG
1vPZjMneiqE7U3whhu7u0b09up/sx1PyINnPlx7i+GT7eNIjHP9sn6+1KTPC
/9b7sF7Ve+IW9+Pn0t63cVXv4aSq93NS1Xsre5/1fLznsvdfz9/XhdzUOh07
3++xdToHbp3M1rFvvq+b1je/nt1r39Qx3r81vVZt8/qU78vl9+KamsbPHUqf
NchuGj9Xldm0fTEyvxvZ1BzZOyCzOd4f2Xsle990vr2Dfg3g1wHeYf+zcFLV
u7KRrOb42pF9fWX7OtX+XYzJ3o5ZTb4UM82XYzpTfCWG7r6Ce6/gfrIfZ8kD
PN9p9nEbJvs4Z/u8jHC+sn1+Rzjva8mTGNn7s57G+7Ze1Xs4rur9HFf13rJ1
k6re80nV3Dr/82y1qvdyuXq5dXoObJ37pue51775/k3Hsc39m5uW79/4+rR0
z7bba1J+1sCm5c8afJ8m+7Unm5Y/S/VrT9lN43dI+L0R2U3ze6+5ObJ3QG5q
zmJV75Lvv30OvYdLVb2fS1W9w8tVvdvZvhaWq/p6Wanq6yvb1ya9Gsf0aoOv
xWRfj+lM8Y2YkrtTfDOmjXst7Z9fcn+K/bxKHuD4ZPt4Zvv4D5N9vrJ9fkc4
77L3Idv7o730XsneN7ZO+zqJkXdrne8Fp7VuUjX3Tb+PfeP9m/vm68HXR9P9
W26aX6u+6j1bU9Pkpqbx84VpnymUPj/1a0/thO/N+F0Rfj9E5vdDfG+Wvxfn
jk3rj3dgsWpuzlJV79JS9XJnmpqzXNW7uly93BnvfLavkZWqvo5odcbXYLav
6+xbMZ3k2zFtfSem+xrcew3u4/G3tY8PPcDxzPbxz/b5Gib7/A5x3rO9J+qI
90cutc5Wg7yfclPr1CvveZu+jaty39i00v3bXu/Z9tI0vudWep8tf6bg154y
mzYTkz8zbfqclN8Pyd8JaerYtO+/+d5Mdse0H/wzJmxRU3+Wqt2bk92mOd7n
7Ka2ZPuayva1SWs/7sZk34uZ5vsxdPdr+MuYNu61tH/+Xt1P9vOdZh+3bB/n
QbLPSza719RANcr7kL1b69QO9m231qk7pb413b+pWbvdv01rmu/ZJtXraRo/
R2h67cnPDqZ9TsrPRt2xad8D8b2Z3NQx2R2T3TGdD36/t6ljsjvWtkXeB7lN
f7x72aXmNHVG9nWRvZe2yL7Gp/lBDN2d4p2Y1+Hea7Iff8n9ZB+fad5LAwc4
j9ltuuf90TXPvpVa19Q3taPUt702jfdsbZq2l/u0/D4b31vby2cH+bNRd0wu
fR7K7340dSx/t22m+tmOyfxOL7/H647JTR2T3THZHXvVFnl/ZO+V3KY/3uFs
7zzdpi3Zvk6n+WFMt6Ufxfw83XtN9vNq4z6O2zSXuldqoM/vIJl9K7Vut76p
UaW+ve6m7fY+W9vXnqX30/jZaOnzUHdM3q1j/G4bO8bvsMlNHcvf123qWP6z
CfzzCH5/TPb7Y/xzWPzzVv8LsJOe/g==
   
   "], {{Rational[-1, 8] Pi, 3.1416926535897933`}, {
    Rational[1, 4] Pi, Rational[15, 8] Pi}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output",
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["04: alien4", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myColorMap", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ab", ",", "ar"}], "}"}], ",", 
     RowBox[{"Hue", "[", 
      RowBox[{
       RowBox[{"Sin", "[", 
        SqrtBox["ab"], "]"}], ",", 
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ar", "2"]}]], ",", 
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ar", "2"]}]]}], "]"}]}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   SuperscriptBox[
    RowBox[{"Exp", "[", 
     RowBox[{"x", "+", 
      RowBox[{"I", " ", "y"}]}], "]"}], 
    RowBox[{"Tan", "[", 
     RowBox[{"y", "+", 
      RowBox[{"I", " ", "x"}]}], "]"}]], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", 
      FractionBox["\[Pi]", "9"]}], ",", 
     FractionBox[
      RowBox[{"5", " ", "\[Pi]"}], "12"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"\[Pi]", "+", "0.0001"}], ",", 
     FractionBox[
      RowBox[{"14", " ", "\[Pi]"}], "8"]}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.3900265981289473`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJykvQfUnlWV9r8RlbF3xZ6xf47dsaCOE3HG7oxTRMdxMPZRLCOWUccSC9hQ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   
   "], {{Rational[-1, 9] Pi, 3.1416926535897933`}, {
    Rational[5, 12] Pi, Rational[7, 4] Pi}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output",
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["05: butterfly", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myColorMap", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ab", ",", "ar"}], "}"}], ",", 
     RowBox[{"Hue", "[", 
      RowBox[{
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ab", "2"]}]], ",", 
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ar", "2"]}]], ",", 
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ar", "2"]}]]}], "]"}]}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"QComplexDensityPlot", "[", 
   RowBox[{
    RowBox[{"Sin", "[", 
     RowBox[{"Sin", "[", 
      FractionBox["1", 
       RowBox[{"y", "+", 
        RowBox[{"I", " ", "x"}]}]], "]"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", 
      RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"y", ",", 
      RowBox[{"-", ".5"}], ",", ".5"}], "}"}], ",", 
    RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
    RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
    RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
    RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["06: crossing1", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myColorMap", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ab", ",", "ar"}], "}"}], ",", 
     RowBox[{"Hue", "[", 
      RowBox[{
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ab", "2"]}]], ",", 
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ar", "2"]}]], ",", 
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ar", "2"]}]]}], "]"}]}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   SuperscriptBox[
    RowBox[{"Cot", "[", 
     RowBox[{"x", "+", 
      RowBox[{"I", " ", "y"}]}], "]"}], 
    RowBox[{"Tan", "[", 
     RowBox[{"y", "+", 
      RowBox[{"I", " ", "x"}]}], "]"}]], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", 
      FractionBox[
       RowBox[{"5", " ", "\[Pi]"}], "4"]}], ",", 
     RowBox[{
      RowBox[{"3", " ", "\[Pi]"}], "-", 
      FractionBox[
       RowBox[{"5", " ", "\[Pi]"}], "4"]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", "\[Pi]"}], ",", 
     RowBox[{"2", " ", "\[Pi]"}]}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.3900266045581923`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJztnQm4pVdVpnclNY+ZEwhJKkpkEBGZIjKEUloRFGwEEYQk0Ao0KlMroLQY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   
   "], {{Rational[-5, 4] Pi, -Pi}, {Rational[7, 4] Pi, 2 Pi}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["07: crossing2", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myColorMap", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ab", ",", "ar"}], "}"}], ",", 
     RowBox[{"Hue", "[", 
      RowBox[{
       RowBox[{"Sin", "[", "ab", "]"}], ",", 
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ar", "2"]}]], ",", 
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ar", "2"]}]]}], "]"}]}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   SuperscriptBox["E", 
    RowBox[{
     RowBox[{"Tan", "[", 
      RowBox[{"x", "+", 
       RowBox[{"I", " ", "y"}]}], "]"}], " ", 
     RowBox[{"Tan", "[", 
      RowBox[{"y", "+", 
       RowBox[{"I", " ", "x"}]}], "]"}]}]], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{
      RowBox[{"-", "2"}], " ", "\[Pi]"}], ",", 
     RowBox[{"2", " ", "\[Pi]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{
      RowBox[{"-", "2"}], " ", "\[Pi]"}], ",", "\[Pi]"}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.3900266128300867`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJztnQu4llWZ95dlZc1X31jTp+VhdLKyHE/jOQ8ggiCoyPkMe7M5qoCACggK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   
   "], {{(-2) Pi, (-2) Pi}, {2 Pi, Pi}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["08: crossing3", "Section"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   SuperscriptBox["E", 
    RowBox[{
     RowBox[{"Tan", "[", 
      RowBox[{"x", "+", 
       RowBox[{"I", " ", "y"}]}], "]"}], " ", 
     RowBox[{"Tan", "[", 
      RowBox[{"y", "+", 
       RowBox[{"I", " ", "x"}]}], "]"}]}]], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{
      RowBox[{"-", "2"}], " ", "\[Pi]"}], ",", 
     RowBox[{"2", " ", "\[Pi]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{
      RowBox[{"-", "2"}], " ", "\[Pi]"}], ",", "\[Pi]"}], "}"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.390026622253637*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJztnQu8ZVdd39eZmYBobBTE+m6iVi1WUYpWkUdGa1HUikBAAoSZvOd1Z+6d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   "], {{(-2) Pi, (-2) Pi}, {2 Pi, Pi}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["09: flames1", "Section"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"myColorMap", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ab", ",", "ar"}], "}"}], ",", 
     RowBox[{"Hue", "[", 
      RowBox[{
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ab", "2"]}]], ",", 
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ar", "2"]}]], ",", 
       RowBox[{"1", "-", 
        FractionBox[".8", 
         RowBox[{"1", "+", 
          SuperscriptBox["ab", "2"]}]]}]}], "]"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   RowBox[{"Sin", "[", 
    RowBox[{"Sin", "[", 
     RowBox[{"Tan", "[", 
      RowBox[{
       RowBox[{"-", "y"}], "+", 
       RowBox[{"I", " ", "x"}]}], "]"}], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "2"}], ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", "2.5"}], ",", "1"}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.3900301864186544`*^9, 3.390030195972392*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJzcvQnYpmdV53l67LZn7JmeHu2eqSwklYRsJIRAQiqp9avvq33ft6SSL+wg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   "], {{-2, -2.5}, {2, 1}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["10: flames2", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myColorMap", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ab", ",", "ar"}], "}"}], ",", 
     RowBox[{"Hue", "[", 
      RowBox[{
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ab", "2"]}]], ",", 
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ar", "2"]}]], ",", 
       RowBox[{"1", "-", 
        FractionBox[".8", 
         RowBox[{"1", "+", 
          SuperscriptBox["ab", "2"]}]]}]}], "]"}]}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   RowBox[{"Sin", "[", 
    RowBox[{"Sin", "[", 
     RowBox[{"Log", "[", 
      RowBox[{
       RowBox[{"-", "y"}], "+", 
       RowBox[{"I", " ", "x"}]}], "]"}], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "20"}], ",", "20"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", "20"}], ",", "20"}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.390026625398158*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJzVvQe0XNWVrT379f9ev/f//8vJuYUD7na37XY7YBswImOySSaai8hRZAQC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   "], {{-20, -20}, {20, 
   20}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["11: flowers1", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myColorMap", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ab", ",", "ar"}], "}"}], ",", 
     RowBox[{"Hue", "[", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"Cos", "[", "ar", "]"}], "4"], ",", 
       SuperscriptBox[
        RowBox[{"Sin", "[", "ar", "]"}], "2"], ",", 
       SuperscriptBox[
        RowBox[{"Cos", "[", "ab", "]"}], "2"]}], "]"}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   FractionBox["1", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", 
       FractionBox["1", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"1", "-", 
           RowBox[{"Sin", "[", 
            RowBox[{"x", "+", 
             RowBox[{"I", " ", "y"}]}], "]"}]}], ")"}], "2"]]}], ")"}], "2"]],
    ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", 
      FractionBox["\[Pi]", "2"]}], ",", 
     FractionBox[
      RowBox[{"3", " ", "\[Pi]"}], "2"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", 
      FractionBox[
       RowBox[{"3", " ", "\[Pi]"}], "4"]}], ",", 
     FractionBox[
      RowBox[{"3", " ", "\[Pi]"}], "4"]}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.390026631446856*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJztvQm4pVV55/ueKhEQwVkcaAHF4IzMxXzAYigGkVJKoBCKUkAqoAKiIYkB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   
   "], {{Rational[-1, 2] Pi, Rational[-3, 4] Pi}, {
    Rational[3, 2] Pi, Rational[3, 4] Pi}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["12: flowers2", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myColorMap", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ab", ",", "ar"}], "}"}], ",", 
     RowBox[{"Hue", "[", 
      RowBox[{
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ar", "2"]}]], ",", 
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ab", "2"]}]], ",", 
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ab", "2"]}]]}], "]"}]}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   FractionBox["1", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", 
       FractionBox["1", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"1", "-", 
           RowBox[{"Sin", "[", 
            RowBox[{"x", "+", 
             RowBox[{"I", " ", "y"}]}], "]"}]}], ")"}], "2"]]}], ")"}], "2"]],
    ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", 
      FractionBox["\[Pi]", "2"]}], ",", 
     FractionBox[
      RowBox[{"11", " ", "\[Pi]"}], "2"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", "\[Pi]"}], ",", "\[Pi]"}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.390026632528411*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJztnXvwpmVZx6+Nmprpj04zDZBg4EozEFGhZFCcSjItLTEpRVoRjBQ5iCEC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   
   "], {{Rational[-1, 2] Pi, -Pi}, {Rational[11, 2] Pi, Pi}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["13: flowers3", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myColorMap", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ab", ",", "ar"}], "}"}], ",", 
     RowBox[{"Hue", "[", 
      RowBox[{
       RowBox[{"Mod", "[", 
        RowBox[{"ar", ",", "1"}], "]"}], ",", 
       RowBox[{"Mod", "[", 
        RowBox[{"ab", ",", "1"}], "]"}], ",", 
       RowBox[{"Mod", "[", 
        RowBox[{"ar", ",", "1"}], "]"}]}], "]"}]}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   FractionBox["1", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", 
       FractionBox["1", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"1", "-", 
           RowBox[{"Sin", "[", 
            RowBox[{"x", "+", 
             RowBox[{"I", " ", "y"}]}], "]"}]}], ")"}], "2"]]}], ")"}], "2"]],
    ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "1.5"}], ",", "2.5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", "1.5"}], ",", "2.5"}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.3900266372952657`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJx03QW4FtUa8P17B5sOA1E4KNit2Img2IKKgeKxwBbBwAADUAE7sbA4diG2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   "], {{-1.5, -1.5}, {
   2.5, 2.5}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["14: flowers4", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myColorMap", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ab", ",", "ar"}], "}"}], ",", 
     RowBox[{"Hue", "[", 
      RowBox[{
       RowBox[{"Mod", "[", 
        RowBox[{"ab", ",", "1"}], "]"}], ",", 
       RowBox[{"Mod", "[", 
        RowBox[{"ab", ",", "1"}], "]"}], ",", 
       RowBox[{"Mod", "[", 
        RowBox[{"ar", ",", "1"}], "]"}]}], "]"}]}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   FractionBox["1", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", 
       FractionBox["1", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"1", "-", 
           RowBox[{"Sin", "[", 
            RowBox[{"x", "+", 
             RowBox[{"I", " ", "y"}]}], "]"}]}], ")"}], "2"]]}], ")"}], "2"]],
    ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "2"}], ",", "6"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", "2.2"}], ",", "3.3"}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.39002663880744*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJycvQnYnlV1770SMkISEiDMhJd5UHACxVYlaHHCoWKtWrWNbVW01qFWPW09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   "], {{-2, -2.2}, {6, 3.3}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["15: flowers5", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myColorMap", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ab", ",", "ar"}], "}"}], ",", 
     RowBox[{"RGBColor", "[", 
      RowBox[{
       RowBox[{
        FractionBox[".6", 
         RowBox[{"1", "+", 
          SuperscriptBox["ar", "2"]}]], "+", 
        FractionBox[".4", 
         RowBox[{"1", "+", 
          SuperscriptBox["ab", "2"]}]]}], ",", 
       RowBox[{
        FractionBox[".4", 
         RowBox[{"1", "+", 
          SuperscriptBox["ar", "2"]}]], "+", 
        FractionBox[".6", 
         RowBox[{"1", "+", 
          SuperscriptBox["ab", "2"]}]]}], ",", 
       RowBox[{
        FractionBox[".1", 
         RowBox[{"1", "+", 
          SuperscriptBox["ar", "2"]}]], "+", 
        FractionBox[".9", 
         RowBox[{"1", "+", 
          SuperscriptBox["ab", "2"]}]]}]}], "]"}]}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   SuperscriptBox[
    RowBox[{"Sin", "[", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"1", "-", 
         FractionBox["1", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"1", "-", 
             RowBox[{"Sin", "[", 
              RowBox[{"x", "+", 
               RowBox[{"I", " ", "y"}]}], "]"}]}], ")"}], "2"]]}], ")"}], 
       "2"]], "]"}], "2"], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", 
      FractionBox["\[Pi]", "2"]}], ",", 
     FractionBox[
      RowBox[{"3", " ", "\[Pi]"}], "2"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", 
      FractionBox[
       RowBox[{"3", " ", "\[Pi]"}], "4"]}], ",", 
     FractionBox[
      RowBox[{"3", " ", "\[Pi]"}], "4"]}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.3900266444555616`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJx0nHk0ltvbxw0lJClDI6GIkgwllVxFCaESkqQMmRIlRXI0UCEqCiVJipAi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   
   "], {{Rational[-1, 2] Pi, Rational[-3, 4] Pi}, {
    Rational[3, 2] Pi, Rational[3, 4] Pi}},
   ColorFunction->RGBColor],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["16: flowers6", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"myColorMap", "[", 
    RowBox[{"ab_", ",", "ar_"}], "]"}], ":=", 
   RowBox[{"Hue", "[", 
    RowBox[{
     FractionBox["1", 
      RowBox[{"1", "+", 
       SuperscriptBox["ab", "2"]}]], ",", 
     SuperscriptBox[
      RowBox[{"Sin", "[", 
       RowBox[{"Log", "[", "ab", "]"}], "]"}], "2"], ",", 
     FractionBox["1", 
      RowBox[{"1", "+", 
       SuperscriptBox["ar", "2"]}]]}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   RowBox[{"Log", "[", 
    RowBox[{"Sin", "[", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"y", "+", 
         RowBox[{"I", " ", "x"}]}], ")"}], "5"]], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "1.5"}], ",", "1.5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", "1.5"}], ",", "1.5"}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.390026645997779*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJzsvQfUpWV5/b3BmmiiSTR/yjDzzsxbpsEwzNCGAV6qNEGKvaGxYmLXWDAQ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   "], {{-1.5, -1.5}, {
   1.5, 1.5}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["17: flowers7", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"myColorMap", "[", 
    RowBox[{"ab_", ",", "ar_"}], "]"}], ":=", 
   RowBox[{"Hue", "[", 
    RowBox[{
     FractionBox["1", 
      RowBox[{"1", "+", 
       SuperscriptBox["ab", "2"]}]], ",", 
     SuperscriptBox[
      RowBox[{"Sin", "[", 
       RowBox[{"Log", "[", "ab", "]"}], "]"}], "2"], ",", 
     RowBox[{
      FractionBox[".3", 
       RowBox[{"1", "+", 
        SuperscriptBox["ar", "2"]}]], "+", 
      RowBox[{".7", " ", 
       SuperscriptBox[
        RowBox[{"Cos", "[", 
         RowBox[{"Log", "[", 
          RowBox[{"Abs", "[", "ar", "]"}], "]"}], "]"}], "2"]}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   RowBox[{"Log", "[", 
    RowBox[{"Sin", "[", 
     RowBox[{"Log", "[", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"x", "+", 
         RowBox[{"I", " ", "y"}]}], ")"}], "7"], "]"}], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "\[Pi]"}], ",", "\[Pi]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", "\[Pi]"}], ",", "\[Pi]"}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.3900266500335827`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJzsvXnU52V93/02adI2bdM+TZ4sbdreNU2axFZjnzRtFtOJWUyGTUSjRCDj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   "], {{-Pi, -Pi}, {Pi, Pi}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["18: flowers8", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"myColorMap", "[", 
    RowBox[{"ab_", ",", "ar_"}], "]"}], ":=", 
   RowBox[{"Hue", "[", 
    RowBox[{
     FractionBox["1", 
      RowBox[{"1", "+", 
       SuperscriptBox["ab", "2"]}]], ",", 
     SuperscriptBox[
      RowBox[{"Sin", "[", 
       RowBox[{"Log", "[", "ab", "]"}], "]"}], "2"], ",", 
     RowBox[{
      FractionBox[".1", 
       RowBox[{"1", "+", 
        SuperscriptBox["ar", "2"]}]], "+", 
      RowBox[{".9", " ", 
       SuperscriptBox[
        RowBox[{"Cos", "[", 
         RowBox[{"Log", "[", 
          RowBox[{"Abs", "[", "ar", "]"}], "]"}], "]"}], "2"]}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   RowBox[{"Log", "[", 
    RowBox[{"Sin", "[", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"x", "+", 
         RowBox[{"I", " ", "y"}]}], ")"}], "3"]], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", 
      FractionBox["\[Pi]", "2"]}], ",", 
     FractionBox["\[Pi]", "2"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", 
      FractionBox["\[Pi]", "2"]}], ",", 
     FractionBox["\[Pi]", "2"]}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.3900266509549074`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJzsvXm0n2V19n9VqbWTba2+fanFxtbWvhYHVEDFIQo4IChOKAoYxoSQmZCQ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   "], {{Rational[-1, 2] Pi, Rational[-1, 2] Pi}, {
    Rational[1, 2] Pi, Rational[1, 2] Pi}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["19: flowers9", "Section"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"myColorMap", "[", 
    RowBox[{"ab_", ",", "ar_"}], "]"}], ":=", 
   RowBox[{"Hue", "[", 
    RowBox[{
     FractionBox["1", 
      RowBox[{"1", "+", 
       SuperscriptBox["ab", "2"]}]], ",", 
     SuperscriptBox[
      RowBox[{"Sin", "[", 
       RowBox[{"Log", "[", "ab", "]"}], "]"}], "2"], ",", 
     RowBox[{
      FractionBox[".1", 
       RowBox[{"1", "+", 
        SuperscriptBox["ar", "2"]}]], "+", 
      RowBox[{".9", " ", 
       SuperscriptBox[
        RowBox[{"Cos", "[", 
         RowBox[{"Log", "[", 
          RowBox[{"Abs", "[", "ar", "]"}], "]"}], "]"}], "2"]}]}]}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   RowBox[{"Log", "[", 
    RowBox[{"Sin", "[", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"x", "+", 
         RowBox[{"I", " ", "y"}]}], ")"}], "11"]], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", 
      FractionBox["\[Pi]", "2"]}], ",", 
     FractionBox["\[Pi]", "2"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", 
      FractionBox["\[Pi]", "2"]}], ",", 
     FractionBox["\[Pi]", "2"]}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "150"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}], ",", 
   RowBox[{"Compiled", "\[Rule]", "False"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.39002710056141*^9, 3.390027102674448*^9}, {
   3.390027936463378*^9, 3.3900279367337666`*^9}, 3.390030227637925*^9, {
   3.390030341061019*^9, 3.3900303514659805`*^9}, {3.3900304465727377`*^9, 
   3.3900304466628675`*^9}, {3.390030514410283*^9, 3.3900305295320272`*^9}, {
   3.3900306949298573`*^9, 3.390030700197432*^9}, {3.39003073044092*^9, 
   3.3900307311719713`*^9}, {3.3900307619962945`*^9, 3.390030762206597*^9}, {
   3.3900331409570723`*^9, 3.390033141417734*^9}, 3.390033209946274*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "150"}],
  SequenceForm["pl = ", 150],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"150", ",", "150"}], "}"}]}],
  SequenceForm["pl = ", {150, 150}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJyMvQfc5VV17v8YzU2uufd/k2tucmNMMjFNp/cZpr7Te2VmYKgDCiggoEHB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   
   "], {{Rational[-1, 2] Pi, Rational[-1, 2] Pi}, {
    Rational[1, 2] Pi, Rational[1, 2] Pi}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["20: fractal1", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "mandelbrot", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"mandelbrot", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"k", ",", "_Complex"}], "}"}], "}"}], ",", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"val", "=", 
          RowBox[{"0", "+", 
           RowBox[{"0", " ", "I"}]}]}], ",", 
         RowBox[{"cnt", "=", "0"}]}], "}"}], ",", 
       RowBox[{
        RowBox[{"While", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"Re", "[", "val", "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{"Im", "[", "val", "]"}], "2"]}], "<", "15000"}], "&&", 
           RowBox[{"cnt", "<", "100"}]}], ",", 
          RowBox[{
           RowBox[{"val", "=", 
            RowBox[{
             SuperscriptBox["val", "2"], "+", "k"}]}], ";", 
           RowBox[{"cnt", "++"}]}]}], "]"}], ";", 
        RowBox[{"cnt", " ", 
         RowBox[{"Exp", "[", 
          RowBox[{"I", " ", 
           RowBox[{"Arg", "[", "val", "]"}]}], "]"}]}]}]}], "]"}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"QComplexDensityPlot", "[", 
   RowBox[{
    RowBox[{"mandelbrot", "[", 
     RowBox[{"x", "+", 
      RowBox[{"I", " ", "y"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", 
      RowBox[{"-", "2.6"}], ",", "1.4"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"y", ",", 
      RowBox[{"-", "2"}], ",", "2"}], "}"}], ",", 
    RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
    RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
    RowBox[{"Compiled", "\[Rule]", "False"}], ",", 
    RowBox[{"QSphereRadius", "\[Rule]", "10"}], ",", 
    RowBox[{"QLightnessRange", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0.9"}], "}"}]}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.3900334023729696`*^9, 3.390033416333043*^9}, 
   3.3900334958872895`*^9, {3.3900335435658484`*^9, 3.3900335447074895`*^9}, {
   3.390033588159971*^9, 3.3900335888509645`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.9000559999999982`", ",", 
   GraphicsBox[RasterBox[CompressedData["
1:eJzN3QuYlWW5//FFaqKWh9TKMgWPeAyzUlODqTTKLFJLy9NganjI8FyWOZQa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     "], {{-2.6, -2}, {1.4, 2}}, {1, 256},
     ColorFunction->Hue],
    AlignmentPoint->Center,
    AspectRatio->1,
    Axes->False,
    AxesLabel->None,
    AxesOrigin->Automatic,
    AxesStyle->{},
    Background->None,
    BaseStyle->{},
    BaselinePosition->Automatic,
    ColorOutput->Automatic,
    ContentSelectable->Automatic,
    DisplayFunction:>$DisplayFunction,
    Epilog->{},
    FormatType:>TraditionalForm,
    Frame->True,
    FrameLabel->None,
    FrameStyle->{},
    FrameTicks->Automatic,
    FrameTicksStyle->{},
    GridLines->None,
    GridLinesStyle->{},
    ImageMargins->0.,
    ImagePadding->All,
    ImageSize->Automatic,
    LabelStyle->{},
    Method->Automatic,
    PlotLabel->None,
    PlotRange->Automatic,
    PlotRangeClipping->True,
    PlotRangePadding->Automatic,
    PlotRegion->Automatic,
    PreserveImageOptions->Automatic,
    Prolog->{},
    RotateLabel->True,
    Ticks->Automatic,
    TicksStyle->{}]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["21: fractal2", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "mandelbrot", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"mandelbrot", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"k", ",", "_Complex"}], "}"}], "}"}], ",", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"val", "=", 
          RowBox[{"0", "+", 
           RowBox[{"0", " ", "I"}]}]}], ",", 
         RowBox[{"cnt", "=", "0"}]}], "}"}], ",", 
       RowBox[{
        RowBox[{"While", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             SuperscriptBox[
              RowBox[{"Re", "[", "val", "]"}], "2"], "+", 
             SuperscriptBox[
              RowBox[{"Im", "[", "val", "]"}], "2"]}], "<", "15000"}], "&&", 
           RowBox[{"cnt", "<", "100"}]}], ",", 
          RowBox[{
           RowBox[{"val", "=", 
            RowBox[{
             SuperscriptBox["val", "2"], "+", "k"}]}], ";", 
           RowBox[{"cnt", "++"}]}]}], "]"}], ";", 
        RowBox[{"cnt", " ", 
         RowBox[{"Exp", "[", 
          RowBox[{"I", " ", 
           RowBox[{"Arg", "[", "val", "]"}]}], "]"}]}]}]}], "]"}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   RowBox[{"mandelbrot", "[", 
    RowBox[{"x", "+", 
     RowBox[{"I", " ", "y"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "1.49"}], ",", 
     RowBox[{"-", "1.47"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", "0.01"}], ",", "0.01"}], "}"}], ",", 
   RowBox[{"Compiled", "\[Rule]", "False"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"QSphereRadius", "\[Rule]", "50"}], ",", 
   RowBox[{"QValueRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"15", ",", "110"}], "}"}]}]}], "]"}]], "Input",
 CellChangeTimes->{3.390026657654541*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJztnQe8VcXVtzc2bNiNHcHYu1Hs5aioiF2xRhFsWKMYK7Fc9NrFaBS714sa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   "], {{-1.49, -0.01}, {-1.47, 0.01}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->True,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["22: kitchen1", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", 
   RowBox[{"f", ",", "g", ",", "myColorMap"}], "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"f", "[", "x_", "]"}], ":=", 
   RowBox[{"1", "-", 
    SuperscriptBox["x", "2"]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"g", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"x", ",", "_Complex"}], "}"}], "}"}], ",", 
     RowBox[{"Simplify", "[", 
      RowBox[{"Nest", "[", 
       RowBox[{"f", ",", "x", ",", "5"}], "]"}], "]"}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myColorMap", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ab", ",", "ar"}], "}"}], ",", 
     RowBox[{"RGBColor", "[", 
      RowBox[{
       RowBox[{"Mod", "[", 
        RowBox[{
         RowBox[{"Log", "[", 
          RowBox[{"ab", "+", "1."}], "]"}], ",", "1"}], "]"}], ",", 
       RowBox[{"Mod", "[", 
        RowBox[{
         RowBox[{"Log", "[", 
          RowBox[{"Log", "[", 
           RowBox[{"ab", "+", "1."}], "]"}], "]"}], ",", "1"}], "]"}], ",", 
       RowBox[{"Mod", "[", 
        RowBox[{
         RowBox[{".8", " ", "ar"}], ",", "1"}], "]"}]}], "]"}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   RowBox[{"g", "[", 
    RowBox[{"x", "+", 
     RowBox[{"I", " ", "y"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "1.4"}], ",", "1.4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", ".8"}], ",", ".8"}], "}"}], ",", 
   RowBox[{"Compiled", "\[Rule]", "False"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.3900266615301137`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJxkm3lczOv7xgehrElItkGUbEUIcQ2irIWQJEkIIdmyZSQpKlNCKCZKi2Ii
aVNT2rRO+7RP+7RP1oj8nvOb5/vH6fzhdc7rc7w+n+e57+t+X9cdZ6rFyW0H
BzAYjAPklyX59c+/8/VVlOW3FKOw/bb1A50GKCXPfGa2pwOMdfs1Q9weIuI1
d0rCtXpsyrwf/6ClALyBo1WTgxsRZHBn19PUOkgihaFZUQ0QyTuvP+9QCsfC
UY/PDa0D/17Xde8VIWDHvqr7HhgFbnHcbWfDWhg+CDQ4bVUO0canE+cFNqN7
TvzTSfdrwOXmjvw1vBn8lFS1vAsi2DgtWuRQIYK+gcO85TdSwHB5NGLQ8hy4
/knIl1ERwcxNJpGbVwNWz/4q16YOsFdldDWWVqHW1zLtDDm/RGmH4tYbjeBO
XSqncbISG2RytKq+FYG15vTSWXkVcP570OuwbAXUplTEl79sAkv7cKm9bgaU
fvWc/RZQBhftlTqDAhPBFtTvKf/TAY3K0cqT9EuRYHCv6u98ct7b3m0xoU0Q
SWIqvnaUgDUpvf7rz3bwDTeuingkgpKg0mXfo2JEPE4esN2pCFzbU4F3bhZA
9ogMT3ZDEb4GbJJcudkE9p+ms4lK0RDPNxxp+acATE89k43DE8F69zNi+Ns2
MN40fX39Nh8qG4etna8sAneg3AmH443Qjmzt/HksD0yj0ltpcW3gO8ov6Noo
gnCI9ZgZMwTwft05dz03H9xXRov17xSC8+pl/9rYbHhpfoi9X9AA9hq14VcO
J4J7sSqxn3om0rcGPH+19A343/2UB13uAKth2LBkj3T83nuuPWxXJUT6Ou9j
AsWQWAYcmy5Jgf8Ti/g//VvBVjCY96O4FsyINcoyqz/Ck88foL48G6IJl66u
li0Dv8M6bbkjHzv6RU9t9KgDo6LQfn9QFhg2k5fMOxwLZnhP1LHcTjBCv7kL
1/jD3HppWtTDd5B5Maf46RghZEce2X56C/lO8OfBsxa+hnj8uIwUPTEMH094
O3ptA1il34Wr+M/AW+6m5aSWBNE49rnX66vASFy8/M/Cm9BI6Dpzpbsa3NNa
O0yK8qCY9crspWMn2r6v719g1AZW7rb2V48jwJ37Z4bHsw7434zparyUBxsL
ybjXE9oQNSXkiOG7duRXOqr0LmuA94X7S/tfbcAG0+DwM7Ft4C3Jjpn+JRT8
OobhbaKzZY6Ft2a+bYVxoOoyQU8FWIMd5jX/zMEG7XwzmUcteHBYten7hlYw
hlcppqp74Em/jZe+WDejdbpwus850rf7sTntQ5vx6lhSsVhVjCzNS+dSdjRA
Y7BS+pAlNTB8ZBAdmNIIwfLVBpOnvoP52D06Z7rycG6zWeYpNID7ZJraXi0R
GILfinEb7ZGlz5I8da1D1YCXgxWUO6Bm/6dsZ1sjQs1+chSf1aB71LvkZJEQ
Gj1vc3tEpdg0kDG4oLYK3cMWvoh3aAGrn3i/781wtDXtnzUztxTJGYUeZccK
0XbMYricVyMk46tT9xUWIEh1f8XQry0IynkZY99ZAPM1OaP36X5CqMuquiXL
y+C6JJz/i9STmbZ/jfuWMDQ9CYiONA+Akmzmb6eR5L139eYOamrF7Bmy147P
aYbx2llznI+1YZ3+KtvInfVQbNCv2dC/BRz6/AB9nn684mLh9zRY0fcU0feI
rat71m/JRBb9Ljdd+t09Lw56fQ3ugIiek03PeWn5vhGWtbVQs5beS15Zei8l
i0vugvnJ4PeX1kFGLK2D4NGY9woZDeDQupnTugmyFh4e+C0FLFpnHq2z7vkA
X987RP+F0r7k0L4wUw1uVJu9Aof28SLto8bHmNdrH1TDmPZ9J+27f9S4xRV7
2hFEdfKG6iTeR270WaOi/+hKp2N0R3hyExiyUh1uozqcYGFxQKWQzAvVrQ7V
7e05bbXjHGrAozrf9D+dz9menK3RAQ6dizg6F6G63yMH3CJcFUjn6AWdoy1x
024OW90ECZ07JTp37A289sO7P4CpROc0STqnzsybaxMHiGBF55pN51orvMK2
6mwrlPpwQPbh8xKxKAeMsH9zQy2sWVHjWx3YlDNMyhnGTVWH6RvswJwo5ZKA
con3xy1uYYUQnD4cYwhGXl0ULgZjvZR7jINS7k10OjtI7utHCPpwUj1PV26W
UQ34lKs8ylXJjtuzM6LaIaIc5lIO6/jecM43LvwPt43UZI+Uvya66cN5lvmi
4I/jY2BIfUGW+oL80aazfpnV//ERnwknZoXFtUJEfUee+o5jxKEpMl0CiKhP
MalPWb6ZZF9M/J/Xx9cY4xJKjTV44PTxwRDzotUMbiX41DeZ1Dd7TklsVt4n
faI+y6E+Oz1Jz0E/SAD58f/25dG93g3yAxtgTn3cmfo4d0aky1t9kh+o73Oo
77Mbvl+JN62AgOaEMzQnbLDepxX+pAWSPrnCxj6RPy49E+wP0hziT3OIq9Yc
49SNtTAfJc0tzjS3CBv552eadkBAc04ozTnGnlteHHIvBEP/37nI+lEEM2JI
I5jr/52j2Il3K8dsCAFDUs9tYb6HLmvtobLAeohGDBw70rUZrIsXDD4eFeOy
h+Yjg6Y6sCa6eZ/7kwwWSuztd1Qgq+7gjUMzSb76kDNg3aQasB/tD738kA+G
2jCP/QdrsbKuyzZxejs4yPv6fV8LdNte33PxJ/rbnLJ4rl8+zNee4peVknrd
PZs1t14ES3b02ey0ejDqYkvW7M6B2oqsP8/+qWfHjMYM7+dgRWmwLz7ogHDe
KPPrWVUw3fnpxTrPckgubx2iWd0A+UO+21O3VWL15vThbbYt4H9sXR9dWQbD
hO4rMRXk923t/b04MwusMq8S+Ukp8NbtmfbraBnGzX3cuFqmHoIrPyV7rdrB
1NUz6t8rhP9iD5nkzT4w35K6bmZeAww/nByger8E6R8VhOc55WDZJm4b/LcC
TG/Tvc4Li6GQvPPeHV4LGKpmihJ+Nvi3WClDigthN+9PcDm5D0O0QiGp+i6E
oREHtl0pQIa9nfK+s+S+Wc0mVwa1oNtkVECSWj74+/sf2OJFeOEwSdWXWwdD
1gfTayUCRO1PqI+cVAGRt1uFfnYZ2MoPr/6Uz4U5M2Dt7BaSS+8cNi5cmw3D
uqoNjWeySL5ca6uzIAOMYsn8MT4BYAzi/Phe8gkSxmZNwQTSN8VN6cvtyBxY
8GdMWJIG4Z21+0d4d4KpaP+GH0DycpT2jtF3kmHzSH/rCRkh+L32Pp2RZI6v
fvhypSIRt9ex2r7sEcP8wK2R1TMKwYzTO5K8JB7C7IEeO54kgW+g2pSuRPLb
1IvHT92NgpifUf67ieTlbJX2opPE31Yez6xLfoOKkN5l15ntSH9Xv2jQbTHR
gyO70SUYMrkd3jHB+bB6sIjjdLcW7PGbgt9rPkJvrNDltzPJVT7NK4yGloJh
bh3IHqoFhtmzqsQRoWAYG7lL7D6hbJ7ucxf1TtikZWXu6FdB8lyIifn2owhZ
fFYcqN+BrKMmji62zZAvPG4/5H0zNjX+mJVEfIudmRFeszANhk4L94zXrMN3
n9UH3S3aYPm21PLO0lowXkyofiJbit4hhbZn97Ri28uPFTKxhLOmQ2qFL1OR
Xrnm6jHdFijb/NyXGliMqJLtmuq+HVj90srqg3IzZk8Js+gYJAZH55BgpSHx
vQ2/T+aXkZx9Y+eC8fsI13fWHHWyroZTk3WL6GojFp6PfZKaSeZLLLSo+56D
yz3NvR4DGnC1573P1FayV5TLDHHaSvp2e2TI7b3Er69pfDvoKITakAfearsb
wXG+fVpA/FJNr1lhZF0zuLmxM01sS6GvsSDq9acqGGp/Tz3hmwv2mej0vU/f
wPSD3W0jkhe8n23vkXnYgLaXqQZbhzQhdGTR5HGKhC/pm5zCj36Es88N+5vn
isCZUr8yfH4GfIwD3u8b1YCIB/YmZsuJn0QdnKmZGI5hl7xM3BZngD/44egS
kwocn6t07dy9NvTqXUDAh2ZEuHjotB1sQ2dEbtaH7fXgmmtOah5XDUn9tWaF
GXxY+s9/3WQVD37AmBO7/paD3Sh9rh4ofR4395u+EaMWg25J36P8TvoehuHh
U0PPiKAmK/2uA/3uyhrl/sfkPuErPScnWnpO2xqTYFOih3R6ryh6L0ODdxxW
JcktodI6RNE6bMixfrJ9RitYZ6V1s6J10441lvueTeokkNbZnNZZ0Xb1Fn8l
Mi+0Lz60L4x6k4iPp9PBoX1UoX18kXB4YPaHOnBp3+1p33kb2VqNm/1gSHXi
TnWSNcolydaqAkFUVzJUV+lmR98uVWtFOtWhLtXhEseNZ9fV54JPdZtDdTtF
f4BgYmg9WFTnMkOlOuduSVn4xsAfbDoXvXQuOA83La5xLYcanaMtdI70N7vd
HbyQ8NDu33Mn2HG/o3QR8WM6pxV0Ts/4jFfe8Zf4YZ+5ZpWNqR2NDtj14YCx
85a2QR2FEFFu8Ck3gryd30572QjDPpwxDhmi1tgTSXTxby5dtD+qP0ij6j8c
e+0a//ioMcknlHscyj2Dxt4tGpwcaFBOSignW97sKUn/Q/LzmH9zlT3G59Xh
3y5glPybw5zeg7770kvB4vyb293RJ+psTjeD+fDfnGddPda8bVc6bKgvGFNf
MDTguql01UC+j4/Il+WOj43sAKNW6jsi6jui/iMzV40pAbuPT/lEsMe4HBH/
x9e0Jf4hiStSYNPHBxc++T46eUktRH180/pcebXHpE7w+/js5hefFlz8QfJa
stSXjakvRx/jsxZkiyF/RerjStTHBZlV6QZaqTDs4/sSo8ehFYtrwaI5YQ7N
CVp+vltPp3aAS3OFI80V3L+uigdtisHrk0OMdvtaHiE5lvVYmltkaG7hNNlY
fa2JBLtPztF1tOg+nVIJQ5qL2DQX9Y+u2P2MT3TeJc1R+jRHaaW0Bn7M/QTe
rfhe7TukL3eSb8iPJDmDeVJD+1U9zL22VyW2VSN9y0DrG/pkHxhWP3l1jTtY
1aOTB+5PJToZP/naNeI3X6+G94iFYA5pVNE73wqZw0Ob1sfUoGN9qvrUB2II
tk+7JrlHOPJJZ8y8LyQvKQ7+3ez4keR23li18ly46gYsd5wjAie44Fb+OREE
Hrlscx8yT24p03dkV2GYjvbPwbLtsBnTvUfVhfBFOGTFEF1S/1/TilVtCsBS
3Wjlb19K9JWd9/VDOQbxfeaoXW8EO/qglmv/RBi/NFxYSfZnpsf7y/tUYyEQ
1PbTHNEG+RCDgxfjhSg8pXPlmjHJ8X+3Ns5VqwfHNvjVgXUl8Fhur+Rj3A6W
5e7xhyaUQvLLOa9fYRHsh15bsjmhEKzP0SPPfU6Ged7U9GuHCyEfMH1T9+Em
8Fqe6LA1O5C+LUyQwSiAs5nN6/W8REgK22pekf8etL5kxIWnefA8unRR9SGy
J/r6lO0jehDeES+sXC0gPKmLa9lOdLJ4Eov/l8y7benZfoXZ8Fap9Zqyrxj8
d3Oy1hokQt69Yc4e1Uz4vHOLeDipCXzDE+uPbe4A03/T+87L6eDJeW2VuxgP
1vA7q6IsxbA5UyjHyUvBzNL569cdJbqrO6WZ5UHyu2DsqU9TP6KlscasqKMN
zOGzz9Y+LIVhtEHF9yN8FBo5r/8am0/2ya69GsuzwChYl/NkUyzS+G9v3opu
ACNBUuk2OAAs2QeH6wPeYRmn4bIdkwfR3ryXfrdbwVjX453w+jVcL/oxt18j
OafHEJyYBvA3ra7Re/cclcevp/8kedj87qaCM7xqMNTzb//SdEPU1wk7ei6R
fLhv4dU7/AKs9BxVtutFJ8zb2od8TSe6q9thedguHllqy71IFoMiZ7es2rcO
8L5sqTrn146Vydq7PrW3Q8De9v2xQTE47V0KVUZNcG98ddarXzuynoc9Nbjd
CO8ZUxyjG0X47jhs+FK5NpivsXE350XCMDfhsFFQIeL3zYreNaAVczydPX0s
qsC+t0ipNSMOvfvvPl7e0YxnyqGHtJ62oltz9KIsXhs2ySy/FpxGeLGUF/SW
9FNpm6LOUc8G7Py1NPKpaxPx8Ql3l0UTnR3L1Ys5WAGj7/MvD1zaiPiSSz+E
bi/Ade1/6PDQDIyNl+3QI/vGn5KWhNuHSa4+pDv3w4kOdLNmKlbNr4Pr4Nv5
05itEA6u/3y0qgH15hP6v7CqgSyDGXXMRgD2i7chXUdLEXfm6ilGehXMVCdd
ab9N+lWWwnzpG4lh7H63vk8ugyjNZnvXvWgY51mcWjlWDF3vE/HXnhfCpqHo
vg7JJ/o7bbx63gjhnDp1aNzELGh/1pwop9EJozrux5+XSH79Ol7Rv/I9tM5+
5t+4VQnFey5up8xqUBbBzFxZ2Y4ziUPNB9t2gpXx1Ey5JBR8swqTzFFN+B17
4GWDaQ289rCyg3qqsfn71ZGQlGLbVwvW+zt8uByT0wmbK8bokQ3rfg9uwpqf
f17yNOrQRZ/70Ofew2WCnjwphYqp9D1h9D1qa5rmK1SQ/JMl/W4u/W6Us3aw
1rpKaNNziuk5WeU3e0cneMCZ3ovxTXovnxSnKR+W1YBD68CjdWDwEy9U9D6E
K62bOa2b+SKjfCeSD9nl0jqr0DprNHkUrcpuA4/2JZ32JSo8ZeAMceF/+riM
IZmekdaEbtr337Tv3NqCd2eukhzuJtWJMtXJfK8hzGVZZH+lujKmugpZ7Wwz
jfiTBtWhyf90uCK99JZbMeQXSHVrSHX7sC5w9osqwhOqc4aFVOfa9w0Ozrue
CBs6F3w6F5Pfx0aG54jAo3P0i85R8g/Nji+z2hFE545D504Q5c83b85HFJ1T
Fp3T9IRimQXCerDpXOfQuWb3O3zxahgXLMoBXcoBtqpKy7gTZeD8jxuzpdxg
DIhW77e/GUqUM4zNUs5or/9cqUT8WtKHS6LVr0/uTKj5D8fmbTkgkfS0g0+5
x6Tc49t7S1pcCf/7cDKlaHn5wGeN4FGu2lCuSj5JZHe4RMGccphNOczcwj1+
enY1BJTbGpTbvfbZRfEabWBFSjnPpJxXuld/Pys5D/w+viBqd19kI6gHv4+P
8M2SNI0SQyBfJPUdHvUdo9lu91YtqAS/j095i6d7td9tBZ/6mhX1NYXIqMMj
NfJg3scHTS7Ccje3ARrUN72pbxqpjNCZsvwdJNRnlajPpky8OfDPlmqwYqS+
bE59WXHEEXnFx4S7fXzcnJ364vXP/P/4Ps+hQ7FNr/E/OYHNXLmYn/AejBvS
XMGhucIwzLjFRIHow0iaQ2RoDlFc3stOmUz40Ce3sEobjS49J7mT5hwZmnMm
DmKpOtwm89cnF7WddPxx0bETApqjftMclRM/9emd58Vg14652qFaC6Nh3hNk
bOsgb/t5xoDrpA4lbis2xmaDvf1ksUpALUqeeN+zCibnaV1kn93Vhqhk9Tuh
JSQf2BYHH+jOIfq74mL1thZRM7yMGuRq4K+x6P1jkudZ8+ev87HOg1LeE8Xp
OiIw9pzZ8PuUC2z+Fnvq/emAVaOzj3NlFdI35U4yEpN9r3DRCQ3iA6LoP9cP
kHrqaC7hWWeJwYhin4xMEYJddHdPdFg59D83/nxlR3Ke5eQd85fEgcuPPnBO
uQzdqRPH/LhNcoNwzPmrP1ugnVZofOO2EFnTbM5peRA/yK/lTIurhfY5iRm/
XwmC3rwP27y1FNzzdnaPnxSBOWrgmuorRSi95Wh9I6v5n587fVHyjEbQn1ty
BxmF+MGaXvO0LBMaM5SGlte2wrD517o7LvkI/b3SOIVZD16nev2pSvLPFRWv
14/PA9tx8pPijf5gxv4a1LK0AvzGDZoGZmQfSY7Y8im6Aizj6RJN6xzYfLRu
SPPKwqumUZF6lmT+U68qBA4MILkvRz2n4BM0fgSExYcJwJF5MjxVi8wBf/Yx
P6U0OKsuXbTImMzX0ewl6WgAx3Klsqx5Mhh/sq3+3vMDO8o03jKvCjzPpPdW
L8jetnnUCC12ObhD1nYXbS4g+WXa+8TcD6jaZCrw29QCdoZ21BdOAswvBSWF
s6KQfrF0ih8jE4yXq04dXE32wfpBH1UfvMGA070+7yfWwbm9zteCQ/aePf1c
u7yC0Zg0/bDSwU44r4l2MSkiOSg8ZOIZvcc4del93rGlJeDcVFdxH1sGhtKh
kyM5ltD4NvA5m/gIv8W1/fHbLFxeOLRHU68TGueLfY9tSQBjtXq0g+lzdCdZ
y8me6kCyl2zDPFkRZMduuak/rRWWA+0e/uS045fm0X3rVNvg7HXwrFNnPSZu
HlXfGtAGO+XP/ZUeCMD7CpUTiysRM2+pe8yrVugeHfTa82kdzNP4B0PXCbBu
PitVLaQFrPX+KyNKCb+9sz5uNwnCGb8hemn3m+G916C9bq0Qst3xp77Et+BM
2TbjTFsxvCbcb0m5LEb3tFXJizLq8GJdl2XB8iZI1oyU//zkI5gD4t4PQil+
6/Ga7nc0IH1h0eT8A4Tn6W/HHMpKxtisi2vZTvWYYTfEb0ZXO6LUuYVuZ9qR
8f5E3tw/teh95BHXM7UIGiPyLDdtacCwcc62eetrYLl8FUtBrwnMRXXh1RtK
oXTyeUsEvwpK99c0yFvzwdiptSknNwZ8Az/+dfMyeI6wMOyvXAONpwoebnrN
0Bm89sfRtUVQqWNNeWrcAcPihr273SsgGyhMyzfLgZrKgrHJzSVgTWfvb34Y
Dt6xyqWhQxMgk1zeFb6yGRyjhNGnJPX4qnfr7+AVnTC84aocsEwADUYGw+v3
J6SGJmzVX9cM3pY593+3NGHb56l2LxWbsKvFYEW6eQ24qjanqtblwW6OUYZd
UhaEbcXOzl2f8M5JvXDplGZIHkxYvDtAjO/p2fnl/k2wG27h69H9DmO8pc9n
fpI+3xK/Paud1Q7judL33GmXvqe7YcdrpeuZ4NDvXqLfNW/TS+OYNkG+n/Sc
1fSc3RaBBYPUiU7ovRj60nt5f4i2DattAktFWgeGtbQOIglHW3sgeT+tmwat
G7Oj8eukofWwonU2pnVm53YNHXEuBKxd0r4U0r4w65bv68kkezHtowrt4xPl
16k99a3Qp32Xp313d7lytfxUHtKpTnKoTkIXvJ/a/2QDuFRXk6mu5rNCf+Sf
40GD6rCX6rDIe0fSHu1KMKdLdRtKdas0//elNUqtkO+jc7Zn4A7xsBywH0nn
4iKdC+OtxnIjKuvAoXO0gc4Rw6Y8TDTjNPh07uLo3HXP/uJXxhcSn5LO6WQ6
p5LPTr2nI5qgRuf6KJ1r+VOvZLJT4v/DgazijkXPOqvBbJVyg025sVJzreuC
sSTH/o8zE6Wc6RjzNbqX7B1cyiU25ZLtapVxGw6QfEA5xqYcswy6OeGL6V0w
KfcYDVLuGSsPSOWkloKRKeUkm3Lykp9cVAma/8NVVrZ7vF12KriUw3zKYevd
P7obUAtRH24bMk/LcRvJft2H8/rhbvfuGZSAkfZvX+DIzA1mBhN+Uh9hUx+R
tYhSrTmRBHPqOwLqO0pPAownPCc5nPpUFPWp7+Mm31/9z8/hqa8ZU18TD3Iw
OBtfAnYfH6yf/36wsVwz+NQ31ahvjipq39ySnQ4u9VkW9dmGmSqzdu+qA7uP
LzP2/DhQ++Y2GIekPs6jPu5om1o0SlwGFvV9Z+r7t9Y4rSrcT3yqT05gL5+O
zTlZ4PTJFYW7lbqK/vn7SjSHMGkOSZ9XPEL7EeFWn9zCfHrPI11GCPM+OefQ
gVtjmlYRffTJRSo3fIP3miSCT3OUMc1R89IGHbdfKQL/y52/z/cUQPxmF++S
oBa3zfLkpt4uAsfv9KLAgg5s0mzObP9dA1UPlu7L9EbYMFeuFi+uh2j2mi1+
aoTXSuXfIrLfg53+ataZiAIoamQm1W0XwbSuQvNhWBVERnbltvxOGLOGsyf1
kn2tqdC3pLIVktvzY33aGuDt/uyIwo5KyNccD25RInnk0QhJ0UchjNepzLpD
6vn1QnegMeEEw872SOvEaER1bXb3kiVcOq0VsSU8kOyNP+NztVrAn1XgvfCw
EJIv6oYaypVgfb0YoeVdA6F55Vl2RjF+zbpYfeM90eugwUNuPM0HP3KFKXth
EbI26x0705wHRqgw3iclAEK+tc/o5wWYlVObtH9lI7iP9N6e+dAM2fpvJ6vG
56PF+FbbgeNxkJ+6pqXNhszHtq6ahd4C5N6LNvcuF4F5itWb1lAMm3WJG4bp
5EB7qk3xhqlkDzt36dimH0mwEXUd+rAjE5In3Q5CXjH4/fV+xm/oAN/xQfKH
S+nQ9WEm2F0Sg79XMMlpFMktmwXJd16mQP5cyPuucamw2Xu7crlcLQTbhjwb
WZsEEVdlwvLHNWDb5KcO2kq4VRu3zHMCH+wh92+oW5P3jhvKvPHgE/h6L7YP
DooBc6xJuOyKYjCM5wyv/Hsf5lfXCmRGvMOCpQIj7psmGHY/Sasa00r6OaC5
KOEVlJjjhp8J5MOmV1ZJ5jzZB6vsnp5Tew4T7hKfa04i0q8zyVMDyX7we2C6
+hA3aA31yXmaSfaRnL2v4vQLsWD1rBOBrzuhuk8vtPUn6e8v7bH6ronwrGf4
pdd14OuE2Me5kxvAf3tsrOatDlxeOk+8dSQ5d9HDAF2z55DcG2itcU8M8VzO
oaGa7YjYYhS9270M4vIGyytssldMMn3C12sjOW1d/etdhG8Xby/q8BTCZnDX
SGwjOXVw68leXgpYqDSfF52GjnEXZhkatIB7aePKV69F8A4Y2hJ0vBNHaydk
hqwgvv68Ie6AsA1WFnUOmqnku7Yc85IJYmjcidqsOzkf7HtbdgZ/q8H3jsQG
hrgR5w6/PBRt1AD2hdnWCZ1FkFc5/jbsKdm3Ht1QdnMOAXuvupL97HhwtrR9
MEU9hhm72ASMqoDzASHLaVsbXliEfRyYVAvFpa797E63QL7/zPg6N8K9mhLh
BWYNPPdWvDH1IT7eGM2+TXyJYVKoouVXhduOnRpnLAk3Yobf17X4gKjrfp88
nMswxidjkqykE6xJ5UlpW1rA2b5p8YfcIjCmWIcYqpZCP9iT3zGF7G9l1Uev
QYBfG7w+7jQndctr3je35CMMZycsHa/wEfL1n7+5J36Co8fD+A8hRI97DsVY
+9xFVtYY+x3ydRCyS2Z2mhQj8/qGhR1GpM7epY/s9j3DlsJh+jNftOFxg/zQ
t2frEflkZkLISBE2PdHUmH6tHF7DJ+wwqSmGz4L1m7hlnWBMXt5d2XUEpoyX
E/c1HQHbdGFZw/ZqnBINbl1T2AgF3nrePc0GKLJyl5cNb4MDfT6LPjdmOE2d
50Fy+BbpexZpSN/TurVRYfhOEYLodwvpdxmCQ+Wlr21hTc/5kp5z4sc310JE
1TC+Jr1XE71X0KFt52TLiB5pHdhm0jpwDxUO2FFN9sF8ad04tG4qetdX+9xp
hiWtswqts7DwVZe/fQYsaV+4tC88yaeQXdbEd2KlfUymfWQsTj+a3LkPItp3
Rdr3iT+YT8YtKYUa1Ukc1UmSWL9ArNSMIKqrV1RXWyfm/pyyIBV8M6kOvakO
VY1i9eb+qAGH6laB6tb5xa5553d1wJvq/BfVublm8AK3nUWwo3PRQufCK2p6
aaWkEVF0jmzoHHFr8pVqdaIhonP3nc6dhtyR68+5VRDQOT1D59R85UBL/t0W
dNO5nkznWjK7/gTLLwOy96UcaKMcKBTumF4QVQsh5YY95UaGxYt3EzJIHuqR
cuY+5Yx1uku6eRLJuZRLWpRLXoYVs4YMaCLzSjn2R8qxsMApt/s9j4ZdH+75
JnDSfn+ugnkfTqpox/yeVtz6H662ue3bmfBIAJaSlMMiyuF537YFXqohezHl
No9ym2/y9MWuFcGEG//mvHN85CeXTyQn9fEFf4sxngMutkBEfURAfcT7zpUB
kxRywO/jO+m86/F3A+vBpj7Fpj4lL/njva01DEzqa2Lqa9xC15W5+mQ/pz4o
T32w/7sTDttt28AI+7dvfjuw7KLO9QIwB0t9Np367PbdphlrhzbBhvqyiPqy
7O5xDuKpiWSf/bePr3RqKWmyJXvoBanv86nvt0TFKbY6knmkOcGc5gTjYSN5
W74Ug+kqzRVcmit4I9e/vHW6CYwd0hxiTnMIwyLU2c+Z1JvmFiWaW5zcRx1e
GCz65+dh/59z6mnOGearu3/9oHbw+uSirEr788En8sH4Ks1RbTRHtRiOu3Ry
GdHNs+pxnoOfwbGI/Wapei2UXJZOnSYmPn5Ea/FJlUYEPR5677FBDVJVrKrb
hOS+lk+vaeUU48zy3TYKZ0S417F2WtfMf/6cd5wg+ZAvRJf6P66cT/aKXQtb
3HXJ3u2rMyT5LZnP0YOdp5yvhDbjqMEP/SaIkg15Ks6lsFH9sjAjvhxcz4bj
rRYkFwRNVwj5EwWb8tsGosFlyHo1ZtbuRSKYf/P8cSSD7Es2u/fd2C6EQK/l
6uEt7bD5KTkoShOBeyXk7bdnxbA8eLz81DAyRzHPM4oWC2D4OPra+R+F+OyQ
sOF1SRMkI8YKvNM6EXUvWHbttgJoTD/YIvqcDMmxxYVp2U1IX7F4w6mIPPTK
9HRbviL5SdPF7cNcEboHKfpnTBZgc/eNziuethCtGtMVnZEH+UDX8KvTszFX
dosZFpSDMV770AXZ15D/ErXNWzkDc5deCfbr3wrDIfdHaq4nuXLBHKfyqWlQ
qVzwXekdOZ/JgwxuE9H773sxW5Ykw/x7RM37Rw1gOTiNDLxB9KFjkTDWOBHc
kyGR8V3vwLab4FvklwdR9toMrX4f4G37u1bmejXYHUWMTJ0oiD4lu2/f/R4V
zk8W/E5vhbPfC/Vrpu3gR019q/c6HNaP1LR5w0meHPYg6EoK2Xt2/ekx/hyI
aRFxsYrTyHcNcvuVa5P9bpqg/vUbb/DeDLhzLTkYDLlFe3tMyN6mIRfp5GGA
aQoFoQxJOUQrrw75eywLk9mjzr3X7cQG9aLPftpkf220euzuE4jaop9zeRc7
4PX42RWmyyd0O5b3n/GR5Nw3+91PBLUjmau3PuBCLbr3HbTe09WAadY/P9zN
aoOqU+0m0XWSS0t47a5/qzE5MLkguaEVvNlRQwpIf7ktDx5lFxVg5izf42Gf
W/Bk1tB5SU6NYJn5WW6+/gGK2ZNDGZ+bwUo5Ofrm1ghoH7n3TMGgHWUbT6ef
rRVDMFilja9eCWfJ2JunUxsROX6SZ2xSE6ymlDnb5DdDzD/vZjJWBJ3xKaNN
PRrBv7btc1hpGuSzco+mXcrHiWOVS5w3NUB+7U2rP2Rv4TuteXg0Lxy9e9vD
PnbVISbDtnvFwQ4EldjdzZFpxe8Vy6ZaXyH3/DlITmJUhChP2aGGnnU4l7vy
vnKLCPaPtr3p/NwIxo8f7a1+JeCrFSnE7atCzOa5QdPyosE6UTuzlPUBvNMV
X12cylBSG+ic+qsKxsZ5jxZ2E75+m7hMVa0YDuz7z05/aAXntdqzcyR3G4dm
qESNzUNQhn3ieQbhvMRZ66FvNoRLfrjmTk1FOqf12aVt9fA6enKgnkYbmMzz
dTsSwsCQ9fe2wkM43z0/xvN5NW7tzl6UgHbi91dyPEaVgzWs0/OU2htYtWY8
SdzYBO0p8yxPipth46B+m1FUh4rtX3PcyP7IvG07pImTi6gP3eEc+VSUpMjt
27wnGf739s6d+qMRU49Xbrt/thkqi5XnVb0SQ+fe5cEf6sgeam4nd5roInmm
6bhzJRUQp+70nV/ZBC59/ps+f2K7LYW7VoBl9D1r6Hu+lzHj5dPEUIuXfndJ
qvS7pkW6AR9G5SOUnnOmkfScJnsPrEgleYExQnqv6/Rekut6J1ebpaOb1oFL
6xA6LFFpyflaRNC6iWjdvlYvi5VZ3AkBrTNDW1pnQZVajOyeEvBoX6xoX4SP
WFOS/xLfoX1so33k9Qac1RySCMZJad/jaN/jRoTG+jiIIE91kkx14rTzV0h5
ShuEVFeXqa52Z5o8ZiqTfZLqsP9KqQ5nXr/cc3MH4f9NqW4ZZlLdmu8qsGOG
BsOc6vwc1bnGua55A3mEb4nSuWDRuUjed+y6l1wLgugcxdA5Ejg8qFgRmQ5D
OnciOnebfre7PD1aQ/KedE7H0jl1HOw5KfdsOwR0rtXpXDPDmp9f+pwPHuXA
NMoBw5bnF+9faICsuZQbMyk35Lklmj2JgVC6IeVMMuVMTrzN8XfDyPmbpFyq
p1xatfuB6L1vM5iQcoxJOea066WW3aQ0MP/HPZaUe+PP3w/+O7gW5pSTjOlS
Tg5MGdn0/kAHlChXGcZSrsp7OzkXxxYhiHJYRDks87a1TvUC+X2dUm4zMqTc
1qi8sWhkBR/cPpyvT73mfmUl8X3qC3zqC9pv1Gelr2qHeR8f4Y6+r8PJKIQN
9R3GQqnvBHk0Op4OIN+dIPUpJvUp7YlDTS58+wjG6n/72uDfQkPRIXLPBVIf
ZAyW+uC8r0yZ8n/2I+t/+6bRd7kRi5+XgTlS6rN86rM27w+l6H0knKK+bEx9
mfuLfdTPkOQ46uNB1McdN69XSCP97ev77KQwRrIr8cFgaU6wozkh6Flh2OZ7
1YT30lxhR3OF12MnlbVNRC99cogMY87+USeID4qluUVMc8szo8ZbUXsb/pNz
WEp+5e2cMLBpLgqluSgycv6qmVsqwAj4d4767ht+/sfaFhiynNIvXRVDd9To
Cb/tCY9jMp0d8+6Ctb1AgZ1YBsuf1db1PiLYSJYb3XtbCrZlSPftkRFQ2ydz
yuR0NbIL7WL2riW51lJjaGVDE4y/qZ/b9rIS+l9W39pnT/Q5j9e+a0o5JJNH
4UJtOWzGBckEvyD9GqTU8HBcHPQrhLb88WWQiQsMP+vdCdH2Xt3wj80Q5FjX
rdsmRPfjB+VpliSHTbTZvIDso8Ye7wJ2uRVDMibzS8C4ZvAlnsOMerJI3VPG
W2SRfr1TXptc/gkag1TFodc6oHHkyaBm+QKkdk0+Z034wbDQ2z54GsmNFrtW
WZnmwfzVpIO/ZIMgOvzgUtKaCrDu+U08aJ6LbamMMxeXVoE1LivVc2AGyUUv
5byVs1A47/VRpw9t4Lus2uW8shOG6can3MXpeKzp4zk/ohCSXhfFJiMxNC7v
2L8rIwUv//IkCc1NYIUdUSv9UANJcYhjYXwSRMJK/waPFDBHyOhy/tmrVyh4
XbuZACvlHgfzj8Sv/xxmBbmnwLwm2Ofpy2gwGAxkb2KCO/PKt1u3SZ1SP+6K
qnkLpakNS6YuJPra6nJhRCWpR5BaxmFWKM7MvH3OQLMZjOWXheWb6sE6rGM2
OpoLb3eL/qNa0sCvebWi6mklmeuNejtNHcHxHfVsj08tNDJf33z+Ph8+h4b0
LvDqhPa655WvlDrBdk++OXBxAubdiYgzzu/As27N4XH1xRCHLx6edrgDk20d
eGZDOvC1h8FTD2uCxpiK54XFYrjf2ZAQt6wd485snxNtHQ+NgtzVRXPq0KFT
2yze34aLnV/l74RXQ74ix887rQwqrjvWm9i3osgiaIl54T85zGdYYXk2bKb2
jnZzb4H32Y111uNzwbhsVL5k3gtI/vieH+zRDNmzP35mRdTBWKZwpu8MksMi
DIeYOYoxRGuSHiegE5yTFvPux9fD8T33k9Zh4psh/ea9WF4CjevLA7tMK7Ds
QKJksXYjHD98XS+bTubwzBhDw7JsDOFuCf79rR6ie8Jwl0+RYDj2jDH65Ykh
eavq85/UYU6OSD6hoxKySgarNMk5gtYfd9+tUYv6+RtMtkwh+fbX8unCvbXw
NtPykrwQQVTGOseak010tuOSWWMRjL7qf2v8WYlf3sk2NbZEn3Lsmco90Ui3
05Zx314G/zOJb3zcCafiWYYxamRfRumbA6uKofJi6vJx80tQ+M0lvNitFoae
De/Cn+SBf+xX4LJbZB+fF+8Q5pYPK6su1Z7j6WDP3pLmz4qFv2XzBTvVTrA+
WK94/SsSoVZRwsxTVZAf6N5yckA9OF/iHFhk31f+yKtpGEb2gagpGpm1OTDf
wQk6UEDmckCprV3/LJQlX/Metr4F2ntmXBiqUYf0uJi8oda1EPqV/XVQF8JV
fueik+xC+BuO99pq2gkeu3rxc70OGJk8G2W6vAOu5yt3/N1SDnXv5uGHJtbB
2zD3yen+1bBOnfFxBLcDD3OCIpN6S2GoeHxJ6e5oGChpWilK6hBOn2uPkT5n
CHznl0wRQpu+J5W+h/n0efbovwHIpd89S7/L0R21g21cC51n0nMW0XOyPxdd
zj7rBaUU6b1M6L2CnG6ZilXLwY+W1sGO1kElYX3d/r9kzmndfGjdjL/qXLDK
+oh0WmcurbMw8/loZeIfEtoXDu1L0rKxh1sftJFcJe2jKe2jUG0o71arACLa
dw7tu8LckxZnrOrBpzoppDrRa3xmUGDsCnOqK1OqK9Occ5NXKJcSH5Pq0Ifq
8OSnk7WdEWLIU92GUN2yVpXcT3UgPusk1fkIqnPmwzvnubIkF9G5GEHnwq49
+VPOTOLvdI5YdI60b4y17jyZjyA6d8507qLU/4rnWNTDvM+csmXfanc8dgLD
XjrX3+lcG44dNezQfSGYb6UcOEM5wJ82N81/Kak/5YYa5QbrrNnUq6/4MKSc
+Uw5c8L5xnltVRH0KZc8KZf8PM6WWS1oQzfl2DTKsROyvU9DAgTgU+5pUO4V
Djj808+N8LQPJ1krH2xUafGBoA9XNVIqYvXJ+ZQoh9mUw+6mhxMuDWxBFOU2
I1jKbcGhghlJyzIRRTnPSJNy3v/kcMuTjaQ/vVJfYFNfmCM/Z2FPL/k+9REm
9RHXTLlfKy+UwZz6DqNE6ju/XdLvOz0mufWv1KdY1KeY/Z5MZs8R/MfXtjSF
LxYHNPzHB8/ecgrJGh0DppXUNw2pb/7w+Zuyuk4EJvVZHvXZHTNe7ba52gFD
6sva1JclE5NGq04iualL6uPO1McF26I7Rrg0g0N935z6Pi+oQMPKLxMSmhOE
NCeoLdBS8NlcB/Zgaa4wpLmCMWLU/bu1LjDvk0O8815/D3tTBhbNLeY0t1iu
OTnJitkCxsF/5xz5sIljXVoywO6Ti2oH1+5YNLMONjRH6dMcdeRZdPnp/cQ3
7fa0NXtVQ7t49uDLOSKoG7zs/GBLfG3a9Mn5zCRwm07M/JxeDecyS5Vrs3PB
n2QYNuAm8bPpR5drfq5Es1r0bjcjMi/zPa6fvFsJYYtJsen4Cogqkrlj67lg
vS3/enpLIqI6OgrWogyCwc4LktvIPUd0Hrmj1gKNOM+UTkshbs1twcihrRA5
pl1kbCY+NOpR9IPbxeh2jD+UcVcAxudwi0vjMmFcMNL3HckxzMmDP2sXEz9Y
duXDTu928B69+vG1hnBrjsP91m9vwDt+5f7BT/XQHjVq7LQxecif7v/RXIb0
qfiOo0uMEFyDHh9Fbg4sqxMWTuK3g+/wtlxikgjB8rSZz9QyERL2ZtjU0WTf
nJeQeTGyDeYvjOPtU9Pg5FERuH4X2Xf0tA97XyR73bLYXXMuJSPp4/niRVey
wDf1y0zUqYS8cIhP2bpEqLXXv4gvqQfz09iuZi2i397UifcM48Ct7mVujnsL
RmCJ2+ntr8A609/t3dJIsNVnyDa3V8OuDoeOfCF9mJO60moVDzIzdr35Q/ZA
oaGefgnhF9vodAszzh+s4JNfBv0huU5r49LPQWS/Wbs4756CJyTjXRr5LxrB
2JOz6uGJYjhzSJAvInO4ZnjSa2EsGL/yFVP2peHiQi1exbhODAraPKRtGtlP
fjOvFzbewTmzsE7lXR2w2dl76jiX7OeMK7uF01sx+W1Pz0KPdpi05MXz2vPg
fd/NanJsA5Tjq+vPJrdh5Y8qxZ3qDeBO3T6jWkeElrpz72d1tkLF9+CyJ3+e
gz/bpWHFmiJcvvvp3L2RrRg5T/BB9mIZuKYvhm93/YgKLzeLv9NJjtJiyg5U
I3wd/tbE6HcHqmRdDN6qN5M5mWmVMSkZ+jJLjjJaxLCuXbdryDQxlum+bxhZ
S/aASXn9L4XUYkFE/bAzsk2I+fHg4qn4NrCn1LIYl4V48US43L6iAcLzm050
6ZJ8zR/7fcqjVHBeP3At9KmHWoV16bToOkT5rryyqqkDCvIyu+fo1yFC83PM
4/edEGqfqLEdIMbv69zvQytrYD5s/dcupxKwV1Z/D/wtgkDz8+I9u0UQurUt
HS5sAqf1ldYLbgEu7y0W7D9NctjsnMvHDieA5flGJj/kLXg2vLUfJKU40/rM
NYD0PZTZ6YMycv9PG6+4KxVjWUJY187mVhgbynDs6mtxhnH028DcPLwJl2d5
1efAkD2pZ3FtIfHf1YOeF39Cfy8VRzkzktNqQtsUjgXARkldUSMjDt1+eywv
FnYgdEjCKRUrsr9G9zdZKjgNvvPbX5YrCM+yjxZVfRFihfdr6xU1xM+n27Nm
rSN73FBbrWE7b0PVfbw335LoybNdxzT/JuKaFC/VbKjDEd2wSZMWVCFdf0rv
cLUSCEUPgld2pWDr5ZUX77KSoTXCaumkI41Y/jgo/fBcMUydL+7hDiY8M/6a
nPY3FjcPDRUq9JD9QDXsxU6bROh4Hot1u16PVyYOFhu9cnArPXDh6fm1uDVU
bPrMtxJl9HkUfR7863bePM0apNP3VKhJ3yNboeZS312M4/S7T+l3lc/uHzyY
3YaoGuk55a9Izymsm++WMrcAtfReD+m9CvdIGD+P1YOhIK3DIloH8fphdyK5
nVDKkdZtE60bM3rA1bDTRRD8r858aZ0Fy184bK5pALtO2hcB7Qurdm3M7qhn
YNM+BtE+bhNG/mhbUwor2nc27XtmQU9n2Tox+FQn/lQn7wasX+pvwgfrrlRX
fKqrY7deLZ0rJwKf6tCZ6jB/2SGOwug2eFPdFlPdcutnujYYCNBNdd7fUarz
XxfKB97UqYeQzoUSnQvG49NrRo13AJ/O0X06RxK9mZtj3IQQ0Ll7RedOP7lU
/+beJjjTOV1M51R+DWv2mWExsKJzbUvnOvm8h5ZoeRXMKQdElAOPznw/6NZN
fHCvlBsiyg3WvKvVrfsyIaKcsaecmdI8zq8nqRY8yqU2yiWTF7snzuR3IIhy
bCLlWPGo1cXbXxZBQrnHpNzrGfKAqypH+tp/6v9z8iLlJG/zcBuv3zFgUa5e
plzV51m2qsZUg0U57Eo5nLMrfnCLYxsYi/7Nbbus9gX2b/IhopznU847z9Z3
unztH/382xc0StcdV2C/AyPo3z7SdrT90lDS976+o68i2nPOuZ3UTepTzP/5
1MzFbxwMi6FBfY1FfU11pMWtNyKiN+qDNtQHd8S4mZc+y/iPb64NfnHh5DmS
/6nP8qjPigb1sxj7Nxz8Pr6cFfXebuWPavD6+PirmPluq//5/x/6+D4juWuC
+/ZiSGhOENOcwP3AzVKOJHv9SGmu0Ka5gr9hF+NjwUfwaQ7h0xxi+Du20Veu
9j+55cSbL/fiu4jOaM6RpznH5pXbpoad//x9VGkuCqK56MSnxsgpi8QQ0Ryl
Q3PUnORtq2QXJYG9funjETJZsGHv0j2uRPLHWYXmVYsbweL75p541Apx9p0X
spuqsOBC4zh9s3dgu717ITkrgsalxW7NByowJ2PS2AlBVWAxRy86vjsVGj6Z
dtNJbta/G5ObMLcNnLva7DF/W8DLOTLRzZXk1DmX1MeQvYa3tn4JK0IE42dL
C5Y9LYZjge/6ypOE339i101uzYDNa94VlzeFEF3a5DE6IB6M0+M2b1VqR9S0
DJvp6fmQuZC766gL0Q9/Ytav/Dpwd9R//lVDctjj4WN+6XSA/4wfHl1TBH5m
f7djMmTvVP5TO5UpBCOrRfNdzFsYDhDMGLEzA/EDhWtm9jRDonHdZOHRFnD4
xrM+x6VCsuJNZ0NBLhhWxk2CDNKHoWk2e3Q+wvvhzcmtnxvAv7Lv4dd5JeBG
K252CU1AtWnP7f6biW8Gxv11nEP66bmcJT82Gl9/2t4vzq6B7CPOwlt7Sd/0
FJ9dfPMGXpMsD+jc60R37vu5FWuIfrmLVmv/CAJz84gfuifKINu1p/jYqX9+
LrtRPtLvIYZ8bPZsDiV1jHGXmR5eCgavKS3u+B7Y1KkkPk7PBfPSGD1OWDY+
516bW2JAcnP7n/0Ks4nO9eJ2zKrlwfV1i+F7jw4wVj058+o5F2rf/tboVLWh
pVqzwjWvHUF1ylHhJ8rBzdEZNS+gCfcHfPW9Nrwd2iFn635otYB9Ja9XcrAW
ro9Gar5f0wam7Cv9QRPIPrN4Y+THOWUYGztwrT7J5fkLs65uDa8lvK3xjH6X
Ay9/x2nrH7RALSpy0ZiYDrDOWejuGR+OiTOqKtN5zdg2cNvrQ1uKkD5u995u
mzZo1E57VBgnhlVZ4vTTVxsRNc7YP8KjEcZ6q+fYRzdBcvvT5hLDNzB8fq5k
oAPJMSEnmU3+jRDNu19+alIF2YcWjNtyrRCKza9nGV5pQMjtm8Gphc1gu/yY
ZXY2Hmd2/3ikvqoeojCO8VZXkqvY1gfc5rdDIy0yUaWzFvIzrofbBpB8r+Bx
I8mqEfEKabumOdVA6J9pfN2D8OW9h8XMftXQTbm8UW6gCMMcdFfOGZ8HybIV
EeIfArj+7XfCtbACq4vGv3jAIb6c/ft0SFQAkkMfjmg6WAp2nkl5+AZPKL2R
c7ug1AJh1GX9TW+LoM1cr1AjUwrOGLG+7LNa2Mj9beoOzsOpH1uuORqQ/f6g
7oJBMUVgDA5MnW6cgbj2kynH9BJgrlucdFHtHeTTfEsqxvOxaUdGtI1DFUR3
ar9ezSV78cTvP5v3cjEubi/Tw6AFPh0tCw3ni9C+6ddytYPtkJ0r2FxC8gVn
xUFBsHUqbqdzHaZXiTFMLtjeW64atn7DrGR/NmNxa3dL74A6wrOqMR6RzXA1
NVVgvC5D0pUbC95vLgZ752y9ZT8/gPGFH2ubbwMbxZYRfp9sIDJOdNfMLkfa
uuFLQ0/WIaVzx8YrQ0XQ/bOg9a2mGKtki5c92kN4unXJusdyZP9TN3922jwb
w3a+2bAnKhay19WmyFWKoeu67vGUjFRY0OfW9Lnr3nnj7Ta34CJ9j/w26Xv8
vx0eVxcoxjf6XXmJ9LvmQ7W2aywlcz0k8f/PaTxbek7u9qfah8fXQkLvJWcv
vZfuPeX3QhbhGK2DHq0DM/pmUOyMT2CtlNbtKa2b16WX/IWXRIijdf5J69zy
9facjM4WKHKkfWFOkvZFRt7FPjYxFYy10j6q0T46Fik/T4sXgUf7rkT7fqL/
tpsK49oQSnXCpjrJmpb8qz0xBzpUV2Kqq58ny8/duUJyUI5Uh1lUh4GzZpwK
Od4JjeVS3fpQ3fJ3ZL0VVBWDR3W+iepcJaFSdOR0E7rpXCTTuRCVPVvacDoW
PDpHi+kcMSN/+ii/qwL/lnTuLtK5iyssD241JXv1JOmcKtM5FebV7G4ITYc5
nesKOtcxYi0FvS6SnykHTCkHQnLWjW141w4B5cYCyg3njZejxvcrAJ9yZjLl
jP2fnUFZxmRfSJRy6T7l0perXv6hZv6QXyLlmBLl2Hvvw4y9vmXwptzjUO5N
tS88vvJCM/EzKSe9KSf5u2bZMuekQZtytYNydaf75+mB+oTj+lIOu1MOq0+M
VEr51gFzyu2vlNvGM2R0yoNLIPgf56OknDdMixxZOlwMpT6+4PzVOGL3JrIn
CKQ+wvCT+oh3qZ1jZTrpQx/fYYV0z2zi3QKb+pSI+pTzMOWCbZYV//E1By/2
q31j2/7jg/LZkqRZyiTvaEp9k0t984nNaMdBQ8k5sv/ts4yWp5JDo9PBeP5v
X2acvjais7MOhtTHef/zcaOZr800QiFPfZ9PfV9txH7D2e0kb9CcYEdzgo9N
sdve2W2Q0FxhRXOF9e5k67VMogeaQ/g0h3QfSTp6Np3kjanS3KJNc4vx4dkO
/TzjwemTc0ZM6Drz+SXZ32kuktBc9E1jwJ/FRiQHbpDmqDM0RwWOKZ97+W4B
DJWXh2RO64DzR+2HPVFVmBe1XT1BLILhVu+Ryg9qIbp5rNsoqQKc+wuPTb1M
cu/K+er3lmZDIN7zNSKtDO4Dx/kWOBWCr/rE33pjG8w55/NjPwmhF3Pn480V
TeCfP53q71EDyZUJP1d9KkZQnujP54BEsNPf8yLvZcEu7rHc0bRCxESotk5J
rYHhlYsnFw4jvq5ivDM8JR/pOSe907Z2wrxyzyKrTXVIr58051eKAMbnHkre
jS4Dw3rlDaeyAjCmlV4cfS8bigdVn5bJk72GU2l2L/UJRLcsPjS8/ATuhKSV
kaoCCIT3dw05IQYndaZ2XlEKFtSM+bZVsxESu6qr5WR/46tv2fl5XBIcG3ee
GXY5EYyImIFl5/PAnrV2XHNbHHJmqRi2cIi+s5l7NswNBtO1pEnU9A535IZq
pbIekHOO/a5cSvJI0DTDqjWvcDAqUC/7USWc90zyOn+D+O2M2yrB8/0QeFRR
Y1VCGwxnz/WfPq4SjMxJSrGjHOC1Z4bvo6FF0Ng6aaxiTB78u4eGZTp0oiwh
6X7SeOKzY14prjCKhd3BxR9qyd5lHr6kuag0BeKB05weqnRg7IKv+W8YHShs
HW02j09y3bmkW9t/iyFzY1mBLov46cvtSz4Gk31rk73Gy7o6zIwV3fe40ob6
BZ/3+UcWQ6BudkjvcSVmuuh3OUe2Yqdf/qTyErIXjDsvijYtwLmX82pGN5M9
8fmEXhl3Pszn3k00aUtE/8w1coqjW/Br4vcc0zUiOB+8NHmxeif8n3gesVjQ
DIMprUf3rmiDc+e0tF1viV+nWs3N1RVDVmt9/QkNATjHnw44N7Ieh1TlDqro
k33W8vg+o/I6iKLy5my3rcCb8/1ezl3WiPlrk9ix7p3gbVwkHG4iwLCzp9NT
lRowTD7DMP1sMRh3VRwW+L3CzLoPws81dfDe/vqJ6T9/P1ch6qcO4WaLetjY
nru14HprbTX5GQHv7I1mIRn1YImEAe5za8DYNE3om0lyWvWykSN+lEPFeu+S
53uqUXHtqvnOQySnjZTMvbw8C6a8ZqUzahVgq5UsNtuQAob8ipEFK01ht/vX
pgl8Mr9WDT8j19cgyGbm37W9xO+XjHr/dnIRzjVW7bde0Y5NNwRvfctJbtP1
zMxblodUcard0IX5YP5KyVo6l+zNiYcnvRuQgRdC7/G81HrCQ3FS1KMoOGsn
aLwI4oPnm335lcJ5MGeNyMuc3QrWfeatp/JBsFBabHb+QjGG7W+KmxtRixcZ
p71cLTrwpV/33CX1ZC/dNv3lcUYBsg4X3Z3hQ+bESV4uMvso8n2s9ow62onC
ae3bvtQ2wNRPQ3L9bQG+N/W49+ytR/wY5fEFN6oRM+f06FcT61A4Z8FNXq0A
SwwWRYR1Z2Nyp4PD7f8j69zDYtzbtz8IRRRCUhoVQohCtue0QYiSSpJKkpAU
IdmNXUJIkiQaSZJd2ihJjUqlnWm/r6lmpqmmaYSE5L298/398Vh/Pce6j3Xc
m+t7XZ/PeT3WmsXrhOu5+Md7R4uQ/njt8KD8Tlhccew2Un2PyNjjLmw9Lg4a
DQmd51UON5vxzTVWNRhecWecysZ8cCeFjlVPq4XP2/Pho2gCWM+8Mne7SiT6
h30z2yjPR7av1inX3gZ0k+uKstLrrlld7EKZZpwg9+lXkd6Hcak3h/enDHnk
ubLG0udyrQ4r3LRuxWXynm3kPV3UtE26UzpBnyP9rjjyXbYmX7+V7GeDLpTW
4TOpAyt6Us3DnBoMkLp1krq5vWwbvnK6AIGkzo2kzlHdvWsX8LuhQ84lnpzL
3n6FqBuLy7GcnCOXnGPvF56FSRYP9CfSc88j517rMuxhbUk3LEifyGZK+0TW
vLgw4iDlE9JXItJXH66EG/sKBOCQPkwhfSgplP0crvQaNF1p3zJJ33rPXSgf
5N4ALulzN9LnsovWHXR73gE6VzoXumQu6D8GnYp2zUcMmSMjMkf0WvG4+WHN
8CFzJyZzp7u/JqncqQvMYOmc6pA5Vdz6IHP1vFJIyFyPJXOt3BdWNfkDxc03
Ug68Jhxgp3t8Ut8UABbhhhvhxlyDaYZb1laDRTizlHBGy7Xual4blYcIl6IJ
l/KGPY5RGp8ONuHYMMIx55xxzquPNcGJcM+XcM924jH2qXrKr4ST2oSToe/s
stdacMAmXNUmXE0RDOM8j6LOkXBYlnCYufnD5cKdjyAh3FYm3GZpVxs/PF0P
2gQp530J56vHxV+4fKkDTsQLMcQLtoO+RX0L/QTm/3mkUOoRF8v9rxb48sH6
xzusZ/ane4emwvMfT8lo6djfWUDxr/h/vVby2LH4x6pu0IkHWcSD4c+SHWc3
Uv49JvUmh3jTd3VHSz+rE4o1Us+yiGfpK8vH29+jPB0i9bKEeNmj5z5XS9QG
BvG4IvF4nLvzuVtpudR7/q/3JSYJw6f1UnsFyQkMkhMYs4b1yOfeAuOjNFcw
Sa7oD4mbyT5E8ZzkkD6SQzgFFxnnQjpB05bmFk+SW7hXuD6S85TvSc6pJjnH
9qT3/JA0PlX//81FFmY7o79OSUQgyVGBJEf9vG779MzcRtB05NyOzONDt/uF
fax+AwIxbXT2uCrQ/E0/Vv8qBacmKXDM7DpozJp07dIyITyNFjyfokh5XfBw
WJJqDd5GX1d67Z+NuDMXzB8JWxDn0Z6iPaIK2Ue71GRXtIC9Va60eTwHKe0W
zxhfyxG2yGvyhSvd4FzINb11vAuekR3GM+tLEbju0b4/f/+89Onhz6cZf3/X
IWvV6SwO1jVmuTmwqbkuLXWSzSyFZ3q+XLZtEZy+OF6atKYE3KZZwx9Q3Irj
lMSLF3zE12IV5R011Pe8OrPStYrKp/vUjr6Y/AGmG9r2xS9jU7lX5nzNyXqw
7JffejbpPbSShJPozi1g/Ln3of4Kla+z5oa8dk2F043Lf1y/hkI2vKbulx/1
vWu+jZ1TH4/Q7PleQTsbKV4Z9P/cQ/WFl9Yz1YWPYSfXfu/6jS6EjoPRnw9c
0Nb4vJp1LQjLqu/I9b2sAnf7mjU8egXqI5ba70nvhvcY2ZdnvreDJRjOuyjK
RJSx1VdViRiM4qvyNsGfoGhxScBa2I3AvljD7hlibHz4IuBdKB/0hc0hd8Xt
2Gh/6HPnzi6weAFpne4pMNVygKCcB12vGLPOByLI05v50YZU/j46KHdzXBNG
B32c/qypE96DVrk93CYCnfXZ5lhpBaIaZn5oUen8+8/F+ptNLQXtnMyLfmEu
BuIq2Ss3dcAqRnD93WVq76O3+C4ti6D2oi2te5ntYIy3ydvc/gwp6iueSzo6
MSyxqVkuWgib3PG6t7PqqLqMO6aWRNU/13hffEYb3lhytKrzqb2PF968zKoZ
+aMXpYjyBcgJGtw0zCQH3OPqiyLPVOFrcF673gc+1ox6M3nb8mYwORvNt5fn
QlKcKzPwlAfL83vLUiuofaotPeZQF1VHtfRRxSda4eQ+9NXSIqrPXi46PvFJ
O1bv9RprtbAFWn8CblT3UfMpfqWRPqsVcz3szO+VcaEip/rn161ueAr4xs96
qlGePdaH9awRtnOWeO0IqAS3i3srnpmDRA3eisxjdRAxl0f80BcgbVrDzc/P
xSi8kiqjpVANxWcbPSyj46h5qzaof0Xt9ztC/Dv3U3ud7Yci7+H1MM2N2PL+
NRd9jc4ODrc46Nmld0ZF0g6L+Q+ZB3zLoLwj6U7o7zzo6H2s4vzJAet98IQf
b5IQc2ju10Pz2VidcYCVSdWrnxl/+sPFTirfbW1TV4qF3nWBeVCgCC6ZiVYH
hvGQdDwr34UnBv9tq6+FWxEsArbJ1Y6sxsGz1eyrcZ2I8ErJqbhH8XNQzaDR
1amoj77s3sJpwzvbimsXzToRMOtErymvHWl3R4641EfN8e3WB17b3+OE+++6
kbObEDCef+iqSw3qF3+eobqF2qvnafNj52XCTvP6JKNB76ncZqRlNIcHdc33
UzzmU17zWijJsBXifsGZrjFxHUg4pamsWF0P+tkbnPP9HyEzm66e1tyNLvmW
qTZFOVDe6P7+ZQg1z8M0VczOvYZC18l98VW3QWu6d3NLSjHl35nV9Tc7ISeW
XmfwpNd1w+wiAuLeY9Mo6X10zaX3sY94eyJkXhe+keeanJM+lza9oolr7AFj
8p5u5D3zSvrWpuADWOS7BOS7ArcdTz7+iOIZqcM7Ugctl+EiL6tWxJK6FZK6
LVe5ePthDJWTSJ3bSZ2j7sVmNxdlwp+cyylyLopq1w9aZdcjgJxjNjnHyWPW
NHc+EkL5jPTc6YrSc0++WMDcf/wtuKRP4kif8M/QM3xtGsAkfaVL+so+YVT+
SAblZdKHMk3SPsxzW/DrFvLwlfStjLO0bw2s1O6s6m9GOenzatLncbMWxS/s
7IKTWDoX2WQuQgf83suHlCGQzFE1mSPvPTvXfHrDg5DM3SIyd8zs3wa0M5eR
R+Z0HZlT+cGC+o2jqXkic32EzPXA2pOPne9QezrhwFfCgbzGpy4zLd9AQrjR
S7gRz7+at/ZiA+VjKWeKCWeObIPpgAeVjwiXZAiX0pLS3SPK88AhHBtBOHZG
9dtu/SnUHq0h5V444V7pfd+svfPFoBNO0l5JORn3cgH3fEc5GISr0YSr6a9/
vd3aLYAn4bAS4TCtt+odT+MdnAi39Qm3GewV+vEnmmBAOG9JOB+608PfuUcE
beKFIOIF9ptIwfliijdtUo/EEI+wAupPdKgIoegg9Q6XeEfnbZPxj6l5YP+f
p0ylnpqupPdriDYP1f94Tfd1suK5QwnQJh5kEg/Wqe3OXVTM/Y83A58OybM3
6AaLeJZNPMsImVuYsawOTsTLisTLYp1hu8Z9purBlXqcTTzOcs7/tGJDOZz+
8b73MItlh/7+vso/OcHzosTebyCb2heluYJJckWP3Zx1tC8toNtJc0geySG0
3fGab6ZcBZfklhSSW4SK07K7L9SB+U/O2Tmh3re0gcoh/+Si/i13c2vGfAKD
5CgDkqO8nLr61I7w4PTArM91QzVkj4ckzXhI7SWl4ju+u6h6GezI0o09B92y
pu17LtRgZUNs8uP2FgR24E5XKg8+l87K/dxRhYBO7UBBN5Vvr/MOvaorhfZl
vZ+2Sypg5z04bsuWWtBH9fS/2ysG7fuj2Rvky4C4W7Kt1D7n5M97ZPm6FYEP
+buH1nNA571fqHyRAyfnxtpBL6g9xnDkU2+rIjgMmlGxVl2AuMej/BWsqfo3
Tf59uycPilGdRkPesmFhbR10NJXKlfLT4iJCs1HPq3W3vEV5+VWw1mmtGkjC
1mfQNrJBo+W0Lnb8uZIm2zh32NIM0DKvjRlYmALhsr6A1EkN8C+lq+/O7ATD
59mJ2wpxCIpL4sfP7YKi6Z9JGVTfMe+MKHbYz4LbihjZic+qoLg58nWaHpUH
ZS/egr8rVGwEzfu9OsDyM+8sWZ4PM8drFjbzu3FjsKlxamMJaJwxYbX063hx
UXV0rI0Yc/Un6c9ltIEVFD91XUkHZIwdgzUvdMFfYQ0/aPsHeNK07MKo/YPe
HxFZkCqiuEZ/bqnTCmFGTXmRCxdG3m9GmX2j+vJDbGEd4zYUvQv1rv6ohMnr
aSVTFlAeVlqdffN3HbjnV4ZFZOajdsq9Qc8pvgT3Dor9NvXv71xZpY8d/BLM
pqDhDjHtML2I6OG7CsAY7HbukE4XBuzyFe7WC9EnJ3FwNG8Fd7n3RRvKQ6dm
BTIKZIUo/cQYE3SpG7JFf05X7W6Fm1LqebNZbbjM+PhUXrUKgbMVnS+x6rEo
paVs30oBnOtPnkzf2QbJJNupzoqlYH6oc3M04mPJpJIFUcEpYG5vfFi5PA2y
vreDVunx4Fi82Pzq6gZ4Kk7uRV4X6LYjP84e3YrwxUF7z6lT/QNus05OG/Ze
DrfR5DQjRtS6P3BNDkLdUzqvMZvBXeX0au8RLqIT29+mDG0Ga7rN2IBvFVAa
HZS063cDGmMv7j/iJwJz97vxM3vZEM19NHPop1rERMfwZVdwQBt5YHbsli7U
y1zfYkb1ufyeAZXIXy1QKr+303u7ANrv3a62vi5DzBaxbGIJ5aGhbi+0ChoR
ePTu0KAnnxBaHtHod6ICBoY2cpuGliDQueHQrVO5UDrmky96SuXOrX6bjV89
BXunS9CzknfQTdAcfqkoAaJbC8/dHdwJ9qCj21RnPsblRxZaM2j10Fexth++
lId5777wZPrEmLIz+XKAcjskm41n2++tRcJwB4/F80QI17idPkrlHRSvuzs0
KubCJ01j028JxdkDPRVminVYXZTelZjWhU+rZtc5DOfD8+aD0vsJf32jIlIc
zMeefdV9N6ZzMeP9Y+EaxW7wPE5MnESvgk7Bkne8d2WgLxm3av/Pj+DqGC72
SYsBa5Z4nqF2DCz88rIWBNaAfiN0eZ9DG07eif2U9pkPD8NV5VfONWNZ0bFi
vVd1yOYe0J73IRsvBjerXM5oxaNr+08o1+dBd61w9ZIpVQgtWJzfe4sLVuNi
1eR9LyjfzBzn9iAHn+hRAwuXdkK5iVzPlF6P8j0wVHtiFgZfl96Hvk56nyi3
wH1jyynPk+cWNkufG/BzRX4nsxrG5D2fk/d0P3l4i7CvnsqN0u96rCP9rr60
yfcFi1uhc0Bahx2kDjnz375wOdMObVK3U6Ru9IbhnYO/34MDqXMdqbNHTZpd
ZijlCXIu58i5ROQNu1VztAXKVtJzfEvOsXzyHNN0exEsyLkvIefOO6RRUXSv
ADIh0j5xGiztkx3Pq/aL/lD5eZu0r+gu0r5iX5rctNGPykWkD2NIH6qOVtg3
PaYUysOkfZtC+rZ8fFxgmiEfy0mfM0ifsy+qeGXwIkEncyEic6E7d0j+W9la
cMkc9ZE5yns2++rxZioXkblTJXOn+zTzplnhW8SROeWROZWE6Phf+9oAHzLX
nmSuv8pc5E2muMUkHNAgHDDRSLbuOZMLhoOUG/KEG73KLrZv7JuhqCLlzFnC
GVrDo2P7ukRgEy4tJVzyWyMz9JxBCeiEY+6EY1zHybE/vHjoI9w7S7jHHF/4
8vjdq7AgnKRtk3LSVqRuZLiD2jsKpVw9T7hqN8VzQvr0dtAuSDlcTzjMnt63
ZaYP9f6E26aE25Jv9WqLBlG5ji3lvAnhvExs2NS//z+SD/GCBvFCsGb6I7vH
NUghHpElHrFY/mL13ZgO0Kql3kkk3qGHFt1+eJr63otST1kQT/0Y2NYYqdj2
H6/Jxbj6mC6l9p9/PLi3+enxrzN5CP3Hm54nLEvdLr8GTU7qWTrxrEXLuTca
h5rBJl6m3ZV6OdEg0LDgD7XnEo97Eo9zFq2bvq2iHhzifQ7xvtLEVfNQ8vf3
8aU5gWYkzQmev19Zu7+ncj7JFaEkV+iMVucfSmuDBckhsiSH9OFJw6GAD2CS
3GJAcsuInIEZVuXUfvlPzpl9W6nqwCQn0Bj/m4tiVsUGxunUgk1ylCLJUTHr
z299IGgH896MoDzDFFS3Wj+JGlILZu6Z0Rcag+B5Zl8RXa4NtpNW6x/hVKHw
3YumLfJUritaHuF9rxKKBUcPngmpQOGIqiLtZ5TvjPPws7YbTuM0W6fZlMFz
sc3XN2M40D2yK7w3jjqXtgVHxihS8x0/1yw/nMoNao/dy2PL4JR5XL38chE4
Ba0Xyla9haLtWdW55t1gSKqtfnTnwXVB3TLD/VRectWVuzSDD8m1nNYKq2wo
1Uz7NfCEynUKzaWryyvBlNPwn3YmA4pem5rXLKE4ecfppqLmKzDy9jfv8HkN
lm1S52ZBJzgTs8Z93Ul99/bBv4aYPkW20rkX03ZUwLYj90mIPvWcofEqE1yC
wZ1gvoV3sx1sV7FN19IyCJZvmL7gXjcEPQnK+/o5VB2mjyv9noicxIPn50eJ
YSHYsfuXXRvoWy4WFMwXQUaQbHAxpQu9274VOdnkUTnG1/3SGgHCDY4MPVwj
QmHaOsXiuzx47vD4mFXIxcar5wpWDhVB0jj0TuOv17AQ7ad9s6mCQYO9xpEl
ndQc5L9cU0Xtf6c7T85bWICYC0p+aw91IEVdZ+rWU1R+jfgds7ctDqqxGQfe
v2yHZ6z+l3GPqZwTpB4uoPi63LwdvE4hzqPi6M+BNjBPJV1//0OIBase6GRr
CeHfsaGYXZ8JXfdoVucuHvo7zQxPbWlDjKXF7zszqHyy4Snti28TLPxF9Pln
BBhs/kVR06YLLMXj96rbKrB0JHOn5D4f+u6LhRNuUn1vPLByVh/FD1tRdvJz
HizO39vlKEdxTiFpxthlYRDNWxLhFNuK8W9NZvWwb4IlVutOde3E19iZ/uE3
WlC0uUC9dGo1+vodRJwkPkzWDGP+cGxGC3+O4RV3ar/yYz5ZcLgJGj2Ko76P
4YKbcvR1OD8J9E3Fmt1tpcgLHXvigGkDJHM5++JVqT3Kr/nOgjnJoB/6kZs2
oxaScL15Bgfa0Wfl3PJ7EnVebcumr02vBI3ZviIkPgtcczvaNU0+XNY+0lpD
zSPbXm3qxgPU92uZ2LGW1kNZ51vjgrnFsDiU4lvs14lA2rizefcKQQ/qOx1Q
+gEMD+2tP5uKqP02R68x+yJ0r1olJCxLhanFcma7Vyu8bbSTbUvbwdUftl7L
g0Xtle+u5q3rhkHsXd3GI60IjvVs8H8uxotq65JJERXw77fdybGsxQ+6bpv1
LBG6Dr64q/6J2is8Px+Ta8yHAd9ZVObYjoTqQ5abnLzQF36sbFBXN2b9Sjyd
qSXAOLsKj/btZYjMV5A5HyHEno2KDkONW3Dis1UMb3sLhk2NbBiVzMW9rNW3
9vBrsfflRQ3DCR2Q7PYbxN/PwT3PbQwrUN+zdvptjW+3oDZlw1Sl9d0YvYs3
bNRaau8ZKJgWeS4XX8N5Cs7bBEhUjji93Y8HjuoG/YCFpVgxycBCgnqkfH65
2cUyFxZLF0d/W1uKJRPGtKefLcaIoy6pimot0HVOGPcyJxknyi+2ZX7OgD0S
0ntdu9DvkPform0Tosl1f3JdkzN5TvxmLk6T+1iR+wTcvKQ2a3gXPMhzg3uk
z81z5ukMrRNA6Z70PVvIewZ51fCX8AVYRr5Lh3yXHe3qkYlUnqe7SevQTupQ
WrS608aiGxqkbpmkboFGzYd/RecijdT5KKmzlZrNy0UmdVC9Jz2X5eRcZDff
Mh25SQAWOUeQc/RW3rYdDd2II+c+aKr03F1VHuaxd5TDlPRJGOmT4nHWEw/p
8hFI+oq5UNpXzAhTU3F0CGjJ0j70JH3Idrz4R4fayyWkbxmkbwN5SXsG+QuR
QvrclvS5p96U8Tm89+gnc+H9f3Oxyavb1pQLeWvpHMmTOVJlXfC0G0ftCRel
c6dL5o52db6xujHldTKn5WROWcKJ+dGfm+FE5no6mevRcvP3mBlTOeb3/3Ig
QD/qZPv0EsQRbvQRbpjYqEgmdLeCNkbKGQnhjEXtc4++HX///R4pl4wIl958
z9Y37KoAh3BsJeGY4jreFe/X1F5PuGdJuEe/tMunbWMSLAgnB4uknOyzcxxa
GNeAQMLVRYSr1y64F34y7aTyj5TDRoTDPmesFUcOpjxCuE0n3P4au2vYER3e
fzhPG76ya79KDJyIF5YSL+QZ7mN5bmr8j0fMTJp0ztMpPhPvsIh3RH57m3bm
VUKXeGoY8ZTJk7OFuzuoHFgs9Voe8Vqk3vL5cutKwCUeFBIPJg6LPpR/pg0+
xJu0YVJvyj7KivSI+whZZalnaQ5Sz3IPp2vvN6f4Hyb1shPxMvP+rE1xrmww
icdZxONxa7Yo2Lf+1/vM95cWGTIvUF6R5gQnkhNc7q71PzmtAUySKzxJrsjx
M76+6rYITiSH+JMconM3cve+nHIEktziSXLLGV75UM3v1F5Aco4yyTlmFQ/z
7M2pPEZykRPJRcseaCuWH6PycIQ0R3FJjvr++UrVyD3d0N0rw80qo/bg+xn7
GNbViCti7vL8uxeNz+xxt6uDRcLvnpUjK5EJ68CYqdReP7Hnul/KM1Tv+Kyn
/6YMwfIVrCAbivPXs1s1dQXgxO2O2+RYgueXguavZlH1E6+p3Pe2Amz56m+L
+4sQK1zM33WZCzYj/m5ldTe4j3vZK+d8RNSo2j+7josRGF6XEa/NB+fmfGdH
62zUC6zSbm+qgVNnYPL0lxVg3BCfj/uaDn3vjZV+1PzSvu54cNvpPlhdZsE1
pkmQXJH8+eZdBgvNXmPjK5SHt8n0fj/2GInmA1suZLRBkn2+T2tnA2ixTDv+
lnOga41c8mFeAZweXHC8WvER9fPVOew53eiY8SEjsZfK0+ZLX2RxxXDSLbbN
nCxGqCeDrvM0Gylew34u/9aGZ0lTXjlN6kKQS6dXrjmP2gc91h6JbYbyruGl
02eIkHFoauwbt1TQt3N5hxqrMOK0xGKSSScezg0TWI1qAbOzZ9vaV/nomeFu
9nN/Bxgu8+lXFQPBPLDx7uzTzzCiQGFM2gNqJkd17D2ypAGcz6b957S7cPau
IEJSJ0R+s90X5nYRTJVmzrZ+LYR47rqxypOEcPJd9VPjQhk4Jxm1q+15mLKm
wTTPug1uW4yUlb9Rfd7tOalgOBdmIR9vvb4mgNPqLb/yXJLAmLdedfqmKqQv
W2j5ks0Ha61227tZjWDKtp+4zy7E69WW5eeFPORHtgnkFlF7xJYYybGARAgG
bQv7NZgH1g+xK3sLlTOeqzuOed8FK5fQhtfyrTBdu1pd4x3VhzOfNdMThRgm
mfN8xuAW2H+65x+gL4byO33FoztaQbdu1DvTxIXCl7JKe4rf3JVXchvO1uHE
lOCJHoZNWOgymb/9HQ9OX/cWCJd+QoxBwHe1c/XY/toocnRbN5g9Rqttj8WA
fcj7/vPjNeCN7x47oFkJ/d64HZHdHXgmPnfnFfXXtdd9J1/fKED1LLr+wbE8
yEQrSJyOlIL9vf/l8fpYxCw+8fC5fC3cDu/uXi8sBDdfXf36yVp4btyysSU5
FwZmPLWV67Oh76JwarauEO6PqzXufKNy9uo3Mxy9qf1tq878rylsuN1rnXvB
VAiao8I5xpZriPk8z8DKiPJR6JfN5/80472VXJWtqhhG1t46+xeLwJl9d1Sc
ZjW+RsSG3UjqRPAnZlHa7E9gts5aoWf1EXHn7oyaodcOTugFjbXpVD2XFY2f
EXgDUUZNXYu8BXiuGqYU0vr3v3PkmKB5sgMT53JpoWat+BL+2vnW0TyI17cy
L/1qxbYrcr8VRzUhYGr9b8u4epjwDe/ebquBicP4iU7sChScujy3zIcPba3H
QUYZHzDG5Uc5d3w2Qh1eRo606sSI0nfbV7K6ENjiGXPevwvJ7EUPS0K7cWpF
0sQmIR8L5smKCrmtYHFXoKw5Fotl/wTvrWhA9XP+uW8bikE7M0PRze4yPpuH
u7v2lSLkUFXBkQntOHdwkUg/sB0jPk6rzen6iNCzhgvHcmowv3RPbMzDcqiR
63nketSEgtjZL/iQs5DeJ53ch7FwurlOxkfsIM8NeCF9bs+ODMeQkG6Ekve0
Je8pnipf2LJJjGnku56T76JVmy+4XXAJ/qQOO0kdJBeMDpe8zIQnqZsLqdv9
wZ2RW0dXgGYmrbMXqfO27jFnKh83Yzk5l+nkXCptLR+rtFB5aIX0HF+Qc6ya
PG+beWsKAsm5x5Nzzy7+mrr/XC36SJ/0kz4xKowoVFwoBJ301UfSV/Qt28ND
TNLh+X996CTtQ1011vUtx5sQSvqWQ/pW2J74JbOkE3RzaZ97kj6PO5A53BKf
IGsgnQsWmYtEFU9Pg/ctEJE5kidzxNKlrTKwFMOKzB2bzF0cd8THZucy0L5J
57SczKn2Yf9empgHLpnrFDLXtLS177Z/dgMNUg4wCQfEiz8uNLlQCW3CDS3C
jahhIcElxwRQJJyRJZz5Lq95ZHHyM+gSLtkSLn3Yf+j92wu1YBOOCQnH0gxN
BAodQrD+4Z7F4R0u1TOy4EQ4mUk4aVW3T/5BBxey/3DVYeItruvmLggJhzUI
h9mbjn/6NL4cToTbPYTb/UYNZz1WUFz6h/PsHu1dfePe/ccLi35czjfZ2Qw2
8chX4pHRhwdCjp+mcgPxjiLxjtHDggU2/DrQiadUiKcyS97mC+NFYBOvvSBe
01k8U9XLqQo+xIPOxIM/W7u/61dQ50q8ySXedI87zAwZWw7dD8SzT6WeXbr4
3avR+u0IJF5mEy8r7xhi8Y1J5YqfUo9zicdNUofTLqwSUH78X++HzsuJ5r3K
AovkBC7JCS8CKqbqb2mFhOQKCckVcTMdB7PWxlC+k+YQLskhKvMCF+p3NiKQ
5BYuyS3DXhYt+T6bmnsVac4Rkpyj2madXmtZAeY/uUjPqSJ5dncbPEmO6iM5
ipbfv2dlbxYk51JDJlziIrDTbPCw25XgHnW0SdxG+bf8svXhTjac6leuT9Kn
9qWlBbGB6xPB7RbH+s4XIq6hOpuXVwL3tuBdjxqawOqrVV9WVw0nTa2ckY+L
cYQWtf9RAPWe7NFxO2fdh+Lzm4vSnnzEhgeTV7/5SOXUMLqomupLZlHMgcC7
2XDXF8Zv8qX2+T/h3UNyKT4lpmRq0DLgtHTyva+MT6Cl3JgbqXEJzAcTktNT
E+GWffuj8R4qF2D2+sDNAtCiwoxffokC51uOsHfiB3A0pu1z9q8BTTk9qrfT
Gp496bwzl1rBvNkSnrEjHYVPwt7dLxaD2dbSFNz+GvRw+Qe/T3ci50Z4/qP7
XWjxljlna9NC5RSnoLNdrQi5zpIbfZrKwXPvcyuyn8NJdolS+bxaxLpPO3Lh
YCckO7dKojdRfd7FkAgrCiBvvOku+1gH1q1Z8spz4k0wf7Bfvxn7GHY1m7e7
3GzH+H337p7UbYLBwOYrNR7UfQcJ9vmkCVFxot9g+koxhC1ai39/aoNu6Yxt
Ez+3wXuX2gtnWg0MjLpicbQVJ749FY6cQ+Ulxa/bkuw6EDfvmO2L4EaELzfZ
pucuQGD8sbM/KE55fpCPHLqrEg4WYzwfxfIhMv99uXoYD+wOvcQez0KMbjsQ
k8jjgSFKUmk4ehs0g57VA4nJiFqpObd2Ag8n8njm+xbWwmAvf2dBvBgO/Gsj
w1a0YmVdjpfabGo/ub0sw8W4A/7FB2e7WlOcjU1dcrYsCxxRy0+jcD7G/slI
PmtH7SGK5c/8mVy4vfrotTaeC/d3ejJd1N6b666y3LuK2nMNU5W6eyuRt6M5
+apBIx59118Sokjt1XUXgn6k5eLEwbGVtyLqYL9iz40nY6lcwZzwVq7HDG7X
ezRi06qhNYPrOGZMF/qv9nQGs6k8yri2YeaFCpjWM59VqXNQrm99SGdKK6y2
DfXR6C5BHtvsuZ9GK5RXblk2zbYaQn09nmxbARTDbyq7O4jBCt16LF0hG7T1
u2qHamWBdtZl7MAran87u/Z3M4fKFcOGa3nbJYDT+C2jL5eHxHe67E3FAqxt
6v+m1v33v4929bL/lbsI/063FC/l4mna+6L8PV3gZI55/tigFtrTq+7zTpTj
spa5w1VxB1aPduhRNWmH0+Hdxp0mWbAZc7nwA1OI0F0Xj8/uyoJ77dPRjmnd
mPtMPtxJXoDBw2+35LU0Ye42D1frhA7EJ8zO093bitdiu96Dn9tx9sCLl2JP
PlKLrNqX7OHiUbeOB2NwMsrv8fNPXG9Cf41T/LfHNdBVtltxclsF6g20eqaZ
l8HlNtdPgcvB89CT6S4mzeA8qtP8PpCCaNa6qRdqknHv6BvtNF8B9Hq/2Mu5
iLB10auGR7NF0DCffPbK4A68pE2bGF/Ch+qOCYHN6a1Y82Id7aV5J14c3796
RFsTTJ8XyB/3KkNeSsKK7oPtkFm+f6FVZyXs3mldCdHsxpAF2cPNFFoh3HQx
N3lQKS7PHhLhtq8Nj3dvmdg2swhicj2YXOdPm7XUL0kIbXKfa+Q+9XePSnY3
14BDnhtAnstSnS9nrJ+PHPKeK8h7Vq+uGDB+nwkz8l2HyXcFZAV9L/mWi8Bo
aR2sH0jrUK949cXFEWWQWSKt2zlSt9pimXOLjjagj9RZqVZa5xFfLXcYWvIR
RM4li5zLHOelPcnqXdhIzjGbnOOhVcHT9ZfmwZec+0Jy7uEnEq/dHc2lPC/t
EyvSJxXTp0xNkumELemrYNJX/rkXFwV05yOO9GEC6cP0KUdnPmtt+U/f/gx5
+vYhVT+Zc9I+55I+rzZSYo72rATjjnQuDMhc6GYU9dJmCGBK5kh2oXSOnBg5
R1ZOioWEzJ0LmbvCU66PelJroHRNOqcSMqfDgjJmKknaQONK59qbzLUkd4B2
+GjyfzgQdGFOw7PaOnAJNziEGwaZ8V61v4RgEs54Es5c2RCiMC+SDS7h0gTC
JaOmLdHigUbIEo5dJhzztL7hpT6yE6aEe06Ee6E3eQaXNxeASTgZTTgpuDvK
dh2f4lKnlKtjCVfzLGZNNDTpRiDhsBPhcKDT0I6NPVXgEm5HEm6fv7kjwpDy
67+cV86YFzjrVg5orVIv6BMv+Gz4HFN8uvU/HglcaOAZrvYMtF9S7zgQ76Ss
m7/g/W4uGGKpp8YST41cdMTXZDfFLeK1F8Rruw02qMfbN8CTeDCUeNBIrfdB
vKYYBsSb+cSbJyZ8/7Fxcx0Yt6SeLSeetX04xKVIowsMTeJlbamXYyafXdQv
qkYg8TiDeLzcqHjV4NQOKj9Ivc8i3j9/lOPuM6cMdNq9/58T6CQnBM5/U505
j8qhJFcEklxhNs9R/82EfNBypDmETnJIyLwee827PNB+SHOLJ8ktYwaagsrD
EiD5J+fsD1i4Yc4Yqm7/5KKK0OiOq0e6oHtemqNYJEelKRpuvm1aAfbjrOMt
EYXw8Yzc0Z5bDjPDVNuDq5qovGi+eHJ8ByRhAvmE9aVgbNo4W9ItAvNSxJTn
gxqg+yvpxh3/Tyie3Wzo0F4O2pRxMtMiEsHk9pVGOucjr2nmsyULhGC8Gevq
IaLyD7PrsfW0D3C8emmd0/M8MI7J7FVWpfor6lvEuYcZmP5nCUPHhfpeYfim
Jf3XwUrsbFSupPKc3sfYW6OSYZC/VDi1mA/2GJGjfONDhP8MSmTHUN7WrN1V
plSFufun6nhxqf64MnOH8SUqJ4SOdqn98AymGUWOrn5i6F5UcKhQ4cJ22g3N
sy+FiJzwLEas34WoRvuXkmYxaDL0PvZGLpRmLRzX/rkTrJtnfFZQ+Tuu5sr8
BcdLEBe+9tjHzA509MQeXHKT+vvDaAtiR8ahkr2wYcnTdiQmcU3CPtQjdOxj
z9jRIvBear6e8kwIufF9pTL9XfBsoNcL8wXoX2zqfC6rDYlXWjftO1MLn5L6
Eua4FrgNXj/doUuA7xZf9B+oUnvEzWQzWf86WCr06kZqCcCzXnnPcGQFLAo0
fa4eprj+4th1VRc+YgVBM44fE8JJL1Oosi8XflHLnnRE86D4fWMmU5QLmn1B
5kSHOFRf3uoXJmxFdrW8NX8RxY+O3K+Hl4hhs/rxgUnTWkEbmZPwQyCGbOrD
P+GWHWDoL195zbYFbE4+78ChSgQ+crRXNRWgccPV7fNON0O4PTRvzeI2xPjS
d9vJt0De4DtL5e8/NxMtDqAZJoOetW3q42l10OUtrnfUofL34Ht9O2Pr4bR3
xdvciBJwpjxcWf+rHl+fiz/r+VP7u2qZn2HOO+hYBr2IulYLudQ8pxm6mdS8
Dc8UGIuhmjbokW9NFRQ6R9rcr21EnP2Ic+f2CGHyS8+47FM53C57dMsu60B5
Sfjw6dNawMh4njjJpQS2qjuNe9Syodv4fMoaSSWE844+cx9TAJnvs8bJlzZB
cnnTk7vJ76F7yWzGtl2Z0C3vd9n9swP6q746baV4YFE2qsX7xCs4nVmyONXl
I/LWH/U2+MiH15nggvV//z3GVRPnVa9qRt+bhuihOxqw2S7bqSdfhPPWPR61
vtTe2mAl2b+TA6WNye/05TpgsbxgYdbPMjCUT1f8tkyE8aM62Zn721D+Kv5L
x2EBlF/J2vyI74JKfusfy8F8sIZ+aQl1SECiul9DrGM7+M9PeAtetCAoapjS
885aFN7dktc0m4+o/SLtD+CiqXhpsci2DSaPZzo0ZnOxceCgxtXvtdD3CFZf
6jIFuquVBg3trYKJT8+stXblVL+OFu5mFYE+/9c2pmwBcuNWej/yy0OGXpat
5xCKk/Mnm13riETfRmevlTcjUZd2cnntbOq8Fz+q2Xq4E3Od2uaoqHSi/qyP
WFaT2uM3ybk8FlG5Oqf0paWgFffn2yVyvvIx8pIZvXtEM/x+VX6of16Fz0P/
6F793oqrbpkLPibUwttrk4GzKhu3Y7e/2tdaj/TGpYMvJVRCZmVIuUWFGGPa
Zh4O9/qALHJ9BLmu7JfUEvqwA5HkPtXkPvdnbZqx7lozJpHnppLn/vwy6D5f
sRq7yXvOJe+ZGL7+tsOEctDJd60m3+X2c7Sm+A21ty6Q1mGOhbQO3n4yF9fH
1MGN1G3wK2nd4Hq/bM2pFliQOtuTOp883j9UcFsIO3Iue8m5eA6Z1yj/pxu1
5Byfk3M87nx55/SNJcgm5y4m517zvP/wzMstmE76ZCrpk70RD1p1grvAJn21
jvTVs/Aw5VlKZegjfTiR9KGKs8qu+4MF0E2V9q0D6VuG9X5/nZmJ4JE+P076
XOhd8sDcuwFOZC7YZC7yIuYfLIluh8UV6RxZkDlivZR3d56QDTcyd7K60rlj
n9/b+2MCFzwyp6ZkTgcvmXHgXB+VL8lcW5C5plWtMqoflwt7wgFtwoHIET/L
44K4YP/DjQclRTN6D3UikHCmnnDmnekqA92mj2AQLukTLlWdejWkOLYZKYRj
ownH1s12FjdN7EIM4R6PcO+8yScPzxslUCScNPo/Tt4OG2Waw/v752X/w1Wm
ZUSWS+ADMAmHawmH+zYKrBcG1YH5D7ddjNw/Ncl0/IfzNNfEHeNUiiEhXrAl
XmBUOM139OXDn3jEnXgk79gu216VDPgQ7wwQ77guHb1lXUYLUoinOoin6rfC
rUY9Dox/vLb+0fWGBfeoPYt4MJ54kFlmUeFWGA3Zf7y5cVGG8Y6NlJeJZ2OI
Z2kqC0Z84z6gvCn1stn/ebmHa2yd3wQu8fgi4vH6hF5eknE3nIj3aWOl3qeN
rA059aIeNJE0J7BJTmDVRK/bMKULLJIr6I+kuWLu4IDTjxdXwYnkEEWSQ4Th
27se//3dHnVpbgkkuSX3jK35pMWfwPon5yzfq9BkcYUPi39yUdyRq49OMVPA
jpHmKH+SowKaIstWb+eCc6O5XS+O8sCwmKYsQSnYC68u03lZBsn4/Yqy45oh
e0rA6d3IgbJQbradSRuVu56GqwozQf/6yU3JisrxrzM7DFhZcEpZbpXwUAjO
Z2uZDeIPeLHH9b5rdgvoMhO6f0fUQHLozuR9+9j4alv/rOjBXdAaloz9MDEK
DPjK/ApJgsuRX8XzDjSCLZbEVw3wwZxsaOpjHAXl+vlh4+uout0f8VRlZiUs
XXKH7OV0QyIKDTO4T+Xg4MKlp0qCENlvXutoJ8a1l7UfSieJoDzUnveVJcCi
C5qFhQIR3Pa/vXzsWBUkjtfMhz+vReziPS6mJzqp97me2lHZAe6xmbsGuWeh
PqVuzksNas5Xu0ZeG1UJU821SxPWdlE5mbv6arMQStv73stYUHvNz8Ioq6cC
sLzw9Ne7NvRtNl/YPLIKefmXLHd+5kL8Regy5o0A9VfOt5ge7wQ9jBE842sV
GKfvMho+8VE9YNt4YeLffabH7fHuYiovavqm9fNgdqH4uojeATadF/hB+x3S
3w46MGIJlZdWT5rwx7kEeafDxoz61o30J7dXbz7VCmXnTWsevaPmbNOKmnF7
RYixrYqryqHm3Vd4Pn/2e3CObj78ukOIROW9LbdGUV6MxgWaP5fax4TrTZ35
cNZ8yJpEzYNJSJrBK90uuD0YdetIRjPc5Xq2rjjPhVl18ROV9lJwT9bN2SCo
o/b5tb71W5ogWqNrds6eD8bCH4NtmsrgsiVqn/hMA9i1m+z/6N6n8k35hhEP
cmCvuOP8g5N1EA6eMz75WDWYh80i1UY9gHBUrw2PUYNYj7Xn9v79PYZKm6dl
aiIIB93yMGZXYmWL8gb3n3FIifK7GJAtgNbjO4GXxGUI9/Cd0h5eA85s7pjH
v7g40bLOzCuNA935KWtv7GoDe032j/HHKqA7qPTlkvn54N6aunLrhZegvzh5
T02ZjeqkqY3x3PdgXZlbNpJ6fkqkQrJ5swjcFREjvT7GIdXjB3v4yTbYC2O9
L+/lo2fPmV7BXmrOA9tKhqQkQfK+tfnrmzqMiPm8wseb2hd3/TQbENdD8WbD
hHqvfBQv2pEYvLkdJmFTmxt+t4M27PkXTuch2F2bEKFtRPks/P7+xWcKEO5Q
s/9bQwe06lI/u5xphXNB2x6Fslb8fCnub00SoLdtwh7/mRSnHllVuUYfxrDA
kFE0w1Ysi9xTnTutEWNpJ39qplSisEd9co+4CSsOn9uU3VYDt7dHGZkz+Ah/
x2PdpvbC9UmHVybLV8B11bAWlcvUnpW9PFpepQRRV1u7ZlkUQ2vlwni7pZQP
m8MFmWVsTG3jh++9n4ENk3LYL/dR+WvTonFPLnYj2m329Wy/biy83RN3u7Qe
7XqcLw+o/dJlfPyfays6gIJYhcyvVG7Y2pszW02AMGW1sRU6PDjP9ZF3ONeM
gVsvp+kotSBn8Wb9Iu86hK1/emnapiak8bTenFZtxImnd476nfyEAeXBdRe1
KlEw6iI9uO7v7w/pxgbXspGr397XpcUGh1yPI9cT4j+cye7tRB65Txy5z54v
p8amdfEgFyJ9rog8V8tu1rL9W7jwIe/5gryn7kOFNuMf9egj33WEfNeJPbHc
4xMa8ZjUIYnUwSfRb0ZiSDO4pG5BpG47v4Sra+f//R1NaZ0/kjpfizu4eY9X
J2LIuViRc2FVFHVrv0wAj5zjBnKOey8pnj4eVwFlcu6W5NzVme8Hix9SPIiT
9skQobRPGmw2LUgo78Yz0ldzSF8F79HVad5aDZqStA/dSB/St4ZqxatTfiF9
W0f6NvD2seP+stT+myntcxXS555FZ2uY/lQ/krmg7ZXOhdLorAt/TLtQTuZI
d6V0jgo1Rp2SDC0Bi8wd7bV07uxT3Sb8dGwFl8ypKZnTQoGae+xwMerJXJ8n
cy24d929Qb8E5YQD+oQD1XmZnlfmt4JOuCEh3GhqtT8Sz+0C84iUMxLCGbHG
/h+zDpTAiXDJiXAp+dSew1pZrbD4h2OWXmpDfUZTnvmHe96eEbt/rK6AD+Gk
J+HkwE3lpwevCOD2D1cZ5pOXHvF9DS7hcArhsFMi89tAdyNMCbdjCbe9mz84
jErqBIdwnk04b7B8u1fS+VLQpv6vF7S3f/FmbmwDi3iERTwS2vmBveFELiyI
d0yId+qYZYorvvIgJJ76Sjz1qqTrcP3YTDgRr0UTr82wsd2l4MuDE/GgFvGg
acCUPlEGtf/6Sr3JJd5Mry98YfCDB4aT1LNxxLMW9zLHW0dlw4B4eTnxsv/a
yKXGxjzQ7kk9/ox4nHFhVsme+RlgE+/bEO/HXEqIb9VtRTXJCWySE+h0r7mb
Z7wCjSvNFUySK0avOqvslscFk+QQmrc0hxxTuxP0WaMbgSS3SEhumThHL2vB
0VqwSc7RJTnnoqX5h88TO6E4QZqLFEkuUtIVPVl/tAS0IGmOkpAcZbdrScse
bQE8HZ4+uDGVB/rS2fnmPRww8rXGLS7P+PvnRHsWHC2C55u6cPgWQrvQUsDz
a0Zc5lDRfWp+FY+VeS3+nIODU2s7aWu7wZywS01hOeXX6gV2cavf4+fMsauF
F2pBAz9rZGci2HW3pptIXuOh6JChxTEqbyycl5VRLwDz1671FxSjEWiWYO4h
KANXPrzeO6sSKeN8E9pruxHAEf+kpwpBM52mf8P+BILDL+cNmFH1f/1o8mCX
T4izu/jZ7ygf0TPa7linUnn8+OeiZX//OfPbTWU3TlH+kXtS46XdCa7SD9dp
CkWgsY5U1L59ip97BD3JEe3g8Bb5xSVTOUm05nOWajs8Bi0fXTpLCKZaJH/U
hiJYJOw400ZxJexAGdO9XADL1FGHJo0TIvCLsfa1yZUIu6PwPechH74HlJ/K
niwB96UR28o5Bx784kiNMB5U/nwXyFD3pXX8KM4MjsCRw6mlrfGtyFIXjNT6
U4K+P9Hvzg3uQthb/aV9/BbEywxxrbWg+vd3S9fKw+2o3a64xG8WtQ+fCoiZ
r1kIxVmGn0XUuQWMX/Tusk8zxBtqv5hTey/rxhVV718t0Bllarn5IxfclrhH
YfKvwTLdt2n7oyYcvLvZaPNELlhPXESdz5rgNMAYPVa/Buz+wRfOv22EzMLh
ulZ3O+E50DlixJlSpKz8fveIbgNYOgvkby78BEb658FrGnIQt3e/atV5ylON
e15XhlP72Z47Y5RHv0T4ulFbI17WoCPHmTZsXDe03/U6bqU4V14W0SOJrkJ/
6jWn7IPlsF+t8efv7/NJaGvy7jtWQP26+CVvEh9xh/Znb3nJg73k9NInhaVI
FjumPszopvbelXXMrka4+TSqbHz3CZ5hn/vmHSgHa1G1G9u/DHQFm8tGwz6i
vTRYL+UGxYHeK4E7fdLgNL7s3XjT94i6M6gzvEUMg/kX1RRLRWCnT7R2jYiD
8u4DwwuvlCIxp1V8NIgP95Fvi6uPdWNpVe1Yz+ZW6C5b8H5QWR14N1/euXeM
4sAs05F7hGKw13FGsezyYPRHODZ5Qjvc1k1he+lVoLJz6pa1t8QYay0Z0XiE
D+36WpOPlgKktgcOjFcWIm774Ur13mY4XUwL3FD3Dpn1T0q26VGcsJnBTVNq
hOB43Z9hLs1Qfq2buCepCa9aMuS2vK+Bd1Vj9O2NVL+fGajuyq/Fhdmn1ffu
qERcaGz86INloK3MZ9zSq0Ru9jXXpQplCLt9MVRufyvSjiT2K8iWoHVMw+5s
PcrvhrcjZC90ImUT435B3QfIiT89M5qbDfW+adY/5r8C1+PbeF3GS2xQHX7w
kvULvNvwPlfnUhEcbjceqlYRQ1KSnt4yUgx7xtREkWMlPptuir03tQNxgbYR
a8+3o5T33HZeVS1q2MUVepsESD8ytzz3KA8pd7+x/iTX4cD6MzsVaa0IzTvj
sfZhI64dKF97xa8aprlnKr9d5KJ4RM+dE6kV+LhH1NowoYQ6F869kfsaYfXD
S7/BhYNTHz0mK0Wn4A257kCuv50edGJFpwgW5D4ich/5ArnRN7zbcJQ8N548
Vydv/LbUtFbwyXsWk/dUXXBhHk2rBX/Id2WS7woZ9WSzW3ALPEgdBpVK61Dt
2P1IjsMD84C0bjxSt0fPFIoT71G8spTWeTWps6zhtgp2TReyyblIyLls/hl5
pqHzPZVzpefYQs6xjyMpFk2rgQU590By7sUJsdF9qwXQIX2SQfqEe3ls0IUJ
T1FI+opP+oopu5bWs7gBhaQPU0kfLj0g3BBiIkIP6duppG9ppuuO7Lcrg+56
aZ9vJH1Oq5Oor3Gl9ncyF51kLmjbTq4/cOwB8sgceZM5Em1aV6u+gsqRZO4U
M6RzxzIz7Z+UKgCbzKknmVP2boH5/MoH4JC5ZpC59rSrDL1eXwk64YAP4UD3
gotD3PIovxJuuBFunLCXcRszkQl3wpk+whm5Fg3WosIK6PzDpWeRKZ/u5VD7
/F4px6IIxxgzHn9BxD04Ee4lEu4pmujctMyh9kTCyTTCSdnDX5bfXUB5g3A1
m3A1UKcVL4Pe/4fDbk+L5C7P5CKGcHsu4XZ569qn7iupfZ5w/hrhPNNlvbLT
kwrYEi80Ei+wuPW39L4LoUh7/P89cp94hCb+sjskrRi0Tql3fIh3nCXyP/fT
2uAUJ/WUN/FUpgzdRXS9ABLitSjiNe+8Q7HHDdvgRjx4n3jQYuJHvbCET2AT
bx4k3sz2ue2X9ozq/0ipZweIZyWLgr7cDyyFJ/FyKfGy0lvD6EVpQnCJx18Q
j7Nl1Vz811O5xULq/RDi/fCgCcf+TKfuP0qaE1JJTvBMN1E6MZHyNckVtH5p
rujbOr06T576a0NpDqHVS3PIzo26P9NVMsEluYVDcsu6zt76ke9awP4n57Ar
355vaQr5++fC/z8XMUkuEu2201vUUw8WyVHaJEcNuN5VOzZUBJbhmMFPv5dT
e8u1/FWlRXA5ZFfWQ/EjrqDvakRQFzyjM+dvmpkH2+Nn73YfqIIi6/AgWxEX
gauKXFPsM9FyJqOs+HY7GEZrWSzFTNC1VFnbOCkQDmqsP51cDLc0u4ZdNlTf
LT7R4aX2BBLnRafD/lBzOmLD4hnZ1UgtuOQb95vK6U+vLqdtyQRt9tLrKx1v
wWVygNB0uxiW14Yref39faMr2WHBy/mwVOhYeeoVta8UC6fIjn+NuLfHBcnb
qPdPNVxo+rsDNtlsv4U+LbANDtNWf9SN8M/mPPFqig9uoduKzZLA6WtRmFXD
h2vm/HXj77TB88OkFdNPtcCz9+BwoVItdDoCUnYOpnJf6GlGcFga6Bs58xvY
H2Cq/zPDKojKSyOH1XLqeVQOpJlfV+9GYVDPH+bmVigWpeitM8mDNjd7WcdA
O5ZP8febu6YFBhrNRjZ8AYTBVd8dS/iYsuNOT/2uZszw0pxPv14IieWewOHm
LRA/PISEh1w4FBysX5EggO74Nf4xBxoRMHxNfmg7lYdc7G1MTXLA8Pis0phY
jcSfRazL96n959j2C2U5VH0uzZqpOb0MpppXxrTbUDlXMOfUuBnRf3+ftvbL
lAIkJu4fSBpWD/acrqZlvHrQEpIPaEzLQEratRuSh7XQUhtkv2Hm33+f93CG
ht5dhOpneJ+g+E67fmdc4qSPsL1z9PDu0C6wbh3RtjxbhfJB9+dZ/GyG/6uT
drXjOmA1KWg4l9q3XcvartzZ2AUl2rIxMyCA/+voMaejy+D2tiDw3B0OrHbu
SP27vymNdZzYYlOCjijO74gB6rzKHnyJ5FEe2UeTL3MtwveSIymyB7qgO2rE
bTabA1paf9ENq1yc1rplfzm2GAz9FeESm9fwOSnr0bKOjQoVx2CN982wEB6Y
m/eI6mvPyADtaXH49HVIDyO3E6IbfZ0KQj4W8jLb8u90I2Xsmy3hu3NhYJTQ
ojG1AftWnDXveSdCX8ulyas0uYiLTHsd9iEfwSctDBS3t0N4snX51Baqv5oW
6KddFKNk8Crv7gN8eFY3QML+CKc7q44HPhTASXXijeIpzTC1WTB/rn0LTEbq
sRf+asLGT8dte7qpeu4Ndlq34DpM398tLpGtwqlBrM68Z2XwPLbQoZ3KDTqr
BOHi9aXQ7C7wTJzIQdS+osY130TQLfm2kWb/CZ8azdzOyBcijx4aLzYvge19
/qY7rvmQG1cQmEDLA+3IZna/QQu0k6Ice1Oz4OC3IfX61kwMU9DnX/jRDtbg
4taye0nQ4HYND2QnQtb5xJGt2T4wUr0qM7q3Gwf23vGRp/7Xylcr6sF4Kn9d
PvDxeL0IxqnhW9bHi3Bx0+aNMyh+uFj4KOjeb4ds2pd776h91ifHNVhdUo5l
NO6cG6cE0ItPUA1K4WFP75TFQ9gVOJl4YszevFYYnuEEbgznQvbc750GM8qg
Mv3pjKAPzZhlG75N274WJuPONRodyYeXyZPvhfFcRDrT156l8pz1k74632/x
8CbXX5DrHzqYURYPuqBO7rOc3EdjmaPpAed2+JHnWpDnHlw5qHinuwCryHsa
kffMO+/pMqDOx0HyXZPIdxWHFy6UXORjJqnDRlKHdVsqbUV+bdhI6nac1M27
uTfk9s8OSEid3UmdrYeP23C0uBsMci6HyLmYZc/bJ5yUB09yjmrkHA2Utp4e
dq0OBuTcK8m5v+1OiJa1bYMW6RM10ieMqGC599YpsCV9dYn01SljvsXIfC48
SR/uI33IfHGyctG5bhwhfbuf9G3c5rv3Hq6pgy/p81bS5zKXZSVZ19uh+1A6
Fw/JXOTJHpQ/fjQbnmSOjpA5YiwV5R1+0wT5IOncGZK5O/t2TIfy9A7Ykzll
kzl1miNZ3N/CBpfMdSCZ64B8w6rUk/VwIxzQJhyQTa51225N9R3hRgzhhpPX
2uWpFxPhRjhDJ5xxf+0gwx5cC1XCpQDCJW+NsMUrE9oQSDhmSzgm2fWepXcl
iZprKfeiCPdY/Vksx5EUP2uknAwnnHQJn/Qr6A3lQcLVNMJVxYBf1/PtcsEg
HE4jHHbaI6fYcrUZnoTbZv/H7alqedfuiWFBOJ9COM8Zteqb7cFqWBAvBBMv
0NvcOro8OtBHPCIhHonao7Lm/J1S9BHvaBDv/NJM/rK4XggG8ZQR8ZRa2grf
zQtL/+O1+sIxh7rE7WASD5oRD+p0fFnvsLoSgcSbc4k3lUZJjj53F0H2h9Sz
nsSzcYcLLyQMqUMo8XI08TLjS4C+q00XxXepx0OIxzlHT/UEK9TBh3jfhnh/
pUCj32EF9ffPleYEN5ITrrVKqtSG1yCO5Ao2yRUWt+ol2NuJGJJDaAbSHHJn
6c1BBbRy0IylucWJ5JYPp/Nk6ZZC6JKcwyI5h/Zj0Kts1kfqvv+bi3wzVy6Q
NeWBZiTNUSySo2hFtvml1N5Fs9w79mvlbdBk540wv/kR3uUyd7zlhXCqSp/w
/hWVG5qsBOzHWXAaXTi7wYnqE7+hl9aGF4AWcX74l2lvoThwvm1IRSu4M91q
Hm/pANOhONR52XPENS61PjE7DgyZ/Ste/6wD7dqzvPgQX6TNuevy/AAXzNse
qkfXPEdInzYjgJrjxe84ZROndIO+P7NiZCMfRntD5sV9FEFo4zFvBephEVed
x1tdDrOdolWRvzoQ7u+mmXvx/5F15nEx7v/7H6SSJElCCCEkIQnVlSRJCEn2
EGVLCEkYJEVSRKVUlET7HhXTok37vjdNM9PULGWJJPzu85v395+Pv87j3Kdz
L6/XdV3P630eB0Ioy1aMdc8U4YQc913dzB4kqOUmpDU3w+YFz2xVVRckK/y/
KOh3wyN95UkOh+L1SYkz9RY1cB/YuDvShoP+d0rd97+3gTZe6htj3CMYGti1
fA6hvrtvguyhnD54jHkT0PiZh9Wq4X8mjmchyyXjxMnYTkhbHC2KqGPhoufm
z0o5TKjee7Li0YwsJLgsbN5o0Ip9ilOXxJ/sgGKAeUcCk+pjLEum9OdaeKiY
8zbntMFWK9qvXqqAmuP2/Vmy5XCkBRx1OdiK2DsnROMsOJTudNwti/KguqNh
9sJ5LShycT57dm0ODA3zin45p8DG6cRlm9/UOcu07/Mb6nxNK/r7onLGFdDc
54yfW9sIGvPjpM7LznBcqy2oLBfCNXCqvmRIAxJyg97aODSj9cTsFfEmfKhx
vAa/WNTDb5Prtu+U71SMVI5KPuDBaUfIQkZXLax3qOQuuZMHwR2fSQFPOVBX
L6Fpba5BSJHTw78XmQg+WvMhl+oRblpFdMNbVZD54jDNIpIPw4ELM774tkEr
8uKr2QPlVE59e7WZXQzTie+H2iLqYKOlfuzSphLUH7M/82mQif6ni7/S44sh
X7Vv8yAvH2OO9O0pXMIH45l95r2LUdCKyZmvZZaNfv2y+W2v8uE3bda960pU
Ps/cfmGSViys7w3wDgjaoHh3y9otMlxwJXPKPSP7ILc1u8A4igfGkNerv6/b
oX/k4YDPf3+eheX5ebrcJCQ4ZT48nFaBgkKjz996e6Ca7LTv/vVGLPztf9SO
Ohc5TFdbeHYaF+V+835LB3KxrM5rDM2c6uW7Nhq10alz5dKWvVEDsXDtW/pG
kNKG5FHKviPsGmH9Z0rZEk/qXHu+cHrnzgoYBvPdnkiVQm3UHOemEUJY+HXd
a7Z+jyGDmNzK2Czs+5ShfYbRDIu6ndqM6amwG62vsMcsBeVzciMWbRfAxmPX
4hErU7FpmLc79hDFDztroUFAJSz2poaqjEuBy8kJCbQtybDrzT1WEdQJRoNX
ytYH0bgZf6P2Z8MbmKu/LiwWUOecYYbJ2ZF07GqYYTKKcx0HkmQURfEinHQw
vLJZm+Lp2nfB3HkitD5qjJtelwGPL4Frr+fzofV6bFv1BT5Uzq/Zc0BYhBFl
RsM/1XtQMnqEyYgtPCwpiZw5YWIlHC+ue+EWxkXqnS/j3/HYeFtf4x7fTfWx
MyyjO6fYuGYc9PjB804U5s70Dikqg/O8pX+i/vtzN1zK99iL2jDwydXR/mo+
HtVfu/uH3wmmsbz/F04zZLW+LVy1NhZB5PowuT6QZXFasEiE6+Q+zuQ+w6XM
rpfMXpwiz71Hnms23nlf430eLpL3fE/eM0TxiZ66VzdkyXc1ku/iPdVwvPS9
Gw/IHHTJHPTmnD0eld+Ds2RuPWRufkI1s30cAWh/xXPe3CqeM63F8sOMES/B
JHsJI3tZdnBvZ6N5KXzIHlPJHncucll2qKgVhmTvmmTvDqU7PJlUHzAkOrEm
OmGwRE0vHLJBJ7qaC7GufJ6GHy0KZcGU6PA80SGD122g/TkdRUS33US3g7Tz
U+zXdMKK6Pw20bnqZiOXl6oimBJfXCO+KK1PGi0qqwbtgthHFcRHTyI/blvF
6ALtl9h3ZsR3Dx8v/aUbLoQG8Wkf8altvgxHcKAMEcTXqqpiXzuvvn2vcgyl
5xBxDtiQHIhOkE2Tru+FVpA4N1RJbhxzMvJjGebBkeSMM8mZ0GVLZh2Xa4c1
ySVDkks1TSOfCNx7EEVyzIPkGF3X+OGHYzkYJrmnRXJP+YxSC02hA2okJ51J
Tlrwncdrz+CDTXJVneSqRbXcu3lepaCTHHYjOcycsSy8oojK7VJxbkuQ3LZ3
qtWXregDneS8Lcn51jOzVN7UU1wjXFAjXLjW+Tw8PJ7/D0dkPi+cwz5I9UTC
HS/CnaXadXtkrvWCQThlQzh1pPLY2hCD+n+4Vqi5OXCxqwAJhINGhIONMg8W
LnhJ9cwJYm4aEW7SvKR/XN56HXTCWW/C2coXTrJvJajzOOGyLOGyjYlRZG1l
AiwIx88Rjn8XhryKeMiCD+G+BeG+Y1YmuyAnGbqkJ5iQnpBWa2p/iNkJWpC4
V4SQXjH5EO/2aP1gMP6vhzwU95ChiGKdl6odkF8o7i0M0lvsI6wv+AxRfCA9
R5X0HFWJ71qDOxtBJ71IlfQij2HZ1KL5vaDtEfcoVdKj5OMFQYvLK6D67vaj
1h4uDF2Pf5yv9hFhh13NWMMUNx7se8a7WAPamuK197qy0Xizo/YBrw9acmNv
VpRT+nfdpfFtZiLMjy198WdkK0w3c7NtpzJBsxekRB57AAc5KR1jM+ocYzZn
fFpUFvwSkww6CkRob5rhb/GoDky15ukFK7vxJ3zjrrW/BWC/uTzd+R7Vjw4v
WHzybx1UljTXnab0Wvj1j0a9URVsIpICr38RoTmu3W/cgh5MCIl+MzCHBw+f
Pa/fq3VBELOPvn1GN3SfzWw1sa0E46bDxlETKuDxeU1fpQQHaga+V74KeQjj
S5ttmSPEibHM45ECFqy383foMWuhywqTdc/ogsh5TvzhL0wIDggTBmv5cBx+
PMlqsAFuTcfU6efbwdg600papg1hg8p3A+4VIX/0xGFktEDLf9OeB0PPQS8Y
nvptSzysHQ3OhiQ2QWbawNHvU7swnL0+UWFABNW+TvsxyxvxkzXvqXcAA8bD
WcczqHOc5f2V5UskG3B74wlNZy3qPK3zISE3mjpPrYudsZ1WD5uQMZJx9u/Q
+J4nOFHOQxFNdObA6DqYBpcqbgzsBC/bTTeczwXdSnjwyahanAg9n2St1QeN
qV3pNUI21Fb9DOaIqpFR0Sp91YLiv+emxZc+Ulxrn7zwfXYVBKv1rnpkdCNL
pvlZ9FEmTnWFWa+zq4RPRlPAK9lsOG/k3Blyo87T/bL55/PLoHZTtJu5oB2D
jh5rFobVYdDQd1GkaQmKcjQ/Gqr0gulxi4kx5ZBWX5w4bV0BrE+ul1cfmQem
euYozpH3sM9bkDz0JAd6KQfr7k5qhyqGmlSf9yFh25ogjbUZyFi4J++yRA9k
XTJ3ZXzuRZjS6GsKJZHIHGElrMtLxT7P6oCzcRzskbRZvdmtD8/kz3Pm2Dci
7FHE7fFeHbAxfX5aeo0QpcsKZCQ9OaCzgqavMa8GZ9wirxqnXiiP9X10ukoE
eodVq3NtNI4oFUiI/vt9TS7FdSc6VEIUUXzo1M0eONvem78ul4UTcdt7R41m
QdnusYzSfSa8X3z9ohnXjJDiHu3IQgGc/ZcEHLpdBuMF0lfMfUqQMS99+bbp
NTg8+PnU3xd9ODW0CJeov9o0RNz6O4K6r6lR2YRRAjxStNMPyabycfUGtUnz
GxG4qXmvxBkBPEU/qkfpCiAzNyOw7p0AE/bR94X1C/F9v2sCq10I6cjMiXfC
qvC+fISaLFOE/LL1h9uaKf6ZbNn85kUnZqkF7fDlinDlRfKbkE4R7pks31V8
kAfFdUURrVNEGJyxbkKDjAgHL/jfOvWM4l6Gt8nINwKU+aw+NceF+p4/ywb/
xrxG3KVC2+dlvTjTYn92vWUv2r4mX1wWn4Mdrq+b/2bzEGMx5FmW343UWr/Y
3/pFsE27mVSVysWxr78aP0ly8M5ihsHUv8XYdXB1YK2IjVHG5W06J1k4aBxx
5IJEIQ5Ih91e9awLVeG3Vs+1YsLPY4tH8KsPWO0+6ZhVD9WX7OV3eFu1I3Fe
i066TyT0yPVr5LrX5ode356IcIjcp57ch6v53UF2rAD7yHPHkudyRUqvWAZU
7yLveZa8Z8NJZ8dV9B7sId+VQr6rruSpH72/B6lkDs5kDrKjdhxe7sfHYzK3
KjK3+ZWuNK2jIswgcx41Uzxny53GXUGKyf/sZTN3WXgzvRyZZI85ZI+vzh85
Gv2O4iTZu4jsffbi2RezlnTjDdFJMNGJapPBsh0L3uE60VUQ0dXqI62nLwi6
4EJ0ePv/dBh+6vOIwU9gEt0GEt2WnlI7tussF7pE5xlE5wkpidbluW8h91Ls
i7vEF/TwPYsGB1rAYIp9dJr4yDFfXfZVUTeKiO++Ed/RtvUVzOt5gSji08PE
pwlPz/d+FdbCkfj6MPF1ySWTcaJPFM9IDqhOFucACzuNbm8QQpvkBpPkRvxO
/y/Kok+wWSDOGTrJmVHzLB6vHqDOwZ7iXFIluWT6UvTrAviQPivOMdpacY7R
fxrzh70L4UFyz5DkntIH6ZrxfCYqSU46k5ycWsyWPXmA0tv/5Cr97p3Rfd6V
MCQ5rEVyeF+A75Wchi4Mktz2IrnNUPZK7PkWCCbJ+UqS85WFx+pPl7X+w4XD
pQboTRLAnHDEmnDE8vSi8PGhDZB+L+aOGuGO94aLu1ub+KAXijllQzh1v2SS
9sSSpn+4lmSyV2YXpcsEwkE/wkFL6/7aXy6dMCXc/EK42e9mpjc9hoEiwlkn
wlmVXTL1nQ84MLwl5rI34XKRdOaEZaalKCIc/044HrO037e1rBsehPtMwn0b
j207FzhWgEl6girpCevc6T1+B7qhPlfcK6QjxL3CYueUuzOri0EzF/eQYNJD
LB45puzS4cD5/3rLcXFvSfAu3Noemw1T0nMMSc/pN++Oo/+hznukFxmSXkQz
vqq6yNEOFqRHWZAeZbhG6uQayVYwi5Ju+v9sQn9Cit17OgPyDsELws80QV7r
0Ovv8ZQPtbaOXSaZAo0auytrqXNJAEdhSnQE1bsVjaIUXJ4iI3mqouR/v9+1
rmzbuNWFKI/eF+il1If8l8njz17lQotvXTGg2oM0u30O0euFkF3ucCVSNx+G
fnmRXrOaYB8semW/n4/5cwdOhbRR/dfmw89P644hTT8gpWx3D/ZtG3FYmZYB
9StW9uZUbxT4HhppS82Pe3gEL5Hqt4bfZizT/k/3g2qGom9sJJykaX/oS0Wl
/VEFy5t8XDya2W7mz0Jv3je+VEoXbCb5z/htzITatlVf0rWoufoP6E72K4Ch
5dUQtWPFyG92MDKvbkHl0idDrYxuKAsnrVJzFiFrpUlB39cG+DjcHfmjoB6u
n+PWff3Bg9PXybZ7p9VhZNQT9cHPImiwzc/evcaB5eMxfT36NbDttMhWi+xE
zMt7rrlbumBa+eztTJlqDG06Ndb5VwZo0xOP5ud0Qpt97Y6pRhXi14y/UnKg
C6cstAo7zjAhnVnjrHC6Egm9TwcGTqcgyjm08su6dmQEGawt31sB52jp9oZG
JhqtsxeyLlI5MMf72pqsMpSvn9Fy54gI0pVLpS/vaoTHo/Ujw7mfIP8mKnj+
g3oMXvdyUC6vhenMUwGeJcXQdVlxbNx7LsI4JceFHyup8wljyvtdhZCnN2/7
rZ8CrcD7dUcsi2DRaDxg5ZoP1dqIPuOYFqr3jpvt65eJok/h3ufn5CBt3PbJ
d5J4oNVfDFnw7R4YtZO/zKN6YX+W1eUTi95ieO+JPw40IWiZGqcXUro94aOV
ouHQDNmN8ZUjJveA9jN+cumVAMx79RMS/VxY7x13uCiCDbncGRYuqn2IvSOU
3GTrjsp7+RHnlnXg0cYLGjILhLi38lFZyu9q+GgMf2rPrcHRsyLb/QW9GC45
nX/cgeKA0Sdrhl4mDmq7f2it74aTxuLlt/T5CNDWbvusIIDX4/28qPlsWHfq
Kk7XZ8Da1+L83ZIuaDIPpz6vbIfuEPuNxaZmtA6oJDtI1cHdbu6+Zs1q2K8W
7nKQ6wZDUPVJ474P+s+UJm066QOm9rUNeQvfUvpgCEVXu3HjT7q2pAoXrZ8/
XLeu6MLvuwIGT5MNlXVrnc8adqK/UsF8WmADwl6NfL8oh4vPH6XCC5Q4iFkx
5dPdDyK8Y7LX/WjpQad7xg4vzR4wcntNPbNrMWpQ28PqIx9rg8b1l1zkU8/X
6BFuY2HkCMnIqCsCFGYs7lhtIgDSDbwnbubhcNWs6QWg/l50xyRmpABMg0hr
yS8C3G3atKd4B+XDtAc32F96sa+3SnfE5AdoWDT6VcC3HnDmfvUZNO/ByII7
uPM1HZa9J+7f2svDqTmZe04bd2NgXmxZLS8HxokTLll+4CJn9OyPSxQ5WF50
2C79fR5injG3btvBga9rW6vdqC580PuTNOZVDk696do4z5gNpqVFu+zhTlRd
Pj53zi6qX6t2KV9K7oK9fGf4bheKi2f3hcrtfY5Mct2BXO/WlVHa2ivCGXIf
NrkPo3J70LQBARLJcwPIcyscDR/pP+HDlLxnIXnPz5vnz9suz6e+W/xdl8h3
rU/zKy25zUcrmQOfzGGD1HjlByOE8CZz0yBzO5yh8iv8Vh+OkjkbkTnbjSlf
XXYmA39o4r3kkb3smzO6MPFTOf7+EO9Rn+xxp+qrxWr7WpBI9l5P9p4Tcnfv
mucc+BGdMIlObKJObdkhFYEBoit5oqt2/++7zy1nw5foMIrosEiFTtt6iDqP
94t1+/y8WLceq+T9XzULoPVdrPOPROcB9+ZazOmqwjniiy3EF/K37r/Z1cxC
CvFREPFRSeHndxn9fBiuE/vuBPFd6ZZNy7+OyQOD+PQM8emWE2qXNOa0YJD4
Ooj4eq+tTuDcG1zYkxxQJDkgYcaucYjsgyLJDUOSGz2JsvQfM6shsU+cM/Ik
Z4Tp3S97/vvvRc3iXOonuXRTvS7hIfW9YSTHKkmOFRWnv+haUQYLkns2JPdm
PFns0baAhSiSk0ySk9oJ+TsNnalco4tz1YLkauPYx8s5t6qhTHLYh+SwRnT0
0lMZ7H9ym/5y2pabha9RRHK+kuS8/WvROost7bD/Hy40vjBfu/acEN9UxBzR
IByheXaOOWnUhBTCHXPCnfKLD//4MAUwJpyyIZxi6x+4nZbcCjfCNWfCNZub
jy9eZYZBlXCQQTjYXvTumlvwf74Vc7ORcDMjuEbG5Bp1Liec1SSc9Xiau3Ro
Ty/kj4u5fJNwudJZSb3fvxGqA2KOzyYcb/5yfYq0kRAWhPvfCPfdV/9eEtzc
DJqzuCe8Iz1hRP2Hr8N/BUggvcKR9AqttEnz8owbYUN6SBbpIfTzaofq/zu3
k95SS3oLLTWAOX10DbVfcc+hk55jEfllwLWzG7qkF9FJL2Ik2ygMWVH6KBb3
KPlEcY/SLV56UtO0C/RBs7mq6ekwvOOQVGGajoQpizYXaJYg6tJKpwtU32Rs
uiU4szccpUFuofz2LvT/lTJ/UVOJY0a7Nn85R/Wva3cmqzgkwv7v3UNHqVwQ
eMXFgDoP9w++VPYNo/pLQV/a641t0JTnVqxM4MNGQjDjrAH1foGBt8q2x+H7
ly6jB5E9WG0XfUR6YSts7ozckbCAA5/BUY5zXbphNs7V1Hq1EFq5lvJvaiuh
feUvy0GNA6293YGjw5qglZ25NTyGj3wjrWt/QlnwMvXWiJASwvDtzifKi9ph
Lbi8en1zB1K+6En1bm4F/dnsuZOOZkDrSk1U8epm0HnhN97O98HwuTO0iXY8
BHwz2HaH8ofsGoXU62e6kDX3d0CdCRNFS3JHJO2sRMxoxrpQH6pPLp3NWHun
HgG1JWHV1dS8lliUy8sLYW3OuXGWXgWLI4/yOMWF+GJXe4MbRJ13VnfDzrIc
CXGnnHfuKYCbyWy75RIpqGyf1jvtxSeY/nnOPeT2ESf3BN6U4lI50JTQEbGi
GIb5nzgduflIGNV373heAtUTlHyLdhWgcs63NW6teZBJqzvYTp3X6VsOKuZl
56IxuflTYUYuSmzCbrWXUnOQN9FjuGVhcOPabQpmObDXcSpYpkmd0y7EGio9
TISFTtbFmWM+wPxRVP7WPg7ooTPOHA/2hQ096Ypp6jswxp20/3noNeStnpVW
+FO9f/mFc2MvpYLRt7tQ9nITYracC1yxUQD6TNev7zbGwVIUHaN/shtuD4LW
LX3y36+fM5obtDgEhla/ji5//gaWin0d9ee4WHctc+yOsj6MFAzkVNAakJDX
sk6xmoWvnMzlKg4inL0z9a7nAJvqby5xeWZteO+i6nqyQIAp0ToR4ydQ5xn5
p0UOGVQ+P+t7OTW/Fx51Lp4hp6lza0707wavHDz+XqbiRPWJ+ON/ZDOy23Hs
Zuiqr7NFSBzZ8WepHgemOmsPZLzuxkOlGdf2/aF63MagzIodnaicZNW3uKYP
hpM4hhov2iDp2Z6wdE8jwmZ938MeXwmbR+32oXkfkRJVcTJoSj6mVnx43Z3A
RMClt19Zhb04uGDTouObe6EZ1xPrNZZPcaFf6sLjDqy+krvmT2k11CdJbYqK
bYTjibejlbiN1Pxzkh6Gx8B+T+PwE5dWrLnj4hk/loXq6DVD76xaQLu90/un
+1us2VKSoxPIxSl/489H2tnoDVO2ParQjJsPxm44atiDLyfkArNO8vDj+rkN
2s+6oLNRtPGjSy8srt5yft/fgx3al+u6I3nQaH3jLpvQCw5jfh5TqxeqIfq/
fZ8L8KHi5RueZi8yNMYxvPx6IPk5+WpEch+6LlkyN6+k+tX6npkrbHhQi5Za
mX0iDmk7YakygodIi9AxJaVc+PlvcnzR8Bax1msYo6jekkUv0X0/kYMVj2cE
bfTLxpZnLQVPQjh4sn+kzuPzXXiwtexxVs07uDyrVrEuYEOz50qQ0mIWln/J
WxIckISqoiLNyO1sfF8/Na3HuROlZsM/L/k8RTW5PkiuR0zXMnRd1Ier5D7L
yX1Sb0i5CGKE2E6eG0yeK2NlctqDOucnk/fMJ+9pvTi49OJmAd6R74oh3zXu
/vfEsR8F4JA58Mgcdt1+oGE5RYRcMrdMMjdfxqhHTKVb/8z5xPiJSo5f32Ep
2YsZ2csXzWlvbGzLcJnssZvscRVt2ln/j41YTPZuTfYeHv91c9l3FhYQnfgR
nfhsXVGuWiGANdGVjbZYVzo1j26k5HbiBdFhAtFhxgXX5kWvu1BIdOtPdBu2
YbdpWVUe+onOl7wW6/zbIXqMVkg73IgvVhBfbOlzqx432I0E4qPjxEe0+QOW
0scvwp34Lof4rvTR1XUPYioRRnz6jPjU8BP955znVG+YIPa1M/F1qtnz/QFq
PfAgOVBAckDX4nHW6L44pJDcGCC5Yf25hC9QrMc+kjMmJGdmyYyeGXCdDS+S
S3SSSx/NwooFO0XIIDnGIDnmNycq0m1BFZRJ7tmQ3DOV3hF8eR3FzTBxTjqS
nDTeevk487gIqiRXbUiuRjXFXbSZVftPDktwPgZz7SnOkdxmktxmDg2zxgYm
/pPzln6Whuu2d8CQcMGacCF35s3yh93CfzgSvKlsqH95MxiEO1GEO1fTpz26
fF0Ie8Ipa8Iphtau4WVF7VAmXAsjXNNyu3b/8vV05BMOlhIO2vyeeSAhvRuK
58XcjCLc1L3b8phG9RtaqJizpoSz0p1flude6wODcNmJcJntfmhYpqkT1oTj
pYTjAdn3g9pzqV5CuK9DuH93u+zy30u64EF6QgDpCbZPZmbpL04DLVjcK/6Q
XvHGfGH5r95O0EkP0SU9hHHl+uzWpQEII73lO+ktUXIp5W3v22FIk/7/PecU
6TkW8uOv6lULUUR6Ec1c3Iuiir4z1I0aQRsS9ygb0qMqV6We2uLUA/nVQxlb
Gnigcfv1v857A9ot03oXkRsqnz1OH5VSD82Zuwdbm/rgbPPaX924DYOWb9+e
nSnClkSrpG9/qR61+8fdi0sFFCc8Rj9V7ESSQaMl5AVQTmb9TD9Zh7DlMdO+
P2LA7+SzFWNH90Kdv2jDjX09kB7xIspzTTdU3GY2aDd0g/nuuZ7OsUpYXH5a
YHanFmYMq2XMqxxUK4cdf2jRDZ8Lb+8cWyTEDBOdA+YiFnR1xrdW2H6CPMfl
dmlNByQ2pRhfVWHiUbL9FSvJbhgObjqW652CgNmnxqb9aYL9oREjgmwrYe9e
eWrsLWovCumC/YY1iOgc6v3MpN5rv+at549rYPh04Mg4uWLYb9oQ8mkhpWcF
yYondeFgcvpf0O9kweaBSGlOcT6k4z+qWFHnY8OR9vHBtETsy2spXh7Ch3mi
yYPvJdS5bbOj9Nl70djTc0vFZBYTxvPO29R38cEseuimohSD8VyvlDPDSdAz
3sOSWC4A7cTUxAflsbi1X38pq5IFx7q12RW7qHPVt4bw9KdxCLstMc1YNR4R
L2d5yawSgBle/P5vXyz6OYb5FZOYkDV4lTVcTekm4ni1VkY03D5pSv04LYQp
W8B3ncuHqvr6fUuFr9Cfp958SasWjT6Vsm8oLjCqVvVbXHmBUxoIqdjMgerh
3IWfe3igP1zhteXCE9C+uNl8/BQIFVudI/upPdBSFFXyex1A33ygT/ioAVE7
6LrzdnNQNc/P8rBtH5LUdvGdTblIiPT0mj3EwuohK9eHV0Uw1GLpjNJxwz6z
zl2j1Zmwv7VJEYeFkJO/+elzdQ2U11g/yoxpwrodpotrpASYYbrynLVyF6QP
n72aIKrChhqbqbtsejH7qlBC9b853hu82yXIQYrWTLs4ax7ec8YNf9bLwXZe
/NWrW/twc+dt9Tt11Dm/sPXn3PxGaKfNw/X9PSinh3omxLKw+/nm6Juz2CgK
Wr9qgwcL0y7HHrqi3wbH+zJxGo/5cL18/uCfbdT5nW9QyjxWDZ8lFnnDW9PA
KF1gp+4dhelrRC0Thl9BLyigkJZVA87hS8oWDVwUfYr83byCgwIPeesJKzvR
XRnrT99fC+bEo38y9vJRK+g7KQjlY26NrZfy90JElYw2m5NXi5TDwYHwFuBA
rHHqYVo7LB/OKC8dXYFTU6zLQ4ZqMbBHVKu3jTrXHUNUwsVOmDtW/L0fwsTp
wMyxadbdKAo80rOghwMdZ91JAzYcPF6+2WiARXH6x8P8i9+of37BQYNh1YMw
QcjkcrseJNn71rg/40HbQK5ufrAAtzU/bIvX7UHlr5CggUM8rAhWD3Na3ofo
nFPPV/tR8z19uTjQuxsPfNrm6l95jr95Dhoy8d34Vry0MeIAFzfkZ42wnJWE
zBNT+evTuZR+ddY1SnNw+mJdhWlFKiatai1eS+3D/5ir3tT8Lpy1kJoTtDAF
Yz6d6zFZyMGMwa+sJx9ZONW0WZe+JAYjNr3/fKqcDYFrcbv7Khb0WHN5XVp+
GEmuC8n13sr3TOcDfZAh95lJ7nNs9pVhKX0RlMlzg8lzP08KDYpxEeIDec+L
5D11to3mKpkIMSpf/F2D5Lv621qu12UKEU/mkEvmMDTvxWJzngh3yNyqydxi
XARDJ30D/5lzlJz3rpwZmXhI9rKC7EVhgcmWvxElsCN7ZJA9xo9/UDfJoA5s
sncG2bvPzEM0dm47NIhO6L5inRz+WhOQcJODR9ViXakVi3WlJVJZPso7E6uq
xDq0URTr0GOzocUWeg5kj4h1+4foNkDUvrXNtw30CrHOs/XFOndPWh78dxbV
R4kvfhFfaA09tqPpCMEmPgLxUUBJ3YbN0TmwIL5rJ757mG37eiirAbbEp/eI
T62CJ4wSUucCeS+xr98RX79xH6bnUTxUJTmwmeSAYKyZ4Pp5BgxJbmwiuSHl
6DttawWV9yRnTpOcURk0mHpHier9JJdAcsnmktrX+fF9SCA5VktyjJ22bYHc
mRqo/l/upYlz70SThfQYHfY/Ofm08Vft86V9qCW5GkZy1fHVjTwb2zqYkxw2
JDmsG8JXyO/mQILkdhjJbdpLz4INPimIITmvGiHOeTfta5dsajtwinCBTrjQ
nL71/a9xon84cm797WdrDjRDj3CHQbjjecRhzLkwiguEU6qEUz8WDxacz+6A
7P9wTferT6PWnRwYThBzMIFw0HKf8er0sl44Em7aE24672nVenqCCR/C2QTC
2QQXRUtd13wwhsRcziJcDtt6/XpSGcV3wnEFwnGXQM0xO75WUecAMffnEe6H
LBuWLp3XAx/SEyxIT0gs/6DRcqAKhqRXzCO9onR2fCe7tBt0bXEPCSY9xMbS
omvofgkCSG95T3pLQqTcKDUzDpR3invOdtJz6EvXR21yeg/5EHEvWkZ6EW/r
ohT+/k6Ykh5FJz1Kuz1S0SSmD/T7DmO7VrWBVsJuao+8CbZi9tzRCr2gH5J6
ZjwYhqTmV3H+JykdSie89JSsAlPpfUtyQRdcfcYfX3pZANP1y5NtNLig9czK
0Y4vQSRbuZl9uheNzod022h5CAtZED6G6pHHfpfMdTOi8mn/sW1rTnbBImLN
0AqdJoysVn77UciB4XK7p4tqqd6POgvTM31wt3u64faRLhhrqI+9a9wJD25B
y4vRLHxPMax7/oiJYpOu196Zj6F69VVgqkwOVONF/jeqmnHdduLp68lM7Avo
0po/lQuPJYn7LjykfMJ42S4rEQP6EbfjZsoVFM9VTT89KQDzHN/BxpeN/li5
m6JzvaDxvi3b8SccPv2ZZhrOtShKfHIzY3onpk874MHIo+Y8WBNgMpAA6cX7
L8TItOKe9FxFmdsC+FkdDGttp777+SMr5YdtaHJw1ubXUn6f5ms28ST1PkfG
1ejYM3F0hs7xuitCeIj2GoS+SINp1lrh/LxODHnNrsiREGG0VnZM+fwusEdM
mlofyULfgn2PDu8XIewjW3taTwI8tKIU16h04cNlvij1rgiStxwTvZWYcOv9
5li4rAurN2WYHXlEnccsN5y8sEGIAI8QfbtvLMg54t6+KyIMeudwE/xr0Ppq
yp3ztiwYm026HrFOhJTJ9ZJZzWzwCltb5W904vzI2X5NvULQTJxGbxp2hnl5
nHqOMXW+NOEFp1K97dSGGAlHWj1oH+R+Ymsblgy5TtpZKMB39oOzRuYc2ARH
HDxp2IRgb3cL/c98yMisnKik0gfD5o7ZDRtqMV+0/4xKVy8OtxmZzuitBP2U
6MCH4FJsPzJv8YqHPfiu5z22dm8nmG0Vm5/3ZuFi5l2Dnf3dOBffsW5KbQ90
iyMuLHTuw1x9Y8b4Xg7aIg99SBsVhxcRt7X2vOqFWux2uShKP317V9Lmj6iB
9Y0Xt/dO5sDz3djq48uZMDNdOVk7iZpbgfxK4cE2rAvqbmvSaMSGRaM1Fp/q
RuVUx3AX0zKklt9p+nigBPJvDhTfCRAieG7Lcvs6ET471W2prRBR/f7itIV9
6Ximu9CzfgwHorXVgUf9WChSNho/8swnnC6/EZ48vhbfA6973Kd6fMLnUXen
2OUiLiF60uLSEjDfpFeNFrQgcMKhfEEaHw0fnn6SnNcK3pxLZR9bCxC+LOic
5i8qV/ga4YYyXZDtSy73/diOuB8RTX2PqX7YfzOgQ5qLhe+UTtmPZ6MyZtKq
UttuHFqs430krhsTig+n5u3ngi1MOu6EXqypmcM6cZiHrOPOwqubu9GY5mBz
oEKAXc82mo9w4MHp4EbN+H1UPnkmPf0gpHqx3Tj2lzE8SD4eO9GnkQvpRWsZ
l0J8MCui4lX20W4cC7jKpf3i4L1f5IU1/lHICrubf/MNF/T0azmN39lIHeE8
SyIgHkOjQ+8HUT/HqAyNnzHcRemifMjvSyy8WCWs9xc4OPxL2q9bpwtv9Y74
PZv/En/54xacXczBOzP9BwdLWIiq3O524Iz3P9ffpYhuG7n3wZvc5yi5z7LS
t/PcY0UYJs/NJ89VnLwjoV5V9M97cq/zsq9LijCHfJc9+a7UJy9NvS6KYEzm
IEvmEP3991rZK33/zG2qQ4OlpNpzrCRzTiNz9tl4d1j29lvsI3uRIXsJMdXe
rTmyEAyyx5lkj2cja+/b6VThAdl7s0i89xT5NHPV2npkEp3YE520PS2eGj6m
GXuIrpSJrv6mXxGZq+UgjejwxFOxDrePuv5s1TE+iohulxiJddt/eYkr1HqQ
Q3SufEGs83NzfLOmjhRCd5rYFzIVYl8YJsik5qxJRgLx0WHioxevB169nF+F
c8R3AcR3CWd8q/yDqO8gPl1MfKq2nX391HUuzIivNYmvIzlj/PIpztHbxTlA
Jznw3OhG/9P4IlSS3NhLcsPcvYu3y7QNNiRnNEnOnPFtb+m/2w1nkkvPSS4x
cmcE6l8NhjzJseUkxyx0mCpX7OpgQXJPhuSeQ+vr34V//83Jp5MHjkuV96GR
5KoRyVVe5Y4vd8zq/8lhhm6o13xtSn8kt3VJbvvs/fjSMj8VdJLzWSTn9w3N
eMaq7oCAcOEL4YKL3ZHAVpYQloQjfwlHDofdjjyc0PQPd67f1FaOofrK/3Iq
qqTtRNOCDsgTrt0nXHOOvHntvls+GgkH5xAOqt3gaOzqFEI+TsxNBuEmW+pF
c08eB462Ys7aEM5KTFswG231CCNcjiFc3h4xadkHiEC7Jua4LuH4i5VNc0cE
tiOBcJ+WKub+N07C3Tnb++BBeoI36Qmlr+J7+BfbkEB6hQzpFY1Tkr1cPgpR
SXqIA+kh34yX7/dc0AQG6S1xpLfIvehOLAIftMninnOT9JyAC4GBBq9qQLMV
96IM0ou+/Zq/ONOV6t/epEeVinuUY+zY4/z0QhjeqjhydUoO1O06RuZ1iLDH
pnO5UjF1jtCfapMQzwUrM0XqL08AGwn3pJYfSbA4PPvSjpxqePCUHbNbe+Gx
ThTJnMVEmGxHRoqBAHukmHvMUniwfVUxyyiP0tHq/Uuv7W6HDV2q5dMxLnz2
mNI6W5thqODcvmRfAq4lr73izeuCy4VhI4MlAoTZS+lmZrCxOnGKRPKCTgh0
B9xOa9eD2aTI3be8FK6+G/Kuzm9F5OPjdOltvZDnRS8xb6RyzyJa/alMHTxN
9t4/Mq0Wg7ab+9pKqF51cs8vfc1iWHu1BHS874XlyaZfl6f/9/sQCLOTprwA
rSXjV0xYMxq9TU0s6moh6ZFc9XgE1eszPX9/LsqGwdtneo88+/BwqMun7guV
06ndmZGSVE8Yce9F8gMRnpl+Zo504iDJxNnA0Irya1SB4crHCUiuzIo+tLwb
o3bm9Nt+oM6dNtP0WA8/Iv/vmTEKH3mYtEW98oH/BxhqHW60G1uB66EKI/kN
PfilsDJuWNiFjJesxo9vqrGkJa3YxZM6DxkajbIvTEHA/bFLvobX4smuwpE/
vlM5Of3g19+7mOi3VrSozaiDjuakQ3uX8ZFRMRjPsRbCWf8LRzSeyuOa5fVf
jfnUebz19inbGtiEFXc8on6+qm1H0Gbq56XvVeoOPGHDx1jiyZjkWnysidin
M9SLIpu/bka5fbCRa7q+/ns1XKYMX+kL7EVEmpSsMKcWPieMfiacrwS/Um1R
hWwvnli8T3zszAZzbHLp72UlWKqq5upk3oOHt2T6VOSo/Miwq/jyKgfX3hu8
DrXgocjayP6ZfyloZ3dJP3SMAd/b4XisHMUjnklKgnsH2Iy0K6kBVI9c0/vr
mTMH/V8Gv27ldUN2azHjd3Uvfrprqzlc6cKr062yFn59WJh1Ky3bnouPL34+
NR1m4vAaBUVV2U947654w3ZTJ6zZk5vVXrXAbXkYe1pNM9R73WUH6urxcOXi
Dbvu1ODlo4DkQ45dKFpgpHe5PRd+at8UvgfmIKXsmnZYBA8ZqYJ7cxcIsV56
gVWLpBDZOfna+V4CHNl4ftduNhtGaycPaexloVrW0cSvVIieWbyXGWrNkO9+
bhta/g7Pmq0+XInoxq8bMtELY+tg9c09qX2PEI6d444dkqPOiZO+7bp/pQ16
l28ZTy8phdbZbVrfsnrwca/5Xs9AFr79vH3BwKQNYQUp5lfW8zDtrtI+E30O
dZ5XWbSOxcIolwzL1OM9aCnetn9lNRcpB1tM0zQ5KHUbq9i7luq5FqP2HQns
Rs4zHSurtVx8c73GXaIuBH9naPz0T92Y+Jvz/qobFx/OfLdzqxXhycL5mS1P
unHY4cCvbn0u3iQtnqm96iyed3Z+vje3G8wDtOjMFxzoP0rqOPItFKUG02o7
n3Jhqrw+kMFkY4eVsm/qqUjczXs78dQkLux3TvMPnsXGAuf5gRzhS2hf3nSw
OJYD//XCH2PvdiHsx7O4a3ah6Ll0rukU1ZeYkt8NTlN9Sas/v3pamvs/1/UF
KT7zX/ZBh9znKbnPWvl7MvkSff8819XjdOSs5yJUkvfcQt7zqYsoUjtQhHDy
XSzyXXYnR3buo/X9M4ewiBz/+2196CFzkydz2yYRSH+b/uKfOXsoy5bS36eh
huzlNdlLj6AqftdgLpTIHtlkj5OVZ8Qu3F2Ct2TvJWTvjhyNMoXecmgQnbCd
xTqZvOadwRLZYnQSXWkRXZ244xppl8WH3GyxDp15Yh2am22/IZ1eiatEtyeI
bk0Dr5fuD0pEDdH5EaJz+pem3e2BmdBdKPbFAPGFa7GE5vXTZTAnPoolPvrS
WB6gN7MJzcR3rsR3Vtz4ceeesKBHfFpBfKp39tfQD70eKBJf/ya+XvaMXTTM
v4tvJAfkSA4YTpVTtllCnctIbvSR3Ki71/rylR7FI5Iz10nOCI73vVRu5v2T
S6rFu8YvnRGPMJJjfSTH5A/eCq6Va4A9yT1XknuVasOXx4Ry4EFyMp/kJC1u
Kz27/RocSa7WklzV7T3gPam6/p8cNvadkF28mQseyW1tktv0W6vo1bqp/+R8
1se1UyQsOhBGuLCAcEElw/+vgp0QWoQjVwlHnJp9wyXVGmFBuJNNuHP3+9Bo
uXECqBJOvSacCra6OkPnRivmEa75EK4NcmZI/o3MxR7CwTDCQfrWMbfUF4VC
/oGYmwqEm5VFWxidJgL4Ec7Sm8ScLX845+breBY8joq57Ey4HFDhG3xD4iNM
CcfVCcfH7N++agv1HJtmMfc9CPebX+/+05ZZAAbpCYakJ8Qdua1fzGGDpiju
FR6kV2hZqfDGub+H1hpxDzlBesi36pbrD3xZ4JHeYk96C817dFfbp3CEkZ7j
TXqO1uU5SQZZHVAlvYhNelEA4j0lngth6CbuUVqkRzVCP6DmdCPVkwPViqt6
MNJllWHCQSHYXuHKhT8oHpz3DP9+vhG1R2oVqrfyMXgRvkW61P1WuNpsFImQ
kCPTaa1O6dOzwZLXXgf7k/6tIys7ITdNS2pNJ5Vn7Ws8d+fxYNhtLhH3Ixe6
ahsdP56hvn9sjK13dBn13fE6V/260R9WX/HqYyeenv9WGa7EhYWZ1+ho61oo
WunOlAtpA2Pn9NaEg3koSsh4w9spgPLZi6PUqR4sP/lo3PKSLvSHJB5WfdEC
XW7o47N1ZUCiR40/MwOVFuqGQgPq/D1vZqWsewq0OryT7BdS3IzirPixuBkd
fKuBhNkC+DQYyi12yMPIcdedhrR6kcP3eFw1uwsyjiua9Sm92Mc8/PW1tAFe
ZzW/2+jVgvapcvK1Wj5aj7mtFu6geqNoxTaDmkasDt6+ZO9UEeY5M78vM+ZC
5nLLjTwuE/OcBrL/HGPjrv/NU3aL+VglMg+8FNMFjuOkQ0mjCvGQM3brw4PU
e1ZBbu9eDmqXsKMip3NA832cOvXDA6w/XOJ58zQXCXNblWr+vgPt9Fb5uRVx
SJte5TtlVjfk5uWr59dTebJ67uNeYRpueNCOzH/cDY8xfsnvwoRgfvfWi7fO
RPPJZ+NvN3bDIjBy1fwnVL/SXeufPpgFU5WCRws/d0N7uFh/yi02GGaTkFCX
idOzNVtEHd345osCB3of6NkSOifK0vFFKs3s1otuVCr/fB7QQvXXo+se8CMT
4LRacoyTNuXTt/fVDlR0gX5To2L5pQBoNphZ33vAxbv6v9kLqLwOYY1vmvhb
hDVF7ciP5oBR3Xjy8VAhzE8f7bmVLkDs7NJ9LdfYULN03a4X0Yr62rr7vj09
GCgc++C/34fvK3tGGnsOFwpTMzULPLlQ/3nQJGF8J4JjFNkTzwpRvuDmzWpv
FixuyITmr2/DU3XXuwsLGNBdVKi3gtmCVBk127Q5DeDVr8nYHVULduDSS11b
qiDV+SfJKbwcC4uZRccdOuBYOjNoXlIqojumfjOvofQjeJlfpsQG+/zMhadX
CGB0VdLp83c+NqpIZrdSeg9K0yzI9eBg7n2t0hU7uqBp2WV2oJT6bt1Hhmd/
dKDyh0B3D3Vea76/SLVwVSc63kXKuW5pheCjT+oav0IkmDoJpVSLsOd+8076
WCa0Fts1nqiqQZ6VzuKlfCFoSYwG63QWmj98qLke0wbVOQGinQZ8mAdX5mU2
UucRm16H/RtZMK8+uuyNHh8bzCyO8sBFVkNf1ecDbORPmCz0NRJgdv+c18kS
3VCTav65jOop8+Wi02QfC9GhLrwRtqkbr248rl7RwMEm6y070ppFiNoZt3Xs
qm74nXnXGJHNQWLRxAJtTh8Sz6g7uYu4ODCm+M1WKw5EYxwXvQx8hNdnPviq
3+Viv/2cFWOL2TCn50rJTg1F1eYLZ7vncmF1VCH5wio2PvS6+18KCcUKg67s
N/UcBPxQWbGIOi8q/Xj8vVLCH51+C37qUPrI1Qxjwb0Lo56Ej3qdQ//neuTZ
z3t//vfn8/3PfUwsNjk3behDNXmuNXnu66J162RH9SGavKcNec8//JzQJVL/
flfK9Yj6ZUf6/plD5baVglWfTqCTzO0NmduVK1M/PbV8AVUy59lkzp+9X5k9
3ZX8z17Gn9vg78jPxnqyxyVkj/Vp9jq7xuVhMFG89/dk77bnQ9dHSeZhG9GJ
LNGJ+8bNTy5SPmwkuiolukoUvpD486MH1USHtEGxDqeyoz6c9WpDEtHtBqLb
h7tWnVhnV40/ROc2ROf0wq+3TfSqYVEm9sVlptgXCfNTLtpWN0D6qdhHhsRH
LkEPpn0yZeIU8V0z8V2YImtODbW/VuLTw8Sn/AGb/RfK+ZhNfL2U+Ppe0J4O
m0spYJIc+EVywFon95FORg2sSW4kk9yQqs+p74ztRATJGQOSM6c/jXm5ZwTF
+1viXFpGckl10wj5pLQ02JAccyY5JnGfnbbuv////39yL+P8qPjlCv/9e+Kc
PE9ykuY1zXEL+8E/ucqUOuVW9rP+nxz2SdY4ItDmgkFy+xbJba3Ug0cWDiWB
cUqc86kk57VSLsVd2t5O8V/MBRAuOF9RezY6TPAPRwrCVsYl5dfBhXBHi3BH
MeKd3URQ/ZFw6rezmFM2ypPcRi6j9kO4tpJwTRX1Mpv3xMGYcPAc4aBWXVaY
VBwDeoSbnYSbymsX+kv/yoPNazFn/xDOXis5k+WkxofiNjGX+wmXZxuxnmpG
N8E6VMxxZ8JxxV7v0cIuAZiE+2r/x/3qFeEBJxvhQ3qCGukJb94MFk1U58OR
9Io/pFeYTpl8IKOa4hRP3EOMSA+59rv2mMpjHsJIb1EmvcXijnxL4JdSMEjP
SSM9x2+NX909eQ4sSC9qJL2IYSpxMXPVeySQHiVNelTJqrjzKm4s2Kx5o9R+
uR3yRZnfswr5oE049uFheilofk+mODS9QbWU3aVDz3qoXj55aqMixfmMUTIO
jmxoDn1y3Lub4rWP08z1QXmgHXJxf+BZhtbMymbVL2xot0Q6Pf7IQlRCq5nZ
gV7wpmmMGNrDguoDGW2nI5Hozy0Vlrs0QzZ9/IW00R1IWXf3fYKoA4PfXzYm
1PVh8ONpHjO8EbbFx4LgLsJgCC3X/lsn9CbuG/5gWoWCml1aCzc1g5m4o9Ty
4Tv0a09MZ/5kQCfGmsnr50N19Afbistd8Dw+MfR1nAg5kq9zHq2oB0OxRy70
VRTKtA8lLvjMxTL/61pz5/NhIZm1Y7NNA1b4pEv8FtTAJ65YYcmLVtT+vuso
Q/WdGSfW376fyIJ699D56FE8MNykspbuj8HQr1TXZUHREMyWvevinw+3r2fM
a20bUfFL0SntTS0cNsU2eJ/nQWc4cs+ySSyUz4jNnxjdCu+WZ+dY5pVwUfTY
WpvDQVDe9SnugUx4Oi0z0HvJheJCWsoCPR4KY9WFsTIs0CqqHCeE5eBatNIk
+z892P3owKzvkl3ghaTInLrfCev4/rWy1LkofNLtzuSMLjgZ3jnkoiJC+ZQT
82+oC+D9VLZx9nI2LGKSlR4uos5FX1afe5sqwI668xk9V9lIMB9jFBzKxuDJ
L6N/8wVIH9W9+NJTNpwnh4wNOd0HlYeWuYcKBOh2PLsqwZ0N41rj2PsWNSg9
JDHXQV8A3yrJC0YGbMjv45tc2t4Fd0OjX+GmlJ5uSwVpZnZBst9ql80ZAWZY
v4o92NGDl587vlYNsKCRp797klQ+2neLLmz/2Y3CmX+MvNmdeNiiEr1WqRlb
VAuUSr05UHjbyFO6QvXI6dKpT+6y4aGoFjI68L9fFzo7KmZ7G9bbmpx2vcJH
QFjWzv1z2xHLXjs2NLkR7KCBLBetWFjaGvnYf6nDn26Fx/nt1Tio+/3pz/4y
9C++N2sOirF8T7FKg0whwrz7pTzDGmHzxumJ3MkXsBqqZ69WfoGHIRJbpKSY
mDr3W9HiWD7SNygv4+3h49e0dE/VKSxsrXsZUbWXC+ZI++W/qfnwF7x9WfGo
E5vnHrX/sYqF3Nm7p4wbakGYZVzXlufNsN+/ffbTp0z41HWKro+sx7urWhKi
yCK0TrUuTLPrRNYDHfdQySbsvL/lsP+Mq4g4+WLlItUuRN3Z56cwvgPyKewF
2fZCaLj+rJuaxcYo6Cltns6CydWNButyBBBKzO07381BvZH+uXftXVi6qXSZ
1WIh5PX8n7oHcRHzQrXxdScbea8nKtOlRHh/0PhEci8Xl+Q+Sf604GCK3K/n
w99E0Dgrx9tI9Y3V00s1fu3kQK2yVc47j+obQ/k3TzO4kNc6ppw5iYOpanuC
2RPd8QtOEvGXuVj+ax0vJJGN/Z5j2YLffhh5896RmCXUz/+OHumzgY3+YDNB
xrEn+HN5vIfgKwdF79IOCvu7wP/1abF6uA9+Scio1FVz4H5pjaUXowvMLUOn
3h53xjC57kGuHxo7775GRd8/91EN833PdemDBHmuAnmucvQz3RLzvn/eMzW8
I0Deug+v/+e7HP29Tye86vtnDh59/r6Dutf+mVv4jK3MFQlh/8xZ3vbjtT93
4sEne6khe/GMfHZ8y+M0zCV7HDQQ7/HLpJRx3IG3CCF79yN7Nz3YnKD0LgUN
RCdviE4aDq+pyv3YhxNEVwFEV1eZX68Z1/fAiuiQRXQYXKfubR3PwhGi22Gi
23kx91NsGluxmOi8jOhcschX8L6sBYxosS/e/BL7Yij3+5HPnzugpSn20XHi
o7wMKdfM+C64Et+p8MS+Gz5rFJhzi4cM4tM84tN9kVqWgxoiPCS+/kF8vekM
10xOIRd7SA5MITnwPb/vcXVvPbgkN0pJbux4eHZhxzcW1EjORJGcsXiQ8Hr1
n154kFwaQXKpn266/rdb1r85Zrrg3IS9TVAiuddFck/iyf4Fb3dx8Y3kZArJ
SZqSvpeB+UP4kVzdTnJ18JjwSiGjHrUkh31JDssujPym08vBdpLboSS3VX/d
696tGwcXkvO7SM7Tc5/0zue0QoJw4T3hwgmvT+G//PlwIBzxIxzZPjno1p3U
aswm3Ekl3KErJBZ4N3XDkHAqhHDqQo3Z8/yl5VC9KebatZ9irrn8cN0/q7kX
HwgH/xwXc9DnqKLW9s8VYBJuhhJuNqoHXeTFdcJwqpizg4SzGhEjDE4eyYMt
4XI44XK/66KZOhw2VFPEHDddIea4icWza/v0GFAOFXPfgnB/g1Jt94lmFhR/
iHuCRIG4J4SderRRMvgVbPLEvUKN9Ipj7e/CXkkzqZ4m7iFfSA+x2PUhqyhQ
BCbpLVzSW7TMkpTqnJpRRHrOMtJzlFtnZ2hOoXj2RNyLGkkvsv1Q+WlrfQ0q
SY+aSnqUrOs8i2nVPNB35ihsUsxFbYvILEWxF7QV3poutsEw7e7mnh/TjZFr
J59xKu+GnmJakGQ/levHgsvKK2pg5vvm1rNTHNhfeiZdxxWhKFZvyHtQAK9X
Zcn9XSzQvUYVXtjeAvuLmh03FZgIvqOTfWsuE8Pz0syKj1HnoQtFceOPZ8D1
57R6qTXNcB7xdlZvYg2kPYZ3FL/nIMZ0yYwOmxo8EQzMuLqEes8FKh0vOyog
nWLyd0VtATzurpw1rawY9kw97x+XesCQs85g1wTj8dKV+rKv2WCqnM6WeFMD
1vCVOb8YvTBpkvXeb5mGSmWztz4GPGyrcmgfK8tCsukn5zOJTNCvOialCVOh
5aLWvEQmFR0VPmMLfYVw2e2k9HFlETzSH5wOOVuH1vETpmRFNCL/uFqZ4l9K
/3KKy7p292C7YqPt+krqPuOTElN/5WCVZbJHZg0DrNDUlnQbIZycvvsdCKqF
gdLOBRZtVZi09kPj8dI62D78MaAl2YpNsa7H5hk0oDj2Ab/ajgfJOXdCM0OY
iN9h/un0x2ZUOuCw1L0CmDDzFwf6seCoFTtKcXMbtk+ZZTzsRPWMdrrZ+mls
MHIPfVbS7kBR8pXFKsUitJ7oOLrdjIP3gZZ9CtuZKJq+zFpqTh1Shj3VE9S5
8I8Zu9+9ntJftucogYAN6Y8vGg6mUNwR0A++VO+ErfYV/a93++BaSBvB4nJx
45rE0TkmnaDT6O2dc2rQ/ifYvCqdi0b3dcKohZ0IsdhhFTeqC1l79d2OzaB+
/viFhLPUXKxatqgUsPjQrWR/sFSgckChcvJhLSbY4Vv99QsYoG2YHmjq1YV7
RcnH3W61Y3ZxBlOe1YBrEdWy6907oXEifJv91xasSw7fHTqzC6XRkguWDLah
KOSvVKdfI86tVbIKSe+BpffW2QcbG2CqunVxsnEtFE6E7K2L6UN+5trjaUmV
4N9v5L6JKYMG7/3cFeF56B+bsDlqVQ4C7caYqC1g4JrbTcNQj2oIuZ4n3nP6
cG3NpNBk6q/psfNdNH824szT/hl0az7uDn5WOz7Yi6ER769wTFsxM272DEfl
bjz4xpnt78OB/jPutcnrW7Bo67px5p1d2JN9VuNCPhOquWFpP6bWoXGX1ot+
e6p35V+fvyikFdpJ09cZDBag+l3gBccgFv58HFhWYt4G5Ssjn05/HInXx1/5
W7zswka3C/Y7jzARsWSbhEKnCLecvOX2lLLxt7V3xf9j602jqXz/932lEUmz
SElCSJJKpFOoVCplyFtCGgyZiqJIlFAUkUISpbkMJYmMSaaQeR72ZNs2m5I0
/+/P2tfvyXf/H7XWnXUP13W+juO8nmCxjob1ZuYrXIYHceQpNzzoLROPzb30
LW9T59ZJaQHvxgaxYOOFDxPMWehzUO7Uofg/s+qflfCpIcxqjbux/z4LYUJW
OtlUDgqNONGKUjwcqVJkKVF56DgconFqEhMaXzMe7HvNw+kR1pvX8Sxo1iUo
XuhhQJ99x8jB6RSi8kwOHnamzoHF80/U3mbgy3FLmxD3MIRVlT3+RJ3X/yb/
Z/5tNwOWIbo342QjMKOu2t5tGgt1jWETSsQZ4Cw9If/oRQiY9g6nr/OYENIT
2R/NoUPjYVPpu9ATAtdf7TsZPNTBE7iPj47b4e3JPIHnMqY/ijt3kSfwnvsS
g2mbo3jwId+lRb5LvcRt9cpGwXXoWnMx2uLZBYF1M2ZnXZiRmyCwzioQNqlI
e4xDZF9SyL7Yd4f8lN6RjvNkH3+QfdxsY/460S+d8hB/3/XIvhtOHszpHkrG
J5KTIZIT/RZbhdSbQ2gjuSojuToSreU+vJkDDZLDYySHPYlp8kEnmVAjuU0h
ubXOCNfzOkhDIMl5Esm58D9Z/avtvZjYx5+LaDIX61Zwix0vUr1AjD9H78kc
VeXl26bnUucNMneS1/hzN3lrZJbGggE4kjndT+Z0XeBRyScVQWCQuW4hc33z
58NHwUvLEUw4sJ5w4HlctnFRYSumEG5EE25cuH0/4ju1/xsJZ24SzsguUI33
qB5AIeFSEOGSbHTJtT6vQrQRjjURjonZ/md6f7xVgHuMfwUSM92oPJXwObmS
cFJoRCNZ3eAqUglXowlXY4O444oKTWgiHM4lHDYeYZn6uTFBI9zOItwWKjVU
0nW7D0PCeVfC+bJPed5T57djXI7vhTvEC37yHoqbDnJgTDyiTTySsr1ob6Zd
NQyJd6SId17aJs2pXEjND/FUuSnfU3UTTeU2/8xECvHaKuK1K8171p2y7UIa
8eDcbL4HE057f01+QfVTf743L/ryvTntwpn7H0+8gsRCvmdPEc/KuzuJL73T
BQ8Zvpcn/OF7We7opt1eaweRSTyuN5vv8YN3rl/qe9EEWWW+9/WI9wNlXq5/
G94PVdITCklP0NqtFCSqX4MAb36vSCC9Iuj8Mp6IIgtJpIc8Jj1E4qhiY9uR
Ygil8nvLTdJbxjf4L/pgQYMe6TkmpOcUhimdTotKhgfpRdNILxIak3/5mzp3
J1nwe1QX6VFJ18LUOhZR59Cd91dwH/aDNeCx+pk+G5xYibUWSRyoL2XMSYpt
ATvvaZ1CNxPpoYF366jnRRq0ZCWY86Bseoh20IYOLZNdJ5O+sZGuvKTlZAwN
LLvqI/NfUe+ft2Oa0FTqe4OfTF197iMefywuHLvVDjF3U46/JxNVzzdP/DW9
H7LL975iyDeicLi2InVRPpTWHfKf2N1M9fire7bfrQTtpXb5665eNLhafFzg
PAR1Q6NrGr8y8dLt5i0N6ryWzWz+3bGrByUWgZP2nxiE26LJO2o+tUFPQlRR
i/YA+gZaUX7tLLD/jEjOusLBvUMvT2vo90Jh/fwTa263Y33guir9jnKMbPqW
sq2ah0Vrj3bIUv9afRblbU3voM7Z1UqNQTT4xMWua/3bAYmnYnufj5RANfJh
186uPrxaemEJElm4bBZX+6aUA+Yb+yNbJ/NQXtqScmACD7/pn8449qYjKaYg
qet0Pm6cuzLnQdk7hHD/7Q0yb4flp27RlYlVuCh8YAatvhzLlz9zPKLPQSHv
Y8aYTT0OXZvwRpNbC7shoxl2QlXQu5pnoOvZDB/nqafbftaDkTBn6r9LdHR4
rm+Ib2tD7iSVoVMuTbDUtiuNuk+tz4Ol+aKvOvHrxfsFb4+2YPiXl6O8bxPm
/im09fvXjXLX/1SrvrQidnDewO3jTFTY9izY3taDTf6n/iXPaYeLsuHQs1aK
o/ujgtfZ9cLqxHQtxeJ2aB2Z8l+eZT1iWSqFT0J64V8ivnntt3Y0TT/l8YFB
g8Lkkfh123qR8ETl6pvkdtDcjT02XRqAmuObvoc3e2C+9WFleU4bpnX4nvII
eAeh9D/3u7u6UCV/cylWt2KM0XVrjVgTSsZ7nyyPb8f0b63fGr804WObx4F4
Zi+Mq1dG7jzQjKIxtsS+/nrI5b1oDVNiwydh5OXt5XWImRsbU/C1BmWL3m0O
Kx6EUKxa2jXxMnzPWyBzUrQUYtXnjQIHM5F071Tm1guZeO20dMH1nFfwC+o7
E7e/Agt26m073jKEjfKbj43WDaGy32dnN7MO64e7gvvnDcDvydzXMk852LH2
3qbu8ka0aUocn/SkDzoiN9ICLFjIS+rzW7SiETfuBrq8msOE7yG7C/b3aTiz
xPOEd3stvrNte//k0XEjZssKt7AehJ8ZPCR5qgR9iysdG+Lo6Mk+WBtv0oPM
jSafPkumYYVp2VRJVQacU9x8fur0onTFypWjn3moiq1LkfzGAOudVXA51bO/
vHj0Or+W6l05olfpj5kwTJp5tPYCHYt+RFmmxQ/hiv4TxgjF/xLxqMVtugx4
/TXNG28aQqDtw10vPaj+EJiSVJvCQN7X4h4TAx4mKg7oyYdSfcPktLhaKQMd
iYH57Fc8HK3vOBjrx0KKbEvb+9cMbLFP/i180Rpsa25GkzV1/5iS8pOhDFyc
ethTWzsQUR60iN51LNz5I5L2dS8DrYnXxZ4oXoJdirPl2fksrF0ToCa/jIFD
vGl9mREBMIv38Jo9iYXftxbcc5vGgER96smuSkeB66p7ry7V7OPhELnPOnIf
g47DqkV5PIHnXnr76cChZzxwyHt+JO9J89SYvyBX8LtM0ib4548IrkPI9ljb
2skXBdbNOTS5jx5zC6FknYvJOvdKFip7Rd5DINkXPbIvBm61p0XMH6KC7CON
7OMH2UsP5o7egxLZdyey7++ljjv1T/YGi+Ski+RE9bcy/aT4EH6SXMWRXEUl
B/W9pXEQS3IYSHKo7bF1m2FUH5gktztIbhNXhRedoDhgSHIeTnJux0y1k5Bm
QpnMhQmZC/kwAyuJqywI3efPkbgzf44GFgcnL3SmfEnmbmU+f+7Mpgw37bEf
RCSZ01wyp15SS748GMiAJZnrCjLXFaLb42oPUnNMOCBGOJC0Y/H2Ss0OiBFu
NBBuJF6V0GT9YECDcMaKcOZWkMnIiDsXcoRLtwiXevSqN00ZKMINwjEfwrEb
3HlLTq1rgwbhnjnh3qRqP9NkUxbKCCc3Ek4KdXE6pyl7Qoxw9T3hqsTyhhVl
So0CHNa88KHStZiBKsLtdMJtIa8Rh93SIdhIOH+KcD68tGVkwmgLMokXLIkX
bpa8+N33jw094hF34hFVF9ODbmOlGL7B986YH987Q9OFmnXNaaggnnpBPPXQ
oHS4jj4IaeK1BOK14hfxebLMEky8w/dgNvGgVeO7gRvvOZgFvjd1iTdtF5wN
7XjUiw7i2RPEs2zbL8a80H4IzeN72YF4ed90rYTc5g+YxOJ7nEY8Lnzyj5Vk
HA2T3PjeTyfeF8ou08w8dg4BpCdYkp7gE3/F9YZ+B6a94PcKPdIrZGkPFJw3
cqEXwu8hhaSH2Mn9pzJxYSN6SG8ZJr1l3GTwD6he+Jj0HHXSc8oKKtb5r6iC
HelFXNKLqnWQPl+S6hekR30hPcrnTp7KI/8PSJdq9NTX7YS/pcfmSmMW+nFn
3j9nBoRs63bp7HqEnHUvTPMr6PCYqTU3zrIAJt63J1TmMDBtsUm0PNXzhXwP
b1CUov6/pC1V9EwthMSSvpdmdSDgueYx25+p8JL0Vi0sH0DJtm603WhCz8Yn
J+codkP96Vutr55dkNWNn+ThXQNVL8tnJgWD6Ck3l9jo9gqyp7L/SqUUIGXW
mWnKus34vfYn/dpuJjoX4qKTHQ/DqnqtRqr9sLvTEKSgXYMnJt33ZTv7IRmd
Pz5LuRzXtnP3bRyi+rd09r1pf2nwWL1E73MyDUo6SZcjcpvh9zN27ctVDai6
YSento3qz+yKjaqPh2DeMDWnIGkItOXznnDq8sCdIfFlyIdF8ez18avpDMha
rvxCe5OKiNG2X9vU+qDakJRelcqE5ZVyrh7lnSg9qaoI6rwn0dbRcjCEg68G
WrOZCz5i4+lnv/1sh5C4cEVVsNkQgoJE9XIUeiD07ZW/y79kvA/aILIzPRne
E+akGszloifSbG/YyVxYDbkJV87OQZWemcRSWi2UMm27nKo+4K5Sy53FS0sg
ItH74+NbBmLfPJU3n/IJV2wuvGEqVUAozm1b9soY2F29kKKQ+xm+q5ecOrmr
BuPXpFJ81rSgh/P5yZKLDfA1yrMdq/iMst3s3JhFLLxTHV7sU9qEX5y3wlzf
eghlacmkVQeD7fUhX8G1BZYPL6W+S2+AMe+AqNeXehidSN0Wb9uK5uWu6ka2
jUjcXenm/Z0GrxzNIxMvtyK688FUtl8jds/1TQqlzsMSc+yNo1tacNL+8vVd
fxugd+nh703j2YjMyW0w1G9GU+a3Fz+aqN42J2ngv9gGLKq1kj9f0IBvZ8Z3
J45/xruuvrazXT3wuDnxgYLJZ+hqH/M88aAas5+sHdpVy8Lj6SLii36U402Z
fX71g4+YonPCZvZ8LoRUXTuljxTAPubbjz0JedAcPTq2yj8astPNV56xS0LQ
wnPvPy9OQqGL3RMpsSKELm1oFOUO4rBJ3KI/rYP4fWXZrH9+lfiYpxuwoIGD
PT6TzX4d4OCF4jzfaq9a7L3s8SpblA3ZRd3zuW2Ut89dvFO9pxaNBs1brAaZ
MNnmWry/j455BXulr3ZUgGZgIZ+azcDuOV8eCEvS4BbjYbfwRRE2hFW++3aE
gacOZRkZMb2I0NsswihLx4VddMOdjxgQnafFWPynF+m+0qtTLoZCPUbrsacS
E+FrmkwaimmY29T9ievGg7//n/sSzyhPvo/YOjWYjuKtM7I3SPGgqtNos0GC
haGH3afnSDGQG/RqkoQyDwVtnwp27mYhS1PlXoInA2H9L3ozj/MgGqfaUuLA
wo4XVUJzYhlw1T0+aUMOD9+amsKVjlI/fzl/t+UNBjy5S0Ve/+EhZGDG9t17
WagIX1006M2A5Xh9/68rXtiTPvfU3PWUv1MP/du4jwGz/OybD0T8oF6yb13b
EhZijxrKZ6kz0JA6cZ5Zzmno3XBae3I2C+Hbb5gayzBwv3G0s1jnkMB1ZVE5
3ZIhnsB97v1s3B9WxxN47sVrdZmcj4LvKWGpOo3TJPhdzScShlP3awisw88U
dWb+iUCBdcut1q84fDkKymSduWSd3zhGOP3LixfYlzbF/X/LshOgSvYxmOyj
ke5kw2NyUQgk+y5C9l0iSsd95A5PICfs5jf+dNYgGCRXJiRXt9voj97eGEAH
yaEVyeG6SJW3HUb9sCS5VSa5ldy8KUOypQ91JOfWJOdKLqeKYz/0IZrMhTuZ
i10yb3QKZftRSOZoiRR/jloYj8x/vBuAB5m7D2TuPGuNA8SP8FBG5vQzmVNL
pdNirnsKEETm+iCZa6HUG8uO2NZBiXDgL+HAa331HSIGXYgl3Ogg3DAeWyVS
/4opwJmTq47LjvdwBbiknjbHv/NOMYwJx9oIx8btneRmzWoT4J7XtH0/P0wR
5KSRTJjBhxQeWghXTxGuptuvCm0orYcJ4bA94XBm9pcXBiw6HAm37Qm3zcJK
dKMseJAknL9OOG/YoyvmEd6EAOIFYeKFwu5Gb961PtiN8j2ygngk9JtDBO1g
ATSId6KJd+y/s8+HvezGJeKpCcRTarwTuVO0ObhMvCZPvJbrfOdTyaqLGCUe
/EI8eFq5szZw7hBaiTd9iTd3HOh8em8LF5nEszTi2Q6rX9xrpa9RRbz8gng5
6U7R+YPv2iCRyPf4O+Jx1wO1gb6v+yG7ju99DvF+kvG/+I7dlUiq4PcEO9IT
dni//+V8iPIg6RV2pFcE2G18cHbySzwnPaSH9JCqQ8+2nnbpQS3pLZKkt3i9
TphuJfG/39vB7zlipOeYiGy4mlHZCiE7fi/KJ71I+a9PJ5fytaw0v0eFkh6V
EvChYYJyMwISxDcv3pSLMrGAkekrGbg760fylugOOJp5+X0MZKGhcHO0V1gv
cgLPLVwxnTq3uJ9bUK1B9RJ20SujmE68K3H/kpDdhp6jYwp6XwZhXdZsKZLe
jCNvKl5c0ObC5MDgx57mXtwQ+9v1RovKm8qr36Z7GpFkWvf289VSKPVG3Qkd
fA/54/VT0+v7kO7y9/HxTVTPLRKudNGLh+VqkQKJ3g9oqXOasXp7K9IXM1Vn
fx2gruubD76mwcJz5KJkPw/xk9rXfJJjYXRh59QwZx6OLept9Tekw2hmyKTl
yl1Q19TT7fFqgtFnx/Smb1VY39Ih4ildAeUjPGMLcybom+PY92YO4s9wV0Qt
hzrnFDlVemlwIVEyyrvxiwUfS4X8C45MfH9cMUG+Zwh3Ut7MVB1nIstPefri
73QMmrh12Xc/hEXYifFF1Llwt5i4lzfF9W95nrpi+2ox9LiHLf2mH5Mi5hfU
CPXD23rDoiA1Gn402JrG1w0g/k6xlkTwAM7ey2y6UjoI2ajqg28aBmEaN22Z
UNEgajOeP9ZorMd9z3lGWcU89Lp27vSl/jXJ51qkirLQw0znedQ8wT8Rj6c7
1J6gUMR/fsytFwgoynce6snBrDzr8eURb2H0ZPuTNbva8LiU3hr76T02aeyc
melUjJ7du1q2T+iDkvrl2FW7y3FXXGa7I9ULhR6EWs6cn4zhD7HTL2Z+gu7J
08LRWRVQzSi/e2e0AY+nhZrrHaHyqZp6+eeqT8hU5MX46tIR1CF/8c3Kz9jf
O+viavdqDH03eL3JZgBCGcvpg/KfESd3JdH8UDXVZwIfdP3LQkPVPobpplpE
31/kceVrFdg3XLaYXq6HpIl931ROFapCOhM/z6qARULk1HDRHpjcOZOw/dZH
vNk5bx3j4QckegyLf01jQn3tiur1CYUwfWFUtGJjAUxP5P1pf8RB7cftq9bs
S8d+Ruv8hpQ0JDs5rPakDWHqdU6cOMVFuStvgy7f5+Eur6SjeN1LHFoWopo7
dxBqkYobXYYpLrtYCenXFcNaf2WobxgH6noyWzmTONBtmb54zfIyeK7ZmJ6x
hY2Uxtrx14p9YD091be7vAwL/D7eTbrIwq3hCaolBQysnVt26cCqD4h7NVl2
hjsTT9SiwjiqdBy9X3ji/Mc8bAxe/dVEmImhqIU3fK7QILzP4pLrtnScDbvs
fnYWEwWJT70nJtEgxE3q1ODGYP9FJe7zg0z4Xlj5c2g2HRtmvyxoauEh3cpy
yvpsqvev/sZ1iaNjx7F6mvEZHm6q3qj5O4kFh3Npy0unM1DTq6J63JWHrHXo
Xa/FwrB31bseUwb+Gh7fbxPJw9bgBrU8E6pX6NxKl6F6iOrZwKkhJTxM4Ahv
d6TO8eWGVyePnGXgyFmpYMtvPNxobZRp2sZCl+7l3xbUOU3LV/+GZ9kxlC5p
69VZx4L0x8oJk/cysLno17bAGSdRohntz13OAoyO/PPbwECeQozv9lA37FXq
6TSj+v4/7uuZiSoMJOro/XoYsF/gOs32wnOM8vCB3EeP3KfVOSjLu5cn8Fz/
aR5/GO2C73kp73j4+n7B7zq/QbTy625zgXX4xNGH37tzAusWXqi23JkdJrDO
WqUXz8bvi0Ia2RcLsi+P10U/ONwUCQuyj2fJPmbTxxSSbngK7HvC+GynRBOe
QE5y15rvmf19EHdIrtJIrq4OfJwlsY0LaZLDBJJD7s+ri1dkcuBDcvuM5HZL
jNzW55/6cZTkXIfkfOzyOa1XY/1wJnOhReZi+Ej/efed1LmJzJEKmaNF9g4p
im8pjpXx526QzN2vY5f7q1qSYULmNIzM6ai2yPRIuw/wIHNdR+Z6ePES2sKk
BigRDrQSDvh87BruY3ejh3DjIeFG0uvtBy10WAKcCanbozJJdhB+hEs7CJcK
ZYeVJ4cUI4lwTIxwrDZ9acO0S61oIdxTJty7piitnnGJCUnCySjCyYZT3iJ3
h4YQS7iqTrgaan90j8ufz5AlHM55x+dwkMoBlWOtNOgRbh8j3PZaEpsarDCE
ZML5bsL57IsK++6z6zGHeGEL8cKmR0+eHZvHwhjxyC3ikcJBqx1PKjPAJt75
c43vnQvbrn78fq8dRsRTOsRT8ku39v5o6sNt4rVXxGs/pRbWTY8aghTxYDDx
YLuHOb3gTzK+Em+Kj/C9qf89I+U/l3xI1PE920Y8271+6TFT80bEEy9fJl6e
w7LtjfjAwBXi8Uzi8cJy60l3U6MgW8/3fgHxfsIdX/PlN9pQRXqCXjG/JzR4
6tbmUj3RxIzfK+xIr9BS/m/4rmUtgkgPSSc9xNH54QQLS2ofj/F7iw/pLXrb
nl7nqOQjgPScBtJzimXKgjPu0xBKelEH6UUSfYqObq5pCEjk96gG0qP8vYVX
HczqheyntX2Wf/rwUE126fv2XhhYRqUqhNbCLmpNagujCXPnrNvmoNkFlwtO
O/R3ssH2WTL+wosHtqPR142eLbB7pB7nPrcGZiOhRe/f0aFqo7F+1rY6qE0T
uXLuKQuyPmcPxcjVQu9iiMaJr6WQcDpsv0miBClnr6TRd3HRo/pLN+NwKgIa
lCO3TqXef0f1OudjPYDIqQWLfQehKh839/ZJ6nyaWb770uMizD6TXpFuwqb2
z2y0vKEFp3+4nZW8woaKVaA0ex0NhofnTt/+mQ0PU+GQ9zvbwIp/5R8yleqr
3Ih9DxyLoKf9p1dU9A0U5Gy0fCWyULejtqSoswO3y/95r3rLgfnuZ471OhxY
6Y7rr9vKQv+s/3aMFzCpfR6tb06hY3rMJP+/jAEkbUgtkRVnIN9VvGaXWC+E
ZWS+3g18jK0ZR79MdGAgZ/bSxP7YXjjY9PbFf6vD7N2J/iG3mbjQ8zJohjsd
J5eesz+WQ4d9ya+Pmsv7YKTmuMLgHhPZQlc3q+2hztfqPySXtLJhO9fFtLqv
D8NwSHerboLFPgv1xVRfWKRzPXiFAgfs4xq+P8704VrIR3ZlIBceGD+2hFpX
2dqCB0vvvUVOeMeyd5ZDiLB8Hu64i5r/x8IiBZUdVA//btBQ44g4Q+Yb7VJH
mLhBuk2aDaGBtdofe59iq6vmzzVrniLgtk7yDkYaTFYcjHNQy8Zg1Ba7jdJv
cIRlmqB0uAl2pmOGw7qF8LjGeCkWkw9HI33h0Gw6AvZ9njL2vRidL2SeKqcX
4emR1c4NtwYQe4FzbmhxCTws38dJpFP8OWUj8TcpC7UhYpN8br9HwEZZXYNN
xZifuei2RG8dfOSeKrw6WoQNoa1c3p5CSGXNqB1t68Kwwueav2K5kH372ln4
zVtcsXab+vwjA7KPgr2Wa6bjCcshd49LGnZrDk5R8ab6+uzZATnbLsE2OKg4
UeUSRKrb9sS5DULTr2e/uP0QXK5G6ensH8J8JamATPto6F82GPLYyUWzeNbX
nplc3HHYklih/gY3Dhpf6d7AQW7QZfvh5H5stz9zexKvEGoG9Rv7D7JxdMNU
j+htfRCTnbRv0aFiPF3mfOMmiwXNXrad9k4m2o5mTpI6XogLbJ+n6iPU+TAh
4+swl44F2lcLxL2zwXAZEp/jx4TckVfSFnp0yE3c9npB3Au8XH7nmIUTE2V1
tfGH5OlY7tDjvepnHFxVaySfRzDhftTO7sxBOqpoZvNfLDsOGY2Vv6oqmNil
/BDvM+l4d3F335w8qoeIH7J7OpGFWo6ixuEpDCyxOulWmszDwzSrtStUWEhd
elZ9xmaqX+3xLCxI42GWr4jXHz0WSs/06AXYMeAhnBOoU8vD7k9Bz6yo+ZDc
VzWy1pHK+8K2Q1u+8JD4N9B0H/XzB0/EKV+xZQC38yIlQiywh3etkK3JQpbo
V9tFexgI+OPiUX3qGGg7uF/frKDu33pWenwTA0mhAwcWTTmC+oXhi/5QnNUO
k5fGWurct6qUPcV+j8B1r+ot/orjPIH7eM9Nof8Y4Ak8Vzg6YvWVfsH3XJP8
xvvdqOB3RYRX7dx46oDAOkw2+isHmjcekHV7TtbNR0iVPlMiSGCdzX2cDh+5
FoJFZF+Myb7US9iqTskNENhH6Wf9+zoYPIF9L/mtF6SmzgOL5ESB5KQ94lmQ
G+XlIJKrfJKrbEtxpcUjXKSSHGqTHIaw6vw3MAegSXLrSnL767v/QLbDAO6Q
nJeSnHs9Wtq6qWQAxmQuaGQu5pquf85aOIj1ZI7cyRyVqf07wDnCQ8Ac/ty1
h/DnbmvkisC1YZnQI3PKJHMqsWonw3RZBSQU+XOtS+Z63rZlI488mhBKOGBE
ODB5+Y9bl6J6BLgh9vDES+8IlgBnjqwe1e1RGYQH4VIN4VK6q/b+1CTqvoRj
WwjHXNx9f/CutUCWcK+IcG9IrCnjeDWVUw6fkwMufE7abeWa504YgpA1n6sJ
enyu2m2bGH0rsgZvCIfDCIfZKn/Pysb3Iphw+xjhtuQUo8rcSq4A54OUbk+f
IfYZAcQLFsQLqnZVj8piGbAiHtElHpEdsq1eL3UTIsQ73sQ7/nmTddaEtcCA
eCqLeGrMZGa4xiUWEonX3hGv6apVC7G8hzBEPKhIPCj6fHDqFJdCvCDedCbe
PHBp5tloz3p46PA9q7uM79m3bbdkllzvRS3xsvJtvpcnTbZ+1vRgAA+Jx7cR
j/d4pra351ShkHh/GfG+1K3Gybdj6HhOesIu0hOE/Ntd7jy/jg7SK2TV+L3C
Y//T7pMj7QggPcSH9JDx5Q4Ow7848CO9RY/0FtnhgYFJz+ow6Qy/54yTnrOv
6dVChjYbkaQXLSK9iD32X9NYXTUsSY96SXqUUkPWY25uH2SFmm4zXrVAa+n+
/L2JXdhr3GK3fMkrCEXNzNteHAhL4T9vgjuo/YhUl7J6R8O0sFUXvnxmYDji
8+7k7jok7Rk3qfr+EgEv6r0Pb69HkoS7J7O7DKrXnL9aGvVCb0uL54SAIeil
/hW+ue41Utzq/klfpvpZcYNViT4dIYOzhkLjhhDw5MCs0hNtkGh7WaedUYOh
L50K9v/7e4fZwbyc6/1g7QoryEjkYnLQlpr3JxjQowcKT9Qoh1TG+4kD93qx
7PYf7cOSHQjgNr4qO0eDR2f3r2fUuePUu53IpM4dDSsPrHYbHoT1itcPnOKH
oOXTEGNL9fKzt3Ofrblei/aWj5t1mikv5trySm6wQFMS+XlrVS9ag2vUXlP7
qb0m62D39h7UXfBdmva/39s7NXV6yONecOQviup8bUXs1Bs680/E4uq25X4f
zHuhepMVerGkBS0eWeKJog1Ym1RGL+L0os192+OQ3jakm/uzz7YysPPv+WEb
YTq8Yr+NHU/tgtA/jWj29xh84bYuKaO+M//Ug5TyJ73wiD5Lp1e3Qqq/vUJI
hoXGCPuum6oMWB7Yt9l+Sj9WhV45JTXcBz8Dy955WRTXZQKOjo8UY9h17FVz
Qz/y5zdcyFlO9YwlZ+avfd2NyFWpOZ6vBvD2nvdqJfv//d06YRkd9368/y5m
ucZrEMkGV+9WWw6iR/vq9Unyb+G5m3tbdQIPzSd73GX/UD1NS0f/6P/+LkDm
VdWF7EAYiP+8pNoQiGuf5qbudab8ecApXib8PhaYrvcaVbsPvUy6fcwULpJy
1r8ISXqOmL/iSb/anqFnQ5rMX9obyLqHG84yS0MuZzx4i2MquKm9Rc+TqLwn
slpWXn6GHRq8JUp1T2G2SSilZFoXkg6YVB1wTsa3Q2Uja2WTcftSrjRvPgNC
STd560Ud4HL/bxNvogNCZpQp215gY0zpd+HP0iFUz8+411w4hM+WmSkdG7no
qL+RsVt7EPM+/ct4Snni+8J5wUIbeFj6JzrYNnwAem2dHnTNAZTcX/LzWdBD
VGyaEPm+ux/aorYqWNOPhj6bXfVTsvC46VLI5gNs/DNVvDPfsA/b5G7NSNmQ
g6JtMXbH5frQ+jkjzjGRidMz6g3WV2RjzdDC2zI7WTCwU7cQc6d8fZWt4XT1
FfSCGvq6G5kowIXxCR/oUE9pD3G6/wT/di8qT0xjQi3dtbAwnI59JuuPFNvE
w1DZc7wkl7qvsY/8k0Q6etXTl5yWCoDBQoUeyW4mFj9evexiHR2jtnefMTk8
vD7PtvkwmYXsHxFpLBEGFpse2PqxjIcNP1RunZCn+FpUcPa/9Qxkr9icvbuc
h2SzHafy17Gg999dmaK9DMidWEx/0s5DzLn2p7N0WUjX0rbusGLg4ZMHX6yp
fnXVpMfgmjYL9pobLTbtZyBMx9jdIWsz1hQ+SPVeQ/XMN35npu1iIHNf6IRP
n6wofuYp/6H6ibKeiiz0GZjW81VfJfI/3Jguf8lGkYXj706HR+swYMw4J2Ti
bShwPaTnnwrtJw9F5D6q5D4SwTac9V95As9t1jm/MO7/5z1HVOWPRf0S/K7Y
zOeZUlsPCKzDbPePHuIqJ6FF1u0OWTch+ePd9y19BdaZ6WFRZ53gK7AveiJJ
dmZSRwX2UQeBijcLeAL7fnZLF3f6Sh42k5wUkpzkbrngUH50CGtJrraSXB0c
ute679og3pMcdpAcmiReWvCWxcULktspZvzc5kjv8q/M5uIzybkBybnIMrOb
yzEIJTIXRmQuaibb6L2+NQQamaNFZI6YRda6vO5Y/CVz10TmrmXDwTaphfkQ
SuHPqdMj/pwqVnSbXRz7hEIy13Pt+XPNHg0tW1rXDD3CAUfCAXbPvxd3fvVA
j3CjjXDjvtLMu0/SWAggnPEmnGmT3LExSXgQeoRLbfv4XOqxobHNKgsQ8JLP
sVIxPsdkjzbdt1Zphivh3mfCvYbDNJtPXXQBTpZZVrueb+fiCuHqS8LVWr0v
YXMMKsAhHM4iHM70eO91dHkPVhBuexFup4/fZQUaDGA+4XwN4bx6dNbP06xK
cIkXMokXvLL0xa9J0rGVeOQI8Yim47FcPRMe1Il3yoh3JpkLrxLZ1STgqaKP
KePqP5m4RLz2m3ityXbZbu+N1PmFeHA78SB3dNZZg/+qwCDelCTetEsdEK1N
6oQ/8ewB4lkHDTnlY2/6oNTF97JoHt/Lw1EKp675pcOYePwI8biHolms26R2
qh/yvS9JvG8SsuT8+Vf9CCA94RfpCeqLmZvnvi2H5Ht+r4gjvcJPZd/3RR9p
cCQ9JJ30ECGHI6JCb69inPQWNuktkgzlhbvdOyFPeo5QJL/naIz1nJqSTPW4
BH4vciG9qLQ7sXJnSBuSSI+yJD3qwxjDVDqB+vnQ4MKkn1ehPvBDiznagr8L
6vSXveYic9lmh6oJTMzt8JxZ/6MO+qGbw37WU73gJE3N3bMB48nSrd+iy+Hn
1/Zk5joqt6M2R+WzeOiZbnfG4Uw23lXmr/yn3QLuzYVC61czMcVVqPyoOXU+
5I6/2a/LgdHw95ZHixpxyeL8AFuC8vtJrB9jVkObffDpplbKA6Z1nh1SLKSI
4ZxoMx0eiX57Xbcw0f7SzlJuTw+E1GQim68Gw0/p9JVoqyaMPWxWVVStx5Go
a8d7P7WgcOXFeLuD6UhU7w1sfpuGY/aT40/S+pBrmnV2hQMHrXmtlsKt1L4Y
+5z9KvQaZz4cXF30lIHVEpNm+06kodqee/3RzFY8Xr3PX2isB5MiF2nPsm3G
Q/O4MREGE2dTgwoZM6g+9FvZ9RBqUPpFxPyZLw9KIY2vht934Aar2Wvh/XIk
WS2Oey7egBce3edXqXViVKnk0TmfSiiN8qxVUpjQUYmv1C3sQpWrTYn8Aup8
ojZbeL1cGppeVv5dcZPqcw477O1HGuHx4+TW+7QOfK4p2Pp2CdVzL++PSrnT
Dit73fVrsziQn6VXXsykzqtLKj/cr+yB3VH/lsyOcmR3d8jfrWHiRebfnKZc
OnXucoxoT+vB4uo9PqPFfTgWsVD8whlqLtqv/TL6x8H2BbcUbB/0467Fzjuv
OWyUeYpNSF1UDBGZV3muNhQ3TqWYWFI/57dFgzfpVhtW/MeWXNPKRcWf9EmP
UrmQys6oDzVjIkv8oqrm5CG8+XfYuXV0EHKv3Z8kP6P6TN/6SR+bhpA/cNha
7/MQQkOYPG5FDjq6ag5IbePB5Luo8G9DHoI2zzjkXVYHNTXhNbJmPJxv/iRh
vI8Hm52Pnxfe7cT7y2Ytl4V4MBSr8fD5Tc1RyHmn2wl0LGfFbvxiOoRlBzq8
DhhR133i3Nd/7MOIRKLQxgODmMt+vuW+7iAuz2Erbdg+gJ9hVmrCK7j43RWu
28oeAOPtp/dfF1JzyNvwsqiRA9GWvDvfD3Jg4Ln7xqurPpjd3Xyyz78fx2ea
DlzPYaMqs8Y3P/sxXE5vW9NrwIbLF+6FvdQ5coH0aJ3k9QyE/nHe4q/XB33R
xj7WRyZ+berzmT3hJfbMGrwvFMyCnIb4Kvn3DKwy2xq1WicV4Q7X5c/PZ+HQ
8aqlKssY+NvnM/Xgtgew+HDmwCQWE49VtQ4ld1D8sP70Vu9sLEZvyYY29jCh
XLMsYUYjHTG1bBeTbUEY933h9u4LEw1TeGIaPDpsk5NCvT7txk3XaRFVolSv
+FZa/2gOA7achDV+DB5mGpR1jC1l4edTgx/v1zDgXLxn9cpOHhihKwuD1Flo
Y36YUWZE9ZnzQXotTB6KJ07Ve0P1jSNW7rlZVN+QHSo6IjfKQyRt6XUj6jqr
T8RoAXWd/uP75JNR8xGUUsdbrMGCS6DuhQU7qb7kOHx5/L/dMOEF1G9ZycK/
s7nOCw0ZqHr+ZlPx/T34sl8udqUylctHk74lgwHHduebpf82CFxP0Vg5tfA3
T+A+8/Z5zFwwzhN4LutiZ6DTuOB76tW6u+z8J/hd8wuX+a3L3Q8aWYdmsg5a
ZwJ8CxSdBdbNrHKxpYPTCcSQdbYj6/wxad7hqovHBfalubnznjH1Pl/JPiqR
fcyzNs6npfKwn+z7U7LvUjGrf66kfHeV5MSe5MT/ROiqK8+GYEpypUhylRvf
zjJeNIRwkkMjkkNZSXp6Z8EgPEhuT5LcJvklMzs+DUKS5NyT5DxrkoGkl9cQ
XpG5ECdzsfhwcyyjmoc/ZI6EuvlzpGcv//69XSrGydxJk7mTfmlrLd1eDCUy
pwpkTqVmfpK9fqIWpWSut5G5VhwRWlle2iLAgaNtz54sUafOv4Qbuwg39tqe
2VNym4VWwplcwhn5PcL29x5SXiNcyiRc6tl/IfPLvRwoEo6VEY4JRSZYDlk0
YjrhXgbhnnXnsdBzmTQYEE7GEk6q0d+db/AZwDzC1f8IV4WiS2Z1TCzBM8Lh
eMJhJRW1XqOkLiwm3E4l3C471lNeF9qPcsJ5Tiif8+reIstUFD+gnnih9P95
4WKpdeq1XqwjHikmHjHqrI7omDGEO8Q7z4l3ZkvKbn3ypgEriaceE0/11nm8
ejIu6DXWk2SbmCuReEE8OJ94sMPs02zO+gaEEm8aEW9mVU9fr+pDRyPx7Djx
7IOx9Mhe20EUqvG9fF6D7+VInW+VUjq1qCIeX/2I7/HxLK0dM+PoCCbeZxPv
24UVJJc94MGI9IR3pCe0sAIn9+9twWPSK+JJr/Bbpn7ywlY2Ft3i9xBJ0kOS
9iySTRb5CA/SW+xE+L3FOl7WKYBGRw/pOZL3+D3HSEY148KeLNSSXiRPetGU
jbY5E2/SIHSD36OMSI8y0nfvZ1rkoPamAp1FccTrkK2jWVwdhqYbpPhu68O0
v9k/nEzqoXVWaFiBUQZLqdczr1XUIqBq6LrSWx48fu9K8wnMRq3ROv10SzaM
rZdbZE1gQWyU4bX/Pg+FOXHdS2urwBZP7sw93oJxsw6b1yUDCL3PCa1VYFI9
7Fjhkn8ZqJ5xfIbBdmquLfZvjVJ6habyD6pmG9iYqEObOms6DQFbXWP713ah
dt280M2VHQgv29CXXNoMuwtX1mdc5SDJbWpM1fMSOE8JeeSm/B7TxGTSG3+W
w0Lyot0C6rxSyFUy/NPBxVzhCnP6bBr2NrwMtrViwXvpb+eBIAYS8wY9NTcP
ojRR75Wcay961m/lhEm14ssdrRt5CrWIs92cb3GoA4a3ozYe6v+Id80/PgVX
0KD1U/9bWV4L0uc6pWodeQLXeRdlVTsH8Uay4EX5hmbYnO1WcIzkQUNzfnu3
eT3Ey68f+KfbDCkrrpvmTR7cK//OUddk4Vr+9svnHVugx9Kfaux5D+q3r11Y
o/IW0j8T3309R/XiBZuYtzXy4SizdlLwzB7cUj8tNMu0EwHfO0e6FlbBmV7p
szGKi1X3j8+8Y/O/34P8RStpYSOy946phTbX4vQq1fszttLgs1x/roc8NU9i
9lrZiTRMLmiU5FEeClXr5mw/3AuXACXmp1ouDPoVQnzAgmSRFtfEhpqbUvkp
Fo/KcOlDl4utNBuFFV0n3/KocxZbYo7osU7ouci61xT1w+1E7rTF4v0wYf6X
V+vDwsGK13YpsgPwNFXwKKP6X517RXRfyCBcLee2DCtzMRRrJS8yMICkcMlD
PgcKcGnajIViFN+O6Uwt9qX49vLfVu9/++vhaK0X56Q0CCv15XNnTBnE3wcR
cvqJnVg4N/2Pq/QgxLYam1V85yJrnYyX5H46Yj+1Rdu85WL3Q61SwzAu2OvX
7pxq0Ae3jdPLXcS5kGjbdupGxQCqf4jPORPBwZ8TjodzVQZgN+yc2faeA7k9
OZXP1g+iQq/2vogiBzdrr7YOXunHdpHq9h/reYjfOXz54sJ+ap6epZhoszGv
88u43NoocPQMRGfKsHHsvO6esREWJpQWFa37mYKcvBj2YqM+GCo8PX2kjomw
dx/0J917jJjBI79/PWbB6t74j/RRBlqs5fuFDj3E4ZEqjTZ9FtY6Jb5YeZgB
65Rdo2v7krDq+JYd0eIshBfP00xbwIBag8+tGMMYbJB1933xl4nYZ2HuN4UY
GOOo/Tn9NQgZexecM53EQu/GLUqfpjGQENr616PMASvn6G0XmsPCjRcfjKUW
MzAoWnFt2nceDsUdLJaVYyGxySZcSJOBBb89pjMHeJj3LmlUhuotjYeu0boN
GMi7u2xs1xAPs9zC7KKo3lJ9QomXR/UWxU5f11tjPPQLr1LduIaFzd98X740
ZmDZwpWqo395EDv8uvMH1U/cwmeMeGxnYH/p66y8yXq4Liv9bIS6/zr/AOtZ
WxjYM/xs9U26PjQXhXkNqrDwwGmip5I+1cdsmKZeU9UFrptdPDXr3x+ewH0m
xXhKbfsl+NyIfVGRf38Jvqdtvauabp6swHf5vtwybfnnfZhL1qGBrMMpA7uO
6S8PCaxbBVs4YkzqiMA6tyxnH6cN7RXYF2+FCRHr2TyBfTQ1FJXXeM4T2Hfr
DYam7ZQfjpKcaJGcvF2emxQ+MoRbJFc2JFcaOa4jPOocn09yuIPkcF1KfW73
qSEMkdy6kNzeo9OjrdOHcJfkfCrJubSTFtvCjYcaMhcJZC6q3mwKLhGKg9BJ
/hwdIXOkHLLyXrJ+FjzJ3M0nczcuWfJKuviDwJzujIxxGjnzWWCus6M+LKjZ
2yrAgVkrs4e6qH71f7mhHWBtKGTGghPhTD/hzLmhjq+Rc7nYT7jkQrikfuH5
ofeFr7CRcMyJcCx2rfElG/F6nCfcyybcKw7zPv3TuBcgnJxNOHml+rS/clM/
fufzuXqWcLXwT25T5Jm3cCUctiQcHmbpREQUt0GFcFuVcNuwOYitc6MPVwnn
zQjnhVhJGxa55WI+8YIH8ULCh5jvX3u6cZF4xITJ90iJampEXBsXYsQ7ksQ7
UU4TW+7eqkcB8ZQz8dRiltdsCXMWdIjXhonXDpj3/Uqcn4Ek4sEjxIOZJmJH
vji3oYZ4c6IW35sLMuhrk7l9cCWevUs8qzc3ys567DVOES/TiJdNfh4329DW
Dj13vsdHicfV88ajdwRS60m8/4x4X/Z741Kd8g/4QnrCbNITeqzL14gye6An
yu8VdNIrjsxb+2YwfhB6M/k9RMSc30Ms3yoP2bxshA/pLTKkt4RfXxj6vZKN
WNJz0knPMetU+L7pTjX0SC+yJL3o58nl49nxfRgnPSqU9Kha7pk30058xvCe
huLHETXweegjEZNbikULrHRXqdNQtTV5ktmWISTdq7rtfisTl1hVrzzl8iE2
f0HI18NMfPaVpLk58kA7zvhW2ExH9hcJpVLtVkSqX7w0c3gAkSZ75nzKzISe
p8qGZfPzcDxaRoz9pQ+M1j47zo4e3PDwPR/sxEEv12d6+2Y6JHMaE9M2DiL8
dsqeOdK9eNy8s2uHYzuSKsYLb+yuR+Gqt0of5tRi/4BDTkxaFYwk/y4w62JA
Zlh30ZUUHqwf/HWZQv0rVLfD787Vm+Bc/KqpacOGSn06/RvVFxl99G37Klpg
k2L+sTachqDdl2lOUp0IXXt5//WJbAR0f4/fHEfl/t7utARq3dLn11g92JuH
jsJ7i/b8bAKDOTjtrBZ1vnU3br8l343JOesvaryqh7pVlO6jnRwc4Zm9D/kz
gLvqDiYr1tSh5e/zvGrqHNN1efCK7KI6xGw79DhBtA7yq5KFf/j14Y6fh9XR
ZMrDF3YUdo/VQbt2MI0+px8+iUH2KW5FGNG+659W0QC57fHVFyZzUXzcSC+1
thecMcn48WPNWLTHP96FOjeKTD1/687DIQzL+S+Uf9KGgCLPqffqClCrwUje
btSI8kulv+Yc64L+M/96R6taWPtWhBxtpDwwe8FpybW9EJPGPLedLRAyrNI8
0z+Eb9MYdvmb6NDY8Xz+rFPdMHESnfVFthYS+/MVr2kyUcMptZRppOF0958o
C6se+Bfax8r9ZSFk9f6nEi6UH0If/sho6YOHyet3xSFsiDOMJaz8+5Bz19dS
KmEILMb0t0rv+5H8Yv54xcx+TKrWee9YUYwp1+4UPnzEwewLc30/KXDgVqAn
L727AYoRvxy+rR6AQ2v46lWVHKybbGnaOaELkc7Lq/oOD+BnUMWP/MlUX3Jf
WeZuSAfriM0qRcMBaPf1FEf2cCDrywrp/MJCmOUuxx+dHATsdHlXTJ2rFD64
MO1FqXP4rMJbo3oceE7nuEU/7Ydj7JS19uFcTL5w56htQD9kD+i/dXrHRuL1
2ZfjKR/JWb8zoWWw8XS6rfXi133wC5dnLa7k4VS9w8ONA334nZ7ZEJ1GPW+p
sKbB7Gjo0+IV54Ka886H50Y/MNHwI2uDl0Mibn+jd2W8ZSHl00SnzWJMzBa6
+WDqsrt4kCHKmWPLwuHInb0PwxhoFP51+8rFOFzliQceo3pFu07W06ObGVCZ
Nkc++Ph1fI1ybn06j4Un0bLh35dSvSK/aKjA5iLuDtZfVqL8vjsx5aEZ5XfX
fYsTet664dBkzXNa1HmvpOZyS4QSA6tT6A2votZg/4fkLSXLWFi5x/y3zToG
7udn56yl+pLFfguLeOq5SvZhvunUc0W4ixicUR6qQ5SSp1G9pe7LQbF5Rgxc
EPttIPODh/eBMjcVVrOgGe/tmkH1mb9fvaptqH41YUETdxb187p67l5y1M97
Yx235KoaHG//lN+sxoL6fp0pQVQv+tpo+entnbXIbbI1/qVK9YSGzu4XVJ85
ruhnfkZqmcD1TY+dLaup+//f+zDNSo2M/wg+N2FjxrIFfwXf8+6KwnTJOcoC
3/Vp7NPM4Ss7BdbhJ1exdL2yhcC6PX174dqVG6YC6xx2+NKTq9T7/N99edF7
eMnWXp7APk79YSh06SUP18i+d5B9z3forr7vxcMjkpNjJCe9irLP6Gt4Arn6
t6e+uG8KD1tJDp1IDlcbadiUiPHgTXL7l+R2jGXLcnXgYTnJeSrJed77t4Ph
HmcxncyFApmLFFaT2/6HjyFB5ug0maM/Gyv3DyXkCsxd7ICxs1fdR/SROdUh
c9o8fc2woVSdwFxfkH3gLuvdKsAB9z+mFrP+9UCYcGMm4YZZUrfrlxYmaIQz
CYQzkWKra47uGhDgUtLkrT5Wmx7Cm3DMn3BMiPdSLGF3LcQI98oJ94w2T2rb
t7Mbw4STcoSTPjvPfg/m9aGIcJUrRbharPPKxTEZLwmHZxMOJ00/Pqxp3gQ6
4fY0wu1C1oxV7QeY6CWclyCcF4qaGapw8wEGiBckiRc6lNzP37fpEPBIbItN
n0XKgIB3dkxdsnNtXh0eEE/RiKd26VzO4VH9+v96rUf/+Zee6YXoIx4cJR5c
X/bf7EzXHkQQb2re53vTUOahWaU5F17Es7X/z7PK7ck532ogHsT3shXxsp6Y
eEdTPB36xOOBxONPn8jMjd7JwzDxfgrxfnaNcO1+gyY8Jj2hivSEqviLd19b
spBDesUY6RU96vumX5TKQoAXv4f4kx7y2Fh9k4pzN6Z95feWZNJbVOffPHjM
aRCypOe0kJ7jprR+rr9JCxikF/WQXmR+OPhyhg6X6jv8HpVOepRX3c5LyXfb
wagdi7i9cgA9b6sKKqUeY1P3zzdLpak+ufqB9qFBGlrXHml+6DME+9oiKTnq
uyX+NRz48LwJDgflt5sdG4CMefWXnqvdKNR5abtUMQ8jJ78Z93/tw7bF089O
NBmCy3jh1v6IAfxrUx6J6KVDyUbLT9K6GcXrV/oyF9NxePE1zrJfnTjSKuvQ
DzYYDyauHZBuxGvu928Gm+pQaHhq62r9IshuuHayfDgDB851N/yzyEChf5/H
YukeJLplyj+5z8HE7foPJ8pRPWe112z1MQ5W0cQlbygzUHp176+oDb2wW1Xj
cXO8Ag2i/ayDKl2Ql7a78vRdDQrrtzUmFtGw8fhISGtzE8aYX0Isp/EQKif5
PlWVOhdXzZ3MmlgHmzrZZ4GufYidu3uzqHULUgomn6szqcbT1RJel9m9ECmS
ni3D7MfmuTqeSxMqYehcK2zc2Invd+M9/UJrIW3jaB2qXYnRPRX5fiMdMDnz
3v7ghD4EWJ/foFlahZPnRb9N1e1BrZ3ssMGDUjh+jsm6LlwL+d+B9xOe0jFu
5pIt8pGOV9s/lp13roPzjzOGocvYKPzZmchRTcJelexP58434ulu85gVCwYx
vrtUVfu/NrSe1PXSX9KKaQUFTUK30+Fxwipt/sE+vLUKXT1M5d6OZSczuqwS
Qn2LS61+p0AhKb5JJLEHPRqDm4pmNsHuwocJef4NiLO3WnbyAQ0Ju54Pqbh3
IqXU6tVMQxr8Xny4OucKA/pP6F1va3rx8kC/bfCpfpSLBy8rWseC5A2VnyVU
r7eTrmtPoc7paz6ob2243Yds5vXLDwxZsGNY3NmUU4rOGS/MzqRR3OtLe/03
ow/DHXv3DDxuhC0jLOq8az8Ov/ery0hh42TU7Q2nP3ahXJF5eJQ6L+54Yrb2
f3+/IF07S8nOno7HwqbXX43048q9Ky+Pbe/HhUTrc4n1LLRo9vO+fezH2n2V
KJrXj/EJntp12f2YK2KvX04lL+3syryAO2xImcXa1szm4uvLk+b6NDbkQjrT
Iqk8nww10s6jD8KMdTZQbQcb1xr2CSWt7IO9324lu9VUD5EbbV/yksr9jGnh
NfYsMCT3dq7fvgtRUPT8taIPChU6fR2PmZjh/f4JQ/QadBdHiKW8Y2HGMzEZ
BwkmxStf75LoaIhtFzv/wI2FaX8nyzCSqXPw6KxNxb+vwyzr1ufATSzknikU
GbJmoKO7+nVWzRXMEz3jfVaBhXeTLi/aoc3AufPpTq0+AcjIWRz7ezELc5/f
jj61ioEj+/bO/eDqDtGcQLUCWRZivv/ZPkGDAS+LqC7LmXth6Zy5aiZ1H+Xw
5sVpGxjgPB8MH/nNQ9Cdm9aGlN9dk2ztNSm/vywNP/WR6lG12luP36H6zLWt
Y/LTtzIwYu0SZvWLh9JBsZrKVSzId/82zdtGvees5+FOVJ+p+4ZLR6nrxdfy
KlZQ12+rncrV8pXEj8RVfw5S99E4aXz/BdWL/orbPFIOUMRRlu3HWytZGJ34
+mmEIQPjtaZz6ujzBK4fOurea/KPJ3CfJVfM/4T8/zx3v7y2wsZ/gu95V4+2
aa3DCoHv+rIxfHddjL7AOkzdwKxaQTMSWLcVL2KjYj21BNa5ISvS1Oc7dR4m
+zKb7MuL1AHtqB6ewD5O2PRf/KJ8nsC+B5QoLS+6wRPIibXHzKGFHjyA5EqC
5KpwqWHHocM8RJMcKpIcyn+xdK0N4cGK5NaY5Pbds2kvX4xTeSY5jyQ5t/VS
H/tZH4dvZC4UyFw8fJe7cj4tHfPJHGWQOXKx1RW+5FQgMHfbx9fMf7OoXGBO
57/fOvm9ZR0+krk2InP97kp0KnRaBThQt06DbpjQg3bCDTHCjduz9lteUWFi
FeHMS8KZn1ufWnrs5eAD4dJswiWhTMZK+QMOOE04pkU4FnrdKoC1ogIxhHuR
hHstJ3fE9Xi3Q5ZwsoJwkrNJ5nxyIRMPCFenEa6mz0o01U4fQjXhcGw+n8OR
MjuT67Z8xhbC7VjC7cctCf329TS8IJy3I5wPkp1tsfrPEI4SLywmXgiwUIzg
nGhBMPGID/GICPu7cJoFdX4k3mkh3qHNOKO95c9nAU8ZpGuUFuX2IZ14LYt4
zfh35Juw42W4RTzoSDxY983S9udnOoyJN8VYfG8WZvxKUHSOx3fi2WziWYal
h3mxXxt2Ei9/IV4+oll1uG1zPwqIx1cQj6vrrMqCYgl6iPdn+hPvT1y9+H15
NyQe8ntCM+kJ3FT7VM+0AbBJr4givULvtt9hK6Ea3CA9ZG47v4fIfaYX/e/3
4RVu5PcWIU9+bxFa4mbb8u8ptEjPcSc952ZjbPDMDT0wJr2oi/SixpGwPWe+
B0PyM79HCeXye1R+5ER595M0Kv/KG1zdOrCmcvNpnzguShXvL1wmXQ+TuPM3
68Rq4OppdnJOSz9c1LhRUOFCT1E7BZ7P4Hh3x/PxKVT/pJmKhfm0wEfqttFf
rQFoycTtzn1Jx5LObZz9zhxUdanIlc1nYIj2HtFq3dBSTZ7tZPwJi3bOOhTM
akFPodia/6Y1It7wzdoSFzosGaHjUwOLsVo+SsHFqAiZV9KnbJfgoVXK8ypX
bBA/N39uqe/jYtztwep8kUZwOktvtFPnUBlHqWNZOxhoCaTvj1jNxJ5J3V/K
PXuwI2dPq6ZLI+y2tAcN6yahg3VU+J1YC+zqRoW2FMb8f2ydeTyU7duHRUko
iiKpRIlSlmzZTklCkaKSVJQiVIpKSLLvJEmyk1TIlp3s+77vBmPGNka2UHhv
71y/f5rnr+fzOZ/pnvu+rvP8Hsc1I4FF9Nlu9oJuMFIhzWmUNoHn63FdjxUC
cJ2JOhz9chz21Z3jIOrUgA5p/1E94T44/yXN8ubuJrBitGWzf10O8TdKzE0L
G4E1WHvywVsiWFwWu9wkWQomyz0lNUUlwO5kN+ViUgfDzZVFayMlEP9J/Mpa
fhG0kMal5E2I4HJnG6fS6TLQ2mCctipTDTivKPkj72qxc5Rz88ixStD9kKvn
eLUNqnkkaH8wYOfLPaQEocBa4GZYoRPfhHlQ+etjP+ryIX+Fg8XkWBMkxZqf
PTiGnb8+2Xrg2DD/LHapy45rBa2LJg7LdRMQ6DBw/xZ2znzOvFPE2qcTkpej
mjytPwGDaOYO0aOFIMKS814qqResJOa4zilVge7t5Y2DT7tgoop5+qsJDoKH
v8oOK7ZB9MHC83FP8JAmdy1kxmAIzGU6Luk79kLs03i3m1oTYBk+/vC5IJb7
Ki9kG6QGQTF07ymPrjhYuZD6VCJlBML4ooo4/TDfTp2QdbepAVwE06v9JAIc
1/uYpqczAvbKed+673bAATuZrX86iDDo56dX/o4AzLyuapoTA0DcZ3Rw4vEo
XPHftr3BkAhNB4ozTZOG4dyHENb+n6OwMOxJXv+5q2f9snvZh9Y59EFwunoU
rvGd5nJpIwL5i6Fs8Vssx+el72dFjEL5DhVl7zAi6NDf/72vfAIaenmsbmmM
Qup2j9UWUSLUlHsfX7AjgZnHOd4beCKoPTrjbhBHALpAd93lgilwGBp2rrci
QvQ39xPvWAggSV/lc+E9GVjOskn5MBNh2/fvVVwOI3Bpj4rWtNg9SLxsfOT5
D4yDoV5nz9OPAKugdQHnZmegs7v57YUVAZyDmeNOfMUD3eHfryyaXGFbjvS7
QjUCiCzOqnE8wENF607uk5xO0CmXgCNj/e95996XFqz/hXdoV33Cv4CtKYsv
+w8ToI6Ufue8HB5CdsZ0pB0zh4siniRRjOOmFle1BjEf8DNxElb5fBm6Qvuu
PxUgAMHw5Fl+eTzw/Cz/+KOPG1hmNXzfY15hNdH4PAHzilK2lWjuFTKQ1N58
JGOek5ZleGcb5jNTmomS/FgdDiw8XsT85MHVl1bemJ/I/joOuZjn1HEnr01i
9arHjr9ssfpAnuJdzndLCqZy/WRDrH77sMHlNOw61/w9hAsXOWB85o7XFHb9
/pLkbmasrujrJxszt5mqXrzd8lgk5kv/XifNafWk4hr1+76SaGirXaO+TxOd
Sdat5w9SPddJH/d36vWSVOsQOmjIMcolRbVu1Qe0d+zGrv/vOp9XGNUXmidT
7cu4vsD55mEy1T72XHuQLllLptr3xHi/PRFpZKo+MeKjUcjBzr/fUV+xob66
sZ8+bPg7dR8Ssr+z1I2RwRH1bSzqWxrLAh9uIU94gPr8POpz1sJnz/VYY6EJ
zUU6movWqg9empzpoILmqBTNUQ3hOq8zsZBq7h7ovtjxO7kK1NCczqI5PXTa
vaxDthlG0FzroLl+lmfrfySmE/ajHOhHObCFLNejy4iDXpQbgig3GvsOfN2h
gfk4ypkglDMubJt+3TgwBo9QLl1BuXSBf8rMOGAKvqEc00I51il4c9CloBjw
KPfsUO7NZby6l8aF5RLKSRGUk3P1ji9ZzgyDMcpVgyVKrnafuJR7Yn4CAlAO
K6Ac9l99reu4Wg7JKLeDUG5rZTCQFDoHoBfl/OpmSs4bVFzQU1OZBGvEBVXE
BXzBuz/vplrgFeKIAOKIQNh7ix/do1TcwR084u8i3AT2iFOFiFMuAwbO/Z9G
wR1xDYe4trtR9/DBrkY4ijhogThoe5S/XMuSCMaIm/6ImyKTNobPt5TDDOIs
TTuFswzCvQHzkUNwG3E5AHG58Vzqb56/U8DYT+H4OcTxyr/SWoOhbUBA3N+i
ROF+oOoxYqTNCAQjT7iPPIEmYUpwYCEM9JFXbCuieIX08zeBz/m7YBp5CNcw
xUMkf1zNHUkYhRDkLerIW/y9cVcir5QDjQDFcx4iz2FiaX+RU4L9f+RFT5EX
zfuEm9DFFQCPBMWjTiGPcjA/bvnwCQEUfcq8f1VGwfN0zp6LuQQQqLcqMR2v
gsZW/nMDa5Ngcu66BDEWD17+F7bHeI2BXfJFIZqsUQj8fr+0U2sI+CwK3K+H
NsPc5uSN+14Ow477SccSLfqhcImNrIf1vYlFotmBiG5w+R625qjUDoUTjtID
bdlgMdJWE9lTBZ/fjyiq+1SA+ftdPGKv+qDYKy34sz4Z5F9Fbd9wnQziv4Uv
D/WPgcDgoKXay1GwyMi5nWhBBP9fNjqW8SVgW/iR10ZnCHx1yE9xV3phbmj3
QbNjOGhKHuD2degGLV5e5uJzBXDeXMGTwD0BenRa7bmNzXD+4y3Zsy2joBqd
R5ybroeS6WRhbVI1sL//dpRGthc7Xx20y7YfgXkGUthHllKovJuH868sBmUu
lssKBsWwT5G/K3BnIcj1aKSdjiRBqDWruiPNCPw54a660ToP0ktmD+hljsGT
8S1nO29UwU/hhq6F7XlgEZsQ03p6DNrl/sQojWO+3PAj46R7Aah+22HvPTcB
kYUJ7I9bm6DmMOOZpYEiMDBSd9dliYSkkBpjk2Rs3W0OSEsMlIEJn8LTkM46
EFk/qbjVgD1p29wl2RrQkg8n6s72AidLhcPrN0PA5l7KZzDWCKrpfeqm90cA
d/7U/mVPEnwb/0OQTWmFVO3eSoV+zLuPqH1dcauF+ajNDA1lneDQyP5XgBHz
+Mu/rq1sGwCWmzpmLtO9IO6u5o8zqYbpAp0u2VECxA/Ef054iQN+CffOsABs
XlqnGnD3SUDmfRHtfn0I4x5J7NWjXqg8VHetXCcbNng50jutDYP7HsLyzBwO
GrNX3z5zaoT2i7SDGjdGQNFiMMJgxzBU4nofBr/ughsxe3+/wc7pnfQvFCux
c7pOdiGDazoORNk6DR6LESG4fI7gmY7xYvk6Z88h7Hz/5trpngQivOg66pVy
nQAPdz5/vsBFhJU7Ne21I0RIwAUsnI/H+OnKyHcheAy+ZLCQ7UcxTuwObhn9
RoCcB8S//VEToE3rkfagmAh/Z+771FoT4GOY9Iy9HAnr69vS/I5EGL8Q1F9y
gAC9+nQ7VB9NwburV2g6JYjAkc0bIJs9AmbaU7u+KZPBTUMttX6MADJFwlHZ
F0Zg492N0XXVZDiV/kWe7zMBcB/en9k5g4cTEkYmqQ23gTMqb5HOkgD0DIUM
++PxcDeeVSvU/jlEGq9saNDE/DO8iKRrhYdlNhEhC1lrGL+1W3VSmgDXw1vc
8Dp4YHLJIeQHPgE3Vx7cVswfrJkzt8lgnsB994WPed49UDxzruj+EQL4ZmQ3
LAIezhZf9P0gfRmG7z62iMTqVjleRWcV8SDYQzcz9k4SVI8Me0gJEUDcouzm
WyU8iN7iTBLD/GGyhvVkBuYhQufb3ucq44GxuXXTc8yjZpc2PXfE3pd91k1p
CHtfnd0r6fuw13/0vr03HKsHFBIPMWI+MxqV9HsBq688XOy6jtWT5zt9vmKv
93VUOFPmRA9Pmm8d5sbq5r/V4jWxukVP0jf/5kWFf+vR8f169f9xHfqJ8elP
//G+LC9F8qJZmxX+vc+9T9v0DQS5qZ7r4a2B1Gg/Aap1eNq+e07u626qdXP5
WaR0DPPMf9d5f4d02h3Mr9zRvtigffno+uTtacxz/t3HBU8x5/ZuMtW+d0Z8
3u/RQKbqk+T+WxW0TWQ4jfpqGPVVzcf3GiIk6j4csa9bOf7BBt6jvuVCfTsZ
IMEbZvoeSlGfk1Gf+4hVD9gzfYHLaC7W0FxcZpN9yMCdQTVHQmq3mZ2ti+AP
mruvaO6ujrRZloRXgR2aUys0pwpt27ezVTeBMJrrIDTXGoyqDWrpHXAd5UA7
ygFzH/up2aB+aEG5IYtyI97vTeUJGIYVT0rOOKKcORRDk6FcQIQJlEv6KJd0
r5eLG7dNQgTKMVqUYyJ6Hg5LyemwCeUeA8o9LccrGu7CzTCBcjK5gZKTDpc3
a7+2xkEEytVwlKvhPx68o+kfBRqUwwdRDhuEzG8Sjk8DM5TbB1Fu8ww9Xv3b
2w1uKOd5UM7rWqxsot03BhmICzS3KVygCWoyuKlQB9sQR3QRR5Yjvjyt2Emk
4s5f1y1ZNkcbqTiV5CI/5RE/BscQ1/QR15hYBsz+rLYB3RYKB3nuUTjoeTLP
dNp4goqbNEuZm31o2+Ap4qwN4mxsB2t58elRWEVcpjlI4XLyBfqOu96Y8yCO
DyGOu8vUZgp+GgBZxH0PxP3004NHhndOQCvyhAvIEyL5FL89c66EaeQV24Mp
XmGfPke+pzgIochDviIPyYkmcyqXTQIzA8VbhJG3TDJ3HlA/1AyeyHNCkeew
VkbKNqdgHEJeZI68yK6gKMeyqQoKfSke5YQ8qrOW98JdlzFQKjx7mYcfD4dn
ZydMTw1Awogra+OlElByeF8fuzwE1UmE2eQ6zFf1nP3naEYhYTpJ+WAADngP
GcR1pndDzivBbkHZWlDM2qjUdqgLfGm8HvZKtMGf/iCz3b3D0NhUdypJsg5a
+YvmVbWq4cn1mg7322TgEREcklSLAf5r/g6vpqJB+dvwHfp7rQDNAa1ylePA
dcDdr1VjHMJfXYnrTcPD/Q7xJ6H+2Dm2Iuw9oWsQZGZXtMPlyNDxvuLC/V39
gN9cKBFzpwEa+VN/hqq2wmvpE/a4/ZjfewXHB7lNgjPxjUcfGQ9XH0fXlMzW
goBuKq/XI+xcIEUn0BD8DZL5jjl6E0oAJ8qe6l5UBJVji8wZW3GQLs322/dZ
HpS+C3me8WMMaDhFs2POv4DARllSI0sGeBPEVlJ9139/wgFZS+4hGOB8cWyT
aBrQiJgF1oh2Q1JAekAoTRlwWH5REqNPA86GG0z60V1whUm9tj9sFPxM8OAs
/wMEXjqad1zFgc6nDzbiou0gQ6Px/n1vNlTqaOV4xBJguq+bdQibV9PtdnfO
uhYCjouV+OzlFCjK1eZuOYZ5vlZtw2nLMtDydHDN1qsBHf5I2wIzAug6ScqV
fKoB8abu9o8TfeDwUed8XFAsHCY+Sg+1aoLlA98ycswJwGn2IunNyTZgmyBq
jlu2QX/Thp9jm0gQYOepTYrH+ssqU4XwsQt0GxpOLDj/gBDVzoo7cWNg1CTG
uvKjD9xfcup1lNfBJX+zb7zFZMivV/3lWokD1Qz+2Dep7UAzGbvMZFsKrB8a
YM1xCAhBuRMOi71g+uMFZ2RCCxBTOad7/w7D9bj+zz+mcRC3Vfcny2IPhNg4
vfh0fATG8c2GorVD0ODDttofNggc11TpLXYRIJTti8xOPjyEN80X1ZThQdmu
JJs/kwBzmnsvJTCMANR830m2IYJojqhhEgcRKodUvnv6jcDMqnV8avEYWK6J
da2cIsJx44achaoR2BTY846E+ZXAviz7vUpEGPz4+Axv9Qi8X6v9zrKPBPn2
hjMbD2HnGZkS+g3RGI/E7ZekFabAvf7lkeu/CfB12mst4vYImNsrum5kJoN5
7K5OyVICzF/wo922ZwQ+i0Qo5L8lg9I+d0mOQAIwjEv/OtCBh61tkcEVw2TY
9oB/TRvbj0nLrxrvwvHA2vV24t3HK0BW8VsWuIC9nvHK3y6Ms7OMNm0jfiaw
X8Cf9YIcARqMyi6WXsOD9g9J85B7xhCfGZVHwM6Df159tC3HzoNaMWQhXjMD
cHxC62Oy/v1aUbzLOcwfaCt8o5ZrLoJOyC2J25g/fO16MduI+YOyyNnIkxUy
sHeiJZ+I1RPmlD+/OY2HLNfZQfULm0BbnKMyD7tO+uHb28qw67zlFdt8FfOZ
SyrTm9Qwf7Bv22nkh/mDfzRrxnOsvu138fmH6/4jIne0DKv3LuQ9IGD1t4ds
3bSxelK8gnc4VmdhUKZ9MDCj8KdPOEIMq4cO++54hNWHoqTrbNhGqeoVYi6T
xP+4jvWfQ6e6/+N97Zz5tpwOGFP49z6NfidOyG/ZTvVcRrGJubv376RaBzHr
1uKv2PX/XTeu3zWcn/+SqdY5VPo7LddvMtW+3GBt6Qskk6n28VzV27sfRslU
+z7vcoTLgkjdJ+ySjBKpc2RQRn3FhPrq9+yRW8bS5lR9WNZ+QjeZ1Qs8Ud8m
oL79LDYt9ehVOFWfW6Vwm1hwJ1DNhTVd8GtHlkx4jOZI6H9zlPwmXg3zK2E0
d+Vo7gw2ct+UmK8ERTSnU2hO396PbD3A2AQ70VyHoLlu1We3M7FqhyCUA3iU
A3b4Tjql0l4YQrmhjXJDZkmbZ5FuCJhQzrSinLl8qGT2uyABMlEuiaBcYi67
skF+cBz0UI7poxyjGZBS/zplDzQo97LqKbnX2ReOs7hXBZtQTlajnGTePKF7
KKkbDqBcnUK56q5eIiB1ZwTOoBxm+F8OZ8h8fJcxBV0otxlQbmel/WRwEmgB
fZTzpSjnOYd289piXvYvF5LdA/OeOxeCA+IIO+JI+LG04rtEPBV3pls66p3d
62AKcUoEcSqx51JP589xKq4978rz2F3RA42Ig3jEQVKYT9a9OjIUIG5Ki1G4
6S+0jfAWPwD+iLMmiLOlta+2WqpMQRTiMrs3hcsmor2TyjztsCWYwvE8BsTx
Sf7IlB6svxD3ryDu04SecpAwDwYN5AlHkSfgabTTVILbMX5TvKJdj+IVvke2
1f+txc7tzRQPET5M8RCH94H0I0lvwQJ5SwryFgulZoG6s52QjjxHCHmOYORT
D4lfo6CCvKgBeVF+0FT+sb8NoII8Sgh51KJK/59zRyew/x4e+LTaAIc0rw8M
+dYCPlvV4OdIGRwkX/K7pd0KzLJVf4vjmuC40336wCIiZM3VQv2fZgjTy+ga
jWgEhh0FBue2lUN83dQr2YvVYLlnuObPVAVotEZcPi42CP5smj2mw8lQ+pnl
QpVeMmj1Or+9WToBv5mbXL99n4Qd9WYZVg6T0LmWnNy4Vg6BHXMLLwYJ8K60
fcNO5RHQyunPcmceAF6VjlgppUGoLaw1FbTrBB4tEr+5xRgUH20e8IjoBJpf
wsm103FQuDp/c4pUAN57zHZ8zGoETyXpPaHMI4CLG7SPeNEPJdd6OTIXy2C6
IkzI80E9uJ82EGCSmIBY7w0jzbl5oHVtMlaKbhwMxusjVulaoXQ4NsDFPg2s
efbzyYr1wE23RfpL4hPQ7M0k6jWaAJEhC9MPuQpBxVhhzCikD5jYTQwtP8ZD
a+Rrvdn+KYg1mmP5dakQQhoX5T5GxYODQ/CnWcIUdt9SM7nhY/DsM8d7ES/s
Oj+v7hydy8P6/8YP4/Qe2L7MecBIIxVYt4rReQt0gPWkDftgHhmCxtSdryRn
gUiWywMdgREQeZqy8JPUBy/oDrV+bi0EhiNmFt1sWM7O1F8+0z8OCluOep0N
KwdzmZPmCixN4P49Qin+QDmY2lmKPffCvHcw6IbLzkFwPk3rQ7+xD4ze1ajd
+dIMvni3fbv8R8GhLuKXFi/GD/tNfXXl7cDLLpz7cHoKasN1nw4kToLC4AVJ
dWI3KKrH/tLdUQTTY1JWpJJUEBQZXhTs7Idptc8aPapNINT8LuAKbx2cO9fx
ill6EDbG3uG8f6UTbs+QjD6Jd4CbgM/75soh6G1MXjw02gdiuzc9vlHaDyoj
LkN1O/Ggb3KHxotzEPBDx2cnk4bA4vlDDybuEbg/ycLPlDwEesHnSg57jQC5
PcPfqm8EKk/xcHI3DsOc1tUDH1aJ8MdPmX74FgGqF2b3NHhjfD/hR5ZnGQcH
c9NswQQCBGZeqDu2hHnIDs9e4+IJuHTO5iVLHcY19oocm8MjsCj4Cwr3kuD7
dmlT2XoC1D78ELZfcARq5Nh+CwhMARtTTQYndm4JH4+Kldk2Ah6fHGl1xqeg
5WnralUYAbpDWqr68Bj3axhphx+SwePIK1NtewKY2F0FmSw8yGRkvv1VRIYR
uk+bDt3G1rv0XfAWLFdsXmu3cU2TIYax2XvPeQLc2MrVZG2BB3xkslDoX2VI
2VttMolx+dW3iz/j9LBzkubD4oS2K+AZJc/dcoIAFjFRit4aeDhLfiLseE0H
aA5/HFMXxu5H6+V9urN4wH398DMVrwYMJy/d+4L5QH6hHscXzAecTFb6B7ZI
QwV/DLszVt9+/JuCJVafsDYb3GS5Gyq3qQQOYHU1YYnTv7H6TNd2y3WfKa5J
TTyBecuh2ZZgC8xb7gY+eJ2D1e23vN6og9Vf9oUqRqx/7xYos3EMqz8Rc350
HqsX+NzWClz3q6KN5OyAboUFNqZWuXX/MT5Y+xKrXxXeqsXt0kZVX+p4LD//
H9exZ4jJmPmP9xWwrzAr+0NQ+Pc+SxsIIbG3NlA9V8qmzWzPy2cU/l2H2LlG
Y3Hs+v+umzbPxtJVzK/+Xed3u7VaXJfIVPvirjKq0DJPptrHrspwwXdz1Pve
2TLXWbhM3ScJW1d0L1ffAi/UV6aory4NHzwoufkVtKE+7EN9KPkmriCCL4Cq
b10/t3X81Iui6vOAE7W+Fi8TqebCKPkZzer2TKo5as/o8DFYKIQlNHflaO6S
RUpOMZRUAAnNaTmaU4GbF/PraBvhAZprIzTXBg0mL5LHWkEJ5cAVlAOqMtG7
+aO7wRHlRjXKjceWL882heHgDMqZqRhKzvTQe2gk3sXDXpRL7SiXHLz0rptm
jsJxlGNzapQca6Z9cdHxMwlCUe5tQ7lnwCh7eeObHLiMctIK5STP3ULum7kt
cBHlag7KVZ1+++y/LwdhL8phVpTDyuy65HfPx8EM5fakICW3/aVK48ddS8EU
5fxiJiXn3c983sizZRBoERdwzBQuOEwqD+ZUOFBxxHrXxUO1h4YgC3EnFnEn
+BKfFKtLFRWnJIm0eacnJoDFB3HtI4VrbrUGXnUYx//loPXwXnzm8RJIRdw0
QNwc9eRMplkdARo9Cmd1qyicxc3Y32w5WgLxiMuFiMuFQn6KIiU4ICGOO8xR
OK6reEBciXcSRBH3aYso3J/+qW50QaEaYpAnfEWeMB3tH5P7AgdsWyleIYq8
QrywpYVJcRxE2CkeohdP8RBzv8kUl6lCGEXeEo+8pfxqoU7Mxz4oRZ4Thzzn
nW3j4Qd241ReRF4Smu2wa4Rk5FETGhSPmmkiHfZ3HYdK91ynJKYp8PPMuln4
hwQ7wl0/Rwu2gP+noyc5L2XBTL+hYbtsJpQ+ObFbrm0UaNSYzdz25YORCEu/
WGAueBTNT+uYF4ND6Ea2uJ54MDhUqb16Jh7kNTQ/uZX0QchBzTbOfBLcObR1
JALr17PCdt8Ur4xCqGpdaM3wKGxrsb3lOU8EnElUThXuE4ica1O2iRyGI979
RpbXcZDgYJX7IKQVLlS1eU+y9IFd9myozSy2Hk4Rd5deDEPCqY9pZcdaIYEz
9bcB6wQUfOK7u6BKAr/tHJwZxtXgfKrC88JyN0Rah1688qgRFgUcOJacCiHL
qdt6zXoKnJsFn21mxMPU7b52Q610YNj22nJvWx+ISAn7uF7KhkRTs0j1n1+A
5sXhOyfsnOGhuYer/D0sb6q2Fl1bjoLpa6+VXVqwnORKNixuaYMtz+66fcsM
g0JPqf4ZK2zev/3lyvDPgFW+2iNmkuFQybB1+KtTP4zbt9W475gAxjPecmSr
aHB/J7C39+sIDMnqSOkZD0JDJb9kj94XyJvs1zvcSgZWvwA/pt9FkPvYRp5T
LRW0+Ax4nHd3QAKpyWmv5jA8Pbgo7X43B3B03UGK54nQ6Yb7cwfLf7cP3VrK
b4ph2h9/v784BZL9B9PY41uhlSvTzpi/ChpJbmJmvR0QbWSirN4xBCUvTpx7
drURlPe8EPXTws7j94SvP5tf/1cpP/GPGrXCkYZrhteMMb+81IO/tEqGQKII
N6tLJxTuMuLZXxUL1v1Fal6JpeDw0ceY3rkX/GVKWp64VIK01ze9pHMtIOV5
VR324EAJx17BxdoKJoIF1zV29cCGiNB80wLMx93mO24pdQNP7VXaO9h59GF2
lGjPgWHIizTpymQfAJ6RT+lh+GEYYhhgCj+FB8+KXdu23hyE5JBHe749IQDb
0yDuz/tGIGe4d3A1bQgcE8u6XkWPwu6epYzOfOz89/cpKTNqGMYrxNIu2YyD
+73B+zSHMV4fKAplksWD0Y57aSFrE/D5eYJR4x0CZDtGvTQPwAN3w51GshQJ
jB5aj3Vg/PJVds5dwPglybjBtFhwCraXeGo4OhLAjofn2LECPGwTuzXd0DsF
ZQqCE8etMU6FGC7YfsdDzBXNb1J6ZIjb/iXwpTEB4j1PSie+x7g8N9LAk4Bx
Vng2ZPwyATJuslh72OPhzlpd4YY+MtwfeBdRoUIA/pNbnjVjZ1VGvgccOQtk
2FLp+tNNlgALoh1Wn3TxoPEo5NAUjxCkn2fkOyVOgFDpBy1HNPGwXCqnlRui
BCEtqq/lRQgwumymaaiKhxuJA8IO5wAyzjBkW2O+8cTro3gr5hvLhTNnbLJE
IaRw79dlzB8ebpe+ug+rjw4bLZx5xQVnpjmEGLDXfxVTbBXF6gOlV971yo8p
bKJzPC26/vkS2zz3A6z+tTzEZATrQ65PbyzPYnUlqff7fbC61+x3lV9Yvemj
VPB63dbumc56vbXdd39tWo6CDscgzfrfB5SP5m92wuq0LYPfXa8UUdUZdG5+
/fMf19lm5/rl73+87xGuMc7IJ31U97k54/LJu+/HFf59ri+mfMXrn7P9uw5d
5xs3CWH1f9ct/lTmdOUKmWqdGTcu++3CvOvffWmYPnrx2h8yMKF9XEb7eC1u
7y/5Vep9j57MPtTkdYmqTwo1aT7sf/WYqq9cJxzpa387U/Xhj6MSQYzZgcCG
+tYe9e3RAeZlmhPRVH1ep0++1XUnkWouwG3Q7rViBriiObJBc+SVYSz2bkMh
cKC5k0JzZ4UTP/FjVzlsR3OaheZ0q/pYMM2zOsChuXZHc/29mqfqC1cLmKMc
yEA50CCRHlQq2Ql/wym5YYdyo9lY68/J/f0ginLmCMoZvQmeS11WQ2CJckkX
5ZKVzNdmAT0CeKAcw+2k5NgTd79jYa3jEIhyjxvlHg2uRT216Qhkopw8jnJy
1FjpLf+RSshBueqPcnVOUvi52cVuMEc57OBHyeF6KZa4+W8jIItyOxjlNo3D
ZrKTAxskoJznQTm/eFB/sfROF5AQF/CIC7r6e7jlHak5gnO/+uacaB/IHaRw
R2sLhTsH6b829v4ooeLUX9NIubFjJCqu9c0cbti3CztH/o+DThQORh4mzTif
aAOmOxRuGiBuHtmtd/x72yTsEaRw9i/irPX1BLZfEx2Qj7g8h7jcWiLaoVNJ
hGbE8VHEcYOWofbzWT/ACHG/E3G/cbFQfCtzFwDyhNPIE8p/nuGtSh2BH8gr
hJFXhIxdNpqNmoJE5CHPkYdYn2a5tBJXDw5hFG/R4ad4y9su3WNy+wapPOeH
X3HJuBF2X8iLKpAXWUuIfWT5VgsRyKOMkEdppsglHDcjwLCh9h5r9lFY4YgR
9CUQQOSs9MIf+kG4Ub3NWgNPgttunRvSm0jg5XA2/0rfOKRph9U2biWD9Fvn
fkFGMtD+3CEaH1EIdMxPm/htSaDaWrP5ugEJ2sTUo5c+d0F6zUWZNf5xqD7Z
MjTgOQaydO8Z9zGNgL8R62pj2QioC4TkWCQOA6NK8Kz6RhLQsTKZGCzi4KZX
gS29SQdIH7xjfo62FNqdrmuEfu6EyF6u0x7Bn2GRGWc9GY717YllFbrYRihd
+TPANYIHVuczgY+vEiApRWHQZ6wUeKo0d9txVQJNUIAKk1og8EXv0X+WmwVC
vGs38iRHwAEn+lagvQvO0hy42SafALj9CbvNZXNAt/dxT7H6BOzZW/y12SAS
lF+aOe7sxDiu6asg97sXeDRZa74wvANcedL2+3r5wFzg+qisohKqBc+W3o33
h07hr9p24pOQzijAzU/3GbhUH99Lu/YGsvZo9Yg5kmAmZutmQ1ESXGLs/L6C
CwLW90UdBhZVwMPMrCXDQoC9Bt9mlqSjoNSgvGbQchiysopVacpagej4hm3D
4lcorCTccu9OAOXGM7MnrcdAmP7NbEBFOkjTBWy5qzoANIo5o/aBFRCT2pfm
n1UAQiRj222HJ0HoMF4Tbz4APlqzAcYfyoDmepyBm3wtWIRcfLcvkgh//E30
7HG1MLmrD6T0cHDl7sZD+rxTsEUo4OQH02bQr+J6kc6HndduS+tnfs+DfIbq
sVyXduhtKfN8FTgF4Q/ERm5aNALH5p3eTd7dYNC3eIG7uACkiXZia/c6Icju
kUnPpX4Qn1x9gRdtBMYvEXuEzg3ACXHVvH09OGj9wsxycakdgnfZM1+NGYKZ
0uHd7VpD4L96bF5AvRf4+0qvv5EdgbVrzBZpLsOwfHf2dKIwDnQiDn4ZsiaC
XMP7nooXeNh33vHq+YxB2Je85yX3nTGI6VQW3ik6Ap9frJgMNAxBn01Dlj7N
BGyRz97hhM1lgCOxfeObYagIhb/zlydhav/Jug56AhTEXrv/lBkP15fW5kTv
k8BNSv9tjhQBOLUlzghp4yFPYYfaKExBZ/suYrkmAaw26GtoWOFB5xvtg27c
FJB3ei0X6BDAIEDscK8dHmq1TE6WaJPhm4nD3mktjPuF/tn3nuNBa2N3MlsU
Gdp5T6d1qxLgY3GzRJ0ZHuxtu/nMmshQmUFPd0iRAKpOj9dUb2HXjzsQOTVJ
hq5fK1GN0gQIZ72YmaeDh9OaeceTlsngaRkkGXqCAGZH5nnkNTA+amy5ct9l
TUH13f4DTzHuByuEu6Zg3Je53B39tloQ5mjFSgfWv0eLm+42UcE8x7XCg/fL
QViTy3NWwepFWzdXeGH+YLhvZXaZjx201sqEj2D1YHc+4m2sbrvlB/ere/MK
mrYiHySxevwN+QYrrG516cPRdc9Rm7iZrbT+807PPmm4YvUCmme4Rax+bq2v
VRmre7gsXfXA6mEtbxev4p4o3NKntVt/fWXNvnmXdZ9hiLR8fSOeql4+ethm
9T+uw6IF3Wv/8b4BtJtXDtvXUt2n0o6b5hIxBQr/PldQ+YYTbdh1/l0HDgMB
3E2s/u+6ud/pePAD86J/1/l6c9mVO6vU+zKlfWy+cI16H7NnyredOXuGat8F
E67139p+h6pP5E8V73io/pyqr5Qn7avYct2o+jCeMN8c7PYO2lHfPkF926Xe
JnxtJoqqz43Yex/abkgAEpqLPDQXqhI0uhYZ2LkUzZE/mqPR3XT7tLTyIRLN
XTSau3j2Ym6f1RKQQXPKjeY0At4w+VpWwwqa6wU010ZFGYky25qAjHLAFeVA
6Zlnqn3ubXAM5UYByg18Mx3xYnY3eKOc4UE543ZUZOY+lmdbUS7RoFyy8C66
51A7DCkox2pRjslU3fNprCDC8lFK7imh3GuetWbfYjoJPSgnK1FO0iRkf5fv
ToRXKFeT9Si5qlOj31b8sxFMUQ5PT1JyOGjSp9NHDQc7UG530lJyW8Si6QyO
axyqUM47lFNyPvnnqBjn/mqQQ1ygMaRwoTYmvOdV2AgVRyKJxFtegy1U3FEk
BFT8tsqALUconNISoXCq/ZV8eJ3fFBXXurX2xW6rHwMZxEFnxMHlziycofMg
OCNu0vBSuGnhJx0w11oMNxBnrRBnaViNMybYh6EDcZmzmsJlZ7OaT5k3yOCL
OE6/SuG49KJVB/fFFphE3Dfop3A/ZMmvxCN5CJiRJ9gjT1iWJB58sDYOccgr
TJFXFDbK34nvy4Qy5CEDyEPql5rGS8baYBvyFh3kLXmE7uN0QsOQjDxHAnmO
plbu7M7OCdBDXnQLeRHb6iDjJ89SyEQeVYw8avGxHxvDTB+4zaUFGO8iQLvM
3r8MfHj4+WUlSu78KPwtijgj/2YcLNvYebqZxkGfYcOJQG7MCy41fH8SPQGX
j6t4v9ScgO9aN2/Va/yEzzfu98eojUNBysLbwz/G4KDfnadeXW2QnWPpn69G
hIWkMxfDW0fgyVkpvFfrIERyTzzerDIMAjPyrSo+AxDvKLG86esoiKrW6Zen
9kEs4+dTq1l1APraSaUfyHCDeG7E72gbbLR2kznzYRJotMPo2D/XwJG7Lvzb
y2ogS0BtdexEPwRbTpiF/cWeK/myiGB2IUx+9tx+f2AKptl+MnibjEKq6nvf
k0GpIJI/qqxyCNu3576sp2Z+Av/gaAv/wxiYTPq6Z/oCEZId8e95uYegtCz/
8l32QIgUG94x+TwFFMXPTSYeqARtook8b68rqObuuLTlMA4cDP++p2e1h9jN
QYxPZhwgcldfBY1GNfwIZhlc7CeDddDC8w3XHGF6WMzcU7YZaLgPix+zdwGj
nW9LjnF4AU/qpRLGoBFIcg06Wqg8CYRb+hOJzO/Bf8fRLvydajBQcu74MoQD
3m3fz+tMxYDziKWrPs8o0CyyPyrFB4OzE65EwDkZRneYJosGNQJ+SiXS7UwX
LIZnebFlZ4PJ3MU7L7Dza8jnP7+OZoxASEoJO768GLQGGpXYotMhZJNlF6F3
ErSnHyuI11cB82/XK3OvuyD517MUKYkfwGscb65e1wj1d65/2y0+AuzzniQo
q4PkLtu0AFwrhFYN9vFgubHxUCzf9oV2kIS9dBYT2HmQyelKwPZEUL2RVK7Z
0wey5x8bWf/shVbbld8n/KogBB/fVD0yCMq+yu3fuHHgzTTHVrKzFVSvLBSc
w/jAvWxisCFqEAQeXZj91dEFvmUlw1zJBIigH/pDTxyCyMRDd/I0+2HqYpOj
UMgovBW9OU47NQzWCb/bRzpx8PuIsVWr3DgYM6+xi2bj4Xyfelcd5xDI2YYd
qw2bgGDXh5vTNEegoEWf4cXfIRA5rXOStngSSqpDK9yxdeJyfzo9EzQMasWK
nIpJJOj0eNsfsjQChIwWEF8ahg0h5VvijKdAb0NBRy8PAUbZ8o8LieGBQV3s
4NzsFPAa3ooLEMfmq4nxfKQmHgZbT555f40M+ZtDBspkMQ5+ej+wqosHheiJ
+ecRZLg6aVg8j9XpfWhqTl/DfKmm7+z7WjJsrvKy3nSSALvUreoYLuPB7+5S
yVsiGT7steAylMDOE5z3aqou4OGU0ptDUgtkoLu7pqEqRoAhekZ+7XN4KB40
TxdYwTguTO4RwLh/hAXndhrj/sTChcUmjO9y1a5W8+veonnloDvmCeZmoZxO
dczw8OceMMPq6m8CSwoxr3CogRum1+mBaW2PsyJW/6lxaN/650iWRxqqnON/
KaRHHjwmv/75T8n38/br37vZyVT1rxYr8PFO1K9/7hR/1PjP+u+n6j/Vkbmy
/n2iY7v76fXv9VTnrd2werDVxuB1LxrZMjW67kWpT5QC1r0o6937xTAdb4V/
6/vcE33X/uM6vafZu4dWPajet37L1l9nTqVS3WeKhqri+udg/z7XbfOFpF6s
/u86bPC6EhD3H+vmGzZb5b1Gvc5FmZ4bO+l+K/y7L4lTv4L1cySp9jHX3rYv
/Y021b7vexxCL2duQtUnxTe5tuunv6DqqywLDhdaXneqPmQxKtPlqAuk6ts3
t3ry8gsjoR31OR71+U6DoDsRLl+gCM0FB5qLWtKS/JpPKgShOcpBcxRMuCEx
Rc6Gu2ju1NHcHWLzVGtwL4I3aE6t0JyK69w5nsRcAaForoPQXN/IypnTeF8H
nCgHuFAOLFhZS1dHNIMiyg07lBt22myOJts74CjKGW+UMzjJJH1T8V4Q/F8u
MVJyyawpPlR3xyBEoxxzRznmSTr28LczHnaj3CtAuVepFLYx2XcUFFFO9i5Q
crLzQNOWa9dIYI9ytbGPkqv+LcbiVZzZUItyWATlMGsKA3f821a4h3LbAuV2
rHuvsNyNYaqcN9V16+2SJsMRxAWHbRQuJNv+1kqQ6afiCE31uZjVrnxYekfh
juIQhTt6RsLnNn8lU3FKdOqEtZ+TBRXXLmxunBkoIQNtOYWDhRIUDjqfIDkH
/h6j4ibNH8nGnIoeICHOLiLO6ohO9WVumIQUxGW5eAqXFVPo+hpu1MMNxHER
QQrHVVUY1VM2DII54j434r76N841mV1jcBp5whzyBMU5AbNWXw9IQ15xD3lF
47wgx+jVOqhCHsL8neIhuo/fSI1e6YVU5C01yFu8pTXPyLCNUHnOHwl2cZVd
k7CEvOgR8qIn+b+Nb+7JhAfIo5KRRx2+tsV6a0M9hHQX3H48hfmezrat5NFh
uPHa7O33rEk4wi/DdCNgFBa1e8eOehPB4d2mF3WEKWjyEopwFR6DxbzPUTpX
RmGgW8dmiCMfRsSNNIZ6iVA4aXJRL5QAxWpxTNt5m8HZOcX2J+sImCfUvCSE
D4FuoY/MQfU+IK5c9y++NAjxItK3jLI7gTtM+PFmLIck0j69HbjbDfEhWxqc
1PLB2jedLZB9HPaor93NtW0Gu6jDq2anRsHBrYtd/uZreHiphNAcWAEOl68n
KeS3QONsohgvdq5gUr5wTkYqFwTSUxuDVUZB4GVp49JSP2yhJWtvuJQADot/
X2ycyQGZlNjEvuUxKDa4Ebv8MwR47jTPnMtux/K8KrFIqxEa4H6KQqsb6PbU
24jYDsJiiW6Ky9MJED5+vkkr9RmwPnPLpy8Yg9FJqUIXHcwPhcwbvxx6CA5+
N5v81btAXHrmuMkqCSq2sOTHPHgMPDGLkzfUByFriXQ01L4U5NpMbcMi7MEh
XuREe0QOKKaNq3BUFAPdVsf8Vwm+4O521SrFbwxwZZtf1G4fh6bZAIJtfRgw
0BEXNsT1waTAjnnjimaYuvrh73zaV1CMX+7SrvwMkczC6nwYB0xtx/ilxn5A
Z72pl/OVQaj+KSrOojMJJ3N/7lkOLITe4w7tuA9TQDPnGrjzQjoY51iONxtW
AD5mil1BrAVwzk/OOs3VQXaaAf7s5gbgWuE7T4wbgr+/vh0h7OyA/gcv+Pt9
WmDqTbF19bZxeLjzkJFTXx8Mu0b/DP3QATSbHU0EDO9Ajv2FGPuGQdh4qw13
4XkPZIVXxTtklcDqD/1ojbN4YFv0vCVKOwB5ghH72iKaQPVzh9UOEwLwvzZ9
3MM6CLveunJwlK7/+0TREUoKoyDCsYPf0WgIKnWFjYp8e8FwZelqdvMYHL8e
qxlrMwzshnufu+3FQblG/YXFoxNwgTVPtA3j0+yBwxGTLwdBIqFdafuFSYjL
X37m/wcPR1b3q8XbDwH/QO5jXUUSZD4P1z1hOQI8B+Ub9pwYBlPjDLMpuimo
z+9/w4ydj9jsb23SShgGzrcTFXzhU1D8rpXkgfFIiS4ycA/GI/XivLFUNjJE
dMfJenESsHPcJRVefjyQbocfrblPhm2WHKeGBTBeK+OWS+XxEOo7/5v0iQzx
TqmiWzEui7B8j9qDzYMAg7b/5zoyPIi4asuBcVnk0c9iZozLE8wXrkoSyKC7
bdtinigBClv4Tnuq4+HmPDlIao4MDLcaS8Ox63A2874lYdfZwFDy8MEfMpxx
vpzvI4zxVySx7g627ot+fSfjV8mgYLy/vAXzBFcnad+rmCeccZaMbcY8QVpv
u9tjrC5zS7Olav3nwP0+1vuIERXu33aaUsfq0hIzBwOweoJORLL8zSEFM7Gp
8XWfOdjeyLf++dIwcYj7eUepgv55R8V1LyryPxuxXmdbOkNY96JH6nY71r0o
/IRk/bovRVXqF6/X4ZLP6vrrNYWsUtbryimfZeYaTRT+rZcxCRP/6zqBJPXv
H5vsqd5XenB5kj7nPtV9VkjzVi9j1/n3uQwvFEeR/msdRJ/Uja5Rr9v5D335
+MQFhX/XWceW5vBtu0NU++K3MjCrrK4Eemgfi9E+rinv5Gn2v0q17z47Z5Pj
q+5T9Yn4pG27lNALqr6SvhZK6+jsStWHVtEnvEPKA6AI9e0p1LekDOPKaI8w
qEN9vgP1+ed6ldcle+PgB5oLbjQX3h4flHcXfaeaIxz9gSdyKz/gHJq7STR3
r8LVx5665YMQmtPtaE4XK8sL92mVwFE011lorv0ZOSO5flTCAZQDtCgHXn15
9nqevgE2o9yIRblhm6K1q0e4Bf7epORMIMoZlkCVXKPHHdD5v1xafP3/uRRM
a5hNa9gLDSjH2lGO6TwlaShsH4Q4lHv0KPeYN38dZ36K7TPKyWCUk903+Y4P
OI7CQZSr6ShX/Q880tj7jATyKIeDUQ770z656XorH1JQbhfGUHJbmRhllv20
gyrnv04823Qe4+q/XLAYx+2pz6oAxRYKRwrfUzjSq33muAzmBf9yZ5fTxnld
3DgVp0ZPP3QvYE6m4prJeZ610cR6qFWkcDC+l8LBxl+hMmOCxQCGFG5qGVG4
eVBij2BEBxEiEWdxKxTOuss+UPNgrwRtxGU7xOXIojAOOicchCGO/0UcF3PS
sfxdMQpHEPeDEPdnPiQwueVgOYY8QfEjxRO0yo+lfE+sgGXkFVPIK2jLI/f+
etQBgchDXJCHTO3PHiLpYa9D3lKLvEW6R8jm9xSBynMSjH4k092YBAHkRQvI
iyzj4x4JvI0Cb+RRV5BHMczHGDnJ5MAMbYWxHjcBHC9VscUdxYP+ZJqoW9MU
tJ5ULTx/hwjqqX0sX3+PAOf3PV8alshAPEZbS/OSCMbEzfVFXARoPD7n/+N2
NlTo01fwY/5WqPJpzzdePHjO10t0pdSCim8vR++9YTBgMFmcWhoA+THf2Eh8
O5jVcn61/jEAApwZqXlbW6C47bwTU9oAjD3te0gY7wCHXRt9vu/xBw1hk1d1
j7D8WIvVGbBphEvMgq4f3+GhOYr9S/qdMWhd8KtmcigFVZkjuxV8yyCgKcFb
hp4M8znbA4/mZAD7IeUaPsch0Gp1myU9qQDh4BbJ4/xx4J0yvNDgMAHuU/dU
04L6YIkzbCI1PwDck+pUarDnnIvjtPV7NAGMbGWh0cW2QCN3nPn05HNwOJct
/iCmDwRDc+9biRoBLjbl0e1mMph8Uoz7Yt4NcyXnFd6QL4PDag23RWAnzOG1
HCfdJ0BTgyC3mqoP0+ERzvXDWO4tMEjcc+yC8q4wNYMvFpD8SGrHd+z545Uj
byVp4kBuJ9soOdYZLH6HF9RUdMG0Xyft37kyiNzK1+rYFQQWnZeXHFSq4HaU
xXCo6TAkmkS5HT4UB61pUaY6ryYgoNmjMKBuEugPBy0lcKRB8hBt7La+LhD5
cOn+BtU8qDD9bp/Zkw80dwbTs+QnoLw6N/WJXBNwt9+1cncvg8K7LP5GXTUg
kN/+O1WmCw6/9a4wZaqD+Id/+mM9cHBlOHGqImMA7t1K19rq1gwGnSG8a5hf
rtlkJbZ6DwHn0h+jDQHtEHzPSn780xTszqbluNCCh+5j7r17n3Vjvqer3UXI
B4EfKzpMZQSYCdf6arm1H5RPNla8km+AZT+36R2PR7Hz/cGNPBE4sCnOiJpk
a4cFnU+7awbHIKpoRVRuZRBKGWKt7/H1QJAck7rfvgkQ7yG779o/DOaah8Ot
mAYgfJM3sURoEl7MurFl0+NB0uxanw3NIIx23TjygZUEirGcUlzY+XryL8nj
K3a+1qlJ4Q6tI4GksNTVXfIj8MQC37Q8iJ0Legwi+YynQMYlbq9zyAjoPBPT
83kwDKP3xALDcFNgmsMyfb9rBOq3t+iKVg+DyfTKhlx5MrxuDbutt4EAXkOd
n2024iHt6VeHF45k6CW6n1zBeKdr2KNXgfEuRjPqElMaGXIlhewqDxFA9IH2
oupJPOx9KF/4qYUMk3Ds++hRAuiNSfsfUMJDJL3CL6VRMqiqfgm/iPFdsomh
5iPG/Ty/g7uMMb/6cu6NzPrvfQqwFWizwbivdeuk3cIyGXDazF/XfSC8Kuvu
ug+4HL5LdxDzqxS5Y06BWJ2j4tu5v+veEqZNY435g4p/+BUjrK57/FRtBlbX
ixxMWP+5dPeTZZzrP0/+ul+u5S1Wdx5rV3f0SlNoe/iAaf17uiND1yXc138v
xG/Z/C806QqvCdMN657TrS3xcN1/OpK+qsUpuyto7r+wvF532Hf523q9dUPu
9LovdRh0r63XTybopK3XOaYJAw9yZRT+rSsHxP9df/2/18m5fu7qX4+bVO/b
dY8Uvv76f+/zWuX4q5X/eK5PGX6+K/+xDnnGcbf2aRMU/l23b9fOes+vsVKt
M22SW2LVz2NU+3L6pyMXC78y1T5yuuEyWkSvUu07j8FS0bZ8Y6o+if2lOrj/
2DOqvtK4mnD1qLwTOKA+9ER9aPD2dluEoR9V3wrPniqMUA2Gk6jPtVGfR77p
yuNfiAJxNBcWaC7i1GeWyny+gAKao3E0R5afHnPtf5sCT9HciaC5GxbZnihp
mwkiaE5vojkVaokowX0ogBA01wlorjE2dBS7lMAXlAOmKAfEEkw/tFhXwjjK
jYMoN4QKB+XTZeuhDuWMAcqZwncbr4w9bYYtKJecUS71+Cw3GV5rBx2UY4oo
xzgY/WKvELphG8o9b5R7t7d9YQg9iIPNKCcVUU7aRrE7x90ehq8oV3G3Kbka
xxUdV25IhDF+Sg5roRwmJbQ1rz2bBAuU23Yot7XMU/7cf5sNhijnWTsoOa+b
v+9k+ZEuyGencGF6nsIF1V61tULHcSqO6Fr/VIy5PQBaahTuWIdQuFPvbDtS
qzBMxamkR3kLGr1FcDicwjWGeArXelMGhL5VDVFxMFgv5q7a8UEqbkYahbEV
nyoGL8RZ9lQKZ1vda6/HiQ+CQi6Fy9aIy6xM+5yU+caB7jeF4xayFI7zNN41
TZKOgLOI+9aI+zjrxOvHHlcDzTOKJyhyUTzBMUp4w12eDvBBXvEQeYXYu2sK
Nvw4uIQ8xB15yBOvrMTvPnhoQN5Si7wlSck71aNllMpzhG0GNb60TEIz8qKz
yIuyM0+/uV6kCpPIo14gj9olU4T3JkYDvlt+h784AUYFcwNDNbFzK9vMQFgC
GfQYG6/vTyAAqyKN0P4lPNwQ3c5w09AT9n9qnMq+TgCDob3KPa5YzvuyRejX
pEGQb/3UWBce5F7oSR07NwS7OHj2/2wqhftOAUZHTg7B4scEfvnfPXAixTM/
0rwR5IaGjieH9YG44cbCPzfqQEd2Yacd5r/J5mYxEofbodWE3WHn1im4HSe5
Ea73QaD5Bd0hnnqgv3/xYsWPQdC4EUGXqj4E9Dg1hd3XisGBidH/66NE8Bbs
wTNbEsBcK0T1fVwa6La8+a34owcG9jz+EPN2ArpdrioXMUfD3wPyXZJzeJh+
nJ1xNiAN4ka2tjqZeIFi7qCQ3/wIuD/Y/OT8iS7YpG8zHmxmATwrTI8+lfaD
s/1+LkY7EhxR1CDyD12AQuamsY6PycC8mvQ32G4SGt/lQmCLJNC0BSQSZWPA
iUhW7agigtbOvaSKN6ch2L5Msd0K849iyWKzyjEQ+eSsYbxmCKpCanbP+CYB
t3jzQJE/GU6Xv9pTNmoHDpdFU48VfQaBzH6jxafDILoqlzukFACNY+d89Q9h
+zQbLjzUNwVcl/LkNTfFAA3Z8YlZFLZv97e6KKRUQtRrxVTWvSmwaGPMIN/b
DAIxD8bX3NpBIMHNldkhF8TzlU3LP46C92pOQu6ufrDPr2c+dKIUGrdUBLl9
LYHpHYumr/8MwmWeZLYDITUQ+mCLYHxjH+woP2fGcQ0PPy5f9q0pb4J9x23N
NtESYQeNhPwjDQI84dnPotrYBtz62728fElg9/AIl9cMEcJTVDXNs7ogWOaw
j01UBhzUOXXg9rUxOLH5Trapbh/wnOZ7LyVaC9Gnng9e9hkHUu2Gr1XqONAX
SB4l1bTC+P38eOmgCVi56XnOJnkQ6m/Ssm5i7QY+Br9nvi8mgTS0/c9g9xA4
BA0l+h/qh+elthW3ZEkwJjvrE10zDHFFlrVOUTiwtMuxqO0nwY+dTEc03fAg
WOSvb1IzCCHOUbXP7k3BrYEkbUP2EdB/LORiGjsEH2bx9c2tUxBr++yUns0I
SNM8f1omPwyZ7vu5bgiRQWx/ZtZs4Qjcu82SpRY3DE/D43VzHpGh5dpjvBt5
BAJubDJ0GR+GwQvvHxVEkeE5+73v4cwE+JG0UNfJjocLXO+P3ygjw/LnUoEv
ewmw7Tu88jyG9WezSJVHHxleHqwfI/IT4MrQPelRGTw4nNlN2Eoiw2XRhn3P
MP6qV+yXsDuFB7fGq29vL5BB4Srp6KNjBPAVO8N2WRkPl8V5Nwb+IQPBsJ2X
D+P45c97c3Uxju/4sZdPAvOr3UrfWjWxusarxdL3WF382gP19b8f8cViqHa9
PvoyP3u9zoMjv2nH6g0clr/Xf26cy727b/3nxj8cUD6whNX7HYr2rnvLJFfd
hXVv+R1XiVv3GV6Rzdv///Ml0p7adc+pFad764R/qjB7Nunvuv/cZy1KWq9P
1pI/rrXsVojh5/9/j+qP10pdr9/y6MpkMp2W/7c+dnRgbf36/16HzkB07b/e
t3tRsnXtP+4zPb6M4ZrUI4V/n+vJ8scXZJsahX/Xgf0XmYUhc07h33Uj2Hfp
pAaxUa3zaHfSwfpEIap9eZtw/Dn5+ymqfcyYuanVJaNNte+vHHIIu2duU/XJ
fvcg4z2yj6n6KrDzSbrJxZfQjPrwDerDjs1CXEb+biCK+vYu6tvHopPj6QwB
EIP6XBL1udDYtjx9qxC4ieZCD83Fq/Lk7S2p0ZCG5ogfzdGTDKLE2+AvQEBz
F43mzpZbyCZ0SwqMozl9huZUp/dA6YZDGfAHzXUlmuu1Ardz3ifyYBTlgCrK
ASNdxht7PxeBIMqNOSVKbrgv8drEV5dBEMoZVZQzL78v2zwPqQZjlEvMKJe2
832rGeNqhE8ox2hRjk2uvXM5cLcFZFHuGaDcG7U0EWzS6ABTlJNaKCeFu1mO
rDD0wso3Sq5uRLn60yFaMHoOB04oh91RDt8/XsVIk4eHPxcpuV07Rcntyk95
N5NbxmBmhZLzBijnC+tZ6fsORFNxofKNkiQLexcVR2gGTsVnNnhRcSd54eBv
rf5KKk6dO8Hz8CLG5X+5RsPiziW9J4+Kg4VbNwp/359OxU1Vtb/l1zCP4XSl
cNacl8JZ2maBLZk+JAhGXB5FXF70lw15nFIEwojjNKwUjjMmqkkSdrdAGuK+
AOK+/ZmixY+mPfATecI48oT2BPxLg0uDoI+8ohB5xb2hjfOnYvDwHHnIkVCK
hzz9LBP1HeNyOPIWNeQtZ5elpP1rxqk8R8x3r3mgK4nKi+RzB9xsgsgwhDxq
BHlUwXFbOvdOZ0jY3Hw4Das7CuH5GrD6cc3ITazzZBDYHKzyWJEABV1tCRm3
MI/a8qPxwz5PYOGJFBlf/3tu/M4v612GwbNRKiF+3xdQ2z8w92sH9ufj2nz/
sg1CfDb/A5DPBvM81QcF1oNgqtWavf9MF0ROeVxYLiiBnl0XCx+r9ELC5cyt
PY8q/o+tM4+H6nvjOFJKlCWltAgVoogsJQ9SKEWytlIRoi+KJAmpKNmypbJk
z77vWcsS2fdtmMVukFDE7445f5nfv+fldefOPc/z/ryPe+dcOFRGVyag/gPO
j1oL8W9ohRhP9v0uhuOwwqoWxhPbCKUnmHTOX66FwuyxWjv7ftBKPXz3yeY2
iDJO2W/5rwR8uHfY0xuR4dvXpMgn2LrHRunKQY6iNDBlFKmaOdQKRayK2VLF
OJAxrb45yxcG/nNc4x9kcCDGqXCTfjsRtkfHH/f78AIMjfSatmf0QDB5S0RM
3hjIxqhpM243Bmd3gdvD6pEQN1eUFWGJzXdIXrFnsDQ4Z/5r8tuVDgrp0rKx
R7NAQLzrtqboRqAz1Kjv+e0N/sf448slcDDwc+zcYsMBUKCbobt9oBm4Kw7c
3Q8DkEc/fP93uzaUhI/RR2m0gkz5+QvG9kRYX6+eoDlqC+G7vsZzhg1DuJL9
bM18MbS6WNtl0XkDt4qBI+fQCNwwZQx4cKgdXB4ZSHNtiwBBk9zbm4/iwVyd
055dDgd/r+/m+5OTAobh0tb3/2J+JKD1WN4MDxF7W5LSlvKAkcehUaiWBNav
PjzaVEQEMbWeaFndcijhNJ9IPVQIJUda9APdh6BGsS0uI74GCpdl9z153w1O
9BWDEQvDYM43XLZVrREW+G72P8P8vWswdTH40CisPDYvO3SpFZaat7OTGsdh
OfCq7eiBMUi4IMX/8WQnCDJuKz1dkAYmJPc6qeUxsB/sMd480AMiZXaHrgjU
wJ2Q2s3+OeOQwighwsmJA4b7n66+JzdD/qma8m7dCaDLccSnPKXsC88UI6PX
CSwKoice9U9ADsv1H4lJg9CU/t7yzptemLqwOIi/PAlftKJzjaPwoMQWVddm
hIMqonrgluxJsNE1JQqZEECx5algQNAAnOl4eiGAiQwf+8YvSv2hPB98vpT9
ySAEHUraOq9Ghs0zdXM/7xDBo95/28c9eAj8Y7CS6USG9hh1nqepWB09K6za
6IUHvdHLu4xjyXBJp2HJFk+E8JA3U4QOPPhy7vqWjXmUaGaYvQgjCWLK3tj4
byQAs1+Whnk3GYxF1OtYuUnAXrn80eIAAQ70R4TIj5LhqOLuZhN+EpxNMR1t
P475z4zYQOcvMmxv2Gt5T4gE+E+eHz3kCfBQedA+A1tH9hiYbdYTIcGHlIm2
EiUCzJ03f7PuH8bp2u70D1i+b2kP1grA8j2VNz6Dsr96wIK7Ixflue7Oik1n
MB+Qr/tb6Y55SAXbJ58T2PhAwDF3B2z8ZUP/aDc23gtvPCk+U+Hzh4XiLR+J
fDXzlPtoRdryFJ8hxT5ipTwHdeZmVjvl+XOFGHVWyvhsY/53yt8LcnfPUDyn
n6iy6ksx+i6rvtT5gOlPj+sO+e7ff6n3AQ04VscD4/+t+tLa8TuddQVaGemn
1h6He+T06t+v/VwZws6UerUr8mvPU3okZfMXt8/ya78X2blKMv9eg/za65Aw
U7jDh3lGfu11O+Cbqk7/k5XmOuP/zFp+UxagmZdbmuFJUXhpmnkMFp+0231M
jWberwbxhJ5S16epE+tu3hvGpcY0dSW/KHWg45wNTR06Ss/ZfBJ4Cm2obreh
uj1rvv5e3sGXwIzq3B3VuUZdyTZDIR/4gPpCCfWFJOtV5r2lgWCN+kgB9dGr
eWVNXdYwiEF9dxL1XXCk+IftYVGQhfq0FvUpx/7zDaWlX2Axm9rXR1BfZ6wE
HDnLmAZxiAMkSyoHVDiSB/arZYEZ4sZwKZUb8yIfIm6ezodwxJmSdVTOPDDp
qRZgKoFZxKVxxCXOKF9hXdNyuIE4Nog4FvVQjv2jUSV8QdyzQtzb95f80H5P
LXAhThpyUDnp9VnvYvNkA7ghruJ2Ubm6b7qKbeJFC4QjDk+FUjk8nn2V9blo
J9xC3B43pnJb6W7n1t5X/aCLOB98lsr5uR9Pj5+1INDkgv+8VcQG+nGaHIl7
bGp8fGsrTe4EO2SKF7n00uTUyqe8JPvgfppc82fu5N58ZYImB0vcfPyC7gzT
5OYdxYonxjtLIRPlLNs8NWeHO58p/mtuhTCUy2zrqbnsR3j6zs+lD36iHMeh
HE/kFTqidH8QWlHuD6Pc7z1krq70lwBXkCcUIk9gfHxVzEZkCP4hr1DWpXrF
s/pn9XMiI+CIPCQKecgttz0WimxjcBl5izzylhbBqxz6HePAjjyHgDxn0VXO
luv+JI0XdR8uCn4RRIYvyKOckEeVRN8unu3XgXiJNtskjA+1C5xqvGex8SiR
sso/gsBkf22yg5UEM4ctGsO3Y/3ey/ky+/5D2DZyo1lWjwgMBx/GKG/Cg6tv
4NEbG3whbuD2gFoGHhTvZYizOeCg8FpenlVhOODw45sdeAZAM27Xn96xdmhJ
uZmUyvEFQk6F138M6waFPNHrlQ8qYA+vFpxZlw41rg8zcx+3QG0Rx5fpV2NA
X/Dok6VGDsxHcN34/uAHaKvubH3yqBfmi4YzWASKoODWuu+bdEvAk3XkrOu+
SdCQ/POWb74Uu/7awhnNqeAu9/S+Z2UjLEfE43lOfIfax3fZRus/At1NnDan
QzdEWdvasPLUQnCZ9taCJVeY+qYpNGrRAkwZPpm2kU1wmNH/N6uMIahe+Ki/
/HMYHOYDqn7JdsOmQsbrX3oOgOFe+9vhKs2gmRUZkb8yDMRrvw/yyyfIN/Ak
xynLkcFH6OnxKVEcrNuefFKKfjPQ2W82aztUAnRtm34Xh1+BMC9BUljGBRi+
Hrr48ycZdnP6XU0XGIAPIenEW3U2YC9QemldLh6elLdc/vaSBML3tlyft/SE
qp1PdpzePwTSFbbiFodHgD7h7+yGQ+GgENCha2gyAHIVw+svlY7CUdaQjWSX
ZLCSW6wrW/4O4aeF3+KbxiD46wenYwp5wLZtm2IGjgizcdZ7l9+PQ7/i7w0H
R8sgtaemaJ1LNjieULg2eYyy3/uma5ESNVDBn1v14UkXTPMXsnIlTYDAP/2O
A5sbofDnxf0c8URQd7ciWLNPQqdEZ6QFRyu0+dFfOvpgHMY4Ba39jSdBwNz9
4di/DihpPsVZ5pkMoUIDZ9iTJqF5X/Lp+egeKNzX7GezpxrWvxZMURudhN5i
L+XpyX74/Fne6rJLM8gWQ0vvXjIorWfe/ezOAIT6mRRW/+0AHseLjDPnySDr
B+tOBg0CRwpfzEUsLV0tZSyv2pAh+WSfANEHD+H9Lvr0SjgQFBfJ+upHhlOm
H68l6BBgwqau5gvGdQbTElJ0EhmK50/vSRgjgMjDcBYjk0HQ1pHV9iknw98r
5WGbr2DrNPu3aU6sePCJMX32r4UMt9N/ZZ2KI8L397dudLvgQRCnq/JgkAw3
hxwt7/QSweT6r8eMDdh65CSPzbNxMrxnrTw2vEKEY+V8pw0ZMG+582g5E/Ol
8/3HZCS5SMBoejzm/H4CDMHMmZgFzFuOevB82E+CZ3ubLoIEAQr/I3v2LpLB
esOZ0GuCJGjMMfBkOEWAHIYzPa2YR61b+p5WfpgEvxgmRcYVCVCuFl/8G/Oo
UrrMAS0s91WVLiuKUPatqn3RSXkeu7U+JuS3KOX3ZULMOyh+ddPHtAUbHx3f
/eUIxgfd0MVrZtj4CYuRQ5R9pcycGOiAct9NVG7SmeJRjd+8KL8T/CmOd6L4
SXjTmT+U55rICtamFM+JvTw3TxkvvDITS/EWd4ftq341rfd71Zda5ZlXvShv
PGHVi9hPUT0q+yL76rjWuY7KPnr1U2vH6f6srPy/46ievhHbyLhefu3nev+t
mWJuviq/9jwfMnINtk/7ya/9XsOvWm0bPQrk116HfwFvDommdsuvvW79S4YN
OT9+y6+9zltDyz1Dq1lo5qW9+fa99VG8NPMYeXa0fc8lcZp5l2ZTVKr1UKCp
k2e36KSHTS5AMKorMVRXi7NS7OsC9WnqMMa2gvDW+TZN3R7o2Bef62kJC6jO
z6I6Z8hNPK/gYgtFqC+EUV/w6ergW6WdQA710TjqI4cVosbLQDeIR30XjPqO
MftXppbmG5BCfboR9Wmbxoy+X6YvyKO+9kJ9HesoZlfsEgiNiAOvEQcYdm60
PyPwESoQN+wRN8pvPmI1vhoBOxFncE1UzgiaTESMHYmGKsSlQsQl81KnidyY
eOBAHAtBHNN/Mp87MJwMvoh7boh77ri9fnpB6VCBOKnQTeXkEZHlPW9Ns+EJ
4uo4J5Wrd3NtWLaKFcAvFsThk1QOu0/r/rDOLIHBL1Ru0yFu85CvDRf8rYBE
cyrnFRDnLcmCJzS0ftDkAsFeV992pokmR252dqt+yO6lyR17h/XhKTNkmpzi
TVgfsC+qnybXFgStH3PR19LkIC9jFs+G24OwvZyamz6V1NyMisk5cCaZBGoO
1JzlNqTm7OfIW5Fyf4bhFcplsVPUXHYL4mJ6bD0KRJTjPSjHp9Ndrkm7jgH7
Z2ru+6DcVxp2NXO6Og49yBOGkSdEbLykzrF9AvKQVzQgr6Ar5NIPKp6AMeQh
9shD1q8X/t1yYxLSkbdcQt4ibX5nVmZ2ElufUT1nI/Ic2/PPt9Q9IsNG5EWz
yIvOdPzy42sn03iUVO/z21YYbxmIQTol2HjuxrPPT2Ljx3qWVSyxcbovlpXn
t5AgkIXpocoOrN+L1l3KaiVDn2BS1jz2uW3vM68aMeNB3jz09iCWX7U/ZWf5
M/FwK+DzbZsnOOjOcfR0xvJxSSvnlMbuARD7OvJ4cbIduEI24hI0JkEp4KDb
rohuuCZ7gpTiUAGbk6eZor5MgFTGytAfhxbI7AsPGvIYgyjfuzFzf8exddMW
vm+2P8A/Z+Uen2MvDL+v1/kkOw7uToYvX+lhdcOiM2XJNwnr9KNrbfXHIDeG
K+RbayqkBkXds6xvhJ+V/yl2nBqF7e2LB4VbP0KJ4UJfwYtuEFFu3MaUOwxj
9ifi/Bieg6bbpFawcwu0nRKqW9QjgZ2a8VMPJUNYqPlzNgY/DFfS5EIriAPw
Q19VaQPLQXD+pqGQ0tgMdD3ng/c+r4KdtYmOaa+T5Z3f1zQ6HsiBDp1Aom9K
D3ze+E6u8c5mcN5xZy5wqRQM320Y77YNB3/lX9snai/Axl0ptetHyaCq0eSa
/b0Pujf2+qi12IB+kyixuBwPL01+ZQVENMNm+9quigeeYNhi7VBzaAgC+JYE
wjVq4Uq9+Lf9QuEgeZLx5RvzAVCrX3j1ougbqKT11Ka7JYOCSGP7VeZKsBVw
HYvTKIXNrr9/DSjmwcL45vnWQSJUcnt6V4sUgvbO9s49Y2WAkx7w2+OVDXvP
zcUOy+TAwkkmy3TJGoha6mkVd+oC9fzJC3LKGTApZ8WbiXGpq7pCYQ/m+232
Bj5HGFNBuOwNaQnjWK6ujBHzw3EY3rpVXyPgC7C4njF5vNwB4aodI3uCk4H/
Bf+fX7+iIW2TkdKzmB4IPu4q0rSvGowZzV+0LkVAef6SsRW5HzjMpmS0XJth
148qOnrhT3BY5Th/HsZhmQV2pbDFDhBztzrOejkIdr54UnAf47br7ug0OuiF
0zrqrccM/CD9b9ZODl88BPlV9p3DOP/26CcLuYue4Nv2waQRy4XhpvkYEpYL
+i5Jdd06L0BYKuZ6DZYje461dPpgObIx/Z6it+YzOCzH4CCA5Y49/cepSCx3
pNg+iE+q2sGrxLRvmlhOFeo5uc9jOZXDnfN3LPQ+GIdI2NtguaZwa2LDTizX
RJdr0m8cvwNLqf+4ZrAcfD0xE3Ufy8GDC8EZOY8NgEvjbNsJLDc1jnGc0sFy
s1JzQqXN7iJw1FSohWE5K+ymduUMlrMGt6p/POZThGSXoe03sVy+ab+yZwOW
y/eK3joFTYlDwfnPm79hOS5mtId3Estxz1Qd41K5/eB6teLtZSz3R7lCZEWx
3HcTcZt818AKr7u+kiiecKx55hHFExQEWCOMfOfkf2v9+U7xCoE/oS8pXsFa
FvbMKK5Hnmtn70aKhwxLMixSPGR26tSmyCeF8vtse51XnyeX5F2ieMsWYgLd
Cd938gKWMguUcdWcS3Grz1/5Wg7tjrwmH8+1sOpFxyS2rHqR1dGYrwsX1stn
yFE96gjyqMF5qketHe+/FV0helLz1Nrj7Kn6svr3az/X0zp5mjK+9jwXj9rd
poyv/V4doW+sKN649jqIKdQnUPbvWnvdSEVyvZTfBay9zuQzqUWU/weunZfo
LyMS4xhv186j6hG2iVLMk9fOeyzrW6MczKvX1kki4/1ztzAPX1tXbhIHBU5j
3r62DhmOX4ioGSPT1O2Q6vptRgNkEEZ1/gjVeX/1R/77zWQQRH2xC/XFR8Gn
AzdKyTR99Fet2ifuCxlSUN/5ob5TfHNhS6UXGXagPnVEfar7ScdMyhI7Pupr
EdTXX59oB949Q4ZsxIEFUyoHYvL+zKdsx7wdccMUcSPvZnzsMdwkMCLO0CHO
JFValpSET8JuxKUoxCW6rc+YE3QnYRBxrAJxjNAwK+qwMgHtiHvaiHtZgkp2
9kETAIiTJVJUThZI0Y8Nck9AowuVq6mIq3jHLT9NHMZhPJXK4RJhKod77fZ9
40ocg92I27knqNzmq6UPDfUZBc9HVM43NFM5f26+ZG7H+DBNLrSpbzhw/CiJ
JkdyOwK/Rn3E0eSObhAofl6YpMmp2lZJVuk42lyz2jRhINM/QpODfp9elttJ
9tDkprVbkvFGzWbwQznbYUTN2ePk/ET8vjogolzWD6bmcglO4W/oxkpIQDle
iHJ8JMy8TjC0DApQ7veg3E8Re36JS+IrKCJP+I48oc2mvPacYC7cQV6xhLzC
29umpP9LOjBfpnrIHeQhrjfsVhTXJ0AL8hYr5C0CrpIy83oRgEOe04k853Du
Mc+MSF+gR14UjLxI+sh+J70DtjQetdQhx+F8WAhGQlXj6yRJwNbCrvv7IgGC
Mzu3lW7VA5Otj5+3KpIgc/boaLYRZR0attgQSIbU+HN7pZOIsGUyhUPSA4/5
bPbWQLNJ+H2yOM+biwCpCZrzh7gHIC6kfKjh5zjMjjAkqD0ZgK/VQ0FWFzuB
XqCn/Rj9GPBLi3v9O9cDU0z0/ZZulaDBHiXPyz0CLUtcxLlNraD6sLTpssk4
iK2bz33FPgS/xH387xjUguPlaoLcy37Ql7ysmt9BAOPf+tF0DKXA1q/UGGpG
BvOBlPe8ZwZhx0lbjsflaaBffa+KCVqBueRTxcfdfaDRpnjPSCIMrn0p3f9Y
DQd8d1O0d+q2wtHuqOvayS9g6miKutdoD9D9Wy7yii+GPTwlKvQnjMF5+JBu
V0YMpOPOnteeGwKbIWkJM3oZKCkcs3J0zYaejO/bGUvH4RY+OOnE5U1AJ7tt
yeFrClzBHT5Ze7gftO1kswb1DgKumO7kf4ktUKIjlmuV1gfNk6d12dfrgDP5
YpJPZysIGt1UM/nVCh+I/V65dHZA98xHL794GF4ESH75cWkcIm8aGo5zeGP9
K9Xf9WcEAkEgMj+OAH+sfq0E7IqA2dKzt2VO4uHvByFnbVI/vErzGLUqSQEr
XZvJgN21IOR52em/x53w11WZtYU+HxQKGFKfNJFAqn+6m6WiBbzfnR0QMigH
q/u1Ff4XC+Fck90/Cb5GOHjkm+5wYg1UiXr5imD1q/2KTX/IphZOl6nmFZ1r
hLhbuzI+7COB1lhzZW5cJQgnnObq1mqFzO+qDx60jENi96fJ25HlcGY6c0Xl
VCf4sBVulalNgzkcz+kx6RLIeiA+FzXYA6rCBYrWh2vg2MXf6irvsPO7eNWu
ifKe6nofkt9MM8h7yZh0e2RBWjCr8Q2nAdg4HLQvx6AT1i+77V6QTgNbkW2y
qsmDmIetzNq/7QVtj/elExwJsKn7aN54FOY3GaT89Fs4+KjM4vmyNwr+tB/d
8NGEADeWO4+oBg8Ac90FPM+9MOCarph/94cA3jy3YsKfDIJ9aahO0PogcA3Z
r7fTmAjaNZ8ub95L+V3ht/PVBj5Q0cLMkp1KBMUU25prXniIIqSVJ5m+BA3V
E61heOzv3W/s4evEwzFvzoAsh6dw/Y9ahxojCRzCmQSKNxLA7gTzDh1nG/iX
8tFoPzcJbjn4L7gfIIDlJYWeQ3QmoMdXxfuAnwRhJbszRo5j/jDu0M3hqw+j
7/OibYRIIBNVs8lfngDKdQeE8h6pwak+idkrIiQ4ZKHw5LsSAYQmyto0+WVg
paHv0ycs9zdOb9kRjOX+4MSOttFHAlj9HL20nXJfzNhX7izmCWzBif8SOLbA
Zlv/Psp9sU9syQuU+2Lskmnp/4nOyEdlM0dS/KQ6577oqv/szz8T86xB/tmW
QiXKfbdRpiZOyn23DYxfr4Y8/iyvv5S0ep9OGFQqV+8PXr8VEy1uID/yVXn1
vt6DWsdVL0q7c2LViw5vW1z1pU0vqPcBGULxahFRGafWjlu8od43XHsc78++
rWYZ3DSfKyp69xfl79eep/IVkw7Kfcy134u/V6SEct9z7XUIDeGNptwnXXvd
Lm7SzHpF2S9rzXV2CdVyycU8au28fD5EDFhcItPM49vsLPbPf8g08/5ayc8u
B/OlJVQnhqhOkov5O1ZGyHAN1dVjVFdvxhp1VrrINHU40NHqz/yNDOWobhVQ
3VrKTKtejyGDC6rzy6jOK1oissSfkoET9YUn6ovZTZfPNKuQYR710RXUR0G+
RN5YRjKsR313EPVd1kkIOpQ+CdaoT1lQn5rdJfE3XpyERNTXs0PUvpY3/G+J
rnMCTiEO+CEOSE/KvKu4OAHxiBvbEDekdY22ByWNgwziDBviTAmT7NH1M5jP
IC6FIi6xe+gzHtoxBpKIY56IY4LrN/9s5BoFFsQ9T8Q9wb5FobOYXzxAnKSz
pHKyJVCZnnR/CHoRV3cjrhqyMfn9+USE84jDzjpUDg/JtimonsZDIeK2G+L2
oPhYcSsLjobzzC45ea+T22hyIVj7avJn6yKaHLmhodlNeb/32tzZ3ToUqRKL
o8kphS3mgzvNcTS5pnClRq1/Po8mB2tlbcz0OBsgCeWmoTg1N5/yKgj+ZB6H
Hyhne1DOnlOP0/mlTQRllMsLKJcn96V9SpvGgTfK8Wsox7lPhdd+te0GlmPU
3M9Eua8UZPDfm4ttMIY8wRx5gs8fdq+pnkYA5BVWG6leEZv/3/pnYT9g/SjV
Q5aQh7T+5x0uuYjNnxzVW8qQtzwTxpfae+RBOvIcduQ56oc/MIaZxNN4UXT5
Ue7S869hCHnUFuRRB97Z7rOfJUNIkVTjbWw801n55zVsfPJT73C32QuYtt4x
FJ1Mggjb489TlgjQuZNP53IQxg2uAqOWmyT4kl8lnO9JgKcOHucHXCeg+L7n
gEMfAVqGis2OaA3C89Oa/90sG4XElfBIr1OD8PnRz4jYDT0wMCC//2f+EGTz
3Jpyie4FwzPBvWXP6iDWM0EuzAvj08BGkVrRNsjs3KQcvH0SToh2GDZJD8Dx
mq+/LA7/hN3C0iHzJQNQtX6PSK9gN5Ci0z3ZjcrAELelBXchGTQNGcX+VTcB
OzHnxmBaBijQKV9a7uwG+2MCtYxTJXBPXM7nHc9nGM/fM2CzhQjOd+nSGrWw
dUqnEcf4mzegGelv8GMvCRQkfiWW7MGDIA/TzAlfK0i9+JAzUbcffIr1Pt/V
TYNN1R915GU1QSFL7UYodxbQmSXbyx7Mg3/ehgfbnkiDgn+2i8ZgKkh/yy4o
gAbYIvT++6el05Bab+aXvh8PvIutJ/7tqAGFwyvejOK3oKr7fobkE2w9NRlu
diEyGG603opo5nwKzpkzJq/yvoDVufaymLBOGHX2NDa/7Adxhh51Z5XIEM7t
zdcXFA035h1jv7NFwriOXXVaJgn+2T97JkM3Ci+LdJ3NBNJAhvFAR+p8E3be
R2Ju4Aiwk720v9cdO7Pq+45MMcNQUGZ2MJljALKOKc4fkqkA3CdtnjM15SB6
9lTs0okeGLbUr/TF6tTtJu9+5d5eyDxUq/XmUTtEvlnn/rka8y+3J3H7mYbg
7Q1jw+lXzaDRcor1YHMrOG9bWVfiNwFJA2SObukGWK8mekiuoBM6rBZ+2BZl
Q2KIia9aTQ38UtoVsOtqLzgah98iy9UC/S1T/7rpb+DyvvQhszoO3I/P4job
WmBrYPvkt++lsGAo9SY0bQBwpnaqB7d1wZ75lZgl40Iw3Ldhgtg9CK5NIeMM
Qn3wM3lI+qF6NnRH9q88r8VDi1Gj5NRnHDRk8yvWHEqD0+rnzVjcCbA3YhfJ
p3YAfr/b+tXnWzxMRI/t+reNiH3/sCXd6EFwmDSQuNL8GUp5FkNaHYjQxPr9
xFN5PDwz/aJZEhECmaq/ai+VYute+d08LTF4GHth1c11yA8ajA6n1ZKJ4CTI
+nVkFA9vHwT+3pfzChQ9vRJ+spCgbHOg9V6svyc2Pzq7yfkpjNWuw1XtIUHt
cbbbuaIECPvhYnVexxp4LUZDFw+SAKwMUhlPEuBZgQAnN+ttMHol/9zlMAni
srNiPBUJ0HE1+U325ctQNxR53BbL9816AdXXsXx/tkmA8b9yRSiI+UV/kPKc
0taU/muU3+Vp/GFcqRABBpmWdZqU//OE6Di8p9z/ImnHVHzkhNDC8WMa2Lhy
6i/1YIp39e3dWhk3K//sExubKjbeknid3gsbn5EUuxC184d8zO2HvJTnxr3a
0y5Tnhu3cHj+svWMiXy48Hs2is/cknj6g+ItAzvxTRTPaXYLWX0u/fPbqNXn
0u+cV1n1paNbBFd9ScH08qovrQj0/P/xhW/ZzhPTp9Yeh3Npm+e77TtoPvfc
lP3WB152NOf5WDh/ZNW71nyvzPFAWcpz+2uvQ9mAdTLlOf+1182grmsn5XcB
a6/zvdStqRKYX62dl+U5eQHvRTLNPDYJ5gvozpFp5l25e3xoZpxMUydmS3vN
dHrJNHU1zP5gYqWCTFOH4jFCzvwRZJq6bXgicKnyPhmKUZ03oDrPT3T4Ny2M
fS7qC2HUFzuMaq7VNE2CIuojHtRHxRv2zcbcnoR21Hc/Ud/xe3TnbOiegKuo
T21Qn/Izr3tsLTUB06ivq1Bfc9Pz3lN4MA6OiAMPEQce7zrndtFrDEiIGwqI
G45DxBDP56OwoErljD7ijKZBzAfLCyOggLhkjrgksNXdjoMwBEGIY2yIY5ma
dRpMclgdIO5pI+7tHX+vewKbt3DESUPESfw1o0SXwQGYZKNy9SPiqpWDSEZk
Vy+YIg5XraNyuMnyc4YStAEgbucibpfwMDDsXamEBMR5GcR5ty3te1xjJmly
4a++f6iIGJ4mR8QW0v2JJ8g0ufPQ5VwancMITU7JfRTg48MNAfM3aq7R2VFz
jfHYaINM9zgc3E3NQWctag42MGVtlmilzU3/oJ0xThWd8BrlLG8BytmKpUc2
ZlmgjnKZjp6ay+q/5mqXy8dgbww1x51J1BwvrP93l/4DCVvHUnNfHuX+tqfT
Zr73BuEW8oQm5AkO0RryV316oRN5xSzyCn9S3MvSwTbIQx7SgDyEV8616Pbr
BqhE3oJD3lI2/VXJda4C1m2nek4q8pyRw5mRv4czaLyoxP1xjvDh1xCIPCoF
eZSHxzUf/S/Y+kWZVSeShwRZuUd5Pgljn7M5N8BeLRKutQwzPL47BNqh70Rv
/iNC3Xz8LTqOy1C5Hx7ceD4EWzqy2yywdRX9z3/T6xrHYSE5vHJxN+Zpuy0/
eYtgnLzJcaigZhg6C2R4L1tiXo53Mw1jwkHGxOmqZScCWJVevzBe0g+BfK2f
WwWaIfA0zwZuDhz0PM5+4T3bDuE+u47P5foCX6tsiuJ4GwhrZrlEuzaAbubc
ndQoAky59Z47P10N6w6ar594UwFioP5Jp+MbGJrNz0hZhsB/Tu+vqv/IBt4Z
TxeZkEHgZbk0lPt7BEqe+77RORUDGz80bz72eQyYt16S6Fqm/K7lSCm+yw/o
nMwNj361BToBnLfN2VL4WzXoEcLkCHSzU4FZRd4gM//3Y8YwDtalneS77HoH
6ELEVBdvuUFD8FBjqucgJNNf/utwWxd4vxkSjuA7wYKzLJMvthS+JlnDF4nr
4CO0Q2STDRE8mTYzbLqHh6zjeKnBDisI3t1i6C+JA59w/f4fff1wn4fpEH+1
G9DFXnCIlO4Cn07/HrfCURjxyb33/FcgNOQkP25vrwZclMEC2bMTzh2/L+Up
FQNWVvhii+AxELOrL9nXnQ/v/5jl+vFlwELKYPEb+i7YRzaoOvp+HFQaM7k+
DheB44gG+6vzY6BisvzP2GkIFEnhtdXe3yBcaqL3KWMthH/l9fjigIceiXUF
cpx1MCzsdUPRDwcebQmvymRwcEz2VtXuN01wjIszjf7aMCj3pTqwb+yGH05R
8dsD20CTiyM1I2ESWtS4ue1t28COra+Ex6ELxEwzkx6yfoXvS+Mee982weHu
olhF9j5wPDUpmnGhHlIZD+bGGv2Ef5H8p4nhOCg7aPCwaFcbfBVRKLcKqoK3
TxrVOpYHoOOYuyavUDfceX3mw+bQcjBtH9F148UDC/PRWjm2fqi0qGsNK/4K
j/Yrhv1mIsANur6PDxkGwCIpuqs1KAdulruGCeQToD6FgDuzaxDYa59+F49L
g28ekUXkU0TQ9FI0UMIPgo7nO9+ChHhIkqN3PvqBCDkur4+9uY+HT9bqjX7M
n4HFnJE41onN6+6YBHINHjB9a+a+HQyx/sS0XHoShOiU/1i3HuvHl8W38S7e
UPeeVVd+JwmYh7SjDhwiwIngZeurGs8h4yxxauYACSQ21mq8lMXWRxIhf9+D
HagoLQcwiJCgWNxg5ixlfwYTieV9dXfBnu60tCGW45bG+Z2pFD95r4WXl9OD
FOuV5F/YeMLmPYEvzhKAi5irO3tIGXYODDO0Y+N3XlfLXsPGlfMSVQ6lisLO
w//KAij7Puls2rmMHef0z/q5uzg2sI+rTKLsS0B3f0UtBxtnOfx28XARUV6p
/SUfZR+DXQc0Byj7GAyKcGpR9jHQqtvCRPGWQOYtRyne4iA6YU0Z/3R0oY7i
P6mi1++t/k6wibi6T0JenOHqvgrX791f3VfBb/bI9rCom/L/tfauepRFnMHq
PgxTDs9X92FYO75DUfy5IOGk/Nrj2KaKru7zsPZz190ZUn5z1l1+7XmO9B/z
D7TKpPley0zTwSb/ZcmvvQ4PGFuURrHjr71urmolS5R9LdZeZ8kXb/5R9sFY
Oy+Vl0YZKPtmrJ3HDF0vnBG2/l077+9r6XHHhsk0dRL8g7nWuZlMU1dh62X8
ytLJNHVIoLe2WXAh09TtwJyx7JAcGRJQnWeiOifIjDXp9k9COeqLC6gvZCpJ
Z5WNJ+E66qNa1EdPzopV/qmZgAeo7/RQ32mcS1BezzoBd1CfLm+i9ultrbPr
WQTHwR319XfU17c+Hdn5lHsM5hAH4hAHfswerefvGoF9iBuqiBs/nebYA+8O
gxniTOpdKmcCXTYVWOWRoAhxSQ5xaUv3r1+nvxHgIOLYXsSxNp9Ypfkng1CF
uNeEuDeoFTfl4NUPIoiTvIiTua5fJ74wd4Iw4uo1xNUbEusYJdc1ggXisCni
sOFLsSIXqQIQQdxWRtzmZScxfYwahy+I81bZVM67izavc5fFA3kXNResoqm5
kBqc+Hac9RtNjuSmXGYLYcPR5A7b9MjihFAnTU6ddajXkg4co8k1t59i10JJ
XTQ5yMvO/D34SB9NbhLmmK7dTh6DfyhnDVHOPhy4/v48Qx/kolx2R7nMXV9w
a7dQFUigHNdXoOZ4sLKpaqAIGeRQ7jug3E8LVRVNej4CM8gTFPypnnCb67Zg
vwfGD+QVXcgrUuyrf9V398MQ8pA/yEPsBez/Y2BoB7oUqreUIG8Rds2+zdla
S+M5Cd7sVrhHeXAFedEl5EUrX36u2GHroELkUQnIo5SfRdmZ/pgEJ19G76oJ
Ith3ZmYsDOHhqMyPO1sT8kGvi1fg8vthuOY6Y38yYAj2nYvdPbEUAVvipcv7
ZUZg/7MRdolbw9DCWcdlenUcrrNYPbIgDYED26UMvngSmOfqNVuRSGBI8sgN
2kmEl2G5spcTB2FS8vPSNfUBuDr5OuvNlQFwzG57ur+tAyzGkpsSVNqBV6jq
zsYHXSCD5x+iry6CqUWCR4RKJQxq+ymavGwCq/6Jd32Xh2Fv4POXnz6Q4bkq
/qFOdCWY8taQ/400g+HxW4aPIofBS/vHRzeNAvB0XfbguT0MSk8evo7Ux8Hd
Z6J1kw8SAZe3U/QffyGEp5bcYL1cCS0mZ3WFJkJgYWoqj/lAO4jxc+D4Pw+B
uY17f+kWdxhOuLvnTvUAOOcInOa+VArmv7OMy+keQUm/bRf+8Cj4bBiLCF3/
E/ZeCfyKj7gPzrnHFMQ0u2DokdgmNucUmN4RG2WXbA36CSyv3TEPd+45EX1j
cRSa93LNjsw4AZ3Y8VwX7SIwDP9VsOHJGPw2S//sRfQCzV6HSrofI5BaVXoy
/EEVxDwNSemd/QR0Te2VUgu9YOVE2FL9nQT6jq7B4bVfgG4pNdAyPB4a6HV2
BRe3Q0iFj53KQhY4t5f8drQYgKn7Oiq3rn0Fg/9ylAs/lYCcf7tW+ZdJ0NAL
vUHnOwEPa9RKvO9VAuFGX1m/ejOE4ug2fcHqQN2wmJjMXg9ltlX4sLRB8PE2
qFTH1qu8iWYfTr5rhpmBEZLPjlHYsC8x33z/AIwdS6j6G4rV77OxeYtYc1B3
IQUtP+mBl5+kv3s4dgNv0C+dzoZyeK/yWCv1VTs8mLp0S5CpHzbc7A3m/NII
LEPpu1iVmuHOTy8FIcrvUQMkTPJr2+HEkdPaEgL1IPvUn/E95fnavXP8GwJ7
YO7xxJVDtVWg2pDY8dQRD9LMf+R9eXGgLdSQ32tUDtrpKqWWegSY+SBr/Z/z
ABx6Il7F8LkIdtz/j3MIW4c8167PS3cehHB1fnkztmw4d+e/KLWHRBhxFZvO
l8SDYwePS1h3Csg3j4STvmHe9e/AK58kPKTxEacEjsYAU2bvlOhfIuQv695l
/ouHcLoDbwZDP4FqRsc8HZZ3ups/fK89iOXp54fT95r8gLl+41UJIRKMsg9H
y1B+L9bEdZDk/RK2/jp8R0EMW7c0bLI3ViWA6pytoI/kYzjT+r37+DES3H25
w0vmPAEOHMLPeP8wg9cp4aZ14iS4Ncg5H0HZX9Rgpr3fXQ9Y3V/FxmDHGTmv
MbNAec+gPfv2e1JKwMk0NuJzlAT94rlGpirY945wvNUjfwBOHXuU1ULJfXHH
IX0s91tN7znkvv4t//ZieShlX80Fs6e/KftqTiSYfxvC/OFyinEbZR9On6eH
GCj7cBq+d9Wk7MOpkHu4m7JvZ4PGNlbKc001dL9KKft2DpEiT1E8h2OLwAfK
uELx9WZl5v/k6xiecVL+v5T7MKF+9fmrwBIONd1n8qxR1qv3AbefUl/dR1T8
ivnqfqRrxx/1T/yMDjCjOU6X5eXSlf/zuaIFv1f3i1h7nmf8OpckFirk136v
eY5wk6cpgzTXwU6CUzvgxLD82usmtNl7lPK7gLXXWapDliMR86u18/L5yQlj
m0UyzTw2b9jmcxLzq7Xz3rmb1UqSRKapE85tKf2BdWSausqKu3u3JJoMZ1Ed
aqM6HL7yMNzCjExTt3/bY8Y2cmK5hur8Aqrz5qTpVNZPk6CG+mIY9YWd0iTT
WYZJ4EJ95Iz6yDzwZMfMqQm4hPpuEvVdYMOfmOLz43AW9akk6tO43+d93Q+N
gRTqa27U104RykJztSNgiDgQgjhgIUkW/iWFeRTixtQNKjceesbY/LlBgseI
M4RAKmdErpW3HZIjYOsxxCVjKpc0ilky6coGYCviWBvi2GBp7Gvhol6QRdxL
RtxrwKlbm5DbwABx8iPipPLTpW6993VwHnGVF3GV9wzr5n+PMuA+4rAC4vDy
Ja6cB5LjIIu47fyHyu0Op9YZCV4CmCLO2zdSOV/CID1oadNEkwtxb7dO78CP
0OSI5lNxNiJnGU3u4N5uu/eCo4Imp5rmLVaWmSZpcu2K/Au9YKdhuPuAmoMy
SdQcTG2UIucljNHkpmpeN+QWNAAO5SzvV2rOVjVrxMlhx2tDuYxDuay5acdY
7sV+yEU5voByfEHokROfTR1Motx3Q7lPZ7frZii3B4ghT2hAnrBU8vLTjiOj
YIG8Ih95hfjruO3C1whgjTwkHHnIREDBhrN3e8EUeYsH8pYlL02DvRJNNJ6z
YBh0qVigCC4hL9JGXqT95vgRcdwkWCOPMkYe9TfYhDP0yzgwDp47nY71XaAu
3rkOW+/YhHAdBLl6CB/LWmH8NAr7LfxvMe0aBeXXicKXTXPA0jxdrSBlDArJ
TjHd18egg/xeCrjGwZx+oOee7ih0X/xQl14xAubqim8zmIlgeWT+vrbOEJAY
t+x7RSDCOcuGw/myPZA8sZPnsCYe4h49ODyS3A/BR/l19m6sgzfFvsmz5b2A
+3k05jfDT3DWmy+Yv/Qc7N59jv8g2wpNs4b3HiePw0L0+l6rBWydkRQYvaf7
B6QGqr+IMemDG6LO/gf7cWAcaNpRUF0CH5Vjc7auTMLGjiudZ0rrYPRxTuCB
5HTQ5yvQsnrUAdomsrd82saAHb/BIc47Eu6IH7lDfjkEVk8Udy1bdEPHWJvp
ntP+YKh1rrunKgMMN3XzZGH9rjyu8NWf9xUsrH/nfv4lDh7xOp6bjSJD82Xn
HSmHXUBhovmj098f8KgApDTUzaCbbnCwwc8VfFyEDebWNYPj7FbuRwVkwN//
pXXu6hswjXIv30UiQsm+qcsbcF8h83zqjiTpINCUZxiqvfQDfB5e3/fsRR98
HszcFrIpCpTXbZLaqD4MqV+CgPM6GbZcmGSgD0iFKf6ky7rtDbDNp3/jRic8
2DOez9lZlwfOheU/9H1J4H61+ej5uhZwyT6ooNlUBiWqI3fCTmVBquujn3q4
PFBYb+2X31UN/rqcdUvhnfDNxjOg8P4ELNRrWow0N4B2lertDAUivDa9pzsY
PgwWRCEbg6EWcDoZdsBVfByO6RWxaPlT3s+mxyA60wHhpw9aT39IgmcSZiqi
+wbgo7MXI933HtDk1/M7nVgNK4X+uh/UesBosKaybB8Owhs3pzDxtYBkPgdr
LF87aFuILPdEDoDV+UrhS4Od0NMk+9Y3qQkEHIVvfRwZBF6TUsEnen0QlBnt
2ZJYB1YX/ILPTeHhRnFLvVcPDjSFnJmY9lSCoNy15oMFBHDtbbFv4RmE5ioe
zh/+pRC8v7VfR5MI36wtFyaWB+HOYmJWPl0+JP5Q+NeVg/Xji0wtYjAebu53
0t/1Ph3+q0r+HoHlUewG1q0ZWB751OtuO/IuHi45b3oUvp8E6vxEVhUJbP1y
aZhhe1U45HALrN9znAQ/7Pan3dUggH54yOvnzf7gox2/h08OywFt94jnBgQw
trxhNC/gDuf5W1iOYuO1ff3ygdj4q+DEKrXUx0CQimU/KEuCsy+Wi6R1CFDG
wfiU57opMDw/HGGNHf/pJqMPA9jxWe6GxZx/eBnOxoTc1MByebsWUfc6lsvJ
rx0OLxtJAeHkI7bDWC7rBLFkn8VyefhSu2Hm+1/yAvimCMp7XkIGk4wp73lp
NtG44oF5QnG9Kd4cG8+6fA4o74VRlRNXiaY8d/RiXpbyHhk9mSMZrtg47H1j
SXmPzIFRHDvlvTNqvFcOUN47w4Gf0aS8dyb2Rcbqe2oKp7avvqdGunjDzu1f
0+XTS2w9KV7UxHDUifIcFKOWdrvXkof8n3Ucq+/B8Vm/3pfiS2zZQ28oXrR2
/GPj5BOODW9pjhNZwbr6np21n9svdyru3/85z2NpWNwVlsmv/V4lfjirE9XT
8muvAxf3ZKpE8gZYe92OHjl/0CKeleY6X5IPqu2g7FO6Zl5SfE6USPwj08yj
eqpI1xlsXbx23icWXQmfh8g0dVIfc98irJZMU1fnXjrwWYSRaepwSTgBf9uA
TFO3ow6NYU6/JmnqnPFd/t46k0mavmg6OZXZkzgBgaiPylAf+Rieix8uHgcB
1HeOqO++scqerAsZg3uoT3VQn8b83if/VWYUeFFfc6C+bvZr66nxH4ZLiAN3
EAe8tt+Wlcby8Drihg/iBu6z3ePE+wR4izjDgjhjF5Glfax7AJgRl3yUqFwa
DRa1ji7thRuIY6aIYy8vvSBdbW+DccQ9ecS9j9e1Yuwc64AXcVIVcZLb7ONl
nteZYIm4mqpC5arfVvENv8vH4QLisCriMLfWdtMr74kwpU7ltjPidm6/wiR5
XSe4Ic5vQ5wv+czMGnzdnyYXHM9cslIowoHRf9QcWUA5Qlh0Ob7CP06TO84Z
DEuyfC40OZVRXJ2agXn72lyj65s3nCh7SZODqV3vfA7XVdHk5m6l/hlntUHY
6EDN2RKUs87Gux59jS4Bb5TL4yiXv6rFeyR7D8P2ZGqOKwdRc3whaqUtXbEP
XqPcn0K533B35I3c3lr4jDxBuZ7qCX58bPtz08lQhrxiFHnFkkSO+ULRMDgi
D/mLPESuescoH34ArJG3EJC3zJk8JQaPttJ4DtlZ7PwG3WIIQV60C3lRzP0v
Ib85JoA0QPUoc+RRBu8dpKbEhmH+MefOkAPDwBuMjz73jwSMH3Ff5eX6QP1D
wcuH8xOg2RVy8QhpAuR6+o9IZ1aA7/jxnk282PpsoPrN2G4yGNxm4E7H/OTY
A7HQ3DcTUG1RKKVlMwEyH/eeYOHHg8Bdl7js46MwfqIvtufjCJguBXjtrG2F
t4rdCeFNRLjKPfEflOAhWO4qZMnnQKDoEv2ZdQMwGpdDv8W3HSTn/bqXSKOw
w4llY1BeB/Byf7thxBoPwiX+TJ3vBkEq7vWcTnoDmKgkbwuhI0Lt2CB379Zm
UOlyW++1UgHDPLaf+8KrwOerzqF4WTLYmhBWpHtyYXZ5SZLBiAiF+qnnHk4O
wuYDXCzfLRPBWSk7wHO8AMRMHEcETpRBUd4rRVm3cAju9679vXsQQnf2LqUb
DUDr0gj5KwSAM1vNOdPfxfAwcliPO3sErPh0U9d1+ICzm0jcA+9xqPGYZefz
mIRL4Y9qCO6+0GCu9bSyYwL0i6+q8Tdlg7/YKJ0PVxCk8ghMHg2oAYJF0All
jhY4/aZl4ph+BAg+UItMxdaPAnNv6G0KiGDox1KcsicB6DJrUhcjU8CK4XOr
7/sauPOpq4yuLxPi/lRqab/th9CkLYlJ0qOgG6H2za3pK+in5HzfrDQOTfZh
+zxu4+DNc8aDY7HfILyv56Xlu1poSOrl8pxogM2LU9dEpmphynpngt89HCj8
9Wri5E6Bl7FPDX9g66ePhzIZyeLD8NDmpjPbk3HgPEOfzeLVBn0K+X8uhGNc
jotIkm8bgvR1E3wbA7oguEHGT5G9GAh8e8redeDhrW042/crfXDQ6lRKkFID
ZP83r3nzDg48fK/Zd/bh4Ebqkos9Uzt4K+i1PqzugkU+L9wZ7UEgGbcsC+n2
gChv/LPYgFZwPug+9cMdD5pK54nFkjiw0zK/uIO7EYoW/MOcHQmwzNWx6Jk/
AL/XL0fKudZAOLeeh5kEETYs/si93TwIZhxvN//bWAEy5neEbDKJQCIdxPe9
w8Push8lCjeKgE1uMubORhJ8lrzbl7UFW/fuUZyY+poJ+EO3/o7JkGBL4ugO
gjYBnpkEe5szJ0Kh9/5/MRokkD75Os/LlgArA79fFvyOgCtnR4yv6JDAiONH
V/5TArj/lE/zeBUA9ay5XdqXsDz9+J/I/CMCVNFPLdFlvoIDA+S87Wok8Lj/
8C6/BQGExX7ebpd+BAPGHxnOKJBAKvUw46ObBPiQjpvqnLwFzUwx2/ux81Fl
u+jeiJ2P9L2t4a69yrBJAPcnXIIEyoLXlk9fIIDf2ei0FMq+oLgvDynvHXaI
27lCee+wQ6Z6i8EyGURMZrP7ME+YTPfUvYt5gmhPrFIaNn77/fTLM9j4zvuD
+q8xr2hx0Eq9jh0n8N2PRCHK/4VeiQhT3oO8n07tFOU9yPVzrxwo702+ocAQ
SXlvMj5473vBmyXyvsN0FRQvMrFbukbxohdGnuP3Z2rl6UZSeyn36cKtMu9Q
3stMuJnVvLovqK+5PeXvoYFMpvx/qUpD1mb5/4wvr3+x7l5PHM1x9itIbNE8
YEfzub3Kd8oX/s955nYeH178P9+rVZH87KrRb/m11yHQLTr1t9o2muvmLnjK
tXVMgOY6u5nkShbcEqaZlyinq8mBnIw083hgSS+s6i+ZZt4v7Nt6kH2CTFMn
YbdlLvk0kmnqagFYlo5GkGnq8M0mu3TKe8bzUd1Korrti/BRicLW42vrPPDx
Bj0DmIStqC/CUV8o9egN7jedoPyufbWPCKiP8l6RhqS1xyEU9R0j6jvDgt1F
Jf9GoQD16SLqUycnunarmyPwFPX1OdTXJW01+rMPhmAOcaANcaD/v8MKIseJ
4Ia4cQ5xI/WmdsCZD4PgjDjDjDjjwMt+Q1G6H+IQl2QQl/jyvz3puNgBDIhj
3xHHGFIKo1wvN8BDxD1HxL2Se4z/nhwqhOm/VE62IE6qs7WXOW2ZBCfEVU3E
1Q1C7L9lPYZABHFYBHF445bSkgmZfjiLuK2JuD2l+k1DbX8lKCLOl6RROc/7
LOj1B4MRmlww5Bxe4vFoocmRULXMwd45Ik3ufHnVElhwYAKOoZzaiHLqy9Uc
XJ9lLE2uVVhdDw89VA09KAdbUA521J7ZECLXC6ooN0uUqbmpSQr/Zmc9BiMo
Z91XqDnLfTHMfAd/F1ihXMbtpuay8yFVru6aYNBGOR6OcpzutdcDdnsSHEC5
r7CTmvv2z9qmLnR0wSfkCRviqZ4wzJfpy3KnAiKRVzxBXqEavE1fiHsCpJCH
MJykesgX+iyZXzuJcAJ5SxPyFtXKsjdDeZ00nnPLZi5MGvPDs8iL1JAX4Taq
Ss40jUIL8qiZIKpHdad5h+MGcKD37JTfdfFJcPxPsrOafxKeqCR+r39JAt4v
Vd5JEnlQ7R0X+3JTLrhtdLcQqa4FhRiNMx97i0BqyN/mNRRC83Rk7rzRGPB6
XbtoKfgFzjKIVPx1jYfsq8F1l7YNwAPXhI2q9RPArYt5ZcEEpIY6h8Sb/QSj
xL3PAn4Nw3EvGduGTcPgFqHXv85rElrpFKrnKT6RkvrS3BoHs0v0sWUhROCX
qGG/d7AXZr3/6zHR/QHBd+/7JrZ0gIeEg8EctECFo8cnEdExcOaun79XmgkX
A1JsT7rVQJVZr+B56Aaf/D0BzLFDIOB42v7n+xLYvTfJQjxyEvQPStRtim0H
BkJUkod1Jjirvmg6daoPzuFKudQyx+HtXHS7w0w8OBOlw7YK+4H6jzKvf39a
YItxQiqR/zP4Cz/t499DhB9SZI9cBiLgLh+N3UMfCobWB6Wuc/SB2l7VoOFD
E2B7lK7c3CkUqhg8rzrw9IPbAfcX71vLYdo7gN3/w2ewr0m/EjhBBMGuekNh
3V5o+tC4q/BlPNA1nBJeTtUFhVMsQxrYuo+rS2zR9VY6GL73MZCzaQf74c15
ekGdINd14cxP13zoOHelJf6/IaCzOL11fa4Z6LSK3/oeUQZ0GuMJ22LSwe06
i4NoORHq57d+c5GpBuXt0dH3mDuggeHTGMuTLjiRRUzYYdQAZbmxmsa3CRAu
Zbdf4UEVWJ4NOt59vwXu71jcNWM3BnT+w9autkawR/93Opt3B2i+9nBqeB8N
+2yNVZgHR8GcfFel9W0PeDrozFqnVcHB77nLJsYkWJJ+bjODrTf965UGKrD1
JmfJ0idup0GoLltcaCobAAUpf5XTWl3AIfeWfc+RPhhX6PcmHcBDxxOVvlN8
/WATubvkvlIHCDz5xgyUfZEMjXirbg8Ak6p33lP+Zjhs1WXbsp8IFVMtAz9z
B8Gi86tmmWsdnAx5dvNtCREEfA0EM6LxUHLcQFx/33fQ4mnJPylEgji7Z0XK
8gToYbXqwm0oAeO3SWqB2Pnm13vv+u1PgFDTiu7pM9kgucG0R8yZBH08fDcG
8gnADKJqOneTwNcxm+TwnAQbtBbfehVjuSxLx5Yv/hlcLS7h2zBOKI59E7qX
SgCWxPGXd9P8YeD1RWXSXRJ8FaspVAomQEWo6thBoht8UX0sfUCXBLjuh1a9
TgRoZz7xXPCaNdif+xWEP4v52/XiyL+mBBjU3Vfvs1sL3l979c73JAnSM8RC
MvQJcE3mscMOzIu2pPYTFCWx9ZTS99HDFwlwQ2bHVaVFMgybdf45geV+dk9a
3HUs908aph/bvEQGphfFy7aYJ8gHnDzTgF3XLFP+lop/ZLh2fmlsQZQEiwZe
zTyU58ML+F8cxjxEf4H+CRP292nWZS1ilP9TfS80p9w3nH8XLCWOjW9V02a2
pPzfSetMrozwmDzRJiGIsi+6uUriZcq+6MVF53mIJb3yF13V4ynjmid4LCnj
7bufeVP2Q1h+Ffyb8n8kvkv9pZT3LFv7LQUu/p/xuYBT2axRpTTHSWJT3snk
VkDzuVds3u6b/j/nWaBS2Ur4P99rYfuk7OmAYfm110GpIND9qvcumuuWqpNP
uNEpTnOdZwSXLcxjgGZe9D32Rr6bU6KZxyUFBwt99yM0865tFZwzPUemqZPR
N3+SVXrJNHXl1qt1514imaYOw0Kqyh9eIdPUreHSyg1CzyRNnbOIWCa8Fpqk
6Ythm1cVitITNH30cul2tcryGMigvtuH+u5VevwvIftREER9WoL6NEyt4Z5G
2DDwob7WQH09e1WGX/E+CYYRB2oQB+STp8Oq+vDwDXFDDHFjykj20fQwlhuI
Mw8RZ8qebaj4ua4bDBGXDBGXQh94MRw73gwciGNWHlSOaR42H2x7VAE3Eff0
Eff0d3g9fT1JBlHEyS+Ik3I2n+eP4kcgC3GVEXHVzUn89eOyQVBFHNa8SOWw
j/LQ5ujHLXAEcbsKcduicOqvCokMC51UztMhzvtsduDXlsTT5IIPc/3tksBS
aEY5ooxyJIQjuH5Ue4Amdw7MfQp9zzxGk1OmzMVnCkqyaXJNxG//uNffVqhB
OUhHpuYgb0ioDP1zbF5Rbnag3BT7N3Up53o+qKCcvYZy1sfCZOcTEQJ4oFzW
N6fmsv3ritbRtAaIRzk+h3I8f/cGl6a7mLej3Ff3oea+QK1qTnwAHgaRJ5xA
nrBxw1KaQ00L2CKv0EBeYUh8wVy2LxZeIA85jDxEtOPm+wDjYaDzpnoLzzqq
t6ir1t+51t5L4zkPZaz27LfHchR5UQLyoqp/vYTMumGQRB6lhzzK3SrWxnCm
CTLV3lVdvIt5Y/E25Q7JOuBSkjph4TMKw80lDayfWuDk0k+DwL5G2DTW+DHP
uQEEM/CCR6WbYW945nJkXQNUyAbG4+1GwVTl60WJ7BrY36gkFK5UBWfvOa+b
d+sFurvDd5/6pUFc4S7vxuVUwDGWm4knl4CbaW9mctk41PEe440MGIcZ8R2/
Bw6PYjlOtGqeJsGLZLAfpewzZHksdskIBxHTAq3LGgMgZz2CE0nB+qCm6fl0
UzWY87JqHszugNSRmei93+NAclb8v0nGcTjlw2afUNMAVfZa5jxHiVB4IYBf
OQoHpZXfyli4vgObbt6wOEc9TFnpSiullIGibPUzkeFCUNesGbaVHAXhE74t
hTgifH2uvfArIgOs/pQoCMR0w4Ka8yF8bRlszBJjHeBPAh8WgYO7eUshrvK4
yku+AdhR5/jue0McCGyp5qD/H1tnHo7V975tlUpF0UQyVUIqJEWTCylCpRRK
ilKkgQypNKBEEioaVKYQZSoyRSFkKvM8e0bjQ4Mk6d291u+fz/P9q+NYh9be
e637Ps9r8dikOdh8ryY3k9GL1GXFBx7WxkDDVUh2vTwH3q+193QHFyNRd/Lh
6Nw4dNhmz/Zufg/2vt1pY2u7MEX5sMLuU2+gNH10S21APXi8XP7eeuGJshjh
j52V6diXX7butREDYbfaM+slOvFi6inPM6wcqPmfCXZR5cCif9odkbZ8dJvO
0PnxuhApdwN3xe6oxPtNrLnKvj0QfO8snBb+GanrD/sEre5EXP67Ex8DOvFg
9PHorLdVSH1b1SoQyUaSyhXx5JZqKB249iWlqg7ROvtXL5tEzX/ts/UQ7zuc
MCx8Jv2tCTFluhde6+WCt1YkpT25H+bDT48m09rwPuTnj0KLSrSEad79mMsG
/59T+re3dCJLpudllS11LgkYd5I5S4f2Elu9b5+7kE6rnvX0VysUH/76fI/a
L+M/fUZXROnILBIXnSTViVbmkXH/uCZcub1CY5MUA6wHLUuOpXZh4Q6BQ2ZD
NZA5khsk2smA2BuPJw61NGz+bhsozFeBb/7d47+OMqF3WJ9/41061Kr2z39W
/An2l6y3vE5gwl4h3/3DGB3ln0YPf/iVA2O1aZf4y5mIXms/pC1P1U/5m27a
vDRkFX/USafGz37uGF63koGDxl9bnlyKx9TzYoPSWUzw7XdblzKHgQX7TIUX
7AvHFZ5J3y6EMBGybuGeWgYdK5eE0PTn3cOr0uuOmVeZMFjUW1WWTofVM7fl
NZ+uYp3W61UHqPtMa9wvuy+ADpsFG/KKhw/jq9InG0UDJrqO7twTZE/d54uQ
bY6jHHhdnR87sJmJv0kPxmIP0mH50Hjnse8c9A+e+ft5LROjb7W6buyk49Hs
hP60H5QXQli+2opMCAqs2Ty6nY6asRpB118cVBn5P4+k8sPmU61qEf/e18Qa
6/n33oA5z+MfuFPjgrHy0XbU+MHKN0fXUTlkreHB5jZq/G6CtdowNX6+Kybc
9+k3deVJ8k1rqbxBCxltsKc82XtKYvOmkEmQXhBhuu/f55e6fmeEUuOVvxjp
FotZ6vdWCd3+93lyzaIlFwKp8SDdxXb/3l9KSzNL3UyNXyszDL9CjZsdVJ33
43+Mj1/R07/bWMs1z8PK2rPFS5rV/3tdUZ1Q+r+/y/zf+7w5Z55Xxv94roQs
ZuG/n+v9dx0U39VExRot4lo3p1s5IjFSalzr/CmRPy5rnh7XvjRXdD2XDtiH
W2QfJ7+e2Ecn9cq9xzeYcO27V46FZ7fydqiSOskkdeJMk3hyY5DDVVcF4ykm
9nkcrjr8MZsWdtyOw1W32Zd8Et73DOA9qfNzpM4fNClpGcsNcPXFlOhij+vi
/ThL+ugs6aPpuzTlAnJ7ufrO9nZP5iOBHkiTPl1E+rTRs2Td998suJK+ppO+
Pmv2a89eLwaMCAfeEg5481fwpMd2QZNwI4FwI7CS36optQ1TCGeCCWdM2ycv
TF5WDyPCpZeES6aXf2UOjZTBhHDMgnBMw2TDkcmWb7CUcO8e4d40w2nVlqF9
8CKcfEo4GXBc81y0EBM/sye4+pRw1f68t/LFthZ8IhzeRzisIXgsRbW8APcI
twUJt32137dcLOpBGOG8GOG8W5+Gj2FeKxhrJrxQNG3CCz5LJjdYxlI5mXgk
5+SER/L3W96d9qiZyztrqhNebXvcjV9lE54aE5jw1JO38SPzT+ZAlnhNUGDC
a7b2gee+ZbZyedCwX3L9J4Ne7CbePES8qXG71fimeg16iGdHiGfdjsk7KO7r
xVHi5dkXJ7wsZ1Wb7pLWhgfE40r9Ex7nMbe/6dvwAYnE+/bE+7o7Z4Vuvd2N
qyQnPCU5oUP6yetpm9upc/BErughucJN9fnKC4afkENyyKHsiRwSrO4f5VjT
iySSW8xJblEswdUbVN2okJwjS3KOhciLA+ekC9FBctE6kotMZz5c8/wDCwEk
R1mTHMU7LCO8+30Brh4stNKzoM7PT55LRHm3w/a6u/A8OSrf6u6LXCpDw6fJ
lXvW8rbBzlz12aLecqSzUC/9tgOXPunMZXU1IVnwT9RNNht8ITvGZ1o2IthI
SNXQohaCpWJioaupHHRydLHes89Y71l8Mf15CXJUD067c+cueNISJHQSI2Et
flFZTCcS9ml7t9inM9Bba8uOaO6Bokp7wS+rHjS0bkvnO1mHpUKjYtLhdEhm
xAwHfusEz2blQQWZIHj+1DHpWdOG+XFuK+0byuE73yDm2QcGVjTMrVJdX4vD
wXOfHIrtQ3rJt3gdZi3s2g4regp/RsyL5MKfDzpAX1JyZZA6lxmYDYqoTskH
e/rZv8cv5iFJZh/LIqkdm42n/HoclYUYF51zZ3u6sa3ZfWRROgetBTc/rVRL
RUp28N5pxdT5aaf12OjLZljJnI//vD8ZPFOz2nioHCDzzTonMbMHM1xuCvHK
JyOG91L4KVGqj0PGFiQ+/IzJTsc3XTr6FmoLzPcb+3cg3362942vdKTzxCVu
5c2Exf7+rku1TChtO/9aKz8HHfPnzfzxJAeXw2+cM34xQOWav3OrXRnwfsqn
9s2nAHzDYi25rFI8ellvvX1XNRzW13/a9aEUhp9/JUXIUOeUEMeZJUUDqBW5
90PMoxJLNc5vP+fJRPB4q80MJwYc361Z3eJeiy92K38nivaDp64vV7+sCbpv
HiweiW2EjdPznc5CqWgoHxta8aQYQUO88W0fW9HiJM/78N/n5c4UzD2u64GZ
G0TdNpV3IE/+xG6JsjqkOTrsj+rpwTWB+0Wmt7ogsd1/7rnZrUhcMP3t/jVM
zBWq3Zg3mY6+CzlPpUc7cGBhgd/vGV2I35JxbbIKAzKVGx7o1XRBvkVh7a7P
Ldi4z+z2usVMmG7epjS+go6ii4zw0kt1UJyzTU7vHRMyClvhNpuBTfdEclRm
V2KNyc1Z8otZyHRv7TcPZCAUobqXfhUhJn+ff6U2C2c+uGxu+sxAZG3kgYsX
c3Hfla6+mxqfParCK0uNXzxJNzYXSMPH3/V7w2VYcF+5aPGmSAYypxyfaTE3
DqZx2/mcR5hQWbIqWc6KgZGRhElFx0PwM1b+OyOfifU8MlK8YgwIlNOcjrT4
4I3g1nznwH+/R3/864t6Kg/8fXqw69opPFl5INT1NOU7J9WmzyF0tC4yNFpK
5SWH+1MHFXczkZAd/pPmRIfrR3oD/7/P4ZyOsd1OeXlc70V72gE6IHuMY87m
ILbFt7SRWtfItM6qZD0qp8X53HHhcOD35ZSLOZUHZNkaXpv+vV9UP0pI8CcH
LrKCm81WMXEq4EfXv/cyeUd7K774932tPoXXdGr8tEW+xp2tlB8bJLckUHlj
ypSZ+7OoecZw1CafmmfWeg/+Po8FyNpT8WoHlVuebrIyCqByS3WT50eb9UJ4
1W40eJYan6WqIlxIjTvE2qZbX+tWF3uqEWf07+u19HNDqPFouePPGqn5C8xU
DypT4w6RAolnqXGvAtVHrP8xrnLy4sYiazbXPFLDcad2rvmm/t/rBuk8cmNS
8/z3PitpGQbn/8dzDTwJZRv95V6HL0cZM9LtebnWbTC3NPN64UaudV4xYvO2
pX0P176MPDRydvC24NpHydyGp4IR1lz7/uXG5sWy32y46mTJtrIjB96ZcNUV
z7EocTc6h6sOP9qv8vAL5OAgqVtVUrfrvb9+qOTncNV5XuKBv43qA1x9Ueaw
bP8+iX6uPgooeRAbHt4LRdJ3qaTvZis1nAnP7ebq05bUb6GMCyyokb7eR/p6
wVYBxeA8OmIIByQIB9ru2jBNH3dCgHCDTrihUu3cxdfXjAuEM9MIZzxqhoL2
PavGFMKlGMKlDqX1L2335+M24VgK4ZhaT8qptlQOthDujThOcO97qdv1dVQ+
sSGczCKcZAfOkGx63IWPhKs8hKtxk7UV+oVrYUo4PJ9wmEe40PSI43M4EW4H
/JjgdqD8/LIWUyYaCedPE84rneBpOsBXy+WFgLMRRpwvVI51nPCIFPGIkgSN
Myxax+Wd6Q1uG+cHs+FIPKVEPMWvbDGusaCAy2saB8rcPeS7YEo8mE88mCN3
q939lzsOEm9q8E1406lvv2+5bAciiGc7iGfdWs2XjVjEYzvxsg/xspPotBUt
/Ay8JB5PIh6PqbNOzHerwSrifQXi/flnr3p/MeZgmOSEzSQn8Ai1PnctpSMn
fSJXLJScyBVZAQc9Jf1rUEFyyA2SQwQVPj+878iBEsktaSS36AcPnVPqpiGH
5BxPknOismbZMMzLQCe5qIzkostybx5tmMrmylGCQ7/Ck498hJtI8seTXlHo
Czy19m0DE/MZvpsFLnXDWz79WppiP+aZN2UMZ9NRdeG7ptPLYgT2BO090c6G
tnbZrVnWXbi1xLepJ4ry87TRlowQGqbw/p1zIrgN0sPjHjumV+H07JlxWY1N
aK4WsWs8VYfvx0wLJZL78Miw4HekVgnOmfc/7Oz6BN28jHn7VnZiqrf1JiXq
ucP5BaqHz3GQVE7LrF1eBL0TuvIpAWyEPje5MdmHhXxnTubSsR7MMosdrbXq
wlJFsQUM/xborjmu2Xm2HXJT6p66PWnCoxbzdJ/dHyC13n7YUOIjLl/WWvac
WQUtB9UVDiNspL8Weq2xgo25tt+TRIRKESBxxOIzNc8hCcn5ItU1cBeevub+
6nxItbVfUvX6CK+TBo0/qRx+1fRU/BfFHKidMwrdXtOPxhXDocup/GZg4XD7
SkwW1HjWnYrp78ag8stT7553Y4nr6rV7NbNgsaSt5/nFbhRu3SjWqFQBP9Yf
Wmr6e4pHTlrrZfsg4la0OFGChWOOD8cf/sqFlGK57/LeCNgPvfuz3vEzfucw
H70aLYDIswvn+y98wXyDpfFBq7vBFF/fmWVaCt6fkW771FqRI8HzPeVPE17P
SNP/OlKBfXHLjKpvMmChurPcQi4Fvrq2j8/k1mDjxRQljZ9UnpYw9/7dx4aM
vm7nrroGdJwYK844/ArDZ1yq+G524EmuX/BvnlbcOHdxx5PXJbCZpuCiq1OF
FlWp1nTPDti6U1x+XQulJ1FHVk5+C+lu6XiNY104vKci7KhPC0Yej2yoa+3D
1BeL7T9No4N/t1TFXZ5ONOgeOM9fwIIgfW8fL8V3+ynuTWfEaFTebZxyRpuG
3VaG8184MSHOWXPI6SWVQ6LuZnmFtOHK4UmpNaos8M4xKh95x0BbRKOqbkY9
upW1gt1fs6Bs6FH5y5KJ6pyG/PKySrxdXSWY1M1CSvOcNe7xVA5S1PnkHFaM
QtPywZBeFkw1dhgJJzGxZ/VHK7jkgvGIdr0yn0V58uwJ3stMTDqiWbVrQSou
xTOrW66zwFYbqBKRZqKO53W2yq8YFKlv3DSynoWVZzf3FGUyoPRh3HZM6yGS
kmjPnHsoz4a6vIswZEDs+EtbN95LSE6OmHzuBZWbQ95LXafytOvP2A61Pg5U
+bQqhR2ZmD4nZu+mGDqEEq8G61dwYNNZfv3jLiZWDm89p0t59krooV6hcg62
SrvOr1Wj1sc1T+LjPjoW+53LDmzi4AvPQ5mflN9nSp3ctujf54tsSjpkuzmI
vjRr2wF5Js5bTp/SCTpu9tS/OvyDQ53XHx16SI1XJxVFb9SgI0A5wVn5Dwcm
MdtOqFD5gZ/vd50flR+U737XH/EXRezjj36p1PzVh1znZVH5QUjD1w/tcnBZ
m//9OpVbrrSon6T/+/lal0L5LEsxVCw+pxZKjX/3Vn8/azsdYayNWofbKtQV
uvPHzf593ymYk/eS+vqzJyWeR1J5xvuvTdBiavxiUHLtTmr8SEpZ9uf/MS7n
XRG1aMkvrnnW3ok8rlY/jeu6tLaKgH/vC/3vfd66Gf5T7N/fs/7Pc20Wns7v
MM69DofCtrj9+znpf9fN7fopv72n13Ot85Ef8oYxWvtRTvZlFtkXVd2H6yQb
TnDt45qvajrhbx249t1KM2nPTJcLXHVy2SUmR7nJhauurnvyGUt+OsZVhyqv
5P8Ol3K46pZ3ZowYexsHl0md95A653CkU07bD3D1hVhN2nLjLf1cfRStUi6j
FNHL1XcrVBT8uh52g0X6VIn06Z3Q++KPFrFwkfT1+OyJvvZUmEc3W0aHPuGA
MOGA7Oq0JVlJHRAg3DhFuGG4Y+fIcpdGTCKcmUo4o3J25zyWcwUkCJf0CJeK
FDcp7RLKQC3hmCHhmIfBrMDMI/24Tbi3mXAvdcwm36yFiYWEkwHHJzjZsUzI
bOBbG9wIV+UIV/lu3G4TMihDOOHwesJhtulC5rdLlDcIt9nDE9x2Gvzrudy9
Cz8I50eeTnBeqfy8fuyOUhwiXrBQmPDCwSl3NZT//b78fzxy+u/ZovC8Sswj
3jEk3mlp7Tp3n8WCDfGUKfHUmLLJnrnniuFJvKZNvMbp4VXqp/KPP/HgBeJB
+wNfgqXi8iBDvJlDvPnFfvCH4wMGgolnzxPP2qtlbPpzsBw6xMtFrRNe1v56
LO+UWi9kiMfdiMflrHce8dftwH7i/QTi/TA5rfnOc/IhRHJCAskJPPw6d32G
uyG4ZyJX1JJcwe/q22ASS+VmkkNm10zkEPu2zZ9nWGcineSWFSS3xN1wUzcb
YyKC5BwDknNCs1h68ilVuEdy0SKSi553enCOBnWDR3QiR42QHCVmWyZQKlwC
XTkVeg2jBV38aXItr/vw3Xb3syGq3gVNNBeuryvHwIjuwNWBboyks5YNqbxH
ztKY0/KSCVA0dD6ZR51LNIpO5jWk0MB+pPxF4HAvPnBOuL4rpKFvv/GCn3ML
oVx5J9JRlY7Y+9pRL43bcXnX6Zp7j9m4fMLVxlOqEZ84zUL8yrVwa8lZy/+7
ARc2y7DusvLAes7eQwvPhcaAncjL38HIaNb21lzWD52gtrpjPP0wXDBJZaEl
A0syjrZ2ajOxOPScn8gJOtTm7qqrsK7FNOasTWHXO3Ba63LAm9hahCi//RbW
OYD+kXaxXJkG8IRqna0SCoZgUl9Zmxd1znluX1XRWInZztn2StIsjAiJPDz+
ugAFCUt6rl8sBb3k1qmWG60QqPqbKniUhUsLVX4ujiiEnO1+MbZYJdr606a1
5lWg7hpv8WnqPFSxvFNvxvV8pHQNVDmFs7Fa1llSmjcfaror2/dcy8Pm8jEt
37WVOMKxslhwtACC6VnPZnwvgcZyPln+Lqpf7l49vXpXESx2OVtlxtWiI1+9
rYa3BstS/+jqp5bhpeXjG3tWdkDQ+vfnZ0Z9SJV033sMleANDd9xbyYThj7+
nMjv7fCrm/rwfEYNApmqCsM9vagQ999auv4TZhdtnhEf1oCww2+POvC9AJ/H
lwsXJvfhAXN5gk9uC5R++GafflQMJ5+lgYv30BDvsGndOocOsGXEFs0+UYuc
9bMOdCnWI8JY0UbZtgvfm2We3YpogWC+0saIrjxINIfL6ijTsdH9g3HRtk7M
aVMsqH88gNKq8TDnDAYyUz/oaD2hwS2m9CZku0GrsjZJ/U7to0Xgu8HDDJyT
d77ksoeO0rHbOdVt1H6keej9CKFyPJSDDwp04MWaQoMfLmzwGsouTj3Jouq2
vzsmugENzWZpAgVsqG9WqOn9zMJyMf2rkluqIHI285J6ORvhJe8t5zWzcPTV
maSqxGLUCdnPXk7xSV0yce3fcBbuB7n8cqfloGVOpcHX3WxoBs04qLqOhV2h
T3af4k+B7jTNNFkmC3/8SqIKY5gIuV0m9/77cwTe3Knf50wdesp7py8WYuJC
kkX71uxbUKo9mbtEgAUn2TyebjcG6JY9oZpsDh47PxZ8+ZaJZffDeQKnMXDJ
4kZsTAIHO+/tEjlHeURla88ZWcoj+9Sn8bU852BpZOy2pzuofTV+L0s7Tcf6
ry+DhZM5WBLauq5wDRMP5BUfPNSjw1DS6vM3yptebKZCliwTxvPHakU301FZ
rWqWTePgnFZ61mIZKm/E9mvkbqDjoGDFAaV/+erosnRbOSbmXkjoXriFjuae
JkEZKj/oPz63KmglE6U//KbHatJxqnHI5tFlVaxcftV9iMoJH8LOSwn9e4/B
zde9OtNVscBaRO83lVvWy9T6BVB5xqJi9FKWtTReBIzmDFDjZVdCL16lxr/7
NEqX/vt5XJv7mCU1HrPgqXUyNQ+PztNPW/79HWcBw9P91Py8q9w2z6LGTXqz
REL/x/jgEvmCig18XPMIfpV04dcR4bquWqvnxvtnRtX/e59Ghmo3Mqgc9d/n
Kr81WVjyD/c6FHR+PS34h3vddkdGRtC3LOZa50j99iMGofu59sVWcVVRpNZp
rn3cbTq4c9G0S5Am+76X7HuYw9b9izZe56qTQlv78T98Xlx1FdjJCqndf4Or
DttEQxZYvLfmqtstVzwGZz7mcNW5lqqjeErOAFpJX2iRvpjry//A4Wo/Vx8F
iUa4S37q5eo7iSC60azAbq4+ZZ5r0rDvZCKK9DUP6etLejNuv4qjcXEgxe1J
qR6jHa2EG3MJN8ZeLpglZF6PQsKZFMKZGPNK+qRjpRAmXFIgXOoQnxx//Wg0
nhKOtRGOsb84njPR6UUU4V4J4Z6b1aXq2TZ0eBFODn6f4GRDT2Z9kXkjeAhX
3cwnuCrIub1hc98HuBEOXyUcNlUTU9yp3IOXhNsDIRPcjtuo5h0/tQPChPMh
hPM5BdIBQsey4U68sJl4wepZR0DQHwZOEI+oEY9UnGw/Vu5SBhniHT7iHalH
7m0bTrDQSjzFIzPhKWEn7ZRVlz5zeW2QL/Ly3RAWqogHeUonPBhtxnOvbVMl
5hJvShNv2v7eEBx9vwezf0141iJywrNOoTIntAabIE68nE+87FYdlz/ICMc2
4vHdxONJww2BwgYMFBLv7yfel6v987B6Sy2SSE4QiZzICRfcNiculeMgh+SK
EZIrfO5KMi960LCX5JAKkkN4CnYlpDz+jJmPJ3JLJcktsH25Ucu1B27LJnLO
OpJz1titct31oAGGJBf9ILkole96s+WGPriRHNVPcpTFCssCsc3VsGrI8TOy
6UWY9zZJMatYXN5z757b4S7wSW7X+ylFwyL9yyHevgOYXvVxbDDIAzmLPvyY
PL0eNwODqy5c7IV01pntQbId0Lgfpy9/4j2C1zkY5k5jI6cp/b5WTSz4a2fM
903uhZmA4OXE7zTsatKeYk9xiN8lLY6mToNpuMH05cvaIKJ5cTgq+gtu5M9o
Etlfi+jFcSkqJ6tg8V74/taH1P2JqRxckvkGrS7Bc2lz30DKoN0i6kE7dB8o
HtyZ0QPfOfvX/NzUgzCt+Psb339EyVCdWjrFr/nb/I7uNu6EYXNYWi/l8cuc
S8G7d7Thzbm0a+/EK9Aip6Owx6AJT8OnWWcP1mGX46xXy1ZS3Fzdobviy0P8
/iTgvVukCqs+lbCd77MQN/fgm/FcGtInR9+LcPkCPdWVZ62FunBIU/rdtq1F
uB9dWL0ltxSrVEwsRFa2octLTaI+jQVnCeXFQUdKIVeopmOi2wqlGfwOyTxV
WKv2Uke3rwzB4w9Y2tYdEC1b/OrXoR6Mvd311U64AqOay5JVq2hQUp1Vz+fZ
gCF/AfGd16pwIv/Azsc6bFipAGl8HPSs/mQoHVQL9ZjJrkaq/ahQ1hLIae9C
3pMfCTfWUX1/bU/92SuvIfJa1/fq4UosCrnFOL+mFRoPqlrbbpYi6chd+d53
A7gle/okYjswe5fju6aNdRjx+LpuWymVd2Z/zbZO6sKqOT/C3ge2oiPc/42/
TzP+xPWkhwRS5+dIvr6lHZ0IYHz2vbOjBM87GrKyNJiQuHl9zTILyps3bZYL
lpzBoGj2hSVRLGT+uv9EfQ8T0nNNy48b9+B8RF3OeBobP14LPOHLZsHQddue
uYoMLLN/aSzn3I2OI6ocowQ2rmSOtjqGdODbrZLria3deNggLRCv1A36g1fm
0lsaIeTHK+j0sxu7ag3Erxp2Y17cejcX0ypECVuVJZd1Q+HOUYvni7ux7/rf
YWeRYmwNWmx+/VQ3DvL2yylHsFGVGbXipNkHKI+v/cnDYMN4+PAckZ8sWN+h
HbOqSEJQVN37qQZsnHB/+45fiYXRH5zESK/HWPXmi6dSMgstKm/ewYqJzJWb
rh0b5sC5YVT6nDz1PMOPL1+LZeDsb1/5Bk8ODK97lCVmUXlgjX20iyADtNAi
53ZLDti6GzecP8tEXYNmfFg4dS7ZIM1YaMfBtYVTLmxRZyJA2T319iE6NA/U
+CgGUnlgYFxvI+XxrKXKJ/opj1+3a17Tkc1Bx5U/Wz9LMKHWoCe2RpFO1ffH
Gbc7OKhsXH//uRTFcYlQ88Y1dCxLmxTg85ODHYf3LOOl5pl/+bRcKDWP9U1f
ka1rN6AlwWaBNpUrouaE/15L5QqfQHa616odaLBbMR5O5ZPH7gvfz6HyiWxm
LwTeaWHcNTGuQpEJ3UX3TPN16LAf+FTh4bICCWnx4jbU+KEM8VurqXGe/nEH
NSovqV+ZmWNOzTNp5UL9+H/vv2JlxV+ncs5lDYPkICrn7G3689JXm47fl4UE
d//lHq8Kt6j4I76Qa56KUe810gWyXNddOn5kadiQCNd9un+r8rKhrvvf5xKa
mrR5z28O1zo0Fwi1pfziXje7gacPG8c4qCLrnEDWOTDj7lh7jyHXvshaHd48
7GTHtY/6ktK3vnm5ce07PbFbT/y7D1edcEoumC0Tv8dVV++n9G0Xyr8PF1KH
e0kdOknnHvpxy4+rbr94x9yatMEIj0mdnyJ1vsvNrSlvLQcqpC9MSV/s8rvu
+I3Rz9VHIbqlTffm9yGW9J0K6TspSQ2ProxuzCV9upv06WAsHfnVTHwlff2A
9PVNFfnKM5Y0LCEcaCMc6PGZqS3ysQ1OhBtfCTc2z3A2bw6vRT/hTOr/cSYt
NM8muhChhEsihEu2D5s467o4GCEccyIcu3Gz7OnTM904Q7gnQbg3opAuk76x
C56Ek+M7JzjZ8Us00PlkDWYSro4FTXDVYkqP8U7NSMQRDktdneCw7ovmjBxD
FhiE22qE21LBroxrGk0YIJy3IJx3O7NIRKQ7BMPECwPECypWlwJds2mQIh65
QTwiWB5gtMi3EHbEO2LEO3K/f22P/kvVN/GUGvFUJsfv456ACi6v+XDW31Xg
dGNS0YQH1YgHTzfpP5l8uQEviTePEm+eHk9/66fJQSzxrKDDhGcPnmqxqW+k
8ivxsgrx8uDdy0v2WJTiKPF4HPG4UuJ8s3OCvdAQn/C+w4UJ73ccCNgdr9KB
MJIT3pGcIDUtPHkoIhcKJFdcJ7nCLdHSw9iYDWWSQ+xIDhnTO/9L0rMeFoET
ueUVyS1iDvl1744OoIPknDsk59h4STvWJrVDjOQiKZKLng7yb455y0HA/+Wo
WxM56vyh5qZxrWa4CU6yzN1YgaKMhy07J3/CaWMx5R6HZhgEyvkfujgAey3N
uCDpt6gveXc4QLYPhuLTTWPuMbCpT7675yIHcs9+G8qtoHKqnBtjwK8RmptW
zQyZ1IcTGwRM+Pb3Uzkux8VI5z1u17le4fCyIfJk+utq9XawMf+rom8PFhj8
8avaRwNPqwTfL4u3ELxz7OqxjZ2YW33/nMDdZjQsXJA3m+oHld3b+R31KyA/
s1HIhe8zvBnwyZtTB1bfg7KYLA743t+8Y0n9O7rKsGPrGsov0Y78rg5sytOl
9rePslCTmebE606DhWgV3J91oUhM6RHdiMoXF/Ocljwox80K/WXMFOo84r82
Y5FFMVTynm720ezD7K3XTNfNrUeA0879rw9w8CH/YUTk4Q680nzaq1VcDYc7
lbtt7XoQ8NapcWXpBxi5S2WV6FbhzRoD2xw2C9FbB4teDzBRKH57T+ySKjSV
iQ2MPmfBoWFSf05OFdqXl50pFqjG+4sfJp/f2A32GT3LqS97cf7mxplhXTX4
2uzN2Sbfh8Hlm45ftWuBKcf+Ee1aPWz2RQifGaTyuno8j6dqNObu8JlbldcE
wZPj7CNWOTDYLMvz6gQDxeZvnyq4tUFPeKff9rsVMDWvCfliW4cNuvNeyml1
osxod9uQawM0Jg0ZGvFHIOOUf/BTfRqW/tKq+RzaDtGPwy51Iywcuy6jNEOT
gSrLjwb9rC5Ih/efemjWDoOQ3vtzZrJwL0jeJOoyA3yZI5O3HKrAgk7Rkj/3
2PhoZq7+x5cFqY/Zv+WCojH9eOKGwM/dMOF9mp4h1o0345NXndrTi9IlLXPl
X/fgwV4pSVXqXGcwS7bhQTMDS5Odk1039kI72eXyutoezKjZssuKpxMeYduN
jpyi+oVDm+oyuxcBnjwvjVwacUvL2dVdrxfyC2/c4mVTffrg1QVdigs7q97c
W0XrAZ9sWMi3Mz2gbW5Q/1LyCT8jv1T90abG1062u5rYjZrkvDgh73cIuCwZ
uP065Ys5exfMzmFD572+K805BibV80uzk9ngs/p74iaVm01d5Oi67RdA/zjl
wNo+FsY1shpbk5iQVzV4727FwYyjT16bgQXfgH0/jQsZcBPINTOayYFHfb+j
VwbF+csiCVP5GeA/KdkvOpUDmT+P014fZiJwurit6G06GjZmpOUoc3Bg9pfp
o/JMaMxzWlisQcdZnj+ZQk4cMAs/rDdcwMRb9QWqTkvo2GRoW3XgDQfvJctO
985lgu8Ua/aYOB1//vjS9ndyMLa1+rmIKBMRBux3lnJ0vMp89TDlDwfHWuZ+
zlzGhJnRV/096+no6t35xqPTCIZfy2RCqFyROjo49pbKFaH4Nex72Bg73EWt
hJSY4K/alLhEl8pvM+tmaKXpY8ES090Ka6g8IOcbkLWDDvamzujVvPK48OCA
1wLq6zdp90TLUl+vU3OcT5DKM5aWq7X//T5gxYbu3H/vm3oYFzxnG3U/oUUP
w3+uYuKMhGJQzFYqB1a3WZeOc4+PFH+e67FjGdc833SfyF+vX8d13dGAdxEX
/ypw3Wff3eXZR6j5//tcV3yuKM6lctR/18Hi+o2Miu/c6zazMVp9zjD3Oj9+
JR+hZaaMD//ZF50pkx3FXp7l2setQ6eO3z55HQfJvmuSfd/19UhY0ZW7XHUi
FdXA1/3iMVddxbTNPRRxKBSzSB0GkDrkNWzzCnwbAiap20maE3VrIXpxfZHd
fRwkdc5P6pw/UfWs5hcO7pG+OET6Yr62flH4hQGM/KePrsVal/zx68Nu0nez
SN+9G9xhbyDQw9WnTSovPzZ/Y8KN9PUZ0tcy2X/ljbVpXByoM7lxTPVvK0oI
NwIJN7Z3v95dpVuDKYQzewln6Lzfdxtl5kGIcOk94VLWMm37+AcD2E445kM4
1hFx/WpnJQvmhHvFhHvGijP0VPZ24DXh5DTCyciLcwWNZldgHeFqFuFqEWew
ZGP7ADIJh+UJh6Urg5IeldAxlXC7wmaC2yJer//GKtfCiHDegnCevfJHotfT
ATgSLwwQLzSdPvpZo6gTLcQjmcQjbjp1l3pMcpFDvFNBvOM779qt0FAml6e2
XpA8+3RpFdKI124Qr6mcrLwRPK0Pi4gHQ4gHjc9XZKZqteM+8eaqgAlvPupf
tDjhWzaciGf5xSc8e9myPMJFlI1x4uULxMsdtwIW/vzTAIH+CY8rEo/zHBz9
cb3+MQyI9w2J9+cLdGRKjNAhQ3KCAskJglW1rl3XqsGvMZErVpBcISU91tZh
1I8w+YkcEkJyiJryZdkHFh1IJ7nFiOQWnrUPdLJN3uI0yTnbSM7Ztsaj/lQv
lR9ILgojuajvxZKZVz6+RwfJUXyZEznKoWaI6bO9C97CMZ5zfOiQDhOcvza9
CuUb5ygVaFQiIKje3S69Gg3zuuoXLSuGhonc58Ab1Hq8aU1rZnMgtaV4WQ9P
BipWSVQa6nyBb9utSgfqXHk0Um7p42QOUl5m+CxPZSNsekObXFYDtIR0Xe/X
Uh7sU3vfeKMWPMomczkyyXhUMnOvqBkL0jxZP7969MM+RbBB2IwNVWfeOi0x
KgeJTzWoLqXW4RhHsGV6K1a4V0ZbDFJ558++7n0WBZAzEdvmuLAAnnG1fVs9
P+LGx/bZwSe6kZDSYaK9qB+a213unvzRh4D2JxLNUc2IW/dc3+Y0E5yzbtve
hNCh8WA5n8i6t2gZ+FVY4doJvk15ykcMG2G3+JeKSwMLJ2XoQhGXWqDm32a7
1awI7A59n2W7mqBlLjk/rrYBHdMmP3i25xUWp/7y96hLwoaiWjfBvfUQnOZh
VfyaA7tegfLCvUwIvnYsdzOpR8XVKF5kUP5befTz1KfV2DcQM8vpegPCRP/G
uL+IhLZKh+rsjj70lglCI7gJSW+a57pt/QD756KdWoPtePxA3Pq0fStWvTh3
4OPjMrhVVuhnv30HZfHfPqFnOnD1rZOByr5alJlNE8h6xkLGzttP5fd34cvz
NQ91D7RA6cW6gHO1Taj5vSO3x4iO08empLEvdcJCMntJT04G8jxNXonuYmJ4
WZuasSMdYS5SlRvauhG0xy/p/Ao2Krw6TGwmU5yWmKHwzagTyyNF7JTLuyEx
W9v0nQSVD7eKyTcsqkbS+3er16zuxa+h3V33CnqQ1Ld/ZBk9GQfrH3rcXt8H
qX17tNaP9OLnN22eSOE+zHl8aPQprQ98zpoCypl9KJB5LpC1n4lT13+2Na/p
h0irtdVJwX608DcK7FrTiZ357s0Jy/qx4OxVZvO/7xvTFBdvovZtg5ydk3lO
HxJzAr7Ovt+H15xr/LLnK1E5dqRJjDpPV0ZvvyVYSZ0zHHOKC1ILIJFVV7VQ
qRcpp+NGfxf1IF2tzUNfJhWdnEh/YfkeDIVtXxHr1436Hw2/T3o8wrXvikUH
qHN4LO+q+TGb2TCZ9cH4gC0HdcOGBi3ibOQ5j8VNozxRbCd5Oi1uAP1+CyyW
67LgdUv44JkqBgw8Vm9nOwzAY1ksyzuGygnmcU6zv9NxuzppvO7hAKYtH/Nc
r8VEV8+do1pH6Zj1dNXFvYMDWDk07csPASZmvHBOLVlIR/q3dAGT/Ryoj2e1
qowzsH3t4q/MvzRMMmF3y8RxsGCTyQAfLxPTFuwpseCjQyHSd5Et1beDuBDe
Rvn9/MV0j27K73PEFlZedd4L5Z893xYvZcIpw/rxr7V0ZE/Ktf9gZAUZE5co
OaqvewrydnOoPDM5wfmzf40lBKI27AxXZqLqyTTHan3qPjVjhp6x98LvCE+u
zlomVhgLWH00oOPGnHvWEnZS+NV6+N44lVueja/OdKHyTIrwuUlff3PQTNMI
4FDzB7bMNpr97+/CJKpd2USN5zzIBovKM6q2rCuSWnR8X1ZwYfQP93ijZUfd
hlVKXPMUKWuKz5q7leu66Qrhm0ZVNbjus/3motm//r1f/T/P1b5y+uvbVF76
7zrs9VG2XznA4Vq3cXV/jbheDjhknZ3IOn91eqhRQM0zn+wLL9kX6WozR2aQ
Ndc+Kmj0eG4bvsG172tudepK7A/iqpOzWH+mYTQMN0hd2ZO6Mv30dX6ARzSG
SB36kjrc2Sz7zLUyBg2kbvNJ3W6bMkP9lkoUrpM6TyB1HhooIVebcg9M0hcj
pC8M1JYk9W3iYAnpo3TSRx1qkf7rN/WjhvRdDem7mTT/O7vu9WAL6dNk0qdi
fDkX+7RZ2E36Wpj0tcVsG4cuExpOEg4sJByQY+6kH3vWipmEG5MJNwzeZNC9
9lXDhHBGnHAm50jDn3atD0ggXPpJuGSq9cPN9Xo/lhKOiRKOjbQO9cxzZMKP
cK+IcC8yr//5ItNWZBFODhBOiiy5+k7EqwhVhKsnCVe/mxW96/rUhyTC4YT/
4/AqhqpOUBdWE27bEm6P6HxjeOVX4C7hPD/hfCTD01j3Vh+6iRfCiBeWJjj4
t03vgB7xSMWiCY90HBrX8PubzuWdnu5di/5Q54r/eipDjm7pcLAaO4nXcqZP
eE26mCdBkdEPO+LBfcSDDnOaNg40dIFBvClHvDleJPBAaHcFiohnhe0mPDtm
8Hv80IF+VBEv2xAvj815Yz93dRcaiMfF4ic8LjLklv/weAnYxPt6xPtOOS8L
el704CrJCSA5QTf/h/UBmzZYrJ3IFQkkV2j8kdSyYSdjhOSQ/SSH3LhFk7q/
hYF0klscSW7R2Leo9qTsZwySnGNIck6yU3rpyrNUHZBcxDN/Ihcp7Uk8Xjdc
gSySozRIjtI9OmL97AgLPOMhl9SagpDlwH9sg2AjHt9/s031dgZWJfk881zH
gGHlTTvhhdXo8KQ77LrahZxb00pHq2tgyHPoybwvxRgULxPMfvgWqxTOBArT
OKgQnP7YcygdUn/Cvp0qpaFsdUDtQjMGwhr4PMWOc+DmnG734FUhBhOfl9+x
rUUr867C0yW9uPzeNefDajayql50KVIcbdWyYjdR59QAw+49ayQacG9V6fvf
xxmoFxdte3m0A++/Hpr23YeDVZveSUber0Oq/QVFK2Mqd/+s/hB/hcqZdyfx
LRZ7jeQlgfVD/kkIm5x2gndqBYZWZoaLXOqB3/fGL/2cbsz/ET6nbHs/Vn/9
cOVuBuWD8FepRxd1YWZ87dilYzScr36f4Dm5E7aRzLXqfvXo3NQfM764Bl/R
628q04b5UfHnzF6UIy9URHuejT+8L+oqb2prwffvfradwiU40nQwPu8nA5oe
upqT9Vvh/fy8YcJwKTrcEsrTs2uQrK6latHehpFcs6MVJpWoURT1tZs3gKFP
nnoh0R34+iFML0epDjaBBi1NQZ0QUarqzVXqQk07rSOhsRlhmc8P2+sUQHd8
wyfBnzSUpHspn2ZRefRDYte6u92oZeQ5sToYYGs+q5atpYFHx90+J6kNlne+
K32rpnLS19455gFMDC5vW/dqST4ay72SGt92Q3b7jGz1v2x03PAJSb3Wi8Lz
7Edrz/Zi2vXDEvlCvehIdI8zzezCUInPanF2H/ZxCjmrP/Rh87SYybssavHz
Nv+8ZdRz/MiBcNKUASjtjbxu+T4TSg/u1YXRBvBmlvLW+20DWBdy81peDHU+
Op/tX7SXA0HVXL5+Qw6+VnyfHPiUiUgD5/Y7hznItV6gMHSIAwSJPolU7ETv
Ie+0iBkcmN+uYDRNpc53l+xCD35sALNjy6cIswEsPJTm5mI4gFdroxa1nqrA
olvbpRwt+zHITh620e7HPH2Nv0WleZh3rWJ56uo++D4fKFEc6MVg1N89AcIJ
EL2fnfC1oQcnb30JLbDsgXmA5UFWMQezIs7KJ7h1w3/9woCyLDbcl0cevmQ3
gMk1q884aLMhM/P2VCEZFuZJGf61LO7HN7croqs0WFCaIlvl/ImBxAqeA7uz
+uHceoC92pMJXTne/JFcOkadHvOzFw3g7c03L7wXMGGWLsxTuIQOy2OKM3Ve
DsCOZmy+isGAepJ35vFmGkb0i9aWUV77rhdru5naX15po6jVNTQ8MvPXmZXI
wfj+u1dOfGXArr3ifcoADXkf3n1IGqHOPcW6uvGzmPD4WHnm0jw6VA/cuaHX
dwrqgz1qv5cw8Wj47/YSKifYKV2I/RRwDsZ/y+cEU/mkIt+6mKlLh5qQPG/E
XlvkHZxb82U9Nf7nwZuGPXTsmLRbIbHcBKm18ssPUON7GaryitQ438vV/vjL
weqWuCRpKrf0OL/6LKn/732tTx/vo+7H0zcnQJPiW/atWu152nRsi+CdNpUa
lxjPdV4mz4RUp63RfdAh3LaXP2OMezxy25yTz1Q3cs2zrvd8P6t3N9d174R6
+z+O38V1n5OH3uuJzhbheq5rDTZyIt85XOvAqf1tHs/gcK1b6Jx4+7YW7nV+
4eD6dh6de1/0fvEITdqyk2sfDUVXiClsucG170vdh29evPkIqaRODpE6WVoy
w9zfLAoXSV3tInV1Jclm/TXLBAyTOlQhdRhw6Zeqp/gbTCV1K0/q1vbv1amD
Ka8hROr8AanzFy+yFxzki4Uk6Qs70hc9X1TL9/K7Qpj00V3SR4+ndozelBiA
OOm776Tv5PsVcwR29qKb9Oki0qdSJ1Y7CpWwuPraQ0s3aW8IDc8JB3IIB0xr
S4pTw1q5uDFsvnC0vrAKCoQzSYQzYQKlU/oyMvGdcOkr4VKY8+3ytRR/egjH
DAjHysZejTfsZuA94d6YxwT3eMLGonfdbkIt4eRSwsnB3lrP5ZPyYEK46kG4
anXeWznoZw/KCYfbCYf3HfrwKfRFB7YRbhcQblvwaDrLvSvGPML5PMJ5JSET
9rKEHvQTL3QRL3TUndDRq2nh8ogFj7n69OIEbCXeCSDeOfRB7tGycAb8iafm
/5jwVIueXtQlQW6vnXrGeXPEgwNX4sGrxIO/LW+94qHqeAvx5jPizaqofdNm
bmiE4KoJzyYTzybtH/QILX6LpHsTXh5YOuFlt6uRZVf+sHCDePzn/3lc0NQt
krcRCcT7bOJ9Hp2PNuNad1BHckIPyQmBJrNMWIM0aCRN5IoBkiuUpgqN8+wo
x3eSQxJJDrHqErIS3U/la5JbNIQmcovu1PcicicbMEhyjj3JOasuM835dvZD
m+QiC5KLvn5bL3psSjN4pob+/xzVQHLUxvgdwo+/ULn6Ze45c8lGOI2eGf6c
2obOeQLJz937wbNLT2JfkB94t0y6vGi4AWyL6A2O+5tgEDv3jftkBopE36ut
H62CzNMKv3rvfhSpbT3CF1WNgL/L9x4WLgbv05X7/6q34oZNwh6VVwMImCmR
4v7qLXgiRK6z/Lzw/Y7ei2OnaLh0/CxnSewA3A7O8gqO74Lgdtf+I9IV0Jtc
pKb2uxuDwauW9O8qRsJCezuT3D5MtbVK47tBpzyiozDwsBsHx76rXMztxKKj
XR1ZaIHFonVemaVNYNsGvVkl+gVXzRqGf/OVImxPUvJO5USI33U6UBNP9bW1
aVZ09AAGwx41BjxlYWfx4/dgsNCwkjY14AUT/rv9GuKdWjE7e+hibxgN6paS
wysPdaDHWtTpytIy5K+rn6+U0omljz7/shNvwn3DspWhqhw876hzun+sEzWl
9u4nfjZAeVR/heFvBgo8pVE62omxrPkeqaLNMDjC1InZX4fJXgI7JBfQ4MHT
+0Wwpw3f+efXqBtzkPx7p9BL6jmzdNzX6H2i/t9RnuSLkjQsS5Bd605xfuRG
40AexXm+mxrig1PKEKZoYFk6hY1H1o7Z5hVM+CqUXuDo9YLVMfe7Obsb6epP
N3Zu6cbpmiHv26s60LxGp+5GYS/eZahryF7ohX1l0KTj5iWQvWxXf/9mPxyv
Lrj/4XQ/LpUsD9/W2YcIsV9TEuZycEkkKmzVHA42HuuPe0Cj8prmtylx028i
0yhC2q/bE96LRAaSlOuRs/7evJkekVhQnDg5dHEkklysFg+2fUDOT5uRSdfj
cTo79K/e6zgE9b3e+oW3H2GDQs86BZNg+GE0YHhFIhbqJ9rcTmSCJ88lPV8j
DusOHcjWcnsFB0eVgrWjHZCSbB9xnBmBwNdvb37NDIdrkIp+UVk9eI62iB+7
YodfsrVVA2fs4GJ1VWs+6zOmVsfeL/w8APc10WWsTwNY7T2u2DWUjTDBv78s
QZ2LvezfqUn247RfWuz0FY/g4pmg+ta/Fw78GY15ar0QsJV1unV3AD/XV2+2
7+yGa9/KbXvWdSP3tOKpaRv6IRl71MeNOk9ZmRceP0mdwxXE/IL3J/fBzvPo
QMQS6ty7Q3Ov0TMGbt81XHqtqw+uopmGX3YwceOh/qSVZ+govGu1POp2Pybr
rh51r2HgVm3vgkkfabgrLvbivtUAbvKkPfdIYMDdW+y0+G0a1qQf2spexUF7
3cOsme8YsDbI7XnwjAYtOc0k/hwOptQsU8xqY2B0eO+N35U0OM2IpvPnHcfG
Qn7exKlMqF9rWVs4k8otBb6sKWxXxCyYdMxTmglX/uSF51XpyG/RPujl6wqf
sc8aNCpvjLJfSgxTeUMvNNt+lrYDKguecVS2MPG+irN4jhkde+MzXxobmuHZ
iy7BqI1M8C3nWXHEhA4hazc5198c+D+pW3V1LRP97XtqhXZSOW38TIvLVw4E
fH+a/FjJxNdXT2vXatFxK3h6ptg3DrLVay/tkmVirHm+5eVNdPQou9bXj3KP
rzJRmRcRps01z9GBksHbhQe4ruvmP0VaR5D7PhWyj42bz9Pieq7YiJc1JtR9
/ncdcm9IdLs0c7jWbft5/SFmEQcbyDpvJut8nqk0okiNTyb7MkL2ZeTA4m9b
ejhoI/t4guzjj/ZT7S9+XePad6VRm8CfR4MxhdSJD6mTubLqfgWpsbhC6sqL
1FXrueaauohkOJI6jCV1+NbRtX9aXzpkSd2eJXUbKOG/0uh4Jv6QOvcgdV73
+EZ0mWoqrpK+cCV9ceLwrWwbVjRiSB9Jkj66ufVNXpo6dR4hfedF+q5PgSfz
o2YfeGwm+vTb6ok+rTphwdl4kw0N0tfJpK+V3h5verKIzsWBUZGHzo9mtSGJ
cOMs4cbLk+EK3WFVCCOc2UU4Yx9s33WnMQ1uhEv1RRNc+mp/f2wJbx/cNCY4
tolwTOrbN+8XJ+h4QrjnQLinvWF5dJVaA5YRTtoRTiq9qhyu+pGOasLVN4Sr
breDP0TbdINGOJxCOJxjLDbAjm7HU8Lt+4TbGtuqxbRq8rCccH6UcN6wLXFk
CpXjXhMvZBIvhG34+5g/txEjNyc8YkE84nbo6pXlakHIJd4ZIN6Rq+BvWVlL
RxjxVBHxlO4sm/gjM2pQSLwmR7xm0+t8U5T/MeYRD24nHhw+/ivC4icbpsSb
TOJNuYb0ebHbOrGFeHYT8Wxd9heDtsIKhJ2a8PK8QxNeThnNl27mH6D6a8Lj
xsTjCx2PZy0N7kIR8f5c4n2lu13bdcKKoUtygjHJCWKNKWnacd2Q9pvIFb4k
V3jrdQ+VRDYhieQQnlkTOUTt2TfL/HAO5DZM5JYiklvy1tjud7PohBXJOTUk
5ySJu97ampEMnn0TuUiQ5KLj5auOba/tovZ/IkcFkhxlffihddfXZAwa/FjN
q8xGZnjnWD2zE8EvfUxeh7HAI7b+w3m9eoQFLEpK0W+D91nN4ounvmBz3Ed7
4ygOpG61Sry43QCDL/tF3i1hI1+ssVyllwbt5SVnMy2o+3mefr77QhUsaFsN
RFwrYNHR6l2z4hP4Z7evvdbVg6dPBy8MuvQhjOfggncGieALTZnHn92M+adO
TJnl14ENwrMH232onJc39Nvxti/cjMVyFm3KQ3pyrkiFJRttegt3brxIw8ui
6QFvX7JxQTJ+temOLiTtXaKXql0GQ7dHPJ/cm1B6Oyglf3UdPrQyOq979cLw
aP7qwJg0zGTPcm8IS8WOM+0mro6deLHT/qFeQQ9yzSUUd+v1wH/LPpvfmtXY
uP00e3kRA3GOLo2tSdQ56k1bbuS6HBzSVpy6QoQO0dLXKfpinWDPjM/nOz+A
rtt8I1uc6Ti4QzxjbWInqvotMrbfYqL9sF3HzngGEHT21f5bNFQUzag3eNiA
Q9ffhxtuYOGgiL0573sG3H4L7M4ceABXhXPu9wbYyL262eoo1X8iu8WlgkPp
iN5QMV8hm1q/qDlBTzR6UBHvRX++qBJVnAzRgYd91LltWGPgeB9EnFNVDav6
8PHVNMFbZwewRJOub055WiX52fIdYZ3gcfqypHjsPLYM2rrt7zsPqbWT9DOj
y6GhbDnvjkwcFm9LdD1p/Qp+ao+rnaYNIEejZOfNonQkzLm0mfUiDenWr3QV
quiwcFN8fuhmDniqyuucF1D5yWeVvG5gAwRv39phtukjCmuG4st7cpHkvnB2
VHQuBGsdaD8P5eMjz6tz4Yw8vDjx46TEqn50JApH1lR/RMihnf2L7PIQsuT4
6VN+TNjbzA/mf5SLVk+zc5su52A8J3m6fACVry7fNWdavUN0bQDPhWmZ0Pdv
7Tc+WQdB1zlXjlxNgrjqlEtNPxMRKR67LWlOCXhm/9xzVtoL66OStddN8cKn
G1ulKw+m4CVvSYe/9QC+SJRun24+gM2nVp4bMufAxaBacXBXH5Z7e62tndcH
RfWMJsu5/QhykU+x2dSDNoNzAhXPu7HT/kBqQVYvOveJt70zpzxVHNdkosPC
vekurQsse1Gq3i+mzGAiJnybqu4OBsruf1dV6+iFWeevb+AwMPuL8lmxHhqW
6lpJKw704fxeUe11lxjw2NFu6reFhukrR4VOLh/ATo9KrX5rBgTSzn3etpSG
Fa8+SrooccCUfGRf7cfA/PVCn1UP0fDITYj3I4M6b6560XOuhIHC8/Nubkmh
6m1KXYH8bzcY7lO/Hz2ZicibB1+ZT6NDV0U6zzzPC9OeKzXto/JJi/EMJQNN
Kv/onLufqHQDJWmaTaqa1HmjTHpjjQUdnUMVowp/XND3Jl3lznYmWl/Ovn3V
ho7gZLzktzXDcZHD9+w1mLiw+0Xs2yN0bBoqTLn/nQNx4fW6HBUmHq0tL1q+
m47vew10TLo5yLWr93i5gokFg/zJdHU65K0dv1f2chC4/f5J+lIm/pwtO7F4
HR0qGh99RUa4xwMU/34te7aba56R2ke+sxSsuK4bOSs+OT3mBNd9avqIOzxo
MOZ6LpmOl7/0qHz133UQOzwzYmoFB1PJujWRdeva+EZzSyKHa50V7X56D4dx
78vXF+x987I4YJB9nEf2kc/22sDvzbYwIPvOT/Z92xOf6WJDj7nqpH63ecfZ
2HiYk7qaS+pK9ENry02/dJSTOownddictFpK8loO2KRuL5C6Telc5hPjmoen
pM6ZpM7fqQfLdPPn4irpi9WkL7bPXi8/dDoNSaSPakgfWT2811SpGQgeoYm+
U4iZ6Lu9/i7NUU790CB9eoH0qXd60MC8y91QIn39lvS1nprQrX2ldFwgHKAT
DrRceTT4qr0NbMKNKMKN4Kx1sitoVZAinKkknJEq2ZfEH5rKxaXLRy5lGT/o
hSnh2FDlBMduZnb3NL+lIYxwL4Bwz1Qp9uxzszrkrJngZLj2BCelnE8XvnmU
CB6HCa5iYIKr85NxZs9CNrIIh0UJhzUe56ec+dCCEsJtacJtDWMDratnMvCc
cP4R4TxdLktAwZEFJ+KFdOKFy5YFH0pz6rCPeGQX8ciIv9dqH10Oaol3lIh3
bBa7Bh6g6q6deMqYeIpPWv2OQ38VDhOviROvhRkqvnUMjcFW4sFM4sFdRt7l
D772IpN4s4F4c1lnrf7kI0wkHZvw7M7uCc9OXq5pOGNRKyKJl6f5Tnj5tLfN
hcHOXNQQjwcQj0u/cjlTRT3nIPF+IfH+/yPjzKOp7N/vrxIqIRokiZJUKpUk
la00KCpFoSRpoFmSJElChgaVJEOEhCREhjKPkXmej+OcYzoDJalUv/v5nffn
n6+/rHVn3cP72nu/9rXW81A8W3vKZ3ozlElP2EJ6AkNDQrXmPg9fSK/ImcDv
FdoJi0euK9BRRXpIGOkhAsV3VCfZF2CM9JaTpLdw30Zs88lnwYL0HFXSc0yK
ol9IXa+EKulFcaQXrY98d1/hVC/k9/B7VAHpUXti+w1Csqrg4tXT+LH1E8Jc
5+1XpvYvxlGLhgeBXfB96RA/M5nar1OYbxT9u5AT9kk1J/8tbO6aPpzpVwdV
hzDJyjftqHvDjsj1osMiIcxUQYELQSHlo03FjUh4faf26dFMGLxYZD8mSIfD
Htd3YgeqYVe/rFhTiwltU9eTI5rFoEVM3iqgWACLWVmlExZVQeSWdXW0FdXv
a4IPr5wbjCpXJxHGuX6I3AtSt/FvRkXVecuVk9hIePb2VbFpCwRE5lreGDBG
49GWvZupHBUo8F33Mfo+JCcM2/gf70bzQNbXc3s78MHxZAg7kuq/gm5xUjvL
8Yl+4sPLo6XQ+rHfvHpGPXY5a7PjpDl4vGnttPxhNm5Zv8vQccrE3Pk7Jo39
YSH5d+aPlnNM7D6WKW238CVmR3retvrHBK+vWfzVBGpfvjttieNTLtaprTn+
7b9+qqvBeqpHzWma3bptnT14GiN1TrisD2vXL/71cG4fksynPtv/vhXnjAK4
R/sGsDb/s4xa2ADkn+8w7RBKwknL509YbA7ecCqEe1o48HbRMlkhy8KlJXcW
WVN7c2jUliWG1E9Vkzm71L7XgnZ2n/93kzcoTPlccyIyFoqsuOOGKlRfL4pw
ufXyI9zn+6at2pwB1T8/X4V3UHsB9Gf4LytAosCumwsz8iARrqrvLkT14Z/J
7iKvP+PIZzPT28ep+dAYFSZfuTDRHJ76QqQCflP1dhf8KkV0xs0j1zyp3D89
dirVpwpH2Cv1ci+WI+Ba5XU7z2ao7k+3vkDNuzR1Y4FQTAUEvm1R3+iVB5v3
FUaZO6rxbN6ZoVD/Chh7yk0/RO270puel80+WoVRoVvRjxXLsXzju1dHN1Bc
NFLeLKtQjgOFk5tStUqx5PijFfr0TthsiN15q7gYjl8OFz6uLIT3aaknn/3r
qF4QtEe4MAdym0Lczd2yoXhKsPiJbiGq1oxE5j9JgFn1bsXJ1L5r/Ee10e/k
S1z//jFLKpbaT9mRT0JjeEhx+65QkcCB48sACRtKD0ZWnrIulB4Ov6leabF1
AA/v8PRa7/Vj5QXFXY+FqJ+nzugHc/oQ/iKF/nNbL7YYdoXnK/VA//DO7KeF
fWj9kHTh0m0WWO2xFlGfGNCp1xHaEN+PhOfBKnrnmeD2q+//otyN6dXh+Vwd
NqY5N5c2CDCxa05FSJIHHTkKzzhXRjiYw9JcliXOxHHt54+OhdAx7yxn/v39
PJS3bJyeZcZEqQZtldeMbvQv1Zrw5aAd7qa6YCCVCabkmZ1/Aihe34754S34
CIyxNXLzJrPgmFGsy5vCwBo/l8N5do8hPxK000OD6i092ZH7jRgI0ZJa7SV+
D5Pz7KNm7meBXfJDNugaAysM2KutG26iakxdKd6QBegoKB+8wUDbG4+VQnIH
kbvzS/fYThbyH++Ie3CW2jPStvKGe3nY8Pjw8i3qLCxNzwoV3c+AutqF488p
/SYeLndrWMxCcJEcjm5gQGVanaVWFw9XvWJyV1F671617rD1cgZiFp53XjY8
/rpffMaSSW+MkUTu84Lcx7hueHlI7sVxz2UxTu3at9Z23HvKR+3MlJhmNe67
Yj71jtl85407h2NLglcZFvAgR87tFzm3ye83tuY+5KGbnLMDOedjv5dzTM/z
xs3l5/ut0a3Xxs/x7Z0/VkaNvHFzl848sNWs4immE53oE50oft3xxEI/AUlE
V4NEVxnX+uhetEx0Eh2yiQ4F3wyy5u0sRDTRrS7RbbM/fWJMXwkCiM43EJ2v
P3e44sGuErgTX1gQX9w6kL7t1kgeXImPbhAftTQbDj64nkTtbXzfvSa+mz94
L+lDHxcGxKe7iE/jWFpirW/74UJ87Ul8HaVvrx6UzEQ0yYFjJAcO71vf1yxG
gyzJjX8kN7y9XiZvu18La5Iz3iRnPD9Ne7Zvcio0SC41kFxSunNpTeLRAYiQ
HDtIcmz2Nun8wo3dMCC5d5/kno234K4ZvDrokpzcSXLShZc440pLGNJIrkaS
XK2L7zHNHGMhp5Cfw/okhzUmiyZnbG2BBcltPZLbLkGrmZ81347L+QxDu9ZT
lJ7MCRdeES5E/nXd96q4FqcIR1QIRxJUGtUUszm4R7ijTLhzpcpRzViJjqWE
U4M7+Zwa6Z3z9NqWqnFc0/Xa+fBodBQWEg7mEg4KLrPLauJwYUa4+Zpwc0Ay
1vOaARs0wlm9bj5nIdhoMOcaE7qEy/PZfC5rT11fqPShAQI2fI4PEY4nfVoq
ddSL4hvhfgPhvoTa0hWytXQok54gUMfvCdLD0zsuGRRB4jC/V8hH8nuF6Pcw
mVvcHriRHuJHekiYUexl7zv18CO9RZb0lgLvh9v3GFH9gvQcbdJzJKKV0pzY
LUggvcib9KIv64bH8hdx4eLD71FxpEe90oEhvbQNqitnO/aHt8PAdPd5mUMs
1DVnht4faoDLNPa6aWdjsXcmNzCorhtaCRUf838OQCLrwsZDAwyIpNtF+6l1
QSX57a5g6vsTzh9zPN9YhU96OcrcL22IU5u14ud3DlxWd7s++DEAt93bvdzi
GmCRLLa5ej8Nvv2vtyV96YDB+umX7WMrkRN5u/3fskTQWq5nbi5Ihgu8Ip5P
yUFgV3Hbf3/nxjNzXp0qNR+zL79qIi7xoC2eo7u3sRRVkZf0mZWVMO16uqKM
4hzj2/NYu6d9kNOVG9ATo3qZ7V3ndPFubP02KrPWvgXyKQPWR3sbYZdLFzQ1
qMPsgcUTDO8n41fYWrXPb7lwNtaUDYriwjhgh7/TfDYmfpFxu2nDQsqJelN3
qp/VT2sW0hbiImNn3eN5a3rw5eQuz5mU/+c3MmgRLzzw5euZfUumDODiSicD
4aB+9CdMrWnc2Y8QX/cfx+25EI3ZHBN8nosEg/Q/ZsWdoKVeUP0XHIHdIW9b
1REBaduopS9eZCNBUrl+v94nLPwXM2VXQwYGc6yYoU974FK94S9zQxH0oiYL
eDsXQFFQ8eSvZw0wyY5QvW5bjlDOLj2uSynu9UYYvjjCg8RypWVZvGpwXwmf
SAmthLzjx8gX1F4ru3jDisSUOrDT463Uh6oRUJegIlpRCxF1mcHIkQa8LFKa
eTa0FvHh3+Z7v+ZBPvuOSXxgE24M/lsYXV0Hc/tjC5+LUHktmWe2z7sZZqpz
RRpu1sOkl9EZKtkCjdlq0/MSmnGNMbBf73k9pIuTp54PzkPTe6WAzX+a0HZm
mXuidD1eM1cGWXay0XZFRvfDiUYsM6BZOPFqUZL1xmwogYmTkyXrvdvrYFG9
gLVbvAbikUkX7mt2wCVSNN/3bDWChu2P/auowFypBaWSRjUQ2Xl5rphSKQr2
PbioVFaMQBs3P//Z2aiamjQzIi4bh2W//qG3ZSJF/lKMO3U+Lt+czP+VhsFG
6I6hgEcY7kr85sm/H8C7DaorxQY5GB4tu7yjg4PUkHZd7qw+vJoqedagvh/p
VWkeNkf7KY6smXAvg9IBbcuvg1N7ccC7b1SqmYXotzt2Klf0oPj1ZD/ZfibW
tN98HEzvhsyZZ9tzt/ShxehjQmsyAyN77s6SlaQjm613zPURtQ+2ZS/JPs6A
1pWskCjfLtSITQ2e1MNBf9036U2vKE5nLvvK+dmF97uYmS2hFK9t9obdWkLt
lf+Wq53IpePnWWZW3fnHOHbZWbgvluLvaGLRF/duPH19LVp6NBgrZvsu8JSg
9oVW73h3GQZ+BF6cJtYXCLiu9C/dywIvVs3/gB0DeUmzX74MfYxTK1VVt1qz
IHSIPakogIHW7qVZZTdu4+S720s9TrFgq7FXosOP2i92n1jR8VIVt5+4GTtT
PWTMOzN6jQMD5ULij1wbqJw8/O3s7PUs3IvfU4MDDJyumW9+s5iHiXb5cakL
WHBW+rPKTZWB2/IWKZo11HU1rzcbJVmQ8di35Y8sAzWPyvKjuOOvT1h5UW2T
6XFMIve5Re5TlM3b94puP+65ZuWzhwy2OI17z/PPnRxSs+zGfZe/oWbBwH//
fyU5h4nkHGxMhKslMnjjzu3M7h9fJ57jYTk55wRyzso+kQ9TlXnj5lKw9ZMm
pHnwIHNMJ3OMrhM5/JzqY31k7uZk7g5+bz0MpnginehkM9FJjNe7Bd3FCWgj
uvpJdLV0T5XrSHouvhAdbiA6jDsS3vdrtBS1RLdmRLefIm0vvztRhXdE54VE
59AP/NPjTeU78cXEn3xfTPXyatsRXQbtYb6PlgnzffQiU0rbaW0uVKfxfZdA
fPesZ17N7od+kCc+bSI+bbt8acUKKm/vEV/HE1+b2w8GjtSzYENy4AzJAdZb
pV3pdBoYJDdUSW7oukbvmP+iblzOSNd4Kg0Ipo/LpYUVB/4wTAbQS3LMlORY
ss90iYQRqs+S3LMnuScgteVjObcWY+v4ORlAclJAf84dh/ke43LV7+nrvp/S
LAiSHG4lOWwyzetD3rImGJDcfk5y26XP1WfdPT8q7/g5r0pyvvQPp3wiNVcL
woWWv3wu0G6qrsupq4I84cijYD5Hnq4u01QRZcObcGc0ms+dG8aLbu+XpeEj
4dRhwqmyjfWpYbQiJBGu5RKuhZ13snF/xYMo4WA+4WCkzqXH/4LeYdZLPjcD
CTc1NpftmTczGwGEs2WEsze3toYevMiBAeHyIsJlFY87tqwCOgZf8Tl+nXDc
b1XAwRWxnxFJuG9FuC8zVnlBZXcfaK38nhBNeoLvJpsdvVcbMUp6hS/pFSpl
+wTDqB6aRnpIGOkhCf5096NmnZC/wO8tTaS3uJgceqZukABV0nMkSM+pWLy0
NVGtGwJi/F50iPQiB61CiSHpHCSQHmVGelSZpILqhrsMpIXeExxp6kP8d5mJ
u/V64bu82fmRaCFUZ68x0vndhAxtcau9vUzEN97Udh9jQuBpwwtBDyrnJt1X
nGXdjTBNFTfXzDJYF81vaqHOqWDeF8mLmTQISu7+cn5RL8Ky5aU+qpfA+sX8
TwviW6FxODOGRa+D7PKM6QfW98E6ScmleVM9XhQXzv0oz4ZI98hZ8/VNiC7b
LxBRWwba1R+fZ0xpg+yiX3UFflxUBTYur7yYAoGVek3uVudxj/eApWFPw+rn
k6Stb3PQEWI2Y6kDHdpiciGhi6Oheku3fzWHhbiTsc6aOoU4G/r4W8vRLgi4
/SuXzm5FYoho9gKzPkyese3dxzoeDsp8MHpB/ew3VVUXudCFnE+vTR6couPB
kKDH6cdtSOO1nP+Q1I9jV94emkHvQSwjY/6eCBa2vYgTXH6uEPN5mlOsZai9
c/HIG/k5PBSnrmI7/OCAlqtvEbUvGwNm7f2zvDPRUy5saXucDusnL3Q7pMoh
smpyF2dtKTRCXA3NpYtRd/Pb111BtVj74IL4FZlqqB+btf+IUB/01TbdO/iu
EZ//3NArUqpDwfcQZau/TVB98O3MhVmtiIrS12h6Su0DrWufv1/hiQLplX0D
A+3wmrzqose9JkjfTQG7kOK3QMG/3zI0yPPoN2fOaUHa7MpbK1fWw+z4rJGV
wzQsFuu/57GuFQI7hlIuzLbDr1sxT944dEFet6Gupuu/OVZO8TZi4dCMNE7w
iy5sV3zzp16yDSd/y4eoq7dA6Jfgs0Az6pxNq65ez2yFtdrOrVNYuVD2/Pvk
YiwNd0qX7K6pa0HlBW2NGxfY+PLk67TZkzqhk3Jhf/SuZkiemzZBYYSB0UbO
sYaMVlyij9GjpzTCt+C7nv0y6jnFiSnqDo34E/70gPPfWpxNXvZDXacSbbY7
p1pp1WDP3eKpK6dXoXpoIKY/JhESvMEyDqXD/cstAs+sKMLaFK10BxMOcm7d
qBTNSsbHh06gD73H55ZDc6LN+2CTKLmI2caFnfmmqoBGLnr95qbChcqXI/me
4nMGcO1bg/D1N/2w2BZUEjydiaL5MZ1NUT0QWz9H9ZEhC60Fp0pEj1F793T7
mJniTNhm6wuaB9NhLno2K8CrB9oSNnsL07rBfpXxbMSVBvvrpx6Fd/bjwsHe
tzlPuzE2yVHp6y4anIQ6y2niXHzdqO6Qv5QBq4qm3p0aXVh0JGV2WpIDtHjn
l2kNM/Bg8syeG1foaM2xXa25LAK5ky2t/kYz4X1o9oxR125M1Rk1iHCOwoBz
+lfvpSw833xIPlWLARhu8rGNDccW/2hF28ssTJPoU6FTPGd8o+188PYZTrSf
ul3tSXHXqkpasZgBqfd9982n3MEvB66+5E1K907C5pc/MPB1iobq4SEehhfO
0ZA4yoKHbOfcQk8GOgTzA5yoHlIxUWdKuzrFyXqRZZz9DJgu024riuVh5hte
5fbZLCy7f0OyaSED4UWt5fszeahOojPzJ1F7v0qq7ZgwA5LVxz9P6xl/vZy7
8oX6sPW4+7wyFvO698Zl3HPpZ77zdPe6j3tPtxKtymaL2+O+y24u+2Go3lEc
J+eQRs5hrWmJe8t73rhzsxjNuf9kK9WLyDn7k3Mu1dRvNKnnIovMxYPMpfLG
YOmsAC40yRw9yRy/1/aamZVzMUjmfpLMfe/TmoeBFbxxOqkMEF/4bcI76BBd
DRFdHZI5m3H1dgHeEh06Eh2a9iy/U8GuQgPRrSLRra692iXWlnpUEZ17EJ1v
ufb5YhGjHi7EF+7EFwsbvjxy+VcDeRe+j2b68n30yS8y/Jh7KXSJ71yI74YP
TlXX3JSCQeLTo8SnCbfzYvNKOXAgvhaN4Pu69sC6rWtW9UKgiZ8DV0kORMyf
3FLQ34U2kht7SG5EL+37kynVgGUkZzxIziT4jm1W8vsEEZJLk0gula65+O6m
x8C4HFN1vZTIYNDxl+SeEsk9E73j7IFDtTAhOSlPctJovp9ibSMPmiRX55Fc
dbOTzum1ZiKZ5LAzyWGHIT+XJXYNUCG5HUFy20Y1aPUuam/aRnI+n+T8qGDi
uS8u3cghXFhEuGDtGvvxMasMqoQjrSv5HLlnsuBc7wYqnwh3rhPuZJ8Y3Dhv
cyuECKcECKc0tF1faGxLwF7CNT/CtYtNdVrrXvXjM+FgHOFg3RY/faV1hVhH
uOlOuPlLL/QJb3E71Sv4nD1FOHvYs3zS7YelyJnB57Ie4XLk6YMWq9/1o4Rw
fBfhOC1/R/tb8VZsItyXD+JzPyN5iWNAN/W+pCfkkJ7wd5vgvGu/u2BEeoUn
6RUSudWrl/zKgksOv4e4kR4Sd0Dhzb0HVD6T3lJDesvgh8YKfdtKqJKeI0p6
jsbQsHmoRB8sSC/KIr3IxEDw1nehOvSSHvWB9KiW5F1hrZ/6oa093/Vpcx6y
itr2f5Tth/VWyzQlAR4MOhvVDTQp3/07NfdvQw/GfBMMdjR1wkJVadFu1Tqc
dBJTTrrKpPb/fzNXfwxFmnbOgKwyB2a+Eye+5NBhkFD8PWwjHb3ek+IsttCQ
bDu3u38VDarxvAr3H7kUX5rDo1rSERm2NOTJ/hYUqLbtlp/GgqzroN4eNhOb
LienP7tO6VhniZg6tceYmKkFbthVBd3j3x6umVaMibzUpdIX+6DN+RruENEH
lyxWpHHhCxzfP2lM81wDPMWUnMvHaiH9NCSXVdOPdxLm5sbiHFhz7w/dP9EL
Z/lzFyUUqPeb8mYTe1MHBrstzKcmf4Do3ht+wTkpCJPianv9TcCHDlmVR0lF
0JVL7bHQq8bEG+yYVKrH7l13pW2mVD/2HGdvcLDtw6q3BYxnFR2wMWq/+sk2
D7kRHzdzN+bC4M6RyJ+JKRjWqzS73VaH9rmxia5iNSityXOMVWJim1Rf0C/d
NtSrP7rmeb4RAa8FA5e4VkBa5pqVcA51Xn7x/WuGWrA1PIenfqIfIzkCG7am
0OHlqzfpsH07Ij/pds1Y0Iqwytjor5oMxEx8ffPb4U7ktIevV4iKgEChzpMS
yt+IrTplcpKGsc2MBrnLTEhP0HEo3k7p20JGKKmHBoOFKy8VL23ASr/8C38r
WWiW8dn3mcpzgfe+Qytc7mOT6U/dHwI9mODzNtHwYBcY65arzbrIwsxjj1QO
VLGg/+nB2l/U71sXFex5PdKMiukFf3etY2EwPtn5RzsN0gyXhJvnc7DQ71Wt
L7Wn166asG3FZhpGX19UkS8bgNvp+uWVod1oT+9tywzqgPTGp7sOTWXAsSQ9
VCCgC9POW8bunUqd0+ZS4YjkZljMLKypmNOBvBMHFa1imzChZtkmyUmfEWbR
EFb4rRFeGn63o0zqcOOS+Zkf9q7QoEsfOvi5Cnl7VqjdrSrHJ1pPiqjoABIO
nBXPP5ILh5fpQ6t25mDNhK29fvEsrGPb2Q3386AnUu/OoX6OHjLNtbDrxslQ
v70lpgMImKexpO9nPyK/BUsol3Vh1mz5hxdn9YBu+P75+XtMrHkg7B+7h47o
fxeCWlu6IWsgLDk7gwZhw2LOQysmFJgia1upfh24/rO6h3sbzj6/mnUK/Sgq
mf7Ryp8ObsWZjdc2tuP88QZps0AuliX8mtoRTu17whmBJsdoaNCiXzPfHo6/
afpdZWUMtJW42N5YR8eGz7wzi8ITsGvDpbfTMpi4+2jW7gXB1PNFIwx2nkyA
eu7BjonUfvV760/NOGcGjIpKZBPKo5FWVpRyLpKFue9j3bS4DHQeuRIwUh2M
0UvBU2aksCB09Z7Zp8lM6IWsdLOIcIXv0L8zBkEsvAhNWnS0i4EezLS+S+Vr
yfdZb1+dpXp3wWeZVcEMfDaSv235iAe/y71zs9RYOC3S2texlwFvxcj3r115
mLJK4J28CKWf345KhmIMJA/vkkgK4yFa/6LFQh4TfteldIf6umFZO2V+Wtv4
65f3zb4xFHEZ08h99pL7dNa4cm7U3h333G3tI7OlDj0c955h5f2px177jPsu
GVHad71PV8edg/d23k3bFB4+kHObTc5twalNejZzeePO+dmVtX8Vr3Cxg8zl
DpmL04uBWde+cfCLzLGBzLFNKO/xxwEOlMjclcjcZdpuS8e2clFIdNJPdPKj
77mt1uEoLCK6CiO68gtcKnRmYjHiiQ6ViA6lD19WpKnWYzHR7SjRbWz3YG3s
yVZcIzqPJzqPlS7qeHemDbrEF+bEF4P799zUmtOMQeKjROKjO39urGoIr4EJ
8d134rtkJwvl8sp8JBCf+hGfLvc4vFDxAw82xNdfiK/P6H232ZjdB1eSA7NJ
DozRZJziF3fDm+QGg+SGv1f9ecOBRiiRnKknORN5bPveW7U5aCC59J3kUkdR
5xMaYwCSJMf0SI7FBc83jRPsxg6Se8Ik9wafxDyrX1SLZSQnG0lOSmRu0P3n
xYMkydUwkqtxXGtd2QEG/hbwc3gzyeGEO+sP5M2vRzDJ7UiS23+r3dkhhVSv
Jjl/m+S858sj4tds6BAlXJAnXNCw/6OteKUIKoQjBYQjffXOc+2P9eIL4U42
4c6IqePUN0n1aDLkc2oz4ZTlQ7uynF0crCZcW0q4ti8mZHpyAA1POvkcTJPn
c1BedEmcr38zBFh8bg7t43PTczO9cSWNg1jC2QjC2WrH2C6vLkp34nwuaxAu
b3YQn77dPRVu/+N4Lp/josZ/b1uWMpBAuO9CuH9SR940dGcptpGeYEB6gsgq
eydbfaqHS/J7RQ7pFaMhO89m99VC3offQ3L+10Nmbrsn5zqAXtJbTpPeQnth
MzPLk+Ip6TlnSc/RFjC5pjSRBwfSi5aRXmSe+rqxTYoG7S38HlVCepRAbOo3
5d+hUB2KM17X2gEup/kvKgeQ9tln0p6UXrjsOyUG2luw26oyLV71Qb5yVZ7O
i1rYFOhUO4Ux8EE3T5F2vAdG6hWPo//0g7ahsGuxbgW873nYbfzNgEligrmy
VguUz7bq3r/fjzZ/m6cFJ+lQK1lrqu/MRU7D7qfJTS3ImR456adMJxLWJUXv
ae6EQM6Df8vjt8DaIDJ1y8cmyEtvcHj+5RWqkjdbCS6iw+60fUGPZTUMDG2f
0D7SYXG4o2m+8CeMKolp09blIqQoLCX3UjbcTrTFR0Z3I2mnYGhADhfa0zcV
PM5mQMDy6M6xmDc4efjNnAiRHriNlSi8ic7D4L4ZUZNrGpGifzJXUrsOIoe2
MvPOdOHlXpuf3VTP9di+xMTArBvyd83Xmj+rBO34/Ijgx++gnxy+a01EPN6q
zHz8RWgAdvtLOvM+NCGccUSPw6zDbi/9v5YeFdB86+nQTfVNR+MverZtbfBf
5NDpxeyBkt7Hhr09TDxyWfSm/B0NYd8UArMLaiAncdzdzaIX2YnC64wX0DG8
88gN1ZABOK+9+9tLoR9XtXK+HZDpRoH9zFY/dhuWiVwqrD9FPT+zupdd2g0X
aZnAVMt4cH9lOzbqsjE5rTNt/zYGaK6vlvweYELSmPPNs46NLHPX2YseMKDr
c/1BVF4D/IdffdGYwsH8gLYrFTHUeR0sv87Y+xjSX0y/TaCzoTX9a4zGUwY6
OI+kGRQ/Gs5UF10wZkPUKn+iuD4Dmx6H1lsZN0PZesvDvxYDMInyUt1f0g3t
CwFGjy98gvzzQ6/ERvqgW3r/1OGJ3UhdL71z7q9+/PKddnG3WC/UBpmtnoNd
cLDR3HKOR8fIQFfgz5dMHBPfvsHbg4amyXrD/9oa4BdxSHBXLB19zRsHna3b
cdsvi8bh5UI5R+15nE4HnjcYX1YvacLjCXUfXiyk9p/oyuaeaQ1YuP67btZQ
DW6b/vA2vtYLac31V5JOf0YmfU9W+/JiaP5cqzISRu2JJ+XOJFaFo2ZLbZO+
YzhsuXNWLq7sxGnvV47C7wZgsEGletrRAVjK+S2lv2pF702vFQtNWNgV73Lt
w20G9JNcpkq/a0Nu95SMRavpCG7l/RRqbYWmiIrPizA6ptsHVvzzpeHrLeeH
l9vrITrxuJxmfR8EI2azC493oSKt3il/sAljJgtXGZXzECOyzGhofjdcv69Z
GCrcia/TD/VU/07GJNbyPKVPDCxreON0bD4dimf6fC2XZGAotb0qhNqvhCwf
+V2gdcMzJ1c8nsrBszMXCEmFsHBSK4De0k3lermOrHX0O1htSTBYzWbBLFpO
U8yQCVHt3RNfpochRebcWQceC0qfmbovjZlo+S53J1bQGdwvif0quSyov6fv
KpnNpPIsSiohioclh2u1Rq+zMPW2xTLhJAY83G21Gw7xcGue/V2f//5eerNj
iMVOBj7NMbJ+rcfDcW0JS6dvTEQ6HmbsGuyGcKW/rKwjDyYW1W81a5iQG6Z1
xmR3Y9uuHQqMCh5MyXV5ct3HvNWMd99h3H0MFihdcPrkO+65d0Rufpf08cdS
8p5i5D1X73s3tH7x03HfZVppPPu3oQdSyTksJecgmfczdiSfN+7c/h5p5Z0a
5o4754X5G52vT+aCQ+YiQObCLRbWOruUgz9M/hzlyRzba+hvpT6yEUnmbkvm
rqU1K9/dgoO/4XydpBGdrPaqvdh78DpmEF0JuPB1pTZ6w2POmxJ8JjpMIzpM
4r3uy8tqwS+i2xNEt21R2b+uRHfBkejciui8anDdgNgKKm9P831xQofvi1SR
YesL0jRYEB/9JD6ymR7wWiqrCbrEd9rEd3HPF9a9XFiBbcSnccSnbczsxzMb
3iKY+Pob8XXR4aq7gm4D+Ety4CTJgV9OLZNdHjEw8RE/NzaQ3GiR87m5VKEF
KiRn9pGcSZAP7Q6cUzAulzS/nk/ZY8sel2NF6qqz9I26x+WeqGJbxfoDteNy
Uu3p6jbPCzxIkFzNJLmqppmfFveCAQbJYQGSw00/kxOmKNdBieS2LcltA3Nz
j7q5XDiQnLclOe9QcPVxihfVLwkXMggXHDofS2r55EKWcMSHcOTvZN6aMEp/
coQ7hwh3jqaJb2GrVUGNcMqOcOpE+3ltK/NeWFjwudadxOfaef13qeWMfOQT
DqYRDoYdVb6iLNNLcZrPTdE9fG7qG95Y56jSDhcrPmcfEs7ezbm/vv7gAOoI
l6sJlwcZd6+1uDRA3ozPce0lfI5/Y+l9FpZko5dw341wf6XpvGYtzxYICDz8
/z3BhvQEy4tO1uoRXKg28ntFE+kVvYyo5aZU/7EmPaSf9BCB6/nnNedFI0eT
31v8SW+ReP26aoBGhy/pOZ9Iz5FXKk4+OqUALvv5vWiQ9CJz6V65DlWqT5Me
NUJ6lHJT1VDZsnKqX013bZ1A9Z/j16yzT3NwONd/zS4rOlSb1b+P1VL9pjtI
ytiY2vPD9vxkmmRCW/b9zcVyPDxYMtfoqGof1L7qrhsYZsDioJnF+39d2KYj
/7uijwWBQb39PW5fIC9eVfozJR8W326WTLjOgOeGI8bPrveiKvziyIyCHihe
iElJrOzCqNHov5rAWmgv/P1+Y1odBK+cV1ZJbkdnSIPqnmkDuOdkXLzagw21
n+WsP98bUBJsnW6wqgm91049KRBso3qZtlWXWAUULqhuCcpnQ5bx8a6tLcVV
z99vmmakIPmXucv1E80QcTNZ1xzaAt81M5fpabLx4Mbr4E6K7w5tz+6+uN6P
n/tW9MTu7oaF7+ukkc35MDN6Jmd9rwmSkw9XGhfXQbvt6XbjyznQ9tTm7Gzr
gPYD5aO3VjWDxnHKbn35Ds43LKM59iysTuwJCBXsgk79Wv1Cg0ZcnMbwqaD2
wdBzd00vF3Tj3cD+omLdfkyFfm+rJxe+F57Hu9owYe/3d2hDdT0EZq2pmVzo
B4s/x34MuLDA9pw+ZVUkGy4f11579TgBxzZsaN6wtgcWr7LNJ5l0QH629hu3
aWmY4JnkrxHVAwHlubKLRN7D4o7+v4lnPuHNZm39mF7qvKWv3K+g9jcBjTKn
z78zsWGZ6VRHgV40db6+NfizAWGOSutdyj5hW0xp9k0udZ8Dmgfx/iEsJIVO
/k1Jw+q2UM2TCT2Y2rbc1VuCus+vuXn6rol4nrS+LG1LD3JOfCsYO9IEl6Lf
CwbfBaHqVF3ltmAWJDZ+cv0R8AFT9zV/VJ3Fw9oVNb0T0pgY/vdwve1YH0p+
POvjlbMxaYrlsVhvBiZHrgqMet2FBnPXt5v+9iHTUmllh1A3JFl2p0uja2EX
obDjv7+Ptbk8WDBVpguf5r1+qaeSjCzZ6JVPQ+kYHDzjO3q4HWv6g84tzh+A
7rWeRrE/rXAVH/tbtr4Ra84c7tj8ggnfr7ZKC09X4y9Py1K4pALfTjVMuapB
g+/E77f8v6ZgZe3w0rdi1F72JtqHFtuIJMWp4uEabCS42jr4/aL2n2OHNp0V
rcGd0strZdyZuJXZ6le4pxtZf5N2CqnWIOaHnHh5ZyfaDHdHfrOshWb5yfRd
Fu1wNjv12kqtDdEpabJz5IrB/nu5+N+vPhypHH65eDINjl+bNKen1WLQoG7y
aelEWF4q+PKb2mfvHZOZ1/C6HYtGFCzD1uUjbd8+2cpmigcWZtMF9ehIrTOt
UlTJR+qFHwo/trAQbjGInuMMSJqrlg+JZWHBjtmNX4V6YL90z6UDDlQvmmfj
12T1HimVvn0v9vRgH21O3ddmJqpGfGe+Mw3HmnrTQB9qX0PboMGHbOr64/JH
QqNnsFr1UsQFqm9Iv7yJx1TfWC/E3ZB+nIfDB7sOXPVmQbv1lYrDZwYUJOLc
bk3m4cBA6Ne4xSxoqsatVNjAQMvwA7W7E6h9UDDTUrSBCYeXjUt0CrsRbyoU
Wr6Dh1P7zsaeiGWCcaOsQti9G4/3zZsZ+Gn8dQFZlZ+OzS64Tu7jSO5zvFRt
gb7SMxiR524mz207qXBxbmXouPds+33zrLhB6LjvStBAXlPJk3HncHbNxZ/C
LN64c3s0+EHsVAt33Dm7j13mCD7h4D2ZSzCZi+awaZGCFhtJZI6fyByrvPvK
RtQGYE7m7kzmbuN6refZmgEcIDo5RHTCvbod39o4uE505Ut01SmoQReYWYJP
RIcTjfg69Hi+Z1Hrvi4EE91GEt0muCi1jfWwUEl0Xk10LlBV11X6kwWXSXxf
TKvj+2JJ7PKMh8uo9yY+0hnk+0hAzZA962EnHIjvIojvLq3Q03HLr8MX4tOJ
Q3yfDh3y3X/2Ww5ciK+3EV+fky3zsXfkgE5yoIDkQNO9wi+By1moILkhRHIj
IzWsLCmhDZIkZ9aTnEn7l62wXqwEAiX8XGogueTiEC3epcKBPMmxQJJjQns9
QibUdI/Lvbid0b/7O2qRQ3JyB8lJv9lvJq69zQNtPT9X15NcHRZXyRG+wxiX
wyaVc5mWfhTvSG5PIrnd1OIU/PsFZ1zOy68IM/hbS4PLTD4XzAkXBj88Sb3b
kYExLT5HvAhHglMEhE/MYcKccOcZ4c5ygzQWra0I5wmnlAinBGaxTEOMqT5I
uKbxP66pimyjU3uBDeGgPOFgmGfgytjuAUQRbkoa8LnpN++QoavqAMIIZ98R
zmpIWN6/PbMA1oTLg4TLRq5V0NnbDV0HPsc1CMe/+jmNLO5ORwDhvgbh/s1E
vQjj3m64kJ4wk/QEac9Xm8pq8iERwe8Va0iv2FY/ycV/DcUpCX4POUt6SJP0
k/DwuRUIIL1lL+ktOZPeLi/q6IUB6TmPSc/RnXfx6dLZDbAhvUiJ9KIWvQbt
biM2bEiPYpEe1eZUE3hhdStyXDrWP2zMheLTn/fPsbgIKJxguF2tCbRBr949
nSxkuT1RmjPIRtgw80OfGwc2j2d2vZCvhWqDWu1eRj8GJ4+ajLh1QvrXH+6J
i2w419fanvzUCwHdTcabZ4XDwvjGkrnpHbCQ8Ff0uMRC9HH5vfPt6cj5mbmt
sy0RJU8m/pn2tRsSQsoRvNeZEFnzeeOxPgZK8P2RsloXmhwWu5tT+Ueb+zqU
8/MLoo+srv2n1Qb7RxscuduL4FdcOOXjNMp30+yfVS+uR4x7M0+9lgnPr6yt
1zTqQDvukHnG5jMM1kZYLr1VALPVEiemHulHjrmdtOC1CLjoH/X44UOHRsle
iaVn6yH6mRHooTCANgXBE1qL2YhbXT74o52HTYdeXtKcT/lxxnd6gWUIfDMn
nxVo5EJs49ZzpwOoffSO1Mcn2XnIaQqNdDichPYxhdtFe3vgdPYKb/BxG2zy
/G6qHCpCwBj9/f3WXszdWnIvQ4raZ/7GzzXIrITsmL5qxkAffl2yMJq8ogkB
l1bOGv1Tg+rLz92Dg/rx4E1G7Y8zHPgm+JpFfa3DQF9Vb7cwlXsqRg/tDDtR
9exh+vNJDbA6put4YOsAaJ2vE1cgBdFSKdNFjRtwINHASsZoAEYzF+sV7aF8
eeVP64qxeiwW/rc8U3sAo7uFtVyqKP28Gaav/1YH3yUuKpNEqPuf/y5xTdIZ
ORdZr+aurcX8y+ccl7/uR+DKso1PgpnQOGeUr/G2Cqskb2zymdsPFZM3ohNE
GjG41tv3bmgpkn6ld+eY9FG9a9PqlIZ4qM61ujs2PQ/hKrJjPma9uLTDbL5d
XS9c2oYf3uK+xfopX5fqyvRAZ9uqaUnraEhSSMxgxnORNDpU9O0OE2riYfaO
clWw8Re8eKaH2kfE5k719OiGSLzGYeZxTwTkbLisdYOF8613nysLd2HP1dyJ
FZQ/ki4Mbss40oV+31NBhz61IrhT9VDzXTq1V7wUvDrUgHLzoMSAIErXyP3Z
8bsJ0m+GvTeL5OPKkdV3jLNyUVWx7arc7HJMltkhwFnJwdZOLfU10zkIuPCj
syYjA59yg6TD6QzkHZw8654JHdrXeBFxdgmQXr5W0FW4BdFFfWa8d+kwGCul
+1tVIT927XF414PxuHSv8hIOxNTnlL78MoDUs4o5RpfbscnZ3iMl5AvUtD2y
b176DJ+iO/pSgXTEC8jOnKvbjsGIXLvaJZWQfJ4d1Acmkqfl79jEouNVwAPh
GqdSPIqqFLerZ8FuFfMH1jIxNEuaJjwtH+vHLu5zDu7B7k9b//lsZyHIYXjl
2JcPGPtXrf++vAfqYevNf9xlQTgxXPxOSjjMVlXdSQvowQ5Nr5iL2iz4ix9u
VmrjITe2LytYqQdyU9SL1SKZEKQtt93+l4tq5c4BlSAWlu+YylrWxYCbtnQW
15+Lig7NTdNnUf5/p6gspcDAVZ0H3vKhXLzacmwVN46JJp6CVYlnN3rqfb6d
msTDd7/mP/vsmBCK9j1ToNYNvblvbd9E8jBCrouQ61/fphstqvcYdx9jHfcH
W56Eooo8N5Y8t7hknvGP6VGoJ++5mrzn8x835GY+iqLmwv8uefJdD/5taY+e
FzbuHB6uHT3whH0Zv8i5qZFz66+bXPm6jou15Jy3kXNm2Sg4WihxcJ/M5SKZ
i9Ovn8JamwcwlcwxmsyxxlTWucyiD7fJ3O+RuYs8q1w4Tb0XYUQnTTf5Ovlz
Sc5XPrIPWURXHURXz5UjGh3Kc6BBdNhbzNehsGvgA7UvPagjuu0hulWc4Xru
7BAHC4jOzYnON0tOiHwVzUYA8QWd+EL7S7ylQ0YvBC35PuonPjposeCOhGs3
Sonvpjzi+85T+4XhnsEWxBGfOhCf3lv2a7h4UxnciK+Hia/XzH921uEFDzkk
B9JJDhwYfVeQN9gD7XZ+bmwmuaFx7WVfkF/nuJypYj7wXhX/hep9/Fx6T3Ip
agM7zk+SC22SY6tJjpUaxl7/d4PiEMk9OZJ7buFnPJuL6uBAcvIxyUmRWR0z
WvN5GCW5qkRy1ejwGC37GWNcDo82pJy5crIWCSS3j5Pc7phUtkvGhANPkvN9
JOer+g6mbT9EgyfhQgXhgmr4MsstCcmQIByZTzhSWVx58GYf1UMIdwIJd/rZ
mc+XW2bDgnCqkXDKavcOz7sfuvCYcE2ccE1owcYZ8wwGEEw4qEY4uMm4t6lv
Jg8WhJuzCDddeB09og1ZiCactbDgc3bT6e3fJHpboPyNz2VtSz6Xm9YYv15H
60cV4TiDcFxiWF/+sSn1ezJ87if/j/tBl4+7XeyHPOkJFaQncKenfN96oQHa
v/m9oob0ipG/gdMm6rGRQ3rIWdJDVDdlnyiUaoUJ6S2epLdob7y3LK6DCxfS
c9aQnsMeu1PrtpkGgSF+L8ojvYhmWqWb1PQeYbf5PUqF9Ciu7K1D7Uyq11SU
PaY/a4fA176fMppu0F06u2PD9mK4RKcoDg9EQKwwRG4v1TfZ1XG1iRY9GF22
b/KCkW5Yxil0HnZhI21mRN5sm1qEMcuPfLxQhonrHv54eJX6/jOSHfaKA1Ct
C6n4Xd8HD1+7/KLdVO+qMzH89bUJYQrKr+SeNSOjRm1xzggT79aeWTchloNe
eVtX3zAe1ecmDFpd6EZv0rY3kqD66e0/zjoadJhppC9Pe0FDrlR7mtM6ar7X
WavmduTCM+LpFPGBFqja+z1NO9pB5Yi/f+4Bag/Nyzsnk1iLPMvl4RIKXMjv
DHb4O6UKo/slhPcziqDYm9GdGdqAezX3T2dm9EOgKdFwhd8rlHx5I38ihgnp
3DEd07Nd4OxtEuJ0caBZuzJkWy21X4Q0zxx+2ob7RvbH779kQ/V0Y0AI1d8G
uQmBp0U6MLWzsLGXzYbonhMy71Upvy/9uJweRENljt0Pc18Ojn0U2rl2FQcC
Dlv/TP7aBYXJO4xnzuHiKyf8zeWUZmhwFjpY1tDR8krsnOZFLs4XLhY3ZXBg
p+lzWmN7N44WvtdzDKT2ik+JmuJ1nfCsvswKNO3GXknv3L3hXKim6+wrzktB
QNLNGbpzu5GTXbvuiRcXBXNydm1ay4JuwbQTrlS/yC0M2Iv9XEhn2M/AzgYI
+h/dciK8C6GyqXPYPzgYjReqDi3jwai0quL0SRo+NvQMGVD7ZOxZy3afMQa0
tfRX7vZpR7GFyjGvWqrHx9mtVTlWj5Lyzp87rjfD+lzieZk/lO+KvfZdbwyC
gSaT8fBmHXiX9u5M4vZj7zXV30FuPfA9vWPC0IRypM+aU/MkpA8/TubcnLmh
HS43Prsdi86E6S3tznu/e7A6sObAYp8SME4IG2o/4KHRxe246neKt6ekurgz
udihnXc0+WM/Thfvj8u52I3N+w7o+9iyULX9wPbypUzcMGW2umvRIONXfeWz
UwdUjtCirF3awdwt4zRXpwmaQwfv6UyshgOvapafczmi1tloFnqXgub0U0r4
dhISbsglbu7k4p/PCivhFi4SDoi/3NPNRtTrH28/TGdix0fHhbspHi78axwS
PULtQdrtlja/aiFWXsf1zepHbNGOziMSA5j7LGFGrGspBK66bXHd2IorgWo7
X53Ng2FXzq2GxW1QNLq7sr+1CCm9Te7sAy2IEF3nNVmsG1Hpv1kbazpg82vY
QWy0AbzuItPw6SyIVcuH7p/NQNxy9fdqu6pRuL42f9b7HsTQ3gbOO8mC/qzz
JjNmFsM3qCDB83QvPjD0TzEO9CCJe3j1lYfpeFa75qTO5V40+Xtc/Ub5vLdb
QEdgzkus806IWCfaS+0zr0wft7Kw+eWc23QHHta+UL42eroHkfOTzrwbY2Lj
ww334/7774v2hEwvjGUhct3QQfUfDHhzpyg5T+JidrPftGm/mfB+dGa//u9u
9OluM9eU48LvVe7Up1Rf0pqQdnr5um5IxZoyc95wUXjTzmNkOROLojxMF5XQ
8SJGTSncnYcicl2RXD/6vN7W/tkD+JP7bCX3GTM/FTuk8wrS5Ln3yHP9Ln+c
d1g2ftx7Ws05Hjq/5B3Wk++KId+lLWqlsr4wGuvJOTiQcwhqnNb+rtsXz8m5
tZBzyxyq/P2Qy8U9cs5J5JwXhNc9n1HBRjaZy0syl5e/GDG/NfvRR+YoROao
2H8y6tSRHjwmc79I5p7h6p4/eof6d6ITA0O+Tl77SGV6NdCR6M/XlYgdX1cv
mtfEHzo1gAdEh7ZEh71MhqtvVj5KiW5vEt16Cod5H138BfVE54vv8XU+eMb9
x2NmGtVf+b4YJr4wqHKZ3/mOAxPio0l6fB/tFa7f0OHcAzbx3T3iu9p1c3YE
5dJgRHx6kfjU/PgRqz6ZWgwTX3cSX8+J/8ydkRiPHJIDR0kOmJnqvVFvp/Y2
khtZJDdOli/IvXqiCzYkZ0ZIzijOCJ1SMlIFGsmlCySXwhPvrNumwIMGybEi
kmNRMdtZZwyYMCG5l0lyz2HZxu29Qg0QJTkZTnJSwC7aaY6qIwxIrhaSXK0L
33bNoo2BMJLD+SSHJapunb/7vBYuJLf1SG7He86Z/XonB+dJzh8mOR+g8Npy
dBYN2oQL7YQL8rXVHuvbEyBIOLKIcKTBc+WynYu7oUK4U06407uwtPiUxwf0
Ek6JEE7JmVbYRlnRxnFNW6n0uTTVf1QJB0cIB11Oes1uPZWIaMLNMMJNidPR
WRveUvs54az2AT5nw3wWiikkMKBcwufyJsJlWuWu9u9UP3Zx5HM8jnA8sGBd
yqJqGjQI908S7he7HWVYGzyHgAK/J0SSnnAx6misvUUXVBfye0UR6RXaFh+M
Rt4lwYX0ED/SQz48NWxwpfo7jcXvLWKkt+ge/nBX8WI+RJbze85J0nNcrsa/
GN7JgkAsvxdJkF7UdCTKJFirGr7/61HD/B518qqjZ+LKfhiEhMhXT+6D9iR1
1WCPWIj8W9CT85DqLZqq+9JmNMJf2XvpRhoP55fvKJu4tQsG5W/iQ/ZyoeL+
OyFbiOr9VZfrkwMy0fQhNV1hWxfYUoqf3eaw8Vd7Q/DYSur7mIxnPw2p/X/z
R7lO0X4kTPOj1SuUQlV4otiG8z04Lbn9UHJnD4YmyRwoS6T2EZcTukn9dfD+
vXmF+l0mBvy1m1SUq2Hi/3LWK3MOcrZrvUwfpcPS+dOzGVTftZFxuuO6gIZf
Wz98OK1Mw4jSpizBj9UQuJgpEmHyAU5P8/MyZVrAsnx05RJ1f8HdPhPdcql9
+3R7+2HLWpjQrzxkihdA9av3hEty1F5S1r8lxeIzJIwqHNmGlJ7vuxmzt0VT
eto7Nm1rJpx1T0hEZvfjpLPXImUNDmyESgyPpSVi9OfYpF+Xc/Fl3/kjN4XY
CLv6aZVoYxy8M+adU/rUieGb9rG/pam++/7OdNbmt7i6+/u9uS85UPSOOOx4
grqem/LTsjUeFgcm3Tl4rAVOLjcfVLlR99GbESTg9w7eURP3TxTjIo4r63/w
v9/vk1v/rCUeAbe4i9fVdiJh1uKYCb+pHpP577Xiszjk7N285K1HMkTy5R2d
9ajrqhMk7uRFo27O/X+DTKo32vdb1RygfPE+rvCSbgSaWNPEz1G9aNRyr8JR
UUoPAYkalyYEwOGaDNdmJQ+eJ/cdWHi5BwJLgj+6XL2KOoOpLmJrGHDxWTti
5ED1ipLwtTNseDg5eDf74KlaNClmhJTM6Ubgj7U+/zy5GE5a2rwnhocxuuSy
FdtosOzPDXa8wEH2pRKNB+LUHuvsGiVLb8Yvn1m5XEk2DCbkZ1ZnNaN3ko61
skIN7hXN2nb6XD/Wn+ljvNXPgcR8VfURvTzszDFPLz7RC3bH8llHXlF56fhV
JfcUDxy/c+JFNCb8Dc3sbvh2Y5Oo2rZVtn1IaSz4qJFOx+7rL/dMDaH0PTMh
eHsQHWqWU5TdD7bD94fO9hXmuRgt2rhRx66e4qLtbl+XGpi9MZ4hIMeBtlOU
XMqtGEwK2lVlRo+G7oPCeQYTWbCIfn/MmcFCSrr2mL4uxX+57IO5a9oxXW+F
+mKnOjiKb17z0WcA2mekfn/Lycbe40nmHJ0SVJ3tXZj6qB6q52VkdyRkQ2qX
4vUCyQ5Yb/qQqWBUibJ9wZ/FqJ5/8+L2a0dMGUiyW//z2p0ubIK172lWB7x5
EnWiFNeXvGfeXP+NiYV+Y6/T9tUj/9KSAyZ9vciQO7UwbqwHOcFz3NnvS9HR
uHyPxIU+bNU59HlxJPXvZWcGrBd+xC1DhUcJm6jrujICU616MTvO141n+hwa
q0wODvj3wuVl9UaZxz2Y+dIjfBmT6lG/Q/9cSOiBbeC8e+4WLETJ2iz/k8rB
2iEjE/tUFkyuf3eUncLEl7/3VFZs5eBFcdjzS3VMnN4WuZiV1w3HaXsO77vG
wYpTpxTSlZnw3frbcnM+HbfqrgTlanKR8+inpdMXBv7a3BX2UafjtL3mQOxR
HnLJdYHL/Otf1iYGhO/2w0pyn8fkPgZpeboswziEk+eeI89dGPlshfv2ZKwj
72lG3jN+SkCQPzcFu8l3OZDv+ihzK3K2dtK4c9DhPm08+PbluHMLcGy9o0rt
fTRyztvIOTsXPaYnBbGRReaSTOaSJ53RIWjYh9tkjjJkjvSUCXGLTJi4QOb+
nMx9L60s9cs9Gli6fJ1Eb+TrZIdF8Xk6xbV8C76uLM7wdfUr0WnTjylsWBId
CkjwdRj2OMSlS5gNd6LbPqLbDhv5g6Gnu0Bz5uv8Tghf56rddiXHftdCu5jv
i4PEFxKRW6/+E09FPPGRIfGR4MR38XbRAzAhvisgvtugs6rZcTUDycSnI8Sn
ISY750zpbIIF8bUB8XX+HYM3T87mQkKQnwNPSQ48eVXoUccegCfJjT8kNzRu
ySk8k++GaDc/Z86QnNGWv/Sgm1aLNpJLQSSXXDY+SfGKc4cbyTFdkmPRwpvk
D+5h4d7/ck+Zn3ui5gP/6EGNGCM56UJyUls0cRN9dSBk/0+unhd1dNhN9TRR
ksNhJIcFqpgCcmp143L770HpI8succblvFCOyIVTs2mwIVywIFzI8avntSx4
N44jQYectQ5W0TFKuCOQzOdOXszBgLU/kkAjnBKw53PKbKFtKVuBBov/w7VD
uQK3bkdQufWAz8EEwkHddtusSxMKYUK46Uu46WJ8YuoSk05sIpyVJ5yVE14U
LDTWD20bPpfDCJfvbWiXVgipQgLhuIgOn+Ma7K6wmwt7YPF/uB82Wexlem0Z
XEhP8CU9wSK580nfuR5Yk15hQ3pFwI+yI2eCqZ7H4veQFtJDmsZa7npf64NE
Kr+3DJPe4nvzfrFQeD2sSc9ZSXpOXeH+U336bCSQXhRMehEt+sqZXzfaEUB6
VA7pUQIrBDSKxO9BoPSGrv2SdGhr+jvvs09Fslb18C9j6rmBszm8TCZcipRO
7vsUgaqdn3sSpjZR+8KiR5dXV0M3awv39HWq98cseDv6gQOD76mfiyi+PXgy
5+6SSA4CDh73meDVAQuV1LKQmHbU+a5iRqYOQODIE42TLX5w6Q07FP0uAVwp
u+1B8X1YeGHFhP1VXdTerPHr+RFK5wdqN3x3pfJ8LOnyM+MUhPVx09M2VkNI
sLd2PdXbbn1WV1Sb1Y3BhWu33aAPQFpc8uf0N9T5d993Y7E/wNdqOkM7pAMn
LSL+FX/rxEgd3tZkdSHHRn3dq70ZiLaZfPGVSQs8pXaL65+mvqP+x74t0b3Q
/9P2YNFQHVyszPbPHm2C6hvr+qteNLD1tiu/vFGFglaVZd4x1NylPJyXCTci
TEl8//QFZdCmm387tiUVo3unzJYaqYZIXpd6wowSuCTqPvya04GctmuTecsq
4RvwsNk7uAi64daMNzc5oJXs2eJn9QVVwSIFGbmFsLzwtNBgdQu03YzSXnE+
I0zYVPf4WAHsL4w8VWZxULV6KJchWIwEl3RbAakCCGq1vM3e24mqUyviYo/n
w8agiF7HzoPFPc8t1ncTQRNcyjX+lIm0fyJK85xyoRbsY7ryNKV717FMXkYS
Bk+UlX4+k42mvDGjYhnqu1luLSJT/GERKaIqIfURvqtLX7JPcSEq0aoUkE71
6UuxNtY1KVD78cWzRKsb56XCJ9mfoXzacUjSeywevkcmL7wqV424A83Kv5P7
IPDrWP/u+6Eo+fPW6vI+LrVHrlq0wZeFAtbY69VNPGw6fbxL2IEBeeVHXvO4
dEwPD/rIdeBCM133hdGkBpRITTZcTe3FzVIBOrrVbARcCT3noxcPE+lGoT0D
tUiVvft+fmU/QiNnOKZJ92Fw40jIU04uZGc+Ucs3oTh2d0HXric0fGIXzeVs
4EJprslXQYojyoJrpX77VGGTp4X7/yPrzMNi7L8/Pqhkj0L2ECVbKgnxHoQQ
QiV7qGQPISEjIQkhWwshlYRSaVGa9r2mfdqnaZ+ZFglJ+N3PNef7x+95/nJd
t8t9fz7nvM/r/T7XQ898JWbPviH+eIXZf1je37fV7D0Ij9boI4Oyq2C++pZc
5hk+3O99/mQgaELHpM+G20elYkOgmuwSi2TY+rz8YL+3Ghvzw7/eqxZBL9na
65yNCC66pmLurCTkF213Ly6pge8kHVaHcSEM2muzu0/XoWqjx7wWjTJws19N
8R0eioEzDx2uLK+FwZjZhUMmChGUwa76/KgCqor8P3IiCV5kq8/q698IV3uV
ugsd9QgIXTffRPafn+8wf6Dr2hZMPnU5ycO+GZdzxrzIL+Qj+ZWv118nEXyc
zu1+18P4f/z3cyrHciBapeTh9lGEpsGOY9/oidA5J7r2WWsMvikO94/SZvYG
+53Co54tqCjtcyB/pxP2bHi+9tSiFqT/uv9Ty7IZXYbOe/uObMMFNcUFAX2b
sS1goiU/txFv/RsKZLgSTLA/9Xl0fCNOC3W3lSs2wOT61k5bQwke7uDvsPVp
wOpdG/fVnqjDjWd+C/SZ/bjwm+Hd9cn1+HIx/s6XGULM0N20RiaoFf3zF+6f
aszkyVcrTL/Z1aIxKznBTqP9P8+brlinHQj1RDG9p4veU7LhTEQVs7f60Hc3
03ff96tYPvF7DKbQOS/QObemrDhqz8wJh+61m+5ln1lgU/Y3GnupDllUBx3b
T01l79/hB9XNmur2oHt/jNtHJt9TncVU57Hl6f2MXkjApb48pr6oGDvVnmXy
Vy31UZH6OGvM1zmjH9bBg/p+hvrO8bZ5sPVbGeaSTixIJxZWGJD+PgrxpCv3
LKmuNF/PnLF6bSX6Fkt1KKMi1SFfRd96qFIZTpNurUi3Sz9opOTpiBGoItW5
mHQ+Zd47/xV5AhTRXDjTXHDKnVzOgAdzmqNemqPVA/xVnja2g09zp0Nzl37R
+kzfmCYY60vnVI3m1LiDu+RIGbNX0Fx/orlWOho9eqJPFgTEgWrigJ/8qjWr
xzF7PnFjOHFj1cuKENM/9dAhzmQSZ9ZqL1nSMawUscQlLnGJM1nlaov8G9gR
x9jEMaXNv4+l2DZBmbhnQdzTG27ddsypDJwmKSdtiZOs+p1185+/hgVxlWUp
5arN08um3p0Mb2WlHE4nDqunNRu91C6GgLhtT9ze0Jb7Y0ReK1haUs5zifO5
ghEzfM0FMCZfCCRfCLlnZPwpNwQd5CN88pEJa+b7bG0Swp18h0O+c1scU+k7
Ivw/PvXVf0fUtWdMfiNfCyRfc53q0adodCv0yQe7yAdVDf3fiIN50CHfNCTf
VMreVNp1pQ7sk1KfjSWf5VyNG+Woex8KNlJf5pAv25hyXm13qoTKVKmPTyEf
11txbO7+KYzfke8PJN9/deRe5KPiMqbe0pxgRDlhi2/G8CGmreCIpLmih3KF
7rRXv3ftqUQI5ZByyiFCy8XVs7a1w51yy0PKLb0mGxdd+iSAgHLOWso5ChF9
Hy0yZPYsykVcykWsnePcAnrqwMmS5igO5SgFmU+zlHwzYXFW7kaqSTmzb525
fewhFyrr7u8fOasOXKUPm4dXt4Mjs6zHTCccKmGjXzU1p0FZz1N2TaEQrG1N
Gp+WekHmXe/lU6lNYJ+4viLzexqG2kc0T5jQjuwN4v1ZeQUIFFf5FBm3wF5L
5XOUEaOb1fLzT5WK4P69aemwK2VYDte58lZiPPb/s2GXAh8s3QdmDYOdEKh9
Z+3EfS3I1JV539bA+Oqkovji+jpU9vfVdGc3ocdc+K7SvxwW6u0bOj1zEdjR
Nn3u33rovenScFNvRfemoelpMWJkps683/hCCPMhNUvyrPlwcbXP9uYIINd/
cl7bEgG4ccu+7TFgcvWE5iG392RCvr/11p/tFVB+tNxqhDkPvQnp87ZGtkHe
7JhmnSKTtyRj56+4XY/eojp9o/ktsP01UDl9cTFYv8yGOK31ZXLP6qUzoxrg
t3mQzqpdheg+C5lRgZUYb3C9L5zrYLhSMTxdrQCvXgqPR1dK4GYXqHT1ay12
rlsxgW2YD2WNmYuP6JchpPrwvgtPBXB/UTjuwXUeBjYfP2DP7DUqn8KuNHlV
Q/6Q4dbDt/Ng/OrrLWfDanRPmuGxKbMCeoqnBBWSHLD2bhBc9HgN+S1V20T+
DAfnNCh7jcyGR2XSgKLweshvStsoGVMM9ehnzhO6M8BPXtcVfr4AHWvWVx8e
mg+9xIXv1Z3T8G5VltD9VCs6Zqw8rlaTDn69nn5eQDI8f7+a56jK5NlbXXv2
P42FfOSZL5obEhBVHzxKfwLzfc/coarf7kJzxdxzL2Jj8CPkXCGnUAJD/ryP
3RNbYRGq3jvzVCgca/aH53KFkAk607xBh8ljD9cc2J39BCsz0vrtH1IAF+2P
w8Xcejgt9LozbmY7dj409UrzagfvwMZpN21qYDro8N8m7VYs+/bGqdmuEZz3
8k693wux9E7fJL98ET4/lVvcKcf4T26fW6flYuGqa1tQIGzCEev7KgfTE1H5
6JH/3ZkSjB3zdkyyTj1uV1cM4lSJ0dPZNH90XR1M9BVzO9qqIUzJ2n46SQid
rTm/3y0qxrEk8dx9pgXIrhzrVvgmHxzFAI/oBR5QXjdyzDpZD5i3+jytjZbA
MuITL+RFExZMVxv2hdmHA803mx1W5+NV2sr6sM31sLsyxOivfS3Yj7b9ltOt
R/XJdm/rskZYTzSJSNNrACs05Jtxkx/cSoeubpW04GIA+47Tihas+KO4uiKo
ASNHu+WyC5j54V9d/vqqGJFTRhmusqpA5/Og0vSrTC7T0Jz4ZJMExe47ivbm
5iL8zc0cj9USPL/hoHJlkASl/XN85HdHIcgoYNDpbWKIzh+Pvf5ThGCnxkvG
l9qRa79Of2F3C1Yp6o06sakFjvP3xhf8kmDu1nh/7V3NuHF4aZffKoZvY850
b7skxsT5XhlsJrc4VHiNUlJqwLYVEoOvA5jzbaqclLipAZqhZqkKfeowRGHo
vQx3MfC3PcdpWT1s9QUywl21aBsl9/jAFwlyVeC2LKQOFyNkM6ecFiC0+M2u
P83MHknPr9Hzra+WTxtr+xzL6T129B6uYR6S7KLgTd/Vpe/WxKdFlsUmQJXO
6UTn9OrcINHiJzH9ld7Lne71xlNt4uO+iSiiOqynOrBMvsscmhyFN1Q3MdVt
4uu5/TpGuiOS6uxHdf65e5VkaZcEHdSXZ9SXiHEtrgeMmqFMffSmPj5s66rY
uV4IR+r7Eer7lH78E2uCipBLOjEmnWR2+XW4hrYhkHRlT7ri84xkO++X4jrp
0Jp0eKSwxCRTzPigslS33I1S3bI8FAZ91YwBh3QeQDq3vNu5cJQGs+9+lc6F
Hc2F5JTN2pHnytFNczSD5sjiqjvv7w0u2HnSuXtEc8eeXhHZsF6MEJpTQ5rT
iyEXhNcuCNFNc72L5lp7ZGDPnFGFeEwcuEEcqLLw1Bgz9MZ/uLFo3HjHDT8b
YU6ccSfOpLP1Hu1zLAfLS8olNnGJfVbL8uf+aAiIYwrEscBUk56QqGawNKTc
ExD37GXCvftFVkB+rZSThsRJhYxXLfb7w6FMXNUkrh6ZUH51WmIjFIjDLsRh
zip7ydaHJZBXkXLbkLjNHfNrcpR1GzSJ8wrE+emz5aLjKgQI/pcvqHzmJOs7
h//HR7iHDgkz9tZBmXzHgHzHXSDi6IyNZnKb1KeCyadCVgh+d72p/Y+vjXfX
qNn6i/G/RKkPKpAPGqtlPjhXV/If3zQM353zvbAJXPLZEeSzHf1/anDaUqG3
WerLfPJl1jmFvY+zGR3OkPr4B/Jx3svtgU8/fQSXfL+efH9r4e3Au8HMn2NL
c8I7ygmuhtXXXXlREFCuMKRccSiYY57/ow48yiGOlEO6hdUztV8kg3VSmluU
KLeYmw/cMPZxI/Qo53Ao50S9lOtVPsUD/1+5SEkxZlSqrAgCylEsvjRHsQ+W
6RUHlEHzaP5UmdFN8JVTGlKjlwLbBIULbrkV4EAyYVdBIdiyz3o9hn2Gjuty
7Zm97dB7N3btxN8SCJodtxVtCIWgzvNcTF0tONG9WmZrBODMclhud94dnP68
CFZFLNiLz2gtmf4Z3cpYcD+H2S/C9Y6ZjG+AbX3iQUubJtzjqU06L9MKa96V
QQaHM+Brortfbn8J7s19ErxquhhaOjtDNVY2wX6FKLVwWjv+KD6/7j+vBTZb
TVyHDc6D4KZ4WZ/tdfCtrvEsUmuCRZXVqXMRzH2GmO5c354HrWV7kmyGNcDl
ZbiyrjgLyr+XJmzc1YrU06l6dt1CNN8bcd7boQF6Jwb0se2qQ4Gbzsvtfxn9
Orgdm7X/n7zZd6rGCT6UfdNUx9ypxuOqb41eR2rBejqyO+9sBtgXfcTWRRXQ
H5+yr6S4DZzPpnqRB0MRuHNJzsacMoyOCyyZ8akMOoo/lw1i9u/e9Dm8yPV8
uHtUepgXiaFQ+Tr2QqkEBkUd86MmlcLph/ar0Q9Lkaw9de/bchGMw36/SVMu
QXp5mGflOwkC571eUNHTDN7c893144vhbaA0+UxKJbo8P+1rHtkE58kv+20e
WwSWy3WZO7ZP4MK1FmiMaoBbSe/q7zKF6LpXNG1TYR0U9i/4PlTyz89hdchH
YT5453oydK8y+lnScCPQVYDkmqL7Gy/wmD0qXNJ/mwQWA95werIq4Pt7j1lh
aw5sy/c0D2xg8mm7tbJlfTFstI8Vck5kQrM2d9bXU2mwfdxYdtUtF8rjpnkH
HU7F9siZVvwFYnAb303KsoiH/ZhlzZfiE3DnUr3pc4kA3udDe57Ft8N2vd2a
pHtRaF/aFq+VlgWX4Be/tIaJwbKLFbQeD8DYc2GFHP1WXGhQ95PNasCg8P4F
P263Q37WsxeH4+sQop5yJDqpBrVWt29prmpFoFr0B7v8InAKFiVXOBVAb/zl
+ckcJk/3bCzdVf8cnMN/7ulcf4uU9denTGPmYFVvrZKI0Y/o3k29iU9b8KCl
9GRgvhBKp1xuJDIctAw9YXA1SIBHSukqHUXMPC03sg//HQX7bvYP7oBcnBld
KH7/KxO9E/Rmz9/aiJh2nr9DejuM3rgarGV+dT+mz9UNyILMrCHRqdMlkGg/
f7ahUQw7wx8P9Jvr0O/1rQu2rhIcfLbuab4Zs0+3VKwWGMeANXOkv1CGmZ/d
uleculpxPG2u9madJshmW3pfaW+DQdiV0bWiNuTYGHnWxlVi0xIFnXudbai9
YW03sK0NVfvXB805n4v4tIqqflPbYGJz+qW7UhtWnGiuqer6AMFy0csHIRJs
fxJXfu6yBOv5W0bp7WnDm4xJ/VIY/2vt1zxJd5sIJsv7zC5wEyM1YLRKcXwz
+iQ/tJiV1gSDjycUrJpbsHzg1183IxqRHufhc1CuAV3L+ooW2rdAM9APaZJ6
zFDf6KNoI8SY718u/10ggtLbs8uqn9TBYpNbXZCxAJVNj3RN5SRQPGDEXdMg
hChT6xWf2R/0Q89vX/mgDUr0XELP+RID86A3/hhD7zlI7zFJYK8zy4nHfPqu
Jn03Qk2vu49KGtbSOQvpnJrd37cqT8pEDt1rIN0rc/rCz4d3pyOE6vCV6nDp
e8+6yOwEiKhuVlS3FIdET8fBQUigOptRnVO+9DWwfN6KrdSXJupLj/8RU529
jI9TH0F9dMwq0xr2vBY/NaR9X0p959v6PtnxKB8s0ok16cTfSeZHXYQE/UlX
7aQrv2PDi67N5SOJdGhKOuyu69ioNaoFzaTbfNJtVNKdVw82FcKZdJ5AOvc9
+912jUAChfvSuQiiudiduvjT07EMR45K56iA5kinv37JTA8eomjuDGjudFg3
p8TUtoFPc9pCczrkyEv/lY8a4E5zrUhz7RuQEKN+mQ8P4gCbONBH13Lfz3Ef
EUzc8CVuTLGVMdYe2gKVJilnXIgzT1kLTOKmV8OXuKROXFKYsunwU5Uk8Ilj
9sQx99XZOv2mimBP3Ash7lWuZc8Vza+GgDiZTZxUOL7bk7sgDuOJqwbEVUli
Xy/LuKb/cPhxmVn4rm38/3DbM6ni+nmddgT/i/MuVafbDXxq/+MLxup7Ncbd
iPmPj8jLWM/uYnKuHvkOK0PqO4HBt4J93jH+Hy/1qXDyqenP3ygdnlIHgY/U
1/TJ1ywWvit88uIFjMkHVcgHLxgoaaVPrwL7X76pV7uzXqdJDEPyWR75rH1Z
+YC0gcVQGSr1ZT3y5fSSgnNasi1QcJP6eCD5eLKBzpghD3h4TL4voyT1/QGi
QxN69ZvBpZzgQTlhz6PVPkGn8xFCucKTcsXrQZN+z3nVAvYSaQ7pO0aaQ1iP
bk/dmlaMqP/lltnS3FLyZ2XLM0sJzCnnsFqkOYellzn4pl4lfP+Vi1ghb8VW
/kthQTkqhHKUwrMfCdHKTA7sUq3VPeYD1oNL6ycHZEDBiR0p2ZMHzqHgjjE1
TG6REX/Njk3CwOjC+ozBIiZPNk3UNsmGICH5XPjmT9AcU/72rwofUR+WGG26
JgKn0/JauudbXE3V2uoiaIXAxmiy17ZKZg8wKPmVehFbb0//bN1aDW7iwX0l
00NQcOxitvXtNtw5cnhK+MgnYFW3pjwc1ogRv57JjMmXYFWxjbFHWi1sN/5d
nx9chCk6Rvc/9BODM2To0vi/H5D+Ja9rbkcbTl70ORqkzvC98/j8eeeF4EZW
f87uXw9LucG8NGYPDbE+Vz5/Ugg0J0QWxiYUYt9LlbVrDzeAEznc5nCHAKyI
wadDVntC7ppZhF1QHVjJbE8bhzPQbM7a0zmzBUs/y71ZOE4II/+0izNcKhGi
NO3xh6F1sPya6HUlR4ADPcdOlNpKoDJ1ePOU4EpUboybZO1cg/CGmQZzZjM5
3++7+u1txeh1qOTbllWhT9a4hWpnRWCNnfC2poDJIa6rBlWcqYRomL2ZqUkx
fN+tdzdyTgZ3ecq+FP0KTC/ROncsiOFL2pci3+oIdJu1C1YplqPDddqioGzG
5x07b+YvdIH70bGbjrbw8fjz+5H9Mtuh/kJvvd+PVqgc8/JJ+1AK5Vu8Yvck
IfjFcpalR8XoyOQ3rLBi5tR0/eWM09nQOSBu8w9vRjJ36pOPP5g92U7QNU1Z
DFW3F3vyPjdglsaBA0KLQviOC2yOk6mBsu3AwcGXhPDzOD407SGzz0Xf03ZY
k4CO3ZXsncVVUJ5l8Hbo+DxMcgstzXFsQWDxWmXFxmKE2P3+7HQyE5Ydfws3
9VYhpPz25LbZmeAXjNM7qZCCDaOHHTijnATuy46Ri/YFgXNJ+DPbMw52zd+e
RWaKUP83IfWSDpPzyx3ep5W8xc5ZD1dO+C2A5PSfphS1RnjpzrhzMbQdJsHn
BmdPy0GI6GvE0fpqTBO4PNo6phVaai79pt5n9Flb7tozkoeHEeMbZLtb0Nk3
NHTCuzqYKRofDuhuR+e2AMeHGo3QHPlr9dlXBdgJ/vmJR5pw3Wq/2rH7tai/
tvrS+4FtUBlmGSH7qwp99RyL8x7yYY+/n19Nq4XFMPn6S+I8xL5MXuJ+KRsq
8UpfBx+ORMe9mN3r7OJxne94MSYoDuaavn77Hwuh+UXrHfp/xF7nmN6wyRHQ
7F1c7XwgDpyT3OBfkgistEh7cGVABDrC9n5VCG+CYMLN4gbPcMwr3/phR34Y
et8lsB4weds3UXB015FgZHzdO+ne6zcIP5e0RG0pszfvsO1l+TlB51G66NBt
5tdzX9t4L/ygLXT+W7y4DQfVvDMEc9qQsIq77E6VBFo/oyPmZYgxZ5GRqv55
MZaaf+Id/NQCU12rnG6NFgjWVVW4GjdjYdLc04clTXhVaFwh69uItPJviXLN
DN/3ttppOzUhcELvraaD9agOLGw4yfjEuWDzeo2rzWiMs+84ncrMVVr77jGC
KvwO48KX8Q/L2kexXxtrsf2vnnNbYTn2P3zYxJ3RBmt6vo+ee3g+vPZl/1u0
0HvM6D16mr9Zlx4k4y19t5G+q7/B55iJOAdv6ZwFdM57S6aNevKLh910Lwnd
69o535Ez1HhYQHXQoTqUPh0RZz4iA0uobqeobvkZv79cVYwGa5e0ziGPpXU2
T1f4UhDRBgH1Ra5L2heHyXc0z3QwPkp9nEJ9PKD+7NJ85rz/7nv1I/0x1Y94
sCWdeJFOjDh6lsO2SKB5X6orLumq6Gn8R728coSQDntJhx0HfPY+HdEKc9Kt
OumWy+njFB5eBUfSuRfpnHPn+iB50/c4RnPx539zcbrvToO3jVAQSufoOc2R
4KePd/JLPrpp7rRp7hRSPtc1WYdh8BnpnAbQnPI8anvdPjajm+aaXSGd6xEB
hXYpsoxf+Ek5EEIcSN+lzplcnIwO4kYzcUNukb61cCGzxxFnuMSZd/1Zo0r7
1EJ5j5RL6sQlVt7C5+kymVAhjgUSx5w2jTv1/C3Tb+KeDnFvaOebQc8vCmBA
nMwmTrJnOc6yeZYEAXG1m7jqFhT87JawGbP+xWGLht0m1fb/5TbrsYr66T8O
4BDne4nzxotahUFPhBCQL6STL4T464xbZZYAFfKR8eQjZT+STIpNG2BLviN/
Xuo7XQXu4/ZPTQWbfKqefKpmsKjKpqUeUeRr1uRrITNd7AzOxUKPfJBNPhg1
a/wy8adasML/v28aOWbNGVTYDg75rM3/fNa9cu6MSdXoJl8+Rr4sKggxGNPb
CvlOqY87ko83j1V54DOA2efJ96eT72suXaos78vUkXKCMuWE5aZz9pTaVoKT
LM0V5ZQrjPwkuzcvaIfxQcohc6Q55HqUxNfohgAdlFtYX6W5xTzj4Kq4XyFg
q0hzDitRmnPYX0fM4LQzeYhyUQjlIva40CP87DSweqU5SoVyFLtmboGMejOT
L6Z2rjZmdFi68rBBQw5YMWetbri9RcfpOUUP4lvBtrBKPrMkHQq/bU6bzK4H
b3z1jJ9jaqEyKVZ+wslEsNe/DR2ql8HkmejgPpmJzPc/zxAy+/zFU683Htdr
hmahb8RNhqPcK1ULfBxfw634Q+LW3EKEJLpFGauWgcWdnrQ3nQ33jsS5lzaJ
wXq1caHqVS/cfpfnssayDbo2Y3ODSvjgFG7YpnC8AVq1azIVoiUw3pnxKXda
Kyz8Z8b7JxXBWqc+gSMrhjn/6okVwnJ0C8UIKm/H0HQ5y+qNLUg22Tpdblkr
NEf8dp+hwPSlWmST8KIJtkqhstcqy2HheP62wLkcGs4/z8UObsScjw21Vjsk
sBj7e7ft81QEXTjwc45PPWIbBjVPnFECzo1P06rN2mHRBtunu+vw4ff2j53C
Flhs1Tl4U0eERb365zdtZva79UWnGicXQDBu+4oSmUbYNNz9w7etxTHz12Kt
IqYucWtsZnOEcA9TXXvinQBtK+d+i7xZAF7hhOIzOdVwlkj63PxTA43J0QPe
2zB7i5n2K6ehZcgu/HTzxwfm909JuPbWfGjWPSj/db4QldvYSVWHquD8Ic7p
7SOG52lhgtikLCRXvK1aPrYSvUZbWYLrTP9Cf5cMmscFNylwk1pkOWxdgpMC
ljE6ublwZWT3M4REn449rleG3OwfM+s+tMAu22AMO64VLquc5UY9KAWv5sjY
773MfhM0u8lhgQhudUvyXZOK0RH52tnRKILZkyQJQXsboVfd9UAjuhDWA1be
CwxqgkxD603luUL4NhUYvD2Yj+bI8+bl7WVQXzE8uG5aBVz0+jz75pYDsZXM
xUU64QgcUeHuxMqHcuXBppfn01DU/W5A7ppmsGe77DLbFAnWts8BB09xYTH9
VLSiZSV4aq7fopIl6KhdE/DYIQSzYgXifkFcREkKzDrlGrHMufT7mlftUHg0
6rnRKBEUsq/rB+ypgnmDpcqbTAnGXyyf/vhhDYxVJV7XV2XhZ/n0lpeHW8Cq
9F//ZmYyFr2Miwh83YZlczqGr77cAPX2w7Ml51vwxzXqiWdaI1buy+1Xp10L
l7q4yZ6TK+DUkNGdNEWAExcuZR+dXA6TBSXKf061QzPihNaCjaX4ecW14ayg
ED3bPzxJ5DI5/0fInjMuBaibvbQwYhUP7p4Lvd/NjYJK88Zltz/noXnoS4V7
O7Lx61zGhOcmTWCNT1tgGZGJ2PAphV3L0qFk9pV7/2oFOCabTGf9TMJ9M2uz
w3cSkTSd1Tmsbzq4ci+H6f2OwJmemy/OKkcgxCFzl2tqO/hWXVk+f9pxqGpd
8j+/Hs3IHJ7XIYLqYbMhvFoJDi74cVQ7SoLPL844e51vwoTZI/ItfVvg+3DG
V8f6ZnzQF4S8Pt+A9vzqvWUXG6H60v65W2Q9uAPOrAlRbIDE2visekoduBrR
MQMeCRA8YdqB04w+RhapDTrIrcWqM42D3NeUo7Wn5mXo9ha4PU8L9g4WQHXI
rrSU1SXIXBbb9+LTVtyi5+r0XKdzgmj04DCMpfeY03uclTi2Cx9k4gd9t4C+
G5gcMW/kkkL8oHPOpXPelCgefcYvxgy61zu6l8c1PA2WL8ZsqsMpqsPBmBE/
0uXzUUt1O0V161deFJWknwzN/tI6V1GdDV4mjs2PuYDH1JdA6ovW90v9Lvxt
+U8fK02uTdJdIvxP37mDjibHmeWj+7tUJ1WkE6ds+1kDeyQwIF3JO0t15bfG
K9vBsRqPSYe3SYdchYUN+py7UL4p1a0J6VZ56OR2reMMt0jnJqRzznLjtRsV
MiGguRhYIZ0LnuHDWSO/icCmOdpLc2SYcE8SpiMAn+ZuPc2d8stX53P801BP
c6oplM5pHq+/7KVihv9zpHOtQnMddGTV1O85tUgnDmgSByaNNyhL4+fCkLjh
S9yYeGpPiP25VgwmzvgRZ+qnJhaOkGd8kbikT1wSSE5XvAviwYY45k4cK0k/
NNfWrBUc4p47ca/7Tkancx8hWG5SToYTJzuyVIxP6WSAQ1zNJq5yq5SGPZzL
1CddymEecdjXatNfdFSCTdwWELe5xapFalf9YUGc5xHnFVJvPEvKrvuPL7g7
jCtbsSsV6eQj98hHynbayKrGNKKDfOcI+Y6Cz/ZNOu+zYUs+tZR8Sr5sZOrg
9sb/+BpX3mP6fLt0CMgHQ8gH/wzuuWkrrkcI+aYW+SZnfUEHTzUOFuSzKuSz
F06r/Y69K4R6ndSXR5EvW2xdmZP7IwJc8vFj5OMuiktWV99mfJF8X4d8X2F8
xGTX/I9gvZHmBA/KCfFZnY+DPOrASqJcwZPmCm/9Lna/mETYUA5hOUtzSN8H
/hOLlBvBjZLmFgHlFpV52xOWT8yD4F85xzXYY0tvaAsUzkhzkTHlInfZOSO3
viqFgHKUgHLUeOfAgwvU2hBiP7Xl9Pp62NuYfDvC7G9rE1YcuSJidL5F5+iF
GblwLxnb786tbOY9/C+H4yphcbvzunCgGIL4PaahsmngPvrjY+t3H9xptTeH
PK5CiMl6ZdudCRDuOuMc/lIIztXC7rN7mXrsNzH3mBgJq22Hdh6NTICyz7Zt
ZwYz/sD+M+PEKX9Ejn524mJXA0IqYq59Vy3Fn9KY1Bu17ZDxDZ8SHcXkyI8B
58teuyJwzdKFE03akH6us36QQjN87arOfQ5qwAbeiDl9EyWQ7Jn1wW1SPmzP
jq3zairBoUlGTyM0xbAM9jx/KYCp7+6gm0dGvcdY7z0J9wNb0G3yturKoQI0
H3xmr7alBe4Wd7wW6DTDpO/cO1FezF5pcq3v7vha6AX7jtSvbgQrWXL2mX0e
NC/+SBp3vQTdc67lF71tgOq6yIzfOk3gBr/V6q+TBoH+0VPL/epxdN6+shqr
LHDCD7f5dL2A+Qi7sV9j6+AYbl159kcj3L9ZfZir3YqPsyaqCtuFKBripz3J
Jxe23R4anS9b0DknKmqZjhC2QXt8+jN7rq33jTvxpo0YyLniO8qpFhbeI8PT
vYqhEG7//fr0OuxTran4UCCAn0fy48QFDBcSMxZt6K7B7jVpg9xUBBB8Txbz
GM52aP9ePsy5DPZfB9l8ZOa1Y+/cpo4zCQhR/XZvU24BZNTP37FfXoWlY3k2
W2YyeWYVP7T6ZBqOhH9tj3lYAWX5DYbFH5ncO2SmxqzXITBc2X7tXAKja6+p
9l88vBFy18jlUb827DwxybcsiuHSeMForzimv43lyR5eLciW9dIPO1GMa+kF
1o+iimHetzW4gVcPlem75YbWFEB+mP3L1fkOkK8JOWA+twZRgS6PR9XlwTrq
saV/bAMEhb4B7zMKwcopWXt9dgb2bVfSSkgsActwoOsWhzgoR3d0O51IgOYo
i7lhX32RrsrV2Nv5z98vTuw92x0C33EeE58FNYKXG7iVHdaAJ+6DipWvtaP+
bdnM2+P4SFecMf7MqEoIKgsL7a5LwPki+zdyhy8zFzuejGtKh0d5/O9P01vg
Z9lj0s+kAW5el1OLo9vwqM/2aJcbDWB51rLSFxTCSX3NCNklzRDNdONqDBKi
e7/ppGG6EogueUYdMatDTvcBoRO7GirXPcfFTqmCzIE7d89+rUGRTsaRdFEZ
2EZysU1Pg6DzcEvQ/L/laCss5y28VgJ1P1mNt+xGKMwKczi/uwQWx+y0B88t
hOEcmVMWymUwHuwaf1UjHxW9br1qh3Lho+9sMSErAXz7WyNWDU9F3u2hSs07
khEmUy+XmMfsUSujE1HxHhrrA5IfN73DIfbR9uK7zZiUdei0RKUNPTd0hpiP
aMO1oUtyzibWY44w5cJkVRESVt+s23W9BcrnPQs8rwpR6jr0/kbjRkRtDLvM
OVOP6tSBpnVKQnj/XeSt1itETEqF4oSH1RAaDH3c9q4O32QmWsRfFsDEZ7vP
oRfF4E1Nyb58qAmT73c5jN1ZjWuru3dYreQhJqN2jmOTBNPpuTs9L/vwxWVo
chR+0XuO0HuuRJ227mXu/4q+y6fvbjlTlzflBR9COmcandP4pMh0YUkFltK9
iuhe4QOytmd2lWMG1UHWVVqHYuFVK4XzjN5WSetWQ3Wb+2ft8sCoHCifk9ZZ
7o60znLlNentl0NhSX3pob587qgO0Lslxnjq40HqI7f5YmreqTroU9+7qO+W
kV1vXjkWYjDppIx0ouAF5+FmbWgjXRWQrnRjtTqfH6uFJ+mwm3SoUGMie33j
ZwSRbv1Jt0Vad/7+83MjjEnnPqRzc91BZj+3FqOb5kJEcxHlO4xj0MPscTRH
z2iOBOqKIr3OOjTT3Cn4S+fOIr00MSGoAFyaUz2a00t/BuQkd7RBuUg61yo0
1+EF36yfPa2HHnGATxz4dTtXN+xaESyIG6rEjcRh0ba7MtsRRZzhEWfueT00
Cp/agGTikg1xKTUFvAznIrCGSjlm/D+O1S21Mx/H5Azini1xr+gXf8cDPyZ3
ECfliZPmD+N3fticBwUdKVc5xFULNZ3UFS/E8CUOWxCHecPPqO+OqfkPty22
VSwUL/sIDnF+MHHed963V+O0GmD/L1/wNb3sWots+JKPxJCPbOsdpR1l3gxO
hNR3tpPvOD/UWq0wvADst1KfaiSfMj8VMMCF6bMF+Vov+dqxg3ePyDzMB598
cBH5oCBXZgp/MqMn8k0P8k357yuSJtzNActC6rMq5LP6pjOGf85k8gz5si35
Mmf9g+F99+WARz5uRj7OzWvxX2jL5INYqe8Hk+/bKOse4cXnMTlVmhNYfGlO
qCvdHJmf3wxDyhVsyhUjeAnjZUf/8/dlpDnEgnLInoMNgar3xGBPl+YWHuWW
DysUfzyMKgeHck4H5ZzxgxTOV+5rB4dykS/lIqXfiv3XPq+FgHKUC+Wo9J8d
GzpNueB8UprxvqIVITu0o6+2FyDE6sSOlCFM7mxeFzUftWCbP4xbZM6Dam7w
lqkr8sHyyJvn75UE3hSPNWEHslB59nnvwT8i+F7UY21h7md7Zavt2H6pEDx1
q1uwkjnnicXexb1lULlh3XzlMhcNYxLvcAa2gzVw0ojbgwMgmHdrmnxqBEQb
HpxzCxGgO7tk2IS5TI45n7a2McAPFp7+oz4dDYH7pEtdyTdLsKHy43GVUmYP
Wlp2zGEsk1ebNaxyJz0B59xP1Tt7mbyYf2v176ufwZq9aOeFokbsC1sa4NAm
gVvIZqHi3HpoOl7KahhZAdfV1/VPOYuhyePPT9iWDE7H6xYJkjHqduvntTOZ
XL+qJWR/M5PPp8w2DXNsxbv1W1Mvi5j+ve9tOu+cCL2bLccSahphubDlwIy0
JiS6tW6ZfpTRaZRVZ5hqLXR222ZvTWzE8YMFtfMaE2B7+9Vs3lY+9M4+cJxc
1gC5ET6fL4jqYcF6lsZWz4Peid1PJbKML3c3BjhuSwMrT2G4mvdnJCYcGDRh
eT0ad40TX17F6M2SFbO4wRaxYWqbVl6vg2BvH6s7pwtg3mz5aeEDCdrGLlwX
lMfk2lkeukFvxbCwnhNoocTkyJHV8UajhHh8PH2MXi0z99ZbL9x3bYBG8MAJ
MWZMTkse8mPM0o/gX1rf4thYC60/vqNz3QQwnm0W732Fmacu/1nDtCph0L1u
YZtNDWQU32if12VyXc0xsGYUIdbHZFS/+1VYfsZJNky2Hb6brq/x1UxDyAwr
h7zbFYzuwfocXQfOiODS3Utfwqa1smPV5jLcU11k0LCdycMd9eIFbAkCnXeW
9uaXQMJhJUUVt8Lmyc7SA42NmCU71DGFVQTenNVmz6cxnPh2bqT6hFqm7y5/
bPJ4MD8pw/85Nv+ffW5DySfmXCr7oxduy4RqavPcZz9bYbtnZu2Cp1yky2im
3xuUiNtL+yjwjOtgY3hRa3M/5vdd+ihMUAuF+wy7iS4NPMgorz65/1YDQo16
jnaebUdybl1ef71WqDvEteX2qcQ9vzteelckMM7Z3rPnbC1Y77IK58pmQa2z
59AyyxZobnroeSCN6aNZftAI9SsYd+D3qgsmjfDUNXQ+NrwFSnkGraw+YiwR
Zcw8wuxd7qIXG3yX8zFl+A0Nt+JG6Dmsn9lfvxbJ149/1Mlqxz4R289mZx0u
puUt6F5djacjpk818qrHhmor3atDBXh4QNfwolw5iraxfY88LYbSz3F7to0o
g1wSd0i1WjESZzzewNsZxvA7Y0rRgnzslTeZO+VSLvKfJ9zaZSWG+oAnfj5/
uZhU9M21ThwPV0GIjyqTY7VV+AGvb7fj4iBPxZpb7Wjca1D/6Vot+lxoOe8b
J0LL3n0XVZeJcNLf5cWSr1UQx/eNXjy+EQ6xaW/OzaxHUKvVeIuiSojnj//x
epgQoj0OW4stK7Bu9aikrpUCKG7bENMyhMn3H2c9PYM8vLN5rX5KUI+m+UXd
TRnlSBZfPIlrXKQtPxwW9EUMCT3n0/PN3zZrmS7kQo3eYxApfU/2nLfvxqAE
P+m74y2k3+WcKDw3yq4GPXROdzqn9oCcL3kPajGC7vWb7hUT/GnOXrlarKQ6
3KE6qOmfXeDSWQlzqpsd1S156dPkj9eLcIHqfIvqvMn+Z/qNQ4nQp75Mpb6E
9cz6buXRiu3UR3/qY09BzfsMzQYcob7foL47lYdFH8gogRbpZDnphCWnn+5n
5QB10pUh6crF9R3/fTXDSXOpDieRDjtUa9pWxmRBk3Q7i3R74fD2MdOHimBM
OvcknXdbrh3EHl4FHZqLJJoLrkstJwEf4UxzxKU5UhGndCy+1QQVC+ncsWSl
c+fi9Ltmz+4ycGlONWlOQ3KiOXLaIfCmuXanuY5K/a145WQT7IkDOsSBaS9O
bIkLL8MF4kYwccPqx6+mhZtDwVGUcsaWOPO4Vbjr1mZm/yUuhROXVFI4f6/8
83PJiWNc4hhHO3TjszXPwPom5Z4Rcc/8jIZ1omUDOoiTusRJ1qstl1O7C2BD
XJ1FXGWvnpigz3DDnjjcSxzuTRv3df8dwX+4bbH+/bStaxLAOiLlPJc4X7/h
0nq9lEawyReSyRf4KzJm9J7MB5d8ZCn5CG+ZnJnvDBHcyXf0yXfOV6Q4yc1k
8g35lB751G6rm9dPXRP/x9fYjarmYdNKwSUfDCEfHFjEkqvdzeRG8k1l8k3N
U7Of6s1l3k8+e5t8NiTrxoc7Lczvky/bkC8bP92+mP+cyUmtUh+/Rj7uuHLo
VvZcCQTk+ybk+5wr66cMyir7Z6/+fznBY9ZMH8POViYnS3OFipY0V3zYWzy4
6Wk1BJRDNCmHKHyK7d/2/il4lFvcKbeYhSwcalklBJtyjoByziwVhyHmcslg
Uy4ypFzEfzzD7NjuJoRQjoqiHKWqM2HN6Xpm/1d9tG+uUQ7SdXj9zXhFuNcZ
Xt+1XAAVm8YZdVUiyCv55zwxKYCtqKv4+IdIcNO9br3eXoWQjc5Zh73ymHsu
8Zpw8p9/5/SV2yvP5MPfJ4puncnE8SFVtS5FuTAe8GXbibFNsMibufPEohQc
tp53+VFsC3wP3NDLPlcK9jgF3y28eCxiK7kmyf+TR0ffW/79ASxaFHPiZCMw
5dwXo1v9GV8bNjy09A/zHZPtBvYr/MCb5DPK959/1xminTHHvhTK41KWaTa1
w2+OYkDiHKYP32ouz2t/j9vKexLdb7VBflzI2sIXNWiue1CfL2RydoWL32V9
ph9lk7UiPvmCE1nb6+AkgCT9gHH5TzE8Y+SWvF3C+Hdxt+szYT6c66dmyGWJ
IAgVuTZVvgIn/lOl3atQnByeFPEmrAVfP1+60HGnFo9/vmP9YObdfFdSQHB4
M27pyq0zdX4PPZdk72+/GrF0YoCtcnYTXO4ulq8JZ/qYnDdhwgohVPVXLhR/
a4Rtd8TO23mfYTvtT/qi+gr0tMqYTZzN6EhVrOZ/grn/nmvNn3MLsXZiW/OV
Iw1IV5p++kd3DlSqrjVGjk5HUMX4AUve1yNZi6tR2NsCtsGU+SeCQ2HrlLW6
vKMOes2zgrSdKtCtNWLRb0bH7PxhMWtnM7mnynzQs6HPodyw+/7PcyKsHegd
+NFCiCAHs0FtcfWwmane3+YYw/OIKRWm12sR3tXZR4aTj/S/ER/fzxPCpXrs
q9U+AkyKmLI7dGgrBA/zxd7HKuA8t9yxQr8GV16c9hrM5HzbnuOzVzYxf654
Z/S9wVWY9XvVcYeIDHBEocL9NZ8R3Gfp6Yhn5dBq3FSRbCNGr+vPPY+smdyc
6XBziLgUF3Qnfnt1TYCoku2/Wm82I/tq5vlxwiIkbxZV9RWkYufoMQc5q4XI
9nwXs+1sPvZ96B76SMTMtUzr8u3zS2E+alyky6IslIfubhwrLwBn+RAbjxWJ
6LA9US65mwjNw1dX3Cpl5iponMygVa3g/P3W6lgdCovpvzTcHv7z72rMZn5s
bUDQyIt3GrzaYXO0pTJgSxW47ZEShTdVqFuiIf+7jOECu33c1zdhUHj8MKc2
mIchwa4s+zEidJ3JvbG/qwHsfdEvBx2JQE1QFHeAYxN8K3wfL/5VAPsTvzP1
CltxfY/5OnnFBkSGvXs+/kYrPq6pebjgQgs4/Iui3M9CPDfpfupzSYjutbZJ
Rw0a4KvdWuiyQQD1wbbVqnpMfavlgqzLBMgs+rbt3IgKZk9IP5rb2Q575Q9n
h4xmclz+Gtf1J4ugvJyfWn2jGeoX1vzRYeoSp8iKDHyfjpPG1uxJxxh99lf5
YnL0Jdqc3OKMhr/E0hsVu/ZcqYDZx+dxo8+JMf/toPlnJoqhf5Hj5epejKTY
/F0icQNqQ7NSlzbUISS2fsL7lkLIbLzj24fJKWPL5xaae5XCRS3e9kJ9KcLw
M3jRy3J0a5j6lD6Jx4i2xOQdF2qRYGFRrfWsBAOvV5g8iWqDgYLfTgtvEdLo
uRI958Qfv+V4OwVJ9B73mdL3jFR1OGPYvwoK9N1V9N2NR3zTEobUo5jO2fu/
cz4fckJ9UCMO0b2M6F7GzEobs4Cpr7y0DsucpXXYs3jEJuOlQthT3f5S3WT3
aF9KmFoBD6rzAqqz3o+n9c78HJRQXyqpL+67Y46r7t+OvuukffSnPub9THNv
3d+EXOr7beq7h9mwxGRWBRxJJ66kE829rp1jI8PAJV01kq4U7r3X3ufE+DLp
cDTpMHn4foneP3t9h1S3HaRb7mgn78uurXAjnYeTztXX3Bw+i8kLMm+kcyGg
uWBZK5rkvcqAgOZI/oR0jpRfTXw/ntn/bWjuODR3vV7LtTPba2BLc8qnOZXX
nZD80jQZXJprHs212uGrhn9XizD4ppQDUcSB+HtJI8M7q8ESS7kRQtwIto66
J9mZCA5xJps4c5s9x/L5vRZ0EJdciEsvYpQyGuMqwSeOuRLH+KaioaddI2FP
3FtF3ONclfv7G0xu/BcnLQduaZnqXAKW9v/nau+JtcNGtbeBSxy2Iw4H32zR
aGT0Y0Hcfkfc1tT5axI2OB0qxHkj4ryvw8cpiZeZHEC+0Eu+4BE5IyRsfDHM
yUfUyUdi2Ad1hk6XgE2+wybfsat0aV/2sBxc8qnd5FPG3hvfxPRnzvkvXyu/
zOo/SVAJ93/5YIzJpuvzXrTBl3yzg3zT2bXz6ivnKsjXS312MPmsurAq2KCr
DaxuqS97kC+zOwO2P5wjgAX5+HjycY6q54Q3ap5gk++zyfd7j4d2vf/M7BOV
0pzAoZygqWv3+EFSJJNbpLmCQ7nCr+bboqfn62FLOcSWcshGq0W/ng9hfOeD
NLfYUm45EuuSVvXPz0HPkOYcLuWcKCvnjORxxTCmXKRMuchj3zLZDwOYvYJy
FI9ylP3Yh7qjHGogOJ/HTQwTQDlVqWcKwy0XA/4IT6VicO7N7/71OAEuSosF
BkuKsLEgQjJxVyvcqx6dureLySdy9VPb8vNh6p5Z491RA5UNq8pqdMpgvDJg
zNSYXDQG18w7xf8A1gKHOMe25xDI7y0Rx2Vgbe/s8bZXmPt2eWBRO1M3tdad
bSHJMLhrGuoYw9y3i33+1LQS+A6cvOvg7HgIro6rPzyGua9LwonC+/fAPZd1
/JMoHPZqut4n+hbCZXMF94Izo5/5qVmm217BfeHtQ2dfieD75GV6opjJud3f
FM8G74XLyt3v5qbywXoxeiKrKx6etpkz3xW3QSHEZ1q0QiuM14b+evSB8XPJ
D46sfytUOQNCRmpVgFclvqWkUo+oA+ZN769JsNIkJ95XiflzPYvDVDzK4XtZ
9dlWBzGOlM48/nAZ04++tdn7vbJxkvfNKuQy04dm2VtjdjB6uf0xIXR0EIzm
72fv9WrBh/izS5N3CmCovLpH7COBbdHQstPJzWAbGsomXHgD9cUbk2N/N2F5
X339IT+bkCjTEWp7pg4qcl5dgS/rMFBBQe27ThM0OS8OXD+RCtuYWqVGUTWE
F33kh9s1wuDDFG9P9WbYWkZ8TowqgVG79q+zHxoQIpf6Y2ZtKQTV++akK+Zg
87sio0sSJj9FWfx5urgdbMvssBGdUeAof/R7PrGe0f2pOWcqhVDuedrvVFkb
ejYMtrNcWYf0pGUPfs7JQtR640yNgyI4aEN18S4hHq8Wfb3D7M++k//4Lk9o
wM7stKeH99WiZvUEq1TmfIL5gVa+JQKkXzq895uxAIfy3cYeO8uFxbgphbeW
MvPVHfX9p2E1wh8NCLPvaIZvlee092vTcUGYvetySAWSr/hz9iZWgBWyt+fG
wtMwUHvfZ3kuH75TWlU/DYuETMDArpS6FnhY9TXyf8RwR0mta6WwCcmxozbf
XlYHoyojzay+BczeXjA62qcMFpdXmEY+4yNwwQO17cOzwXlwMdZO9TXYdoqD
pj5n9hWOxfHBm5PAPqE20GB0I8IfLDma/Y3J5TdUtMbHh8FWe3a0k3EJ9FQK
qi9/bcT1nN1OvN52tCU43t2u1A676OmLQvcI8Ha4X98T9q3ou+WX2vZiIQxf
BKZcTyrCc/X81MTfItiXDzQvjskEq/lK6Yb0JBwwaVgj787owUgmd2+3CEq9
yYWs8naUTq9JXDKuEW/6r0s815fJmSP9rmzPEaE+7nHtTnsmJ/ut/bm5P5Oz
F3o8V7vegGejen2vnmH8QvBcPmx4K8w97u8eHF6Dld/5x45yy6Bu8XrZjuUN
UD/ZsO3A4UIkzZ0eyf3Gw74JO5SjNzP5ZHxqwFajaOz62l77c1IULG5d+PhY
UICYt3uPimwkOLBvD6bNk0Czj0OI0sM0HNBRVL6b34AfgSdmhn6uQ3fL1lB1
s0R8vfmlYmxVDZQS+e43dhWCG6R4QzcpA0op854cLStB4LNUx9sD29Edk/It
4H0ZFg7nfFoszodtidzx7Sua0Pqzdsf7oGaAnl+g56PH2LsvN82BOr0nmd4T
MGWC7Cz1Oii4Sb+7gb57YOSnXVp3WuBI5xzzWnpOZ9mgwtOHxCigeznRvV5c
qi10GCsCj+owuEtah9XRCX5hdxphQXVro7qtPlFw6iWTQzhUZwuq83G8vbDm
fDE+U1/eUF+UuztiDpyMRDL1UUR9jDd7u2zmnxZoUd8bqe9OHQfPD/5VA2PS
iS3pxD63Lc1+YjJsSVdBpCu9Nb9fr93UAnfSYRTpMHjbgJB1puUwId3eJd1y
4tR4b+18UEQ6tyCdq1sffPm5tB48mgtbmgu9CaHis/mFeExzlE5z5DpdLzzq
aBsENHfmNHcxgaPsps5jOEZz6kdzqloaOtpuMQ+sWOlcG9Nc39mxz0t+IpPn
iAPOxIHOuGDkTRTCl7gxi7jh7lumPWFpNrqJM7nEmUtZO8+pCsUIJC7tJi41
7jiR4+0qAJc4doE4JlNrpsn9mQQV4l4vca/59nvPZTot4BInnYmTHU+VXyqu
Lge7RspVM+IqV9KsreL2GL7EYWPi8JHe8TKHhtWD8y9uG+uevvNgVx70iPND
ifOZe0ZPf7qf2RPJFwzIF5S+T/KeqMhwjHzEjnxk5fk37dqGTJ5xl/qOMfnO
aI7ClbOrasDuJ/Upe/Ip567le3K9PaBJvvaCfE1N6VC6zp7a//ggJ1qjX1fE
O5iTb/4h38zFL7Vv1sxeRj7rTT5rcXFEoa5DJLj/8uXeSp56szez35CPc8jH
BcppplMOpfzH9zUk9fPSHBvBpZzApZyg3Hz3uLFnHnwpVxhTrjjpFbTF4m4L
WIukOaSDcsgXOb+0Sf/8/3Ypt9hSbtEYmPzki18rQijnKFDO+Rg51j20P6MT
ykXulIt8j2zylB8dAZUL0hylQjnKqer+9w+nG8DzOf5tADPH6kn6kSN28dHs
f/TF3TuMrhzuqO9ndGzzxbz0gmIJlO1T7YeVNIGVPWPmlVPvwRuaXV+XUIjf
sWeXJU8uhmCtaYPrTqY+kX22OdjkIzdgl2wfpVao9Lr9CR1ZAoXykD03h+Wi
PGrrDU4ss3dwFcLet62FgsveiFfLMhCSueWvS7U/OiyFbru3MvvYeScb8aFk
7Eu7r7KEmU+VEN78wUOZet4xej9vSjzkBy7JNN2VBNbfwkHbtF+Aq9mzhW8f
gSP8TZ1b5jXC/err6eNim8C58FGuqzoAgQ13K+O382DbNH0WL6UKrPEBw2T+
OsPGe/rV2VNawL4xsbHJKxNz7j9KfqHTDsGqUwLHL4Ww94/cv3FEO7NXDnxZ
PaUNWxPHnR6UKoJhYcKHw9OZPav8uvNrlVZoyhf4FRmUwvjSubW/ypl83Xki
/NYcCVxeldtt95fA9+b1+HdL+JizRkfl7joxdu8LDtiXWwGVu93eZw9nofkV
z8/+NMPXKitloR3DuYnWr81M38NCY0jpwtctqN+7fYzptlpwPSXPP5q0wmm1
VsXBOsZ/b+7On5Eei2Yjl8E7a5th0r/s5E2VZkSNLby+WKcRUSn3eLOv1eN2
2Xxf3V1NME75buPXrwjse14pLzQF8HNYc3TKw0a8k3spWrOWyc/H9q6p9i2F
7tqoN+PTmXzkM+TczloBfAdcYdvvzEF4UoDSz3ZmzsbNeRDWh8kdw7U+xcdF
IHfZ18cXB9ajOf1ZrJqgGZw/wWevNbZCwbZn5EalOozfZW7z27YcnB898n/5
zfgod/OuYIAQeTHbtM7KB4MrO3RG9dU6OJjNe3GgSQDL+4n2K5m9kZ0xzVg2
qwJuOzuzIjbVgOU1clf2kRJmH8++5JqZB51r4RH9PSrBerynyfb7VnAqddY3
RwZB9Wz43oO3ytCh5XR+g6AedmMjvaMVmdwXfifeXrcE/E25LmNWFcKuYcLI
Wt16FM1SOm3nWgBbZQYSK9pgf82RfcSsHOpGhlm3GV66bVIV1oYKETIj5WO7
UTo4ZoZZLIdksNYF8OL65EHZ69vB72PbYRE4Y51c2UeUv5s05W0is7/zdk4/
cJTpk+foJ0ql7nD7bT/DcncNugtYleOnCmFupyy7SKMNtvbiwKUlceDOWJjs
Z83H8o4+qS+TGL692aEbrsbsrZmXDvbmZsDvRY/1uGWML50d4epsw9Qzh6N6
+K8HSm8MnVd4uREhLZ/mhJyIgpnVly/jHoiw+GL5jQ/b6lCs3+lyqaAFFi3y
2+MYrltb2L8ST6uB1gNFTb2HtTAK7rqY278cx4xGB64tLcbtj5PmtTcUQdPP
9ox7awqSiu9MPL4kGSqyPtarNyfibfL3DRlMqZzeGw98cIfJm61ur6z+OqHc
2XTYz5YG9Dj+DbcR1uGz5sfijVptaDue5Bj3z8+lDN7/7KN6LhyvcVMKTrbD
duLA+ZcaC+BuPthkfjdzz31Lfn3XZPYs7YW5KvEZ4BovfXF1Zxlis4Zydng3
wJOeszZJnw85KPxrmFYEV3pPLL3HOUBzWSKzf4+ylX5X8630u2zt9tUBr+6i
k8458ZL0nJwub7++FcFIo3s9pXstyD54sG94O9ypDiNLpHUI/mw+/dhhMThU
t/tUt6eqtgfkbzXgJNX5AtXZqpj/7E5hJU5SX9ZSX/YPXlDZ5yOjpzxpH2uo
j2+d1HsyvkjAo76/ob4PTP4eE5clhIB0soZ0kr757vS5e/MgXyjV1W7SVYG7
UENvpYTxUakOWV5SHbIurxDnMLljFunWl3Qrb3D57LpvXChoSHXuSzqXn11y
0x3NSKe5MKe5CNwZuVjfuxyPaY6aaY7YZ59O65P9Bh40dyE0d4KSx4u//uMT
1dI51aE55T35Jb8fzO/TXBvSXI83dkuT6WqHMXHAnThgrnU6/0lGPfjEDUfi
RjvfYejpqUzeI87EEGfmWJa72aMNj4lLo4hLm/ML111OE4JDHOMRx3gVnCF7
L2VDQNz7SNxrtA8fex9i8IiT+sRJVkr9kNVLqmFIXPUnripkNAWFvI0Anzjs
QRwe7+OcNfuf/x6zXsptc+I2O3Nbh8ujQqQT552J8zZzilJ6rCVgTZL6wj7y
hcrF63JM11eB839knXlcjPv7xseePQnZkyhLzkhSKdcgSaKQSlIjWxRCSMKg
SKVCEpKJKAlp12ZoT8u07zXVVFNNiz2E38dvPueP4/tnz+u8nufz3Pd1X+/r
1mmGckRMOcJbrvsxedc1CCh31Ch3PAvmRRTINYFNOTWQckpcNH/bwvxEmFOu
FVOu5f3wf3ZzhxDulINhlIPSX3wXFLelk/z9X27aW2b6sIaR/PMvZ6dLOGt+
dU/nupNkH6dc5lEui0fFj+MWk1wxsPT/OS6gHA/YaDnV/F4J2e8l3BdQ7u9O
cbgartwJxp7/5gTGkcpUtzMkt3HG/X+uYNJcMSlthMOk693g0BwiT3PI1tkm
Sa+DBZBeL8kt7jS3FL2WKftqHQ9G/n9zzuFZug3TD7eA9Vcuejz22JjGLXxI
B0lyFJPmKBn/3bemTiPnHLPzB9/3FQI25VYxR1ajq8MiIdqoByz7Rwdil7RB
46jU4X2VFUgWJ3YP39kEngWDPfBTOfiHt4+uCypDB/Pq0lPd5P04WxnP+Ovh
W3S06b1VCabEhx7hNLSBazm74FEdmYPciCMmk4tQviKywCagFPJH2/BgSCmk
LccXbQ3Kx1GHQ46dHDF8c7r3bDjTA/7yVT8uDs4BN8hPBd8JP3sTEyxNyfsp
xO13/FPPPHv1p4YccMThUSuWVIC3v9Xb//lrlNpm/3QqJPlGtkzt6MMoME5L
3xwUFgdm+cjLulN5YDPXjTvg1w7Oef3mk0+fIixtemSsTgtsfywxXLmH7J2d
s08Eqt5E9rytQ44cyoHgp9yiBYElSLdw3Vf98M/fv5y65UnOzomX7V3jHIdV
mkm/Bz7thvLkvQt/qRSh7+COaVd2kj3C6JHnidddqH159JRrXTsqXa7+OHei
FdVqLyNOCsRoDRxifjuyHAmGFyflfye52FRB1XCUGNNkB9U6TOgC65fCot+R
FZAKHKLozupEwumoOh/terAmrVgwuuUdTCsUTlg6d4CxOkAtzjkUrEvDw3kr
ojDqVW9lSVw7RlmncdMVhCS/Ri3Kj+xCfe477y8fRIRnBwbX1xWicvnPwJck
V8puNNtTuUCEMV0151cR35BTHXzoy1UhfH3sCm9ZtcGxoq3X8V4del3tZ+7N
aQBHc1KGiXsremNZ/yT+jgF79JmB5Wrl6Bfc0VF83AL1wuG+yyNaIX9yuE+g
TzYUTYVOg2KFaParH/mD8Iex8qzRu5YgfH/44Xr+y2Z8uPLtU7lpD2wto5XD
PDtxdMtYTfXbTchN3fdkTzPhXn3ntxPVLdg4N7Izf18jwpjl8w3HFKG3qttY
M7UBqa09MyZNEUBstFbNXL0LXGb7jp9M4q9xj5Yf3laHvVFX0kcJSV9f3vgR
vzkBuhc/PwnXJHy431ezfkQu7C/8UPy6QgzZIsVAs6JypPdmFYUM7kTI4TfH
6/RbkJ6gdPixfAncA16ZbdQUQHB+39IMt1oIuj7PrtxWgN7S2EXqYW/ATc9L
Ga2WD/0FnzJmDciE41OlBYNPEZ5FlN4b7uaLSJcL9rOCEsGY/XhQ7a9KSM+5
dtXwC8n1kzuaOrSCwVO7Nf1WrR+E8423Dr7dDBU1m7Z7id2YtTI7XEuhBZG2
oy+bX6/GWpv2p+eWEV6e2Z0YrFsKzlmT3/xN7zD59wH9Dvt2RO8fNyMrvwcM
gUrk+XnrMXTApAFhZN/4kvMyZhSrFeZ+31UKYkVQGnI84q5REz70P2a1T6+B
o3L2A4ccAdK1l9z1+lmNwVfU9o41yoLUdNsL2Vf4uC1MzHtul49Dh5NMUtV7
MNO3/3Tvlh70bq/dMnVzDw78uj+qmtGB/OUnxdesWsFdMqBH7CFE95ja3CZe
K45Jlb65q1WL/oQJWsu1ssBwjd348nkrIg6PHhy7i9TRRXfn0X7S3wnJP0xL
xIjeWuZhevc1Yi6f6uwa24UO00yf3MUCdNLrYnr98+ZjggOBDRjpILmPyRnJ
feS3zFTQ3/MKEf8+95XkubVf/WzuPShGOz1nDj0ns2ntAa2zJBfQ9xpnKXkv
5lBO/q3+LGjTOnTSOvDGXLddqnwbAbRuH2ndntUdnau9sANOtM6atM4nzkdY
jzzSBIbxov/viyztS5VdoH/ruRJwaR+n0z5Kj1n/1WHaKVTSvhvRvs94k6/w
oq4Fn6hOllCdOJw5HO6RVAZFqisu1ZW63vi1U7cRPbyQ6LCX6tC8e+Gva5xm
MDMkunWiuuVu2rmlZ2oB+qjORVTnvh6bHT/cJD5E54L/71wUeBnvutoAVzpH
8nSOmNqCBPuuNHCiJHNnQueur2LA8INn2sGncxpC57TyjvhE6zLi89WSuebR
uXY4l7FjxYY4+FIf2Ex9wK3Nr6PKqQ0Of/nGrAvMZ2qalcQXJT7TT33meXIB
e9acm+BSX5pPfck+Vtp5oGwLHKiP/aI+ZrBT3OAeWgxpN4nvuVLfm9I+KVGK
3QV56pPXqU/KyXpdXrO4ESLqqxOpr7LW+/Z9HpgGeerDTdSHmU4tZ92lROBR
3x5Dfdt95anUE4kV/+PzYafjAk3cu8GmXBhBufDrzdxxCWT//Zsj/B1tM0+Z
kJxwRsIdAeWO3uKfW6JDCRcOSTjlRDnlvmzplO4r2WAkSLimS7lm8DGvnbG/
DdK/JBzMpBz0fTWqSDSlCE5/cfNAbHjqrhDCV8pZHuXsQqf0+36uZeBMkHCZ
RbmsNkKnftG3TggoxwWU49kqhweP2VkL6b+4n2A4d0t+KtkH/soJ7pFL9wsu
/Pk8E0mukKe5guvsZRnSlwQezSECmkN2CEIDCqJJvmWZ/n9u4dLcgmuR036z
+GBsl+ScSppzwqIDo3amd4D9Vy7Sm+uhINapAWesJEdxaY5i3H3wyX24P9Hx
tA07uUSPx398OxJeg0JNGZ+7+e1gMK9tKFrqjbCZd3xkyH6htHq0S+qPKjB3
J2u/ayF+9Plj4tADFejdM1tvcR/Z25kTD+WoloCnOnpJwcoyzGBuDBmYQvb9
b6829/l3I+HnqrXMCSVQdHy6+8niDDCUBL1ahJ9hu93ytrfw0b0uJcLuMcm1
qx9tmDe8hOwNLt3jbPPBFC03KPQn/hOu83ztReJ3By+sUxmWA4fMSk5KMsmH
hd/Ea8geyoq32sx8ng7zrVK105+TOVUfqiR3sQq+j/K5V3bzYHVh9Oe5XSQ3
Ose73t/OA6uk5En77gSc8Nw5zHtnI5grq2W0PpB9tPHlCDVWJKQzu++/GhoH
46NrCy+ZC8F5uLTwUhEXnNgrgQ+mkPzhclLtDfE9hsvTj9Lv7WE0kznakfiT
vKyul6gvF0OPf91Vu4zoMM7D48Qgks+Pn7ZM3n4T6h0T2nV3dMO85qzuzH0k
jyU7BJf/6kBUW+vMPM8uOH9KD55+rwPca0nRq0Jb4D7ONMjljRi7dRcm+1+r
QmT0RPPLvgKM6otcOLC/E673x/6+srwHvO3xP2YaVaD4xZd7lRpEr1ba72W+
NkLAyNKcduAdtA7IPD1A9hqH1a0TdYyzwIubPG1ETiTe+sYvmvWiHYd2zvPN
nUbm+8Oq4Idkb1Ke8TNkL9mbXDqX2i3aUAX34ONF2edEWKjT7fhwLMkNOmHz
Bo72ROU2ZiLXuxlOvU/46ovbIOT0rT+5Twh564nMPnEtdHc8H3d4TStUzM9u
fatQhEidkLeC7cWwSLkwwHVdC77eTVo5pUkMjs+Ojx/2paAgKfjVTS0hgn59
2d1mTHJyynDPvV1dSK3fwIue0IyQsc9TT5Lzs9hzzJZ1tkF13pcn7ysaUbxx
Zvm8nR0IKImMCXrUCNeBnfq+ZwWQG5Ll5lpVB/4MmV2fyH4cmD5jnduYekRO
di6o3Z4EQdwPzzbPN1h4UeONfGM11DoWGn0n+xTb0yxw+5kuRCw8ctLdkeT3
ld/KJnZWIeDHvQGbL7ai8srboG3vSsDqnnlZ7XMwnE5aGfAmEi5MtGz9XloI
87hbeVmqLdDoOyoaaFwE361b5NdHZCHd4HjXcbLfcyxOj9vw9Rnk6ye9EDJT
kTBr7Az3k10wf/618+ScTrDExnoTukIx6Lb2xF+9jXDSMGsVWAgx+uyjU2cG
98CIv9G0xfodeCGvDDRCq/Fu1JHwouViXI3VPbT0z/dkMQdqXjiXhSNy+Y/D
f4rQ1LZtTsSeJoyomFCuXdaFpVZq7PGEK7kKc5eVHC5GqzmzY8ikFrwShmqo
L/nz+5JVY34MvQvjvKczXq6swKRpTaZXy0uQ+26rcsce4pvm3zbNPP0EFevm
P/9VF4b+g27PZCc1Y5jo9s/1e9qg56DusOc7mct12+UGhdUiOGg5Z3xUDVyD
daQ1c9JR6d6Y0zypErJqiqWTcrJR2//wrVZXOaZZ2F9pZ9XDumDvgcndz0gO
+3LJyysdctsKjr7Nf4OKf69fklzfcMBwl49XJ3bR+4T9lNxHrmTdcs8GAarp
c70eSJ7rvXzs1DGfmqFMz3mUnlNve/GxX0/JvFhI3qt+veS9smd/DikvIn2n
dVhP6xDAD7Mys80FY5ukbjm0bo5dV5WylXqgTutsSOt8qdt+98TsVjAWS/ri
TPvS9Y9hbKhaDUS0j6W0j2abq6+aBKXAl/Z9Iu17xnr78KHK7XChOpHvkugk
O/9SyaAZtWBZSnTFprqSf39p7OvVMTCmOgygOjxrsWZ4RmIrwqhuK6luGTf4
ex3ulCGB6lxEdS5keMdHCEiOo3MRQ+ei12rQTccRzZCPl8yRGp0jjQWPd460
IHv9TMnchdG580tm7nX1EyOMzqk7nVODM9NSdMY3Qp/OtRqda/n0C6HmpzMh
R30gnfpA12kT6aYFHeBR3+BT33AaKKO+2KoOPOozltRnfs2Y/UGlJwHG1Jf0
qS+ll7DKgg61QUR9zJn6WP3PMbbjFlcggPoek/perdW2r+9u9CCA+uR86pMT
R+zJtd1G9sF4ia9mUl8NOGXxuszrHRgDJD6sTX2495BeYu+f8//l26KVl6dX
j6v9H5/nDB81g+cdgATKBS/KhaC69mC1Zc2QTpFwJI5yxFhF7HTFk3DWWcId
bcqdUdd5OxwM28ClnBpBObXrekeHzLci8P7lGkfCtTF90f0PzpF9gXKQRzkY
7ifOTMwlOZByk0u5Ka80ME8ZXWCclnCWTTnrJ9J+36xdBzblMpdymWFz5sOI
KhswKcfZlONyZT/GvQ9rBI9yn0W5H9kS2D989BsI/soJvJFNFgFqrWDRXBFJ
c0Wkndtj5Wg+emkOyaY5JFmh7PHN6A4IaG7Jprllz/1d8d/dqmFMc44GzTlO
3fuOtmkZgaEhyUWRNBcZWH17Iz+Q6L9NkqOkaI6qnBSfdo34EeugyQdf8xYE
+PhdjVteh+rfd5z27yA5j7Oj74BRCRImPfx1QrUGL/L/iVF7WQD+5O41N7S6
EZCy5PZJxSqYvHywWDOzAwwE6qWNb0alyehgQ5kKrLgYcaXdrAaCSXmv7Q/y
YTtbjZXbXwreobAZ3m6XIR3f5f87tAu8XXJJ01qKodal8kW3rgnyPg9nexxs
hn7LnQ2CPD4CyldkHw9PB6fhrK7B9BLwz08x2Wufj8HH9rSo6//5fIefDMOc
XWDw4+8eXZsDp5aa9lNzyV6fU7Xm15A2RF6ZPFea5DmNfu3QoHrCrXtWz0Nr
iA+9ef6q8Z83ML+qlFIkVwOeZ03fWP0csKZqGz7yS8Soxid+3cQneXnMd79i
u8FKubv2yMhoxI+VWlWaKoD7+9/L3waQ56fxVBpvhiJd2G5ynRENqUFXTm3/
JCD5TpC1assNJFh7ZQf8yWOWfWuPu5RBNrYvKjSd6GVhavrzHpJzKlBZx02D
riBikN0XUlfN21/GkjmWj+/a3b3zz++bvt3QX/Tne/tET/viS9GrJWycMKUD
Ay0rgzYc6IJum5KTVxvhqeDcRY/fQqjHjP28JYzklchLiff3NYDfIRraP0CA
SzMbyqa2kjk1SPneV5gMBzX5W56K5UiPdyq2mNUJYcONG8NbWsHun/hmsUc2
/IcGxr8x64D7xn+CsspKwYjcNiNx1wM8nvvwZP+ldvzeGuYonNsNjaDX876P
I89jR316HiHCi2bD54fHN8G4Y8G6kJpWuCYvnn4ksw2pAeWvV4ZlQ1pBcYS5
cyPsTa+65xUR3gSaXL6wqwPSI3JSNfMrEPTCoF4nrwUxOjZH7A+TeRzCDEg6
QPaqvJnF2tEk1/O22GV2J4DhHsfa98kG4UcdFm90JflOh/PiqFcbwgyZZkEF
7TA9ve3tbJ0mDD7p62H7sQKcH/JPdhs3Q3djknReDemP8iOtk0Mug53Ro7l9
RRX577LSa1KI//fIqp58SvKhlcmDWrks+B7nib561sD4vbVWkhPZH76/qZlc
2g219oIFtgqV0N88sGtbbReyl1tN7alog2XOM+HqC6UIG/HN17irEdnror/b
1xLulpzwTA3nQ+5Xb8qFQdnIlpmzVimd7A2Ffhr/zM/Bg8bZmc1Mkqunbqmo
LI9DQsDh+lvWPMwZMC3CcVkdBG5LTRKekH1ixR40F4VDabDoRvC3RGSv11Le
MVeIdI3puz17iU5e2Z656CeC8mcv1pC1ldjZyMdvsofI5WiZDAsie8eWdfpa
L5IQdD9pVmhdG8R5Tw/EnnyLfpv1n6X728GUOuP6dEgzLvjaze306UTvgCc+
Olsa4KyscdHnXBWkHiSYXSR+bqx6/fzJQ29h/vZJu6rGG4jHqFbKsSsQyFik
OESb7Gdv5nKZe8n+fvryCtH1dPRXrjjq0FQLMeOKh/v8XPRBzdxR2IOYIOOa
E4WZcCmatSsssAjm49iCI6VijFmp8vLAjVvgvTr+OXZtPnyX+fO1fyRBapXk
uvRryXW2ZteeAI4YMvcl93EtltyHe/x3v4PlK6yokjxXeqDkuYza1+9CPe2Q
Rs+pSs8pP8uqd00N8bElkvf6QN/Lua/1xkcj4rMDJXUIp3XgJ3kbf/OtwpRd
krqxaN3MUzbt3cIj/k/rHEHrfH/U1kcyNp1g074cpH3ZeLdpa39kI4S0j4W0
j/lTdoz48CwffbTvHNp3098X8t9xxWBMk+gkm+pkye/3T9bsbkQv1VUC1ZXQ
yaHNQTsLtVSHLlSHJryLxjN/t5P9X6JbNtWtfsfoeqs5tcijOtemOmfad91i
Tn8JDp2LQDoXisP4Ng9Wt8KXzpE0naMA+fK7456Ug0vnzoDO3SK7CbM7HvYg
ks6pOZ1T4/T4xPVDhWBwJXP9nM41z/U3d0obHzzqA0OpDxRevqmqRPYyJvWN
B9Q3fgg2Zq3TJ7mM+swh6jPWPs+aPh/JBJv6kjv1pYhtI3x7ytqhT33MlfrY
Co29D+WraoiP/tf3pO85xyZcewYu9ckA6pO+5iLmtJst8KW+mvmvr6KyUry0
GCLqwx7Uh11MwwSMvWLwqG9rUN8OCE1WYBoSf1ou8fnB1OfZ6bwVgWkJ0KBc
sKFcWPF1ZPigkBbIV0o4okc5IpC7O4eXVohIyp2JlDsxjvyNsoM7IPcXp6az
2g4VkLzkS7nGolxbn2YT23Wf+BvlIJty0ELQc9rsQj1YXhJusik3GTVpB1QG
3gX7L87KatrPunWsCZGUyzzKZakxo3a5uZGc5SDhuDzl+ODRMiPTxC3g/sX9
yMlLl14/XATmXzlBzaD3qk1ZB1g0V2TTXCE/J2T8iTXVkJf7bw4ZZeWW1R7c
AybNLQKaW5RT5y0fadIEAc05XJpz7LWOa05tSwfj/H9z0ZFRn4qjtojAoTkq
jOaoyt85Y1TWVEB++dbPruu70WfhtSr/dT1afNxiE37WgaHzPfosj+j+0tOb
zjm1UJ1bemQaOxq8QTbfMxrzybm/Vi4urMbm4w456x61gJEQNmP6CTHRQfup
G0WVULVb+eK8WRF6fTrkvyQ3gms5tvlkUTniVPZ+t+V0ghfoUXFwTT4EgUUu
A/il2K2cuqJavxYCtTVtcRpd6EvpHvKsoBi86p7oOf/cgOCXTPqJU81gRVm8
3l7Ih8PN8HalpiawbJ/l3l5aCqZWsNuDsHw4+J73aDucRfj5c/P8VQ8gOB58
TpuXgxETbjKrq9rArBQFrwgUkTmL3t/akYHddeWNwc6Ecx5+Novdid6OdVjE
q7xFlMY2m5u+ZD9kiGeeKC2CvGnJ7IjJKThuovhm3WuS7y3yb/4cEwHWe07r
iwlxeKiddMGJ7AF8xjnT6qHEDzun3Vsa/BziH2b9XxUbie4+nYyPIv5REJBk
7R0MrvVttyX3XyGy7QXsHUi9t7Ev1fhexAiDJcOHHWoBc3eDxa9xxZDRbDhf
5t6DXleRocsBsl8yT1XIPE9GVLyj9ddMsk/2uo3bdrYDlasDB0006kbT9UWD
ioaSfra2r63eXwt5vfVqsnNJ3jj/yNNerwssYW/IwYQQcPQHOcaPEiKv6Uxf
2kWSz4waJ464Q/LLRubYMWKSa/wmqWUT/9e9bub5QYsP3wMFiw0jSuDxyMtw
V08HUjadPR5dRHiRhk+WV95C9+SJLycnd8Dszh2nNbsaSF02Txq4qQdS7C3G
Ao12XLD91lqQmEJyVOzBjKp2DA5yTTi4XoS81vujlWrbILdhmMe9pUJslhvh
2bmR/OzVE7B2cSVYbOGadam10LtpkzZ3Jclja1culltyBbwJE6waHvFR/GWR
b//MFtQrVzepJJD96pzV0wmcSDgZBhgfaW9GwuCFVWdlixF5y9tTP6AT9UeT
phbebUKCo2IJY0sXpNcYajb0CjFwzvCZGUsbwVS8kvnNsxGCbpXFuatr4dJc
uEZs3wAHVvKmCwtzwXMO5hqW5UHae4xpuFYtwgK3x3ODCJdrm3/fgzMEG/Zr
6pVUQqMnZC13Yx1kdy5dY6zYDnn5oeGFqmVgy27YuepuLMJeae1mDGjCtIIC
mWKjIkx5d3nloIpWGDcq7v19vAzuHaseWv+Ti6PdhQERm8vBCt5SuzD6FTSy
tF6LyghPd9nsGP6mB2pLxm/Q2kD6cHXiUO28MMz8orRowUshXI7IDI8racLG
Z42BKYe6EXgxVSl1bCm4Gq5J05aX4J1/y6b2MJKfg2evLS3ogXTRsNE1rj1o
DfXqd+xtgfG8rRHBo1phOvvpwcTjQlhckUoretCAdI8hP8o9qqDs+M6sZkQR
Rj5eYbJIowCM90UaBSmvMdl63dzlM8RYtHa2q39tJ9LNI5alepFceF0uZQ4E
OBFUe2N2UCnM38iNs5jajC2vQy7eO5gLDW+G/j2pGgRobDpQR/zrqsu7vliX
B5BaeiTTJKEFfatDXMXbBVA6I7kuqy65zjkR/2WufgOy6X1MfCT38dtVYTDO
vBWK9LmD70ueyzhTULc+pwp69Jw76TkjAzfOGpb/Bl70vZ7Q98p+06KhJ0X2
B1qHq7QO7lpnLvhpNWMRrdtnWrdRz7aGJreWQETrXEPrzFD651DtTScE0r5s
p30Zliovlu8je/u/ffSW9DFgfOTDJ/fKIaB9N6Z9zzcu2Ljp9Dm4U52EUZ3c
lgqIflJOcjbV1UKqK5f9vS7fNpH9lepQjepw847y6Ft/vg9WKNGtiOpW312l
9oU50T/VuSzVeYDXNYUzbhlg9EjmwpXORUyPr12DI9E3naOhdI7047uOjhLV
IpvOnZDOHde8du+3fbFgcSRz6kzn1PHn/hu8z60Q0Lkup3MtfrZeK8SnHA7U
BwyoD+R+17q/5M/fNVLfMKW+Ua3pvsSyrRnK1GekqM9MVuUJX67lkz1J4kuj
qC/lFsvPfqxPcg71MX3qYxFRUat3bBQgkvqeN/W93QlD56u+4oFNfXI39cko
dZPES01t8KW+WkB91ZCtU9z5rhxM6sMW1Ie93kfOORXTjV7q263Ut81HjayV
edcIDvX5OOrzXB3Rq7tDST6kXJhIuXCnPkPMniMC/1+OWEg44mt3u+v97DJI
U+6wKHcqSgx9JueJ0Us5xaOcejbMeJh0XN3/cC15b0BYj/RtsCgHWZSDT8sO
ztzObQKPclOacjNh2t0T4SlvwKacdaCclbomO/bFKuLf8yRc7qVcdigrC1pv
yAeXcpxFOR5y13r6UEUyN78l3Nen3OeXhi+MXF6N3r9ywuDz7OJLe3vAuSfJ
FSKaK3w7uYLrNSRv+0pySBjNIfr6+yZa/noDJs0ttjS3KBpYK8OE5MvBkpyj
THOO/ISZB+1jSyFNc5E5zUVypzOO9N8gXP0rR/GX6TyaPaARrPHK6Y6P8tA7
pMcpcSrxL+f7u413lUDaQHOZQ1onmHN7pn7YXI/wJN6VMO0uGO90O3vzz/ds
P7VtUrT78/8Vb6nt9ROAt9HJ4szALPSdTvWN5lQjstV+jE99CriRwqmf5pG8
kWSd5ehXCZmCd8seRLWCq338ZnSDANztcrddQsvBi3CcIfpcDI7n10sz7r5D
wgifzUpJpXBy++BWP5H4W5KvZ7Eu4eGYF1tnFBVD+mC53qMM8h66w8Z2/W6G
u88mz82dfHCUr/ukMJ6Cp61cbKJdDsZhBVHhpAK4X105L7euGZwa19iZSrHg
XzjCnnkgFw6ado5GBVngaWwIa7jVAWbV/vKHRZmY96t0w6Pvbeidllx3so/k
OCOPK/c2pcFoaasO80cJBG8jvouvVIDlZGAzvPI13NNeG/OGkTl9PKNmeWMa
GEYLPfesegXbErfyKJIbeccKRxt4kDm5W6jIaYgCTy94ZIzdZXDnn/ki60Dy
78vz8rs2P8FbrW+j7ZKbkG295MjIe03gCE3yWrbdQQDfSCZ8Wgbkb3uLJw8j
+c5xasQAt11wzRzxzHSgCAw774Ya0wLMNV1bG7m1B+52scGHb1QQ/pw6dOmf
KDRlrToXeKsbletU+l53dMPXtc547bAuGH64eOFsRRd8x72yTiB7pL60p0Fw
fhv09zq+OD/+z78zx8/ZwciC/OQ7a6uvNmEoO8c/e50Y5f+I5y891A6uxbRz
D05Xw3fZnoZDTp346vBg+vtLRI/363ZUbSqE10d20ZV7HVjtM7/cJvQhOKmu
BwcZRWHG9LVX58W3w9Ulr3r4hhZomLk+crglho1Y09gyTQRm9zHjelExpLaa
OA5LaAXL5Nq4pTyyb97cM27Xky4Izmd5GYQJEK43y6n8aSuGHi5V3y5sJHvk
gOFt+aVIvtMx74JrCyIbdpz0OJkNzsnJd3zCXmPaoSnjwvUJhxqniFSMOiB1
8VjXJNMuXCpvP9T/hfhF2NC0xoB6cM+Loxx8WpEQEMKw9G4ET76mei8jCdLr
6iufbGyA4NqvOxGjBaj8edju5e42cOy+uBmtKYKc0m/ui9ZasHeMtqrgEf1N
LlcsmBkGvwvB0e1OVfBLlAthW/XAwe5rgN2KDugekp6sklWGqqFVO2vIPqax
NeuJelYThN7syPjUIjyYU9oXyCmARsTTKxYDyyG1qq7w4MVcXNzTrlkwRAzG
6gzlWsQhTPHdyTlrebjeuclsfoQAtquq933rbQfbXdB1d1owdms9ZdQezoR5
0oU58kYCbFDm5lQc6YKFebVxdWUHIieZfV4izMTsrsWpdZ0itH4u6pXaKUDq
q9rbXNkOKBydeE+d7Hm8yWsDVCOysftV2c0wQTX8gp45v3Ioxyeft1ubp5Hz
XB5QLwo6jQWafuKFl0+j3+qnV3VXIyxm7NUdeKEZsm1p++/PE5D9Y+nlVOsC
mK1dsyv6dDFqg1NmWq0UYZHf2vlBNiJYLDGbdk01GdlJF2rabdux4/j0+o/j
yDnp9YXJkussZY/LzIQs3KD30X0guY/D80bTy8cLcJk+14o+d7CbqpW0LZmX
O5JzjjSUnNN95ZBWA8VaxND3yqfvxTvbualF7jU6aB20aB2cNbu3fSxtB1NO
UrfFtG6C8ZYzunUa4ErrbE3rrBHkn+wfmA532hf5K5K+YHHZb6ndHeDRPmbT
PnopW256GloPB9p3Zdr3rOIYpnfRa5hTnfRRnbh/yX1TaS2CK9WVMdVV//sN
2uv+fJ71FIkOA6kOeWEil84lfoikupWnulXe9M3Y774QxlTnIqpz4SD5effU
i8Cjc5FK5yLmbojix/tiyNE5cqdz5PDUZYZPNOmzk2TuFOncyTv5WeunZYJP
5/QtnVPRZWWb6U3t6KVzHUHnmjvL7Pq6ZYRj1Ad0qQ+wN5U4tsVEQZ/6xl7q
G8lSmRW+noTjryU+o0B9JmToPgPxnjJwqC/5Ul9qNb1XfCWjGzzqY9epj5lk
LTpymfiwBvW9EdT3pFlGJgvW5sKc+qQB9UlXtetxInSAS311I/VV19qrLB/X
GjBOSnxYSH1Y3nrBGYZJMFjUt5Wpb8sazPlHX53kob98Xr6ldP2jL4UQUC4w
WiRciBxpf7osgczFXxyR9xgtnHe5Gvy/uBPuXPTPpgc9YIRKOCVPOTV+Yorf
Y5JD/uZacdMA559vXv8PB9VaRjXuG9gKPuUmi3LTeZDp7tgRhMO1Es4KKGdt
F7co2I/qgOAvLrsKXZ7Z6VZCQDnuSzmuXTdU6RPJnb5/cZ+d42epco/kjb9y
AqPEjj9y3Wvwaa4Io7nC9klG70i3VvBoDnGnOcR+l61v0p1i8IwkuUXKRZJb
PtuozlikK4YDzTlONOdUFJqvnVFeDybNRRo0F2XP+v4yaUocGLKSHPWJ5qin
qyLzzTaS517veCkQNiBi9k35F0UC2M/UKReWvQHr0yQX47S3YGvIVD0obEBw
mtXE2ukiGNvJXFwV2Y4EcUnJom91iJMddvXYPMLxJnvurLY6sC+nxT+Rr0Xm
tfBi/xqi24+uyfP73kBUHAGhHuFw2MqoMZ8a4cB42eaxpQMB0W8iGu0rEXmV
OWUkJw3cA0fzOE5kb9AZcq7+RjkcI8YIK4JJnsz5kG2d9Q6CHqXcTSmlYCvk
Jx5bWA5+h9LdhPQuZEsbi1s6iqFhGl913LsLvHU296d+EUKQncjbM6MIhqZy
UZolDZDfGrvDTrYKvt/9f2lGF4Cbfn/k9eMvwYqLeDl/7FuwDkz5ZqH9Dtxg
s5uye4VgXpdJvNRM8v67e0Ep1Vl4feVgyuvdueCE/xpwPrQFrJUrBzb5pONU
gEnctyIRWMt+80b718F3hNLsr9vfYLfrtbfX1cohf/wL89RVPhibmraeOZ2M
AMPhm1ROk/3Rf+8oI/dI8E5GSKvuIfvCbHZUXF8DeOdOHI0j+zUnWDV818FI
sJ2+FF6fEQv2ZH91v13EP5q3jOiTfYTBc5Mj//wdKEcp1XRKCfEz6UtLWjOu
g/nkzIoLd4vBXyFfwntTjrc748fmVfdgxeSVRkGnxBAk3nhRPigbl75smXx9
eg8+cQNXpj9sAEN8b5LCoJtQfNv4ocGS6HvVi3szHiZC8ItvUbO+EyE63syR
t7rgPXD6oLafrXAfUWAY29SC8JZVPJtcMe7pWae4+5Whd8/n6im7BahdN+k6
83Mn/G20X3iQfc1BO/iR4rUyjHq/9kbShE7UVy9/OTW+CfID4+yNldIx7cbi
TwbzO+BSlvBh3j/v4LBhTMYnhR70j77hvGBxO6Sd52wcJO6AyLKpxF+a+PFu
2dnxyiJULvu0a7RCPdjsjJdTiprgPWeSjtVYsk9eHK8+48ErcJef0TmSU4kP
ODr4RTPZi3ck1dcatRE9nOVeWZqFbPsQua0hRC8Xvlx/cYTocWrfaZ0xPQjx
KFceYtwMhuWCsQOHnIGvmcGni/NFqHd+vVS3pRGurC3Ta1WEsFVePidmdiNM
FWI29pFckvA14FP7YrJvjVdSsu0uQcJcs50H3erQ30wAN4j0taD47IiGGLhu
FsoGD6qGkqicdfdDA9KV9nsuZ3RC2bt52lai77BxLYqb5HiItKzbnzS8GWrS
DorpzUW4U727bc1SwvMlDj0/5pfBd6H6FcuBuWALK5x/TSf8tGkvnh0cit7+
df8sUEqB9D+BMfPtglGrtz1+t0cbGEds5o2SdUbYl4KsEblEP0N+PytaWoW5
s27dHtrTiYifF43qVpSRffaql+mdaxA2v99s7dgK/Qtf3dde6EGf89ZBV2yb
8XVlVbKMWT0WOh4/Y0/4qS+zesUnq0wMUWirlgtKB1t1s5BRXQzNu/cGH9AT
IVLL4luYchvCP8gPVTYSI2/E/md282rgeka11nn9W5S+4i9WvFaOpbcNrQ52
F2CR4p3VywnfODE4Nkq2EfvodRt6XdmfM7mtog5SIyX34Z6l93kwqe1gvBDW
9LlN9LkC276C15dSIKLnvErP6d5afvX39VbMPy15r6WrJO9lPGnCEKXDxN/Y
kjr8onVg7J/5dvohH1TSumnQuskNWGF+jd2C/n/rfFJSZ3MZG4cOu1Lwdkv6
wvop6UutdpzJhvZuyKlJ+hhJ+/hotN9bxpFm1NK+G9K+M2/a24jmF6KW6kSN
6uRQUmZc5zeyxxZKdOVHdZXm6WcdNVQAeVmJDvOoDlkNoo82Ta/hRHVrQXX7
4WL0kJFtbUigOm+lOo84uy1jbUgFsulchNO5kH3S+ONzRg94dI74dI4cUl3V
lbOaIaBz10fnjrf6yio9dhGpp2RO/eicOvroaCa/I/l8h2SuP9G57msesvFb
N/E16gO/qA/ITTqQ9cEoHRzqG/LUN07kbrq4VI3sfdRnpKnP9K1zv5d9jeSV
vRJfqqe+NK1Uc9TW+BCEUR97Tn3ssqVxcNLSFjB+S3wv/F/f+57xbfHgYjC6
JT6pTH3SY5LBwsCn5PxJ//VVixeLG6YvEoABiQ+nUx9mqn11G70kGb5/+bbu
gjeCFVGtsP3L57nRl4P+fG4S/y8uZDMHPFax7ALnL470dxU+DhkkAJtyR55y
x/h27Mmbc2PAppziUk756MQHxbUI4fsX11zvPZs7OSEPrL846HgvRJq/t/1/
uOmRbFxWvoTMC+Usl3JWdZhX6oYishf/xWVu6ce5uyYKIKAc51OO+xquUd3k
FQ0e5b6Icj/y0G2FzHYheDQnJNCcIHjctMw4mQ/fv3KFRe0xzZ8zyVx8kuSQ
XppD7J+73G2aVAsBzS22NLewBaZl1/UegE1zDo/mnIvHXTT17ITkPpJcZEtz
0cL3ro993AuJ/iQ5KpLmqO/xDrmXPDrBM1OInRIqgpf1+gt/Pod/9tEkuy/r
esAqHFz+a1wNydtOfbO4Atisn5yiNZZwxnHCaWvTWDCFqlXTnUku46xT5/nm
ovesRpvhWBHcg4Yps6PqYOOp7eQ6guTVKztrth6rAUe/ZlhJew2YtaKGce7V
4Mh48PLtUsCZ3pq3bxbRfcB+vaNxrhDYl8XEdrdDNMIlTduiEpWXx2Zf0yR7
c8Dy/WHnBWC2TWQF3ixHpOrlS/1b3oFz6O3lmPB8sKcPf/yptBRWdoVm9icI
h7OPB5Q8I/lAba9X1uQSSOeZG9h8q4Cxyh3rPSRXSp29f9pnTxFaBMyqLYe6
weYonEkIrIW01c/m5ccKoZLbfbW7WADWmSnZnHjyfuIQYadKHlZdG7LE6tYr
RH4OsW870APOE4Xnm/uzsbfLbfKUWy2Q/nFeecx5Ecltqvln6zOw4kLXMENp
wtmpQSEv2hv/fH7f0c0lb1FZsM5T2pvsrbdkThzTqABX4/wI2ZevEbH3251j
1tVgnb40L0grE4Lfgvi4sldw3fP7jc+HHog8Zu0eEUf6c9b9ztpRMTBUjIvb
W92E7Jc7ZEYO6wDn1a74/f4RcOjXGf1uJMknUy+4bj0hBOO+4aa+scHITR2y
7/03snffGVZV8r6O9HO06HGSG8THX5cdjCTPdR3xLFyK1CtEPZV3m/imwtNA
uzN3weDJfLBazgMr77fBt4purDIs02JeECI70bPxrE834iJmKE6Q7oZB+rU9
nt58MJVU0rR/iZDenNbnxeqCbSNj39O5YjjN1/vya0szyBbuX7FPjFxhpClI
3ucrzNriN6EGZx8PbLF37UTCwKm/z5zhQX7HjNe2JwswYk2aMMevA4cqjVxk
Vcie5vbliUpgGMxlh5w9fqsdeXqtjrddKqEsP7n7smEn/MI66gd7ish772+W
M7wI5VOOWneIf8xYErZz2iHCj1+B2n4mQvDzBg3XeVqLVaKx+vvQisqjpx+M
MSH+6Wx21IE8922xzvYzP4Xo8B92yV2O5OGY8gL31lvQLtbhhz9uRhACgwd9
J/7wyLDgdkI7jj77nuSt0QRRvlNcrV4awlbeWLbQrQnTHB/edn9BOGe29Lvb
IxEc/Ix2BJLcLf28fm37yHoM7BOPW/e9Eqypg+WsFydB8cSsKMe91fhWvG7R
g3nusPQZET/WqBP9XIP5hlvLkVj50v+56Z/vwzxz92E06f9zk7t50UV4PmDb
4WUZfLANc25MqyyGQN1CacndbPQVbrvQdUEMqcYVMeMNib9H83bJ7YiH/sH6
Ha/JuUqTWTf3KQlxTnFCTG9vN8kNg8VfbdIQeUTuet7hQtRcebp4SnM7xEnO
RisbRYio+q5jbdcBdetlA+pXNYNz3Hf38aZqGMf0bTUjvjbbRMV3biTZKyu+
Rzq990bomFkapnu78SX+86yU7d2YcWhY5vaFTdjfXWSmpdGMaR/Dt9hYN4Cd
2hwUvOchLhdunT1atRoaMz/FDNmUhLqHzIpRZH4D6XWGvOQ679k+v1ODhPCk
9/Gm96lOG9f6KKcbNfS5KgmS59rPfn9MP7QGgfScFvSclrLa07Vce1BO32sD
fa/BGfEbMrybYHxUUocuWofIsRPefnfOw6gUSd38aN2cfv0avEqpE4q0zg60
zrJzVuue59YjjPZFepmkL71nF3fdECWhlPaxlPZx0Lkt+rvvtcGd9l02WNL3
LRsDix9srgBrmkQnalQnHpV6J7fd7wGX6moa1VXpxeEnay40g091qEh1qPhw
/iTR/nz0Ud2eoLo1z9tZrL+oE4wEic5XUZ07cLKS+8j9uHQuculc8NamFSYt
//P37JI50qNz5KD39nlfWStYdO4U6NxJDcy8VGFKzk/nNIDO6e5tRwMLo3rA
uCyZa0s612drvdZEv2kGm/rAKOoD3Hu8Mc/XF6KX+gaH+oYyNDlcQSfcqc8k
UJ+ZoB60+XxOA/SpL2VTX9r8sS8rJDcVldTHEqmPxUxftiExgez11PdWUd9j
7tvqcf9HOdjUJ2WoT6Y/nvJQVrEHxn/5qlBXfkJOaBN8qQ+zqA8LBrXv0pIj
/ae+zaO+HcOJ1B6t044+6vNs6vPy8Teuq67/Xy4wwtQGuAgsIPiLI2cNL6rq
8Zsg+Is7CWmVxSlZGWBSTkVSToWuGN5f9bXtf7j2UfFT3ruiUnAoBxldEg6m
bLivKZtPch7lpjzlpqOdglDBoh4OlLPSlLOM7vf2b6c8AZNyOYByOfV2SfVG
Mr+Mw//leMCmt9yNc3P/h/tqS6zqGQvawTgoyQm9NCfI9woO13pVgTNekivc
aa6w3daqPvcZ4aWHJIe40xwSeW2k9Nckstefk+QWX5pbuEWjZ3aXZYFxXJJz
NGjOSbh/8EPh9nayv0hykYjmokn78908GqvQS3OUL81Ry2Ovuau+8QfjumdZ
2ekQRD6fHa+t0oRdkSdy0hZ1wPjsxlnGhq3oz1DNfb25EZfKE9xek3zH/ZKg
qTizAq6RWlHmJ0k+DrzvL7soAgzXD2e7VgdDeo9J5Qv1Bmg7DXo18c/f8epV
xprWtUJwfka3xtk6sueOOvRhNdlf1lZkb6gle1+KoGFBZg10lwdoFBaTnNmV
uHDM5kToH1Zo9hhbDeZxm80hBxvA6/moaPO5Hfom6iVO2yvBGHlz6b6PMZB+
rCqz9KMA5me/z1v8tBwrVMt/rN9H8venUI/xhPPZa3tmfftdCrFBOF9OhXDn
xTRf+4890G8o6xq+vQQVTnPProgi+p60OCjhG/GzmBqv4ylFUOpYpxh4oZrk
6FW/X24jOfkT23W4Mh9XH1T+XvuO6D57ieCEQjE4n3XH2qrnw/9IciLDvAmM
VxGzUhojwQeMBCq5YLJ2/7yXnIbeNR4lgY6dkD54+boT2fNk7drE5bfIPi3e
MzxOpgUcq2DnZevTwRi6zsD2D5de+cdJ1dSBbfMhfLX9GyTMWdmtvJTsKdPq
qrd/KQJ7oLTTqPkpqFRWrtFRJfuPq9RBY89XZA9aE/XwQjy4We7hyyJ44MTe
7Kh36wIv+OEW6aqX0JoZYLD3YBvMJ5QW23UT/3hRsb5VLwxcJ/6a6yFlYNdX
r2fvbALncE17947bWFqdfmGWOcmf5xoOr31aBYZo4lL5hxbglJx/aNPfSHxv
drDByHy8vbHsvpZBD8burLN63J0JxuS99QcnhCM9qCbbkNMNrpu4q/ZaOwI0
Njec6ejEW0+/DOXnXZBxTd4Q8KkaAeZ+i33lWsFcfOmQQzGZ25VBmso/H8PX
ZfVj0VwBJi5rvpTT2Qmn8eIPAw62wDd7+uSxiqWo9ti/TtDXAUbD4hXxhFu8
cyoyGcmpcE9cZmbb1w7+29jVd227kH1qgKfliS5Med+tM7ldhOSNfoPldjTC
ty07QNDVCvaxqrbgnDYYD9Heln82G1L7IrrEOgJoyCiv8ApohaKm7xabH+0Q
KH4uWfiuGHNjrjZL/fnewXm33ndeJ/7e8LsieGw0os4/Nb37jezta+a8kUoJ
A6saFbcWdiL5kbR/MOGbb0xW2HS9Fih7ak4dltiM+UH2cnEfBFhevPex3Hcy
F1cNdce+rwDTOXND+ZF6/LL9WTI6VgxWSkd3welUWH5zORt+uxoK7x1z0xwE
GMxvarcg+5nU0V1hrtrl8A1W7d6SwoPD0/Vi+7eNcAnaZRa6sAjTlijHZnW0
gZn3MaFXowCCBbFaVZsyERG957r3wgpoGC4MkDEj+3dTWmuQ/3MYey4782kq
4d0iOUPm+wYMivc/YWDUhd7YLKeIbU3gbdkhLW0VjWvusfvsrdrANuXiekUW
DqSOPPX5ezNeHTNPshjVgO5qha7wz0RvG7y/GQZl4mr87NBvJekwtks9pPq6
CBdHass732hHnF73u22lIvSz702XCxZBReHwfqFvExKM3jNOKdchOFZz5EsF
PlTp9Vx6vVdxl3Mo0fFTeh8RvY/uaY8r90/ng7lR8lzdBMlzF+Zys/fKtyGE
npPhKDmncF3inHBZPgQmkvd6SN9LuWOSw1SyJxrTOoyndci+4TAm1LAGlrRu
PFo3js+YbQ7nAyGdL6mz/EJJnQfPzY9L2C9EAO2LH+2L7+0XNx3m8yFH+yhL
+3jQ7Njgp/xOsFIlfbenfWdPn2jnNKMBfKoTXaoTo3hZ9jWzRLCorhZRXSX4
50V8HEj6Q3WYTnV41N/hefqxMvAEEt0mUt16P+069o9jNxhzJDpXpjpn2KkN
81BvgjKdC206F5VTXr88IZONBDpHtnSO3H9FPjEUEP7RuVOgcydvU/GlcE0d
4bJkTr3onDJEm5rmJMUgks51LZ1rc+k2dsiNVoRRH5hCfYBXqTSlf04Zwqhv
qFLfeDt1r4uKbzfCqM+kU58pSplVFU3emzFF4kuZ1JfaRBv0vRJywKU+lk59
zD9U39ZUl+yr//per8T3Hpj3J2gNq4Ut9UmGg8QnWRZRJW1nwmH7l69eSptU
F6fSAi71YQH1Yek3hQeWzCG5w03i21zq25/yd0z4UNoJ7l8+z3j+Yp3r3Qaw
KBccKBe4/C9XE0zjSO6RcMSXcoTvUlP9LpTs+3r/5c7ueYvnbubzyR7+X04l
9jzmVEoT/v7FNa9plz1+GdWA+xcHHZ48H8+wdP4fbhaMqOl6mUp8jnLWmHI2
YureYUOXZYBLucynXDauDr91rJ3Ui3KcTTkufXeLlO3iMvD/4r6r1YbJN3YS
DnZLcoIxzQmGGQXlKuvInktzBYvmCobM/elpMTz00hwiojkkJuNb9lAy34xL
ktwiR3OL/NhvV4sPk7n4K+coDvWPm6HT/T+56L7fbB1uTyMYfv/NUc46c6p7
dbPA0fvAc35SgoXJRR62JU2Q9fRpuji7BZH96ZcW/OhG6QH+sO+/G6HIPPcg
f2UxmdNHF39bCcFeJn6Uv6ARAWaHzlr1iokOToQkDSmFhtKlRqaZAK0XlFYV
3SDvq1Baebe9B/wI4aHvg8n5R/s6myzlQb7+seeqia3glorrVC3JvHkWFBta
k/dd+0BmmFwVbMXBlz7G1IC3edqekXnlYJgUxQgdEsF58zhEW5rknqErzMVH
ukmO2tT+2LoD7io552ccrkTE8/6F19mN4L97EL05rRGck+9ORxSXw1x8e5BB
DQ8C4aLjSl+KITr0uadjeRksfgToG33/8++wsRM8XMIQufcW+9XTEvhKXQuM
sikBv+DT/g5RO6TMpu5IIrrrf3XGNsVGjF6pOc82Bzah1yr0/OpgPrTq7sz6
NL8enJEfZ+lrV0C64Vzy5Y0FYETpXt899BkY1fVl+x6mQdpqjIaX7TvUdiXu
Kr8thPFppWUux7vhgBfyld7ZYHppjVtUlw+BXePEruVkv+gUWvakZsDWze3C
soMd8LW1f796SROYKnYykZ/fkpwx4Xp4ZC142qFfjUIqITigYXTkHx5YZ0ad
FDmHkXo+vn1JOxfc9VIjV/MTke2eEHiljOwLl2acSJwaCI66pWqaeiw4Gq5B
My7y4e6vscBmBdlfLZuH2cu8gJnplB87czvhUHX1vtp9klu50V9XuzzE94l1
yidr6gmPyxZ8/NIAxtKoMbVW3siLHXLWSvcVHBiWh+UCSuH9bIvO+NgeaLRW
D5z4leyz+5s4G+e/hf7ZHNVNf34vopP5a2Z4GSrDvg/zf0l+vv/C94dsN94e
NjqVntINqckyGe7JIgToRY2pUe9C9b7xh7UjyPv3HRRWkr0wV8s5LmmDGO7v
I1TyD+bCeJ/gcHZjJfKGNp+yMyc+vMT6ccClDsgHGV/cZpaNMcv4k2O2kv33
1R69eLd62O54x992swdTEi8+MtQl+Ulgt+5xQQJ82elbR3WL4B66/k6KgghB
G9ij3zQRvepuio6YTvpuGjlQtYfs6blb39/RLINvT//bEaxyGA6/e5pJfG3D
tS0Tb+oTfb4UWB85+BoXut0LNdYKYTD95bz8NqLLwj0ocRZD/0VDiG5uE2Ii
k7/P/ZAB3hGD5RpxQlhd2vdEeV4j9nmHLZopRXLi2Lvs1eOqYMLNHxwQTfbD
z4U5x8dXgbfc6X7Mo1SEbPOxWxpYDcWTlSql/T1QPrbwjfXJDtjvrLvwVVgG
g8llg93MmsG/F5lZri9AwPcS13JTPuR3nvJrVciFb+GDrUUu6ej9feilt14a
DO+ES489QZ6b3mVQRvIKR+rJ+YkXboKb4l1R96gKUvqD9zlGlqPMO9172r5O
xKyP9vd61Y0QC+6AOr4YimfGXVPzIjqLKXQ5casBvYZKeouKaqDhW5uuM7kC
V++nbFL8SfLbM4564WEX3BQ9LAg1dUHlmVa9ZhUBUk78o2nT3Aq11Vzea5JH
OxZmnBlbn4oSen0Lvb7GYq30CkWSC9Ml97H/LbnP90FLjmqGNEB3g+S5x+hz
GZEDxh+tU0cTPacuPWflnIA3umoCGNP3EtL34rHe+anvC0EvrQNjhKQO/VwN
2XEqzXDgS+rGZBz+/7o5+G9NSVMie36QpM4xtM5+oY0WeifboUv74kL70pu3
9ctnY7J3a0v6GEn7yGCu0b5mpQmGtKTvbNr36UPkpq450YxKqhMbqpPIjB1r
Lja8gzvVlQHV1d0S45UnLnaAFSXRoTvVoULGvYg9TOIbVLfGVLfS7Zl3np94
igSq809U5wFTtl5W3dWCADoXHnQuzHej5YwW4TGdoxl0jiIvFqsvaBWDS+dO
hs5dv0K+pWKKAA50TgvonLKZF/fea8kAi851AZ1rpzS/GK/d7ZCjPnCH+kD/
P1WzLodWQ0B9Ywr1DfaA4XYJs0PAoz5jSH3GsDGQ0Tuf7LHUl3ypL61aYvC9
ZFwx5P/1MXWJj81eB6ZjBfEV6nsc6ntGjtfsdK8LEEZ9kkV9UvjIf7xTOQ8M
d4mvcqmvBjSnRDqktoFDfZhHfXhwt3em2KT8f3y7b5mP0yanbkRSn2dRn/eb
+jkuPLQRvZQLPMqFyPb2z7PWZ8KBcoRDOZJt+fJ0jLuI7HcS7shT7pTO9tDY
QnjK+4tT3JiauNv/kFw9/L9cW3NNtcetQ/A/HHT4RyZZtiMZjJb/cjPwdPzL
xxtbId8i4Wwv5WylVMGhnSPJz5TL7pTLka6Tz6nkEL5QjvtSjqs9GPq4ZkUd
GFv/y335mzNdtBrDIE9zggPNCa7lRzZ8Jzxl01wRRnNFgMjGYu/KIkjPluSQ
SppDHPu3N4Z/IPNNc4s2zS2R3yJP76gk/vpXzmGvujpq1JkE8GkuqqS5iL+P
t2hyQyt4NEcxaY7a0y7Q6ksog/Ryh9YLG5ug8XViT41jMy6tcWFVKQnAKh3b
9t6sAA5tLKspYWRfHaFfGjPgDRz2W2hfmtwFvxcfbHnVjTjrbjFMZyHp4xLj
8Cc1TVCsXT5abnQjzM2l7p+rrgCvbZ+5fGoR+k4rzzwOASKmH6kLcuuBw84T
HefnEn+Nu9xrJ6wH97H5h/yv5Dl2eb8qY1pQaR9geGk92U/5hz9d4fz53pJw
tbzJJHfVrd4wNbYGd34t3cYwIvvFOl39eR4p0Lh2L2CiQjV5ztXhz9Oqwdso
9U1BsRMJWwfA5kYlOGcHdTQbXQYzsF+3r7WJ8Mo2umJ4BV7ryK48rkf22HDr
Ua6fyhDACpq+6FIZBKxbqt+9ssHxzsSojkTwvDJ/tEmVIufxphbeN9Gfz+VS
Khwthq/6ca9V14vRtPhSgiu3Ag5P1fX1pYhPjQiwm6dQhAcVl342RHRDXiX/
gIZPLXpDxjLZDoXwalQ/tuomyb0zpzaWTyiEvNa32tmheahM948+oPkWjNus
LQ9DQ+HAcS493JqD9AHXZBYFtkH6jtlOP6tOyAfOm+W2IAsJSpvcdqlXEN2l
JLH2t4CZWjRx4bF0aFgWndlY3Q2uY/W2sNkNMM58pJP/6g0ijLyycsjcsmff
4DvcLwFr8LI70l9TIPA8Zu1/LxMM+wypYwd4YKtmB/jYJqAmen7s+yVkvzTf
diTwAJnfTetHPHgdhWdX5wtznlfBfOnXXZYJpP/Zh7NWtT2BVvbhnp/118Dz
KRo0+yfZ7+IezpjXcxfiJ/6FLDMhBFMN1lXsIfzQSFka3L4f5grr1ILNCedX
PlpYuSkfGgrxJk8Ne7C344Rs6XAxOFse90uVPULHO57noxPdUBz8IcuxvAHs
TJ+6ynUkB6oMm87zJ3thydxRM6xT4b5jqunCSS1Qvmr5RPBUDOaz88e3hbZB
/v57q363Osy17ZjXHNOJXt/64eMfV8L31tvv3v58hN3FGM/IDjB2O1vPvWNL
+pJfsas2HHPHehjsv9eOKUWPS6QKCddutMnrtXXgqHbQsehTJH/9rmvcWZKP
hA+npr/6QOYnzshdanUbrhdVep8c3onIw6aN+32roDiqfOGsLy2IajLhpU+u
AytzoobgcAYSBejd7y3EHpUfq4f3vID89rM+gfe6cMgiu8p1CHleo2CJsRXR
08+FQv+TLbj+f2VdeVyM3dsfoifLQ5GKQg/RSGpkK9I16Ee2TMtLCxoqQmVS
6MnSaFHWphKDZEgLifbGlkmLjDY11VRTTetM0zYokvAe733eP8zvz86nzznn
vq7vdnV/mjl67vVDNDc5hMC3AOsPwHG9fkc7rh5MJwjn3K9uhiwVr/cTqlBd
1gV8jtJ9CeRy1kal4w2gOmHG7o/3m8AqbjLT8Vg3cJZ+ZY461wDdeWT/lSkP
gbk3vKS7UwRig5KZtyLLocNwecgG9DzMV3eOnBibCQwzv+9Tzr0GufSB0tbr
fBi0yRB+nNMJgTf/WRluPQDDyr9cQwskQI/Z2LzsOR+iyU9bi5yRL3q9+bHx
bQXw/Mq4m9dLwHzvk09rT6E8MEvf+9hyCdBSwoxNrMuA7Sur9nTlQ9KHB9s3
vuWDdL4gtSxBCtPjmyb4JkjAPuhwk1y9D9T0iHUrvH6+L0X9X4dcoDwm9rH2
I/Zxf3/d08i6G0bxuS74XCo/fb72jQ8gx/eMx/dkuEVQjaZIQcuWeK4r+LlY
H8eYDvlXAj2PqEMarsPlIg3wcuqGElw3lcVE3YQJN776awvBH9eZi+us51S5
SPMo4s96oi9U3Bf6qSGbMKNWEOM+WuE+hou2VglzeUDBfWfjvieupQe/cUB1
wzjxxzjRMrdgu2yoASbG1UuMq3HbeUemjukHOcahAcZh0NWau7N9W6EE4zYc
45byLLbwn4YCkGKc+2GcSwPDw/R/v2/XJXhBxrxg7Mlw6N8nBA7m0SPMI89Q
Tr27BOk85p0h5p19ruqpL4/bgYV5aoB5apUwqr49vRL8Ma/HYl47LThlobMc
6YA9oQO9WAd4P5Rji9TFwMG6YY51Q5xitdsl/TWQdP7Umafeje5PqyVQoqBL
4pQq+lmbOmBiHWNiHbt1LkRVxWcAVBV07ztX9vdtcjtQFXSSRPmfxWaU98DE
ukrHuirQme9xy0GG8E/oMB3rMJUuYXiyG4GCdZuKdZu3WjZPy/k+6GKdp2Cd
lx/ZtbbcDc0rtwhfYGJfoIw2NcS/rQQm9hEK9pE461l2CT4otyr4jr951MxT
NSLgYJ/Swj4lPhelTzJJABr2NTb2tY9dL+NfGXf8lw+6uT07PKpVCroKvplq
7OSbL+oGqoLPqobUnZnhWw9i7Ms87MuVH3cxePcHgLmV8HEq9vHCdWbfyXFt
wFHwfcp48u2fBcUgxjlBjHPCg0kdQU0tUmAp5ApahrW2G8plJOmfOYQUxo23
Yl8FBs4tZJxbONWnKLxpHcDCOYeNcw67KaZt0o4KINUQucgP56JVP/r4Ua97
gIZzlDnOUSF+fwVuT0P93jh+l3+2DEqnyvcJ1DpAPoldV36iFlj984eWzhaD
SDXf231rO/TnnC+O8kZ1MEpzOfmzGAwX8kvPhLRB17NVZE00T5Eip2iQ7yI8
VDGzXV8h/X84/Ei4owzonep067xWUMlu2uP+BeW2lck+bhNR7u8+am8KlaAX
OLCi3lgMKi1GWRm/v5/ucXR/bFE/mK6KjlgtQD/XbNRMX5ALtK0uQW8ykM40
+hUYoHqOezfTdt2KLiBNI184U1UPHAfd6B1FjUA//YrPqqlCfTpetyklH9gp
+TzGhgbw1HZWir/QC7ppi25FmvZC2lu315bFQtApLQ90WtICaTd22DyxRXXd
omKz37oODNWLf155lgW6CRX3zjHqQRh6vGypqAY0ri9cVsDqBN491wU3Y4uA
FBva23JUAPQwU7up01BONim//cm5H9jV00R2E6phbE2kyalQ5Od56xZ/j5SA
h2Rq16LkD2D5LcZgv0YTVFaz8/yVUP+O3dfM3FoJvOHs5PTuh0DaNWp7wg7l
vB28/k+dZWBv9+HD7oIO4H1ds+LOEOLnVzP7CLP3oHJ6TqhdygdgOavW5iLd
UT1nQnaPKIGfwpP3rNBcz9MzOf/8ihToTyR6mzrQfOge5PxpWgswF/osOv+8
DfFT+Xjh8gLgFRQPBpC56PeuLamQ1aNcrzJodoYHNdrnppOudwHzzZwKTVEp
8Bz378/3fQFOy0JO9voJgKSxR/lrWBJwlOouZpZlQ+mjALpsoA/l5/X9gZko
B2ZlxBwlpYFexGbpmsxW4HZGRdi2IT1ZsiJDS+UBCBeU5y5VKgb6Mclq0S80
P9p5ycUhV8GS00WNs0f6ffH2tqGpAji/ntRXlDIAvbGyoX77BoTDrHb3nNcw
svv2vzcFaO7ybzB1yr8L1LfWLNq7PoiKrn934XMfJB8eIS983wHhMcdNm05L
YOEvrbDQv9C6LC6VV14F7EPO6ZIprfCopXMOfWovXI3coeyk0wdUGct69jcB
zBovD36OclJGQ0NbMQ/hN2/Nd+nUPDDfdZQmHuwGj+d1tZpb8mG4dVPRo/pe
CNhhMvNGuRTM9cw3fa2QANeCYnXpBfKR8zYhzdESyKuNJi09Wgu6d5peX+GK
QGOl2iwOtQvsQ8nPncJQDh/X+JqewYcTbzbEbavoAH+6LDr9RSswsv7x7GT3
g6cGiUc3Rbmrx9f4qlkB+OeXcOdO6ILBVVenF/i0QlZI6QG9bAmIp/F7GvPr
ocRl16X8xmYQ+tyVD74WADXxfszNMVwQtiU9zVneAKVz72W75yJeXKXnzrOU
QqlvxOZPLwSQdibb+KKkGaTuY9ccnoZ08Vi+85T/KQVS+rQXJqnX4bGyyKUt
FemB1sL5NFcubLs1XzZrNtKnqUGZzKhWGHk1zTv9Yx8wIz/UHNe5BEy2bX+S
Whrs9d7w5tBiCfxtPk/P46sI5QNI3LJEDMs27JxwxLgB5glWaM9N6kF4ijQj
r5BCSX20WauaBL58Lec5pQvhr3HEuk0DsS4MZt+1TWqHQrwPG+9z7gk5Otg1
H/kRce5xfG5aa3/f1hA0d+J7Ts4j7hnwLJl8NXoARPi5SDOJ5xrlcxfzi1Ae
PkDUQdWXqMO2w/qHr28ZAAdcNymu2+R1C6SrLyP9TiLq3IHrzPv1xbbK8jno
Tif6IsB96diYFjae0wkc3MdR3EeHLwdMcmMqIRz3nfH/fR833y92KdJPjBN/
jJNYi8XC2HAR0DGudDCuKMkn99hfTIJKjEMjjMMT6jmBB0s6QKWNwO1ZjFu9
3DAfNTrSIYxzC4xzE7UtEwNSe4CBeTEH84KyZcvqY2ebIQ3z6AnmEfVYcJnx
7FzkjwTvDDDv/C9SHt5ldwEN8/Qa5qmDwaQnmYdqgWRM8HoU81rL0MxoaAzK
uVgHwrEOMNInhobXtP3+vO4/dIN61n30fCofeAo643ZictSyQDSPY12iYl3y
bPU0Dr8pApImoWM8rGP/STYYfcxM+y/dm9O1tvBCUScwFXRyXMkP4a/qKmAp
6KqT/sF78pFeqFTQ4YVlFlXnT/z+//k/dVt11brB5015QBr+U+cF+Yy1Ow3Q
nIh9QY59QdVxz+6N/9ai+fJPH9nC8et024f2fU34DgP7joFStqzcrxUoCj6l
lcJdMV2G+HL/T19b/tJ15/AJCVCxD4qxD8ZWZ9z8t1kAldg32dg3XdbqU8s9
+oCi4LPcm1o820OIF1GEL3OwL5fM0++UZLwAOvbxZOzj7HX3bq2K6QK6gu8X
epU6n4mpBlIqkROoOCeQF2g2bdbuAw7OFWScKyxZ0oTEi2Jg4RwyGecQykIX
M+/aAtBVyC1+NetqJjyWAg/nHArOOaqXWU8//H5fgHORGOci0txFxklLooCJ
c1QlzlEV35ZdYxeiXDqu2nUbjQshunyrS49QvrL0qrE++Q7o1nGv1kZK4cFF
FxNtWTuYDo27PPJIBvTakanB8SIQLTS23L6oHTjeZnlat5uAeXIg5sKqfCDb
R9S4erQBnTUmL9A8B6irg/79HIbmIJfbiUlJrWD1RZBjcbsLVPveyzeNbQV1
6zWVv783qoS1duk/mxG/BDNVr42Wg4fJkk/D+mK4Vn0sb28mquP7FnjE6wea
aXrBATTfXZ0tSHHU+/2e6EztU5UucGjfsVvs2ARaSaodN3qLgLd7Z4np0kYQ
NnKaJ3Q0QuGlDYmNL9B8cvSJTm1hEajENmieOdYAl4tr9v+oqQNqRmMIIwrl
grt7nuSMr4cIS93Yw38PAGd0377N3ztBd+dOmBhXB7dsqp5RyG2gq+YTut2u
CUij8YU31tRCpT0jIXUMOveRYePaQ+XAILNvbED4o4lfhUx0RnPf5f+kn5xz
E8hHpEoZodXAZWU4rf1WCwwDM9hMkQHjVETRxqVVsJy6xWLRw99/p40vWMFv
B0qcTJ8srgQdR2n0UHsr0D97zFDWbQTVOY6fL8yrAGbTs0PS+SXA3NJxeJes
DDhS6iXu+VKwWN8a0HigG0gRU8YvN0gG5uP1zEmyd2CaWXad8xTxJLHsmpiD
5pu8NE2/zW+BeaVYbVX/eeAV3rtuWN4J8oNdoVn3CmFWs/GS5I8IR3G1lulK
YuDVVg95DuSDU0OJTa1LOZCU+yb/PC4A8Y8Ludv25gG1de4mm209wDyofzGM
mw9i9UdGOc+5cIs0TztHsxmSiwNXpd9A8+Bn03sjvRmg6/bZq/5mDrDH5l2p
z0D90aa6yRrR/Birr5UX14V43SXZ+wLNmxYvtZrqb4Gl/kE1iUAAjKfjZGTn
ejRPH5HW3dkBOnc+HLIx6Qfx8eBx6jP5oLHfsFyLMgClwwtSHT+1Aunncscb
95ggpxmK3KwRjvQXaMpJfDSne7X7tHRDh09yZ/q+PhhcdDcubAHiSVbM2DzH
dpAXFtnlePQCxd0laaMPqrva0LxjTvVwf4/jtcZ9PSB2ezS0zyseSDuDDNNp
b2HQgv9cY6sMkm4IUiPvID6FWZzOLu6HYkNtjZx/uuFBXeIxk+QKSCu7+HJx
gATeaIiM57dJoM7S9zhHC9XtkHnM5vwWWLmjuflGWBeQmeO7qjeJQPzi+6L0
+xWg/L3t8ptJyDdifJ1lqzlALqdnrjwzAFTmky+0A+0wUhjhfTGlHTyU1TYK
QrsgIPHnL9pllM+8K+cGLSgByixqGq28HkRL1UpjmptBfppXEKSE9KT5S/Li
2VkweHaXEu8r0pFLI+ThuwLkt9P+sjOXgDSta9agkQAu6rkUz5ncA1yv/fvt
xDVQUjcwZtpRPgy+V/4QnFAD5qJOzTe+KFftX5EgdM4Ad3/2wES7bpBOHe7h
VovAx4V2xOAJyv215iUWJ0rAWyreY5vcCzVXmF7CEx0wy8grMnO9GNLkVpc2
Z+eD0Wdr674YHripz5l1EukH/ROx/nCQWFetLXivli6CRLzPZ7zP3/ELmtgB
LeChSpx7CZ+rzpdvZ7R1w2l8T4orcU/27Lh1gkw+mHoTz6UlJJ5LePO0psWS
TtiN66CSTtSB6hXcFVJRANQWom6juG6iZ0E9SmmdQMN17sB11p30Uz74sQJY
uC9BuC/LOyb53/3SDTTcRyvcx8rj4nk9yUKgviT6PhH3/XSxlZPNaD+arwmc
mGKc5MCPyRoilBsxrkowrr71mBicvVMAKuEEDvkYh/2bd7p+noH8HuN2GOPW
rXLnA1FhLVAxzhMxzgMcE8SL3vWDHPNiEPPCaDvt8e5NbTCMedSFecTcFRza
eugtkMau+D/efcK8G81eOVIQ2A3UEwRPZ2Geqk7JStimLQKmAq95i7v2dzc8
BgesA0ysA0ae205ejukELtYNHtYNVnFaRfCbKuBineFgndkeva6q0LAPSB6E
LpFmELoU+sv7k97vz9lU0DG1AGb78/I3oKqge9R3V45M9JCCGOskyYPQSY8N
nISnOkKQK+jqrU7u/E0PUF5nETrMwjpMWXgzy5jcDjys22lYtx2+BKhHufHR
vQmd18U67xce2Tn+azeoYl8wxb7A2G4sVDcVAQv7iD/2Eeo3Z21PrwR0LuE7
FOw73rmJtvuCO4CaQviUP/Yp+RmVvprocqBjX1PBvpYqi79+7bUMeNgHydgH
r+12vhz3tBFoCr7Zcvjrqce2cUBiED6rin02UCfGesyMDiDtIXxZjH1Zq/Lj
gUjVMqBjH6djH/9Z8125IUQGNAXff1y4K3lbbiNQcU7wxDlBt6hKS53/AHQV
coXbqn0HjzZ2AE0hh6hM3xkV9A7VyZ/ILYY4t3DLQIVhivCrkHNYVu0cX5NW
YOJclIxzUZpBb8UMe5RHlYkcFY5zVPiTB1sM73fD/wIujxie
   "], {{-1.4, -0.8}, {1.4, 
   0.8}},
   ColorFunction->RGBColor],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["23: kitchen2", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", 
   RowBox[{"f", ",", "g", ",", "h", ",", "myColorMap"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.3900338162980175`*^9, 3.390033824039149*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{"x_", ",", "y_"}], "]"}], "=", 
   FractionBox["1", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"x", "+", 
        RowBox[{"I", " ", "y"}]}], ")"}], "5"], "-", "1"}]]}], ";"}]], "Input",\

 CellChangeTimes->{{3.3900338076255474`*^9, 3.3900338303582354`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"g", "[", 
    RowBox[{"x_", ",", "y_"}], "]"}], "=", 
   RowBox[{"Simplify", "[", 
    RowBox[{"f", "[", 
     RowBox[{
      RowBox[{"f", "[", 
       RowBox[{"x", ",", "y"}], "]"}], ",", 
      FractionBox["1", 
       RowBox[{"f", "[", 
        RowBox[{"x", ",", "y"}], "]"}]]}], "]"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"h", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", ",", "_Complex"}], "}"}], ",", "y"}], "}"}], ",", 
     RowBox[{"g", "[", 
      RowBox[{"x", ",", "y"}], "]"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{3.390033800555381*^9, 3.3900338333124833`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"myColorMap", "[", 
    RowBox[{"ab_", ",", "ar_"}], "]"}], "=", 
   RowBox[{"Hue", "[", 
    RowBox[{
     RowBox[{"5", " ", "ab"}], ",", 
     RowBox[{
      FractionBox["1", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"1", "+", 
        RowBox[{"Sin", "[", 
         FractionBox["ar", "\[Pi]"], "]"}]}], ")"}]}], ",", 
     RowBox[{
      FractionBox["1", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"1", "+", 
        RowBox[{"Cos", "[", 
         FractionBox["ar", "\[Pi]"], "]"}]}], ")"}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   RowBox[{"h", "[", 
    RowBox[{"x", ",", "y"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "1.4"}], ",", "1.4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", "1.4"}], ",", "1.4"}], "}"}], ",", 
   RowBox[{"Compiled", "\[Rule]", "False"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.3900266630022306`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJzEvQm4ZVdVrj32qSaVSiWphIT0yUlC+j4BISHAoe8EIYoGQnMgXmkUBK/0
XYEKXhAEBJEgUKAiVxC5oCCdlCAICFGQHoHKpZUoPQRSJNzzPme9Gd/a7J2c
Evz/9TybDHatvdZcc47xjW80c52jH/DrFzxsXVVdvPL55ZUPMseLVz5XDfJz
Vj4/GuTf4X8mq/LjQn5YnPOA+O0vrXx+MMh3Xvl8d5CXVj5fH+QbrXy+Msin
rHw+N8hHr3w+NsiHrHz+aZA59/2DzLkfHeSPxm/fH9f8u5XPNwb5dSufKwb5
FSufXYP8BzH+347nemTIvzLnue648vnOIJ+/8vmPQT5j5fP5Qb7ByueTg3zo
yueD8Sw+12dXPh8f5H9d+fzfQX7PyufyQf7blc+3B/l/xxhetPK5epD/V4z5
MSE/MMb/i/Hb2618vjXIP7Py+fdBPnHl85lBPmzl888zxvzplc+nBvkDK58v
D/JbVz7fnDHOF8Q4t8XYfjW+z7m9TVzn7JXPFwf5mOq5OnDl84+DvLNaH/5l
5fOFQU4dePWc8Tw5xvPg+P4XVj7fH+Slar09vXqNjqzVNeP46srnvYPM/H1i
kC9d+Xxpxnj+Mq7/x9Vr9MwYT9raQ2Js91r5XDnIP1ttX7dY+XxtkM+J+55U
vaZHrXw+MsgHrXzeN8is4QcGGXvSBvnvZwf5g3HNd618/nOQ31ytn39VbWt/
Wm1rL4pn/P2abXf5vL8e5/yP+O2945p/FnJi17zrp138WvV8Lq98fjjIue53
iOe6WTwvOuk8pL2kPly/Wj/RX+0obedD1Vjx7moMeVPNtvd8xt+LZ3xCPNdD
4/v7Vs/Pz618vjfIt6zW5xtW228+yxHD+DjAofcMMrr/4UHGFtUNsEHsfXu1
nr+2Wh9eVj3Pz45xPjHGn+tyUYz/TtV4e141Np5aq/bPMW2Pzn9iLOswDx9c
9xcOY2BITx3GhuzcIl9Yq+uCLF4hi1fI4hUyePWeQda+JjGeyTBv4t6rqtf9
eTV7rdMucn1zfnJN8af6oC9U6+THqn1o+p2/rsaWP6lei+fGeLbFeNJm71e9
1ner1oFbVc85fue6OMDBNZsD7KzGsU/EM4K3+ot/qLapt1T7u8SoxJBL6rox
5LFznjfXIvX2grjX7t73t+Jejw45fUHaeN4r/XvqQ/KT46qxKH192s5l1TY1
be/iRtrR/6nWw3lcK5/rN+N7uKjzcI9q/b99PMu58SynDePjSJ3Br6kn4PSl
g5zYmzzhndXYnjqfa/T86jnP8f9GjP/+NVvn0y+fGfdNPqO/mFTP+aSaEyL/
/XAdZHEVWd+KrK4iy8+R71mrnOEnxatL4/y3xXX+PK7/nLivPhd5eZgfZPEc
OWOBY6u5k2PgSAxPG0/M/Itqnfmj6vXK+CX17eJqfUu/f9tqfbtJrdoDR+pb
rl1ye3i0nCpthzjg3wb5n2P86fffGvfNmOW/C6PkkHePeyXn2V1e96y411Ni
DBlPPah6XZLLpf+6dTWe3Lg6NklfP89HJI9NHg5nkbfnnM+LrXKenxHjf3zI
Gb8k5icnv3m17Z9Vve7H1yoecST2pv7wrMY1H4lnJ85QJ9Ovpf7PG/90bOj4
jQ35J/Rfu8Y2vzzIjhkZLvGBQQbP/mWQ5RLIzjOyfgHZeAf5acN4Fqp1FVm8
WojxIItXyNrgwnCfdw/yZ+N74y9k7Yv7alPIclFkdRX53nFOzok+FBnO+eFB
Tt1LX5n69vpqPU8+nHw+7TftJWPktNOb1nXnHw6r9oPp35PP4xvVyfSPyQkz
xk9fz3yKJz8JRiUHyDh3Hkalb52OSWfxvcSu9BE5/9tibP8zvs9ckPrJsZYY
fDpOdM7Tx7FuxlzJaTPH8o7q2A0f4fy/vFqXkp8nDj+ixvk6z4c36vsy95J5
oZOr+XnmEDIPIw5wpL/LvEGOP/Unx5+5R/Mz03wmbZMco1xCrjuJMSMfPowN
HDCXhazOI793eF7kv6nVNUXWTsmT/u4wHmTnE1ldRZY/IItXyPqsddUxNbJ4
hSxeIb8lrqPeIv9e3Ff+yTgvrMbMpWGeF6bmQV+JvLNW13Ga1yUPeWW1nmcc
+qRhHjimY3bPz9xv5k+SA2cM4hpVjXMOaReJUZk/+cdqjJKXcqSO5bPsLkZl
rnseRiXnz5glOUDGvGnXa8GujMETuzJnmOuS9p5cN/PAiaX/lTy2cRY6LB+b
l8fOvFbmN/JZtC+O5CrpB9VzjrXk4TOmkLdwMPfyxrVwxTdV40zGO5kvks/I
JcQr5x9ZXzZtp44ZGayA+01jBeMEK9bX6jp8eZDl6sjG3RuquR+yWIGsnSLL
bZDlqBuq8WpDjAFZvEIWr5DFDWTxCvn34r6PCNkYnzHLYZDVN55XrrJQ7VOQ
ze3MioVdi7Tlh9fsvOhdq/EhfXfWHU6o9mWZFz2gVvWDI33fvBzv7mLUWnLa
8zAq7T25SsZZmSNKfM54nJhXW8g8Yfr0f4jfzsuzJXb9Yc2uW2VuM59luWZz
FfWWI+N0cok7B3mxOoYix2KNZh5XSUzOvFY+S/L2XJesJeX47xvnz8vDnxP3
nadv03n46bwct7YGmvnkaRt5aXUeJmMf5x/7MseYvAJ71K8hqyfYrLEhdn3w
MI4NMc6N1bawR4xtU7XO71mdQ0D+nyFfFOfIUZHBqy8M11ms1bXeFHO1qRqv
kOU5yG+L6ziGzTEG5IeH/PO1qnucb16C6xwVz6heMSfmeZgr8wzM5/aYf/Uf
2dwU8rxc4lKNdUYbnKczGV+sFaO0o5+ER2WONP141qeyZyB9emJy2nj696xb
kRcyL5d8MnOGWcfMmuy8GkHWrTKPnbXj5CpZu8y80H3qumtw89Yx+XDWl7M+
m1wrucobq3lRrldi77y86HQeXrxNf5F4m/MvZ+Cy1pcn1Xn4yfB8nxxk8wzI
cmBk469JjB/bcf6R9YPYl1iBvFyrtoMNqktg0fnD9bFTfQT26zxvHsaJb9ur
2t/tXZ1737c6T7i1Wuf3izlENi7gHGte/FaOt091HL13Nc9BVm/3jvnh/HcM
1+Ga5vquVx3vY+tyPHyd9SC+t2bN2OTAe8dagGnWQZifVw3jXx/PyJxnPn8W
p0W+ZbXf/O/CqOviUfPyUVl3nldfy3g2cTjzb3epjm0zZ5K5uOwTyPpmxlM5
D+pe1djGM5e4lvom2KXNZl4la1jGKdPPnnFW8sasvycO0w/jumRNJHtLsv8h
uVbWELOOc229Ma5d5uHn9cbMyj/wEzkDsv4C2Rwpsng7nSex32wyPJ/5IvVw
IcYPFr10GD+yvAKbkmttiPHDi9Qx7FHfBxZpR+CA84wty3OwIe2FGF0bIdeo
PoDNYgXXEK/4Xv/L+cRBXx+uY82F9VRXD6yuUxADiVd8T+yMPsABxXN+p23C
E43RWAfxBH8itydHYp/VKcNz8DzgpXgI7plLZ370C8yhusE868tYl2mM+vog
O7fI8zDK+kLV2vJRmfdIjJpX18s6WmKU/SccaafJMZarbTxzO5mXzr6L7AfI
mhR9YuJM2nLykIyhUs7ax87qmCvz2PPqnpnvStvfHs+Vuce19g/MqudmPTGf
N9d6HtfK9c183Ruq5zm54rw+GWMK/u9ydSwgz0d2vZCNsyYxZmTjDmT7B9Bz
8RZZTFio7jVaF+NHhmuB7diLXAtckivCB+SK8CXnHLvTP4JLci1sUz94cIx5
sZp78N1lw/ixP/MJN4/5JNbUZ5Hnk+MxN9brwQ31GczQfo+Pe3Ff9RN8M9eH
bF0Vn2zsjE3oj7AhuRBrqN8Ee8xf/Uk8I71s4gN4Jm8kj6IvO6i6521Ltd6C
XcYIrIX5qIW6dp/1z9U+a1ZdL31u1vXSBuflbebVy9ZS60+/nP1gaZvWbjiS
N2Z8kfWCrKMZa3BkfTP5ZOJS1t2Sn2QeW53kmO7ZmDVv87Arc0RZz9rG/8zI
SS7Xdcf7idWZ085+p+TPyRvlKo458/BryWMr20OCrP4j3364JnLW4PTjCzFm
ZHsRF4ZxMX6waGd13luutX74d84Dl7RNcEldBZe2D3NIHOecE+OYl4Mvye2Z
M/ntYrWPgO/I229anVPCLq0bPnj4jrHCSa2Js/5yG/DB9cV36YsfGte/W1z/
ptVczjlDV8EzMRxMc005x9j/dtX4Sa4dzPn68NziObGn3OAGcd8/i+vvXx1X
Mk9yVOZE+8W2jAWYZ/vxNlbnAbLWyfpmzVFbk1d/dJBTV9fSZ54xTvpcdbhq
XBfbFnJyhuz5z7qAuT6OrDEtVdtjcomMg5JDLlbXyrP/NnO52aO4s5o75XWy
5p77YrJfaC29HNPY5Xy+Jp73JTW7p+va+rRnxYnZq5n9A8m1cvz6ZW6R9Wjz
8MhiLHL2EbnWyI8LWR+EbGyITmqPyNZn7S+yX8j5XzeM+cODrH9ZX91Hjawd
bahxrts5J6YzDwx30tawTWs9B8Sz4NOs7zAW+0/QO3M74JKxLTUHeQg4ZB7e
PBb/xcfaP4b9Wpch5jL2BxflHh+PeWBc+s17Dc/HnNy0Wt9Oqbbl46v5/2mx
vvDEnbWq5++MsVGr1OeCbeoGvM9YA75gTeE5sb5frOafPxtzsljNH5hnuVPm
/TZUx3frYk0XapyrxP4uq3GdhWNHNSZk3jX3FGyv9u+ZQ876V9bus684+/Sy
ZzL7vbOOn/0G07W8WXny3DeRvZQZByWvyLx64va8WDJ5WvYPGNdzZEy9lj0y
iV3Z95g10+x7zDpp7olLLpp9p9kXLW8xzjIvJFecDP9VNzKPbY4C2VwccuZU
XXfkh8X3+iNkOeFCjBNdtUcL2Zw2WET8/rFBNg7aUM21NlbntbAF1xFM0P8S
mxi/kHM2Jw/+2f+A7hgXgPHWRMAPeSPcwTgRm5Xzc2/rOGDM4vAdayhPSExg
zuSxrIk88+PDvT87jEUdY770iU+PNYIvuV/pxOqa7xHVdaKjqv3yedU+6/3V
eHhhfM+1xJCrQz6pukfuOdW8i7UyD/zgav9+aqzXgdU8AX9hXL+x2gbXx9oZ
46u35qCQs1c/e1Dl/xzp9zPf8vRhHTmyRp/xy7ze8oxl5vUbTNfydg7yvDx5
xm7qM4f+iOPf4rdydY6s/a1lDBln4S/MY0/3fvh93nceds3rOc/4Ouc8+epy
NYZYg5iOuTJHlDntrMHpl+17/Oog67+QzU8iu38H2ZzkdA+kPaXI5meyZ2C6
z0e/DC7ZP4Buu9ZwJ21nzxj/5mouQXxnDof4Tr8MTttrCkaZU8LujImWquOF
i2POXxFzTi5Ln/uGav4PvokDO6q5FvEoegT+vCzGfEi1b91UbeNfrO5DQw/s
PcOm7ZcDL+W9zI35un1rjNXyUuzDWifcyZo7+Cd+sgZiC/+mPl9Z7ZdvX405
8Eb93WOr+dsNYy14JrF6n1iX7LvYUN1jsC7WaKHG/XXJ881BIa9lH/Hu9kRl
v8FaankZy0znycWB7IvOHNRl1XFcxmj6Jo6si2VvVcZWS3XdcWjuMTF+4ci9
yRl77k4PJIccgCM57bXtmZ21B83cJjqwWKt6hSz/XFfNCddV+25k82/rYjzI
5q7BGXPXyNo1sj4LnbTuhuw8IxsHgUv6CHBJHUC35bfgktxjS3U+FluQ3+4f
z4KdmsMBo7RNrq1dYMfuScSv2jcLHst1GYM9lvD6Rw3PC680nwyu6xduPozz
0mE+tH1+r+0zHvHkrXHOt6vx7TvVugEX2Dbcl1jEGhyYbIzDnFsHXIjrs17m
YdDNHcOz82/mtx9Wnb+9qBq7XlPt4y6J3948ZPDAWvmNq3nR4TF++K1cF18j
X9pY7RMZv/lGxq++IWdPnTWL6byE81OxRhxZy5Mnc8yr5WWckjmWteTJXfeq
cU0zcSBrXlm/Sz6T+WTnjSPjsuyXWAvGZvyY48/cV+a6E1czJp2HXRmbp0+x
Toes3rKm4sNCjB9dtTaBbByNboiryM7hdI+0eoUs90OWh6Bv5t6RHx3yg+Ic
c1l7xNiQrRtuinGiz+aUiSP0C3AnbWrfmFs4j7z6oOoY6shq7GJ99FnYlJgM
3pDTQR+o7YvP2Jz7leAk2g61bmsli9X5KPIf6upjYmz2M71n+L06zNolJ/f7
O8f5PLdx8f2r/SZYoR7ad+E+weyHMf/DYSyMDmh3XJ+8KLqE3sjz/yLG8LhY
F55XHEDXrR38SswP+Rtz3fhw8We/+D5zieiAvB09FJcYpzq2UOO+X/0j8hNC
zhxUzkPmyXPf9Fr4Se7LuLb8tnadNc3MQSVXyb707I/KHozs5U4cyJpd7u+b
zrPNeidM7heYfp/VzkFOfpjPNf0+q+znNI+U+2flAJlHYh31L9kDybqLD2AL
3J21yL7HPWI8m6p5KbI5hOyRRlbHkI1VkY2bkB8Tsj59z5hPZHMC8CVzAuiw
fWVwJ/k23Emuhc4fO8wntm7+lrjKvNyR1XU69EvsosfA2Jx7Wlu8OJ4Xv61f
g3PLvZlvfdz58f3Z1ZjwoGofSt5LHnVgzcGoyZh3KX8uzgdj1P8LqmOxm1XH
X8xz2kLWrHMfnzqfsRVzLlbfuFoP4e9yhh3VtvyW6hjqydV1VfyPsRj2rW6A
V3LU/UPeUmP9kS+hq+ISOqxeLcS9kPWJPEvuCbWPAnkt7zf4r+TJs39A+80c
VNbgMo+dviP3mGR8l+uVObrMb2v7HNlrkfW43Nt7YY17Vme9YwEOaa/FWt8P
M6s3Musj8u2FeBYwypoda22ciA6YkwcT9CPwE33rPjEGuIrzvG+1/XKOeIVs
rMR1jAfRPfkV8qNDfkico49DthbMddQfri/+EzuYh0fPjcGxfX0cWItNYbNH
VscscAZ5I8+qX7hFtb/Dx2qbj6rOG7y62vetj/knj6sfeVGcY56Y/75imDPG
8JlYU3TC2Mp65DSnShw7O75/bYz5AdV7KH6uOrcM5hircp30udlPmHVb5zBr
dsyfuVm4mz2cvxRrdN9YR3yOuUrOt+ZCTKw+cG1rRtcLmXW3roH/MteEPxWX
mH99X+ZF0X/9IPJydZxofIQsh0fOfsXstc5e+uQY0znqWb2a83JQ6S/s3eWY
F99l7jr7xHKPSdYRsuc895tkHWEez8y9M9mPMe+9fNf1vgVul+81EkNYF2sE
rKOYnP1Fm+NZsHd5O37AGiJz7/5u+Kx57KOqa9acY84WTHA+WSt5O7pnvh08
Ud/yvsjaCLL9OchycviSPfBc09gQ27VecFDMGzZl7YMxGyfiBxaHOTq9GofP
q67FgNlyFWxffcO2zH2BzeoeuP254ZrPCvkW1XlmxiznAcP0sw+sxijrZMgb
Qma9xKUvhvyharuAU2k7PLf1lF+JOeR62iPXF4tSZ5DtN0vuvWfMOfeUb3AN
cyDwLvkwuqzO45PFDXyyNkJNTJxhHcxXMFfaFL7JHhjmQZ2B8+f+a/NOjFle
B3bJ1Xku7Q57yfdsZH9y1p7W0k8+r1dzXg6K32UPuTmu3AeU8Z31EY7sa8o9
yMbaHNmfmXW3zGnn+2Gy/3wt74iYV2tIDmZOgEvJP5GdN9ZCzpk9KmCU/h2u
Yu8B8ypvxH7NwbJW5hjpa3PtsF/x6tbV849+Wh/ERszfYjfmFvidfWJgmj4R
nTcmZTzqM/iT/YTmOTlfjsqzandcU9s5vDqfAJ/K3It7rE6r5q6cu1irtnqb
av/4i9Xx78OrOdjzq7FFH/i+YZxys20h71vNnd4Wv310jXmdOPmxkG9bXSe6
Q/z2/XHNb8b3rK31BezS+vhvVmP18bGOWXfA9uXk6Ix74rKfn/PFH8YoBh5T
jQPMsbyF55Jf4RfMl+J79U2skXZxaHXPADrge5+YQ+tE8EP1hLFpRxvjXutj
DNgC+GlNQW6A7D4m7EidQc73dzHW7NW093K6VzNzUMZB83JQ0+9GkL/lu7My
vssegOwbn4c/uSco8WctHCx7oraFPK8esVzjvn35ar6/cVb/JOuir8TXyBlY
X/Mz2If8E5sVu+AP9vTuqPZZzL152rSFP47rcK6+4C4xJ/AyORV+SS56dMwh
dqNOcm1jUuT7DvOArO7xTNbXwBV1D+50y2EMx8SYmQ/zDDyv2MV3YtdStS3f
tRr/H1C93+cJ1Tj28pgH/s3eYPsNwJJHV/cefLsa954dv71jrNGJcf0bVPtr
7FHsYg7FRuZQnoZPEHNuEt9zTTnh06tjT+IbY9Ijq2N5+Iy1ZvBBvILfbhvW
CwwRNw6rznswZ+LYCSGTR9TvnBTn4yvEmcXqmPTQkMFp8W1r/DZzBfA9fT14
K2ai/9bvwC65HzZi/ILtqBvIvncF+1IfkHPP17xezeylzH6kzEFZr+EwB8iR
9bvp9xXPei9N7nHL9yFkL+sj45zMUyVPyzx87ovM/b/5DuF5fbOzevv5J9cI
2Xw+8zyrNwk910cQQ2mD6JL+Gj0S//FN1hpYY3NNcB/rbrlXF4y3Jx8ckaM+
NZ4R7DLuQy/EKPTWWAAOJlYvVuvwUfG8yOIV55gL5Znk8+i/e425vnktxi4/
ZAz2P4D3Yhd+wJgR3Tfufmg1B/vdmMNXV/vZT1fXfQ6r1u2Hh/y54XzOAZvN
S58a1zyrui7wN3GONX4w5aBq/Pn16hzau6prAcdUx3SnVPcvsV7WvuGN7kUi
bnYv0jHVf2OCe6oDPJfxAv5F/eSZ1E/myfiL+VuuxijxKr+/QciLcQ73ct3x
rcZx5BDsvYHjyevwxcYj6Lz+C302vgO7smfPXMG66nVfqF5r5My35DtAjN/N
vdhznn1QWbvPHNSOmt1jn/Fd7gPKXsrcT519YvnOluxlzf7wrHUml3t7zY43
14Jj2VsrZjKMJ4Vs3JT5w3UxHtZI/4IPcv8dPsv1goOIP/Qu2ufPemsL6Ik5
W3TM9QInjHFYdzntrWMO4bPZ4y13Ah/E+TNi/Fxff4oO60OxXX0036urx8Uz
gr3qKraprp4xjAkbvFE1F8Wv2htz+2rfyvXMUVwc8/Po6pwq64PtgWN/U81j
OU9ffHjIDw75g3H+3arxkPWyjpnvVcAv6Ot/IcbA/fXj2ITcGD0zH3uDkM8L
mZyiHBi9to8UrFY/mY/tw/xzT/04Yze+4JmMC3gOudZJsY4nxzomvzox1pR1
vGiQwWbzY8yhvungalw6oDonBtfST5GHNydAXks/tam6hsU8u3djQzWvXl/j
9woaI2Nf5gORjU2wQXOVk+r4YlLz35eVOSjr9Rzz4rt5cdmTQ15LjS/zS9lv
n1iaNcfsiZrXF5Hvv5oVVzIkucokxs8cbhtk5tz6Gmuk/rCmchXWRN1Yrt7T
BP8xP4ZOmKcFvzKHbx3q8JDxCYu1unass3z7jXF9fKN6CFaot/Aa82Mnh5z+
N2OKE2P+Od8Y+bRYC/BG/k8cKlZzX/0sei12EVPKu+5dXVtk/fW/T6num4WP
y1ffVo0nnOee3MNDXq7OMYIV+a4Ved37417EBOZP8D/2luNP7jg8S77/imdS
T8iz5vuyxB/WVh27tNqPw+vtfWKc26tzlXIq7q++gWPaC/OhvfAc6htzI16d
EWt6WrU/5dntGWBNrZsfW13fgZPKn8Fw6wXXj3nYv9of7TusKXO1JeZwc43f
W6hf2Fit5xuqY/B11RiFrO/G1nZWY5S9mpMav1dt1j4gjsxBZS93cpWM76yT
cmT97hE15k7X1Sc23cvqeDLnn5ww49bcv5PYu6OaH1pTY3i+bwfZvU7M2/b6
8b445txaALG83IxchBwe36e+/VrIr435+XR1PpDnN3/IGKxDZTzIOOTGcHLj
F3RZ2wcf5DznxHxi0+Jnxgspo1+uRZ6PLajnXFOuRd7GvBYYpf9lDMbLic9c
Q19MjkT+CXfQLz8jnoU6jLHkO6tx5t9DPiTke1S/H5V1u6ZXZ9I29cQa59mW
alUf0jaxRfkGsZp89b6xdjur/TI6Z64DTHvpIGNj4hvfmZt6ZPwW3DZni77L
mbmONTvmVp7PPOnfb1yda+JZl4d1PLs6pnPt0GewS33g2fU7x1Tb2pE1jhGy
LmkOav/q+QS79AVglxgFdolR2Y+xsRqjNlTvafKdHh8Z5Nx3lnsVs9fRPqhp
+814ai3v9HtMyPne0Xn7gJI7OVcc2eeZvayZc8vaJXpjbJv5t6wR5DsZzCcw
1Nw7LFYzV9lvbN8vcy5ugxvGv/AgeSa+bPsgXxjnsw6+84e8ib1qxAvmgY35
Lh2eQf7MHJsbvDieF/s+brgW+COfIY4QT/h3eRe6mnilXwajlgcZPRe7wAx9
NBilLXMfseu2cS8wSuwib+Oach/9DhiwVP2+FP013NbeG+qo8jFwXEy7vLpu
e2B8f8fq2JNct7j04ZgTOJc2iC6L7digekieT3tHNtfH85i74HmtZYBpYg56
qi/7UsjojvbC2ORR2IcYxT2fOMiMRQ7Pc5knYf6M67EPber8WC/mRvti7eTG
Z8YzstZi1AmxRsfE/IBdYtSh1TYI3sijmDcxCuxSV8EuYz2wy/wG2GWshx0Z
622ojvWIX/KdaWIU9pgYpS0g22OAbA5qUvP/vsy8d9FoF1U//r4suZPzw6F+
cky/d3TnIOce6uxFzx6M7EvPfvvkZhkLZ8+n3Iz5sWeAObS+QE1ETGNd5Gng
lbHnDeJ79GpHrdrmQ6rj7pfHOLmHvIIYRu6KflnbQpe0lwNq/A7JxVrN+TIH
2j5zY5yI7hnr4ffUc/BKH40tG8/CU4zN0X9tgWuLXdis2IXui12ss9jF+rvW
PLv48JjqHAj8NDFkaZir/1Ptsz5QjTmXxzNuje/Pj/Op1d9yuA582dj5KzEG
e6UY54Oq9fOZ8bzkMtVnri/PQW/kP8QQTxlkbFf8YS3FtDfH938Y54NPYhS4
KEZxf+sjjEtORe7QGJDn0NfwfGLsLWJdwGx14GeqY72zYx6IH/UjJ8Z8Hlvt
F46qzkdlfx04ZM4cPTQ3y7pY1yPfZY0V7LJfcVN1fAR2YRfuozfWw+7EKLAr
9y0mRhlTZw5nUuN3PWV8l/262WM/zZ3sX1XfOLLH3pwDR+4DWqzuwfCdHlXj
XvQ8J/tdrXdXdWzFIxgvW3ew59P6XfJS5tDe6dy3zlpYMwUrrE0shgxXkdPe
tHqNbhbjYf3FZ9bdfMWGGvcT5t5Dv99QncOB74kPxJLaHdhl/YI40j4c8Er+
gP/VNvGlrilrJb6BMfogdN+1xqa0Ee4pdoEDYhe2rk/HXuENYDscRP/+imoc
Iz8tzjB/xpXMmfi2V3x/RpzPWNxDBC/VNl8XY2AdzNvsqvE7lu3r2BXzgH7Y
s0RuQB8Nf7YOi96JY9i5+ZB3hvyKOB/bMpfOd2IUa2Jscv8av4PLtbgg1vcu
1ZyKZzXuu3XMP+tof865Mc9wsKVaxXbmUM5wSnU8DlfXNtEfbfOI6v0L2Jm+
Nd9Rgx7KGfapjn1YO/e5M+dyg+QDG6pjPbBLjMq9/NiCdf9JjfcqGstPahzf
5R7kzDsld8o91Lv7PmT5JIeYz5G96Il1ec18v411cMZv/DWJ6y9U91evq35X
/IbqWipzK6btU83T4MnGntxHPDkhvmdcxoCc4zrCpd2Hy30z/s1YO2um2SNt
byd+SixibPIBnk+7QL/Ew1PiHHTTuAMuoU+/bXW+Ebtw3cEo7Zr/Gm8+MMaD
TxO78HX6r2dU+3r4u/EjOWxxDJ0VZ8AKMc1a47eG5/X7G8T52Km2CWdUJ3+7
uo9xe7Vdk6PV3v+2GgdeU805nxNzC65rC8yVOJP5+a/GPL835L+MOX9hzDlc
TIx6TLXv4Dv9FGNZHuYcvmyO8ZeqbfCCGtd3nOc7xFwxP9rmzatrDdiouoc+
mJsCx4z7+K+5KfiC8c5idW4K25IPHFS97wBfrC3AwXwHC3HKO2oVM/eszpnn
Puvcvw92mTfGXvQRCzV+F4pzNYm1RjZXWTV/f6IcoGr+/sTp3jD7V+27qxrv
o8le9NxTMwvTGKeYhiymIYtpPK+YxpzYQ5h5ws3VObStMbZDquNxMEFMO6Ea
0/BTrteh1XHo5uq650KN6wX2E3JkbjD3VtvTzjjtvwWvzJkwr64vz2qtHD2U
M+NXjUfgKfJn9F3sAofELniWMSP+Xxvnd2LXE6qxi7yN2EWeSm5DXGyPxOuq
7QtOIlf8aDWmMffZx+v3zKe4dE5ch+/kGGCtfXSPjLFtizE/NZ7lsdV/7+l/
VPsL9D05qjYC9xbHsHmx6IPxPTlRedrLY26pWYlRv12NUYzBuI+5Na7nXDGW
59Ie7xPPe2HMFXMvp2K+l2oVH3I/BesupyKu1F74d/0jNr04rMmZ1Zzq5Opc
I/ZtHwjn6qOxJ/MY2JO9SeTB1HP3wPJc2IV7XphbYz2wS4xaH3OYvUbYUf7t
DPk2srlN5Hnvb7GeVTV+/0zuLZXPcGj7VeM91Jmrz3qia8Shj+bId0yZ30CW
7/FcrhHP6xoxJ/b/MFf6FGJAfMqHh7Ebb4K74lj27ZwV42d9XS+u9/fVe4fd
p8P8m99gnIm9+b4a/R1jdk8vvsn9BeiA6w4GWyvBb7hPnzmw5sU82XvGXIpd
zLd2h+0aP/JfYyU4hTYF/mlTYJQ4gD2KDy+JtYNTyQfgNnKwf4w1Zd7FtK+G
fHXI+8f5x8d1zo3r3zHuy1jkJPeJcd4nxv+L8VxwE7kWsbu4jS5nj7E49pWY
qw+F/PY4/9VxHeZEjHpOtS9gfeSxzK0x+KNiPh8ez8j9zctxPf37cnWeilyX
ORnmQB27oDpPxbyaN2be5BjoiXkq7Mn9aOdW52NvWP1uCvJj+ugTqt/NeHR1
3yxzaD0Lm7LGStzhnjLiR2O97CPaUOM9QfLnddV8ZqF+fG+dvCW5U+bGc9+0
NYKq8bsd/i1+azzFke+fyX00mTdTrzjMPXLkfvDEW30xzyL28oxL1e91EdPA
efNLYIvryDyLY2CzMeZ51diFfzPPf0E1d0KP5L3Mn/wn+/RYF3vjWRdrkRur
/SbrKGbCt82zgVHmAbi+/UL4FrGLMcjBsGNzwnAndQPuZIyDf5cb4Pe1O/BV
W8YHLg9r8QexFtQQ9DV/Huvy+moMISaQg32g2gY/HWt3eY3zTsp7hXxYNY86
Oa5Pzlkbx9as9WOD4hU6Ll7dLJ4Fv5N7YXxebEo95N/En6+E/NFqbvYPcf7f
xHX+PK7PXJknzDl8Roz5t+O58DPqDOuibrNuxs5gmjEgfMQY8EHVtR7GYgy4
XO0Tub96zv21a9ZKuwbf5Bvgm/EF9zR2wC6s92FP1qewHeO+o6p7IMHCbSsf
9JBYW46a+4OwEf0UNmKdBezKvT8+C7LcCRywZoecMV3W6XyWqnG/gXnpqs6/
ceyo7puSA3Bkb3z2oGafvPjGP+m/MrZl/Ooqz6gOEDsbUzA/cmnww7UmthLH
wAE5Mz7QWPKp1TEjuQvfX4EO21tIj4M5jQfGPHNNeNPHh/kWZ7ivtgmXcP5P
r/4b9+iGe8/RWbGR5xP3mAN7gZgn85bouT1j8G65Frk048RnxNzCnbRB8lHa
Hf0d2t2rqvMt1P7ELvig2EV+1vr7h2MteCZx7PKQfxBy7jW4Xvz2yLjmCXGv
02MMZ1XzFuxVvDolxg9G6ROvH8+IvZjT437iz1dC/mTI/xTn/11c8/VTc+UY
mEP1k7kVb58XzwsH0WZZQ30ZOTF9MXqoLbO2xkT4JfUNe7JHl9yDOZNHVueW
0VNyy9gqdmZdHr7jPkr8mDEFz2pMwRwbUzBe632My5o4sbzvNziu2i7gBsvD
/CSXxjaXhufaVJ3/yfdgwEPMO2UvvXW3Dw6yvALZWlvVuA9qHi7ZD8Dh+wc4
sjf1WSHn3kB5Av/3ySGLbwvV+LauGt821Hgfunq1b7U+g0NiBVgljqFT5iov
iblCb8znP7kaq+FE9khY00IP8VvqxoXVeH5qPC9cydwperVYq/iHzhhH44us
XRI7mB8gprC3n3kyhwn+PHO4PvGiMSl4Zj0LezG3/JJqTg7+WcOCJ2ib4PNy
reoY+TZt8G0xn/ANceODMQ88v7aJbxZz/jPkK6r7EBbi+y3x2wPjmodX85Oj
477HxniYe/kMOq4fZw3Fk33iudAfMQcdFce+GvJn4pwPxW/fE9d8W8zPX8d4
XhNjfmU8I7kv9ZC1MC5G98yHsF7qJ/W0pVq1L3Kc4hWyeMVai1d8L169oFon
uY5xxAur8Yp7iVfknMQrvjc+gje594Q41xzII6o5FetmboqYVY5xcjzv4fEs
YJc5KDi28d0eMc71Na6jZT+A9fpJNQ9Ezn2IYixH4pK1Sw5zLxzbq2M9czjT
39tLXNU5ukmN3yNt/x7j1DbXVedwwCvtdHON3wskzsNtzFEQT6nPzLc6gy0Y
L7PervWO6jgRbmWOizUS04gHxX9sWfwBZ+R1jMc8G1zaOsWXq/nYpdV49a6Y
Z3IG4hXf2V9HbGJ/Hfsczcnj9x8XsvOALF7xW3Gea6pj2KB2ypprm+i3mABG
yR/AXDEEG7evHl0Uc75WjUVXhMz4lDfF+fvGdQ6I6x8c9yUmuCi+d2zYohiy
tcYxiM/C8y8P8ndDvjzky+L8j1fnoy6N67+nmn++I8b55ngWcEzeiA/XftEl
YwF8hHgF1rknne+XatXG4TXiFdcRr7i+eMU6ildgjPEgOmnt3veGgQMfq44j
0Ev7FsAm4yae+e3DdzyPeshvzaWTuzLn8KuxvjyH8Sy+W4w6LMa2X/W7O7Bf
a5Qba/y3frJHSLsBH+QPk+p3OCAnLuUeH/vDOeyv4DA/zJH7Ad9Q3TuR13Tf
Fvcyb4Ps+yIY5yuq/aNYx3OJdfhosQ78MM9DLki+Cv+yvoYPM75mzrP329iZ
+RZb0El7VMAq9+BcHvJt45zjq2NteJx1GXic+UD00FokWGI8iJ+ynsjvxPwf
xTxznjoDVojbYKX8Cl0Ur95fjVdgq3gF5opXnCPHYKzL1RxDO0VXxQfGLYag
89osfk6O8Y2Qr6jGoqtDXh/nbK7xewa85n5xL3z0RfG948l3T20OmeuLXVeF
/N1qLPrP+P6LIX9mzrP/S4zn/TFm1l6bxe+oV+C/Oasd1XjF9+IV58uvuM5S
reIVupL9/+avrAl9Yfiot8y5NXTm2fwVc6WfBdvEB2q24sMTq3nL+6q5Cr/T
HsFH46DzY37eUY23xEPWAogtxKhTYgyH1LhHiOdBp6hDiZlwEuuS2P5bhvkB
E3Jfj7iEnFzI+jtH9s/nOxyeELI2WDV+V4PYWNW9HPzEGsRkamzGR+ietgyH
NPakDug+Vmxc3LtRdW4QO95eq/pJjvPFw3OBY+aasFP50q9Xvy+O57ymJ38y
/jsd18iTrp++Kc4HV+VpYIO9YV+rzrcTA6ob3Mtc2Uur6y/Yh/E1PkueiZ2a
J+SZzV8x7+avuKd4hQ2KV5+qxiv0+wEhLw9zhQ5pv+ZzkdFbOcaXq+0X2xdn
vhXy90O+KuSFkDeGvGdcc6+Qt1Tj1V41fo+9MtfRjiYh/zDk74X8jZD/PeTP
xzU/G/f9ZIwT/y+nYj2NAc1xf3+QxSvOEa/AQPNXxDnmr1ivW1b33Nr/w++N
BXyvAvkf/Ko9SPho330BVtjzQ65VH/oH1Tr5keqY6zbVPvSM6ncUMGZzqsyN
udBnxpitVzIebN04hfyqvO7W1XvTjq/mTgdW55pYX2tPaePYvhwPnTGHNqnZ
7z1A1o9zPDbk7J83/qoa5+d911DV+L0NuY/b2Bn5g8NcyQO/MozZWHWPajzE
Zq1lwH/dY0jNUGzcVo0J51a/A3mxxn8PwrXbr8b7HxOXEq+yx8Pvr6zx3mc5
1UnVtePbxbOT15LT4jPtDcb3yRt5JvN+v1Fdi1mqXgt00/og9iuHZDzqGDZg
/ornE6/QM/kVc7EcsnjFeqRda8vMo7bM9cWW74SMDc/Cq0nI6+P8jSFvmpIv
mpLFKMezLmRuoLwrxn9FnPOtqWdR/ve41xdrzCflVKyneIWu3yNk8QobFa/s
P8AG0RHxirU3z8ma2c+AbtvbQ+7ivOp9mfb2cE9jQ/yVvUnEQOIGmGmOCN0W
Qx5ezbX2r85pwM3NazGfxoz8Xjxk3nz/zFHVtaQ7xPXfVa3z+Ex1nufQHvnt
G4Y5YWziEjosr4Br6a8Zj3ET+KC/RpceFXLuic7+BPW5atw/b86HI3shnJOq
7qfl+vLSSXWueyHman3MYe5p2jfkw6t9xLnx/ePit3vGvRZjnPQpuH8KnBaL
vlPjvz3n91eFvGfI+CmxC6wS94hrzRssVPeg8qzmAL84tS7yqBNjXVhb+1FZ
EzFqqRqj4OD25jE/ciru69qxTubbsRPxCsxZHmT0yfVF1t7hxNo1diVufKfa
xrFPbfzKkK8KmWdQnq7/Km+Yki+ako37lCcxth+F/MOQvx/yd+O3+Sxfi/uy
nuIq6yleoS/qPN+LV5wvXjGf5q/4XNN/OOl+BniFvdzosLEhnMVaITgn1wJn
bjLc/8XV8Rd44p7BzGmDl+AC3AY9lGsRm+hbyVGIXW+s1lViOPu1sAPzJOiL
GEie1tgEHNXef6WaD5Bne/PwXMyZcRyYR27nqhrnc5gT94zAT8zNstZyJHQm
e+rMwU5i7TjsmeHIfT3yw6rx3pzs6RJ7J4P87kHOPYP23+a+G2sHXGdTyPvG
OYfHb8kdiSEPjzF8JdbrIzHnd6nOT74zzrlljPmgGOeXQv5yNabdJO6Ff7Pn
akdckxyL/ggc0heA39ZDH1r97lY40W/EOdZJnx2/xc7kUTep5lHcR4zCf4lR
2LsxIOe51tibOWfsKnM+9wl5lr1fEfIPqrGF/z8Lr34UcupYYte6uM4seddw
vnKFfHWN8crvr5wz5u/G9b8dY8jYFky7ML4Xr8Bq8QrbMN/OGOxPw+7kWti4
XAv/Yv8/ftbeUfJh9lGTa1mqVewiZ0D+CFwEv8hrodfwFvNavkcR3wz2yrXg
dOo5eRTtFGxTn8EW7etG1TbFtbWFvwwZHNXXww3lG/B3czs8kzUjcmBypHOr
80iMy7wrdq0+Zw9A+ql1Mc/ogLE2umQ/2yTmrWr8robc12OerWq83xDZ3omj
p2S51mL1301WXh/ypvjtPnF98lfmG+nvkLsyb64d6+T74t5evb7G//gGMMHe
4FvFeh0T6/KS+H5bdb6L9bH2wRyb83xnjO0N1f0nf13d55y537tWx2vgkz2N
Xw/5ZSETi4tLt4/fnla91uiDPAp9EKOIp5arayji1Q9jPNineIVN3nuGnDiw
q9r2fxgyY52FV1Vj7PL8heuQd8X5xoCJV8pXzRjbNK5+P8ZwxZScuTjxio94
xfXNt9uzwbn4JrkW9i12oQv2eONf1AHsz/1N8Gixi54Cc9rgj7wL/DEnTz5Y
G0QH1Gf33nxquK+6zfrrQ8ESdfhfa5z3EMfACjnAN+Icri1nAA98/xs6ps7s
rO7F3V7tK28b5xwdc7t/NS9iDuVF6OqsvvGFGv/tHucB2XiZw9oEh/jAkft9
fA8Ph/gwWYO8EPL6uCZ+yvwenNZYHi5jbxV9BK7jhfFcb4jnJfYSk8lti9UH
VPuXK6vj1ldX1zXWxbqAy66171C7dLjP4vBbdEnM3BLzSU1AHwrWGTtwH22E
tcoa1nKtYsD2asx5eMh3CflGIR8bv2W84tLeIW+Me2W++uqQ0bNZcVZymJTF
CvFK+UchizPTeHVd8jSnSrz6Uc3Gq8TPXVPyvebImX/TprhX1jrFLmIxsQtd
EruwR7EL/c39R/Iu4htjxhdU6ycYYr6LmM98FzzH/gE4tb00jEW7OKla97i+
eXjq0eIVft+4gGc0Xri6xnlaZeZfvDogzj81rglGiA/0gGmP5GiXqvFW7AX3
zNcdEM/CfGatwfc9Mudikb3xn63x/p3kPxxyy6rxe64cQ9WYjylPrkX2twux
FoxNuwavxJyM/Zlv15ecj/rAnKhj9FzIMegxyD2t1jpZL+t0d6vOEz6purf2
xOp43/hdDmaMeYdq7geemJ8kR36j4fe3i2e/rLrGTfwndrF26j+8TTsiHy9W
wJ3FE+ZAzLlxyMeHfGjIW2t2P8CGkBdqdh47Ocx1ydMYNQ+vqq4br2qGPA+v
pnEy5YxPU57mgbNqBMzPdD8Ga0TcLXYdFmsHZhgz3rQax/ivvpKcuZhGXlMd
Rj/ENMYlpmG/co/EMXiU+a4bV9cQ8blyfvBAjALrxCL0eWYdfI4Mh5ODbYnr
kP8WE8BnYyj6K+3TxkbENL7TRq5fjbF7x/nYvrEbmCD2rqu2O3TV/ZLTuabc
Y5h7eZznefJkSnaNkDMmdX3Tf2X9COzV7phneSZzYy4O3TBHh10/vlZvRP3C
GijXhmMZE9GXwLqjF+4l/IVhzpiru1fvw7pnNV5xfTH8qTG3z6/ez4jf1PcR
u6tX6J251v2r9fCT1dj1qpiTbdX2shxzgg5os9xHbDk+5MNC3j/kLSHvEfL6
kDOnXTWbz8ySrxWj5uDVmuQ515l332l5Znx6LbUA8WpTyNkzdmCsy2LIxGfq
8FLI9wz5CSG/NuQrQn5IjfcriZNviu/RGW0KnTYeoY/P2MGaEPaLfpnr4N+1
cf0l54IJYlRysEfG+fABa2rYnv79K9X28qj4LXxETEM/xZyDqvfK7RPnMOdi
7IYa98O7Z4f1gnN9uLqWZ94JzmUMmD0M+d6tlM33Tqbk3DNonJv5f/TE/Bt4
ZS0Mv2aODr9m7QzMsLcNzmh+D/x53nA+9Qd7X5kL90mB2dZbwT/fNfqs6tri
X1Tzq3dV5y2ZjyOGucInZv+etRueS7xCn+TJ/7eaT3JNeSM1HXOzjFmcxyeK
XeibugrP13Z4dm0t97PYKy5eKW8JOXsGpvOcymuJy+bi0n8zXq0FuxKXqubn
/JXTV24OeWvM+UEhHx1y7oVcit/+UsiPiTmktq9fwN71y/Cs5eq+fe2I69hT
hC5a38c+Xj6c/+JqfUbf1Wc4P3gCPoBziVfiw92rc7bcS+zCRhZrlTu9K+5F
HsIeS3qm7GViDsQx4jxrkcdV4ye5He0r3we1ubqPkbX4THU/vL0BWX9HP61R
Ivveiarx377Jfc2PmiHzfx8d8mNCftwgc19rB+iM+IPtWOukl0AsAofdu4fO
2Et2XHUvBxzYPlJ0wZ40ruEehDfHc321+j0wB1T7DnuN0Yufr67zUq+zdgy/
MqcHRprnZyziFTJ4dflwjrEwsak1DvTNHOxydSzwc9U+lLhf7MKXil3waP07
OKq9oINpa8lXteW14NXcvHf9BHi1u9j1U7r+pOZzKuU9puZq1hzm3B4V1z8l
5uS8aixiHY278cnmEsmvmnsEO8Ql9ND6L/qu7+Y69uPx32cOMv9uXxPXofYD
n6dPHIwBOw6t7u05uhqj4PPWyOiRsfZHP/zOWrUFeoGM1/ap8TtA5APophyA
eMJ+6TOqe5bw7XKhQ+Ne+FN50Zbqfk7Wwr7N9dV9p+viGVnT7EU3buKwD5ZD
DKka/63V3O+c/ajPDFnMQWfc48MY5EXYjvtTsKlLhvnZEvNzvVgj6iH2wOPj
2EdHjoh8mPiPXtiP+tSYz1fGvL23us8FrNG/rK/OsYMJRw9zDm64jreu3kuI
XmW/H7oLXpHnWBrGgy8zNrxTda4VPJNr3bg6TgQLjaOPq86ZMB6xCwwXu+Ci
Yhf5Ae0re8vxX9ra+pCzryBrdteVX/qp4tVP6Trz8mP5jPnse4S8OeR9qrHo
gJCJacQl1kVconYkX4JTL9eq3rJW5lfxU8Yd1O+MNfCl+c4WOQA4l/tJtRdy
E9oC39lDTg5W7gFO7axV/GKPiP6Xf1dXc48qOGZv9t2qYzdy4fpQru9eYOzg
LbWqz3lfZPkA8+P7FrBZOdL1q3MyzDOYi72gq3IMMME9gKyd+Wf00/5G1vcV
1b511l5mjkvi+xeHnH31LwlZzJneP+h+uvUxBvTHfYj4O/nS1hj/IbFGzIn9
7Teq7p8n9hKjwC6wHEyH39n/Ro1DTsXamdPjGuYw8VHmrBiTOUPm1lrnQdX1
QbDNeBAutlSreMU6ilcnVOfhGb81hcXqHCzxr3rC9c3lgttiV+4pRs9z/4ty
8ofs1cyYaB5GrQmv5sn/HXi1hvvOG38+Y/ajbgw5+Wfm/baGjK2JUehI1j6W
68ff2bVUzU+I++VR/NceYDDJHAg518cPz01cYI8xftT9EfhnbYe4RLuAk8tJ
+Dc5DPumzS/9TnUNDh2U+/Esxn1bqvNFzIm+FTx3Hx9zot2BbeaOjo/7njz1
vftcuJfc7HrVfAN7990RrMv2YTysl71erKM92Kzv74WcnMrrcLw6ZLGFQ87D
T8wdIcNTvzfcy179dTFOxuN7DBinWITd7ahVe899Osy98R24bb0v5weOIy8l
zy1G3b86piZ+Na/+rOrcIPVZcOejw7iMAYm9rcvgM+xDZkz2XfD81m15dvgT
/iJroKy7OoBu5Pu+xCvk7FeRazEn4hXXzJqFXAtcmrUvZhqjlOfZ+E+EV7Wb
+POT5LvWMP6MczP+ncYucWlTyNiRGIUeZj1Ie8eHGusRM5rjxTfZRweOmY9C
P61lExe4rwG9MGeCLF6Rw5RfEc9tr+4Bdr8wGLCjVu0FO8bfYhess/qPj9UX
M377VNEZa1jMm7aGrH4yh9opumcuJXvmT6uu0xEnwrU+MzyvPIo42ncmYHNy
JPyvmMz8G8exRtsGmTEYI09i3pDlqBxyP463hiy2cMjl+O07QnYOuZd7AJHd
N814wAz7McQixixv3FLNd4h/xSX0RDw/tjrvd0bMG2ttbxWxlxi1XB334d+s
paIj5tXhldZnX13jv30srwb7zh3W4hPV79LPfDv/Jl4ZE4JX6Jm9hdiVeKU/
tqdavGI98l0uuWfqmj6iGr+/+l67Kf9UMGqe/JPktf6LsvHCLDn7OnKvYtZP
sw8EOxWv8B3LtYobYJp4lTkfeIj1JvRWvAIrtC++Nz9MLC9eMc7fGWTsxFwK
+mpcQ6znXhjywXAAcAAdMb4jptN2iAt3VvdpW1fiXtaPOLKmlrlra+LMiXEE
nMFYA53FpuBS4IqcgWuYq8EGxShsyzzSgdVYhI8Wz7mXcTRrpI+YXlNjjarx
e7TeH7LxF4d7wCfVf38U2dgW3ZALIVtfYAzm3Bib8e8e1TUC9MG8074x59ev
ziseWV2DIP6y9nFOrAt8x9jcfVjce7kao/Azxn1wdfNU+DfryMy3nIqYVT8F
914a1oV5sseMe9jbzHOYb/9Cjd+Rbv6K3xsP8hvjQc4Tr/iIV7uq8Yq1zH5I
sWheL/q0vFsx2v9f+avdkHddhzxrfyLyfebIvrsGeXmQmVtjQOZcvOI34hXr
J16xptojeipP2FmdU8JunjLIxIbaNbkWa0/kyc29MEbzKvhM88/4cTkG+CmO
YWu+ZwA7lUtwvLBmv1dK+wWr7Rf6xXgusFdMu7g6hw+WG08Re8kbbxe/hfe5
duC2tgCH0b+DFfp9MIQ8srwI27YvVAyZls2ncYgt/FbOM6nOHU1qvPfZ2uu6
avxhDq1v7lHNkcBb+z3wX/LP61XXasH13Ot3yHD+qdUcmHqrsR5YYt8sfkGM
Ym7d+0BfqHEf/ss6Inv9bjrcm7ydfbx/EXP4xpjbd8acg6dybMbhPo7PVPc2
8NxyKu5nrZB7iFFcV4zi97nHRJmPWPTDkKdxTD3Z7d6q3cWo/2bsmjX+XVPy
vH092du/q8Z7lMQr1ib3YOrrWQv5FWskP2HtjAfRJ3Ps2Ip7RfGd7ifdUV1n
J7/yxEEmf2P/M3poPQveIXbhAy+pfr/x9ur41HzX1pCxL3ka9rhtuGbmMZAz
Dspe2czhGx/9aXU9nZhIm+UZjdHIveiLyeH5/gpiZ+LGy4dxGr8wTvkbWCF/
YMzmZ6b5ofEvR76rIXtKv1D9twXtr5jEmCfV/Id7iTnuhXnfMCfyInDVPCF4
LkcCe93TR11MvsS/2ysF11qsVTxgHsyZn1Vd44MTGevdYZgHng1skEfhM8Wo
h8d80htv3EeuwLiP+FGMooZjXh1fZ78ocbM9DPbBXFHj9yx9qhqjWA8xijmT
9+b7qb4ZMvcQi3IP3a4aY5d2/VPDq/oJsGh3z98NvMpnnMaoXSHn3sl8Rw3r
J14xz+IVNixeodfL1e81lWuBS8Yy2In7Q8kP2X9ITsjYEH5k7p2auX1B8Hzf
6QHO2f+DXpjnQQetxZO3kBcdFTL+XZ5GzGLsuTGeBRt0PyxYYY4Fm7MuyTyI
veiseZX11XX2fUL+6zgHnT96uObTqvENjmBv9n5x/h7VMea66l7ThWpOMolz
OK55x131Pm6OfFdDxsLZPy8OT6bulWMQc9bHmJlD8WdTdey2JZ4XHBaLGJd8
iesRE/7b8O/i0qnV3Imanri9VI3n4IqxHthgnI7eyaPwh2IU/k6MArvEqBdV
YxQ9YmIU2CVGgV1iFPOY74gToz5VzaPsz7tyGJ/4k+9LyXcR+L4CuZbyrP2A
0zzk/8ve9Z/Wdebt2ck9j4nVzokYpcz6iVffCBlfJXbl+8Tg5do7einXQr/l
WnBssYvYyFoh+SexizhM7MKWxS7iRXkX1378IKNT4hj5beNHfK+9ENiKvY7Y
ohgIJhiLgS3yeWT1n/OxJXjInapjImIK39H3t9X5Ya5r/3xiFzUB61zEUMZc
4KgxGmM2Rtsvztmjxn//XfxcqI7LwBbjMg6xgmNxSnbvoRjCb91nNy2LLdzL
fBFjEGd4Rutx4JVcCLzy3WX4BfGHXJbxMr5DLAIXzY0zRvdWcw1x6axYl/Oq
uRN9B+6pIVZbqtW4GF2Wu+InxSh0yVgPfRGjiBnFKHi3sR4+VN3A78ixd1Rj
FOsqRjG/YhT6IhZ9qdru/qPa1r4V8vdqzB8y9pkVHyVejfI89ePyT4xRP618
+27i1a6QfxByvn8m352Vc8uci13on9iFfotdYJTYRa5I7KK3UewiTy52oRti
F3GeGAImmePiO+t3rL84ht7Zr0W8IKadETK2ZX6MuMM49JAYD7ikTsIR9MX4
c22He2qbXPvY4bn/oPodEYzNPTU8l7UtbG6xVnEDvDT3RbxrjwG2aW8Gdm39
bmN1zg2ssGcDPMn3wJsn5xATOE6dki8bZLGC354+JX9+kHP/dcaks97bAA6b
O9pUHaORy3JP9z7V+ANe+e6gfLcDc3nu8IysnXyJ8YpL/G6pVnHpFrFecCX7
plhPc+boo+84wjeaM2ctrPFR25E7kauXO22vxiVyFOIS8b1YBJfVdj5UbS/o
ivbyhRr7feVvhvy9kK8MOd/JkHg1D6MmM+SfKl79FLHLOcw6QubrdoX8g5Az
js53q3415M/HvT4dc/KhmE/sUxx7azWO0fO4XP0uDmNG+onMz8OJzHeBD+IY
/y6OoUf2bqGPYhp4JTc7K74/Kc6HIxiTHh3jgVOok3ABa9ZL1X4ZHmf96M7V
NaMjYpyXVPfYP6sau5gvbf8/Y37w58akXMe4lRjKfk5wwDgUrMi+Kd+pjkxN
wl533//AYe2e47yQxYHJlHz+DHmhuk+JMYgb60PeEPKmkMGrmw/yPtV5JGoT
xmsHhXxENRbhE8Qi1kW+BBaKS4xdXGJ95E53rc6Tw3fMk6NL9h6Q47KWx/qq
A8+s5kisqfoP11LP3xgy/sQ1/eeQPxXyF0L+j5C/HfL3Q8588tUhZ41sFi5d
G16tCbt+Sng1777T8r3q2rEr6w7TeOUafaPGcbfyZSF/In57acjvjPH8Tczb
q2ItyBWIY+iGfAzblY/Bg+Q//Lt8DF0T08AS8Qe9FTfOie9PjfNPjGseH/c9
McYJd9CH3irkh8SY0U+xkd+Z839wdZ0xa2dwKOMv5kBbhgdpI/ANcXXPahwD
E8zRgRv2obHWl1T3pb+8+v0GvteC4w4h33FK/s7w2ztdh7xQ4xql8vqQN8Q1
N4W8OcawT8j7h3xQyIeHfEyM+aS4F2ukv4CXiUtL1TEdvzOmgzfJn+9b4732
6vOjq3XgadW6+oLqdaeeos7QQ7c8yO8M+Z9D/nSc/6WQfdfxD6v/Xsyu4eN9
rw6ZtRCjpnuQLpohZ3/4mvbs7GY+fBZeXRs2zhtP9sGmXDWuh7pGu0K+osb1
C+0035l8Wcgfr659fCC+f0fIr49r/mnIL4jxPC2e99GxFg+OdeQ71xp9XB5k
/Kn4Bl7J2W4Y8ulxzsnx25PimieHfGacwzXFSfy1GEVMYa6M8dtTgWxPJueY
m8LOzHuDn+Z2iIPEri01fq+O40EPjWFZ09yb7Bg4cm/gWvoSlSdT8r1mfL8Q
8rq45vR7ZsztgFfGU+SvMjcotwGvzGODV7m3xXk4JuQT4pzT47c3jGueX82X
bhNju2uMn7GrY6ytOkZOfnmYc+ZVnYH/o0usL31c+rvX1zg3q/yhOP+zNf7b
oF7z23EvxuRa2y8k/ohXG2rcD5nyRTPkefsN14JX8zBqXu9BxqTJ62b1qKsz
Kd9rhpy9/amHV9f4/aLK34m1/o+QiWfUyU+EDF6pn+8I+XU1/lurnv+8kH8r
rv+b1Rj4KzGee8aY7xLPcuuYk/NiHc+JtT49ZHBJfQAz1BOwS12C84hRYJec
Bz8uVhC7iA+M4WXDb/neHnjGgK8lDrhB9R4fbPPQ6t4k+4iwcfM2rJ0ciXXX
3ic17he1RslhTwjHg+L7B8+Rja8na5AX4rfr4vrrq7kreGVcz7Nop3vFmPMd
rfBJ1xHczvcYqBvglfpzQsinxjn5PgTiYPVkqVof4FryGbiWa824xBnyB677
k+K5iNHl6i8P+a9rXO/2/I/Fdb4Yc/LNahy7ssbv51+ufr+oc7VnzFW+xy//
flbuocu9wBtqjFfXxWEy/sq89yx5V+0+XuU+mhxb7pecll3HjTXbVzIG152P
+sA8qyfkSbWdT4fse1uQd1T7u9dVc/KXh/zckJ8S5/9GXOeXQ/6luNedYzy3
jHGeG+M/u8bv6fJ5T4p5y/c9ktfVvpDVQ86RU2Evjx/WB6yzB4xz5FfYl32q
h1XvwaFmKNfau7qvaVN1/XFDjf+WvT0VrLu5HfQh4zjnh+Oa/EaN/+bX40J+
/Ax5ci2yv819QIzNWgZ6aP//xmrMzL9pBW+0/ut+h6uHORErqH0sD/NPvl3b
zHdGMTfq8ykhnxnPTs+DawrXEhPgWuLGXat7mLm2OPyQGCdzoJ96dsh/EvKb
4vwPVGM7vkc8Z83EsV01++9N5N4Q8Fy82lqz30G6Tzxvvi8r3+2Qtp+8ZYRX
kzFGKV81JV80JRurzsKuxKuMT5P7JcbmOxby/TC5H5zvtWWeURvnmrnXSUwA
U8QK4hZthNhQTk6ORq5ObkcO/4aQ4dLWZZ4b8raQHxHyA0K+R1znTiEvxX3h
P2LgmTHOk6uf5fh4dm2BeVusthd6AdRnbEe7I07RlsEY+RXc4EXVNuj+YnSM
Gt83h/l3zwvrBSZ9cphz+xxYX3nXQnUvATqQtTzraFXjdx1nnkos5XBPNMez
ZsiTa5H9LeOxb41xmk9j/O6ZwkaMT9FDe33BZ/GNGFBMg2dq+4dX4xhroS3D
S7Vf8Er7TZ58WpyTe+2ZJ3kOOX5xCbw3vkbfXV9wxRr0Y2Ocvx/yK0NmL49/
KwdOlfs1xLEfxn03V2MX3FLsQn/Eq0PjWQ6NZ+QcMSrfl5LvfdoS8qaajVcZ
7ydGZU5715R80ZS8q368vyJ9ZebWlHNv8ub47d5zngU9Ea+Yqwvje/Fq32q8
AscyBpH/gBFiFDYjbnyqGlvoQbSnBR9kXpQePPOl5KflCU8J+RHV+dUHhPwL
cc4d4jo3r65T3yjue3qMBy4kph0bz3VUdeyJvei/8PXqEjalbpN/0u6YQ/vt
t1bn25n/P4l1cV8z+pN5ePf1oEtiF5iQveXZz2lPKbIcjCP/9kTimLVFjpeG
bNya8uRaZPsuFuKa6OGLqnlCvgvLd0fgE8XMrTFXzKE4BubLzRZjnpPf8qzi
D+u4XB2/K58S54BdYlTurydOFJeoodgPg164plxDDkncL7d8bnxPbdacwPtj
/MR94thVcS/66MQufJ+8C7+pjp0U48+cKueok9RqtHfmTQxhPrVx7Fcbz3fU
5DuEK+SrQr4yrpM1uO/HvbIf7MoYT9Yxf1Sz8Srfn5Dvftkv5OvHvQ6NMWCb
mSsQr5gHeQi/lZ/AP8UoxiBGgRHWZbA9cYN8o9iCDzIPw14tewPQcerRcA/y
V/YJgFfWrO8f8s+HfLs4//zquvY5cX10WL5xXDVmLlbj2GHxjJk/QQfUGfBc
HcPu1ElwyT7VtE305JJa1QX4hvsZses3VdfU3OMzvaeYGPtTq5e6Zu8Msnvu
qsZ97NYcOczhc7y2Ok70XTEcrwvZd1VNrkX2/IXqd3AxfjkkftP95jz7K6r/
5suLq7mENU3mXG5G7Cx2MXYxhLWTk5CnMmbMmkjmHsW0qwbZ9eI62d8itty0
mifj97YNMlzGHmMwyT0Ufxjf0zNi7/G/xXW4t7iHv1NP8Jvi8E3juZaqcRU5
a0Y+V7773XdjMs/Mm7bMvcQcbF8d3hxy5nym89Wzam3Z15Q9mcq7arz/JbEr
a5pZL9gUcuYwrx+/PSLkY2MMJ8bYsp6ObG2FdZdT4dvlJ3BRMSpr2WBE7mcX
N6iHiTPYgb00L66OZaipmZ9BZ+3PQQetnV0Q8m3jnPPit2fGNU+Mex1TjWlg
tZzt4HgWnkvsync/7lvt+zIPgz7Yp8paqM/Yrz1UYJR5LebKd1Kxjm+qzqXb
Cwo++H5RZPNdyDur+9LNfXHkXhvzYBx/Nzw7x47qmPHvq/d0y/cmU2N455Ts
+2S8Ds9inMvz2veVfBJsF9OwqZcPc0h85ztz4Ibu+6b/U3uHC8mL8EHLw2/B
rsw9qtusr/a+WI0J6LwYCG8RG7mmMSx+zziXaxsLPy5k+Kb+iLjd94yhK7mH
wv4T9F3dQI+MGS+O50IWr+4d44cXGAPeJp4X7qxOgttiDpwt+2GUN4e8UD/+
Xgjk78V1vl3j3gCx4hs17iF3zr8dMtcRf3bF+Cc1fpeL8n4hH1bjHLLXOWPq
2R3PzeJZlqrxCjwwF8T5chLmSq6CHopR6LB8CV0XK+jnFk/eWo05+GF8CRwD
XbCnkXW21/HikC+ojnduE9/fJGR82XnDddDn84frH1WNb4fE2LB1cZU5NJbM
mnvmBPas9vVglH4TmzVeyHwOuGR8hGxchv6IXaypPe3I4glyvvvFd5NyGD9y
5L5m9yNziHsc8jcO36tT1e+H4V7/OiVfNsj5Dhl/y1rbS79HjBNOle/oE9PA
EzkbMTtrjx6iO2I7MbrxF/zdPM9StU2dUa2TXFP/gs5rO/gg8Yq1NnZDB8TG
k2Nd0Mc/quYdcuMnhQyv9H0dX47fgr2JdWIXsa/6QNxsTox4WryCbxoPbqvm
VL8az0JuRgwBA7XTs6rzNkfXj/+tPftJ/P5HIV8R17FnCTn3YmM/6jx2dVF8
79iy5zn3FF9R494MZcazXP2ubOXFOOf0GtdKvP4dYwy/UK0PF8XzMi5jQJ7V
fu87Vsd6+ER5CzgvDmCP4gM6LS6xj0RseU31XgzW3z0a6Kz7OH65eq/HBSHf
OuSfqd4Dgv+1poZv9Z23cCqx7qAYT/ZXw6OMMcEoY9s9a/z3rF3HrN2DUeZa
seuso4ldCzV+L6i5IOR8v7H8hOPt1RzJOiNH7rvJd8XAxT4xyIlpn63++870
3cvTwKSPDPe1r1UZnFoIed1w/r8Oz+t19ox7bY3xHB7jZH18F+svVr+zFLsW
E+AwxlbZq4ltyeG5jjpG7KB/RP9dI9bRWgnjMXYj7jDHiI3L935mmP8fDPfS
j8Ch/vcgw199pwfXce0eGtfEFxvnok9yLc4zNuR7sZS5kbe/Op6XGo648ZBq
nIEvaJvosjnYzMdeL+QNIV9ZzUO+Fde8vBq7sBlx4AsxhtzPgixefSnGzHXE
HK6/PMj8m7xxY8gHxLPfIL6/cfz2znHN+8V9HxFje3I8C7okXpEPl3vAr+VU
jBfsQH+WqjEKXybPUSfACvIy4gkxhHvQ0Fn3rzEG97ihd+6Dw5e6Z/ZWId+o
+m/8nVy9Vw6dtKZ2aNzrgBjD1mocgxuA6e4f0S6yT3tjzEm+wxZblnddW2/n
b4Usr5jU+P3qf1ZdZ8m8kz1dHBnT5bv4kjsl18I/+E4G919zuLdxOrd/8PDb
hZDXhbwhzmeufBfE9eK+x1djHb7O2PaS6v2P+Bz3TmK/8it0WH8BbrherJW9
atzT9WUMt6zVWBWfkhzYGi7r654meJecFr3QL7CextFwrrfUqv3x//Uj+Chx
DP023gdb6HtgTcnJm9sEhzMeNAYEb+USn63WJbBTXALz5JBgiZiMv1UPjw15
a8gV538nrsO8i2PZS8kYxK5Px3iwWfGK78WNz8b4uc5y/XhPLPNlXnEh1iVr
oyeGvBTn3Lsa09ALseuFMYa/ijG/PZ4XWbyCGxtDPa86X/Swat6C35M7gRXy
H3yhuIROi0WvrM4Vo5/uKYYb+06Aewwyen+r6lr/OSGfGOcfFdc5OK6f79Mj
7rO+Rs5cjmffJvoIRt1ieEbs9PbDsxMTWXcwn2muwPWaVOe7kM1pID8lZOJh
ewystXFsr67l6fc5koO9rfr9xnIYDviN77mC+/reBnBcfuW8cfi+hUnM20LM
27qQN9T43Quef1CN1wWbBEPJS/nOCnBm5zAGxicHg3vqg95c/beteTbfgYM/
FAPBH2uprIU1U8ac+wLEE3DMHN31quNc9NF3NV9cHRczBt+/+jPxW8aq7/ho
rAvP79rhq+V12aO1R4wNG1Rn0EnxhDkyV4xeqGP8zpzM+SEvhrxXnP+DuA42
KY5dFvfSrpDh2do+ayZesT7iFeslp/rXeBauo85fVo05zJf4zL8Z42cP3kkh
3yHkh8X5l8Q131tjLuc4M4eDrpmzQh+Mm9BX81H4FLkKnEKMgn+dNjzH1mr/
uLPaRvB15COIJZ4zrAHzB/fD1j48zKX6zzW1x7NDPj7OOaLaTtF37TTfv0Tc
p54T14hj+X48bFPszb2962IemBPxHFn/OKlxL7p1K2TzwBzmgTmeHvJz4xxr
bRzGJlW995nD95dWzHPF/Fc8F4e8peIZuTVY7fsf9EHraryv2Th6r5APjXPw
Kcb+2L5+Cv1wjd5d/R4e7NsaKDYnZyOvaN8a+rh9mE/WyLok4xRzOLQpvve9
0OuqeSYxIxzrAzV+9/K26lqGHAKdhE+Z0/uran7IuhEHYjfYrHk54kVzAujz
8jBmnjPxQWxh/tQfcM/4l7qE3H455JvEOcy5XDT1EE5oDueyuBfzJHZljzfP
JNdiXYwNwWnx6h9ibv8xnov1kQthY2IU6y3+gO/GwtiaGI792YN315C3xTnv
qLYduIR87OdjPGC7vPGPY07gBcZK4KE8JPux8fnaCNilL2aexQrufdSwbqyj
mENOWl3itwcN/86c6WfBK+tiZ4d8XJxzWLUeHhDX3Le6jrZXdV/Tpur3w+R7
nMAo49PpnnOfHdl4BBvJ/nM56qTG7/1wTTny70QYP3I8NeTs+XxBNQezP4pD
+61qzOEQQ6rG70w298iRe5z11zl+nl19Bq/M4zGH6gzzKj6jL8uDfHi1zhNj
EcODC9ia68WzGasy//Ix8HKxVvWEa+gHWRf5MEf2yuZc+T1rpJ3CZ6wRY9/i
KnogP/9fMR64g7k++JSxLfZnDI4+WXe4S3Xd+e7V+oA9GZvsXZ1/wxfr+7iW
9fTnx/f3CvnMWDv8r/1CV8RzZX/RR2M82JLYRV7B9QV3xVL4o9zvjdVrDVd0
rd9SzanAEzGEtTR/yHxZX2DNxCI+6jz2Zz0CvTPX9+z4/ks1freAOMb38q77
VfNDcsbqM3OgP2VuzWnD+fWh4IcYRSwpDsDx5EvggX1E6Jx6S7+P72B5anXv
9/2q+f9tq9/fe1bIx8Y5h1T7SvTT9/eiJ/rxfGev7/DkOhuqudn6aozFZuUt
6L+xJHad/ER8QFY/EweqxvuRl6sxx/XlmMfBnhZyvlv+j+K3rheH+smhD+LI
HsLkgWJpxrM8r/rDnNh7QO7IXM3BMbZbx9i4n7VRYmI5z3ur14XfiFH5XmV6
puDg8A5swrwT55gn57DewWEMyLGzVu2GMdtLtlida3p1PGO+awsb14c+sJoH
klNizcEp4hTjU2JB+RscxhiZfIu4mnVtuL053p+P+/Ib86hg5s0HGd3R7ohF
9Dusi/jGGMUxxiBWY0ti19uq9ZC8ghgOT1c3wGT1kzkWr5DNWTFv4hWcU50B
0+RRcLNfG+aZ+TSPx/P4Pjr4s7rE2OWl5JrMZ26K78FG8yrMu7ni3GuJXuc7
Y+UVYKw2y/yLUeisNo4OHzGsKc8Cjog/+q99q7HlfdX6AJfeWc0xrX+BAdak
zgr5mDgHHJLP44P0j1vi+pvivui/+oZui59gFLwBHEM3FofrIGfOx1gS2fom
sjrJIS/lSDyZtwcwOVjao+vLsS3kfD9DXkeuzpH7oJPX2UvJz58Ysn3pPK+8
hfmR4zGH9sPjF8zn4Lvsy8LvmAvCHzr/2Khcl+cTo8AIcQz/a14d32pOhnXM
/eDGmIzZfDhrl++xcU3vWt3jwbiIo787zIf+lHXVn2IT6urXYmyssTqDjur7
sLXFWtUT8nDaxb/Es4BLZw1jwOfL2/+hmgNg0+ZYzqvxu/3FNPRJHDO3+41h
rGLXm2IeqG8YM5JX0JfhT9RDcg9yaexdvEIWrzhHvWL9zTWRixb/X1etYzuq
MQqbFH+4rvoGbzSvi96pe+iVfhCfZs2duTRPyN5zff3dYh54RnMgfxdrkTmo
LbHu6Lix2JHV3OYNw/qAA2D/zmHNWQP96Suq3zn80OocKXOvvp0Z8mI1P0c3
zTmAh+Y3Nsf14VRiWr6ran01TzPvgY/CLrKOpv+dVP+N+0mN+8xzv4wcjEO/
ySEH41gLB0vMkfNw6I84EoseFd8nT/vtkDOGsp+Tf5KrIOc7A19X/d4bc/5b
qrkTc2NvA/5fv/Cyaj350+qaMnGJnOdD1etojut9wzypV6yDfJg1/cxwzT2r
eRR2LQ6DB+ow+rdYq7qFHbjWYKd8HpwRP1l/uV/y8wvjnN+szl3sXa0b+HZy
p+gGeKCPg4+dNIwbzLeuTZ+FOsP6Gz9wDeNodEKehh7Lzd4d8/mGeN5XVusY
uR19JT7HHBdcWK6FLohX+EDxClm8ou/MWIy+WnWS/g45D5hmfIcPMddBzCUn
h2uoh/tU97MtV/8t9W/W+O8Uv6j6XT3qJ+vmPrinVPuLc2L+wTlzUOibPg6d
FKMWY01fX62351VjFHxS3YZHo4PoEDH1Pw7rgt5Zo2G+7bs+I2R02RrZAdV/
qwv9ef/wTHtW91Jia9aDwCjzEutq/HessjeJ8bu/T72dxPNOalyPMx/CYd8F
h3kYDvkqBzxfDianreo8A4f6w6HOcOjXODL3lbwrc/gvivPdR8ORvaz5/lJr
/cyJOef11X5hU3VfFjxfGwKf5NvYjTwh+6xYW+NH9E4MxG/ra86r7gE7ppoz
H1LdTwvm2IdwVjwj+i52wX/kG9iW2MJ11dVJjD9za/i1WXErY5S/XVQdR7yq
mkOqo9zn56p9HGMXxzLfCz6JY8yzOMb/l48xDjkYOC12gXvyLp7dmBEskpdi
v3KtbdXcFZ9mLgsfaN4ATiRe8b26R4xsDAjf0Vf+Tsz571f3pIEP9vaQK7M+
jp3pK8kXiUX4Nft8WDv3hXF9OTzjlcNzvnvZbhXr8ufVdop+HTys0WlxDmsj
Z35+XOcm1TznN6vfQw7WahdgrzUv8Nge6TtV+9DT4nv0TvuCQ1p33lKt/9jU
31fvO7YPc308L/aID//8IOe+GHFsUh3XIGe/k31NHMaSHNYoq8Z7ls37cRgL
cGSe3Nwsh7lTDvWKQ9vnyJy8/Y0c2d+VtUXzQhzkCMRMcWkyNSfG4GC7+UPm
Ux2A82iDrIV8YLHaLrA5cxrYk7rKs7qO2J++BnsVu06o/tsc+Djx6uwYM/OB
PoNde8XzwnPcD/Wz1frJumV/2jXypOWr4/t/DxnbEse4lzh2QbUtvCzmB50U
x/itdR9iDfnn1uo8GHNsv8e7q7HrL6ux68Uxz3BVY0awwXzXE6rjRHi3XAue
bmxIbG4ui/yTeIU/FK+IvcQrvhevfi3WlP+au+Bexm7wH3trwR/3lr6uGqPA
CfOTYI76wHzvGJ7l+tW4wb9rpx+rts1Pxfo+oLo297VqO31FrN1zq/n8J6rx
6plxfXwBXAp7hI9oUzy/tgZfsK/yDtX1fdbQGtmh1TVl1tp6GbZjXyIxoO+n
AqPENOxOO12o8d+jF8eQc9+K9sLxuep9x+iW+/iyV9N+Sw5zehzZB6VOchgL
cKiHHEs1+x0ymVfPWmH2nbrnuqr/3nRV2zhH7t/xGSc15pkZI8tPskdrY4yZ
+bcfhnVRz4mf1GHs1TVlPtTP46o59nHVXO74av05LcbJ3MjV8YPi2OaQn13t
d8gn6Gu4hvjDM6rDP6oxp/IcfJ8c+++q13pjXPPYkO8b58DRtR04pH7/iOo4
hXONZd5djV3wX/WEuFLsohfG3D57BeTtxGTGieCJdUbqCNYTWRNzEcvVuSzG
bG/DfWr8vl/XET5p7ghZvOIcY0B+Z26Kuon5B/itdaJttWr73As7VgfQO2Mo
Ygt79vDzcp77Vevnk6vjuH1izg+vsd/Rp1w+fPh+/2rdRofEru/ENZ9XnTfG
X8gbwRbzLU+Kc/DFxrysofEOeGkOhHHqr4kBtYWN1ZgGXpmbxe7yXQTiGHqr
XU9q/PegjVs5zLNV9TsWOIxfqn58n3K+b0EOtljNwdRPjuyhyjq+/J/DvQYc
8oeqMS5p+xy++5Qj9/UY002POXN3jjNrDdnjAXbpu+Fd6jk2bt4VjqHegmP2
lYEN8nwxHr05rPpveWDHYs4p1fO8VO0TsRtzU5+v1tvPV+ea4Dzm1Y+IZzyh
GouSR6FLrt1jqvX8t2KuHhDj2Vyd9yAWtJ74zBjbadU28vUYG7kNfcGfVWMX
sZW64X4Q7gEu6ePgO8aJjEeuBeaY14I3GRu65w58gFuZf+C/5lHvVuO/kyJe
3aWaX/1sNV7dqRqvkJ8asnbKb+UkxAryCvJh7jsgjyVG8THnDM6Zb/xkrMvZ
IX+iZsf1WY/O2B8d8HtiXrkxGC8HBlvMafDv5v2+EPMDR7XHg3PN1RxdndOj
b9m6A7YgnoN7Yhq2k3uQzW9ga/A8a/riGHK+X8X4gsO4ksM8M0fyE+bV3Hvu
49OmOJKDqatV7Wc5koNlL2j2f6qfHOIqR+59zr5K7d3xyAnz3TWHxfeZi8te
DveVI2fvojVK5tmcBvOvr2Fd5MzYtXkJ1s653Vo9n+iRvuCQ6jk8NuaQmoYY
h+6Yn8dXG5+SH7ZOd8OYw0fGb+Ea4s8Dq/+G9WvjnJtX86LPVcfFnC+X43s5
+aXVnBD9Vx++E/P/9/EsL4t7wZ3Uh0dV6wM4Yd5+ufpvsd2z2q/BlbQvcGap
VnERbDEPf6dqDgzn0gbBM/HqdrGmjN/6IPhnvv0WsdbohfEgXFt7vFH1uy+I
ZY0BGa+cAT8s9/m5WHdyVmLR0dW9UuibORauow+6fo1zkmJR1Wz+/M3hfORX
VuMVvkXf8cdxL+bAHA5xjHH3PattAf0yvj4s5hDdtgcA/c/3J1gjg1MZg/CM
vxty7kfWjpCtOyC/ujrnI1/lyH0xxpIcxpIc+Y6FaQ72kUFOPpNz6/xzHBXn
a4Mc+lnvpZz30r9z2FPEkbFqXj9rnfZpcBhfMCfmbJFdC2R7MJjbbbX6Jdgl
72UtjFtZI33BpmpOxTpq4/i1zw/jxzeJq+iA+6uxXXGVPI24yprLRbfGmNFN
a/Hk/+Wl+GxzdHAZMYTriBvol3HZjuoYivsYgxOvy28vru4p5Tw5ObojdhHf
mU94Yjwj/hlsRAfuF2t0j2quhV3b23zHWC9wxv5hOLh5+FtW50WXqu2LfzfH
yDpbKwR/7HnmGe9fq3h1k5hDbNe+ULieXPqU6lgJnZU/LFbbGmsoXh0ec3Vs
tS9gDGLFS6pzcaynfAZ9Us+vqPHf1LvGpiInuX7qe+ec9bJPcN9qn/WB6noK
PEbeu626pgae6y+Oq7YX9E1Oi1/WL6Dz1tE2xLNgL7kfWfyfVNsU8jNCdg9+
1bgX0fo+x7y9yTvqunNEGYvN42DX9FVWxywcuU/wmPjtzhpjoFiXcWhyp+Rv
iXvZg5H9Y5kry/0vzi2yNWtka9PMufE4ayHmozPiPOtlnmqP6niKWEzcyJ4B
sEub5TvtFK6hv2O87i9jPY1fmEvXGuy0F4t8KjrIeuyI+14Zc0VMJ29hLuUb
2JC1TuxYHwcuWEuCs+vLuL868Ih43uXh37ELsFBfD66ImbeJ8YAz4id2bX8X
Y5R7gy320ty4OpfFOhu/I4tXyO4vA4u0KWzRHmPuo32dHnOL79BnoZtyBuZq
2yCDE+oG/sgcOzULfZb1C7CC/NLSsKbnxDXRaespjNPYwXeyYC83i+t8txqX
vhXfY7tiF9c2J8Z66Pcvqravv6r2+/hq+fbZ1faITpoz2Vrdl7K5Gsfw1+ai
sQXj8YxTkPUL2JRxN3L2MmWfknE3xwura3DGkhxr3Zt8XRzMXGLVmIOZ96ga
17OcKw59it//0yAnd8oaZcaV2eOadcnc+6+ucmQva2JUvoNie3Wsbe8W80/8
Y5+2+bTsJ2HtzFGwpub6wC65JX5KjAK7xChsWozCjtzLhl1uG8aJ/7deA46a
M6G/whw+H3H++zFvcNKl4V7o5vJwHT7GPqyzPg7u5Lt0HlXdnwNP0X8xXu3i
9tU+6xbV9nVuzAO6r39nDuTDrKfckv/6jhTmwbozsnh1ejVeIYtX/Fa8Qhav
8l1VxNZiNRhljAPPNU5kLeRdzKU5nHxPL+sr90YHtJeFGr8nxHkGx/QF51Tz
YeZlsVYxhvWV5/DM2s7dq2OZfavrtksxn/ybeVR4jTVB5sX8D/k5fQ2/Fd+O
jfUC//St6Kp6u0d1bwB6rk/hGdVhnt04fVLz9ybnvhg5bcXccmRfU3Kw7B9Y
Cwdbax5sdzhY8q7kTtkDlvuds3cia465f0f+z5H9qDkniefiAP/02uq/BSAm
TNdexWfWSP7P2qlLuTcK3RZj4dLGOOiDtkYPmDlh7NvcJlzM2B8dMceLvZoz
wYeKmQ+ptpfHVPtK8sfWOPBp6s/jq9/lRRxnbwm2jn2hbz9b/T7DW1bHPmCR
a836mLcHK1zfU2JOwIfFWvVBJ1RjKc9oLgtZnozsWiOLVydU49WJMYdc33jw
5Gq8QhavkJer302tnhwb8mFxzv4xV5vjHNZaTEMfUpfsT+DwPRjoiZgDb1Ef
7h7jZ55dd+ZRDLxnNUe6c3Ws8ZZh3rlu7jNinbVx1tNcK/UC7Zd50GZZH/Vn
nxinewDRpQ2xdtN7/ezPnM4P5/4+n7Fq3JOZ+2LSTtfCwXJPn3EQx7w8mDrM
kXkw31FQtbY8WNYWs78rc/JiOIcYzpF99dmPmr305gCnnzfjZXGAQ9/EIc/n
59omcvazyc/dA4WvZE2tk4Jd5qNYd/NR+CztCFvQj+NDzVVy7e3Ve7H1HfAB
axPm48mBwGHU5/NjPjP/c6u471J1XYzzjXfADOv1Z8Z9T6mu22Lvri/jsU8G
/mZsyHwY4xxVXbeituC6HxnjPLIarzjXeHCxOv9zdMwh9zIeYQzqA9c2f3Vi
PC92Jv74PfJxcT73tVeQ+Vff9qrmqKxv1r7NxXHoj6qaM6Mnxp74LHtTGefT
BxmdNf/sO0H+Y/i3ncNcZ26B6xivYVdiNfUBcQ881Acxx+o5OmK8D35qs+ih
XB0/q55n3iz7gqb3+rmOyNP2a37GtasYM8dyNcfIfnLnimNbyM8I+flxvtyD
g5hdbLQWyZG1SPtPOLT3qvE7+sxLc2T/fMZ3+lOO3PuT+GyMzJF7rvO55mGU
/IHDPnOO7KPIWsDO6necms9Etj8NOf2O64Vs3hI9t0cIHXBdNsaY0UN1DBtR
x8Ax/SOxpPVT8ECfiB6JCfzXnoFjYgxwCe0U2bwK51gLAxP0WWCIforrmx84
rLovlzheH42/NpZBNpd1UI33KIlXyPKrg2PeuKa2D+fUX3Nf9YG5l1cvVuuG
z7hruI+86KSQT415OC3WhXO0Ka6pr2HOMz6ynsK6Zz+znJZDjJ3UeG+F+M+z
mIeES1rvIGbVXh5bjRtPrsYK1s31BYOtQWA32gXPaLzPvcwLwefVfzBKLsdz
2fOMfuqj11VzjIVq/jyprq8hmweb1Dg+Smw3D8aR8ZHYzpH95OZGOPTpHPaS
cTw95NwXY32Ewx4kjnn952I4h/ELR/aD5fi1KY7MQeWe6NyTmPGv68KR74KY
h1EZ5+bY3PvJkfFs1ijdC8DttDtk41Zkc/7I1uNYazkwOiC3QTfs74KDmWME
x5zP6VqkOZDcJwimqTPopPVr30vDdfCTy9Xv+xWvDogxIG+vfoe5uSxkc1lc
05hi/xjPftX7bZHFK2Txar+YN35r/ir3F3CvpVq13wOr+0XBN3UDTBNnsGMx
arGabx8Xc44Nacv4yQdU9x2JdXABbR9ZWwNHjbWPjXkghjIns6HGnMQ8Mzhg
rmnfmCtrxczhaTGf51fznNvEfBLrJX+W151ZHYPcIOaE8RpnwaPcf40ftD+K
sWkXjJn4AoxFP835oLfWiNFnuTSyOdtJjWv3rMGs2n32lmdPptjOkTWyfDeL
tRKOzEXLJzl+K+Rn1e7n8L2v88DhWnOYi+DIHFTmyTNHl3uic2zGMhyJUTme
xKisIyT3U5c4sl5pHoYj82zmIhiG8Quy+3On843GquhAvlPCeBw9kVOhP/rc
DTFOcMx86R7VegKmmRci/5B9+3IGdNWcMzk0687ZE4u8Pc5xnwWy/BlZ+9oS
84ZsTg9ZvEJ2T+LeMc69Yw6zvxG7Nue2X4wfTMt4TV/GHIhR8C7jL9bKWjD2
rZ5nb2T2lIIx+vo/jjlk7c3nwAf1xfAtY4F7Vfti8Ee9hevKvY+q5p/HVudC
sWn5DLiaPcbiFXNmreqYmMPDYpwH1Pjv0SxX598eWY2f5hzQMXkIuid2oZPG
HeitcUdyxUmNeyOzvua6c+inODL/k+9YyPfgZX0/48fM/2SOOuPHxIdtIc+L
HxMr5u21SdzIMWhHHOoYhzyEI/Nvyf1eWrMxKvf1uMecI3vmM7eWe5Ryzher
e+kzX5F7wJPf6heyJoIsj0XOuqS5C/RETpX7tdEl42hkdQN9M0YA04xHkO0V
B9/MeaKr+ilkfROy/gjc0wchi1fI4tWmarxCFq82VdtXvrcN2Zovsr2j2JE1
071izOCbMeDeMf59a/x3DNWZA2KesWn9AusmJjM35nJvETKcyzwJvlQOT9wg
L31NtQ48Np6RPS/6Mr7Xp2CLxg6/HGt00+pYMvdnYcfiFVikTh5Rzdl4LvOl
PK+cbWuN35e1VKs+jnmW26MDckt0RptCl7Rl9M39Neik2IWc2DUvHklfn3aU
vUlZX8ueTPOwHNaIK56FYx63Ma7n0E9xzHsv3+7Gj5nfTo43r0bwtJCfV2OM
cpwZ6yVGJZfL/vncU5k1nZzb9AvZq5+9Xplny/eD5XNlzlC/xv/NXl/nBFnf
jez+joXq+BTZGHA6z6/eZq0Z/XR9kbVHZPETfBM/kc1fIdvrsiHmc5rv6WeR
zachy/2Qjx7GCTbaf4Vs3jjrp9ia+gmmyanANPkq/EH9BLvMb7CG1j2xD3k4
HMZaOVgiPj8inh08NOdJzUHd3qsal3JPH/9mrw5Y7N7kO8YauccPbME/y0/O
ifWFa4n/h1bnJw+s8fvxjL/2rvF7iuRazKf6yRqJ+aw1caLvW3PeFmKu0Dc5
NrL9A2KXup356sSujFOSA2RvwLw9KfP6KvO9CteW+5qVO8r8trE5x0/SA5a1
vMyh/W7I2YPxsrpujIIDJEaZK8s8f/bPmz/nSPxP7po989iUmJ+1APMnHIm9
2+JZEm/FcP5JTpX+C1m9Rc66pDjw/3o5exy7iiCMtv1sEyASRIIgGAmBCFjE
sALEDrwAhywAIxEgEiJSBohBYgdDREDKAoYFsAQkdOQ56vOub795gxEdlazx
TN/++arqq68aWz/F2ZNvwZYTwJbTwDZ+5txaPz1kbofMDdu3R7DNox+N6Vux
vVPVoWGbv2DLrXGPrJ9iy7GDaeaAr2VtseWXwLH2hnh+uL+eH7BLH0d8J1cA
XvomFfdATCYX9D4yJ8/SszF1LOCPeS7fZ73s7zHxyn4o76MxPHMwPiEe9L7D
5VhnxzfqC97K3/VNPNb39TF1I+1hZ92ss7Ce3nf2S45IfdS2B43zs9JOn1NP
vy92NTbovWtvcvXkfVdhpWsSkxkr7cSK+2ru9kXsr2P3DZmVTqw55tU4xijX
aotRYm81FdWDWeNgVAPWtdrrPWSoLWE0Li0f2Fi0GN5vr+atdYrWJsRY/osx
ttymGCWuYhv7YcsDPBjHb933fIqlnFv3CNt9wTaXx7b3B9sz+XBMPQP3gjNl
jOfaYlubwBZXGwdW74FNzYWzx70zXsV+OmYNQu0Td9Z8FuySTwa7xF6wyzgc
7JQrINbSR/A35QqIL4x/mKOxwXf5FnyVdbFfsi+su3p+9IzW6+0nZG5wZXKV
H4ypeyQuM44Fo65v/w7fJXaB2+YFj8fs5WTd5NZYW2MV9khuENvYlTMgp73l
V1d+tjWs5jXFrtbZi13VZ1ZbXuyqbrw9fea8jOY4jR+2vP2erkB/zSjv5Fli
NA+qDqq41Lv87diPwbomK4yq9r690jdj8k59+6sa++J5uSk5TEbrF82dq/tq
vtw4s/hcXW5rqd3f6ij0p4zmquZW/An1AHL+e2dSfhLbuiq2cQi29wjbe73l
Llozcp0fZp7bXNX72z5fdYact0PmjK3eaav9MLZ/nPmDXc9v589d9k4RgxkP
wOcYh8PVqwMHe+QG2XP92idjcoOsh3ELvtrz8Gnm8OWYsQT3QL7uj/w88aA1
evBDjOUMGsOAUVdjcuPqhfiu6zFrLuaPh3H8tpsxLWur/2W/Wr829sY2jsWu
lqB3ufqBxh7lfJrv1P/eF7ua7/Rdhd7TizHv8qrmWE0U37j3Ft9K99W63or7
ap74w/hvMMp66xgvvzPTeqvf3piz+fKqD7F5316PD2NPP89o3aExVWPCxs+t
ubwd+/0x97r6lu6RuMqU2s9lDIzd/tPyFT/l/5qzYP8+5pk3v97mqnLC2N53
bNeZOyXHgu06YxvPY7vO1ZNjy7c8yvy5y9bowTG1gsRgnh+wy7tD3OWbP9xv
4yh8uDoDuEnvAr7XeA8skdcib7aG+1l+D7grPn80Js/JHj4dL84DGCV2gbGe
GeZvrso3GneZj/91uz6+p4F9M6b/qj5QDrbrz6gvbj7Vu1z9ubVFRn1xuZrG
GOrqGa+CXfZxM87BrvbrrWqO1ZzLizJ6r6ubKmf149jXcfVdrGKU92WM47e8
qpU6VVc1VrnrPYftdzXv67c0/y3n1n6B6myLvduYyrjuZsy9KB9S/r96vO6L
OqXtnFvzrb9onH81ZgzWt4mux3xXzZoFtriKbb1sy60Ru6rVkbMyb5WjMwfE
9vxwB41dsfV3zWG3nJv+7lHmD479Ol7sKbGKsVDf+4K395zAHYkzrPfl7beD
Y+q1OC9y9eCZ9e4Px3FdTz6Z36lPIVc1TwFL9SNPsi/MXxzgu4zJH47j9+76
pu6q/6Lnv7lVe0maT624jp55662M1sLK4bRHpj66uqYV33UKu+Rwqi+y7sxo
r3H5nOLzx2O/5th62Yq3/zk/U53DFqOMeZorVfshF8rQvzPMRxjlz4vD3a/W
HezZZDTvOydOrq/x7RGGvpjRmKp71G+pBm9P98UoxjZnr4at9ZHOuf6xNVx1
14zqQHqu9BFya+oGq3mWD3wwjrVq8qXY7ZOVI8WWD8Q2DuHOfhVb/hzbHJY7
rk8/jGPNdvtNnDO4oS8Ax4yp3hjH782K+eSVcvhvZq34Gc8nsZy+hthJ/whm
yiOBpep5mGffNao2z7iRdVjVkVtXanze/KK8ULUEzafaT6SGkNH7W99Xnrb+
utxOY5LeBXX1jN/G/4dd7RW6HPM+1te35kivlrh9LkYZe2wxynpo3394d+y/
3Vef0pjknLcdqgdrHbP8m9oVxkqnuoqp5KvHOM7fW69sTNi8tZqKcmutiVRH
V/1J40DffmTI8zDu6lFyzns11p6Z8if6OGxywFPcL7b8J/Y3+fer8XJMyB33
LGF7/g9jxlTYag/AMe8vGGLsB6bpc8EZ8wLwzbiOf1efhi2/9GRMv8Pvv8jf
Vb+xrdmVY2zObiyH/X1+pm+q1KeYJ45xHKt3j+rHV+dKjotxzpsnvRfVZ/aM
oRERu9rT137kvmf1qthlPrXqU67Wq1jdXpvWHM/BqOrQTmk5jElWWo7Orfls
YxL7Xhmt961i4HLp9SOeYcb1uDumstbD6LthjeH71nRjQjWNjMbw5Q/lfBhq
bBitUzcfr+/rvlQTstKt1ceVM6lfW9UvxNhtXcD6Gra+G9s6JvZduRKYIIZg
vxNbnS144rkt53/IfLZ5qDHDIWslNtpzZ2yDbU1ni9tyR9h9/+H57bczet58
g4tRjqh9u92v3p1ztdB7HFf3yxyZsXpnoOetmszWnnpfyusWu6pNPRe7zE1W
2FWd6krr1W/vXbCOzChG3bfu2TtSLrr4qV9jVGPfmKR5X8/Mtra7p0s51UPd
mMo5d52ro1tpwC7HxMDWAlY54OeZf/PW8gwrjK3OdlsL2MtbW29d8SQrjK2P
aF9Y/fWpuoD5kX4Bu5qQ9uq2FtBc1Xi1OiVsfRa2eIUtp4FtXrP9u+I5dt80
kFfBbu+JsRajsXHj+dZxmmfJ1zHKaW97zYxDWsdpPlLOwb4DRjkfNaiM8vOt
r63uTrU95Xj7Dgz8idjVN6yKXT2T52BXeeytxt67Weyq1qsYtdWVrWoErvPF
mH7B/g5G33zoOq/ey3o6/n18Wx/Rde6Z6X3fxlQrf2d80n7/VU2nPNu2prMX
E5ZbWOkr2l+56gGvPuRVuJH24zfvbr21/aHlBoux1Ql3zjdj1s6KG++Nyb/5
lhS252SbnzZHE6+2nJJ5GXbrp/bLY1+Mub9dn+b11Qs9G/t3v/x297H8Q31l
61N/Zg7NU1Z5Yv3+5Zi+rPtYnqd+p3i76qdb9fyWR73O320M2fz3FIbcB7sa
Z/rt/wBlwsnJ
   "], {{-1.4, -1.4}, {1.4, 1.4}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["24: quantum1", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", 
   RowBox[{"Gauss", ",", "p0", ",", "f", ",", "doplot"}], "]"}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Gauss", "[", 
    RowBox[{"x_", ",", "t_", ",", "a_", ",", "x0_", ",", "p0_"}], "]"}], ":=",
    
   FractionBox[
    RowBox[{
     SqrtBox[
      FractionBox["a", "\[Pi]"]], " ", 
     RowBox[{"Exp", "[", 
      RowBox[{"-", 
       FractionBox[
        RowBox[{
         FractionBox[
          RowBox[{"a", " ", 
           RowBox[{"x", ".", "x"}]}], "2"], "-", 
         RowBox[{"I", " ", 
          RowBox[{"p0", ".", "x"}]}], "+", 
         RowBox[{
          FractionBox["1", "2"], " ", "I", " ", "t", " ", 
          RowBox[{"p0", ".", "p0"}]}]}], 
        RowBox[{"1", "+", 
         RowBox[{"I", " ", "t", " ", "a"}]}]]}], "]"}]}], 
    RowBox[{"1", "+", 
     RowBox[{"I", " ", "t", " ", "a"}]}]]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"p0", "=", 
   RowBox[{"{", 
    RowBox[{"2", ",", "2."}], "}"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{"x_", ",", "y_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"Gauss", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"x", ",", "y"}], "}"}], ",", "t", ",", 
     FractionBox["1", "2"], ",", "x0", ",", "p0"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"doplot", "[", 
    RowBox[{"t_", ",", "opts___"}], "]"}], ":=", 
   RowBox[{"QComplexDensityPlot", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Exp", "[", 
       RowBox[{"I", " ", "10", " ", 
        RowBox[{"Sin", "[", 
         RowBox[{"3", " ", "x"}], "]"}], " ", 
        RowBox[{"Cos", "[", 
         RowBox[{"3", " ", "y"}], "]"}]}], "]"}], " ", 
      RowBox[{"f", "[", 
       RowBox[{"x", ",", "y", ",", "t"}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", 
       RowBox[{"-", "5.5"}], ",", "5.5"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"y", ",", 
       RowBox[{"-", "5.5"}], ",", "5.5"}], "}"}], ",", "opts", ",", 
     RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
     RowBox[{"Frame", "\[Rule]", "False"}], ",", 
     RowBox[{"QSphereRadius", "\[Rule]", ".5"}], ",", 
     RowBox[{"PlotPoints", "\[Rule]", "100"}]}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"doplot", "[", 
  RowBox[{
   RowBox[{"-", ".8"}], ",", 
   RowBox[{"QLightnessRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{".4", ",", ".4"}], "}"}]}]}], "]"}]], "Input",
 CellChangeTimes->{3.390026668490122*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJyV3XmQ9GdVxfFpQEDihCggICDdgLIOm8YgJNgDiiAMGoQYlpAegiDLEBBZ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   "], {{-5.5, -5.5}, {5.5, 5.5}}, {
   1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["25: quantum2", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", 
   RowBox[{"Gauss", ",", "p0", ",", "f", ",", "myColorMap", ",", "doplot"}], 
   "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Gauss", "[", 
    RowBox[{"x_", ",", "t_", ",", "a_", ",", "x0_", ",", "p0_"}], "]"}], ":=",
    
   FractionBox[
    RowBox[{
     SqrtBox[
      FractionBox["a", "\[Pi]"]], " ", 
     RowBox[{"Exp", "[", 
      RowBox[{"-", 
       FractionBox[
        RowBox[{
         FractionBox[
          RowBox[{"a", " ", 
           RowBox[{"x", ".", "x"}]}], "2"], "-", 
         RowBox[{"I", " ", 
          RowBox[{"p0", ".", "x"}]}], "+", 
         RowBox[{
          FractionBox["1", "2"], " ", "I", " ", "t", " ", 
          RowBox[{"p0", ".", "p0"}]}]}], 
        RowBox[{"1", "+", 
         RowBox[{"I", " ", "t", " ", "a"}]}]]}], "]"}]}], 
    RowBox[{"1", "+", 
     RowBox[{"I", " ", "t", " ", "a"}]}]]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"p0", "=", 
   RowBox[{"{", 
    RowBox[{"2", ",", "2."}], "}"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{"x_", ",", "y_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"Gauss", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"x", ",", "y"}], "}"}], ",", "t", ",", 
     FractionBox["1", "2"], ",", "x0", ",", "p0"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myColorMap", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ab", ",", "ar"}], "}"}], ",", 
     RowBox[{"Hue", "[", 
      RowBox[{
       FractionBox[
        RowBox[{"Sin", "[", 
         RowBox[{"1", "-", "ab"}], "]"}], 
        RowBox[{"1", "+", 
         SuperscriptBox["ar", "2"]}]], ",", 
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ab", "2"]}]], ",", 
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ar", "2"]}]]}], "]"}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"doplot", "[", "t_", "]"}], ":=", 
  RowBox[{"QComplexDensityPlot", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Exp", "[", 
      RowBox[{"I", " ", "12", " ", 
       RowBox[{"Sin", "[", 
        RowBox[{"3", " ", "x"}], "]"}], " ", 
       RowBox[{"Cos", "[", 
        RowBox[{"3", " ", "y"}], "]"}]}], "]"}], " ", 
     RowBox[{"f", "[", 
      RowBox[{"x", ",", "y", ",", "t"}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", 
      RowBox[{"-", "4"}], ",", "3"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"y", ",", 
      RowBox[{"-", "4"}], ",", "3"}], "}"}], ",", 
    RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
    RowBox[{"Frame", "\[Rule]", "False"}], ",", 
    RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
    RowBox[{"PlotPoints", "\[Rule]", "100"}]}], "]"}]}]], "Input",
 CellChangeTimes->{3.39002667060316*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"doplot", "[", 
  RowBox[{"-", ".3"}], "]"}]], "Input",
 CellChangeTimes->{3.390033902982664*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJykvXm0nlWV7jvvuP+fozRCgCQ7ARJ6QhMggYSE0PcIRHoSGmkFlEZBkIAC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   "], {{-4, -4}, {3, 3}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["26: satin1", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myColorMap", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ab", ",", "ar"}], "}"}], ",", 
     RowBox[{"Hue", "[", 
      RowBox[{
       RowBox[{"Sin", "[", "ab", "]"}], ",", 
       RowBox[{"1", "-", 
        FractionBox[".8", 
         RowBox[{"1", "+", 
          SuperscriptBox["ar", "2"]}]]}], ",", 
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ar", "2"]}]]}], "]"}]}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   RowBox[{"LegendreP", "[", 
    RowBox[{"5", ",", 
     RowBox[{"LegendreP", "[", 
      RowBox[{"5", ",", 
       RowBox[{"x", "+", 
        RowBox[{"I", " ", "y"}]}]}], "]"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", ".5"}], ",", ".5"}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.3900266755202303`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJzsvQe0ZVW1rvtTRAUMGMAMKgYMGBBREAsQUCSjRIESAQOCAVHMoIARIygG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   "], {{-1, -0.5}, {1, 0.5}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["27: satin2", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myColorMap", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ab", ",", "ar"}], "}"}], ",", 
     RowBox[{"Hue", "[", 
      RowBox[{
       RowBox[{"Cos", "[", 
        RowBox[{"ab", "+", ".7"}], "]"}], ",", 
       RowBox[{
        FractionBox[".5", 
         RowBox[{"1", "+", 
          SuperscriptBox["ar", "2"]}]], "+", 
        FractionBox[".5", 
         RowBox[{"1", "+", 
          SuperscriptBox["ab", "2"]}]]}], ",", 
       RowBox[{
        FractionBox[".7", 
         RowBox[{"1", "+", 
          SuperscriptBox["ar", "2"]}]], "+", 
        FractionBox[".3", 
         RowBox[{"1", "+", 
          SuperscriptBox["ab", "2"]}]]}]}], "]"}]}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   RowBox[{"LegendreP", "[", 
    RowBox[{"5", ",", 
     RowBox[{"LegendreP", "[", 
      RowBox[{"4", ",", 
       RowBox[{"x", "+", 
        RowBox[{"I", " ", "y"}]}]}], "]"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", ".3"}], ",", ".3"}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.3900266765416994`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJzsvQe4bVV57v+pUTRGUWNsKBwpoggoYANBAbEAClKkSTlKkSK990M9dKUr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   "], {{-1, -0.3}, {1, 0.3}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["28: satin3", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myColorMap", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ab", ",", "ar"}], "}"}], ",", 
     RowBox[{"Hue", "[", 
      RowBox[{
       RowBox[{"1", "-", 
        FractionBox["1", 
         RowBox[{"1", "+", 
          SuperscriptBox["ab", "2"]}]]}], ",", 
       RowBox[{"1", "-", 
        FractionBox[".8", 
         RowBox[{"1", "+", 
          SuperscriptBox["ar", "2"]}]]}], ",", 
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ar", "2"]}]]}], "]"}]}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   RowBox[{"Zeta", "[", 
    RowBox[{"Zeta", "[", 
     RowBox[{"y", "+", 
      RowBox[{"I", " ", "x"}]}], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", ".15"}], ",", ".15"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", ".84", ",", "1.01"}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.390026678814968*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJztnQnY7mPV9tdDMs9jG9lmKhQhmnZKkuY0T6RXvc16Ka+pjR0yxN6IjBuZ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   "], {{-0.15, 0.84}, {0.15, 1.01}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["29: sinsin1", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myColorMap", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ab", ",", "ar"}], "}"}], ",", 
     RowBox[{"Hue", "[", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"Sin", "[", "ar", "]"}], "2"], ",", 
       RowBox[{"Mod", "[", 
        RowBox[{"ab", ",", "1"}], "]"}], ",", 
       RowBox[{"Mod", "[", 
        RowBox[{"ar", ",", "1"}], "]"}]}], "]"}]}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   RowBox[{"Sin", "[", 
    RowBox[{"Sin", "[", 
     RowBox[{"x", "+", 
      RowBox[{"I", " ", "y"}]}], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{
      RowBox[{"-", "\[Pi]"}], "-", "0.6"}], ",", "0.6"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{
      RowBox[{"-", 
       FractionBox["\[Pi]", "2"]}], "-", "0.6"}], ",", 
     RowBox[{
      FractionBox["\[Pi]", "2"], "+", "0.6"}]}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.3900266832112894`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJyMvQmUnlWV/X0CYZ6HgAKBggJFwNkmKiihnUBARURREINDg9qgSKuIA0Ha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   
   "], {{-3.741592653589793, -2.1707963267948966`}, {0.6, 
   2.1707963267948966`}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["30: sinsin2", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myColorMap", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ab", ",", "ar"}], "}"}], ",", 
     RowBox[{"Hue", "[", 
      RowBox[{
       RowBox[{"Mod", "[", 
        RowBox[{"ab", ",", "1"}], "]"}], ",", 
       RowBox[{"Mod", "[", 
        RowBox[{"ar", ",", "1"}], "]"}], ",", 
       RowBox[{"Mod", "[", 
        RowBox[{"ar", ",", "1"}], "]"}]}], "]"}]}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   RowBox[{"Sin", "[", 
    RowBox[{"Sin", "[", 
     RowBox[{"x", "+", 
      RowBox[{"I", " ", "y"}]}], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "\[Pi]"}], ",", "\[Pi]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", "\[Pi]"}], ",", "\[Pi]"}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.390026685354371*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJx0vQu86GOV/7/OcU8yjaQIm6Khe0npZuc0LuFfmWpKYUf9GtRINVOUOhNJ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   "], {{-Pi, -Pi}, {Pi, Pi}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["31: sinsin3", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myColorMap", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ab", ",", "ar"}], "}"}], ",", 
     RowBox[{"RGBColor", "[", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"Cos", "[", "ab", "]"}], "2"], ",", 
       RowBox[{
        FractionBox[".5", 
         RowBox[{"1", "+", 
          SuperscriptBox["ar", "2"]}]], "+", 
        FractionBox[".5", 
         RowBox[{"1", "+", 
          SuperscriptBox["ab", "2"]}]]}], ",", 
       RowBox[{
        FractionBox[".7", 
         RowBox[{"1", "+", 
          SuperscriptBox["ar", "2"]}]], "+", 
        FractionBox[".3", 
         RowBox[{"1", "+", 
          SuperscriptBox["ab", "2"]}]]}]}], "]"}]}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   RowBox[{"Sin", "[", 
    RowBox[{"2", " ", 
     RowBox[{"Sin", "[", 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{"x", "+", 
         RowBox[{"I", " ", "y"}]}], ")"}]}], "]"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", 
      FractionBox["\[Pi]", "2"]}], ",", "\[Pi]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.390026686315754*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJyEnXdUDv7//ksklTIiZY+sbCoSTyMhSWQmWyGzSNllRKIkpVIkMtp77721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   "], {{Rational[-1, 2] Pi, -1}, {Pi, 1}},
   ColorFunction->RGBColor],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["32: spider1", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myColorMap", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ab", ",", "ar"}], "}"}], ",", 
     RowBox[{"Hue", "[", 
      RowBox[{
       FractionBox["1", 
        RowBox[{
         SuperscriptBox["ab", "2"], "+", "1"}]], ",", 
       FractionBox["1", 
        RowBox[{
         SuperscriptBox["ar", "2"], "+", "1"}]], ",", 
       FractionBox["1", 
        RowBox[{
         SuperscriptBox["ar", "2"], "+", "1"}]]}], "]"}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   SuperscriptBox[
    RowBox[{"Sin", "[", 
     RowBox[{"x", "+", 
      RowBox[{"I", " ", "y"}]}], "]"}], 
    RowBox[{"Sin", "[", 
     RowBox[{"x", "+", 
      RowBox[{"I", " ", "y"}]}], "]"}]], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", 
      FractionBox["\[Pi]", "2"]}], ",", 
     FractionBox[
      RowBox[{"3", " ", "\[Pi]"}], "2"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", "\[Pi]"}], ",", "\[Pi]"}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.3900266900711536`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJzt3QvUbllVHuj5USmDd2J0aLrb9ifdHZpuLx0vHTUtnAJjEi8Ro91KvKTQ
VulERRswXgCruIqCgGhEC6hDGRFBKTSiYKOcMrQiaVFi7AiteJDQiEYEBaGo
qnNyXr8193r29t+kSI/0wDH+PcYcNes/+zLXnO8711pzrb2/u3/5g/7+111V
Vf/rkOg5HjAkx18bkuONQ3J8xJActw7J8V8MyfH6ITn+yyE5Lg7J8ZFDcvzO
kO35/2pIjr86JMf3D8nxCUNyfNaQHN8yZPusXxuS475Dctx9yPb+zxyS46OH
1IH2XtE/uY4S/aF1lOi/VEeJvvjqMOXtV/5zachfuSK/PSTHU4bk+MwhOf7r
IVtf/f0h5TOuHH9nSI7/fEiOhw3Jcd2QHMb3iUNyfMGQrQ0+678akuMrhmx1
z/Fa7+mztEHbtNm22Ebb7rP0lT7UBn3esTBGxs6YbmMtBsSGmJE7C8ZqjT0x
KVbFsNgW83JBjsgd7y/X5KC+krM+62JNjnu+OcH2mkOMrznHXGSO2sstHzsk
x98bkuN/G5LjnkNyvGzI1rbLQ3L8Z0O2zxLP3zEkx6cMyXGfITkePySHfvvN
ITkWbFw5/rshOf5oSA6xrW2fOyTHO4cEY4+to2yxd1JHib74/4BvDxNv76x5
/sfVmmvq+vZ+Q3L8ypAcLxiSQxzqE+P1fw7J8d8MqVr7Siz9t0Ny3GtIjl8e
sn3W1w7Z6p7jtd7TZ2mDtmmzbbGNPkuf6Ct9qG/1ecfCGBk7Y7qNtRjwfDGz
YKnAWK2xJybFqhgW2/pKLugTuSOn5JoclJsXa3JW2+S43Dcn6FvjZW4x5/gs
c9TNQ3J8zpAcYmOvP33gkBz2j/YXrxiS468MyWHuFatfOiTHXh+tf/TDuSE5
fmtIDnO1GP53Q3J8+ZAcHzUkbb5HHSX6P6yjRH91HSX6H9RRoj+9jhL96+so
0buPeFfNMW583eOEuPenh2zbaIz+7pAcJ0NyGK9/MiSHuLXt9qf/bEiOrxuS
w77vG4bkkGvG1Oeqe47Xek+fpQ3aps22xTb6XH1yUtNX+lDfts+NhTEydsZ0
G2sxIDbEjFhaMFZgr9aYFKu2XWyL+XM1uSBHxNXe2FgOyk05a0zluNw3J5gr
9sbbxstcZIyuHZLDnOlY8X8ZkkPM2A+a5z97SA7xaZ7/3SE5PmZIDse6Pzwk
h/759iE5Fvxs7H/MkBzOL05q4vbjh+S4ZUh890N1lG2fuIx/Dvhhc84X1lGi
L9w5zD7rNux/UE3OZn5uXy8mtVkf/s0hOZ47JMfLh1St+1zHb188JMebhuT4
60NyyIVfHJLji4bk0OePHLLVPcdrvafP0gZt02bbYhttuz7RV/rwpKZv2+fG
whgZO2O6jbUYEBtiRix5zoK9ApO1xqo2i20xr8/liNyRU3JNDspNOasP5bjc
NyeYK8wh5hZzjvZfWzNH2a85T3z+kByOx5ZYXDluGLJtu2NO8ekc/A+H5DAW
nzgkx1uH5DD33ntIjqX/unL890NymOedj9t3n9SM++LbK5j51jpK9KU/uqI/
qo4SfZnLHODapsblvKD9f3tN/Uk1z/mLtR5Ly1/nTQ8fkuO2ITmsfZnrjIs4
/OMhOZa+vtZ54JohNdrQ/aD2OB54wpCt7jle6z19ljZomzbbFtto2/WJvtKH
2tM+NxbGyNgZ022sxYDYEDNiSYwt2Ks1309qYlUMi20xLxfkiNyRU3JNDspN
OSuX5bjcNy7mCnOIucWcYy7St47Zfn9Ijo8bkkO/ec65mmNO65ZvGJLj04bk
cNz7jiE5/schOfSzOflTh2yf9dIhOb5qSI493r19SI5/NCR+WO55WM8HP72O
Ev3GOsoWY0st5Yp+fR0l+rU1+oXDrPfeXlNP399j7w+qdY30RUOq1mP4/2NI
jv95SA59a9vl41IHrjV+9Lk4/KYhORzzW2dwbP99Q7a653it9/RZ2qBt2mxb
bKNt1yf6Sh/q2/a5sTBGxs6YbmMtBsSGmBFLYkzsifMFq7XG8F6elwtyxHvK
Kbmmz+WmnNW3clzu+6xzNXOFOcTcYs5ZzjFHHbDhsH6uff03DsnheFVcac/b
huQw52vPHUNyWOuzjb8wJMeXDMnhmPD+Q3L83pAcf2PIti1iLPWIdw19WW86
YOeBPmLTVy44OdDfXdGfU0eJ/m/qKNYx0ua3DAmGXjck/fSLh2TIsWC+1jVY
fWi7jMVnDMnxc0NyWKOW4/aDxsK+2/mdMbI28qwhW91zvNZ7+ixt0DZtti22
0bbrE32lD9u3+txYGCNjt61NGWsxIDbEjFgSY2JPTC5YrfVap+0S83JBjsgd
OSXXjJGxkLP6UI7L/XM1c4KxMIeYWxyXLrlok6P0oeMu13btg7Rhb/30piFb
O83nzgWsLZzUHPd+95Acju2d//4PQ3LY5zqXt2a+2HNY1xnEnnj723WUd4cx
+8SlPnlgLH2Ycb8DP7yyZv+VeHetIPUTMWO/qc/FjNiT+66FWcM8qelnn2Vd
dFm7r/X4ZM+G5w3Z6p7jtd7TZ2mDtmmzbbGNtl2f7NnQvtXnxsIYGTtjuo21
GBAbe32fGBN7YlKsyinHOWJeLsgRuSOn5NpJTT/LTTkrl7VH7u/tLzpXM4c4
5lxyziYX7eWo/6mOEn3ZF3RFf3AdJbptcW772iE5HlBz7rm3JmuuXp67aZfz
dPeZ+Fznv8tzD9z/QE1P+w/MUw7Uug/rMfyyLnNFf0QdJfrSjx+Y42zuubTl
MHlxBz75jZp/z7pyY/t9an9N07HWk4fkcE38Z4ZUrWv41iv28POSITkctxgj
xzzOF35yyFb3HK/1nj5LG7RNm22LbbTt+kRf6cP2rT43FsbI2BnTbazFgNgQ
M2JJjIk9Mek9T2piWGzLKTkoR+SOnJJr+tkYef+9vUA+15xgPtd+c4s5x1y0
l6NuqblGtqwXbMao7jMxP7gnxHmlY4N/MCSH48a98107sP9d9r8d1vhZ6oQH
1kAPrEMdqNMeTt8ftc3nrgEtNckDe9sO1FWu6F9TR4nuvLvX0BPvxu3Fmn3K
F9Pe96/jutSDjpfW+SE5XC/T5/Jx6a9rnffElX2uewDsQ795SA7H/N7H2P3s
kK3uOV7rPX2WNmibNnsf22jb9Ym+ah/qW31uLC7WjJGxM6bbWIsBsSFmxJIY
E3tiUqyKYbEt5uWCvnLvqJySa3LQ2MlZuXzq+eaETa6wLeYWc465yBxlPl/q
cof1cx3HLn3lFf3n6yjRl/rqYb3W5hru/15Hib7U5Tb2+CzHyYuvDuzbOdDP
Hvbnesu6xmG9N8Y2Om63r1zW1jd92VPrKNH/nzpKdMe3zrv/3zpKMN/9zhuQ
h1yRPxnyoeOayF1qva94qdPWui+zzmB92/GMdVRrGvazrxySw3GL93S9z3qR
NSV1z/Fa7+mztEHbtNm22EbvqU/0VftQ3+pzY2GMjJ0x3cZaDIgNMSOWxJjY
E5NiVQyL7VPnjBuOyB05JdfkoNyUs3JZjst9c4K5whzis2yjucgctexRPKzX
KcyB1n+6D4q+N+fy2gVXB+bLV/RfraNEX+bIh/2x37+so0R3f9Qn1VG214qB
62uu1yjfWUeJ7lqzY2/96Rqiz132nBzYu3ugnnCY/Wz89YND0sf3+xexo/cA
nNTEw9U1fXuodX/n2sppNZntOe4ldr+N4/xlnb3W+5qs61pTcm7l3nV1z/Fa
7+mztEHbtNm22Ma9Wl+fow/17UlNnxsLY2TstmMnYy0GxIaYEUtiTOyJSbGq
iG0xLxd8rtyRU3vzL7kpZ+WyHJf7p85VNznE3OK15iJz1Lka9YFNPjc/L7XN
A3uJD+w93uRM2+teuJ+qo0Rf6gyHdd1yz89LHW9jg7Zp80kdJfrfqqNEd8yZ
9enbhy4eHPMva1uH9fqF89C9/mWpOx3W9fbGRmoWjYdn1vRh5jw9Tn7fmvOd
Q63HJ671+Hf3DFiX/p4hOdyP1GthOayBmwd8lut9jnn+9ZCt7jle6z19ljZo
mzbbFtto20/zlT7Ut/rcWBgjY7fFrbHem1+IGbEkxsSemBSrYlhsn9TE/J3h
r5za6/dto5yVy3Lca80J5oo7w19z0bmaOcqcZnuvqTmPfnYdJbq1wWUOe9jf
v2TNfy/XLeuwh/213WXt+7B+59RcvezF3fhN/DhmXmosB7hwRf+BOkr0hTsb
/7u+4xj+tD2i4vOOmue/Gf35Nc+5b00/fDD6VTV9G975roRr2dYi5LV7An2H
5SFDcrgetOCn1uMW60i+x/GaIVvdc7zWe/osbdA2bbYtttG2t0/0lT7Ut/rc
WBgjY/fu+i8x4PliRiyJMbEnJsWqGBbbPlcuyBG5I6f29vDIzb2cI8dP3T+2
yRXmEHOLOeekZi4yR5n/l72vV/TvqqP8abCRx9VRtj43XvYXvm/lu8PZN/yk
oZvb9ZXfRvjHdZToy962w3ouf65mHra9YsZ+UNucX1t/WPZtHtZzf2v+xlSb
G5+Xa/o5mG9u/iH6z3DOF9a89iNq3vN9avo84fA9OPcTur/ltDpzDvef+w6F
dSHzyeLPWudG15X81oS653it9/RZ2qBt2ry37mDb2yf6Sh/qW31uLIyRsTOm
21wkBsSGmBFLYkzsiUmxKobF9uLPDRfkiNyRU9osB+WmnJXL2ib3zQnmCnOI
Ys4xF+lnx5MLHg7r8eeP1lG2udT8aV9jjPZscy+ca536yrYv744d1jXMpTa7
yeH6f2/u5l6aZc/PBj/2R6+ro0Q/X2PN7rCuPS7vJhzmmm/wd26Ic4q3oL+s
ZoyCoZ4XxL+fP+T9atZGrqoZ37jU/X5L7Gr9jY6nDcnhHG3BZK335zhf23sf
yj02/3bIVvecvfdBfJY2aJs22xbb2G3XJ/pKH+pbfW4sjJGxM6bGeosBsXG+
JmbEkhgTe2JSrO7NMcW8XJAjckdOyTU5KDfl7N4YRu6bE8wV5hBziznHXGSO
Wr63sznf8adtd5ypr1yDsB9Z9uEc1nuDze3LHGFzT2txS812E9Pl3YQD74Ud
1nUG47u3Z8/z7S+sry59+qYfdN/IC+so0f9pHSXndQ05/m3+Zj7T76Gk7tl7
ch5eM17pR3scfrea7/NeXdOfh1rv6V32Wl85vndIDt/Fc1+N88q9erVjHvfe
uEfR92HVPcdrvefeeoG2abNtsY3ddn2ir/ShvtXnxsIYGTtjaqy3GBAbYkYs
iTGxJyb39iqLbc+XC3JE7sgpuSYH5aaclctyXO6bE7ynOcTcYs4xF5mjlKW2
eVjvl3Avx966v2NU10fsR5a9xwe+WbGx31jYXtdbneMr7g3ufjO6e/ZO28MZ
3T3JronoT/uRZX/jYT0fV/odimC09yonBv1djnDh4pBX1Vwvzvv2jYfUVXpt
JX7pvviuNccVV9X6XR7fl1/8XOuxjXuf9mr4jnPcZ+j7IO6R9psn6p7jtd5z
b1ynbdpsW7qNtl2f6Ct9qG/1ubEwRsbOmBrrLQbEhpgRS2JM7InJvb2pYlvM
ywVF7sgpuSYH5aaclctyXO7vzVvNIeYWc465SDG3f1kdJbrjur3xiTnZti97
+w/rdczlm2mH9Xt81gz39g9cU7MWZ7/mvrtfr6NEtz7gurA2iwHHrp6jH1zL
8F3p5ZseV/Qfr6NE7/XfYLvnR6mfvnnIW5Ff5+83cH72IL5qSHzRfdkH1fTz
1TXbe5eaexcPtX7nzrUz84P7Cd1j4Lsevn9hDlxqI7V+f1/dc7zWe/osbdA2
be622Ebbrk/0lT7Ut/rcWBgjY2dMjfUWA2JDzIglMSb2xKRY9RyxLeblghyR
O3Jqb3+F3JSzclmOy31zgjbvjQ/1g7nIHKUN1tbcB+K6w979v7KOEn35ruCm
33HPhvN9+5G9vSiOda0J/N91lOiOe92r5t5j8/nePiv7Avsd97ecq1m70Ift
h+C55xHpy/u7Q8F8r2cFQ12HyXsfPafIeknPTTI36LlJxuW9d/rDavbL71vT
n3+h5hp0+Nv946HWeymdc/2HckIOv9/i+33uibLure45Xus970wubZtti220
7fpEX+lDfavPjYUxMnbG1FhvuSA2ztXEjFgSY2JvjwtiWGyLebkgR+SOnNrj
gtyUs3JZjst9/bA3/jG3aIO5SB+6jrl3n5M6yvb8Zc/eYb12YD533dOYGjvn
5nu50T0e+sE9IcotNffz2+9Yu3Msffeaez+Ml32c9rhHTvx0f3e5ZnyD7c8b
krj2enR81DzK2n/Xk/NORa9xP6Smn+9d0/6s7/bcIeOHnltl/atrp1fVtPku
NdtyqPWcy/fLHOf43p97nNxj8BNDcvi+hrrneK339Fna0LZps22xjbZdn+gr
fahv9bmxMEbGzpga6+2YR2yIGbEkxsSemBSrYlhsi3lFjsgdObXHfbl5riZn
5bIcl/vmBHOFOeSkZm7Z4/7e+4Ce7xzTdQf3nZqr3TOvWHu0lug+tDszlnM9
1HWNZ9X4bslhvkMR3Xcllv05m2vtX8SSPrROqK5/7B97/BwMd60yY+XuL4L5
xklqJT0n+k30n+Kc67n2c7jnPWva/GE1bXi/mvZnTuT3B+R79+MJU88FojvO
cS7pN9auq/m7Dy+p+W7y4v+N7jle6z37WdqgbdpsW2yjbdcn+kof6lt9biyM
kbEzpsZaDGyxIWb2cNX2bzEpVsWw2BbzckGOeK2cujNjHjkrl+W4Yk4wV5hD
zC36RxvMUfrKfO59ztXMq8vev8N6Xrl8+/GwXvtwPuvY1Vqcz93LV55vXdH6
wMUa36g/rNcXlu+zbfoF10fsa8SbtllD0M7uHy/X9PnlmrEOtrufyt6XHmOH
Cz0e+C30l3DOE7n2S2ry95Nq2vaRyN1qxjRzou6vr6413x13+a6K+7t6zHCo
Gffo7oPyPcFlT85G73O8dntPn9U2aJs22xbbaNv1ib7Sh/pWnxsLY2TsjKmx
FgNiY4sZsSTGxJ6YFKtiWGyL+Ys1uSBH5I6c2stX2ub5clmOy31zgrnCHHKu
Zm4x5/jcjunlWtfZHHddU7MWZ+1L2/be872l5rh0+R7XYT1mdnx7UrNP2ctX
7gOxVul3OZZvd2/s8R1V10e+uo6ytc1+Z+/ditPqDJdr2p/5QmPy9prnh8Pd
X6Qe3dh+LfoF9O/j/Ky7NP7vUzOO9+C5H87fP5Dn3rXWfHe/d/s2+aHX3aJ3
3SA5xO9FdD0ner8nGL33Im51z/HavqfP0gZt02bbYhttuz7RV/pQ3+pzY2GM
jJ0xNdZiQGxsMSOWxJjYE5NiVQyLbTEvF+SI3JFTd6YWJGe1TY7Lfe0xV5hD
zC3mnM5F5qj42TGt7y5dW/P7ru7d+r/qKNHd59brvNG/v8Z34zf3cV5vjBzv
7a0dWJ/0Ox6OM5c6ySa3++68a8qPrqNs/WZ9QPyY/3ucf7lmTKM3F+KX5kjq
tI2Bd6C/uWZ7X4v+MvQbOf+h6J/L/T+B534U8pdr9vsfUGu+9/wleaDnO6lX
Nw6TK3otLznkW4dE7z1L0XsdKnrXXbe653ht39NnaYO2abNtsY22XZ/oK32o
b/W5sTBGxs6YGmsxIDbEzBZLYkzsiUmxKobFtpiXC3JE7sipvT0AclPOyuVr
a3Jc7psTzBXmEHNL38dcZI7KmLa/45Fzeu9E9OXbQYf1N3bcz+/3KNyHtuxn
Pqy/Obb3/t3ee1j60PqAMXUPcPB349Av1hwPK+7f890lx9jLPurNs1zj6L4m
vmruxLfOAU+GBNsd99QUup/9g5ptD0e6j/4F/v5s9Ou4NvOa5ua9a/oq85bm
Re7VGPuQWvO95xqZT/U6XerVPV9Irmg8JIf0WlhyS39nKXr3m1fVfN9tq3tO
X+s9fZY2aJs22xbbaNv1ib7Sh/pWnxsLY2TsjKmxFgNiYzs+EUtiTOyJSbEq
hpWLNTEvF+SIz5JTe3vm5ebe+49yXO6bE8wV5pDOLeYcc5E56raa94+fex9p
zl++B7LJ4a5dmifNb19e43fTDutvi+kf63jO5dU9x7mq93xAzT1mrkf4rQzf
G7UOaR52f7JrLr5b2jk//ul+P3qPq+PD7jczX2isJjbNo7fVbFf2ETZOUjO5
+5CXoz+Pcx7PtQ/knp/Fsz4RGzKG7j40/aV8v8+QzKd67Sb16t43nvFMj/OT
Q3oPefReb0qeed2Qq2vWT7Z6n+O13tNnaYO2abNtsY22XZ/oK32ob/W5sTBG
xs6YGmsxIDbEjFjaYkzsiUmxKobFtpiXC3JE7sip95SPctl7yn1zgrnCHNK5
xZxjLjJHZT7etf34vNclL9X0w+WN7j608zXfh9LP5ltj5HpH42Gbz+9Mvrqm
5jzX9Q73wpnb3YesuN/P72M4Ru13ddP2LxoSvcfV8VWPjdMXdL8ZfzWP4ufu
W/+4ph/yewCn1dt/Cf1m9Kdwbcb3ve8o3yNo/GRc0WvoH1vT//G1fO/3iDOf
6vX6fK+g9yZ9QE1/Joc8a0jGPL3HJnmm16TuWnPf4Fbvc7zWe/osbdA2bbYt
ttG26xN9pQ/1rT43FsbI2BlTYy0GxIaYEUtibIs9MSlWFbEt5uWCHJE7dyZf
7a3HyWU5LvfNCefrKOYQc4s5x1xkjsp+st6H/K6a9bH4vPPepc39e+9fdN9/
1CfmW9cU9r7NddpaW/S9tYDlXYzD+ptgjp/9RpC1PvfiLmtYB77XdJi/s5k2
uifQ3NXtjX96DBA/91pt/PlpQ+LHHg9kb0rPx99U0z+JTWP+Fej/vGYf/TTO
j997TJ6+s9ea8x5pc+STsafX8iPBXPMlWPyqIdkD8M1D/lLNPYHJIT82JGOe
7geTZ/p3l96/pm/VPcdrvafP0gZt02bbYhttuz7RV/pQ3+pzY2GMjJ0xNdZi
QGyImS2WGmNib4tJsSqGxbaYlwtyRO7Iqb1vF+ztK5DLclzu297OFeYQc4s5
x1xkjnpLzTpYzutcnRzXuMp9eu6c864fknv12C+67zm6T8P3j8zhez7Z21uu
b631uT779TV/E9y63/JbIYf1N6WVfs8rbem1lehdH47e76Zdqvms+OfaIfFb
r/Peik/i864bxO/nhryxZluyR7H32IQ7vb/xhfz9GTV58Wjuk3dQmyPZ79j9
eHDT/V041rWR1Hp632DmOb2HM/Ho99qy1tZj8nwruN/nyrysMZY80++n3K0m
ltQ9x2u9p8/SBm3TZttiG227PtFX+lDf6nNjYYyMnTE11mJAbIgZsbTFmOt6
e++limGxLeblghyRO3Jqb4/93lhCLsvx5r45wVxhDjG3mHPMReao9B2dnzPW
7XWH+L/rdekvrhuS+zdmcr21+s7b2zm79caTOsp2zLm3HrGXrxpv2/Hn3jt9
vjvZ4+rL+GE75/2dIdEvDMl5PX+JH7pPjH8eOiT9Qo8H4vMvGRLfNoYTg+5z
8z5tf1vj1TXXxcKdfvfqp9Hz7OZjfp+g50cPIRap+/Q+7Tyj193ujW3pC3vN
KGOVRw5J/9r7D1Ob7r4+cfoXQxKnHqsnzzR3Phx/bvU+x2u9p8/SBm3TZtti
G227PtFX+lDf6nNjYYyMnTE11mJAbIgZsSTGttgTk66vOTcU23vvKsoRubOX
r/bW6+XsSR1Fjm+53znBXGEOMbeYc8xF5qj45UeGpP/rMXn6jq5x5T69lzX3
730yeW7XEGJPj7djp/XGHmderumHy7Xes7S3b9z1COt77vV1P4nv7vX6b/T+
LbzoPcbusebba+4RiaQtvZYR/TVD0t7epx0/3DQkeOqaQ3zVa/S5b4/530qM
Mi/qder4v8cJ+QZKj9vTXzbvXoT+Q5zzZK7NOLs5m9py78fL3Km/ARJ+9lgl
eO0+7hNr1kU/hnZlrN/1k/Sj3ffdvSanPgpfdVwvb3TP8Vrv6bO0Qdu02bbY
RtuuT/SVPtS3+txYGCNjZ0yNtRgQG2JGLIkxsScmt1gVw2LbPQ++dyNH5I6c
kmvNQbm55axcluPNfXOCucIcYm4x55iLzFHx0XVDfrfmGDKx6DWarEN0bkzf
0fPTPNd3EzpXx86e20a/e83vMbovpceZ0d1rd1Izb/d9Ltf07eVafzPNMee1
NfdvOF923bP3Wl+u+Y2mSzVrApeQ/L/fgem5THDY+4rj8x7Pp5/tvXDpI7rm
mXlE1xP088Wa/WDu2/3yy4nji9Ez528cZl2t9xBm/NE16m9AUsPpWtDnY0/G
EL1+dC/8E2z1foP4+ueHfDRtvyd+u0fNmkPv87680T3Ha72nz9IGbdNm22Ib
bbs+0Vf6UN/qc2NhjIydMTXWYkBsiBmxJMYu1Z/F3x21xuoWw2JbzF9bkwty
RO7IKbl2UpODp+2DlctyXO6bE8wV5hBzizlHP5uj8h2N7o9eV7OemTlz5+SM
4zs3JhY9P00suiaZWPQemNjW+TY2dx5OWzpXX6r1XjvzmPnNcaa+7TpD9B6r
R+/+NHqvj+f+vlPfayjbsVP3X13HjNxec+01bex3DdL2/v5PfNJ9ZfrF7gvi
/64nxJ89Vs/8orGddajGWH6zqvGfdeueR4RTPZb4Ef6eunH/Nl/mGN2n5/oe
h2T/yuOHZK7SPMo7uuaB7qPP1eRU1nt6DfqThq2Rj69ZV0k8ug7ce5Evb3TP
8Vrv6bO0Qdu02bbYRtuuT/SVPtS3+txYGCNjZ0yNtRgQG2JGLIkxsScmt2Mw
MdzYFvNbLsgRuSOn5Jr5x7x02v5YOS73zQnmCnOIucWcYy4yR2UM3O8lZc7e
8/fE5dyQ5P3OjX/Mc5Mbe03TvSjvor23097oPZ68Y6e98UPvdYne68iXeO4l
fB7dfQW99h29+9ncv8fk0fv7q9F7PhLbfm9Ir6c2jho/t9bc9/KOmt9zC54a
h/H/S4dkTNvjhOCy5wIZ3/Z3P4K/HjMkLv27mek7u+YcXD5myHPRb+CctK1/
S+URNcfS6Z/6/ZeMG7pmcn/8cL+a+I8f+xvC6f96DJBxTu9Vjt/fMuRTanIt
+Ogcpe45Xus9fZY2aJs22xbbaNv1ib7Sh/pWnxsLY2TsjKmxFgNiQ8yIJTEm
9sSkWN1iuLEt5rdckCNyR07Jteag3JSzclmOy31zgrnCHGJuMeeYi8xRF2ru
x0iMeqz42ppzz6yPdB5+M89NXHrO+ye08Z20MTb3eNL3F27j72lvz5e3Y7PO
z9G7jndHrX9fqeui0Tsn5z49F4jemNzWnW4ZEj//9pDY3P1a2tK8670dl8a/
9bsP8UPXPzNH6N/3TK2j9xolFl2fyTN6zJA6Sb8Hmu9rP2NIrun+OpxqjuSa
3kOYcUbzNPWZG4ak7z8/JOOJXjtOfac5lTlU12nvj52pKfzqkNSme/wT3PR4
4NNp+31q1oSvwVfqnuO13tNnaYO2abNtsY22XZ/oK32ob/W5sTBGxs6YGmsx
IDbEjFgSY2JPTIpVMbzFdmNeLmw5Infk1GnzI7kpZ+WyHJf75gRzhTnE3GLO
MRddqJmjsj7Se9uSx3ofRWLUc8/Xcf+uWdxjPLfXNP+IdmU83Pk2Nncefif6
rbXOySdDondOvo3n3oZvb6Mt0TsPR+8aafTez597Xjcker8remvN32GMbV3D
DJZ6zvInNbnW7xJE3lqzDww++5s/wW3jM7Ho76T9Ts1156xz9NpHnteYz77k
5mDqOM2L2PecIfn/5uaN/P2pnJ950AuGZA7Te+2yrvazQ+KXroEkD/zKkPTH
rx7yBbTlfjX3JHxOzbHBZ+KHfictou45Xus9fZY2aJs22xbbaNv1ib7Sh/pW
nxsLY2TsjKmxFgNiQ8yIJTEm9sSkWBXDYlvMb7kgR+SOnGquyUG5KWflshyX
++YEc4U5xNxizjEXmaNu5pwLNfdRpH/p8WFw07nxdzbP7bbEv71nI2P9HjfG
5s7Db+PvvoeVGPUY8p08K3qvNbwT+2/F57cSi5zT6zvRu14avesYeVb/Jnhs
eP6Q9GevGPK2mmtYf1Szv3MfSPzfePv9mvOdN9ac72Se0+P/9B1df/iNmhzM
GKLfs3h5zXXtzCW6hhOe9Lii81ckc6GeK2Ws0O+5ZP7TnEodp/cMpBbT/HpQ
zTHMA4dNkeSH1w1JjajnI+mnm3Pt30jq3l0T7u8/RtQ9x2u9p8/SBm3TZtti
G227PtFX+lDf6nNjYYyMnTE11mJAbIgZsSTGxJ6YFKtiWGxvMd9ckCNyR05t
udYclJtyVi7LcblvTjBXmEPMLeac+L1zkTkqfUyPAxPLrrmldtA59lXc/zU8
N3jqsV/etegxYTDX+fYP0N/COX/EtRkf+35W596348O349u34/O3E4vovR8m
eq/15J5da43ev+v0xzX72XCmx9tvqYnDN9dcu4n/G3v9fZ9I2t7YC26bj6+t
idvE5Y1Dfq3mN0BSj+h+OeOG5kLGDf0uWL4z3nsIn1eztvMs9Kdxzndx7WO4
58Nr9tcPpY1fV+v80OtQX1Fz/PMPa3Kq9xhFuj4SuT8+Ufccr/WePksbtE2b
bYtttO36RF/pQ32rz42FMTJ2xtRYiwGxIWbEkhgTe2JSrIphsS3mt1yQI3JH
TjXX5KDclLNyWY7LfXOCucIcYm4x55iLzFHnOSdj6J63/hx2/hL3/1c8N/Hq
sV/m2J1XMw7ufPtG/v57nJ94nTZW/EPsjE86x0bv98Leis/fSiyi9z7enN/7
aaP3uk/u33XUPLdrtvF598W/X7NG+qaa7769sSa/0sbGYWLROAxueyyRuDRu
f70mnn+lZt0mvu3+NPOR3kuZPr7HFf+8Zi06teKujdyEnppL8ze15ubXt9ec
W2Uc0jWib+G5D651fmjuZF7R+zx7/2skuaXnLw/AD9fiB3XP8Vrv6bO0Qdu0
2bbYRtuuT/SVPtS3+txYGCNjZ0yNtRgQG2JGLIkxsScmxaoYFttiXi5sOdLc
kVNyTQ7KTTkrl+W43DcnmCvMIeYWc4656HzNHPUDnJP11x77vRDbLmzu3899
Ffb8G+xMjDvHXqyZY9Pv9PjwDdj5Rmx7E76KT/p90uj93lZi2nt0f59Y/D4x
6nnFA4be+6Jz/34PIhzommps+Mkhsa372dfX/EbH62ris2ugt432dr00WG98
hiddr8icormZ+/Y8KP141yteWrO/flHNtbMfx4Zwrfv3m9B/AP2pnJ/5UXPt
0TXnLI+oybvMrcwPnTcy/+paU2q4tw5Jbmk+fjVt/yraru45Xus9fZY2aJs2
2xbbaNv1ib7Sh/pWnxsLY2TsjKmxFgNiQ8yIJTEm9sSkWBXDYlvMy4UtR5o7
ckquyUG5KWflcuxujst9c4K5whxyoWZuMeeYi8xR38M5N3Ht87HnRdz/Fp77
cmx7Jbb9Wq3zWF/7Gu7/W9iTOHb+TBx7Dhv89bz19fj29fj89cQieu+bjX7t
kNyn13mbt48ez+r1oNjQddrguOsV8X+/CxD7G7fBZGM1fXn3m2l7YzV9cWM1
+Gs8/yLX9lg/kj06/V5nrzlFej/Em0Ycm0c3od+Anpj2mCR7g7ou9Djsv55n
pV5tfuh+P3uSeswQ3zXXMv7pnPO1tPFraLu653it9/RZ2qBt2mxbbKNt1yf6
Sh/qW31uLIyRsTOmxloMiA0xI5bEmNgTk2JVDIttMb/lghyRO80puSYH5aac
lctyXO6bE8wV5hBziznHXGSOejLn3FCnf8fs5s39+7kvxZ6X8fdf4vxfrvV4
rJ+bdnXOzLi6a25pe89VfwMfZjzZa52vxuevIRavIUavJnbRe69s7tnvo0Xv
tew8t78VEIx1/fZf1/y94NjcXEhb+r2JYLVxm/2OjdvUSRu3mZs3bjPXaJ5m
DNw8Tc23eZp5SvM0cey+OOtYXee5Cf0G9O9FfxLXZv7QY4NHY/91Ncc2Xbft
vNE1lofSxgfTlm+gvf3dkMhW73O81nv6LG3QNm22LbbRtusTfaUP9a0+NxbG
yNgZU2MtBsSGmBFLYkzsiUmxKobFtpiXC3JE7sgpuSYH5aaclctyXO6bE8wV
5hBziznHXGSO+k7OyXpKj+tu5D4/vLl/P/eF6D+zsa3v8y94VuJ7MiT5tufF
r8AniXWvb8YPvTfjl/Ft8va9hrySWLySGEXvvf259kFDol8/5Jdr7sPJc3st
O3rXOf9lzfpG7Ow6Q+LS2E5bmmtpY2M18WoM38Lfg//GefzW+E//3vOjzFO6
L86+wOZO11sjqWV0v/909H/KOU/h2uwpbO4/jvs/kud+W63zRnPtm7H5n9CW
h9LGO6N7rff0WdqgbdpsW2yjbdcn+kof6lt9biyMkbEzpsZaDIgNMSOWxJjY
E5NidYvhxraY33JBjjR35JRck4NyU87KZTku980J5gpziLnFnGMuMkd9O+c8
Bf37uc957v9s9Odxzgu49ie5f9aMe1z3Ytr4EtqedZlej/g5fPVSfHgB317A
5xeIxQVidIHYXSCm0TvW0b93yIWa2Ih+C9JYit41k+jdv0dvbOfaxnPa0hh+
CXri2HiOf3p+lP69OZuYNs6fy/3j/+7rn4UebjbXvo+/fzf6E7nP47l/1tea
j9djwyOw7WG061tqnXO6Xd+0o3uO13pPn6UN2qbNtsU22nZ9oq/0ob7V58bC
GBk7Y2qsL9Q6p/X9t5gRS42xCzVxF11MilUxLLbFvFyQI3KnOSXX5KDclLNy
WY7LfXOCucIcch7dnGMuMkc9hnOegP5U7vM09Geg38T5eee98+FzsPl5tCV9
Wc9/Mw7vOt7N+ORmfPUCfPgCfPsCfP4CYnEzMbqZ2L2AmEbvWKc/7T7rJ2qO
vePzHpOnX+j1psSl+8SMaZ07NCajN1ajN4ZfhP5T6OFF4/8FXBs7miOptzR3
4ufm73n0p6N/H/pT0b+L+3wH+uN41qOw4fpa5xPzTOvfSlvujO613tNnaYO2
abNtsY22XZ/oK32ob/W5sTBGxs6Yvniji4HGhpgRS2JM7IlJsSqGxbaYlwty
ZMsdOdVck4NyU87KZTku980J5gpziLnFnGMuMkc9Cv3xnP9E9O9G/17Oz7M6
792AbcFJjw+fWXOeeyNtvJG234hPbsRXN+LDZ+LbZ+LzZxCLZxCjZxC7ZxDT
XNtz//M139G4qeZ+5vi59+fE/70fJjHqum7O6zlFcNX9cvDWWI3eOI9ujmr9
xznn+ejP4z7PQc97/s2p8+hPR38a+lPRn4T+BO757eiPwYZHol+H/m3Y//D3
UPfa63aepQ3aps22xTbadn2ir/ShvtXnxsIYGbuf3OjGuu8jNsSMWBJjYk9M
nq+JVTEstreYlwvNEbkjp+TaloNyU842l+W43DcnmCvMIeYWc465yBz1SP7+
mJ3zv3Nzz772SdjwZGx7CjY/hbY8hTY+mbY/CZ98F756Ij58Ar5NP9s1w9j5
GUOif96QxxO76F2TzLW9tvKEmu+XxZ5eO44/+1sB+beLQzKn6DWjzLt7rhGc
dx8tztNnmZda9+83o/8Y1z4X/YfRfxD9PPrT0Z+G/j3oT0Z/Avrj0R+L3t9R
ijwS/Tp0888jdvRtjjrtnj5LG7RNm22LbbTt+kRf6UN9q8+NhTG6MzHd9juN
DTEjlsSY2BOTYlUMi+0t5uVCc0TuyCm5JgflppyVy1uOWyfvnGCuMIeYW8w5
5iJz1PW1zmOtPxr9sRu9n/VYbHgstj0Gmx9NWx5JG/Pcngt/Gz55GL5KraPX
LFIP6bXXb8TnDyUWDyFGDyZ20XvN96E1966k9nvjkDy39+bFzt5nGH/2enfG
4l3jzTWNw8zH7aPFvBg+Tfec56H/CPqz0X8Q/cY6nZvbfNXnPxn9iejfgf44
9MegPwr9evTr0L9tR79u59pH7TxLG7RNm22LbdzLV/pKH+pbfW4s/mPi2M8S
G2JGLIkxsScmxaoYFttiXi7IEbkjp+SaHJSbclYuy3G5b07Y5gpzyGk5x1xk
jrpu5+9753wbz3oENjwM29L2nsPGzz23jQ9PhnwDbc+enF6DyHv3vZb6QHz4
lfg2+3d7j+61Nb+hnTjdb8iXEbsvI6b5//4m7QNqvnef/Y23DMl+od8eklpx
rx8FV4297GE7Df/PQf9R9B/b0T3nPyZftf509Kehy+WnoP9/yVeOi/Zy13U7
5zxy557vab6yLdt8dZpP9NV7mq/e0zg+Z+dZYkYsiTGxJybFqhgW22JeLlxb
kyNyR07JNTkoN+WsXJbjct+cYK4wh1xX71n+eU/PP8tXZ/nqLF+9+zie5av/
dPnKv5/NB8/mg2fzwT8bi7P54HvPfPCs3n5Wbz+rt5/V2/+81NvP9jOc7Wc4
289wtp9Brm05+N60n+Fsv+jZftGz/aJn+0X/vOwXPXsf5+x9nLP3cc7ex5Fr
clBuvje8j3P2vvPZ+85n7zufve/85+V9Z7/VcAP6sznn7HsyZ9+TOfuezNn3
ZN4bvidz9r2+s+/1ZV529r2+s+/1NdfkoNyUs3L5/8/v9WUP29n3kM++h3z2
PeSz7yH/efge8vk6+72JyNnvTZz93oQ+NxZnvzfx3vN7E6l3nf2e19nveb2u
zn7PS5+f/Z7Xe+fveWX+3vPWC3X2e6mvqLPfS+38kNzSdaEvqrPfS/3FTeyM
qbEWA2JDzIglMSb2xKRYFcNie4v55oIckTtyasu15qDclLNyWY7L/f8Uv5ea
mP2NOvs9+tjW/fU76uz36CNfUGe/R3/zJkbGzpgaazEgNsSMWBJjYk9MilUx
LLbF/JYLckTuyKnmmhyUm3JWLstxuW9OMFeYQ8wt5hxzkTnqwhW5ZsiraEv6
jl7rTCx6b8abuX/s6dwYm3vcmLb02C9t7DFhfNJ5+Db+fjvtit7z1vTHPQ68
A99G/5tDov+dIdG/aEju8zVDet/hY4Z+05DY0OPtxOW3h8Tm7u/Slq6XBkvd
J+bfes0ofhCfXRsJbl82JP1uYz7PaF4kRv3effqXrqnmml4Pypig17JzTe/D
eWbNdzSy5nvDkOwPPz8k84TmZt67bw4mD/S44v7YmXFRj2E+u2aNKFjvus2n
0/b71OTaNfhK3XO81nv6LG3QNm22LbbRtusTfaUP9a0+NxbGyNgZU2MtBsSG
mBFLYkzsiUmxKoa32G7My4UtR+SOnGquyUG5KWflshyX++YEc4U5xNxizjEX
XaiZo5LHPndI+rPOw4nFuSGZn37ykPj6rw/5E54bm83JPYZMezvfRu88HP/0
uPEO2niJNkbv8eElnnsJn0f/u0Oi/4Oa9ZJe5839nzAk+o8Oif7LQ2Jbz0du
q9kPpi3dD95ac2zwjprj/2Cra6SJy0uHJC7NqXCyMZZ+uTmSucSThiRGXUfN
GOJxQ8KpxuRz0W/gnLTtO4dkHe0pQ7Ivp3maukbXbO+PH+5Xc64UP3Zt5741
a9HZX9hrZ/F7z3c+peZ8J/ho3ql7jtd6T5+lDdqmzbbFNtp2faKv9KG+1efG
whgZO2NqrMWA2BAzYkmMiT0xKVa3GG5si/ktF+SI3JFTcq05KDflrFyW43Lf
nGCuMIeYW8w55iJzVOLyFUMyNuucnPHo5wyJ/+8zJD7/lCG38tz4+bT8vM1F
3d7onZ8v0d7LtDd619yidx6+jM+jf+aQ6F86JPo3Dsn9v2dI9MZ/9K4/xM7u
16I3fjpnvWu0sccDabs4fMmQjId/ZEjG/91f9z6hJ464PGpIxhMPG5L++JuG
pFbS6z4ZTzxkyI/w9/Cza62ZB3UdI9dfPyRzg8cPSY3ou4fkvf1nDgkmesxw
rmYN51Nrfivgk2qOfz6+5pwl8eh61MfVzFHqnuO13tNnaYO2abNtsY22XZ/o
K32ob/W5sTBGxs6YGmsxIDbEjFgSY2JPTIrVLYYb22J+ywU5InfklFxrDsrN
LWdPG2/IfXOCucIcYm4x55iLzFFp/3VDMq/svbLxf+/t71rD/YZP7zsktnVu
jM0990xben56qdb5x7zUbY/e48noJ0Muc5+tb3usGP2zhkS/dkj0xlj0xnn0
roFcrrkOdalmTfgSkv/vtZu0t8cAiUVzKrH4Z0MSi+ZR+pHGduYIjX/9fPGK
/KMhuW/XS1Pv7bWeF6NnzvOVQ7KX+6uHZK9zzwW+Ackc6puHfD72JA/0mOFe
+Cdj8R4nxNc/P+Sjafs98ds9au5J+Gs1c5S653it9/RZ2qBt2mxbbKNt1yf6
Sh/qW31uLIyRsTOmxloMiA0xI5bE2KX6s/i7o9ZY3WJYbIv5a2tyQY7IHTkl
105qclBuylnzmPmt72NOMFeYQ8wt5hz9bI7K+Lb7goxpeyydnP4tQ24ljnlu
75uNPZ0bY2fPPdOWXmuI3vk2eufh6D2ejN65Ov1Kz5ej95w3evs2evs8eufk
6I3D3LP3AEfvPjp61zAv1+y/Lo/rI2lLvzexHYN1X7mtg/WYPL7qPj33/boh
b61pW2odXzYk/r//kNRJukaaMXCv77wI/Yc458lcG4x2f5o1te6P0j/94yGZ
D/RYIvsFexzyieNekY+hXcFc12dSJ+35yN1rrmH1+srbiOvlje45Xus9fZY2
aJs22xbbaNv1ib7Sh/pWnxsLY2TsjKmxFgNiQ8yIpe3YqX0lJrdYFcNiuzEv
F7YckTtySq41B+XmlrNyWY43980J5gpziLnFnGMuMkdlPNrvAqSP6LWP3OfG
mt9EvG7IHcQo9nRujJ33HhK963iOM+OHkzpK9M7b0dsP0dsP0Xv+G71zdfTe
oxu9bYje/WD0HqtH7zpt79v7t8O27seid034ck1ORb9Q8537rpnED/1N2vin
5xHpO7pfTj/7JUPi294Pk3707w3J/KLH6pmzd100a9g9f/9p9Dy7axGprXz2
kIcQi/RPzbs8ozl7b2zLukvvi05Nod9HC+Z6fhG8PndI4tR7BhKn3kv54TX3
eX44/tzqfY7Xek+fpQ3aps22xTbadn2ir/ShvtXnxsIYGTtjaqzFgNgQM2JJ
jG2xJybbn2J4i20xLxfkiNyRU3JNDspNOXtScwzmXFLud04wV5hDzC3mHHOR
Ocp9IO+oOS6Nj6zvPbvmt127j4s9vZ/WMWfa0vPT6L2PInqvaUbv8WT0HmdG
7z0b7y5fZZ5wr6H3Pt7oPYaM3vWE6L3uHL25032Xe/j+Xc336l8+9B7PR+/3
IC7VfFb8c+2Q+K0xfCs+Sd/xt4ZkHnFuyBtrtiX9a/d94U7XGV7I37Ne1Xh7
NPfJ2OLTh4SrzceMuRsnqRf1OCE1hcZM6qo9Vkk8uuacPNNzjQ+tyYsPqTmX
+Us19yTcrSaW1D3Ha72nz9IGbdNm22Ibbbs+0Vf6UN/qc2NhjIydMTXWYkBs
iBmxtMVYY09MbrEqhsW2mJcLckTu7OUrOSg35axcluPN/e08tHOFOcTcYs4x
F5mj3l7zvc531RzD31Gz5pDzGmP5t147dr4cO7+4jhK956fRex9F9J7bbnP4
nk96nLn1beft6D1uj971hOhtZ/TOz13H7Lpm14Gj9xg7evf1aWP3idF7zH+5
Znvjn+5z0180R+LP5kX82PP3zCP6va031fRPxhxdk3wFet4Z6T7raZwfv/d4
OzWcxknGAY3JT8aejFV6XpB5WdczM9bv8cCH1awFJc/0mPyDa34r4ANr4uoD
ar4j/P41favuOV7rPX2WNmibNtsW22jb9Ym+0of6Vp8bC2Nk7IypsRYDYkPM
bLHUGBN7W0yKVTEstsW8XJAjckdOybW9sYSclctyXO7bXmtrbae5xZxjLjJH
pf291+72mmP13MexqHrfP/acr6NE7zrndvyZtZ9PHXqvaUYP/u4+dOtXjjnN
4fr2mjrKNk92zTN6r6FE71y97accV3f9JHrXHKKLq+ZLdNeImyPxee9ti78a
8/Fzv0/6xzX9kLXqXkf+LfRfQr8Z/Slcm3lR92vBYtcigsV+L+xja/o/vu4a
Qsb3PT/6yzXXvpNDukb9ATX9+X415y/vW3Ov0V1rvvt215o82up9jtd6T5+l
DdqmzbbFNtp2faKv9KG+1efGwhgZO2NqrMWA2BAzYkmMbbEnJsWqIrbFvFyQ
I3JHTsk1OSg35axcluNy35xwvuZ39B3/dG4x55iLzFH5t65x5XzXGrofzLM6
70XvtdfovQYdveui0Xvst8231vf2/HNn8pX3TD/6gKH3+lH0fhcsevsheu97
id7vWUTvPYRbP39hHSX6Z9RRLteMkXP2+LDrjfF5z+vdi/K2Wr9D2ntdwpHG
w8vRn8c5j+faB3LPz+JZn4gN6S+bR+F7jwcyL2u+Z8zTY4DkkF6jSZ7p9fH3
qfl9y+j9nsvVNd+dubrmvqGt3ud4rff0WdqgbdpsW2yjbdcn+kof6lt9biyM
kbEzpsZaDGzrOfK6sbTFmNgTk2JVDIttMS8X5IjcuTP5ao+Pctl7yn1zgrnC
HNK5xZxjLjJHpS+wZt77+S/X+ln66tYRq+i9lrHN7X3/bb7tMXl01yMcf5rP
Gydb/3TdIHqw+MChd70u+sU6yrY/+u06SvSeO2/Hz10b2T6r+83o3bdab7xU
M6bx7cmQ+LzH0r6H9Qc12/7amtj4Bf7ud8yu49rUdnsd5941fZVxQPMo9+o9
MB9S04cZzzTfM+ZpvCWHdK0jeabx8BdqzlmuqrnfIHrPR66qyZ2t7jl9rff0
WdqgbdpsW2yjbdcn+kof6lt9biyMkbEzpsZaDGzXmxozWyyJMbEnJsWqGFYu
1sS8XJAjPktOyTU5KDflrFyW43LfnGCuMId0bjHnmIvMUfFtz8dzjjn/2prf
BXGMap50zaL3Kke37ud9rPU1xqLvrUfow+5Do/e4OnqvGUX/iTpK9H73YTs+
7HWH6K7/9jr1NqbGpfvN6M2L+O1kyOVa70vpuPv+wjvQ31yzva9Ffxn6jZz/
UPTP5f6fwHM/Csl8qrkTvjd3Mp5prCZXdL+fHNLrYsktve52l5pr9NG7Phy9
x/bRu9/c6p7jtX1Pn6UN2qbNtsU22nZ9oq/0ob7V58bCGBk7Y2qsxYDYEDNb
LIkxsScmxaoYFttiXi7IEbkjp+Ta3nq9nJXL19bkuNw3J5grzCHmlr6PuWib
o/Zq2tfUnOfqt94LF91aX9fHot9SR4ne4/zojcNtX2NMre/pwx5XR3c/ySNq
/t6dayvaE/y9duj9rmj03kO4tU38GCPtcQ3FfSliuPvo22uen/l4YyA1xObX
a9EvoH8f56e/aS5kXtQcuQfP/XD+/oE896417U8eaL5fVdO3ySFdj43e+2TS
fT9oSPR+/yV6j+Gjd7+51T3Ha/uePksbtE2bbYtttO36RF/pQ32rz42FMTJ2
xtRYi4Et98WMWBJjYk9MilUxLLbFvFyQI3JHTmmP9p/U5Kac1TY5Lve1x1xh
DjG3mHM6F21z1F49zfx/ro4SvffjRe/3QLe5vd812NrW67zRXVPwuXv5yvNd
n3XPycWa4+Gun0TvfcXRHWP3e0nRe66xxY+2OZc/bU0zvm1sR3du2GPpcLh5
l7Xa02q8L+GcJ3LtlxC7T6pp20cid6sZ08ynmi9Xc85VNX17l5rtTQ5pvkRv
TB5qxj360ifW3GudY1nD2uh9jtdu7+mz2gZt02bbYhttuz7RV/pQ3+pzY2GM
jJ0xNdZiQGxsMSOWxJjYE5NiVQyLbTF/sSYX5IjckVN7+UrbPF8uy3G5b04w
V5hDztXMLeYcn2uO2ttXoM+9T6/VRnecueD2sJau7UfvvazR3aNlHW8vX9nv
uN7xrDpKdOuQvSYVvfcfbq91r6/rHSc1+xR9tbcP33F+4y247flLsN1j2syR
GofhQs93fhP9pzjneq79HO55z5o2f1hNG96vpv3hdfPiqpq+TR5oLidMzd/o
Sx9ajPmLmnDNPX45ep92Ff7f6J7jtd6zn6UN2qbNtsU22nZ9oq/0ob7V58bC
GBk7Y2qstzlHbIiZPVy1/VtMilUxLLbFvFyQI14rp/byldyUs3JZjivmBHOF
OcTcon/29lE4B7wzY0LP77Wb6K5H9Pd+ove65LZfOFczr1rH28tX9jW+a9Dr
TVtf3VJzXOqai/UBx7fmc8fw9o/aI5Z6DT2669c9Zsh84fOGBPPN0/io1+uz
R7FrralHu0+7/XzvmvYnfr2/8YNqju1Tr+7xwHYc1W05FH4u8FDsqynwXMwj
au7hzLHUSYqa6kb3HK/1nj5LG9o2bbYtttG26xN9pQ/1rT43FsbI2BlTYy0G
ttgQM2JJjIk9MSlWxbDYFvOKHJE7cmqP+3LzXE3OymU5LvfNCeYKc8hJzdyy
x33P77pB9O4vtnl17z6uceiH3mMT3bXL3hMY3fi6hmK8tMH+xVzd30mL3u9Y
Re/vfkR37Cp+vP9encH+0fqksdOHvsPYMQ22ey0sddruExPXruf8Rs3166wD
9bwpfVjXH9J/996kjB+aR+9b05+pV3c/GF73vqND0d5iPlL070V8az3WWtas
i/WmYo9ise96o3uO13pPn6UN2tY22xbbaNv1ib7Sh/pWnxsLY2TsjKmx3nJB
bJyriRmxtLe3fI8LYlhsi3m5IEfkjpza44LclLNyWY7Lff1grrgzYzlzkT50
Xmnta+/+tsU9rv3uVXTns+7FfUEdZWuD42fvb53tmpq1OPfCmc/dy2q/Y3+n
zdYQ7OM8Rz9Yq9x7V/HH6yjRf7WOEjz3fr9gvt9P8duTv87fb+D8YPFVQ+KL
nxzyQTX9fHXN9t6lJoYPtZ7T3ZmcsPih1uOcZe9QzbW8HE8fkqP3IW91z/Fa
7+mztOE/lEtto23XJ/pKH+pbfW4sjJGxM6bGeouBvXf6xJIYE3tiUqx6jtgW
83JBjsgdOSXX5KDclLNyWY7LfXOCNu/tAdAP5iJzlOK7kw+oucfMfO4c077A
WlzvnYj+upq/E9Lf/4m+4Pawfv/Ittjv2HbXR5Tn1FGiW1d0LK3NYsB+pOcO
0btfjt7vtUV3TXnvPa/ud4Jtv0vT67zB0MUh4cwbhgTfbxmS8UfjIX7pdeGs
kb1xyFU1Y5dH97tpVev8sLyLUdQNipjWep5ofljWkgrcFv1s4YeN7jle6z19
ljZomzbblm6jbdcn+kof6lt9biyMkbEzptsxhiI2xIxYEmNiT0yKVTEstsW8
XFDkjpySa3JQbspZuSzH5b45QZvNIeYWc465SOl9pNFdX/jZOsrWn9pvndD2
+u5z92XRGw/R+xsg0a1Vek9zsusj7kNz3PvwOkp0x7p740Pn5p5vLdHc7tqH
a7Xu37P22HOKnNf9Wvzbe+rChZ5HZF9yz03Shp6bJN69ZynrX71OfXVNfx4K
e2r+XnCO3kOeo/eNV9Heghe1nicuGCtqL8W8oOb3OXP87pCt7jle6z19ljZo
mzbbFtvYbdcn+kof6lt9biyMkbEzpsZ6iwGxIWbEkhgTe2JSrO6NTzxfLsgR
uSOn5JoclJtyVi7LcblvTvCe5hBziznHXGSOsnbn3i3P7++vRu9va0Tfe0fS
cZ39iNLvP0Z3H4ix6NppdNc13MuRb22/8pS2u+6wt2d+mY8c1nuAjan73OxH
ztd8d9KawBfVfC+suRmMnhuSvrxx9Rb0zPl7zBBM9H65+Lf7zax/df+V8UP/
rlNc2t8QriJ2NccVOboOk2PZR13UCmp/nLNgr9Zjm2U/czE/2uie47Xec29c
p23abFtsY7ddn+grfahv9bmxMEbGzpga6y0GxMb5mpgRS2JM7IlJsbq3F13M
ywU5InfklFyTg3JTzipyXO6bE8wV5hBziznHXGSOsu0Llg4zvu/ONtdD9Ztt
t79Y9uQc1msH1tb0lf43t7uff6mNHNb4OamjRN+bt2rbz9dRtv2g+LE+YD9i
jLTZ/Tnt52D+7kP+EP1nOOcLa177ETXv+T613h+17FUuaiN1/CbdDw3duvfS
p9d6D1W/R5zDMY/jnMWfRXuLd3WLscRG9xyv9Z4+Sxu0TZtti2207e0TfaUP
9a0+NxbGyNgZ0+34RwyIDTEjlsSY2BOTYnVvnrX4c8MFOSJ35JQ2y0G5KWfl
srbJfXOCuWJvDGPOMRfpZ/c22N5ratbifHfJ+bg53Pzp/NRc6rpnj8O3MdVX
rmM+so6y7Y96j3R0x5muO1gz9N1Sx5++N+ocv9e4tz4Xn2LG5xqv7mvuqHn+
m9GfX/Oc+9b0wwejX1XTtwmxcyXzwPL9kGJ/TlF3LWJR6z0GS/2kWE+vdT5Z
5gjF3r+a38nc6p7jtd7TZ2mDtmmzbbGNtr19oq/0ob7V58bCGBm77fjBWIsB
z997/06MiT0xKVbFsNj2uXJBjsgdOSXX5KDclLNyWY7LfXOCucIcYm4x55zU
zEXmqHM187DPWvBzWK9l7OUc/a/f3OPh9zHcE+I+NK81t99UR9naoG3abHsd
T/qNIMef/Vsq0X2P0v5iD59771bYD3YbnVMkZs3NZ9b0Yfjc+Mm6fOP8UPu1
cf/uOqB7nOT1woWitllwp2jL5lmLT2o9R1u+17TRPcdrvafP0gZt02bbYhtt
+2m+0of6Vp8bC2Nk7La4NdZ773Cd1n9tMSb2xKRYFcNi+6Qm5u8Mf+WUXJOD
tlHOymU57rXmhL1ct8dfc9G5mjkq34T96aG779ScZi3OXO06pt8E81r3yLln
w/Vf10Rsr9jwWxnLmP/d+Nn9LdfXUf4UrIh4cF14by+K/YXPXeYsh/X6yFJn
Pszfwou/ur6a+UzXEmNHz3dOatY9rq7p25j9aUNyyPG9erXnLHv2ar0naolj
UUMucFjrPQaLH4qabfENoo3uOV7rPX2WNmibNtsW27i3XuA7R+1DfXtS0+fG
whgZO2O6jbUYEBtiZm//gNgTk2JVEdtifq/flztySq55rdyUs3JZjst9c4K5
whyy9w1Pc5E5yj3zL62jRF94sfGnOdP5tXvV9voXa4+9Xrz1lc9yvdXvUTj3
dw3C/OyYedkLdFjP5W3j3j4NMaD/+zfBo++99+278/2uRGLW76i+Acmcp9+h
+NBxTSR7in5nSI4bh+R4wJAcSz9Y632hiz9rXRdyzNO/75ljqc0W3wnZ3NNx
yzIeqPW7hOqe47Xe02dpg7Zps22xjd5Tn+ir9qG+1efGwhgZO2O6jfXee75i
RiyJsb11f7EqhsW2mJcLckTuyCm5JgflppyVy3vzC3OCucIc4rNso7nIHKU9
t9Tcz++Y07mtOXDBwGFd83QMYH+3zGFr/b7G3vnWIpZ5d63rlo6x3cvR31mK
7ve7lj3Vm7YY32tqzqP7dzOju+9uWZM6rPvZ3jsd3b3HXe8N5pu/F2vO/b+Y
9uZ7wn4P4fyQHHt15uV7ZQX2ippJrfdeWkfq3+arYeufSs3vEudw75P3MXZL
jWWje47Xek+fpQ3aps3exzbadn1y2rqDvtXnxuJizRgZO2O6jbUYEBtiRiyJ
MbEnJsWqGBbbYl4u6KuFO7XmlFyTg8ZOzsrlU88/rL8zb67Yex/QnGMu2hv7
+Xf3jTy4jhL9pI6Sw/1+S79T675MX7n3xnG++2ocw7uuZD/rc333TZ/szVUX
+w/rsbffHDOfWxv0uxzi1j173vO0val34JPfqPn3+9TsT7OGJQeX2kit51by
cVmvKeoMtR63uH9AP58bkuMlNd9NXvY31jpGjmd872/Zh7PRPcdrvafP0gZt
02bbYhttuz7RV/qwfavPjYUxMnbGdBtrMSA2xIxYEmNiT0x6z5OaGD51zlVr
DsoRuSOn5Jp+NkbeXy7LcZ9rTnBfhPabW8w5eznKfO7fl5p/rfcinqvT8eMe
m6WOV+t1H9+/sA91PWhpS7G3v9Z7pN2fY5+7/FZIrfcoOke2z3L87F5l9wY7
Tt7bs+e43T3Jxsh6e/shc57mcsYZvaaTNSzrPHs1maVvqvl7UjmWvcdXjl8Z
ksO69ElNP/ss3++z/7UGvmdD/xbeVvccr/WePksbtE2bbYtttO36ZM8Gv1/R
zzIWxsjYGdNtrMWA2NjbqyzGxJ6YPHWuV+v3mMS8XJAjckdOybWTmn6Wm3JW
LmuP3Dcn7PWPzp1dZ9zLUc67HftZC12+x1Xr/S3aqQ2OGx3zixlztXUG8eke
SLHnupI1jd8bksP1fdviO1C+Q+FeuL21XTGmD/29APG27DMHe2lz7w3O/Px1
Q9I/+U7N9UNyOB7Qh7bLWFgvctxiTcn3Ch3zGAu5v6x91zpGrvc9a8hW9xyv
9Z4+Sxu0TZtti2207fpEX+nD9q0+NxbGyNht84mxFgNiQ8zs9X1iT0wuWK31
O022S8zLBTkid+SUXDNGxkLO6kM5LvfP1cwJxsIcYm5xzdR5pTlqmXfX/N2l
rT2uv3iO9tjGpS5X6xxoX7DYUOv5gnXUZW9brfO5z1rqcjV/+ymHY5IFJ8W7
CbV+/3Rv/5jrFEut+7DuFxzDL3nmsP5ObGPs9pp6frfpS4fknVz7o2UPYa3H
t8v32Wo9PtG3tl2O2+cudZVa+1xcLd8cqOPvXb1i6O4ZsL691Fc3uud4rff0
WdqgbdpsW2yjbdcn+kof6tv2ubEwRsbOmG5jLQbEhpgRS2Ls1P1Ltd4PLIbF
tm2XC3LEe8opuabP5aac1bdyXO77rHM1c4U5xNxizlnO2eQo8+0Das49nz8k
h/Nfv7EmfuwHHz8kx/JuQq1x6Pv7zqmXOmGt9wTuvQ+y9HG19rM5377befpJ
zXGv6xrfWkfZYsz3xVxr3vvtjNP2jt5eU39SzXP+Yq37SutX8vHhQ3K4H8mx
vfUu4+L3W5ZvP9b6Xby9PO9cQHtc73P/ubrneK333OOdtmmzbbGNtl2f6Ct9
qD3tc2NhjIydMd3GWgyIDTEjlsTYgr1ac/mkJlbFsNgW83JBjuy9DyXX5KDc
lLNyWY7LfeNirjCHmFvMOeYic9S1Q3JYN/v+ITlcO3D+a07Wt75D4bcCnMP6
PuyyF7fWOHS/jW3v7w7lWN4F2NjvXH75pket424sbqm5HuEeFffCWV91Dddz
rDMsa1uHuQZ0G/ZnLarXl1M/sW4s77RZH1pbWH5ns/h2d633NZkHlnfHar3m
uKyt1zpnLutQtcanPnd+p+45Xus9fZY2aJs22xbbaNv1ib7Shyc1fds+NxbG
yNgZ022sxYDYEDNiyXOcKy2YrDVWtVlsi3l9LkfkjpySa3JQbspZfSjH5b45
wVxhDjG3mHO0/9qaOWpZA611DdPcaz/o2N66qGM287xjfv3T3+XI4XslPfbO
4XscPtcxvLWRczXHnL81JMfeWpvfPPnyITkck1urX/r6K/qr6yjbOoZ7Y76+
5m+Cd3+RGkTjJ77uGkXcu+yL27TRGPnO4ElN3Bov9wD4DottFzPL3uNa9932
a8u+nVrj05j6XHXP8Vrv6bO0Qdu02bbYRp+rT05q+kof6tv2ubEwRsbOmG5j
LQbEhpgRS86tHIuKSbFq28W2mD9XkwtyRFzJKbkmB+WmnDWmclzumxPMFT7X
3GK8zEXGyPHYxSE5rDO4l3jZu1trPyzfFaz1vEb/mDN9lvYvfVata333GZLD
Mad5Xtza55rn3YeztyZrfdv3OheubfrEkxpc2MwLTtvf9c6a52eu7jhBXd86
7nItbPlmWq0xrE+MlzVqsaGv7AfFqvMm9677rK8dstU9x2u9p8/SBm3TZtti
G32WPtFX+lDf6vOOhTEydsZ0G2sx4Plixrztvguxt7cXSAyLbX0lF/SJ3JFT
ck0Oys2LNTmrbXJc7psT9K3xMreYc3zWqXuWap2r/UaHfZ9Y0jZz714O1G/u
2fZ85+P2p44VxaH73/bWT91n4ljXObj3X763U+v31q1LiL38pt1Dh77UgQ/r
d+FbUifttfv0T37HwHqL+0msW+or14nsp8S2OXBZb6r17z4YX+cX9o/asDfe
to/eGxt7rff0WdqgbdpsW2yjbfdZ+kofaoM+71gYI2NnTLexFgNiQ8zIHcck
Yk9MilUxLLb39hfJEbnj/eWaHNRXctZnXazTc4s5wfaaQ4yvOcdc1Dnq3wMc
yFUG
   "], {{Rational[-1, 2] Pi, -Pi}, {Rational[3, 2] Pi, Pi}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["33: spider2", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myColorMap", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ab", ",", "ar"}], "}"}], ",", 
     RowBox[{"RGBColor", "[", 
      RowBox[{
       RowBox[{
        FractionBox[".8", 
         RowBox[{"1", "+", 
          SuperscriptBox["ar", "2"]}]], "+", 
        FractionBox[".2", 
         RowBox[{"1", "+", 
          SuperscriptBox["ab", "2"]}]]}], ",", 
       RowBox[{
        FractionBox[".2", 
         RowBox[{"1", "+", 
          SuperscriptBox["ar", "2"]}]], "+", 
        FractionBox[".8", 
         RowBox[{"1", "+", 
          SuperscriptBox["ab", "2"]}]]}], ",", 
       RowBox[{
        FractionBox[".5", 
         RowBox[{"1", "+", 
          SuperscriptBox["ar", "2"]}]], "+", 
        FractionBox[".5", 
         RowBox[{"1", "+", 
          SuperscriptBox["ab", "2"]}]]}]}], "]"}]}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   RowBox[{"Sin", "[", 
    RowBox[{"2", " ", 
     RowBox[{"Sin", "[", 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{"x", "+", 
         RowBox[{"I", " ", "y"}]}], ")"}]}], "]"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", 
      FractionBox["\[Pi]", "2"]}], ",", "\[Pi]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", "2"}], ",", "2"}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.390026691533256*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJysvWVU1mvbNI6K2OIWMRDsAhUDW/RQRMVCRcRA7G4Etw0GdksKKCEgKd15
0d2NNFJiIZgg/wuOuT88ey3e/fzv9/2w1+x1fjhjZo6Z3+VaLkfvPaV2oJuI
iMh+/Nf+/2UffYwX30khRjdaK/lUZuLZSHIxO6z1LKMKGE05YZFN37oX0te9
Gw9l55UDBeQePP5awrpc6jln+/itO2OBDnQvykP+Y3MwnbB74y2l6AM0ojvr
zHePXuNEnvlOJ2tXpAM9qFhV/Z7t1mja6/Bw5TalGmAc/RLXfv5pYDGpPV90
46JYONCKsvTFnvXd702NLlINCxNSgG50zuWJTsyLSCoY7rx9565coB+l2ap/
mf09idRd81fMV4oCvqLDeY/M4gYG0PBp/krLdmUAPan8136JQf4x9Dbt2VHX
pgJgINUJLCLfz0snRz2zJY9u+ZKffjsak/HLsPQZtc6kYNrr/McZseRzbbYQ
Hcjq++DBR8OCqZ92dXVEShHlOix8n5ASTHvcCj2e786gt3dVj3rWvaMzFzYp
PP0dR/MOj/U3ri2iPKyvwboc1nNP8z5W2IewjzjOfYJzz+DcSNwzHff0wz3P
p/O7GANpTSS/67c888DoSUHg4Q14ewPeCsDbJvC8CTxPBM+d6ZJixjoyWtFV
6NgNuneD7jegu1wB+4TRg3TXs08K4KsC+EpclX1VOJt9yOhAG+DD3fDtbvj2
RxD7dt5z9jljNN0KZ5/Hf+K5YHSjYszFDhfjVfJqgcDnNHPw76JAB3daa+Bw
PN8sBGhJ7gNrl90u9STrQY5ajr+ygN4UN2zgWbWf8XTwo1jyxgV1wARqc+pJ
1XdKace7A5tXFicCnWmb+y7jtYkRdMfuQ41Wuh/QhJz61e05a+BKb/faOqm2
lgLDKeFKd7mNf7Jo+rBvYT9WpgDdKCxt27nfayOpa8/zKSW7ooF21LV37tXc
c4E0T3uOjbFoPjCAbvUz6BXcPZV2OEgqz16dAHSiRvHhxpGW4VRrPXP0wrGR
QFsas0q/rperH71uurRt655aSujAeCr4UnjqzYUSmmdSe/+PbDadv/9HiD4k
Ob9WXm9WAt0I3XM3aK6ABsn2fZY814bi4qrc3G760vGXf7K36QRSWT/D/lO6
2JPbwHEfv//0JA2sD8W6NtZ9sM9y7NM9nvfZjXOtce5KnLsJ9zyLe57GPY3x
LmO8K0iF31UMHorBg9EA5sHxNPPGGECZfZk3A/BsAJ7twHPTENaF0Y1+pLIu
A/ezjozhdF+PdWyB7i3QfUF/1r2ikn3C6Ewn3rBPxsJXY+ErF/iqTZJ9yOhN
esPZh9Nusm8ZLWmRBPvW3Jl9zviclkiyz/M+8FwwupHDIJ6Li+iLi+gLM/TF
4308d4wC0gnhuQtAXwSgLw5G85xuxlxvxlwfRV/MRV/MRV8sR19cRm5cRm4M
H8C50d2cc4bRipZd5ZzZgFzagFwaiFyaIc05xuhHi15xjg1B7g1B7kkj9xTR
F4roiz6/OScd0ReO6IsE9MUj5LAlcvjsf9kXxeiFq+gFRfRCDtZHY136X/qi
3oTPPYZzlXFuZ73WWV+Yoi9M0ReL0BfKbswb4ysyzmfeXNAXLugLY/RFsivr
wuhGbdDF2JR1ZLSiUD3WcSR0HwndHdAXKugLFfSFOfpiuT37itGITNAXyrPY
h4wOtFnAPpyDvpiDvohCX+ShL/LQF2sj2Od3vvBcMLpR82Ceiyp8X1Xh+0oV
PRLRT3Dwgk4dMIGmbdSO2JJeSlcPuHpXqKcDPahMz3T38KPRdM61aM2V895A
QzIyzXxpqeFICqFHl0c/Swd6kP7xGI2z/tHUc6Dmc/m3VcBoupfrSQskCslM
rvePlCVBQHOy2i4WfPWwB/Xc/Mv6xqAsoDetrp93164gjkrPnbQY8CED6Em5
kqsj1Q/H0vKC0aaD98cA7clkfb2idGgQvdzUcqIkJAPoSTcPX+o1eEosuVZ2
b+prkQ30IZMdPWakCxLI8E/OhEQFPzLqQBOaq3ffIyTCheIDM0wf6uVRl7B2
9CcV2djW6TtTqMeM6PcLGmrIaolkw4KGOJK/luCwqqaYRCfEl197mkPzo8aK
WT/1pczBGyvrmhNJenjMzl0lyfRx1Z8FNr4+1HvUmD+mllH0F9bvYL1+JK93
ts8InOuHc9fgXAvcMzuU7ymOe+7Au7TwrsYr/K4W8NACHn6DB03wpgneZoK3
e+D5HnhuAM9l0KUMuuRDl3vQ8R50bICO06D7NOiuBd3N/mKfMEZTaw77xB2+
coevpE6wr3bDh7vhw3PwYX/4tj98uxG+LYPPy+DzLfB5EL6vgvB9lY25sK8P
kg/z9gYakqr9kSA3a0daW9z3lWFcItCZ1veuXR4SEkFuC4O6ZhjUARPILPHw
su7VpXS8buYAhyOJQGeqzBiZ2ON6BHWf63Fq1cpA4HOKVbZMGv7cnRL+OiRr
1VAJjKKFuRFfY2vyyVttXEKwchzwNe06dCK4XjKUJMY36S43jgc60hb5GZkm
QWFknFcVsV0iB+hLF/uN0J6xNZEWf56lPi8jHuhI66YoHbnVPZwOrbK2MkmK
AdpThOadlZcXB9OyjcteGqXXAOPIcHx2TGB0MbVQtOCAWRSdVGnHVzRUbfZM
fb0AGnItdvDAQX5U2Ko6bOAgE3q4ROrF4OcuNFMrV1WyNZosTR+abmy1o2FR
gT5HFIPo52+r0WY/08gk6JjlomV+ZNtQf87iYQzVYL0n1v/G+scdvM9W7BMd
yfuUXeVzX+HcVTj3/BK+Zz3u6YV76uJdunhX+n/epcI8MNrTQvBQD97qwdu4
qczbUPA8FDyX9GWeq6FLNXTZNY11EYWOotDx5kHWUXUg684YReXQvQ0+aYNP
/ixnnxjUsq8YnUkNviqGD4vhw2z4ULmEfcvoTPXwrSx8Lgufe9uxzz/j++oz
vq/88H3lhznywxyNxRz9dZDnjtGDjPV57ooxp8WY00FmPKeimGtRzPVy9MV9
9MV99MXUPM6BcORGOHLDH7lhrs45w+hNEXWcMyvPcy4xetLWwZxLbfmcY4z2
tBg51k2dc4/Rk7od+T/3RSJyNQm5ehl98RM5PBN9cfNf+qIbcn4ecj4DOT8a
vSC5mnthMPqiK9b7oy+i/6UvVHFuNc69jXM767XO+mJdFfPA6EO7tZgHbzXm
jdFTmEvMW2e9vAV9sQV9MRV9cUiDdWT0JpEG1lEduqtDd13ofg59cQ59kY2+
mBzGvmL0IF/0xVkX9iGjIamZsA/L9rNvGT1oKPpCtD/7nDGBDODzLfi+2oLv
q1x8X9Xg+6oG31cb0CPhZUfSBz6sAcbRx60GBltOF1P+7fMu224JgDaU6qQx
+HONL01vnJ9q/iMSaEshJ+OvCfz8qfjUdP1Vz+uAwrm+31Ji0VZKv6Pzgltd
I4DWNN8vRW5Rvg9tWWL8eGtECtCN5qiEbptrFkmnX49RrorIBfrRmo2TN8to
J5N+0o0nU+cJgDa0K8yuaZ7w+ztX7lSXv48WAAPpfnPiNYmYNCprjJStCUgA
OlFy3rCLTyQi6MGY01u+/REAbWja5LUfvPX8KHuffraYTB3ldmACJSha3VXZ
X0pJ37XMK3anUen+nUJ0p2vxF60fCv28601FtF6dP83Irs7WqzOlY/ukL5ZU
ulH4CbGvtjEBJCHi8P2vODOK+vDp9KHvb2j4R9nwGksBbUhZofK3iydFqa+r
tD0TRP2wnprM67pYt8I+X9vsO/Yxxz56OHcDzn2Kczu7ZxDeFYJ3OeFd2uBB
Gzz0Aw/zvjJvjE7ULZ950wHPOuBZHDx3pssfB9aR0Y9CN7COv6D7L+huCd29
4BMv+KTEl31SCF8Vwlfp8JUqfKgKH5bDh2fh27PwrRp8uws+3wWf/w2fu+P7
yh3fVymYC8W7E1+v0fcDmtD9Y1+ejBjpSoXl/cT0JxUBg2mSW/y+9T/TKdnW
Inr55XxgAOndj5y9xSKVLB5vGymo9wI+o2tfpx3Stn9N0mLK3yJFi4GhlHGj
vuDnmEwaMqrXI7cNKUA36u8Tc/64WiQN2dP/je+waKAd7fj5dNjYUYHkE/x8
c2xyETBYONdXm38Lf2f6X351wexkJNCWqjT7mi3u7U+fspatcu6XAfSk9/FG
RYM2xtDRu99m7ZhfCAyiCkkpm4J96bT8wzibHTo+pN6BRvR80skEhSdO9Kvx
QVG/rCTqv/uhEF1Ib/w8p9p9AvJR8N2YolBNpwJ8N6UoxFK6ja1DREQR7Wmd
0HeZ5jty+RTUcFozhtYEbl450quQBqrL7e4xo4TWS56sEr0dRTvcp9cp/8oi
Eaw3DuJ1RaxbY5967GOOfUxw7kac64xzD+KeF3fxPcfgnmfxLgO8Kxfv6nGP
eWAMIuXBzMNj8PYYvJ0Bb5/B82fwPGUH83w4iHVhDKZrg1gXe+hoDx37/2Id
tUay7oxuZO7Nurv3YJ8whlKKAfuk5xP2FeMz6t3Evtphwz5kDCDvu+zD+gr2
LWMwZcK3ifB5InyueJx93h190R19EYXvqzLMURnmaMg2nqNVd3juGG1I05nn
LvgLzymjLcn+S1/sjOEcYLQmX/TFKOTGKOTGIOTGd/TFd/RFE/pCAbmkgFxq
C+Vc6j2Zc4wxkJKRY9vRF9vRF+fRF6+Rk6+RkxuRk5+Rq43I1Qbk6jPksANy
WOv/Z19EI+dHoxeuoi/K0QstH3h9Nvpi8L/0xQucewrnBvyXfbEAPCwAD/Fy
zEMD+qIBfaGDvrBHX9ijL1TBcw10qYEuZeiLz/asI6MfHVvPOkZD92jofhG6
m6EvzNAXYeiLPPgqD75Khq8OfWUfMtrSktPsQ3X0hTr6QhZ90VzKPmeMo8Yt
7HM5fF/J4fsqCN9X9ZiXesyLGnpkqloPmZcKFcBICpiwyu6m8J2O3cNGjG/z
BRrTwNsnDn7UcyGzviZmY8zygP4U+jh48WS9FEo8PcIzViET6EUtfQ9qBUTE
0vpIsSH7JcKBVqQ9f/+jEce9yf6lTaSiSjUwllTFVn9dU1hEBfpzN0qu8AUa
U9qum/bmps60rd+yE+5faoBx5J5uqDL8YzGNfCm9uGRpMNCCsioyquV6edJO
8fX145OygT40O6ti/6fWBJr2scsHaa8CYCAVzxo9SE1c+P26U/yR1UI/ktjd
jib0bo6h9u1EF2oqNPkrblosjd5pIUQHuuzVmhMQHEw9YsYoFmwrox07h68s
2BZBLyzrHq4/nU0rSz/M7WFaRwOMDDfeNk2g1dFqfetaSumH7vD9M+5WE932
Hvz3jWSK04iv6+9cTJ+w/v4WrztjfTX2kcA+qtinJprPVcS5Z3FuOu7ZqsX3
XI97Dt3F71qDd02ey+8yBg/G4EFmNvPQF7z1BW+pmczbYfB8GDx3qWSeO9NF
6irryGhMo3azjprQXRO6f+nOumfDJ9nwSTR8shW+2gpfnYSvYuHDWPjwO3y4
Ab7dAN/a3WLfBm9knzNGkix8bovvK1t8X8VgLn4tWCtzYnII0JLe2q2fOc7C
k9qu/LVx7p8aYBzpFW8eH9NaTAmR9mY2wu8Axld0SSHtz90jAbR1nKRuryXx
QEc6eUIqo8+GMCp1OXprx/tSYDgdc+maPexrFsWOPPrAPtsfaEr+rw0+pSa4
Uf6nk9/ePKsFxlOqS2+f7f4ldPidWWV4Pz+gCUWO0H877akLSe6YVZrVVgGM
JCmdLnnx4XlU4TUj+YlNDNCezEcubmvqFUxZV5dJjlgZCbSlHnuMzN/l+9HX
Oq2u7gk1NKy+HePo/P7mSKmQYsqY0bVqw5RsOqQjKkQfclbtJnNgbgLptl3K
rL8ZRlKJS+Te33xJrUNvpXpJeNP8nBbPaHFvunlJTj9GwpCkdOP+8st+Tb13
SEsZyQVT1UL/Y/LhXvQtqU6pvMmP8jV5XRzrOljvh322Yp9KHd4nEeeexrkK
w/jckTP5nr64Z9/1fM8ovKuujt+liHfpgwd98PByN/Owxpt5Y7QnmVHMW6Ym
88wYSbFnmOcq6FIFXXaMZF0eQMcH0PEkdJw5inVnNKXFjqz7Wmf2CWM4nXJm
nwwYz75idCT1k+yrJgH7kPEVnZ7JPjysx75ljKOuJexbt4Xsc0ZLWmvPPpdG
X0ijLzLwfbUJfbEJfVGLOZIU47ljNCZz9MWXPjynjP704RHP6QfM9QfM9aR+
PNdeAs4BRisSRw5EITeikBs66IsH6IsH6IuT6Asj5JIRcukHcunHC84xRgsK
Rl94IPc8kHsW/9IXTuiLt7s4V63/pS+6oy+2I7ct/qUvfqEXtqMv0tELtVh/
hL64/y99EYe+kMa5qji3s3t21hct4KEFPGihL5b0Z94YfSg9g3lTRl8ooy/q
wbMYdBGDLiuhiyP6whF9kYm+OALdj0D3btBdJIp9wmhFZfBJ7in2FaMXJfVh
X+3pzz5k9KefT9mH8ugLefTFBfQFoS8IffFoPPtcBN9XIvi+ssb31SfMyyfM
y2b0iGvCXyrrMwqBQdRnRJ+HGZHpFLzh7Oe+wnswmlKd3bMpA/TcaO7NO0/W
DawDJpCBg3NTmUYptWRsllzuHwQ0p0WvNk+97+VBbjfiQ0/9LgaG0lq1F7nj
QzNpZKXF+6JIAdCGaoquVz9T8KO9I47d2Z/wFhhCrY1KOSWtGXTFaJld3eRg
oAWlKjqLnfntQUsXPtIY/rgOmEB/Jc6YUtFYSmOuHLsQ2DMcaEWz5Svf6xzw
pklatNDhbixN7kAHOjNLPFF8Sgg1XhgcVP6zljZebsd4EhtmOWn7vFJKu2H5
UUMum1yOeAjRhxa8VHFumJVAy8VvdlFdHkFLvqlkqy63puy1pn42Gj70pLt+
3cBNPqQR67nugJYRTTj3XvH4USeS6HHWaZtLIPnUf/K8Pl2Yu+e9pn784E81
YrzeUsfr2ljvbJ/Ozv2Oe2bhnsdwz2y8SwHvKhvK7yrewTyU7WAeJMHDOvC2
DrxtAG8q4FkFPA8Dz53p8hk6foaOGl9Zx7cVrDujDRlCd2UD9gljKDnDJwbw
lQF81WjLvloBH66AD03hQ6mN7FtGU9ptz74Vgc9F4HNrGfa5Kb6vTPF9FYa5
mLIu8dJWTwHQhs6VhZi/kfGjUaYybodvVANjqbZlf77SuLek1mVPrzm3fIBG
5LA08H7UaydSHlV+tK9eJTCKNh9+fvOxVj7J9L9V92BFBNCa7J7WjJ2+1Ycc
P202O/amEBhECb6zqZtbOtnLzlSxiI0AWlN/n8VSSj986Gtz2FRjiUpgFJ32
lr9wMT+PBujIzmtV8QYaUnaawi3F4Y60/naKuvyvUmA4PdUVad7RkkX9M1/1
PZci7O0O9KZnsclp2RHx9PzblYDu7j5k0ktPiEb05/yLCXGFTjTiedlJyxmx
lLu/5ynLGQ6k7mV7r1tYMAXdNQu+/aeY5l75Ne7On1AyMDpWMSg8k+pX2vRu
la+jrtYTt6ycnkD1046V9b9USjbhAQN2N9XShN5LpSyOpdN224bUwcZl9Bzr
1b14fQXWM7BPnRXvk4R99HFuy2U+dxfOzTbjez7APb978j2T8K54vEvxAr/r
OngwAA9j45gH41vMG2M4yYC3/DPMM6MhPQTP576xLoxRJOnDuiyDjsugo5E3
67j6M+vOGETyfqy7eT/2CaM1icAnCfBVAnz1G77yEmEfMhpRyRL2oaMJ+5Yx
ljbCt1bwuRV8LlbOPp+GvpiGvijG91UV5qgKcySPvlBFX6iiL/agL3phTnth
TtdgTiUzea4ZzWkd+qLtOucAYyiVbuQcmIq+mIq++IHcyJbhnGEMITv0RSFy
qRC51HMR55IickwROdYHORZzmXOP0Yp+T/0/90WXi5yru5Grw/4v+2LLP3J+
LPqiCX3R9wL3gjf6YhL64sO5/9kXm7HPeOyji3NP4VzxdXxuZ/fsrC8C0BfB
6ItqBeahDX3Rhr4IR18og2dl8CwJnsWhizh0uYu+EB/FOjKGUK8m1rEefVGP
vrCD7vroC330RRr6YgD6YgD6whl9EQAfBsCH41+zD+ehL+ahL26hL5bEs88Z
g2gS+qLwM88FoxvdwfdVI+alEfOyBT0S3FM/++CgbKAP1SW+vHt1WAJ1d7Uz
rXAQAIVzl/dwkcUAP3IrOjlZsaIMGEFmz1bPSZLKoR/R2/RcJIKBFjR8+A/5
NQ0eJHok9IzYyUpgFC2POi5+Z1U+rdyqdFbiQCjwBU05t0p2yi4v0pr1p7Xr
ygJgIOkKroRGmaXRk21V3w9fTga60i//j+aCFAGZRK88KPM4AmhNwb4F4VVe
PsJe98uTnlwHTCBXh1f9qs6WkqvI17s91ibQxw50otqIU8d7WYXTh5nvNplM
8CXlCdVCNKYrISW1C087U8XP8CPp25Jo1Knao+nbXCin9yKlMUMF9G3+lkMh
t8tp75cjK0NvC0jVb0agS10Oqb5dsijWpI7qb425V/w8gXrqJVgtai0l89q3
O4+q1NKe7Otf4hUz6IBs2rFDOmX0FOuTsb4J6+uwTx32EcM+OTh3Kc4dj3O9
cM/PJ/me/rhnIN41AO9ahHd9AA/TujAP8wXMQwl4KwFvxuDtAHg+AJ71wHNn
upQqsI6MgZQbwTq6Q3d36H4CunseZp8wRlFxJPtEK4Z9xWhB2+ErE/jQBD68
BB+qwbdq8O1B+FYFPleBz7fB5w/xffUQ31cBmItBVTm3Bq6IBTrQMK37k0zz
gunobSrZWF0IDCJbnxq5r2np1JjvljEgNRT4gkxf972RaO1Ffreqva7KVgKj
aN8k69Kyr3l023+j5Ov4IKA5+QzcdOJwqAdNW1K1W12hHCigBRtkLshezSHX
des1/r4eCbSlIaMr30uM9SeZUWcG/LZOBb6hgeor6y3HRZFlz4f7J60pBobS
nvH5i3+qZlKW4d7wR9t8gEYUN2/hqcZTTmT84khYpVYG+XWgJ3XrLvpmh28M
uU0fluXvVEtD17djPOkYlMaNzy6hs6McNud8SSNv41SNnC/u9KXCdfP471F0
JDNqbs2vEEp4uG3ex1+W9FHqw94MaS/6MeKKu4WkNxXr3BusIW9IH4NHzp5R
/prOTRixol9ZMB3sXbBNbXswnTT7nTbtRTDNxLpyL14PMuX1fiN5n8/Yp1cI
7/MU537BuT2G87kNuKecCd9zYSXfMxzvksO77uFdSuBhH3iwEGUeRI2YN0Yj
Ep3PvE3pxTwzhlIweF40knVhfEO3NrEuXVVZR0Zbug4dS4l1ZxRQ7nrWfV0A
+4TRnKZLsE/mw1fz4au4ieyrTwXsQ8YXNNuRfWh+i33LGESj4Ft7+NwePk+F
z5egL5agL2rxfVWBOarAHEkl8Ry5u/DcMdqQeS7PneRbnlPGCJIy5DktRl8U
oy96Y64Poy8Ooy8E6IvpyI3pyI0RyA0B+kKAvviCvhiAXBqAXHJGLrkjx9yR
Y4XIsT4jOfcYEygFuWeKnCxAX6SgL3oocK7uRK6+/H/cF5boBV30wjH0wi2s
N2Tx+uJ/6QtfnDsa57b68rmd3bOzvuCeSCBt9MV99IUH+sIDfaEJ3u6B53vg
2RU8y21nXRhdaXsA6/J7JuvIGEiB4azjTeh+E7pvgu5P0BdP0Bfe6Isc9EUO
+sIevrIvZB8yRpDqU/bhCfTFCfTFHfRFbA/2OaMP+Sewz/3xfeWP76sz+L5q
wrw0YV62okdEvX8nfJBOA7pTxfpFLwediaLpppvH6OxIArqQnt7Z5rhhAtre
VdMrb2Aa0J3Ol59Piz0WRe5qd70uTUsDupNblvF6tytR1HL7kIzbr1igA/06
Iti/NTCEVNbeb0yuLAIGk5u46ZDdJzPo9A4pwYpIf6ApPfi6Wzvfy43i5sxq
NvpSDYyl88rXZv5ye0tGT+6lbo2LBTpQ4KypXaddCKHRd/8yuG4YSpM78AX5
Nd97NuOyFz29eHn2tLHl9PpSOwro4vOirr+P5dDR5cPTtSeW0Yqb0kKMoOCD
Y5RVZ2TTzPzn5c2KMeTw5Xdts6I9WWbnayo8CKKMoPGfK7b5kYLhsEet20zo
xUjXTXs/u9AQNxrhXulHy+cp7eg93JQmLdzgq+3sSvtne2woGyWgTxWyZ8af
F5Bc3xPl19cKaBXWz2K9pA+v/2cfZewzEft0dq4S7umHe3rgno54lwHe1esQ
v0sbPNwGD2rgIfUO85Z/h3m7Dt6SwXMyeG4Dz53pMgs6zoKOS6GjP3T3h+6N
0N0CPrGAT67BJ+s2sa8Y3WlANvtqGnw4DT7Ugg894FsP+DYVvj3sxT5ndKdr
8LkBvq8M8H3lhbnwlRlqkNMvGehKTwc+PmN/R0DrJs99YvkxDSi8z6UxhaOb
o2hvN9P3zoOTga4ks3dxltwjAVnVmz5zbE0CutCrxpead/UEtFp/hWSyZhbQ
m6RHmkmFz4yn18FXv1ydKgDa0JP6sqnl531pXqllxpzt1cBY6j5E9Fb05yKa
KLhxrmC8P9CUTnvuz1n/yZWOXDqd5vwoC+hNy8zX3th8Jp7Sgk1uTREvp/QO
FFDkUUXH72o5FL7mRXnIhVA6q96OL+jkhq79Bh/3oivRl1L3JIfR7qibQnxJ
zub9R4eSN83/vK3LFaUc+trSu/sVJV/6291SSdowkfos23gi9U0NDQ0TF2S8
iaOnM42Lr5sXk3/rr33ys2pIZn1w0rCFcaSXUZfb1LWYntven2JdW0kLQpRj
VjxIJ18R+cdR8m/pJtbLgnn9LNY/YB817BOBfRqV+Ny2UD5XG+e++8T3tMI9
++Ke4/GunniXDt41cC3zEA8ebMHDyBDmbVQI8zbxGPMmDZ6lwXPlc+bZErpY
QpduXqzLE+j4BDrqQsft0H07dJ8O3cWusk8YvanHKPbJpzr2FaMLzYavzoqy
DxldKQE+1Jdj3zK60+OL7NujI9jnjK7UVYJ9roa+UENffMX31Tz0xTz0RbcN
PEdWJjx3jC40FnPXoxvPKaM7VWFODdAXBuiLZ+gLzTucA4wOpH2Uc2AAcmMA
cuMecsMNOeOGnMlHzojM5VxijKVw5NIp5Ngp5Ng95NhO9MVh9MVb5J4NctIf
OfkIObkauToBuWqIvhiNHDZGDt/6l74YipxXQc7LIudPohekK7kXlNEXsliv
Kuf1W//oC6V/9MUHnLse50bg3M56rbO+WAIeNMCDJHjYgL7Yjr5QAG8O4NkB
PKeBZ130hS76YgB0cdVkHRlNyaSRdbSE7pbQvRy666Mv9NEXO9EXDeiLBvSF
DvrCXYR9yOhOJ8rYh4Xoi0L0RS18G+TJPmd0p0HoCzN8X5nh+2oXvq++Y16+
Y162o0fqPI7qDGiOBzrSWK3lT9RnhtN+NwUL95gcoC+FmolcHz43iezzl5UP
fhcBtKbUVKPnO0b4klbUs8PrEyqBUTRL11/pVXA+9V01fNHsHG+gIakO3/vg
eqIj/TBv0ol/WAWMJl/BlycLPAto6bJVdLVbLNCB/jIavn63QTCpya6otNKM
BNqSg/eQt7cb/SjezGPSqS11wAT6Ffa2cbVbKRlaewxNiEigPx3oREf2n3S/
MjSCli70qZjww4fEvNrRiCIP7bGYPseZRvxcnH/YKJmioo8K0ZUm5ky5LXgr
oPdfm6JIvYpWeE1IJfVoumSt57/+SAHJ/br1fYR8Dekll+6Uk4+jVR8WP3b6
+ZYiFm8ZnZfyls6pDFVprA+hU45vClaJZpLGV7uDrS359LbIJUxauJXt6daK
7j2zaQHW52D9KNbfYh9D7HMV+8zFufdx7jacW4R7zsE9D+GeGT/4XX/jXd+z
+V0TwUODJ/NgCx7CwZucDfN2E7yFgudQ8NwAnjvT5T50vA8dT0BHX+juC933
QvdsFfYJoyH1gk884CsP+MoSvnoGHz6DD/3gwwz4NgO+Xf6cfXsFPr8Cn1vu
YJ9fxPfVRXxfuWIu0pX2ldpaZwA9yUBC0TSpp/A7XlNg25IVDbSjC17aSb97
BtEpl1WttTfeAkNofB/Vl9qZGSRHPyV3HgwCmtPFuOCUX/oelPNTfa/JoTpg
Aq2eV//rkqCU9D9/dxtoFgh8Ti8bzm4b8sedbinrnjPekQ30odebN+YE6STQ
4RF5UR4PCoCB1LU6uentJ2H/9bTL3tnqTQs60JBW9E6569fiSB8VNob/6ZFB
Y2e1oyd9uzr+1J9VMeSTl6WuklhLD6vbMZ62ZPQ8WthSQjIbB0oE3UqlSOnF
QnxDExo+r5k1KIpeH5fZUpAlvOcuF8eCrOe0bIxd2aFxHpTqetP59zs/Ok1r
32lUm5BTSP6+p1autOlX19QDCREU0+/mcMtuNhS9qCp7bW9fErlXZP6iKp7E
lg84E1MWRzdqzt7y3BtP2Xd5XV6Z10WwPhX7mGGfZ9jHC+eq49xbOLce94zE
PU/gntp412AZfpcm3pUHHjzAww3wEAbeviowb0HgTRM8a4HnM+B5+EjWhTGQ
yqFLf+jYHzr+BR2zoXs2dO/5gXUX+cU+YUygq/CJ/2L2FaM5zYSv7rmyDxlD
KBc+nADfToBvP3myb2kZ+5zRk8QHsc93oy92oy9a8H3Vx5PniNGR1qEvPrry
3DH6kiz6oh5zWo85HZfGc6qMuVbGXEtirpXRF8roC2PkQE8Lzg3GaEpAbvxS
4pxhdKAAQ86Z+8il+8ilD8ilTORYJnJMLJxzbD1yLwh9Qcg9VeSkBPoiEzm5
EX3xDbm6En3Rghzeghx++r/siwv/6Is96IXBb7kX/NALQ7B+oZDX5f7RF8/+
0RebcO5rnHsZ53bWa531RVfwEIe+0AEPK2yYN0/0RQF466yX+0CXPtBFFboM
V2YdGR1otDHrGIe+iENfnIfu0eiLaPRFkxT7RDSafcUYRaJn2VfNeexDRmtq
TGEffkNffENf6KAvuqMvuqMvxNAX5/F9dR7fV2vwffUD8/ID86KJHnE7bzrK
0C4aaEf5GqG1/imBZKnTK+Cw8Pcao4BupPepVtqUQz6SP3/ePOQLNKbRmufO
HxI40+GixEcb46uBwu+jZ1caD1x8S3EP3GU+344E2tLPU8PHN8j606OapCNS
i+OBjqRdvljUZl0YrYy5fmSA5ztgDJn9XXlDXaaIuhoJbk54HELdOtCSrrWY
SZwv8CSZCJ8TeuUxNLsD7Un28+heJduCKeXl1MFXVtTSdet2jKcGsUBN6a0l
ZFFwL+XX1GyaN+6BEH3IqeLD1QvzEygluOq1xvkQUr90wlnjvCVt7LdEdk6S
J60xmXTZ/LQ/iTQHilmeNqX7EnlXBGvd6M1jW4HZ0miSuONbMkHJjry+rL8w
Ri2QNt9wGPTTLI0iRq14sFLMgwoT9p/onxpFga+7XhT/nkdlG4fM3F2STskt
fXP9j2aTOdYNsP4Y6//ZJ/wf+7zEud1w7kucuxj3/NrE97yIezbjXZfxrqN4
VxR42A0eisCDP3g7Dt4ywFt1OPP8J5x5fveJee5Ml5vQ8SZ0rIaOy6H7cug+
GLqLPGSfMLb//Q72yST4ahJ89eUp+8oGPrSBD3vBh1XwbRV8Wwzf7oHP98Dn
F+BzXXxf6eL7yhFzoWScXrq+Wy7Qj54FReeNNUgig8MjAmYJf68xviC1LhJf
RYS/12LTRhVqX6gFxpN/N6cNqaYlNHWb7YWKDf5AUzr/qffQ4ePdqE92etT6
RQXAQOrXK8bn1YM00g1fu/jI7EygF238VPowKjKWzn8ZM3j3yQCgGeW6uQz/
cfcNvfgkkH9bWg4UkMGxMwuytuTSLYVxm3ddz6ZnHehDR5eckPtum0C9FG99
XuzoS26r2tGYdFNfu6oOdKF+pwtUFOVjaVX3t0IU/h7UHqCZFBhMMxdPfJTT
WE5jNc4LUfi7blPMl3uncilztoWvl1M1Oe35UO3lFEuKjzROjVR/SwG/lr4c
eDeX3v+d5Lrtrh+dydftdWRyMjU+3L/CzTOJxq1dvXrZeFe68uRSX30tAQ3u
M22DpUIS9ZaYHzzyUTINEPfpXlKWRJW9eX3oQF6/0p/Xxz7ifVZhH3PsU4lz
B5/jcx1xbt85fM9K3NMA9xTBu75s5nddxrsiTzEPf0SZhy7gYTp4ywZvduAt
FzxXgOdU8DwZukyGLs1HWZfh0HE4dNwDHaOhezR094Xus7LYJ4yBJNKTfXIb
vroNX9XCV+PT2YeMwu9TUfZhD/i2B3wrCt/Gw+fx8PmCYPb5KfTFKfRFd8xF
LOYoFnP0HXOkgrlTwdxJYe6+Y06/Y051MadXMNdXMNc70Be30Re30ReO6Atb
5IYtcuMecmMocmYockYLORNjyLkUa8i5pIlcWoq+2Ia+0EBfNCD3bJB7Ej04
924jJ8chJ58gJ18jV+WRq/ORq5rI4QHoC6v/ZV9EIeeLkPPR6IUuatwLFeiF
c1gfjr5Qwbo69gnDPgXYRw/nVt7mc//GuZ31WiPedQHvOox3vQcPd8BDv0rm
4W/wNhm8XUVf9AHPY8GzBHjei77Yh74IhS6N0LEROp48xzpqQndN6D4Dukui
LyTRF9vRF7WF7CvGWLJBX/SBD/vAh+u2sw9b4dtW+LYFvu0Ln/eFz8Xhcy18
X2nh+2oBvq9+YV5+YV52YF6ObR48xe+mAGhDMfXD/cfW+JLuMmmNqXY1wDja
qP7KZd7DYjpun3t75ewAoBnFqiYfea70hkrrVBfNvJID9CUNO9tkvbeJ1E3m
tKZPbS7Qj5aNW3um3DaZTrjQthMPfIBGpGLy2P2OuxMVh/lUZnYtAgbT6u8b
UqPK0ylex3FMbGohpXZgEO1UccpZKEinqc9G2LeVB5CLcTuakencg4/0pdxp
TY1v2/vcKKKlfkJ8RRbnXrU5JAdQ1cU18ZV65SR3ZqMQBXQyb1eQZUkOWS06
n3T9SxW5GV9Muf4lmnpV5upVTi2ktg+jKkUUMmhwxI27XRU8SbTk2k2Bfgwd
22dk/sU/knQCd/S/HWhLcU8n79x00J9Cli78oOMSRGUXnkVlalmS1Gfx9brl
HjSlZnSznSCC1hncvVz8MIFCfHPmCSbGUH+sJ9/g9RNYT8Q+VdhnNPa5gHOv
4dwCnCv+ke85Afccinum4V3FeNdyvCsaPPQGDyvBgzR4GwDedoM3LfBcBJ6z
wHMcdEmBLlrQ5QF0fAAdh0BHJeiuBN0HQvcd8MkO+MQbPrkKX12Fr0pfsa+u
wodX4cNS+PA1fPsavn0C386Ez2fC53rw+XF8Xx3H99UrzIV62LTsVNkiYDA1
5b5QHv0rnRp6OGsEH/ADmlDNXfUpcm0u1KY59ue6undA4XfozOPnbPYVkUvT
QodV/WKA9pTzJVHcckMQZW2o0UhtjQK+oh7jbRqWtgaQ6IniqR7964AJpByv
Pn+HmvB33+JKkyQzAdCGHvazvPznjy9Vz8tKtPQSUGMH2tCYHndal430o+DS
LZ9W0TuaXNaOMRSbN/WY9ONCUjw9v3D66Dya86Id/elOXtyAK3XJdGjhqrWV
3UPpafN5Ib6gAyHe4m/GetGUdSmex2YE0vN+ZYJjM57Tsw3LbWOvuNO90wMu
Wd0Ufo8Pyo2zvelEAc3dp/asCadc73FnE+ryqbCk5zSj+gB68L766slxaSSd
aDvl79eVNNbzRI/I4ig6MEfvTfXLfNJp2jKq94Y6eqmfPC1cI49GS41863Ol
gvZhXRnrXbFen8D7NHjwPuOxTxjOjcS5J3GuCu55Fvc8gHv2wbvO411H8C5r
8JAIHu6Ch+vg7TJ4KwNv98Hz71Lm+S54fgVd3KFLqhjrsg46roOOZdBxNHQf
Dd2PQHd1+EQdPjk8jn3Su5l9xWhPaxvZV+fhw/Pw4Wb40Aq+tYJvb8G3H+Dz
D/C5eh77/CL64iL6oi/mQh9zpI85KsEczcfczcfc9cDcjcecjsecnsScNmOu
mzHXZ9AXDtKcA4x+lDaWc0AbuaGN3FiH3JgWzjnDGExXkDPnkUt6yKUpyCVp
5Jg5cuw6cuwYcm8Xci8KudeEnFyAnLyBnHyMXH2BXG2u+O/6Ihg5/wk5L42c
n4NeuIy+yEcvNFTz+gL0xUCsp2CfYuwzBvvcxblPce4HnNvZPTvrCzvwUK/N
PIwAD2nVzFvUEuat7W/mTRE8B4FnJ/A8S5d1WaDLuthAl7RQ1pExmGy+sY5r
ofta6D4SuuuiL3TRF+noi1Hw1Sj46hD6YpID+5DRjLzWsw+D4Nsg+NYRvm1S
Z58z2tAM+FwZ31fK+L6Sw/dVC+alBfOihXkZMurHuKgDYcCX1H/Q8O/P27xo
k07VlbhtdcAEmvhES2u+RylN7SFm33VcJNCWftxZqdrNzY9sM4++t38aA7Sn
1beXHxT/FkT9JSoyr+bXAuMpw9S8h03/UrK42T1qfKSALDvQhrrWxCq4z/Sj
puEL46MPCKitA22oqWLVr9nhvrTw5K4QmvKOznZgDOWtOuey/lwhHfl2PLps
Wi4pdjshRD+aO/dOc35wEoV0c3ZXTQqioRaeQjSnpbOTpv4V7kE/IsUOis4N
pYzfFw6Jzn1B36wbw/cqeVFc/NuaOZczqKTha+ucy570J2rba+WiGPL5Irr5
Sp8qKizue+Vqn2iSlVXR1epfQE++5e69MKOOjny82d1gZgLJnpm3X0qvlCap
LPJYe6KSLM+8UDkj/P2QZrdfqW53PmmIxtnZy1fS90tOx7xG5pFapEWhXlgJ
LcG6HdYHYn0+9nHBPpXYxxjnnsS503FuZ/e0x7tC8K5svCsdPDiBh2LwUA7e
FoG3reDNGDwfBs8nwXNnuqRBx0LoGAMdo6F7DHSfDd2nwCdT4JNv8MkO+GoH
fNV4i321GD5cDB/2u8s+XALfLoFvxeHbmJHsc8aXVCTBPt+P76v9+L56ibkY
VRGQOV27DBhBjlKHelx5mE0id2Szcr57Aw2pre7ryqRmRzpe9dN8ztcCYCBF
yb8d1XVeOhU9+H7Jpjgb6EN/nW4u7CWZSP3cuhf2rvQBGtGGobefpAx3JiWf
nj7Z7oXAIHI6ffNzlVs6pUjslu97vIiyOjCY9vRtsbw9NoPyZJ9GHmwOIPvJ
7WhGEbqzdSJmulP3tyZ9utyIJpfmdrSjU88vqwpeBdKeOQNnGx+rpLWyEkKM
Im+RLdcvrsgnm/XzhzhsKacQv0NCFNCGRRIvHrvlkOioOxkjxyXQzMN+rSPH
OZF4UHP6xBPhZNow9K7cqCBKWLc1YdkocxK96h8qo+5BFotEfcar+tDulksJ
FzWMyFtx38HofU5kmjqnfo2osL/HzHM73+8lmRyL7J2zzJPUF30oOpsSQ69s
otT7KwnzdEmQUsKkJJqE9flYTyFe34F9ckfzPkewzw2cq4JzjXFuIu7ZgnuO
wz2n4V1aeNdyvCsTPHwADw/BwzTwJgXezoK37CLm+SJ4ngme5eRYlx/QZeJZ
1kVlEOuoOoh1fPMfHb1Zd8YgOgjdfVzZJ4xGVD6EfbL0IfuK0YeewleWlexD
xkBaCh8uhG8Xwrer6tm32vC5Nnzebzj7/Db64jb6YhDmYi76Yi76Ygz64jDm
7jDmTglz1yjGc8poS2/QF96Ya2/M9S70xZeBnAOM8eSGHHiM3HiC3Gis5tyQ
luacmSjNOTO+knPmEnLJF7k0cTXn0mLk2F/IsS7IsefIvWZzzr3JyL3fyMls
5OQP5GQecvUjclUimnM1Gjlcjxxe/L/sCyvkfDpyfhd6Yehl7oWj6IWJWFdC
X8QL/mdfOP6jL2xx7iWcu+S/7AsH8KAPHqLAw0JR5s0UffEEvOmDZzXwrAae
e0MXZehijr64ir64j77Yh77wh+4B0H0c+mIe+mIe+qK7GftEEr6ShK/M0Rcx
6IsY9IUf+mI+fDsfvu0B3x5CXxxCX1ijL6bg+2oKvq+G4/uqFfPSinnZiXnx
6iO3WKwgCGhO61cWVr+K86Ch5S9T1We8A8aQ1vMVXZ2vFJJ705A7ZyYmA13p
w4/+xq+Fv8eO6qhv7S0IpGMd+JzGxM865zDUg/wMHm1+uLWc/DtQQJXjG509
3uRQo8bZ2RfDc+h3B/qSh2tm5NZpSTQ1yUtmgosvVSW3ozEd0Btp/0nShfwd
LRKtuyeQp187OtE+E6XWnivDyVtn7/wFATU0JaId4yhsbHCcikMx3X1A7w48
zqcu5huEGEC9vA13kncq+fyZLaP3UEDyid3H6D20IdL2ty/+7EsbcvzLdih5
U2DXc/d2KhnSH+PNN5oGOtKojxMmKchHUNLoVK+d8tZk+HdhduJ8H3osn/mi
TCGRLMp7dOm72JkuSfRO89wTQcqNk76vGZ1FQ80fFj876UOJWx+8+awYTy6u
x9UVlErIa8/TOX1O5ZN8D/3lVluLSA/rG7BeLcbrM7BP23PexxP7dHbuINwz
HPc0wD0X4l12eFcl3iUADwvBw3rwEATeZoK3FeDtOXgeCJ5fgOfOdFkHHUVT
WMdH0FFhC+uuuIV17+LGut+ET27BJw7wyTT4ajp81RrHvoqHD+Phw1b4sKyM
fcsYQ0PgW134XBc+nwSf78L31S58X1lgLuq77/f+vbMSGEX7eyyyDpqTT9MW
OW6a9zkAaEYX1z47vGKKOwX17F4Y05YKfEO645c5Fu2KonkTswrXHawCRlP3
L7c3lusXkN3a7V3LZwu/bzvwBdWcOm/jutSLtogOmZydH01HO9COtGPezu4l
HkRLXPYeGqdcS393YDzJaCy8NnVzCcXLP/0cbJhBMbPb0ZPmX/ZtcP4RQ9IN
NpPV1vtR2Xo7IZqQhMh9vwmVLvQu1qVS5kMk5ZanCNGWtpPVjgOO/iRZ/kn+
7bAiyozdMuPtsGDaX2f7QvZzOm0rPuWw4kkdiUW9LlrxJIGkUrdpbW0Sfve2
6S7dNq9E+D0RL3ZqXhhlPzO8tqAyk+wnHgjoHZBOk7uIHB0V6kGfJ77LVKuJ
pjnb3Z/JXYqj707ndwVIOpO3wUNnM5EwcnXpGrZ9URLNeDAnUHZFNnX7MvzB
scZ0uoh1r/u87v+Z17dinyrsk4F9fHHuQpwrOonP9cc9H+Ke/Qz5njvxrj54
1yi8K6qMebgKHmaAhwDw5gTexoC3OeD5F3ieBZ6lprEuQ+awLtbQZQB0XAEd
yzazjtLQfRp0nwjdJ65jn0xZxz6Zf5p9ch2+ug5f7YWvGnqwDxnf0Ndx7MNL
8O0l+LYAvp0nxj5njKJY+PwJ+uIJ+kIKcxGPOYrHHGlhjnagL3agL7wwd6sx
p6sxp28wpwaY65uY65noi0/Igc/IgckTOAdq0Rdf0Bev0BeiyJlg9MVM5Mx3
5NIX5JI3cikcOTYPOZaCHFuC3At4zrnn5sW5Z4+cHImcnIKcPIRcTUeuSplw
rk5BDuchh23/pS+WI+fHoi9SkPOB6IVs9MIK9MVarJfu5nUT9MV07PMDfeGO
ffbg3BM4dyfO7azXOuuLAvCwCTycBA8yD5k3Y/RFJPrCATyPBM9vwPNx6HIQ
unSHLi+h42b0xVfoOAJ9MRF9UQPddeGTs/DJ0/98V5xhX309w77KQ184N7MP
GV3p3k/2YTL6Ihl90WbGvpWDz+Xg818r2OdD8H01BN9X/fF91YZ5EUGP7MK8
nNBxU98hEUAnO9CMVNLzjygMFfbD6OWDpNxK6WwHhlPtLJe105OF3w2TQq/u
P5BLwzrQj2TONFiH/kmidwXy47tc9KEfHWhE+55cPHnNzIk0XXev+VydRg86
0J3Cj0lP0/gcRV1idUav3FNLWzswnkz80+vMLpTQ8Y1VMt/04yhyZzu+puFH
tOy/7gqlXbPST88X8aULzzKEaEwtZqIpK5c4k5SR8qiismRK3rJBiK4ktnlK
052+kaTS/1ug4plqWu0zJ0jxTCxtD75Rm9T/LQ0Ji9kivracjk122Sm+VkCN
d05Pk7DKoce21vq3MpMpb2lxn3uZrnT955prRt0iSUZZ5b50RjhJrC05MzXT
ip6+b/uaVeFNBt81AwM/+VKTfir1HWFCV03XJyiec6HFGhuOXY/1oevLgp9p
L3hBIya2pY2QdKUt1lGH7sdEkuoUB639czPoYsvtv+TME2kx1odifS7Wp2Mf
dezTHfv859yv/zh3Fu4ph3ta4Z5ReFcfJX6XLd41FzzcBw9Sd5kHOfA2BbzN
Bm/vDJlnU/D8Tp157kwXK+jYDB01oeNw6H4GuvtD927wySz45A580q+QfTWq
kH1lDF/Jw4fT4MP58KE4fDsAvnWCbxXh80XweW/4fAu+r7ZiLkwwF5qVerPf
9K4GxtIMy5LIbQ+LqEJpqs3TJeFU1YFWlLIqRFTzjjcVGkwKPasZTcUdaEd/
OZyZsulMIPnUzahbNayO/Dswgcbs19tnuauU1szyiQzZm0SXO9CFfK09fvrK
CGju1tCxdrd86cv2dhTmzBvXKetqnenJnS4hC4ozaeHjdvSinQ1djt01iqNf
rbOaw6JrKVR2thDj6bDKda+PzcLveHHjlcazUml/lrUQ39By27C1xWWRFNY1
V8LNyZ/iZxZJujmZ0um0tduGGrnR0vP3YhyWh9LbCt8Wh+Uv6PIPTaU7a7yo
ZXDQmfRlqaTs9+1P1rI3tKJiYeDeeiE/q+XHnX9fQluTJXaGvA+jgJcFLxQ2
ZdHG77Wh6v1raZeKf/+Rg4T3STC5tkOihKY/+CW7bH0dVdTV9BdoJZLjLZ8T
3tWlZFKiU/JOsZ5cX9eUjm8tojkFCodCZKrIAOs7sS6F9fnYJw/7vME+y3Hu
Jpy7DeeOwj1lcE9D3DMa7+qLd43Fu0TBgxV40AAPReDtHXgzBm/fwbMTeD4N
nkf/YV2qoMtz6LISOjY+Yh27Qcc+0D0GugdD92L4pP9s9skAG/aJGXxlAV/9
2ce+Og4fnoIPb9uzD5cvY9+uXMa+7bOafWsNn1vD57fhc3P0hQW+r0ZhLvQw
R/qYIy3MUQ3mrhZzN302z13SRJ7T5Ik8p6HaPKdv0Ref0BdbMNe+yIFq5MB3
5MDvGM4NNeTGY+SGInLmGXKm9DDnjB5yyQy5NPY559Is9EU1cmww+uIAcu8Q
ck8fuTcUOXkcOfkVffE3cjUMfXHmv+wLJeR8DHJ+FnJ+N3phH3rBEL0wGut5
k3m95jevT8U+ctjnxwTe5wXO7XOVz7XGuZ3ds7O+2AoeXMHDMvTFp37M2xdv
5u19EPPWhr7wBM/N6IuZ0GUtdIlGX4iosY4au1jHHPTFYPTFCejuAd3fubBP
BsMnZ+GThegLVfRFKnz1Hn3RgL4Yr8M+/DCKfftxFPvWCH3RBT7vCp8L0tjn
XfF91Q3fV13wfdUFPdIPPbIX82LeV3PQjRJfCuhAY4pP0y4Yq+FCkl0ClFVP
5dHoDvQnrS7VOaPXpFDVxqg7GufK6WsHCmjPhCa5KTk5dEeh4ICyQhh1n92O
L2l/xOvw3mVedLi2JqiPkoAi37ejDW2ttHF69MyXLMX0lQ2kKyi0TztG0v2E
l5vtX+WShaHsaNNfxWRS046hFCEWpPsyJJMMy26/CVotoKT3j4RoQ7csJm8/
8sKXcsekX+83wJ8MlJ/d6DfAlL55+1QOyHalqmPdD6THJFKXtqfH0mOcKbO5
iWyDI+jl0GyDuiXlNEtr1+u6JQKq/+TycezTHDqu0nTvlnQdWed/a7grnUBG
f2aIOe0rJVIYJHLKr4Rca7aFzfQPI2ONnDyZmVkk7iXyd6teJk06vWVPuKkX
DXukofphXBz9+Krm57I5iV72kTMT6+VJlS0K14Z8EFDT6q2mp+3SaXVB3b5n
BoU0YqDNlWtGOVSB9Zp8Xq/6i9e/YZ9n2KcU+/zn3In/OFcT94zHPV/jnmfw
Lge8ywLv0gMPg8BDJHjwA29Zf5g3X/DmB573g+c08NyZLu+gYxl0/AgdL0P3
F9B9H3RXgU9s4ZPp8EksfLUMvroPX32GD9vgw9PwYZoI+7ZIhH3bHb49DZ8/
gs8N4fPNmIs9//i+2iN7+mf2sFo60IHxFKSQPHf1qBLq6p6f+bdtFEl14CuK
K3x0e9D9AFqQdbBitXgovenAFzRQ+WrKI1kvWn7zzwwVkzI624ERZDtBb3Fh
cDbtqX7Z/6V/EfWrbcdgmrFhbEKkWgbpJe/KfrMihG43t6Mlfa4KVp7q6kkB
707MqFoRQW4PTwrRmsbeTVj0YqsPrflQ/mmYfRHVnHknxGDqZv9h9vSVGXT/
2vc6DbEaEsSI12uIxdHvM5fsGuLe0tYD9k5iGulU/ea9q5iGB/21sUz9zrFo
umYc5RS/KZRmBMmEJWx6QWLX7R9oqXuRkXduV2VdH5o9+/3AXbpGVHb1aZzm
UydSupb38+PPcPqq9rGxtcWK2u6tWmja4k2ffQr2NnyIozEzbs8/PdKR5l1b
qB3fHEq2fgW/ai6n0x6VQv+hRd4UqJI+6Y1ODCWFtUXkDiwiHb9NHyePKqaX
QU5brg17SyZYr/Xl9cVY18c+XbGPHvZ5i3P74dwROHcA7inAPctwz8t4lwze
FYl3hYGHk+BhCnj4G7yJujNvc8GbI3guAc8jdZjn4gbW5QJ0CbNjXR5BxxvQ
8dMd1jEQukdCd8l3rPtP+MQePnkCn/SGrxbAV1vhq5BM9uF0+DBqGftwMXyr
Ad9KFLFvH8Hnz+DzRvjcHt9Xnvi+moS5eI05isEc5WOODqAvtNEXAsydB+Y0
HHM6EnOqjbn+OIvnWgNz/Qg5UIscuIMccENupCI3rJEbe5AzO5AzlsiZC8gl
T+TSAeRSOXLsMXJM1IdzrMdxzr1x6Ivaf+mLncjVZ8jV6/9lX/xGzvsj598j
53+hF06gL2ahL5Kxfg59YY2+aMI+17FPIfYZjXPn49xpOLeze3bWFyfBgyh4
cAcPe9EX+uiLa+AtCTxfAM916Isg6NIIXfygywUj1lG3lnVc04N1vAHd7aD7
Mei+ED4xhk9GwSf9Z7Gv7qMvAuArHTX24XU19qHERPahP/oiCn1RI8K+XQKf
74TPt8Dnn9EXLeiL75iLgVjvju+ue1if9LTpkp6kD+01bEcjEvH7IlUi5UTb
Uq6l52dlEKW2oyfVLrEVTWv/d2yiZMfeqK2h1dHtGEe7ZzW9f1VRTO9at5tN
7B9H9r00hfiaVCO09TwzQujgqjKpfgm+ZPy6HY1pjY7Zng0LXWiIeg9Ncet0
Wp7Rjh7C390JMyTio+nxtua/DM7VkVbf70JMIAdpncEHckrp2uto+13dcmhP
U4UQfen5/kWP5yonkqJ05MmZM8NpWMtA3ZkzrcjmROXNXfrC/I7eeMX0h/C7
3Tgr1vSHMf12EjsV+rcL3V3/5mdtRTRZF+XveF9hR9q3qtJfDQ8ig0vF4hbl
2aQ8UXyzV7kPqS8fbd9DKpHSuh2QHTapkoanNYtIy0aRyTlb2d2NeSS9oufl
uWpCHyq2/Bh7LIFWbF55UUT4Hdw6ad6LJZ61ZGF631NpehJV9rdYtfFKKVnY
HLVTUa2n/oVSJGNfQht7tNgcS6+ix1gvKuD1xVjvIcv7WGKfD9jnP+fa/+Pc
zu55Cu+ahnctwLvOgYdH4EETPFiCt49GzFseeDsCnleA51Dw3Jkud6HjFuj4
Ejp+38S6j4fuS6G7MnyiA5+Mhk+k/rCvCuGrJ/DVFvhwE3x4DD6Ug28HwLdu
8K0YfL4EPs/xZZ9rYS42YC4CMReL3aP2xq+to5sdmEAjLXQv/bIqJdPn4xef
8YmnPRbt6EinRNxTHpWHUfW5xSlaQ/0oTr8dTShvyKMlZ1+6UN7nF+fTVLLI
/1s7Cn/HnozR6DY8nlYdW3XYY3ItiZ9rx3jqm6B53XJOCV1SGmNdvzmRchXH
CtGZ8u9I2nmdFeb88a2Va19706BuB4VoSHs/zJAe9dSRnPY1zrmjHUNHWuXn
3tG2p25jw5TrU4Jo/BcX44wIoV9kRUwzIsLJr9W39Xp+Fi1rKh2W2q2WVOUH
jE3tFk+5ba9sBaIlNP39hF/PvXJo4YyQyRZevqT3XTJmycgk2qlaMlfpUQyV
2S+NPP3InnL8Tx4u+RJExefvr8yRDKZlWX+JNshYUN63leMcP3rQm8o9mcek
fKjFtEtQWrkRua6qiChVc6L8HJE1zc5+5Do+9dv1jfa0pUlhX5GTB41M2C8d
phFN+mPOeKT3zKZ+t+3Cps1Kpa5YV8R66C1ed8Y+P8fxPhLYJxjndjPjc4Nw
7sALfM9zuOcP3PM63iXqwO/qGsDv0gQPB8GDN3jYDd6Ogbff4C3/M/OcNol5
vgWej0OXKdDFbwzrEgsdFaGjEXSMgO7DF7Hu3e6y7jfhkzXwyVb4xBi+Ogtf
zYSvTOHD8/DhWfiwFr71gm+z4Vt1+NwCPl8An4fi+8rj0//89z5mYY5OYo7E
0RePMXdnMXdTlvLcyWFOJ2BOFTCnGeiLp5jr2ZjrR8iBUOTAFeTACPTFWuSG
NnLjOXLmEHLGCzmjilySRy5tQy5NQI51Q44ZIsdSkHsj0ReDnDn3niInHZGT
l5CTbsjVE8jV28jVUuTwZOSw87/0xR/0hQtyvhY5b4VeGIW+2IpeuIb1l+iL
SVgXQ188wz712GcozrXBufRf9sUu8PA3eFAED7rgLRZ9YY++WAmex4FnR/Ds
C12MoUshdDkHHZdDxzvQsQ19MRW6r4bui+GTE/DJMPhkM/pCsjf7qgC+coMP
neDDPPhQCr4VgW/N0RcVT9jn0vC5I/riE/qiAn0xCH9+JYI/14r9w+v5Q3jd
6bfi9SN/eVOpzCIhGtLoZckVUTmv6e6YDxauw5Kp36J2dKV383TL+zwR0KyT
DesTqY7qLrVjAsk/rKkMeVxKXt0czz+RzaQY1Xb0on1DMk2avGOF3zOrr2yP
9Ke5t1WFaEoaX15If/Fyo9CNp7L2ZwhIe9IFIdpQvsW6nmHL/UgjaP2P0eVF
pDbshBCDafPV2Qt9j2fQ1cvPmsPaamhbpci3sLY42uw70PtNWzGV6uzZPela
JtX4HTg86ZoXmZVfvHhmdBw1yAsWvDgaQUNubfj04qg1ndlg7Kd804d6/tgg
ZfbHi6Zkl390/vOMJE70ylUKe01OoiNvPtkRSoqBO594a72gy9ss/j/Gzjus
576N+4hoyMgIUbIqoYwIdaIhm8rKKIqo7IpklV2itCntpb333mnvvadIS2h5
uq7v+/rjcRye+/nrexzncdyf8/N5v1/n++x3X9d93KFjJ4NplrZBlG1hOs1b
lD7vabUHlfWJ/NZ/EUu+TZzLf5UW0IwWg6HNm4IooNpHYPeyNBIzDOT2mVhO
Z75vFiy/E08vH8TXNjjm0ZR5cR8e8TVTY8bEqREeDdR/tb5oUXMjfZ7L1KNR
L0R9Ac4RxTnqOMcVfSejryv6TsU9OXHPbNzzNd61FO/Swru4ocNm6MAPHX5D
N2Ho9hK6/YLOEyMYnaOg89984YWPPPBRDD46w/ej8D0cvvuAk5PgxBCc1IKr
HnBlB65EwWErOBQCh3vBbd52htt4cJsKzofAuTQ4/+9/D8L+m5kLFh5mLlZH
9dUXWHfSzbh/vln0/NK3+U9H6um42+oUzRn5VB3/zzeAWPcOR7lcTiFLja2L
p4qFkkTpP19LauJ92SIv6U27j4/0rtyfSVe+/vP1Ipe3Bv6iKvEUfS/n6TLl
NgpkyR3/ppNHHGn49lRTlNOPWbHmVcT9amT8G01157aZzH1fQG4Xf/rGrk0i
h+Ub/GLXOtOndbIVWnfD6HfQyZfKo2GU2ZHzSnnUisS+vBIJuudDpbll0XJt
GdSqWJ8t1+ZJuS6vr1TUx9HOwuMJDS61VHbPcUGjSxxtq6g3ynlaRAaX12Uo
2HbShKdJS4/aZlF0tuOMS2P1pCGWd5t7ayP9WiVeuXlrEl2IWXTV+UUpzVyi
8Pvi02Ky0P2x5oxxCOkM5+8d08ikwsG82p2Ls8n2l69xQvVHer9QsoFnnCfW
0e70vbJp1NQ3drtwQTDp6XUbbVCIJ9mcDDujDTl0fax21hKZatrTpxrwzb+E
BFHnRp0bdR6c8wjnnMA57ejrjb5e6LsZ94zCPd1xz4d41yxB5l1GeJc+dBh7
wugQCR2mQTcH6DYfuvlD53jobA+dC+CLHXwZ7GJ8+QQfU+HjCHxkc2Z8V4Pv
M88zvt8CJ5fByUFwMhlcCYOrg+CqBBzqg8P1ixkOY8HtjgSGW1lwKwjOr4Hz
x+A8EHtED/vCA/viA+aoBHPEjTkqxNwdxb6YLcHMnRzmdBhzKoM5dcNcR2Cu
FTDX55EDM5EDK5ADWciN+8iNDuTGa+SMMXLmDnLmMnJpF3JpC3IpFzlWin1h
hByrRu6xIvdOI/dYkJOrkJOcyMkk5KoSctUEuboIOcyPHG7+H/tCHDlvipy3
Q85zYV/0YC/8xl74hH1hgroP6vNxDjfOOY1zLNG3r5npa4a+f9trJnjXErzr
Mt61DDrshg6boIPQOkY3JegWD906oHMPdHaFzg7wRQ++6MMX/yjGx488jI9f
HjI+esH30/A9Hr5HgJML4MQMnEyfzHDFf4jhqhRcsYPDBHA49IrhUAzchmFf
+GNftIPzxUsYzq+C80/YF+bYF8exLzSHmfrFsLB/6+c2MvX4h3K723RD6YmQ
/PjXkpYY81SqWHgTi4K2fEJzOnWYXRn/epBk+POFAfaxZHAnsMKrp4XE5gSN
f1PpeVwNV7FIFaWVvlMfyakhls8O499YWibYGL94QhFdY1nHrRabRKYc0uNf
ZzqS/rvcYHU4TZW6orPh+Pjv0J7ZuhuOW5PsUNCRCV99KPljziveF9nUn3H3
Ne+Lj6Tiv6lillUiRT274FXg30STa+f7FPgn09D91cVPlMvpljlbmOfZNuIr
nZfteTadYuTeeB4ZrKYj1wqzX/gXk9VpAUVj/xDSZX2nsvpdJp0SGP9TsTyF
7vv27Vcud6W7uQZ3ZuRG0kHFrTasqZEk8+jIbPkMWzobkve+4rs/LbhzQ25z
WQjNDQv+4sZiSWdXfDDkL/ciiwtXP4f6RVFVsuuZhyr2tNynSzX7fCAtGQ0R
VbRNpOmPK9oD/YPJTcrJzedbFHUZnC7nXvWJ+JzWDifYVRP7tav9btYlVIF6
qyNT77jK1OfgnK9GzDnGOMcYfVPRdzH6cuOeM3DP47jnJbzrLN71AO9Sgw5P
ocMT6HAFugVBN1voZgKdN0PnCuj8N18s4WMafNwFH7skGd9r4bsAfDcBJ97g
RBecTC5juNoFrvb+xxU4XAEO74DDiiMMt3Hgdiq4rQbnzuB8Fzj3wVx4Yi6M
MReGXaqO7xZ1UjT/ufFvFgVukNhteL6enLvm+p69WUzcJvPGvyF04fWi44MH
M2lH9/wve9dHUecn3vGvHW1dyudZ+yCANtUXm3CkJVLI1KrxrxMJnMu+enAw
lN7Vi7nIxlSRd/O28W807dJWfcoRVkDpZXarNwS103JT//FvBnHdfTr40KGW
UiRdzt18UECn9+ufv/kgkKrMdgxxfUil9HLqmeIQQ3zGRVNYHd7TKsmqyGW7
gmh32lh2TFwEeTZduhUbZ0M9oxXtwn5+pJehcLTAM53GnAcuFHt60MC3/LVp
R2KpbdPa06n85TQ4NHVmKX8EuW4UdfrVkUNq/J3fh0630iTW/SvUzqRR89Lt
CYqhVfR5yhe7qEudZLwm9KDG1SzaP8vuimJqPX1St3o436eFdGaO5B6emEYd
l+aEaHVV0kZ/H2/9tAay6NE8EjeYRic0WccEpctIQLFOtUWolYQXn6xoV2+k
lZHveS/ubqbfCkx9hJepj0YwdVmccw7nnME5VehriL5D6NuPe1rgnsdxz/14
V9cU5l3peFcKdCiGDrehw0HoVgbdkqGbEHR+CZ2zofOECsaXk/BlH3zphI9G
8JHdnPGxFb5Lw/dN8J0fnKwDJ4NaDCec4MoYXH1TZbiyAIfbcxgOXcBhDbg9
CG4DwK02OHcH51bgXABzsQRzsRFzEYI50scczcQcSWBfLDFn5u4R5s4Nc6qM
OY3GnAZgX3ztZOZ6Lub6EHLgCnJAGDnAhX0xoZfJjYPIjV7kjEAmkzP3kDPV
yKWVyKXlD5hcUkeOzUSOeSHHNiL37iD3jiD3lJCTesjJG8hJQq5uQq7uQ67y
I4d5kcMaf+yLyj/2xTLkvDT2RSpyvhd7YSP2xTzsi1TU3bAvkrEvZuKcTOyL
WzjnNvo6ou8M9P3bXvvbvjCADg7QwQU6fIBuTdCtDLoZQOdV0DkVOtvCl+Ya
xpdC7Itd8PEW9sUM+NiHfdGCfbEavluAkyBw8hCcKGFfvMW+uAeupoPDFG6G
w7FYhsNM7As/7Iv+MIbbEXCeDM61wfnKEWYuBMKZuSjHXNy+FvZvnXUw9t/6
vWNv/61vuOaUmNoQRjsNgsa/VjQpL6fn+wkf4lfKLHq/NoXygkvGv65U9Oh+
xq6hCHITnz3FSb6eOP0Wj38TqL9AhEXdtJh0vrG9PindSmba88e/aWST/aZ3
nWUVOfbt/tZy8RMdGWwf//rQgp9Ocw/xJ1GYxPCXCpEI6nvwvrdCxIauLzgv
NOOnL8meyztOq+IpZ81nbVr1gU7rPbF7XhFM5T21JmprCil8yypvtTVBNCVh
lV7snTRK8Fjnu86shXJHHk4XNUulPvc3AdtCK2mg6rvQJdF2StDZvkxLNIOW
/0ztMxqpISGpA6Nx3hU0e9PChH7v8b8nPnF/3VqSR3MbqlxW+n0i9sjWXbeC
fIhzVXZU0v4kql/qvtvqejIZzdm2RNXZhX63bTj0lWf83tcSrhiqx9K39vAy
yedO9P2GqchMpWASPzBZUXBPDPnJ7/tq7BFKvdmrnCp/RNDWj2bdTeWppOKu
9/70uTKSuq/C5nmxgFajvh11IdQ34BwTnFOPc4bR9zP6/kDfNtzzBe45rZ25
5zq8aznexYt37YQOwtDBBDpMq2Z0K4ZuktDNAzqHQudi6FwBXyLgCyt8WQgf
neHjTviYD985HzK+m8P3bHByD5xsBCeW4CoQXCWAKzNw+NOX4bAdHE4At/7g
9iO4PQLOVcC5IDgPxlzsxFw4Yy6unch27pnfQb36OePfTOrSk8z+sriOfozu
qB9bVkEH1+we/0bSvqzZNmMieTQp14tSjBPI0aNs/OtIR7flT40ODyH9KtPP
NbWRdCLl1/jXlg7t279CaW4Axd602dzQmE9X9TZvaWgMoNVeRQIiX1Po+rkD
91o/t9F8zcbxbzp9j3la/cG9hiLUJjiKOjaQfK2rs6hjIrGdupwxkl5Cq0v4
nKVNs6hdLbha2tSbll3htLXsSaBTF1U2WFtFUeG8Jkk7Kzua8aXcaGQogPIC
1vs/bw6nZQ807jg2W9OxN/tfXbP3pUeml/hu1SaTosEk17paFzoqIpT52TKC
QoOblPhe55O38dcBrbcBFMA+Q+hMVArVTjOoOnW1ktZNSpdNehZFl15+Wc1Z
kE+HM0/uSjvcRActDxlPbksmNb+v6nLs5TRhlL9a42cblcy/uPtFUy4NGA19
YbtRR221msatH7vo9W9tw87cBoru7nqqsb2NalC/jbo36i0jzDmaOCcO5xD6
bkXfA+gbgXtOwz0lcE8bvOsV3uWAd6lBB3HoIAodAqHbFOgmCt0eQOcB6LwN
OmvBl1XqjC868KUePl6Hj9vgowJ8/3WZ8T0fvhM46dVlOKn1ZDjZA64EwRU3
uNIGh2yeDIc24HDHGMOtDbj1ALcHwXkBOE8C5xsxFynfmblQwFwIYo7EMUff
c5k5ksHc9WHuJhgyc/cZc6qAfcFXyMzpZcz1U8z1S8z1ZeTAIuTAjx9MDgQh
N7qxL7SRGxeQM1+QM9eRMz+RS+XIpbXIpXzkWC1yjNWDybE27Isg5N5s5N4S
5ORU5KQKcnK0nsnVvggmV0dXMrlagRw2QA7/+GNfdCO3B5DbMsj5buQ8zycm
5wl7QRN7QQZ7YSHqPKjPRl0I55zHOWnYF1zXmb4t6Psbff+21/62LzSgw2Ho
EAMdVmNfDEK3S9AtHDqnQud27Itw+GIFX2riGV8KVBkfj8NHbviYDd+nYl+Y
wPcacPIanBD2RSy4qgBXveCqaRPD4V7si7vYFwOKDLcu4NYe++ImOH8Ezo9g
Xyy/zsxFCPZFB+YiQC7q37rR5aF/68973jP/3PDhpkkChyPI4X3G+NeG2i60
Gwkv96MXd1K1Luglksn11doX9Jzoo1J++phFKC2uKZOs3VBOXKliUrUbIoj/
gJppyYRcGlR7IZL6tJN0NL+Mf7Po7YOUzZva68l7RsCO6LBSiijN3xUdFkbz
p6gVCi77RNdsiw6zSybR/bFpKuySzsRVYdmx/1UY6Zda7jMPDCEOu4h480AL
ionZFRNm70Wv7kzftetAMnXsUTopfcCFWE3O81g0hNP7nrlc2ZOKaYtj+5rc
SSF0ueLNyfspGbTWWeBAdVgz/eAXDWkNSyEt7+4oJdcKWnv+9afHcp00aWFY
XZFcFvl7bXu8xbaevrGyiq11aqTdX/e+KXAZ//sjuUKpaVoZhfbUnw2zKqNz
Ot4KvBnhlHPA+Na93TnkFfR1kqtSAQX5e2pMlQsmJYfymuVtqVTJX+x2MTmH
LPXTUvKnJ43/vbV/46KOdNq4Z4ZkWXMR6YR02SwNbiKuE7fCOZfXkDDqR1Bv
O87U63DOA5yzFucEoG8A+p5C3wTcUwv3rMQ9x/CuY3iXEt4lBh0mQ4dg6LAY
urVBt2PQ7QF0XgSdZaHz33xRh4+DtoyPzvDxKXy3gO/C8D0fnNSCEzlwMgKu
7oKrd+BqJjicCA5ngsNr4PYmuDUBt0vAeQw457zIcN6KuYjEXHhjLiJHflWw
Z7XRvh2clexZ6ZTNKfnmgEENhcWleCs/q6VwiR/j3zialc7/8eetIorPOpEs
lZ1CjnZcKVLZrrTiOu+tvNhI4iWNxI86ITTr1vOUjzoW1LBoz43SI15UeyFr
VKchlX71sk/SbXCjUwL7pd7yRFPCrrlmR5OrKV1X1OZocgzxBPbc1T1ZSHEn
7z99ptNJ64IbLZ7pZJF8uq6wV2E9TVmyTKrGo5KCn6Vn1XiM/z5W2Ru5Z1oB
FSxS+/3qeAax67OSxXFPmh29PP/Gmjjy0b8UI2oQTd9ul3Q9NnhHxZ/WVk8z
DSStLZJOm5eF0uQfNyUCV1jS6kE7f1cRb9q/vbNFsS6Wpl+bWX6x356m+R7L
tZ4eTIc/OvBd2plKPkGiU8Xd3Cim7LKzrmYUcelw3J4t9Yn4rz7cUc0fQNme
Vq7LHyfRmeVC6p/nFFGCtPt+n/Zkkkw3YE988YnybY112hbVk8Hi6g/LJFqo
d6OZtklzI9miLod6FuqbcA7hnJI05pyJ6DsBfRPRdwvu+Rr3fI97rsW7vl1l
3vXdh3nXSejQN8joMBc6pEM39juMbi3Q7St0XgWdxaGzAnz5DV/YVBlfMuCj
BHxUgo8v4Psb+D4YwPj+AZxEgxNOcDIBXH2/yXAVC65+g8MGcKgHDqfEM9yy
bmW4lQa35uB8OTi3A+fXMBe7MBcqmIvBB8wcWWCOqrAv3DB3npi7XMydAuZU
GnN6HnM6gLm+ibk2w1w/QA68Qg40TmZyQBu5cRe5wY7ceImcWYR9UYCc8UIu
Td7L5JIQcskdObYLOaaLHJND7s1YyuSeMXJPGDk5toDJyY/IyTrkqjhyVQS5
+hE5fBw5nPDHvgj8Y180IecTkfNHkPNbsBeMsBeWYF/woS6MeiT2RQXOOY9z
FuOcMPT9iL4q6Pu3vfYb7zqOdx3FuySgAyf2RSx06HdidIvFvhCAbm+h8zro
fAY6X4cvOdgXzcaML1LwsQD74iZ8fAjfTeG7AHyvBydfwYky9sVEdYarh+DK
CVy9q2Y4NElhOLy3n+HwErjVArdG4HYH9kUtON+MfXFgNzMX/JrMXHzGXLzM
LPm3LvQ82OafulBYw7/1lrMjgjOTx39fXxZYMzPZlrb+ktlo2edPp4JvNq0o
jqWo2GUdK4rtKa1A0v8HSzDNHtGS0A0tpG624m26oUFkGq/64u3SdBKSKz7M
pdxO6UqHFLmUM0h5kWeruWgtWR+uOWfW00CvNy6+ZNaTSNEOzQahIqWkGVg9
Q+55Ns2RKj0l9/wj/Xh4qdjEMpEGbue2K2yNHt//q2cc3fqOdgjmzNVRD6Qp
Sg+lv1SEU8fDwfvfK6zpd/+Ii9AbX/rU0D6hMy2Fnqot5ZiQ7kq+EoEslyMi
qZHvxfxt64pIaYeE4P11wfT9mvW5hTHppPDNfaXX2wbSn8Nx4IFFIl0P7Yh6
HlFCu08KvL+k0UEntG8cvnYlk06HSl1LNaoj1UtdFo88O2il/J5bOWWZdDk6
LnBDbR2lnPI41aPeQgJn7cxyv6TRvpCchGUNlaTz3CK0ZUET5RQPWr5ZkEca
2wvt5J0qqSq1YZdmRzs5s61WE1ZpJsn2KYrZrq2UjPoL1FegfhfnROGcSzjn
E/ouR19F9FXHPYVxz6u45xa8aw/etRfv2gsdbkCHC9ChELrJQLdG6BYLnfWg
sx10/psvM+4wPu6Hj0fgowl83wTfOR8xvoeAEx9wUgdOZoErf3C1HVyxgcNW
cGgEDreAW2twGwBuWVQYzm+Dc1VwHou5OI252IO5mNzEf8XnbCspuLqPf9Oo
Yq+x/LHwKkpranP97tlIsyY7uH33TKKsZVzT+2eVkXj19Ovl1Zn0bbBap7za
i/oqZA7kzk4gNZFo86bwcDrk/M69Kdyawkv4t71X96VapXTbvTFxZK+Tm703
xoEmVca6c1kF03tpFbcJDwvoGvvp7AkPA0lcbrAs8UMqreX9Ghzb10y6lRU/
YvvG81+huUfrWwXlj514fXJrO+1v8QxT3ppBTT+St/VMq6X1M5re7N1aTm8K
xuYc2xpBR1OWzm2YlktmLwx69GUzSS+rdtaYrBd9GerT+XUsnvbvtBIN2hJP
xprxaufpA/24ktwi0xJM7ueOrzY1CiPRofzS5GArElP91CD57SOxDim+ktod
RhI/dz13SLKmOZHLu91/fKT7VWsSA/2jKXawaUOunit9/TjDj2NTMEmV9fXK
lCYR76QG3hGdRFqjffk5/4ok6unSs/Jb/4k0X1se1xWtI9FJgrPaWkqpCfVN
qHOjvhDnuExkzqnTYs65hb7v0LcBfft+Mfech3v+jGDuGYJ3SeJdu/Cuy9DB
EzqwXWV08IdultBt9jCjmxJ0DoLOD6FzD3y5Al84fjK+sMHHw/DxNHx8BN+V
4Psc+G4BThTASUkFw8kTcHUdXDWAq5fgcNcPhkPlSobDr+BWAtyOgNvqRoZz
IXDuBs6PYC56nzFzwYq5qMa+OIQ5Woc5eo65a8XcNWDuxDGns9mZOY3GnIpi
rnMx12qY69vIgWvIAQvkgBpyYzpyox/7ogr7Yj5yRhQ5swi5NIZcWjjA5FI1
cswKOZaAHPuO3LuE3OO6zuSeMnLSEDl5Fzm5H7l6Brmq9v+5L5Yit/cit//L
+VHkvD5yvgl74SP2ghz2ggfq2qizoX4b57zDOWdwThH6LkNfZfT92z3/ti+2
Qodz0EEBOrhDNx7o5oN98be93KnI+JIDX2qwL2SwLwzh4034+Ai+C8P3KdgX
seAkFJx0gpMBWYYrc3DFA64ihxkOLbEveMGhMLh9AW49wO0K7As7cG4Cznmz
mLmIxb54irl4fO++9T91y3RJl3//vffbz/j+qXenC8U5boym6LxZ5Y4b39HB
xbcrm1UDKfFmrEutVxQFCWqk1XrZkd4SDmNjtkC61Fme9mrkEx04/7Xx1YgP
jTkJ/S6/n0QH9hs/XsHZRO01O8xXcCbTs8dV7WvflFHCYs7zhvfaqbNnmZHh
vQz6tcAq2flULY1OmSP9pb+EaHng9y/9oTR9xqNH7aLZlFN9wI8jKIXuT1Lc
MTPIlbTsz8cdcoik1U/dp8q6h9PkT/eGNdytqSviCavjQV/qSrz4KY4iKcZQ
+tXxHbak7Hzbv/iQP9XopEaFrRj/e0B4c8emVa6kT0UbHn+LIPWP13TTa/NI
pM1yYHWbP/28sTfKa18KOTxekDBPv4J6+yX6fr4d13WC13CJQx5tt+C0f9rS
SP3Tl9onb02mqLI50uK3ymjd6IjMpdw2OrzSsb19aRalpLdINc2tpTF9i162
/k5qbFk6aTZLCW0WV9DfM9RIvzocy36ofqGxez79Sz800nDwitZXv9toEPUB
1AdQ/4lzSnCOGM5Zi77y6JuIvhtxz8+4ZwDu+RTvqsS77uFdu6HDDOjQBh1y
oZsGdLsI3Wqhsx903gOd/+ZLK3y0ho/G8HEeK+P7CfguAt+LwckoOJm1kOFE
FFxlg6vz4OoUONwBDgfBoTu4tQa3F8AtRwbDeTk4VwXnQZiLZMyFGObCeHQg
eMnPZno8Jha+5GcKCd83VF05WEFNI25XLe63kJPi0E2L+6mUba6276d9Jb0f
1pNPFs2nRzqv1JNFA+gsZ+51/QcpJDUYcYRXOZa4DLQTeJXt6ZFxVuKRyCB6
cUne/o55OFVNnOqgbz7+d6mgMUv5Jl9a1PNs5ocHaeQkkqDr+MCdpGMGv8rW
RdPa7My6syfL6fmZfYLnTkaQmXO1YMS6XLqpxCFyXLOd1kg+Mz6lmUGxzb5c
3TK1VNSvzs9Z30pfTB6d4KtPo5x5TweLj1XTic1d9HZ9Ba14u/y4woZIajgV
WnqR8oj115xsP7kcytu4dQ/3IV9anCJ2UMIviaapZO2a/DuZAondO3O9K/V3
c1wJ/xRBHnlpLjrxsXTO7bLr6euOVL+ooFX2QDD5mhgu+RocSWc1sxbo+rjT
Ff6QUsXUINJ0eHloqkAsuT92HlG2SSJlhYtuZ70S6IXXinVXilPJPrrLeZNH
OV3bncA7OLWQ7qH+CPWzqEvinI04p+gIc44/+oqjrzr6JuOe6rhnN+45D++K
w7umfGPeJQIdBqDDJuhwHbpJQrcx6NYMnUehcyN0VoIvM+CLDXyZCB+V4eMJ
+DjyjfHdEL7zwvdT4MQPnKSDk8vgSgJcvQdXE0YYDlvBoQM4nDrKcJsGbn+C
2wvg/Cw4H77HcH4Ac3EWc9Gnx8xFC/ZFIOZIHnPUhLkrxtyZYO5sMKf3MKeL
nJk5PYq5/oa5NsNceyEHipADNdgXvdgXG5AbY1xMbsQgZ7SRM0eRMwuQS33Z
TC6VIpcGkGOpyLELyLEu5N5T5N4r5N4t5OQ25OTEm0xOhiFX2QeYXI1Ersoi
h0eRw6l/7ItDyO1k5PbEu0zO/0LOSyHnh7AXJt1n9sIo9sIA6h3YF1//2yPY
F6k4RwTnLEFfEfT1Rl9J3HMA94z+H/tiNXTob2V0KMG+CIBue6DbTujWAp1D
ofMR6CwDXxZjX7BHMr5w1TA+JsDHOPjIjX1xFL4LYl9UghOWXoaTBdgXM8CV
N7iSAFddHQyHBecYDgOxL96BWxNwexbcimFfjILzO+Cc7T4zFwszmLm4hH0x
LDFm90990Yc9Vf/Ukw7LC/5T93py0b/DPYZe3eXa1un+ntxnWT0I2h1Ecm/N
n2sJR9DrV4akLWxDl7/fsGH97kuWdnMLravSqf3TE1WbKg8yzRH33m4eSx+O
zMiJiaqi2uVdh2Ojoklz+orrc0MLSD7d3OL5lU7yLO1e/uJKFgUZ5Np3ZtZT
cXRMj7ZqA3EE5V+4qppI8zzXGb+4U0Lsl8O6/UzyaLjXSynBxJ/Sir41Hpyf
Qkqsat99kxPp+vXYhI5kJ7ouoLBCY1xvw0Wy3fw3w2jMWbRy4JYV/ZY2X74k
/yOJ8vyKaQ2KIKn2pbU9UTb023xn/zknPzr9Q2Cizffx38/DWWJb2ZxppZ6V
aevmMLq0L3e19bxMypQfKbj+0IvsznCq3hSPp7omTW0TlwKa+yHK9F5+EKnt
rXyjOS+N7pRsOJ7NU0bdTubvjvPFUcj2Ckkn1Vxy9D2zeHt2LZ3Wd6nNZC+k
0dmP7htpllPSdg/+j/s6SHmfZkZcTQtNl5k2sMGyjbxRl0N9UJqp2+KcvTin
B+dooW82+jqgbxnuyYF7Hsc9FfCuILzrAd51GDrwQ4f50GEZdBODbt3Q7TV0
5nBhdJ4pw+j8N1/Wwke+PsbHavjYDt/54Ps6+C4NTlzAiQ840QNXYeDqCLgy
Boe14NAQHK4Bt7rgdg+4TQXnXuA8Apyv2MrMhSzmohpzYVZV7j/Rs4kOVGon
TvRMpuaZ/UuSD5fTybRG/c1f28gltd5s89d0MgxfcPapVw2pbhNdqWNQTC9L
2q/oGISQoejHnUeUM+mauYqwUFwSfds8f6NwnDNpqVyIuCUSTpfFH6eMpYaQ
apDWQZY0C0oSWGJD4V40tWana8m9eEoxbdzXcO8DxfItPsXKFUK/y79NaXiT
Qz2qzVq947/3lpzQEe2uTCLnq49n3S+qoYBlgYPvimIpPcam3ZyliKqi7cPV
bnSQ9+qhDR9uZFKq7HvpT6Z19GnGOtU8zTbyE1JKzddKJ/GFMkeSp9TQsTre
+xrna4hUr8+V0I6lDQbbG6PjC6lr1b1tHqpF9N6D+zW3VTC9nTWRr2tKBnFv
Em4Tts2mFz4ZgtP7fGloxlqe458TKfTKyUSfhjQ6/Nh4WfNgEJ0KWKKwuCae
Flfap217n049CqMOu8oziXVerhhFZVBTGtfYHL8cMvda+sbNsoHYld3zOhPK
qRL1e6j3nWTq7DjnDM4JncucE4m+W9BXCX0Fcc+XuOeUmcw9R/EuP7zLC+8y
hA6a0OEgdGiCbqnQTQG65UPnD9A5HDqfgy/X4MsH+JIDH+Pg49hxxsfGasZ3
B/juBt9NwckTcNINTl6BK84tDFdvwJUTOMwCh4XgUA/cJoBbD3CrBc4FwXkY
OM/Cvuh0YObCBXNhjzl6hDn6gDk6j7n7iLkzwdyFYU6n5TBzGo459cVcd2Gu
b2GuDyEH/JED0ciBAuTGNOTGbORG5yUmZ6qwLwKRMzLIJVXk0mnkkh5yrB/7
og/7Yidy7xByb87b//e+uI5cLUKuuv2xL3iQw+p/7Isu5HYwctsHOX8XOT+b
m8n5TOyF89gLc7AvXqEugnoN9oUZztmEc5qwL86grwP6PkHf/+4554+99rd9
IQ0duKHDVOjACd0WQrdC7Isn0Hki9gUL9oUGfHkMX57DFx3siwvYF1OLGR+b
4fsi+C4M33eAE0dw4glO5MHVc3C1EVxJgUNz7AsBcLgS3F4FtzvAbQc4zwHn
ueD8DvaFLfbFjCPMXCimH/r4T91J7Xw9jdcd1s058k+dbUu//ew5cSS/7/D2
uXMc6Io7y6CJSDAtEJp76rpwGAUffBN+S9iK0kYi5afrfiSOGc/OvRNLIW97
kw57MVfaNOj1fuJYBA0LpssF8ZfQL21DwxD+UNryZOWqOsGs8d8RM9ycXrZS
VcLXbpeXacT1PVuF7UsVSfG+elDL0k6ajYoDTSwZZDvHeVtZag2tFanXklhW
Qf61OQePL4ukd2pvaYZIHl3tfnnNyDWLDq2oCdnu5k0NBjvy5FkTyZyl6pP7
qwR6vaJrXrK5I929mC1oEhlCfD0xF3fUhZHxjU+HqM9qfP/6by1Q9qHGmUHv
93mEkbLKtaEzI1YU2bF/odoKH/ppv9jltXwseTi2vtWtcqDrRlos1vVBdLRj
Q+EG/mSq2WDyOW+aB+0PKzPRcIkg/6NfE6JG02kw+6WPnkUYWS6uNI5uTiTj
pUbb3LjyKWjOXeGmNQWku2Kb/MawfBLtDDCNNy0jpxc8W154tJG91J2EkMEG
Ekb9JeqvUdfBOe9wjhLOcUHfMvR9hr6HcM8C3HMX7tmCd5niXep4VzV0UIQO
AdBhDXSzhW4W0M0ZOjtA5yfQ+W++yMHHdPgYCR/3wffb8N0Lvv9+wnCSCE66
BxhO6sFVLbiaC66mgENXcLgWHE4Btx/ArS+4XQnOL4Dzh+BcH3ORjrkIw1wM
fxP8YnirkdTeztlhdCuJLOesvXe+oJT8xePn9Hp00DmDNY96PTLpmVWWjl9B
HbVYbZywlq+CQk5HSK/ji6RZVwQKf63Mo92xUR1WW9NJWkb07LutHnQ8V/bR
9qwY+rWvPVjTOpImCVSy3be2JX3LM9/ZYv2Jb1e+ms/sCLqjNX12+WwbGrFX
EDxX7kscEepBi9alEOfCHbePr3Ol13Eb3/QORVBvpWnoxsWFdP57eJn54iDa
Y7NN2E49jYzrQ8+5ydUTm6C+TtfuBEq3vOx46FUxqW8xp6OLOshcMIS3dHEm
vZgU/yxboI4ST21Qv3Cgg96s4AhcdSKTlq8QP3xNrY787vNnDW1sokkPJDw3
ayTTmtru4My8Mupx8UxWe19FkfKKh4LexNLrrVIcXvUFtLR723X9m6Uky5W/
jft5AgUFrSMppVyyslZfKC5XSgcGXYTeOxTSjXQWl44bxbRlNb+a+bRa+maW
/rZJt51kFVe7zB1tonmoZ6MujPprnNP3nTlnB85Zhb4r0TcQfae5MvdMxD3t
cc9QvIsN7xLHuz5Bh3fQYT10OAHdnkI3feimBp2bVjE6+0PnPPhC8GUJfOkJ
Z3zsXcD4eAU+ssH38/C9Eb6v2M9wshacBIKTy+DqArgyAlci1gyHw+BQExwm
gttb4NYe3FaDc2lwrg3OV2IubmEuDDEXY5uZOZLEHF3EHK3B3KVj7uowd0sx
pwmY0yOY08lCzFxPusLMtQzmWhj7ogs5sBz7Yity4yJywwK5MQs5Y46c0UXO
KCKXNiKX8pFLj5Bj95Fj15Bj85B7Rsg9PeReJ3JSDTmZhJyc6MDkaihy9TZy
9RhyuBU5fOh/7It3yPlS5LwLcn499oI39oIT9sJy1K+i/gh1TZzzBOdswTlv
0dcNfW+i7397rQr33PfHvniFd6nhXXnQQRY6OEIHGegWCN0CoJstdLaEzg+g
swl80YYvffDFCD5OqmN8ZFFnfJSH7zrw3R2+j2BfxIGTz9gXidgX8dgX3Y8Z
rjK5GA4vgsOm7wyHvwUZbm3BrRu43Yd9YQzOrcF5JuZirjozF22Yi/sHnL/+
U99YdneOzD//3HCi6q9/f4/YutPp+/G01cX3jMb9D2Sl+oLj1PjvpvWtU9zt
X4ZQyy07u48vLWj1Lu19U7W96NQZtblzDROJIyV8B5+hEy3mMm897RxKLm2Z
NpeU84n9eVScvnIArXd7//ixbQqZ9vP1++5oIJd3R+XCdiSSuHKYTNXhElq+
TNp9ikkn2Y1+XMljkkXGTV/9P32tJ7Ha09m5ek20cEbmyxG9ZPrRkPfl1Zxy
Or/ne2HJ0hJ6sf7ISJ1AKLFYs5m/Es4i+TsrnJV/ZpByxv0jg2OetOTzk008
P+PIKvres5F5iWS5UOD42rVOtHWnLO+L8f9cM5GxWG4EnRPVSmp6YEvDHO+2
fv7iR/3zb+53nRtCa72jq9/2WpFUsirLtLdeNKBctq/wTATZXBe+l1nvTI5Z
31OrVgbSA4V5Gi6P4sig1d1ueloojSw6O6FCIIrY5iQqGqik0tYpwcMpnHnk
zBnlySc+/veAqRp3eW42vda6NDs8o56mLZied9imjCahro/6KA9TH+VmzuHB
OcY4Rxt9j6FvA/p24Z4vcE8b3HMY7xLDu3bjXd3QQQs6TONkdPgA3Rygmyx0
Ow6dL0NnUeisBl9ewpfJ8GUXfFwNHzkbGR+Xwndr+P4Mvt8DJ+bghB+cGICr
5mcMVwvB1TFwyAoO54NDAXBbAm7ng9sYcH4SnHuBc1fMhTLmwgFzMefERx/2
8d9N8hJaMzjHfzddXPk0YPr476Zwf+Gsg4866RV/3PPDj7KoYnBB1/mGejIo
8unw1q4lxadHx/y146jDjr/ks1oReYql8NpXfKIZRSG7/Cp8aC53ZLiaRhIl
ffNR7PeKp8uCJ79u8v5A3byGGk5CIRQRULQt+0wInX88a/fZsxb0ZqZN7rlN
XlTRqNSzuzaGBG74d8XVvaeZ5BRmoBlEAVcc86Ik0mmEji/L2+ZB/pK6x/0y
Y4izXKD+640ish47GLFNN5iCrQdzts/IoBhLL9PW23UULTOs++BxPA0EW1id
5yympZxK2u9Ot5GYsYRFwc108r8qYH7wVzUNZ2+6/MqxkwRql5xiH84igaSx
h6tmNFD2veV6exzayepAqJahUBaJms0q9G6vpfelMSUyfi3EMeajFKHyiRaX
tnmvf1pN8u2Or/oetVBjf2BlV3QZbTl3R/9SXT2x24dtz3z3mX6rrfYMqG2h
yY96erfFttPAe6b+BfWeh0x9F86xwTnTcY4T+g6NMn2XoG8F7mmDe0rinv+9
aynetRTvmgkd+KGDLXRwgG720K0Rug2WMTo/gs7W0PkNfKmBL8/gSzh8nAAf
h6UYH9Pg+3X47gzfh8GJKziZvZjhpAhcbQVXouDqEzh0BYcL3zEcRoNbM3Bb
B24HjzOcC4PzHeBcEXPBjrnQwlxYYI7EMEdvMEcymLtBzN1OzN0VzOlizOlG
zKkf5nou9oUE5toFORCBHDj8P/YFP3KGDTnThH1xELmkg1z6bMXkkgRybB9y
jAM59gq59wq5tw65V4+cPIucHMS+6EaubkCubkWufkcOuyCHXZHDd5Hbd5Db
v5Db3NgXN5Dzych5VuwFC+wFDuyLsVdM/TzqvdgXA9gXvZOZc27inJPoK4C+
cejbj3ta4p4f/tgX/+1BObzrF3TQgQ7zsC9CoFswdFOBbgehsyp0XgGd78EX
F/iyDPtCAz4eho/bsC+WwXdb+P4CvuuAk5fgZB44OQyuIrEvfrsyXImAw5pk
hsOm6QyHS8BtAbidDW6rwbk+OI8F558xF86YiyzMhYRMZyuN1wd2DtkqjtdP
fea+KPzP/5/UueWflzYkkMuNUeH9DY7UX3fj7sIv4znfM/9FytMQOirR6vDj
qQVxkvm1nRpe9HZsDXd0XyzxRUTFsPTbk4g4l2Hc/GByepqwcqtMFn100orQ
lfGm7CvvLI9bJRDXxLa9C8UqyHtg0vIjYpG0Jb7N4qpkHvWclHJdM7eNAk4v
Lrk4N50WBcmeeP6umiRYjIdztDuoRjN5ueKVTOpxiE5d+qKOFi/zixe6Wk++
4SInOq4nkJrg6c9i7sW0dWbhfdeoIuJ9c2S7TVIwTdRp3c+mmUECxlefpt7I
pO2O4muk3byoRyz2cOm7eJr3wMDL2iGJDk/UV969z4U04xc8vCQaTl6H3+j2
BccQS898p0nHnUjJLbdJ4lsQpU/d2d06EkGnE46PTr/rTtG39PQGTwdR3a1H
GV1mkfRk4TFeMeMIEtvddeP19Eha2c/JUnJ+/HcJy6xVYoa5pDLMZTWpL51G
EzpFhnzTSNAiSblTupIsJN5v2TC9iH6ivhj1V6gvwTmPcM4hnFOBvufQlxd9
M3DPw7hnBO4ZgndNwruU8S5e6HAUOtyADuuh2z7oNnk9o5scdBaEzrOg8998
kYaPbfBxFD42wXdn+M4O3wcmMJy8ASc84OQpuHoDrrzBlQk45AGHguAwDdyq
gdsl4PY9OE8A51PqGc5PYy7m72Lm4gbmImXO1Oqw8jrKOSGplVgeTwabuoaK
5Iqp6ZL9k9OGnTQ469hUNcMsUvl6Qy6ssZ7W748XVn7dSDIp+Spqr5OIRTJ4
Un9vKVkdu6WZo1ZIC9Sz1n9WC6L3ASt+pMemUWwYj+CzlSnUY69Q1b/SlX6c
9lsi3RNB29dpTS+cFkmbZNUs37Hb0ssJT4Jvc/jTMr7QidvuhJHU6dv7tO9Z
0fpB8znHyj9S9/rOsshrCSRs2L495K4j2b+cVbzeM4S6tpYH2XllUMCWirWG
kZ40d4/paJR1HCU4/Hjgw1VE+7yqiz6vCqbC5/4bPGzTae3q1HlX5lVTzpfk
a7JXYkj0lrJE/c8CkpWLXqDU20RiJ7huhZul0JGa++GGpeWU/1XMXvlROwXH
JpzZWZ9FwzXO/cI/a4lPwfSwe18n6Qi2rC9QKCSRkN+hrAcbKfTpSXUj6S46
5aEo+/toFV33NWTZubmFWH8uvR1s9oVsfZS9jmg1Uf6R+BmTHrTTFNTfov4J
dX+cI4BzVHEOL/pqoO8q9E3EPS1xzxbcUwLvWoZ37cK7uKFDIHTggw7+0G0r
dIuFbuXQ+T10HpVndE6HLxPhy2P4wgUfheAjD3w8DN/3wHcX+F4PTrgdGE6W
nWE48QNXYuAqClydAYfXwKEEOKwFt73g9iS4fQPObcH5KXDOjbnIw76Qxlzc
whzZYI6+YF94YO5kMHc/pJi588OcbsWcSmJOPTDXgZjrIsz1GuyLTOSACnLg
O3IjFLmxFLmxDjlThJxpQc6MCjC59Aq5dAi5tA45Ngs5NnCLybHZyD0R5F4t
9gU3cvIAcvIictIZuToFuXoEuZqKHNZCDsf9ZV+IIrfFkPMZyHkD5PwY9sIa
7AVr7IU+1Gei/gT1+ThHHedI4pwC9F2FvkNyTN//9tpx3DPyj30x4Y99sRI6
nIEOD6CDAnTThG4i2BeS0JkPOk/BvpDGviiCLwbw5QR8HIaP8z8wPrbDdw/4
PhO+f8a+eApOOMDJJXB1E1w9AVd7wWF5OMPh700Mh1ngVgPcLsO+iAfnTeB8
EfbFB8zFSeyLd5iLU5lNfNLj9a7Hb7c8H6/7hLLcXj9en9cok3XDIZGSJole
inZwovTGw2oNqaFU16OrxtMcSqe2Gd3IbLak0IdlD9l5P47/vR786X1+FN2P
mNtjWmBHLzlqXOuXBtLSovnt3cOpdFk2c7/PiBsZNO+JXrAtmuxeekhe0C+i
EeEWTfa7wXTCUqWifX4GsVqJP0wsaKBfQkeqzxcm0pN6dvEFU0opbPGs9VzT
OonTZn/lDbYsui2t7qW5d/z3ad+D+0tU2mjN3l/a986nk7BsnG3cj2q6l3Xp
0p2DtVS16k7cOdU4irrcrCWvWEQP12SWdM4uJhfxne2xciG0ysCWy64lgw4u
lplo3ZFNj+rZgnVW+5LN3mbrqF+JNHyo7F5JZCqZrynvfdDiRSqL9ksM9EZT
S2bBy0yrJHKa51XQWBxAr/KGRhONo+jN0x5tOpBEd2se3vnyK4HaJGxLFron
Um+2/SQVvVRSPrSAbUleIRWFahgfYMshRwnrA6trM2iFB39O68YautuoGxU3
q4TsUV+A+i3Uu3HOQZyThnNM0Xcf+laibxvuaYZ7GuOerIeZd1nhXVp41yno
YAwdPkAHU+gWAN22QLe30PkbdM6Azn/zxR8+ToGPV+HjT0vG9y74fgu+PwMn
neBkBzjhAlfHwJUauFoLDu+Aw2fgsBvcaoHbTHC7HJzng/NycH4HczHtCTMX
cZgL3xdR9nuFaslowbovV4Xi6Jln0+piiSJy91vPY1jRQd/FBxbYVGTSuZiA
Og+ueoqZOFn+/IkWeqJm9urRiVR6v1BQy+hqJemvkOJisSglXpV5O89ZhNGO
uMdnq39kU8Ong0Y7v2WQO1d2jUmPJ708ZKiv3hVHUl08Uo9a46ji7dclFp8d
aGjVZ6PZgcGUyMoz38w2lAwK+w/7uVrSkmElLYMk7/G/s4OD5RMiKLvsydot
VTbEfVgxvizMj8pSPoTamyWS+aO4s9dCnWho2cFXkXGhNGAXeWzscAZt9f1o
4Db+9wCLrKvuKoqjxKOyD7efKKAqca9yn1+BpKe2TtfTOpUeHdeIz9cso73p
7T/dHkXS7D3OIz+Nc6jsSnTmOetaOsfOLR6hnEyPfzUG1Z0rptWuF22u5zTR
1OuaP+fp5dEaMjw6eLmKWtOl494+bKcXdhrPF/2uopIsgx1mD5voWU6I+e4d
X8ihyXVusG8zmZf7FN8bbKcHqFui/gL1LJyzC+eE4hw+9K29xvSdh74RuOd2
3FMT97yKd23Au4bkmXcFQYdM6HAeOjRBt1XQrUuG0S0HOj+Hzl+h8174Egxf
fhxifCmGj6/hozh8PAHf++A7uyDjO0cOw0kBOEkAJ7bgahu40gBXTeDQBxzm
gEMbcNsObg+D2wfgXAmcXwbnmzAXadgXFpiL2ZijOMxREuYoD3O3D3PnhLnb
izm1xJzaYE5XYa6vYa4fY679kAMzVzM5cAM5wI59MYrceInciEPOzEHOGCJn
3iKX+JBLPMglHeRYEXIsFDmmh9yzQ+4tQu7JIyfvIictkJMD2BeWyNUTyNVm
5HAoctgMOfwaua2L3G5Bbg8h568j57uR807YC4LYC/ewF2xR50BdC/U2nCOF
c4Jwzgv0nYy+aej7316z/cu+sPhjX2hDB1vo4A0dPkK3HOh2GroZQed66BwH
nf3hyzb4sh2+OMPHn9aMj2fh4xD2xVf4rgvfDcFJC/bFVnAyWMhwJQWuZMHV
kBfDoQw4vAAO+8HtdXCbB243gfMGcP4FnIdjLuSwL1owF3wiQ037x+trEqdp
LnvpR3PXxtftHa9HXbX1XiGXRNpNRTvZdjuT0qrpQ+02YbR2eUCMokw4ZS6a
OHxTzpr4tK7d7Sn1ISd3oYXmxyIoP/397ssnbUjRZqZ2l4gfaS7QjMtpSCLN
UnPNb03OlPqay/bFyXA6emlh9fCGXEqoH/n2ZrMfCRS86vq2I5lKWx0ctOZW
Egt/p3giTxSVrTnh8/5UPq0rfHjHxamFRGU9O7ncUsnApXO9R1YlHWk6d3Hx
u04yyOkyKnPOokjN14/3TGygH7mpu5qOtNLppWc9qtXT6MCx0yILfKpIPERD
dHlyLfFnTJhfsTKeHvZxL/rqXkQalpUO4Ryl1Cczqu8nFE5urSdGllpn045Z
7Tr1o/l0lXenwr2TIRSTz5J/sDSVzJRyHBMDs0l8v4TF1+QIOno/NHTRnhTq
rhtLuaebRYm/Vx8TX59Ox67NNa0szqAnPXcnLH2fQ1/cOBY7DZWTzWu3gzzL
i0kv8PGTKuV80tvV/pDTspks105bmqJZTXdQv4K6CerPcE4dzjHDOZ/R1wl9
ZdDXAvcUwT2VcM+jeNdNvCsD77oJHYahQyB0kINuotDtJXT7DZ3VofNx6Pw3
X/jgIz981ICPyfC9h4/xPR6+S4ITT3DCDU5Og6vT4CoYXNmBw0xwuBccSoHb
CnC7Adymg3N9cK4OzrdjLvZiLlZhLiwKL6+YHFBNVzie9+oGxNDm7NaC6wcK
KWPt2pkCye0UfYXn54XkDNq0d+PrmOBaSgqZLzE9pY3af6RJXUpJp2ixLPn2
WzV0aVbbvt5bVXTpm77LJN1ocpIb/+HzooAuCFXGStTlUr5TTGFuox/lTHOz
PeaTTF0XlF5uckym3VWfY+rcXege76EtfvIR9HLK6mq3Z1Fk5n5RfZqLHX3x
lxlY+S2ALLpPlsZWB1MTeUc9XWBBJlKp0ktcPOni9IAs/apIMvPSb5O5Of47
6PYBFo7NAfSgp0Tmx5VEUuCZOltlszO9DT/Glh8TSptFVBpinNLooKmHrXeN
Jxnnv5Brs4khvurFYepLc+i04ek+Ka0gap++cr6RVjJFKnSnpXIVUaTOTjYF
/VhqC5CSX7Ami1jXnk9ZoVJO7Cq1rJpPPlHa1UtqfmbjczdJ9TnbggbaKO+b
mDiuy6OV4e2dCnU06/553dCwTorw+DHwiaODWmat2ZTY0EF995i6K+rZqMvi
nNLdzDnrcM6vNUzf6rNM3xD0dcM9tXDPeNyTA+/ai3dV4F0roIM4dLgJHW5B
Nzno9gS6KUPnx9D5LnQ2gi+F8EUPvrjAR2/4yBHA+DgC30/Ad3P4/h6cTHFm
OOkGJ6bgyhRcRYCrRnA49SfDYRM4DAe33uCWD9xqgXNZcL4EnI+sZuaCHXPx
ew0zF6GYIw3M0QHMER/mLgpzx4m5C8Sc1mNONTCnVzDXVzDXmZjrm8iBGuTA
RuRANXKDDfuiFrmxDTmzHTlj/D/2xTfkmBJyTBo5xo/c40Du3UDuqSEnvyEn
PyAnNyBXryFXQ5Grr5DDMsjh48jhr8jtWOS2EnL7LXKe053J+Y9/7Is72AvW
f+wLNdQfoW6AczKwL4xwTgv66qKvCPq+wT1FcU/FP/bFDbwrHe+6Dx1YZBkd
4qCDJnRTgG6O0O0ndD4DnQ/+j30xET5Ogo+74GMafO/HvkiG7xvAiQM4mQ5O
9oOr/eDqA7i6Dg4dweF8cCgNbmvBrQS4LQPnL8H5HXCug31hhLnYi32xRdNt
QGO8Hufs+6TE2I8Us9qNX//z3/caWq9735lEQ+sszwt/caYbptsy3dXDqX7K
0YN6nhEkpqFZaeBnQz6OATKlb/3IVCfmHs+pMCoySuQdOm9FO626WMpDPtLb
pASet3wJlMvWPnnKakealDziK6QbQk7hHtO/VWRQqfjW3KVNnqQk8/3R7cI4
mnetwL5hRTGls9msnLw2hF6I6Xmc682gzsWVt07z1lOULV/cMpEE4jAyCwq+
VUz3upZZqxW3Ud3V18rsLem04XmGyMVXNbTRsyrR+mEnFRaPOFu6Z5FgzcnO
4Y560qkWurLjcStJLeRp3tCQRpOavl04MFxFi8Jbvh2TayCWm1LcPzYnkb1C
aKiSZQltVl10K3xRJbkL2L3r3RVP10satCWOFdCkKpFjKl9KyDpOpaQkLIly
Ik8n7snIpWFtqW+fY4tpoPN5xCrWPBoSNbzwW6WQrKtVPMvbSkn+maJqk2Yd
dQdWstQKVlFWxbnMpIWVxJ0lbCor3UmxVX5bFDuaKAP1aaiHo/7fOZtwThvO
GULfIvTtRV9W3NMc98zDPaXxLm+8Sw/vWgYd2KCDO3S4B93koBsndBOHzsXQ
WRg6/82XXPjoAB+HDBkfp8H3SPiuB9/NwEkaONkBTgzAVTi4GkhiuDIBhwXg
UAocfgG3UuA2Ftw+BOesogznhuBcAXNRjrlQw1x8UxT43Ha+itavdn68Vj2a
zh56Kvvr/jgPnxVulum00RrrsGU2uulkemjHtu2za+hg0q055x530Hw1Hfkz
TzJpTpXZxTtudRRrc7kmrrqesi/fsnlXl0DzJZbtDO4sJq7HZ0+lmxaRkLRg
jMrbYKqwENx7dn0G3UyRHeHdmEHFP4tmqMt4kofh1gniU+Lo8d3Fk38GJlDq
hc1Cc3scach7k+eDshD6ab+c9fKhCGKPvCPkWGIz/nvYqMBMxo8eRaz3OOYd
Qpd5ZQJkKy3pXTzLbK96L1LXP7crqSSSwq8V956OfU9HjpY9qEgJoNvuZt5X
tiSQ6q515yPOu5LiMiW/Fp1QctwwV/Hb/RTSj5zpciHHj+LO2cVMG/89O8mt
yMFOIZNk+w2W2nhFU9FBeZubxckk0KXmn7sgn2T8DX1WdH+i4Jw+31+Pc+nk
OvGy5v4SuriITXvF2jpSdedznmxeSXF6hV3cUjVUGWtbGODZSYtiE5tPNzeT
L+oZqHOhvhnnjC1kzlmIczjRdxb6vkbfTlfmnj19zD0Dcc+3eNcxvMsT71KH
Dgeggyx0OA7dXKEbQbc70FkFOptBZ04HxpfF8GXOJ8YXB/hYBx/ZPzI+PoHv
rfA9Dr6vAyd7wMlES4aTFnDVB65EwdV5cLgaHK4Bh92dDLcLwK0OuM0B5zPA
uTw4X4i5cMRc7MBcXMUcfce+0MIc5WHu+DF31pg7B8xpE+b0KOb0Pea6DHM9
DfsiFDnQiRy4ihzgQW5kIjeMkRsTlzA5U4icWYh9YYxc+oJckkEubUCOFSDH
ViLHLiH3NiD3Bhr/730x+Y99IfLHvtBEro5VMjn8ATmcixz+L7f7kNu/kNvO
yPnryPlpQUzOZ2MvzMdeiMdeSEd9OJOpB6D+Bufw4pxS7Iv/9pQT+rai7397
zRL3/PTHvvD4Y18IQwcu6OAPHd5At+PQbTn2xd/28nX4UgRflsMXZ+yLG/Cx
DvuCA77HwPe78N0YnCSAEwlwcglcuYKrDuyLS+DQDxxyg8M+cCsDblPA7Rtw
Pg/7wgqcP8dccLowc/EEc5HHs3C73z//nonJ1BHBN36UPWoat00vmW5nVTSc
e55MzlfLdNa/diG+C1wPuoUjKG04McaKPYr8BX+1aSy0ox7Zh2HrjgSQXmOn
t1N8CM3PWJ7WWGRB27k3LDoZ4kUXxE1f6FrF0AwJQd2xoPek5NL1dmhHEOW8
XKhUMi2Vjuf6qzUucaOlkqx+r2ZGkWTc5WgNyTzivKQa5Xben5y6VHWmdSXT
jzre/Cdq5WTnwLUz9GkEnXmmWt8rk0sBvlv7jgQ2Uv7PjMDtRUnU9FF7SePy
MsqrnNfmENJOuSFKU8qHMyiJVdvTYvx3GSff4P7zpp2kQ9sip4llk85tI8ki
zgbat9Zf1iCrjU5pck3oPpRJjZlBaXq1NSS+W2zR8PcmSlrvl+Abm05tFrz7
VrpX0LyvX8O37GugmZWL2ZtSskhZTVdCLLaMpLw0FqyYVU/C4l7u/lLFtPHb
8/rOVZX0lLWZ2g0aSUlvb1yeeQNpx1uPVbE20idd47DpDzpJg33ebjaPTuLZ
cfgBx9tOCkZdGXVW1B/jnL045yLO2Y6+C9BXGH0X4J5suOcp3HMX3pWOd/Xj
XYrQQRU6fIYOXNBNF7rpQrcM6JwOnSOhsz98yYMvjfClFT4+h49y8FEEvg9r
ML5bwfdEcLIPnMwGJwrgamQLw5U8uNIBh3PBoQQ4LAG3ceCWRY7h9ik4DwDn
YuC8DXOxDnNRibmYJexyTJHGc2bEySlmVxRJXM6WmmSWTwFCTms9eVpp/7qp
Os4L08jwu7PJIZUqYouu6ta43klx83bN7rk5/ruMtPqX5dbTg2ZjN7WIJiqW
XpI8EJ1MN3Vml/JfLKcdUkea4qrLaIrqIs3UtnDaOa9i5KpVDsU+sqEztTlk
svq11LtJfpT/1KF94oxkOnNletipwBRy+txjGz3gSpO99pwy9ouk1pcb9ibX
xlL7Zwcpnq8OdMjoq8gTkWCSrVk4XeNROO3wZZsqmmxLSrtrufoDfSlk2oK6
6ooQ6uHYtWXGoC35mwj5qsz7SCMLLaa3bo2khPt7r0tudqFukZqmnT6BJFej
NHfQNo7keN/Z72z3o43fC27skwunMtHXq6fJJ5N+z8rYY3Nj6WVJQepJoUQ6
aSc3r09q/Pex1FqZ56k5NKC11S92YjYNVs3KWtGdS5/d0iyuLquh9XtWxe4w
LaVJzwdVhGIKqFIyeErXmVZSNvQLexhTS5NRz0P9MOo5OMcC54zIM+fsR9/z
6FuGvqm452Lc8zLuKYF3rcC7VuBdPdDBFzo0Q4eP0K0WujlDt0PQWQ46q0Ln
UfgyCl/OwBdt+OgPH+fDxyL47gzfa+H7RXCyGpxcByfm4KoNXBmDq1ngMA0c
JoPDd+B2M7jVALe9Qgznz8D5bHAegrkYMWbmIgpzcQtz5IA5Wog5CsHc2WPu
KrEvDDGnApjTvZhTTcz1HOyL45jrGuSABnJgE3JAFrkxB/vCHbkxtZ7JGW/k
zA3kTDRyqRq51INcKkGOlSDHMpFjHH/si1vIvd3IyRPIyVrk5EbkahpytQW5
Oh85vBA5fPZ/7Is3yHl15Pxd5HwJ9oI29gIf9oI16ntR7yGmfh/n7MA5Z3HO
FvSdiL686PvfXuPEPU/+sS9S/9gXJ6GDGnTohQ7c0O0OdLsD3f62lz/Al3j4
kgtfkrAvzsPHhfBxHXwfw76wge9R4EQGnLD/93cFuOrAvtgOrk6Aw1/pDIfz
wGE1uE0FtxzYF5bgPB6cS4NzngXMXJzDvviJueh8efXXkZe5lGl/sFDHzo9M
eRqty28nk0LhhSFxvpTxXHvbt1DUlbxusEdIdkVQQ/CZaxO1o6mtsXhi7Nt3
5He0c/OJF4EUbVawgA6E0DaJoi+F9y1oGt8UqxARLwryjbS8UBxJS7W+d3XM
tSP9qeH59TwBNOCR8OWrZBKJzgyRMLrmTBO0RAzy7MKoqHe7x0O/TEr8+IHb
d9SLjOyX+3z+Ek+zWEW6xOYWkUue36a1p4IpSt/mzqhPOnUq1Ld+tami36ty
z81ZFEM6B4Ue/h9GzjQq5zbs+iokUkkRypCEIg1k7qiUEpGESpKEVFKGJKSS
OVRIyhBSqaSB5nmex6t5nufQTaTU2/te27PWc611r/v96FzWOez92/v4Xx/Q
+1hM0/PFbSNSWyg6KMLWTzWNfkXumyPyuJKalbz328R30i0eAS3n4mxyUvU5
9fRrPRVxdDx6ntJNA2P72bgW51NGiohZkUkTabPbabckddHnWXFVqXyFdOSY
jjOXaBPdzklvNf3WQVu9eo4qHy6hd3NiBab7NJJ33VRbR50Oej7ecnKKXiWt
lRoVdJrUTJemTNcqXNlFkyNfdJ3Oa6LE+n3CL+3aaIqO/S+hW310Yub+Sz+r
2+lzaPeG29LdNLaPuW6E9RCs22KfP5+Z+8RiHy+c+xDnyuDcW7jnetzzDe65
H+8Kw7uM8K6/OvRAh3ToUAfdHKDbFejGDp1DoXM/dP43XybDx8fwMQA+psP3
MPh+Br73gBNxcPLdjMnJG3DFD65OgKsEcKgADmeAw6/gdgjcRoFbI3BuCs5j
wTnbXWYumpCL58jF6ox+hhpVkYzI5JIYtWi6lC+1Q35vIUlqzJ+XkdlK3Hyn
zJ/npZHUg+9ywTFVlDF7zel9D7upXP1h1vEnOTTczt8Y/L2Rfj9Y/+Wudjtl
iy/Sr9XPoGDdpAyl9zX07NbJcsmcWlq/pXnkR3Mc7etdE8gwKiF961uyFk0l
VKK54ZTTtHAqdC7PdzHOpPiNdu4Xzk30e+7V5ZbjAXQ2WE94b0QiicyMe7C6
LYU0ojxu2vW8oa+xRtL5LyPJK1RsWLYijmaH3PuT2/+KeAu31ZV2hVFNWGK9
6WAkTZt7I6zK/RV5/1w0fVFlCM3eEVi3UuYTcc8S9N3u/paelXyS0vz9gZQl
5fOuHYymJ+/3jPtEhZJ5j/Jx11ufaEj+Gh1RSKSEt5Nci5YkkV5/0sm5oYm0
IuvZ113r08jicfsjA8kCyv1AqToy2RQzo+utXGsWVZlFiByyryL96LKLv9eX
0q/6JwydvbmkVmF/e/vhJkrX9E3w/lbxP+tKWE/Eui/28cM+W7DPQpyrgnPD
cG4F7nkI9xTCPTfiXdfwLgO8ax50+M3H1OEJdGiCbuNzmLq9hG4B0Hk+dJ4L
nVfBl4PwhS2O6UsJfPwGH+/Dx1PwvQm+N8P3DHByDJycASfcD5lc1YCrVHCV
DA6LweE/4FAI3A7zMrmdA255wflscL4PnBdjXnxELq4iF7uRowPIkS9yVITc
VSB3b5C7j8ipNHL6eyEzp5+QazHk+gpyzebP7IGt6AE+zItK9EYGeuM2ekMY
PfMBPZOFnhlDLwmuYPaSE3qJFz2WiB4bQ491ofdc0Xsu6L0S9ORX9GQmelIT
vRqLXj2MXr2JHlZGD/v9x7xwRM/zYF5ko+c5MS9MMRciMReGMC/0sR6AdWvs
M4R5EYF9nuFcO5wrgXP/zrVNuKcvy7wI+Zd50cEyLwqgmxV0OwXdpkLnT9B5
EDqXw5cezIuT8KV/CtPHq/DRBT5mwvdP8P0sfO/GvBADJ4OYF27gig1c7QVX
FeDwIDiUxLz4AW5HwG0suLUE5zbgPA+cr8K8mPGCmYso5OLuG7VVyXcLqFK8
aexg6AdaVDEuaH45lWSP/p65ICyN2K9NZmhWvaVvDiNOUv7RpLIzetOl3DgS
OW25Z+/S56T6z44sv1NhtE1L21ju5Sf6WLhv/PqMJzSwbbmxQPN7YvO6y77i
XSRtHX6YMCD/lLQ8B8JazwZT4Kk363V3JtKuomXBjiKvKHFg/14Drgjy9bBY
2xSXTl/T/e7NO/KOGtUOPPWbHUsxU3ZsMPuST6JS45Um4x/otalY1Ov1qcQ7
SaZimjGDHHYICiTu+0yW2vKHfkXmENd+6auXTtQRr6qTptbqRBI/p57d8auE
ZEJbvyzpaaFF+dsKHAPTaePbtcdSBarI/pTtKyGpTrrO03q4KyGXFq7+Fule
P/H9oBtSFBrZTRHfte8PXysmqW7JvAMjTbTE5bTBoEMP5TunDaw8x6Bt/p3v
tx1uodUvSnK/pvXQiu0vB45p19BiI76jbkOtpN/0ItP+US85bwpdcSK3ic7M
9L84KNVBTtqu7SrJfXQr//Lqo46t5Mx7k6tscRc5Yt0Z605YP4R9nLCPJfaR
wrkiOHchzl2Me2binkq45/hB5rtC8C4JvMsWOthBhznQYSV0E4RuUtDttw5T
5z8qTJ1FoPO/+RIAH/ngowd89ILvnfC9DL4/BycK4OQzOPn1jMnVWnClAq62
g8NwcDgIDveDWylwqwVuCZzzgnN2Rybnr5GLAeRiNXJRujm3wcGwkmI1ipTm
n42a+N5UOMO5uYD21DYu/ubSQndf2Szl9kwl6esJK7/LVtKAyNBaM65uitf0
UBeYm0OHwxaordnTSKpTPJQ9znfS6rCheUU3s8g1bDaXxoF6ujk408/TtYk2
H3nJqA9Mpv131y3QimPQ55mxl2x+lVOt2MFlszUiqbeCd9vI3TyqYOQoOGoU
ksKfaw2X9T/S7J61C1z50qjiw6vXp1dnUqyiqUgVXwBNThwOeCsTTwWiowdO
bEmhGnafn7Nzfen60jadXstIKhXZOMwwiyefMeHBbvl35Fa4dFjsegSNLNlq
eIY7hhYvywmwyQgk1RotvSfrI+j7uVVrGxZGUyr9Ft3i9Ylq9B5J3jeIJAWu
kyckpsdRoGFHgV9mCqm3zmOL35lEjB8mLo+lU8hzlfO60VWFtM7tu1F2SBZ5
7fnwuL8/nZql/H1yTSvoQK/956YzxdSTonT12M9M4g6oPOjiWEsOKltmvLIv
+591Tqxfxfpt7JOAfRSxTz7OtcO5S3HuYtzzCu7JjXv+wLuC8a5ivGscOkyD
DsrQoQG6eUM3D+jGgM510PkedO6GL7nwZQF8+QYfdeGjGHzMgO+98H0cvr8E
J9rg5Bw40QZXm8CVD7hqAYcfweFucKgAbi+B2wXgNhacvwHnxuD8LHKRhFzw
IRdSyNGYPTNHfZgXa5G7mcjdZuRuA3Lqi5w2I6dTMC+UkOv9yHU8euAIeiAX
PeCH3hhEb7SgN3LQM3LomSD0zAL00gP0kj16iRfzQhDzQgI9tgm9J4neU0Xv
OaInb6MnRdGTkzEvYtCr0izzohg9rMoyL8RZ5oUhet4FPX/+P+aFA9adsO6A
dV3sY499zLHPKpw7HefOY5kXubinMsu8CGaZF9bQwRI6cEMHEeg2BbqJQLcp
0JkLOotB57Fx6f/niyV80YMvt+HjwGqmj47w0Ru+d8F3Bnz3ACfrwUkYOOnF
vBAHV2vB1RFwmAsOZ6kwOdQHt3Lgdh+41QTnIpgXszEvcpGLucuZuVBDLhhn
Fc4l3C8gW0lNy1VNH6i6wVStzSmVIj7qOp7fmk7cdXpvhB75UqT4l/ebjGPo
GMc2lSvLEmir1XtX+/YXFPNR/vvpPeE0k70kRub7Z8qNim7fWudB42tITiko
iEJvXRS42/WJZj+XZDs1x4M+WabqiFkG0vnpbf6/hOLI/vyHo6mJzyknYrmP
jlgYuQ+sC1BvSJn4PWfSWzrvLU39w3MswTeSztf2Ns7ozaJQelz4oOU9GfW6
Pvp4LJGWvInX0ltRTA5tkU8ulYaR0iGJyJ/P08knX7SyrbCctJrzyqUyo+l7
+hMXq0n51NLPkPvZX0f3skaer16fQgzTlWxZ6aW03ExwT8TFFjLdEi2b+zGL
FC+l1b47VkUWH1rd7qhO/L6I3jJzOW8xxXA5CYloNtBV0WnvjDd206K5VXF7
LpTT1Ll2SlN+N9O2Vbdi5izpJS+f/jrO9lpSHxvbV3CnjeJCfBV4N/RRvv7M
KSrTmyltcvXrla87KLBLKdUqfmK9bXxd12ALxXUsvhJc2fk/67lYj8V6LPbJ
wT4p2EcR5z7Audtw7hXccz7uORn3PI53PcK7QvAuUehgCB02QodS6OYA3XKg
mwd0VobOHdB5MYsvivDlJHx8Ax93w8c78D0bvo+MMn0/BU6swUkqOAkAV9PB
VSC44gWH+eCQXZrJ4Tlwqwlu08BtEjgXAucZ4LwbubiPXHQgF8PH77a4PKsg
8dHp/ximRxLHMs1Xvhr51P7RWs3Mupkcro65mt1JIZU9Yb3pZeV0W47ni8uc
LiKJe+cfyGXT3DFpF6ulDVScUCXs19ZFBopnfh+dnkMd9897KyxvpHenxHmT
lVqpw+PI9ks2aWS4J9HIcFHVxPd1Weiu4BoSXWdWOksjjo4XRrqbJhVPfK+0
Rp4OKqVIw/VSa/wjaPtka6eNT7JIJvH1t4hzeTSo7HH5e1YIXfTfdrvKPYUU
Srf7JOtl0NsRzxc66kEkPutDVrlkPGXZiXy65p1CUgnGyWrPg4lfQt7XLTWa
BI5e/pLGnUSFV8TSDOXC6evAHr1ZPtH06/yM4wGpCVRkYikhJxpHbX3xWafP
xlN07uMdskeSaMW7mgVLX2SQ8g3txYZ+qVTEPXapPz2FZuefWjP/fjHZlZr2
uZ7JobNtZw/M00ynsliPJ0udKujmHNPFujrFVNztMlTJlkn2Z0O5ytRqKOLc
ZacjvqVUiHVbrIdi3RT7xGKfi9gnF+dOwblmONcP9xzyZd5TDPccwbsi8a4q
vEsIOiRBh17oUADdRKGbIHRTY9FZGjrvgC8c25i+3IAvAfAxAz7qwcen8F0e
vp+F7wngZAycXAYnleDqGLj6Aq5swOFqcMgGDsvBrRW4lQa3reCcD5x3ijE5
T0IujiEX+cjFR+RoGnIUhhwdQO7WIHehyB07cpqMnH7DvIhArucg19HItRN6
4D56oBQ98Ai9UYbemIZ5cQ09k4ieOYOeWYleuote2ole8kOPHUSPjaDH+tF7
j9F7teg9SfSkJXpyO3rSGr3qi15NQ686oIeXo4eno4eV0due6G019HY8er4E
PZ+Jng/CXCj4l3mRjfVorMdgn0zsk4R9tuBce5yrgHP/zrWFuOcU3PMY3uWG
dwXhXULQQRM6SEKHaOhmCd2ioNtz6LwDOg9AZ0H4cg6+yMIXLfh4Gz7KwMd7
8D0Xvv/BvDABJxbgJAmcPAdX495MrrzB1Qpw2AYOl2Be2ILbveA2E9wWgHNx
cF4BznnPMXMRgVyMIRdvX/oV+z4qoFNdRSpvFELIcFd/qJ9bKq39ZLd0W0Y6
HV0ruHWnzTvaxZY8WC4aSxLKtuGRVxPpfqeAghqHD/3ST1nxgSLI7s3Q8/Y9
UVRZs/tOoOgzktbTIFfPD9S5MsSu+1YE3bH1rZhb/YS41A9arRkLIDvTpzEr
fKPJ8evT1u5sb/IyuSp7zOMjFXjv6exxSCLhGy7ln6a8odVfpr9vUfpEojev
fLp0OJ1WcO2IV6jyJ4vsB8fu7Iula6pWBf/cyiXevdlaZwRCiWN9T6zgjhQS
UJlsrbK3hCoSOV7pzImkHLkMz+qKTKrw8Fm5yq6C2MLGbk+uSaQ525p2fpAu
JE2D98qfY+vIOmhNdHNlBrE/M+LdV1ZGn2YrXNXomOB7jtia8HlFZDKPkysq
uJpU5607GnO/nYp3Ne7YVFJOFUf4t0lunvieH0x3GejoorufVERmLKmnQ34R
M2PjWmlAZPPbyxd6aYXQ1tTg2maasSBe9nFAB5ntCkntYvSRCldANGdWK1VI
BG39pdn1P+vKLOt/91mGfbiwjznOvYpzdXDu33vm4p4M3DMI7wrHu3TxLhXo
YAIdfnsydciGbr2hTN04oNtM6JwHnZOg819feOALO3yZBx9F4KMRfEyE7wLw
XQy+W4GTC+DEBZw0gisHcDWuxuTKARzWg0N5cLgB3D4Ct2yHmNwqgXNLcG4A
ziORiyvIhRlywfssg6O5v5zEq935jm+PJN09PWp21/LoYrtso5BNEwnOPVUl
559MWyQ1y9tfMWgqe7/n1N4OmsVlK39kWRZpTL53wvDTxO833SdHTzzspvol
AqtdqnNIUfp0ZBt7E+Xc+Mf1bdSEXx5ykZuFM+myuYN9uEItvbGI+3a9tJEW
tR0ocbNLJr0zcqc7OBlkkfS8IaG0gr5Mum682DWaAq5t/ZkxlE+2L++aO36Z
+L7I9yofSoqgoL3fFl9rzKChdz9vfbPMpWTNkW+3R8PI3M9ukndkCqnaLDqS
x8igXeX9PAoaEdRj5HGi51ISJefU9uUrpJF941Lf84+jyT6mX4HzThLNH28K
dN6fSlXKL+uys1No2CcpTZw3labduLP01sDEvG02HzEozqVDEXa8ZxszyWqn
nclh73RKcba+Pq5YRraBIetCHhRQzkPf/nSTTNKXc9Nam1NFrxLCjuX9KSE/
P6sXL+SzaCWbf5rl9DpSCFyT01BaRr5YX4b1zVhPxT7bsI879jHDuRE41xTn
suGeF3HPLbinMN6Vi3cN4l0Z0MEMOthBh53QbRt064Nu7H7/W2dr6HwbvlTA
l0T4Yg8fx+FjInyMhu8r4fsJ+F4HTurAySNwwg2umsCVMrgaZGNy+Gcak0MZ
cHgK3HKCWwlwO+zJ5JwPnG8F567IxQHkQhu5kEaODiNHasjRfOTuBnI3gHlh
hZwWIqdLkdMe5NoFuebGvLiJHniIHniPHihEbyxEb0ihNyTQM3LoGRv0zD30
khB6iRu9NBc9VoMeK0CP1aP3ZmNeiKL3tNCTNujJKZgX0ejVKvTqafTqTvRw
DXq4jmVe3EZv66O3B9Hzq9DzvCzzQhVzoRJzwRTrilhnYL0P+yzGPlOxjzHO
NcO5O1nmRSHLXPs7L0Lwrv1413rocAA6fMO8iIJu+ZgX3cpM3WZB52LonAGd
LeDLZPjyU57pyzT4OA0+7oCPyfBdEL4vg++nwYk1OLkNTirB1UVw9QPz4hk4
/A4O94NDArdPwe0UzIvd4PwyOLcA57XIxWvkwhG52Cptxh31smBiDnCxZX4J
oSMbDmzX+TDxe3OGg4apRgYNvHBN2ODnRxpybZeG/omdyD/ffvfyJBrN9/Lo
NH1NiRFSfByqn+hY+i6hoIho2v4zyURQ5jmtTb1i/TXpI/EvclLtY0RMfA81
uMlJetLqkuGVMY3v6X5MkPPdjEj6aC7bvsj+Odnq/LrK3/6B/rmSvPm+YTyp
7hp5ZyT1hu7OOpLVEBtOZdp/Zq0tTSHvXVKMw1H+9Lt92q0ZyVFkrCskv+Nm
Jn0NX2je/P4jLZzS5JpilUi3bgcnOt+f+L1FQm9vBX0mOR1hdd+wNAo4FbvP
YU4p2Xv8s91u4u+p77nY/jI8h3julrs9YlRQ0nk3IfaWDFovyrsnt6WYuF3E
+7e8rqOinqF3Jowiip00yydKtYLurbXM589pJvk9n5ceKqqY+J5VN0yWriee
BZVSFVntVG4Revng1EaKDzg45PCuhb6Otyx3/6eb5gimt8080kpOd/dN1Xrf
Qe7cF5e95umn557POtuz28lD+/bNNPNueoh1L6w/xvoA9uHHPg7YZxrOzcS5
n3DubdxTGvdcjHuO32O+KxbvCsG7OKFDKHRYAx3eQDdL6KYA3eyh82fovAo6
H4MvAyy+5MPHh/DxC3zshu8K8N0Zvt8EJ37gxBKczANXVuBqHbgyBYca4HAT
OJzJxuSWo4DJbRa4rQbnv8D5AXCuhVzMQy7MkYtiTlUa0iunRq0C2wbOSOI0
sK3vr80lxRjtndfXNZL9IgNH9+9J5NrE+LHvWRmN951t6g1pJ4N9bBLPFk58
3yv9I+EuX0tWuZb35e50k/eR3zajs3NJMsxjorKaaNfj8vCp4Z3katZhOO9U
Nn2VejhdM7+eHs5U0vx+v4XYS9SKYu+nURHbbLZj1pXEM2ZxjuNBLX2zu6Hj
6ZBIx6XU0gWellC6AtvTa5kM4rZwDtupG0OZesbX3YXzaF7nAYsX+4vpWYtQ
qHFsFCUtOiwkr5JJgUfPj/kI5FHA4XNnNdpiiKZVLjjGkU7H3RTVZ7dm0RxG
5JKrVsmUED6yNntlOj1MeTO4+XUmSe7nXiW4KosO3zv4MKAjk0ZVL/sdG8mk
uBeasgq5xRSteG/KV7V8el6w4LG8Vjax6WXLn1tVSV2LzBXVAkvo29gR/9g7
OTQ/0Gn63R91lKMz76NcPYPeipfUFL7LodbOrntei5ronswLZZ47Ff+z3oT1
O1j/in2mYp9E7PMM537XZZ7bhHN/4p4vcU9P3NMd7xLCu3TwLjPowAUdYqDD
Z+j2CropQ7fF0NkTOmdCZwZ8EYQv1fBlCXxku8z00Qo+esF3bvheB98NwIkP
OOFYw+TkDLjyAlcS4KoXHKqBw3hwuAzcHgO318BtEjgvBOeDh5icr0IufrAx
c6GDXOQhRz3IkSpy1I958R3zIgK500dOtyKnK5FTXuT6FHK9Erl+gh6IRQ9c
Rw/8RG+oozfuozdq0DN+6BmODmbPWKKX2CKYvbQavXQfPZaBHtuAHgtD73mg
9w6g92ahJ1PRkxvRk3MxLzrRqxnoVXf0sDJ6WJZlXjBY5sUv9LwIev4uy7x4
gbngiblwD+ueWHfDehf2mYl97LDP6HzmuaE4NxDn/ttc+zsvIvGuD3jXyB2m
Dj7QQRw6PIZumtBtOXS7AZ3joLMMdNaFL1WYFzPhSxJ8tIOP9ZgXffBdEb7f
hO9O4MQHnJiBkyXgygZcbQVXDuDwCDg8+Pe7BfNiCuZFLrjtAOeTXzI5PwnO
zyIXmzEvHJALb9skp+vhBdSQJhWxfU4oqZ6WtCxrSCXuKy4lk4sz6NxDNUUp
gwDqPvLyKF9nHMWdtwr+7plMKypUzB4bviXVQLasZ1afyb9ccN06sVja+bBS
P0DZh/wKFcPSRcOo6WLlyhkqn+nre8nFY1Ne0CHjlrC7N4JJ+OzZF1tPfqbv
LsWifgavSO+JuPhJhQ/UW5rNiBOKJU67lffys9/SzyypmW5jYXTx4PvpdzmS
KfCFKsPs6XtqToxNMjkVSc9rrdvjk9Ko536Y8INJYeT6JvcfDsV4uv7zWJrs
mWySjT5zrE9x4ndS9307Bmcq7U1qtvu8s5C81F+sn2+UTEPXqbpzfxadde7d
VF1SSrs+mHO1+2aR0KHXSTt9C2mHyoeao+craVp60aWPiiW0wlkh8LY+gw4M
eS9bcrSeqlxSFfdlVpGH0hq52Nm1dPoK5/qfD1rI6KToyy19TdTx7XHPs4Rm
Gv7If3ZpdCdpSW5yOGXVTr6nxS9tf9xB3OxZoz+39pOahuz8ls9dtNLvyoz+
4Z6J713mugrWxbD+D/bRxD4+2OcEztXFuQ04dy/uWYx7uuKeW/GuP2nMdwnj
XaehwxboMP2vDtDtEXQbgG6XobM4dM6Ezn996WLx5Sx8fA0fa+BjG3xnh++D
8F0AnHSAE01w0gGuBsGVMbgKBIe7wGEAOEwFt2vA7W5wOxec24Pzn+A8ELlo
Ry52IxcX3mx/fXZibq9Y5at7a2kk2X84c1HnRC5tMFU51xxfT188pbTDFiXR
Rs6u1hOppcT37KkPXW0jqdRAvmM7Muj5jlQTz6JqOinwyp9zUje99zo1dfuv
ib6+vHHlPPtG2tb7MXvD1G56FXKoul0ll3oqptLZp43EvXefajt3O8k8ce6f
FJ9JujafHNrjamjNuYObfus2UbxzosVgQiodFrhVbjOtnG5NK74solhN+3Ta
3E9IJ5PuvNfGK4KLibM5IllJmEFXXlinPl6WRMfVtb75tefRuVNmSscmvjOu
zfS6ZCibQrJW6+3dj+XQrupUhbKIAjJ/7be5PyCDDiWs3WoUkkMPWy7Ouft/
/1/bho57rjkF9MBozd773fkk8bCCv2hhPq09Glnd9o1BJgxXGT+HEnL+dk/+
5rICin/gfjJk3cT3zzaLlA0m5ZSulJs7v6aAzi811mI3aqa2CAGOOxVV5JUx
s1F/dQEtOvEte+PGVrpR+cnB1bmGXmBdGOvOWM/CPqewTw32uY5zP+DcaJwr
hnvy4Z57cU93vMsW73LGuw5AB0PooA8dLkI3Gxbd+KCzHXS2hM4v4IsefDGB
LxvgYxp8PAUfBeH7Rvh+DL7vAid+4OQHODkKrt6AKy5wNerJ5HA+OLwODpeA
20pwKwluTcD5XHB+CpzfRi4YyMVG5IILObJCjtqRowjkbglytxm5e4GcqiGn
PshpPXLdh1wfRK5F0QOT7jN7wBg90IfemI7e+IXeuIqeiUbP9KBn3qCXBtFL
XuilO+ix9eixUvTYEfSeP3qP3ZnZezboyX3oyUXoSW30qiDmhRx61RA93IIe
9vmPeTE5lNnzeuj5EPQ8L+bCDsyFVZgLf9iY6wpYF8F6H+aFOvbxwj76OHcX
zi3DuX/nWgXLXNuCd41gXizAu05Ch1XQYVSfqcNa6OYA3WowLxygswR0zoXO
bvClHr44wJeT8PEufMyBjx3wfSp8/w7f+cFJC+aFBjj5Aq6GwdVpcJUBDg+C
w2hwmPYv82IpOL8LzqcYMTnPRS5+IBfGyMWSb473TzIKaOZtZa6Ty8Losdes
O/fl0mjh97xVPfqZZOWm+V12IJAu6TZ228xJILIIFzbVTKGDXpOXp232I4b7
uQ7dukga1bzCK/8+jsR2LdrjavaWOH/fWEAfw2n695P3hEsiaUUx53wnnjck
XGfuebjqI30bZgiJOnwi23jNao6Hb2hV1eUpvlkf6NSFeEH9+9GU6VpoLCTh
Tx0NbkvlZoVT7oqYB8tcEyiz/PWxF5EfqIJLdXHsukiyyV9+/11jCoWIZZhO
fRFB+YZ3U1aExNGQsPe88Y8ZdHYfm9q1D7Gk23rf8Mj1FDJS1pScZppLC6Ov
ZC0OTiX6Y3JJ4G4mnRBxLYpZVky2X0Y9Nsbn0pInPN4V1wtoqqrJW42GMvK5
7njmsUYZ7Ws7e+fM4zK6smjoguK7KuI5G/rV9UgtpavvizBlryEtHcupuhO/
B46UZfdL326h9LFrtiFTmilgplyH6Ik2WnjVwWGVbScdy9CLWP6lnfIbq7Yu
+dhHcsOty9LTe+maRUPY4MY+SsK6FNZtsP4a+yzAPobYZzvOPYhzE3DuRdxz
Ku6ZiHtOwrue4F0aeJcRdDgBHWZDhwPQTQC6rYVuX6CzMXTWgs4X4EsgfMmF
L2nwMR4+FsNHE/ieCt9b4HsPOLEGJ6LghB9cSYIrcXDFtvt/c8gFDreDW0Nw
Ww9uV4Lzy+D8FjiXRi4EkYsXyEXARakBw0sM0t7+ZS9HWiRt2RGYukgil4Iv
Hn4X7TAxvx/clOj4mUgf7o/P1VtUSjwPF5xJP9FKB6v+SVbKTyfRG7qNjGdV
NOV2duCklV304Gr9q6ulOWTamcTv86KBzrfs8foT3E37w29rxibnkTNn9RyP
3U3kf+uGuI9kJ61+4Thq15ZDgwPv2hN968nv3pOHxT9baKzWT/5cRCb5KdPh
D75VFHPFsXqLSgM9PVy9Ndo2nVIF11/ZpcigVxUCJ6UfV1Ghpi9f1dp0SrOP
WL/7YAldL9cq2JrOIJXSvW+3ns+kd4nDLw3GiqjBfqHeuF0pbV6cfqiuI4/s
1RzS5NcXU4nP4sFeyxJy85JOiw8ppdAXAfoMnlKS/fzzcPmBErpVGSoxdK+a
DLbrpB+3Lif2jCgJ9c8lZMumPcjT0ES/Wj9wRkRX07vkH/wm8SVUZK29Q9qj
jUS1C7SNz9dTuHAG18inYqoy8Vvv/bGDpP+5HS80vZE+Yb0c61JYD8Q+adhn
LvYZT2eeexrn9uFcadzTCPdUxz3/vssZ7wrEu9qggyR0uAIdbkO3LdDND7oF
s+icA53z4csz+JIPXyLgI2cd08dP8PEjfJeD76Pw/SI4OQBOboKT0VtMrpzB
lQG4+v2AyaEyOOQCh8/ArS+4fQ1uvcC5CjiXBOc8yMVk5OI2ciGMHFkiRzbI
0Xrkbi9yl4fcDWJeLEJOJyGn05Dr5cj1fOT6F3rAHj2wHj1gid7IQ2/0oDfK
0TOV6Jkm9MwV9FIkeqkKvfQbPWaDHtNHj51C70mi9zTQeyfRk/boSXH0JB96
NQS9aohedUIPz0UP5//HvIhAzy9Hz59Gz5diLqzFXHDCXAjBujjWzbD+DPsI
YJ8D2Gcdzt2JcyNw7mXckxv3TMU9x1WY73LHu9TxLn3osB86TIYOKtBtLIqp
22Lo9h06m0JnHehsCV9ewpd4+BIKH33gYzJ8PMEyL1pZ5oUlOFkMToTA1Spw
JQWu5mFerAaHc8GhGsu8aAS3a8H5DXD+GJxrIhdLkYuPyEWwlqa5MHchmTu/
+DmzNZysuR5ElCSm0eWa608EBjOpRHPO64uVIfTldM7VQf9EsmmsTVrNk0qZ
Bw7nBF4OpOXr5M4KfYgmed3ry1o0E2jqoKz9Pu4Aem66e3BK4Cc6meDDUL0V
TUabvNlF1vnTztU/JXr4wmnykSAlI5nP5FsZoGLQ5U9m7eMyWe6hFJJu+HBt
UBS9b9klqhYXTGzeynwWkyNomP3Kj18z40ndt0WvYk04+ez76N+5MIqmV1r8
WB+WTJ5LtE2+hkVRReeT5dJ348mjfM3vA1/SSN9urp5CWhIdzTk8+3JXCoWv
uXFPpiOLLEus9QqrMsjV4fLgtcZMOlx66GJjVj7xb0l+OdWnkDZw8sz4Z+L3
Kt09vyfYtIQYvRcP6biX0/TNNvsdrpaRafuKAt2F5ZQUcr4vuame+HML61yu
VVPdJLmNSf7V9Gz7/I5j+a10vVUvZYlWE438OdYq0d5INwQnKfI86CKl6dVV
exXbiM1fK5Kh10U6nS07NQT7SSJrnuPDqB7q8mOu78L6fKwPYB8H7LMF+5Th
XDecexXnnsA9o3BPHtxzA96Vg3dNwrt0oMOfzUwdVkCHQOhmDN1uQjc36LwT
Oh+Czlzw5RF8KYUv/8BHJfjoBR8D4LsPfB/1YvrOBU58wIkFOLEEV8bgag+4
2gwOp4HDV+DQHtwWglsZcOsMzmvA+W9wHoNcWCMXl5CL7g2cJ49LM+jR2eK+
OUuj6f7+zaKSG3Ppx4D81laHWnoR+DPldVQSFVrUVByPKqHZx23171xooQeR
Oj+aC9Mp0r2gUul9JZ27n91oITfxfaxSezzvSw7dNaSYIx/raY/vTJ365G7S
tWQ/nC1QQIOL/D9zXm+iG+4XbHYmdZHXoPCHyF/5JFkUv+TSn0YK80pyNrZs
pwsWbI8eTM6jgh/XVt5cU0e3Qx5VieQ1k/udozOzL+ZQH19fLe/KKno4LSFG
MmiC110NGkficmjXXI+Pp1XKqeEn+8kYh2qyYOMPfPkqnzRGpxiqdJWReM03
D8eBCto1XybT/2wJbbRzC3k88X4er8cznT+XU/qXcY277yqoW0Vj1a3VFfR0
+p5bhnfKSWjDmL7TSD0dfC1ubZVaTaX7LWVPtZTTxjClZTH7Wkm0zjaxf+I7
w/eykr19bDkpZxyKiFbvpLtKCsMj/M2UMJpVv06zjFYncNZcNu2mzPoLJvdu
NVMG1ldhPQvrH7DPWuxzA/sU41wpnDsH5z7GPb+uZ95TGffkx7ti8K4OvEsC
OihCh3XQoR26mUC3XdDNCzrzQmct6OzK4ssP+BILH+3gYxV8vAPfn8P3NfBd
E5zsAyffwIkZuNIBV/bgig0cXgaHPuCwBdzeAbep4LYenN8G5/bg3Bu5MEYu
TiAXNshRPnLUjRxZInfJyN1C5G4VcsqOnHogp8eQa0PkWh25HjNk9sBb9IAp
euAjeiMYvcGOecHBwewZbfRMIHqGF73ki15qQi89R48Zo8dOoMdi0HuX0XvP
0HvG6Mn5mBcK6EkV9GorenU2etUSPZyLHhb+j3kxbYzZ8w/Q8+roeW7MCz3M
BVnMhVzMC2WsT/k7RzAvrmCf9dgnG+dex7nnWOZFHO7JxzIv0vGusU3Md+2E
Dr2YF/Ohgzd004BuZ6HbE+isDZ0NoTMHfLkJXzLhSyvmhRx8vAsfA+G7L3wf
Y5kX3uDEHJycZ/kO0QZX2uCQGxwGgcNrLPNCFtw+AOdt4JzTksl5BXLhgFzc
Qy4SX+zmUjIppKE+/05Z388krOGmsNo7ndp1is8LvcoinR1ndggahVP2aJqL
28lkWjxsPsmjNZWSCi9UXzwaSteaxPv3CsTRtI0nvvAsSaLffo+WHM4KIeWp
QsmGylHkZLDyGc/5WPrztdRV7EEIOZjlWLnWf6LQxOo/IfJRtPTPi5cLeUKp
u35z4Yk3ETR1Rkp0jlkUfdn1+Ff0uXD6HRSoq/TlE/nwWIWEGMXRvt88PP90
RNIlZ2ooXhpD3o++v3/El0zddVZ3LNclEC/7hwGNoEQaHfrcM9qTSvzOpjGB
wml0aytvaRlHGn3pDMzaPPH9+Wc4o22K+0S/WW2oWDyUReHGZUcbr+SSDqPM
nM+ghIJvJAg7eBUS36R76yedKqJbqjH+i15WUrZxUvOe92Xkt0r2SaNzKeWd
/z6X52kjRWs9qhiaWT3BR58Yw6qcLGN4hW9vbyPp3BtSg2MNtLzi2ZOk+kpq
l501smAiB6M8X1r+LG0hxbZXmmmjlZRxSvQcR2APDa4edpHVaZ34ffC/179j
XRz7NGOf39jHEeeewLniONcX90zDPSNwz2l4lz3elYx3BUGHndDBBzp0Qrev
0K0Huv2GztzQ2Qk6e8GXLvjCA1884aM6fLSCj5Pgew98/wHfI8GJKDjpAyd3
wNUYuHIGVzNYOFQBh8vBbRq4vQ1uB8C5LjgvAOd5yMUociGGXDgc1lWZxcmg
o1eO7PN6GktPt0hX/wzOJaFPfYsOJtXQ0m0eo46VycToWcDr8qSE7lraijR/
bKblbl9veVzNoDL380nbeSvJJE7aLWRbBwnsUmz57JlLpwqTueq66+icmdxx
TatukpbJYtx0LaQj3rL3199topwzLzfvjesmq02mij0cxeS4vXv124YmqutU
sj50q5Okj10WTXpQRAHJTRtnmDcSd2Kz5BazNnrJKXKzcFIR/TNuWnBGoo6W
LVr4+rh7M9kvq16093sROZsKdF96WE3cXFUGwm0NpPzmrWtqZAn5lyfYnNGs
Ipq6n22xTB25vx+4Fi9UTl2VrxUHfKvoRE18SF5vDQkLX3QZNK8hxZFnIo1B
NbRp+lPrmQ01tLDiZlZwRRNtOqTtEKHaQLslXxVO+1RLAfO3hw3wtNPyKg2X
S+ebaVceZ/HZafXk9s3Q+OxYF3mby7TvT2ml5ewm8RwiTZR9+tavoLBe+hl1
oeZgZzvNw3oq1vuxvh37OGIfd+yzC+e+wLmiOHcd7jkD9xTDPS3wrll4lwLe
pQEd7kGHHuggCN22QLcA6LYKOttB57vQWQC+PIcvI/ClDT6uhY+h8LEAvlvD
dyf4bgFOxMHJIXCiD66mgSsDcHUJHAqCwwRwOBnczgO32eD2HDjfD85vgfMQ
5GIAueBHLlqQI23kKBM5EkLu4pG7i8jdnw3MnH5HTjcjpw7I9ShyfQ25DkEP
rEQP9LLMi6/ojRH0hj965jB6xgk944NeGkYvCaOXxtFjc9Bjd9FjP9B73L+Z
vTfJmtl7n9GTuujJcPTkAvSqB3q1FL36Hj3MQA8n/n/Oi170/Dh6XgFzIZNl
XmhhPQ3rP1jmRSP2+YV9ruDcQzhXCOe+wz2zcc9I3HMK3mWLdyXgXW+hgyJ0
cIMODdCtBvOilmVezITOztDZA77UwBd2+HIHPm6Aj0bwkQ2+D8D3n/A9Cpws
ZuHEjWVe3AZXYpgXY+BwLzhcAW5TWObFMDg/DM7LwHkHcjG9n5mLdciF0g4h
jfieQuLf8aOfuyqG2m3uGB3tzaCvAWEJ+pbZxPFzpsXKlEjaqF0Xc7Ezhe7u
3pgtV5VGt73U2258/USF3odjbbQSaXfHtbdqXslUHHL93v7VE3MxQenUYGgs
7RXhHGbziyeDFeZKPkafqEJ0o8je69FUWhi6avvNGHrru3pH/6PPFLvxS+y8
+1GkFbvVYnd2FNXpbOKwZo8mD874qCPtUWQUcTIi/kwcSYeNiL7kSiD1cBnz
D1viadsOgeWMie/gNCUnsQeTUklWbqrwlm/JpJw089sS61QK3vkla2N5Jl28
YV+3+m06PTr+SC7vQgYdfcK14pt9AdldLVxtaJdDAtsLy2VGs2lHopfwS+Uy
2tfsyBmVXEQNUz8lB4zmU66UcmmsaDUJz9v97FZxGY0kXHVb9LyYGucnr967
uInqh+2HjCyrSPa7lcA6+1LysXXzyvnSStGDOWJtUvUkf2bRvkDTMnot8W5I
e1MniVjP59O52ETD3NM8hreWEr/O4BKuP12ktvryEDtPM/3B+iyW9XXY5yX2
EcY+a3CuF84Nw7nDuGct7lmJe1biXQl4lwDexQMdCDpshw4PoNsB6HYWuhF0
fgedz0JnZfiSAl9k4IsBfJSAj8rwcRd8r4Lv7vC9HJy8YuHkIAtX1eBKCxwW
gcNocOgKbl3AbQW4/QXOucC5AjjfjVwIIRf9yIWfpMvWcQ4GSe9O0Ri5n0iy
17bw2fvn0cKrw+U3FWtIOprf/NKbVPK2SMlozi6hPbM32Jyb30xi3+btc5me
RZ4Px+7Oqa8gpyrdnRfOtdPUO/2rpvDmU+qmxx7yF+soQGtPelJ/F7lODiix
2VZMtu/dPBJuNVHQgku9nbI9ZKqWtefeixL6eCQkp8KhmQwlFz5QX9RN6Tef
TuZbUEpGP9eqm85pppojw/d3ZHbQZpd8D6mTpTT07kh103AjucfPcuPa1UYb
8g0tr074GyKqpK7eX09D2kJL3+5uIdtbC5P4NpRTuIe/U1BPHS0Nn5LAObmZ
nhceS17wroqsnq1/8eRAPT1cpffj7IUmklOpWtuYUE8BuzMeGus00kbP57dT
3kys34l6cXVdC0XySTxaZNFMk3ZPWffndTN5fVBRdZrWQfl/xBKovJUeaybz
nhJuIzEtma8HGd1UV/jrmtGvDppqJOiS499LNoHy2lte9ZPCFPEvTm/7qOcI
c90C65JYd8I+s7FPLvYZ02Se645zs3GuPO4pinsG4Z7ueNdyvMsX7xKHDu7Q
4Qx0+AXdrKBbKHTzgM7y0DkcOjfDl43wZQy+HGXx0Rg+/vX9BHwPge8+4OQu
ODkLTmzB1e/bTK4iwZUCOJwFDq+DQ15wuwLcuoFbL3AuDs5FwbkMcjEduahF
LnqRo0nIkTxydINlXmQjdxrIaSFyGoac7kKu9ZHrUuS6ED3gz9ID2uiNRvSG
J3rjFHpmK3pGGz2zE72Ui17agF7ajh77iB6zQ495ovcs0XtO6L056Mld6MmD
6Mku9Go5elUMvfoHPdyKHm75j3nxt+d90fOL0PO/MBdms8wF9pn/e17swPrf
ueONfRZgn1U41w3n+uPc37hnA+5Zi3tW4F2xeBc/3jUVOqyDDhuggxN02wDd
DKCbEnT2h87nofNm+BIDXyThixZ8FIKPa+GjJnyvge+P4XvVv8yLwyxc1YIr
A3BYAg7TweFjlnlRDW453jM55wHnquD8OHIhhlyMIBdBCt4mcaFFJGs3tZl9
cjJZ1hWLHqnJInUTHd0SyxzKFlpd2jk7gQ7YfvYxHk2joVm1y3f+Sqe+9WHX
4k3iaKiiOkJnfwpF1S4LCSpIodEVFocS82Jp7dfa2rtySTT0NlReTCKJJjmG
PTmyMY4u88t0pMgkUHxMZdFF8XjqtxiOFYqPJ51Hpqrq1vGkUdss2qMWS0NL
VT68c0gin3H+LrW2eBqQIEndpfFkUiaY8UkplU5/0j8dujKZ7mhP8uOdmUwP
Cm1O8XRn0D3XOE7ekVSSfpbislc/ldR8r5meLM2luJc+dgYemTQ3cO+lpZMy
yO0ze/tAYDE9vxv0epJkHqVPK5J6OJpFk3xGJKc3M0jGPaw54Ewx6VZsVq2X
z6OihTsy3efU0B/nc58mVU/MpdktvudPFtJixlQNda4mmqFH55a9raTxND+X
mqvFtME7Y3WTYCtt+CN0vkCnluYZLP/mMqOEhK+qkPerdlohWRV1dFkDuaVc
P3bxXjGxpx/MeM7WSbm+8y7qqjXSI6yzsawLYZ952EcM+/zBuXI4VwbnBuOe
IrjnNNxzL96VjncN4V2J0GHoFVMHCejAD93uQbfH0G0VdFaAzp+h819fXODL
XfjSBR8N4aMpfNwJ37/A9+fwPQmcdIATbXAyyZfJFRu4cgBXfzkcYeFwDNwO
gNvf4FYLnOeD80PgPBq5WIdc2CAXYf3y6e1rJnTpEb4guz2VMowzxsa0CyhW
JM7dfkYN6ZygpOl2GXTWdPWwkkkpZbV+e+MY00QHHRe2/wjKIWHnjcLXxCpp
4aW0RHFGGy1elsnufq+QTrBfoBkTc1CCSyu2x6SL1rV18Qi8KqE3W1uMNnk1
UaSgi9YvzR4SGzL95mdcRibqPm3vxptJNSTS6Lh0z8QclHWttGFQUM/Jx10T
3wtH3eVtgkP/77+XmHN5885yihtsmrcsqJma/Scdf5LWQV8Gl6/7dK2CgrVy
k6LYmkkwNei1o2A7HTgasj3oSRUZXXJ8IxnaRIEnA/zdf7dS0uCPZykNtXQj
dG/s8Pemibn83n3m01ZSkbn8z8utTbTyIP+2Az3NdPzHT8WtnG2kEjq0x0es
lRjaF7MHnrTSP1kGV2bca6fI72pxnaYdpGNY6j083E7LvrVv2VzQRR0z9axs
w7rp5rVXFXrUTQLpy1PeD/dTxRaZHZ6WfXTeYdWUJWv7iQvrpVi3xDo/9qnE
Puexz1ecG4Jzd+NcI9xTDvfMxT3/vmsr3iWOd32ADlHQwRk6CEE3LRbdOqDz
AHQOgc4n4IsYfImHL+rwUQw+BsPHCPi+BL4bw/cl4EQKnDwHJ4Lgag640gNX
0eBwOzicDg6DwO0ucGsKbt+BczFwHgnOXyAXUsjFCeRCFTnKRI50kKOvyF0P
cjeI3IWzzIs1yOkPlnlhh1xHowe+scyLHeiNnyzz4jt6xgw9cxY98wC99BS9
5I5ekkWP7UCPJaDHhNF7nug9P/ReJnqSHfNCFj1pgF6tQq+y32D2ajh6WAw9
zMcyL9ajt+XR2397fhF6XhI974q5wIG5kIe58IRljuSzzIu52EcU+4zgXEmc
uxLn/p1rC3FPLtxTC+9Kxbu+Y15EQ4d+zAsx6MAF3RygmzN0Ww2dlaBzFHR2
hi/O8OU6fKmDjwfgozF83AXfv8P3V/A9GZy0s8yLqZgXfxyYXDmCq1QWDhXB
4Ti47WeZF4fAeRE4PwrOi5CLTciFM3Ix+vS5x0+TYjIle45S03QKjTrkafQs
l7Zaua3oDM2liod/TgwMpRKXQ1ZWT1km5clT/0GtTFIVWiZftyiVbi8PnzV/
Qzo17n5b9FM2jcaV62/GlqTQgLqipJ5TKvlJexXOnpxCIg/dDE5wpVJz4xPB
hJspJLxWyzRfcOJ3hLuFdWNxKlkcyC1tWZ1CVX5a9QvUE2gDW3LlCoV02iux
9tgC6xRy9o2+tfFoAi1+2DZvs2smhVo1DZj7pNI64QXsk68kU2946GdamUsP
O9trX2tnkHhQ+VKbW6lUFCCxJKixkH6vdHjjGpFNQeY7hReWptPZwwdq954v
JbXZV7qN6vNJXcTzCf/eLJpVdqK8VKaCaqzm8Un1F9PCgPkzbrbn0Gwxp/BT
3jXEF89pMCerjCQiizj1tPLJ4hI778YzjaQocSEnPK+CpklcUMiRLKRS/tK3
FrNaSOfwmX8Mn1dTqO4s7bZJRaRqsE0t6UcrWYdnHfr1uZakX3Gkrn1YSO6T
LRh+I21k2Nc3+fyrOpLD+iOsH8H6331UsI8V9pmKc4twrhbOFcc9TXFPBdxT
CO+ajndNx7tUoMN06FABHQKg22nopgjdlkLnbOj8AzqvhS+d8OU+fLkCH+fB
x2D4+Nd3Wfi+B74vASc5LJy8B1cLwFULuGpi4fALOCwDt2rg9j64VQXnVeCc
B5yzezJzYY5cRCMXlYaLbPc+mejT1i7dw08yyar4kdikponfrZMWH4tdWUNT
dx63eOuTQ+v2veeeerWMZF6vy33k1kQX+SQOu3oU0J5lDs3rf1bSNIfjTdet
2+iqUoDvoHAJZfrEyP4ZryNjt1+jL4c6ifHgg5Xx2TI6Eew/v6uyiQ4Lvspu
29hD9Q76/DdOllO7REdAmU0L7R5/uMujuIcG4hPn76+poJaQnxZNC1vprctA
4+w5PaS0xUVjJV8VHd3vpSzA20qCKi1blmd20ayUe2a/HatpN7/ARve0FlJR
YcTwOHWSMH/UeUHvWnrfMTc237+FcudpmD4O6iCtuqpIuxcN9GtbbfOKPy30
vfhyh6dtB6Udkgp30mwm/Tn782e2tlL40q89yrM6SUUjOawzspV+y+vuFjRp
pzdLk9Pa/LpoQU6N4indDjK00WT3v9ZJV2wHDXbq91J9m2H1trouCg5Z4a22
oIcMcu4M8csP0M8tA6obfHpoJceWl8vM++gA1n9gXRzr5tinBPu8xD7Pce5s
nHsQ537APTfgnoO45yDelYh36eJd+dBBg0WH7dBtLnQLgG5C0JkHOu+Bzn7w
ZTOLL5rwsQc+NsFHffheBd9b4Ls+OCkEJ0bgZOwakytbcBULrsTAoSk4VACH
teB2XIPJ7Wpwmw/OV4NzY3A+hHlhglwEIBcbkaNy5GgqcpTJMi9uIHfVyOkf
5LQXOX3HMi+akOu56IE89ID5f8yLO+gZMfTMJ/TMBvTSF/TSI/TSCvQYAz02
hh4LRe9dQu/tRe9poCcF0JN16Ekx9KowenUOenUVevg8elgFPfy3t0vQ29os
80IdPX+OZV6wzgV5lnUjrIdhn23Y5wz2YcO52ThXHeeuwD3NcE/CPefgXZx4
1zS8SxE6cECHEujgA92MoZsMdPs7lwug8zB0loYvdfDlJnw5Bx9nwcd3LPNi
LXzXgu+i4CSLZV4Eg6t5LPOij4XD7+CwAtxuZ5kXe8B5HTgXBOc8mBfWyEUm
crFHdypDZVkJjSvlD2w2n/jzJYu8Fv1CUssZmDy2LZ/4ZtSZDXzLJkmz9vVD
XrkU9+s5u3pPFk3bxRb9cSSLjEX31I8MZpGM7HELm8IJPcd3Vc5xyKIcoerX
25dnkrzfwnDGy1TyUgx/oXsji0zPdfnP4c+gt3HWYX1LU+jAlRiG3YJsGvUW
TIt4lU5Bv2a3WGxKpik55Z2b9XKosum7CSMnne78GLdS8k6ijk+uOQ36eVR+
cFoKPc4gvYIzj37OT6GKWbet/rgX0tMvI46/K7IoP/b3429NqcTHeLOo7FQJ
2cgs7tlZk0v6/CcFVk74/lXnm2HiLAYJN4h1l9pN+By0weZGYybVbDqyK1yx
ksyzE65v0i0hqnSU7u7Jps6w/LLD9TWUVjhV/tT7MvJKzC04nJpLGXNEvgYF
NBDXZkfz/skV1FPykVNFNJ8ODJfrFi9opuL+FP2jYlWkZfikRj4rn2xnGUuV
Xmsh7lOc7a/Cqmn85MkN+g/zydwrouL91lYyO19g4nC0hiaZ/u91c6zvxj4X
sM807NOBc7Vxbj7OfYp7JuGeU3DPLXhXM96VhHcNQYdy6GAKHXShWx90mwvd
cqEzN3S2hs7/5std+NgKHxnwMRi+s8H3CvjuC072gZMRcLKJhSszcCULDo3A
YTY4TAG308HtcXCrBc75wbkMON+PXLArM3NxALnQybf4M0Dl1JV4wLTDIZ/a
BPOFZJJLaPKNJfnxfjWkX2wmckWniDxtzE9fXVFOI05beQr8m6jTa8OypXYl
ZFlc7dNgUk1hGtmphfvaqNFqeF6MZxkt5sjYI7m4gdqtDs3njukkCTJozsgo
p63Zzjd+Tsz5X2sdN8sunJi/46YXTn2qJJua+ZflRluIb7LnzkixXhI97Hxu
zvZq0pr3IbVp4nvg0EyVH8fZeumroG1y6cS9VnInB56UaKMq64t5R6mHfNcl
lR5fXUf7b5rrT5dso5Cl9w8vP9lNdpMle7klG+iHtlv3AYU2SqxexXOvsosW
7Tjovs20iSxnKWt1vmoj7qsyia05XZTqZbb9En8Ldc7Y7SRl2U6b9Ir8jY52
U87nZaq3J7WRwy07m++hHSQm8Mb8xYceUnTcdm72lA4Kzz17Rlevi/IeFZ6U
deyjO0bB+kFlE++229psKdFDj6cpyN+cPkCz/OKT9//sIu2DV0f6HXrJHeu8
WNfCeir2ccQ+S7CPKM7djHNDcK4c7pmEe17CPWfgXYl4VxvelQAd5kMHc+gQ
Ct0usOj2V2cf6KwDnQ3gSw98WQFfeODjQvi4Gz4OwfcO+H4evteCEzFwIg9O
3oOrGnA1B1wNgMNmcGgIDn85M7ndD27dwK0GOG8B5w3gXB25GMW82I5cbEeO
eJEjCeToM3LHhdwZIneSLDnN/Jd5cRK5fsUyL37/y7yoQm/87Zk+9EwVesYA
vVSHXnqOXipCj/Gjxy6hxwzRez/Qe2LovVH0ZB168jR6UgW92o9ezUKvPkcP
56CHZ6CHu9Hb+/9lXlz6l3lhwTIXOFjmhQXW/+5jg324sE8bzt2Nc7Nw7t+5
lop7cuKem/CuBrwrAe8ahA4F0OEYdNCCbi3QbTp0+zuXeaDzOeh8AL4UwRd3
+HIPPtbCx2KWecHB8p3gx8LJH3CiAK6egysLcLUVHB4Fh4XgMJVlXpwAt/rg
XBCcrwfnR5GLaZgXx5GLDToHh6KWlNIbvkAHs+UldDrcYbObbwl5Pnj84bdM
IaUu/7E1Vr+YeM4kPY46V0Rqnl8KRXJyaJtGjMN18yJStWb4a3nlU2jyUrXi
55nEO9qZGnClkI40TLkR75JLlnqHen0M0inK7tYXF45C4jez7p8jnUNTBEoX
eJek0tYLmrwOIoX0Ov3hk/sXs+lXCePq98oUUpnnoTLQVkgvL6w7am2YTTMO
T9mpqZdCHsLcBVc3FNOCvAdnxdqzSVtvUot3VQr9unH5w9iVEnodfvnlrlO5
lHx9zgWD/Wn03eVx+A31MmIULuqX+yefXPmOFLZ/SSdlnYXa8Y7ldKdOc+PH
4SJafSDLT8Azkzrmlm2fvrCK7I8u0Jz+uYRmnbhodFwumxyFxf84j9dQemdo
hbdLGVnuT7v01XtCh5ONNY0D9fTndtMMC8Nyco8t3Orgm0vyYr/LIzSbqKb0
dHvZUAVlJhmu0BTLo/4f1s7C+5qp4d2dwjXCVaStavOWhzOPPlY7+X6ua6bJ
5kaal15WkQ7WQ1nWU7FPD/apxT5uOHcdzq3GuWdwT2XccxT35MG7LuNdyXiX
JHRogQ5XoIMLdNsC3Zyg21+dB6FzKXT+N1/++vgYPs6Dj79ZfH8F36eBk03g
xAecWIOrSHA1G1z95XAGODQEh3vArSq41QC3b8B5OjgXAOcKyIUvcmGFXGxY
qVaqtXni+9SIIbxnaSnFpt//uWcNg+T3MAZeP6yljgw5p5tGZfRguvMGAduJ
76B1SRbr2JspePEXnvJeBuVnyH9fZFtLjB4rcQvHNvLMcj3dFVlBr3ZvPNxU
10h1ub6/+ifmpku9vc91tyoqFj7bdpCthQZarz2/P7eHDEyKjWhbDfHMOO/n
8byVhuUHK+5p95KVixivV2QteSoon+nzaKNSLvXsFtdeir525InB9HqqaLt8
Z9qMdjowe21s+aZeksicwaZ6uYHWjRiIbNnSTl+3GWjPT+ih2RrJRuJLmohN
7YHe4jPtxB25LLDftIcEnG9uS6Jm4hOpsDnAaCehhdYykVY9dLyx81OMZwsl
2q/6J8q5g6xyi6739018B1plbe8TaqPgFzvb+5w6yfITh5BeVS8FLFso5PSs
nWb083n2R3eRiunc3sjhPmJvC7NQbewgUb82Sfu+bjrXPKNEvaifok4/GJjj
00lPlhc0fozq+dd1Bewz2srcRwT7mOPcdzh3Gs61wD1bcE8/3HMu3nUU74rH
u3igAz904IUOf3Xjg26ToNtfncWh81ro/NeXCPhSDl9+wkdL+OgBH/vgux58
54bvxeDEGZzkgJMCcPUIXHmAKzNw6AcO48HhCnDbBm5vgttV4HwTOI8A5zLI
hQ9yYYpceLDMi5nIkRLLvFBG7t4jp1zIqR5yepol17OQa3b0wBb0wCv0wDB6
Qw294cMyL56hZxaiZ/ajl0bQS37opXT02C/0WDV67BF6TxW99wC9J4We7ERP
XkNPCqBXb6JXs9GrVuhhNfTw+H/Mi3SWeVH/H/NiP8u8mIL19H+ZFw9wrgzO
ZeBcq3+Za9x4ly3elYh3LYcOddDBFjrcgm5rodvF/5gXe+DLd/jiDV+44aMb
fJwLH0fguzp8f/0f8+I8C1eC4CoeHPKBwxPgUBvcqoPbXeA2GJxngPP54FwD
ufBDLmyRi4RvgZtEDcso8uYvt9nzJvIUv7fDK45BTnRIcN2WYrJ/+GfrEYly
cjgZvFIjopSu7TlaNLk1jy7Wqvu+XMcg75TRG66zS6ii4vVczofZNM+lvPWn
Thk5X/Q7c/l5EemvGT56WDKTls0oOHjrfSmVvsmZJp9UQA/OTd4X6pBOyVPN
5cs1S+mmYv+R1Q/yiUNn2xYuszQylWl5dVS7lMxqORROcObTMW1v3fzCVGrS
EmzsSSol/t/GM/Yuz6ebqduCxfxSadBwUG2BdxmtUxR4uXogn15mbIyvrUwj
LUvruiy2clq490Hs5/7Cie+mj8mHF2fQ+eRJX8UyK8hrw6qulUHFlLDKPuTH
mUxK6ZXvca6sIj62uKYkzlJ6Z5hiIB2aRbtkTQ23zK2lIiuT7YEXyyjvt9di
k/Ts/8Pem4fV2Ld930mUIgmhkilz0kCK2I1JkiQkSQhNIlKSBlOGQhoMlYRM
mZI0SfMszfM8rOZVCJmSZ92b7/rjPrZ3vZ3n+V7X9d7P9jz/Hlvbb9i/n+++
H63jOH47Zfj7B27bXE1XPWxLNaqKaLCaiIbVxUwKcFcLbNlSS2I+t+p0hpeQ
hXaH4q8PmeRfZmhbZFVHEY+LE0a2lVDCmpgqgYZMEmKZfDfOr6OjdpcV6taU
8rxujnFuYJyXGGcQ5vXDvKKYN5vHOu9gX5rYVzb2FYs4xCMOoojDMcTNBnG7
hrgFIM7rEGdJxJmry0foMg+6mELHWugoBh0FGLqbQ3dPcJIITs6BEyNwNY3B
VRk4lASHbuDQDdw6gNtAcOsGzk+A89PgPBm+iIIvFsMXEt0zAmyvlNKU3tAJ
z3+XkPRXdvUVzVJy01qTbfupimYIHFgs1VNKm6dXCWTYVtCAL1m1Bx7WU/zb
tA86Y8qpzlDeYMy8GqrSXfxRna+JHl6Uk11oWkFlbx8NmDOunl542GV08rXS
w7hVYipxlcQfoW0m8KGBdGZEbDJf0E7DK+dNKearpnath5/atrGo/caLnCL7
Djr70vD1z0k1VOIwd7P5yiZSDdYUUGrpoATVS+Hv1WpJu271+1FJTdS3Mm1Z
XEkHiT5pThqtVkduRzfztws202etyMC+Gx1Up353hcfXOmpfMccmVbOZTpmz
pF4c7CD7p3I739fV03H3c0vyHjbTqu5x8cNPdlC2i/y4CR8bSEhN1uqaWguV
8rGybas59wkyTgkrJVh053JoWtX3FnLPsp5zz55NdSljNg9WbKKI1cmJm4pa
qd1vxsUlsp3kqeW2skiimd4cKjU7f7ONZJ9caJni10lH2u3vpd9oJu21J9K+
jGvneb0F43hgnBiMcwHz1mDecMxbgnWaY523sE4N7CuLsa+TiMMRRhx6ELca
xK0NceNb9f8cZ64usdBFC7q0QMcz0LEYOmpB96HQvRW6PwQnd8HJr1d/OCkC
V8HgqgBcff78h8MYcJgHDh0Y3OqC26HgfBI4lwDnEfBFOHwxH744wagXLvCR
A3xnC99dh+/y4dMx8KkLfGrAqBd58PUF5IF45IHT/dQLbp5pRJ4ZjTxzHnnp
C/KSWj/1whV57yjyXgDyXhzyZArypDjy5H3kVV3k1XzkVW4efos8fP3fVC8E
cd2un3rBj3l9MK8w5uVV17j1QgP7eot9xSAOrxGHoYiDHeJmgbh5IW43Eef1
iLM04nyaUS+UoMte6FjDo16YQ3dLRr3g3lecBSc7GPWiBFyxwKEMOPQEhxfA
rSO4DQK37uD8JDg/C85z4YsY+GIlfKE/ulzubXERiY9+YRj5tZKUR6euSNxb
Rs9chcSDXPLJ2kdZakFuBef/8cCrltNL6Khx/Y0Lkjm0uNlnhuG2corNSopZ
P6+Inrbpp808m0WC0eK5mypKqeuqzNYvlwpoiqKttOj7dAoTz/ZJvlhCd0af
2nBveT4ZK6lVXRybRg775wmvuF1M+qdm9KZsziND5468tVUpJPg1b9TLGcWk
HKHxq+wq577XeFaPj0wKTUyfbnOAc92lR50dvSCXnuh/tdv3NpkUdoq9uxNQ
TG4iHuc2n8ylsPH2Z12MU8hCZcvL7v+ad9f5RTeM8yjxvPLGXdtSqbdDPzWh
rpSsDey/qq7OJ+f5IaE3jdPI8cea6idHyunmhdwKJ45ui2TGD5bZlE6TrTeu
8ZCvpFt9CvcvWhVSqO5uo0czMuidxTyZ3Y1V5DVEubnSsYjzf6zos6SkDE6e
jX0+fE8NHQ3feUp1N4ejZy/VV43IpJmr/B5knqolKdEfk1UGl9Ck5B1Xx4tk
0qN079MKQ+vIJmp2+1edEprMuH4I13mNI4p5WZj3CObltc6F2NcE7Osm9uWE
ODggDgGIQzzi9gNx24+4hSLOZohzEOLMSxeujhOgozN0NGLoPg+6mzA42QRO
ZoCrV+AqGFwxOXwPDp3B7VJwGw9uX4LzQ+DcC5wbwhej4IsF8IXk0CZpRa1y
WhPZvr0ysYpe1O9bPHhwJWn5nNuzqLmGXGtjnl3bUE2s6EkX2qRqKMZhWtfZ
ew30xGDbxXlV1SQ00kIyP7OOpiQ0zksoaaJAq+cdyxxrSO31dl+FCw0UWLj+
6IQrrdSrUCycOI1zn6xS3UfiLApx/bBB70w7eaU/Ti7IrSU1pcbymypNFJN9
S2Pwgw4qUCj/MVy9jtKcYye6f+SMmxsYkD+RTfMPLN2ce6OOzG8dcNDd3kxe
XbLf1s1j02/TxbWyovWUMfOFpdrtZjra+KggYwKblkZMOhRwp56qsvfcUarn
XFcSLCgYwSa3adKaO7c2kPOTEcP85rdQpWCg+JppbJLd9/6c2+xGco1Me9fy
sIVeKpwwWb+XTRe0bfNjhFgUGPdj7JkFrfREaML0hHQ2Of9OiVhbxqJRAt1d
g/JaybI77sqFOZ20LthGVvl4ExmtHVtZo99GAf556i8PdJJx985jlqVN1Ktx
Y9vlu7yvm2GctRjHEOM8wrzHMa845g3DOt2wzgCsswL7moR9OWNfDojDKUYc
HBC3JYhbJeLGjXMf4pyOOAdAF2Xosg+6RELHfOiYAh3vQ/dL0F0Ful8FJ9/A
iRA4kQZXAeBqPrh6Cg4fg8MB4HAhuHUCt5XgdjiD88fgXBu+EIMv5OGLJ4x6
cRE+OgDfLYTvouC7YPiUHz5th08nw9cv4Ovb8LURIw9sRB7YhrwhjLyhgrwR
x8gzTox6oYS8dA55KRx5zJJRL1KQ9/jYf/LeQeQ9Fx71Qh15dSry6m3k1TDk
4TzkYZ9/c73gXpdhjCONcYZi3nrMe/gf1ovjiIMd4uCHOMQibj2oFxaIG6+6
zNVFHrqchi5JDB25dX87Q3dV6L4LnDiCk83gRA5cRYGr++AqBhyKgsMv4NAV
3C5h1IsocH4YnPuCczP4Ygx8QfCFgfwo5332JXQlNmf9KKqnN4VlohnHqsg6
s8+wKa2AdiqPamh3qaMx170vb28sp4lRC7w2SudRbXZtydyqGnJJcg9fdryU
VIojlGy3ZJP0KJnFZ85W0bZ49ce3thbT3Yv8o+6eyiS5m8nv9h+soD6bhscT
bhSS0DpWUaRTOkk+P10RWFJGORaOu95oFZDpx1najyTTqPBUUsyG3FJa3mep
sNE6n0SdlDMFF6WSavWyKxlbSqn2XefC+z/ySCt/0jLZ9BQSd1x8/PeuUqpU
3Jej2ZhHm9dOiuCvT6FVg1h6hs2ldHTTj5L9s/NpTGVvrrBfKhVFXZv5Lr+M
5o58q2mem0+nen3WXjBKo8m9Y6eu6C0npTc2U1f6FpB2Jas3ZUw6GU4QbhHX
rCRXjbiwH+ac+9x1e22fRKcTf+jSBdpeVWTaEa16ZUERyW8O+PZVIYPeNBlr
j3tVTRbnNgdnNReR4RjzPqmDGbT2xpac3R415Cie1qy6q5hsbbdel+Vcj5K6
IZb+s4a+t+96xr7B+zqvcXjNewnr7Hv+Z527sM612NdW7MsZ+zqJOExEHBQQ
BwnErRBxk0fc9BHn5YizHeLMSxeujgsYOvLSXRicSIGTPHByH1wpgKsBh/5w
pQoOx4NDI3A4Bdw2gNuT4NYWnO8C59Lg3Bi+8IIvEuGLQ+ofLHutKylhp1JK
p089fXeZ9cNSu5aG2Ten+abWkcLFhvZu/3qSOckySFGop0sfZDcfEWFRdRk7
dMaJegpnnRu4Pq2B6nJfb5k1tIXCbniuFdxdT5nz7udfX8Cibjknz+NmbfR9
+8hR3uvq6cgqzQLBbU0kcOqeWebvdvL5lH7nw6p60n4dMah6cjPVXdTMMGnp
oOUVE/pE9OppxLx5uyMCmsn0w49NFevZdNHIfbiAQz0dnZasdIDVTEsmdM7/
up9Nb+a7v/VLqCenWbF8a8a0UIu4iBSfNZsOFV7qSp3VQHKfsgV2a7fQtEcB
eRsOsMnL4MfOOeENVOW69kb8lRZKWn27quAUm1pH1lac2NhIxXMStoV0tFBF
0XsD81A2Gbf6htz73kgBNUknBDe1UvmGr+bBnWzqPjFlWbMXi5rML25aldBK
ogseDGfP7KRPQnNnPx7SRHwTLW++HddGxSLGyk/Xd9IFh0D3bK0mer7cuVZ9
He/rvMbhNS93nUZYpx/WmYh9tWBfRdgXMw6ViAM3bjaMuC1GnGMR5+OI8y7o
4gFd7KBLDUNHMejIz9BdC7q3Mzg5CE4qGFylgauT4LAGHIaCw8EMbiXB7V4G
55/AuS584QlfRMEXVvCRCXwkAR9Jw3fV8N1x+E4BPh0Dn26BT4Pg65nwdS/q
hQDywFjkgXf91IthjDxTgzyz5m/Wi7HIe6XIe4r91It1yKvbkVdP/ofrRSSu
f+unXshh3ijMuw/z8lonr3rhgjhIIQ5zEAdxxC0HcZuNuPGKM7deDIcuFdBF
HDouhI4N/dSLYeBEBpwUgJMQRr3gR73QAIfTweEecCgLbpvB7Rlwewyc7wHn
k8D5fvjCB77Igi+eFcf4+08uoyUi0u7HfVkk/Pq43J6NddTKCvVWrS2kZunN
fi0BjbT1sjzrlEc12aovpXVf8mjhpeRddyc10JEnu/m1AipoxrXkRZJp70g+
Yr/9FOc6+uZvPDjkfSk5Xeef8Swxi+r4Ys4ZCNbQ5fvbc8aEF9O+HUkbHHwz
aHJLrGp6ZCVZiuR+WDagiBLe3/M+OzKd8ndZrHw3sYIMnsqc1SwuIEPzPYM3
jUuj+2+cIprml9NMtTJpzUkFlPVyo4mYRyplOS648Ki2jEpDFKrLCvJJps/u
Gr/Bf53XtCL5qgCH/wsugtpl+fREYsLu3uJU2m6dFVVwrZzKe+WOmC0roNSt
/k4Fu9IoW7Uly9uygkoOVZx2zuFwMuTc2Wnv0+jOAv6I9bsqSUKm8fhHzn17
Lr+LgJkl537+1iRXc6MqWuGpkCg6pIgk/BZIr81MpwX642NEFlbTbU/pYIPL
nP/XjN0b43+lU/iV6Nz9NdV0Kii+7n5XEYnM2mecwZ9Bn0tMp0Yr19Cv+doh
lySLeV7njvMS45zEOLzm5bVO7r5uY1+jsa8UxCETcShGHHjFbTyPOPPSxQg6
PoKOs6FjMnQvgu7boLsFOJkKTvaDk5PgqhlceYOrWeBwLjj8AQ4dwO0ScHsM
3LLBeQs4NwbnEfAFwRfD4Isiid+Zo8fX0MTGCcvusli0KnL4yGvf62lH9BPL
qGsNJPAqrbjlIYuUIqMn6gY1Etv8sci4r02ksiVA4twEFl2pm3JFV6SJBr43
k5sQ3EoPxb/K3XDl3J/whcoZiTeT7tCn9omO7fTMc86e4x8byEVnf1dgRDOx
i0y/+vl30EPBouZRbg2kZS39vlSshV6FrlSRV2GTZbj52DaNBvJU2vLAfUkL
Lb95oEPmOJu+m3c7SKk00M2BL5TGb22hFUIZXQ4+bKowqnrZtqGBLinp5Sse
bqHebtfRXv5sOnXSW8rat4HiF14Y63q9hbrbBXfU3WWTW/22Yp9vDaTerOi3
MKeFLj8Yd3B0JJumDLqc8+NAIxls1p9wbmQrucoaa3aVcMYxq8tX62ykExoT
ho3a1UqrPS97z+rj3FeMmpv3cweLLPRqFUc+byXHVkHJc5M7SWzYtA+9kSxK
/SFxZnlnK6XI39vvrsq5j1o5v7WghUX31TpnzRVr43n9GMYZhnGSMQ6veZ2w
zhNYpwvWyd3XZMa+uHE4jTgsQhy4cTuJuMUhbrzivAy6fIMuAdAlHDpaMXRs
Y+iuCd21wckLcOIMTn51/XeuCsBVAzhUBYcXweEWcDsI3MqD2zRwPhmcLwfn
wfDFYvhCEL5gwUcs+GgzfHQQvlOD7w7CdxPh09nwaXc/9cIUeWAS8oB5P/Vi
K496kYG8lIG8VPI360UF8l4ao16UIk+OQl4NRl4dg7yahzw8EXl45b+4Xgz9
H1YvHiFuWxG3UsSNV5wzeehiDB1DoKMcdExi1AtD6G4FTmaCkwPgxA1ctYOr
q+BqIThUAYcCAX84PAZuCdw6gttP4LwdnO8G52nwxTL4Qhy+OHpn8Wv9R+VU
HDwpq+tOCwktqvPOXsT5v8bXeP4T+yK6cVx2zobdzXSpse2u93pOvfK5KJVu
mk8SUnnLZjuzyKvpau4lhWpa7rjL3Ph6Dun8+Dh93Z4GkrmhlWQqUkGfvUaW
DXB7S8qCx+c/OVlHHzpcH8uZltJC1uqdslqZZGo5s+CFWA0lPNs7oMGwmA58
NetN5dw/77jAN3/zz0oSOeV3Q5pdSOWyQvNnh6WR5Lyskm9PK2jFlAeqZwcW
0qLgkOGfJNJol53g2cGCFUSm1oJLzxdQe8cSO4X8VBrwbqfD8KZycry9tKJw
bwGppIktnDUkjQyKxk+VU6igaKmRt038Cjj/B+x7J2qZRp2/Kja1JlZQuMPq
msifBbTZZa/56cI0shZwnPTRoZKGTh7zuNymkMRzy+Q056XT99DTZiELq2iZ
x1Ha115IBi1n9193TacfkYuz3rdUUUGxK5+MfhF5NNtPuPgsnSQUJk82Na+m
ZyPeG2wKKiKrJ6mfDaPSyUl0qPCop9VkeGhBUlAK7+sXMM4ojPMU4/CaVwzr
/Ip1LsU6ee2LVxzm84gbrzjz0oWXjoeg+27oPgy6LwEn5uAkBZx8A1fzwdUn
cLUSHK4HhxPB4S9wKwlur4PboeA8AJxfAeen4YsS+GIofDFILPdbDauOvu9/
Yuo9rJV+x72RV3Ng0d6e9a+CljXRiL1h7j9ut9Ai+RNB72yaKSiO32js+VZa
dtLIYFhgMx0Mmlvh5d1ChU0lr4ovtpPd2YNnEvc3UXDE3Ztvf7XQkq9ea48/
7aDFX1ivRnPuB5b0vOZbM7iVFk7fGa25nHOf83tEW5BXI5l4q+XbtrfQWv/F
oU7n2OTVN0X2+7BG6spWsFj0toW+T98Qv/cem9iG25P2v22gQ8tKl1pEtZBI
dIGAUCibBt0/EWGX2EBn54Zsj+dcZ735pOYSwabl7ywWT2lqIK9Vzxflv2uh
WxU2UfkJbOqyU35iP4tTdx/qhQV/aSHN1JsXNAvY5OOeY6V3upGsZ0Q4rpvd
Sn03fujNaWeTyCHlwQGsRhIYfO2J6b5WErp2IuucQCdpJTd7ii1m0Yw54U+z
g1pJtvjTtV1SncS+JNF025VTr5XOLOHPaaVUl2NjImZ0krn29UrFeyyS2Hv+
+6dm3te547RjnMEYhzuvJmNe7jqFGevk7ssb+9qPfd1GHN4jDumIAzduyxhx
4xVnXrpoQ0cf6PgBOqpB93MM3dXByVIGJ/ngyoHBlS84XAkOrcChEbgVB7cL
wG3v8D+c/wDnP8G5FXxRCF8IwBcC8NF1+OgifPQZvhOH79zhu0XwqRZ8Og4+
7YavFeHrTvhaFXlgJ/JAHPKANfKGMfKGMPJGCfLMGOSZZcgzC//N9WIL8upB
5NVh/7BeuP/NemH9N+vFZsz7jTEvr3XyqhffEIcOxOEl4jAPcduEuEX2Uy/U
GboshS5V0FEGOmpARxvovhO6D4XuS8GJBThJBSd94EoVXH0FV/rgcAs4lAOH
v8GtFLj1A7ejwfltcO4Lzv3gi3L4Qhy+cF466FNqSQWx6kLV8n+10ZWEG+P8
whspatwn3UiJIgrbr7xqljnn/0Cnkp7lC+tJYsT5LNXF+RQ492vzLZVmmhG1
SFu9uIZ05eXjN9/PoZn1V04fS2mkWKvt88SsK+mijNfuuzFvaci2iAxfr3p6
3OB3e8mJMopOi6XSS5kktM7x1exXtaRU9iRk7ZIS+qaza83MwRnULDtg3q4V
1aQwbavVortFVJ1UGT6OP52Tl+R1jz+vpDkyOr4rrxfSrxxPz297Of+nCH6+
WP5f788skOkIGVBIFRP0d3iMT6N7AvFf5JwqaM2FcK/vqQV0PFq3rmhOGsWP
tXiQfa2C8/9mpvu20gKKv599cPeBNArg2/8mubeC+h61xSyU5NT3Gb+3asal
kdU2qan67pWUaj9spvqRQrp23Ghx7NB0urSkkf/ztCraH5ch9qikkEaF5ilc
1kmnPDE5Sa9XVWQ1T6vcSp6zftfDkyYcS6cBRvFaayZV04JLsfvlDhTRSadF
m9acTyeHA3fPaBpW0/hrE6/bXeZ9vYbHOCMxbw7mtcS8vNapjn1ZYF8p2Fcc
4uCPOPxCHHjFjVeceelSAx27oKM8dPwO3VuhuxJ0jwUnouBkPjjxBFci4OoZ
uGJyGAcOJcHtHXArD24TwPkrcK4Fzr3gi2b44hp8wTr0cNjeuSxSnu5SI9fd
TiLvRtpleraQmFrOgoXb2ije/8a29D3tnJz0MtO1ro1cugs+hd/toPoY7038
ya0Uuvll8DbldpI7ZfkswYlNrePGxm3raaYLOctURGM4f79w/CfFFDZFzjjf
GH6oifwUN1oelmojMYmB91w4dX+w8mPrrk2cOiVbOmSBTyvtlBu9Y0gOm0xi
L7Fk/Dn193P+JJ95reRbNFf7ehqbhNi9ulOUG0nH5GpOdW8LZT/PeiCQzKY1
7KoaZ04dF87ZmLystYU0svUzhnD+vlw51d5dspGG8w0L0vrQQgflPU0k8tjk
nPiObNY2Us9X+bxkTr0NNbWu/VHLJmP9zJUpVxrpbJWUV61qK3kcTzq29BOb
7thpGznXNtJVh7OWOzhxDS7+KHZ+cCflHVplfHI6i1Pn/CMHXWulk4u3Gk+V
6CT91BphD2MWWQwICDeIaSWBT0PGN8p00v3jCbaax1lUtKApw/cd7+vccTYy
xuHOm4N5VTGvO9YZhHX6Yp3cfW3HvtywL15x4BU3bpw1GXG+Cl2Eoct66LID
Ou5i6DgCugtBdw/o7gxOXoOTAHAyC1yxwZU7uHIEh23gMAwcCoHbZHBrCm6L
wbkSOB8Izq3gi3r4wh2+iICPXsBHGvCROHznB99Nhu804VNZ+DQCPr0AXwvC
14/g63DkAX7kAXnkgR7kjUbkjbnIG1XIM23IM7ORZ3qRl4KRl670Uy9ckPeS
kPcS/mK9sEZeTUNevY487Ik8bP0vrhen/ma9EMe87zCvxf/P9aIPujyELt7Q
pQE6foaOStCReZ+gyKNeqICTq+BKDFy9Alcm4HA+OMxm1Iu74FYB3KaB82hw
rgPOw+GLdvjiDnzRvK91jr9jJcXPtX/uuLqDyl7Mvxt8kkU7D5VZDNMoII/T
smtC1rdRjbGmi4FaPY3eJzH1nFMenbmXZvRQtIWGl284YLWulvTXVBiO+PCO
lr9xrbQ+zKKr6xcoHX5fST5TBotHGL2lb3ciH9SNa6Dq2Wpyoh/LqNe2yUt1
ZyaN9t6QWLy8jvQ/CDdP8S8hNivXJPFrOpmtlPPJaqomaymtO8UfOH5pnxF6
bkg6bbi2dPuEuVW0wy2k4mM55/+Cm/fkbjilkdPv9wfeKlcS/75JA7XVCumk
weKqmMVpJD5y44ZDmRUUUhtVO/1zAU1eJHHh/pI0shF78GJAdgUNGLPVMeRL
Acm1T73kY5FG4QfGC+WPq6Q9StcfXJxeSBvFP+p+epRGG9pPbLb1rqQ3ak6/
VWwLadfBc1tbu9IoMME6IFe6ivYdv8GalMWZd97cxulz0sl8VP2UiIAq0t8S
K3xvXBHF9Mimhxmmk8ucIZIb+jj+Knq3pHhTEdkNCBS4fTCdhrMfe71QqiZT
OyfhE0d4X4/GOM4YZz/G4TUvr3XqYV/rsa9Y7Isbh5eIgyniwCtuvOLMSxeu
jnrQ0QQ6dkJ3S+h+ELr3gZOx4GQLOLkOrnrBVQO42gwOV4DDa+BwDLg9B25H
gds94NwTnNeDc0mzP75IhS/a4Iv31tFjp65h04/8r/WK47rINsjAabZiJyXn
1Ds5UBfpawdEFl9i04Vh8X7T93XSpf0levPlu0g0/JzkzwntlOP5XGXs9P96
31pk/613nRSwtGKSU2kLDZ14ePWlgnYKnmHwPfpAJ3WucogIFWomzcjWLfNj
20h3wKoEvemdpGoqyVr8hkXJYXP1NLtbKaT0uMTMHjY1daubn2A1kkZTFW3a
30rX7p5/oFfOpkJRsVwTs0aaV679unxiKwVV8vF/y2ZTW/seR6l5jbRy98u4
83yttG9B8Cydd5z7NKXKH+bzOfeR4fHiRgNaaaxQ4YiIEjYtlrodz97WSCEj
Ysw2jGulL+Pypz5rYtOJWfdd3K430sI7r/UkqZUuWgQoF3DW0/XpTPGc6kZS
n3tFgM+ilSx0nnt/59znnLV49E1jPIuOGslpTPRqJZecwCrrkZ30+Uf0mK06
LIo2lfvRGNpKaa9GdytKdtKuU1fmV5qxyDug0Nstkfd17jifME4UxuE1rwfW
2YF1LsI6uftyZeyLGwd1Rhy4cduBuDUibrzizNWlCLrMhy5cHZuhoyZ05Oq+
CLqnQvf74KQbnGiDEydwdRtcDQdXl8HhGHBYAA6TwO1WcHsJ3LIYnFuD8xDU
i0j4Ig2+MIGP3OGjavhIHL5zhe+GwHfr4dPF8KknfOoNX3+Fr6vg62/IAyOQ
B9YjD7Qjb+xF3tiPvFGBPLMeecYYeUYMeckReWnAf6hemCIP30YeNu+nXkQx
6sX+f3G9cMW8+zDvxn9xvZiIuB1A3Pj+Yr1whi4C0KUGOm6Bjrv/Yb0IBFcD
7v7hqhVc7QaH68HhA3A4FtxeALcS4NYcnHuB80Zw7op6UQBfDAn74wsW/PIG
10vhF1OGX7j3VxKY9zTmFcW83Lq2jHF/dZXH/VUf4/5qE+P+au9fvL8awfDL
/72/+p9xf8VLF173V10Mv9hAd74j/90vBuDEj8f9lQHj/uo6wy/M+yszcH6Z
cX81Dn5Jhi9a4Yta1J0vqDsWqDtxqFN6qFPnUKfcUdeGoa5lo645Mu6vRP5N
91cF/+H7K+59C/v/3l/9v95fMe97+7u/UoPuKf3cXx1n3F+JgisPcCgBDvPA
YSy43czgtgCc94BzU3D+EPXiFXyR0k+94N5fucB3QvCdLnyqzri/8uFxf/WD
cX+l20+96O/+6hjyEt//ofXif+r9FfP/cV73V8zfYQ70Uy9ugSt+xv3VXsb9
1UNwKAlu3cHtGHBrBc6vgHMW4/4qD74Qwv3VcfyuVYPftS7hd61Y/A72FL+D
rcLvYGPxu5k/fjebxng+OA2/s8XgdzZ3/C43CL/LPcTvcjH4HW8wfsdT5PF7
rzzj+WAHfieU+4e/9zrz+N0yoZ/fe5m/i/7d54O8fqf9V/3e+3efDy7Gvsyx
r+R/+HtvJY8489KlhvGcV57H770KjN97RcDJPHDiBa6EGc8H9cDhdHD4GhxK
MZ5rzwa3SYznGmvA+SX4og6+8IEvuM9NZuG5ye/sP89N+PCcJQrPWYzwnMUe
z2Wq8FwmBM9lpuM5TjOe47jhOc5xPPd5hec+1xnPBwfhOdEFPCcyZjwfNP6L
zweZz63+6fNB5nO0C3judpvx3O1/2vPB81jnLazT5z/0fJCXLtzngzsZzwe5
ugsyngs7gpNoxnPkaeCqHVydBVc24LCBweE31T/cvga3W8FtGoPzb+DcDL6o
hC/OwRcxjHqxEj4aCd/5wncy8N1a+HQyfPqyn3rxCnngt/afPCDHqBf1yBtz
kDcqkWdakGdmMerFXeQlz39xvVj6L3o++O+uF9zng28xr/l/qF783fdJ6qFj
N3RU/If1gvt8cBi4CgNXO8GhEjjMBIfS4PYWuJUDt1ngPByca4PzUMZ7VoHw
hS3ey3qL97L48V6WIN7j8sZ7XO54j+s73vsSw3tfHoz329cw3lf8hPfK5uK9
sg7G++3GeA/tDeP9diPG+4pleM9tLI/3FXfgvbjFeC+uDe/R/c7+8x7dMbxH
x31Pewveu4tivK/IfE+P+17ffrzXJ4L3+ni9N/4/9f32HqyTGO8rMvfFKw68
3lfkFWdeulRARynG++0Heei+GJzsZnwH8ZXxHux7xvvt3PdmJRnvt48Et56M
99t9GO/lusAX2fDFYPjiN977/YD3fr/ivd+teE9YEO8JK+A9Ye57xap4r3gv
3it+h/eQbfAe8i28h7wQ7y2r4b3lhXhvWRXvOZ/Be87GjPfbr/B4v70d71Hb
4D1qYbx3LcDj/Xbme9r9vd/OfA/8f9f32wcy3m/3wr6ssC9ecWhE3JYy4sYr
zlxdOhjvt6/h8Z3CAh66LwAn6gxOssCVLbgKBFce4HAxODQFh3rgdgi4lQe3
nxmcd4PzffBFJnzRt/C/v99+BT46z3i/XQS+OwPfqcOnGvDpKPj0I3w9B75u
g69VkAcMkQdeM95v34a8MYTxfvso5Bn6D9ULA8b77aL/oe+h/u777QY83m/v
73soZr34iji0IQ5hiIMy4qaPuEX8xXphwvjuoJJRL1b2Uy+430PtASdJ4KSX
x/dQ3Pfb9cHhDHDYB25HgVtvcDuK8R0H93soX/gil/E91BN8V6jK43vbOnyH
qIfvEA/ju0UVxve2Mxjf237B97bH8F1kOb6LdMd3kXvxHeUExve2cfjuMhvf
XW5ifG97D99pzsB3mry+6+zve9tt+G607C9+b8s8n4HX97Zj8N2rGr57vdPP
97bc72p78F1tH76rHcbjOq9xeM2b9y/63jaER9ykEeeNiLNCP9/bGjJ0nMn4
3pb7nfVWxvkMzHM8XMFVLeM7biaHn8HhUR7ninSA8wZwbgDOw+ELNca5JTn4
bl2a8d36Nnzn/jv8z3fuc/Cdezu+i1difBfPh/MZghnf0a/Hd/dP8N29E767
b8d3+sH4Tl8D3+lzv+s3x3f9l/BdP69zAPo7n4F5zgCv8xku8jifwRnnHpzE
uQeujPMZLv7N8xnccQ7DA8b5DMzrRxnnMyT9xfMZXLFOZ6yT1754xeFffT6D
BXS8zDif4QHjfIY14OQpODkGTn7gfIb74CoPXFWDw3ng8Bw41Ae3A3A+wyxw
m8DgfAk4vwNfLIAvBOCLRvioFj7ShY+s4Ttl+M6CcT7DDPj0A6NelMHXF+Dr
3cgD0sgDe/upF5uRZ24jz0z7m+czKP+HzmfglbfvI89HIs+f+RfXC17nM/Ba
59+tFw8Rt82IW/E/PJ/BiMc5G3+3XpwBVyzGeT4LwKESOPzNOM9nEbi1A7fd
jHNItoPzFPhiEXwhCl9sxLlYZ3EuVijOxbLAOVqbcY6WCM7RGo9zt3Jx7pYd
47y4YTinSw/ndAXgXC8ZnOvVg/PiBuIcMDGcA5aBc8BMcG5YNs4NW8I9ZxLn
jCninLEKnDOmgXPJhHEuWQnOJeOee7kM55gdYZwvWsg4X/Q041xNxX7OF73C
OIdtD85hU8S5bUk4t20/zm0zwjlvOjjnzQnnvNnhXLg3OBeuF+fC2fO4zmsc
XufU8VqnNuO8OBfsi1ccRjHOi5NjnC/KjDMvXYYydKyEjrt46M49h3YkOHkL
Tu6AK1lw9QNcKYFDUXCoDw4ngdtCcHsM3B4A5wbgfDg4N4QvzsEX4fCFJc5d
jMC5ix04d3EIzmmchnMax+CcxnM417EI5zo+w7mOVTgH8gnOgUzBOZBsnBvZ
jXMjD+DcSO45k5dxzqQmzpmsxbmUS3Eu5XCcS7kb51i6M86xVMe5lzE49/JY
P+eLyvI4VzORcb4o9xzOSpzbuYNxvmgFzvn8jHM+mxnni37BuaADGOeLeuAc
0VDG+aLM69xxPv/F80XLeJwvmsA4X7Swn/NFmxG3A4jbbMb5otw4OyLOO6HL
OehiC12qoSNBR1HoyMfQfTV0bwUnn8DJfnBSyuAqEVw5gcNicPgIHPIzuB0F
bncwOGeBcy34wg2+CIEvzOCjTfDREPhoLHyXCd8dhO/k4NMh8Ol6Rr0Yz6gX
A5AHRJEH0pEHdiBvZCFvqPdTL1b9zXoxlnEetUI/9UIHeXUH8uop5FUv5OGB
yMN7/8X14iiuv8b1nzzqhTPGmfs364UW9rWFcb6oaz/ni2YjbrP6qRdcXYSg
SzHjPGqujlV/sV6MAifZjPOop4OrX+BqGeOcW+551JN51As7cG4IzkeBczP4
4gLjnHY9nOs+BOe6z2H0O9iDc+DP4Rz4wzg3Xhnnxkfg3PgQnDPPx+h3MAnn
0j/BufSBjP44R3CO/QacY2+Ac+8H4tx7eZx7H4tz8qVxTv4xnJMfwjhX/ySj
Pw6zb0sSoz/OAUZ/HGZfGF79cV6gj0Au+gh4M/odtKPvgAP6DnD7FMihT8F4
9Cng9jV4jL4Gh9HXQJZx3baffgfceVswr/0/7HfAKw7c/jjfEDcrRr8DZpx5
6cLtWyHF0NGQobsCdGdyogdOZMHVM3AVBK64HP6O+u8cHgO3quA2BtyGgXMz
cO4Bzg3gCxH4QhG+kEDfkKXoGxKCviGr0WfEDn1GatBnJBp9Se6jL8lgRn8c
f/QxUUUfk5voe9KDvidDGf1xuH1SVBn9cfIYfVW4/XHmoQ+LGaM/Th+jP449
+rwQo88Ltz/OafSFcUJfGG4fmSmMPjK8+uOE8OiPY46+Njo8+uOYoA9OH6M/
DvO6OaPPTn/9cV4w+uP48+iP44R92SMOJxGH44iDPaM/TkU//XH8oYsSdNnL
6I/D1TGZ0R/HndEfxxecfAYngoz+ONfB1Txw9QQcBoPDPvE/HKqBW3twWwZu
hcH5EnAezO0DBV8Iwhcz4IvHPOqFJXynBN+9ZPTH6YNPW+DTiTzqhQHygA3y
gM7frBcOyDOPGX27zvzDevFP++MUIA/7/pvqRQjjOq968Xf7qalhX+Oxr4C/
2B/nM+Jm/hfrhRx0OcHop/b/tV7M5NGnL4rRT+0jOHTkUS+4/XGswPkVcL4H
vhCFL9Tgi2j0JbyHvoTy6EvojD6G1uhjaIc+hkfR93Af+h5eQd/DPPRJFEaf
xEPok6iHvopj0FcxAX0Vz6EPYzj6MB5FH8a+jX/6Nhqjb+N29G00Rp/HMvR5
HIw+j9y+kGz0hZRHX0hu32Et9JEcgz6STjz6FCfw6O/8AH0t16GvZR76WuYy
+nXeQB/MIeibGYS+meLom3kAfTYD0WczGn02U9CXcyj6cjqhL2caj+u8xhHC
vLcw7wjMm4N1ZjL6igYz+ju/w77iePR3dkTcrBG3q4ibH+K8GnEeyejX2QFd
5kAXE4aOgtCRqbsRdD/PgxN9cDUWXCWBK14cOoNbc3DrA265fcwPgPOj4Dwe
vrgPXyjBF2LoeyuOvrcj0Pf2NPrkSqJPrg765P5GX93n6Ktbgr66ZejDe4PR
hzcMfXsD0beXD/2d16HP7yD0+W1Bn9829AV2RV/gIvQFXoA+wlGMPsLc/s5D
0Xf4NPoOd6NPcRX6FLegT/EJ9DW2Q19jR/Q1Xsmjv3MR+iYfZPR35vZZbmT0
d25DX2Yv9GWOQ1/mqejj7IA+zjro48zrOre/8xWME4txzmHeekZ/50Ks0wLr
DMQ6lzP6Ow/CvrhxsEUcjiEOHxG3CsStGXH7hTgPQZxPIc5cXcKhiyZ0aYaO
TtCxADquge580J0F3R+Ak+vg5Af6O+eDq6vgKofR3/kJOMwEh7bgdjS4XQlu
hcD5CHA+FJyHwRd34YtZ8MVx+Gg/fGQLHx2C7/bCd5fhu7fw6RD41LqfenEa
eeAl8sARRr3YgbxhzKgX5cgzQsgz3L7zXchLioz+ztw+9VKM/s5HkPf8+qkX
Dxn9nQuQVwuQh/OQhwP6qRfWyPO3kOdf91MvUnFdBNcdcf0go17EYBxuX2k/
zCuKeXnVNW69WI19ZWNfrxGHaMRBBHGwR9z2IW5XEDd/xHkN4iyBOJ+ALm3Q
ZSajXhRDR4F+6oU7OIkAJ47gZCu4kgRXaeCqBhyKgsPj4NAV3Jox6oUbOD8I
zh3B+Vv4IgS+WAhfqOhv6YmcVEB+YiGuFtM5/8eHuS66EpxPPpd8nv5QzKGY
6V8WxxjmkcCBeJ/Iw7m08vr7nPGZmTRfK9r1lGUuqdsUPdD1y6aQhCmr8wLS
qPdnS9LD4zmkVzPoTKxHFlls3dYRZJRCD4+dfe8xMIeELGw6JRQyiX9UgZR/
fhIpHlk33HV8Dt1Iuex70T6DevKLnD6XJtLCcVdXdrFy6PqR+TttjDNIcPug
teu2JtJl6aHvnFTzaOTbS4dkmzJow1a+Bv+yRPp8xvFp3/F8CghzDNQ2z6Kk
UxJHjDYl0ycPn7AzmoVUlDOhU/lTNnmK7chpep9CpC+jF3uimNyq1qk9/55L
8pvT74+6nkZNYwo1hGXKyHmn1DrhV/k0cq+9yR7lDHKVnvbr9O8KSmkJLfH3
4NTtTckOH/wzafW+2orarmriO18nYmVcTD4xOYtdg7NIVfZH8ct1dVRdsL+p
sKeEMuKNZ6yTfUtdX2xOS2+sp7p753PmSpfR5lV2d0UF31JY+cngV1X1JGRp
ss4hsIy28LiejnE6MU4NxvHCvCqYtwLz2mCdK7DOvnN/1imKfTlgXwnYlxzi
UIc4OCIOHojbIsTNFXGLR5zfI845iLMudOmGLn7QRYihozh0/Ard1Ri6C4AT
eXByHZxYM7gSBldPwOEvcKgPDrXB7QJwuwzcBoLz1+BcGJyrwxcB8IUVfKEy
c3WB7qISmmdSJL1+SgG9Srn4df3cIpq3vqjr9uVKqklVPulmUsj5//206qij
pXRkfrzVfP56Cp74XrS4o4jSUlU+TzhaSUXtB6dZnWCRV7rn/taIEgrQUdte
V1VLlVnB3zqjW8it2jno1JUyypE+xNoyoIE+NLoEXBzTTgameSa0ooJERWzv
Xw1opB8q3SXueh1k5SE73C+ikq4vWX6AfZVFBUM0Mxo8OXXcZYevkXA1FbMc
zwuJNNGmkfNiihd20NQ0kQGrHGtI+afReHX1JupaYaQn+aadxLUSTKZNqqPf
Gpe2TjzQRMIRU0M6zdpp5Gm3FfFUT6LjS+w2FzWRhIyNYsTBdtpd2xIefb2B
Yp3lPkWebibrrNxTnex2aj+YrsEey6InN9c2sU+2kFX4wLFbyzooZKrM2JM3
OON2il3vjGql5WZjOiK+s2kg64XVqtpmmnyfNduZ3Ua29SL5mrmdFLP/UpdE
UAtdm/6u9nlkO8/r6hjnd+OfcWQwjgXmvY95hTCvOdbZgHXexzpHY187sa8Y
7IsbBzHEYSji8B5xG4649SFumxFnWcR5HuKcD12eQ5ci6PINOlpAx6vQkQ3d
9aG7MHTPBycu4CQDnGSDKw9w5QWuzMDhTXAYAQ5ngNsqcHsS3M4C50rg/Bk4
nwtfXIcvTOELL/goGj7ih48Ww3fK8J0qfHef4dMN8Kk5fH0fvhaEr3+P/JMH
FJAHriEPcPPGEuQNP0a98EaekUCe0Ude6kFeCuqnXngh7y1H3rvQT72QQF51
Q17NQF61RR5eizw8sJ96wc3z75Hn6/upFwa4/hLXh+B6GsZhY5xqjHMF8ypj
3lLMy61rq7DO36gXYjzqxQzEoRJxsEcc3BA3RcTNFnFLQJw/Is55iLMOdPkA
Xa5BlyHQ0R06DoeO3xj3CTd41AvufYUNuAoBV8PAVRQ45Ov9w6ERONQBt6rg
djm4fQjOY8G5KDjXhC8C4YvD8IWOweCilVPz6fOy7K5Fllmk6WD1tsEwh1Zl
dgn0rcgmIZEqi66PGTTNomlBj18WRX8L4NdsTyc+7QFRz3+mk9Hk9dU/u9Np
rtIeK7ucFDL4rV0q4ZpOyWPLb2tMT6P592XCigKTyHtp2E2DM+lkerj1gYR4
KgW9tnnBnpJIOseji45JcXziPzr55a0UevRtZIPVwgT6lVHcsmhrJhXUfTYt
ykyh019+H1zmH0/V4Z6ZNYZvKW+LUCL5pJLBuwPeXyUTKX/EuYO/vHLI5/3P
Ez9K0ik75ofPx7okGlp0Z0KheT4dUpzYvrYiiwzF942aWZBCXfofjeNGFJFU
jWxbwbEc+v5Y1e5MbRqVLtyhHba0lCwy3pxaaJBPVHpCoa09g1gvsgu3V1dQ
Us5gFfNHhRQQl/Vue1IWpUqM//D4YQ0JLzph2SlQQu35zwVXTs6mzd+LDfKk
6im/M9Fwp2wZ6Rn7VqikZ5PDiF3yBS4NNMxcsOnWi3IaYLZP1fByNu33e1ny
aHEjWdm+M3XdWUEDGdf347ouxrHHOCIYhzuvPubNwbw3sM54rHMw1rkE+6rH
vhKwr6+IQyHisA9x2Iy4tSFuIxG3LMRZGHG2Rpy5uuRCF2/o4gYdK6FjDnQM
ge4/GLrfASda4KQHnKiCKy8GVwrgcCs4TAGHceC2b+0fbo3B7ToG57PAuT58
8RW+WA9fbMy2+tVFxVQVt9ms2TWb6kdnj1VMyKeBZyZlx97n6JNnMf64fi75
2lnud5pRTD9PLhZ996COGv1Up045xrlPyysPqjEtpxdaGUk5G1lUefD7uOjr
hSQzMHX97Ik11HRwm+RQTt2cRkb1qanFpJ5x+szXdfX0bd6JRUoy7dT82+yI
eTjnvq1C0lG5t4GGC1xfGyHbQdLbTx+W0CgnnXFPk+oaG8lw2MovewZ0UMfo
owkFnHXNGJoQsm8Wi0ps7N/upHYKmh9fsGdOFW10szQUns2ikCkXt0/f10b2
ArM7hs6uoU96V9o2L2FRTLmcqHtpK0mv2eK1wqyOLEcs1225xSJhJ8W4xsxW
SvCz0HAQb6AmEZ2T8tZNpLI194HJzjZKfzV11Tk+FjmfPWb3ObSZpoy6Y3nz
aTstO7Hi8MhBzRSWdeiAwdZWyvLO2ad0gk0XTJ4YPi5soZnHFtdbz2onX6El
Km7CXTTyfmzCpq+ttHGL089O1w7ywvURuK6L64kY5wTGmYhxJmFedcz7DPPO
xTrjsE47rHMI9hWHfTViX9w4jEMczBGHZ4jbIUbcShHnAMRZD3HeBl2aocs0
6MLVUZKh4xfo3gDdD0P3CnAiA07mg5MH4KoIXI0AV53gsBIcGoLDntN/uF0H
bi+B2zXgvAKcV4Lz1fDFB/iC4IsV8NFg+EgWPgqD737Ddwbw3Uz4dAt8mgCf
zmP4ejd8HYg8oMvIAw+RN3oZeYObZ1jIM4XIM9uQl0qQl64jL+Ugj4kij9ki
j21H3utG3puIvNeLPFmBPGmFPLkSebUTeTWdUS/S/2a9OIY8P5xRL6wZdWEQ
o15Y4/oGjGOHcYQxTgvm1cW8WZjXD+tMwjqFsE517KsG+4rDvj4iDlmIw07E
QRdxq0XcBiNu3LosgjjbIM760CUNunhAl3PQsQg6Zv/DerEIXHmCqz3gaiGD
wwxwGM/gdge43QLORcC5Ajg3gS9+wRfG8MWPawFXv5rmkQE5Dyww4/zfELnt
usmNLFpw8MqMltAsSrv8a29XTxL1uaSntxemUboKdW7R5dw3jZ2qUjUhiZyn
h42QVE2hCp27uV+VkqlzebVbTH4isTSXzt56MomCFPxyRgokkvDlK0Z7hyRR
ca3v6DduiTRynq5Z9uh4eu1lZVObl0S7NmcVNMxJpKz7utVSmm9IdkBC6Ywl
KbRy1rzdUjaJ5BQcdVZt5xsSu8wat8gzjR4crOuyDEoiJWkpfoHjCVQRFvqK
ZmaRW0tT5W29VJr2uHiK3dkkSnk4a9Lj2hz6MtP1jufLDHpquVZahqO79fbN
lRtsC2jlyONtJtXZpDX+uq/4hnQaVri3uECxhMoPjhOT78yjiQ8lRdyaMklM
9mSYuX8FicUKGkmkF5J8RK7gVl0O5w78w9UO1NLyWUcyw96WkOisI0syZ+dQ
kXjBXasRDWSw/cAn44ByijAYocfiy6U1RitWx39pJPuw9G3fXlWS6q2BSfMu
c/7vErAquv+TRXvZbAHbW1W0ENevM65H8RhHGPPmYV49zDsL6zTDOgnrlMa+
BLEvYexrNeIgiDgUIw4PETczxE0dcZuMOMcizu8RZ64u5dDlDHSxh47C0PEO
dHwH3acwdJcAJ1EMToLBlQi4KgVXlTw4zGVwexLcLgPnGeBcwPUP53zX//hi
G3zxDL4oMZ5wdINvEU1ubDXY7ptGe/O8ZfnqcqmFb+LumJkV9EVrj9XdoExS
2Pho6GCnQlK8PT/L+0odmYnN2u559R2tnepav+BrKQ1x3VN3yoZT15c9DO6W
zqfkoGilX7+raM+Vb72BPS2Ue+npwV2HCmnvkweSraV1tH30rQyWWjuVuBqK
n9lXTKxZzQ8L7RpI6/dl7at57cSKjZPcVFFCNc++WtXJNFKgR1ftSIl2WqDu
oTVTrIyMNvktHzW8kcRWNqhPT2sloUR3ix8nyklLfJSaV3IDLVlZFC36X7/f
iEfajvavpODmMTHZDxooeZyWmc/jZtKqKos4drOGPq+orJ/xq4E68hybrx9t
pvht8mEnOXVdX2JT9jDO/cCTKR/al49ooWVaCS9aIhqpR8VAZ7RpE/lNSUhm
3W8lycyKpeYGzWRot47/gUsL2R3tNlpr2EG1LOPyFVWtFPJshv9qqXYyyjzf
I67SRb/Uu1apBrWT3ED1wKmWbE5++3O9F9dn4Po+jFOIcW5iHB/MOwrz6mPe
e1inCtbJxjrZ2Fcs9rUR+0pFHJYhDt2IwzLETZQRt5GIswAjznegixJ02Q5d
tKFjA0NHQ+heAN3robshOEkHJ8bghA9c2YCrKHA1BRwagEM1cFgObtngdjq4
zQLn48G5ATj/jHqhD18Ewhfz4aNU+KgX9SKJ4TtH+K6Y4dN6HvWiCL4WRR6I
ZeSBt8gb05A3NJA3TiHPjEKeeYw8o4K81IC85I68NAN5LA15rAd57AXyni3y
3lrkPW3kSTHkyUrkyanIq+OQVyUY9eIQ8vDKv1gvtJHnHRj14gajLizC9Wu4
vodHvbDDOEKY9y3mXYd5ueu0wjqXYZ2S2JcA9iWEfS1HHH4X/IlDHuIQhLht
R9zk+qkX8tAlF7o4Qxdr6DgAOgZAxxzGfQJX97HgJJLByUMGV2XgqpnBYTs4
LAC3KuD2NLhdA87fgnNh1IuhqBc74YtY+CJkib/p69Bckjs2uJ5fIIEsq/Im
76hIJw1TfYN860yKGTunoGXkG9I7+ipoV28yfR5ROX3tN8596YIXLrGmr6m7
pPyl/qZECq+c+uzxu0T6MMNqW9zbGJL7UFl5QTmePt0NVZGdFU+fXV/47lB7
Tbbiis2Jim8oIro0135aLNVafY8ZGxtLWt5mqzRtYmlVZf3k9tUx1DJl5dN7
rvF09bd462pWLLFn0WyDKbFkXDg6NXxZElmEG+4PnZlAF/T47g8flkDnc+zM
RdtS6bzna8HhP5No7o1Ejw2GSbQ02MVsX0EWRQYGHTO6mkZjQzY4TOFLpcuv
+Ju6QvLI/8Lj23yz31KaUK785d506rv1c7ZwfREpeL2of3ggjwxLFq2qVuHc
N8usSfOSqKDe04fD+coLKWxkQ7DtvhyaWDRYS3NIHQ3dSoen3i0l/pT7HhVO
eaTmnzqnbnQjLfw11vadfiVJGU3/6CGST+OdVpL/rSaaNbsscufUGvJJPLXb
3j2PBqdsSQ0YwKmrwePsDVbXki+uD2Jcl8Q4UhhnBsbhw7zKmFcZ84ZineOx
TiGsczP2lYJ9fcW+4hGHT4jDTMRBHHE7h7h5Im6zEWcVxDkUcebqcg66nIMu
zdDRADruhY4a0L0BuvtA99fgpBKcrAUnP8DVR3B1BFy9AofvGRz2gtt6cNsD
bnXAeRw43wLOI+GLufDFYfgitFMlpWluEU1plz6ipJFESbtS+/r03lH0+Nde
ziIVpLOX4oWPpdJBsznfl5kWUHrjxzsnoutI74RM05fHmSR1Wk3aRbaUJjgk
x00rYpHU1DR+L/cc2sN/hETsq0huiG5Mu2krKbNaRUfdyqe7ixtMFvrV0avR
Hrrf1rXTpB6zj/d3FdIuzSDWvd/1tPxZhMkehXaaqKPkWWpXRI/a9/m0zm+g
HV4qdk9CW0n2voTjorXFFNVdN27q43qqecC3xze5mTq6p88PdymhR7pZ8ZED
6kks6fHtE6ObSG/nM43Hvpz7MYcTd2aH1lHwvocPvH400uvuLzcSayrpROiG
mO+f60jY6ZHXsGuNtFzR8VPg4jqatkV8xeb2etrx5evSxYIsWhrasz5ItpHy
9Owzunw59y/pRsdF3Jvo1efVr1vMmknXuMD/+/cmmvSxSX3Ru1ZqHbb14NEX
bXTa5VbJVmoj8ZTpiY++d1K5uuKa69ZssnOVGzRpXicJ4XoJrlvjugjGKcU4
BzFOO+Z9innXYl5DrFMJ60zDOrn7Use+pmBfDxCHF4iDK+IwEnFby4hbA+Lc
ijiHIM6m0GUidImBLqug4wToGAIdX0J3Gei+E7pPAiezwYk/OBkDrsTAlSG4
igSHS8GhEDgMAbca4HYvuL0PzmXA+StwHgBfzIAvdsEXK+CjKPhIFz7qgu9q
4Lsu+O4ZfNoJn8rCp93wdTd8bQNfv0QeYDHqxUrkjXZGvfiEPGOKPGONPOOB
vHQJeeki8pIi8tgq5LEY5DEZ5D1f5L1gRr3gC/qTJxWRJ02QV8uQVwXO/Mmr
EcjDssjDI/qpF9w8PxF5Xg553ht1QRB1IQ914RquCzDqhTTGkcQ40zFOX/Kf
eedgXjnM+5JHXdPHvuKxr8+oFzGIAxv1QhZxEELcjiFuzoibHOKshjiHIc6n
oYszdDkBXSqh43roaAwdNaF7M0P3WHBSweBkQPAfrt6Dq6PgKgEcdoFDFXDY
B24bwO1XcLsVnCeD8+3gPAe+mA9fnIAvaM1Yrdj2HOJb86VzaFk01dmdN9nZ
kUrshy/eGFpn0OeeYVYzEyNISa8q2r4lkdx01DKUy5LpqJ8m68yHcEry3x5j
pxtH2s0ud1f7JVDcs1Pum+aE0+M3y8y7Q2NIc7zg9wH3Y2nNDMtlQSbhlDFZ
bfyGU1GUnRMqp+EWTZ7Bc9Z0er+iZ2rvY8ZdjCSdmMVWOhmRlKm/cKANfxS5
C8ZG7miKpF0v972MPfCaZF78nBw45A1RmKLlU3XO/diaUdOLkuMpYtlJ2Ut8
SSSnPFha/WMCrYgf9nGSDef/obXv09WK0+jQGeeqOXdTyHuPt/LbI6mk5ztk
xkfnd3TIKWeO8bFMktDIKVbszaDlcX7SgcsLSaf+hGBkQi41Dw5PeNibTany
ywtiJpeT9DidG2fzCokvzunKhIA8qpFMmLNhYh3VfHfuMbEuI9XPB0fNdy6g
O0ev+GW+b6Q33ZmyLPlqWnxgwsYQs0K6O+tej97CFppkIymmb19Hv4cKXf2+
uIDG6HdPGvKrldbOcezhF60ngWH//bo2ri/BOHcwzkSMo4J5AzBvBObte/Nn
nRVYZznWWYN9vca+RmNf4ojDQsRhOeLgibhpIW5WiNtSxPka4myBOPPSxRA6
SkDHBdBRC7qnQvez0D0XnLiDkyfgZD240gRXb8GVDjh8w+DwErg9Dm6zwW0P
OP8BztXAuTZ8IQhftMIX92Z7LP49sIgm6yRq/bwYR3Nc1MWcH7ylCU7fi92W
VtCsKHFLhztJdMMqMbU+I5/Wj1S1OyxZT1Ifx230EE6nq5f7LkhUl9CpMoO1
Rw430e9znXKDhmdT8kKfqyqcOvhQd31KfGcreQg8zLdbkUf2j65cfXO2jh5K
OXS0KLXTrtXp691v5tOTHc8yS1zryWC2zCXNCW30xu2agJhUAW37Ok/TTKKe
inZ8v7gmrZmUPbKvyu8rIPa9HeV132vJI3bElSHaLFLINrZ24uh7d/IyTc3O
aurUGzvlrk4DWZ2ViRdTLaa7Vx+cfNxeRZJhg94ICtSTV87uBKl7ZbTrxoKb
vpur6azc1i+HjtTRrJVl82rfVJO/TurlXfq1NPd6wLnEO3WkcD7yphPn/iBU
bJb3BKt6+rRu0Pxft+vp2tOVq04KNVPqL9k3VNxIp9YlDDeXZtEUXcUPW4ra
qCDnm4vJt2b6umO0R+aDDrILUdFTv9VJKoOmvT95l01VuG6N61Nx3QbjjMA4
CRjnA+a9hHnjMe9srFMG6wzGOrn7ksa+fLCv8YjDecTBBHF4j7jtQdyCEDdP
xFkOcb6POFdCFwXo0g1djBk6GkPHEIbuT6F7EDhxAyc24OQYuPoMriLB1VJw
KAwOT4LDEeB2Erj1BLc3wPkYcD4enM+FL75p/vFFCXzRBh91w0eK8NEJRr2I
he804NNY+DQYPtWAr1fD12nwdTrygA8jD+gib7xF3riIvGGGPDMTeUYTeUYb
eSkOeUkJeWk18tg95DE75LEbyHsmyHvHkfckkSc1kCf1kCe7kFcLkVenIq8O
RL1gIQ+z+qkX6sjzD5HnpyDP96FejGXUCyHUi9GMekEY5xbGmYBxFDGvF+YN
wby86lo19hWJfYljX0MQh7mIgzLicBpxm4O4GSBu3LrsjzhbI86LoMtT6DIV
uqyDjvzQcTpXR+ieBt3PMerFeQYnWxhc5YCrrYz7lkhw6Aluj4Hbd+B2wKM/
nPeB86XgfDd8IY560QNfvLmpM2SZaQ69Zz9oUQp+RZJaV5bM8U8hln6e7dhb
6aSz5sCa0SZhlNqb7HFlXwJN+G7Jd7UxiaJyjpTb7wylY3XTOjeMek0Canvf
i06Kp/b73pO2pz8j9cFjE4yXR5Kz0cwborYx1PGhwFP20jM6ZpF50LM6nJ7H
lf96phJJ43/dDJQRDaXm6kU5e++8JAGRxKhMi0hq1vb5FnU4jD4/DjFY9j6c
AkUPPntm8ppW/xAV/dQcQYdOU03elGgK8P78yFssgWqrDp63nv+GRPifdmk9
jqOfPa/ae9uTaNhps+gQ6WQ6vXh4QeHAZOpsCUlfZJhGn76nsgZ5ZdKHg6ol
E3vSKXxX4c7a41mkW1RoKWaUTyFn3ki7+uXQSD73BXzmHF5WRT+YEFhK6bvi
69c/4nAvp+Rbe7qA0m0/jxG9VktRut4lPcPK6cIktmzRwWKyih4ufU6DU8ey
zsh399WQfMkN3/jqUmpSGvFTamsr9Ym+b/g1pYH0WbfWJfdyxjWffHhgSDt9
m/PdQ0m/kYwY13/gOnecBozTi3HcMK8p5p2Bee9hnfFYZxjWKYx92WNfcdjX
E8RBA3EIQBzaELdmxK0ZcfuGOPMjzk6Isz90qYAuQtDlGnQk6GgBHfmgez10
/wDdw8DJWHDSBE7cGFw5gashDA6XgMNp4DYW3J4Gt53gXA+cvwXnWfBFD3wx
Gb5w2W6wcoRgERke37HR71oM+aorlH99kkVjwtkTtsRX0MQVV3tPlCZQYbvU
cA/ffLpgfXR8/fN6kr3y4exVp1Qq9LKN1xheSntfK1x5tqKZxLWXNry6nkXm
OQlDqtqq6LCF8p51B9totmJ6kZtnDhn7K11ccKGOMg8ELtrwuo2sF5otbR+Y
R64abXPu1tRRRcsym21nW2jGbsfJ8Zdy6W5CnZqIZS0JxdXPVrdgkY/geLcc
Pk7e/2327sCsKpo8Qeb2Hq96zj+l5RM2fM4lZ7NRbQ6Xy4l/SJmRNKuGFt65
65kUkU+BxW/sDqwro4WDNw2YqFhFHo+6XGLHFlND6e2lXcFltLsi9tnbjgoa
K23v0W1ZQeo/b4yvfVxBc4Wv2QyrqSDJErf0JyWcPL5Nz/XlqhpaPvtWjlB4
JQVLarzoEm2iKWVaHg629bT0rWDeIaFq8vxovOtQXytds1Rs2pTYSKP4TWMH
jq+jlP1nvz1+0UEdkUcqtrQ00WBcf4Pr9biuinGcMc5FjLMS8wZgXhnMOwvr
FMQ6pbHOXdiXMPY1H/taijicRBzqEIdhiJsK4nYbcZuOOFsjzicRZ3GGLj3Q
pQE6zoGOT6HjWx66W4GTieDEEJwYgavB4Go7uHIAh+Lg8A04FAC3o8FtOrg9
BM7XgfMz4PwpfNECX4jBF/XwkTZ8lAIfjYHvIuA7G/iuV/WPT1vhU1X41BG+
boevj8LXD5EHZJAHGpEHBJE3WpA3epA3HiLPbESecUKeCUJeYiMvjUZe6kMe
E0Uec0Me+4a8N/DHn7zXi7wXgTy5EXnyGfLkBORVb+TVfG5eRR4uQB6O66de
KCDPs5Hn+Yf/yfMbUBcyGfXCFNfTcP0no17UYZyfqBeumHcb5pXEvNy6loJ1
RmCdQtiXLfYVi33dRRzUEYeLiEMV4paDelGEuHHrsiDifAJx9oUuOdCld8Af
XdyhowJ0NOqnXrwEJxLghAVOPBhcuYKrCagXneBQCxxOB7cxjHrxDZxvBuf5
4LwJvhjY+ccXyvDFY911ltJDc8j09M2vwxrD6MCQSy/z4zj/b1Sc8h3VnUYJ
6yRu25c+o5b9mU7dD+LItrYyfo4o5/5q8/bMEMcQmjRf+dDYp1GkaHBqasO6
N/Tpo5LzxqEPyddMp3tQSDgZvwkqWnU2ijQX+vOPn/+Als35OqtdLIx+Gz9e
ZqL4ijxKH640an1Au5p+K6Z7hdKzFOPL8x5H0uUG7cmrXz+hn37LxawEXtIv
/uNfvg2LpUXBDVtL5oZRwMbnD1pkImlYqdWXBS8S6NwkPdMPLyIpr8V3usKF
WLpZPPfH5vfJpHFszNYlnPtik8ztIx1bEylq7hl3xeZ02p1vszWnLJUuujp2
u9Sm0b6Cbfa16dk0SD0hcHBQDqkIiop8evmONC7Yrn9ilk85Hfbb9L2KSXiR
3SZXp0KybZrxzkCmmKKe2bIT6qpJIiunysOlnDr5lNXiH5TTVQ3J5t3ZjeTW
uDVxkm4dje/b3TirqZZOj+ZbKnqpldYIl5dtWMqiyQ90I4o43Bu0NKzVGt1J
6unjTlyObKf/xdh7R/X8////Eg1kS6Xs1RCKyOhWCYlEZUdSlCQqM7RESSVp
E2nvvffepb33niRklHx7n+fV5w/nvH/v37/3w31cr5fr9fZ4nvM6LyxYP4p1
AazPxz4m2Gcf9unGuTY41wTn3sQ9Q3HPubinFN6ViXcx4V3nocPIboYO66FD
BHQ7Bd3MoZszdN4Jnc9A57++mMOXUvjyDT6KwEcH+BgI363g+3f4Pl2VwYkV
OFEHJxrg6hC4kgFXouDwKzh0AocPwG0yuOUHt6bgvBCcfwHn8ciFNnJxF7kY
3MmqeXlLFb3QLxviXBNPL07sXi0oXkg/P4rt7TRpJLfA7xnv49KoTKeh5nJc
OS29fO/ss9sdZBur/K29NJsS7UtqpQJq6bZNfquOaC+pyDReLvpUQM8vUIJq
WDMd8+ZQbk7vJ2Xd6efzF5fQ6Aq/GNbHbWRif/vO4bQ+ejHKGxL7o5jWfkhe
df93KwW6pZlf0u0mTR2mV7YziijrmzH/081N9CD0VR1fUTsZPVPjyL9bQI3z
hxrn8dfRQ7aUBMGgqd+pR1rkVJMKSGKpU9h1mWoq/j5dM8Gknk4zLQx8+66Y
9kzMvCDTV0mLGj47mX6sIeLZmuunX078hi9DHabeP83NgcM8ppoSPv2Rs/Kp
oSYZOSGLTTX0dJaCxYVn1cS+c/Ks2XgzHXy/Xu9mZj1lntAVudpRTSIRUusS
lDppYdO91GGZFrJ/IGVklFhNu3LORcXL9pKplMTP8YXtFDaR17xdvpLWp7A2
PNDqp7Tm2xrPLdopHutrsZ6O9dfYRwj7PMI+GTh3I85lw7mPcc/2HYx7iuCe
THhXMN5Vg3fxQIdt0GEDdKiEbvL/6PYMOk8cZui8DzpbwBdT+NILX6Lhoy58
LIWP5vDdHr5vhO/y4OQoOBkBJzrgShFcmYArFnD4EBz6gMMecPsU3GaD2yZw
bgLOH4JzN+TiHHKhgVzcRo5SkaMu5Og6cheO3HEjd+uR00/Iqe0/8+LQP/Pi
F+aFDXrgInogAr3hhN6YRG+wMDN65gB6xhc9swC99Aq91Ihe8kCPKaDHNNBj
6eg9XfSeM3rvGnpyDubFbvTkEfRqI3p1MXr1Hno4Cz3M9z/mBT963gY9fww9
L4i5cBZzQRpzod+XsS6DdU6ss2MfQ+xD2KcB55rg3Ds49+9ci8U9F+OeknhX
Gt71ZxfjXcrQoQnzggs6eEI3Seh2E7q5Qudd0PkcdGaDL/fgSzZ86cW8WA8f
n8LHIPhuDd9/wPeZmBcW4EQDnNwAV0fAlRy4kgOHP8ChFzh8+M+8EAS3NuC8
CpxP12VwXo1c3EcuniEXKz+b2mhWlRC7pTS75roIsndb8MxGNIv4vhYJDUx9
f2q9lP8q8jGQ7p5u7b/DmUJ7dCJ5teQz6IjbjA1Zu33pg71Bz+mmWBqTfzhP
LCCJOI+sULDT9qJpv54so7BImvFV8zlveSwtK2PlMZvrSQuarrmcrwujLz+r
uFabRJNOsnw98wtP4q97MNM7L4R0bicvOWsTT/F2pZe4BPyoq+XlGtEFU9+J
GxNs19mlUHT1e3X32BCqZt+/MnF7LBkWb7Dxac0gj7U5WizuUVR4wSpjY+jU
7yPe19x/wnLoihLTQeOQRDrdaXNB9XEGXZOWF2TTKqSl8Q/zVgZnkuRvjfuL
rXJJh8/uQ8K6MtL5NOEknlxIqxznvq55XEIL92t4ybVUkstj0xsOclM8dek/
u+FQSZYrxm5L+tQRi374iJ1qI+XKKkVpTW8gDWVdltMsrXSuMn94i2UH5U4a
3wud2U6xHKI9q690EfcjExOhe710JedM1IZP3VTeWrd3VdgQCf7sXJedPUhG
Oi0Ro+JDlI31dVi/hfVQ7LME+6hhnzM4VwnnpuNcc9xzXI9xzzTckxXvssG7
5PAuLehwATpwQoeL0G0OdBODbmPQ+RR0Pg6d/5svBfAxED5+gI+a8D0KvrfB
9yFwcgWcrAAns//higtcjYNDLnDIDA6lwe1xcFsLbjeAc31w/hicb0YuFiAX
r5GLwLvCHy/cryKFA5+OM2fF0t5DgZkrBAop6u55n3iTJkq0fSrQ8z2Vwm3+
LD2zooLmv1h2I/tKJ52s+5IuVZxN656cbq1yrSNWy/zAafx9ZPOo+d2jigLS
7k1b6OHeQrc6FNx+B/eTYqSlfGJ6EZmz1nM6HW0jT4sn6z0Ee2mtu+mEYVcB
DXz06U71bibP544vyr530NdGXzGDqFx6K03nQ7zryP+haf2eqe8Iy/P1e+Pv
ZVPMkh0Pj0hWkUPNYs0tDnVTv5u859dtm/reM4racfRUOd2pPlayN7uKtlUc
99p7K5fsUn++VZn8QJVGy8/8Mawg/pXZ55p6iujWQZMssR1llO2xcnRQt5ye
uG3JSg6tIG93/7NVcytoY8z389Uny+lebbjA2PN6OnZAOfuyXjWNZccJyMaU
0w0mxdG5LW3U3xnCGhVfT2/Svy3USC6nXD3FQ1ucumixYonipVvNFMCbwz4e
XUbFGr47Xof10Movlslcs1opBOtFWF+F9bfYJx77sGKfv+dewbltOHct7nkc
99yFe+bhXXfxrrd4VzV04IQON6GDMXQTgm4O0M0VOidD5yTonAJfrOFLBnwJ
hI/j8DEAPgbCdwH4/gW+3wYnx8DJE3AyacHgyhxcXQBXE7YMDveBwzng8C24
9QG3XuD2NTiXBOcC4JwDuZiGXDxFLriQo0vIkR5ytA2524/c5SB3H5HTxcjp
xE9GTpn/yfV85PonekAfPSCCHtBDb6SiN/rQG43omRz0TCt6xhS9FIheqkEv
MfExekwLPXYWPXYLvbcGvXcYvaeLnryFntyAnlyKXvVBr15Ar9qih+djXpT+
j3mRjp5fg57XQ8/XYi4IYS6YYS5EYZ0P61pY98U+c7HPWexzCOcewLkxOPfv
XJuBe+b8My+e412yeJcadJCFDjOggwJ0G4tj6LYauo1jXqhAZ2XobABfHOFL
CnyJg4/28DEdPmrD92j43g7fB8GJOjjhBSeLwNVycLUKXHEeZXC4GhwuwrzY
D26VwW0DuBUF5w/B+UtwfgS5WIFchCAXr++lmT2OLKGmLOGoA5zhJHNdULey
JZPmPLQun1E29d354qCksIo/9am+VZvfm0Txt24Gf3VJp9U1MtoOF7xIKpAp
z/VmDPlVL9m+fW0iyb6oPesv7UE+pZIR2asjqOluLf9smRjqDhBcOTnTnU5c
6oiwehJMy/X13fdqxtBH67LVvirv6LTj+vWaEiE0XJFflcSVSDMM+Z8X53vR
9zxhjpeTEWR4KmCWFXM6+bvvr9J2DqDO1MQ0jaux9LZRrzs5LYt6bCJ4badN
fR96Fn5hlkwmy+/qWSI38kk4/ob6kGQcfei3MaxizaRzae2GMYdLyVXWfQfP
xXT69Zjqe0/k0T3zwV315RV0IOQae7d3HvGce5922LuUjsqENKjdqiWW7A/3
wyTLScBcItDybBWdH3u9bpVaM1VaZ0oq5daRi9Rm0cRFjaT3kHXHd9sOuqC5
+u2eoTbq++ww4JrSTr/DFuqvie+lo4K7TK7e7Cbv6+vvH3DooTnT8ya+7536
HSInwtMR00frfR/OHv45QCxYJ6yvxvoY9pHFPu+wjzbOPYlz23DuadyzEPd8
iXvK4F3jWYx3rcC79KDDdugw+68O0M0Wun2EbqbQeQV0zofOf33pgi928MUA
PrrAxzr42A3fJ+4zfP8M3znBSRs4OQpOOsFVP7g6D678weHBfzhMB7cC4PYw
uOUE54bg/Cs4D0QuupALeeTilueB9/rNVbRByPu0xZpYehRy467ylULarSVj
0J7cTCMuwooRK9Kmfs/2dV7JrKBFrs4e9KiLtmQGzlc/lEPvDmVquHyoJ83F
7/xYp/WTv9tVlgM/Cmj+A3F+bqNWkhoMy9/J0k9vQs/Vd8sUUl8NC+k7txL7
caX93XO6aZOj+fC05FxSvhNt0p3UQIIGp3b9Ot1GseapOqMpmXR6sUX1HbZq
MmEre8AnWU/yyl32V7akkxL3+0sbg8uIqT0qXYq3inTd9TId1qWRiuyxz77d
RaRzVVtKvbCM7nK43b8gkkEbb+4wslcvIKn6TInKqd+jau99dw/759CxlG17
L4YW0JOOu5xW7MVk2tLz3K6ghMwubj5u019MK1/ULPywvJgE1WLruz5X0YUq
u62+JuX08PNzsafrSija1l4zdHsTJezTydipUU2pUoWFPA0ldHPNpWPTL7ZT
a9Ri5mc1deSQw9F6dlMJ8Vz5nC8u3klGtdEmduYN5Iz1pVh/iPW/+6hjnwrs
8/dcP5wbiXN5cU923PMw7mmBd+ngXffxrkPQQQk6HIUOBtDtJnQTgG5s0Pkq
dL4EnZ3hixJ8uQhfxOBjKny8DB8Xwfft8F0VvsuBEy9w8gWcqIGr9+CKDVxN
ujA4XAYOn4DDVeC2BtwKglsNcL4UnF8F55bIRSVysRO5YEWOdJCjDuQoArnj
Re52InevkdN9yOlb5LQeue5ErhWR63Xoge/ogUvogRH0BivmxS/0hjF6JhI9
M4ie8UQvDaOXXNFL1uixLeixCvSYJnrPG73HYs7oPUP05BH05Er05Bn06kLM
i+3oVU30cAt62PN/zAvWcEbPn0TPh6LnOf6ZFxsxF34zMdbFsb4M658xL/Zh
Hzfso4pz5XBuFc5VwT0rcE9n3HMf3vUd84IX79KGDhuhw++zDB12QLeH0K0e
88IcOq+FzkXQ2RG+NMAXU/hyFT4+hY+F8LEXvv/BvPgC3xeBk2ZwIgdOhsDV
R3ClCa4ywKEiOIwBh1ngVgjcyoPb1eD8KThnvsjgvBC5+IpcqCEXe7Zoz4l7
W0LM09mZcj+F0oWdJw8oh2RS8WwTOS25HOp3t0vZ6etLB0S77o99SaTv0+af
sK9Oo7FiN6derfeUECU8n3l/NKlmH+EKioqn3d/TNJZsfUPCmQ/1RtLCaO4K
s/1DVVGkmt/yUlTQhdaV/+RPaA2gVwlB5lY5sRR8TaR7hdEbMlL+8WhhdwiN
P0zfbXMhmaSPjPtcFPYk2wWqeS2JU79HFH8v2FaRQS5HhKvOx/nRtB42i9np
caR7mkvs0NNc+hi5/Fp7QBjxz2yzy7iZSq6WwanmNsWURlxeFkExtEeZV9Y7
IoviryYqmXBWkJnTlwOGU3/utMLd7reRBbTCqvrlq6oaSr71kmt6Rw5JrJ6n
UNhRRiut1w/ved9ERQNjPhpVHyhr2gKPuP015LJNt3hhQTuJKsSsOfehhkSa
ZC+kb2km7mW1wjV53VSoE/7g1NTvgzT/U2MmPh3040/HBvsv/TR/SXYXh2on
mVspsRwL6CG7OXfXvZ87TPYurr3d+d30UtHyada1frL9Z90O61+wz2zsY4x9
FuPcVJybgHMdcU8h3FMQ91yMd8XjXfF4Fyd0CIUOYtAhHLrpQrcj0M0WOsdA
523Q+a8vQ//4UgkfbeHjWDfDx1H4vge+W8J3K3DyDpzog5PF4EodXAmBq78c
7gKHm8AhOxOD20lwmw1ua8H5V3CuDM4VkAtO5EIbuShn3U9jZ6qp9VjJvRbW
WGJTudc83FhI+xMUDz/e3kpmK1RM7b+mkX1b1Tel//w7m8P6bYOh3aSqxCTg
ujyXGqW+CNiLNZJ+oa6N6LOp7yjVX3cmFhWSUITTCW7WNjroUB3JEtlLVto9
F7iv5lOf8ItZ8sXNZMEhJf/VZoqjsoMfEm2yKIdpEZO63tT38W8dA2bbRqoz
fKLsYjLFs/DB7MXO5RQrweRsnFtFP66ZRxw+nUDRZy49tuctIo7ekzruJ8rI
vIMr/FJiHIWsOM8lJpNLjmq3Jj0WF9HL8wb6cl0JJMxWu0ydOZuUXkrKLurM
I9aq2FWPbqZTcOT4tnz+bDLO8Bzd/T6Xlp6YI7REKI8OPz/1wr9nyu/9D3zV
x3Mp1F1eRGLqu8NL8vnMkYPFZFOyzEHsWD6Nns4XMxCqpaYV1yQPBpZT16Sq
X+Kzqe+AQLNZVt+aKEWZO0x0aq67rC9vKPUpoIbevuduK9rIZKu79NxnNeSK
9TqsP8J6B/b5EsDYJxL7WOPcHpxbg3N7cM8XuKcZ7mmCd83Eu6TxrjPQ4Vcl
Q4cA6OAB3aygmwh0WwSdH0PnaOicD1+m6zB8yYcvCyYZPg7Bxyvw0R6+Ty9n
+F4B30+CExdwMg5OboArV3AlAK4+DjE4PAQO08GhALjVBLem4DYdnJeC89Fz
DM4FkYtRJkYulJCLfOSoGzmSQo4GMS9GkLsI5E4ROd2MnC5HTucg1yrI9Wrk
2h09EI8esEIP/EFv7Edv2KE32tEz3uiZWZgX99FL06IYvbQTvfQOPZaNHpNE
j6Wi9xzQe+rovVXoyXT0pCR6chN6tQO9WopedUcP70EP7/4f82ISPc+JnrdC
z7/EXHDAXLDHXHiOdVusW2N9BPuwYZ+H2IcN5wbi3FCc+3eubcE9t+CeC/Gu
CLwrBu/igA5voYMwdHgP3Q5DN1Ho5gidE6GzOHRWgy/VmBec8CUPPt6Dj12Y
F9/guwR8t4LvluDEHZzoghNecKUNrsTA1SNweAIcHgeH8zAvmEoY3OaD205w
Pu0tg3N1cK6HXOzEvDBGLrze+pZ5vyohrb4PMp4SU99XR4bDfV9m0rZowzX7
crJJbduSvYfv+NARpvTR6tWJJCB9LzL2USpZ9y6WOMjsQT/OZmwMoSgy9Bx7
060QR9UNR58FrnalzWfkyM4lhAb4Qw37LaLI6p53zdJ6R5oje+rm5kl/MtJy
TtjoHU9mI86d/fmv6a3GIxF1pzAqe63QO2CSRrxPrKujZ3rS5k+zAjqkomnj
04fR989nkyD7oWSJOj/Sz7dVf6aUSOb7b5Z8sSik+cfzj91YHE4sOwYSlxzK
IG6ZGXoyx8upLpX5nTJnLH0QzXGpr8mlJicPfiHDGpoRMWk5oyGV+Pa1HQ7Z
UkonVAKkYxKbSD9oc3x7bQ6xul6cp1RZSYmLJB7J9bRTPufazZHcH0iHm5U9
LriejnBvV0uw6aYPR1oP7SqvpnrVhfsEd7fRndFs6489fWQZLcM3e1UznfeN
4khM6qQRvt1eD24P0jquvZnBje00e1myiIN/D2kfCc3sqxoiaXb/eNa8TqoW
CNr7Q76PrmJdEutVWP+EfdZgH3bscwPnGuLckzhXDvcswD1rcc8IvCsK7zr/
913Q4TJ0+OPC0KEaun0NZ+i2ELotgs4l0DkXOv83X/jg43L4eBk+ZsL3xfB9
PXzXBye3wMkLcNIBrkzBFdNfrsBhPTgUBYc7wK09uJ12jsGtFDjXBecq4DwW
uXiIXGgjF/Ndc5jbh6tpfb39/MsHYumMwsBBQ+Miut8t0sp1p42WLr1aJ+qX
TiQoX939rorYpg+7sAz20CL2e2Kq66a+Q2Y8v3IhuonYTzuqXXnRT82rFm+y
ri8gyS3XY7umt1H2ky92XnHdlO0kGrubN5duXzMxipRoJDedpM+PK1qJp+tk
+UvDdFK6IXq9h7WKrqS9aUmpqKG+aY8vrbSLp3fGe7/njBWTwVura6afyiir
2K16LC2KvI5/XmncmkOffL5bfNYtpHj58c+WExF02ddw2uvYDNp1Z4VqUVUO
SVcPz5WQi6KGi05XBu6nUUJB41CxRBYZtK7xvuUQT/oJwxKsz9KI809boPmJ
TCqTftuUn59Box5pWevnZdL0J8/WWHzMJM/2a+MqZYWkGGU4T781l7QPG2qc
f51NyeZ6j/9IVtLNwNDtobYllP3CezhbI5dOir48tq2gjt6kRKgX/S4nT9+b
7u5iebSWyS9Ld1YT7QrcXNBSUfl/6yuxvhPr6dhHCvvYYZ8rODcE517Aub/N
GffUwz3FcM+leFcG3jWEd6VBh4vQ4QZ0kIFuu6FbB3T7+Y/O2tD5MXwphS+x
8OUOfByDj1HwMQy+r4XvF+F7PTipBye24GQWuGoBV1Lg6gsTg8NJNgaHW8Gh
NrhlA7dC4PanC4PzeeB8Lzi3Qy5OIheKyMUW5Og8cnQQOeJB7syRu4+YF7rI
aSFyuhI5HUKurZHreci1BXrABj0QiB6oRm8sR29sRW9sQc+IoWceoWdeoZd4
0Evz//YSeqwZPVaFHutF73FhXgih9y6iJw3Rk3MxL9LRq83o1bvoVSX0cD16
uBU9fBu9/RS9rYLe/oaeF0LPz/8f80IT6xJYr8D6R+yzEvuwYh8tnKuJc+Vx
7mHcsxj3rMM9w/CuULzrHN4lDR1OQ4fvmBdp0K0U82JUmqEbJ3Quh86F0PkO
fGGBLxNiDF9mw0dW+HgEPubAd074vhG+3wAneuDkOThpAFd3wdX3gwyunMDh
CDhUBId7wa0juJ2BeXEUnD8A5zrgvBG58EAuTJGLKn0JgxSbErorKK8r1BZC
9S1aB7vMMiki7LTprb3ZxN50xpPrlTdFrf8UsOtSAl1g3ifzcF0Kid0MsDPq
dqfYMLGv1xUip+ZIecLWrzGUFRffvbfJiX5sJlGpoCCKsbi72Kovmpa+EWS6
yulESbqZymt1A8l4VpffD64kenorRC0z9Q2VRW3wUF4bQW4ft/vLtmRQrbjG
YAW3F837PVc9xTuWzBoHW2cP5lEyOZTadgTQzUG7V2HqqSTomXzszMYyetYV
63i/IoKOnhOI/f4mm6KLV9d2lVaTWntRtXBuPM3JcbS+Oa2YJoerRL8PN5FT
3vibTTsyqEuLnykvu4LEtJcoRN3tIP098SKFYXmkeD+r0Ue9jh6EdL58tr+H
POL3cGyYV0aF7GZcfPIt9Gw1m88l8X5asbQuSeF2Nc1Zaig181c7HRSySOBc
NUjOHsNNrN2NdGhyUqnkWRclhXpLzNs5RIVnOWbKzGqnrBn17/nf91Bgn1Tm
zeQhKu76s71vtIOSelY+DK7t/b/1QqwnYv3vPvnYJwP7yOBca5y7H+ea457c
uCcb7qmHdznhXcl411booAkd5KDDKHSzhW5N0C0IOp+AzkzQ+a8vlv/4chc+
hsHHS/DxFXz/AN9Z4fstcHIfnBSAk2BwxQGuwsDVLHCYAw5/gcPr4HY/uE0D
tyngfAk4zwTn/ciFNXLRg1xMXLbqsHatIf6JWV8uZMcSyzr5d95yxdQfpndQ
W6+dzB9N2mk/y6BDChGD2ZXVZC0695M1Zx/tE3h+y1Y0n3gnt1jfXNNCRSl1
vL5dfXRK8sYvtVkF1GZz67XEhlZ6fXX9vHSpzqnfAaoH7t/JouMKqRcvrKij
6/Mqw48EN9Cc7doVC+SS6HRprL1WWhk9F+qMvR70n3/3bIfwZr8o2j5Dz0zc
MY/4Ut9/jjIoonpppwdf80Lpqt8+yzr7DNpUccAj/UwOWY67uCvLBtG8BSF5
1YLJlGDIF238OoMWp1xKP/gmmCb5xbxfZsbT+MUHn7LmpFHaw7VZF0Qjqemj
wpkFHvHUdWv2Zf/MFErR0BUQXZ1EZUPJedf1k8m70OGQiOrU7yOfhmVr3HNo
xxPFlRd8M6lgzuT94ewMmld8dTOPTRkZVGgN2d0ooOtd+ie55bPpQ6KT4xqz
GnrMqbXytHIZlfVbj9Uy5ZKRfjh75cEGijJ4YKbqXUHFWL+H9TCsa2CfROxz
G/tk4txpOFcN57rgnsPejHvy4p7deJcn3pWFdzGrMXQIhQ510CEdurFCNyYB
hm47oLM1dOaGztvgyyB8uQtfXsPHOPgoCx/N4PtK+H4ZvkeAk8/g5Do4qQJX
F8HVILi6Dw63gMOZ4LAO3BqA223gthucLwTnA2sZnKchF5eQi2LkIgQ5mokc
hSJHx5C7jchdCHL3m4mR0wTkdBA5TUCueZDrNOTaBj3giB6oRw+8Rm/UozcW
oDfs0DNF6Bkz9Mx29JIDeukseikFPaaNHluKHpv7kdF7nui9T+i93ejJ++jJ
0+hJY/SqP3q1Er1qjR7eiB6e/8+8cPpnXqSg58vQ87n/P+dFPtbjsZ6IfXKw
Txr2kcK5hjhXAuda4J7Lcc/Z/8wLB7wrCe8SgA5noQNBh3rMiwfQrQi6hUHn
s9CZBTqvgS9G8EUavpyHj6/goxR8dITvFfCdHb7fACe3wEkuOPEEV9PBlRe4
Wod50QoOebcwODQAt4fBbRa4LQLna8B5JTifZ8DIRSRyMYlcWHkeFEq3KqGa
9W2Tp8JDaEXNnyXXHmSSiNovjmURWTTdeEaVfJ0XfTYZNxP2iyfpw/G77hcm
0bLrugrH17whmS+H8nyvRpDUMcVLom+jKaRU6c/j2Y40tG/DpcXtAcTsZjV9
o08sSfx8kfJRzJkUXT5GdOoHT/WD547Th1NJ8cO6YFO+d5T58cRxFfYo8nPS
2daWlE1fs32fc6v6UNfBk86+ixIpY+ahndqfimmj8J9ajT8h5Ku1Nu79jkxa
PG1rDdulKnpyaMniVKUYuq0odu5HbAEtObHl0f0rTcS930z+2KZUEjaQze/5
UU7i4Z2fVg100PrifSWmgdm0z2ubeubiOnpy9d47LuFespzbeb4vpZDWb/oc
a9/cTMynQz+Ex/ZT5FdFm5/GZSTcL1h0cryNVlpfVxk1GaB886yP/AZVJO3X
G7DvfAcJuJcXjmQN0LoDbz+qKzbQ8ovz1V6OddKZNvdco1eD9GRX+MYrhW10
g8Pv7qhwD5kp2nXLpA+RZfGDTWqmnWQ+7yl75co+MsX603/WT2MfU+yjg30E
cS43zl2Gc//eMxv3lMQ9p+NdoXiXEN71EDo8hA480GErdOOFbjug20zozAqd
10Dnv76Y/+NLBHzkgo+v4eNb+D4A3+vg+3twIg1O4sHJD1cGV6Lgaj+4kgGH
4eBwBBweB7f84FYe3BI4nwfOp5syOH+PXAwjF5uQi4rdhS0mF2opUe6DFI9+
HJnMlbjBuruEjje2rvxs3UHW7+6smeOSSaKPU/i/itTSCN/YNm32fkqRd5Jd
vLSAVCOWHdys0ErSM52knW71kkDEGPeHp3lkHbGIXe5kM5mNcvi62LWRmOrb
qubAdDpmtX3ZsaQqCuFIvH/nRzVVrj21bpFcLLXXzNs3blVEZVUFEqZypbTj
t3HLg7NhNG9g2zK7+VlUHvLu/fVNuRQtqcVXN9+ffqf89Pfamky5qydOXtmT
QeXTPb4vKvQmwzVdyoO6sVTMJ/6zSjuZ3CZ5R/vFfMi6dM3PtY+j6OuqvRdu
zEmgResK/O/kBJJEw7EzjjuiaNhAaFvL8nhKpF+r97hFU+WZV4I2KrFTv6s1
rwjMSqKACz0lvrkZJNfJzZR8OI1qv2lYO2zJIBch8+0TQlP3ffn1Yn5oHr1R
CHEYHs6mdmE/j0KtGjo1aBTTdqOMPmZIPVL/nkvz/WtPWZs20mOZPbPfGVXS
ENZnYd0E68+xTyr2kcY+JTj3Ds5dhXNX4Z53cE8W3PPvu3zwrny8axw6TPtH
h2roZg/dXkC3MuhcA52fQOcu+JINX5akMnzpg49H4SMffEyD793w/Qd8dwMn
R8GJLjg5Bq52git3cNUFDiPA4TFwSODWENzygdtEcO4Jzi+Bc33kIhW5mI9c
CCNHk0aMHA1hXoggd7ORu13InRhy6omctiCnrJgX0sj1KeQ6Az2gjh4oRg8E
oTd+oDf60BuV6Jk96JkI9Mxq9JIDeukpeokPPbYaPSaKHpNC721F7x39Z15Y
/TMvWNCrCejVrf/Mi1L08L7/MS9U0PPW6Plb6PnHmAvPMBeeYi4YYf0x1k2w
fgr7PMI+2tiHH+ey4VzOf+ZF7j9z7e+8CMa7BPGue9DBADoshg6roBsbdFsN
3eZB54XQmR86z4AvBvBFDb68hI9jmxg+WsDHd/B9EL43wPc34GQPOIkBJ0OY
FxvAlRi4Og8O88HhXBkGh6fA7WZwexzcyoNzPsyLRZgXhcgF5wZGLg4iF/3P
dH8ef1ZC+W+Olt9yDSEbrnan2ruZdKz88i+xFVl0ytR+lGeLF/nozYrbOxhH
jZHnbzDpJFJ7eyVTsr0bBZ7o33HaMpxi7Mq4ST6KxMQrhsofvaLpK2Y6Rgn5
U1xwvMPlynjacO3bYN8SVzJhjf3QyhVGk75pQ8N7M0h8fpS42Y33xHFN6EGp
awy1f97jaxyST6WBbxcF//YnhzdrgwaGUomPRWhw65KKqe+3kO3C5yIp/77z
vd9BuTRNqbV72LmBlmwsUVu8LInMjvIbnwkrI87i9feiMjsoOyjqnu/+LGKJ
U+Lkc6ilj1KvT9xJ7iX7uYuPmZflk/1+j6vOI81Uztzz6k1GPw1PnmBiX1lM
uRl82h802khmuqFiR1ofeS1IqsucX0on1ZXN2Ve30e2C7E6tzz202W1ATfp8
OVlzJi6e5dFKFk0s90yVe8j+T4fmzDO1U9+DE0vMprWT1sxZx0r5+2hGrHvf
9aI2im1W4n1r2EVMykY/uCyGpr63Ttz/Xt9N8eH9Oy239NO4EmP9GtYjsa6J
fX7HMPaJxD5/zzXHuWtx7j3cUwj3tMU95fAuT7zrDN71V4d+6JADHXqh23Po
ZgXd5kHnZOj8O5ah83/zZSF89ISPifCxAr6nwXdT+D4KTjaBk0ltBidB4GoZ
uNIDV/HgUBwczgSHg+D2M7iNBrfnwbkGOI8B50xWjFy0IRdvkIstOcNVB6mO
tvHNKE84GE+PioUPiR0vpc1yPNw5uZ00f/7Va2+KskjU9qtocEId5SzafF3p
RT9Vy77Iu+xYQD+7F7YGf22lUdsdn6wUuyl9/YqzjWdzyPN0Wo5UQANZWWhW
CxY00sY97ePf2pNIbnBzYNXFcjquZyGi01ZO+fI7r5qxRVKmeXWx9aVcChQ3
tL9tkE+FhY826P7xJ5XgM7zHo1JpAUeS7aauDNoT5/TUcMCT2hIvbil+G0sW
4Wt/itQk0c+Q578Lh98Rc+m+poq+CMqPSG3WGo2lAc4nEXX278j0+4pZK2pD
ifVQYBP/1mj6NX+J9wF7L3pcHi0s/yuEjgqKFRmfiiebAIU/HnHhdH1A+rKd
RTSx7DAmVYlUivSaZvdhVRqpDKdpLg1PJZE815EjO7JI16H7lYpgCZWFUKby
1nzKn93nJdqZR13aUXznjOpII77y7q8dFcTS4lilfLyQjtcYWR4430aV8t4p
rz/X/N+6PNZLsR6JfcKwjwz22YhzD+DcOJzbLca453Hckwf3lMa77uFdKnjX
HOjQBx1MoEM5dOuCbs+gmyN0Zgpl6DwbOq+CL3LwZRS+ZMDHNvh4Hz6eh++V
8L0CvieAk5PgRBOczHjB4KoCXCWAq0xwWAEOv4FDPnA7OY/BLS+4XQTOl4Lz
M+C8DPMiDLl4hFwcQY6UkCMP5KgQuStD7t4hd8HI6SbkdGw5I6eJyLUAcm2O
XLP6MXpAGj3AhXnRhd4oR2+4oDf40TMJ6Jla9MwS9JIgeukFemkFeqwEPbYA
8+Ires8Nvef2P+aFNHo14J95oY8e3vZf5oUdensDelsPPT8X8yINPT8T80If
cyEJc+Er5sVlrAdjXQ37jGFeBGEfc5yri3P5cO4t3FP4n3v+nRfv8K7TeFcZ
dOiGDtnQoRG6PYRuhtDt71zO+mcuDysyfJkOX+7Al/GZDB+t4KM7fKyF79nw
3QK+f8K8EAAnvzAvXMEVG7g6B67KweFxcLgO82IE3I6B2zhwqwPODcB5PjgX
wryY7c7IRRxyUcrFsyfEsoSWPmed2PgihAp/26TsvpM59f1Q16ZmMfWdqFtz
S8TWk3gvzzX6KBBHmePpSY6zEiho488eTR5XGtpvHLP5eBgZtvcHeKRG0bK8
tTntFa9IcpHosjNR/nRdzMbytmMScYlvvD0Z8ZrOeg7a/5KMoLpnPMpVbNmk
XhKq3r7cm7bsZQmxnp9AJ1KuJmruLaW1WhcTvC+FUtjgxVtsg5k0u5X3g7l6
LQW7z5WKfhJHd55ebP0sM/U7KnjX6PHwdmr5kRe+pyKDvgXqLG9fW0PN9Zw9
7lG91BSlPLN2PI+qWHT8XkU2E/uKsSOXbPrpJu2OZ9taSPp3zfZWzGkjceHQ
/Q8KeuiA9txpHxWm5lV+RM6d5iZadXDrsvFvHeQrEpIWnJxLha94D6/3qSPm
4eHYnYfb6Gsd36yOrAKSVL8tvjW5hub5a3KvW9BKy8T8fUIlKmnxJ4vW/g31
pMPSSb0P2knxjlxK6cs2upDqNNnA0k4Rt61iOIz6SX8W50F2335aIHnMaLZ9
P7liXQPrk8RY18Y+h7DPGewzF+fOwblzcO5M3PMj7imFewrgXYF4VxXetfsf
HYqhw2zopgfdDKBbHXSugc4l0Pm/+fKzheGjG3y8CB+l4ft8+O4F3wvByQlw
sgKcqIArVnClAK7ugkMucLgHHJaB2wRw+xvcmoLzIHAuCM57kIvNyEU9crFY
wPOkEtVTwISHR5J0Au29Wigx3e4DRfJ7CPtxdZPCZtZb73ly6Mm3988VVBuI
LbHho+bNfkrmlF44ol9AsXTty5qSVjLotPJWj+ug/H3LM78mZpLGrYXVK6/U
0haJ4x0pjTX0TXWZdnZPLG3irJvQdSwmHxNnOt9cTLcEbSXcpodQxhP3XqZ5
mSR/nSPmXHgWvRgYcUn86kUjfofOWYXEU9EzUbnM5mQqGnCX4Bp2JwmzYSFz
oUja0cTDoWkytW8wO+uWTBeSPtg890t4MGWwcbc01kXRt9nSO+eNuVDqc/5g
Vc5AmrvsFUf3rnhKfyR3c+8OTxoTauqQCgqnM03KS8ZcUoh43d5I9YbQnm9l
eocPxFLvFltBNtlM0h9Zn3xySTLZVZVln+FPp+uuBzhHJfLITkJYxiK7mCav
7QpJZiqkhY0LCtZ9LKHv3jmvdNc0keShDcmSNtW00GJMlT+pjAb3Rs4cPN9N
t0xDYoyTpvoZ651Yv471gQbGPh7YZxb2UcW5aji3DedW4Z6cuOdN3FMB71qO
dwnjXTOgQyh0GIYOcdCtHbqFQTdp6LwdOitA5z740gNfjsMXVfjoAR9Z/Bk+
psF3G/heDt9PgpNlFxmcqICTZ+CqBVwZg6v54DAbHKaDw3fgdg+41QW33/gZ
nFuBc05wHoVcTFgxcpGAXNxAjlyRoyXIUThy54rcVSN3T5HTDcipAnKqj1zz
Idfnketu9MBN9IAUeuA0emMDeiMCvcGHeZGGnrFGzxSilwbQSzOCGL3Uhh5r
RY/VosfY0Hs30Ht66D0x9KQMejIPPbkcvRqEXi1Ar05DD0+ghyX+x7y4g55X
R8/roOeTMBfuYi5wYV4YYf0E1vswLzSwjwT2OY59ZuHc0e2Mcyc/Ms6djnuO
/HNPfrzLH++qxLsOQgcF6NAIHTigmz50uw3dGqBzHXQug86h8KUUvrRjXtRj
XhjCx53wcT98XwTffeB7Ljg5Dk54wMlBcDW6k8GVNLhSAYe/cxkc8oDDKnCb
Am6ZDzC4fQnOk8A5gXMubkYu1DAvfiAXO7W9v2pOrae8DzavsgohpYJeK9vb
maRn6rT5dX8G/djscElg6D3dsNmd76MRS80zTxy94xdHmzW16x+EOFPAuzCZ
avsQsruV9JDrXAxVm6Xz/rrkSPsdB5lrowLJPSONy35FGtWx986YKfiO2DMn
gvlvR5F/rC/Hp7o8ahbbVbKqw49UZb6Z3C2f6rUbZW/a1lVSKbvz+hnCUfRy
6x1ftc959I2v3kCFt5VyXFakrBFKo0VmdhGRBpX0dHCNk3plD/Xp2p6d1ZVL
ZJEndMW6ibb6NaQ7GfdTaeXEewefAlrbdKZ/vK+VtBr5r0s+7iYRHq5O0bYc
+tr+6bL8eAMtiu36dPJAG/3Sk1j0fUcGvVSMjlZ2qKK1F5cZxC6rJ7vVrm6f
pVNJo6pNR/xkGX2uFzqpOlRFz1NUq6piMig2XiX9UF4JDelIfBpIrqTefou4
DSylNLLF9PIf1XJ62qjqV9tTTZJPlS52aLdQZ3g9c/PGBgqsU8vP4KmnuQUC
Nvv39VNEQ8hOpb4pjrHOjPUgrD/BPpuwTx32+XtuLs7txbljuOcT3DMe99yM
d9njXdp411Lo8Bs6OEGHG9BtJ3SbgG7boHM5dN4Inf+bL83wMQQ+zoCP8+B7
Gnw3hu9O4KQInMiCEwtwlQmuxjMYXFmDw3JwKAkOh8CtBLhNBrfG4JxlC4Nz
U3CuiFzUIhfqyMWo0uqBnksNtF3w/WNhjUS6qPBk/89HZTRjQFG/5lYPbXaK
WeN8O5fsFCR371nYRPIZBovVHvcRp/ot2fPm+bSwwe7KPe8WinC+2pTS2Epp
Vw2c3VrSaL74GqnI/kpie3zhXK5NBa3etzFJ1T6Syl5tlLsgkkfXsvZP8G7L
o+IfFfM0ZPzoremuaWIzU+ieId+MH+FplHB5B/+SkXf0JWC7n1FNFI28Wcty
VSGOpsff439X5UwjhWZldjIhZBEn4nsyIIpu8MqE7a93IM9U5oX+rf507b6a
dEZVPCXeqPyskvyazpyoMarLCiMLH7uA6zvT6LL05ktxl7xIZY1ySNetaPIW
XaL06VEW3Ymf73m5OIQy1VyT2D4n0nzvCndXxXyS+fJglbN/ItUdlXXWr8yk
7YPqoSXcH2hfqGnQuo9FlFw8GvzzcQld2SxW0/mlii4vY9dZJ9xCl31WvJ/x
sp6675QPLpJooqZkl/Iwv37akJzeqdLZSdVYL8I6D9ZlsM8fHsY+q7APH86d
j3OdcO53L8Y9R0YZ94zDPd3xrtN4VxTedRs6KEEHBeigCt38odsh6GYEnTWg
sxN0nu7O8GUhfGEpYvjyCj5WwscZgQwfjeB7K3yPge8bwYk0OPkBThrBVT+4
EgBXauBQABwKgcORfga3y8DtHXBbAs7ng/ND4JwHuXiHXEgiFzrI0RfMi6vI
URFyx4fcvULu3iKnncjpSeTUC7luQa45MC+S0QOf0QN30QOr0RsV6A0n9Ma8
5YyeaUXPrEHP2KOXvqKXjv6PeXERvSeA3utH781HT/5AT9qiJ7nRq7bo1Qvo
1U/oYYd/evhvb3ehtz+ht+3Q81ro+Qn0fBDmwiLMhWjMBW+sf81nrHth3Qj7
LMI++djnI851wrlNOHcU93z2z1z7Oy9e4F1X8a410IFVn6GDN3R4DN2OQLcl
HQzdRKFzGXReD53vwZcG+CIMX4IxLx7Cx25Tho8L4HsGfDeF7/bgJA+c7AMn
18GVP7gaxLzQAYfh4JALHI6CWxlwmwVubcH5EsyLV+DcArmY48nIhTlysULo
V8eRqfVN6Wzaa56F0BLh1Ba5qfVYXZeAdQcy6GpHhRT7wfd0bAPHr17nGBJc
G5akJBNLOcuYxvUPONGyazcMR6qDyMeHn+flyTiqyn198OoZZzrjPF9nUCiE
9Li1U4rbMkiv+qX2p473VGI718XyTCxpafE0jouWUEXrxKcXO6Z+J5ZZD36S
zKSBbnf3a0vqiWtlv1g6VwKNbDod9PrcB9pdbnzP06OL9uz365/rnU1Wnv0i
vgX1dLxD7QqfWz89KB40q3lfQPHato8PMbVRV0m2dMfxbjq06oJvo0YObT+p
IsQd1EALozS3rM1sponcaUvr1qeS1uiiZcM+FXTUod49dnY11cr8vh/CH0uP
u09PrHIqJJ4Fvbdaf3+gU7xSig/PRJHXB+YPR6uz6ZZy8bv08EJacUT81XBm
HEk9io5ediiLPrRMZj28XUB+fwRPionk0q4bS2zqK/PoyojhtFWvi6nGezaf
x69aMrf1Psq1tpLOhj82bzj7gS5J9xrPceike8Jsq7K0G/9v/SzWb2JdA/vk
YZ+H2KcY51rjXCGcq497zsc99+KeG/95VwDedRo6tEIHO+iwErqx5zF004Nu
n6CzMnTeB53/my+b4eMW+PgAPtbBdxb4Xg/f5cFJHDjhAyfq4EodXCWBKw9w
+AEcKoHD3eC2GtxuBrfZ4PwuOL8EzvcgF3LIxQbkwqn86roZYY10c7bF59th
SSRe2F12U76cCoWF56/O7KWU61w/LmfmkbjcNtukqd9lSVFLxTmyeqj9e46E
VlYuRW4tkO01aKJzC3oOfzZooHOf7ntOv51ILgekf7lYltEJ/vpk8ZYSSvNI
Ki9pn/puYfN2ORmUSe2XlZ9tf5dJexsGklp8POkmr8LOENk4Mp4p2Oj9NIGe
+lzRYPN0pbZQma/rP4WR+8cz1cmNkTREAQlPuF+Ri0T2vuWefvSAI6zgfkM8
efvf75HRdyWvu/LMs3eEkdVIlcz36+l0not1oeqO9/Q+9iT7h6RoOiqk2pbk
kUNqNr4uAU1+5PPB8kCPcxLJNPLFaKwqJm1TlVGJaxE0wbF+qdm1TCpR/JiT
PbeCMm9JsSveT6aJMAlZ7k1T80r4UtY61VqardrMom1eRNW6WuohdhX0YPpF
C3buNtokG5yePqWL3frY3n7FFlr56NLt6Jh+CvT9/rVodh91L9i0Pb2tj5iw
7oT1YqyfwT6lBxn7SGKfRTi3+QLj3HScm4R73sM963DPLXjXabxrGO/aBR1k
oYM1dDCFborQzRG6XYPO9tDZFjrbwJdG+GIOX+zhowt8/A4fR+D7EfhuBt+f
g5Nv4KQLnJiAKxNwFQaumsDh9B8MDpvAYTy4DQK3q8GtLjiXBecrwfmEICMX
s5CLP5sYuYhAjjSQIznkiAe5i0buWJC7GOS0Czm9jpzeQa7vINeVyPUT9MAn
9IAkeuAzemMFeuMHeuMIekYePfPuf8yLRvSYJHpMAD32O5LRe72YF+fQe4fQ
k5XoSSP0JAd69TR69Q16VRc9vBE9LPnPvPD9Z17oo+fH0PPO6PkzmAuXMRce
/DMvjmL9MtbPY59I7HMT++Ti3As4dyHOvYl7cuGeEv/MixP/zAt16NALHd5A
h/2YF0KYF5bQrRc6H4HO4v9jXiyEj4vg4yn42Ajf2eF7M3w/AE7CwQk3OFEC
V0rgyg9c3QaH3uCQDxxKgNs6cCsKbqvAuQU4vwPOb2FemCEXcpgX5/I7Vuyb
Wh98bL/TwjKEgqKZ74pMrXO2yxTouadTxvQtWonuHpTTfky9LTuamkZuq3N1
RtOZ3WZ6+Z0OFGlcYzyLN5B2BUQWvf6QQOZxS0ZsylzJZnaTV+uqcNpQsbT3
43g23diffyRowpsedx5K5N6dSB7PfPdevl9BzIJd2rMMI0nNQbWud+nU7w5H
MeP0sjaaLnC88VJ5Or1onSXGPbOaEvkWiMxl66cFzkfq9dgL6NE+DX9tuVZy
HDV6tFy1Z6oXfuo8vJRLvPtTXFK+N5JmgZbWvaPNlLPhXoraxRQKvtp5TVap
ggw25Vf1L6wkRzGp3uQDUbT0gctc16482sMnw+TUV0jXW9kjbwkGk7Vcp1PC
z3QaUKh5WBWfTUabaj8bdfnToWVHxL9O/Z6tyy97lu+YQdac/mXtlWFkWPrr
d7rV1PufjOiQfAZpNRnfG/qZRk3iLlU8PunUW/hmuuqdbFJU4GZfXlpOOdGa
VvLsxeQk7iQv2JxHXL4ri7u3NZFe++2ElAVVZI/1OVjXxnon9tmPfWKxjxnO
3YVzP+DcBtzzEe55H/ccw7vM8K5TeJcsdLgDHeyhgzF0ew/dNkC3h9C5Hjon
QOf/5ksUfJwFH2/Dx2nw/Qs/w/dH8N0WnHwWYHByGJwsBFdnwZUWuBIBhw/B
oSU4HAS3WuA2G9yuAeel4LwGnN9DLtjMGblIQS7CLBPeyPE30xPuzUO6/Clk
6dchWCleQb4hIlymdX00JvaV27kun9STwlp857ZSNNMM2Uunu+iRup21yels
esWz8ZqZbj3prZOYy/yqmharckqpvYohsZTHFxq/F1Jt0VEzqU959GZuYdPz
ET96pGB6X2MwhUQHuSRMulOoxH54+asBd/qyYcBsYXgkJbBwLbVziaZb5V+O
hXg5EOe48rUHGQGkwxMZKZsWR9U15sI7G5xp1TGl1JqYEGrNehv9xi6d3ExS
LtyI9qCZa49ax6dE0wy3+JOTx/JILjjwgXe8H3Hu97q9gVKo+MR+4z2ny6hb
zL826Gc4PVHffNvPKZscT2mmftCuoQu5vT+8TeJJ6ND7iR9WxfTtemK+mlMz
3Zi1SCzu7NT36M/2iBa1StrtdcX5ZnEHLbyp/YPzTikRmZ4Yu9pAn3P3pdgb
95Klq6bFsj8N1FDwQNLOuIMsi6NeHpQcIpcOryWRwZ1kVxtU+XCsl0yw/gLr
FlivwT77sE8S9tmMc/tuMM7diHMrcU9Z3PMh7mmOd8niXZx4VzJ0qIQON6HD
J1eGbtug27gMQ7ca6PwSOv9cw9D5BHxJgS8z4UsBfDSHjwLw8TB874bv0zcy
fJ9ZzOCkEJzEgRM7cCUKrs6DqxZw6AcO88ChK7jtB7eK4NYUnJ8C59fA+Xbk
Igfz4hVysQA5SkaO0pCjIuROFrl7g9zJI6fOyKkrciqIXBsg15bIdTR6YCnm
hSF6YBF6gwXzwgG9kY6eWYqeeYKeeYpeWoRemoVeOo8eS0KP+aLH1NF7T9F7
zOg9MfSkJnrSEj3Zi159il6VRa/WoIc90cNG6OHH6O0r6O1G9PYIel4bPd+O
nnfBXFiOuXAbc8EO6398GOsaWG/CPjuwjzf2Mca5PxsZ56bg3L9zzfK/zAtj
vOsk3qUMHYyggxt0eA3dkqGbHHS7Dp0LoXM4dPaGL1vhy2b44g0fx50YPl6C
jzPg+3fMCzP4bglOBjEv9oGTiXIGVzLgSh5cTfozOJQFh1rg8BO41QG3heB2
GzhvBeeD4DwWuTiAedGFXIjL9HfT1PpXqV8uSlPr5wYWXRGYWr+vtnZgVVsa
uev9FjjS9o5GWvQMeYaiKGpkqWXWk//892Dd7t+fvCJWenlDStOfXk9uWpQ4
mkwb4xKSmL+8IRGxuaYpSyMp/Ena+l0yBZTocS3utkwAtVx3czjlmEYrmXrk
eLbWUdLX6WuPb40nhdSeV7p7S4nprITXpiU9lK7CV3VlSS5tidh/2sKtkTYz
W40X6/RRhXbmWqXr+dTlnpi9yrKFxlaHpPLrttKTWKHTfTfT6PBGlYGtPpW0
en75I6+ECprx4vge54xI6jLoPsKunUesVrpPsvXyaeU7sU37vP3pw9bkY9Vu
qcRk9MDfyX3q+4Tp/tmDhz3pWCq3sdaWWLI69uL2aGQSjXxa6jH9lAdJeZd0
iH+KoCBWqY/dE3EknXbqN4ehD/ka3LkzphJB+QYmeYN28XST5yTvVqs4WnZw
UM+WI54EvsxhrrqUTvbMCzZsNS2hC+NzHaeP5tL09H6hX8E5JPwq42z/vnpy
Fn+9U5SjgibTGOtrsP4S62uxz0Pscwz7ZOLcIziXBeeG4J5iuKc37umCd33G
u2TxLnboIAkdzkCHpdBtC3TrhG6bofMC6PwVOv83X8ThYxN8HIGPI2cYvofB
92XwfSY48QAnm8CJA7h6C66SwJUjOFwNDoXBYQa4VQW33ODWFZyngPMZrQzO
VZCLpdKMXOghF3mLWRtjaluo7PTea+m1qWS0ffBXxYFK6tJ6Y65i2k8/Fpxk
VTctILVhvQMx7a3EfyRV4KxtO+3O+qCqbptBP/ZETv/yuZosThpoF6uX02yN
ApEB9QiyClv3PTc5h8JjuDY+XZ9FXW8UG76s96J+lZDl+0biSHDzNY5ytnha
u1/dwW2WC92aZh55d3YobVkRzbT7XgwdVrl7WOehI0mPvVx8sjaQ5or218Tf
SKNDpr17ogzfUfizBZUiflPzYXdthKt/HuXtrBM2nfoeED1k8zvBKYWq3b8b
Bc2tIDX/xoqBDZE0bBEq6uuSS6cEszmvczbS16HMG/uvJ5GcwVnx1h9ldP5A
Irfy5w6SPT3XINYui240PYo1ra6lnuGtb86a9FJuctp5qdYC4mx+/0XgRzOt
V7Q55jPaT7c2domUKZbTlqg/0SxH2yniyRkNs32DdNJXaf+fEw2kG2zKLLWj
i1h+rLobaTdEr4PO+h+/1kFlx1PnTTfqpRlYd8B6MdYDsQ8v9lHBPmtx7hWc
K4hz63BPf9yTCfc8jnftwrsu4l27oUMtdNgJHbKh2zHoVgvdPu1i6BwJnZdB
5x4Rhi+r4IszfOGBj9vh4wb4KAHfd8B3a/heBU7Y3BmccJ5ncOIJrtaDq2Bw
pQwOr4BD4b0MDlvB7Rdwew7cvgLnb8D5eXC+CLkoxbzYh1zcRI4ckKMBzAsv
5E4KufsiwchdPHK6HzmVQU7jketM5LoPuZZAD9SjB26hB2ZiXmSjN7ajN9ah
Z/LRM7XomRr00mX00g70Ejd6bNyW0WN16LHvzxi9twC9l4Te+/OI0ZPi6El5
9KQpenUUvboXvRqAHj6OHg5AD+eht2+gt3nQ22Lo+VT0/H30/AzMCxHMBVfM
hZ+YF1xYt8I6H/a5gH32YJ94nLsQ53YdYJwbiHvu+S/z4hPedRDvWoh5IQsd
NKDDDswLBeg2W4Sh2wbozI55MQCd+dYwfAmGL+rw5Sh8/AwfZ71l+PgN8yIa
vq+C7xPTGJw4g5PV4OQeuDIDVy7gShEctsYyOJwBDvPA7WVwuxLzIh6cN4Hz
pZgXb5GLM5gXbsjFI/n3wxJT69tqDBfLTK0/Ybr4c8PUurOLD6k8+s//LyT4
vOajt2R/0XL2ublRtKN7ps+bZ1HUb+DqGvjsFW2V1jnMquNPaufVlywxTacF
WbGSK0w9aN3cl90q76MpviffWevsB1ppkZBy/2wYSXq/fvzYJYvcv6z4EizZ
RpFuJw7ESKaT3NkYmYZjVbR+zT6fmc/76fXvwPVczwvIumM4tGi4lZY1qxSW
3OmgGfPyn03cyaSmttIh68W1dODQt/KqVVWkI3J8omV1NLU5sr+0FigggXvr
3p/9kUd78h4dH5v0ox/95tu5fqSQeeLDpxOc6fSYZ/UpYWEPWiu1n9dy6u8V
EVltLYmjQ1uuZXQYuVDzbLddA0NTv/eX6h/xWhJFSwMSG+0/O9K6zIvMbPb+
xHau5nD5+ThyuSnwML/1PQUWfMtuWB9ObxQ5NT1NUuhet48rR040zeK9MK1u
dQLxLE5XeqCaTZIzI8ez5pRS8JwEvxViBbTSRn1RbUkhRV7TWhib10qruDlK
jznX0HKsu2N9CdbnYZ8V2McJ+zzHuco499MyxrksuKct7umHe/bhXVx4lzDe
VQEdjkKHbuhgDd2eQ7fN0E0cOh+GzrMHGDr/N19E4OMy+PgDPq6F767w3Qq+
PwMnb8HJVnBiB65+PGVwtQFcqYDDOeBwBTjcAG7rwS0fuI0F58rg3AeceyEX
Z5ELd+SC83Rg0KyLbSQnfm3enIvpdHX9kzCOe1UUHSpQcNSkn6xWplgcMymg
mjHuwUttrXS1IqgvQKeZpJ6cmAzVSaFm15VVA+oV9GprFu+buiIaL4+SDqkL
IvZF8bHqmhkU+ilI6Yt/Kp3ceGZ4e8Bb6uA11fTgj5qakxW7C89H0anHCw5e
uPCKTOc7l6ht96eRduWRg81JtEsvdDCl5TWtII+YB9oRVHT9XWmCeC5xSZ5a
U7rbl8r33j4Vkp9EG2pXtw7rVVDE5NG43bcjqdJprHjPvDzqcfC36b7bQt0y
47eNHqfSvKhXjpfmVNLeOco6bio9JG8l/qpMP5fKdFe/PPqzkcYLt1+1ftdP
q5uXn5s1XkCrMiaNN8xro4iHa+8ccu8lQ/noa6b8BTTPbkF5QG8z3a9OqpIJ
6aKu30HKcapFNFrVEyDyZOp3U+8761GTLqr6El4/mFhDXGr37mu1tNKv1zF7
8t0GaI6GoF9YcxdNNxn5vDu5l7qw/kOdsf7RmLG+Gvs8wz49Fxn7GOPcKpz7
Beem4J4PcE9e3PPvu1b+8y5+6CAOHeKgQz50y4duvyMZus2Hzm7QOQ46+8GX
r8TwxRu+lMJHLvg4Cz4mwHcN+G4P3/vAiR04YeFjcJIGrjZUMLhaC66SwaED
OFzgxuAwEdzagdsWcPvzFIPzTeBcBpwrIRezkItryMUL5GgTcmSDHMkhdxPI
nSxyZ4CcrkFOdyOnWci1IObFQeQ6HD2Qgx64hB7YgN54g96wRW/MRs98ncvo
mXz0jAh66TR6KQ+9tAw9tgk91oN5cQ+9dw+9x4ney0FPSqMnq9GTDehVbvTq
SvTqPPSwP3o44p95cRe9zYZ5sR49fx89X4aeX4u5kIi5sB5zgQfrFlifhXU2
7PNzBmMfE+xzB+fy4dwCzAtW3NMB9wz4Z15w4l2b8K4W6HAaOnyFDm+hmzt0
2w/dNkNnKej8Bzpfgi+W8IXZieGLInzcjXmxqv3/e148Bicu4IQfnFwGVyWY
F3PAlQg47MhkcDjAweBwNbitArdLwW09OL8LzpPA+QBy8R65KEAulHIVAv+z
7qF+qZX+M0c2Lz7+n3X2nV/eLFycQrKHj+1Zstidrvswjz0XiiRu/iXnbgrE
UOTRF7EGAo6UMxEvy3E7kGbPe6rmtjWLAt4873uz1Yu2j/m/ZpqMo/GNuQci
VlbRTx1T06iV0bTTfP2Glo0FxP1knrfHs25qSBv+6Pksh+Z+K1RlH2ogCV5r
o2bmXtJuV/rawZxHLovf767JbiJhodZr4mvqKLS5+OipNfHkpm5P84RKSffj
sxtmXgWksK4pao93ALU9kCyVZUmnl8wNRT7WaWS7bpAz8+U7MrxSuPF5fBSt
GEm6ItkSQ1Z6RQo06kimxaG7ys4GUfv8iNeHfWPorOqNX+cnHCm+7wiP+rog
+vGGz9NWNpl833Xb325wp5tm15idWiPoRJ9ouejKqe880ecDpWy+dCSm5rmm
5//j5L2jev7/93Ezu+wdIVSSKJm5ESojkpRdyJaSzJStpBBNFe299957771L
A5EoofF95n69/vg5x3n/zuev2zm3P273+/26rtt1PZ4cQsnnQHtseF8KdWc8
9bz+OphMecsNIxrjyHDBgw1O3LnkP/W2UMPyPLq2eIOseHAuibb5GscYl5Cd
wcy1Bi7NZLPpZmxgdx0Jof8U/efoa2POG8xRxBwHnFuCc5/g3L24Zx7uKYV7
NuFdxniXGt5VCRz2Awdf4LAcuFkCt9fAzR442wLnR8D5X7xIg8cU8BgGHneB
9xvg3Q28DzxiOomDTj5/Zzqpha6qoatp0NVI6NAROhSBDkdCt2+hWy/odgl0
fho6vwud38JepGAvgrEXv78IfLp/tZ5OvZq6+cHVeDKdKnLnZF4x+UjETP3q
0kondJbf++qSRk/M0rW982qoyUx8iMj8Mgo8Grp1xfwwmqS+MP/nkhySiQpv
NVufQlu3iR5/s96FlLO339uYHkk/d7UEXDAPo2ELy8fomlvSLdNjXWOifGi+
VO4pz8mhdPPihMmlky2o10ZB4ESpF40LVfOfsyKRxs/efEN5hSM9jxZ/8fVX
KH0tNw4S582nk10hJSa8/rTDYoOQlVoyGdYGnXCSrqUxAre0P8rEUorp+Xd7
jQpJba0JHZjTSiYCgXOLedPIYFjMk4yFNRR3REzttFwrvVg8zm/pwTTiXywh
r3Gqhrx1+dJ/iTfQML11rmvOJtDy6s8BaTkl1OHgmnDKuoLCZPfv9X8RRc/X
bxrnVptHCz5v0LylVUzbuXM3TNGPJX//FbRJMZvMzNVmS0gXk1y3g6C1bT5d
SRnu0HqlkNYu4ztlMrqavrxMedVwrYW271/mMK2vgaajn4G+EPrPMaezi83Z
jDlLce4SnOuHc0c7snvG4Z42uGcQ3jUG75LAuzKBwxvgsAo4HARuj4HbLeB2
Cjg3LGU4+wDnHPBC4GUeeOkIYTx+ncV4VAePY8D7SfBeD94X72Y6EYFO/KCT
89DVaejqAXQlbM50+Bs6vAAdxkG3V6FbG+i2EjrfCp1fgs6XYC+uYi/uYy/6
17A9ksQencEeLcfepWDvarB3C7CnsdjTfdjTEYJsr4eps73ehr0WQl58hA/w
Iy/WwzfOwDdewzcmwWdM4DPX4DP74Uvi8KVc+NI9+JgufEwDPjYdvvcAvncd
vtcGnzwFn4yHTw61Zb4aBF+9AV9Vgg+/hw/v/R958QY+Xwyfd4DPr0IuuCMX
7JAL/OhfRv8e+hcw5xHmrMWcVzjXCedq4dz/cq0C99z1V14Y4V2n8K4c4LAd
OLwDDtuAmx9w8wVulsDZFDjrAedn4OUSeOkELw/A47AaxuNwNcajLHjXBu/O
4L0XeRENnXxAXsQhL2KQF58fMl2lcTMdnoEOG7qYDgcEmG4toVsn6HYX8sIQ
OjeHztOwF9PU2F40/5cX6/qtBvtz3u6o2MTpx8vLCgz2HR+d8Wl1jiT929wb
2pytyWGSmZ6/jD/JvTLRvygUSmZG9+mSkAVd6bpiwdXlRQ5W0/LNK1KoK/OR
qkWFC73NknDfaBJF4ft4siLDK6iH/6N8VHgE6UxYrDktKI92p5i81ldvI8/i
z/wG6ukUopNt05ZWS5ERkR2XOL+bvvnlnr7M+d3U47LC0IDzu6n+XPBn72c5
VPDVTTH2mQ95Fnyp3zMjkTZxnerySoijw5pRsa0JdqS4UGHx2W9BdGbO9s98
WsHUaC9a/v3q4P9fZ8I/L9eD5Gf+jHzvH0rHWxZUd4Rb0NJXW76dsPOmaz8W
DrXoiqPtv9NXrh9jT9LXzYzfrwmmF7uyl5lPT6PPsr15mnfdKOPYeFUtiRga
2Xjh0jOHPNrwNtz4Tq4/vd5Z/uLC9GTyKRJTzphZQjPsTd4oz4+mbxvLJO1U
s6nO6xjvxoxqUr3lUJ02Np/4ptzTfXChlKo2uvB57GqlHbsupEZXNdHkbaO/
i5k2UwL6Yuj/2sr6FZizF3OmYo49zu2xY+fW4NzeBnZPUdzTAPe8g3dV4F3+
eJcGcCDgsAk4bAFue4HbVOB2FTh3AOfPwPlfvIw4z3jsAo/x4DEXvI/yZ7xP
cmW8S0MnztCJL3RiBF1lQlcq0JUFdPgBOjSBDldDt7rQ7X7oNh46d4bOg6Hz
xevZXmzHXlRiL15XlPoMdW0g+fJLcUNdE6hl4rd5CfKlpJBcf2tNezNZJ9W+
XNOeQjdDZh1/7FZFihtEl2jrFJJuUYu6tk4gaYl6bNl3OI3UTFSEBKPjqWnN
DHGhaHs6pnI69KpwCB2ReJjYnxRICv4X9wxPfk2+C+dZUIgbLaza4lh0J4Zq
jOt31d15S9nzeY9wcX43LSn7MrLuRRbNONF48esLL5I4qC36uTyeCi8/nKRb
UEX1i/y63xRE0ftIixaT4QXUHmETcupKK0Ut+yX29koaVW233pppXEM+PCtU
cy4000tBxaTciyk0afa2fQkjq2huzVzdsyeraKSq5rR1l6KIR2djfURMPsUu
vbPBRbWANFymPJ9iFkBHJg2d/3FkKpWLCzULWWbQTs9UgQmdXlTCIzJT+UMc
6asfivOsSybRh4aLGrs53xW+8xR4q2Loe5lN8gbrFMpV6LOVKk2jumnZKyk8
lUKSufunemfRDbcFL5xM66jpkHNOW2wp+aJ/Ev1s9BsxRxJzXmKOEc6djnNn
4NzPuOcu3LMO98zHu+7hXVfwrrXAYQFw4AMOScDNBbgtB251wNkXOGcB59fg
xR68RIOXzlLGY6Mq43E6eBwK3qPBexh4vwudXIdOqqATPeiqH7rSg65eQYeR
0GEcdKgJ3YZDt2+hW03ofDl0HgmdpyMv2mzZXjhgLyyxR3ewR9bYo7PYO1/s
3SvsXSL2dHoW29M07GkK9nrkYrbXD7HXCvABf/hADHwgFL7xBXnxDXkRg7zw
hc+YwWdWwJd2wZe2wJdU4WPV8LES+JgKfE8dvrfur7yQ/isvHOGrI3YwX22E
r3IhLzbBh03/yovpyItO+HY7fP4efF4CPt+EXJBHLsxGXrijPwf9BuRFHuas
x5yRmGOKc9ORF8k4979cW/9Xrv2XF+V/5cVJ4LAKOIgAByHgtha4/TBhuF0A
zi3AufG/XAYvl8DLRfCyE3mxoZPx2A4eM8H7cOTFeOTFNujEETrxhk5Uoat3
yIut0NVO6NAWeSEKHS6Hbq9Btzug20boPBU6z4DObyIvLJEXPPvYXjy8o2s+
2DdNkXQY7E+98WT+YP9zimD0O/EIisiZVPpO/A3t4b1R3qjqR6laUQ7VbuEU
JnA2udrNim7PG2doOMaP1NtKk416M0nhZHu9US/nO95ecKBUN5727TZ8uHh8
A32s2myyeHwCPXtY0SLyooTCecefvH+nhRo7Fj24fyeV87vNLMH+SDW1j5y6
9dO3IlrB79f1iYNvD/e9ey2iGZRaKec9zj+Rbg7bv3mivyOp2ZyM3msbRosf
O4/a7hxC/Rl3fp91NqfG0Edc7/Z40fe4M5nRFEZJ97caKW+2pNP2N3wK9/rQ
R+2k8ODFifRYaE3r6qWOZEQFYg+/hNJdD41rKdU5JNNs+n1Zsw+N0doZ7rYr
kYIfzoqdfquMxnxf19nzKozChrj9LrLNIbnX420eN9XTcO4FNgnrEyinZOpW
iaslJNnXu+1cdjMdXfKupWVBOhWmNG1qmFZNQ2+//jrmWxvVNS0YNnl4Ea2V
ULi141c9/Wp9V/JD9RP13/H8tuBtPfUGLH5vNNBMP9D/hn4X+gO32JwizBHH
nI04VxHn5uPcbbjnrwnsnom4py3e9fUbe5cV3qUGHISBQ88VhkMVcLsO3G4B
tybgHASc9wHnf/FSAR6fgcc74HECF+N9L3jn42G850InP6GTCbOZTsSgq2zo
6gx0dRI6lIEOf9kxHbpCt5bQ7Rnodlwq03kpdK4KnftjLxKwFyuxF8Z93wPm
9TTSk/6VIfN6EklE977qku4yqu51uvxat4ms9//Seq2bREkmp3b12JTTy9/X
ZRNEc+matpFagqgvKY/P1ryll0hi3aH75h6OouE6l2LnHrahW4bpcfvC/OnF
OVmbmyYhVD90lO0tE3MaJWg4vHS1Fy3seDLxrV4yuQrHXnun50y7I7vbt9dE
0JaMtJrjh0rpzbFdAicOhZKbfaVA6Ips0lEcJ6x8oYUkJJ8YHrmQSqmNXtyf
t1VT4jc1vvG176n62b2D82uTKXz64+5CpUqiNR/p1aoyGv+KX1lBLIzijwQV
n6Ec+tQzNcNbOouCxNfvmLLXiyYkrtyzzpuT+yrpUiMGEsidxrqnrXKkD5/H
qYdkhpJJTrKDdkwU7XA673hU8x3lz8l7v10ugGye3Z/XHhBG8hfSZ13zdKaT
fIHF+5P86Yjt072jFkaR3UP73sMW8SSncMbpuFssPXFbvEK9MIlsIj7ar3Yp
JQ2Z2Lndo/LpJvp66B9FfyXmiGBOyj42xw7nCuLcwzjXB/eUwz3rcE8evCsE
7+rFu6b8ZDhUAofFwEENuK0Ebh3ArRI4dwHnUuB8HLzMAi+O4GUqeFQHjxrg
cSR4NwTvAuBdFToJgE6yBZhOVKCrFdCVGXT1CzqshQ6toMPhfUy38dDtN+j2
NHR+HDr/fYfpXA57cRx70Xmd7UUj8sIXeySDPWrE3hVh74ywd7bY0wfY0/nI
i0PY62/YazPstRN8IAc+UIq8aEVeCME3OpEXvvAZVfiMDHxmInypDb6UDl8a
Fs98rAA+dgM+9gu+Zwrfe/s/8iIevjodeZECXz0AHx6PvCj7Ky+O/JUXI5AX
3fB5gs//Ri4M1WW50I9c+IZ+C/LiE/r9yItkzBHFnOU4dz3OjcS5/8q1f+WF
DHDgAQ7NyItY4HYIuCkCt3/l8nrwMiWT8dIPXkZUMR6DwaM/eORCXsiC95nI
i1LoZOhXppPpyIuZ0FUAdLUNuupsZTosPcF0GIS8sIVun0O3qtCtCPLiF3R+
HTofo8v2YnYq24tzyIunaUVmg31B/QCLwb5gcB3vYL/2eK/AxIQw2n9+4fKJ
CZYk9nObuGmnD50L0GpYXMj5nRG1qHVxoQ1l50n6/BgeQHy9F9ddC8qn3jGF
G64F+dPbGFWDVwtSaI10oTz34RYqUty7n/twKl2c4/reRJTzXS5fdeJlRx3d
FOc997Ijjt7aNuoECReTjF8lj7R+Bn2VLD4ire9B9XfPFT4zjaOCG9ktCusj
iOfkMp4D69/QMoGsadpqfrRA8e7WT2UhNOJet25XmTkt/N7rIMj5/fetrmVI
W3IieZ9aMG5IiiNVr/Mbfj6Uk+N8BjM2rOD8Pt28TkB3RQAJa5qfmB2ZQg++
OC9xe1VH7lPHyem9jiPPoNZw/dAiOnFoofW5s62kdemKvIZ6GukGbdJIelBD
suc+vr7n2krcsjuuZpWk0a6IaD+x6hp6dcTlSIdaE30+ZvUy+1MyTQjMil1U
V07b9V8HNc1qIMvCbtMXs3Jo2cZ8K1m7cgpKqpO60NpC3mOWnRJSaSTRlpH7
Mxzfkw36pujPQH8n5jz6a44Vzu3AudNw7n/3nPDXPZXxrnN4lybepQUcrICD
BXDom89w0wRuM4FbA3C2Ac5pwPlfvLSDRyHwuBE8ngLvEzYx3r+Bd2vo5DV0
EgedzIeuoqArOehqJnT4HTo0gw6loFs76DYYuh2Azq9C50eh8yjsxVHsxQ7s
xegGPnXP4+9JydGZU5OpeqehrFJIBUU1NDt2udbT6BG2Tl2u8RSziHvCt0kl
tLRygmZpZRrVdVdql1a6UW3ZNrnsybG0XzjCpCEkhMj+jXNDiDk5FvFtsFbz
oh+KKZY7I6MpRDs7Y2ekLU0uj3LmNguggK0qTkPu5tGTsUczhtz1o/3S3SVx
b5No59z2gKjORjIpL/sR1ZlIPgqNHRe/lFFg/8Hnh9a3kGiTa/Dh9akU+yNh
Q8foahrG0/Bi5/pSUsnrn6q0PpRmJS6YVjea804DnY5b29NIKr16Uv92N/L6
1an9UymGpm4xE/VfG0MKF2JOneSkS5F6QtO2pgB6dEJ5mfGDYOL+lVucEGBG
o1Uz6yS/eJDIr/1Gm2SC6XiPlL5tvDltD+P/7PzDg3wrlsf5+URQVneDWPZ1
RxrryeM9bnUAHSnp/LqtOJ7mD6ub26sdR5KXzuvzLY4n3k/XzbxXZZLlc1Pl
a6I1tH+YwKTmpmLiRl8NfQn012GOy1A258tFNscW57rg3B4Pdu4s3HM97rkU
93yFd83Fu2bhXeLAQQs4tAKH28BNFbjlAzd+4HwfOG8DziXgZT94+QBeloBH
dfB4DzxagPeL4H0lePeGTi5BJ01lTCc3oKtT0FUudHULOlz1g+lweznT4Xvo
diV0+xW6ratnOheBzj2h833Yi69P2F5wYS9KsEc7sUcC2KNX2LsO7F0L9m4r
9nT+WLanGdjTzdjrauz1Dez1KfjAEfjAbfjAevhGE/KiHL4RCp/5foL5zGz4
zFr40hz4EsGXfsHHguBjzfCxecgLE/ie1P/Ii0vwVV34qiF8dRt8eCx8WPp/
5MV/Pp8Gn98In49CLoQgF9YiF+6jfxP93mbWl8YcNcyZhzn2f+XF3P9jXpwE
Dk+Bw0PgkI+82Abc6jUYbh+BszNwLgTOI8FL613Gy8A3xsuCm4zH48iLo+BR
BbyPQV50gHdH6MQaOsmATkZAV/bQlTB0lfab6dAJeSH633cLdGsG3fr8952j
wnRuCp0/hs7nprO9iEJePMZe+EqH/+k/OP/LbLCv32H9p9+vt3rYQvlQsrZO
5VQLajrd8kCI35uMbiZdPH09jl5oLrt0+rod+SnmpvS/DqLlVSWS1WKlNC9p
5aZqsVASlztlXDQkm76fMhBOetzG8aVPnJpOL/US16xuqaXXPL6bI4KLyb44
VyoiOJh+jziVL7Aok05aFsiPlYwnzf7RKmMl7am/1LR1t1EwPSo23WXiF0jT
rUJjTPxeU3qkVGSwjRvZ3ZwgJSWXQL07FA9tlXMgvmcnZ76uC6GQjmncGcMK
af+7luXZwwLJsOzFId3EVJK2XyhXGdxIPAtEA98HJ5Kh++dwRccyWnTyeeZD
ac7vhVnBNQXS6WTvtuHhWstayuHiWiliV09L2ne+yHOIpxkJZYoNo0vIoKP2
eLBZCYlruyvMTQ0hMznDq3dkssjAv32Yo2IevfBxPTtKmvP9Zltaxd+cRC58
hU5nErLo/q3kxNwJ8TRFbrf4nNYUmrKDR7KksYDUAj9aLAhooHblqyHj+ato
DPpb0E9G3wdzLmHOBMx5iXOf41wxnPsa9yTc0wH3rMW71uBdy/EuIeDQBxzc
gYMEcBvgY7hdA27WwHktcD4PnP/Fy1XwOBI8BoFHbfD+ALxPLWO8x0In2dDJ
qpFMJz+hqxvQlTl0xQcdToIO+aFDbej2BnRrAt3Ovct0Hg6djznDdP4eexGG
vXDHXkT0/iwbm95McpvHl49NT6Hs8ZIv5HSqyCM60f3wk2ryXPeDU6NpbAqf
R8/VAgpOP5iwKSORzK24EzdlONIszblXc6LCiJ/OxnloB9Ksq/qJHtqv6cOc
HVeK97lR++n0Pu26JBrTOXbYtTonurxw96ZXMyOoQmraywMJlVR/TdTiQEIk
Cfp13L52KJ/CDuk+fqLdRkIB9a+faKeTVMo1Ibf8WqrlXbSpyqWcjJ6kpFe5
hFO8ys6wHaPzyH/OqQEj5VT6eJOLXiu70sdw/twry6PJ4Na5SFEdzvfPjaKP
D3XeUHSmSOVoYz+6s1bSbs2iIJr2Q2ud32JT2tJt5eMo7E4XN7Y17a+JImGN
iaVnvtnQHC+lbPMJAaTpYTv/3JYkyvAXHSXh5EQNJeftr10IJ0ntcTcmb8ok
uct3N1fy+VKrq5kj/8N4ussvqPZhagFlbnXe7dmSQGopOmPjDDKJ28pQu3lO
Ld3grXy7aF0TDYi/vPSssZ7KLVlfCv1S9A9gjizm/Epmc/hx7nycW4xz5XFP
F9wzHPdUwrum413ceNdF4DACOCwDDp7A7QtwKwRuecB57C2G8+QIhrPwPMZL
OXj5Cl4SwKM4eNwDHh3Buwd4HwXe3aGT5K9MJ7Ohk9HQVZ8W01UKdNUDHZZD
h+rQ4W/othe6lYRuX0PnS6BzG+hcA3shhb1QwV50Ii9eYI+KkRdu2DtP7F0h
9k4FeyqPPb2KPe3EXmtir59jr8/BB67DB1KQF0fhGxfgG93ICyv4zDL4TCN8
JgS+NHkn86UN8KVY+NhR+Jg5fEwFvieAvHCH7/HBJ7/DJ23hkzHw1enw1eHw
VV34sBB8WP+vvPjPt5fBt/3h8z7w+RXw+enIhSvIhd/IhQFZ1p+FviX6rpgj
jzm9u9kcC5xriHPX4dy/c80U96zBuyTwLmG8SwQ4DJ3NcPABDuOAWxHyYjNw
8wLOO4CzHnB+DF7qkRf9howXOfBYa8l4fAwetcD7PfA+GXmRDp2UQieEvBiA
rnShq7fQlWsl06FlItOh8W6mw0vQrQZ0awDdbkJeVELn4sgLORm2F3wX2F58
wF7c0Aj+0+fqjvrTv6P06k9/hYZdXFJdMEnq+HOqGfVlZ3V0HfQkAcW0AmuR
RCoJKOJUR6q+p5sq9SuU4iQmj7STrSUBb15OjaWR+cLD1Yw5ev0y5vmhre/p
5qUZnJpMtzJefF1hWkHXO2W+NJ3JJIHuFk71pIEfdtP28sWT9brfn8qEQ6lM
z/prmbAFyc86KcjT40WaJ3KUaWkM/Vj+4RItfUs3rj+y0i8LoKFfq5+dWp5P
dWuXup9a7k/rYpdej7qZTC0uK7xWvGyirt67E0RfJhG/ywvfDUHlVFjRJXhO
tIXeam9cdFE0lX79SOp80FtFtZJyfdHuZZQmPjv2m3sYTcqc0r6+KIeyaisc
lnhnUnzoe6mr/p5UsyQjPH53PPkscJYx00wg+akb5qnaO1B8s9je9pmh5KsR
q35fLYrSW0JKJPXtKP2KsfBExQDq3z1iv8COSHohu6vd0CWIUjKW2pX/CKVF
Hi8/N5Qm0U7n69ZHT5TQcl2VMa5n8mg6+svQn41+H+ZoYE4I5oTi3CScm4Vz
g3FPZdwzA/dswrvK8a52vKsHONQDB1HgUAvcPIDb2B6GWxlwbgbOk4Dzv3jZ
Dh6zwONR8BgM3jvBuyZ4d4FODkInvD1MJ/egKxvoygW68oUOZ0CHv/KYDsdA
t6HQbQh0uws6PwSd8+UwnQdgL7ZgL+yxF1cPZth3zGilrltZnJpGHdclMz7x
1lBb3+ba/kVltHm5DKeG0br0yRb9wjn0OcuNEg1j6ZlLCae+I9kNuaMiQgLp
RYXxh6rqMLqc+JNTLenCrt2LFaf5UpuWxZq6+lyyvr5mbV29L8m4FSwUbk+k
5yfk7rz/0EyrL9RzagrNjnpc+da5iixODXkn+q6OBKod7UXfxVH94fOpvSlF
9K1wvv1W43TyPRVQudXYnaovjbc07YillWdUxMzNwsl1eoOklZkVVX4sfdD7
y5c6fVf56DeGkIze2ZvvGs3p4YvdRho2XuRpfG7+1eoE0tMZ5lhT7UDPhAXT
PpiG0uzABsX5z3OJ51n794uvfOnLWB7BY+GJJDFGp+LI5XIyGJayPf5JOJU9
/bRsfF4uGaYdkkqWbyB9072GI5oTyNO7XU16bCkJ9fFVnu1ppp4ZZ2QMGrJJ
4OGvT2Ou1NC36guG7z0+0tOBS/fbsuso5vPHx2c3NtMH9K+g743+FMx5hTk9
D9icmzhXC+ea4dxhuOd+3NMC92wPYO9qMGTvKsS7XgCHk8DhLHCoBG6iwO0M
cNsPnNOB88hPDGeJIsZLJ3hZqc54iQGPe8Hj5COMx8vgfRp474xkvGtBJ/zQ
SZ8r08kJ6GozdLUSulLOZjr8Dh0aQoei/Uy3+tDtK+hWATovgs5ToHNx7EVi
F9sLBewFP/ZoFfboC/JiD/auF3s3/j7bu7Fr2J5qYU/FkBeK2GtN7LUG9noT
fKCji/lAFfLiNXwjD74hDd94CZ+ZIcJ85hl8ZgF8acg65ksn4Es98DGuPuZj
6+BjsfA9Q/hePfKiED4ZBZ8cBp8Mgq+6w1dz4asu8OEd8OEw+LDPP/Jighzz
+QL4/Bf4vAByQRG5II5c4EJ/JPoj0P+GvJDGHBvMScO50Ti3EOf+K9f+lRfz
NjEcRq1mOKgAh9GVDLdC4CaJvKgDzp+RF3OAc20H4yUGecEDXkaCRyPkhTh4
9AbvH8D7WfAeCp2cR14IIC/coKsk6KoMuhqGvLjgxXT4HHnRt5/p1hO6dUVe
qEPnOtD5LuQFvybbi0DkRSv24sLvoD/9M8HBf/onxE3+9MPuSss0XwsiPUFZ
TjWlGYYzy1Veu9NkhUuysY0p9OOlOqe6kHyI/mxfmyh6dtOvzK2jibZM9efU
JLKOruIuFK6gd8Vv1Hqzqqi6zZZTo2icQH0M75ACOjR8xZRTUfF0Y9xWTrUn
iZSBUp1lITR3k7q2mHIIcX+dfE1M2ZxUfvnvG9LuSV89sozmGmTQwrTbz+ca
eNAdn9Vlk8ziqPHJabc8nwYSqZ7hmeeTQCJ6ywofHS6lvSZjgl2PN9PvoukZ
rsdTyFj6heu+7kqaqZGfYeDD+e47unC/oU8giXC9UVn2Jo0WLqT3a0sTaZtX
5+7DpY4klq1zkyc7jObtX2/BlRRGU+/tmyybakkLA3Osy7p8SOrmFek1JYG0
KTjgk9NwUzJZ/PY+X6kbZZ++/CHIO5wmJjoeu6tiQ7KeH1UzTvqRVF+g6H7L
OJr5sKzFzyeAcjbZOXl+CaeFd46WTlnK+V1vJ/I71qqSRDQuf3MyL6JJ6E9B
nxv9dZjT84DNCcKcBJz7M4GduxnnbsQ91+KehrinBN4ljHdtxrvEgYMScNgB
HMSB203gtg+4HQPOY4oZzu+A8794yQWPQ8DjWfA4Crz/7GC8bwfv2tDJK+jk
IHRSD13N/8B0JQ5d3YUOV0OHRtDh+31MtxnQ7QzothQ6t4XOJaFzT+yFK/bC
EHvx8KPquzdz2iiK7wSnplOA2DqZ+ydr6dXHaV7HtQppxLPpnBpI+5/PUe7e
k0Zin2d82rkqnKoy53KqFQktmO9aredL22oLn41LjqOEURWcakerT2Rc3tMd
RAm1Kx22R1ZQfuMGTo2gI5dUH48LzqPAEqtlYv4tNMnYh1NT6cetx913bavJ
VtLhhJZeHknsvnVSS8+PPF5u/sX9NoksS6ljpG0kfX1aMJLL1prGSlaELZLy
p8vJ/RmR0aGU03DualS0BfH1l7UIeXuTd6rCgTzXFFrt8P10oasLrevIFUne
F0XLJESOJvGV0obfoyYW84XST3FRu5+tWfSar63r19H3tIpr9+JTx5Jp/sKN
sfuDKqhi5Cer8HNtdHt50J6zl9Np7SQr9f1JHHzUzO7O8GyiDRN7s+WHJpPd
uamBFz+WU5+3p/ut5Do61HFhX3R3Mk25wNUvsLWE2hVqVJsE35Mo76GyFrV6
4g6znntGppHS0B+CflMo63P7sDlLMGc65jjg3B04NwjnNuKeD3HPbbjnQ7xr
Id41Du8aAxxmAYdc4KAP3HiA21jgpgScQ4DzcOBcAF6WGTJeRMBLJnhUA49N
4DELvAuB9+m3Ge87oZND0MlE6IQPunoLXY2Crh5Ch8JZTIfG0GEOdLsZun0L
3apD5y7QuQV0vhB7MQ97IY698MUeXcMejccebUNeCJqwvXuOvQvCnl7AnuZg
Tw2x12nIi/6lbK/XwQeU4QPc8IFF8I1pyItz8I35nsxn5OEzVvCZEfrMl/bC
l5ThS5vhY5+QFzrwsV+Xme9tgu+N+R95wQNfHQlf5YGvysKHt8OHrf7KC56/
8kIWPr8HefEZPi+EXDiFXNiIXPitw/p571i/+TLri2NOIfLiLeYE4dwy5MXq
/3LqH7n2r7zYCRwuAYfTwEEdeeEP3CyB279yOR55MR68jAAvV5AXDqmMR2Hw
yA3eh4D3PeBdDzqxgU7UoJMVJUxX6siL08iL2dBh3hSmQ27osBh5EYG8GA7d
9kDnMdD5Weh8SS/bi4UhbC9KsRdDOlg/pT/oT79sBuu7/9744PykQKrlleTU
17Rga1ZDYrErPV3Ybu01K4smSA5WLw642vXjXsaT+OVPezOojdp0Bms6iRi3
NEa9qKWA4W43XwoWUPKewRpAp2YUmH8PTKGne3fqHk4IpTX6ezjVgpS+2s79
GuBN0fs0CtXy4+mKwC1Otacya7nRMdtDSClib8+C+kpSmKXOqZF04N7qDcGX
8unenVddMQMtHJ8Z0h0zkEoHgicH+gxUU+3VE6oC9wuoJeT0OYH7AWRZf/u2
1oJU+iQSv972QhzNeCL/xfaCHWnJm4VsexxEo3vkZ1v2B5BwUf1nj/5XNEV9
TIlUjCu5j5j/+OXRaNoYfvxl4DFbunPIOqj/UABNuqQTbpmfQtPnpEx/XOlC
JZ3CA7cMosirYTz/z+I84mnS+bVmtT/5VnoulFmUTCvv+03xHFpKx7rWCJTe
jKGnejHVde9yaOT06Lf35jdSferQUaEudfTtcm3BnMZ6+jCN9SPQz0d/FuaI
Yo4a5jji3BE41xHnjsI9x+OeGbjnc7xrAd51Ee+aAhzWAAc+4DAA3ISA21Pg
9hM4Dw1lOIcD53/xMhc8zgSPK8GjPXg/AN5DwLsndHIIOrkPnVRDVx3QlRV0
JQodvocOBaHDndBtzkam2xjoNgk6/wWdb4XOt2Evxg6wvRg+k+3FsvDO2jzz
NtKKHqzppH/uy4zHvbWk7LQs8QJPLlXGDFZf4tr5O9zhfCKZnl3PO2plEK0r
Hqym1DD3aZOspDvJKPd+XbKbk2Ptg9WNHF7p+IiqxFDEnazHiw43k9/wbE5N
IZdoOuvVUUnhdj8mRZlU0BSjXk6NoJoTG55Ns84jpzM9XlEi8WTLL+YdJWJP
mSu2l128HUwD/oeeHu4LprTWLKPDfWa08pORsP8dTyrOLomQbk6l9/trM6Sb
XSnb4bl6WW00bclXjq1zqKaSO+9m1TtE04ay2gdZjwtI5/yKVAXLNhryOH7B
Act0ish4x3Ouv5bOrsy5MWV9Pf1cKlG+Zn08nY6cc9neoJgmzlMYOPO4kF5f
+7H8mGEgaf/O3dl/No3yu3Oqt/BmkOVPL8PYSg+yni1ZN5OjJ66+zyk7tydT
Q2f/jfxZAXT9+ucHYgoxtD0r1eqBWBZp9ldPmretknZ0qvp+8SkiAfSnoD8F
/ZmYcw9zDmJOC851x7luOHcN7hmOezrjnnfxrkkC7F0P8K5bwKH/EcMhDDiM
Bm62wG0GcPMBzjHA2QY454EXK/DS/ZHxkgkek8BjL3gcY894PwXeJ55kvF+F
Ts5DJ3ugkxHQlRB0tQe6KoIOb0GHq3iZDqOg282xTLfboVsB6FwDOn8Inft9
YXtxHXnhgrx4iz0qwh5NwR7lY+8OIC8mr2N7J409/Y093YY9dcJeh2KvFbDX
J+EDE+EDi+ED6fANXfhGK3zjOXzGED5zEz5zHr4kBV9aC1/Kho8VIy8ewMcq
4Xtc8L2j8L3h8Mml8Mnx8Ml4+KoifPUZfHUOfJgPPtz4P/JCAj5vDJ+3gs9z
Iy86kAsDyIVM5MUz9D3Rn4E5UzDnKOaY4tzORnbuS5z7r1x7hnfNw7vO412L
gIMMcFgNHARXMNwUgVsMcGsFzh3A2RE424KX6+DlFnjxCWc8esxkPH66y3h0
A+9HwXsMeA+FTk5DJy+hkwkjmK749jJdFUNXY6HDWOjwlxHT4UroNhh54YO8
aIHOeecxnV+GzjORFybIC2XkxWTsy8gO1jecxvrLTb7r6E0LorOvB6spcYV8
nV0z253OZN/PKyvMp905g9Wffm52GJErm0KUKLjoYWsLbUgarKkkJ/79o2ND
NaX3HbZcyp1KT8cc4VRXEoq7ouefH0X3dtTNnpAeTL6ug9WMLl21PCG/wZMk
FUcd4bHLo4v5g9WPLGXTV05JSyKrQ12THt1oo7Pjf3BqOgXMvTr9dHEtbXNN
clYZXkz83xs4NZjk1CRfrNmWQd1zEi6vWhVLhb8na69a9Y6uqTc+VrkbSPVJ
+3QteoJJzKwwxaLHjJZ7cGlEX/ekpL0+P1sbkqiysuzoxwYninzSlOc4J4JK
dKp5rOuLyHwpz4GA+iCK2L7AedTsDJo04rTgLIFGOpbbNWSuYCK13nAQVO0s
pVXSo++sUWijqI29PYsuptOJAzK3h3jXUonAWtvN/q2kafHMX0o0kwK4rXfs
062l2/YXnGT3fKCxFbOJ17mGaFSv/cW8JlJHv6Sc9YXRr8WcK5gTgTn/nRv5
17lDcU853LMI9/TGu9TxLn28KxA4ZAAHF+CQDdzmA7epwG3JXIbz8F6G82vg
/C9eXoFHVfDoAR5ngPft4P0ceFeBTp5DJ1LQSR90FQ1dXYSudkOHstDhMehQ
ArqdC91GQ7djofNt0HllMNP5MeyFPPYiHHsh6Zt4Mm13Gz36U9OJ11pb59e7
WjKwWrxJKyiNFKwHqxsdHeKb/bw+hr7d2JR9bGYIFd0drObUOuP55mtvPel7
h+3NXNlCKuoerIHkdjlZaficNFp5ccc5v2Wt9OP6YE2jL2lHHthI1NAeqYV2
Hw5wvhM2LuJUD4oymOYUcC2O7l462LjbNZC6hp3h1Ne0sX3lXD4TN6o+1Slh
cCWZbPtE1hhccaZti2K2fciOoIdfPc3y4zi6ExxikR8XS1/7gvselBXS/O+1
s3KGt5KQyMRFOcPTyGPA0SF+RA29/7Dkl1VAMXWIRi2zDggmvh/TkjfPz6QZ
e2rWSD1PJivnLQmaz51JN/TyuZqvEeR885lM8bRIGlc4acQnXmsK6pbhd/vs
R3cbTxRcnB1EmRZDI3LrTenKjoa4WgV3GlsyZFeXB8cXF+d0P9jnTFrfxU5V
uvvR+nS1uTFKSfRyoZZf3ugiWq7vFLNCPIfmob8H/eonrF9VzOaMwJwVmPMM
5+bjXD2cm4F7rsA9M3HPjXhXDN7liXfxfGQ4zF7JcFAEDmuAmxRwSwNu24Hz
RuCcAZwtwctB8NK+kPHiBh7nDWc8aoBHE/D+Cbw3g3cN6GT1DaaTjelMJ77Q
1QvoSgG6coIOH0OHj6HDUujWHrqNhW73QeeW0LkEdB79me0F+87ypgPYi3XY
o6vYo2nICxvs3QPs3cYtbO/mYE9nYE/5sKfB2Otr2OvJ2GsH+EA+fMAcPrAd
vnEVvuEA33CAz2jCZ2LgM7PhS7++MV+aCV9qRF7EIS/Ow8d64Xt74Ht74XuZ
8MkG+GQifPIDfNUDvlr0f8yLUvj8Pfh8KHxeD7kwF3khjVxQRd8GeTEd/RrM
OfvXnKU41xfnKuDcf93zX3lhAhzcgIMOcLABbp9NGW6l7gy3EciLeuCsB5zv
g5cTyAsr8HIfPO4Fj6bgcR543w3er4B3BejkPnSyCjpZ08909QO6CoKu3kKH
b6DDJOhwKXQ7Abr1QF40v2Q654POvZEXX5AXDciLqdPZXgxFfwL25ST25e34
I1Mf1gRT1J9qRlm5V8oXKXnSwqFh2/ZolJLInxpK6kObixfsyqayfYkGSjfq
qeVPjSeFJd+FhIuL6bxY+eltYjHULD5Y39K+ONfYsXUBdKe1JWKcVDyVfBys
9nSl0d79+atgiuG6u+3R3AaqHTdYEyg4/e0BZ8cS0nwtuMDiVzVdbBms0eTM
FaH9NqqAbtXp+0TsjCf/j8851Z5OWy87fN42mH4tzHswYWIouW179XDCRAta
FBTUOLGI8/vp0sjTeckZtGLA5GJesgd96fpODpFxFDOz6FHb5npSPKbi2rY5
nqZ0eH5eZFJMyrLfDZ/MbaNnZd2fns5Np1v9K7ncT9XSMLGpQzRCakij5VDM
qtAY0lAqLuVdVUgV/kOu9+kV0DcN5ROxFgFUaqy0p50/lXy/KYR4Hsikm+OE
LLnG+FNwr9j9Ge3xlL3zoIWmUx4tK2879epRBfVOste9b1pMEehnlLF+Avo+
mHMJc/wx579zO/8691/3VMO7LPEuQ7zLBzhsAw4DXxgOXy4y3MYDt3LgVgWc
DYFzfyDD+V+8uINHJ/CYDh5NwXsweL8P3o9AJ8HQiSx04g1dCa1mutKBrpqg
w2/Q4QnosHoI023bEKbb6dDtVejcBDq3gM4PYC9OYC/MsRdHBTV/Fs1qJdU/
NY2CxLLW7OSroR8+ZQXXHRKJx3ewOlJExXP9qc/CaGfhmYadPNEU86fa0txt
97Kfc34/HXrcv1LWvI6e/qlxFLlEb1NFZBGtbn7L/Ta0kupbBmskTZdflJ6g
kE9qWSpFPtJRpN41WG2orily23Ivf8p7r76ySTqOko0vc6odbXmaLml7MIge
tNd/meVcSfOuvufUSFrs3L5aVCafLtz/0abE1UJ2yTwflLhSKV9Lx+lTahWt
PO3szqWUR7E+H724lPyoXr5O0eBiEh00S3RP2x9NoyN4Y9L329Kn+85GxxQD
yCuwZNg27SCSW/1xsoq2KQ3cM0k9YuJOT+6X/vz8M5bW7P/c2df7joSe7dhg
0RtIQsHlJz+1p9LRlfrrNOe70f37G66kdUVTTkj5r5Y7efRMtiJ0ZmUgfZXN
E/C5mkzTYwfiSiZXknbI/s/L+KopOcJd+f6sKqqLYf2WYNZXQ98bc0QwJwxz
ZuLcHTj3PM49iHuOwT0n4J52eNcWvKsD77oPHFYCB64HDIeDwK0ZuE3ax3B7
BpzjgfNv4DwNvPhrMV7qnRgvLuDRGjyOB4824N0ZvA9/z3jPhk5utTKdXINO
ZkNXO6GrS9BVVgHT4WboMHsr0+FqX6ZbOeh2VCXTrRF0/hI6/wKdO+P7yv8z
2wsB7IUH9igNe1SFPbqCvNBFXuRh7+ywp77Y03HY0yPY6zLkhSz22gE+0Asf
sIcPpMA3WuAbsfCNJfCZBfCZG/CZQ/AlE/iSJHyJaxHzsSD42CrkhRTyQgW+
N6Gb+V4OfFINPrkUeSEPX30EX70CX32/ivmwAnz42P/IiwD4/Ev4fDR8vgi5
sBd5MXEyywUn9A8iL/SQF16YcxRzvDCnHeeO0GTntv8f8+I5cOAHDqXIi7fI
i8h+hpsL8uID8sISOE8EznbgpQy8mIMXSVPG49pWxuPiUYxHG/AeDd6fgffd
0IkLdLIaOumBrq4hLzygq1MKTIdaCkyHA9BhMvIiH3nRM4TpVho6V4POj0Pn
HciLXuTFD+zFAPZlCPoq6F+96q14dEoYaf+plrQ3r+y82EwferVg+9TZnO/c
139qLP0W99wtmlVIP5ZG31M7XUI9f2oIfb/yyS66P5PyykUWD70dRI1/qinJ
vrx9+b6lO1l4qe7qaM6ltD/Vl5ouzl2h1JFILclXF8icaKVNKYM1ja6H5rVZ
3qqhVfuaeLvvppLB8cHqSoXnjjl/U4mmZ+J5muuGBJPzq3xONSMxqxHZMps9
SNt0G19lXRYtOSjPqV606YDwd4PxCfSQuzt8o1YzPQ6SiNiolUJOkQ9bM7mr
qDQ6WZlndz0tW+Z5nGd3PNkbaK6Y8q6YeBzs7j4pyCLlLdXjDAu8aNXPXfdN
hydQ3FbZZ3PzYylkV43W8oJ3pPxx4FthQyBt+nEkPPxLMPnczaHx88xJ0GJv
+sYbnnRbSf7ig5Qg8t8a+erKels6s3Qgd940L7pul3j2WXICHRF2Oaa2Jp/M
e/UnCb3JoGPo86OvgP4lzHmAObsx579zvf8691/33Ip3PcO7FPCuFuCwATgE
A4fTwO0scLsL3DYD505lhvM84PwvXtTAYwR4nHSe8Tg0hfF+ELybg3c56EQP
OgmBTnqgq3EVTFe3oasJAkyH3AJMhxO1mA6XQ7ci0G0UdLsFOpeCzidB58rY
l4N/fV8pNuqt9hnbjJpCAjY1CYeMKylLarm9yeZYyvlT31HsjqgRRwwC6dsj
gehrR5Ko+091IlEXLeH9WuGU2LaybcesNkr5U9NJXE3vlI1KLfGIByVEncyk
zX+qJ72w8/sZzBtPhw9GL3J6EkxTjwxWM5rn6yUs1+pByQZDo9ZXF5Dmi8Ea
QPafhl58appKlX3iXTFJrfRWcDWnptEW2QcBn7tqaBuPmYyZeA51F9hxqg8N
cYjZXV3H+Z4dVjLF2z2Unq2qnObtbkH8ubsPzTT1JtObhsku26NpRWNwr8t2
W/LrOSJlsCuAzs+I0MrbmkP+Id39hVt9yLBhQ/jJDwn0bacI/82PNdSRNeV4
1McYmvGu3FZsfyE9/dEarcjdShayodzzp6aRb7r5/aNTamim0S/BrXvbKL2t
hTv+WAYZPglSD2yupYs1V2veb/xA9q4ttYv7ON+J5WJno3ib6Aj6CugPQ38u
5oT/NUcP5xrhXHucG4Z7BuOenW/ZPcXwLh286xzedRE4cAEHc+DwDri5A7ed
wM0QOG8sZDgLA+fh/YyXFPByFbzogkcB8CgNHleB90+HGe+ffBjvntBJFXTy
FTpxhq7coCse6OoRdKgPHbo4Mx0u38p0u2Ir020vdGsJnVtC57rQ+RvkhTW+
r/iwF0+wR/rYo7PYo5EL2d5xLWR7t3U12zs35IU78uIV8iIJe12KvFiHvR6A
D4h6Mx9YfIn5QDXyQhy+cR6+8UOe+YwcfOYp8sITvpQMXzoKX7KBj+1BXijD
x+zhe47wvXT4Xip8cg580hg++d2e+epG+OoC+KoHfNgOPrzjf+SFHnz+G3xe
Bz6vh1zQQC74IBcI/YZlrD8M/fOYcxBzCHMO4dxUnLvr/5gXo2IYDkeAQw1w
EAJuwsBtNXA7C5xnAWcJ4KwCXm6Bl15Lxksy8mKOCuPxGfKiG7zLgXcD8C4K
nZxAXjghLxZVMF1JIC/8oKt05EUG8uI38mI88mIC8sIVeTEaOh8DnWfkMp0P
w98PDsefXw3Fn1/1YV/6sC/HsS8R44Q2cZVHoL4hRZmKZsdUP+Krf5ujuPI9
ajKds5Ie5qFbQTbfZxhoLc1C9aL6Hm4zV8t4unNV8eDY+HDS/VOtaHma+A2X
mX6U+uj5AeOD9ZT2p8ZTz+JODz+fYqpWurb6dmwxNf+pwWTuVZBwcEUmzc4M
4F3iGUz5WYPVjPbozXf+Ms2Tyt2sM+xGplNhyGB1J2Nzqb7RMrEUe/XkuvVh
LbQ2brCmUvaiyFRZl2o6ZETvT78oo3wreU4No+yA18cpMIcs+1fz6hnH0+SM
kQv1jO1p7pVQ5+qOYDpbHFp3VCqQ8obdMDwu9Zpmmx94+H2yG639vERATCSO
GhfkBBwXsSP/6xVFGeuCKEakwLZOLIMy6kcNHb/JgyynjM31PxFH5zsFfuxa
UEgb3xhXv7ocRD8OGvl0bEyjWq9LimJSNRRwwkRinEYZyY66u/3dwUoKRl8e
/WHoK2HOIsypwZx/nSuMe5bing64pzzeFY539ZuxdzkDh/nAQRg4PAVuQ98w
3MYEMtxcgPN84OwDnP/Fyxbw2AMe9cDjPGXG+1JlxnsLeDeGTp5DJ37QCUFX
m6GrUdBVFHQYBR12QYetdUy3rCbTQuj2GnR+DToXhM5V8H2lgu8ra+zF+5Fq
gb+PN6Im0vFRknYREmUkKOm2f21HGKolXd396py0sC/ljh5ZkTyQg+pD9xdv
datUSaSVSwsr5M40oSZRX4f+vvq75fRi9+Fh9aujyfRPtaUKjZv2XlsC6OyI
GcuKypLo9p/qRI+Tq1aP4YmgXZ4nz/Jva6X7f2oaCSltuL/8QA35iJh0RL7O
J8/Vg9WfZt0J/uTRk0z9H+2XKewNoYS9TpxqTp8HDEOWNHrS9xTPRt52zvvq
sznVgdTp3dHTbqEkWv9FpGpWJbWnKK+smhVJ19ocbAU78kipWsNF+mUbDU90
rZR+mU7Tcw4dO/i9lhYOaG85tLaGeizTuDTWxpDfq9f31zcW0JOlp8PGhuXR
+KFDLvBF+1H60vcFCi1JxH3Y95WQDmc/3W+qhE3zoHuPjD0sh8TQI89hMYcl
M4nXSCJcULqIOjrmGF3szKMD6Ns9Y31n9FdhThTm2GGOKc6djXMrce4d3FPB
it2zFfc8gneNxrvm4l0/6xgOIcBhP3AoAW6pwE0GuAl9Yjh/Bs6Lhzz7g/Ow
FYyXAfDyALzwgUcl8Pj9AONxOXjfDN43gvdpckwns+WYTkQ0mU5uQ1e3oauD
X5muekYxHbLqQ7/5mQ5vQrc3odti6FaCi+mc1URKhM5fIi9eIi9mYy9ysUe5
2KMT2CNV5IUq8iIMeyeJPZXEnrpgT42w18bY6zXY6wH4wJDHzAc2LWE+UIC8
qEReGMA3ejKYz/jCZxbBZ2a5M1+aEsp8qRq+VAYf2wcf+wQfWwrfe4e8eI68
MIRPjoBPjoNP6sFXW+Cra5AX2+DDX+DDsf8jLy7A52Xg87/h85+RC8XIhZPI
hYfo16uyfgDX/zcveDGnGnNe4FxrnKuDc/+Va//Ki2zgIAMcjgOHT8BNDXnx
GHnxr1w2Ql48QV4sBy+GyIst2YzHCvA4GnkxCXmRCd51oRM95IUN8qJXi+mq
T4vpqjqV6cq2i+mQVS968JPpsAh5UYS8GA3droDOV0DnfdJM5zPwfTUD31fc
+L7qxb70Yl+OYV/4+Hr4E0/HoL6lqVPn/LAaCKBDV5t0Uw+1oabTipfHjq3z
q6Wpo7ich/EnoDpQuYHMnuHeIZRUcOGjs0kyqjNd099+hqc7gnomNxTcK2tF
TaMQizej7LlrSf3xyMTFCfF0+U+1p7TmFDHfVSHEP3dDWtLpeFrxp9pzfk/v
+LWag6vhZZUoEn5PSX9qMq3fecNz740K+t11KaluRQnlDFPn1BDykTDoKovM
pFPDPXz3ZEZQ7Bt/Tn1DY1ZnLp8U60fHE7nOjFgTTWt6b50dscaWJOw7Y09K
BdDs9KoWiTv5tLL9W5/EHX86k3TIdVtlMs3oHHFAd1wTSdaM1703LomcBWW1
j3GXk0Z3yclbK9tI9vPjkY9WpdMQrbVqs/VqKVlG0m+3eiOt1bKV1cpJpP1O
alJtqmX0c3iqk7NII1XpuF8MmF9KzfHWFXoxNdSA/hP0ndEvwRwZzDmLOf86
lwv3FMY9X+KeP9PYuybiXZJ4lwRw4AEOvMDBCrh1AbdlwG1TN8N50nCG89A1
DGcj8JICXjb+x8scxiMXeOxtYDyag3cL8P4bvM+YwnTCahpVQSf60JU+dCUA
XS2GDhdDh5+hQ2noVhq6nQHd5s1nOmf1LTVMYTpXw/eVGr6v3mIvZjSEFYhe
qUONI5vZZ0fpGhfRV33BwuIfgaivqb3tm0xmlxu9avr5RuJbOWo4VYpU8Q1b
m0d+Rj907KuLUIOoSqOrYsy0DFrlPbJibGMQqinpzdR/mT3Hg14FjQ4q8q1A
jaBKzccdTd55pDpFVWT8pUo686dGEu/4Xhv9RflkLmiScKYrjNSWDVZLuqy9
+mrcKl+SqTIfN/RhErV3DVYnSrC6syfeMZyCJCavNrvYSO8Ep3BqIs0Zqvzg
tnQZye9dN8NFuZ4uhpzl1Hj6uHGK7QvvYnow3yB/Pn86+Z0N6ZvP707q4V15
S9VjafmnmU+F+CJIRe5g+la+N5RwNzSaV9GPyiRHBC3eE0S2vTrpt5VMaYjk
qTNJp9xpRK7Eh10jIilq4VrvmxPeUvnFhLHFW/3JUrK98lp2MkXbJypyS+WR
weYIqXSBTLqM/gH0x6OfnsPmSGKOF+Zk4VwTnPt1Izv3JO5pins24p6JeNdX
vCsG77oPHGyAg4AkcABu6sCtagjDbTpwjgLOj4BzC3gJBi+F4KUKPDaAxwvg
UQC8C4B3K/Be5MV0wqopjYZOZhozXbEaRMc0ma4SGpkOWQ2nQ9ChiAHTLauv
ad0Hptvz0Pl56Hz4HKZzfeSFPvJiKvZiM/JiM/JCAHmhgb3TwN7txN5Vc7E9
ZdWBTLGnJdjrEuz1M+x1GfKiDHlhDB84C984B9+Ig2/sQl4owWdUkReR8KUu
+NIN+FIT8iIUeWGCvNgE33sD3+sUZ753FT65Gz65FT65AXmhAF99AV9dCh+W
gw+H///Miw3weUX4/NARLBd+IRf6kAvpyAsB9I/9lRfbMecM5ujgXEWcy4Nz
/5Vr/8oLbuDQ8pvhMAI41AM3SWuG20HkxUTkRSNwTgPOu8DLY/DycQfjxQd5
ET6H8WiDvHgB3l+C907wzoe84ENetEAna6GrtdBV8hOmKy/khRfywuIp0+Em
6HYTdDsBulVHXqgjL5yQF8L4vhLG99UcfF/9wr78wr4c/e/vBw9MFw55HI9q
T7kf5oQuagmm01vnKi13akFNpU2Kjp5rjatpt3OJvszqMFRLercn67yVlA9N
/bBHcpVuMWowWTk5ZOlVZVD0XM0jQa0lqCGc3wu7teodssjCkw6pGwWhmtId
8xe+Br7uJBsb1FgwrBI1kox/yOck1ueR1FW3hSk5FST7p0bQWFn34g3xeVRl
Ms95oD6MDpsNVktauebM87uzfcm6JXjgY0kivdgSwqmO1H/DccAlK4ykdXal
NerVk5HWPk6Np7ZSlQibmmIiyZuZD75y9sLsdvaDr0lk0lCi17i8gmzb+RqH
iOVTUOzDp8PE/Mmw+v7j+LvJxHXK9M3X0ATiDj/KrR/uQDtMlh3ffyaUaMuG
9quenNy59Sqx4JgNZX7h2atd78f5jljQ5RQfR/yPnt6pNk6nF8HFa+OXJlM1
+i4PWX8L+vsw5xzmFGPOdJw7D+cew7k+uGcy7vkO91TBu9TxrnC8awlwuAQc
4oGDDnDTAG4lwG38K4azIXBWB87/4qUnhvHIaiQdAY9XwPsV8C4H3sfzMp2w
GkJ7+ZlOYtuYrlgNJhHoShU6VIUOw6BDS+jWErrVhW4loXNJ6NwIOr+E76tL
+L5y/C9HYlYU5QhWokZSa4nttgW/8qhvlIdS5OkQVHPqfaooLDTgSbxHF/2U
a3uPmkz8Ypdu2J+qpAffN7jsmJCM6kwOXzN4bOQjaPi+FqWcvkRUR9q62P7T
lr4warlUvdyPuw01nfjSFNcdVailNMlG80zO7y1W7Wn9BJs7/f3BJLiuMMMm
IJ5W/6n2dH2UQd/W+SE0rk75yw56Tzp/ajIJli2/OPdFBQVorKsQXVBKXjaD
NZSqSlIn6rZl0Yf1O3Y3joymqV03OdWWZkQF8vgsCqAXctn+F1eGU++EuviL
K62Ie992hxRdX+K/MlHn3eN02jetJNXhsTvd6h65fHRLLD0J4r+W3lZGhrWj
V5h+CKNzn5rvXebPJZMMB+Hrro1k6a8+KqE6kX5I6Pk0vy2jiO/KfGPl2yjw
btaKWKVS2jl7flWQbgO5o38I/WXoq2COKuaEYI4CzlXCuQI4t0mT3XMS7imA
e17Guyrxrvfy7F1LNzAc9gEHSeDAo8lwG2PLcJMvZbhV1jKcdwHnz6UM5+a1
jJfOtYyXhf/xsonxyKo9GYPH0eqMd1Y53yfg3Uie6YRVR/LhZzqpha5qoavp
nUxX5keYDllNJp1VTIf+0K0/dGsL3VZB51XQ+Y5SpvPbyIvbyIvx2At97JE+
9qgVeySIvRPE3n3bz/aOC3vKhT2VwZ4KIi8EkReB2Gtl5IUy8uIS8sIRvuEI
33gG3whGXgQjL/LhMx1azJe6tZgvlcowX8pCXuyEj82Dj6XD96Lhe2tvMt9T
gk/awCf74JO88NVl8NWz8FVj+LATfPj6/8iLzfD5K/D5HPj8iBaWC/uQFx7I
hQjkxSTkRXMQ68tjzjHMKcQcfpwrjHM1cO6/cu043nUJ7wrDu0YAh53AwQI4
LAduC4DbNeTFJ+B8AThvB87eyItA5IUc8mIj8mIj8uJ9N+PxJni/Cd4PgHdR
5IUo8kIHeaGKvFBFXoQ5Ml2NcWE6ZNWSnu1lOnSHbt2h29fIi35FpnNW7UkS
Ov/v35tvw/eVEL6verAvPdiXI9gX/5sWfK+dklCdqFopujU0O5xeXB0Tdo6n
HpXz/ZU3rllqfzFFTfv58/HZYFQzEjpy4+bZeA/Srcx4vi+tGTWFjr7S7Tx9
u4rCjXx5O/QTUB2oRWPO4k+CoeTVknl+9qY0VDeyqN80wl4uhtYkPzg/0f89
ajI9ut74UJG3kipexz9e8iKKKv9UGzrXaznlZrk/bYkLUterT6ZDf6ozKXUs
GFNzKJI+vV0+XVe6leztBmsaTRkVfmTuwRpSLTfM/rW8iPoWGXFqEKk0tN+7
tY7zvRPZ5Kp0M4qm66h7KN20IaEJmwUlMv3plrnAnTeaoSTSFc5lo2lBSVNK
deN3e1PNC4d4yy1JtNUguGaJlBN9/rr31kKFcPJ66DL1p2UufeKTNpLh4nzH
Z6ipc3N+T/S6DrvN86OUWvfNWKVak0dfe8eXhF4ooir0jdAPQv+/OR/+mpOD
c8VwbgXOvYh78uGeAbhnFt61H++Sx7v0gQM/P8PhJXDIBm4PgNsnLoYbL3Be
DZwFgfO/eDkDHs+Ax1jwqAbe1cC7JHjvhE46oRMRTaaTjdDVRuhqHHTlAx36
QIezoMMy6LYMus2Bbk9D56ehcz3oXBvfV9r4vnL7798PmuXV7h1eghpCRhFJ
pYseZdKLc/PCxF9Ho9rSoaFTvg25E0BRuXwVV261oqaRx3B3+RyLGpp/yOFW
g3woqgWd+TJ25pzF3iRWlJe4V7IcNZxoTHKQo1EuHYndven86gLUAFr5pdY4
MSGFLL8unK56OQzVkvq9Pef0PPWh2C/xIlW19ajx5HNRa32hcgkdF+M/oPKg
iC78qUG0drO60A+HdKrf8KRjk1swPd4xWM1IKsfVa89kT5LSLJfdKJJCeiOr
ONWF7K9MPJIZHkkKm5Y+L+6sp61KNzk1nhr3J3811Cih4NXWwQHuzWR0or05
wD2FZj5X0pivWEXav7a8nfy0hFyvZ3odehpC88q0x5xflkVuxmrS3v6ZVL5r
586ti71o70ud8XePxVP52BXyNmKZ9HHyusj5z7OogTtoZE1dJjmj/2sS6+9F
vxxzBjDnEubY49winKuEc+txzzDccy/uuQrvWvT/WjvzqJzXL+w7yFikZGoy
5RBK5iFthyjDQZmKEDIPRTiZyhAyD82TBkmj5vFpepqflEqlopRKkyGSufLG
vs4f71mr3/mt3/v+9VnrXuvc9/e+9rX3dT9rOeBesbjXN2PWYTp00IIOMuqs
Wwx0OwfdgqCzADq7QuelqMtS1EUBdZmFOs5CHS+ijn6oux/qboO6GxSwT5gx
NKUX+8QMvjKDr4rgK9k89iEzk0Z0Zx9KwbdS8G1f+DYZPk+GzycL2OfGyAtj
5IUY+iILfZSFPmpFH81D381D30mi79rQp23o05Po06vo66voa2P0tRnmgBnm
gB3yIhlzIxlzIxRz42sqzxlmGs3AnLHCXLqEuTQNc2k75tgpzLFzmGO/ufHc
i8TcU0Fe/Ik52Yi80MKc3IW52nSc56rU/2NevMWcH4A539OHc6G3LudCrzbO
hWjkxWjkxX7khT/2acA+/bHPA5wrjXNDcW5n39lZXqRAB0PkxTPoYI+8WAbd
fJEXqtBZCzrPg85zbbguGjZcF2/UpRB5UYi80P6L62iGupuh7mtR9y7X2CdM
T5qHvPj4lH3FTKfQW+yrIfDhEPhw43r2YQt82wLfvodvJeFzSfh8IHy+Ee+r
jXhfzcb76jP65TP6ZT365UvwHlPJj5mgD03fuPDm6smJpBk4xTkorQiMIBuH
LmdlZzyk1JIFLwa9TALd6eMjG0cDhQiakHJ71wpRNZhCn0yj5t8VlNDUxbJz
pxWFgdZ0Rnbr1bNZPjTLucU081oNmEo9k9/fnB1SSnXzF9PpbumgN7lby64w
tBTQ/XGLqt02JIOeJBk+uOxicyS9cggea7yuARSRUmJZ85LACvrsFjxElCSi
re4/6Uu9jA4EnRqSRE5zwqvGfOl4V4X+pA2p7driPGm6H+V/1SjZZZNNR9P2
dDCA7IsmXBSWCWlTS0sKra6hmNAxj2h1KvX1MI9asbuUsr5e+KygUkcK2RWb
lFUyKP+1xg3fr2VUNnfdiOKcMrLWGqLd3BhHlfcflC7u/phsm712tLWW0Nln
/glyHVuNN2mrEutVSGZYL3vK6x+Mef0L9nmAfZqxTynOnYhzq3HucnynL77z
szt/53ncSxn32ol7nYYOC6HDAOgwx51184Zuk6FbGXQug87S0LmzukxawHVk
elMr6vjFievOTKUIIde9O3zSHT7RhE8s4CsL+GrTYfZVKHwYCh8+hw+D4dtg
+FbOkX17HT6/Dp+HGLDPj+N9dRzvqwD0Rer8bRWe7vlgCB2RVrd/2CudAjcI
PVsLUkEvcgg9+PB7r1ia4b+4rf5cGRhHXfsuv3PwcT5tpq8ym3bEgk7knCHI
+WYRTOlfV2+129kAimjGzMZvJ4QVpP7uc6CUQwzoSEteH9Ef3B5E9ZqH/7I1
KATDacRanaJYUxE1yBenBF8tBWNo28vslrKmXHrX06twU1sYNf+iNb3pnXMp
stWHNk3VSWzvmU/WvxhCZ88oGbcvTiOz4oLV2ln1pFP7k5kknt9rz9PW56S/
Uko69sIjapfV6OADCnj1bunUgSmkuk9+XWlBDPXc7O9TWuBIX0Z4Ve4cHUxT
A8/7fX8ZSU9o2cu1tXY0Nr5k2y23AIr81vXRdlES/d7vvKxLNw8ijZrCZX0i
6PrlZ06uNZk0baHkobTKDKqsO3IhZGsmGWJ9vSavG2D9Fvb5LsH79Mc+g3Bu
NM7thnMP4zuX4TtHjeTvfIB7rZLje73AvZyhw2HoMBM6yEE3Heg2CbrJ9GKd
B/dinWX7sM5nFbguzBgaWct1MUEdTVDHq2u4jjdR95uoezjq3gSfNMEne+CT
1xrsK6YTrYKvVgawD5lxFAofzoNv58G3/eHbKQvY58wQ+gGfGyIvDJEXrQO5
L+RCuI+YPrQVeZETwH3HjKAuyItupdynTHfSzOU+lUZfS6Ovi5EX+zEH9mMO
ZGIOaCMvtJEXw5AXbsgLN+TFIsyZfMylfMylhZhL7zHH3mOOTcQcC0deKGLu
Pd/Gc+8+5uQ+zMn5mJNi33iu+mGuRmGummAOP8QcHvFf5oXNP/LCDbkQj7zQ
QF7oYV0ReRGOvPiMfQKwz3vs8wHnauHc7m/43M5yrbO82AAdhkOHqp2sQyLy
QtmDdTuPvKiFzrXQWRE6G6IuhqhLdBjXRYi8ECIvQmy4jr1Q916ouwh50aLN
PmFa0yT4JB++yoev0pAX/ZEX/ZEXUsiLIuRFEfJiCfJiIPJiIPJiKPLCDO8r
M7yvluJ91YJ+aUG/6CFH+od9F72RywWDqGHF3DsDD6WQov2akaYGD0F/MjU/
8jFjqJDOd90QWiyVCwaR9Quz3PS9KWSpeyn0hGouGES3C2xXBJ5KoSFWO+UD
v6WD3iS/R2ikFxNHH5deac6ufgYK6Fh/+8GGB/LJ3WCYcFFyFGhPGR8MD5aE
BlL19Kkfbd7XgunkrHlm8rfAMlp/8/IjvYx00JtMpk7sqnosjjZdGmB5tuP3
2q5fdKWyj5dvq3X8XhMePzlNddQLKj7xk0KKdHzW9fveIpqzUDbv4O+VNPi8
XAeT6MyOkZrL1QrpTbHji4/qabTl/ff6j+r3aGFhyYYpV2PJOVbpXZV+JLXd
Hnq9Td+O9BUDVm19508rA0khqDqSzGfON+gja08mc1ZGHPQLIKtpwSsrhwvp
e9W4Q0pmQpovvv/F2WVCOoD101j/1JfX/97nFPYxxj6dnduO7zyE71yP79yD
ey3CvQS4lwd0iIIO16HDCOg2HrpFQjcb6GwDnWOgc2d10UUddVHHHajj6WVc
d6aA8lB34UX2CbOjv3azT0auYl8xg+gjfLUePlwPH5rBh67wrSt8mwzfHgll
nzOD6BJ8bon3lSXeV6HoiwD5IZZFEtlgAF2RunHonpWQjMbPuOnyNhcMIoUT
I5+O+JhC2t3sX/kNygYDSGKrRoHydSGlNdrf9ml7CPqToPnOhkvmQhplsUgm
e0MBGEavFByGJU7OpETB6fenJwpBDwpqrJz4wiyC+la45E9fXwumU86g7hdS
3z2j1qRzf5UqRYH2NDvEqGhFUwB5nDDJ9bteAIbRJadl59YcyqQ7ArsLEzp+
r7n9Ykc/71H3+axbRGeXur6IOxZPM1f/pCtpr+wqMWhfKCWmnni0JTuBPFPO
d/AOFTj1GxFPYXT1nf5vp+YX0aK2PmKn5kdQeZDLfDnrLJq7QGf/owd1tCSh
vzD/QQblTrYtP+tUTmfavm1TmVpHdcsFD4fOyaDZ+Q1PWrqWk57nlQnu9dU0
LE4zbdHVPNreReVGikoZaWJdKOB1JaxHYR+5FbzPKewzGueq4dxgnDsL3/mh
lb/zKL7zFO5liHv54V6F0OEydDgBHSqgWyV0K4Zua6DzGuisDJ1NhFwXpj09
RF0Oo46HUceVg7mOpqi7Keq+AnWvhE8q4ZNiRfbJYPhqMHy1HL4y6M4+ZAZQ
DHxoq8y+ZQaR63H2raEC+5wZQK3wuS7yQhd58QHvK03khSbyos9K7qNrdtx3
TH8agr4b2Y37lNnRv+jT9ciL9cgLM/T1GeTFGeSFB/KiBHlRgrzYiblRhjlT
hjkj3cJzZuwMnkvMdGrGXJqOOTYdc2wB5tgdzD0fzL3un3juPcacrMecfIg5
KYu5+tmS56o+5moZ5vBKzOFJ/2VenPlHXtxGLvxezblggLxYjvV3L3j9zj/y
4sQ/8iIC5w625nOP49zOcq2zvDCBDheggw50uGnFujlZsW4bkBfG0NkYOl+G
zi7ICxfkxQLUpXQD15FpT7nNXEcj5IUR8iIBdfdCXnghL64iL5KQF0nIC71C
9lVWF/YhM4jMK9mHmciLTORFOXybGcI+ZwaRPPLCAe8rB7yvNuN91Yx+aUa/
rEOOZPeyKNwxsBAMp/asO5dODxVRk7+XfZW3EPSgV0+uzXWWjKTyZwfGq1dV
gklUeHvJ9IfDiig6Vd/cX1oAOtOHYV9Ulr4OpkW74w/1OFANppB9yr7+VotL
qGLd/CPS2+NBV3p1dPG4CZtDqWVqe1tXrVIwhtSST8WnOORShV7N510ns8GO
vIt66yTMEdKbVK0d8jeSQHcaG1maWBMaTuMVI4vlxjeAIvrofVei5kgFtf9o
vtRzmYjmdfnQQV/ql2S8r7dbIq2Z8nKV3ZgIChhT20Fb+hH3vH6OiR95fUvc
naf/kKqM6/fk6XfMsb5z548cIiSH2et2xl18Qd2bd2vFXxTS0Ci1GP+GItIv
mzc33a6BPl4YebncUUSDzEVuc9sqaHh92aY92vU0qvDs+0z1fBo6LnfvTtNK
GoL1twW83hPretinBfsMxD5mOLfmPZ/7PpLP7ew7Z+Net3Cv57jXuF/3F9Fe
6KAJHSShmyR0K4RuGdA5Azp/jWCdO6vLUdTxKOpoLeQ6TtLjujNdSeEv1H0X
+4SZQpPgE/k09hXTmSbLsq/S4MM0+DASPpwTwL5letDiYvatPnyuD58bw+fX
8L66hvdVNPqie03RBalF6aA3dd94Zax9sYDcLtJzndqnYCylhNcpf8jNo/CS
wHzJR/GgK5ncFz+X5R5K7y7Uhp4eVw2mUMBY94rKD8VEUToy9zNjQSe6KLVq
/674YLo5r8Zw9ZQXoJDCV8ofG3e6iBb/uWLt0bPJoCc5D69+JT0qiqqGH5L8
7v4IfEDXVms1uoxOobye14zGLi0H46lqdInG1+WP6ZD11sTr+uGgDS2fOce4
2diXxt3ZnVC9MZ+W/GIICcS6PzCI6MiJSUMLonzrSbD8JzPpx7mKDKXC5/Sn
oveaove5dMXm0dqi90E0sypgjdLnFHLPT5lR9y2OulzTn/n2mwuZDHuzNV8u
lKwUTgU5y4TRftPLg9aqWNM2geI0tRf36dsYhUUSlQI63adUX3e9gOIdvueq
ugooEutbe/O6ONbvYJ8z2Mcc+7zDuStxrifOTcV3drXl7zyL7zyCe1XiXhMt
+V7FrqzDF1fWYQ108IRuntDtMnTT7MU6M+NJWYl17oK6dEFduqAuAahjAOo4
eATXURV1V0Xd56DuYtHsE6YTdZNmn5jCV6bwlSR8FVzKPmS6Uj8f9mHeBfYt
M5bmw7fW8Lk1fB4Fn89DXsxDXtTjffUBffQBfaT6kPvIBnlhg7w4hrxYVMZ9
ykyiddbcpzbICxvkRRnywhd54Yu8kE7lOZCNvMhGXlQgL9ZP4znDjKFg5MVm
zKXNmEtzMJe6p/EcY7rTQuSFBuaeBuaexH2ee/XIC2XMyeZEnpNWmKsVmKsU
z3O1CHNY2oTn8P7/MS+UkAszkRfDkQsSWA9CXjSP/c95oYdzU5EXmf9jXuxF
XgRCBzvkRY0C68YUkQvyogJ5UYG8GAydzfS5LswAeoW63J7CdWTG0NEkrqMc
8kIOeTEAeVGDvKhBXsghL2yQFzbIi5PIiydP2YfMJLpyi32oi7zQRV7sQF6U
9mSfM8MpV8Q+j8L7Kgrvq0N4XzWhX5re/t9/H2+EaID2ivynYCwNUOh7LT85
j4JXHnkn7hAF2lOZ1+0JkuaBpHbe6uafUg2giE55+7VUrq2ggY/XyCyMigWd
aNndNROvhAbTkXOZ8cbfy8F4Gqfr+kQp/jHNqXZ+9SxZCHpQ77KztbenRNJY
hb1WRqIyMI5Km+cXPW/LJ0+bBV4N4wWgM71U9+tx6Hswqc+5vlb2RgMoor5Z
ahOqmiso7eTeYzG9EkE3+j6x+pXp9jBatpHmeF9KpxW/6E3uU/tn9Z8QRwOO
D4p98bWeTE7+ZCZNGOoydv3MCvI55/J2rXIhWe4O7mA49b6j7fd6qohk+p//
bfnCJJL8pF24fKE7OSyzj/RYG06bxSwapFaFk1J6yJ/bN9pQ/dFX6vv2+NKi
nkd89f1jKKmxKeTspEjqeix04ts3UTQU62JYv27G653t09m5nX1n8zG+lw7u
1QP3GgsdxkOHQ9Bh5CnWjelG01RYtz+g8x/QeQB07qwuLqijC+o44gPX8XMV
153pQRHPuO49LdknzHiyhE8889lXTCcSg68IPiT48Dp8KKnDvmXa06p77Ns+
8Hkf+Nxbnn1uj/eVPd5XCeiLsX9mndALEYIedKgyzumBfCRNsJcP3HWuFkyn
T61GJfNHl9Hm37b0nn4hHLShyD9irqTc96Xxw1/sETevBlNo2i7H8zc2ltC0
fhcari5KAt0p+VbdqEl64XS4aY3D3gdPwVjyiZhG3QLzKGncZG3n9CTQnSaF
awyb/yWckj8mTLSVrgZTaHKYyrHjJcU02XTczDbtMNCa2nOnXFCX9aGrF3NW
q3yrABMp6nCXjwatBSTMvyv+V04Bpfxix3+Xnp1bmJRJZz6dihYLCifz3uYd
tKFKM9cxGU99aYdj5QEXtXRS2N7L2EXNmx6Eel7uliCgSZcdBBfbyynw1LfR
Vu3x9Mhmb9XAxMfUXdujT5tKA8m5/75Oa1JHn07aW9nvRAUdT4yWNGypJ6k+
fwxz3ptHf3i+fjTItpIOYf1xb16fiPVGLd6nK/ZpVOV9Pl7icw/hXF+cq4Dv
fGLE37ka3+mIe9nhXu2419THrMP0x6xDMnSQhG6S0G0PdPt6iHVmWlMUdF73
ievCTKHXqMtu1HE36pgSxnUc+I7rzowl8Uiu+30J9gnTnSThk2D4Khi+egZf
JXRhHzJt6M089mGYHfuWmU6b4VtH+NwRPv8Bn6siL1SRF+V4XzWhj5rQR9OQ
F5rIC03kxWrkRbsl9ylTRHPRp5OQF5OQF5vQ18lneQ4w4ylGh+eANvJCG3kx
DHnhIM9zhhlHFsiLLrY8l5jONHwuz6XpmGPTMce6YY6ZIC9MkBfpyItDmJNH
MCdzMSeHIy8sMFcJc/UM5vBGzOES1/8uL8b8Y86vQy50ecW5MB55UduD19Ua
eF38H3kxGvvUYZ+FOHcezi38H/Oi3IB1qDRgHWSgQyHyohB54YW80IDOGtBZ
AjpPQ15MQ154IC+iFbmOzDgqQF6Ioe5iqHs68kITeaGJvPBDXigjL5SRF7Ge
7Ktg5EUw8kLuPvtQFXmhirw4jrxYlsk+Z8aSKvLiKd5XT/G+ssL7qhH90oh+
0f3738fR7Sl/Z0oVmEzRYxZ7nU96Qj5iCQpKPyJAW5K6uH/HW3N/chC3cxjp
UAxGUfwNgcZ48xzKMlEISZ/yGAylVvEdG6OTOt4xyT0GG0kngm50cJbRdYV9
YXTvjkeyunYtmE7Leyz5sPTpMyq1mKEjsygCtKXczefvOdn7kb7Egv1B7+vA
DArKs9aWfVtOinfkNJ7/IQCdqaAqv1a5dwht6r+iUelhIRhO0wqqjJraRB3z
4rc3cqGlYAyVTx0xULd/HlVu6n/dbU4kSRv+pB29nG598GKWP7U8tRuQoZpO
IzY5d9CbToa2FUULBNQzbaR6qX4lGWyS1SrVTyJXl4ZrK0wKSavizYye9g0k
aWOtc9FeREtSdcUbWivoy2FZI7VLtUQXwwYdPZdNGWszG/r5lVMT1l9d4HU/
rC/BPtLYZzn2qUvlc9Vx7hGcm4fvbNvI37kC3zlkM99rKe41fgbfyxY62EIH
+Wmsgzh0E4dujx6zbrug8y7o/Fs169xZXYad5joybWm4IddxA+q+AXV/L8Z1
L4RPCuGTVPhED77Sg68OwFfp8GE6fPgZPlwJ366Eb70usG8FOuxzZjKNg889
8b7yxPsqDX3xbfYy+f3j40CXjt8XKyaPdg6hH6cG6MxorwMzyLx8jVJaWzmJ
ku85eFxMAe/SiSm57Zd2R5PeaJnDvedlgj50YP+w/L4rE6jCf88Fg1cVYCLt
9e9aOPRDAaUr7rl6rzAKtKeo+5ZNj0SBVNJ04NOD2/VgJj3y7xO+Puo57Xrp
UJ0oEQnaUbKCRZnqLX+SMZhaUfCjCkymYaa/FWcmFlNVqFr2TY808B45KWr8
aOktoILTC2QUtJJBT+q5xcbpZUkkfWjY2DVIVEdDG38yg8yMPiYPiyunfLWu
NSsnFNJO0+4dDCe/5d3kt88Q0eEfJx43nk+gYVnzlF+dv0NtQy48CpUOo1lF
rSGp/cPo/AllizRpaxp2OGNAZOF96mMgN8xGWUA1c6L2qiSG0qeHDfNftERS
yQZe7491U6xLYB897FNtyvtk4VwTnDtlKJ+rOJm/MwLfKb6CvzMF92po4Hup
414W0MECOtwxZB2WhrFuzHskP5x1e7yBdWYmU/oh1rkGdalBXQwUuS5XUcer
qOMB1HHycK470540fLjuy/zYJ8xEMvZjn0gqsa+YPrT6APuqRcg+ZN4lk8ns
w13m7FtmBnV9zr4NnMM+Z7rQsnvscznkhRzyIh/vq1XIi1XIi3r0kUwP7jum
LTkhL9735T5lRtGb69ynb9DXb9DXYyW4r0OFPAeYbtQfcyAFcyMFc8MUeXEV
eXEVeXEAeWGDuWSDufQFc+mLK88xpjMJkBfBmHvBmHvO/5IXvsiLss08V93/
JS/EkBfrMbed/yUvviEX1iMv8pAL9Vi/jry48i95kYG8kMO5y3FuZ9/ZWV60
QodW6LAReTGvH+vGDKe8fNZNE3mhibxohM49UJceqIsW6uKDvPBBXjxGXuxG
3Xej7t1Q9y4p7BOmG1XCJ0+M2VfMUHrYl321pR/7kBlFX2+xD1WQFyrIi2PI
C0JeEPLiuhL7vMt7/D3t77kv3PG+qkO/1KFfVv7998VV7s6TulYHdvSdvqXl
OpNy+nDRzF//ghD0oCbftYPe1UVQ2/tZj5y+JIOedPJA5hlhZBRVGE+yWOzY
AIroyZXW584/KmhhWrGgLSAJdCe7yBzluSXh5ES2N/SScsBAStKK15/hkEyj
fEZq1nTcg9nxbtcdv0b+YDYNeXju5sSZQtCDnsd7tcy8EEFPxhv/dnRPKRhD
Dz9lnZFOy6Wuzcnj6qJFoC/9UTz0+E3pJFIbZbLuU7sQ9KC28cvehJlH0u9G
FoU95Bto3C+KSG2u2yVto47fNZ82OlUZ5pLltk0dDCKfjOPu11xS6H5gVap5
QxQdK6gtNG+wp/Nb5Y4/rw6kswd6fPBMi6YnXbw/D8hwoMi3TSY7Pz+ggLfj
EutchGSTs0j7qH8IHV3zZ7XnoVhyxXp/rI/G+hbsE4F9rmGfdJzrgHP9ca4f
vjMB35mN72zbxvf6sY3v1abO9/o2knVgelA0dFgP3dZDNzPopg6d1aFzX+jc
WV1c73MdmZFkqcN1VJ/HdWcGkpI2170plX3CdCdl+KQMviqDrx7DV7LN7EOm
J3nAh/bwrT18exK+NYbPjeHzy3rs8yC8r4LwvspBX6he+v3+UotI0I6O731/
U0ExgGxfSPSwGPsMFFBbQOa2FV/zaORd59SFJ0vAaGq8kjxtnfMjMr6hryhs
DAVv05IPqjsP3rtPxmKan5K7l4PxtP1cY+nXkY8pYnjv64Erc8BAOhieZrZP
N5m8Dfs9iBiaCnpR7pdbQ0cNj6HbcY5r0rOfgQJ6JnP64/fN+aR88u4xhwPJ
oCdN3SDuoNEnim4ULljsJ5EPhlCYyObZQJ00MrL6NNVg1lMwljRkhnmUbsuj
kNejPQxMwynpF23oxe8HRFNu+tKolqvPJAoe0owt1zroTzPHzPSt3yakPVMj
dHKm1JJSTMSqnCnpFObp6Z2U9Izuto4RX7DhJb15G/vaZEMabYxeo6UY+pSC
dJUNe6o9J+eBB2q6d7wLzB9MatD8VkB3sD4X65uw3q2N9/mzifcpwj4TcG5d
NJ+7A+de+8Df6WHI3/lUib+zGvdqxr0Ux/K9ulxiHZix5A4dVKGbKnSbA90s
oLMFdL4DnU8JuC5MAamgLmO2cB2ZXmT8levYqsh1ZwZSWxjXvWtP9gkznrZY
sk8ew1eP4at0+CrMg33IjCbFy+zDe1XsW6agY36xb6Ph82j4fMw+9rkY8kIM
eZGC99Vb9NFb9NE45MU2K+47Zkff+XHfXUReXERe1O//z3nhjLxwRl50jeI5
YIG8sEBeBCEvApAXAciLJ8iLh1k8l5geZI651GcCzzFmDIl/5jnm857nHtOX
ev9LXkxDXsxAXixAXkzHHF6COWyJOeyFuX0Uc/tMJ3kRhTkfilxIRS5YIxeu
Y/14Nq9nrf7PeVGDcyNxbj7O7SzXOsuLFORFCvLCFDpIfGDdmL5UD93uIi/u
Ii+uIC92IC92IC8ogety1ZvryIwklZVcx+HIi+HIi4HIi2LkRTHyQhx58RS+
egpf5cFXUz6wD5me9MOYfXgEeXEEeaGLvBCDz8Xg8x7IC2W8r5TxvorF+6oG
/VKDflmOHEmQEO44ZtoAimiizsGkdXkVZL89IKxqdR4YTK3m9oaye1LpesCz
pafMwkBrCrF/fMdlrQ9Jx+9ZmHo7DwwmvX1pa49EpdIsqQ2OKmU1YCplPAmh
2dJPqUK5z5ecebGgEz1c30NwelcwPV79zf3cwAIwjHo0zrzkVZpBSmYHnCXf
5IMhpDJoSfLqXelUVjLCfpBRGniPpFY0qsvFx1L31a37n8flgyEktvtE70ET
0mlDtViLuHMhGE5qBj3V8oQiqm0vGpM1JZLqftGO3MyvBMcl+dPa2Hz7a+bF
dCPhJzvevePS2yZtyqHESamvZr+uo3XzZF7Pfp1BhadF3ovryum8UuaLM7eK
KDZ5VA/3WxE0ZZBOdcPHLLIclrZp8/NsurO4fbZHRMfvfcWR7fYdfv4L69Ox
fgnrne1TjHONcW4zzv0d32mI70zAd1rjXja41wzc6yx0OAsdtkIHt1WsGzOE
Luxi3RaWss7Me2QHnTurC63hOjLD6AbquB113466W6DumQPYJ8xUmg2fnIav
TsNXlfDVQfjwIHx4DT6Uh2/l4Vsj+PY5fP4cPl8Dn8fifRWL91Xh33++vTFW
JSEsDLSmtfd2xwa6+9DYcvG71hlZoB/N6lO/MC7u5/9nFNs137IBFNHNrF0L
xGoryKFhsqT37izQj37kK2b1PJtEhdODjRdrxYCOZKXp8lDWMYjsB+wc5/a6
Gkyh1qKkD+l1JdR11WiRQDMDvE/JO/cLGmXiKWl0y+GFtpmgD72bqPbYLjah
Y87VJK2XLgIj6IWEwkE1vSxqbJq6emZ+JuhDNePn774glkj5i93d7B6mgfdo
oYGV1kkNAX1dueCOTV4dmEEzlArTYlLLKYFShdsdUmiS9k/epSydaZMtzKNp
95n0QVIDI2lC+/KhUgPtqGXeMNdBjv5kv/HJcpm2VPrN4Zq9TpsX2afEhO9W
j6X7rW4jHL7mUmPsXpe5CyKp35vGv5yvpZEN1jdgvfI1r+tinzh73ucP7DMZ
535s43ODcK7sPP7Oa/hODV3+zgk6fC9mBh3FvXZCh53QIWkD67DnHevG9CHL
CazbPui8Dzovgs63lLguTB/6qMJ10dDlOjLvU84OrqOcFNedmUKeT7juOTPY
J0xHSlrIPgmtZ18x/cgYviqGD4vhw2z4cMJz9i3Tj57Bt6rwuSp8HuvFPn/3
hvuCGUiReF+FoY/C0EeK6CPFHdx3zGC6b8F914Q+bUKfKjtwn9bEcV8zg0ke
fS1AXgiQF7uLeQ68w9x4h7nxBHNjBfJiBfJiWwPPmSuYS1cwlwIxl6KRF9HI
i9rlPMe0kBdayAs95IUa5qQa5mTbBp6TXX/wXO32g+dq8v/nvLiEXEhFLkQg
F3ZhPUGb17f9S170VONz3XCuyhk+t7Pv7CwvWqFDK3T4jrwYjLwYjLwo28m6
/YDOP6CzBvLC+S+uCzOEzspwXeTXch2ZYeT1iutohbpboe62qLsD8sIBedFW
xD7pm8C+YgbT9f3sq4v+7EOmNe2yYx82GbFvmcE0HnnRpR/7nCmi0/D5Oryv
1uF99QTvq0q8ryrxvlqGHPF32LXxdn4NmEpFCcktn8Se0oetOjsLi1+AQgoS
KJ0R/fmEek1fr6S3KR30psspwSpvPwpov9eDsGHq4aANWf3pZDhiqS+FlPge
qF+UBwZT+fLVlz31Ummr9zUt/fl1YAZ963/QsUmqnHQd55473iMRdKMCix63
xY3CqNl/2Os5ohwwkP7yv2ma5ppMpbJ+6zdtfgJGUq7n6vfTPj+k1QEli2bN
TwHv0q7i6w4ZUtEkqxo1f0HH70hmCL34ZiQ9MKrjfZZ7e09ASykYQw1C5+RX
M/PIx9xh3vWO93WkxU/aku2dhDy1ej+aYt/b7K1aOoWfmdZBb3L7PGjQngQB
SRysrU3KeUZPvOe8EuUIaEvg02BHw3wqu7R8T0jDSzp0bNWUW98zaOauUVG2
9c+oGOtLsa6M9ScmvI8b9iHs0x/n3sS5h3BuMr4zD98Zie80y+N7MWNoaTLf
67sK68AMoVjo8AC6PYBupdBtFXReBZ1/h86d1SXHgevIdKPTqGM31L0b6n4O
dVcuZZ8wg+nwCvZJKXxVCl/1X86+ejqNfcj0ppXwoSF8awjffoll3850ZJ8z
U+lCIvs8E++rTLyvyv/+9539bRer6MaAjjR50PdnMd5BtMzSe1+JQxzoQkFS
9QsuVoSQ+0CfjT7fCsAwyhgqdUT3aybteNsjW2d2AyiiH769qNaqggxebl+j
VZ4F+pF+0GbbZVlJZOX1pm5jXiRoR74SDVuOWAZQ2VZP3+VtFWAiiU6JKeu0
F9CkoZ8SvmjlgIGUkKv/1/dlydS1l1nO882poBd17fPk9JO/Ymjmwekett1L
wGi6IGHZWyD2iAy8ZTSnLRGBvtTcX9Y22SWR6t0nj5gzKhn0pJGLLRp6B0TS
/ZYT+npb6kn0i5lU+v6p8YNjz2mmXf2V9nGFZHalvYPhJDOrXsV8qojOxW+5
FDtDSAPHid/OnuFBGRk1gYHnI2jfnfZCfdMYqpSw7jfht3sUKDX67eevIbQW
60OwfhDr4dhnIfYRy+R9DHGuO87Vwrmr8J1H8J0m+E5b3MsW94rV5nuVQ4dy
6GAjyTr4mLBuzGh6LM66WUJnS+jsBZ1bBnNdmIH05RHXRcqI68hMpCvmXMdW
1L0VdZ/dj+teVc0+YfrR/gfsk1Hw1Sj4yh+++iHDPmSGkbks+1D1PPuW6UJz
pdm3Tn7sc6YjzZNhnxfjfVWM95U33lfHkRfHkRcOyIsb27jvmEIyjeO+i0Ze
RCMvdqRyn65BX69BX+9BXsxAXsxAXixEXpzE3DiJuSEryXNDzInnDNONFpzm
ObMSc2kl5pIU5pKaHM8xZiTNvctzbDDm3mDMPTnMPXXkhTryou93npM+yAsf
5IUIeXEdc9gFc/jI/5gX5ciF08gFdeRCEdZHYF3uX/Ki0Y7P3YtzNXFuZ7nW
WV7YIy/skRdzkReagawb8y7ZlrBu/sgLf+SFLfIiO4DrwgykH6iLrT3XkelG
8eZcR0XUXRF190ZeaCMvtJEXTsiLhffYV0wbskNeaE5lHzK9aY2QfTgdeTEd
eZGCvChGXhQjL5Ylsc+t8L6ywvvqI95X/wdHQrSK
   
   "], {{Rational[-1, 2] Pi, -2}, {Pi, 2}},
   ColorFunction->RGBColor],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["34: spider3", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"myColorMap", "[", 
    RowBox[{"ab_", ",", "ar_"}], "]"}], ":=", 
   RowBox[{"Hue", "[", 
    RowBox[{
     FractionBox["1", 
      RowBox[{"1", "+", 
       SuperscriptBox["ab", "2"]}]], ",", 
     SuperscriptBox[
      RowBox[{"Sin", "[", 
       RowBox[{"Log", "[", "ab", "]"}], "]"}], "2"], ",", 
     RowBox[{
      FractionBox[".1", 
       RowBox[{"1", "+", 
        SuperscriptBox["ar", "2"]}]], "+", 
      RowBox[{".9", " ", 
       SuperscriptBox[
        RowBox[{"Cos", "[", 
         RowBox[{"Log", "[", 
          RowBox[{"Abs", "[", "ar", "]"}], "]"}], "]"}], "2"]}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   RowBox[{"Log", "[", 
    RowBox[{"Log", "[", 
     RowBox[{"Log", "[", 
      RowBox[{"Sin", "[", 
       FractionBox["1", 
        RowBox[{"x", "+", 
         RowBox[{"I", " ", "y"}]}]], "]"}], "]"}], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", ".3"}], ",", ".3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", ".2"}], ",", ".2"}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.3900266957593327`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJzMnQe0ZVWVrqe5W+22zRZFqCYoFjlnJYhKEEGQFgUbUII2SjIUuRBBoQUk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   "], {{-0.3, -0.2}, {0.3, 
   0.2}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["35: spider4", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"myColorMap", "[", 
    RowBox[{"ab_", ",", "ar_"}], "]"}], ":=", 
   RowBox[{"Hue", "[", 
    RowBox[{
     FractionBox["1", 
      RowBox[{"1", "+", 
       SuperscriptBox["ab", "2"]}]], ",", 
     SuperscriptBox[
      RowBox[{"Sin", "[", 
       RowBox[{"Log", "[", "ab", "]"}], "]"}], "2"], ",", 
     RowBox[{
      FractionBox[".1", 
       RowBox[{"1", "+", 
        SuperscriptBox["ar", "2"]}]], "+", 
      RowBox[{".9", " ", 
       SuperscriptBox[
        RowBox[{"Cos", "[", 
         RowBox[{"Log", "[", 
          RowBox[{"Abs", "[", "ar", "]"}], "]"}], "]"}], "2"]}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   RowBox[{"Log", "[", 
    RowBox[{"Log", "[", 
     RowBox[{"Log", "[", 
      RowBox[{"Sin", "[", 
       FractionBox["1", 
        RowBox[{"x", "+", 
         RowBox[{"I", " ", "y"}]}]], "]"}], "]"}], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", ".5"}], ",", ".5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", ".3"}], ",", ".7"}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.3900266974818096`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJy1vQm45VV15r3S3Xa0M/TXGmPjkFxxnmNMjEmMojFGbTXRdCdqEsUpKg6R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   "], {{-0.5, -0.3}, {0.5, 
   0.7}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["36: spider5", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"myColorMap", "[", 
    RowBox[{"ab_", ",", "ar_"}], "]"}], ":=", 
   RowBox[{"Hue", "[", 
    RowBox[{"ar", ",", 
     RowBox[{
      RowBox[{".8", " ", 
       SuperscriptBox[
        RowBox[{"Sin", "[", 
         RowBox[{"Log", "[", "ab", "]"}], "]"}], "2"]}], "+", 
      RowBox[{".2", " ", 
       SuperscriptBox[
        RowBox[{"Cos", "[", 
         RowBox[{"Log", "[", "ab", "]"}], "]"}], "2"]}]}], ",", 
     RowBox[{
      FractionBox[".1", 
       RowBox[{"1", "+", 
        SuperscriptBox["ab", "2"]}]], "+", 
      RowBox[{".9", " ", 
       SuperscriptBox[
        RowBox[{"Cos", "[", 
         RowBox[{"Log", "[", 
          RowBox[{"Abs", "[", "ar", "]"}], "]"}], "]"}], "2"]}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   RowBox[{"Log", "[", 
    RowBox[{"Log", "[", 
     RowBox[{"Log", "[", 
      RowBox[{"Sin", "[", 
       FractionBox["1", 
        RowBox[{"x", "+", 
         RowBox[{"I", " ", "y"}]}]], "]"}], "]"}], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", ".5"}], ",", ".5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", ".3"}], ",", ".7"}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.3900267331631165`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJys/Qe0VtX1/Q8viiiKqCgqVuyKJaJ0FJCioliCPTbsMfae2GuM0dhiNyp2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   
   "], {{-0.5, -0.3}, {0.5, 0.7}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["37: spider6", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"myColorMap", "[", 
    RowBox[{"ab_", ",", "ar_"}], "]"}], ":=", 
   RowBox[{"Hue", "[", 
    RowBox[{
     RowBox[{"Sin", "[", "ar", "]"}], ",", 
     SuperscriptBox[
      RowBox[{"Sin", "[", "ab", "]"}], "2"], ",", 
     SuperscriptBox[
      RowBox[{"Cos", "[", "ar", "]"}], "2"]}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   RowBox[{"Log", "[", 
    RowBox[{"Log", "[", 
     RowBox[{"Log", "[", 
      RowBox[{"Sin", "[", 
       FractionBox["1", 
        RowBox[{"x", "+", 
         RowBox[{"I", " ", "y"}]}]], "]"}], "]"}], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", ".5"}], ",", ".5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", ".3"}], ",", ".4"}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.3900267322017345`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJykvQe4XWW19T8SQgtIL9JD71WK4AWjiCKIogKigkYBAQH1CiigQpDepIZe
Qu+92BVULFhQkd4SQXqHQCDtf353z+EccO/9/t/zfOd5Npnss9a7ZhljvPN9
19r7LP3lr3/qazNJ2rle2I8PvX429Fq37J8OvdYbej029PrJ0Gv9sn889Npg
6PXPodePhl4bDr0mDr1+OPTaaOg1Yeh1y9DrfWFvPPR6dOh189Brk6HXI0Ov
m4Zeo4deDw+9bhx6fXDo9VDZmw69Hhx63TD02mzo9UDZHxl63V/2R4de9w69
rh96bTn0uqfsrYZed5f9iaHXP8r+5NBr1Xh/lbI/NvRaqewthl4rlr350Gv5
svFhubLxc5ny4f1Dr1HlMzEuVXGRkyUqdvK2WOXqPUOvRSqfaw293j30+vnQ
a/Wh10JDr1vLrwWHXr8pv+Yfev2hfJlv6HXn0GvZode8FdvSZT9YvsxT9bD9
VNkc82Idzziv1TgLDL3eHHqtUD5MKx/wc0b5SSwaNqj7MmW/v/ybXnlbc+g1
Zei1rQbHMf6Xy3fbf4tj7otzH4kxH4trPRk+PBO+PRc+vxCxvBgxvhSxvxQ5
eTFy9WLk8IXI7fOR82ejFs9EjZ6K2j0ZNf2XutbEYgxwfWNjghozxG4sgX9j
7EE19u5XY/I+NVaNeTCcmF9N/2fMw5GV1dxZUc2pFdRcW17NweXU3FxWzdll
1Fwmj+b4KDX3baMJcMRasaRaQ8CYtWUJteYsrtYibGvUv+p98vx4Hb9e2T9W
69WP1HrFdd9btbhFrV03q/WKWP5DA726Sa1XN1aNHq6cfKDqdYNar1yvB8q2
Xl0Xtbsuandd1O46tV5dF7W7Nmp3bdVupbKtV9eq9era8mHZsl2j69R6dX3F
uGT5v2Hl9sbK4aKVk3U00CjyBr8XrjzjFxr1C7VG/UqtUb+ra6I/f65rzj30
uquuOXfliGvOVTWw/VTZHPNi2Yzzep2LjqEhS9d1p9d10S50Y9XyHxv/l6pj
NqrjOPdDFcOkodfWFSf8//zQ6y9h3x3HPBjnTowxn4hrPRM+PB++vRg+vxSx
vBwxvhKxvxI5eTly9XLk8KXI7QuR8+ejFs9GjZ6O2j0VNX0iag13jAH4YmxM
UGMGLhhLcMEYe1CNvfvVmLxPjdV71Bi+W43t1fR2zJsLq6g5spKaOyuqObWC
mmvvnK/NzWXVnF1GzWXqYY6PUnN/KbUmYFsrllRrCBplbaE21pzF1VrEvG+N
+tc7bI53r/Ujda/1Q7V23VLX+mf5Y73CT/daN6r1ihjfOac8XDmxXl0X9cIG
zw9ULT78/1O7a9R6dY1ar66J2vl9ane1Wq+ujtpdrdarq9V6dXX5tnTZ1qtr
qhZLlj/vrTxfVzlctOJdWwO9Ij/uqcgnc90ClX/wgkb9sq6Jtvym6ozm3FG1
RYv+VmO/q/JCTzRn1eDdZcOrhct+qd7n+DfqeMaZWv6iXTMqJvxBK1ao86eX
zxyHVoAHsDpJ3Wu9oO61wNC25aPt++KYR+Lcx2PMp+Naz4UPL4Zvr4TPr0Ys
r0WMr0Xsr0VOXo1cvRI5fDly+1Lk/IWoxXNRo2eidk9FTZ+IWhOXMQBHjI0J
aszAC2PpYTXGHlRj7341Ju9TY/UeNYYT86vpf8e8OfK/zdErqLm2vJqDy6m5
SS7M2VxrLK3mOHUy973ucE9lrfAaxHplbaGu1hw0ylqUekWe0THmhexpH69x
rF3ubx+r67rXuknda92o1qsb1L3W9eq1IbG718Ierf8+v1yj1qtr1Hr1P9Xu
3rKtV9gf10Cvro7aXaXB/L5K2darK6N2V6r1Ctt6daVar64s/5cq+32V86sq
J4vVdb0GJC7WCwtXHsDUgpU391TkHFygG/S9ozTQqNtqbHTm9xrwC/25s8ab
o+JnvJGV6/nLfjrsl8OerIEmMc60GgftQh8W03/XrqmVG2J5vWIxn91r/aty
tVrVb+vKv+0H45iJce6TMeZzca13apR9ey18fj1ieT1ifD1ifz1yMily9Vrk
8NXI7cuR8xejFs9HjZ6N2j0dNSUW15pYjQH4YmwQuzHzqBpLD6sx9qAae/er
MXmfGqv3qDF8txrbq+ntmDcXVlZz5H+bo9Erc215NQf/p/XFaA24YC6PUnMc
29wHJ9aEJdVaQX6tIWiUtQXbmpN7KdTGGoVeoWNrq/XK2nWLWq8Y373WTWrt
ulHda7nvnVj+W6+Iy73WtWq9ukatV9eo97KuVuvVVWq9uipqd5Var65U69WV
ar26smq3WtnWqyvUa/nL1Wv5y9V6hW29ukytV5ep9epytV5drtarKyrP7y4/
19Cgp7qmrr9A5QSMzFv5BAtzV/6pLRr1i6oVuvSbGgPN+ZMGHJy9YmaM2Sqn
jDGrBnO/bTg/Vx3zZr3PufQz89X4aMKCddz0ui7jTq64ufaL5T+4Ah/rlt/U
5v2Vz79UPu8N+5E45vE495kY88W41ivhw6Tw7Y3weXLE8mbE+GbEPjly8kbk
6vXI4aTI7auR85ejFi9GjZ6L2j0TNX0qak18xgAcMTYmqjEDF4ylh9UYe1CN
vfvVmLxXjdV71Bg2zq1Xxvyqai6srObISmrurKjm1ApqrqHL5uCyam4uo+Ys
dTKXR6k5Di/M/dwngSPWiiXUGgKWrC1wx5pjvVqn6mGNeqLet/1DtV7drO61
blJrl9fmj5U/7rWuV/da+O9e61q1XhGv14bZD5Mrrw2xvZeVcw01Yn19X9nW
qyvUenW5Wq8uV+vV5VVT6nhZ1PFStV5dqtarS9R6dYlary5R69UlFeMSZa9f
ub1UvWfFtcDwguUn15y/YgQX81SuqPlclWdqRf/DnEIPgEbdWuehOX+o89Ci
v2ugbbNUHtGemTXgz8iyXwn7rTqG42fUuYyJJsxTv5ui1i7OXbyu/UT5vEjV
YM3KA+sp8Anmf1E1eiDsiXHMk3Hu8zHmK3GtSeHD5PDtrfB5SsQyJWKcErG/
FTl5M3I1OXL4euT2tcj5K1GLl6JGz0ftno2aPhW1JiZjAI4YGxPVmHlUjaWH
1Rh7UI29+9WYvFeN1bvVGDbOrVfG/KpqLqys5shKau6sqOZUrimWU3NwWTU3
0StzFi6Yy+TFHF9KzX30ypoAR6wVuce7uFpbyLU1Bx5Zi7CtUU/W+2tVnm9R
a9dNau26Ua1dN6j16vqqxT/LT/da+O9eixjda12t1qur1Hp1pVqvrlDrleea
B8q2XlEv3yu8TK1XWcfL1Hp1adQRPHyi6nhJnUvtLlbfK7lIg7mGel2o1its
69WFar26UK1XF1YOF65x2N9YsK4FXuYr3xh77ordPRV5Qw9GVu3oxdAT1ibw
C535TR2L/txZx46oHHHsTFW/Wcp+JewpZXM8OjBbjUPfMked/3r5xf+z5lmo
rn1/xc3/sy+9it6+/7BcxfKhqqPt3P98Js59KcacFNeaHD5MCd+mhs/TIpZp
EeO0iH1q5GRK5OqtyOHkyO3rkfNXoxYvR41eiNo9GzV9OmrN9Y2Bx9TYmKjG
zKNqLD2sxtgDauyRG2PyXjVW71ZjGJwb26upMf/OedkcWUnNHXBuTuWagjqa
g94PsV6Zs6PUXF5KzXHv5Xpf3ZoATqwVaJQ1BI2ytsAdaw62tYgaWKOerPfX
LPtmtXbdqF4n3qDek+e67rWuU2sXeute62q1Xl2l7rWI3b3WFeq1IbkarQHO
L1fPNeTWa8NL1Xp16f+hjtarS6p2q5f9qarjxep9yIuijhdGHS9Q98bnq/Vq
fNVrqbKtV+PVejVerVfY4GiBGgddnLeuRf3nKt+o4ZyVE85jvUbd0bZZq0bW
KPaB4R36c0f9fnjlYuayH6/fD9H7v7hse2rZHMMbI+r/eX+WOv+lGh+ew6V5
6troDNihFwEDy1e852mAMbB9kgbYfjjsJ+KY5+PcV2PMyXGtKeHDtPBtevg8
I2KZETFOj9inR06mRa6mRA7fjNy+ETl/LWrxStToxajdc1HTp6PWT6gx8Jga
GxPVmHlUjaWH1BgD58befWpMsgY0VsG5MXyXGturqTHvedl6ZY6Ac3NnBTWn
qIe5BubNwWXU3FxazdlRai7DBXMcjTL34YU1wfehrFHWEGxrC1iy5oAJa9Ei
ao16qt6nj0WvblJr1w1q7WJ891rXqXuta9R6dbW618J/rw2vVOsV8Xr9Th7c
a12m7rUu1dt7Y+vVJfqf5x1sr+svUuvVRWq9wv501RQMeC8LnHjvfbwGfTJ1
PK/qSO3O1WCuoXbnqPXqbPX+1dlqvTpbrVdnq/XqHPWe1bnqNeD4qtsc5Se9
2GyVH/dU19Xv4SFreWsUfQL8RYv+Xu+pajCs7JfDnho2xrD6/8l1bvZmvP5U
14Xb11Yc+DSu/GfNe7AG+AHP+2qA50fDfiqOeSnOfT3GfCuulT3SjPDNzg4r
+9+xDIsYh3XsMyIn0yNX0yKHb0VuJ0fOJ0UtXokavRi1Q69cU/TKtX5CjYHH
1NiYqMbMI2osPaTGGDg39u5TYxJsG6tg2xi+S2/HtjHvudh6ZY68cx1hTnkP
xHtW5iCYNzeXVnOW+M1l33uyXpn78MKagF5ZKxZTawh8sbagS9YcbGsRGLNG
5T1Z7BvV2nW9Wq+uU/da16q1C3/ca12l1i78d69FjO61LlOvDS9V69Ular0i
n14bXhx1zD6Zenldf6Fary5Q69UFVbu/l+2anq+eg86Lmp4bNT1HvZd1VtWR
2p2pwVxDvc5Q69Xpar06Xa1Xp6n1ivfByDx1LmO8q8b0GpDr0r/MWj6jZyMq
b/Rcw6su9Axw8udqjfqz+ueRsF8Ke1rYM8J+pf4dVjm1dt1UvqCfJ2qwRsOn
b2qAO9ZvO1YuwTPzvO8F234mjnklzp0cY06La80IH3DIvmHPCHta2C+F/UjY
zgnnOVdol3OI5jm39FrOOVrqWtAHukZcx7VDr1xT9Mq1fkKNgcfU2Jigxswj
aiwxLxtj96uxRw6MSXoqYxVsG8Nr6O3YNuZXUXMBbJsjK6q5k/sezMvm2rJq
DoJ5c5OYzdml1FxGo8xxuGDuo1HWBN83d09lDYEv1pZF1JqDRlmL8lmgp+t9
YkWvblA/u8s47rV8r/aJuq61C3/ca12p1qvL1b0WcbnXIl73WuTBa0Py47X8
Req14YXqvSzy733I86Om56v1Kueg8RrMQX8v+9MVG3X32hBseC+Luc97WWCJ
OQi9OkO9935a1ZHanVr+U6NxVSPyf3LVYuGyrVesj6xXvE/N0atTqiaz15is
R2YpH1gjzlT+w+VhlVve4+dm9c/tYd+v7p1Ygw0ve5p6bThdrXVPq/uuX9W/
wysujofbe2igqfQizPkLVVzrVs5GVazkZELYz8Uxk+LcKTHm9LgWjtgHbPs2
I3yeEbHMiBg50LHzn9tjHOdqRuRwWuSWXss5Z23oWqBXrhF65doRl2v6tLrW
T6gx8JgaGxPUmAHnxhLYNsbuV2MPbBuT6JWxSk9lDK+hxvZqaszn2mFlNUdW
VHMHbJtT6JW5Bs7NQeYgc3OUmrPEby6jV+a475W7p7ImoFHWCjhiDclnfrCt
OeTaWgSPrFHWq1XVz5O417pW3Wtdo36O9yr1nvyV6l4LP91rXabWrkvVvRax
+1lfcuK1ofceH6kckqtc1z9YtXBNx6ufbaB27pnPU+vVuWq9OrdqSjznRH3P
ivqeqV4bnq5eG4Ix772fot7LAqsbV71OrHpRl+PVevUDtV4dp9YrbGo+Z9nW
K45nvThzjQmXh5cP7qnOVWvLFfUv///T+pfjeR5yRL3ob7yvPk29Pz9V3bM9
WO+p6uLx9ywbbsOtWeoc5t356jVP5WZU/Y6cTAj7hTjmjTh3WozJRXwt/mMf
podvU8PnKRHL1IhxesQ+I3LCf5yrGZHD6ZHbKZHzN6IWr0WNXoraPaeu6dPq
WsMFY+CfamyAc2MGvTKWHlBjDDwbe+DZmATPxip4NobXUGN7NTXmwbO5sJKa
I+jVeLVeYVuvzDXqYw7mPSbqa86CeXMZjTLH0ShzHy5YE9AoawW2NQSNsraQ
X2uOn1dMvUKjqPMNFR85v06tY9eotetqtXZxXfdal6t7Lfx0r3WJWq+Iy73W
Repe60L12vB89dqQHHpteJ5ar6iF97KwvcY/J+p7TtT37Kjv2VVT8kLdt63Y
wIN7LXDiXos50fPRyeq9LLC3adXx+PKT2h1XtaNGx1SNyP9RdR306gi1Xh2u
1qvD69jZyub38OdItZ4cq9aTk9U/59a/HHNtHQP/f6ved6IerGvo5aaqn4Fn
/QMW6C3Y25mrxtkvrrVxjA+2ZqrjZ6nx5q7fL1nxEAM1nRj2y3HMm3Hu9BgT
8fC1pocPb4Zvr4XPr0Ysr0eMb0Xs0yMnjO9cYTuH0yO3UyLnrFVdi9fUNXpJ
XTv0yjWFI641XDAGwLyxgV4ZM8zFxhLzrzFGT2Xs0VMZk/RUxip6bAzDSWN7
NTXm4bO5sJKaI/hq7iyv5hRzibmGXo1XrwHNTd8Tt16Zy+iVOY5emftwwZqA
RlkrqLk1hJxaW6irNQfbWkQvbo1Cr9Cxlcumvu61rlavE69SaxfzlNeJl6m1
y8+c/Kv8d691kfr50gvUekUevDYkP14bkkOvDcmz97LOUevV2Wq9OkutV2ep
9epMtV6dWTVdq+ztKkenR63HRa1PjlqDK68NmUM3q5qC89FVx6OqjtTriKoX
dTm0csW+7fdrPObssRpgAd04UK1X39OAIyPqfbgzrI73GvDw+pf3wT9agr6d
V8fDVZ5Xoc7zV758Lw9OGjOvVT7mqXzizyyV+znqGvOq15V+TanfoYXeWwKb
E8vHjSontueKY2aOc2fEmPzH13orfHgpfHsyfH4iYnk+YpwUsU+JnMyIXHER
53BG5HZq5Hxy1OI1dY1eVNeO5xlcU3xzrcG8MUBOjA3mYmOG+ddYAsPGGBg2
9sCwMQmGjdW/6u0YNrZXU2MeDJsLK6k5soKaO8y/5tSyaq4xF5uDxDlevQY0
Z9EocxnMm+PU2NzPZxHBm7WCPFpD/Ey1NcqaQ/2sRdjWKHJ+nfpz6H5e95ka
x9p1hVq7Llf3Wpeq14n46V4L/91rXaDutYjXa0Ow5LXhueq14TnqteHZ6rUh
OffeO3Vx/3xG1Jqafqxqffr/Umvsz5Tf9OHutcCJey3w414LXG1ZtT4mas1c
7LXhYRrMQdTx++q9rIMqJ+T8uzUevNy/zqPP+HbVE73inpr1au96Dw7vo95r
+nb9O7zGH1HHHVfnzl+1WL6ucUflk3jpRbar2sDzHeuY7dQagWahr97bdv9j
DZuifr7en4MBs9SUdZ6fvbE9dxwzc5wr9Zhef3KtSeHDo+Hbn8Pnv0Usj0aM
z0fskyMn0yNXXMw5nBG5nRo5nxy1eFVdI/TKtYMjrukT6lqDc2MA34wNMGzM
gGFjiZ7KGKOnMvboqYxJ4jVW4bwx/M4515hfRc0FuG2OgGFzBwybU8y/5hqx
mYOj1NykpuYsODeXqas5jl6Z+4uqNYHcWSvQKGsIGmVtgSPWnPw8CDiwRqFX
6NiKZV+t7rWuVK8TL1drF9d1r3WJWrsuUs99+O9ei7jcaxGv14bkxL0WuXKv
dZZ6beh7KA9Vzr02pBbeyzpNvT95qnp/Epu6o1fj1Ho1rmqNr6eo6w5Otqnc
MRf7PgtzouepI9W91mHqXuv76l7rwIqF2h1QtaMu366x4TG8YE6j9/h6HQuH
96zfw6vd1f3Vbmq92q3+hdt7acBt+jTWUGABLMFTsASGWPt/UQMcP1Mx7KLB
esbPKfLDfbsXysY3+AumWf/hxOj6He9vodarzeua1BQeUNMlw547jhkR50o9
5oi41nzhwy/Ct7PC5+silj9GjI9F7K9FTqZFrojFOZwRuZ0aOWf/yrWgh3SN
yI9rBxdcU3DuWoNnYwAMGxvg1pi5T40l5lljjHnW2AO3xiS4NVbBrTG8hhrb
8NOYX1nNBbhtjoBbc4d8m1PvvPdtDvq5He+rm7NolLlMXsxxNMrcR6OsCdTA
WgEXrCHY1hY0ypqDRlmLsK1R1B8dQ3fRq6vU2uXPGoCLy9TadYn6+zT8TO+T
5ad7Lfx3r3WeutciXvda5MGfqyJX7rXIob9zg9x6L+s0tV6dqtYr6uV7hdTU
89TJevs8Zb06uWqNf+x/frZyeoK6xz5WPWfRz29duWAdYQx8v65L3Q8s36j1
Aepe61tVU2r3n5Uf9rL2qpzD5d3qWNZIO9Xv6UO+WO+hV59Xrwc/W//C8c9p
0A/Qs32pro+f36w8fLziZa3C+obP9HFfn+eF6Tmo85drbPJKP7BF+c+6jJ4B
LcSJE+q6UytX1ivyA5b/Wce7v7I9dxwzIs6VesxZ4lpLhw/nhm8fDZ8PiVhu
iBjvjdhfjJxMjVwRi3M4I3I7JXL+etTi5agRfHHtwLxrCp5d6wlqDDDPGhtg
1ZjBT2PpH2qMoVHG3l/VmOTZ/MSqMex1gfXKmAcD5gJYNUfQK3PH97utV+Ya
+TIH0Stz088Zuqcyl9Ercxy9MvfBvDUBvbJWoFHWEDTK2pKfWcO2FlEDaxT9
89UVB7W4Uq1d6J57sEvV2nWxWruY67xOxE/3WueptesctXYR77pV3zPVvRa5
cq9FDr02JLfeyyL/vv9LXfy8CnV0X32S/ue++kS9HQPU+i9lf778oyf/bPlN
r75d5fcINR4OUePhIPU68Tsa9BPUHdyOrlrDHfdae1ZNqdGulUPrDByds3yh
nvB323pveF3Te+Bb1r/8PxicvWrq+z/r1bljhl5fHXp9TQNMMEddoH4Wk9rD
XeKgh7hNg3XKQ5XPN+v/31XH3VPXnVK5g9dowq/K/wnq76MbFfa8cczMcS4/
HnPWuNYK4cPB4dtb4fMnI5ZTI8Y/RuzPRU6mRq443zmcHrl9K3L+WtTCuuc1
oGsHnl1TcOtaM88aA8RkbIBPYwZ8Gkvg0xhj3WfsoVHGJP2Kseq51XplbL9z
H8NcWFHNEXhu7vh+t/eszDVwaw4yx5ib6JU5C57NZTTKHEejzH00yppAHq0V
4N8agi5ZW7CtOeDZWsT+gTUqP7+JfYVauy5Ta5c/J4V9kVq7LlCvE8ertetc
tXadrf6ejTPV32lGHtxrkR/3WuPUn1kgn14bkvPRevu9lQeqdt57p5cGo3eX
TX9xV9WdfYC/lg0e/lI2GABzx6q1i/nO2nWYWrsOVu9xfa/GJy/71XXJHesF
8AAG4NEmVevdKkZq+iV1r/X5yjNrQ3DH3EUfwNzL/glzP/Phf61lajx+4Bg4
gutgYP2qywcqf8z9R5Zfv6/cfkMD7nIMayq4CL6eq5qR8xmV8yll/9ce07De
Z4fX8Bleg5lnKu4H6/Wh+H/s+eOYWeJcqcecOa61dPjwpfDtl+HzUhGLc//7
OsaxPxM5mRq5IhbncFrkdnLk/JWoxXNRoyfVtQO3ril5c62J2xi4L/wDk8YM
mDSW0ChjDI0y9tivMybhjLG6lhrDq6uxDSaN+ZXUXMBvcyTvcTO3mlPk3Fwj
t+YguDU3yYU5i16Zy+DZHH+3mvv5eTTwaa1Al6wh+blauGDNATPWImxrFOtx
dAytBXuXq7XrUrV2oYdeJ16g3p/HH/da56rXiWer14lnqnstP/eLdqHn7rXI
j3st9N9rQ3LrtSE5973g49V6Rb28936c+l7hsWpsHKvWK89ffyn78+UH9hc0
0C7mux3KV+ZBa9dBapwcoMbJvmqcwKOPVh7pb6xdO2uAB+rOnvF6VdNt1fta
7GGAoznqPOoMl5nrvDZcR/2cFbkfUbVermr90aon3Lisxv5nHUOe6CnYY2bv
hc/AeE21b9X0kRoXLnvv289qct38zoSn631yAB9ZR32i/t/2wnHMbHHusBgz
99kXCh82Dd+ODZ+fi1i2jxh/GLE/FzmZHrmaETmcErmdFDl/IWrxdNTo8ajd
o1HTh6LW4NAYuFuNDXBozKBRxhIaZYyhUcYecRqT66ixupbe3v8b295rNQ7N
BeI3R8iHuUNs5tQoNdfoqcxBPwvtnsqczc9xoFfmOHpl7lN3awI5Ha/uqawh
YNLagi5Zc7CtRfDCGkVdrijf87Pn6NUlau26UK1d56u16zy1dp2j1q6z1L0W
cbnXOlXda52i7rXIDxjI5+smVm69NiTnXhtSF+9lUTvvYR6t3sNkngJzd5Vt
nByl1iv6DzDwx7K/WH4cXjbaxZ7DDuUrPPhcxbCfGjPMp+7J2d8wZnZVa9cY
DfA8SgNsWrvYt1i3avqRGmMedW8+e/2eusFr5iX3WkvVv6yjwAc42qKOhQ9/
qnEnqe8ZeB2/VdWGvB1c53HcjXXOjPqddQV8jKhj3qxjnqjfrV355GD3A7YX
jWPmiHOHx5jD4lpzhA8rhW87hs83RizOHX7dErG/GjlhYOdqWuTwjcjtS5Hz
Z6IWj0eNJkTt0Kgxao1yrdEoYwDsGRvMlcYMuTKW0ChjDBwae+DQmFxHjVV0
wRheXY3tVdSYJzfmArw1R+C2ucMcak4Rs7mWzz+DT3MTvTJn0Shz+d1qjpNv
c5+aWBPQKGtFfvYfXbK2YFtzqIe1CNsaRW99efmOfalauy5Wa9cFau0ar9au
c9TahZ9eJ56hXieepu61xql7LfLgXov8uNcib14bMkd4bUjO/WwwtfBeFjXy
XhY1ZZ8CvTpCfc8FDLDn8beyjRn6cOsV74+pa7N3yz4088rYeh//uH/t+e7b
6vmO9cf2VTM+Y+L7jMy/3quHb1tW7pivN6u84yOYARub1jWp+4Zq7WJcaxf1
8bNKHDdTvVhPzlpj0cfAmRnqPRvmUj/X8teKB59frPjwBS5znO+j8blgfuhD
2H+hD3m56rpE1euVyhljvlq5XSvsJeKYuePcmWNMfnytmcKHecO3tcLnb0Qs
wyLGX0Xs0yInMyJXfsaesV+K3D4dOX88avFo1OjBqN29UVMw5lqzZ24MkBNj
A7wZM8wjxhLPYoytOP+gxp7nTd4Hb8bqmmoMr6bGNrpgzMNJcwG8mSPw2dwh
p+YU8Ztr1MYcRKPMTX/WzD2VuezPya5eNTH30Shrgj/vb40ar9Yoawu6ZM2h
NtYibGsU9bqs/AW36Nso9fdmcAzadb5au86r8dErer8Vyj5TrV2nlZ9PV1xe
J56s1i7y4F6L/Pge4nHqXot8em1Izr02pBajNZjjDlfr1WHq+y/U3fdfDlHr
1SFqvcK2XmGP0eA7TrC5588cw1y9c9WPft5Y2letY9zvBrvginvi7sE41ved
uY57MP51D0bv4B6MmDatenAN5jtwAhbXrvoy/1HrOer/3avA2RH13vAab4+y
yQE9wLvqeq9U3cAjvIZz1ig/m+TP1vE+a7vbauznKr8rVF0eq7rcVbVkrl89
7FFxzAJx7sgYc6a4Fj/2Yabwba7weaWIZVzE6M8REvv0yMmkyNXzkcMnIrcT
I+cPRS3uixrdHbVDl1xTdMm1BlfGALpkbIAxYwZdMpbYdzPGwLqxt54ak+uo
sbqmGsOrqbEN34x5eGgukAtzBP6bO9TFnCIf5hrzpjnoz5fBTTTKnKWnMpcX
VnMcjTL3qbU1AdtaQR2sIWiUtQVdsuZgW4uYr61R6NWl6r8VcrFauy5Qa9d4
tXado9aus9Ta5c+qo1f0il4nEpfXiSeq14nkAR4/UfkBM+gV+u9ei3y612Lu
8NqQWngvixp5L4u+2vcKWUd47hurwdz3t7KZI/9SNrgCS2BmjAZ6xfu71PW4
H82+KnMPzwruWnXdp47BP/Zauf+9fh3Lnux69a/3xFhHeE/sk+o9sc3Vz6xu
op4rOcdz5Upq7LkP89zz/rJnLRs/6Tm+on6G3M98eg+M/1Bb1TGj1Z+H3rXG
eqPqAqbAAL3MqKrX7yoO8k8vy/oCvXpQ/R0+tpeLYxaNc+eNMWePa80UPgwL
36T2mR/H4hfn/TZifyRy8kjk6qHI4f2R23si53dFLf4WNbpTXbs/q2sKflxr
+iVjAF0yNojbmMFPY+l2NcbA1Vi1XhmT66ixuqYaw6upsY1GGPPw0Fwg/+YI
HDZ34Lk5RU9lrpEbc3BRNTfRKHMWjTKXmQPMcfTK3M/vJEGjrBXU2BpCTcar
Ncqag0ZZi7CtUS+XvUTZF6m163y1dp2n1q6z1dp1hlq7TlNr1ylq7TpJrV3E
616LPLjXIj/utdB/cINeMS+gARMr5/4cFnOK14bUiP72gaqd994PVN87PlCt
V99T6xU9ORi4o+wxGuCKY8D8r8ves64HxuhZmJPYl2CPlXqz/mBORSt2rddG
Nd5Xym/WBO7Z8AUMgw8w5bWn74mtXzn4Uh2zUh3DmIvXmPDfz57CmdmrBvg8
oup5WdknVl3mrLq8XrVdoOz3lU3fBecWrZqSW/chYGntyj+fIwbzzAt/rjz/
Xd1/rBH2ynHM0nHuojHm/HGtucKHkeHbyPB5gYhl+4hx44j9osjJLyJXv40c
/iFye0fk/I6oxR+iRr+P2v0uavrbqDW2MYAWGRu/UWOGGhlLXMcY21CNPTBg
TK6jxuqaagzDJWN7JTXmyctY9ef0zRHyb+6QS3MK/ptr9FTmID2VuYlembNo
lLmMRpnj1MXcn1+tCeTeWoEuWUOwrS3okjVnbrUWYVujwOcl5SM1vlD9N9rG
q//O0TnqfuwstXadrtaucWrtwn/vcbEX4XUi8XqdeIxau8iPey3y5l6LfHpt
yHwB1ydULbw2pEajNZjPqaP33nnuxfeR6b39rDs2ewvoFZixXmGP0QCH2NYr
jt+zrsF+KXsn8A3sfb3svcIGp18re0zY24e9ddgfqXPBEzhiv4O5ELywfti8
ckY8n6waki/WD2CFGrNvyvroz1UL9nBm1PmzV3zs7cxbvtJjLFPXYV5AEzYo
m3z4ucfvVJ7Jrb/zhHzyfUSsiZhT/1A2OKZHYQ14V9j3xDGrxrnLx5ij4lpL
hA9LhG/Lhc+bRCzHRIzzRezfjJycHbm6NnL4o8jtzyPnv4xa3BY1ui1qd1vU
9LaoNfauYe8V5xozt6qxxHWMMbTD2Ftfjcl11FhdQ43hVdXYXkmNeXJrLpAz
c4Q8j1WvAc0pOG+uMU+Yg+9Wc3NhNWfJpbmMRpnjaJS5Ty2sCdTKWoEuWUOw
rS3o0ngNao5tLWL+skaxJ3px+Yh2XVDvY5+n1q6z1dp1hroHO02tXaeotetE
tXb9QP13J4+puOi16D+9TiQ/7rXIm3st5gX3WvTGXhtSC3+GlBq5z6d23ssC
D95zYD/Eew7Y1iue6/N6EHuMBvMjNpj5Vdlg7hd1LpjbtP7dp2x+v2/Zu5b9
oRqP88HN9nU+cx6YB3+b14u5c8uKY2z9HuwcUeeBkZNqbPB3UR3H/HerBs//
ghX0nmeN/ut7E4YN1ivzqj/zxPHnq//+J/6zxwL/6Wn+VvnYrmzfz1q76ru9
/vvfOCN3G1ctH6083xP2o3HMw3HunTHmmnGtNcOHdcO3D4bPO0csl0aMIyL2
T0VOMldnRA4vi9xeFzm/OWrxo6jRT6J2P42a/kxd65+rMYC9V9jGzAfVWNpE
jbGN1NhbT43JddRYXUON4VXV2F5RjXm03VwgVnMEXTB34LM5BefNNX9OHw6i
UeYmPZU5i16Zy+TYHEejzP151ZqALlkr5lFrCLWytqBL1hxsaxFzjjXq1arb
ImWfr9auc9XadZZau05Xa9c4tXadpNau49X3GY9V/91J4vU6EVx4nXioutc6
WN1r+TOkj1ee3WuRf68N96+aP1S182cf3JvAHTDCvgR65fvLd5YNp+ET8/EY
DfQK23rFMWAp8faRsg8oe6/yieuBVbANbr5Ufm9Z1xlbfsAl+vKt6zjmMPp7
eixqz/7seyvX7GusWTUjVjDy48o9+PtH+bmo+rtbwBmc3axq+3z5AXZ+XXVe
s+rKswHoLHMtvNuhcnerugfw8/zs5eZ9arBKL+XnnF5T/30c28/HMfncy70x
5ui41ofCh63Cty+Gz4dELD+NGInZsW8SOdkjcnVE5PD0yO2FkfMroxbXRY1u
jNrdHDXluS/XGn0zBriGsYFtzOR8t4kaY+iVsbeeGpNrq7G6hhrDq6ixvaIa
8+TaXCBP5siSau4sruZUfjafnsocpNbm5oJqztJTmctolDmORpn76JI1Adta
MbdaQ+ZSawu2NQeNshZhW6NeqzotXPZ49d/DPUetY2eqdew09frxFLV2najW
rh+otesY9d/4PlK9x3WYWrvAo9eJ5I19PHotau5ey58npdcC214bMu/Q3z5c
tfNeFrX25yDo6+EFvIETYA69oje3XmGP0UCvsMHcbWXT2/+8zgUTP6wxrUX8
fqwGuNmt4mHO+3LFyTru8xU/8yJYO66ujV/sV8LHD1X9dquYLqnr0WfAj6Mq
N+yjXKy+R8eaB1yyF/XvvfRhgzXFslVb9lnAN2vlIyp/cIj5jTUE+KDf+FLV
ktzBm/1rDGIB/7NWfOCLPW76Gu7fgRuefQfnT4U9LY55M859OMb8SVxr+/Bh
5/Bt//D5rIjljoiRH8f+nsiJn08nV/6MMjk8JnJ7euT8/KjFpVGjq6J210ZN
b1DX+kY1BhjX2EDzjBn0ylii7zHGNlJjbz01JtdWY3V1NYZXVmOb+Ix58mEu
kA9zBK6aO/l5fHhurqEF5uCCam4uoObs/Gouz6vmOLpk7s+j1gQ0ylpB7a0h
aJG1Bduaw97kePV3tVmjJpWNjlLv89TadbZau85Qa9epau06Wa1d6Co6iV4d
q/5bk0eV//Rah6u1iznKe1zU0+tE6gkWn6p8utcCZ36+lDp7H35v9ee2qN1o
DfZbqK+fkwEL6Ap6xT4B+nFn2fADvWIORjN+W7b1ao86/2d1PLVHr5gP2Re4
vq7l52B2r1qC650qF9znpmfyZ8BY6zDfjNFg74Fcg3H6EfjyzYrj+soN8+Gt
6uf1iAO+eA19V+WMZ8+5jw++4OwqdQw6Ab7XrVySJ7DMmviI+p17NnTjivKf
94+pOuEPHOZn68Hw//6bWGCDPSR4BH7fCPvVOIaTfO6DMebVca09wof9w7fj
w+drIpZ7IsbhEfvqkRPfs3yyaugcZm5/EDk/PWoxPmp0cdSOZ4JcU/x3ramL
McB5xgZ6Zcx8SI2l96sxtpEae+upMbm2GqurqzG8shrby6sxD8fMBfpKcwR+
mjtw2JyC5+YaWmAOLqDmJhplzs6n5jI1NsfnUXN/brUmoFHWCjBjDcG2tsyp
1hxsaxHP7Vij0CvWiQuWfV7YZ6l17HS1jo1T69iJdS206wdq7TparV1HqLXr
ULV20XN6j+t76j0u8uZnlz0XPll5dq8Fn8Era0N44bWhP8P1UNXUzx5zT4Z5
mvUAWIIzPDODRqAl6BUYsV75/o73TsHBT+t4fLqlxmEeYr2wR8W5bdnkwveO
6Il3qLGZV+gdmCfPqbHh48UVB7gD7/Tsm9U1ySWY/kOdQ6z0j2jmepUz34uH
s2+V7T6L/Qn2tD5YuaEOX6h8uCf1s0IHlZ+/rJjJ3zXqZw33Vf/devK/dF2H
8aeUz+jWzGE/Gsf8K879WYx5Ylzr4PDh2PDtgvD5FxHLhIhxlojdz4SQk49G
rnaKHFovyO1hkfMTohZnRI3Oi9pdFDW9LGrNWtIY2FqNDXw3ZriOseR7NOQ7
7y+vp8bkWmqsrqbGMDwxtul1jHnqYi7APXOE2pk7i6g5tbCaa/DfHESjzM35
1JylhubyPGqOo1Hm/lxqTXiXWivQImsItrUFXbLmzKHWImxrFL3y+eUXNutH
NBW9OlOtXaeFfbJax05Qa9exau06Sq1dh6m16/tq7TpIvT//HfUeF/0qPecz
lU/3WuTZvRa1ZV5gbcicBV4nqJ81eLhq6r0s6s66nr13cMW8h16x3rBeYcNj
9GontV7trNarncu/m2ucsRpwbNeK+bK6Lrjdsd6nHmjUlyu/vId+gXcwxXzN
OgO8gMFbKk9g8FdV583KX84ZrX6mYOPK370V91tVR96Hs9SJnoSeC4x+tGrI
WnWbyuuXK94Hqs47Vp78fUj3VVzU8J+V85kq51yLfmH+qjXX9P0d2yPjmCXi
3O1jzK/EtXYPH84N324In/8UsTwTMY6M2FeOnGwWudohcuh+5K/lo3N+VNTi
pKjRGVG786KmF0att1dj4FNqbKB/xgzXNJber8bYRmrsrafG5FpqrK6qxvCK
amwvq8b8KDUXyLk5sqiaO/DWnAIn5toCag6iUeYmGmXOolHmMrU2x+dScx+N
siagS9YK9Mcagm1twbbmjFRrEbY16o2y5yv7nLJfr3OtY6eqdewktXYdr9Yu
enl0Eu06Qq1dh6r3PceqtYv1sp/v2r/yQK/FnMdc8Wzl070WeXavtWfVkLXh
V9XfPevnnh6pmo7WAOtggP4eDoAx+Mr9L3QELIL/MWq9+mKNdWu9z7V/Uja+
3lTnUjOvBYj/krou67bxNQY97lfq/QvLd8ZnvUOfTv9wY+WD/oy+g7UEfPl9
1dPPkbEGAZtwnB5js8rfvWW/WTlnD4N7+nPVuTMqj6zx4O/7K9aX6j18ove5
vuJ6svLwkaonGgNP0AH6DNYBp1a9OHf5yvnfK+fLhD1/HPP5OHdajLlRXOsT
4cPV4dtt4fN9EcsrEeNcEfvKkZMPRq4+EzncPXK7X+T80KjFcVGjU6J2Z0ZN
x0StP6fGgPcEwMbH1Jj5sBpLo9UY20iNvXXVmFxTjVVyZgyvoMb2MmrML6Xm
AnwzRxZRc2dhNafgs7lGvczBedXcRKPMWTRqrFqjzHE0ytyfU60JaJG1Atsa
ghZZW0aqNYe+3FqEbY1ibT9evedwdthnqHVsnFrHTlTr2HFqHTu6fEC7Dldr
F/VBV8HVgWrtouf0Xv23Kw/Mf3ur97joY8E5cyhzBBhkfgVnzAv/qrrQ94M/
5iD2Eh+tmvozX2M0wCvrJurPvjh6xTwOLtGrHer/4Sm8QmNuLRs/flLHUL8b
61jW81fVmOTi4rrWqZVvsHRe/T/jXVZj8T5rjf3qXO4p0Wuz1ritxrJf9MjM
0/fVteDz4+XbVpU/4gK/b1YO4R2cnaXGca+Dz+jEOpW31ysv+9Z5xP7Vqhu5
3a6OX7aOm6Lu/bwm8714xmdvib2Rx8LOZ0IOjHMfjjEXi2ttFD78NHz7S/j8
WMTyVsQ4X8S+SuRkdORqm8jhLpHbvSPnB0UtjowanRC1+2rUdKeoNXgZUzXf
Ro2NrdSYQa+MpdFqjG2kxt66akyuqcbqKmoML6/GNppizNP3mAuLqTkCD82d
hdScml/NNT9X6/1zcxONMmfnUnMZjTLH0ShzH12yJoxUawW2NQQtsrZgnxG2
tYjn6caXPblyPHfZZ5eP2KeXjV6dotaxE9Q6dqxax45Ua9eh6h5srLoH+65a
u/ZTa9e+au1in3CUBhgln14ngg/636cr/8wzT1ZdmC/AMHOQn3WkpvTD7GVR
d+7J0Icw//l5bDDJHMj9te3rOJ4XZQ62Xn2mavnjOoaa3VDn0gdfUWMyf1xY
Y5B3dIa5krU3/S9YurJyxtg3VT7A+c/Un8u/veoJblmnnFvXRXdZe8LrJ8q3
7Spn/yh7cuUH31gfzag8wOUF1J/RWbHyDNfB9PeqPuge2vJm1XT3OneV8v/N
im33yuXmlXvWDLPV740l2z+KY3aNc++JMUe+41r24ffh2/3h83N6++eNHOPC
EftqkZPRkatPRQ6/HLn9euT8gKjFoVGjfaJ2e0VNd4laoyvGANcxNj6uxsxH
1FjCN2NsQzX23qPG5BpqrK6sxvByamyPUmMeLpkL9DrmyMJq7oAHc2o+Ndfm
UXOQ+pmbaJQ5iy6Zy+jSWHXvZO5TU2sC+mOtwD4hbGsL+LHmYJ8dtjXqzbLn
KvussE9TY+/ksI9X69gxah07Qq1dzCHuwQ5S69h31Nr1LbV20Yt6v+vrlR96
d9bgXifuql4n7qzutcaoe60d1d/3CPaYr5jHqTt7HMyj4IG9D+4PMV+BX/SK
udB6BS6sV9jWq09XDNfX8WgMazRwicacX9c6q3IPri6u+oCla6uGYOlHVWdf
56iK6w9VQ45BU8drwIWH6nxitF5hv1THfbHqQx7g0bT6f/Zp/dnBr5dNH39A
HePP+FPzCRrgjb1rOP+9Oh5Mg0+0Ivsc5vEVahxe6Dg93Uxh/zOOyb7u/hhz
eFxrVPjg79bAt8fD59cjltkixsUi9jUiJ6MjV5+MHI6J3O4ZOf921CJrtF/U
7htRU67vWjOeMQDWjA16OmMGvTKWRqsxtqEae+9RY5J7gsbqSmoML6vGNj2Q
Mb+YmgvvVnNkQTV35ldzCg6ba3OrOYgWmJtolDk7h5rL4Mocxzb30SJrAra1
gnpZQ7CtLTwnc1rY1iJsa9RbZc9ZNsfQ31mvbJ+s1rEflA0Ojlbr2OFqHTtY
rWPfU+vY/mrt2letXd9Qa9eeau0CB96fJ8/0qMytYAYOPK3+W+fgD8xw7/ex
qil7jI9UrdmzgSPgAbzABbDL+uEv9S/no1dgiznz1rLx70dlw7Hr6njqcWmN
Q47OrfH5lz4XrKJnftaKdeSh5fPPaiz4xWfLmLvBD+uRU8vGxwvKF3T32rLp
K39ZNrm5s44n3xM0wN7UyiFzJX0ImgDfvG76fr0PRo9X379jbwG+83NCHU/e
T6ljJpff/l5i+zMy6pu1Tp9ni3MfiTH//f3wpTn24dHw7YXweXrEMnfEOCpi
Xzty8oHIVfr8hcjtVyPnx0YtDo0afTdq5+c3b6xrutYcawx8Vo2NrdWYoS81
lkarMbahGnvrqDG5mhqrK6oxTF2MbfTFmF9UzYWF1RyhHuYO/DSnyKG5BrfN
wTnV3ESjzNmsNVpkjmOb+9TamoBtrUB/rCGzqrXFeoXmMM+dFXZq1LnlC/aZ
YZ8ax5wUtj+rhnYdFfZhah0bq9ax76p17Ntq7dpbrV1fU2vXV9Xa9ZXKG/Mi
OPAeFzhjbnm26sKc85T6u1uYi5mb2GOcULX25+DAgz8Ht1X9P3xnPxR8sQbZ
smp5qwb7LdarLSuea8sm/otrHPTp7Brv/MoZ172qjgNXP6zcfLbGJh9giXmc
OY11B+uRXDd5j/6Rui7cgGv+rAcc9PeWTFJ/H++Uyg+4ml61A0vuSY4pm7yf
Vscw7nj1WvL8Ogb8o8toiPdFef+5ioHj76nYyeMqYS8Qx7we5z4RYw6Lay0S
PjwTvk0On2eJWBaMGJeP2NeNnPgzU9Yr53DHyG2uc0+MWhwVNXpn7VzTr6lr
vbMaA/Q7xga/N2Y2V2NptBpj71Vjb201JldVY3V5NYZHqbFNz2zMwx9zgfyY
I3DP3IGT5hS8NdfguTk4h5qbI9WcnV3NZc9B1iVyZF2yJsyq1gpq94OwrS3Y
p4ZtLcK2Rk0pe2TZZ4Z9atgnqnXsuLCPVOvYoeq/G3yQWscOUOvYvmrtYk5E
h8EwmKGHBFu7VX7YbwUzvs/4xaoJmPt81er5qov35D+t7rWYi5ijJlat6ZPp
+8ED94vgFz01mACXH6mx0KsP17VvLRss/bBs5pJr6njmDO5tg7/TKn9b1HvH
13Wvq7rhD/vH9NHgir0Rem3wyHp094oRHWUOZI59sH6/Z8VxbZ3zRI1FT4Z+
wyXwg97fW9fwGgpeT1M/ZzKjXsxr8J2anav+Tjs4N73qfl0dAy5/VMdQ15vL
pk4X1PG+p0Vezwj7mTjm1Tj3hRhz5rjW0+HDa+HbsPDZ9815f/GI0Z9dJvYN
IiebRa4+GTm8NHJ7XuT8tKjFD6JGh0XtvhM1/c+oNbw3BljLGRus5YwZcGcs
jVZj7L1q7K2lxuQqaqyyfjOGl1Jje1E15qmLuTC/miPwzdyZS82pd6m5BrfN
wZFqbs6u5iy6ZC6jReY4trmPzlgTsI8M2xpC3U8M25qDfWbY1qipZc9e9pll
g9VxYZ8Y9rFqHTsi7EPUOnZg2PurdWwftXYx96HDaNdX1dq1i1q7vqTWrh0q
n6xxWLPT/4I/8ENP+6z6Gbqnqo7cV2FtCH7Ye2QeBQOjNdjL2qx+x71C5mB6
ergPFph7btVgLWG9+kDFdlUdQy7g4ocqd+ASvF1avwOr7M3CJ7DGGu5b5SfX
AQvM0eyZgBfmwQfqX2KfoAHOwQ/82rLy9kxdc7/Kwe3l16s11vcrx8R7ZNWW
/LDmml45d5/DOvHissEZ+04zqhY/r/ep8W/D/mPZHPersllHf7d82DDsYXHM
63Hu6+8Y09d6NXyYHr7NHj4vGLEsEzGuEbG/L3Lys8jVzZHDayK3l0TOz4la
nBI1Ojpq588+UtN9otbMO8bAF9TYAKfGDNpjLI1WY2wDNfbArjEJpo1V4jWG
l1Bjm37ImF9QzYX51ByBb+YOtTan5lBzjVqYg+Tc3JxNzdlZ1VxGf8zxWdTc
R2cOCdtaMbNaQ6xXfsbYmjNCrUXYqVHn1vWxzyi/pta5tk8I+9iwjyjb97mt
ad9T69h+ZaNd31Rr117qHmw3tY7tpNauL6i163Nq7QI/zCfeP2Weeb7qxfzD
HA0mmF/YkwE/7D1O0AADPDvIXhY4ofZg8f1VVz6PtknV79Y6ljrdUjb1uLKO
YZ4YX+eyFjylxuSeIRwCq/QQ363rsAfCHIXusGfGXAyu6InADthkz4a5ccfy
3euDZ8p3NA7esQYAM/RN65df5GntuvYbFePxlXt6LfR0mvo7Y2dU/tFfa9eP
ygZnvytblSNs9tIfKnuWyik217u57NnDfiuOmRbn8vKYk+Ja08OHWcO3+cLn
xSOWFSPGOyP2OyIn3h8kVz+PHN4cub06cn5R1OLMqNHxUbtDo6b7Ra0Z2xgY
o8YGmDVm0BhjabQaY+ursbeGGpMrqrE6So1h1m/GNr2xMQ9nzAV6IHPkXWru
UF9zCn6aa/DZHCT/5ia2OUvdzeWZ1RzH/l7Y1gQ0x1qBfWzYJ4Q9LuwzwrZG
TSt7lrLPCHtc2CfE8ceo9e3wsA8O+7tqTfuWWsf+U61je6p7sK+odexLau3a
ofKNdn2m8kl//6mqD/j7eNWNdSI4YP/42aojdWafgjmLeYd7VWCA+f/Bqjk4
Y4/1fVXXP9bv6ZV5/oYe3XqFzbx6RR1zfOWPfpz1BHMFOsK8fWhdi7mddQkY
Y63J3gm44/7kGA3maDjs+z+Pq/ctrFFjKj58BD9o9rqVT3LD/aPvVM6YUw+r
vC5btSPf4PXsqknuHaH57mc4n2ejrCd/Kpu6PlA2OHyq7GFVA/daz8f7z8f7
PmZKnPtmjMmPrzVb+DBP+JZ7aBMjlocjxvsi9rsiJ3+KXN0eOfx55PamyPmV
UYvxUaNxUbujoqbg3bUGK8YAfZKxgf4ZM+DUWHq/GmPrqbHHHrsxyVrOWF1S
jWF6I2ObnsqYn1fNBbhkjsyp5g48NKfIublGrc1B+G9uUndzdmY1l9ETc3yE
mvvY1gRsawXz3TFhnxC2NQf7jLBTo86t61uvbI8L+4Swjwn7sBhnbNjfCXtf
tY59Q61jX1Xr2C7qHuyL6r+jgv77M7XbqrVra7V2Md/RC79c9fI6kTrCV3ot
5izqD/7AALgA5+CEeYt9ivWrrn+q3zPXso5BF9hzvKVsMHZ52ejTWXU868Lj
aswbKh9g9VYN5iJ8+HPFim9wC36wjphY1+YFn+HBNhUHY4Al+MXexM6Vg1XL
L/Lk7w+xLn2/crxo+UQdFqz60r/Qq7D+QQfo728sG1zepv5bgOzrWWceKZtX
6tJbfj9+n68Z7zjG584cY06Ka70WPrwcvr0QPj8bsTwRMU6M2B+KnNwdufpL
5PC3kdufR85viFpcGjU6K2r3g6jpWHWt2ZM2Bqi1sYE2GDNbqLG0iRpj66qx
R32NyWXVWKW3NIYXVmN7PjXmqam5QE3NEThm7sBDcwp+mmvw1hycWc1NNMec
xTaX0RNzHNvcn0mtCdiHhW0NsV7ZHhf2GWFbo6aXPaLsM8IeF/YJYR8d9mFh
j40xD1Br2j5hf12tY7urdWwn9bpyR7WOba/Wrk+ptWsrtXaBJe5ZgDl4zpxD
z8AcBHefq/oy74Bt9ghYM9Hrgw/mLXr9ddS9Pnud7EuiV8xd9MG3qP8G4WV1
zEmVM8a7uHLznjqWuoE35nPmJeZD5mX6bjCGXn6yfMYv9ifo558vf7eumNau
PLDGYT+ZuQ/+M3d+Va1L+6h16aCqAzkCG9af08om16x50Afqd33ZSMqtZXPc
nWVzDWvL8Mqp127/1qJ8xXvD45jpce6IGHNYXGtG+DA1fJscPvszrTOq1o7x
2Yj9icjJo5GreyOHd0Zub4+c/zRqcW3U6IKo3SlR08Oi1vQaxsCuamxwHWMG
vTKWNlFjbB019lZWY5JaG6vU2hgG98Y2tTbm36XmAlpjjlB3cwfumVMzq7mG
bQ5SI3MT3TBnsc3lmdQcp9YHhD02bGsF9tFhnxD2uLDPCDs16ty6JvaZYY8L
+4Swjw77sLAPCnv/GH/vsL8W9m5qHfuyWsd2UOvYdmod21rdg22h1q7NKv9g
eLQGfH2l6kgP/ELV170WdWc+Ym0INsAc6xL2C+j176zfU79fazDPsYa7uWzi
vKRsMHtqHY+GHV7j8Nkd5hwwxT6K59n7KibmYu4DwAkwBs/+o2J5RX0vGx6B
xW0qbrC4Y+XGzypbl/wMEpjzM07k98iyh5W/3ke6oGzmgmvLRv+9FmMcr9EY
9/6wnygbLL1UNnV9suypYU+PY4bFucNizElxrZHhw6zh20zh84yIZUrEOCli
92clZqg/E4EvEyOH90du/UwLOf9N1OKHUaMronbuq6npMVFr+GoMoJHGBv2L
MbOlGksbqzG2thp7K6gxyRrQWKXuxjA9lbHN+s2Yp3ajNeDCbGqOwDFzB2yY
U9TOXCPP5iC2uUmtzVlscxl777D3D9uaYL2yfXTYJ4Q9Luwzw7ZGUcvzwj6z
fj9D/XeSZtSYto8O+7CwDwp7/7D3jvH3UmvaV8Ieo9axz6l1bJvKFdj6eOUQ
fG6u1q4PVv7BGbVn7Q5u6Z/ZS3y56kvNn6u6owH/qprzO9YO3DNmTuJeIXtA
9Mq/qWOZV5gvV6g4Ly4brTq5jmdf+OAah3tGzD/sK7HO+UJd95GKA5yz1oAH
YAxuMZfS779e1/xoxUdP/8nKxwKVH/JEn7+T+nOC4Me9zf5lT6kaWYtOKhse
nlfnsndztXpt+GP18wO/qeNnr7xYl7zvNExv33u/suy3wp4Rx0yNc9+IMWfE
tV4OH+YN3+YKn0dGLDNHjLwc+1uRk9ciV89FDh+L3N4fOf9L1OK2qNENUbsL
oqYnRq3BhzEA3o0NeiZjZgs1lv5DjbE11dhbTo1JNMhYXUCN4bnV2EajjHnq
ZS5QF3OEXJk71NGcgofm2nA1B7HHqPXHnB2m5jL23mGb+8PUmjBMrRXYR4d9
Qtjjwj4z7NSo8WptOUv/u14NU+uV7cPCPijs/cP+Zth7xpi7hP3FsLdX69in
Klfg6WNqHfuwWrtGV33A2UZq7QJXzD+vVn3pjem1qDv3dsEcPTdzGOsF8ENv
zt47WAHHv61jmZtvKZs4Lyz79MoN8xt9AHM7e6Nwj7kI7PEM0rbq/TP2TNcu
X/CR+XFSHbtxxcEexWYVKzFspf7bp8yD1ooxZZOHPcpmrG+VDccOUfP0ePVz
oeeUzXx9Wdnk1M9B0bv+tGxq8Ouyqc3vy0YHfql+Tp59oanqv9WOPWccMy3O
nRpjzhbXelNvfxbLvi0SPi8QscwTMc4Rsc8cOZkRuXojcvhC5PbxyPl9UYs/
Ro1+FrW7Mmp6etSa+yzGAHs8xgY9nTHj5xLA0kZqjDGesUc+jUn2nozV+dQY
BuvGNtgw5mdRc4E8jK7fUztzB9ucgnvmGrgyB7HNTexdwt4zbHPcemX7oLAP
C/vosE8IO/XqrLDHq/VhvFpPziqb/5wa9olhHxP2YWEfFPZ+YX8z7D3C3iWu
+4WwPxP21uHnFmod21StYxtXfeDsBlUXML9O1cv3z9hjpK+g7ugB/T24Ames
Dak/8xP7G6OqlnCLuQvuoVdLVA0uqPeZA7g3yzqN+0twBZyxLhijAfYeqrG4
DveeNip/0FHmT3+nHD5tWDHBwQ+oe6ctKmf87tNlE9eOZdMj7VbHc419Kk/0
kGPVe9HsS0+tWM+snIFvP5PJzyU1Jpy8Lt6/sd5HW7ynNFeNwzFghHUE98W+
HvYrcczkOJeXx5wvrzXs7T7Yt0fC56UilsUixoUi9nkjJyMjVzNFDr23YG13
zh+PWtwTNfpt1O6mqOl4da3hojGwjxobYNyY4ZrG0nvVGAObozTAy5JqTLKu
M1bnUmN4pBrbaJAxjwaZC8RrjsAlc2eYmlPY5hq2OThMzU1+dgl7j7C/GfZ+
YR8U9mFhHxP2iWFbc6xXtsf7mGGDZ1psnx32aWGfWONZr2wfFvZBYe8X9jfD
3iPsncPeMezt4rqfUGvX5mGDK3ILLunVyTnYYn6xdtFbW7vQBerMHEndwTUc
X7pe7KmAD+ZO1itggXkIzLG/OUaDfQzmsQMrf9jki+dK4AzP3vhzoqwLqPko
DdYc+LtcXZM5ErxNrmPXLN/p7dev+Ihnk8rBm+ULNmulrdW65HUN/NqlxoHX
7Nt47cXzMPAS3h1d+cCvE+oY8nKy+lksP5Pp55D8eZnz6v1F6xjeZ99538r3
k+XnmpUb28vGMU/EuTNizMXiWrOGD6+EbxPC51UilhUixmUi9sUjJwtFruaO
HM4auZ0WOX85avFY1OjvUbtfRk0vj1qfqMYAvYaxMUaNmY+qscR1jLEV1Njj
PWMS7TVW6SGN4VnV2J5ZjXnwYy7AE3OEOpo72OYUP58Ie7uw/81NBWcVXFZw
XMF9hSYotEKhIQptUWiOQov0do26wAcPG/Tbtk8P+6Swjw378LBTr/YP+/9G
r3YIe9uwPx72R8IerdauDcNmHrR2Me9QQ3jCWp86gjMwTf/CHAkewAXcB1fU
Hr7Tf7O3xL1qcIF/YI55jjXfuWXDN+6fs6Zij5j1PTikR2OOBHusacDOKA30
kn/BG5yCO6uq94vWqfjonTYqm3Ucc6J1aSu1Vmyvfu6I/Rk/d/T1ipV9GO5T
vl5jH6b+W6mHlQ/M0UfUtdCHY8sffDytxvfnvzjm7or5zfJ9y7Lpkb6h/rtI
tu+OY1aNc9+MMR+Ma80VPrwevk0Mn/8SsawTMa4esa8UOVk2crV45HCByO3I
yPmMqMXLUaMJUbs/Rk1vilqjtcYAemRssG9tzGyuxhKYNcbIl7HHe8YkPZOx
Opsaw+Db2J5Jjflhai5gmyP8jA7735xScE3BQQU39X+nV/uHnXp1eNjHhn1S
2KeHfU7YqVEXhn1u2GeEfXLYx4V9RNhjw/5/0attwt4q7M3C3iTsDcJeS61d
1Nk9GPOgtQuugk3wDR7opZkjwRXzF3MkOGZc9oPhIfPNj9XfD3u2+nnr79YY
7FcQBzhk/wbMgT3WGZ434Q5z5TLqddZK5TtasWa9/2RdH9/Rzw+U7/RI9PBw
nfkbXLEfA3fGqJ8TZ28BPWZ/iFr4nsN31ZrJ+/BuVvUeNdp4YF2X8f2sOHHs
Xe+z5tmx/KHHQaNvrnHZj/b3INn+VhyzXJz7VIy5VlxrgfDhzfBtQvj8p4hl
44hxw4h9vcjJWpGrVSKHy0RuF4mczxW14Nqu0YtRO/fh1PQ2da3pE40B9rKM
DfBuzHxEjaU11RhbQo09cGJMgldjFRwbw8PV2B6mxrzUXJCCIwruKDil4JqC
g/p/06uxYR8R9nFhnxz2GWGfG3Zq1MVhnxf2mWGfEvYPwj4y7IPDPkDdv+0d
du5f7Rz2DmFvE/bHwt4s7I3jWuuHvWbYK5UJtsCYtQtMUF9wBsbYD2bfeIH6
f3gEVli30RvQAzH3sB/s7yw7o2xyt08dA4e2rnPRG/ACbt+ofxep6/OzVPkI
/pcvH9lDYx8VPtIvrae+z75JjcPn1jYvf9Ei9mHQOdYm7DO8WH6SZ54dYg3L
M5Ps1bEP8p91LvnYS/03GrwvzXqNe2r0HrfXuf68z051LrlzH0Id4AgasXCd
QwxPh/3rOGZKnHtHjPm+uNai4cNb4dsj4fPvI5aPRoybRewfiJxsHLnaIHK4
duR2pcj5klGLeaNG+OHaPRs1/WvU+jo1BugjjA32042ZzdRYWlWNsXersQcu
jclZ1FgFx8bwMDW2pcY8P2uGvX7YG4f9b04puKbgoIKbCs7q7ftXe4d9QNj/
1gSFVig0RKEtCs1RaJHerlGXhH1+2GeFfWrYx6t14yh1P/P9sL8TNnz2nnne
H9w57B3C3ibsj4X9obD/I+x1w5/Vw14hfB5VJscvWsf42UrqzvoBnIAb1jJg
iTUE65Q569o8/zxHxTOubPYtdq9j0LbRdS7zLJiar66DrwuVL3Bp8fKFNR14
g8f0cvAazNMjsR6hT7i14mVMnulhb4Mx2P/nPhJrWXoZ+ha09tryCe1hT4L1
EdyBl+w/04ewhviSBtz0ZzG85sKGp6x/2f+h5/pivfAXzYBPfy9fZip/+Lk9
8m/7J3HMyXHu+THm5nGtpcIHfyaLge4Pn38VsVwfMX4yYv945GSLyNWHI4ej
I7cbRs7XjFosGzWaP2o3PWr6WNSaehkDrHGNDfhtzGyqxtLyaozNp8YedTEm
yZ+xSk6NYX5GqX9WCNtc4LVu2ObOMDWnsD8W9jZh7xD2zvqf7w/uE/Z34lrW
BH6OCvv4sP+tLQrNUWiR3q5Rl4WdfVfuZXkfzPeBbOezWIeE/V11P8Neq+/r
gXU/Z7VL2DuGTa783EI+w/Ah9fPz7wv7PXHuaurnNJYvH3B2STWP3h1xza/W
Ln/unPUDWAKL9DzsDdCb0PP7M+onls39rx3Uf7dqtTp3co01d12HuXj+8pF9
DeZl1iP0S6M0mOvZ42B+pB9Ai5j7OY/5mp4EPl5ZeWAfi7ptXT4y//B8zwQN
agiX6Ue2L5tjWIPQb6AJa1ae4SN7I76nzzlwG27ybCQch4sHVb3o49BG9OXQ
im27Sic5fEr9GQrb/PiYj8e5B8eY28W1lgkfXg3f/hY+/zhiuSxi3DFi/3zk
ZPvI1baRw60jt1tEzj8YtdggarRy1A79cE35f9ea52GMAdaGxga4MWber8bS
kmqMzaHG3kxqTEqNVX7+jWE1tnkZ87zMBWxzxHvvrtGHws5nG7YJe8c4d5ew
9wrbHMcHcx/7kLDzmau8J5h77LlndWHYqVFXhH2xmtfnqXXpjLBPVuvSsfqf
n3U/UK0h31JrDn2+n2P/ivpZULDnZ9rBkz9jSA792ZwPhf2+sN+j/q6JVetc
xl+2xsw1ID4vpNaxeSP2OSvGKTUe+wPP1hhg97c1xlcq99jsyWxex8Cdeevc
GXXuu+r6f1N/3yr8ZO3E/Mv9fPDG+gktYn6Es+AcTQHn4zWY+9k3op7MzfQa
9M7wH/1jn5L7X+xvUX/WHszpX64Xey1gEF2Ap6wJWEfRn4D9TdR7ZRuov6vj
P2p81m2fKl/oM+HZPuU3+6Wr17j8P2uPxcNeOI7ZNM49J8bcNa61TPjwVPj2
6/D5qojl3Ihx94h9t8jJVyJXO0UOx0Rud4icbxu12Cpq9IGo3ZpR0wWj1mic
MUCPamyYB2BmIzWWFlRjbGY19vgxJqXGKv9vDOfaEO4Z83DPXOB35gj2+8I2
pzj+Y2Gbg9jm5kxqzmKby8MjNmxzH9uagM/Hhm0Nwba28HNe2BeHnRp1lVqL
Lg37fP3vz5Fal36g1qUj1Lo0Vq1F++m/f94ZTdhVrTmsC/xdDdupv6sBrPg7
Z8itv7eBnM9dNrWYp+xVyn5T/blP7MVqHK4FPkaqe6pZ1M8aOvZZy55UMXHO
nRX3Zys+7FvrmiPUf5tm1hrjjhrT94/wC8zT24K3CRrs24M3uIPOMD/CNfjN
ugmd4x7+e+v65Ju5n/uW7Fn689lgAW6y78694q/U9dEG5kf2V+jlP1HXopdh
jqevQEfWqvw9VT54/oGz7BWP1kA3qN+q9e/Hy768xr27/P1Evf+3eD+P8bm3
xpjfiWu9O3y4K3y7Nnw+O2I5MWLcN2LfJ3Kyd+Tqm5HDb0RuvxY53z1qsXPU
yN+/Qu0+FjVdP2rNms4YQCuNjbFqzKytxpI/swfGhqmxN0yNSfdas8SxI+tY
YxvbmIdj5gL2e8I2dzjenMI217iGOci1zE1sc3aEmsvY5vhMau6jF9YEbGsF
8VhDsPO50PPDthbxkxp1TdiXq3XpQvWa7hy1Rp2mflYzv7fhKLUufV+tSwfo
7d9/5c8v767+DOAY9Xf38dySv/8KjPn7rzZT/70w5mLmuVfV36mPDU8WLXuZ
sKnpIup99YVr/Llq/LeqDnNWjJ5D/CwUr4crH8yxXrvfVfHye5J4W+Xjpcqf
1wKn1LXYtzmmfIBTaAK9yO2VM+71sDfDWgkdYbyxGvQbt2qwLoNrvyz742F/
LuyvlP29ep/ehD1p1jysQdlL8Wd1WROhFU9WzbwnQ33g3qWVd/xZueL4euWW
PLJ2Ze1Bf3B/XeO+sJ+OY16Jc5+NMU+La00OH64L38aFz0dFLIdEjAdH7AdH
Tg6OXB0cOXxnbp3zA6MW34ka7Ru12zNq6s8H+76wMQBPjA0wYMwsq8aS9chr
N2NvFjUmR6qx6n14cg9/jG1sYx571bDNEc41d+ChOTVSzTWuaQ6ChzFqfTNn
8c1cJlZznHjN/ZnUmoBtrRiu1hBsawt5sOZgXx52atR1ao26Uq1Rl6g1arxa
o85U904nqzXqWHW/dKi6X/quWpd8L+2Nqrd1aaeqAXryWbUWba3+m1/svzDP
+X7PKA36GfYSqf9zVSPw/Wy9Bxfy2aqn6zqr1vtz1fvPl+/Mqa9UXheruIZX
/idVzqivn2fxd2FyDL2JP3f+h6oh57HfvHddi2O/of47DF+v6/iZcDhytfqz
h9eov3eS47kn9sGqGXgBY9fr7X93lRzD2ZuqJtuXjU5uXjZ+ci8MHTmkcsk6
b4wGmGZ9RM9A/b9U8S1TsfEztv4lPtZY85Tv5GjTOt/25DiGpPnc6THmHXGt
8eHD98K3b4XP+0Ys+0SMx0bsx0VOfhC5Oj5yeELk1p8BvKZs1+LEqNEJUTvG
d00PV9f6W2oMfFyNjd+pMeO+aUE1b2dTP/fpZxeMyYXUWF1YjeF8RouxjHn/
Hi4wD5gj8M3cgYfmFPw018i9OYgGmZv4Ys7OpuYyWmCO47u5P0KtCdjWCups
DRmu1hZsaw65sRYN09s16ka1Rl2j1qjL1Rp1oVqjzlF/du9U9Wf3jlf3Tkeo
NYp5y9/98m11v/Q19feF7qLWpR3UusT+BphGiz6q/jv171f//UEwzJqBvYzV
1H/rGbyxT/BI1ZRzHqzc0wfTA8xV495TcYCvf1Tc1O/uyg3v87w3P8xV3yob
7nxavT7/ZJ17ZPlOXZljtlH/fe7tygewDy4Wr1ruUD4z97BnsEbVFQ6/V/33
VzetGsPLT6j319lbZs3vPZxL6/zP1THsDWxR4/C7Deq6xLB01Qld8fcl7V2x
s24aowGXz66asS+zSsU9vf4lx8xD61aubM+IY56Jc8fHmMfFtVYOH/ydcwdV
ruzzlyKWr0SM3rsjvtMjJ2dGrrwHOL5y4txeEDm/KGpxadToiqjdVVHTa6LW
aJkxcLUaG8wRqnwvVjZYWlWNsdXU2CNHxqQ/hwNWl1djeBk1tpdSY35xNRcW
UXNkITV3FlBzCg0y19Agc5D4zE24bc6iY+bybGqOowvmPnphTUDHrBUj1Boy
k1pbyIE1Z7hai8hfatQtao26Xq1RV6k16hK1Ro1Xa9QZao06Sb2/dIxaow5R
9070HP5e0P9U/+0b+DZKgz7ni+q/H7Ft1Q1d2lL9d1Hpo5n/6b/BG3MGWgTe
6HHvqTGo9V/Vf+uIOY95iP3Z26o21OCHFdOBFSv5pX8/vPL0k6oNPzPUfQbn
zFvHwCs/u7dp+ereesPKDbUfXT58vXwFW6z/tyifqT3cWrOu/+k6/5gaC5zC
Fzi6dfnJegSMnV75I5/sC1xav+OY6yof4OmXlTN6tHsrx+TXz02Cx99XrXYu
38dU7NQETPiZP+5rDqv8nFoxPB82v/QxV8a5r8SY741rzRw+rBy+fSB83jpi
+WzEeH7EfmHk5JLI1eWRw6sjt9dHzm+JWvw0anRr1O53UVMw5lo/pMYAc66x
MbXyNbv6+Q9iP0CNMX8vAn5+VI1JNMZY3UCNYTBmbK+ixjz/mgtLqzmyhJo7
i6g5hQaZa9TRHJxHzU1iNWfnVHOZOpnjs6m5j15YE9ARawXxjVdrl7UFDbLm
DFdr0XC9XaN+rNaom9T7Udeq13qXqzXqQrVGoZPuo05Va9QP1BpF/Vk7v1K5
X7zquG/lkt6JPhOdf7rqwNzAHE2fzxw0serAXiF4AG/gAV1iPmHvE8ysrdal
FdV/M4K5h3XVzZXjsRpwGX9Prhd5h097V87Ym92g8kQc9AXDyt+ly/azwjPV
vyPrXD/n58+jeh/0AxUb+KAX83eBfEH9vWu71XHEAj4+XL/jueEt6//xH01j
Tjys/mUM5nD6N753ibnt+soBx7Cm2rbyzudZWE/Bjycrh+AWPKxT8TxTfoBh
eg/uKdN7oD3zV55U8UqN0VEazEG2SZSP+WCc+4UYc6G41vDwYZHwbZ3wedOI
5RMR4xUR+zWRk+sjVzdHDn8Suf1l5Pz2qMWfokb/iNqxn+mast5zrV9XY4DY
jY1H1Zjx5/bA0h/UGDtOjb291Jj8rBqrYMAYHq3G9gZqzK+l5sIqao4wn5o7
nGdOoUHm2sJqDhKDuTmvmrNokLmMdpnj+Gvuo13WBLBgrSA+awi6Y20hB9Yc
bGsReLBGgc+fqTXqh+qe6gb1nvnV5Qsadalao85XaxS483cUn6Tuo45Ra9T3
1Rq1f+X3qcq374nTz8PjCRrMm8wL3KsGb+DsH1Vv5kvWD9SfefT2Oo/10E+r
TvtVDrgm/fl49WdO6dHnq7x8o/LOfaLNKu783l4/g0YeWSP6/s72lTvfh/b9
neHqvdL5Ki9+TnqJqi34AgejKi5/DvrTFQfYgSMbVQ7Iy+iK/RvlJ+tM+nLm
4jEa7Pdwn2fPyvWPKx/MhTyP8cUah54Kvn+savHxqsX0yivxgIG9Kg606/eV
09eqZgtXXratuJ+vOJevf21Lfcycce5CMeaIuNbw8GGB8G3l8HmjiOUjEeON
EfsPIyc/jVzdGjn8beT2T5Fzf66ZWjwQNXosavd81PQNda19f5k4pqix4fvL
25e/xpLUGMNPY+8ENSb3UWOVmIxhf+8M2N5MjXk0yFwgJnNkNTV3wJ45RXzm
2uJqDr5bzU1qYc7Oq+Yy2mWO46+5j15YE4jJWkFurCHEbW0BA9acmdRaRB5T
o36p1ijnizzfor6vd335gkah994zv1i9H8VzMGAQjYLbaDYadXzlA14coe6j
DlRr1L5qjdpDrVFfqnxyT+UzVR9wtVXll3tCH6xz4CZ9L7ilr4T/aBK6DqbY
q2CuRS+YY5lH6avpNT5RMdFrMO/Mof4OcOfjiMoT64CN1N/F7/vB3pMgZ8Mq
z3NXHcAr9Z6vXhxLz7Vo/UvvwN4BeBqtAe7o2eHmWvXeZ+q9LSsv76v3dq/f
s7cFtpg/Wa98q2r7n/Xivj69B3xk7czaa7Oqy05VF+Lcs2IgCK/50DE47Htd
vtFMjv5QOUKHDq+8vzvsEXGM1OcOzzHV15o5fFggfFs+fF4vYvlAxPiziP3W
yMntkas7Iod/jdzeEzl/JGrBs/KuEfd3XDt/1nE+9fc+jKj4jAG4YGygjcbM
/mosLafGGOsIY+96NSbhkLFKbMbwTmpsb6PGPBpuLsAlcwSOmTurqzm1gppr
o9QcRIPMTTTInCVuc3leNcfhgrlPHNYE8G+tIFZryCxqbaHu1hzyaC0CH6lR
v1bvTf1Sve77iXrdd3P5Qo2uVX+H5+WVR/bM6fHgH2sBtB/eM0+cUjl9unLJ
Xh9rPe4joP0TNLhnDP/gEbgDk/RR6DxzwZ8qx7x4RudTao36SNXl6srxMZWn
91QduSb8v0ndz4LZz5a/E+tYckmfaJ0BcL+pHHgtSy5ZD72/8uT9xznUfYL1
i5/56l/ef3e9P7JyY6wzX6FVzJlr1WuUuq/i92gQ8yJYQVs/XT6zNmLeBJvw
YGf1Hs8e6s87w/+/VPycgw6zJmNdQO/0rfLf+43kIu+z0DP43sO0eoGv1+qc
hSuPd5XfS4U9ZxwzU5w7LMbMfVfvh8yv/rw0vi0XPq8TsfjzzntWvI7995GT
P0eu7ooc3h+5nRA59+c/eb0QNfJ3W1C7qVFTgnGted8YILfGBvulxsz31Vj6
DzXGrHuM92c1JulHjFXwbQzvo8Y2azhj/tNqLnxEzRGuZe6gQeYU2myuwVVz
cJSam2iQOUstzWV8NMfpn8z9d6k1wZ9DWrByYA0hR9YWam/NAR/WohFqjSK3
6L71/Vfqdd8v1Ou+H6vXfTep96auUf9tGvYillD/bTvqy3MF9IT01sxT9Ipg
4J8a9Dp+tphnYrwfRc/OGgB+sUYAZ8yPzCXsJ6C729dx19axR9Y1GYO91hNq
7GvVn4G/vWoK9h5Ur+HpARevOODScxUf2LlR/Uzgtyuv9KkfLHvviplj1q88
zlf/P6zq6naE44ZXLZarPFMb72dxHn0DczfatUn5jY+s98AM8xlYZB4EK6yJ
wBjr069WXvADLo/RgH/kFizSSxxUxxD/WA24CjcPr7pQx3Hla85N4OEfFTMY
mFQ5IlcER07vrXO2rHrbXiCOGRnnzhpjJobnDh8WCd+WD5/Xilh+GzHeEbHf
GTm5O3L1QORwYuT2ycj5C1GLSVGjKVG7GVHTGVHryWoMPKbGxi/UmDlKjaWt
1BhbQI091h3GJHuPxipzsTF8mBrb9JHGPLGaC/RP5shmau4QszmF/ptrq6g5
uJyam+zhmbNokLmMBpnj/v4J32e0JjBnWSvInTUEvlhbZlVrDhyxFo3Q2zWK
Grun4n33VL9S91S/KF8m1fhe992o7qnob0ZpgGHmA7jFGmG8et13uvrvLKMp
9Cn3VS65P8azxfQx7BkzPzJ/7FzX/2rVmjXuGA20nh6TPQ329cbVGOg489fo
Om+PuqafjcSXJ6vO3nsgF+ABwD1QMb1VcRDrxLr2AnX9zcr+dtVmwbreu6om
8Gwm9WclVMfxA75XrjpQo7XrPGq6UZ3PnAUvl6uxwM5a5SN4e1/5vpN63+Jr
dRxx7V85ZA4F037+G56wBiBGrxvg1MmVI+YX8LZS1fSHdc2XK3/oOlymxstW
TafUucwv8JZnCNYJe/E4Zr44d+4Yc5641oLhwxLh2wrh81oRy58ixr9F7PdE
Th6MXE2MHD4ZuX0+cv5a1OLNqNH0qB1YcU2nqWv9qhoDj6ixwTxrzBynxhJ9
vjFGjMbeZDUm/6bGKvd5jeFxamwfqLdj3lygBuYIsY7RgDvMueYU/bm5tqaa
g9TA3CRWc5aamcvEOl69TjT3qak1AR2xVqAv1pDZ1doCF6w59FrWInJqjQJH
4Mz7VHeoe6rfqXuqX6l7ql+oe6ofq3sq/AGHz5SfcOiJ8n+dqge9qtd9xOt1
H+tf700xn36hrslzaOg+WKbPZ++Ue0C71TnMH+CU+1fgl3kznydkz4Y1APPm
o1VP5k24uGTl2N8DQI7BBj7Qb6PN51R88Duf7fxw2furv9sKHtEfLFXnz1z5
8GfBVlb3WmvVv+D+vVWr2cq/eapmW9Q1yPs2aiyTm7XrOrurtQvcfajyDt6Y
t9dQP/9Fv8GcCD/ohcDvlpUX4ty46nVF1Ys6sh/kZ2v9/RHw5HH131p4qfwG
R+Djc3W+7WXimEXi3IVizIXiWouGD0uHbyuHz2tHLH+PGO+J2B+MnEyMXD0R
OXwucvtK5Hxy1OL/6+xMoPUuq3P/HCQxDAFE5jERGRMmBRVEjEVUblEZrENp
FXsdr7VeROlgFfVq7bXO11bUqtBiF9pbL1iHMihRQJQ5JCQkgeSEhCRMISHJ
CSdn6vpxnoe9w7X3rtWzVs7Z+b7//333u/fzPO/4/77RlqOJljt+JafwKLl+
XIUBcBNsXKPCTD+Dio4GYyeosDdNhUnmI8HqXBWGOScWbOc8MJhHE8MFzluE
I2hQuMO4MpyijHANH8LB2SpugrFwlvaFy+QrHCeP4T4aFE2gb4pW0C9HQ4hX
tIX2RnPQq2gRPOoaRb4zpgLTGVPR7qxTUU7WqW5Qjamod4Ym8361akz1I8d3
pf3PmIp2zdEkX2gv2KPfZ43uTa6H+LDuAH7pN9GeKx3PzzjuzK85zwFm3+06
mB+gTawTgE3m6cwDXu884lc+72hINU4Hg/CRfoX+c67jTfv+1m1CU1kfoe+F
S692HlgTONKvn+kYHe5/xH+Wc7Wdak+Rnxf7L6+/3K9Pcbk7qr7Xcg/fD+8P
cvn0gUeoPifqBc4BPGB9Alyx/vNK+/Z5+3aM43yu4/JtxwZe/5OK16x/gkN0
5XrHCBts5Cwue/mMP+AY+vUq5/Eh5/G4Zh/Wrjm43XtAK/OAVtfBzYdDm2+z
ms93t7Ysam1c2to+2GKyusXqkRbDDS22Qy3mIy0X4y1HaEdyN95y+qQq15Qf
DOBbsAE/gpmvqrD0PhXG5qiwB/eDyfUqrBKzYJg4BdvEI5hnDhcu0O5wBA0K
d2hrOEW8wjXme+HgcSpugqVwlj4oXD5QxXE0KNzfQ6UJcC1agQZFQ9CdaAvj
p2jOs1VahHZFo9Ar+qTs/c13zIkF+Mv5BMYcezu/zNMyL7hRtU51vWqd6hrn
EmzneXSw9APV97/n81Ao+1LVmAoNz5iKuGVM9VeqMRVxZi2PfvWDvvbDzgtt
zjoiGsncG1zT74ER+hXwCM7YWwKDy+zjiMs6wW36onOGvr/XeSBPr1Z9B/xs
22c5X8f5/zuqvn+Juk5SnUOa478DLmvA17ze15CvNztf5Obtjj+5BN/0wXu5
zfCEvvpi1zfDcWFMAt/R9NOdG9ZCz7R/zKXBN/Nx9lMZW7CW+BPHjD4bvKf/
vsP2fb7nd/x3uWouT76Zexzb7CPaNc9v985sZc5sdT2/+XBE821+83lRa8vS
1sbB1vYHW0webrF6vMVwU4vtcIv5WMvFRMsRv5I7rklOh1S5RjuCgQUqbFyt
wszXVFi6QIWxM1TYg1fBJHUFq/epMMy6VrANH4J5xlrhAmOtcISxVrjDWCuc
OkPFNXIQDuLLpaqxVjgLrsJl+pdwHA0K99GgaAJjrWgF+hINAdvRFvgSzaGv
jxYR32gUOaKPzBwwz3kNOdbEnzEVWGQsv95xpT9ivs+YDXzRl/5Kdc7zF24r
OL3OMVjsmJ3h8hgz/Z59R48YF6PBjJ+yTsU6HnNtxr1fdp7pm+gT2BOlH/mY
4/I+3zffOXyvfaE/ZP1xk9tAneOq74kGA7T1dG37OQDoLtinn2c/nv7mJfbl
1S4X7B3j1891vk5WrXec6jpz3jlntM5QPXt+lv8y1oIv2/v6853DHVTPHsMl
sJfny/7S8Ucn6R/hAP0dZ/bo7+DkJfYDjF2m0i7WNuA7/fFV9gMtgVfoA7ig
H2RchCbc7nsX+x769YW+jjjSB8Kl7KvHPqpdc3i799BW5mGtriOaD7Oab4ua
z0tbW5a3Nq5sbV/bYvJYi9UTLYZbWmxHWszHWy5IVHI00XI30nK6UZVrcBMM
EJNggzYFM99QYSlr32Asn5tBGbNVmJymwipj1WCYe4Ltf1FhnnlBuIBmhyPM
EcIdxl/h1OtUXKMvCQfxMdykDwpnyWO4DMbCcWIa7qNB0QSwGq1AX6IhcCTa
QtyjOf25VPruaBR6hR5nDgjGMgcEl5kDgsvMAYnBDE3OAdHDzAHRSXAKZn7t
3DD2R1df4TKI2Zn2N88fsLfBPPc9fo24/7n/0p/mfMK3/JexP5r1Q1/H+zc7
pp9w24gjOMhnALzfed9V9b2ZjA/Ps00/dK7bTZ8HNhgfgPfXqD5/G/zAE/SK
vuUvnMdXOo/0GXDuRNd1hl+jjwAT9BEDritzw9/3X16nr8/+/nudK/5d4Bzt
5Dr3Un3vXtaoWYsGR+QevNJnH+S4gTv6NuZWc5yzrMOhJz9WaRd9NmOLRW73
mbbJ7+86j/MdowV+Dz7DTTATvYo9u11zVLv3yFbmka2uWc2Hhc23Jc3nZa0t
D7Q2Zg2atj/SYrK+xWpzi+Fwi+1Yi/lEywWJSY4mWu62tpwy70iuV6kwABeD
jatVmGFsGCyhX8HYW1TYO1GFSfQ0WKWuYJhYBtvoRzBPPMIF9CkcQYPCnT9S
cYp6wjV8DwfR0XCTuXQ4Sx7D5UNUHKevDPf3UWkCGhSt2E2lIdNV2gK2ozn0
AdEi8hGNGna+c/5z0LHZ5GtyRn2p6ow6Zc50XKiL/nCN6rOKHrBvJ/m+21Rz
QNqSOSD9aeaAP3fcmQfw3Av9AX0pfSt9APNtdP5y1RoW+f+g37vd+fyS20Ds
GOuCUzSdvjVnjZ96jmRgsh9if2TMvpA/sACG6HvQ2o9rctw8V5P4Pdf5Jadn
O6fH2n6j83WO80VewOnBjvlbVGtZb1XtG75D9fM+1VlT/MxYC4xnrPVJ54hc
g6/9/H/20dAkeMicOfNExv4vtB/0q/T9YOyHjtPhqnNsR7mtcAad+ZXbtkD1
+fTY82yT83vcZvQKXYpexT66XTO73Tv7GWWmroXNh8XNt/ubzytaWx5sbXy4
tX1di8nGFqstLYYjLbbjLeYkIbngV3I03nIHd5JTcJZcr1RhYJ4KG9eoMJMz
UmDp4yqMMVYK9pjTBpPkMVgdUWEYTgfb6Ecwj/aEC4w3wxHqDneYL4ZTrE+F
a4xrw0HGTOEmY6ZwltyFy4eqOE7/GO7Tb0YT9lRpxXNUGoIGRVvQ/WjODiot
gjvRKHR5tWrNir4ha1Zck31A7s0+IGVmH5B4ZR8QLX+BrwWrp/g1sJgz6nc5
T7c4fm93nGk7/dy/OiafdMyJVc4q/Jvjy3tX+j3Gt+gaXIEbl2oSv8SLce5G
x+LzqnOOn2q5/ohzzV7zhW7fN10ucyOwkLV/sAR+GDfTN56vyWcfjvN7b3S+
znf9z/H1z3P83+XX6JvBXtay/rtqbniR//L/j/ra7e3zNOcO3mWsBR8z1vqm
66cvA5OHOX/E5xjHgbgxV5qhyf4YTqDnjGeY2xyu+hxP8sp8I9pFvs52rOc5
3tgLbPMamMz6Vexj2jXHtHsXtDIXtroWNx/ua74NNp9XtbasbW18tLV9fYvJ
5har4RbD0RbbiRZzEpBcYCdHYy136F5ySj+eXMOb8zWJAfAebFyrwsxlKix9
SoUx1peCPTAWTJKjYPVZKgwTh2Cb8VMwz5rfpZrkwldVHPmYijvocTh1nopr
r1VxkL4h3HyRirNHq7icvYKcjQ/36RuiCXuptAINiobsqtKWnVWaQ/8RLSI3
0Sj0irWnrFmR+6xZrVHpFe3OmtUq1ZoV5ee8OvVmzQp/wNVi1frgPP8/a1Zg
lH4LPKJlnP9A49kXoJ8Ad7c6p+g+80bmA2BqrmNKH3G1Y/Q63/+QY/Q9t2kP
546+cbNzQq7p88l1zu8z90OD/9Z5xSfG1+ALntDvgR8wQV8ChsE03OSsA5g8
2PbJqu/jy/nCCx3/7V3utEk3nv68a34+4b+8/hnV3JA+cqr/fVX1GW7fdJm7
OF4Za9H2jLXA51F+7yeODbmEHyf5Osa4rJvQV4J7uEJfSL+ZdaQ7HWOwCvbR
GTSCOL5B22pU16672zXz2733tDLvbXUtbT4sa7490Hxe3drycGvjutb2J1pM
hlqstrYYjrXYTrSYk4DkYqLlaLTlbqjllHqTa8Z+wcA8FTauU2HmchWWWCMN
xtCQYI95cjBJ/INV+qlgGH2+VJPYvkOFecah4QJjq3CEOfFcTXIHv8Ipxkzh
2utVHDxNxc2XqDhLrsNl+rhwfKaK+4wJB1VrWdEK4hUNQYOiLcwXojlgO1o0
TaVR6FWe26SdtD9rVsz3ssZOzPJ8Dffm+RrKnO36wdCJ9hcfchaU8TE4vMu5
zLkFdC39CuPfnFdf4hx+xzFmjv0Fxy1rVne6DPoo+mUw/hrn4VH7dK3zSxwY
t2XN6vu2ye9lbjc5u8RtYBz9ZdcBpj6rOtcHfuinWZP8hN8ndxer5gPg7aXO
y8WO1Y6+nvhnvJS1d87jRa/yDB0/X1HNDS9RzQ2/7fLI4+XO485u197+/1Wq
50bgAZjax7HOWOuXqu9xJZb0ifR5t7p9hzsGzMVYX0JnznRs6U9z9hYMn+s4
LFLpVez57ZoF7d5FrczFra77mw+DzbdVzee1rS2Ptjaub23f1GLyZIvVSIvh
eIstwU7M+7OTEy1HGY+RO3CTnMKL5BqMBwPzVNj4uQozrJEGS9QTjKFnwd45
Kkyiy8Hq7ioMw9Ngm7gG84ybwgXaGo58RsWdD6o49XYV1+hLwsFXqrhJPxHO
HqfiMv1aOE4/Ee4foNKEvVVaAS+iIeQj2gJPozk7qrQoz3+jUVuc46yxr1fp
FWPcrLGvc5mP2s4aO+Xk7Drv5ZwV9c5xDPDndx0n/MwzFNzP+Ih5PbpKH/Mv
bu/fOHbE4Tv+P/HhbMdFroMcvNWxJab0Bfc0/6iPvvTZzk/2Vph/jvu1n7it
V7juPCcEtpmzfsB+oGf0kfSHX3TbwBrjZDiDxr3BMUJbT3ZevuC87KR6ZhXN
+Yrq+y4Zz2Xt/euqn2+p9OofVHPDK1RzQ3zO3PBfXT7/B6/gBWwzfqUvAzPw
PWcyaX/GWuhDxlrELWMt9CRjLWKbsVbGs+SePgluHWs7ehV7frvmnnbvva3M
pa2uZc2HB5pvq5vPD7W2rGttfKK1fXOLyXCL1WiL4XiLLcFOzPmVXIy3HI20
3FF+cgoPkusVKgzQpmBjrgoz6E2w9GUVxjL2AntvVmGS9ger8DkYHlZhe4kK
8z9TcYH2hSOfVXHnwypOsX4frtGvhIPM8cJN2hPOHq/iMn1ZOJ6zTXOcg2jC
Piqt2EPF0d1U2oJeRXPAc7RoB5VGbXEMolcbVXq1UaVX4GCG639CpVfY2RPc
oNKrDSq9wj7TsaQsxik3qT6n7xr7ybjpf9tmX+fv3ZZLHefHnLMP2Q/mD+c5
boOqfWDKnelcjE6m8ik+YHDdfNU54Vt8PTn9hXNKv3e1fWStgv76e87j951H
1iFZX3u/c0Dfd47q3G/0ir4teoXN/GyKr+G1aNFT/bva90K6Trv9FG6f5X/4
g1ZNdTwyN7zWdU5328gdeb3R8SCv4BTtAfOsRdBPsnbA2PfFvo5+OutajIuy
roW+ZF2L8dBr3fYlKr3Cjl7Fnt+uWdjuXdzKvK/Vtbz5sLL5tqb5/Ehry+Ot
jRtb24daTLa2WI21GE602BLoxBw7uYimkSP0KrmDL8npY6pcr1BhgLYHG+hr
MEOdwdLfqTDGGlWwR18cTM5RYXWGCsP4H2wvV2Ge+8IF6gxH4FW4c5GKU8zx
wrU3qDj4KhU3X6ri7PEqLqNX4fjzVNw/UKUJ6FW0Yg+Vhuym0pbpKs3ZSaVF
OS8SvRpS6dWQSq+GVHo1pNKrIZVeDan0asi+3W8bnxfZjl4NqfRqi0qvtqj0
aotKr7DJ9/+0zbj/Qtu3u6wh54k5wya3/wDHKvspeS53ifMIfsHFMueMtQ36
I/hzsfNJH8bYFy6w/8S8nn7rr1wXvqBn9PE/9PuMVZjTMteHJ+AyeoUNtqfa
jl5xffTqKtXa+4/8d8B1ZS3rWtX3pFzvcsjpDc4jnKTPBSPggfFJ5oZ3qtbh
6fuPdqxoe+aGxABsMjckXpkboinMuY+yHb0i39ErXmccOq/Z89s1C9u9i1uZ
97e6BpsPq5pva5vPj7a2rG9t3NTavqXFJPtnxGqsxXCixZZAJ+bYycV4y9HW
lrvNqpw+psr1AyoM4HuwcaMKM5QdLH1DhTHmi8HeO1WYZJ4crNLmYHiKCtvE
Kpi/UcUF5p7hyOdV3PlTFafeoeLaG1QcfJWKmy9VcfZ4FZePVHH8EBX3D1Rp
wj4qrdhDpSFZN5+hSb2K5uyk0qLMvaNXjCnzLPOw6jPVh1V6NazSq2GVXmHn
uZutqvHVVpVegZXMB0fcRuaDoyq9ws58kHhnPoh9qSb1CtzQF1xom5i/2XbG
zOOq503HVGvsW53T+f4/6xng6HG3g/XXnMkgN/RV6Bnz+oX2JW34hOtizwBM
wd8bnC9ylOdK0KvMU4j/tSq9ulalV6zDRK9+ptKr6/2X//9S9X1zaP0U/8OH
rGXB2d1c7p2ui/XMu+0X/tyjmhvCA9YTWBelj86Zhzwvc4hzSR4Pc0xy5mFQ
pVfYaNFxvoY1jbuaPb9ds7Ddu6SVeX+ra0Xz4cHm20PN58daWza0Nm5ubX+y
xWSkxWq8xXCixZZAJ+bYycV4yxE4Su42q3L6mCrXK1UYoL3Bxs0qzPxYhaXv
qDD2GRX22CcMJpk/B6tHqzCcZ1RwlPeC+ZtVXLhCxZHPq7jzpypO/VcV185V
cRC9CjdPVnH2eBWXj1Rx/Hkq7h+o0oQ8N575YDRkN5W25Dxo9CpatINKo7a4
royvsDO+GlHpFX5m/WpUpVejqvUr2pj1duystxOTrLdjZ70drLzLOZ9w/H7g
2KNP37b9bccOmzx/yDb90Jt9L7l4ie1Nqs8wGXeesXnvNl9DW//ZbVzpPG5W
fT8dY2P0jPkmfRljdngFlj7iHNA29IyxB5jKeUHW8U9wfOHLIY7/bxxb9OrX
Kr3Czlms36j06hb/HbBfWcui/O39b57z+Wz7kLWshaq1rMWqtayl9mcv5yn7
hoOaxCDjQ/Qia1nEZo7qu0NzvhQ7eoV9rur5K3JyZ7Pnt2sWtnuXtDKXtboe
aD6sbr493Hxe19ryRGvj5tb24RaT0Rar7KURv2Aia1aJOXZyMd5ytLXlbrMq
p/iTXK9SYWCxChuUHcygR8HSd1UY+4IKexeqMMm8Olg9QYXh59hX/p9xVvwN
F65QceRzvoYyP2wbTv2Rbbh2jm04eLqKmyepOHucistHqDgOZsL9A1SasI9K
K/ZQachuKm2ZrtKcnVRalM9lyPhqTKVXYyq9GlPp1bhqfxA7+4PY2R+cUJ1n
wM4zOLQ95xmwc54h5/B+ZhsdutL3Mk6+zOV/03kcdR6Zd4NFsMJZ24x1X+z8
bGrtmbCfW13Wjf6bve48O/hpVb/7x76e9YU3ujzWRU9z+/GT/h6Mnadaszjd
8WKO8wL7AD5ZV6FfX+hcoVcLVHqFPWXSlafs6NUC/+X/96j0ijFr9Ir4Zu0d
/u7iuu53Xbs4H1nLWqFaywL79I2sZTGOyXl42pXz8PTvYK8/s5zPaEBzjvH1
b3CbVzkndzT77nbNwnZvf1Z6WavrgebDmubbI83nx1tbnmhtHGptH24xGW2x
Gm8xnGixzXNZUu23Ra+So60td0Mtp/iTXON7MLBUhY3bVZgB88ESOAvGvqrC
3kdUmHyr/SNOp6gwvLcK2/gQzP9GxYV/UnHksyrufFDFqbepuHaWioOnud6c
Zwhnj7Wd8wzh+EwV9w9QacLeKq14rkpDdlNpC3oVzcm8PXoVjdri68PvcZVe
UVfOM2Dn/BV2zl/hZ85fYcPjQds5L4rN2ORu2zkvmmce5tof9nLZz0FLOat2
ueNMv/MV1Wdskcc1ziPPd2Yugf7fY9/B6AJVv7LcucnaPu24RPW891/6Wsb9
77Cflzl3o87dqY7X+50v8IKeHeLyXqHJfmSt3ycXq5yLHe0HuZpif7I/iO/b
T1b5lB29Wqb6yevbOb7RKzAIB6e6/Ona9qwd/1/tPD7HcZuhyRyTv6y95/Mp
GMOjC2h/9nqiV9jRK+zX+R7uBbcvdNvh4+3NnteuWdjuXdLKXNbqWtl8WNN8
e6T5/Hhry8bWxn7meWuLyWiL1XiLYTRgoNn8TLSYj7ccjbTcDalyul6V67Uq
DIDNYIM4BDPMQYOlK1UY+3v7AfY+pcLke1RYzRo45R6kwjZlB/PMX8KFf1Rx
5NMq7vyJilN/oOLaa1UcnKPi5otUnD3Gfua8aDg+wzbc3982uNjbds5fRUN2
U2nLdJXm7KTSooyJwe+Tvibnr7BzXpTyuXeDbcpcZzvn27Fzvh2bPmWFbea7
S11mnseh7Xkeh5igNzfYD8Y1P3XcORP6PdfBfvAljjl9BXN+xt6sEaAtaBx9
J/tJ3/T9tPerrpv3vu98/8B14+/nnbvMAcHPVc4d97HffIbbgj9ZH2P9YbbL
O8s5InanqJ5fIi+sHYGrAx1zxuvMydAreBe9AttZv1qj0qs1qp+8vp3tnG3g
3uwVUubOqu+HyDM71EsewcZj9jd7yjmXhZ/RK2ywSV8JntHprGu/WvUZB6wh
H+cymRfkcwXe6pjGvqtdc0+7d0krc3mra2XzYW3zLWdpcs4qbdnY2riltX1r
i8loi9V4i2HOuAw0W894fbzlaKTlbosqpxtUueb9YGBQhY0FKsxkfQks/dT1
gTH66GDvCypMXqjC6jkqDBOLYJvxRjAPp8IFNDAcYS8p3EEDw6k3qbh2hoqD
cCrcPEHFWfAfLh+q4vjBbgvc3892zrdHK3a3nfPt0ZbptnP+KlqUZw+iV1yT
dXpsrtlkO+fbsfM8Dnb6XsrJ84PYeX6QXLzE8QAr9C/0DYxFs1dIfLP2Tq45
13Gt85Czo4yxc3aUHDC/ZkzLnJ95N30Ez3/BCfod5tDgeAfHF19vdT42uxzO
0qCjH9ckbua67tUu91zHgrn+77gtH2rl/aHbPOHcHujcMR6gP9iq+n5L8La/
/ck8Fb16QvVdMHAvevW4Sq/WqX7WqfQKO3rF9dEr7OgVZe6uehY3e4XUm73C
7CnzHpw/xm3Bz+gVdvQKG22Zrfru7eNd/lt8Dz7Q/97S7DvbNQvavUtamctb
XSubD2ubb/0sTc7YZE8wbcx5wuhVYjLaYjXeYkhQE9vYsp1cjLccjbTcbVHl
9AlVrh9RYYC2BBtgOZi5zXWApZ+rMMZYJ9jj/Fcw+XEVVolpMMzaULBN/cEo
84hwgblJOHKRijuUE06dreLa6SoOnqzi5vEqzqI74fIhKo4fpOL+vipN2FOl
FdErNCR6ledxojk72o5eRaN+m17l+UHsjBcmVN93gw2WH7VvaHz2JrL2PuJY
LncdWcuiXPavmRuCDdY0wPaDqrkhawsfcExvUH1XIP3A/3KumBMy/uIc8RxN
jq9Z2828iDbu5Db82O2n/wAHjMfRsAucK8bjjKnA0NecZ9pDX3SK48Jz+8e6
jcxrnu+yT1ONe+lHMjfn/cxD9rU/W1XfoZLPCEF/wFnOi25W6dVm1c8mlV5t
Up3Fwo5eYUevsha7s+q7HpJjMLWH641ebVGdbcC36FVyd7ivIXezbYPxF7ou
5n2MP8EKY4abm317u2ZBu3dxK7PjZFXzYW3z7dHmc87YHKRtv/NiS2v71haT
sRar8RZDgprY8uvpmA9ULsZbjkZb7p5U5XSjKtdZ4wUDWScBG/fZBjPzVFi6
UYWxn6iwx1grmGRtK1hF14Jh9uSC7ewXUgdr7OFC5nVwhOd9wh10J5yinHCN
a8PBF6m4SZ8Rzh6u4nLHCfgJ97NHnzX2aAW5iYbsotKW6NXeKr3K84PP1Kt8
PgP35mwDds42UFfW3vEha+/kKGvv5C5rWZSVuSH3c3ZjqeozspkXgEvWMVib
RdtZm/2V/6HnzNvpby5ynngmIs8Vkmf6H84G0z7meX/mWMCVoyab9VQf8EXH
hXo+bH9/5ByQe/aR6e8Zz7GfzJgKXn3QcQcLb3OZtIv5/QzH4RTnhcqOVu3X
kIudWgzDE/zY3vfyWp7xiF5tVenVVtXPsEqvhlV6NazSqzxblv3f6NWwY7Sr
y4TjWd851P6DsehV8DbTdvQK+zW+jnuZl5zg8t+q0nbOF93Y7NvaNfPbvYtb
mctbXb2/W9t8e6z5vKG1ZVNrY87nRK8Sk7EWq/EWQ4Ka2PLr6ZgPVC7GW45G
W+6GVTntfMl+9Ez7H2wMqjCzUIUldCcYY6wV7IH5YJJ5QbD6NyoMX6DC9kIV
5v9axYUPqDhCDsKds1ScgpPhGmOqcBAOh5uzVJw9VMVltCAc31/F/ewFZM0q
WrGbSkOmq7Sl82UHlRZFr3Z12yZUc5fME5+59k75WcsCP5kbcj/jQcbA4JO9
G+aG9K2MRxgP0y8yHma9L88WgVXmA4yN71Z9/hR/f+zcoff0LfQJjG+Z+/25
83iS88jYFq7+nXNJ33+pc4P/zB3Zf2UcyrrjHzpG5Is+/hznjLijfdm3BTeM
DY6xn29wrB/1tQe4vSc6RqOuY1fV96b0Me1U2+jHs3zN1MmXnrKjV+OqnzH/
HbAdvRpT6RX1Rq+wyfk029Erro9ejanOYo25TXur+sG+Bz3T/kSvsF/jmIw7
Xy/y9edrsl8GG6zNzm32Le2au9u9i1uZy1td/WzMQ823x5rPOReZ+Wva+GRr
+9YWk7EWq/EWw4kWW34l5tjJxUTL0Zgqdzn3lDFscr1BhYFHVNhYpcIMfXew
REyCMfQl2KO/DibRrWAVHAfD31Bh+xwV5jlPFS6cp+IIGhXusH4STrFOFa7h
XzhIrsPNw1Wc7c8JHqjiOBoR7u+p0gR0J1oBDqMh2RP8bWvs0aI8h5DPvxpX
fb4o92beQJyzlkX5mRtyHbh+3Ndlbpi1BTD2mOqzRvnHvIGxFmPil/ov6xqM
jRl7sr/LXJhxMhrEeBecs0dyvia16y2O53WOHWOlnE3+qOqZV+aI4J59WvZf
ci7kYsd6iq9hHEVf8iH/BRdoG/N05p/ML8HCr13GDE3y6hTVuP9Y52LIucgZ
kn217b5GziaiYenXc4aBX3ne+enBla/PSxMqvcLO/iB29Ao7epVc76z6rqmu
pX2PZm/b5Ct70NGr7E0fbjt6hQ2/Xmyb8zv9rE4/U/ebds3d7d7Frcx+BiZn
Y6JX8a3vTXdN2NTa2PePtraY5NmV6FViGC1KHhLzp14YaHkYqL4iuRtR5bTv
s+fsIhigTw821qgwM6jCEuOXYAxOBHu/UmESDQtWwXkwzPwx2Ga+F8yfoeIC
djjyShV30Khwin4nXMO/cBCtCTefr+IsY6NwmTFVOL63ivt9jTT7eZkDRkN2
VmnLjirNmabSouRsunM6ptIrrs/ckPcyN+S9zA35f+aG4Cb7ho+pxlqrVWOt
ZY4BOJzv2JCrGx0z+hrGTOCdMW32VOep5ur5rBHGs/RB4JQ+iH1ZYPUev76f
X0dzLnTeT3I83umcoxfvth+MVZkXHue/b7Lf9Gs5rwsm5jgvrFkyB6IPWOFr
s+9GDnOOMHPAvmZFnLPGHr5Ei36bXsUOl6JXGNEr7OgVdvQK+zmq9Yycdc/a
5nNtR6+wo1fYtHGmbXKUMzasjR9tO+MrbGJ3mtsCF37W7FvaNfPbvYtbmctb
XauaDzkzE72Kz30PaHNrY9Y3oleJyWiL1XiLYbQoMX46/s/Qq+Soa13mhpl/
J9ebVRjIPibYgBfBDPwOluBHMEbfHezxfjBJDINVdCwYvlmF7XyeSDAfLrxU
xRFeD3eoO5yCn+Ea6x7hIGPZcBM8hLMHqLi8r4rj8CLc312lCbuptGK6SkN2
UmnLDirNyRyBeE5VaRTvo29Z5+Bf5ob8iy5SfsZa2Yt5wv5krLVGNdYaVI21
FjrWtIO+Fi1hnsgZX/pnsMraIn0r2vXXjg+5e5dzR2wYezKHm+f63+icDjin
dzsucOFe30ffcrTbzXiZsQG4o3+fbX8ZIx/hvxnrZh2f3P6j8wIu5ro8YrrA
udjF7SH+Ozomz9W2a+BZY8+a1YhKr56ej2jb+eAzx1d5Huf/pVfU9Z/VK3KU
vQOwPMM2Y4DD7MOr3P4J5+IE1WdugmPwxN73Dc2+rV0zv927pJU52OrKGnXX
q1n6z+tV9gf/I716ur94xviqzweTI15L7rL2nn4puR5SYQCOBBv06cEM7Zqr
ej4gGIP3wR4YDia5L1i9W4Xh+Spsg+FgHpyHC8epOIKmhDvUHU6hL+EaGhUO
Ume4SazDWdoTLoOfcHwPFfdz3i9jqmhF37PO51xwX/ZcMweMFk1VadSIak1y
2NdnrLVZNTfc4Phvdr0Za62177zPXkTGWvSd8B7tYi3w8JYjdALtYk7+Aufr
EseBtubcB2ch2Ldj3rbE779K9RnPZ9gesL3UbSbOy50nynm+Xz/NuSDXr/Tr
+6m+q+U0X09eGFPnuSf2g/Oc5v9xLsDIr9xeYrfIMZmm+k5fxuyPqOaAT6jW
rIZUa1ZPr/c6H/kZ9d9oWvQKzkSvxlXnReFV9gczJ+prmH0+uLtqbrWX7egV
ZTK2n2H7FMdtzLGdZd84Q0X+4CrrsfTd4CP7g7HvaNfc0+5d2socbHU92Hx4
uPm2rvnc54N9rbuvwWY+GL1KrMZbDKNF0avEHHuk2ckR1yd33Jeccm1yDXeC
AXwLNvA5mIEXwRIcCsa4L9gDw8Ekc69gdakKw7webIPhYP5QFRcOV3EEO9yh
7nAKDQ3XZqg4SHzDzf1UnE3fnXlfOA6uwn3wFk2YrtIKMBkN2VGlLdNUmvNs
lRbBo2jUiF/fUdv2EcMuJ3NDsJKx1kOqsRbxzViL2IF3xsD0AQf7vptV80Q+
m+gIl/FdlXaxh4dWMxdgTMvaBWNn+hB4MOg4neI8Uv+ptrdzbFe4bXlOljid
4JhPdcwP8vUv8et7+fX9/doL3R7GAtmH/aRzQRy/43wSR9YRcrbtFsecOC/y
+9urzlZQ58OqNavHVXPAjaqfoWY/qZqWbFXp1YjLSR8fvRpV6VXWnLPWHL3K
GvXuvibPEmLnWcJRx/pg10U8n28fTvN1+MZ8AX6AsTeqvsOTddpbm31Xu2Zh
u/e+VuaKVtfq5sPDzbd1zee+3p7nyKJXz1xvj14lVhkjZU0xsSXQiTm/ns7F
QOWI65O7cVVOKTu5hjvBAGUEG5QRzBCbYAntCsbgdLAHhoPJlSqsPqDCMK8H
22A4mD9YxYWZKo5ghzszVJzi+nCNMVA4CB7CTTQqnEXzwmWwFI7DkXCfsVE0
YWeVVsCXaMgOKm15tkpz+txkikqjRvx65o45Gz7scjLWeti+bHEcs661TNt+
h072B25zu8nL9Y4N5TEXBvto17dU4y72VOlDGT+y/se4Fdy+1jF90K+Ri1Vu
64l+/Vl+fbXqe0kp87mOLTGC20fb5ppjbO/u18EC3DvK9zG+PtRxvND+E/8v
qfqIK1Sff3id40xsb/X7+LVINQccVM0B+zn2R1U/65u9UaVXm1V6tUWlVzkL
lPMM0av/6DzDsOrZ5+zX7GU7evWk6vujqesk1eeE0P8eYd/+i+OIz+eqvouU
/ag7mn13u+bedu/9rcwVra7VzYeHm299P3pDa8v/7zxD9Cqxynmq6FViG12K
Xj2di4HKEdf3c++DqrXJ5BpOBQPUGWxQTzADR4Il9C8YY9wR7NH2YHKtCqtr
VRjOufuj3f5gfj8VF9CLcIR4hTu8Hk5xfbgGh8NBygw38TucxZdwGV0Ix+FI
uI92RBOIRbQCfEZDpqm0hdhFc6aqtIg8RaPy3NE0x5nrM9YCb8H7atW6PjnK
WGuxap5IP5p54k2q8w//ptKuK1TaxblPsAcWmLPnGTXmEEe6HWc4Vg85J7NV
2ny07e39OvfK9z7mth5q/55lm/sGXNdzHadDHTvue55fYw04fcQ7VRr1CdXa
4DccB2L7A9WY6ud+nzpvVY2pFqrmgMtUerVK9fOQ/w44LtGrx1VnGDao9Gqj
60p/NF3bnqXf2bmJXm1W6dVmlV5xffSKMvNZDWAjzz6TvzmaxCv5y9lR8sS6
CtwCh6yT3NXsBe2axe3eZa3MB1pdq5sPDzff1jWfN7S2bGpt7GfOM1/Iud3E
aqzFMOOlnG1IzPmVXGCvanZyx/XJKWUm19QVDOQZsOn2LZjB32Ap52qzhxXs
0d5gEjwHq/HxML8ebIPnYH5Pu3uk4xKOYIc7e6s4hR/h2p4qDnJtuAlmwll8
CZfBVThOO8N9fIkmoCnRCrRmUDWmiraA4WjOFJUWba/SqP6cQubg+fyGNarP
18lzo+jVUtU8cYFKu25x+8jF9Srtuko17rpMpV1fcF6on7OeM+3j21TPEJ/e
cjTLcXvUMU2+pviaPEtxiMuf6vKJ7XbNHmg2bTrI8TqkxfxlLeZvUWnUh1Tf
MfQ530f9l6rWqa5SjamuV42pblHNAeer9GqJ6mfQf3lvlWpPcK1Krx5WfV7f
o6rPk0nftINjmudxsKNX61V69bjbvKftfFYD5aA9/VzN81zvy/V/f1ZDPh+G
/h4uM6+4u9kLte1nyOTeZa3Mfk5vTfPhkebbuubzhtaWTa2NW1rb+xxhpMVq
rMVwvMV2osWcX8kF9pJmJ3dcn5xSTnJN+cEA9V6qSWzgTzCDn8FSnnEJX4K9
J1SYXK/C6noVhtdrW2wH87vbXe7PM/bpr8Md7HCKa8I17g0H4Xa4SfnhLG0I
l6erOL6TivtZJ8m8L1qB7kRD+nP6+BjNydrvDtr2ubMx5xE7z4RmrLVSNdZa
pponZm6Odt2p0q6bVGeRrlZp1z+3XPAMZta72INgTAr2GA8f6Pif1/LC2sYM
+zur2XvZXm/fD3Y5cjmUDwb3s6/bNXug2Ts5zju7zpwDObHF+XWqvuDdqr7g
Y76feH7FcaPOy1Vjqh/6NeyfqzTq16qfu5q9yH+57j7VHHDQ9TxLtWZBHQ/a
X3xbo9KrzB9o10MqvcKOXmGjQ/mshuhVPluAONL35dkcuJy1XOZvr/Q9zOvY
V6KvZr7HfGZ+sxe1a5a2e5e3Mle2utY0H/KZDNGr+LyhtWVTa+OW1vbhFpOR
FquxFsM+p5toMScByQX2Xc1O7rg+OcVOrik/GIBTwQY8CmbwLVjC52As50mz
PhxMblRhNXPWjIWD7V1UmN/VPpLDzLUOdh3hDnY4xTWnNTscpJxwk/LDWWIa
LuNXOE58w31wGE2g/dGKvsZLDqItU1Sak2f5w6loFLF9SLVe+6BKuwZVY60l
Ku2i78w88RbVPHGuSrt+1OLPGl3mjIwn93Qu/kfLxZ+0XLzZMUdzXuH4YB/V
7L1tb3SbkrsBxzDPfu3lWAy43m7v6PYltvu22GY/mFicrtKoP1Bp1AWqed+n
/T7Y+ZpqTPVd1Zjqhyq9uk71c1Ozb1PNAbNvzb+FKr1aYr/I7/0qvVpuP/Fp
hUqvHlDpFXbW27GjV1yfz5YZ1KRWHOAy4c8M18XaLGsWjO1P8z34lrkEZ03Y
+7un2fe2a+5r9w62Mle1utY0H3JmiXrWNZ+zH53zomnjltb24RaTkRarsRbD
8RbbiRZzEpBc8OvpHA203A1UTrk3uabMYIC6gg04FczAo2AJP4MxeBTsbVFh
ckiF1WB7z2fYwTaYT7+8j+1wZBcVd7CPana4xvXhIHa4CZbCWeoKl3dUcRwf
w33qnKvSqGjFVJWGTFFpC/EaVI2pokVgPhpFPFf7PuwVLbb3qbTrHpV23aHS
rptU2nWNatz1A5V2Xar6rK0vtfh/rMX/v7X4/55qTeXljk/WMGLvY3uzfdnL
ZW6nepZXLYYD9qHb0/xvN//ds8X2cJX+n6LSqLNVGvVO1bzvz1Qa9VnVmOrr
Ko36rurnymZf479c9wuVXt3scij/dtWa1TyVXi2wb/wf3u3ieNzrtpObxSq9
AhvRK+zoFfYRjivXwxn0in4QDaFvBgOcPWSeAN5e4TiBh1c7P2DytfYl9uJ2
zf3t3hWtzFWtrrXNh0eab+uazxtaWza1Ng61tg+3mIy0WI22GI632E60mE+0
XPArOcK+stnJKdcn19jBAOUHG/ArmMGHYAkeBWPDKuw9qcJk9gqiY8Fwt3ew
L7Q9n1W0h8sNR8BDuLOzilPY4RrXh4PY4SZlhLPUFS5jX6riVLhPO6MJ8Cha
QayjIcQl2oIdzSF20SLsaBTxXNniuUylXfeqtGueap54i0q75qq06ycq7fqe
6vOYGTem7/hci/lftJi/W6Vj5zrm2Kc2e1az97W9xXXurnqG/jmuS6ox0oB9
eLZfjz21xTPjEdo/s8XzBJVGvVqlUb/va7H/WKVRH1WNqT6n0quvq34ub/YP
/HfAMcya1XWqNatfqvTqZtWeILnImXZ4t6v9vlOlVzlfu6vt6BW6Eb3i9egV
cx+0Yn+Xk/1xyj/Z91DvHE2ufXCmiPWoI+0nz24savaSds2ydu+KVuaDra61
zYdHmm/rms8bWls2tTYOtbY/2WKytcVqtMVwrMV2vMV8ouWCX8kR9uXN/nqz
k2vuDQYoM9jADmaoN1gaVWEMTgV7OesdTgWrw64yz9MF29P8+m4qvEbHwhHK
DXewZzX71GaHg9wbbmKHs9QVLtOGcBw73J+q0gTaPNdlT1FpSPrfzPuiOdjR
IuIYjerzd+ylLZ73qMZgd7TY3qRaq7+uxfmqFmf6nazVf63F/DMt5hepdOwd
qn7kbNVnR7ys2bOavX+zp6v26rdveZTqDP+A/Znq1zNfntJiuGuL4QEthrNb
DE9tMTxLNaZ6u1/DvkClURerfj7X7Eua/Q+qMdX3VXPAq1RzwJ/ah+0d8+jV
XNWZ9htUekWOoldwM3rF2kv0Cjt6hR29wj7aMUZnOP/DWgbPYpzkexh70B+z
lsUazum+n/6Uswr3Nntpu2ZZu/eBVuaDra61zYdHmm/rms8bWls2tTYOtbY/
2WKytcUquI1eJbbjLeYTLRckJjni19O5G2g5Hahcc28wgB1sUH4wQ73BEnYw
hm/B3ogKkzkvFn7xEzwH2+mLp9sOF6apOIId7mDPavbLmn12s8NNygxnscNl
fAjHscP9KSpNCG7Dr2gI2I62YEdziFe0qK/f/jtLJbJY
   "], {{-0.5, -0.3}, {0.5, 
   0.4}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["38: tantan1", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myColorMap", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ab", ",", "ar"}], "}"}], ",", 
     RowBox[{"Hue", "[", 
      RowBox[{
       RowBox[{"Mod", "[", 
        RowBox[{"ab", ",", "1"}], "]"}], ",", 
       RowBox[{"Mod", "[", 
        RowBox[{"ab", ",", "1"}], "]"}], ",", 
       RowBox[{"1.", "-", 
        RowBox[{
         FractionBox["3", "4"], " ", 
         RowBox[{"Mod", "[", 
          RowBox[{
           FractionBox["ar", "2"], ",", "1"}], "]"}]}]}]}], "]"}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   RowBox[{"Tan", "[", 
    RowBox[{"Tan", "[", 
     RowBox[{"x", "+", 
      RowBox[{"I", " ", "y"}]}], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", 
      FractionBox["\[Pi]", "2"]}], ",", "\[Pi]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", 
      FractionBox["\[Pi]", "2"]}], ",", "\[Pi]"}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.3900267260228496`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJztnQeQXtWZps/s7OzMbN6pjbWhavNu7W5tzlu72J7xrNnFxtgGgxVanf6/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   
   "], {{Rational[-1, 2] Pi, Rational[-1, 2] Pi}, {Pi, Pi}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["39: tantan2", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myColorMap", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ab", ",", "ar"}], "}"}], ",", 
     RowBox[{"Hue", "[", 
      RowBox[{
       RowBox[{"1", "-", "ab"}], ",", 
       RowBox[{"Mod", "[", 
        RowBox[{"ab", ",", "1"}], "]"}], ",", 
       RowBox[{"1.", "-", 
        RowBox[{
         FractionBox["1", "4"], " ", "3.5", " ", 
         RowBox[{"Mod", "[", 
          RowBox[{
           FractionBox["ar", "2"], ",", "1"}], "]"}]}]}]}], "]"}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   RowBox[{"Tan", "[", 
    RowBox[{"Tan", "[", 
     RowBox[{"x", "+", 
      RowBox[{"I", " ", "y"}]}], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     FractionBox["\[Pi]", "8"], ",", 
     RowBox[{"\[Pi]", "+", "0.01"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", 
      FractionBox["\[Pi]", "3"]}], ",", 
     FractionBox["\[Pi]", "2"]}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.390026727414851*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJysvQnYpWV553nT3Zk9M5menu6Znul0ucUdBZEd6iuqitqXrzY2hQ8EBEVc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   
   "], {{Rational[1, 8] Pi, Rational[-1, 3] Pi}, {
    3.151592653589793, Rational[1, 2] Pi}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["40 tantan3", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myColorMap", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ab", ",", "ar"}], "}"}], ",", 
     RowBox[{"Hue", "[", 
      RowBox[{
       RowBox[{"1", "-", "ab"}], ",", 
       RowBox[{"Mod", "[", 
        RowBox[{
         RowBox[{"ab", " ", "2"}], ",", "1"}], "]"}], ",", 
       RowBox[{"1.", "-", 
        RowBox[{
         FractionBox["1", "4"], " ", "3.5", " ", 
         RowBox[{"Mod", "[", 
          RowBox[{
           FractionBox["ar", "2"], ",", "1"}], "]"}]}]}]}], "]"}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   RowBox[{"Tan", "[", 
    RowBox[{"Tan", "[", 
     RowBox[{"x", "+", 
      RowBox[{"I", " ", "y"}]}], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     FractionBox["\[Pi]", "2"], ",", 
     RowBox[{
      FractionBox["\[Pi]", "2"], "+", 
      FractionBox["\[Pi]", "8"]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", 
      FractionBox["\[Pi]", "16"]}], ",", 
     FractionBox["\[Pi]", "16"]}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.3900267223475647`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJztvQm4XVWVrv2l7PX6l//1Vv1l1fW/h7LUqluNWlplU1IGQZHeBNJ3J31C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   
   "], {{Rational[1, 2] Pi, Rational[-1, 16] Pi}, {
    Rational[5, 8] Pi, Rational[1, 16] Pi}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["41 tantan4", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myColorMap", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ab", ",", "ar"}], "}"}], ",", 
     RowBox[{"Hue", "[", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"Cos", "[", "ab", "]"}], "2"], ",", 
       RowBox[{"Mod", "[", 
        RowBox[{
         RowBox[{"ab", " ", "3"}], ",", "1"}], "]"}], ",", 
       RowBox[{"1.", "-", 
        RowBox[{
         FractionBox["3", "4"], " ", 
         RowBox[{"Mod", "[", 
          RowBox[{
           RowBox[{"ar", " ", "2"}], ",", "1"}], "]"}]}]}]}], "]"}]}], 
    "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   RowBox[{"Tan", "[", 
    RowBox[{"Tan", "[", 
     RowBox[{"x", "+", 
      RowBox[{"I", " ", "y"}]}], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     FractionBox["\[Pi]", "2"], ",", "\[Pi]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", 
      FractionBox["\[Pi]", "5"]}], ",", 
     FractionBox["\[Pi]", "5"]}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.3900267183017473`*^9, 3.39003922587661*^9, 
  3.390039259985656*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJy0nQe8WFWV7j9AcdRR56nPMs4bcWYcZ8Y6b8YyzrwxgCiCAtKRlJsCaZDQ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   
   "], {{Rational[1, 2] Pi, Rational[-1, 5] Pi}, {Pi, Rational[1, 5] Pi}}, {1,
    256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["42 waves1", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myColorMap", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ab", ",", "ar"}], "}"}], ",", 
     RowBox[{"Hue", "[", 
      RowBox[{
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ab", "2"]}]], ",", 
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ar", "2"]}]], ",", 
       FractionBox["1", 
        RowBox[{"1", "+", 
         SuperscriptBox["ar", "2"]}]]}], "]"}]}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   SuperscriptBox[
    RowBox[{"Sin", "[", 
     RowBox[{"x", "+", 
      RowBox[{"I", " ", "y"}]}], "]"}], 
    RowBox[{"Tan", "[", 
     RowBox[{"y", "+", 
      RowBox[{"I", " ", "x"}]}], "]"}]], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", 
      FractionBox["\[Pi]", "2"]}], ",", "\[Pi]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{
      RowBox[{"-", "2"}], " ", "\[Pi]"}], ",", 
     RowBox[{"6", " ", "\[Pi]"}]}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.3900267155978594`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJzEvXe4ZkWVt72eJgmnc9N0k0+TFATFgBHhYBp1jBhGfQ3NmMcR8xjG0UbU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   "], {{Rational[-1, 2] Pi, (-2) Pi}, {Pi, 6 Pi}}, {1, 256},
   
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["43: logsine", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"myColorMap", "[", 
    RowBox[{"ab_", ",", "ar_"}], "]"}], ":=", "\n", "\t\t", 
   RowBox[{"Hue", "[", 
    RowBox[{
     RowBox[{".52", "+", 
      RowBox[{"1", "/", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         RowBox[{"ab", "^", "2"}]}], ")"}]}]}], ",", 
     RowBox[{
      RowBox[{"Sin", "[", 
       RowBox[{"Log", "[", "ab", "]"}], "]"}], "^", "2"}], ",", 
     RowBox[{"1", "/", 
      RowBox[{"(", 
       RowBox[{"1", "+", 
        RowBox[{"ar", "^", "2"}]}], ")"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell["\<\
QComplexDensityPlot[Log[ Sin[Exp[y+I x]]], {x,-2.5,4.5}, {y,-0.5,4},
QComplexToColorMap->myColorMap, PlotPoints->100, Mesh->False, Frame->False]\
\>", "Input",
 CellChangeTimes->{3.390026717240221*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJy0vQe4ZdV1pTvc3faz3ZJsqdttgSQLWVawsDKyIlBQ5CJnRCoQGUTOIChy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   "], {{-2.5, -0.5}, {4.5, 4}}, {1, 
   256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["44: etantan", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"myColorMap", "[", 
    RowBox[{"ab_", ",", "ar_"}], "]"}], ":=", 
   RowBox[{"Hue", "[", 
    RowBox[{
     RowBox[{".5", "+", 
      FractionBox["1", 
       RowBox[{"1", "+", 
        SuperscriptBox["ab", "2"]}]]}], ",", 
     SuperscriptBox[
      RowBox[{"Sin", "[", 
       RowBox[{"Log", "[", "ab", "]"}], "]"}], "2"], ",", 
     RowBox[{
      FractionBox[".1", 
       RowBox[{"1", "+", 
        SuperscriptBox["ar", "2"]}]], "+", 
      RowBox[{".9", " ", 
       SuperscriptBox[
        RowBox[{"Cos", "[", 
         RowBox[{"Log", "[", 
          RowBox[{"Abs", "[", "ar", "]"}], "]"}], "]"}], "2"]}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   SuperscriptBox["E", 
    RowBox[{
     RowBox[{"Tan", "[", 
      RowBox[{"x", "+", 
       RowBox[{"I", " ", "y"}]}], "]"}], " ", 
     RowBox[{"Tan", "[", 
      RowBox[{"y", "+", 
       RowBox[{"I", " ", "x"}]}], "]"}]}]], ",", " ", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", " ", "\[Pi]"}], ",", 
     RowBox[{"2", " ", "\[Pi]"}]}], "}"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{
      RowBox[{"-", "2"}], " ", "\[Pi]"}], ",", "\[Pi]"}], "}"}], ",", " ", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", " ", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", " ", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.390026711221566*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJzEvQm4VtWV5726qru/6v6+6v66q7qrvBflIo6JmjgPOJA4x6k0UTERvWqc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   
   "], {{-Pi, (-2) Pi}, {2 Pi, Pi}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["45: etan", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"myColorMap", "[", 
    RowBox[{"ab_", ",", "ar_"}], "]"}], ":=", 
   RowBox[{"Hue", "[", 
    RowBox[{
     RowBox[{".65", "+", 
      FractionBox["1", 
       RowBox[{"1", "+", 
        SuperscriptBox["ab", "2"]}]]}], ",", 
     SuperscriptBox[
      RowBox[{"Sin", "[", 
       RowBox[{"Log", "[", "ab", "]"}], "]"}], "2"], ",", 
     RowBox[{
      FractionBox[".1", 
       RowBox[{"1", "+", 
        SuperscriptBox["ar", "2"]}]], "+", 
      RowBox[{".9", " ", 
       SuperscriptBox[
        RowBox[{"Cos", "[", 
         RowBox[{"Log", "[", 
          RowBox[{"Abs", "[", "ar", "]"}], "]"}], "]"}], "2"]}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   SuperscriptBox[
    RowBox[{"Exp", "[", 
     RowBox[{"x", "+", 
      RowBox[{"I", " ", "y"}]}], "]"}], 
    RowBox[{"Tan", "[", 
     RowBox[{"y", "+", 
      RowBox[{"I", " ", "x"}]}], "]"}]], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", 
      FractionBox["\[Pi]", "3"]}], ",", 
     FractionBox["\[Pi]", "3"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"\[Pi]", "+", "0.0001"}], ",", 
     FractionBox[
      RowBox[{"13", " ", "\[Pi]"}], "8"]}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.3900267045920334`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJycvQfYnlWx7n9jx+12i569NQmEEKQ3QUARkAgiIGChCCpIQNgIAoqIqIgG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   "], {{Rational[-1, 3] Pi, 3.1416926535897933`}, {
    Rational[1, 3] Pi, Rational[13, 8] Pi}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["46: flowder1", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"myColorMap", "[", 
    RowBox[{"ab_", ",", "ar_"}], "]"}], ":=", 
   RowBox[{"Hue", "[", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"Cos", "[", 
        RowBox[{"Log", "[", 
         RowBox[{"Abs", "[", "ar", "]"}], "]"}], "]"}], "2"], "+", 
      FractionBox["1", 
       RowBox[{"1", "+", 
        SuperscriptBox["ab", "2"]}]]}], ",", 
     SuperscriptBox[
      RowBox[{"Sin", "[", 
       RowBox[{"Log", "[", "ab", "]"}], "]"}], "2"], ",", 
     SuperscriptBox[
      RowBox[{"Cos", "[", 
       RowBox[{"Log", "[", 
        RowBox[{"Abs", "[", "ar", "]"}], "]"}], "]"}], "2"]}], "]"}]}], 
  ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   FractionBox["1", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", 
       FractionBox["1", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", "-", 
           RowBox[{"1", "/", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{"1", "+", 
               RowBox[{"Sin", "[", 
                RowBox[{"x", "+", 
                 RowBox[{"I", " ", "y"}]}], "]"}]}], ")"}], "2"]}]}], ")"}], 
         "^", "2"}]]}], ")"}], "2"]], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", 
      FractionBox[
       RowBox[{"11", "\[Pi]"}], "8"]}], ",", 
     FractionBox[
      RowBox[{"3", "\[Pi]"}], "8"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", 
      FractionBox[
       RowBox[{"7", " ", "\[Pi]"}], "8"]}], ",", 
     FractionBox[
      RowBox[{"7", " ", "\[Pi]"}], "8"]}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.3900267057236605`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJzsvXe03VW5tj3TCIQEQglFOBhARDT0jpSAIBJp0kGRQASkiAiCiggxIk1E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   
   "], {{Rational[-11, 8] Pi, Rational[-7, 8] Pi}, {
    Rational[3, 8] Pi, Rational[7, 8] Pi}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output",
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["47: sin", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"myColorMap", "[", 
    RowBox[{"ab_", ",", "ar_"}], "]"}], ":=", 
   RowBox[{"Hue", "[", 
    RowBox[{
     RowBox[{".85", "+", 
      RowBox[{
       RowBox[{"Cos", "[", "ab", "]"}], "^", "2"}], "+", 
      RowBox[{"Sin", "[", "ar", "]"}]}], ",", 
     SuperscriptBox[
      RowBox[{"Cos", "[", 
       RowBox[{"Log", "[", 
        RowBox[{"Abs", "[", "ar", "]"}], "]"}], "]"}], "2"], ",", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Cos", "[", "ar", "]"}], "^", "2"}], "/", "2"}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"Sin", "[", 
         RowBox[{"Log", "[", 
          RowBox[{
           RowBox[{"Cos", "[", 
            RowBox[{"ar", "+", "ab"}], "]"}], "^", "2"}], "]"}], "]"}], "2"], 
       "/", "2"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   RowBox[{"Sin", "[", 
    RowBox[{"x", "+", 
     RowBox[{"I", " ", "y"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{
      RowBox[{"-", "Pi"}], "/", "2"}], ",", "Pi"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", "2"}], ",", "2"}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.3900267068452735`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJykvQfUVkXStltjdsxZx4Q5AZJUFFHEhAkzxlEMGBGzOCr6ChjA7JhlFPOM
OuY0RsyOOQ2OYRRzGDEroqjnudxXne4Xz7e+86+ftZ71NvvZu7vqrruqq3t3
97PE7gduNWTaiNjTD+URrU/H1ufc1mdo67Ny63Nx6zO49enW+lze+uzW+qzS
+lzX+mzb+vRsfW5rfTZpfXq1Pve1Pn1an7Vbn8e9t2/r80Lrs1Lrs0Hr82rr
s0Trs3HrM6H1WbD16d/6vNv6zNH6bNX6vNP6zND6bOf9P7U+O7U+T7Q+37Y+
A1ufG1qfico/uvV5v/XZt/XZo/V5s/UZ0vqs2/q80vocpkzPtz5/8r4nW59j
W5+/tj6PtD4jW5/dW58HrA997259zvRze+tznhjc3PqMaX1OUg7uXVVc/tb6
zGq9N7Y+87Y+V/s8eFzZ+tyrble0Pg+1PgdZxz9bn2GWn2t92iyPb32O8v43
Wp8DredtZab+D7UJ7X7a+vRTni9bn7WU8ztlQP7J2hi5pkRje/T9WVuBQ/yu
KT9iuZO4/dL6dBXPKdYJzt+3PmuI/1fRcAG7fBKN7bHXhGj4gh1fan221L4P
i8kMyrdzNHy4WB3hCRzdOxr+7NP67K+sm4ghcnTW3n3E/ki/h+PHiNF0Yost
V2h9ToiG5ydHY/uh6niGbW4RjV+c4fWL/Zysfa6xHvC+1fpvE0PaxS+eUp7H
xQo58Yv3lA+ef6FeE8SVtvGLmWwPv5hHfLh/cXHDL1YUzxvEYYK6rC3++MXG
2gW/2Fp78f0fbQ+/2Esd4RE+hN3B/wjL6AtH4Ql+MdJnd7M9eHWS38E3/OKc
aHiIX1wYDT/BAh/6UHkvjYbPcGBsNDxPvxgfv/UL+N/Rti6Ihsu0f2o08eq0
1ud46z5L+XtqR3jTS0wHigFtDogmXmHPTaPh7A1eA7s7rIN4db82JF491vos
4/NPtz6LRBOvnlfPXS2jP/EKLkwn3g+IH1hfG41fHNr6nNL6fBZNvBrU+nwU
TbxC1nfEfXlxGq1cr4g7nH8xmnh1euvzrFgvFI3/guP20XDxr17Hx28UT2IS
caFP6zMumng1o7Ly3czqT7xayTJ22dLyePV5QPmOtR7kHm4dH3qdduHDYcoD
T/ZRTvizs/LDqy3UC7w2VN9flJN24OeatkN5DXHjnjXEk2d7iTP8X8trX2ln
7PKJ9WOvt6LhAnZ8SXtj34xX2P2maHwIPmS86qmue1svf/e3LrgEt+FLJ+09
0DaGet800cT8I5QJ/sNn/Py4aHg+yjYuUEc4gV9srh1v9Dr8uMf7TxNn6jlL
naj/XHWdRh3+qzxjW59JyolfTKv8+MUc6oVfLKws2H8568AvuonP02INbvjC
RrZNeWvbxi92EX+4s492wS8O1l74xdHaDL8YoY5riwl2xy/+bBk7XOQ9+MVl
Pnu6+sAr+H99NHzDL4ivX3odP/hUPO+Khrd9oukr4Rl+gX/AP/yCWA//p/YL
rncUo1uiiVfIfrUYjRSzVSzjx8h+YjT9Uy9lOEpdsSV5QF8xAYsN1GtHbXS2
bROvzo+G0/CXWNA7Gv4RC3pEE6/+pty7arOloolXad991Zs+aIh6zxINf7kO
j/4kfvTp+PfYaPgzUj3AFT7uLq70qfjmB9HEq4XFFRnxszeVcZvW57Vo4hX6
4vs32v6/tNPvo+EUcpFH0L8Sa/AbeEa8WtMydpnW8njreUE7drWed8TyX9p9
qO1+ql1eq/RBzu/Uk+cmayv0mqKteA4e7uFz8HNH8aG8lbhxz2biOUVbIh/8
30j8v/I6eOEvm2ov/GgL7fiiuPUQhx20e8Yr+HCRtoAnGa+QiTyAODQoSrxC
h4xXRynHUO06jbY/U7nJteAzcQYeXC1OxK5b1BG74Bf9bfufXh8h5qPF823r
oTzR+k8UD9o9yWvIg1/MqZzwZBHlxy+WVy/8ortt4hdri8MY7wUfOLetuOEX
A8WT7/YTZ/ziMPG/Wz2xy73K9H2UMQR2xC8yVo/1g93B76+WscUNUfqszM8X
tq7J6kR8/E770od+qb7kgvAMvhBP4S1+8UyUcRXcfyOKX8D/qf2Cv8Srtmhi
9sqWiYHdLBMzV9GmxNWelvHjXuJxivgO85m+lg+PJl7Bk/3Fnevkkf0t079u
ZZ34wwBlJn8lXo0UI+IVuTh+S7yijyMfJ17RD3SIJl6RezKOgL/km3NF0yfQ
Z84cZTw4rXVfI/aj1A+8z7BO8KbPPFq84Q7+Rb5xubK8Z33wcUI08Yo+803t
upF2gC9wkXgC31o0+DW/xx92t4w9FrY83nZf8/nMA99Rdur/0PtpF5/ZUHmQ
+0Dl/E7ckH9ypdcUy3wPD0/1GoIdbxuUjxA37hksnlO0ATjD/13En3p31C4f
R9PPgtFblrHjC96zrjgQozYTt4HR8OFC64cnyJLjwL2U4XBxJV61RcPhzKcz
XsHPjFfYdfpoeAifiZvw4C6fgXsPq2Obduhv+bXq+gfeT/kr6+HZn63/OLGa
xvrnUx54voRyDvPvRsrTU72GicnxlvuLA3XuID58t4e4wd0h4ol9jxTnHBN9
rH1P0S4X+n/shV+M0Y5X+snxIONb7I5f3GIZ/P7hPdSR4xn4SVyarIxwGT5h
82dsd05lhJ/4xcvR8Bb94XiOF/4dDb/TL/huar/4d5R4Ra6Z8erBaOIVWOUc
DPhfK77HqHMvZWasSp4KN/B74tURPkO8It7DJ+LVIdHwD3vAN7i+ldgTl4lX
B3jvjpb7RBOvKK8WDZe5n7HDvtazjNdoaxHbw4Y5n4Ess/kXW8OvEdrzZ+U+
S3sSr+A7nCRenS/eF2uvjFf0pxmviA8TouEVecV/xG7p1uf1aOLV9OL/kLb7
tzbewfJz6vhv7bW592PHg60H++5n/dj9j1Hi1dzKg9xzRYlXvfz+B5/l+5/U
HX3h4RhxgDBjlZXyGHHjnrPFE66eLM7kYcPE/0vxxy4faw/s9aZ2wo5wd2/t
C8/wVex+g9fhA/39Pn7aoolV2HOQdWLDfmKC3Vay3XPEgHgFP3M8eG2UeIVN
OloH7ROn4fnTUeIS+G9m+b3q+ufV/T9W9cxQ1T+37eIXi/p//GIF5cQveih/
zrGi1yFiQjsHickF6rur+Bwgjg9aPlQ8D7DtnMM9Ufyp5/TKLudrryPFZ0qU
PDPHwn9vfbA73LjNMn5xt/fgF+Oi4Q/1PRoNr/CLjFe0T7yCZ/gFORL8wy8Y
I8Bb/IJ89R2xY4wAz9Mv4P/UfvFqlHjF/EbGK/ypq3a5M0q8IvZmvBobjR/A
h4xX8CTj1WFR4tXBYruReMJD+IDv4G/k/XCVMTjxag/tSbwaaN3YDC738Hvm
ZFayLvKwJdSJZzK/4vqc2od2GCcPs07sQLyCK98r9/7a9gyfZ/4h8yvmkIlX
9AUToulzThbjHA+C7Q3aK8eDcJWc/V7bfV58e2kHbDyPZT7hX+y1gve/odzU
g33njjIeXM924cOS3vtVJSf86aj88Gpe9fpJW6IvPNxXHMDlTP/yuVrciGV/
F094fp04w//LxZ+6LtAuxMrTtBd+NFI7os8x2ndcNJzZ2roP007Ukf3acZax
x57+hVcben/GK+JAxivsPTZKvELWjCd3RokzDykjbTyrzm3R+AW+1SZuef2L
6v4pVT0zxm/jFe1mvELH5aPE1e7Kf7CYZP/bz7r3ExNw2FtMwAfe7yVuA6OJ
R+CJX/xJnPGL4eIP/0dHyXvP1l5cv1AdqPsy7Uud16jvoCixC7/gncIv4k9e
Cn/wC8bOP9gmcTTzeeIYPMzxIPzEL5hT/VB56SPesV3wh7vpF/B/ar94KUq8
os6MV/hWxivGtz20y1VR4tVFUeLV6VHiFTEg4xX/X19cyeM38i/yEa/gI+Px
LbUZ/QvxitjUR4zBdhVtwlhhJe2GPTtYB3nIAtHEK3hGPpD9FXMI8Bcu0CcO
s/y1siIT/jvKZ+lPiVfMt5DDnKtMxP+LlYO4gY3hEv0I8QruEXtuEBPsRbwi
N39ITGlvnP9Hjgd8Zn/LzynDA9prae9/Qz14DvvOa/1wcj/rmCgm2a/1Uk74
M4fyZz+IXvBt3Sj5w6LiAD97iw/lA8UNPl+oHPz/ZnGG/3eLP/58l3ZBtpu0
F+1drR2fF8c+6nmWdrpeG8AH+m3i28BoeAcP4c0e4gavcp4BTFe0DA/nisLP
abwf3k5vXcjdyTrv9RnKT6hvWzR+wfs7ym9V1z+p7v/Oeo5tvv5/49WsUeLV
AsqDvEsoJ37RUfnxi1XVa7CYUN8+YgIO+MU24oNf/FHc8Iu9xBO/GCLOW9se
+G9ufdgFXp2ovaj7NO2IL5yjffuJGwrhF2MtbyqG8AS/IF+FP/gFfIdX+MUt
4oJfwH14iF/AiYlico+y5TzhO9qQefXsl5P/U/vFE2J3vLbLeIVNM14hZ8Yr
+JrxCrtlvCK2Zryin8t4hS9lvALrjFdgmvEKjLbUZsyvbavutE+8Iidc1mvY
bWHvxRZzVbYlJz/A9ug3iFfrRON3yIUPfiIezKO8LU7oQ39Kn8DY5QV14/qT
2pL7md/Az9YW58tsC1v8VX1u037w5yZtxhj879oJTtHXE3f+pN3/Kc7Xapcs
j1eH67Tj9tbzjtjcpN1ns92J4oE8X9kucsKfg5UfXp2hXvCtg/rCw0vEIefY
37a8grgln8ETnuf7Jfh/lfjjF/crB33o49oL+R/Vjs8pFzrAxVu0O/54jW0w
5vhLNDzBXvQZu6oTtkleEd/gGzLii/CQ/G+k9/3Oaxcq33BlZbwK528Wj+R/
zt3hFxt5/SWvt0WTv5yhjT62HvziG+s/RixpF7/4vfLAv3mVE79YLIpfLK9e
uYYIfeF0L3HYw3vBBx/YTNzwiwHiuY3fgTN+sZf44xcHaJf1bRt7raP9sGNv
9cS+8Pwk9YX/p1nGL85Wt57K9ZP3E9/gFX4Bh76zLfzjK+Vlfgx+4hf0Wx+p
M34Dn/GLvyl/+sX4+K1fZG48QjxWVvax0fAJG8GVjFejosSrYVHiVea12IU8
MuPVDpVdkDvjVd8o8Yr+JeMV/U/Gq8V8ns88XoO/M3lvfzHLmPeVtllXPA7R
Fm8qF3K/rNzk47ynGK7OxI+To6xHOl1ZyIGJVyuIz0Vev1p7dBSra8SLuZ4b
bItcnni1jhgSr+gj/hzlvRgcIF7RH52lXQ7z+ngx/3OU9ybnat8jrR89B9ou
fFhDecBiPuWEPwsof85f3Sp2Q6PMS4xQLvi5tPhQ/pu4wecB4gnPV4kyP7a9
beIXp1o3/nKb9nrD+rAjef+L2pfY9ryYwkviW67dIX7AE7hGXNlMXa8WO8bB
+Ad8w+fxIXg4p1gOa8T/1efg7Qz+BUNsfpJYce8J4sMDxDr8op+YPOz145Wd
+9uiie0ra6P3osSrz6LEq++jxKtplRP7zqr8+MX86oVfLG4d+3gvOMD1ruKz
iza+Xwz6KhN+sbE44xdbin8/bYNd8ItdtVdvccOOcHV/7dtdmVAA3XL9VUf1
+SXKWhBsjF/gQz94/cQo8w9g/ZWyMyacGCUGwhv8gtgPn/GLM5V/BzGG/1P7
xdTrr+gvRlpnV2XBTj20Ee9liLVwgfmOjFfkqMSrI8VqHetf3w88Wj2aeEVM
IY/OeLVElPEgPpbxauYo8eqnKPlV5g7Y5QN1wi6vR4mFz9sm/TbrV+ALfoAP
HKNut6of+sMF4hVrXohDxCvewxOf4EyHaOIG/8fPTvW+xcSOeLW49RGvmEeC
x7dZJ30Z8QoO0H88JB7IRbzawPJz6j9Uew3y/jfE82jt22b92P0E252oXU4S
o0HKSX+6tvLDq4XVA0yXVY+MV+CQ769vtby/uP1sPeAJz68X53x/Df7MR6+n
XT7Q1l/53aW2yXj1kSjrbP6t3YmLjGeWUFbiATw5ynbAa1efhVP4H++sthRj
cIeHjH3h9b6N+L/mafCWOSZiGnzG/vgBPD/bvxeo72if3zDK+quMY3d7P3g/
GqVPfy7K3OhrUeLVe8qDX3ymnJnjIP/B3pvvkn7vc/jF3OKwpzYDH3xgKXHD
L1YST/DvJs79xQr88Ys+2gW/2EB74Rebakf8Yivtiz7bqS9+sbNl5OZd7S+2
PyhKfg7O8ApfGKxulJlHwPaLq2fOrzL/9ZFYYBf4nLHxDWWnPD5+6xf4JXLl
ml7i1Sne01V7IUMPvx8YTbxqEyPiVc6d5vtBsFhHWZAj4xV6ZrxaQLz21UZb
aJeflWtXdcUmO4r9H6Osd8p4Rd+S8erhKPHqrijx6u/qj12uUN4u0fgsvKNf
OFNdecdF3DpNW6An8WoRdeMZ5pjJ3cZG0zfSJvFqPmW5wTL94m3qCm7EK3gH
54hX+CV8yPWiO2uX1SyPF7vM8wdYD/bdx/o/Ui7azfnMwVHmOZET/hzvfT94
L3r9pM3Q9xf1zXn2XPtD+Upx+1kMwROery7O8H+U+H8uLtiFeeqttNdryo8d
mWM9QRnvESPqZmxAHgcfiAnvit2ffG4psZgQDa/WibJGh7jLGBcezB5N/rFN
I/6vcRfciCd/izLuIk4P1b7oOkJ9+f/pYo9fXOh18LzS+zOOdfO5f1h/mzjQ
Ljx7RnnQ4RXlPFwdkB+OfqxeGdtzL8aP4oBfTCs++MUs4oZfzC2e6LqQOPfX
lu+Lx3JR4lWnKPGqe5R41VMb4Be91XdFn6GMXxC/kyfM7cGfRWwPXuEX9B3w
bf4ouQW+gO0nWibf+0jbogN8xi+Yj4Pn6Re5XrT2C+I+8WqMdiRekS8TS7tq
L+IH8Yp+ZyOfO0G91tBG3LuWNlpGWwwVQ+xOTJ1V/HK9f8Yr5kMyXn2onruK
fcYr+J/xCl8nXjFOZ23c3sp1nTrQb42Nhgu91SfjVa7rxS7HR1nrjKwZr+gX
iFf0C8wD0Mf9QZku0hbb2wZ8IRcnXs2h7fDn2ZUJX2R+iX7iHu9ZTx14Fj78
U1v31S4dLKe91ovS72ygfTe2/o/EhXbpvw9SnnyXgJyTtCPyw8+z1Au+jVTf
X8R/qLbpJTa5NyfXfOwvnvDzZjH41vqvU4bDtQv+8pi6viq+2JFx4bbal3wF
Pnwohpcq+5niQvtD1Rn+wF14D6/6RLOmbBk/tNdV7Km7d/P4r22C1UzKQtvd
lXE39SFXHGx7xOYjxX1MlDjGfadF2aNzgfXkPi7qz3El7bbZBvLgFw8oZ+bU
yJ/rEXYWh9f8fl91PlO7fCI+2Y+DG34xWTzxixBn/GJG8UDv2bQLfjGP9uot
htgRv1g8SrxaWn3xi+UtIzex7hfvBWf4g18QG+AVfkFsgG9wew1tCefXVAaw
IE7kvGsf9eQeYsYbUfwC/k/tF9iLeIWPEQOJV/QhGytTzi8j0znaZzXttlyU
+bmFleN4Zcr5PN6TEK+IF8T1XDNDn0K8gv+MAYhX+a4D7AdGk/fjb8Rd5hPw
S7h2nXJjl0t9biPl21+7kDMcpF2OtX3kPlS50G1f5V3J9jJebRclXtFGxiv6
F/ra+dRvbDTzyvj21eKf+cfvbYN4NbO4Eq9mEu+HvA4nmNud1fJz1rmi9lrI
+7HjUtZD3tHF+nMegHY/i5Lzfq1+fDcpyvt/eHWmesG3i9QXHp4fZZ3H8T7/
i/gf6v2rWdcP2hWc4f814k8sOVC7MBY6THsx5huiHcmDHtO+/4jyTp55cPoh
+HCGdX+s7XjuO2VhzAOv1o7iA2DDnPfC2oH39/AzxKh7lHk0nsv3lPkOjpxo
gLqTM8Kvvtoy4xh+MdT78YvjrQe9T7H+jGf8O0kcZvPevyon/LtZOdDtbvXC
Lx7y2v7iBA7w+0XxwT6viRvPvO1926rnv7XtZ+K/kfbBLvjFj1Hi1e+0Izad
QfvCKfI37I5fzKnu+MW8UeLVglHi1SLWO5/fwTc4zNjva/VfWpnwi2WVFf4T
D3Ov5ApR9uMk/6f2C/I6/AHu9InGH+7SDl21F/UQr+h3FlW/i3yeeEV/Pb0Y
5N5H/GSEONFP0b9MiCZe0f+S9xOv4DZzrMQr/AOfxucY6zCuIl4Rp+i/6IPg
FHMQu/nMaO0J747VzrSHf+EHxNB9osy7kXdnvKKdNu3C84ylMjdk3qdDNDnN
2dqF3D/jFc/k2nmeyXjVQcxm1o7EK/q6hdQNnOiHHrJMXcQr+DKvdpnZ6+Ot
c37tOK/1YN/FrP9jZaFd+NBLeeBJf7+bJJ7ID6/+pF70p2eob74fBIfMMbaJ
EsfADX4eKZ7wfHNx/kacwX+ith6tnI9pr1dsCzv+07/Ylzj0kthc4XX4cJrt
wJNcFzMhmnw73zXBt9wTuaRYgOmsUXwm/H5RcQUX+Lyq96+iTYmRfdSdOLaJ
/8cvpo5jF2rj/aPMMRxu/fjFcbaLX5ysPKP9P3KO8L7eYnWFelHHdeqLX9wq
DukXV2jHh6P030+KJzK+IM5b+Pdd7flmFD98r7LXJ1HiFePFjFd8n/FqsrrD
qZ+jjAen8f/4BdzNeDVzhf2sUeIVfcpnfj+X2OMXcysndiP3e8MyXB8fv/UL
ZM3+m7+do+ReyP6QchCv7qieuS7K+8yx0fRn4A+3yUPXEetXxYk8mXE28Yo5
yXHRcOJw7QL36X/ppxnPMMajDyNeDYyG6ztrV+ac4A1zI/k+MnO1fM/NM9ic
Pmxb2+lpm8xndlPGNvVGJ+IVsZ+5kYxX/J94Rb9N33GBeNC/ZLwC54xX2CXj
1czqNoOf3MvI58Fo5iSw+xP+nUbsp7M8Xpvyf/xpFuvBvnNbP3ZfJAofllee
r9Up98tsrPw/ig16/RzlTA54eLrP5P6LjGPXR5mvuEY84fGZ1kVbuV+G8dmG
UcZwu2kvdHlUOz6unbAvnHrSei8XdzAjX2E97Lho5uDIT+DPdtoVXhGbiQHw
jTkP5qC/FKe3lJd/34gVffsU7ddTLLDxGO/pqO4fiwM+/YY4cD3XAHE/fjHA
eu4QB+q/Tt34NzaaeI48+MUxyolf5N7bE8QdvfCL89QXvl4iDvjF1eKDX/y9
avN28cQv7hXnraKs00Pef2oX/OI57bW2+E5Wh9fFC7+YEI3dc40xusOXT6LE
K7gGf/CLr6LEK3Jg+DaXdWe8mhIlXvEcGM9g3fB5uqb4K8/TL8bHb/1iaeWi
Tfpu4hVzCfCfePWa9xOvmDucpC3HRbO2kHgFv+AH8Yqcl/6yTzTvZcCSeIXP
00fAZfoc+lZ84DjtQLw6QnttrY1Ga0fyJ/pu4tUu2hNbDfC7QT7PeB6u9PP/
8HqdKOtO1vBDvOoeZX0Z43LGDnAHu+DX8ARuLRAlXpEbE6/mF/eMV+B7lXbB
Ftf7/Q9R4lWuqeRe8vEHxRV7PqGtvtYuWR5f3f+fqh7sO4v1Y/e5bfcz5ZzW
55dTju+1GfLDq43Ui2d2Vd/cJ7ic5dPFB35eKW7w+RbxnByln/laG4L/f8UU
u7yt/bDXy5axIznXtdqX/PM+7T42Gr5RF/P35Mc3+Bc94U+enQOvVteuL0V5
NwsPf2/9v659jSbvmCRmE8V9DbGA55eo50LqznX8gjkX/KKz1z+0zUuUc13r
wS82s/5xYT4WZZ7/98q8r3L+RUwydz9KvfCLEeoLhqdEWV/2Z/HBLy4St73E
Cjzxi1x3Qvs3iT/2uVMs+ok19sIvHtaO6EBMw774xXPi0cn6Ml7l2jz8gpwt
49U7UeaveC+S8erjKPGKeJnx6vMo8Yr+BT5PK9b/ieIX4+O3fkFf3FH5ZtI2
k6Pkhl9YD/GKXBKbwX3yzdfV9UnrI17dKw59xAysiFfksvQ9xCt8Hl8nXp2m
LcAVvjHmJ14x7mDclv0pH8brxCZy4oHR5MPkX7n+F87ACfqUvtp2PeWF12tq
A8ZDjAPg5HHqDO7EK/IT/D/jFRykD19ETM/XLuBziXbJ9ZKziW3GK+yS8Qp7
ZbzCVhmvGMdnvPooyr6Djyp7fRyl3/lU+84Yxe6Zd3+uPZGHfGFR5YQ/K/g8
/VxP9YJvG6svDQ0UB8pHig+8OFPcePYa8aTOu8QZftwn/vTBN2uXCWKHvYgp
+2jHR7Qr9iX+4bMniSfP/DnKmR/w5AD1hD/kDvDtTuVgXge+MT4+Mcq4gWdf
F0vGbx+K04PKCxfeFh9ySfKCmdT9F3FcQ/0X8zp+sbz34xfdrOc9/1I/frG+
7SLn5spzr5hQF36xm/LjF/uqV+5BQt/T1AEchvsBH+wyWtwOFF/wHKSe4Ixf
jBXLbbRZnpVxg/rjF7dpR2SHm9gXvxin3vjFY+qO3rkeD79gzAx/8It/+Sx+
8WqUeAVn4eFsyvK5WLwbJV69HyVefRAlXmGv5H/tF4wViVc/+Mm+hLq7KhP2
IF5NUh5wxm+e0abIQv7ZW7zIVftEk//Tz9EPwW24R7yCd3ByY22ATZhrgbdH
a7sTtccArxGTiFf0PdidvID+ZTO/I/fCd7D91so4xDbwNfJy+hTmO7E5fRtj
72PVE3x5V7aimI72XrDDX/F9+hDiFbnJt8o+t3bIePWJ+s0s5hmvwDDjFX6S
8QrsntAWEyq7TKjs9XZlR+rJ8T71f2xbnyjHHP6FJ/NHOZ+qg/Lnnrecf+jl
8zTUXxwo7yE+v4j/Uj57jnh+r57gTB95v/gjz6PahT44/ZQcZ6y2giuMF7Fv
niFDG4wPyS9OinL+DTxhnEmOAX+YmyE/gVf4Fv0KfFtEOaibuEPe8YxYUver
2me0GDIeulGsiBnEvR/U91Px7em1ObwOLgtFWduxlPVMEpPMI1arbNhHefCL
jZQTfLZSfvxiJ/UCzz3UF24NFgf84jDxwS+OEbej/R6dDxILcM532rmX8CLt
gl9cJk74xV+14zpigX3xi9ujnM14j3rjFw9aRu/HxAC/eMpn8YvnrRO/IDfL
ePVKlHj1WpR4Ba8zXr0ZhefkyeMrHNMvqLuj95KvdY6So3VVDt459FA+OLCq
dsw1j/Tt9HfE6Y/EHTsx9h8bTbxiXhSub6Deo8QMf2bcjq/g84dE6WvAfYB2
wKaMN47TtuS/xCD6MmzMuJ7YuY824v3ZAda1rPX2E1/iFTyb17bRh3wDX+kk
dqN8Dv2JV+Tx+D7jlgXFHr+hHyZWMGZibPG++jMGwN9u0S7gn/NX/4kSr8gB
Ml69Xtnl9cpe/4n2dnzPet6x7Xx3jUyzeg2eZCwlL/iD8qPP0v7N9YHoCw/7
iEPux5nXe/YRN55pExd89EJxJte4Ocq5frmeE5mf1l7o9ah2HBdNDop9mYsZ
qX2ob1CUM/WO0Q60DwfhD3GQWDU2mrHLqeKNfvjvneKBrg+JH/LAW+LOidbV
1zbgOXND2BL+/6J8+AXx5E3x4Tp+MZv3p3/2FaMlrD/XefPva2WcUVx6Kyd+
sYHf4Rf91QtObKe+OZZExqu8Bj74xUHihl8cqbzHRVm3il+cLP74xZm2j1+c
r72w76XasV+l09rWnWvtcs0wfnG3Zez/gPfgF4/6LH5BTgnfch8DPMQv6A/g
In5BPgZHZ9AWyefXovC85n/tF9go1zPwHPHqaZ/t6jXu7SH+j6gHfTXvVFe3
TJ5JvCKvviLKu2bwJV7Bi1O10yvagXjFuPkI7UUsp2/ZUnmw17biyt9cp89z
xKuRtsM7cPrrVbQPc1i82xjsfbyTyTi4kPeS5zGfg2/AI/pA4lV3ccFPVhT7
M6K8myVeMWZiHgSOM8f4kTrPIV4ZryZEE69m1A7/0C7gn/EKnDNevVbZ5bWp
7JV2/E8U+1J/xiuuwcNZlOdb9ZuoDgsoP7xa3Gv44Aq2AQ9XjTIm2sDnfxZ3
cMPOQ8QCO58ozp9rI/Anp79TuyD3k9qLeEGOgR3v8/8DxYr+fP8o57jCB/Je
+ht4kmv84c8mPguviJUvi/2C6g0PZ4jyzj+xfER93hdj4ulFfkf8Oy7Kvm9k
gLuM9/5eYYVfTOv93Dub9YDr/FHmBzvY7o/iO4N4dVPOfI+L/PjFeuqFX2ym
vvnODxzwi13FB3n2FjcwP0g88YsjxQa/aBN/uH6ydsEvztReu4ghcuEXY5UX
v2D8iN17iyd86K6dMte6R0zgAlyGV/jF4+KFX5B3wUP8grwLfuIX8CB5Oz4K
n4ldyfM8/yrtl37BmCL3DxL3O2tH4iHxinwVf+puO2Czqs/hi8QrxsPklsQr
4jV+vLZYkMvSB9E3kefjB/CKvnMj5TtYO2UuS7xiToL+YFtx4toOYgmmfxRv
MCVejVJGnqefhSf4AJzvYBsDo/Fb/GFrcTzG+sBuuDrw72T1B3v4y/gdXrK2
Bn+Hc7wngn/ktvgMYyn8AT6T5zCOu9m6yYsyXqHzuPi/z6+ov86vkg/Ikzz5
LAp/kD/XJKcPLmMb8LCLdeX5DDz/s7YBtx/EsIP1HyHOn4lRd+W4RrvAufvF
F+5iX+x4j99h33w/k2OYe7UVNjxP+2Bfcgb4Qy6AX8Ir+nx4At+IF59a3/Ta
6RZVIy+At3P7/RNRzuqA5/m+7Q31JQ7Db96b4hdfev0in+H+u7RBP+Wf3fp5
bkHbxS+WUB78YkXl/E6sOkV5r049+MUGUdagbS4O4La9+DysDdADv8i1DmB+
iDhfEGUMPEod3vfaqdoLvzgnyu8iwGXsi19cqb7YLfdU4heZa3WNsucUvxgX
Da/wC8aJk8XgaevHLzJe4Rf0MclbYnTy+dX43/MrOE+8yjUyxKu2aOJ07lN8
SHyv1E7EK3hI7CVekd/hr2uKC3kr8Yo4Do9zTpa4v75y018Sr3hvwTwGc4GM
b5jfyPUjjP+YL3zSa9jsPm26c5ScjrnLS6PEK/oQuLG/usAZ+iH8gDyBd8V7
iCM2JCYyrqYv7xflLFv6wCnqgC/DM2xMPgzPmE9gnISfMF+SuRbxKseGN1s3
uhGvphfzcVHiT863v1XZ5S0xSBu96f3Uk/Z9X/xyvJHxCnkyXiEnPJxX+eHV
wuqVe8GoN/eLzez11cTnJ22Wa6G3FU/y/H3EmRzuWPEnXzhPuyD/jdqLfOEx
7XhXZd+r1BW7k8fSx8IHYhL9NjwhF8Cf4M96XsMmK1m+UP1yjRLYMJ67QVyP
tk3uaYuGz3lWzDPaaIoyoe+/or1f5FlhbeLK/bnen3rwi5ms/35x+531LFzZ
amnvwS86K/9XYr2e9uujvrS5kThg863EBxl3Fjf8YpC64RdDopxnNNR7L1Vu
7IJfjNJeXDtbO+IXF2vfPdQHfbE18Qo+4BfMz+feh8yvuohn7kV9wnrwi2fF
C794Sf1mVy70g2vEIviMXxCrkufJ/6n94j9R1jO8JI43qjfxir6GsWp3dSSm
r6ru5KTEK2I983i9vJcx8lrWRzwnXmF/8he4/3GU86j+K86beh3ebq4O8Hlr
daJfZmxPnN5Qmz1sOwOjnIGK/RiL03eR/8I3eHKgupBbMHd5hDjyDmYvcWyz
jRm9d6Mo+/vRjb6VPo5+F9+/yLrBjngFN+m7eK88t7rB7VnV7S7rpg8eF+Wd
SMarHKuE19NeH1Z2/Nj7prP+jFe0m2uPvopy5hV/M14hf65Jpl74tkSUnGpF
ceB6D/Hh/j5RcrL+6kyutos4I8Mh4o+dT9Yu+OtY7fWY+m+prZ7VvpepG3aH
K9h9iDYi34EnzHkxnoM/zA1f6r307Yznzxfvo61vWtu4Tj1/9P8LaEs4Csfa
ouH57dYJX4lbxAf8YgWx/NDr+MUX3n9ilDWzbbZJ/fjFbLaLX8zvd7TZQTlf
su7lxau7emHbNdUXTDcQh49tJ/HZQT3xi92VB78YLM74xRFRzos8znbwi1HW
jV+crR3RZYz25TliIXbHL4h98AG/IAdFMfzifq/3sk3uxy+eEZel1RP+4Rf0
Q1+q/3+UIccg8BkOElNzvpBryf/aLxgb5vwVeWe+H2Qc2kUc7hNTeEGMJV7R
n5JH9NReY6Os3wML+HqNGGML8nzy0vXUiX6ReEUOSCyhn5ogRpsrO2O+rdVp
GzF7w3t3Eg9iIWN7fJ7YCe+J/bnfhv4BbgzRJoy3D1WuhZUDns8b5VxHfJa1
DfjTTN67sThiYziV44PO2gf+LKV94Cn5CFzgPc6c2gp/Jf+hfx0XTZ9CnAHr
aSznOpPPKnt9HiVefRklXlE/udSMtpvx6gf/n/NPuZZv2ihrZtAr4xX60tAK
UfZcdIuSh60pbt9rs/Tj7cQZf9pH/OkLj9EuL4oX9sKHrtWOxPAHtS/vwcZr
d+Ic/ggfcs3Vn2yHeuEPY6fHtAljWfqJ89S1LcqZosNsbxptcFuUHPG+KOcv
g/0/xOWFKOcS4hfkCzmv+6MYfeb9+MUk6zlfO1H/sdp4GuWZ1zJ+sahy4hfL
KT9+kXuM8307+uY6a3CYEE0/AT74xfbihl/sLp74xWBxxi+OEP8nopw5iF+M
Flf84hztOMb6vhdX+Ivdc98QfNgvyrvCHaOcdY9fPCVma9o2WHUWQ3i4lO1/
KUbg+alYMC54X8yIYfAcvyAHfCV+6xcZr47XBjkezHj1RZR4RRm7048StzNe
gUHmV+QrjJ9zfwZ5+zr+PUEbcG/OG1HfIer9iLj0Vz7GeVspNzYaoB0ZU4DZ
v61jF3FiDEo++6Ry4UMPRHlXCPYr2B48xwfpv3N91bHqTp/I2IMYR25P7k8+
N6v3kgsSIy7URsSBsVFiPfYmDtDn0Cfn3A/z0OQ8k5WLmPG9WBN/Jqn3NF7P
eDU5Srying+8n/ozXnEj3JtFeahrTr/LeIX8Ga9y3cKS6pvxChzyXTbfwds1
xI0614/Ci63E+UNtBf74zaHaBV1O0l7o+xfteIN2300Mn9LuI9QT+zCWZ47j
GNtBT/hDTJwQDZ/AmNgF38hpGAteJmbUda1YHGt76IafwmdylG/F/j5lfT7K
e+/X1D/fIXK9TTnuU//vrOcLMaN+/GJm28Uv5o5ytswiUX4/YFnlR4+V1esy
/26lvOuKA37RX52eE0Nwgx+7iyd+cYA44xdDvRe/GK5d8IvTtFeOg/Ld7uVR
zrW+XrvD9TuinF0/LhplhlgXZfyCuAivNhU3+IZfvGOd+AVxKefzJorjAl6D
z3NoD+w/k8+h39R+kfEK/6zzK+IyvCSmkvsRr+6KMu76Sb1yLT9cgdfkkrk3
arg694lmzuhkbfAn9SffPUlsiVfwDiyY32VMlXPvtEtMYjxNXCduwZOntCP9
NOOLDbUf/RS+w3sU8n04MFi8yRsYr9+m3rRNngx/cj9Eh2i4ASb49iifwa/J
98nn5lZeZCU/uVQb0U/Qf3YS9xutjz6DeDWfNoAvcBde4zMzRvt9B9NG6V/4
/1tem177Tm/9E/1Lu9h+VuXJNXvImWv5+OQav1wXuqTyZbzKfTpdxedH8QO3
77Qf/6fP21yc6R93Fv9XxRq7MN96rPYijzg7yjrsv2rfc8ViiPe+rE32st42
MSbHhD+r2Da8Yu3lhGj4Rpwmdl3RqPLr3+vFiTh3u3qSQ9+vHNgJnj9knS+I
G/nHa9YPB96Ncq79RO+/RzyoB37mPhX8YmbbfT1KvLpK3JFzuDhTP37RRb3w
izXUF79YTxwuEM9jxW4HcaPdPb2GXxwozvjFUeL/rPpjF/ziTHHFL8ao9zix
QB/8gvcU2B2/IDbDB/zicTHIWEH5YPXMvRJgNUUd/htlPepXfnLt3kTx4l74
PJ/1wXP8At7D/6n9grZyvShxknhFTvikONK3k7t3F4fcCw12+CJzhW+q2xrK
Qm6Ze1HBtU80fSuxHb4PE0Pi1UH+nzHxwdqPOE2/Sn9B30x+w7wUuTfzgnCE
Pob+E1/AV+gH8B/67Putj2fgFmOT/SyvaZvcs6rt3a6u5L/4EfOgcIo+DF7h
J/CHvoE5T+YUFtWGA6PJcS/RRvOLBbkma1BuEN+5lBGOkvfgo4wV6FfgwayW
n9NGc0TpX7j/LctzRYlX1J/xina/tp6FosSrXDe1gPL/4jX0yrV/K1leURzg
Xrco6+zWELdvtB94fhrlvFj60R3En7Hd3tqFWDBUexGrR2uTK8UL++I/t2kH
/BU/GqYdmeeAJ/1s7zTly/e0zMN9bl3EiAlizz9yEvjJWgJyBnjLnApzNfAZ
P2+LhuePew/8p49nvIJfLKz+73k9363lO/tJUX63N6w/x4O0+6Y2Cu2xqHLi
F8spP37RVb3Qu5f6wtENxAG/2EJ88IudxA2/2Es88YtDxPla6wJ/ODdKu9wf
5XxafOEycaXMuPcb77lLvW8XR/iAX/w6h/S7xi9es3xSlHUg+AV8hG8Do6xV
hic/aTf0m9b78Au4C5/xC7gLz/GLnJef2i/e9Dm+Y34u86unxfBuZYa/xGD6
lVXUi/ebGa+Y24TXcCrfFX4Y5TwaYitz1XD9SbFmbEGMH6GNjtYexKv91B8f
YFwHt5nLOlAb5X6OgdqP+vAZYvxF3runMuc+HfJecoAh2nUd28i1rvRJt0Q5
j/Cv6n1SlLkq8vdcm4XtmSfA9+Ed3ML/r7LNDtHwYA3tcYfPLSp+i/iB//TD
C2uXWbz+b23E/ROiiVGLiesMUfadzhzlPJXZlGeyddIeXFlQ+XN/STftvIz6
cn0lcch9ZOtE2V+2YZTfNM99N5tG+U3h7cSf8Uful4I3h2ivW7QzdiQXPU/s
iWH4F3ZnHPmA92HTl7UrbZITwB+4mu9micG53g+cGNvAQ/7BSfiJP2Q+RP54
eZR5B+wKz8k72qLhPz41xuep/xXx/lq8vvD+u8WGeuDYNNZ/v/abyTrmUx78
YvEo71tWiDIe6q5e+EVv9UWGfuJwtDgfanu7iNs+/h888YsjxJn7h6sn+J2u
XS7y85Yy5z6dm8UKWe7UBpMtE4tz3yhYwBP84l3xoL6JXscvJnn/EepNPbuJ
B/yEO7nOBl7NKx7wFI5OiIb7xHL4P7Vf5Hvbc/0eThOXGcsTr37w/92ivNfi
3RGcIg7DdXJH5jQZJ7wTZS0WOBDDiVfkJcTldZSJsUG+wwXTnO8g12R+Fl7l
uvdh6s+4Hu7T9wzQbthrRzHZ02v7+xe/IQZtr21pB7sfoN3ADn8ivuS6HsYQ
8OU4y8hOH58506k+i97nRFn3NUYZwexK5elkHfg1sYD+qqe2gde8H4K3j2mf
FaKc87OC9pjF+8Fspii/kzWT9eeaYdqlP5tdeXKNMXLCn4WUP8ct6JVrldeN
sr8VHKZoY/CZ5HNbRzkDHDw/VC9wfkN7gP+z6o5d4H3OnRO3jxUjeHCGdh2p
rUdqlzu0E2N//Bee4MP/8TnyEnKCy9Qv32/kGid4GOJyl/gh37houM1f+Hyw
dcBz8qqTxZvn2qLxC+rHLz72Os99FWXvzo/WAxbTWf8IbTCL3y+gPPhFrsW6
WTvOIw6rqNeEaMYivcV6E3F4LspvcoLV7uI1zGvggl8cJc7YJN/rY4s/axf8
Yqx45L0fivfdtgm3H4+yJzbfmV4f5dyG67UB/Lnae3+J8ntE8G2UGOQ7R+LS
t8qzsHhuKibwGb9YxjaW94M9pvYLcrmO2ijXixKP6dvwAfhDH9Qtyt6wHsqU
a0fHeR/xCk4yBoLjxG5iOHPvN4pVvisEY+LVMeJJvGIskO/i/ivmmyk38m1p
PWC9jTajj95ee+Ebf9TG+4pN5ll8z7hxV+3GHAtx7mBl5hnGLmfYDlwYI6bp
U/207WVRzpy4QJ2wVb7H597DxeA62wQbxj2Mn1YTu1XE7zFttap2WdAyes/p
/dhxFuvBvr+3/s+9Tru55gp5cg0DcpLP/0H5c7/qppaXV1/uWVkcfrT9HaOc
7YYO9Inri+d7UfJW5Bwg/sSYgdrlH+KMva7xb5v2Hq19wZlc6dQo789z/zt5
0YXqn2sjkP2DKOe5fu0zjDHwjTsatX4dh9znPfT/yVvmceAzYyv6jJeVH5vC
f3g3Ksq5620+z/XTxePftvOz9eAXM0bxizl9dlyUtSNwbGnlxC86ew987qle
+MW66otf9BcH/GIH8UGO3LOEHAeL1422AS7w9RTxz/mvJ7XtVT6HX9yqnjsr
62faK9dNUR/jYvhwhnqi8xi/R7HL1S/PIJrNe2gz3zPhFx18Br+Ac/B21yhz
8fgFMWZCNH6xinJO7RfI0FH93o7yfhC7Eq/ImV+wrivUu4fPP2xd34k1uMMf
8oheYgiX4HvODfVR7suj5Pnov4F4nafscJ8xOn4FZ0drO3LSE6KMQ5CVscj9
tpFje2L6wGjmUrApPtZfe+BjG/rdgda7h1jyzC7WRR347HDbxyfhMu9VyPWY
873I55HxHOXFbrl+i7HOIbZ3q/KtL178JZ4Qr7paBtclovwG6vzeh23msB5s
Npv1f2F5U+0wl/L86LPImXPsyJ9zVjtaXjHKWXxdxWGyttw7yllJ4JZrgsBz
gniC0UtR9qU/IoZt6ru/9hobTX8AjqeKT9rpPD/wF46MUU/GcJdH+W3zv6oH
dodXeT7M7Y0qv/bz4DqN+sPPPK8C3vZShxe0KVyC54zb7hPjD2zzY+vHL770
eluUd7Un2Q714BfZfzxnm5n7LRrlfJTllPPGKOd4Hu1z6PWt+PIdfrGlOOAX
fxQf+LpvlDz1cPF8RVnGis+Z4o9f/EW75JrJl7TtPyo7Pi42tJfj4lxXNFmb
gkOO9cinUGZ0lD2neRZArvVZUjvgFx2j7OEifsDbQ9QdrPGLPtoAvhMb/h2/
9Yuc97pSjDK/wibEq+PFoluUNQ+0R25B3CdeMbalL+xpG3BmDeWjH+qtruTq
cJ/cEz/uG2V9GjLCT8bi5AGHqjM5MfNN54rrfeKOHxKDcuyA/HBoR/HGdvgh
PKWfwA/px+kP94ly5ij5dPrgoeI32Ps2iPJ76Dvb3klRYiF9DvaCS9iKfo5+
cKwyEkd5d8MYYRtxGqLsYISPw8nHtNWW2qWjZXxpMe9nPDGv9dA/z2n9X1qm
XfxybuXJOXbkzDkr5M85q32inMk22Pu7i8Mk7Qc+8God8fhQ22S/u4U4P6ss
YHO/bWIXxg25Z+1CccOOI6OcsQn+cOlS68D3rxIP8lF4sqIY3aROmd/MpExg
yT/mR+AhYzP8BH4uqG7wFh+Ak68oH34Cz8kx4Az8nxANx/I9Brz6Nsoaip+8
H1+Y3nooz279Zysj7WLvDlH2xuU+aGToYf3Yby2fwS828j6w2lYc7hFT8DlU
zPgM0S4jlfkkccbO54g/8l+uXf4lhtgLv3hAO17sNbCEF+SZ2B3+E8NyTuAH
sdwgyvo9/CLf7SHzkl7HLzp5P37R03rOVCf0znnrT6KsC4Dn+AXcejV+6xdf
+vf+KHkqOSVxdWXxR/6u4sJ7Bnh9rHWs4nW4QR5LP0m/tob2ol7mshjz36Vt
4AKxHz+AY/SV6ynf38QDDOEtY0P8aaz6MIcwRn3AljiBD+f+M8aG5KJniCVj
fTjF2PBmbbqX8oJXnt3BfNXB3k+fN1Qdhqor8RqukCsQW+nnmEPdI8pvMHDf
3tZHn5fz/adEWdfHPQOj4SDy02cyVqEv20VMV/c6eCwX5TdxFrYeODSv9TOf
Mo/tMgaaT3lyzgo5c84K+eHVcur1i/ZG3x+1JTjkGHBElHN0we1d7YE+8CD3
iNA37yD+d4o1dsFX9tNe8PRw7Uh7x2vfPa0Du/cXO/gAv8lD4An54Lho+EM8
Jj+CV7m3Cb7x73PtOqNYgOViYoG85KD4zxva61t1gvcf+hfMGSN+af20M9nr
1zTw/XoffjFzlPg7l/Vj3z9EWQuxVJTfEeuknPlOh/rxi77qNdE68t3J9lF+
yzfHB/jFQf4fvxgmnk9oF3CGVxeI/17KfWeU38PEXvDoEXG5WZ3fFb93lQVb
fCVOu0fZK4Bf0EfmumJ05Uv8oqPX91D2n8Rk3SjrUTeN8vtN+O5/vYdc4x3l
/2OU386o/eI723hZe4PpuGjGlMSr66P8nvAF2pp4le9Ik+PPR/ldwqesm76X
frGXOPIeeS1xpE/sE02/cJf6UB/8zD078LWf2JHHkmv9S/w313bUtZV4jo0m
NoP/GPXHjy9UZ+4/VyzpqxjPkVuToxJ7iPEPqBvzL3CS+HS0GB+vTiuo52jt
RR+HrzJGPdy2eZ58l/lIfJ24cZNl+kb6Uub3iS3Eq4GWscuGll8Vd+6HQ0ta
D3OdC1l/ng1Au/jVAsqT+5qRM9dZHW95BfXKdVajoowBwQEurS0+2GEDcWPO
ezPxxN7bqOu4aPJ58L8hyh60i9XjGjE+XDvSL7Rp351sM+3OM/Chi/feqx7E
d/jD2Jf5J3jFOAs+5nqCT5SL8Qk+Bp/IJTNGDVDnd6KcR5D7LsER/hMLGft9
Z/0vi+ULUfYkcP/1ykI9+MV8UeYLFrNd/GI55RmmTvyDQ2tEOVttfesCsy3U
9z6x6RJljUj6xaF+h18cL574xenifLF24e/u4ohd8Iu71SPXJj7vfbnXeDvb
+Ni64AV8wC9m9v/4xYJR1oiiY/JqFcv4RV+xwy/6+2zON4M3vjAwGj5nHgHP
8Yv9lInva7/AZsQrcmFieifxYcxMvHpEm8Jr+gVsTby6TP1ybEi+u6o6Mj6G
+8TMZ6OcQZzva/F/fHRt8YaH5FrEXvqz9cT7bmWE1/QPuQYebuc8/E3aGD15
37GN8l8b5Te3iBk7Rxlv7Cb+l0Z5r0J7g5UZ/hEL4MU54keb5AFt0cQP+gbm
EVcUa3jHfGqOc5AbvsJBYgzzp3CGPpd4wtzBcOt+xHtyTfIAr78mXtz/nm1R
DzxdPEou9Afb/cEy8sClxZQT/iyt/DkGRK98D4i++GevKGsI+0b5jZuNxO0V
sQbPJ5Tz2ii/fQD+mTNhl7PE9FblPtx79xLHu60v7U77xEL4gA8wXwVP5ovy
uzqMQ8gPnm3U+tXW8G0aZc13FMQU+En+iJ/A293FBT5fJxafiSF+8I11f+X/
+fe+dT9uWzN7Pzaby3rwi4WsH79YwnbP1mbIg190t65jo/w+DdzbUL3wi63U
F7/YRRzAZ7/KPkeIG34xUjwfEe9jxGCs/3/X+66Ksgbrzii/EYkd8Ys3te8D
UX5b56gKI2Rm/hSe4Bfwb4q2Xdn78AvykZwX3US84R6++IO67iGGcJ5YNDHK
GWi0O1wdX4vf+kXy9yvtSryif/laOV5VXuLVk9qa2HmPWMB3+hrm+IhXl9hO
zyi/qU68wofIfRkbHmf76IYv0j/2iYbP4EeuRRyGq+srO/MSuUbrAbHIcTux
m/zmbu2d+e8A72UehFwLXtPHwAP66MwFwAe/29fvrhYncufLbX9XdaOfzLUZ
5CjkBuQe9G30hcQtxkKM3eDZZVF+X+z6KPuuySFyDf0j6pX7Lfb0OnMLG3k/
ftPderDHMtYPfxazXfizqPLk3kDkzDEg8ud7QPSCP6upb86rg0PmVOADjzcT
N2TbWjzBdQdxJk4NFH9i9V7aBXwyZz2osiO2aKvse4p2zxz+KXW62jbnsE34
w1wQ4z94FV6Db+Q9E6LhITkmfTb8zDHER2I/RfxyTfrX4gAu8P/eKOeM8g+/
mMHr+MVsfo9fzGs9+MUi1o9fLG272LuT8oD9qsqJX/SxfvxiY/XCL7ZV3z+J
aa6BHyw+h/ld+sWJ4gl/zhHnfJ8N/n/WdtgFv3hQe2GTF/yLX7wj1k+rM3bH
L3ifCR92Ud98n7aMOOAXq0T5rSXkgm/4xTZRznDYLcoZWQdE2StxhPYAO3wL
no+w7dfjt34BTh211c/ii7yMNVeOcmZ8lyhnkBKv4An5Nz70mLZE7rvVHT9I
vmO7K2w/+1D6APoY/Ixcdu0oMW2dKGNMci1yDfoCfIgYDJ830obktOSkxBr6
wJzXwhe29h58hD5kqPYh1wLPe8UROfPd7xPaY7By3mybxET4R59zubYfrm78
H5+j7+NdzDlihO+RoyzgNWIB/UvuyzpcG+N/9Kena5eDvQ5eA7z/A/GiHnKH
lawfOy1pu+TeSyhP5lTXWF5J+XMtKHrlPOrtUXIqcMh5KvAZH2X/32PK85DP
7CzOuSbo8Si/P/VP7XaQ9tqjsmP/KO9uVhc77M5aMGI9fGCMS24KT8hpbhEP
cpJx0fDqd9YH33g/94Y4dYgmPsFPOPNVlBwBH/tauX8Wv5/FiL83+P/pouzd
/H2Uc8/nivL7LgtaD7Isbv3osZztglcX5blbXbOuvlH2DW1qGb5tp75niik4
4BdDxAe/ONq6wPNk8YSj54nzYLE7UlvcFuX30R7WXtzzsnbEL95TXu4jloxX
Tube4AN+sZA4Iif50yTbhkM/WW+uk0G+nSyPUQ7uwS+OEHf8oi0aPm8Y5VzB
w5UDe0/tF+ShHbUBsbRTlNi1sradLO70WeTMXaOcZ9ndesnpVtFe2JK+5J+2
T641Tr3JteiTmCcl12LcQJ9IrkX+Tw7fJ8p5GX3Fh/6M2I3vvlLpB64ba0fw
30x9kYP3CuSb+AX9FrEJfpMX0CfhR3/0/09EOVf5EW17qnIT++HbPWJNzCLe
wM/ro8xLYaNrlTHfF4zRtmPVb0Gv3a5M9Hv4/zD1xC5Hex3/29378b8NrAe/
6Wb92GY528VmyyhPrq1Czl+0JfJnTjUuSk71SJSc6gnb3Ux86G+3Ebd7xO15
dRkozsTQQeJ/nJhjF8Ywh2ivbcUt3/lk/9lJnLE7OQrxCj7wvg1fhifTqdOv
a6qjGdN84PWnlJ04QmyDh8Tm5Cft4mPwFt5/J2bIPSVKrAIj+H+J/8cvpvf7
2b3Oc/P6PdgtbD34xZLWn+P2bdWvm/JwXy/lxC/WU37s0F+98IsdKn33tG2w
Pkh88ItjxA2+jBLPs8SOdvGLq8Ufe9zuX/zi0Si/K/wv695NPNO+xJKnlYXY
CR/2Vt+PtRfjXTDAL+AQvMK++FyuKd1NrFZXfrDEL44VyxWVHz7jF+SdH6nD
+bY7tV+wP6ej9c4oDtQ7TZSzUvh/lyhnKRGv4MIk7fFfZWdsSD7HuxViF7yD
K/jHq9oV2ck/P9F+zyrjmtqR9zTErofFkFwLfsKDdbUpvCV25Xwa40T8FW5v
ok0nRMMD9Kb/Y5w4Osq8K9i+pn2I/fgUue12yjfIup7TtuRq5P34X+51oI6b
lW+E+jM+om840u9yXRbzHMQX+ktyHsb99Jf0g+TocOsK22vzOj58gPfDky2s
B6x7Wn/OP9IuPMk9IXn+HnJmTvVklJyKdiaK6UsVduDwoni9pmzbixv97S7i
STzdQ5wZd+xjHcT2IdoFbA+r7HWMdoQzI7XvElHODOAdGTkjfJhRPD5pVPnV
fz+1TN6X+5DgDHzDl8aLzRrKgq65xhLejvV7+PyW16YRq59t81T/j1/kuqK5
vc49C3qNZxezHvxiGev/Qntkuz2UB7/orZw8s4Hy5zuw32mHHaPkjYOUCR0P
Eh/8Ypi4Yed853OX2KVfXG1d+MXt2gW/eFR7nS9eF1nHh9oXThNLsPu+YgAf
8ItFo/CEeAB/8Iu+Yo1f0LfBt5vFAGXAINdsHWn7uS7rrCjvjIhl8HzZKPPj
U/vFnFHiFbJ1sgxOnaPkXfTPU/yua5Tf/AIr5s6wHz6RY8ncR48tV9V29HH4
2UfKuLp25Bl8CI5/qV3f0vbE7lwPso725T0OfvYvy9j+BdtjnPiU5U21D7jC
iXuVA4xvEhvsMDYa3yF/hZf4F30D84r4Jn3LANvGF/tEOWOIeumL6L8usT36
DHIb5nCYS4J35Dm5x+jmKOckM5cwLpqYe712GeZ12h7qvci9vfWA6VrWjw26
2C626aQ8cKOrcv4Q5b34l8r/ptj3U98c931gXQPEB+7vbPvot7t4niwun4nT
4CjrGg/WLhspP+XcI48dGaeeaJm55TOUC/8lvsMH4ggx+9d9e9HMGcOf6bUj
GOC/5IPwbYVK103VD2zwv4/E5jbbhM/EvknaYIof+H+k2OEXmUvN53XKC0fJ
y5awHv5Pjkv9+EVn28UvVo3y/n6tKOdHbxjlTMct1As5d1RfbDMoypmyB4sP
fjFM3PCL0eKJbc8XZ/ziavHHL27XLk9p20P9+y/tmHNz2Pcm8cTuY6MZv3Iv
fkG8Gu/zXZQRnqwbhVcDopxjuG+U3w0/SvyWV+af1C/f9aDTFVHOn/ybNpza
L7BJrplgni/j1axR9j6TG+c7gOmVlfawe1ftRZ/QPcpcZQ/1/kkMv/O+1aK8
j8n18LmmI9/L9FJ/ysSuzy2vra0pE7s+tQxeH0dZy/ZhlN8FfddycjixJI+A
W8SuZ5WP2PWgMu2s7WgPHz3PNrABOVZyiDZe1x5wk76I3IFxN/0i/ceS1ovd
GA+QQ9OXEa+JP/STcOG2KON0ric3uZ8YMdB6wHU96/85yjg83yO/Lu6rKid4
r6X8Oafzmff1V1/q2lYc4PdO4jM2mtgNbvBsL/Ekb9rPcq67Bv+N/S739Bxl
mfyjLcq6+5MtzyJGlOHTRVHezV1VXb/J+pnvvifKeeHMmcG3XmLwO+V9w+dy
7g/ePqHeM6kn8W9W8UHXOXyWuueNsvZxwShjm0Wj7EVZynoyr30iSv9xVhS+
J35rKyf266f8PL+lelHvjlF+73aQOOAXB3kdvzhG3PCLUeKZe0PAGb+4Uvyx
+a3aBb94OMq5Ly9pq1wzi31fUaax0fgFWMEHeEes/qe2IB68bpvkV/AKvxig
HvjFflF4mOtqcj7qZ/G9JMpZIuSC8By/oI+G/1P7BXNeHa2XdYedovQvGa9m
Uz7Kubf2F3XpapmY2E054EZ366TcQ9tNp+xZzr3SlHtqU+4ndk22nPblHQ2x
K/vEtaP8xmWuO6X/WVf7UsYv6U9/L34Tvb6pduTZzbUV9W8d5TxWcpl/qRt9
0xPKxJzCLbbBOPEMbYXd8fEJUX4zmr6IuWTmDch5LlKfh6P8Jgux6j7bvEe7
7OR1cu/h3o8ee1sPOPWz/pwfmKCsqyoPsq4ZZa55vSh7STaN8nsv29j2nbYL
DuTiA8Unz4CZOUqMAk/GhUO8DraHWCZ2DvVZ+v1h2gufGmH9vxeX6W37z5b5
x1wdfCCvuNoyvnmzZdZr3Ov98O1xy4xfX7At5jleV4brtG/Ox+PLuccNbOA5
eQ28mifK71AsIJZweGGv57rbzNeWiTL/tZL1Z16b6yV6Kg/39VHOn6wP+fGF
LdSL8o5R9krvGeWMxgPFZ7L1gRt+cZK6If85UX7P4nLxxy9u0i7oOy7KPq7n
tSN+Adew78feNzrKuerwAd4Qt+EJfoG+8Ae/WNvv8Qv4BN/wi32VF14Ni7Kn
/kwxO9y60Q+/uFG58Iuc457aL8CpY5Qz3TpF6V8ydhH7OkfJwTLXYi4wY9ds
UWLXrNoiy921F+UeVTn5wLOrKcPs2vhH21pDfeaKErvgVW/tMq94faP864g3
fCN2fS7OjOs/tdwvyvsdxuIfeH/ul6LO3J8In4ldzOXMon3zvTp+/LDy4btX
iXeuA8XejNUZK9BvwacTtTO+yHiAXPtu8XpIu6zpdWQZ4f30t4Oj/CbCJtaP
3de03Sk+izz0w32j/E7hJsr/rLqhF5zYUX3hzW7iQO6zl/jQp+1neUf1pEz8
O8T7sfsR4t8hyvnS2KnNvzOpO3bEB0/Xvvyjj8bucJP3qbMpK3PB8ASO3moZ
7t1nGZ0et4w/v+CzF2iv2SscaQs/+iTK2apfKCd5xrfKT/2TbXMLMV3M6+C+
pPf/EmWdZK4J+izKmu9HLa+uPJT7RNlvsGGU9UlbWD/lHWybtvYQB2Q4QHzw
iyPFDTuPjHIe7VlRcsdLxB2/+Lt2wS/u1V74xVPaEb8g1mJf/IL4hd0/UNaT
xRC+whP8gjmmu8QZH4VX+AU+BN/wC/wDHuIXR6sHfpHxCvyuUNcTtfHXtnO/
bU7tF4tEOa8vY1f2Lxm7GCvnXNa8Fbbg0cXy3FOVu1blblHyt+7KTT09oowP
VlGG7NfAn/enPcUfOVdXn8XE6AvtsKYYk58z7oGT9H19xBxc+orj8troLcsb
aivK5MyvagvGSeOtE/9+2XaJXS8pM3k1fRQ+Qu7zRGOGX219s7IyZ5Pr4PIM
pPHanevEIvqsNp9/Tjs8rewjvB8MhkQ5G21T608/oN1v1RN53lef+ZVxa+XP
PAq94PRu6jtCHcCB+LS/ZfKBAy33VmbwzD1uyyjvUdY5uzpjF/rmkbaLfKO1
I//O0r7cw5zVAj57uTIzDiBHgSfwhzHJPOrN2gp4NVjM5ra+Fy2zJuM1758Q
Ze8lXMMf4TNc+9x27xBL9CAWT1LOQ7yvg9d5fmnv/0lM8n1Upyhnv3e3Xcqr
RzkfPvcMUN5Q+aln8yh7hbZT3x+1zaK2sX+Uc2MPFzc4cbx4Iv9p4oxfXBTl
HPS/+uwnyo6ecOsx7Qg+cCznjfkOu79he7nWnDwu+YhN4c/LyguvXlKv56Pk
bu9rI2SGn/gFc2G/RDmXnPLiUeZmuZ7v29qivV9wX0flytiV/UvaAF51tryg
5dxPu3JV7hLld167Vs92jRIPu1n/Itp1krr3UJ8OUXJX+E/sglf4FDkY83HM
FRDHPlTe1cWc9snB3rTu3uJMfWuJLc/1ibL2hxzsacv5Pp8yOfOjtr+Z+CE7
cw3Ee/Lh7Swz70rs4l0cfrGP5em0023qQ65FjjwhGj7le276FebLnolyTnWu
Cxnp/ZnDUQ+k3Mz6wXJ922WcsrHyZIyijVwrhfzMye6hXkdH+Z2IgdH04ZTh
3EFRzkE+VNywzRHiSVw4Spzp048Vf2Qfrl34d5L2Ijc4XTvC+3O1L7HpL9bN
d/TF8IE+iDk+eMKcCH05/GHMwlzKQtbBO88F/R5/gYfEqtctw6l3orzX+8hn
4c5E6zxDXGlrBp/BL3hflevbZhDrZbz/R/XKudKc950iJtQxRdxes7yWcv6o
zc7VrpupF/Vvq774xS7i8K12WlY5DxE3/GJYlN9qGCXOH1p3qPvl2gW/uEl7
4RcZ9/GLZ7XB896H3Z+2DfiQffHAaPyCvgD+4Bf0X/DqIeW93fLW1ocvEG8/
sjxSXfGLXD+yr8+BB5wlb4b/U/sF3IZb30WJXWC1aJSzlheJsi6Lcuf/obxy
lHNfu1jP4lHOkqMtYhd9wVJi8qV2oAx/4AD2/kRZco0EdeAnb3sNfyP+wwfi
2Hix6ilGfaLxt8x5e4kz2BPHxkXj2/R5jI/p3xhL3mGZOMZ4fJMoe4Iyh7/e
e7eIcoY9HPu7WO/qs/gCfRPjPWIL/n2Tep5pG6/6HM+8oF3yPTfji5O8/2fr
oh765M2s/9Mo5wHmnr7cK/xH5aTfHRSlT99fPejXD/R58DzEMv55uPjA5yOj
/Z4R8Mx9JeDMv+FRfnvjRGXAj07VXvjX2doRn78wSl9DrrmCePJeaFl1BcOl
rA8bwR/iGZyHV/gKPgXfJkTjd/AQLr9hGV94xzJjo48sPySui4rTF9aJT35t
W09E+X0Xrk9SNu7Pd5LUk/tCvrHczXYpr2Yd3N9bOSeJIfJ/qy2ust2t1Be/
2EkckHNP8cEvsl95X9uAJ34xQpzR/Qzxxy8u1i65bo5/+MUd2vFRP9h3nDjO
Lebk80t6b77vxC7Eb/gDz4j/8CrX859meT2/h9872z7PMr8Jb/EL5kjgM35B
7gXP143ybmRqv0CWjmJdx64Otp1x7P+rzD2dLS9h+RvrzLWmS1uGD8ThXLex
ouVPrI849oF/u4o/2HQTf8Z/3cWfGNMjyvp3yvSz5ETENHjQ3+eZ6yDOwxty
VnKifP+wk/WCbfZlrHUaGOXcPcr4/CXeQ3vYn7E+82DMTcAx4gd9DDnaNl5f
xWd4dhFt+ndtAreuVE+4ypzuy1H2Kb0tlqdFmdPdzXrAcgvrf0adNrC+gcpD
bNxHOYmlQ5SFOHJIlN9pPtwy8ztDvWd6n/91fXI076O2a0T4dS4UPJnXHi7O
zAedKP7kRqeIU+5PWtv/ny/eO0bZn0LuxlxeV3Ulp4IP+BU+Ak+ICeQC8Idc
hb4XXn0uXvANzjO3Bw8ZX/wnypwN8yDwltj2kWXk/dR7lhNTnp3fZ5b07ze2
lfPYy3r/d8p2ouVcR/KdunwVZS3PL9aT68Dxi3WUn3b7qVeuu0HfT8QcHPCL
XcXnbe0Kbm9oS/DEL4aJM35xsvjjF+doF/xirPbCL/6uHfGLu7UvfkGs4N/1
YgofronyOzb4xSSxw47EOXiFX8Bj+Ab/8Xt4hN3p7670Ou/bn/HZ4WKALPAA
IbjvMTGb2i/oOzqKYR27pi53mqqcOVLu3yHGd44Slyh/pk0p/9d70eFDbUoZ
Lq1imTx0dXH+t/JRflE78EzuxaH8qPYlpjE/t22UsTS+lvtO8HNiGvMhe9se
MYL4AZ+Y42XOiZjGOO0w5WAemPEPPkZugD/TRzKuwc/xQ3yTsVPuEaR++AeP
4Rtx5QTvpY8ZrQ1oj7zic9sh/3lFuxzrdcYQp3k/fdsO1vO4+m2nLINsl2sH
KA+6H6qcC6oH8sPPI6Ocr32U+sLVY6Ks8zlWfJjrabN+bDxCPFdXR9rsH+Uc
Mubpz9Qu1HGu9kLHMdqRmHSF9n0kyu/uklPeqn3w13u0M7kJvpbvf+DAipYZ
A8K3b3weGYlVzMPAT8YY5CFLqt/HlrH7RMu/E2/KzE/mXAT9xNfWub1tLOf9
lFewHsodrT/3rt1uPT2U5wt15N7P1B35/ysmlPGLTdQXv9hKHPCLHdUPv9hD
3F7URjeIyRHi/Kg2A//7o/wuxZ1+j73wi8uVKc9vQM+rxB274xesX4AP+MV4
dT9PTPl3tjrBK7hI/r+gbRLjc88r+T9cgTPrR/l9P7gLn0dHOb/1MuWeEr/1
i6Wj7HeuY1ddru9ZJsq5Wdiuk7ZewfKnfk8Zbqxs+X1lp0wfQWwgjjHPsKZl
8FjHMvJtGOU8+c2sK8fFlO/WjpThOX4EV4jV+1jOvcvwfqw2pUxfQH/UTfxH
WAb/UcoKfsSLHuLPXOmq4n+2Opzg9dWt4zT1afP/69gOMQS/PUpdiDXsW6W/
YoxFX/Oqenfz+k8+x/23qx//Jwc6wPqxU+63yliEPGG9yEn/d4zX/6A8p9nO
ceq7aSUzsWm49/F3pLiNUXfwvE1MwJk+/Mwo657O1S7/FWfsRX98uboQB/6m
fXGAm8UJff8RZc7mQTGmX/+nNsm53LQ5XO5o+8QFeIhvMgZcTrsQB+AteQX+
v7T45DuazP8pE/++9B54lJznOr6wbJSxwwrWwz0rWT/Pdrbdz8UYefCLVZUT
v+il/O+rOzq9LSbo+7pY5bk424oP/PijuOEXg8QTvxgizvjFUPGnjeO1C/qc
qr3wi/O141htM0Z7cR92P097wAd4RByEJ/jFC+p2irjDq5PFF77hF5PFB04R
8+BnWzRjBLCGX9gXPuMX5JDwHL+4TH2n9os6Fv1PcWlZy9h0+Sjvble0nDGq
o7boEuWMmu6WJ0STM3bUFr0ts5ajr2Xm/MhNiGnE3P5RzuPa1vI/tBdleL6n
ZdYzDbZMPnm45b+IC+ULxJHyn8Udbp2hbTqL/18sg/9VlukX6IfgKv6ba+aw
xW1RzmJlDNNVu9whxm3Kmmu84QY52zHKmeuX0i5tUcZpQ7yff0dbz3LWQ/1w
6TjLl1lGnmesBzk/sYz8P0Q5Kw/OD1ffn9ULHL5VX/DB10eJG7Y6TTyJJWcp
57U+B/7weox2OUrZsRdjgL9px03Uo7+YMAeN3RmPjYuGD/g+HIAnzJXAjZ7i
gBzwijmZ18We9RkTouERvHzfv8RKOLm8ssNV+Jz7IZYRh1x3T06R8YpcIX3h
UsvLen/219TzhW2dZp0dbZe2Vva7j5V5TmXrqfzIvKZ65X7ZGdSxnzig++bi
AyYDxG2c2IJn7tffXWyHiD+2Gapd8IvjvYbNR2tH/OIc28QviBHYHb9gPAof
Tokyv4Rf3CMO8IQ4lmsrkBW+4Rf0Jc/YJrEYfrZFE0f3tvx9FD7PKg74BXYf
GL/1i/8/MWq5+N9j1Mrxv8eoNeN/jlEdtQV2yRg1wPJd0T5G7WX5b9qF8uXa
hfIYMaJ8XpSzWc/SLjkniF2IRaO0S8almy2P1C6Uh2uXla37Wctt2qVLlN9V
IV4dp10yh8k9SmkX4tXR2mUj9ajtUturtmNt39ruNR9qntT8qXlV863mYc3P
mrc1n2ue1/yv/aL2l9qPav+q/a72x9pPa/+t/br29zoO1PGhjht1PKnjTB1/
6rhUx6s6jtXxrY57dTys42QdP+u4WsfbOg7X8bmO23U8r+N8Hf/rfqHuL+p+
pO5f6n6n7o/qfqruv+p+re7v6n6w7h/rfrPuT+t+tu5/63657q//T+PVCpbJ
n1eyjEz4cSd162aZPHxVy+SJvSzjQ+tYJvfNsR55/maWHxR/yplHdVYn8t6V
o8SolaPEKMoXixecYDxyarQ/j4oYckqU3y/AN/8eZe/tPyxnXOouPvCxh9j+
J8reAvQnlyWGkPeuLs7kvfgV47JF1Plw7yUuMY7LfdW7aUOeOyja5711Plzn
yXX+XOfVdb5d5+F1fl7n7XU+X+f5df5fjwvq8UI9jqjHF/W4ox6P1OOUevxS
j2vq8U49DqrHR/W4qR5P1eOsevxVj8vq8Vo9jqvHd/W4rx4P1uPEevxYjyvr
8WY9Dq3Hp/W4tR7P1uPcevxbj4vr8XI9jq7H1/W4ux6P1+P0evxej+vr8X49
D1DPD9TzBvV8Qj3PMPX8A/+OjvbzFfU8Rj2/Uc971PMhtV/U81QZr76KklPV
c1MTo/3cVJcoc1P4NPx4S1t1tp21LGPv9bQTc4ObWGYedaso553saL3Yco8o
5+LSRxBnrogyBzVG7Lppi1MsYwt8rHuUGIV89BHMK64iZrS3qnXAD7hCnB+v
DseoG7kQ8YF5xd5iTt+7dpR13cy3M7+NX/QTf+YVmW8f5If5duYciQvMazN/
fr+yYs96XrGeb6znIev5yXresp7PrOc56/nPel60ni+t51Hr+dV63rWej63n
aev523pet63So54HrueH63njej65nmeu55/reel6vrqex67nt+t573o+vJ4n
r+fP63n1er69noefen6+nrfP+fx6nr+e/6/fC9TvC+r3CPX7hfq9Q/0+on5P
Ub+/qN9r1O876vcg9fuR+r1J/T6lfs9Sv3+p38vU72vq9zj1+536vU/9Pqh+
T1S/P6rfK9Xvm2q/qN8D1rnWsuKYY/OcS+/k5yOxJRa9q906R8mpVo6SU61s
e/hxlyjjPsoPKD91kSfuahkb0BcQf+i7D1F38g3iOLyknz/ZMrY4Rxtji7FR
zmemnyQW0UfcrY3oF+jXcx4JWdcUf/q3tbTDpCg5Ev1G7kPJPWWZ622mvPR9
rGcYGE1uvov60TY82EJ98j0b722v0i71e9v6fW79nrd+/1u/F67fF9fvkev3
y/V75/p9dP2eun5/Xb/Xrt931+/BwTnfj9fvzev36fV79vr9e/1evn5fX7/H
r9/v1+/96/UA9TqBev1Ava6gXm9Qr0Oo1yfU6xbq9Qz1OocOUdY/1Osi6vUS
HaKso6jXV9TrLur1GPU6jXr9Rr2uo17vUa8DqdeH1OtG6vUk9TqTev1JvS6l
Xq9Sr2Op17cMjLLuZWCU9TD1Opl6/Uy9rqZeb1Ovw6nX59Trdur1PLVf1Guu
6lyrjld1fsXfzlHORaZMrks8wE/gXm+xernCDVtuIu6PqAvP3yM+lFlLxrib
uER+SywmFpFXHKmerDMcoc3IDc6wbcYlF4sbtsDOq2sL7A2nic+PRjlD/mVt
TN2si8u4RIxfV/yZSyAWkDsvpA77i0PmTutpI8Y35FD43jbalfV+5NDnRHk3
iM+f6XfEBPyfdaH1urh6vVy9jq5eX1evu6vX49Xr9HKu7vpov66vXu9XrwOs
1wfW6wbr9YT1OsN6/WG9LrFer1ivY6zXNx4QZd1jvR6yXidZr5+s11XW6y3r
dZj1+sx63Waux6Fcr/Os13/W60Lr9aL1OtJ6fWm97nTq9aj1OtVcv1qva63X
uy4RZR1svT62Xjdbr6et19nW62/rdbn1et16HW8f7TU82q/7rdcD1+uE6/XD
9brier1xvQ4ZH4Jvg6L9uuV6PXO9zrle/wyOuS66Xi9d+0W9HydzrW/FINdW
gVXn+G28Iq4Qo95RNrBjnoE4SFwi110/ytnvyASHcp6K55kz3EWsM6fK9VGH
iE++44OL5LejbO90/7+6epEf9NIW+HNvbfGAMhEvntVm5KuMN3IcR19HH0AO
CzdzrmmeKLkT/SD7cRjHwAH6B+IE/dSu/t3fe7H5SO0MBvR/J0QZozPeog+i
T73Ve5+Jsu+g3o9Q71Oo9y/U+xrq/Q71Poh6f0S9b6LeT4G+A6OJP/X+i3pf
Rr1fo97HUe/vqPd91PtB6n0i9f6Rel9Jvd+k3odS708Bu9y3Uu9nqfe51Ptf
6n0xncSNcr2Ppt5fU++7qffj1Pt06v079b6eer9PvQ+o3h9U7xuq9xPV+4zq
/Uf1vqQOUfYr1fuY6v1N9b6njuKPX9T7pOr9U/W+qnq/Vb0Pa+r9WQOj8Qv8
MvdzcW/u86r3fxGH4Qt8rPeL1fvIvhI3bDshyr4z5ghyP1rtF/V+5w5R9uOQ
r2Jj+hTefRCjPo+y5jPfz3bRLrnOE9uQs3S1zQ3VB/5v7jM5BuwRZX4N7BiP
7y8WcP9wcaTvON42mMMjfpMHM4dzkTZFL/r6taK86++jLRhb9dUW9Pc5pvsg
Su70vVjSPvYnb2bMvZh476YezGPs5HMZo3bx3o2V+WDbOzXKnpXLlXvTKO9K
sAvxgnHViGi/r7Pe78n938Vv94eeGmXfaOaFN0f7fabImXO5uZb2pWi/X7Xe
x1rvb633vfJd7oet98neEWX/bL2vtt5vi97ExC+i/f7cw6Ps263384J17vOt
9/9uHmVfcPbp9M31PuJ6f3G977hTlP3I9T7lpaPsX+4QZV9zvd+53gdd74+e
L8q+6Xo/db3Peu4o+6/njvb7sgdro3ofd72/e/4o+77r/eD1PvF6//hiUfaV
d4iy37zeh75MlP3py0XZt17vZ596n3vuf6/3xdf75Wk312gjc8ao2aLsu0cu
+Nk32u/Tfy/K/n04BhfwC/rI3O9f+0WeyZDnZmBXYl+uY/82ynr1L7yXePWJ
uBKjGKfDk67apY/fkcfBK3hDvkA+S4xiroM5NeJSzquTO8F3Yjcxijle8lHi
0nniTu50urYB2xOjjNvbohn/kI8S9zk3Y11twVhlfW2B7+W8E760ifYAM/yB
ecm5o8QobESM2lFZdvE77h3k84Osr69tIzcxlDHfCZaZDz3H+9GZuQf4ynz0
3drloSjnZtwf5TyNW6Ocs0E9ef4G9ee5HPV5HciT53ggJ37zcLQ/9wO98jwQ
6iV+vhXtzw8BnzxX5I0o543U55DcG+V8EvDvEE2fiV3ww2+i/Tkn2JF+8vto
fy4Kdic3wOfhQ56jAk/+oM7wB+4RZzaMch5LnyjntMDDPL8F7B6N3573spy4
UYbnL0X5DeyVbesPUc6TmT/KOTP1+TNzWQ9+MUeU82pmj3KOzWxRzreZNcq5
N7NG+/NwqD/PyaHtW6P9uTqzRzlvZw5xOy/an88zT5Rze+rzfOaPcs5Pff5P
fS7Q1OcFYff9ov35QvW5Q6zLyPOI6nOKyJ3eUTfsCg/xiy+inHf0nrhRfiXK
+UjEKHg+ONqfp1T7xQJRzutjbjb7IfQjRtGnLCOeuY6XGPVplL1+tE+MgR/k
vX2iiVHYhdjQI0q8Ikbdq57EqFzzyfO5Fh2/GiO+xCjW/JDXEqOYM8Tv11YX
+htiFOMMxibEKOZ1xkX5PR3ifa5xYt5kU22BPhl3phfvgdor8yj0Zfy9jW3u
6fP8f3/tzdjsMPFEjuOivGNH3pxruEg8GNOzBoZ+CpvDU3L1e6KcS3ZXlPPK
boxyjhn1/BDlN2jz3DPa/UX71eekISd+dEa0P1etPm8Nfekf/xXtz2cDnzy3
7e0o57mBJ3F7YrQ//w38V/E57ELu8k20P0cOO+b5ckdHOXcOu0eUMxGnUc/0
BzgJf/DlKdH+vLs+Uc7B6xnlfDy4eZ24oO8blpeJMrfbIcr5e/kOhXsWiHI+
2zxRzvGbM8r5frNFOfevPg9wpijnBM4Y5fzA6aOcKzh9lPMGp4tyDuF00f58
QnC4OtqfZzh9lHMOZ4hy/uGMUc5FnCnKeYn1OYr1+YpTn7uIffeL9uc0Uj98
GBjtz3UEI/izTbQ/BxIs4Bt+8UmUcyMnRDlPcrx4rx7tz5+kT4VH+AX9NZyb
2i+ImR2jnPvaSRuRh3a2vVxzknumuigTf3OfMrGnuzbL/cgvKscq2mVL78t1
C3DrJnEhRpEPME7GF/HtXOdK/zzKeon19CvEqLZo3jkQo8hv6ZM2EPeHtQt+
8FKUGAWHMo/CfzKPwkbbRXmvyvh7W/XmPRzjQ/qJfW2DOQb6oLWs96goa7PI
IeBi5oLof4kf7EisYk5iYJTfq+C526Kc+8r1t7QD9xMbVo/258dSP/6yZLQ/
b3a/KOfQIue3YoH8cOiQaH+eLfrmObfgwFjkwWh/Li64weFXov05uuDMuPjd
aH/u7pVRzuOlDWLEp9H+/F7si2+Sm2F3/IH+GD7wD+7vGOV8YPgzm/rAq8WV
EV70EpfVopw/3CXKucT1ecVLRTnHeNEo5xsvGOXc43mjnIc8R5RzkmeNcn5y
5oz4xfRR3pFMG+Uc5t9FWf/0S5Rzm3+Kcp7zlCjnPP8Y5fznH8Xhqur6RfHb
c6TB8+T47bnT4H9U/Pacaux1YLQ/1/qHKOddo9tY5UY/+LBTtD83Gz3hD36B
jeEVfvFOlPO3X4/CwxfFtVu0P8f7wSjnezPOgOcDo/154LVf0F90tC7G47kW
BA53VlfmzeA3fQq5NDHqC7+n/Y+UiXj1prIiC7ze0O+wJf0jMSrzK/yP3GGQ
drxSmYhRFyorXDxDLIlR5LrM7RGjGI9fLU5DxXUj68g1kOSW5A3EKOI2HCJG
7S5OOWf+e22Cfy+qrZC3S5Q9zLRJX9TH+45QH2LYceICP0aJIzKfax2MCS4X
G8Z0xCNyON73jFP+66Ocq/+3KOftcz/z0zPG/9PZm4B9PWbx/2eYyWQd22As
kxlLKLsGUySyRpYRWZtBGksxkiVLdgmFkXX0jF1MBsPYSnbZDSORJCJGoYam
lH7Xy/d1/ud56rr+1+/6ua48n+f7fL73fc77/T7nPvf9uT+fT8vn89P+fPE7
XQ7bRsvn/GPnXP3H/q+1H5umNvOXMe+kaPl+AfChFmuKlu8jAM+L5fHZqPcX
gD+16EvR8n0H8LWV38Nmxv0J0fK9CbnP54No+Z4FdBLaCh/kBfSGrlYQFzAl
HxFDaC3H+Y2j3vuAbtE9sYieyZ1oe/Wo90dgy6niSFys5PFSUe+hWCLq/RSL
R723ovn7LOZHvedibtS7A/IZNG9Gy/dlZL4Dz1n6e2+0fO/G11Hv4/gq6j0d
uW+VuPgy6h6KL6Pe9zEj6j0gM6LeDzI96h79/0S9T4T8A+/ExSdR7x9hTHpb
HiZFva9kQpSu3ox6v8nLUc+HflaMiIvRUc+5z7VZOLwv6r0qf4t630rzuGC8
yHwFfnkdZdWoZ4quZdvYsW7UM6z43uZR7y3iHzVcJ3HDN3IJYx45lZjZJipf
kaOyvso1nhO0/Vrty+uj6Jx4Oz/qvUWMz7eLGeME8cI8uK/4kPv7aAd1OOsi
n4o9dfAc7QAfNHm4560ZtTaF38faN/XJn7SbOhhNkqMZk84TjwHaiy5Zu8rn
PDEW3ha1/+kBeafWZow5Plq+t4jP2bNzTLR8z9EVUe8/on3GyD9Hy/cl9ZbD
LaPl+5Wwn9g6N1q+j2nNqPc0gQNj4QHR8r1O4DZKn8Hzejl5Nur9UODfT9tv
j3qfVPJ1f7R8/xT8kjvGRMv3VYE9eWFctHy/Vfeo916hq3xOGHoj75Dr8Jla
fWq0fK8Wum2KRnyvFfUerryuNzdavrdr6aj3eS0R9Z6vxfz5fbR8L9jcqPeF
zW7W76yo94t9HfXesRlR7yP7Iuo9ZZ9Hvb9sWtR7zT6Net/ZJ1HvQfs46h6N
j6LiInN9/2j5nrXJUe9fmxT1XraJUe9rY60N3okL6mn0sG+0fO8bOkU/xMWL
Ue+J45o0etsiWr5Xjnkk+iQu0OJ38nN31Pvp2POHzokLcjT6Xzgufhr1vlTW
9DJfMbZsElVroW/GFGrkzaL23W0R9c7nrcQPuxnn2Ku8k7Yw5u4ld/jBOmrH
qPdCZt2BbjtHQ//gntfZBsvFuVHP0yF2iAtqXfII8cJ1UfILccT8Ae0zrlPD
9pLrnlHvhcy1qdbytqf+5h63jeV6+6j97L+x7bPEiFqMdZm22k9OWde/s96w
ftQ9iuB0pbbuHLW/gHHtuqj3QvI5Gsp7HKbpA+0w7q4aLd87eULU+yhzTrNN
tHx/JfbPFTf8QiePRsv3YLaOej8m+LwsXuCW8/h/R71nE5z/LOb4RJ15crR8
X+c1Ue/xHBz1fk/4ba/f2M+1LnIdeljWPuibdRx0gn74j1y3U9T7RtEb8+Nx
0fL9pBtGvbcU3TJ//jhavuc0a6hZ0fK9qMTFT6LeZ5f7SHLda0G0fO/qf6Pe
x/pV1Jwp72H7OFq+1/WTqPe9fqRfuWaT/n4Q9d7Y96PeJ/te1HtmJ0S9f3Z8
1Htp3456X+1bUe+x/VfU+21fj3rvLeNB8otNuTZF/YceiItnot6rC+5gtn20
fA8v9RLYbBIt39s7Mup9vrdFvee3Ker9v8TKNNu/Pup9wc3jAmzaRb13u73t
Lh/1ThxiY1M//6Vtz7W/LeyPv29lf9toO/118XfwoC7+rb4yfjMu3q9N+J33
ye4gL9iHLllLvFg/WJ9hPYRa91S/Q7vonXqc3Ew9wDid8763ouZ9H/uTGgtt
5byPcYwxkdyX+yqxHc1nTcW/rKnoE43kfTDYuI59owfGP3Ib13OIlUFytpG+
PKT/l2kr9lweNU/n8/f0l/PR98nR8v3dtE9OWSVavu/7YHm7MVq+Hzz3gd8c
Ld8njr/E1nXR8v3j4IPuR0XL95WDJ7nplGj5fvN/RL33/Nao96HD1xGec3HU
+9PhdxvPg3fqx7Oj5XvY4Y0cOixavre9i8e3RMv3vKPDNtHIw+hzZ+1Gt+R3
4hU9k2epL9A5uv4sWr53/se2Pydavqf+f1E5a1bUe+1nRL3vPtdyZsob9hAX
U6LW5SdF1Y3veTw2GrkMf4mLf4vDrVHXdImL1/V5iN8HT3SS9ziB5zh/EhfP
y8sftfdmMX4qaq/UE+KEDqnzct7HXgz0sLvnzhB7aqSsqRgLs6a6Q2w2sh+O
iYu/eA5xcW3Ufk9iBT1zfq75ERdDxWXhuMhrvWDLtYD2Ys76X147ZgzazHPJ
8XldBjuov7/1e9RUX/iTmmpyNHL/tuKNz9RUaJxxgJoKjffyvLzGv6M+cX9v
V3lhfs68b1DUvoABYgJffcWQupW886S+0jZrKIwbjNVoJvckzIyqqRaPWkvH
36yp1o2qqcjHZ4oLdlFXExO/E1O4IP+hKeZY1LLUC2vZxj22RxsPyy85j3GL
/HuevBwZ9b72gZ4/1b/TzgztO17sd7TfXI/5nVxtqp1o5jLth8+h2gHPh+sv
sfegOHwmRuAzSVumaNvL4om+3xTnv/k7+DMu3iEv4HOVfBE358vjQfqV/A6I
uj+0v5yBGfkZnSznubnXkJzWORr/kdPQG7UQdSg6JH9TD6HPbcR9DTF7TH/B
n3nMMuJAnYL+idVPxC/EYI6fzxWn3Ls1w3Y4d1rUOxhzfw/cfKA9xMW72jk5
vEYWjbj4l349Lcb4S1y8JA7ExQviQ1w8GxUXT4nnIDk4QTxHiz9x8ai85L2d
fxPjf/idnnJH38QFuRneiQvG2Kypbo+qqW6OqqluEg/igrEk99hcLTbExTCx
IS4uFRt8uzjquT5oA50TF2gF/S8cFzmP5Dzm5rmnvbX98/flo/LValHv58KO
LaPmJOjmK/9OvmJMJlduG3WvPb+T7/fVb/AjNqipqE2pYXfS7wF+51Lt311e
rhBL9NwUjXEAvbNG+Tv9orZm/s18jzEoayp4yJpqhhzupm9ZU6GXrKkYl0/V
z03sH7y2E2tiYg/5WMs+0QbXp8mFt3pMLT7K9mj7kai9es+Iyynykmv56Ltf
1Pt1zrWd6eJB+994Dv3OjZb7ktbRTvSzYdT8/njtQD8P6C/6Ocm/Z60ww35P
FzfmG0+IJ/FzgDjnfYTgf5tcN8nTR1Fzv3e0u7c+D5AHvneMeFMTogf0Du/o
hLXlc6LlPSOdGm79f/Vu3leCn6yVo330ybyXPIZuqSkYP9Az6ybUMK3F8Jmo
+wqwP6/NERf/9fO8Vv+Nf58WtV9iiu2D+/tR19bf0R7i4k37+FgMf2RfL+oX
fTyrv2D6pDjcKc7gQ1w8Km6Xeh54DpJLcCYu/i7uaPpv8kJcjPQ7xEXmQfR2
i/zS7ght3E0+sqbKaz3YwRiEfoiLoVF1Dzk0a6qLota/yTm59/bsqHsFsBc9
o8fT7BcOibl3Y9G4+NZ+qPGp18hXjC+MA+ibfMoYtJkcgWXmK+zYKup93L/R
DsZ1xhfq4m31jxqYMYH4eVlcOkfdP0iNAPdHex7rMMQPNRW1AXrdQ16oZbrL
C3ii6WP9Pj4dEfXsUdqmNiYGyGUTomoqxo2sqdAWYxE5NPeikHOZOwzUf3wk
Z2yg36wd/0ouqSMYw/fSJsZUcgiaoKbvFQ0NETe9tY/vEqfP2v6xUc9VzT3m
vTz/Iznkd8bqi2x/lsd76UN/7YHL/bQzryPnPskN9AsOu+kv373Sn8TlE+LD
vwu1hTg8QDxflyP8GCtv4E9Ovi/q2TN/kK8Lo575Q13/b/mFn+lRvH/mZ2go
5yTgOtq/o0XGAHRF7A/z3JAbfGSdnJoUfTJuEx/oFm0Mto2maMQfOieX5Jzm
Df351vaJiy/9nLiY5vngPsV2iIuJtv+9dtPvPHHCHuLiJe0kLp6zfb73pDYR
F4/rL3HxcNTzSR6Iiot7xegy/waeg6JR14LzyWLEucdqI7wcIS9j/f51Uff0
UQvBL3FxRVRNRR/oYXuxQx/ExQVRe9zgNHWV+2rQ9mliuoY28V3i4k9R14wY
a9EzccE4is5/pd0TYtG44JoF+WqM7W0sR+R+8hXXJVr7vdzTu4VckB+3itrT
Qr5iXkrOQ09f+hlzQNYRiSPyFdwTJzuIHXGX9+geKU7UmNS21FRDxWFPsSGH
7y0G14r3MfJFW9RPOS+n1n3Gz/bVt1yn+ihqzyec5F4KfD9F3xgbzxKXtaLm
gO3kso38XS3mO2r7yvqITStqI3HMXIQ8ih6ZuzHOPWubh8nLNn4OX909n9jo
Yzv/0aa8bj/EfsH+Uu2Z7zm5t+4I7ce5rvqVe8Dx9zs5mynPf4ja436TuPH7
OPF8W3/A+XnPx9d/ygn/bhVr7Bum/fBI/n9BfuH7OH+Sc2+Qw631Hx/RFjUO
+llBm9AVuYa51A/XiKLep8MaHGtmv9BnarrlbQtMWsvX+fpO/shcPUE/vrb9
Mf59QjMM+J24mGg734hH3iv0mv3OF68lop4Rw39f2i7tExcP6xdx8YA+jI1G
jsl7hu8SH7jIWom4aBJP4uJGcSYurhF/4uIqefm934GvnvKS184vilqnOk+M
iYushfIZAegk956iH+KCcSfngHmfQZtoxBYYEBd5/WZlbUdnxAW6Qc/ERa9o
6Jy4OFzMF44L7Gqnb9/Yf/P9EbnnGV2Tc7lekWtWuQd4XlTc4g/1HNr/r+d2
inpmEXmaGjn3LzLOsEZBDOVaVu45It9SU6Gl0+TrPLEFY+YTf/b78MKc+qCo
WgY/qQeoFYhXchx1eB/7mGgfO4jZSVHvk6a/vDd0kBitbN+MnYwBcI8uiZdr
xAMMmqKhv87R0NnP7O9+j8m7j8vR3mJAO/vISzs/z2s/e8pjD9shXnIf2Ndi
Qr+sfw7THji5KmrP4yXaj64G6BefH6K/nL+zOOQ6Kvh85eevymEf8WRt4QZx
5m8vij9x+G95oWa5VcwYy48UV/LEaLE/Xl96ydVJ8olv5HR0kvPm7cSYmm5L
ffjav/MfmmwTjbWgdzx3VXFHt+R0tISeiVViHJ1/7DH6nxyNenC67WP/VD/H
5kmej4/jbSf3/O+oDePsF0yf0h7i4nHtJC4esv1PxXAVbR6pP2jiNnEgLkaI
D3FxvbjhS+ZdNDAsah1liPgfI/5jxPhc+aKts+Rxb3nBFvRFvoN34oIxHD0Q
F8dF3UeODnIOCK/oal37yHubDo16N2lP21nBvuGNWGD8/dxjYhqdExfEwoRY
NC4+i5p30gb5Ct0x1mwqR9Stm8sP1zfQCromP3aIel9h5ity4m/9nLqMfDXd
z8CfNcid5fcV7eH3x/QRjTJnZ67GmDVcDKk1LhRz5jqnyBH+k8u5BnGwmN3t
T2L8kajx+/moOSCay3X1j6LurwaHnAMusD80RF1PTfxrOaBuptYlJ6CTn4tl
rpGAE/pbJup5mMuIAdpdTgye87s7ycvafv6ONnA+43BX28k5E+2jqz9FPRPo
Qu3JfVnYSd1+fdR+omv0i+NL9Je4HSAOc8QOfGaJO7jB+S7iOVk8wZl1maPF
/xk5Gaa/Y8SvKRq8gzE14Hh5GCB28J77rvexnxnyBAZT9R28v/GzpeRp/YYr
P9Qh6HAx7V7Rf+CypN9/LepZBbdE3UtykTjkPeaf2f4Qsedz4mKC5xMXb0Tt
I3jR9sHoafudLc+LyceDUc+rGWX7xMWd+vVB1PNviYsbxYG4GC4+d4rtgKhn
fYMnvF8gzqdErWcdK0bPyOkA+cr7M5LH47R9Z7nMdfUjouaAjP/zte+Hfbyu
i/bwmLgg53yvP92j9t93kwfiIu/fWkb/PvO4q1gTF8TCO7FoXDBWtNM/sCNf
EZO5DvVX+ydfoT1q2y3ljZzYIeo9udRr1HzryF3OL7eL2hNGzob7HbXj3+KF
3YxF5OHd5ZM9ItQWN4jtPvLDuJD7NdHTgeLNHORQeWFs+r1Ywltv2xorh2Dz
atQc8D157iB+eR0QHBi/2jYo+aG/tcX3Sv0mJ5APGBfaRENzy/kd4nkpcWCc
XFIsR/s5tfVzzbCEl1X9HL7W9fzJfkY7rKHsZvvMLQ63X2w9WXuomS6Ousft
Wu3nf3+NugflZv3lnBvFAY0NFZ9vxBvcZogzeOa+kNu0cz/7IG73F6dHxJ3v
3yGO4E3dl/c1nOn3e0btsUQPaJ06BZ0w92Nddmuxf1wclrMd9MZ1zc/l40f2
+TPPwfbU7Xx9PU2/v7F/6p3p9p/7tPjuoGjUmXxOXLzl+WD4su0QF89GxcVo
vzvddrGHuBilnXPEg3OIiyb9Ii6u099/ixM4EBeXig9tXChuN2gf5xEXp9su
P/uLP3z1k5de0RgjsJ+4OFI/iYvD5Ze4OChqDgiXeW/PPlHXAXOfDBrc1WN8
2Mlz4IFYR4fExXa2ic/kBnSL/skZ6HlJ/Z7s57l/c+G4IJ+Rr66Wv43FIPdU
5X1Jm8vRj+VlbDRq7NxntUrUs7KxmzUrxhH2VZCnya1b6QPra531jTU+5ny7
iGkPf4fnvG+PmKL+JXcPlQ/OOyvq+ng/eWMMIE81RWNs4Dyuj/QR40dtizig
ZsjrgIw5eR0QzJi3t9dWxq319AfdtInGmEn9spr4Eusrys2tYsM8/B65ABPG
V+YA5KAxHsP3c7axgbws7+fw9QvPn+xna8vv1rb/pX7R7zf6TlvExADtJEYH
az911fX6hcZuj7oX9R5xmOfx837nNnH7yu+CJ3OHIeI8UU7Anxx3rLw8oT2n
+53DxJ6Y7C2/5LAz5Ym/wTX87y8WnaMRN/SNftbX3o3Ee6iYMMdjPQ0d8t8U
Och7pdEt2j7P30/UB3TOGhhx9Zl4srb2oe1fLQ98Piga8z7Ov9jzaIe4GGv7
/HzYfsHl79rD9+/Uztl+J2u/6/QLTq7U3/ej3un4ojiBz0NR96iCVdan4HCC
OBMXx4g/cXGkvPxeDvLawIFigH72E2PiYs+oNQFyUV4H5G95Tz0xjX7W87xc
syIf5JoV88Y5+kku+SZqTzp4LCmP6JlYoFab7PEG4r5wXFDbkq/GazuY3xj1
/OtBUe/YpmZG/1uJ1VLaR/5nbWRb2+PaZMeoZ5NmvsJ+1qy+lhNyed6DxL+3
xK2bWIMt48tI8edvw+WIOSDaox5nbkTeYVxknCBPXS9PnMeYk3NLaoO+Yk8f
/bWbazd5bXy8foPlR1FrVtPtYy35RMs/3M8RjbnQCmJNfFPXMJagH9Z3ySdo
rZXfyfVW8hF5gfF/dXlZys/fsc1V5XEN2/lUbml/hvbTL+sie2oPnP1eO3Ou
h/3oarB+obfr9ZfP7xQHzr9ffGjnn+I202PwpJbJ/VYf+BnYvOlnua9siH3T
3kB5ZC2mn/xSF/SR25P8+65ymvsW4OrGqPcuEb/oahVt+qX4gcvK+n+ZOIIL
mlzM7+R1pbw+g0aZez3q92eJyyTbB5ucF2PDy1HvfHkman3oMdu/Wazol7i4
W3to7xbtpM8bbJ+4uCrqPS957ST324EDcXGG+KRmwY24OF48iYve4sz3emkf
cdFTXo4QZ/giLvaUR+JiF7Gk7x3ECF0Rx7kXlFhHJ8TF5uJEXGwctWa1oeej
2fVsBw3nNRPiok3U/UhriTtxwVx+cjTiAq2/E4vGxXtRz+v72L4/jnr+AvjP
jnpfDP+Rrxg/s4bjnBX17xn77WTfjBudo5FPNxMPuCcnM1f+wp/MbxhT9ha/
cXLF7/+QC+aAN8nBgXLGWMO8hLkccUiMMv8mlxwlV4xlx8gP+eME7WAcPLmZ
3QPlAWxyzYo9IBeLN7gM1T9iH42guW+0a3m5ombh/jNqf+r3n/rvIbngb+iO
MZ/x5nmPl7PvJfx8vOfyD42tZDvMG9ay/bxeMr8Zrtjzv6hnzDI+9op6Nlp/
/UJvF9knx9eJA/X8neKDLh4UN/Q2Rjzh7SlxnuIx+L8d9byJF/QZvshzI+Ux
r7XDL2sxeb2XuRVxubM8nhg1xsMZ+tnUttEVOf0v8oH2qSHQYeg/+iQOJvkZ
80ZqD/TMWEZuniH+c8QH7KihiYvV9f9ffs7fX/R8NP+k7QwSK9onLu61X865
3b7B9SbtnCxu2E9cXKZf8HeB/n6hn+BAXJwsPsTF8eJGXBwlnrR9mDgTFz3E
n7joLi/ExW7yRVx0sR90Ql6CX+Kig35vG3VfS97nlGtW63lMXFDf5rMu+Jdr
7KtFPUvk5/q3tBzl9Qz+lvunmPuh84yF8bFoXKBb9E59yZhLvvrM9jaVz6yN
7tAu8tXZYp97MVbQP8YQciP56jn96ixHm4pH1gQ7yxfzQtasGFP2Er+3xJQ5
M7GNLqiNiaN+/o3amZqINauztIM5xnHyCT/kshv9rLs+nKgd98sn81jWHNAH
MYEeGaMYL1lTJQ5Zw37PdtcQq2vlgbWNEWI/0z6Wlqt75QUO/ykv8IxP5Jvc
o7iY/16Leu7ReDnifHhcxnbgd2Xbh6c29su/9trD3zpq5zwxxX60d7h+0e6J
9s9xXrfjnOHiw9zpdn1CB/+I2kM4VjzgbZz4s37zirxQi7wqX6w5vSSPXDN9
Wg6J24fl/Sz/hh56i++22k5ezXu+iFt0lXvg4YOxlzFmhYYrP9S4rcUbv3O+
Ml8/DrNf8CPXPiZW/xOLd6P2Qrzh58TFC56P/2NsB2wetH3i4h77BY9bou4b
ymsexMUVtj85GuPhVrY5SH+Ji1PEAXz7ik/uxwG3nIOAGXGR99JcKZ4vRz2b
HV6Ige1tj7jYxj6Ji82j7pUgH3zn38E591m1ERviYnWPiYufR62xg/88P1s2
Kl8xdqBP4uKnUfkKnDJfoXt0vrj/0P/CcZH56pyofAUf5N1NPS/zFWNM5is4
yXx1lnYyFp6n3dvJEf7uIEes0+0kR9uI4xdiRL76XJz5x5hIDcWaFTr/g3wQ
L4wvrFn9RU7hDD2jV2pe6ua8j43x6Zqo9S/GPMb7rvrAmhUx8ZB+wMvYaOgU
XhhPGf/hhXH2CrkCN+oR8kbuG2R8YC6OdvLaD/lqiai51I/F8Ql5YKwnX+R1
o9eafT6+2fmT5DfvScv1XPBbxX7529raM1susRPtddT+3COKX3R6iP5y3Dfq
ftVB4jNPn7POvlk8mdPcJ86f6g/4T7Tt3JP8ptjz93fkkZpzgvxep5+5V4Xv
oAfWwKhJtpE3eN8k6tmH6+krc1D0tkyUDnPfO/pE95dE3UtGXkHPPfUB/EZ7
jP7n6ju4wTlx8bqfPyVXo7VztO18o31riNlI+z1LW3+kb9dq5y1R928RFxfo
1+Ro1Jf7atefxIG4OMY28v4McHtJDEdF3TsLzsTFLtpIXGwvL8TF1vJFG5vq
N/1tIL9d9Rvet9UnsCMuyEGZr5b3mHPJQeiKuEDrua98cdv8meeiT+Jivv4t
Ia7oGZ3ntVr0T54bH4vGReYr6v2cD9J3zge/j5oPjvL7WS8uGfXMPeYq5Ctq
FGrC7eSdup18dbPt7SRHYEHd+4GfscbK+JXXhj6WC+bb1LLonDk3NTjjBHMJ
dMHaCPXClfZ3pJyjUdasyFmsi/f1+zdF7ZsnrzBWE8vEHmMj4wxj7QVySB3A
/ABtvWQ/8JLrNCuL4V+j6tnMVx+LDbyQTx6WF+qfseJPzsl8NUNefuTnOb78
Rx5/YjtTxZ72v2jWL+sfq2kPMflr7fxOXl+Sz7yHEx3uHvWu54PEgc+P0Q90
OzDqOYGXiifau1Gc4e0e8f/A9uCF/PiCfD2tLX+wj4nyS7usbcM7dQY5Az0w
/5scDZ3AG/kN/VC3U7+hK9ZOqCfWFAPqnJXE7AoxWky7fiQ2OR9kbeEaMec6
2uvi+Z1YgCE5kbh4I+rZ789HPR9+tO1kvqB94mKk/cJ9k8fExXDtHCSO2E9c
nBc1HzlNf8GxnziAb2/xwcZDxe3tqH3IxMUe4kz/XcQfvW5je8TFZrZHXGwg
j/T3K/klLtA3vHcUz9xXvKzY8L3cv0dcLB6Vr76Pyldg8a0+8xN9EhfMCzNf
8Rl6Ji6oKTNfwUvmq+Zxwdw473fO9XbG56/0bYT95voVtpGvGFeXEYvcY0G+
yrqYfDVQTLrIF210tc2OYsMYyryQMR8Nd4/KV4xd++lD5ivmF+ifuoD4Jy56
RUMX5Jujop5bks/TgLNc3yXGWL8hPpgzMA53jobm8I/rINQYrEMwTx+jrzkW
sjbKugMxk3sYiMPcw0ANcZe8gO198vJhVL4iJsfKC1y9IBdT5SU8Tr4+EoPF
bWeqbdI+uWxp+4X7FaKeB7e6dsLfOtrP2LaJfi2QM/ylo93EYYFY3+b5vcUt
rzmC5ywxGqwNw8Uf+273PNapH5Qv+n5aHvns9ai1ReZrcE4Mw/vOUc9eRicH
2wf62cHP0VVb+2vTzO+fixN1xjLid6efUYcOi3p+4aBo6PxBfUH/xOm9Ypvx
nNdMiYtxnj/CPrKdB20/75Gh3+/0b3H/fo12DtRX7CcuztUv2jxVf4mLvuJA
XBzZDJ+Dop5ZvY9tExe7iDM62078m6IxHwIn4iLXC4iLdeSRuFhDfokLcg28
d45GjkIPxEWuTWyof5mvvot65v23UXsYZvp7XhfKfPWfqOdPwmXmq0+j8hU1
V+q/eVzkfobhtrGxfuaePHgiZ2a+Iq928JiacBvxJzbQVu6/ogZ9zPbIV4wb
rM139Rgs8n5KfpKvGCOoTalriT9yEuMHYwN56EB96Ovf4IZ5B7XtvdrUW1+I
I2ppxlPii3GKMYV1XtY8WAu7VdtzbniONjJWXiQu+EDtTq3CfODP+vqCbcEL
Wsl8hbYZ35bU1sxXxOQj8kIsjJWXD6LeS/pBM14+aMbX+814pB00+xP/Dmet
7ZfYWzZq39TKto1+1tR+dLWefqG3TfWXjn4rDny+q/jMF6sbbOcI8WRt9kRx
ni72/CO3DpMX7BshX8R57qkjnkZH7d9i/ph7VfhOPiuI/NA5GvH0H7nZKerZ
tqz/EhPryMOXcgPGaHx58UPPP/Uc8jR63iPqng44GmIf8/WjeVyM93PaecXz
c+963tP6sO0PipqHoK1btIe4uM5zUlPYn/v9d/L4tKhxtZ845Lo++BAXB4kb
GtnbvrFxZ3EmLjr6GXGxhbzgy4byRVysLY/ERdaG+0ddNyIu0NYCsUd/+fy9
eX5OXHwb9f6Hr6Py1RdRe67yvssl5SbzFRx/LEZgnjpP/Ue0jItP5eW8qP3t
jDXkTPLVQ7a1hZ8RD+QrxiyuWZKvHo3ac3W1mGyvvbmHgbG1k5ieGnVNkLkZ
Y9Se4klOYsygzu8lfsQiczvGY+YFjA3UxNTpaIIxe4w+kK+YFzAWka/QBTmG
2GLMvzGqJkOvZ9g3cXquNnI9mrFoU32jDWr3sfrHWPScbZETWNe8RV6oGzJf
sTaa+QrsM19NiMpX5J/MV+8144Xjtz3Oe41zDT75pf3PbZ9+0cPS2pP3emLn
HLmiT3T1S/1Cb231Fx1uJg4cdxQfYnsXfUWH+4nnN2IIzoydx4v/VDkZqN2D
5Yu56LXy+Jgc8X10RH0A79Su6KS734dfdPIH/Ycb8ijaR1fUCnk/DnUVcZN7
sWZH7TUhp6FbaptPxY52cq7MnO9h28p7AThvE32a6OfExb88/yF529U2Rts+
/txvv49G7blCN3/Rztxn1V5/L7Id4uIM/SUuThSH3KsAPsTFIbaL/fuI5wR5
ekncOvnZGLGaKuYbyBeabiOPxEPuMYaT5fSXuMiaqpM+cUxcoCn0Q1xQo+V9
zTP8G3GR+3GXs//MV+Sl1C25KPX8fpTOm+u/eVzQfq5f5X50fKGGRb/U+NTV
W8ojeHewf/wlX70rDh09Zq5LvkJXjNmsnxJbjCHo7xWxYO2E8aSHv6P5w+UA
G7i+R75Cz9RHPf0uvB4mX4OiwS9j9MV+h3VtxnjmIYzvxAkxg96aojH295W/
M22XMfE87aO/wfqJFobqH+Mc2iHeWcdAf/CMRshXzNPJG7mnHfvulxewz3z1
jm3BC5rKfPVuM14mNOOL85NHPkt+aX/hfLWk9qRO8lr9yvbJuLh6VH31K/3N
On+Ux3k9GN12inoOwS5RzxXZV5y/lA/wp44/Rl7ynoFT5e18eSS2r4h6pgMc
HS626DD3AmMjuqAmo27IdU5yNroibqgb0Bu5aYY8tYq6lzCi1it+5nHe+0Gc
ofORYpox+HLUM3NzXZfPL5cjzj9brmiHuHjG9omLR+z3XfFs5fEt2jlNDLGf
uLhUv8DoHP0lLgaIA3FxrPhgQy9xg7f9xZPv7u5PdLG9+BMXW8kLcbGRfIH5
r6Lu4VpNbPrqB7wTF9Sl6GFnfeK4QzOsNhBHdEVcTI96DgN+fmN7YIg+iYvJ
Ubp9P0rP70XpHLxS/83jgjqVfJXrI7lHF07JVxmTW4ol41eHqPWdbeQKfXeM
2sffORpjQ94nCCd7+Tt8HyC+74g/f2NsoTbdV87QNtpF52ieOjjXdYkP7oEg
RpijoLF8BgD25DXB++SGfEU9frtt0QZaPNPvUGOcZ9+Mi4P14Qnb3cz+0Azz
jxfFhjk78y9qEcZXxl+0RJ3D/CHz1Tu2m9dkM1/939RXnJ880k7yS/vw3sp+
c88w9lDnL6uduccY+9HVqvqFJrPWyr3KWWu1Fx/O6SBufHd78aR+2TXqWmHm
LjR6iLxg89FijV8nif3YaMR77sceIgfwQxz3iobWmPMc0Iyf3OsCduhoC33t
JA8T/Syvb+R90MQTuiV/U0Og5y76tkwz39A/MUidRVwQ58TFDD/H7o88/y9y
0UVbX7F94uIp+0WH/9Qe4uJu7SQumrQ1r1/gF3ExWH9p+0zbJi5Sv8RFb3Ej
Lg7WLuJib3GG567iT1xsG/Xu77wmSFzkPRu0m/um4GolscGOvOd0X/HhuHPU
OlE+t4XfiQs0OEc/0SY6JC6m+jfiYkr8v9dX1Kq5XxS8yVfUq6tFvaOZOSr5
irkwOmYue4k/txV/fEA3rOPsFvXsAPwkXw2Jei8yXJNjyFeMIdRBe+lHf79z
njyRrw5txs0ptsV43BSNGpm5PXUy49wf5Yv1SMYK6mFqZ2Llbn0gZtAI49NZ
2v9Y1DPwntQ/bGI+RRwynlO3XCNe1DNco2kTjTGS2GIMH28/YEmeIc5ay8Oj
4j0pKl/l+hXrHZOj5fpV8jUpikd+EjOL2z41dyv7/dK+sCf3EmMnsbeC9ude
PvxCb2tqS+79y2uFG4pPXsu+1+9uI560uYM4fy2X12vPfvKC3YfIF3OU3mJM
PJOr4Pfv4g/vrJER78d53pXycaxY5/5h8EUj5BPmceiNuQi8d45GTUCN8Rvx
A9f24oG28ZO8S6yi81vEYrmo+8DhiWvfc/23wO9+6fnExce2AzYTbB/dvGK/
A/UVe4iLh7QTvEZqP9iM0K+/iBv+EhcXicMU2wIf4qKfuBEXR9oHcXGgODdF
I/+BP3HRRV6eFRP6HyMm/xNPMIFf4uIX+kufuW8BHlp7TFwsJmbExXd+l7jI
vcptxAodEhef+zsYf6o9rcUQPeezdVLncPZ2LBoXK0Y9I4n9a7m3nnUNdEq9
SF28pe111Gc0Tg4nX5GfWXMgX50ubp2joT3Wwqnj4ZdckvcJMl7kHobT/T4/
WUPaTxvhnvEV3TJ/ONjf6a9XNPJ/k5zx97zvhrGDOpx8Ra3MejLxgLbJGad5
7thojPPko9wHQy4jrofYx+u2TTxSz5Cv0DH6JIY2EGPyVd5PcI+4wgP5ijFl
qn0zpnwifovLXearadFyXTH5+iSKx6lR+eqjqHxFv5mvODfzFXbmnj3sz718
+JV7/PA371d9zuO24oMWNtZ2vruVeOZ+VHBmfWinqPt0uskLNv1Ovsgfh4ol
4zx1AvxSfxJ/54on+KOD3E+HTqiTue4Gt+QpajB0hQ7JHegN7RLD3cWAOO0q
li9qKzyQ1zePes8wfpJzv4p6PljWo9vqf86JOJ7v+d/qN+18KF9Ly+t79ovf
r2kPPj1tH8TFP7U/6/F9tXOE/sLrVeKQeNwkpgPFLa97gydxcYQ49xGTyVHP
4s37p7po9+io59URF+SueXK4jn4TF7kvFF5W8LhXNOou9ENcLO53txYf2sz9
XOiQuKDG/VJcv9Q/8GIMQMd5rfB92yMWUv/N4wLbcj/Dxv7Lva35PBXqoy39
fE/9ZJ7aQ07J6fhAvmLcQFedo/SGlvuLO/mK8ZlxIvMVuQGdnSBH5Ctqh+vl
AWwZnw6x/buinpeERo/yGA2wdnK23+nn9562f3LHOO24Xww4l7kcNQn5ivWB
CVH3vTFGoQvqOcY5YoXxnBwyQizwgXy1vviTr1aXF/qEZ8a0R+UZrNF1K3l8
IeqdoLnPZFYzvmZG5auvo/JV8t7KfvOerM+inguLnZmvpkSt5eBX7pnB38xX
b3m8nrbAf3txy/vIwDPvLxttXzuIP9d9dpWXD+X1LtvtIR/kxMPkN++1gvem
aIwxYD9Y/NHJCfIEb6zXMNdEV9vIDXpjXkt90isac+Lc7x1ysKfYj42GnrH9
lah30KK3jfWd3ENcdBT71fw8rzfdJC6L2Q6fz47aI/wf+yUuPtAedPumdhIX
z9s3unws6t6NUVF7rm4WB34fLj7YeYm4gf1Z4jky6l04B4jphx4fpE1ny8cs
2+4qjw/oR943mnFPXKyv75drez4/jRo975XIdUHiAv2hN+IiNb1+1J7OvMcJ
/1YQt4+insGKzjMu3o5F44J2yVfM/XeIuh+nm3aTH+E981Uvv0M+PSbquXzM
tzpF7UvbIeqZ9+Qr8jw+o+V+4k++Yn5+kzgy/rM2QL5iXs8cnXGCcYe5ySH2
ydjJ+Ep9Sy18lL+P8++5t5d++tguexgYLxj70D3jyOSofdzULYx7jJ2f2//l
YkxMUG+j3VxfYXwlXzG+zNHuxAW728oRWlhDnh+N2n/9pByhefLVErHo/Thv
+zP3NOT6wUee831UvpoXla/mROUr7Mx8NTNqL9+MqHyFv3m/ylT7WC/quQ3t
ot7Vu7l40sfW2ktf20e9S6+rvJAfu0W9b2XfqOeR9oy6ztZL/Igh1rvQAzmL
3IVO/iR/8EH8sa6Drn7rT/RGrqUGO0lcqYPRAv81ReUxapke2v6Q9t2ovTvo
+2tyyR6miVHv5/xMjDj/v+K4tZi2tv154hSek3uOPhBb7CQuJmg/cfGyfqHX
sfpLXDwgDuB9h/igyxu0nbgYKp74eq44ExeniD9xcay87CIO8EVc7C+PxMUe
2n2HOMD7PeKQzyDaJOq6Q96nQz+/jMoNrI2iN+JiRfuiDfI8mmsbNaYSF2gX
PRMXuYaVcYEvC8fFHvLxjDySr17Xz838Pv5uaR8n2z92kK+31WfGPPLVbDnY
wfMZQ8lXrItSu+e+HnTTTX/hZW8xZKz5XVRdRL56U24O8fM3op7tSn3PWMIY
M0lb0S+1PuMSmvs86nl/1CTU09S36GlQNLSNDxdGrWWgi5vE6kr7Altq/NPE
nHxFDU5dS7ztJS/EIVokRzAObiQf+LaWPBIfK/mPfLWMxzl/WEm+Wnk+cfMT
28n5/vJinPdHZ75aOipftZaT5W0r81WuP/zCn5mvwCHvu/9v1P344IaeNxNP
xsjfiPN/xRn8p8v3+Kjn674R9S4DeCSv9ZBfarJDxeYW8UQPOf9DJ9Qg/fx7
T7lEV53kGr1xrYta7GL9pwZBn+HfTxYLvodGOtje7+U0r0viO/Mj4qJzNOKi
s58TF1t5fu4n7SAHec/1LLEOscz722f7+6+i7s/FfuJisn5lnY+/xMXzUfeV
JT7of5S45T7lcf7MvWJD9R/8iYuz7A+sTmrGV5+ovbPUu/BLXBB/8H6N/OE7
fWceIy6oDb+3L8YwdEVcbKSv8Jh7HYiLNtHQJ3Gxhhhw7mraSVxQ96NztI9G
0f/CcQFfuZ+BdU7y1cPihTbRFLmefAVf5PvfRD27Oq89jBADxpM79Q1cmDN3
1VbqC/IVYzXx2k3c0TBaRmv/Ei/2KFD7kq/IO9S2h4r/NO3uG/WOGzimnkSL
/cSP793RgPuHGhY7iNmB+rCUvrEuRk17kb+Dz2XyxXjBXOV6cUarjEeMKU3R
0AIaJF8RX+uIJT4wBv1DLBhX0BzXZzbQf9Y4Wdd+Qd42jHrOz4bytbTnT5Sv
tvL7U9uf5ufriMWS2pP5CjszX60Rla/wK/ck5z0Xv4y612td8cl81SoqX4Fn
5qvvoq4J85O42DHq3Tq7aSPXu7rLI3zvL7+PiD+8M6fuFVU39BanAfL6qOey
Lwldbe/f7hRLeB2h3/CIPkNeh4jZZf6NvMPa8inyTM1ynL6TN9BXl1g0j+3u
+U/bP+0QF1vYPjytb7/4vbr2EBfLRj1XZTG/D07f6hdx8Zk+ERfvi8PF9gE+
xMUz4nZCVPwQF3eLc9+od9z010946Wdb8ERcnCWPxMVJ8gun6HhB1LN68R3M
DojKY909n7igbkMHxAVan23b5KZZttdBX4mLzcQm53PombjIeV7GBfpfOC5O
9Txq3PP8/i3ymvsHmQPlfSo3Rz3zir+Rr171b+QrNPiEtuR9++BBPco4u6vY
cR75aor/9vbcz/UJvJm/9BR/xibyFToPeUOH1BZHyxe6QHOMpdQkJ8o3PqMB
NE5Momv0T4zDA/HY1vbAuL3+HykG5Kvzo/agXSEGTbYPBneIH2seaDxru39o
d+do5KvdxQb8GKO66PdGHsNLG4/ha1XPh8cVbAe8lrP9afpNv+hhWe2Z5TnY
Odvv5pz+5/qV+Qp/M1+BQ+ardTze0L8xnm4atRe6gzhnvlpGG3aUF/jfTb6Y
/3SXR9b34Rh+yUGsr3wuhrlOOEz80Qk5Bc2jH2rs3C8PFsQZelvX8x4SA2qJ
exrm/1BDo1vyCfNH9Ex+oQaB56vl8Tz95XfigjqRuMg8lvXY1ba9v+08pJ8/
lddO9vuqOC+rH+v6b6oYYj9xsbR+TbefnMvNEgd8/VR8RtjWo+Lwqnjub9/g
fJE20U/mMXghLprka6h+wGPu74df4uIc7dhITPPeB/LdAnlhLEE/58sZuiIu
qC9m2/5++gCue2nDCVFjGHGxs77uLt7oPOMC/S8cF8xF29k+tQP5inx8q1gT
n1yvRevMYcnrxABrqIwx+EH9ztol8cN1UmpSYoy8Pzka+Yq+P9NWPkOr5KtP
9JV8levIYE9dzPhOviLmV9C/C6LeJddDH8hXrI0wrh0nJ8Rg3i+CpgaIA+sN
aJlYQ1fn6muXqHvZGS8us+09xGM7Mb/e/vaxH7gl9tDSEG2iXr9Q7rCdOpmY
JF9RgzNuPeW56DRz+iFRayeHNOPrYHlcO+pdOavbPryvYlvofeWoXLCydqKf
n2s/+lw1ar1iDf3N+SA4ZL7q5PGG4pb5Ku916CDO1Acdxf9L26At4mXXqHuw
95LHl7Vxqah3oMM7dVJqnzn4kfoBfuQr9MO8hetnk6Peh4DeqDWJR3S4rN95
umH+D3yjW3LooGjomTkMcXSr/ML9dfoLj5fLSfM8lvUYP9HBcVHP3e1l+3fp
G/0+oP/YM0aMsJO42ET74fjX+jVZLrGduFhSHD6KeibZW2ICbv/wPPC8QIz5
hxZeFf99xQFemqJRI8IXcfG3qGtruQ7SQd+yzhzmMXFBbZY6AUf0s532oivi
grU39HaO+GDrEP+GPokL5orT/B64oec+0fLZp6n/heMCm9rJFzySrxh3yN/k
K+KW9Tvy1RXNvjdcbMhXXJf5WNtpb4ZcjIp6RsUjcki+ekYuuGZDnYuGiSvG
BuKQfPW+PJKvqHE30h++i0bIVyPEGizQcGcxyn30aBnt5DrZPfKHfsGf+pZ8
dWzUHkfaZQxFr+iUtXz0ic7+KJer289fbYdx5w7PPVG/WRNm7EdTaIlYekye
yZ3kq/4ej/O7p4jv/n7+tn4MiBp3TpbfTWwf3te3X/TwK+1BJ7/UTjhYS/vn
ekx/6K2N/ua9YOAAUeuLT94vBm65PxA8M19hX+Yr8M98ha3E0q7yxVxnL3l8
RV7ob7Qcg+ldcowecl8dOuH6SJ+GKT/ks+P1aUf9niEG4IkOqSupj96NerYM
uC4hL8/qB1ygc3ICOrhXf6kZ0DF5nLi41s+zHrsyal5JO7k+RvvDxZl+iYtD
tYf29tFO9NFV+x8RK/xC2+21l7hYWxyIi5XFh7hoLW7vyOErfnemOI8Q+0/E
6r2o58y9Ll/g/5w8EhePyy96zj3DxMVIj4kL9I5OiIvro+5Fxca5cnip3Gwv
dth0lPhOF3/miuiWuGCuM0UuyF8To9bv3o5F44L5b94/SB4mDuAra9kLxGtL
+0cPjLXk51w/gzuuCWwnd4yVXeRuablhjY4xnnzFWg+xRL6iFm0bdc/gZtrF
WiPzmp5iTHtomXUOakdy9Et+Dy2TFw6UHzCD/z/5WfrMWsOxUc8LP0Ec0QE5
gXzFHAi9ZL5izTbH1XPliWsxeS1xFbEgX6EpxpJR2g4uXA/rFY1xmnxFHDFm
ka/OEa9xcpjjQX+P4etoz58oFkOjrrtdFrUOQL/Tm3EzUyyx81s5xn50taF+
obcN9HdB1HuzMked4PEm4pb3vYPnd/Z1uO13Emv2Me0oL8xhdpUv7N9LHtHW
fmI0Rr+wdaQctxVzOEYned/Dz8WSHIyudtLGxf0OuKHDn8nXjKh8hW5b2RZ6
3kTMXxXrQeH7AV2r+ae2o4G8p5K4aPJ84uJq27lIzFv580z7hZ8TtecKccPO
4fqM/cTFnlHXSLtoI3HRQRwelxvwIS7WEjfiYkV9IC5y/ewluQV/4mKmvBAX
0+QLriZFvU/wLfntEbUfj7h4ymPigtye+eoB+8g6AF0dow+0v4pYZZ3PvBp9
wvm12tEr6vm0xMVV2g/+jAlvx6JxwVpvO895U/zP0ZfNPKYOz/txWCtEp+dG
PY+EY8ZB8hWcriTuF4ptV3mEK/IVuWBzebraNtD3XzyXfHWHf2eucL84ouXR
/kTLjJPMo/vI4/H+G6Nv5CvWKAeK60B9QE9o8aKo54iiLfLVvmKFjWgJbaGZ
NtHIVazLDvVv5KsT/UmtzhjEOEq+Wl7uyFfL+PfHxDR1gGbQ4DhtHRm1J2yk
fJ3t+fB4mu3Ab1/bn9asX/RwgPbMFD/s/DbqPjd0tb1+obdt9XeBvIID+vyN
+HC8tbh9H3U/AzrvJM6MpzuI/Vf2BS/Ey+5RdX53ecTH32kfa089o56Rwbm/
1T443jzqnqm2Uc+PTV2Rr1aSQ2xBh5mvFkTlK3Sb+Qo9M48gFibJJzi/qb+D
ohEXu+n3437O8X2ez/FttsPxdbZPLFxuv+f6D3uIiwHaSVwcExUXh0e9B3s/
/SUudhUH4qKj+BAXm4kbcZH75IiL1cWZuFhe/J/XLnghLuZFvUNkljzC5+fy
S58f6i+25t484uKNqOuDzGmzPifPzxUX7EBvxAVzz5n6Rt6crl3szct1V+ak
U/Tp7qjnoqX+F44LOCJfwRF7RDbx+POo+53pf0uP4WFruWaM+634kk/JV7kX
JseJdvKSa0a7eX7naOiVNtH0PnKKjnvII7U0+Yr8iv7RMvmCWEXLxCu67CMG
eW8oOF2or4wJxCD5G6392f7RF/Gb9zc02WcnuQT7NcUK7VAzjLJ/xgG0Ql6g
1icn3SkWD3seunvMvx0jj+ie2EI35Cv2k6A78tU6HsNLN4/h61jPn6jdo+X3
PNuH93Oinu17un2ikxO1E/6O1q654ohf88Uaf9Hh3uIAyd2i1jF2E7fUM3h+
5zE4z5ZD8P8qav6Nbd3l630xgUfqggPkFz8Plnfi8fCoa3DMBzvbLhyjH3gn
X6Er5qLkK/RG/UG+QofL2z/6XExM0C17C84Qj/a2i87JE+jwA/0dFI24yGfF
vNDs88ebnX+f7Zwthj+xzeujntF3ufacIWcbenyK9p+hP0d4fi/boM39xeF8
MQGfwVH3tRIXeX0SXW4kzsRFG/H/u5jACxpZRr6Iix/L431RzyMkLmbpL3Hx
H4+Ji7zve037QT/EBeuH6Iq4+FfUM6+Yp6JD4gKNo0808Ky2ERe5T4+4GBsN
nWdcoP+F44L8mfmK9bnMV/m8PnDL+zDAkTl6rlmvoC2nRz1TlLzAOEi+YjzZ
Iur5a/wd7bPugtbJV3/Sh32inhPTw/PR9kFyiy4Pt9/T5BceGb/QMjHE+gL5
inrzSvu5WQ6wi/xNjkEX5PW8pne7v6MF5l7UuuSrP8rx1fpMrrhBbJ+0rXbi
f6e2oG3yAutIuc+Vup810cfF7FV5OlMe4HIDj/O+g9fkK+v9iWL9svz2sv1P
xYN+cz0Te3KdEzvRz8XaP1c78Qu9DdTfBeIFDrl2cYfHR0W9h7KXeM6TK3BG
/wdHvXv1IPuY5ufYl/cSwiPxdZj8jrXNQ+0bXveTQ3IsOkGDaBv9kMPIf+iK
9SVyMnojVtHMmmJ8ij8zX6HbzFd5r1vm18vsI/fKDopGXBAfg8Q/P3+q2fn/
bNbO3c3ab2rW75+1A/0N1s4BUc9V7y/uR/qzj21nLOSzRfYTn/5iMlYftxNP
+t1SnM+Pek8Wtq4tL7S1atRzrX8mFneLz/dR/POPuPifvw+zDXRCXOSzwrYW
N3RFXEyOWn9As+gQLbDWluur7H9Ft3mf3hQxej1qXE79LxwX70TlK/Jn5ivW
5zYXu5zjgnGr+P/PV6dEPVO0vxjm2jfzEMaJvv5kPYO4YGzdV67ISeSro8UF
7feWy8P9/Az55e8X+pM2L496JuM1fgffWX9EL8Ts7VHP8IMb8hVza+pV9NQU
jbFnqJ+PjUa+IvbJA+QrYif36OE/4wj5ankxvzfqeZfkq0Pl6HG/y3j0lJi8
Y1vdPYaX4z2Gr56eD4/L2Q78/sT2894R+kUP3aLGtX62/W1UHstxEL/mi1Ou
SwyPenbfFVHaHSJunHO+eM4TR74/W3zB/2vxpz/mXXlf8yS5gZM39BHenox6
jtAoj9HDtVE6GWQ76IfcSfyiq13tC71lHsh8dVpUviInt4rKJ+g58ww6Z10H
nWf9MEj89/T45WafP9ns/AebtTOyWfsjYtF8RVxkvurvd7D/RDE5Sn/72A9x
cZg49Il67vLRYoIdxEVn8eTz34gzuOZ9KrS5bjNe1pAv4mIleSQultHHK6Pe
iYL+v/e4KRq5i3P4HH2hn1w3zfWrXB/7rXhyHnEx2fPAgtoM3RIXXAuY4ncZ
IyZGxQX6XzguXovKV1xPyXzFtYXMV/SZ+Qr7M1+11i60wZra9vKS88GTomXd
vrVYU/9SR+4lF/wkX5GDiE/yVa9ojLUH6xe4o2ny16meyzz3HPk5MOoeDs65
yv4PE9/8zq36gQ2MK7muwLxpsO1xzYZ8hS6pRa+2vWdsi+PMV3Cd+Yr8gHbI
V4ydrGGi6dXFnny1uPiTr9rKEW0N9Pi1qHv6+M4fPB8ef+Rn8Huu7cP7bvaL
HrIOR58/i8pXm2n/HDHGr3n2i78Loua5ef/F3R7fIW4L9PNGv3ur30HHI8Sf
fq+LWve8SuyZa10mP29EPQfoKTnoKW58by/bQHs7Rq05op/eHrfT71x/aOfn
6HBFj9HnYlG6/UmUnjfy+BPtHhQN/Wdeek07OH662ecPez7fvcd2aP+vUflq
uP2e5rnYc4rfx050ebL2ExfH6hea/L3+EhcHiAO63VN8eokJuBEXef/uQXIM
zsTCBuLPcZuoujfvlyEulpfHI6OeI4oNed/XAP+ea3ffes6ltjdPftlDNsfj
zFfw+7HnEReTo6FP4oKcim5zvW1K1PNG+D3jAq0vHBejo64doe1cb2ccz3zF
+lvmq7wGdYZ+Z75izpn5ijjJfEU+z3yFHbvLD/U8ejja75Gv/uDfe0TtDSVf
9ZQn8hXjzDFizHoLYwb5qpv25Zh8kf2jiWHayOfXeh78N0UjVuibOLxEThnH
yFfEE/lmuJhThxCrjJvE+81+/5lm32dehq6oL17y+2BCDnnc9sEevaGv18V9
KY9fk7vX5aujn+V1k1fkt5Xtw/uS9vuF7WAPOhminXn9GvvnRL0nBr2tHvXM
9m31I69fN3mceywX+Nkwv3ufOP/PtsF1pn7Cy+d+Dl/EUd4ryBzg71F7HO+S
m7+LKXq4PuodqWiTOg79ENdX6gPfoaZDb+31Fx2SHwZr12LaON2/wQt63lDu
/i1v58tDXv8iLvay7/v9HPxvj7rX5wbbOUtMaJ+4uCAqX50ela/6aWfWU9hP
XOQcgrjYxz6Ji13EAS46RT1DZktxIy42iron5NdRz0jO+0K72f9M+1tKXHbT
3nl+Pl8fiYvZ8k5cfO0xNn7hOcTFJ34Xn9DjHH0lF30rnu/6ffAa7/cvjJoP
gsmbfp/2ybvoPOPi7Vg0Lhbef5X7GVjL3TzqXvKtotbnyVdnik3OB5nPZr7C
38xX9JH5armofAWema/Yk5L5ij4zXzG/JF8dIL6Hex5z+SPlgnHij3LLmNTX
vrPOpg1y9GlR7xtDX8TmpfrK8VXiSn5AI+ivczTGzeG2T51BDKEHxlZiizUZ
4vZO+39QHInXh6OesUXcPi6e/HzKz4mLcVHP6oKXffw894s+HnU/zqPyu4Tt
f6qd9POFPGIPOllNO/P6NfbP8XP8Qm+X6y86zGvKeU3wUo/bitsCOR7od3O/
Gfofov/ERe5LIF6eiHqn8EvyiIZel5unPYZ34pG5F3ogF7A/aHUxAxv0g17I
e+gKLeUev/b6hw6Z5zTZ52K2RZ/EJ7UyeibPMOdF5+S2y+QNH9EBcdE9av9V
zoUv9XxsOidqb8gA2ycujo2aD/bSHuKiR1S+6haVr7roF9/La+/8vrG2Exfr
RuWrNcTtgKj3nRAXS+tzN20B/13kaqb95POpwH6GfyMupkWNWVPkneOJHhMX
4z2nczRy5DzbZ30PXeX9oejtEHFGh+iCMRR9EhdoH92i/7FR84Un7O8s+Ub/
C8cFOmwXdU/CJv4Ep83lDh2Qrxg3HhDTQbaPr4wpaBEtn6pfuX5FTZj56quo
fIWvma9+GpWvVpQH/GXvTeYrdNFLm7GFfLWHff7RPnJtDDz3i3rXPByf5vd6
R71nkHPRIBoaoK/kBzQ3zGP+PlyM+PuNYsTfb466D40aAQ1Qv6P7XT03n1nL
fOnxqLoFzvpF3a96usevRd0/8ra2NMnjCbbzoZjQ/qdR9+Cih5W0B50srZ3o
p4P2o6uL9AsOfm8f33tu7r+6WXyy1gK3nA8e6nf/KM7/i1qbJC529Hfi5Qjt
m2z/8Eh8jZJf4o7aEN7Js9T+6AH9TbEfNMoeb/TTx7bQ1R6ej96Ibcbnl8SA
WEGf5DVyP7olhlmfQ8/Ml+62n4vFe6g+3mCfxB9xkXsj+Ptxnn+J2NHOBXL8
Y7nbw36Ji87ac6o4YCe63Mjz8vomfhEXq9o3cbFcVL5qJT7ExfyofPVN1HPc
p4sN7U4Vf+Jikrx0Fiv4Ii5ej7qW9mLUsxnJLfDeXgzz3hzGyu+1PevzdtqF
rjaR129tE3zRIXGBbtAncUGN+mnUO18+1Ef22KPzjAv0v3Bc/B8jDl+T
   
   "], {{Rational[-1, 2] Pi, -2}, {Pi, 2}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["48: xmas1", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Remove", "[", "myColorMap", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"myColorMap", "[", 
    RowBox[{"ab_", ",", "ar_"}], "]"}], ":=", 
   RowBox[{"Hue", "[", 
    RowBox[{
     RowBox[{".85", "+", 
      RowBox[{
       RowBox[{"Cos", "[", "ab", "]"}], "^", "2"}], "+", 
      RowBox[{"Sin", "[", "ar", "]"}]}], ",", 
     SuperscriptBox[
      RowBox[{"Cos", "[", 
       RowBox[{"Log", "[", 
        RowBox[{"Abs", "[", "ar", "]"}], "]"}], "]"}], "2"], ",", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Cos", "[", "ar", "]"}], "^", "2"}], "/", "2"}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"Sin", "[", 
         RowBox[{"Log", "[", 
          RowBox[{
           RowBox[{"Cos", "[", 
            RowBox[{"ar", "+", "ab"}], "]"}], "^", "2"}], "]"}], "]"}], "2"], 
       "/", "2"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QComplexDensityPlot", "[", 
  RowBox[{
   FractionBox["1", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", 
       FractionBox["1", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", "-", 
           RowBox[{"1", "/", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{"1", "+", 
               RowBox[{"Sin", "[", 
                RowBox[{"x", "+", 
                 RowBox[{"I", " ", "y"}]}], "]"}]}], ")"}], "2"]}]}], ")"}], 
         "^", "2"}]]}], ")"}], "2"]], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{
      RowBox[{"-", "\[Pi]"}], "-", ".2"}], ",", "0.2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{
      RowBox[{"-", 
       FractionBox["\[Pi]", "2"]}], "-", ".2"}], ",", 
     RowBox[{
      FractionBox["\[Pi]", "2"], "+", ".2"}]}], "}"}], ",", 
   RowBox[{"QComplexToColorMap", "\[Rule]", "myColorMap"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "100"}], ",", 
   RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{3.39002675472412*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", "100"}],
  SequenceForm["pl = ", 100],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"pl = \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"100", ",", "100"}], "}"}]}],
  SequenceForm["pl = ", {100, 100}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJzsvWfUnVX1rz1pAamC9BrS6L1XgzRpiiCCChiaFJEqvSQ0kSJdOhikCIhK
l04QEKVK6CAQeg+Q0EKAnH2xr3nWfALP/7zjHefD+eAzxj32eva+77Vm/c25
6j3/trtvsttkEbG9F+WXOtefOtd3OtdGnevmzjVT5/pP57ovun8HdK5nOtdq
nevpzvVG51q/c63auT7sXHt2ri861+eda87OdRMPTRKxSOdjqs41vnNd17l2
7Vxvdq51O9cWnevezrVH51qgc53eubbz+n7nOqFz3d+5vtG59u9cj3eulW3n
Cus/qvPxVOca3LkO7Vxr2t6D1v99n12oc53ZuW7vXD+w3s06142d6+3OtU7n
uqFzfda5Lu5c+1n/EtIN/bt1rvk71/uda1rl8GLnmrxzbdO5RnauNaRzROea
onMd1rn+3Ll+1LlGda6zO9fqneufnes3nWtA51quc+3buZbvXAd3rh061087
11LSyL0nSeMR6malznV45/qgcy3Wue5Q7iuoj7WV8fV+/qRzDe1ccyjvH3au
BzrXcZ1raX+7vHPNE107QD5zd65+yof2+iuTQ+ST7+9UVjy/U+e6tXPNru6m
sfwfdTFUen7vs9/rXM9aJ3L9lt9PVWgY2Ll+IW2HKLPj1OVx8sJv28nj1p1r
UnnevXP9Spmcbd3I6jJ1Ak+Pdq7HlC31L6zMt1b+6OLoznWVOrqoc80iHfA7
tXXP3LlOsV7awDZGdK4po9kGdIyKrs1M5v/v2xbfjVfmdykT6lk0uja5vPy9
rRwHKh/0vqzyQAbnRdfm/9q57laWx0nj4M71a2lM35ktuj6F72wQXV/7e+ea
z3p2lyZ889/RtTl89tLOdU90fXkd9T3eOn5u/Wd1Pv4VXUxYpnP1ja5tzhBd
/3xDGWI/YMte0RNz0CFYtLFt/0k5X2X7a9kOfosPjouur1C+oHM9oc549lV5
W6VzjbHdsdI8m/qkLvxxRmk+WB2+17n+Fl2fesz6zpGGHdULPINt2Opcneu0
6OLqdJ3r6mh2jq2eIS/QOK28HhRdTMPmDlCO6OSF6GIUePpb2+A7/Ag8+rhz
fdu2qB/7xX7ASOxr8871kfU/rL42kz5sdNvo+gt66tO5npNXZLhedO0dv8Se
sO/t/RwWXd/DvneXBmwDWwYDEntv9N7B0bVDdLWM92K7S/o7v4GL2AKxYGPb
AnvxwfX9Dttf1HbPV1cvWtekyvll6f9YPWK/xCH0DeZgZyeqP/Drm53r0841
q2X0PcR7fhld3+HZi22HOheMLs5nW88VGl6WNnB9tDSf31XPl7zQ5izy84hl
eAd/XlUmPHOwsjpdvSFDfH5PZYt9bKLM95DO1MVY5Uo8fF2d9Vem6BRbei26
ukaW4NG//KS9p33+OdteS91gS/jLhOja2DzSlbb9gXI/JrqY/oK0YuvY8FbR
teFD5YW2sfkx0hXyDR/4yHHWSf3E0hWj61PPqgd8DR/HZvDBUzvXcHW7tzLF
Zw/0HnRL7PqDzxNXV7N+fB/MHC/t/aWL9v6pbg6x/iek8xN/m0S+se8Z1QU0
9O1cV0bXdlaxrmuj62vUh69OqkyIRdgBWPii94Ch70YXo6ePLsatYXmC9V9h
efFoGLuXdaA/sG6X6OoaGR0u7Rf5DPL/lTTyd5D3n+s94233SeUNHhBnsBd8
5gHpJs5hh5P73BDli41vq9zg5Q5lgfzB0auVPzr5je1BF7ZBfJzGtl+y7mWt
E1qusb5h0vFXdbSLNA6Krj9iR8dG1w7B1w2jG1uQzXrqD8wCyxfxu13V5SrK
5Abp+rX8EpPxI2yGGHiMfFIHNjmfeiDX+lRZENOxQWxgWvkmf51UWscp3219
Bgy/SrrBDmIYMf0y76H8mXxzz6jO9YrPbmv94yzjr0/5zCvR7PB2aYPeM6UZ
fxkqL2kv8LiPPMM7cfBCZUJeer6yOtD21lPmbylbbGJ7Zf4T+UQXmytzdDS/
tKC7eaPr5/CGfV6vrsnl0wao4/no2gbYBR5hM/h05u3U+7mygJfL/Z+8F7/C
JrGHn1rnKdYFvc/Ic9o2/GHz9CnutX3qel3dkduCn+faLn7K30HWvZQ6GKJM
0SNY+DfrJK48Jz1g2Yf+/t3o+ji5232W6VO8Gs1mjpIX4geY96JtJLYQK/CF
u7qi+NLmrlVm5AMX+yy2gS2dqNyv8p73osU44tq/rWc3dUDuQf5GjkqswK7A
RGyF+IH9gYtTqgt86GfSv5JyfFVeybXwqwNs4yTbQK7kAu90rj9a1ye2sZL1
oxdwdlh07Yd8B1s+RL2iX/xqDXmiDewUu51dHsmBD1MWB0gn7ayijDPXukW+
+ksL/G4Z3XwYu8dnnlQuK6vv+22L2E5MxFaJJ8QL+nb0FbAh4uws6pXYtY36
WEWayPe3Ul6byteLyo5cbyf5/FQZL2YZn13Ve8lXyYkeiq6/4p/Yx4ryf6l1
4As7Wzf0Y6fkvsRHbG+4clhC+VyrHBhPmDRaP5jyrcqPe65UbzxLvN7YOo8r
bWF7c0kD/vSM9YEZJ0szcthQXkZFty+8qnz3kfdXpDX5WcY2yKfWV4bIeDll
SxsvK/N15B9dZM58unWNVk7TqOObLF8u/cOk+T/KFRvGNlZTjtjMN/1EDgPk
BRldpJ6wPfrES8nH9vKGrYKt+Co2DA4Tn7DtveR9Mu9F75t5D7EyfWQ/dfC0
bU8pb5sqF7AJPznJdpEDvolPMJaCz4K1z9kOOXOOzyCPb8gXefh58rmWNIAV
R0Y3XuPfe0RPbHlbHhlLwd7AIvrj/4guRuEv4BaxZ35lhZ3RLwcfsLWdlDO6
mF49PGP96OkN5XR8dPF2Pmn4XNqyb0V8Jn4Qb8lpsRNwiDyCGI/P4U9PKutt
lBH2sqt0Y5eHyj9tkzu+q9zusTzS539WdAddYAw50nesDznjF9jZ1tJBnwSc
P0Gd7ayc4eVq9QUv8JrjXdQBjo2Rth3VKfLEzp+QjqeUe8aO65Tbj6UXvCL+
gUPrSNewzvW76PoYOf018g1v2C35Hz6PTeOv2C3Yhr3jny9L/1KFhzWUO239
SH1i/3vIx5HSgM8fpVyw+TPlax3lDiPkLowpkMviG1+O+0U3Xr9g+RRp455H
pY9n97Vd6vy5sqMtfHxDn5tHeUEb/n6pNG/vPfDyd/ldSp5ukfc/KFfuwbf+
qqx2UyfIcBPrR7b09b6hzMGCJ6UB2e9jXfw2rbrDH58uOp3MeqeUXvgaKs/Y
Br5Dn/Ez5XSechgob5lTZT+LXB5/wSbvVM7YxLPStKufxIn+0ph00PfDX0eq
T/jCD4hfxHhsCH8iXuNT6IK8EF/7bTQf3Nv78E1sD1zDdtD1dPJCLnmyvMDH
TfKyhXLF98BBcq431ClySwxJbPmWOrtVvdDmn5UNseR8aXxXHqB7fmWyhjq7
0XuROfaGHd4tP8hlZ/VyoDJBL8RBct7xymMleWHslpwZ7F3K9hgX+IF0PKOc
31Q2f5Au7PsS5baU/JC78kdeSe4+LLrxAJw+TZqpm5wIPyD3m1r6kSHjWafY
Nj6DDYBRdypn9Jm5Kc8TI8gP/qjcsfn9LC+kDrCHnKcYrTxGRddv6SeC1yO9
B//Hz75rmzWODJfu46UJDCePIV/EX8EfsHZTadtKmWAT5Df4BFjDuMsH0raU
+iCuY1+fqwewh74CeLe1crvXti5RX5f4G3EX3yGHf8o6yG/6KT+wgDxpMnUW
luH1596DLV/sswdaP3WCPefZFuOs20sDdM9s+5tKAzTjR5vJC3HjQ+9FZxfK
O7Tsokyw0R8qq/2kBxkSu063LbD3fmW+rHJBF9jVw+qIGPprdTe06HR6ZfiC
PI/2Qt7gILZxlPRPkJY+li/zmcxDbpaP9aQR+yBnIAfEVkdZDzZ8n7xi2/gn
/r2wtOGn+MKw6Poyn/jV6uomxy6R223RxSz0hh1+6r1bKkN8cxN/G2s7tI8N
kdeBr/g4OsRnsDXsbowXGD9EPcwtT0/K0yPSzh/4DOas7DNg0TTyAhbQ18U+
GbMebJ3YzEzSgj1NIb13d0n6cnwB/JpcXsjJwDl8DewDD95WR8j5fvWypzKa
oJxWVUfY8NLqiJhCnMAesFnwGPvZQdnhkzkWOrXy5fcF1Ql5G7gCPoHXJ9re
dMoDfwSjFlbu/7S+h5XRn5QXNDwm/bSN7dGXIHfMmPKkOhpke/jgvOqEcVTs
n9jwlnUw1ofP4i/Y+EPy8LDPwxt9h7/4Pf8TU4iP+CNxj1wLO8K/hkUX03aV
3kOUEeMMYAF+eoQywkZPkVewb3h0cZ7+zV/9xP6wNexjR2Xzou32ja5vQCd4
/ozyQ185drCJ8p5eHmexfLA8cM9l6pZnwZN7rPOX0kRb+OlW0rC+9EAbtnal
tB5ue/Bys/JZXdmfJ+/Y+mbKBBtbINrY9NzKaxnr+6263U2Z4xdXqAvqelw6
Fyq6+5b6QqfbSiu6xtbwGWxgBmnDNsB1Yjk2c5q6xJau7X71pY3dIB+PSSv0
YZNbK7M9/J48GRvGVvHXUM6vqoM/yfe13ou/LijvzBFjX9jJzNKNbZIb4ktX
Sdvf1UH2k+GVvBKfxa6x9fTl4yxDLziA74Pz2CqYQE4MNmFPxJrEEHJRYl9i
C/EVzAF/1pU3bJx+BBhF7sNYEPLD9rAr4tOQ6NoSvBJPsM87oo0T4mPYD/5M
P3Rn5QBO5ZjGx37HX/Zn11dfCyqXT20fu0bX5BvEvAetE93Rb73Me7azDXid
R3qhB/sgLyP3AH+HKU/amqDsyPXuU5bTq1dsNee4VvA7sIXYNyK6von8iZvk
ANhy5reXy9d2tk1bZyhn7HBH70cHOa/6hjQ/pEwXk78HpJm+IbEO+8TW8Rf6
R09aFz6H751gGXvEBjPvxN/WkE/qyH7296QH2RCP91e3S0rPMbbDJ3gO7jOe
ij3vqYxflFZ8Gdt+3t+RJf6ILRAz6N/s4+dA2xruPdjfIz6L3u+0TmzyN7YF
rStLA3nQq9KGP54izcfLBxe4musjwK5z5B0f2USZEN/mUFbQM04ZEmvGR5sb
7avMyY23UhdzFx3x/MhoY2r/Uafkfjeo63XV92e2iQ9n/wKMw2bwO7DrQ2WS
/WfiOXndiOhiBth9ovQdLI/I6FfR4gL+Mrsy+YZtwd/76htfxP/xkdfU0zzW
CVZgv8SLVeVzG9vAB/Fx8nt88251hs+C5WNti3z5Hvla0edyjG7qaGtUtlfv
2ERiCNiFbYMtzCdg21crZ3QPFs3VbeZLjEKe2Az+vKT0MQYwi9dfbOtFdce4
yyh1x98M6mxy9fuGOvmLulvJ77LP/oU8Xuj32PAG6hr5Ej+PiDbWs530ozPi
yoHq80Jld7d1wg92ik/ib0crA3Lxw6QFuQ9VlmFd+A52s4/6200ZYm+XSAtY
Br782TbetF0+sQv6J9gouAhunGF5U2X7qXQRr8iN8cuz5QN/Ig5i/4x9Ykfo
mxiNnonpfaTzKnWBv/xB2TM2iA/PqYyhlxxgJvnA7+dTjuQJc0cbZ55S+XAf
dpTrsJDtCpa3kEZ8hTyHnIQ+MeMP2EOOH6ODU72PdvZUPxtZ/83ewxjY49Fs
6ULrRDcb29b20eYeqe9P0na4fKypjOeVlyelh9/A59NtE/vO8WbunyvaGpL3
lSFye0HZIo/3lPmkRRc5XviM9V2s7pa33ZwvGC3dubYFG1g92hjCovKJzeDX
4Aw29APL2NgN0o7tDYlu3NnN58A9bJVYDTbxd6T0pm3fGc3mL7Bu6iIvw0fA
xw2kHX8kP8APyBGxc3yNmIXvPKVO+imvwdHN1fFZ7Iq8nEr5jv4p9reQMv7A
Mu3j5+ArNg1WYLejomEIbWMP9InBVLBjMWkDi4iB+AwYRY6HD5APr69M+Q5s
wMf+JP3Y1U22jw7pb86oXrAX8BAfflUez1CPjKmj6/Hysbg8gj+zqkdk2Te6
+TMYS1wFH7GNc6JhL7LO/GfDaOvZTlAG+ylD8BIfIAZ8ok7hb6zPY9f482Xq
gDgFHqyoDLBZ7P5oZXGq+rhB2T8lPeQxxK61vXe8deZcyTnR1nIQ77Fp7Jm4
8CPlQ0wAT15XD2PV36Tq9mF1SjnneWkz++nT2yY+tYTymkX5YNPz23aOBdFv
wf6/Iw1bKecc497FeokzxHx0jp/gC+R+4ANYlPkJfa/nlcuM6gb7ynnIY23v
Ee+Bzht9FlzZwjqJpR/YFrb0B2mA7m2kDT0uLM1/U7fZf7vbe8l5T5L3QdKM
TMCQAcpqXem/Xvk9V2RbZf6RupjSdl9XTrupO3i7Vp1iI+QgE6SbGEOFx/od
trGW9b+r/KEVW/q+8oUnch9ynaPlCdxZX95vll6emcmLesEZ7Abfvke5HGb7
6QvYBD4C/k9lm9gm/Tx84kH1gq+NUGb4IP253ykLcO3KaP2+N2yjb3THR6AF
nx6mfHI+HUwgZjzpBa6AVWAIMQQbws6I29hCrgslB8L/4IHcJcclPlH26JkY
QQxkvJ88m9iADZ6gftHF93x+LusF76f12RfV70vqZWp5zbHogfI7j7rJdQLX
Ros7nykL7GoFeWMNDOMt2DG+sL18M4b5I+u9zOf5o/8/mzJGjsR2cIG5DewL
Hz5GmiJanoSN7xNt3nx35U0+hD2Bv9g2OQJ+Oyq6Y+AnqSO+O1EeyOX3lscl
ouVg80kbdM2inPBV4g+2+JYypM+V47dv+D84hr3drz7B/xHKGYy9zjr3UR7E
RnwXvS+pbqGPuE9MGRbdOIb+iRvEI2LVVl7LSze/4deL+Qy62NR7kUHmfqsp
e/ry+1pGX5kfIu/EPe55tNR5oG0Riwfb/jBlRfvEoo/8LufNTrR8jTxip8fJ
Ozr+mTKhzqWU1cJFhsj5TWULXj1bZP6aupjaCx2hx+9EWzf1a3XK/NyX67In
6doDz+e6s1wnSR6yoPIij8QPsSVsG9/Bxogn+Nb2ygNMG2Td+CJ/5FN3yt+T
0jKf9Ga+RB2/936wjvxooHIgV91RvaALfGoX23xAOvCpN9QZNo9/sK4m1+QN
sT14z7XT3NPPcvo+GJ/r+RMrwPINpZE+KD58vfK5Uj6nVVfY61bKMW0UOo+W
H2QLntIfJp/JNTDI+BrbYh6Evs2kXg9LD/bwvHRii/RnsDVyM+yMXBa7IJ9c
N9oauV3UHfo9xWfg9wbpRifkytjL1eoLvyYOHy5fq8tP0niNzw/wvkei51wM
csS+11GO2OWp0fAVm2VsZGS0MZiL1eOl1kmcxeaJ/8REYhVYvpm08vyS8kU8
ynVoC3lvYteD3gPfxBz6VzMqK3I94uIYaf6WukEfYP4aymdxZY6ciOM5lv1T
6SbfBnvBXPyXmEssJcbic9g/dgu2MQ6GDZzg70coZ+xqJ9tZwLbBxcQu7H+8
fJ2r/LOcGLWLbfDsvtK9k/VMYVsXyHeu43hR2uZQj9CUfRZ4oW9yuTxmXnmP
PFaZfEdZzarsRtlmrqfEbt6ybezgTXWRY07oaCl5hy9ymCHqFIy40HuI6cQ4
bOBHtoW+yY/oq2Ez4AaxAVvKsfr9pRc8wXb2su2plT12mWtHXpFmcC5xI9eP
DFCnS8gTvoStrq5M0cuh6gtfwKfIbb6rfDZWdrlO9ffytY9yhl90NFzZ5bg3
9GG32Nzb0kwsAhPotyVWTK5O/96t6ksMS7+lfTBnrmg2sKt8Hy0N+Al2gm0Q
e8CMg5XhcHnnt7/6DHMJtylfdAAmE9uxJfLd/tH6CH0sf6C8HpV38vPb5R1e
6WPnOOTx3k/d+6oHMAFbHxFdG9lFOpEbuIxdML5AbJhd2d2nbMiVvyHd0PJ9
/yffBr/B6z/5fCgj8gDGevBTcjZ8DRtbTxngY2Ad8Qf8B4/+7X3IFntC/+/b
PryPlHfiFZgwQX1erxwWUibIgTK5Cf6/tPxwYe+5RhobA/ffVk5gJraLnZB3
vBTNxp6XhlujjTfeK8/LSxt9e/zlccv01bAfYsOGfvLbWtHGF5FDjv+DS4ta
xx7KKcvjyj25ZmkZy49bz9PWu1G0MSl89VHL5J/3SzP3jJAXfO8v8riP+oN3
7Hc/ZYJN/khZ5ZqO3Ac6a7S5vIiW936iLubtqud/r4ecR93Rh1tPnWI/h6tr
vscXsYHTffbfyhv7vsr7t1VG+BE2vKmy4vm55WUfacIf8QFs9Vh5w4axF+zp
rGhra9AteI4vzy6fK0dba0d8OUW9/UY5/tLPF5QtOIQPki+vqhyWTHom6eYe
F8l7zitz//HqK33/37ZJ/YkV5Hb4PxiyhXIAW+aU/uHRxb33pHMuacuxWeof
Ft38kjhwgjqizfO8B/1foX5G2vb28nyvfGCLTypvckRiDFi0pTJDN4Oj2x9D
Tosq49wbNUFdUM4+KjGCOHKcbd8fbZ/rhZb/oDyOUzZL+Cw6n9T6t1Kv0ACm
ETePUp7EqMOlC76xf+wVvWNHfaOLK3tbZ65fxdZzDTD5D7Ge2IL/EUewv1uU
KTq9TDmso07wYca+0PsX3kMd2MNPpI0yvI4t8iHPwJ7OVJ7j1V+ugc6xaWS4
uPx+Ih+Xy2fOY+Qc1tXWjx5zfJjYf6K8IattvA/e07fRM7EYG99LWhhnwBZu
LOUVyj0XlWc3sU5sOPtNlHNtUuWdmPmSNINvj8gLfni7PH5b+injr8cpE/T2
y2j7W9Mn4X3JaGMUcypz+jXTW15MftERMWi+aOupvq1ORygXdL2+PD6sHkZF
V6+HSwc2Q98a+8CWyF/wmcWUJeNx2B45G348r9+Tg2Crw6I795P7Cnl+Oe/F
dydVjtSJL+DDv5L/C6P5DjaAvvGpR+Xndes8XxmuJ28ZW9+I1i+YVfkTn9dW
DjOqY3wf3MD2wATyO+wBrFjHNm+OhmlXKO87pfckZQg/35dO+F5aHR6iLh5W
x8vJE7g9wPux39mjrdlHh/SViJW5b+wxP8Hal/yeHPtd6XlbOcArOscG+son
trFooYm8Zstoa5+wfbCAPJr4hZ1vpczIzXf02XXUyY7RxuHPkBbiFLZ9ufoE
30JZjPb3odI/hf8n7dRD/FlFnRAryKmxU3Kk02yTnA/7xB4YB3vQMhg9ynuO
VRbUQwym7429/El+8HHGD/Epcoe0E3Q4nbJiXHFJywuqV8rY2s5Fngf5LHHy
yGhrNXPeFEylr0huAwZio7+WX3ID8hHsA9shluwTbZ4Xuc4ezZ6o407rfsYy
8jwn2hqxxGToHGidOVeSbW0rDeBS7mn7kfr8QBrHygt5Y87HLRstb4f3C4t8
jrK8gDrInHbDaPN3y0XPdeboghyjnzrKvYToDiwfrE7vk+YHo+3/vs7vyD1O
U8ZgLTbzgm1hS/gr/YXplT8Xtge+kQ+nTTI+EMoZXMn9D9gntk3+MVoZgRPk
lfgCPv2sOnna//EdcnJ8F5/KPtNodQG24IOMJ92grLaVni+UAfaJL9NnfbLQ
wRhx7pdkDOQxywNsD3uhb/43dcoYI9iCvveXFuR/nNd5yg+7wN5yncDSyu83
1g0dua+NfJlxkOmk5dpo60jvjDZWOVKZvqEewXr0/7a0zxkt91412vkz2ykT
7PeEaDaT+1yIY4wn5F5LaMq1i9+RH+rAvsEwYglxIvcp1JiOnBg/wQbuiDaf
cqftHqbulrN8jc8OtR7kTQ7GeNealrGV3b1nX3870u+ukifK9yh/9IKOr1Te
L0vL76Od05K8/1454y9gNXZEbCWfmkP6yfeX8XvyC/qJY5XbJsoO/RHTsU/8
a23vHa6cct8DsRF7HhZdG8I2fxUtx9gj2hg7PsZYxOLe//doOVbucaB/9nC5
Z6TPgg9/sE6wa5Bt7a/8r1ae+0jbnuoNmn8m7xm/Pol2tslThfeblckq0XKG
leXzY8s7KttVrQuZfyfamTmMSa6kjnb2PnRHTFlPnZKfbCtfYDO6P1VawOHd
pQ+5YTP4Jzn87NJKPfyNiC4+LafciF+fW9+j0j9MeqHv0OiZ6/ZTlwco7618
FhzHRy6LhunIjniHT4GNC8o7mDSv7cLDl3NSk3RxrE+0cxjAcnyZPGd0tPHq
l+WTHLFiAjJLrKAMhoAJ2Da2Sb8PbDldPVAnvnFBtBhLW+Q6+6kXMIg4NDja
mhr8gPiDbRFzplIv8yrvEdJJvpjrfteRVsq5nnKWaGc2zWV9nynfu5RJ7lPN
tWd9lAljJgP87QfSd6/6yP13YBV5J3a9l7pcUT6PkW7az/UPyII4SP6Cf+VZ
G+RER6hbdLy/tBP3wHYwGMyhf0RefJE0rGtdjOWBl/SVN7NuYhF9RXwtx2vw
JeLdQfKC3dNff0y6yPnJ/8ibGGMdFV07Rwa5Hu4Lv/+nsn5aPZKzEOuxd3Dg
Wn+bTxkQM4mF5Be5t3xHfztHHqELnCAm0j99SV0wRvGJ8uJZdE+Ogw2B7ROs
58NofcxTvIc6r/JZMPBO64T2nBvL+VVoGKYeoG2o8l/IZ1e1nR3VT+65HiPv
YPMjRSbXKSv63Wcpw/Vtg++Jg79Q5puoL3TxM/+/V/o3VnfIdgt1yjNgGbr+
m3T9Ul6wj3XV7/3qAJmMi3aWwELyQJz/sTwjS/LwtEnqxb6wTfIzbJicA5/H
trFTMAebx/7BxdxnmD4CTbvIy2HS8Yq8HKwubrDe9EH8B9/sb72fWXeuQYFv
8qQ3rYPfnlU/iQlgHv3GEdHFGLAu17wh+1xPiIzAnDn9HSwCU4k12NcQ5Z/r
JTdT1mA741knKfNcR3qsckFuxF58/lafOUt9DJX/p6UfvyI3BfOJs2NsGxwe
b5lYN8F7vmH5YOkep2wXViZHK0ds8QTlQ54CLuceFPBkU/+/SHnOLl3k9KE8
wJbvqqONbRd611cXjMmDldvK2+rKmFi3srJ/VnrAiFf9nmeIN4Oj22+jvEG0
NdH4ArktPgbGXqocjvK77eSBHIt+AP3NEdH6oSOkAbsFF8Ft8pk/q8dct/ii
ZWwK3yNnJ4e93P+JVfjYZfJ7mPeuYFuzy9OGtksugC2Qn6HvdbwPm8I+Rnk/
/vOG5SnklXvAqeE+S353TbS9Obt4PznaTNH2G9wtbeQJR0szsWizaHP0i0j3
L9RP5tuvKxNs+V5lBfZeqQzx4zMt/9Q2kPm21nut/O2ijrC3n8vPTbaJzF7x
/h1sBzqxgb3lF9u4IZq/3uIz0Erfa7poZ+qsKL9PW98E9QVmfVd98Xwoh1fV
E3k9Y6rY/O7KG18AzzeWZ+QAZuE7YC329qbfoxN8bX95naBOMwb9Nlo/cRbL
uVYWOeHjc0srtG8TDRP2lRawYnB04/VfvH/daGvUyQ9P8nvsFlzB7+kzg1Hk
uf9UJ6coa/DmMtuBxmHRjVfnKheezbj9iHrLsat/SRt5MH6GbTynbrAzYjTj
/nmuA/JZxO+QCfbz5frzSbr8faF8LlbP78rzNMoEuhj7utk2Z412zgr9hSHR
tdm55PsdZUGu+r5l5PSh9/xJmsBJcrscj35JWiivYpn7J4u2/mVK6xzk/+AY
NrOqMgLrwEryE3ImcAo/2FYZI9/9/P1BPxNziD9nWCd5AjGI2Ikvkzseq15+
pVzQBXjN2CH5OHbwQLR9WszH5tg7OsbniM3oHp/4ofRxL7kXeHW1fJNj49vE
HnLvSW0fvonP5CLg2duWwY0bvWe18uyK0fY8bGi7tHWHsoQGYso/pe1kL2j+
tXKDPnKBleUR25tN3vHZT5QJcv6PssJX7lSG+PKflS1Yd7YyB2eOUwdHKh90
RN/kIGWDrR+sTuH1CHVN/pzndcHfhfKD7rBTbAZ/oE+ELeGT2Bw2hn3Sb+nj
d2BenrUAfdjqH6QdGwYXsa2Fo42vDokuJlFHrudf2LbxR+ImNoINgre55hk+
sWX6wsRKhPs9dTDBOv8uXbnGFL/9uTzD04zyhe/3jYYJ08jDTcr/FvnC3shP
sX0wFRvBHpeJllPjQ7m+A/nOoW7/ar3oALz6PNoZcvB0VrRzPi+NNgZInrqV
9JPPMydxj7T2kUd8jphJv3m0bZPHYE9rRpvHP8n7c40l9X2mzMBS8BxbIscc
pcxph/7kT9UDWE0Oi+3n3mhoxleIA+DXDNaNP+NvI+VxI3kaKz2Uwc/co/y5
vPMs9oLvEn+mjbYnaDb5xsYHKN/91AGyxxbW8sI+yf02UwfYzzbSdbA6yz3f
6Jl4SP6Er+I3YA5+DQ5dLY/vyefcPvvlvono2g263FAefhPtrDEwBd0Pi55z
FjmekesKc8yHMfCcJ5pfuYAJG6gnfH0fy+hntPfMbnlCtDj6sXVPZ1vg8avS
gMxvlDbs+rfSjB/mPgd8dg15HCY91IsNTKFM9lWmyAr8yvEmaLxd2XL/X6Kt
Qx6uLsCy09XRNdKN7rCNE9UpcgJD0DXxGf/DBo5UR9D8jvwtIM+PqIcLoo1z
XyuvXygvcGYjf4dXbBU8By9zXP1keRzq83v4Oxjye/mkvVybmuv3cl9Ojg+D
8fjasj6f4zO/lh7Gw7BvfHZwdHEeusBDsBAfB7PxDXRO3vRANEzAZ/8WbU4S
3vEbYsdZ0fYCHxttrVL6ADrGP8A0bG5X6QX76ZPRH3rd+9a2rdPVE/ZDvsMY
4R+VEc+eqUywfeLSSGk7UvkcrkyJRfgycfGjaHvm0U/ufcpzD1JW1I+tfqhs
wM2X1SG2cV+0M0nB2m2U56HKETmgd3IwbIa56yujnV0Eb+AtOdNx6voi+X9e
meDfb0hP7u8ZFl2bwf/6Rrffhg6wKzByTmUKn4xRfNNncl8ffoPdYBf4EeMv
9IPeUy6r2ha8EYv3kw5s6q/qFTkR2/CrmWzrDPlCh9jzrMroCPlENjm/n/3o
GZQ34xTgLP2C0eoMHGXOmRhDjB6ijIgR2NAtyhYfJdfOfVC512tn7/mj8uJZ
4s4/rZO+0kPR1i7dKg30MYZL223KDpovlAZ4wRa+I4/obgF5h5fplcnhymWQ
5ReV4ZG2u4rPjlDmyO8adYE9Xa6OsO+L1F3mvej0Y79H1y95PzYAhmF7ff28
Ux64B91jS9gOdnWAtIV1IU/sB5vEN5aTL54lb51Hue2u/k5V1tj8ftIA7eAJ
+QP6Jza+HO28GtrGp1azLeghFhGzcv5rvWjnYG+lHMFlcBFfnl+9jVJ26fvE
gNOVGTzjp9cqR3D0Ymk9P9oYO7aBz4FvJyuTv6inXX0GmyQ2gkv3yuvy8kqc
WUg5IA9yVTAIe8E+ck3OzMrwDnkBW+mfYB+DoxtzVo52vgqxanVlhZ+uoUxO
9tncE/19ZQh/2VfE53JsYiv1leudcjz3RPk+Rv1C+0+VDXUxN0UMICclV0fv
YD/+ho/8XX6Q04PSSbvMZWBnv7b9MdHma3ZVH9hsjsPjK+RhV6pX/ojjS0Vb
Tw3v+BXxemNlCSYxpjC7OvuF8v6D8p9R+sH8XD8yzPrBCuxnGmk8QNk9JI3I
+Xq/w14vUBcDlT92mfMuYPJytkMOx9gz2Ild7eAnud8S3ku+j+/T9xlv+Rj5
XUK5/dxnwcPfWid923Nsi2dPl4b3fQbasKc9pBnf30JekPNg+SFOLyTv6Pxb
yuRkn5vK8jvK8DTbQ7bnKiNkTuy/S13cHG3v3khlh+7w1Wut/1NpQtfYDzF9
PvV1h3LHJskrxvgJHuXaR8aQsKHL5WkTeZnRttAjfeIpo60zw4bpT5AzYdtX
KR/kfbyffId/oGN85CfqiPrW87ucP4XuL9QTeVaOsc8QLTfuG+2dEeSVbylr
/Po5v0/fp3/+ffmnLXJbsOIgab5Q3jdXV+AvuJDr2vCF/fyN58Gow6O954Hx
A/wT/GBsYXt/X8j70TNYT79meHTjCjThi9gBfevb1Rm5/UPKmvFzbAJ95z6H
ebwn97jOrIzA2LmVFXTk3gdyt2XVFXaxhrK6S77wXeIvtn9DtLmQU5RJ5pXY
zXHyh0/jb9jE9cqBv7f9f4A03SW/6BN7fT5afxQ/BuPpS2J//1G3tIUdkGuB
2eOVHfw/oUyJlRdE2/NIrCFm5LwYNjuZbW/p97S1g/e/pQ6p51P5nUYZgW/4
/hve/8doZxsSh0ZEO9vnr8r2R9JDnJ1CmX9u+XT5nlF67omWy98hbdDwqPL6
i3RQxmf6eU/OyU0rjztaJ7LPPTn4876W8Y3dpA0b+pk0g53fkxcwYxV5vEzZ
5vrB2ZQJNPdRVpQ/UoYXKSNou8I2kDl480iR+X3ejwwyJx8rf+gUO4EmdI19
/1PekC3x7in5RfeHKKsXrQ+fezfaGdNc2B79jFyr8CdlOKP1wx92857y3FHZ
I59T5IX8FV8gbhP/0D85D+MR2E6OwaMn8AZd4muLW1eum3052n4xdJN7XT+X
lj5eKbv0/SmlF0wgHnwr2vllM8lvrj0GW85RlmDObtb1K8ufyuM+PgM2gYX0
TfaS/iWU8aLSSnwgv8KOzpcWfDLPnCFmgRnoGPzO8yLw82ds/01/m1pZj422
dhSfzv0EyOo7toEMc+0A9x0pbeAKdg4WM8b3D3mgL4F9DIlujKJt4jexh7iG
HeScInLCP8HwC6JhxZXK/RDlTfwC9x+Uh1PUzwzKGf3TJztXGmnnd8ol14LB
J7gAThK3Zo82nog8yDVWVb7wta4yAxs2Vi7YzubWm+dZgPXYzlbW9Ynf76mM
iFkvKQf6z8j1vmjvfblVeUI7faCMSei1j3K40nun8P5Xo+3Ppm7iGTY1Wt19
WPSIH/T1HvKHlaONVX3fOl+VZr7HN7aOtp7sJ9IG7z+QDvKCteSFXHh56cA/
F5R3YnmuX8+x2Znl5QtlCNaNkVb6q68rc/BmlLQ9IZ+5f/cxdYecH4l2Xuuj
6ho/oL+ba7iRO7ZBfg5GYTO3qxts6Rz5zPw/432uB8ImycnBjjmlH9msqUyI
cdg2uIttYPPgxc7KGAwnXj8pDSfYdq6tx16/q4wmWPdt6g47xVby7IXHlT/0
PR9t3+Ur0c43Tt+fRJpui/b+mr+ogxfkdWllC7aAQfjXsOj6InoFi8Dq66Kd
gb2etMMf2ET++L1oZ1EsLx/kmNAPzpxru8gLLM8c6hb1/bT6Cvl4OtpcU/L4
qrxPb53jo53jRHvMG05teXB0bW+cegXXsAtyy5WtH5rA2xHqaUtlgp6wo32V
36Hel2uU4elFdQddD6k77OZG5Y1cL1EPxO0z5QtaGX99U9qQ0cd+T3kK5cA9
xJShPgvub2SdjKnSX8LXlpUe9EbMmCfaOyXmU17QvqBtoCf6cvgfmEb8/MTv
sWvwY31/p56MAfgc8Rm/uU9ZY0N32SbYTN6Eb4ERd6tXMOLf6opYjv4/lfZP
1PGXcWqS7rMPKI++3vOusuTZMep1Letbz7a+UDa5x3cDaXtDOdyuvleVlweV
3xfRcpxcrzCnMrlNXlaP1gdAhndGm9/N+Zx5lM9b0WzjVdvATl+SvvHRztj7
wvL98pg5CTEUe885tXf9/kTlcmG089uW9rfppO+SaPPUN0sv9GDHK0Tb47Vu
tDl5+sjnS8tOygCaDlBmxEHGs/Ed4vbZ0ra1deFr9F3+ZnnhIqM5oq1pgcZH
5W0K204fr76P7yUm0DcBK4gbzP3gr3P5+/HyxLgqmEMuBY6DReTN5IhgVMbZ
XBOPrvFt8BhbAA82i7Z3jTz0fflEr+ROl0v/C8oVWT8fLYceVXh52Wfg8W15
p2/wUTQMnzTaGR0zWEaX+Di+wbwROeto5Qz9YAcxCJ/Fz8CEIdHNOZH9ntHO
5yPf/EG098iBfeS4jDWAj+QGV0k79jVCuUI7vkE+Ssyl7/eKfIJX2Am6/Vi6
R6inCdF8n+ew4TnVB74/QPqeka+fRDsP7vvKaFV5Q0bYE7kUdo6v08cnvuPL
jEd+4vf4Gf41WLkh91Wivd9vmWjj5sgABZG34LtPSRv6wPexTXSM7RCXcr84
GEKe8pk65J5tpC3HN6l3/mhxaDGfpbxCtP2Kq9vWZ9L8lLoYbB1vKQdoxtaW
k5cnor0LKOUJ7/erN2QCFs2krO6Ldn4osWlSZfu48kXmz9v2T9Tt++qI70ar
O2T+ju1PiDbvOcb70ffDPoNtDLd+bGZYdG0DWwI/p5VW/IS+DH/4Hj6ATYIT
S6onbBObx4Yzt8C2mSPYOlp/nX7UHdIHpmJf5I0nRXvHz/Bo65wyvyLO32qZ
mPkv6Uau+M9YZQmt2BT9vuei+fhz0XyfNnMNB89eLj/QB4YwhoR/gi2M/Vwk
37n/DiyiX0u/EIwiH95Q3VEfNsCYCJiHjYHL5I+MVeUYMG2Tc+JPYDO4PGe0
8dapo53FOqn6S14+ioZXn0fDqz7R8Cr3EtL+vNH61IvaHjLE77AV7CzXkzBO
g32NiG4OtUu0tYkHKoecL2Xs7yF/w9ZfVn/TS9P1yhtdknPMIH3ECPIVMAmb
vkpe35Lmt/0tx0jhH/1n/2mC/M/m7zzXN7o2C7YQQy9QTvTRTrFd9IKdgQOs
mTs32p7A65Tlasrhk2h7ncZ4/3O2lXNML1v/VPJBu9gWGDy/vDyjXj/0+VmU
Dc/OJC1vW87x4lmUYY7XTe99/bwnY3bOiy0VbR/g8taBzFeUBuLICtL2jvRD
Mz6/ZDT/WNhPMK6/9YE/c0dbjzaLsnpcus6N9r7DXOM/qTLPsedcyzM+2p7q
ceruC/+f4PWpuoYHMLeP8pxEXuhr49vYzG+kYX3L2AM2NiravAx+tZj3gTsr
+z905Pg1eSM5FbZN/4icanh0cxR8d0R0c5ZjlQM4cY6yJLZeIT/YfmIUfcb7
pXlLZTTe9l9WX32jq/vXovWPqH9yy/fKN/eACZP5LDxNE23PIjq5K9oewL9I
N/pnLgQfpo9L/ghG5Z5S/Jw5t038nzw191hcp5yOtg7iL1i+re3B6/LSTf9p
dmmC5qmire2czHuyPzhztNxybnUMXi2grNDb0tHwipiCfYBVGykL9E3/8Ek/
d4l21hPzT+fLI7Qf6P+MKzGX+Ldo79Mkb8KOwOXc64K8E3eJx9gjsQqfyf7V
bywTr8AD/AT9P6qO4flDvwOXD7E8i/VxzRNt/G6Aes21N4/KP9jy72jjC0/Y
LrS8Fu2d5Z9Gez/jZ8qXPmbu7Qcf+CMvWtR68X3wCjuhr4O/P2uZePGR98xp
3W9EW/c+2jLjfx94T65nmNfyeOs8VZoW9FnKi3kvdC9pW+D90tIwNto7ukdL
L9fr0h/SuaA8joouNn7m8/NYN+XZlBW4NlO08xCni57vLEfm2NYU0fxzsmjv
w8q5hQmWD7GMr34oP7kHArlh/9gGeTWxGJvBDudQf0OlmbZPka9JpBW/xSYZ
wyHnwFbBXrBqU5/Fb7FtsPFX1s24D3aPHQ+Jbn8DH2Ec9XL5AxtulV766g/K
B/ku2A0RRyn3HFP8VHmsLL+vq2dox8excfKVe/091+9Txl4vVR5gGhgyq7rB
P5kDJu/O9QncCxZtpMzAKPIt+o5b+Mn4O/PNP5ZP/gcDyDlzDR24wXwRedey
1ruVcoY27GtBaZ7F8tPyhA9mHwK7w8bBsBWizUEMVlbY+AbR5liZI8q51+2s
h3HC3dUhOEw/FjzfUd2crS6hd19pBqPIy+gnkAPjMzlnjW1gu/d0SfjSnkcq
99HKdmF1/Jptgy2Ja+h7vPRzoaft/R9dPuk9YPWhlslD3rY8f3l2gWhnGjK+
lD6yuG3w25KlzaX87uNoZyd+EO197O/5bMa9RfwdOS4gTdgfcj/Ict/oxmfu
wS6v9lniC/n6u5ZzPQP30E/5yGdzL/IA7xlnW7+1vLB1cs+itvWhOoGGMZZn
tq1FpRn/zzlW9DBIHsGHftHeqTyvMqE8h7ICg2ZRZjw7o7Klzum9H1lNq+zh
ZWr5Hm85dTdNtPckTaeuueebfjfatrCNJ9Q1NvM35Qz/p8oXNob9Li9fjHGt
aV20sbGyIN/5mf8/bH3YNvlSjkk9Zr34wvBo88PkIzerT+qgj4G9MGdO/MXX
LlEHEMEYCbiNb/5aPXzgs/NZD/k7WIOPEx/S96fzezABf8dOco0ocs6zFJD9
UeoQfwNzVpQfsGgDeQCj8HviEmNUj8srMgfPmYfbW1ns4f/kM0Oim6OQQ2+q
LA633huVCWNM9GGxKeIC2M/4PWODb0oz93+kPngGO8Gmfqqs4HUHv19KvSAr
8j4w6BXpOUq5g2cnyw88nKvO9pbG3b33ap8j5tyifMmL71GOI6JrB9gMWEVf
pI+yfFkdvOO1jbqFD/Kz8dH6CV/4HM9PiDaOSsUzR+vz5JgwZWzgAe/Hx9+1
vGCpE5zB17Eh7HyMOseWwVPsD1y6XHktJr/v+yz2l3tss08HhtAfeSvamnzK
+P6a3tPXenh2vmj7qCiHdHDPDcqkn+WPrDOkLc9h+Fi+1veehW3rA8tzRDsH
fAZlkTS/I83w8qY6oZ43bBfeM/Y/G21P1pho+eE4+ZpF2VL/t/wEe2dUF59Y
3kmZz6ju0Euur0N31IOuc63SJ8oBH8Y2XlUO2MxD6g66sUfyrNw/M7hb3ZfY
xVgNNjksuvkF8mKMb+eiC3wB2wZzcp0kfkNfFl8gL7rONhnzwS/xHcbjXoh2
Fu5H0g59U8vTb6Wf73PshPvAgpWVPdjFPA4+Dnal7/Ncri/Hxxmj+KO6pJ7f
qeM5rG9BZbq3dY+Krh+TX+Gnm0kbeA3u0H+6ze/g9ZJo73ICmw73N2LLadK1
pzSQy5DT3OR326pr+BgS3Rx8fr/HRmaOtkaJMmOD2CEYvK+yyv1sOX5DG/jH
Gurk+Wjv2QRrtpDmXNObZ7Xsqcz4blh050mp42Tlin1fqC6nll9kzx85PBhF
fHhc2vE34jU2Dba+I42MFaL7SdXjJ9H6Q1zTK8PP5XmdaHsS54h2luWa0cZ8
Tvd+7Jq4mP0p8mYwamF1mfswmeP8yPJO0Xwf28FX8feZoo2V5T6kQeoU+8en
Mp/sH+0MAejJPZw5xvK+5UHW309eP/BZyh9a5yDLg3yWcmLOB9IzqXLNtZHv
Wz5NmheQ5tHWM5M6GSCPibE7WZ5Pmbzl77d4/5zKEF5mV7a0NVu088hmVRef
WJ4jWh8A3U3wuy/XpE/SxcPx0fYpvamO0N1F0eIFNoP9LCMP4Ahjj9gK46bf
VT6MM9HvyzVJ28kb+Qc+jQ2TT+Hr2Da4RwzG5sEjYjC+QP8Km8dH6F/g7/jO
xtKEXDMHgn4wZl55WlE6sUd8lNwDnwXHNo22V4X+1XN+pu/TZxoSXUygPvps
f1SWG6hTZLZ8tH3N/I+/rqrswJvcc7epdX8u7ckHeMm8BeNg4Da+/Gfpol+c
+wixR8bxsj84LLr2QNsHqQd0A/6Piu6YBDEDG5nH+qBpTmWc/aFci0z8Id/D
XlaSjreUGbp4XLqZM7jZtmifvGoH6T9A/hk/JP8Ca3NeAZmTw2Jz50ljH3ml
n8Af+A0eTasO8KtFfA7aiV2vqNvh3pu5GP47jb9lHonN5PjV/IV/7GG8cpnH
8vz+luNa3/d+bHynaH0rypmrZF9sIf//wHLOe/Adse39aOOE71nnhGjnN+xk
mXb7W841Y9yP3yxhPZSPs37uwU/HlvIH1nOc5YHRxgIGRTsDaoFoZ4wmX0nn
hGh0QnPSCS8VV9+RngUt9412ltK8ha95Sv1zRZuHnTPaGW1zRhtbnCvaGe5z
q5uML/dIU+3LQ9sL0fLh4dZDP2Fr213J38i7wEVsjD4jYy/8Yb/YNPbEWBPj
Mtgq4z74VI59MY6Dbe9uO9g8/SGwD1/AV7FdfASsx17xnUOkD58Cu8HhT5TL
EtHeJbF2tP1mYA42SH8o90rj72DoKOtM399COm+RVjD2UmW3g3T1VfZgAf6+
jHzQhyIW/cy2cn3r9sp/I+WLPrZUVuAl41vrys+w6OZwf7Wt05RPzg/y263S
Sz6WfWvKxBNyLfKitMezbQ+a6SfnnkH6omAo+AYWjY129niuj4Ye5lk3Uxd/
kjew6FhlDv7S/wVjwWHiBniVfTr8h77B9NIyukvCl/zQJvZPfwYsoJ9wT7Rx
7Ue8dyflRI5AbgfOTa3u3ol2pu4Y64B+Yje28QvrRnaM52Ir8/jbOOWyouX+
tpdjQblnLf39I2X6zWh5C3V+EO3MssSELdRBYtT7yoN237P8lmXaOsMyNAz1
fsrnlvKb0fDqeMsDLI+1nDg20GfHRht3yvIZhba3op13MVuhM7FrgLwklk5l
uZ+8Zx74Tct9o+HtvH4/Jtr57B8o/xWVZ9XFvNHO4uir7j63rRelaaC65iJX
GBVtrgTbABuwHWwGn1892pwoORX2gQ2SR2B72OkQq8SW8SXs6V5llmND9CGx
7QttB5vHB26N9k6Z3Hu2tW3gO7+M9m7R6+UFPZDT5Pwyfa4fyhc5IpiJD4Df
B0fbK5373vgufZ+87mTp+IH34Fv4NRgBfmCPYNewaO+Qgq411C9YtJltglE7
RnvvHuNbxH36yU+oP/Is8uMnlRH13GFb0Ea/nLECcjxyvRHRxY0/KKODlONr
0gy92MKSPvtFtHcW535wsGhctHfFZk4FTY9aD/jDmCXxh3gBVoK9+MRe0o2N
Eqv2Ua6LR9vrB5/4Ev4P1pAzfjlOLh/QxfzAdV2yvrT7O70X2yJ+TCV/5Frk
UYwV5PplYiZYOJN85bgr3+F3s6tLZDyX8oMu/OKH0car57OMLe0WbSxoZsvY
bM5LDbKcecsMpUx8S0w4LRomQFviwD+ilf9jmbY+LeU3S/mjUh4XDaP69VIe
Fw2vPirlxLqBtpXlpGFi2jInHBCtn9hfHjP3m6GUJ4/Wb5082jhbzk/3VbYf
TCTzvuriE5+dVfr6q7vPpCf3wOcatlxjhg3kfO410d5NeJD340vYEriwQbR8
m1wJ2yMe0QcM6aSfgP0RB8lpZrENbB/bZpwO/8Lmsc8R0fWF3IvOfRdYF76D
/4J/j9omGJtnWKytvshNfiZf+H+e/wOmHKucsF/8Bx8H59L3yQ0ulA6wlhh1
ebQzvNAd2MX4D3Uzd8ecGb67lr+BRVsoDzCKOIH/g2sXSzPP4JuMH4Cn9H1f
t/3VlQW8kw/SX7pCWV7rc9B8VbQ9U/g8/dzsr41Tb5lHI+tH1Bs0k6/k2uc3
1MNGfjdC+qmDPtw2ym9/6SEubSo9k8sz+TE5NbHyCPnif/ryYA5/ZysTvgev
yJHmlPZppBeczjkgcjp8AiwHRxmDZS0eOEqcxFewpVnkETuYXZ2Otm7ox4/I
b3MdUu53St8hTuTY9QGW8Zcbo40LbRutn7VBKf84GiYcXcqPRsOE8aU8WTRM
mCcaFo0s5bHRE7sSiy4s5cl6KV9Y2qoYNbaUR5Zy0lBpGyDNWX40Wo53dCn/
OFpfdYNS3tb651eGY6OtM0w5I/OMEegic1p0NE56zpOOBaOtJ1lEXee8LTaQ
a06wDfCKue3TLdMH3D7a+V8rWcb/sA/qAa/6WD/Yw1+u7+wjbfjibNKf+Qv8
joh2Bhq+iI/k+Dr0Mw/+Te8DT+mPPWvd6yknxulyzgv/PVQe17LdcfJHXoeP
g39X2h64yZhazlte4YUOzvV5MPsI611ZfsGcdZUVsthSOvmOuEKORfvkcYz7
/VC+iSc72R480jelP/q0be2mHv4qT7f7f9/o4tjz8n6X8oPOh+T928oG3pdX
ph/bPjIgFwZPsRdi64+UBfFhSHRjY2IUOADmkveiZ/BvX8u5boAy+HW09/NH
Ho2ep1J/8At2EAPAGeznesu0+fdoedkD3o/NP2492MJz1j9eHuaQX3CXOPSf
aOsBsFtywHnVFTKqc23zqy98Ct/5KNo4ds6VDlBuWX4wGi7dU8rPlXKuIx2o
jrI8cylfFg0Tli3lQ0p532hYtFApb9pLeaFS/76lfEgpLzsRDZW2bPf5Uv68
lJ+Lhlf3lPKD0fqqHxd57h5tnK3KdvP46pzmQHUEPiyg7nI9BjrN+VnoybUl
fOb4QY61EMtvjHam8PGWwashlrH1lXwen5nbtrDvXNs2VNqgk1xvcnm5RFmB
leQG+AL5GPkOPpL9fHwHnGXMDJ8aJS/4GjGDHAj7ZdxsB2XAJ7iDz9J3O1t+
d/O+95XdTeqIGAMO/FPaGVsBK4gNjGOBIYxNkWthA2AXWAXmEGMGRxeLyO/o
p4BRYPbB/sY8w/fk61R1Co3XSNch8vaC+uJ38AcMzbFk/Jhcg/7xK9b5SOH3
RfnNeWv0Q85GrCDmgqHgAPN29Fv7R8Mo8J8xOzBqqWjvHV4m2r4qytPI05Lq
FBoWU6fIBr8hB2KsZFC09x5Qx4bKvm90c7Sc2yDfBXeJ+eAyPjC3ssh11NjU
s9HOl3jJMvbxus9Sx1vWic3l2oClvK9vtHWq4BV2kfNu2FH61y+ijQutVMrv
lfKEaBj1RTTff6eUh5fyNKWcfcZc25DlPqU8VSlPXsqP9FKe/GueHVjqHDhR
W5WGaaLh0vBSfqeUvyj3576Y/sokyymriWWIbBPzkfmH0oMuwIRB6ijnNYZa
XkSdgiGL21aOq/Mw5dwHwgVeveD9jOfebj3E5rNsK/sH0MZ404bWC73YKvGO
vAEbxq6OU7bYG2tF+cMOr1Vu5Dr3yzO4BU6tq27AlL3lH3vD18hZB0fXB/l+
SHR9k/4FsR+fHRZdvMozHK6Sj+W9D9+n35uYgEyJ92DFr70fDCG/RM+HKAOw
D5xgDB78AovIuWaQJsZ2TlKO6IB5PvpeRyp/7mU+mDwHDAAf6ecwbzqbPF+v
LB/w/xn8RK7kMy/K43bRzmlHXrmOf11lCfb9UJ5pk74euLaT9aM3ck8wlrkC
cHF/eZtEnjeM9p5g6p1TfcL7SspmZflEt8RzYsZlyvRC9Zz7PLEn4imxmziB
PWP7jCeiJ3zkyWjzRcSVvtH2vPZVLrn2EpvMPTj0a9+Jdh5mXSeQYy/4Q/Zl
Lo3md5kjZQ6Q5SlK+ehSzmfxwRNLee1SrvW8Xsrvl/KapbxoKe9ayp/1Ut61
l2fX7KWtSkOlbe1SPjG+yuPEvE8xUT2JUZ+XMs9mPzrX0U687mJuy8Q7dJfr
RhJn8JcHLINX73p/7oegvLJl2h0cba8htoqNEb+wZ2wPjKBfi01iS/j2Yd6H
D2DDj/sdtk0MJc/A5v8hT/BOjoMd4yPkONgrvnOtfEIfOEIOg6+Rn9CnxAfJ
BcFPfBMcAi/es0x+hC+DP7nvExmB3/h+zqmDBeApOQxYcZB0gCH7Kj+whVyL
nBecAAfIYzaSJuIbGEVuRg7JWgmwE3xbz+cZ7yYXIydhPAdMB6/esI1Byosc
ZJVo7yKZU5kii2P9RFjUBZaBWeAKNpt7C+gns96E/Jcxts2lZadoa7LIpUJ+
97QuMJaxv8HRxdify8dx6vMadbipeqLfl+vs0Cc4Nk59raz8mc9YRprRO3aI
bWMP4NhH0o0N41P0B3LshdjZ3/pesXyM94E52HmuEeL5nH/PdU2UyTdyDJn+
cfZr/ljKNZeoecixpTxnKU9fyrOU8qSlPFcpP1fKk8TX48l9pTymlCeeH/y6
e+7rpc7aVqWh0lZprrxUHivvVSbIKjEqcWOgss0yMk+86hNtjjXHFRfy/1zb
skW0dbnHWA+5/dW2iy3lGCB+hi1gf/gftoQ9kGthY+QC+B+CGBXd/AKbzP1m
2CrYsaN1MDe1j7/jh/gANg9+0H84VN4YNxoc3dwULMR3zlHG/IEd9D9znAJ5
b24d3/Ze4juY+Zj1sQYJXwZ7LpPmX0UbP6J/RFwHEw6WbmgEg/AxMOQo6z5Q
/vBlcAKcXl5Z8N1nyu5gnwVPh3rRTwTHiHtb+t03/Q7e88we8Iyx5hHRxU9k
Rk66nDqAf+LcN9TFT9QpMvphtPNPwVNwFezYUx4mtz2wZn7lfobtE0+wO+wN
bMcn0D24Oiy6NoLMyXMnqCNyTDCfsaktfPY6ZYdt3Sa/+As5LPg7nzImB1td
PZGDkduxjnkR+RgV3T4CdREHsXNsnrwb+19AmeEjM0pHXS9EOfGqv88kXtH3
TYy6M1rfJ/s7lO8o5WlLeXwpn1nKi5XyMqV8UCnX+b6KD6eU8jul/HApT9VL
+eFenj2ll7YqDZW2SnPlpfJYea8ySVnluojEqDujzZ/uXMo5H5B4lWtZc13r
wuqUOhf1PnREjFtNnZJrbSTP2BI2Q8zHf7Al4hr2i42Rn+MH2B6+xTgSNvmA
92Kr9KvwZ2yYfAUMSdveK9p7bQ6L5gun2QZ9qct89njlkeu9c687Pgs+4IP4
V19pOkp68FliAP6FL5/rhY+Dk7cqf/AFnAYTGJeinzkiunNWtAuGnOiz+Dq5
CvkJmEOugX+tJU/wlxj1hLIeJl+53wX5gLHgN3FsX+t+y09yWDCKHGQbZU29
Q6LtocPOcszkx+qKPvJ2tr24fF8gH+AneP2p9NwpfUdYvi7aO0H2ll+wfgfl
eLb0XuJ9YPjVfkc+eov6/SDaOD5yxh428LtH1Q//0/cn154i2hks8EUetYR0
vqxs4Q+MIv6SJ+W+vFmUWa6VeicajuV6cvo4uT4zc27KxMqvG6up48zYbPrj
oFJesZSXKuVcD0b52fJ9zXM+KOVvlfLSvdTZv5TX7qXcv5dnl+6lrUpDpa3S
XHmpdVbeq0xSVhPPO9QxvWssD4q2FmaBwsuC0XBpoWhruhZW17nPsZ/1YuPY
BjELvMFmXo12Rvqo6MZ4bCztDdvLde/YJH2szMFYi7Ce8gGjiPvYNn0HchRs
nhiwo/SDLfguPkL+hV/gO8yF4cf4FNhG//AJyyOjvTedHBDfHBZdPIMfxl7o
v+G35BJHySt9xKvk63x5ABPWiHaW0rzyc650ni190Mt4HZgDdpEjg0WsoXgs
GkaR0y1ru6ztXkF6wCAwi5xnbnlkDQC+By1gD3hFPynPR4E3sJq+MBid4043
Wt/R6pq88SZpAbfAuhelBz1MHW39+2TR5uOJlad6L2OJ9O3pWxALsDOwib87
1Bu53f3qhnbBIvqZ5JzkqeD4xdH2J7wa7Xw8ZA4Wraqs+Y0c9Zfyin+QP2CH
i0U7oxm7/Vw9Ydt59k2u0869JGBbrtMeFQ2j6lpNZFjHk7P8VCnDW/rj7KVc
+0R1LGirUj6hlO8u5SNLufbRRpTyOqVcc56xvZSX6eXZEb20VWmotFWaKy+V
x8p7ymRgkdVAZZjlOh+RY265ZjXLo6Ktt38t2p6m3BeQus69mZ9Hs4c1pI+4
hs2Qc2Az2BL4CwZgY5nDIwjsGhvD7m5TftgqOQx2iA1fYJvQdKrtYPP01b7s
V0bXpxgPwUfIH4ZE13cY+2WMZVrL5AT4GrkO2IcPkj8w9pW+iU2nzw6w3cyN
bvS385QheASO5nheYgX5ErgFhpDvgXE5j7ZxtHMnoSX7gOQwiVFgUeLYg9HW
VfL/d61/oLIdGg2vyJGeUsbcz3giej/dul+QD3BnNeu5L9r8AvQQUw6R75Wk
DRrBMzCKuHOsvOc6omvVyV7KAzv4gbQgB2wCe7hEXbxpHfy9o2yI4++qO3wH
29tE+b0b7b3m7yqbtEloeyPaOZm55w6ZLez3tD9ptP25v7A88b6SSaOtJ+9T
ysSqXHd0STRcOr2Udyxl/OR/5xulvGwpjyzlW0r5tfh6rKj11D5XxZ+a89T+
1/m9lMf38uzYXtqqNFTaKs2Vl8rjsl9Tz8Aiq4HKMMunl/IlpZy6yLGsLKfu
6j4m9IuuMzblPlLKC0s3ZWzmRcvY0n9KPY9Gw9J/ST82yV4a8lPskv4EuRs2
fK5l/o73Hmx+aLR95fv6iY8w7gLu4Tv00fBLfIpxX3I6/IQ8DCzEB8EQfHIl
n6HvmHNvxPL05SWsi3wOTMH3wTH6hmDC76JhBX7GmMv50pzn8tEW2JtnrlB/
YhHPr26d4PBGyvoT+cnxIu4Be5eTj73UB3ICzxjHIT6Qq34j2nzB9vKOTNg/
8Jbt0i+f1ToZR7pYee5jPScrC9ogX9oj2tli5HPgEJhEbnifdSOPi6Kr65E+
gz2w/4B+wbvKhfFYcsm91e949ZJ79NAnuSg5HfnVJdHO4s/ckbiW58gRM8jR
xirXHC/ANrCh7C9MGm3f7vej2fwzpfxyNIyaspRnLuXcr4cNTx3NH2cs5ZVK
+eRSrn2rOtZdcWDKUn6olJ/t5fvdS3mjUu5tvehGvTz7/6WtSlulufJSeay8
p0wGFlkNLDJM2Wa5yrzqAh1lTHmmlNHpaMs5Jkm72ECOW2Ib4BV4gs1gx1/O
BU/S5Rm/wMbwZ/IHbA/bJjfEJsFl8AlbBa/on/F3tjRi27+xfWwe/8Iv8IXc
O4SPDPE55IY/9ZEW5qBmkPbvSgNl1jctpV6gg/ETfBb/pi+HL+8b7ezJYT6D
76Nj+qtgQp6/RT3kePgwGAJWgTVgy47RcifoB6MTi8CyjeSH57fzGfyOcWd8
+m5pHapOtpB39AqWnWz7jBeS4xIvwAb6W5cpO7An+/v7RsPe/eUFfaGbm5Qp
MqF/Pl7dXKDcoecA72ecDnz8WJmhr6flYaTypEws2FZZESexG8apdo1mV4wr
YLdgFP3W1aLtlThGmpg3fFSalvV3xoJzDTM4Rl4APpJjLmxdOZcEfTlmm/4y
SJ6ynH0ZbP7tUt6olOeN5oO3l/IzpVzH4evYzqvl+wPi632/jtuPLuW6LrTm
cnVNxTGl/G4v5WN6eXbBXtqqNFTaKs2Vl8pj5b3KpMqqyhDZJi5tFC0uvF3K
r0frp+9dyjNGmytB1znniw3gw+AStkEuNZ/P4TP4JrY0Krq4hI2Rg3xLWYHX
5ErYJGPsYD+2yvgQcRQbPj/aGUDgFf64rfQxv4EvMA5DroaPgCWsy8lz5B5S
Vtjyk9IDXhHvsfe9or3rlv7iFN6P/2YfCl9jPh773kca0/ePVzbkNo/L90D5
AUNyDRbYknOeiTn4LFhEnrei7UDPn6WFusGarXx2Su+jXfps63j/59KXZxoQ
Q8B7+sh5Ng/4NXm0/Y/4K7jIWBJ+vLLyIZ4MjvYeKPgE+8FfcDbHANH9kep6
1Whn+mEXn1on+dgI6wHfybuZN7hJOSIH7HvraO95+Z58vqFsVoz23jzkha2T
Wz4c7V2S8E8evJh85Vku0IodgFHgINiFL+Y+PvTZJ9rc00alzPhE4lL6Ws5b
fV0OUMelnyrln5TyxfH1Pl7LdVz6+VIeWsq7lHJd71TXVg0o5YG9lAf08uyJ
vbRVaai0VZp746vyXmVSZVVlWHPXOg+b+ysHqKOcq0V3OYeLTnMtCrrG58Ao
bADsAqOwDXBlbnUO9oFR2NLL0vKwfDK2hO1h95kvEWexpymlA7/Hhon57KvB
LskfiPXYPPYOlt0lHbmuCJ/6tXxB75nRznG8zjaHRddP8UHW7z5uPX2i9enI
c16zzj2Vw1i/B5vT94dEFxPQK+ud3pQH5j3AEPJDxljWsZ4romEOfRywaFfb
PsL7R1rnXH7CD2N39JXI/3aW/iXkhfvAPDAMfH1beYMP5NL72V6e7/CEegAL
6beCc8SPC5QtdZEXkj9iW4yngivMP9K3BofBBjCY2IB/z6j+wB7G3ND71dEd
Y+d/8rS9peNq613U9sl9sQPiEWtLplfnyG095b50dPO4L+QbPvOMm9wjlmMS
4BsYNTzafg1ozzMTvhFtP/LU0db2/CDafPrN8fVr0euY8E/j631zk1KufaIz
SnmeUq5rDGq+UXOYii3H99Ju7ZPWft/4XsqT9fLsJL20VWmotFWaKy+Vx8p7
lckmvbRbZVvnMuocR56R1V/d5TpeeMHn51fXeYYhNpBn1wyPtm8Um8lzAiEC
HwbnXoj2/gZs76Vo7w0ibwLvsFX8HgzBhsER4jF2ia8zzoPN51gMvoCPYOfY
LP0rsOlD6wLL8Km1ooshyG0fZYMPgj3knrlfGb870PvgAV8GZxiDw8eJ+fgR
vv8L5ZCYMNJn4BWcxO8PkufEFvqoP5J27AUsOlvZXKm+c43+dOrqTp9FDrk+
gvyGXGRXL/7fLdo53/jTB/6WayRog34bfbY9lANYR367v+0uZlvIfUnL5F1L
W0+u1Roiv/A5q88So4gxF6iL9eXzdutAz6zlYE4IzJ5JPS4ZLV9dVP2Ra4E5
YDB2RV70gPWCObk+gzI4hB1nXjQ+2rqdcZax0dx3xl/u70g5DYx2jsrAaGeV
57qgxKjqp98t5SjldUu5rrF8sZQXKOUNSnlMKW9W6y/9qU/L998o5bpWs+LG
0b2Up+rl2VpnbavSUGmrNFdeKo+V9yqTKqsqwyrbKvO6rwcdJV7luTr91FPu
n4po+0BzvzafuY6OfAWbAdPw448sgx3jLWPT2B5YR/xLm8RHcz8g/ThsmH4B
a7hnkmf6INg88Rd/xRcu9XdsGSwYFe0dCjm2tFQ0X6MO9qctbZl+9pLyjAzS
Z3kufRne97AMduH7K9t+YgIYC1YsqB4SQ3aUVnIb/AfMAQeJS2ARWPd8tPfU
gEfYCfnWjfLA/Y8WvpAPY1L4/Uq2t6e8k798Q96oEywmvyGWgbE7FXrojzOG
dZK6OdPniDHkRvS98iwKZAe+gtn0WY+Rv82lB30S26ZRptgc8QLcpo/fN7p+
QR3YxmTKcmZ5QIaswyCPHhFdLH0u2vlI+MsX1vOm/6MXxtv4vp88YmfY8IzR
zqrClsCr7GvkWQr4b+5Tyz3I6ZtZruPM25VyzUnquoW6vr2OC91cyrOW8hql
XNfJ1zUGvy3l2qeL+Hr8OauXdm/u5Z6peqmztlVpqLRVmisvlcfabpVJlVWV
YZVtlXnqor86ynJiYL9omIxtZJ8dvMo1peAVtoHeya82sYxtYktgFjkHNobN
TGlb+OR42/rM30+Wf3xzJXkjL8K2R8sf9ZKLMMaPL9BXYj0APkIuBa7l3Dh0
4lPDo41dk7/h2/gg+dht6g7/Y9wJn2Xc6Y/ew/fEjfTxLaP5PvJ4zTaoN7GC
HC0xhP4w2MJcFriUeQj8g0XkmnnO+8HqBf9krBO8IE/ZK7rrILAZ8JG87T/S
cIfyI/dcW1oTR5JOxvbI1ch3+ih/5JZj8nsU+hjnX9IydYyIrn+vFk3/0yqX
LeUV3AWvb5X2ef2e9tE9ueFHtjPUNsFB8H2c9FAf/WzGAsjHyC/zPNOZo63B
IiZOo+zm9pnP1fUyyrFvdHGXZ/vZFu3397k8B6aet4kevm7Pcu2nzF/Kve1t
qee67FTKvfV3ag5zUinXvcx1jUE9w6qOO9Xx84oPO/RSvrmXZz/rpa1KQ6Wt
0lx56a2/XGVSZVVlWGU7fy/15Dxj/6K7nAPM8oRoZwE9Fu1MM2wDe8B+sBns
BLycXB4YT8HGsCtsfe1o79/JfcXU+4r15LkB5GEPRltThF9g84f7fO5/O1Y6
cv0ovoOvMDaPT+Gns8kfPri0tDMvmflNzt1T/lW0sXTq/LP0gJdPee0cPbGL
PCaxYkw0DCEHSmwhViTmgIWJRSdGw6harvfUZ2udta1KQ6Wt0lx5qTxW3qtM
qqyqDKtsq8yrLqqOqu6qTquuqw1U26g2U22p2li1vWqTfaPZarXhatvV5qsv
/Bev/t/Gq9RX6q7GoBqbaszqGy2W1RhXY1+NiTVW1hhaY2uNuTUW1xhdY3eN
6TXW1xyg5gYjouUMNZeoOUbNPWpOUnOVmsPU3KbmPDUXqjlSzZ1qTlVzrZqD
1dys5mw1l6s5Xs39KnbVXLHmkDW3rDlnzUVrjlpz15rT1ly35sA1N645c82l
a45dc++ak9dcvebwNbevOX/tC9Q+Qu071D5F7Wv8tz/4/2Z/cIA6Sryqffna
x8++fx0TqGMFdQyhji30jTbmUMci6hhFHbuoYxp1rIN6cgykjo3UMZM6llLH
WOrYSx2TqWM1dQynju3UMZ+KUdyXY0SjomFFHVOqY011DKqOTdUxqzqWVce4
6thXHROrY2V1DK2OrdUxtzoWV8fo6thdHdOrY311DLCODY4qfM0UbSyxjjHy
TI491jHJOlZZxzDr2GYd86xjoXWMtI6d1jHVOtZax2Dr2Gwds61juQOi973M
/x1v/yrN/7fH2+scx8RzH3VOpM6V1DkU/nIuOOdcJp6LyTmaOndT53TqXE+d
A6pzQ3XOqM4l1TmmOvdU56TqXNXIaHNYdW6rznndFW0urM6R1bmzU6LNqdW5
tjoHV+fm6pxdncurc3x17q/OCda5wjqHWOcW65xjnYuE35yjrHOXdU6zznXW
OdA6N1rnTOtcap1jrXOvdU62ztXWOdw6t1vnfOtccJ0jrnPHdU65zjXXOeg6
N13nrOtcdp3jrnPfdU4884H/0z7B/65n+CrN/zfWM9S1Iv/TvsJcc1LXotQ1
KnXtSl3TUte61DUwdW1MXTMDTuRamrrGpq69qWty6lqduoanru2pa37qWqC6
RqiuHapriupao7oGqa5NqmuW6lqmusaprn2qa6LqWqm6hqqurRoZbc1VXYtV
12jVtVt1TVdd61XXgA2Jtjasrhmra8nqGrO69qyuSatr1eoatrq2ra55q2vh
6hq5unaurqmra+3qGry6Nq+u2atr+eoav7r2r64JrGsF6xrCurawrjkcEG0t
YuYVX7d2sa5p/O960f+Zr//TetG65nbitbh1jW7meHVNb13rW9cA17XBdc1w
XUtc1xjXtcd1TXJdq1zXMNe1zXXNc10LXddIj4i2drquqR4sX9gmfZBcg13X
Ztc123Utd13jXdd+1zXhda14XUM+Ktra8rrmvK5Fr2vU69r1uqa9rnWva+Dr
2vjto62Zr2vp6xr7uva+rsmva/XrGv66tr+u+a97Aeoegbp3oO4pqHsN6h6E
ujeh7lmoexnqHoe696Huiah7Jeoeirq3AozKPRd1L0bdo0H7uXdjQPTc01H3
euQekIn3hsxbyv/dj/NVXv7/7sepe50m3gOVe6Pqnqm6l6rusap7r+qerLpX
q+7hqnu76p6vNaLtBat7xOresbqnrO41q3vQyElyb1rds1b3stU9bhdE2/tW
98TVvXJ1D13dW1f33NW9eHWPXt27h7xyT1/d61f3ANa9gXXPYN1LWPcY1r2H
dU9i3atY9zDWvY11z2PdC4mMco9k3TtZ91TWvZZ1D2bdm8lf7tmseznrHs+6
97PuCa17Rese0rq3tO45rXtR51df0Fr3rmIvjPW/ZjnnAuse2In3xtY9sy+X
8v+03zn9qO7V/e9+56/y+HX7nTNeTNwnTdl+ncxTF3VPet2rXvew173tdc97
3Qtf98jXvfN1Tz19nNxrX/fg1735dc9+3ctf9/iDV7n3v54JUM8KqGcI1LMF
6pkD9SyCekZBPbugnmmAP+f6+noGQj0boZ6ZUM9SqGcs1LMX6pkM9ayGl6Od
4VDPdqhnPtSzIOoZEfXsiMRe6qxnTVB/nkFRz6aoZ1bUsyzqGRf17It6JkY9
K6OeoVHP1qhnbtSzOOoZHdhwnt2R4xj/ip5nfRAv8gwQyvVskHpmSD1LJM8Y
qWePTHwmSZ5VQrtHlXI926SeebJjKf/3PJmv8vh158lMnMvtGF89qydl/nW6
yDN/qu7Qb54RVM8OqmcK1bOGlo52BlE9m2j1aLZRzzLCr/KMoyHRzj6qZyLV
s5LqGUr/iHa2Uj1zqZ7FBF/4FH3BenZTPdMJHvOsp3oGVD0bCjxezXqzD0We
V8+YqmdP1TOpoCHPqqpnWNWzreqZV/UsrMminZFVz86qZ2r9I9pZW/UMrno2
13nRzuyqZ3kNiHbGVz37i9w8zwTj2Twr7IhoZ4jVs8XQXZ45Vs8iq2eUrRHt
7LJ6plk966yegQbd9CkYpwBj88y0epZaPWOtnr1Wz2SrZ7XVM9zq2W7Qgt+l
/Y+KFrvzjLiB0cZ5/qd9hf89r++rPH7deX0DJpJVb/sE6xmJeXbiwGhnKg6K
dtZi4lXqN89mxB6wAWy8nuWIjeUZj/XsR8aP8kzIXHtFbK9nSMIrtprj2tjw
09HzLMp6RuXV0f2jL13PtKSuPOsS/eQZmPVsTHI+fPCw6HmWZj1jM7EMXKtn
cs4U7azOeoZnPduznvkJFu2tXreJdkZoPTu0nilazxqtZ5AyppfrNOuZpXdF
O8u0nnH6ULSzT69S5tB0VLSzUokXF9gufas8W7WeuToy2lmsOQZ4WPQ8u/Xo
aGe61rNeoS/X8NazYYkteWZsPUuW3BS7or++WrSzZ+uZtPWs2qXla1T0PNu2
nnlbz8LFbvOM3Hp2bj1Tl09sNMdG8gxefOSaUq5n9tbxn/+eh/zVOv9P5yEP
iJ5nStezpq8p5Xo29drRzoNFZ3lO7OzRzuhD129FzzOx61nZxME8Q3sZ6ScX
wd6wMebV8J88i5tPbBIsqmd3bx/tTG9ozLO+8QtsHqyqZ4PXM8PrWeL0/a7z
Ocp59vgN0c4kx1/xTfKHHFfi+R9GO9s87RM+61noYF2ekf5ktLPTkRH+vG70
PGsdrNpcuslDwaIDo+eZ7fUsd2S7VnTzpXr2O3Xluafg9TnyixxHyA84Q65B
H+3WaGfOnxvtLPq9o51RX8+u7xvtTHuwmVhAn5d+1/zKk3bpQ4LV9cx8+CI+
ke/lOMHp0fPs/e2jncn/42hn9SMfcOfv0fNsf2iZTzmtGu1dACtGe0fAstHe
HYCvbBTtvG6w793o+Q4CsCvHT+o7C/h9ivjqOw5y3U6u87mzlCdEW69Yx6Kn
LeXa/zqzlGufq+Y8B5XyuFKetJRPKeV3SvnhUp6ql/LDvTx7Si9tVRoqbZXm
ykvlsfJeZVJlVfcy5zxs/+j5jg9knetPpog2t1vP5UOnb0fPd4ugO2zgtej5
LhL8C5sZFT3fXYItYWPkDuRo2B6xkvFmbPKe6PluFMaRsGGwZodotr13tHes
DIvmC/gJdsy4zmnR3tXCeBE+hd+QY+Fr5BL4IT6IzzPOgj+Q8xD38dntoue7
Y8jp8HHmYt+K9q4ZMApMIObjSyOsFxmBIfjTN6O9ywZ+wZyh0fPdN+BJYhT4
kH3A+g4dLnLOH0fPd+7w7FnyhExvUpbY49PWg/7Bf9b5gS0InbyRvhm5Hfi9
fbR3A3072juDWGOxubKE77mVObLh78zo+U4inh+sjs6I9g4j8i1iJTi5T7R3
HpGDERvvjp7vSEIXE6Kd7ZB5VOZg5FHoPN/BhMxety58CLzGzrEp4tuY6PmO
J2x1i/jqO6ESr/KcmT7R1h+uVsp/jK++i4ryaaV8bHx9n2j6Uq5rNSs+zFXK
dY1l7aPVPKeOdY8p5eN7KY/p5dn3e2mr0lBpqzRXXiqPlfcqk1zf3j9a/7p/
9HxX2mrR1vfmmFW/6PnONXSW61XQac4Fo+vXo+c73dAdtvFC9HwHHPEr3w1H
jMPG6C9l3kUuwrwhNonPgRnjrJc5J+yOddGMh2Cz9B3ILbB5sIdxXnyB+Xl8
eXB0fQLcwXfwfcZY+CMekJ/ga+Q64A4+SL+Tvjt+Rq6yaLR38OW+t5wHheaf
Rc93+bHe6Wn1Ud/9R4wHQ3L9GtjCmm7wizp+Gz3fLUiOuIL8EAPxQ3AMjN9I
2sHV7aTtrGjvNARrzpQv8tDr5feBaO9GpE8F5hPbwIvPlMXvos0XZK59R/R8
JyM4Sk5Ef3imaO9wRAbLqS/0Q1yjv3h7V1Rf0oX+sKvboue7I4kf5Nw51raV
8ts+2jsowWX69MRB5nzAROLAutHeZTk4ujaOPWNLiRvLRXsnJrnWu5YXj/YO
zUWivVsTvNoi2hlZc0dbFx3Rzp9ZJNpej0tL+fMoZzuU8hTRfLOuLb+0lOt6
p9pfi1KuY0EVT9Ys5UVLeddS/qyX8q69PLtmL23VM1ejlNcu5cpL5bHyXmWS
9fSP1qfuV57tF+0dtbk3Ic+2QkfotW+0d91SRqdvWX4g2rtxuffl6Pku3X7R
3rGbGE6cxwawMdbpgGPYHnZNDoZNYufkYNgq8R/bw4aJ7+AYtg0WY6vYPBi1
YbR3B2PP+AhjL+R0+M7+0fMdxOgBX2PcibwFH9wmer7LePpo7zgGqyZYZnwL
XYMXYMDb6iAxYYbo+W5lYj8YMlv0fBfz4tHe0Qx/YBEYS98JjBoaPd/1DO8/
8BnwbQd5PS3aO6MZm8pzTOlHgR/gzz3R3j0N7hBbDoqe76rO/cTo7dBo8wtD
or3zGlqIvyOj5zuyGQvc2+9fi/ZO7fujvWsbvTKW+FT0fDc3+gNHyU3BcsZJ
Xo+e7/jeOtq7v8FP4tDY6PmucHKt2+UH+REzP4me7xxfSh7HRc93lOMHd3n/
gtHeaT4o2rvOB3Yf/bKtAdHzLBr4zn21v4i233alUqatifs7mY/VvliOzwwv
5WlKOfMQ6JmjlOu+49rXm7yUH+mlPHkvz9Y6a1uVhmlKeXgp1zG9ymPtL79X
ylVWKcN+ynbi/Z591UXusVoq2llD6I74NU+0fRJz+/vrlnn4Jcvo/1nL2Mzj
PostPWidZ1kX7WJ7N0knNvmX6NoJtsp+NeIdNpzr0ZEtOQm5PX/4+DLK7WDL
1LWvfOA7e1hfzqPxO742JLrrFPDBzdRPxm3sD58lr8lzA3J/L/YLXpDDvCIN
iQngI5gHVnxTGZyu7Ikbh9h2P+sgP2E9E1h0qjwMtk5yGPqJe/kdOEpOxjgU
eLuBsqBO8IxYlGPn5DTgIrkZOc0/lQOyJp5MpqzROfnl2dIHFh4mv0tIY/LL
ONLLPkesIT/CnqaVp1HRtQviA7j1mHxhM4w55n5AcsiZvfdkdYpNDlWnuZcd
3wFDyHfTthjbxLbpj26oDtETujneMnh1o+Xl1QHlpXyO8mLyO15e6AcwjrmA
+qItfG1lacgcIPfJbh7tXBo+85yH3aP53celjO1n/nBPNPx5rpTrOPPz8fXr
ji6LhgPLlvIhpVzXdi5Uypv2Uq731GcPKfQsOxENlbZs9/lS/ryUcw/yQHlP
jHqwlD+Ohvm7F3mmbOdX5tw/n7qg/nnVEfTk+AT501zqlHg3h/TQ12EMHp1j
x+QQL0gfOIDNgFf0ZbGlBywPiW78p4ztXe/9tHe59WCr51k/f4z9kU9h20db
RlaHWl5cWXM/PrKHv+M7YBT5UmIUeQ6+hu2Pk3Zi9VvKjX4ymIvPYu8PyS9j
OXcpG3zoNnXEXBh4DCb09ROswMfpI4Ihuyl3bOBA+QVz7rQ9sAg7YAwHjCLP
Au/W9dlV1BdjXuRYu/jbEcoAnZ1rW2DXFV7QebXtgEP0AclxwMhL1Of0Pocs
lrM+eF9LHuCd8ST6rNgI/eZFldHh0jkiWt8Qvf3bdpD/C/6+qTKGz6WVI+2i
7w+ti5gBPoDR/H0qv3z/mTKYM9oaY+yH+A1ebR9t3wR2O0F+/mk51zBzzyLy
SD3kVK9Z/0Dpx6bxo6Wi7d+fIto5WvQb8CX86ADpp4zNZ26wbbS+zAbRxmF+
XMpHR8OER0t5fDRMmKyU5ynlkdEwYWwp17NZLizl3taL1nvqs2MLPSPj62mo
tOVY+kB5yfLR0fDqx6WMTLIfvW00vLox2tkvKdu+yjzPrcpxxbnVEXXOqe6g
Z3Z1Sq6Ve0uxPXIfbIB+D7kFtoEP05fBZhh7Zo4KW8r9X9gY/YupbYOxdMax
sUnG18Aq/oj5U/r9CX5i2/gouLW0/DDOl/0EbDExapz0b6PsRkS3fwvO4mv0
wXLscHB0cyfsl1hyjzyCg7dY10rSio/jv/Sn8Mlh0e3vgAnEdnDsimhnMvxO
eshHwYLV5QvMAbeJ75v7PFgLDl2sTMDRX8jrT6SfexlzAE/A6KHSnHMM3At2
gf3kRBfKO3GLnDXPpDhLnUIzY3D4H3FpP+WY4/rIgTbWVg5/lWb0n33nnNfF
bliDcLCygfcLbH8vZQlefa/IZvFo2IdtvarssEnyMfAqrPtHyoW4gr2DEWDa
GvL6uXo6SB7It8FR8Gox66a8kHxRxnfe8H78hZzvM+ueNdq5f/D6ieX5op2h
lLGI8szR+i/Zz+pnOceK8Y2c28qzQLKvl2uKoC3L/4iGCf8p5TwzoWLUQOnK
vGVcKdczrGq5zvd9FA2j3izlT3uhIWkbJM1Zzj7jIHnMedXcxzdImeScBeXs
X88sHf2Ubco5ZT6vukh7ndVnZ1d31DmLOoWemaRttO2/6m9gUa57AnOwGfIL
7C73mU2iLvFN7APbAwO+XBsQXbz6xGfOko7Mz+A/11WS82Hz+yhXfAEfpn+G
D4CP4Ca+A7aAm/gUeEAeiq/hf3erL8a/b5Iv7PqqaOdJXKZsiM/4/mvWk75/
kHSPiK5/8+zlyvEOdYf/Eh/w+zWinedwuvz9RBqhY33lg8+AqTuqR3Ir+q3g
2S+tZ4do70U8yLagOdcWLK0Ob5Vm8jbGEffznpfif7V1HtBWlcffHjvYBcWC
KJYgahDsJYLYYjdq1Pi3IvaGaKJExXAVjQV7jC2oWGLBWGLDEgXFil2xYeEG
sWDBgkqHbz2cZzInWd9d66677zl7v+/Mb35T3rL3rn3s4EsNeax9w4W9xeTW
aMQ14gH5AC7BQXIBtsbm1Mlw4kXlAxvuQzjY/omJ1L3UdUf6P3XzduIKNnCX
/EpsJQ5yP0Gfhgjz4h95ifxETQyn2tsnOYBa+Cf/UrvOEJfbtCUxtjUqRs1r
1Dm7CX7eOeq9cPgF9QPcpKYgV073GDul78ynPTleOmqOJfd0rebnuYa1QNT6
e5uo+xb3i9pHlHuzu9hv857VPM5nHXSJ/36OytCoGPVl0/FFUePT9Kc1omLd
Gl6btV8+V2dN+8r42bxXc/kmOedG7VvbzzbXUsfcH7JA1Nrr0lFrGStEzQ3y
eY67wTZzxGZN+KctVrLNaX52nNe216b5rJDcD0aN1BqNPEiMgRuf2w+coX7J
eYS3mtrKcSzcY65lstfgY5Psg/pqorgwL0v8gdvUBnAXzp9suxfaHzUcPkLc
IBbhO/j4E8pPLHjQvnt5LtzEr29Wry72PUuc/tKEPf6Oj28T5fv41l/tg5gx
TD3AklrkSn+Rp8Vz87kOO9gPsYixJvUtMYoxFLxgzYDY206dmJva0D6Rnzl3
xnmMz46wH2Rmro75LHyF8Sd1zcbKR96AT6doO8ao1H/4CjUONfsM+yKmpx93
95g24MY0MZ+jrOhHvnjHNrEF99AwliCWDVU+8Ml98fjNvupzujLiJ8RM5h3w
E/INtTXzZuSoECvGgMydEeepqRjjkTsZHxAjv/cz+sv5qH8r77w1JuMVcSbr
6ZWj1o06+neWtuwU5SP8Zn7HBlP8/mj7XcXjXLciV8Bx4hXjT/yleb8QsnWL
2uuILM37tPMek2x/rah9nunvGa+wU46zhkSNs778n+Mctw6JqgPzfr01ouLt
mv6fsTT1ar6PaS1lzj233eK/7937weOuytBVTHIdljZzjeNocV5TbGdok7xH
r5O/ed/vHI+Xi6q3l9amtLOYstEXPPq38vDZMPHnGjjzqfLBm3F+B8fgMnGN
H8ZPrbZ1t+fBVcZyxCI4zFiH9Sa4TbyCr3CecQO+iC/gr8QifASO5zOV8Hny
d9ZU+RxTOH551PugzvcYnc9SB/AYGPX+n9OVlfiWvk9fLcqBvxM7iRWs+1Ej
4qNrqBvtnqz8JyoXcZhYRL01W3nznh7qib7Kcqj266je1Dub2Ce1wy72xffE
HGo+arw+Uc9e7Sv2myrfx/6ln6+VD4ymijmxET70i3q+InmOnAlHyB/wEp4+
qtzj1YGxLpy5RFsQw6nrrhHrXKsgHjPHRozdWhkYA+I/xFRqvEXVk5qKH/ad
9FSOh5Wzi/ZYV5uQV/CvW6PWkr4Uy47Kj71XtG10Iw7/2s87+N0sj4mPmcep
i3MMks/IgivE7OTKv8SUflkH/kqdjo56LkUnj5G9nefjv8T4vJckx4kc532+
/799rTnOiqixVZeo52rOjJoXmhk1j537zHOfQI5b/3cf5mTbXCQqruY9d12V
Oe8XaKfM60Q9/2pddc94ACY/e07OuXQVwxlRz7aapTxgnnUvtpijfVb0eFlt
x/GS2pTzk6szo+4fneb/tyoPOMKZb7QBusGfiV5PHPlQfBgDE5fgJGPDl5QP
X4bzcJi4gj/A7ZZozBvDefyWmu5OvyPG4SPUD9RF+A7zsheIH20NUlbiIPM/
+ezAfupKjmUMNlUM+qoDx32i4eOrRfn+Np7/mPLwm/u9/6RNwfEyMTxUmU9S
3yf9jHmeHPP8URvu43et6kfthS/h21toq3Oictpf/WxLZWC8vp3YLa0c4NtJ
ubHB5raFHQ+xnZ9tl/Ow+bXaB6wu9vvZyka+wuZb2fanTbq9pm3RjbkCctjF
2ob8QvwnbpPjqfFyv93+6nSf9oN/LdGYZ6QsGqj9yHGMKck38Ju4t6Zyvab8
jM+pu4gt8DTnGD5QB3xrQXVjDnYzMeYY/5rl8Stez/gAv5xiO9P8n/bxObiS
+ygZK+BfzM8xfwmvh3v8C/XlnIw52AB/h4d539BStr9O1L4CjreNqlV21iZ5
X/nU+O81Tfp9JGou6BFxaF5rW9Vrv/OctMXqUc90zL0NXynzUlH3uczxnHXV
hWu72Q4xcD11/9HPP1K23PM1w2uni3lXsZ1jv99ql1W1BccraSNst6y2y+er
p03hyAeeP6epTbCAGzPF+lAxyTHdN16XNdkbjebmxWViDJwcFQ0/RVZy8v3q
S07G/+A2+7epp+A8fksswxcOj9rbTR2GL3zq9blGyhgq17wYNxygTsi3V9Qz
5ndRn1yjg0fUhdQe+DicTd9f2/MftY3fKiuxqU80YgixivHPOcpzjrITL69T
9rvUgxhFPB8WjbhLrcF81pH2A3anaJsZ6txF/MhB5H9i7h3KnHNCbcSVOaEl
/JxaFp+GA/gNsWtZsSJ2wTu482dtCVaMmydo89vEG94Rcx63nUPEIJ8Rga0e
U6ez1Plt7XKq+nENddkb/oXTxO9ttQPzf5s0RJi3HxGek+uJXdQA69hPR9tl
7nI59X3XNnLeNdeA4Opits13xGRyDNzm+GvPWdzvfxZneIJfL6MNvrIvzmXc
sKJtwP1VPJ98je+P9zjnneDo2ur1uXr9aJvdtMe3Huf9L5xztzZBb/wEPySG
kA9yPNXd487RGH9NU54ci63sOVM8vti+VrHNr732bu26hjJ8YV/Lepz7Zzln
PXXh2h7qCFbrq/uPUc/Em+o5c8VuPf/Ots0ZfreWn9Fw81i+o/3MjdovMFd7
vefxgtqa82c2XUsMHeM54H6ifXwkpmBFbMp68nkvxf/gOXxiHfxebcNn1Fhw
l/UyfBFuM76gdoHzxCT4mvdz0Cd+wBraYeLHGGLfqOfI4fNztFNv5ccOjJHw
zeWj5gCWVHbaSX/Hx9tG+T48hT+PagfW6nI8uJVyIzNzL8SWYdGIY8ScY/1L
LGIMx3zOfv6lfqTOOMH2yCPUV9TLjKs2EVvWi4k9cIL5ng7qxzWLifej2m1R
ZV5QHcZpW/6Hd/AUjsDfLbQTmFAXjRcrdHhVOxOTntC25GzyJ/xhrokc8562
It49qd7c/8Oe0ZO1ETYkBzDuI9bnWuR62jLvZbpLnNOvV1cH6iviQ859LSN2
N6g/fv+cOLDm8mnUelQ+I/DMqPc0pY8wjzvd/4nbE8Wlre2gJ3EPzlK/UacR
F+Hz8spILd7JNj5S5lkewx/yU6t68UMc6+4v9oBPp2mHDaKeVY6PE2/y2ZjT
7KOrstAHnP6Lx9g1x0UrRe1tW8FzfvT4s6j3HfN91oc/i9saUc/77qpsnyh/
d89ZX124diN1pM1NlGWKxzPsY+Oo/SQbimH65+vaYh0x53hNbTE7ak4x569m
+Lu4Np2uHX+K2jc81vOR6SvbIf+eb7/vRt0Xga+0tU34C/eIc9RlS3sdMQqu
4hPUJnCYXIlfwm14+Ef/4gfH2haxinrpPf/i23CKemYrsSFebaDe9JHzhMTs
laPmKnJdGj4Ss+AOdSH8HycGi6lPG895VJuD2x3ahLqFGMLYsZs65P3NxBzG
TNQfR6s3f4lR+DGx+SB1vDLqPTzEAniCHz8jztiQmuxa+8eedyojGDymzN8p
83weo8uCnvO5eqRfdo5GfYNtiV0vixVrA0+IIXtLqIOnqzPzSvlssQtt/3nl
HRWNPI2Ow6LBJ2Iz+ZqxIPu79taWW4jXSP8iF2utOceCbsQExiOtfr+zWBCX
/unxfMr8gLbFhrk3FD0yZs31Fz2wdc7hjowaPzCmpfYgpl0hXtie2nm8MgwV
s+W0LTZZWRyIvXm/CZ/jgx97fg9l5f+NbRffJ4d+IKY51iG25FooeuT+o9m2
OUddiI1fetxR+eYo25Lq185zfvI4x3TL2SZcWMnvcx4EGZCpq7KBZ3dlHq/8
83nt5l5HXMj1ZerDnmIyzc8fitqLMjRq79xlUbHrCmXuqi1ybIuN4PLyUc+g
XCKqbppfWxOLie8LN52HoF8rN5whFlFT7ey5+NdS9ss8aYj5HWIIRvhcN/WF
83CYMQex6gx1w3eHeS7+PSoateAu4sCYL595Q31G/J8uTqtExatllHk19UCu
DlH3gS0uLui9kFiPi4qzY6LmPYkJi9pPrg8uoIzwi3iMD1Gr0B8+fIl2QR/G
wczVEKOY86EOYYzEGkI+l48x5K3iQA0yQoyIu8QFas7lvH648sCpx5UF7rwk
7h836TJOHRfxWvSBvy9G7Z3MGAWG93p8oHbAhox/4Vfe7znIvtmTx9wg8eZd
9R+mvLtHvduCnMI4hbyD/zJGgkNrqAdzFssr+xnapmNU/mC9YqTy4ust0fCz
G6Lub4F7byhj3hfzjXLN9XNibu41P8k2+IVnX9o+6x5wHNvDL+JPW233mtgN
EGtkZHwwRhzJycT+buJGDQEHFox67kSfaNRpvSP+s9eR4/eVhXyX938S026P
2mP2bNS60ayotYNDbWsJj4kDbTznZ+XnWuqiJWzzI6+lL+qSVZXh9ag9bOC5
oTKDw6/U5WNlRke4tZ26E1u2F5Npfj5AzLcWwznisId4bxK1DruetvhCPLER
PFtJ272unfOZSguIAfyb6nejtCXcILcScz/VXtRJx0c9twD7EYvgIj/kLOZE
FtUWV4gFeQQf2FPZkAtuvyU2w6Lhr3soC+OPrZT9dPWCf/hHjnN+rbzIT501
v8d5X0XGq6+j4tXEqHjVGuXj46N8P8e0C3ousQKbw3liSCf7IbbATeIkPsdc
FfMYxKKByk+Mwt+pxfraB3bAj+EQNQq1CeNfYhxjRzhLbBkqvuQI6p6cm/mX
dkP2l5X5/SZdxka9S+iVqNpytJisKNZZm9/p8dbqN12c0Q//Z47qNNvPsewo
+6amukHMmBs8VVnQYRfb7hp1XxU8XFh8cx71NuXGhvgbPKQ+heff+tml6jTd
44l+/px9v+y5HA/yWsY2u6lz7o3Av/I+f/ztB+35tufhwznG7hr1jApiysio
OeEntfV9YrG+fRIj8E04ga9s73XjouYrJykXY3F4Si26p9f3jobPzxSPQ6L2
gD2ijktp16lRz+bhYIbn5HrZIeK0gG2m7++pnJ2UAX9bV9moQTZTZri6jbqA
z87KCG67qzs+uYeYTPPzXp63ixjOEgewhYs97Zt2Noq6x2kdcUSHVcQIbrTT
psSEhbT1ZepP2/gM8Wq6xx/4+WVic4vXPimmtHmvNmYcxdomnCQ3XRrFjayZ
iB/49KnajrwO59/z+1HRyP9baGf8lpiH7xAXVlQ2xonExbnaOfNProvk+P4z
caLmIy7lePCD+O+aJH2fPJQxAXsTK9raJrIsaz8XqRvz9cQc8ga134naDR4S
oxgfUUcyT0jNxTop8z13aaeWqPXESz3v32KygfLfY7+TxD3r4FeU/1P1ydib
Or4TFa9eiorhtJFzfVlfbajtwJZ5wku0FTrkHtnB6oN9iEWMaak5sTWc7K+M
tLWt+K2uHe4TK/IKOWhB5cq1pQPUmT7z2T3wBh8bqX6fKsM72ou4R3yGT/CD
/IcPkN/xsVxHBNtFxaU1Grx/1/+xM3XjD8qE7bNeyXu7l1V+uNdDzPBF8i5z
2die+IivERu20c57ij91C3XEUDE5WIxSb3CAv79RNmy+hZ+Ddedo8Gl41FoZ
sXSyxz/ZP+fM8Pol1HtK1HteFvZzdOmoDHCru7Ix1t7Kazh/V3XJ+3I/0Q4H
qDu2OkhMpvn5slHvTgBDYvIeyo3uO4g5NfmWynaHbWAjxh+5Z4w41UGbDrI9
bM08KjyFA8wzkYM+VQf4BGfgD/4Fl6ibRirfueoPDnkPMJxsicZ6H1xlHpA8
va3nomt/++sTDc5/oL2eEGdww0eeE8+v1WtVZWVdjtiPrx3aZMdtoubtiHN5
73gHZf9CXD63zwWifJ8feP9k1Ltj7xEncL3BPhnLEFtWiXqvF7GKeEssIodQ
hx4etU+OGuoU+2N82C9qL1I/Mb5cnMkpw2yjvZhtat8j1QW8iRN5n9NHtj1b
PfOe/B89N2sSZB8rVsSS0VFjavqZpvzsJ8l9rox98X/yKONY/JKakRwOF4jp
O6sjbVN3bhb1XKNllGkpMSZW59o2vPhWfTN/E3OxLzHlTPGDe3nP1Av2Rdyg
Vuhi+8+IadZexG9sn7Uz7RLTsO2YJszIs5P8HF1+9vxbxXE+5QO7xTxeWz3J
34ybqJfz/iT+Phh1DxQyYuN97ZNcDZ+IM6dEzU8cJVZ3iOfi6ru6sk3QPuj5
z6j3QMHRp6LWGJ732tHqvIwyz7Yv1o1W8pjaYyNlu8l+kRls91eXUdGYl8l1
4uPUHcxOEBPkOlasZng+P3DowCbM9xJz5Nk+KlZs5vl/ilpjRR7iKjZ9V52w
da475l4l+AM3GAu2RoMz9PWm+r8mTvSJ3zN/BfdmiTU/xJfrxIrvhygTcXKg
fR2vvldEvcsPX8i1RLAhdmyt3s+ILdhQs+SeYTBfWZyJBcsqC/kNX8VnN1Rv
eE09Ss0Ed5aM8n24yZwRMYH4B0/w4TZej77MXZHHLhdLYtLZYoyP/UE971DH
HT3/WM9DL8b4xHByFvGe2Pw7P+vk8S1iwpwfOZzaktj3lO2Rp15XRuILcwzE
gx5iBYc6R8MnlvIXG68gNllfYb+cB6UOyLVX5vC+sK+bomoEchu+Qh3Z3/8v
UuYD/A59iN/Urbk3EpstE1UbIgvcvTZqnpj6Ad6NVXfy2Q/+RW/4Qp4gvpC3
mHt4KGrsTozgB3952L7gIXUYdoa/cGhZP3tNPLBVzu+Qo79RVtZG4Bf2x97t
bZ/jtcUYm/WOxpowuuwX9T6h/l6PjKy1/EKbES+IFcwxkJ82tn9qC3yAfM46
Mz7PetT62oc2ZmqfC6Kezca1l0TtZ7gu6tm8z9rmwKjniOHPKysD/OulbJer
BzLfFLXvGSxP8zswG6TuX2gnfqbZLljhb6dEjSWPE1vsdbCY094e2oJ8uLU2
eld9sR08znUZavLcX4dPEvNWFeeflPXBqGd6IRf1yFhxe9XzqM+pfeEe/jrC
vuAPeRSuEoMYa/xS+bFx1v/nRd3/hk/CdziJP+AjrCkephwHidnUqGdkzbFt
/BcfvEXdZ4t/L+UkjvQUA+JE72j4+BZRvt9FPfANan/qtvvFFPxvEffcJ0l9
Rf7B5vCXmJ77E6jViVHbiwt4/l2ZsBU58Tr17y4W8HhP9eFccs1Q7dDN75/R
FsQI/I+aB/60RqO+xkfw0dXE62d12UlM4OkGYgVH8z4IODJXrMBpor9cR76m
RiEHD4/a03uxMg7QXruJCxgSd7F7rjF9Yl8dlYMfuEZ8miQmxPOxUetio9WV
a/H5zv5FhvntizYW8Bz8Ie9Z43P4PMHzu4kdY55No+oGMGONAD/E94dEza2c
E7V3E9nwp3ZR7x9AL3yBmr6D1/aJhm9cqezEF3gzLBo5g/zLGIL8/nmU/+ND
L/g9uMENxr3Us4zBGaferkxtxH7hqP0Me3jO+1Hz5XlvCG2SaybbVz6LCRn+
qN2WUpcTlBmuDVYX5lkvj3pm09Xq/oXH7cTpCrFibHa+GFKnDRRbeHJ81HOD
D9QW1OG7aCP8dAttBxfW1qZgmete9EWtBQewO1zpHI24OsVzwS/vfacv6qzT
lAts4Md56tdR22VNfpU2gsPExofE6H2xgPMTbAMeESeGig26Ynt8Z1f7xqce
F1/kIaYxrsh7BmkPn2D8eab6Ha+t8l4V7N4aDe6l7xOTqNXgNTESfjxo++R0
Ygj++5o6bafN0Hv7qPfM7Czex3st3+PPcOqmqPoSHPJeYXzzWuWinsg9ovQ1
QtleiRof85fYv4NYgN9g8YSXxBNyN1zu3YQV475+YtVN+ai34BZxNu+VXkF8
4FGuB/9OWxNP+4pNf9uDzz3FOMe312kzeEUOYE51N+VbTHlm+/8A23tL3cEJ
zqxl/8MbIv/nvWoLixUxi3hM7iHP9VCHy7TPu9qKHEWsGhYNfuFn92gH8g01
NbGfPD5W/cA8n12BHy2kjoznct8M55Mf+WmJRl2Dztj8YmXGvo+qM5xifon1
IjiXc1QP2R663hy17pnrtFkbIxN58W1tw/EQ5eEccmTmnqxBaXMLcaEvcv4G
yoD9DlM25M+9I5dpg121x73q+KJ2Qlbiwf1i8p02AitquhvEkBrkErEF5+Q+
1x4T9W6EfbXRidqNX2y9oTZlfiL35MMVxuZwAF9eXJnhO/UXnHlAPJD9OTEY
EPXcwNyfxVhyN4+RY37Pwa+X8tpP1BG9x4sZ3Gb8gC+crj3wEXIEPtsajXpq
jLj/3rbwNWoy8hNC48sjlQfMyS347LZ+ju1P9nx8nNzwvP2h/6tRMQEdiBUH
KXOu7aNj3k8IJhdq28W0x53ieqJ23so+t9RWg7QB1+W9ffAYP8w8eZfnEbse
097I+GLUWiHydY56B3SuS8GdI9Ux19TACA6TP68XK+Za8HnyKTVC3itN7M39
JGBOnCEm7h9130yuC/Sxzcc8L8e68AafZSzAGBu/2U690GkV28q9f+D7jXqR
i8+O2ruBbtQo1OofaddV7WuwuL4j9sRUchJ+0k+84HSLNsK3L1Lv67QhXCLO
X628o9UBWzEnckHUGhhxi5iAv/xWGxMLGJcTx/pGPReXGM7YlFxB7dTVz9Bj
b23CfAE+8FTU83+IEfCe8QVzC8Q6uNJT3ahfiTE/e0xen99zujRdC6/uj3o+
Sa7H7Gq/e0c91xHZ4NtwZYaXz3oNa2pvqiM5M++Hxb9fF5Nxnn+Bcjwkhidp
+6u13UVR75Qc4P+vihs2wgd+o1zg3Eubcl4Pbf2Yeq5qm6tG3Y9JrQRnci38
V1H3J37ica5Tz/J8OAkHicFw9SePF2xqE25/Yzt9ohHPqDevUBd8D75Sg+Xe
TvTKezefiNoLSN1D/ARc+Iu/4pvkkUlRz8r9xO9oj3z8qX3SB76/f1RMgHtv
iU0b7XKXOL0qruS7l7UtWI/R5sQr+Hmy+nAteZE9dLm/Cf2x/2ViQ510o3jh
G/f4+WB17R2NGPmysg0XE/ITcf9zz4fbxPMhHud+4QfEBFnviHqvzbXiw7iA
/MdcEnMMJ6g/8zHE0Pv8vIcy4cvkNXITPpv39SEb8ZYf6lz8c0f7PcJ+kR0/
/FEdqDH6ak9skvv4R0XD3rQ3LBo1RM71E/uIM3+wXbAn3p5lG8Trq7Xfsep8
iec/qG3OsW36vsb+Dojap/B/tksbuU/obnFARvyOuLGufRPrqBF+EfUuzA7i
8Kz60ic5knHQpcoGdk9roynabpi22V49evj5JM/PPTkct1GfHtr0Ja+lfso9
LcdG7XvfVXk39bo7lQ3s3o6a8/hWXTKGHeP3M6JizvdikvtawOog+wVDatWH
o56bdr2Y42/nagsw7KeNBmiDq/1uF+Uib/SOelcNOWI7j+HSalHrqmBDHOsV
dS8E52Jn/Gdn9cUmcBtOUtcQv+EqHCNG8kNO2Vfs8H/iNJy/1b+D1SnXEq9S
HnwHvuHn+BRcIE5OV5+29kuOneoxYzPGkTPFgZzzg8fI97nnIMc4r82YQNyh
NiZWEF/wZ2JI7sOHF+QRuEvMgXv7eO5fxeEU7XiDffZRf+wEt6jr/qIerdHI
Q9SgMzwPrJk7G6UNGL/lHsZttcmW4vml8h+pLeAW/JjludSsOcdO7p8jrlzz
szZC3k/FAx3HiC1+S31zgm2RMxnrdrWNk5W1TdRzSJeynxHiNli8kC3ndfET
fJQ5CmJq5hZqhcnaYVXlgh87iNnZynKpel5sP8RW6tQPxf9xddlLXai58AV8
jjno4zyX71u0Ab4MB/BFeP1Q1LgYDsIT6lJ4RN76rbLjWwdqD+TL9SR0+0PU
M2cHaZNcF8OHnot6Vx22YF4f+w/xWnJeL+Un988Vk8xB9Ld81L0W07wWXn5i
m9QnPe2L2Ps3ZaDN8cpGO22VOWtYPoMbXdQR/VdT97uUNTGZI1bk6k/FED97
0c+JR3eL+W/U6211P0MbYd+jtd047YVNqZ/xr4ttn9iSdT1cghvw7NCoefjj
1ZF48kftRF4arD2ujHpOz+XKcY99EGvgMDGAmrGNWJB74Dz+RIzFF06O8hFk
760uxO01xYJ56YW1xXZ+lj7IODLnZ4gBeQ/W2cpGfUdM/1IdyEMfiX3GBI6J
qaOinusMh3JPPe2uZZ/EHGrvXG9BXnLUAO04W/2I3ZPFgLEJXMo5dtYy/q5s
jIngxiIej26oM09G6nJ+iMOt0ahZafNr5ZwhJjuKD33DvXZiMiDqvl3kJubk
vAYx9GNxYj6L8fM7Hl+pjdfTDtQJq4vraVF7FdKnmNPBtuQ2/Kcl6hllXEuc
ZC6e2L6T147SHuA+Rf2Ijzl/cqI2y3kr6uK+2uVvUWsD9Lml2FEHby5exyjX
11HP2kd3xsJHKTfXEaNzzucG5ef48aj109zbtJXfIxc8WlHM9rQvfI3YSHwh
XsBvfJhxGrErn8OAP8GpOfb5pnqxRvVG1PMtqWdmitvTHuNfE5rOafXaJbUL
bTLn8mf7IgZ+pgwtUe/YQu/tlXlYNGIxulCv7aWOcHSHJt1zH9nt6p5YzfRz
5g0+EFsw+ZeY3+gvtoDf52ujp8QC21GvnKhNr/cYvYgl1PFwAC4M9rtBfgZn
4B4xawWPie3hOeC/sfrS92yPn1J+jkdEPQPmVq+F2xeIF/YkNuELxE04lPU2
fojv7Cdm+NSuTbpvpQ70u4HX5PpX3uu0UtSz2XJtH6yICeT11mjE4uaY8EFU
rCC2EEMWsg3sxjiGuJbPUvhV1D2AxPbTlQc9qDWocYf4mzH/2qi9zPhlrjeB
17629Zy4Iyv5iPH5N8rEMbaldic3vhD1rJQcM8PRR8QE/85x4n3aLue76Sfv
XcRH4POpYoIN816fX0fdC5Pz6tk/3GHcxBiiJRp1f3iMPMSp5v3KxEZquSX9
bGNxZAxNnqEGyr3X4AgvyYPUPX39JdeT43prx5fU+WvPJa7iL9T45Bhiek/7
Ss7AJ/IQ8eEosaLOONdjrrvFY2L8SI9z3wbXEtfz/r9eUc8MJo4yB/6+NkVm
7AzficOsf8EP+Lpa1P2otE/sIhaf4jnJyUG2N1rb5Fw8OM/xWvh2iW2e0vjo
P33tpAy5tvu++J+lzPjWOVG59gx1xJ+OadL9N01y9mjCZwmPGb9N8lr8/gXb
pI65S1uQU67SRuB9obbDpudrU2xyubbGv28SI44Zb17pOYxD4Qz+B29+ZTvw
c0n1RT+4BxcnR3GSui/EEPsO93NwbBu1VrCU7TSvWcwQO74nj+A7xDJySeav
A+1ve6/FB6mv8l7CvNcQ+3UWA3yZ2uerJh2Q/xNloo5627/w57mo56+PUFZs
T2yBI3AY/yau4MPEoo2VBV7sGMWjDbQZHMLOcI24Rf2Df/5DeXLvNvURsWWM
uBAXiE39tAE5cm31/169O4hd7ofPdaJRUfelZc0AZ+EH8eUC5aTvsdoA3eAD
+fiEqGcdd1du9CDGZ75Eb/yPmn+qbeY9/S1+t4q4IAO14em22c3Pyel9bW9l
sSE/tUaDn2eL4Rsew8uJUffCf6+uua/43KgaY8eoZ+rS30oeE8/yWSisYTMv
gU+tGLW+v7y2zXnZ28WQ+YmRHtP/W+JPXIJTxFzi0hTbP0m7zhXjNaL2ilLD
3O9xc07fVJmwOXzAB4ZG1VZDxS7Paa51c09O7l37QUzyPsU7/D7HmPcrM7rn
vBUcuC7qecZne0w9d7iYwO3eytQS9S6ba5p0Jw69Lub9o2Ly/uqb9/EMs014
fac2hf/E0ImeQwzI/U4/2BZjR+ZUL1Mu7NRX3ZEbmzPnSQ0P905TPzj5J2Wb
rH7oO1WsiDtw+zzbhPNwO+8PRy9qY7gCR9J3OMbHXvaYc7+IWs+d5rWrNeHz
S48viLpnk5h9oDicp13xffIJticm4EOMjcdoR2qux8UBHv5DWYgJ+O7aYnmJ
+ueeR+a3RkWj7sj99ozbO/o/tllGXPFnfIhcQd2zn32SM44Szw/FZax9w31q
D2IucxTYfZY2RW74QK1BXoQP+OVsz++pbOSh3cRllO3vr5xgQH2/b9Q7c1eI
ekbugmKLPa8Va2IHMR4/PlTMLkx5ovLZyn4Gb+Bdf3HAJtSejOOo/+9qqDFP
f+xB/mL8T06GW1Nsbyd15/xOyjIo6tkO6JHPaFhFWXI9gjiftfdG6re4epPn
qEvIqdSXxBjGY5+oK7XCeHUZJU7wH55+ILbYK9cW3vN4Uf++7zkc44dHe0yu
ZLyF78KroWJzTNS77fL4saZz3vfaoz2mze3sl/939TP6gGNvenyM+CIze1ue
UBdsPFwdqRsvVXf0PUFM8OsdxOrUqHfvHCiu/DJOy3s1iW+5jg+PHtZGa0fd
j99N2bDpxrY3RRnIx3BgRzGCG3CIuAZniDE7iRc4bOb35F7mRuDe2eoa2pPY
ClfhD/VE3jfO32uj7jNeMGpf7grqBof3tT3GC/jOTR6PisbcNbjga/gEvooP
4t+3icPmUXU7sWisul8Z9byxXh6n77+ljdaIihV7Ky8xBD8iR94rFvB1WDRq
63WUE38kHw/23GU9F/t+JzbEfPhDfOmobA8oA3MRTzfEnhfL3xCjbsoKT8kl
cAOe5L7G5vsuiD3D1R3cblT3HFdQW+a6NnIwP83YkvwKxxg/4nPU1v3ECbm7
K+O3DTPPqyvRHc7O1R7wiDwyWbnQnzlXxqiLKjN8zX0eO0e9C4m6mZxP3ILf
+Dwcg/8Ttcln2g5+3KJe2LqP+lKz5b0P1Ohzot5byHzCVGX5SnssKmZ53yV9
tUZjzLCN+HRQPuY28X349rxYgvOT4skcCLme2AiXyGsXaFNy8IpRHDgp6v3A
2Ih65WhtRPvMn8Bx5nLxtWO1C3plfZXH+3vOceq+te2ca5vHKAt94adPKAMx
brSyXaT8/9CmI9QFHf8e9aynIep+wP9gsrVYoUfu3wPLH8QWzN9Vf/LCi9oi
9+Hk/WITo8YFi2hT4liPqGdrMfcOB+DQCV7DnCRchTPEBPgBl+AxY6U/qse/
vY5YyHwX9oenxDq4Ci/yPr2h2hF57hT/NaPmxBb276ho+Ah5+X2/z/lyfIo4
tLj4vKae39pee/UidsxW32PFiv/JA7dEvcMczHO+Gk6PV4eMFbkH6Gk/a22S
lTE3MYdxRuZdbEWcPk+c7o16pwx6EceJ/8wf3a4e8Crve4D35MctxW9s1H0s
E5TnDfVd1GNyMOOajBtnRD1Ls0808hPn/Eo7Ep//GfV+NnIFfgFvwYp4Qk0H
/7+PWqMn7vFDjCTWna6s+M6q9g//D9Y2t3t+zgcwv/yS1xzu9ZPtv6+2y72p
XcV6r6hx02nqP0tcX1LHnPub67nbKNMQMYI33ZXvKz9jXu7ftkcce7fR1Ly5
ADiPr6yvzchFzDE+JA60e6d6wgV4je9Tt1B/wFX41RK1ftdfmXtHw/4t6r+D
bWLnbn6eMQpuvqWtqAuIX3mPS0+Pj4maY0f2jF3kxI+b2twk6r3DxysDv8lN
csXVnottr1EX/PAidSRmnKLu8G1vMaFGXU+sFm3CkDw7SWznj3oOIpi/ry3A
+duoe12Xj6p7d4iKXYzlco8lcsEBaijGf3DjJH/BAj7A+We0CTkOjhGDmQeD
e8ReYlQXdZq3JhX1bEdsS66ZY//fKNvpUXNPoQ3wqV9E3fuR+5Cxxc3ai3E9
vvY7ZZ+kfg9GvdvraHVcJ+r9SsT8nz0HGT+J8n18Z0LUXgI4hN8Tu/ExaiF4
SWwhpzN/iE8e4bXXiEeOTS/zOuYZOms/7I5fwqm77IcxJTX9ger0YtRa4XtR
z5wBd3JkS9TzvzeKevbCJPWlr/uinr3wV3UG63z2AX7GfAg5dVQ0chQ8INfB
N2pici28IR8OtO18JsXzUesp1EnUyDeKaTtlynv33vc8ailiHTmReE69zZht
hLbKewjp+0Ptsk3Uc3bJ4wsrB/h+5XfYArvDM3JmP/G5UIyI9ZuJwUfal/aJ
9fzg1880mp3Hi4eVm1wKB+HV/4ndldoVH+6iDllnTbPvG+z3IHUj3++q7jcr
L9fkva3kY8ZmcOJI7QLPGPO+FfWevd3ViT6O8BhfyDHCMlHjx+uj4h5+/Jh9
gfdCUe933EzZyFP7KjP1S1/7Af+DlXd9z0V35l26i8ly4nWncn4rhguJc2Lb
jPkUbUEcW1UbtVGGH2xziDYlTsLFucrCMQ3SNzyaad9bR+15Bg+4RB34F+1A
PbO31wyNeu8eNV9vdQBv5uza2ebFynqp/U7TjuSe9IXc/zBAfOAv8fAnrztC
nJ5SHr6fIM6do8FLPu8QNbe5njpy/SHqSA12tfgQj+EeMYHxEWN6YgVcwoeJ
IfCB2gzbk4+II8Qc4vAvxeD4qHu8wIKx2WV+R97KvVvYkrmPbtqNepu80xr1
vAjyzYdRzy7CBrknB66vrR1meXyvOlKH8Es9mfuvv9cmT2lHdCTO3K/8WS8h
C/Nfm4h//0aT87C/STkYc1NHMWYjDlE/4Tf4wkXKjk3xD2pY4j61UD/P+71t
9ImGP1AjP+Lxl/bBeZl3cn64eT2OvE+OyP2L46L2ylL3f6EO2Jp5BbjRSVvi
U98pN/5Czs89b+BETMn7HXM/P9eTK+DLVeqQ/cJv/IB5MeLDe362refDEWrW
s5WXXDtQG+xl+/DqTPWmJnhS2Yj55Phr1Gc3+x7oMbFoatTemnFe21U9zxS/
/exroH1tqr3GK9sg9d/W9ldWl9zTdJj9tYt6T8qsqLnZT6Le3/5m1H7CrA/x
ia/FPMfd2AKOH6iNyA/XaTtkfVqb4kOjo551AqZzlS9rsBzvwxnqpr/5Pz4N
z+EYa0HUqLcpN3mln/ogJ1w9RTz5wR9+jOL2U1GcRw54ep7Yz2eft2gn+s19
Al+IH752qX3nPCO8h3PUF/g3PrtL1DrvLX4O1vjjPcpEvjnI79qLdc43tkbF
kNwHB5eWi3rmTO4jXU5siVE91ANeM3dxpPrgB48pP/HwvajYS339hhjCY/hE
bUj997V9vSqmzAM/H7X3bIo6/j3q2dr/EO+vxPAkr0dexpzEoEPFD0ypfah1
OmlP+Els/TzqWay5f4G+4AdxhnEwsYp4mmvH+Mbu9rmTtrpB+42KRt1Hjide
wAf4Qly8TRvkPOSG9pX3veQa1iPabgdthe3gMP4C/89Sv1fE+SBthy921nbo
MTPqeUFZn3SxD65j7gP/gdv4E7wkz+d8PDoSV7byl7ob3+0hJp9F7c/Fz8mv
cOF8seH60crK+W9pd2w4LBocBpNfRM3FXOjxKVHvzsBu73otdfsE2+zg//QF
v/ZUBuLbBcrG3/uV+RztvJV9vaGM+OhL6k58fkJMqGseFCviRs7/M5f2nthy
vJSY09//aYuDm2zUEo24gu3I469r03NsA1ufGvW+T+QhJuX+DWpcOHOY58Ml
agbiAhyj9iFWjopG/M77DI7wGB2JbdPFlO/w4xXUexX7mqu8Y5QZTBdSFnJz
J2VeUlngSC/1pGbDX/DBYzwH36ReIK7js5tEPduHvBNR7/zC9/D9Ff2lbepc
bD8+6pl72OxYsSS2MH/wvDb+s/IRi/rYHzEKPsJtsNxcnOFdrg9QMxMPiBV5
v1YbdV5EXeEcsesBZd1X/KeLJfVEjvtyP+svlXlq1F5AxnvUeLnXEFvD7UvE
aVkxulYciI9/VBYwg4vkwQfV+TPteZc68TPQ7xgLMaYhfuZ8yYZR74nGdm/7
OTpTJz2ibLPsY13tBY7wDn8kXuUeuKWUF1svEfW8NsYxxKvc90utQk6nBiB2
4y/UCTluw5fhKvUwdQScOFN511JGdN9H/fHF621jeNRzjcGDmAZ3iF353DJq
Kuoccib8OUjsc62GfnOvK3KeoH3y3hx4RS0Iz3PcNVDsMy908hz6/8Br4d5L
UXsXJ0TNSyymDC9EPQ+WMdv+/n+vduse9V5cdCQGnx51L8qZYgJPLhKr3HvP
dy3ic7G23VnMcy0eW+Qc7Ei/T9vx+bXaFB17ausjxBoOrOd1cAPuExvzuQfY
Ei5tFDVfs73ywT38m9wBJx8Vi/7KQjxfJ+p5l/zgQ+3FHR8j3j9o//gQPkIO
uU7ZTxObnaOeZXmJspyijalHqG0nai/qB3wWjg2K8uWXo/bDEJ+mKz9xgJjA
2PNYbcw4PWMItRZ8yNhCzMtxep+oZ7gSd4lRvT0XHfAL5sLgOHw5W5xyLelF
cRkdNZ7oo23woR/8pQ24RU2QPEUYfCjrSHLEeeJ9d1Schc/E7wfE8lRxPU9c
yI3kvF7KMkgbtHgNulzp/7S7judspY3QYYRyvKQO2Lc1Gn44Oup+C2yEny6g
jXJuKtdiRnhMHt82aj/VZH+pLVkHyflbOArX8WHyx9PKBv+Ix9SQ1HXDohHn
4QD8gc+M7/Av+H9k1J5l+ia+DBDD3bTHLepMPPlJ3NBnH9t7RPkZl5BryAXE
aWq/w6PmgrAZnCGPwSNiKzUA9XHu1UQefPYxbYKug+17Ae23uucwvnzfa3dX
hrwX/GH7Qo4xynCbGPJZrqX+WpyWjnpeayflhXPd1J18u7OYUHtn/UrN9YoY
om93+2IO/CIxp95/TlvAodHa6LyoZ1Tt2GRTfKo1GrbGB6ml4UDbqHfrtNNO
cIbclHMI/ZQNjhFfFtNWz0StjxLPiVGXiiP+D4eJz8QjuM04YIry5n6tK8WN
ON8SDV69pZ3eVp/1xeQ6bQXm1GH4083Kim8SN1eyjz211xyvp9bJ8UXe47CC
+hMTct0QbsIZ/JQYAj+IWy8qP778qDIT14hFOed3nf3z3Q3Khe/g5z2VFT2I
IcRm/Cfvx/qgId48vCdpk5y/gyNwd7Y4X64utEkuyJyCD7KmsY+4vSFG6IG/
/V6ZqEcuVn/kPVXZF/M6MMk1E2SEP+co3wLiTOzL+xTy/sLcywEmcID4jO/A
D+oW4uz5YpH7zK9XF8Y5n0eN2cdqF2JRf+0yyP6pEReJ2rOEXMR06svc44W+
Lep+hbK19bMLvQZ8t7W9Q7QLdQYxoF/Us9WYm2ftY0NtyPwrnGIuYFfPwd/w
T2rhw22/v7ruqgzYFN4TuxkfXapexKfTxAFbkHNfFAP8Nbx+gMdnRO0ZwdYv
eC2x95qo59eca1+DtR9tdLPf/up0Y9Reu/vVBQ69EvWuwWnqPlHZtlbev4kV
8W8xMdxPrMEW7uV9D3+1ryuULe+/Y/1tfm1H7so5ntzbg22w+VHqtVrUfS5g
SZ6AM8y37CkOFyn/bHUiRsG9Y8QDTu7lNchH7CZXwWFiLj4Ht29WJ/rHH/Dx
N5X7Wz+jRt5O/bED/rOSsnFt7l9NHySHkhPxTWL4xuKNXGdHPSf7E3Uhfk+O
uq/toqj35dJn7svCfhlDMrbgO8RwYg41A7UYsYhYTH68QTzBm3FrO3HOPEJd
x7hyIWV43fbJ44xD+4vz18qGL2cdgq/leDbXSqjTPha3E7TXZ9oe2zwrZtgq
9wCT/3LvPnOd+D1xDl8gpo9TBuJgixjnvPPCTbajnoG31HXw/0LPma6O8Ba+
f2q/YAXv4cWuyn9r1LzxmuKcc3SMHRgL9Ix69xQ+PlxbIBv6P6bdwPMOj+dX
p03FAb+FN/gbc234CzyhFsw1zJxnxi8ZQ+Bz7W3jVnXvpj7wcUt1RRfqmtao
dfODoubSyIMdo/bAbK/8xC58PdcLNheHXMOnLmB8ebf6wLndPKZd/CXvuR4W
9czYo2yzfVNfXZv0OEYczxKvnZSZ2uM4dWFsdos6TlBPdCd/HW1bM/yOPvC9
HcWQeHSa2B4kpmDOvprltAW5682o+9reVZZ24o9Nsfudyk/9gL9/qM2JP19p
R/g7VRz+JQ5riE/uE8AHpinDWdoQGcnXcPVp/7/QvtAPbuM31IkLqBc5GF8g
b1Kvpo9sov3I+bdHPV8XDsOtb6KeMTZezJ6NWh/7TPwOEevNo/ahMe7ZTr1o
f6T6biIexAp8t59to29zbPku6tnZ2IFY1EkMiFH4LjUj8auH5z0Ytf8x13mJ
ueRP4vzb2n6g9vzWaz9TfrjHWIvaa1914xguM39KvCWew8u815L/iUnMM/SJ
ek4XtWve2zbvXpVo5OurxBff+cJ+b9Qu7cUVe3ewTWoYuNhfe+IL+OgB9oF9
t4japzo8qqbKPXLkE2Jr7pvKPdI9o96lTNsb2SZyPCz+1Inw517tABeI78xp
rBt1DwTXwAv2wOCX+Bu1PHzfT7mJZfgtfL1BPeEtPkvNQhymHoUjr4vlr8Vg
kO2hC7XNyV47MuqZB3ADDhDz5oh9GzGYbv+cMyPqvge41Or/nEvezfvHsOM/
PecIz8k1yC5RnKF9+oKD3ynD4p6LbN2UD5n3th10ocZ7Vh2Zk++l7mDwiJiQ
r5YRqzPEHAxzfnI/23pczHdS3txvOFEbgfmx2g7ZX1F/4sQD2roligO55wdu
ME7M9V3yFLX7TOW7X/1vVkY4Rz04LBqcfCpqLNfXtuBCH/VNblPDdlCujuJ6
vf1MVxbkH+r1+H2ICXLga/ATfgz0HM7HN6mr8fuPxahzNHwZe+ReDtpYMuqZ
qkvaL/H0WnXBzz6wrYWjYkvOsRJz4ADxNXl4n7KCDXHwDtsnFj6ibYm5L0S9
z+BddXjBdrFR7pOk7WfFBb/rKf7UITdHPYchx1lbez2x73TbR4bfew7Y/MXr
wB1Obhb1zpofbZ/8Qy3PmG9I1H1tyJn7KX+ImpMm/8FDcuJBysoY5je2mfcV
9rb94WKP3xKfyA3Moy6v3Rjnk0+In9R6LdGIWS/6lzHjCGU5XPnwI2LsFeI3
xeu21Rbow/wLuYHaHx6epIx/ss/e6oKeG9o/NiFvEJOoDXqJMcfURm9pS2IG
Y97JnkvMm+jfjdUrc9w7fregn2GPrsqJ3HCoTzTyHjUzvkK9+Z19cEy8yboW
G6/rtbuLN22urF2zrwlNMiwStTa6hzLneg26UBOs5XdgPCTq2TVPiQn4PiJW
Z/t/b8/dT2wf9Tswf8J+0xZXeE3uqc09JDtp0/biNkL5kRsO4Gu5p4Q484b2
o+4hJ8ElYgU+CseI4Tm2ovboJ0bIkPdIP6yscBguMY/1J78jbsB54uy8/di2
n3tWbhLn5DZ94VPES+rVjspL7D7SPv8W9b4Hai18ljw4Khp2HqFtsCN8PN/2
8X3snPe3EmvwxaOj3lXeVpmILfg4se2FqPfzPOLxP5Vhu6hn/THHwLwqOQbb
U7+9pM7YnpgI71jnnBT/HWPz/gVk3lBd6Y+Yeokyg+vzUc82gRtjlAO+XqUs
5NDcz7aLOj2tzltom7ttn/oO3+kdjRxKzCSn5zMmaAd+ke+oJ6j7mXvC15/S
LtSo6QPUBeTmZ2yfnD5B+ZERPpEf4XRr1H5+8tubfj9OW8L3K5SVdhZVVnBe
TrserS2wFbyHa/gg43H4uo99vS1mZ9oeNfhYZadtapg/2D99zK8tT1Tn/W0n
73cdbv9DxOU4P6P9bcQHnNpH7VFJnhPjb4zaJ7Gf+OFH1A2MW+DOEx5j/wGe
g75bRz2nZQ3bpN/V/XzzJhlWjnquKPniFWWm7l1RXfZW1/09d/uo9YGVxASO
/ihW2L2d/cDBy8R2sPjnHNtAbUENvrE22sTvwA5/bBVv4uNZ2oYxBXF+VDTm
UpIb6A034Aw+CI55H2jW7f3EKvfJ53wXeeFR8UHWPcUHWdfWhrnOCucZU2wb
tbeIurV3NHwn52fOVU58ao7ttfXaw2wn93xc5We0PyZqTf1LsW6j/OjxmO2v
G/VM8ZyTwA4niQMxhFxDbCW2UOM0xxzmEx6P2jNPjIIX/w8FXIx1
   
   "], {{-3.3415926535897933`, -1.7707963267948965`}, {0.2, 
   1.7707963267948965`}}, {1, 256},
   ColorFunction->Hue],
  AlignmentPoint->Center,
  AspectRatio->1,
  Axes->False,
  AxesLabel->None,
  AxesOrigin->Automatic,
  AxesStyle->{},
  Background->None,
  BaseStyle->{},
  BaselinePosition->Automatic,
  ColorOutput->Automatic,
  ContentSelectable->Automatic,
  DisplayFunction:>$DisplayFunction,
  Epilog->{},
  FormatType:>TraditionalForm,
  Frame->False,
  FrameLabel->None,
  FrameStyle->{},
  FrameTicks->Automatic,
  FrameTicksStyle->{},
  GridLines->None,
  GridLinesStyle->{},
  ImageMargins->0.,
  ImagePadding->All,
  ImageSize->Automatic,
  LabelStyle->{},
  Method->Automatic,
  PlotLabel->None,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  PlotRangePadding->Automatic,
  PlotRegion->Automatic,
  PreserveImageOptions->Automatic,
  Prolog->{},
  RotateLabel->True,
  Ticks->Automatic,
  TicksStyle->{}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"AbsoluteTime", "[", "]"}], "-", 
  "$starttime"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{
   RowBox[{"MemoryInUse", "[", "]"}], ",", 
   RowBox[{"MaxMemoryUsed", "[", "]"}]}], "}"}]}], "Input",
 CellChangeTimes->{{3.392469818705207*^9, 3.392469839226327*^9}}],

Cell[BoxData["177.05206`8.699645977634033"], "Output"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"31735584", ",", "364864960"}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
AutoGeneratedPackage->None,
WindowToolbars->"EditBar",
WindowSize->{1035, 971},
WindowMargins->{{7, Automatic}, {Automatic, 0}},
Magnification->1.,
FrontEndVersion->"6.0 for Linux x86 (32-bit) (June 19, 2007)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[590, 23, 47, 0, 83, "Title"],
Cell[CellGroupData[{
Cell[662, 27, 28, 0, 38, "Subsection"],
Cell[693, 29, 115, 5, 83, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[845, 39, 28, 0, 38, "Subsection"],
Cell[876, 41, 178, 6, 29, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1091, 52, 30, 0, 38, "Subsection"],
Cell[1124, 54, 321, 9, 29, "Text"],
Cell[1448, 65, 120, 3, 29, "Text"],
Cell[CellGroupData[{
Cell[1593, 72, 337, 8, 52, "Input"],
Cell[1933, 82, 253, 6, 31, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2235, 94, 31, 0, 72, "Section"],
Cell[2269, 96, 143, 2, 31, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2449, 103, 29, 0, 72, "Section"],
Cell[2481, 105, 86, 2, 31, "Input"],
Cell[2570, 109, 472, 15, 44, "Input"],
Cell[CellGroupData[{
Cell[3067, 128, 675, 18, 69, "Input"],
Cell[CellGroupData[{
Cell[3767, 150, 150, 4, 24, "Print"],
Cell[3920, 156, 208, 6, 24, "Print"]
}, Open  ]],
Cell[4143, 165, 35166, 601, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[39358, 772, 29, 0, 72, "Section"],
Cell[39390, 774, 86, 2, 31, "Input"],
Cell[CellGroupData[{
Cell[39501, 780, 1086, 31, 86, "Input"],
Cell[CellGroupData[{
Cell[40612, 815, 150, 4, 24, "Print"],
Cell[40765, 821, 208, 6, 24, "Print"]
}, Open  ]],
Cell[40988, 830, 67566, 1133, 375, 33791, 578, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[108603, 1969, 29, 0, 72, "Section"],
Cell[108635, 1971, 86, 2, 31, "Input"],
Cell[108724, 1975, 472, 15, 44, "Input"],
Cell[CellGroupData[{
Cell[109221, 1994, 843, 24, 70, "Input"],
Cell[CellGroupData[{
Cell[110089, 2022, 150, 4, 24, "Print"],
Cell[110242, 2028, 208, 6, 24, "Print"]
}, Open  ]],
Cell[110465, 2037, 85804, 1433, 375, 41976, 713, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[196318, 3476, 29, 0, 72, "Section"],
Cell[196350, 3478, 86, 2, 31, "Input"],
Cell[196439, 3482, 490, 16, 44, "Input"],
Cell[CellGroupData[{
Cell[196954, 3502, 871, 25, 70, "Input"],
Cell[CellGroupData[{
Cell[197850, 3531, 150, 4, 24, "Print"],
Cell[198003, 3537, 208, 6, 24, "Print"]
}, Open  ]],
Cell[198226, 3546, 88354, 1475, 375, 43993, 746, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[286629, 5027, 32, 0, 72, "Section"],
Cell[286664, 5029, 86, 2, 31, "Input"],
Cell[286753, 5033, 523, 17, 44, "Input"],
Cell[CellGroupData[{
Cell[287301, 5054, 655, 18, 73, "Input"],
Cell[CellGroupData[{
Cell[287981, 5076, 150, 4, 24, "Print"],
Cell[288134, 5082, 208, 6, 24, "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[288403, 5095, 32, 0, 72, "Section"],
Cell[288438, 5097, 86, 2, 31, "Input"],
Cell[288527, 5101, 523, 17, 44, "Input"],
Cell[CellGroupData[{
Cell[289075, 5122, 921, 27, 70, "Input"],
Cell[CellGroupData[{
Cell[290021, 5153, 150, 4, 24, "Print"],
Cell[290174, 5159, 208, 6, 24, "Print"]
}, Open  ]],
Cell[290397, 5168, 23580, 411, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[314026, 5585, 32, 0, 72, "Section"],
Cell[314061, 5587, 86, 2, 31, "Input"],
Cell[314150, 5591, 472, 15, 44, "Input"],
Cell[CellGroupData[{
Cell[314647, 5610, 843, 24, 52, "Input"],
Cell[CellGroupData[{
Cell[315515, 5638, 150, 4, 24, "Print"],
Cell[315668, 5644, 208, 6, 24, "Print"]
}, Open  ]],
Cell[315891, 5653, 25087, 436, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[341027, 6095, 32, 0, 72, "Section"],
Cell[CellGroupData[{
Cell[341084, 6099, 775, 23, 31, "Input"],
Cell[CellGroupData[{
Cell[341884, 6126, 150, 4, 24, "Print"],
Cell[342037, 6132, 208, 6, 24, "Print"]
}, Open  ]],
Cell[342260, 6141, 27995, 483, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[370304, 6630, 30, 0, 72, "Section"],
Cell[CellGroupData[{
Cell[370359, 6634, 1332, 39, 106, "Input"],
Cell[CellGroupData[{
Cell[371716, 6677, 150, 4, 24, "Print"],
Cell[371869, 6683, 208, 6, 24, "Print"]
}, Open  ]],
Cell[372092, 6692, 38422, 654, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[410563, 7352, 30, 0, 72, "Section"],
Cell[410596, 7354, 86, 2, 31, "Input"],
Cell[410685, 7358, 555, 18, 44, "Input"],
Cell[CellGroupData[{
Cell[411265, 7380, 697, 19, 52, "Input"],
Cell[CellGroupData[{
Cell[411987, 7403, 150, 4, 24, "Print"],
Cell[412140, 7409, 208, 6, 24, "Print"]
}, Open  ]],
Cell[412363, 7418, 34544, 591, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[446956, 8015, 31, 0, 72, "Section"],
Cell[446990, 8017, 86, 2, 31, "Input"],
Cell[447079, 8021, 463, 15, 31, "Input"],
Cell[CellGroupData[{
Cell[447567, 8040, 1043, 32, 87, "Input"],
Cell[CellGroupData[{
Cell[448635, 8076, 150, 4, 24, "Print"],
Cell[448788, 8082, 208, 6, 24, "Print"]
}, Open  ]],
Cell[449011, 8091, 32417, 556, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[481477, 8653, 31, 0, 72, "Section"],
Cell[481511, 8655, 86, 2, 31, "Input"],
Cell[481600, 8659, 523, 17, 44, "Input"],
Cell[CellGroupData[{
Cell[482148, 8680, 940, 28, 87, "Input"],
Cell[CellGroupData[{
Cell[483113, 8712, 150, 4, 24, "Print"],
Cell[483266, 8718, 208, 6, 24, "Print"]
}, Open  ]],
Cell[483489, 8727, 10338, 194, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[493876, 8927, 31, 0, 72, "Section"],
Cell[493910, 8929, 86, 2, 31, "Input"],
Cell[493999, 8933, 457, 14, 31, "Input"],
Cell[CellGroupData[{
Cell[494481, 8951, 857, 25, 87, "Input"],
Cell[CellGroupData[{
Cell[495363, 8980, 150, 4, 24, "Print"],
Cell[495516, 8986, 208, 6, 24, "Print"]
}, Open  ]],
Cell[495739, 8995, 49802, 841, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[545590, 9842, 31, 0, 72, "Section"],
Cell[545624, 9844, 86, 2, 31, "Input"],
Cell[545713, 9848, 457, 14, 31, "Input"],
Cell[CellGroupData[{
Cell[546195, 9866, 850, 25, 87, "Input"],
Cell[CellGroupData[{
Cell[547070, 9895, 150, 4, 24, "Print"],
Cell[547223, 9901, 208, 6, 24, "Print"]
}, Open  ]],
Cell[547446, 9910, 43353, 735, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[590848, 10651, 31, 0, 72, "Section"],
Cell[590882, 10653, 86, 2, 31, "Input"],
Cell[590971, 10657, 897, 29, 44, "Input"],
Cell[CellGroupData[{
Cell[591893, 10690, 1128, 34, 89, "Input"],
Cell[CellGroupData[{
Cell[593046, 10728, 150, 4, 24, "Print"],
Cell[593199, 10734, 208, 6, 24, "Print"]
}, Open  ]],
Cell[593422, 10743, 185815, 3071, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[779286, 13820, 31, 0, 72, "Section"],
Cell[779320, 13822, 86, 2, 31, "Input"],
Cell[779409, 13826, 460, 15, 44, "Input"],
Cell[CellGroupData[{
Cell[779894, 13845, 729, 20, 74, "Input"],
Cell[CellGroupData[{
Cell[780648, 13869, 150, 4, 24, "Print"],
Cell[780801, 13875, 208, 6, 24, "Print"]
}, Open  ]],
Cell[781024, 13884, 38151, 650, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[819224, 14540, 31, 0, 72, "Section"],
Cell[819258, 14542, 86, 2, 31, "Input"],
Cell[819347, 14546, 662, 22, 44, "Input"],
Cell[CellGroupData[{
Cell[820034, 14572, 748, 20, 52, "Input"],
Cell[CellGroupData[{
Cell[820807, 14596, 150, 4, 24, "Print"],
Cell[820960, 14602, 208, 6, 24, "Print"]
}, Open  ]],
Cell[821183, 14611, 47082, 796, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[868314, 15413, 31, 0, 72, "Section"],
Cell[868348, 15415, 86, 2, 31, "Input"],
Cell[868437, 15419, 662, 22, 44, "Input"],
Cell[CellGroupData[{
Cell[869124, 15445, 837, 24, 74, "Input"],
Cell[CellGroupData[{
Cell[869986, 15473, 150, 4, 24, "Print"],
Cell[870139, 15479, 208, 6, 24, "Print"]
}, Open  ]],
Cell[870362, 15488, 45903, 777, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[916314, 16271, 31, 0, 72, "Section"],
Cell[CellGroupData[{
Cell[916370, 16275, 2081, 55, 128, "Input"],
Cell[CellGroupData[{
Cell[918476, 16334, 150, 4, 24, "Print"],
Cell[918629, 16340, 208, 6, 24, "Print"]
}, Open  ]],
Cell[918852, 16349, 107232, 1783, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1026133, 18138, 31, 0, 72, "Section"],
Cell[1026167, 18140, 86, 2, 31, "Input"],
Cell[1026256, 18144, 1168, 36, 72, "Input"],
Cell[CellGroupData[{
Cell[1027449, 18184, 915, 22, 52, "Input"],
Cell[CellGroupData[{
Cell[1028389, 18210, 150, 4, 24, "Print"],
Cell[1028542, 18216, 208, 6, 24, "Print"]
}, Open  ]],
Cell[1028765, 18225, 26852, 464, 199, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1055666, 18695, 31, 0, 72, "Section"],
Cell[1055700, 18697, 86, 2, 31, "Input"],
Cell[1055789, 18701, 1168, 36, 72, "Input"],
Cell[CellGroupData[{
Cell[1056982, 18741, 735, 20, 52, "Input"],
Cell[CellGroupData[{
Cell[1057742, 18765, 150, 4, 24, "Print"],
Cell[1057895, 18771, 208, 6, 24, "Print"]
}, Open  ]],
Cell[1058118, 18780, 35964, 614, 347, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1094131, 19400, 31, 0, 72, "Section"],
Cell[1094165, 19402, 120, 3, 31, "Input"],
Cell[1094288, 19407, 148, 5, 31, "Input"],
Cell[1094439, 19414, 329, 11, 31, "Input"],
Cell[1094771, 19427, 673, 21, 31, "Input"],
Cell[CellGroupData[{
Cell[1095469, 19452, 658, 17, 52, "Input"],
Cell[CellGroupData[{
Cell[1096152, 19473, 150, 4, 24, "Print"],
Cell[1096305, 19479, 208, 6, 24, "Print"]
}, Open  ]],
Cell[1096528, 19488, 243346, 4014, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1339923, 23508, 31, 0, 72, "Section"],
Cell[1339957, 23510, 201, 5, 31, "Input"],
Cell[1340161, 23517, 353, 12, 46, "Input"],
Cell[1340517, 23531, 362, 12, 45, "Input"],
Cell[1340882, 23545, 368, 11, 31, "Input"],
Cell[1341253, 23558, 580, 20, 43, "Input"],
Cell[CellGroupData[{
Cell[1341858, 23582, 634, 16, 52, "Input"],
Cell[CellGroupData[{
Cell[1342517, 23602, 150, 4, 24, "Print"],
Cell[1342670, 23608, 208, 6, 24, "Print"]
}, Open  ]],
Cell[1342893, 23617, 42865, 727, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1385807, 24350, 31, 0, 72, "Section"],
Cell[1385841, 24352, 134, 4, 31, "Input"],
Cell[1385978, 24358, 773, 25, 62, "Input"],
Cell[1386754, 24385, 118, 4, 31, "Input"],
Cell[1386875, 24391, 302, 9, 43, "Input"],
Cell[1387180, 24402, 927, 25, 52, "Input"],
Cell[CellGroupData[{
Cell[1388132, 24431, 241, 7, 31, "Input"],
Cell[CellGroupData[{
Cell[1388398, 24442, 150, 4, 24, "Print"],
Cell[1388551, 24448, 208, 6, 24, "Print"]
}, Open  ]],
Cell[1388774, 24457, 26183, 454, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1415006, 24917, 31, 0, 72, "Section"],
Cell[1415040, 24919, 154, 4, 31, "Input"],
Cell[1415197, 24925, 773, 25, 62, "Input"],
Cell[1415973, 24952, 118, 4, 31, "Input"],
Cell[1416094, 24958, 302, 9, 43, "Input"],
Cell[1416399, 24969, 590, 19, 45, "Input"],
Cell[1416992, 24990, 889, 24, 52, "Input"],
Cell[CellGroupData[{
Cell[1417906, 25018, 120, 3, 31, "Input"],
Cell[CellGroupData[{
Cell[1418051, 25025, 150, 4, 24, "Print"],
Cell[1418204, 25031, 208, 6, 24, "Print"]
}, Open  ]],
Cell[1418427, 25040, 47554, 804, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1466030, 25850, 29, 0, 72, "Section"],
Cell[1466062, 25852, 86, 2, 31, "Input"],
Cell[1466151, 25856, 504, 16, 44, "Input"],
Cell[CellGroupData[{
Cell[1466680, 25876, 708, 19, 52, "Input"],
Cell[CellGroupData[{
Cell[1467413, 25899, 150, 4, 24, "Print"],
Cell[1467566, 25905, 208, 6, 24, "Print"]
}, Open  ]],
Cell[1467789, 25914, 49636, 838, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1517474, 26758, 29, 0, 72, "Section"],
Cell[1517506, 26760, 86, 2, 31, "Input"],
Cell[1517595, 26764, 748, 24, 44, "Input"],
Cell[CellGroupData[{
Cell[1518368, 26792, 708, 19, 52, "Input"],
Cell[CellGroupData[{
Cell[1519101, 26815, 150, 4, 24, "Print"],
Cell[1519254, 26821, 208, 6, 24, "Print"]
}, Open  ]],
Cell[1519477, 26830, 41474, 704, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1561000, 27540, 29, 0, 72, "Section"],
Cell[1561032, 27542, 86, 2, 31, "Input"],
Cell[1561121, 27546, 586, 19, 44, "Input"],
Cell[CellGroupData[{
Cell[1561732, 27569, 625, 16, 52, "Input"],
Cell[CellGroupData[{
Cell[1562382, 27589, 150, 4, 24, "Print"],
Cell[1562535, 27595, 208, 6, 24, "Print"]
}, Open  ]],
Cell[1562758, 27604, 36231, 618, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1599038, 28228, 30, 0, 72, "Section"],
Cell[1599071, 28230, 86, 2, 31, "Input"],
Cell[1599160, 28234, 458, 14, 31, "Input"],
Cell[CellGroupData[{
Cell[1599643, 28252, 788, 22, 69, "Input"],
Cell[CellGroupData[{
Cell[1600456, 28278, 150, 4, 24, "Print"],
Cell[1600609, 28284, 208, 6, 24, "Print"]
}, Open  ]],
Cell[1600832, 28293, 54419, 917, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1655300, 29216, 30, 0, 72, "Section"],
Cell[1655333, 29218, 86, 2, 31, "Input"],
Cell[1655422, 29222, 457, 14, 31, "Input"],
Cell[CellGroupData[{
Cell[1655904, 29240, 651, 17, 52, "Input"],
Cell[CellGroupData[{
Cell[1656580, 29261, 150, 4, 24, "Print"],
Cell[1656733, 29267, 208, 6, 24, "Print"]
}, Open  ]],
Cell[1656956, 29276, 49132, 830, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1706137, 30112, 30, 0, 72, "Section"],
Cell[1706170, 30114, 86, 2, 31, "Input"],
Cell[1706259, 30118, 753, 24, 44, "Input"],
Cell[CellGroupData[{
Cell[1707037, 30146, 755, 21, 69, "Input"],
Cell[CellGroupData[{
Cell[1707817, 30171, 150, 4, 24, "Print"],
Cell[1707970, 30177, 208, 6, 24, "Print"]
}, Open  ]],
Cell[1708193, 30186, 156931, 2597, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1865173, 32789, 30, 0, 72, "Section"],
Cell[1865206, 32791, 86, 2, 31, "Input"],
Cell[1865295, 32795, 526, 18, 44, "Input"],
Cell[CellGroupData[{
Cell[1865846, 32817, 808, 23, 70, "Input"],
Cell[CellGroupData[{
Cell[1866679, 32844, 150, 4, 24, "Print"],
Cell[1866832, 32850, 208, 6, 24, "Print"]
}, Open  ]],
Cell[1867055, 32859, 29341, 505, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1896445, 33370, 30, 0, 72, "Section"],
Cell[1896478, 33372, 86, 2, 31, "Input"],
Cell[1896567, 33376, 897, 29, 44, "Input"],
Cell[CellGroupData[{
Cell[1897489, 33409, 755, 21, 69, "Input"],
Cell[CellGroupData[{
Cell[1898269, 33434, 150, 4, 24, "Print"],
Cell[1898422, 33440, 208, 6, 24, "Print"]
}, Open  ]],
Cell[1898645, 33449, 167524, 2771, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2066218, 36226, 30, 0, 72, "Section"],
Cell[2066251, 36228, 86, 2, 31, "Input"],
Cell[2066340, 36232, 662, 22, 44, "Input"],
Cell[CellGroupData[{
Cell[2067027, 36258, 740, 20, 73, "Input"],
Cell[CellGroupData[{
Cell[2067792, 36282, 150, 4, 24, "Print"],
Cell[2067945, 36288, 208, 6, 24, "Print"]
}, Open  ]],
Cell[2068168, 36297, 39119, 666, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2107336, 36969, 30, 0, 72, "Section"],
Cell[2107369, 36971, 86, 2, 31, "Input"],
Cell[2107458, 36975, 662, 22, 44, "Input"],
Cell[CellGroupData[{
Cell[2108145, 37001, 740, 20, 73, "Input"],
Cell[CellGroupData[{
Cell[2108910, 37025, 150, 4, 24, "Print"],
Cell[2109063, 37031, 208, 6, 24, "Print"]
}, Open  ]],
Cell[2109286, 37040, 37350, 637, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2146685, 37683, 30, 0, 72, "Section"],
Cell[2146718, 37685, 86, 2, 31, "Input"],
Cell[2146807, 37689, 773, 25, 44, "Input"],
Cell[CellGroupData[{
Cell[2147605, 37718, 740, 20, 73, "Input"],
Cell[CellGroupData[{
Cell[2148370, 37742, 150, 4, 24, "Print"],
Cell[2148523, 37748, 208, 6, 24, "Print"]
}, Open  ]],
Cell[2148746, 37757, 50782, 857, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2199577, 38620, 30, 0, 72, "Section"],
Cell[2199610, 38622, 86, 2, 31, "Input"],
Cell[2199699, 38626, 359, 11, 31, "Input"],
Cell[CellGroupData[{
Cell[2200083, 38641, 740, 20, 73, "Input"],
Cell[CellGroupData[{
Cell[2200848, 38665, 150, 4, 24, "Print"],
Cell[2201001, 38671, 208, 6, 24, "Print"]
}, Open  ]],
Cell[2201224, 38680, 52661, 888, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2253934, 39574, 30, 0, 72, "Section"],
Cell[2253967, 39576, 86, 2, 31, "Input"],
Cell[2254056, 39580, 580, 19, 43, "Input"],
Cell[CellGroupData[{
Cell[2254661, 39603, 703, 19, 69, "Input"],
Cell[CellGroupData[{
Cell[2255389, 39626, 150, 4, 24, "Print"],
Cell[2255542, 39632, 208, 6, 24, "Print"]
}, Open  ]],
Cell[2255765, 39641, 20898, 367, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2276712, 40014, 30, 0, 72, "Section"],
Cell[2276745, 40016, 86, 2, 31, "Input"],
Cell[2276834, 40020, 556, 18, 43, "Input"],
Cell[CellGroupData[{
Cell[2277415, 40042, 732, 20, 69, "Input"],
Cell[CellGroupData[{
Cell[2278172, 40066, 150, 4, 24, "Print"],
Cell[2278325, 40072, 208, 6, 24, "Print"]
}, Open  ]],
Cell[2278548, 40081, 40777, 693, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2319374, 40780, 29, 0, 72, "Section"],
Cell[2319406, 40782, 86, 2, 31, "Input"],
Cell[2319495, 40786, 586, 19, 43, "Input"],
Cell[CellGroupData[{
Cell[2320106, 40809, 787, 22, 69, "Input"],
Cell[CellGroupData[{
Cell[2320918, 40835, 150, 4, 24, "Print"],
Cell[2321071, 40841, 208, 6, 24, "Print"]
}, Open  ]],
Cell[2321294, 40850, 34342, 588, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2355685, 41444, 29, 0, 72, "Section"],
Cell[2355717, 41446, 86, 2, 31, "Input"],
Cell[2355806, 41450, 615, 20, 43, "Input"],
Cell[CellGroupData[{
Cell[2356446, 41474, 751, 20, 69, "Input"],
Cell[CellGroupData[{
Cell[2357222, 41498, 150, 4, 24, "Print"],
Cell[2357375, 41504, 208, 6, 24, "Print"]
}, Open  ]],
Cell[2357598, 41513, 54291, 915, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2411938, 42434, 28, 0, 72, "Section"],
Cell[2411969, 42436, 86, 2, 31, "Input"],
Cell[2412058, 42440, 523, 17, 44, "Input"],
Cell[CellGroupData[{
Cell[2412606, 42461, 810, 23, 69, "Input"],
Cell[CellGroupData[{
Cell[2413441, 42488, 150, 4, 24, "Print"],
Cell[2413594, 42494, 208, 6, 24, "Print"]
}, Open  ]],
Cell[2413817, 42503, 43500, 738, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2457366, 43247, 30, 0, 72, "Section"],
Cell[2457399, 43249, 86, 2, 31, "Input"],
Cell[2457488, 43253, 565, 18, 52, "Input"],
Cell[CellGroupData[{
Cell[2458078, 43275, 211, 4, 48, "Input"],
Cell[CellGroupData[{
Cell[2458314, 43283, 150, 4, 24, "Print"],
Cell[2458467, 43289, 208, 6, 24, "Print"]
}, Open  ]],
Cell[2458690, 43298, 48503, 820, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2507242, 44124, 30, 0, 72, "Section"],
Cell[2507275, 44126, 86, 2, 31, "Input"],
Cell[2507364, 44130, 692, 23, 44, "Input"],
Cell[CellGroupData[{
Cell[2508081, 44157, 844, 23, 52, "Input"],
Cell[CellGroupData[{
Cell[2508950, 44184, 150, 4, 24, "Print"],
Cell[2509103, 44190, 208, 6, 24, "Print"]
}, Open  ]],
Cell[2509326, 44199, 41382, 703, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2550757, 44908, 27, 0, 72, "Section"],
Cell[2550787, 44910, 86, 2, 31, "Input"],
Cell[2550876, 44914, 693, 23, 44, "Input"],
Cell[CellGroupData[{
Cell[2551594, 44941, 843, 24, 70, "Input"],
Cell[CellGroupData[{
Cell[2552462, 44969, 150, 4, 24, "Print"],
Cell[2552615, 44975, 208, 6, 24, "Print"]
}, Open  ]],
Cell[2552838, 44984, 48715, 823, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2601602, 45813, 31, 0, 72, "Section"],
Cell[2601636, 45815, 86, 2, 31, "Input"],
Cell[2601725, 45819, 681, 22, 44, "Input"],
Cell[CellGroupData[{
Cell[2602431, 45845, 1216, 37, 100, "Input"],
Cell[CellGroupData[{
Cell[2603672, 45886, 150, 4, 24, "Print"],
Cell[2603825, 45892, 208, 6, 24, "Print"]
}, Open  ]],
Cell[2604048, 45901, 74477, 1247, 375, 38689, 659, "CachedBoxData", \
"BoxData", "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2678574, 47154, 26, 0, 72, "Section"],
Cell[2678603, 47156, 86, 2, 31, "Input"],
Cell[2678692, 47160, 830, 26, 31, "Input"],
Cell[CellGroupData[{
Cell[2679547, 47190, 632, 17, 52, "Input"],
Cell[CellGroupData[{
Cell[2680204, 47211, 150, 4, 24, "Print"],
Cell[2680357, 47217, 208, 6, 24, "Print"]
}, Open  ]],
Cell[2680580, 47226, 60368, 1014, 375, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2740997, 48246, 28, 0, 72, "Section"],
Cell[2741028, 48248, 86, 2, 31, "Input"],
Cell[2741117, 48252, 830, 26, 31, "Input"],
Cell[CellGroupData[{
Cell[2741972, 48282, 1146, 34, 100, "Input"],
Cell[CellGroupData[{
Cell[2743143, 48320, 150, 4, 24, "Print"],
Cell[2743296, 48326, 208, 6, 24, "Print"]
}, Open  ]],
Cell[2743519, 48335, 57967, 975, 375, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2801523, 49315, 306, 8, 52, "Input"],
Cell[2801832, 49325, 54, 0, 31, "Output"],
Cell[2801889, 49327, 88, 2, 31, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
